<?php

define('sugarEntry', 1);
define('DEFAULT_ASSIGNED_USER', 1);
define('ONECRM_CSRF_PROTECT', 'none');

require_once('include/entryPoint.php');

require_once 'include/database/ListQuery.php';
require_once 'include/database/RowUpdate.php';
require_once('modules/Campaigns/WebToLeadForm.php');

$wtl_reponse = array();
$wtl_json = !empty($_REQUEST['wtl_json']);
global $timedate, $log;

$user_id = array_get_default($_REQUEST, 'assigned_user_id');
if (empty($user_id))
    $user_id = AppConfig::setting('company.web_lead_user');
if (empty($user_id))
    $user_id = DEFAULT_ASSIGNED_USER;
$current_user = new User();
$current_user->retrieve($user_id);

AppConfig::session_start();

$app_strings = return_application_language('base');
$app_list_strings = return_app_list_strings_language('base');
$mod_strings = return_module_language('base', 'Leads');
$error = null;
$dup = null;

$form_token = $_REQUEST['form_token'] ?? null;

if (! $form_token) {
	if ($wtl_json) {
		returnJson(['error' => $mod_strings['LBL_WEB_TO_LEAD_FORM_TOKEN_ERROR']]);
	} else {
		die($mod_strings['LBL_WEB_TO_LEAD_FORM_TOKEN_ERROR']);
	}
}

$form = new WebToLeadForm();
$validation_result = $form->validate($form_token, $_REQUEST);

if (! $validation_result['success']) {
	if ($wtl_json) {
		returnJson(['error' => $validation_result['error']]);
	} else {
		die($validation_result['error']);
	}
}

if (isset($_POST['email1'])) {
	$lq = new ListQuery('Lead');
	$row = $lq->queryRecordKey('email1', $_POST['email1']);
	if ($row && !$row->failed) {
		$note = RowUpdate::blank_for_model('Note', $user_id);
		$note->setSource('leadcapture');
		$note->set('parent_type', 'Leads');
		$note->set('parent_id', $row->getField('id'));
		$note->set('name', 'Repeated Lead Contact');
		
		$lead = RowUpdate::for_result($row);
		$lead->setSource('leadcapture');
		$lead->loadRequest();
		$changes = $lead->getChanges();
		$desc = '';
		foreach ($changes as $k => $v) {
			if ($k == 'assigned_user_id')
				continue;
			$fname = $lead->translateFieldName($k);
			$desc .= $fname . ': ' . $v . "\n";
		}
		
		$note->set('description', $desc);
		$note->set('assigned_user_id', $row->getField('assigned_user_id'));
		$note->save();
		$dup = true;
	}
}

if (! $error && !$dup) {
	$lead = RowUpdate::blank_for_model('Lead', $user_id);
	$lead->setSource('leadcapture');

	$req = $_REQUEST;

	$input_fields = array_merge($lead->column_fields, $lead->custom_fields);

	foreach($lead->fields as $f) {
		if ($f['type'] == 'date' && isset($req[$f['name']]))
			$req[$f['name']] = $timedate->to_display_date($req[$f['name']], false);
	}

	$lead->request = $req;
	$lead->loadRequest();

	if (!empty($_POST['_ex1'])) {
		$partner_code = (int)$_POST['_ex1'];
		if ($partner_code) {
			$partner = ListQuery::quick_fetch_key('Partner', 'code', $partner_code);
			if ($partner)
				$lead->set('partner_id', $partner->getField('id'));
		}
	}

	if (!empty($_POST['_ex2'])) {
		$source_number = (int)$_POST['_ex2'];
		$sources = array_keys($app_list_strings['lead_source_dom']);
		if ($source_number > 0 && $source_number <= count($sources)) {
			$lead->set('lead_source', $sources[$source_number]);
		}
	}

	try {
		if(! $lead->validate() || ! $lead->save()) {
			$log->error("Lead capture failed: " . $lead->getErrors(true));
			$fail = true;
		}
	} catch (Exception $e) {
		$log->fatal("Lead capture failed: " . $e->getMessage());
		$fail = true;
	}
	if(! empty($fail)) {
		$error = $mod_strings['LBL_SERVER_IS_CURRENTLY_UNAVAILABLE'] . '. ';	
	}
}

if ($error && !$wtl_json) {
	$get = array();
	foreach($_POST as $name => $value) {
		if ($name == 'error_message_text')
			$value = $error;
		$get[] = urlencode($name) . '=' . urlencode(from_html($value));
	}
	$get[] = 'isDuplicate=1';
	if(! empty($_SERVER['HTTP_REFERER'])) {
		$parts = parse_url($_SERVER['HTTP_REFERER']);
		$uri = "{$parts['scheme']}://{$parts['host']}{$parts['path']}";
		$uri .= '?' . join('&', $get);
		header('Location: ' . $uri);
	} else {
		die($error);
	}
	exit;
}

if ($error && $wtl_json) {
	$wtl_reponse['error'] = $error;
}

if ($campaign_id = array_get_default($_POST, 'campaign_id')) {
	$campaign = ListQuery::quick_fetch('Campaign', $campaign_id);
	if ($campaign) {
		$clog = RowUpdate::blank_for_model('CampaignLog', $user_id);
		$clog->setSource('leadcapture');
		$clog->set(array(
			'campaign_id' => $campaign_id,
			'activity_date' => gmdate('Y-m-d H:i:s'),
			'activity_type' => 'lead',
			'target_id' => $lead->getPrimaryKeyValue(),
			'related_id' => $lead->getPrimaryKeyValue(),
			'target_type' => 'Leads',
			'related_type' => 'Leads',
			'assigned_user_id' => $user_id,
			'more_information' => $lead->getField('email1'),
		));
		$clog->save();
	}
}

// handle an array of target list id's the lead is added to
if ($subscriptions = array_get_default($_POST, 'subscriptions')) {
	require_once 'modules/Campaigns/utils.php';
	require_once 'modules/Leads/Lead.php';
	if (!is_array($subscriptions)) {
		$subscriptions = array($subscriptions);
	}
	$leadObject = new Lead();
	$leadObject->retrieve($lead->getPrimaryKeyValue());
	foreach ($subscriptions as $prospectlist){
		CampaignUtils::subscribe_to_list($leadObject, $prospectlist);
	}
}

if (isset($_POST['redirect_url'])) {
	if ($wtl_json)
		$wtl_reponse['redirect'] = $_POST['redirect_url'];
	else
		header('Location: ' . $_POST['redirect_url']);
} else {
	if ($wtl_json)
		$wtl_reponse['message'] = $mod_strings['LBL_THANKS_FOR_SUBMITTING_LEAD'];
	else
		echo $mod_strings['LBL_THANKS_FOR_SUBMITTING_LEAD'];
}

if ($wtl_json)
	returnJson($wtl_reponse);

function returnJson($data) {
	header('Content-Type: application/json');
	$json = getJSONObj();
	echo $json->encode($data);
	exit;
}
