<?php
if(!defined('sugarEntry'))define('sugarEntry', true);
define('ONECRM_CSRF_PROTECT', 'none');
require_once('include/entryPoint.php');
require_once('include/Notify/NotifyMessage.php');
require_once('modules/Meetings/Meeting.php');

AppConfig::restrict_entry_point();

function failRequest($msg) {
	$title = 'Error updating event attendance';
	NotifyErrorMessage::show_error($msg, $title);
}

$targ_mod = array_get_default($_REQUEST, 'module');
$targ_id = array_get_default($_REQUEST, 'record');
if(! $targ_mod || ! $targ_id || ! ($bean_name = AppConfig::module_primary_bean($targ_mod))) {
	failRequest('Missing input parameters');
	return;
}
if($bean_name != 'Meeting' && $bean_name != 'Call') {
	failRequest('Not supported');
	return;
}

$status = 'accept';
if(! empty($_REQUEST['accept_status']))
	$status = $_REQUEST['accept_status'];

AppConfig::session_start();
$error = false;

if (! empty($_REQUEST['user_id'])) {
    $user = ListQuery::quick_fetch('User', $_REQUEST['user_id']);

    if ($user == null) {
        $error = true;
        $message = 'Unknown user';
    }

    $current_entity = $user;
    $extern = false;
} elseif (! empty($_REQUEST['contact_id'])) {
    $contact = ListQuery::quick_fetch('Contact', $_REQUEST['contact_id']);

    if ($contact == null) {
        $error = true;
        $message = 'Unknown contact';
    }

    $current_entity = $contact;
    $extern = true;
} else {
    $error = true;
    $message = 'Missing input parameters';
}

if(! $error) {
	if($extern) {
		$ctx = 'email';
	} else {
		$current_user = new User();
		$current_user->retrieve($user->getField('id'));
		$ctx = 'notify';
	}
	$bean = ListQuery::quick_fetch($bean_name, $targ_id, true, array('formatted' => 'html', 'formatted_context' => $ctx));
	if(empty($bean)) {
		$error = true;
		$message = 'Event not found';
	}
}

if($error) {
    failRequest($message);
    return;
}

$upd = RowUpdate::for_result($bean);
Meeting::set_invite_status($upd, $current_entity, $status);

$sep = translate('LBL_SEPARATOR', 'app');

$props = array();
$props[] = array(
	'name' => to_html(translate('LBL_STATUS', 'Meetings') . $sep),
	'value' => $bean->getField('status', null, true),
);
$start = $bean->getField('date_start', null, true);
if($start) $props[] = array(
	'name' => to_html(translate('LBL_DATE_TIME', 'Meetings') . $sep),
	'value' => $start,
);
$dur = $bean->getField('duration', null, true);
if($dur) $props[] = array(
	'name' => to_html(translate('LBL_DURATION', 'Meetings') . $sep),
	'value' => $dur,
);
$stat_dom = 'dom_meeting_accept_status';
$badge_dom = AppConfig::setting('statuses.'.$stat_dom);
$stat_fmt = to_html(translate_dd($stat_dom, $status, $status));
$badge_cls = array_get_default($badge_dom, $status, 'neutral');
$stat_fmt = '<div class="badge badge-'.$badge_cls.'">'.$stat_fmt.'</div>';
$props[] = array(
	'name' => to_html(translate('LBL_ACCEPT_STATUS', 'Meetings') . $sep),
	'value' => $stat_fmt,
);

$msg = NotifyFormatter::simple_record(
	'<div class="input-icon theme-icon module-'.$targ_mod.'"></div> '.$bean->getField('name', '', true),
	$props,
	to_html(translate('LBL_STATUS_UPDATED', 'app')),
	'success');
$fmt = NotifyFormatter::create_standalone($extern);
$fmt->setTarget($current_entity);
echo $fmt->renderMessage($msg);

sugar_cleanup();

?>
