<?php
/*
  This entry point acts like index.php, but assumes you are replacing part of
   a page or showing a dialog window. It does not redirect to the login page
   when your session has expired, and does not output headers or footers.
   
   For async calls with non-HTML results, use json.php
*/

define('sugarEntry', 1);
define('ASYNC_ENTRY', 1);

require_once('include/entryPoint.php');

function indexRedirect($params, $async=true) {
	if(is_string($params))
		$path = $params;
	else if(isset($params['path']))
		$path = $params['path'];
	else
		$path = 'index.php' . encode_query_string($params, true);
	if($async) {
		$path = javascript_escape($path);
		echo "<script nodisplay>SUGAR.util.navigateTo('$path');</script>";
	} else
		header('Location: '.$path);
	if(function_exists('sugar_cleanup'))
		sugar_cleanup(true);
	else
		exit;
}

// Async page load does not apply (index.php only)
if(! AppConfig::installed() || AppConfig::upgrade_in_progress())
	return;
// Prevent access via alternate site urls
if(AppConfig::is_extern_url() || AppConfig::get_auto_redirect())
	return;

insert_charset_header();
if(! AppConfig::check_database_version($info))
	return;

$module = array_get_default($_REQUEST, 'module', '');
$action = array_get_default($_REQUEST, 'action', '');
$record = array_get_default($_REQUEST, 'record', '');
// Used for current record focus
$focus = null;

$authController = new AuthenticationController();

if(! AppConfig::session_start()) {
	header('HTTP/1.1 503 Service Unavailable');
	die('Error starting session');
}

// If recording is available, call the recorder to record this round trip.
if(is_file("recorder.php"))
	include("recorder.php");

$current_user = new User();
if(empty($_SESSION['authenticated_user_id']))
	$not_auth = true;
else {
	if(! $authController->sessionAuthenticate()) {
		session_destroy();
		$not_auth = true;
	}
}
if(! empty($not_auth)) {
	$GLOBALS['log']->warn('Cancel async request: user not authenticated');
	header('HTTP/1.1 401 Not Authorized');
	die('User is not authenticated');
}



if(! AppConfig::check_init_user()) {
	$GLOBALS['log']->warn('Cancel async request: user not initialized');
	exit;
}

$current_user->update_access_time();

$GLOBALS['log']->debug("*** Starting Async Request");

ACLController::filterModuleList($moduleList);
ACLController::filterModuleList($modSemiInvisList);

// Initialize page controller
AppConfig::init_page_controller();

$doRawOutput = $skipHeaders = $skipFooters = true;

function indexCheckRawOutput($params) {}
function handlePDFPrint() {}

function startDirectOutput() {
	end_flush_all_buffers();
	$GLOBALS['async_direct_output'] = true;
}
ob_start();
include('include/layout/render.php');


if(empty($async_direct_output)) {
	$pageContent = ob_get_contents();
	ob_clean();
	
	echo $pageInstance->format_includes('body');
	echo $pageContent;
}

echo $pageInstance->format_includes('foot');
echo $pageInstance->format_includes('end');

$pageInstance->post_output();

// finished with $_SESSION variable, avoid lock contention with other requests
session_write_close();

// page output is finished
ignore_user_abort(true);
if (class_exists('Webhook') && Webhook::has_pending_calls()) {
	$content = '';
	foreach (enumerate_and_end_buffers(true, true) as $buf_content) {
		$content = $buf_content . $content;
	}
	ini_set('zlib.output_compression', '0');
	header('Content-Encoding: none');
	header('Content-Length: ' . mb_strlen($content, '8bit'));
	echo $content;
} else {
	end_flush_all_buffers();
}
flush();

include('include/layout/post_request.php');

sugar_cleanup();

?>

