<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
********************************************************************************/
/*********************************************************************************

* Description:  Defines the base class for all data entities used throughout the
* application.  The base class including its methods and variables is designed to
* be overloaded with module-specific methods and variables particular to the
* module's base entity class.
* Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
* All Rights Reserved.
*******************************************************************************/


require_once('modules/ACLActions/ACLAction.php');
require_once('modules/Currencies/Currency.php');
require_once('include/config/ModelDef.php');
require_once('include/utils/db_utils.php');
require_once('include/database/RowUpdate.php');
require_once('data/Link.php');


//TODO remove save2, deleterelationship, subpanelviewer from each module.  Use generic include instead

/**
 * SugarBean is the base class for all business objects in Sugar.  It implements
 * the primary functionality needed for manipulating business objects: create,
 * retrieve, update, delete.  It allows for seraching and retriving list of records.
 * It allows for retrieving related objects (e.g. contacts related to a specific account).
 *
 * In the current implementation, there can only be one bean per folder.
 * Naming convention has the bean name be the same as the module and folder name.
 * All bean names should be singular (e.g. Contact).  The primary table name for
 * a bean should be plural (e.g. contacts).
 *
 */
#[\AllowDynamicProperties]
class SugarBean
{
    /**
     * A pointer to the database helper object DBHelper
     *
     * @var DBManager
     */
	var $db;

	var $model;

	/**
	 * Does this bean participate in the new schema.  All modules should be new schema at this point.
	 * New schema means that you have a GUID in the id field.
	 *
	 * @var BOOL - default false.
	 */
	var $new_schema = false;

	/**
	 * When createing a bean, you can specify a value in the id column as
	 * long as that value is unique.  During save, if the system finds an
	 * id, it assumes it is an update.  Setting new_with_id to true will
	 * make sure the system performs an insert instead of an update.
	 *
	 * @var BOOL -- default false
	 */
	var $new_with_id = false;

    /**
     * holds the full name of the user that an item is assigned to.  Only used if notifications
     * are turned on and are going to be sent out.
     *
     * @var String
     */
	var $new_assigned_user_name;

	/**
	 * An array of booleans.  This array is cleared out when data is loaded.
	 * As date/times are converted, a 1 is placed under the key, the field is converted.
	 *
	 * @var Array of booleans
	 */
	var $processed_dates_times = array();

	/**
	 * Whether to process date/time fields for storage in the database in GMT
	 *
	 * @var BOOL
	 */
	var $process_save_dates =true;

    /**
     * This signals to the bean that it is being saved in a mass mode.
     * Examples of this kind of save are import and mass update.
     * We turn off notificaitons of this is the case to make things more efficient.
     *
     * @var BOOL
     */
	var $save_from_post = true;

	/**
	 * When running a query on releated items using the method: retrieve_by_string_fields
	 * this value will be set to true if more than one item matches the search criterea.
	 *
	 * @var BOOL
	 */
	var $duplicates_found = false;

	/**
	 * true if this bean has been deleted, false otherwise.
	 *
	 * @var BOOL
	 */
    var $deleted = 0;

    /**
     * Should the date modified column of the bean be updated during save?
     * This is used for admin level functionality that should not be updating
     * the date modified.  This is only used by sync to allow for updates to be
     * replicated in a way that will not cause them to be replicated back.
     *
     * @var BOOL
     */
    var $update_date_modified = true;

    /**
     * Should the modified by column of the bean be updated during save?
     * This is used for admin level functionality that should not be updating
     * the modified by column.  This is only used by sync to allow for updates to be
     * replicated in a way that will not cause them to be replicated back.
     *
     * @var BOOL
     */
    var $update_modified_by = true;

    /**
     * Setting this to true allows for updates to overwrite the date_entered
     * @var BOOL
     */
    var $update_date_entered = false;

    /**
     * This allows for seed data to be created without using the current uesr to set the id.
     * This should be replaced by altering the current user before the call to save.
     *
     * @var unknown_type
     */
    //TODO This should be replaced by altering the current user before the call to save.
    var $set_created_by = true;

    /**
     * The database table where records of this Bean are stored.
     *
     * @var String
     */
	var $table_name = '';

	/**
	 * This is the singular name of the bean.  (i.e. Contact).
	 *
	 * @var String
	 */
	var $object_name = '';
    var $bean_name = '';

	/**
	 * The name of the module folder for this type of bean.
	 *
	 * @var String
	 */
	var $module_dir = '';
	var $column_fields;
	var $additional_column_fields;
	var $current_notify_user;
	var $fetched_row=false;
	var $force_load_details = false;
	var $audit_enabled = false;
	var $optimistic_lock = false;
	var $disable_custom_fields = false;
	var $number_formatting_done = false;
	var $format_relative_dates_times = false;
	var $prohibit_workflow = false;

	function __construct() {
		global $current_user;

		$this->db = $GLOBALS['db'];
		$this->model_obj = new ModelDef($this->getObjectName());

		$this->table_name = $this->model_obj->getTableName();
		$this->optimistic_lock = $this->model_obj->getOptimisticLocking();
		$this->audit_enabled = $this->model_obj->getAuditEnabled();

		// these are currently referenced in many places, but should be generated when needed later
		// (using getColumnFields/getExtraFields)
		$this->column_fields = $this->model_obj->getColumnFields();
		$this->additional_column_fields = $this->model_obj->getExtraFields();
		$this->required_fields = $this->model_obj->getRequiredFields();

		$this->populateDefaultValues();
	}

    function getObjectName(){
        if ($this->object_name)
        	return $this->object_name;

        // This is a quick way out. The generated metadata files have the table name
        // as the key. The correct way to do this is to override this function
        // in bean and return the object name. That requires changing all the beans
        // as well as put the object name in the generator.
        return $this->table_name;
    }


 	function is_AuditEnabled(){
        return $this->audit_enabled;
    }


    function getTableName() {
		return $this->table_name;
    }

    function getFieldDefinitions(){
       return $this->model_obj->getFieldDefinitions();
    }

    function getIndices(){
    	return $this->model_obj->getIndices();
    }

    function getFieldDefinition($name){
		return $this->model_obj->getFieldDefinition($name);
    }

    function getColumnFields() {
    	return $this->column_fields;
    }

    function getExtraFields() {
    	return $this->additional_column_fields;
    }

    function getPrimaryFieldDefinition() {
    	$key = $this->model_obj->getPrimaryKey();
    	return $key && $this->model_obj->getFieldDefinition($key);
    }

    function getFieldValue($name, $default=null) {
        if (!isset($this->$name))
           return $default;
        return $this->$name;
    }


    /* Mis-named: creates a Link object for a named link field.
     */
    function load_relationship($link_name) {
    	if (empty($link_name)) {
    		$GLOBALS['log']->error("SugarBean.load_relationship, null link name passed.");
    		return false;
    	}

    	$link_def = AppConfig::setting("model.links.{$this->object_name}.$link_name");
    	if(! $link_def) {
			$GLOBALS['log']->error("SugarBean.load_relationship, link not found ($link_name).");
			return false;
    	}

		require_once('data/Link.php');
		$this->$link_name = new Link($this, $link_def);
		return true;
    }

    function load_relationships() {

    	$GLOBALS['log']->debug("SugarBean.load_relationships, Loading all relationships of type link.");

    	$linked_fields=$this->get_linked_fields();
    	foreach($linked_fields as $name=>$properties) {
			$this->$name=new Link($this, $properties);
    	}
    }

    /*
     *
     * deleted: 0  adds deleted=0 filter.
     * 		    1  adds deleted=1 filter.
     * 			(anything else) deleted filter is ignored.
     */
    function get_linked_beans($field_name, $bean_name, $sort_array = array(), $begin_index = 0, $end_index = -1, $deleted=0, $optional_where="") {
		require_bean($bean_name);
    	$this->load_relationship($field_name);
    	$old_d = $this->$field_name->add_distinct;
    	$this->$field_name->add_distinct = true;
		$ret = $this->$field_name->getBeans(new $bean_name(), $sort_array, $begin_index, $end_index, $deleted, $optional_where);
		$this->$field_name->add_distinct = $old_d;
		return $ret;
    }

    function get_linked_fields() {
    	return $this->model_obj->getLinks();
    }

    /** iterates thru all the relationships and deletes relationship for each record.. */
    function delete_linked($id) {

    	$linked_fields=$this->get_linked_fields();

    	foreach ($linked_fields as $name => $value) {
	    	if ($this->load_relationship($name)) {
				$GLOBALS['log']->debug('relationship loaded');
				$this->$name->delete($id);
	    	} else {
	    		$GLOBALS['log']->error('error loading relationship');
	    	}
    	}
    }

	/**
	 * Save the bean.  All changes to this bean will be recorded in the data store.
	 */
	/**
	* This method implements a generic insert and update logic for any SugarBean
	* This method only works for subclasses that implement the same variable names.
	* This method uses the presence of an id field that is not null to signify and update.
	* The id field should not be set otherwise.
	* todo - Add support for field type validation and encoding of parameters.
	* Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
	* All Rights Reserved.
	* Contributor(s): ______________________________________..
	*/

	function save($check_notify = FALSE)
	{
		global $current_user, $action;

		$isUpdate = ! empty($this->id) && ! $this->new_with_id;

		if ($this->deleted != 1) $this->deleted = 0;

        $this->check_date_relationships_save();

        $this->unformat_all_fields();

        $fdefs = $this->getFieldDefinitions();

        if (! empty($this->bean_name)) {
            $bean_name = $this->bean_name;
        } else {
            $bean_name = $this->object_name;
        }

		if($isUpdate) {
			require_once 'include/database/RowResult.php';
			// init 'fake' rowresult based on earlier retrieve
			$result = new RowResult();
			$result->base_model = $bean_name;
			$result->fields = $fdefs;
			$result->module_dirs = array($this->module_dir);
			$result->primary_key = array('id');
			$result->row = $this->fetched_row;
			$upd = RowUpdate::for_result($result);
		} else {
			$upd = RowUpdate::blank_for_model($bean_name);
		}
		$upd->prohibit_workflow = $this->prohibit_workflow;

		$newvals = array();
		foreach($fdefs as $field=>$value) {
			if(isset($value['type']) && $value['type'] == 'ref')
				continue;
			if(!isset($value['source']) || $value['source']['type'] == 'db') {
				if($isUpdate && ('id' == $field)) continue;
				if(isset($this->$field))
					$newvals[$field] = from_html($this->$field);
				else if($isUpdate)
					$newvals[$field] = null;
			}
		}
		$upd->set($newvals);

		if(! $upd->save())
			return false;
		$this->id = $upd->getField('id');
		$this->new_with_id = false;

		// let subclasses save related field changes
		$this->save_relationship_changes($isUpdate);

		return $this->id;
	}

	function has_been_modified_since($date, $modified_user_id){
		global $current_user;
		if (isset($current_user)){
			$query = "SELECT date_modified FROM $this->table_name WHERE id='$this->id' AND  modified_user_id != '$current_user->id' AND (modified_user_id != '$modified_user_id' OR date_modified > " . db_convert("'".$date."'", 'datetime') . ')';
			$result = $this->db->query($query);

			if($this->db->fetchByAssoc($result)){

				return true;
			}
		}

		return false;
	}


	/**
	* This function is a good location to save changes that have been made to a relationship.
	* This should be overriden in subclasses that have something to save.
	* param $is_update true if this save is an update.
	* Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
	* All Rights Reserved.
	* Contributor(s): ______________________________________..
	*/
	function save_relationship_changes($is_update)
	{
    	if (isset($this->relationship_fields) && is_array($this->relationship_fields)) {
    		foreach ($this->relationship_fields as $id=>$rel_name) {

	    		if(!empty($this->$id)) {
					$this->add_link($rel_name, null, $this->$id);
		    	}
		    	else {
					//if before value is not empty the attempt to delete relationship.
		    		if(!empty($this->rel_fields_before_value[$id])) {
		    			$GLOBALS['log']->debug('Attempting to remove the relationship record, using relationship attribute'.$rel_name);
						$this->remove_link($rel_name, $this->rel_fields_before_value[$id]);
		    		}
		    	}
    		}
    	}

	}

	/**
	* This function retrieves a record of the appropriate type from the DB.
	* It fills in all of the fields from the DB into the object it was called on.
	* param $id - If ID is specified, it overrides the current value of $this->id.  If not specified the current value of $this->id will be used.
	* returns this - The object that it was called apon or null if exactly 1 record was not found.
	* Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
	* All Rights Reserved.
	* Contributor(s): ______________________________________..
	*/

	function check_date_relationships_load(){
		global $disable_date_format;
		if(!empty($disable_date_format)){
			return;
		}
		global $timedate;
		if (empty($timedate)) $timedate=new TimeDate();

		if(! ($field_defs = $this->getFieldDefinitions()) ) {
			return;
		}
		foreach($field_defs as $fieldDef) {
			$field = $fieldDef['name'];
			if(!key_exists($field, $this->processed_dates_times)) {
				$this->processed_dates_times[$field] = '1';
				$format_relative = $this->format_relative_dates_times && empty($fieldDef['disable_relative_format']);
				$raw = "raw_$field";

				if($field == 'date_modified' || $field == 'date_entered') {
					if(!empty($this->$field)) {
						$this->$raw = $this->$field;
						if($format_relative)
							$this->$field = $timedate->to_relative_date_time($this->$field);
						else
							$this->$field = $timedate->to_display_date_time($this->$field);
					}
      			} elseif(!empty($this->$field) && isset($field_defs[$field]['type'])) {
					$type = $field_defs[$field]['type'];

					if($type == 'relate'  && isset($field_defs[$field]['custom_type'])) {
						$type = $field_defs[$field]['custom_type'];
					}

					if($type == 'date') {
						$this->$raw = $this->$field;
						$this->$field = from_db_convert($this->$field, 'date');

						if($this->$field == '0000-00-00') {
							$this->$field = '';
						} elseif(!empty($field_defs[$field]['rel_field'])) {
							$rel_field = $field_defs[$field]['rel_field'];

							if(empty($this->$rel_field)) {
								if($format_relative)
									$this->$field = $timedate->to_relative_date($this->$field, false);
								else
									$this->$field = $timedate->to_display_date($this->$field, false);
							} else {
								$rel_raw = "raw_$rel_field";
								$this->$rel_raw = $this->$rel_field;
								$this->$rel_field=from_db_convert($this->$rel_field, 'time');
								$mergetime = $timedate->merge_date_time($this->$field,$this->$rel_field);
								if($format_relative)
									$this->$field = $timedate->to_relative_date($mergetime);
								else
									$this->$field = $timedate->to_display_date($mergetime);
								$this->$rel_field = $timedate->to_display_time($mergetime);
							}
						} else {
							if($format_relative)
								$this->$field = $timedate->to_relative_date($this->$field, false);
							else
								$this->$field = $timedate->to_display_date($this->$field, false);
						}
					} elseif($type == 'datetime') {
						$this->$raw = $this->$field;
						if($this->$field == '0000-00-00 00:00:00') {
							$this->$field = '';
						} else {
							if($format_relative)
								$this->$field = $timedate->to_relative_date_time($this->$field);
							else
								$this->$field = $timedate->to_display_date_time($this->$field);
						}
					} elseif($type == 'time') {
						if($this->$field == '00:00:00') {
							$this->$field = '';
						} else {
							//$this->$field = from_db_convert($this->$field, 'time');
							if(empty($field_defs[$field]['rel_field'])) {
								$this->$raw = $this->$field;
								$this->$field = $timedate->to_display_time($this->$field,true, false);
							}
						}
					}
				}
			}
		}
	}

	function check_date_relationships_save(){
		global $disable_date_format, $timedate;
		if(!empty($disable_date_format)){
			return;
		}

		if($this->process_save_dates) {
			if(! ($field_defs = $this->getFieldDefinitions())) {
				return;
			}

			foreach($field_defs as $fieldDef) {
				if ( !isset($fieldDef['name'])) {
					continue;
				}

				$field = $fieldDef['name'];
				if($field == 'date_modified' || $field == 'date_entered'){
					//do nothing we need to do this so we can keep seconds on the time
					$raw = 'raw_'.$field;
					if(! isset($this->$field) || ! preg_match('~^\d{4}-\d{1,2}-\d{1,2} \d{1,2}:\d{1,2}:?\d{2}?$~', $this->$field)) {
						if(isset($this->$raw))
							$this->$field = $this->$raw;
						else
							$this->$field = null;
					}
				} else if(!empty($this->$field)) {
					if(empty($field_defs[$field]['type'])){
						$type = 'varchar';
					}else{
						$type = $field_defs[$field]['type'];
					}

					if($type == 'relate'  && isset($field_defs[$field]['custom_type'])) {
						$type = $field_defs[$field]['custom_type'];
					}

					if($type == 'date') {
						if(!empty($field_defs[$field]['rel_field'])) {
							$rel_field = $field_defs[$field]['rel_field'];

							if(empty($this->$rel_field)) {
								$this->$rel_field = $timedate->to_display_time('12:00:00', true, false);
							}

							$mergetime = $timedate->merge_date_time($this->$field,$this->$rel_field);
							$this->$field = $timedate->to_db_date($mergetime);
							$this->$rel_field = $timedate->to_db_time($mergetime);
						} else {
							$this->$field = $timedate->to_db_date($this->$field, false);
						}
					} elseif($type == 'datetime') {
						$this->$field = $timedate->to_db($this->$field);
					} elseif($type == 'time') {
						if(empty($field_defs[$field]['rel_field'])) {
							$this->$field = $timedate->to_db_time($this->$field, false);
						}
					}
				}
			} // end foreach()
		}
	}

	/*
	 * unformat all fields for vardefs, currently only for numbers
	 */
	function unformat_all_fields() {
		global $disable_num_format, $current_user;
		if((!empty($disable_num_format) && $disable_num_format) || empty($current_user)) return;

		$field_defs = $this->getFieldDefinitions();
		// turned off at bean level?
		if((!empty($this->disable_num_format) && $this->disable_num_format) || empty($field_defs)) return;


		// longreach - start added - prevent double unformatting
		// note: you must call format_all_fields before updating a SugarBean with user input and saving it
		if(empty($this->number_formatting_done))
			return;
		$this->number_formatting_done = false;
		// longreach - end added


		foreach($field_defs as $fieldDef) {
			$type = (empty($fieldDef['type']) ? $fieldDef['dbType'] : $fieldDef['type']);
			if($type == 'relate' && isset($fieldDef['custom_type']))
				$type = $fieldDef['custom_type'];
			if(in_array($type, array('int', 'float', 'double', 'uint', 'ulong', 'long', 'short', 'tinyint', 'currency', 'decimal'))) {  // is number?
				if (!empty($fieldDef['disable_num_format']) && $fieldDef['disable_num_format']) continue; // turned off at field level?
				$field = $fieldDef['name'];
				if(!empty($this->$field)) {
					$this->$field = unformat_number($this->$field);
				}
			}
		}

	}

	/*
	 * format all fields for vardefs, currently only for numbers
	 */
	function format_all_fields() {
		global $disable_num_format;
		global $current_user;
		global $locale;

		if((!empty($disable_num_format) && $disable_num_format) || empty($current_user))

		// longreach - start added - prevent double formatting
		if(! empty($this->number_formatting_done))
			return;
		// longreach - end added

		if(! ($field_defs = $this->getFieldDefinitions())) {
			return;
		}

		foreach($field_defs as $fieldDef) {
			$type = (empty($fieldDef['type']) ? $fieldDef['dbType'] : $fieldDef['type']);
			if($type == 'relate' && isset($fieldDef['custom_type']))
				$type = $fieldDef['custom_type'];

			if(in_array($type, array('int', 'float', 'double', 'uint', 'ulong', 'long', 'short', 'tinyint', 'currency', 'decimal'))) {  // is number?
				if (!empty($fieldDef['disable_num_format']) && $fieldDef['disable_num_format'])
					continue; // turned off at field level?
				$field = $fieldDef['name'];

				/* longreach - removed
				if(!empty($this->$field)) {
				*/
				// longreach - added
				if(isset($this->$field) && !number_empty($this->$field)) {

					if(in_array($type, array('int', 'uint', 'ulong', 'long', 'short', 'tinyint'))) $this->$field = format_number($this->$field, 0, 0);
					// longreach - start added
					elseif ($type == 'currency' && (! isset($fieldDef['dbType']) || $fieldDef['dbType'] != 'int')) {
						$fmt = array('currency_symbol' => false);
						if(isset($fieldDef['currency_id']))
							$fid = $fieldDef['currency_id'];
						else
							$fid = 'currency_id';
						if(isset($field_defs[$fid]))
							$fmt['currency_id'] = isset($this->$fid) ? $this->$fid : '-99';
						if(! empty($fieldDef['extra_precision']))
							$fmt['round'] = -1;
						$this->$field = currency_format_number($this->$field, $fmt);
					}
					// longreach - end added
					else {
						// longreach - start modified - for forecasts
						$decimals = 2;
						if(isset($fieldDef['dbType']) && $fieldDef['dbType'] == 'int')
							$decimals = 0;
						else if(isset($fieldDef['decimal_places']))
							$decimals = $fieldDef['decimal_places'];
						$precision = $decimals;
						if(! empty($fieldDef['extra_precision']))
							$decimals = -1;
						$this->$field = format_number($this->$field, $decimals, $precision);
						// longreach - end modified
					}
				}
			}
		}
		$this->number_formatting_done = true;
	}

	function retrieve($id = -1, $encode=true) {

		// longreach - added
		$this->_parent_fields_loaded = false;

		if ($id == -1) {
			$id = $this->id;
		}

       	if(! empty($this->no_db_cache)) {
			$query = "SELECT $this->table_name.* FROM $this->table_name ";

			$query .= " WHERE $this->table_name.id = '$id' ";
			$GLOBALS['log']->debug("Retrieve $this->object_name : ".$query);
			//requireSingleResult has beeen deprecated.
			//$result = $this->db->requireSingleResult($query, true, "Retrieving record by id $this->table_name:$id found ");
			$result = $this->db->limitQuery($query,0,1,true, "Retrieving record by id $this->table_name:$id found ");
			if(empty($result)) {
				return null;
			}

			$row = $this->db->fetchByAssoc($result, -1, $encode);
		} else {
			$row = AppConfig::db_row($this->object_name, $id, true, $encode);
		}

		if(empty($row)){
			return null;
		}

		$this->populateFromRow($row);

		global $module, $action;
		//Just to get optimistic locking working for this release
		if($this->optimistic_lock && $module == $this->module_dir && $action =='EditView' ){
			$_SESSION['o_lock_id']= $id;
			$_SESSION['o_lock_dm']= $this->date_modified;
			$_SESSION['o_lock_on'] = $this->object_name;
		}

		$this->processed_dates_times = array();
		$this->check_date_relationships_load();

		if(empty($this->no_additional_detail))
			$this->fill_in_additional_detail_fields();

		//make a copy of fields in the relatiosnhip_fields array. these field values will be used to
		//clear relatioship.
    	if (isset($this->relationship_fields) && is_array($this->relationship_fields)) {
    		foreach ($this->relationship_fields as $rel_id=>$rel_name) {
    			if (isset($this->$rel_id))
					$this->rel_fields_before_value[$rel_id]=$this->$rel_id;
				else
					$this->rel_fields_before_value[$rel_id]=null;
    		}
    	}

		return $this;
	}

	function populateFromRow($row){
//TODO loop through vardefs instead
//runs into an issue when populating from field_defs for users - corrupts user prefs

		//make copy of the fetched row for construction of audit record and for business logic/workflow
		$this->fetched_row=$row;

		foreach($this->getFieldDefinitions() as $field=>$field_value)
		{
			if($field == 'user_preferences' && $this->module_dir == 'Users')continue;




			$rfield = $field; // fetch returns it in lowercase only
			if(isset($row[$rfield]))
			{
				$this->$field = $row[$rfield];
			}else{
				$this->$field = '';
			}
		}
	}


	/**
	 * Changes the select expression of the given query to be 'count(*)' so you
	 * can get the number of items the query will return.  This is used to
	 * populate the upper limit on ListViews.
	 */
	function create_list_count_query($query)
	{
		// change the select expression to 'count(*)'
		$pattern = '/SELECT(.*?\s+FROM\s+)*.*?\s+FROM\s+/is';  // ignores the case
		$replacement = 'SELECT count(*) c FROM ';
		$modified_select_query = preg_replace($pattern, $replacement, $query, 1);

		// remove the 'order by' clause which is expected to be at the end of the query
		$pattern = '/\s(GROUP|ORDER) BY.*/is';  // ignores the case
		$replacement = '';
		$modified_order_by_query = preg_replace($pattern, $replacement, $modified_select_query);

		return $modified_order_by_query;
	}

	/**
	* This function returns a paged list of the current object type.  It is intended to allow for
	* hopping back and forth through pages of data.  It only retrieves what is on the current page.
	* This method must be called on a new instance.  It trashes the values of all the fields in the current one.
	* Portions created by SugarCRM are Copyright (C) SugarCRM, Inc..
	* All Rights Reserved..
	* Contributor(s): ______________________________________..
	*/
	function get_list($order_by = "", $where = "", $row_offset = 0, $limit=-1, $max=-1, $show_deleted = 0) {
		$GLOBALS['log']->debug("get_list:  order_by = '$order_by' and where = '$where' and limit = '$limit'");
		if(isset($_SESSION['show_deleted'])){
			$show_deleted = 1;
		}
		$order_by=$this->process_order_by($order_by, null);

		if($this->bean_implements('ACL') && ACLController::requireOwner($this->module_dir, 'list') ){
			global $current_user;
            /* longreach - replaced
			$owner_where = $this->getOwnerWhere($current_user->id);
             */

            $level = ACLAction::getUserActions($current_user->id, false, $this->module_dir, 'module', 'list');
            $owner_where = $this->getOwnerWhere($current_user->id, $level['aclaccess']);

			if(empty($where)){
				$where = $owner_where;
			}else{
				if (!empty($owner_where))
				$where .= ' AND '.  $owner_where;
			}
		}
		/** eggsurplus - SECURITYGROUPS */
    	if($this->bean_implements('ACL') && ACLController::requireSecurityGroup($this->module_dir, 'list') )
    	{
			require_once('modules/SecurityGroups/SecurityGroup.php');
    		global $current_user;
            $level = ACLAction::getUserActions($current_user->id, false, $this->module_dir, 'module', 'list');
    		$owner_where = $this->getOwnerWhere($current_user->id, $level['aclaccess']);
    		$group_where = SecurityGroup::getGroupWhere($this->table_name,$this->module_dir,$current_user->id);
    		if(empty($where))
    		{
    			$where = " (".  $owner_where." or ".$group_where.") ";
    		} else {
    			$where .= " AND (".  $owner_where." or ".$group_where.") ";
    		}
    	}
    	/** end eggsurplus - SECURITYGROUPS */
		$query = $this->create_list_query($order_by, $where, $show_deleted);

		return $this->process_list_query($query, $row_offset, $limit, $max, $where);
	}

	/**
	* This function prefixes column names with this bean's table name.
	* This call can be ignored for  mysql since it does a better job than Oracle in resolving ambiguity.
	* All Rights Reserved..
	* Contributor(s): ______________________________________..
	*/
 	function process_order_by ($order_by, $submodule) {
 		if (empty($order_by)) return $order_by;
 		$bean_queried = "";
 		//submodule is empty,this is for list object in focus
 		if (empty($submodule)){
 			$bean_queried = &$this;
 		}else{
 			//submodule is set, so this is for subpanel, use submodule
 			$bean_queried = $submodule;
 		}
 		$elements = explode(',',$order_by);
 		foreach ($elements as $key=>$value) {

 			if (strchr($value,'.') === false) {
				//value might have ascending and descending decorations
				$list_column = explode(' ',trim($value));
				if (isset($list_column[0])) {
					$list_column_name=trim($list_column[0]);
					$listField = $bean_queried->getFieldDefinition($list_column_name);
 					if (isset($listField)) {

 						// longreach - start added - fix sorting by certain virtual fields, like case age
 						if(isset($listField['sort_on']))
 							$list_column[0] = $list_column_name = $listField['sort_on'];
 						// longreach - end added

 						$source=isset($listField['source']) ? $listField['source'] : 'db';
 						if (empty($listField['table']) && $source=='db') {
	 						$list_column[0] = $bean_queried->table_name .".".$list_column[0] ;
 						}
 						$value = implode(' ', $list_column);
 					} else {
 						$GLOBALS['log']->debug("process_order_by: ($list_column[0]) does not have a vardef entry.");
 					}
				}
 			}
 			$elements[$key]=$value;
 		}
 		return implode(',', $elements);

 	}


	function get_related_list($child_seed,$related_field_name, $order_by = "", $where = "",
		$row_offset = 0, $limit=-1, $max=-1, $show_deleted = 0) {
		$GLOBALS['log']->debug("get_related_list:  order_by = '$order_by' and where = '$where' and limit = '$limit'");
		if(isset($_SESSION['show_deleted'])){
			$show_deleted = 1;
		}

		$this->load_relationship($related_field_name);
		$query_array = $this->$related_field_name->getQuery(true);
		$entire_where = $query_array['where'];
		if(!empty($where))
		{
			if(empty($entire_where))
			{
				$entire_where = ' WHERE ' . $where;
			}
			else
			{
				$entire_where .= ' AND ' . $where;
			}
		}

		$query = 'SELECT '.$child_seed->table_name.'.* ' . $query_array['from'] . ' ' . $entire_where;
		if(!empty($order_by))
		{
			$query .= " ORDER BY " . $order_by;
		}

		return $child_seed->process_list_query($query, $row_offset, $limit, $max, $where);
	}


	/**
	* This function returns a full (ie non-paged) list of the current object type.
	* Portions created by SugarCRM are Copyright (C) SugarCRM, Inc..
	* All Rights Reserved..
	* Contributor(s): ______________________________________..
	*/
	function get_full_list($order_by = "", $where = "", $check_dates=false, $show_deleted = 0) {
		$GLOBALS['log']->debug("get_full_list:  order_by = '$order_by' and where = '$where'");
		if(isset($_SESSION['show_deleted'])){
			$show_deleted = 1;
		}
		$query = $this->create_list_query($order_by, $where, $show_deleted);
		return $this->process_full_list_query($query, $check_dates);
	}

	function create_list_query($order_by, $where, $show_deleted = 0)
	{

		$query = "SELECT ";
		$query .= " $this->table_name.* FROM $this->table_name ";

		 $where_auto = '1=1';
				if($show_deleted == 0){
                	$where_auto = "$this->table_name.deleted=0";
				}else if($show_deleted == 1){
                	$where_auto = "$this->table_name.deleted=1";
				}
		if($where != "")
		$query .= "where ($where) AND $where_auto";
		else
		$query .= "where $where_auto";

		if(!empty($order_by))
		$query .= " ORDER BY $order_by";


		return $query;
	}



	function process_list_query($query, $row_offset, $limit= -1, $max_per_page = -1, $where = '')
	{
		$arg_list = func_get_args();
		$toEnd = $row_offset == -100;
		$GLOBALS['log']->debug("process_list_query: ".$query);
		if($max_per_page == -1){
			$max_per_page = AppConfig::setting('layout.list.max_entries_per_page');
		}
		// Check to see if we have a count query available.
		if(! AppConfig::setting('site.performance.disable_count_query') || $toEnd){
			$count_query = $this->create_list_count_query($query);
			if(!empty($count_query) && (empty($limit) || $limit == -1))
			{
				// We have a count query.  Run it and get the results.
				$result = $this->db->query($count_query, true, "Error running count query for $this->object_name List: ");
				$assoc = $this->db->fetchByAssoc($result);
				if(!empty($assoc['c']))
				{
					$rows_found = $assoc['c'];
					$limit = $max_per_page + 1;
				}
				if( $toEnd){

					$row_offset = (floor(($rows_found -1) / $limit)) * $limit;

				}
			}

		}else{
			if((empty($limit) || $limit == -1)){
				$limit = $max_per_page + 1;
				$max_per_page = $limit;
			}

		}

		if(empty($row_offset))
		{
			$row_offset = 0;
		}
		if(!empty($limit) && $limit != -1 && $limit != -99){

			$result = $this->db->limitQuery($query, $row_offset, $limit,true,"Error retrieving $this->object_name list: ");
		}else{
			$result = $this->db->query($query,true,"Error retrieving $this->object_name list: ");
		}

		$list = Array();

		if(empty($rows_found))
		{
  			$rows_found =  $this->db->getRowCount($result);
		}

		$GLOBALS['log']->debug("Found $rows_found ".$this->object_name."s");

		$previous_offset = $row_offset - $max_per_page;
		$next_offset = $row_offset + $max_per_page;

		$class = get_class($this);
		if($rows_found != 0 or $this->db->dbType != 'mysql')
		{
//todo Bug? we should remove the magic number -99
	//use -99 to return all
			$index = $row_offset;
			while ($max_per_page == -99 || ($index < $row_offset + $max_per_page)) {

				//if(!empty($sugar_config['disable_count_query'])){
					$row = $this->db->fetchByAssoc($result);
				//}else{
				//	$row = $this->db->fetchByAssoc($result, $index);
				//}
					if (empty($row)) {
						break;
					}

				    //instantiate a new class each time. This is because php5 passes
			    	//by reference by default so if we continually update $this, we will
				    //at the end have a list of all the same objects
					$temp = new $class();

					foreach($this->getFieldDefinitions() as $field=>$value)
					{
						if (isset($row[$field])) {
							$temp->$field = $row[$field];
							$owner_field = $field . '_owner';
							if(isset($row[$owner_field])){
								$temp->$owner_field = $row[$owner_field];
							}

							$GLOBALS['log']->debug("$temp->object_name({$row['id']}): ".$field." = ".$temp->$field);
						}else if (isset($row[$this->table_name .'.'.$field])) {
							$temp->$field = $row[$this->table_name .'.'.$field];
						}
						else
						{
							$temp->$field = "";
						}
					}

					$temp->fill_in_additional_list_fields();
					$list[] = $temp;

					$index++;
			}
		}
		if(AppConfig::setting('site.performance.disable_count_query') && !empty($limit)){

			$rows_found = $row_offset + count($list);

			unset($list[$limit - 1]);
			if(!$toEnd){
				$next_offset--;
				$previous_offset++;
			}
		}
		$response = Array();
		$response['list'] = $list;
		$response['row_count'] = $rows_found;
		$response['next_offset'] = $next_offset;
		$response['previous_offset'] = $previous_offset;
		$response['current_offset'] = $row_offset ;
		return $response;
	}


	function process_full_list_query($query, $check_date=false)
	{

		$GLOBALS['log']->debug("process_full_list_query: query is ".$query);
		$result = $this->db->query($query, false);
		if($result === false) {
			$GLOBALS['log']->debug("process_full_list_query: query failed");
			return false;
		}
		$class = get_class($this);
		$isFirstTime = true;
		$bean = new $class();

		//if($this->db->getRowCount($result) > 0){


			// We have some data.
			//while ($row = $this->db->fetchByAssoc($result)) {
			while (($row = $bean->db->fetchByAssoc($result)) != null) {
				if(!$isFirstTime){
					$bean = new $class();
				}
				$isFirstTime = false;

				foreach($bean->getFieldDefinitions() as $field=>$value)
				{
					if (isset($row[$field])) {
						$bean->$field = $row[$field];

						$GLOBALS['log']->debug("process_full_list: $bean->object_name({$row['id']}): ".$field." = ".$bean->$field);
					}else {
						$bean->$field = '';
					}
				}
                $bean->fetched_row = $row;
				if($check_date){
					$bean->processed_dates_times = array();
					$bean->check_date_relationships_load();
				}
					$bean->fill_in_additional_list_fields();

				$list[] = $bean;
			}
		//}

		if (isset($list)) return $list;
		else return null;
	}

	/**
	* Track the viewing of a detail record.  This leverages get_summary_text() which is object specific
	* params $user_id - The user that is viewing the record.
	* Portions created by SugarCRM are Copyright (C) SugarCRM, Inc..
	* All Rights Reserved..
	* Contributor(s): ______________________________________..
	*/
	function track_view($user_id, $current_module) {}

	/**
	* return the summary text that should show up in the recent history list for this object.
	* Portions created by SugarCRM are Copyright (C) SugarCRM, Inc..
	* All Rights Reserved..
	* Contributor(s): ______________________________________..
	*/
	function get_summary_text()
	{
		return "";
	}

	/**
	* This is designed to be overridden and add specific fields to each record.  This allows the generic query to fill in
	* the major fields, and then targetted queries to get related fields and add them to the record.  The contact's account for instance.
	* This method is only used for populating extra fields in lists
	* Portions created by SugarCRM are Copyright (C) SugarCRM, Inc..
	* All Rights Reserved..
	* Contributor(s): ______________________________________..
	*/
	function fill_in_additional_list_fields()
	{
	}

	/**
	* This is designed to be overridden and add specific fields to each record.  This allows the generic query to fill in
	* the major fields, and then targetted queries to get related fields and add them to the record.  The contact's account for instance.
	* This method is only used for populating extra fields in the detail form
	* Portions created by SugarCRM are Copyright (C) SugarCRM, Inc..
	* All Rights Reserved..
	* Contributor(s): ______________________________________..
	*/
	function fill_in_additional_detail_fields()
	{
	}

	/** This function should be overridden in each module.  It marks an item as deleted.
	* If it is not overridden, then marking this type of item is not allowed
	* Portions created by SugarCRM are Copyright (C) SugarCRM, Inc..
	* All Rights Reserved..
	* Contributor(s): ______________________________________..
	*/
	function mark_deleted($id)
	{
        if (! empty($this->bean_name)) {
            $bean_name = $this->bean_name;
        } else {
            $bean_name = $this->object_name;
        }
		$result = ListQuery::quick_fetch($bean_name, $id);
		if(! $result)
			return false;
		$upd = RowUpdate::for_result($result);
		$upd->prohibit_workflow = $this->prohibit_workflow;
		if(! $upd->markDeleted())
			return false;		
		
		$this->mark_relationships_deleted($id);

		return true;
	}

	function mark_undeleted($id)
	{
        if (! empty($this->bean_name)) {
            $bean_name = $this->bean_name;
        } else {
            $bean_name = $this->object_name;
        }
		$result = ListQuery::quick_fetch($bean_name, $id);
		if(! $result)
			return false;
		$upd = RowUpdate::for_result($result);
		$upd->prohibit_workflow = $this->prohibit_workflow;
		if(! $upd->markDeleted(false))
			return false;

		return true;
	}

	/** This function deletes relationships to this object.  It should be overridden to handle the relationships of the specific object.
	* This function is called when the item itself is being deleted.  For instance, it is called on Contact when the contact is being deleted.
	* Portions created by SugarCRM are Copyright (C) SugarCRM, Inc..
	* All Rights Reserved..
	* Contributor(s): ______________________________________..
	*/
	function mark_relationships_deleted($id)
	{
		$this->delete_linked($id);
	}

	/**
	* This function is used to execute the query and create an array template objects from the resulting ids from the query.
	* It is currently used for building sub-panel arrays.
	* param $query - the query that should be executed to build the list
	* param $template - The object that should be used to copy the records.
	* Portions created by SugarCRM are Copyright (C) SugarCRM, Inc..
	* All Rights Reserved..
	* Contributor(s): ______________________________________..
	*/
	// longreach - modified - added $encode parameter
	function build_related_list($query, &$template, $row_offset = 0, $limit = -1, $encode=true)
	{
		$GLOBALS['log']->debug("Finding linked records $this->object_name: ".$query);

        if(!empty($row_offset) && $row_offset != 0 && !empty($limit) && $limit != -1){
            $result = $this->db->limitQuery($query, $row_offset, $limit,true,"Error retrieving $template->object_name list: ");
        }else{
		    $result = $this->db->query($query, true);
        }

		$list = Array();
		$isFirstTime = true;
		$class = get_class($template);



		while($row = $this->db->fetchByAssoc($result))
		{
			if(!$isFirstTime){
				$template = new $class();



			}
			$isFirstTime = false;

			// longreach - modified - added $encode parameter
			$record = $template->retrieve($row['id'], $encode);

			if($record != null)
			{
				// this copies the object into the array
				$list[] = $template;
			}
		}

		return $list;
	}

	// longreach - start added
	function get_related_list_size($query, &$template, $where='', $in='')
	{
		// No need to do an additional query
		$GLOBALS['log']->debug("Finding linked records $this->object_name: ".$query);
		if(empty($in) && !empty($query)){

			$idList = $this->build_related_in($query);
			$in = $idList['in'];
			if ($in == '()') $in = '(NULL) AND 0';
		}
		$query = "SELECT COUNT(id) AS c FROM $this->table_name WHERE deleted=0 AND id IN $in";
		if(!empty($where)){
			$query .= " AND $where";
		}
		$result = $this->db->query($query, true);
		$row = $this->db->fetchByAssoc($result);
		return (int)$row['c'];
	}

		/**
	* This function is used to execute the query and create an array template objects from the resulting ids from the query.
	* It is currently used for building sub-panel arrays. It supports an additional where clause that is executed as a filter on the results
	*
	* param $query - the query that should be executed to build the list
	* param $template - The object that should be used to copy the records.
	* Portions created by SugarCRM are Copyright (C) SugarCRM, Inc..
	* All Rights Reserved..
	* Contributor(s): ______________________________________..
	*/
	function build_related_list_where($query, &$template, $where='', $in='', $order_by='', $limit='', $row_offset = 0)
	{
		// No need to do an additional query
		$GLOBALS['log']->debug("Finding linked records $this->object_name: ".$query);
		if(empty($in) && !empty($query)){

			$idList = $this->build_related_in($query);
			$in = $idList['in'];
			if ($in == '()') $in = '(NULL) AND 0';
		}
		$query = "SELECT id FROM $this->table_name WHERE deleted=0 AND id IN $in";
		if(!empty($where)){
			$query .= " AND $where";
		}
		if(!empty($order_by)){
			$query .= " ORDER BY $order_by";
		}
		if (!empty($limit)) {
			$result = $this->db->limitQuery($query, $row_offset, $limit,true,"Error retrieving $this->object_name list: ");
		}else{
            $result = $this->db->query($query, true);
        }

		$list = Array();
		$isFirstTime = true;
		$class = get_class($template);

        $disable_security_flag = ($template->disable_row_level_security) ? true : false;

		while($row = $this->db->fetchByAssoc($result))
		{
			if(!$isFirstTime){
				$template = new $class();
                $template->disable_row_level_security = $disable_security_flag;
			}
			$isFirstTime = false;
			$record = $template->retrieve($row['id']);

			if($record != null)
			{
				// this copies the object into the array
				$list[] = $template;
			}
		}

		return $list;
	}

	function build_related_in($query)
	{
		$idList = array();
		$result = $this->db->query($query, true);
		$ids = '';
		while($row = $this->db->fetchByAssoc($result))
		{
			$idList[] = $row['id'];
			if(empty($ids)){
				$ids = "('" . $row['id'] . "'";
			}else{
				$ids .= ",'" . $row['id'] . "'";
			}
		}
		if(empty($ids)){
			$ids = '(';
		}
		$ids .= ')';
		return array('list'=>$idList, 'in'=>$ids);
	}

	/**
	* Portions created by SugarCRM are Copyright (C) SugarCRM, Inc..
	* All Rights Reserved..
	* Contributor(s): ______________________________________..
	*/
	function build_related_list2($query, &$template, &$field_list)
	{
		$GLOBALS['log']->debug("Finding linked values $this->object_name: ".$query);

		$result = $this->db->query($query, true);

		$list = Array();
		$isFirstTime = true;
		$class = get_class($template);
		while($row = $this->db->fetchByAssoc($result))
		{
			// Create a blank copy
			$copy = $template;
			if(!$isFirstTime){
				$copy = new $class();
			}
			$isFirstTime = false;
			foreach($field_list as $field)
			{
				// Copy the relevant fields
				$copy->$field = $row[$field];

			}

			// this copies the object into the array
			$list[] = $copy;
		}

		return $list;
	}


	/* This function assigns all of the values into the template for the list view */
	function get_list_view_array(){
		return array();
	}

	function get_list_view_data()
	{
		return $this->get_list_view_array();
	}

	function get_where(&$fields_array)
	{
		$where_clause = "WHERE ";
		$first = 1;
		foreach ($fields_array as $name=>$value)
		{
			if ($first)
			{
				$first = 0;
			}
			else
			{
				$where_clause .= " AND ";
			}

			$where_clause .= "$name = '".PearDatabase::quote($value,false)."'";
		}

		$where_clause .= " AND deleted=0";
		return $where_clause;
	}


	function retrieve_by_string_fields($fields_array, $encode=true)
	{
		$where_clause = $this->get_where($fields_array);
		$query = "SELECT $this->table_name.* FROM $this->table_name ";
		$query .= " $where_clause";
		$GLOBALS['log']->debug("Retrieve $this->object_name: ".$query);
        //requireSingleResult has beeen deprecated.
		//$result = $this->db->requireSingleResult($query, true, "Retrieving record $where_clause:");
		$result = $this->db->limitQuery($query,0,1,true, "Retrieving record $where_clause:");


		if( empty($result))
		{
			return null;
		}
		if($this->db->getRowCount($result) > 1){
			$this->duplicates_found = true;
		}

		$row = $this->db->fetchByAssoc($result, -1, $encode);
		if(empty($row)){
			return null;
		}
		foreach($this->column_fields as $field)
		{
			if(isset($row[$field]))
			{
				$this->$field = $row[$field];
			}
		}
		$this->fill_in_additional_detail_fields();
		return $this;
	}


	/*
	 * 	RELATIONSHIP HANDLING
	 */
	function set_relationship($table_name, $relate_values) {
		$update = RowUpdate::for_model($table_name);
		$update->prohibit_workflow = $this->prohibit_workflow;
		$update->set($relate_values);
		if(! $update->validate())
			return false;
		return $update->putRow();
	}


	function add_link($link_name, $link_values, $link_key='', $as_user_id=null) {
		if(! is_array($link_values)) {
			if(empty($link_key))
				return;
			$link_values = array();
		}
		$link_model = $this->model_obj->getLinkModel($link_name);
		if(! $link_model)
			return false;
		$spec = $link_model->getLinkSpec();

		if($link_model->is_join) {
			$rel = $spec['relationship_spec'];

			$my_pk = $this->getFieldValue($rel['left']['key']);
			$set_pk = $rel['left']['join_key'];
			if(empty($link_values[$set_pk]))
				$link_values[$set_pk] = $my_pk;

			// FIXME - to ensure DB integrity, check that key exists in target table
			if(! empty($link_key)) {
				$set_pk = $rel['right']['join_key'];
				$link_values[$set_pk] = $link_key;
			}
			if(isset($rel['join']['role_field'])) {
				$role_f = $rel['join']['role_field'];
				if(! isset($link_values[$role_f]))
					$link_values[$role_f] = array_get_default($rel['join'], 'role_value');
			}

			$rowupd = RowUpdate::blank_for_model($link_model, $as_user_id);
			$rowupd->prohibit_workflow = $this->prohibit_workflow;
			$rowupd->set($link_values);
			if(! $rowupd->validate()) {
				return false;
			}
			if(! $rowupd->putRow()) {
				return false;
			}

			return true;
		}
		else {
			// need to update the target row - check permissions first?
			if(! empty($spec['bean_name'])) {
				require_bean($spec['bean_name']);
				$bean = new $spec['bean_name'];
				if($spec['type'] == 'link') {
					$lkey = $spec['relationship_spec']['left']['key'];
					$rkey = $spec['relationship_spec']['right']['key'];
					if(! empty($link_key) && $bean->retrieve($link_key)) {
						if($bean->ACLAccess('edit')) {
							$bean->$rkey = $this->getFieldValue($lkey);
							if($bean->save())
								return true;
						}
					}
				}
			}
		}
	}

	function remove_link($link_name, $link_key) {
		// NYI
	}


	function bean_implements($interface){
		if($interface == 'ACL') return true;
		return false;
	}


	/** eggsurplus - SECURITYGROUPS */
	function ACLAccess($view,$is_owner='not_set',$in_group='not_set'){

		global $current_user;

		if($is_owner == 'not_set'){
            /* longreach - removed
			$is_owner = $this->isOwner($current_user->id);
             */
            /* longreach - start added */
            if (isset($this->assigned_user_id)) $is_owner = $this->assigned_user_id;
            elseif (isset($this->created_by)) $is_owner = $this->created_by;
            else $is_owner = $current_user->id;
            /* longreach - end added */
		}
    	if($in_group == 'not_set')
    	{
			require_once("modules/SecurityGroups/SecurityGroup.php");
			$in_group = SecurityGroup::groupHasAccess($this->module_dir,$this->id);
    	}

		// longreach - added - nobody can delete users
		if($this->module_dir == 'Users' && strtolower($view) == 'delete') return false;
		//if we don't implent acls return true
        if(!$this->bean_implements('ACL'))
		    return true;

		switch ($view){
			case 'list':
			case 'index':
			case 'ListView':
			    return ACLController::checkAccess($this->module_dir,'list', true, $in_group);
			case 'edit':
			case 'Save':
			case 'PopupEditView':
			case 'EditView':
                return ACLController::checkAccess($this->module_dir,'edit', $is_owner, $in_group);
			case 'view':
			case 'DetailView':
			    return ACLController::checkAccess($this->module_dir,'view', $is_owner, $in_group);
			case 'delete':
			case 'Delete':
			    return ACLController::checkAccess($this->module_dir,'delete', $is_owner, $in_group);
			case 'export':
			case 'Export':
			    return ACLController::checkAccess($this->module_dir,'export', $is_owner, $in_group);
			case 'import':
			case 'Import':
			    return ACLController::checkAccess($this->module_dir,'import', true, $in_group);

		}
		//if it is not one of the above views then it should be implemented on the page level
		return true;
	}
	/** end eggsurplus - SECURITYGROUPS */

	/**
	 * function isOwner($user_id)
	 *
	 * returns true of false if the user_id passed is the owner
	 *
	 * @param GUID $user_id
	 * @return boolean
	 */
	function isOwner($user_id){
		//if we don't have an id we must be the owner as we are creating it
		if(!isset($this->id)){
			return true;
		}
		//if there is an assigned_user that is the owner
		if(isset($this->assigned_user_id)){

			if($this->assigned_user_id == $user_id) return true;
			return false;
		}else{
			//other wise if there is a created_by that is the owner
			if(isset($this->created_by) && $this->created_by == $user_id){
				return true;
			}
		}



		return false;
	}

	/**
	 * function getOwnerWhere($user_id)
	 * gets there where statement for checking if a user is an owner
	 *
	 * @param GUID $user_id
	 * @param $level
	 * @param $tableName
	 * @param $field
	 * @return STRING
	 */
    /* longreach - replaced */

	function getOwnerWhere($user_id, $level, $tableName = null, $field = '') {
        $field = '';
   		if($this->getFieldDefinition('assigned_user_id')) $field = '.assigned_user_id';
        elseif($this->getFieldDefinition('created_by')) $field = '.created_by';
        if (empty($field)) return '';
        $users = ACLAction::getAllowedUsers($user_id, $level);
        if (empty($users)) return ' 0 ';
        if (empty($tableName)) $tableName = $this->table_name;
        return " $tableName$field IN ('" . join("','", $users) . "') ";
	}


	/**
	 * function toArray()
	 * returns this bean as an array
	 *
	 * @return array of fields with id, name, access and category
	 */
		function toArray($dbOnly = false, $stringOnly = false, $upperKeys=false){

		$arr = array();

		foreach($this->getFieldDefinitions() as $field=>$data){
			if( !$dbOnly || !isset($data['source']) || $data['source'] == 'db')
				if(!$stringOnly || is_string($this->$field))
					if($upperKeys){
						$arr[strtoupper($field)] = $this->$field;
					}else{
						$arr[$field] = isset($this->$field) ? $this->$field : null;
					}
		}

		return $arr;
	}

	/**
	 * function fromArray($arr)
	 * converts an array into an acl mapping name value pairs into files
	 *
	 * @param Array $arr
	 */
	function fromArray($arr){
		foreach($arr as $name=>$value){
			$this->$name = $value;
		}
	}



	function loadFromRow($arr){
		$this->_parent_fields_loaded = false;
		$this->populateFromRow($arr);
		$this->processed_dates_times = array();
		$this->check_date_relationships_load();
		$this->fill_in_additional_list_fields();
	}


	function populateDefaultValues($force=false){
		$field_defs = $this->getFieldDefinitions();
		if (empty($field_defs)) {
			return;
		}
    	foreach($field_defs as $field=>$value){
    		$default_value = !empty($value['default']) ? $value['default'] : (!empty($value['default_value']) ? $value['default_value'] : null);
    		if(($force || empty($this->$field)) && (!empty($default_value))){
    		    $type = !empty($value['custom_type']) ? $value['custom_type'] : $value['type'];
		    	switch($type){
		    		case 'date':
						global $timedate;
						require_once('modules/DynFields/templates/Fields/TemplateDate.php');
						$td = new TemplateDate();
						if(!empty($td->dateStrings[$default_value]))
							$this->$field = $timedate->to_display_date(date('Y-m-d',strtotime($td->dateStrings[$default_value])), false);
						else
							$this->$field = '';
						break;
					default:
						$this->$field = $default_value;
		    	} //switch
    		}
    	} //foreach
   }


	// longreach - start added


    // determine if a field value has changed since the object was retrieved
    // must be called after unformat_all_fields() and before save()
    function field_value_has_changed($fieldname) {
    	if(! isset($this->fetched_row) || ! is_array($this->fetched_row))
    		return -1;
    	if(! array_key_exists($fieldname, $this->fetched_row))
    		return -1;
    	$value = isset($this->$fieldname) ? $this->$fieldname : '';
    	if($this->fetched_row[$fieldname] == $value)
    		return 0;
    	return 1;
    }

	function get_recurrence_forward_instances()
	{
		return 50;
	}

	function get_recurrence_scheduled_interval()
	{
		return 3600*24*30; // for one month by befault
	}

	function get_recurrence_date_field()
	{
		return 'date_start';
	}

	function get_recurrence_time_field()
	{
		return 'time_start';
	}

	function recur_pre_save() {}

	function cleanup() {
		if(! empty($this->_cleaned_up))
			return;
		if(isset($this->field_defs)) {
			global $_cleanup_fields;
			if(! isset($_cleanup_fields)) {
				$_cleanup_fields = array();
			}
			if(! isset($_cleanup_fields[$this->object_name])) {
				$_cleanup_fields[$this->object_name] = array();
				foreach($this->field_defs as $k=>$fd) {
					if(array_get_default($fd, 'type') == 'link') {
						$_cleanup_fields[$this->object_name][] = $k;
					}
				}
			}
			foreach($_cleanup_fields[$this->object_name] as $k) {
				if(isset($this->$k) && ($this->$k instanceof Link || $this->$k instanceof SugarBean)) {
					$this->$k->cleanup();
					unset($this->$k);
				}
			}
		}
		unset($this->db);
		$this->_cleaned_up = true;
		/*$s = print_r($this, true);
		if(strpos($s, '*RECUR') !== false) {
			echo $s;
			exit;
		}*/
	}

	function cleanup_list(&$lst) {
		if($lst && is_array($lst))
			foreach(array_keys($lst) as $k)
				if($lst[$k] instanceof SugarBean || $lst[$k] instanceof Link)
					$lst[$k]->cleanup();
	}

    /**
     * Update activity's parent record
     * Activity - modules like Calls, Meetings, Email, Tasks and Notes
     *
     * @static
     * @param RowUpdate $upd
     */
    static function update_parent_activity(RowUpdate $upd) {
        require_once('modules/Activities/ActivityUpdater.php');
        $updater = new ActivityUpdater($upd);
        $updater->update();
    }

    /**
     * Set Email's parent record
     *
     * @static
     * @param RowUpdate $upd
     */
	static function set_email_parent(RowUpdate $upd) {
    	if (array_get_default($_REQUEST, 'return_module') != 'Emails' || empty($_REQUEST['return_record']))
    		return;
        $email_result = ListQuery::quick_fetch('Email', $_REQUEST['return_record']);
        if (! $email_result)
        	return;
        
        $parent_type = $upd->getModuleDir();
        $parent_id = $upd->getPrimaryKeyValue();
        $update = array();
        if($parent_type == 'Contacts') {
			if(! $email_result->getField('contact_id') || $upd->getRelatedData('force_email_account')) {
				if ($parent_id)
					$update['contact_id'] = $parent_id;
				$upd->setRelatedData('force_email_account', null);
				if( ($acc_id = $upd->getField('primary_account_id')) && ! $email_result->getField('account_id')) {
					$update['account_id'] = $acc_id;
				}
			}
        }
        $eptype = $email_result->getField('parent_type');
        $epid = $email_result->getField('parent_id');
        if(! $epid || (($eptype == 'Contacts' || $eptype == $parent_type) && $parent_id && $parent_id != $epid)) {
        	$update['parent_type'] = $parent_type;
        	$update['parent_id'] = $parent_id;
        }
        
        if($update) {
			$email_upd = RowUpdate::for_result($email_result);
			$email_upd->prohibit_workflow = true;
			$email_upd->set($update);
			$email_upd->save();
		}
    }
}
?>
