<?php

mb_internal_encoding('utf-8');

if(! is_dir('html/search-index'))
	die();

$zapi_path = getcwd()."/../include/ZendApi";
set_include_path(get_include_path() . PATH_SEPARATOR . $zapi_path);

require_once('Zend/Search/Lucene.php');
Zend_Search_Lucene_Analysis_Analyzer::setDefault(
	new Zend_Search_Lucene_Analysis_Analyzer_Common_Utf8_CaseInsensitive());
try {
	$index = Zend_Search_Lucene::open('html/search-index');
} catch(Exception $e) {
	die('{"count": 0, "error": "'.$e->getMessage().'"}');
}

$query = isset($_REQUEST['query']) ? $_REQUEST['query'] : '';
$max_results = intval(isset($_REQUEST['max_results']) ? $_REQUEST['max_results'] : '10');

require_once('Zend/Search/Lucene/Search/Highlighter/Default.php');
class HL extends Zend_Search_Lucene_Search_Highlighter_Default {
	protected $_highlightColors = array('#ffffcc');
}
$hl = new HL();

require_once('../include/JSON.php');
$json = new JSON(JSON_LOOSE_TYPE);

try {
	Zend_Search_Lucene_Search_QueryParser::setDefaultEncoding('utf-8');
	$query = Zend_Search_Lucene_Search_QueryParser::parse($query);
	$hits = $index->find($query);
} catch(Exception $e) {
	die('{"count": 0, "error": "'.$e->getMessage().'"}');
}

function excerpt($value) {
	$lines = explode("\n", $value);
	$ret = array();
	$max_line_len = 150;
	foreach($lines as $l) {
		$len = mb_strlen($l);
		if(! $len || ($p = strpos($l, '<b ')) === false)
			continue;
		if($len > $max_line_len) {
			$left = max(0, $p - ($max_line_len / 2));
			$right = $left + $max_line_len;
			$p2 = strpos($l, '</b>', $p);
			if($p2 === false) $p2 = $len;
			$p2 += 4;
			if($p2 > $right)
				$right = $p2;
			$l2 = mb_substr($l, $left, $right - $left);
			if($left) {
				$l2 = preg_replace('~^\w+~', '', $l2);
				if(! count($ret))
					$l2 = '.. '.$l2;
			}
			if(mb_substr($l, $right, 1) != ' ' && mb_substr($l2, -1) != ' ')
				$l2 = preg_replace('~\w+$~', '', $l2);
			$p3 = strrpos($l2, '<');
			if($p3 !== false) {
				$p4 = strrpos($l2, '</b>');
				if($p4 === false || $p4 < $p3)
					$l2 = substr($l2, 0, $p3);
			}
			$l = $l2 . ' ..';
		}
		$ret[] = $l;
		if(count($ret) == 3)
			break;
	}
	return implode("\n", $ret);
}

$ret = array('count' => 0, 'totalCount' => count($hits), 'terms' => '', 'results' => array());
$resultCount = 0;
foreach($hits as $hit) {
	$highlighted = $query->htmlFragmentHighlightMatches($hit->body, 'utf-8', $hl);
	$body = excerpt($highlighted);
	$ret['results'][] = array(
		'score' => $hit->score,
		'file' => $hit->file,
		'bookmark' => $hit->bookmark,
		'title' => $hit->title,
		'body' => $body,
	);
	if(++ $resultCount >= $max_results)
		break;
}
$ret['count'] = $resultCount;
$rewrite = $query->rewrite($index);
$terms = array();
foreach($rewrite->getQueryTerms() as $t)
	$terms[] = $t->text;
$ret['terms'] = implode(',', $terms);

header('Content-type: application/json; charset=UTF-8');
echo $json->encode($ret);
