<?php
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/
define('sugarEntry', true);


$query = null;
$local_location = null;

$ltype = strtolower($_REQUEST['type'] ?? '');
$signature = $_REQUEST['g'] ?? '';

$signed = $signature && $ltype === 'email';

if ($signed) {
	define('ONECRM_CSRF_PROTECT', 'none');
}

require_once('include/entryPoint.php');
global $locale;

// Prevent access via alternate site urls
if(! AppConfig::installed() || AppConfig::upgrade_in_progress()
  || AppConfig::is_extern_url() || AppConfig::get_auto_redirect())
	return;

if(! empty($_REQUEST['check_download_id'])) {
	require_once('include/DLManager.php');
	$check_ids = $_REQUEST['check_download_id'];
	if(! is_array($check_ids)) $check_ids = array($check_ids);
	DLManager::handle_check_downloads($check_ids);
	return;
}

ob_start();

session_cache_limiter('public');
AppConfig::session_start();

if(empty($_REQUEST['id']) || empty($_REQUEST['type'])) {
	die("Not a Valid Entry Point");
} else if(empty($_SESSION['authenticated_user_id']) && !$signed) {
	die("Authentication required");
} else {
	// cn: bug 8753: current_user's preferred export charset not being honored
	if (!empty($_SESSION['authenticated_user_id']))
		$current_user->retrieve($_SESSION['authenticated_user_id']);
	$current_language = $_SESSION['authenticated_user_language'] ?? '';
	$app_strings = return_application_language($current_language);
	$local_location = AppConfig::upload_dir().$_REQUEST['id'];

	if ($ltype == 'documents'){
        $query = get_document_file_query($_REQUEST['id'], true);
    }
    else if($ltype == 'documentrevisions'){
        // cn: bug 9674 document_revisions table has no 'name' column.
        $query = get_document_file_query($_REQUEST['id']);
    }
    else if($ltype == 'notes') {
		$query = new ListQuery("Note");
		$query->addSimpleFilter("id", $_REQUEST['id']);
    }
	else if ($ltype == 'backups') {
		if (!$current_user->is_admin)
			die($app_strings['ERR_INVALID_FILE_REFERENCE']);
		$filename = $_REQUEST['id'];
		if (strpos($filename, '/') === false && strpos($filename, '\\') === false && strpos($filename, '..') === false) {
			$backupDir = rtrim(AppConfig::setting('site.paths.backup_dir'), '/');
			$local_location = $backupDir . '/' . $filename;
			$row = [
				'name' => $filename,
				'file_mime_type' => 'application/zip',
			];
		}
    }
	else if ($ltype == 'email') {
		$filename = $_REQUEST['id'];
		$uid = AppConfig::unique_key();
		$hash = sha1($filename . '/' . $uid);
		$verified = !$signature || $hash === $signature;
		if (strpos($filename, '..') === false && $verified) {
			$dir = rtrim(AppConfig::files_dir(), '/');
			$local_location = $dir . '/email/' . $filename;
			$row = [
				'name' => basename($filename),
			];
		}
    }
    else if($ltype == 'incoming') {
		$query = new ListQuery("incoming_files", ["id", "file_mime_type"]);
		$query->addField('upload_filename', 'name');
		if (empty($_REQUEST['thumb'])) {
			$query->addField('filename');
		} else {
			$query->addFieldLiteral('filename', 'IFNULL(thumbnail_filename, filename)');
		}
		$query->addSimpleFilter("id", $_REQUEST['id']);
		$query->addSimpleFilter("user_id", AppConfig::current_user_id());
		$query->addSimpleFilter("is_image", 1);
	}
	else {
		$model_name = AppConfig::module_primary_bean($_REQUEST['type']);
		if($model_name) {
			require_bean($model_name);
			$seed = new $model_name;
			if(method_exists($seed, 'can_download') && method_exists($seed, 'get_local_filename')) {
				if($seed->retrieve($_REQUEST['id']) && $seed->can_download()) {
					$local_location = $seed->get_local_filename();
					$row = array('name' => $seed->get_display_filename());
				}
			} else if(! empty($_REQUEST['field'])) {
				$fname = $_REQUEST['field'];
				$def = AppConfig::setting("model.fields.$model_name.$fname");
				if($def && $def['type'] == 'file_ref' && empty($def['protected']) && empty($def['disable_download'])) {
					$query = new ListQuery($model_name, array('id', 'filename' => $fname), array('acl_filters' => 'view'));
					$query->addFilterPrimaryKey($_REQUEST['id']);
				}
			}
		}
		if(! isset($local_location))
			die($app_strings['ERR_INVALID_FILE_REFERENCE']);
	}

	if(! empty($query)) {
		$row = $query->runQuerySingle();
		if(!$row || $row->failed){
			die($app_strings['ERR_INVALID_FILE_REFERENCE']);
		}
		$row = $row->row;

		if(! isset($row['name']))
			$row['name'] = $row['filename'];
		if(strtolower($_REQUEST['type']) == 'notes') {
			if(preg_match('~\.(\w+)$~', $row['filename'], $m)) {
				$ext = strtolower($m[1]);
				if($ext == 'jpeg') $ext = 'jpg';
				if(preg_match('~.+\.(\w+)$~', $row['name'], $m)) {
					$ext2 = strtolower($m[1]);
					if($ext2 == 'jpeg') $ext2 = 'jpg';
					if($ext2 == $ext)
						$keep_name = true;
				}
			}
			if(empty($keep_name))
				$row['name'] = $row['filename'];
			else {
				$prefix = translate('LBL_EMAIL_ATTACHMENT', 'Emails');
				$row['name'] = str_replace($prefix.': ', '', $row['name']);
			}
		}
		// strip leading subdirectory name
		$row['name'] = preg_replace('~^(.*/)+~', '', $row['name']);

		require_once 'include/upload_file.php';
		if($ltype == 'incoming') {
			$local_location = CacheManager::get_location('incoming/') . $row['filename'];
		} else if (empty($row['email_attachment'])) {
			$local_location = UploadFile::get_path($row['filename'], $row['id']);
		} else {
			$local_location = AppConfig::files_dir() . $row['filename'];
			if (!file_exists($local_location)) {
				$local_location = UploadFile::get_path($row['filename'], $row['id']);
			}
		}
	}
	// longreach - end added


	if(!file_exists( $local_location )) {
		die($app_strings['ERR_INVALID_FILE_REFERENCE']);
	} elseif(strpos($local_location, "../") || strpos($local_location, "..\\") ) {
		die($app_strings['ERR_INVALID_FILE_REFERENCE']);
	} else {
		$name = download_safe_filename($row['name']);
		if(!empty($_COOKIE['ANDROID_APP']) || !empty($_COOKIE['IOS_APP']))
			$name = $row['name'];
		$mtime = @filemtime($local_location);
		if($mtime !== false) {
			$gmt_mtime = date('D, d M Y H:i:s', $mtime) . ' GMT';
			$last_mod = array_get_default($_SERVER, 'HTTP_IF_MODIFIED_SINCE');
			if($last_mod && $last_mod == $gmt_mtime) {
				end_clean_all_buffers();
				header("HTTP/1.1 304 Not Modified");
				exit;
			}
		}

		if(empty($row['file_mime_type'])){
			$file_mime_type = 'application/force-download';
		} else {
			$file_mime_type = $row['file_mime_type'];
		}

		if(isset($_REQUEST['inline']))
			$inline = !! $_REQUEST['inline'];
		else
			$inline = ($current_user->getPreference('file_download_mode') == 'open');

		session_write_close();

		$compress = AppConfig::get_server_compression();
		end_clean_all_buffers();

		header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
		if(! $compress)
			header("Content-Length: " . filesize($local_location));
		header("Content-Type: " . $file_mime_type);
		if(! empty($gmt_mtime))
			header("Last-Modified: $gmt_mtime");

		if (! $inline) {
			header("Content-Disposition: attachment; filename=\"".$name."\";");	// forces save dialog
		} else {
			header("Content-Disposition: inline; filename=\"".$name."\";");
		}
		header("Expires: 0");
		set_time_limit(0);

		readfile_chunked($local_location);
		if(! $compress) // corrupts output when ob_gzhandler is enabled
			@ob_flush();
	}
}

function get_document_file_query($id, $by_document = false) {
	if ($by_document) {
		$lq = new ListQuery("Document", true, ['link_name' => 'revisions']);
		$lq->addSimpleFilter("~parent.id", $id);
		$lq->addField('filename', 'name');
	} else {
		$lq = new ListQuery("DocumentRevision", true);
		$lq->addSimpleFilter("id", $id);
		$lq->addField('filename', 'name');
	}
	return $lq;
}
