<?php
/*
	Demo JSON client for info@hand
	Author: Andrew Whitehead
	The Long Reach Corp.
*/

include('../include/JSON.php');
$json = new JSON(JSON_LOOSE_TYPE);

include('../config.php');
$json_url = $sugar_config['site_url'].'/json.php';

session_name('JsonTest');
session_start();

$tests = array(
	'get_version',
	'get_supported_actions',
	'get_available_languages',
	'login',
	'logout',
);
$session_tests = array(
	'check_authenticated',
	'get_available_modules',
	'get_user_locale_settings',
	'get_module_fields',
	'get_full_list',
);

$action = isset($_REQUEST['test_action']) ? $_REQUEST['test_action'] : '';

function json_make_request($action, $parameters) {
	global $json, $json_url, $json_error, $json_headers;
	
	$request = $json_url . '?action=' .urlencode($action);
	$body = $json->encode($parameters);
	$headers = array('Content-Type: application/json; charset=UTF-8');
	$json_headers = array();
	
	$session = curl_init($request);
	curl_setopt($session, CURLOPT_POST, true);
	curl_setopt($session, CURLOPT_POSTFIELDS, $body);
	curl_setopt($session, CURLOPT_RETURNTRANSFER, true);
	curl_setopt($session, CURLOPT_TIMEOUT, 30);
	if($headers)
		curl_setopt($session, CURLOPT_HTTPHEADER, $headers);
	//curl_setopt($session, CURLOPT_HEADER, true); // return headers
	//curl_setopt($session, CURLOPT_SSL_VERIFYPEER, false);
	$response = curl_exec($session);
	
	if($response === false) {
		$json_error = array('curl', curl_errno($session), curl_error($session));
		curl_close($session);
		return false;
	}
	
	$code = curl_getinfo($session, CURLINFO_HTTP_CODE);
	if($code != 200) {
		$json_error = array('http', $code, $response);
		// 400 - bad request
		// 404 - not found
		// 501 - http method not supported
		curl_close($session);
		return false;
	}
	curl_close($session);
	$GLOBALS['json_raw_response'] =& $response;
	return $json->decode($response);
}

function show_response($response) {
	if($response === false) {
		echo "got error: ";
		print_r($GLOBALS['json_error']);
		echo "\n";
	}
	else {
		echo '['.gettype($response).'] ';
		print_r($response);
		echo "\n";
		//echo '(raw data) ';
		//print_r($GLOBALS['json_raw_response']);
		//echo "\n";
	}
}


function run_test() {
	global $action, $json_error;
	if(! $action)
		return;
	echo "<div class=\"body\">\n";
	$run = true;
	
	$params = array();
	if(! empty($_SESSION['json_session_id']))
		$params['json_session_id'] = $_SESSION['json_session_id'];

	if($action == 'login') {
		if(empty($_REQUEST['username'])) {
			echo <<<EOF
				User name: <input type="text" name="username" size="15"><br />
				Password: <input type="password" name="password" size="15"><br />
				<input type="submit" />
EOF;
			$run = false;
		}
		else {
			$params['username'] = $_REQUEST['username'];
			$params['md5_password'] = md5($_REQUEST['password']);
		}
	}
	
	if($action == 'get_full_list' || $action == 'get_module_fields') {
		$modules = array('', 'Bugs', 'Cases', 'Resources', 'Tasks');
		$mod = isset($_REQUEST['module']) ? $_REQUEST['module'] : '';
		$opts = '';
		foreach($modules as $m) {
			$sel = ($m == $mod) ? 'selected="selected"' : '';
			$opts .= "<option value=\"$m\" $sel>$m</option>";
		}
		echo <<<EOF
			Module: <select name="module">{$opts}</select>
			<input type="submit" />
EOF;
		if(! $mod)
			$run = false;
		else {
			$params['module'] = $_REQUEST['module'];
			echo '<hr>';
		}
	}
	
	if($action && $run) {
		$ret = json_make_request($action, $params);
		if($ret === false) {
			echo 'JSON error: ';
			print_r($json_error);
		}
		else {
			echo '<div class="response">';
			print_r($ret);
			echo '</div>';
			if($action == 'logout' && @$ret['result'] == 'ok') {
				$_SESSION['json_session_id'] = '';
				echo "Logout succeeded";
			}
			else if($action == 'login' && @$ret['result'] == 'ok') {
				$_SESSION['json_session_id'] = $ret['json_session_id'];
				$_SESSION['json_user_id'] = $ret['user_id'];
				echo "Login succeeded";
			}
		}
	}
	
	echo '</div>';
}

ob_start();
run_test();
$body = ob_get_contents();
ob_end_clean();

if(! empty($_SESSION['json_session_id']))
	$logged_in = '<div class="yes">Logged in</div>';
else
	$logged_in = '<div class="no">Not logged in</div>';


$action_options = '<optgroup label="Public actions">';
foreach($tests as $act) {
	$sel = ($act == $action) ? 'selected="selected"' : '';
	$action_options .= "<option value=\"$act\" $sel>$act</option>\n";
}
$action_options .= '</optgroup>';
if(! empty($_SESSION['json_session_id'])) {
	$action_options .= '<optgroup label="User actions">';
	foreach($session_tests as $act) {
		$sel = ($act == $action) ? 'selected="selected"' : '';
		$action_options .= "<option value=\"$act\" $sel>$act</option>\n";
	}
	$action_options .= '</optgroup>';
}


header('Content-Type: text/html; charset=UTF-8');

?>
<html>
<head>
<title>JSON Demo</title>
<style type="text/css">
	body {
		background-color: white;
		color: black;
	}
	.header, .body {
		width: 90%;
		margin: 1em auto;
		padding: 1em;
		border: 1px solid gray;
		background-color: lightgray;
	}
	.yes, .no {
		width: 30%;
		text-align: center;
		float: right;
	}
	.yes {
		color: green;
	}
	.no {
		color: gray;
	}
	.response {
		white-space: pre;
		overflow: auto;
	}
</style>
</head>
<body>
<form action="#" method="POST">
<div class="header">
	Select action: <select name="test_action" onchange="this.form.submit();"><option value=''>(No action)</option><?php echo $action_options; ?></select>
	<?php echo $logged_in; ?>
</div>
<?php echo $body; ?>
</form>
</body>
</html>

