<?php

require_once(__DIR__ . "/matrix.php");

class Captcha {
	private $text;
	private $chars = "befhABDEFGHQNRTL2345678";
	private $complexity = 0;
	const LETTER_SIZE = 28.346456692913386;
	const HEIGHT = 60;
	const WIDTH = 200;
	const NUM_LETTERS = 6;


	public function __construct($complexity) {
		$chars = str_split($this->chars);
		shuffle($chars);
		$this->text = substr(join('', $chars), 0, self::NUM_LETTERS);
		if ($complexity < 0)
			$complexity = 100;
		if ($complexity > 100)
			$complexity = 100;
		$this->complexity = $complexity / 100 * 50 + 50;
	}

	public function getText() {
		return $this->text;
	}

	public function generate() {
		$segments = [];
		$font = require_once(__DIR__ . "/font.php");
		for ($i = 0; $i < strlen($this->text); $i++) {
			$matrix = $this->generateMatrix();
			$offsetX = self::LETTER_SIZE * ($i + 1) ;
			$offsetY = self::LETTER_SIZE;
			$m = TransformMatrix::translate($offsetX, $offsetY)->multiply($matrix);
			$lines = $this->split($font[$this->text[$i]], self::HEIGHT / 80);
			$wave_period = 2 - rand() / getrandmax() * 100 / ($this->complexity + 1);
			$wave_amp = rand() / getrandmax() * 0.008 * $this->complexity;
			$lines = array_map(function($l) use($wave_period, $wave_amp) {
				return [
					$this->wave($l[0], $wave_period, $wave_amp),
					$this->wave($l[1], $wave_period, $wave_amp),
				];
			}, $lines);
			foreach ($lines as $l) {
				$segments[] = array_map(
						function($p) use($m) {
							return $m->transform($p[0], $p[1]);
						}, 
						$l);
			}
		}
		$segments = array_merge($segments, $this->generateLines());
		shuffle($segments);
		return $this->generateXML($segments);
		return $xml;
	}

	private function generateXML($segments) {
		$path = "";
		foreach ($segments as $s) {
			$start = $s[0];
			$end = $s[1];
			$path .= sprintf("M%f,%fL%f,%f", $start[0], $start[1], $end[0], $end[1]);
		}
		$xml = '<' . '?xml version="1.0" encoding="utf-8" ?' . '>
<svg xmlns="http://www.w3.org/2000/svg" xmlns:cc="http://creativecommons.org/ns" xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:ev="http://www.w3.org/2001/xml-events" xmlns:inkscape="http://www.inkscape.org/namespaces/inkscape" xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns" xmlns:sodipodi="http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd" xmlns:xlink="http://www.w3.org/1999/xlink" baseProfile="tiny"  version="1.2" width="' . self::WIDTH .'" height="'.self::HEIGHT.'">';
  		$xml .= '<g fill="none" id="layer1" inkscape:groupmode="layer" inkscape:label="1" stroke="#0000ff" stroke-width="1.5" style="display:inline">';
		$xml .= "<path d=\"$path\" />";
		$xml .= '</g></svg>';
		return $xml;
	}

	public function generateLines() {
		$segments = [];;
		for ($i = 0; $i < $this->complexity / 2; $i++) {
			$x1 = rand(0, self::WIDTH * 100) / 100;
			$y1 = rand(0, self::HEIGHT * 100) / 100;
			$x2 = $x1 + rand(0, 10000) / 1000;
			$y2 = $y1 + rand(0, 10000) / 1000;
			$segments[] = [[$x1,$y1], [$x2,$y2]];
		}
		return $segments;
	}

	private function wave($p, $period, $amp) {
		list ($x, $y) = $p;
		$dx = sin($y / $period) * $amp;
		return [$x + $dx, $y];
	}

	private function split($segments, $max_len) {
		$ret = [];
		foreach ($segments as $seg) {
			$x1 = $seg[0][0];
			$y1 = $seg[0][1];
			$x2 = $seg[1][0];
			$y2 = $seg[1][1];
			$len = sqrt(($x2-$x1) * ($x2-$x1) + ($y2-$y1) * ($y2-$y1));
			if ($len <= $max_len) {
				$ret[] = $seg;
				continue;
			}
			$nSteps = (int)($len / $max_len) + 1;
			$stepX = ($x2 - $x1) / $nSteps;
			$stepY = ($y2 - $y1) / $nSteps;
			$startX = $x1; $startY = $y1;
			while ($nSteps > 0) {
				$endX = $startX + $stepX;
				$endY = $startY + $stepY;
				$ret[] = [
					[$startX, $startY],
					[$endX, $endY],
				];
				$startX = $endX;
				$startY = $endY;
				$nSteps--;
			}
		}
		return $ret;
	}

	private function generateMatrix() {
		$angle = (rand() / getrandmax()  - 0.5) * 0.014 * $this->complexity;
		$scaleX = ((rand() / getrandmax()) * 0.8 - 0.4)  + 1;
		$scaleY = ((rand() / getrandmax()) * 0.8 - 0.4)  + 1;
		$shearX = (rand() / getrandmax()) * 0.6 / 100 * $this->complexity;
		$shearY = (rand() / getrandmax()) * 0.6 / 100 * $this->complexity;
		$offsetX = ((rand() / getrandmax()) * 0.2 -0.1) * self::LETTER_SIZE;
		$matrix = TransformMatrix::rotate($angle)
			-> multiply(TransformMatrix::shear($shearX, $shearY))
			-> multiply(TransformMatrix::scale($scaleX, $scaleY))
			-> multiply(TransformMatrix::translate($offsetX, 0))
			;
		return $matrix;
	}

}

