<?php

class TransformMatrix {
	private $m11, $m12, $dx;
	private $m21, $m22, $dy;

	public function __construct($m11, $m12, $dx, $m21, $m22, $dy) {
		$this->m11 = $m11;
		$this->m12 = $m12;
		$this->dx = $dx;;
		$this->m21 = $m21;
		$this->m22 = $m22;
		$this->dy = $dy;
	}

	public static function identity() {
		return new TransformMatrix(1, 0, 0, 0, 1, 0);
	}
	
	public function multiply($o)  {
		return new TransformMatrix (
			$this->m11*$o->m11 + $this->m12*$o->m21,
			$this->m11*$o->m12 + $this->m12*$o->m22,
			$this->m11*$o->dx + $this->m12*$o->dy + $this->dx,
			$this->m21*$o->m11 + $this->m22*$o->m21,
			$this->m21*$o->m12 + $this->m22*$o->m22,
			$this->m21*$o->dx + $this->m22*$o->dy + $this->dy
		);
	}
	public function transform($x, $y) {
		return [
			$x * $this->m11 + $y*$this->m12 + $this->dx,
			$x*$this->m21 + $y*$this->m22 + $this->dy,
		];
    }

	public static function rotate($angle) {
		return new TransformMatrix (
			cos($angle), cos($angle + pi() / 2), 0,
			sin($angle), sin($angle + pi()/2), 0
		);
	}
	
	public static function shear($sx, $sy)  {
		return new TransformMatrix (
			1 + $sx*$sy, $sx, 0,
			$sy, 1, 0
		);
	}

	public static function scale($sx, $sy) {
		return new TransformMatrix(
			$sx, 0, 0,
			0, $sy, 0
		);
	}

	public static function translate($tx, $ty) {
		return new TransformMatrix(
			1, 0, $tx,
			0, 1, $ty
		);
	}
}

