<?php

class CreditCardProcessorAuthNet extends CreditCardProcessor {

	const URL_PRODUCTION = 'https://api.authorize.net/xml/v1/request.api';
	const URL_SANDBOX    = 'https://apitest.authorize.net/xml/v1/request.api';

	static private $customerFields = array(
		'firstName',
		'lastName',
		'company',
		'address',
		'city',
		'state',
		'zip',
		'country',
		'phoneNumber',
		'faxNumber',
	);

	protected function __construct($params = array()) {
		parent::__construct($params);
	}


	protected function validate() {
		if (!isset($this->param_login))
			throw new CreditCardProcessorException('Missing Authorize.net login');
		if (!isset($this->param_password))
			throw new CreditCardProcessorException('Missing Authorize.net transaction key');
		if (!$this->param_refTransId && $this->param_refund)
			throw new CreditCardProcessorException('Missing reference transaction ID');
		if (!is_numeric($this->param_amount))
			throw new CreditCardProcessorException('Invalid amount');
		if (!isset($this->param_cardNumber))
			throw new CreditCardProcessorException('Missing card number');
		if (!isset($this->param_expirationMonth))
			throw new CreditCardProcessorException('Missing card expiration month');
		if (!isset($this->param_expirationYear))
			throw new CreditCardProcessorException('Missing card expiration year');
		if (!isset($this->param_cardCode))
			throw new CreditCardProcessorException('Missing card code');
	}

	protected function prepare() {
		if ($this->param_refund)
			return $this->prepareRefund();
		return $this->prepareCharge();
	}

	private function prepareCharge() {
		$data = array();
		$data['merchantAuthentication'] = array(
			'name' => $this->param_login,
			'transactionKey' => $this->param_password,
		);
		$data['transactionRequest'] = array(
			'transactionType' => 'authCaptureTransaction',
			'amount' => $this->param_amount,
			'payment' => array(
				'creditCard' => array(
					'cardNumber' => $this->param_cardNumber,
					'expirationDate' => $this->param_expirationMonth . '/' . $this->param_expirationYear,
					'cardCode' => $this->param_cardCode,
				),
			),
		);
		if (isset($this->param_invoiceNumber))
			$data['transactionRequest']['order']['invoiceNumber'] = $this->param_invoiceNumber;
		if (isset($this->param_email))
			$data['transactionRequest']['customer']['email'] = $this->param_email;
		foreach (self::$customerFields as $f) {
			$p = 'param_' . $f;
			$v = $this->$p;
			echo "$v\n";
			if (isset($v))
				$data['transactionRequest']['billTo'][$f] = $v;
		}
		return array('createTransactionRequest' => $data);
	}

	private function prepareRefund() {
		$data = array();
		$data['merchantAuthentication'] = array(
			'name' => $this->param_login,
			'transactionKey' => $this->param_password,
		);
		$data['transactionRequest'] = array(
			'transactionType' => 'refundTransaction',
			'amount' => $this->param_amount,
			'payment' => array(
				'creditCard' => array(
					'cardNumber' => $this->param_cardNumber,
					'expirationDate' => $this->param_expirationMonth . '/' . $this->param_expirationYear,
					'cardCode' => $this->param_cardCode,
				),
			),
			'refTransId' => $this->param_refTransId,
		);
		return array('createTransactionRequest' => $data);
	}

	protected function sendRequest($data) {
		$json = getJSONobj();
		$url = $this->param_sandbox ? self::URL_SANDBOX : self::URL_PRODUCTION;
		$ch = curl_init($url);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $json->encode($data));
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array('Content-Type: application/json'));
		$res = curl_exec($ch);
		$res = preg_replace("~^\xEF\xBB\xBF~", "", $res);
		$res = $json->decode($res);
		if (!is_array($res) || !isset($res['messages']['resultCode']))
			throw new CreditCardProcessorException('Invalid response from Authorize.net');
		if ($res['messages']['resultCode'] != 'Ok') {
			$messages = array();
			if (isset($res['transactionResponse']['errors']))
				foreach ($res['transactionResponse']['errors'] as $e)
					$messages[] = $e['errorText'];
			throw new CreditCardProcessorException($res['messages']['message'][0]['text'], $messages);
		}
		return $res;
	}

	public function decodeResponse($data) {
		return $data['transactionResponse']['transId'];
	}
}

