<?php

abstract class CreditCardProcessor {

	protected $params = array();
	protected $requestData = null;
	protected $responseData = null;

	public static function factory($name, $params = array()) {
		$cls_name = strtolower('CreditCardProcessor' . $name);
		$map = creditcard_class_map();
		if (!isset($map[$cls_name]))
			return null;
		return new $cls_name($params);
	}

	public static function getSupportedGateways() {
		return array(
			'authnet' => 'Authorize.net',
			'stripe' => 'Stripe',
		);
	}

	protected function __construct($params = array()) {
		$this->params = $params;
	}

	public function __get($name) {
		if (strpos($name, 'param_') === 0) {
			$n = substr($name, 6);
			return isset($this->params[$n]) ? $this->params[$n] : null;
		}
	}

	public function __set($name, $value) {
		if (strpos($name, 'param_') === 0) {
			$n = substr($name, 6);
			$this->params[$n] = $value;
		}
	}

	public function __isset($name) {
		if (strpos($name, 'param_') === 0) {
			$n = substr($name, 6);
			return isset($this->params[$n]);
		}
	}
	public function process() {
		$this->validate();
		$this->requestData = $this->prepare();
		$this->responseData = $this->sendRequest($this->requestData);
		return $this->responseData;
	}

	public function refundRequiresCard() {
		return true;
	}

	abstract protected function validate();
	abstract protected function sendRequest($data);
	abstract public function decodeResponse($data);
}

