<?php

class CreditCardProcessorStripe extends CreditCardProcessor {

	static private $customerFields = array(
		'address' => 'address_line1',
		'city' => 'address_city',
		'state' => 'address_state',
		'zip' => 'address_zip',
		'country' => 'address_country',
	);

	protected function __construct($params = array()) {
		parent::__construct($params);
	}

	protected function validate() {
		if (!isset($this->param_login))
			throw new CreditCardProcessorException('Missing Stripe API key');
		if (!$this->param_refTransId && $this->param_refund)
			throw new CreditCardProcessorException('Missing reference transaction ID');
	}

	protected function prepare() {
		$GLOBALS['log']->fatal($this);
		if ($this->param_refund)
			return $this->prepareRefund();
		return $this->prepareCharge();
	}

	private function prepareCharge() {
		$data = array(
			'customer_name' => $this->param_companyName != '' ? $this->param_companyName : $this->param_firstName.' '.$process->param_lastName,
			'customer_email' => $this->param_invoiceEmail,
			'amount' => (int)sprintf("%d", str_replace(",","", $this->param_amount) * 100),
			'currency' => $this->param_currency,
			'invoice_name' => $this->param_invoice_name,
			'invoice_description' => $this->param_invoice_description ?? null,
			'address' => $this->param_address,
			'city' => $this->param_city,
			'country' => $this->param_country,
			'state' => $this->param_state,
			'postal_code' => $this->param_postal_code,
			'customer_id' => $this->param_customer_id ?? null,
			'invoice_id' => $this->param_invoice_id ?? null,
			'account_id' => $this->param_account_id ?? null,
		);
		return $data;
	}

	private function prepareRefund() {
		$data = array(
			'charge' => $this->param_refTransId,
		);
		return $data;
	}

	public function refundRequiresCard() {
		return false;
	}


	protected function sendRequest($data) {
		$GLOBALS['log']->fatal($data);
		\Stripe\Stripe::setApiKey($this->param_login);
		$invoice = null;
		try {
			if ($this->param_refund) {
				$ret = \Stripe\Refund::create($data);
			} else {
				$customerId = $data['customer_id'];
				if (!$customerId) {
					$customers = \Stripe\Customer::all(['email' => $data['customer_email']]);
					if (isset($customers->data) && is_countable($customers->data) && count($customers->data) != 0) {
						$customer = $customers->data[0];
						$customerId = $customer->id;
					} else {
						$customer = \Stripe\Customer::create([
							'name' => $data['customer_name'],
							'email' => $data['customer_email'],
							'address' => [
								'line1' => $data['address'],
								'city' => $data['city'],
								'country' => $data['country'],
								'postal_code' => $data['postal_code'],
								'state' => $data['state'],
							],
							'description' => 'Created from 1CRM'
						]);
						$customerId = $customer->id;
						if (!empty($data['account_id'])) {
							$acc = \ListQuery::quick_fetch('Account', $data['account_id']);
							if ($acc) {
								$upd = \RowUpdate::for_result($acc);
								$upd->set('stripe_customer_id', $customerId);
								$upd->save();
							}
						}
					}
				}
				$new_invoice = \Stripe\Invoice::create([
					'customer' => $customerId,
					'collection_method' => 'send_invoice',
					'due_date' => strtotime("+1 week"),
					'description' => $data['invoice_description'] ?? ('1CRM Invoice ' . $data['invoice_name']),
					'metadata' => ['invoice_id' => $data['invoice_id']]
				]);
				\Stripe\InvoiceItem::create([
					'customer' => $customerId,
					'amount' => $data['amount'],
					'currency' => $data['currency'],
					'description' => $data['invoice_name'] ?? $data['invoice_description'] ?? '',
					'invoice' => $new_invoice->id,
				  ]);
				$invoice = \Stripe\Invoice::retrieve($new_invoice->id);
				$invoice->finalizeInvoice(['auto_advance' => true]);
				$invoice->sendInvoice();
				$webhook_url = AppConfig::setting('site.base_url') . '/json.php?action=stripe_webhook&u=' . sha1(AppConfig::unique_key());
				$create_webhook = true;
				$cursor = null;
				do {
					$req = ['limit' => 100];
					if ($cursor) $req['starting_after'] = $cursor;
					$cursor = null;
					$webhooks = \Stripe\WebhookEndpoint::all($req);
					if (isset($webhooks->data) && is_countable($webhooks->data) && count($webhooks->data) != 0) {
						foreach($webhooks->data as $webhook) {
							$cursor = $webhook['id'];
							if (in_array('invoice.payment_succeeded', $webhook->enabled_events)
								&& $webhook->url == $webhook_url) {
								$create_webhook = false;
								break 2;
							}
						}
					}
				} while ($cursor);
				if ($create_webhook) {
					\Stripe\WebhookEndpoint::create([
						'url' => $webhook_url,
						'enabled_events' => [
						  'invoice.payment_succeeded',
						],
					  ]);
				}
			}
				//throw new CreditCardProcessorException('Something went wrong... Please, try again');
		} catch (Exception $e) {
			throw new CreditCardProcessorException($e->getMessage());
		}
		return $invoice->jsonSerialize();
	}

	public function decodeResponse($data) {
		return $data['id'];
	}
}

