<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


class DLManager {
	static $batches = array();
	static $batch_count = 0;
	static $current_batch_id = 0;
	static $proxy_info;
	static $multi_handle;
	static $run_count;
	static $batch_open_counts;
	static $open_conns;
	static $debug = 0;
	static $auto_redirect = 1;
	
	static function init() {
		if(ini_get('open_basedir'))
			self::$auto_redirect = 0;
	}
	
	static function start_batch($id=null, $params=null) {
		self::end_batch();
		if(! isset($id))
			$id = 'batch~'.self::$batch_count;
		self::$batches[$id] = new DLBatch($id, $params);
		self::$batch_count ++;
		self::$current_batch_id = $id;
	}
	
	static function end_batch() {
		if(self::$current_batch_id) {
			self::$batches[self::$current_batch_id]->closed = true;
			return self::$current_batch_id;
		}
	}
	
	static function can_extend_batch() {
		if(self::$current_batch_id && ! self::$batches[self::$current_batch_id]->closed)
			return true;
		return false;
	}
	
	static function add_download($params) {
		if(! self::can_extend_batch())
			self::start_batch();
		return self::$batches[self::$current_batch_id]->addDownload($params);
	}
	
	static function single_download($params) {
		self::start_batch();
		$inst = self::$batches[self::$current_batch_id]->addDownload($params);
		$batch_id = self::end_batch();
		while(self::download_batch()) {
			@ob_flush();
		}
	}
	
	static function start_downloads() {
		if(self::$debug) pr2('starting downloads');
		$dls = array();
		foreach(self::$batches as $batch_id => &$batch) {
			$batch->closed = true;
			if(! $batch->started && ! $batch->finished) {
				$batch_dls =& $batch->getDownloads();
				foreach($batch_dls as $id => &$dl)
					$dls[$batch_id.'-'.$id] = &$dl;
				$batch->started = true;
			}
		}
		self::$run_count = 0;
		self::$open_conns = array();
		if(count($dls)) {
			self::$multi_handle = curl_multi_init();
			foreach($dls as &$dl) {
				if($dl->started || $dl->finished) continue;
				$dl->begin();
				$ch = self::_make_connection($dl);
				$code = curl_multi_add_handle(self::$multi_handle, $ch);
				if($code != CURLM_OK && $code != CURLM_CALL_MULTI_PERFORM) {
					$dl->finished = true;
					$dl->failed = true;
					$dl->failure_code = $code;
					if($dl->callback)
						call_user_func_array($dl->callback, array(&$dl));
					continue;
				}
				self::$open_conns[] =& $dl;
			}
		}
		else
			self::$multi_handle = null;
		return count(self::$open_conns);
	}
	
	
	static function _make_connection(&$dl) {
		$ch = curl_init();
		$dl->handle = $ch;
		$user_agent = $dl->user_agent;
		if(! isset($user_agent))
			$user_agent = ini_get('user_agent');
		$params = array(
			CURLOPT_URL => $dl->url,
			CURLOPT_HEADER => false,
			CURLOPT_RETURNTRANSFER => true,
			CURLOPT_CONNECTTIMEOUT => $dl->connect_timeout,
			CURLOPT_TIMEOUT => $dl->timeout,
			CURLOPT_HEADERFUNCTION => array('DLManager', 'receive_headers'),
			CURLOPT_USERAGENT => $user_agent,
		);
		if(self::$auto_redirect) {
			$params[CURLOPT_FOLLOWLOCATION] = true;
			$params[CURLOPT_MAXREDIRS] = isset($dl->max_redirects) ? $dl->max_redirects : 5;
		}
		if($dl->response_file_handle) {
			$params[CURLOPT_FILE] = $dl->response_file_handle;
			$params[CURLOPT_RETURNTRANSFER] = false;
		}
		if($dl->request_method === 'POST')
			$params[CURLOPT_POST] = true;
		else if($dl->request_method !== 'GET')
			$params[CURLOPT_CUSTOMREQUEST] = $dl->request_method;
		if($dl->send_cookie)
			$params[CURLOPT_COOKIE] = $dl->send_cookie;
		if(is_array($dl->send_headers) && $dl->send_headers) {
			$h = array();
			foreach($dl->send_headers as $k => $v) {
				if(is_int($k))
					$h[] = $v;
				else
					$h[] = "$k: $v";
			}
			$params[CURLOPT_HTTPHEADER] = $h;
		}
		if($dl->post_fields)
			$params[CURLOPT_POSTFIELDS] = $dl->post_fields;
		if($dl->follow_progress) {
			$params[CURLOPT_PROGRESSFUNCTION] = array('DLManager', 'receive_progress');
			$params[CURLOPT_NOPROGRESS] = false;
		}
		
		$proxy = AppConfig::get_proxy_info();
		if($proxy) {
			$params[CURLOPT_PROXY] = $proxy['host'];
			$params[CURLOPT_PROXYPORT] = $proxy['port'];
			if($proxy['auth'])
				$params[CURLOPT_PROXYUSERPWD] = $proxy['username'].":".$proxy['password'];
		}

		if(function_exists('curl_setopt_array'))
			curl_setopt_array($ch, $params);
		else
			foreach($params as $k => $v)
				curl_setopt($ch, $k, $v);
		return $ch;
	}
	
	
	static function run_downloads() {
		if(! self::$multi_handle) {
			if(! self::start_downloads()) {
				return self::cancel_downloads();
			}
		}
		
		if(self::$run_count) {
			$selStatus = curl_multi_select(self::$multi_handle, 0.1);
			if($selStatus < 0) {
				//return self::cancel_downloads();
				// in recent libcurl versions this apparently doesn't indicate failure
				// see php bug 61141
				usleep(50);
			}
		}
		self::$run_count ++;

		do {
			$execStatus = curl_multi_exec(self::$multi_handle, $activeCount);
			usleep(1);
		} while($execStatus == CURLM_CALL_MULTI_PERFORM);
		
		if(version_compare(phpversion(), '5.2.0', '>=')) {
			// curl_multi_info_read was not implemented until 5.2.0 (sigh)
			// this effectively means we have to wait for all downloads to complete
			// before retrieving the results
			while($done = curl_multi_info_read(self::$multi_handle, $more)) {
				foreach(self::$open_conns as $idx => &$dl) {
					if($dl->handle === $done['handle'] && $done['msg'] == CURLMSG_DONE) {
						$found = true;
						self::finish_download($dl, $done['result']);
						array_splice(self::$open_conns, $idx, 1);
						break;
					}
				}
				if(empty($found)) {} // log error?
				if(! $more) break;
			}
		}

		if($execStatus != CURLM_OK) {
			return self::cancel_downloads();
		}
		
		$doneBatches = self::process_finished_batches();
		if(! $activeCount && ! $doneBatches)
			return false;
		
		return compact('activeCount', 'doneBatches');
	}
	
	
	static function download_batch() {
		while($status = self::run_downloads()) {
			if($status['doneBatches'])
				return $status;
			usleep(1);
		}
		return false;
	}
	
	
	static function receive_headers($ch, $header) {
		if(self::$open_conns) {
			if(preg_match('~^([\w\-_]+)\:(.*)$~', $header, $m)) {
				foreach(self::$open_conns as &$dl) {
					if($dl->handle === $ch)
						$dl->receiveHeader(strtolower($m[1]), trim($m[2]));
				}
			}
		}
		return strlen($header);
	}
	
	
	static function receive_progress($ch, $expect_dl, $current_dl, $expect_ul, $current_ul=null) {
		if(version_compare(phpversion(), '5.5.0', '<')) {
			// no first parameter. only works for a batch with a single download
			$current_ul = $expect_ul;
			$expect_ul = $current_dl;
			$current_dl = $expect_dl;
			$expect_dl = $ch;
			$ch = null;
		}
		if(self::$open_conns) {
			foreach(self::$open_conns as &$dl) {
				if(! isset($ch) || $dl->handle === $ch)
					$dl->updateProgress($expect_dl, $current_dl);
			}
		}
	}
	
	
	static function finish_download(&$dl, $failure_code=null) {
		static $set_params = array(
			'status_code' => CURLINFO_HTTP_CODE,
			'response_url' => CURLINFO_EFFECTIVE_URL,
			'response_length' => CURLINFO_CONTENT_LENGTH_DOWNLOAD,
			'response_size' => CURLINFO_SIZE_DOWNLOAD,
			'response_type' => CURLINFO_CONTENT_TYPE,
			'response_connect_time' => CURLINFO_CONNECT_TIME,
			'response_time' => CURLINFO_TOTAL_TIME,
		);
		if($dl->started && ! $dl->finished) {
			$dl->finished = true;
			if($failure_code) {
				$dl->fail($failure_code);
				if(self::$debug) pr2("download failed ($failure_code): $dl->url");
			}
			else {
				if(! $dl->response_file_handle) {
					$dl->response_text = curl_multi_getcontent($dl->handle);
				}
				foreach($set_params as $k => $code) {
					$dl->$k = curl_getinfo($dl->handle, $code);
				}
				if($dl->status_code != 200 && $dl->status_code != 304) {
					if($dl->status_code == 404) $fail_msg = "Not Found";
					else $fail_msg = "HTTP Status: $dl->status_code";
					$dl->fail($dl->status_code, $fail_msg);
					if(self::$debug) pr2("bad status code ($dl->status_code): $dl->url");
				}
			}
			curl_multi_remove_handle(self::$multi_handle, $dl->handle);
			curl_close($dl->handle);
			if(self::$debug && ! $dl->failed)
				pr2("download complete: $dl->url");
			$dl->complete();
		}
	}
	
	
	static function cancel_downloads() {
		if(version_compare(phpversion(), '5.2.0', '>='))
			$fail_code = -1; // completion was not detected for some reason
		else
			$fail_code = 0;
		foreach(self::$open_conns as &$dl)
			self::finish_download($dl, $fail_code);
		self::$open_conns = array();
		if(self::$multi_handle)
			curl_multi_close(self::$multi_handle);
		self::$multi_handle = null;
		$done = self::process_finished_batches();
		if(! $done) return false;
		return array('activeCount' => 0, 'doneBatches' => $done);
	}
	
	
	static function process_finished_batches() {
		$done = array();
		foreach(self::$batches as $batch_id => &$batch) {
			if($batch->started && ! $batch->finished) {
				$batch_dls =& $batch->getDownloads();
				$doneb = true;
				foreach($batch_dls as &$dl) {
					if(! $dl->finished)
						$doneb = false;
				}
				if($doneb) {
					$batch->finished = true;
					if($batch->callback) {
						call_user_func_array($batch->callback, array(&$batch));
					}
					$done[] = $batch_id;
				}
			}
		}
		return $done;
	}
	
	
	static function handle_check_downloads($dl_ids) {
		$list_strings = return_app_list_strings_language(null);
		
		require_once('include/JSON.php');
		header('Content-type: application/json; charset=UTF-8');
		$dl_ids = array_filter(array_unique($dl_ids));
		$q = new ListQuery('active_downloads');
		try {
			$result = $q->queryRecordSet($dl_ids);
		} catch(IAHDatabaseError $e) {
			$GLOBALS['log']->error("database error while checking download status: {$e->getMessage()}");
		}
		$ret = array();
		$indexes = array_flip($dl_ids);
		foreach($dl_ids as $id) {
			$ret[] = array(
				'id' => $id,
				'url' => null,
				'status' => null,
				'progress' => -1,
				'fail_message' => null,
				'date_entered' => null,
				'date_modified' => null,
				'expect_size' => 0,
				'file_size' => 0,
				'progress_text' => "",
			);
		}
		if(! empty($result)) {
			foreach($result->getRowIndexes() as $idx) {
				$row = $result->getRow($idx);
				$id = $row['id'];
				if(isset($indexes[$id])) {
					$ridx = $indexes[$id];
					foreach($row as $k => $v)
						if(array_key_exists($k, $ret[$ridx]))
							$ret[$ridx][$k] = $v;
					$text = array_get_default($list_strings['download_status_dom'], $row['status'], $row['status']);
					$size = format_file_size($row['file_size']);
					if($row['status'] == 'in_progress') {
						if($row['expect_size'])
							$exp_size = format_file_size($row['expect_size']);
						else
							$exp_size = translate('LBL_SIZE_UNKNOWN', 'app');
						$text .= " ($size of $exp_size)";
					} else if($row['status'] == 'complete')
						$text .= " ($size)";
					$text = translate('LBL_STATUS', 'app') . translate('LBL_SEPARATOR', 'app') . $text;
					$ret[$ridx]['progress_text'] = $text;
				}
			}
		}
		$json = getJSONobj();
		echo $json->encode($ret, true);
	}
	
	static function register_handler() {
		register_cleanup_handler(array('DLManager', 'process_downloads'));
	}
	
	static function have_active() {
		foreach(self::$batches as &$b)
			if(! $b->finished) return true;
		return false;
	}
	
	static function process_downloads($flush = true) {
		if(! self::have_active())
			return;

		if ($flush) {
			end_flush_all_buffers();
			flush();
			// close session to prevent lock contention from multiple requests
			session_write_close();
			// keep running
			ignore_user_abort(true);
			set_time_limit(0);
		}
		
		while($info = DLManager::download_batch()) {
			if ($flush) @ob_flush();
		}
	}
}


class DLBatch {
	var $id;
	var $callback;
	var $dls = array();
	var $closed = false;
	var $started = false;
	var $finished = false;

	function __construct($id, $params=null) {
		if(is_array($params)) {
			foreach($params as $k => &$v)
				$this->$k = $v;
		}
		$this->id = $id;
	}
	
	function addDownload($params) {
		$inst = new DLInstance($this->id, count($this->dls), $params);
		$this->dls[$inst->id] =& $inst;
		return $inst;
	}
	
	function &getDownloads() {
		return $this->dls;
	}
}

// Extending StdClass for dynamic property creation
class DLInstance extends StdClass {
	var $batch_id;
	var $id;
	var $handle;
	var $started = false;
	var $finished = false;
	var $failed = false;
	var $failure_code;
	var $failure_msg;
	var $progress = -1;
	var $expect_size = 0;
	var $current_size = 0;
	var $db_track = false;
	var $user_id;
	var $status = 'init';
	var $source = '';
	
	// caller can set
	var $callback;
	var $url;
	var $timeout = 120;
	var $connect_timeout = 15;
	var $max_redirects;
	var $redirects = 0;
	var $send_headers;
	var $send_cookie;
	var $post_fields;
	var $follow_progress = false;
	var $user_agent;
	var $request_method = 'GET';
	var $response_file_handle;
	var $temp_directory;
	var $temp_filename;
	var $target_directory;
	var $target_filename;
	
	// set when response received
	var $status_code;
	var $response_headers = array();
	var $response_url;
	var $response_length;
	var $response_size;
	var $response_type;
	var $response_connect_time;
	var $response_time;
	var $response_text = '';
	var $response_filename;

	function __construct($bid, $bindex, $params) {
		if(is_string($params))
			$params = array('url' => $params);
		if(! is_array($params) || empty($params['url']))
			throw new IAHError("missing URL for download");
		foreach($params as $k => $v)
			$this->$k = $v;
		$this->batch_id = $bid;
		if(! $this->id)
			$this->id = ($this->db_track) ? create_guid() : 'dl~'.$bindex;
		if($this->db_track) {
			if(! isset($params['follow_progress']))
				$this->follow_progress = true;
			if(! $this->temp_filename) {
				if(! $this->temp_directory) $this->temp_directory = AppConfig::cache_dir() . 'incoming/';
				$this->temp_filename = tempnam($this->temp_directory, "");
			}
			$this->response_file_handle = fopen($this->temp_filename, 'wb');
		}
	}
	
	function getUrl() {
		return $this->url;
	}
	
	function getHeader($name) {
		$name = strtolower($name);
		return array_get_default($this->response_headers, $name);
	}
	
	function getHeaders() {
		return $this->response_headers;
	}
	
	function getResponse() {
		return $this->response_text;
	}
	
	function getStatus() {
		return $this->status;
	}
	
	function begin() {
		$this->started = true;
		if($this->db_track) {
			$upd = RowUpdate::blank_for_model('active_downloads');
			$upd->set(array(
				'id' => $this->id,
				'url' => $this->url,
				'source' => $this->source,
				'user_id' => isset($this->user_id) ? $this->user_id : AppConfig::current_user_id(),
				'status' => $this->status,
				'progress' => $this->progress,
				'expect_size' => $this->expect_size,
				'file_size' => $this->current_size,
				'temp_filename' => $this->temp_filename,
				'target_filename' => $this->getTargetFilename(),
			));
			if(! $upd->validate() || ! $upd->save()) {
				if(DLManager::$debug) pr2($upd->errors, "error saving active download");
				$this->db_track = false;
			}
		}
	}
	
	function receiveHeader($name, $val) {
		$this->response_headers[$name] = $val;
		// TODO: check here for filename given in a content-disposition header
		$this->status = 'headers';
	}
	
	function updateStatus($status, $force=false) {
		if($this->db_track && ($status != $this->status || $force)) {
			$row = ListQuery::quick_fetch('active_downloads', $this->id);
			if($row) {
				$upd = RowUpdate::for_result($row);
				$upd->set(array(
					'status' => $status,
					'progress' => $this->progress,
					'expect_size' => $this->expect_size,
					'file_size' => $this->current_size,
					'fail_message' => $this->failure_msg,
					'target_filename' => $this->getTargetFilename(),
				));
				$upd->save();
			}
		}
		$this->status = $status;
	}
	
	function updateProgress($expect_dl, $current_dl) {
		if($expect_dl) $progress = $current_dl * 100 / $expect_dl;
		else $progress = -1;
		if(DLManager::$debug && ($expect_dl !== $this->expect_size || $current_dl !== $this->current_size))
			pr2("expect: $expect_dl, current: $current_dl ($progress %)");
		$this->expect_size = $expect_dl;
		$this->current_size = $current_dl;
		$this->progress = $progress;
		$this->updateStatus('in_progress', true);
	}
	
	function getTargetFilename() {
		if($this->target_filename)
			return $this->target_filename;
		else if($this->response_filename)
			$fn = $this->response_filename;
		else {
			$url = $this->url;
			$p = strrpos($url, '/');
			if($p !== false)
				$fn = substr($url, $p+1);
		}
		if($fn == 'index.html')
			$fn .= '.txt';
		return $fn;
	}
	
	function getTargetPath() {
		if(! $this->target_directory || ! is_writeable($this->target_directory) || ! is_dir($this->target_directory))
			return false;
		$filename = $this->getTargetFilename();
		if(! $filename) $filename = create_guid();
		$p = strrpos($filename, '.');
		if($p !== false) {
			$ext = substr($filename, $p);
			$base = substr($filename, 0, $p);
		} else {
			$ext = '';
			$base = $filename;
		}
		$iter = 0;
		do {
			$path = $this->target_directory . '/' . $base;
			if($iter) $path .= '_' . $iter;
			$path .= $ext;
			$iter ++;
		}
		while(file_exists($path));
		return $path;
	}
	
	function moveFile() {
		if($this->response_file_handle) @fclose($this->response_file_handle);
		$targ_path = $this->getTargetPath();
		if($this->temp_filename && $targ_path) {
			if(DLManager::$debug) pr2("move to: $targ_path");
			rename($this->temp_filename, $targ_path);
		}
	}
	
	function fail($code, $msg=null) {
		$this->failure_code = $code;
		$this->failure_msg = $msg;
		$this->updateStatus('failed');
		if($this->response_file_handle) @fclose($this->response_file_handle);
		if($this->temp_filename) @unlink($this->temp_filename);
	}
	
	function complete() {
		$this->updateStatus($this->failed ? 'failed' : 'complete');
		$this->moveFile();
		if($this->callback)
			call_user_func_array($this->callback, array(&$this, ! $this->failed));
	}
}

DLManager::init();

?>
