<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/

class Dashlet {
   /**
     * Id of the Dashlet
     * @var guid
     */ 
    var $id; 
    var $dashboard_id;
    /**
     * Title of the Dashlet
     * @var string
     */
    var $title = 'Generic Dashlet';
    var $titleCaption = '';
    /**
     * true if the Dashlet has configuration options. 
     * @var bool
     */
    var $isConfigurable = false;
    /**
     * true if the Dashlet is refreshable (ie charts that provide their own refresh) 
     * @var bool
     */
    var $isRefreshable = true;
    /**
     * true if the Dashlet should link back to its primary module
     * @var bool
     */    
    var $showListViewLink = false;
    /**
     * true if the Dashlet supports simple filter controls
     * @var bool
     */    
    var $showFilterLink = false;
    /**
     * true if the Dashlet should show a Print icon link
     * @var bool
     */    
    var $showPrintLink = false;
    /**
     * true if the Dashlet contains javascript 
     * @var bool
     */
    var $hasScript = false;
    /**
     * Language strings, must be loaded at the Dashlet level w/ loadLanguage
     * @var array
     */
    var $dashletStrings;
    
    var $dashletIcon;
    
    var $estimContentHeight = 100;
    
    var $asyncDisplay = false;

    var $forceRefresh = false;

	var $requestedWidth = '';

    var $autoRefresh = false;

    var $listViewModule;
    var $seedBean;
    var $module;
    var $listViewAction;
    var $listViewACL;

    /**
     * If enabled - auto refresh dashlet time in minutes
     *
     * @var int
     */
    var $autoRefreshTime = 15;

	var $locked;

    function __construct($id, $options=null) {
        $this->id = $id;

	    if (! empty($options['dashboard_id']))
		    $this->dashboard_id = $options['dashboard_id'];
    }
    
    /**
     * Called when Dashlet is displayed
     * 
     * @param string $text text after the title
     * @param bool $is_sample
     * @return string title html
     */
    function getTitle($mode=null) {
        global $app_strings;
        if(isset($this->locked))
        	$locked = $this->locked;
        else
        	$locked = AppConfig::setting('layout.lock_homepage');

		$tools = [];
                
        if($this->isConfigurable) {
            global $mod_strings;
            $title = $this->getTitleText($mode) . ' : ' . $mod_strings['LBL_OPTIONS'];

            $tools[] = '<div class="uii uii-gear active-icon tool-icon" onclick="SUGAR.sugarHome.configureDashlet(\'' 
							. $this->id . '\', \'' . addslashes($title) . '\');" title="'.$app_strings['LBL_EDIT_DASHLET'].'"></div>';
        }

		if($this->showFilterLink && $mode != 'sample') {
			$onclick = 'SUGAR.sugarHome.showDashletFilter(\'' . $this->id . '\');';
        	$tools[] = '<div class="uii uii-search active-icon tool-icon" onclick="' .$onclick . '" '
        					. 'title="'.$app_strings['LBL_FILTER_DASHLET'].'"></div>';
        }
        if($this->isRefreshable && $mode != 'sample') {
			$onclick = 'SUGAR.sugarHome.refreshDashlet(\'' . $this->id . '\');';
            $tools[] = '<div class="uii uii-refresh active-icon tool-icon" onclick="' . $onclick . '" '
            				. 'title="'.$app_strings['LBL_REFRESH_DASHLET'].'"></div>';
        }
        if($this->showPrintLink && $mode != 'sample' && ($onclick = $this->getPrintAction())) {
            $tools[] = '<div class="uii uii-print active-icon tool-icon" id="dashlet_print_'.$this->id.'" '
            				. 'title="'.$app_strings['LBL_PRINT_DASHLET'].'" onclick="' . $onclick . '"></div>';
        }
        if($this->showListViewLink && $mode != 'sample') {
        	if(isset($this->listViewModule))
        		$listMod = $this->listViewModule;
        	else if(isset($this->seedBean))
        		$listMod = $this->seedBean->module_dir;
        	else if(isset($this->module))
        		$listMod = $this->module;
        	else
        		$listMod = '';
        	
        	$listAction = empty($this->listViewAction) ? 'list' : $this->listViewAction;        	
        	$listAcl = empty($this->listViewACL) ? 'list' : $this->listViewACL;
			
        	if($listMod && ACLController::checkAccess($listMod, $listAcl, true)) {
				$listAct = isset($this->listViewAction) ? $this->listViewAction : 'index';
				$link_params = ['module' => $listMod, 'action' => $listAct];
				$href = to_html('index.php?'.encode_query_string($link_params));
				$tools[] = "<a href=\"$href\" onclick=\"SUGAR.util.handleLink(event)\" "
							. 'class="uii uii-extern active-icon tool-icon" '
							. 'title="'.translate('LBL_GO_TO_MODULE', 'app').'"></a>';
			}
        }
        if(! $locked) {
			$tools[] = '<div class="uii uii-cancel active-icon tool-icon" onclick="SUGAR.sugarHome.deleteDashlet(\'' . $this->id . '\');" '
						. 'title="'.$app_strings['LBL_DELETE_DASHLET'].'"></div>';
		}

        $icon = '<div class="dashlet-icon no-flex">' . get_image($this->getIcon(), '') . '</div>';
        
        $title = $icon . '<h4>' . $this->getTitleText($mode) . '</h4>';
        $caption = $this->getTitleCaption($mode);
        if($caption) $title .= "<small class=\"dashlet-caption\">$caption</small>";
        $extra = $this->getTitleExtra($mode);
        if($extra) $title .= $extra;
        $title .= implode('', $tools);
        $cls = 'tools-row dashlet-title';
        if(! $locked)
        	$cls .= ' moveable';
        $str = "<div class=\"$cls\">$title</div>";

        return $str;
    }
    
    function getTitleText($mode=null) {
    	return to_html($this->title);
    }
    
    function getTitleCaption($mode=null) {
    	return to_html($this->titleCaption);
    }
    
    function getTitleExtra($mode=null) {
    }
    
    function getTitleSubheader($mode=null) {
    }

    function getFooterText($mode=null) {
    }

    function getFooterContent($mode=null) {
    	$text = $this->getFooterText($mode);
    	if($text) {
    		return '<div class="dashlet-footer-text text-right">' . $text . '</div>';
    	}
    	
    }
    
    function getIcon() {
        if(isset($this->dashletIcon)) {
        	$icon = $this->dashletIcon;
        }
        else if(isset($this->seedBean) && ! empty($this->seedBean->module_dir)) {
        	$icon = $this->seedBean->module_dir;
        }
        return $icon;
    }
    
    function getPrintAction() {
    	return '';
    }
        
    function display() {
		$ret = '<div id="dashlet-outer-'.$this->id.'" class="card-outer panel-outer panel-border panel-system dashlet-outer">';
        $ret .= $this->displayHeader();
		$ret .= $this->displaySubheader();
		$ret .= $this->displayBody();
		$ret .= $this->displayFooter();
		$ret .= '</div>';
		return $ret;
    }
        
    /**
     * Called when Dashlet is displayed, override this
     * 
     * @param string $text text after the title
     * @return string title html
     */
    function displayBody() {
    }
        
    function displayHeader($mode=null) {
		$ret = '<div id="dashlet-header-'.$this->id.'" class="card-header panel-header dashlet-header">';
        $ret .= $this->getTitle($mode);
        $ret .= $this->displayExtraHeader($mode);
        $ret .= '</div>';
        return $ret;
    }
    
    function displayExtraHeader($mode=null) {
    }
    
    function displaySubheader($mode=null) {
        $subhead = $this->getTitleSubheader($mode);
        if($subhead)
        	return "<div id=\"dashlet-subheader-{$this->id}\" class=\"card-header panel-subheader dashlet-subheader\">$subhead</div>";
    }

    function displayFooter($mode=null) {
        $footer = $this->getFooterContent($mode);
        if($footer)
        	return "<div id=\"dashlet-footer-{$this->id}\" class=\"card-footer panel-footer dashlet-footer\">$footer</div>";
    }
    
    function displaySample() {
		$ret = '<div id="dashlet-outer-'.$this->id.'" class="card-outer panel-outer panel-border panel-system dashlet-outer">';
        $ret .= $this->displayHeader('sample');
		$ret .= $this->sampleContent();
		$ret .= '</div>';
		return $ret;
    }
    
    function sampleContent() {
    	return '<div style="height: '.$this->estimContentHeight.'px" class="card-body panel-body dashlet-body sample-dashlet">'
    		. $this->sampleContentBody() . '</div>';
    }
    
    function sampleContentBody() {
    	return '&nbsp;';
    }
    
    function displayPending() {
		$ret = '<div id="dashlet-outer-'.$this->id.'" class="card-outer panel-outer panel-border panel-system dashlet-outer">';
        $ret .= $this->displayHeader('pending');
		$ret .= $this->pendingContent();
		$ret .= '</div>';
		return $ret;
    }
    
    function displayPendingScript($refresh = false) {
    	$scr = '';
    	if($this->hasScript)
			$scr = $this->displayScript();
		$refresh = $refresh ? 'true' : 'false';
    	$scr .= '<script type="text/javascript">SUGAR.sugarHome.pendingDashlet("'.$this->id.'", ' . $refresh . ');</script>';
    	return $scr;
    }

    function displayScript() {
        return '';
    }

    function pendingContent() {
    	global $theme;
    	return '<div class="card-body panel-body dashlet-body" id="dashlet-body-' . $this->id . '" style="padding: 0px; text-align: center"></div>';
    }
    
    /**
     * Called when Dashlets configuration options are called
     * 
     */
    function displayOptions() {
    }
    
    /**
     * override if you need to do pre-processing before display is called
     * 
     */
    function process() {
    }    
    
    function save() {
    }
        
    /**
     * Override this if your dashlet is configurable (this is called when the the configureDashlet form is shown)
     * Filters the array for only the parameters it needs to save
     * 
     * @param array $req the array to pull options from
     * 
     * @return array options array
     */
    function saveOptions($req) {
    }
    
    /**
     * Sets the language strings
     *
     * @param string $dashletClassname
     * @param string $module
     */
    function loadLanguage($dashletClassname, $module = 'Home') {
        $this->dashletStrings = return_dashlet_language('', $module, $dashletClassname);
    }
    
    function translate($var, $mod=null) {
    	if(isset($this->dashletStrings)) {
    		$ret = array_get_default($this->dashletStrings, $var);
    		if(isset($ret)) return $ret;
    	}
    	if(! isset($mod) && isset($this->module))
    		$mod = $this->module;
    	return translate($var, $mod);
    }

    /**
     * Generic way to store an options array into UserPreferences
     * 
     * @param array $optionsArray the array to save
     */
    function storeOptions($optionsArray) {
        global $current_user;
        if($this->id) {
			$current_user->setPreference('dashlet_options~'.$this->id, $optionsArray, 0, 'home');
			UserPreference::savePreferencesToDB();
		}
    }

    /**
     * Generic way to retrieve options array from UserPreferences
     * 
     * @return array options array stored in UserPreferences
     */
    function loadOptions() {
        global $current_user;
        $options = $current_user->getPreference('dashlet_options~'.$this->id, 'home'); // load user's dashlets config
        if(! isset($options)) $options = array();
		return $options;
    }

	// longreach - start added
	function canDisplay() {
		return true;
	}

	function noDisplay() {
		return '';
	}


	function setWidth($w) {
		$this->requestedWidth = $w;
	}
	// longreach - end added
}
?>
