<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FilterForm.php');

class DashletFilter extends FilterForm {

    /**
     * @var DashletGeneric
     */
	var $dashlet;

    /**
     * @param DashletGeneric $dashlet
     * @param ListQuery $list_query
     */
    function __construct($dashlet, $list_query) {
        $this->dashlet = $dashlet;
        parent::__construct($list_query, $dashlet->id.'-FilterForm');
		$this->expand_select_filters = true;
	}

	function loadLayoutIndex() {
        $dashlet_name = get_class($this->dashlet);
		$this->layouts = array();
        $detail = AppConfig::setting("views.detail.{$dashlet_name}.search.Standard");
        if($detail)
            $this->layouts[''] = array('type' => $detail['type'], 'dashlet_name' => $dashlet_name, 'title' => 'LBL_BASIC_FILTER');
	}
	
	function loadLayoutSpec($name) {
		if(! isset($this->layouts[$name]))
			return false;
		$spec = $this->layouts[$name];
		if(empty($spec['type']))
			return false;
		if(! $name) $name = 'Standard';
		$cfg = AppConfig::setting("views.layout.{$spec['dashlet_name']}.{$spec['type']}.$name");
		return $cfg;
	}
	
	function getDisplayFilterOptions($basic_search=false, $hidden=false, $overrides=true) {
		$opts = parent::getDisplayFilterOptions($basic_search, $hidden, $overrides);
		foreach($opts as &$opt) {
			$opt['required'] = false;
			unset($opt['basic']);
		}
		if($basic_search) {
			$ret = array();
			if(isset($opts['filter_text'])) {
				$ret['filter_text'] = $opts['filter_text'];
				// necessary to support multiple search inputs on one page
				$ret['filter_text']['override_id'] = 'filter_text-'.$this->dashlet->id;
			}
			return $ret;
		}
        /*if (isset($sorted_opts['any_user_id'])) {
            //$sorted_opts['any_user_id'] =
        }*/
        return $opts;
	}
	
	function loadFilter($input, $source=null, $input_prefix='', $defaults=null) {
		$ret = parent::loadFilter($input, $source);
		
		$sess_key = 'dashlet_filter_text_'.$this->dashlet->id;
		$req_list_id = array_get_default($_REQUEST, 'list_id');
		if($req_list_id == $this->dashlet->id) {
			if(! empty($_REQUEST['filter_clear']))
				unset($_SESSION[$sess_key]);
			else if($ret === 'request') {
				if(isset($this->filter['filter_text']))
					$_SESSION[$sess_key] = $this->filter['filter_text'];
				else if(isset($_REQUEST['filter_text']))
					unset($_SESSION[$sess_key]);
			}
		}
		if(isset($_SESSION[$sess_key]))
			$this->filter['filter_text'] = $_SESSION[$sess_key];
		
		return $ret;
	}

	function formatDisplayOptions($list_id, DashletViewManager $mgr, DashletGeneric $dashlet) {
        require_once('include/layout/forms/FormGenerator.php');
        require_once('include/database/RowResult.php');
        global $mod_strings, $app_strings;

        $basic_search = false;
		$sorted_opts = $this->getDisplayFilterOptions($basic_search);		
		unset($sorted_opts['filter_text']);

		foreach($sorted_opts as &$opt)
			$opt['editable'] = true;
		unset($opt);

        $le_fields = array(
            'columns' => array(
                'type' => 'varchar',
            ),
            'sort_order' => array(
                'type' => 'varchar',
            ),
            'add_column' => array(
                'type' => 'varchar',
			),
			'dashletTitle' => array(
				'name' => 'dashletTitle',
				'field' => 'dashletTitle',
				'source' => array ('type' => 'db'),
				'type' => 'varchar',
				'editable' => true,
				'updatable' => true,
				'label' => $mod_strings['LBL_DASHLET_CONFIGURE_TITLE'],
				'width' => 30,
				'filter' => false,
			)
		);
		
		$optionsList = AppConfig::setting('layout.dashlet_display_row_options');
        $displayRowOptions = array();
        foreach($optionsList as $v)
			$displayRowOptions[$v] = $v;

		$le_fields['displayRows'] = array(
			'name' => 'displayRows',
			'field' => 'displayRows',
			'source' => array ('type' => 'db'),
			'type' => 'enum',
			'editable' => true,
			'updatable' => true,
			'label' => $mod_strings['LBL_DASHLET_CONFIGURE_DISPLAY_ROWS'],
			'options' => $displayRowOptions,
			'required' => true,
			'label_key' => 'label',
			'filter' => false,
			'no_search' => true,
		);

		if ($dashlet->autoRefresh) {
            $le_fields['autoRefreshTime'] = array(
                'name' => 'autoRefreshTime',
                'field' => 'autoRefreshTime',
                'type' => 'enum',
                'editable' => true,
                'updatable' => true,
                'label' => translate('LBL_DASHLET_CONFIGURE_AUTOREFRESH', 'Home'),
                'options' => 'dashlet_auto_refresh_dom',
                'options_add_blank' => false,
                'filter' => false,
				'no_search' => true,
            );
        }

        //Init Layout Editor
        $editor = $mgr->getLayoutEditor();
        $editor->loadLayout();

		$result = new RowResult;
        $model = $this->base_model;
        if(is_object($model)) $model = $model->getModelName();
        $result->base_model = $model;
        $result->module_dirs[] = $this->list_query->getModuleDir();

		$row = array_merge($this->filter, array(
			'columns' => $editor->encodeColumns($editor->columns_ext),
			'sort_order' => $editor->encodeSortOrder($editor->sort_order),
		));
		$row['dashletTitle'] = $dashlet->title;
		$row['displayRows'] = $dashlet->displayRows;
		$row['autoRefreshTime'] = $dashlet->autoRefreshTime;
		$result->row = $row; // FIXME will need to unformat values like dates

		$fmt = new FieldFormatter('html', 'editview');
		$formatted_row = array_merge($this->filter, $fmt->formatRow($le_fields, $result->row));
        $result->formatted = $formatted_row;

        $result->fields = array_merge($sorted_opts, $le_fields);

        $layout = $this->getLayout($list_id, $sorted_opts);
        $model = new ModelDef($this->base_model);
        $model->addDisplayFieldDefinitions($sorted_opts);
        $model->addDisplayFieldDefinitions($le_fields);
        $this->form_name = 'ConfigForm';
        $this->form_id = 'configure_'.$list_id;
		$gen = FormGenerator::html_form($model, $layout, $this->form_name, $this->form_id);
		$gen->setFormObject($this);

        $cmds[] = "SUGAR.ui.setFormInputProperty('ConfigForm', 'add_column', 'model', '$editor->model_name');";
        $cmds[] = "SUGAR.ui.setFormInputProperty('ConfigForm', 'add_sorted', 'model', '$editor->model_name');";
        $gen->getLayout()->addFormInitHook($cmds);

		$gen->renderForm($result);
		$out = $gen->getResult();
		$gen->exportIncludes();

        return $out;
	}

    /**
     * Get Dashlet option layout
     *
     * @param string $list_id
     * @param array $filter_options
     * @return array
     */
    function getLayout($list_id, $filter_options) {
        $hidden = array(
            'list_id' => $list_id, 'id' => $list_id, 'query' => true, 'filter_clear' => '', 'filter_layout' => $this->layout_name,
            'dashlet_filters' => true, 'module' => 'Home', 'action' => 'ConfigureDashlet', 'configure' => 'true', 'resetDashlet' => ''
        );
        if(! empty($_REQUEST['edit']))
        	$hidden['edit'] = $_REQUEST['edit'];
        if(! empty($this->dashlet->dashboard_id)) {
	        $hidden['layout'] = $this->dashlet->dashboard_id;
        } elseif (! empty($_REQUEST['layout'])) {
	        $hidden['layout'] = $_REQUEST['layout'];
        }

        $hidden = array_merge($this->add_hidden_fields, $hidden);

		$layout = array(
			'form_hooks' => array('onsubmit' => "var c=SUGAR.ui.getFormInput('configure_".$list_id."', 'columns');if(typeof(c)!='undefined'){c.beforeSubmitForm();} c=SUGAR.ui.getFormInput('configure_".$list_id."', 'sort_order');if(typeof(c)!='undefined'){c.beforeSubmitForm();}return SUGAR.sugarHome.postForm('configure_".$list_id."', SUGAR.sugarHome.uncoverPage);"),
			'form_buttons' => [
				'cancelDashlet' => [
            		'vname' => 'LBL_CANCEL_BUTTON_LABEL',
            		'icon' => 'input-icon icon-cancel',
            		'perform' => 'SUGAR.sugarHome.hideConfigure();',
            		'type' => 'button',
            		'align' => 'right',
            	],
            	'resetDashlet' => [
            		'vname' => 'LBL_RESET_BUTTON_LABEL',
            		'icon' => 'input-icon icon-delete',
            		'perform' => "this.form.resetDashlet.value='1'; this.form.onsubmit();",
            		'type' => 'button',
            		'align' => 'right',
            	],
            	'saveDashlet' => [
            		'vname' => 'LBL_SAVE_BUTTON_LABEL',
            		'icon' => 'input-icon icon-accept',
            		'type' => 'submit',
            		'perform' => '',
            		'align' => 'right',
            	],
            ],
			'form_hidden_fields' => $hidden,
			'indicate_required' => false,
			'type' => 'configform',
			'base_module' => 'Home',
			'buttons_position' => 'bottom',
			'class' => 'dashlet-filter',
			'editable' => true,
			'current_tab' => 'general',
			'tabs' => array(
				array(
					'name' =>  'general',
					'label' => 'LBL_LAYOUT_GENERAL',
					'icon' => 'icon-sources',
				),
				array(
					'name' =>  'columns',
					'label' => 'LBL_COLUMN_LAYOUT',
					'icon' => 'icon-layout',
				),
				array(
					'name' =>  'filters',
					'label' => 'LBL_LAYOUT_FILTERS',
					'icon' => 'icon-filter',
				),
				array(
					'name' =>  'sorting',
					'label' => 'LBL_LAYOUT_SORTING',
					'icon' => 'icon-sortlist',
				),
			),
		);

		$layout['sections'] = array(
			array(
				'id' => 'layout-general',
				'tab' => 'general',
                'label_position' => 'left',
				'columns' => 1,
                'widths' => array('60%', '40%'),
				'elements' => array(
					'dashletTitle',
					'displayRows',
					'autoRefreshTime',
				),
			),
			
            array(
                'id' => 'layout-columns',
                //'vname' => 'LBL_DASHLET_LAYOUT',
                'label_position' => 'top',
				'columns' => 2,
				'tab' => 'columns',
                'widths' => array('60%', '40%'),
                'elements' => array(
                    array(
                        'name' => 'columns',
                        'colspan' => '2',
                        //'vname' => 'LBL_COLUMN_LAYOUT',
                        'widget' => 'DynamicListInput',
                        'class' => 'listLayoutConfig',
                        'renderer_class' => 'SUGAR.ui.ListLayoutRender',
                        'show_action' => 'true',
                        'cols' => array(
                            array(
                                'name' => 'label',
                                'label' => 'LBL_NAME',
                                'width' => '40%',
                                'render_method' => 'renderName'
                            ),
                            array(
                                'name' => 'type',
                                'label' => 'LBL_TYPE',
                                'width' => '40%',
                                'render_method' => 'renderType'
                            ),
                            array(
                                'name' => 'width',
                                'label' => 'LBL_WIDTH',
                                'width' => '15%',
                                'render_method' => 'renderWidth'
                            ),
                            array(
                                'name' => 'format',
                                'label' => 'LBL_FORMAT',
                                'width' => '15%',
                                'render_method' => 'renderFormat'
                            ),
                            array(
                                'name' => 'hidden',
                                'label' => 'LBL_HIDE',
                                'width' => '5%',
                                'format' => 'check'
                            )
                        )
                    ),
                    array(
                        'name' => 'add_column',
                        'vname' => 'LBL_ADD_COLUMNS',
                        'colspan' => '2',
                        'widget' => 'FieldSelectInput',
                        'list_name' => 'columns'
                    )
                )
            ),
			array(
				'id' => 'layout-sorting',
				'tab' => 'sorting',
				'label_position' =>  'top',
				'columns' => 2,
				'widths' => array('60%', '40%'),
				'elements' => array(
					array(
						'name' => 'sort_order',
						//'vname' => 'LBL_SORT_ORDER',
						'widget' => 'SortOrderInput',
						'list_name' => 'columns',
						'colspan' => 2,
					),
					array(
						'name' => 'add_sorted',
						'vname' => 'LBL_ADD_SORTED',
						'widget' => 'FieldSelectInput',
						'list_name' =>  'sort_order',
						'sel_mode' => 'order',
					),
				),
			),
			array(
				'id' => 'filter',
				'tab' => 'filters',
                //'vname' => 'LBL_DASHLET_FILTERS',
				'label_position' => 'top',
				'columns' => 2,
				'widths' => array('50%'),
				'label_widths' => array('0%'),
				'elements' => array_keys($filter_options),
			)
		);

        return $layout;
    }
}

