<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/
require_once('include/Dashlets/Dashlet.php');
require_once('include/Dashlets/DashletViewManager.php');
require_once('include/Dashlets/DashletFilter.php');

class DashletGeneric extends Dashlet {

    /**
     * Bean file used in this Dashlet
     * @var bean
     */    
    var $seedBean;

    /**
     * @var DashletViewManager
     */
    var $mgr;
    var $inited;
    var $savedFilter;
    var $showFilterLink = true;
    var $columns;
    var $filters;
    var $listViewModule;

    /**
     * Number of Rows to display 
     * @var int
     */    
    var $displayRows = 5;

	function __construct($id, $options = null) {
        parent::__construct($id, $options);
        $this->isConfigurable = true;
        $this->showListViewLink = true;
        $this->columns = array();
        $this->filters = array();

        if(! empty($options['title']))
            $this->title = $options['title'];
        if(! empty($options['displayRows']))
            $this->displayRows = $options['displayRows'];
        if(isset($options['autoRefreshTime']))
            $this->autoRefreshTime = $options['autoRefreshTime'];
        if(! empty($options['filter'])) {
            $filter = unserialize($options['filter']);
            if(is_array($filter))
				$this->savedFilter = $filter;
		}
        if (! empty($options['columns']))
            $this->columns = $options['columns'];
        if (! empty($options['filters']))
            $this->filters = $options['filters'];
		if (! empty($options['dashboard_id']))
			$this->dashboard_id = $options['dashboard_id'];

        $this->estimContentHeight = (int)$this->displayRows * 20;
    }
    
    function getSavedFilter() {
    	return $this->savedFilter;
    }
    
    function initManager() {
    	if(! $this->inited) {
			$this->mgr = new DashletViewManager($this);
			$this->mgr->loadRequest();
			$this->inited = $this->mgr->initModuleView();
		}
		return $this->inited;
    }


    /**
     * Sets up the display options template
     * 
     * @return string HTML that shows options 
     */
    function processDisplayOptions() {
    	$this->initManager();
    	
        if($this->inited) {
			return $this->mgr->renderDisplayOptions();
		}
    }
        
    /**
     * Displays the options for this Dashlet
     * 
     * @return string HTML that shows options 
     */
    function displayOptions() {
        return parent::displayOptions() . $this->processDisplayOptions();
    }

    /**
     * Filter the $_REQUEST and only save only the needed options
     * @param array $req the array to pull options from
     *
     * @return array options array
     */
    function saveOptions($req) {
        $options = array();

        foreach($req as $name => $value) {
            if(!is_array($value)) $req[$name] = trim($value);
        }

        if(!empty($req['dashletTitle'])) {
            $options['title'] = $req['dashletTitle'];
        }

        $options['displayRows'] = (int)(empty($req['displayRows']) ? 5 : $req['displayRows']);
        $options['autoRefreshTime'] = (int)(isset($req['autoRefreshTime']) ? $req['autoRefreshTime']: 15);

        if($this->initManager()) {
        	$filter =& $this->mgr->getFilter();
        	if($filter) {
				if(! empty($req['list_id']))
					$filter->loadRequestFilter($req['list_id'], 'listview');
				$filter_val = $filter->getArray();
				unset($filter_val['filter_text']);
				$options['filter'] = serialize($filter_val);
			}

            $editor = $this->mgr->getLayoutEditor();
            $columns = $editor->getColumns($req);
            $options['columns'] = $columns;
		}

        return $options;
    }

    /**
     * Does all dashlet processing, here's your chance to modify the rows being displayed!
     */
	function process($lvsParams = array()) {
		$this->initManager();
	}
    
   /**
     * Displays the Dashlet
     * 
     * @return string HTML that displays Dashlet 
     */
	function displayBody() {
        if($this->inited) {
            ob_start();
            if (!$this->canDisplay()) {
                //$this->mgr->renderTitle();
                echo $this->noDisplay();
            } else {
                $this->mgr->render();
            }
            $content = ob_get_clean();
            return $content;
        } else {
        	echo '<p class="body">' . translate('LBL_CANNOT_RENDER_DASHLET', 'Home') . translate('LBL_SEPARATOR', 'app') . $this->title . '</p>';
            //ACLController::displayNoAccess();
        }
    }

	function canDisplay() {
		if (!ACLController::checkAccess($this->seedBean->module_dir, 'list', true)) {
			return false;
		} else {
			return parent::canDisplay();
		}
	}

	function noDisplay() {
		return '<p style="text-align: center;" class="error dashlet-body">' . translate('MSG_DASHLET_NO_ACCESS') . '</p>';
	}
}
?>
