<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/ListView/ListLayoutEditor.php');

class DashletLayoutEditor extends ListLayoutEditor {

    /**
     * @var string
     */
    var $dashlet_name;

    /**
     * @var DashletGeneric
     */
    var $dashlet;

    function initFromManager(ListViewManager &$mgr, $perform) {
        return $this->initFromDashletManager($mgr, $perform);
    }

    private function initFromDashletManager(DashletViewManager &$mgr, $perform) {
        $this->module = $mgr->module;
        $this->action = $mgr->action;
        $this->model_name = $mgr->bean_name;
        $this->layout_name = $mgr->layout_name;
        $this->layout = $this->layout_name;
        $this->filter_name = $mgr->getFilterName();
        $this->override_filters = $mgr->getOverrideFilters();
        $this->view_type = $mgr->view_type;
        $this->list_id = $mgr->list_id;
        $this->list_query = new ListQuery($this->model_name);
        $this->filter_form = new FilterForm($this->list_query, $mgr->context);
        $this->dashlet_name = $mgr->dashlet_name;
        $this->dashlet = $mgr->dashlet;
        $this->form_name = 'ConfigForm';
    }

	function loadStandardLayout($name) {
		if($name == 'Browse') $name = 'Standard';

        $custom = $this->dashlet->columns;
        $custom_columns = array();
        $custom_sort = array();
        if (isset($custom['columns_spec']))
            $custom_columns = unserialize($custom['columns_spec']);
        if (isset($custom['sort_order']))
            $custom_sort = unserialize($custom['sort_order']);

        $std_layout = AppConfig::setting("views.layout.{$this->dashlet_name}.list.{$name}");
        if (is_array($custom_columns) && sizeof($custom_columns) > 0) {
            $layout = array('columns' => $custom_columns);
        } else {
            $layout = $std_layout;
		}

		if (!is_array($custom_sort) || empty($custom_sort)) {
			if (!empty($std_layout['default_order_by']))
			$custom_sort = ListQuery::parse_order_by($std_layout['default_order_by']);
		}

		if($layout) {
			$this->loadColumns($layout['columns']);
			$this->loadSortOrder($custom_sort);
			return true;
		}
	}

    function getColumns($input=null) {
        if(! isset($input))
            $input =& $_REQUEST;

        $arrs = array();
        $vals = array();

        if(isset($input['columns'])) {
            $val = array();
            if(is_string($input['columns'])) {
                $json = getJSONobj();
                $val = $json->decode($input['columns']);
                if(! is_array($val))
                    $val = array();
            } else if(is_array($input['columns'])) {
                $val = $input['columns'];
            }

            $arrs['columns_spec'] = $val;
        }
		
		if(isset($input['sort_order'])) {
            $val = array();
            if(is_string($input['sort_order'])) {
                $json = getJSONobj();
                $val = $json->decode($input['sort_order']);
                if(! is_array($val))
                    $val = array();
            } else if(is_array($input['sort_order'])) {
                $val = $input['sort_order'];
            }

            $arrs['sort_order'] = $val;
        }

        if(isset($arrs['columns_spec']))
            self::clean_column_input($arrs['columns_spec']);
        if(isset($arrs['sort_order']))
            self::clean_sort_order_input($arrs['sort_order']);
        foreach($arrs as $k => $v)
            $vals[$k] = serialize($v);

        return $vals;
    }
}
?>
