<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/ListView/ListViewManager.php');

class DashletViewManager extends ListViewManager {

    /**
     * @var DashletGeneric
     */
    var $dashlet;
    
    /**
     * @var string
     */
    var $dashlet_name;

    var $show_edit_view_links = false;

	var $bean_name;
    /**
     * Object for dashlet rendering
     *
     * @param  DashletGeneric $dashlet
     * @param string $context
     */
	function __construct($dashlet, $context='dashlet') {
        $this->dashlet = $dashlet;
		parent::__construct($context);
        $this->list_id = $this->dashlet->id;
        $this->module = $this->dashlet->seedBean->module_dir;
        $this->bean_name = $dashlet->seedBean->object_name;
        //$this->model_name = $this->bean_name;
	}

    function initModuleView($view=null, $layout_name=null, $layout_module=null, $report_id=null, $data_id=null, $model_name=null) {
        if(! $this->bean_name)
            return false;

		if(AppConfig::setting("display.list.{$this->bean_name}.disabled"))
			return false;

        // force creation of standard records (like base currency), but only if caching is enabled
        AppConfig::db_check_standard_records($this->bean_name, true);

        if(! isset($view))
            $view = 'Standard';

		$dashlet_name = get_class($this->dashlet);
        $detail = AppConfig::setting("views.detail.$dashlet_name.list.$view");

        if ($detail) {
            $this->layout_name = $detail['name'];
            $this->dashlet_name = $detail['dashlet_name'];
            return true;
        } else {
            $this->layout_name = null;
            return false;
        }
    }

    /**
     * @return DashletLayoutEditor
     */
    function &getLayoutEditor() {
        if(! $this->layout_editor) {
            require_once('include/Dashlets/DashletLayoutEditor.php');

            $this->layout_editor = new DashletLayoutEditor();
            $this->layout_editor->initFromManager($this, $this->layout_perform);
            $this->layout_editor->loadRequest();
        }
        return $this->layout_editor;
    }

    /**
     * Get List Formatter object
     *
     * @return ListFormatter
     */
	function &getFormatter() {
        if(! $this->formatter) {
			require_once('include/ListView/ListFormatter.php');
            $fmt = new ListFormatter($this->bean_name, null, $this->list_id, $this->context, array('auto_init_filter' => false));
            $filter = new DashletFilter($this->dashlet, $fmt->list_query);
            $fmt->setFilterForm($filter);
            $fmt->action = 'DisplayDashlet';
            if(! $this->list_id)
                $this->list_id = $fmt->list_id;
            $hidden = $this->getHiddenFields();
            $fmt->getMassUpdate()->addHiddenFields($hidden);
            $fmt->getFilterForm()->addHiddenFields($hidden);
            $fmt->addButtons($this->getButtons());
            $fmt->show_checks = false;
            $fmt->show_title = false;
            $fmt->show_mass_update = false;
            $fmt->can_shrink = false;
            $fmt->table_class = 'listView';
            $fmt->show_edit_view_links = $this->show_edit_view_links;
            $fmt->layout_module = $this->dashlet_name;
            $this->formatter =& $fmt;
        }

        return $this->formatter;        
	}
	
	function getHiddenFields() {
		$fs = parent::getHiddenFields();
		if($this->dashlet)
			$fs['dashboard_id'] = $this->dashlet->dashboard_id;
		return $fs;
	}
	
    /**
     * Get Dashlet Filter object
     *
     * @return DashletFilter
     */
    function &getFilter() {
    	$fmt =& $this->getFormatter();
    	return $fmt->getFilterForm();
    }
    
    
    function initFormatter(ListFormatter &$fmt) {
        if (! $this->loadFormatterLayout($fmt))
            return false;

		$fmt->loadRequestFilter(); // checks for filter_text
		$saved = $this->dashlet->getSavedFilter();
		if($saved)
			$fmt->loadFilter($saved, 'saved');
		if(! $fmt->show_favorites && ! empty($saved['filter_favorites']))
			$fmt->addFavoritesToQuery();

		return true;
    }

    /**
     * Custom method for Dashlets
     * (checking custom layout from dashlet's preference)
     *
     * @throws IAHError
     * @param ListFormatter $fmt
     * @return bool
     */
    function loadFormatterLayout(ListFormatter &$fmt) {
        $custom = $this->dashlet->columns;
        $custom_columns = array();
        $custom_sort = array();
        if (isset($custom['columns_spec']))
            $custom_columns = unserialize($custom['columns_spec']);
        if (isset($custom['sort_order']))
            $custom_sort = unserialize($custom['sort_order']);

        $layout = AppConfig::setting("views.layout.{$this->dashlet_name}.list.{$this->layout_name}");
		
		if (is_array($custom_columns) && sizeof($custom_columns) > 0) {
            $columns = $custom_columns;
        } else {
            if (!isset($layout['columns']))
                return false;
            if (!isset($fmt->list_query))
                throw new IAHError("cannot load list layout without query");
            $columns = $layout['columns'];
            $order = array_get_default($layout, 'default_order_by');
        }

		if (is_array($custom_sort) && sizeof($custom_sort) > 0) {
            $order = $custom_sort;
        } else {
            $order = array_get_default($layout, 'default_order_by');
        }

        $fmt->columns = array();
        if (!$fmt->loadColumns($columns, $fmt->columns))
			return false;
        if(! empty($order))
			$fmt->loadSortOrder($order);

        return true;
    }
    
	function getCountOptions() {
		$opts = AppConfig::setting('layout.dashlet_display_row_options');
		$max = $this->getMaxDisplayRows();
		$ret = array();
		foreach($opts as $o)
			if($o <= $max) $ret[] = $o;
		return $ret;
	}

	function render() {
        $fmt =& $this->getFormatter();
        if(! $this->initFormatter($fmt))
        	return false;

        if($this->async_list) {
            $fmt->performMassUpdate();
        }

        if(! $this->async_list) {
            //echo '<div id="dashlet-outer-'.$this->list_id.'" class="card-outer dashlet-outer">';
        }
        
        $sess_ord_key = 'dashlet_' . $this->list_id . '_order_by';
        $def_order = array_get_default($_SESSION, $sess_ord_key);
        $order_by = array_get_default($_REQUEST, 'list_order_by', $def_order);
        if($order_by)
        	$_SESSION[$sess_ord_key] = $order_by;
        
        $offset = array_get_default($_REQUEST, 'list_offset', 0);
        $limit = array_get_default($_REQUEST, 'list_limit', $this->dashlet->displayRows);
        $this->list_result = $fmt->getQueryResult($offset, $limit, $order_by);

        $fmt->formatResult($this->list_result);
        $fmt->outputHtml($this->list_result);

        if(! $this->async_list) {
            //echo '</div>';
		}

		global $pageInstance;
		$json = getJSONobj();
		$params = [
			'type' => 'dashlet',
			'count_options' => $this->getCountOptions(),
		];
		if($this->list_result) {
			$rc = $this->list_result->result_count;
			if(! is_numeric($rc)) $rc = null;
			$tc = $this->list_result->total_count;
			if(! is_numeric($tc)) $tc = null;
			$params['result_count'] = $rc;
			$params['total_count'] = $tc;
		}
		$jparams = $json->encode($params);

		$pageInstance->add_js_literal("sListView.init('{$this->list_id}', $jparams);", null, LOAD_PRIORITY_FOOT);
		$fmt->exportIncludes();

		if ($this->dashlet->autoRefresh && $this->dashlet->autoRefreshTime > 0) {
			$pageInstance->add_js_literal("SUGAR.sugarHome.initDashletAutoRefresh('{$this->dashlet->id}', {$this->dashlet->autoRefreshTime});", null, LOAD_PRIORITY_FOOT);
		}
    }
	
	
	function renderDisplayOptions() {
		$fmt =& $this->getFormatter();
		if(! $this->initFormatter($fmt))
			return '';
		return $this->getFilter()->formatDisplayOptions($this->list_id, $this, $this->dashlet);
	}
}
?>
