<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormGenerator.php');
require_once('include/config/ModelDef.php');

class DetailManager {
	var $perform;
	var $layout_name, $layout_path;

    /**
     * @var FormLayout
     */
    var $layout;
    var $layout_tab;
	var $form_name = 'DetailForm';
	var $form_id;
	var $form_def;
	var $form_sources;
	var $form_gen;
	var $module;
	var $model;
	var $bean_name;
	var $action;

    /**
     * @var RowResult
     */
	var $record;
    var $record_id;
	var $duplicate_of_id;
	var $in_popup = false;
	var $track_view = false;
	var $form_title, $page_title;
	var $form_title_prefix, $form_title_prefix_link;
	var $debug;
	var $editLayout;
	var $format = 'html';
	var $context;
    var $default_buttons_pos;


	function __construct() {
	}
	
	function loadRequest() {
		global $current_user;
		if(empty($this->module))
			$this->setModule($GLOBALS['module']);
		if(empty($this->action))
			$this->action = $GLOBALS['action'];
		if(empty($this->perform)) {
			$perform = array_get_default($_REQUEST, 'record_perform');
			if ($this->providesInlineEditing()) {
				$this->editLayout = $perform == 'edit_layout_edit' || $perform == 'edit_layout_view' || $perform == 'edit_layout_record';
				if (!is_admin($current_user)) {
					$this->editLayout = false;
				}
				if ($this->editLayout)
					$perform = ($perform == 'edit_layout_edit' ? 'edit' : 'view');
			}
			if(! $perform) {
				if($this->action == 'EditView')
					$perform = 'edit';
				else if($this->action == 'Duplicate')
					$perform = 'duplicate';
				else
					$perform = 'view';
			}
			$this->perform = $perform;
		} else {
            $perform = $this->perform;
        }

		$lname = array_get_default($_REQUEST, 'layout');
		if(! $lname) $lname = 'Standard';
		$this->layout_name = $lname;
		if (AppConfig::is_mobile())
			$this->layout_name = 'Mobile';
		$ltab = array_get_default($_REQUEST, 'layout_tab');
		if($ltab) $this->layout_tab = $ltab;

		if(empty($this->record_id)) {
			$record_id = array_get_default($_REQUEST, 'record');
			$no_record_perform = array(
				'edit',
				'validate',
				'save',
				'save_layout_edit',
				'save_layout_view',
				'save_layout_record',
				'default_layout_edit',
			);
			if(empty($record_id) && !in_array($perform, $no_record_perform) && !$this->editLayout)
				throw new IAHError("record id not provided");
			$this->record_id = $record_id;
		}
        if(! empty($_REQUEST['in_popup']))
            $this->in_popup = $_REQUEST['in_popup'];
        if(! empty($_REQUEST['duplicate_of_id']) && $this->perform != 'view')
        	$this->duplicate_of_id = $_REQUEST['duplicate_of_id'];
		$this->format = array_get_default($_REQUEST, 'format', 'html');
	}
		
	function setModule($module) {
		$this->module = $module;
        $this->bean_name = AppConfig::module_primary_bean($module);
		if(! $this->bean_name) {
			throw new IAHInternalError('primary bean not defined');
		}
		$this->model = new ModelDef($this->bean_name);
	}
	
	function setRecordId($id) {
		$this->record_id = $id;
	}
	
	function setModel($model) {
		$this->model = new ModelDef($model); // may throw error for unknown model
		$this->bean_name = $this->model->getModelName();
		$this->module = $this->model->getModuleDir();
	}

	function loadLayoutTemplate() {
		if(empty($this->form_def)) {
			$perf = $this->perform;
			$act = $this->performingEdit() ? 'edit' : 'view';
			if ($this->providesInlineEditing()) {
				if ($perf == 'save_layout_edit' || $perf == 'edit_layout_edit') $act = 'edit';
				if ($perf == 'save_layout_view' || $perf == 'edit_layout_view') $act = 'view';
				if ($perf == 'save_layout_record' || $perf == 'edit_layout_record') $act = 'view';
			}
			
			$detail = $layout = null;
			
			// combined view/edit form layouts
			if(AppConfig::setting('site.features.combined_detail') && empty($_REQUEST['std'])) {
				$detail = AppConfig::setting("views.detail.{$this->module}.record.{$this->layout_name}");
				if($detail) {
					if(empty($detail['type']))
						$detail['type'] = 'record';
					$act = 'record';
				}
			}
			
			if(! $detail)
				$detail = AppConfig::setting("views.detail.{$this->module}.{$act}.{$this->layout_name}");
			if (! $detail && $this->layout_name == 'Mobile' && $this->editLayout) {
				$detail = array(
					'type' => '$act',
					'title' => 'LBL_MODULE_TITLE',
					'name' => $this->layout_name,
				);
			}
            if (! $detail && $this->layout_name != 'Standard') {
				if(AppConfig::setting('site.features.combined_detail') && empty($_REQUEST['std']) && $this->layout_name != 'QuickCreate') {
					$detail = AppConfig::setting("views.detail.{$this->module}.record.Standard");
					if($detail) {
						if(empty($detail['type']))
							$detail['type'] = 'record';
						$act = 'record';
					}
				}
				if(! $detail)
					$detail = AppConfig::setting("views.detail.{$this->module}.{$act}.Standard");
            }
            
            if($detail && ! $layout)
				$layout = AppConfig::setting("views.layout.{$this->module}.{$act}.{$detail['name']}");
			if (! $layout && $this->layout_name == 'Mobile' && $this->editLayout) {
				$layout = array(
					'sections' => array(
						array(
							'id' => 'sec1',
							'elements' => array(
								array(
									'name' =>  'name',
									'label' =>  '',
								),
							),
							'columns' => 1,
						),
					)
				);
			}

			if(AppConfig::is_mobile()) {
				$layout_columns = AppConfig::get_mobile_columns();
				foreach ($layout['sections'] as &$sec) {
					if(is_array($sec)) $sec['columns'] = $layout_columns;
				}
			}

            if(empty($detail) || empty($layout))
                throw new IAHLayoutLoadError("template not defined: {$this->module}.{$act}.{$this->layout_name}");

            $layout_data = $detail + $layout;
			$this->form_sources = AppConfig::setting("views.source.{$this->module}.{$act}.{$detail['name']}");
		}

		$this->form_def = $layout_data;

		if(!$this->form_def) {
			throw new IAHLayoutLoadError('form template not defined or missing "layout"');
		}
	}
	
	function isStandardLayout() {
		if(! $this->form_sources) return true;
		foreach($this->form_sources as $src) {
			if(strpos($src['name'], 'new_views') !== false)
				return false;
		}
		return true;
	}


	function getStandardButtons() {
        return array();
	}
	function getStandardTabs() {
        return array();
	}

	function getFormGenerator() {
		if($this->form_gen) return $this->form_gen;
	}
	
	function getLayout() {
		if($this->form_gen) return $this->form_gen->getLayout();
	}
	
	function getFormName() {
		if($this->form_gen) return $this->form_gen->getFormName();
	}
	
	function getHiddenFields() {
		$hidden = array('module' => $this->module, 'action' => $this->action, 'record' => $this->record_id);
		if($this->in_popup)
			$hidden['in_popup'] = $this->in_popup;
		if($this->layout_name)
			$hidden['layout'] = $this->layout_name;
		if($this->perform == 'duplicate') {
			$hidden['record'] = '';
			$hidden['duplicate_of_id'] = $this->record_id;
		} else if(! empty($this->duplicate_of_id)) {
			$hidden['duplicate_of_id'] = $this->duplicate_of_id;
		}
		if($this->perform == 'edit' || $this->perform == 'validate' || $this->perform == 'duplicate')
			$hidden['record_inited'] = '1';
		if(! empty($_REQUEST['std']))
			$hidden['std'] = 1;
		if(! empty($_REQUEST['debug']))
			$hidden['debug'] = $_REQUEST['debug'];
		return $hidden;
	}
	
	function performingEdit($perform=null) {
		if(! isset($perform)) $perform = $this->perform;
		if($perform == 'edit' || $perform == 'save' || $perform == 'duplicate' || $perform == 'validate' || $perform == 'delete')
			return true;
		return false;
	}
	
	function initLayout() {
		$layout = new FormLayout(array("in_popup" => $this->in_popup, 'editing_layout' => $this->editLayout));
		$layout->addFormHooks(array('onsubmit' => 'return SUGAR.ui.sendForm(this);'), false);

		$hidden = $this->getHiddenFields();
		if(! $layout->getEditingLayout()) {
			if($this->performingEdit()) {
				$layout->setDefaultEditable(true);
				foreach(array(
						'return_url', 'return_module', 'return_action', 'return_record', 'return_layout', 'return_panel',
						'target_name', 'target_id', 'target_layout', 'primary_name', 'role_field_name'
					) as $f)
						if(isset($_REQUEST[$f])) $hidden[$f] = $_REQUEST[$f];
				if(! empty($hidden['primary_name'])) {
					$primary_value = array_get_default($_REQUEST, $hidden['primary_name'], '');
				}
				if(! empty($hidden['role_field_name'])) {
					$role_value = array_get_default($_REQUEST, $hidden['role_field_name'], '');
				}
				$add_f = AppConfig::setting("display.edit.{$this->bean_name}.add_conversion_fields");
				if($add_f)
					$layout->addRecordHiddenFields($add_f);
				$add_scripts = AppConfig::setting("display.edit.{$this->bean_name}.scripts");
				$add_styles = AppConfig::setting("display.edit.{$this->bean_name}.styles");
				$add_hooks = AppConfig::setting("display.edit.{$this->bean_name}.form_hooks");
			}
			else if($this->perform == 'view') {
				$add_scripts = AppConfig::setting("display.view.{$this->bean_name}.scripts");
				$add_styles = AppConfig::setting("display.view.{$this->bean_name}.styles");
				$add_hooks = AppConfig::setting("display.view.{$this->bean_name}.form_hooks");
			}
		}
				
		$buttons = $this->getStandardButtons();
		if($buttons)
			$layout->addFormButtons($buttons);
		$tabs = $this->getStandardTabs();
		if($tabs)
			$layout->addTabs($tabs);

		$layout->init($this->form_def, $this->model);
		if(isset($hidden['primary_name']) && isset($primary_value)) {
			$primary_found = false;
			if($this->model) {
				$def = $this->model->getFieldDefinition($hidden['primary_name']);
				if($def) {
					if($layout->findElementPath($hidden['primary_name']))
						$primary_found = $hidden['primary_name'];
					else if(isset($def['for_ref']) && $layout->findElementPath($def['for_ref']))
						$primary_found = $def['for_ref'];
				}
			}
			if(! $primary_found)
				$hidden[$hidden['primary_name']] = $primary_value;
		}
		if(isset($hidden['role_field_name']) && isset($role_value)) {
			$found = false;
			if($this->model) {
				$def = $this->model->getFieldDefinition($hidden['role_field_name']);
				if($def) {
					if($layout->findElementPath($hidden['role_field_name']))
						$found = $hidden['role_field_name'];
					else if(isset($def['for_ref']) && $layout->findElementPath($def['for_ref']))
						$found = $def['for_ref'];
				}
			}
			if(! $found)
				$hidden[$hidden['role_field_name']] = $role_value;
		}

		if($hidden)
			$layout->addFormHiddenFields($hidden, false);
		if(! empty($add_scripts))
			$layout->addScripts($add_scripts);
		if(! empty($add_styles))
			$layout->addStyles($add_styles);
		if(! empty($add_hooks))
			$layout->addFormHooks($add_hooks);
		if($layout->getType() == 'record') {
			if(($tabs = $layout->getTabs()) && ! $layout->getCurrentTab()) {
				if($this->layout_tab) {
					foreach($tabs as $t) {
						if(isset($t['name']) && $t['name'] == $this->layout_tab) {
							$found_tab = $this->layout_tab;
							break;
						}
					}
				}
				if(! $layout->getDefaultTab())
					$layout->setDefaultTab($tabs[0]['name']);
				if(empty($found_tab))
					$found_tab = $layout->getDefaultTab();
				$this->layout_tab = $found_tab;
				$layout->setCurrentTab($found_tab);
			}
		}

		$this->layout = $layout;

		$this->form_gen = FormGenerator::new_form($this->format, $this->model, $layout, $this->form_name, $this->form_id);
		if($this->debug)
			$this->form_gen->debug = $this->debug;
		if($this->perform == 'view' || $this->perform == 'load_subpanels' || $this->perform == 'subpanel_render')
			$this->form_gen->requireAclAction('view');
		else if($this->perform == 'delete')
			$this->form_gen->requireAclAction('delete');
		else if($this->perform == 'duplicate') {
			$this->form_gen->requireAclAction('view');
			$this->form_gen->requireAclAction('create');
		}
		else
			$this->form_gen->requireAclAction('edit');
		$this->form_gen->requireDisplayName();
		if($this->default_buttons_pos)
			$this->form_gen->default_buttons_pos = $this->default_buttons_pos;
	}

	function loadRecord($id=null) {
		global $themeInstance;
		
		if(isset($id))
			$this->record_id = $id;
		$this->beforeLoadRecord();

		if($this->record_id)
			$result = $this->form_gen->getRowResult($this->record_id, true, ($this->perform == 'duplicate'));
		else
			$result = $this->form_gen->getBlankResult();
		if($this->debug > 1)
			pr2($result->fields, 'fields', false, true);
		
		if($result->failed) {
			throw new IAHRecordLoadError($result->error_msg);
		}
		if($this->perform == 'duplicate') {
			$this->duplicate_of_id = $this->record_id;
		}
		$this->record = $result;
		
		if ($this->record->acl) {
			$layout_acl = array_get_default($this->form_def, 'acl');
			if (!empty($layout_acl)) {
				if (empty($this->record->acl[$layout_acl]))
					ACLController::displayNoAccess();
			}
		}
		
		if($this->debug)
			pr2($this->record->row, 'record', false, true);
		
		if($this->record->new_record) {
			$this->form_title = translate('LBL_NEW_RECORD_TITLE', 'app');
		} else {
			$dn = $this->form_gen->getDisplayNameField();
			$title = null;
			if($dn) {
				$title = $this->record->getField($dn);
				if(! strlen($title))
					$title = translate('LBL_MISSING_VALUE', 'app');
				else
					$title = str_limit_length($title, 80);
				$this->form_title = $title;
				$this->page_title = $title;
			}
			if($this->track_view) {
				$owner = $this->record->getField('assigned_user_id');
				require_once('data/Tracker.php');
				Tracker::track_view(AppConfig::current_user_id(), $this->module, $this->bean_name, $this->record_id, $title, $owner);
			}
		}
		if(isset($themeInstance->classic_title))
			$this->form_title_prefix = translate('LBL_MODULE_TITLE', $this->module);
	}
	
	function beforeLoadRecord() {
		if($this->perform == 'delete' || $this->perform == 'view') {
			$gen_perform = $this->perform;
		}
		else if($this->perform == 'subpanel_render') {
			$gen_perform = 'view';
		}
		else {
			$gen_perform = 'edit';
		}
		if($this->form_gen) {
			$this->form_gen->setPerform($gen_perform);
			if( ($checks = $this->getAclChecks()) )
				$this->form_gen->addAclChecks($checks);
		}
	}
	
	function getAclChecks() {
		$checks = array();
		if($this->form_def && ($acl = array_get_default($this->form_def, 'acl'))) {
			$checks[] = $acl;
		}
		return $checks;
	}

	function performUpdate() {
        return false;
	}

	function getFormTitle($html=true) {
        $title = $this->form_title;
        if($html)
        	$title = to_html($title);
        if($this->form_title_prefix) {
        	$pfx = $this->form_title_prefix;
        	if($html) {
        		$pfx = to_html($pfx);
				if($this->form_title_prefix_link)
					$pfx = '<a href="' . to_html($this->form_title_prefix_link) . '" class="titleLink">' . $pfx . '</a>';
			}
			if($title)
				$title = $pfx . translate('LBL_SEPARATOR', 'app') . $title;
			else
				$title = $pfx;
        }
        return $title;
	}

	function getPageTitle($html=false) {
        $title = $this->page_title;
        if($html)
        	$title = to_html($title);
        return $title;
	}
	
	function getRecord() {
		return $this->record;
	}
	
	function getParentRecord() {
	}

	function standardInit($id=null) {
		$this->loadLayoutTemplate();
		$this->initLayout();
		$this->loadRecord($id);
	}

	function renderLayout() {
		if(! $this->form_gen)
			throw new IAHError("form generator is not defined");
		if(! $this->form_gen->layout)
			throw new IAHError("form generator layout is not defined");
		if(! $this->record)
			throw new IAHError("record not loaded");
		$gen =& $this->form_gen;
		
		$this->initLayoutTitle();
        $this->runDisplayHook();
		$gen->renderForm($this->record);
		$result = $gen->getResult();

		$gen->exportIncludes();

        return $result;
	}

	function providesInlineEditing()
	{
		return false;
	}
	
	function runDisplayHook() {
	}
	
	function initLayoutTitle() {
	}
	
	function getPerform() {
		return $this->perform;
	}

}
