<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/DetailView/DetailManager.php');

class MiniDetailManager extends DetailManager {
	var $form_name = 'MiniDetailForm';
	var $action = 'DetailView';
	var $context;
	
	function __construct() {
		parent::__construct();
	}
	
	function loadRequest() {
		parent::loadRequest();
		$this->context = array_get_default($_REQUEST, 'context');
	}
	
	function loadLayoutTemplate() {
		$layout = $this->model->additionalDetailsLayout();
		if(! $layout) {
			$this->form_def = array(
				'autogenerated' => true,
				'type' => 'view',
				'sections' => array(
					array(
						'id' => 'mini_details',
						'columns' => 1,
            			'label_position' => 'top',
						'elements' => array('date_entered', 'date_modified'),
					),
				),
			);
			if($this->context != 'refinput') {
				$this->form_def['sections'][] = array(
					'id' => 'activities',
					'widget' => 'ActivityButtonsWidget',
				);
			}
		}
	
		if(empty($this->form_def)) {
			$detail = AppConfig::setting("views.detail.{$this->module}.additional.{$layout}");
			$layout = AppConfig::setting("views.layout.{$this->module}.additional.{$layout}");
	
			if(empty($detail))
				throw new IAHLayoutLoadError('template not defined');
			if(! $layout) $layout = array();
			if(isset($layout['sections']) && is_countable($layout['sections']) && count($layout['sections']) == 1 && $this->context != 'refinput') {
				$layout['sections'][] = array(
					'id' => 'activities',
					'widget' => 'ActivityButtonsWidget',
				);
			}
	
			$layout_data = $detail + $layout;
			$this->form_def = $layout_data;
		}

		if(! isset($this->form_def)) {
			throw new IAHLayoutLoadError('form template not defined or missing "layout"');
		}
		if(! isset($this->form_def['show_empty_rows']))
			$this->form_def['show_empty_rows'] = false;
	}
	
	function getCaption() {
		global $app_strings, $image_path;

		$title = $this->getFormTitle();
		$hook = AppConfig::setting("display.view.{$this->model->getModelName()}.additional_details_title_hook");
		if ($hook) {
			call_hook_function($hook, null, array($this->record, &$title));
		}
		$mod = $this->module;
		if($mod == 'ReportData') $mod = 'Reports';
		$icon = get_module_header_icon($mod, '');
		$caption = $icon . '&nbsp;' . $title;

		return $caption;
	}
	
	function getHiddenFields() {
		$fs = parent::getHiddenFields();
		unset($fs['layout']); // don't persist mini detail layout in form
		return $fs;
	}
	
	function getStandardButtons() {
		$perf = $this->perform;
		if($this->context != 'listview' && $this->context != 'dashlet' && $this->context != 'subpanel' && $this->context != 'unified_search' && $this->context != 'view')
			return array();

		$bs = array(
			'view' => array(
				'icon' => 'icon-view',
				'vname' => 'LBL_VIEW_BUTTON_LABEL',
				'async' => false,
			),
			'edit' => array(
				'icon' => 'icon-edit',
				'vname' => 'LBL_EDIT_BUTTON_LABEL',
				'async' => false,
				'acl' => 'edit',
			),
		);
		$p = $perf == 'duplicate' ? 'edit' : $perf;
		$hook = AppConfig::setting("views.layout.{$this->module}.{$p}.{$this->layout_name}.buttons_hook");
		if ($hook) {
			call_hook_function($hook, null, array($this, &$bs));
		}
		$hook2 = array_get_default($this->form_def, 'buttons_hook');
		if ($hook2) {
			call_hook_function($hook2, null, array($this, &$bs));
		}
		return $bs;
	}
	
	function renderLayout() {
		if(!$this->form_gen || ! $this->form_gen->layout || ! $this->record) {
			global $app_strings;
			$result = "<div class=\"panel-pad-body no-details\">".$app_strings['LBL_NO_ADDITIONAL_DETAILS']."</div>";
		} else {
			$this->form_gen->getFormObject()->setFormAction('index.php');
			$this->form_gen->getLayout()->setClass('miniDetailView');
			$result = parent::renderLayout();
		}
		
		return $result;
	}
	
	function fetchJsonData($model, $record, $layout=null, $context=null) {
		$this->setModel($model);
		$this->perform = 'view';
		$this->layout_name = $layout ? $layout : 'Standard';
		$this->context = $context;
		$this->record_id = $record;
		
		try {
			$this->standardInit();
		} catch(IAHRecordLoadError $e) {
		} catch(IAHLayoutLoadError $e) {
		}
		
		global $pageInstance;
		$result = $this->renderLayout();
		$body = $pageInstance->format_includes('body')
				. $result
				. $pageInstance->format_includes('foot');
		$details = array(
			'body' => $body,
			'width' => 350,
			'caption' => $this->getCaption(),
		);
		return $details;
	}
}
