<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/DetailView/DetailManager.php');

class StandardDetailManager extends DetailManager {

	var $subpanels;

	/**
	 * @var SubPanelManager
	 */
	var $subpanelMgr;

	var $track_view = true;
	var $parent_record;
	var $default_buttons_pos = 'topbottom';
	var $custom_redirect;
	var $row_update;

	function __construct() {
		parent::__construct();
	}

	function loadLayoutTemplate() {
		parent::loadLayoutTemplate();

		if(isset($this->form_def['subpanels']) && ! $this->performingEdit()) {
			$this->subpanels = $this->form_def['subpanels'];
		}
		if(isset($this->form_def['ext_subpanels'])) {
			$this->extendSubpanels($this->form_def['ext_subpanels']);
		}
	}

	function getStandardButtons() {
		// FIXME these definitions should move into a global config file
		$perf = $this->perform;
		global $current_user, $themeInstance;
		if($this->perform == 'view' || $this->perform == 'delete')  {
			$edit_layout = 'Standard';
			$els = $this->getViewLayouts();
			if(isset($els[$this->layout_name]['edit_layout'])) {
				$edit_layout = $els[$this->layout_name]['edit_layout'];
			}
			$buttons = array(
				'edit' => array(
					'vname' => 'LBL_EDIT_BUTTON_LABEL',
					'accesskey' => 'LBL_EDIT_BUTTON_KEY',
					'order' => 10,
					'icon' => 'icon-edit',
					// 'group' => 'main'
					'acl' => 'edit',
					'params' => array(
						'record_perform' => 'edit',
						'return_module' => $this->module,
						'return_action' => $this->action,
						'return_record' => $this->record_id,
						'return_layout' => $this->layout_name,
						'layout' => $edit_layout,
					),
				),
				'duplicate' => array(
					'vname' => 'LBL_DUPLICATE_BUTTON_LABEL',
					'accesskey' => 'LBL_DUPLICATE_BUTTON_KEY',
					'order' => 20,
					'icon' => 'icon-duplicate',
					'acl' => 'create',
					'params' => array(
						'record_perform' => 'duplicate',
						'layout' => $edit_layout,
					),
					'type' => 'button',
				),
				'delete' => array(
					'vname' => 'LBL_DELETE_BUTTON_LABEL',
					'accesskey' => 'LBL_DELETE_BUTTON_KEY',
					'order' => 30,
					'confirm' => 'NTC_DELETE_CONFIRMATION',
					'icon' => 'icon-delete',
					'acl' => 'delete',
					'params' => array(
						'record_perform' => 'delete',
					),
					'type' => 'button',
				),
				'print' => array(
					'vname' => 'LBL_PDF_PDF',
					'order' => 40,
					'icon' => 'icon-print',
					'type' => 'group',
				),
				'pdf_print' => array(
					'vname' => 'LBL_PDF_PDF',
					'order' => 40,
					'icon' => 'icon-print',
					'params' => array(
						'format' => 'pdf',
					),
					'conn_params' => array(
						'url' => 'async.php',
					),
					'target' => '_blank',
					'type' => 'button',
					'group' => 'print',
					'auto_ungroup' => true,
					'widget' => 'PdfTemplateButton',
				),
				'convert' => array(
					'vname' => 'LBL_CONVERT_BUTTON_LABEL',
					'order' => 50,
					'type' => 'group',
					'icon' => 'icon-convert',
				),
			);
			if (AppConfig::setting("model.detail.{$this->model->name}.personal_data")) {
				$buttons['view_personal'] = [
					'vname' => 'LBL_VIEW_PERSONAL_BUTTON_LABEL',
					'order' => 50,
					'icon' => 'theme-icon module-DataPrivacy',
					'url' =>  "?module={$this->module}&action=ViewPersonal&record={RECORD}&{RETURN}",
					'acl' => 'manage_personal',
					'group' => 'personal_data',
				];
				$buttons['erase_personal'] = [
					'vname' => 'LBL_ERASE_PERSONAL_BUTTON_LABEL',
					'order' => 50,
					'icon' => 'theme-icon ticon-EraseDataPrivacy',
					'url' =>  "?module={$this->module}&action=ErasePersonal&record={RECORD}&{RETURN}",
					'acl' => 'manage_personal',
					'group' => 'personal_data',
				];
				$buttons['personal_data'] = [
					'type' => 'group',
					'vname' => 'LBL_PERSONAL_DATA',
					'order' => 50,
					'icon' => 'theme-icon module-DataPrivacy',
				];
			}

			$disabled_buttons = AppConfig::setting("views.layout.{$this->module}.view.{$this->layout_name}.disabled_buttons", array());
			foreach ($disabled_buttons as $b) {
				unset($buttons[$b]);
			}

			if (AppConfig::is_mobile()) {
				unset($buttons['pdf_print']);
			}

			if (!AppConfig::is_mobile()) {
				if ($this->model->audit_enabled) {
					$popup_request_data = array(
						'call_back_function' => 'set_return',
						'form_name' => 'EditView',
						'field_to_name_array' => array(),
					);
					$json = getJSONobj();
					$encoded_popup_request_data = $json->encode($popup_request_data);

					$buttons['change_log'] = array(
						'icon' => 'icon-changelog',
						'vname' => 'LNK_VIEW_CHANGE_LOG',
						'type' => 'button',
						'order' => 70,
						'align' => 'right',
						'perform' => "open_popup_window({module: '{$this->module}', action: 'Popup', query_string: 'popup_type=audit&record={$this->record_id}', request_data: {$encoded_popup_request_data}, inline: true});",
						'params' => array(
							'format' => 'html',
						),
						'group' => 'tools',
					);
				}
				$buttons['arrange_subpanels'] = array(
					'vname' => 'LBL_ARRANGE_SUBPANELS_LABEL',
					'order' => 10001,
					'icon' => 'icon-editlist',
					'async' => true,
					'perform' => "SUGAR.ui.arrangeSubpanels('" . $this->module . "', '" . $this->record_id . "')",
					'group' => 'tools',
					'mode' => 'any',
				);
				if (is_admin()) {
					$buttons['arrange_tabs'] = array(
						'vname' => 'LBL_ARRANGE_TABS',
						'order' => 10002,
						'icon' => 'icon-editlist',
						'async' => true,
						'perform' => "SUGAR.ui.arrangeDetailTabs('" . $this->module . "', '" . $this->record_id . "')",
						'group' => 'tools',
						'mode' => 'any',
					);
				}
				$buttons['show_actions'] = array(
					'type' => 'button',
					'vname' => 'LBL_SHOW_ACTIONS',
					'order' => 10003,
					'icon' => 'icon-view',
					'perform' => 'return SUGAR.popups.openUrl("async.php?module=RecordActions&action=PopupActions&parent_type='.$this->module.'&parent_id='.$this->record_id.'", {}, {}, {});',
					'mode' => 'view',
					'group' => 'tools',
				);
			}

			if(isset($themeInstance->classic_title)) {
				$list_layouts = AppConfig::setting("views.detail.{$this->module}.list");
				if (is_array($list_layouts)) {
					$buttons['return_list'] = array(
						'type' => 'button',
						'vname' => 'LBL_RETURN_TO_LIST_BUTTON_LABEL',
						'order' => 80,
						'icon' => 'icon-return',
						'align' => 'right',
						'onclick' => 'return SUGAR.ui.returnToList({FORM});',
						'acl' => 'list',
					);
				}
				if (AppConfig::is_mobile()) {
					$buttons['return_list']['vname'] = '';
				}
				if($this->module == 'Users' && ! is_admin()) {
					$buttons['return_list']['hidden'] = 1;
				}
			}

			$std = AppConfig::setting("model.detail.{$this->model->name}.standard_records");
			if(is_array($std) && in_array($this->record_id, $std)) {
				unset($buttons['delete']);
			}

		} else {
			$buttons = array(
				'save' => array(
					'vname' => 'LBL_SAVE_BUTTON_LABEL',
					'accesskey' => 'LBL_SAVE_BUTTON_KEY',
					'order' => 10,
					'icon' => 'icon-accept',
					'params' => array(
						'record_perform' => 'save',
					),
					'mode' => 'edit',
				),
				'validate' => array(
					'label' => 'Validate',
					'order' => 20,
					'params' => array(
						'record_perform' => 'validate',
					),
					'type' => 'button',
					'group' => 'tools',
					'mode' => 'edit',
				),
				'cancel' => array(
					// FIXME - need to set record when canceling a duplicate (set to duplicate_of_id or redirect to list)
					'type' => 'button',
					'vname' => 'LBL_CANCEL_BUTTON_LABEL',
					'accesskey' => 'LBL_CANCEL_BUTTON_KEY',
					'order' => 50,
					'icon' => 'icon-cancel',
					'onclick' => 'return SUGAR.ui.cancelEdit({FORM});',
					'mode' => 'edit',
				),
			);

			$disabled_buttons = AppConfig::setting("views.layout.{$this->module}.edit.{$this->layout_name}.disabled_buttons", array());
			foreach ($disabled_buttons as $b) {
				unset($buttons[$b]);
			}

			if(! AppConfig::setting('layout.show_validate_button', false))
				unset($buttons['validate']);
		}

		$buttons['tools'] = array(
			'order' => 60,
			'type' => 'group',
			'icon' => 'icon-action',
			'align' => 'right',
			'promote' => 'tools',
			'mode' => 'any',
		);

		$disabled_models = AppConfig::setting("layout.disable_layout_editor", array());
		$editor_disabled = in_array($this->model->name, $disabled_models);


		if (!$editor_disabled && $this->editLayout) {
			$key = ($perf == 'edit') ? 'save_layout_edit' : 'save_layout_view';
			$buttons = array(
				$key => array(
					'vname' => 'LBL_SAVE_LAYOUT_BUTTON_LABEL',
					'order' => 10,
					'icon' => 'icon-accept',
					'pre_perform' => 'SUGAR.le.fillForm(document.forms.' . $this->form_name . '); SUGAR.le.destroy();',
					'class' => 'admin-action',
					'params' => array(
						'record_perform' => $key,
					),
					'mode' => 'any',
				),
			);
			if ($this->record_id) {
				$return_layout_name = $this->layout_name;
				if ($return_layout_name == 'Mobile')
					$return_layout_name = '';
				$buttons[$perf] = array(
					'vname' => 'LBL_CANCEL_BUTTON_LABEL',
					'order' => 20,
					'icon' => 'icon-cancel',
					'class' => 'admin-action',
					'params' => array(
						'record_perform' => $perf,
						'layout' => $return_layout_name,
					),
					'conn_params' => array(
						'no_validate' => true,
					),
					'mode' => 'any',
					'pre_perform' => 'SUGAR.le.destroy();',
				);
			} else {
				$buttons[$perf] = array(
					'vname' => 'LBL_CANCEL_BUTTON_LABEL',
					'order' => 20,
					'icon' => 'icon-cancel',
					'class' => 'admin-action',
					'async' => false,
					'params' => array(
						'module' => 'NewStudio',
						'action' => 'index',
						'wizard' => 'EditLayout',
						'edit_module' => $this->module,
					),
					'conn_params' => array(
						'no_validate' => true,
					),
					'mode' => 'any',
					'pre_perform' => 'SUGAR.le.destroy();',
				);
			}
		} else if (!$editor_disabled && is_admin($current_user) && !AppConfig::is_mobile() && !$this->in_popup && !AppConfig::read_only()) {
			$key = $perf == 'edit' ? 'edit_layout_edit' : 'edit_layout_view';
			$key2 = $perf == 'edit' ? 'restore_layout_edit' : 'restore_layout_view';
			$key3 = $perf == 'edit' ? 'default_layout_edit' : 'default_layout_view';
			$key4 = $perf == 'edit' ? 'mobile_layout_edit' : 'mobile_layout_view';
			$key5 = $perf == 'edit' ? 'mobile_layout_edit_create' : 'mobile_layout_view_create';
			$buttons2 = array(
				'edit_custom' => array(
					'vname' => 'LBL_EDIT_CUSTOM_FIELDS_BUTTON_LABEL',
					'order' => 10000,
					'icon' => 'icon-editlist',
					//'class' => 'admin-action',
					'async' => false,
					'params' => array(
						'module' => 'NewStudio',
						'action' => 'index',
						'wizard' => 'EditCustom',
						'edit_module' => $this->module,
					),
					'conn_params' => array(
						'no_validate' => true,
					),
					'group' => 'tools',
					'mode' => 'any',
				),
			);
			//$ltype = $this->form_gen->getLayout()->getType();
			//		'name' => ($ltype == 'record' ? 'layout_tab' : 'form-tabs'),
			$buttons2[$key] = array(
				'vname' => 'LBL_EDIT_LAYOUT_BUTTON_LABEL',
				'order' => 10001,
				'icon' => 'icon-editlayout',
				//'class' => 'admin-action',
				'params' => array(
					'record_perform' => $key,
				),
				'conn_params' => array(
					'no_validate' => true,
				),
				'group' => 'tools',
				'mode' => 'any',
			);
			if (! $this->isStandardLayout()) {
				$buttons2[$key3] = array(
					'vname' => 'LBL_DEFAULT_LAYOUT_BUTTON_LABEL',
					'order' => 10002,
					'icon' => 'icon-editlayout',
					'params' => array(
						'record_perform' => $key3,
					),
					'conn_params' => array(
						'no_validate' => true,
					),
					'group' => 'tools',
					'confirm' => 'MSG_RESTORE_LAYOUT_WARNING',
					'mode' => 'any',
				);
			}
			$action = $perf;
			if(AppConfig::setting('site.features.combined_detail') && empty($_REQUEST['std'])) {
				$action = 'record';
			}
			$backup = AppConfig::custom_dir()."modules/{$this->module}/new_views/$action.{$this->layout_name}.backup.php";
			if (is_readable($backup)) {
				$buttons2[$key2] = array(
					'vname' => 'LBL_RESTORE_LAYOUT_BUTTON_LABEL',
					'order' => 10003,
					'icon' => 'icon-editlayout',
					'params' => array(
						'record_perform' => $key2,
					),
					'conn_params' => array(
						'no_validate' => true,
					),
					'group' => 'tools',
					'mode' => 'any',
				);
			}

			if (AppConfig::is_mobile_module($this->module) && ! $this->in_popup) {
				$wireless_detail = AppConfig::setting("views.detail.{$this->module}.{$perf}.Mobile");
				if (empty($wireless_detail)) {
					$buttons2[$key5] = array(
						'vname' => 'LBL_CREATE_MOBILE_LAYOUT_BUTTON_LABEL',
						'order' => 10004,
						'icon' => 'icon-editlayout',
						'params' => array(
							'record_perform' => $key,
							'layout' => 'Mobile',
						),
						'conn_params' => array(
							'no_validate' => true,
						),
						'group' => 'tools',
						'mode' => 'any',
					);
				} else {
					$buttons2[$key4] = array(
						'vname' => 'LBL_EDIT_MOBILE_LAYOUT_BUTTON_LABEL',
						'order' => 10004,
						'icon' => 'icon-editlayout',
						'params' => array(
							'record_perform' => $key,
							'layout' => 'Mobile',
						),
						'conn_params' => array(
							'no_validate' => true,
						),
						'group' => 'tools',
						'mode' => 'any',
					);
				}
			}

			$buttons = array_merge($buttons, $buttons2);
		}

		$p = $perf == 'duplicate' ? 'edit' : $perf;
		$action = $p;
		if(isset($this->form_def)) {
			if(isset($this->form_def['buttons_hook'])) {
				call_hook_function($this->form_def['buttons_hook'], null, array($this, &$buttons));
			}
			if(isset($this->form_def['buttons_hooks'])) {
				foreach($this->form_def['buttons_hooks'] as $hook) {
					call_hook_function($hook, null, array($this, &$buttons));
				}
			}
		}
		return $buttons;
	}

	function getViewLayouts() {
		if(AppConfig::setting('site.features.combined_detail') && empty($_REQUEST['std']))
			return;
		return AppConfig::setting("display.view.{$this->model->name}.layouts");
	}

	function getStandardTabs() {
		if($this->perform == 'view')
			$layouts = $this->getViewLayouts();
		$tabs = array();
		if(! empty($layouts)) {
			foreach($layouts as $k => $info) {
				$tab = $info;
				$tab['name'] = $k;
				$tab['label'] = translate($tab['vname'], $this->module);
				$tabs[] = $tab;
			}
		}
		return $tabs;
	}

	function loadSubpanels() {
		if($this->subpanels) {
			if (! $this->subpanelMgr) {
				$this->initSubPanelManager();
			} else {
				$this->subpanelMgr->setRecord($this->record);
			}
		}
	}

	function getParentRecord() {
		if(isset($this->parent_record))
			return $this->parent_record;
		$this->parent_record = false;

		if($this->perform == 'duplicate' && $this->duplicate_of_id) {
			$lq = new ListQuery($this->model->getModelName());
			$lq->addDisplayName('_display');
			$lq->addAclChecks('edit');
			$rec = $lq->queryRecord($this->duplicate_of_id);
			if($rec) {
				$rec->setRelatedData('layout', $this->layout_name);
				$rec->setRelatedData('panel', null);
			}
		}
		else {
			foreach(array('return_module', 'return_action', 'return_record', 'return_layout', 'return_panel') as $k) {
				if(empty($_REQUEST[$k])) {
					if($k == 'return_panel' || $k == 'return_layout') $$k = null;
					else return false;
				} else
					$$k = $_REQUEST[$k];
			}
			if(($return_action != 'DetailView' && $return_action != 'index') || ! ACLController::checkAccess($return_module, 'view')) {
				// 'index' added specifically for email create-related-record action from listview
				return false;
			}
			if($return_module == $this->module && $return_record == $this->record_id) {
				// don't reference self
				return false;
			}
			$lq = new ListQuery(AppConfig::module_primary_bean($return_module));
			$lq->addDisplayName('_display');
			$lq->addAclChecks('edit');
			$rec = $lq->queryRecord($return_record);
			if($rec) {
				$rec->setRelatedData('layout', $return_layout);
				$rec->setRelatedData('panel', $return_panel);
			}
		}
		if(! $rec || $rec->failed)
			$rec = false;
		$this->parent_record = $rec;
		return $this->parent_record;
	}

	function getReturnParams() {
		$params = array('return_url', 'return_module', 'return_action', 'return_record', 'return_layout', 'return_panel', 'no_redirect');
		foreach($params as $f)
			$$f = array_get_default($_REQUEST, $f);
		if(! $return_url) {
			if($return_module && ($return_module != $this->module || ($return_action != $this->action && $return_action != 'DetailView'))) {
				$return_url = "index.php?module=$return_module&action=$return_action&layout=$return_layout";
				if($return_record) $return_url .= "&record=$return_record";
				if($return_panel) $return_url .= '#subpanel-'.$return_panel;
			}
			if (!$return_url) {
				if ($this->perform == 'save') {
					if(! empty($_REQUEST['in_popup']))
						$return_action = 'EditView';
					else
						$return_action = 'DetailView';
				}
			}
		}
		return compact($params);
	}

	function performUpdate() {
		global $pageInstance;
		$perf = $this->perform;

		if($perf == 'subpanel_insert' || $perf == 'subpanel_render' || $perf == 'subpanel_remove') {
			$pageInstance->set_partial_page();
			$link_name = array_get_default($_REQUEST, 'link_name');
			$update_link = array_get_default($_REQUEST, 'update_link', $link_name);
			$list_id = array_get_default($_REQUEST, 'list_id');
			if($link_name) {
				$offset = array_get_default($_REQUEST, 'list_offset');
				$order_by = array_get_default($_REQUEST, 'list_order_by');
				if($perf == 'subpanel_insert') {
					$sel_ids = array_get_default($_REQUEST, 'selected_id');
					$this->subpanelMgr->handlePanelInsert($update_link, $sel_ids);
					$offset = null;
				} elseif($perf == 'subpanel_remove') {
					$link_id = array_get_default($_REQUEST, 'link_id');
					$this->subpanelMgr->handlePanelRemove($update_link, $link_id);
					$offset = null;
				}
				if($list_id)
					$this->subpanelMgr->renderPanelByName($link_name, $list_id, $offset, $order_by);
			}
			return true;
		}

		if($this->performingEdit()) {
			$this->row_update = RowUpdate::for_result($this->record, $this->duplicate_of_id);
			$this->row_update->setSource('editview', $perf);
			$upd =& $this->row_update;
			$this->loadUpdateRequest($upd);
			$this->form_gen->loadUpdateRequest($upd, $_REQUEST);

			if ($upd->new_record) {
				$upd->check_duplicates = true;
				$this->record->secondary_queries = true;
			}
			if($perf == 'delete') {
				// ACL is checked in DetailManager initLayout
				if(! $upd->markDeleted()) {
					pr2('delete failed');
					$perf = 'view';
					return;
				} else {
					$this->model->execHooks('after_edit', array(&$this), true);

					$cust_redir = $this->getCustomRedirect();
					if($cust_redir) return $cust_redir;

					$return_p = $this->getReturnParams();
					extract($return_p);

					if($return_url) {
						return array('redirect', array('path' => $return_url));
					} else {
						$redir = array('redirect', array('module' => $this->module, 'action' => 'index', 'layout' => '', 'record' => ''));
						return $redir;
					}
				}
			}
			if($perf == 'validate' || $perf == 'save') {
				$edit_validated = $upd->validate();
				$gen_val = $this->form_gen->validateInput($upd);
				if(isset($gen_val)) $edit_validated &= $gen_val;
				if(! $edit_validated) {
					if(AppConfig::setting('site.allow_debug'))
						pr2($upd->getErrors(), 'validate errors', false, true);
					else {
						$errors = $upd->getErrors(1);
						foreach ($errors as $e)
							add_flash_message($e, 'error');
					}
					if($perf == 'save')
						$this->perform = $perf = 'validate';
				}
				else if($perf == 'save') {
					$this->form_gen->addUpdateHooks($upd);
					if(! $upd->new_record) $act = 'update';
					else $act = 'insert';
					if($upd->save($act)) {
						$this->record_id = $upd->getPrimaryKeyValue();
						$parent = $this->getParentRecord(); // for attaching to subpanel
						if($act == 'insert' && $parent && $this->record_id && $parent->getRelatedData('panel')) {
							$stop = false;
							$this->model->execHooks('before_subpanel_create', array(&$this, &$stop));
							if (!$stop) {
								$parent->setRelatedData('panel', $this->getCorrectPanelName($parent->getRelatedData('panel')));
								require_once('include/SubPanel/SubPanelManager.php');
								$psubmgr = new SubPanelManager($parent);
								$psubmgr->setPanelList([$parent->getRelatedData('panel')]);
								$psubmgr->handlePanelInsert($parent->getRelatedData('panel'), $this->record_id, null, true);
								$this->model->execHooks('after_subpanel_create', array(&$this));
							}
						}

						$this->trackView($upd);

						$this->model->execHooks('after_edit', array(&$this), true);

						$cust_redir = $this->getCustomRedirect();
						if($cust_redir) return $cust_redir;

						$return_p = $this->getReturnParams();
						extract($return_p);
						if(! empty($_REQUEST['new_after_save']))
							$return_url = '';

						if($return_url && ! $no_redirect) {
							return array('redirect', array('path' => $return_url));
						} elseif (! $no_redirect) {
							/* FIXME for some reason fields containing HTML may not be escaped
							   when form is rendered after this point? */
							/*$perform = 'view';
							load_layout_template();
							init_layout();
							load_record();*/

							// doing an internal redirect instead for now
							$record_id = $this->record_id;
							if (! empty($return_record))
								$record_id = $return_record;
							$action = $this->action;

							if($action == 'DetailView' || $action == 'EditView' || $action == 'Duplicate') {
								if (! empty($_REQUEST['new_after_save'])) {
									$params = array('module' => $this->module, 'action' => 'EditView');
									foreach(array(
										'account_id', 'parent_id', 'parent_type',
										'user_invitees', 'contact_invitees', 'lead_invitees',
										'resource_invitees',
										'phone_number', 'direction',
										'booking_category_id', 'assigned_user_id') as $f)
										if(isset($_REQUEST[$f])) $params[$f] = $_REQUEST[$f];
									if(isset($_REQUEST['date_start']))
										$params['prev_date_start'] = $_REQUEST['date_start'];
									if(isset($_REQUEST['quantity']))
										$params['prev_quantity'] = $_REQUEST['quantity'];
									return array('redirect', $params);
								}
							}

							if (! empty($return_action))
								$action = $return_action;
							if($return_module == $this->module && $action == 'DetailView') {
								$custom_redir = AppConfig::setting("display.edit.{$this->model->name}.redirect_action");
								if($custom_redir)
									$action = $custom_redir;
							}
							$record_perform = 'view';
							if($action == 'EditView')
								$record_perform = 'edit';

							$redir = array('perform', array('module' => $this->module, 'action' => $action, 'record' => $record_id, 'record_perform' => $record_perform));
							if($return_module == $this->module && $return_layout) $redir[1]['layout'] = $return_layout;
							if($this->debug) $redir[1]['debug'] = $this->debug;
							return $redir;
						}
					} else {
						// FIXME
						$errors = $upd->getErrors();
						if (!empty($errors)) {
							$msg = null;
							foreach ($errors as $e) {
								if ($msg)
									$msg .= "\n";
								$msg .= $e[1];
							}
							$GLOBALS['forceFlashMessages'] = true;
							add_flash_message($msg, 'error');
						} else {
							pr2('error saving');
						}
						$this->perform = $perf = 'edit';
					}
				}
			}

			if($perf != 'save' && $perf != 'view') {
				if($upd->getChanges() && $perf != 'edit' && $perf != 'duplicate') {
					if(AppConfig::setting('site.allow_debug'))
						pr2($upd->debugGetChanges(), 'record changes', false, true);
				}
				$upd->clearHooks();
				$upd->updateResult($this->record);
			}
		} else if ($perf == 'save_layout_edit' ||  $perf == 'save_layout_view') {
			require_once 'include/config/format/ConfigWriter.php';
			require_once 'include/config/LanguageManager.php';
			$act = ($perf == 'save_layout_edit') ? 'edit' : 'view';
			if(AppConfig::setting('site.features.combined_detail') && empty($_REQUEST['std'])) {
				$act = 'record';
			}
			$detail = AppConfig::setting("views.detail.{$this->module}.{$act}.{$this->layout_name}");
			if (empty($detail)) {
				$act = ($perf == 'save_layout_edit') ? 'edit' : 'view';
				$detail = AppConfig::setting("views.detail.{$this->module}.{$act}.{$this->layout_name}");
			}
			if (empty($detail)) {
				$detail = array(
					'type' => 'view',
					'name' => $this->layout_name,
					'title' => 'LBL_MODULE_TITLE',
				);
			}
			$layout = AppConfig::setting("views.layout.{$this->module}.{$act}.{$detail['name']}", array());
			$form_def = array(
				'detail' => $detail,
				'layout' => $layout,
			);
			AppConfig::invalidate_cache('views');
			AppConfig::cache_reset();
			$lang = array();
			$this->layout->updateFromRequest($form_def['layout'], $lang, $_REQUEST);
			$writer = new ConfigWriter;
			$filename = AppConfig::custom_dir()."modules/{$this->module}/new_views/$act.{$this->layout_name}.php";
			if (is_file($filename)) {
				$backup = AppConfig::custom_dir()."modules/{$this->module}/new_views/$act.{$this->layout_name}.backup.php";
				if (!@rename($filename, $backup)) {
					throw new IAHError("Cannot save backup file : $backup");
				}
			}
			$writer->writeFile($filename, $form_def);

			if (!empty($lang)) {
				$curLang = AppConfig::setting("lang.strings.current.{$this->module}");
				foreach ($lang as $vname => $vtext) {
					$curLang[$vname] = $vtext;
				}
				AppConfig::set_local("lang.strings.current.{$this->module}", $curLang);
				AppConfig::save_local('lang');
			}

			$return_layout_name = $this->layout_name;
			if ($return_layout_name == 'Mobile')
				$return_layout_name = '';
			if ($this->record_id || $this->action == 'EditView') {
				return array(
					'perform',
					array(
						'module' => $this->module,
						'action' => $this->action,
						'record' => $this->record_id,
						'record_perform' => $perf == 'save_layout_edit' ? 'edit' : 'view',
						'layout' => $return_layout_name,
					)
				);
			} else {
				return array(
					'redirect',
					array(
						'module' => 'NewStudio',
						'action' => 'index',
						'wizard' => 'EditLayout',
						'edit_module' => $this->module,
					)
				);
			}
		} else if ($perf == 'restore_layout_edit' ||  $perf == 'restore_layout_view') {
			$act = ($perf == 'restore_layout_edit') ? 'edit' : 'view';
			$lcls = $act;
			if(AppConfig::setting('site.features.combined_detail') && empty($_REQUEST['std'])) {
				$lcls = 'record';
			}
			$filename = AppConfig::custom_dir()."modules/{$this->module}/new_views/$lcls.{$this->layout_name}.php";
			$backup = AppConfig::custom_dir()."modules/{$this->module}/new_views/$lcls.{$this->layout_name}.backup.php";
			if (!@rename($backup, $filename)) {
				throw new IAHError("Cannot save layout file : $filename");
			}
			/*
			if(! file_exists($filename) || ! unlink($filename)) {
				throw new IAHError("Cannot remove custom layout file: $filename");
			}
			 */
			AppConfig::invalidate_cache('views');
			AppConfig::cache_reset();
			return array(
				'perform',
				array(
					'module' => $this->module,
					'action' => $this->action,
					'record' => $this->record_id,
					'record_perform' => $act,
				)
			);
		} else if ($perf == 'default_layout_edit' ||  $perf == 'default_layout_view') {
			$act = ($perf == 'default_layout_edit') ? 'edit' : 'view';
			$lcls = $act;
			if(AppConfig::setting('site.features.combined_detail') && empty($_REQUEST['std'])) {
				$lcls = 'record';
			}
			$filename = AppConfig::custom_dir()."modules/{$this->module}/new_views/$lcls.{$this->layout_name}.php";
			$backup = AppConfig::custom_dir()."modules/{$this->module}/new_views/$lcls.{$this->layout_name}.backup.php";
			if (is_file($filename) && !@unlink($filename)) {
				throw new IAHError("Cannot delete layout file : $filename");
			}
			if (is_file($backup) && !@unlink($backup)) {
				throw new IAHError("Cannot delete layout file : $backup");
			}

			AppConfig::invalidate_cache('views');
			AppConfig::cache_reset();
			return array(
				'perform',
				array(
					'module' => $this->module,
					'action' => $this->action,
					'record' => $this->record_id,
					'record_perform' => $act,
				)
			);
		}
	}

	function loadUpdateRequest(RowUpdate &$upd) {
		$upd->loadRequest();
	}

	function getFormTitle($html=true) {
		$t = parent::getFormTitle($html);
		if($html && ! $this->in_popup) {
			$parent = $this->getParentRecord(); // for attaching to subpanel
			if($parent) {
				$parent_name = $parent->getField('_display', '');
				$parent_mod = $parent->module_dirs[0];
				$detail_link = get_detail_link_info(
					$parent->row['id'],
					$parent_mod,
					null,
					['layout' => $parent->getRelatedData('layout')]
				);
				$parent_link = '<a href="' . to_html($detail_link['url']) . '" class="titleLink" onclick="SUGAR.util.handleLink(event)">' . to_html($parent_name) . '</a>';
				$parent_icon = get_module_header_icon($parent_mod, '');
				$t .= ' <span class="parentLink">&laquo; ' . $parent_icon . ' ' . $parent_link . '</span>';
			}
		}
		return $t;
	}

	function standardInit($id=null) {
		try {
			parent::standardInit($id);
		} catch(IAHRecordLoadError $e) {
			if(AppConfig::setting('site.log.detail_errors') && (! ($e instanceof IAHInternalError) || AppConfig::setting('site.log.detail_internal_errors')))
				throw $e;
			throw new IAHError("The specified record could not be found.");
		}
		if($this->subpanelMgr) {
			$this->subpanelMgr->setRecord($this->record);
		}
		$this->loadSubpanels();
	}

	function initLayout() {
		if($this->perform == 'load_subpanels') {
			if($this->form_def)
				foreach(array('form_buttons', 'summary', 'sections', 'scripts', 'styles') as $k)
					unset($this->form_def[$k]);
		}
		parent::initLayout();
		if($this->perform == 'edit' && $this->form_gen && ! $this->record_id) {
			$this->form_gen->getFormObject()->setAutoFocus();
		}
	}

	function renderSubpanels() {
		if($this->subpanelMgr && !$this->editLayout) {
			if($this->format == 'html') {
				$fname = $this->form_gen->getFormObject()->getFormName();
				$ret = $this->subpanelMgr->renderHtml();
				$GLOBALS['pageInstance']->add_js_literal("sugarListView.registerSubpanelContainer('{$fname}-subpanels');", null, LOAD_PRIORITY_FOOT);
				return $ret;
			}
		}
	}

	function renderLayout() {
		global $pageInstance;
		if($this->perform == 'load_subpanels') {
			$pageInstance->set_partial_page();
			return $this->renderSubpanels();
		}

		$defer_panels = false;
		if($this->subpanelMgr && ! $this->performingEdit() && ! $this->editLayout && ! AppConfig::is_mobile()) {
			$pc = $this->subpanelMgr->getPanelCount();
			$defer_min = AppConfig::setting('site.performance.defer_load_subpanels_count');
			if($defer_min && $pc > $defer_min)
				$defer_panels = true;
			if($this->format == 'html')
				$subp_open = $this->subpanelMgr->openContainer($this->form_gen->getFormObject(), $defer_panels);
		}

		$result = parent::renderLayout();

		if(isset($subp_open))
			$result .= $subp_open;
		if(! $defer_panels && ! $this->performingEdit()) {
			$result .= $this->renderSubpanels();
		}
		if ($this->editLayout) {
			theme_hide_side_menu(true);
		}

		if($this->format == 'html') {
			$this->addPageState();
			if($this->subpanelMgr && isset($subp_open))
				$result .= $this->subpanelMgr->closeContainer();
			if(! $this->in_popup)
				$result = '<div class="detailOuter">' . $result . '</div>';
		}

		return $result;
	}

	function addPageState() {
		global $pageInstance;
		if($this->in_popup) {
			$pageInstance->add_js_language($this->module, null, LOAD_PRIORITY_BODY);
			$pageInstance->set_partial_page();
		} else {
			$show_list = $this->record_id && (ACLController::checkAccess($this->module, 'list') && ! $this->editLayout) ? 1 : 0;
			$pageInstance->set_page_state(array(
				'context' => 'detail',
				'module' => $this->module,
				'layout' => $this->layout_name,
				'record' => $this->record_id,
				'query' => $this->getQueryString(),
				'show_list' => $show_list,
				'edit_layout' => !! $this->editLayout));
		}
		if($this->editLayout)
			$pageInstance->add_js_language('NewStudio', null, LOAD_PRIORITY_FOOT);
	}

	function getQueryString() {
		global $pageInstance;
		$vars = array('module'=>'', 'action'=>'', 'record'=>'', 'layout', 'layout_tab', 'return_module', 'return_action', 'return_record', 'return_layout', 'return_panel', 'primary_name', 'std', 'debug');
		$q = array();
		foreach($vars as $k => $v) {
			if(is_int($k)) { $k = $v; $v = null; }
			$q[$k] = array_get_default($_REQUEST, $k, $v);
		}
		if($q['action'] == 'EditView') {
			foreach($_GET as $k => $v) {
				if(! isset($q[$k]) && ! is_array($v)) $q[$k] = $v;
			}
		}
		if($q['action'] == 'DetailView' || $q['action'] == 'EditView' || $q['action'] == 'Duplicate') {
			if($this->perform == 'view')
				$q['action'] = 'DetailView';
			else if($this->perform == 'edit')
				$q['action'] = 'EditView';
			else if($this->perform == 'duplicate')
				$q['action'] = 'Duplicate';
		}
		if($this->editLayout)
			$q['record_perform'] = 'edit_layout_'.$this->perform;
		if(! empty($q['primary_name'])) {
			$pn = $q['primary_name'];
			if(isset($_REQUEST[$pn]))
				$q[$pn] = $_REQUEST[$pn];
			if(substr($pn, -3) == '_id') {
				$pnt = substr($pn, 0, -3) . '_type';
				if(isset($_REQUEST[$pnt]))
					$q[$pnt] = $_REQUEST[$pnt];
			}
		}
		$add_f = $pageInstance->get_preserve_url_fields();
		foreach($add_f as $k => $v)
			if(! isset($q[$k])) $q[$k] = $v;
		return encode_query_string($q);
	}

	function providesInlineEditing() {
		return true;
	}

	function runDisplayHook() {
		if($this->form_gen && $this->form_gen->layout && ! $this->form_gen->layout->getEditingLayout()) {
			$hook_cat = ($this->perform == 'view') ? 'view' : 'edit';
			$this->model->execHooks($hook_cat, array(&$this), true);
		}
	}

	function initLayoutTitle() {
		if($this->form_gen && $this->form_gen->layout)
			$this->form_gen->layout->setTitle($this->getFormTitle($this->format == 'html'));
	}

	function trackView(RowUpdate $upd) {
		require_once('data/Tracker.php');
		$owner = $upd->getField('assigned_user_id');
		$lq = new ListQuery($this->model->name);
		$lq->addFilterPrimaryKey($this->record_id);
		$display = $lq->addDisplayName();
		$res = $lq->runQuerySingle();
		$title = $res->getField($display);
		Tracker::track_view(AppConfig::current_user_id(), $this->module, $this->model->name, $this->record_id, $title, $owner);
	}

	/**
	 * Correct panel name for multi-modules panels
	 *
	 * @param string $curren_name
	 * @return string
	 */
	function getCorrectPanelName($curren_name) {
		if ($curren_name != 'activities' && $curren_name != 'history') {
			return $curren_name;
		} else {
			return strtolower($this->module);
		}
	}

	function extendSubpanels($ext_panels) {
		$panels = array();
		// normalize subpanel definitions
		if($this->subpanels)
			foreach($this->subpanels as $k => $p) {
				if(! is_array($p))
					$panels[] = array('name' => $p);
				else {
					if(empty($p['name'])) {
						if(! is_int($k)) $p['name'] = $k;
						else continue;
					}
					$panels[] = $p;
				}
			}

		if(is_array($ext_panels))
			foreach($ext_panels as $panel) {
				$mode = $find = null;
				foreach(array('before', 'after', 'replace', 'remove') as $m) {
					if(isset($panel[$m])) {
						$mode = $m;
						$find = $panel[$m];
						break;
					}
				}
				if($find) {
					foreach($panels as $k => $p) {
						if($p['name'] == $find) {
							if($mode == 'before')
								array_splice($panels, $k, 0, array($panel));
							else if($mode == 'after')
								array_splice($panels, $k+1, 0, array($panel));
							else if($mode == 'replace')
								$panels[$k] = $panel;
							else if($mode == 'remove')
								unset($panels[$k]);
							$find = null;
							break;
						}
					}
					if(! $find) $panels[] = $panel;
				}
				else $panels[] = $panel;
			}
		$this->subpanels = $panels;
	}

	function getRowUpdate() {
		return $this->row_update;
	}

	function getCustomRedirect() {
		if(is_string($this->custom_redirect)) {
			return array('redirect', array('path' => $this->custom_redirect));
		}
		else if(is_array($this->custom_redirect)) {
			if(isset($this->custom_redirect['module']))
				return array('redirect', $this->custom_redirect);
			return $this->custom_redirect;
		}
	}

	function setCustomRedirect($redir) {
		$this->custom_redirect = $redir;
	}

	function getAclChecks() {
		$checks = parent::getAclChecks();
		if(! $checks) $checks = array();
		if($this->subpanels && ! in_array('edit', $checks))
			$checks[] = 'edit';
		return $checks;
	}

	
	function loadRecord($id = null) {
		parent::loadRecord($id);
		if ($this->subpanelMgr) {
			$this->subpanelMgr->setRecord($this->record);
			$this->subpanelMgr->setPanelList($this->subpanels);
		}
	}

	function beforeLoadRecord() {
		parent::beforeLoadRecord();

		if($this->form_gen) {
			$this->form_gen->setLinkParentRecord($this->module, $this->action, $this->record_id, $this->layout_name);

			if ($this->subpanels) {
				if (! $this->subpanelMgr)
					$this->initSubPanelManager();

				$reqd = $this->subpanelMgr->getRequiredFields();
				if($reqd)
					$this->form_gen->addRequiredFields($reqd);
			}
		}
	}

	function initSubPanelManager() {
		require_once('include/SubPanel/SubPanelManager.php');
		$this->subpanelMgr = new SubPanelManager($this->record);
		$this->subpanelMgr->setModel($this->model);
		$this->subpanelMgr->setLayoutName($this->layout_name);
		$this->subpanelMgr->setCurrentTab($this->form_gen->getLayout()->getCurrentTab());
		$this->subpanelMgr->setDefaultTab($this->form_gen->getLayout()->getDefaultTab());
		$this->subpanelMgr->setPanelList($this->subpanels);
	}
}
