<?php

use Spatie\Dropbox\TokenProvider;
use League\OAuth2\Client\Provider\AbstractProvider;
use League\OAuth2\Client\Provider\Exception\IdentityProviderException;
use League\OAuth2\Client\Token\AccessToken;
use Psr\Http\Message\ResponseInterface;
use League\OAuth2\Client\Grant\RefreshToken;

require_once 'modules/Configurator/Configurator.php';
require_once 'modules/Users/User.php';

class DropboxAuthProvider extends AbstractProvider implements TokenProvider {


	protected $systemAccount;

	public function __construct($systemAccount = true) {
		$this->systemAccount = $systemAccount;
		$clientId = $systemAccount ? Configurator::DROPBOX_KEY : User::DROPBOX_KEY;
		$clientSecret = $systemAccount ? Configurator::DROPBOX_SECRET : User:: DROPBOX_SECRET;
		$options = [
           	'clientId' => $clientId,
			'clientSecret' => $clientSecret,
		];
		parent::__construct($options);
	}

	public function getBaseAuthorizationUrl() {
    	return "https://dropbox.com/oauth2/authorize";
	}

	public function getBaseAccessTokenUrl(array $params) {
    	return "https://api.dropboxapi.com/oauth2/token";
	}

	public function getResourceOwnerDetailsUrl(AccessToken $token) {
		return '';
	}

    protected function getAuthorizationParameters(array $options) {
		$options = parent::getAuthorizationParameters($options);
		$options['token_access_type'] = 'offline';
		return $options;
    }

	protected function getDefaultScopes() {
		return "";
	}

	protected function checkResponse(ResponseInterface $response, $data) {
		if (empty($data['error_description'])) {
			return;
		}
		$error = $data['error_description'];
		throw new IdentityProviderException($error, 0, $data);
	}

	protected function createResourceOwner(array $response, AccessToken $token) {
	}


	public function getToken() : string {
		$tokenKey = $this->systemAccount ? "backup.dropbox_token" : "userpref.current_user.global.dropbox_token";
		$tokenData = AppConfig::setting($tokenKey);
		if (!is_array($tokenData)) {
			return "";
		}
		try {
			$token = new AccessToken($tokenData);
			if ($token->hasExpired()) {
				$refreshToken = $token->getRefreshToken();
				$newToken = $this->getAccessToken(new RefreshToken(), ['refresh_token' => $refreshToken]);
				$tokenData = $newToken->jsonSerialize();

				// we need to keep the old refresh token for reuse. When refreshing an access token,
				// new refresh token is not generated - the old one should be reused
				$tokenData['refresh_token'] = $refreshToken;

				AppConfig::set_local($tokenKey, $tokenData);
				AppConfig::save_local();
				$token = $newToken;
			}
			return $token->getToken();
		} catch (Exception $e) {
			$GLOBALS['log']->fatal($e->getMessage());
			return "";
		}
	}
}

