<?php

use League\OAuth2\Client\Token\AccessToken;
use League\OAuth2\Client\Grant\RefreshToken;

require_once 'include/utils/oauth_utils.php';

class GoogleSyncClient {

	private $user;
	private $client;
	
	public static $requiredScopes = [
		Google\Service\Calendar::CALENDAR,
		Google\Service\PeopleService::CONTACTS,
		Google\Service\Drive::DRIVE,
		Google\Service\Sheets::SPREADSHEETS,
	];

	const OK = 0;
	const TOKEN_NOT_AVAILABLE = 1;
	const NO_ACCESS = 2;

	public static function create($user = null) {
		return new self($user);
	}

	private function __construct($user) {
		$this->user = $user;
		$this->createClient();
	}

	private function xxx(array $a) {
		$ret = '';
		for ($i = 0, $len = count($a); $i < $len; $i++) {
			$ret .= chr($a[$i]);
		}
		return $ret;
	}

	private function createClient() {
		$client = new Google\Client();
		$client->setRedirectUri(buildOAuth2RedirectUri('google_sync'));
		$this->client = $client;
	}

	public function getClient() {
		return $this->client;
	}

	public function assignStoredToken() {
		$tokenJson = UserPreference::getPreference('google_access_token', 'global', $this->user);
		if (is_string($tokenJson)) {
			$tokenJson = @json_decode($tokenJson, true);
		}
		if (!$tokenJson)
			return;
		$expiresIn = array_get_default($tokenJson, 'expires_in');
		$refreshToken = $tokenJson['refresh_token'];
		$src = $tokenJson;
		unset($src['expires_in']);
		$token = new AccessToken($src);
		if ($token->hasExpired()) {
			$t = refershOauth2Token('google_sync', ['google_sync', $this->user->id], $refreshToken);
			if ($t) {
				$tokenJson = $t;
			$tokenJson['refresh_token'] = $refreshToken;
			}
		} else {
			$tokenJson['refresh_token'] = $refreshToken;
			$tokenJson['expires_in'] = $expiresIn;
		}
		$this->client->setAccessToken($tokenJson);
		return true;
	}

	public function authenticate($authToken) {
		$this->client->authenticate($authToken);
		$token = $this->client->getAccessToken();
        UserPreference::setPreference('google_access_token', $token, 0, 'global', $this->user);
	}

	public function verifyStoredToken() {
		try {
			try {
				$this->assignStoredToken();
			} catch (\InvalidArgumentException $e) {
				return self::TOKEN_NOT_AVAILABLE;
			}
			$token = UserPreference::getPreference('google_access_token', 'global', $this->user);
			if (is_array($token)) {
				$json = getJSONObj();
				$token = $json->encode($token);
			}
			if (!$token)
				return self::TOKEN_NOT_AVAILABLE;
			$json = getJSONObj();
			$token = $json->decode($token);
			if (!$token)
				return self::TOKEN_NOT_AVAILABLE;
			$this->createClient();
			$service = new Google\Service\Oauth2($this->client);
			$result = $service->tokeninfo(array('access_token' => $token['access_token']));

		} catch (Exception $e) {
			pr2(get_class($e));
			return $e->getMessage();
		}
		$scopes = explode(' ', $result->getScope());
		if (count(array_diff(self::$requiredScopes, $scopes)))
			return self::NO_ACCESS;
		return self::OK;
	}

}
