<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/**
 * SubPanelTiles
 *
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 */


class GroupedTabStructure
{
	var $add_websites = true;
	var $add_home_tabs = true;
	var $add_other_tab = true;
	var $add_hidden = false;
	var $add_manual = false;
	var $apply_acl = true;
	
    
    // longreach - start added
    
    function get_submodule_list() {
    	$mods = AppConfig::setting('modinfo.index.submodules');
    	$ret = array();
    	foreach($mods as $m => $ms)
    		foreach($ms as $k)
    			$ret[$k] = $m;
    	return $ret;
    }
    
    function group_modules($tabStructure, $modList) {
        global $app_strings;
        $subRevList = AppConfig::setting('modinfo.index.submodules');
        $allMods = self::get_user_modules(true, $modList, false);
        $modList = array_intersect($modList, $allMods);
        
        $retStruct = array();
        $mlhUsed = array();
        $reParent = array();

        // Only return modules which exists in the modList
        foreach($tabStructure as $mainTab => $tabInfo) {
        	$newTab = array();
        	$label = isset($tabInfo['label']) ? $tabInfo['label'] : $mainTab;
        	$newTab['key'] = preg_replace("~\W~", '', $mainTab);
        	$newTab['vlabel'] = $label;
        	$newTab['label'] = array_get_default($app_strings, $label, $label);
        	$newTab['color'] = array_get_default($tabInfo, 'color');
			$newTab['modules'] = $newTab['hidden'] = array();
            foreach($tabInfo['modules'] as $subModule) {
				if(in_array($subModule, $modList)) {
					$newTab['modules'][] = $subModule;
					$mlhUsed[$subModule] = true;
				}
				if($this->add_hidden && ! empty($mlhUsed[$subModule]) && isset($subRevList[$subModule])) {
					foreach($subRevList[$subModule] as $subSub) {
						$reParent[$subSub] = $mainTab;
					}
				}
            }
            $retStruct[$mainTab] = $newTab;
        }
        
        $newOther = array('modules' => array(), 'hidden' => array());
		foreach($modList as $module) {
			if(! isset($mlhUsed[$module])) {
				if(isset($reParent[$module])) {
					$retStruct[$reParent[$module]]['modules'][] = $module;
					$mlhUsed[$module] = true;
				}
				else {
					$newOther['modules'][] = $module;
					$mlhUsed[$module] = true;
				}
			}
		}

		$hidden = AppConfig::setting('module_order.group_hidden', []);
		if (!is_array($hidden)) $hidden = [];
		if(count($newOther['modules'])) {
			foreach($newOther['modules'] as $m) {
				if (array_search($m, $hidden) !== false)
					continue;
				$gid = AppConfig::setting("modinfo.by_name.$m.detail.default_group");
				if(! $gid || ! isset($retStruct[$gid]))
					$gid = 'LBL_TABGROUP_OTHER';
				if($gid == 'LBL_TABGROUP_OTHER' && ! isset($retStruct[$gid])) {
					if(! $this->add_other_tab)
						continue;
					$lbl = array_get_default($app_strings, $gid, $gid);
					$retStruct[$gid] = array(
						'key' => $gid,
						'vlabel' => $gid,
						'label' => $lbl,
					);
				}
				$retStruct[$gid]['modules'][] = $m;
				if(! empty($newOther['hidden'][$m]))
					$retStruct[$gid]['hidden'][$m] = true;
			}
		}
		
		return $retStruct;
    }
    
    
    function mark_active(&$groupTabs, $activeTab, $activeGroup=null) {
    	if(! isset($activeGroup))
    		$activeGroup = array_get_default($_REQUEST, 'parentTab', array_get_default($_COOKIE, 'parentTab'));
    	$subModuleList = $this->get_submodule_list();
	
		$sel = $sel_sub = $alt = $adv = null;
		$adm = array();
		foreach($groupTabs as $grp => $groupInfo) {
			if($activeGroup === false)
				$modList = array($grp => $groupInfo);
			else
				$modList = $groupInfo['modules'];
			foreach($modList as $key => $mod) {
				$chkmod = $mod;
				$sel_iframe = false;
				if(! is_int($key)) $chkmod = $key;
				if(is_array($mod)) {
					$adv = true;
					if(! empty($mod['dashboard']))
						$chkmod = 'Home';
					else if(! empty($mod['iframe'])) {
						$chkmod = 'iFrames';
						if(array_get_default($_REQUEST, 'record') == $mod['id'])
							$sel_iframe = true;
					}
				}
				if($chkmod == 'Administration' && ! isset($adm[$grp]))
					$adm[$grp] = $key;
				if($chkmod == $activeTab) {
					if((isset($_REQUEST['layout']) && $_REQUEST['layout'] == $key) || $sel_iframe) {
						$sel = array($grp, $key);
						break;
					}
					else if(! $alt)
						$alt = array($grp, $key);
				} else if(isset($subModuleList[$activeTab]) && $chkmod == $subModuleList[$activeTab])
					$sel_sub = array($grp, $key);
			}
		}
		if(! $sel) {
			if($alt) $sel = $alt;
			else if($sel_sub) $sel = $sel_sub;
			else {
				if(isset($adm[$activeGroup]))
					$sel = array($activeGroup, $adm[$activeGroup]);
				else if($adm) {
					reset($adm);
					$sel = array(key($adm), current($adm));
				} else
					return;
			}
		}
		if($activeGroup === false)
			$groupTabs[$sel[1]]['active'] = true;
		else {
			$groupTabs[$sel[0]]['active'] = $sel[1];
			$groupTabs[$sel[0]]['modules'][$sel[1]]['active'] = true;
		}
		return $sel;
    }
    
    
    function filter_more_tabs($tabStruct, $limitTabs, $limitSubTabs=null) {
    	$ret = $more = array();
    	$lastgkey = $foundActive = null;
    	$idx = 0;
    	foreach($tabStruct as $gkey => $gtab) {
			$active = array_get_default($gtab, 'active');
    		if($limitSubTabs && isset($gtab['modules'])) {
				$keys = array_keys($gtab['modules']);
				if(isset($active)) {
					$pos = array_search($active, $keys);
					if($pos !== false && $pos >= $limitSubTabs) {
						array_splice($keys, $pos, 1);
						array_splice($keys, $limitSubTabs-1, 0, $active);
						$gtab['modules'][$active]['promoted'] = true;
					}
				}
				$jdx = 0;
				$ntabs = array();
				foreach($keys as $k) {
					$tab = $gtab['modules'][$k];
					if($jdx == $limitSubTabs - 1)
						$tab['final'] = true;
					if($jdx >= $limitSubTabs)
						$tab['more'] = true;
					$ntabs[$k] = $tab;
					$jdx ++;
				}
				$gtab['modules'] = $ntabs;
			}
			if($limitTabs && $idx >= $limitTabs) {
				if($active && ! $foundActive) {
					$more = array_merge(array($lastgkey => $ret[$lastgkey]), $more);
					unset($ret[$lastgkey]);
					$gtab['promoted'] = true;
					$ret[$gkey] = $gtab;
					$lastgkey = $gkey;
				} else
					$more[$gkey] = $gtab;
			} else {
				$ret[$gkey] = $gtab;
				$lastgkey = $gkey;
			}
			if($active)
				$foundActive = true;
			$idx ++;
    	}
    	if($lastgkey && $more)
    		$ret[$lastgkey]['final'] = true;
    	foreach($more as $gkey => $gtab) {
    		$gtab['more'] = true;
    		$ret[$gkey] = $gtab;
    	}
    	return $ret;
    }
    
    
    function get_home_tabs($flat=false) {
		require_once('include/layout/DashletManager.php');
		$dmgr = new DashletManager();
		$pages = $dmgr->get_dashboard_page_info($flat);
		return $pages;
    }
    
    function get_iframe_tabs() {
    	if(! include_bean('iFrame'))
			return [];
		$iFrame = new iFrame();
		$frames = $iFrame->lookup_frames('tab', false);
		$frameLinks = array();
		foreach($frames as $v) {
			$frameLinks[$v[4]] = array(
				'id' => $v[0],
				'url' => "index.php?module=iFrames&action=DetailView&record={$v[0]}",
				'iframe' => true,
				'label' => $v[4],
				'module' => 'iFrames',
			);
		}
		return $frameLinks;
    }
    
    
    function extend_group_tabs($tabStruct) {
    	global $app_list_strings, $app_strings;
    	
		$retStruct = array();
    	$pages = $this->add_home_tabs ? $this->get_home_tabs() : array();
    	if($this->add_websites) {
    		$frameLinks = $this->get_iframe_tabs();
			$framesShown = false;
		} else
			$framesShown = true;
		
		foreach($tabStruct as $mainTab => $subModules) {
			$newGroup = $subModules;
			$mods = $post_add = array();
			
        	if(isset($pages[$mainTab])) {
        		foreach($pages[$mainTab] as $pid => $pg) {
					$url = 'index.php?module=Home&action=index&layout='.$pid;
					$url .= '&parentTab='.htmlentities($mainTab, ENT_QUOTES, 'UTF-8');
					$def = array('module' => 'Home', 'url' => $url, 'dashboard' => true, 'label' => from_html($pg['title']), 'icon' => $pg['icon']);
					if($pg['position'] >= 10) {
						$post_add[$pid] = $def;
					}
	        		else {
						$mods[$pid] = $def; 
					}
	        	}
        	}
        	
			foreach($subModules['modules'] as $k) {
				$label = array_get_default($app_list_strings['moduleList'], $k, $k);
				$url = 'index.php?module='.$k.'&action=index';
				$mods[$k] = array('module' => $k, 'url' => $url, 'label' => $label);
			}

			foreach($post_add as $k => $v)
				$mods[$k] = $v;
			
			if(! $framesShown && ! empty($frameLinks) && isset($mods['iFrames'])) {
				$framesShown = true;
				foreach($frameLinks as $name => $info) {
					$info['url'] .= '&tab='.htmlentities($mainTab, ENT_QUOTES, 'UTF-8');
					$mods[$name] = $info;
				}
			}
			
			$newGroup['modules'] = $mods;
			$retStruct[$newGroup['label']] = $newGroup;
		}
		
		if(! $framesShown) {
			$mainTab = 'LBL_TABGROUP_OTHER';
			$mainTabLbl = translate($mainTab, 'app');
			if(! isset($retStruct[$mainTabLbl])) {
				$mainTab = 'LBL_TABGROUP_MY_PORTALS';
				$mainTabLbl = translate($mainTab, 'app');
			}
			foreach($frameLinks as $name => $info) {
				if(! isset($retStruct[$mainTabLbl])) {
					$retStruct[$mainTabLbl] = array(
						'key' => $mainTab,
						'vname' => $mainTab,
						'label' => $mainTabLbl,
						'color' => '',
						'hidden' => array(),
						'modules' => array(),
					);
				}
				$info['url'] .= '&tab='.htmlentities($mainTab, ENT_QUOTES, 'UTF-8');
				$retStruct[$mainTabLbl]['modules'][$name] = $info;
			}
		}
		
		return $retStruct;
    }
    
    
    
    
    static function get_groups_definition() {
    	$groups = AppConfig::setting('module_order.grouped', array());
		foreach($groups as $idx => &$g) {
			if(empty($g['modules']))
				unset($groups[$idx]);
		}
    	return $groups;
    }

    static function get_user_modules($hidden=false, $show_manual=null, $apply_acl=false, $current=null) {
    	$mods = AppConfig::setting('modinfo.current_user', array());
    	$ret = array();
    	foreach($mods as $m => $info) {
    		if($info['hidden'] && ! $hidden) {
    			if($info['hidden'] == 'acl' && ! $apply_acl)
    				;
    			else if($info['hidden'] == 'manual' && $show_manual && in_array($m, $show_manual))
    				;
    			else if($info['hidden'] == 'user' && $current === $m)
    				;
    			else
    				continue;
    		}
    		$ret[] = $m;
    	}
    	return $ret;
    }
    
    
    function get_grouped_tabs($activeTab=null, $activeGroup=null, $limitTabs=null, $limitSubTabs=null) {
		$tabStructure = self::get_groups_definition();
		$manual = array();
		foreach($tabStructure as $tab)
			if(isset($tab['modules'])) array_extend($manual, $tab['modules']);
		
    	$modList = self::get_user_modules($this->add_hidden, $manual, $this->apply_acl, $activeTab);
    	$grouped = $this->group_modules($tabStructure, $modList);
    	$retStruct = $this->extend_group_tabs($grouped);
		
    	$this->mark_active($retStruct, $activeTab, $activeGroup);
    	$retStruct = $this->filter_more_tabs($retStruct, $limitTabs, $limitSubTabs);
		
		return $retStruct;
	}
	
	
	function get_flat_tabs($activeTab=null, $limitTabs=null) {
		global $app_list_strings;

		$ret = array();
		$pages = $this->add_home_tabs ? $this->get_home_tabs(true) : array();
		$post_add = array();
		foreach($pages as $pid => $pg) {
			$url = 'index.php?module=Home&action=index&layout='.$pid;
			$def = array('module' => 'Home', 'url' => $url, 'dashboard' => true, 'label' => $pg['title'], 'icon' => $pg['icon']);
			if($pg['position'] >= 10)
				$post_add[$pid] = $def;
			else
				$ret[$pid] = $def;
		}

		$mod_list = $this->get_user_modules($this->add_hidden, null, $this->apply_acl);
		foreach($mod_list as $mod) {
			$info = array(
				'module' => $mod,
				'url' => 'index.php?module='.$mod.'&action=index',
				'label' => translate('LBL_MODULE_TITLE', $mod),
			);
			$ret[$mod] = $info;
		}
		
		foreach($post_add as $pid => $pg)
			$ret[$pid] = $pg;
		
    	if($this->add_websites) {
    		$frameLinks = $this->get_iframe_tabs();
			foreach($frameLinks as $name => $info)
				$ret[$name] = $info;
    	}
		
		$this->mark_active($ret, $activeTab, false);
		$ret = $this->filter_more_tabs($ret, $limitTabs);
		
		return $ret;
	}
	
	
	static function get_top_level() {
		$tabStruct = self::get_groups_definition();
		return array_keys($tabStruct);
	}
		
	static function get_max_tabs() {
		global $current_user;
		if(isset($current_user) && $current_user->id) {
			$user_max_tabs = $current_user->getPreference('max_tabs');
			if($user_max_tabs > 0)
				$max_tabs = $user_max_tabs;
		}
		if(!isset($max_tabs) || $max_tabs <= 0)
			$max_tabs = AppConfig::setting('layout.defaults.max_tabs');
		return $max_tabs;
	}
	
	static function get_max_subtabs() {
		global $current_user;
		if(isset($current_user) && $current_user->id) {
			$user_max_tabs = $current_user->getPreference('max_subtabs');
			if($user_max_tabs > 0)
				$max_tabs = $user_max_tabs;
		}
		if(!isset($max_tabs) || $max_tabs <= 0)
			$max_tabs = AppConfig::setting('layout.defaults.max_subtabs');
		return $max_tabs;
	}
	
	function get_theme_tabs($grouped, $activeTab, $activeGroup=null, $max_tabs=null, $max_subtabs=null) {
		if($max_tabs === -1)
			$max_tabs = null;
		else if(! isset($max_tabs))
			$max_tabs = self::get_max_tabs();
		if($grouped) {
			if($max_subtabs === -1)
				$max_subtabs = null;
			else if(! isset($max_subtabs))
				$max_subtabs = self::get_max_subtabs();
			$tabs = $this->get_grouped_tabs($activeTab, $activeGroup, $max_tabs, $max_subtabs);
		}
		else
			$tabs = $this->get_flat_tabs($activeTab, $max_tabs);
		$ret = array('use_group_tabs' => $grouped, 'tabs' => array(), 'more_tabs' => array(), 'active' => '');
		foreach($tabs as $name => $tab) {
			$active = array_get_default($tab, 'active');
			if($active)
				$ret['active'] = $name;
			if(! isset($tab['url']))
				$tab['url'] = '';
			if(isset($tab['label']))
				$tab['label'] = to_html($tab['label']);
			if($grouped) {
				$tab['tabs'] = array();
				$tab['more_tabs'] = array();
				foreach($tab['modules'] as $name2 => $tab2) {
					if($active && $active == $name2)
						$tab2['active'] = true;
					if(empty($tab['url']))
						$tab['url'] = $tab2['url'];
					if(isset($tab['label']))
						$tab2['label'] = to_html($tab2['label']);
					if(! empty($tab2['more']))
						$tab['more_tabs'][$name2] = $tab2;
					else
						$tab['tabs'][$name2] = $tab2;
				}
				unset($tab['modules']);
				$tab['all_tabs'] = array_merge($tab['tabs'], $tab['more_tabs']);
			}
			if(! isset($ret['default_tab']))
				$ret['default_tab'] = $name;
			if(! empty($tab['more']))
				$ret['more_tabs'][$name] = $tab;
			else
				$ret['tabs'][$name] = $tab;
		}
		$ret['all_tabs'] = array_merge($ret['tabs'], $ret['more_tabs']);
		return $ret;
	}

}

?>
