<?php

# vim: set encoding=utf-8 :

if (!defined('JSON_LOOSE_TYPE')) {
	define ('JSON_LOOSE_TYPE', 0);
}

define ('JSON_ARRAY',		1);
define ('JSON_START',		2);
define ('JSON_OBJECT',		3);
define ('JSON_KEY',		4);
define ('JSON_IN_STRING',		5);
define ('JSON_PROCESS_VALUE',		6);
define ('JSON_VALUE',		7);

class JSON_Circular_Guard {}

class JSON
{
	var $mb_encoding;
	var $use_native = false;

	function __construct($type_ignored=null) {
		if(function_exists('mb_internal_encoding'))
			$this->mb_encoding = mb_internal_encoding();
		if(function_exists('json_encode') && version_compare(phpversion(), '5.2.0') >= 0)
			$this->use_native = true;
	}

	function encode($value, $no_circ_ref=false)
	{
		if($this->use_native)
			return json_encode($value);

		static $cache = array();
		if(! $no_circ_ref) {
			$cache_key = 'JSON_encode_'.gettype($value) . '_' . $value;
			if(is_array($value) || is_object($value)) {
				$cache_key = 'JSON_encode_'.md5(serialize($value));
			}
			if(isset($cache[$cache_key])) {
				$cache_value = $cache[$cache_key];
				if (is_object($cache_value) && is_subclass_of($cache_value, 'JSON_Circular_Guard')) {
					return 'null';
				}
				return $cache_value;
			}
			$cache[$cache_key] = new JSON_Circular_Guard;
		}

		switch (gettype($value)) {
			case 'boolean' :
				$result = $value ? 'true' : 'false';
				break;
			case 'integer':
			case 'double':
				$result = (string)$value;
				break;
			case 'string':
				$result = $this->encodeString($value);
				break;
			case 'array':
				$result = $this->encodeArray($value, $no_circ_ref);
				break;
			case 'object':
				$result = $this->encodeArray((array)$value, $no_circ_ref);
				break;
			default:
				$result = 'null';
				break;
		}
		if(! $no_circ_ref)
			$cache[$cache_key] = $result;
		return $result;
	}

	function encodeArray($array, $no_circ_ref=false)
	{
		if (is_array($array) && count($array) && (array_keys($array) !== range(0, sizeof($array) - 1))) {
			$ret = '';
			foreach ($array as $k => $v) {
				if ($ret !== '') $ret .= ',';
				$ret .= $this->encodeString((string)$k) . ':' . $this->encode($v, $no_circ_ref);
			}
			return '{' . $ret . '}';
		} else {
			$ret = '';
			foreach ($array as $k => $v) {
				if ($ret !== '') $ret .= ',';
				$ret .= $this->encode($v, $no_circ_ref);
			}
			return '[' . $ret . ']';
		}
	}

	function encodeString($str)
	{
		$ret = '';
		if(function_exists('mb_strlen')) {
			if($this->mb_encoding) {
				$str = mb_convert_encoding($str, "UCS-4BE", $this->mb_encoding);
				$len = mb_strlen($str, "UCS-4BE");
			}
			else
				$len = mb_strlen($str, 'latin1');
		}
		else
			$len = strlen($str);
		for ($i = 0; $i < $len; $i++) {
			if($this->mb_encoding) {
				$c = mb_substr($str, $i, 1, "UCS-4BE");
				$v = unpack('N', $c);
				if($v[1] > 0x80) {
					$ret .= sprintf('\\u%04x', $v[1]);
					continue;
				}
				$c = chr($v[1]);
			}
			else
				$c = $str[$i];
			$ord = ord($c);
			switch (true) {
				case $ord == 0x08:
					$ret .= '\b';
					break;
				case $ord == 0x09:
					$ret .= '\t';
					break;
				case $ord == 0x0A:
					$ret .= '\n';
					break;
				case $ord == 0x0C:
					$ret .= '\f';
					break;
				case $ord == 0x0D:
					$ret .= '\r';
					break;
				case $ord == 0x22:
				case $ord == 0x2F:
				case $ord == 0x5C:
					$ret .= '\\' . $c;
					break;
				case (($ord >= 0x20) && ($ord <= 0x7F)):
					$ret .= $c;
					break;
				case (($ord & 0xE0) == 0xC0):
					$ret .= sprintf('\\u%04x', (($ord & 0x1F) << 6) + (ord($str[$i+1]) & 0x3F));
					$i ++;
					break;
				case (($ord & 0xF0) == 0xE0):
					$ret .= sprintf('\\u%04x',
						(($ord & 0x0F) << 12)  +
							((ord($str[$i+1]) & 0x3F) << 6) +
							((ord($str[$i+2]) & 0x3F))
					);
					$i += 2;
					break;
				case (($ord & 0xF8) == 0xF0):
					$ret .= sprintf('\\u%04x',
							(($ord & 0x07) << 18) +
							((ord($str[$i+1]) & 0x3F) << 12) +
							((ord($str[$i+2]) & 0x3F) << 6) +
							(ord($str[$i+3]) & 0x3F)
						);
					$i += 3;
					break;
				case (($ord & 0xFC) == 0xF8):
					$ret .= sprintf('\\u%04x',
							(($ord & 0x03) << 24) +
							((ord($str[$i+1]) & 0x3F) << 18) +
							((ord($str[$i+2]) & 0x3F) << 12) +
							((ord($str[$i+3]) & 0x3F) << 6) +
							(ord($str[$i+4]) & 0x3F)
					);
					$i += 4;
					break;
				case (($ord & 0xFE) == 0xFC):
					$ret .= sprintf('\\u%04x',
							(($ord & 0x03) << 30) +
							((ord($str[$i+1]) & 0x3F) << 24) +
							((ord($str[$i+2]) & 0x3F) << 18) +
							((ord($str[$i+3]) & 0x3F) << 12) +
							((ord($str[$i+4]) & 0x3F) << 6) +
							(ord($str[$i+5]) & 0x3F)
					);
					$i += 5;
					break;
			}
		}
		return '"' . $ret . '"';
	}

	function skip_white($str, $len, &$i, &$c)
	{
		while (ctype_space($c) && $i + 1 < $len) {
			$i++;
			$c = $str[$i];
		}
	}

	function decode($str)
	{
		if($this->use_native)
			return json_decode($str, true);

		$seeking = null;
		$stack = array();
		$str = (string)$str;
		$state = JSON_START;
		$value = '';
		if(function_exists('mb_internal_encoding'))
			$mb_charset = mb_internal_encoding();
		else
			$mb_charset = false;

		$len = strlen($str);
		for ($i=0; $i < $len; $i++) {
			$c = $str[$i];
			switch ($state) {
				case JSON_START:
					$this->skip_white($str, $len, $i, $c);
					if ($i >= $len) break 2;
					switch ($c) {
						case '[' :
							$stack[] = array(
								'type' => JSON_ARRAY,
								'value' => array(),
							);
							break;
						case '{' :
							$stack[] = array(
								'type' => JSON_OBJECT,
								'value' => array(),
								'seeking' => JSON_KEY,
							);
							break;
						case '"':
							$value = '';
							$state = JSON_IN_STRING;
							break;
						case '-':
						case '0':
						case '1':
						case '2':
						case '3':
						case '4':
						case '5':
						case '6':
						case '7':
						case '8':
						case '9':
							$value = $c;
							$i++;
							while ($i < $len && strpos("0123456789Ee-+.", $str[$i]) !== false) {
								$value .= $str[$i];
								$i++;
							}
							$i--;
							if (!is_numeric($value)) {
								return null;
							}
							if (strpos($value, '.') === false && strpos($value, 'e') === false && strpos($value, 'E') === false) {
								$value = (int)$value;
							} else {
								$value = (float)$value;
							}
							$state = JSON_PROCESS_VALUE;
							break;
						case ']':
							if (empty($stack)) {
								return null;
							}
							$top =& $stack[count($stack)-1];
							if ($top['type'] != JSON_ARRAY) {
								return null;
							}
							$value = $top['value'];
							$state = JSON_PROCESS_VALUE;
							array_pop($stack);
							break;
						case '}':
							if (empty($stack)) {
								return null;
							}
							$top =& $stack[count($stack)-1];
							if ($top['type'] != JSON_OBJECT) {
								return null;
							}
							$value = $top['value'];
							$state = JSON_PROCESS_VALUE;
							array_pop($stack);
							break;
						case ',':
							if (empty($stack)) {
								return null;
							}
							$top =& $stack[count($stack)-1];
							if ($top['type'] == JSON_OBJECT && $top['seeking'] != JSON_KEY) {
								return null;
							}
							break;
						case ':':
							if (empty($stack)) {
								return null;
							}
							$top =& $stack[count($stack)-1];
							if ($top['type'] != JSON_OBJECT || $top['seeking'] != JSON_VALUE) {
								return null;
							}
							break;
						case 'n':
							if ($i + 3 < $len && $str[$i+1] == 'u' && $str[$i+2] == 'l' && $str[$i+3] == 'l') {
								$value = null;
								$state = JSON_PROCESS_VALUE;
								$i += 3;
							} else {
								return null;
							}
							break;
						case 't':
							if ($i + 3 < $len && $str[$i+1] == 'r' && $str[$i+2] == 'u' && $str[$i+3] == 'e') {
								$value = true;
								$state = JSON_PROCESS_VALUE;
								$i += 3;
							} else {
								return null;
							}
							break;
						case 'f':
							if ($i + 4 < $len && $str[$i+1] == 'a' && $str[$i+2] == 'l' && $str[$i+3] == 's' && $str[$i+4] == 'e') {
								$value = false;
								$state = JSON_PROCESS_VALUE;
								$i += 4;
							} else {
								return null;
							}
							break;
						default:
							return null;
					}
					break;
				case JSON_IN_STRING:
					switch ($c) {
						case '\\' :
							$i++;
							if ($i >= $len) {
								return null;
							}
							switch ($str[$i]) {
								case '"':
								case '/':
								case '\\':
									$value .= $str[$i];
									break;
								case 'r':
									$value .= "\r";
									break;
								case 'n':
									$value .= "\n";
									break;
								case 'f':
									$value .= "\f";
									break;
								case 't':
									$value .= "\t";
									break;
								case 'b':
									$value .= "\b";
									break;
								case 'u':
									$hex = '';
									$hexlen = 0;
									$i++;
									while (($i < $len) && ($hexlen < 4)) {
										$hex .= $str[$i];
										$hexlen++;
										$i++;
									}
									$i--;
									if (!preg_match('/^[0-9a-f]{4}$/i', $hex)) {
										return null;
									}
									$dec = hexdec($hex);
									if($mb_charset) {
										$value .= mb_convert_encoding('&#'.$dec.';', $mb_charset, 'HTML-ENTITIES');
									} else if ($dec < 0x80) {
										$value .= chr($dec);
									} elseif ($dec < 0x7FF) {
										$value .= chr(0xC0 | (($dec >> 6) & 0x1F));
										$value .= chr(0x80 | ($dec & 0x3F));
									} else {
										$value .= chr(0xE0 | (($dec >> 12) & 0x1F));
										$value .= chr(0x80 | (($dec >> 6) & 0x3F));
										$value .= chr(0x80 | ($dec & 0x3F));
									}
									break;
								default:
									return null;
							}
							break;
						case '"':
							$state = JSON_PROCESS_VALUE;
							break;
						default:
							$value .= $c;
							break;
					}
					break;
				case JSON_PROCESS_VALUE:
					$i--;
					$this->skip_white($str, $len, $i, $c);
					if (empty($stack)) {
						break 2;
					}
					$top =& $stack[count($stack)-1];
					switch ($top['type']) {
						case JSON_ARRAY:
							$top['value'][] = $value;
							break;
						case JSON_OBJECT:
							if ($top['seeking'] == JSON_KEY) {
								$top['key'] = $value;
								$top['seeking'] = $seekeing = JSON_VALUE;
							} else {
								$top['value'][$top['key']] = $value;
								$top['seeking'] = $seekeing = JSON_KEY;
							}
							break;
					}
					$state = JSON_START;
					break;
			}
		}
		if (!empty($stack)) {
			return null;
		}
		return $value;

	}
}

