<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/database/ListQuery.php');
require_once('include/layout/forms/FilterForm.php');
require_once('include/layout/forms/FormButton.php');
require_once('include/layout/FieldFormatter.php');


class ListFormatter {
	var $sources = array();
	var $columns = array();
	var $totals = array();
	var $scripts = array();
	var $scripts_export_count = 0;
	var $hooks = array();
	var $sort_order;
	/**
	 * @var ListQuery
	 */
	var $list_query;
	var $format = 'html';
	var $context = 'listview';
	var $table_class = 'listView';
	var $row_actions = array(
		'onmouseover' => "sListView.row_action(this, 'over', '{ID}');",
		'onmouseout' => "sListView.row_action(this, 'out', '{ID}');",
		'onmousedown' => "if(event.button == 0) return sListView.row_action(this, 'click', '{ID}', '', event);",
	);
	var $table_width = '100%';
	var $fill_width = true;
	var $show_navigation = true;
	var $nav_by_page = true;
	var $hide_disabled_buttons = false;
	var $hide_remove_links = false;
	var $show_checks = true;
    var $show_favorites = false;
	var $show_title = true;
	var $show_filter = true;
	var $show_advanced_filter;
	var $show_mass_update = true;
    var $show_mass_form = true;
    var $disable_mass_update_data = false;
	var $force_mass_actions_enabled = false;
	var $show_edit_view_links = false;
	var $show_select_all_selector = true;
	var $show_compact_empty = false;
	var $hide_navigation_controls = false;
	var $compact_pager = false;
	var $no_sort = false;
	var $can_shrink = false, $shrunk = false;
	var $uids = array();
	var $apply_filter = true;
	var $colspan;
	var $list_id;
	var $current_id;
	var $module;
	var $layout_module;
	var $link_name;
	var $report_id;
	var $report_run_method;
	var $report_owner;
	var $report_result;
	var $report_title;
	var $data_id;
	var $data_result;
	var $action;
	var $form_name;
	var $title = '';
	var $icon;
	var $additional_title;
	var $buttons;
	var $show_additional_details = true;
	var $show_detail_in_popup = false;
	var $listview_primary = null;
	var $auto_init_filter = true;
	var $add_filter_clauses = array();
	var $disable_query = false;
	var $default_use_key_index;
	var $add_detail_links;
	var $detail_link_parent;
	var $detail_link_url;
	var $detail_link_action;
    /**
     * @var FieldFormatter
     */
    var $field_formatter;
    var $field_formatter_spec;
    var $display_currency_id;
	/**
	 * @var FilterForm
	 */
	var $filter_form;
	/**
	 * @var ListMassUpdate
	 */
	var $massupdate;
	var $mass_panel_open;
	var $pre_title;
	var $post_title;
	var $title_class, $title_style;
	var $have_rows;
	/**
	 * @var ListResult
	 */
	var $last_result;

	var $listview_manager = null;

	var $widths;
	var $pdf;

	var $unbuffered = false;
	var $is_images_carousel = false;

	static $default_column_width = 25;
	static $default_column_perc_width = 5;
	static $max_fixed_column_width = 80;


	static function create($model = null, $link_name = null, $list_id = null, $context = 'listview', $params = null)
	{
		return new self($model, $link_name, $list_id, $context, $params);
	}

	function __construct($model = null, $link_name = null, $list_id = null, $context = 'listview', $params = null) {
		if(isset($GLOBALS['pageInstance'])) {
			$this->module = $GLOBALS['pageInstance']->module;
			$this->action = $GLOBALS['pageInstance']->action;
		}
        $this->context = $context;
		if (!isset($list_id))
			$this->list_id = ListFormatter::createId();
		else
			$this->list_id = $list_id;
		$this->buttons = array();
		$this->nav_by_page = AppConfig::setting('layout.list.navigate_by_page');

		if (is_array($params)) {
			foreach ($params as $k => $v) {
				if($k == 'parent_key')
					$pk = $v;
				else
					$this->$k = $v;
			}
		}

		if ($model instanceof ListQuery)
			$this->setQuery($model);
		else if ($model)
			$this->setModel($model, $link_name);

		if(isset($pk)) $this->setParentKey($pk);
	}

	static function createId() {
		return 'listView-' . dechex(mt_rand(4096, 65535));
	}

	function setFormat($format = 'html', $context = 'listview') {
		$this->format = $format;
		if (isset($context))
			$this->context = $context;
		if ($format == 'pdf') {
			require_once 'include/pdf/PDFManager.php';
			$this->pdf = new PDFManager;
			$this->pdf->new_page();
		}
	}

	function getFormat() {
		return $this->format;
	}

	function getContext() {
		return $this->context;
	}

	function setTitle($title) {
		$this->title = $title;
		$this->show_title = true;
	}

	function setIcon($icon) {
		$this->icon = $icon;
	}

	function addButtons($buttons) {
		if (is_array($buttons)) {
			foreach ($buttons as $k => $spec) {
				if(is_array($spec)) {
					if(empty($spec['name']) && is_string($k))
						$spec['name'] = $k;
					$widget = array_get_default($spec, 'widget');
					if($widget) {
						$wspec = AppConfig::get_widget_spec($widget);
						if(! $wspec || $wspec['type'] != 'form_button')
							continue;
						if(isset($wspec['params']))
							$spec = array_merge($wspec['params'], $spec);
						$button = new $wspec['class']($spec, $this->list_query->base_model);
					}
					else
						$button = new FormButton($spec, $this->list_query->base_model);
					$button->translate($this->list_query->getModuleDir());
				} else if($spec instanceof FormButton)
					$button = $spec;
				else
					continue;
				if (is_int($k))
					$k = $button->name;
				$button->list_id = $this->list_id;
				$this->buttons[$k] = $button;
			}
		}
	}


	function setModel($model, $link_name = null, $init_filter = null) {
		if (isset($link_name)) {
			$params = array('link_name' => $link_name);
		} else {
			$params = null;
		}

		$this->link_name = $link_name;
		$lq = new ListQuery($model, null, $params);
		$this->setQuery($lq, $init_filter);
		/*$mod = $this->list_query->getModuleDir();
		$this->detail_popups_edit = true; /*$this->seed->ACLAccess('EditView');*/
	}

	function setQuery(&$list_query, $init_filter=null) {
		$this->list_query =& $list_query;
		if ($this->auto_init_filter || $init_filter) {
			$this->filter_form = new FilterForm($list_query, 'FilterForm', $this->list_id . '-FilterForm',
			'', $this->context, $this->show_advanced_filter);
		}

		if(! $this->checkAdditionalDetails())
			$this->show_additional_details = false;

		$model = $list_query->getModelName();
		if ($this->show_additional_details)
			$this->show_additional_details = AppConfig::setting("display.list.{$model}.show_additional_details", true);
		if($model) {
			$this->field_formatter_spec = AppConfig::setting("display.list.{$model}.field_formatter");
			if ($this->field_formatter_spec && !empty($this->field_formatter_spec['required_fields'])) {
				$this->list_query->addFields($this->field_formatter_spec['required_fields']);
			}
		}

		$this->unsetKeyIndexForUnion();
	}

	function checkAdditionalDetails($model=null) {
		static $checked = array();

		if(! $this->show_additional_details)
			return false;

		if(! isset($model)) {
			if(! $this->list_query || ! $this->list_query->base_model)
				return false;
			if($this->list_query->base_model->is_union)
				return true; // currently, easier to assume that union queries do support additional details
			$model = $this->list_query->base_model->getModelName();
		}
		if(! isset($checked[$model]))
			// now auto-generating additional details layout, don't need to check for layout file
			$checked[$model] = true; //ModelDef::additional_details_layout($model);
		return $checked[$model];
	}

	/**
	 * @return ListQuery
	 */
	function &getQuery() {
		return $this->list_query;
	}

	function &getFilterForm() {
		return $this->filter_form;
	}

	function setFilterForm(&$form) {
		$this->filter_form =& $form;
	}

	function setUids($uids) {
		if ($uids == 'all') $this->uids = 'all';
		else {
			$callback = function($x) { return $x || $x === "0"; };
			$this->uids = array_unique(array_filter(explode(';', $uids), $callback));
		}
	}


	function setParentKey($parent_key) {
		$this->list_query->setParentKey($parent_key);
	}
	function setParentInfo($parent_key, $parent_action=null, $parent_layout=null, $parent_panel=null) {
		$this->setParentKey($parent_key);
		if($parent_key && $parent_action && $parent_layout) {
			$this->detail_link_parent = array(
				'parent_module' => $this->module,
				'parent_action' => $parent_action,
				'parent_layout' => $parent_layout,
				'parent_panel' => $parent_panel,
				'parent_record' => $parent_key,
			);
		}
	}

	function getParentKey() {
		return $this->list_query->getParentKey();
	}

	function getParentModel() {
		if($this->list_query->haveLink('~parent'))
			return $this->list_query->getLink('~parent')->getModelName();
	}

	function loadLayout($layout_name, $layout_type = 'list', $target = null) {
        if(! $target) {
        	if($this->layout_module)
        		$target = $this->layout_module;
        	else
				$target = $this->module;
		} else if(! $this->layout_module) {
			$this->layout_module = $target;
		}

		$layout = AppConfig::setting("views.layout.{$target}.{$layout_type}.{$layout_name}");

		if (!isset($layout['columns']))
			return false;
		if (!isset($this->list_query))
			throw new IAHError("cannot load list layout without query");
		$this->columns = array();
		if (!$this->loadColumns($layout['columns'], $this->columns))
			return false;
		$scripts = array_get_default($layout, 'scripts', array());
		foreach ($scripts as $scr)
			$this->addScript($scr);
		$this->hooks = array_get_default($layout, 'hooks', array());
		if(isset($layout['field_formatter']))
			$this->field_formatter_spec = $layout['field_formatter'];
		if(isset($layout['sort_order']) && ! $this->sort_order)
			$this->loadSortOrder($layout['sort_order']);
		if(isset($layout['default_order_by']) && ! $this->sort_order)
			$this->loadSortOrder($layout['default_order_by']);
		return true;
	}

	function addScript($scr) {
		$this->scripts[] = $scr;
	}
	function addScriptLiteral($scr, $prio=null) {
		$scr = array('literal' => $scr);
		if(isset($prio)) $scr['priority'] = $prio;
		$this->scripts[] = $scr;
	}

	function getScripts() {
		return $this->scripts;
	}

	function loadSavedSearch($module, $id, $layout_type = 'list') {
		$l = SavedSearch::load_search($module, $id);
		if(! $l)
			return false;
		$cols = unserialize($l['columns_spec']);
		if (!$this->loadColumns($cols, $this->columns))
			return false;

		$filt = unserialize($l['filters_spec']);
		if($filt)
			$this->loadFilterLayout($filt, true);
		else if(! empty($l['filter_name']))
			$this->loadFilterLayout($l['filter_name'], true);

		$vals = unserialize($l['filter_values']);
		if($vals)
			$this->filter_form->setDefaultFilter($vals);

		$ord = unserialize($l['sort_order']);
		if($ord)
			$this->loadSortOrder($ord);

        $this->filter_form->setEditStatus($l['assigned_user_id'], $l['created_by']);

		return true;
	}

	function loadReport($module, $id, $data_id = null, $layout_type = 'list') {
		require_once('modules/Reports/Report.php');
		$l = Report::load_report($module, $id);
		if(! $l)
			return false;
		if($data_id)
			return $this->loadReportData($l, $data_id);
		else
			return $this->loadReportResultArray($l);
	}


	function loadReportData($report, $data_id=null) {
		if($report instanceof RowResult) {
			$this->report_result = $report->row;
		} else {
			$this->report_result = $report;
		}
		$this->report_id = $this->report_result['id'];
		$this->report_run_method = $this->report_result['run_method'];
		$data = ListQuery::quick_fetch('ReportData', $data_id, null);
		if(! $data || $data->row['report_id'] != $this->report_id)
			return false;
		$data->row['report_name'] = $this->report_result['name'];
		return $this->loadReportResult($data, true);
	}

	function deleteReportData($module, $id, $data_id, $check_acl=true) {
		$data = ListQuery::quick_fetch('ReportData', $data_id, true, array('acl_checks' => 'delete'));
		if($data && $data->getAclAllowed('delete')) {
			$upd = RowUpdate::for_result($data);
			$upd->markDeleted();
			return true;
		}
	}


	function loadReportResult(RowResult $report, $is_data=false, $scheduled=false) {
		if($is_data) {
			$this->report_id = $report->getField('report_id');
			$this->data_id = $report->getField('id');
			$this->data_result = $report;
		}
		return $this->loadReportResultArray($report->row, $is_data, $scheduled);
	}

	function loadReportResultArray(array $report, $is_data=false, $scheduled=false) {
		if(! $is_data) {
			$this->report_id = $report['id'];
			$this->report_result = $report;
			$this->report_run_method = $report['run_method'];
			$this->report_owner = $report['assigned_user_id'];
		} else {
			$this->report_id = $report['report_id'];
			if($scheduled) $this->report_run_method = 'scheduled';
		}
		$this->getQuery()->join_personal = false; // disable erased_fields
		$this->getQuery()->setAclUserId($report['assigned_user_id']);

		if(! strlen($report['sources_spec']) && isset($report['sources'])) {
			require_once('modules/Reports/upgrade_util.php');
			$spec = upgrade_report_spec($report, $is_data);
			$report = array_merge($report, $spec);
			$this->loadSources($spec['sources_spec'], $this->sources);
		} else {
			foreach(array('columns_spec', 'filters_spec', 'filter_values', 'sources_spec') as $f)
				$report[$f] = unserialize($report[$f]);
			$this->loadSources($report['sources_spec'], $this->sources);
		}

		$cols = $report['columns_spec'];
		if (!$this->loadColumns($cols, $this->columns, 0, $is_data))
			return false;
		
		if(count($this->sources) > 1 || $is_data) {
			$this->default_use_key_index = false;
		}

		if($is_data) {
			$this->disable_mass_update_data = true;
		}

		if (isset($report['sort_order'])) {
			if(is_string($report['sort_order']))
				$report['sort_order'] = unserialize($report['sort_order']);
			if (is_array($report['sort_order']) && !$is_data) {
				$set_order = array();
				foreach ($report['sort_order'] as $ord) {
					$ord['name'] = $ord['field'];
					unset($ord['field']);
					$src = array_get_default($ord, 'source');
					if($src) {
						if(empty($this->sources[$src]))
							continue;
						if(isset($this->sources[$src]))
							$ord['link'] = $this->sources[$src]['link_name'];
						unset($ord['source']);
					}
					$added = $this->list_query->addField($ord);
					$set_order[] = array(
						'field' => $added,
						'order' => array_get_default($ord, 'order', 'ASC'),
					);
				}
				$this->list_query->setOrderBy($set_order);
			}
			if (is_array($report['sort_order']) && $is_data) {
				$this->loadSortOrder($report['sort_order']);
			}
		}


		if(! $is_data) {
			if($this->report_run_method == 'interactive')
				$this->disable_query = 'need_input'; // re-enabled in loadRequestFilter when appropriate
			else
				$this->disable_query = true;

			$report_name = $report['name'];
			$this->report_title = $report_name;
		} else {
			$this->setModel('ReportData', null, true);
			$this->filter_form->show_advanced = true;

			if(isset($report['report']))
				$report_name = $report['report'];
			else if(isset($report['report_name']))
				$report_name = $report['report_name'];
			$this->report_title = $report_name;
			$report_name = isset($report_name) ? $report_name . ': ' : '';
			$dt = $GLOBALS['timedate']->to_display_date_time($report['date_entered']);
			if(isset($report['name']) && strlen($report['name']))
				$report_name .= $report['name'] . ' (' . $dt . ')';
			else
				$report_name .= $dt;
		}

		$this->filter_form->setIcon('Reports');
		$description = isset($report['description']) ? $report['description'] : '';
		$this->filter_form->setTitle(to_html($report_name), to_html($description));
		$this->filter_form->setIsReport(true, $this->report_run_method, $is_data ? 'data' : false, $this->report_owner == AppConfig::current_user_id());

		if(! $is_data)
			$this->loadReportFilter($report, $scheduled);

		// not supported for report (reports should be self-contained)
		//else if(! empty($l['filter_name']))
		//	$this->loadFilterLayout($report['filter_name'], true);

		return true;
	}

	function loadReportFilter(array $report, $scheduled=false) {
		if(is_array($report['filters_spec'])) {
			if($scheduled)
				$this->loadFilterLayout($report['filters_spec']);
			else
				$this->loadFilterLayout($report['filters_spec'], true);
		}

		$vals = $report['filter_values'];
		if($vals) {
			if($scheduled)
				$this->filter_form->loadArrayFilter($vals);
			else
				$this->filter_form->setDefaultFilter($vals);
		}
	}


	function loadMassUpdate() {
		if (empty($this->massupdate)) {
			require_once('include/ListView/ListMassUpdate.php');
			$this->massupdate = new ListMassUpdate($this);
			if(! $this->uids)
				$this->setUids($this->massupdate->getUids());
			$this->form_name = $this->massupdate->getFormName();
			if($this->massupdate->model) {
				$handlers = AppConfig::setting("display.list.{$this->massupdate->model->name}.massupdate_handlers", array());
				foreach ($handlers as $handler) {
					$this->massupdate->addHandler($handler, 'action');
				}
				if ($this->massupdate->model->getModelName() == 'ReportData' && $this->context == 'listview') {
					$handlers = AppConfig::setting("display.list.{$this->listview_manager->model_name}.report_massupdate_handlers", array());
					foreach ($handlers as $handler) {
						$this->massupdate->addHandler($handler, 'action');
					}
				}
				$handlers = AppConfig::setting("display.list.{$this->massupdate->model->name}.massupdate_hooks", array());
				foreach ($handlers as $handler) {
					$this->massupdate->addHandler($handler, 'update_hook');
				}
			}
			if(isset($this->mass_panel_open))
				$this->massupdate->panel_open = $this->mass_panel_open;
		}
	}

	function &getMassUpdate() {
		$this->loadMassUpdate();
		return $this->massupdate;
	}

	function performMassUpdate() {
		$this->loadMassUpdate();
		$no_update = (!$this->show_mass_update || !$this->show_checks) && !$this->force_mass_actions_enabled;
		if (!$no_update)
			return $this->massupdate->handleUpdate($this, $this->disable_mass_update_data);
	}


	// currently duplicated in FilterForm
	function loadSources($spec, &$sources) {
		if($this->list_query)
			return $this->list_query->loadSources($spec, $sources);
		return true;
	}

	function getSources() {
		return $this->sources;
	}


	function loadColumns($spec, &$columns, $depth = 0, $report_data = false) {
		$add_defs = array();
		$have_link = false;

		foreach ($spec as $idx => $col) {
			if (is_string($col))
				$col = array('field' => $col);

			// used to fill in revenue_thismonth, revenue_lastmonth column headers in Projects
			if (isset($col['dynamic_label'])) {
				$dyn = $col['dynamic_label'];
				$cls = $dyn['class'];
				$fn = $dyn['function'];
				require_bean($cls);
				$lbl = call_user_func(array($cls, $fn), $col);
				if (isset($lbl))
					$col['vname'] = $lbl;
			}
			else if(isset($col['label']))
				$col['vname'] = $col['label'];

			// TODO - add fields in one go, using addFields instead of addField (more efficient)
            if ($this->list_query && ! $report_data) {
				if (!empty($col['field'])) {
					$qspec = array('name' => $col['field']);
					foreach(array('alias', 'format', 'total', 'grouped', 'sort', 'label', 'display_converted', 'max_length', 'width', 'detail_link') as $k)
						if(isset($col[$k])) $qspec[$k] = $col[$k];
					$src = array_get_default($col, 'source');
					if($src) {
						if(isset($this->sources[$src])) {
							if (isset($col['join_name']))
								$qspec['link'] = $col['join_name'];
							else
								$qspec['link'] = $this->sources[$src]['link_name'];
						}
						else {
							$ssplit = explode('.', $src, 2);
							if(count($ssplit) > 1) {
								if(isset($this->sources[$ssplit[0]]))
									$ssplit[0] = $this->sources[$ssplit[0]]['link_name'];
							}
							$src = implode('.', $ssplit);
							$sdef = $this->list_query->getFieldDefinition($src, $err);
							if($sdef && $sdef['type'] == 'ref')
								$qspec['link'] = $src;
							else
								continue;
						}

					}
					$is_total = ! empty($qspec['total']);
					if($is_total)
						$falias = $this->list_query->addTotal($qspec);
					else
						$falias = $this->list_query->addField($qspec, null, false, false, true);

                    $widget = array_get_default($col, 'widget');

					if ($falias) {
						$col['alias'] = $falias;
						if($is_total)
							$fdef = $this->list_query->getTotal($falias);
						else
							$fdef = $this->list_query->getField($falias);

						self::expand_column_from_definition($col, $fdef);
                        $widget = array_get_default($col, 'widget');
					}
					else if(! $widget)
						continue;
					else
						$fdef = null;

                    if ($widget) {
                        $wspec = AppConfig::get_widget_spec($widget);
                        if($wspec && ($wspec['type'] == 'column' || $wspec['type'] == 'field')) {
                            $wparams = array(
                            	'name' => $col['field'], 'list_id' => $this->list_id,
                            	'vname' => array_get_default($col, 'vname'),
                            	'width' => array_get_default($col, 'width'));
                            $widget_model = $this->list_query->getBaseModel();
                            $p = strrpos($col['field'], '.');
                            if($p) {
                            	$link_name = substr($col['field'], 0, $p);
                            	$wparams['list_source'] = $link_name;
                            	$link_model = $this->list_query->getLink($link_name);
                            	if($link_model) {
                            		$wparams['name'] = substr($col['field'], $p+1);
                            		$widget_model = $link_model;
                            	}
                            }
							if($fdef)
								$wparams = array_merge($fdef, $wparams);
                            $wclass = new $wspec['class']($wparams, $widget_model);
							$wclass->translate($this->list_query->getModuleDir());
							$this->list_query->addFields($wclass->getRequiredFields());
							$col['widget'] = $wclass;
                            if(method_exists($wclass, 'renderListValue') || method_exists($wclass, 'renderListCell')) {
								$col['label'] = $wclass->getLabel();
								$col['width'] = $wclass->getWidth();
							}
                        }
                        if(! $falias)
                        	continue;
                    }
				}
				if(! empty($col['dynamic_module'])) {
					$falias = $this->list_query->addField($col['dynamic_module']);
					$col['dynamic_module'] = $falias;
				}
			}
			if (!isset($col['type']))
				$col['type'] = '';

			if (isset($col['add_fields'])) {
				$c = $col['add_fields'];
				$col['add_fields'] = array();
				$this->loadColumns($c, $col['add_fields'], $depth + 1);
				foreach($col['add_fields'] as $addc) {
					if(! empty($addc['detail_link'])) $have_link = true;
				}
			}

			if(! $depth && empty($col['align'])) {
				$rght = ['int', 'float', 'currency', 'raw_currency', 'base_currency', 'duration', 'time'];
				if(in_array($col['type'], $rght))
					$col['align'] = 'right';
			}

			if(! empty($col['detail_link'])) $have_link = true;

			if (isset($col['hidden']))
				$col['hidden'] = eval_filter_flag(expand_filter_flag($col['hidden']));
			$columns[] = $col;
		}

		$show_remove = ($depth == 0 && $this->context == 'subpanel' && empty($this->hide_remove_links));
		if($show_remove && (($mname = $this->list_query->getModelName()) == 'Activity' || $mname == 'ActivityHistory'))
			$show_remove = false; // remove links hidden for activities subpanels (for now at least)
        if ($show_remove) {
        	$mname = $this->list_query->getModelName();
            $params = array('name' => 'remove_link', 'list_id' => $this->list_id, 'label' => '');
            $wspec = AppConfig::get_widget_spec('SubpanelRemoveLink');
            if ($wspec) {
                $columns[] = array(
                    'field' => 'remove_link',
                    'alias' => 'remove_link',
                    'vname' => '', 'type' => '',
                    'widget' => new $wspec['class']($params, $this->list_query->base_model)
                );
            }
        }

        if($depth == 0 && $this->context == 'sidebar') {
        	// reduce to name column(s)
        	$found_name = $found_link = $found_side = array();
        	foreach($columns as $idx => $c) {
        		//if(! empty($c['detail_link']))
        		//	$found_link[] = $idx;
        		if(! empty($c['show_sidebar']))
        			$found_side[] = $idx;
        		$ctype = array_get_default($c, 'type');
        		if(($ctype == 'name' || $ctype == 'sequence_number') && array_get_default($c, 'show_sidebar', true))
        			$found_name[] = $idx;
        	}
        	if(! $found_name && ! $found_link && ! $found_side)
        		$found = array(0);
        	else if($found_side || $found_link) $found = array_merge($found_side, $found_link);
        	else $found = $found_name;

			$c2 = array();
			foreach($columns as $idx => $c) {
				if(in_array($idx, $found)) {
					$c2[] = $c;
				}
			}
			$columns = $c2;
        }

		return true;
	}

	function loadSortOrder($ord) {
		$this->sort_order = ListQuery::parse_order_by($ord);
	}

	function getColumns() {
		return $this->columns;
	}


	static function expand_column_from_definition(array &$col, array $fdef) {
		if (!isset($col['vname'])) {
			if (isset($fdef['vname_list'])) {
				$col['vname'] = $fdef['vname_list'];
				if (isset($fdef['vname']))
					$col['std_vname'] = $fdef['vname'];
			}
			else if (isset($fdef['vname']))
				$col['vname'] = $fdef['vname'];
			if (isset($fdef['vname_module']))
				$col['vname_module'] = $fdef['vname_module'];
		} else if (isset($fdef['vname']) && $fdef['vname'] != $col['vname'])
			$col['std_vname'] = $fdef['vname'];
		foreach(array('type', 'dbType', 'widget', 'dynamic_module', 'reportable', 'detail_link', 'sortable') as $f)
			if (isset($fdef[$f]) && ! isset($col[$f]))
				$col[$f] = $fdef[$f];
		if (!isset($col['type']))
			$col['type'] = 'varchar';
		if ($col['type'] == 'ref') {
			if (isset($fdef['bean_name']))
				$col['bean_name'] = $fdef['bean_name'];
			if (isset($fdef['id_name']))
				$col['id_name'] = $fdef['id_name'];
			if (!isset($col['show_icon']))
				$col['show_icon'] = true;
			if ($col['show_icon'] && !empty($col['bean_name']))
				$col['icon'] = $col['bean_name'];
		}
		if($col['type'] == 'currency' || $col['type'] == 'base_currency') {
			if (isset($fdef['base_field']))
				$col['base_field'] = $fdef['base_field'];
			if (isset($fdef['display_converted']))
				$col['display_converted'] = $fdef['display_converted'];
		}
		self::add_column_width($col, $fdef);
	}

	static function add_column_width(array &$col, array $fdef) {
		if (!isset($col['width'])) {
			if(isset($fdef['width']))
				$col['width'] = $fdef['width'] + 5;
			else if (($w = self::default_column_type_width($col)))
				$col['width'] = $w;
			else if (isset($fdef['len']))
				$col['width'] = min($fdef['len'], 40);
		}

	}

	static function default_column_type_width(array $col) {
		switch($col['type']) {
			case 'date':
			case 'time':
			case 'email':
			case 'url':
			case 'phone':
				return 25;
			case 'datetime':
				return 30;
			case 'enum':
			case 'status':
			case 'ref':
				if(array_get_default($col, 'bean_name') == 'User')
					return 30;
				return 40;
			case 'name':
				return 50;
			case 'text':
				return 60;
		}
	}

	function getDefaultLimit($context=null) {
		if(! isset($context)) $context = $this->context;
		if($context == 'subpanel')
			return AppConfig::setting('layout.list.subpanel_display_rows', 10);
		else if($context == 'sidebar')
			return AppConfig::setting('layout.list.sidebar_display_rows', 15);
		return AppConfig::setting('layout.list.default_display_rows', 20);
	}


	function getQueryFields() {
		if (!isset($this->list_query))
			return null;
		return $this->list_query->getFields();
	}

	function addQueryFields($fields) {
		return $this->list_query->addFields($fields);
	}

	function getFieldFormatter() {
		if (empty($this->field_formatter)) {
			$cls = 'FieldFormatter';
			if (!empty($this->field_formatter_spec)) {
				if (!empty($this->field_formatter_spec['file'])) {
					require_once $this->field_formatter_spec['file'];
					$cls = $this->field_formatter_spec['class'];
				}
			}
			$this->field_formatter = new $cls();
			$add_detail_links = $this->add_detail_links;
			if(! isset($add_detail_links) && $this->context == 'sidebar') $add_detail_links = true;
			$params = array(
				'format' => $this->format,
				'context' => $this->context,
				'list_id' => $this->list_id,
				'listview_primary' => $this->listview_primary,
				'show_detail_in_popup' => $this->show_detail_in_popup,
				'add_detail_links' => $add_detail_links,
				'detail_link_url' => $this->detail_link_url,
				'detail_link_action' => $this->detail_link_action,
				'display_currency_id' => $this->display_currency_id,
				'disable_relative_time' => ! empty($this->report_id) || AppConfig::setting('layout.disable_relative_dates_times'),
			);
			if($this->detail_link_parent)
				array_extend($params, $this->detail_link_parent);
			$this->field_formatter->init($params);
		}
		return $this->field_formatter;
	}

	function formatResultRows(ListResult &$result) {
		$fmt = $this->getFieldFormatter();
		$fmt->formatListResult($result);
	}

	function formatRowColumn(&$col, &$row, ListResult $result, $id) {
		$alias = array_get_default($col, 'alias', array_get_default($col, 'name', array_get_default($col, 'field')));
		$db_row = $result->getRow($id, false);
		if ($alias)
			$fval = array_get_default($row, $alias, '');
		else
			$fval = '';

		$widget = array_get_default($col, 'widget');
		if ($widget && $widget instanceof FormElement) {
			$lparams = array();
			$lparams['module'] = $this->module;
			if($this->list_query->getLink('~parent')) {
				$lparams['link_name'] = $this->list_query->getTableAlias();
				$lparams['parent_key'] = $this->getParentKey();
			}
			if(method_exists($widget, 'renderListValue')) {
				$fval = $widget->renderListValue($this, $result, $id, $lparams);
			} elseif(method_exists($widget, 'renderListCell')) {
				$fval = $widget->renderListCell($this, $result, $id, $lparams);
				return $fval;
			}
		}

		if (isset($col['add_fields'])) {
			$lines = '';
			foreach ($col['add_fields'] as $subcol) {
				if (!empty($subcol['hidden']) || !empty($col['total']))
					continue;
				if (!empty($subcol['require_primary']) && ! strlen($fval))
					continue;
				if (isset($subcol['row_hidden'])) {
					if (eval_filter_flag(expand_filter_flag($subcol['row_hidden']), $db_row ))
						continue;
				}

				$lval = $this->formatRowColumn($subcol, $row, $result, $id);
				if (isset($lval) && $lval !== '') {
					if (!empty($subcol['show_icon'])) {
						$icon = $this->getColumnIcon($subcol, $db_row);
						if($icon) $lval = $icon . '&nbsp;' . $lval;
					}
					$fmt = array_get_default($subcol, 'list_format');
					$pos = array_get_default($subcol, 'list_position');
					$prefix = array_get_default($subcol, 'list_prefix', '');
					$suffix = array_get_default($subcol, 'list_siffix', '');
					if (is_array($prefix)) {
						$prefix = translate(array_get_default($prefix, 'vname'), array_get_default($prefix, 'vname_module', $this->layout_module));
					}
					if (is_array($suffix)) {
						$suffix = translate(array_get_default($suffix, 'vname'), array_get_default($suffix, 'vname_module', $this->layout_module));
					}
					if (strlen($prefix)) {
						$lval = "$prefix $lval";
					}
					if (strlen($suffix)) {
						$lval = "$lval $suffix";
					}
					if ($fmt == 'hyphen') {
						if($pos == 'prefix') $lval .= ' -';
						else $lval = '- ' . $lval;
					} else if ($fmt == 'parenth')
						$lval = '(' . $lval . ')';
					else if ($fmt == 'bracket')
						$lval = '[' . $lval . ']';
					else if ($fmt == 'separate') {
						if($pos == 'prefix')
							$lval .= ':';
						else if($pos == 'suffix')
							$lval = ': ' . $lval;
					}

					if ($pos == 'prefix')
						$fval = $lval . ' ' . $fval;
					else if ($pos == 'suffix') {
						$fval .= ($fmt == 'separate' ? '' : ' ') . $lval;
					} else
						$lines .= '<div class="listAddLine">' . $lval . '</div>';
				}
			}
			$fval .= $lines;
		}

		return $fval;
	}


	/** hide columns automatically depending on context
	    for example the assigned user column is hidden when showing only the current user's records
	*/
	function autoSuppressColumns() {
		$hide = array();
		if($this->filter_form)
		foreach($this->filter_form->getBasicFilterOptions() as $k => $v) {
			$target = $name = array_get_default($v, 'name', $k);
			$negate = false;
			if($v['type'] == 'flag') {
				$negate = ! empty($v['negate_flag']);
				if(isset($v['field']))
					$target = $v['field'];
			}
			if(! empty($v['auto_suppress'])) {
				if($v['type'] == 'section_ref') {
					if($this->filter_form->getFilterValue($v['id_name']))
						$hide[$target] = 1;
				}
				else if($this->filter_form->getFilterValue($name)) {
					if(! $negate)
						$hide[$target] = 1;
				}
				else if($negate)
					$hide[$target] = 1;
			}
			else if($name == 'filter_owner') {
				if($this->filter_form->getFilterValue($name) == 'mine')
					$hide['assigned_user'] = 1;
			}
		}
		if ($hide) {
			foreach ($this->columns as $k => &$col) {
				if (isset($col['field']) && !empty($hide[$col['field']]))
					if (empty($col['add_fields'])) {
						$col['hidden'] = true;
					}
			}
		}
	}


	function calcColSpan() {
		$span = 0;
		if ($this->show_checks || $this->show_additional_details)
			$span++;
        if ($this->show_favorites)
            $span++;
		foreach ($this->columns as $k => &$col) {
			if (!empty($col['hidden']) || !empty($col['total']))
				continue;
			if (!empty($col['show_icon']))
				$span++;
			$span++;
		}
		if ($this->show_edit_view_links)
			$span++;
		$this->colspan = $span;
		return $span;
	}

	function beforeRender(ListResult $listResult) {
		$this->have_rows = (!$listResult->total_counted || $listResult->total_count);
		if(! $this->report_id)
			$this->autoSuppressColumns();
		$this->loadMassUpdate();
	}

	function getMassForm(ListResult $listResult) {
        if ($this->show_mass_form) {
			$no_update = !$this->show_mass_update || !$this->show_checks;
		    $mu_form = $this->massupdate->getFormBody($listResult, $no_update, $this->disable_mass_update_data);
        } else {
            $mu_form = array('buttons' => array(), 'body' => '');
        }
        return $mu_form;
	}

	function outputHtml(ListResult &$listResult) {
		$this->beforeRender($listResult);
		$mu_form = $this->getMassForm($listResult);
		$this->addButtons($mu_form['buttons']);
		if($this->context != 'dashlet') {
			$cls = 'card-outer panel-outer listview-panel';
			if($this->context == 'sidebar')
				$cls .= ' panel-default';
			else if($this->context == 'popup')
				$cls .= ' panel-system';
			else
				$cls .= ' panel-system panel-border';
			echo '<div class="'.$cls.'">';
		}
		$this->outputTitleHeader();
		$this->outputPostHeader();
		$this->outputChart($listResult);
		$this->outputNavigation($listResult);
		$this->outputHtmlTable($listResult);
		$this->outputNavigation($listResult, true);
		if($this->context != 'dashlet')
			echo '</div>';
		echo $mu_form['body'];
	}

	function outputPdf(ListResult &$listResult, $title)
	{
		$cols = array();
		foreach ($this->columns as &$col) {
			if (!empty($col['hidden']) || !empty($col['total']))
				continue;
			$lbl = array_get_default($col, 'vname', '');
			$mod = array_get_default($col, 'vname_module', $this->layout_module);
			$cols[$col['alias']] = array(
				'title' => preg_replace('~:\s*$~', '', translate($lbl, $mod)),
			);
		}
		$data = array();
		foreach ($listResult->formatted_rows as $id => &$row) {
			$dataRow = array();
			foreach ($this->columns as &$col) {
				$fval = array_get_default($row, $col['alias'], '');
				$dataRow[$col['alias']] = $fval;
			}
			$data[] = $dataRow;
		}
		$opts = array(
			'alt-background-color' => array(240, 240, 240),
		);
		$this->pdf->DrawTable($data, $cols, $title, true, $opts);
		ob_clean();
		$this->pdf->filename = $this->list_query->getModuleDir() . '.pdf';
		$this->pdf->serve_dynamic();
		exit;

	}


	function outputPostHeader()
	{
	}

	function searchHidden() {
		if ($this->context == 'dashlet' || $this->context == 'sidebar') {
			if($this->show_filter && $this->filter_form) {
				$val = $this->filter_form->getFilterValue('filter_text');
				if(! strlen(trim($val ?? '')))
					return true;
			}
		}
	}

	function outputTitleHeader() {
		if($this->show_filter && $this->filter_form) {
			$limit = ($this->last_result ? $this->last_result->limit : null);
			$search = $this->filter_form->formatSearch($this->list_id, $this->module, $this->action, $limit, ($this->context == 'sidebar' ? 'basic': null));
		}
		else
			$search = '';
		if ($this->show_title)
			$title = $this->title;
		else
			$title = '';

		if(isset($this->icon))
			$title = get_image($this->icon, '', '') . '&nbsp;' . $title;

		$can_shrink = $this->can_shrink;
		if($this->last_result && ! $this->last_result->total_count)
			$can_shrink = false;

		if ($can_shrink && $this->show_title) {
			$title = '<div style="cursor: pointer" onclick="return sugarListView.toggleShrink(\'' . $this->list_id . '\');">'
					. '<span class="rbullet rbullet-inline">' . $title . '</span></div>';
		}

		if (isset($this->pre_title))
			$title = $this->pre_title . $title;

		if (isset($this->post_title))
			$title .= $this->post_title;

		if ($title) {
			$cls = 'card-header panel-header listview-header listview-title-header';
			if($this->context == 'dashlet') $cls .= ' dashlet-header';
			echo '<div class="'. $cls . '">';

			$cls = 'tools-row listview-title';
			if($can_shrink) $cls .= ' sectionOpen';

			echo '<div class="' . $cls . '" id="' . $this->list_id . '-title">';
			echo '<h4>' . $title . '</h4>';
			if($this->additional_title)
				echo $this->additional_title;
			echo '</div>';

			echo '</div>';
		}

		if($search) {
			/*$filt_cls = 'listview-filter ' . ($search ? (($this->filter_form->isBasicLayout()) ? ' basicFilter' : ' advancedFilter') : '');
			if($this->can_shrink) $filt_cls .= ' sectionOpen';
			$filt_id = "{$this->list_id}-title";
			$filt_style = '';
			if ($this->context == 'dashlet' || $this->context == 'sidebar') {
				$val = $this->filter_form->getFilterValue('filter_text');
				if(! strlen(trim($val)))
					$filt_style = 'display: none;';
				if($this->context == 'dashlet')
					$filt_id = "dashlet-filter-{$this->list_id}";
				else
					$filt_id = 'sidebar-list-filter';
			}
			echo "<div class=\"$filt_cls\" id=\"$filt_id\" style=\"$filt_style\">";
			echo $search;
			echo '</div>';*/

			$cls = 'card-header panel-header listview-header listview-search-header';
			if($this->context == 'dashlet') $cls .= ' dashlet-header';
			$style = '';
			if($this->searchHidden())
				$style = ' style="display: none"';
			echo '<div class="'. $cls . '"'.$style.'>';
			echo $search;
			echo '</div>';
		}
	}


	function outputChart(ListResult &$listResult) {
		if($this->data_result && $this->data_result->getField('chart_type')) {
			$cls = 'card-header panel-header listview-header listview-chart-header';
			if($this->context == 'dashlet') $cls .= ' dashlet-header';
			echo '<div class="'. $cls . '">';
			require_once('modules/ReportData/Chart.php');
			display_report_chart($this->data_result);
			echo '</div>';
		}
	}


	function outputHtmlTable(ListResult &$listResult) {
		$this->calcColSpan();

		$empty_body = ! $this->have_rows && $this->show_compact_empty;
		$cls = 'card-body panel-body listview-body' . ($empty_body ? ' empty' : '');
		if($this->context == 'dashlet') $cls .= ' dashlet-body';

		echo '<div id="' . $this->list_id . '-body" class="' . $cls. '">';
		if (! $this->disable_query && (($this->context != 'dashlet' && $this->context != 'subpanel') || $this->nav_by_page))
			$this->outputCustomOffsetPopup();
		echo '<table id="' . $this->list_id . '-main" class="' . $this->table_class . '" cellpadding="0" cellspacing="0" border="0" width="' . $this->table_width . '">';

		if (! $empty_body && ! $this->disable_query) {
			$this->outputHtmlHeader();

			echo '<tbody>';
			$this->outputHtmlRows($listResult);
			echo '</tbody>';
		}

		echo '</table>';
		echo '</div>';
	}


	function outputHtmlHeader() {
		echo '<thead class="listViewColumns">';

		echo '<tr class="listHead">';

		if ($this->show_checks || $this->show_additional_details) {
			$thc = 'listViewThS1 listViewMeta';
			if ($this->show_checks && $this->have_rows) {
				$thc .= ' complex';
				$th1 = $this->outputItemsSelector();
			}
			else
				$th1 = '&nbsp;';
			echo "<th class=\"$thc\">$th1</th>";
		}

        if ($this->show_favorites)
            echo '<th class="listViewThS1">&nbsp;</th>';

		$labels = array();
		foreach ($this->columns as $k => &$col) {
			if (!empty($col['hidden']) || !empty($col['total']))
				continue;
			if(isset($col['label']))
				$labels[$k] = $col['label'];
			else {
				$lbl = array_get_default($col, 'vname', '');
				$mod = array_get_default($col, 'vname_module', $this->layout_module);
				$labels[$k] = preg_replace('~:\s*$~', '', translate($lbl, $mod));
			}
		}

		$widths = $this->normalizeColumnWidths();
		$current_sort = $this->getResultSort();
		$json = getJSONobj();
		$ms_form = null;
		if($this->massupdate)
			$ms_form = $this->massupdate->getFormId();
		$ms_form = $ms_form ? '"' .javascript_escape($ms_form) . '"' : 'null';

		foreach ($labels as $k => $lbl) {
			$lbl = to_html($lbl);
			if (!empty($this->columns[$k]['show_icon']))
				$colspan = ' colspan="2"';
			else
				$colspan = '';

			$sort_info = $this->getColumnSort($k, $current_sort);
			$h_cls = '';

			if(! empty($this->columns[$k]['label_nowrap']) || strlen($lbl) < 15) {
				$h_cls .= 'nowrap ';
			}
			if ($sort_info['image'] && $lbl != '') {
				$arrow = "<div class=\"input-icon list-sort icon-{$sort_info['image']}\"></div>";
			} else
				$arrow = '';

			if ($lbl != '') {
				if ($sort_info['order_str']) {
					$h_cls .= $arrow ? 'with-arrow ' : '';
					$href = '#';
					$onclk = 'return sListView.orderBy(\'' . $this->list_id . '\', \'' . $sort_info['order_str'] . '\');';
				} else {
					$arrow = '';
					$href = $onclk = '';
				}

				$colId = $this->list_id . 'ColHead' . $k;
				$colCls = 'listColHead';
				$innerCls = 'listColHeadInner';
			    if(! empty($sort_info['links']) || $sort_info['order_str'])
			    	$colCls .= ' responsive';
				if ($sort_info['active'])
					$colCls .= ' sort-active';
				$align = array_get_default($this->columns[$k], 'align');
				if($align == 'right' || $align == 'center' || $align == 'justify')
					$innerCls .= ' align-' . $align;

				$lbl = "<div class=\"$colCls\" id=\"$colId\"><div class=\"$innerCls\"><div class=\"listColLabel $h_cls\">{$lbl}</div><div class=\"listColArrow\">{$arrow}</div></div></div>";
				if($href || $onclk)
					$lbl = '<a href="' . $href . '" onclick="' . $onclk . '" class="listViewThLinkS1 listColLink">' . $lbl . '</a>';
			    echo '<th class="listViewThS1 complex" width="' . $widths[$k] . '"' . $colspan . '>' . $lbl . '</th>';

			    if(! empty($sort_info['links'])) {
			    	$s_opts = array('keys' => array_keys($sort_info['links']), 'values' => $sort_info['links'], 'width' => '20em');
			    	$jso = $json->encode(array('options' => $s_opts, 'hover_show' => 1, 'delay' => 400, 'icon_key' => 'icon'));
					$this->addScriptLiteral("SUGAR.ui.registerMenuSource('$colId', $jso, $ms_form);");
			    }
            } else {
                echo '<th class="listViewThS1" width="' . $widths[$k] . '"' . $colspan . '>&nbsp;</th>';
            }
		}

		if ($this->show_edit_view_links) {
			echo '<th class="listViewThS1" width="1%">&nbsp;</th>';
		}

		echo '</tr>';
		echo '</thead>';
	}


	function getResultSort() {
		$ret = array();
		if (isset($this->last_result)) {
			$current_order_by = ListQuery::parse_order_by($this->last_result->order_by);
			if($current_order_by) {
				$ret['fields'] = array_column($current_order_by, 'field');
				$ret['order'] = strtoupper($current_order_by[0]['order']);
			}
		}
		return $ret;
	}


	function getColumnSort($index, $current_sort) {
		$col = array_get_default($this->columns, $index, array());
		$sortable = array_get_default($col, 'sortable', true);
		if(! empty($col['widget']) && ! isset($col['sortable']))
			$sortable = false;

		$order = 'ASC';
		$image = 'sort';
		$active = false;
		$order_by = array();
		$links = array();

		if($sortable && ! $this->no_sort) {
			$field = array_get_default($col, 'field', '');
			if($this->report_id && $this->data_id) {
				$src = array_get_default($col, 'source');
				if($src && isset($this->sources[$src])) {
					$src = $this->sources[$src]['link_name'];
					if($src) $field = $src . '.' . $field;
				}
				if(isset($col['format']))
					$field .= '!' . $col['format'];
			}
			if($field)
				$order_by[] = $field;

			if (isset($col['add_fields'])) {
				foreach($col['add_fields'] as $k2 => $add_f) {
					if(! empty($add_f['hidden']) || empty($add_f['field']) || ! empty($add_f['widget']))
						continue;
					$order_by[] = $add_f['field'];

					if(isset($add_f['label']))
						$sub_lbl = $add_f['label'];
					else {
						$sub_lbl = array_get_default($add_f, 'vname', '');
						$mod = array_get_default($add_f, 'vname_module', $this->layout_module);
						$sub_lbl = preg_replace('~:\s*$~', '', translate($sub_lbl, $mod));
					}
					if(! empty($sub_lbl)) {
						$links['asc'.$k2] = array(
							'label' => $sub_lbl,
							'icon' => 'icon-down',
							'perform' => "sListView.orderBy('$this->list_id', '{$add_f['field']} ASC');",
						);
						$links['desc'.$k2] = array(
							'label' => $sub_lbl,
							'icon' => 'icon-up',
							'perform' => "sListView.orderBy('$this->list_id', '{$add_f['field']} DESC');",
						);
					}
				}
			}

			$i=0;
			foreach($order_by as $ord) {
				if($ord == 'list_location' && ($this->module == 'Leads' || $this->module == 'Contacts')) {
					$order_by[$i] = 'primary_address_city';
				} else if($ord == 'list_location' && $this->module != 'Leads' && $this->module != 'Contacts') {
					$order_by[$i] = 'billing_address_city';
				}

				if($ord == 'billing_address') {
					$order_by[$i] = 'billing_address_street';
				} else if($ord == 'primary_address') {
					$order_by[$i] = 'primary_address_street';
				} else if($ord == 'alt_address') {
					$order_by[$i] = 'alt_address_street';
				}
				$i++;
			}

			if (isset($current_sort['fields']) && $order_by == $current_sort['fields']) {
				$active = true;
				if (empty($current_sort['order']) || $current_sort['order'] === 'ASC') {
					$order = 'DESC';
					$image = 'sort-down';
				} else {
					$order = 'ASC';
					$image = 'sort-up';
				}
			}
		}

		if($order_by)
			$order_str = implode(" $order, ", $order_by) . ' ' . $order;
		else
			$order_str = '';

		return compact('sortable', 'order_by', 'order_str', 'order', 'image', 'links', 'active');
	}


	function outputItemsSelector() {
		$disabled = ($this->uids == 'all') ? ' disabled="disabled"' : '';

		$selector =
				'<div class="input-field compact muted listview-select rbullet" id="' . $this->list_id . '-Selector" tabindex="1" role="button" style="margin: -1px">'
						. '<div class="input-label">'
						. '<div class="input-check">'
						. '<input type="checkbox" class="checkbox" ' . $disabled . ' id="' . $this->list_id . '-check-all" name="check-all" value="" onclick="sListView.setSelected(\'' . $this->list_id . '\', \'page\', this.checked);">'
						. '<label for="' . $this->list_id . '-check-all"></label>'
						. '</div>'
						. '</div>'
						. '</div>';

		$actions = array(
			"page" => array(
				"label" => translate('LBL_SELECT_PAGE'),
				"perform" => "sListView.setSelected('$this->list_id', 'page')",
			),
		);

        if ($this->show_select_all_selector) {
            $actions['all'] = array(
                "label" => translate('LBL_SELECT_ALL'),
                "perform" => "sListView.setSelected('$this->list_id', 'all')",
            );
        }

        $actions['none'] = array(
            "label" => translate('LBL_DESELECT_ALL'),
            "perform" => "sListView.setSelected('$this->list_id', 'none')",
        );

		$json = getJSONobj();
		$opts = array('keys' => array_keys($actions), 'values' => $actions, 'width' => '20em');
		$opts_json = $json->encode($opts);

		$ms_form = null;
		if($this->massupdate)
			$ms_form = $this->massupdate->getFormId();
		$ms_form = $ms_form ? '"' .javascript_escape($ms_form) . '"' : 'null';

		$this->addScriptLiteral("SUGAR.ui.registerSelectOptions('{$this->list_id}-MenuSelectors', $opts_json, $ms_form);", LOAD_PRIORITY_BODY);
		$menu_opts = array('options' => $this->list_id . "-MenuSelectors", 'target_offset' => array('left' => 3));
		$menu_opts_json = $json->encode($menu_opts);
		$this->addScriptLiteral("SUGAR.ui.registerMenuSource('{$this->list_id}-Selector', $menu_opts_json, $ms_form);");

		return $selector;
	}

	function formatActions($footer = false) {
		$acts = $toplevel = array();
		$module = $this->list_query->getModuleDir();

		foreach ($this->buttons as $k => $button) {
			$button->list_id = $this->list_id;
			if($button->acl) {
				$acl_mod = $button->acl_module ? $button->acl_module : $module;
				if(! ACLController::checkAccess($acl_mod, $button->acl))
					$button->hidden = $button->hidden_flag = true;
			}
			if(! $footer) {
				$button->loadSystemFlags();
				$button->translate($module);
			}

			if ($button->getVisible()) {
				$massp = $button->getParam('mass_perform');
				if($massp) {
					unset($button->params['mass_perform']);
					$button->perform = "sListView.sendMassUpdate('{LIST_ID}', '$massp');";
				}
				if($button->getParam('toplevel'))
					$toplevel[$k] = $button;
				else
					$acts[$k] = $button->getArray();
			}
		}

		$ret = '';
		$form = new EditableForm('SubPanel', 'SubPanel');
		$idx = 0;
		foreach ($toplevel as $name => $button) {
			$button->id = $this->list_id . '-' . $name . ($footer ? '2' : '');
			if($footer) $button->name .= '2';
			$button->class = 'input-button';
			if(! $idx) $button->class .= ' first';
			$ret .= '<div class="inline-elt">' . $form->renderFormButton($button) . '</div>';
			$idx ++;
		}
		if (count($acts)) {
			if (!$footer) {
				$w = 20;
				foreach ($acts as $act) {
					$m = array();
					if (preg_match('@^(\\d+)em$@', array_get_default($act, 'width', ''), $m)) {
						if ($m[1] > $w)
							$w = $m[1];
					}
				}
				$opts = array('keys' => array_keys($acts), 'values' => $acts, 'width' => $w . 'em');
				$json = getJSONobj();
				$opts_json = $json->encode($opts);
				$this->addScriptLiteral("SUGAR.ui.registerSelectOptions('{$this->list_id}-MenuActions', $opts_json);", LOAD_PRIORITY_BODY);
			}
			$which = $footer ? 'Foot' : 'Head';
			$ret .= "<div class=\"inline-elt\"><button id=\"{$this->list_id}-ActionButton{$which}\" class=\"input-button menu-source\">"
				. "<div class=\"input-arrow select-label\"><span class=\"input-label\">"
				. "<span class=\"uii uii-gear\"></span> "
				. translate('LBL_ACTIONS', 'app')
				. "</span></div></button></div>";
		}
		$init = $form->getExtraInit();
		if($init)
			$this->addScriptLiteral($init);

		return $ret;
	}


	function outputNavigation(ListResult &$listResult, $footer = false) {
		global $app_strings;

		if(! $this->show_navigation || $this->disable_query) return;
		if($footer && ($this->context == 'subpanel' || $this->context == 'dashlet' || $this->context == 'popup' || $this->context == 'unified_search')) return;

		$navStrings = array(
			'selected' => $app_strings['LBL_LISTVIEW_SELECTED_OBJECTS'],
			'next' => $app_strings['LNK_LIST_NEXT'],
			'previous' => $app_strings['LNK_LIST_PREVIOUS'],
			'last' => $app_strings['LNK_LIST_END'],
			'first' => $app_strings['LNK_LIST_START'],
			'of' => $app_strings['LBL_LIST_OF'],
			'none' => $app_strings['LBL_NO_RESULTS'],
			'updated' => $app_strings['LBL_UPDATED_RESULTS'],
			'result' => $app_strings['LBL_LIST_RESULT_ONE'],
			'results' => $app_strings['LBL_LIST_RESULT_MANY'],
			'page' => $app_strings['LBL_LIST_RESULT_PAGE'],
		);
		$parent_list_id = empty($this->listview_manager->parent_list_id) ? 'null' : "'{$this->listview_manager->parent_list_id}'";

		if($footer)
			echo "<div class=\"card-footer panel-footer listview-footer\">";
		else
			echo "<div class=\"card-header panel-subheader listview-header\">";
		echo "<div class=\"row listview-nav-block compact\">";
		echo "<div class=\"column listview-actions buttons-inline align-left\">";

		if($footer || $this->context != 'sidebar') {
			$acts = $this->formatActions($footer);
			echo $acts;
		}

		echo "</div><div class=\"column listview-status align-left\">";

		if ($this->show_checks && $this->have_rows) {
			if ($this->uids)
				$selc = ($this->uids == 'all') ? translate('LBL_SELECTED_ALL', 'app') : count($this->uids);
			else
				$selc = 0;
			echo '&nbsp;' . $navStrings['selected'];
			echo '<span id="' . $this->list_id . '-SelectCount' . ($footer ? 'Foot' : 'Head') . '" class="text-number">' . $selc . '</span>';

			if($this->nav_by_page && $listResult->total_counted) {
				echo ' ' . $navStrings['of'] . ' ';
				echo '<span class="text-number">' . format_number($listResult->total_count, 0) . '</span>';
			}
		}
		else if (!$this->have_rows && !$footer && !$this->nav_by_page) {
			echo '&nbsp;' . $navStrings['none'];
		}
		else if (($this->nav_by_page && $this->have_rows && ! $this->listview_primary)
			&& (! $listResult->total_counted || $listResult->total_count > $listResult->limit)) {

			$str = $navStrings[$listResult->total_count == 1 ? 'result' : 'results'];
			echo '<span class="text-number">' . str_replace('{COUNT}', $listResult->total_count, $str) . '</span>';
		}

		if (defined('ASYNC_ENTRY') && !empty($_REQUEST['list_id']) && !$footer && !empty($_REQUEST['query'])) {
			echo "<span id=\"{$this->list_id}-msgUpdated\" class=\"updated\" style=\"margin-left: 1.5em\">{$navStrings['updated']}</span>";
			$this->addScriptLiteral("SUGAR.util.fadeOutAfter('#{$this->list_id}-msgUpdated', 1000);");
		}

		echo '</div><div class="column listview-nav align-right">';

		if($footer && $this->context == 'sidebar' && ! $this->nav_by_page) {
			if(isset($listResult->limit)) {
				$c = $listResult->limit;
				$r = translate('LBL_ROWS_SHORT', 'app');
				echo '<div id="' . $this->list_id . '-sel-limit" class="input-field flat rbullet"><div class="input-label"><span id="' . $this->list_id . '-limit-label">' . $c . '</span> '. $r . '</div></div>&nbsp;';
			}
		}
		else if($footer && is_countable($listResult->rows) && count($listResult->rows) < 20) {
			//
		} else if (($this->show_compact_empty && !$this->have_rows && !$this->nav_by_page) || $this->hide_navigation_controls) {
			// skip
		} else if($this->nav_by_page) {
			$lim = $listResult->limit;
			$offs = $listResult->offset;
			$counted = $listResult->total_counted;
			$count = $listResult->total_count;
			$page = $lim ? ceil($offs / $lim) + 1 : 1;
			$show_before = 2;
			$show_after = $this->listview_primary ? 3 : 2;
			if(! $this->have_rows && ! $footer) {
				echo $navStrings['none'];
			}
			else if($offs == 0 && $counted && $count <= $lim && ! $this->listview_primary) {
				$str = $navStrings[$count == 1 ? 'result' : 'results'];
				echo '<span class="text-number">' . str_replace('{COUNT}', $count, $str) . '</span>';
			}
			else if(! $counted || $count > 0) {
				echo '<div class="input-button-group">';
				if($lim) {
					$start = max(1, $page - $show_before);
					if($start > 1) {
						$action = "onclick=\"sListView.navigate('$this->list_id', 0, null, null, $parent_list_id);\"";
						echo '<button type="button" class="input-button nav-button" ' . $action . '>';
						echo '<span class="uii uii-first"></span>';
						echo '</button>';
					}
					for(; $start < $page; $start ++) {
						$oprev = ($start - 1) * $lim;
						$action = "onclick=\"sListView.navigate('$this->list_id', {$oprev}, null, null, $parent_list_id);\"";
						echo '<button type="button" class="input-button nav-button" ' . $action . '>';
						echo '<span class="input-label single text-number">' . $start . '</span>';
						echo '</button>';
					}
				}
				if($counted && $lim) {
					$compact = $this->compact_pager;
					if($offs == 0 && $count <= $lim) {
						$str = $navStrings[$count == 1 ? 'result' : 'results'];
						$dd_lbl = str_replace('{COUNT}', $count, $str);
					} else {
						$dd_lbl = '';
						$action = "onclick=\"sListView.navigate('$this->list_id', {$offs}, null, null, $parent_list_id);\"";
						$str = str_replace('{COUNT}', $page, $navStrings['page']);
						echo '<button type="button" class="input-button nav-button nav-current" ' . $action . '>';
						echo '<span class="input-label single text-number">' . $str . '</span>';
						echo '</button>';
						for($p = 0; $p < $show_after; $p++) {
							$onext = ($page + $p) * $lim;
							if($onext < $count) {
								$action = "onclick=\"sListView.navigate('$this->list_id', {$onext}, null, null, $parent_list_id);\"";
								echo '<button type="button" class="input-button nav-button" ' . $action . '>';
								echo '<span class="input-label single text-number">' . ($page + $p + 1) . '</span>';
								echo '</button>';
							}
						}
						if(! $this->listview_primary && ($page + $p) * $lim >= $count)
							$compact = true;
					}
					if(! $compact) {
						$onclick = "onclick=\"sListView.showOffsetPopup('$this->list_id', this); SUGAR.ui.stopEvent(event);\"";
						echo "<button type=\"button\" class=\"input-button bar-insert nav-button\" $onclick>"
							. "<span class=\"input-label rbullet rbullet-ddown rbullet-inline text-number\">$dd_lbl</span></button>";
					}
				} else {
					// generic next button
					$onext = $offs + $listResult->result_count;
					$action = "onclick=\"sListView.navigate('$this->list_id', {$onext}, null, null, $parent_list_id);\"";
					echo '<button type="button" class="input-button nav-button" ' . $action . '>';
					echo '<span class="uii uii-next"></span>';
					echo '</button>';
				}
				echo '</div>';
			}
		} else {
			$buttons = array(
				'first' => 1, 'previous' => 1, 'position' => 1, 'next' => 1, 'last' => 1,
			);
			$offsets = array(
				'first' => 0, 'next' => 0, 'previous' => 0, 'last' => 0,
			);

			if ($listResult->offset == 0 || !isset($listResult->limit))
				$buttons['first'] = $buttons['previous'] = 0;
			if ($listResult->total_counted && isset($listResult->limit)) {
				if ($listResult->total_count - $listResult->limit <= $listResult->offset)
					$buttons['next'] = $buttons['last'] = 0;
				$offsets['previous'] = max(0, $listResult->offset - $listResult->limit);
				$last = $listResult->total_count % $listResult->limit;
				if (!$last) $last = $listResult->limit;
				$offsets['last'] = max(0, $listResult->total_count - $last);
				$offsets['next'] = min($listResult->offset + $listResult->result_count, $offsets['last']);
			}

			if($this->compact_pager) {
				unset($buttons['first']);
				unset($buttons['last']);
				//unset($buttons['position']);
			}
			if($this->hide_disabled_buttons) {
				$buttons = array_filter($buttons);
			}

			$bidx = -1;
			echo '<div class="input-button-group">';
			foreach ($buttons as $b => $enabled) {
				$bidx ++;
				if ($b == 'position') {
					if (!$this->have_rows || $this->context == 'sidebar') {
						continue;
					}
					$last = $listResult->offset + $listResult->result_count;
					$first = min($listResult->offset + 1, $last);
					$count = $listResult->total_counted ? $listResult->total_count : '?';
					$position = "$first&ndash;$last {$navStrings['of']} $count";
					if ($this->context == 'dashlet' || $this->context == 'subpanel' || $this->compact_pager) {
						$onclick = '';
						$cls = ' nohover';
					} else {
						$onclick = "onclick=\"sListView.showOffsetPopup('$this->list_id', this); SUGAR.ui.stopEvent(event);\"";
						$cls = '';
						$position = "<div class=\"rbullet rbullet-ddown rbullet-inline\">$position</div>";
					}
					echo "<button type=\"button\" class=\"input-button bar-insert nav-button $cls\" $onclick><span class=\"input-label single text-number\">$position</span></button>";
					continue;
				}
				$disabled = $enabled ? '' : ' disabled="disabled"';
				$action = $enabled ? "onclick=\"sListView.navigate('$this->list_id', {$offsets[$b]}, null, null, $parent_list_id);\"" : '';

				echo '<button type="button" title="' . to_html($navStrings[$b]) . '" class="input-button nav-button" ' . $action . $disabled . '>';
				echo '<span class="uii uii-' . $b . ($enabled ? '' : ' off') . '"></span>';
				echo '</button>';
			}
			echo '</div>';
		}

		echo '</div></div>';
		echo '</div>';
	}


	function outputCustomOffsetPopup() {
		global $app_strings;

		$offset_popup =
			'<div id="'.$this->list_id.'-offset-edit" style="display: none; width: 200px" class="card-outer dialog-outer popup-default flat-top listview-offset">'
			. '<div class="row form-row">'
			. '<div class="column form-cell"><div class="form-entry"><p class="form-label">' . $app_strings['LBL_LISTVIEW_OFFSET'] . '</p>'
			. '<div class="form-value"><input type="number" class="input-text" min="1" id="'.$this->list_id.'-offset-input" style="width: 6em"></div>'
			. '</div></div>'
			. '<div class="column form-cell"><div class="form-entry"><p class="form-label">' . $app_strings['LBL_LISTVIEW_COUNT'] . '</p>'
			. '<div class="form-value"><div class="input-field rbullet" id="'.$this->list_id.'-count-input"><span class="input-label select-label" id="'.$this->list_id.'-count-input-label"></span></div></div>'
			. '</div></div></div>'
			. '<div class="row form-row">'
			. '<div class="column form-cell"><div class="form-entry"><div class="form-value">'
			. '<button type="button" class="input-button" onclick="return sListView.loadCustomOffset(\'' . $this->list_id . '\');"><div class="input-icon left icon-accept"></div><span class="input-label">' . $app_strings['LBL_UPDATE'] . '</span></button>'
			. '</div></div></div></div></div>';

		echo $offset_popup;
	}

	function normalizeColumnWidths() {
		$perc_tot = 0;
		$fix_tot = 0;
		$is_perc = array();
		$fix_c = 0;
		$perc_c = 0;
		$widths = array();
		foreach ($this->columns as $k => &$col) {
			if (!empty($col['hidden']) || !empty($col['total']))
				continue;
			$width = array_get_default($col, 'width');
			if (!$width) {
				$width = self::$default_column_width;
				if (!empty($col['show_icon']))
					$width += 4;
			}
			if (($is_perc[$k] = (strpos($width, '%') !== false))) {
				$widths[$k] = $width = (int) $width;
				$perc_tot += $width;
				$perc_c++;
			} else {
				if (self::$max_fixed_column_width)
					$width = min($width, self::$max_fixed_column_width);
				$fix_tot += $width;
				$fix_c++;
			}
			$widths[$k] = $width;
		}
		if ($this->fill_width) {
			// adjust to percentages
			foreach ($widths as $k => $width) {
				if (!$is_perc[$k]) {
					$perc_remain = max(0, 100 - $perc_tot);
					$perc = $fix_tot ? $width * $perc_remain / $fix_tot : 0;
					$perc = max($perc, self::$default_column_perc_width);
				} else
					$perc = $width;
				$widths[$k] = round($perc) . '%';
			}
		}
		else if (!$this->fill_width && $perc_c) {
			// adjust to fixed widths
			if ($perc_tot) {
				$fix_add_tot = $fix_tot * 100 / (100 - $perc_tot);
			} else {
				$fix_add_tot = 0;
			}
			foreach ($this->widths as $k => $width) {
				if ($is_perc[$k]) {
					if ($width > 0 && $width < self::$default_column_perc_width) {
						// small column
						$width = min(1, round($width / 2));
					} else {
						$width = $perc_tot ? $fix_add_tot * $width / $perc_tot : 0;
						$width = max($width, self::$default_column_width);
					}
					$widths[$k] = round($width);
				}
			}
		}
		return $widths;
	}


	function outputHtmlRows(ListResult &$listResult, $offset_count = 0) {
		global $app_list_strings, $app_strings, $themeInstance;

		$ret = array();
		$ridx = 0;

		$row_act = array();
		foreach ($this->row_actions as $k => $v)
			$row_act[] = "$k=\"$v\"";
		$row_act = implode(' ', $row_act);

		if ($listResult->failed) {
			return;
		}

		$union_key = $listResult->union_key ? $listResult->union_key : ($listResult->is_union ? 'query_source' : null);
		$model_name = $listResult->base_model;
		$checked_rows = 0;
        $totals = array();
        $total_values = array();

		$tot_idx = 1;
		if($this->show_checks || $this->show_additional_details) $tot_idx++;
		if($this->show_favorites) $tot_idx ++;
		foreach($this->columns as $idx => &$col) {
			if (!empty($col['hidden']))
				continue;
			if (! empty($col['sum_total'])) {
				$totals[$tot_idx] = $col;
				$total_values[$col['field']] = 0;
				if(isset($col['base_field']))
					$total_values[$col['base_field']] = 0;
			}
			if (!empty($col['show_icon']))
				$tot_idx ++;
			$tot_idx ++;
		}

		$field_position = 0;
		foreach ($listResult->formatted_rows as $id => &$row) {
            $field_position = 1;
			if (!$id) $id = $ridx;
			$db_row = $listResult->getRow($id, false);
			$cls = 'listViewRow ' . (($offset_count + $ridx) % 2 ? 'evenListRowS1' : 'oddListRowS1');
			if ($ridx && $themeInstance->layout_version < 8) {
				echo '<tr><td class="listViewHRS1" colspan="' . $this->colspan . '"></td></tr>';
			}

			$special = array_get_default($listResult->special_rows, $id);
			if ($special == 'static')
				$cls .= ' staticRow';

			if ($this->show_checks) {
				if ($this->have_rows) {
					$checked = ($this->uids == 'all' || in_array($id, $this->uids)) ? ' checked="checked"' : '';
					$disabled = ($this->uids == 'all') ? ' disabled="disabled"' : '';
					if ($checked) {
						$cls .= ' markRow';
						$checked_rows++;
					}
					if(!$this->data_result) {
						$check_id = $id;
						if (!$listResult->use_key_index) {
							$check_id = array_get_default($db_row, 'id');
						}
					} else {
						$check_id = $listResult->offset + $id;
					}
					$checkVal = '<input onclick="sListView.checkItem(\'' . $this->list_id . '\', this)" type="checkbox" class="checkbox" name="mass[]"'
							. ' value="' . to_html($check_id) . '" autocomplete="off"' . $checked . '' . $disabled . '>';
					$checkVal = '<div class="input-check">' . $checkVal . '<label></label></div>';
				} else {
					$checkVal = '&nbsp;';
				}
			} else
				$checkVal = '';

			if($id && $id == $this->current_id)
				$cls .= ' currentRow';

			echo '<tr data-id="'.$id.'" class="' . $cls . '" ' . str_replace('{ID}', $id, $row_act) . '>';

			if ($this->show_additional_details) {
				if($union_key) {
					$detail_model = $listResult->sources[$row[$union_key]]['base_model'];
					if(! $this->checkAdditionalDetails($detail_model))
						$detail_model = null;
				} else
					$detail_model = $model_name;
				if($detail_model) {
					$detail_id = $db_row['id'];
					$row_id = $this->list_id . '_' . $detail_id;
					$detail_link = '<span id="adspan_' . $row_id . '" '
							. 'class="uii uii-info active-icon" '
							. 'onmousedown="SUGAR.ui.stopPropagation(event);" '
							. 'onmouseout="return SUGAR.util.clearAdditionalDetailsCall()" '
							. "onmouseover=\"return SUGAR.util.getAdditionalDetails('$detail_model', '$detail_id', 'adspan_{$row_id}', null, '{$this->context}')\" "
							. '></span>';

				} else
					$detail_link = '&nbsp;';
			} else
				$detail_link = '';

			if ($checkVal || $detail_link) {
				if ($checkVal && $checkVal != '&nbsp;' && $detail_link) $checkVal .= '&nbsp;';
				echo '<td class="listViewTd listViewMeta" width="1%">' . $checkVal . $detail_link . '</td>';
                $field_position++;
			}

            if ($this->show_favorites) {
                $favorite_link = $this->getFavoriteLink($id, array_get_default($row, 'favorite', 0));
                echo '<td class="listViewTd">'.$favorite_link.'</td>';
                $field_position++;
            }

			foreach ($this->columns as &$col) {
				if (!empty($col['hidden']) || !empty($col['total']))
					continue;
				$hide = false;
				if (isset($col['row_hidden'])) {
					if (eval_filter_flag(expand_filter_flag($col['row_hidden']), $db_row))
						$hide = true;
				}

				if ($hide) {
					$val = '';
				} else {
					$val = $this->formatRowColumn($col, $row, $listResult, $id);
					if (!empty($col['show_icon'])) {
						$icon = $this->getColumnIcon($col, $db_row);
						if(! $icon) $icon = '&nbsp;';
						echo '<td class="listViewIconTd" width="1%">' . $icon . '</td>';
						$field_position ++;
					}
				}
				$cls = 'listViewTd';
				$ctype = array_get_default($col, 'type');
				if($ctype == 'image') $cls .= ' listImageTd';
				$align = array_get_default($col, 'align');
				if($align == 'right' || $align == 'center' || $align == 'justify')
					$cls .= ' text-' . $align;
                echo '<td class="' . $cls . '">' . $val . '</td>';

                if (isset($totals[$field_position])) {
					$value = array_get_default($db_row, $col['field']);
					$total_values[$col['field']] += $value;

                    if(isset($col['base_field'])) {
                    	$value = array_get_default($db_row, $col['base_field']);
                    	$total_values[$col['base_field']] += $value;
                    }
                }

                $field_position ++;
			}

			if ($this->show_edit_view_links) {
				$edit_icon = get_image('edit_inline', '');
				$view_icon = get_image('view_inline', '');
				echo '<td class="listViewTd" nowrap="nowrap">';
				echo '<a title="' . translate('LBL_EDIT_BUTTON_LABEL') . '" href="index.php?action=EditView&amp;module=' . $this->list_query->getModuleDir() . '&amp;record=' . $id . '&amp;return_module=Home&amp;return_action=index">' . $edit_icon . '</a>';
				echo '&nbsp;<a title="' . translate('LBL_VIEW_BUTTON_LABEL') . '" href="index.php?action=DetailView&amp;module=' . $this->list_query->getModuleDir() . '&amp;record=' . $id . '&amp;return_module=Home&amp;return_action=index">' . $view_icon . '</a>';
				echo '</td>';
                $field_position++;
			}

			echo '</tr>';
			$ridx++;
		}

        if (sizeof($totals) > 0) {
            $empty_row = array();
            echo '<tr class="listHead">';

			$total_values['currency_id'] = '??';
            for ($pos = 1; $pos < $field_position; $pos++) {
                $val = '&nbsp;';
                $align_cls = '';
                if ($pos == 2 && ! isset($totals[2])) {
                    $val = $app_strings['LBL_LIST_TOTAL'];
                } elseif (isset($totals[$pos])) {
                    $val = $this->field_formatter->formatRowValue($totals[$pos], $total_values, $total_values[$totals[$pos]['field']]);
					$align = array_get_default($totals[$pos], 'align');
					if($align == 'right' || $align == 'center' || $align == 'justify')
						$align_cls .= ' text-' . $align;
                }
                echo "<th class=\"listViewThS1 $align_cls\">$val</th>";
            }

            echo '</tr>';
        }

		if ($this->show_checks && $this->have_rows) {
			$num_of_rows = is_countable($listResult->formatted_rows) ? sizeof($listResult->formatted_rows) : 0;
			if ($num_of_rows > 0 && $num_of_rows == $checked_rows)
				$checked = 1;
			else
				$checked = '';
			if ($this->uids == 'all') $checked = 'all';
			$this->addScriptLiteral("sListView.setCheckedAll('{$this->list_id}', '$checked');");
		}
	}

	function getColumnIcon($col, $row) {
		if (isset($col['icon']))
			$icon_name = $col['icon'];
		else if (isset($col['dynamic_module']))
			$icon_name = array_get_default($row, $col['dynamic_module']);
		else if ($col['type'] == 'module_name')
			$icon_name = array_get_default($row, $col['field']);
		else
			$icon_name = '';
		$chkval = isset($col['alias']) ? array_get_default($row, $col['alias']) : '';
		if ($icon_name && strlen($chkval ?? '')) {
			$mod_name = translate_dd('moduleListSingular', $icon_name, '');
			$icon = get_image($icon_name, 'title="' . to_html($mod_name) . '" alt="" border=0');
			return $icon;
		}
	}

    function getFavoriteLink($record_id, $favorite) {
        $favorite_link = '&nbsp;';

        if ($this->have_rows) {
            require_once('modules/Favorites/Favorite.php');
            $favorite_link = Favorite::get_star($this->list_query->getModuleDir(), $record_id, $this->list_id, $favorite);
        }

        return $favorite_link;
    }

	function loadRequestFilter($source = 'listview', $session_filter = null) {
		if(isset($session_filter))
			$this->filter_form->setSessionFilter($session_filter);
		$ret = $this->filter_form->loadRequestFilter($this->list_id, $source);
		$this->restrictFilterOptions();
		if(($ret == 'request' || $ret == 'session' || $ret == 'encoded') && $this->disable_query === 'need_input' && $this->report_run_method === 'interactive')
			$this->disable_query = false;
		if ($this->massupdate && $this->filter_form)
			$this->massupdate->loadFilterForm($this->filter_form);
		return $ret;
	}

	function loadFilter(array $input, $source=null, $input_prefix='', $defaults=null) {
		$ret = $this->filter_form->loadFilter($input, $source, $input_prefix, $defaults);
		if ($this->massupdate && $this->filter_form)
			$this->massupdate->loadFilterForm($this->filter_form);
		return $ret;
	}

	function loadFilterSpec($layout) {
		if(is_array($layout)) {
			$l2 = array();
			foreach($layout as $filt) {
				if(is_array($filt)) {
					if(! empty($filt['join_name']))
						$filt['link'] = $filt['join_name'];
					else if(! empty($filt['source'])) {
						$src = $filt['source'];
						if(isset($this->sources[$src]))
							$filt['link'] = $this->sources[$src]['link_name'];
					}

					if(! empty($filt['multiple']))
						$filt['multiple'] = $this->loadFilterSpec($filt['multiple']);
				}
				$l2[] = $filt;
			}
			$layout = $l2;
		}
		return $layout;
	}

	function loadFilterLayout($layout, $default=false) {
		$layout = $this->loadFilterSpec($layout);
		if($default)
			$this->filter_form->setDefaultLayout($layout);
		else
			$this->filter_form->loadFilterLayout($layout);
	}

	function restrictFilterOptions() {
		if($this->context == 'sidebar') {
			$opts =& $this->filter_form->filter_options;
			$found_text = false;
			if($opts) {
				foreach($opts as $k => &$opt) {
					if(isset($opt['name']) && $opt['name'] == 'filter_text')
						$found_text = true;
					else
						$opt['hidden'] = true;
				}
			}
			if($found_text)
				$this->show_filter = true;
		}
	}

	function getFilterSource() {
		return $this->filter_form->getFilterSource();
	}

	function getFilterArray($include_blank=false) {
		return $this->filter_form->getArray($include_blank);
	}

	function getFilterLayoutName() {
		return $this->filter_form->getFilterLayoutName();
	}

	function applyFilterClauses(ListQuery &$lq) {
		$clauses = $this->filter_form->getFilterClauses();
		$lq->addFilterClauses($clauses);
		$clauses = $this->getAdditionalClauses();
		if($clauses) $lq->addFilterClauses($clauses);
	}

	function getAdditionalClauses() {
		return $this->add_filter_clauses;
	}

	function addFilterClauses(array $cs) {
		array_extend($this->add_filter_clauses, $cs);
	}


	function getReportDataRows($offset=0, $limit=null, $order_by=null, $page_size=null, $limit_keys=null) {
		require_once('modules/Reports/utils.php');
		$result = load_report_result($this->data_result, $offset, $limit, $page_size, $limit_keys, false, $this->getOrderBy($order_by));
		return $result;
	}

	function getReportSummationRows($page_size=null) {
		require_once('modules/Reports/utils.php');
		$result = load_report_result_sums($this->data_result, $page_size);
		return $result;
	}


	function pageResult(ListResult &$result, $format = false) {
		if($result->result_source == 'report') {
			$ret = load_report_page($result);
		} else if($result->result_source == 'query') {
			$ret = $this->list_query->pageResult($result);
		}
		if($ret && $format) {
			$this->formatResult($result);
		}
		return $result;
	}

	function freeResult(ListResult $listResult) {
		$this->list_query->freeResult($listResult);
	}


	function getOrderBy($order_by=null) {
		if(! isset($order_by))
			$order_by = $this->sort_order;
		if($this->list_query)
			$order_by = $this->list_query->getOrderBy($order_by);
		else
			$order_by = ListQuery::parse_order_by($order_by);
		if(! $order_by)
			return null;

		if(is_array($order_by) && count($order_by) == 1) {
			$sort_field = $order_by[0]['field'];
			$sort_order = array_get_default($order_by[0], 'order', 'desc');

			// add column additional fields to order by
			foreach($this->columns as $col) {
				if (! empty($col['field']) && $col['field'] == $sort_field) {
					$add_fields = array_get_default($col, 'add_fields', array());

					foreach($add_fields as $subfield) {
						if(! empty($subfield['field']))
							$order_by[] = array('field' => $subfield['field'], 'order' => $sort_order);
					}
				}
			}
		}

		return $order_by;
	}


	/**
	 * @param int $offset
	 * @param int $limit
	 * @param string|array $order_by
	 * @return ListResult
	 */
	function &getQueryResult($offset = 0, $limit = null, $order_by = null, $page_size = null, $limit_keys = null) {
		if (!isset($limit))
			$limit = $this->getDefaultLimit();
		else if ($limit == '-1')
			$limit = null;

		if($this->data_result) {
			$this->last_result = $this->getReportDataRows($offset, $limit, $order_by, $page_size, $limit_keys);
		}
		else if($this->disable_query) {
			$r = new ListResult();
			$r->result_source = 'blank';
			$r->fields = array();
			$r->rows = array();
			$this->last_result = $r;
		}
		else {
			if ($this->apply_filter) {
				$this->applyFilterClauses($this->list_query);
			}
			if (isset($limit_keys)) {
				$this->list_query->addFilterPrimaryKey($limit_keys);
			}

			$this->list_query->addAclFilter('list');
			$order_by = $this->getOrderBy($order_by);

            if ($this->show_favorites)
                $this->addFavoritesToQuery();

			$this->list_query->unbuffered = $this->unbuffered;
			$this->last_result =& $this->list_query->runQuery($offset, $limit, true, $order_by, $page_size, $this->default_use_key_index);

			if($this->last_result && $this->report_run_method == 'interactive') {
				$this->last_result->grouping = $this->list_query->getSummationGroups();
			}
		}

		return $this->last_result;
	}

	function getSummationResult($page_size=null, $limit_keys=null) {
		if($this->data_result) {
			$this->last_result = $this->getReportSummationRows($page_size);
		}
		else if($this->disable_query) {
			$r = new ListResult();
			$r->result_source = 'blank';
			$r->fields = array();
			$r->rows = array();
			$this->last_result = $r;
		}
		else {
			if ($this->apply_filter) {
				$this->applyFilterClauses($this->list_query);
			}
			if (isset($limit_keys)) {
				$this->list_query->addFilterPrimaryKey($limit_keys);
			}

			$this->list_query->addAclFilter('list');
			$this->last_result =& $this->list_query->runSummationQuery($page_size);
		}

		return $this->last_result;
	}

    function addFavoritesToQuery() {
        require_once('modules/Favorites/Favorite.php');
        Favorite::add_join($this->list_query);
    }

    function formatResult(ListResult &$listResult) {
		if ($listResult->failed || !isset($listResult->rows))
			return $listResult;
		$this->formatResultRows($listResult);
	}

	function &getFormattedResult($offset = 0, $limit = null, $order_by = null, $page_size = null, $limit_keys = null) {
		$listResult =& $this->getQueryResult($offset, $limit, $order_by, $page_size, $limit_keys);
		$this->formatResult($listResult);
		return $listResult;
	}

	function getListViewManager() {
		return $this->listview_manager;
	}

	function exportIncludes() {
		global $pageInstance;
		$scripts = $this->getScripts();
		$count = count($scripts);
		$scripts = array_slice($scripts, $this->scripts_export_count);
		$this->scripts_export_count = $count;
		foreach($scripts as $scr) {
			if(! is_array($scr))
				continue;
			$prio = LOAD_PRIORITY_FOOT;
			if(isset($scr['priority']))
				$prio = $scr['priority'];
			if(isset($scr['file']))
				$pageInstance->add_js_include($scr['file'], null, $prio);
			else if(isset($scr['literal']))
				$pageInstance->add_js_literal($scr['literal'], null, $prio);
		}
	}

	/**
	 * Don't use key index for union queries
	 * because different modules might have the same ID values
	 */
	function unsetKeyIndexForUnion() {
		if ($this->getQuery()->isUnionQuery()) {
			$this->default_use_key_index = false;
		}
	}
}
?>
