<?php

require_once 'include/layout/widgets/DynamicListInput.php';
require_once 'include/layout/forms/EditableForm.php';
require_once 'modules/UserPreferences/UserPreference.php';

class ListLayoutArrange {
	private $mgr;
	private $perform;
	private $request = array();

	public function initFromManager(ListViewManager $mgr, $perform) {
		$this->mgr = $mgr;
		$this->perform = $perform;
	}

	public function loadRequest() {
		$this->request = $_REQUEST;
	}

	public function render() {
		if ($this->perform == 'arrange_save') {
			$this->save();
			return false;
		}
		else if ($this->perform == 'arrange_reset') {
			$this->reset();
			return false;
		}
		$this->renderEdit();
		return true;
	}

	private function save() {
		$json = getJSONobj();
		$data = $json->decode($this->request['tabs_data']);
		$pref = array();
		if (!empty($data)) {
			foreach ($data as $d)
				$pref[] = array($d['key'], array_get_default($d, 'visible', true));
		}
		UserPreference::setPreference('list_tabs_' . $this->mgr->module, $pref);

		// this is also called in sugar_cleanup(), but it will not work for demo sites,
		// because on demo sites userprefs are saved in session, and session is closed
		// defore sugar_cleanup() is called
		AppConfig::save_local('userpref'); 
	}
	
	private function reset() {
		UserPreference::setPreference('list_tabs_' . $this->mgr->module, null);
	}

	private function renderEdit() {
		$pref = UserPreference::getPreference('list_tabs_' . $this->mgr->module);
		if(! is_array($pref)) $pref = array();
		$tabs = $this->mgr->getLayoutTabs(true);
		$rows = array();
		foreach ($tabs as $tab) {
			if (!empty($tab['separator']))
				continue;
			if (!empty($tab['disable_select']))
				continue;
			$visible = true;
			foreach ($pref as $p) {
				if ($p[0] == $tab['name'])
					$visible = $p[1];
			}
			$rows[] = array('key' => $tab['name'], 'value' => $tab['label'], 'visible' => $visible);
		}

		$frm = new EditableForm('configform', 'ConfigForm');
		$frm->addHiddenFields(array(
			'module' => $this->mgr->module,
		));

		$attrs = array(
			'name' => 'tabs_data',
			'cols' => array(
				array(
					'name' => 'value',
					'label' => translate('LBL_ARRANGE_LAYOUTS_NAME', 'app'),
					'width' => '100%',
					'editable' => false,
					'multiline' => false,
				),
				array(
					'name' => 'visible',
					'label' => translate('LBL_ARRANGE_LAYOUTS_VISIBLE', 'app'),
					'width' => '1%',
					'format' => 'check',
				),
			),
			'depth_attrib' => '',
			'show_delete' => false,
		);
		$tabs_data = new DynamicListInput($attrs);
		$tabs_body = $tabs_data->render($frm, $rows);
		
		$save_spec = array(
			'name' => 'save_tabs_data',
			'label' => translate('LBL_SAVE_BUTTON_LABEL', 'app'),
			'icon' => 'input-icon icon-accept',
			'perform' => 'var ret = SUGAR.ui.sendForm(this.form, {list_layout_perform:"arrange_save"}); SUGAR.popups.close(); return ret;',
		);
		$save_btn = $frm->renderButton($save_spec);
		$reset_spec = array(
			'name' => 'reset_tabs_data',
			'label' => translate('LBL_RESET_BUTTON_LABEL', 'app'),
			'icon' => 'input-icon icon-delete',
			'perform' => 'var ret = SUGAR.ui.sendForm(this.form, {list_layout_perform:"arrange_reset"}); SUGAR.popups.close(); return ret;',
		);
		$reset_btn = $frm->renderButton($reset_spec);
		
		// ajrw - would be better to use a form generator here...
		
		echo $frm->open('card-outer form-outer default-form expand');
		echo '<div class="card-header form-buttons">';
		echo '<div class="row buttons-row"><div class="column buttons-inline">';
		echo '<div class="inline-elt">';
		echo $save_btn;
		echo '</div><div class="inline-elt">';
		echo $reset_btn;
		echo '</div>';
		echo '</div></div></div>';
		echo '<div class="card-body form-body">';
		echo '<div class="form-section">';
		echo '<div class="row form-row"><div class="column form-cell">';
		echo '<div class="form-entry"><div class="form-value">';
		echo $tabs_body;
		echo '</div></div></div></div></div></div>';
		$tabs_data->exportIncludes();
		echo $frm->close();
		$frm->exportIncludes();
		
		$form_lbl = translate('LBL_ARRANGE_LAYOUTS_FORM_TITLE', 'app');
		$form_icon = '<div class="input-icon icon-editlayout active"></div>&nbsp;';
		$GLOBALS['pageInstance']->add_js_literal(
			'SUGAR.popups.initContent({title: "'.addcslashes($form_icon.$form_lbl, '"').'", width: 500, fill: true});',
			null, LOAD_PRIORITY_FOOT);

	}
}
