<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/database/RowResult.php');
require_once('include/layout/forms/FormGenerator.php');
require_once('include/layout/forms/FilterForm.php');
require_once('include/ListView/ListFormatter.php');
require_once('modules/SavedSearch/SavedSearch.php');
require_once('modules/Reports/Report.php');

class ListLayoutEditor {
	var $form_name = 'LayoutEdit';
	var $list_id;
	var $form_id;
	var $module;
	var $action;
	var $model_name;
	var $list_query;
	var $layout;
	var $report_id;
	var $view_type;
	var $filter_name;
	var $filter_values = array();
	var $override_filters;
	var $columns;
	var $columns_ext;
	var $sources;
	var $sources_ext;
	var $primary_source;
	var $sort_order;
	var $add_hidden_fields;
	var $loaded;
	var $name;
	var $description;
	var $run_method;
	var $run_interval;
	var $run_interval_unit;
	var $next_run;
	var $assigned_user_id;
	var $shared_with;
	var $chart_name;
	var $chart_type;
	var $chart_options;
	var $chart_title;
	var $chart_rollover;
	var $chart_description;
	var $chart_series;
	var $export_ids;
	var $perform;
	var $from_saved_search_mod = false;
	var $disabledTabs = array();
	var $teams;
	var $max_keep;
	var $context;
	var $wf_module;
	var $layout_name;
	var $filter_form;
	var $tab;
	
	static $report_fields = array(
		'run_method', 'next_run', 'run_interval', 'run_interval_unit',
		'chart_type', 'chart_title', 'chart_series', 'chart_description',
		'chart_rollover', 'chart_options', 'export_ids', 'max_keep');
	static $chart_options_fs = array('exploded', '3d', 'percent', 'stacked');
	
	static $banned_join_fields = array(
		'date_entered', 'date_modified',
	);

	function __construct() {
		$this->columns = array();
		$this->columns_ext = array();
		$this->sources = array();
		$this->sources_ext = array();
	}
	
	function initFromManager(ListViewManager &$mgr, $perform) {
		$this->module = $mgr->module;
		$this->action = $mgr->action;
		$this->model_name = $mgr->model_name;
		$this->layout = $mgr->layout_name;
		if($this->layout == 'Reports')
			$this->report_id = $mgr->report_id;
		$this->filter_name = $mgr->getFilterName();
		$this->override_filters = $mgr->getOverrideFilters();
		$this->view_type = $mgr->view_type;
		$this->list_id = $mgr->list_id;
		$this->form_id = ($this->list_id ? $this->list_id . '-': '') . $this->form_name;
		$this->list_query = new ListQuery($this->model_name);
		$this->context = $mgr->context;
		$this->filter_form = new FilterForm($this->list_query, 
			$this->form_name, $this->form_id, 'fvalue_$UNIQ$',
			$this->context);
		$this->filter_form->show_text_filter = true;
		$this->filter_form->expand_select_filters = true;
		$this->perform = $perform;		
		$this->tab = array_get_default($_REQUEST, 'list_layout_tab', 'general');
	}
	
	function initFromModel($model, $list_id=null, $form_name=null, $context=null) {
		$this->module = AppConfig::module_for_model($model);
		$this->model_name = $model;
		if(isset($list_id))
			$this->list_id = $list_id;
		if(isset($form_name))
			$this->form_name = $form_name;
		$this->form_id = ($this->list_id ? $this->list_id . '-' : '') . $this->form_name;
		try {
			$this->list_query = new ListQuery($model);
		} catch(IAHModelError $e) {
			return false;
		}
		$this->context = (isset($context) ? $context : 'listview');
		$this->filter_form = new FilterForm($this->list_query,
			$this->form_name, $this->form_id, 'fvalue_$UNIQ$',
			$this->context);
		$this->filter_form->show_text_filter = true;
		$this->filter_form->expand_select_filters = true;
		return true;
	}
	
	function loadStandardLayout($name) {
		$index = ListViewManager::load_model_layouts($this->model_name);
		if(isset($index[$name])) {
			$view_name = $index[$name]['view_name'];
			$layout = AppConfig::setting("views.layout.{$this->module}.{$this->view_type}.{$view_name}");
			if($layout) {
				$this->loadColumns($layout['columns']);
				if(isset($index[$name]['label']))
					$this->name = translate($index[$name]['label'], $this->module);
				return true;
			}
		}
	}
	
	function loadLayoutDetails($layout, $class) {
		if($class == 'report')
			$this->report_id = array_get_default($layout, 'id');
		else
			$this->layout = array_get_default($layout, 'id');
		$this->name = array_get_default($layout, 'name');
		$this->description = array_get_default($layout, 'description');
        foreach(self::$report_fields as $k)
			if(isset($layout[$k]))
				$this->$k = $layout[$k];
		$this->assigned_user_id = array_get_default($layout, 'assigned_user_id');
		$this->shared_with = array_get_default($layout, 'shared_with');
		foreach(array('columns_spec', 'filters_spec', 'filter_values', 'sources_spec', 'sort_order') as $f)
			if(isset($layout[$f]) && is_string($layout[$f]))
				$layout[$f] = unserialize($layout[$f]);

		if($class == 'report') {
			$srcs = $layout['sources_spec'];
			if($srcs)
				$this->loadSources($srcs);
		}

		$cols = $layout['columns_spec'];
		if($cols)
			$this->loadColumns($cols);
		else
			$this->loadStandardLayout('Standard');

		$filt = array_get_default($layout, 'filters_spec');
		if(! $filt) $filt = array();
		$this->loadFilterOptions($filt, $class=='report');
		$fvals = array_get_default($layout, 'filter_values');
		if(! $fvals) $fvals = array();
		$this->loadFilterValues($fvals);

		$order = array_get_default($layout, 'sort_order');
		if(! $order) $order = array();
		$this->loadSortOrder($order);
	}
	
	function loadCustomLayout($id) {
		$layout = SavedSearch::load_search($this->module, $id, null, true);
		if($layout) {
			$this->loadLayoutDetails($layout->row, 'custom');
			$this->loadTeams($layout);
			return true;
		}
	}
	
	function loadReportLayout($id) {
		$result = Report::load_report($this->module, $id, null, true);
		if($result) {
			$layout = $result->row;
			if(! strlen($layout['sources_spec']) && isset($layout['sources'])) {
				require_once('modules/Reports/upgrade_util.php');
				$spec = upgrade_report_spec($layout);
				$layout = array_merge($layout, $spec);
			}
			$this->loadLayoutDetails($layout, 'report');
			$this->loadTeams($result);
			return true;
		}
	}
	
	function loadTeams(RowResult $result) {
		$teams = $result->getLinkResults('securitygroups', array('id', '_display'));
		$this->teams = array();
		if($teams && $teams->rows) {
			foreach($teams->rows as $team) {
				$team['module'] = 'SecurityGroups';
				$this->teams[] = $team;
			}
		}
	}
	
	function getPrimarySourceDef() {
		$src = array(
			'name' => 't0',
			'display' => 'primary',
			'bean_name' => $this->model_name,
			'module' => AppConfig::module_for_model($this->model_name),
			'required' => true,
		);
		return $src;
	}
	
	function initNewReport() {
		$blank = Report::blank_report();
		foreach($blank as $k => $v)
			$this->$k = $v;
		$loaded = $this->loadStandardLayout('Standard');
		if($loaded) {
			$this->filter_form->is_report = true;
			$this->loadRequestFilter('Browse');
		}
		$src = $this->getPrimarySourceDef();
		$this->loadSources(array($src));
		return true;
	}
	
	function loadLayout($reload=false) {
		if(! $this->loaded || $reload) {
			if(! $this->layout)
				return false;
			if($this->layout == 'Reports') {
				if($this->report_id)
					$this->loaded = $this->loadReportLayout($this->report_id);
				else
					$this->loaded = $this->initNewReport();
			}
			else if(preg_match('~^\w+$~', $this->layout)) {
				$this->loaded = $this->loadStandardLayout($this->layout);
				if($this->loaded) {
					$this->filter_form->is_custom = true;
					$this->loadRequestFilter($this->filter_name);
					if($this->override_filters)
						$this->filter_form->addOverrideFilters($this->override_filters);
				}
			} else
				$this->loaded = $this->loadCustomLayout($this->layout);
		}
		return $this->loaded;
	}
	
	function save($input=null) {
		if(! isset($input))
			$input =& $_POST;
		$id = array_get_default($input, 'layout');
		if($id == 'Reports') {
			$is_report = true;
			$id = array_get_default($input, 'report_id');
		} else
			$is_report = false;

		$vals = array();
		$fnames = array_merge(
			array('name', 'description', 'assigned_user_id', 'shared_with', 'filter_name', 'view_name'),
			self::$report_fields);
		foreach($fnames as $k)
			if(isset($input[$k]))
				$vals[$k] = $input[$k];
		$arrs = array();
		$json = getJSONobj();
		foreach(array('columns', 'filters', 'sources', 'sort_order') as $k) {
			if(isset($input[$k])) {
				if(is_string($input[$k])) {
					$val = $json->decode($input[$k]);
					if(! is_array($val))
						$val = array();
				}
				else if(is_array($input[$k]))
					$val = $input[$k];
				else
					continue;
				$arrs[$k == 'sort_order' ? $k : $k.'_spec'] = $val;
			}
		}
		if($is_report) {
			$chart_opts = $this->getChartOptions();
			foreach(self::$chart_options_fs as $f) {
				if(isset($input['chart_'.$f]))
					$chart_opts[$f] = $input['chart_'.$f];
			}
			$arrs['chart_options'] = $chart_opts;
			if(isset($arrs['sources_spec'])) {
				$this->cleanSourcesInput($arrs['sources_spec']);
				$this->loadSources($arrs['sources_spec']);
			}
		}
		if(isset($arrs['columns_spec']))
			self::clean_column_input($arrs['columns_spec'], $is_report);
		if(isset($arrs['filters_spec'])) {
			self::clean_filter_input($arrs['filters_spec']);
			$this->loadFilterOptions($arrs['filters_spec'], $is_report);
			$this->filter_form->loadFilter($input, 'editor', 'fvalue_');
			$arrs['filter_values'] = $this->filter_form->getArray();
			self::clean_filter_prefix($arrs['filters_spec']);
		}
		if(isset($arrs['sort_order']))
			$this->clean_sort_order_input($arrs['sort_order']);
		foreach($arrs as $k => $v)
			$vals[$k] = serialize($v);

		if($is_report)
			$ret = Report::save_report($this->module, $id, $vals);
		else
			$ret = SavedSearch::save_search($this->module, $id, $vals);
		
		if($ret) {
			$inp = array_get_default($input, 'teams');
			if(isset($inp)) {
				if(substr($inp, 0, 1) == '{') {
					$teams = $json->decode($inp);
					$teams_ids = array_column($teams, 'id');
				}
				else {
					$teams_ids = array_filter(explode('^,^', $inp));
				}
				if(isset($teams_ids)) {
					$ret->removeAllLinks('securitygroups');
					$ret->addUpdateLinks('securitygroups', $teams_ids);
				}
			}
			
			$ret = $ret->getPrimaryKeyValue();
		}
		return $ret;
	}
	
	
	function delete($input=null) {
		if(! isset($input))
			$input =& $_POST;
		$id = array_get_default($input, 'layout');
		$report_id = array_get_default($input, 'report_id');
		if($id == 'Reports')
			return Report::delete_report($this->module, $report_id);
		else
			return SavedSearch::delete_search($this->module, $id);
	}
	
	
	function render() {
		if($this->module == 'Reports') {
			return $this->renderModuleSelect();
		}
	
		$this->loadLayout();
		if($this->perform == 'create') {
			if($this->layout == 'Reports') {
				if($this->report_id) $this->perform = 'duplicate';
				$this->report_id = '';
			}
			else {
				if($this->layout) $this->perform = 'duplicate';
				$this->layout = '';
			}
			if($this->name) {
				if($this->layout == 'Reports')
					$this->name = '';
				else if(preg_match('~^(.*)\((\d+)\)\s*$~', $this->name, $m))
					$this->name = $m[1] . '(' . ($m[2] + 1) . ')';
				else
					$this->name .= ' (2)';
			}
		}
        if($this->perform == 'duplicate') {
			if($this->layout == 'Reports')
				$this->report_id = '';
			else
				$this->layout = '';
            $this->assigned_user_id = AppConfig::current_user_id();
            $this->shared_with = '';
        }
		$is_report = ($this->layout == 'Reports');
		$layout_model = $is_report ? 'Report' : 'SavedSearch';
		$model = new ModelDef($layout_model);
		
		if($is_report) {
			$name_lbl = 'LBL_REPORT_NAME';
			$form_lbl = $this->report_id ? 'LBL_EDIT_REPORT_FORM_TITLE' : 'LBL_CREATE_REPORT_BUTTON_LABEL';
		} else {
			$name_lbl = 'LBL_LAYOUT_NAME';
			$form_lbl = $this->layout ? 'LBL_EDIT_LAYOUT_FORM_TITLE' : 'LBL_CREATE_LAYOUT_FORM_TITLE';
		}
		
		$filt_srcs = array();
		
		$hooks = array(
			'onsubmit' => 'return false;',
		);
		$hidden_fields = array(
			'list_id' => $this->list_id,
			'module' => $this->module,
			'action' => $this->action,
			'layout' => $this->layout,
			'report_id' => $this->report_id,
            'from_saved_search' => $this->from_saved_search_mod
		);
		if(isset($this->add_hidden_fields))
			$hidden_fields = array_merge($hidden_fields, $this->add_hidden_fields);
		
		$le_fields = array(
			'name' => array(
				'vname' => $name_lbl,
				'type' => 'varchar',
				'width' => '30',
				'required' => true,
				'colspan' => 2,
			),
			'description' => array(
				'vname' => 'LBL_DESCRIPTION',
				'type' => 'text',
				'width' => 60,
				'height' => 3,
			),
			'filters' => array(
				'type' => 'varchar',
			),
			'columns' => array(
				'type' => 'varchar',
			),
			'add_column' => array(
				'type' => 'varchar',
			),
			'add_filter' => array(
				'type' => 'varchar',
			),
		);

		if($is_report) {
			// Values for numeric keys are expected to be strings.
			// Values for string keys are expected to be arrays.
			$report_add_fs = array(
				'name',
				'sources' => array(
					'type' => 'varchar',
				),
				'run_method',
				'next_run',
				'run_interval',
				'run_interval_unit',
				'chart_type',
				'chart_title',
				'chart_series',
				'chart_description',
				'chart_rollover',
				'chart_options',
				'export_ids',
				'max_keep',
			);
			foreach($report_add_fs as $k => $f) {
				if(is_int($k))
					$le_fields[$f] = $model->getDisplayField($f, true);
				else if(isset($le_fields[$k]))
					array_extend_recursive($le_fields[$k], (array)$f);
				else
					$le_fields[$k] = $f;
			}
			$chart_opts = array();
			foreach(self::$chart_options_fs as $f) {
				$chart_opts['chart_'.$f] = array(
					'name' => 'chart_'.$f,
					'type' => 'bool',
					'vname' => 'LBL_'.strtoupper($f),
					'vname_module' => 'Reports',
					'editable' => true,
				);
			}
			$model->addDisplayFieldDefinitions($chart_opts);
			array_extend($le_fields, $chart_opts);
		}
        $le_fields['shared_with'] = $model->getDisplayField('shared_with', true);
        $le_fields['assigned_user'] = $model->getDisplayField('assigned_user', true);
        

		if((! $this->layout || ($this->layout == 'Reports' && ! $this->report_id)) && ! $this->assigned_user_id)
			$this->assigned_user_id = AppConfig::current_user_id();
		if($GLOBALS['locale']->getUseRealNames())
			$assigned_user = get_assigned_user_name($this->assigned_user_id);
		else
			$assigned_user = get_user_name($this->assigned_user_id);


		$result = new RowResult;
		$result->base_model = $model->getModelName();
		$result->primary_key = 'id';
		$result->row = array(
			'id' => ($this->layout == 'Reports' ? $this->report_id : $this->layout),
			'name' => $this->name,
			'description' => $this->description,
			'assigned_user_id' => $this->assigned_user_id,
			'assigned_user' => $assigned_user,
			'shared_with' => $this->shared_with,
			'filters' => $this->encodeFilters(),
			'columns' => $this->encodeColumns($this->columns_ext),
			'sort_order' => $this->encodeSortOrder($this->sort_order),
			//'teams' => $this->encodeTeams(),
		);

		if($is_report) {
			$add_report_fs = array(
				'sources' => $this->encodeSources($this->sources_ext),
			);
			foreach(self::$report_fields as $k)
					$add_report_fs[$k] = $this->$k;
			array_extend($result->row, $add_report_fs);
			$opts = $this->getChartOptions();
			foreach(self::$chart_options_fs as $f) {
				$result->row['chart_'.$f] = $opts[$f];
			}
		}
		$fmt = new FieldFormatter('html', 'editview');
		$result->formatted = $fmt->formatRow($le_fields, $result->row);
		$result->fields = $le_fields;
        $result->filter_form = $this->filter_form;

		$layout = $this->getEditorFormLayout($is_report);

		$gen = FormGenerator::html_form($model, $layout, $this->form_name, $this->form_id);
		$gen->setDefaultEditable(true);
		$gen->getLayout()->setCurrentTab($this->tab);
		$gen->getLayout()->addFormHooks($hooks);
		$gen->getLayout()->addFormHiddenFields($hidden_fields);
		$cmds = array();
		foreach(array('add_column', 'add_total', 'add_filter', 'add_source', 'sort_order', 'add_sorted') as $f)
			$cmds[] = "SUGAR.ui.setFormInputProperty('$this->form_name', '$f', 'model', '$this->model_name');";
		$cmds[] = 'reparentLayoutButtons(this);';
		$gen->getLayout()->addFormInitHook($cmds);
		if($is_report) {
			$gen->getLayout()->addFormInitHook('updateChartSettings(this);');
			$gen->getLayout()->addJSLanguage('Reports');
		}
		$gen->renderForm($result);
		$body = $gen->getResult();
		$gen->exportIncludes();

		echo $body;
		$form_icon = '<div class="input-icon icon-editlayout active"></div>&nbsp;';
		$GLOBALS['pageInstance']->add_js_literal(
			'SUGAR.popups.initContent({title: "'.addcslashes($form_icon.translate($form_lbl), '"').'", width: 650, fill: true});',
			null, LOAD_PRIORITY_FOOT);
	}
	
	
	function renderModuleSelect() {
		$models = AppConfig::setting('model.index.reportable');
		$mods = array();
		foreach($models as $m) {
			$mod = AppConfig::module_for_model($m);
			$mods[$mod] = translate('LBL_MODULE_TITLE', $mod);
		}
		natsort($mods);
		$rows = ceil(count($mods) / 2);
		echo '<table border="0" cellpadding="5" cellspacing="0" class="tabForm" width="100%">';
		$ri = 0;
		$rc = 2;
		foreach($mods as $m => $mlbl) {
			if($ri == 0) echo '<tr>';
			$icon = "<div class=\"input-icon theme-icon module-$m\"></div>";
			$link = "<a href=\"#\" onclick=\"return sListView.showCreateReport('$m');\" class=\"tabDetailViewDFLink\">$icon&nbsp;" . to_html($mlbl) . '</a>';
			echo "<td class=\"dataLabel\">$link</td>";
			if(++ $ri >= $rc) {
				echo '</tr>';
				$ri = 0;
			}
		}
		if($ri) {
			while(++ $ri < $rc)
				echo '<td>&nbsp;</td>';
			echo '</tr>';
		}
		echo '</table>';

		$form_lbl = 'LBL_CREATE_REPORT_BUTTON_LABEL';
		$form_icon = '<div class="input-icon icon-editlayout active"></div>&nbsp;';
		$GLOBALS['pageInstance']->add_js_literal(
			'SUGAR.popups.initContent({title: "'.addcslashes($form_icon.translate($form_lbl), '"').'", width: 600});',
			null, LOAD_PRIORITY_FOOT);
	}
	
	
	function showSql($input=null) {
		if(! isset($input))
			$input =& $_POST;
		require_once('modules/Reports/utils.php');
		$report_id = array_get_default($input, 'report_id');
		$report = Report::load_report($this->module, $report_id, null, true);
		if(! $report || ! ($qs = get_report_queries($report))) {
			pr2(translate('LBL_ERROR_LOADING_REPORT', 'Reports'));
			return;
		}
		echo '<table class="tabForm" cellpadding=5 cellspacing=0 border=0 width="100%">';

		echo '<tr><td class="dataField"><p class="topLabel">' . to_html(translate('LBL_REPORT_SQL', 'Reports')) . '</p>';		
		echo '<div style="margin: 10px 2px; border: 1px solid #999; background: #fcfcfc">';
		echo '<div style="overflow: auto; font-family: monospace; white-space: pre; padding: 2px;">';
		$hl = SqlFormatter::format($qs['query'], true);
		$hl = preg_replace('~<pre\b.*?>~', '', $hl);
		echo $hl;
		echo '</div></div>';
		echo '</td></tr>';
		
		if(! empty($qs['sum'])) {
		echo '<tr><td class="dataField"><p class="topLabel">' . to_html(translate('LBL_REPORT_SUM_SQL', 'Reports')) . '</p>';
		echo '<div style="margin: 10px 2px; border: 1px solid #999; background: #fcfcfc">';
		echo '<div style="overflow: auto; font-family: monospace; white-space: pre; padding: 2px;">';
		$hl = SqlFormatter::format($qs['sum'], true);
		$hl = preg_replace('~<pre\b.*?>~', '', $hl);
		echo $hl;
		echo '</div></div>';
		echo '</td></tr>';			
		}
		
		echo '</table>';
		
		$form_lbl = $report->getField('name');
		$form_icon = '<div class="input-icon icon-editlayout active"></div>&nbsp;';
		$GLOBALS['pageInstance']->add_js_literal(
			'SUGAR.popups.initContent({title: "'.addcslashes($form_icon.translate($form_lbl), '"').'", width: 600});',
			null, LOAD_PRIORITY_FOOT);
	}
	

	function getEditorFormLayout($is_report) {	
		$layout_def = ConfigParser::load_file('modules/Reports/views/widget.Editor.php');
		$layout = $layout_def['layout'];

		if(! $is_report) {
			unset($layout['tabs'][4]);
			foreach($layout['sections'] as &$sect) {
				$sid = array_get_default($sect, 'id');
				if($sid == 'layout-general') {
					unset($sect['elements'][5]);
					unset($sect['elements'][6]);
					unset($sect['elements'][7]);
					unset($sect['elements'][8]);
					unset($sect['elements'][9]);
					if (! AppConfig::is_admin())
						unset($sect['elements'][1]);
				}
				if($sid == 'layout-columns')
					unset($sect['elements'][0]['show_action']);
				if($sid == 'layout-columns-buttons')
					unset($sect['elements'][1]);
				if($sid == 'layout-chart')
					$sect['hidden'] = true;
			}
		} else {
			 //unset($layout['sections'][0]['elements'][1]);
		}

		if (is_array($this->disabledTabs)) {
			foreach ($layout['tabs'] as $idx => $t) {
				if (in_array($t['name'], $this->disabledTabs))
					unset($layout['tabs'][$idx]);
			}
		}
		return $layout;
	}
	
	function getChartOptions() {
		$opts = null;
		if(isset($this->chart_options)) {
			if(is_string($this->chart_options)) $opts = unserialize($this->chart_options);
			else $opts = $this->chart_options;
		}
		if(! is_array($opts))
			$opts = array();
		foreach(self::$chart_options_fs as $f) {
			if(! isset($opts[$f])) $opts[$f] = 1;
		}
		return $opts;
	}
	
	
	function encodeSources($sources, $for_select=false) {
		$restrict = array(
			'name',
			'parent',
			'field_name',
			'module',
			'bean_name',
			'vname',
			'vname_module',
			'std_label',
			'label',
			'icon',
			'depth',
			'display',
		);
		if($for_select) {
			$ret = array();
			foreach($sources as $k => $s)
				foreach($s as $f => $v)
					if(in_array($f, $restrict))
						$ret[$k][$f] = $v;
			return $ret;
		}
		return $sources;
	}
	
	
	function encodeTeams() {
		$ret = $this->teams ? $this->teams : array();
		$ret = join('^,^', array_column($ret, 'id'));
		return $ret;
	}
	
	
	function encodeColumns($cols, $for_select=false) {
		$out = array();
		$this->translateColumns($cols, $for_select);
		$defs = array(
			'name' => '',
			'field' => '',
			'type' => '',
			'label' => '',
			'std_label' => '',
			'vname' => '',
			'vname_module' => '',
			'depth' => 0,
			'hidden' => 0,
			'grouped' => 0,
			'total' => null,
			'source' => null,
			'format' => null,
			'sort' => null,
			'list_format' => null,
			'list_position' => null,
			'dynamic_module' => null,
			'show_icon' => null,
			'join_name' => null,
		);
		foreach($cols as $idx => $c) {
			$row = array();
			foreach($defs as $k => $v)
				$row[$k] = array_get_default($c, $k, $v);
			
			$width = array_get_default($c, 'width', '');
			if(empty($row['depth']) && $width)
				$width = min($width, ListFormatter::$max_fixed_column_width);
			$row['width'] = $width;
			if($for_select && isset($c['bean_label']))
				$row['bean_label'] = $c['bean_label'];
			else if(isset($c['ref_label']))
				$row['ref_label'] = $c['ref_label'];
			$out[$idx] = $row;
		}
		return $out;
	}
	
	function loadRequestFilter($name) {
		$inp = array_get_default($_POST, 'list_filter');
		if($inp)
			$inp = getJSONobj()->decode($inp);
		if(! $inp) $inp = array();
		$this->filter_form->loadFilterLayout($name);
		$this->filter_form->loadFilter($inp);
		$this->filter_values = $this->filter_form->getArray();
	}
	
	function loadFilterSpec(&$filters) {
		foreach($filters as &$f) {
			if(isset($f['source'])) {
				if($f['source'] != $this->primary_source) {
					if(isset($this->sources[$f['source']]))
						$f['link'] = $this->sources[$f['source']]['link_name'];
					else
						$f['link'] = $f['source'];
				}
				unset($f['source']);
			}
			if(! empty($f['multiple']))
				$this->loadFilterSpec($f['multiple']);
		}
	}
	
	function loadFilterOptions(array $filters, $is_report=false) {
		$this->loadFilterSpec($filters);
		$this->filter_form->setIsReport($is_report, $is_report ? 'interactive' : null);
		$this->filter_form->loadFilterLayout($filters);
	}
	
	function loadFilterValues($vals) {
		if(! is_array($vals))
			$vals = array();
		$this->filter_form->loadArrayFilter($vals);
		array_extend($vals, $this->filter_form->getFilterValues());
		$this->filter_values = $vals;
	}

	function encodeFilters() {
		//$this->filter_form->loadArrayFilter($this->filter_values);
		$filters = $this->filter_form->getDisplayFilterOptions(false, true);
		foreach($filters as &$f) {
			if(! empty($f['link']) && ! empty($f['field'])) {
				$f['field'] = $f['link'].'.'.$f['field'];
				unset($f['link']);
			}
		}
		$render = $this->filter_form->formatFilters($filters, $this->filter_values,
			$this->form_name, $this->form_id, 'fvalue_$UNIQ$');
		foreach($render as &$filt) {
			// do reverse translation of link name to source name
			if(! empty($filt['source'])) {
				foreach($this->sources as $src) {
					if(isset($src['link_name']) && $src['link_name'] == $filt['source']) {
						$filt['source'] = $src['name'];
						break;
					}
				}
			}
		}
		$this->translateParentRefs($render);
		$ret = array();
		foreach($render as $f) {
			if(! empty($f['basic']))
				$f['fixed'] = true;
			$ret[] = $f;
		}
		return $ret;
	}

	
	function loadRequest() {
        if (isset($_REQUEST['from_saved_search']) && $_REQUEST['from_saved_search'])
            $this->from_saved_search_mod = true;
	}
	
	function loadColumns(array $cols) {
		$this->columns = $cols;
		$this->columns_ext = array();
		$this->expandColumns($this->columns, $this->columns_ext);
	}
	
	function getSourcePath($name) {
		$ret = null;
		foreach($this->sources_ext as $src) {
			if($src['name'] == $name) {
				$ret = $src['link_name'];
				break;
			}
		}
		return $ret;
	}
	
	function expandColumns($cols, &$ext, $depth=0, $root_link=null, $for_list=true, $for_update=false) {
		static $ban = array('add_filters', 'source');
		if(! isset($ext) || ! is_array($ext)) $ext = [];
		foreach($cols as $idx => $col) {
			if($this->list_query) {
				if(is_string($col))
					$col = array('field' => $col);
				
				$link_name = array_get_default($col, 'link_name');
				$source = array_get_default($col, 'source');
				$join_name = array_get_default($col, 'join_name');
				$path = $join_name ? $join_name : ($link_name ? $link_name : '');
				if(! $path && $source)
					$path = $this->getSourcePath($source);
				else if($root_link && $root_link != $link_name)
					$path = $path ? $root_link.'.'.$path : $root_link;
				if($path) $path .= '.';

				$field = array_get_default($col, 'field');
				if($field) {
					$falias = $this->list_query->addField($path.$col['field'], array_get_default($col, 'alias'));
					if(! $falias)
						continue;
					$fdef = $this->list_query->getField($falias);
					$fsrc = array_get_default($fdef, 'source');
					if($for_update && isset($fsrc['type']) && $fsrc['type'] == 'non-db')
						continue;
					if($path) $fdef['source'] = $source;
					else unset($fdef['source']);
					if($for_list) {
						$w = array_get_default($col, 'width');
						ListFormatter::expand_column_from_definition($col, $fdef);
						if(isset($w)) $col['width'] = $w;
						else unset($col['width']);
					} else {
						foreach($fdef as $dk => $dv) {
							if(! in_array($dk, $ban) && ! isset($col[$dk]))
								$col[$dk] = $dv;
						}
					}
				}

				$widget = array_get_default($col, 'widget');				
				if ($widget) {
					$wspec = AppConfig::get_widget_spec($widget);
					if($wspec && $wspec['type'] == 'column') {
						$wparams = array(
							'name' => $col['field'], 'list_id' => $this->list_id,
							'label' => array_get_default($col, 'vname'), 'width' => array_get_default($col, 'width'));
						$wclass = new $wspec['class']($wparams, $this->list_query->base_model);
						$col['widget'] = $wclass;
						$col['vname'] = $wclass->getLabel(array('edit_layout' => true));
						$col['width'] = $wclass->getWidth();
						$col['required'] = $wclass->getRequired();
					}
				}
			}
			if($depth)
				$col['depth'] = $depth;
			$ext[] = $col;
			if(! empty($col['add_fields']) && ! $depth)
				$this->expandColumns($col['add_fields'], $ext, $depth+1, $root_link, $for_list, $for_update);
		}
	}
	
	function loadSources(array $srcs) {
		$this->list_query->loadSources($srcs, $this->sources);
		$this->sources_ext = array();
		$this->expandSources($this->sources, $this->sources_ext);
		$this->primary_source = null;
		foreach($this->sources_ext as $src) {
			if($src['display'] == 'primary') {
				$this->primary_source = $src['name'];
				break;
			}
		}
	}
	
	function expandSources($srcs, &$ext, $for_select=false) {
		$seen = array();
		foreach($srcs as $idx => $src) {
			if(! $for_select && $src['display'] == 'primary') {
				$src['std_label'] = translate('LBL_MODULE_TITLE', $src['module']);
				if(! $for_select) {
					$src['depth'] = 0;
					$src['_fixed'] = 1;
				}
				$link_name = '';
			} else {
				$link_name = $src['field_name'];
				$parent = $src['parent'];
				if($parent) {
					if($for_select) {
						$link_name = $parent . '.' . $link_name;
					} else {
						if(! isset($seen[$parent]))
							continue;
						$link_name = $seen[$parent]['link'].$link_name;
						$src['depth'] = $seen[$parent]['depth'] + 1;
					}
				}
				$ldef = $this->list_query->getLink($link_name);
				if(! $ldef)
					continue;
				$src['module'] = $ldef->getModuleDir();
				if(isset($src['vname'])) {
					$vmod = array_get_default($src, 'vname_module');
					$src['std_label'] = translate($src['vname'], $vmod);
				} else {
					$src['std_label'] = translate('LBL_MODULE_TITLE', $src['module']);
				}
				$link_name .= '.';
			}
			$seen[$src['name']] = array('link' => $link_name, 'depth' => array_get_default($src, 'depth'));
			if(empty($src['label']))
				$src['label'] = $src['std_label'];
			$src['icon'] = 'theme-icon module-'.$src['module'];
			$ext[] = $src;
		}
	}
	
	static function clean_column_input(&$columns, $is_report=false) {
		$nc = array();
		$keep = array(
			'name', 'source', 'field', 'vname', 'vname_module',
			'label', 'width', 'hidden', 'total', 'format', 'grouped',
			'sort', 'list_format', 'list_position', 'show_icon', 'dynamic_module',
			'join_name',
		);
		foreach($columns as $c) {
			$c2 = array();
			foreach($keep as $k) {
				if(! empty($c[$k]) && ! is_array($c[$k]))
					$c2[$k] = $c[$k];
			}
			if(! $is_report)
				unset($c2['grouped']);
		
			$d = array_get_default($c, 'depth');
			if($d && count($nc))
				$nc[count($nc)-1]['add_fields'][] = $c2;
			else {
				if(! empty($c2['width']))
					$c2['width'] = min($c2['width'], ListFormatter::$max_fixed_column_width);
				$nc[] = $c2;
			}
		}
		$columns = $nc;
	}
	
	static function clean_filter_input(&$filters) {
		static $keep = array('name', 'field', 'source', 'label', 'hidden', '_iprefix');
		$nc = array();
		$ldepth = -1;
		$group_idx = array();
		$gidx = 0;
		$names = array();
		foreach($filters as $c) {
			$c2 = array();
			foreach($keep as $k) {
				if(! empty($c[$k]) && ! is_array($c[$k]))
					$c2[$k] = $c[$k];
			}
			$type = array_get_default($c, 'type');
			if($type == 'filtergroup') {
				$op = array_get_default($c, 'operator');
				if($op != 'and' && $op != 'or' && $op != 'not' && $op != 'not_and')
					$op = 'and';
				$c2['type'] = $type;
				$c2['operator'] = $op;
				$c2['multiple'] = array();
				$c2['name'] = 'group-'.($gidx ++);
			}
			else {
				if(isset($c2['name'])) {
					if(! empty($names[$c2['name']]))
						$c2['unique'] = $names[$c2['name']] ++;
					else
						$names[$c2['name']] = 1;
				}
			}
		
			$d = array_get_default($c, 'depth', 0);
			if($d > $ldepth + 1)
				$d = 0;
			while($ldepth > $d-1) {
				$ldepth --;
				array_pop($group_idx);
			}
			if($d) {
				$parent =& $nc[$group_idx[0]];
				for($i = 1; $i < $d; $i++)
					$parent =& $parent['multiple'][$group_idx[$i]];
				$idx = is_countable($parent['multiple']) ? count($parent['multiple']) : 0;
				$parent['multiple'][] = $c2;
			}
			else {
				$idx = count($nc);
				$nc[] = $c2;
			}
			if($type == 'filtergroup') {
				$ldepth = $d;
				array_push($group_idx, $idx);
			}
		}
		$filters = $nc;
	}
	
	static function clean_filter_prefix(array &$filters) {
		foreach($filters as &$f) {
			unset($f['_iprefix']);
			if(isset($f['multiple']) && is_array($f['multiple']))
				self::clean_filter_prefix($f['multiple']);
		}
	}
	
	function cleanSourcesInput(array &$sources) {
		static $keep = array('name', 'display', 'label', 'vname', 'vname_module', 'module', 'bean_name', 'field_name', 'parent', 'required');
		$primary = null;
		$links = array();
		foreach($sources as $s) {
			if(! is_array($s))
				continue;
			if(isset($s['label']) && isset($s['std_label']) && $s['label'] === $s['std_label'])
				unset($s['label']);
			$s2 = array();
			foreach($keep as $k) {
				if(! empty($s[$k]) && ! is_array($s[$k]))
					$s2[$k] = $s[$k];
			}
			if(! isset($s2['display']))
				$s2['display'] = 'joined';
			if(empty($s2['name']))
				continue;
			if($s2['display'] == 'primary')
				$primary = $s2;
			else
				$links[$s2['name']] = $s2;
		}
		if(! $primary) $primary = $this->getPrimarySourceDef();
		unset($primary['required']);
		foreach($links as &$src) {
			if(empty($src['parent']))
				$src['parent'] = $primary['name'];
		}
		$ret = array($primary['name'] => $primary);
		array_update($ret, $links);
		$sources = array();
		$this->list_query->loadSources($ret, $sources);
	}
	
	function translateColumns(array &$cols, $for_select=false) {
		$tl = $GLOBALS['app_list_strings']['total_type_short_dom'];
		if($for_select)
			$ml = $GLOBALS['app_list_strings']['moduleListSingular'];
		foreach($cols as $k => &$c) {
			$src = array_get_default($c, 'source');
			if($src && isset($this->sources[$src]))
				$vmod = $this->sources[$src]['module'];
			else
				$vmod = $this->module;
			$vmod = array_get_default($c, 'vname_module', $vmod);
			$def_name = array_get_default($c, 'field', '');
			if(isset($c['total']))
				$sfx_total = ': ' . array_get_default($tl, $c['total'], $c['total']);
			else
				$sfx_total = '';
			if(isset($c['vname']))
				$vlbl = translate($c['vname'], $vmod);
			else
				$vlbl = '';
			if(isset($c['std_vname']))
				$std_lbl = translate($c['std_vname'], $vmod);
			else
				$std_lbl = '';
			if(! isset($c['label']))
				$c['label'] = ($vlbl ? $vlbl : ($std_lbl ? $std_lbl : $def_name)) . $sfx_total;
			if($for_select && ! $std_lbl)
				$c['std_label'] = $c['label'];
			else
				$c['std_label'] = ($std_lbl ? $std_lbl : $vlbl) . $sfx_total;
			if($for_select && isset($c['bean_name'])) {
				$bmod = AppConfig::module_for_model($c['bean_name']);
				if($bmod && AppConfig::module_primary_bean($bmod) == $c['bean_name'])
					$c['bean_label'] = array_get_default($ml, $bmod, $bmod);
			}
		}
		if(! $for_select)
			$this->translateParentRefs($cols);
	}
	
	function translateParentRefs(array &$cols) {
		if($this->list_query) {
			foreach($cols as $k => &$c) {
				$field_name = array_get_default($c, 'field', '');
				$src = array_get_default($c, 'source');
				$link = array_get_default($c, 'link');
				if($link) $field_name = $link . '.' . $field_name;
				if( ($p = strrpos($field_name, '.')) !== false) {
					$path = substr($field_name, 0, $p);
					$field_name = substr($field_name, $p+1);
					// getFieldDefinition's second arg is by reference, so no need to initialize $err
					// Initialization is just to silence Intelephence
					$err = null;
					$f = $this->list_query->getFieldDefinition($path, $err);
					$lnk = $this->list_query->getLink($path);
					if($f && isset($f['vname'])) {
						if($lnk) $vmod = $lnk->getModuleDir();
						else $vmod = $this->module;
						$c['ref_label'] = translate($f['vname'], array_get_default($f, 'vname_module', $vmod));
					}
				}
			}
		}
	}
	
	function translateTypes(array &$cols) {
		$types = isset($GLOBALS['app_list_strings']['field_type_dom']) ? $GLOBALS['app_list_strings']['field_type_dom'] : array();
		foreach($cols as $k => &$c) {
			$t = array_get_default($c, 'type', '');
			if($t == 'ref' && ! empty($c['bean_label']))
				$c['type_label'] = $c['bean_label'];
			else
				$c['type_label'] = array_get_default($types, $t, '');
		}
	}
	
	function loadSortOrder($order) {
		$this->sort_order = $order;
	}
	
	static function clean_sort_order_input(array &$order) {
		static $keep = array('field', 'order', 'format', 'source');
		$ret = array();
		foreach($order as $ord) {
			if(! is_array($ord) || empty($ord['field']))
				continue;
			$ord2 = array();
			foreach($keep as $k) {
				if(! empty($ord[$k]) && ! is_array($ord[$k]))
					$ord2[$k] = $ord[$k];
			}
			if(array_get_default($ord2, 'order') != 'desc') $ord2['order'] = 'asc';
			$ret[] = $ord2;
		}
		$order = $ret;
	}
	
	function encodeSortOrder($sort_order) {
		$ret = array();
		if(! $sort_order) {
			// fetch default sort order
			$sort_order = $this->list_query->getOrderBy(null);
		}
		if(is_array($sort_order)) {
			foreach($sort_order as $ord) {
				if(! is_array($ord) || empty($ord['field'])) continue;
				$name = $ord['field'];
				$src = array_get_default($ord, 'source');
				if($src && isset($this->sources[$src])) {
					$vmod = $this->sources[$src]['module'];
					$link = $this->sources[$src]['link_name'];
				} else {
					$vmod = $this->module;
					$link = null;
				}
				$link = ($link ? $link . '.' : '');
				// getFieldDefinition's second arg is by reference, so no need to initialize $err
				// Initialization is just to silence Intelephence
				$err = null;
				$fdef = $this->list_query->getFieldDefinition($link.$name, $err);
				if($fdef) {
					foreach(array('vname', 'vname_module', 'type') as $k)
						if(isset($fdef[$k])) $ord[$k] = $fdef[$k];
				} else
					continue;
				if(empty($ord['order'])) $ord['order'] = 'asc';
				$ret[] = $ord;
			}
		}
		$this->translateColumns($ret);
		return $ret;
	}
	
	
	function sortColumns(array &$ret, $field='std_label') {
		$names = array_map('strtolower', array_column($ret, $field));
		array_multisort($names, $ret);
	}
	
	function getFieldsJson($link_name=null, $link_alias=null) {
		$cols = $this->list_query->getAvailableFields($link_name);
		$module = $this->list_query->getModuleDir($link_name);
		if(! $cols) return array();
		foreach($cols as $idx => $c) {
			$cols[$idx] = array('name' => $c, 'field' => $c, 'link_name' => $link_name, 'source' => $link_alias);
		}
		if ($link_name) {
			$join_name = $link_name . '~join';
			$join_fields = $this->list_query->getAvailableJoinFields($link_name);
			if ($join_fields) {
				foreach($join_fields as $c) {
					if (in_array($c, self::$banned_join_fields))
						continue;
					$cols[] = array('name' => $c, 'field' => $c, 'link_name' => $join_name, 'join_name' => $join_name, 'source' => $link_alias);
				}
			}
		}

		$this->expandColumns($cols, $extcols);
		foreach($extcols as $idx => &$c) {
			if(! array_get_default($c, 'reportable', true)) {
				unset($extcols[$idx]);
				continue;
			}
			if(array_get_default($c, 'type') == 'ref' && !empty($c['join_name'])) {
				unset($extcols[$idx]);
				continue;
			}
			if(! isset($c['vname_module']))
				$c['vname_module'] = $module;
		}
		$cols = $this->encodeColumns($extcols, true);
		$this->translateTypes($cols);
		$this->sortColumns($cols);
		return $cols;
	}
	
	function getFieldsForUpdate($link_name=null, $link_alias=null) {
		$cols = $this->list_query->getAvailableFields($link_name);
		if(! $cols) return array();
		foreach($cols as $idx => $c) {
			$cols[$idx] = array('name' => $c, 'field' => $c, 'link_name' => $link_name, 'source' => $link_alias);
		}
		if ($link_name) {
			$join_name = $link_name . '~join';
			$join_fields = $this->list_query->getAvailableJoinFields($link_name);
			if ($join_fields) {
				foreach($join_fields as $c) {
					if (in_array($c, self::$banned_join_fields))
						continue;
					$cols[] = array('name' => $c, 'field' => $c, 'link_name' => $join_name, 'join_name' => $join_name, 'source' => $link_alias);
				}
			}
		}
		$render = $this->formatFieldsForUpdate($cols, null, $link_name, true);
		return $render;
	}
	
	function formatFieldsForUpdate(array $cols, $values=null, $link_name=null, $edit=true, $for_list=false) {
		if($link_name) $this->list_query->addLink($link_name);
		$model = $this->list_query->getModelName($link_name);
		$module = AppConfig::module_for_model($model);
		$this->expandColumns($cols, $extcols, 0, $link_name, $for_list, true);
		$add_alts = ($this->context == 'workflow');
		if($add_alts) {
			require_once('modules/Workflow/procs/UpdateRecordProc.php');
			$wf_mod = $this->wf_module;
			if(! $wf_mod) $wf_mod = $module;
			$wf_proc = new UpdateRecordProc($wf_mod, []);
		}
		$ext_alts = [];
		foreach($extcols as $idx => &$c) {
			$ftype = array_get_default($c, 'type');
			if($ftype == 'ref' && !empty($c['join_name'])) {
				unset($extcols[$idx]);
				continue;
			}
			if(! isset($c['vname_module']))
				$c['vname_module'] = $module;
			if($add_alts) {
				$opts = $wf_proc->getRelatedFields($c);
				if($opts) {
					$c['alt_options'] = $opts;
					if(! isset($c['alt_name']))
						$c['alt_name'] = $c['name'] . '_alt';
				}
			}
		}
		$cols = $extcols;
		$this->translateColumns($cols);
		$this->translateTypes($cols);
		$this->sortColumns($cols, 'label');
		$frm = new EditableForm('editview', $this->form_name, $this->form_id, 'fvalue_$UNIQ$', $this->context);
		$render = $frm->formatFieldOptions($model, $cols, $values, $edit);
		return $render;
	}
	
	
	function getTotalsJson($link_name=null, $link_alias=null) {
		$cols = $this->list_query->getAvailableFields($link_name);
		$module = $this->list_query->getModuleDir($link_name);
		$prefix = $link_name ? $link_name . '.' : '';
		if(! $cols) return array();
		$totals = array();
		// getFieldDefinition's second arg is by reference, so no need to initialize $err
		// Initialization is just to silence Intelephence
		$err = null;
		if ($this->list_query->getFieldDefinition('id', $err)) {
			$cols[] = 'id';
		}

		foreach ($cols as &$c) {
			$c = array('field' => $c, 'is_join' => false);
		}
		unset($c);

		if ($link_name) {
			$join_name = $link_name . '~join';
			$join_fields = $this->list_query->getAvailableJoinFields($link_name);
			if ($join_fields) {
				foreach($join_fields as $c) {
					if (in_array($c, self::$banned_join_fields))
						continue;
					$cols[] = array('field' => $c, 'is_join' => true);
				}
			}
		}


		foreach($cols as $idx => $col) {
			$c = $col['field'];
			$pfx = empty($col['is_join']) ? $prefix : $link_name . '~join.';
			$def = $this->list_query->getFieldDefinition($pfx . $c, $err);
			if($def && (($src = $def['source']['type']) == 'db' || $src == 'custom_field' || $src == 'subselect')
			  && ! empty(ListQuery::$valid_totals[$def['type']])) {
				foreach(ListQuery::$valid_totals[$def['type']] as $ttype) {
					$total = array(
						'name' => $c.':'.$ttype,
						'field' => $c,
						'total' => $ttype,
						'link_name' => $link_name,
						'source' => $link_alias,
					);
					if (!empty($col['is_join']))
						$total['join_name'] = $link_name . '~join';
					$totals[] = $total;
				}
			}
		}
		$this->expandColumns($totals, $extcols);
		foreach($extcols as $idx => &$c) {
			if(! array_get_default($c, 'reportable', true) && $c['field'] != 'id') {
				unset($extcols[$idx]);
				continue;
			}
			if(! isset($c['vname_module']))
				$c['vname_module'] = $module;
		}
		$totals = $this->encodeColumns($extcols, true);
		$this->translateTypes($totals);
		$this->sortColumns($totals);
		return $totals;
	}
	
	
	function getFiltersJson($link_name=null, $link_alias=null) {
		$filts = $this->filter_form->getAvailableFilters($link_name, true);
		$module = $this->list_query->getModuleDir($link_name);
		if(! $filts) return array();
		foreach($filts as &$f) {
			if(! isset($f['vname_module']))
				$f['vname_module'] = $module;
			if($link_alias) {
				$f['full_link'] = $f['link'] ?? '';
				$f['link'] = $link_alias;
			}
			unset($f['source']);
			$f['tabindex'] = 1; // may want as an input param
		}
		$this->translateColumns($filts);
		$this->translateTypes($filts);
		$this->sortColumns($filts, 'label');
		$render = $this->filter_form->formatFilters($filts, $this->filter_values);
		return $render;
	}
	

	function getSourcesJson($link_name=null, $link_alias=null, $single=false) {
		$cols = $this->list_query->getAvailableLinks($link_name);
		$module = $this->list_query->getModuleDir($link_name);
		if(! $cols) return array();
		$depth = $link_name ? substr_count($link_name, '.') + 2 : 1;
		foreach($cols as $idx => &$col) {
			$type = array_get_default($col, 'type');
			if($type == 'ref' xor $single) {
				unset($cols[$idx]);
				continue;
			}
			$col['field_name'] = $col['name'];
			$col['parent'] = $link_name;
			$col['depth'] = $depth;
			$col['display'] = 'joined';
			if(! isset($col['vname_module']))
				$col['vname_module'] = $module;
		}
		$this->expandSources($cols, $extcols, true);
		foreach($cols as $idx => &$col) {
			$col['parent'] = $link_alias;
		}
		$cols = $this->encodeSources($extcols, true);
		$this->sortColumns($cols);
		return $cols;
	}

	function disableTabs($names)
	{
		$this->disabledTabs = $names;
	}
}
