<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/**
 * MassUpdate for ListViews
 *
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 */

define('MASSUPDATE_PAGE_SIZE', 200);

class ListMassUpdate {
	var $list_id;
	var $form_name;
	var $form_id;
	var $module;
	var $action;
    var $context;
	var $fields;
	var $handlers = array();
	var $handler_buttons = array();
	var $request;
	/**
	 * @var ModelDef
	 */
	var $model;
	var $filter_form;
	var $uids;
	var $add_hidden_fields;
	var $panel_open = false;
	var $field_prefix;
	var $subpanel_mgr;
	var $subpanel_name;
    var $subpanel_module;
	var $list_formatter;
	var $current_filter;

	var $no_mass_panel;
	var $no_mass_delete;
	var $no_mass_export;
	var $no_mass_print;
	var $mass_merge_duplicates;

	function __construct(&$listFmt) {
		$this->list_id = $listFmt->list_id;
		$this->module = $listFmt->module;
		$this->action = $listFmt->action;
        $this->context = $listFmt->context;
		$lq = $listFmt->getQuery();
		if ($lq)
			$this->model = $lq->base_model;
		$this->list_formatter = $listFmt;
		$this->current_filter = array();
		$this->setFormName();
		$this->setUids();
		$form = $listFmt->getFilterForm();
		if($form) $this->loadFilterForm($form);
		$this->loadHandlers();
	}

	function getFormName() {
		return $this->form_name;
	}
	function getFormId() {
		return $this->form_id;
	}
	function setFormName($name=null, $id=null) {
		if(! isset($name)) {
			$name = 'ListUpdate';
			$id = $this->list_id . '-' . $name;
			if($this->list_formatter && $this->list_formatter->context == 'subpanel' && $this->list_formatter->link_name)
				$name = 'PanelUpdate-' . $this->list_formatter->link_name;
		}
		$this->form_name = $name;
		if(! isset($id)) $id = $this->list_id . '-' . $name;
		$this->form_id = $id;
	}

	function loadFilterForm(FilterForm &$filter_form) {
		$this->current_filter = $filter_form->getArray(true);
	}

	function setUids() {
		$this->uids = '';
		$chk_list = array_get_default($_REQUEST, 'list_id');
		if($chk_list && $chk_list == $this->list_id) {
			$uids = array_get_default($_REQUEST, 'list_uids', '');
			$callback = function($x) { return $x || $x === "0"; };
			$uids = implode(';', array_unique(array_filter(explode(';', $uids), $callback)));
			$this->uids = $uids;
		}
	}
	function getUids() {
		return $this->uids;
	}

	function addHandler($spec, $category) {
		if(! is_array($spec) || ! isset($spec['name']))
			return false;
		$this->handlers[$category][$spec['name']] = $spec;
	}

	function loadHandlers() {
		require_once('modules/Workflow/WFProcessorBase.php');
		$acts = WFProcessorBase::get_action_options($this->module, true);
		foreach($acts as $id => $act) {
			if(isset($act['acl'])) {
				if(! ACLController::checkAccess($this->module, $act['acl']))
					continue;
			}
			$this->handlers['action']['wf_'.$id] = array(
				'name' => 'wf_'.$id,
				'perform' => $id,
				'class' => 'WFMassUpdate',
				'file' => 'modules/Workflow/WFMassUpdate.php',
			);
			$act['type'] = 'button';
			$act['perform'] = "sListView.showActionConfig('{LIST_ID}', '$id');";
			$this->handler_buttons['wf_'.$id] = $act;
		}
	}

	function addHiddenFields(array $fields) {
		foreach($fields as $k => $f)
			$this->add_hidden_fields[$k] = $f;
	}

	static function getHash() {
		global $current_user;
		if($current_user)
			$guid = $current_user->getUserPrivGuid();
		if(empty($guid))
			$guid = AppConfig::setting('config.unique_key');
		$guid .= session_id();
		return substr(md5($guid), 0, 8);
	}

	function checkHash($hash) {
		/* A hash value is added to each massupdate form, ensuring that:
		 * 1) the submitted form wasn't created for another user, or during another session.
		 * 2) an outside attacker can't create a form which would cause all records in
		 *    a module to be deleted or otherwise updated when submitted by a logged-in
		 *    user (presuming they cannot determine the hash value).
		 */
		return ($hash === self::getHash());
	}

	function setSubpanel($mgr, $name) {
		$this->subpanel_mgr = $mgr;
		$this->subpanel_name = $name;
        $visib = $mgr->getVisiblePanels();
        if (isset($visib[$name]))
            $this->subpanel_module = array_get_default($visib[$name], 'module');
	}

	function handleUpdate(ListFormatter &$listFmt, $no_edit_data=false) {
		require_once('include/formbase.php');
		global $current_user, $db, $locale;

		if(! isset($this->request))
			$this->request =& $_POST;

		$hash = array_get_default($this->request, 'mu_hash', '');
		if(! $this->checkHash($hash))
			return false;

		$perform = array_get_default($this->request, 'mass_perform', '');
		$export_formats = AppConfig::setting('locale.list_export_formats');

		if(! $perform)
			return false;

		set_time_limit(290); // 5 minutes minus 10 seconds
		$last_ignore = ignore_user_abort(true);

		if($this->uids === 'all')
			$uids = 'all';
		else if(strlen($this->uids))
			$uids = explode(';', $this->uids);
		else
			$uids = null;
		$update_uids = $uids;

		if($perform == 'delete' && ! $no_edit_data) {
			$lq = clone($listFmt->getQuery());
			$lq->clearFields();
			$lq->addColumnFields();
			$lq->addPrimaryKey();
			$lq->addFields(array('deleted', 'date_modified', 'modified_user_id'));
			$listFmt->applyFilterClauses($lq);
			$lq->addAclFilter('delete');
			$hook_fields = $this->model->getHookRequiredFields(['pre_delete', 'post_delete', 'before_delete', 'after_delete']);
			if($hook_fields)
				$lq->addFields($hook_fields);
			if($uids !== 'all')
				$lq->addFilterPrimaryKey($uids);

			$list_result = $lq->runQuery(0, null, null, null, MASSUPDATE_PAGE_SIZE);

			while(! $list_result->failed) {
				$upd = RowUpdate::blank_for_model($list_result->base_model);
				foreach($list_result->getRowIndexes() as $idx) {
					$row = $list_result->getRowResult($idx);
					if($upd->setOriginal($row)) {
						$upd->markDeleted(1);
					}
				}

				if($list_result->page_finished)
					break;
				$listFmt->pageResult($list_result, true);
            }
			// FIXME special handling for EmailMan - actually remove records from table
		}

		else if($perform == 'subpanel_remove' && $this->subpanel_mgr && $this->subpanel_name) {
			$pmod = $listFmt->module;
			$pmodel = AppConfig::module_primary_bean($pmod);
			$prec = $listFmt->getQuery()->parent_key;
			if($pmodel && $prec && ($presult = ListQuery::quick_fetch($pmodel, $prec))) {
				$lq = clone($listFmt->getQuery());
				$lq->addPrimaryKey();
				$lq->addAclFilter('list');
				if($uids !== 'all')
					$lq->addFilterPrimaryKey($uids);
				$list_result = $lq->runQuery(0, null, null, null, MASSUPDATE_PAGE_SIZE);

				while(! $list_result->failed) {
					foreach($list_result->getRowIndexes() as $idx) {
						$row = $list_result->getRowResult($idx);
						$link_id = $row->getField('id');
						if($link_id) {
							$this->subpanel_mgr->handlePanelRemove($this->subpanel_name, $link_id);
						}
					}

					if($list_result->page_finished)
						break;
					$listFmt->pageResult($list_result, true);
				}
			}
		}

		// need to run some actions by custom handlers:
		// move TakeFromGroup, ReturnToGroup to Email handler
		// move ContactSync to Contact handler
		// contact categories handled automatically, as a multienum field
		// QuickCampaign managed by custom handler
		// Team management might be a custom handler
		else if($perform == 'update' && ! $no_edit_data) {
			$mu_fields = $this->getUpdateFields($this->model->getFieldDefinitions());
            $update_uids = array();

            // RowUpdate needs to load request into $input
            // create a new listquery, add updated fields, primary ID,
            // assigned user / team depending on ACL,
            // plus any fields required by workflow hooks.

			require_once('include/database/RowUpdate.php');

			// temporary update object used to unformat the input
            $updTpl = RowUpdate::for_model($this->model);
            $updTpl->limitFields($mu_fields);
            $updTpl->loadRequest('', true, true);
			$inputParams = $updTpl->getInput(true);

			$limitParams = array_filter($inputParams, function($v, $k) use ($mu_fields) {
				return isset($mu_fields[$k]) &&  $mu_fields[$k]['type'] != 'multienum';
			}, ARRAY_FILTER_USE_BOTH);

			$multienumParams = array_filter($inputParams, function($v, $k) use ($mu_fields) {
				return isset($mu_fields[$k]) &&  $mu_fields[$k]['type'] == 'multienum';
			}, ARRAY_FILTER_USE_BOTH);

			$teamsList = array_filter(explode('^,^', array_get_default($_REQUEST, 'securitygroups', '')));

			// load update hooks
			$check_hooks = array_get_default($this->handlers, 'update_hook', array());
			$hooks = array();
			foreach($check_hooks as $h) {
				$handler_name = array_get_default($h, 'name');
				if(! $handler_name)
					continue;
				$handler_cls = array_get_default($h, 'class', $this->model->getModelName());
				if(isset($h['file']))
					require_once($h['file']);
				else
					require_bean($handler_cls);
				$method = array_get_default($h, 'class_function', 'listupdate_hook');
				if(method_exists($handler_cls, $method)) {
					$h['class'] = $handler_cls;
					$h['class_function'] = $method;
					$hooks[$handler_name] = $h;
				}
			}

            if(count($inputParams) || count($teamsList) || $hooks) {
                $q_fields = $mu_fields;

                // set up list query
                $lq = clone($listFmt->getQuery());
                $lq->clearFields();
                $lq->addColumnFields();

				$uq = $lq->isUnionQuery();

				if($uq) {
					$lq->addField('query_module');

					$listFmt->applyFilterClauses($lq);
					$lq->addAclFilter('edit');
				}

				$listFmt->applyFilterClauses($lq);
				$lq->addAclFilter('edit');
                $hook_fields = $this->model->getHookRequiredFields(array('pre_save', 'post_save'));
                if($hook_fields)
                    $lq->addFields($hook_fields);
                if($uids !== 'all')
                    $lq->addFilterPrimaryKey($uids);

                // execute query and update rows
                $list_result = $lq->runQuery(0, null, null, null, MASSUPDATE_PAGE_SIZE);

                while(! $list_result->failed) {
                    foreach($list_result->getRowIndexes() as $idx) {
                        $row = $list_result->getRowResult($idx);
						if($uq) {
							$q_module = $row->getField('query_module');
							$q_model = AppConfig::module_primary_bean($q_module);
							$upd = RowUpdate::for_model($q_model);
							$upd->setOriginal($row->getRow());
						} else {
							$upd = RowUpdate::for_result($row);
						}

						if(count($limitParams)) {
							$upd->set($limitParams);
							$perform_save = true;
						} else {
							$perform_save = false;
						}
						foreach ($multienumParams as $k => $v) {
							$mode = array_get_default($_REQUEST, $k . '-mode');
							$orig = array_filter(explode('^,^', $upd->getField($k, '', true)));
							$v = array_filter(explode('^,^', $v));
							switch ($mode) {
							case 'add':
							case 'remove':
								$v =
									$mode == 'add'
									? array_unique(array_merge($orig, $v))
									: array_diff($orig, $v)
									;
								// fall-thru to next CASE is intentional
							case 'replace':
								$v = join('^,^', $v);
								$upd->set($k, $v);
								$perform_save = true;
								break;
							}

						}
						foreach($hooks as $k => $h) {
							try {
								if(call_user_func(array($h['class'], $h['class_function']), $this, $k, $upd, $inputParams))
									$perform_save = true;
							} catch (IAHActionCompleted $e) {
								$perform_save = false;
							} catch (IAHActionAbort $e) {
								$perform_save = false;
								break;
							}
						}
						if ($perform_save) {
							$upd->save();
							$update_uids[] = $upd->getPrimaryKeyValue();
						}
						if(count($teamsList)) {
							// need to do sanity/security checks here?
							$mode = array_get_default($_REQUEST, 'securitygroups-mode', 'add');
							if($mode == 'add')
								$upd->addUpdateLinks('securitygroups', $teamsList, null);
							else if($mode == 'replace')
								$upd->replaceLinks('securitygroups', $teamsList, null);
							else if($mode == 'remove')
								$upd->removeLinks('securitygroups', $teamsList);
						}
                    }
					if($list_result->page_finished)
						break;
					$listFmt->pageResult($list_result, true);
				}
                // FIXME - add flash message for update results / validation errors
            }

        }
		else if ($perform == 'report_mass_action' && $listFmt->report_result) {

		}

		else if($perform == 'pdf' || $perform == 'html' || isset($export_formats[$perform])) {
			if($perform != 'pdf' && $perform != 'html' && ! $this->canExport($this->model->getModuleDir()))
				return;
        	require_once('include/ListView/ListViewExporter.php');
        	$export = new ListViewExporter($perform);
        	if($listFmt->data_result)
				$export->setDataResult($listFmt->data_result->row);
			else if($listFmt->report_result)
				$export->setReportResult($listFmt->report_result);
        	$export->outputFormatter($listFmt, $uids);
        	exit;
		} else {
			$update_uids = array();
			$lq = clone($listFmt->getQuery());
			$lq->clearFields();
			$lq->addColumnFields();

			$listFmt->applyFilterClauses($lq);
			$lq->addAclFilter('list');
			if (!$listFmt->report_result) {
				if($uids !== 'all')
					$lq->addFilterPrimaryKey($uids);

				// execute query and update rows
				$list_result = $lq->runQuery(0, null, null, null, MASSUPDATE_PAGE_SIZE);

				while(! $list_result->failed) {
					foreach($list_result->getRowIndexes() as $idx) {
						$row = $list_result->getRowResult($idx);
						$update_uids[] = $row->getField('id');
					}
					if($list_result->page_finished)
						break;
					$listFmt->pageResult($list_result, true);
				}
				if(! $update_uids) return;
			}
		}

		foreach(array_get_default($this->handlers, 'action', array()) as $k => $h) {
			$handler_name = array_get_default($h, 'name');
			if(! $handler_name || $perform != $handler_name)
				continue;
			$cls = array_get_default($h, 'class', $this->model->getModelName());
			if(isset($h['file']))
				require_once($h['file']);
			else
				require_bean($cls);
			$method = array_get_default($h, 'class_function', 'listupdate_perform');
			$acl = array_get_default($h, 'acl', 'list');
			if(method_exists($cls, $method)) {
				$first_loop = true;
				do {
					$loop = false;
					$uus = $update_uids;
					if ($listFmt->report_result) {
						$limit_keys = $uids;
						if (empty($uids) || $uids === 'all')
							$limit_keys = null;
						$lr = $listFmt->getQueryResult(0, null, null, MASSUPDATE_PAGE_SIZE, $limit_keys);
					} elseif(empty($h['no_query'])) {
						if($first_loop) {
							$req_fs = array_get_default($h, 'require_fields');
							$lq = clone($listFmt->getQuery());
							$lq->clearFields();
							$lq->addColumnFields();
							$lq->addPrimaryKey();
							if($req_fs)
								$lq->addFields($req_fs);
							$ulim = MASSUPDATE_PAGE_SIZE;
						}
						if($uids === 'all') {
							if($first_loop) {
								$listFmt->applyFilterClauses($lq);
								$lq->addAclFilter($acl);
							}
						}
						else {
							if($first_loop) {
								$filters = $lq->getFilterClauses();
								foreach (array_keys($filters) as $filter_name) {
									$lq->removeFilter($filter_name);
								}
							} else {
								$lq->removeFilter('primary');
							}
							if(count($uus) > MASSUPDATE_PAGE_SIZE) {
								$update_uids = array_slice($uus, MASSUPDATE_PAGE_SIZE);
								$uus = array_slice($uus, 0, MASSUPDATE_PAGE_SIZE);
								$loop = true;
							}
							$lq->addFilterPrimaryKey($uus);
							$ulim = null;
						}
						$lr = $lq->runQuery(0, null, null, null, $ulim);
					} else {
						$lr = null;
					}

					$hname = array_get_default($h, 'perform', $perform);
					$ret = call_user_func_array([$cls, $method], [$this, $hname, &$listFmt, &$lr, $uus, $no_edit_data]);
					if($ret)
						return $ret;
					$first_loop = false;
				}
				while($loop);
			}
		}
	}

	function getUpdateFields(array $list_fields, $display=false, $sort_order=null) {
		global $app_list_strings;
		$mu_fields = array();
		$can_edit_types = array(
			'enum', 'date', 'datetime', 'time', 'bool', 'ref', 'multienum', 'url',
			'name', 'char', 'varchar', 'int', 'number', 'float', 'double',
			'percentage', 'rate_percent', 'email', 'phone', 'status', 'progress', 'star_rating',
		);
        //Reassign Objects multiselect right after Assigned to
        if (! empty($sort_order) && is_array($sort_order))
            array_push($sort_order, 'reassign_objects');

        if(is_array($list_fields)) {
			foreach($list_fields as $k => $def) {
				if(empty($def['name']) || empty($def['massupdate']))
					continue;
				if(isset($def['editable']) && ! $def['editable'])
					continue;
				if(isset($def['updateable']) && ! $def['updateable'])
					continue;
				if($def['source']['type'] != 'db' && $def['source']['type'] != 'custom_field') {
					if(! empty($def['source']['field']))
						continue;
					if($def['type'] == 'multienum' && ! empty($def['multi_select_group']))
						;
					else if($def['type'] == 'ref') {
						if(isset($def['bean_name'])) {
							$bnm = AppConfig::module_for_model($def['bean_name']);
							if(! iml($bnm)) continue;
						}
					}
					else continue;
				}
				if(! in_array($def['type'], $can_edit_types))
					continue;

				if($def['type'] == 'ref' || isset($def['width'])) {
					$w = array_get_default($def, 'width');
					$def['width'] = $w ? min($w, 17) : 17;
				}
				$def['required'] = false;

				if ($def['type'] === 'multienum') {
					$def['mode_select'] = [
						'options' => $app_list_strings['multi_update_mode_dom'],
					];
				}

				$mu_fields[$k] = $def;
				if(! empty($def['format'])) {
					$def['hidden'] = true;
					$mu_fields[$k . '!' . $def['format']] = $def;
				}

				if($def['type'] == 'ref') {
					if(isset($list_fields[$def['id_name']])) {
						$mu_fields[$def['id_name']] = $list_fields[$def['id_name']];
						$mu_fields[$def['id_name']]['hidden'] = true;
					}
					if(isset($def['dynamic_module']) && isset($list_fields[$def['dynamic_module']])) {
						$mu_fields[$def['dynamic_module']] = $list_fields[$def['dynamic_module']];
						$mu_fields[$def['dynamic_module']]['hidden'] = true;
					}
				}
			}
		}

		if(is_array($sort_order)) {
			$ret = array();
			foreach($sort_order as $f) {
				if(isset($mu_fields[$f])) {
					$ret[$f] = $mu_fields[$f];
					unset($mu_fields[$f]);
				}
			}
            $mod = ($this->subpanel_module) ? $this->subpanel_module : $this->module;
			foreach($mu_fields as $f => $def) {
				$vmod = array_get_default($def, 'vname_module', $mod);
				$vnam = array_get_default($def, 'vname', $f);
				$mu_fields[$f]['label'] = translate($vnam, $vmod);
			}
			array_csort($mu_fields, 'label');
			$mu_fields = array_merge($ret, $mu_fields);
		}

		return $mu_fields;
	}

	function addSecurityGroups($model, &$fields) {
		global $app_list_strings;

		if(in_array($model, AppConfig::setting('model.index.securitygroups'))) {
			if(AppConfig::setting("model.links.$model.securitygroups")) {
				$fields['securitygroups'] = array(
					'vname' => 'LBL_SECURITYGROUPS',
					'type' => 'multienum',
					'options' => array(),
					'options_icon' => 'icon',
					'width' => '16em',
					'label_key' => '_display',
					'mode_select' => array(
						'options' => $app_list_strings['multi_update_mode_dom'],
					),
					'options_lookup' => [
						'bean_name' => 'SecurityGroup',
					],
				);
			}
		}
	}

	function canExport($module) {
		if(! AppConfig::setting('export.enabled')
		|| ! ACLController::checkAccess($module, 'export'))
			return false;
		if(AppConfig::setting('export.admin_only') && ! is_admin())
			return false;
		return true;
	}

	function getExportFormat() {
		global $locale;
		return $locale->getPrecedentPreference('default_export_format');
	}

	function getFormBody(&$list_result, $no_update=false, $no_edit_data=false) {
        global $app_strings;

		if($this->current_filter) {
			$json = getJSONobj();
			$enc_filter = $json->encode($this->current_filter);
		} else {
			$enc_filter = '';
		}

		$hooks = array(
			'onsubmit' => 'return false;',
		);

		$hidden_fields = array(
			'list_id' => $this->list_id,
            'module' => $this->module,
			'action' => $this->action,
			'list_offset' => $list_result->offset,
			'list_limit' => $list_result->limit,
            'list_order_by' => $list_result->order_by,
			'list_filter' => $enc_filter,
			'list_uids' => $this->uids,
			'mu_hash' => self::getHash(),
			'mass_panel_open' => $this->panel_open,
			'mass_perform' => '',
			'list_format' => 'html',
		);

        if ($this->context == 'dashlet') $hidden_fields['id'] = $this->list_id;

		if(isset($this->add_hidden_fields))
			$hidden_fields = array_merge($hidden_fields, $this->add_hidden_fields);

		$model_name = $this->model ? $this->model->getModelName() : null;
		$mu_fields = [];
		$buttons = [];

		if(! $no_update && $list_result->result_count && $model_name) {
			$model = $list_result->base_model;
			$mod_dir = AppConfig::module_for_model($model);

			$buttons = $this->getButtons($model_name, $mod_dir, $no_edit_data, $hidden_fields);
			$mu_fields = $this->getFields($model_name, $no_edit_data, array_keys($list_result->fields));
		}

		$mu_keys = [];

		foreach($mu_fields as $mk => $mf) {
			if( ($p = strpos($mk, '!')) !== false) {
				$mkn = substr($mk, 0, $p);
				if(isset($mu_fields[$mkn])) {
					$mk = $mkn;
					$mf = $mu_fields[$mkn];
				}
			}
			if(in_array($mk, $mu_keys)) continue;
			if(empty($mf['hidden'])) $mu_keys[] = $mk;
		}


		if(count($mu_keys)) {
			require_once('include/database/RowResult.php');
			require_once('include/layout/forms/FormGenerator.php');
			$result = new RowResult;
			$result->row = array();
			$result->fields = $mu_fields;
			$cols = min(max(3, count($mu_keys)), 4);

			$layout = array(
				'editable' => true,
				'indicate_required' => false,
				'form_hooks' => $hooks,
				'form_hidden_fields' => $hidden_fields,
				'form_buttons' => array(
					'update' => array(
						'list_id' => $this->list_id,
						'icon' => 'icon-accept',
						'vname' => 'LBL_UPDATE',
						'vname_module' => 'app',
						'params' => array(
							'mass_perform' => 'update',
						),
					),
				),
				'title' => $app_strings['LBL_MASS_UPDATE'],
				'type' => 'massupdate',
				'sections' => array(
					array(
						'label_position' => 'top',
						'columns' => $cols,
						'widths' => array(round(100 / $cols, 2) . '%'),
						'label_widths' => array('0%'),
						'elements' => $mu_keys,
					),
				),
			);

			$model = new ModelDef($this->model);
			$model->addDisplayFieldDefinitions($mu_fields);
			$gen = FormGenerator::html_form($model, $layout, $this->form_name, $this->form_id);
			$gen->setFormClass('card-outer panel-outer panel-border panel-system listview-massupdate');
			$gen->setFieldNamePrefix($this->field_prefix);
			$gen->getFormObject()->max_text_width = 25;
			$gen->renderForm($result, $model->module_dir);
			$body = $gen->getResult();
			$gen->exportIncludes();
		} else {
			require_once('include/layout/forms/EditableForm.php');
			$frm = new EditableForm('massupdate', $this->form_name, $this->form_id);
			$frm->setFormClass('listview-massupdate');
			$frm->setFieldNamePrefix($this->field_prefix);
			$frm->addHooks($hooks);
			$frm->addHiddenFields($hidden_fields);
			$body = $frm->open() . $frm->close();
		}

		return compact('buttons', 'body');
	}

	function getFields($model_name, $no_edit_data = false, $fields_order = null) {
		$fields = $this->model->getFieldDefinitions();
		$this->model->execHooks('mass_update_fields', array(&$fields), true);
		$mu_fields = $this->getUpdateFields($fields, true, $fields_order);
		$this->addSecurityGroups($model_name, $mu_fields);

		if ($model_name && $no_edit_data || !empty($this->no_mass_panel) || AppConfig::setting("display.list.{$model_name}.no_mass_panel")) {
			$mu_fields = [];
		}

		return $mu_fields;
	}

	function getButtons($model_name, $module_dir, $no_edit_data = false, $hidden_fields = []) {
		require_once('include/utils/basic_utils.php');
		$buttons = [];

		if(! $no_edit_data && empty($this->add_hidden_fields['subpanel']) && ACLController::checkAccess($module_dir, 'delete') && empty($hidden_fields['report_id'])) {
			if (empty($this->no_mass_delete) && !AppConfig::setting("display.list.{$model_name}.no_mass_delete")) {

				$buttons['delete'] = array(
					'vname' => 'LBL_DELETE',
					'vname_module' => 'app',
					'params' => array(
						'icon' => 'icon-delete',
						'pre_check' => "return sListView.checkSelected('{LIST_ID}', 'delete', true);",
						'confirm' => 'NTC_DELETE_CONFIRMATION_MULTIPLE',
						'mass_perform' => 'delete',
					),
				);

			}
		}

		if($this->canExport($module_dir)) {
			if (empty($this->no_mass_export) && !AppConfig::setting("display.list.{$model_name}.no_mass_export")) {

				$export_format = $this->getExportFormat();

				$buttons['export'] = array(
					'vname' => 'LBL_EXPORT',
					'vname_module' => 'app',
					'params' => array(
						'icon' => 'icon-export',
						'perform' => "sListView.sendMassUpdate('{LIST_ID}', '{$export_format}', 'export', null, true);",
					),
				);

			}
		}

		if(! $no_edit_data && empty($this->add_hidden_fields['subpanel']) && ACLController::checkAccess($module_dir, 'edit') && empty($hidden_fields['report_id'])) {
			if ( (! empty($this->mass_merge_duplicates) && $this->mass_merge_duplicates) || AppConfig::setting("display.list.{$model_name}.mass_merge_duplicates")) {

				$json = getJSONobj();
				$form_params = $json->encode(array('module' => 'MergeRecords', 'merge_module' => $module_dir));

				$buttons['duplicate_merge'] = array(
					'vname' => 'LBL_MERGE_DUPLICATES',
					'vname_module' => 'app',
					'params' => array(
						'icon' => 'icon-duplicate',
						'perform' => "sListView.sendMassUpdate('{LIST_ID}', 'duplicate_merge', false, {$form_params}, false);",
					),
				);

			}
		}

		if(! $no_edit_data && !empty($this->add_hidden_fields['subpanel'])  && empty($hidden_fields['report_id'])) {

			$sp_buttons = AppConfig::setting("display.list.{$model_name}.subpanel_mass_buttons");

			if (!empty($sp_buttons)) foreach ($sp_buttons as $k => $b) {
				if (!empty($b['parent_modules']) && !in_array($this->module, $b['parent_modules']))
					continue;
				$buttons[$k] = $b;
			}

		}

		if (empty($this->no_mass_print) && !AppConfig::setting("display.list.{$model_name}.no_mass_print")) {

			$buttons['pdf'] = array(
				'vname' => 'LBL_PDF_PDF',
				'vname_module' => 'app',
				'params' => array(
					'icon' => 'icon-print',
					'perform' => "sListView.sendMassUpdate('{LIST_ID}', 'pdf', '_blank', null, true);",
				),
			);

		}

		/** @var array $layout_buttons */
		$layout_buttons = AppConfig::setting("display.list.{$model_name}.buttons", array());
		array_extend($layout_buttons, $this->handler_buttons);

		foreach ($layout_buttons as $k => $b) {
			if (is_numeric($k)) $k = count($buttons);
			if(isset($b['acl']) && ! ACLController::checkAccess($module_dir, $b['acl']))
				continue;
			$buttons[$k] = $b;
		}

		$hook = AppConfig::setting("display.list.{$model_name}.buttons_hook");

		if ($hook) {
			call_hook_function($hook, null, array($this, &$buttons));
		}

		if (!empty($hidden_fields['report_id']) && !empty($hidden_fields['data_id'])) {
			$report_model = AppConfig::module_primary_bean($this->module);
			$hook = AppConfig::setting("display.list.{$report_model}.report_buttons_hook");
			if ($hook) {
				call_hook_function($hook, null, array($this, &$buttons, $hidden_fields['report_id'], $hidden_fields['data_id']));
			}
		}

		return $buttons;
	}
}
