<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/ListView/ListFormatter.php');
require_once('include/pdf/PDFManager.php');


class ListViewCSV {
	var $format;
	var $header;
	var $fields;
	var $charset;
	var $format_spec;
	var $fp;
	var $data_size;
	var $data;
	var $filename;	
	
	function initOutput(ListViewExporter &$exporter, $format) {
		global $locale;
		
		set_time_limit(0);
		$this->format = $format;
		
		$cols = $exporter->getColumns(true);
		$this->header = $this->fields = array();
		foreach($cols as $c) {
			$this->header[] = $c['title'];
			$this->fields[] = $c['id'];
		}
		$this->charset = strtoupper($locale->getExportCharset());
		
		$delimited = array(
			'csv' => array('delim' => ',', 'ext' => 'csv', 'mime' => 'text/csv'),
			'scsv' => array('delim' => ';', 'ext' => 'csv', 'mime' => 'text/csv'),
			'tsv' => array('delim' => "\t", 'ext' => 'tsv', 'mime' => 'text/tab-separated-values'),
			'xlscsv' => array('delim' => ',', 'ext' => 'csv', 'mime' => 'application/vnd.ms-excel', 'keep_newlines' => true),
		);
	
		if(! isset($delimited[$format]))
			throw new IAHInternalError('Unknown export method: '.$format);
		$this->format_spec = $delimited[$format];

		$this->fp = @tmpfile();
		if (!$this->fp)
			throw new IAHInternalError('Failed to open temporary file');
		$this->data_size = 0;
		encoding_filter::register();
		stream_filter_append($this->fp, 'encoding.'.$this->charset, STREAM_FILTER_WRITE);
		
		$this->filename = $exporter->getFilenameBase() . '.' . $delimited[$format]['ext'];
	}
	
	
	function formatRowColumns(ListResult $result) {
		$rows = $result->getRows(true);
		if($result->page_index <= 1) {
			yield $this->header;
		}

		foreach($rows as $r) {
			$r2 = array();
			foreach($this->fields as $k) {
				$r2[] = array_get_default($r, $k);
			}
			yield $r2;
		}
	}
	
	
	function outputResult(ListViewExporter &$exporter, ListResult &$result) {
		$newline = "\r\n";	
		$rows = $this->formatRowColumns($result);

			

		$keep_newlines = array_get_default($this->format_spec, 'keep_newlines', false);
		$delim = $this->format_spec['delim'];
		foreach ($rows as $row) {
				$row = array_map(function($v) use($keep_newlines) {
					if (!$keep_newlines) {
						return strtr($v, ["\r\n" => "", "\r" =>  " ", "\n" => " "]);
					} else {
						return strtr($v, ["\r\n" => "\r\n", "\r" =>  "\r\n", "\n" => "\r\n"]);
					}
				}, $row);
			$this->writeSeparatedLine($row, $delim);
		}


		/*

		$lines = make_separated_values($rows, $this->format_spec['delim'], $this->format_spec['replace'], $no_nl);

		foreach($lines as $line) {
			fwrite($this->fp, $line . $newline);
			// $this->data_size += strlen($line . $newline);
		}
		 */
	}

	private function writeSeparatedLine($row, $delim) {
		$row = array_map(function($v) use ($delim) {
			$quote = false;
			$translate = [];
			if (strpos($v, '"') !== false) {
				$quote = true;
				$translate['"'] = '""';
			}
			if (!$quote && strpos($v, $delim) !== false) {
				$quote = true;
			}
			if (!$quote && strpos($v, "\n") !== false) {
				$quote = true;
			}
			if (!$quote && strpos($v, "\r") !== false) {
				$quote = true;
			}
			if (!empty($translate)) {
				$v = strtr($v, $translate);
			}
			if ($quote) {
				$v = "\"$v\"";
			}
			return $v;
		}, $row);
		fwrite($this->fp, join($delim, $row));
		fwrite($this->fp, "\r\n");
	}
	
	
	function finishOutput(ListViewExporter &$exporter) {
		// prevent locking until user finishes downloading the file
		session_write_close();
		// keep running until complete
		set_time_limit(0);
		
		fflush($this->fp);
		$this->data_size = ftell($this->fp);

		
		end_clean_all_buffers(true);
		ini_set('zlib.output_compression', '0');
		header('Content-Encoding: none');
		header("Pragma: cache");
		header("Content-Type: {$this->format_spec['mime']}; charset={$this->charset}");
		header("Content-Disposition: attachment; filename=\"{$this->filename}\"");
		header("Content-Transfer-Encoding: binary");
		header("Expires: Mon, 26 Jul 1997 05:00:00 GMT" );
		header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT");
		header("Cache-Control: post-check=0, pre-check=0", false);
		header("Content-Length: ". $this->data_size);

		rewind($this->fp);
		fpassthru($this->fp);
		fclose($this->fp);

	}

    function &getOutput() {
		fflush($this->fp);
		$this->data_size = ftell($this->fp);

		rewind($this->fp);
        $data = fread($this->fp, $this->data_size);
		fclose($this->fp);

        $output = array(
            'format' => 'string',
            'filename' => $this->filename,
            'mimetype' => $this->format_spec['mime'],
            'disposition' => 'attachment',
            'data' => $data,
        );

        return $output;
    }

    function outputTotals($rows, $title) {
	}

}
?>
