<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


define('EXPORT_PAGE_SIZE', 300);

class ListViewExporter {
	var $format;
	var $result;
	var $sums;
	var $grouping;
	var $report;
	var $report_data;
	var $target;
	var $title;
	var $filename_base;
	var $columns;
	var $totals;
	var $id_columns;
	var $primary_source;
	var $primary_module;
	var $sources;
	var $pending_rows;

	var $depth;
	var $group_sums;
	var $group_sum_cols;
	var $group_levels;
	var $group_depth;
	var $group_rows;
	var $last_group;
	var $grand_total;
	
	
	function __construct($format) {
		$this->format = $format;
		$opts = AppConfig::setting('locale.list_export_formats');
		if($format == 'pdf') {
			require_once('include/ListView/ListViewPDF.php');
			$this->target = new ListViewPDF();
		}
        else if($format == 'html') {
            require_once('include/ListView/ListViewHTML.php');
            $this->target = new ListViewHTML();
        }
		else if($format == 'csv' || $format == 'tsv' || $format == 'scsv' || $format == 'xlscsv') {
			require_once('include/ListView/ListViewCSV.php');
			$this->target = new ListViewCSV();
		}
		else if(! empty($opts[$format]['class'])) {
			$path = array_get_default($opts['format'], 'path', 'include/ListView/' . $opts[$format]['class']);
			require_once($path);
			$this->target = new $opts[$format]['class'];
		}
        else
			throw new IAHInternalError("unsupported export format for listview");
	}
	
	function getFormat() {
		return $this->format;
	}
	
	function getGroupedColumns() {
		$this->flattenColumns($this->columns, $cols);
		$headers = array($cols);
		
		if($this->grouping) {
			$d = 1;
			foreach($this->grouping as $grp) {
				$grp_src = $grp['source'];
				if(! isset($this->sources[$grp_src]))
					$grp_src = $this->primary_source;
				$headers[$d] = array();
				$seen = array();
				foreach($headers[$d-1] as $idx => $hdr) {
					$srcname = array_get_default($hdr, 'source');
					if(! empty($hdr['grouped']) && $hdr['id'] == $grp['field'])
						continue;
					if(isset($this->sources[$srcname])) {
						$src = $this->sources[$srcname];
					}
					if($srcname == $grp_src) {
						$headers[$d][] = $hdr;
						$seen[] = $idx;
					}
				}
				$seen = array_reverse($seen);
				foreach($seen as $idx) unset($headers[$d-1][$idx]);
				$d ++;
			}
		}
		
		return $headers;
	}
	
	function getColumns($flat = false) {
		if($flat) {
			$this->flattenColumns($this->columns, $cols);
			if($this->report && ! empty($this->report['export_ids'])) {
				$this->addIdColumns($this->columns, $cols);
			}
			return $cols;
		}
		return $this->columns;
	}
	
	function flattenColumns(array $cols, &$ret, $def_width=null, $depth=0) {
		if(! isset($ret)) $ret = array();
		foreach($cols as $c) {
			if(isset($c['add_fields']) && is_array($c['add_fields'])) {
				$add = $c['add_fields'];
				unset($c['add_fields']);
			} else
				$add = null;
			if(! isset($c['width']))
				$c['width'] = $def_width;
			if($add) {
				$preadd = array();
				foreach($add as $k2 => $c2) {
					if(array_get_default($c2, 'list_position') == 'prefix') {
						if(! isset($c2['width'])) $c2['width'] = 10; // probably a short value
						$preadd[] = $c2;
						unset($add[$k2]);
					}
				}
				if($preadd)
					$this->flattenColumns($preadd, $ret, $c['width'], $depth+1);
			}
			if(! empty($c['field'])) {
				$ret[] = $c;
			} elseif ($c['title'] == translate('LBL_EMPTY_COLUMN', 'app')){
				$ret[] = $c;
			}
			if($add)
				$this->flattenColumns($add, $ret, $c['width'], $depth+1);
		}
	}
	
	function findFields($columns, $type) {
		$ret = array();
		foreach($columns as $col) {
			if(! empty($col['type']) && $col['type'] == $type)
				$ret[] = $col;
			if(isset($col['add_fields']))
				$ret = array_merge($ret, $this->findFields($col['add_fields'], $type));
		}
		return $ret;
	}
	
	function addIdColumns($orig_cols, &$columns) {
		$refs = $this->findFields($orig_cols, 'ref');
		$add = array();
		foreach($this->id_columns as $id) {
			if(empty($id['hidden'])) continue;
			$id['hidden'] = false;
			if(empty($id['for_ref'])) {
				$id['label'] = translate('LBL_ID', 'app');
				$add[] = $id;
			} else {
				foreach($refs as $ref) {
					if($ref['field'] == $id['for_ref']) {
						$id['label'] = $ref['title'] . ' ' . translate('LBL_ID', 'app');
						$add[] = $id;
					}
				}
			}
		}
		if($add) {
			$newadd = $newtots = array();
			$this->formatColumns($add, $newadd, $newtots);
			array_extend($columns, $newadd);
		}
	}
	
	function getTotals() {
		return $this->totals;
	}
	
	function getTitle() {
		return $this->title;
	}
	
	function getFilenameBase() {
		return $this->filename_base;
	}
	
	
	function outputFormatter(ListFormatter &$listFmt, $uids, $download = true) {
		$this->primary_module = $listFmt->module;
		$listFmt->setFormat($this->format, 'export');
		$this->fetchFormatterSources($listFmt);
		$this->fetchFormatterColumns($listFmt);
		
		if($uids === 'all') $uids = null;
		
		if($this->totals) {
			$this->sums = $listFmt->getSummationResult(EXPORT_PAGE_SIZE, $uids);
		}

		$listFmt->default_use_key_index = false;
		$listFmt->unbuffered = true;
		$this->result = $listFmt->getFormattedResult(0, -1, null, EXPORT_PAGE_SIZE, $uids);
		$this->grouping = $this->result->grouping;
		$this->group_levels = is_array($this->grouping) ? count($this->grouping) : 0;

		$this->processSumRows();
		
		$this->initTitle();
		$this->initFilenameBase();

		$this->initOutput();
		
		while(! $this->result->failed) {
			$this->outputResult();
		
			if($this->result->page_finished)
				break;
			$listFmt->pageResult($this->result, true);
		}

		$listFmt->freeResult($this->result);
		
		$this->outputTotals();

        if ($download) {
		    $this->finishOutput();
            return true;
        } else {
            return $this->getOutput();
        }
	}
	
	
	function initOutput() {
		$this->depth = 0;
		$this->group_depth = 0;
		$this->pending_rows = array();
		$this->group_rows = array();
		$this->last_group = array();

		$this->target->initOutput($this, $this->format);
	}
	
	function outputTotals() {
		if($this->grand_total !== null) {
			$total_rows = array($this->grand_total);
			$this->target->outputTotals($total_rows, translate('LBL_PDF_GRAND_TOTALS', 'Reports'));
		}
	}
	
	function finishOutput() {
		$this->target->finishOutput($this);
	}
	
    function getOutput() {
        return $this->target->getOutput();
    }

	function outputResult() {
		if(method_exists($this->target, 'outputResult'))
			return $this->target->outputResult($this, $this->result);

		$depth_change = 0;
	
		$rows =& $this->result->rows;
		if (is_array($rows))
			$rows = array_values($rows);
		$formatted_rows = $this->result->formatted_rows;
		if (is_array($formatted_rows))
			$formatted_rows = array_values($formatted_rows);

		$finished = $this->result->page_finished;
		$i = 0;
		$nrows = is_countable($rows) ? count($rows) : 0;
		reset($rows);
		
		$grouping = array();
		if($this->grouping) {
			foreach($this->grouping as $grp)
				$grouping[] = $grp['field'];
		}
		
		while ($i < $nrows) {
			$row = isset($rows[$i]) ? $rows[$i] : [];
			$frow = isset($formatted_rows[$i]) ? $formatted_rows[$i] : [];
			while ($this->depth < $this->group_levels) {
				if(is_countable($this->group_rows) && count($this->group_rows) && $this->group_depth != $this->depth) {
					$this->target->outputGroupRows($this->group_depth, $this->group_rows, $grouping, $this->group_sum_cols[$this->group_depth]);
					$this->group_rows = array();
				}
				$field = $grouping[$this->depth];
				$this->last_group[$field] = $row[$field];
				$key = implode("\t", $this->last_group);
	
				if(isset($this->group_sums[$this->depth+1][$key])) {
					$frow = array_merge($frow, $this->group_sums[$this->depth+1][$key]);
				}
				$this->group_rows[] = $frow;
				$this->group_depth = $this->depth;
				$this->depth++;
			}
			$depth_change = 0;
			while (!$depth_change && $i < $nrows) {
				$key = implode("\t", $this->last_group);
				if(isset($this->group_sums[$this->depth][$key])) {
					$frow = array_merge($frow, $this->group_sums[$this->depth][$key]);
				}
				$this->pending_rows[] = $frow;
				if(++$i >= $nrows)
					break;
				$row = isset($rows[$i]) ? $rows[$i] : [];
				$frow = isset($formatted_rows[$i]) ? $formatted_rows[$i] : [];
				$depth_change = 0;
				$ngroups = is_countable($this->last_group) ? count($this->last_group) : 0;
				$j = 0;
				foreach($this->last_group as $k => $v) {
					if(isset($row[$k]) && $row[$k] != $v) {
						array_pop($this->last_group);
						$depth_change = $j - $ngroups;
						break;
					}
					$j++;
				}
			}
			if(($depth_change || $finished) && is_countable($this->pending_rows) && count($this->pending_rows)) {
				if($this->group_rows) {
					$this->target->outputGroupRows($this->group_depth, $this->group_rows, $grouping, $this->group_sum_cols[$this->group_depth]);
					$this->group_rows = array();
				}
				$this->target->outputRows($this->depth, $this->pending_rows);
				$this->pending_rows = array();
			}
			$this->depth += $depth_change;
		}
		
	}
	
	
	function formatColumns(array $input, &$cols, &$tots) {
		foreach($input as $def) {
			if(! empty($def['hidden']) || ! empty($def['no_query']))
				continue;
			if(isset($def['exportable']) && ! $def['exportable'])
				continue;
			if(! empty($def['label']))
				$title = $def['label'];
			else if(! empty($def['vname'])) {
				$vmod = array_get_default($def, 'vname_module', $this->primary_module);
				$title = translate($def['vname'], $vmod);
			}
			else
				continue;
			$hdr = array('title' => $title);
			$type = array_get_default($def, 'type', 'varchar');
			if($type == 'float' || $type == 'double' || $type == 'currency' || $type == 'base_currency')
				$hdr['is_float'] = true;
			$hdr['type'] = $type;
			$w = array_get_default($def, 'width', '');
			if($w)
				$hdr['width'] = "$w%";
			if(isset($def['field'])) {
				$hdr['field'] = $def['field'];
				$hdr['id'] = $def['field'];
			}
			if(isset($def['alias'])) {
				$hdr['id'] = $def['alias'];
			}
			if(isset($def['grouped']))
				$hdr['grouped'] = true;
			if(isset($def['list_position']))
				$hdr['list_position'] = $def['list_position'];
			$src = array_get_default($def, 'source');
			if($src && !isset($this->sources[$src])) {
				$ssplit = explode('.', $src, 2);
				if(count($ssplit) > 1) {
					if(isset($this->sources[$ssplit[0]]))
						$src = $ssplit[0];
				}
			}
			if(! $src || !isset($this->sources[$src]))
				$src = $this->primary_source;
			$hdr['source'] = $src;
			if(! empty($def['total'])) {
				$hdr['total'] = $def['total'];
				$hdr['is_total'] = true;
				$tots[] = $hdr;
			} else {
				if(isset($def['add_fields']))
					$this->formatColumns($def['add_fields'], $hdr['add_fields'], $tots);
				$cols[] = $hdr;
			}
		}
	}
	
	
	function fetchFormatterSources(ListFormatter &$listFmt) {
		$this->sources = $listFmt->getSources();
		if(! is_array($this->sources)) {
			$this->sources = array();
		}
		foreach($this->sources as $id => $src) {
			if(array_get_default($src, 'display') == 'primary') {
				$this->primary_source = $id;
				break;
			}
		}
	}
	
	
	function fetchFormatterColumns(ListFormatter &$listFmt) {
		$fmtCols = $listFmt->getColumns();
		$this->columns = $this->totals = array();
		$this->id_columns = $this->findFields($fmtCols, 'id');
		$this->formatColumns($fmtCols, $this->columns, $this->totals);
		
		return true;
	}
	
	
	function processSumRows() {
		require_once('include/layout/FieldFormatter.php');
		$fmt = new FieldFormatter($this->format, 'export');
		$this->group_sums = array();
		$cols = [];
		foreach ($this->grouping as $_) {
			$cols[] = array_column($this->totals, 'id');
		}

		$this->group_sum_cols = $cols;

		if($this->sums && $this->sums->rows)
		foreach($this->sums->rows as $row) {
			$d = $row['depth'];
			$sum_values = array_slice($row, 2*$this->group_levels + 1);
			
			$fake_row = array();
			foreach($sum_values as $sum_id => $val) {
				if($val === '#SKIP#')
					unset($sum_values[$sum_id]);
				else {
					$fake_row[$sum_id] = $val;
					$sum_values[$sum_id] = $val;
				}
			}
			
			if($d == 0) {
				$this->grand_total = $sum_values;
			}
			else {
				$key = array();
				for($k = 1; $k <= $d; $k++) $key[] = $row['value'.$k];
				if(count($key)) {
					$key = implode("\t", $key);
					$this->group_sums[$d][$key] = $sum_values;
				} else {
					$this->group_sums[$d][] = $sum_values;
				}
			}
		}
		
		$headers = array();
		foreach($this->totals as $t) {
			unset($t['source']);
			$headers[$t['id']] = $t;
		}
		foreach(array_keys($this->group_sums) as $k) {
			$this->group_sums[$k] = $fmt->formatRows($headers, $this->group_sums[$k]);
		}
		if($this->grand_total) {
			$this->grand_total = $fmt->formatRow($headers, $this->grand_total);
		}
	}
	
	
	function initFilenameBase() {
		if($this->report)
			$pretty = translate('LBL_FILENAME_PREFIX', 'Reports').'-'.$this->report['report_name'];
		else {
			$mod = AppConfig::module_for_model($this->result->base_model);
			$pretty = $mod ? $mod : $this->result->base_model;
		}
		$this->filename_base = filename_safe_string($pretty);
	}
	
	function initTitle() {
		global $timedate;
		if($this->report) {
			$title = $this->report['report_name']. ' - '. $timedate->to_display_date_time($this->report['date_entered']);
			if($this->report_data && ! empty($this->report_data['name']))
				$title .= " ({$this->report_data['name']})";
		} else {
			$mod = AppConfig::module_for_model($this->result->base_model);
			if($mod) $title = translate('LBL_MODULE_TITLE', $mod);
			else $title = '';
		}
		$this->title = $title;
	}
	
	
	function setDataResult($result) {
		// for previously run report
		$this->report = $result;
	}

	function setReportResult($result) {
		// for interactive report
		$result['report_name'] = $result['name'];
		$result['date_entered'] = gmdate('Y-m-d H:i:s');
		$this->report = $result;
	}
	
}

