<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/ListView/ListFormatter.php');
require_once('include/pdf/PDFManager.php');


class ListViewHTML {
	var $title;
	var $shown_title = array();
	var $columns;
	var $totals;
    var $charset;
    var $header_text;
    var $levels;
    var $filename;
    var $body = '';

    function init() {
		set_time_limit(180);
	}
	
	function initOutput(ListViewExporter &$exporter) {
        $this->charset = AppConfig::charset();
        $mgr = new PDFManager();
        $this->header_text = $mgr->get_company_header_text();
        $this->levels = $exporter->group_levels;

        $this->title = $exporter->getTitle();
		$this->filename = $exporter->getFilenameBase() . '.html';
		$this->columns = $exporter->getGroupedColumns();
		$this->totals = array();

		foreach($exporter->getTotals() as $total) {
			$this->totals[$total['id']] = $total;
		}
	}
	
	function finishOutput(ListViewExporter &$exporter) {
        global $locale;

        $data = $this->getDocument();
        if($this->charset != 'UTF-8')
            $data = $locale->translateCharset($data, 'UTF-8', $this->charset, true);

        header("Pragma: cache");
        header("Content-type: text/html; charset={$this->charset}");
        header("Content-Disposition: attachment; filename=\"{$this->filename}\"");
        header("Content-transfer-encoding: binary");
        header("Expires: Mon, 26 Jul 1997 05:00:00 GMT" );
        header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT");
        header("Cache-Control: post-check=0, pre-check=0", false);
        if(! AppConfig::get_server_compression())
			header("Content-Length: ".strlen($data));

        echo $data;
    }
    
	function outputGroupRows($gdepth, $rows, $grouping, $totals) {
		$gcols = array();
		$idx = 0;
		foreach($this->columns[$gdepth] as $c) {
			$gcols[$c['id']] = $c;
		}
		foreach($totals as $t) {
			$gcols[$t] = $this->totals[$t];
		}
		$this->formatRows($gdepth, $gcols, $rows);
	}

	function outputRows($depth, $rows) {
		$cols = array();
		foreach($this->columns[$depth] as $idx => $c) {
			if($idx < $depth)
				continue;
			$cols[$c['id']] = $c;
		}
		$this->formatRows($depth, $cols, $rows);
	}

	function outputTotals($rows, $title) {
        if($title)
            $this->body .= '<tr><td colspan="'.($this->levels+1).'"><h4 class="reportGrandTotals">'.$title.'</h4></td></tr>';
        $this->formatRows(-1, $this->totals, $rows, $title);
	}
		
	function formatRows($depth, &$cols, &$rows, $alt_title = null) {
        $nc = is_countable($cols) ? count($cols) : 0;
        foreach($cols as $idx => $c) {
            if(! empty($c['is_total']))
                $cols[$idx]['font-weight'] = 'bold';
            if(! empty($c['is_float']))
                $cols[$idx]['text-align'] = 'right';
            $cols[$idx]['width'] = sprintf('%d', 100/$nc);
        }

        $row = '<tr>'."\n";
        for($i = 0; $i < $depth; $i++) {
            $row .= '<td class="reportDataSpacer" style="width: 15pt">&nbsp;</td>'."\n";
        }
        $row .= '<td colspan="'.($this->levels - min(0, $depth) + 1).'">'."\n";

        $row .= '<table class="reportData" border="0" cellpadding="0" cellspacing="0" width="100%">'."\n";
        $row .= '<thead>';
        $row .= '<tr class="reportDataHeader">'."\n";

        foreach($cols as $name => $col) {
            $width = array_get_default($col, 'width');
            if(!empty($width)) $width = ' width="'.$width.'%"';
            $title = ($alt_title) ? $alt_title : $col['title'];
            $row .= "  <th{$width}>{$title}</th>\n";
        }
        $row .= '</tr>'."\n";
        $row .= '</thead>';
        $row .= '<tbody>';

        foreach($rows as $idx => $r) {
            $row_class = $idx % 2 ? 'odd' : 'even';
            $row .= '<tr class="'.$row_class.'DataRow">'."\n";
            foreach($cols as $name => $col)
                $row .= '  <td>'.$r[$name].'</td>'."\n";
            $row .= '</tr>'."\n";
        }
        $row .= '</tbody>';
        $row .= '</table>'."\n";

        $row .= '</td></tr>'."\n";
        $this->body .= $row;
    }
	
	function formatTotals(&$cols, &$rows, $title) {
		$this->formatRows(-1, $cols, $rows, $title);
	}
	
	function getDocument() {
        $version = AppConfig::version();

        $content = <<<EOH
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"
    "http://www.w3.org/TR/html4/loose.dtd">
<html>
<head>
<title>{$this->title}</title>
<meta name="generator" content="info@hand $version" />
<meta http-equiv="Content-Type" content="text/html; charset={$this->charset}" />
	<style type="text/css">
		body {
			background-color: #fff;
		}
		h3.reportHeader {
			font-family: Helvetica, Arial, sans-serif;
			font-size: 14pt;
			font-weight: bold;
			text-align: center;
		}
		h4.reportGrandTotals {
			font-family: Helvetica, Arial, sans-serif;
			font-size: 12pt;
			font-weight: bold;
			text-align: center;
			margin: 0; padding: 0;
			margin-top: 1em;
		}
		.reportFooter {
			margin-top: 2em;
			font-family: Helvetica, Arial, sans-serif;
			font-size: 8pt;
		}
		.reportFooter a:link, .reportFooter a:visited {
			color: #666;
		}
		table.reportData {
			border: 1px solid #aaa;
			border-top: none;
		}
		table.reportData td {
			vertical-align: top;
			font-family: Helvetica, Arial, sans-serif;
			font-size: 9pt;
			padding: 1px 0.3em;
			color: #333;
		}
		table.reportData tr.oddDataRow {
			background-color: #ddd;
		}
		tr.reportDataHeader {
			background-color: #666;
		}
		tr.reportDataHeader th {
			color: white;
			border: 1px outset #999;
			white-space: nowrap;
			font-family: Helvetica, Arial, sans-serif;
			font-size: 9pt;
			font-weight: bold;
			padding: 1px 0.2em;
		}
		table.reportData a:link, table.reportData a:visited {
			color: #239;
			text-decoration: none;
		}
		table.reportData a:hover {
			text-decoration: underline;
		}
		table.reportMultilineText td {
			font-size: 80%;
		}
	</style>
</head>
<body>

EOH;

        $content .= '<h3 class="reportHeader">'.$this->title."</h3>\n";
        $levels = $this->levels + 1;
		
		$link_url = AppConfig::setting('layout.pdf.footer_link_url');
		$link_title = AppConfig::setting('layout.pdf.footer_link_title');

        $content .= <<<EOH
<table class="reportDataWrapper" border="0" cellpadding="0" cellspacing="0" width="100%">
<tfoot>
    <tr><td colspan="{$levels}">
    <p align="center" class="reportFooter"><a href="{$link_url}">{$link_title}</a></p>
    </td></tr>
</tfoot>
<tbody>
EOH;

        $content .= $this->body;

        $content .= <<<EOH
</tbody>
</table>
</body>
</html>
EOH;

		return $content;
	}

	function &getOutput() {
        $output = array(
            'format' => 'string',
            'data' => $this->getDocument(),
            'filename' => $this->filename,
            'mimetype' => 'text/html',
            'charset' => $this->charset,
        );

        return $output;
    }
}
?>
