<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/ListView/ListFormatter.php');
require_once('include/layout/forms/EditableForm.php');
require_once('include/layout/forms/FilterForm.php');
require_once('modules/SavedSearch/SavedSearch.php');
require_once('include/ListView/MobileListManager.php');
require_once 'modules/UserPreferences/UserPreference.php';

class ListViewManager {
	var $module;
	var $action;
	/** @var ListFormatter */
	var $formatter;
	var $detail;
	var $model_name;
	var $layout_options;
	var $layout_name;
    var $layout_module;
    var $layout_perform;
    var $layout_save_from_admin = false;
    var $layout_session;
    var $layout_title;
    var $report_id;
    var $data_id;
    var $record_perform;
	var $view_name;
    var $view_type;
    var $filter_name;
    var $override_filters;
    var $layout_override_filters;
    /** @var ListLayoutEditor */
    var $layout_editor;
    var $layout_arrange;
	/** @var ListResult */
	var $list_result;
	var $list_id;
	var $current_id;
	var $parent_record;
	var $parent_model;
	var $parent_layout;
	var $parent_tab;
	var $parent_key;
	var $parent_list_id;
	var $subpanel_mgr;
	var $subpanel_def;
	var $subpanel_name;
	var $subpanel_title;
	var $context;
	var $hidden_fields = array();
	var $formatter_class = 'ListFormatter';

	var $audit = false;
	var $show_title = true;
	var $show_tabs = true;
	var $show_filter = true;
	var $show_advanced_filter;
	var $disable_mass_update_data = false;
	var $force_mass_actions_enabled = false;
	var $disable_add_layout = false;
	var $show_mass_update = true;
	var $show_mass_form = true;
	var $mass_panel_open = null;
	var $show_create_button = true;
	var $show_checks = true;
	var $show_additional_details = true;
	var $show_favorites = true;
	var $show_compact_empty = false;
	var $show_help = true;
	var $hide_navigation_controls = false;
	var $disable_request_filter = false;
	var $display_currency_id;
	var $pricebook_id;
	var $no_sort = false;
	var $outer_style = '';
	var $outer_class = '';
	var $debug = false;
	var $async_req = false;
	var $async_list = false;
	var $custom_layout = false;
	var $custom_title, $custom_title_html;
	var $custom_icon;
	var $is_primary = false;
	var $default_list_limit;
	var $add_detail_links;
	var $format = 'html';
	var $is_images_carousel = false;
	var $hooks = [];
	var $add_query_params;

	static public function create($context='listview', $params=null)
	{
		if (AppConfig::is_mobile()) {
			$manager = new MobileListManager($context, $params);
		} else {
			$manager = new ListViewManager($context, $params);
		}
		return $manager;
	}

	function __construct($context = 'listview', $params=null) {
		$this->async_req = defined('ASYNC_ENTRY');
		$this->context = $context;
		
		if($context == 'sidebar') {
			$this->show_tabs = $this->show_checks = $this->show_help = $this->show_filter = false;
			$this->show_additional_details = false;
		}
		
		if(is_array($params)) {
			foreach($params as $k => $v) {
				if($k == 'hidden_fields')
					$this->addHiddenFields($v);
				else if($k == 'override_filters')
					$this->addOverrideFilters($v);
				else
					$this->$k = $v;
			}
		}
	}

	function loadRequest() {
		global $pageInstance;
		
		// intentionally only via $_POST (avoid simple attacks via links)
		$list_id = array_get_default($_POST, 'list_id', '');
		$list_id = preg_replace('~[^A-Za-z0-9\-_]~', '', $list_id);
		if ($list_id) {
			$this->list_id = $list_id;
			$this->async_list = $this->async_req && empty($_POST['initial_load']);
		}
		if($this->async_req || ! empty($_POST['new_from_ext'])) {
			$this->layout_perform = array_get_default($_POST, 'list_layout_perform');
			$this->record_perform = array_get_default($_POST, 'record_perform');
			if($this->layout_perform) {
				$update_title = ($this->layout_perform == 'save' || $this->layout_perform == 'delete');
				$pageInstance->set_partial_page(true, $update_title);
			}
            if ($this->layout_perform == 'save') {
                $from_saved_search = array_get_default($_POST, 'from_saved_search', false);
                if ($from_saved_search)
                    $this->layout_save_from_admin = true;
            }

		}
		$this->format = array_get_default($_REQUEST, 'list_format', 'html');
		$this->display_currency_id = array_get_default($_REQUEST, 'display_currency_id');
		$this->pricebook_id = array_get_default($_REQUEST, 'pricebook_id');
		$this->current_id = array_get_default($_REQUEST, 'list_current_id');
		if(isset($_REQUEST['mass_panel_open']))
			$this->mass_panel_open = ! empty($_REQUEST['mass_panel_open']);
	}
	
	
	function initModule($module, $model_name=null, $layout_module=null) {
		$this->module = $module;
		$this->layout_module = $layout_module ? $layout_module : $module;
		if (!$model_name) 
			$this->model_name = AppConfig::module_primary_bean($module);
		else
			$this->model_name = $model_name;
		return !! $this->model_name;
	}
	

	function initModuleView($module, $layout_name=null, $layout_module=null, $report_id=null, $data_id=null, $model_name = null) {
		if(! $this->initModule($module, $model_name, $layout_module))
			return false;
		
		if($this->loadLayoutSession($layout_name)) {
			$layout_name = $this->layout_session['layout_name'];
		}

		if($layout_name == 'Reports') {
			if(! $this->reportsEnabled())
				$layout_name = 'Browse';
			else {
				$this->report_id = $report_id;
				if(empty($data_id)) $data_id = null;
				$this->data_id = $data_id;
			}
		}
		
		$read_only = AppConfig::read_only();

		if($this->layout_perform == 'save' && ! $read_only) {
			$lid = $this->layoutEditorSave();
			if($lid) {
				if($layout_name == 'Reports')
					$this->report_id = $lid;
				else
					$layout_name = $lid;
			}

            if ($this->layout_save_from_admin) {
                $this->setLayoutAdminModule();
                $layout_name = 'Standard';
            }
		}
		else if($this->layout_perform == 'delete' && ! $read_only) {
			$this->layoutEditorDelete();
			$this->layout_name = 'Browse';
		}
		else if($this->layout_perform == 'show_sql') {
			$this->getLayoutEditor()->showSql();
		}


		if($this->show_tabs !== false)
			$this->show_tabs = AppConfig::setting("display.list.{$this->model_name}.show_tabs", true);
		$this->disable_add_layout = AppConfig::setting("display.list.{$this->model_name}.disable_add_layout", false);
		if($read_only) $this->disable_add_layout = true;
		
		if(AppConfig::setting("display.list.{$this->model_name}.disabled"))
			return false;
		
		if(! ACLController::checkAccess($this->module, 'edit'))
			$this->disable_mass_update_data = true;

		// force creation of standard records (like base currency), but only if caching is enabled
		AppConfig::db_check_standard_records($this->model_name, true);
		
		return $this->loadLayoutInfo($layout_name);
	}

	function getLayoutSessionKey() {
		if($this->is_primary)
			return 'ListView_' . $this->module . '_Layout';
	}
	
	function loadLayoutSession($check_name=null) {
		$key = $this->getLayoutSessionKey();
		if($key && empty($_REQUEST['report_id'])) {
			$layout = array_get_default($_SESSION, $key);
			if($layout && (! $check_name || (isset($layout['layout_name']) && $layout['layout_name'] == $check_name))) {
				$this->layout_session = $layout;
				if(! isset($this->mass_panel_open) && isset($layout['mass_panel_open']))
					$this->mass_panel_open = $layout['mass_panel_open'];
				return true;
			}
		}
		$this->layout_session = null;
	}
	
	function saveLayoutSession($layout) {
		$key = $this->getLayoutSessionKey();
		if($key) $_SESSION[$key] = $layout;
	}

    function setLayoutAdminModule() {
        global $currentModule;
        $currentModule = 'SavedSearch';
        $this->module = $currentModule;
        $this->model_name = AppConfig::module_primary_bean($this->module);
        $this->layout_module = $this->module;
    }

    function initSubpanelView($module, $parent_id, $subpanel_name=null, $layout_name=null, $tab_name=null) {
    	global $themeInstance, $pageInstance;
    	
		$this->module = $module;
		if(! $layout_name)
			$layout_name = 'Standard';
		$this->parent_model = AppConfig::module_primary_bean($module);
		
		$q = new ListQuery($this->parent_model, true);
		$dname = $q->addDisplayName();
		$q->addFilterPrimaryKey($parent_id);
		$this->parent_record = $q->runQuerySingle();
		if(! $this->parent_record || $this->parent_record->failed)
			return false;
		$this->parent_key = $parent_id;
		
		if(AppConfig::setting('site.features.combined_detail') && empty($_REQUEST['std'])) {
			$detail = AppConfig::setting("views.detail.$module.record.$layout_name");
			if($detail) {
				$panels = AppConfig::setting("views.layout.$module.record.$layout_name.subpanels");
				$tabs = AppConfig::setting("views.layout.$module.record.$layout_name.tabs");
			}
		}
		if(! isset($panels))
			$panels = AppConfig::setting("views.layout.$module.view.$layout_name.subpanels");
		require_once('include/SubPanel/SubPanelManager.php');
		$mgr = new SubPanelManager($this->parent_record);
		if(! empty($tabs)) {
			$mgr->setDefaultTab(key($tabs));
			if($tab_name) $mgr->setCurrentTab($tab_name, array('is_primary' => $this->is_primary));
		}
		$mgr->setPanelList($panels, $subpanel_name);
		$visib = $mgr->getVisiblePanels();
		if(! $visib || empty($visib[$subpanel_name]))
			return false;
		$pdef = $visib[$subpanel_name];
		
		$this->subpanel_def = $pdef;
		$this->subpanel_mgr = $mgr;
		$this->subpanel_name = $subpanel_name;
		$this->parent_layout = $layout_name;
		$this->parent_tab = $tab_name;
		$this->model_name = $pdef['bean_name'];
		$this->layout_module = $pdef['module'];
		$parent_name = $this->parent_record->getField($dname, '');
		$detail_link = get_detail_link_info($parent_id, $module, null, array('layout' => $layout_name, 'layout_tab' => $tab_name));
		$parent_link = '<a href="' . to_html($detail_link['url']) . '">' . to_html($parent_name) . '</a>';
		$parent_icon = get_module_header_icon($module, '');
		if(empty($pdef['title']))
			$title = $pageInstance->get_module_name($pdef['module']);
		else
			$title = translate($pdef['title'], array_get_default($pdef, 'vname_module', $module));
		$this->subpanel_title = $title;
		$chtml = '<span class="parentLink">&laquo; ' . $parent_icon . ' ' . $parent_link.'</span>';
		if(! $themeInstance->version(8))
			$chtml = to_html($title) . ' ' . $chtml;
		$this->custom_title_html = $chtml;
		
		return $this->loadLayoutInfo($pdef['layout_name'], $pdef['layout_type'], $pdef['filter_name']);
	}
	
	
	function reportsEnabled() {
		if($this->subpanel_name) return false;
		$rep = AppConfig::setting("model.detail.{$this->model_name}.reportable");
		return ! empty($rep) && ACLController::checkAccess($this->module, 'report');
	}
	
	
	static function load_model_layouts($model, $layout_module=null) {
		$layouts = array();
		if(! $layout_module)
			$layout_module = AppConfig::module_for_model($model);
		
		$filter_layouts = FilterForm::load_layout_index($layout_module);
		$prios = array();
		if($filter_layouts) {
			foreach($filter_layouts as $k => $l) {
				if($k == 'Browse' || $k == 'Standard') {
					$layouts[$k] = array('name' => $k, 'filter_name' => $k, 'view_name' => 'Standard', 'label' => $l['title'], 'priority' => $l['priority']);
					$prios[] = $l['priority'];
				}
			}
		}
		
		$std_layouts = AppConfig::setting("display.list.$model.layouts");
		if(! empty($std_layouts)) {
			foreach($std_layouts as $k => $l) {
				if(! is_array($l))
					continue;
				if(isset($l['name']))
					$k = $l['name'];
				else
					$l['name'] = $k;
				if(empty($l['label']) && ! empty($l['vname']))
					$l['label'] = $l['vname'];
				if(! isset($l['filter_name']))
					$l['filter_name'] = 'Browse';
				if(! isset($l['view_name']))
					$l['view_name'] = 'Standard';
				if(! isset($l['priority']))
					$l['priority'] = 15;
				$layouts[$k] = $l;
			}
		}
		return $layouts;
	}
	
	
	function loadLayoutIndex() {
		$layouts = array();
		$prios = array();

		$std_layouts = self::load_model_layouts($this->model_name, $this->layout_module);
		foreach($std_layouts as $k => $l) {
			$layouts[$k] = $l;
			$prios[] = $l['priority'];
		}
		
		if($this->reportsEnabled() && !AppConfig::is_mobile()) {
			$layouts['Reports'] = array(
				'name' => 'Reports',
				'label' => 'LBL_LAYOUT_REPORTS',
				'icon' => 'icon-reports',
				'view_name' => 'ForModule',
				'filter_name' => 'Standard',
				'override_filters' => array(
					'primary_module' => array(
						'value' => $this->module,
						'hidden' => true,
					),
				),
			);
		}

		$custom = SavedSearch::get_index($this->layout_module);
		if($custom) {
			foreach($custom as $c) {
				$layouts[$c['id']] = array(
					'name' => $c['id'], 'label' => $c['name'], 'is_custom' => true,
				);
			}
		}
		$this->layout_options = $layouts;
	}
	
	
	function getLayoutType() {
		if($this->context == 'listview' || $this->context == 'sidebar')
			return 'list';
		return $this->context;
	}


	function loadLayoutInfo($layout_name = null, $layout_type = null, $filter_name = null) {
		$type = $layout_type ? $layout_type : $this->getLayoutType();
		if($type == 'list' && ! $this->subpanel_name) {
			$this->loadLayoutIndex();
			$layouts =& $this->layout_options;

			if(! isset($layout_name) || ! isset($layouts[$layout_name])) {
				if(AppConfig::is_admin()) {
					// look up another user's custom layout (via saved search module)
					$cl = SavedSearch::load_search($this->module, $layout_name, null, false);
					if($cl) {
						$sel_lay = array(
							'name' => $cl['id'], 'label' => $cl['name'], 'is_custom' => true,
						);
					}
				}
				if(empty($sel_lay)) {
					// select first tab from user's preferred order
					$order = $this->getUserTabOrder();
					if($order) {
						foreach($order as $ord_tab) {
							if($ord_tab[1] && isset($layouts[$ord_tab[0]])) {
								$layout_name = $ord_tab[0];
								$sel_lay = $layouts[$layout_name];
								break;
							}
						}
					}
				}
				if(empty($sel_lay)) {
					// select first tab from standard order
					foreach($layouts as $k => $l) {
						$layout_name = $k;
						$sel_lay = $l;
						break;
					}
				}
			}
			else
				$sel_lay = $layouts[$layout_name];
			if(isset($sel_lay)) {
				if(! empty($sel_lay['is_custom']))
					$custom_layout = true;
				else {
					if(! $filter_name)
						$filter_name = $sel_lay['filter_name'];
					$view_name = $sel_lay['view_name'];
					if(isset($sel_lay['override_filters']))
						$this->layout_override_filters = $sel_lay['override_filters'];
				}
				$this->layout_title = translate(array_get_default($sel_lay, 'label'), $this->layout_module);
			}
		} else {
			if(! isset($layout_name))
				$layout_name = 'Standard';
			$view_name = $layout_name;
		}
		
		if(! empty($custom_layout)) {
			$this->custom_layout = true;
			$this->layout_name = $layout_name;
			$this->view_type = $type;
			return true;
		}
		else if($layout_name == 'Reports')
			$layout_mod = 'Reports';
		else
			$layout_mod = $this->layout_module;
		
		$this->custom_layout = false;
		if(isset($view_name)) {
			$detail = AppConfig::setting("views.detail.{$layout_mod}.{$type}.{$view_name}");
			if(! $detail && $type == 'popup')
				$detail = AppConfig::setting("views.detail.{$layout_mod}.list.{$view_name}");
		}

		if(! empty($detail)) {
			$this->layout_name = $layout_name;
			$this->view_name = $detail['name'];
			$this->view_type = $detail['type'];
			if(! $filter_name && ! empty($detail['filter_name']))
				$filter_name = $detail['filter_name'];
			$this->filter_name = $filter_name;

			$layout = AppConfig::setting("views.layout.{$layout_mod}.{$type}.{$view_name}");
			$this->hooks = array_get_default($layout, 'hooks', array());
			return true;
		} else {
			$this->layout_name = null;
			$this->view_name = null;
			$this->view_type = null;
			$this->filter_name = null;
			return false;
		}
	}
	
	
	function &getLayoutEditor() {
		if(! $this->layout_editor) {
			require_once('include/ListView/ListLayoutEditor.php');
	
			$this->layout_editor = new ListLayoutEditor();
			$this->layout_editor->initFromManager($this, $this->layout_perform);
			$this->layout_editor->loadRequest();
		}
		return $this->layout_editor;
	}

	function &getLayoutArrange() {
		if(! $this->layout_arrange) {
			require_once('include/ListView/ListLayoutArrange.php');
	
			$this->layout_arrange = new ListLayoutArrange();
			$this->layout_arrange->initFromManager($this, $this->layout_perform);
			$this->layout_arrange->loadRequest();
		}
		return $this->layout_arrange;
	}

	function layoutEditorRender() {
		return $this->getLayoutEditor()->render();
	}
	
	function layoutArrangeRender() {
		return $this->getLayoutArrange()->render();
	}
	
	function layoutEditorSave() {
		return $this->getLayoutEditor()->save();
	}
	
	function layoutEditorDelete() {
		return $this->getLayoutEditor()->delete();
	}
	
	function getFilterForm() {
		return $this->getFormatter()->getFilterForm();
	}
	
	function getFilterName() {
		return $this->filter_name;
	}
	
	function getListResult() {
		return $this->list_result;
	}

	function getButtons() {
		if($this->subpanel_mgr) {
			return $this->subpanel_mgr->getPanelButtons($this->subpanel_def, $this->list_id);
		}

		$buttons = array();
		
		$show_create = AppConfig::setting("display.list.{$this->model_name}.show_create_button");
        if ($show_create !== null && $this->show_create_button)
            $this->show_create_button = $show_create;

		if($this->layout_name == 'Reports') {
			if(empty($this->report_id) && ACLController::checkAccess('Reports', 'create')) {
				$buttons['create'] = array(
					'icon' => 'icon-edit',
					'label' => translate('LBL_CREATE_REPORT_BUTTON_LABEL'),
					'perform' => "return sListView.showCreateLayout('$this->list_id');",
					'type' => 'button',
					'params' => array(
						'toplevel' => 1,
					),
				);
			}
		}
		else if($this->show_create_button && ACLController::checkAccess($this->module, 'create')) {
			if ($this->context == 'sidebar') {
				$perform = "SUGAR.util.fetchContentMain('async.php?module={$this->module}&action=EditView&list_layout_name={$this->layout_name}');";
			}
            else if (! $this->showDetailInPopup()) {
                $perform = "SUGAR.util.navigateTo('index.php?module={$this->module}&action=EditView&list_layout_name={$this->layout_name}');";
            }
            else {
                $qlayout = '';
                if(! AppConfig::setting("views.detail.{$this->module}.edit.QuickCreate"))
                    $qlayout = '&layout=Standard';
                $perform = "SUGAR.popups.openUrl('async.php?module=".$this->module."&action=EditView&in_popup=true&mgr_type=quick_create&target_name=listview&target_id={$this->list_id}{$qlayout}', null, {width: '750px', resizable: false}); return false;";
            }
            
            if($this->module == 'Reports') {
				$buttons['create'] = array(
					'icon' => 'icon-edit',
					'label' => translate('LBL_CREATE_BUTTON_LABEL'),
					'perform' => "return sListView.showCreateLayout('$this->list_id');",
					'type' => 'button',
					'params' => array(
						'toplevel' => 1,
					),
				);
            } else {
				$buttons['create'] = array(
					'icon' => 'icon-edit',
					'label' => translate('LBL_CREATE_BUTTON_LABEL'),
					'perform' => $perform,
					'type' => 'button',
					'params' => array(
						'toplevel' => 1,
					),
				);
			}
		}
		

		return $buttons;
	}

	function getHiddenFields() {
		$hidden = $this->hidden_fields;
		$defaults = array(
			'layout' => $this->layout_name,
		);
		if($this->layout_name == 'Reports' && $this->report_id) {
			$defaults['report_id'] = $this->report_id;
			$defaults['data_id'] = $this->data_id;
		}
		if($this->debug)
			$defaults['debug'] = 1;
		if($this->subpanel_name) {
			$defaults['subpanel'] = $this->subpanel_name;
			$defaults['parent_layout'] = $this->parent_layout;
			$defaults['record'] = $this->parent_key;
		}
		if($this->display_currency_id)
			$defaults['display_currency_id'] = $this->display_currency_id;
		if($this->pricebook_id)
			$defaults['pricebook_id'] = $this->pricebook_id;
		if($this->context != 'listview')
			$defaults['list_context'] = $this->context;
		if(property_exists($this,'current_id'))
			$defaults['list_current_id'] = $this->current_id;
		if(! is_array($hidden)) $hidden = array();
		$hidden = array_merge($defaults, $hidden);
		return $hidden;
	}
	
	function addHiddenFields(array $fields) {
		if($this->hidden_fields)
			$this->hidden_fields = array_merge($this->hidden_fields, $fields);
		else
			$this->hidden_fields = $fields;
	}
	
	function getOverrideFilters() {
		$s = $this->override_filters;
		if(! is_array($s)) $s = array();
		$l = $this->layout_override_filters;
		if(! is_array($l)) $l = array();
		return array_merge($l, $s);
	}

	function addOverrideFilters(array $filters) {
		if($this->override_filters)
			$this->override_filters = array_merge($this->override_filters, $filters);
		else
			$this->override_filters = $filters;
	}
	
	
	function getCountOptions() {
		$opts = AppConfig::setting('layout.list_display_row_options');
		$max = $this->getMaxDisplayRows();
		$ret = array();
		foreach($opts as $o)
			if($o <= $max) $ret[] = $o;
		return $ret;
	}
	
	function getMaxDisplayRows() {
		if($this->context == 'sidebar')
			return AppConfig::setting('layout.list.sidebar_max_display_rows', 50);
		return AppConfig::setting('layout.list.max_display_rows', 100);
	}


	/**
	 * Get list formatter
	 * @return ListFormatter
	 */
	function &getFormatter() {
		if (!$this->formatter) {
			$fmt_params = array('show_detail_in_popup' => $this->showDetailInPopup());
			if($this->is_primary)
				$fmt_params['listview_primary'] = true;
			if($this->layout_name == 'Reports' && ! $this->report_id)
				$model = 'Report';
			else {
				if($this->layout_name == 'Reports')
					$this->layout_override_filters = null; // remove primary_module filter
				$model = $this->model_name;
				if($this->subpanel_name)
					$model = $this->parent_model;
				if($this->layout_module && $this->layout_module != $this->module)
					$fmt_params['layout_module'] = $this->layout_module;
			}
			
            $selector_disabled = AppConfig::setting("display.list.{$this->model_name}.item_selector.disabled");
            if($this->show_favorites) {
				if($this->layout_name == 'Reports' && ! $this->report_id)
					$show_favorites = AppConfig::setting("display.list.Report.show_favorites");
				else
					$show_favorites = AppConfig::setting("display.list.{$this->model_name}.show_favorites");
			} else
				$show_favorites = false;
            $fmt_params['show_checks'] = $selector_disabled ? false : $this->show_checks;
            $fmt_params['show_additional_details'] = $this->show_additional_details;
            $fmt_params['show_favorites'] = !! $show_favorites;
			$fmt_params['show_mass_update'] = $this->showMassUpdate();
			$fmt_params['disable_mass_update_data'] = $this->disable_mass_update_data;
			$fmt_params['force_mass_actions_enabled'] = $this->force_mass_actions_enabled;
			$fmt_params['mass_panel_open'] = $this->mass_panel_open;
            $fmt_params['show_mass_form'] = $this->show_mass_form;
            $fmt_params['show_title'] = $this->show_title;
			$fmt_params['show_filter'] = $this->show_filter;
			$fmt_params['show_advanced_filter'] = $this->show_advanced_filter;
			$fmt_params['show_compact_empty'] = $this->show_compact_empty;
			$fmt_params['hide_navigation_controls'] = $this->hide_navigation_controls;
			$fmt_params['display_currency_id'] = $this->display_currency_id;
			$fmt_params['no_sort'] = $this->no_sort;
			$fmt_params['current_id'] = $this->current_id ?? null;
			$fmt_params['add_detail_links'] = $this->add_detail_links;
			$fmt_params['is_images_carousel'] = $this->is_images_carousel;
			$fmt_params['listview_manager'] =& $this;

			$cls = $this->formatter_class;
			$def = AppConfig::setting("modinfo.by_name.{$this->layout_module}.list_formatter_factory");
			if (!empty($def)) {
				require_once $def['file'];
				$cls = $def['class'];
			}

			$fmt = $cls::create($model, $this->subpanel_name, $this->list_id, $this->context, $fmt_params);
			$fmt->setFormat($this->format, $this->context);
			if($this->parent_key)
				$fmt->setParentKey($this->parent_key);
			if (!$this->list_id) $this->list_id = $fmt->list_id;
			
			$this->formatter =& $fmt;
		}
		return $this->formatter;
	}

	function getTitle($html=true) {
		global $themeInstance;
		if($html && $this->custom_title_html)
			$title = $this->custom_title_html;
		else if(!$html && $this->custom_title)
			$title = $this->custom_title;
		else if(! isset($themeInstance->classic_title) && $html && $this->context == 'listview' && $this->show_title !== 'formatter')
			$title = $this->layout_title;
		else {
			$title = AppConfig::setting("display.list.{$this->model_name}.title", 'LBL_MODULE_TITLE');
			$title = translate($title, $this->layout_module);
			if($html) $title = to_html($title);
		}
		return $title;
	}

	function renderTitle() {
		global $pageInstance;
		
		$title = $this->getTitle();
		if($this->show_title === 'formatter') {
			$this->formatter->title = $title;
			if($this->context == 'listview' && ! $this->formatter->icon)
				$this->formatter->icon = $this->module;
			return;
		}
		if($this->context == 'sidebar') {
			$this->formatter->show_title = false;
			$this->formatter->title = $this->layout_title;
			return;
		}
		
		$icon = $this->custom_icon;
		$mod = AppConfig::module_for_model($this->model_name);
		if (!$icon) $icon = $mod;
		if (isset($this->hooks['pre_title'])) {
			echo call_hook_function($this->hooks['pre_title'], null, array($this));
		}

		if($this->subpanel_def) {
			$mod = array_get_default($this->subpanel_def, 'icon');
			if($mod) $icon = $mod;
			if(! $mod || strpos($mod, '-') !== false) $icon = $mod = $this->layout_module;
			$pageInstance->set_title_module($mod, $this->subpanel_title, true);
		}
		echo get_module_title($icon, $title, $this->context != 'popup' && $this->show_help);

		if (isset($this->hooks['post_title'])) {
			echo call_hook_function($this->hooks['post_title'], null, array($this));
		}
	}
	
	
	function initFormatter(ListFormatter &$fmt) {
		if($this->layout_name == 'Reports') {
			if($this->report_id) {
				if($this->data_id && $this->layout_perform == 'delete_run') {
					 if($fmt->deleteReportData($this->module, $this->report_id, $this->data_id))
						$this->data_id = null;
				}
				if(! $fmt->loadReport($this->module, $this->report_id, $this->data_id, $this->view_type)) {
					$this->report_id = '';
					$this->data_id = null;
				} else {
					require_once('data/Tracker.php');
					if($fmt->report_title && $fmt->report_owner)
						Tracker::track_view(AppConfig::current_user_id(), 'Reports', 'Report', $this->report_id, $fmt->report_title, $fmt->report_owner);
				}
			}
			if(! $this->report_id) {
				if(! $fmt->loadLayout($this->view_name, $this->view_type, 'Reports'))
					return false;
				$fmt->detail_link_action = "sListView.setLayout('$this->list_id', 'Reports', '{ID}'); return false;";
			}
		}
		else if($this->custom_layout) {
			if(! $fmt->loadSavedSearch($this->layout_module, $this->layout_name, $this->view_type)) {
				return false;
			}
		}
		else {
			if(! $fmt->loadLayout($this->view_name, $this->view_type, $this->layout_module)) {
				return false;
			}
			if($this->filter_name)
				$fmt->getFilterForm()->setDefaultLayout($this->filter_name);
		}
		
		$hidden = $this->getHiddenFields();
		$fmt->getMassUpdate()->addHiddenFields($hidden);
		if($this->subpanel_mgr) {
			$fmt->getMassUpdate()->setSubpanel($this->subpanel_mgr, $this->subpanel_name);
		}
		$fmt->getFilterForm()->addHiddenFields($hidden);
		$add = $this->getOverrideFilters();
		if($add) $fmt->getFilterForm()->addOverrideFilters($add);
		$fmt->addButtons($this->getButtons());
		
		return true;
	}
	
	function loadRequestFilter(ListFormatter &$fmt) {
		$session_filter = null;
		if(is_array($this->layout_session)) {
			$session_filter = array_get_default($this->layout_session, 'filter_values');
			if(is_array($session_filter))
				$session_filter['filter_layout'] = array_get_default($this->layout_session, 'filter_layout');
		}
		$fmt->loadRequestFilter('listview', $session_filter);
	}
	
	function getUserTabOrder() {
		return UserPreference::getPreference('list_tabs_' . $this->module);
	}

	function reorderTabsFromPreferences(&$tabs, $all) {
		$pref = $this->getUserTabOrder();
		if (!is_array($pref))
			return;
		$prio = 0;
		$ret = array();
		$later = array();
		foreach ($pref as $p) {
			if (isset($tabs[$p[0]])) {
				if ($all || array_get_default($p, 1, true)) {
					$t = $tabs[$p[0]];
					$t['priority'] = $prio;
					$prio++;
					$ret[$p[0]] = $t;
				}
				unset($tabs[$p[0]]);
			}
		}
		$tabs = array_merge($ret, $tabs);
	}
	
	function getLayoutTabs($all = false) {
		$tabs = $this->layout_options;
		if(empty($tabs) || $this->subpanel_name)
			return;
		$this->reorderTabsFromPreferences($tabs, $all);
		$ret = array();
		$custom_sep = array('name' => 'custom_sep', 'separator' => true);
		$custom_sep = null;//array('name' => 'custom_sep', 'separator' => true);
		foreach($tabs as $n => $t) {
			if(! empty($t['hidden'])) {
				$flag = expand_filter_flag($t['hidden']);
				$hide = eval_filter_flag($flag);
				if($hide) continue;
				unset($t['hidden']);
			}
			if(! empty($t['is_custom']) && $custom_sep) {
				$ret[] = $custom_sep;
				$custom_sep = null;
			}
			$t['label'] = translate($t['label'], $this->layout_module);
			$t['perform'] = "sListView.setLayout('$this->list_id', '$n');";
			$ret[] = $t;
		}
		
		if(empty($this->disable_add_layout) && ! $this->subpanel_name && !AppConfig::is_mobile()) {
			$ret[] = array('name' => 'custom_sep2', 'separator' => true);
			$ret[] = array('icon' => 'icon-editlayout', 'name' => 'ArrangeLayouts', 'disable_select' => true, 'hide_label' => true,
				'perform' => "sListView.showArrangeLayouts('$this->list_id');", 'label' => translate('LBL_ARRANGE_LAYOUTS_FORM_TITLE', 'app'));
			$ret[] = array('icon' => 'icon-add', 'name' => 'EditLayouts', 'disable_select' => true, 'hide_label' => true,
				'perform' => "sListView.showCreateLayout('$this->list_id');", 'label' => translate('LBL_CREATE_LAYOUT_FORM_TITLE', 'app'));
		}

		return $ret;
	}
	
	
	function renderTabs() {
		$tabs = $this->getLayoutTabs();
		if(! $tabs) return '';
		
		$frm = new EditableForm('tabgroup', 'TabForm', $this->list_id.'-TabForm');
		$frm->setFormClass('form-tabs');
		$spec = array(
			'name' => 'layout',
			'tabs' => $tabs,
			'class' => 'listview-tabs',
		);
		
		$body = $frm->renderTabGroup($spec, $this->layout_name);
		$ret = $frm->open() . $body . $frm->close();
		$frm->exportIncludes();
		return $ret;
	}
	
	function getLayoutTitle() {
		
	}
	

	function render() {
		global $pageInstance;
		if($this->context == 'sidebar')
			$pageInstance->set_partial_page();

		if($this->layout_perform == 'edit' || $this->layout_perform == 'create' || $this->layout_perform == 'duplicate') {
			$this->layoutEditorRender();
			return;
		}
		if($this->layout_perform == 'arrange' || $this->layout_perform == 'arrange_save' || $this->layout_perform == 'arrange_reset') {
			if ($this->layoutArrangeRender())
				return;
		}
		if($this->layout_perform == 'show_sql') {
			return;
		}
		
		$perf = $this->record_perform;
		if($perf == 'subpanel_insert' || $perf == 'subpanel_render' || $perf == 'subpanel_remove' || $perf == 'report_data_remove') {
			$list_id = array_get_default($_REQUEST, 'list_id');
			if($this->report_id) {
				$submgr = $this->reportSubpanelMgr();
				$link_name = array_get_default($_REQUEST, 'link_name');
			}
			else {
				$submgr = $this->subpanel_mgr;
				$link_name = $this->subpanel_name;
			}
			if($submgr && $link_name && $list_id) {
                $offset = array_get_default($_REQUEST, 'list_offset');
                $order_by = array_get_default($_REQUEST, 'list_order_by', null);
                if($perf == 'subpanel_insert') {
                    $sel_ids = array_get_default($_REQUEST, 'selected_id');
                    $submgr->handlePanelInsert($link_name, $sel_ids);
                    $offset = null;
                } elseif($perf == 'subpanel_remove') {
                    $link_id = array_get_default($_REQUEST, 'link_id');
                    $submgr->handlePanelRemove($link_name, $link_id);
                    $offset = null;
                } else if($perf == 'report_data_remove') {
                    $sel_ids = array_get_default($_REQUEST, 'list_uids', '');
                    if($sel_ids != 'all') $sel_ids = explode(';', $sel_ids);
                    $submgr->handlePanelDelete($link_name, $sel_ids);
                    $offset = null;
                    $_REQUEST['list_uids'] = null;
                }
				if($this->report_id)
					$submgr->renderPanelByName($link_name, $list_id, $offset, $order_by);
            }
            if($this->report_id)
				return;
        }

		$isHTML = $this->format == 'html';
	
		$fmt =& $this->getFormatter();
		if(! $this->initFormatter($fmt))
			return;
        if (!$this->disable_request_filter)
			$this->loadRequestFilter($fmt);

		if ($this->async_list && ($this->showMassUpdate() || $this->force_mass_actions_enabled)) {
			$r = $fmt->performMassUpdate();
			if ($r)
				return $r;
		}

		if (!$this->async_list && $isHTML) {
			$ostyle = $this->outer_style;
			$oclass = 'listViewOuter '.($this->is_primary ? 'listViewPrimary ' : '') . $this->outer_class;
			if($this->context != 'popup' && $this->context != 'sidebar')
				echo '<div id="' . $this->list_id . '-outer" style="position: relative; '.$ostyle.'" class="'.$oclass.'">';
		}

		if($this->show_title) {
			$this->renderTitle();
		}
		
		$nav = $this->getNavParams();
		$addc = $this->getAdditionalClauses();
		if($addc) $fmt->addFilterClauses($addc);
		$this->list_result = $fmt->getQueryResult($nav['offset'], $nav['limit'], $nav['order_by']);

		if($this->list_result->failed && ($this->debug || AppConfig::setting('site.allow_debug')))
			pr2($this->list_result->getError(), 'query error', true);

		if ($this->debug) {
			prq($this->list_result->query, 'list query', true);
			if($this->list_result->count_query)
				prq($this->list_result->count_query, 'count query', true, true);
			pr2($this->list_result->fields, 'list fields', true, true);
			if($this->debug > 1)
				pr2($this->list_result->rows, 'rows');
			if($this->debug > 1)
				pr2($this->list_result->formatted_rows, 'formatted rows');
		}

		if ($this->audit)
			$this->formatAuditedFields($this->list_result);
		
		if($this->show_tabs && $isHTML)
			echo $this->renderTabs();

		$fmt->formatResult($this->list_result);

		$this->renderResult($fmt, $this->list_result);
		
		if($this->report_id && ! $this->data_id && $fmt->disable_query) {
			$this->outputReportDetail();
		}
		
		if ($this->debug && isset($fmt->list_query->field_warnings))
			pr2($fmt->list_query->field_warnings, 'field warnings');

		if ($this->debug)
			pr2($_REQUEST, 'request');
		
		if (!$this->async_list && $isHTML) {
			if($this->context != 'popup' && $this->context != 'sidebar')
				echo '</div>';
		}

		if ($isHTML) {
			$this->addPageState();
			if(! $this->report_id) {
				$filter_vals = null;
				$filter_src = $fmt->getFilterSource();
				if($filter_src != 'none')
					$filter_vals = $fmt->getFilterArray(true);
				$session = array(
					'layout_name' => $this->layout_name,
					'offset' => $nav['offset'],
					'limit' => $nav['limit'],
					'order_by' => $nav['order_by'],
					'filter_layout' => $fmt->getFilterLayoutName(),
					'filter_values' => $filter_vals,
					'mass_panel_open' => $this->mass_panel_open,
				);
				$this->saveLayoutSession($session);
			}
		}

		if (!$this->async_list && $isHTML) {
			$GLOBALS['pageInstance']->add_js_include("include/javascript/dropzone.js", null, LOAD_PRIORITY_FOOT);
			$GLOBALS['pageInstance']->add_js_include("include/javascript/dnduploads.js?" . time() , null, LOAD_PRIORITY_FOOT);
			$this->exportDocumentDropJS();
		}

	}
	
	function exportDocumentDropJS() {
		if ($this->context == 'popup')
			return;
		if ($this->model_name != 'Document' && $this->model_name != 'Note') {
			return;
		}
		$opts = [
			'list_id' => $this->list_id,
			'model' => $this->model_name,
			'module' => $this->module,
		];
		$json = getJSONobj();
        $GLOBALS['pageInstance']->add_js_literal("DNDUploader(" . $json->encode($opts)  . ");", null, LOAD_PRIORITY_FOOT);
	}

	
	function getNavParams() {
		$offset = 0;
		$limit = $this->default_list_limit;
		$order_by = null;
		if($this->layout_session) {
			if(empty($_REQUEST['filter_clear']) && empty($_REQUEST['query'])) {
				$offset = array_get_default($this->layout_session, 'offset', $offset);
				$limit = array_get_default($this->layout_session, 'limit', $limit);
			}
			if(empty($_REQUEST['filter_clear'])) {
				$order_by = array_get_default($this->layout_session, 'order_by', $order_by);
			}
		}
		$offset = array_get_default($_REQUEST, 'list_offset', $offset);
		$limit = array_get_default($_REQUEST, 'list_limit', $limit);
		if(isset($offset)) {
			if(! is_numeric($offset)) $offset = null;
			else
				$offset = max(0, round($offset));
		}
		if(isset($limit)) {
			if(! is_numeric($limit)) $limit = null;
			else {
				$limit = round($limit);
				$max = $this->getMaxDisplayRows();
				if($max) $limit = min($limit, $max);
			}
		}
		$order_by = array_get_default($_REQUEST, 'list_order_by', $order_by);
		return compact('offset', 'limit', 'order_by');
	}
	
	function renderResult(ListFormatter &$fmt, ListResult &$result) {
		if ($this->format == 'pdf')  {
			$fmt->outputPdf($result, $this->getTitle());
		} else {
			$fmt->outputHtml($result);
		}
	}
	
	function reportSubpanelMgr() {
		require_once('include/SubPanel/SubPanelManager.php');
		$parent_list_id = array_get_default($_REQUEST, 'parent_list');
		$layoutListId = empty($parent_list_id) ? $this->list_id : $parent_list_id;
		// FIXME - avoid multiple queries for the report
		$r = ListQuery::quick_fetch('Report', $this->report_id, true, array('acl_checks' => 'view,edit'));
		$panels = array(
			'report_data' => array(
				'hide_create_button' => true,
				'default_order_by' => 'date_entered desc',
				'detail_link_action' => "sListView.setLayout('$layoutListId', 'Reports', '$this->report_id', '{ID}'); return false;",
				'icon' => 'History',
				'show_checks' => true,
			),
			'notify_users' => array(),
			'securitygroups' => array(),
		);
		if($r->getField('assigned_user_id') != AppConfig::current_user_id() && ! AppConfig::is_admin()) {
			$panels['securitygroups']['updateable'] = false;
		}
		$mgr = new SubPanelManager($r);
		$mgr->setPanelList($panels);
		$mgr->show_browse_panel_link = false;
		$mgr->show_config_panel_link = false;
		$mgr->parent_list_id = $layoutListId;
		$mgr->addHiddenFields(array('report_id' => $this->report_id));
		return $mgr;
	}
	
	function outputReportDetail() {
		$mgr = $this->reportSubpanelMgr();
		echo '<div class="subpanel-list report-subpanels">';
		$mgr->outputHtml();
		echo '</div>';
	}
	
	
	function addPageState() {
		global $pageInstance;
		$params = array("type: 'listview'");
		if($this->is_primary) {
			$params[] = 'is_primary: true';
			$params[] = 'query: "'.javascript_escape($this->getQueryString()).'"';
		}
		if($this->context == 'sidebar')
			$params[] = 'is_sidebar: true';
		$params[] = 'count_options: ['.implode(', ', $this->getCountOptions()).']';
		$params[] = 'sidebar_limit: '.((int)AppConfig::setting('layout.list.sidebar_display_rows'));

        $fmt =& $this->getFormatter();
        if(! $fmt) return false;

        $mass_perform = array_get_default($fmt->massupdate->request, 'mass_perform', '');
        $params[] = 'mass_perform: "' .$mass_perform. '"';
        if($this->list_result) {
        	$rc = $this->list_result->result_count;
        	if(! is_numeric($rc)) $rc = 'null';
        	$tc = $this->list_result->total_count;
        	if(! is_numeric($tc)) $tc = 'null';
        	$params[] = "result_count: $rc";
        	$params[] = "total_count: $tc";
        }

		$params = '{'.implode(',',$params).'}';
		$pageInstance->add_js_literal("sListView.init('{$this->list_id}', $params);", null, LOAD_PRIORITY_FOOT);
		if($this->is_primary)
			$pageInstance->set_page_state(array('context' => 'listview', 'module' => $this->module, 'layout' => $this->layout_name));
		
		if($this->layout_module && $this->layout_module != $this->module && $this->context != 'unified_search') {
			$pageInstance->add_js_language($this->layout_module, null, LOAD_PRIORITY_FOOT);
		}
		
		$fmt->exportIncludes();
	}
	
	
	function getQueryString() {
		global $pageInstance;
		$vars = array('module', 'action', 'layout', 'report_id', 'data_id', 'record', 'subpanel', 'list_limit', 'list_offset', 'parent_layout', 'parent_tab', 'debug');
		$q = array();
		foreach($vars as $v) {
			if(isset($_REQUEST[$v]) && $_REQUEST[$v] !== '')
				$q[$v] = $_REQUEST[$v];
		}
		$add = $this->getAdditionalQueryParams();
		if($add) array_extend($q, $add);
		if($this->layout_perform == 'save' && $this->report_id) {
			$q['layout'] = 'Reports';
			$q['report_id'] = $this->report_id;
		}
		if ($this->layout_save_from_admin) {
			global $currentModule;
			$q = array('module' => $currentModule, 'action' => 'index');
		}
		$add_f = $pageInstance->get_preserve_url_fields();
		foreach($add_f as $k => $v)
			if(! isset($q[$k])) $q[$k] = $v;
		return encode_query_string($q);
	}
	
	
	function getAdditionalQueryParams() {
		if(isset($this->add_query_params))
			return $this->add_query_params;
	}


	// used by popup_related_data JSON method
	function fetchJsonData($rows, $fields, $acl_filter='list', $add_filters=null) {
		$fmt =& $this->getFormatter();

		$fmt->setFormat('json');
		/** @var $lq ListQuery */
		$lq =& $fmt->getQuery();
		$lq->clearFields(true);
		if($acl_filter) $lq->addAclFilter($acl_filter);

		$get_fields = array();
		// reverse field map
		foreach ($fields as $k => $v) {
			if($v == '_display')
				$dname_alias = $lq->addDisplayName(is_numeric($k) ? null : $k);
			else if(is_string($v))
				$get_fields[$k] = $v;
		}

		if($get_fields)
			$lq->addFields($get_fields, true, false, true);
		else
			$lq->addPrimaryKey('id');
			
		if($add_filters) {
			$filter =& $fmt->getFilterForm();
			$filter->addBasicFilters();
			$filter->addAutoFilters();

			$fmt->loadFilter($add_filters);
		}

		$limit = null;
		if (is_countable($rows) && sizeof($rows) > 0) {
			if ($rows !== 'all')
				$lq->addFilterPrimaryKey($rows);
		} else {
			// cannot filter, return no results
			$limit = 0;
		}
		$result = $fmt->getFormattedResult(0, $limit);
		return $result;
	}


	function showDetailInPopup() {
        if ($this->context == 'popup') {
            $popup = false;
        } else {
            $popup = AppConfig::setting("display.list.{$this->model_name}.view_in_popup", false);
        }
		return $popup;
	}
	
	function showMassUpdate() {
		if(AppConfig::setting("display.list.{$this->model_name}.mass_update.disabled"))
			return false;
		if($this->show_mass_update)
			return true;
		return false;
	}
	
	function getAdditionalClauses() {
		return [];
	}

	function formatAuditedFields($listResult) {
	}
}

?>
