<?php

require_once 'include/ListView/ListFormatter.php';

class MobileListFormatter extends ListFormatter
{
	var $show_additional_details = false;
	var $compact_pager = true;

	static function create($model = null, $link_name = null, $list_id = null, $context = 'listview', $params = null)
	{
		return new self($model, $link_name, $list_id, $context, $params);
	}


	function outputTitleHeader()
	{
		global $app_list_strings, $pageInstance;
		if ($this->context == 'unified_search' || $this->context == 'subpanel')
			return parent::outputTitleHeader();
		echo $this->filter_form->formatSearch($this->list_id, $this->module, $this->action);
	}
	
	function outputHtmlHeader()
	{
	}
	
	function renderTabs()
	{
	}

	function outputHtmlRows(ListResult &$listResult, $offset_count = 0)
	{
		/** @var $pageInstance IncludeManager */
		global $app_list_strings, $app_strings, $image_path, $pageInstance;

		$ret = array();
		$ridx = 0;

		$row_act = array();
		foreach ($this->row_actions as $k => $v)
			$row_act[] = "$k=\"$v\"";
		$row_act = implode(' ', $row_act);

		if ($listResult->failed) {
			return;
		}

		$have_rt_col = 0;
		foreach ($this->columns as &$col) {
			if (!empty($col['hidden']) || !empty($col['total']))
				continue;
			$widget = array_get_default($col, 'widget');
			if ($widget && $widget instanceof FormElement) {
				if ($widget->mobile_align == 'right')
					$have_rt_col = 1;
			}
			if(! isset($col['show_label'])) {
				$t = array_get_default($col, 'type');
				// used to fill in revenue_thismonth, revenue_lastmonth column headers in Projects
				if (isset($col['dynamic_label'])) {
					$dyn = $col['dynamic_label'];
					$cls = $dyn['class'];
					$fn = $dyn['function'];
					require_bean($cls);
					$lbl = call_user_func(array($cls, $fn), $col);
					if (isset($lbl))
						$col['vname'] = $lbl;
				}
				else if(isset($col['label']))
					$col['vname'] = $col['label'];
				if(! empty($col['field']) && $t != 'ref' && $t != 'name' && $t != 'sequence_number' && ! empty($col['vname'])) {
					$col['show_label'] = true;
					$col['label'] = translate($col['vname'], array_get_default($col, 'vname_module', $this->module));
				}
			}
		}

		$detail_img = '<span class="uii uii-info text-info" style="vertical-align: middle"></span>';
		$model_name = $this->list_query->getModelName();
		$checked_rows = 0;
        $totals = array();
		foreach ($listResult->formatted_rows as $id => &$row) {
            $field_position = 1;
			if (!$id) $id = $ridx;
			$cls = ($offset_count + $ridx) % 2 ? 'evenListRowS1' : 'oddListRowS1';
			if ($ridx) {
				echo '<tr><td class="listViewHRS1" colspan="' . $this->colspan . '"></td></tr>';
			}

			$special = array_get_default($listResult->special_rows, $id);
			if ($special == 'static')
				$cls .= ' staticRow';

			echo '<tr class="' . $cls . '" ' . $row_act . '>';
            echo '<td class="listViewTd">';
            
            $rt_col = '';
            if ($this->show_favorites) {
                $favorite_link = $this->getFavoriteLink($id, array_get_default($row, 'favorite', 0));
                $rt_col .= '<div class="listViewLineRt">' . $favorite_link . '</div>';
                $have_rt_col = true;
            }

			foreach ($this->columns as &$col) {
				if (!empty($col['hidden']) || !empty($col['total']))
					continue;
				
				$val = $this->formatRowColumn($col, $row, $listResult, $id);
                $widget = array_get_default($col, 'widget');
                if ($widget && $widget instanceof FormElement) {
					if($have_rt_col && $widget->mobile_align == 'right') {
						$rt_col .= '<div class="listViewLineRt">' . $val . '</div>';
						continue;
					}
                }
				
				if(! empty($col['show_label']) && strlen($val)) {
					$val = '<span class="listLineLabel">' . $col['label'] . $app_strings['LBL_SEPARATOR'] . '</span>' . $val;
				}

				echo '<div class="listViewLine">' . $val . '</div>';

                if (! empty($col['sum_total'])) {
                    $value = 0;
                    if (! empty($listResult->rows[$id][$col['field']])) {
                         $value = $listResult->rows[$id][$col['field']];
                    } elseif (! empty($listResult->formatted_rows[$id][$col['field']])) {
                        $value = unformat_number($listResult->formatted_rows[$id][$col['field']]);
                    }

                    if (isset($totals[$field_position]['value'])) {
                        $totals[$field_position]['value'] += $value;
                    } else {
                        $totals[$field_position]['value'] = $value;
                    }

                    if (!isset($totals[$field_position]['type']))
                        $totals[$field_position]['type'] = $col['type'];
                }

                $field_position ++;
			}
            echo '</td>';
            
            if($have_rt_col) {
				echo '<td class="listViewTd" width="10%" style="text-align: right">' . $rt_col . '</td>';
            }

			echo '</tr>';
			$ridx++;
		}

        if (sizeof($totals) > 0) {
            $empty_row = array();
            $spec = array('type' => '');
            echo '<tr class="listHead">';

            for ($pos = 1; $pos < $field_position; $pos++) {
                $val = '&nbsp;';
                if ($pos == 2 && ! isset($totals[2])) {
                    $val = $app_strings['LBL_LIST_TOTAL'];
                } elseif (isset($totals[$pos])) {
                    $spec['type'] = $totals[$pos]['type'];
                    $val = $this->field_formatter->formatRowValue($spec, $empty_row, $totals[$pos]['value']);;
                }
                echo '<td class="listViewThS1">' .$val. '</td>';
            }

            echo '</tr>';
        }

		if ($this->show_checks && $this->have_rows) {
			$num_of_rows = is_countable($listResult->formatted_rows) ? sizeof($listResult->formatted_rows) : 0;
			if ($num_of_rows > 0 && $num_of_rows == $checked_rows)
				$checked = 1;
			else
				$checked = '';
			if ($this->uids == 'all') $checked = 'all';
			$pageInstance->add_js_literal("sListView.setCheckedAll('{$this->list_id}', '$checked');", null, LOAD_PRIORITY_FOOT);
		}
	}
}
