<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/
/*********************************************************************************

 * Description:
 * Portions created by SugarCRM are Copyright (C) SugarCRM, Inc. All Rights
 * Reserved. Contributor(s): ______________________________________..
 * *******************************************************************************/


class Localization {

	// longreach - added
	var $conversionError = false;
	var $_user;

	static $config_map = array(
		'default_locale_name_format' => 'locale.defaults.name_format',
		'use_real_names' => 'locale.defaults.use_real_names',
		'default_export_charset' => 'export.charset',
		'default_export_format' => 'export.list_format',
		'default_email_charset' => 'email.default_charset',
		'default_number_format' => 'locale.defaults.number_format',
		'default_address_format' => 'locale.defaults.address_format',
		'default_date_format' => 'locale.defaults.date_format',
		'default_time_format' => 'locale.defaults.time_format',
		'default_holidays' => 'locale.defaults.holidays',
		'week_start_day' => 'locale.defaults.week_start_day',
		// for compatibility only - not a user preference
		'default_space_separate_currency' => 'locale.base_currency.space_separate',
		'default_currency_significant_digits' => 'locale.base_currency.significant_digits',
	);


	/**
	 * sole constructor
	 */
	function __construct($user = null) {
		$this->_user = $user;
	}


	/**
	 * abstraction of precedence
	 * @param string prefName Name of preference to retrieve based on overrides
	 * @param object user User in focus, default null (current_user)
	 * @param bool ignore_user If true, always return system setting
	 * @param bool ignore_empty If true, empty user preference is considered not set
	 * @return string pref Most significant preference
	 */
	function getPrecedentPreference($prefName, $user=null, $ignore_user=false, $ignore_empty = false) {
		if(! $ignore_user) {
			global $current_user;
			if($user != null)
				$userPref = $user->getPreference($prefName);
			elseif(!empty($this->_user))
				$userPref = $this->_user->getPreference($prefName);
			elseif(!empty($current_user))
				$userPref = $current_user->getPreference($prefName);
		}
		if(isset($userPref) && (!$ignore_empty || !empty($userPref)))
			return $userPref;

		if(isset(self::$config_map[$prefName]))
			return AppConfig::setting(self::$config_map[$prefName]);
	}


	///////////////////////////////////////////////////////////////////////////
	////	CHARSET TRANSLATION
	/**
	 * returns a mod|app_strings array in the target charset
	 * @param array strings $mod_string, et.al.
	 * @param string charset Target charset
	 * @return array Translated string pack
	 */
	function translateStringPack($strings, $charset) {
		// handle recursive
		foreach($strings as $k => $v) {
			if(is_array($v)) {
				$strings[$k] = $this->translateStringPack($v, $charset);
			} else {
				$strings[$k] = $this->translateCharset($v, 'UTF-8', $charset);
			}
		}
		ksort($strings);
		return $strings;
	}

	/**
	 * translates the passed variable for email sending (export)
	 * @param	mixed the var (array or string) to translate
	 * @return	mixed the translated variable
	 */
	function translateForEmail($var) {
		if(is_array($var)) {
			foreach($var as $k => $v) {
				$var[$k] = $this->translateForEmail($v);
			}
			return $var;
		} elseif(!empty($var)) {
			return $this->translateCharset($var, 'UTF-8', $this->getOutboundEmailCharset());
		}
	}

	/**
	 * prepares a bean for export by translating any text fields into the export
	 * character set
	 * @param bean object A SugarBean
	 * @return bean object The bean with translated strings
	 */
    function prepBeanForExport($bean) {
        foreach($bean->getFieldDefinitions() as $k => $field) {
			$bean->$k = $this->translateCharset($bean->$k, 'UTF-8', $this->getExportCharset());
        }

        return $bean;
    }

	/**
	 * translates a character set from one encoding to another encoding
	 * @param string string the string to be translated
	 * @param string fromCharset the charset the string is currently in
	 * @param string toCharset the charset to translate into (defaults to UTF-8)
	 * @return string the translated string
	 */
	function translateCharset($string, $fromCharset, $toCharset='UTF-8', $force=false) {
		$fromCharset = $this->normalizeCharset($fromCharset);
		$toCharset = $this->normalizeCharset($toCharset);
		if ($fromCharset == $toCharset && ! $force) {
			return $string;
		}

		//$GLOBALS['log']->debug("Localization: translating [ {$string} ] into {$toCharset}");
		$this->conversionError = false;
		if(function_exists('mb_convert_encoding') && array_search($fromCharset, mb_list_encodings())) {
			// longreach - modified
			if(function_exists('mb_check_encoding') && ! @mb_check_encoding($string, $fromCharset) && ! $force) {
				$this->conversionError = true;
				return $string;
			}
			$ret = @mb_convert_encoding($string, $toCharset, $fromCharset);
			if($force) {
				return $ret;
			}
			$question = @mb_substr_count($string, '?', $fromCharset);
			if ($question != @mb_substr_count($ret, '?', $toCharset)) {
				$this->conversionError = true;
				return $string;
			} else {
				return $ret;
			}
		} elseif(function_exists('iconv')) { // iconv is flakey
			return iconv($fromCharset, $toCharset . "//IGNORE", $string);
		} else {
			return $string;
		} // end else clause
	}

	/**
	 * translates a character set from one to another, and the into MIME-header friendly format
	 */
	function translateCharsetMIME($string, $fromCharset, $toCharset='UTF-8', $encoding="Q") {
		$mimeString = $this->translateCharset($string, $fromCharset, $toCharset);
		/* longreach - removed
		return mb_encode_mimeheader($mimeString, $toCharset, $encoding);
		*/
		// longreach - start added
		if ($this->conversionError) {
			return mb_encode_mimeheader($string, 'UTF-8', $encoding);
		} else
		return mb_encode_mimeheader($string, $toCharset, $encoding);
		// longreach - end added
	}

	function normalizeCharset($charset, $use_map=true) {
    	static $map;
    	$charset = strtolower(preg_replace('/[ _]/', '-', $charset));
    	if($use_map) {
			if(! isset($map)) {
				$map = [];
				$m = AppConfig::setting('locale.map_charsets', false);
				if($m) {
					foreach($m as $from => $to) {
						$from = $this->normalizeCharset($from, false);
						$to = $this->normalizeCharset($to, false);
						$map[$from] = $to;
					}
				}
			}
			if(isset($map[$charset]))
				$charset = $map[$charset];
		}
		return strtoupper($charset);
	}

	/**
	 * returns an array of charsets with keys for available translations; appropriate for get_select_options_with_id()
	 */
	function getCharsetSelect() {
		return AppConfig::setting('locale.charsets');
	}

	/**
	 * returns the charset preferred in descending order: User, Sugar Config, DEFAULT
	 * @param bool system_default
	 * @return string charset the chosen character set
	 */
	function getExportCharset($system_default = true) {
		$charset = $this->getPrecedentPreference('default_export_charset', null, false, $system_default);
		return $charset;
	}

	function getExportFormatSelect() {
		$opts = AppConfig::setting('locale.list_export_formats');
		$ret = array();
		foreach($opts as $k => $v) {
			$lbl = array_get_default($v, 'vname', 'LBL_EXPORT_'.strtoupper($k));
			$lbl = translate($lbl, 'Configurator');
			$ret[$k] = $lbl;
		}
		return $ret;
	}

	/**
	 * returns the charset preferred in descending order: User, Sugar Config, DEFAULT
	 * @param bool system_default
	 * @return string charset the chosen character set
	 */
	function getOutboundEmailCharset($system_default = true) {
		$charset = $this->getPrecedentPreference('default_email_charset', null, false, $system_default);
		return $charset;
	}
	////	END CHARSET TRANSLATION
	///////////////////////////////////////////////////////////////////////////

	///////////////////////////////////////////////////////////////////////////
	////	NUMBER DISPLAY FORMATTING CODE
	function getDecimalSeparator() {
		$fmt = $this->getNumberFormat();
		return $fmt['dec_sep'];
	}

	function getNumberGroupingSeparator() {
		$fmt = $this->getNumberFormat();
		return $fmt['grp_sep'];
	}

	function getLocaleDefaultNumberFormat() {
		return AppConfig::setting('locale.defaults.number_format');
	}

	function getLocaleNumberFormatName($override=null) {
		$arr = $this->getNumberFormats();
		$name = $this->getPrecedentPreference('default_number_format', null, ($override == 'system'));
		if(isset($arr[$name]))
			return $name;
		return $this->getLocaleDefaultNumberFormat();
	}

	function getNumberFormat($format_name=null) {
		if(! isset($format_name))
			$format_name = $this->getLocaleNumberFormatName();
		$info = AppConfig::setting('locale.number_formats.'.$format_name);
		if(! $info)
			return AppConfig::setting('locale.number_formats.'.$this->getLocaleDefaultNumberFormat());
		return $info;
	}

	function getNumberFormats() {
		return AppConfig::setting('locale.number_formats');
	}

	function getLocaleNumberFormatOptions() {
		$arr = $this->getNumberFormats();
		$opts = array();
		foreach($arr as $k => $v)
			$opts[$k] = $this->sampleFormatNumber($k);
		return $opts;
	}

	function sampleFormatNumber($format_name=null, $number=null, $decimals=2) {
		if(! isset($number)) $number = '12345678.90';
		$fmt = $this->getNumberFormat($format_name);
		return number_format(round($number, $decimals), $decimals, $fmt['dec_sep'], $fmt['grp_sep']);
	}

	function getPrecision() {
		$precision = AppConfig::setting('locale.base_currency.significant_digits', 2);
		return $precision;
	}

	function getLocaleSpaceSeparateCurrency() {
		$separate = AppConfig::setting('locale.base_currency.space_separate', 0);
		return $separate;
	}

	/**
	 * returns a number formatted by user preference or system default
	 * @param string number Number to be formatted and returned
	 * @param string currencySymbol Currency symbol if override is necessary
	 * @param bool is_currency Flag to also return the currency symbol
	 * @return string Formatted number
	 */
	function getLocaleFormattedNumber($number, $currencySymbol='', $is_currency=true, $format=null) {
		$fnum			= $number;
		$majorDigits	= '';
		$minorDigits	= '';
		//$dec			= $this->getDecimalSeparator();
		//$thou			= $this->getNumberGroupingSeparator();
		$fmt            = $this->getNumberFormat($format);
		$dec = $fmt['dec_sep']; $thou = $fmt['grp_sep'];
		$precision		= $this->getPrecision();
		$symbol			= empty($currencySymbol) ? $this->getCurrencySymbol() : $currencySymbol;

		$exNum = explode($dec, $number);
		// handle grouping
		if(is_array($exNum) && count($exNum) > 0) {
			if(strlen($exNum[0]) > 3) {
				$offset = strlen($exNum[0]) % 3;
				if($offset > 0) {
					for($i=0; $i<$offset; $i++) {
						$majorDigits .= $exNum[0][$i];
					}
				}

				$tic = 0;
				for($i=$offset; $i<strlen($exNum[0]); $i++) {
					if($tic % 3 == 0 && $i != 0) {
						$majorDigits .= $thou; // add separator
					}

					$majorDigits .= $exNum[0][$i];
					$tic++;
				}
			} else {
				$majorDigits = $exNum[0]; // no formatting needed
			}
			$fnum = $majorDigits;
		}

		// handle decimals
		if($precision > 0) { // we toss the minor digits otherwise
			if(is_array($exNum) && isset($exNum[1])) {

			}
		}


		if($is_currency) {
			$fnum = $symbol.$fnum;
		}
		return $fnum;
	}

	/**
	 * returns Javascript to format numbers and currency for ***DISPLAY***
	 */
	function getNumberJs() {
		$json = getJSONobj();
		$number_formats = $json->encode($this->getNumberFormats());
		$out = <<<eoq
			var numberFormats = $number_formats;
			var exampleDigits = '123456789.000000';
			
			// round parameter can be negative for decimal, precision has to be postive
			function formatNumber(n, sep, dec, precision) {
				var majorDigits;
				var minorDigits;
				var formattedMajor = '';
				var formattedMinor = '';
				
				var nArray = n.split('.');
				majorDigits = nArray[0];
				if(nArray.length < 2) {
					minorDigits = 0;
				} else {
					minorDigits = nArray[1];
				}

				// handle grouping
				if(sep.length > 0) {
					var strlength = majorDigits.length;
					
					if(strlength > 3) {
						var offset = strlength % 3; // find how many to lead off by
						
						for(j=0; j<offset; j++) {
							formattedMajor += majorDigits[j];
						}
						
						tic=0;
						for(i=offset; i<strlength; i++) {
							if(tic % 3 == 0 && i != 0)
								formattedMajor += sep;
							
							formattedMajor += majorDigits[i];
							tic++;
						}
					}
				} else {
					formattedMajor = majorDigits; // no grouping marker
				}

				// handle decimal precision
				if(precision > 0) {
					for(i=0; i<precision; i++) {
						if(minorDigits[i] != undefined)
							formattedMinor += minorDigits[i];
						else
							formattedMinor += '0';
					}
				} else {
					// we're just returning the major digits, no decimal marker
					dec = ''; // just in case
				}
				
				return formattedMajor + dec + formattedMinor;
			}

			function setSigDigits() {
				var sym = document.getElementById('symbol').value;
				var fmt = document.getElementById('default_number_format').value;
				if(! fmt) fmt = 'English';
				var thou = numberFormats[fmt].grp_sep;
				var dec = numberFormats[fmt].dec_sep;
				// longreach - modified
				var precision = document.getElementById('sigDigits') ? document.getElementById('sigDigits').value : currency_significant_digits;
				//umber(n, num_grp_sep, dec_sep, round, precision) 
				var newNumber = sym + formatNumber(exampleDigits, thou, dec, precision, precision);
				// longreach - modified
				if (document.getElementById('sigDigitsExample')) document.getElementById('sigDigitsExample').value = newNumber;
			}
eoq;
		return $out;
	}

	////	END NUMBER DISPLAY FORMATTING CODE
	///////////////////////////////////////////////////////////////////////////

	///////////////////////////////////////////////////////////////////////////
	////	NAME DISPLAY FORMATTING CODE
	/**
	 * get's the Name format macro string, preferring $current_user
	 * @return string format Name Format macro for locale
	 */
	function getLocaleFormatMacro($system_default=false) {
		$format = $this->getPrecedentPreference('default_locale_name_format', null, false, $system_default);
		$supported = $this->getSupportedNameFormats();
		if(! in_array($format, $supported))
			$format = AppConfig::setting('locale.defaults.name_format');
		return $format;
	}
	function getSupportedNameFormats() {
		$formats = AppConfig::setting('locale.name_display_formats', array('s f l'));
		return $formats;
	}

	/**
	 * returns formatted name according to $current_user's locale settings
	 *
	 * @param string firstName
	 * @param string lastName
	 * @param string salutation
	 * @param string format If a particular format is desired, then pass this optional parameter as a simple string.
	 * sfl is "Salutation FirstName LastName", "l, f s" is "LastName[comma][space]FirstName[space]Salutation"
	 * @return string formattedName
	 */
	function getLocaleFormattedName($firstName, $lastName, $salutationKey='', $format=null) {
		$salutation = '';
		if(!empty($salutationKey)) {
			$salutation = translate_dd('salutation_dom', $salutationKey);
		}

        //check to see if passed in variables are set, if so, then populate array with value,
        //if not, then populate array with blank ''
		$names = array();
		$names['f'] = trim($firstName ?? '');
		$names['l'] = trim($lastName ?? '');
		$names['s'] = trim($salutation ?? '');
		if(! strlen(($firstName ?? '') . ($lastName ?? '')))
			return '';

		if(! $format) $format = $this->getLocaleFormatMacro();

		// parse name format
		$formattedName = '';
		for($i=0; $i < strlen($format); $i++) {
			// longreach - modified - PHP 5.1+ compatability
			$k = $format[$i];
			$formattedName .= array_key_exists($k, $names) ? $names[$k] : $k;
		}
		$formattedName = str_replace('  ', ' ', trim($formattedName, " \t\n\r,"));

		return $formattedName;
	}

	function getLocaleExampleName() {
		return array(
			'salutation' => translate('LBL_LOCALE_NAME_EXAMPLE_SALUTATION', 'app'),
			'first_name' => translate('LBL_LOCALE_NAME_EXAMPLE_FIRST', 'app'),
			'last_name' => translate('LBL_LOCALE_NAME_EXAMPLE_LAST', 'app'),
		);
	}

	function getLocaleSampleName() {
		return array(
			'salutation' => translate('LBL_LOCALE_DESC_SALUTATION', 'Users'),
			'first_name' => translate('LBL_LOCALE_DESC_FIRST', 'Users'),
			'last_name' => translate('LBL_LOCALE_DESC_LAST', 'Users'),
		);
	}


	/* Currently used by installer localization.php */
	function getLocaleNameFormatSelector($sel=null) {
		$formats = $this->getSupportedNameFormats();
		$default = $this->getLocaleFormatMacro(true);
		$sample = $this->getLocaleSampleName();
		$example = $this->getLocaleExampleName();
		if($sel || ! in_array($formats, $sel))
			$sel = $default;
		$json = getJSONobj();
		$opts = array();
		$samples = array();
		foreach($formats as $k) {
			$opts[$k] = $this->getLocaleFormattedName($example['first_name'], $example['last_name'], $example['salutation'], $k);
			$samples[$k] = $this->getLocaleFormattedName($sample['first_name'], $sample['last_name'], $sample['salutation'], $k);
		}
		$formats_js = $json->encode($samples);
		$script = <<<EOS
<script type="text/javascript">
	if (typeof(update_name_format) != 'function') {
		update_name_format = function(sel, div_id) {
			if (typeof(sel) == 'undefined') {
				sel = def_name_format;
			}
			var div = document.getElementById(div_id);
			if(div) {
				if(typeof(name_formats[sel]) != 'undefined')
					div.innerHTML = name_formats[sel];
				else
					div.innerHTML = name_formats[def_name_format];
			}
		};
	}
	
	if (typeof(name_formats) == 'undefined') {
		var name_formats = {};
		var def_name_format = {};
	}
	name_formats = {$formats_js};
	def_name_format = '$default';
</script>
EOS;
		$ret = array(
			'OPTIONS' => get_select_options_with_id($opts, $sel),
			'JAVASCRIPT' => $script,
			'SAMPLE' => $samples[$sel],
		);
		return $ret;
	}
	////	END NAME DISPLAY FORMATTING CODE
	///////////////////////////////////////////////////////////////////////////



	// longreach - start added
	function getLocaleAddressFields() {
		static $fields = array(
			'contact_id', 'contact_name', 'contact_salutation', 'department',
			'account_id', 'account_name',
			'contact_name_owner', 'account_name_owner',
			'address_street', 'address_city', 'address_state', 'address_postalcode', 'address_country',
		);
		return $fields;
	}
	// --
	function getLocaleAddressFormats() {
		return AppConfig::setting('locale.address_formats');
	}
	// --
	function getLocaleAddressShortFields() {
		return array(
			's' => 'address_street',
    		'c' => 'address_city',
    		't' => 'address_state',
    		'o' => 'address_country',
			'p' => 'address_postalcode',
			'a' => 'account_name',
			'n' => 'contact_name',
			'd' => 'department',
			'h' => 'phone',
			'f' => 'fax',
			'l' => 'contact_salutation',
		);
	}
	// --
	function getLocaleAddressTemplate($format='display', $name=null) {
		$arr = $this->getLocaleAddressFormats();
		$tpl = $arr[$this->getLocaleAddressFormatName($name)];
		if(isset($tpl[$format]))
			$template = $tpl[$format];
		else
			$template = $tpl['display'];
		return $template;
	}
	// --
	function getLocaleDefaultAddressFormat() {
		return AppConfig::setting('locale.defaults.address_format');
	}
	// --
	function getLocaleAddressFormatName($override=null) {
		$arr = $this->getLocaleAddressFormats();
		if(isset($override) && $override != 'system')
			$name = $override;
		else
			$name = $this->getPrecedentPreference('default_address_format', null, ($override == 'system'));
		if(isset($arr[$name]))
			return $name;
		return $this->getLocaleDefaultAddressFormat();
	}
	// --
	function getLocaleAddressFormatOptions($format='display') {
		$arr = $this->getLocaleAddressFormats();
		$opts = array();
		foreach($arr as $k => $v)
			$opts[$k] = $this->getLocaleAddressTemplate($format, $k);
		return $opts;
	}
	// --
	function getLocaleBeanFormattedAddress(&$row_result, $prefix='', $format='display', $template=null, $addMapLink = false) {
		global $app_strings, $image_path;
		$info = array();
		foreach($this->getLocaleAddressFields() as $v) {
			$f = "{$prefix}{$v}";
			$val = $row_result->getField($f);
			if(isset($val))
				$info[$v] = $val;
		}
		$ret = $this->getLocaleFormattedAddress($info, '', $format, $template, $addMapLink);
		return $ret;
	}
	// --
	function getLocaleFormattedAddress($addr_info, $prefix='', $format='display', $template=null, $addMapLink=false) {
		global $app_list_strings;

		if($template === null)
			$template = $this->getLocaleAddressTemplate($format);
		$rfrom = $rto = array();
		foreach($this->getLocaleAddressShortFields() as $k=>$v) {
			$rfrom[] = '{'.$k.'}'; $rto[] = '{'.$v.'}';
		}
		$template = str_replace($rfrom, $rto, $template);
		$html = ($format == 'display' || $format == 'display_list');
		if($html) {
			$ct_tag = array_get_default($addr_info, 'contact_name_tag', 'a');
			$ct_link = '<'.$ct_tag.' href="index.php?module=Contacts&action=DetailView&record={contact_id}" onclick="SUGAR.util.handleLink(event)" class="tabDetailViewDFLink">{contact_name}</'.$ct_tag.'>';
			$ct_link = array_get_default($addr_info, 'contact_link', $ct_link);
			$ct_link .= '&nbsp;<div class="input-icon theme-icon bean-Contact"></div>';
			if(! empty($addr_info[$prefix.'contact_field'])) {
				$info_id = 'DetailForm_' . $addr_info[$prefix.'contact_field'] . '_info';
				$ct_link = '<span id="' . $info_id . '" '
					. 'onmouseout="return SUGAR.util.clearAdditionalDetailsCall()" '
					. "onmouseover=\"return SUGAR.util.getAdditionalDetails('Contact', '{contact_id}', '$info_id', null, 'view', 750)\" "
					. '>' . $ct_link . '</span>';
			}
			if(! empty($addr_info[$prefix.'contact_id']))
				$template = str_replace('{contact_name}', $ct_link, $template);

			$ac_tag = array_get_default($addr_info, 'account_name_tag', 'a');
			$ac_link = '<'.$ac_tag.' href="index.php?module=Accounts&action=DetailView&record={account_id}" onclick="SUGAR.util.handleLink(event)" class="tabDetailViewDFLink">{account_name}</'.$ac_tag.'>';
			$ac_link = array_get_default($addr_info, 'account_link', $ac_link);
			$ac_link .= '&nbsp;<div class="input-icon theme-icon bean-Account"></div>';
			if(! empty($addr_info[$prefix.'account_field'])) {
				$info_id = 'DetailForm_' . $addr_info[$prefix.'account_field'] . '_info';
				$ac_link = '<span id="' . $info_id . '" '
					. 'onmouseout="return SUGAR.util.clearAdditionalDetailsCall()" '
					. "onmouseover=\"return SUGAR.util.getAdditionalDetails('Account', '{account_id}', '$info_id', null, 'view', 750)\" "
					. '>' . $ac_link . '</span>';
			}
			if(! empty($addr_info[$prefix.'account_id']))
				$template = str_replace('{account_name}', $ac_link, $template);
		}
		$tpl = strtr($template, '|', "\n");
		$tpl = str_replace(array('[', ']'), array('[~', '~]'), $tpl);
		$fields = $this->getLocaleAddressFields();
		$keys = $vals = array();
		foreach($fields as $k) {
			$keys[] = '{'.$k.'}';
			$val = array_get_default($addr_info, $prefix.$k, '');
			if($k == 'contact_name') {
				$sal = array_get_default($addr_info, $prefix.'contact_salutation', '');
				if($sal)
					$sal = isset($GLOBALS['app_list_strings']['salutation_dom'][$sal]) ? $GLOBALS['app_list_strings']['salutation_dom'][$sal] : $sal;
				if($sal) $val = $sal . ' ' . $val;
			}
			$vals[] = $val;
		}
		$addr = str_replace($keys, $vals, $tpl);
		$lines = explode("\n", $addr);
		$ret = '';
		$csv = ($format == 'csv' || $format == 'tsv' || $format == 'scsv');
		$between = $html ? '<br />' : ($format == 'display_company' ? ' • ' : ($csv ? ', ' : "\n"));
		foreach($lines as $l) {
			while(preg_match('/^\s*\[~[^\]]*?~\](.*)$/', $l, $m) || preg_match('/^(.*?)\[~[^\]]*?~\]\s*$/', $l, $m))
				$l = $m[1];
			$l = preg_replace('/\[~|~\]/', '', $l);
			if(strlen(trim($l))) {
				if($ret) $ret .= $between;
				$ret .= $l;
			}
		}
		if($html) $ret = preg_replace('/  /', '&nbsp; ', $ret);

		if ($html && $addMapLink && $format != 'display_list' && ! empty($addr_info[$prefix.'address_city']) && ! empty($addr_info[$prefix.'module']) && ! empty($addr_info[$prefix.'record'])) {
			require_once('include/javascript/gmaps.php');
			if(gmaps_enabled()) {
				global $image_path, $app_strings;
				gmaps_add_includes();
				$bean_mod = $addr_info[$prefix.'module'];
				$bean_id = $addr_info[$prefix.'record'];
				if($addMapLink === 'text')
					$ret .= "<p><a href=\"#\" onclick=\"IAHMap.fetch('" . $bean_mod . "', '" . $bean_id . "', '$prefix'); return false;\">". $app_strings['LBL_SHOW_ON_MAP']  ."</a></p>";
				else {
					$maplink = "<a href=\"#\" onclick=\"IAHMap.fetch('" . $bean_mod . "', '" . $bean_id . "', '$prefix'); return false;\">";
					$w = $h = '';
					if(AppConfig::is_mobile()) $w = $h = 20;
					$maplink .= get_image($image_path.'ShowOnMap', 'title="'.$app_strings['LBL_SHOW_ON_MAP'].'" border="0"', $w, $h);
					$maplink .= '</a>';
					$maplink = '<div style="float: right; clear: right; padding: 0 5px">' . $maplink . '</div>';
					$ret = $maplink.$ret;
				}
			}
		}

		return $ret;
	}
	// --
	function getLocaleSampleAddress($format='display') {
		if(defined('IAH_IN_INSTALLER')) {
			$app_strings = $GLOBALS['mod_strings'];
		} else {
			global $app_strings;
		}
		$addr = array(
			'account_name' => $app_strings['LBL_ACCOUNT_NAME_FORMAT'],
			'contact_name' => $app_strings['LBL_CONTACT_NAME_FORMAT'],
			'address_street' => $app_strings['LBL_STREET_FORMAT'],
			'address_city' => $app_strings['LBL_CITY_FORMAT'],
			'address_state' => $app_strings['LBL_STATE_FORMAT'],
			'address_postalcode' => $app_strings['LBL_POSTAL_CODE_FORMAT'],
			'address_country' => $app_strings['LBL_COUNTRY_FORMAT'],
			'phone' => $app_strings['LBL_PHONE_NUMBER_FORMAT'],
			'fax' => $app_strings['LBL_FAX_NUMBER_FORMAT'],
		);
		if($format == 'display_company') {
			$addr['account_name'] = $app_strings['LBL_COMPANY_NAME_FORMAT'];
			$addr['contact_name'] = '';
		}
		return $addr;
	}
	// --
	function translateAddressFormatName($override=null) {
		if(defined('IAH_IN_INSTALLER')) {
			$app_list_strings = $GLOBALS['mod_strings'];
		} else {
			global $app_list_strings;
		}
		$name = $this->getLocaleAddressFormatName($override);
		$labels = array_get_default($app_list_strings, 'address_format_names_dom', array());
		return array_get_default($labels, $name, $name);
	}
	// --
	function getLocaleAddressFormatInfo($format='display', $templates=null) {
		if(defined('IAH_IN_INSTALLER')) {
			$app_list_strings = $GLOBALS['mod_strings'];
		} else {
			global $app_list_strings;
		}
		if(! $templates)
			$templates = $this->getLocaleAddressFormatOptions($format);
		$opts = array();
		$smpl = array();
		$addr = $this->getLocaleSampleAddress($format);
		$names = array_get_default($app_list_strings, 'address_format_names_dom', array());
		foreach($templates as $k=>$v) {
			$opts[$k] = array_get_default($names, $k, $k);
			$smpl[$k] = $this->getLocaleFormattedAddress($addr, '', $format, $v);
		}
		$ret = array(
			'options' => $opts,
			'samples' => $smpl,
		);
		return $ret;
	}

	/* Currently used by installer localization.php */
	function getLocaleAddressFormatSelector($sel='', $format='display', $templates=null, $format_id = 'default') {
		$info = $this->getLocaleAddressFormatInfo($format, $templates);
		$default = $this->getLocaleDefaultAddressFormat();
		$json = getJSONobj();
		$info['samples'][''] = $info['samples'][$default];
		$formats_js = $json->encode($info['samples']);
		$script = <<<EOS
<script type="text/javascript">
	if (typeof(update_address_format) != 'function') {
		update_address_format = function(sel, div_id, format_id) {
			if (!format_id) format_id = 'default';
			if (typeof(sel) == 'undefined') {
				sel = '$default';
			}
			var div = document.getElementById(div_id);
			if(div) {
				if(typeof(address_formats[format_id][sel]) != 'undefined')
					div.innerHTML = address_formats[format_id][sel];
				else
					div.innerHTML = address_formats[format_id][def_address_format[format_id]];
			}
		};
	}
	
	if (typeof(address_formats) == 'undefined') {
		var address_formats = {};
		var def_address_format = {};
	}
	address_formats['$format_id'] = {$formats_js};
	def_address_format['$format_id'] = '$default';
</script>
EOS;
		if(! isset($info['samples'][$sel]))
			$sel = $default;
		$ret = array(
			'OPTIONS' => get_select_options_with_id($info['options'], $sel),
			'JAVASCRIPT' => $script,
			'SAMPLE' => $info['samples'][$sel],
		);
		return $ret;
	}

	// --


	function getUserFormatCity($fields) {
		// needs localizing
		if(! is_array($fields))
			return $fields;
		$city = $state = $country = '';
		$c = count($fields);
		if($c > 0) $city = $fields[0];
		if($c > 1) $state = $fields[1];
		if($c > 2) $country = $fields[2];
		$test = array($city, $state, $country);
		$ret = implode(', ', array_filter($test));
		return $ret;
	}

    function getUserFormatWeighty($fields) {
        if(! is_array($fields))
            return $fields;
        $unit_1 = $unit_2 = '';
        $c = count($fields);
        if($c > 0) $unit_1 = $fields[0];
        if($c > 1) $unit_2 = $fields[1];
        $ret = '';
        if ($unit_1 != '')
            $ret = $unit_1 . translate('LBL_PRODUCT_WEIGHT_UNIT_1', 'ProductCatalog');
        if ($unit_2 != '')
            $ret .= ' '. $unit_2 .translate('LBL_PRODUCT_WEIGHT_UNIT_2', 'ProductCatalog');
        return $ret;
    }

	function getUserFormatPhone($phone, $params=null) {
		if(! is_array($params))
			$params = array();
		return make_call_link($phone, $params);
	}

	function getUserFormatEmail($email, $recip_name='', $parent_type='', $parent_id='', $params=null) {
		global $current_user;

		if(! $email)
			return '';

		if(! is_array($params))
			$params = array();
		$context = array_get_default($params, 'context');
		$client = $current_user->getPreference('email_link_type');
		if (!$client)
			$client = AppConfig::setting('email.default_client');
		if($context == 'export') $client = 'mailto';
		if($recip_name)
			$to_addrs = "$recip_name <$email>";
		else
			$to_addrs = $email;
		$class = array_get_default($params, 'link_class', 'tabDetailViewDFLink');
		$max_len = array_get_default($params, 'max_length');
		if($max_len) $max_len = max(20, $max_len);
		if($context == 'export') $max_len = 0;

		if(! empty($params['use_recip_name']) && $recip_name) {
			$display = $recip_name;
			if($max_len) $display = str_limit_length($display, $max_len);
			$display = to_html($display);
		}
		else if($context != 'view' && ($max_len && strlen($email) > $max_len)) {
			// try to keep domain name
			$p = strpos($email, '@');
			if( ($no_addr = ($p === false)) )
				$p = 0;
			$addr = substr($email, 0, $p);
			$p2 = strrpos($email, '.', $p);
			if($p2 === false)
				$p2 = strlen($email);
			$domain = substr($email, $p, $p2 - $p);
			$ext = substr($email, $p2);
			$max_len = max(10, $max_len - strlen($ext));
			if($p2 - $p > $max_len) {
				// keep 3 chars of addressee
				if($no_addr)
					$domain = to_html(substr($domain, $p2 - $max_len)) . '&hellip;';
				else {
					$addr_len = min(3, $p);
					$max_len -= $addr_len;
					$domain = to_html(substr($domain, 0, $max_len - $p2 + $p)) . '&hellip;';
					$addr = to_html(substr($addr, 0, $addr_len)) . '&hellip;';
				}
			} else {
				$max_len = max(3, $max_len - $p2 + $p - 1);
				$addr = to_html(substr($addr, 0, $max_len)) . '&hellip;';
				$domain = to_html($domain);
			}
			$display = $addr . $domain . to_html($ext);
			$title = to_html($email);
		} else
			$display = to_html($email);

		if (AppConfig::is_mobile()) {
			if(!AppConfig::is_mobile_module('Emails'))
				$client = '';
		}
		//$title = ! empty($title) ? " title=\"$title\"" : '';
        $title = ! empty($title) ? ' onmouseover="return SUGAR.popups.tooltip(\''.$title.'\', this);"' : '';
		if($client == 'sugar') {
			return '<a href="index.php?module=Emails&action=EditView&type=out'.
				'&contact_id='.array_get_default($params, 'contact_id', '').
				'&parent_type='.$parent_type.
				'&parent_id='.$parent_id.
				'&to_email_addrs='.urlencode($to_addrs).
				'&return_module='.array_get_default($params, 'return_module', '').
				'&return_action='.array_get_default($params, 'return_action', '').
				'&return_id='.array_get_default($params, 'return_id', '').
				'" class="'.$class.'"'.$title.'>' . $display . '</a>';
		} else {
			// straight mailto:
			return '<a href="mailto:'.urlencode($to_addrs).'" class="'.$class.'"'.$title.'>' . $display . '</a>';
		}
	}

	function getUserFormatExtUrl($value, $params=null) {
		if(! $value)
			return '';
		if(! is_array($params))
			$params = array();
		$new_window = array_get_default($params, 'new_window', true);
		$link_class = array_get_default($params, 'link_class', 'tabDetailViewDFLink');
		$title = array_get_default($params, 'title');
		if(! preg_match('~^\w+:~', $value)) {
			$display = $value;
			$value = 'http://' . $value;
		} else {
			$display = preg_replace('~^https?:/?/?~i', '', $value);
		}

		// may want to add user preference - open external URL in new window?
		$targ = $new_window ? ' target="_blank"' : '';
		$title = $title ? ' title="'.to_html($title).'"' : '';
		$max_len = array_get_default($params, 'max_length');

		if($max_len && $max_len < strlen($display)) {
			$display = to_html(substr($display, 0, $max_len - 1)) . '&hellip;';
		} else
			$display = to_html($display);

		return '<a href="'.to_html($value).'" class="'.$link_class.'"'.$targ.$title.'>'.$display.'</a>';
	}

	function getUseRealNames($system_default=false) {
		$pref = $this->getPrecedentPreference('use_real_names', null, false, $system_default);
		if($pref && $pref !== 'off')
			return 1;
		return 0;
	}

	function getWeekStartDay($system_default=false) {
		return $this->getPrecedentPreference('week_start_day', null, false, $system_default);
	}

	function getFiscalYearStart() {
		return AppConfig::setting('company.fiscal_year_start');
	}

	function getCurrencySymbol() {
        return AppConfig::setting('locale.base_currency.symbol');
	}


} // end class def


class encoding_filter extends php_user_filter {
	var $fromCharset = 'utf-8';
	var $toCharset = 'utf-8';

	function filter($in, $out, &$consumed, $closing): int {
		global $locale;
		// $GLOBALS['log']->fatal("conv from $this->fromCharset to $this->toCharset");
		while ($bucket = stream_bucket_make_writeable($in)) {
			$consumed += $bucket->datalen;
			if($this->fromCharset != $this->toCharset) {
				$bucket->data = $locale->translateCharset($bucket->data, $this->fromCharset, $this->toCharset, true);
					//@mb_convert_encoding($bucket->data, $this->toCharset, $this->fromCharset);
			}
			stream_bucket_append($out, $bucket);
		}
		return PSFS_PASS_ON;
	}

	function onCreate(): bool {
		if(substr($this->filtername, 0, 9) === 'encoding.') {
			global $locale;
			$this->fromCharset = $locale->normalizeCharset($this->fromCharset);
			$this->toCharset = $locale->normalizeCharset(substr($this->filtername, 9));
			return true;
		}
		return false;
	}

	static function register() {
		static $done = false;
		if(! $done) {
			$done = true;
			return stream_filter_register('encoding.*', 'encoding_filter');
		}
		return $done;
	}
}


?>
