<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');



 /*********************************************************************************
 * $Id: EmailClientIMAPPEAR.php 7685 2011-04-30 18:08:37Z andrew $
 ********************************************************************************/

require_once 'Net/IMAP.php';

class _Net_IMAP extends Net_IMAP {
	function __construct($host = 'localhost', $port = 143, $enableSTARTTLS = true)
	{
		Net_IMAPProtocol::__construct(); // skip auto connect
	}
}

class EmailClientIMAPPEAR
{
	var $mbox;
    var $Params;
    var $uids = array();
    var $maxUid = '';
    var $lastError;

    function __construct($params)
    {
        $this->Params = $params;
    }

    function connect()
    {
        $host = $this->Params['host'];
        if ($this->Params['use_ssl']) $host = 'ssl://' . $host;
        $this->mbox = new _Net_IMAP();
        $ret = $this->mbox->connect($host, $this->Params['port']);
        if(PEAR::isError($ret)) {
        	$this->lastError = $ret->getMessage();
        	$ret = $this->mbox = null;
        }
        return $ret;
    }

    function login()
    {
    	if(! $this->mbox) return false;
        $ret = $this->mbox->login($this->Params['username'], $this->Params['password']);
        if(PEAR::isError($ret)) {
        	$this->lastError = $ret->getMessage();
        	return false;
        }
        return $ret;
    }

    function numMsg($since=null)
    {
    	if(! $this->mbox) return false;
        //$lastSeen = max(1, (int)$this->Params['last_check'] + 1);
        if($since) $since = ' SINCE '.$since;
        else $since = '';
        $response = $this->mbox->cmdUidSearch('UID 1:*'.$since);
        if (PEAR::isError($response)) {
        	$this->lastError = $response->getMessage();
            return false;
        }
        if ($response['RESPONSE']['CODE'] != 'OK') {
        	$this->lastError = $response['RESPONSE']['STR_CODE'];
            return false;
        }
        $this->uids = $response['PARSED']['SEARCH']['SEARCH_LIST'];
		sort($this->uids);
        return count($this->uids);
    }

    function disconnect()
    {
    	if($this->mbox) {
        	$ret = $this->mbox->disconnect(true);
        	$this->mbox = null;
        	return $ret;
        }
    }

    function getUID($msgNum)
    {
        return isset($this->uids[$msgNum-1]) ? $this->uids[$msgNum-1] : '';
    }

    function getMsg($msgNum)
    {
    	if(! $this->mbox) return false;
        $id = $this->uids[$msgNum-1];
        $response = $this->mbox->cmdUidFetch($id, "(FLAGS BODY.PEEK[])");
        if (PEAR::isError($response)) {
        	$this->lastError = $response->getMessage();
        	return false;
        }
        if($response['RESPONSE']['CODE'] != 'OK') {
        	$this->lastError = $response['RESPONSE']['STR_CODE'];
            return false;
        }

		$hd = $content = '';
		$flags = array();
        foreach ($response['PARSED'] as &$parsed) {
			if(isset($parsed['EXT']['BODY[]']))
				$content .= $parsed['EXT']['BODY[]']['CONTENT'];
			if(isset($parsed['EXT']['FLAGS']))
				$flags = array_merge($flags, $parsed['EXT']['FLAGS']);
		}
		foreach(array('Seen', 'Draft', 'Deleted', 'Answered', 'Flagged') as $h)
			$hd .= "X-Flag-$h: ".(in_array('\\'.$h, $flags) ? '1' : '0')."\n";

        $this->maxUid = $id;
		if (! empty($this->Params['mark_read'])) {
			$this->mbox->setFlags($id, '\Seen', 'add', true);
		}
		
        return $hd . $content;
    }

    function getHeaders($msgNum)
    {
    	if(! $this->mbox) return false;
        $id = $this->uids[$msgNum-1];
        $ret = $this->mbox->cmdUidFetch($id, 'BODY.PEEK[HEADER]');
        if (PEAR::isError($ret)) {
        	$this->lastError = $ret->getMessage();
        	return false;
        }
        if(strtoupper(@$ret['RESPONSE']['CODE']) != 'OK') {
        	$this->lastError = @$ret['RESPONSE']['STR_CODE'];
            return false;
        }
        return $ret['PARSED'][0]['EXT']['BODY[HEADER]']['CONTENT'];
    }

    function deleteMsg($msgNum)
    {
    	if(! $this->mbox) return false;
        $id = $this->uids[$msgNum-1];
        $dataitem="+FLAGS.SILENT";
        $value="\Deleted";
        $ret = $this->mbox->cmdUidStore($id,$dataitem,$value);
        if(PEAR::isError($ret)) {
        	$this->lastError = $ret->getMessage();
        	return false;
        }
        if(strtoupper(@$ret["RESPONSE"]["CODE"]) != "OK"){
        	$this->lastError = @$ret['RESPONSE']['STR_CODE'];
            return false;
        }
        return true;
    }

    function selectBox($boxName)
    {
    	if(! $this->mbox) return false;
        $ret = $this->mbox->selectMailbox($boxName);
        if (PEAR::isError($ret)) {
        	$this->lastError = $ret->getMessage();
            return false;
        }
        return true;
    }

    function getCheckDate()
    {
        return $this->maxUid;
    }


	function getFolders() {
		return [];
	}
}
