<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');



 /*********************************************************************************
 * $Id: EmailClientIMAPPHP.php 6034 2009-05-04 21:48:09Z andrew $
 ********************************************************************************/

class EmailClientIMAPPHP
{
    var $Params;
    var $mbox;
    var $maxUid;
    var $imapErrors;
    var $lastError;
	var $uids;

    function __construct($params)
    {
        $this->Params = $params;
    }

    function connect()
	{
		static $nocert_hosts = array();
		$ssl = $this->Params['use_ssl'] ? '/ssl/novalidate-cert' : '/novalidate-cert/notls';
		if ($this->Params['use_ssl'] && isset($nocert_hosts[$this->Params['host']])) {
			$ssl = '';
		}
		$folder = mb_convert_encoding($this->Params['folder'], "UTF7-IMAP", "UTF-8");

		$retry = 2;
		$retry_params = array();
		
		if (AppConfig::setting('email.compatibility.disable_gssapi') || !empty($this->Params['disable_gssapi']))
			$retry_params['DISABLE_AUTHENTICATOR'] = 'GSSAPI';

		if($this->Params['host'] == 'outlook.office365.com') {
			// save some time
			$retry_params = array('DISABLE_AUTHENTICATOR' => 'PLAIN');
		}
		
		$conn_timeout = array_get_default($this->Params, 'connect_timeout', AppConfig::setting('email.connect_timeout', 30));
		imap_timeout(IMAP_OPENTIMEOUT, $conn_timeout);

		while ($retry) {
			$this->imapErrors = imap_errors();
			$this->mbox = @imap_open("{{$this->Params['host']}:{$this->Params['port']}$ssl}{$folder}", $this->Params['username'], $this->Params['password'], 0, 1, $retry_params);
			if (empty($this->mbox) && !isset($nocert_hosts[$this->Params['host']])) {
				$this->lastError = imap_last_error();
				if (strpos($this->lastError, 'invalid remote specification') !== false) {
					$nocert_hosts[$this->Params['host']] = 1;
					$this->mbox = @imap_open("{{$this->Params['host']}:{$this->Params['port']}}{$folder}", $this->Params['username'], $this->Params['password'], 0, 1, $retry_params);
				}
			}
			if (!empty($this->mbox)) {
				break;
			}
			$retry --;
			if ($retry == 1)
				$retry_params = array('DISABLE_AUTHENTICATOR' => 'PLAIN');
		}
		$this->lastError = imap_last_error();
		$this->imapErrors = imap_errors();
		if(! $this->mbox || strpos($this->lastError, 'Unknown Mailbox') !== false)
			return false;
        return true;
	}

    function login()
    {
        return !empty($this->mbox);
    }

    function numMsg($since=null)
    {
    	if($since) $since = ' SINCE '.$since;
    	else $since = '';
		$this->imapErrors = imap_errors(); // clear any pending errors, including "Mailbox is empty"
        $this->uids = imap_search($this->mbox, 'ALL'.$since, SE_UID);
		$this->lastError = imap_last_error();
        if($this->lastError) {
        	$this->imapErrors = imap_errors();
        	return false;
		}
		if($this->uids)
			sort($this->uids);
        return is_countable($this->uids) ? count($this->uids) : 0;
    }

    function disconnect()
    {
    	$this->imapErrors = imap_errors(); // clear any pending errors, including "Mailbox is empty"
        return imap_close($this->mbox, CL_EXPUNGE);
    }

    function getUID($msgNum)
    {
        return $this->uids[$msgNum-1];
    }

    function getMsg($msgNum)
    {
        $id = imap_msgno($this->mbox, $this->uids[$msgNum-1]);
		if (!$id) return false;

		$options = 0;

		if (empty($this->Params['mark_read'])) {
			$options |= FT_PEEK;
		}
		
		$hd = '';
		$info = imap_headerinfo($this->mbox, $id);
		if($info) {
			// add these IMAP flags as fake headers
			$seen = ($info->Recent == 'R' || ($info->Unseen == ' ' && $info->Recent != 'N')) ? '1' : '0';
			$hd .= "X-Flag-Seen: $seen\n";
			$hd .= "X-Flag-Draft: ". ($info->Draft == 'X' ? '1' : '0')."\n";
			$hd .= "X-Flag-Deleted: ". ($info->Deleted == 'D' ? '1' : '0')."\n";
			$hd .= "X-Flag-Answered: ". ($info->Answered == 'A' ? '1' : '0')."\n";
			$hd .= "X-Flag-Flagged: ". ($info->Flagged == 'F' ? '1' : '0')."\n";
		}
				
		$body = $hd . imap_fetchheader($this->mbox, $id);
	    $this->lastError = imap_last_error();
	    if(!$this->lastError) {
			$body .= imap_body($this->mbox, $id, $options);
	    	$this->lastError = imap_last_error();
		}
	    if($this->lastError) {
		    $this->imapErrors = imap_errors();
		    return false;
	    }

	    $this->maxUid = $id;
        return $body;
    }

	function getHeaders($msgNum)
	{
        $id = imap_msgno($this->mbox, $this->uids[$msgNum-1]);
        if (!$id) return false;
        $headers = imap_fetchheader($this->mbox, $id);
        $this->maxUid = $id;
        return $headers;
	}

    function deleteMsg($msgNum)
    {
        $id = imap_msgno($this->mbox, $this->uids[$msgNum-1]);
        if (!$id) return false;
        return imap_delete($this->mbox, $id);
    }

    function selectBox($boxName)
    {
		//return imap_reopen($this->mbox, $boxName);
		return true;
    }

    function getCheckDate()
    {
        return $this->maxUid;;
    }

	function getFolders() {
		return [];
	}
}
