<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');

use Webklex\PHPIMAP\ClientManager;
use Webklex\PHPIMAP\Exceptions\ConnectionFailedException;
use Webklex\PHPIMAP\IMAP;
use League\OAuth2\Client\Token\AccessToken;

require_once "include/utils/oauth_utils.php";

class EmailClientIMAPPHPIMAP
{
	var $mbox;
    var $Params;
    var $uids = array();
    var $maxUid = '';
	var $lastError;
	var $token;
	var $folder;

    function __construct($params)
    {
        $this->Params = $params;
    }

    function connect()
    {
		$host = $this->Params['host'];
		$opts = [
			'host' => $this->Params['host'],
			'port' => $this->Params['port'],
			'encryption' => $this->Params['use_ssl'] ? 'ssl' : false,
			'username' => $this->Params['username'], 
			'password' => $this->Params['password'],
			'protocol' => 'imap',
		];

		if ($this->Params['imap_auth_type'] == 'o365' || $this->Params['imap_auth_type'] == 'gmail') {
			$token_src = $this->Params['o365_auth_token'];
			if (!is_array($token_src)) {
				$this->setLastError(new Exception('Access token is missing'));
				return null;
			}
			unset($token_src['expires_in']);
			try {
				$token = new AccessToken($token_src);
				$accessToken = $token->getToken();
			} catch (Exception $e) {
				$this->setLastError($e);
				$ret = $this->mbox = $this->folder = null;
				return $ret;
			}
			if ($token->hasExpired()) {
				try {
					$token =  refershOauth2Token(
						$this->Params['imap_auth_type'], 
						[
							'imap_account',
                	        $this->Params['mail_account_id']
						],
						$token->getRefreshToken()
					);
					$accessToken = $token['access_token'];
				} catch (Exception $e) {
					$this->setLastError($e);
					return null;
				}
			}
			$opts['authentication'] = "oauth";
			$opts['password'] = $accessToken;
		}


		$cm = new ClientManager;
		$client = $cm->make($opts);
		try {
			$client->connect();
			$this->mbox = $client;
			$folder_name = $this->Params['folder'];
			/*
			if (strtolower($folder_name) == 'inbox') {
				if ($this->Params['imap_auth_type'] == 'o365')
					$folder_name = 'Inbox';
				if ($this->Params['imap_auth_type'] == 'gmail')
					$folder_name = 'INBOX';
			}
			 */
			$ret = true;
			$path =  mb_convert_encoding($folder_name, "UTF7-IMAP", "UTF-8");
			$this->folder = $client->getFolderByPath($path);
		} catch (ConnectionFailedException $e) {
			$this->setLastError($e);
        	$ret = $this->mbox = $this->folder = null;
		}
        return $ret;
    }

    function login()
    {
		if(! $this->mbox) return false;
		return $this->mbox->isConnected();
    }

    function numMsg($since=null)
    {
		if(! $this->folder) return false;
		$uids = [];
		try {
			$q = $this->folder->query()->where([["CUSTOM UID 1:*"]]);
			if ($since) $q = $q->since($since);
			$q = $q->all()->limit(500);
			$q->fetchOrderDesc();
			$raw_query = $q->generate_query();
			$uids = $this->mbox->getConnection()->search([$raw_query], IMAP::ST_UID);
		} catch(Exception $e) {
			$this->setLastError($e);
			return false;
		}
		sort($uids);
        $this->uids = $uids;
        return count($this->uids);
    }

    function disconnect()
    {
    	if($this->mbox) {
        	$ret = $this->mbox->disconnect(true);
        	$this->mbox = null;
        	return $ret;
        }
    }

    function getMsg($msgNum)
    {
    	if(!$this->mbox || !$this->folder) return false;
		$uid = $this->uids[$msgNum-1];
		try {
			$q = $this->folder->query();
			$q->setFetchBody(true);
			if (! empty($this->Params['mark_read'])) {
				$q->markAsRead();
			} else {
				$q->leaveUnread();
			}
			$message = $q->getMessageByUid($uid);
			$hd = $message->getHeader()->raw;
			$delim = substr($hd, -2) === "\r\n" ? "\r\n" : "\n";
			$prefix = "";
			$content = $message->getRawBody();
			$flags = $message->getFlags();
			foreach(array('Seen', 'Draft', 'Deleted', 'Answered', 'Flagged') as $h)
				$prefix .= "X-Flag-$h: " .  ($flags->contains($h) ? '1' : '0')  . $delim;
			$this->maxUid = $uid;
			$msg =  $prefix . $hd . $content;
			return $msg;
		} catch (Exception $e) {
			$this->setLastError($e);
        	return false;
		}
    }

    function getHeaders($msgNum)
    {
    	if(! $this->mbox || !$this->folder) return false;
		$uid = $this->uids[$msgNum-1];
		try {
			$q = $this->folder->query();
			$q->setFetchBody(false);
			$q->leaveUnread();
			$message = $q->getMessageByUid($uid);
			$hd = $message->getHeader()->raw;
        	return $hd;
		} catch (Exception $e) {
			$this->setLastError($e);
        	return false;
		}
    }

    function deleteMsg($msgNum)
    {
    	if(! $this->mbox || !$this->folder) return false;
		$uid = $this->uids[$msgNum-1];
		try {
			$q = $this->folder->query();
			$q->setFetchBody(false);
			$q->leaveUnread();
			$message = $q->getMessageByUid($uid);
			$message->delete();
        	return true;
		} catch (Exception $e) {
			$this->setLastError($e);
        	return false;
		}
    }

    function selectBox($boxName)
    {
		if(! $this->mbox) return false;
		if (!$this->folder) {
			$e = new Exception("Error selecting folder");
			$this->setLastError($e);
			return false;
		}
        return true;
    }

	function getFolders() {
		if (!$this->mbox) return false;
		$ret = [];
		foreach ($this->mbox->getFolders(false) as $f) {
			$ret[] = $f->full_name;
		}
		return $ret;
	}

    function getCheckDate()
    {
        return $this->maxUid;
	}

	private function setLastError(Exception $e) {
		$msg = $e->getMessage();
		$e = $e->getPrevious();
		while ($e) {
			$msg .= '; ' . $e->getMessage();
			$e = $e->getPrevious();
		}
		$this->lastError = $msg;
	}
}
