<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
require_once('Mail/RFC822.php');
require_once('Mail/mimeDecode.php');
require_once('modules/Emails/utils.php');
require_once('include/utils/file_utils.php');
require_once('include/utils/html_utils.php');
require_once('include/upload_file.php');


class IAHEmailParseError extends IAHInternalError {
	var $error_name = 'Email Parse Error';
}

class IAHEmailParser {
	var $message_id;
	var $to;
	var $from;
	var $reply_to;
	var $cc;
	var $bcc;
	var $subject;
	var $in_reply_to;
	var $references;
	var $mime_parts;
	var $headers;
	var $raw_headers;
	var $body;
	var $html_body;
	var $files;
	var $source;

	var $no_relate_domains;
	var $relate_inbound;
	var $bad_exts;
	var $image_types;
	var $max_message_size;
	
	
	function loadFile($path) {
		$content = file_get_contents($path);
		if($content === false)
			throw new IAHEmailParseError("error reading source file: $path");
		return $this->loadSource($content);
	}
	
	function loadSource($mailtext) {
		$this->source = $mailtext;
		$decode = new Mail_mimeDecode($mailtext, "\r\n");
		$structure = $decode->decode(
			array(
				"include_bodies" => true,
				"decode_bodies"  => true,
				"decode_headers" => false,
			)
		);

		if (PEAR::isError($structure)) {
			throw new IAHEmailParseError($structure->getMessage());
		}

		return $this->loadMime($structure);
	}
	
	
	function loadMime(stdClass $structure) {
		$this->raw_headers = $structure->headers;
		$this->headers = self::decode_quoted_header($this->raw_headers);

		$this->message_id = array_get_default($this->headers, 'message-id');
		if(! $this->message_id)
			$this->message_id = '<' . uniqid('msg-') . '@unknown>';
		$this->to = self::make_unified_emails_list(array_get_default($this->headers, 'to', ''));
		$this->from = self::get_first_email(array_get_default($this->headers, 'from', ''));
		$this->reply_to = self::get_first_email(array_get_default($this->headers, 'reply-to', ''));
		$this->cc = self::make_unified_emails_list(array_get_default($this->headers, 'cc', ''));
		$this->bcc = self::make_unified_emails_list(array_get_default($this->headers, 'bcc', ''));
		$this->subject = array_get_default($this->headers, 'subject', '');
		$this->in_reply_to = array_get_default($this->headers, 'in-reply-to', '');
		$this->references = array_get_default($this->headers, 'references', '');

		$this->mime_parts = array();
		self::extract_mime_parts($structure, $this->mime_parts);
		$message = null;
		$htmlMessage = null;
		$mergingTextImages = false;
		$fileNumber = 1;
		$this->files = array();

		// contributed by GroupWare-AG: recognize bounces to pass message text to bounce handling
		/*if (match_bounced_email($from[2], $subject)) {
			$plainMessage = self::concat_text_parts($this->mime_parts);
		}*/
		foreach ($this->mime_parts as $part)
		{
			if ($part->ctype_primary=='text' && $part->ctype_secondary=='plain' && !isset($message) && !$part->force_attach) {
				$message = self::decode_part_body($part);
			}
			elseif ($part->ctype_primary=='text' && $part->ctype_secondary=='html' && !isset($htmlMessage) && !$part->force_attach) {
				$htmlMessage = self::decode_part_body($part);
			}
			else {
				$attach = $this->decodeAttachment($part);
				if($attach['is_image'] && $attach['disposition'] == 'inline') {
					if(isset($message) && ! isset($htmlMessage)) {
						// handle case of plain text parts with inline image attachments (Mail.app does this)
						if(! $mergingTextImages) {
							$mergingTextImages = true;
							$htmlMessage = nl2br(to_html($message));
						}
						$alt = to_html($attach['name']);
						$message .= '(image)';
						$htmlMessage .= '<img alt="'.$alt.'" src="cid:'. to_html($attach['cid']) . '">';
					}
					$this->files[] = $attach;
				} else if ($attach['filename'] == 'winmail.dat') {
					require_once 'include/tnef/tnef_attachment.php';
					$attachment = new TnefAttachment(NULL);
					$attachment->decodeTnef($file['content']);
					$tnef_files = &$attachment->getFilesNested();
					foreach ($tnef_files as $tnef_file) {
						$tnef_name = $tnef_file->name;
						if(preg_match('~^(.+)\.(\w+)$~', $tnef_name, $m)) {
							$file_ext = strtolower($m[2]);
							if(in_array($file_ext, $this->getUploadBadExts())) {
								$file_ext .= '.txt';
								$tnef_name .= '.txt';
							}
						}
						else {
							$file_ext = 'txt';
							$tnef_name .= '.txt';
						}
						$this->files[] = array(
							'content' => &$tnef_file->content,
							'filename' => $tnef_file->name,
							'file_ext' => $file_ext,
							'ctype' => $tnef_file->type,
							'name' => $tnef_file->name,
							'disposition' => 'attachment',
						);
					}
				} else {
					if($mergingTextImages && $part->ctype_primary=='text' && $part->ctype_secondary=='plain') {
						$message .= $attach['content'];
						$htmlMessage .= nl2br(to_html($attach['content']));
					}
					else
						$this->files[] = $attach;
				}
			}
		}

		// truncate very long messages so we don't hit MySQL's packet size limit (whatever that is on the current platform, may be 1MB)
		$max_message_size = $this->getMaxMessageSize();
		if($max_message_size) {
			if(strlen($message) > $max_message_size) {
				$message = substr($message, 0, $max_message_size) . "\r\n -- message truncated by 1CRM -- \r\n";
				$htmlMessage = '';
			}
			else if(strlen($htmlMessage) > $max_message_size) {
				$htmlMessage = substr($htmlMessage, 0, $max_message_size) . "\r\n -- message truncated by 1CRM -- \r\n";
			}
		}
		
		$this->body = $message;
		$this->html_body = $htmlMessage;
	}


	function getNoRelateDomains() {
		if(! isset($this->no_relate_domains)) {
			$skip_doms = AppConfig::setting('email.relate_skipdomains', '');
			$skip_doms = trim($skip_doms);
			$domains = '';
			if(strlen($skip_doms)) {
				$skip_doms = array_unique(array_filter(array_map('trim', explode(' ', $skip_doms))));
				if($skip_doms) {
					foreach($skip_doms as $k => $d)
						$skip_doms[$k] = preg_quote($d, '~');
					$domains = '~('.implode('|', $skip_doms).')$~i';
				}
			}
			$this->no_relate_domains = $domains;
		}
		return $this->no_relate_domains;
	}
	
	function getRelateInbound() {
		if(! isset($this->relate_inbound)) {
			$this->relate_inbound = AppConfig::setting('email.relate_inbound', 1);
		}
		return $this->relate_inbound;
	}
	
	function getEmailSeen() {
		return ! empty($this->headers['x-flag-seen']); // fake header potentially returned by EmailClient
	}
	
	function parseHeaderDate($date) {
		 $date = trim(preg_replace('/\([^\)]*\)/', '', $date)); // spec allows for additional bracketed fields, like time zone
		 if(! $date) return false;
		 $ts = strtotime($date);
		 if($ts === -1) $ts = false;
		 return $ts;
	}
	
	function getDateSent($timestamp=false) {
		global $timedate;		        
        $email_dt = false;
        $rcvd = array_get_default($this->headers, 'received');
        if(is_array($rcvd)) $rcvd = $rcvd[0]; // final receive step
        if($rcvd) {
        	if( ($p = strrpos($rcvd, ';')) !== false)
        		$email_dt = $this->parseHeaderDate(substr($rcvd, $p+1));
        }
        if(! $email_dt)
        	$email_dt = $this->parseHeaderDate(array_get_default($this->headers, 'date', ''));
        if(! $email_dt)
            $email_dt = time();
        if($timestamp) return $email_dt;
        return gmdate($timedate->get_db_date_time_format(), $email_dt);
	}
	
	function getEmailRecord($user_id, $mailbox_id=null, $folder_id=null) {
        $email = RowUpdate::blank_for_model('Email');
		$email->set(array(
			'message_id' => $this->message_id,
			'date_start' => $this->getDateSent(),
			'in_reply_to' => $this->in_reply_to,
			'references' => $this->references,
			'isread' => $this->getEmailSeen() ? 1 : 0,
        	'from_addr' => $this->from[0],
        	'from_name' => $this->from[1],
        	'reply_to_addr' => $this->reply_to[0],
        	'reply_to_name' => (! empty($this->reply_to[0]) && empty($this->reply_to[1])) ? $this->from[1] : $this->reply_to[1],
        	'to_addrs' => $this->to[2],
        	'to_addrs_emails' => $this->to[0],
        	'to_addrs_names' => $this->to[1],
        	'cc_addrs' => $this->cc[2],
        	'cc_addrs_emails' => $this->cc[0],
        	'cc_addrs_names' => $this->cc[1],
        	'name' => strlen($this->subject) ? $this->subject : '[empty]',
        	'description' => $this->getTextBody(),
        	'description_html' => $this->getHtmlBody(),
			'mailbox_id' => $mailbox_id,
        	'folder' => $folder_id,
        	'assigned_user_id' => $user_id,
        ));
        
        return $email;
    }
    
    function getAttachments() {
    	return $this->files;
    }
    
    function saveAttachments($email_id, $user_id, $limit_indexes=null) {
    	global $log;
    	$ret = array();

    	if($this->files)
    	foreach($this->files as $idx => $file) {
    		if(is_array($limit_indexes) && ! in_array($idx, $limit_indexes))
    			continue;
            if($file['is_image'] && $file['disposition'] == 'inline') {
                $file['filename'] = $file['uniqid'] . '.' . $file['file_ext'];
				$rel_dir = 'email/' .substr($file['filename'], 0, 2) . '/' . substr($file['filename'], 2, 2);
				$dirname = create_files_directory($rel_dir . '/');
                $destination = $dirname . '/' . $file['filename'];
                $rel_name = $rel_dir . '/' . $file['filename'];
                $fp = @fopen($destination, 'wb');
                if(! $fp || @fwrite($fp, $file['content']) === false) {
					if($fp) @fclose($fp);
					$log->error("error saving email attachment to $destination");
					continue;
                }
                if($fp) @fclose($fp);

				$note = RowUpdate::blank_for_model('Note');
				$note->set(array(
					'name' => $file['name'],
                	'email_attachment' => 1,
                	'parent_type' => 'Emails',
                	'parent_id' => $email_id,
                	'file_mime_type' => $file['ctype'],
					'filename' => $rel_name,
					'assigned_user_id' => $user_id,
				));
                if($note->save())
					$ret[] = $note->getPrimaryKeyValue();
            } else {
		        $fname = preg_replace('/\\?/', '_', basename($file['filename']));
                $fl = new UploadFile('');
                $fl->use_soap = true;
                $fl->stored_file_name = $fname;
                $filename = $fl->create_stored_filename();
                $destination = AppConfig::upload_dir() . $filename;
                $fp = @fopen($destination, 'wb');
                if(! $fp || @fwrite($fp, $file['content']) === false) {
                	if($fp) @fclose($fp);
                    $log->error("error saving email attachment to $destination");
                    continue;
                }
                if($fp) @fclose($fp);
                
                $ext = $file['file_ext'];
                if( ($p = strpos($ext, '.txt')) > 2)
                	$ext = substr($ext, 0, $p);
                $note_name = $file['name'];
                if($ext) {
                	$ext = '.' . $ext;
                	if(substr($note_name, -strlen($ext)) != $ext)
						$note_name .= $ext;
				}
                
            	$note = RowUpdate::blank_for_model('Note');
				$note->set(array(
					'name' => $note_name,
                	'email_attachment' => 1,
                	'parent_type' => 'Emails',
                	'parent_id' => $email_id,
                	'file_mime_type' => $file['ctype'],
					'filename' => $filename,
					'assigned_user_id' => $user_id,
				));
				if($note->save())
					$ret[] = $note->getPrimaryKeyValue();
            }
	    }
	    
	    return $ret;
    }
    
    
    function findRelatedRecords($sent = false) {
    	$no_relate_domain = $this->getNoRelateDomains();

		$checkaddrs = array();

		if (!$sent)
			$checkaddrs = array(strtolower($this->from[0]));
		$checkaddrs = array_merge($checkaddrs, explode(';', strtolower($this->to[0])));
		$checkaddrs = array_merge($checkaddrs, explode(';', strtolower($this->cc[0])));
		if ($sent) {
			$checkaddrs = array_merge($checkaddrs, explode(';', strtolower($this->bcc[0])));
		}
		
		if($no_relate_domain) {
			$addrs = array();
			foreach($checkaddrs as $addr) {
				if(! preg_match($no_relate_domain, $addr))
					$addrs[] = $addr;
			}
			$checkaddrs = $addrs;
		}
		
		if(count($checkaddrs) > 10)
			$checkaddrs = array_slice($checkaddrs, 0, 10);
		return lookup_email_addresses($checkaddrs, $this->getRelateInbound());
	}
	
	
	function fromNoRelateDomain() {
		$no_relate_domain = $this->getNoRelateDomains();
		if($no_relate_domain && $this->from[0] && preg_match($no_relate_domain, $this->from[0]))
			return true;
		return false;
	}
	

	function getUploadBadExts() {
		if(! isset($this->bad_exts)) {
			$this->bad_exts = AppConfig::setting('site.upload.badext', array());
		}
		return $this->bad_exts;
	}
	
	
	function getImageTypes() {
		if(! isset($this->image_types)) {
			$this->image_types = AppConfig::setting('email.image_types', array());
		}
		return $this->image_types;
	}
	
	
	function getPlainBody() {
		return self::concat_text_parts($this->mime_parts);
	}
	
	function getTextBody() {
		return strlen($this->body) ? $this->body : html2plaintext($this->html_body, false);
	}
	
	function getHtmlBody() {
		return $this->html_body;
	}

	function getSubject() {
		return $this->subject;
	}
	
	
	function getMaxMessageSize() {
		if(! isset($this->max_message_size)) {
			$this->max_message_size = AppConfig::setting('email.max_message_size', 0);
		}
		return $this->max_message_size;
	}
	
	
	function findEmailThread() {
		$references = array();
		if (!empty($this->references)) {
			$references = preg_split('/\s+/', $this->references);
			$references = array_map('trim', $references);
		}
		if (!empty($this->in_reply_to) && preg_match('/(<[^>\s]+>)/', $this->in_reply_to, $m)) {
			array_unshift($references, $m[1]);
		}
		$references = array_filter(array_unique($references));
		if(count($references)) {
			$lq = new ListQuery('mail_threads', array('thread_id'));
			$lq->addSimpleFilter('message_id', $references, '=');
			$result = $lq->runQuerySingle();
			if ($result && ! $result->failed) {
				return $result->getField('thread_id');
			}
		}
	}
	

	function decodeAttachment(&$part) {
		if(! empty($part->ctype_parameters['name']))
			$orig_name = $part->ctype_parameters['name'];
		else if(! empty($part->d_parameters['filename']))
			$orig_name = $part->d_parameters['filename'];
		else
			$orig_name = null;
		if(isset($orig_name)) {
			$name = $orig_name;
			$orig_name = filename_safe_string($orig_name);
		}
		$disposition = isset($part->disposition) ? $part->disposition : null;
	
		if(isset($part->headers['content-id']))
			$cid = str_replace(array('<','>'), '', $part->headers['content-id']);
		if(! isset($cid) || ! strlen($cid))
			$cid = md5($orig_name . $part->body);
		else
			$disposition = 'inline'; // override for attachments with content-id
		$uniqid = md5($cid);
	
		if ($part->ctype_primary == 'image' && in_array($part->ctype_secondary, $this->getImageTypes())) {
			$file_ext = $part->ctype_secondary;
			$is_image = 1;
		}
		if(isset($orig_name) && preg_match('~^(.+)\.(\w+)$~', $orig_name, $m)) {
			$orig_name = $m[1];
			if(! isset($file_ext)) {
				$file_ext = strtolower($m[2]);
				if(in_array($file_ext, $this->getImageTypes()))
					$is_image = 1;
				}
			}
		if(! isset($file_ext)) {
			if($part->ctype_primary == 'text') {
				if($part->ctype_secondary == 'plain')
					$file_ext = 'txt';
				else
					$file_ext = $part->ctype_secondary;
			}
			else
				$file_ext = 'txt';
		}

		if(in_array($file_ext, $this->getUploadBadExts())) {
			$file_ext .= '.txt';
		}
		$filename = (isset($orig_name) ? $orig_name : $uniqid) . '.' . $file_ext;
		
		$ret = array(
			'cid' => $cid,
			'uniqid' => $uniqid,
			'name' => $orig_name,
			'filename' => $filename,
			'file_ext' => $file_ext,
			'ctype' => $part->ctype_primary.'/'.$part->ctype_secondary,
			'disposition' => $disposition,
			'is_image' => ! empty($is_image),
		);
		if($part->ctype_primary=='text' && $disposition != 'attachment')
			$ret['content'] = self::decode_part_body($part);
		else
			$ret['content'] = &$part->body;
		return $ret;
	}
	

	static function decode_quoted_header($text) {
		if(is_array($text))
			return array_map(array('IAHEmailParser', 'decode_quoted_header'), $text);
		// decode headers in quoted-printable format (defined by RFC 1522)
		return preg_replace_callback('/=\?(.+?)\?([qb])\?(.*?)\?=\s*(<.*?>)?/i',
			function($m) { return IAHEmailParser::decode_header_word(array_get_default($m, 1, ''), array_get_default($m, 3, ''), array_get_default($m, 2, ''), array_get_default($m, 4, '')); },
			$text);
	}
	
	static function decode_header_word($encoding, $text, $mode='q', $address = '') {
		global $locale;
		$enc = strtolower($encoding);
		if(! $enc) $enc = 'utf-8';
		$text = str_replace('_', ' ', $text);
		$mode = strtolower($mode);
		if($mode == 'q') {
			$text = quoted_printable_decode($text);
		} else {
			$text = base64_decode($text);
		}
		// force conversion to utf-8 to replace any invalid character sequences
		// otherwise, mysql may produce a query error
		$text = $locale->translateCharset($text, $enc, 'utf-8', true);

		if (! empty($address))
			$text .= " {$address}";

		return $text;
	}

	static function decode_part_body(&$part) {
		global $locale;
		if(isset($part->ctype_parameters['charset']) && strtolower($part->ctype_parameters['charset']) != 'utf-8') {
			$ret = $locale->translateCharset($part->body, $part->ctype_parameters['charset']);
			if (!$locale->conversionError) return $ret;
		}
		if (!isset($part->ctype_parameters['charset'])) {
			$ret = $locale->translateCharset($part->body, AppConfig::setting('email.default_charset'));
			if (!$locale->conversionError) return $ret;
		}
		// force conversion to utf-8 to replace any invalid character sequences
		// otherwise, mysql may produce a query error
		return $locale->translateCharset($part->body, 'utf-8', 'utf-8', true);
  	}

	static function concat_text_parts($parts) {
		$ret = '';
		foreach ($parts as $part) {
			if (isset($part->parts) && is_array($part->parts)) {
				$ret .= self::concat_text_parts($part->parts);
			} elseif ($part->ctype_primary == 'text' && $part->ctype_secondary != 'html' && ! $part->force_attach) {
				$ret .= self::decode_part_body($part);
			}
		}
		return $ret;
	}

	static function extract_mime_parts($message, &$parts, $forceAttach = false) {
		if ($message->ctype_primary != 'multipart' && ($message->ctype_primary . '/' . $message->ctype_secondary != 'message/rfc822')) {
			$message->force_attach = $forceAttach || (isset($message->disposition) && $message->disposition == 'attachment');
			$parts[] = $message;
		} else if(isset($message->parts) && is_array($message->parts)) {
			foreach ($message->parts as $part) {
				foreach($part->headers as $key => $val) {
					$part->headers[$key] = self::decode_quoted_header($val);
				}
				if (!empty($part->ctype_parameters)) {
					foreach($part->ctype_parameters as $key => $val) {
						$part->ctype_parameters[$key] = self::decode_quoted_header($val);
					}
				}
				if (!empty($part->d_parameters)) {
					foreach($part->d_parameters as $key => $val) {
						$part->d_parameters[$key] = self::decode_quoted_header($val);
					}
				}
				self::extract_mime_parts($part, $parts, $forceAttach || ($message->ctype_primary . '/' . $message->ctype_secondary == 'message/rfc822'));
			}
		}
	}
  
  	static function join_email_name($email, $name='') {
  		if(! strlen($email))
  			return '';
  		$email = '<'.$email.'>';
  		if(strlen($name))
  			$email = $name . ' ' . $email;
  		return $email;
  	}

	static function get_first_email($addrs) {
		$name = "";
		$parser = new Mail_RFC822();
		$structure = $parser->parseAddressList($addrs, null, null, false);
		if (PEAR::isError($structure)) {
			// Improperly formatted email address --
			// trying best to guess. Better than nothing
			$m = array();
			if (preg_match('/(.*)<([^>]*)>/', $addrs, $m)) {
				$name = trim($m[1]);
				if(preg_match('/^"(.*)"$/', $name, $mq))
					$name = $mq[1];
				return array($m[2], $name, self::join_email_name($m[2], $name));
			}
			return array('', $addrs, $addrs);
		}
		$ret = $structure[0]->mailbox."@".$structure[0]->host;
		if ($ret=="@localhost") $ret="";
		else {
			$name = trim($structure[0]->personal);
			if(preg_match('/^"(.*)"$/', $name, $m))
				$name = $m[1];
		}
		return array($ret, $name, self::join_email_name($ret, $name));
	}

	static function make_unified_emails_list($emails) {
		$buffer = array();
		$names = array();
		$emails = (array)$emails;
		$joined = array();
		$parser = new Mail_RFC822();
		foreach ($emails as $email) {
			$parsed = $parser->parseAddressList($email, null, null, false);
			if (PEAR::isError($parsed)) {
				continue;
			}
			foreach ($parsed as $addr) {
				$address = $addr->mailbox."@".$addr->host;
				if ($address!="@localhost") {
					$buffer[] = $address;
					$name = trim($addr->personal);
					if(preg_match('/^"(.*)"$/', $name, $m))
						$name = $m[1];
					$names[] = $name;
					$joined[] = self::join_email_name($address, $name);
				}
			}
		}
		return array(implode('; ', $buffer), implode('; ', $names), implode('; ', $joined));
	}
	
}

?>
