<?php

namespace OneCRM\MultiFA;


use Katzgrau\KLogger\Logger;

class MultiFactorAuthFormRenderer {

	/**
	 * Render Verify Two-FA Code form for login
	 *
	 * @param string $login_message
	 *
	 * @return string
	 */
	public static function render($login_message) {
		$label = translate('LBL_MULTI_FA_LOGIN_AUTH_CODE', 'Users');
		$button_label = translate('LBL_MULTI_FA_LOGIN_VERIFY_BUTTON', 'Users');

		preg_match_all('#<p>.*</p>#', $login_message,$results);
		$last_paragraph = isset($results[0]) ? array_pop($results[0]) : '';
		$modified_login_message = $login_message;

		if (! empty($last_paragraph)) {
			$auth_code_desc = translate('LBL_MULTI_FA_LOGIN_AUTH_CODE_DESC', 'Users');
			$recovery_desc = translate('LBL_MULTI_FA_LOGIN_RECOVERY_CODES_DESC', 'Users');

			$modified_login_message = str_replace($last_paragraph, "<p>{$auth_code_desc}</p>", $modified_login_message);
			$modified_login_message .= "<p>{$recovery_desc}</p>";
		}

		$form = <<<EOD
<div class="row page-row sm-unstack">
<div class="column page-column md-7 xs-hidden login-message">$modified_login_message</div>
<div class="column page-column md-5 md-offset-0 sm-8 sm-offset-2 lw-form login-form">
<div class="form-outer">
<form name="LoginForm" action="index.php" method="POST" onsubmit="return verifyCode();" role="form">
	<div class="form-group">
		<label class="control-label" for="login_user">$label</label>
		<div class="input-field input-field-group input-block">
			<input id="login_otp" name="otp" class="input-text input-lg" type="text" tabindex="1" autocomplete="off" autocapitalize="off" autocorrect="off" spellcheck="false" value="" placeholder="123456">
			<span class="input-label input-field-icon lg"><span class="uii uii-lock uii-lg"></span></span>
		</div>
	</div>
	<section class="card-outer panel-outer panel-border panel-round panel-alert form-group login-error" role="alert" id="error-outer" style="display: none">
		<div class="card-body panel-body panel-pad-body">
			<span class="uii uii-alert"></span>
			<span id="login-error"></span>
		</div>
	</section>
	<div class="login-foot">
		<section class="card-outer card-inline panel-outer panel-border panel-round panel-info login-status" role="alert" id="status-outer" style="display: none">
			<div class="card-body panel-body panel-pad-body">
				<span class="uii uii-info"></span>
				<span id="login-status"></span>
			</div>
		</section>
		<button id="login_button" class="lw-iconbutton lwib-accept" type="submit" tabindex="2">
			<span class="uii uii-arrow-right"></span>
			<span class="btn-label">$button_label</span>
		</button>
		<div class="clearfix"></div>
	</div>
</form>
</div>
</div>
</div>			
EOD;
		return $form;
	}

	/**
	 * Render PDF with recovery codes
	 *
	 * @param \User $user
	 * @param Logger $logger
	 */
	public static function renderRecoveryCodesPdf(\User $user, Logger $logger) {
		require_once 'include/layout/forms/FormGenerator.php';
		require_once 'MultiFactorAuthenticator.php';

		$title = translate('LBL_MULTI_FA_RECOVERY_CODES', 'Users');
		$auth = new MultiFactorAuthenticator($user, $logger);
		$codes = $auth->getRecoveryCodes();

		$html = "<div style=\"text-align: center; font-weight: bold; font-size: 14pt;\">{$title}</div>";
		$html .= "<div style=\"font-size: 14pt; font-family: monospace, monospace;\"><ul>";

		foreach ($codes as $code) {
			$html .= "<li>{$code}</li>";
		}

		$html .= "</ul></div>";

		$gen = \FormGenerator::new_form('pdf');
		$gen->pdf->writeHTML($html);

		ob_get_clean();
		session_write_close();

		header('Content-Type: application/pdf');
		header('Content-Disposition: inline; filename="1crm-recovery-codes.pdf"');
		header('Content-Transfer-Encoding: binary');
		header('Accept-Ranges: bytes');

		$gen->pdf->Output('1crm-recovery-codes.pdf', 'I');
	}
}