<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


require_once('include/Notify/RecordActionMessage.php');

class InviteMessage extends RecordActionMessage {
	var $invitees = array();

	function __construct($data = null) {
		parent::__construct($data);

		$this->format_hooks[] = [
			'class' => get_class($this),
			'class_function' => 'extra_formatting'
		];
	}

	static function extra_formatting($result, $fmt) {
		if (isset($result->fields['online_meeting_join_url'])) {
			require_once('include/OnlineMeetings/OnlineMeetingsFactory.php');

			try {
				$online_meeting = OnlineMeetingsFactory::getInstance();
				$result->fields['online_meeting_join_url']['vname'] = $online_meeting->getJoinUrlLabel();
			} catch (OnlineMeetingException $e) {}
		}
	}

	function getReplaceParams() {
		$record = $this->getSingleRecord();
		$params = parent::getReplaceParams();
		$params['{DATE}'] = $record ? $record->getField('date_start', null, true) : '';
		return $params;
	}

	function getInvitees($reload=false, $filter=true) {
		$this->populateRecords();
		$record = $this->getSingleRecord();
		$ret = array();
		if($record) {
			$rid = $record->getField('id');
			if(! isset($this->invitees[$rid]) || $reload) {
				$list = array();
				if($this->model == 'Task') {
					$info = get_user_info($record->getField('assigned_user_id'));
					if($info) {
						$info['bean_name'] = 'User';
						$list[] = $info;
					}
				} else {
					$lq = new ListQuery(
						$this->model,
						array('id', 'query_module', 'name', 'email' => 'email1', 'accept_status', 'receive_notifications', 'email_opt_out', 'invalid_email'),
						array('link_name' => 'invitees', 'parent_key' => $rid));
					foreach($lq as $inv) {
						$row = $inv->row;
						$row['bean_name'] = AppConfig::module_primary_bean($row['query_module']);
						$list[] = $row;
					}
				}
				$this->invitees[$rid] = $list;
			}
			$ret = $this->invitees[$rid];			
		}
		if($filter) {
			foreach($ret as $idx => $inv) {
				if($this->action == 'reminder') {
					if($this->model != 'Task' && $inv['accept_status'] !== 'accept' && $inv['accept_status'] != 'tentative')
						unset($ret[$idx]);
				}
				else {
					if($inv['accept_status'] && $inv['accept_status'] != 'none')
						unset($ret[$idx]);
					else if($inv['bean_name'] == 'User' && $inv['id'] == $this->performer_id)
						unset($ret[$idx]);
				}
			}
		}
		return $ret;
	}

	function getAcceptUrl(NotifyFormatter $notify) {
		$module = AppConfig::module_for_model($this->model);
		$rid = $this->getRecordField('id');
		$target_id = $notify->getTarget('id');
		$target_bean = $notify->getTarget('bean_name');
		$param = strtolower($target_bean).'_id';
		if($module && $rid && $param && $target_id)
			return 'acceptDecline.php?module='.$module.'&record='.$rid.'&'.$param.'='.$target_id;
	}
	
	function getMessageButtons(NotifyFormatter $notify) {
		if($this->action == 'invited') {
			$url = $this->getAcceptUrl($notify);
			if($url) {
				$btns = array();
				$btns['accept'] = array(
					'label' => $notify->translate('LBL_INVITE_ACCEPT'),
					'url' => $url . '&accept_status=accept',
					'class' => 'active',
				);
				$btns['tentative'] = array(
					'label' => $notify->translate('LBL_INVITE_TENTATIVE'),
					'url' => $url . '&accept_status=tentative',
					'class' => 'default',
				);
				$btns['decline'] = array(
					'label' => $notify->translate('LBL_INVITE_DECLINE'),
					'url' => $url . '&accept_status=decline',
					'class' => 'default',
				);
				return $btns;
			}
		}

	}

	function preFormatMessage(NotifyFormatter $notify) {
		if($this->action == 'invited') {
			$base = AppConfig::setting('site.accept_url');
			if($base) {
				if(substr($base, -17) == 'acceptDecline.php')
					$base = substr($base, 0, -17);
				if(substr($base, -1) != '/') $base .= '/';
				$notify->setBaseUrl($base);
			}
		}
	}

	function postFormatMessage(NotifyFormatter $notify, array &$msg) {
		if($notify->getFormat() != 'text') return;
		$record = $this->getSingleRecord();
		if(! $record) return;
		if(($this->model == 'Call' || $this->model == 'Meeting') && ! $notify->getStandalone()) {
			require_once('modules/Meetings/utils.php');
			$for_user = ! $notify->getExternal();
			$ical = create_result_ical($record, $for_user);
			if($ical) {
				$content = wrap_ical_result($ical['body']);
				$notify->addDataAttachment($content, $ical['title'] . '.ics', false, 'base64', 'text/calendar');
			}
		}
	}
}

