<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


require_once('include/Notify/NotifyIncludes.php');

class NotifyFormatter extends NotifyIncludes {
	var $standalone;
	var $external;
	var $language;

	var $template_path = 'modules/Notify/templates/Notify.tpl';
	var $link_style = "color: #56d; text-decoration: none";
	var $prefix_subject = true;
	var $remove_images = false;
	var $attach_images = true;
	var $target;
	var $base_url;
	var $format = 'html';
	
	var $text_line_length = 76;
	
	static function create_standard() {
		$sel = AppConfig::setting('notify.formatter');
		if(! $sel || ! ($info = AppConfig::setting("notify.formatters.$sel")))
			$info = AppConfig::setting('notify.formatters.default');
		require_once($info['path']);
		return new $info['class_name'];
	}
	
	static function create_standalone($extern=true) {
		$fmt = self::create_standard();
		$fmt->setStandalone();
		if($extern) $fmt->setExternal();
		return $fmt;
	}
	
	static function simple_record($name, $props=null, $title=null, $kind=null) {
		$msg = array(
			'title' => $title,
			'kind' => $kind,
			'subject' => strip_tags($title),
			'rows' => array(
				array(
					'records' => array(
						array(
							'name' => $name,
							'props' => $props,
						),
					),
				),
			),
		);
		return $msg;
	}
	
	function reset() {
		$this->resetIncludes();
		$this->target = null;
		$this->external = false;
		$this->language = null;
		$this->remove_images = false;
	}
	
	function getStandalone() {
		return $this->standalone;
	}
	function setStandalone($alone=true) {
		$this->standalone = $alone;
	}

	function getExternal() {
		return $this->external;
	}
	function setExternal($extern=true) {
		$this->external = $extern;
	}
	
	function getLanguage() {
		return $this->language;
	}
	function setLanguage($lang=null) {
		$this->language = $lang;
	}
	
	function getTarget($prop=null, $default=null) {
		$ret = $this->target;
		if($ret && $prop)
			$ret = array_get_default($ret, $prop, $default);
		return $ret;
	}
	function setTarget($targ) {
		if($targ instanceof RowResult) {
			$arr = $targ->row;
			$arr['bean_name'] = $targ->getModelName();
			$this->target = $arr;
		}
		else if(is_array($targ)) {
			$this->target = $targ;
			if(isset($targ['remove_images']))
				$this->setRemoveImages($targ['remove_images']);
			if(isset($targ['language']))
				$this->setLanguage($targ['language']);
			if(isset($targ['external']))
				$this->setExternal($targ['external']);
		} else
			$this->target = null;
	}
	function setTargetId($bean_name, $id) {
		$this->target = compact('bean_name', 'id');
	}
	
	function getFormat() {
		return $this->format;
	}
	function setFormat($fmt) {
		$this->format = $fmt;
	}
	
	function setRemoveImages($remove=true) {
		$this->remove_images = $remove;
	}
	
	function setAttachImages($attach=true) {
		$this->attach_images = $attach;
	}
	
	function translate($label, $mod=null, $default=null) {
		$lang = $this->language;
		if(! $lang) $lang = 'current';
		if(! $mod) $mod = 'Notify';
		return AppConfig::setting("lang.strings.$lang.$mod.$label", $default);
	}

	function translate_dd($dropdown, $label, $default=null) {
		return translate_dd($dropdown, $label, $default, $this->language);
	}
	
	function getFieldFormatter() {
		require_once('include/layout/FieldFormatter.php');
		$fmt = new FieldFormatter($this->format, $this->getExternal() ? 'email' : 'notify');
		if($this->getTarget('bean_name') == 'User')
			$fmt->setUser($this->getTarget('id'));
		return $fmt;
	}
		
	function formatLink($url, $text=null, $escape=true, $attribs=null) {
		if(! $attribs) $attribs = array();
		$attribs['href'] = $url;
		if($escape) $attribs = array_map('to_html', $attribs);
		$attribs['style'] = isset($attribs['style']) ? $this->link_style . ';' . $attribs['style'] : $this->link_style;
		$a = array();
		foreach($attribs as $k => $v)
			$a[] = "$k=\"$v\"";
		$attr = implode(' ', $a);
		if(! strlen($text)) {
			$text = $url;
			$escape = 1;
		}
		if($escape)
			$text = to_html($text);
		return "<a $attr>$text</a>";
	}
	
	function formatUser($uid, $short=false, $format=null) {
		if(! isset($format)) $format = $this->format;
		$info = get_user_info($uid);
		if(! $info) $user = '';
		else {
			$ufull = trim($info['first_name'] . ' ' . $info['last_name']);
			if($format == 'html' && ! $this->external) {
				$user = $this->formatLink('index.php?module=Users&action=DetailView&record='.$info['id'], $ufull);
			} else {
				$user = $ufull;
			}
			if(! $short) {
				if($this->external) {
					$uname = $info['email1'];
					if($uname && $format == 'html')
						$uname = $this->formatLink("mailto:$uname", $uname);
				}
				else
					$uname = $info['user_name'];
				if($uname)
					$user .= " ($uname)";
			}
		}
		return $user;
	}
	
	function defaultRecordUrl($model, $id) {
		$mod = AppConfig::module_for_model($model);
		if($mod && AppConfig::module_primary_bean($mod) == $model)
			return 'index.php?module='.$mod.'&action=DetailView&record='.$id;
	}
	
	function formatRecordLink($name, $model, $id, $url=null, $icon=null) {
		$mod = AppConfig::module_for_model($model);
		if($this->format == 'html' && ! $this->getExternal() && $mod && $id) {
			// name should be formatted as HTML already
			if(! $url) $url = $this->defaultRecordUrl($model, $id);
			if($url)
				$name = '<a href="'.to_html($url).'">'.$name.'</a>';
		}
		$img = get_image_info($icon ? $icon : $mod);
		if($img && $img['found']) $name = $img['html'] . ' ' .$name;
		return $name;
	}
	
	function styleValue($html) {
		if(! isset($html)) return;
		$html = preg_replace_callback('~<a\b[^>]*>~i', array(&$this, 'addLinkStyle'), $html);
		if(! $this->standalone) {
			if($this->attach_images || $this->remove_images)
				$html = preg_replace_callback('~<img\b[^>]*>~i', array(&$this, 'replaceImage'), $html);
			$html = preg_replace_callback('~<div\b[^>]*>(</div>)?~i', array(&$this, 'replaceIconImage'), $html);
		}
		return $html;
	}
	
	function addLinkStyle($link) {
		$link = preg_replace('~\bstyle\s*=\s*(?<q>[\'"])([^(?&q)]*)(?&q)~', 'style="'.$this->link_style.'; $2"', $link[0], -1, $c);
		if(! $c)
			$link = substr($link, 0, -1) . ' style="'.$this->link_style.'">';
		return $link;
	}

	function replaceImage($img) {
		if($this->remove_images)
			return '';
		$img = $img[0];
		if($this->attach_images && preg_match('~\bsrc\s*=\s*(?<q>[\'"])(.*?)(?&q)~', $img, $m, PREG_OFFSET_CAPTURE)) {
			$src = $m[2][0];
			$offset = $m[2][1];
			if( ($p = strpos($src, '?')) !== false)
				$src = substr($src, 0, $p);
			if(file_exists($src) && ($cid = $this->addImageAttachment($src))) {
				$img = substr($img, 0, $offset) . 'cid:'.$cid . substr($img, $offset + strlen($m[2][0]));
			}
		}
		return $img;
	}
	
	function replaceIconImage($img) {
		$img = $img[0];
		if(preg_match('~\bclass\s*=\s*(?<q>[\'"])(.*?)(?&q)~', $img, $m)) {
			$cls = array_filter(array_map('trim', explode(' ', $m[2])));
			if(! in_array('input-icon', $cls))
				return $img;
			if(! in_array('theme-icon', $cls) || $this->remove_images)
				return '';
			foreach($cls as $c) {
				if(substr($c, 0, 7) == 'module-') {
					$icon = substr($c, 7);
					break;
				} if(substr($c, 0, 5) == 'bean-') {
					$icon = substr($c, 5);
					break;
				}
			}
			$istyle = 'vertical-align: text-top';
			if(! empty($icon)) {
				$info = get_image_info($icon, true, "border=\"0\" style=\"$istyle\"");
				if($info && ! $info['found'] || ! $info['src']) {
					$info = get_image_info(AppConfig::module_for_model($icon), true, "border=\"0\" style=\"$istyle\"");
					if(! $info || ! $info['found'] || ! $info['src'])
						$info = null;
				}
				if($info) {
					if($this->standalone || ! $this->attach_images)
						$img = $info['img_html'];
					else {
						$src = $info['src'];
						if( ($p = strpos($src, '?')) !== false)
							$src = substr($src, 0, $p);
						if(preg_match('~/Default/images/(.*?).gif$~', $src, $m)) {
							$hires = "themes/Default/images32/{$m[1]}.png";
							if(file_exists($hires)) $src = $hires;
						}
						$cid = $this->addImageAttachment($src);
						// Mail.app messes up icon size without styling the width and height (2016/09)
						$img = "<img src=\"cid:$cid\" width=\"{$info['width']}\" height=\"{$info['height']}\" border=\"0\" style=\"$istyle; width:{$info['width']}px; height:{$info['height']}px\" alt=\"\">";
					}
				} else {
					$img = '';
				}
			}
		}
		return $img;
	}
	
	function styleMessages($msgs, $depth=0) {
		$ret = array();
		foreach($msgs as $k => $m) {
			if(is_array($m))
				$ret[$k] = $this->styleMessages($m, $depth+1);
			else
				$ret[$k] = $this->styleValue($m);
		}
		return $ret;
	}
	
	function getIconsCss() {
		$icons = <<<EOQ
			<link type="text/css" rel="stylesheet" href="themes/Default/theme-icons.css">
			<link type="text/css" rel="stylesheet" href="themes/Default/print-icons.css" media="print">
EOQ;
		$custom = AppConfig::custom_dir();
		if(file_exists($custom . 'themes/Default/icons.css'))
			$icons .= <<<EOQ
			<link type="text/css" rel="stylesheet" href="{$custom}themes/Default/icons.css">
EOQ;
		return $icons;
	}
	
	function getBaseUrl() {
		if(isset($this->base_url)) return $this->base_url;
		return AppConfig::site_url();
	}
	function setBaseUrl($base) {
		$this->base_url = $base;
	}
	
	function getAppVersion() {
		$appver = AppConfig::version();
		return ocrm_e_s() . $appver;
	}
	
	function getSystemName() {
		$sys = $this->translate('NTC_APPNAME', 'app');
		$sys = str_replace('{VERSION}', $this->getAppVersion(), $sys);
		return $sys;
	}
	
	function getFooterText($html=false) {
		if($html)
			$custom_footer = AppConfig::setting(array('notify.footer_html', 'notify.footer_text'));
		else
			$custom_footer = AppConfig::setting('notify.footer_text');
		if($custom_footer)
			$footer = $custom_footer;
		else if($this->standalone)
			$footer = $this->translate('LBL_FOOTER_STANDALONE');
		else
			$footer = $this->translate('LBL_FOOTER_EMAIL');
		$footer = str_replace('{SYSTEM}', $this->getSystemName(), $footer);
		return $footer;
	}
	
	function getAdjustPrefsUrl($user_id) {
		return 'index.php?module=Users&action=DetailView&record='.$user_id;
	}
	
	function getTemplateParams() {
		$params = array();
		$params['site_url'] = $this->getBaseUrl();
		$params['standalone'] = $this->standalone;
		$params['external'] = $this->external;
		$params['styles'] = AppConfig::setting('notify.styles');
		if($this->standalone)
			$params['std_include'] = $this->getIconsCss();
		foreach(array('head', 'body', 'foot') as $c)
			$params[$c.'_include'] = $this->getIncludeData($c);
		$appver = AppConfig::version();
		$params['version'] = $this->getAppVersion();
		$params['title'] = $this->translate('LBL_BROWSER_TITLE', 'app');
		$params['app_signature'] = AppConfig::setting('notify.hide_app_signature') ? '' : '1CRM';
		if($this->standalone) {
			$params['favicon'] =
				'<link rel="shortcut icon" href="favicon.ico?s='.$appver.'" type="image/vnd.microsoft.icon">'
				.'<link rel="icon" href="favicon.ico?s='.$appver.'" type="image/vnd.microsoft.icon" sizes="16x16">';
			if(empty($params['logo'])) {
				$logo_file = AppConfig::setting('site.top_logo.primary', '');
				if($logo_file == 'include/images/iah/default-header.png')
					$logo_file = 'include/images/iah/1crm-large.png';
				$logo_h = 50; //AppConfig::setting('site.top_logo.auto_height', 50);
				$size = custom_get_image_size($logo_file);
				if($size === false)
					$size = array('', '');
				if($size[1] >= $logo_h * 1.5) {
					$size[0] = round($size[0] * $logo_h / $size[1]);
					$size[1] = round($logo_h);
				}
				$params['logo'] = '<img src="'.to_html($logo_file)."\" width=\"{$size[0]}\" height=\"{$size[1]}\" alt=\"\">";
			}
		}
		$footer = $this->getFooterText(true);
		if(! $this->getExternal()) {
			if($this->getTarget('bean_name') == 'User' && ($user_id = $this->getTarget('id'))) {
				$adj_url = $this->getAdjustPrefsUrl($user_id);
				if($adj_url)
					$footer .= '. ' . $this->formatLink($adj_url, $this->translate('LBL_ADJUST_PREFS', 'Notify'));
			}
		}
		// else: unsubscribe link
		$params['footer'] = $footer;
		return $params;
	}
	
	function getTemplatePath($custom=null) {
		$path = ($custom && file_exists($custom)) ? $custom : $this->template_path;
		if(substr($path, 0, 8) == 'modules/') {
			$cst = 'custom/'.$path;
			if(file_exists($cst))
				$path = $cst;
		}
		return $path;
	}
	
	function combineParams($base, $ext) {
		if(! is_array($base)) $base = array();
		static $concat = array('head_include', 'body_include', 'foot_include');
		if($ext) {
			foreach($ext as $k => $v) {
				if(isset($base[$k]) && in_array($k, $concat))
					$base[$k] .= $v;
				else
					$base[$k] = $v;
			}
		}
		return $base;
	}
	
	function initTemplate($params=null) {
		require_once('include/Sugar_Smarty.php');
		$tpl = new Sugar_Smarty();
		$params = $this->combineParams($this->getTemplateParams(), $params);
		$tpl->assign('params', $params);
		return $tpl;
	}
	
	function renderMessage($msg, $params=null) {
		return $this->renderMessages(array($msg), $params);
	}
	
	function renderMessages(array $msgs, $params=null) {
		if(! is_array($params)) $params = array();		
		$render = array();
		$custom_tpl = null;
		$custom_params = null;
		$idx = 0;
		foreach($msgs as $m) {
			$first = ($idx ++) == 0;
			if($m instanceof NotifyMessageBase) {
				$fmsg = $m->formatMessage($this);
				if($first) {
					$custom_tpl = $m->getCustomTemplate();
					$custom_params = $m->getTemplateParams();
				}
			}
			else {
				$fmsg = $m;
				if($first) {
					if(isset($m['template_path']))
						$custom_tpl = $m['template_path'];
					if(isset($m['template_params']))
						$custom_params = $m['template_params'];
				}
			}
			if(is_array($fmsg))
				$render[] = $fmsg;
			if(! $first) {
				// ignored if multiple messages are combined
				$custom_tpl = $custom_params = null;
			}
		}
		if(empty($params['title']) && count($render) == 1 && ! empty($render[0]['subject']))
			$params['title'] = $this->translate('LBL_SHORT_BROWSER_TITLE', 'app') . ': ' . $render[0]['subject'];
		
		if($this->getFormat() == 'html') {
			if(is_object($custom_tpl)) {
				$tpl = $custom_tpl;
				$custom_tpl = null;
			}
			if($custom_params)
				$params = $this->combineParams($custom_params, $params);
			$tpl = $this->initTemplate($params);
			$tpl->assign('messages', $this->styleMessages($render));
			return $tpl->fetch($this->getTemplatePath($custom_tpl));
		} else {
			return $this->formatTextMessage($render, $params);
		}
	}
	
	function renderMessagesText(array $msgs, $params=null) {
		$fmt = $this->format;
		$this->format = 'text';
		$ret = $this->renderMessages($msgs, $params);
		$this->format = $fmt;
		return $ret;
	}
	
	function formatWrappedText($text, &$ret, $offset=0, $indent=0, $add_indent=false) {
		$lines = explode("\n", $text);
		return $this->formatWrappedLines($lines, $ret, $offset, $indent, $add_indent);
	}
	
	function formatWrappedLines(array $lines, &$ret, $offset=0, $indent=0, $add_indent=false) {
		$first = true;
		foreach($lines as $text) {
			while(1) {
				$maxlen = $this->text_line_length - $offset;
				$strlen = strlen($text);
				$strlen_blank = strlen(trim($text));
				if($first && ($maxlen < 1 || ! $strlen_blank))
					break;
				if($strlen < $maxlen) {
					if($add_indent) $text = str_repeat(' ', $indent) . $text;
					$ret[] = $text;
					break;
				}
				if(preg_match('~\s~', substr($text, $maxlen-1, 2))) {
					// if there's a space right before or after the break, safe to just trim
					$first = rtrim(substr($text, 0, $maxlen));
					$text = ltrim(substr($text, $maxlen));
				} else {
					$first = substr($text, 0, $maxlen);
					if(preg_match("~^(.*\S)\s\S+\$~u", $first, $m)) {
						$first = $m[1];
						$text = ltrim(substr($text, strlen($first)));
					} else {
						$text = substr($text, $maxlen);
					}
				}
				if($add_indent) $first = str_repeat(' ', $indent) . $first;
				$ret[] = $first;
				$offset = $indent;
				$add_indent = true;
			}
			$first = false;
		}
		return $offset;
	}
	
	function formatTextMessage(array $msgs, array $params) {
		$ret = array();
		$idx = 0;
		$title_sep = "\n".str_repeat('-', 40)."\n";
		$grp_sep = "\n".str_repeat('=', $this->text_line_length)."\n";
		foreach($msgs as $msg) {
			$text = array();
			if(! empty($msg['title'])) {
				$this->formatWrappedText('>> '.$msg['title'], $text, 0, 3);
				$text[] = $title_sep;
			}
			if(! empty($msg['rows'])) {
				foreach($msg['rows'] as $row) {
					$add_blank = 0;
					if(! empty($row['head'])) {
						if($add_blank) $text[] = '';
						$this->formatWrappedText($row['head'], $text, 2, 2, true);
						$add_blank = 1;
					}
					if(! empty($row['records'])) {
						foreach($row['records'] as $rec) {
							if(! empty($rec['name'])) {
								if($add_blank) $text[] = '';
								$this->formatWrappedText('  * ' . $rec['name'] . ' *', $text, 2, 4);
								$add_blank = 1;
							}
							if(! empty($rec['props'])) {
								if($add_blank) $text[] = '';
								foreach($rec['props'] as $prop) {
									if(! isset($prop['name']) || ! isset($prop['value']))
										continue;
									$pt = $prop['name'] . $prop['value'];
									$this->formatWrappedText($pt, $text, 2, 2, true);
								}
								$add_blank = 1;
							}
							if(! empty($rec['url'])) {
								if($add_blank) $text[] = '';
								$pfx = isset($rec['url_text']) ? $rec['url_text'] : $this->translate('LBL_RECORD_DETAILS');
								if($pfx) $text[] = '  '.$pfx;
								$url = $rec['url'];
								if(strpos($url, '://') === false)
									$url = $this->getBaseUrl() . $url;
								$text[] = "  <$url>";
								$add_blank = 1;
							}
						}
					}
					if(! empty($row['body'])) {
						if($add_blank) $text[] = '';
						$this->formatWrappedText($row['body'], $text, 2, 2, true);
						$add_blank = 1;
					}
					if(! empty($row['buttons'])) {
						foreach($row['buttons'] as $btn) {
							if(! isset($btn['label']) || ! isset($btn['url']))
								continue;
							if($add_blank) $text[] = '';
							$text[] = '  '.$btn['label'].':';
							$url = $btn['url'];
							if(strpos($url, '://') === false)
								$url = $this->getBaseUrl() . $url;
							$text[] = "  <{$url}>";
							$add_blank = 1;
						}
					}
				}
			}
			if($text) {
				if($idx ++)
					$ret[] = $grp_sep;
				array_extend($ret, $text);
			}
		}
		if($ret) {
			$ret[] = $title_sep;
			$ret[] = $this->getFooterText();
			if(! $this->getExternal()) {
				if($this->getTarget('bean_name') == 'User' && ($user_id = $this->getTarget('id'))) {
					$adj_url = $this->getAdjustPrefsUrl($user_id);
					if($adj_url) {
						$adj_url = $this->getBaseUrl() . $adj_url;
						$ret[] = '';
						$ret[] = $this->translate('LBL_ADJUST_PREFS').':';
						$ret[] = "<$adj_url>";
					}
				}
			}
			$ret[] = "\n";
		}
		return implode("\n", $ret);
	}
}

