<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


class NotifyIncludes {
	var $attachments = array();
	var $includes = array();

	function resetIncludes() {
		$this->attachments = array();
		$this->includes = array();
	}
	
	function copyIncludes(NotifyIncludes $target) {
		array_extend($target->attachments, $this->attachments);
		array_extend($target->includes, $this->includes);
	}
	
	function copyMailAttachments($mailer) {
		foreach($this->attachments as $aid => $att) {
			$fname = array_get_default($att, 'filename', '');
			if($fname) $fname = filename_safe_string($fname);
			if(! empty($att['inline'])) {
				if(isset($att['path']))
					$mailer->addEmbeddedImage($att['path'], $aid, $fname);
				else if(isset($att['data']))
					$mailer->addStringEmbeddedImage($att['data'], $aid, $fname);
			} else if(isset($att['path']))
				$mailer->addAttachment($att['path'], $fname, $att['encoding'], $att['type']);
			else if(isset($att['data']))
				$mailer->addStringAttachment($att['data'], $fname, $att['encoding'], $att['type']);
		}
	}
	
	function addImageAttachment($path, $inline=true, $encoding='base64', $type='', $id=null) {
		return $this->addFileAttachment($path, '', $inline, $encoding, $type, $id);
	}
	
	function addFileAttachment($path, $filename='', $inline=false, $encoding='base64', $type='', $id=null) {
		if(! isset($id)) {
			foreach($this->attachments as $chkid => $chkv) {
				if(isset($chkv['path']) && $chkv['path'] == $path && $chkv['inline'] == $inline)
					return $chkid;
			}
		}
		if(! isset($id)) $id = create_guid();
		$this->attachments[$id] = compact('path', 'filename', 'inline', 'encoding', 'type');
		return $id;
	}
	
	function addDataAttachment($data, $filename='', $inline=false, $encoding='base64', $type='', $id=null) {
		if(! isset($id)) $id = create_guid();
		$this->attachments[$id] = compact('data', 'filename', 'inline', 'encoding', 'type');
		return $id;
	}
	
	function addNoteAttachment($note, $filename='', $inline=false, $encoding='base64', $type='', $id=null) {
		if(is_string($note))
			$note = ListQuery::quick_fetch('Note', $note);
		if($note instanceof RowResult && ! $note->failed) {
			require_once('include/upload_file.php');
			$path = $note->getField('filename');
			if(! $filename) $filename = UploadFile::get_display_filename($path);
			if(! $type) $type = $note->getField('file_mime_type', '');
			$path = UploadFile::get_path($path, $note->getField('id'));
			return $this->addFileAttachment($path, $filename, $inline, $encoding, $type, $id);
		}
	}
	
	function addDataInclude($data, $context='body') {
		if($context == 'css') $context = 'head';
		$this->includes[] = compact('data', 'context');
	}
	
	function getIncludeData($context) {
		$ret = '';
		foreach($this->includes as $i) {
			if($i['context'] == $context)
				$ret .= $i['data'];
		}
		return $ret;
	}
}
