<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


require_once('include/SugarPHPMailer.php');
require_once('include/Notify/NotifyFormatter.php');
require_once('include/Notify/NotifyTargets.php');

class NotifyMailer extends NotifyTargets {
	var $decorate_sender;
	var $prefix_subject = true;
	var $format_params;
	var $from_user_id;
	var $from_system_name;
	var $from_system_addr;
	var $mark_auto_generated = true;
	var $in_reply_to;

	
	function __construct() {
		$this->decorate_sender = AppConfig::setting('notify.decorate_sender', true);
		$this->from_system_name = AppConfig::setting('notify.from_name');
		$this->from_system_addr = AppConfig::setting('notify.from_address');
	}
	
	function setFromUserId($id=null) {
		$this->from_user_id = $id;
	}
	
	function setFormatParams($params=null) {
		$this->format_params = $params;
	}
	
	function setInReplyTo($message_id) {
		$this->in_reply_to = $message_id;
	}
	
	function setPrefixSubject($pfx=true) {
		$this->prefix_subject = $pfx;
	}
	
	function getSender() {
		$from_addr = $this->from_system_addr;
		$from_name = $this->from_system_name;
		$custom_addr = false;
		if($this->from_user_id) {
			$user = get_user_info($this->from_user_id);
			if($user && ! empty($user['email1'])) {
				$from_addr = $user['email1'];
				$from_name = trim($user['first_name'] . ' ' . $user['last_name']);
				$alt_name = UserPreference::getPreference('mail_fromname', 'global', $user['id']);
				if($alt_name) $from_name = $alt_name;
				$alt_addr = UserPreference::getPreference('mail_fromaddress', 'global', $user['id']);
				if($alt_addr) $from_addr = $alt_addr;
				$custom_addr = true;
			}
		}
		return compact('from_addr', 'from_name', 'custom_addr');
	}
	
	function renderEmail(NotifyFormatter $fmt, array $msgs) {		
		$queue = $text_queue = array();
		$textf = clone($fmt);
		$textf->setFormat('text');
		foreach($msgs as $m) {
			$mf = $m->formatMessage($fmt);
			if($mf) {
				$queue[] = $mf;
				$text_queue[] = $m->formatMessage($textf);
			}
		}
		if($queue) {
			$do_prefix = $this->prefix_subject;
			if(count($queue) == 1 && ! empty($queue[0]['subject'])) {
				if(! empty($text_queue[0]['subject']))
					$subject = $text_queue[0]['subject'];
				else
					$subject = $queue[0]['subject'];
				$do_prefix = array_get_default($text_queue[0], 'prefix_subject', $do_prefix);
			} else
				$subject = $fmt->translate('LBL_SUBJECT_DEFAULT');
			if($subject && $do_prefix)
				$subject = $fmt->translate('LBL_SUBJECT_PREFIX') . ' ' . $subject;
			
			$body = $fmt->renderMessages($queue, $this->format_params);
			if($body) {
				$text_body = $textf->renderMessages($text_queue, $this->format_params);
				$textf->copyIncludes($fmt);
				return compact('subject', 'body', 'text_body');
			}
		}
	}
	
	function sendMessage($m) {
		return $this->sendMessages(array($m));
	}
	
	function sendMessages(array $msgs) {
		$targets = $this->prepareTargets();
		if(! $targets) return 0;
		$fmt = NotifyFormatter::create_standard();
		$mail = new SugarPHPMailer();
		$mail->internalMessage = true;
		$sender = $this->getSender();
		
		$sent = 0;
		foreach($targets as $index => $target) {
			if($index)
				$fmt->reset();
			$fmt->setTarget($target);			
			$render = $this->renderEmail($fmt, $msgs);
			if(! $render) {
				if($this->debug_targets) {
					$tgt = $this->debugTarget($target);
					$this->log("skipped email: no message body [$tgt]");
				}
				continue;
			}

			$mail->InitForSend();
			
			$fname = $sender['from_name'];
			if($sender['custom_addr'] && $this->decorate_sender)
				$fname .= ' ' . $fmt->translate('LBL_SENDER_SUFFIX');
			$mail->setFrom($sender['from_addr'], $fname);
			// we may be sending on behalf of a user, but this is still the sender
			$mail->Sender = $this->from_system_addr;
			
			$mail->addAddress($target['email'], $target['name']);
			$mail->Subject = $render['subject'];
			$mail->Body = $render['body'];
			$mail->AltBody = $render['text_body'];
			$mail->isHTML(true);
			
			$fmt->copyMailAttachments($mail);
			
			if($this->mark_auto_generated) {
				$mail->addCustomHeader('Auto-Submitted', 'auto-generated');
				$mail->addCustomHeader('Precedence', 'bulk');
			}
			if($this->in_reply_to) {
				$mail->addCustomHeader('In-Reply-To', $this->in_reply_to);
			}
			
			$mail->prepForOutbound();
			$mail->send();
			$sent ++;
		}
		return $sent;
	}
}



