<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


require_once('include/Notify/NotifyIncludes.php');
require_once('include/Notify/NotifyFormatter.php');


abstract class NotifyMessageBase {
	var $prefix_subject;
	abstract function formatMessage(NotifyFormatter $fmt);
	
	function renderStandalone($external=true) {
		$fmt = NotifyFormatter::create_standalone($external);
		return $fmt->renderMessage($this);
	}
	
	function getMailer() {
		require_once('include/Notify/NotifyMailer.php');
		$mail = new NotifyMailer();
		$pfx = $this->getPrefixSubject();
		if(isset($pfx)) $mail->setPrefixSubject($pfx);
		return $mail;
	}
	
	function getCustomTemplate() {
	}
	
	function getTemplateParams() {
	}
	
	function getPrefixSubject() {
		if(isset($this->prefix_subject)) return $this->prefix_subject;
	}
}


class NotifyMessage extends NotifyMessageBase {
	var $title;
	var $subject;
	var $kind;
	var $rows;
	var $form;
	var $replace_params;
	var $includes;
	
	
	function __construct($data=null) {
		if($data) {
			foreach($data as $k => $v) {
				if($k == 'body') {
					$this->rows = array(
						array(
							'body' => $v,
						),
					);
				}
				else if(property_exists($this, $k))
					$this->$k = $v;
			}
		}
		$this->includes = new NotifyIncludes();
	}
	
	function formatString(NotifyFormatter $fmt, $value) {
		if(is_array($value)) {
			$value = $fmt->translate($value[0], $value[1]);
			$value = $this->replaceParams($value);
		}
		return $value;
	}
	
	// custom variables for subject and title
	function getReplaceParams() {
		return $this->replace_params ? $this->replace_params : array();
	}
	function setReplaceParams($params) {
		$this->replace_params = $params;
	}
	function replaceParams($val, $ext_params=null) {
		$replace = $this->getReplaceParams();
		if($ext_params) array_update($replace, $ext_params);
		if($replace)
			$val = str_replace(array_keys($replace), array_values($replace), $val);
		return $val;
	}
	
	function formatMessage(NotifyFormatter $fmt) {
		$title = $this->formatString($fmt, $this->title);
		$subject = $this->formatString($fmt, $this->subject);
		$kind = $this->kind;
		$rows = $this->rows;
		$form = $this->form;
		if(! is_array($rows))
			$rows = array();
		foreach($rows as &$r) {
			if(! is_array($r)) $r = array();
			if(isset($r['body']) && is_array($r['body'])) {
				$r['body'] = $this->formatString($fmt, $r['body']);
				$r['text_body'] = $r['body'];
			}
		}
		$this->includes->copyIncludes($fmt);
		return compact('title', 'subject', 'kind', 'rows', 'form');
	}
	
	function getIncludes() {
		return $this->includes;
	}
}

class NotifyErrorMessage extends NotifyMessage {
	static function show_error($body, $title=null, $kind='error', $extern=true) {
		$err = new NotifyErrorMessage(compact('title', 'body', 'kind'));
		echo $err->renderStandalone($extern);
	}
}

