<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


class NotifyTargets {
	var $targets = array();
	var $default_language;
	var $debug_targets = false;
	var $debug_log = array();


	static function target_list(array $ts) {
		$list = new NotifyTargets();
		$list->addTargets($ts);
		return $list;
	}

	function setDefaultLanguage($lang=null) {
		$this->default_language = $lang;
	}
	
	function copyTargets(NotifyTargets $t) {
		$this->targets = $t->targets;
		if(! $this->targets) $this->targets = array();
		$this->default_language = $t->default_language;
	}
	
	function addTargetId($model, $id, $params=null) {
		$info = $this->loadTargetId($model, $id, false);
		if($info) {
			if($params)
				array_update($info, $params);
			return $this->addTargetInfo($info);
		}
	}
	function loadTargetId($model, $id, $extend=true) {
		if(! $id) return;
		if($model == 'User')
			$info = get_user_info($id);
		else if($model == 'Contact' || $model == 'Lead' || $model == 'Prospect')
			$info = AppConfig::db_row($model, $id);
		else
			return;
		if($info) {
			$info['bean_name'] = $model;
			if($extend) $info = $this->normalizeTargetInfo($info);
			return $info;
		}
	}
	
	function addTarget($email, $name=null) {
		return $this->addTargetInfo(compact('email', 'name'));
	}
	function addTargets(array $ts, $normalize=true) {
		foreach($ts as $target) {
			if(is_array($target)) {
				if(isset($target['bean_name']) && isset($target['id'])
				&& ! array_key_exists('email', $target) && ! array_key_exists('email1', $target))
					$this->addTargetId($target['bean_name'], $target['id']);
				else
					$this->addTargetInfo($target, $normalize);
			}
		}
	}
	
	function normalizeTargetInfo($target) {
		if(empty($target['email'])) {
			if(! empty($target['email1']))
				$target['email'] = $target['email1'];
			else if(! empty($targ['email2']))
				$target['email'] = $target['email2'];
			else
				$target['email'] = ''; // no bueno
		}
		$target['email'] = strtolower(trim($target['email']));
		if(empty($target['name'])) {
			if(! empty($target['last_name']))
				$target['name'] = $target['first_name'] . ' ' . $target['last_name'];
			else
				$target['name'] = '';
		}
		$target['name'] = trim($target['name']);
		if(! isset($target['bean_name']))
			$target['bean_name'] = null;
		if(! isset($target['id']))
			$target['id'] = null;
		if(! isset($target['external']))
			$target['external'] = ($target['bean_name'] != 'User' || ! $target['id']);
		return $target;
	}
	function addTargetInfo(array $target, $normalize=true) {
		if($normalize)
			$target = $this->normalizeTargetInfo($target);
		if(! empty($target['email'])) {
			$this->targets[$target['email']] = $target;
			return $target['email'];
		}
		else if($this->debug_targets) {
			$tgt = $this->debugTarget($target);
			$this->log('skipped target: no email address ['.$tgt.']');
		}
	}
	function getTarget($email) {
		return array_get_default($this->targets, $email);
	}
	
	function removeTarget($email) {
		unset($this->targets[$email]);
	}
	function clearTargets() {
		$this->targets = array();
	}
	
	function addRelatedTargets($record, $link_name) {
		if(! $record->linkDefined($link_name)) {
			if($this->debug_targets)
				$this->log("unknown link name: $link_name");
			return;
		}
		$added = 0;
		$lq = $record->getLinkQuery($link_name, true);
		$r = $lq->runQuery();
		if($r && ! $r->failed) {
			foreach($r->rows as $row) {
				$row['bean_name'] = $r->base_model;
				$this->addTargetInfo($row);
				$added ++;
			}
		}
		return $added;
	}
		
	function prepareTargets() {
		$ret = array();
		foreach($this->targets as $target) {
			$skip = false;
			if(! empty($target['email_opt_out']))
				$skip = 'opted out';
			else if(! empty($target['invalid_email']))
				$skip = 'invalid email';
			else if(isset($target['receive_notifications']) && ! $target['receive_notifications'])
				$skip = 'notifications off';
			if($skip) {
				if($this->debug_targets) {
					$tgt = $this->debugTarget($target);
					$this->log("skipped target: $skip [$tgt]");
				}
				continue;
			}
			if(! isset($target['language']) && $this->default_language)
				$target['language'] = $this->default_language;
			$ret[] = $target;
		}
		return $ret;
	}
	
	function debugTarget($target) {
		$parts = array();
		if(isset($target['bean_name']) && isset($target['id']))
			$parts[] = $target['bean_name'].'/'.$target['id'];
		if(isset($target['email']))
			$parts[] = $target['email'];
		if(isset($target['name']))
			$parts[] = $target['name'];
		return implode(' ', $parts);
	}
	
	function log($msg) {
		$this->debug_log[] = $msg;
	}
}

