<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


require_once('include/Notify/NotifyMessage.php');

class RecordActionMessage extends NotifyMessage {
	var $model;
	var $records;
	var $fetch_ids = array();
	var $action;
	var $fields = array();
	var $required_fields;
	var $performer_id;
	var $apply_acl = false;
	var $body;
	var $display_name = '_display';
	var $format_hooks = array();
	var $url_template;
	var $template_path;
	var $buttons;
	
	
	function __construct($data=null) {
		if(is_array($data)) {
			if (isset($data['override_fields'])) {
				$data['fields'] = $data['override_fields'];
			}
			foreach($data as $k => $v) {
				switch($k) {
				case 'model':
					$this->setModel($v);
					break;
				case 'action':
					$this->setAction($v);
					break;
				case 'performer_id':
					$this->setPerformerId($v);
					break;
				case 'fields':
					$this->setFields($v);
					break;
				case 'required_fields':
					$this->setRequiredFields($v);
					break;
				case 'display_name':
					$this->setDisplayName($v);
					break;
				case 'body':
					$this->setBody($v);
					break;
				case 'format_hooks':
					$this->addFormatHooks($v);
					break;
				case 'url_template':
					$this->setUrlTemplate($v);
					break;
				case 'template_path':
					$this->setCustomTemplate($v);
					break;
				default:
					continue 2;
				}
				unset($data[$k]);
			}
		}
		parent::__construct($data);
	}
	
	static function for_action($model, $act, $performer_id=null, $ext_params=null) {
		$spec = self::load_spec($model, $act);
		if(is_array($spec) && empty($spec['disabled'])) {
			if(isset($spec['class'])) {
				$cls = $spec['class'];
				unset($spec['class']);
				if(isset($spec['file']))
					require_once($spec['file']);
				else
					require_once('include/Notify/'.$cls.'.php');
			} else
				$cls = 'RecordActionMessage';
			$params = $spec;
			$params['model'] = $model;
			$params['action'] = $act;
			$params['performer_id'] = $performer_id;
			if($ext_params)
				array_extend($params, $ext_params);
			$obj = new $cls($params);
			return $obj;
		}
	}
	
	static function load_spec($model, $act) {
		$base = AppConfig::setting("notify.messages.$act", array());
		$mod = AppConfig::setting("display.notify.$model.messages.$act");
		if(! is_array($mod))
			return;
		$base = self::merge_spec($base, $mod);
		if(! empty($base['inherit'])) {
			$base2 = self::load_spec($model, $base['inherit']);
			unset($base2['title']);
			unset($base2['body']);
			$base = self::merge_spec($base2, $base);
			unset($base['inherit']);
		}
		return $base;
	}
	static function merge_spec($base, $ext) {
		foreach($ext as $k => $v) {
			if(! isset($base[$k]) || $k == 'class' || $k == 'file')
				$base[$k] = $v;
			else if($k == 'fields')
				$base[$k] = $v;
			else if($k == 'required_fields')
				array_extend($base[$k], $v);
		}
		return $base;
	}
	
	function setModel($model) {
		if($model instanceof ModelDef)
			$this->model = $model->getModelName();
		else
			$this->model = $model;
	}
	
	function getFields() {
		return $this->fields;
	}
	function setFields($fields) {
		if($fields) $this->fields = $fields;
		else $this->fields = array();
	}
	function setRequiredFields($fields) {
		$this->required_fields = $fields;
	}
	
	function setBody($body) {
		$this->body = $body;
	}
	
	function addFormatHooks(array $hooks) {
		array_extend($this->format_hooks, $hooks);
	}
	
	function setUrlTemplate(array $tpl) {
		$this->url_template = $tpl;
	}
	
	function getCustomTemplate() {
		return $this->template_path;
	}
	function setCustomTemplate($path) {
		$this->template_path = $path;
	}

	function getQueryFields() {
		$fs = array();
		foreach($this->fields as $k => $f) {
			if(is_string($f)) {
				if( ($ep = strpos($f, '!')) )
					$f = array('alias' => $f, 'name' => substr($f, 0, $ep), 'format' => substr($f, $ep+1));
				else
					$f = array('name' => $f);
			} 
			if(is_int($k))
				$fs[] = $f;
			else
				$fs[$k] = $f;
		}
		if($this->required_fields)
			array_extend($fs, $this->required_fields);
		// add fields required by the display name (used by meeting invite: ical attachment)
		$disp = AppConfig::setting("model.detail.{$this->model}.display_name", 'name');
		if(is_string($disp))
			$fs[] = $disp;
		else if(is_array($disp) && isset($disp['fields']))
			array_extend($fs, $disp['fields']);
		$fs['_display'] = $this->getDisplayName();
		return $fs;
	}
	
	function setAction($action) {
		$this->action = $action;
	}
	
	function setPerformerId($uid) {
		$this->performer_id = $uid;
	}
	
	function addRecordId($id) {
		$this->fetch_ids[] = $id;
	}
	function addRecordIds(array $ids) {
		array_extend($this->fetch_ids, $ids);
	}
	function setRecords(ListResult $records) {
		$this->records = $records;
	}
	function setRecord(RowResult $record) {
		$this->records = $record->makeListResult();
	}
	
	function populateRecords() {
		if($this->fetch_ids) {
			$fetch = array();
			foreach(array_unique($this->fetch_ids) as $id) {
				if(! $id || isset($this->records[$id])) continue;
				$fetch[] = $id;
			}
			$this->fetchRecords($fetch);
			$this->fetch_ids = array();
		}
	}
	
	function fetchRecord($id) {
		$c = $this->fetchRecords(array($id));
		if($c) {
			$idx = $this->getRecordIndexes();
			return $idx[0];
		}
	}
	
	function fetchRecords($ids) {
		$ret = array();
		$qfs = $this->getQueryFields();
		if(! $qfs) $qfs = true;
		$lq = new ListQuery($this->model, $qfs);
		if($this->apply_acl && $this->performer_id) {
			$lq->setAclUserId($this->performer_id);
			$lq->addAclFilter('view');
		}
		$r = $lq->queryRecordSet($ids);
		$this->records = $r;
		return $r->getResultCount();
	}
	
	function getRecords() {
		return $this->records;
	}
	
	function getSingleRecord() {
		if($this->records && $this->records->getResultCount() == 1) {
			reset($this->records->rows);
			$idx = key($this->records->rows);
			return $this->records->getRowResult($idx);
		}
	}
	
	function getRecordField($name, $default=null, $formatted=false) {
		$rec = $this->getSingleRecord();
		if($rec) return $rec->getField($name, $default, $formatted);
	}
	function setRecordField($name, $value, $formatted=false) {
		if($this->records->getResultCount() == 1) {
			$idx = $this->records->getRowIndexes();
			if($formatted)
				$this->records->formatted_rows[$idx[0]][$name] = $value;			
			else
				$this->records->rows[$idx[0]][$name] = $value;
			return true;
		}
	}
	
	function getRecordCount() {
		return $this->records ? $this->records->getResultCount() : 0;
	}
	function getRecordIndexes() {
		return $this->records ? $this->records->getRowIndexes() : array();
	}
	function getRecord($idx) {
		if($this->records) return $this->records->getRowResult($idx);
	}
		
	function formatRecords(NotifyFormatter $fmt) {
		if($this->records) {
			$field_fmt = $fmt->getFieldFormatter();
			$field_fmt->formatListResult($this->records);
			foreach($this->format_hooks as $hook) {
				call_hook_function($hook, $this->records->base_model, array($this->records, $fmt, $this));
			}
		}
	}
	
	function getCustomRecordUrl(NotifyFormatter $fmt, RowResult $record) {
		if($this->url_template) {
			$url = $this->replaceStringFields($fmt, $this->url_template, false, $record);
		}
	}
	
	function getDisplayName() {
		return $this->display_name;
	}
	function setDisplayName($name) {
		$this->display_name = $name;
	}
	
	function getRecordIcon(RowResult $record) {
		return $record->getModuleDir();
	}
	
	function renderRecords(NotifyFormatter $fmt) {
		if(! $this->records) return;

		$recs = array();
		foreach($this->records as $rec) {
			$rname = $rec->getField('_display', null, true);
			$subj = $rec->getField('_display');
			$mod = AppConfig::module_for_model($this->model);
			$recrow = array();
			$custom_url = $this->getCustomRecordUrl($fmt, $rec);
			if($fmt->getFormat() == 'html')
				$recrow['name'] = $fmt->formatRecordLink($rname, $this->model, $rec->getPrimaryKeyValue(), $custom_url, $this->getRecordIcon($rec));
			else {
				$recrow['name'] = $rname;
				if(! $fmt->getExternal())
					$recrow['url'] = $custom_url ? $custom_url : $fmt->defaultRecordUrl($this->model, $rec->getPrimaryKeyValue());
			}
			foreach($this->fields as $f) {
				if(is_array($f)) $f = array_get_default($f, 'name');
				if(! $f) continue;
				$def = $rec->getFieldDefinition($f);
				if(! $def || ! isset($def['name'])) continue;
				$fn = isset($def['alias']) ? $def['alias'] : $def['name'];
				$val = $rec->getField($fn, null, true);
				if(! isset($val) || $val === '') continue;
				if(isset($def['label']))
					$lbl = $def['label'];
				else {
					$vmod = array_get_default($def, 'vname_module', $mod);
					$lbl = $fmt->translate($def['vname'], $vmod);
				}
				$recrow['props'][] = array(
					'name' => $lbl . $fmt->translate('LBL_SEPARATOR', 'app'),
					'value' => $val,
					'field' => $fn,
				);
			}
			$recs[] = $recrow;
		}
		return $recs;
	}
	
	function getMessageSubject(NotifyFormatter $fmt, $custom=null) {
		if(! empty($custom)) {
			$found = $custom;
		} else if($this->subject) {
			$found = $this->formatString($fmt, $this->subject);
		} else {
			$test = array();
			$count = $this->getRecordCount();
			$single = $this->getSingleRecord();
			$name = $single ? $single->getField('_display') : null;
			if($this->action) {
				if($count > 1)
					$test[] = 'LBL_SUBJECT_ACTION_MULTI';
				else if(strlen($name)) {
					$test[] = 'LBL_SUBJECT_'.strtoupper($this->action).'_'.strtoupper($this->model);
					$test[] = 'LBL_SUBJECT_'.strtoupper($this->action);
					$test[] = 'LBL_SUBJECT_ACTION_TEMPLATE';
				}
			}
			if(strlen($name) && $count <= 1)
				$test[] = 'LBL_SUBJECT_TEMPLATE';
			if(! $test)
				return;
			foreach($test as $t) {
				$found = $fmt->translate($t);
				if(isset($found))
					break;
			}
		}
		$found = preg_replace('~\*([\w\s]+)\*~u', '$1', $found);
		$found = $this->replaceStringFields($fmt, $found);
		return $found;
	}

	function getMessageTitle(NotifyFormatter $fmt, $custom=null) {
		$check = array();
		$model = $this->model;
		$action = $this->action;
		$count = $this->getRecordCount();
		if(! empty($custom)) {
			$found = $custom;
		} else if($this->title) {
			$found = $this->formatString($fmt, $this->title);
		} else {
			$pfx = array(strtoupper($action).'_'.strtoupper($model), strtoupper($action), 'TEMPLATE');
			$sfx = array();
			$uname = $fmt->formatUser($this->performer_id);
			if($fmt->getTarget('bean_name') == 'User') {
				$uid = $fmt->getTarget('id');
				if($uid && ($uid == $this->performer_id || $uid == $this->getRecordField('assigned_user_id'))) {
					if($count > 1) {
						if($uname)
							$sfx[] = 'MULTI_OWN_BYUSER';
						$sfx[] = 'MULTI_OWN';
					} else {
						if($uname)
							$sfx[] = 'OWN_BYUSER';
						$sfx[] = 'OWN';
					}
				}
			}
			if($count > 1) {
				if($uname)
					$sfx[] = 'MULTI_BYUSER';
				$sfx[] = 'MULTI';
			}
			$sfx[] = '';
			$found = $lbl = null;
			foreach($pfx as $p) {
				$plbl = 'LBL_TITLE_'.$p;
				foreach($sfx as $s) {
					$lbl = $plbl;
					if($s) $lbl .= '_'.$s;
					$t = $fmt->translate($lbl);
					if(isset($t)) {
						$found = $t;
						break 2;
					}
				}
			}
			if($found) {
				if($uname && strpos($lbl, 'BYUSER') === false) {
					$user_lbl = $fmt->translate('LBL_TITLE_BYUSER');
					if($user_lbl)
						$found = str_replace('{TEMPLATE}', $found, $user_lbl);
				}
			}
		}
		$found = preg_replace('~\*([\w\s]+)\*~u', $fmt->getFormat() == 'html' ? '<b>$1</b>' : '$1', $found);
		$found = $this->replaceStringFields($fmt, $found);
		return $found;
	}
		
	function getMessageKind(NotifyFormatter $fmt) {
		if($this->action == 'assigned')
			return 'assigned';
	}
	
	function getMessageBody(NotifyFormatter $fmt) {
		if(isset($this->body)) {
			if(is_array($this->body)) {
				$str = $this->formatString($fmt, $this->body);
			} else
				$str = $this->body;
			if($fmt->getFormat() == 'html')
				$str = nl2br($str);
			$ret = $this->replaceStringFields($fmt, $str);
			if($fmt->getFormat() == 'html') {
				$ret = preg_replace('~(?<=\s)\*([^\n]+)\*(?=\s)~', $fmt->getFormat() == 'html' ? '<b>$1</b>' : '$1', $ret);
				$ret = preg_replace_callback('~<(\w+://[^>]+)>~', function ($matches) use($fmt) {
					return $fmt->formatLink($matches[1]);
				}, $ret);
			}
			return $ret;
		}
	}
	
	function getMessageButtons(NotifyFormatter $fmt) {
		if($this->buttons) {
			$ret = array();
			foreach($this->buttons as $k => $b) {
				if(isset($b['label']))
					$b['label'] = $this->formatString($fmt, $b['label']);
				if(isset($b['url']))
					$b['url'] = $this->replaceStringFields($fmt, $b['url'], false);
				$ret[$k] = $b;
			}
			return $ret;
		}
	}
	
	function replaceStringFields(NotifyFormatter $fmt, $str, $formatted=true, $record=null, $ext_replace=null) {
		// only call after record has been formatted, if necessary
		if(! isset($record)) $record = $this->getSingleRecord();
		$replace = array();
		$replace['{SYSTEM}'] = $fmt->getSystemName();
		$replace['{APP_VERSION}'] = $fmt->getAppVersion();
		$replace['{BASE_URL}'] = $fmt->getBaseUrl();
		if($formatted) {
			$replace['{ACTION}'] = $this->action ? $fmt->translate_dd('activity_log_status_dom', $this->action, $this->action) : '';
			$replace['{BY_USER}'] = $fmt->formatUser($this->performer_id);
		}
		else {
			$replace['{ACTION}'] = $this->action ? $this->action : '';
			$replace['{BY_USER}'] = $this->performer_id ? $this->performer_id : '';
		}
		if($record) {
			$replace['{MODULE}'] = $module = $record->getModuleDir();
			$replace['{MODEL}'] = $record->base_model;
			$replace['{RECORD}'] = $record->getPrimaryKeyValue();
			$replace['{RECORD_COUNT}'] = '1';
			if($formatted) {
				$replace['{MODULE}'] = $fmt->translate_dd('moduleList', $module, $module);
				$replace['{MODEL}'] = $fmt->translate_dd('moduleListSingular', $module, $replace['{MODEL}']);
			}
			$replace['{DISPLAY_NAME}'] = $record->getField('_display', '', $formatted);
			foreach($record->fields as $f) {
				if(isset($f['name'])) {
					$k = $f['name'];
					$replace['{'.strtoupper($k).'}'] = $record->getField($k, '', $formatted);
				}
			}
		} else if($this->records) {
			$replace['{MODULE}'] = $module = $this->records->getModuleDir();
			$replace['{MODEL}'] = $this->records->base_model;
			$replace['{RECORD}'] = '';
			$replace['{RECORD_COUNT}'] = format_number($this->getRecordCount(), 0);
			if($formatted) {
				$replace['{MODULE}'] = $fmt->translate_dd('moduleList', $module, $module);
				$replace['{MODEL}'] = $fmt->translate_dd('moduleListSingular', $module, $replace['{MODEL}']);
			}
			$replace['{DISPLAY_NAME}'] = '';
		}
		if($ext_replace) array_extend($replace, $ext_replace);
		if($replace)
			$str = str_replace(array_keys($replace), array_values($replace), $str);
		return $str;
	}
	
	function renderRows(NotifyFormatter $fmt) {
		return array(
			array(
				'body' => $this->getMessageBody($fmt),
				'records' => $this->renderRecords($fmt),
				'buttons' => $this->getMessageButtons($fmt),
			)
		);
	}

	function preFormatMessage(NotifyFormatter $fmt) {
	}
	
	function checkSkipMessage(NotifyFormatter $fmt) {
		if(! $this->getRecordCount())
			return true;
	}
		
	function formatMessage(NotifyFormatter $fmt) {
		$this->populateRecords();
		if($this->checkSkipMessage($fmt))
			return;

		$this->preFormatMessage($fmt);

		$this->formatRecords($fmt);
				
		// fields like $title and $kind may be manually set as in a plain NotifyMessage
		// in order to override the default values
		$msg = parent::formatMessage($fmt);
		$msg['title'] = $this->getMessageTitle($fmt, array_get_default($msg, 'title'));
		$msg['subject'] = $this->getMessageSubject($fmt, array_get_default($msg, 'subject'));
		if(empty($msg['kind']))
			$msg['kind'] = $this->getMessageKind($fmt);
		$msg['rows'] = $this->renderRows($fmt);
		
		$this->postFormatMessage($fmt, $msg);
		return $msg;
	}
	
	function postFormatMessage(NotifyFormatter $fmt, array &$msg) {
	}
}

