<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


require_once('include/Notify/RecordActionMessage.php');

class ReportMessage extends RecordActionMessage {
	var $default_format = 'pdf';
	var $list_fmt;
	
	function getReplaceParams() {
		$record = $this->getSingleRecord();
		$params = parent::getReplaceParams();
		if($record)
			$params['{REPORT_NAME}'] = $record->getField('report_name');
		return $params;
	}
	
	function getDisplayName() {
		return 'report.name';
	}
	
	function getRecordIcon(RowResult $result) {
		return 'Reports';
	}
	
	function getCustomRecordUrl(NotifyFormatter $fmt, RowResult $record) {
		return 'index.php?module='.$record->getField('primary_module').'&action=index&layout=Reports&report_id='.$record->getField('report_id').'&data_id='.$record->getField('id');
	}

	function getNotifiedUsers() {
		$this->populateRecords();
		$record = $this->getSingleRecord();
		$ret = array();
		if($record && ($report_id = $record->getField('report_id'))) {
			$tg_fields = array(
				'id', 'user_name', 'first_name', 'last_name', 'salutation', 'email1', 'email2',
				'format' => '~join.report_notify_format', 
			);
			$lq = new ListQuery('Report', $tg_fields, array('link_name' => 'notify_users', 'parent_key' => $report_id, 'no_default_order_by' => true));
			$lq->addSimpleFilter('~join.report_notify_enabled', 1, 'eq');
			$targets = $lq->runQuery();
			foreach($targets->getRows() as $targ) {
				$targ['bean_name'] = 'User';
				$ret[] = $targ;
			}
		}
		return $ret;
	}
	
	function getListFormatter() {
		require_once('include/ListView/ListFormatter.php');
		global $log;
		
		$data = $this->getSingleRecord();
		if(! $data) return;
		$data_id = $data->getPrimaryKeyValue();
		if(! isset($this->list_fmt)) {
			$list_fmt = false;
			$model = AppConfig::module_primary_bean($data->getField('primary_module'));
			if(! $model) {
				$log->fatal("Cannot format report data ($data_id): unknown model");
			} else {
				try {
					$list_fmt = new ListFormatter($model);
				} catch(IAHModelError $e) {
					$log->fatal("Cannot format report data ($data_id): {$e->getMessage()}");
					$list_fmt = false;
				}
				if($list_fmt && ! $list_fmt->loadReportResult($data, true, true)) {
					$log->fatal("Error loading report data");
					$list_fmt = false;
				}
			}
			$this->list_fmt = $list_fmt;
		}
		return $this->list_fmt;
	}
	
	function getReportExport(NotifyFormatter $notify) {
		require_once('include/ListView/ListViewExporter.php');
		$list_fmt = $this->getListFormatter();
		if(! $list_fmt) return;

		$format = $notify->getTarget('format', $this->default_format);
		if($format == 'link')
			return;
		$export = new ListViewExporter($format);
		$export->setDataResult($list_fmt->data_result->row);
		$output = $export->outputFormatter($list_fmt, null, false);
		return $output;
	}
	
	function checkSkipMessage(NotifyFormatter $notify) {
		if(! $this->getRecordField('record_count'))
			return true;
		return parent::checkSkipMessage($notify);
	}

	function postFormatMessage(NotifyFormatter $notify, array &$msg) {
		global $log;
		if(! $notify->getStandalone() && $notify->getFormat() == 'text') {
			$output = $this->getReportExport($notify);
			//$for_user = ! $notify->getExternal();
			if($output) {
				if($output['format'] == 'string') {
					$notify->addDataAttachment($output['data'], $output['filename'], false, 'base64', $output['mimetype']);
				} else if($output['format'] == 'file') {
					$notify->addFileAttachment($output['path'], $output['filename'], false, 'base64', $output['mimetype']);
				} else {
					$log->warn("Scheduled Report: unknown output format '{$output['format']}'");
				}
			}
		}
	}
}

