<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


require_once('include/Notify/RecordActionMessage.php');

class SurveyMessage extends RecordActionMessage {
	var $rated = false;
	var $accept_comments = true;
	var $comments;
	var $display_name = 'name';
	

	function __construct($model=null, $action=null, $performer_id=null) {
		parent::__construct($model, $action, $performer_id);
	}
	
	function getMailer() {
		$mailer = parent::getMailer();
		$from_addr = AppConfig::setting('notify.survey_from_address');
		$from_name = AppConfig::setting('notify.survey_from_name');
		if($from_addr) {
			$mailer->from_system_addr = $from_addr;
			if($from_name)
				$mailer->from_system_name = $from_name;
		}
		return $mailer;
	}

	function getStyle() {
		return <<<EOQ
			.rating_outer:hover .star_grey {
				display: none;
			}
			.rating_box:hover > a .star_off {
				display: none;
			}
			.rating_outer:hover .star_blue {
				display: none;
			}
			.rating_outer.flash:hover .star_blue {
				display: inline-block;
			}
EOQ;
	}
	
	function getScript() {
		return <<<EOQ
			function show_rating(val) {
				var outer = document.getElementById('rating_input'), input = document.forms.DetailForm.elements.rating, s;
				if(! outer || ! input) return;
				if(val !== undefined)
					input.value = val;
				val = input.value;
				var stars = outer.getElementsByClassName('star_grey');
				var blues = outer.getElementsByClassName('star_blue');
				for(var i = 0; i < 5 && stars[i]; i++) {
					s = stars[i].style;
					if(i < val) (s.removeProperty||s.removeAttribute).call(s, 'display');
					else s.display = 'none';
					if(blues[i]) {
						s = blues[i].style;
						if(i < val) (s.removeProperty||s.removeAttribute).call(s, 'display');
						else s.display = 'none';
					}
				}
				outer.className = 'rating_outer flash';
				setTimeout(function() { outer.className = 'rating_outer'; }, 200);
			}
EOQ;
	}
	
	function getSurveyUrl(NotifyFormatter $notify) {
		$contact_id = $this->getContactId($notify);
		$record = $this->getSingleRecord();
		if($record && $contact_id) {
			$module = $record->getModuleDir();
			return 'survey.php?module='.$module.'&record=' . $record->getPrimaryKeyValue() . '&contact_id=' . $contact_id;
		}
	}
	
	function getContactId(NotifyFormatter $notify) {
		$contact_id = $notify->getTarget('id');
		if($notify->getTarget('bean_name') == 'Contact')
			return $contact_id;
	}
	
	function getMessageButtons(NotifyFormatter $notify) {
		if(($this->action == 'survey' || $this->action == 'survey_comments') && $notify->getStandalone()) {
			$buttons = array(array(
				'label' => $notify->translate('LBL_SEND_FEEDBACK', 'Cases'),
				'onclick' => 'document.forms.DetailForm.submit(); return false;',
			));
			return $buttons;
		}
		else if($this->action == 'survey') {
			$buttons = array(array(
				'label' => $notify->translate('LBL_SEND_FEEDBACK', 'Cases'),
				'url' => $this->getSurveyUrl($notify),
			));
			return $buttons;
		}
	}
	
	function getMessageTitle(NotifyFormatter $notify, $custom=null) {
		/*if($this->action == 'survey') {
			return $notify->translate('LBL_TITLE_SURVEY_ACASE');
		}*/
		return parent::getMessageTitle($notify, $custom);
	}
	
	function getMessageSubject(NotifyFormatter $notify, $custom=null) {
		if(! empty($custom)) {
			$subject = $custom;
		} else if($notify->getStandalone()) {
			$ofmt = $notify->getFormat();
			$notify->setFormat('text');
			$subject = $this->getMessageTitle($notify);
			$notify->setFormat($ofmt);
		} else if( ($record = $this->getSingleRecord()) ) {
			$subject = case_email_subject($record->getField('name'), $record->getField('case_number'));
			if($this->action == 'created') {
				$base = $notify->translate('LBL_SUBJECT_CREATED_ACASE');
				if($base) $subject = str_replace('{DISPLAY_NAME}', $subject, $base);
			}
		}
		else {
			$subject = null;
		}
		return parent::getMessageSubject($notify, $subject);
	}
	
	function getMessageKind(NotifyFormatter $notify) {
		if($this->action == 'survey' || $this->action == 'survey_comments') {
			return 'query';
		}
		if($this->action == 'survey_thanks') {
			return 'success';
		}
		return parent::getMessageKind($notify);
	}
	
	function preFormatMessage(NotifyFormatter $notify) {
		if(isset($this->records->fields['rating']))
			$this->records->fields['rating']['vname'] = 'LBL_YOUR_RATING';
		if(isset($this->records->fields['account']))
			$this->records->fields['account']['vname'] = 'LBL_ACCOUNT';
		if($this->action == 'survey' || $this->action == 'survey_comments') {
			if($this->action == 'survey')
				$this->setRecordField('rating', 0); // hide previous rating
			array_extend($this->records->fields['rating'], array(
				'base_url' => $this->getSurveyUrl($notify), // set to null to disable editing
				'instant' => ! $notify->getStandalone(),
			));
			if($notify->getStandalone() && $this->accept_comments) {
				$this->records->fields['survey_comments'] = array(
					'name' => 'survey_comments',
					'vname' => 'LBL_YOUR_COMMENTS',
					'type' => 'html',
				);
				$this->setRecordField('survey_comments',
					'<input type="hidden" name="commented" value="1"><textarea name="survey_comments" rows="6" cols="50" style="border: 1px solid #888; font-size: 110%">'.to_html($this->comments).'</textarea>'
				);
			}

			$base = AppConfig::setting('site.survey_url');
			if($base) {
				if(substr($base, -10) == 'survey.php')
					$base = substr($base, 0, -10);
				if(substr($base, -1) != '/') $base .= '/';
				$notify->setBaseUrl($base);
			}
		
			if($notify->getFormat() == 'html')
				$notify->addDataInclude('<style type="text/css">'.$this->getStyle().'</style>', 'css');
			if($notify->getStandalone()) {
				$notify->addDataInclude('<script type="text/javascript">'.$this->getScript().'</script>');
				$params = array();
				foreach(array('module', 'record', 'contact_id', 'rated') as $f)
					if(array_key_exists($f, $_REQUEST))
						$params[] = ['name' => $f, 'value' => $_REQUEST[$f]];
				$this->form = array('name' => 'DetailForm', 'action' => 'survey.php', 'params' => $params);
			}
		}
		if($this->action == 'survey_thanks' && $this->accept_comments && $this->comments) {
			$this->records->fields['survey_comments'] = array(
				'name' => 'survey_comments',
				'vname' => 'LBL_YOUR_COMMENTS',
				'type' => 'text',
				'multiline' => true,
			);
			$this->setRecordField('survey_comments', $this->comments);
		}
	}
}

