<?php

namespace OneCRM\OAuth;

use \UserPreference;
use \AppConfig;
use \ListQuery;
use \RowUpdate;

class OAuthTokenOwner {
	private $type;
	private $id;

	public function __construct($type, $id = "") {
		$this->type = $type;
		$this->id = $id;
	}
	
	public function load() {
		$encoded_token = null;
		switch ($this->type) {
		case 'user_smtp':
			$encoded_token = UserPreference::getPreference('o365_token', 'global', $this->id);
			break;
		case 'user_extra_smtp':
			$account = ListQuery::quick_fetch('EmailPOP3', $this->id);
			if ($account) {
				$encoded_token = $account->getField('smtp_o365_auth_token');
			}
			break;
		case 'system_smtp':
			$encoded_token = AppConfig::setting("email.smtp_o365_token");
			break;
		case 'imap_account':
			$account = ListQuery::quick_fetch('EmailPOP3', $this->id);
			if ($account) {
				$encoded_token = $account->getField('o365_auth_token');
			}
			break;
		case 'google_sync':
			$encoded_token = UserPreference::getPreference('google_access_token', 'global', $this->id);
			if (is_string($encoded_token)) {
				$encoded_token = @json_decode($encoded_token, true);
			}
			break;
		}
		if ($encoded_token) {
			$decoded_token = @json_decode($encoded_token, true);
			if ($decoded_token) {
				unset($decoded_token['expires_in']);
				return $decoded_token;
			}
		}
		return null;
	}

	public function persist(array $token) {
		$json = json_encode($token, JSON_UNESCAPED_SLASHES);
		switch ($this->type) {
		case 'user_smtp':
			UserPreference::setPreference('o365_token', $json, 0, 'global', $this->id);
			UserPreference::savePreferencesToDB($this->id);
			break;
		case 'google_sync':
			UserPreference::setPreference('google_access_token', $json, 0, 'global', $this->id);
			UserPreference::savePreferencesToDB($this->id);
			break;
		case 'user_extra_smtp':
			$account = ListQuery::quick_fetch('EmailPOP3', $this->id);
			if ($account) {
				$upd = RowUpdate::for_result($account);
				$upd->set('smtp_o365_auth_token', $json);
				$upd->save();
			}
			break;
		case 'system_smtp':
			AppConfig::update_local(['email.smtp_o365_token' => $json], true);
			AppConfig::save_local();
			break;
		case 'imap_account':
			$account = ListQuery::quick_fetch('EmailPOP3', $this->id);
			if ($account) {
				$upd = RowUpdate::for_result($account);
				$upd->set('o365_auth_token', $json);
				$upd->save();
			}
			break;
		}
	}
}

