<?php

namespace OneCRM\OAuth;

use PHPMailer\PHPMailer\OAuth;
use PHPMailer\PHPMailer\Exception as PHPMailerException;
use League\OAuth2\Client\Token\AccessToken;

require_once 'include/OAuth/OAuthTokenOwner.php';
require_once 'include/utils/oauth_utils.php';

class SMTPOAuth extends OAuth {

	private $oauthService;
	private $tokenOwner;
	private $smtpUser;
	private $token;

	public function __construct($smtp_user, $service, $tokenOwner) {
		$this->smtpUser = $smtp_user;
		$this->tokenOwner = $tokenOwner;
		$this->oauthService = $service;
		$owner = new OAuthTokenOwner($tokenOwner[0], $tokenOwner[1] ?? "");
		$this->token = $owner->load();
	}

	private function refreshToken() {
		try {
			$t = refershOauth2Token($this->oauthService, $this->tokenOwner, $this->token['refresh_token']);
			if ($t) {
				$this->token = $t;
			}
		} catch (\Exception $e) {
			throw new PHPMailerException($e->getMessage(), $e->getCode(), $e);
		}
	}

    public function getOauth64()
    {
		$token =  new AccessToken($this->token);
        if ($token->hasExpired()) {
            $this->refreshToken();;
        }
        return base64_encode(
            'user=' .
            $this->smtpUser .
            "\001auth=Bearer " .
            $this->token['access_token'] .
            "\001\001"
        );
    }
}

