<?php

require_once('OnlineMeetingException.php');

class OnlineMeetingsFactory {

	/**
	 * @param string|null $type - app type
	 * @param array|null $token - API access token
	 *
	 * @return GoToMeeting|MsTeams
	 * @throws OnlineMeetingException
	 */
	public static function getInstance($type = null, $token = null) {
		if (! $type)
			$type = AppConfig::setting('company.online_meeting_app');

		switch ($type) {
			case "gotomeeting":
				require_once('include/OnlineMeetings/GoToMeeting/GoToMeeting.php');
				$result = new GoToMeeting($token);
				break;
			case "ms_teams":
				require_once('include/OnlineMeetings/MicrosoftGraph/MsTeams.php');
				$result = new MsTeams($token);
				break;
			case "zoom":
				require_once('include/OnlineMeetings/Zoom/Zoom.php');
				$result = new Zoom($token);
				break;
			default:
				throw new OnlineMeetingException('Wrong Online Meeting App type');
				break;
		}

		return $result;
	}
}