<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


class AuditPopupManager extends PopupManager {

	var $title;
    var $audited_module;
    var $audited_record;
    var $audited_model;
    var $field_formatter;
	var $show_additional_details = false;
	var $unformat_values = array();

	function __construct($context='popup') {
        $this->audit = true;
		parent::__construct($context);
	}
	
	function loadRequest() {
		global $pageInstance;
		parent::loadRequest();
		$this->audited_module = $pageInstance->module;
		$model = AppConfig::module_primary_bean($this->audited_module);
		if($model && AppConfig::setting("model.detail.{$model}.audit_enabled")) {
			$this->audited_model = new ModelDef($model);
			$this->audited_record = array_get_default($_REQUEST, 'record', '');
		}
	}
	
	function init($module, $layout=null) {
		$this->module = $this->layout_module = 'Audit';
		$this->model_name = 'audit';
		$ret = $this->loadLayoutInfo($layout);
		if($ret) {
			$rec = ListQuery::quick_fetch($this->audited_model, $this->audited_record, array('_display'), array('acl_filters' => 'view'));
			if($rec) {
				$this->title = $rec->getField('_display');
			} else
				$ret = false;
		}
		return $ret;
	}
	
	function getButtons() {
		$buttons = array();
		return $buttons;
	}
		
	function getTitle($html = true) {
        global $app_list_strings;

		$module_name = $app_list_strings['moduleList'][$this->audited_module];

        $title = get_image($this->audited_module, '') . '&nbsp;' . $module_name .": ". to_html($this->title);

        return $title;
	}

    function renderTitle() {
        $audited_fields = AppConfig::setting("model.audited_fields.{$this->audited_model->name}", array());
        sort($audited_fields);
        $fields = '';

        if(sizeof($audited_fields) > 0) {
            for ($i = 0; $i < sizeof($audited_fields); $i++) {
                $def = $this->audited_model->getFieldDefinition($audited_fields[$i]);
                $fields .= str_replace(':', '', translate($def['vname'], $this->audited_model->module_dir));

                if($i < (sizeof($audited_fields) - 1))
                    $fields .= ", ";
            }

            $fields = '<div class="form-label">'. translate('LBL_AUDITED_FIELDS', 'Audit') ."</div>". $fields;
        } else {
            $fields = '<span class="text-help">'. translate('LBL_NO_AUDITED_FIELDS_TEXT', 'Audit') ."</span>";
        }

		$title = '<div class="panel-outer panel-default panel-border panel-round" style="margin: 0.8em 0.5em; flex: none">';
        $title .= '<div class="panel-body panel-pad-body">' .$fields. '</div></div>';

        echo $title;
    }

    function getHiddenFields() {
        $hidden = parent::getHiddenFields();
        $hidden['popup_type'] = 'audit';
        $hidden['record'] = $this->audited_record;
        return $hidden;
    }    

	function &getFormatter() {
		$fmt =& parent::getFormatter();
        $this->formatter->list_query->base_model->table_name = $this->audited_model->audit_table_name;
        $fmt->can_shrink = false;
        $fmt->show_navigation = false;
        $fmt->show_filter = false;
        $fmt->show_checks = false;
        $fmt->show_title = true;
        $fmt->show_mass_update = false;

        $clauses = array(
            /*"created_by" => array(
                "value" => AppConfig::current_user_id(),
                "field" => "created_by"
            ),*/
            "parent_id" => array(
                "value" => $this->audited_record,
                "field" => "parent_id",
                "operator" => "eq"
            )
        );

        $fmt->getQuery()->addFilterClauses($clauses);

		return $fmt;
	}

	function formatAuditedFields($listResult) {
        global $app_list_strings;
        if($listResult->failed || ! isset($listResult->rows))
            return;

		$rows =& $this->list_result->rows;
        $module_dir = $this->audited_model->module_dir;
		foreach($rows as $rid => &$row) {
			if ($row['field_name'] == '!teams!') {
				$this->unformat_values[$rid]['before_value_string'] = $this->formatAuditedTeams($row['before_value_text']);
				$this->unformat_values[$rid]['after_value_string'] = $this->formatAuditedTeams($row['after_value_text']);
				$this->list_result->rows[$rid]['field_name'] = translate('LBL_AUDIT_TEAMS_FIELD_LABEL');
				continue;
			}

			$def = $this->audited_model->getFieldDefinition($row['field_name']);

            $before_value = array_get_default($row, 'before_value_string', '');
            $after_value = array_get_default($row, 'after_value_string', '');

            if (strlen($before_value))
                $before_value = $this->getAssociatedFieldName($row['id'], 'before_value_string',  $row['field_name'], $before_value);
            if (strlen($after_value)) 
                $after_value = $this->getAssociatedFieldName($row['id'], 'after_value_string', $row['field_name'], $after_value);

            if ($def['type'] == 'enum' || $def['type'] == 'status') {
                $opts = array_get_default($app_list_strings, $def['options'], array());
                
                $before_value = array_get_default($opts, $before_value, $before_value);
                $after_value = array_get_default($opts, $after_value, $after_value);
            }

            if ($def['type'] == 'multienum') {
                $opts = array_get_default($app_list_strings, $def['options'], array());
                
                $before_value = $this->formatMultiEnum($opts, $before_value);
                $after_value = $this->formatMultiEnum($opts, $after_value);
            }

            $blank = translate('LBL_MISSING_VALUE', 'app');
            if(! strlen($before_value)) $before_value = $blank;
            if(! strlen($after_value)) $after_value = $blank;


            $this->list_result->rows[$rid]['before_value_string'] = $before_value;
            $this->list_result->rows[$rid]['after_value_string'] = $after_value;
            $this->list_result->rows[$rid]['field_name'] = $this->translateFieldName($def, $module_dir);
        }
    }

	function translateFieldName($def, $module) {
		$vname = array_get_default($def, 'vname');
		$vname_module = array_get_default($def, 'vname_module', $module);
		if ($def['type'] == 'id' && ($for_ref = array_get_default($def, 'for_ref'))) {
			$ref_def = $this->audited_model->getFieldDefinition($for_ref);
			$vname = array_get_default($ref_def, 'vname', $vname);
			$vname_module = array_get_default($ref_def, 'vname_module', $vname_module);
		}
		return translate($vname, $vname_module);
	}

	function formatMultiEnum($opts, $value) {
		$value = array_filter(explode('^,^', $value), 'strlen');
		return join(', ', array_map(function($v) use ($opts) {
			return array_get_default($opts, $v, $v);
		}, $value));
	}

    function getAssociatedFieldName($id, $audit_field, $fieldName, $fieldValue){        
        $field = $this->audited_model->getFieldDefinition($fieldName);
        if($field && isset($field['type'])) {
        	if(($field['type'] == 'id' || $field['type'] == 'ref_id') && isset($field['for_ref']))
				$ref = $this->audited_model->getFieldDefinition($field['for_ref']);
			else if($field['type'] == 'ref')
				$ref = $field;
        }
        
		if(! empty($ref) && ! empty($ref['bean_name'])) {
			$result = ListQuery::quick_fetch($ref['bean_name'], $fieldValue, array('id', '_display'), array('acl_filters' => 'view'));
			if($result && ! $result->isFieldEmpty('_display')) {
				$fmt = $this->getFieldFormatter();
				$result->fields['_display']['detail_link'] = true;
				$result->fields['_display']['type'] = 'ref';
				$result->fields['_display']['id_name'] = 'id';
				$result->fields['_display']['detail_icon'] = true;
				$result->fields['_display']['bean_name'] = $ref['bean_name'];
				$row = $fmt->formatRow($result->fields, $result->row);
				$value = array_get_default($row, '_display', $fieldValue);
				$this->unformat_values[$id][$audit_field] = $value;
				return $value;
			}
		}
		
        return $fieldValue;
	}

	function formatAuditedTeams($ids) {
		$ids = explode(',', $ids);
		$lq = new ListQuery('SecurityGroup', ['_display']);
		$lq->addSimpleFilter('id', $ids);
		$result = $lq->fetchAll();
		if ($result) {
			$fmt = $this->getFieldFormatter();
			$fmt->formatListResult($result);
			$values = array_column($result->formatted_rows, '_display');
			return join(', ', $values);
		}
	}
    
    function getFieldFormatter() {
    	if(empty($this->field_formatter)) {
    		$this->field_formatter = new FieldFormatter('html', 'popup');
    		$this->field_formatter->add_detail_links = true;
    	}
    	return $this->field_formatter;
    }
	
	function renderResult(ListFormatter &$fmt, ListResult &$result) {
		$rows =& $result->formatted_rows;
		foreach ($this->unformat_values as $id => $values) {
			if (isset($rows[$id])) {
				foreach ($values as $f => $v) {
					$rows[$id][$f] = $v;
				}
			}
		}
		return parent::renderResult($fmt, $result);
	}

}
?>
