<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

require_once('include/Popups/QuickCreatePopup.php');

class CallLoggingPopup extends QuickCreatePopup {

    const LAYOUT_NAME = 'Logging';

    /**
     * @var string
     */
    var $phone_number;

	function __construct() {
        parent::__construct();
	}
	
	function loadRequest() {
        parent::loadRequest();
        $this->phone_number = array_get_default($_REQUEST, 'phone_number', '');
        $this->target_id = array_get_default($_REQUEST, 'parent_id', '');
        $this->target_name = array_get_default($_REQUEST, 'parent_type', '');
        $this->view_after = false;
	}

    function setLayoutName() {
        $this->layout_name = self::LAYOUT_NAME;
    }

    function getFormTitle($html=true) {
   		$title = translate('LBL_CALL_LOGGING', $this->module);
   		return $html ? to_html($title) : $title;
   	}

    function getHiddenFields() {
        $hidden = parent::getHiddenFields();
        $hidden['popup_type'] = 'call_logging';
        $hidden['direction'] = 'Outbound';
        return $hidden;
    }

    function getStandardButtons() {
        $buttons = StandardDetailManager::getStandardButtons();

        unset($buttons['edit_custom']);
        unset($buttons['edit_layout_edit']);

        $params = array('target_name' => $this->target_name, 'target_id' => $this->target_id);
        $json = getJSONobj();
        $params = $json->encode($params);

        $buttons['save']['onclick'] = 'return SUGAR.ui.saveQuickCreate({FORM}, 0, '.$params.');';
        
        $buttons['save_view'] = array(
            'type' => 'button',
            'vname' => 'LBL_SAVE_VIEW_BUTTON_LABEL',
            'order' => 11,
            'icon' => 'icon-accept',
            'perform' => 'return SUGAR.ui.saveQuickCreate({FORM}, 1, '.$params.');',
        );

        $buttons['full_form'] = array(
            'type' => 'button',
            'vname' => 'LBL_FULL_FORM_BUTTON_LABEL',
            'order' => 80,
            'icon' => 'icon-popup',
            'perform' => 'return SUGAR.ui.loadFullForm({FORM}, '.$params.');',
            'align' => 'right',
            'group' => 'tools',
        );

        $integration_type = AppConfig::setting('telephony.integration');
		if (!empty($_SESSION['mobile_detected'])) {
			$pref = AppConfig::setting("userpref.current_user.global.mobile_calls");
			if ($pref == 'tel') $integration_type = 'tel';
		}

        //if ($integration_type != '') {

            $buttons['call'] = array(
                'type' => 'button',
                'vname' => self::getCallButtonLabel($integration_type),
                'order' => 80,
                'icon' => 'theme-icon bean-Call',
                'onclick' => 'return SUGAR.ui.makeCall({FORM}, "'.$integration_type.'", this, true);',
                'align' => 'right'
            );

        //}

        return $buttons;
    }

    static function getCallButtonLabel($type) {
        global $app_strings, $app_list_strings;
        $label = 'LBL_START_CALL_BUTTON_LABEL';

        if ($type != '' && isset($app_list_strings['telephony_integration_dom'][$type])) {
            $label = $app_strings['LBL_CALL_OUT_BUTTON_LABEL'] .' '. $app_list_strings['telephony_integration_dom'][$type];
        }

        return $label;
    }
}
