<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/ListView/ListViewManager.php');
require_once('include/Popups/IPopup.php');

class PopupManager extends ListViewManager implements IPopup {
	var $show_mass_update = false;
	var $show_clear_button = false;
	var $show_create_button = false;
	var $multi_select = false;
	var $request_data;
	var $display_name_field;
	var $display_currency_id;
	var $custom_buttons;
	var $custom_title_html;

	function __construct($context='popup') {
		parent::__construct($context);
	}
	
	function loadRequest() {
		parent::loadRequest();
		$multi = array_get_default($_REQUEST, 'multi_select');
		if($multi)
			$this->multi_select = ($multi && $multi != 'false');
		else if(strtolower(array_get_default($_REQUEST, 'mode', '')) == 'multiselect')
			// support old interface
			$this->multi_select = true;
		$rdata = array_get_default($_REQUEST, 'request_data', '');
		if($rdata) {
			$json = getJSONobj();
			$this->request_data = $json->decode($rdata);
		} else
			$this->request_data = array();
		$cid = array_get_default($_REQUEST, 'display_currency_id');
		if($cid)
			$this->display_currency_id = $cid;
	}
	
	function getRequestData($str=false) {
		$rdata =& $this->request_data;
		if($rdata) {
			if($str && is_array($rdata)) {
				$json = getJSONobj();
				return $json->encode($rdata);
			} else
				return $rdata;
		} else if($str)
			return "";
		else
			return array();
	}
	
	function getButtons() {
		if(isset($this->custom_buttons))
			return $this->custom_buttons;
		$buttons = parent::getButtons();
		if($this->multi_select) {
			$buttons['accept'] = array(
				'label' => translate('LBL_ADD_SELECTED_BUTTON_LABEL', 'app'),
				'perform' => "sListView.popupReturnSelected('$this->list_id', '$this->module');",
				'icon' => 'uii-accept',
				'params' => array(
					'toplevel' => 1,
				),
				//send_back_selected('$currentModule',document.PopupMassUpdate,'mass[]','" .$app_strings['ERR_NOTHING_SELECTED']."', fetch_request_data().field_to_name_array);
			);
		}
		if($this->show_clear_button) {
			$buttons['clear'] = array(
				'label' => translate('LBL_CLEAR_BUTTON_LABEL', 'app'),
				'perform' => "popup_return_empty();",
				'icon' => 'icon-cancel',
				'type' => 'button',
				'params' => array(
					'toplevel' => 1,
				),
			);			
		}
		return $buttons;
	}
		
	function getTitle($html=true) {
		if($this->custom_title_html) return $this->custom_title_html;
		$title = translate('LBL_MODULE_TITLE', $this->module);
		$title = get_image($GLOBALS['image_path'] . $this->module, '') . '&nbsp;' . to_html($title);
		return $title;
	}
	
	function renderTitle() {
	}

	function getHiddenFields() {
		$hidden = parent::getHiddenFields();
		$hidden['request_data'] = $this->getRequestData(true);
		$hidden['multi_select'] = $this->multi_select ? 'true' : 'false';
		$hidden['display_currency_id'] = $this->display_currency_id;
		return $hidden;
	}
	
	function &getFormatter() {
		$fmt =& parent::getFormatter();
		$fmt->context = 'popup';
		$fmt->show_checks &= $this->multi_select;
		$fmt->detail_link_action = "return sListView.popupReturn('{$this->list_id}', '{MODULE}', '{ID}');";
		if(isset($this->add_detail_links))
			$fmt->add_detail_links = $this->add_detail_links;
		if(isset($this->display_currency_id))
			$fmt->display_currency_id = $this->display_currency_id;
		return $fmt;
	}

    function init($module, $layout=null) {
        if(! $this->initModuleView($module, $layout))
        	ACLController::displayNoAccess();
    }
    
    function initFormatter(ListFormatter &$fmt) {
    	$ret = parent::initFormatter($fmt);
		$this->display_name_field = $fmt->getQuery()->addDisplayName();
		return $ret;
    }

	function render() {
		if(! defined('ASYNC_ENTRY')) {
			insert_popup_header($GLOBALS['theme']);
		}
		parent::render();
		if(! $this->async_list) {
			echo get_form_footer();
			insert_popup_footer();
		}
		
		$row_data = array();
		if($this->list_result && ! $this->list_result->failed) {
			$rows =& $this->list_result->rows;
			$namef = $this->display_name_field;
			foreach($rows as $rid => &$row) {
				$row_data[$rid] = array_get_default($row, $namef, '');
			}
		}
		
        $fmt =& $this->getFormatter();
        if(! $fmt) return false;
		
		global $pageInstance;
		$params = array(
			'title' => $this->getTitle(),
			'first_load' => ! $this->async_list,
			'request_data' => $this->getRequestData(),
			'row_data' => $row_data,
		);
		$json = getJSONobj();
		$j_params = $json->encode($params);
		$pageInstance->add_js_literal("sListView.initPopup('{$this->list_id}', $j_params);", null, LOAD_PRIORITY_FOOT);
		$pageInstance->title = null;
		$pageInstance->set_partial_page();
		$fmt->exportIncludes();
	}	
}

?>
