<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/DetailView/StandardDetailManager.php');


class QuickCreatePopup extends StandardDetailManager {

    const LAYOUT_NAME = 'QuickCreate';

    /**
     * @var string
     */
    var $target_id;

    /**
     * Quick Create popup caller name - subpanel, dashlet, etc
     *
     * @var string
     */
    var $target_name;

    /**
     * Subpanel, dashlet, ... layout name
     *
     * @var string
     */
    var $target_layout;

    /**
     * Link primary key name
     *
     * @var string
     */
    var $primary_key_name;

    /**
     * Link primary key value
     *
     * @var string
     */
    var $primary_key_value;

    /**
     * Got DetailView after save
     *
     * @var bool
     */
    var $view_after;
    
    /**
     * Returning result to a ref input
     * @var bool
     */
    var $for_ref;
    
    var $default_buttons_pos = 'top';
    
    var $qc_count;
    
    var $no_expand_view = false;

	function __construct() {
        parent::__construct();
	}
	
	function loadRequest() {
        parent::loadRequest();
        $this->target_id = array_get_default($_REQUEST, 'target_id', '');
        $this->target_name = array_get_default($_REQUEST, 'target_name', '');
        $this->target_layout = array_get_default($_REQUEST, 'target_layout', '');
        $this->primary_key_name = array_get_default($_REQUEST, 'primary_name', '');
        $this->primary_key_value = $this->primary_key_name ? array_get_default($_REQUEST, $this->primary_key_name, '') : '';
        $this->view_after = array_get_default($_REQUEST, 'go_to_view', false);
        $this->for_ref = array_get_default($_REQUEST, 'for_ref', false);
        if(isset($_REQUEST['qc_count']))
        	$this->qc_count = (int)$_REQUEST['qc_count'];
	}

    function standardInit($id=null) {
        $this->in_popup = true;
        $this->setLayoutName();
        $this->form_name = $this->layout_name . 'Form';
        if(isset($this->qc_count))
        	$this->form_name .= '_' . $this->qc_count;
        $this->editLayout = false;
        $model = $this->model->getModelName();
        if(AppConfig::setting("display.list.$model.view_in_popup") && $this->layout_name == 'QuickCreate')
        	$this->no_expand_view = true;
        return parent::standardInit($id);
    }

    function setLayoutName() {
		if(! $this->layout_name)
			$this->layout_name = self::LAYOUT_NAME;
    }

    function getHiddenFields() {
        $hidden = parent::getHiddenFields();
        $hidden['mgr_type'] = 'quick_create';

        return $hidden;
    }

    function getStandardButtons() {
        $buttons = parent::getStandardButtons();

        unset($buttons['edit_custom']);
        unset($buttons['edit_layout_edit']);

        $params = array('target_name' => $this->target_name, 'target_id' => $this->target_id,
            'target_layout' => $this->target_layout, 'primary_name' => $this->primary_key_name,
            'primary_value' => $this->primary_key_value);
        $json = getJSONobj();
        $params = $json->encode($params);

        $buttons['save']['perform'] = 'return SUGAR.ui.saveQuickCreate({FORM}, 0, '.$params.');';

		if(! $this->for_ref && ! $this->no_expand_view) {
			$buttons['save_view'] = array(
				'type' => 'button',
				'vname' => 'LBL_SAVE_VIEW_BUTTON_LABEL',
				'order' => 11,
				'icon' => 'icon-accept',
				'onclick' => 'return SUGAR.ui.saveQuickCreate({FORM}, 1, '.$params.');',
			);

			$buttons['full_form'] = array(
				'type' => 'button',
				'vname' => 'LBL_FULL_FORM_BUTTON_LABEL',
				'order' => 80,
				'icon' => 'icon-popup',
				'onclick' => 'return SUGAR.ui.loadFullForm({FORM}, '.$params.');',
				'align' => 'right',
			);
		}

        return $buttons;
    }
    
	function initLayout() {
		$ret = parent::initLayout();
		$this->form_gen->getFormObject()->max_ref_width = 24;
		$this->form_gen->getFormObject()->max_text_width = 30;
		$this->form_gen->getFormObject()->setAutoFocus();
		$this->form_gen->getFormObject()->setFormAction('async.php');
		$this->form_gen->getLayout()->setDefaultLabelPosition('top');
		return $ret;
	}
    
	function performUpdate() {
		global $pageInstance;
		$perf = $this->perform;
		$nextAction = parent::performUpdate();
		if($perf == 'save') {
			$result = ($nextAction && $this->record_id ? 1 : 0);
			$name = '';
			if($result) {
				$name_q = ListQuery::quick_fetch_row($this->model->getModelName(), $this->record_id, array('id', '_display'));
				if($name_q && isset($name_q['_display'])) $name = $name_q['_display'];
			}
			$sess = array(
				'quick_create_result' => $result,
				'quick_create_record' => $this->record_id,
				'quick_create_name' => $name,
			);
			$json = getJSONobj();
			$sess_js = $json->encode($sess);
			$pageInstance->add_js_literal("SUGAR.session.update($sess_js);",
				null, LOAD_PRIORITY_BODY);
			$pageInstance->set_partial_page(true);
			echo '<script nodisplay></script>';
		}
		if ($nextAction) {
			if($this->view_after && $this->record_id) {
				return array('redirect', array(
					'module' => $this->module,
					'action' => 'DetailView',
					'record' => $this->record_id));
			}
			return true;
		}
	}
    
}
