<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

class ObjectsReassign
{

	private static $meta = array(
		'Activities' => array(
			'Meetings' => array(
				'where' => array('field' => 'status', 'value' => 'Planned'),
				'parent_fields' => array('parent_id', 'account_id'),
			),
			'Calls' => array(
				'where' => array('field' => 'status', 'value' => 'Planned'),
				'parent_fields' => array('parent_id', 'account_id'),
			),
			'Tasks' => array(
				'where' => array('field' => 'status', 'value' => array('Completed', 'Deferred'), 'operator' => '!='),
				'parent_fields' => array('parent_id', 'account_id'),
			),
			'Emails' => array(
				'where' => array('field' => 'status', 'value' => array('sent', 'draft', 'send_error')),
				'parent_fields' => array('parent_id', 'account_id'),
			),
		),
		'History' => array(
			'Meetings' => array(
				'where' => array('field' => 'status', 'value' => 'Planned', 'operator' => '!='),
				'parent_fields' => array('parent_id', 'account_id'),
			),
			'Calls' => array(
				'where' => array('field' => 'status', 'value' => 'Planned', 'operator' => '!='),
				'parent_fields' => array('parent_id', 'account_id'),
			),
			'Tasks' => array(
				'where' => array('field' => 'status', 'value' => array('Completed', 'Deferred')),
				'parent_fields' => array('parent_id', 'account_id'),
			),
			'Emails' => array(
				'where' => array('field' => 'status', 'value' => array('sent', 'draft', 'send_error'), 'operator' => '!='),
				'parent_fields' => array('parent_id', 'account_id'),
			),
            'Notes' => array(
                'parent_fields' => array('parent_id'),
            ),
		),

		'Contacts' => array(
			'Contacts' => array(
				'relation' => 'contacts',
			),
		),

		'Opportunities' => array(
			'Opportunities' => array(
				'relation' => 'opportunities',
			),
		),

        'Quotes' => array(
            'Quotes' => array(
                'relation' => 'quotes',
            ),
        ),

		'Invoices' => array(
			'Invoice' => array(
				'parent_fields' => array('billing_account_id', 'shipping_account_id'),
			),
		),

		'Cases' => array(
			'Cases' => array(
				'parent_fields' => array('account_id'),
			),
		),
	);

	private static $children = array(
		'Account' => array('Contacts', 'Activities', 'History', 'Quotes', 'Invoices', 'Opportunities', 'Cases'),
		'Lead' => array('Activities', 'History',),
	);

	public static function reassign($parentModelName, $parentIds, $userId, $objects)
	{
		if (empty($userId) || empty($objects)) return;
		if (!isset(self::$children[$parentModelName])) return;
		require_once 'include/database/ListQuery.php';
		$lq = new ListQuery($parentModelName);
		if (! empty($parentIds)) {
			$lq->addFilterPrimaryKey($parentIds);
		}

		$result = $lq->runQuery(0, null, null, null, MASSUPDATE_PAGE_SIZE);
		
		while(! $result->failed) {
			foreach($result->getRowIndexes() as $idx) {
				$row = $result->getRowResult($idx);
				self::doReassign($parentModelName, $row->getField('id'), $userId, $objects);
			}
			if($result->page_finished)
				break;
			$lq->pageResult($result);
		}

	}
	
	public static function listupdate_hook($mu, $perform, RowUpdate $upd, $input) {
		if($perform == 'ObjectsReassign') {
			$objects = array_get_default($_REQUEST, 'reassign_objects');
			if($objects) {
				$objects = explode('^,^', $objects);
				self::reassignRecord($upd, $objects);
			}
		}
	}
	
	public static function reassignRecord(RowUpdate $upd, $objects=null)
	{
		$model = $upd->getModelName();
		if(! isset(self::$children[$model]))
			return false;
		if(! isset($objects))
			$objects = self::$children[$model];
		else
			$objects = array_intersect($objects, self::$children[$model]);
		$pid = $upd->getPrimaryKeyValue();
		$uid = $upd->getField('assigned_user_id');
		if($pid && $uid && $objects) {
			self::doReassign($model, $pid, $uid, $objects);
			return true;
		}
	}

	private static function doReassign($parentModelName, $parentId, $userId, $objects)
	{
		foreach ($objects as $module) {
			if (!in_array($module, self::$children[$parentModelName])) continue;
			$children = self::$meta[$module];
			foreach ($children as $childModule => $def) {
				$modelName = AppConfig::module_primary_bean($childModule);
				if (isset($def['relation'])) {
					$params = array('link_name' => $def['relation']);
					$lq = new ListQuery($parentModelName, null, $params);
					$lq->setParentKey($parentId);
				} else {
					$lq = new ListQuery($modelName);
				}
				if (isset($def['parent_fields'])) {
					if (is_countable($def['parent_fields']) && count($def['parent_fields']) > 1) {
						$clause = array('multiple' => array(), 'operator' => 'OR');
						foreach ($def['parent_fields'] as $f) {
							$clause['multiple'][] = array('field' => $f, 'value' => $parentId, );
						}
						$lq->addFilterClause($clause);
					} else {
						$lq->addFilterClause(array('field' => $def['parent_fields'][0], 'value' => $parentId));
					}
				}
				if (isset($def['where'])) {
					$lq->addFilterClause($def['where']);
				}
				$result = $lq->runQuery(0, null, null, null, MASSUPDATE_PAGE_SIZE);
				$ru = RowUpdate::for_model($modelName);
				while(! $result->failed) {
					foreach($result->getRowIndexes() as $idx) {
						$row = $result->getRowResult($idx);
						$ru->setOriginal($row);
						$ru->set(array('assigned_user_id' => $userId));
						$ru->save();
					}
					if($result->page_finished)
						break;
					$lq->pageResult($result);
				}
			}
		}
	}
}

