<?php

class RemoteFSCmdLine {

	private $stdin, $stdout, $stderr;
	private $proc;

	public function __construct($cmd, array $args = array()) {
		$descriptorspec = array(
			0 => array("pipe", "r"),
			1 => array("pipe", "w"),
			2 => array("pipe", "w")
		);
		$cmd = escapeshellcmd($cmd);
		foreach ($args as $arg) {
			$cmd .= ' ' . escapeshellarg($arg);
		}
		$this->proc = proc_open($cmd, $descriptorspec, $pipes);
		if (!is_resource($this->proc)) {
			return;
		}
		$this->stdin = $pipes[0];
		$this->stdout = $pipes[1];
		$this->stderr = $pipes[2];
	}

	public function writeToStdIn($data) {
		return fwrite($this->stdin, $data);
	}

	public function readFromStdOut($len) {
		return fread($this->stdout, $len);
	}

	public function readFromStdErr($len) {
		return fread($this->stderr, $len);
	}

	public function stdOutEof() {
		return feof($this->stdout);
	}

	public function stdErrEof() {
		return feof($this->stderr);
	}

	public function close() {
		if ($this->stdin)
			fclose($this->stdin);
		fclose($this->stdout);
		fclose($this->stderr);
		return proc_close($this->proc);
	}

	public function closeStdIn() {
		fclose($this->stdin);
		$this->stdin = null;
	}

	public function select() {
		$read = array($this->stdout, $this->stderr);
		$write = array($this->stdin);
		$except = null;
		$status = stream_select($read, $write, $except, 0);
		if ($status === false) {
			throw new RemoteFSException;
		}
		$ret = array();
		if (in_array($this->stdin, $write))
			$ret['stdin'] = true;
		if (in_array($this->stdout, $read))
			$ret['stdout'] = true;
		if (in_array($this->stderr, $read))
			$ret['stderr'] = true;
		return $ret;
	}
}

