<?php

require_once 'include/RemoteFS/RemoteFS.php';
require_once 'include/Dropbox/AuthProvider.php';
use Spatie\Dropbox\Client;


class RemoteFSDropbox implements RemoteFS {

	const CHUNK_SIZE =  8 * 1024 * 1024;

	private $options;
	private $client;

	public function __construct($options) {
		$this->options = $options;
	}

	public function connect() {
		$provider = new DropboxAuthProvider(true);
    	$this->client = new Client($provider);
		return true;
	}

	public function ls($dir) {
		try {
			$folderContents = $this->client->listFolder('/' . $dir);
			$files = [];
			do {
				$items = $folderContents['entries'];
				foreach ($items as $item) {
					$size = -1;
					$fname = $item['name'];
					if ($item['.tag'] === 'folder') {
						$fname .= '/';
					} else if ($item['.tag'] === 'file') {
						$size = $item['size'];
					}
					$files[] = array($fname, $size);
				}
				if (empty($folderContents['has_more']))
					break;
				$cursor = $folderContents['cursor'];
				$folderContents = $this->client->listFolderContinue($cursor);
			} while (true);
			return $files;
		} catch (Exception $e) {
			throw new RemoteFSException($e->getMessage());
		}
	}

	public function rm($path) {
		$path = '/' . $path;
		try {
			$this->client->delete($path);
		} catch (Exception $e) {
			throw new RemoteFSException($e->getMessage());
		}
	}

	public function rmdir($path) {
		throw new RemoteFSExceptionNotImp;
	}

	public function mkdir($dir, $create_parents = true) {
		throw new RemoteFSExceptionNotImp;
	}

	public function copyToRemote($src, $dst) {
		$f = @fopen($src, 'r');
		if (!$f) {
			throw new RemoteFSException("cannot open file");
		}
		try {
			$this->client->uploadChunked('/' . $dst, $f);
		} catch (Exception $e) {
			throw new RemoteFSException($e->getMessage());
		} finally {
			@fclose($f);
		}
	}

	public function copyFromRemote($src, $dst) {
		throw new RemoteFSExceptionNotImp;
	}

	public function readRemote($src) {
		throw new RemoteFSExceptionNotImp;
	}

	public function writeRemote($dst, $data) {
		throw new RemoteFSExceptionNotImp;
	}

	public function disconnect() {
	}

}

