<?php

require_once 'include/RemoteFS/RemoteFS.php';


class RemoteFSFtp implements RemoteFS {

	private $options;
	private $conn;

	public function __construct($options) {
		$this->options = $options;
	}

	public function connect() {
		if(!$this->options['ftp_encrypt'])
			$this->conn = ftp_ssl_connect($this->options['ftp_host']);
		else
			$this->conn = ftp_connect($this->options['ftp_host']);
		if (!$this->conn)
			throw new RemoteFSException('Unable to connect to FTP host');

		if (!ftp_login($this->conn, $this->options['ftp_username'], $this->options['ftp_password'])) {
			ftp_close($this->conn);
			throw new RemoteFSException('Login failed');
		}
		ftp_pasv($this->conn, array_get_default($this->options, 'ftp_pasv'));
		return true;
	}

	public function ls($dir) {
		$dir = $this->options['ftp_dir'] . '/' . $dir;
		$files = ftp_nlist($this->conn, $dir);
		if ($files === false)
			throw new RemoteFSException('Remote directory listing failed (' . $dir . ')');
		foreach ($files as &$f) {
			$f = array(basename($f), -1);
		}
		return $files;
	}

	public function rm($path) {
		$path = $this->options['ftp_dir'] . '/' . $path;
		$res = ftp_delete($this->conn, $path);
		if (!$res)
			throw new RemoteFSException('Failed to remove ' . $path);
	}

	public function rmdir($path) {
		throw new RemoteFSExceptionNotImp;
	}

	public function mkdir($dir, $create_parents = true) {
		throw new RemoteFSExceptionNotImp;
	}

	public function copyToRemote($src, $dst) {
		$f = fopen($src, 'rb');
		if (!is_resource($f))
			throw new RemoteFSException("Cannot open file for reading : $src");
		$dst = $this->options['ftp_dir'] . '/' . $dst;
		$result = ftp_fput($this->conn, $dst, $f, FTP_BINARY);
		fclose($f);
		if (!$result)
			throw new RemoteFSException(sprintf('Failed to copy %s to %s', $src, $dst));
	}

	public function copyFromRemote($src, $dst) {
		$src = $this->options['ftp_dir'] . '/' . $src;
		$f = fopen($dst, 'wb');
		if (!is_resource($f))
			throw new RemoteFSException("Cannot open file for writing : $dst");
		$result = ftp_fget($this->conn, $f, $src, FTP_BINARY);
		fclose($f);
		if (!$result)
			throw new RemoteFSException(sprintf('Failed to copy %s to %s', $src, $dst));
	}

	public function readRemote($src) {
		throw new RemoteFSExceptionNotImp;
	}

	public function writeRemote($dst, $data) {
		throw new RemoteFSExceptionNotImp;
	}
	
	public function disconnect() {
		ftp_close($this->conn);
	}

}

