<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

require_once 'include/SVGCharts/impl/SVGChartLayout.php';

abstract class SVGChart
{
	protected $layout;
	protected $doc;
	protected $plot;
	protected $data;
	protected $width;
	protected $height;

	protected function __construct($width, $height)
	{
		$this->width = $width;
		$this->height = $height;
		$this->createLayout();
		$this->createDoc();
	}

	public function getXMLDoc()
	{
		return $this->doc;
	}

	public function setTitle($title)
	{
		$this->layout->setTitle($title);
	}

	private function createLayout()
	{
		$this->layout = new SVGChartLayout($this->width, $this->height);
		$this->layout->setVGrid(true);
		$this->layout->setHGrid(true);
	}

	private function createDoc()
	{
		$impl = new DOMImplementation;
		$doctype = $impl->createDocumentType("svg", "-//W3C//DTD SVG 1.1//EN", "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd");
		$this->doc = $impl->createDocument(null, 'svg', $doctype);
		if($this->layout->getHideLabels())
			$this->doc->documentElement->setAttribute('class', 'hideLabels');
	}

	public function setLinkTemplate($tpl)
	{
		$this->plot->setLinkTemplate($tpl);
	}

	public function setStatus($default, $series = '', $point = '')
	{
		$this->layout->setStatus($default, $series, $point);
	}

	public function render($returnXMLObject = false)
	{
		$this->layout->render($this->doc, $this->data, $this->plot);
		return $this->doc->saveXML();
	}

	public function setData($data)
	{
		$this->data = $data;
	}

	public function getHeight()
	{
		return $this->layout->getTotalHeight();
	}
	
	public function removeLegend() {
		$this->layout->removeLegend();
	}
	
	public function setHasBorder($flag) {
		$this->layout->setHasBorder($flag);
	}

	public function setHasBackground($flag) {
		$this->layout->setHasBackground($flag);
	}
}

