<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

abstract class SVGChartLabelAxis
{
	const HORIZONTAL = 1;
	const VERTICAL = 2;

	protected $layout, $size, $data;
	protected $otherSize = 0;
	protected $padding = 8;

	abstract protected function addLabel($doc, $x, $label, $font, $fontSize, $offset, $index);
	abstract public function getOtherSize();
	abstract public function getMinHeight();

	public function __construct($size, $data, $layout)
	{
		$this->layout = $layout;
		$this->data   = $data;
		$this->size   = $size;
	}
	
	public function setSize($size)
	{
		$this->size = $size;
	}

	public function render($doc)
	{
		$font = $this->layout->getLabelAxisFont();
		$fontSize = $this->layout->getLabelAxisFontSize();
		$labels = $this->data->getLabels();
		$nLabels = is_countable($labels) ? count($labels) : 0;
		$x = $this->size / 2;
		$step = 0;
		if ($nLabels > 1) {
			$step = $this->size / $nLabels;
			$x = $step * 0.5;
		}
		$g = $doc->createElement('g');
		$offset = $this->getOtherSize();
		$i = 0;
		foreach ($labels as $col => $label) {
			$g->appendChild($this->addLabel($doc, $x, $label, $font, $fontSize, $offset, $i));
			$x += $step;
			$i++;
		}
		return $g;
	}


}

class SVGChartLabelAxisH extends SVGChartLabelAxis
{
	public function getOtherSize()
	{
		$font = $this->layout->getLabelAxisFont();
		$fontSize = $this->layout->getLabelAxisFontSize();
		$s = 0;
		foreach ($this->data->getLabels() as $col => $label) {
			$metrics = $font->measure($label, $fontSize);
			if ($metrics['h'] > $s) $s = $metrics['h'];
		}
		return $s * 3.25;
	}

	public function getMinHeight()
	{
		return $this->getOtherSize();
	}
	
	protected function addLabel($doc, $x, $label, $font, $fontSize, $offset, $index)
	{
		$metrics = $font->measure($label, $fontSize);
		if ($metrics['h'] > $this->otherSize) $this->otherSize = $metrics['h'];
		$text = $doc->createElement('text');

		$offsets = array(
			-$fontSize * 2.25,
			-$fontSize * 1.25,
			-$fontSize * 0.25,
		);

		$text->setAttribute('font-family', $font->getName());
		$text->setAttribute('font-size', $fontSize);
		$text->setAttribute('x', $x);
		$text->setAttribute('y', $offsets[$index % count($offsets)]);
		$text->setAttribute('text-anchor', 'middle');
		$text->setAttribute('pointer-events', 'none');
		$text->setAttribute('fill', $this->layout->getTextColor());
		$text->appendChild($doc->createTextNode($label));
		return $text;
	}
}	

class SVGChartLabelAxisV extends SVGChartLabelAxis
{
	public function getOtherSize()
	{
		$font = $this->layout->getLabelAxisFont();
		$fontSize = $this->layout->getLabelAxisFontSize();
		$s = 0;
		foreach ($this->data->getLabels() as $col => $label) {
			$metrics = $font->measure($label, $fontSize);
			if ($metrics['w'] > $s) $s = $metrics['w'];
		}
		return $s + $this->padding;
	}

	public function getMinHeight()
	{
		$font = $this->layout->getLabelAxisFont();
		$fontSize = $this->layout->getLabelAxisFontSize();
		$s = 200;
		foreach ($this->data->getLabels() as $col => $label) {
			$metrics = $font->measure($label, $fontSize);
			$s += $metrics['h'] * 1.2;
		}
		return $s + $this->padding;
	}

	protected function addLabel($doc, $x, $label, $font, $fontSize, $offset, $index)
	{
		$metrics = $font->measure($label, $fontSize);
		if ($metrics['w'] > $this->otherSize) $this->otherSize = $metrics['w'];
		$text = $doc->createElement('text');
		$text->setAttribute('font-family', $font->getName());
		$text->setAttribute('font-size', $fontSize);
		$text->setAttribute('x', $offset - $this->padding);
		$text->setAttribute('y', $x + $metrics['h']/2);
		$text->setAttribute('text-anchor', 'end');
		$text->setAttribute('pointer-events', 'none');
		$text->setAttribute('fill', $this->layout->getTextColor());
		$text->appendChild($doc->createTextNode($label));
		return $text;
	}
}

