<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

require_once 'include/SVGCharts/impl/SVGFont.php';
require_once 'include/SVGCharts/impl/SVGChartLabelAxis.php';
require_once 'include/SVGCharts/impl/SVGChartValueAxis.php';
require_once 'include/SVGCharts/impl/SVGChartPlot.php';
require_once 'include/SVGCharts/impl/SVGChartLegend.php';

class SVGChartLayout
{
	protected $defaultFont, $defaultFontSize = 12;
	protected $labelAxisFont, $labelAxisFontSize = 10;
	protected $valueAxisFont, $valueAxisFontSize;
	protected $titleFont, $titleFontSize = 16;
	protected $statusFont, $statusFontSize = 14;
	protected $legendFont, $legendFontSize;
	protected $opaqueLabels = true;
	protected $fonts, $width, $height;
	protected $labelAxis;
	protected $valueAxis;
	protected $title;
	protected $legendWidth = 0;
	protected $hasLegend = false;
	protected $reverseOrder = false;
	protected $hasVGrid;
	protected $hasHGrid;
	protected $padding = 15;
	protected $spacing = 10;
	protected $hasBorder = true;
	protected $hasBackground = true;
	protected $minContentWidth = 100;
	protected $seriesColors = null;
	protected $hideLabels = true;
	
	protected $defaultStatus = '';
	protected $seriesTemplate = '';
	protected $valueTemplate = '';

	protected $script = '';
	protected $style = '';

	protected $totalHeight = 0;

	var $strokeColor = "#222222";
	
	var $textColor = "#333333";

	public function __construct($width, $height)
	{
		$this->width = $width;
		$this->height = $height;
		$this->fonts = array(
			'Verdana' => new SVGFont(dirname(__FILE__) . '/fonts/Verdana.php', 'Verdana'),
		);
		$this->defaultFont =& $this->fonts['Verdana'];
	}
	
	public function getHideLabels() {
		return $this->hideLabels;
	}

	public function setStatus($default, $seriesTemplate='', $valueTemplate='')
	{
		$this->defaultStatus = addcslashes($default, "'");
		$this->seriesTemplate = addcslashes($seriesTemplate, "'");
		$this->valueTemplate = addcslashes($valueTemplate, "'");
	}

	public function getTotalHeight()
	{
		return $this->totalHeight;
	}

	public function getSeriesColor($index)
	{
		if(is_array($this->seriesColors)) {
			if(isset($this->seriesColors[$index])) {
				$index = $this->seriesColors[$index];
				if(is_string($index))
					return $index;
			}
		}
		global $themeInstance;
		return $themeInstance->generate_graph_color(null, $index, true);
	}
	
	public function getStrokeColor() {
		return $this->strokeColor;
	}

	public function getTextColor() {
		return $this->textColor;
	}
	
	public function getOpaqueLabels() {
		return $this->opaqueLabels;
	}
	
	public function getFont($name) {
		if(isset($this->fonts[$name]))
			return $this->fonts[$name];
		return $this->defaultFont;
	}
	
	public function getDefaultFont()
	{
		return $this->defaultFont;
	}
	
	public function getDefaultFontSize()
	{
		return $this->defaultFontSize;
	}

	public function getTitleFont()
	{
		if($this->titleFont)
			return $this->titleFont;
		return $this->defaultFont;
	}
	
	public function getTitleFontSize()
	{
		return $this->titleFontSize;
	}

	public function getLabelAxisFont()
	{
		if($this->labelAxisFont)
			return $this->labelAxisFont;
		return $this->defaultFont;
	}

	public function getStatusFont()
	{
		if($this->statusFont)
			return $this->statusFont;
		return $this->defaultFont;
	}

	public function getStatusFontSize()
	{
		if($this->statusFontSize)
			return $this->statusFontSize;
		return $this->defaultFontSize;
	}

	public function getValueAxisFont()
	{
		if($this->valueAxisFont)
			return $this->valueAxisFont;
		return $this->defaultFont;
	}

	public function getLabelAxisFontSize()
	{
		if($this->labelAxisFontSize)
			return $this->labelAxisFontSize;
		return $this->defaultFontSize;
	}
	
	public function getLegendFont()
	{
		if($this->legendFont)
			return $this->legendFont;
		return $this->defaultFont;
	}

	public function getLegendFontSize()
	{
		if($this->legendFontSize)
			return $this->legendFontSize;
		return $this->defaultFontSize;
	}

	public function getValueAxisFontSize()
	{
		if($this->valueAxisFontSize)
			return $this->valueAxisFontSize;
		return $this->defaultFontSize;
	}

	public function addLabelAxis($orientation)
	{
		$this->labelAxis = $orientation;
	}

	public function addValueAxis($orientation)
	{
		$this->valueAxis = $orientation;
	}

	public function addLegend($useSeries = false)
	{
		$this->hasLegend = $useSeries ? 1 : 2;
	}
	
	public function removeLegend() {
		$this->hasLegend = false;
	}

	public function setTitle($title)
	{
		$this->title = $title;
	}
	
	public function setPadding($value)
	{
		$this->padding = $value;
	}

	public function setSpacing($value)
	{
		$this->spacing = $value;
	}

	public function render($doc, $data, SVGChartPlot $plot)
	{
		$root = $doc->documentElement;
		$root->setAttribute('xmlns', 'http://www.w3.org/2000/svg');
		$root->setAttribute('xmlns:xlink', 'http://www.w3.org/1999/xlink');
		$root->setAttribute('version', '1.1');
		$root->appendChild($this->getStyles($doc));

		$dynHeight = true; // !$this->height;
		$minPlotHeight = 0;

		// some default values to not cause overflow/underflow - changed later
		$plotHeight = 800;
		$plotWidth = 800;

		$root->setAttribute('width', $this->width);

		$defs = $doc->createElement('defs');
		$root->appendChild($defs);

		$bg_colors = array('#F6F6F6', '#DDD', '#EEE');
		$this->seriesColors = $data->getSeriesColors();

		// background
		if($this->hasBackground) {
			$bg = $doc->createElement('linearGradient');
			$bg->setAttribute('id', 'bg');
			$bg->setAttribute('x1', '0%');
			$bg->setAttribute('x2', '0%');
			$bg->setAttribute('y1', '0%');
			$bg->setAttribute('y2', '100%');
			$stop = $doc->createElement('stop');
			$stop->setAttribute('offset', '0%');
			$stop->setAttribute('stop-color', $bg_colors[0]);
			$bg->appendChild($stop);
			$stop = $doc->createElement('stop');
			$stop->setAttribute('offset', '50%');
			$stop->setAttribute('stop-color', $bg_colors[1]);
			$bg->appendChild($stop);
			$stop = $doc->createElement('stop');
			$stop->setAttribute('offset', '100%');
			$stop->setAttribute('stop-color', $bg_colors[2]);
			$bg->appendChild($stop);
			$defs->appendChild($bg);
			
			$background = $doc->createElement('rect');
			$bgsize = array('x' => 0, 'y' => 0, 'width' => $this->width, 'height' => $this->height);
			foreach($bgsize as $k => $v)
				$background->setAttribute($k, $v);
			$background->setAttribute('fill', 'url(#bg)');
			$background->setAttribute('stroke-linejoin', 'miter');
			$root->appendChild($background);
		}
		
		// dropshadow filter
		$dsf = $doc->createElement('filter');
		$dsf->setAttribute('id', 'dropshadow');
		$blur = $doc->createElement('feGaussianBlur');
		$blur->setAttribute('in', 'SourceAlpha');
		$blur->setAttribute('stdDeviation', '1');
		$blur->setAttribute('result', 'blurred');
		$dsf->appendChild($blur);
		$offs = $doc->createElement('feOffset');
		$offs->setAttribute('in', 'blurred');
		$offs->setAttribute('dx', '0.5');
		$offs->setAttribute('dy', '0.5');
		$offs->setAttribute('result', 'offset');
		$dsf->appendChild($offs);
		$sat = $doc->createElement('feColorMatrix');
		$sat->setAttribute('type', 'matrix');
		$sat->setAttribute('in', 'offset');
		//$sat->setAttribute('values', '1 0 0 0 0  0 1 0 0 0  0 0 1 0 0  0 0 0 1 0');
		$sat->setAttribute('values', str_repeat('0.2 ', 20));
		$sat->setAttribute('result', 'final');
		$dsf->appendChild($sat);
		$merg = $doc->createElement('feMerge');
		$m1 = $doc->createElement('feMergeNode');
		$m1->setAttribute('in', 'final');
		$m2 = $doc->createElement('feMergeNode');
		$m2->setAttribute('in', 'SourceGraphic');
		$merg->appendChild($m1);
		$merg->appendChild($m2);
		$dsf->appendChild($merg);
		$defs->appendChild($dsf);

		$titleHeight = 0;
		$plotX = $this->padding;
		$plotY = $this->padding;

		if ($this->title !== null) {
			if (!is_array($this->title)) {
				$this->title = array($this->title);
			}
			$y = $plotY;
			foreach ($this->title as $t) {
				$title = $doc->createElement('text');
				$title->setAttribute('font-family', $this->getTitleFont()->getName());
				$title->setAttribute('font-size', $this->getTitleFontSize());
				$title->setAttribute('x', $this->width / 2);
				$title->setAttribute('text-anchor', 'middle');
				$title->setAttribute('fill', $this->getTextColor());
				//$title->setAttribute('filter', 'url(#dropshadow)');
				$title->appendChild($doc->createTextNode($t));
				$metrics = $this->getTitleFont()->measure($t, $this->getTitleFontSize());
				$title->setAttribute('y', $metrics['h'] + $y);
				$plotY += $metrics['h'] + $this->spacing;
				$y += $metrics['h'] + $this->spacing;
				$root->appendChild($title);
				$titleHeight += $metrics['h'] + $this->spacing;
			}
		}
		
		$legend = null;
		if ($this->hasLegend) {
			$legend = new SVGChartLegend($plotHeight, $data, $this, $this->hasLegend == 2, $this->reverseOrder);
			if ($dynHeight) {
				$h = $legend->getMinHeight();
				$legend = new SVGChartLegend($h, $data, $this, $this->hasLegend == 2, $this->reverseOrder);
				$minPlotHeight = max($minPlotHeight, $h);
			}
			$this->legendWidth = $legend->getWidth();
		}


		$horizAxisHeight = 0;
		$vertAxisWidth = 0;

		if ($this->labelAxis == SVGChartLabelAxis::HORIZONTAL) {
			$labelAxis = new SVGChartLabelAxisH($plotWidth, $data, $this);
			$horizAxisHeight = $labelAxis->getOtherSize();
		} elseif ($this->labelAxis == SVGChartLabelAxis::VERTICAL) {
			$labelAxis = new SVGChartLabelAxisV($plotHeight, $data, $this);
			if ($dynHeight) {
				$h = $labelAxis->getMinHeight();
				$labelAxis = new SVGChartLabelAxisV($h, $data, $this);
				$minPlotHeight = max($minPlotHeight, $h);
			}
			$vertAxisWidth = $labelAxis->getOtherSize();
		} else {
			$labelAxis = null;
		}

		if ($this->valueAxis == SVGChartValueAxis::HORIZONTAL) {
			$valueAxis = new SVGChartValueAxisH($plotWidth, $data, $this, $plot->additiveValues());
			$horizAxisHeight = $valueAxis->getOtherSize();
		} elseif ($this->valueAxis == SVGChartValueAxis::VERTICAL) {
			$valueAxis = new SVGChartValueAxisV($plotHeight, $data, $this, $plot->additiveValues());
			if ($dynHeight) {
				$h = $valueAxis->getMinHeight();
				$valueAxis = new SVGChartValueAxisV($h, $data, $this, $plot->additiveValues());
				$minPlotHeight = max($minPlotHeight, $h);
			}
			$vertAxisWidth = $valueAxis->getOtherSize();
		} else {
			$valueAxis = null;
		}


		$plot->setData($data);
		$plot->setLayout($this);
		$plot->setOrientation($this->valueAxis);
		$plotY += $plot->getOverflowTop();
		$overflowRt = $plot->getOverflowRight();
		$plotWidth = $this->width - $plotX - $this->padding;

		if ($dynHeight) {
			$plotHeight = $plot->getPreferredHeight($plotWidth, $minPlotHeight);
			$this->totalHeight = $plotHeight;
			$this->totalHeight += $this->padding * 2;
			if ($this->valueAxis == SVGChartValueAxis::HORIZONTAL) {
			}
			if ($this->labelAxis == SVGChartLabelAxis::HORIZONTAL) {
			}
			$this->totalHeight += $horizAxisHeight;
			$this->totalHeight += $titleHeight;
			if ($this->defaultStatus) {
				$this->totalHeight += $this->getStatusFontSize() + ($this->spacing * 2);
			}
			$this->totalHeight += $plot->getOverflowTop();
		} else {
			$this->totalHeight = $this->height;
			$plotHeight = $this->height - $plotY - $this->padding;
			if ($this->defaultStatus) {
				$plotHeight -= $this->getStatusFontSize() + ($this->spacing * 2);
			}
			$plotHeight -= $horizAxisHeight;
		}
		
		$plotWidth -= $vertAxisWidth;
		$plotX += $vertAxisWidth;
		if($this->hasLegend)
			$plotWidth -= $this->legendWidth + $this->spacing;
		if($plotWidth < $this->minContentWidth) {
			// enforce minimum size
			$this->width = ceil($this->width + ($this->minContentWidth - $plotWidth));
			$root->setAttribute('width', $this->width);
			if($this->hasBackground)
				$background->setAttribute('width', $this->width);
			$plotWidth = $this->minContentWidth;
		}
		$this->totalHeight = ceil($this->totalHeight);
		
		$root->setAttribute('height', $this->totalHeight);
		if($this->hasBackground)
			$background->setAttribute('height', $this->totalHeight);
		$viewb = '0 0 '.ceil($this->width).' '.ceil($this->totalHeight);
		$root->setAttribute('viewBox', $viewb);
		
		if ($this->labelAxis == SVGChartLabelAxis::HORIZONTAL) {
			$labelAxis->setSize($plotWidth - $overflowRt);
		} elseif ($this->labelAxis == SVGChartLabelAxis::VERTICAL) {
			$labelAxis->setSize($plotHeight);
		}

		if ($this->valueAxis == SVGChartValueAxis::HORIZONTAL) {
			$valueAxis->setSize($plotWidth - $overflowRt);
		} elseif ($this->valueAxis == SVGChartValueAxis::VERTICAL) {
			$valueAxis->setSize($plotHeight);
		}
		

		// plot background
		$rect = $doc->createElement('rect');
		$rect->setAttribute('x', $plotX);
		$rect->setAttribute('y', $plotY);
		$rect->setAttribute('width', $plotWidth);
		$rect->setAttribute('height', $plotHeight);
		//$rect->setAttribute('fill', 'url(#plot-bg)');
		$rect->setAttribute('fill', 'white');
		$rect->setAttribute('fill-opacity', '0.6');
		//$rect->setAttribute('stroke', '#666');
		//$rect->setAttribute('stroke-linejoin', 'miter');
		//$rect->setAttribute('stroke-width', '0.5');
		$root->appendChild($rect);
		
		
		if ($labelAxis) {
			$labelAxisEL = $labelAxis->render($doc);
			if ($this->labelAxis == SVGChartLabelAxis::HORIZONTAL) {
				$this->moveElement($labelAxisEL, $plotX, $plotY + $plotHeight + $horizAxisHeight);
			} else {
				$this->moveElement($labelAxisEL, $this->padding, $plotY);
			}
			$root->appendChild($labelAxisEL);
		}

		if ($valueAxis) {
			$grid = $this->valueAxis == SVGChartValueAxis::HORIZONTAL ? $this->hasHGrid : $this->hasVGrid;
			if ($grid) {
				$grid = $this->valueAxis == SVGChartValueAxis::HORIZONTAL ? $plotHeight : $plotWidth;
			}
			$valueAxisEL = $valueAxis->render($doc, $grid);
			if ($this->valueAxis == SVGChartValueAxis::HORIZONTAL) {
				$this->moveElement($valueAxisEL, $plotX, $plotY + $plotHeight + $horizAxisHeight);
			} else {
				$this->moveElement($valueAxisEL, $this->padding, $plotY);
			}
			$root->appendChild($valueAxisEL);
		}

		if ($legend) {
			$legendEL = $legend->render($doc);
			$this->moveElement($legendEL, round($this->width - $this->legendWidth - $this->padding), $plotY);
			$root->appendChild($legendEL);
			$this->script .= $legend->getScript();
		}

		if ($this->defaultStatus) {
			$status = $doc->createElement('text');
			$status->setAttribute('font-family', $this->getStatusFont()->getName());
			$status->setAttribute('font-size', $this->getStatusFontSize());
			$status->setAttribute('id', 'statusLine');
			$status->setAttribute('x', $this->width / 2);
			$status->setAttribute('text-anchor', 'middle');
			$status->setAttribute('fill', $this->getTextColor());
			$status->appendChild($doc->createTextNode($this->defaultStatus));
			$status->setAttribute('y', $this->totalHeight - ($this->padding * 2));
			$root->appendChild($status);
		}
		
		// plot background
		if($this->hasBackground) {
			$plot_bg = $doc->createElement('linearGradient');
			$plot_bg->setAttribute('id', 'plot-bg');
			$plot_bg->setAttribute('x1', '0%');
			$plot_bg->setAttribute('x2', '0%');
			$plot_bg->setAttribute('y1', '0%');
			$plot_bg->setAttribute('y2', '100%');
			$stop = $doc->createElement('stop');
			$stop->setAttribute('offset', '0%');
			$stop->setAttribute('stop-color', '#DDD');
			$plot_bg->appendChild($stop);
			$stop = $doc->createElement('stop');
			$stop->setAttribute('offset', '50%');
			$stop->setAttribute('stop-color', '#FFF');
			$plot_bg->appendChild($stop);
			$stop = $doc->createElement('stop');
			$stop->setAttribute('offset', '100%');
			$stop->setAttribute('stop-color', '#DDD');
			$plot_bg->appendChild($stop);
			$defs->appendChild($plot_bg);
		}
		
		$plot->setSize($plotWidth - $overflowRt, $plotHeight);
		if ($valueAxis) {
			$plot->setValuesRange($valueAxis->getMin(), $valueAxis->getScale());
		}
		$plotEL = $plot->render($doc);
		$plot->renderDefs($defs);
		$this->moveElement($plotEL, $plotX, $plotY);
		$root->appendChild($plotEL);
		$this->script .= $plot->getScript();



		if($this->hasBorder) {
			$rect = $doc->createElement('rect');
			$bgsize = array('x' => 1, 'y' => 1, 'width' => $this->width-2, 'height' => $this->totalHeight-2, 'stroke-width' => 2, 'stroke' => '#666');
			foreach($bgsize as $k => $v)
				$rect->setAttribute($k, $v);
			$rect->setAttribute('fill', 'none');
			$rect->setAttribute('stroke-linejoin', 'miter');
			$root->appendChild($rect);
		}

		$root->appendChild($this->getScripts($doc));

		return $root;
	}

	protected function moveElement($el, $x, $y)
	{
		$el->setAttribute('transform', sprintf('translate(%f,%f)', $x, $y));
	}

	public function setHGrid($enable)
	{
		$this->hasHGrid = $enable;
	}

	public function setVGrid($enable)
	{
		$this->hasVGrid = $enable;
	}
	
	public function setHasBorder($enable)
	{
		$this->hasBorder = $enable;
	}

	public function setHasBackground($enable)
	{
		$this->hasBackground = $enable;
	}
	
	public function setReverseOrder($flag=true)
	{
		$this->reverseOrder = $flag;
	}

	private function getScripts($doc)
	{
		$scriptEL = $doc->createElement('script');
		$script = file_get_contents(dirname(__FILE__). '/svgcharts.js');
		$script .= $this->script;
		$script .= $this->getStatusScript();
		if(AppConfig::setting('layout.jsmin_enabled')) {
			require_once('include/javascript/jsminplus.php');
			$script = JSMinPlus::minify($script);
		}
		$scriptEL->appendChild($doc->createCDATASection($script));
		return $scriptEL;
	}
	
	private function getStyles($doc)
	{
		$styleEL = $doc->createElement('style');
		$style = file_get_contents(dirname(__FILE__). '/svgcharts.css');
		$style .= $this->style;
		if(AppConfig::setting('layout.jsmin_enabled') && ! AppConfig::setting('layout.jsmin_css_disabled')) {
			require_once('include/javascript/cssmin.php');
			$style = cssmin::minify($style);
		}
		$styleEL->appendChild($doc->createCDATASection($style));
		return $styleEL;
	}

	private function getStatusScript()
	{
		return <<<SCRIPT
SVGCHARTDATA.default_status = '$this->defaultStatus';
SVGCHARTDATA.series_template = '$this->seriesTemplate';
SVGCHARTDATA.value_template = '$this->valueTemplate';
SVGCHARTDATA.width = '$this->width';
SVGCHARTDATA.subscribe('statusLine', SVGCHARTDATA.statusGroup, 'statusGroup', null, null, null, null);
SVGCHARTDATA.subscribe('statusLine', SVGCHARTDATA.statusDefault, 'statusDefault');
SVGCHARTDATA.subscribe('statusLine', SVGCHARTDATA.statusValue, 'statusValue', null, null, null, null, null);
SCRIPT;
	}

}

