<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

require_once 'include/SVGCharts/impl/SVGChartLayout.php';

class SVGChartLegend
{
	private $width, $height, $data, $useSeries, $reverseOrder;
	private $script = '';
	private $padding = 5;
	private $spacing = 5;
	private $layout;

	public function __construct($height, $data, $layout, $useSeries, $reverseOrder=false)
	{
		$this->height = $height;
		$this->data = $data;
		$this->layout = $layout;
		$this->useSeries = $useSeries;
		$this->reverseOrder = $reverseOrder;
	}
	
	protected function getLabels()
	{
		if ($this->useSeries) {
			$labels = $this->data->getSeriesLabels();
		} else {
			$labels = $this->data->getLabels();
		}
		return $labels;
	}

	public function getWidth()
	{
		$w = 0;
		$font = $this->layout->getLegendFont();
		$fontSize = $this->layout->getLegendFontSize();
		foreach ($this->getLabels() as $col => $label) {
			$metrics = $font->measure($label, $fontSize);
			$nw = $metrics['h'] + $this->spacing + $metrics['w'] * 1.2;
			if ($nw > $w) $w = $nw;
		}
		return $w + $this->padding * 2;
	}

	public function getMinHeight()
	{
		$labelsCount = is_countable($this->getLabels()) ? count($this->getLabels()) : 0;
		return $labelsCount * ($this->layout->getLegendFontSize() + $this->padding * 2);
	}

	public function render(DOMDocument $doc)
	{
		$g = $doc->createElement('g');
		$g->setAttribute('class', 'legend');
		$font = $this->layout->getLegendFont();
		$fontSize = $this->layout->getLegendFontSize();

		$i = 0;
		$y = $this->padding;
		
		$max_w = $this->getWidth();
		$labels = $this->getLabels();
		$nLabels = count($labels);
		
		foreach ($labels as $col => $label) {
			$row_g = $doc->createElement('g');
			$id = uniqid('P');
			$cval = addcslashes($col, "'");
			$row_g->setAttribute('id', $id);
			$row_g->setAttribute('class', 'legend-row');
			$row_g->setAttribute('onmouseover', "SVGCHARTDATA.emit('highlight', '$cval'); SVGCHARTDATA.emit('statusLabel', '$cval'); ");
			$row_g->setAttribute('onmouseout', "SVGCHARTDATA.emit('unhighlight', '$cval'); SVGCHARTDATA.emit('statusDefault'); ");
			$row_g->setAttribute('stroke-width', '0.5');
			$row_g->setAttribute('shape-rendering', 'geometricPrecision');
			$this->script .= "SVGCHARTDATA.subscribe('$id', SVGCHARTDATA.highlight, 'highlight', '$cval');";
			$this->script .= "SVGCHARTDATA.subscribe('$id', SVGCHARTDATA.unhighlight, 'unhighlight', '$cval');";
			$this->script .= "SVGCHARTDATA.subscribe('$id', SVGCHARTDATA.highlight, 'highlight', '{$cval}-lbl');";
			$this->script .= "SVGCHARTDATA.subscribe('$id', SVGCHARTDATA.unhighlight, 'unhighlight', '{$cval}-lbl');";

			// to catch hover events
			$box = $doc->createElement('rect');
			$box->setAttribute('x', 0);
			$box->setAttribute('y', $y - $this->padding);
			$box->setAttribute('width', $max_w);
			$box->setAttribute('height', $fontSize + $this->padding * 2);
			$box->setAttribute('fill', 'white');
			$box->setAttribute('fill-opacity', '0.6');
			$row_g->appendChild($box);
			
			$rect = $doc->createElement('rect');
			$rect->setAttribute('class', 'segment');
			$rect->setAttribute('x', $this->padding);
			$rect->setAttribute('y', $y);
			$rect->setAttribute('width', $fontSize);
			$rect->setAttribute('height', $fontSize);
			$rect->setAttribute('fill', $this->layout->getSeriesColor($this->reverseOrder ? $nLabels - $i - 1 : $i));
			$rect->setAttribute('stroke', 'black');
			$row_g->appendChild($rect);
			
			$text = $doc->createElement('text');
			$text->setAttribute('class', 'legend-label hl-text');
			$text->setAttribute('font-family', $font->getName());
			$text->setAttribute('font-size', $fontSize);
			$text->setAttribute('x', $this->padding + $fontSize + $this->spacing);
			$text->setAttribute('y', $y + $fontSize);
			$text->setAttribute('fill', $this->layout->getTextColor());
			$text->appendChild($doc->createTextNode($label));
			
			$text->setAttribute('pointer-events', 'none');
			$row_g->appendChild($text);
			
			$g->appendChild($row_g);

			$y += $fontSize + $this->padding * 2;

			$i++;
		}
		return $g;
	}

	public function getScript()
	{
		return $this->script;
	}

}
