<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

abstract class SVGChartPlot
{
	protected $script = '';
	protected $threeD = false;
	protected $linkTemplate = null;
	protected $data;
	protected $layout;
	protected $width;
	protected $height;
	protected $min;
	protected $scale;

	abstract public function render(DOMDocument $doc);
	
	public function additiveValues() {
		return false;
	}

	public function getLayout()
	{
		return $this->layout;
	}

	public function setLayout($layout)
	{
		$this->layout = $layout;
	}
	
	public function setSize($width, $height) {
		$this->width = $width;
		$this->height = $height;
	}
	
	public function setValuesRange($min, $scale)
	{
		$this->min = $min;
		$this->scale = $scale;
	}

	public function setOrientation($orientation) {
	}
	
	public function setData($data)
	{
		$this->data = $data;
	}
	
	public function renderDefs(DOMElement $defs) {}

	public function getOverflowTop()
	{
		return 0;
	}
	public function getOverflowRight()
	{
		return 0;
	}
	
	public function getPreferredHeight($width, $height) {
		return $height;
	}

	public function getScript()
	{
		return $this->script;
	}

	public function set3D($value)
	{
		$this->threeD = $value;
	}

	public function setLinkTemplate($tpl)
	{
		$this->linkTemplate = $tpl;
	}
	
	public function replaceSeriesVar($series, $value, $name)
	{
		$vars = $this->data->getSeriesVars();
		if (isset($vars[$series][$value][$name])) return $vars[$series][$value][$name];
		if (isset($vars[$series][$name])) return $vars[$series][$name];
		return '';
	}

	public function createLink($doc, $el, $series, $value = null)
	{
		if (empty($this->linkTemplate)) return $el;
		$url = str_replace('$series$', urlencode($series ?? ''), $this->linkTemplate);
		$url = str_replace('$value$', urlencode($value ?? ''), $url);

		$url = preg_replace_callback(
			'~\$([a-z_0-9]+)\$~i',
			function($matches) use ($series, $value) {
				return urlencode($this->replaceSeriesVar($series, $value, $matches[1]));
			}, $url);

		$ret = $doc->createElement('a');
		$ret->setAttribute('target', '_top');
		$ret->setAttribute('xlink:href', $url);
		$ret->appendChild($el);
		return $ret;
	}

	protected function highlight($el, $values, $labels=null, $count = '', $rel_values = null, $hl_all = false)
	{
		$id = uniqid('P');
		$el->setAttribute('id', $id);
		$group1 = addcslashes($values[1], "'");
		$value = addcslashes($values[0], "'");
		if(! isset($labels)) $labels = $values;
		$lbl_group1 = addcslashes($labels[1], "'");
		$lbl_value = addcslashes($labels[0], "'");
		if($rel_values) {
			$js = getJSONobj();
			$rel = $js->encode($rel_values);
		} else
			$rel = 'null';
		$count = addcslashes($count, "'");
		if($hl_all) {
			$onmouseover = "SVGCHARTDATA.emit('highlight', '$id'); SVGCHARTDATA.emit('highlight', '$group1');";
			$onmouseout = "SVGCHARTDATA.emit('unhighlight', '$id'); SVGCHARTDATA.emit('unhighlight', '$group1');";
		} else {
			$onmouseover = "SVGCHARTDATA.emit('highlight', '$id'); SVGCHARTDATA.emit('highlight', '{$group1}-lbl');";
			$onmouseout = "SVGCHARTDATA.emit('unhighlight', '$id'); SVGCHARTDATA.emit('unhighlight', '{$group1}-lbl');";
		}
		if (is_countable($values) && count($values) < 3) {
			$onmouseover .= "SVGCHARTDATA.emit('statusGroup', '$lbl_group1', '$lbl_value', '$count', $rel);";
		} else {
			$group2 = addcslashes($values[2], "'");
			$lbl_group2 = addcslashes($labels[2], "'");
			$onmouseover .= "SVGCHARTDATA.emit('statusValue', '$lbl_group1', '$lbl_group2', '$lbl_value', '$count', $rel);";
		}
		$el->setAttribute('onmouseover', $onmouseover);
		$el->setAttribute('onmouseout', "$onmouseout SVGCHARTDATA.emit('statusDefault');");
		if(! $hl_all) {
			$this->script .= "SVGCHARTDATA.subscribe('$id', SVGCHARTDATA.highlight, 'highlight', '$id');";
			$this->script .= "SVGCHARTDATA.subscribe('$id', SVGCHARTDATA.unhighlight, 'unhighlight', '$id');";
		}
		$this->script .= "SVGCHARTDATA.subscribe('$id', SVGCHARTDATA.highlight, 'highlight', '$group1');";
		$this->script .= "SVGCHARTDATA.subscribe('$id', SVGCHARTDATA.unhighlight, 'unhighlight', '$group1');";
		
		return $id;
	}
	
	protected function extendHighlight($id, $value) {
		if(is_string($value) && $value)
			$value = [$value];
		if(! is_array($value)) return;
		foreach($value as $v) {
			$v = addcslashes($v, "'");
			$this->script .= "SVGCHARTDATA.subscribe('$id', SVGCHARTDATA.highlight, 'highlight', '$v');";
			$this->script .= "SVGCHARTDATA.subscribe('$id', SVGCHARTDATA.unhighlight, 'unhighlight', '$v');";
		}
	}

	protected function createElement($root, $el, $attribs=null, $text=null) {
		$doc = ($root instanceof DOMDocument) ? $root : $root->ownerDocument;
		$elt = $doc->createElement($el, $text ?? '');
		return $this->setAttributes($elt, $attribs);
	}
	
	protected function addElement($root, $el, $attribs=null, $text=null) {
		if($el instanceof DOMNode)
			$elt = $el;
		else
			$elt = $this->createElement($root, $el, $attribs, $text);
		if($elt)
			$root->appendChild($elt);
		return $elt;
	}
	
	protected function addTextNode($root, $text) {
		$doc = ($root instanceof DOMDocument) ? $root : $root->ownerDocument;
		$node = $doc->createTextNode($text);
		$root->appendChild($node);
		return $node;
	}
	
	protected function createLabelBackground($root, $x, $y, $width, $height, $padding=null) {
		if(! isset($padding)) $padding = 3;
		$hpad = $padding + 1;
		$vpad = $padding;
		$x -= $hpad;
		$y -= $vpad;
		$width += $hpad * 2;
		$height += $vpad * 2;
		$attrs = compact('x', 'y', 'width', 'height');
		$attrs['class'] = 'bg';
		$attrs['rx'] = '4';
		$attrs['ry'] = '4';
		$rect = $this->addElement($root, 'rect', $attrs);
		return $rect;
	}
	
	protected function createSegmentLabel($root, $value, $x, $y, $attribs=null, $subscribe=null, $indicator=null) {
		if(! $attribs) $attribs = [];

		$id = array_get_default($attribs, 'id', '');
		unset($attribs['id']);
		if(! $id) $id = uniqid('P');
		$cls = array_get_default($attribs, 'class', '');
		$attribs['class'] = 'text';
		$opaque = array_get_default($attribs, 'opaque', $this->layout->getOpaqueLabels());
		unset($attribs['opaque']);
		$attribs['x'] = $x;
		$attribs['y'] = $y;
		$attribs['pointer-events'] = 'none';
		
		if(empty($attribs['font-family']))
			$attribs['font-family'] = $this->layout->getDefaultFont()->getName();
		if(empty($attribs['font-size']))
			$attribs['font-size'] = $this->layout->getDefaultFontSize();
		if(empty($attribs['fill']))
			$attribs['fill'] = $this->layout->getTextColor();
		if(empty($attribs['text-anchor']))
			$attribs['text-anchor'] = 'start';

		$font = $this->layout->getFont($attribs['font-family']);
		$size = $font->measure($value, $attribs['font-size']);
		$w = round($size['w'], 2);
		$h = round($size['h'], 2);
		$attribs['textLength'] = $w;

		$group_attrs = ['class' => 'segment-label '.$cls, 'id' => $id];
		$outer = $this->addElement($root, 'g', $group_attrs);

		if($opaque) {
			$anchor = $attribs['text-anchor'];
			if($anchor != 'start' && $anchor != 'end' && $anchor != 'middle')
				$anchor = 'start';
			if($anchor == 'middle')
				$bx = $x - $w / 2;
			else if($anchor == 'end')
				$bx = $x - $w;
			else
				$bx = $x;
			$this->createLabelBackground($outer, $bx, $y-$h, $w, $h * 1.25);
		}
		
		if($indicator)
			$outer->appendChild($indicator);
		$text = $this->addElement($outer, 'text', $attribs);
		$this->addTextNode($text, $value);
		if($subscribe)
			$this->extendHighlight($id, $subscribe);
		
		return $outer;
	}
	
	protected function setAttributes($elt, $attribs) {
		if($elt && $attribs) {
			foreach($attribs as $k => $v) {
				if($v !== null)
					$elt->setAttribute($k, $v);
			}
		}
		return $elt;
	}
}

