<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

require_once 'include/SVGCharts/impl/SVGChartPlot.php';

class SVGChartPlotBar extends SVGChartPlot
{
	private $orientation = 2;
	private $additive = true;
	private $additiveStatus = false;

	public function additiveValues()
	{
		return $this->additive;
	}

	public function setAdditive($value)
	{
		$this->additive = $value;
	}

	public function setAdditiveStatus($v)
	{
		$this->additiveStatus = $v;
	}

	public function setOrientation($orientation)
	{
		$this->orientation = $orientation;
		/*if ($orientation == 2) {
			$this->width -= $this->layout->getLegendFontSize();
		}*/
	}

	public function getOverflowTop()
	{
		$pad = 15; //$this->threeD ? 15 : 0;
		if($this->orientation == 2 && $this->additive) {
			$fontSize = $this->layout->getLegendFontSize();
			$pad += $fontSize * 1.5;
		}
		return $pad;
	}
	public function getOverflowRight()
	{
		$pad = ($this->threeD && $this->orientation != 2) ? 10 : (($this->orientation == 2) ? 6 : 0);
		if($this->orientation == 1 && $this->additive) {
			$fontSize = $this->layout->getLegendFontSize();
			$pad += $fontSize * 5 + 5;
		}
		return $pad;
	}

	public function renderDefs(DOMElement $defs) {
		$grad = $this->addElement($defs, 'linearGradient',
			array('id' => "rt-depthGrad", 'x1' => "0%", 'y1' => "0%", 'x2' => "100%", 'y2' => "30%"));
		$this->addElement($grad, 'stop',
			array('offset' => "15%", 'stop-color' => "white", 'stop-opacity' => "0.1"));
		$this->addElement($grad, 'stop',
			array('offset' => "100%", 'stop-color' => "black", 'stop-opacity' => "0.2"));

		$grad = $this->addElement($defs, 'linearGradient',
			array('id' => "top-depthGrad", 'x1' => "0%", 'y1' => "100%", 'x2' => "100%", 'y2' => "0%"));
		$this->addElement($grad, 'stop',
			array('offset' => "20%", 'stop-color' => "white", 'stop-opacity' => "0.4"));
		$this->addElement($grad, 'stop',
			array('offset' => "100%", 'stop-color' => "white", 'stop-opacity' => "0.0"));

		$grad = $this->addElement($defs, 'linearGradient',
			array('id' => "flat-grad", 'x1' => "0%", 'y1' => "0%", 'x2' => "100%", 'y2' => "30%"));
		$this->addElement($grad, 'stop',
			array('offset' => "20%", 'stop-color' => "white", 'stop-opacity' => "0.3"));
		$this->addElement($grad, 'stop',
			array('offset' => "150%", 'stop-color' => "white", 'stop-opacity' => "0.0"));
	}

	public function render(DOMDocument $doc)
	{
		$g = $doc->createElement('g');

		$font = $this->layout->getLegendFont();
		$fontSize = $this->layout->getLegendFontSize();

		$labels = $this->data->getLabels();
		$series = $this->data->getSeries();
		$ser_labels = $this->data->getSeriesLabels();
		$nLabels = is_countable($labels) ? count($labels) : 0;
		$labelElts = [];
		$showTotals = $this->additive;
		$plotW = $this->width;
		$plotH = $this->height;

		// count number of bars needed
		if($this->additive)
			$barsPerLabel = 1;
		else
			$barsPerLabel = $this->countSeriesBars($series);

		if ($nLabels) {
			$fullX = ($this->orientation == 2 ? $plotW : $plotH);
			$estX = $fullX / $nLabels / $barsPerLabel;
			if ($this->threeD) {
				$extrude_depth = min($fontSize, $estX*0.3);
				if(! $this->additive)
					$extrude_depth /= 2;
			} else
				$extrude_depth = 0;

			$stepX = $fullX / $nLabels;
			$visX = ($fullX / max($nLabels, ($this->orientation == 2 ? 8 : 5))) - $extrude_depth;
			if($nLabels > 1) {
				$spacer = max(10, min(25, ($visX + $extrude_depth) / 4));
				$visX -= $spacer;
			}
			$barWidth = $visX / $barsPerLabel;
			$x = 0;

			$reverse_draw = ($this->additive || ($this->threeD && ! $this->additive && $this->orientation != 2));
			$rSeries = array_keys($series);
			if($reverse_draw)
				$rSeries = array_reverse($rSeries, true);

			// debug - show bounds of graph box
			//$r = $this->addElement($g, 'rect', array('x' => 0, 'y' => 0, 'width' => $plotW, 'height' => $plotH, 'opacity' => 0.5));

			foreach ($labels as $col => $label) {
				$sum = 0;
				$offset = ($stepX - $visX - $extrude_depth) / 2;
				if($this->orientation == 1)
					$offset += $extrude_depth;
				$y2 = $sum - $this->min * $this->scale;

				if ($this->orientation == 2) {
					$bX = $x + $offset;
					$bY = $plotH + $this->min * $this->scale;
					$bottomAttrs = ['x1' => $bX, 'x2' => $bX + $visX, 'y1' => $bY, 'y2' => $bY];
				} else {
					$bX = (- $this->min) * $this->scale;
					$bY = $x + $offset;
					$bottomAttrs = ['x1' => $bX, 'x2' => $bX, 'y1' => $bY, 'y2' => $bY + $visX];
				}
				$bottomAttrs['class'] = 'segment';
				$bottomAttrs['stroke'] = $this->layout->getStrokeColor();
				$bottom = $this->createElement($doc, 'line', $bottomAttrs);
				$g->appendChild($bottom);

				foreach ($rSeries as $i => $name) {
					if(! $this->additive) $sum = 0;
					$ser = $series[$name];
					if (isset($ser['points'][$col])) {
						$theBar = $this->createElement($doc, 'g', ['class' => 'shadow']);
						//$title = $this->addElement($theBar, 'title', null, $ser['points'][$col]['s']);

						$value = $ser['points'][$col]['n'];
						if($value >= 0) {
							$y1 = ($sum - $this->min) * $this->scale;
							$y2 = ($sum + $value - $this->min) * $this->scale;
						} else {
							$y2 = ($sum - $this->min) * $this->scale;
							$y1 = ($sum + $value - $this->min) * $this->scale;
						}

						if ($this->orientation == 2) {
							$w = $barWidth;
							$h = $y2 - $y1;
						} else {
							$h = $barWidth;
							$w = $y2 - $y1;
						}

						if (!$this->threeD) {
							// shadow
							$bar = $this->createElement($doc, 'rect', ['class' => 'shadow']);
							if ($this->orientation == 2) {
								$bar->setAttribute('x', $x + $offset + 3);
								$bar->setAttribute('y', $plotH - $y2 - 3);
								$bar->setAttribute('width', $barWidth);
								$bar->setAttribute('height', max($y2 - $y1 + 1, 4));
							} else {
								$bar->setAttribute('y', $x + $offset + 3);
								$bar->setAttribute('x', $y1 + 3);
								$bar->setAttribute('height', $barWidth);
								$bar->setAttribute('width', $y2 - $y1);
							}
							$bar->setAttribute('fill', '#666');
							$bar->setAttribute('opacity', '0.5');
							$theBar->appendChild($bar);
						}

						if ($this->orientation == 2) {
							$bar_path = $bar_top_left = sprintf("M%0.3f %0.3f", $x + $offset, $plotH - $y2);
							$bar_path .= sprintf("h%0.3fv%0.3fh%0.3f", $barWidth, $y2 - $y1, -$barWidth);
						} else {
							$bar_path = $bar_top_left = sprintf("M%0.3f %0.3f", $y1, $x + $offset);
							$bar_path .= sprintf("v%0.3fh%0.3fv%0.3f", $barWidth, $y2 - $y1, -$barWidth);
						}
						$bar_path .= 'z';

						if ($this->threeD) {
							// bar top side
							$path = $bar_top_left;
							$path .= sprintf("l%0.3f %0.3f", $barWidth/0.7*0.3, -$extrude_depth);
							$path .= sprintf("h%0.3f", $w);
							$path .= sprintf("l%0.3f %0.3f", -$barWidth/0.7*0.3, $extrude_depth);
							$path .= 'z';

							$top_ext = $this->addElement($theBar, 'path',
								array('d' => $path, 'fill' => $this->layout->getSeriesColor($i), 'class' => 'segment'));
							$top_fill = $this->addElement($theBar, 'path',
								array('d' => $path, 'stroke' => $this->layout->getStrokeColor(), 'fill' => 'url(#top-depthGrad)', 'class' => 'segment'));

							// bar right side
							$path = $bar_top_left;
							$path .= sprintf("m%0.3f %0.3f", $barWidth/0.7*0.3 + $w, -$extrude_depth);
							$path .= sprintf("v%0.3f", $h);
							$path .= sprintf("l%0.3f %0.3f", -$barWidth/0.7*0.3, $extrude_depth);
							$path .= sprintf("v%0.3f", -$h);
							$path .= 'z';

							$rt_ext = $this->addElement($theBar, 'path',
								array('d' => $path, 'fill' => $this->layout->getSeriesColor($i), 'class' => 'segment'));
							$rt_fill = $this->addElement($theBar, 'path',
								array('d' => $path, 'stroke' => $this->layout->getStrokeColor(), 'fill' => 'url(#rt-depthGrad)', 'class' => 'segment'));
						}

						$bar_back = $this->addElement($theBar, 'path',
							array('d' => $bar_path, 'fill' => $this->layout->getSeriesColor($i)));
						$bar = $this->addElement($theBar, 'path',
							array('d' => $bar_path, 'stroke' => $this->layout->getStrokeColor(), 'fill' => 'url(#flat-grad)', 'class' => 'segment'));

						$sum += $value;

						$count = $ser['points'][$col]['c'];
						if ($this->additive && $this->additiveStatus) {
							$nodeValue = $value;
						} else {
							$nodeValue = $ser['points'][$col]['s'];
						}
						$relValues = array_get_default($ser['points'][$col], 'l');
						$hlId = $this->highlight($theBar, array($nodeValue, $name, $col), array($nodeValue, $ser_labels[$name], $label), $count, $relValues);

						$link = $this->createLink($doc, $theBar, $col, $name);
						$g->appendChild($link);

						// add segment label
						if ($this->orientation == 2) {
							$labelX = $x + $offset + ($barWidth + $extrude_depth) / 2;
							$labelY = $plotH - (max(0, $sum) - $this->min) * $this->scale - 12 - $extrude_depth;
						} else {
							$labelX = $y2 + $extrude_depth + 10;
							$labelY = $x + $offset + ($barWidth - $extrude_depth) / 2 + $fontSize / 2 - 1;
						}
						$labelAttrs = ['font-family' => $font->getName(), 'font-size' => $fontSize];
						if($this->additive) {
							$labelAttrs['class'] = 'hidden';
						}
						if($this->orientation == 2) {
							$labelAttrs['text-anchor'] = 'middle';
						}
						if ($this->additive && $this->additiveStatus) {
							$labelText = format_number($value, 2, -1);
						} else {
							$labelText = $ser['points'][$col]['s'];
						}
						$labelElt = $this->createSegmentLabel($doc, $labelText, $labelX, $labelY, $labelAttrs, [$name, $hlId]);
						$labelElts[] = $labelElt;

                        /*if (!$this->additive) {
							if($reverse_draw)
								$offset -= $barWidth;
							else
								$offset += $barWidth;
						}*/
					}
					if (!$this->additive) {
						if($reverse_draw)
							$offset -= $barWidth;
						else
							$offset += $barWidth;
					}
				}
				$displayTotals = $this->data->getTotalsDisplay();
				if (isset($displayTotals[$col])) $sumDisplay = $displayTotals[$col];
				else $sumDisplay = $sum ? format_number($sum, 2, -1) : '';
				if ($showTotals && strlen($sumDisplay)) {
					if ($this->orientation == 2) {
						$labelX = $x + ($stepX - $visX + $barWidth) / 2;
						$labelY = $plotH - ($sum - $this->min) * $this->scale - 12 - $extrude_depth;
					} else {
						$labelX = $y2 + $extrude_depth + 10;
						$labelY = $x + ($stepX - $visX + $barWidth) / 2 + $fontSize / 2 - 1;
					}
					$labelAttrs = ['font-family' => $font->getName(), 'font-size' => $fontSize];
					if($this->orientation == 2) {
						$labelAttrs['text-anchor'] = 'middle';
					}
					$labelElt = $this->createSegmentLabel($doc, $sumDisplay, $labelX, $labelY, $labelAttrs);
					$labelElts[] = $labelElt;
				}

				$x += $stepX;
			}
		}

		foreach($labelElts as $lbl)
			$g->appendChild($lbl);
		return $g;
	}

	public function countSeriesBars(&$series) {
		if(! is_array($series)) return 0;

		// override - should match the number of series?
		return count($series);

		$scount = array();
		foreach($series as $g1 => $ser) {
			if(isset($ser['points'])) {
				foreach($ser['points'] as $g2 => $pt) {
					$scount[$g2] = @$scount[$g2] + 1;
				}
			}
		}
		return max(1, max($scount));
	}

}

