<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

require_once 'include/SVGCharts/impl/SVGChartPlot.php';

class SVGChartPlotFunnel extends SVGChartPlot
{
	protected $offset = 0;
	protected $tip_size = 0.15;
	protected $tip_width = 0.15;
	protected $hl_text = true;
	protected $threeD = true;
	protected $use_spacer = true;
	protected $spacer_size = 3;

	public function setOffset($offset)
	{
		$this->offset = $offset;
	}
	
	public function renderDefs(DOMElement $defs) {
		$grad = $this->addElement($defs, 'linearGradient',
			array('id' => "flat-grad", 'x1' => "0%", 'y1' => "0%", 'x2' => "100%", 'y2' => "60%"));
		$this->addElement($grad, 'stop',
			array('offset' => "20%", 'stop-color' => "white", 'stop-opacity' => "0.3"));
		$this->addElement($grad, 'stop',
			array('offset' => "50%", 'stop-color' => "white", 'stop-opacity' => "0.0"));
		$this->addElement($grad, 'stop',
			array('offset' => "130%", 'stop-color' => "black", 'stop-opacity' => "0.1"));

		$grad = $this->addElement($defs, 'linearGradient',
			array('id' => "alt-grad", 'x1' => "0%", 'y1' => "0%", 'x2' => "100%", 'y2' => "30%"));
		$this->addElement($grad, 'stop',
			array('offset' => "20%", 'stop-color' => "black", 'stop-opacity' => "0.1"));
		$this->addElement($grad, 'stop',
			array('offset' => "100%", 'stop-color' => "white", 'stop-opacity' => "0.2"));

		$grad = $this->addElement($defs, 'linearGradient',
			array('id' => "dark-grad", 'x1' => "0%", 'y1' => "0%", 'x2' => "100%", 'y2' => "30%"));
		$this->addElement($grad, 'stop',
			array('offset' => "30%", 'stop-color' => "black", 'stop-opacity' => "0.2"));
		$this->addElement($grad, 'stop',
			array('offset' => "80%", 'stop-color' => "black", 'stop-opacity' => "0.5"));
		$this->addElement($grad, 'stop',
			array('offset' => "100%", 'stop-color' => "black", 'stop-opacity' => "0.4"));
	}

	public function render(DOMDocument $doc)
	{
		$g = $doc->createElement('g');

		$font = $this->layout->getLegendFont();
		$fontSize = $this->layout->getLegendFontSize();

		$labels = $this->data->getLabels();
		$nLabels = is_countable($labels) ? count($labels) : 0;

		$series = $this->data->getSeries();

		if (!$nLabels || !is_countable($series) || !count($series)) {
			return $g;
		}

		$ser = array();
		foreach ($series as $k => $v) {
			$ser = $v;
			break;
			foreach ($v['points'] as $kk => $vv) {
				if (!isset($ser['points'][$kk])) $ser['points'][$kk]['n'] = $vv['n'];
				else $ser['points'][$kk]['n'] += $vv['n'];
			}
		}
		$series = $ser;


		$sum = 0;
		foreach ($series['points'] as $point) {
			$sum += $point['n'];
		}

		$slices = $shadows = array();
		$totals = $this->data->getTotalsDisplay();
		
		$runSum = 0;
		$plotH = $this->height * 0.8;
		$plotW = min($plotH * 2.0, $this->width * 0.8) / 2;
		$tip = $sum * (1-$this->tip_size);
		$tipW = $this->tip_width * $plotW;
		$tipH = $sum ? ($plotH * $tip / $sum) : 0;
		$i = 0;
		$yoffs = ($this->height - $plotH) / 2;
		$first = true;
		$alt = false;
		$lblCount = 0;
		foreach($labels as $name => $label) {
			if (! isset($series['points'][$name]) || ! $series['points'][$name]['n']) continue;
			$lblCount ++;
		}
		$spacer = $this->use_spacer ? $this->spacer_size : 0;
		$totalSpace = max(0, $lblCount-1) * $spacer;

		foreach ($labels as $name => $label) {
			if (! isset($series['points'][$name])) continue;

			$value = $series['points'][$name]['n'];
			$valueCount = $series['points'][$name]['c'];
			if (!$value) {
				$i++;
				continue;
			}
			if (isset($totals[$name])) $strvalue = $totals[$name];
			else $strvalue = $value;
			
			$sectH = ($plotH - $totalSpace) * ($value / $sum);
			$startAfter = ($yoffs > $tipH);
			$endAfter = ($yoffs + $sectH > $tipH);
			$spanTip = $endAfter && ! $startAfter;
			
			if($startAfter)
				$startW = $tipW;
			else
				$startW = ($plotW - $tipW) * (1 - $yoffs / $tipH) + $tipW;
			
			if($endAfter)
				$endW = $tipW;
			else
				$endW = ($plotW - $tipW) * (1 - ($yoffs + $sectH) / $tipH) + $tipW;
			
			$topArcSize = $startW / 20;
			$bottomArcSize = $endW / 20;
			
			$path = sprintf("M%f %f", $this->width/2 - $startW, $yoffs);
			if($this->threeD)
				$path .= sprintf("A%f,%f 0 0,0 %f,%f", $startW / 2, $topArcSize, $this->width/2 + $startW, $yoffs);
			else
				$path .= sprintf("L%f %f", $this->width/2 + $startW, $yoffs);
			if($spanTip) {
				$beforeH = $tipH - $yoffs;
				$path .= sprintf("L%f %f", $this->width/2 + $endW, $yoffs + $beforeH);
			}
			$path .= sprintf("L%f %f", $this->width/2 + $endW, $yoffs + $sectH);
			if($this->threeD)
				$path .= sprintf("A%f,%f 0 0,1 %f,%f", $endW / 2, $bottomArcSize, $this->width/2 - $endW, $yoffs + $sectH);
			else
				$path .= sprintf("L%f %f", $this->width/2 - $endW, $yoffs + $sectH);
			if($spanTip) {
				$path .= sprintf("L%f %f", $this->width/2 - $endW, $yoffs + $beforeH);
			}
			$path .= "z";

			$sect = $doc->createElement('g');
			
			$pathEL = $this->createElement($doc, 'path', ['class' => 'segment']);
			$pathEL->setAttribute('d', $path);
			$pathEL->setAttribute('fill', $this->layout->getSeriesColor($nLabels - $i - 1));
			if(! $this->threeD)
				$pathEL->setAttribute('stroke', $this->layout->getStrokeColor());
			$sect->appendChild($pathEL);

			if($this->threeD && ($first || $spacer > 4)) {
				$topPath = sprintf("M%f %f", $this->width/2 - $startW, $yoffs);
				$topPath .= sprintf("A%f,%f 0 0,0 %f,%f", $startW / 2, $topArcSize, $this->width/2 + $startW, $yoffs);
				$topPath .= sprintf("A%f,%f 0 0,0 %f,%f", $startW / 2, $topArcSize, $this->width/2 - $startW, $yoffs);
				$topPath .= "z";
				$pathEL = $this->createElement($doc, 'path', ['class' => 'segment']);
				$pathEL->setAttribute('d', $topPath);
				$pathEL->setAttribute('fill', $this->layout->getSeriesColor($nLabels - $i - 1));
				$sect->appendChild($pathEL);

				$pathEL = $this->createElement($doc, 'path', ['class' => 'segment']);
				$pathEL->setAttribute('d', $topPath);
				$pathEL->setAttribute('fill', $first ? 'url(#alt-grad)' : 'url(#dark-grad)');
				$pathEL->setAttribute('stroke', $this->layout->getStrokeColor());
				$sect->appendChild($pathEL);
			}
			
			if($this->threeD) {
				$pathEL = $this->createElement($doc, 'path', ['class' => 'segment']);
				$pathEL->setAttribute('d', $path);
				$pathEL->setAttribute('fill', 'url(#flat-grad)');
				$pathEL->setAttribute('stroke', $this->layout->getStrokeColor());
				$sect->appendChild($pathEL);
			}
						
			if (isset($totals[$name])) $strvalue = $totals[$name];
			else $strvalue = $value;

			$numLabel = sprintf('%0.2f', $value);
			$metrics = $font->measure($strvalue, $fontSize);

			$tY = $yoffs + $sectH * 0.6;
			if($this->threeD) {
				/*if($first) $tY -= $topArcSize * 1.2;
				else*/ $tY += $topArcSize;
			}

			if($metrics['h']*2 < $sectH) {
				$tX = ($this->width - $metrics['w']) / 2;
				$alt = false;
				$indEL = null;
			} else {
				$sgn = $alt ? -1 : 1;
				if($tY < $tipH) {
				
					$startW = ($plotW - $tipW) * (1 - $yoffs / $tipH) + $tipW;
			
					//$endW = ($plotW - $tipW) * (1 - ($yoffs + $sectH) / $tipH) + $tipW;
					//$indX = ($this->width + $sgn*$startW + $sgn*$endW) / 2 + $sgn*2;
					$posW = ($plotW - $tipW) * (1 - $tY / $tipH) + $tipW;
					$indX = $this->width / 2 + $sgn*($posW + 2);
				} else {
					$indX = $this->width / 2 + $sgn*($tipW + 2);
				}
				$tX = $indX + $sgn*$metrics['h']*1.5;
				if($alt) $tX -= $metrics['w'];
				$indX2 = $indX + $sgn*$metrics['h'];

				// indicator line
				$path = sprintf("M%f %f", $indX, $tY);
				$path .= sprintf("L%f %f", $indX2, $tY);
				$path .= "z";
				$indEL = $this->createElement($doc, 'path', ['class' => 'indicator']);
				$indEL->setAttribute('d', $path);
				$indEL->setAttribute('stroke', $this->layout->getStrokeColor());
				$indEL->setAttribute('stroke-width', 0.25);
				$alt = ! $alt;
			}
			
			$hlId = $this->highlight($sect, array($strvalue, $name), array($strvalue, $label), $valueCount);

			$labelAttrs = ['font-family' => $font->getName(), 'font-size' => $fontSize];
			$labelX = $tX;
			$labelY = $tY + $metrics['h'] / 2;
			$labelElt = $this->createSegmentLabel($doc, $strvalue, $labelX, $labelY, $labelAttrs, [$hlId, $name], $indEL);
			$sect->appendChild($labelElt);

			$g->appendChild($sect);
			$slices[] = $this->createLink($doc, $sect, $name);
	
			$yoffs += $sectH + $spacer;
			$runSum += $value;

			$i++;
			$first = false;
		}

		foreach ($shadows as $shadow) {
			$g->appendChild($shadow);
		}
		foreach (array_reverse($slices) as $slice) {
			$g->appendChild($slice);
		}
		
		return $g;
	}

}

