<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

require_once 'include/SVGCharts/impl/SVGChartPlot.php';

class SVGChartPlotGauge extends SVGChartPlot
{
	private $offset = 0;
	private $percentFormat = true;
	private $needleKey;

	public function setOffset($offset)
	{
		$this->offset = $offset;
	}

	public function setPercentFormat($val)
	{
		$this->percentFormat = $val;
	}

	public function render(DOMDocument $doc)
	{
		$g = $doc->createElement('g');

		
		$font = $this->layout->getLegendFont();
		$fontSize = $this->layout->getLegendFontSize();

		$labels = $this->data->getLabels();
		$nLabels = is_countable($labels) ? count($labels) : 0;

		$series = $this->data->getSeries();

		if (!$nLabels || !is_countable($series) || !count($series)) {
			return $g;
		}

		$ser = array();
		foreach ($series as $k => $v) {
			$ser = $v;
			break;
			foreach ($v['points'] as $kk => $vv) {
				if (!isset($ser['points'][$kk])) $ser['points'][$kk]['n'] = $vv['n'];
				else $ser['points'][$kk]['n'] += $vv['n'];
			}
		}
		$series = $ser;


		$sum = 0;
		foreach ($series['points'] as $point) {
			$sum += $point['n'];
		}
		$scale = pi() / $sum;

		$r1 = min(($this->width - 30) / 2, $this->height - 25);
		//$r1 = $this->width;
		$yoffs = ($this->height + $r1) / 2;
		$a = pi();
		$i = 0;

		$r1 -= $fontSize * 1.5;
		$r1 -= $this->offset;

		if ($this->threeD) {		
			$r2 = $r1 * 0.8;
			$pieH = $r1 * 0.2;
		} else {
			$r2 = $r1;
			$pieH = 0;
		}

		$slices = $shadows = array();
		$totals = $this->data->getTotalsDisplay();
		$needleValue = 0;

		foreach ($labels as $name => $label) {
			if (isset($series['points'][$name])) {
				$value = $series['points'][$name]['n'];
				$valueCount = $series['points'][$name]['c'];
				if (!$value) {
					$i++;
					continue;
				}
				if (isset($totals[$name])) $strvalue = $totals[$name];
				else $strvalue = $value;

				if ($this->percentFormat) {
					$perc_label = sprintf("%0.1f%%", $value / $sum * 100);
				} else {
					$perc_label = $strvalue;
				}
				$value *= $scale;
				if ($name == $this->needleKey) $needleValue = $a + $value;

				$flag = $value >= pi() ? 1 : 0;
				$path = sprintf("M%f %f", $this->width/2 + cos($a) * $r1, $yoffs + sin($a)*$r2);
				$path .= sprintf("A%f,%f 0 $flag,1 %f %f", $r1, $r2, $this->width/2 + cos($a+$value) * $r1, $yoffs + sin($a+$value)*$r2);
				$path .= sprintf("L%f %f", $this->width/2 + cos($a+$value) * $r1 * 0.75 , $yoffs + sin($a+$value)*$r2*0.75);
				//$flag = 1-$flag;
				$path .= sprintf("A%f,%f 0 $flag,0 %f %f", $r1*0.75, $r2*0.75, $this->width/2 + cos($a) * $r1*0.75, $yoffs + sin($a)*$r2*0.75);
				$path .= "z";
				
				$sect = $doc->createElement('g');

				$pathEL = $this->createElement($doc, 'path', ['class' => 'segment']);
				$pathEL->setAttribute('d', $path);
				$pathEL->setAttribute('fill', $this->layout->getSeriesColor($i));
				$pathEL->setAttribute('stroke', $this->layout->getStrokeColor());
				$pathEL->setAttribute('transform', sprintf("translate(%f, %f)", $this->offset * cos($a + $value/2), $this->offset * sin($a+$value/2)));
				$sect->appendChild($pathEL);

				$metrics = $font->measure($perc_label, $fontSize);
				$tX = cos($a + $value/2) * ($r1 + 4);
				$tY = sin($a + $value/2) * ($r2 + 4);

				if ($tX < -0.001) {
					$tX -= $metrics['w'];
				} elseif ($tX > 0.001) {
				} else {
					$tX -= $metrics['w']/2;
				}
				
				if ($tY < -0.001) {
					$tY -= $metrics['h']/2;
				} elseif ($tY > 0.001) {
					$tY += $metrics['h'];
					$tY += $pieH;
				} else {
					$tY += $metrics['h']/2;
				}

				$hlId = $this->highlight($sect, array($strvalue, $name), array($strvalue, $label), $valueCount);

				$labelAttrs = ['font-family' => $font->getName(), 'font-size' => $fontSize];
				$labelX = $this->width / 2 + $tX + $this->offset*1.25 * cos($a + $value/2);
				$labelY = $yoffs + $tY + $this->offset*1.25 * sin($a + $value/2);
				$labelElt = $this->createSegmentLabel($doc, $perc_label, $labelX, $labelY, $labelAttrs, [$hlId, $name]);
				$sect->appendChild($labelElt);

				$g->appendChild($sect);
				$slices[] = $this->createLink($doc, $sect, $name);

				$a += $value;
				$n = floor($a / 2 / pi());
				$a = $a - $n * 2 * pi();
			}
			$i++;
		}

		foreach ($shadows as $shadow) {
			$g->appendChild($shadow);
		}
		foreach ($slices as $slice) {
			$g->appendChild($slice);
		}
		
		$needleSize = max($r1 / 18, 5);

		$path = sprintf("M%f %f", $this->width/2 + $needleSize * cos($needleValue+pi()/2), $yoffs + $needleSize*sin($needleValue+pi()/2));
		$path .= sprintf("L%f %f", $this->width/2 - $needleSize * cos($needleValue+pi()/2), $yoffs - $needleSize*sin($needleValue+pi()/2));
		$path .= sprintf("L%f %f", $this->width/2 + $r1 * cos($needleValue), $yoffs + $r1*sin($needleValue));
		$pathEL = $doc->createElement('path');
		$pathEL->setAttribute('d', $path);
		$pathEL->setAttribute('fill', 'black');
		$pathEL->setAttribute('fill-opacity', '0.5');
		$pathEL->setAttribute('stroke', $this->layout->getStrokeColor());
		$pathEL->setAttribute('stroke-width', 0.5);

		$g->appendChild($pathEL);

		return $g;
	}

	public function setNeedleKey($k)
	{
		$this->needleKey = $k;
	}

}

