<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

require_once 'include/SVGCharts/impl/SVGChartPlot.php';

class SVGChartPlotLine extends SVGChartPlot
{

	public function render(DOMDocument $doc)
	{
		$g = $doc->createElement('g');

		$font = $this->layout->getLegendFont();
		$fontSize = $this->layout->getLegendFontSize();

		$series = $this->data->getSeries();
		$ser_labels = $this->data->getSeriesLabels();
		$labels = $this->data->getLabels();
		$nLabels = is_countable($labels) ? count($labels) : 0;
		if ($nLabels) {
			$stepX = $this->width / $nLabels;
			$i = 0;
			foreach ($series as $name => $ser) {
				$path = "";
				$x = $stepX * 0.5;
				$rVal = null;
				foreach ($this->data->getLabels() as $col => $label) {
					if (isset($ser['points'][$col])) {
						$value = $ser['points'][$col]['n'];
						$y = ($value - $this->min) * $this->scale;
						if ($path == "") {
							$path = sprintf("M%f %f", $x, $this->height - $y);
						} else {
							$path .= sprintf("L%f %f", $x, $this->height - $y);
						}
						$rVal = array_get_default($ser['points'][$col], 'l');
					}
					$x += $stepX;
				}
				if ($path) {
					$id = uniqid('P');
					$sect = $this->createElement($doc, 'g', ['id' => $id, 'class' => 'line']);

					$hlEL = $this->createElement($doc, 'path', ['class' => 'hl']);
					$hlEL->setAttribute('d', $path);
					$hlEL->setAttribute('stroke', '#000000');
					$hlEL->setAttribute('stroke-width', 3);
					$hlEL->setAttribute('stroke-opacity', 0);
					$sect->appendChild($hlEL);

					// could be optimized with filter effects, feOffset etc
					$shadowEL = $this->createElement($doc, 'path', ['class' => 'shadow']);
					$shadowEL->setAttribute('d', $path);
					$sect->appendChild($shadowEL);

					$pathEL = $this->createElement($doc, 'path', ['class' => 'fg']);
					$pathEL->setAttribute('stroke', $this->layout->getSeriesColor($i));
					$pathEL->setAttribute('d', $path);
					$sect->appendChild($pathEL);
					
					$this->highlight($sect, array($value, $name, $col), array($value, $ser_labels[$name], $label), null, $rVal);
					$this->script .= "SVGCHARTDATA.subscribe('$id', SVGCHARTDATA.bringOnTop, 'highlight', '$name');";
					$this->script .= "SVGCHARTDATA.subscribe('$id', SVGCHARTDATA.highlight, 'highlight', '$name');";
					$this->script .= "SVGCHARTDATA.subscribe('$id', SVGCHARTDATA.unhighlight, 'unhighlight', '$name');";
					$g->appendChild($sect);
				}
				$i++;
			}
		}
		return $g;
	}
}

