<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

require_once 'include/SVGCharts/impl/SVGChartPlot.php';

class SVGChartPlotPie extends SVGChartPlot
{
	private $offset = 0;
	private $percentFormat = true;

	public function setOffset($offset)
	{
		$this->offset = $offset;
	}

	public function getPreferredHeight($width, $height) {
		return max(120, max(min($width, 1000) / 2, $height));
	}

	public function setPercentFormat($val)
	{
		$this->percentFormat = $val;
	}
	
	public function renderDefs(DOMElement $defs) {
		$grad = $this->addElement($defs, 'radialGradient',
			array('id' => "top-depthGrad", 'gradientUnits' => "userSpaceOnUse"));
		$this->addElement($grad, 'stop',
			array('offset' => "0%", 'stop-color' => "white", 'stop-opacity' => "0.4"));
		$this->addElement($grad, 'stop',
			array('offset' => "100%", 'stop-color' => "white", 'stop-opacity' => "0.0"));

		$grad = $this->addElement($defs, 'linearGradient',
			array('id' => "dark-grad", 'x1' => "0%", 'y1' => "0%", 'x2' => "100%", 'y2' => "30%", 'gradientUnits' => "userSpaceOnUse"));
		$this->addElement($grad, 'stop',
			array('offset' => "30%", 'stop-color' => "black", 'stop-opacity' => "0.2"));
		$this->addElement($grad, 'stop',
			array('offset' => "80%", 'stop-color' => "black", 'stop-opacity' => "0.5"));
	}

	public function render(DOMDocument $doc)
	{
		$g = $doc->createElement('g');

		
		$font = $this->layout->getLegendFont();
		$fontSize = $this->layout->getLegendFontSize();

		$labels = $this->data->getLabels();
		$nLabels = is_countable($labels) ? count($labels) : 0;

		$series = $this->data->getSeries();

		if (!$nLabels || !is_countable($series) || !count($series)) {
			return $g;
		}

		$ser = array();
		foreach ($series as $k => $v) {
			$ser = $v;
			break;
			foreach ($v['points'] as $kk => $vv) {
				if (!isset($ser['points'][$kk])) $ser['points'][$kk]['n'] = $vv['n'];
				else $ser['points'][$kk]['n'] += $vv['n'];
			}
		}
		$series = $ser;


		$sum = 0;
		foreach ($series['points'] as $point) {
			$sum += $point['n'];
		}
		$scale = $sum ? 2 * pi() / $sum : 0;

		$r1 = $this->width;
		if ($this->height < $r1) {
			$r1 = $this->height;
		}
		$r1 /= 2;
		$a = 0.4;
		$i = 0;

		$r1 -= $fontSize * 2.0;
		$r1 -= $this->offset;

		if ($this->threeD) {		
			$r2 = $r1 * 0.8;
			$pieH = $r1 * 0.2;
		} else {
			$r2 = $r1;
			$pieH = 0;
		}

		$slices = $shadows = $walls =  $outerWalls = array();
		$totals = $this->data->getTotalsDisplay();
		$xoffs = $this->width / 2;
		$yoffs = $this->height / 2;
		if($this->threeD) {
			$yoffs -= $fontSize * 1.5;
		}
		
		foreach ($labels as $name => $label) {
			if (isset($series['points'][$name])) {
				$value = $series['points'][$name]['n'];
				$valueCount = $series['points'][$name]['c'];
				if (!$value) {
					$i++;
					continue;
				}
				if (isset($totals[$name])) $strvalue = $totals[$name];
				else $strvalue = $series['points'][$name]['s'];
				$relValues = array_get_default($series['points'][$name], 'l');

				if ($this->percentFormat) {
					$perc_label = sprintf("%0.1f%%", $value / $sum * 100);
				} else {
					$perc_label = $strvalue;
				}
				
				$sect = $doc->createElement('g');
				$sect->setAttribute('stroke-width', '0.5');
				$sect->setAttribute('shape-rendering', 'geometricPrecision');
				
				if($value == $sum) {
					// draw circle instead of arc
					$a = 0;
					$value = pi();
					$pathEL = $this->createElement($doc, 'ellipse', ['class' => 'segment']);
					$pathEL->setAttribute('cx', $xoffs);
					$pathEL->setAttribute('cy', $yoffs);
					$pathEL->setAttribute('rx', $r1);
					$pathEL->setAttribute('ry', $r2);
				} else {
					$value *= $scale;
					$flag = $value >= pi() ? 1 : 0;
					$path = sprintf("M%f %f", $xoffs, $yoffs);
					$path .= sprintf("L%f %f", $xoffs + cos($a) * $r1, $yoffs + sin($a)*$r2);
					$path .= sprintf("A%f,%f 0 $flag,1 %f %f", $r1, $r2, $xoffs + cos($a+$value) * $r1, $yoffs + sin($a+$value)*$r2);
					$path .= "z";

					$pathEL = $this->createElement($doc, 'path', ['class' => 'segment']);
					$pathEL->setAttribute('d', $path);
				}

				$pathEL->setAttribute('fill', $this->layout->getSeriesColor($i));
				if(! $this->threeD)
					$pathEL->setAttribute('stroke', $this->layout->getStrokeColor());
				$pathEL->setAttribute('transform', sprintf("translate(%f, %f)", $this->offset * cos($a + $value/2), $this->offset * sin($a+$value/2)));
				$sect->appendChild($pathEL);

				if (!$this->threeD) {
					$shadow = clone $pathEL;
					$shadow->setAttribute('transform', sprintf("translate(%f, %f)", $this->offset * cos($a + $value/2) + 4, $this->offset * sin($a+$value/2) + 4));
					$shadow->setAttribute('stroke', 'none');
					$shadow->setAttribute('fill', '#666');
					$shadow->setAttribute('opacity', '0.5');
					$shadow->setAttribute('id', null);
					$shadows[] = $shadow;
				} else {
					$shine = clone $pathEL;
					$shine->setAttribute('fill', 'url(#top-depthGrad)');
					$shine->setAttribute('stroke', $this->layout->getStrokeColor());
					$sect->appendChild($shine);

					if ($a+$value < pi()/2 || $a+$value > 3*pi()/2) {
						$path = sprintf("M%f %f", $xoffs, $yoffs);
						$path .= sprintf("L%f %f", $xoffs + cos($a+$value) * $r1, $yoffs + sin($a+$value)*$r2);
						$path .= sprintf("v%f", $pieH);
						$path .= sprintf("L%f %f", $xoffs, $yoffs + $pieH);
						$path .= 'z';
						$wsect = $doc->createElement('g');
						$wall = $this->createElement($doc, 'path', ['class' => 'segment']);
						$wall->setAttribute('d', $path);
						$wall->setAttribute('transform', sprintf("translate(%f, %f)", $this->offset * cos($a + $value/2), $this->offset * sin($a+$value/2)));
						$wall->setAttribute('fill', $this->layout->getSeriesColor($i));
						$wsect->appendChild($wall);
						$wall = $this->createElement($doc, 'path', ['class' => 'segment']);
						$wall->setAttribute('d', $path);
						$wall->setAttribute('transform', sprintf("translate(%f, %f)", $this->offset * cos($a + $value/2), $this->offset * sin($a+$value/2)));
						$wall->setAttribute('fill', 'url(#dark-grad)');
						$wall->setAttribute('stroke', $this->layout->getStrokeColor());
						$wsect->appendChild($wall);
						$walls[(string)($a*1.01)] = $this->createLink($doc, $wsect, $name);
						$this->highlight($wsect, array($strvalue, $name),  null, $valueCount, $relValues, true);
					}
					if ($a > pi()/2 && $a < 3*pi()/2) {
						$path = sprintf("M%f %f", $xoffs, $yoffs);
						$path .= sprintf("L%f %f", $xoffs + cos($a) * $r1, $yoffs + sin($a)*$r2);
						$path .= sprintf("v%f", $pieH);
						$path .= sprintf("L%f %f", $xoffs, $yoffs + $pieH);
						$path .= 'z';
						$wsect = $doc->createElement('g');
						$wall = $this->createElement($doc, 'path', ['class' => 'segment']);
						$wall->setAttribute('d', $path);
						$wall->setAttribute('transform', sprintf("translate(%f, %f)", $this->offset * cos($a + $value/2), $this->offset * sin($a+$value/2)));
						$wall->setAttribute('fill', $this->layout->getSeriesColor($i));
						$wsect->appendChild($wall);
						$wall = $this->createElement($doc, 'path', ['class' => 'segment']);
						$wall->setAttribute('d', $path);
						$wall->setAttribute('transform', sprintf("translate(%f, %f)", $this->offset * cos($a + $value/2), $this->offset * sin($a+$value/2)));
						$wall->setAttribute('fill', 'url(#dark-grad)');
						$wall->setAttribute('stroke', $this->layout->getStrokeColor());
						$wsect->appendChild($wall);
						$walls[(string)($a)] = $this->createLink($doc, $wsect, $name);
						$this->highlight($wsect, array($strvalue, $name), null, $valueCount, $relValues, true);
					}
					
					$sa = $this->normalizeRadians($a);
					$ea = $this->normalizeRadians($a + $value);
					if($sa < pi()) {
						if($ea < $sa || $ea > pi())
							$lea = pi();
						else
							$lea = $ea;
						$path = sprintf("M%f %f", $xoffs + cos($sa) * $r1, $yoffs + sin($sa)*$r2);
						$path .= sprintf("A%f,%f 0 0,1 %f %f", $r1, $r2, $xoffs + cos($lea) * $r1, $yoffs + sin($lea)*$r2);
						$path .= sprintf("v%f", $pieH);
						$path .= sprintf("A%f,%f 0 0,0 %f %f", $r1, $r2, $xoffs + cos($sa) * $r1, $yoffs + sin($sa)*$r2 + $pieH);
						$path .= sprintf("v%f", -$pieH);
						$path .= 'z';
						$wsect = $doc->createElement('g');
						$wall = $this->createElement($doc, 'path', ['class' => 'segment']);
						$wall->setAttribute('d', $path);
						$wall->setAttribute('transform', sprintf("translate(%f, %f)", $this->offset * cos($a + $value/2), $this->offset * sin($a+$value/2)));
						$wall->setAttribute('fill', $this->layout->getSeriesColor($i));
						$wsect->appendChild($wall);
						$wall = $this->createElement($doc, 'path', ['class' => 'segment']);
						$wall->setAttribute('d', $path);
						$wall->setAttribute('transform', sprintf("translate(%f, %f)", $this->offset * cos($a + $value/2), $this->offset * sin($a+$value/2)));
						$wall->setAttribute('fill', 'url(#dark-grad)');
						$wall->setAttribute('stroke', $this->layout->getStrokeColor());
						$wsect->appendChild($wall);
						$aoffs = $ea > pi() / 2 ? 0.99 : 1.01;
						$walls[(string)($a + $value*$aoffs)] = $this->createLink($doc, $wsect, $name);
						$this->highlight($wsect, array($strvalue, $name), array($strvalue, $label), $valueCount, $relValues, true);
					}
					if($sa > $ea) {
						// for segment wrapping around the end of the circle (a=0)
						$lsa = 0;
						$path = sprintf("M%f %f", $xoffs + cos($lsa) * $r1, $yoffs + sin($lsa)*$r2);
						$path .= sprintf("A%f,%f 0 0,1 %f %f", $r1, $r2, $xoffs + cos($ea) * $r1, $yoffs + sin($ea)*$r2);
						$path .= sprintf("v%f", $pieH);
						$path .= sprintf("A%f,%f 0 0,0 %f %f", $r1, $r2, $xoffs + cos($lsa) * $r1, $yoffs + sin($lsa)*$r2 + $pieH);
						$path .= sprintf("v%f", -$pieH);
						$path .= 'z';
						$wsect = $doc->createElement('g');
						$wall = $this->createElement($doc, 'path', ['class' => 'segment']);
						$wall->setAttribute('d', $path);
						$wall->setAttribute('transform', sprintf("translate(%f, %f)", $this->offset * cos($a + $value/2), $this->offset * sin($a+$value/2)));
						$wall->setAttribute('fill', $this->layout->getSeriesColor($i));
						$wsect->appendChild($wall);
						$wall = $this->createElement($doc, 'path', ['class' => 'segment']);
						$wall->setAttribute('d', $path);
						$wall->setAttribute('transform', sprintf("translate(%f, %f)", $this->offset * cos($a + $value/2), $this->offset * sin($a+$value/2)));
						$wall->setAttribute('fill', 'url(#dark-grad)');
						$wall->setAttribute('stroke', $this->layout->getStrokeColor());
						$wsect->appendChild($wall);
						$aoffs = $ea > pi() / 2 ? 1.01 : 0.99;
						$walls[(string)($a + $value*$aoffs)] = $this->createLink($doc, $wsect, $name);
						$this->highlight($wsect, array($strvalue, $name), array($strvalue, $label), $valueCount, $relValues, true);
					}
				}



				$metrics = $font->measure($perc_label, $fontSize);
				$tX = cos($a + $value/2) * ($r1 + 4);
				$tY = sin($a + $value/2) * ($r2 + 4);

				if ($tX < -0.001) {
					$tX -= $metrics['w'];
				} elseif ($tX > 0.001) {
				} else {
					$tX -= $metrics['w']/2;
				}
				
				if ($tY < -0.001) {
					$tY -= $metrics['h']/2;
				} elseif ($tY > 0.001) {
					$tY += $metrics['h'];
					$tY += $pieH;
				} else {
					$tY += $metrics['h']/2;
				}
				
				$labelAttrs = ['font-family' => $font->getName(), 'font-size' => $fontSize];
				$labelX = $xoffs + $tX + $this->offset*1.25 * cos($a + $value/2);
				$labelY = $yoffs + $tY + $this->offset*1.25 * sin($a + $value/2);
				$labelElt = $this->createSegmentLabel($doc, $perc_label, $labelX, $labelY, $labelAttrs, $name);
				$sect->appendChild($labelElt);
				
				$this->highlight($sect, array($strvalue, $name), array($strvalue, $label), $valueCount, $relValues, true);
				$slices[] = $this->createLink($doc, $sect, $name);

				$a += $value;
				$n = floor($a / 2 / pi());
				$a = $a - $n * 2 * pi();
			}
			$i++;
		}

		uksort($walls, array($this, 'sortWalls'));

		foreach ($shadows as $shadow) {
			$g->appendChild($shadow);
		}
		foreach ($walls as $wall) {
			$g->appendChild($wall);
		}
		foreach ($outerWalls as $wall) {
			//$g->appendChild($wall);
		}
		foreach ($slices as $slice) {
			$g->appendChild($slice);
		}

		return $g;
	}

	public function sortWalls($a, $b)
	{
		return (sin($a) > sin($b)) ? 1 : -1;
	}
	
	public function normalizeRadians($r) {
		$n = floor($r / 2 / pi());
		return $r - $n * 2 * pi();
	}

}

