<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

require_once 'include/SVGCharts/impl/SVGChartPlot.php';

class SVGChartPlotRadar extends SVGChartPlot
{
	private $plotWidth;
	private $centerX;
	private $gridStep;

	private function calcPlotArea()
	{
		$font = $this->layout->getLegendFont();
		$fontSize = $this->layout->getLegendFontSize();

		if ($this->width > $this->height) {
			$this->plotWidth = $this->height;
		} else {
			$this->plotWidth = $this->width;
		}
		$this->plotWidth = $this->width;

		$labels = $this->data->getLabels();
		$labelsCount = is_countable($labels) ? count($labels) : 1;
		$dA = 2 * pi() / $labelsCount;
		$a = 0;
		$oXP = $oxM = $oYP = $oYM = 0;
		$sizes = array();
		foreach ($labels as $label) {
			$metrics = $font->measure($label, $fontSize);
			if (abs($a) < 0.001) {
				$dX = $metrics['w'];
				$dY= 0;
			} elseif (abs($a - pi()/2) < 0.001) {
				$dX = 0;
				$dY = - $metrics['h'];
			} elseif (abs($a - pi()) < 0.001) {
				$dX = - $metrics['w'];
				$dY = 0;
			} elseif (abs($a - pi()*3/2) < 0.001) {
				$dX = 0;
				$dY = $metrics['h'];
			} elseif ($a < pi() / 2) {
				$dY = $metrics['h'];
				$dX = 0;
			} elseif ($a < pi()) {
				$dY = $metrics['h'];
				$dX = - $metrics['w'];
			} elseif ($a < pi()*3/2) {
				$dY = 0;
				$dX = - $metrics['w'];
			} else {
				$dY = 0;
				$dX = 0;
			}
			$sizes[$label] = array(cos($a), sin($a),$dX, $dY);
			$a += $dA;
		}

		$r = $this->plotWidth;
		for (;;) {
			$min = -$r;
			$max = $r;
			foreach ($sizes as $size) {
				$w = $size[0] * ($r  + 12) + $size[2] * 1.2;
				if ($w < $min) $min = $w;
				if ($w > $max) $max = $w;
			}
			if ($max - $min <= $this->plotWidth) {
				break;
			}
			$r *= 0.99;
		}
		$this->plotWidth = $r*2;
		$this->centerX = $this->width / 2 - ($min + $max)*2;
	}


	private function renderWeb($doc)
	{
		$font = $this->layout->getLegendFont();
		$fontSize = $this->layout->getLegendFontSize();
		$labels = $this->data->getLabels();
		$labelsCount = is_countable($labels) ? count($labels) : 1;
		$dA = 2 * pi() / $labelsCount;
		$g = $doc->createElement('g');
		$a = 0;
		foreach ($labels as $label) {
			$line = $doc->createElement('line');
			$line->setAttribute('x1', $this->centerX);
			$line->setAttribute('y1', $this->height / 2);
			$line->setAttribute('x2', $this->centerX + cos($a) * $this->plotWidth / 2);
			$line->setAttribute('y2', $this->height / 2 + sin($a) * $this->plotWidth / 2);
			$line->setAttribute('stroke', '#CCC');
			$g->appendChild($line);


			$metrics = $font->measure($label, $fontSize);
			$bX = cos($a) * ($this->plotWidth / 2 + 3);
			$bY = sin($a) * ($this->plotWidth / 2 + 3);
			if (abs($a) < 0.001) {
				$tX = $bX;
				$tY = $bY + $metrics['h'] / 2;
			} elseif (abs($a - pi()/2) < 0.001) {
				$tY = $bY + $metrics['h'];
				$tX = $bX - $metrics['w'] / 2;
			} elseif (abs($a - pi()) < 0.001) {
				$tX = $bX - $metrics['w'];
				$tY = $bY + $metrics['h'] / 2;
			} elseif (abs($a - pi()*3/2) < 0.001) {
				$tY = $bY ;
				$tX = $bX - $metrics['w'] / 2;
			} elseif ($a < pi() / 2) {
				$tY = $bY + $metrics['h'];
				$tX = $bX;
			} elseif ($a < pi()) {
				$tY = $bY + $metrics['h'];
				$tX = $bX - $metrics['w'];
			} elseif ($a < pi()*3/2) {
				$tY = $bY;
				$tX = $bX - $metrics['w'];
			} else {
				$tY = $bY;
				$tX = $bX;
			}
			
			$labelAttrs = ['font-family' => $font->getName(), 'font-size' => $fontSize];
			$labelX = $tX + $this->centerX;
			$labelY = $tY + $this->height / 2;
			$labelElt = $this->createSegmentLabel($doc, $label, $labelX, $labelY, $labelAttrs);
			$g->appendChild($labelElt);

			$a += $dA;
		}


		$r = $this->gridStep * $this->scale;
		while ($r <= $this->plotWidth) {
			$a = 0;
			foreach ($labels as $label) {
				$line = $doc->createElement('line');
				$line->setAttribute('x1', $this->centerX + cos($a+$dA) * $r / 2);
				$line->setAttribute('y1', $this->height / 2 + sin($a+$dA) * $r / 2);
				$line->setAttribute('x2', $this->centerX + cos($a) * $r / 2);
				$line->setAttribute('y2', $this->height / 2 + sin($a) * $r / 2);
				$line->setAttribute('stroke', '#CCC');
				$g->appendChild($line);
				$a += $dA;
			}
			$r += $this->gridStep * $this->scale;
		}
		return $g;
	}

	private function calcGrid()
	{
		$max = 0;
		foreach ($this->data->getSeries() as $series) {
			foreach ($series as $value) {
				if ($value > $max) $max = $value;
			}
		}
		$minSpacing = 40;
		$gridStep = 1;
		for (;;) {
			$nSteps = ceil($max / $gridStep);
			if ($this->plotWidth / 2 / $nSteps >= $minSpacing) {
				break;
			}
			if (substr($gridStep, 0, 1) == '2') {
				$gridStep *= 2.5;
			} else {
				$gridStep *= 2;
			}
		}
		$this->gridStep = $gridStep;
		$this->scale = $this->plotWidth / 2 / ($nSteps * $gridStep);
	}

	public function render(DOMDocument $doc)
	{

		$g = $doc->createElement('g');
		
		$this->calcPlotArea();
		$this->calcGrid();
		$g->appendChild($this->renderWeb($doc));


		$font = $this->layout->getLegendFont();
		$fontSize = $this->layout->getLegendFontSize();

		$labels = $this->data->getLabels();
		$nLabels = is_countable($labels) ? count($labels) : 0;
		if ($nLabels) {
			$dA = 2 * pi() / $nLabels;
			$stepX = $this->width / $nLabels;
			$i = 0;
			foreach ($this->data->getSeries() as $series) {
				$path = "";
				$a = 0;
				$x = $stepX * 0.5;
				foreach ($this->data->getLabels() as $label) {
					if (isset($series[$label])) {
						$value = $series[$label];
						$r = $value * $this->scale;
						if ($path == "") {
							$path = sprintf("M%f %f", $this->width / 2 + cos($a+$dA) * $r, $this->height / 2 + sin($a+$dA) * $r);
						} else {
							$path .= sprintf("L%f %f", $this->width / 2 + cos($a+$dA) * $r, $this->height / 2 + sin($a+$dA) * $r);
						}
					}
					$a += $dA;
					$x += $stepX;
				}
				if ($path) {
					$path .= "Z";
					$pathEL = $doc->createElement('path');
					$pathEL->setAttribute('stroke', $this->layout->getSeriesColor($i));
					$pathEL->setAttribute('fill', 'none');
					$pathEL->setAttribute('d', $path);
					$pathEL->setAttribute('stroke-width', 2);
					$g->appendChild($pathEL);
				}
				$i++;
			}
		}
		return $g;
	}
	
}

