<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

class SVGFont
{
	private $name;
	private $font;
	private $used = array();
	private $kern = array();

	public function __construct($fontfile, $name)
	{
		$this->name = $name;
		include $fontfile;
		$this->font = $font;

	}


	public function getName()
	{
		return $this->name;
	}

	public function measure($str, $size)
	{
		$w = 0;
		$n = mb_strlen($str, 'UTF-8');
		
		if (!$n) return array('w' => 0, 'h' => $size);

		$c = mb_substr($str, 0, 1, 'UTF-8');
		for ($i = 0; $i < $n; $i++) {
			$nc = mb_substr($str, $i+1, 1, 'UTF-8');
			$this->used[$c] = true;
			$w += (int)@$this->font['glyphs'][$c]['a'];
			if ($nc !== '' && isset($this->font['kern'][$c][$nc])) {
				$w += $this->font['kern'][$c][$nc];
				$this->kern[$c][$nc] = true;
			}
			$c = $nc;
		}
		return array(
			'w' => $w * $size / $this->font['emsize'],
			'h' => $size,
		);
	}

	public function generate($doc)
	{
		$font = $doc->createElement('font');
		$fontFace = $doc->createElement('font-face');
		$font->appendChild($fontFace);

		$font->setAttribute('id', $this->name);
		$fontFace->setAttribute('font-family', $this->name);
		$fontFace->setAttribute('units-per-em', $this->font['emsize']);

		$missing = $doc->createElement('missing-glyph');
		$missing->setAttribute('horiz-adv-x', $this->font['missing']['a']);
		$missing->setAttribute('d', $this->font['missing']['d']);
		$font->appendChild($missing);

		foreach ($this->used as $c => $unused) {
			if (isset($this->font['glyphs'][$c])) {
				$glyph = $doc->createElement('glyph');
				$glyph->setAttribute('glyph-name', $this->font['glyphs'][$c]['n']);
				$glyph->setAttribute('unicode', $c);
				$glyph->setAttribute('horiz-adv-x', $this->font['glyphs'][$c]['a']);
				$glyph->setAttribute('d', $this->font['glyphs'][$c]['d']);
			}
			$font->appendChild($glyph);
		}
		foreach ($this->kern as $first => $others) {
			foreach ($others as $other => $unused) {
				$kern = $doc->createElement('hkern');
				$kern->setAttribute('u1', $first);
				$kern->setAttribute('u2', $other);
				$kern->setAttribute('k', $this->font['kern'][$first][$other]);
				$font->appendChild($kern);
			}
		}

		return $font;
	}
}

