<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');

require_once 'modules/UserPreferences/UserPreference.php';
require_once 'modules/Users/User.php';
require_once 'include/database/ListQuery.php';
require_once 'modules/Users/authentication/SugarAuthenticate/SugarAuthenticateUser.php';

class DAVAuthenticateUser extends SugarAuthenticateUser {

	function authenticateUser($name, $password, $params=null) {
		$lq = new ListQuery('User');
		$lq->addSimpleFilter('user_name', $name);
		$lq->addSimpleFilter('status', 'Active');

		$user = $lq->runQuerySingle();
		if (!$user || $user->failed)
			return '';

		$id = $user->getField('id');
		$user = new User;
		$user->retrieve($id);


		$dav_password = UserPreference::getPreference('dav_password', 'global', $user);
		if (strlen($dav_password)) {
			return $dav_password === sha1($password) ? $id : '';
		}

		$user_id = parent::authenticateUser($name, $password);
		if (! $user_id)
			return '';

		return $user_id;
	}


	
	/**
	 * this is called when a user logs in
	 *
	 * @param STRING $name
	 * @param STRING $password
	 * @return boolean
	 */
	function loadUserOnLogin($name, $password, $params=null) {
		$GLOBALS['log']->debug("Starting user load for ". $name);
		if(empty($name) || empty($password)) return false;
		checkAuthUserStatus();

		$user_id = $this->authenticateUser($name, $password, $params);
		if(empty($user_id)) {
			$GLOBALS['log']->fatal('SECURITY: [SabreDAV] User authentication for '.$name.' failed');
			return false;
		}

		$ret = $this->loadUserOnSession($user_id);
		if(! $ret) return false;

		return true;
	}


}

