<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/**
 * SubPanelTiles
 *
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 *
 ** Adapted from SubPanelTiles/TilesTabs for infoathand 7.0
 */


require_once('include/ListView/ListFormatter.php');
require_once('include/ListView/MobileListFormatter.php');
require_once('include/ListView/ListViewManager.php');


class SubPanelManager
{
	/**
	 * @var ModelDef
	 */
	var $model;
	var $record_id;
	var $record;
	var $layout_name;
	var $panel_list;
	var $visib_panels;
	var $list_ids = array();
	var $filter_fields = array();
	var $add_hidden_fields = array();
	var $show_browse_panel_link = true;
	var $show_config_panel_link = true;
	var $show_refresh_panel_link = true;
	var $is_primary = false;
	var $current_tab, $default_tab;
	var $nav_params_session = array();
	var $parent_list_id;
	
	function __construct(RowResult $record=null, $params=null) {
		if(is_array($params) && isset($params['is_primary']))
			$this->is_primary = $params['is_primary'];
		if(AppConfig::is_mobile())
			$this->show_browse_panel_link = false;
		if($record)
			$this->setRecord($record);
	}
	
	function setRecord(RowResult $record) {
		$this->record = $record;
		if($record) {
			$this->setModel($record->base_model);
			$this->record_id = $record->getField('id');
		}
	}
	
	function setModel($model) {
		if(is_string($model))
			$this->model = new ModelDef($model);
		else if($model instanceof ModelDef)
			$this->model = $model;
		else
			return false;
		return true;
	}
	
	function setLayoutName($name) {
		$this->layout_name = $name;
	}

	function setCurrentTab($tab) {
		$this->current_tab = $tab;
	}
	
	function setDefaultTab($tab) {
		$this->default_tab = $tab;
	}
	
	function addHiddenFields(array $fs) {
		array_extend($this->add_hidden_fields, $fs);
	}
	
	function setPanelList($panels, $limit_names=null) {
		$this->panel_list = array();
		$this->filter_fields = array();
		if(! is_array($limit_names))
			$limit_names = $limit_names ? array($limit_names) : null;
		if(is_array($panels)) {
			foreach($panels as $k => $p) {
				if(is_int($k)) {
					if(is_string($p)) $p = array('name' => $p);
					if(! is_array($p) || empty($p['name']))
						continue;
					$k = $p['name'];
				}
				else if(is_array($p)) {
					$p['name'] = $k;
				}
				else
					continue;
				if($limit_names && ! in_array($p['name'], $limit_names))
					continue;
				if(isset($p['hidden'])) {
					$hidden = expand_filter_flag($p['hidden']);
					$fs = get_filter_flag_fields($hidden);
					if($fs)
						$this->filter_fields = array_merge($this->filter_fields, $fs);
					$p['hidden'] = $hidden;
				}
				if(isset($p['no_create'])) {
					$no_create = expand_filter_flag($p['no_create']);
					$fs = get_filter_flag_fields($no_create);
					if($fs)
						$this->filter_fields = array_merge($this->filter_fields, $fs);
					$p['no_create'] = $no_create;
				}

				$link = $this->model->getLinkTargetModel($p['name']);
				if(! $link) continue;
				$p['module'] = $link->getModuleDir();
				$p['bean_name'] = $link->name;
				$p['model_type'] = $link->type;

				$joined = $link->getJoinModel();
				if($joined) {
					$lspec = $joined->getLinkSpec();
					$p['count'] = $lspec['relationship_spec']['right']['count'];
					$p['join_table'] = $lspec['name'];
					$p['target_key'] = $lspec['relationship_spec']['right']['key'];
					$p['source_key'] = $lspec['relationship_spec']['left']['key'];
					$p['managed'] = array_get_default($lspec['relationship_spec'], 'managed', false);
				} else {
					$lspec = $link->getLinkSpec();
					if($lspec && isset($lspec['relationship_spec'])) {
						$rts =& $lspec['relationship_spec']['right'];
						$p['count'] = $rts['count'];
						$p['target_key'] = $rts['key'];
						$p['source_key'] = $lspec['relationship_spec']['left']['key'];
						if (isset($lspec['add_existing']) && $lspec['add_existing'] == 1){
							$p['add_existing'] = true;
						}
						if(isset($rts['role_field'])) {
							$p['role_field'] = $rts['role_field'];
							$p['role_value'] = $rts['role_value'];
						}
						$p['managed'] = array_get_default($lspec['relationship_spec'], 'managed', false);
						$rkey = $lspec['relationship_spec']['right']['key'];
						$def = $link->getFieldDefinition($rkey);
						$required = array_get_default($def, 'required');
						if ($required)
							$p['removeable'] = false;
					} else {
						$p['count'] = 'many';
						$p['target_key'] = '';
						$p['managed'] = false;
					}
				}
				if(! isset($p['updateable']))
					$p['updateable'] = array_get_default($lspec, 'updateable', true);
				if(! isset($p['removeable']))
					$p['removeable'] = array_get_default($lspec, 'removeable', true);
				if(! isset($p['no_create']))
					$p['no_create'] = array_get_default($lspec, 'no_create', false);
				if(isset($p['managed']) && $p['managed'] === 'admin' && AppConfig::is_admin())
					$p['managed'] = false;
				if(empty($p['filter_name']))
					$p['filter_name'] = array_get_default($lspec, 'filter_name');
				if(empty($p['icon']))
					$p['icon'] = array_get_default($lspec, 'icon');
				if(isset($lspec['add_filters'])) {
					if(! isset($p['add_filters'])) $p['add_filters'] = array();
					array_extend($p['add_filters'], $lspec['add_filters']);
				}
				if(isset($lspec['initial_filter_fields'])) {
					if(! isset($p['initial_filter_fields'])) $p['initial_filter_fields'] = array();
					array_extend($p['initial_filter_fields'], $lspec['initial_filter_fields']);
				}
				if(! isset($p['detail_link_url']))
					$p['detail_link_url'] = array_get_default($lspec, 'detail_link_url');
				if(! isset($p['detail_link_action']))
					$p['detail_link_action'] = array_get_default($lspec, 'detail_link_action');
				if(empty($p['layout']))
					$p['layout'] = array_get_default($lspec, 'layout', 'Standard');
				if(isset($lspec['vname']) && !isset($p['title']))
					$p['title'] = $lspec['vname'];
				if(isset($lspec['vname_module']))
					$p['vname_module'] = $lspec['vname_module'];

				$this->panel_list[] = $p;
			}
		}
		$this->calcVisiblePanels();
	}
	
	function handlePanelInsert($link_name, $sel_ids, $params=null, $no_replace=false) {
		if(! isset($params))
			$params = $_REQUEST;
		if(! $this->record->getAclAllowed('edit'))
			return false;
		$upd = RowUpdate::for_result($this->record);
		$upd->setSource('subpanel');

		if ($sel_ids === 'all') {
			$sel_ids = $this->getAllIds($link_name, $params);

			if (! $sel_ids)
				return false;
		}

		if(! is_array($sel_ids)) $sel_ids = array($sel_ids);

		return $upd->addUpdateLinks($link_name, $sel_ids, $params, false, false, $no_replace);
	}

	function getAllIds($link_name, $params) {
		$link_model = $this->record->getLinkTargetModel($link_name);
		if(! $link_model)
			return false;

		$fmt = new \ListFormatter($link_model);

		$lq =& $fmt->getQuery();
		$lq->clearFields();
		$lq->addPrimaryKey();
		$lq->addAclFilter('view');

		if (isset($params['filter_favorites']) && $params['filter_favorites']) {
			require_once 'modules/Favorites/Favorite.php';
			\Favorite::add_join($lq);
		}

		$filter =& $fmt->getFilterForm();
		$filter->loadFilterLayout('Standard');
		$fmt->loadFilter($params);

		$result = $fmt->getQueryResult(0, -1);
		$rows = $result->getRows();

		return array_keys($rows);
	}

	function handlePanelRemove($link_name, $sel_id) {
		if(! $this->record->getAclAllowed('edit'))
			return false;
		$upd = RowUpdate::for_result($this->record);
		$upd->setSource('subpanel');
		return $upd->removeLink($link_name, $sel_id);
	}
	
	function handlePanelDelete($link_name, $sel_ids) {
		if(! $this->record->getAclAllowed('edit'))
			return false;
		if(! is_array($sel_ids) && $sel_ids !== 'all')
			$sel_ids = array($sel_ids);
		$link_model = $this->record->getLinkTargetModel($link_name);
		if(! $link_model)
			return false;
		$lq = new ListQuery($this->record->getModelName(), array('id', 'deleted'),
			array('link_name' => $link_name, 'parent_key' => $this->record->getPrimaryKeyValue(),
				  'acl_filters' => 'delete'));
		if($sel_ids !== 'all')
			$lq->addFilterPrimaryKey($sel_ids);
		$list_result = $lq->runQuery(0, null, null, null, 50);
		$c = 0;
		while(! $list_result->failed) {
			foreach($list_result->getRowIndexes() as $idx) {
				$row = $list_result->getRowUpdate($idx);
				$row->setSource('subpanel');
				if($row) {
					$row->markDeleted();
					$c ++;
				}
			}
			if($list_result->page_finished)
				break;
			$lq->pageResult($list_result);
		}
		return $c;
	}
	
	function calcVisiblePanels() {
		$panels = array();
		if (AppConfig::is_mobile()) {
			$mobile_modules = AppConfig::setting("modinfo.mobile_modules");
		} else {
			$mobile_modules = array();
		}
		foreach($this->panel_list as $p) {
			if(isset($p['hidden'])) {
				if(eval_filter_flag($p['hidden'], $this->record))
					continue;
			}
			if($this->current_tab) {
				$ptab = empty($p['tab']) ? $this->default_tab : $p['tab'];
				if (is_array($ptab)) {
					if (!in_array($this->current_tab, $ptab))
						continue;
				} elseif($ptab != $this->current_tab)
					continue;
			}
			if (!empty($mobile_modules) && !in_array($p['module'], $mobile_modules))
				continue;

            $views = array(
                "views.detail.{$p['module']}.subpanel.{$p['layout']}",
                "views.detail.{$p['module']}.list.{$p['layout']}",
            );
			$detail = AppConfig::setting($views);

            if (!empty($detail)) {
                $p['layout_type'] = $detail['type'];
                $p['layout_name'] = $detail['name'];
            }

			if(empty($p['layout_type'])){
				continue;
			}
			if (!ACLController::checkAccess($p['module'], 'list')) {
				continue;
			}

			
			$panels[$p['name']] = $p;
		}
		$this->visib_panels = $this->reorderSubpanels($panels);
	}

	function reorderSubpanels($panels) {
		$ret = array();
		$tab = $this->current_tab;
		if(! $tab) $tab = 'Standard';
		$order = UserPreference::getPreference('subpanels_order_' . $this->model->getModuleDir() . '_' . $tab, 'global');
		if (!$order)
			$order = array();
		foreach ($order as $o) {
			if (!is_array($o))
				$o = array('name' => $o, 'visible' => true);
			if (isset($panels[$o['name']])) {
				if (array_get_default($o, 'visible', true)) {
					$ret[$o['name']] = $panels[$o['name']];
				}
				unset($panels[$o['name']]);
			}
		}
		foreach ($panels as $k => $p)
			$ret[$k] = $p;
		return $ret;
	}

	function getVisiblePanels() {
		return $this->visib_panels;
	}
	
	function getPanelCount() {
		return is_countable($this->visib_panels) ? count($this->visib_panels) : 0;
	}
	
	
	function renderHtml() {
		ob_start();
		$out = $this->outputHtml();
		return ob_get_clean();
	}
	
	
	function getPanelButtons(array &$panel, $list_id) {
		$buttons = array();
		
		$popup_mode = ($panel['count'] == 'many') ? 'MultiSelect' : 'single';
		$can_update = $panel['updateable'];
		if (AppConfig::is_mobile())
			$can_update = false;
		if(! $this->record->getAclAllowed('edit'))
			$can_update = false;
		$managed = $panel['managed'];
		$no_create = eval_filter_flag($panel['no_create'], $this->record);
		
		if(isset($panel['add_filters'])) {
			$filt_set = array();
			foreach($panel['add_filters'] as $p) {
				if(! is_array($p) || empty($p['param'])) continue;
				$filt_set[$p['param']] = $p['value'];
			}
			$initial_filter = encode_query_string($filt_set);
		} else
			$initial_filter = '';

		if(isset($panel['initial_filter_fields']) && is_array($panel['initial_filter_fields'])) {
			foreach ($panel['initial_filter_fields'] as $field => $alias) {
				if ($this->record->getField($field)) {
                    if ($initial_filter != '')
                        $initial_filter .= '&';
				    $initial_filter .= urlencode($alias) .'='. urlencode($this->record->getField($field));
				}
			}
		}

		//field_to_name_array
		$fton_array = array('id' => 'selected_id');
		if(isset($panel['field_to_name_array']) && is_array($panel['field_to_name_array'])){
			array_extend($fton_array, $panel['field_to_name_array']);
		}
		
		$reload = array_get_default($panel, 'reload_page', false) ? 'true' : 'false';

		$popup_request_data = array(
			'call_back_function' => 'update_subpanel',
			'form_name' => $list_id . '-ListUpdate',
			'field_to_name_array' => $fton_array,
			'passthru_data' => array(
				'list_id' => $list_id,
				'reload_page' => $reload,
				'bean_name' => $panel['bean_name'],
			),
		);

		if (isset($panel['add_to_passthru_data']) && is_array($panel['add_to_passthru_data'])) {
			array_extend($popup_request_data['passthru_data'], $panel['add_to_passthru_data']);
		}		
		
		$json = getJSONobj();
		$json_encoded_php_array = $json->encode($popup_request_data);

		$initial_filter .= '&reset_list_pager=1';
        $key_value = $this->record_id;
        $key_name = '';
        $role_name = '';
        $role_value = '';
        if (! empty($panel['source_key']) && $this->record->getField($panel['source_key']))
            $key_value = $this->record->getField($panel['source_key']);

		if($panel['model_type'] == 'bean' && $panel['name'] != 'securitygroups' && $can_update && ! $no_create) {
			if($panel['target_key'] != 'id') {
				$key_name = $panel['target_key'];
				$mod_primary = AppConfig::module_primary_bean($panel['module']);
				if($panel['bean_name'] == $mod_primary) {
					$create_meta = "$key_name={$key_value}";
					if(isset($panel['role_field'])) {
						$create_meta .= "&{$panel['role_field']}={$panel['role_value']}";
						$q_create_meta = array($panel['role_field'] => $panel['role_value']);
						$role_name = $panel['role_field'];
						$role_value = $panel['role_value'];
					}
				}
			}
			else {
				$key_name = strtolower($this->model->name).'_id';
				$create_meta = "$key_name={$key_value}";
			}
		}
		
		if(! empty($create_meta)) {
            global $pageInstance;
            if(isset($panel['quick_create_layout']))
            	$quick_create_layout = $panel['quick_create_layout'];
            else if(! empty($panel['bean_name']))
            	$quick_create_layout = AppConfig::setting("display.view.{$panel['bean_name']}.quick_create_layout", 'QuickCreate');
            else
            	$quick_create_layout = 'QuickCreate';
			$quick_create_layout = AppConfig::setting("views.detail.{$panel['module']}.edit.$quick_create_layout");
            $panel_layout = '';
            if (! empty($panel['layout_name']))
                $panel_layout = $panel['layout_name'];

            if (! $quick_create_layout) {
                $return = 'return_module='.$pageInstance->module.'&return_action='.$pageInstance->action.'&return_record='.$this->record_id.'&return_layout='.$this->layout_name.'&return_panel='.$panel['name'];
                if ($panel_layout != '')
                    $panel_layout = '&layout='.$panel['layout_name'];
                $perform = "SUGAR.util.loadUrl('index.php?module={$panel['module']}&action=EditView&{$create_meta}&{$return}{$panel_layout}');";
            } else {
                $quick_create_params = array(
                    'target_id' => $list_id,
                    'target_name' => 'subpanel',
                    'target_layout' => $panel_layout,
                    'primary_name' => $key_name,
                    $key_name => $key_value,
                    'return_record' => $this->record_id,
                    'return_module' => $pageInstance->module,
                    'return_action' => 'DetailView',
                    'return_layout' => $this->layout_name,
                    'return_panel' => $panel['name']
				);
				if ($role_name) {
					$quick_create_params['role_field_name'] = $role_name;
					$quick_create_params[$role_name] = $role_value;
				}
                if(isset($q_create_meta)) array_extend($quick_create_params, $q_create_meta);

                $quick_create_params = $json->encode($quick_create_params);
                $perform = "SUGAR.ui.openQuickCreate('{$panel['module']}', $quick_create_params)";
            }

			$buttons['create'] = array(
				'icon' => 'icon-edit',
				'label' => translate('LBL_CREATE_BUTTON_LABEL', 'app'),
				'perform' => $perform,
				'acl' => 'create',
				'params' => array(
					'toplevel' => 1,
				),
			);
		}
		
		// Automatically showing Select button for many-to-many relationships using a join table
		// Allows select for one-to-many relationships, but only for panels with 'add_existing' set
		if( (! empty($panel['join_table']) || isset($panel['add_existing'])) && $can_update && ! $managed) {
			$buttons['select'] = array(
				'label' => translate('LBL_ADD_EXISTING', 'app'),
				'icon' => 'icon-popup',
				'perform' => "return open_popup(\"{$panel['module']}\",undefined,undefined,\"$initial_filter\",true,true,$json_encoded_php_array,\"$popup_mode\");",
				'params' => array(
					'toplevel' => 1,
				),
			);
		}
		
		if(! $panel['updateable'] || ! $panel['removeable'] || $panel['managed'] || ! $this->is_primary) {
			// no remove button
		} else {
			$lang_confirm = addcslashes(to_html(translate('NTC_REMOVE_CONFIRMATION', 'app')), "'");
			$buttons['remove'] = array(
				'label' => translate('LBL_REMOVE', 'app'),
				'icon' => 'icon-delete',
				'confirm' => $lang_confirm,
				'perform' => "return sListView.sendMassUpdate('$list_id', 'subpanel_remove');",
			);
		}

		$layout = AppConfig::setting("views.layout.{$panel['module']}.{$panel['layout_type']}.{$panel['layout_name']}");
		if($layout && ! empty($layout['buttons'])) {
			foreach($layout['buttons'] as $k => $bspec) {
				if(! is_array($bspec))
					continue;
				if(is_int($k)) $k = count($buttons);
				if(isset($bspec['template'])) {
					$tpl = $bspec['template'];
					unset($bspec['template']);
										
					if($tpl == 'add_existing') {
						if(empty($bspec['icon'])) $bspec['icon'] = 'icon-popup';
						$addmod = $bspec['module'];
						$addfilter = '';
						
						$fton_array = array('id' => 'selected_id');
						if(isset($bspec['field_to_name_array']) && is_array($bspec['field_to_name_array'])){
							array_extend($fton_array, $bspec['field_to_name_array']);
						}

						$popup_request_data = array(
							'call_back_function' => 'update_subpanel',
							'form_name' => $list_id . '-ListUpdate',
							'field_to_name_array' => $fton_array,
							'passthru_data' => array(
								'list_id' => $list_id,
								'reload_page' => $reload,
								'bean_name' => AppConfig::module_primary_bean($bspec['module']),
							),
						);
						if (isset($bspec['passthru_data']) && is_array($bspec['passthru_data'])) {
							array_extend($popup_request_data['passthru_data'], $bspec['passthru_data']);
						}
						$json_encoded_php_array = $json->encode($popup_request_data);

						$bspec['perform'] = "return open_popup(\"{$addmod}\",undefined,undefined,\"$addfilter\",true,true,$json_encoded_php_array,\"$popup_mode\");";
					}
				}
				$bspec['params']['parent_module'] = $this->model->getModuleDir();
				$bspec['params']['parent_id'] = $this->record_id;
                $bspec['params']['list_id'] = $list_id;
                $bspec['params']['panel_name'] = $panel['name'];
				$buttons[$k] = $bspec;
			}
		}
		
		if($panel['name'] == 'report_data' && $can_update) {
			$lang_confirm = addcslashes(to_html(translate('NTC_DELETE_CONFIRMATION_MULTIPLE', 'app')), "'");
			$buttons['remove'] = array(
				'label' => translate('LBL_DELETE', 'app'),
				'icon' => 'icon-delete',
				'confirm' => $lang_confirm,
				'perform' => "return sListView.sendMassUpdate('$list_id', null, null, {record_perform: 'report_data_remove'});",
			);
		}
		
		return $buttons;
	}
	
	
	function renderPanelByName($name, $list_id=null, $offset=null, $order_by = null) {
        $panels = $this->getVisiblePanels();
		foreach($panels as &$p) {
			if($p['name'] == $name) {
				if(isset($list_id))
					$this->list_ids[$name] = $list_id;
				$this->renderPanel($p, false, $offset, $order_by);
				return true;
			}
		}
		return false;
	}

	function renderPanel(array &$panel, $showContainer=false, $offset=null, $order_by = null) {
		$params = array('listview_manager' => $this);
		if (AppConfig::is_mobile()) {
			$fmt = new MobileListFormatter($this->model, $panel['name'], null, 'subpanel', $params);
		} else {
			$fmt = new ListFormatter($this->model, $panel['name'], null, 'subpanel', $params);
		}
		if($this->record->base_model == 'Report') {
			$fmt->report_id = $this->record_id;
		}

        $fmt->show_compact_empty = true;
		if(! $panel['updateable'] || ! $panel['removeable'] || $panel['managed'])
			$fmt->hide_remove_links = true;
		$icon_links = '';

		$show_config_panel_link = array_get_default($panel, 'show_config_panel_link', true);
		if($show_config_panel_link && $this->show_config_panel_link && AppConfig::is_admin() && ! AppConfig::is_mobile()) {
			$detail_layout = array_get_default($_REQUEST, 'layout', 'Standard');
			$options = array(
				'module' => 'NewStudio',
				'action' => 'index',
				'query_string' => 'wizard=Subpanel&edit_module=' . $this->model->getModuleDir() . '&record=' . $this->record_id . '&subpanel=' . $panel['name'] . '&list_id=' . $fmt->list_id . '&detail_layout=' . $detail_layout,
				'inline' => true,
			);

			$json = getJSONobj();
			$encodedOptions = $json->encode($options);
			$icon_links .= '<div class="uii uii-gear tool-icon active-icon" onclick=\'open_popup_window(' . $encodedOptions . ');\'></div>';
		}
		
		if($this->show_refresh_panel_link) {
			$icon_links .= '<div class="uii uii-refresh tool-icon active-icon" onclick="sugarListView.reloadSubpanel(\''.javascript_escape($panel['name']).'\');"></div>';
		}

		$parent_module = $this->model->getModuleDir();
		$layout = array_get_default($_REQUEST, 'layout', '');
		$list_url = "index.php?module={$parent_module}&action=ListView&record={$this->record_id}&subpanel={$panel['name']}&parent_layout=$layout&parent_tab=".$this->current_tab;
		if($this->show_browse_panel_link) {
			$icon_links .= '<div class="uii uii-extern tool-icon active-icon" onclick="SUGAR.util.loadUrl(\''.javascript_escape($list_url).'\');"></div>';
		}


		if ($icon_links) {
			$fmt->additional_title = $icon_links;
		}

		if(! empty($this->list_ids[$panel['name']]))
			$fmt->list_id = $this->list_ids[$panel['name']];
		else
			$this->list_ids[$panel['name']] = $fmt->list_id;

		$buttons = $this->getPanelButtons($panel, $fmt->list_id);

		if($fmt->loadLayout($panel['layout_name'], $panel['layout_type'], $panel['module'])) {
            $this->setFormatterParentInfo($fmt, $panel);
			$fmt->setFormat('html', 'subpanel');

			$nav = $this->getNavParams($panel);
			$limit = $nav['limit'];

			if (empty($offset))
				$offset = $nav['offset'];

			if (empty($order_by))
				$order_by = $nav['order_by'];

			$this->saveNavParamsSession(
				[
					'offset' => $offset,
					'limit' => $limit,
					'order_by' => $order_by
				],
				$panel['name']
			);

			if(isset($panel['detail_link_url']))
				$fmt->detail_link_url = $panel['detail_link_url'];
			if(isset($panel['detail_link_action']))
				$fmt->detail_link_action = $panel['detail_link_action'];

            $result = $fmt->getFormattedResult($offset, $limit, $order_by);
			$fmt->show_mass_update = false;
			$fmt->show_filter = false;
			if(! empty($panel['show_checks']))
				$fmt->show_checks = $panel['show_checks'];
			else
				$fmt->show_checks = false;
			$fmt->can_shrink = true;

			$fmt->getMassUpdate()->addHiddenFields(array(
				'link_name' => $panel['name'],
				'layout' => $layout,
				'layout_tab' => $this->current_tab,
				'record' => $this->record_id,
				'record_perform' => 'subpanel_render'));
			if($this->add_hidden_fields)
				$fmt->getMassUpdate()->addHiddenFields($this->add_hidden_fields);
			
			$mod = $panel['module'];
			$icon = array_get_default($panel, 'icon', $mod);
			$title = array_get_default($panel, 'title');
			if($title)
				$title = translate($title, array_get_default($panel, 'vname_module', $parent_module));
			else
				$title = $GLOBALS['app_list_strings']['moduleList'][$mod];
			$title = get_image($icon, '', '') . '&nbsp;' . $title;
			$fmt->setTitle('<span id="subpanel-'.$panel['name'].'">'.$title.'</a>');
			if($buttons)
				$fmt->addButtons($buttons);
			
			if($showContainer) {
				$have_rows = (!$result->total_counted || $result->total_count);
				echo '<div class="subpanel-entry' . ($have_rows ? '' : ' noResults').'" id="' . $fmt->list_id . '-outer">';
				$outer_end = '</div>';
			} else {
				/*echo '<div class="subpanelOuter" id="' . $fmt->list_id . '-outer">';
				$outer_end = '</div>';*/
				$outer_end = '';
			}

			$fmt->outputHtml($result);
			$GLOBALS['pageInstance']->add_js_literal("sListView.init('{$fmt->list_id}', {type: 'subpanel',"
				. "parent_module: '{$this->model->getModuleDir()}', parent_record: '" . javascript_escape($this->record_id) . "',"
				. 'count_options: ['.implode(', ', $this->getCountOptions()).'],'
				. "subpanel_name: '" . javascript_escape($panel['name']) . "', subpanel_tab: '" . $this->current_tab ."'});", null, LOAD_PRIORITY_FOOT);
			$fmt->exportIncludes();

			echo $outer_end;

			if ($showContainer) {
				$this->exportDocumentDropJS($panel, $fmt->list_id);
			}

		}
	}

	function getCountOptions() {
		$opts = AppConfig::setting('layout.dashlet_display_row_options');
		$max = $this->getMaxDisplayRows();
		$ret = array();
		foreach($opts as $o)
			if($o <= $max) $ret[] = $o;
		return $ret;
	}

	function getMaxDisplayRows() {
		return AppConfig::setting('layout.list.max_display_rows', 100);
	}

	function getNavParamsSessionKey($panel_name) {
		return "SubPanel_{$this->model->getModuleDir()}_{$panel_name}_Nav";
	}

	function loadNavParamsSession($panel_name) {
		if ($panel_name) {
			$key = $this->getNavParamsSessionKey($panel_name);
			$params = array_get_default($_SESSION, $key);

			if (! empty($params)) {
				if (array_get_default($params, 'record_id') === $this->record_id)
					$this->nav_params_session[$panel_name] = $params;
			}
		}
	}

	function saveNavParamsSession($params, $panel_name) {
		if ($panel_name) {
			$key = $this->getNavParamsSessionKey($panel_name);
			$_SESSION[$key] = $params;
		}
	}

	function getNavParams($panel) {
		$this->loadNavParamsSession($panel['name']);

		$offset = 0;
		$limit = AppConfig::setting('layout.list.subpanel_display_rows');
		$order_by = array_get_default($panel, 'default_order_by');

		if(!empty($this->nav_params_session[$panel['name']])) {
			$offset = array_get_default($this->nav_params_session[$panel['name']], 'offset', $offset);
			$limit = array_get_default($this->nav_params_session[$panel['name']], 'limit', $limit);
			$order_by = array_get_default($this->nav_params_session[$panel['name']], 'order_by', $order_by);
		}

		$offset = array_get_default($_REQUEST, 'list_offset', $offset);
		$limit = array_get_default($_REQUEST, 'list_limit', $limit);

		if(isset($offset)) {
			if(! is_numeric($offset)) $offset = 0;
			else
				$offset = max(0, round($offset));
		}

		if(isset($limit)) {
			if(! is_numeric($limit)) $limit = 10;
			else {
				$limit = round($limit);
				$max = $this->getMaxDisplayRows();
				if($max) $limit = min($limit, $max);
			}
		}

		$order_by = array_get_default($_REQUEST, 'list_order_by', $order_by);

		return compact('offset', 'limit', 'order_by');
	}

	function exportDocumentDropJS($panel, $list_id) {
		if ($panel['bean_name'] != 'Document' && $panel['bean_name'] != 'ActivityHistory') {
			return;
		}
		$opts = [
			'list_id' => $list_id,
			'model' => $panel['bean_name'] == 'Document' ? 'Document' : 'Note',
			'module' => $panel['module'],
			'parent_model' => $this->model->name,
			'parent_id' => $this->record_id,
		];
        $json = getJSONobj();
        $GLOBALS['pageInstance']->add_js_literal("DNDUploader(" . $json->encode($opts)  . ");", null, LOAD_PRIORITY_FOOT);
	}

    /**
     * @param ListFormatter | MobileListFormatter $formatter
     * @param array $panel
     */
    function setFormatterParentInfo(&$formatter, $panel) {
    	global $pageInstance;
    	
        $key = null;
        if (! empty($panel['source_key']) && $panel['source_key'] != 'id') {
            $key = $this->record->getField($panel['source_key']);
        }
        if (! $key)
            $key = $this->record_id;
        $formatter->setParentInfo($key, $pageInstance->action, $this->layout_name, $panel['name']);
    }
    
    function getRequiredFields() {
        $reqd = $this->filter_fields;
        $panels = $this->panel_list;
        foreach($panels as $panel) {
            if (! empty($panel['source_key']) && $panel['source_key'] != 'id')
                $reqd[] = $panel['source_key'];
        }
        return array_unique($reqd);
    }

    function showTabsEnabled()
    {
    	return false; // disabled for now, maybe permanently
    	
        global $current_user;
        $showTabs = $current_user->getPreference('subpanel_tabs');
        if(!isset($showTabs)) $showTabs = AppConfig::setting('layout.defaults.subpanel_tabs');

        if ($showTabs) {
            $tabs = array();
            $selectedGroup = "All";
            $panels = $this->getVisiblePanels();

            foreach($panels as $p) {
                $tabs[$p['name']] = $p['module'];
            }

            $GLOBALS['pageInstance']->add_js_include('include/SubPanel/SubPanelTabs.js', null, LOAD_PRIORITY_FOOT);

            return $this->getTabs($tabs, $selectedGroup);
        } else {
            return null;
        }
    }


	//commented out, see comment in showTabsEnabled()
    function getTabs($tabs, $selectedGroup = 'All') {
        // require_once('include/GroupedTabs/GroupedTabStructure.php');
        // $groupedTabsClass = new GroupedTabStructure();
        // $group_defs = $groupedTabsClass->get_tab_structure('', '', true);

        // $revTabs = array();
        // foreach($tabs as $tab => $mod)
        //     $revTabs[$mod][] = $tab;

        // $groups = array();
        // $shown = array();
        // foreach($group_defs as $nm => $def) {
        //     foreach($def['modules'] as $mod) {
        //         if(isset($revTabs[$mod]) && empty($shown[$mod])) {
        //             if(! isset($groups[$nm])) {
        //                 $groups[$nm] = array('modules' => array());
        //                 if(isset($def['label']))
        //                     $groups[$nm]['label'] = $def['label'];
        //             }
        //             $groups[$nm]['modules'] = array_merge($groups[$nm]['modules'], $revTabs[$mod]);
        //             $shown[$mod] = true;
        //         }
        //     }
        // }

        // /* Move history to same tab as activities */
        // if(isset($tabs['history']) && isset($tabs['activities'])) {
        //     foreach($groups as $mainTab => $group) {
        //         $lmodules = array_map('strtolower', $group['modules']);
        //         if(in_array('activities', $lmodules)) {
        //             if(!in_array('history', $lmodules)) {
        //                 /* Move hist from there to here */
        //                 $groups[$mainTab]['modules'] []= 'history';
        //             }
        //         } else if(false !== ($i = array_search('history', $lmodules))) {
        //             unset($groups[$mainTab]['modules'][$i]);
        //             if(empty($groups[$mainTab]['modules'])) {
        //                 unset($groups[$mainTab]);
        //             }
        //         }
        //     }
        // }

        // if(count($groups) <= 1) {
        //     $groups = array('All' => array('label' => 'LBL_TABGROUP_ALL', 'modules' => array_keys($tabs)));
        // } else {
        //     $groups = array('All' => array('label' => 'LBL_TABGROUP_ALL', 'modules' => array_keys($tabs))) + $groups;
        // }

        // $mainTabs = array();

        // foreach ($groups as $key => $tab) {
        //     $display = false;

        //     foreach($tab['modules'] as $subkey => $subtab) {
        //         if(isset($tabs[strtolower($subtab)])) {
        //             $display = true;
        //             break;
        //         }
        //     }

        //     $selected = 'other';
        //     if($selectedGroup == $key) $selected = 'current';

        //     if($display) {
        //         $mainTabs[$key] = array('label'=>$key, 'type'=>$selected);
        //         if($key == 'All') $mainTabs[$key]['label'] = $GLOBALS['app_strings']['LBL_TABGROUP_ALL'];
        //     }
        // }

        // $this->renderTabs($mainTabs, $selectedGroup);

        // return $groups;
    }

    function renderTabs($mainTabs, $selectedGroup = "All") {
        echo '<ul class="subpanelTablist" id="groupTabs">';

        foreach ($mainTabs as $tab => $params) {
            echo '<li id="'.$tab.'_sp_tab" style="margin-right: 5px;"><a class="'.$params['type'].'" href="javascript:SUGAR.tabUtils.switchTab(\''.$tab.'\');">'.$params['label'].'</a></li>';
        }

        echo '</ul><table width="100%" cellspacing="0" cellpadding="0" border="0" class="subpanelTabForm" style="border-top: 0px none; margin-bottom: 4px;"><tr><td>';
    }

    function loadGroups($tabs) {
        $groups = array();

        foreach ($tabs as $name => $params) {
            foreach ($this->list_ids as $module => $list_id) {
                if (in_array($module, $params['modules'])) {
                    $groups[$name][] = $list_id;
                }
            }
        }

        $json = getJSONobj();
        $json_encoded_groups = $json->encode($groups);
        $GLOBALS['pageInstance']->add_js_literal("SUGAR.tabUtils.initGroups(".$json_encoded_groups.");", null, LOAD_PRIORITY_FOOT);
    }
    
    
    function openContainer($form, $defer=false) {
    	global $pageInstance;
		$pageInstance->add_js_include("include/javascript/dropzone.js", null, LOAD_PRIORITY_FOOT);
		$pageInstance->add_js_include("include/javascript/dnduploads.js?" . time() , null, LOAD_PRIORITY_FOOT);
    	$form_name = $form->getFormName();
    	if($defer) {
			$form->onInitForm('sugarListView.loadSubpanels("' . javascript_escape($form_name) . '");');
		} else {
			$pageInstance->add_js_literal("setTimeout(function() {sugarListView.setupSubpanelsList('{$form_name}-subpanels')}, 50);" , null, LOAD_PRIORITY_FOOT);
		}
		return '<div id="' . to_html($form_name . '-subpanels') . '" class="subpanel-list">';
    }
    
    function closeContainer() {
    	return '</div>';
    }

	
	function outputHtml()
	{
        $panels = $this->getVisiblePanels();
        $tabs = $this->showTabsEnabled();

        foreach($panels as &$p) {
        	$this->renderPanel($p, true);
        }

        if ($tabs != null && is_countable($tabs) && sizeof($tabs) > 0) {
            $this->loadGroups($tabs);
            echo '</td></tr></table>';
        }
	}
}
?>
