<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/

require_once('include/dir_inc.php');

class Sugar_Smarty extends Smarty
{
	var $inc_mgr;
	var $display_export_includes = true;

	function __construct()
	{
		global $themeInstance;

		parent::__construct();

		$smarty_dir = AppConfig::setting('site.paths.smarty_dir', 'cache/smarty/');
		if($smarty_dir && substr($smarty_dir, -1) !== '/') $smarty_dir .= '/';

		if(!file_exists($smarty_dir)) mkdir_recursive($smarty_dir, true);
		if(!file_exists($smarty_dir . 'templates_c')) mkdir_recursive($smarty_dir . 'templates_c', true);
		if(!file_exists($smarty_dir . 'configs')) mkdir_recursive($smarty_dir . 'configs', true);
		if(!file_exists($smarty_dir . 'cache')) mkdir_recursive($smarty_dir . 'cache', true);

		$this->addPluginsDir('include/Smarty/plugins');

		$this->setTemplateDir('.');
		$this->setCompileDir($smarty_dir . 'templates_c');
		$this->setConfigDir($smarty_dir . 'configs');
		$this->setCacheDir($smarty_dir . 'cache');

		// longreach - added
		$this->assign('SUGAR_VERSION', js_version_ident());
		$this->assign('JS_CUSTOM_VERSION', AppConfig::js_custom_version());
		$this->assign('JS_PREFIX', js_prefix());
		$this->assign('JS_SUFFIX', js_suffix());
		$this->assign('CSS_PREFIX', css_prefix());
		$this->assign('CSS_SUFFIX', css_suffix());
		$this->assign('STATIC_URL', AppConfig::site_static_url());
		$this->assign('LIST_ADD_HR', ! isset($themeInstance) || $themeInstance->layout_version < 8);

		require_once('include/layout/IncludeManager.php');
		$this->inc_mgr = new IncludeManager();

		$this->error_reporting = error_reporting() & ~E_NOTICE; // suppress undefined variable errors
	}

	function &get_includes() {
		return $this->inc_mgr;
	}

	function display($template = NULL, $cache_id = NULL, $compile_id = NULL, $parent = NULL) {
		$ret = parent::display($template, $cache_id, $compile_id, $parent);
		if($this->inc_mgr && $this->display_export_includes)
			$this->export_includes();
		return $ret;
	}

	function export_includes() {
		global $pageInstance;
		if(isset($pageInstance))
			$pageInstance->merge_includes($this->inc_mgr);
	}

}

?>
