<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */



class TallyUpdate extends RowUpdate {
	var $orig_rows = array();
	var $groups = array();
	var $totals;
	var $group_model;
	var $line_model;
	var $adj_model;
	var $comment_model;
	var $currency_id;
	var $currency_decimals = 2;
	var $exchange_rate;
	var $uniq_row_idx = 0;
	var $record_id;
	var $auto_retrieve_groups = true;
	var $groups_updated;
	var $rel_id_field;
	
	// used by group update
	var $temp_id_map;
	var $id_map;
	var $cleanup_delete;

	var $input_line_items;
	var $orig_totals;

	static $total_fields = array('subtotal', 'total', 'cost', 'tax', 'discount', 'shipping');	
	
	
	function setModel($model) {
		$ret = parent::setModel($model);
		$this->group_model = AppConfig::setting("model.links.{$this->model_name}.line_groups_link.bean_name");
		$this->line_model = AppConfig::setting("model.links.{$this->model_name}.lines_link.bean_name");
		$this->adj_model = AppConfig::setting("model.links.{$this->model_name}.adjustments_link.bean_name");
		$this->comment_model = AppConfig::setting("model.links.{$this->model_name}.comments_link.bean_name");
		$rel = $this->model->getLinkRelationship('lines_link');
		$this->rel_id_field = $rel['right']['key'];
		return $ret;
	}

	function setOriginal($original) {
		$ret = parent::setOriginal($original);
		if($ret) {
			$this->record_id = $this->getPrimaryKeyValue();
			if ($original instanceof RowUpdate)
				$rows = $original->getField('line_items');
			if(! empty($rows)) {
				$this->clearOrigRows();
				$this->replaceGroups($rows);
			}
			else if($this->auto_retrieve_groups)
				$this->retrieveGroups();
			else
				$this->clearOrigRows();
		}
		return $ret;
	}
	
	function clearOrigRows() {
		$this->orig_rows = array(
			$this->group_model => array(),
			$this->line_model => array(),
			$this->comment_model => array(),
			$this->adj_model => array(),
		);
	}
	
	function loadCurrency() {
		$this->currency_id = $this->getField('currency_id');
		$this->exchange_rate = $this->getField('exchange_rate', 1);
		$currency = AppConfig::db_row('Currency', $this->currency_id);
		if($currency)
			$this->currency_decimals = $currency['decimal_places'];
		else
			$this->currency_decimals = Currency::getDefaultDecimalPlaces();
	}
	
	function performSet(array $update, $as_user=false) {
		if(isset($update['line_items']))
			$groups = $update['line_items'];
		else if(isset($this->input_line_items))
			$groups = $this->input_line_items;
		unset($update['line_items']);
		unset($this->input_line_items);
		$ret = parent::performSet($update, $as_user);
		if(isset($groups)) {
			if(is_string($groups) && substr($groups, 0, 1) == '[') {
				$json = getJSONobj();
				$g = $json->decode($groups);
				if(is_array($g)) {
					$groups = [];
					foreach($g as $grp) {
						if(is_array($grp) && ! empty($grp['id']))
							$groups[$grp['id']] = $grp;
					}
				} else
					$groups = null;
			}
			if(is_array($groups))
				$this->replaceGroups($groups);
		}
		return $ret;
	}
	
	function loadRequest($prefix = '', $formatted = true, $ignore_blank = false) {
		parse_str(file_get_contents('php://input'), $formData);
		
		if(isset($formData[$prefix.'line_items']))
			$this->input_line_items = $formData[$prefix.'line_items'];
		
		return parent::loadRequest($prefix, $formatted, $ignore_blank);
	}
	
	function loadBlankRecord() {
		return parent::loadBlankRecord();
	}
	
	
	function getGroups($strip_ids=false) {
		if($strip_ids) {
			$g = $this->groups;
			$this->makeDuplicateGroups($g);
			return $g;
		}
		return $this->groups;
	}
	
	function getTotals() {
		return $this->totals;
	}
	
	function getLineCount() {
		$c = 0;
		foreach($this->groups as $g)
			$c += is_countable($g['lines']) ? count($g['lines']) : 0;
		return $c;
	}
	
	function fillDefaults() {
		if($this->new_record) {
			if ($this->getModelName() == 'Invoice' && !$this->getField('terms')) {
				$this->set('terms', AppConfig::setting('company.invoice_default_terms', ''));
			}
			if($this->getField('pretax') === null) {
				$this->set('pretax', 0.0);
			}
		}
		parent::fillDefaults();
	}
	
	function afterSave() {
		$this->record_id = $this->getPrimaryKeyValue();
		if($this->groups_updated)
			$this->saveGroups();
		$this->updateStock();
		parent::afterSave();
	}
	
	function markDeleted($deleted = true)
	{
		$ret = parent::markDeleted($deleted);
		$this->updateStock(true);
		return $ret;
	}
	
	function saveGroups() {
		if(! $this->record_id) return false;
		$updates = $this->collectUpdates();
		$this->commitUpdates($updates);
		$this->groups_updated = false;
		return true;
	}
	
	function collectUpdates() {
		$updates = array('ins' => array(), 'upd' => array(), 'del' => array());
		$this->collectGroupUpdates($updates);		
		$this->collectRowUpdates($updates, 'lines', $this->line_model);
		$this->collectRowUpdates($updates, 'comments', $this->comment_model);
		$this->collectRowUpdates($updates, 'adjusts', $this->adj_model);
		$this->fixTempIds($updates['upd']);
		$this->fixTempIds($updates['ins']);
		return $updates;
	}
	
	function collectRowUpdates(&$target, $attrib, $model) {
		$orig_rows =& $this->orig_rows[$model];
        global $timedate;
		foreach($this->groups as &$grp) {
			if(empty($grp[$attrib])) continue;
			foreach($grp[$attrib] as $idx => $row) {
				if(! empty($row['is_comment']) && $model != $this->comment_model)
					continue;
                if(isset($row['event_session_date']) && empty($row['event_session_date']))
                    $row['event_session_date'] = null;
				$row[$this->rel_id_field] = $this->record_id;
				$row['line_group_id'] = $grp['id'];
				if($this->makeRealId($row['id'])) {
					$cls = 'ins';
				}
				else if(isset($orig_rows[$row['id']])) {
					$cls = 'upd';
				} else
					continue;
				if(! empty($target[$cls][$model][$row['id']]))
					continue; // validation error - line is repeated:  FIXME - check in validate
				$this->addBaseAmounts($row, $model);
				$target[$cls][$model][$row['id']] = $row;
			}
		}
		if(isset($orig_rows)) {
			foreach($orig_rows as $k => $row) {
				if(empty($target['upd'][$model][$k]))
					$target['del'][$model][$k] = $row;
			}
		}
		if(isset($this->cleanup_delete[$model])) {
			array_extend($target['del'][$model], $this->cleanup_delete[$model]);
		}
	}
	
	function collectGroupUpdates(&$target) {
		$model = $this->group_model;
		$orig_rows =& $this->orig_rows[$model];
		foreach($this->groups as &$grp) {
			$grp['parent_id'] = $this->record_id;
			$grp['comments'] = array();
			if(! empty($grp['lines'])) {
				foreach($grp['lines'] as $idx => $l) {
					if(! empty($l['is_comment'])) {
						$grp['comments'][$idx] = $l;
					}
				}
			}
			if($this->makeRealId($grp['id'])) {
				$cls = 'ins';
			}
			else if(isset($orig_rows[$grp['id']])) {
				$cls = 'upd';
			} else
				continue;
			$g_copy = $grp;
			unset($g_copy['lines']);
			unset($g_copy['comments']);
			unset($g_copy['adjusts']);
			$this->addBaseAmounts($g_copy, $model);
			$target[$cls][$model][$grp['id']] = $g_copy;
		}
		if(isset($orig_rows)) {
			foreach($orig_rows as $k => $row) {
				if(empty($target['upd'][$model][$k]))
					$target['del'][$model][$k] = $row;
			}
		}
		if(isset($this->cleanup_delete[$model])) {
			array_extend($target['del'][$model], $this->cleanup_delete[$model]);
		}
	}
	
	function fixTempIds(&$updates) {
		if(isset($updates[$this->line_model])) {
			foreach($updates[$this->line_model] as &$line) {
				$this->makeRealId($line['pricing_adjust_id']);
				$this->makeRealId($line['parent_id']);
			}
		}
		if(isset($updates[$this->comment_model])) {
			foreach($updates[$this->comment_model] as &$line) {
				$this->makeRealId($line['parent_id']);
			}
		}
		if(isset($updates[$this->adj_model])) {
			foreach($updates[$this->adj_model] as &$adj) {
				$this->makeRealId($adj['line_id']);
			}
		}
	}
	
	function commitUpdates($updates) {
		$blanks = array();
		$rowupd = array();
		foreach($updates['ins'] as $model => $rows) {
			$blank = ListQuery::quick_fetch_blank($model);
			$upd = new RowUpdate($model);
			foreach($rows as $row) {
				$upd->setOriginal($blank);
				$upd->set($row);
				$upd->insertRow(true);
			}
		}
		foreach($updates['upd'] as $model => $rows) {
			$upd = new RowUpdate($model);
			foreach($rows as $id => $row) {
				$upd->setOriginal($this->orig_rows[$model][$id]);
				$upd->set($row);
				$upd->updateRow(true);
			}
		}
		foreach($updates['del'] as $model => $rows) {
			$upd = new RowUpdate($model);
			foreach($rows as $row) {
				$upd->setOriginal($row);
				$upd->markDeleted();
			}
		}
	}
	
	function retrieveGroups($parent_id=null) {
		if(! isset($parent_id)) {
			if($this->duplicate_of_id) $parent_id = $this->duplicate_of_id;
			else $parent_id = $this->record_id;
		}
		if(! $this->model_name || ! $parent_id)
			return false;

		$lq = new ListQuery($this->model_name, true, array('link_name' => 'line_groups_link', 'parent_key' => $parent_id));
		$lq->setOrderBy('position');
		$bm = $lq->getBaseModel();
		foreach($bm->getFieldDefinitions() as $fk => $fv) {
			if(array_get_default($fv, 'type') == 'ref' && $fv['name'] != 'parent')
				$lq->addField($fv);
		}
		$lq->formatted = true;
		$r = $lq->runQuery();
		$orig_groups = $r->getRows();
		$this->clearOrigRows();
		$this->orig_rows[$this->group_model] = $orig_groups;
		$this->groups = array();
		foreach($orig_groups as $idx => $g) {
			$g['lines'] = $g['adjusts'] = array();
			$this->groups[$idx] = $g;
		}
		$this->totals = array();
		$this->retrieveLines($parent_id) && $this->retrieveAdjustments($parent_id);
		if($this->duplicate_of_id) {
			$this->makeDuplicateGroups($this->groups);
			$this->groups_updated = true;
		}
		$this->groups_updated = false;
		
		return true;
	}
	
	function makeDuplicateGroups(array &$groups) {
		$this->clearOrigRows();
		$ret = array();
		foreach($groups as $grp) {
			$this->makeTempId($grp['id']);
			$grp['parent_id'] = null;
			$lines = array();
			foreach($grp['lines'] as $line) {
				$this->makeTempId($line['id']);
				$this->makeTempId($line['line_group_id']);
				$this->makeTempId($line['pricing_adjust_id']);
				$this->makeTempId($line['parent_id']);
				$line[$this->rel_id_field] = null;
				$lines[$line['id']] = $line;
			}
			$grp['lines'] = $lines;
			$adjusts = array();
			foreach($grp['adjusts'] as &$line) {
				$this->makeTempId($line['id']);
				$this->makeTempId($line['line_group_id']);
				$this->makeTempId($line['line_id']);
				$line[$this->rel_id_field] = null;
				$adjusts[$line['id']] = $line;
			}
			$grp['adjusts'] = $adjusts;
			$ret[$grp['id']] = $grp;
		}
		$groups = $ret;
	}
	
	function makeRealId(&$id) {
		if(empty($id)) return;
		if(strpos($id, '~') !== false) {
			if(! isset($this->id_map[$id]))
				$this->id_map[$id] = create_guid();
			$id = $this->id_map[$id];
			return true;
		}
	}
	
	function makeTempId(&$id) {
		if(empty($id)) return;
		if(! isset($this->temp_id_map[$id]))
			$this->temp_id_map[$id] = 'temp~'.($this->uniq_row_idx++);
		$id = $this->temp_id_map[$id];
	}
	
	function retrieveLines($parent_id, $limit_groups=null) {
		if(! $this->model_name || ! $parent_id)
			return false;
		$this->orig_rows[$this->line_model] = array();
		$this->orig_rows[$this->comment_model] = array();
		
		$lq1 = new ListQuery($this->model_name, null, array('link_name' => 'lines_link', 'parent_key' => $parent_id, 'use_key_index' => false));
		if($limit_groups)
			$lq1->addSimpleFilter('line_group_id', $limit_groups);
		$lq1->setOrderBy('position');
		$lines = $lq1->runQuery()->getRows();

		// FIXME - test union query for speed
		$lq2 = new ListQuery($this->model_name, null, array('link_name' => 'comments_link', 'parent_key' => $parent_id, 'use_key_index' => false));
		if($limit_groups)
			$lq2->addSimpleFilter('line_group_id', $limit_groups);
		$lq2->setOrderBy('position');		
		$comments = $lq2->runQuery()->getRows();
		foreach($comments as &$c)
			$c['is_comment'] = 1;
		
		$j = 0;
		$c_lines = count($lines); $c_comments = count($comments);
		$all = array();
		$depths = array();
		$nest_under = array();
		for($i = 0; $i < $c_lines || $j < $c_comments;) {
			unset($row);
			if($i >= $c_lines || ($j < $c_comments && $comments[$j]['position'] < $lines[$i]['position']))
				$row =& $comments[$j++];
			else
				$row =& $lines[$i++];
			if(empty($row['line_group_id']) || empty($this->groups[$row['line_group_id']])) {
				if(! empty($row['is_comment']))
					$this->cleanup_delete[$this->comment_model][$row['id']] = $row;
				else
					$this->cleanup_delete[$this->line_model][$row['id']] = $row;
				continue;
			}
			if($row['parent_id'] && isset($depths[$row['parent_id']]) && $nest_under[$row['parent_id']])
				$row['depth'] = $depths[$row['parent_id']] + 1;
			else
				$row['depth'] = 0;
			$depths[$row['id']] = $row['depth'];
			$nest_under[$row['id']] = ! empty($row['sum_of_components']);
			if(! empty($row['is_comment'])) {
				$this->orig_rows[$this->comment_model][$row['id']] = $row;
			} else {
				$this->orig_rows[$this->line_model][$row['id']] = $row;
			}
			$this->groups[$row['line_group_id']]['lines'][$row['id']] = $row;
		}
		
		return true;
	}
	
	function retrieveAdjustments($parent_id, $limit_groups=null) {
		if(! $this->model_name || ! $parent_id)
			return false;
		$this->orig_rows[$this->adj_model] = array();

		$adjusts = array();
        $all = array();
		$lq1 = new ListQuery($this->model_name, null, array('link_name' => 'adjustments_link', 'parent_key' => $parent_id));
		if($limit_groups)
			$lq1->addSimpleFilter('line_group_id', $limit_groups);
		$lq1->setOrderBy('position');
		$adjusts = $lq1->runQuery()->getRows();

		foreach($adjusts as &$row) {
			if(empty($row['line_group_id'])) {
				if(! empty($this->totals['id']))
					$this->cleanup_delete[$this->adj_model][$row['id']] = $row;
				else
					$this->orig_totals = $this->totals = $row;
				continue;
			} else if(empty($this->groups[$row['line_group_id']])) {
				$this->cleanup_delete[$this->adj_model][$row['id']] = $row;
				continue;
			}
			$this->orig_rows[$this->adj_model][$row['id']] = $row;
			$this->groups[$row['line_group_id']]['adjusts'][$row['id']] = $row;
		}
		
		$this->updateGrandTotals();
		
		return true;
	}
	
	function replaceGroups(array $new_groups, $recalc=true, $date=null, $update_taxes=true) {
		if(! $this->model_name || ! $this->group_model) return false;
		$this->loadCurrency();
		
		$bm = new ModelDef($this->group_model);
		$grp_fields = $bm->getColumnFields();
		foreach($bm->getFieldDefinitions() as $fk => $fv) {
			if(array_get_default($fv, 'type') == 'ref' && $fv['name'] != 'parent')
				$grp_fields[] = $fk;
		}
		
		$deleted = array();
		$lastpos = 0;
		$groups =& $this->groups;
		$orig_groups =& $this->orig_rows[$this->group_model];
		$groups = array();
		$blank = ListQuery::quick_fetch_blank($this->group_model);
		foreach(self::$total_fields as $c) {
			$blank->row[$c] = 0.0;
		}
		
		foreach($new_groups as $id => $newg) {
			if(isset($newg['id'])) {
				$id = $newg['id'];
				unset($newg['id']);
			}
			if(! empty($newg['deleted']))
				continue;
			if(isset($orig_groups[$id])) {
				$groups[$id] = $orig_groups[$id];
			} else {
				$groups[$id] = $blank->row;
				$groups[$id]['id'] = $id;
			}
			$grp =& $groups[$id];
			foreach($grp_fields as $f) {
				if($f == 'date_entered' || $f == 'date_modified')
					continue;
				if(isset($newg[$f]))
					$grp[$f] = $newg[$f];
			}
			
			if(isset($newg['lines']) && is_array($newg['lines'])) {
				if(isset($newg['lines_order']))
					self::reorder_lines($newg['lines'], $newg['lines_order']);
				$this->updateLines($id, $newg['lines'], true);
			}
			if(isset($newg['adjusts']) && is_array($newg['adjusts'])) {
				if(isset($newg['adjusts_order']))
					self::reorder_lines($newg['adjusts'], $newg['adjusts_order']);
				$this->updateAdjustments($id, $newg['adjusts']);
			}
			
			if($recalc)
				$this->updateGroupTotals($id, $date, $update_taxes);

			$pos = $lastpos + 1;
			if(empty($grp['position']) || $grp['position'] < $pos)
				$grp['position'] = $pos;
			$lastpos = $grp['position'];
		}
		
		if($recalc)
			$this->updateGrandTotals();
		$this->groups_updated = true;
	}
	
	
	function replaceLines($group_id, array $lines) {
		return $this->updateLines($group_id, $lines, true);
	}

	function updateLines($group_id, array $new_lines, $replace=false) {
		if(! isset($this->groups[$group_id]))
			return false;
		$this->mergeRows($this->groups[$group_id]['lines'], $new_lines, $replace);
		$this->groups_updated = true;
		return true;
	}
	
	
	function replaceAdjustments($group_id, array $adjusts) {
		return $this->updateAdjustments($group_id, $adjusts, true);
	}
	
	function updateAdjustments($group_id, array $new_adjusts, $replace=false) {
		if(! isset($this->groups[$group_id]))
			return false;
		$lines =& $this->groups[$group_id]['lines'];
		$adjusts =& $this->groups[$group_id]['adjusts'];
		$this->mergeRows($adjusts, $new_adjusts, $replace);
		$this->groups_updated = true;
		
		// enforce ordering
		$clscount = array();
		$lastpos = 0;
		$idxs = array_keys($adjusts);
		if(! count($idxs))
			return true;
		$line_ids = array();
		
		foreach($lines as $idx => $line)
			if(! empty($line['id']) && empty($line['deleted']) && empty($line['is_comment']))
				$line_ids[$line['id']] = $idx;
		foreach($idxs as $idx) {
			$adj =& $adjusts[$idx];
			if(! empty($adj['line_id'])) {
				if(! isset($line_ids[$adj['line_id']])) {
					$adj['deleted'] = '1';
				}
			}
			if(! empty($adj['deleted']) || ! array_key_exists('type', $adj)) {
				unset($adjusts[$idx]);
				continue;
			}
			$cls = self::_adj_class($adj);
			@$clscount[$cls] ++;
		}
		if(! count($clscount))
			return;
		$maxcls = max(array_keys($clscount));
		$clspos = array();
		$pos = 0;
		for($cls = 0; $cls <= $maxcls; $cls++) {
			$clspos[$cls] = $pos;
			$pos += @$clscount[$cls];
		}
		foreach($adjusts as $idx => &$adj) {
			$cls = self::_adj_class($adj);
			$pos = $clspos[$cls];
			$clspos[$cls] ++;
			$adj['position'] = $pos;
		}
		
		return true;
	}


	static function reorder_lines(array &$lines, $order) {
		if(is_array($order) && array_keys($lines) != $order) {
			$nlines = array();
			foreach($order as $o) {
				if(isset($lines[$o]))
					$nlines[$o] = $lines[$o];
			}
			$lines = $nlines;
		}
	}	
	
	static function _adj_class(&$adj) {
		switch($adj['type']) {
			case 'StandardDiscount':
			case 'StdPercentDiscount':
			case 'StdFixedDiscount':
				return 5;
			case 'TaxedShipping':
				return 10;
			case 'CompoundedTax':
			case 'StandardTax':
				return 15;
			case 'UntaxedShipping':
				return 20;
			default:
				return 25;
		}
	}

	static function update_row(&$old_row, &$new_row) {
		foreach($new_row as $k => $v) {
			if($k == 'id' || $k == 'date_entered' || $k == 'date_modified' || $k == 'depth')
				continue;
			if(is_array($v) || ($k == 'related_type' && ! empty($new_row['is_comment'])))
				continue;
			$old_row[$k] = $v;
		}
	}
	
	function mergeRows(&$old_rows, $new_rows, $delete_rest=false, $reposition=true) {
		$ret = array();
		
		if(! is_array($old_rows))  $old_rows = array();
		$row_idx = array();
		$last_idx = 0;
		foreach(array_keys($old_rows) as $idx) {
			$rid = $old_rows[$idx]['id'];
			$row_idx[$rid] = $idx;
			$last_idx ++;
		}
		$pend_add = array();
		$in_new_rows = array();
		$depth_last_id = array();
		$last_row_id = '';
		$lastpos = -1;
		
		foreach(array_keys($new_rows) as $idx) {
			$new_row =& $new_rows[$idx];
			if(! isset($new_row['id']))
				$new_row['id'] = $idx;
			$rid = $this->normalizeRow($new_row);
			$depth = $new_row['depth'];
		
			if(isset($row_idx[$rid])) {
				$old_idx = $row_idx[$rid];
				self::update_row($old_rows[$old_idx], $new_row);
                if($reposition) {
					if($old_rows[$old_idx]['position'] <= $lastpos) {
						$lastpos ++;
						$old_rows[$old_idx]['position'] = $lastpos;
					}
					else
						$lastpos = $old_rows[$old_idx]['position'];
				}
			} else {
				if(empty($new_row['parent_id']) && ! empty($depth_last_id[$depth-1]))
					$new_row['parent_id'] = $depth_last_id[$depth-1];
				$pend_add[$last_row_id][] = $new_row;
			}
			
			$in_new_rows[$rid] = true;
			$last_row_id = $depth_last_id[$depth] = $rid;
		}
		
		// rebuild old_rows if necessary
		if(count($pend_add) || ($delete_rest && count($old_rows) != count($in_new_rows))) {
			$lastpos = -1;
			$rows = $old_rows;
			$old_rows = array();
			$delete_pids = array();
			$last_row_id = '';
			foreach($rows as $row) {
				$rid = $row['id'];
				while(! empty($pend_add[$last_row_id])) {
					$ins_id = $last_row_id;
					foreach($pend_add[$last_row_id] as $ins_row) {
						$lastpos ++;
						$ins_row['position'] = $lastpos;
						$ins_id = $ins_row['id'];
						$old_rows[$ins_id] = $ins_row;
					}
					unset($pend_add[$last_row_id]);
					$last_row_id = $ins_id;
				}
				if(empty($in_new_rows[$rid]) ||
					(! empty($row['parent_id']) && ! empty($delete_pids[$row['parent_id']]))) {
						$delete_pids[$rid] = true;
				}
				else {
                    if($reposition && $row['position'] <= $lastpos) {
						$lastpos ++;
						$row['position'] = $lastpos;
					}
					else
						$lastpos = $row['position'];
					$old_rows[$rid] = $row;
				}
				$last_row_id = $rid;
			}
			// add any rows missed earlier
			foreach($pend_add as $ins_rows) {
				foreach($ins_rows as $ins_row) {
					$lastpos += 5;
					$ins_row['position'] = $lastpos;
					$ins_id = $ins_row['id'];
					$old_rows[$ins_id] = $ins_row;
				}
			}
		}
	}


    function normalizeRow(&$new_row) {
        /*if (isset($new_row['pricing_adjust_id']) && strpos($new_row['pricing_adjust_id'], '~') !== false) {
            $padjid = $new_row['pricing_adjust_id'];
            $new_row['pricing_adjust_id'] = create_guid();
            if (isset($new_row['adjusts'][$padjid])) {
                $adj = $new_row['adjusts'][$padjid];
                unset($new_row['adjusts'][$padjid]);
                $adj['id'] = $new_row['pricing_adjust_id'];
                $new_row['adjusts'][$adj['id']] = $adj;
            }
        }*/
        //$new_row[$this->rel_id_field] = $this->parent_id;
        if(empty($new_row['depth']))
            $new_row['depth'] = 0;
        if(empty($new_row['id']))
            $new_row['id'] = 'temp~'.($this->uniq_row_idx++);
        return $new_row['id'];
    }
    
    
	function updateGroupTotals($group_id, $date = null, $update_taxes = true) {
		static $amounts = array('cost_price', 'list_price', 'unit_price', 'std_unit_price', 'discount');
		if(empty($this->groups[$group_id])) return false;

		$format = sprintf("%%0.%df", $this->currency_decimals);
		
		$update_sums = array();
		$subtotal = 0.0;
		$cost = 0.0;
		$tally = array();
		$all_taxcodes = array();
		$discount_taxes = array();
		$old_taxes = $taxes = array();
		$rates = array();
		$added_rates = array();
		$adjusts =& $this->groups[$group_id]['adjusts'];
		$lines =& $this->groups[$group_id]['lines'];
		$editableRates = [];

		// FIXME - use dbcache
		require_once('modules/TaxCodes/TaxCode.php');
		$seedcode = new TaxCode();
		require_once('modules/TaxRates/TaxRate.php');
		$seedrate = new TaxRate;
		
		$allRates = array();
		foreach ((array)$seedrate->get_full_list() as $r) {
			$rate = array('id' => $r->id, 'name' => $r->name, 'rate' => $r->rate, 'compounding' => $r->compounding);
			$allRates[$r->id] = $rate;
		}

		foreach($lines as $idx => &$line) {
			if(! empty($line['is_comment']) || ! empty($line['deleted']))
				continue;
			if(! empty($line['sum_of_components'])) {
				$update_sums[$line['id']] = array('idx' => $idx);
				foreach($amounts as $f)
					$update_sums[$line['id']][$f] = 0.0;
				$line['ext_quantity'] = $line['quantity'];
				continue;
			}
			if(! empty($line['parent_id']) && ! empty($update_sums[$pid = $line['parent_id']])) {
				foreach($amounts as $f) {
                    if (isset($line[$f]))
						$update_sums[$pid][$f] += sprintf($format, $line[$f]) * sprintf("%0.2f", $line['quantity']);
                }
				$line['ext_quantity'] = sprintf("%0.2f", $line['quantity']) * sprintf($format ,$lines[$update_sums[$pid]['idx']]['ext_quantity']);
			}
			else
				$line['ext_quantity'] = $line['quantity'];

			$line['ext_price'] = currency_round(sprintf($format, $line['unit_price']) * sprintf($format, $line['ext_quantity']), $this->currency_decimals);
			$line['net_price'] = $line['ext_price'];
			$line['discount'] = 0.0;
			
			if(empty($line['sum_of_components'])) {
				$tally[] = array(
					'id' => $line['id'],
					'unadj_amount' => $line['ext_price'],
					'amount' => $line['ext_price'],
					'taxcode' => array_get_default($line, 'tax_class_id'),
				);
				$subtotal += $line['ext_price'];
				if(! empty($line['tax_class_id']) && $line['tax_class_id'] != '-99')
					$all_taxcodes[$line['tax_class_id']] = 1;
				if(isset($line['cost_price']))
					$cost += currency_round($line['ext_quantity'] * $line['cost_price'], $this->currency_decimals);
			}
			foreach ($adjusts as $adj) {
				if (array_get_default($adj, 'line_id') != $line['id']) continue;
				if($adj['type'] == 'StandardTax' || $adj['type'] == 'CompoundedTax') {
					if (isset($allRates[$adj['related_id']])) {
						$r = $allRates[$adj['related_id']];
						$type = $r['compounding'] != 'compounded' ? 'StandardTax' : 'CompoundedTax';
						$taxuid = $type  . '~' .  $r['id'] . '~' . $r['rate'];
						$r['editable'] = 1;
						$r['line_id'] = $line['id'];
						$r['uid'] = $taxuid;
						$rates[] = $r;

						$added_rates[$taxuid] = $adj['position'];
					}
				}
			}
		}

		foreach($update_sums as $upd) {
			$line =& $lines[$upd['idx']];
			foreach($amounts as $f)
				$line[$f] = currency_round($upd[$f], $this->currency_decimals);
			$line['ext_price'] = currency_round($line['ext_quantity'] * $line['unit_price'], $this->currency_decimals);
			$line['net_price'] = $line['ext_price'];
		}
		$subtotal = $total = currency_round($subtotal, $this->currency_decimals);

		if ($update_taxes) {
			foreach ($all_taxcodes as $code_id => $_z) {
				$r = array_reverse($seedcode->get_tax_rates($code_id, false, $date));
				foreach ($r as $rr) {
					$rr['code_id'] = $code_id;
					$type = $rr['compounding'] ? 'CompoundedTax' : 'StandardTax';
					$rr['uid'] = $type .'~'. $rr['id'] .'~'. $rr['rate'];

					$exists = false;
					foreach ($rates as $_r) {
						if ($_r['id'] == $rr['id']) {
							$exists = true;
							break;
						}
					}
					if (!$exists) {
						$rates[] = $rr;
						$idx = array_search($rr['id'], array_column($adjusts, 'related_id'), true);
						$positions = array_column($adjusts, 'position');
						$added_rates[$rr['uid']] = isset($positions[$idx]) ? $positions[$idx] : 1;
					}
				}
			}
		}

		if(! is_array($adjusts))
			$adjusts = array();

		foreach($adjusts as $idx => $adj) {
			if(! empty($adj['deleted'])) continue;
			if(! empty($adj['line_id'])) {
				if (array_get_default($adj, 'related_type') == 'TaxRates') {
					$total += $adj['amount'];
				}
				continue;
			}
			if ($adj['type'] == 'TaxedShipping') {
				if (empty($adj['tax_class_id'])) {
					$adj['tax_class_id'] = 'all';
				}
				if ($adj['tax_class_id'] != 'all' && $adj['tax_class_id'] != '-99') {
					$all_taxcodes[$adj['tax_class_id']] = 1;
				}
			}
			if($adj['type'] == 'StandardDiscount' || $adj['type'] == 'StdPercentDiscount') {
				$adj['amount'] = 0.0;
				foreach($tally as $tdx => $tally_row) {
					$disc = currency_round($tally_row['unadj_amount'] * $adj['rate'] / 100, $this->currency_decimals);
					$tally[$tdx]['amount'] -= $disc;
					if(! empty($tally[$tdx]['id'])) {
						$lref = &$lines[$tally[$tdx]['id']];
						$lref['discount'] += $disc;
						$lref['net_price'] = $lref['ext_price'] - $lref['discount'];
						if(! empty($lref['parent_id']) && isset($lines[$lref['parent_id']])) {
							$lines[$lref['parent_id']]['discount'] += $lref['discount'];
							$lines[$lref['parent_id']]['net_price'] -= $lref['discount'];
						}
						unset($lref);
					}
					$adj['amount'] -= $disc;
				}
				$adjusts[$idx]['amount'] = -$adj['amount'];
				if (!empty($adj['tax_class_id']) && $adj['tax_class_id'] != '-99') {
					$r = $seedcode->get_tax_rates($adj['tax_class_id'], false, $date);
					foreach ($r as $rate) {
						$discount_taxes[$rate['id']] = 
							array_get_default($discount_taxes, $rate['id'], 0.0) + $adj['amount'] * $rate['rate'] / 100.0;
					}
				}
			}
			else if($adj['type'] == 'StdFixedDiscount') {
				$disc = $adj['amount'];
				if (count($tally) > 0) $tally[0]['amount'] -= $disc;//because we need subtract fixed discount one time
				$adj['amount'] = -$adj['amount'];
			}
			else if($adj['type'] == 'TaxedShipping') {
				$tally[] = array('amount' => $adj['amount'], 'unadj_amount' =>$adj['amount'],  'taxcode' => $adj['tax_class_id'], 'is_shipping' => true);
			}
			else if($adj['type'] == 'StandardTax' || $adj['type'] == 'CompoundedTax') {
				if ($update_taxes) {
					$taxuid = $adj['type'] .'~'. $adj['related_id'] .'~'. $adj['rate'];

					//Only for Invoices: if extra tax make it editable
					//because invoice_adjustment doesn't have `editable` field
					if (! isset($adj['editable'])) {
						if (! isset($added_rates[$taxuid]) || $added_rates[$taxuid] != $adj['position']) {
							$adj['editable'] = 1;
						}
					}

					if (empty($adj['editable'])) {
						if (empty($old_taxes[$taxuid])) {
							$old_taxes[$taxuid] = $adj;
						}
					} else {
						$adj['amount'] = 0.0;
						if (empty($editableRates[$taxuid])) {
							$editableRates[$taxuid] = true;
							$existing = -1;
							foreach ($rates as $_i => $_r) {
								if ($_r['id'] == $adj['related_id']) {
									$existing = $_i;
								}
							}
							if (isset($allRates[$adj['related_id']])) {
								$r = $allRates[$adj['related_id']];
								$r['editable'] = 1;
								$r['uid'] = $taxuid;
								$rates[] = $r;
								if ($existing >= 0) {
									array_splice($rates, $existing, 1);
								}
							}
							$old_taxes[$taxuid] = $adj;
						}
					}
					unset($adjusts[$idx]);
					continue; // do not add yet
				}
			}
			$total += $adj['amount'];
		}

		if ($update_taxes) {
			foreach($rates as $rate) {
				$type = $rate['compounding'] ? 'CompoundedTax' : 'StandardTax';
				$taxuid = array_get_default($rate, 'uid', $type .'~'. $rate['id'] .'~'. $rate['rate']);
				if(! isset($taxes[$taxuid])) {
					$adj = array(
						'type' => $type,
						'name' => $rate['name'],
						'related_type' => 'TaxRates',
						'related_id' => $rate['id'],
						'rate' => $rate['rate'],
						'amount' => array_get_default($discount_taxes, $rate['id'], 0.0),
					);
					if(! empty($rate['editable']))
						$adj['amount'] = 0.0;
					//$adj[$this->rel_id_field] = $this->record_id;
					if(isset($old_taxes[$taxuid])) {
						$adj['id'] = $old_taxes[$taxuid]['id'];
						$adj['position'] = $old_taxes[$taxuid]['position'];
						if(! empty($old_taxes[$taxuid]['editable'])) $adj['editable'] = 1;
						unset($old_taxes[$taxuid]);
					}
					else {
						$adj['id'] = 'temp~'.($this->uniq_row_idx++);
					}
					$taxes[$taxuid] = $adj;
				}
				$adjamt = $taxes[$taxuid]['amount'];
				foreach($tally as $tdx => $tally_row) {
					if (empty($tally_row['taxcode']) || $tally_row['taxcode'] == "-99") continue;
					if ($tally_row['taxcode'] != 'all') {
						$code = $seedcode->retrieve($tally_row['taxcode']);
						if (!$code) continue;
						$code_scheme = $code->taxation_scheme;

						$applicable = false;
						switch ($code_scheme) {
							case 'list':
								$_rates = array_reverse($seedcode->get_tax_rates($tally_row['taxcode'], false, $date));
								foreach ($_rates as $_r) {
									if ($_r['id'] == $rate['id']) {
										$applicable = true;
										break;
									}
								}
								$applicable = $applicable || ( !empty($rate['editable']) && empty($tally_row['is_shipping']));
								break;
							case 'tax':
								$applicable = !empty($rate['editable']);
								break;
							case 'nontax':
								break;

						}
						if (!$applicable) continue;
					}
				
					$prevtax = array_get_default($tally_row, 'tax', 0.0);
					if (! $this->getField('discount_before_taxes')) {
						$base = $tally_row['amount'];
					} else {
						$base = $tally_row['unadj_amount'];
					}
					if($rate['compounding'])
						$base += $prevtax;
					$amt = $base * $rate['rate'] / 100;
					$tally[$tdx]['tax'] = $prevtax + $amt;
					$adjamt += $amt;
				}
				$taxes[$taxuid]['amount'] = currency_round($adjamt, $this->currency_decimals);
			}

			$seedcode->cleanup();
			$pos = -1;
			foreach($adjusts as &$a)
				$pos = max($pos, $a['position']);
			foreach($taxes as $taxuid => $adj) {
				$adjusts[$adj['id']] = $adj;
				$total += $adj['amount'];
			}
		}
		
		array_extend($this->groups[$group_id], array(
			'subtotal' => $subtotal,
			'total' => currency_round($total, $this->currency_decimals),
			'cost' =>  currency_round($cost, $this->currency_decimals),
		));
		$this->groups_updated = true;
		
		/*foreach($lines as $idx => &$line) {
			pr2($line['unit_price']."\n".$line['ext_price']."\n".$line['net_price'], $line['ext_quantity'] . ' ' . $line['name']);
		}*/
	}
	
	
	function updateGrandTotals() {
		$totals =& $this->totals;
		if(! isset($totals))
			$totals = array();
		foreach(self::$total_fields as $t)
			$totals[$t] = 0.0;
		$totals['taxes'] = array();
		$totals['shipping_taxed'] = true;

		foreach($this->groups as $idx => &$group) {
			if(! empty($group['deleted']))
				continue;

			$totals['total'] += $group['total'];

			if ($this->isMilestoneInvoicesGroup($group))
				continue;

			$totals['subtotal'] += $group['subtotal'];
			if(isset($group['cost'])) // only defined for Quotes
				$totals['cost'] += $group['cost'];
			if($group['adjusts'])
			foreach($group['adjusts'] as $adj) {
				if($adj['type'] == 'TaxedShipping' || $adj['type'] == 'UntaxedShipping') {
					$totals['shipping'] += $adj['amount'];
					if($adj['type'] == 'UntaxedShipping')
						$totals['shipping_taxed'] = false;
				}
				else if($adj['type'] == 'StandardTax' || $adj['type'] == 'CompoundedTax') {
					$tname = $adj['name'] . '(' . format_number($adj['rate'], -1) . '%)';
					if (!isset($totals['taxes'][$tname]))
						$totals['taxes'][$tname] = array(
							'name' => $adj['name'],
							'rate' => $adj['rate'],
							'amount' => 0.0,
						);
					$totals['taxes'][$tname]['amount'] += $adj['amount'];
					$totals['tax'] += $adj['amount'];
				}
				else if(empty($adj['line_id']) && ($adj['type'] == 'StandardDiscount' || $adj['type'] == 'StdPercentDiscount' || $adj['type'] == 'StdFixedDiscount'))
					$totals['discount'] += $adj['amount'];
			}
		}

		foreach(self::$total_fields as $t) {
			$totals[$t] = currency_round($totals[$t], $this->currency_decimals);
		}

		$this->set('subtotal', $totals['subtotal']);
		$this->set('amount', $totals['total']);
		$this->set('cost', $cost = $totals['cost']);
		$std = $totals['subtotal'] - $totals['discount'];
		$pretax = $std;
		if($totals['shipping_taxed'])
			$pretax += $totals['shipping'];

		$this->set('pretax', $pretax);
		$this->set('gross_profit', $gp = $std - $cost);
		if($this->getModelName() == 'SalesOrder');
			$this->set('gross_profit_so', $gp = $std - $cost);
		$this->set('markup', $cost != 0 ? currency_round($gp * 100.0 / $cost, 2) : 0);
		$this->set('margin', $std != 0 ? currency_round($gp * 100.0 / $std, 2) : 0);
	}

	function customUpdateResult(RowResult &$row_result) {
		$row_result->assign('line_items', $this->getGroups());
	}

	function isMilestoneInvoicesGroup($group) {
		require_once('modules/Invoice/FinalMilestone.php');
		return FinalMilestone::is_milestone_invoices_group($group);
	}

	function fetchPayments() {
		$ret = array();
		$pk = $this->getPrimaryKeyValue();
		if($pk && $this->model->getLinkDefinition('payments')) {
			$lq = new ListQuery($this->model, null, array('link_name' => 'payments', 'parent_key' => $pk));
			$ret = $lq->fetchAllRows();
		}
		return $ret;
	}
	
	function addBaseAmounts(array &$row, $model, $fs = array()) {
		if(! $this->exchange_rate) return;
		
		if($model == $this->adj_model) {
			$fs[] = 'amount';
		} else if($model == $this->line_model) {
			$fs = array_merge($fs, array('cost_price', 'list_price', 'unit_price', 'std_unit_price', 'ext_price', 'net_price'));
		} else if($model == $this->group_model) {
			$fs = array_merge($fs, array('cost', 'subtotal', 'total'));
		}
		foreach($fs as $k) {
			if(array_key_exists($k, $row))
				$row[$k.'_usd'] = round((float) $row[$k] / $this->exchange_rate, 5);
		}
	}

	function extractQuantities(&$products, &$assemblies, $sign, $orig)
	{
		if ($orig) {
			$groups = array(
				array(
					'lines' => $this->orig_rows[$this->line_model],
				),
			);
		} else {
			$groups =& $this->groups;
		}
		foreach ($groups as $group) {
			if (!empty($group['lines'])) foreach ($group['lines'] as $row) {
				if($row['related_type'] == 'ProductCatalog') {
					$id = ! empty($row['related_id']) ? $row['related_id'] : $row['name'];
					if(! isset($products[$id])) $products[$id] = 0;
					$products[$id] += $row['ext_quantity'] * $sign;
				}
				else if($row['related_type'] == 'Assemblies') {
					$id = ! empty($row['related_id']) ? $row['related_id'] : $row['name'];
					if(! isset($assemblies[$id])) $assemblies[$id] = 0;
					$assemblies[$id] += $row['ext_quantity'] * $sign;
				}
			}
		}
	}

	function updateStock($onDeletion = false)
	{
		if (($model = $this->getModelName()) != 'Receiving' && $model != 'Shipping')
			return;
		global $db;

		$sign = ($model == 'Receiving') ? -1 : 1;
		$stageField = ($model == 'Receiving') ? 'receiving_stage' : 'shipping_stage';

		$stageOld = $this->getField($stageField, '', true);
		$stageNew = $this->getField($stageField, '', false);

		$stageOld = (strpos($stageOld, 'Shipped') !== false) || (strpos($stageOld, 'Received') !== false);
		$stageNew = (strpos($stageNew, 'Shipped') !== false) || (strpos($stageNew, 'Received') !== false);


		$widOld = $this->getField('warehouse_id', null, true);
		$widNew = $this->getField('warehouse_id', null, false);

		if (!$widNew || (!$this->new_record && !$widOld)) return;

		$products = $assemblies = array();
		if ($stageOld)
			$this->extractQuantities($products, $assemblies, $sign, true);
		$products2 = $assemblies2 = array();
		if (!$onDeletion && $stageNew)
			$this->extractQuantities($products2, $assemblies2, -$sign, false);
		
		$sum = $products;
		foreach ($products2 as $pid => $q) {
			$sum[$pid] = array_get_default($sum, $pid, 0) + $q;
		}

		foreach ($sum as $pid => $delta) {
			$res = ListQuery::quick_fetch('Product', $pid);
			if ($res) {
				if ( ($tracking = $res->getField('track_inventory')) != 'semiauto' && $tracking != 'automatic')
					continue;
				$p = RowUpdate::for_result($res);
				$whq = $p->getLinkQuery('warehouses', array('id', 'in_stock' => '~join.in_stock'));
				$rows = $whq->fetchAll();
				$stock = array();
				if($rows) $stock = $rows->rows;
				if ($widOld) {
					$q = array_get_default($products, $pid, 0);
					if($widNew == $widOld)
						$q += array_get_default($products2, $pid, 0);
					if ($q) {
						$qty = (isset($stock[$widOld]) ? $stock[$widOld]['in_stock'] : 0) + $q;
						$p->addUpdateLink('warehouses', $widOld,
							array($widOld => array(
								'name' => 'in_stock',
								'value' => $qty,
							))
						);
					}
				}
				if ($widNew && $widNew != $widOld) {
					$q = array_get_default($products2, $pid, 0);
					if ($q) {
						$qty = (isset($stock[$widNew]) ? $stock[$widNew]['in_stock'] : 0) + $q;
						$p->addUpdateLink('warehouses', $widNew,
							array($widNew => array(
								'name' => 'in_stock',
								'value' => $qty,
							))
						);
					}
				}
			}
		}
	}

}
?>
