<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/

if(class_exists('DateTimeZone'))
	TimeDate::$USE_DATETIMEZONE = true;
else
	require_once('include/timezone/timezones.php');

class TimeDate {
	var $dbDayFormat = 'Y-m-d';
	var $dbTimeFormat = 'H:i:s';
	var $supported_strings = array('a'=>'[ap]m', 'A'=>'[AP]M', 'd'=>'[0-9]{1,2}', 'h'=>'[0-9]{1,2}', 'H'=>'[0-9]{1,2}', 'i'=>'[0-9]{1,2}', 'm'=>'[0-9]{1,2}', 'Y'=>'[0-9]{4}', 's'=>'[0-9]{1,2}');
    static $USE_DATETIMEZONE = false;
    
    static function getTimeZoneInfo($tz) {
    	if(self::$USE_DATETIMEZONE) {
    		try {
    			$dtz = new DateTimeZone($tz);
    		} catch(Exception $e) {
    			return false;
    		}
    		return $dtz;
    	}
    	global $timezones;
		if(empty($timezones[$tz]))
			return false;
		$arr = $timezones[$tz];
		$arr['id'] = $tz;
		return $arr;
    }
        
    static function getDateTime($date=null) {
    	if(! isset($date))
    		$date = 'now';
    	if(is_int($date))
    		$date = '@'.$date;
    	try {
    		$dt = new DateTime($date);
    		return $dt;
    	} catch(Exception $e) {}
    	return false;
    }
    
    function guessTimeZone($gmtOffset=null) {
    	if(function_exists('date_default_timezone_get')) {
	    	$system_tz = @date_default_timezone_get();
	    	$system_offs = $this->getTimeZoneOffset($system_tz, true);
	    	if($system_tz == 'UTC')
	    		$system_tz = null;
	    } else {
			$system_tz = $system_offs = null;
	    }
    	if(! isset($gmtOffset)) {
    		$gmtOffset = date('Z');
    		if(date('I'))
    			$gmtOffset -= 3600;
    	}
		if($system_tz && $gmtOffset == $system_offs)
			return $system_tz;
		$defaultZones = array(
			'America/Toronto'=>1, 'America/Vancouver'=>1,
			'America/Winnipeg'=>1, 'America/Edmonton'=>1,
			'America/Juneau'=>1, 'America/Halifax'=>1,
			'Europe/Amsterdam'=>1, 'Europe/Athens'=>1, 'Europe/London'=>1,
			'Australia/Sydney'=>1, 'Australia/Perth'=>1, 'Asia/Tokyo'=>1,
		);
    	$selected_zone = '';
		$timezones = self::getTimeZones();
		foreach($timezones as $zone_name => $zone){
			if($zone['gmtOffset'] == $gmtOffset) {
				$selected_zone = $zone_name;
				if(! empty($defaultZones[$selected_zone]))
					break;
			}
		}
		return $selected_zone;
    }
    
    static function formatTimeZones($zones, $grouped=false) {
    	$ret = array();
    	foreach($zones as $id => $z) {
    		$gmtOffset = $z['gmtOffset'];
    		$dst = (!empty($z['dstOffset']) ? ', DST' : '');
			$positive = ($gmtOffset >= 0);
			if(! $positive)
				$gmtOffset = -$gmtOffset;
			$secs = $gmtOffset % 60;
			$gmtOffset = ($gmtOffset - $secs) / 60;
			$mins = $gmtOffset % 60;
			$gmtOffset = ($gmtOffset - $mins) / 60;
    		if($mins || $secs) {
    			$gmtOffset = sprintf('%d:%0.2d', $gmtOffset, $mins);
    			if($secs)
    				$gmtOffset .= sprintf(':%0.2d', $secs);
    		}
    		$gmtOffset = ($positive ? '+' : '-') . $gmtOffset;
    		$display = strtr($id, '_', ' ') . " (GMT {$gmtOffset}{$dst})";
    		if($grouped) {
	    		$parts = explode('/', $id);
				$partsCount = is_countable($parts) ? count($parts) : 0;
	    		$grp = ($partsCount == 1) ? 'America' : $parts[0]; // WET, CET, MET, EET
	    		$ret[$grp][$id] = $display;
	    	} else
	    		$ret[$id] = $display;
    	}
    	ksort($ret);
    	if($grouped) {
    		foreach($ret as &$zones)
    			ksort($zones);
    	}
		return $ret;
    }
    
    function formatTimeZoneId($zone_id) {
    	$zones = self::getTimeZones();
    	$info = self::getTimeZoneInfo($zone_id);
    	if(! $info)
    		return '';
    	if(is_object($info))
    		$id = $info->getName();
    	else
    		$id = $info['id'];
    	if(isset($zones[$id])) {
    		$arr = array($id => $zones[$id]);
    		$fmt = $this->formatTimeZones($arr);
    		return $fmt[$id];
    	}
    	return '';
    }
    
    static function getTimeZones($display=false, $grouped=false) {
    	static $zoneinfo = null;
    	if($zoneinfo)
	    	$ret = $zoneinfo;
	    else if(self::$USE_DATETIMEZONE) {
			$t = time();
			$all_zones = DateTimeZone::listIdentifiers();
    		$ret = array();
    		if($all_zones) {
    			foreach($all_zones as $zone_name) {
					if(strpos($zone_name, '/') === false)
						continue;
					if($zone_name == 'US/Pacific-New')
						continue;
					$z = new DateTimeZone($zone_name);
					$transition = $z->getTransitions($t, $t);
					if (isset($transition[0]))
						$transition = $transition[0];
					$row = array('abbr' => $transition['abbr'], 'gmtOffset' => $transition['offset']);
					if($transition['isdst']) {
						$row['dstOffset'] = 60;
						$row['gmtOffset'] -= 3600;
					}
					$ret[$zone_name] = $row;
				}
				$zoneinfo = $ret;
    		}
    	}
    	if(! $ret) {
			global $timezones;
			require_once('include/timezone/timezones.php');
    		foreach($timezones as $k => $zone) {
    			$zone['gmtOffset'] *= 60;
    			$ret[$k] = $zone;
			}
			$zoneinfo = $ret;
		}
		if($display)
			return self::formatTimeZones($ret, $grouped);
		return $ret;
    }
    
    function getTimeZoneOffset($timezone, $from_gmt=false) {
    	if(is_string($timezone))
    		$timezone = $this->getTimeZoneInfo($timezone);
    	if(! $timezone)
    		return false;
    	if(is_object($timezone)) {
			$offset = $timezone->getOffset(new DateTime('now', $timezone));
			if(! $from_gmt)
				$offset -= date('Z');
		} else {
			$offset = $this->adjustmentForUserTimeZone($timezone) * 60;
			if($from_gmt)
				$offset = date('Z') - $offset;
		}
		return $offset;
    }
    
	/**
	 * function getUserTimeZone()
	 * Returns the current users timezone info or another user;
	 * 
	 * $user user object for which you want to display, null for current user
	 * @return Array of timezone info 
	 */
	
	function getUserTimeZone($user = null, $reload=false){
		global $current_user;
		static $current_user_timezone;
		$usertimezone = array();
		if (! empty($user) && !is_object($user)) {
			$uid = $user;
			$user = new User;
			$user->retrieve($uid);
		}
		if(empty($user) || (!empty($user->id) && $user->id == $current_user->id)) {
			if(isset($current_user_timezone) && ! $reload)
				return $current_user_timezone; // current user, return saved timezone info
			$user = $current_user;
		}

		if(isset($user) && ! empty($user->id)) {
			if( ($timezone_name = $user->getPreference('timezone')) ) {
				$usertimezone = self::getTimeZoneInfo($timezone_name);
				if(! $usertimezone) {
					$GLOBALS['log']->fatal('TIMEZONE:NOT DEFINED-'. $timezone_name);
					$usertimezone = array();
				}
			}
			if($user->id == $current_user->id)
				$current_user_timezone = $usertimezone; // save current_user
		}
		
		return $usertimezone;
	}
	
	/**
	 * function adjustmentForUserTimeZone()
	 * this returns the adjustment for a user against the server time
	 * 
	 * $timezone pass in a timezone to adjust for 
	 * @return INT number of minutes to adjust a time by to get the appropriate time for the user
	 */
	function adjustmentForUserTimeZone($timezone_to_adjust = null){
		static $current_user_adjustment;
		static $tz_to_adjust;
		if(isset($current_user_adjustment) && $tz_to_adjust == $timezone_to_adjust){
			return $current_user_adjustment;
		} 
		
		$adjustment = 0;
		$tz_to_adjust = $timezone_to_adjust;
		
		if(empty($timezone_to_adjust)) {
			$timezone = $this->getUserTimeZone();
		}
		else { 
			$timezone = $timezone_to_adjust;
		}
		if(empty($timezone)) {
			return $adjustment;
		}
		
		if(is_object($timezone))
			$user_offset = $timezone->getOffset(new DateTime()) / 60;
		else {
			$user_in_ds = $this->inDST(time(), $timezone);
			$user_offset = $timezone['gmtOffset'] ;	
			//compensate for ds for user
			if($user_in_ds) {
				$user_offset += 60;
			}
		}
		
		$server_offset = date('Z')/60;
		//$server_in_ds = date('I');		
		
		//either both + or -
		$adjustment += $server_offset - $user_offset;
		if(empty($timezone_to_adjust)) $current_user_adjustment = $adjustment; // save current_user adj

		return $adjustment;
	}
	
	/**
	 * function getWeekDayName($indexOfDay)
	 * Returns a days name
	 *
	 * @param INT(WEEKDAY INDEX) $indexOfDay
	 * @return STRING representing the given weekday
	 */
	static function getWeekDayName($indexOfDay){
		static $dow = array ( 'sunday', 'monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday' );
		return $dow[$indexOfDay];
	}
	/**
	 * function getMonthName($indexMonth)
	 * Returns a Months Name
	 *
	 * @param INT(MONTH INDEX) $indexMonth
	 * @return STRING representation of the month
	 */
	static function getMonthName($indexMonth){
		static $months = array ( 'January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December' );
		return $months[$indexMonth];
	}
		
	/**
	 * function getDateFromRules($year, $startMonth, $startDate, $weekday, $startTime )
	 * Converts the rules for a timezones dst into a string representation of a date for the given year
	 *
	 * @param STRING(YEAR) $year
	 * @param INT(MONTH INDEX) $startMonth
	 * @param INT(DATE INDEX) $startDate
	 * @param INT(WEEKDAY INDEX) $weekday
	 * @param INT(START TIME IN SECONDS) $startTime
	 * @return unknown
	 */
	function _getDateFromRules($year, $startMonth, $startDate, $weekday, $startTime, &$ts){
		if($weekday < 0)return date( 'Y-m-d H:i:s', strtotime("$year-$startMonth-$startDate") + $startTime);
		$dayname = self::getWeekDayName($weekday);
		if($startDate > 0)$startMonth--;
		$month = self::getMonthName($startMonth);
		$startWeek = floor($startDate/7);
		//echo "$startWeek week $dayname - $month 1, $year<br>";

		// longreach - added - avoid timeout error on bad input
		if($year <= 1970) // bad date
			return date('Y-m-d H:i:s', $ts=0);

		return date( 'Y-m-d H:i:s', $ts = strtotime( "$startWeek week $dayname", strtotime( "$month 1, $year" ) ) + $startTime );
	
	}
	function getDateFromRules($year, $startMonth, $startDate, $weekday, $startTime){
		return $this->_getDateFromRules($year, $startMonth, $startDate, $weekday, $startTime, $ts);
	}
	
	/**
	 * 	function getDSTRange($year, $zone)
	 * 
	 * returns the start and end date for dst for a given timezone and year or false if that zone doesn't support dst
	 *
	 * @param STRING(Year e.g. 2005) $year
	 * @param ARRAY (TIME ZONE INFO) $zone
	 * @return ARRAY OF DATE REPRESENTING THE START AND END OF DST or FALSE if the zone doesn't support dst
	 */
	function getDSTRange($year, $zone){
		$range = array();
		if(is_object($zone)) {
			if(constant('PHP_VERSION_ID') > 50300) {
				$trans = $zone->getTransitions(strtotime("$year-01-01"), strtotime(($year+1)."-01-01"));
				$start = $end = null;
				foreach($trans as $t) {
					if(! empty($t['isdst']) && ! isset($start))
						$start = $t;
					if(isset($start) && empty($t['isdst']))
						$end = $t;
				}
				if(! $start || ! $end)
					return false;
				$ret = array(
					'start' => gmdate('Y-m-d H:i:s', $start['ts'] + $start['offset']),
					'start_ts' => $start['ts'],
					'end' => gmdate('Y-m-d H:i:s', $end['ts'] + $end['offset']),
					'end_ts' => $end['ts'],
				);
				return $ret;
			}

			global $timezones;
			require_once('include/timezone/timezones.php');
			if(isset($timezones[$zone->getName()]))
				return $this->getDSTRange($year, $timezones[$zone->getName()]);
			return false;
		}
		
		if(empty($zone['dstOffset'])){
			return false;
		}
	
		$range['start'] = $this->_getDateFromRules($year, $zone['dstMonth'], $zone['dstStartday'], $zone['dstWeekday'],  $zone['dstStartTimeSec'], $ts);
		$range['start_ts'] = $ts;
		$range['end'] = $this->_getDateFromRules($year, $zone['stdMonth'], $zone['stdStartday'], $zone['stdWeekday'],  $zone['stdStartTimeSec'], $ts);
		$range['end_ts'] = $ts;
		return $range;
	}
	
	// ajrw: modified to accept numeric dates (timestamps) as well, to avoid cycling between date/strtotime
	function inDST($date, $zone){
		if(is_object($zone)) {
			$dt = $this->getDateTime($date);
			$dt_next = clone($dt);
			$dt_next->modify('+6 month');
			return ($zone->getOffset($dt) > $zone->getOffset($dt_next));
		}
		
		if(is_numeric($date)) {
			$datestamp = $date;
			$dstRange = $this->getDSTRange(date('Y', $date), $zone);
		} else {
			$datetime = explode(' ', $date);
			$dateSplit = explode('-', $datetime[0]);
			if(empty($dateSplit[2]))return false;
			$dstRange = $this->getDSTRange($dateSplit[0], $zone);
			$datestamp = strtotime($date);
		}
		if(!$dstRange){
			return false;
		}
		$startstamp = $dstRange['start_ts'];
		$endstamp = $dstRange['end_ts'];
		if((($datestamp >= $startstamp  || $datestamp < $endstamp) && $startstamp > $endstamp)
			|| ($datestamp >= $startstamp && $datestamp < $endstamp)
		){
			return true;
		}
		return false;
	}

	function get_regular_expression($format) {
		static $expcache = array();
		if(isset($expcache[$format]))
			return $expcache[$format];
		$newFormat = '';
		$regPositions = array();
		$ignoreNextChar = false;
		$count = 1;
		for ($i = 0; $i < strlen($format); $i++) {
			$char = substr($format, $i, 1);
			if (!$ignoreNextChar && isset($this->supported_strings[$char])) {
				$newFormat.= '('.$this->supported_strings[$char].')';
				$regPositions[$char] = $count;
				$count++;
			} else {
				$ignoreNextChar = false;
				$newFormat.= $char;
			}
			if ($char == "\\") {
				$ignoreNextChar = true;
			}
		}
		$expcache[$format] = array('format'=>$newFormat, 'positions'=>$regPositions);
		return $expcache[$format];

	}

	function check_matching_format($date, $format, $toformat = '') {
		$regs = array();
		$startreg = $this->get_regular_expression($format);
		if (!empty($toformat)) {
			$otherreg = $this->get_regular_expression($toformat);
			//if the other format has the same regular expression then maybe it is shifting month and day position or something similar let it go for formating
			if ($startreg['format'] == $otherreg['format']) {
				return false;
			}
		}
		preg_match("~^{$startreg['format']}\$~", trim($date ?? ''), $regs);
		if (empty($regs)) {
			return false;
		}
		return true;
	}

	function swap_formats($date, $startFormat, $endFormat) {
	
		$startFormat = str_replace('g', 'h', $startFormat);
		$startreg = $this->get_regular_expression($startFormat);
		preg_match("~{$startreg['format']}~", $date, $regs);
		$newDate = $endFormat;
		$endFormat = str_replace('g', 'h', $endFormat);

		//handle 12 to 24 hour conversion

		if (isset($startreg['positions']['h']) && !isset($startreg['positions']['H']) && !empty($regs[$startreg['positions']['h']]) && strpos($endFormat, 'H') > -1) {
			$startreg['positions']['H'] = sizeof($startreg['positions']) + 1;
			$regs[$startreg['positions']['H']] = $regs[$startreg['positions']['h']];
			if ((isset($startreg['positions']['A']) && $regs[$startreg['positions']['A']] == 'PM') || (isset($startreg['positions']['a']) && $regs[$startreg['positions']['a']] == 'pm')) {

				if ($regs[$startreg['positions']['h']] != 12) {
					$regs[$startreg['positions']['H']] = $regs[$startreg['positions']['h']] + 12;
				}
			}
			if ((isset($startreg['positions']['A']) && $regs[$startreg['positions']['A']] == 'AM') || (isset($startreg['positions']['a']) && $regs[$startreg['positions']['a']] == 'am')) {
				if ($regs[$startreg['positions']['h']] == 12) {
					$regs[$startreg['positions']['H']] = 0;
				}
			}
		}
		if (!empty($startreg['positions']['H']) && !empty($regs[$startreg['positions']['H']]) && !isset($startreg['positions']['h']) && strpos($endFormat, 'h') > -1) {
			$startreg['positions']['h'] = sizeof($startreg['positions']) + 1;
			$regs[$startreg['positions']['h']] = $regs[$startreg['positions']['H']];
			if (!isset($startreg['positions']['A'])) {
				$startreg['positions']['A'] = sizeof($startreg['positions']) + 1;
				$regs[$startreg['positions']['A']] = 'AM';
			}
			if (!isset($startreg['positions']['a'])) {
				$startreg['positions']['a'] = sizeof($startreg['positions']) + 1;
				$regs[$startreg['positions']['a']] = 'am';
			}
			if ($regs[$startreg['positions']['H']] > 11) {
				$regs[$startreg['positions']['h']] = $regs[$startreg['positions']['H']] - 12;
				if ($regs[$startreg['positions']['h']] == 0) {
					$regs[$startreg['positions']['h']] = 12;
				}
				$regs[$startreg['positions']['a']] = 'pm';
				$regs[$startreg['positions']['A']] = 'PM';
			}
			if ($regs[$startreg['positions']['H']] == 0) {
				$regs[$startreg['positions']['h']] = 12;
				$regs[$startreg['positions']['a']] = 'am';
				$regs[$startreg['positions']['A']] = 'AM';
			}
		}
		if (!empty($startreg['positions']['h'])) {
			if (!isset($regs[$startreg['positions']['h']])) {
				$regs[$startreg['positions']['h']] = '12';
			} else if (strlen($regs[$startreg['positions']['h']]) < 2)
				$regs[$startreg['positions']['h']] = '0'.$regs[$startreg['positions']['h']];
		}
		if (!empty($startreg['positions']['H'])) {
			if (!isset($regs[$startreg['positions']['H']])) {
				$regs[$startreg['positions']['H']] = '00';
			} else if (strlen($regs[$startreg['positions']['H']]) < 2)
				$regs[$startreg['positions']['H']] = '0'.$regs[$startreg['positions']['H']];
		}
		if (!empty($startreg['positions']['d'])) {
			if (!isset($regs[$startreg['positions']['d']])) {
				$regs[$startreg['positions']['d']] = '01';
			} else if (strlen($regs[$startreg['positions']['d']]) < 2)
				$regs[$startreg['positions']['d']] = '0'.$regs[$startreg['positions']['d']];
		}
		if (!empty($startreg['positions']['i'])) {
			if (!isset($regs[$startreg['positions']['i']])) {
				$regs[$startreg['positions']['i']] = '00';
			} else if (strlen($regs[$startreg['positions']['i']]) < 2)
				$regs[$startreg['positions']['i']] = '0'.$regs[$startreg['positions']['i']];
		}
		if (!empty($startreg['positions']['m'])) {
			if (!isset($regs[$startreg['positions']['m']])) {
				$regs[$startreg['positions']['m']] = '01';
			} elseif(strlen($regs[$startreg['positions']['m']]) < 2)
				$regs[$startreg['positions']['m']] = '0'.$regs[$startreg['positions']['m']];
		}
		if (!empty($startreg['positions']['Y'])) {
			if (!isset($regs[$startreg['positions']['Y']])) {
				$regs[$startreg['positions']['Y']] = '2000';
			}
		}
		if (!empty($startreg['positions']['s'])) {
			if (!isset($regs[$startreg['positions']['s']])) {
				$regs[$startreg['positions']['s']] = '00';
			} else if (strlen($regs[$startreg['positions']['s']]) < 2)
				$regs[$startreg['positions']['s']] = '0'.$regs[$startreg['positions']['s']];
		} else {
			$startreg['positions']['s'] = sizeof($startreg['positions']) + 1;
			$regs[$startreg['positions']['s']] = '00';
		}
		
		$merid = isset($startreg['positions']['a']) || isset($startreg['positions']['A']);
		if($merid && isset($startreg['positions']['h']) && isset($regs[$startreg['positions']['h']])) {
			if($regs[$startreg['positions']['h']] > 0 && $regs[$startreg['positions']['h']] < 10)
				$regs[$startreg['positions']['h']] = substr($regs[$startreg['positions']['h']], 1);
		}

		foreach($startreg['positions'] as $key=>$val) {
			if (isset($regs[$val])) {
				$newDate = str_replace($key, $regs[$val], $newDate);
				if($key == 'h')
					$newDate = str_replace('g', ltrim($regs[$val], '0'), $newDate);
			}
		}
		return $newDate;

	}
	function to_display_time($date, $meridiem = true, $offset = true, $user = null) {

		$date = trim($date ?? '');
		if (empty($date)) {
			return $date;
		}
		if ($offset) {
			$date = $this->handle_offset($date, $this->get_db_date_time_format(), true);
		}
		return $this->to_display($date, $this->dbTimeFormat, $this->get_time_format($meridiem, $user));
	}

	function to_display_date($date, $use_offset = true, $user = null) {
		$date = trim($date ?? '');

		if (empty($date)) {
			return $date;
		}
		if ($use_offset)
			 $date = $this->handle_offset($date, $this->get_db_date_time_format(), true, $user);

		return $this->to_display($date, $this->dbDayFormat, $this->get_date_format());
	}

	function to_display_date_time($date, $meridiem = true, $offset = true, $user = null) {
		$date = trim($date ?? '');
	
		if (empty($date) || preg_match('/^[0:\-\. ]+$/', $date)) {
			return $date;
		}
		if ($offset) {
			$date = $this->handle_offset($date, $this->get_db_date_time_format(), true, $user);
		}
		
		return $this->to_display($date, $this->get_db_date_time_format(), $this->get_date_time_format($meridiem, $user));
	}

	function to_display($date, $fromformat, $toformat) {
		$date = trim($date ?? '');
		if (empty($date)) {
			return $date;
		}
		return $this->swap_formats($date, $fromformat, $toformat);
	}

	function to_db($date, $offset=true, $user = null) {
		$date = trim($date ?? '');
        $date = str_replace(',', '', $date);
		if (empty($date)) {
			return $date;
		}
		$date = $this->swap_formats($date, $this->get_date_time_format(), $this->get_db_date_time_format());
		if(! $offset) return $date;
		return $this->handle_offset($date, $this->get_db_date_time_format(), false, $user);
	}


	function get_time_separator() {
		if (preg_match('~[^a-zA-Z\s]~', $this->get_time_format(), $match)) {
			return $match[0];
		}
		return ':';
	}
	
	// longreach - start added
	function lax_parse_time($val, $to_display=true, $default=null) {
		$ret = '';
		if(preg_match('/^(\d+)[:h\.](\d+)?([m:\.]\d+)?\s*(am|pm)?/i', $val, $m)) {
			$hrs = max(0, $m[1] % 24);
			$min = max(0, $m[2] % 60);
			$sec = (! empty($m[3])) ? substr($m[3], 1) % 60 : 0;
			$merid = strtolower(array_get_default($m, 4, 0));
			if($merid == 'pm' && $hrs < 12)
				$hrs += 12;
			else if($merid == 'am' && $hrs == 12)
				$hrs = 0;
			$ret = sprintf('%02d:%02d:%02d', $hrs, $min, $sec);
		}
		else {
			if($default !== null)
				return $default;
			$ret = '00:00:00';
		}
		return $to_display ? $this->to_display_time($ret, true, false) : $ret;
	}
	// longreach - end added
	

	function to_db_date($date, $use_offset = true) {
		$date = trim($date ?? '');
		if (empty($date)) {
			return $date;
		}
		if ($use_offset) {
			$date = $this->to_db($date);
			$date = $this->swap_formats($date, $this->dbDayFormat, $this->dbDayFormat);
		} else {
			$date = $this->swap_formats($date, $this->get_date_format(), $this->dbDayFormat);
		}

		return $date;
	}

	function to_db_time($date, $use_offset = true) {
		$date = trim($date ?? '');
		if (empty($date)) {
			return $date;
		}
		if ($use_offset){
			$date =$this->to_db($date, $use_offset);
		 	$date = $this->swap_formats($date, $this->get_db_date_time_format(), $this->dbTimeFormat);
		}else{
		 	$date = $this->swap_formats($date, $this->get_time_format(), $this->dbTimeFormat);
		}
		return $date;


	}
	
	/* takes a local Date & Time value, concats it, and returns the separate but
	 * properly calculated GMT value as an array*/
	function to_db_date_time($date, $time) {
		global $current_user;
		if(is_object($current_user)) {
			$timeFormat = $current_user->getUserDateTimePreferences();
		} else {
			$timeFormat['date'] = $this->dbDayFormat;
			$timeFormat['time'] = $this->dbTimeFormat;
		}
		$dt = '';
		$newDate = '';
		$retDateTime = array();

		// concat: ('.' breaks strtotime())
		$time = str_replace('.',':',$time);
		$dt = $date." ".$time;
		$newDate = $this->swap_formats($dt, $timeFormat['date'].' '.$timeFormat['time'] , $this->dbDayFormat.' '.$this->dbTimeFormat);
		$retDateTime = explode(' ', $newDate);
		return $retDateTime;
	}
	
	function getUserEventOffset($user_in_dst, $event_in_dst){
		if($user_in_dst && !$event_in_dst ){
			return -3600;
		}
		if(!$user_in_dst && $event_in_dst ){
			return 3600;
		}
		return 0;
	}
	
/**************************************************************
U	S	E	Time	GMT	Delta Server Client	U/E	Delta Server GMT
USER IN LA and server in NY
D	D	D	12	19	-3	0	-4
D	D	S	12	20	-3	-1	-4
D	S	D	12	19	-2	0	-5
D	S	S	12	20	-2	-1	-5
S	D	D	12	19	-4	1	-4
S	D	S	12	20	-4	0	-4
S	S	D	12	19	-3	1	-5
S	S	S	12	20	-3	0	-5
							
							
User in LA and server in gmt there are no DST for server
D	S	D	12	19	-7	0	0
D	S	S	12	20	-7	-1	0
														
S	S	D	12	19	-8	1	0
S	S	S	12	20	-8	0	0

***************************************************************/
	
	/**
	 * handles offset values for Timezones and DST
	 * @param	$date	string		date/time formatted in user's selected
	 * format
	 * @param	$format	string		destination format value as passed to PHP's
	 * date() funtion
	 * @param	$to		boolean 
	 * @param	$user	object		user object from which Timezone and DST
	 * values will be derived
	 * @return 	 string		date formatted and adjusted for TZ and DST  
	 */
	function handle_offset($date, $format, $to = true, $user = null) {
		$date = trim($date, '');
		
		if (empty($date)) {
			return $date;
		}
		$time = strtotime($date);
		if ($time == -1) {
			return $date;
		}
		$seconds = $this->get_seconds_offset($date, $to, $user);
		return date($format, $time + $seconds);
	}
	
	// date may be numeric (a timestamp)
	function get_seconds_offset($date, $to = true, $user = null) {
		$deltaServerGMT = date('Z');
		$timezone = $this->getUserTimeZone($user);
		
		if(is_object($timezone)) {
			if($to) {
				// have a GMT time, produce offset to user timezone
				$dt = $this->getDateTime(strtotime($date));
				$offset = $timezone->getOffset($dt);
			}
			else {
				// produce offset from given user time to GMT
				$dt = $this->getDateTime($date);
				if($dt)
					$offset = - $timezone->getOffset($dt);
				else
					$offset = 0;
			}
			return $offset;
		}
		
		$deltaServerUser = $this->get_hour_offset($to, $timezone);
		$event_in_ds = $this->inDST($date, $timezone);
		$user_in_ds = $this->inDST(time(), $timezone);
		//$server_in_ds = date('I');
		$ue = $this->getUserEventOffset($user_in_ds, $event_in_ds);
		$zone = 1;
		if (!$to)
			$zone = -1;
		return $deltaServerUser * 3600 + ($ue + $deltaServerGMT) * $zone;
	}
	
	
	/*	this method will take an input $date variable (expecting Y-m-d format)
	 *	and get the GMT equivalent - with an hour-level granularity :
	 *	return the max value of a given locale's 
	 *	date+time in GMT metrics (i.e., if in PDT, "2005-01-01 23:59:59" would be 
	 *	"2005-01-02 06:59:59" in GMT metrics)
	 */
	function handleOffsetMax($date, $format, $to = true) {
		global $current_user;
		$gmtDateTime = array($date); // for errors
		/* check for bad date formatting */
		$date = trim($date, '');

		if (empty($date)) {
			return $gmtDateTime;
		}

		if (strtotime($date) == -1) {
			return $gmtDateTime;
		}

		/*	cn: passed $date var will be a "MAX" value, which we need to return 
			as a GMT date/time pair to provide for hour-level granularity */
		/* this ridiculousness b/c PHP returns current time when passing "today"
			or "tomorrow" as strtotime() args */
		$dateNoTime = date('Y-m-d', strtotime($date));

		/* handle timezone and daylight savings */
		$dateWithTimeMin = $dateNoTime.' 00:00:00';
		$dateWithTimeMax = $dateNoTime.' 23:59:59';
		
		
		$offsetDateMin = $this->handle_offset($dateWithTimeMin, $this->dbDayFormat.' '.$this->dbTimeFormat, false);
		$offsetDateMax = $this->handle_offset($dateWithTimeMax, $this->dbDayFormat.' '.$this->dbTimeFormat, false);
		
		
		$exOffsetDateMax = explode(' ', $offsetDateMax);
		$exOffsetDateMin = explode(' ', $offsetDateMin);
		$gmtDateTime['date'] = $exOffsetDateMax[0];
		$gmtDateTime['time'] = $exOffsetDateMax[1];
		$gmtDateTime['min'] = $offsetDateMin;
		$gmtDateTime['max'] = $offsetDateMax;
		
		return $gmtDateTime;
	}
	

	function get_gmt_db_datetime() {
		return gmdate('Y-m-d H:i:s');
	}

	function get_gmt_db_date() {
		return gmdate($this->dbDayFormat);
	}

	/*
	 * assumes that olddatetime is in Y-m-d H:i:s format
	 */
	static function convert_to_gmt_datetime($olddatetime) {
		if (!empty($olddatetime)) {
			return date('Y-m-d H:i:s', strtotime($olddatetime) - date('Z'));
		}
	}



	function merge_date_time($date, $time) {
		$merged =  $date.' '.$time;
	
		return $merged;
	}
	function merge_time_meridiem($date, $format, $mer) {
		$date = trim($date, '');
		if (empty($date)) {
			return $date;
		}
		 $fakeMerFormat = str_replace(array('a', 'A'), array('!@!', '!@!'), $format);
		$noMerFormat = str_replace(array('a', 'A'), array('', ''), $format);
		$newDate = $this->swap_formats($date, $noMerFormat, $fakeMerFormat);
		return str_replace('!@!', $mer, $newDate);

	}
	
	/*
	 *  AMPMMenu
	 *  This method renders a <select> HTML form element based on the
	 *  user's time format preferences.  
	 * 
	 *  todo: There is hardcoded HTML in here that does not allow for localization
	 *  of the AM/PM am/pm Strings in this drop down menu.  Also, perhaps
	 *  change to the substr_count function calls to strpos 
	 * 
	 */
	function AMPMMenu($prefix, $date, $attrs = '') {

		if (substr_count($this->get_time_format(), 'a') == 0 && substr_count($this->get_time_format(), 'A') == 0) {
			return '';
		}
		$menu = "<select name='".$prefix."meridiem' ".$attrs.">";

		if (strpos($this->get_time_format(), 'a') > -1) {

			if (substr_count($date, 'am') > 0)
				$menu.= "<option value='am' selected>am</option>";
			else
				$menu.= "<option value='am'>am";
			if (substr_count($date, 'pm') > 0)
				$menu.= "<option value='pm' selected>pm</option>";
			else
				$menu.= "<option value='pm'>pm</option>";

		} else {

			if (substr_count($date, 'AM') > 0)
				$menu.= "<option value='AM' selected>AM</option>";
			else
				$menu.= "<option value='AM'>AM";
			if (substr_count($date, 'PM') > 0) {
				$menu.= "<option value='PM' selected>PM</option>";
			} else
				$menu.= "<option value='PM'>PM</option>";

		}

		return $menu.'</select>';
	}

	function get_hour_offset($to = true, $timezone = null) {
	
		$timeDelta = $this->adjustmentForUserTimeZone($timezone) /60.0;
		if ($to) {
			return -1.0 * $timeDelta;
		}
		return 1.0 * $timeDelta;
	}
	
	function get_time_format($meridiem = true, $user = null) {
		global $current_user;

		if(!empty($user) && !is_object($user)) {
			$uid = $user;
			$user = new User;
			$user->retrieve($uid);
		}
		if(empty($user)) {
			if(! empty($current_user) && $current_user->id)
				$timeFormat = $current_user->getPreference('default_time_format');
			else
				$timeFormat = AppConfig::setting('locale.defaults.time_format');
		} else
			$timeFormat = $user->getPreference('default_time_format');
		
		if (!$meridiem) {
			$timeFormat = str_replace(array('a', 'A'), array('', ''), $timeFormat);
		}
		
		// strip leading zero from hour
		if(strpos($timeFormat, 'a') !== false || strpos($timeFormat, 'A') !== false)
			$timeFormat = str_replace('h', 'g', $timeFormat);
		
		return $timeFormat;
	}

	function get_date_format($user = null) {
		global $current_user;
		
		if(!empty($user) && !is_object($user)) {
			$uid = $user;
			$user = new User;
			$user->retrieve($uid);
		}
		
		if(empty($user)) {
			if(! empty($current_user) && $current_user->id)
				$user =& $current_user;
			else
				return AppConfig::setting('locale.defaults.date_format');
		}
		
		return $user->getPreference('default_date_format');
	}

	function get_date_time_format($meridiem = true, $user = null) {
		return $this->get_date_format($user).' '.$this->get_time_format($meridiem, $user);
	}

	function get_db_date_time_format() {
		return $this->dbDayFormat.' '.$this->dbTimeFormat;
	}

	function get_db_date_format() {
		return $this->dbDayFormat;
	}

	function get_db_time_format() {
		return $this->dbTimeFormat;
	}

	function get_cal_date_format() {
		$from = array('Y', 'm', 'd');
		$to = array('%Y', '%m', '%d');
		$format = $this->get_date_format();
		return str_replace($from, $to, $format);
	}
	
	function get_cal_time_format() {
		$from = array('H', 'h', 'g', 'i', 'A', 'a');
		$to = array('%H', '%I', '%l', '%M', '%p', '%P');
		$format = $this->get_time_format();
		return str_replace($from, $to, $format);
	}

	function get_user_date_format() {
		$format = str_replace('Y', 'yyyy', $this->get_date_format());
		$format = str_replace('m', 'mm', $format);
		$format = str_replace('d', 'dd', $format);
		return $format;
	}

	function get_user_time_format() {
		$time_pref = $this->get_time_format();
		
		$fmts = AppConfig::setting('locale.time_formats');
		if(!empty($time_pref) && !empty($fmts[$time_pref])) {
		   return $fmts[$time_pref];	
		}
		
		return '23:00'; //default
	}

	function get_microtime_string() {
		$now = (string) microtime();
		$now = explode(' ', $now);
		$unique_id = $now[1].str_replace('.', '', $now[0]);
		unset($now);

		return $unique_id;
	}
	
	function to_relative_date($db_dt, $has_time=true, $params=null, $user = null) {
		if(! is_array($params)) $params = array();
		$params['show_time'] = false;
		if(! $has_time) {
			$db_dt .= ' 12:00:00';
			$params['no_offset'] = true;
		}
		return $this->to_relative_date_time($db_dt, $params, $user);
	}
	
	function user_local_time($user = null) {
		$base_time = time();
		$base_time += date('Z') + $this->get_seconds_offset($base_time, false, $user);
		return $base_time;
	}
	
	function to_relative_date_time($db_dt, $params=null, $user = null) {
		global $app_list_strings, $current_user;

		$show_time = true;
		$relative_seconds = false;
		$no_offset = false;
		if(is_array($params))
			extract($params);
		
		$daylen = 24 * 3600;
		$current = time();
		$rel_dom = $app_list_strings['relative_time_dom'];
		if (!$user)
			$user = $current_user;
		$user_tz = $user->getPreference('timezone');
		$user_offset_sys = $this->getTimeZoneOffset($user_tz, false);
		$user_current = $current + $user_offset_sys;
		$user_midnight = strtotime(date('Y-m-d 00:00:00', $user_current)) - $user_offset_sys;

		if(! $db_dt) {
			return '';
		} else if (is_numeric($db_dt)) {
			$time = $db_dt;
			$db_dt = gmdate($this->get_db_date_time_format(), $time);
		} else if(strpos($db_dt, '0000') !== false) {
			return '';
		} else {
			$time = strtotime($db_dt . ' GMT');
			if(! $time)
				return '';
		}

		if($show_time)
			$timeStr = $this->to_display_time($db_dt, true, true, $user);

		if($show_time && $time <= $current && ($diff = ($current - $time)) <= $daylen/2) {
			if(($diff < 2 && $relative_seconds) || ($diff < 30 && ! $relative_seconds))
				return $rel_dom['moments_ago'];
			if($diff < 60 && $relative_seconds)
				return str_replace('{N}', $diff, $rel_dom['n_seconds_ago']);
			if($diff < 3600) {
				$diff = round($diff / 60);
				return str_replace('{N}', $diff, $rel_dom[$diff == 1 ? 'n_minute_ago' : 'n_minutes_ago']);
			}
			if($diff < $daylen) {
				$diff = round($diff / 3600);
				return str_replace('{N}', $diff, $rel_dom[$diff == 1 ? 'n_hour_ago' : 'n_hours_ago']);
			}
		}
		
		if($time >= $user_midnight) {
			if($time <= $user_midnight + $daylen) {
				if($show_time)
					return str_replace('{TIME}', $timeStr, $rel_dom['today_time']);
				else
					return $rel_dom['today'];
			}
			else if($time <= $user_midnight + 2*$daylen) {
				if($show_time)
					return str_replace('{TIME}', $timeStr, $rel_dom['tomorrow_time']);
				else
					return $rel_dom['tomorrow'];
			}
			else if($time <= $user_midnight + 6*$daylen) {
				$wday = date('w', $time + $user_offset_sys);
				$wday = $app_list_strings['weekdays_long_dom'][$wday];
				$ret = str_replace('{WDAY}', $wday, $rel_dom[$show_time ? 'weekday_time' : 'weekday']);
				if($show_time)
					$ret = str_replace('{TIME}', $timeStr, $ret);
				return $ret;
			}
		} else {
			if($time > $user_midnight - $daylen) {
				if($show_time)
					return str_replace('{TIME}', $timeStr, $rel_dom['yesterday_time']);
				else
					return $rel_dom['yesterday'];
			}
			if(($diff = $user_midnight - $time) <= 7*$daylen) {
				$diff = ceil(($diff + 1) / $daylen);
				$ret = str_replace('{N}', $diff, $rel_dom[$show_time ? 'n_days_ago_time' : 'n_days_ago']);
				if($show_time)
					$ret = str_replace('{TIME}', $timeStr, $ret);
				return $ret;
			}
		}
		if($show_time) {
			$dateStr = $this->to_display_date($db_dt, true, $user);
			$ret = str_replace(array('{DATE}','{TIME}'), array($dateStr, $timeStr), $rel_dom['date_time']);
			return $ret;
		} else
			return $this->to_display_date($db_dt, true, $user);
	}
	
}
?>
