// vim: set filetype=markdown tw=80:
Two endpoints which return a list of records, [Get list of records](#endpoint_dataList_get) and [Get list of related records](#endpoint_dataRelatedList_get), support two query parameters to get a specific data set.

1. `filter_text` - Generic search string. Fields involved in search depend on a `model`. 
Include this parameter to a query for filtering a list of records by __Subject__, __Name__, __#__ (like __Invoice #__) etc.
<br />__Examples__: `/api.php/data/Invoice?filter_text=2020-100` `api.php/data/Task?filter_text=Send%20Contract`          
2. `filters` - A list of filters to apply. To get a list of available filters for a model, use [metadata endpoint](#group_meta). 
Some filters differ from field names. __Check a model metadata carefully to avoid mistakes.__ For example if you want filtering records by email 
you have to use `any_email` filter parameter instead of `email1` or `email2`.    
__Examples__: `/api.php/data/Account?filters[account_type]=Supplier&filters[any_email]=sales@1crm.com`
Also for some filters you can use a part of the value like this - `api.php/data/Account?filters[any_email]=sales`          
<br />For __Ref__ fields (e.g. Billing Account, Primary Contact etc.) there are two ways of filter applying:
    1. Using text value of Ref field, e.g. Account Name - `/api.php/data/Invoice?filters[billing_account]=Microsoft`
    2. Using ID. You can find the correct name of a filter for this case in a filter metadata, under `id_name` - `/data/Invoice?filters[billing_account_id]=2e1a3464-47e0-5201-4a5c-5c9b25575447`             
3. __Date Filters__: filtering by date fields may require additional parameters. The most used second parameter for the date filter is `operator`.
If you omit `operator` parameter the default value will be `on_date`.
<br />All possible `operator` parameter values:
* `not_empty` field value is not empty <strong>(provided without date_field value)</strong> 
* `empty`  field value is empty <strong>(provided without date_field value)</strong>
* `before_date`  field value is before the date passed in a query
* `after_date` field value is after the date passed in a query
* `on_date` field value equals the date passed in a query
* `not_on_date` field value does not equal the date passed in w query 
* `yesterday` field value is yesterday's date <strong>(provided without date_field value)</strong>
* `today`  field value is today's date <strong>(provided without date_field value)</strong>
* `tomorrow`  field value is tomorrow's date <strong>(provided without date_field value)</strong>
* `between_dates` field value is between dates passed query, inclusive <strong>(required end date value `date_field-end`)</strong>    
<br />You can also pass `period_prev`, `period_current`, or `period_next` as `operator` value.
In this case filed value will be checked against previous, current, or next
period relative to current date. Period length is determined by `date_field-period` parameter:
* `day` one day
* `week` calendar week
* `month` calendar month
* `quarter` calendar quarter
* `year` calendar year
* `fiscal_quarter` fiscal quarter
* `fiscal_year` fiscal year
* `days_N` N days (dayes_7, days_14, days_30, etc)
  
   __Examples__: 
<br />`/api.php/data/Invoice?filters[due_date]=2020-01-01` - get all invoices with `due_date` = __'2020-01-01'__ (`due_date-operator` = 'on_date' by default)    
`/api.php/data/Invoice?filters[due_date]=2020-01-01&filters[due_date-operator]=before_date` - get all invoices with `due_date` < __'2020-01-01'__
`/api.php/data/Invoice?filters[due_date-operator]=empty` - get all invoices with empty `due_date` value
<br />`/api.php/data/Invoice?filters[due_date]=2020-01-01&filters[due_date-end]=2020-02-01&filters[due_date-operator]=between_dates` - get all invoices with `due_date` >= __'2020-01-01'__ AND `due_date` <= __'2020-02-01'__
<br />`/api.php/data/Invoice?filters[due_date-operator]=period_prev&filters[due_date-period]=week` - get all invoices with `due_date` between start and end of previous week from today  
