<?php

use Slim\Factory\AppFactory;
use Slim\Middleware\ContentLengthMiddleware;
use Slim\Routing\RouteCollectorProxy;
use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Server\RequestHandlerInterface as RequestHandler;
use OneCRM\API\Middleware\RequestUri as RequestUriMiddleware;
use OneCRM\API\Middleware\Check1CRMMobile as Check1CRMMobileMiddleware;
use OneCRM\API\Middleware\SecureConnectionGuard as SecureConnectionGuardMiddleware;
use \OneCRM\API\Middleware\Cors as CorsMiddleware;
use OneCRM\API\Handlers\ErrorHandler;
use \OneCRM\API\Middleware\CssUrl;

if(! AppConfig::setting('site.security.api_enabled')) {
	header("HTTP/1.1 403 Forbidden");
	exit;
}

$serverDirectory = __DIR__;

// DONE: Speichern von Logindaten
// Todo: Remember me | https://paragonie.com/blog/2015/04/secure-authentication-php-with-long-term-persistence#title.2
// DONE: auth_code Server übernehmen
// DONE: CSRF validation testing?

// TODO: input parameter validation https://github.com/DavidePastore/Slim-Validation
// DONE: Was passiert bei Aufruf ohne Parameter? Login möglich?

// DONE: Security!!! Keys auslagern, darf nicht erreichbar sein über Server
// DONE: Repositories fertig programmieren
// TODO: UserRepository Passwort default umbauen, email versenden
// DONE: API Ausgabe auf Benutzer
// TODO: API, Gruppenmitgliedschaft
// TODO: WooCommerce, Zuordnung vorhandene Benutzer - wie?

//session_start();

$container = new \DI\Container();
AppFactory::setContainer($container);

$app = AppFactory::create();

$scriptPos = strpos($_SERVER['REQUEST_URI'], '/api.php');
$basePath = $scriptPos !== false ? substr($_SERVER['REQUEST_URI'], 0, $scriptPos + 8) : '';

$app->setBasePath($basePath);

$callableResolver = $app->getCallableResolver();
$responseFactory = $app->getResponseFactory();

$serverDirectoryMiddleware = function (Request $request, RequestHandler $handler) use ($serverDirectory) {
	$request = $request
		->withAttribute('docsDir', $serverDirectory . '/docs')
		->withAttribute('templateDir', $serverDirectory . '/template');
	return $handler->handle($request);
};

$app->add(new ContentLengthMiddleware());
$app->add(new RequestUriMiddleware());

require $serverDirectory . '/src/app/dependencies.php';

$app->get('', '\\OneCRM\\API\\DocsGenerator')->setName('docs')->add($serverDirectoryMiddleware)->setName('docs');

$registryContainer = $app->getContainer()->get('registry');

$eachEndpoint = function($callback) use($registryContainer) {
	foreach($registryContainer->getEndpoints() as $ep) {
		foreach ($ep->allowedMethods() as $method) {
			$callback($ep, $method);
		}
	}
};

$apiGroup = $app->group('', function (RouteCollectorProxy $group) use ($eachEndpoint) {
	$eachEndpoint(function($ep, $method) use ($group) {
		if (!$ep->isAuthEndpoint()) {
			$route = $group->$method($ep->endpoint(), $ep);
			$ep->addMiddleware($group, $route);
			$routeName = $ep->getRouteName($method);
			if ($routeName)
				$route->setName($routeName);
		}
	});
})
	->add(new OneCRM\API\Middleware\StartOneCRMSession)
	->add(new OneCRM\API\Middleware\DetectLanguage)
;

$authGroup = $app->group('/auth', function (RouteCollectorProxy $app) use ($serverDirectory, $eachEndpoint) {

	require $serverDirectory . '/src/app/routes.php';
	$eachEndpoint(function($ep, $method) use ($app) {
		if ($ep->isAuthEndpoint()) {
			$endpoint = substr($ep->endpoint(), 5);
			$route = $app->$method($endpoint, $ep);

			$ep->addMiddleware($app, $route);

			$routeName = $ep->getRouteName($method);
			if ($routeName)
				$route->setName($routeName);
		}
	});
});

$app->add(new CssUrl());
$app->add(new SecureConnectionGuardMiddleware());
$app->add(new Check1CRMMobileMiddleware());
$app->add(new CorsMiddleware());

$app->add(function (Request $request, RequestHandler $handler) use ($app) {
	$container = $app->getContainer();
	$container->set('request', $request);
    return $handler->handle($request);
});

$responseFactory = $app->getResponseFactory();
$errorHandler = new ErrorHandler($callableResolver, $responseFactory);

$errorMiddleware = $app->addErrorMiddleware(true, true, true);
$errorMiddleware->setDefaultErrorHandler($errorHandler);

$app->run();
