<?php
/**
 * Created by PhpStorm.
 * User: brafreider
 * Date: 26.07.2016
 * Time: 17:14
 */

use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Server\RequestHandlerInterface as RequestHandler;
use League\OAuth2\Server\ResourceServer;
use League\OAuth2\Server\AuthorizationServer;
use League\OAuth2\Server\Grant\AuthCodeGrant;
use League\OAuth2\Server\Grant\ImplicitGrant;
use League\OAuth2\Server\Grant\PasswordGrant;
use League\OAuth2\Server\Grant\ClientCredentialsGrant;
use League\OAuth2\Server\CryptKey;
use Psr\Http\Message\ResponseFactoryInterface;
use v4\oAuth\Repositories\ClientRepository;
use v4\oAuth\Repositories\AccessTokenRepository;
use v4\oAuth\Repositories\ScopeRepository;
use v4\oAuth\Repositories\AuthCodeRepository;
use v4\oAuth\Repositories\RefreshTokenRepository;
use League\OAuth2\Server\Grant\RefreshTokenGrant;
use v4\oAuth\Repositories\UserRepository;
use \Slim\Csrf\Guard;
use \DI\Container;
use Defuse\Crypto\Key;

new UserRepository();

/* @var $container Container */
/* @var $responseFactory ResponseFactoryInterface */

$container->set('csrf', function () use ($responseFactory) {
	$guard = new Guard($responseFactory);
	$guard->setFailureHandler(function (Request $request, RequestHandler $handler) {
		$request = $request->withAttribute("csrf_status", false);
		return $handler->handle($request);
	});

	return $guard;
});

$container->set('view', function ($c) {
	global $app_strings;
	$request = $c->get('request');
    $css_path = $request->getServerParams()['SCRIPT_NAME'].'/css';
	$css_path = preg_replace('~/api\.php/~', '/include/api/', $css_path);
    $attrs = [
		'css_path' => $css_path,
		'LANG' => $app_strings,
	];
    return new \Slim\Views\PhpRenderer(__DIR__ .'/../../templates/', $attrs);
});

$container->set('flash', function ($c) {
    return new \Slim\Flash\Messages();
});

$userRepository = new UserRepository();
$clientRepository = new ClientRepository();

$container->set(ClientRepository::class, $clientRepository);

$container->set(AuthorizationServer::class, function () use ($userRepository, $clientRepository) {
    // Init our repositories
    $accessTokenRepository = new AccessTokenRepository();
    $scopeRepository = new ScopeRepository();
    $authCodeRepository = new AuthCodeRepository();
    $refreshTokenRepository = new RefreshTokenRepository();
    $privateKey = AppConfig::setting('oauth.private_key');

	$encryptionKey = AppConfig::setting("oauth.encryption_key");
	if (!$encryptionKey) {
		$key = Key::createNewRandomKey();
		$encryptionKey = $key->saveToAsciiSafeString();
		AppConfig::set_local("oauth.encryption_key", $encryptionKey);
		AppConfig::save_local();
	}
	
    // Setup the authorization server
    $server = new AuthorizationServer(
        $clientRepository,
        $accessTokenRepository,
        $scopeRepository,
		$privateKey,
		$encryptionKey
    );
    // Enable the authentication code grant on the server with a token TTL of 1 hour
    $server->enableGrantType(
        new AuthCodeGrant(
            $authCodeRepository,
            $refreshTokenRepository,
            new \DateInterval('PT2M') // authroization code TTL
        ),
        new \DateInterval('PT24H') // access token TTL
    );
    $server->enableGrantType(
        new ImplicitGrant(
            new \DateInterval('PT24H') // access token TTL
        ),
        new \DateInterval('PT24M') // not used
    );
    $server->enableGrantType(
		new PasswordGrant(
			$userRepository,
			$refreshTokenRepository
        ),
        new \DateInterval('PT24H') //access token TTL
    );
    $server->enableGrantType(
        new RefreshTokenGrant($refreshTokenRepository),
        new \DateInterval('PT24H') // access token TTL
    );
    $server->enableGrantType(
        new ClientCredentialsGrant(),
		new \DateInterval('PT24H') // access token TTL
    );
    return $server;
});

$container->set(ResourceServer::class, function ($c) {
    $accessTokenRepository = new AccessTokenRepository();
    $publicKey = AppConfig::setting('oauth.public_key');

    return new ResourceServer($accessTokenRepository, new CryptKey($publicKey));
});

$container->set('UserRepository', function($c) use ($userRepository) {
    return $userRepository;
});

$container->set('RememberMeRepository', function () {
	return new \v4\utils\RememberMeTokenRepository();
});

$container->set('registry', function () use($app) {
	return new OneCRM\API\Registry(include __DIR__ . '/registry.php', $app);
});
