<?php
/**
 * Created by PhpStorm.
 * User: brafreider
 * Date: 26.07.2016
 * Time: 17:15
 */

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use v4\oAuth\Entities\ClientEntity;
use v4\oAuth\Entities\UserEntity;
use Slim\Routing\RouteContext;

$app->get('/', function (Request $request, Response $response) {
	return $response;
})->setName('Home');

$app->get('/logout', function (Request $request, Response $response) {
	unset($_SESSION['user_id']);
	$routeParser = RouteContext::fromRequest($request)->getRouteParser();
	$redirectUri = isset($_GET['redirect_uri']) ? $_GET['redirect_uri'] : $routeParser->urlFor('Home');
    return $response->withRedirect($redirectUri);
})->setName('logout');

$app->any('/lostpassword', function (Request $request, Response $response) {
    $errors = [];
    /** @var \v4\oAuth\Repositories\UserRepository $userRepository */
    $userRepository = $this->UserRepository;
    $redirParam = $request->getParam('redirect');
    if ($request->isPost()) {
        //CSFR Check
        if (false === $request->getAttribute('csrf_status')) {
            $this->flash->addMessage('error', 'Validationsfehler, bitte noch einmal!');
			$routeParser = RouteContext::fromRequest($request)->getRouteParser();
            return $response->withRedirect($routeParser->urlFor('lostPassword'));
        }
        $username = $request->getParsedBodyParam('username');
        $email = $request->getParsedBodyParam('email');
        $email = $userRepository->resetPasswordForUser($username, $email);
        // Security -> mail eigentlich nicht anzeigen?
        $emailHint = $userRepository->getEmailHint($email);
        $this->flash->addMessage('error', 'Es wurde eine Mail versendet. Bitte prüfe Dein Postfach ' . $emailHint);

        // RememberMe Tokens Löschen
        $userEntity = $userRepository->getUserEntityByUserName($username);
        $this->RememberMeRepository->invalidateTokensForUser($userEntity->getIdentifier());

		$routeParser = RouteContext::fromRequest($request)->getRouteParser();
        return $response->withRedirect($routeParser->urlFor('login', [],  ['redirect' => $redirParam]));
    }
    // CSRF token name and value
    $csfr['nameKey'] = $this->csrf->getTokenNameKey();
    $csfr['valueKey'] = $this->csrf->getTokenValueKey();
    $csfr['name'] = $request->getAttribute($csfr['nameKey']);
    $csfr['value'] = $request->getAttribute($csfr['valueKey']);

    $messages = $this->flash->getMessages();
    if (isset($messages['error']))
        $errors = array_merge($errors, $messages['error']);
    $username = '';
    $emailHint = '';
    if (isset($_SESSION['username'])) {
        $username = $_SESSION['username'];

        /** @var UserEntity $user */
        $user = $userRepository->getUserEntityByUserName($username);
        if ($user) {
            $emailHint = $userRepository->getEmailHint($user->getEmail());
        }
    }
	$routeParser = RouteContext::fromRequest($request)->getRouteParser();
    $response = $this->view->render($response, 'lostPassword.php', ['username' => $username, 'csfr' => $csfr, 'errors' => $errors, 'router' => $routeParser, 'emailHint' => $emailHint, 'redirect' => $redirParam]);

    return $response;
})->setName('lostPassword');

$app->any('/changepassword', function (Request $request, Response $response) {
    $errors = [];
	$routeParser = RouteContext::fromRequest($request)->getRouteParser();
    /** @var Slim\Flash\Messages $flash */
    $flash = $this->flash;
    $redirect = $request->getParam('redirect', 'Home');
    if (isset($_SESSION['user_id'])) {
        if ($request->isPost()) {
            // validate CSFR
            if (false === $request->getAttribute('csrf_status')) {
                $flash->addMessage('error', 'Validationsfehler, bitte noch einmal!');
                return $response->withRedirect($routeParser->urlFor('changePassword'));
            }
            $new_password = $request->getParsedBodyParam('new_password');
            if ($new_password != $request->getParsedBodyParam('new2_password')) {
                $flash->addMessage('error', 'Die Passwörter stimmen nicht überein!');
                return $response->withRedirect($routeParser->urlFor('changePassword'));
            }
            // revalidate old password
            /** @var \v4\oAuth\Repositories\UserRepository $userRepository */
            $userRepository = $this->UserRepository;
            $userEntity = $userRepository->getUserEntityById($_SESSION['user_id']);
            $old_password = $request->getParsedBodyParam('old_password');
            $userEntity = $userRepository->getUserEntityByUserCredentials($userEntity->getUsername(), $old_password, '', new ClientEntity());
            // on success store new password
            if ($userEntity) {
                $res = $userRepository->setPasswordForUser($userEntity, $new_password, $old_password);
                if ($res === true) {
                    // redirect to origin
                    $message = 'Dein Passwort wurde geändert';
                    $result = 'success';

                    if (strpos($redirect, 'http') === 0) {
                        $querystring = '?result=' . $result . '&message=' . $message;
                        $redirect = $redirect . $querystring;
                    } else {
                        $flash->addMessage('error', $message);
                        $redirect = $routeParser->urlFor($redirect);
                    }
                    return $response->withRedirect($redirect);
                }
            }


        }

        // CSRF token name and value
        $csfr['nameKey'] = $this->csrf->getTokenNameKey();
        $csfr['valueKey'] = $this->csrf->getTokenValueKey();
        $csfr['name'] = $request->getAttribute($csfr['nameKey']);
        $csfr['value'] = $request->getAttribute($csfr['valueKey']);

        $messages = $flash->getMessages();
        if (isset($messages['error']))
            $errors = array_merge($errors, $messages['error']);

        $response = $this->view->render($response, 'changepassword.php', ['csfr' => $csfr, 'errors' => $errors, 'router' => $routeParser, 'redirect' => $redirect]);
        return $response;
    }
    $redirect = $request->getParam('redirect', 'login');
    if (strpos($redirect, 'http') === 0) {
        $querystring = '?error=Bitte zuerst anmelden.';
        $redirect = $redirect . $querystring;
    } else {
        $flash->addMessage('error', 'Um das Passwort zu ändern musst Du angemeldet sein!');
        $redirect = $route->urlFor($redirect);
    }
    return $response->withRedirect($redirect);
})->setName('changePassword');

