<?php

namespace OneCRM\API\Endpoints\Auth;

use \v4\utils\RememberMeMiddleware;
use \OneCRM\API\Middleware\StartOneCRMSession;

trait SessionTrait {

	abstract protected function resourceOwnerType();
	protected function sessionUseCookies() {
		return false;
	}

	public function addMiddleware($app, $route) {
		$container = $app->getContainer();
		$mw = new RememberMeMiddleware($container->get('RememberMeRepository'), $this->resourceOwnerType() . '_');

		$route->add($mw);
		$route->add('csrf');
		$route->add(new StartOneCRMSession(true));
	}

	protected function sessionGet($name, $defaultValue = null) {
		$key = $this->resourceOwnerType() . '_' . $name;
		if (isset($_SESSION[$key]))
			return $_SESSION[$key];
		return $defaultValue;
	}

	protected function sessionIsSet($name) {
		return isset($_SESSION[$this->resourceOwnerType() . '_' . $name]);
	}

	protected function sessionUnset($name) {
		unset($_SESSION[$this->resourceOwnerType() . '_' . $name]);
	}

	protected function sessionSet($name, $value) {
		$_SESSION[$this->resourceOwnerType() . '_' . $name] = $value;
	}
}

