<?php

namespace OneCRM\API\Endpoints\Calendar;
use OneCRM\API\Endpoint;
use OneCRM\API\Errors;

class Reminders extends Endpoint {

	public function endpoint() {
		return '/calendar/reminders';
	}

	public function descriptor($method) {
		if ($method == 'get')
			return $this->descriptorGet();
	}

	public function requiredScopes($method) {
		return ['read'];
	}

	public function allowedMethods() {
		return ['get'];
	}

	private function descriptorGet() {
		static $desc = [
			'hidden' => true,
			'title' => 'Get list of event reminders',
			'docs' => '@endpoint/calendar/reminders',
			'params' => [
			],
			'return' => [
				'records' => [
					'type' => [
						'type' => 'Array',
						'element_type' => [
							'type' => 'Object',
							'schema' => [
								'id' => [
									'type' => 'String',
									'docs' => 'Record ID',
									'required' => true,
								],
								'start_date' => [
									'type' => 'DateTime',
									'docs' => 'Event start date/time'
								],
								'name' => [
									'type' => 'String',
									'docs' => 'Event name',
									'required' => true,
								],
								'reminder_time' => [
									'type' => 'String',
									'docs' => 'Reminder time',
									'required' => true,
								],
								'type' => [
									'type' => 'Enum',
									'docs' => 'Event type',
									'options' => ['Call', 'Meeting', 'Task', 'ProjectTask'],
									'required' => true,
								],
							]
						]
					],
					'docs' => 'Array of retrieved records',
					'required' => true,
				],
			]
		];
		return $desc;
	}

	protected function get($input, $res, $req) {
		$this->disabledForPortal($req);

		$meta = [
			'Call' => [],
			'Meeting' => [],
			'Task' => ['has_due' => true],
		];

		$data = [];
		foreach ($meta as $model => $m) {
			$lq = new \ListQuery($model);
			$lq->addFields(['name', 'date_start', 'reminder_time']);
			$lq->addSimpleFilter('assigned_user_id', \AppConfig::current_user_id());
			$now = gmdate("Y-m-d H:i:s");
			$then = gmdate("Y-m-d H:i:s", time() + 180 * 24 * 3600);
			$clause = [
				'field' => 'date_start',
				'operator' => 'between',
				'value' => $now,
				'end' => $then,
			];
			$lq->addFilterClause($clause);
			$lq->addSimpleFilter('reminder_time', "CAST(IFNULL(reminder_time, '-1') AS SIGNED) > 0", 'literal');
			$result = $lq->runQuery(0, 200, false, null, 200);
			if (!$result || $result->failed) {
				throw new Errors\InternalError;
			}
			foreach ($result->getRowIndexes() as $i) {
				$row = $result->getRowResult($i);
				$data[] = [
					'name' => $row->getField('name'),
					'date_start' => $row->getField('date_start'),
					'id' => $row->getField('id'),
					'reminder_time' => $row->getField('reminder_time'),
					'type' => $model . 's',
				];
			}

		}

		$ret = [
			'records' => $data,
		];

		return $this->withJson($res, $ret);
	}

}


