<?php

namespace OneCRM\API\Endpoints\Data;
use OneCRM\API\Endpoint;
use OneCRM\API\Errors;

class Favorites extends Endpoint {

	public function endpoint() {
		return '/data/favorites/{model}/{id}';
	}

	public function descriptor($method) {
		if ($method == 'post')
			return $this->descriptorPost();
	}

	public function requiredScopes($method) {
		return ['write'];
	}

	public function allowedMethods() {
		return ['post'];
	}

	private function descriptorPost() {
		static $desc = [
			'title' => 'Add or remove a record from user\'s favorites',
			'docs' => '@endpoint/data/favorites_post',
			'params' => [
				'model' => [
					'location' => 'path',
					'type' => 'String',
					'min_len' => '1',
					'docs' => 'Model',
					'required' => true,
				],
				'id' => [
					'location' => 'path',
					'type' => 'String',
					'min_len' => '1',
					'docs' => 'Record ID',
					'required' => true,
				],

			],
			'return' => [
				'status' => [
					'type' => [
						'type' => 'Int',
					],
					'docs' => 'New favorite status. 0 if added, 1 if removed',
					'required' => false,
				],
			]
		];
		return $desc;
	}

	protected function post($input, $res, $req) {
		$this->disabledForPortal($req);
		$model = $input['model'];
		$module = \AppConfig::module_for_model($model);
		if (!$module)
			throw new Errors\NotFound("Model $model not found");
		Endpoint::check_module_access($module);
		require_once 'modules/Favorites/Favorite.php';
		$fav = new \Favorite;
	    $result = $fav->addOrRemoveNew($input['id'], $module);
		$ret = [
			'result' => $result,
		];

		return $this->withJson($res, $ret);
	}

}


