<?php

namespace OneCRM\API\Endpoints\Data;
use OneCRM\API\Endpoint;
use OneCRM\API\Errors;

class RelatedListItem extends Endpoint {

	public function endpoint() {
		return '/data/{model}/{id}/{link}/{rel_id}';
	}

	public function descriptor($method) {
		if ($method == 'delete')
			return $this->descriptorDelete();
	}

	public function requiredScopes($method) {
		return ['write'];
	}

	public function allowedMethods() {
		return ['delete'];
	}

	private function descriptorDelete() {
		static $desc = [
			'title' => 'Remove relationship between records',
			'docs' => '@endpoint/data/relatedlistitem_delete',
			'params' => [
				'model' => [
					'location' => 'path',
					'type' => 'String',
					'min_len' => '1',
					'docs' => 'Parent model',
					'required' => true,
				],
				'id' => [
					'location' => 'path',
					'type' => 'String',
					'min_len' => 1,
					'docs' => 'Parent record ID',
					'required' => true,
				],
				'link' => [
					'location' => 'path',
					'type' => 'String',
					'min_len' => 1,
					'docs' => 'Link name',
					'required' => true,
				],
				'rel_id' => [
					'location' => 'path',
					'type' => 'String',
					'min_len' => 1,
					'docs' => 'Related record ID',
					'required' => true,
				],
			],
			'return' => [
				'result' => [
					'type' => 'Bool',
					'docs' => 'True if deleted successfully',
					'required' => true,
				],
			]
		];
		return $desc;
	}

	protected function delete($input, $res, $req) {
		$this->disabledForPortal($req);
		$model = $input['model'];
		$module = \AppConfig::module_for_model($model);
		if (!$module)
			throw new Errors\NotFound("Model $model not found");
		Endpoint::check_module_access($module);
		$record = \ListQuery::quick_fetch($model, $input['id'], true, ['acl_filters' => 'edit']);
		if (!$record)
			throw new Errors\NotFound;
		$link = $input['link'];
		$upd = \RowUpdate::for_result($record);
		$result = !!$upd->removeLinks($link, [$input['rel_id']]);
		$ret = ['result' => $result];

		return $this->withJson($res, $ret);
	}

}


