<?php

namespace OneCRM\API\Endpoints\Files;

use OneCRM\API\Endpoint;
use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Message\ResponseInterface as Response;
use Slim\Psr7\Stream;

class Download extends Endpoint {

	public function endpoint() {
		return '/files/download/{model}/{id}';
	}

	public function descriptor($method) {
		if ($method == 'get')
			return $this->descriptorGet();
	}

	public function requiredScopes($method) {
		return ['read'];
	}

	public function allowedMethods() {
		return ['get'];
	}

	private function descriptorGet() {
		static $desc = [
			'title' => 'Download a file',
			'docs' => '@endpoint/files/download',
			'params' => [
				'model' => [
					'location' => 'path',
					'type' => [
						'type' => 'Enum',
						'options' => ['Note', 'Document', 'DocumentRevision'],
					],
					'docs' => 'Specifies model',
					'required' => true,
				],
				'id' => [
					'location' => 'path',
					'type' => 'String',
					'min_len' => 1,
					'docs' => 'Specifies ID',
					'required' => true,
				],
			]
		];
		return $desc;
	}

	public function getUnchecked($input, Response $res, Request $req) {
		$input['unchecked'] = true;
		return $this->get($input, $res, $req);
	}

	protected function get($input, Response $res, Request $req) {
		$this->disabledForPortal($req);
		$meta = new Metadata($input['model'], $input['id'], array_get_default($input, 'unchecked'));
		$ret = $meta->getMetadata();
		$f = @fopen($ret['path'], 'r');
		if (!$f)
			throw new InternalError;
		$name = download_safe_filename($ret['name']);
		$res = $res
			->withHeader("Content-Disposition", "inline; filename=\"".$name."\";")
			->withHeader('Content-Type', $ret['mimetype'])
			->withHeader('Content-Length', $ret['size'])
			->withHeader('X-OneCRM-Filesize', $ret['size'])
			->withHeader('X-OneCRM-Filename', $ret['name'])
			->withBody(new Stream($f))
		;
		return $res;
	}
}


