<?php

namespace OneCRM\API\Endpoints\Files;
use OneCRM\API\Endpoint;
use OneCRM\API\Errors;

class Upload extends Endpoint {

	public function endpoint() {
		return '/files/upload';
	}

	public function descriptor($method) {
		if ($method == 'post')
			return $this->descriptorPost();
	}

	public function requiredScopes($method) {
		return ['write'];
	}

	public function allowedMethods() {
		return ['post'];
	}

	private function descriptorPost() {
		static $desc = [
			'title' => 'Upload a file',
			'docs' => '@endpoint/files/upload',
			'params' => [
				'CONTENT-TYPE' => [
					'location' => 'header',
					'type' => 'String',
					'required' => false,
					'default' => 'application/octet-stream',
					'docs' => 'File content type',
				],
				'CONTENT-LENGTH' => [
					'location' => 'header',
					'type' => 'Int',
					'required' => true,
					'docs' => 'File size',
					'min' => 0,
				],
				'X-ONECRM-FILENAME' => [
					'location' => 'header',
					'type' => 'Filename',
					'min_len' => '1',
					'required' => true,
					'docs' => 'File name',
				],
			],
			'return' => [
				'id' => [
					'type' => 'String',
					'docs' => 'Uploaded file ID',
					'required' => true,
				],
			]
		];
		return $desc;
	}

	protected function post($input, $res, $req) {
		$this->disabledForPortal($req);
		require_once 'include/upload_file.php';
		$name = $input['X-ONECRM-FILENAME'] ?? '';
		$f = $req->getBody();
		$f->rewind();
		$tmpfname = tempnam(sys_get_temp_dir(), "OneCRMUpload");
		$dst = fopen($tmpfname, 'w');
		if (!$dst) {
			@unlink($tmpfname);
			throw new Errors\Internal;
		}
		while (!$f->eof()) {
			$chunk = $f->read(65536);
			if (fwrite($dst, $chunk) === false) {
				$f->close();
				@fclose($dst);
				@unlink($tmpfname);
				throw new Errors\Internal;
			}
		}

		$f->close();
		@fclose($dst);
		$fl = new \UploadFile('');
		$fl->source_file_path = $tmpfname;
		$fl->stored_file_name = $name;
		$fl->mime_type = $input['CONTENT-TYPE'];
		$id = $fl->save_incoming(\AppConfig::current_user_id());
		$ret = ['id' => 'INCOMING~' . $id];

		return $this->withJson($res, $ret);
	}

	public function expectsParsedBody() {
		return false;
	}

}


