<?php

namespace OneCRM\API\Endpoints\Files;
use OneCRM\API\Endpoint;
use OneCRM\API\Errors;

class UploadBase64 extends Endpoint {

	public function endpoint() {
		return '/files/upload/base64';
	}

	public function descriptor($method) {
		if ($method == 'post')
			return $this->descriptorPost();
	}

	public function requiredScopes($method) {
		return ['write'];
	}

	public function allowedMethods() {
		return ['post'];
	}

	private function descriptorPost() {
		static $desc = [
			'title' => 'Upload a small file',
			'docs' => '@endpoint/files/upload_base64',
			'params' => [
				'filename' => [
					'location' => 'body',
					'type' => 'Filename',
					'min_len' => '1',
					'docs' => 'File name',
					'required' => true,
				],
				'mimetype' => [
					'location' => 'body',
					'type' => 'String',
					'min_len' => '1',
					'docs' => 'File MIME type',
					'required' => true,
				],
				'data' => [
					'location' => 'body',
					'type' => 'String',
					'docs' => 'File data, `base64` encoded',
					'required' => true,
				],
			],
			'return' => [
				'id' => [
					'type' => 'String',
					'docs' => 'Uploaded file ID',
					'required' => true,
				],
			]
		];
		return $desc;
	}

	protected function post($input, $res, $req) {
		$this->disabledForPortal($req);
		require_once 'include/upload_file.php';
		$name = $input['filename'];
		if ($name === '.' || preg_match('!(\.\.)|\\\\|/!', $name)) {
			throw new Errors\BadRequest('invalid filename');
		}
		$type = $input['mimetype'];
		$data = @base64_decode($input['data'], true);
		if ($data === false) {
			throw new Errors\BadRequest('invalid base64 data');
		}
		$fl = new \UploadFile('');
		$fl->use_soap = true;
		$fl->stored_file_name = $name;
		$fl->mime_type = $type;
		$fl->file = $data;
		$id = $fl->save_incoming(\AppConfig::current_user_id());
		$ret = ['id' => 'INCOMING~' . $id];

		return $this->withJson($res, $ret);
	}

}


