<?php

namespace OneCRM\API\Endpoints\Meta;
use OneCRM\API\Endpoint;
use OneCRM\API\Errors;

class Fields extends Endpoint {

	public function endpoint() {
		return '/meta/fields/{model}';
	}

	public function descriptor($method) {
		if ($method == 'get')
			return $this->descriptorGet();
	}

	public function requiredScopes($method) {
		return ['read'];
	}

	public function allowedMethods() {
		return ['get'];
	}

	private function descriptorGet() {
		static $desc = [
			'title' => 'Get fields definition for a model',
			'docs' => '@endpoint/meta/fields_get',
			'params' => [
				'model' => [
					'location' => 'path',
					'type' => 'String',
					'min_len' => '1',
					'docs' => 'Model to query',
					'required' => true,
				],
			],
			'return' => [
				'fields' => [
					'type' => 'Array',
					'docs' => 'An array with fields definitions',
					'required' => true,
				],
				'filters' => [
					'type' => 'Array',
					'docs' => 'An array with filters definitions',
					'required' => true,
				]
			]
		];
		return $desc;
	}

	protected function get($input, $res, $req) {
		$model = $input['model'];
		$module = \AppConfig::module_for_model($model);
		if (!$module)
			throw new Errors\NotFound("Model $model not found");
		$fields = \AppConfig::setting("model.fields.$model");
		$links = \AppConfig::setting("model.links.$model");
		if (!$fields)
			throw new Errors\NotFound("No fields for model $model");
		$return_fields = [];

		foreach($links as $k => $v) {
		    if($k == 'lines_link') {
		        $ln_arr['name'] = 'line_items';
		        $ln_arr['vname'] = 'Line Items';
		        $ln_arr['type'] = 'array';
		        $ln_arr['mode'] = 'choose';
		        $ln_arr['validate'] = ['minItems' => 1];
		        $ln_arr['labels'] = ['add' => 'Add Line Item'];

		        $line_bean = $v['bean_name'];
		        $line_fields = \AppConfig::setting("model.fields.$line_bean");

		        $ln_fields = [];
		        foreach ($line_fields as $k => &$f) {
                    if (!array_get_default($f, 'api_exposed', true))
                        continue;
                    if (array_get_default($f, 'type') == 'reminder') {
                        $f['type'] = 'enum';
                        $f['options'] = 'reminder_time_options';
                    }
                    if (isset($f['vname'])) {
                        $f['vname'] = translate($f['vname'], array_get_default($f, 'vname_module', $module));
                    }
                    if (isset($f['vname_list'])) {
                        $f['vname_list'] = translate($f['vname_list'], array_get_default($f, 'vname_module', $module));
                    }
                    if (isset($f['vdesc'])) {
                        $f['vdesc'] = translate($f['vdesc'], array_get_default($f, 'vdesc_module', $module));
                    }
                    if (isset($f['options'])) {
                        $dom = $f['options'];
                        if(is_array($dom)) {
                            $f['options'] = $dom;
                        } else {
                            $opts = \AppConfig::setting('lang.lists.current.' . $module . '.' . $dom);
                            if (!is_array($opts)) {
                                $opts = isset($GLOBALS['app_list_strings'][$dom]) ? $GLOBALS['app_list_strings'][$dom] : array();
                            }
                            $opts_list = array();
                            foreach ($opts as $ok => $ov) {
                                $opts_list[] = ["label" => $ov, "value" => (string)$ok];
                            }
                            $f['options'] = $opts_list;
                        }
                    }
                    if($k == 'name')
                    	$f['vname'] = 'Line Item Description';
                    if($k == 'unit_price' || $k == 'name' || $k == 'quantity' || $k == 'mfr_part_no')
                        $ln_fields[] = $f;
                }
		        $ln_fields[] = array("name" => "line_tax_perc", "vname" => "Line Item Tax Percentage", "type" => "number");
		        $ln_fields[] = array("name" => "line_tax_amount", "vname" => "Line Item Tax Amount", "type" => "currency");
		        $ln_fields[] = array("name" => "line_discount_perc", "vname" => "Line Item Discount Percentage", "type" => "currency");
		        $ln_fields[] = array("name" => "line_discount_amount", "vname" => "Line Item Discount Amount", "type" => "currency");
		        foreach($ln_fields as &$line_f) {
		        	$line_f['key'] = $line_f['name'];
				}
		        $ln_arr['spec'] = $ln_fields;

		        $return_fields['line_items'] = $ln_arr;

		        $return_fields['total_tax'] = array("name" => "total_tax", "vname" => "Total Tax", "type" => "currency");
		        $return_fields['total_discount'] = array("name" => "total_discount", "vname" => "Total Discount", "type" => "currency");
		        $return_fields['total_shipping'] = array("name" => "total_shipping", "vname" => "Total Shipping", "type" => "currency");
            }
        }

		foreach ($fields as $k => &$f) {
			if (!array_get_default($f, 'api_exposed', true))
				continue;
			if (array_get_default($f, 'type') == 'reminder') {
				$f['type'] = 'enum';
				$f['options'] = 'reminder_time_options';
			}
			if (isset($f['vname'])) {
				$f['vname'] = translate($f['vname'], array_get_default($f, 'vname_module', $module));
			}
			if (isset($f['vname_list'])) {
				$f['vname_list'] = translate($f['vname_list'], array_get_default($f, 'vname_module', $module));
			}
			if (isset($f['vdesc'])) {
				$f['vdesc'] = translate($f['vdesc'], array_get_default($f, 'vdesc_module', $module));
			}
			if (isset($f['options'])) {
				$dom = $f['options'];
				if(is_array($dom)) {
					$f['options'] = $dom;
				} else {
					$opts = \AppConfig::setting('lang.lists.current.' . $module . '.' . $dom);
					if (!is_array($opts)) {
						$opts = isset($GLOBALS['app_list_strings'][$dom]) ? $GLOBALS['app_list_strings'][$dom] : array();
					}
					$opts_list = array();
					foreach ($opts as $ok => $ov) {
						$opts_list[] = ["label" => $ov, "value" => (string)$ok];
					}
					$f['options'] = $opts_list;
				}
			}
			$return_fields[$k] = $f;
		}

		require_once('include/ListView/ListFormatter.php');
		$fmt = new \ListFormatter($model);

		$filter =& $fmt->getFilterForm();
		$filter->loadFilterLayout('Standard');

		$options = $filter->getFilterOptions();
		foreach ($options as $k => &$f) {
			if (!array_get_default($f, 'api_exposed', true))
				continue;
			if (isset($f['vname'])) {
				$f['vname'] = translate($f['vname'], array_get_default($f, 'vname_module', $module));
			}
			if (isset($f['vname_list'])) {
				$f['vname_list'] = translate($f['vname_list'], array_get_default($f, 'vname_module', $module));
			}
			if (isset($f['vdesc'])) {
				$f['vdesc'] = translate($f['vdesc'], array_get_default($f, 'vdesc_module', $module));
			}
			if (isset($f['options'])) {
				$dom = $f['options'];
				if(is_array($dom)) {
					foreach ($dom as &$val) {
						if (is_array($val)) {
							$val = translate($val['vname'], array_get_default($val, 'vname_module', $module));
						}
					}
					$f['options'] = $dom;
				} else {
					$opts = \AppConfig::setting('lang.lists.current.' . $module . '.' . $dom);
					if (!is_array($opts)) {
						$opts = isset($GLOBALS['app_list_strings'][$dom]) ? $GLOBALS['app_list_strings'][$dom] : array();
					}
					$f['options'] = $opts;
				}
			}
		}

		return $this->withJson($res, ['fields' => $return_fields, 'links' => $links, 'filters' => $options]);
	}

}


