<?php

namespace OneCRM\API\Endpoints\Meta;
use OneCRM\API\Endpoint;
use OneCRM\API\Errors;


class GenericModuleLayouts extends Endpoint {

	public function endpoint() {
		return '/meta/generic_module_layouts/{module}';
	}

	public function descriptor($method) {
		if ($method == 'get')
			return $this->descriptorGet();
	}

	public function requiredScopes($method) {
		return ['read'];
	}

	public function allowedMethods() {
		return ['get'];
	}

	private function descriptorGet() {
		static $desc = [
			'title' => 'Get generic module metadata',
			'docs' => '@endpoint/meta/generic_module_get',
			'params' => [
				'module' => [
					'location' => 'path',
					'type' => 'String',
					'min_len' => '1',
					'docs' => 'Module',
					'required' => true,
				],
			],
			'return' => [
				'list' => [
					'required' => true,
					'type' => 'Array',
					'element_type' => [
						'type' => 'Object',
					],
					'docs' => 'Default ListView layout and view',
				],
				'popup' => [
					'required' => true,
					'type' => 'Array',
					'element_type' => [
						'type' => 'Object',
					],
					'docs' => 'Default Popup layout and view',
				],
				'detail' => [
					'required' => true,
					'type' => 'Array',
					'element_type' => [
						'type' => 'Object',
					],
					'docs' => 'Default DetailView layout and view',
				],
			]
		];

		return $desc;
	}

	protected function get($input, $res, $req) {
		$module = $input['module'];

		if (!$module)
			throw new Errors\NotFound("Module {$module} not found");

		$model = \AppConfig::module_primary_bean($module);

		if (!$model)
			throw new Errors\NotFound("Model for {$module} not found");

		$result = ['list' => [], 'popup' => [], 'detail' => ''];

		require_once('include/ListView/ListViewManager.php');
		$manager = \ListViewManager::create('listview', ['is_primary' => true, 'module' => $module]);
		$manager->initModuleView($module);

		$result['list'] = [
			'layout_name' => $manager->layout_name,
			'view_name' => $manager->view_name,
			'view_type' => $manager->view_type,
			'filter_name' => $manager->filter_name
		];

		require_once('include/Popups/PopupManager.php');
		$manager = new \PopupManager();
		$manager->initModuleView($module);

		$result['popup'] = [
			'layout_name' => $manager->layout_name,
			'view_name' => $manager->view_name,
			'view_type' => $manager->view_type,
			'filter_name' => $manager->filter_name
		];

		require_once 'include/utils/misc_functions.php';
		require_once('include/DetailView/DetailFactory.php');
		require_once('include/ListView/ListFormatter.php');

		/** @var \StandardDetailManager $mgr */
		$manager = \DetailFactory::getManager('standard', $module);
		$manager->setModel($model);

		try {
			$manager->standardInit();
		} catch(\Exception $e) {
			$GLOBALS['log']->error("API Generic metadata call: DetailManager init error - {$e->getMessage()}");
		}

		$result['detail'] = [
			'layout_name' => $manager->layout->name,
			'view_name' => $manager->layout->name,
			'view_type' => $manager->layout->type,
		];
		
		return $this->withJson($res, $result);
	}
}
