<?php

namespace OneCRM\API\Endpoints\Meta;
use OneCRM\API\Endpoint;
use OneCRM\API\Errors;

class Locale extends Endpoint {

	public function endpoint() {
		return '/meta/locale';
	}

	public function descriptor($method) {
		if ($method == 'get')
			return $this->descriptorGet();
	}

	public function requiredScopes($method) {
		return ['read'];
	}

	public function allowedMethods() {
		return ['get'];
	}

	private function descriptorGet() {
		static $desc = [
			'title' => 'Get locale-related information',
			'docs' => '@endpoint/meta/locale',
			'params' => [
			],
			'return' => [
				'date_format' => [
					'required' => true,
					'type' => 'String',
					'docs' => 'Date format',
				],
				'time_format' => [
					'required' => true,
					'type' => 'String',
					'docs' => 'Time format',
				],
				'address_format' => [
					'required' => true,
					'type' => 'Object',
					'docs' => 'Address format',
					'schema' => [
						'display' => [
							'type' => 'String',
							'docs' => 'Address format descriptive name',
							'required' => true,
						],
						'format' => [
							'type' => 'String',
							'docs' => 'Address format',
							'required' => true,
						],
					]
				],
				'number_format' => [
					'required' => true,
					'type' => 'Object',
					'docs' => 'Number format',
					'schema' => [
						'display' => [
							'type' => 'String',
							'docs' => 'Number format descriptive name',
							'required' => true,
						],
						'dec_sep' => [
							'type' => 'String',
							'docs' => 'Deciamal part separator',
							'required' => true,
						],
						'grp_sep' => [
							'type' => 'String',
							'docs' => 'Groups separator',
							'required' => true,
						],
					]
				],
				'base_currency' => [
					'required' => true,
					'type' => 'Object',
					'docs' => 'Base currency',
					'schema' => [
						'name' => [
							'type' => 'String',
							'required' => true,
							'docs' => 'Currency name',
						],
						'iso4217' => [
							'type' => 'String',
							'required' => true,
							'docs' => 'Currency code according to ISO 4217',
						],
						'symbol' => [
							'type' => 'String',
							'required' => true,
							'docs' => 'Currency symbol',
						],
						'significant_digits' => [
							'type' => 'Int',
							'required' => true,
							'docs' => 'Number of symbols after decimal point',
						],
						'symbol_place_after' => [
							'type' => 'Bool',
							'required' => true,
							'docs' => '`true` if symbol must be placed after amount in monetary values, `false` if the symbol goes before amount',
						],
						'space_separate' => [
							'type' => 'Bool',
							'required' => true,
							'docs' => '`true` if a space is required between numeric value and currency symbol',
						],
					],
				],
			]
		];
		return $desc;
	}

	protected function get($input, $res, $req) {
		$address_format = \AppConfig::setting('locale.defaults.address_format');
		$number_format = \AppConfig::setting('locale.defaults.number_format');
		$meta = [
			'date_format' => \AppConfig::setting('locale.defaults.date_format'),
			'time_format' => \AppConfig::setting('locale.defaults.time_format'),
			'timezone' => \AppConfig::system_timezone(),
			'address_format' => [
				'display' => $address_format,
				'format' =>  \AppConfig::setting("locale.address_formats.$address_format.display"),
			],
			'number_format' => [
				'display' => $number_format,
				'dec_sep' => \AppConfig::setting("locale.number_formats.$number_format.dec_sep"),
				'grp_sep' => \AppConfig::setting("locale.number_formats.$number_format.grp_sep"),
			],
			'base_currency' => [
				'name' => \AppConfig::setting('locale.base_currency.name'),
				'iso4217' => \AppConfig::setting('locale.base_currency.iso4217'),
				'symbol' => \AppConfig::setting('locale.base_currency.symbol'),
				'significant_digits' => \AppConfig::setting('locale.base_currency.significant_digits', 2),
				'space_separate' => \AppConfig::setting('locale.base_currency.space_separate', true),
				'symbol_place_after' => \AppConfig::setting('locale.base_currency.symbol_place_after', false),
			],
		];

		return $this->withJson($res, $meta);
	}

}


