<?php

namespace OneCRM\API\Endpoints\Meta;
use OneCRM\API\Endpoint;

class Modules extends Endpoint {

	public function endpoint() {
		return '/meta/modules';
	}

	public function descriptor($method) {
		if ($method == 'get')
			return $this->descriptorGet();
	}

	public function requiredScopes($method) {
		return ['read'];
	}

	public function allowedMethods() {
		return ['get'];
	}

	private function descriptorGet() {
		static $desc = [
			'title' => 'Get list of available modules',
			'docs' => '@endpoint/meta/modules',
			'params' => [
			],
			'return' => [
				'list' => [
					'type' => [
						'type' => 'Array',
						'element_type' => [
							'type' => 'Object',
							'schema' => [
								'module' => [
									'type' => 'String',
									'docs' => 'Module name',
									'required' => true,
								],
								'primary_model' => [
									'type' => 'String',
									'docs' => 'Module\'s primary model',
									'required' => true,
								],
								'models' => [
									'type' => [
										'type' => 'Array',
										'element_type' => [
											'type' => 'String',
											'docs' => 'Model name'
										]
									],
									'docs' => 'Models',
									'required' => true,
								],
							]
						],
					],
					'docs' => 'List of modules',
					'required' => true,
				],
			]
		];
		return $desc;
	}

	protected function get($input, $res, $req) {
		$beans = \AppConfig::setting("model.index.by_type.bean");
		$available_modules = [];

		foreach ($beans as $b) {
			$available_modules[] = \AppConfig::module_for_model($b);
		}

		$available_modules = array_unique($available_modules);
		\ACLController::filterModuleList($available_modules);

		$disabled = self::$disabled_modules;
		$beans = array_filter($beans, function ($item) use ($disabled, $available_modules) {
			$module = \AppConfig::module_for_model($item);
			return ! in_array($module, $disabled) && in_array($module, $available_modules);
 		});

		sort($beans);

		$list = [];
		foreach ($beans as $item) {
			$module = \AppConfig::module_for_model($item);
			$list[$module][] = $item;
		}

		$result = [];
		array_walk($list, function ($item, $key) use (&$result) {
			$result[] = [
				'module' => $key,
				'primary_model' => \AppConfig::module_primary_bean($key),
				'models' => $item,
			];
		});


		$list = ['list' => $result];

		return $this->withJson($res, $list);
	}
}


