<?php

namespace OneCRM\API\Endpoints\Printer;

use OneCRM\API\Endpoint;
use OneCRM\API\Errors;
use Slim\Psr7\Stream;

require_once 'modules/LicenseInfo/LicenseHistory.php';
require_once 'modules/PDFTemplates/utils.php';
require_once 'modules/PDFTemplates/PDFTemplatePrinter.php';

class Pdf extends Endpoint {

	public function endpoint() {
		return '/printer/pdf/{model}/{id}';
	}

	public function descriptor($method) {
		if ($method == 'get')
			return $this->descriptorGet();
	}

	public function requiredScopes($method) {
		return ['read'];
	}

	public function allowedMethods() {
		return ['get'];
	}

	private function descriptorGet() {
		static $desc = [
			'title' => 'Print PDF',
			//'docs' => '@endpoint/printer/pdf',
			'params' => [
				'model' => [
					'location' => 'path',
					'type' => 'String',
					'min_len' => '1',
					'docs' => 'Record model',
					'required' => true,
				],
				'id' => [
					'location' => 'path',
					'type' => 'String',
					'min_len' => 1,
					'docs' => 'Record ID',
					'required' => true,
				],
			]
		];
		return $desc;
	}

	protected function get($input, $res, $req) {
		$this->disabledForPortal($req);
		$model = $input['model'];
		$module = \AppConfig::module_for_model($model);
		$id = $input['id'];

		if (!$module)
			throw new Errors\NotFound("Model $model not found");
		if (!$id)
			throw new Errors\NotFound("Empty ID");

		$tpl = PDFTemplate_get_default($module);

		if (empty($tpl))
			throw new Errors\NotFound("PDF Template Not Found for $model");

		$printer = new \PDFTemplatePrinter($tpl->getField('id'), $id);
		$pdf = $printer->render();

		$temp = tempnam(sys_get_temp_dir(), 'PDF');
		$pdf->Output($temp, 'F');

		$f = @fopen($temp, 'r');
		if (!$f)
			throw new Errors\InternalError;

		$name = download_safe_filename($printer->getFileName());
		$size = filesize($temp);

		$res = $res
			->withHeader("Content-Disposition", "inline; filename=\"".$name."\";")
			->withHeader('Content-Type', 'application/pdf')
			->withHeader('Content-Length', $size)
			->withHeader('X-OneCRM-Filesize', $size)
			->withHeader('X-OneCRM-Filename', $printer->getFileName())
			->withBody(new Stream($f))
		;

		return $res;
	}
}

