<?php

namespace OneCRM\API\Endpoints\Printer;

use OneCRM\API\Endpoint;
use OneCRM\API\Errors;
use Slim\Psr7\Stream;

require_once 'modules/LicenseInfo/LicenseHistory.php';
require_once('modules/DataPrivacy/PersonalInfoViewer.php');

class PersonalDataPdf extends Endpoint {

	public function endpoint() {
		return '/printer/pdf/personal/{model}/{id}';
	}

	public function descriptor($method) {
		if ($method == 'get')
			return $this->descriptorGet();
	}

	public function requiredScopes($method) {
		return ['read'];
	}

	public function allowedMethods() {
		return ['get'];
	}

	private function descriptorGet() {
		static $desc = [
			'title' => 'Print Personal Data PDF',
			//'docs' => '@endpoint/printer/pdf/personal',
			'params' => [
				'model' => [
					'location' => 'path',
					'type' => 'String',
					'min_len' => '1',
					'docs' => 'Record model',
					'required' => true,
				],
				'id' => [
					'location' => 'path',
					'type' => 'String',
					'min_len' => 1,
					'docs' => 'Record ID',
					'required' => true,
				],
			]
		];
		return $desc;
	}

	protected function get($input, $res, $req) {
		$this->disabledForPortal($req);
		$model = $input['model'];
		$module = \AppConfig::module_for_model($model);
		$id = $input['id'];

		if (!$module || !\AppConfig::setting("model.detail.{$model}.personal_data"))
			throw new Errors\NotFound("Model $model not found");
		if (!$id)
			throw new Errors\NotFound("Empty ID");

		require_once('modules/DataPrivacy/PersonalInfoViewer.php');
		$viewer = new \PersonalInfoViewer($module, $id);
		$pdf_file = $viewer->generatePdfFile();

		$f = @fopen($pdf_file, 'r');
		if (!$f)
			throw new Errors\InternalError;

		$name = download_safe_filename('PersonalData.pdf');
		$size = filesize($pdf_file);

		$res = $res
			->withHeader("Content-Disposition", "inline; filename=\"".$name."\";")
			->withHeader('Content-Type', 'application/pdf')
			->withHeader('Content-Length', $size)
			->withHeader('X-OneCRM-Filesize', $size)
			->withHeader('X-OneCRM-Filename', 'PersonalData.pdf')
			->withBody(new Stream($f))
		;

		return $res;
	}
}

