<?php

namespace OneCRM\API\Endpoints\Profile;

use OneCRM\API\Endpoint;


require_once 'modules/LicenseInfo/LicenseHistory.php';

class Me extends Endpoint {

	public function endpoint() {
		return '/me';
	}

	public function descriptor($method) {
		if ($method == 'get')
			return $this->descriptorGet();
	}

	public function requiredScopes($method) {
		return ['profile'];
	}

	public function allowedMethods() {
		return ['get'];
	}

	private function descriptorGet() {
		static $desc = [
			'title' => 'Get information about authenticated user',
			'params' => [
			],
			'return' => [
				'id' => [
					'type' => 'String',
					'docs' => 'User ID',
					'required' => true,
				],
				'name' => [
					'type' => 'String',
					'docs' => 'User name',
					'required' => true,
				],
				'first_name' => [
					'type' => 'String',
					'docs' => 'First name',
				],
				'last_name' => [
					'type' => 'String',
					'docs' => 'Last name',
				],
				'role' => [
					'type' => 'String',
					'docs' => 'User role',
				],
				'groups' => [
					'type' => 'Array',
					'docs' => 'User groups',
				],
				'email' => [
					'type' => 'String',
					'docs' => 'User email',
				],
				'timezone' => [
					'type' => 'String',
					'docs' => 'User\'s timezone',
				],
			]
		];
		return $desc;
	}

	protected function get($input, $response, $request) {
		$params = [
			// this keeps JSON reply as an object instead of an array if user is not found
			'dummy' => true,
		];
		$container = $this->app->getContainer();
		/** @var UserEntity $userEntity */
		$uid = $request->getAttribute('oauth_user_id');
		if (!$uid)
			$uid = 'Users:' . \AppConfig::current_user_id();

		$userEntity = $container->get('UserRepository')->getUserEntityById($uid);

		if ($userEntity) {
			$params = [
				'id' => $userEntity->getIdentifier(),
				'name' => $userEntity->getUsername(),
				'first_name' => $userEntity->getFirstName(),
				'last_name' => $userEntity->getLastName(),
				'role' => $userEntity->getRole(),
				'groups' => $userEntity->getGroups(),
				'email' => $userEntity->getEmail(),
				'timezone' => $userEntity->getTimezone(),
				'is_partner' => $userEntity->getIsPartner(),
			];
		}

		return $this->withJson($response, $params);
	}
}

