<?php

namespace OneCRM\API\Endpoints\Tally;

class GroupFormatter {

	static private $copy_group_fields = [
		'name', 'id',
	];

	static private $copy_line_fields = [
		'id', 'related_type', 'related_id',
		'sum_of_components', 'is_comment',
		'quantity', 'ext_quantity', 'parent_id',
	];

	static private $copy_line_fields_Invoice = [
		'unit_price', 'unit_price_usd',
		'ext_price', 'ext_price_usd',
		'mfr_part_no', 'vendor_part_no',
		'serial_no', 'serial_numbers', 'weight',
		'list_price', 'list_price_usd','tax_class_id','cost_price'
	];

	static private $copy_line_fields_Shipping = [
		'mfr_part_no', 'vendor_part_no',
		'serial_no', 'serial_numbers', 'weight',
	];

	static private $copy_group_fields_Invoice = [
		'group_type', 'status', 'pricing_method',
		'cost', 'subtotal', 'total',
		'cost_usd', 'subtotal_usd', 'total_usd',
	];

	public function formatGroup($group, $model) {
		$result = [];
		$this->copyFields($group, $result, self::$copy_group_fields);
		$method = 'formatGroup_' . $model;
		if (method_exists($this, $method)) {
			$this->$method($group, $result);
		}
		$self = $this;
		$result['line_items'] = array_map(
			function($line) use ($group, $model, $self) {
				return $self->formatLine($group, $line, $model);
			},
			array_filter(
				array_values($group['lines']),
				function($line) {
					return empty($line['parent_id']) || !empty($line['is_comment']);
				}
			)
		);
		return $result;
	}

	public function formatLine($group, $line, $model) {
		$result = [];
		$this->copyFields($line, $result, self::$copy_line_fields);
		$result['name'] = (isset($line['is_comment']) && $line['is_comment']) ? $line['body'] : $line['name'];
		$method = 'formatLine_' . $model;
		if (method_exists($this, $method)) {
			$this->$method($group, $line, $result);
		}
		if (!empty($line['sum_of_components'])) {
			foreach ($group['lines'] as $_ => $l) {
				if ($l['parent_id'] == $line['id']) {
					$result['components'][] = $this->formatLine($group, $l, $model);
				}
			}
		}
		return $result;
	}

	public function formatLine_Shipping($group, $line, &$result) {
		$this->copyFields($line, $result, self::$copy_line_fields_Shipping);
	}

	public function formatLine_Receiving($group, $line, &$result) {
		return $this->formatLine_Shipping($group, $line, $result);
	}

	public function formatLine_Invoice($group, $line, &$result) {
		$this->copyFields($line, $result, self::$copy_line_fields_Invoice);
		if(! empty($line['pricing_adjust_id'])) {
			$prc_id = $line['pricing_adjust_id'];
			if (!empty($group['adjusts'][$prc_id])) {
				$prc_row = $group['adjusts'][$prc_id];
				if($prc_row['type'] == 'PercentDiscount' || $prc_row['type'] == 'StdPercentDiscount')
					$result['discount'] = [
						'type' => 'percent',
						'value' => $prc_row['rate'],
					];
				else if($prc_row['type'] == 'StdFixedDiscount' && $prc_row['amount'])
					$result['discount'] = [
						'type' => 'amount',
						'value' => $prc_row['amount'],
					];
			}
		}
	}
	
	public function formatLine_Quote($group, $line, &$result) {
		return $this->formatLine_Invoice($group, $line, $result);
	}

	public function formatLine_SalesOrder($group, $line, &$result) {
		return $this->formatLine_Invoice($group, $line, $result);
	}

	public function formatLine_Bill($group, $line, &$result) {
		return $this->formatLine_Invoice($group, $line, $result);
	}

	public function formatLine_PurchaseOrder($group, $line, &$result) {
		return $this->formatLine_Invoice($group, $line, $result);
	}

	public function formatLine_CreditNote($group, $line, &$result) {
		return $this->formatLine_Invoice($group, $line, $result);
	}

	public function formatGroup_Invoice($group, &$result) {
		$this->copyFields($group, $result, self::$copy_group_fields_Invoice);
		$adjusts = $this->findGroupAdjusts($group);
		if(! empty($group['adjusts'])) {
			$pretax = $group['subtotal'];
			foreach($group['adjusts'] as $idx => $adj) {
				if (!empty($adj['line_id']))
					continue;
				$row = [
					'type' => $adj['type']
				];
				$row['name'] = $adj['name'];
				if($adj['type'] != 'StdFixedDiscount') {
					if(isset($row['rate'])) {
						$row['rate'] .= $adj['rate'];
					} else {
						$row['rate'] = $adj['rate'];
					}
				}
				if($adj['type'] == 'StandardDiscount' || $adj['type'] == 'StdPercentDiscount' || $adj['type'] == 'StdFixedDiscount') {
					$pretax -= $adj['amount'];
					$adj['amount'] = -$adj['amount'];
				}
				$val = $adj['amount'];
				if($adj['type'] == 'StandardDiscount' || $adj['type'] == 'StdPercentDiscount' || $adj['type'] == 'StdFixedDiscount') {
					$row['value'] = $val;
				}
				else if($adj['type'] == 'StandardTax' || $adj['type'] == 'CompoundedTax') {
					$row['value'] = $val;
				}
				else if($adj['type'] == 'TaxedShipping' || $adj['type'] == 'UntaxedShipping') {
					if($adj['amount'] != 0) {
						$row['value'] = $val;
					}
					$shipping_taxed = ($adj['type'] == 'TaxedShipping');
					if($shipping_taxed) {
						$pretax += $adj['amount'];
					}
				}
				$result['adjusts'][] = $row;
			}
		}
	}

	public function formatGroup_Quote($group, &$result) {
		return $this->formatGroup_Invoice($group, $result);
	}

	public function formatGroup_SalesOrder($group, &$result) {
		return $this->formatGroup_Invoice($group, $result);
	}

	public function formatGroup_Bill($group, &$result) {
		return $this->formatGroup_Invoice($group, $result);
	}

	public function formatGroup_PurchaseOrder($group, &$result) {
		return $this->formatGroup_Invoice($group, $result);
	}

	public function formatGroup_CreditNote($group, &$result) {
		return $this->formatGroup_Invoice($group, $result);
	}

	private function copyFields($from, &$to, $fields) {
		foreach ($fields as $f) {
			$to[$f] = $from[$f] ?? null;
		}
	}

	private function findLineAdjusts($group, $line_id) {
		$adjusts = [];
		foreach ($group['adjusts'] as $a) {
			if ($a['line_id'] === $line_id) {
				$adjusts[] = $a;
			}
		}
		return $adjusts;
	}

	private function findGroupAdjusts($group) {
		$adjusts = [];
		foreach ($group['adjusts'] as $a) {
			if (empty($a['line_id'])) {
				$adjusts[] = $a;
			}
		}
		return $adjusts;
	}

}

