<?php

namespace OneCRM\API\Endpoints\Util;
use OneCRM\API\Endpoint;

require_once 'modules/LicenseInfo/LicenseHistory.php';

class Version extends Endpoint {

	public function endpoint() {
		return '/version';
	}

	public function descriptor($method) {
		if ($method == 'get')
			return $this->descriptorGet();
	}

	public function requiredScopes($method) {
		return [];
	}

	public function allowedMethods() {
		return ['get'];
	}

	public function requiresAuthentication() {
		return false;
	}

	private function descriptorGet() {
		static $desc = [
			'title' => 'Get 1CRM version',
			'docs' => 'Returns 1CRM version. Can be used to validate login info',
			'params' => [
			],
			'return' => [
				'version' => [
					'type' => 'String',
					'docs' => '1CRM version',
					'required' => true,
				],
				'products' => [
					'type' => [
						'type' => 'Array',
						'element_type' => [
							'type' => 'String',
						],
					],
					'docs' => 'List if licensed products',
					'required' => true,
				],
				'authenticated' => [
					'type' => 'Bool',
					'docs' => 'True if the request contains valid authentication header',
					'required' => true,
				]
			]
		];
		return $desc;
	}

	protected function get($input, $res, $req) {
		$data = [
			'version' => \AppConfig::version(),
			'authenticated' => !!$req->getAttribute('isAuthenticated'),
			'products' => \LicenseHistory::weather('PRODUCTS'),
		];

		return $this->withJson($res, $data);
	}
}


