<?php

namespace OneCRM\API\Middleware;

use AppConfig;
use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Server\RequestHandlerInterface as RequestHandler;
use Tuupola\Middleware\CorsMiddleware;

class Cors
{
	public function __invoke(Request $request, RequestHandler $handler): Response
	{
		$cors = false;
		
		if (AppConfig::setting('site.security.api_enable_cors')) {
			$domains = AppConfig::setting('site.security.api_cors_domains');

			if ($domains != '') {
				$domains = explode(',', $domains);
				$domains = array_filter($domains);

				$domains = array_map(function ($item) {
					$item = rtrim($item, '/');
					return trim($item);
				}, $domains);
			}

			if (! empty($domains)) {
				global $app;

				$cors = new CorsMiddleware([
					"origin" => $domains,
					"headers.allow" => [
						'X-Requested-With',
						'X-Onecrm-Filename',
						'Content-Length',
						'Content-Type',
						'Accept',
						'Accept-Language',
						'Origin',
						'Authorization'
					],
					"credentials" => true,
					"cache" => 0
				]);
			}
		}
		
		return ($cors instanceof CorsMiddleware) ? $cors->process($request, $handler) : $handler->handle($request);
	}
}
