<?php

namespace OneCRM\API\Middleware;

use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Server\RequestHandlerInterface as RequestHandler;

class DetectLanguage {
	public function __invoke(Request $request, RequestHandler $handler): ResponseInterface {
		$request = $request->withAttribute('onecrm_lang', $this->getPreferredLanguage($request));
		return $handler->handle($request);
	}

	protected function getPreferredLanguage($request) {
		$header = $request->getHeader( 'Accept-Language');
		if (is_array($header)) {
			if (!empty($header)) {
				$header = $header[0];
			} else {
				$header = '';
			}
		}
		$parts = array_filter(preg_split('/\s*,\s*/', $header));
		$parts = array_map(function($item) {
			$parts = explode(';q=', $item);
			$lang = null;
			$quality = 1;
			if (count($parts) == 1) {
				$lang = $parts[0];
			} elseif (count($parts) == 2) {
				list($lang, $quality) = $parts;
			}
			if (!is_numeric($quality))
				return null;
			if ($lang !== '*' && !preg_match('/^[a-z]+(-[a-z]+)*$/', $lang))
				return null;
			return [strtolower($lang), $quality];
		}, $parts);
		$parts = array_filter($parts);
		usort($parts, function($a, $b) {
			return (int)($b[1] * 1000) - (int)($a[1] * 1000);
		});

		$allLang = array_keys(\AppConfig::get_languages());
		$allLangLower = array_map('strtolower', $allLang);
		foreach ($parts as $p) {
			list($l, $q) = $p;
			if ($l === '*')
				continue;
			$lparts = explode('-', $l);
			$ccode = null;
			if (count($lparts) > 1) {
				list($lcode, $ccode) = $lparts;
			} else {
				$lcode = $lparts[0];
			}
			$key = $lcode . '_' . $ccode;
			if ($ccode !== null && isset($allLangLower[$key])) {
				$pos = array_search($key, $allLangLower);
				return $allLang[$pos];
			}
			foreach ($allLangLower as $i => $lc) {
				if (strpos($lc, $lcode . '_') === 0) {
					return $allLang[$i];
				}
			}
		}
		$base_lang = \AppConfig::setting('locale.base_language');
		return \AppConfig::setting('locale.defaults.language', $base_lang);
	}
}
