<?php

namespace OneCRM\API\Middleware;
use OneCRM\API\Errors\Forbidden;

use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Server\RequestHandlerInterface as RequestHandler;
use Psr\Http\Message\ResponseInterface as Response;

class SecureConnectionGuard {
	/**
	 * @throws Forbidden
	 */
	public function __invoke(Request $request, RequestHandler $handler): Response
	{
		$secure = \AppConfig::detect_https();
		if (!$secure && !\AppConfig::setting('site.security.insecure_api_calls'))
			throw new Forbidden(translate('API_ERROR_INSECURE'));
		if (!function_exists('openssl_pkey_get_private'))
			throw new Forbidden(translate('API_ERROR_NO_OPENSSL'));
		return  $handler->handle($request);
	}
}
