<?php

namespace OneCRM\API;

class Registry {

	private $registry;
	private $types = array();
	private $endpoints = array();

	public function __construct($registry, $app) {

		$ext = \AppConfig::setting('api_config.endpoints', []);
		foreach ($ext as $def) {
			require_once $def['file'];
			$cls = $def['class'];
			$id = 'ext_' . str_replace('\\', '_', $cls);
			$registry['endpoints'][$id] = [
				'class' => $cls,
				'group' => $def['group'],
			];
		}
		$ext = \AppConfig::setting('api_config.groups', []);
		foreach ($ext as $gid => $g) {
			if (!isset($registry['groups'][$gid]))
				$registry['groups'][$gid] = $g;

		}

		$this->registry = $registry;
		$this->registerTypes();
		$this->registerEndpoints($app);
	}

	public function getRegistry() {
		return $this->registry;
	}

	private function registerTypes() {
		foreach ($this->registry['types'] as $cls) {
			$type = new $cls($this);
			$this->types[$type->getName()] = $type;
		}
	}

	private function registerEndpoints($app) {
		foreach ($this->registry['endpoints'] as $name => $meta) {
			$this->endpoints[$name] = new $meta['class']($app);
		}
	}

	public function getTypes() {
		return $this->types;
	}

	public function getEndpoints() {
		return $this->endpoints;
	}

	public function getEndpointByName($name) {
		return array_get_default($this->endpoints, $name);
	}

	public function getType($name) {
		return array_get_default($this->types, $name);
	}

	private function findEndpoint($req) {
		$method = $req->getMethod();
		$path = $req->getPath();
		foreach ($this->endpoints as $ep) {
			if ($ep->supportsMethod($method) && $ep->servesPath($path))
				return $ep;
		}
	}

}


