<?php

namespace OneCRM\API\Types;
use OneCRM\API\Type;
use OneCRM\API\Errors;

class ArrayType extends Type {

	public function __construct($server, $name = 'Array') {
		parent::__construct($server, $name);
	}

	public function parseValue($name, $val, $params = array()) {
		$type = $this->getName();
		if (!is_array($val))
			throw new Errors\BadRequest("Param {$name} should be of type {$type}");
		$len = count($val);
		if ($len) {
			$i = 0;
			reset($val);
			do {
				if ($i !== key($val))
					throw new Errors\BadRequest("Param {$name} should be of type {$type}");
				$i++;
			} while(next($val) !== false);
		}

		if (isset($params['min_len']) && $len < ($min_len = $params['min_len']))
			throw new Errors\BadRequest("Param {$name} should contain at least {$min_len} elements, but passed value has {$len} elements");
		if (isset($params['max_len']) && $len > ($max_len = $params['max_len']))
			throw new Errors\BadRequest("Param {$name} should contain at most {$min_len} elements, but passed value has {$len} elements");
		if (isset($params['element_type'])) {
			$type_def = $params['element_type'];
			if (is_array($type_def)) {
				$el_params = $type_def;
				$typeName = $type_def['type'];
			} else {
				$el_params = array();
				$typeName = $type_def;
			}
			$type = $this->server->getType($typeName);
			if (!$type)
				throw new Errors\InternalError('No definition for type ' . $typeName);
			foreach ($val as $k => $v) {
				$val[$k] = $type->parseValueEx($name . '.' . $k, $v, $el_params);
			}
		}
		return $val;
	}

	public function getDocs() {
		return '@types/array';
	}
}

