<?php

namespace OneCRM\API\Types;
use OneCRM\API\Type;
use OneCRM\API\Errors;

class ObjectType extends Type {

	public function __construct($server, $name = 'Object') {
		parent::__construct($server, $name);
	}

	public function parseValue($name, $val, $params = array()) {
		$type = $this->getName();
		if (!is_array($val))
			throw new Errors\BadRequest("Param {$name} should be of type {$type}");
		if (isset($params['schema'])) {
			foreach ($params['schema'] as $elName => $elDef) {
				if (is_array($elDef)) {
					$elParams = $elDef;
					$typeName = $elDef['type'];
				} else {
					$elParams = array();
					$typeName = $elDef;
				}
				if (is_array($typeName)) {
					$elParams = $typeName;
					$typeName = $typeName['type'];
				}
				$type = $this->server->getType($typeName);
				if (!$type)
					throw new Errors\InternalError('No definition for type ' . $typeName);
				if (!empty($elParams['required']) && !isset($val[$elName]))
					throw new Errors\BadRequest("Param {$name}.{$elName} is required");
				if (isset($val[$elName])) {
					$val[$elName] = $type->parseValueEx($name . '.' . $elName, $val[$elName], $elParams);
				} elseif (isset($elParams['default'])) {
					$val[$elName] = $elParams['default'];
				}
			}
		}
		return $val;
	}

	public function getDocs() {
		return '@types/object';
	}
}

