<?php

namespace OneCRM\API\Types;
use OneCRM\API\Type;
use OneCRM\API\Errors;

class OneOfType extends Type {

	public function __construct($server, $name = 'OneOf') {
		parent::__construct($server, $name);
	}

	public function parseValue($name, $val, $params = array()) {
		foreach ($params['choices'] as $choice) {
			if (is_array($choice)) {
				$ch_type = $choice['type'];
				$ch_params = $choice;
			} else {
				$ch_type = $choice;
				$ch_params = [];
			}
			$type = $this->server->getType($ch_type);
			if (!$type)
				throw new Errors\InternalError('No definition for type ' . $ch_type);
			try {
				if (!empty($ch_params['required']) && !isset($val))
					throw new Errors\BadRequest("Param {$name} is required");
				if (isset($val)) {
					$val = $type->parseValueEx($name, $val, $ch_params);
				} elseif (isset($ch_params['default'])) {
					$val = $ch_params['default'];
				}
				return $val;
			} catch (Errors\BadRequest $e) {
			}
		}
		throw new Errors\BadRequest("Param {$name} has invalid value");
	}
	
	public function getDocs() {
		return '@types/oneof';
	}
}

