<?php
/**
 * Created by PhpStorm.
 * User: brafreider
 * Date: 10.08.2016
 * Time: 16:29
 */

namespace v4\crm;


require_once 'modules/Emails/widgets/EmailWidget.php';
require_once 'modules/Emails/Email.php';
require_once 'modules/EmailTemplates/TemplateParser.php';

class Email
{
    /** @var \RowUpdate */
    protected $email;
    /** @var  \RowResult */
    protected $contact;
    /** @var array */
    protected $recipients = [];
    /** @var array */
    protected $body = [];

    public function __construct($user_id = 1)
    {

        /** @var \User $current_user */
        $current_user = \ListQuery::quick_fetch_object('User', $user_id);
        if (!$current_user) throw new \Exception('User not found');

        $this->email = \RowUpdate::blank_for_model('Email');

        $this->email->set(
            [
                'assigned_user_id' => $user_id,
                'modified_user_id' => $user_id,
                'type' => 'out',
                'from_addr' => $current_user->getPreference('mail_fromaddress'),
                'from_name' => $current_user->getPreference('mail_fromname'),
            ]
        );

    }

    public function setSender($fromAddr, $fromName = '')
    {
        if ($fromAddr)
            $this->email->set(['from_addr' => $fromAddr,]);
        if ($fromName)
            $this->email->set(['from_name' => $fromName,]);

        return $this;
    }

    public function setRecipient(\RowResult $contact)
    {
        $this->contact = $contact;
        $this->recipients = array('to' => array(array(
            'display' => $contact->getField('name'),
            'email' => $contact->getField('email1')
        )));

        $this->email->set([
            'to_addrs' => $contact->getField('name') . '<' . $contact->getField('email1') . '>;',
        ]);
        return $this;
    }


    public function useTemplate($template_id, $extraReplacements = [])
    {
        $template = \ListQuery::quick_fetch('EmailTemplate', $template_id);
        if (!$template)throw new \Exception('Template not set, check 1CRM settings.');
        $subject = $template->getField('subject');
        $html = $template->getField('body_html');
        $text = $template->getField('body');

        $subject = str_replace(array_keys($extraReplacements), array_values($extraReplacements), $subject);
        $html = str_replace(array_keys($extraReplacements), array_values($extraReplacements), $html);
        $text = str_replace(array_keys($extraReplacements), array_values($extraReplacements), $text);


        $template_data = \TemplateParser::parse_generic(
            array(
                'subject' => $subject,
                'body_html' => $html,
                'body' => $text,
            ),
            array(
                'Contacts' => $this->contact->getField('id'),
            ),
            array()
        );

        $this->email->set('name', $template_data['subject']);

        $this->body = ['html' => $template_data['body_html'], 'text' => $template_data['body']];

        return $this;
    }


    public function send()
    {
        if ($this->email->validate()) {
            \EmailWidget::send($this->email, $this->recipients, $this->body, []);
            $this->email->save();
            \Email::saveDescriptions($this->email->getPrimaryKeyValue(), $this->body['text'], $this->body['html']);
        }


    }
}