<?php
/**
 * Created by PhpStorm.
 * User: brafreider
 * Date: 26.07.2016
 * Time: 16:57
 */

namespace v4\oAuth\Repositories;


use League\OAuth2\Server\Entities\AccessTokenEntityInterface;
use League\OAuth2\Server\Entities\ClientEntityInterface;
use League\OAuth2\Server\Exception\UniqueTokenIdentifierConstraintViolationException;
use League\OAuth2\Server\Repositories\AccessTokenRepositoryInterface;
use v4\oAuth\Entities\AccessTokenEntity;

class AccessTokenRepository implements AccessTokenRepositoryInterface
{
    public function getNewToken(ClientEntityInterface $clientEntity, array $scopes, $userIdentifier = null)
    {
        $accessToken = new AccessTokenEntity();
        $accessToken->setClient($clientEntity);
        foreach ($scopes as $scope) {
            $accessToken->addScope($scope);
        }
        $accessToken->setUserIdentifier($userIdentifier);
        return $accessToken;
    }

    public function persistNewAccessToken(AccessTokenEntityInterface $accessTokenEntity)
    {

        $token = \ListQuery::quick_fetch_key('AccessTokenEntity', 'uid', $accessTokenEntity->getIdentifier());
        if ($token) throw  UniqueTokenIdentifierConstraintViolationException::create();

        $client = $accessTokenEntity->getClient();
		$user_id = $accessTokenEntity->getUserIdentifier();
        $scopes = $accessTokenEntity->getScopes();
        $crmScopes = [];
        $dateTime = $accessTokenEntity->getExpiryDateTime();
        if ($dateTime instanceof \DateTime)
            $expiryDateTime = $dateTime->format('Y-m-d H:i:s');
        else
            $expiryDateTime = null;

        foreach ($scopes as $scope) {
            $crmScopes[] = $scope->getIdentifier();
        }


        $uid = $accessTokenEntity->getIdentifier();

        $rowUpdate = new \RowUpdate('AccessTokenEntity', 1);
		$rowUpdate->new_record = true;
		if ($user_id) {
			$parts = explode(':', $user_id, 2);
		} else {
			$parts = [null, null];
		}
        $rowUpdate->set([
            'client_id' => $client->getIdentifier(),
            'user_id' => $parts[1],
            'user_type' => $parts[0],
            'scopes' => implode('^,^', $crmScopes),
            'uid' => $uid,
            'expiry_date_time' => $expiryDateTime,

        ]);

        if ($rowUpdate->validate()) {
            $rowUpdate->save();
        } else {
            throw new  \Exception('Error saving Access Token: ' . $rowUpdate->getErrors(true));
        }

    }

    public function revokeAccessToken($tokenId)
    {
        $token = \ListQuery::quick_fetch_key('AccessTokenEntity', 'uid', $tokenId);
        if ($token) {
            $tokenUpdate = \RowUpdate::for_result($token);
            $tokenUpdate->deleteRow();
        }
        $sql = "DELETE FROM oauth_access_tokens WHERE expiry_date_time < now()";
        $GLOBALS['db']->query($sql);
    }

    public function isAccessTokenRevoked($tokenId)
    {
        $token = \ListQuery::quick_fetch_key('AccessTokenEntity', 'uid', $tokenId);
        if ($token) return false; // Access token hasn't been revoked
        else return true;
    }

}
