<?php
/**
 * Created by PhpStorm.
 * User: brafreider
 * Date: 26.07.2016
 * Time: 16:52
 */

namespace v4\oAuth\Repositories;


use League\OAuth2\Server\Repositories\ClientRepositoryInterface;
use v4\oAuth\Entities\ClientEntity;

class ClientRepository implements ClientRepositoryInterface
{
    public function getClientEntity($clientIdentifier): ?ClientEntity
	{
        // Check if client is registered
        $clientRowResult = $this->getClientRow($clientIdentifier);
        if (!$clientRowResult) {
            return null;
		}
		if (!$clientRowResult->getField('enabled')) {
			return null;
		}

        $client = new ClientEntity();
        $client->setIdentifier($clientIdentifier);
        $client->setName($clientRowResult->getField('name'));
		$client->setIsConfidential(true);
		
		$redirect = $clientRowResult->getField('redirect');
		$redirect = array_filter(preg_split("~[\\r\\n]+~", $redirect));
        $client->setRedirectUri($redirect);
        return $client;
    }

	public function validateClient($clientIdentifier, $clientSecret, $grantType): bool
	{
		$clientRow = $this->getClientRow($clientIdentifier);
		if(!$clientRow) {
			return false;
		}

		$types = explode('^,^', $clientRow->getField('grant_types'));
		if ($grantType !== 'refresh_token' && !in_array($grantType, $types)) {
			return false;
		}

		$client_hash = $clientRow->getField('secret');
		if ($clientSecret !== null && password_verify($clientSecret, $client_hash) === false) {
			return false;
		}
		if ($grantType === 'client_credentials' && !$clientRow->getField('user_id')) {
			return false;
		}

		return true;
	}

	private function getClientRow($clientIdentifier) {
		return \ListQuery::quick_fetch('ClientEntity', $clientIdentifier);
	}
}
