<?php
/**
 * Created by PhpStorm.
 * User: brafreider
 * Date: 27.07.2016
 * Time: 11:33
 */

namespace v4\oAuth\Repositories;


use League\OAuth2\Server\Entities\RefreshTokenEntityInterface;
use League\OAuth2\Server\Exception\UniqueTokenIdentifierConstraintViolationException;
use League\OAuth2\Server\Repositories\RefreshTokenRepositoryInterface;
use v4\oAuth\Entities\RefreshTokenEntity;

class RefreshTokenRepository implements RefreshTokenRepositoryInterface
{
    public function getNewRefreshToken()
    {
        return new RefreshTokenEntity();
    }

    public function persistNewRefreshToken(RefreshTokenEntityInterface $refreshTokenEntity)
    {
        $token = \ListQuery::quick_fetch_key('RefreshTokenEntity', 'uid', $refreshTokenEntity->getIdentifier());
        if ($token) throw  UniqueTokenIdentifierConstraintViolationException::create();

        $dateTime = $refreshTokenEntity->getExpiryDateTime();
        if ($dateTime instanceof \DateTime)
            $expiryDateTime = $dateTime->format('Y-m-d H:i:s');
        else
            $expiryDateTime = null;

        $uid = $refreshTokenEntity->getIdentifier();
        $accessToken = $refreshTokenEntity->getAccessToken();
        $accessTokenId = null;
        if ($accessToken)
            $accessTokenId = $accessToken->getIdentifier();


        $rowUpdate = new \RowUpdate('RefreshTokenEntity', 1);
        $rowUpdate->new_record = true;
        $rowUpdate->set([
            'uid' => $uid,
            'expiry_date_time' => $expiryDateTime,
            'access_token_id' => $accessTokenId
        ]);

        if ($rowUpdate->validate()) {
            $rowUpdate->save();
        } else {
            throw new  \Exception('Error saving Refresh Token: ' . $rowUpdate->getErrors(true));
        }

    }

    public function revokeRefreshToken($tokenId)
    {
        $token = \ListQuery::quick_fetch_key('RefreshTokenEntity', 'uid', $tokenId);
        if ($token) {
            $tokenUpdate = \RowUpdate::for_result($token);
            $tokenUpdate->deleteRow();
        }
        $sql = "DELETE FROM oauth_refresh_tokens WHERE expiry_date_time < now()";
        $GLOBALS['db']->query($sql);
    }

    public function isRefreshTokenRevoked($tokenId)
    {
        $token = \ListQuery::quick_fetch_key('RefreshTokenEntity', 'uid', $tokenId);
        if ($token) return false; // Access token hasn't been revoked
        else return true;
    }

}