<?php
/**
 * Created by PhpStorm.
 * User: brafreider
 * Date: 26.07.2016
 * Time: 16:43
 */

namespace v4\oAuth\Repositories;


use League\OAuth2\Server\Entities\ClientEntityInterface;
use League\OAuth2\Server\Repositories\UserRepositoryInterface;
use v4\crm\Email;
use v4\oAuth\Entities\UserEntity;


require_once 'modules/UserPreferences/UserPreference.php';

class UserRepository implements UserRepositoryInterface
{
	protected $crm_user_id;
	protected $userType;

    public function __construct() {
		$id = \AppConfig::setting('oauth.user_id');
		if (!$id) {
			require_bean('User');
			$user = new \User();
			$user = $user->retrieveAdminUser();
			$id = $user->id;
		}
		$this->crm_user_id = $id;
    }

	public function setUserType($type) {
		$this->userType = $type;
	}

    public function getUserEntityByUserCredentials(
        $username,
        $password,
        $grantType,
        ClientEntityInterface $clientEntity
    )
    {

		if ($this->userType == 'Users') {
			require_once 'modules/Users/authentication/SugarAuthenticate/SugarAuthenticateUser.php';
			$sa = new \SugarAuthenticateUser();
			$id = $sa->authenticateUser($username, $password);
			if ($id) {
				$user = \ListQuery::quick_fetch('User', $id);
				if ($user) {
					$userEntity = new UserEntity();
					$identifier = 'Users:' . $user->getPrimaryKeyValue();
					$userEntity->setIdentifier($identifier);
					$userEntity->setUsername($username);
					$userEntity->setEmail($user->getField('email1'));
					$userEntity->setTimezone(
						\AppConfig::setting("userpref.by_user.$id.global.timezone",
							\AppConfig::system_timezone())
						);
					return $userEntity;
				}
			}
			return null;
		}

        $lq = new \ListQuery('Contact', ['id', 'portal_user_hash', 'email1']);
        $lq->addSimpleFilter('portal_name', '_p_' . $username);
        $lq->addSimpleFilter('portal_active', '1');

        // assume that there is only one (check on contact save with portal_name and portal_active)
        $contact = $lq->runQuerySingle();


        // new portal auth
        if (password_verify($password, $contact->getField('portal_user_hash'))) {
            $userEntity = new UserEntity();
			$userEntity->setIdentifier('Contacts:' . $contact->getPrimaryKeyValue());
            $userEntity->setUsername($username);
            $userEntity->setEmail($contact->getField('email1'));
			$userEntity->setTimezone(\AppConfig::system_timezone());
            //check if rehash is necessary
            $this->changeContactPassword($contact, $password);
            return $userEntity;
        }


        // old portal auth
        $user = $this->get1crmUser($username, $password);

        if (!$user->failed && !$contact->failed) {
			$userEntity = new UserEntity();
			$identifier = 'Contacts:' . $contact->getPrimaryKeyValue();
            $userEntity->setIdentifier($identifier);
            $userEntity->setUsername($username);
			$userEntity->setEmail($contact->getField('email1'));
			$userEntity->setTimezone(\AppConfig::system_timezone());
            // rehash Password and save to contact
            $this->changeContactPassword($contact, $password);
            // Phase 2 disable old User?
            return $userEntity;
        }


        return null;
    }

    public function getUserEntityByUserName($username)
    {
        $lq = new \ListQuery('Contact');
        $lq->addSimpleFilter('portal_name', '_p_' . $username);;
        $lq->addSimpleFilter('portal_active', 1);
        $contact = $lq->runQuerySingle();
        if ($contact) {
            $userEntity = $this->mapCrmContactToUserEntity($contact);
            return $userEntity;
        }
        return null;
    }

    public function changeContactPassword(\RowResult $contact, $password)
    {
        $oldHash = $contact->getField('portal_user_hash');
        if (empty($oldHash) || password_needs_rehash($oldHash, PASSWORD_DEFAULT) || !password_verify($password, $oldHash)) {
            $contactUpdate = \RowUpdate::for_result($contact, null, $this->crm_user_id);
            $contactUpdate->set(
                [
                    'portal_user_hash' => password_hash($password, PASSWORD_DEFAULT)
                ]
            );
            if ($contactUpdate->validate()) $contactUpdate->save();
        }
    }

    public function getUserEntityById($id)
	{
		list($module, $id) = explode(':', $id, 2);
		$model = \AppConfig::module_primary_bean($module);
		if (!$model)
			return null;
        $contact = \ListQuery::quick_fetch($model, $id);
        if ($contact) {
            $userEntity = $this->mapCrmContactToUserEntity($contact);
            return $userEntity;
        }
        return null;
    }

    public function resetPasswordForUser($username, $email)
    {

        $userEntity = $this->getUserEntityByUserName($username);
        if (!$userEntity || strtolower($email) != strtolower($userEntity->getEmail())) return false;

        $newPassword = $this->generatePassword();
        $email = new Email(\AppConfig::setting('oauth.assigned_user', 1));
        $contact = $this->get1CRMContactForUserEntity($userEntity);
        $email->setRecipient($contact)
            ->setSender(
                \AppConfig::setting('oauth.from_addr'),
                \AppConfig::setting('oauth.from_name')
            )
            ->useTemplate(\AppConfig::setting('oauth.email_template'), ['$password' => $newPassword])
            ->send();

        // Password zurücksetzen...
        $this->setPasswordForUser($userEntity, $newPassword, '');
        $email = $userEntity->getEmail();
        // und per email versenden
        return $email;

    }

    /**
     * @param UserEntity $userEntity
     * @param string $newPassword
     * @param $oldPassword
     * @return bool
     * @throws \Exception
     */
    public function setPasswordForUser(UserEntity $userEntity, $newPassword, $oldPassword = '')
    {
        $contact = \ListQuery::quick_fetch('Contact', $userEntity->getIdentifier());
        if (!$contact) throw new \Exception('Contact does not exist in Database');
        $this->changeContactPassword($contact, $newPassword);

        // old auth via User in Phase 2 deaktivieren! be deaktiviertem Portal in der Administration sowieso keine Passwörter speichern!
        if (\AppConfig::setting('portal.enabled')) {
            $user = $this->get1crmUser($userEntity->getUsername(), $oldPassword);
            if ($user) {
                $userUpdate = \RowUpdate::for_result($user, null, $this->crm_user_id);
                $userUpdate->set('user_hash', strtolower(md5($newPassword)));
                if ($userUpdate->validate()) $userUpdate->save();
            }
        }

        return true;
    }

    /**
     * @param $username
     * @param $password
     * @return \RowResult
     */
    public function get1crmUser($username, $password = '')
    {
        $lq = new \ListQuery('User');
        $lq->addSimpleFilter('user_name', '_p_' . $username);
        if (!empty($password)) {
            $lq->addSimpleFilter('user_hash', strtolower(md5($password)));
        }
        $lq->addSimpleFilter('portal_only', '1');
        $lq->addSimpleFilter('status', 'Active');

        $user = $lq->runQuerySingle();
        return $user;
    }

    public function get1CRMContactForUserEntity(UserEntity $userEntity)
    {
        return \ListQuery::quick_fetch('Contact', $userEntity->getIdentifier());
    }

    public function getEmailHint($email)
    {
        $atPos = strpos($email, '@');
        if ($atPos === false) return false;
        $hint = '(' . substr($email, 0, 1) . '...' . substr($email, $atPos - 1, 3) . '...' . substr($email, -3) . ')';
        return $hint;
    }


    /**
     * @param \RowResult $contact
     * @return UserEntity
     */
    public function mapCrmContactToUserEntity(\RowResult $contact)
    {
		$userEntity = new UserEntity;
		$type = $contact->getModuleDir();
		$userEntity->setIdentifier($type . ':' . $contact->getPrimaryKeyValue());
		if ($type == 'Users') {
	        $username = $contact->getField('user_name');
		} else {
	        $username = $contact->getField('portal_name');
    	    if (strpos($username, '_p_') === 0) {
        	    $username = substr($username, 3);
	        }
		}
		$userEntity->setUsername($username);
		$userEntity->setRole($type == 'Users' ? 'user' : 'customer');
        $userEntity->setEmail($contact->getField('email1'));
		if ($type == 'Users') {
			$id = $contact->getField('id');
			$userEntity->setTimezone(
					\AppConfig::setting("userpref.by_user.$id.global.timezone",
						\AppConfig::system_timezone())
					);
		} else {
			$userEntity->setTimezone(\AppConfig::system_timezone());
		}
        $userEntity->setName($contact->getField('name'));
        $userEntity->setFirstName($contact->getField('first_name'));
        $userEntity->setLastName($contact->getField('last_name'));
        $userEntity->setIsPartner($contact->getField('partner_id'));

        return $userEntity;
    }

    function generatePassword($laenge = 15)
    {
        if ($laenge < 8) $laenge = 8;
        mt_srand(crc32(microtime()));

        //Welche Buchstaben benutzt werden sollen (Charset)
        $buchstaben = "abcdefghijkmnpqrstuvwxyz123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ!$%=*+#,.;:?-";
        $buchstaben = str_shuffle($buchstaben);
        $str_lng = strlen($buchstaben) - 1;

        do {
            $passwort = "";
            for ($i = 0; $i < $laenge; $i++) {

                $passwort .= substr($buchstaben, mt_rand(0, $str_lng), 1);
            }

        } while (!preg_match("%^(?=.*\d)(?=.*[a-z])(?=.*[A-Z])(?=.*\W).*$%", $passwort));
        return $passwort;
    }
}
