<?php
/**
 * Created by PhpStorm.
 * User: brafreider
 * Date: 11.08.2016
 * Time: 09:17
 */

namespace v4\utils;


use Psr\Http\Message\ServerRequestInterface as Request;
use Psr\Http\Server\RequestHandlerInterface as RequestHandler;
use Psr\Http\Message\ResponseInterface as Response;

class RememberMeMiddleware
{
    /** @var RememberMeTokenRepositoryInterface */
    protected $repository;
    /** @var string */
    protected $cookiename;
    /** @var string */
    protected $sessionPrefix;

    public function __construct(RememberMeTokenRepositoryInterface $rmrepository, $sessionPrefix, $cookiename = 'ErinnerDich')
    {
        $this->repository = $rmrepository;
        $this->cookiename = $cookiename;
		$this->sessionPrefix = $sessionPrefix;
    }

    /**
     * Remember Me middleware invokable class
     *
     * @param  Request $request
     * @param  RequestHandler $handler
	 *
     * @return Response
     */
	public function __invoke(Request $request, RequestHandler $handler): Response
	{
        $error = false;
        // check if cookie exists
		$cookies = $request->getCookieParams();
        if ( isset($cookies[$this->cookiename])) {
            $token = $cookies[$this->cookiename];
            if (substr_count($token, ':') === 1) {
                $parts = explode(':', $token);
				$tokenEntity = $this->repository->getRememberMeTokenById($parts[0]);
                if ($tokenEntity && password_verify($parts[1], $tokenEntity->getToken())) {
					$_SESSION[$this->sessionPrefix . 'user_id'] = $tokenEntity->getUserType(). ':' . $tokenEntity->getUserId();
                    $expire = time() + 365 * 24 * 60 * 60;
                    $tokenEntity->setExpires($expire);
                    $this->repository->persistToken($tokenEntity);
                    //expire verändern - oder Token neu?
                    setcookie($this->cookiename, $tokenEntity->getId() . ':' . $parts[1], $expire);
				} else {
					$error = true;
				}
			} else {
				$error = true;
			}
        }

		$response = $handler->handle($request);

        if ($request->getMethod() == 'POST' && !empty($_SESSION[$this->sessionPrefix . 'user_id'])) {

			$params = $request->getParsedBody() ?? [];
            if (isset($params['remember'])) {
                $token = bin2hex(random_bytes(20));
                $expire = time() + 365 * 24 * 60 * 60;
				$tokenEntity = $this->repository->getNewToken();
				$parts = explode(':', $_SESSION[$this->sessionPrefix . 'user_id'], 2);
                $tokenEntity->setUserId($parts[1]);
                $tokenEntity->setUserType($parts[0]);
                $tokenEntity->setToken(password_hash($token, PASSWORD_DEFAULT));
                $tokenEntity->setExpires($expire);

                $id = $this->repository->persistToken($tokenEntity);

                if ($id) {
                    setcookie($this->cookiename, $id . ':' . $token, $expire);
                    $error = false;
                }

            }

        }

        if ($error){
            setcookie($this->cookiename, '' , time() -3600);
            if(isset($tokenEntity)) $this->repository->deleteToken($tokenEntity);
        }

        return $response;
    }
}
