<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

require_once 'include/application/BaseController.php';

class ModuleController extends BaseController  {
	var $module;
	var $action;
	var $default_action = 'index';

	var $generic_actions = array(
		'ListView' => 'include/generic/ListView.php',
		'index' => 'include/generic/ListView.php',
		'DetailView' => 'include/generic/Detail.php',
		'ErasePersonal' => 'modules/DataPrivacy/ErasePersonal.php',
		'ViewPersonal' => 'modules/DataPrivacy/ViewPersonal.php',
		'EditView' => 'include/generic/Detail.php',
		'Duplicate' => 'include/generic/Detail.php',
		'Popup' => 'include/generic/Popup.php',
		'Audit' => 'include/generic/Audit.php',
		'DNDUpload' => 'include/generic/DNDUpload.php',
		'ImportDB' => 'modules/ImportDB/ImportDBController.php',
		'ShowDuplicates' => 'include/generic/ShowDuplicates.php',
		'PrintTemplate' => 'modules/PDFTemplates/PrintTemplate.php',
		'ManualInvoke' => 'modules/Workflow/ManualInvoke.php',
	);

	var $access_checks = array(
		'audit' => 'view',
		'index' => 'list',
		'listview' => 'list',
		'save' => 'edit',
		'popup' => 'list',
		'popupeditview' => 'edit',
		'editview' => 'edit',
		'duplicate' => 'edit',
		'detailview' => 'view',
		'manualinvoke' => 'view',
		'dndupload' => 'edit',
		'erasepersonal' => 'manage_personal',
		'viewpersonal' => 'manage_personal',
	);

	function __construct($mod) {
		$this->module = $mod;
		$this->set_action(array_get_default($_REQUEST, 'action', 'index'));
	}

	function set_action($action) {
		if(! $action) $action = $this->default_action;
		$this->action = $action;
	}

	function get_module() {
		return $this->module;
	}
	function get_action() {
		return $this->action;
	}

	function check_access() {
		if($this->module == 'Users' && ($this->action == 'Authenticate' || $this->action == 'Logout'))
			return true;
		if($this->module == 'NewStudio' && $_REQUEST['wizard'] == 'ArrangeSubpanels')
			return true;
		if(! AppConfig::current_user_id())
			return false;
		// check ACL
		$act_info = AppConfig::setting("modinfo.by_name.$this->module.web_actions.{$this->action}");
		if(isset($act_info) && isset($act_info['acl'])) {
			$check_action = $act_info['acl'];
			if($check_action == 'allow')
				return true;
			if($check_action == 'deny')
				return false;
		} else {
			$check_action = $this->action;
		}
		$check_action = strtolower($check_action);
		if($check_action == 'index' && ! AppConfig::module_primary_bean($this->module))
			$check_action = 'access';
		if($check_action == 'importdb')
			$check_action = 'import';
		if($check_action == 'printtemplate')
			$check_action = 'view';
		if($check_action == 'editimage')
			$check_action = 'edit';
		$check_action = array_get_default($this->access_checks, $check_action, $check_action);
		$allowed = ACLController::checkAccess($this->module, $check_action);
		if(! isset($allowed) && isset($act_info) && $check_action != 'access') {
			// if module defines this web action but it doesn't map to an ACL role
			$allowed = ACLController::checkAccess($this->module, 'access');
		}
		return $allowed;
	}


	function init_globals() {
		// Initialize globals
		global $current_language, $theme, $app_strings, $currentModule;
		if(! AppConfig::current_user_id())
			$GLOBALS['app_strings']['NTC_WELCOME'] = '';
		$currentModule = $this->get_module();
		$GLOBALS['export_module'] = $currentModule;
		$GLOBALS['log']->info('current module is '.$currentModule);
		$GLOBALS['mod_strings'] = return_module_language($current_language, $currentModule);
	}

	
	function create_theme_instance($theme=null) {
		global $theme_config, $theme_path, $image_path;
		if(! isset($theme)) $theme = AppConfig::theme();
		$theme_config = AppConfig::get_theme_config($theme);
		$theme_params = array('mobile_mode' => array_get_default($_SESSION, 'mobile_mode', false));
		if(! empty($theme_config['class_name'])) {
			$clsPath = array_get_default($theme_config, 'class_path', "themes/$theme/theme.php");
			require_once($clsPath);
			$instance = new $theme_config['class_name']($theme, $theme_params);
		}
		else {
			require_once('include/layout/CompatTheme.php');
			$instance = new CompatTheme($theme, $theme_params);
		}
		$theme_path = $instance->theme_path;
		$image_path = $instance->image_path;
		return $instance;
	}


	function check_raw_output($params) {
		if(preg_match('/^(Save|Delete|Popup|ChangePassword|Authenticate|Logout|Export)/', $this->action)){
			$params['skip_headers'] = true;
			if(preg_match('/^(Popup|ChangePassword|Export|Save|Delete)/', $this->action))
				$params['skip_footers'] = true;
		}

		// longreach - added  export_vcards and pdf_print
		if($this->action == 'PDF' || $this->action == 'ExportVCards' || $this->action == 'pdf_print')
			$params['raw_output'] = true;
		if(isset($_REQUEST['from']) && $_REQUEST['from'] == 'ImportVCard')
			$params['raw_output'] = true;

		if($this->action == 'Import' && isset($_REQUEST['step']) && $_REQUEST['step'] == '4')
			$params['raw_output'] = true;

		$doRawOutput = !(empty($_REQUEST['to_pdf']) && empty($_REQUEST['to_csv']) && empty($_REQUEST['to_iif'])
			&& empty($_REQUEST['to_js']) && empty($_REQUEST['asyncListView']) && empty($_REQUEST['sugar_body_only']));
		$doRawOutput |= ($this->module == "ZMerge" && ($this->action != 'archivePopup' && $this->action != 'archiveClose' ));
		if($doRawOutput)
			$params['raw_output'] = true;

		return $params;
	}


	function action_allowed_without_user() {
		if($this->module == 'Users') {
			if($this->action == 'Authenticate') {
				$GLOBALS['log']->debug('We are authenticating user now');
				return true;
			}
		} elseif($this->module == 'Home' && $this->action == 'About')
			return true;
		return false;
	}


	function get_menu_shortcuts($override_mod=null) {
		global $mod_strings;
		$mod = $this->module;
		if($override_mod && $mod != $override_mod) {
			$mod = $override_mod;
			$old_mod = $mod_strings;
			$mod_strings = return_module_language(null, $mod);
		}
		$module_menu = array();
		$path = "modules/{$mod}/";
		$test = array(
			$path . 'Menu.php',
			AppConfig::custom_dir() . $path . 'Ext/Menus/*.php',
			AppConfig::custom_dir() . 'application/Ext/Menus/*.php',
		);
		foreach($test as $t) {
			$files = glob_unsorted($t);
			foreach ($files as $f)
				include($f);
		}
		foreach($module_menu as $k => $v)
			if($v[0] == '#') unset($module_menu[$k]);

		$bean_name = AppConfig::module_primary_bean($this->module);
		$detail = AppConfig::setting("model.detail.$bean_name");
		$importable = array_get_default($detail, 'importable', false);
		$import_mod = $this->module;
		if (!$importable) {
			$bean_name = AppConfig::module_primary_bean($mod);
			$detail = AppConfig::setting("model.detail.$bean_name");
			$importable = array_get_default($detail, 'importable', false);
			$import_mod = $mod;
		}
		if ($importable) {
			if(ACLController::checkAccess($import_mod, 'import', true)){
				$module_menu[] = array(
					"index.php?module={$import_mod}&action=ImportDB",
					translate('LBL_IMPORT_DB_TITLE', 'Administration'),
					"Import",
					$import_mod,
				);
			}
		}

		if(isset($old_mod))
			$mod_strings = $old_mod;
		return $module_menu;
	}


	function create_page_instance() {
		require_once('include/layout/pages/ModulePage.php');
		$pageInstance = new ModulePage();
		if(! empty($_REQUEST['asyncListView']) || ! empty($_REQUEST['sugar_body_only']) || defined('ASYNC_ENTRY'))
			$pageInstance->async_page = true;
		return $pageInstance;
	}
}

