<?php
/* contains code from include/charts/Charts.php under SPL */

require_once('include/Dashlets/Dashlet.php');

class BaseChart extends Dashlet {
	var $is_sidebar = false;
	var $modules = array();
	var $acl_level = 'list';
	var $order = '';
	var $chart_type = '';
	var $refresh = false;
	var $cache_filename = '';
	var $cache_file_date = '';
	var $show_border = true;
	var $show_background = true;

	// Dashlet properties
	var $dashletIcon = 'Dashboard';
	var $isConfigurable = true;
	var $isRefreshable = true;
	var $hasScript = false;
	var $title = 'Generic Chart';

	var $layoutChanged = false;

	var $estimContentHeight = 100;

	var $chart_text_options;
	var $chart_date_options;
	var $chart_select_options;

    /**
     * @var EditableForm
     */
    var $form_obj;


	function __construct($id, $options=null) {
		global $themeInstance;
        parent::__construct($id, $options);
		if(! isset($options))
			$options = array();
        if(! empty($options['title']))
        	$this->title = $options['title'];
        else if(! empty($this->default_title)) {
        	$mod = isset($this->default_title_module) ? $this->default_title_module : 'Charts';
        	$this->title = translate($this->default_title, $mod);
        }
        $this->init_chart_options($options);
        if(isset($themeInstance)) {
        	$this->show_border = $themeInstance->get_config('add_chart_borders', true);
        	$this->show_background = $themeInstance->get_config('add_chart_backgrounds', true);
        }
	}

	function get_style() {
		$s = '';
		if($this->show_border) $s .= 'br';
		if($this->show_background) $s .= 'bg';
		return $s;
	}

	function get_chart_info() {
		if($this->chart_type == 'hBar' || $this->chart_type == 'vBar')
			$t = $this->chart_type.($this->is_sidebar ? 'S' : 'F');
		else if($this->chart_type == 'pie')
			$t = 'pieF';
		else
			$t = '';

		if($this->is_sidebar) {
			$ratio_h = 400 / 350;
			$min_w = 280;
			$max_w = 500;
			$default_w = 350;
		} else {
			$ratio_h = 1 / 2;
			$min_w = 400;
			$max_w = 1200;
			$default_w = 800;
		}

		if(isset($this->width))
			$width = $this->width;
		else {
			$width = isset($this->requestedWidth) ? $this->requestedWidth : $default_w;
			$width = min($max_w, max($min_w, $width));
		}
		if(isset($this->height))
			$height = $this->height;
		else
			$height = round($width * $ratio_h) + 1;

		return array($t, $width, $height);
	}

	function init_chart_options($options) {
	}

	function init_svg_chart(SVGChart $chart) {
		$chart->setHasBorder($this->show_border);
		$chart->setHasBackground($this->show_background);
	}

	function gen_xml() {
		return '';
	}

	function chart_display_options() {
		return '';
	}

	function get_footer_text() {
		return '';
	}

	function create_options_hash($options) {
		global $current_user, $theme;
		if(isset($options['title']))
			unset($options['title']); // does not affect data
		$vals = array();
		foreach($options as $k => $v) {
			$vals[] = $k;
			if(is_array($v)) {
				sort($v);
				$vals = array_merge($vals, $v);
			}
			else
				$vals[] = $v;
		}
		$data = serialize($vals) . $current_user->getUserPrivGuid() . $theme . $this->get_style();
		return abs(crc32($data));
	}

	// utility method to populate useful defaults for dropdown lists
	function get_selected_values($sel_var, $dom) {
		global $app_list_strings;
		$ret = array();
		if(! is_array($dom))
			$dom = $app_list_strings[$dom];
		if(is_array($sel_var) && count($sel_var) > 0) {
			foreach($sel_var as $key)
				if($key && isset($dom[$key]))
					$ret[$key] = $dom[$key];
		}
		else
			$ret = $dom;
		return $ret;
	}

	function get_select_options($sel_var, $dom, $defaults=array()) {
		if(! is_array($sel_var) || ! count($sel_var))
			$sel_var = $defaults;
		if(! is_array($dom))
			$dom = $GLOBALS['app_list_strings'][$dom];
		$sel = array();
		foreach($sel_var as $opt)
			if(isset($dom[$opt]))
				$sel[] = $opt;
		if(isset($dom['']))
			unset($dom['']);
		return get_select_options_with_id($dom, $sel);
	}


	function process() {
		if(! $this->forceRefresh)
			return 'pending';
		$this->refresh = true;

		$this->check_cache_date();
		if (! $this->cache_file_date || $this->refresh) {
			$svg = $this->svg_charts_enabled() && method_exists($this, 'gen_svg');
			if ($svg) {
				$this->gen_svg();
			} else {
				$this->gen_xml();
			}
		}
	}

	function displayBody() {
		if (!$this->canDisplay())
        	return $this->noDisplay();

		$body = "<div id=\"dashlet-body-{$this->id}\" class=\"card-body dashlet-body\">";
		$body .= "<div style=\"margin: 0 auto; text-align: center\">";
		$body .= $this->display_chart();

		$footer = $this->get_footer_text();
		if($footer)
			$body .= '<div class="chartFootnote" style="text-align: center; padding-bottom: 0.8em">'.$footer.'</div>';
		$body .= '</div></div>';

		return $body;
	}

	function getFooterText($mode=null) {
		global $timedate;
		if($this->cache_file_date && $mode != 'sample')
			return translate('LBL_CREATED_ON_REL', 'Charts').' '.$timedate->to_relative_date_time($this->cache_file_date);
	}

	function displayOptions() {
        require_once('include/layout/forms/FormGenerator.php');
        require_once 'include/layout/FieldFormatter.php';
        global $app_strings;
		$mod_strings = return_module_language(null, 'Home');

		$result = new RowResult;

        //$result->base_model = $this->base_model;
        $result->module_dirs = array('Home');
        //$result->fields = array_merge($sorted_opts, $le_fields);
        $title = to_html($this->title);

        $result->fields = [
        	'dashletTitle' => [
        		'vname' => 'LBL_DASHLET_CONFIGURE_TITLE',
        		'type' => 'varchar',
        		'len' => 60,
        		'width' => 30,
        		'editable' => true,
        	],
        ];
        $result->row = [
        	'dashletTitle' => $this->title,
        ];

		$layout = array(
			'editable' => true,
			'form_hooks' => array(
				'onsubmit' => "return SUGAR.sugarHome.postForm('configure_".$this->id."', SUGAR.sugarHome.uncoverPage);",
			),
			'form_hidden_fields' => array(
				'id' => $this->id, 'query' => true, 'configure' => 'true',
				'module' => 'Home', 'action' => 'ConfigureDashlet', 'resetDashlet' => '',
			),
			'buttons_position' => 'bottom',
			'form_buttons' => [
				'cancelDashlet' => [
            		'vname' => 'LBL_CANCEL_BUTTON_LABEL',
            		'icon' => 'input-icon icon-cancel',
            		'perform' => 'SUGAR.sugarHome.hideConfigure();',
            		'type' => 'button',
            		'align' => 'right',
            	],
            	'resetDashlet' => [
            		'vname' => 'LBL_RESET_BUTTON_LABEL',
            		'icon' => 'input-icon icon-delete',
            		'perform' => "this.form.resetDashlet.value='1'; this.form.onsubmit();",
            		'type' => 'button',
            		'align' => 'right',
            	],
            	'saveDashlet' => [
            		'vname' => 'LBL_SAVE_BUTTON_LABEL',
            		'icon' => 'input-icon icon-accept',
            		'type' => 'submit',
            		'perform' => '',
            		'align' => 'right',
            	],
            ],
            'sections' => [
            	[
	            	'id' => 'general',
	            	'label' => $mod_strings['LBL_DASHLET_CONFIGURE_GENERAL'],
	            	'columns' => 1,
	            	'elements' => [
	            		'dashletTitle',
	            	],
	            ],
            ],
        );
        if(! empty($this->dashboard_id))
        	$layout['form_hidden_fields']['layout'] = $this->dashboard_id;
        else
        	$layout['form_hidden_fields']['layout'] = $_GET['layout'];

        $opts = $this->chart_display_options();
        $filters = [];

        if (! empty($opts)) {
            if(! empty($opts['values'])) {
	            array_extend($result->row, $opts['values']);
	        }
            foreach ($opts['spec'] as $name => $def) {
            	if(isset($opts['labels'][$name])) {
            		$def['label'] = $opts['labels'][$name];
            	}
            	$def['editable'] = true;
            	$def['filter'] = true;
            	$result->fields[$name] = $def;
                $filters[] = $name;
            }
        }

        if($filters) {
        	$layout['sections'][] = [
            	'id' => 'filters',
            	'vname' => 'LBL_DASHLET_FILTERS',
            	'columns' => 1,
            	'elements' => $filters,
            ];
        }

	    $fmt = new FieldFormatter('html', 'editview');
		$formatted_row = $fmt->formatRow($result->fields, $result->row);
        $result->formatted = $formatted_row;

        $model = new ModelDef();
        $model->addDisplayFieldDefinitions($result->fields);

        $gen = FormGenerator::html_form($model, $layout, 'ConfigForm', 'configure_'.$this->id);
		$this->form_obj = $gen->getFormObject();

		$gen->renderForm($result);
		$out = $gen->getResult();
		$gen->exportIncludes();

        return $out;
	}

	function saveOptions($req) {
		$options = null;
		if(!empty($req['dashletTitle'])) {
            $options['title'] = $req['dashletTitle'];
        }
		$options = $this->save_chart_options($req, $options);
		return $options;
	}

	function save_chart_options($request, $options) {
		if(isset($this->chart_date_options))
			foreach($this->chart_date_options as $f) {
				$val = array_get_default($request, $f);
				if(! $val) $val = null;
				else
					$val = $GLOBALS['timedate']->to_db_date($val, false);
				$options[$f] = $val;
			}
		if(isset($this->chart_text_options))
			foreach($this->chart_text_options as $f)
				$options[$f] = array_get_default($request, $f);
		if(isset($this->chart_select_options))
			foreach($this->chart_select_options as $f) {
				$val = array_get_default($request, $f);
                if (! is_array($val) && ! empty($val)) $val = explode('^,^', $val);
				if(! is_array($val)) $val = null;
				$options[$f] = $val;
			}
		return $options;
	}

	function canDisplay() {
		foreach($this->modules as $mod)
			if(! ACLController::checkAccess($mod, $this->acl_level, true))
				return false;
		return parent::canDisplay();
	}

	function noDisplay() {
		return '<p style="text-align: center;" class="error dashlet-body">' . translate('MSG_DASHLET_NO_ACCESS') . '</p>';
	}


	function get_cache_file() {
		if(!file_exists(AppConfig::setting('site.paths.xml_dir')))
			mkdir(AppConfig::setting('site.paths.xml_dir'));
		return AppConfig::setting('site.paths.xml_dir').$this->cache_filename;
	}

	function check_cache_date() {
		$path = $this->get_cache_file();
		if (file_exists($path))
			$this->cache_file_date = gmdate('Y-m-d H:i', filemtime($path));
		else
			$this->cache_file_date = '';
	}

	function svg_charts_enabled() {
		return AppConfig::setting('layout.svg_charts_enabled');
	}

	function display_chart() {
		$xmlFile = $this->get_cache_file();
		$chartType = $this->get_chart_info();
		if(! file_exists($xmlFile) || ! $chartType || ! $chartType[0])
			return '';
		list($chartName, $width, $height) = $chartType;
		$svg = $this->svg_charts_enabled() && method_exists($this, 'gen_svg');
		if (!$svg) {
			$chartPath = "include/charts/$chartName.swf";
			$chartUrl = $chartPath . '?filename=' . htmlentities($xmlFile, ENT_QUOTES, 'UTF-8');
			$html = <<<EOH
				<object classid="clsid:D27CDB6E-AE6D-11cf-96B8-444553540000"
					codebase="https://download.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=6,0,0,0"
					width="$width" height="$height" id="$chartName" align="">
					<param name="movie" value="$chartUrl">
					<param name="bgcolor" value="#FFFFFF">
					<param name="wmode" value="transparent">
					<param name="quality" value="high">
					<embed src="$chartUrl" wmode="transparent" quality="high" bgcolor="#FFFFFF"
						width="$width" height="$height" name="$chartName" align=""
						type="application/x-shockwave-flash" pluginspage="https://www.macromedia.com/go/getflashplayer">
				</object>
EOH;
		} else {
			$tm = substr(md5(@filemtime($xmlFile).'0'), 0, 5);
			$height = '';
			$f = @fopen($xmlFile, 'r');
			if ($f) {
				$str = fread($f, 1024);
				fclose($f);
				$m = array();
				if (preg_match('~<svg[^>]*?width="(\d+)"[^>]*?height="(\d+)"[^>]*>~', $str, $m)) {
					$width = $m[1];
					$height = 'height="' . $m[2] . '"';
				}
			}
			$html =  '<object data="' . $xmlFile.'?'.$tm . '" type="image/svg+xml" width="100%" height="auto" title="" wmode="transparent"></object>';
		}
		return $html;
	}


	function generate_graphcolor($input,$instance) {
		global $themeInstance;
		return $themeInstance->generate_graph_color($input, $instance);
	}

	function save_xml_file($filename, $xml_file) {
		if(! ($handle = fopen($filename, 'w')) ) {
			$GLOBALS['log']->debug("Cannot open file ($filename)");
			return;
		}
		$svg = $this->svg_charts_enabled() && method_exists($this, 'gen_svg');

		if ($svg) $header = '';
		else $header = '<?xml version="1.0" encoding="'.$this->get_chart_charset().'"?>'."\n";
		if(fwrite($handle, $header) === false || fwrite($handle, $xml_file) === false) {
			$GLOBALS['log']->debug("Cannot write to file ($filename)");
		}

		$GLOBALS['log']->debug("Success, wrote ($xml_file) to file ($filename)");

		fclose($handle);
		return true;
	}

	function get_useful_max($numbers, $type='') {
		$max = max($numbers);
		$top = pow(10, ceil(log10($max)));
		$min = 10;
		// decimal point not supported (even in sugar 4.5)
		//if(($type == 'hBarF' || $type == 'hBarS') && $max < 1)
			// vertical bar chart has rounding issues, shows 0.xxxxE^-xx near zero
			// $min = 1;
		if($top/5 > $max)
			$top /= 5;
		else if($top/2 > $max)
			$top /= 2;
		return max($min, $top);
	}

	function get_chart_charset() {
		return AppConfig::charset();
	}

	function displayPendingScript($refresh = false) {
		return parent::displayPendingScript($refresh);
	}

	function setWidth($w)
	{
		parent::setWidth($w);
		if ($w) $this->forceRefresh = true;
	}
}
