<?php

require_once 'include/chat/ChatProvider.php';

class ChatAPIServer {

	private $endpoints = [];

	public function __construct() {
		$provider = ChatProvider::get();
		if ($provider && $provider->isEnabled()) {
			$provider->registerEndpoints($this);
		}
	}

	function authRequired() {
		$handler = $this->findEndpoint($_SERVER['PATH_INFO'], $_SERVER['REQUEST_METHOD'], $params);
		if ($handler) {
			list ($handler, $auth_required) = $handler;
		} else {
			$auth_required = true;
		}
		return $auth_required;
	}

	public function processRequest() {
		$provider = ChatProvider::get();
		if (!$provider || !$provider->isEnabled()) {
			return $this->reply(['error' => 'Not configured'], 400);
		}
		$handler = $this->findEndpoint($_SERVER['PATH_INFO'], $_SERVER['REQUEST_METHOD'], $params);
		if ($handler) {
			list ($handler, $auth_required) = $handler;
			$post = file_get_contents('php://input');
			$GLOBALS['log']->fatal($post);
			if (!empty($post)) {
				$json = getJSONObj();
				$post = $json->decode($post);
			} else {
				$post = $_POST;
			}
			$params = array_merge($_GET, $params);
			$handler($this, $params, $post);
		} else {
			$this->reply(['error' => 'Not found'], 404);
		}
	}

	public function reply($content, $statusCode = 200, $isJSON = true) {
		http_response_code($statusCode);
		if ($isJSON) {
			$json = getJSONObj();
			$content = $json->encode($content);
			header('Content-Type: application/json');
		}
		echo $content;
	}

	public function registerEndpoint($path, $method, Callable $handler, $auth_required = true) {
		$method = strtoupper($method);
		if (!isset($this->endpoints[$method]))
			$this->endpoints[$method] = [];
		$this->endpoints[$method][$path] = [$handler, $auth_required];
	}

	private function findEndpoint($path, $method, &$params) {
		$params = [];
		if (!isset($this->endpoints[$method])) {
			return null;
		}
		$names = [];
		$path = rtrim($path, '/');
		foreach ($this->endpoints[$method] as $ep => $handler) {
			$pattern = '';
			$parts = array_filter(explode('/', $ep), function($s) {return strlen($s);});
			foreach ($parts as $p) {
				if (substr($p, 0, 1) == '{' && substr($p, -1) == '}') {
					$name = substr($p, 1, -1);
					$pattern .= '\/(?<' . $name . '>[^\/]+)';
					$names[] = $name;
				} elseif ($p == '*') {
					$pattern .= '\/(?<_>.+)';
					$names[] = '_';
					break;
				} else {
					$pattern .= '\/' . $p;
				}
			}
			if (strlen($pattern)) {
				$pattern = '/^' . $pattern;
			} else {
				$pattern = '/^\/';
			}
			$pattern .= '$/';
			if (preg_match($pattern, $path, $m)) {
				foreach ($names as $name) {
					$params[$name] = $m[$name];
				}
				return $handler;
			}
		}
	}

}

