<?php

abstract class ChatProvider {

	private static $providers = [];

	public static function get($id = null) {
		
		if (!$id) {
			$id = AppConfig::setting('chat.integration');
			if (!$id) {
				return;
			}
		}
		if (!array_key_exists($id, self::$providers)) {
			$provider = null;
			$info = null;
			$modules = AppConfig::setting("modinfo.index.all");
			foreach ($modules as $m) {
				$provider_info = AppConfig::setting("modinfo.by_name.$m.chat_provider");
				if ($provider_info && $provider_info['id'] == $id) {
					require_once $provider_info['file'];
					$provider = $provider_info['class']::instance();
				}
			}
			self::$providers[$id] = $provider;
		}

		return self::$providers[$id];
	}

	public function render() {
		$logo = $this->logo();
		return <<<HTML
<div id="chat-container">
	<div id="chat-header">
		<img src="$logo" class="chat-logo">
	</div>
	<div id="chat-content">
		<div id="chat-tabs-container"></div>
		<div id="chat-tabs-pages"></div>
	</div>
</div>
HTML;
	}

	public function finalize_page($page) {
		$json = getJSONObj();
		$path = 'themes/Default/chat.css';
		$page->add_css_include($path, null, LOAD_PRIORITY_THEME);
		$page->add_js_include('include/chat/chat.js', null, LOAD_PRIORITY_HEAD);
		$options = [
			'tabs' => $this->getTabs(),
		];
		$this->extendOptions($options);
		$page->add_js_literal('new LiveChat(' . $json->encode($options) . ', ' . $this->jsImplName() . ');', null, LOAD_PRIORITY_FOOT);
	}


	public function jsImplName() {return '';}

	abstract public function logo();
	abstract public function getTabs();
	abstract public function isEnabled();
	abstract public function isGuiEnabled();
	abstract public function extendOptions(&$options);
	abstract public function registerEndpoints($server);
	
	abstract public function renderTranscript($text);
	abstract public function renderTranscriptForSummary($text);
	abstract public function getLocalizedName();
}
