var LiveChat = function(options, implName) {
	SUGAR.onDomReady(function() {
		this.state = {};
		this.options = options || {};
		this.impl = implName ? new implName() : {};
		this.tabsContainer = document.getElementById('chat-tabs-container');
		this.pagesContainer = document.getElementById('chat-tabs-pages');
		this.initialize();
	}.bind(this));
}

LiveChat.prototype.initialize = function() {
	window.addEventListener('beforeunload', this.saveState.bind(this));
	this.restoreState();
	if (this.impl.initialize)
		this.impl.initialize(this);
	this.renderTabs(true);
}

LiveChat.prototype.restoreState = function() {
	var value = window.sessionStorage.getItem('chat.state');
	if (value !== null) {
		try {
			var decoded = JSON.parse(value);
			this.state = decoded;
		} catch (e) {
		}
	}
}

LiveChat.prototype.getState = function(key, defaultValue) {
	var parts = key.split('.');
	var source = this.state;
	for (var i = 0; i < parts.length; i++) {
		var k = parts[i];
		if (source.hasOwnProperty(k)) {
			source = source[k];
		} else {
			source = defaultValue;
			break;
		}
	}
	return source;
}

LiveChat.prototype.saveState = function() {
	 window.sessionStorage.setItem('chat.state', JSON.stringify(this.state));
}


LiveChat.prototype.renderTabs = function(initial) {
	var self = this;
	this.options.tabs.forEach(function(tab) {
		if (!tab.tabEl) {
			var tabEl = document.createElement('div');
			tabEl.className = 'chat-tab';
			tabEl.setAttribute('data-tab', tab.id);
			var icon = document.createElement('img');
			icon.className = 'chat-tab-icon';
			icon.setAttribute('src', tab.icon);
			tabEl.appendChild(icon);
			this.tabsContainer.appendChild(tabEl);
			tabEl.onclick = function() {
				self.activateTab(tab.id);
				return false;
			};
			tab.tabEl = tabEl;
		}
	}.bind(this));
	if (initial)
		this.activateTab(this.getState('currentTab') || this.options.tabs[0].id);

}

LiveChat.prototype.withTab = function(id, callback) {
	var tab = this.options.tabs.find(function(tab) {
		return tab.id == id;
	});
	if (!tab) {
		return;
	}
	callback(tab);
}

LiveChat.prototype.activateTab = function(id) {
	var tab = this.options.tabs.find(function(tab) {
		return tab.id == id;
	});
	if (!tab) {
		return;
	}
	this.options.tabs.forEach(function(tab) {
		tab.tabEl.className = 'chat-tab';
		if (tab.contentEl)
			tab.contentEl.className = 'chat-tabs-page';
	});
	if (!tab.contentEl) {
		tab.contentEl = document.createElement('div');
		tab.contentEl.className = 'chat-tabs-page';
		this.pagesContainer.appendChild(tab.contentEl);
		if (this.impl.renderTab) {
			this.impl.renderTab(this, tab);
		}
	}
	tab.tabEl.className = 'chat-tab active';
	tab.contentEl.className = 'chat-tabs-page active';
	this.state.currentTab = tab.id;
}
		
LiveChat.prototype.renderTab = function(id) {
	this.withTab(id, function(tab) {
		if (this.impl.renderTab) {
			this.impl.renderTab(this, tab);
		}
	}.bind(this));
}

LiveChat.prototype.tabLoading = function(tab) {
	if (tab.contentEl) {
		tab.contentEl.innerHTML = '';
		var img = document.createElement('img');
		img.className = 'loading';
		img.src = 'include/chat/images/loading.gif';
		tab.contentEl.appendChild(img);
	}
}


