<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

require_once('include/utils/basic_utils.php');
require_once('include/utils/external_cache.php');

require_once('include/config/ConfigManager.php');

function AppConfig($n, $d=null) {
	return AppConfig::setting($n, $d);
}

class AppConfig {

	// manage config
	/** @var ConfigManager $_config  */
	static $_config = null;
	static function load_config() {
		self::$_config = new ConfigManager;
		self::$_config->loadConfig();
	}
	static function setting($name, $default=null, $standard=false) {
		if(! isset(self::$_config))
			self::load_config();
		return self::$_config->getSetting($name, $default, $standard);
	}
	static function get_sequence_prefix($name, $category=null, $format=true) {
		if(! isset(self::$_config))
			self::load_config();
		return self::$_config->getSequencePrefix($name, $category, $format);
	}
	static function current_sequence_value($name, $category=null) {
		if(! isset(self::$_config))
			self::load_config();
		return self::$_config->currentSequenceValue($name, $category);
	}
	static function next_sequence_value($name, $category=null) {
		if(! isset(self::$_config))
			self::load_config();
		return self::$_config->nextSequenceValue($name, $category);
	}
	static function local_config_found() {
		if(! isset(self::$_config))
			self::load_config();
		return self::$_config->localConfigFound();
	}
	static function local_config_path() {
		return ConfigManager::local_config_path();
	}
	static function update_local($settings, $merge=false, $push_db=true) {
		return self::$_config->updateLocal($settings, $merge, $push_db);
	}
	static function set_local($key, $val, $merge=false, $push_db=true) {
		return self::$_config->setLocal($key, $val, $merge, $push_db);
	}
	static function save_local($category=null, $push_db=true, $out_path=null) {
		return self::$_config->saveLocal($category, $push_db, $out_path);
	}
	static function extract_settings($keys, $standard=false, $prefix='') {
		return self::$_config->extractSettings($keys, $standard);
	}
	static function read_only() {
		return self::setting('site.performance.read_only', false);
	}
	static function invalidate_cache($category, $limit=null) {
		global $db;
		if(! $category || $category == 'themes')
			self::cache_clear('AppConfig.get_themes_config');
		if ($category == 'acl' && $db->checkConnection()) {
			self::set_local('misc.acl_updated', gmdate('Y-m-d H:i:s'));
			self::save_local();
		}
		return self::$_config->invalidateCache($category, $limit);
	}
	static function set_expiry($category, $value=null) {
		if(! isset($value)) $value = microtime(true);
		return self::$_config->setExpiry($category, $value);
	}
	static function get_expiry($category) {
		return self::$_config->getExpiry($category);
	}
	static function clear_expiry($category) {
		return self::$_config->clearExpiry($category);
	}

	static function is_mobile()
	{
		global $themeInstance;
		if(isset($themeInstance)) {
			return $themeInstance->isMobile();
		}
		return false;
	}

	static function is_mobile_module($name)
	{
		$mods = self::setting("modinfo.mobile_modules", array());
		return in_array($name, $mods);
	}

	static function is_mobile_bean($name)
	{
		$beans = self::setting('modinfo.primary_beans');
		$mods = self::setting("modinfo.mobile_modules", array());
		$mod = array_search($name, $beans);
		return $mod && in_array($mod, $mods);
	}

	static function get_mobile_columns() {
		$layout_columns = self::setting('layout.mobile_columns', 0);
		if ($layout_columns < 1) {
			if(isset($_SESSION['res_width']) && isset($_SESSION['res_height'])) {
				$max = max($_SESSION['res_width'], $_SESSION['res_height']);
				if($max >= 1024) $layout_columns = 2;
				else $layout_columns = 1;
			}
		}
		if($layout_columns > 2)
			$layout_columns = 1;
		return $layout_columns;
	}

	// manage DBCache
	/** @var DBCache $_dbcache */
	static $_dbcache = null;
	static function load_db_cache() {
		require_once('include/database/DBCache.php');
		self::$_dbcache = new DBCache;
	}
	static function db_is_cached() {
		if(! isset(self::$_dbcache))
			self::load_db_cache();
		return self::$_dbcache->getStats();
	}
	static function db_cache_disable($state=true) {
		if(! isset(self::$_dbcache))
			self::load_db_cache();
		self::$_dbcache->setDisabled($state);
	}
	static function db_cache_get_stats() {
		if(! isset(self::$_dbcache))
			self::load_db_cache();
		return self::$_dbcache->stats;
	}
	static function db_row($cls, $id, $autoFetch=true, $encode=false) {
		if(! isset(self::$_dbcache))
			self::load_db_cache();
		return self::$_dbcache->getRow($cls, $id, $autoFetch, $encode);
	}
	static function db_object($cls, $id, $autoFetch=true, $noDefault=false) {
		if(! isset(self::$_dbcache))
			self::load_db_cache();
        return self::$_dbcache->getObject($cls, $id, $autoFetch, $noDefault);
	}
	static function db_all_rows($cls, $autoFetch=true, $encode=false) {
		if(! isset(self::$_dbcache))
			self::load_db_cache();
		return self::$_dbcache->getAllRows($cls, $autoFetch, $encode);
	}
	static function db_all_objects($cls, $autoFetch=true, $noDefault=false) {
		if(! isset(self::$_dbcache))
			self::load_db_cache();
		return self::$_dbcache->getAllObjects($cls, $autoFetch, $noDefault);
	}
	static function db_update_row($cls, $row) {
		if(! isset(self::$_dbcache))
			self::load_db_cache();
		return self::$_dbcache->updateRow($cls, $row);
	}
	static function db_invalidate($cls, $id=null) {
		if(! isset(self::$_dbcache))
			self::load_db_cache();
		return self::$_dbcache->invalidate($cls, $id);
	}
	static function db_check_standard_records($cls, $ifCached=false) {
		if(! isset(self::$_dbcache))
			self::load_db_cache();
		return self::$_dbcache->checkStandardRecords($cls, $ifCached);
	}
	static function db_is_auto_fetched($cls) {
		if(! isset(self::$_dbcache))
			self::load_db_cache();
		return self::$_dbcache->isAutoFetched($cls);
	}


	// check install status (used by entryPoint)
	static function installed() {
		if(! self::local_config_found())
			return false;
		if(! self::setting('database.primary'))
			return false;
		return true;
	}
	static function can_install() {
		return ! self::setting('config.installer_locked');
	}
	static function in_installer() {
		return defined('IAH_IN_INSTALLER') && constant('IAH_IN_INSTALLER');
	}
	static function lock_installer($demo=false) {
		if(! self::can_install())
			return true;
		self::set_local('config.installer_locked', true);
		if($demo)
			self::set_local('config.is_demo_site', true);
		return self::save_local();
	}
	static function check_database_version(&$info) {
		$key = 'info.sugar_version';
		$app_ver = self::version();
		$db_ver = self::db_version();
		$stored_ver = self::setting($key);
		if(empty($stored_ver))
			self::set_local($key, $db_ver);
		else if($db_ver != $stored_ver) {
			$info = array('message' => "1CRM application version $db_ver does not match database version $stored_ver.");
			return false;
		}
		return true;
	}
	static function restrict_entry_point($redir=true, $die=true) {
		$msg = null;
		if(! self::installed()) {
			$msg = 'Application not yet installed.';
		}
		else if(self::upgrade_in_progress()) {
			$msg = 'Application upgrade in progress.';
		}
		else if(! self::check_database_version($info)) {
			header('HTTP/1.1 503 Service Unavailable');
			$msg = $info['message'];
		}
		else if($redir && ($redir = self::get_auto_redirect()) ) {
			header("Location: {$redir[0]}");
			$msg = $redir[2];
		}
		if($msg && $die) {
			echo $msg;
			// not sugar_die because this entry point shouldn't be called at all
			exit;
		}
		return $msg;
	}

	static function update_sugar_version($version, $db_version = null)
	{
		if (!$db_version)
			$db_version = $version;
		$text = '<';
		$text .= <<<EOT
?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
\$sugar_version      = '$version';
\$sugar_db_version   = '$db_version';
\$rcs_revision_number = '';
\$sugar_flavor       = 'IAH';

EOT;
		if ( $f = @fopen('sugar_version.php', 'w') ) {
			fwrite($f, $text);
			fclose($f);
			self::set_local('info.sugar_version', $db_version);
			self::save_local();
			return true;
		}
		return false;
	}

	static function get_installer_strings()
	{
		global $current_language;
		if (defined('IAH_IN_INSTALLER')) {
			$default_lang = self::setting('install.defaults.language');
			$lang = array_get_default($_SESSION, 'language', $default_lang);
		} else {
			$lang = isset($current_language) ? $current_language : 'en_us';
		}
		$lang_path = "install/language/lang.{$lang}.strings.php";
		if(! file_exists($lang_path)) {
			$lang_path = "install/language/lang.en_us.strings.php";
			$lang = 'en_us';
		}
		$mod_strings = ConfigParser::load_file($lang_path);

		if($lang != 'en_us') {
			$lang_base = ConfigParser::load_file('install/language/lang.en_us.strings.php');
			$mod_strings = sugarArrayMerge($lang_base, $mod_strings);
		}
		return $mod_strings;
	}


	// quick config accessors
	static function host_name($default='') {
		return self::setting('site.host_name', $default);
	}

	static function request_host_name() {
		$domain = null;
		$extra = self::setting('site.extra_urls', []);
		foreach ($extra as $extra_url) {
			$parsed = parse_url($extra_url);
			if ($parsed && isset($parsed['host']) && $parsed['host'] === $_SERVER['HTTP_HOST']) {
				$domain = $parsed['host'];
			}
		}
		if ($domain === null)
			$domain = self::setting('site.host_name');
		return $domain;
	}

	static function site_url($default='') {
		if(defined('OVERRIDE_SITE_URL')) {
			$url = constant('OVERRIDE_SITE_URL');
			$override = 1;
		}
		else
			$url = self::setting('site.base_url', $default);
		if($url && substr($url, -1) !== '/') $url .= '/';
		if ($url && empty($override)) {
			$parts = parse_url($url);
			if ($parts) {
				$port = array_get_default($parts, 'port', array_get_default($_SERVER, 'SERVER_PORT', 80));
				$https = (array_get_default($parts, 'scheme') == 'https' || self::detect_https());
				$url = $https ? 'https://' : 'http://';
				$host = array_get_default($_SERVER, 'HTTP_HOST', $parts['host']);
				$col_pos = strpos($host, ':');
				if ($col_pos)
					$host = substr($host, 0, $col_pos);
				$url .= $host;
				if($port != 80 && ($port != 443 || ! $https))
					$url .= ':' . $port;
				$url .= $parts['path'];
			}
		}

		return $url;
	}
	static function site_static_url($default='') {
		$url = self::setting('site.performance.static_url');
		if(empty($url))
			return $default;
		if(substr($url, -1) !== '/') $url .= '/';
		if($url == self::site_url())
			return $default;
		return $url;
	}
	static function multiviews_enabled() {
		return self::setting('site.performance.multiviews');
	}
	static function detect_https() {
		static $secure;
		if(! isset($secure)) {
			$secure = false;
			$override = self::setting('site.performance.https_override');
			if(defined('OVERRIDE_SITE_HTTPS'))
				$secure = constant('OVERRIDE_SITE_HTTPS');
			else if(isset($override))
				$secure = $override;
			else if(! empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off')
				$secure = true;
			else if(! empty($_SERVER['HTTP_X_FORWARDED_PROTO']) && $_SERVER['HTTP_X_FORWARDED_PROTO'] == 'https')
				$secure = true;
			else if(! empty($_SERVER['HTTP_X_FORWARDED_SSL']) && $_SERVER['HTTP_X_FORWARDED_SSL'] !== 'off')
				$secure = true;
		}
		return $secure ? 'https' : false;
	}
	static function detect_http2() {
		if(self::setting('site.performance.http2_disabled'))
			return false;
		if(isset($_SERVER['SERVER_PROTOCOL']) && substr($_SERVER['SERVER_PROTOCOL'], 0, 6) == 'HTTP/2')
			return true;
		if(self::detect_https() && isset($_SERVER['HTTP_VIA']) && strpos($_SERVER['HTTP_VIA'], '2 nghttpx') !== false) {
			// nghttpx acting as http2 proxy
			return true;
		}
	}
	static function charset($default='UTF-8') {
		return self::setting('layout.charset', $default);
	}
	static function unique_key($default='') {
		return self::setting('config.unique_key', $default);
	}
	static function version() {
		return $GLOBALS['sugar_version'];
	}
	static function db_version() {
		return $GLOBALS['sugar_db_version'];
	}
	static function flavor() {
		return $GLOBALS['sugar_flavor'];
	}
	static function dev_branch() {
		static $branch;
		if(! isset($branch)) {
			$branch = '';
			if(self::setting('config.git_enabled') && is_readable('.git/HEAD')) {
				$fc = file_get_contents('.git/HEAD');
				if($fc && ($br = explode('/', trim($fc), 3))) {
					$branch = $br[count($br)-1];
				}
			}
		}
		return $branch;
	}
	static function js_custom_version() {
		if(self::setting('site.js_always_reload')) {
			global $js_reload_ver;
			if(! isset($js_reload_ver)) $js_reload_ver = substr(md5(time()), -6);
			return $js_reload_ver;
		}
		return self::setting('site.js_custom_version', '0');
	}
	static function promote_js_custom_version() {
		self::set_local('site.js_custom_version', self::js_custom_version() + 1);
		self::save_local();
	}
	static function language_version() {
		$val = self::setting('lang.index.current.version', '0');
		return substr($val, 0, 10);
	}


	static function theme() {
		if (!empty($_COOKIE['ANDROID_APP']) || !empty($_COOKIE['IOS_APP']))
			return 'Claro';
		if(! empty($_SESSION['authenticated_user_theme']))
			$theme = $_SESSION['authenticated_user_theme'];
		else
			$theme = self::setting('layout.defaults.theme', '');
		return $theme;
	}
	static function get_themes($mobile=null) {
		if(! isset($mobile)) $mobile = self::is_mobile();
		$cfg = self::get_themes_config();
		$ret = array();
		foreach($cfg as $k => $v) {
			if($mobile === 'mobile_support') {
				if(empty($v['mobile_support']))
					continue;
			}
			else if($mobile && empty($v['mobile_friendly']))
				continue;
			else if(! empty($v['mobile_support']))
				continue;
			$ret[$k] = array_get_default($v, 'name', $k);
		}
		ksort($ret);
		return $ret;
	}
	static function get_themes_config() {
		static $banned;
		if(! isset($banned)) $banned = self::setting('layout.obsolete_themes', array());
		$key = "AppConfig.get_themes_config";
		$th = self::cache_get($key, $found);
		if(empty($found)) {
			$th = array();
			foreach(glob('themes/*', GLOB_NOSORT|GLOB_ONLYDIR) as $theme) {
				$theme = substr($theme, 7);
				if(in_array($theme, $banned))
					continue;
				if(! is_readable("themes/$theme/config.php"))
					continue;
				$cfg = ConfigParser::load_file("themes/$theme/config.php");
				if(! empty($cfg))
					$th[$theme] = $cfg;
			}
			self::cache_put($key, $th);
		}
		return $th;
	}
	static function get_theme_config($theme=null) {
		if(! isset($theme)) $theme = self::theme();
		$themes = self::get_themes_config();
		return array_get_default($themes, $theme, array());
	}
	static function detect_mobile() {

		$checker = new Mobile_Detect();
		if($checker->isMobile()) {
			return array('tablet' => $checker->isTablet(), 'grade' => $checker->mobileGrade());
		}
		return false;
	}
	static function init_user_theme($custom=null) {
		if(empty($_SESSION['authenticated_user_id'])) {
			// disabled in 8.5 - using mobile-friendly themes
			// $default_mobile = self::detect_mobile();
			// if($default_mobile) $_SESSION['mobile_detected'] = $default_mobile;
			$set_mobile = array_get_default($_REQUEST, 'mobile');
		}

		if(self::setting('site.force_mobile'))
			$mobile = true;
		else if(isset($set_mobile))
			$mobile = !! $set_mobile;
		else
			$mobile = array_get_default($_SESSION, 'mobile_mode', ! empty($default_mobile));
		$_SESSION['mobile_mode'] = $mobile;

		if($mobile)
			$default_theme = self::setting('layout.defaults.mobile_theme');
		else
			$default_theme = self::setting('layout.defaults.theme');

		if(! empty($custom))
			$theme = $custom;
		else if(! empty($_REQUEST['user_theme'])) {
			$theme = $_REQUEST['user_theme'];
			unset($_SESSION['no_persist_theme']);
		}
		else if(! empty($_SESSION['authenticated_user_theme']))
			$theme = $_SESSION['authenticated_user_theme'];
		else if($mobile && ! empty($_COOKIE['ck_mobile_theme']))
			$theme = $_COOKIE['ck_mobile_theme'];
		else if(! empty($_COOKIE['ck_login_theme']))
			$theme = $_COOKIE['ck_login_theme'];
		$avail_themes = self::get_themes($mobile);
		if(empty($theme) || ! isset($avail_themes[$theme]) || ! is_dir('themes/'.$theme))
			$theme = $default_theme;

		if(! empty($_SESSION['authenticated_user_theme']) && $_SESSION['authenticated_user_theme'] != $theme)
			$GLOBALS['theme_changed'] = true;
		$GLOBALS['theme'] = $_SESSION['authenticated_user_theme'] = $theme;
		return $theme;
	}


	static function language($custom=null, $guess=null, $no_default=false) {
		if(! empty($_SESSION['authenticated_user_language']) && ! $custom) {
			$lang = $_SESSION['authenticated_user_language'];
		} else {
			$lang = null;
			$langs = self::get_languages();
			$test = array();
			foreach($langs as $nm => $lbl) {
				$lnm = strtolower($nm);
				$test[$lnm] = $nm;
				if( ($p = strpos($lnm, '_')) ) {
					$lnm = substr($lnm, 0, $p);
					if(! isset($test[$lnm]))
						$test[$lnm] = $nm;
				}
			}
			if($custom) {
				if(! is_array($custom)) $custom = array($custom);
				foreach($custom as $name) {
					$name = strtolower($name);
					if(isset($test[$name])) {
						$lang = $test[$name];
						break;
					}
				}
			}
			if($no_default) return $lang;
			if(! $lang && ! empty($_SESSION['authenticated_user_language']))
				$lang = $_SESSION['authenticated_user_language'];
			if(! $lang && $guess) {
				if(! is_array($guess)) $guess = array($guess);
				foreach($guess as $name) {
					$name = strtolower($name);
					if(isset($test[$name])) {
						$lang = $test[$name];
						break;
					}
				}
			}
			if(! $lang) {
				$accept = parse_accept_language();
				foreach($accept as $l) {
					if(isset($test[$l['name']])) {
						$lang = $test[$l['name']];
						break;
					}
				}
			}
			if(! $lang) {
				$base = self::setting('locale.base_language');
				$lang = self::setting('locale.defaults.language', $base);
			}
		}
		return $lang;
	}
	static function get_languages() {
		return self::setting("lang.labels", array());
	}
	static function init_user_language($custom=null, $guess=null) {
		global $current_language, $app_strings, $app_list_strings;

		$checks = array();
		if($guess) {
			if(! is_array($guess)) $guess = array($guess);
		} else
			$guess = array();
		if(! empty($custom)) {
			if(is_array($custom)) $checks = $custom;
			else $checks[] = $custom;
		}
		if(! empty($_REQUEST['user_language']))
			$checks[] = $_REQUEST['user_language'];
		if(! empty($_COOKIE['ck_login_language']))
			$guess[] = $_COOKIE['ck_login_language'];
		$language = self::language($checks, $guess);

		if(! empty($_SESSION['authenticated_user_language']) && $_SESSION['authenticated_user_language'] != $language)
			$GLOBALS['lang_changed'] = true;
		$current_language = $_SESSION['authenticated_user_language'] = $language;
		$app_strings = return_application_language($current_language);
		$app_list_strings = return_app_list_strings_language($current_language);

		return $language;
	}


	static function cache_dir() {
		return self::setting('site.paths.cache_dir', 'cache/');
	}
	static function files_dir() {
		return self::setting('site.paths.files_dir', 'files/');
	}
	static function custom_dir() {
		return self::setting('site.paths.custom_dir', 'custom/');
	}
	static function upload_dir() {
		return self::setting('site.paths.upload_dir', self::files_dir() . 'upload/');
	}
	static function system_timezone($default=null) {
		$tz = self::setting('site.timezone');
		if(! $tz) $tz = (! empty($default)) ? $default : self::setting('locale.defaults.timezone');
		return $tz;
	}
	static function is_B2C() {
		return (self::setting('company.business_model') == 'B2C');
	}

	static function current_user_id() {
		global $current_user;
		if(isset($current_user)) return $current_user->id;
		return null;
	}
	static function user_preference($name, $category='global', $uid=null, $default=null) {
		if(! isset($uid)) $user_key = 'current_user';
		else if(! $uid) return null;
		else $user_key = "by_user.$uid";
		$key = "userpref.$user_key.$category";
		if(isset($name)) $key .= '.' . $name;
		return self::setting($key, $default);
	}
	static function is_admin($user=null) {
		if(! isset($user)) $user = self::current_user_id();
		else if(is_object($user)) $user = $user->id;
		return is_admin($user);
	}


	// timezone handling
	static function set_system_timezone($tz=null) {
		if(function_exists('date_default_timezone_set')) {
			if(! $tz)
				// no longer set in database - must be in standard config or local_config
				$tz = self::system_timezone();
			if(! $tz)
				$tz = ini_get('date.timezone');
			if($tz)
				date_default_timezone_set($tz);
		}
	}
	static function check_init_user() {
		global $current_user;
		if(! empty($current_user) && $current_user->id) {
			$module = array_get_default($_REQUEST, 'module');
			$action = array_get_default($_REQUEST, 'action');
			$ut = $current_user->getPreference('ut');
			if($module == 'Users' && ($action != 'InitUser' || $action != 'Logout'))
				;
			else if(empty($ut)) {
				if(is_demo_site() && ($sys_tz = self::system_timezone()) ) {
					$current_user->setPreference('timezone', $sys_tz);
					$current_user->setPreference('ut', 1);
				} else
					return false;
			}
			else if($current_user->need_new_password)
				return false;
		}
		return true;
	}

	static function check_2fa() {
		global $current_user;
		$action = array_get_default($_REQUEST, 'action');

		if (!empty($current_user) && $current_user->id) {
			require_once('include/MultiFA/MultiFactorAuthenticator.php');
			$auth = new \OneCRM\MultiFA\MultiFactorAuthenticator($current_user, $GLOBALS['log']);
			$require2fa = self::setting('layout.list.require_2fa', false);
			$userHas2FAEnabled = $auth->isEnabled();
			$check2fa = $_COOKIE['check_2fa'];

			if ($check2fa == '1' && $require2fa && !$userHas2FAEnabled && $action != 'MultiFARecoveryCodesPDF') {
				return true;
			}

		}
		return false;
	}


	// this is mainly for compatibility
	static function load_module_index($fetch_beanlist=true, $set_globals=true) {
		$GLOBALS['moduleList'] = self::setting('modinfo.index.normal');
		$GLOBALS['modInvisList'] = self::setting('modinfo.index.hidden');
		$GLOBALS['modSemiInvisList'] = self::setting('modinfo.index.manual');
		$submods = self::setting('modinfo.index.submodules');
		foreach($submods as $mod => $sms)
			foreach($sms as $submod)
				$target['subModuleList'][$submod] = $mod;

		if($fetch_beanlist) {
			// only putting 'primary' beans into beanList, use another method for full index
			$GLOBALS['beanList'] = self::setting('modinfo.primary_beans');
			$GLOBALS['beanFiles'] = self::setting('model.index.bean_files');
		}
	}

	static function module_primary_bean($module) {
		return self::setting("modinfo.by_name.$module.detail.primary_bean", false);
	}

	static function module_for_model($model) {
		if($model instanceof ModelDef)
			$model = $model->name;
		return self::setting("model.detail.$model.module_dir");
	}

	// page init

	static function init_page_controller($params=null) {
		global $pageController, $module, $action;
		global $beanList, $beanFiles, $controllerList, $controllerFiles;
		global $defaultModule, $defaultAction;

		if(isset($params)) foreach($params as $k => $v) $_REQUEST[$k] = $v;
		$module = array_get_default($_REQUEST, 'module', '');
		$record = array_get_default($_REQUEST, 'record', '');

		// Initialize module controller
		$defaultModule = self::setting('layout.default_page.module', 'Home');
		$defaultAction = self::setting('layout.default_page.action', 'index');
		if(! $module) {
			$_REQUEST['module'] = $module = $defaultModule;
			$overrideAction = $defaultAction;
		}
		$modControllerCls = self::setting("modinfo.by_name.$module.detail.controller");
		if(! $modControllerCls) {
			if(isset($beanList[$module]) && isset($beanFiles[$beanList[$module]]))
				$modControllerCls = 'ClassicController';
			else
				$modControllerCls = 'ModuleController';
		}
		$clsFile = self::setting("modinfo.controllers.$modControllerCls");
		if(! $clsFile)
			$clsFile = "include/application/$modControllerCls.php";
		require_once($clsFile);
		$pageController = new $modControllerCls($module);
		if(isset($overrideAction))
			$pageController->set_action($overrideAction);
		$_REQUEST['action'] = $action = $pageController->get_action();

		$pageController->init_globals();
	}


	// display

	static function get_widget_spec($widget) {
		if(! $widget || ! is_string($widget)) return;
		$spec = self::setting("display.widgets.$widget");
		if($spec) {
			if(isset($spec['path']))
				require_once($spec['path']);
			if(class_exists($spec['class']))
				return $spec;
			else
				$GLOBALS['log']->error("Widget class not defined: {$widget}");
		}
	}


	// external cache accessors

	static function cache_get($key, &$found, $no_local=false) {
		return sugar_cache_get($key, $found, $no_local);
	}
	static function cache_get_timed($key, $expiry, &$found, $no_local=true) {
		$val = sugar_cache_get($key, $found, $no_local);
		if($found) {
			if(! is_array($val) || ! isset($val[0]) || $val[0] < $expiry)
				$found = false;
			else {
				$ret = $val[1];
				unset($val);
				return $ret;
			}
		}
	}
	static function cache_get_tagged($key, $tag, &$found, $no_local=true) {
		// $tag may be null to skip check
		$val = sugar_cache_get($key, $found, $no_local);
		if($found) {
			if(! is_array($val) || ! isset($val[0]) || (isset($tag) && $tag != $val[0]))
				$found = false;
			else {
				$ret = $val[1];
				unset($val);
				return $ret;
			}
		}
	}
	static function cache_put($key, $val, $no_local=false, $replace=false) {
		return sugar_cache_put($key, $val, $no_local, $replace);
	}
	static function cache_put_timed($key, $val, $no_local=true, $replace=true) {
		$val = array(microtime(true), $val);
		return sugar_cache_put($key, $val, $no_local, $replace);
	}
	static function cache_put_tagged($key, $tag, $val, $no_local=true, $replace=true) {
		$val = array($tag, $val);
		return sugar_cache_put($key, $val, $no_local, $replace);
	}
	static function cache_clear($key) {
		return sugar_cache_clear($key);
	}
	static function cache_reset() {
		return sugar_cache_reset();
	}
	static function cache_enabled() {
		return sugar_cache_enabled();
	}
	static function config_cache_enabled() {
		return ! defined('CONFIG_CACHE_DISABLED') || ! constant('CONFIG_CACHE_DISABLED');
	}
	static function cache_file_read($path, $cache_key=null, $last_update=null) {
		$index = null;
		if(! self::config_cache_enabled()) $cache_key = null;
		clearstatcache(false, $path);
		if(file_exists($path)) {
			if(isset($cache_key)) {
				$mt = @filemtime($path);
				$index = self::cache_get_timed($cache_key, max($mt, $last_update), $key_found);
			}
			if(empty($key_found)) {
				$index = include($path);
				if(is_string($index)) $index = unserialize($index);
				if(isset($cache_key))
					self::cache_put_timed($cache_key, $index, true, true);
			}
		}
		return $index;
	}
	static function cache_file_write(array $data, $path, $cache_key=null) {
		$temp = tempnam('cache/temp/', basename($path));
		if(! $temp) return false;
		$tfp = fopen($temp, 'wb');
		if(! $tfp) return false;
		$content = self::cache_export($data, true);
		//$content = "<<<EOS\n" . serialize($data) . "\nEOS"; // slower and takes more memory, surprisingly
		fwrite($tfp, '<?php return ');
		fwrite($tfp, $content);
		fwrite($tfp, ";\n?>\n");
		fclose($tfp);
		rename($temp, $path);
		if(function_exists('opcache_invalidate'))
			opcache_invalidate($path);
		if(isset($cache_key) && self::config_cache_enabled())
			self::cache_put_timed($cache_key, $data, true, true);
		return true;
	}
	static function cache_export(&$data, $return=false) {
		//return var_export($data, $return); // for better readability
		$ret = '';
		if(is_array($data))
			self::cache_export_array($data, $ret);
		else if(is_object($data))
			throw new IAHInternalError('Cannot cache objects');
		else
			$ret .= var_export($data, true);
		if($return) return $ret;
		echo $ret;
	}
	static function cache_export_array(array &$data, &$ret) {
		$ret .= 'array(';
		$i = 0;
		foreach($data as $k => $v) {
			if($i ++) $ret .= ',';
			$ret .= var_export($k, true) . '=>';
			if(is_array($v))
				self::cache_export_array($v, $ret);
			else
				$ret .= self::cache_export($v, true);
		}
		$ret .= ')';
	}


	static function get_request_url() {
		$url = self::detect_https() ? 'https://' : 'http://';
		$host = array_get_default($_SERVER, 'HTTP_HOST');
		if(! $host) $host = self::setting('site.host_name');
		$url .= $host;
		$port = array_get_default($_SERVER, 'SERVER_PORT', 80);
		if($port != 80) $url .= ":$port";
		$url .= array_get_default($_SERVER, 'REQUEST_URI', '/');
		return $url;
	}

	static $override_compression = null;
	static function get_server_compression() {
		if(self::$override_compression)
			return self::$override_compression;
		if(php_sapi_name() == 'cli')
			return 'cli'; // disable compression on CLI output
		if(ini_get('output_buffering') && ini_get('output_handler') == 'ob_gzhandler')
			return 'ob_gzhandler';
		else if(ini_get('zlib.output_compression') || get_cfg_var('zlib.output_compression'))
			return 'zlib';
		return null;
	}

	static $proxy_info = null;
	static function get_proxy_info() {
		if(! isset(self::$proxy_info)) {
			self::$proxy_info = false;
			$proxy_config = self::setting('proxy');
			if(is_array($proxy_config)) {
    			if(! empty($proxy_config['enabled'])){
					$info = array();
    				$info['host'] = $proxy_config['host_name'];
    				$info['port'] = $proxy_config['port'];
					if(!empty($proxy_config['auth_req'])){
						$info['auth'] = 1;
						$info['username'] = $proxy_config['user_name'];
						$info['password'] = $proxy_config['password'];
					} else
						$info['auth'] = 0;
					self::$proxy_info = $info;
    			}
			}
		}
		return self::$proxy_info;
	}

	static function is_extern_url($exclude=null) {
		// is the current URL limited to certain entry points (returns name)
		$base = self::site_url();
		$nonsec_base = preg_replace('~^https~', 'http', $base);
		$urls = array(
			'survey' => 'site.survey_url',
		);
		$url = self::detect_https() ? 'https://' : 'http://';
		$host = array_get_default($_SERVER, 'HTTP_HOST');
		if(! $host)
			return;
		$url .= $host;
		$port = array_get_default($_SERVER, 'SERVER_PORT', 80);
		if($port != 80) $url .= ":$port";
		$url .= array_get_default($_SERVER, 'REQUEST_URI', '/');
		foreach($urls as $nm => $cfg) {
			if(isset($exclude) && $exclude == $nm)
				continue;
			$chk = self::setting($cfg);
			if(! $chk) return;
			if(substr($chk, -1) != '/')
				$chk .= '/';
			if($chk != $base && $chk != $nonsec_base && substr($url, 0, strlen($chk)) == $chk) {
				return $nm;
			}
		}
		return false;
	}

	static function get_auto_redirect() {
		$base = self::site_url();
		if(self::setting('site.security.auto_https_redirect') && ! defined('DISABLE_HTTPS_REDIRECT')) {
			if(substr($base, 0, 6) == 'https:' && ! self::detect_https()) {
				$pfx = substr($base, 0, strpos($base, '/', 8));
				$url = $pfx . array_get_default($_SERVER, 'REQUEST_URI', '/');
				return array($url, 'secure', 'Secure URL required');
			}
		}
	}

	// session handling

	static $session_config_category = 'site.';
	static $_new_session = false;
	static function session_start($cfg_prefix=null, $basic=false, $no_close=false, $params=null) {
		// do not allow session to be started on 'static' URL (no cookies = less bandwidth)
		$static_url = self::site_static_url();
		$req_url = self::get_request_url();
		if($static_url && ! strcmp(substr($req_url, 0, strlen($static_url)), $static_url))
			return false;

		self::$_new_session = false;

		if(! $no_close)
			@session_write_close(); // TODO - check session auto-start setting?
		if(! isset($cfg_prefix))
			$cfg_prefix = self::$session_config_category;

		// Custom session name
		$sess_name = self::setting($cfg_prefix . 'session.name');
		if($sess_name)
			session_name($sess_name);
		// Custom session path
		$sess_path = self::setting($cfg_prefix . 'session.dir', '');
		if($sess_path)
			session_save_path($sess_path);

		$domain = null;
		$url = self::setting('site.base_url');
		$parsed = parse_url($url);
		if ($parsed && isset($parsed['host'])) {
			$domain = $parsed['host'];
		}
		$extra = self::setting('site.extra_urls', []);
		foreach ($extra as $extra_url) {
			$parsed = parse_url($extra_url);
			if ($parsed && isset($parsed['host']) && $parsed['host'] === $_SERVER['HTTP_HOST']) {
				$domain = $parsed['host'];
			}
		}
		session_set_cookie_params([
			'httponly' => true,
			'secure' => !self::setting('site.bypass_secure_session_cookie'),
			'samesite' => 'Lax',
			'domain' => $domain,
		]);
		ini_set('session.use_strict_mode', 1);
		// Allow for the session information to be passed via the URL for printing.
		$sess_name = session_name();
		if(isset($_GET[$sess_name])){
			if(! empty($_COOKIE[$sess_name]) && strcmp($_GET[$sess_name],$_COOKIE[$sess_name]) == 0) {
				session_id($_REQUEST[$sess_name]);
			} else {
				unset($_GET[$sess_name]);
			}
		}

		if(@session_start() === false)
			return false; // effective on PHP 5.3.0+ only
		
		if (empty(self::$_new_session)) {
			$csrf_checked = false;
			do {
				if (defined('IAH_IN_UPGRADE')) {
					$csrf_checked = true;
					break;
				}
				if (version_compare(self::db_version(), "8.7.9", ">=")) {
					$protect = constant('ONECRM_CSRF_PROTECT');
					if ($protect !== 'none' && php_sapi_name() !== 'cli') {
						if (!isset($_SESSION['csrf_cookie_value'])) {
							break;
						}
						$cookie_name = 'UNKNOWN';
						$cookie_value = $_SESSION['csrf_cookie_value'];
						$input_name = $_SESSION['csrf_input_name'];
						if (($_COOKIE[$cookie_name] ?? null) !== $cookie_value) {
							break;
						}
						if ($_SERVER['REQUEST_METHOD'] !== 'GET' && $protect !== 'relax') {
							$header_value = $_SERVER['HTTP_X_ONECRM_CSRF_TOKEN'] ?? null;
							$input_value = $input_name ? ($_REQUEST[$input_name] ?? null) : null;
							if ($header_value !== $cookie_value && $input_value !== $cookie_value) {
								die();
							}
						}
						$csrf_checked = true;
					} else {
						$csrf_checked = true;
					}
				} else {
					$csrf_checked = true;
				}
			} while (false);
			if (!$csrf_checked)
				self::$_new_session = 'csrf';
		}

		if(self::in_installer())
			return true;

		$server_unique_key = self::unique_key();
		if(isset($_SESSION['server_unique_key'])) {
			if($_SESSION['server_unique_key'] !== $server_unique_key) {
				$GLOBALS['log']->warn('Destroyed competing session: different unique_key');
				self::$_new_session = 'unique_key';
			}
		} else {
			self::$_new_session = 'blank';
		}

		if(! validate_client_ip($mismatch)) {
			$GLOBALS['log']->warn("Destroyed session due to IP address mismatch, session/client: {$mismatch[0]}/{$mismatch[1]}");
			self::$_new_session = 'ip-mismatch';
		}

		if(self::$_new_session) {
			if(self::$_new_session !== 'blank') {
				if(session_regenerate_id(true) === false)
					return false;
				session_unset();
			}
			$_SESSION['server_unique_key'] = $server_unique_key;
			if(self::$_new_session == 'unique_key')
				$_SESSION['display_session_ended'] = 'LBL_SESSION_UNIQUE_KEY';
			else if(self::$_new_session == 'ip-mismatch')
				$_SESSION['display_session_ended'] = 'LBL_SESSION_IP_MISMATCH';
		}

		return self::session_init($params, $basic);
	}

	static function started_new_session() {
		return ! empty(self::$_new_session);
	}

	// used by installer
	static function test_session() {
		if(! empty($_SESSION['_test_session_']))
			return true;
		$_SESSION['_test_session_'] = 'sessions are available';
		@session_write_close();
		unset($_SESSION['_test_session_']);
		$start = self::session_start(null, true);
		if($start === false || empty($_SESSION['_test_session_'])) {
			@session_destroy();
			return false;
		}
		return true;
	}

	static function upgrade_in_progress() {
		if(self::setting('site.upgrade_in_progress')) return true;
	}

	static function session_init($params=null, $basic = false) {
		if(! is_array($params)) $params = array();

		$sess_ver = array_get_default($_SESSION, 'app_version');
		if(! self::$_new_session && (! $sess_ver || $sess_ver != self::version())) {
			unset($_SESSION['authenticated_user_theme']);
			unset($_SESSION['authenticated_user_language']);
		}
		$_SESSION['app_version'] = self::version();

		if (self::$_new_session || (defined('IAH_IN_UPGRADE') && empty($_SESSION['csrf_cookie_value']))) {
			$cookie_value = sha1(random_bytes(128));
			$_SESSION['csrf_cookie_value'] = $cookie_value;
			$_SESSION['csrf_input_name'] = '_' . sha1(random_bytes(128));;
			setcookie(
				'UNKNOWN',
				$cookie_value,
				[
					'expires' => time() + 365 * 24 * 60 * 60,
					'samesite' => 'Lax',
					'domain' => self::request_host_name(),
				]
			);
		}

		if (!$basic) {
			self::init_user_language(array_get_default($params, 'language'));
			self::init_user_theme(array_get_default($params, 'theme'));

			if(! self::upgrade_in_progress()) {
				require_once('modules/LicenseInfo/LicenseHistory.php');
				LicenseHistory::checkSessionCache();
			}
		}
		return true;
	}


	static function set_login_params() {
		$token_value = sha1(random_bytes(128));
		setcookie(
			'LOGIN',
			$token_value,
			[
				'expires' => time() + 365 * 24 * 60 * 60,
				'samesite' => 'Lax',
				'domain' => self::request_host_name(),
				'secure' => !self::setting('site.bypass_secure_session_cookie'),
			]
		);
	}

	static function timer_start() {
		return microtime(true);
	}
	static function timer_end($start) {
		$ts = microtime(true) - $start;
		$s = round($ts, 2).'s';
		return compact('ts', 's');
	}
	static function timer_print($start, $title=null) {
		pr2(self::timer_end($start)['s'], $title);
	}
	static function timer_log($start, $level='info', $message='') {
		global $log;
		$log->$level(sprintf("timer:%0.3f  %s", self::timer_end($start)['ts'], $message));
	}

	static function create_logger($name=null) {
		require_once('include/log/Logger.php');
		return new Katzgrau\KLogger\Logger($name);
	}

	static function track_login($success) {
		$found = false;
		$attempts = self::cache_get('failed_logins', $found);
		if (!$found) $attempts = 0;
		$updated = $attempts;
		if (!$success && $attempts <= 100) $updated++;
		if ($updated !== $attempts) {
			self::cache_put('failed_logins', $updated, false, true);
		}
		if (!$success) {
			self::cache_put('failed_login_last_time', time(), false, true);
		}
	}
	
	static function captcha_complexity() {
		$attempts = self::cache_get('failed_logins', $found);
		if (!$found) $attempts = 0;
		$cooldown_time = self::cache_get('failed_login_last_time', $found);
		if (!$found) $cooldown_time = 0;
		$delta = (int)((time() - $cooldown_time) / 60);
		if ($attempts <= $delta) {
			return -1;
		}
		return $attempts - $delta - 1;
	}

	// for compatibility only (7.8.11)
	static function check_user_timezone() {
		return true;
	}
}

?>
