<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */



require_once('include/config/ConfigBase.php');

class DisplayManager extends ConfigBase {
	var $local;
	var $init_time;
	var $cfg_loaded;
	var $skipped;

	// saved in system cache file
	static $detail_keys = array(
		'detail', 'source', 'list', 'widgets', 'index',
	);
	// looked up on demand
	static $cfg_keys = array(
		'fields', 'filters', 'basic_filters', 'popup_filters', 'auto_filters',
		'view', 'edit', 'hooks', 'status', 'notify',
	);
	
	
	function init() {
		parent::init();
		$this->local = array();
		$this->cfg_loaded = array();
		$this->init_time = microtime(true);
	}
	
	
	static function cfg_cache_file() {
		return CacheManager::get_location('system/') . 'display_cache.php';
	}

	
	static function scan_models() {
		// scan all model files
		$fnames = glob_unsorted(
			'modules/*/display/*.php',
			'include/config/display/*.php');
		$custom_dir = AppConfig::custom_dir();
		$skipped = array();
		$to_parse = array('main' => array(), 'ext' => array());
		foreach($fnames as $f) {
			if(! is_file($f) || ! is_readable($f)) {
				$skipped['unreadable'][] = $f;
				continue;
			}
			$parts = explode('.', basename($f));
			array_pop($parts);
			if(count($parts) < 2) {
				$skipped['notype'][] = $f;
				continue;
			} else {
				list($stype, $name) = $parts;
			}
			$finfo = array('name' => $f);
			if($stype == 'display') {
				if(isset($to_parse['main'][$name])) {
					$skipped['duplicate'][] = $f;
					continue;
				}
				$to_parse['main'][$name] = $finfo;
				$ext_files = AppConfig::setting("ext.display.$name");
				if($ext_files) $to_parse['ext'][$name] = $ext_files;
				$notificationsFile = $custom_dir . 'notifications/' . basename($f);
				if (file_exists($notificationsFile)) {
					 $to_parse['ext'][$name][] = ['name' => $notificationsFile];
				}
				$fcust = $custom_dir . $f;
				if(file_exists($fcust)) {
					if(! is_file($fcust) || ! is_readable($fcust))
						$skipped['unreadable'][] = $fcust;
					else {
						$finfo = array('name' => $fcust);
						$to_parse['custom'][$name] = $finfo;
					}
				}
			}
		}
		
		return compact('to_parse', 'skipped');
	}
	
	
	function createIndex($scan=null) {
		$to_parse = $skipped = array();
		if(! isset($scan))
			$scan = self::scan_models();
		extract($scan); // set to_parse and skipped

		$data = array();
		foreach(self::$detail_keys as $k)
			$data[$k] = array();
		$data['hash'] = self::cfg_hash();
		$data['time'] = microtime(true);
		
		foreach($to_parse['main'] as $k => $f) {
			$finfo = array($f);
			if(isset($to_parse['ext'][$k])) {
				$finfo = array_merge($finfo, $to_parse['ext'][$k]);
			}
			if(isset($to_parse['custom'][$k])) {
				$finfo[] = $to_parse['custom'][$k];
			}
			$files = array_column($finfo, 'name');
			$src = array('type' => 'files', 'files' => $finfo);


			try {
				$model = self::load_files($files);
			} catch(IAHConfigFileError $e) {
				$skipped['noparse'][] = $f['name'];
				$this->parse_errors[] = $e;
				continue;
			}
			
			$detail = array_get_default($model, 'detail', array());

			if(isset($detail['name']))
				$name = $detail['name'];
			else
				$name = $detail['name'] = $k;
			
			$data['source'][$name] = $src;
			$data['detail'][$name] = $detail;			
			$data['list'][$name] = array_get_default($model, 'list', array());
			
			$widgets = array_get_default($model, 'widgets', array());
			foreach($widgets as $idx => &$w) {
				if(! is_array($w) || empty($w['type']))
					continue;
				if(empty($w['name'])) {
					if(is_string($idx)) $w['name'] = $idx;
					else continue;
				}
				if(empty($w['class']))
					$w['class'] = $w['name'];
				$w['model'] = $name;
				
				if(isset($data['widgets'][$w['name']])) {
					$GLOBALS['log']->error("Duplicate widget definition: {$w['name']}");
					continue;
				}
				$data['widgets'][$w['name']] = $w;
			}
		}
		$this->skipped = $skipped;
		
		return $data;
	}
	
	
	function extendIndex(&$data) {
		$data['index']['show_menu_create_button'] = array();
		foreach($data['list'] as $name => $info) {
			if(array_get_default($info, 'show_menu_create_button', true)
				&& array_get_default($info, 'show_create_button', true))
			   		$data['index']['show_menu_create_button'][$name] = 1;
		}
	}
	
	
	function loadIndex($lastUpdate=0) {
		$cache_key = 'AppConfig.display.detail_index';
		$cache_file = self::cfg_cache_file();
		$hash = self::cfg_hash();

		$index = AppConfig::cache_file_read($cache_file, $cache_key, $lastUpdate);
		if(! is_array($index) || ! isset($index['hash']) || $index['hash'] != $hash) {
			$from = self::config_debug("create display cache");
			$index = $this->createIndex();
			AppConfig::set_expiry('display');
			AppConfig::cache_file_write($index, $cache_file, $cache_key);
			$this->init_time = microtime(true);
			self::config_debug("done display cache", $from);
		}
		$this->extendIndex($index);
		
		foreach($index as $k => &$v)
			$this->local[$k] = $v;
		if(isset($this->skipped))
			$this->local['index']['skipped'] = $this->skipped;
	}
	
	
	function loadModel($name, $lastUpdate=0) {
		$cache_key = "AppConfig.display.full.$name";
		$no_cache = $this->memoryCacheDisabled();
		if(! $no_cache)
			$model = AppConfig::cache_get_timed($cache_key, $lastUpdate, $found);
		else
			$found = false;
		
		if(! $found) {
			$source = $this->getSetting("source.$name");
			if(! empty($source) && $source['type'] == 'files') {
				$finfo = $source['files'];
				$files = array_column($finfo, 'name');
				// may throw parse error
				$model = self::load_files($files);
			} else {
				$model = array();
			}
			if(! $no_cache)
				AppConfig::cache_put_timed($cache_key, $model);
		}
		
		if(! isset($model['fields']))
			$model['fields'] = array();
		$this->addAutoDisplayFields($name, $model['fields']);
		foreach($model['fields'] as $k => &$f) {
			if(! isset($f['name']))
				$f['name'] = $k;
		}

		foreach(self::$cfg_keys as $k)
			if(isset($model[$k]))
				$this->local[$k][$name] =& $model[$k];
		
		$this->cfg_loaded[$name] = 1;
	}
	
	function addAutoDisplayFields($name, &$fields) {
		$links = AppConfig::setting("model.links.$name", array());
		foreach($links as $lname => $link) {
			if(array_get_default($link, 'link_type') != 'one' || empty($link['relationship']) || isset($fields[$lname]))
				continue;
			$rel_name = $link['relationship'];
			if(is_array($rel_name)) {
				$rel = $rel_name;
				$rel_model = $name;
			} else {
				$rel_model = AppConfig::setting("model.index.relationships.$rel_name");
				if ($rel_model)
					$rel = AppConfig::setting("model.relationships.$rel_model.$rel_name");
				else
					$rel = null;
			}
			if(! is_array($rel))
				continue;
			
			$side = array_get_default($link, 'side', 'auto');
			if ($rel['left']['model'] == $rel_model && $side == 'auto')
				$side = 'left';
			else if ($rel['right']['model'] == $rel_model)
				$side = 'right';
			else
				continue;
			if ($side == 'right') {
				$targ = $rel['left'];
				$rel['left'] = $rel['right'];
				$rel['right'] = $targ;
			}

			if(! empty($rel['right']['key']) && ! empty($rel['left']['model'])) {
				$spec = $link;
				unset($spec['link_type']);
				$spec['source'] = array('type' => 'non-db');
				$spec['type'] = 'ref';
				$spec['bean_name'] = $rel['left']['model'];
				$spec['id_name'] = $rel['right']['key'];
				$spec['detail_link'] = true;
				$fields[$lname] = $spec;
			}
		}
		
		if(isset($links['securitygroups']) && ! isset($fields['teams'])) {
			$fields['teams'] = array(
				'vname' => 'LBL_SECURITYGROUPS',
				'vname_module' => 'app',
				'widget' => 'ListTeamsWidget',
			);
		}
	}


	function getSetting($name, $default=null, $standard=false) {
		$lastUpd = $this->getConfigExpired();
		if($this->init_time < $lastUpd)
			$this->init();
		if(is_array($name))
			$parts = $name;
		else
			$parts = explode('.', $name);
		$c = count($parts);
		if($c > 0) {
			$key = $parts[0];
			if(in_array($key, self::$detail_keys) && ! isset($this->local[$key])) {
				$this->loadIndex($lastUpd);
			}
			else if(in_array($key, self::$cfg_keys) && $c > 1) {
				$name = $parts[1];
				if(empty($this->cfg_loaded[$name]))
					$this->loadModel($name, $lastUpd);
			}
		}
		return $this->findSubPath($parts, $this->local, $default);
	}
	
	
	function getConfigExpired() {
		return AppConfig::get_expiry('display');
	}
	
	function invalidateCache($category=null, $limit=null) {
		// setting stored in external cache only
		AppConfig::set_expiry('display');
		@unlink(self::cfg_cache_file());
		$this->init();
		$GLOBALS['log']->debug('Display cache invalidated');
	}

	function setLocal($path, $val, $merge=false, $push_db=true)
	{
		if(is_array($path))
			$parts = $path;
		else
			$parts = explode('.', $path);
		$local =& $this->local;
		foreach($parts as $p) {
			if (!isset($local[$p])) $local[$p] = array();
			$local =& $local[$p];
		}
		$local = $val;
	}
}

