<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */



require_once('include/config/ConfigBase.php');

class ExtManager extends ConfigBase {
	var $local;
	var $init_time;
	var $cfg_loaded;

	// saved in system cache file
	static $cfg_keys = array(
		'admin', 'config', 'models', 'display', 'views', 'language',
	);
	
	
	function init() {
		parent::init();
		$this->local = array();
		$this->cfg_loaded = array();
		$this->init_time = microtime(true);
	}
	
	
	static function cfg_cache_file() {
		return CacheManager::get_location('system/') . 'ext_cache.php';
	}

	
	static function scan_extensions() {
		$index = array();
		foreach(self::$cfg_keys as $k) $index[$k] = array();
		$index['hash'] = self::cfg_hash();
		$index['time'] = microtime(true);
		// scan all extensions files
		$config = glob_unsorted('modules/*/ext/config/*.php');
		if($config) {
			foreach($config as $f) {
				if(is_file($f) && is_readable($f)) {
					$path = explode('/', $f);
					$index['config'][substr($path[4], 0, -4)][] = array(
						'name' => $path[1],
					);
				}
			}
		}
		
		$mods = glob('modules/*/ext/modules/*', GLOB_ONLYDIR | GLOB_NOSORT);
		if($mods)
		foreach($mods as $moddir) {
			$path = explode('/', $moddir);
			$extmod = $path[1];
			$targmod = $path[4];
			$files = glob_unsorted($moddir . '/*/*.php');
			foreach($files as $f) {
				$path = explode('/', $f);
				$subdir = $path[5];
				if($subdir == 'models') {
					if(substr($path[6], 0, 5) == 'bean.' || substr($path[6], 0, 6) == 'union.' || substr($path[6], 0, 6) == 'table.') {
						$dotpos = strpos($path[6], '.');
						$bean = substr($path[6], $dotpos + 1, -4);
						$finfo = array('module' => $extmod, 'name' => $f);
						$index[$subdir][$bean][] = $finfo;
					}
				} else if($subdir == 'display') {
					if(substr($path[6], 0, 8) == 'display.') {
						$bean = substr($path[6], 8, -4);
						$finfo = array('module' => $extmod, 'name' => $f);
						$index[$subdir][$bean][] = $finfo;
					}
				} else if($subdir == 'views' || $subdir == 'new_views') {
					$parts = explode('.', $path[6]);
					if(count($parts) == 3) {
						$finfo = array('module' => $extmod, 'name' => $f);
						$index[$subdir][$targmod][$parts[0]][$parts[1]][] = $finfo;
					}
				} else if($subdir == 'language') {
					$parts = explode('.', $path[6]);
					if(count($parts) == 4 && $parts[0] == 'lang') {
						$finfo = array('module' => $extmod, 'name' => $f);
						$index[$subdir][$targmod][$parts[1]][$parts[2]][] = $finfo;
					}
				}
			}
		}
		
		$admin = glob_unsorted('modules/*/ext/modules/Administration/administration.*.php');
		foreach($admin as $f) {
			$path = explode('/', $f);
			$finfo = array('module' => $path[1], 'name' => $f);
			$index['admin'][] = $finfo;
		}
		
		$lang = glob_unsorted('modules/*/ext/include/language/*.php');
		foreach($lang as $f) {
			$path = explode('/', $f);
			$extmod = $path[1];
			$parts = explode('.', $path[5]);
			if(count($parts) == 4 && $parts[0] == 'lang') {
				$finfo = array('module' => $extmod, 'name' => $f);
				$index['language']['app'][$parts[1]][$parts[2]][] = $finfo;
			}
		}
		
		return $index;
	}
	
	
	function loadIndex($lastUpdate=0) {
		$cache_key = 'AppConfig.ext.index';
		$cache_file = self::cfg_cache_file();
		$hash = self::cfg_hash();
		$index = AppConfig::cache_file_read($cache_file, $cache_key, $lastUpdate);
		if(! is_array($index) || ! isset($index['hash']) || $index['hash'] != $hash) {
			$from = self::config_debug("create ext cache");
			$index = self::scan_extensions();
			AppConfig::set_expiry('ext');
			AppConfig::cache_file_write($index, $cache_file, $cache_key);
			$this->init_time = microtime(true);
			self::config_debug("done ext cache", $from);
		}
		$this->local = $index;
	}


	function getSetting($name, $default=null, $standard=false) {
		$lastUpd = $this->getConfigExpired();
		if($this->init_time < $lastUpd)
			$this->init();
		if(is_array($name))
			$parts = $name;
		else
			$parts = explode('.', $name);
		$c = count($parts);
		if($c > 0) {
			$key = $parts[0];
			if(in_array($key, self::$cfg_keys) && ! isset($this->local[$key])) {
				$this->loadIndex($lastUpd);
			}
		}
		return $this->findSubPath($parts, $this->local, $default);
	}
	
	
	function getConfigExpired() {
		return AppConfig::get_expiry('ext');
	}
	
	function invalidateCache($category=null, $limit=null) {
		// setting stored in external cache only
		AppConfig::set_expiry('ext');
		@unlink(self::cfg_cache_file());
		$this->init();
		$GLOBALS['log']->debug('Extension cache invalidated');
	}

	function setLocal($path, $val, $merge=false, $push_db=true)
	{
	}
}

?>
