<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */



require_once('include/config/ConfigBase.php');

class LanguageManager extends ConfigBase {
	var $local;
	var $standard;
	var $index_loaded;
	var $modules_loaded;
    var $dashlets_loaded;
    var $init_time;
	var $base_language;
	var $diff;
	var $skipped;

	static $mod_cache_key = 'AppConfig.lang.mod';
	static $index_cache_key = 'AppConfig.lang.index';
    static $index_keys = array('index', 'detail', 'labels', 'hooks');

	public function getSetting($name, $default = null, $standard = false) {
		global $current_language;

		$lastUpd = $this->getCacheExpired();
		if($this->init_time < $lastUpd)
			$this->init();
		if(is_array($name))
			$parts = $name;
		else
			$parts = explode('.', $name);
		$alt_parts = null;
		$c = count($parts);
		if($c > 0) {
			if ($c == 1 && $parts[0] == 'base') {
				return $this->base_language;
			}
			if ($c == 1 && $parts[0] == 'current') {
				return isset($current_language) ? $current_language : $this->base_language;
			}
			if($c > 1) {
				if($parts[1] == 'current')
					$parts[1] = isset($current_language) ? $current_language : $this->base_language;
				else if($parts[1] == 'base') {
					$parts[1] = $this->base_language;
				}
			}
			if(in_array($parts[0], self::$index_keys)) {
				if(! $this->index_loaded)
					$this->loadIndex();
				if($standard && $parts[0] == 'detail' && count($parts) >= 3) {
					// allow loading of standard module language details for rename tabs wizard
					$lang = $parts[1];
					$mod = $parts[2];
					if(! isset($this->standard['detail'][$lang][$mod])) {
						$standard = false;
						$cpath = "modules/$mod/language/lang.$lang.meta.php";
						if(is_readable($cpath)) {
							try {
								$meta = ConfigParser::load_file($cpath);
								if(isset($meta) && isset($meta['detail'])) {
									$this->standard['detail'][$lang][$mod] = $meta['detail'];
									$standard = true;
								}
							} catch(Exception $e) {
							}
						}
					}
				} else
					$standard = false; // only one index
			}
			else if($c > 2) {
				if($parts[0] == 'jscache') {
					return $this->getJavascriptCache($parts[1], $parts[2]);
				}
				if(! $this->index_loaded)
					$this->loadIndex();
                if ($parts[0] == 'dashlet_strings') {
                    if(empty($this->dashlets_loaded[$parts[1]][$parts[2]][$parts[3]][$standard]))
                        $this->loadDashlet($parts[1], $parts[2], $parts[3], $standard);
                } elseif (empty($this->modules_loaded[$parts[1]][$parts[2]][$standard])) {
					// avoid loading language file just for (previously indexed) module title
					if($c == 4 && $parts[0] == 'strings' && $parts[3] == 'LBL_MODULE_TITLE' && ! $standard) {
						if(! $this->index_loaded)
							$this->loadIndex();
						if ($parts[1] !== $this->base_language)
							$alt_parts = array('index', $this->base_language, 'module_labels', $parts[2]);
						$parts = array('index', $parts[1], 'module_labels', $parts[2]);
                	} else
						$this->loadModule($parts[1], $parts[2], $standard);
				}
				$check_inherit = true;
			}
		}

		$nul = null;
		if($standard)
			$val = $this->findSubPath($parts, $this->standard, $nul, false);
		else
			$val = $this->findSubPath($parts, $this->local, $nul);
		if(! isset($val) && $alt_parts) {
			$val = $this->findSubPath($alt_parts, $this->local, $nul);
		}
		if(! isset($val)) {
			if(! empty($check_inherit)) {
				while(1) {
					$inh = array_get_default($this->local['index'][$parts[1]]['inherits'], $parts[2], 'app');
					$parts[2] = $inh;
					if($parts[0] == 'lists' && $inh == 'app')
						break;
					if(empty($this->modules_loaded[$parts[1]][$parts[2]][$standard]))
						$this->loadModule($parts[1], $parts[2], $standard);
					if($standard)
						$val = $this->findSubPath($parts, $this->standard, $default, false);
					else
						$val = $this->findSubPath($parts, $this->local, $default);
					if(isset($val) || $parts[2] == 'app')
						break;
				}
				if(! isset($val))
					$val = $default;
			} else
				$val = $default;
		}
		return $val;
	}


	public function init() {
		parent::init();

		$this->base_language = 'en_us';
		$this->local = array();
		$this->standard = array();
		$this->modules_loaded = array();
		$this->index_loaded = false;
		$this->diff = array();
		$this->init_time = microtime(true);
	}
	
	
	static function scan_meta() {
		// scan all module meta language files
		$fnames = glob_unsorted(
			'modules/*/language/lang.*.meta.php',
			'include/language/lang.*.meta.php');
		$custom_dir = AppConfig::custom_dir();
		$skipped = array();
		$to_parse = array();
		foreach($fnames as $f) {
			if(! is_file($f) || ! is_readable($f)) {
				$skipped['unreadable'][] = $f;
				continue;
			}
			$f = strtr($f, '\\', '/');
			if( ($p1 = strpos($f, 'modules/')) !== false ) {
				$p1 += 8;
				$p2 = strpos($f, '/', $p1);
				$mod = substr($f, $p1, $p2-$p1);
			} else
				$mod = 'app';
			$parts = explode('.', basename($f));
			$lang = $parts[1];
			$to_parse[$lang][$mod][] = $f;

			//find corresponding meta files in extensions
			$ext_pattern = 'modules/*/ext/' . $f;
			$ext_names = glob_unsorted($ext_pattern);
			foreach ($ext_names as $e) {
				if(file_exists($e)) {
					if(! is_file($e) || ! is_readable($e))
						$skipped['unreadable'][] = $e;
					else {
						$to_parse[$lang][$mod][] = $e;
					}
				}
			}

			$fcust = $custom_dir . $f;
			if(file_exists($fcust)) {
				if(! is_file($fcust) || ! is_readable($fcust))
					$skipped['unreadable'][] = $fcust;
				else {
					$to_parse[$lang][$mod][] = $fcust;
				}
			}
		}

		return compact('to_parse', 'skipped');
	}
	
	
	function updateIndex($scan=null, $custom=null) {
		$to_parse = $skipped = array();
		if(! isset($scan))
			$scan = self::scan_meta();
		extract($scan); // set to_parse and skipped

		$data = array('hash' => self::cfg_hash(), 'time' => microtime(true));
		$init_data = array(
			'labels' => null,
			'detail' => array(),
			'index' => array(
				'module_labels' => array(), 'model_labels' => array(), 'inherits' => array(),
				'have_custom' => array(), 'have_lists' => array(),
				'version' => md5(time()),
			),
			'hooks' => array(),
		);
		
		foreach($to_parse as $lang => $mod_fs) {
			if(! isset($mod_fs['app']))
				continue;
			$data[$lang] = $init_data;
			foreach($mod_fs as $mod => $files) {
				try {
					$meta = self::load_files($files);
				} catch(IAHConfigFileError $e) {
					$skipped['noparse'][] = "$lang $mod";
					$this->parse_errors[] = $e;
					if($mod == 'app') {
						unset($data[$lang]);
						continue 2;
					}
					continue;
				}
			
				if(! isset($meta['detail'])) {
					if($mod == 'app') $meta['detail'] = [];
					else {
						$skipped['nodetail'][] = "$lang $mod";
						continue;
					}
				}
				$detail =& $meta['detail'];
				
				if($mod != 'app') {
					if(isset($detail['label']))
						$data[$lang]['index']['module_labels'][$mod] = $detail['label'];
					if(isset($detail['model_labels']))
						$data[$lang]['index']['model_labels'][$mod] = $detail['model_labels'];
					if(isset($detail['inherit_from']))
						$data[$lang]['index']['inherits'][$mod] = $detail['inherit_from'];
					
					$paths = self::language_file_paths($lang, $mod);
					foreach($paths['lists'] as $k => $v) {
						if(file_exists($v)) {
							$data[$lang]['index']['have_lists'][$mod] = true;
							if($k == 'custom')
								$data[$lang]['index']['have_custom'][$mod] = true;
						}
					}
				}
				else {
					$data[$lang]['labels'] = empty($detail['label']) ? $lang : $detail['label'];
					$data[$lang]['index']['have_lists'][$mod] = true;
					$paths = self::language_file_paths($lang, $mod);
					$data[$lang]['index']['have_custom'][$mod] = file_exists($paths['lists']['custom']);
				}
				
				if(isset($meta['hooks'])) {
					if(! isset($data[$lang]['hooks'][$mod]))
						$data[$lang]['hooks'][$mod] = array();
					array_extend($data[$lang]['hooks'][$mod], $meta['hooks']);
				}
				
				// not currently used elsewhere, so saving memory by leaving it out
				//$src = array('type' => 'files', 'files' => $files);
				//$detail['_source'] = $src;
				$data[$lang]['detail'][$mod] = $detail;
			}
		}
		$this->skipped = $skipped;
				
		$cache_file = self::index_cache_file();
		AppConfig::set_expiry('lang');
		AppConfig::cache_file_write($data, $cache_file);
		$this->init_time = microtime(true);

		return $data;
	}
	
		
	function loadIndex($lastUpdate=0) {
		$cache_key = self::$index_cache_key;
		$cache_file = self::index_cache_file();
		$hash = self::cfg_hash();
		clearstatcache(false, $cache_file);
		if(! file_exists($cache_file)) {
			$from = self::config_debug("create language cache");
			$index = $this->updateIndex();
			AppConfig::cache_put_timed($cache_key, $index, true, true);
			self::config_debug("done language cache", $from);
		} else {
			$mt = @filemtime($cache_file);
			$index = AppConfig::cache_get_timed($cache_key, max($mt, $lastUpdate), $key_found);
			if(empty($key_found)) {
				$index = include($cache_file);
				if(! is_array($index) || ! isset($index['hash']) || $index['hash'] != $hash) {
					$from = self::config_debug("create language cache");
					$index = $this->updateIndex();
					self::config_debug("done language cache", $from);
				}
				AppConfig::cache_put_timed($cache_key, $index, true, true);
			}
		}
		if(! is_array($index))
			throw new IAHError("Error loading language index");
		foreach($index as $k => &$mod_meta) {
			if($k == 'hash' || $k == 'time') continue;
			foreach($mod_meta as $lang => &$meta)
				$this->local[$lang][$k] = $meta;
		}
		if(isset($this->skipped))
			$this->local['index']['skipped'] = $this->skipped;
		$this->index_loaded = true;
	}


	/**
	 * Set local lang value.
	 * $name can have the following values:
	 * $lang is either a language name (en_us) or one of the special values 'current' or 'base'
	 * $mod is either a module name or the special value 'app'
	 * - index.[various]  (not currently supported)
	 * - detail.$lang.$mod  (not currently supported)
	 * - strings.$lang.$mod[.string_key]
	 * - lists.$lang.$mod[.list_key[.option]]
	 *
	 * @param array|string $name
	 * @param array|string $value
	 * @return void
	 */
	public function setLocal($name, $value, $merge=false, $push_db=true) {
		if (!is_array($name)) {
			$name = explode('.', $name);
		}
		if(count($name) < 2) return;
		
		$key = array_shift($name);
		if($key != 'lists' && $key != 'strings')
			return;

		$lang = array_shift($name);
		if ($lang == 'current') {
			$lang = AppConfig::setting('lang.current');
		} if ($lang == 'base') {
			$lang = AppConfig::setting('lang.base');
		}
		
		if(! $name) {
			$this->diff[$key][$lang] = $value;
		}
		else {
			$mod = array_shift($name);

            if(! isset($this->diff[$key][$lang][$mod]))
				$this->diff[$key][$lang][$mod] = array();
            $base =& $this->diff[$key][$lang][$mod];

			if(count($name) == 2 && $key == 'lists') {
				$val = $this->getSetting(array($key, $lang, $mod, $name[0]), array());
				$val[$name[1]] = $value;
				$value = $val;
				array_pop($name);
			}

			if(count($name) == 1) {
                $base[$name[0]] = $value;
                $this->local[$key][$lang][$mod][$name[0]] = $value;
				return;
			}
			else if(! $name) {
				$base = $value;
			}
		}
	}


	public function saveLocal($category=null, $push_db=true, $out_path=null) {
		require_once 'include/config/format/ConfigWriter.php';

		$cp = new ConfigParser();
		$cw = new ConfigWriter();
		$cw->setAppendOnly();
		
		$types = array('lists', 'strings');
		foreach($types as $t) {
			if(! empty($this->diff[$t])) {
				foreach($this->diff[$t] as $lang => $mods) {
					if(! isset($mods)) {
						// clear all custom modules
						$paths = glob_unsorted(AppConfig::custom_dir() . "modules/*/language/lang.$lang.$t.php");
						$app = self::custom_language_path('app') . "/lang.$lang.$t.php";
						if(file_exists($app)) $paths[] = $app;
						foreach($paths as $p)
							unlink($p);
						continue;
					}
					
					foreach($mods as $mod => $vals) {
						$save = array();
						$deleted = array();
						$std = $this->getSetting(array($t, $lang, $mod), array(), true);
						
						$path = self::custom_language_path($mod);
						$filepath = $path . "/lang.{$lang}.{$t}.php";
						
						$custom = $std;
						if(! isset($vals))
							; // setting to null clears all custom values
						else {
							if(is_readable($filepath)) {
								try {
									$cp->reset();
									$cp->parseFileWithTarget($filepath, $custom);
								} catch (Exception $e) {
								}
							}
						
							foreach($vals as $key => $value) {
								if(is_null($value))
									unset($custom[$key]);
								else
									$custom[$key] = $value;
							}
						}
						
						if(! is_dir($path)) mkdir($path, 0777, true);
						$cw->writeDiff($filepath, $custom, $std);
					}
				}
			}
		}
		
		$this->invalidateCache();
	}

    public function invalidateCache($category = null, $limit = null) {
		// setting stored in external cache only
		AppConfig::set_expiry('lang');
		@unlink(self::index_cache_file());
		$this->init();
		$GLOBALS['log']->debug('Language cache invalidated');
	}

	public static function index_cache_file() {
		return CacheManager::get_location('system/') . 'lang_cache.php';
	}
	
	public static function custom_language_path($module) {
		if($module == 'app')
			$path = AppConfig::custom_dir() . 'include/language';
		else
			$path = AppConfig::custom_dir() . "modules/$module/language";
		return $path;
	}
	
	public static function language_file_paths($lang, $mod, $standard=false) {
		if($mod == 'app') {
			$files = array(
				'strings' => array(
					"include/language/lang.{$lang}.strings.php",
				),
				'lists' => array(
					"include/language/lang.{$lang}.lists.php",
				),
			);
		} else {
			$files = array(
				'strings' => array(
					"modules/{$mod}/language/lang.{$lang}.strings.php",
				),
				'lists' => array(
					"modules/{$mod}/language/lang.{$lang}.lists.php",
				),
			);
		}
		$ext_files = AppConfig::setting("ext.language.$mod.$lang");
		if($ext_files) {
			if(isset($ext_files['strings']))
				array_extend($files['strings'], array_column($ext_files['strings'], 'name'));
			if(isset($ext_files['lists']))
				array_extend($files['lists'], array_column($ext_files['lists'], 'name'));
		}
		if(! $standard) {
			$cpath = self::custom_language_path($mod);
			$files['strings']['custom'] = $cpath . "/lang.{$lang}.strings.php";
			$files['lists']['custom'] = $cpath . "/lang.{$lang}.lists.php";
		}
		return $files;
	}

	
	public function loadModule($lang, $name, $standard=false, $raw=false) {
		$cache_key = self::$mod_cache_key.".$lang.$name";
		$last_upd = $this->getCacheExpired();
		$no_cache = $this->memoryCacheDisabled();
		if($standard || $raw || $no_cache)
			$key_found = false;
		else
			$cached = AppConfig::cache_get_timed($cache_key, $last_upd, $key_found);
		if($key_found) {
			$this->modules_loaded[$lang][$name][$standard] = 1;
			$this->local['strings'][$lang][$name] = $cached['strings'];
			$this->local['lists'][$lang][$name] = $cached['lists'];
			return true;
		}
		if($standard)
			$target =& $this->standard;
		else
			$target =& $this->local;
		
		$files = self::language_file_paths($lang, $name, $standard);
		if(! $raw)
			$this->modules_loaded[$lang][$name][$standard] = 1;
		$strings = $lists = null;

		try {
			$strings = ConfigParser::load_files($files['strings'], ['extend' => false]);
			$lists = ConfigParser::load_files($files['lists'], ['extend' => false]);
		} catch (Exception $e) {
			$GLOBALS['log']->error($e->getMessage());
			return false;
		}
		
		if(! $raw) {
			if($name == 'app') {
				$lists['moduleList'] = $this->getSetting(array('index', $lang, 'module_labels'));
				// FIXME - generate moduleListSingular, modelList
			} else if($name != 'install') {
				$label = $this->getSetting(array('index', $lang, 'module_labels', $name), null, $standard);
				if($label && !isset($strings['LBL_MODULE_TITLE'])) {
					$strings['LBL_MODULE_TITLE'] = $label;
				}
				$strings['LBL_MODULE_NAME'] = $name;
			}
		}

		if($lang != $this->base_language && ! $raw) {
			$base = $this->base_language;
			if(empty($this->modules_loaded[$base][$name][$standard]))
				$this->loadModule($base, $name, $standard);
			if(isset($target['strings'][$base][$name])) {
				if(! is_array($strings)) $strings = array();
				$strings = array_replace($target['strings'][$base][$name], $strings);
			}
			if(isset($target['lists'][$base][$name])) {
				$app_lists = $target['lists'][$base][$name];
				if(! is_array($lists)) $lists = array();
				//self::merge_values($lists, $app_lists);
				$lists = array_replace_recursive($app_lists, $lists);
			}
		}
		if(! $standard) {
			$hooks = array_get_default($this->local['hooks'], $lang, array());
			if(isset($hooks[$name]['post_process'])) {
				foreach($hooks[$name]['post_process'] as $hook) {
					call_hook_function($hook, null, array($lang, &$strings, &$lists));
				}
			}
			if(isset($hooks['app']['post_process_all'])) {
				foreach($hooks['app']['post_process_all'] as $hook) {
					call_hook_function($hook, null, array($lang, &$strings, &$lists));
				}
			}
		}
		
		$target['strings'][$lang][$name] = $strings;
		$target['lists'][$lang][$name] = $lists;

		if(! $standard && ! $raw && ! $no_cache) {
			$cache = compact('strings', 'lists');
			AppConfig::cache_put_timed($cache_key, $cache, true, true);
		}
		return true;
	}

    public function loadDashlet($lang, $module, $name, $standard=false, $raw=false) {
        $cache_key = self::$mod_cache_key.".$lang.$module.$name";
        $last_upd = $this->getCacheExpired();
        $no_cache = $this->memoryCacheDisabled();
        if($standard || $raw || $no_cache)
            $key_found = false;
        else
            $cached = AppConfig::cache_get_timed($cache_key, $last_upd, $key_found);
        if($key_found) {
            $this->dashlets_loaded[$lang][$module][$name][$standard] = 1;
            $this->local['dashlet_strings'][$lang][$module][$name] = $cached['dashlet_strings'];
            return true;
        }

        if($standard)
            $target =& $this->standard;
        else
            $target =& $this->local;

        $file = "modules/{$module}/Dashlets/{$name}/lang.{$lang}.strings.php";
        if(! $raw)
            $this->dashlets_loaded[$lang][$module][$name][$standard] = 1;
        $strings = null;

        try {
            $strings = ConfigParser::load_file($file);
        } catch (Exception $e) {
            $GLOBALS['log']->error($e->getMessage());
        }

        if($lang != $this->base_language && ! $raw) {
            $base = $this->base_language;
            if(empty($this->dashlets_loaded[$base][$module][$name][$standard]))
                $this->loadDashlet($base, $module, $name, $standard);
            if(isset($target['dashlet_strings'][$base][$module][$name])) {
                if(! is_array($strings)) $strings = array();
                $strings = array_replace($target['dashlet_strings'][$base][$module][$name], $strings);
            }
        }

        $target['dashlet_strings'][$lang][$module][$name] = $strings;

        if(! $standard && ! $raw && ! $no_cache) {
            $cache['dashlet_strings'] = $strings;
            AppConfig::cache_put_timed($cache_key, $cache, true, true);
        }
        return true;
    }

    public function getCacheExpired() {
		return AppConfig::get_expiry('lang');
	}
	
	
	public function getJavascriptCache($lang, $module) {
		$js_cache_dir = CacheManager::get_location('javascript/');
		$sugar_version = AppConfig::version();
		$js_version = AppConfig::js_custom_version();
		$js_lang_version = AppConfig::language_version();
		
		$cache_fname = $js_cache_dir . "jslanguage_{$module}_{$lang}_{$js_lang_version}_{$sugar_version}_{$js_version}.js";
		if(file_exists($cache_fname))
			return $cache_fname;
		
		$json = getJSONobj();
		$data = array('language' => $lang, 'name' => $module);
		$inherit = $this->getSetting("index.$lang.inherits.$module", '');
		if($inherit)
			$data['inherit_from'] = $inherit;
		$strings = $this->getSetting("strings.$lang.$module");
		$lists = $this->getSetting("lists.$lang.$module");
		if(isset($strings))
			$data['strings'] = $strings;
		else if(! isset($lists))
			return null; // no point
		if(is_array($lists)) {
			$enc_lists = array();
			foreach($lists as $k => &$dom) {
				if(is_array($dom)) {
					$ks = array_keys($dom);
					foreach($ks as &$kk)
						if(is_int($kk)) $kk .= '';
					$enc_lists['keys'][$k] = $ks;
					$enc_lists['values'][$k] = array_values($dom);
				}
			}
			$data['lists'] = $enc_lists;
		}
		
		$data_enc = $json->encode($data);
		$data = "SUGAR.language.populate($data_enc);";
		if($fp = @fopen($cache_fname, 'w')) {
			fwrite($fp, $data);
			fclose($fp);
			return $cache_fname;
        }
		return null;
	}

	public static function cleanJSCache($module = null)
	{
		AppConfig::promote_js_custom_version();
		if ($module) {
			$patterns = array(CacheManager::get_location('javascript/') . 'jslanguage_' . $module . '*.js');
		} else {
			$patterns = array(
				CacheManager::get_location('javascript/') . 'jslanguage_*.js',
				CacheManager::get_location('javascript/') . 'mingroup_global_*.js',
			);
		}
		foreach ($patterns as $pattern) {
			$files = glob_unsorted($pattern);
			foreach ($files as $file) {
				@unlink($file);
			}
		}
	}

}

?>
