<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/config/ConfigBase.php');

class LayoutConfigManager extends ConfigBase {

    var $local;
    var $init_time;
    var $scan_time;
    var $auto_stat = null;
    var $cache_layouts = null;

    static $all_cache_key = 'AppConfig.views.all_info';
    

	function init() {
		parent::init();
        $this->local = null;
        $this->auto_stat = AppConfig::setting('layout.stat', false);
        $this->cache_layouts = AppConfig::setting('layout.memory_cache', true);
		if($this->memoryCacheDisabled())
			$this->cache_layouts = false;
        $this->init_time = microtime(true);
        $this->scan_time = null;
	}

	function getSetting($name, $default=null, $standard=false) {
		$lastUpdate = $this->getCacheExpired();
		if($this->init_time < $lastUpdate)
			$this->init();
        if(! isset($this->local))
            $this->loadIndex($lastUpdate);

        if(is_array($name))
            $parts = $name;
        else
            $parts = explode('.', $name);

        $data = $this->findLocalSubPath($parts);
        if(! isset($data)) {
        	$data = $default;
        	if(count($parts) > 3 && $parts[0] == 'layout') {
				if($this->loadLayout($parts, $lastUpdate))
					$data = $this->findLocalSubPath($parts, $default);
			}
        }

        return $data;
	}
	
	function loadLayout($parts, $lastUpdate=0) {
		$p = array_slice($parts, 0, 4);
		$p[0] = 'source';
		$srcs = $this->findLocalSubPath($p, array());
		
		if($this->auto_stat) {
			$pd = $p;
			$pd[0] = 'detail';
			$detail = $this->findLocalSubPath($pd);
			if($detail) {
				if(isset($detail['module_dir'])) {
					$dname = $pd[1];
					$pd[1] = $detail['module_dir'];
				} else
					$dname = null;
				$chk_src = self::find_sources($pd[1], $pd[2], $pd[3], $dname);
				if($chk_src != $srcs) {
					$srcs = $chk_src;
					$this->invalidateCache();
					$this->loadIndex();
				}
			}
		}

		if(! $srcs) {
			//pr2($p, 'layout not found');
			return false;
		}
		
		
		// FIXME - load from cache if possible
		if($this->cache_layouts) {
			$ck = $p;
			$ck[0] = 'AppConfig.views.layout';
			$cache_key = implode('.',  $ck);
			$layout = AppConfig::cache_get_timed($cache_key, $lastUpdate, $found);
			if(! $found)
				unset($layout);
		}
		
		if(! isset($layout)) {
			try {
				$info = self::read_sources($srcs);
				$layout =& $info['layout'];
			} catch(IAHConfigFileError $e) {
				$this->local['detail'][$p[1]][$p[2]][$p[3]] = array();
				$this->local['layout'][$p[1]][$p[2]][$p[3]] = array();
				//$this->parse_errors[] = $e;
				$GLOBALS['log']->error($e->getMessage());
				return false;
			}
		}
		
		if($this->cache_layouts && empty($found)) {
			AppConfig::cache_put_timed($cache_key, $layout);
		}
		$type = $p[2];
			if ($type == 'view' || $type == 'record') {
				if (!isset($layout['subpanels']))
					$layout['subpanels'] = [];
				if (!in_array('__personal', $layout['subpanels'])) 
					$layout['subpanels'][] = '__personal';
			}
		$this->local['layout'][$p[1]][$p[2]][$p[3]] = $layout;
		return true;
	}

    function findLocalSubPath($path, $default=null) {
        $src =& $this->local;
        $layout_source =& $this->local;
        $i = 0;

        foreach($path as $p) {
            if(@array_key_exists($p, $src)) {
                $src =& $src[$p];
                $index = $p;
                if ($i == 0 && $path[0] != 'dashlet') {
                    $index = 'source'; 
                } elseif ($i == 1 && $path[0] == 'dashlet') {
                    $index = 'source';
                }
                $layout_source =& $layout_source[$index];
            } else {
                return $default;
            }
            $i++;
        }

        return $src;
    }

    function isLayoutUpdated($layout_source) {
        if (!empty($layout_source['name']) && !empty($layout_source['update_time'])) {
            $source_updated = filemtime($layout_source['name']);
            if ($source_updated != $layout_source['update_time']) {
                return true;
            }
        }

        return false;
    }
    
    function getCacheExpired() {
		return max(AppConfig::get_expiry('views'), $this->scan_time);
    }

    function invalidateCache($category = null, $limit = null) {
        // setting stored in external cache only
        AppConfig::set_expiry('views');
        @unlink(self::cache_file());
        $this->init();
        $GLOBALS['log']->debug('Layout cache invalidated');
    }

    static function cache_file() {
        return CacheManager::get_location('system/') . 'views_cache.php';
    }
    
    
    static function read_sources($srcs) {
    	$fs = array_column($srcs, 'name');
		$info = ConfigParser::load_files($fs);
		if(empty($info['detail']))
			throw new IAHConfigFileError("missing 'detail' section for layout definition");
		if(empty($info['layout']))
			throw new IAHConfigFileError("missing 'layout' section for layout definition");
		return $info;
    }
    
    
    static function get_source_info($path, $layout_cls) {
		if(! preg_match('~modules/([^/]+)/(Dashlets/[^/]+/)?(views|new_views)/(.+)$~', $path, $m))
        	return false;
        
		$mod_name = $m[1];
		$file_name = explode('.', $m[4]);
		if(count($file_name) < 3)
			return false;
		$type = $file_name[0];
		$name = $file_name[1];
		if($layout_cls == 'dashlets')
			$dashlet_name = substr($m[2], 9, -1);
		else
			$dashlet_name = null;
		
		return compact('mod_name', 'dashlet_name', 'type', 'name');
	}
	
	
	static function file_metadata($path) {
		if(file_exists($path))
		return array(
			'name' => $path, 'update_time' => filemtime($path),
		);
	}
	
	
    /* find sources given info on a specific layout */
    static function find_sources($mod_name, $type, $name, $dashlet=null, $ext_only=false) {
    	if($dashlet) {
			$sub_path = "Dashlets/$dashlet/";
			$layout_cls = 'dashlets';
		} else {
			$sub_path = $layout_cls = '';
		}
		
		$ret = array();
		$file1 = "modules/$mod_name/{$sub_path}views/$type.$name.php";
		$custom_dir = AppConfig::custom_dir();
		$cust = array($custom_dir . "modules/$mod_name/{$sub_path}new_views/$type.$name.php");
		if(! $dashlet) {
			$ext = AppConfig::setting("ext.new_views.$mod_name.$type.$name");
			if($ext) $cust = array_merge($cust, array_column($ext, 'name'));
		}
		$ext_allowed = true;
		foreach($cust as $idx => $file2) {
			$info = self::file_metadata($file2);
			if($info) {
				if(! $ext_only)
					$ret[] = $info;
				if(! $idx)
					$ext_allowed = false;
				break;
			}
		}
		if(! $ext_only) {
			if (!$info) {
				$info = self::file_metadata($file1);
				if($info)
					$ret[] = $info;
			}
		}
		if($ext_allowed && ($ext_only || $ret)) {
			if(! $sub_path) {
				$ext_files = AppConfig::setting("ext.views.$mod_name.$type.$name");
				if($ext_files)
					array_extend($ret, $ext_files);
			}
			$custom = $custom_dir . $file1;
			$info = self::file_metadata($custom);
			if($info)
				$ret[] = $info;
		}
        return $ret;
    }

	/* scan all layout definitions */
    static function scan_layout_info(&$target, $layout_cls='') {
    	if($layout_cls == 'dashlets')
			$sub_path = 'Dashlets/*/';
		else
			$sub_path = '';
        $files = glob_sorted("modules/*/{$sub_path}views/*.php");
        if(! $layout_cls) {
			$ext_files = AppConfig::setting("ext.new_views");
			if($ext_files) {
				foreach($ext_files as $mod => $clss)
					foreach($clss as $cls => $names)
						foreach($names as $n => $all)
							$files = array_merge($files, array_column($all, 'name'));
			}
		}
		$files = array_merge($files, glob_unsorted(AppConfig::custom_dir() . "modules/*/{$sub_path}new_views/*.php"));
        if(! isset($target))
	        $target = array('detail' => array(), 'layout' => array(), 'source' => array());
        $skipped = array();
        $to_parse = array();
        $dashlet_module = array();
        $sources = array();
        $load_idx = 0;
        
		foreach($files as $f) {
			if (substr($f, -11) == '.backup.php') continue;
        	$src = self::get_source_info($f, $layout_cls);
        	if(! $src) continue;
        	extract($src);
        	if($layout_cls == 'dashlets') {
        		$dashlet_module[$dashlet_name] = $mod_name;
        		$mod_name = $dashlet_name;
        	}
        	$srcs = array(
        		self::file_metadata($f)
        	);
        	$ext_srcs = self::find_sources($mod_name, $type, $name, $dashlet_name, true);
        	if($ext_srcs) array_extend($srcs, $ext_srcs);
            $sources[$mod_name][$type][$name] = $srcs;
            
			try {
				$info = self::read_sources($srcs);
			} catch(IAHConfigFileError $e) {
				$skipped['noparse'][] = $srcs;
				$target['detail'][$mod_name][$type][$name] = array();
				$target['layout'][$mod_name][$type][$name] = array();
				//$this->parse_errors[] = $e;
				$GLOBALS['log']->error($e->getMessage() . " ($mod_name.$type.$name)");
				continue;
			}

			$detail = $info['detail'];
			$detail['name'] = $name;
			if($layout_cls == 'dashlets') {
				$detail['dashlet_name'] = $mod_name;
				$detail['module_dir'] = $dashlet_module[$mod_name];
			}
			$target['detail'][$mod_name][$type][$name] = $detail;
			//$target['layout'][$mod_name][$type][$name] = $info['layout'];

		}
        
        if($target['source'])
        	$target['source'] = array_merge($target['source'], $sources);
        else
        	$target['source'] = $sources;
		
		if(! $layout_cls) {
			self::scan_layout_info($target, 'dashlets');
		}
    }

    function loadIndex($lastUpdate=0) {
        $cache_file = self::cache_file();
        $cache_key = self::$all_cache_key;
        $hash = self::cfg_hash();
		$index = AppConfig::cache_file_read($cache_file, $cache_key, $lastUpdate);
		if(! is_array($index) || ! isset($index['hash']) || $index['hash'] != $hash) {
			$from = self::config_debug("create views cache");
			$index = null;
			self::scan_layout_info($index);
			$index['hash'] = $hash;
			$index['time'] = microtime(true);
			AppConfig::set_expiry('views');
			AppConfig::cache_file_write($index, $cache_file, $cache_key);
			$this->init_time = microtime(true);
			self::config_debug("done views cache", $from);
		}
        $this->local = $index;
		$this->scan_time = $index['time'];
    }
}
?>
