<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

require_once('include/config/ConfigBase.php');
require_once('include/utils/CacheManager.php');

class ModInfoManager extends ConfigBase {
	var $local;
	var $init_time;
	var $skipped;
	
	static $attrib_defaults = array(
		'tab_visibility' => 'normal',
		'actions' => array(),
	);
	
	function init() {
		parent::init();
		$this->local = null;
		$this->init_time = microtime(true);
	}
	
	static function info_cache_file() {
		return CacheManager::get_location('system/') . 'module_info_cache.php';
	}
	
	static function scan_module_info() {
		$files = glob_unsorted('modules/*/metadata/module_info.php');
		$ext = glob_unsorted('modules/*/ext/modules/*/metadata/module_info.php');
		$all_info = array('by_name' => array(), 'controllers' => array(),
			'primary_beans' => array(),
			'index' => array(
				'normal' => array(), 'hidden' => array(), 'manual' => array(),
				'submodules' => array(), 'by_group' => array(), 'all' => array(),
				'icons' => array(), 'by_name' => array(), 'procs' => array(),
			));	
		$skipped = array();
		$all = array();
		$ext_bymod = array();
		foreach($ext as $f) {
			if(preg_match('~/modules/([^/]+)/metadata/~', $f, $m)) {
				$ext_bymod[$m[1]][] = $f;
			}
		}
		
		foreach($files as $f) {
			$p = strpos($f, '/', 8);
			$mod_name = substr($f, 8, $p - 8);
			$paths = array($f);
			if(isset($ext_bymod[$mod_name]))
				array_extend($paths, $ext_bymod[$mod_name]);
			$paths[] = AppConfig::custom_dir() . $f;
			$info = ConfigParser::load_files($paths);
			if(! isset($info['detail'])) {
				$skipped['nodetail'] = $mod_name;
				continue;
			}

			$attr =& $info['detail'];
			$all_info['by_name'][$mod_name] = $info;
			foreach(self::$attrib_defaults as $k => &$v)
				if(! isset($attr[$k]))
					$attr[$k] = $v;

			$visib = $attr['tab_visibility'];
			if($visib != 'manual' && $visib != 'hidden')
				$visib = 'normal';
			$all_info['index'][$visib][] = $mod_name;
			$all[$mod_name] = 1;
			
			if(isset($attr['submodule_of']))
				$all_info['index']['submodules'][$attr['submodule_of']][] = $mod_name;
			
			if(isset($attr['default_group']))
				$all_info['index']['by_group'][$attr['default_group']][] = $mod_name;
			
			if(isset($attr['controller']) && isset($attr['controller_file']))
				$all_info['controllers'][$attr['controller']] = $attr['controller_file'];
			
			if(isset($attr['primary_bean']))
				$all_info['primary_beans'][$mod_name] = $attr['primary_bean'];

			if (!empty($attr['mobile_interface']))
				$all_info['mobile_modules'][] = $mod_name;

			if(isset($attr['icon']))
				$all_info['index']['icons'][$mod_name] = $attr['icon'];

			if(isset($info['procs']) && is_array($info['procs'])) {
				foreach($info['procs'] as $k => $p) {
					if(! isset($p['module'])) $p['module'] = $mod_name;
					$all_info['procs'][$k] = $p;
				}
			}
		}
		
		$flat = AppConfig::setting('module_order.flat', array());
		foreach($flat as $m) {
			if(isset($all[$m])) {
				$all_info['index']['all'][] = $m;
				$all[$m] = 0;
			}
		}
		foreach($all as $m => $v)
			if($v) $all_info['index']['all'][] = $m;
		

		return compact('all_info', 'skipped');
	}
	
	
	function loadIndex($lastUpdate=0) {
		$cache_key = 'AppConfig.module_info.all_info';
		$cache_file = self::info_cache_file();
		$hash = self::cfg_hash();
		$index = AppConfig::cache_file_read($cache_file, $cache_key, $lastUpdate);
		if(! is_array($index) || ! isset($index['hash']) || $index['hash'] != $hash) {
			$from = self::config_debug("create module info cache");
			$info = self::scan_module_info();
			$index = $info['all_info'];
			$index['hash'] = $hash;
			$index['time'] = microtime(true);
			$this->skipped = $info['skipped'];
			AppConfig::set_expiry('module_info');
			AppConfig::cache_file_write($index, $cache_file, $cache_key);
			$this->init_time = microtime(true);
			self::config_debug("done module info cache", $from);
		}		
		$this->local = array();
		foreach($index as $k => &$v)
			$this->local[$k] = $v;
		if(isset($this->skipped))
			$this->local['index']['skipped'] = $this->skipped;
	}
	
	
	function getSetting($name, $default=null, $standard=false) {
		$lastUpd = $this->getModuleInfoExpired();
		if($this->init_time < $lastUpd)
			$this->init();
		if(is_array($name))
			$parts = $name;
		else
			$parts = explode('.', $name);
		if(! isset($this->local))
			$this->loadIndex($lastUpd);
		if($parts) {
			if($parts[0] == 'current_user')
				array_splice($parts, 0, 1, array('by_user', AppConfig::current_user_id()));
			if($parts[0] == 'system' && ! isset($this->local[$parts[0]]))
				$this->local[$parts[0]] = $this->loadSystemModules();
			else if($parts[0] == 'by_user' && count($parts) > 1 && ! isset($this->local[$parts[0]][$parts[1]]))
				$this->local[$parts[0]][$parts[1]] = $this->loadUserModules($parts[1]);
		}
		return $this->findSubPath($parts, $this->local, $default);
	}
	
	function loadSystemModules() {
		require_once('include/utils/misc_functions.php');
		$ret = array();
		$mods = $this->getSetting('index.all');
		foreach($mods as $m) {
			if(! iml($m)) continue;
			$ret[$m] = array(
				'hidden' => false,
			);
		}
		foreach($this->getSetting('index.hidden') as $m)
			if(isset($ret[$m])) $ret[$m]['hidden'] = 'app';
		foreach(AppConfig::setting('module_order.hidden', array()) as $m)
			if(isset($ret[$m]) && ! $ret[$m]['hidden']) $ret[$m]['hidden'] = 'config';
		$flat = AppConfig::setting('module_order.flat', array());
		foreach($this->getSetting('index.manual') as $m) {
			if(isset($ret[$m]) && ! in_array($m, $flat) && ! $ret[$m]['hidden']) {
				$ret[$m]['hidden'] = 'manual';
			}
		}

		return $ret;
	}
	
	function loadUserModules($user_id) {
		require_once('modules/ACLActions/ACLAction.php');
		if($user_id) {
			$ret = self::getSetting('system');
			if(! AppConfig::setting('module_order.disable_user_edit'))
				$hide_tabs = AppConfig::setting("userpref.by_user.$user_id.global.hide_tabs", array());
			else
				$hide_tabs = array();
			foreach($hide_tabs as $m => $value) {
				if(is_int($m)) $m = $value;
				if(isset($ret[$m]) && (! $ret[$m]['hidden'] || $ret[$m]['hidden'] == 'manual')) $ret[$m]['hidden'] = 'user';
			}
			if(! is_admin($user_id)) {
				foreach($ret as $m => &$info) {
					if($info['hidden'] && $info['hidden'] != 'manual') continue;
					if(! ACLAction::userHasAccess($user_id, $m, 'access', 'module'))
						$info['hidden'] = 'acl';
				}
			}
		} else
			$ret = array();
		return $ret;
	}
	
	
	function getModuleInfoExpired() {
		return AppConfig::get_expiry('module_info');
	}
	
	function invalidateCache($category=null, $limit=null) {
		// setting stored in external cache only
		if(! $limit) {
			AppConfig::set_expiry('module_info');
			@unlink(self::info_cache_file());
		}
		$this->init();
		$GLOBALS['log']->debug('Module info cache invalidated');
	}


}

?>
