<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


class IAHModelError extends IAHInternalError {
}

class ModelDef {
	var $name;
	var $type;
	var $bean_name;
	var $module_dir;
	var $table_name;
	var $table_alias;
	var $default_order_by;
	var $audit_enabled = false;
	var $audit_table_name;
    var $activity_log_enabled = false;
	var $optimistic_locking = false;
	var $custom_display_defs = array();
	var $primary_key;
	var $link_spec;
	/** @var $join_model ModelDef */
	var $join_model;
	var $is_join;
	var $is_union;
	var $cached_links = array();


	function __construct($model_name = null, $link_spec = null, $alias = null) {
		if ($model_name)
			$this->initFromModel($model_name);
		if ($link_spec)
			$this->link_spec = $link_spec;
		if (isset($alias))
			$this->table_alias = $alias;
	}


	function initFromModel($model_name) {
		if($model_name instanceof ModelDef) {
			$mdef = $model_name;
			$model_name = $mdef->getModelName();
		}
		$detail = AppConfig::setting("model.detail.$model_name");
		if (!is_array($detail)) {
			throw new IAHModelError("unknown model: $model_name");
		}
		// load basic details. field defs are only loaded on demand
		$this->name = $model_name;
		$this->type = $detail['type'];
		$this->table_name = array_get_default($detail, 'table_name');
		$this->is_union = array_get_default($detail, 'is_union');
		$this->primary_key = array_get_default($detail, 'primary_key');
		$this->bean_name = array_get_default($detail, 'bean_name');
		$this->module_dir = array_get_default($detail, 'module_dir');
		$this->default_order_by = AppConfig::setting("display.list.$model_name.default_order_by");
		if(! $this->default_order_by)
			$this->default_order_by = array_get_default($detail, 'default_order_by');
		$this->optimistic_locking = array_get_default($detail, 'optimistic_locking', false);
		$this->audit_enabled = array_get_default($detail, 'audit_enabled', false);
		$this->audit_table_name = array_get_default($detail, 'audit_table_name');
        $this->activity_log_enabled = array_get_default($detail, 'activity_log_enabled', false);
	}

	function getAuditEnabled() {
		return $this->audit_enabled;
	}

	function getAuditTableName() {
		return $this->audit_table_name;
	}

	function getOptimisticLocking() {
		return $this->optimistic_locking;
	}
	
	function additionalDetailsLayout() {
		return self::additional_details_layout($this->name);
	}
	
	static function additional_details_layout($model_name) {
		if(! $model_name) return false;
		$layout = AppConfig::setting("model.detail.{$model_name}.additional_details");
		if(isset($layout)) {
			if(! $layout) return false;
			if(is_int($layout) || is_bool($layout)) $layout = 'Standard';
			return $layout;
		}
		$mod = AppConfig::module_for_model($model_name);
		$primary = AppConfig::module_primary_bean($mod);
		if($primary == $model_name) {
			$detail = AppConfig::setting("views.detail.{$mod}.additional.Standard");
			if($detail)
				return 'Standard';
		}
	}

	function haveFieldDefinition($field_name) {
		return !!$this->getFieldDefinition($field_name);
	}

	function getFieldDefinition($field_name) {
		$def = AppConfig::setting("model.fields.{$this->name}.$field_name");
		if ($def && is_array($def)) {
			if (!isset($def['vname_module'])) {
				$def['vname_module'] = $this->getModuleDir();
			}
		}
		return $def;
	}

	function getColumnFields() {
		return AppConfig::setting("model.column_fields.{$this->name}", array());
	}

	function getCustomFields() {
		return AppConfig::setting("model.custom_fields.{$this->name}", array());
	}

	function getExtraFields() {
		return AppConfig::setting("model.extra_fields.{$this->name}", array());
	}

	function getRequiredFields() {
		return AppConfig::setting("model.required_fields.{$this->name}", array());
	}

	function getAuditedFields() {
		return AppConfig::setting("model.audited_fields.{$this->name}", array());
	}

	function getFileFields() {
		return AppConfig::setting("model.file_fields.{$this->name}", array());
	}

	function getImageFields() {
		return AppConfig::setting("model.image_fields.{$this->name}", array());
	}

	function getListFields() {
		return AppConfig::setting("model.list_fields.{$this->name}", array());
	}

	function getFieldDefinitions() {
		return AppConfig::setting("model.fields.{$this->name}", array());
	}
	
	function getDisplayFieldDefinitions() {
		return array_merge(
			AppConfig::setting("display.fields.{$this->name}", array()),
			$this->custom_display_defs);
	}
	
	function getDisplayName() {
		return AppConfig::setting("model.detail.{$this->name}.display_name");
	}
	
	function addDisplayFieldDefinitions(array $defs) {
		array_extend($this->custom_display_defs, $defs);
	}
	
	function getDisplayField($name, $resolve=false) {
		if(isset($this->custom_display_defs[$name]))
			$display = $this->custom_display_defs[$name];
		else
			$display = AppConfig::setting("display.fields.{$this->name}.$name");
		if($display) {
			if($resolve && ($def = $this->getFieldDefinition($name)) ) {
				$display = array_replace_recursive($def, $display);
				$display['source'] = $def['source'];
			}
			else {
				if(! isset($display['type'])) {
					if(! empty($display['widget']))
						$display['type'] = 'widget';
					else
						$display['type'] = 'varchar';
				}
				if(! isset($display['source']['type']))
					$display['source']['type'] = 'non-db';
			}
			return $display;
		}
		else if($resolve) {
			if( ($p = strpos($name ?? '', '.')) !== false) {
				$lnk = substr($name, 0, $p);
				$name = substr($name, $p+1);
				if($lnk && ($lm = $this->getCachedLinkModel($lnk)) )
					return $lm->getDisplayField($name, $resolve);
			}
			return $this->getFieldDefinition($name);
		}
	}
	
	function getCachedLinkModel($name) {
		if(isset($this->cached_links[$name]))
			return $this->cached_links[$name];
		$lm = $this->getLinkModel($name);
		if($lm) $this->cached_links[$name] = $lm;
		return $lm;
	}

	function getAllDisplayFields() {
		$disp = $this->getDisplayFieldDefinitions();
		if(! $disp)
			$disp = array();
		else
			$disp = array_keys($disp);
		$dbf = $this->getListFields();
		if($dbf)
			$disp = array_unique(array_merge($disp, $dbf));
		return $disp;
	}
	
	function getExtendedFieldDefinitions() {
		$defs = $this->getFieldDefinitions();
		$display = $this->getDisplayFieldDefinitions();
		foreach($display as $k => $v) {
			if(isset($defs[$k]))
				$defs[$k] = array_replace_recursive($defs[$k], $v);
			else
				$defs[$k] = $v;
		}
		return $defs;
	}


	function getFieldType($field_name) {
		return AppConfig::setting("model.fields.{$this->name}.{$field_name}.type");
	}

	function getHooks($category = null, $display = false) {
		$base = $display ? 'display' : 'model';
		$k = "$base.hooks.{$this->name}";
		if (isset($category))
			$k .= ".$category";
		$ret = AppConfig::setting($k);
		if (!$display) {
			$k = "model_hooks";
			if (isset($category))
				$k .= ".$category";
			$ret2 = AppConfig::setting($k);
			if (is_array($ret) && is_array($ret2)) {
				if (isset($category)) {
					$ret = array_merge($ret, $ret2);
				} else {
					foreach ($ret2 as $cat => $hooks) {
						$ret[$cat] = array_merge(array_get_default($ret, $cat, array()), $hooks);
					}
				}
			}
			else if (is_array($ret2))
				$ret = $ret2;
		}
		return $ret;
	}

	function getDisplayHooks($category = null) {
		return $this->getHooks($category, true);
	}

	function getHookRequiredFields($select_hooks, $display=false) {
		$hooks = $this->getHooks(null, $display);
		if (!is_array($select_hooks) || !$select_hooks || !$hooks)
			return array();
		$req_f = array();
		foreach ($hooks as $k => $v) {
			if (in_array($k, $select_hooks)) {
				foreach ($v as $hook) {
					if (isset($hook['required_fields']) && is_array($hook['required_fields']))
						$req_f = array_merge($req_f, $hook['required_fields']);
					if (isset($hook['required_fields_function'])) {
						$fields = call_hook_function($hook['required_fields_function'], null, [$this]);
						if (is_array($fields)) {
							$req_f = array_merge($req_f, $fields);
						}
					}
				}
			}
		}
		return $req_f;
	}

	function execHooks($category = null, $params = null, $display = false) {
		$hooks = $this->getHooks($category, $display);
		if ($hooks) {
			if (!isset($params)) $params = array();
			foreach ($hooks as $hook)
				call_hook_function($hook, $this->name, $params);
		}
	}

	function execSpecialHook($name, $params = null) {
		if (!isset($params)) $params = array();
		$hook = array('class_function' => $name);
		call_hook_function($hook, $this->name, $params);
	}

	function getIndices() {
		return AppConfig::setting("model.indices.{$this->name}");
	}

	function getLinks() {
		return AppConfig::setting("model.links.{$this->name}");
	}

	function getLinkDefinition($link_name) {
		return AppConfig::setting("model.links.{$this->name}.$link_name");
	}
	
	function haveLinkDefinition($link_name) {
		return !!$this->getLinkDefinition($link_name);
	}
	
	function getLinkRelationship($link_name) {
		$link_def = $this->getLinkDefinition($link_name);
		if($link_def && ! empty($link_def['relationship']))
			return $this->loadRelationship($link_def['relationship']);
	}

	function getLinkModel($link_name, $target_alias = null, $required=null) {
		$link_def = $this->getLinkDefinition($link_name);
		if ($link_def) {
			if (!isset($target_alias))
				$target_alias = $link_name;
			$link_def['required'] = $required;
			if(isset($link_def['vname']) && ! isset($link_def['vname_module']))
				$link_def['vname_module'] = $this->module_dir;
			if (!empty($link_def['relationship'])) {
				$rel_spec = $this->loadRelationship($link_def['relationship']);
				if ($rel_spec) {
					$link_def['relationship_spec'] = $rel_spec;
					if (isset($rel_spec['join'])) {
						$link_def['is_join'] = 1;
						$link_def['target_alias'] = $target_alias;
						$link = new ModelDef($rel_spec['join']['model'], $link_def);
						$link->is_join = 1;
						return $link;
					}
					if (isset($rel_spec['target_model'])) // change to target_model
						return new ModelDef($rel_spec['target_model'], $link_def, $target_alias);
					$lhs = $rel_spec['left']['model'];
					$rhs = $rel_spec['right']['model'];
					if ($lhs == $this->name)
						$model = $rhs;
						// set link_def['side']?
					else if ($rhs == $this->name)
						$model = $lhs;
					if (!empty($model))
						return new ModelDef($model, $link_def, $target_alias);
				}
			}
			else if (isset($link_def['bean_name']))
				return new ModelDef($link_def['bean_name'], $link_def, $target_alias);
			else if(isset($link_def['custom_table'])) {
				$ctbl = $link_def['custom_table'];
				$cspec = array(
					'relationship_spec' => array(
						'left' => array('model' => $this->name, 'key' => 'id'),
						'right' => array('model' => $ctbl, 'key' => 'id_c'),
					),
				);
				return new ModelDef($ctbl, $cspec, $target_alias);
			}
		}
	}

	function getJoinTarget() {
		return new ModelDef($this->link_spec['bean_name'], null, $this->link_spec['target_alias']);
	}


	function getLinkTargetModel($link_name, $alias = null, $src_name = null, $required = null, $link_type = null) {
		$link_spec = $this->getLinkDefinition($link_name);
		if ($link_spec) {
			if (!isset($alias))
				$alias = $link_name;
			$link_spec['required'] = $required;
			$link_spec['link_type'] = isset($link_type) ? $link_type : 'joined';
			if (!empty($link_spec['relationship'])) {
				$rel_spec = $this->loadRelationship($link_spec['relationship']);
				if(isset($src_name) && $rel_spec && isset($rel_spec['sources'][$src_name])) {
					$rel_name = $rel_spec['sources'][$src_name]['relationship'];
					$link_spec['relationship'] = $rel_name;
					$rel_spec = $this->loadRelationship($rel_name);
					$src_model = $link_spec['bean_name'];
					$model = AppConfig::setting("model.sources.$src_model.$src_name.model");
					$link_spec['bean_name'] = $model;
				}
				if ($rel_spec) {
					$link_spec['relationship_spec'] = $rel_spec;
					if (isset($rel_spec['target_model'])) {
						return new ModelDef($rel_spec['target_model'], $link_spec, $alias);
					}
					if (!empty($link_spec['bean_name']))
						$model = $link_spec['bean_name'];
					else {
						$lhs = $rel_spec['left']['model'];
						$rhs = $rel_spec['right']['model'];
						if ($lhs == $this->name)
							$model = $rhs;
							// set link_def['side']?
						else if ($rhs == $this->name)
							$model = $lhs;
					}
					if (!empty($model)) {
						if (isset($rel_spec['join'])) {
							$link_spec['is_join'] = 1;
							$link = new ModelDef($model, null, $alias);
							$link->join_model = new ModelDef($rel_spec['join']['model'], $link_spec);
							$link->join_model->is_join = 1;
							return $link;
						}
						return new ModelDef($model, $link_spec, $alias);
					}
				}
			}
			else if (isset($link_spec['bean_name']))
				return new ModelDef($link_spec['bean_name'], $link_spec, $alias);
			else if(isset($link_spec['custom_table'])) {
				$ctbl = $link_spec['custom_table'];
				$cspec = array(
					'relationship_spec' => array(
						'left' => array('model' => $this->name, 'key' => 'id'),
						'right' => array('model' => $ctbl, 'key' => 'id_c'),
					),
				);
				return new ModelDef($ctbl, $cspec, $alias);
			}
		}
	}

	function getLinkSpec() {
		return $this->link_spec;
	}

	/**
	 * @return ModelDef
	 */
	function &getJoinModel() {
		return $this->join_model;
	}

	function loadRelationship($rel_name, $side = 'auto') {
		if(is_array($rel_name)) {
			$rel_model = $this->name;
			$rel = $rel_name;
		} else {
			$rel_model = AppConfig::setting("model.index.relationships.$rel_name");
			if ($rel_model)
				$rel = AppConfig::setting("model.relationships.$rel_model.$rel_name");
			else
				return;
		}
		if(is_array($rel)) {
			if ($rel['left']['model'] == $this->name && $side == 'auto')
				$side = 'left';
			else if ($rel['right']['model'] == $this->name)
				$side = 'right';
			else if ($side == 'auto')
				throw new IAHError("could not determine relationship side: $rel_model.$rel_name");
			if ($side == 'right') {
				$targ = $rel['left'];
				$rel['left'] = $rel['right'];
				$rel['right'] = $targ;
			}
			return $rel;
		}
	}

	function getModelName() {
		return $this->name;
	}

	function getTableName() {
		return $this->table_name;
	}

	function getPrimaryKey($as_array = false) {
		$pk = $this->primary_key;
		if ($as_array && !is_array($pk))
			$pk = array($pk);
		return $pk;
	}

	function getModuleDir() {
		return $this->module_dir;
	}
	
	function getIsUnion() {
		return $this->is_union;
	}

	function getDefaultOrderBy() {
		return $this->default_order_by;
	}

	function getTableAlias($non_empty = false) {
		if (empty($this->table_alias) && $non_empty)
			return $this->table_name;
		return $this->table_alias;
	}

	function setTableAlias($alias) {
		$this->table_alias = $alias;
	}

	function getJoinInfo(ModelDef &$link_model, $reverse = false) {
		/** @var $db DBManager */
		global $db;
		$info = array();

		if ($this->is_join && (! $link_model->link_spec || $reverse)) {
			$spec = $this->link_spec;
			$rel = $spec['relationship_spec'];
			if ($rel['left']['model'] == $link_model->name)
				$side = 'left';
			else if ($rel['right']['model'] == $link_model->name)
				$side = 'right';
			else
				throw new IAHModelError("cannot determine link side from relationship ({$link_model->name})");
			$other = ($side == 'left') ? 'right' : 'left';
			$link_key = $rel[$side]['key'];
			$self_key = $rel[$side]['join_key'];
			$many = ($rel[$reverse ? $other : $side]['count'] == 'many');
			if($reverse && isset($rel['join']['role_field'])) {
				$self_role_field = $rel['join']['role_field'];
				$self_role_value = $rel['join']['role_value'];
			}
		}
		else {
			$spec = $link_model->link_spec;
			if (!$spec)
				return null;
			if(isset($spec['relationship_spec'])) {
				$rel = $spec['relationship_spec'];
				if ($rel['left']['model'] == $link_model->name)
					$side = 'left';
				else if ($rel['right']['model'] == $link_model->name)
					$side = 'right';
				else
					throw new IAHModelError("cannot determine link side from relationship ({$link_model->name})");				
				$other = ($side == 'left') ? 'right' : 'left';
				if($this->is_join && isset($rel['join']) && $rel['join']['model'] == $this->name) {
					$link_key = $rel[$side]['key'];
					$self_key = $rel[$side]['join_key'];
				} else {
					$link_key = $rel[$side]['key'];
					$self_key = $rel[$other]['key'];
					
					if(isset($rel[$other]['role_field'])) {
						$self_role_field = $rel[$other]['role_field'];
						$self_role_value = $rel[$other]['role_value'];
					}
					if(isset($rel[$side]['role_field'])) {
						$link_role_field = $rel[$side]['role_field'];
						$link_role_value = $rel[$side]['role_value'];
					}
				}
			}
			else {
				$link_key = $link_model->getPrimaryKey();
				$self_key = array_get_default($spec, 'id_name', 'id');
			}
			$many = $reverse;
		}

		$del_model = $reverse ? $this : $link_model;
		$j_deleted = $del_model->haveFieldDefinition('deleted');
		if ($j_deleted)
			$del = $db->quoteField('deleted', $del_model->getTableAlias(true)) . '=0';
		else
			$del = '';

		$link = array(
			'base_table' => $reverse ? $link_model->getTableAlias(true) : $this->getTableAlias(true),
			'link_table' => $reverse ? $this->getTableName() : $link_model->getTableName(),
			'link_alias' => $reverse ? $this->getTableAlias() : $link_model->getTableAlias(),
			'many' => $many,
			'filter_deleted' => $del,
			'reversed' => $reverse,
			'required' => array_get_default($spec, 'required'),
		);

		if (isset($spec['function'])) { // FIXME - change to class_function, add separate function support
			$cls = $spec['class'];
			$fn = $spec['function'];
			$link = call_user_func(array($cls, $fn), $link);
		}
		else {
			$link_field = $db->quoteField($link_key, $link_model->getTableAlias(true));
			$self_field = $db->quoteField($self_key, $this->getTableAlias(true));
			$link['link_key'] = $link_key;
			$link['self_key'] = $self_key;
			$link['clause'] = "$link_field=$self_field";
			if(! empty($link_role_field)) {
				$lf = $db->quoteField($link_role_field, $link_model->getTableAlias(true));
				$lv = $db->quote($link_role_value);
				$link['clause'] .= " AND $lf='$lv'";
			}
			if(! empty($self_role_field)) {
				$lf = $db->quoteField($self_role_field, $this->getTableAlias(true));
				$lv = $db->quote($self_role_value);
				$link['clause'] .= " AND $lf='$lv'";
			}
		}

		return $link;
	}

	function getStandardWhere($use_table_name = true) {
		/** @var $db DBManager */
		global $db;

		$wc = array();
		if ($this->haveFieldDefinition('deleted')) {
			if ($use_table_name)
				$table = $this->getTableAlias(true);
			else
				$table = null;
			$wc['deleted'] = $db->quoteField('deleted', $table).'=0';
		}
		return $wc;
	}

	function getParentKeyField() {
		if (($ls = $this->getLinkSpec())) {
			$rs = array_get_default($ls, 'relationship_spec', array());
			if (isset($rs['right']))
				return array_get_default($rs['right'], 'key');
		}
	}

	function getParentKeyWhere(&$link_model, $parent_key, $use_table_name = true) {
		/** @var $db DBManager */
		global $db;

		if(is_array($parent_key)) {
			$pkq = array_map([$db, 'quote'], $parent_key);
			$pkq = '(\''.implode('\',\'', $pkq).'\')';
			$pkop = "IN $pkq";
			$pknop = "NOT IN $pkq";
		} else {
			$pkq = $db->quote($parent_key);
			$pkop = "='$pkq'";
			$pknop = "!='$pkq'";
		}
		if ($this->is_join) {
			$info = $this->getJoinInfo($link_model);
			$k = $info['self_key'];
		} else {
			$k = $this->getParentKeyField();
		}
		if (!$k)
			return array();
		$kenc = $db->quoteField($k, $use_table_name ? $this->getTableAlias(true) : null);
		
		// custom for related emails subpanel
		if($link_model->getModelName() == 'Email' && $this->link_spec && $this->link_spec['relationship'] == 'emails_related_emails') {
			$idenc = $db->quoteField('id', $use_table_name ? $this->getTableAlias(true) : null);
			return array($k => "($kenc=(SELECT thread_id FROM emails WHERE id $pkop) AND $idenc $pknop)");
		}
		
		return array($k => "$kenc $pkop");
	}
	
	function getSources() {
		return AppConfig::setting("model.sources.{$this->name}");
	}

}


?>
