<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */



require_once('include/config/ConfigBase.php');
require_once('modules/DynFields/DynField.php');

class ModelManager extends ConfigBase {
	var $local;
	var $init_time;
	var $models_loaded;
	var $skipped;
	
	static $types_paths = array(
		'include/config/supported_types.php',
	);
	static $app_field_paths = array(
		'include/config/app_defined_fields.php',
	);
	static $model_keys = array(
		'fields', 'raw_fields', 'links', 'raw_links',
		'indices', 'raw_indices', 'relationships', 'hooks',
		'column_fields', 'extra_fields', 'required_fields', 'audited_fields',
		'custom_fields', 'file_fields', 'image_fields', 'list_fields', 'sources',
		'personal_fields', 'consent_fields', 'portal',
	);
	
	
	function init() {
		parent::init();
		$this->local = array();
		$this->models_loaded = array();
		$this->init_time = microtime(true);
	}
	
	
	static function model_cache_file() {
		return CacheManager::get_location('system/') . 'model_cache.php';
	}

	
	static function scan_models() {
		// scan all model files
		$fnames = glob_sorted(
			'modules/*/models/*.php',
			'include/models/*.php');
		$custom_dir = AppConfig::custom_dir();
		$skipped = array();
		$to_parse = array('main' => array(), 'ext' => array(), 'custom' => array());
		foreach($fnames as $f) {
			if(! is_file($f) || ! is_readable($f)) {
				$skipped['unreadable'][] = $f;
				continue;
			}
			$parts = explode('.', basename($f));
			array_pop($parts);
			if(count($parts) < 2) {
				$skipped['notype'][] = $f;
				continue;
			} else {
				list($stype, $name) = $parts;
			}
			$finfo = array('name' => $f);
			if($stype == 'bean' || $stype == 'link' || $stype == 'table' || $stype == 'template' || $stype == 'union') {
				if(isset($to_parse['main'][$name])) {
					$skipped['duplicate'][] = $f;
					continue;
				}
				$to_parse['main'][$name] = $finfo;
				$ext_files = AppConfig::setting("ext.models.$name");
				if($ext_files) $to_parse['ext'][$name] = $ext_files;
				$fcust = $custom_dir . $f;
				if(file_exists($fcust)) {
					if(! is_file($fcust) || ! is_readable($fcust))
						$skipped['unreadable'][] = $fcust;
					else {
						$finfo = array('name' => $fcust);
						$to_parse['custom'][$name] = $finfo;
					}
				}
			}
		}
		
		return compact('to_parse', 'skipped');
	}
	
	
	static function load_custom_fields() {
		$custom = array();
		foreach(DynField::load_available_fields() as $row) {
			if(empty($row['custom_bean'])) continue; // does not exist during upgrade
			$custom[$row['custom_bean']][] = $row;
		}
		return $custom;
	}
	
	function loadCustomFields($lastUpd=0) {
		if(defined('IAH_IN_INSTALLER') || defined('IAH_NO_DATABASE'))
			$custom = array();
		else {
			$cache_key = 'AppConfig.model.custom_field_defs';
			$custom = AppConfig::cache_get_timed($cache_key, $lastUpd, $key_found);
			if(empty($key_found)) {
				$custom = self::load_custom_fields();
				AppConfig::cache_put_timed($cache_key, $custom, true, true);
			}
		}
		$this->local['custom_field_defs'] = $custom;
	}
	
	
	function createIndex($scan=null) {
		$to_parse = $skipped = array();
		if(! isset($scan))
			$scan = self::scan_models();
		extract($scan); // set to_parse and skipped

		$data = array(
			'hash' => self::cfg_hash(),
			'time' => microtime(true),
			'detail' => array(),
			'source' => array(),
			'index' => array(
				'relationships' => array(), 'securitygroups' => array(),
				'sequences' => array(),
			),
		);
		
		foreach($to_parse['main'] as $k => $f) {
			$finfo = array($f);
			if(isset($to_parse['ext'][$k])) {
				$finfo = array_merge($finfo, $to_parse['ext'][$k]);
			}
			if(isset($to_parse['custom'][$k])) {
				$finfo[] = $to_parse['custom'][$k];
			}
			$files = array_column($finfo, 'name');
			$src = array('type' => 'files', 'files' => $finfo);
			
			try {
				$model = self::load_files($files);
			} catch(IAHConfigFileError $e) {
				$skipped['noparse'][] = $f['name'];
				$this->parse_errors[] = $e;
				$GLOBALS['log']->error($e->getMessage());
				continue;
			}
			
			if(! isset($model['detail'])) {
				$skipped['nodetail'][] = $f['name'];
				continue;
			}
			$detail =& $model['detail'];
			if(! isset($detail['type'])) {
				$skipped['notype'][] = $f['name'];
				continue;
			}

			if(isset($detail['name']))
				$name = $detail['name'];
			else
				$name = $detail['name'] = $k;

			if($detail['type'] == 'union') {
				$detail['is_union'] = true;
			}
			$is_union = ! empty($detail['is_union']);

			if(substr($f['name'], 0, 8) == 'modules/') {
				$p = strpos($f['name'], '/', 8);
				$mod_name = substr($f['name'], 8, $p - 8);
				$detail['module_dir'] = $mod_name;
			} else {
				$mod_name = '';
			}
			
			if($detail['type'] == 'bean') {
				if(! isset($detail['bean_name']))
					$detail['bean_name'] = $name;
				if(! $is_union) {
					if(! isset($detail['bean_file'])) {
						$skipped['nobean'][] = $f['name'];
						continue;
					}
					if(! is_readable($detail['bean_file'])) {
						$skipped['beanmissing'][] = $f['name'];
						continue;
					}
					if(isset($detail['module_dir'])) {
						if($detail['bean_file'] == 'modules/' . $detail['module_dir'] . '/' . $detail['bean_name'] . '.php')
							unset($detail['bean_file']); // filled in later to save index space
					}
				}
			}
			
			if(! isset($detail['table_name']) && $detail['type'] != 'template' && ! $is_union) {
				$skipped['notable'][] = $f['name'];
				continue;
			}
						
			$data['source'][$name] = $src;
			unset($detail['comment']);
			$data['detail'][$name] = $detail;			

			$is_personal = !empty($detail['personal_data']);
			if ($is_personal) {
				$this->addPersonalRel($model['relationships'], $name, $detail['module_dir']);
			}

			if(isset($model['relationships'])) {
				foreach($model['relationships'] as $k => $v) {
					if ($k == 'app.signed_documents')
						$k = 'signed_documents_' . strtolower($name);
					$data['index']['relationships'][$k] = $name;
				}
			}
			
			if(isset($model['links']['app.securitygroups'])) {
				$data['index']['securitygroups'][] = $name;
			}
			
			if(! empty($detail['edit_sequence'])) {
				$data['index']['sequences'][] = $name;
			}
		}
		$this->skipped = $skipped;
		
		return $data;
	}

	function processRelatedValues($model_name, &$fields) {
		static $recursion_guard = [];
		if (in_array($model_name, $recursion_guard))
			return;
		$recursion_guard[] = $model_name;
		static $ban = array('name', 'type', 'vname', 'vname_module', 'source', 'editable', 'massupdate', 'unified_search');
		foreach ($fields as $fn => $f) {
			if ($f['type'] == 'related_value') {
				$ref = $f['ref'];
				if (isset($fields[$ref]) && $fields[$ref]['type'] == 'ref') {
					$ref = $fields[$ref];
					$ref_model = $ref['bean_name'];
					if ($ref_model == $model_name) {
						$ref_fields =& $fields;
					} else {
						$ref_fields = AppConfig::setting("model.fields.{$ref_model}");
					}
					if (isset($ref_fields[$f['field']])) {
						$target_def = $ref_fields[$f['field']];
						$type = $target_def['type'];
						if ($type == 'name')
							$type = 'varchar';
						$f['type'] = $type;

						foreach ($target_def as $k => $v) {
							if (!in_array($k, $ban)) {
								$f[$k] = $v;
							}
						}
						$f['inferred'] = true;
						$f['is_related_value'] = true;
						$f['related_value_orig'] = array(
							'ref' => $f['ref'],
							'field' => $f['field'],
							'source' => $f['source'],
						);
						$f['source'] = array(
							'type' => 'field',
							'field' => $ref['name'] . '.' . $f['field'],
						);
						unset($f['ref']);
						unset($f['field']);
						$fields[$fn] = $f;
					} else {
						$fields[$fn]['source'] = array('type' => 'non-db');
					}
				}
			}
		}
		array_pop($recursion_guard);
	}

	function extendIndex(&$data) {
		$custom = $this->getSetting('custom_field_defs');
		foreach(array('by_type', 'by_module', 'system', 'bean_module', 'unified_search',
				'custom_models', 'reportable', 'icons', 'bean_files', 'personal_data') as $f)
					$data['index'][$f] = array();
		foreach($data['detail'] as $name => &$detail) {
			if(! empty($detail['audit_enabled']) && empty($detail['is_union'])) {
				if(empty($detail['audit_table_name']))
					$detail['audit_table_name'] = $detail['table_name'] . '_audit';
			}

			$data['index']['by_type'][$detail['type']][] = $name;
			
			$mod_name = array_get_default($detail, 'module_dir');			
			if($mod_name) {
				$data['index']['by_module'][$mod_name][] = $name;
				$data['index']['bean_module'][$name] = $mod_name;
			} else {
				$data['index']['system'][] = $name;
			}
			
			if(! empty($detail['unified_search'])) {
				$us = $detail['unified_search'];
				if(! is_array($us) || array_get_default($us, 'enabled', true))
					$data['index']['unified_search'][] = $name;
			}
			
			if(! empty($detail['reportable'])) {
				$data['index']['reportable'][] = $name;
			}

			if(! empty($detail['personal_data'])) {
				$data['index']['personal_data'][] = $name;
			}

			if(! empty($detail['icon'])) {
				$data['index']['icons'][$name] = $detail['icon'];
			}
			
			if($detail['type'] == 'bean') {
				if(! isset($detail['bean_file']))
					$detail['bean_file'] = 'modules/' . $detail['module_dir'] . '/' . $detail['bean_name'] . '.php';
				$data['index']['bean_files'][$name] = $detail['bean_file'];
			}

			if(! empty($custom[$name]) && ! empty($detail['table_name'])) {
				$data['index']['custom_models'][] = $name;
				$cust_tbl = $detail['table_name'].'_cstm';
				$data['detail'][$cust_tbl] = array(
					'type' => 'custom',
					'table_name' => $cust_tbl,
					'primary_key' =>  'id_c',
					'comment' => "Custom field table for '$name'",
				);
				$data['source'][$cust_tbl] = array('type' => 'custom', 'base_model' => $name);
				$data['index']['by_type']['custom'][] = $cust_tbl;
			}
		}
	}
	
	
	function updateIconsCss($index) {
		$mod_icons = AppConfig::setting('modinfo.index.icons');
		$bean_icons = $index['icons'];
		if(! $bean_icons && ! $mod_icons)
			return;
		$all = array();
		foreach($bean_icons as $bean => $icon) {
			$all[$icon][] = '.bean-' . $bean;
			$mod = $index['bean_module'][$bean];
			if(! isset($mod_icons[$mod]))
				$mod_icons[$mod] = $icon;
		}
		foreach($mod_icons as $mod => $icon) {
			if(! empty($index['by_module'][$mod])) {
				foreach($index['by_module'][$mod] as $bean) {
					if(in_array($bean, $index['by_type']['bean']) && ! isset($bean_icons[$bean]))
						$all[$icon][] = '.bean-' . $bean;
				}
			}
			$all[$icon][] = '.module-' . $mod;
		}

		$path = AppConfig::custom_dir() . 'themes/Default/';
		if(file_exists($path) || @mkdir_recursive($path)) {
			$css = '';
			$greyscale = implode('', array(
				'filter: url("../../../themes/Default/ui/grayscale.svg#grayscale");', // Firefox 3.5+
				'filter: gray;', // IE6-9
				'-webkit-filter: grayscale(1);', // Google Chrome & Safari 6+
			));
			foreach($all as $icon => $cls) {
				$cls_norm = $cls_grey = array();
				foreach($cls as $c) {
					$cls_norm[] = $c;
					$cls_norm[] = "$c.active";
					$cls_norm[] = "$c.grey.active";
					$cls_norm[] = "button:hover $c";
					$cls_norm[] = "button:focus $c";
					$cls_norm[] = ".menu-option.current $c";
					$cls_norm[] = ".tab-current $c";
					
					$cls_grey[] = "button $c";
					$cls_grey[] = ".menu-option $c";
					$cls_grey[] = "$c.grey";
				}
				$cls_grey = implode(',', $cls_grey);
				$cls_norm = implode(',', $cls_norm);
				$css .= $cls_grey . "{opacity:0.9;$greyscale}\n";
				$css .= $cls_norm . ',' . $cls_grey . '{background-image:url("../../../' . $icon . '");background-size: 16px 16px;background-position: 0 0}' . "\n";
				$css .= $cls_norm . "{filter:none;-webkit-filter:grayscale(0);opacity:1.0}\n";
			}
			file_put_contents($path . 'icons.css', $css);
		}
	}
	
	
	function loadIndex($lastUpdate=0) {
		$cache_key = 'AppConfig.model.detail_index';
		$cache_file = self::model_cache_file();
		$hash = self::cfg_hash();
		$index = AppConfig::cache_file_read($cache_file, $cache_key, $lastUpdate);
		if(! is_array($index) || ! isset($index['hash']) || $index['hash'] != $hash) {
			$from = self::config_debug("create model cache");
			$index = $this->createIndex();
			$created = true;
			AppConfig::set_expiry('model');
			AppConfig::cache_file_write($index, $cache_file, $cache_key);
			$this->init_time = microtime(true);
			self::config_debug("done model cache", $from);
		}
		$this->extendIndex($index);
		if(! empty($created))
			$this->updateIconsCss($index['index']);
		foreach($index as $k => &$v)
			$this->local[$k] = $v;
		if(isset($this->skipped))
			$this->local['index']['skipped'] = $this->skipped;
	}
	
	
	function loadModel($name, $lastUpdate=0) {
		$no_cache = $this->memoryCacheDisabled();
		$cache_key = "AppConfig.model.full.$name";
		if(! $no_cache)
			$model = AppConfig::cache_get_timed($cache_key, $lastUpdate, $found);
		else
			$found = false;
		
		if(! $found) {
			$source = $this->getSetting("source.$name");
			if(! empty($source) && $source['type'] == 'files') {
				$finfo = $source['files'];
				$files = array_column($finfo, 'name');
				// may throw parse error
				$model = self::load_files($files);
			} elseif (! empty($source) && $source['type'] == 'custom') {
				$rel_model = $source['base_model'];
				$model = array(
					'fields' => array(
						'app.id' => array('name' => 'id_c', 'comment' => 'Custom field table identifier'),
					),
				);
				$cust_fields = $this->getSetting("custom_field_defs.$rel_model");
				$custom_defs = DynField::get_custom_field_definitions($cust_fields);
				foreach($custom_defs as $k => $v)
					$model['fields'][$k] = $v;
			} else {
				$model = array();
			}
			if(! $no_cache)
				AppConfig::cache_put_timed($cache_key, $model);
		}


		$is_personal = !empty($model['detail']['personal_data']);

		if ($is_personal) {
			$model['links']['__personal'] = [
				'relationship' => '__personal_' . $name,
				'module' => 'DataPrivacy',
				'bean_name' => 'PrivacyActivity',
				'vname' => 'LBL_PRIVACY_ACTIVITY_SUBPANEL_TITLE',
				'vname_module' => 'DataPrivacy',
			];
		}

		if(isset($model['links'])) {
			$model['raw_links'] = $model['links'];
			$links = $model['links'] = $this->expandLinks($model['links'], $name);
		} else
			$links = array();


		if(isset($model['fields'])) {
			if (!empty($model['detail']['personal_data'])) {
				if (!isset($model['fields']['personal_info_source']) && !isset($model['fields']['app.personal_info_source'])) {
					$model['fields']['app.personal_info_source'] = [];
				}
			}
			$model['raw_fields'] = $model['fields'];
			$fs = $model['fields'];
			$model['fields'] = $this->expandFields($model['fields'], $model['links'], $name);

			$pk = $this->getSetting("detail.$name.primary_key");
			if(! is_array($pk)) $pk = array($pk);
			foreach($pk as $pke)
				if(isset($model['fields'][$pke]))
					$model['fields'][$pke]['required'] = true;

			$model['custom_fields'] = array();
			$custom = $this->getSetting("custom_field_defs.$name");
			if($custom) {
				$cust_tbl = $this->getSetting("detail.$name.table_name").'_cstm';
				$custom_defs = $this->getSetting("fields.$cust_tbl");
				if($custom_defs) {
					foreach($custom_defs as $k => $v) {
						$model['custom_fields'][] = $k;
						$c_src = array();
						if(isset($v['source'])) {
							$c_src = $v['source'];
						}
						$c_src['type'] = 'custom_field';
						$c_src['field'] = "_cstm.$k";
						$v['source'] = $c_src;

						$model['fields'][$k] = $v;
					}
					$model['links']['_cstm'] = array('custom_table' => $cust_tbl);
				}
			}
			
			$col = $ext = $req = $aud = $file = $img = $lst = $personal = $consent = [];
			foreach($model['fields'] as $k => &$v) {
				$src_type = $v['source']['type'];
				$non_db = ($src_type != 'db' && $src_type != 'multienum');
				if($non_db) {
					if($v['type'] != 'link')
						$ext[] = $k;
				} else
					$col[] = $k;
				if($v['type'] == 'file_ref')
					$file[] = $k;
				if($v['type'] == 'image')
					$img[] = $k;
				if(! empty($v['required']))
					$req[] = $k;
				if( (!empty($v['audited']) || !empty($v['personal_info'])) && (!$non_db || ($src_type == 'custom_field')))
					$aud[] = $k;
				if( (!empty($v['personal_info'])) && (!$non_db || ($src_type == 'custom_field')))
					$personal[] = $k;
				if( (!empty($v['privacy_consent'])) && (!$non_db || ($src_type == 'custom_field')))
					$consent[] = $k;
				if(! isset($v['reportable']) || $v['reportable'])
					$lst[] = $k;
			}
			$model['column_fields'] = $col;
			$model['extra_fields'] = $ext;
			$model['required_fields'] = $req;
			$model['audited_fields'] = $aud;
			$model['personal_fields'] = $personal;
			$model['consent_fields'] = $consent;
			$model['file_fields'] = $file;
			$model['image_fields'] = $img;
			$model['list_fields'] = $lst;
			
			$indices = array_get_default($model, 'indices', array());		
			$model['raw_indices'] = $indices;
			$pk = $this->getSetting("detail.$name.primary_key", '');
			$model['indices'] = $this->expandIndices($indices, $pk, $model['fields']);

			$rels = array_get_default($model, 'relationships', array());
			if ($is_personal)
				$this->addPersonalRel($rels, $name);
			$model['raw_relationships'] = $rels;
			$model['relationships'] = $this->expandRelationships($rels, $name, $pk);
		}

		foreach(self::$model_keys as $k)
			if(isset($model[$k]))
				$this->local[$k][$name] =& $model[$k];
		
		if(isset($model['fields']))
			$this->processRelatedValues($name, $model['fields']);

		$this->models_loaded[$name] = 1;
	}

	function addPersonalRel(&$rels, $name, $module = null) {
		if (is_null($module))
			$module = $this->getSetting("detail.$name.module_dir");
		$rels['__personal_' . $name] = [
			'key' => 'id',
			'target_bean' => 'PrivacyActivity',
			'target_key' => 'parent_id',
			'relationship_type' => 'one-to-many',
			'role_column' => 'parent_type',
			'role_value' => $module,
		];
	}

	function expandFields($defs, &$links, $model_name) {
		if(! isset($defs))
			return;

		$updefs = array();
		$app_fields = null;
		$supp_types = $this->getSetting('supported_types', array());
		$chk_currencies = array();
		
		foreach($defs as $k => $v) {
			// populate field defs referencing app-level field templates
			if(substr($k, 0, 4) == 'app.') {
				$ak = $k;
				if(! isset($app_fields))
					$app_fields = $this->getSetting('app_defined_fields', array());
				$k = substr($k, 4);
				if(isset($app_fields['single'][$k])) {
					$appf = $app_fields['single'][$k];
					$appf['from_app_field'] = $ak;
					self::merge_values($v, $appf);
					if(isset($v['name']))
						$k = $v['name'];
					$v = $appf;
				}
				else if(isset($app_fields['multi'][$k])) {
					$new_vs = $app_fields['multi'][$k];
					// allow property overrides of expanded fields
					self::merge_values($v, $new_vs);
					foreach($this->expandFields($new_vs, $_l, $model_name) as $k => $v) {
						$v['inferred'] = true;
						$v['from_app_field'] = $ak;
						$updefs[$k] = $v;
					}
					continue;
				}
				else {
					$GLOBALS['log']->error("Undefined application field in model $model_name: app.$k");
					continue;
				}
			}
			if(! is_array($v)) {
				$GLOBALS['log']->error("Field $k removed from model $model_name, not an array");
				continue;
			}
			if(! isset($v['name']))
				$v['name'] = $k;
			else {
				if(! is_int($k))
					$v['alias'] = $k;
				else if(isset($v['alias']))
					$k = $v['alias'];
				else
					$k = $v['name'];
			}
			if(! isset($v['type'])) {
				$GLOBALS['log']->error("Field $k removed from model $model_name, missing type");
				continue;
			}


			$orig_type = $v['type'];
			// expand fields according to extended 'supported_types' field attributes
			if(isset($supp_types[$orig_type])) {
				foreach($supp_types[$orig_type] as $k2 => $v2)
					if(! array_key_exists($k2, $v) || $k2 == 'type') $v[$k2] = $v2;
			}

			self::fix_field_source($v);

			// handle 'ref' field types
			if($v['type'] == 'ref' && (empty($v['source']) || $v['source']['type'] != 'field')) {
				$id_name = array_get_default($v, 'id_name');
				if(! $id_name)
					$id_name = $v['id_name'] = $k.'_id';
				if(! isset($defs[$id_name])) {
					$id_def = array(
						'name' => $id_name,
						'type' => 'id',
						'reportable' => false,
						'inferred' => true,
						'for_ref' => $k,
					);
					if(! empty($v['required']))
						$id_def['required'] = true;
					if(isset($v['default'])) {
						$id_def['default'] = $v['default'];
						unset($v['default']);
					}
					if(! empty($v['editable']))
						$id_def['editable'] = $v['editable'];
					if(! empty($v['updateable']))
						$id_def['updateable'] = $v['updateable'];
					if(! empty($v['audited']))
						$id_def['audited'] = true;
					if(isset($v['vname']))
						$id_def['vname'] = $v['vname'];
					if(isset($v['isnull']))
						$id_def['isnull'] = $v['isnull'];
					$extdefs[$id_name] = $id_def;
				} else
					$defs[$id_name]['for_ref'] = $k;
				$links[$k] = $v;
			}
			else if($v['type'] == 'currency') {
				$base_name = array_get_default($v, 'base_field');
				if(! $base_name) {
					if(substr($k, 0, 4) == 'raw_')
						$base_name = substr($k, 4);
					else if(isset($defs[$k.'_usdollar']))
						$base_name = $k.'_usdollar';
					else
						$base_name = $k.'_usd';
					$v['base_field'] = $base_name;
				}
				if(! isset($defs[$base_name])) {
					$base_def = array(
						'name' => $base_name,
						'type' => 'base_currency',
						'inferred' => true,
						'for_currency' => $k,
					);
					if(! empty($v['required']))
						$base_def['required'] = true;
					if(isset($v['default']))
						$base_def['default'] = $v['default'];
					if(! empty($v['audited']))
						$base_def['audited'] = true;
					if(isset($v['vname']))
						$base_def['vname'] = $v['vname'];
					if(! empty($v['editable']))
						$base_def['editable'] = $v['editable'];
					if(! empty($v['updateable']))
						$base_def['updateable'] = $v['updateable'];
					if(! empty($v['source']) && $v['source']['type'] != 'db')
						$base_def['source']['type'] = 'non-db';
					$extdefs[$base_name] = $base_def;
				} else
					$defs[$base_name]['for_currency'] = $k;
				$chk_currencies[] = $k;
				$chk_currencies[] = $base_name;
			}
			else if($v['type'] == 'phone' || $v['type'] == 'fax') {
				if(empty($v['source']) || $v['source']['type'] == 'db') {
					$raw_name = $v['name'] . '__raw';
					$raw_def = array(
						'name' => $raw_name,
						'type' => 'varchar',
						'reportable' => false,
						'editable' => false,
						'inferred' => true,
						'len' => 40,
					);
					$extdefs[$raw_name] = $raw_def;
				}
			}
			else if($v['type'] == 'raw_currency')
				$chk_currencies[] = $k;

			if($orig_type == 'currency_ref' && ! empty($v['exchange_rate'])) {
				$rate_name = $v['exchange_rate'];
				if(! is_string('rate_name'))
					$rate_name = 'exchange_rate';
				if(! isset($defs[$rate_name])) {
					$rate_def = $supp_types['exchange_rate'];
					$rate_def['name'] = $rate_name;
					$rate_def['inferred'] = true;
					$rate_def['for_currency'] = $k;
					if(! empty($v['required']))
						$rate_def['required'] = true;
					if(! empty($v['audited']))
						$rate_def['audited'] = true;
					$extdefs[$rate_name] = $rate_def;
				}
			}

			if($v['type'] == 'multienum' && ! empty($v['multi_select_group'])) {
				$v['source']['type'] = 'multienum';
				$grp = $v['multi_select_group'];
				$mc = array_get_default($v, 'multi_select_count', 10);
				for($c = 1; $c <= $mc; $c++) {
					$enum_name = $grp.($c == 1 ? '' : $c);
					$enum_def = array(
						'name' => $enum_name,
						'type' => 'enum',
						'options' => $v['options'],
						'reportable' => false,
						'massupdate' => false,
						'multi_select_group' => $grp,
					);
					foreach(array('vname', 'options', 'len') as $f)
						if(isset($v[$f]))
							$enum_def[$f] = $v[$f];
					$extdefs[$enum_name] = $enum_def;
				}
			}
			
			if($v['type'] == 'image') {
				if(! isset($v['filename']))
					$v['filename'] = $k . '_filename';
				if(! empty($v['thumbnail']) && ! isset($v['thumbnail_name']))
					$v['thumbnail_name'] = $k . '_thumb';
			}

			$updefs[$k] = $v;
			if(isset($extdefs)) {
				foreach($extdefs as $k2 => $v2) {
					$t2 = array_get_default($v2, 'type');
					// expand fields according to extended 'supported_types' field attributes
					if($t2 && isset($supp_types[$t2])) {
						foreach($supp_types[$t2] as $k3 => $v3)
							if(! array_key_exists($k2, $v2) || $k3 == 'type') $v2[$k3] = $v3;
					}
					$updefs[$k2] = $v2;
				}
			}
			unset($extdefs);	
		}
				
		foreach($chk_currencies as $k) {
			$v =& $updefs[$k];
			if(empty($v['currency']) || ! isset($updefs[$v['currency']]))
				$v['currency'] = isset($updefs['currency']) ? 'currency' : null;
			if($v['currency']) {
				$v['currency_id'] = array_get_default($updefs[$v['currency']], 'id_name');
				$v['exchange_rate'] = array_get_default($updefs[$v['currency']], 'exchange_rate');
			} else {
				$v['currency_id'] = $v['exchange_rate'] = null;
			}
			if($v['type'] == 'currency' && isset($v['base_field'])) {
				$base = $v['base_field'];
				if(isset($updefs[$base])) {
					$updefs[$base]['currency'] = $v['currency'];
					$updefs[$base]['currency_id'] = $v['currency_id'];
					$updefs[$base]['exchange_rate'] = $v['exchange_rate'];
				}
			}
		}

		foreach($updefs as &$def)
			self::fix_field_source($def);

		return $updefs;
	}
	
	static function fix_field_source(&$field_def) {
		if(isset($field_def['source'])) {
			if(is_array($field_def['source'])) {
				if(! isset($field_def['source']['type'])) {
					if(! empty($field_def['source']['field']))
						$field_def['source']['type'] = 'field';
					else if(! empty($field_def['source']['formula']))
						$field_def['source']['type'] = 'formula';
					else if(! empty($field_def['source']['calc_formula']))
						$field_def['source']['type'] = 'db';
					else if(! empty($field_def['source']['function']) || ! empty($field_def['source']['value_function']))
						$field_def['source']['type'] = 'function';
					else
						$field_def['source']['type'] = 'none';
				}
			} else {
				$field_def['source'] = array('type' => $field_def['source']);
			}
		} else {
			$t = array_get_default($field_def, 'type');
			$field_def['source'] = array('type' => ($t == 'ref' ? 'non-db' : 'db'));
		}
	}
	
	function expandLinks($links, $model_name) {
		$ret = array();
		foreach($links as $k => &$v) {
			if($k == 'app.securitygroups') {
				$k = 'securitygroups';
				$v = array(
					'module'=>'SecurityGroups',
					'relationship' => array_get_default($v, 'relationship', ''),
					'bean_name'=>'SecurityGroup',
					'vname'=>'LBL_SECURITYGROUPS',
				);
			} else if($k == 'app.signed_documents') {
				$k = 'signed_documents';
				$v = array(
					'module'=>'Documents',
					'relationship' => 'signed_documents_' . strtolower($model_name),
					'bean_name' => 'Document',
				);
			}
			$v['name'] = $k;
			$v['type'] = 'link';
			$v['source']['type'] = 'non-db';
			$ret[$k] = $v;
		}
		return $ret;
	}
	
	function expandIndices($indices, $primary='', $fields = array()) {
		$ret = array();
		if($primary) {
			if(! is_array($primary))
				$primary = array($primary);
			$ret['pk'] = array(
				'name' => 'pk',
				'type' => 'primary',
				'fields' => $primary,
			);
		}
		foreach($indices as $k => &$v) {
			$v['name'] = $k;
			if(! isset($v['type']))
				$v['type'] = 'index';
			$ret[$k] = $v;
		}

		foreach ($fields as $f) {
			if ($f['type'] == 'phone' || $f['type'] == 'fax') {
				$raw_name = $f['name'] . '__raw';
				if (isset($fields[$raw_name]) && $fields[$raw_name]['source']['type'] == 'db') {
					$idx_name = 'idx_' . $raw_name;
					$ret[$idx_name] = array(
						'name' => $idx_name,
						'type' => 'index',
						'fields' => array($raw_name),
					);
				}
			}
		}

		return $ret;
	}
	
	
	function expandAppRelationship($name, $rel, $model_name, $pk) {
		$module = $this->getSetting("detail.$model_name.module_dir");
		if ($name == 'app.signed_documents') {
			$name = 'signed_documents_' . strtolower($model_name);
			$rel = [
				'key' =>  'id',
				'managed' => true,
				'target_bean' => 'Document',
				'target_key' => 'parent_id',
				'role_column' =>  'parent_type',
				'role_value' =>  $module,
				'relationship_type' => 'one-to-many',
			];
		}
		return [$name, $rel];
	}
	
	// need to update relationship definitions to simplify this
	function expandRelationships($rels, $model_name, $pk) {
		$ret = array();
		foreach($rels as $k => &$v) {
			list($k, $v) = $this->expandAppRelationship($k, $v, $model_name, $pk);
			$nrel = array();
			$type = array_get_default($v, 'relationship_type', 'many-to-one');
			$type = explode('-to-', $type);
			if(isset($v['target_bean'])) {
				$nrel = array(
					'left' => array(
						'model' => $model_name,
						'key' => array_get_default($v, 'key', $pk),
						'count' => $type[0],
					),
					'right' => array(
						'model' => $v['target_bean'],
						'key' => $v['target_key'],
						'count' => $type[1],
					),
				);
				if(isset($v['join'])) {
					if(isset($v['join']['model']))
						$jmod = $v['join']['model'];
					else
						$jmod = $v['join']['table'];
					$nrel['join'] = array(
						'model' => $jmod,
					);
                    if (isset($v['join']['role_column']) && isset($v['join']['role_value'])) {
                        $nrel['join']['role_field'] = $v['join']['role_column'];
                        $nrel['join']['role_value'] = $v['join']['role_value'];
                    }
					$nrel['left']['join_key'] = $v['join']['source_key'];
					$nrel['right']['join_key'] = $v['join']['target_key'];
				}
				else if(isset($v['role_column']) && isset($v['role_value'])) {
					$role_side = empty($v['role_column_reverse']) ? 'right' : 'left';
					$nrel[$role_side]['role_field'] = $v['role_column'];
					$nrel[$role_side]['role_value'] = $v['role_value'];
				}
				if(isset($v['sources'])) {
					$nrel['sources'] = $v['sources'];
				}
			}
			else {
				$nrel = array(
					'left' => array(
						'model' => @$v['lhs_bean'],
						'key' => @$v['lhs_key'],
						'count' => $type[0],
					),
					'right' => array(
						'model' => @$v['rhs_bean'],
						'key' => @$v['rhs_key'],
						'count' => $type[1],
					),
				);
				if(isset($v['join_key_lhs'])) {
					if(isset($v['join_table']))
						$jmodel = $v['join_table'];
					else
						$jmodel = $model_name;
					$nrel['join'] = array(
						'model' => $jmodel,
					);
					if(isset($v['relationship_role_column']))
						$nrel['join']['role_field'] = $v['relationship_role_column'];
					if(isset($v['relationship_role_column_value']))
						$nrel['join']['role_value'] = $v['relationship_role_column_value'];
					$nrel['left']['join_key'] = $v['join_key_lhs'];
					$nrel['right']['join_key'] = $v['join_key_rhs'];
				}
			}
			$nrel['name'] = $k;
			if(isset($v['managed'])) {
				$nrel['managed'] = $v['managed'];
			}
			$ret[$k] = $nrel;
		}
		return $ret;
	}
	
	
	function loadTypes($lastUpd=0) {
		$paths = array();
		foreach(self::$types_paths as $p) {
			$paths[] = $p;
			$paths[] = AppConfig::custom_dir().$p;
		}
		$cache_key = 'AppConfig.model.supported_types';
		if(! $this->stat_disabled)
			$lastUpd = true;
		$this->local['supported_types'] = self::cache_load_files($cache_key, $paths, null, $lastUpd);
	}

	function loadAppFields($lastUpd=0) {
		$paths = array();
		foreach(self::$app_field_paths as $p) {
			$paths[] = $p;
			$paths[] = AppConfig::custom_dir().$p;
		}
		$cache_key = 'AppConfig.model.app_defined_fields';
		if(! $this->stat_disabled)
			$lastUpd = true;
		$this->local['app_defined_fields'] = self::cache_load_files($cache_key, $paths, null, $lastUpd);
	}


	function getSetting($name, $default=null, $standard=false) {
		$lastUpd = $this->getModelExpired();
		if($this->init_time < $lastUpd)
			$this->init();
		if(is_array($name))
			$parts = $name;
		else
			$parts = explode('.', $name);
		$c = count($parts);
		if($c > 0) {
			$key = $parts[0];
			if($key == 'supported_types' && ! isset($this->local[$key])) {
				$this->loadTypes($lastUpd);
			}
			else if($key == 'app_defined_fields' && ! isset($this->local[$key])) {
				$this->loadAppFields($lastUpd);
			}
			else if(($key == 'detail' || $key == 'source' || $key == 'index') && ! isset($this->local[$key])) {
				$this->loadIndex($lastUpd);
			}
			else if($key == 'custom_field_defs' && ! isset($this->local[$key])) {
				$this->loadCustomFields($lastUpd);
			}
			else if(in_array($key, self::$model_keys) && $c > 1) {
				$name = $parts[1];
				if(empty($this->models_loaded[$name]))
					$this->loadModel($name, $lastUpd);
			}
		}
		return $this->findSubPath($parts, $this->local, $default);
	}
	
	
	function getModelExpired() {
		return AppConfig::get_expiry('model');
	}
	
	function invalidateCache($category=null, $limit=null) {
		// setting stored in external cache only
		AppConfig::set_expiry('model');
		@unlink(self::model_cache_file());
		$this->init();
		$GLOBALS['log']->debug('Model cache invalidated');
	}

	function setLocal($path, $val, $merge=false, $push_db=true)
	{
		if(is_array($path))
			$parts = $path;
		else
			$parts = explode('.', $path);
		$local =& $this->local;
		foreach($parts as $p) {
			if (!isset($local[$p])) $local[$p] = array();
			$local =& $local[$p];
		}
		$local = $val;
	}
}

?>
