<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */



require_once('include/config/ConfigBase.php');

class UserPrefManager extends ConfigBase {
	var $prefs;
	var $db_updates;
	
	static $save_all_users = false;
	
	function init() {
		parent::init();
		$this->prefs = array();
		$this->db_updates = array();
	}

	function getSetting($name, $default=null, $standard=false) {
		$lastUpd = $this->getPrefsExpired();
		if(is_array($name))
			$parts = $name;
		else
			$parts = explode('.', $name);
		$c = count($parts);
		if($c >= 1 && $parts[0] == 'current_user') {
			$cuid = AppConfig::current_user_id();
			array_splice($parts, 0, 1, array('by_user', $cuid));
			$c ++;
		}
		if($c >= 3 && ! AppConfig::in_installer()) {
			list($toplevel, $uid, $categ) = $parts;
			if($toplevel == 'by_user' && $uid && $categ) {
				if(! isset($this->prefs['by_user'][$uid][$categ]))
					$this->loadPrefs($uid, $categ, $lastUpd);
			}
		}
		$ret = $this->findSubPath($parts, $this->prefs, null);
		if(! isset($ret) && $c > 0 && $parts[0] == 'by_user') {
			$ret = $this->getSettingDefault(array_slice($parts, 2));
		}
		return isset($ret) ? $ret : $default;
	}
	
	function getSettingDefault($path, $default=null) {
		if(! isset($this->prefs['defaults']))
			self::merge_values($this->loadDefaultPrefs(), $this->prefs);
		$nul = null;
		$chk = $this->findSubPath($path, $this->prefs['defaults'], $nul, false);
		if(isset($chk)) {
			if(isset($chk['value']))
				return $chk['value'];
			else if(isset($chk['config']))
				return AppConfig::setting($chk['config']);
			return $chk;
		}
		return $default;
	}
	
	function getPrefsExpired() {
		return AppConfig::get_expiry('userpref');
	}
	
	function loadDefaultPrefs() {
		$paths = ConfigManager::$standard_config_dirs;
		$cache_key = "AppConfig.userpref.defaults";
		$mtime = $this->stat_disabled ? 0 : true;
		$vals = self::cache_load_files($cache_key, $paths, 'userpref.php', $mtime);
		return $vals;
	}
	
	function loadPrefs($uid, $category, $lastUpdate=0) {
		$cache_key = "AppConfig.userpref.by_user.$uid.$category";
		$prefs = AppConfig::cache_get_timed($cache_key, $lastUpdate, $found);
		if(! $found) {
			global $db;
			$table = AppConfig::setting('model.detail.user_preferences.table_name');
			$cat = $db->quote($category);
			$sql = "SELECT contents FROM `$table` WHERE assigned_user_id='$uid' AND category='$cat'";
			$result = $db->query($sql, false, 'Failed to load user preferences');
			if($result && ($row = $db->fetchByAssoc($result)))
				$prefs = unserialize(base64_decode($row['contents']));
			else
				$prefs = array();
			AppConfig::cache_put_timed($cache_key, $prefs);
		}
		
		if(is_demo_site() && $uid == AppConfig::current_user_id() && isset($_SESSION['DEMO_USER_PREFS'][$category])) {
			array_extend($prefs, $_SESSION['DEMO_USER_PREFS'][$category]);
		}
		
		$this->prefs['by_user'][$uid][$category] = $prefs;
	}
	
	function setLocal($path, $val, $merge=false, $push_db=true) {
		if(is_array($path))
			$parts = $path;
		else
			$parts = explode('.', $path);
		if($parts[0] == 'current_user')
			array_splice($parts, 0, 1, array('by_user', AppConfig::current_user_id()));
		if($parts[0] == 'by_user' && count($parts) == 4) {
			$this->setUserPreference($parts[3], $val, $parts[2], $parts[1]);
			// ignoring push_db currently, done in saveLocal
		}
	}
	
	function setUserPreference($name, $value, $category='global', $user_id=null) {
		if(AppConfig::in_installer())
			return;
		if(! isset($user_id)) $user_id = AppConfig::current_user_id();
		if(! $user_id) return;
		if(! isset($this->prefs['by_user'][$user_id][$category])) {
			$lastUpd = $this->getPrefsExpired();
			$this->loadPrefs($user_id, $category, $lastUpd);
		}
		$this->prefs['by_user'][$user_id][$category][$name] = $value;
		$this->db_updates[$user_id][$category] = 1;
	}
	
	function saveLocal($path=null, $push_db=true, $out_path=null) {
		if(! $push_db) return;
		$cuid = AppConfig::current_user_id();
		
		if(! isset($path))
			$path = array();
		if(! is_array($path))
			$path = explode('.', $path);
		if(count($path) && $path[0] == 'all_users') {
			$save = $this->db_updates;
			$this->db_updates = array();
		} else {
			if(count($path) == 2 && $path[0] == 'by_user')
				$uid = $path[1];
			else if(! count($path) || $path[0] == 'current_user') {
				$uid = $cuid;
				if(! $uid) return;
			}
			if(! isset($this->db_updates[$uid]))
				return;
			$save = array($uid => $this->db_updates[$uid]);
			unset($this->db_updates[$uid]);
		}

		if(is_demo_site() && ! AppConfig::in_installer() && empty($GLOBALS['OVERRIDE_SAVE_PREFS'])) {
			// persist preferences only in the current session for demo sites
			if(isset($save[$cuid])) {
				foreach($save[$cuid] as $category => $upd) {
					if($upd) $_SESSION['DEMO_USER_PREFS'][$category] = $this->getSetting("by_user.$cuid.$category");
				}
			}
			return;
		}
		
		global $db;
		$qparts = array();
		$now = gmdate('Y-m-d H:i:s');
		foreach($save as $uid => $updates) {
			$quid = $db->quote($uid);
			foreach($updates as $category => $upd) {
				if(! $upd) continue;
				$prefs = $this->getSetting("by_user.$uid.$category");
				$contents = base64_encode(serialize($prefs));
				$qparts[] = sprintf("('%s','%s','%s','$now')", $quid, $db->quote($category), $contents);
				AppConfig::cache_clear("AppConfig.userpref.by_user.$uid.$category");
			}
		}
		if($qparts) {
			$table = AppConfig::setting('model.detail.user_preferences.table_name');
			$sql = "INSERT INTO `$table` (assigned_user_id, category, contents, date_modified)"
				. ' VALUES '.implode(', ', $qparts)
				. " ON DUPLICATE KEY UPDATE contents=VALUES(contents),date_modified=VALUES(date_modified)";
			$db->query($sql, false, "Error updating user preferences");
		}
	}
	
	function invalidateCache($category=null, $limit = null) {
		// setting stored in external cache only
		AppConfig::set_expiry('userpref');
		$this->init();
		$GLOBALS['log']->debug('User preference cache invalidated');
	}
}

?>
