<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/config/format/ConfigCompile.php');


class ConfigCacheMgr {
	var $cache_dir;
	var $cache_mdate;
	var $cache_disabled = false;
	var $open_cache = [];
	
	function __construct($cache_dir) {
		$this->cache_dir = $cache_dir;
		$this->cache_mdate = max($this->mtime(__FILE__), $this->mtime('include/config/format/ConfigCompile.php'));
	}
	
	function mtime($path) {
		clearstatcache(false, $path);
		if(! is_readable($path)) return false;
		return @filemtime($path);
	}
	
	function getCacheEnabled() {
		return ! $this->cache_disabled;
	}
	
	function getCacheDirectory() {
		return $this->cache_dir;
	}
	
	function getCachePath($path) {
		if(! $path) return false;
		if( ($p = strrpos($path, '/')) !== false) {
			$path = sha1(substr($path, 0, $p)) . '_' . substr($path, $p+1);
		}
		$path = $this->cache_dir . '/' . $path;
		return $path;
	}
	
	function readCache(ConfigTarget $target, $path, $version=null) {
		if($this->cache_disabled) return false;
		$cpath = $this->getCachePath($path);
		$mtp = $this->mtime($path);
		$mtc = $this->mtime($cpath);
		if(! $mtp || ! $mtc || $mtp > $mtc || $this->cache_mdate > $mtc)
			return false;
		$_ = $target;
		$_->init($path, $version);
		include($cpath);
		$_->complete();
		return true;
	}
	
	function getCacheTarget($path, $options=null) {
		if($this->cache_disabled) return false;
		if(! is_array($options)) $options = [];
		$cpath = $this->getCachePath($path);
		if(! $cpath) return false;
		$tg = new ConfigCacheTarget($cpath);
		$tg->setCacheManager($this);
		if(! empty($options['on_demand']))
			$tg->on_demand = $options['on_demand'];
		return $tg;
	}

	function generateTempFileName($file) {
		$info = pathinfo($file);
		$tempName = $info['dirname'] . '/' . uniqid('');
		if (isset($info['extension'])) {
			$tempName .= '.' . $info['extension'];
		}
		return $tempName;
	}
	
	function openCache($path) {
		if($this->cache_disabled) return false;
		$cpath = $this->getCachePath($path);
		if(! $cpath) return false;
		$temp_path = $this->generateTempFileName($cpath);
		$fp = @fopen($temp_path, 'wb');
		if($fp)
			$this->open_cache[] = array('fp' => $fp, 'path' => $path, 'cache_path' => $cpath, 'temp_path' => $temp_path);
		return $fp;
	}
	
	function closeCache($fp) {
		if(! $fp) return;
		@fclose($fp);
		foreach($this->open_cache as $idx => $c) {
			if($c['fp'] == $fp) {
				if(function_exists('opcache_invalidate'))
					opcache_invalidate($c['cache_path']);
				@rename($c['temp_path'], $c['cache_path']);
				@unlink($c['temp_path']); // in case rename failed
				unset($this->open_cache[$idx]);
				break;
			}
		}
	}
	
	function clearCache($path) {
		$cpath = $this->getCachePath($path);
		if(! $cpath || ! file_exists($cpath)) return false;
		@unlink($cpath);
		if(function_exists('opcache_invalidate'))
			opcache_invalidate($cpath);
	}
}


class ConfigCacheTarget extends ConfigCompileTarget {
	var $cache_mgr;
	
	function setCacheManager(ConfigCacheMgr $mgr) {
		$this->cache_mgr = $mgr;
	}
	
	function complete() {
		if($this->cache_mgr) {
			$result = $this->getCompiled();
			$fp = $this->cache_mgr->openCache($this->filename);
			if($fp) {
				fwrite($fp, "<?php\n");
				if($this->filename)
					fwrite($fp, "// {$this->filename}\n");
				foreach($result as $r)
					fwrite($fp, "$r\n");
				fwrite($fp, "?>\n");
				$this->cache_mgr->closeCache($fp);
			}
		}
	}
}

