<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/config/format/ConfigParser.php');

class ConfigCompileTarget extends ConfigTarget {
	var $target;
	var $keys = [];
	var $stack = [];
	var $complex_stack = [];
	var $complex_keys = [];
	var $on_demand = false;
	var $demand_results = [];
	var $proxy;
	
	function __construct() {
		$this->target = new ConfigArrayTarget();
	}
	
	function isBlank() {
		if($this->proxy && ! $this->proxy->isBlank()) return false;
		return true;
	}
	function getTarget() {
		return $this->target;
	}
	function setTarget(ConfigTarget $t) {
		$this->target = $t;
	}
	function setTargetArray(&$arr) {
		if(! $this->target || ! method_exists($this->target, 'setTargetArray'))
			$this->target = new ConfigArrayTarget();
		$this->target->setTargetArray($arr);
	}
	
	function getInfo() {
		return $this->target->getInfo();
	}
	
	function setResultCallback(Callable $cb, $keys=null) {
		return $this->target->setResultCallback($cb, $keys);
	}
	function getResultKeys() {
		return $this->target->getResultKeys();
	}
	function setPreloadCallback(Callable $cb) {
		return $this->target->setPreloadCallback($cb, $keys);
	}
	
	function compile() {
		$args = func_get_args();
		$method = array_shift($args);
		if(count($args)) {
			$depth = $args[0];
			$params = [];
			foreach($args as $a) {
				$params[] = self::encode_value($a);
			}
			$params = implode(', ', $params);
		} else {
			$depth = 0;
			$params = '';
		}
		while(count($this->stack) <= $depth) $this->stack[] = [];
		$this->stack[count($this->stack)-1][] = "\$_->$method($params);";
	}
	
	function compileStr($str) {
		$this->stack[max(0, count($this->stack)-1)][] = $str;
	}
	
	static function encode_value(&$data) {
		$ret = '';
		if(is_array($data))
			self::encode_array($data, $ret);
		else if(is_object($data))
			$ret = null;
		else if($data === true)
			$ret = 'true';
		else if($data === false)
			$ret = 'false';
		else if($data === null)
			$ret = 'null';
		else
			$ret .= var_export($data, true);
		return $ret;
	}
	
	static function encode_array(array &$data, &$ret) {
		$ret .= '[';
		$i = 0;
		$unmatch = 0;
		foreach($data as $k => $v) {
			if($i) $ret .= ',';
			if($k !== $i || $unmatch) {
				$ret .= var_export($k, true) . '=>';
				$unmatch = 1;
			}
			if(is_array($v))
				self::encode_array($v, $ret);
			else
				$ret .= self::encode_value($v, true);
			$i ++;
		}
		$ret .= ']';
	}
	
	function setComplex($depth, $key=null) {
		for($i = 0; $i <= $depth; $i++)
			$this->complex_stack[$i] = true;
		if($this->keys) {
			$c =& $this->complex_keys;
			foreach($this->keys as $k) {
				if(! isset($c[$k])) $c[$k] = array();
				$c =& $c[$k];
			}
		}
	}
	
	function getComplex($depth) {
		if(! empty($this->complex_stack[$depth]))
			return true;
		if($this->on_demand && ! $depth)
			return false;
		if($depth && count($this->keys) >= $depth) {
			$c =& $this->complex_keys;
			foreach($this->keys as $i => $k) {
				if($i >= $depth || ! isset($c[$k])) return false;
				$c =& $c[$k];
			}
			return true;
		}
		return false;
	}
	
	function init($filename='', $version=null) {
		parent::init($filename, $version);
		$this->log('init', $filename);
		$this->proxy = new ConfigProxyTarget($this->target);
		$this->proxy->init($filename, $version);
		$this->stack = [];
		$this->complex_stack = [];
		if(! $this->on_demand && ! $this->target->isBlank())
			$this->setComplex(0);
	}
	
	function initArray($depth, $value=null, $key=null) {
		$this->log('initArray', $depth, $value, $key);
		$this->compile('initArray', $depth, $value, $key);
		$this->proxy->initArray($depth, $value, $key);
		if($depth) {
			for($i = 0; $i < $depth; $i++) {
				if(! isset($this->complex_stack[$i]))
					$this->complex_stack[$i] = false;
			}
			$this->complex_stack[$depth] = false;
		}
		if(count($this->keys) >= $depth-1) {
			$this->keys = array_slice($this->keys, 0, $depth);
			if(isset($key))
				$this->keys[$depth] = $key;
		}
	}
	
	function appendValue($depth, $value, $key=null) {
		$this->log('appendValue', $depth, $value, $key);
		$this->compile('appendValue', $depth, $value, $key);
		$this->proxy->appendValue($depth, $value, $key);
	}
		
	function appendResult($depth, $from_depth, $key=null) {
		$this->log('appendResult', $depth, $from_depth, $key);
		if(! $this->getComplex($from_depth)) {
			$result = $this->proxy->getResult($from_depth);
			$this->stack = array_slice($this->stack, 0, $from_depth);
			if(! isset($key))
				$this->compile('appendValue', $depth, $result);
			else {
				$this->compile('initArray', $from_depth, $result, $key);
				$this->compile('appendResult', $depth, $from_depth, $key);
				for($i = $from_depth; $i < count($this->stack); $i++) {
					foreach($this->stack[$i] as $s)
						$this->stack[$depth][] = $s;
					$this->complex_stack[$i] = false;
				}
			}
			$this->stack = array_slice($this->stack, 0, $depth+1);
		} else {
			$this->compile('appendResult', $depth, $from_depth, $key);
			for($i = $from_depth; $i < count($this->stack); $i++) {
				foreach($this->stack[$i] as $s)
					$this->stack[$depth][] = $s;
				$this->complex_stack[$i] = false;
			}
			$this->stack = array_slice($this->stack, 0, $depth+1);
			if(! isset($this->complex_stack[$depth]))
				$this->complex_stack[$depth] = false;
		}
		$this->proxy->appendResult($depth, $from_depth, $key);
		if($this->on_demand && ! $depth && isset($key)) {
			$this->demand_results[$key] = $this->flattenStack();
			$this->stack = [];
		}
	}
	
	function appendInfo($from_depth=1) {
		$this->log('appendInfo', $from_depth);
		$this->compile('appendInfo', $from_depth);
		$this->stack[$from_depth] = $this->flattenStack($from_depth);
		$this->stack = array_slice($this->stack, 0, $from_depth);
		$this->proxy->appendInfo($from_depth);
	}
	
	function appendOrigValue($depth, $key) {
		$this->log('appendOrigValue', $depth, $key);
		$this->setComplex($depth);
		$this->compile('appendOrigValue', $depth, $key);
		$this->proxy->appendOrigValue($depth, $key);
	}
	
	function removeValue($depth, $key) {
		$this->log('removeValue', $depth, $key);
		$this->setComplex($depth);
		$this->compile('removeValue', $depth, $key);
		$this->proxy->removeValue($depth, $key);
	}
	
	function getResult($depth=0) {
		$this->log('getResult', $depth);
		return $this->target->getResult($depth);
	}
	
	function getCompiled() {
		$this->simplify();
		if($this->on_demand && ($keys = array_keys($this->demand_results))) {
			$ret = [];
			$ret[] = "\$_->setResultCallback(function(\$_, \$key) {";
			$ret[] = 'switch($key) {';
			foreach($keys as $k) {
				$v = self::encode_value($k);
				$ret[] = "case $v: ";
				$ret[] = implode("\n", $this->demand_results[$k]).' break;';
			}
			$ret[] = '}}, '.self::encode_value($keys).');';
		} else {
			$ret = $this->flattenStack();
		}
		$info = $this->getInfo();
		if($info) {
			$val = self::encode_value($info);
			array_unshift($ret, "\$_->appendInfo(1);");
			array_unshift($ret, "\$_->initArray(1, $val);");
		}
		if($ret) {
			$val = self::encode_value($this->version);
			array_unshift($ret, "\$_->checkVersion($val);");
			array_unshift($ret, 'if(! isset($_)) return;');
		}
		return $ret;
	}
	
	function simplify($depth=0) {
		for($d = $depth; $d < count($this->stack); $d++) {
			if($this->getComplex($d)) continue;
			$result = $this->proxy->getResult($d);
			$this->stack = array_slice($this->stack, 0, $d);
			if(count($result))
				$this->compile('initArray', $d, $result);
			break;
		}
	}
	
	function flattenStack($from=0) {
		$ret = [];
		foreach($this->stack as $idx => $stack) {
			if($idx < $from) continue;
			foreach($stack as $s)
				$ret[] = $s;
		}
		return $ret;
	}
	
	function clearResult($depth=0) {
		$this->log('clearResult', $depth);
		$this->setComplex($depth);
		$this->simplify($depth);
		$this->compile('clearResult', $depth);
		$this->proxy->clearResult($depth);
	}
	
	function reset() {
		parent::reset();
		$this->proxy->reset();
		$this->stack = [];
	}
	
	function complete() {
		$this->log('complete');
		$this->proxy->complete();
		// flatten stack, clear context
	}
}


// Allow compiler to collect its own results while
// adding to an accumulated result target as well.
// Only methods that modify the target result are overridden.
class ConfigProxyTarget extends ConfigArrayTarget {
	var $target;

	function __construct($proxy=null) {
		$this->target = $proxy ? $proxy : new ConfigArrayTarget();
	}	
	function getTarget() {
		return $this->target;
	}
	function setTarget(ConfigTarget $t) {
		$this->target = $t;
	}
	function setCustomArray(&$t) {
		$this->target->setCustomArray($t);
	}
	
	function init($filename='', $version=null) {
		parent::init($filename, $version);
		$this->target->init($filename, $version);
	}
	
	function initArray($depth, $value=null, $key=null) {
		parent::initArray($depth, $value, $key);
		$this->target->initArray($depth, $value, $key);		
	}
	
	function appendValue($depth, $value, $key=null) {
		parent::appendValue($depth, $value, $key);
		$this->target->appendValue($depth, $value, $key);
	}
	
	function appendResult($depth, $from_depth, $key=null) {
		parent::appendResult($depth, $from_depth, $key);
		$this->target->appendResult($depth, $from_depth, $key);

	}
	
	function appendOrigValue($depth, $key) {
		parent::appendOrigValue($depth, $key);
		$this->target->appendOrigValue($depth, $key);
	}
	
	function appendInfo($from_depth=1) {
		parent::appendInfo($from_depth);
		$this->target->appendInfo($from_depth);
	}
	
	function removeValue($depth, $key) {
		parent::removeValue($depth, $key);
		$this->target->removeValue($depth, $key);
	}

	function clearResult($depth=0) {
		parent::clearResult($depth);
		$this->target->clearResult($depth);
	}
	
	function reset() {
		parent::reset();
		$this->target->reset();
	}
}

