<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


if(! class_exists('ConfigParser'))
require_once('include/config/format/ConfigParser.php');

class ConfigWriter {
	var $tempFilename;
	var $filename;
	var $fp;
	var $origData;
	var $result;
	var $appendOnly = false;
	
	
	function writeFile($file, $data, $info=null) {
		return $this->writeDiff($file, $data, array(), $info);
	}
	
	function writeFilePointer($fp, $data, $info=null) {
		return $this->writeFilePointerDiff($fp, $data, array(), $info);
	}

	function generateTempFileName($file) {
		$info = pathinfo($file);
		$tempName = $info['dirname'] . '/' . uniqid('');
		if (isset($info['extension'])) {
			$tempName .= '.' . $info['extension'];
		}
		return $tempName;
	}

	function writeDiff($file, $data, $orig, $info=null) {
		if(! is_array($data))
			throw new IAHConfigFileError("Expected array parameter");
		$this->checkDir($file);
		$tempFilename = $this->generateTempFileName($file);
		$fp = @fopen($tempFilename, 'wb');
		if(! $fp)
			throw new IAHConfigFileError("Cannot open file for writing: $tempFilename");
		$this->tempFilename = $tempFilename;
		$this->filename = $file;
		if(preg_match('~.php$~i', $file))
			fwrite($fp, '<?php return; /* no output */ ?'.">\n\n");
		try {
			$this->writeFilePointerDiff($fp, $data, $orig, $info);
		} catch(Exception $e) {
			$this->closeFile(false);
			throw $e;
		}
		$this->closeFile(true);
		return true;
	}

	function checkDir($fileName)
	{
		$dirName = dirname($fileName);
		$parts = explode('/', $dirName);
		$path = '.';
        if (isset($parts[0]) && strpos($parts[0], ':') !== false)
            $path = '';
		foreach ($parts as $part) {
            if (strpos($part, ':') !== false)
                continue;
            $path .= '/' . $part;
			if (!is_dir($path)) {
				if (!@mkdir($path)) {
					throw new IAHConfigFileError("Cannot create path: " . $path);
				}
			}
		}
	}

	function writeFilePointerDiff($fp, $data, $orig, $info=null) {
		if(! $fp)
			throw new IAHConfigFileError("Invalid file pointer provided");
		if(! is_array($orig))
			$orig = array();
		$this->fp = $fp;
		$this->writeInfo($info);
		$this->writeArray($data, $orig);
		return true;
	}
	
	function encode($data, $orig=null, $info=null, $string=false) {
		if(! is_array($data))
			throw new IAHConfigFileError("Expected array parameter");
		if(! is_array($orig))
			$orig = array();
		$this->fp = null;
		$this->result = array();
		$this->writeInfo($info);
		$this->writeArray($data, $orig);
		if($string)
			return implode("\n", $this->result);
		return $this->result;
	}
	
	function encodeToString($data, $orig=null, $info=null) {
		return $this->encode($data, $orig, $info, true);
	}
	
	function setAppendOnly($flag=true) {
		$this->appendOnly = $flag;
	}
	
	function closeFile($finalize) {
		static $is_windows;
		if (!isset($is_windows))
			$is_windows = substr(strtoupper(PHP_OS), 0, 3) === 'WIN';

		try {
			if($this->fp) {
				fclose($this->fp);
				$this->fp = null;
				if ($this->filename && $this->tempFilename) {
					$success = @rename($this->tempFilename, $this->filename);
					if ($is_windows && !$success) {
						$contents = file_get_contents($this->tempFilename);
						$f = @fopen($this->filename, 'r+b');
						if ($f !== false) {
							if (@flock($f, LOCK_EX) !== false) {
								if (@ftruncate($f, 0)) {
									if (@fwrite($f, $contents) !== false) {
										$success = true;
									}
								}
								@fflush($f);
								@flock($f, LOCK_UN);
							}
							@fclose($f);
						}
					}
					if (!$success) {
						throw new IAHConfigFileError("Cannot rename config file to {$this->filename}");
					}
				}
			}
		} finally {
			if ($this->tempFilename && file_exists($this->tempFilename)) {
				@unlink($this->tempFilename);
			}
			$this->tempFilename = null;
			$this->filename = null;
		}
	}
	
	function writeLine($line, $depth) {
		$ind = str_repeat("\t", $depth);
		if($this->fp) {
			fwrite($this->fp, $ind);
			fwrite($this->fp, $line);
			fwrite($this->fp, "\n");
		}
		else {
			$this->result[] = $ind . $line;
		}
	}
	
	function writeInfo(&$info) {
		if(! is_array($info) || ! count($info))
			return;
		$this->writeLine('@info', 0);
		$old=array();
		$this->writeArray($info, $old, 1);
	}
	
	static function arrays_differ(&$data, &$orig, $append_only=false) {
		if($data === $orig)
			return 0; // no difference: checks keys, values, and ordering of keys
		if(! count($orig) || $append_only)
			return 1; // keys only added (use merge)
		if(array_key_exists(0, $data))
			return 3; // has numeric keys (use @clear)
		$diff = array_intersect_key($orig, $data);
		if(! $diff)
			return 3; // array cleared, or no common keys (use @clear)
		$kd = array_keys($data); $i = 0;
		foreach($diff as $k => &$v)
			if($kd[$i ++] !== $k)
				return 2; // keys have been reordered, or new ones inserted (use @reorder)
		return 1; // use merge
	}
	
	function writeArray(&$data, &$orig, $depth=0) {
		$append_only = ($depth == 0 ? $this->appendOnly : false);
		$diff = self::arrays_differ($data, $orig, $append_only);
		if(! $diff)
			return;
		if($diff == 3)
			$this->writeLine('@clear', $depth);
		else if($diff == 2)
			$this->writeLine('@clear', $depth);  // @reorder
		if($diff == 1 && ! $append_only) {
			foreach(array_keys($orig) as $k)
				if(! array_key_exists($k, $data))
					$this->writeLine('@remove '.$this->encodeKey($k), $depth);
		}
		$ind = 0;
		$all_int = true;
		foreach($data as $k => &$v) {
			$exact_match = array_key_exists($k, $orig) && $orig[$k] === $v;
			if($diff == 1 && $exact_match)
				continue;
			$is_arr = is_array($v);
			if(is_int($k) && $all_int && $k == $ind && $diff != 2)
				$key = $is_arr ? '--' : '-';
			else {
				$all_int = false;
				$key = $this->encodeKey($k);
				if($diff == 2 && $exact_match) {
					$this->writeLine("@orig $key", $depth);
					continue;
				}
				if(! $is_arr)
					$key .= ':';
			}
			if($is_arr) {
				$this->writeLine($key, $depth);
				if(isset($orig[$key])) {
					$o =& $orig[$key];
				} else {
					unset($o);
					$o = array();
				}
				$this->writeArray($v, $o, $depth + 1);
			}
			else {
				$val = $this->encodeString($v);
				$is_arr = is_array($val);
				if($is_arr) {
					$this->writeLine($key, $depth);
					foreach($val as $v2)
						$this->writeLine($v2, $depth + 1);
				}
				else {
					$this->writeLine($key . ' ' . $val, $depth);
				}
			}
			$ind ++;
		}
	}
	
	function encodeKey($val) {
		return $this->encodeString($val, true);
	}

	function encodeString($val, $as_key=false) {
		static $replace_nl = array(array('\\', "\t", "\r"), array('\\\\', '\\t', '\\r'));
		if($val === null)
			return 'null';
		else if($val === true)
			return 'true';
		else if($val === false)
			return 'false';
		if(! is_string($val)) {
			if(is_numeric($val))
				return strval($val);
			else // is_resource || is_object
				throw new IAHConfigFileError("Cannot encode object");
		}
		if($val === '')
			return '""';
		$c = $val[0];
		$quote = false;
		$have_nl = (strpos($val, "\n") !== false);
		$special = array_key_exists($val, ConfigParser::$special_values);
		$tdiff = (trim($val) !== $val);
		$cslash = addcslashes($val, "\0\n\r\t\\\"");

		if(! $as_key && ($have_nl || strlen($val) > 80)) {
			$val = str_replace($replace_nl[0], $replace_nl[1], $val);
			if($tdiff || $c == '"')
				return '"""' . $val . '"""';
			return explode("\n", $val);
		}
		if($c == '"' || $c == '-' || $c == '[' || $c == '@' || $special
			|| $cslash != $val
			|| strpos($val, '#') !== false
			|| strpos($val, '"') !== false
			|| ($as_key && $have_nl)
			|| $tdiff)
		{
				return '"' . $cslash . '"';
		}
		if(is_numeric($val)) {
			return '"' . $val . '"';
		}
		return $val;
	}

}

?>
