<?php

define('DB_CHECK_COLUMNS', 1);
define('DB_CHECK_INDICES', 2);
define('DB_CHECK_AUDIT', 4);
define('DB_CHECK_RUN_CHECK', 8);
define('DB_CHECK_RUN_REPAIR', 16);

define('DB_CHECK_STANDARD', 7); // columns, indices, audit
define('DB_CHECK_EXTENDED', 31); // + check/repair

class IAHDBCheckerError extends IAHDatabaseError {}

class DBChecker {
	var $status;
	var $limit_tables;
	var $limit_models;
	var $verify_table_types = array(
		'bean' => 1, 'link' => 1, 'table' => 1, 'custom' => 1,
	);
	
	function __construct() {
	}
	
	function reloadModels() {
		AppConfig::invalidate_cache('model');
	}
	
	function setStatusLogger(&$logger) {
		$this->status =& $logger;
	}
	
	function checkRepairModel($model_name, $mode=DB_CHECK_STANDARD, $execute=true, $output=false) {
		global $db;
		
		if(! $model_name || (isset($this->limit_models) && ! in_array($model_name, $this->limit_models)))
			return null;
		$detail = AppConfig::setting("model.detail.$model_name");
		if(! $detail)
			return null;
		$tbl = array_get_default($detail, 'table_name');
		$source = array_get_default($detail, 'module_dir', '');
		if($source)
			$source .= '/' . array_get_default($detail, 'bean_name', '');
		if(! $tbl || (isset($this->limit_tables) && ! in_array($tbl, $this->limit_tables)))
			return null;
		$v_indices = $mode & DB_CHECK_INDICES;
		$ret = array();
		
		if($mode & DB_CHECK_COLUMNS) {
			$fields = AppConfig::setting("model.fields.$model_name");
			$indices = AppConfig::setting("model.indices.$model_name");
			
			try {
				$sql = $db->repairTableParams($tbl, $fields, $indices, $execute, $source, $v_indices);
			} catch(IAHDatabaseError $e) {
				throw new IAHDBCheckerError($e->getMessage());
			}
			if($sql) {
				if($output === 'text')
					echo "$sql\n";
				else if($output)
					echo '<pre>' . htmlspecialchars($sql) . '</pre>';
				$ret[] = $sql;
			}
		}
		
		if (($mode & DB_CHECK_AUDIT) && array_get_default($detail, 'audit_enabled')) {
			$audit_table_name = $detail['audit_table_name'];
			$fields = AppConfig::setting("model.fields.audit");
			$indices = AppConfig::setting("model.indices.audit");
			try {
				$sql = $db->repairTableParams($audit_table_name, $fields, $indices, $execute, '', $v_indices);
			} catch(IAHDatabaseError $e) {
				throw new IAHDBCheckerError($e->getMessage());
			}
			if($sql) {
				if($output === 'text')
					echo "$sql\n";
				else if($output)
					echo '<pre>' . htmlspecialchars($sql) . '</pre>';
				$ret[] = $sql;
			}
		}
		
		if($mode & DB_CHECK_RUN_CHECK) {
			// extended check / extended repair?
			$result = $db->query("CHECK TABLE `$tbl`", false);
			if($result && ($row = $db->fetchByAssoc($result))) {
				if($row['Msg_text'] != 'OK') {
					$sql = "REPAIR TABLE `$tbl`;";
					if(! ($mode & DB_CHECK_RUN_REPAIR) || ! $execute)
						$sql = "/* Table $tbl appears crashed, repair recommended: */\n$sql";
					else {
						$result = $db->query($sql, false);
						if($result && ($row = $db->fetchByAssoc($result))) {
							$sql .= "\n/* Result: {$row['Msg_text']} */";
						} else
							$sql .= "\n/* Error performing repair */";
						$sql = "/* Table $tbl appears crashed, performing repair: */\n$sql";
					}
					if($output === 'text')
						echo "$sql\n";
					else if($output)
						echo '<pre>' . htmlspecialchars($sql) . '</pre>';
					$ret[] = $sql;
				}
			}
		}
		
		return $ret;
	}

	function checkTables($actions, $output=false) {
		$mode = $columns = 0;
		$execute = ! empty($actions['execute']);
		
		if(! empty($actions['columns'])) {
			$mode |= DB_CHECK_COLUMNS;
			if(! empty($actions['indices']))
				$mode |= DB_CHECK_INDICES;
		}
		if(! empty($actions['audit']))
			$mode |= DB_CHECK_AUDIT;
		if(! empty($actions['check']))
			$mode |= DB_CHECK_RUN_CHECK;
		if(! empty($actions['repair']))
			$mode |= DB_CHECK_RUN_REPAIR;
		
		$all_tables = AppConfig::setting('model.index.by_type');
		$verify_tables = array();
		foreach($this->verify_table_types as $t => $v) {
			if(! $v) continue;
			if(isset($all_tables[$t]))
				$verify_tables = array_merge($verify_tables, $all_tables[$t]);
		}
		
		$ret = array();
		
		foreach($verify_tables as $model_name) {
			$cmds = $this->checkRepairModel($model_name, $mode, $execute, $output);
			if($cmds)
				$ret = array_merge($ret, $cmds);
		}
		
		return $ret;
	}
	
}

?>
