<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
********************************************************************************/
/*********************************************************************************

* Description: This file is an abstract class and handles the Data base functionality for
* the application. It is called by the DBManager class to generate various sql statements.
*
* All the functions in this class will work with any bean which implements the meta interface.
* Please refer the DBManager documentation for the details.
*
* Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
* All Rights Reserved.
* Contributor(s): ______________________________________..
********************************************************************************/


class DBHelper
{
	var $dbType = '';


	function createTableSQLParams($tablename, $fieldDefs, $indices) {
		$columns = $this->columnSQLRep($fieldDefs, $tablename);
        $keys = $this->keysSQL($indices);
        if (!empty($keys)) $keys = ",$keys";
 		if(empty($columns)){
 			return false;
 		}
		$sql = "CREATE TABLE $tablename ($columns $keys)";
 		return $sql;
	}

    function keysSQL( $indices, $alter_table = false, $alter_action = '') {
		return [];
	}

	/**
	* This method genrates sql for insert statement.
	* Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
	* All Rights Reserved..
	* Contributor(s): ______________________________________..
	*/
	function insertRowSql(ModelDef &$model, array $row, $insert_mode='insert', $add_default_required=false) {
		global $db;
		// get basic insert
		$sql = ($insert_mode === 'update' ? 'UPDATE ' : ($add_default_required ? 'INSERT IGNORE INTO ' : 'INSERT INTO ')).$model->getTableName();
		
		$pk = $model->getPrimaryKey(true);

		// get column names and values
		$keys = $values = $where = array();
		foreach($row as $field => $value) {
			$fieldDef = $model->getFieldDefinition($field);
			if(! $fieldDef)
				continue;
			$q_key = $db->quoteField($fieldDef['name']);
			$q_value = $this->massageValue($value, $fieldDef);
			if($insert_mode === 'update' && in_array($field, $pk)) {
				$where[] = "$q_key=$q_value";
			} else {
				if($fieldDef['type'] == 'multienum' && isset($fieldDef['multi_select_group'])) {
					$me_count = $fieldDef['multi_select_count'];
					$me_base = $fieldDef['multi_select_group'];
					$vals = array_filter(array_map('trim', explode('^,^', $value)));
					for($c = ''; $c < $me_count; $c++) {
						$me_fn = $me_base . $c;
						$keys[$me_fn] = $db->quoteField($me_fn);
						$me_val = array_get_default($vals, $c ? $c-1 : 0);
						$values[$me_fn] = $this->massageValue($me_val, $fieldDef);
						if(! $c) $c++;
					}
				} else {
					$keys[$fieldDef['name']] = $q_key;
					$values[$fieldDef['name']] = $q_value;
				}
			}
		}
		if(! count($values)) return ""; // no columns set
		
		if($insert_mode == 'insert') {
			$sql .= " (".implode(",", $keys).") VALUES (".implode(",", $values).")";
		}
		else {
			$sql .= ' SET ';
			$comma = '';
			foreach($keys as $i => $k) {
				$sql .= "{$comma}$k={$values[$i]}";
				$comma = ', ';
			}
		}
		
		if($insert_mode === 'put') {
			foreach($pk as $k)
				unset($keys[$k]);
			$v = array();
			if($keys) {
				foreach($keys as $k => $q_key)
					$v[] = "$q_key=VALUES($q_key)";
				$sql .= ' ON DUPLICATE KEY UPDATE '.implode(', ', $v);
			}
		}
		else if($insert_mode === 'update') {
			if(! $where)
				return ""; // must have a filter
			$sql .= ' WHERE '.implode(' AND ', $where);
		}
		
		return $sql;
	}
	
	function updateRowSql(ModelDef &$model, array $row) {
		return $this->insertRowSql($model, $row, 'update');
	}
	
	function deleteRowSql(ModelDef &$model, array $row) {
		global $db;
		$sql = 'DELETE FROM '.$model->getTableName().' WHERE ';		
		$pk = $model->getPrimaryKey(true);

		// get column names and values
		$keys = $values = $where = array();
		foreach($pk as $pkf) {
			$fieldDef = $model->getFieldDefinition($pkf);
			if(! $fieldDef)
				return "";
			$q_key = $db->quoteField($fieldDef['name']);
			$value = array_get_default($row, $pkf);
			if(! isset($value))
				return "";
			$q_value = $this->massageValue($value, $fieldDef);
			$where[] = "$q_key=$q_value";
		}
		if(! count($where)) return ""; // no columns set
		$sql .= implode(' AND ', $where);		
		return $sql;
	}
	

    /** This function just returns a where clause without the 'where' key word
     * The clause returned does not have an 'and' at the beginning and the columns
     * are joined by 'and'.
     */
    function getColumnWhereClause($table, $whereArray=array()) {

       foreach ($whereArray as $name => $val)
       {
          $op = "=";
          if (is_array($val)){
            $op = "IN";
            $temp = array();
            foreach ($val as $tval){
                $temp[] = "'$tval'";
            }

            $val = implode(",", $temp);
            $val = "($val)";
          }
          else $val = "'$val'";
          $where[] = " $table.$name $op $val";
       }
       if (is_array($where)) $where = implode(" and ", $where);
       return $where;
    }


    // returns the type of the variable in the field
    function getFieldType($fieldDef)
    {
        // get the type for db type. if that is not set,
        // get it from type. This is done so that
        // we do not have change a lot of existing code
        // and add dbtype where type is being used for some special
        // purposes like referring to foreign table etc.
        if(!empty($fieldDef['dbType']))
       	 	return  $fieldDef['dbType'];
        if (!empty($fieldDef['type']))
        	return  $fieldDef['type'];
        return null;
    }

	/** private function to get sql for a column
	*/
	function oneColumnSQLRep($fieldDef,  $ignoreRequired = false, $table=''){
		global $db;

		$name = $fieldDef['name'];

        $type = $this->getFieldType($fieldDef);

		$colType = $this->getColumnType($type);
		if(! $colType)
			return '';
		
		$charset = '';
		$colChar = array_get_default($fieldDef, 'charset');
        if($colChar == 'ascii' && ($colType == 'char' || $colType == 'varchar' || $colType == 'text')) {
        	$charset = ' CHARACTER SET ascii COLLATE ascii_bin';
        }
        if($colChar == 'utf8mb4' && $db->getConnectionCharset() == 'utf8mb4') {
        	$charset = ' CHARACTER SET utf8mb4 COLLATE utf8mb4_unicode_ci';
        }
        $auto_increment = '';
        if(!empty($fieldDef['auto_increment'])){
        	$auto_increment = ' '.$this->setAutoIncrement($table , $fieldDef['name']);
        }
		
        if (( $colType == 'varchar' or $colType == 'char') ) {
			if($fieldDef['type'] == 'enum')
				$len = AppConfig::setting('site.db_defaults.enum_field_size', 255);
			else if($fieldDef['type'] == 'multienum' || $fieldDef['type'] == 'status')
				$len = 255;
			else if(! empty($fieldDef['len']))
				$len = $fieldDef['len'];
			else if($type == 'id')
				$len = 36;
			else
				$len = 255;
			$colType .= "($len)";
        }
        if($colType == 'int'){
        	//only do below if db type is mysql.  Setting a length on the int will fail for oracle or sql server.
        	if($this->dbType == 'mysql'){
        		if( !empty($fieldDef['len']) && $fieldDef['len'] < 20)
            		$colType .= "(".$fieldDef['len'].")";
        	}
			if (isset($fieldDef['default']) && empty($fieldDef['default']) && empty($auto_increment))
				$fieldDef['default'] = 0;
        }
        if($colType == 'decimal' || $colType == 'float'){
        	if(!empty($fieldDef	['len'])){
        		if(substr_count($fieldDef['len'], ',') == 1){
            		$colType .= "(".$fieldDef['len'].")";
        		}
        	}
			if (isset($fieldDef['default']) && empty($fieldDef['default']) && empty($auto_increment))
				$fieldDef['default'] = 0;
        }
		
		if($colType == 'double'){
			if (isset($fieldDef['default']) && empty($fieldDef['default']) && empty($auto_increment))
				$fieldDef['default'] = 0;
        }

		if (isset($fieldDef['default']) && $type != 'date' && $type != 'time' && $type != 'datetime' ){
            $default = " DEFAULT '" . $db->quote($fieldDef['default'])."'";
		}else if(!isset($default) && $type == 'bool'){
        	$default = " DEFAULT 0";
        }else if(!isset($default)){
        	$default = '';
        }
                
        // longreach - added - defaults are not supported for blobs
        if($type == 'text' || $type == 'blob') $default = '';


        $required = ' NULL';  // MySQL defaults to NULL, SQL Server defaults to NOT NULL -- must specify
        if (!$ignoreRequired && ! empty($fieldDef['required']) && empty($fieldDef['isnull'])){
			 $required =  " NOT NULL";
        }
        if($name == 'id' && ! isset($fieldDef['required'])){
        	$required =  " NOT NULL";
        }


		$rep = $db->quoteField($name) . " $colType{$charset}{$default}{$required}{$auto_increment}";
        return $rep;
	}

	/** private function to get sql for a column
	*/
	function columnSQLRep($fieldDefs, $tablename){
		if ($this->isFieldArray($fieldDefs)) {
		  $columns = array();
		  foreach ($fieldDefs as $fieldDef) {
		  	if($fieldDef['source']['type'] == 'db') {
		  		$rep = $this->oneColumnSQLRep($fieldDef, false, $tablename);
		  		if($rep)
		  			$columns[] = $rep;
		  	}
		  }
		  $columns = implode(",", $columns);
		}
		else $columns = $this->oneColumnSQLRep($fieldDefs);

		return $columns;
	}

	//returns the next value for an auto increment
	function getAutoIncrement($table, $field_name){
		return "";
	}
	//either creates an auto increment through queries or returns sql for auto increment that can be appended to the end of column defination (mysql)
	function setAutoIncrement($table, $field_name){
		$this->deleteAutoIncrement($table, $field_name);
		return "";
	}
	//deletes an auto incremnet (for oracle not mysql)
	function deleteAutoIncrement($table, $field_name){

	}

	/**
	* A private function which generates the SQL for changing columns
	*/
	function changeColumnSQL($tablename, $fieldDefs, $action, $ignoreRequired = false){

		$columns = array();
        if ($this->isFieldArray($fieldDefs)){
          foreach ($fieldDefs as $def) {
          	$rep = $this->oneColumnSQLRep($def, $ignoreRequired, $tablename);
          	if($rep)
				$columns[] = $rep;
		  }
        } else {
          	$columns[] = $this->oneColumnSQLRep($fieldDefs, $ignoreRequired, $tablename);
        }

        $columns = implode(", column ", $columns);
		$sql = "alter table $tablename $action ($columns)";
		return $sql;
	}

	/**
	* This method generates sql for adding a column to table identified by field def.
	* Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
	* All Rights Reserved.
	* Contributor(s): ______________________________________..
	*/
	function addColumnSQL($tablename, $fieldDefs){
       return $this->changeColumnSQL($tablename, $fieldDefs, 'add');
	}

	/**
	* This method genrates sql for altering old column identified by oldFieldDef to new fieldDef.
	* Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
	* All Rights Reserved.
	* Contributor(s): ______________________________________..
	*/
	function alterColumnSQL($tablename, $newFieldDefs, $ignoreRequired=false){
       return $this->changeColumnSQL($tablename, $newFieldDefs, 'modify', $ignoreRequired);
	}

	function dropTableNameSQL($name){
		return "DROP TABLE IF EXISTS ".$name;
	}

	/**
	* This method generates sql that deletes a column identified by fieldDef.
	* Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
	* All Rights Reserved.
	* Contributor(s): ______________________________________..
	*/
	function deleteColumnSQL($table_name, $fieldDefs) {
        if ($this->isFieldArray($fieldDefs)) foreach ($fieldDefs as $fieldDef) $columns[] = $fieldDef['name'];
        else $columns[] = $fieldDefs['name'];
		$columns = implode(", ", $columns);
		$sql = "ALTER TABLE $table_name DROP ($columns)";
        return $sql;
	}

	/**
	* This is a private (php does not support it as of 4.x) method.
	* It outputs a correct string for the sql statement according to value
	* This will be overwritten in derived classes.
	* Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
	* All Rights Reserved..
	* Contributor(s): ______________________________________..
	*/
	function massageValue($val, $type){
		return $val;
	}


	/** returns the valid type for a column given the type in fieldDef
	*/

	function getColumnType($type){
       return $type;
	}

    /**
     * function to see if passed array is truely an aray of defitions
     * Such an array may have type as a key but it will point to an array
     * for a true array of definitions an to a col type for a definition only
     */
    function isFieldArray($defArray)
    {
        if (!is_array($defArray)) return false;

        if (array_key_exists('type', $defArray))
        {
          // type key exists. May be an array of defs or a simple definition
          $type = $defArray['type'];
          return is_array($type) ? TRUE : FALSE; // type is not an array => definition else array
        }
        // type does not exist. Must be array of definitions
        return TRUE;
    }

    // returns true if the type can be mapped to a valid column tyoe
    function validColumnType($type){
        $coltype = $this->getColumnType($type);
        return ($coltype) ? TRUE : FALSE;
    }


    function concatExprs($exprs, $separator=null) {
    	if(! is_countable($exprs) || ! count($exprs))
    		return '""';
		if(isset($separator))
			$fn = "CONCAT_WS('$separator',";
		else
			$fn = 'CONCAT(';
		return $fn . implode(',', $exprs) . ')';
    }

}
?>
