<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
********************************************************************************/
/*********************************************************************************

* Description: This file generates the appropriate manager for the database
* 
* Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
* All Rights Reserved.
* Contributor(s): ______________________________________..
********************************************************************************/

class DBManagerFactory
{
	static $managers;
	static $helpers;
	
	/** This function returns the correct instance of the manager
	*   depending on the database type
	*/
	static function &getInstance($instanceName=null) {
		$temp_var =& DBManager::getInstance($instanceName);
		return $temp_var;
	}
	
	static function &getManager($instanceName=null, $no_create=false) {
		if(! $instanceName)
			$instanceName = 'primary';
		
		if(isset(self::$managers[$instanceName]))
			return self::$managers[$instanceName];
		if($no_create) {
			$n = null;
			return $n;
		}
		
		$spec = AppConfig::setting("database.$instanceName");
		if(! $spec)
			throw new IAHError("Missing database credentials: $instanceName");
		$mgr = self::getManagerFromSpec($spec);
		
		self::$managers[$instanceName] = new $mgr;
		self::$managers[$instanceName]->connect($spec);
		return self::$managers[$instanceName];
	}
	
	static function &getManagerFromSpec(array $spec) {
		$mgr = array_get_default($spec, 'manager_class');
		$type = array_get_default($spec, 'type', 'mysql');

		if(! $mgr) {
			if($type == 'mysqli')
				$mgr = 'MysqliManager';
			else if($type == 'mysql') {
				if(extension_loaded('mysqli') && ! AppConfig::setting('site.features.mysqli_disabled') && empty($spec['force_type']))
					$mgr = 'MysqliManager';
				else
					$mgr = 'MysqlManager';
			} else
				throw new IAHDatabaseError("Unknown database type: $type");
		}
		
		if(file_exists("include/database/$mgr.php"))
			require_once("include/database/$mgr.php");
		
		return $mgr;
	}
	
	static function disconnectInstance($instanceName) {
		if(! empty(self::$managers[$instanceName])) {
			self::$managers[$instanceName]->disconnect();
			unset(self::$managers[$instanceName]);
			return true;
		}
	}

	static function disconnectAll() {
		if(! empty(self::$managers))
			foreach(self::$managers as &$d)
				$d->disconnect();
	}
}

?>
