<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

class ListQueryIterator implements Iterator, Countable {
	private $query;
	private $result;
	private $pageSize;
	private $index;
	private $count;
	private $prev_count;
	
	function __construct(ListQuery $lq, $pageSize=300) {
		$this->query = $lq;
		$this->pageSize = $pageSize;
	}
	
	function getResult() {
		return $this->result;
	}
	
	function rewind() : void {
		$this->result = $this->query->runQuery(0, null, null, null, $this->pageSize, false);
		$this->count = $this->result->getResultCount();
		$this->index = ($this->result->failed || !$this->count) ? null : 0;
		$this->prev_count = 0;
	}

	#[\ReturnTypeWillChange]
	function current() {
		return $this->result->getRowResult($this->index);
	}

	#[\ReturnTypeWillChange]
	function key() {
		return $this->prev_count + $this->index;
	}

	function next(): void {
		$this->index ++;
		if($this->index < $this->count)
			return;
		else {
			if($this->result->page_finished) {
				$this->index = null;
				$this->count = 0;
				return;
			}
			$this->prev_count += $this->pageSize;
			$this->query->pageResult($this->result);
			$this->count = $this->result->getResultCount();
			$this->index = ($this->result->failed || !$this->count) ? null : 0;
		}
	}

	function valid(): bool {
		return isset($this->index);
	}
	
	function count(): int {
		return $this->count;
	}
}

