<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/database/RowResult.php');
require_once('include/database/ListResultIterator.php');

class ListResult implements IteratorAggregate, Countable {
	var $base_model;
	var $links;
	var $fields;
	var $module_dirs;
	var $primary_key;
	var $use_key_index;
	var $is_union;
	var $union_key;
	var $sources;
	var $grouping;
	var $acl_checks;
	var $acl_owner_field;
	var $acl_teams;
	var $acl;

	var $query;
	var $result_source;
	var $page_size;
	var $page_index;
	var $page_pointer;
	var $page_result;
	var $page_header;
	var $page_finished;
	var $result_cache;
	var $limit_keys;

	var $total_counted, $total_count, $count_query;
	var $offset, $limit;
	var $order_by;
	var $result_count;

	var $rows;
	var $formatted_rows;
	var $sum_rows;

	var $failed, $error_msg;
	var $secondary_queries = false;
	var $unbuffered = false;

	var $special_rows = array();
	
	
	function getModuleDir() {
		if($this->module_dirs)
			return $this->module_dirs[0];
	}
	
	function getModelName() {
		return $this->base_model;
	}
	
	function getModel() {
		if($this->base_model)
			return new ModelDef($this->base_model);
	}
	
	function fail($msg) {
		$this->failed = true;
		$this->error_msg = $msg;
	}
	
	function getError() {
		return $this->error_msg;
	}

	function prependRows($new_rows, $special = null) {
		if (!is_array($new_rows))
			return;
		if (!is_array($this->rows))
			$this->rows = array();
		$this->rows = $new_rows + $this->rows;
		if (isset($special)) {
			foreach (array_keys($new_rows) as $k)
				$this->special_rows[$k] = $special;
		}
		$rowsCount = is_countable($this->rows) ? count($this->rows) : 0;
		$diff = $rowsCount - $this->result_count;
		$this->result_count += $diff;
		if ($this->total_counted)
			$this->total_count += $diff;
	}

	function getRowIndexes() {
		return $this->rows ? array_keys($this->rows) : array();
	}

	/**
	 * @param mixed $index
	 * @return RowResult
	 */
	function &getRowResult($index) {
		$r = new RowResult();
		foreach (array('base_model', 'links', 'fields', 'module_dirs', 'primary_key', 'union_key', 'secondary_queries', 'acl_checks', 'acl_owner_field') as $k)
			$r->$k = $this->$k;
		$r->row = $this->rows[$index];
		$r->formatted = $this->formatted_rows[$index] ?? [];
		$r->acl = array_get_default($this->acl, $index, array());
		return $r;
	}
	
	function getRowUpdate($index, $as_user_id=null) {
		$r = null;
		$row = $this->getRowResult($index);
		if($row)
			$r = $row->getRowUpdate($as_user_id);
		return $r;
	}
	
	function getRow($index, $formatted=false) {
		if($formatted)
			$this->formatted_rows[$index];
		return $this->rows[$index];
	}
	
	function getRows($formatted=false) {
		if($this->failed)
			return array();
		if($formatted)
			return $this->formatted_rows;
		return $this->rows;
	}
	
	function getRowResults() {
		$ret = array();
		foreach($this->getRowIndexes() as $k) {
			$ret[$k] = $this->getRowResult($k);
		}
		return $ret;
	}

	function getResultCount() {
		return $this->failed ? 0 : $this->result_count;
	}
	
	function getFieldDefinition($name) {
		if(isset($this->fields[$name]))
			return $this->fields[$name];
	}
	
	function getIterator(): Traversable {
		return new ListResultIterator($this);
	}
	
	function count(): int {
		return $this->getResultCount();
	}
}

?>
