<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
********************************************************************************/
/*********************************************************************************

* Description: This file handles the Data base functionality for the application specific
* to oracle database. It is called by the DBManager class to generate various sql statements.
*
* All the functions in this class will work with any bean which implements the meta interface.
* Please refer the DBManager documentation for the details.
*
* Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
* All Rights Reserved.
* Contributor(s): ______________________________________..
********************************************************************************/
require_once('include/database/DBHelper.php');

class MysqlHelper extends DBHelper
{
	static $FULLTEXT_INNODB = '5.6';
	var $dbType = 'mysql';

	// longreach - start added
	function createTableSQLParams($tablename, $fieldDefs, $indices){
		$sql = parent::createTableSQLParams($tablename, $fieldDefs, $indices);
		$fulltext = false;
		foreach ($indices as $idx) {
			if (strtolower(array_get_default($idx, 'type')) === 'fulltext') {
				$fulltext = true;
				break;
			}
		}
		if($sql !== false) {
			$sql .= " CHARACTER SET utf8 COLLATE utf8_general_ci ";
			if ($fulltext) {
				$mgr = DBManager::getInstance();
				$version = $mgr->getServerVersion();
				if(version_compare($version, self::$FULLTEXT_INNODB) < 0) {
					$sql .= "ENGINE=MyISAM ";
				}
			}
		}
		return $sql;
	}
	// longreach - end added


	/**
	* This is a private (php does not support it as of 4.x) method.
	* It outputs a correct string for the sql statement according to value
	* Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
	* All Rights Reserved..
	* Contributor(s): ______________________________________..
	*/
	function massageValue($val, $fieldDef) {
		if(! isset($val))
			return 'null';
        if($val === '')
			return "''";

        $type = $this->getFieldType($fieldDef);

		switch ($type) {
		  case 'int':
		  case 'uint':
		  case 'ulong':
		  case 'long':
		  case 'short':
          case 'tinyint':
          case 'duration':
          case 'reminder':
		    return '' . (int)$val;
		    break;
		  case 'double':
		  case 'float':
		    return '' . (double)$val;
		    break;
		}

		$qval = $this->quoteValue($val);
		return $qval;
	}

    /** returns the valid type for a column given the type in fieldDef
    */

    function getColumnType($type){
        static $map = array( 'int'      => 'int'
                    , 'double'   => 'double'
                    , 'float'    => 'float'
                    , 'uint'     => 'int unsigned'
                    , 'duration' => 'int unsigned'
                    , 'reminder' => 'int'
                    , 'ulong'    => 'bigint unsigned'
                    , 'long'     => 'bigint'
                    , 'short'    => 'smallint'
                    , 'varchar'  => 'varchar'
                    , 'text'     => 'text'
                    , 'date'     => 'date'
                    , 'enum'     => 'varchar'
                    , 'multienum' => 'varchar'
                    , 'status'     => 'varchar'
                    , 'datetime' => 'datetime'
                    , 'time'     => 'time'
                    , 'bool'     => 'bool'
                    , 'tinyint'  => 'tinyint'
                    , 'char'  => 'char'
                    , 'blob'  => 'blob'
                    , 'decimal' => 'decimal'
                    , 'decimal2' => 'decimal'
                    , 'mediumtext' => 'mediumtext'
                    );
        return array_get_default($map, $type, 'varchar');
    }

    /** private function to get sql for a column
    */
    function oneColumnSQLRep($fieldDef, $ignoreRequired = false, $table=''){
        $rep = parent::oneColumnSQLRep($fieldDef, $ignoreRequired, $table);
        return $rep;
    }

    /**
    * A private function which generates the SQL for changing columns
    */
    function changeColumnSQL($tablename, $fieldDefs, $action, $ignoreRequired = false){

        if ($this->isFieldArray($fieldDefs)){
          foreach ($fieldDefs as $def) $columns[] = $this->oneColumnSQLRep($def, $ignoreRequired);
        } else {
          $columns[] = $this->oneColumnSQLRep($fieldDefs, $ignoreRequired);
        }
		$action = strtoupper($action);
        $columns = implode(",\n$action COLUMN ", $columns);
        $sql = "ALTER TABLE $tablename $action COLUMN $columns";
        return $sql;
    }

    /**
    * This method generates sql that deletes a column identified by fieldDef.
    * Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
    * All Rights Reserved.
    * Contributor(s): ______________________________________..
    */
    function deleteColumnSQL($bean, $fieldDefs){
		if (is_string($bean)) {
			$table_name = $bean;
		} else {
			$table_name = $bean->getTableName();
		}
        if ($this->isFieldArray($fieldDefs)) foreach ($fieldDefs as $fieldDef) $columns[] = $fieldDef['name'];
        else $columns[] = $fieldDefs['name'];
        $columns = implode(",\nDROP COLUMN ", $columns);
        $sql = "ALTER TABLE `$table_name` DROP COLUMN $columns";
        return $sql;
    }

    /**
    * This method genrates sql for key statement for any bean identified by id.
    * The passes array is an array of field definitions or a field definition
    * itself. The keys generated will be either primary, foreign, unique, index
    * or none at all depending on the setting of the "key" parameter of a field definition
    * Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
    * All Rights Reserved.
    * Contributor(s): ______________________________________..
    */
    function keysSQL( $indices, $alter_table = false, $alter_action = ''){
       // check if the passed value is an array of fields.
       // if not, convert it into an array
       if (!$this->isFieldArray($indices)) $indices[] = $indices;

        $columns = array();
        foreach ($indices as $index){
          if(!empty($index['db']) && $index['db'] != 'mysql')continue;
          $type = $index['type'];

            $name = '`' . $index['name'] . '`';

            $fields = (array) $index['fields'];
            foreach ($fields as &$field){
                $parts = explode('(', $field,2) + [''];
                if (count($parts)==2) {
                    $field = '`' . $parts[0] . '`(' . $parts[1];
                } else {
                    $field='`' . $parts[0] . '`';
                }
            }
            unset($field);

            $fields = implode(", ", $fields);

          switch ($type){
            case 'unique':
                $columns[] = "UNIQUE $name ($fields)";
				break;
			// longreach - added fulltext
            case 'fulltext':
                $columns[] = "FULLTEXT $name ($fields)";
                break;
            case 'primary':
                $columns[] = "PRIMARY KEY ($fields)";
                break;
            case 'index':
            case 'foreign':
            case 'alternate_key':
                // to do - here it is assumed that the primary key of the foreign
                // table will always be named 'id'. It must be noted though
                // that this can easily be fixed by referring to db dictionalry
                // to find the correct primary field name
                if($alter_table){
                	$columns[] = "INDEX $name ($fields)";
                }else{
                	$columns[] = "KEY $name ($fields)";
                }
                break;
          }
       }
       $columns = implode(", $alter_action ", $columns);
       if(!empty($alter_action)){
       	$columns=$alter_action . ' '. $columns;
       }
       return $columns;
    }

    function quoteValue($string){
    	global $db;
        $string = $db->quote($string);
        return "'$string'";
    }

   	function setAutoIncrement($table, $field_name){
		return "auto_increment";
	}
	
    /**
     * Returns definitions of all indies for passed table. return will is a multi-dimensional array that
     * categorizes the index definition by types, unique, primary and index.
     * return format $indices = array ('index1'=>('name'=>'index1','type'=>'primary','fields'=>array('field1','field2'))
     * This format is similar to how indicies are defined in vardef file.
     */

    function get_indices(&$db, $tablename) {
        $tablename=$tablename;
        $indices=array();
        //find all unique indexes and primary keys.
        $query="SHOW INDEX FROM $tablename";
        $result=$db->query($query);
        while (($row=$db->fetchByAssoc($result)) !=null) {
            $index_type='index';
            if ($row['Key_name'] =='PRIMARY') {
                $index_type='primary';  
            }   
            $indices[strtolower($row['Key_name'])]['name']=strtolower($row['Key_name']);
            $indices[strtolower($row['Key_name'])]['type']=$index_type;
			$indices[strtolower($row['Key_name'])]['fields'][]=strtolower($row['Column_name']);
			// longreach - added
            $indices[strtolower($row['Key_name'])]['unique']= !$row['Non_unique'];
        }
        return $indices;
    }
    
    /**
     * function generate alter constraint statement given a tabe name and vardef definition.
     * supports both adding and droping a constraint.
     */
    function add_drop_constraint($table,$definition, $drop=false) {        
        $type=$definition['type'];
        $fields=implode(',',$definition['fields']);
        $name=$definition['name'];
        $foreignTable=isset($definition['foreignTable']) ? $definition['foreignTable'] : array();
        switch ($type){
            // generic indices
            case 'index':
            case 'alternate_key':
					if ($drop) {
						// longreach - modified
                        $sql = "ALTER TABLE {$table} DROP INDEX {$name}";
                    } else {
                        $sql = "CREATE INDEX {$name} ON {$table} ({$fields})";
                    }
                    break;
            // constraints as indices
            case 'unique':
                    if ($drop){
                        $sql = "ALTER TABLE {$table} DROP INDEX $name";
                    } else {
                        $sql = "ALTER TABLE {$table} ADD CONSTRAINT UNIQUE {$name} ({$fields})";
                    }
                    break;
            case 'primary':
                    if ($drop) {
                        $sql = "ALTER TABLE {$table} DROP PRIMARY KEY";
                    } else {
                        $sql = "ALTER TABLE {$table} ADD CONSTRAINT PRIMARY KEY ({$fields})";
                    }
                    break;
            case 'foreign':
                    if ($drop) {
                        $sql = "ALTER TABLE {$table} DROP FOREIGN KEY ({$fields})";
                    } else {
                        $sql = "ALTER TABLE {$table} ADD CONSTRAINT FORIEGN KEY {$name} ({$fields}) REFERENCES {$foreignTable}({$foreignfields})";
                    }
                    break;
        }
        return $sql;
    }

    function rename_index($old_definition,$new_definition,$table_name) {
        $ret_commands=array();
        $ret_commands[]=$this->add_drop_constraint($table_name,$old_definition,true);
        $ret_commands[]=$this->add_drop_constraint($table_name,$new_definition);
        return $ret_commands;
    }

}

?>
