<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


class RowResult {
	var $base_model;
	var $links;
	var $fields;
	var $module_dirs;
	var $primary_key;
	var $acl_checks;
	var $acl_owner_field;
	var $acl_teams;
	var $acl;
	var $union_key;
	
	var $query;
	
	var $row;
	var $formatted;
	var $update_errors;
	var $grouped_errors;
	var $new_record = false;
	var $for_duplicate = false;
	var $secondary_queries = false;
	var $related_data = [];
		
	var $is_union = false;
	var $use_key_index = false;
	var $order_by;

	var $failed, $error_msg, $error_code;

	var $filter_form;
	
	function fail($msg, $code=null) {
		$this->failed = true;
		$this->error_msg = $msg;
		$this->error_code = $code;
	}
	
	function getResultCount() {
		return $this->failed ? 0 : 1;
	}
	
	function getModuleDir() {
		if($this->module_dirs)
			return $this->module_dirs[0];
	}
	
	function getModelName() {
		return $this->base_model;
	}
	
	function getModel() {
		if($this->base_model)
			return new ModelDef($this->base_model);
	}
	
	function isForDuplicate() {
		return $this->for_duplicate;
	}
	
	function setForDuplicate($flag=true) {
		$this->for_duplicate = $flag;
		if($flag) $this->new_record = true;
	}
	
	function isNewRecord() {
		return $this->new_record;
	}
	
	/**
	 * Basically same as getField(), but better support
	 * for null values. Allows to distinguish a field with null value
	 * from field that was neither fetched nor updated (by passing $default
	 * that is not likely to be used as field value, objects of predefined
	 * class work well)
	 */

	function getFieldValue($name, $default=null, $formatted=false) {
		if($formatted)
			return array_key_exists($name, $this->formatted) ? $this->formatted[$name] : $default;
		return array_key_exists($name, $this->row) ? $this->row[$name] : $default;
	}
	
	function getField($name, $default=null, $formatted=false) {
		if($formatted)
			return array_get_default($this->formatted, $name, $default);
		return array_get_default($this->row, $name, $default);
	}
	
	function isFieldEmpty($name) {
		$val = $this->getField($name);
		return empty($val);
	}
	
	function isFieldBlank($name) {
		$val = $this->getField($name);
		return (! isset($val) || $val === '');
	}
	
	function addField($name, $spec=null) {
		if(! is_array($spec)) $spec = array('type' => 'varchar');
		if(! isset($spec['source'])) $spec['source'] = array('type' => 'non-db');
		$this->fields[$name] = $spec;
	}
	
	function getFieldNames() {
		if($this->fields)
			return array_keys($this->fields);
		return array();
	}
	
	function getFieldDefinition($name) {
		if(isset($this->fields[$name]))
			return $this->fields[$name];
	}
	
	function getRow($formatted=false) {
		if($formatted && isset($this->formatted)) return $this->formatted;
		return $this->row;
	}
	
	function getRowUpdate($as_user_id=null) {
		return RowUpdate::for_result($this, null, $as_user_id);
	}

	function addUpdateErrors($errs) {
		if(! is_array($this->update_errors))
			$this->update_errors = array();
		$this->update_errors = array_merge($this->update_errors, $errs);
		$this->grouped_errors = array();
		foreach($this->update_errors as $e) {
			$this->grouped_errors[$e[1]][] = $e;
		}
	}
	
	function getUpdateErrors() {
		return $this->update_errors;
	}
	
	function getFieldUpdateErrors($field_name) {
		return array_get_default($this->grouped_errors, $field_name);
	}
	
	function makeListResult() {
		$lr = new ListResult();
		foreach (array('base_model', 'links', 'fields', 'module_dirs', 'primary_key', 'union_key', 'secondary_queries', 'acl_checks', 'acl_owner_field') as $k)
			$lr->$k = $this->$k;
		$lr->use_key_index = false;
		$lr->rows = array($this->row);
		$lr->formatted_rows = array($this->formatted);
		$lr->acl = array($this->acl);
		$lr->result_count = 1;
		return $lr;
	}
	
	function assign($name, $val=null, $formatted=false) {
		if(is_array($name)) {
			foreach($name as $k => $v)
				$this->assign($k, $v, $formatted);
			return;
		}
		if($formatted)
			$this->formatted[$name] = $val;
		else
			$this->row[$name] = $val;
		if(! isset($this->fields[$name]))
			$this->addField($name);
	}
	
	function getAclAllowed($action) {
		if($this->acl && isset($this->acl[$action]))
			return $this->acl[$action];
		if($this->acl_checks && isset($this->acl_checks[$action]))
			return $this->acl_checks[$action]['allow'];
	}

	function getAclLevel($action) {
		if($this->acl_checks && isset($this->acl_checks[$action])) {
			return $this->acl_checks[$action]['level'];
		}
	}
	
	function getOwnerId() {
		// must add 'owner' acl check to listquery
		if($this->acl_owner_field) return $this->getField($this->acl_owner_field);
	}
	
	function getPrimaryKeyValue($force_array = false) {
		$pk = $this->primary_key;
		if(! is_array($pk)) $pk = array($pk);
		$val = array();
		foreach ($pk as $k) {
			if (isset($this->row[$k])) {
				$val[$k] = $this->row[$k];
			}
		}
		if (! $force_array) {
			if (count($val) == 1)
				return current($val);
			else if(! $val)
				return null;
		}
		return $val;
	}
	
	function linkDefined($link_name) {
		$ltarget = $this->getLinkTargetModel($link_name);
		return !! $ltarget;
	}

	function getLinkModel($link_name) {
		$base = $this->getModel();
		if($base)
			return $base->getLinkModel($link_name);
	}
	
	function getLinkDefinition($link_name) {
		$base = $this->getModel();
		if($base)
			return $base->getLinkDefinition($link_name);		
	}
	
	function getLinkTargetModel($link_name) {
		$base = $this->getModel();
		if($base)
			return $base->getLinkTargetModel($link_name);
	}
	
	function getLinkQuery($link_name, $fields=null, $params=null) {
		if(! is_array($params)) $params = array();
		$params['link_name'] = $link_name;
		$params['parent_key'] = $this->getPrimaryKeyValue();
		$lq = new ListQuery($this->base_model, $fields, $params);
		return $lq;
	}
	
	function getLinkResults($link_name, $fields=null, $params=null) {
		$lq = $this->getLinkQuery($link_name, $fields, $params);
		return $lq->fetchAll();
	}
	
	function getLinkCount($link_name, $distinct=false, $params=null) {
		$lq = $this->getLinkQuery($link_name, null, $params);
		return $lq->fetchResultCount($distinct);
	}
	
	function fetchRelated($name, $fields=true, $params=null, $update=false, $as_user_id=null) {
		$def = $this->getFieldDefinition($name);
		if($def && array_get_default($def, 'type') == 'ref' && ! empty($def['id_name']) && ! empty($def['bean_name'])) {
			$id = $this->getField($def['id_name']);
			if($id) {
				$ret = ListQuery::quick_fetch($def['bean_name'], $id, $fields, $params);
				if($ret && $update) return RowUpdate::for_result($ret, null, $as_user_id);
				return $ret;
			}
		}
	}
	
	function setRelatedData($key, $value) {
		$this->related_data[$key] = $value;
	}

	function getRelatedData($key, $default_value=null) {
		return array_get_default($this->related_data, $key, $default_value);
	}

}

?>
