<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('data/Tracker.php');
require_once('include/database/Duplicates.php');
require_once('include/Notify/NotifyManager.php');

class RowUpdate {

	const ERROR_INVALID_DATE 		= 1;
	const ERROR_INVALID_TIME		= 2;
	const ERROR_INVALID_DATE_TIME	= 3;
	const ERROR_INVALID_INT			= 4;
	const ERROR_INVALID_CURRENCY	= 5;
	const ERROR_INVALID_FLOAT		= 6;
	const ERROR_STRING_TOO_LONG		= 7;

	/** @var  ModelDef $model */
	var $model;
	var $model_name;
	var $primary_key;
	var $as_user_id;
	var $orig_row;
	var $input;
	var $input_unformat;
	var $invalid;
	var $input_format = 'html';
	var $input_charset = 'utf-8';
	var $input_gmt = false;
	var $updates;
	var $erased;
	var $failed;
	var $errors;
	var $hooks;
	var $column_fields, $custom_fields, $fields, $display_fields;
	var $source, $source_action;
	var $link_models = array();
	var $related_data = array();
	var $related_files = array();
	var $new_record = false;
	var $new_updates = false;
	var $auto_trim_values = false;
	var $remove_whitespace = null;

	var $update_date_entered = true;
	var $update_date_modified = true;

	var $prohibit_workflow = false;
	var $disable_notify = false;
	var $auto_add_teams = true;
	var $prohibit_activity_log = false;

	/** @var $link_update RowUpdate */
	var $link_update;
	var $link_update_key;
	var $saved;
	var $request;
	var $duplicate_of_id;
    var $check_duplicates = false;
    // used by workflow processor
    var $workflow;

    static $null_empty_string_types = array(
    	'int', 'long', 'float', 'double', 'uint', 'ulong', 'short', 'tinyint', 'bool',
    	'decimal', 'duration', 'date', 'datetime', 'time',
    );


	/**
	 * @throws IAHInternalError
	 * @param string|ModelDef|RowResult $base
	 * @param int $as_user_id
	 */
	function __construct($base, $as_user_id = null) {
		$this->updates = array();
		$this->erased = array();
		$this->invalid = array();

		if ($base instanceof ModelDef || is_string($base)) {
			$this->setModel($base);
		} else if ($base instanceof RowResult) {
			if (!$this->setOriginal($base))
				throw new IAHInternalError("incomplete base for RowUpdate");
		}
		if(! empty($_SESSION['auto_trim_values']) || AppConfig::setting('site.database.auto_trim_values'))
			$this->auto_trim_values = true;

		$this->failed = false;

		if (!isset($as_user_id))
			$as_user_id = AppConfig::current_user_id();
		$this->as_user_id = $as_user_id;
	}

	static function for_result(RowResult $base, $duplicate_of_id=null, $as_user_id=null) {
		$upd = self::for_model($base->base_model, $as_user_id);
		$upd->duplicate_of_id = $duplicate_of_id;
		$upd->setOriginal($base);
		return $upd;
	}

	static function for_model($base, $as_user_id=null) {
		if(is_string($base))
			$model = $base;
		else if($base instanceof ModelDef)
			$model = $base->name;
		else
			return null;

		$def = AppConfig::setting("model.detail.$model.row_update");
		if (is_array($def)) {
			require_once $def['file'];
			return new $def['class']($base, $as_user_id);
		}

		return new RowUpdate($base, $as_user_id);
	}

	static function blank_for_model($base, $as_user_id=null) {
		$blank = RowUpdate::for_model($base, $as_user_id);
		$blank->loadBlankRecord();
		return $blank;
	}

	function setModel($model) {
		if (is_string($model))
			$model = new ModelDef($model);
		$this->model = $model;
		$this->model_name = $model->name;
		$this->primary_key = $model->getPrimaryKey(true);
		$this->column_fields = array_flip($model->getColumnFields());
		$this->custom_fields = array_flip($model->getCustomFields());
		$this->fields = $model->getFieldDefinitions();
		$this->display_fields = $model->getDisplayFieldDefinitions();
	}

	function setSource($src, $action=null) {
		$this->source = $src;
		$this->source_action = $action;
	}

	function getSource() {
		return $this->source;
	}

	function getSourceAction() {
		return $this->source_action;
	}

	function getInputCharset() {
		return $this->input_charset;
	}

	function setInputCharset($c) {
		$this->input_charset = strlen($c) ? strtolower($c) : 'utf-8';
	}

	function limitFields($limit_fields) {
		$defs = $this->fields;
		if (!is_array($defs) || !is_array($limit_fields))
			return false;
		$this->fields = array();
		foreach ($limit_fields as $k => $v) {
			if (is_array($v)) {
				if (is_int($k)) {
					if (empty($v['name'])) continue;
					$k = $v['name'];
				}
			} else {
				$k = $v;
				if (!isset($defs[$k])) continue;
				$v = $defs[$k];
			}
			$this->fields[$k] = $v;
		}
	}

	function setOriginal($original) {
		$this->updates = array();
		$this->invalid = array();
		$this->orig_row = null;
		$this->clearRelatedData();

		if ($original instanceof RowResult) {
			/** @var $base RowResult */
			if($original->base_model != $this->model_name)
				$this->setModel($original->base_model);

			/** @var $original RowResult */
			$this->new_record = $original->new_record;
			$original = $original->row;
		}
		if (!is_array($original))
			return false;
		if($this->new_record) {
			$this->updates = $original;
			return true;
		}
		foreach ($this->primary_key as $k) {
			if (!isset($original[$k])) {
				return false;
			}
		}
		$this->orig_row = $original;
		return true;
	}

	function loadBlankRecord() {
		if(! $this->model) return false;
		$lq = new ListQuery($this->model);
		$row = $lq->getBlankResult();
		$ret = $this->setOriginal($row);
		return $ret;
	}

	function isNewRecord() {
		return $this->new_record;
	}

	function retrieveRecord($id) {
		$row = ListQuery::quick_fetch($this->model_name, $id);
		if($row)
			return $this->setOriginal($row);
		return false;
	}

	function getInputFormat() {
		return $this->input_format;
	}

	function setInputFormat($format) {
		$this->input_format = $format;
	}


	function fillDefaults() {
		$this->execHooks('fill_defaults', array(&$this));
	}


	function validate($params = null) {
		static $skip_reqd = array(
			'id',
			'date_modified',
			'modified_user_id',
			'date_entered',
			'created_by',
			'deleted',
		);
		if (!is_array($params))
			$params = array();

		$this->fillDefaults();
		if ($this->new_updates) {
			$this->cascadeChanges();
		}

		$orig =& $this->orig_row;
		$upd =& $this->updates;
		$this->invalid = array();
		if ($this->primary_key) {
			foreach ($this->primary_key as $k) {
				if ($this->new_record) {
					if (!isset($upd[$k]) && $k != 'id') {
						// 'id' is hardcoded for now. may wish to use field metadata to specify
						// fields that will automatically be filled with a GUID
						$this->addValidationError('missing_primary', $k);
					}
				} else if (!isset($orig[$k]))
					// must retrieve primary keys for existing record in order to perform update
					$this->addValidationError('primary_not_retrieved', $k);
				else if (isset($upd[$k]) && $orig[$k] != $upd[$k])
					// cannot change primary key for existing record
					$this->addValidationError('no_update_primary', $k);
			}
		}
		if($this->new_record) {
			$reqd = $this->model->getRequiredFields();
			foreach($reqd as $f) {
				if(! isset($this->fields[$f]) || (! isset($this->column_fields[$f]) && ! isset($this->custom_fields[$f])) || in_array($f, $skip_reqd))
					continue;
				if($f == 'id_c' || $this->fields[$f]['type'] == 'sequence_prefix' || $this->fields[$f]['type'] == 'sequence_number')
					continue; // value filled in by this class
				if(! empty($this->fields[$f]['auto_increment']))
					continue; // auto-increment column, handled by DB
				if($this->fields[$f]['type'] == 'ref')
					continue; // ignore custom ref field
				if(! isset($upd[$f]) && ! isset($this->fields[$f]['default']))
					$this->addValidationError('missing_required', $f);
			}
		}
		foreach ($upd as $f => &$val) {
			$reason = null;
			if (!$this->checkValue($val, $f, $reason)) {
				$this->addValidationError('invalid_value', $f, true, $reason);
				$this->invalid[$f] = array_get_default($this->input, $f, '');
			}
		}

		$this->execHooks('validate', array(&$this));

		return !$this->failed;
	}


	function getFieldType($field_name) {
		if (!isset($this->fields[$field_name]))
			return null;
		return $this->fields[$field_name]['type'];
	}

	function getFieldDbType($field_name) {
		if (!isset($this->fields[$field_name]))
			return null;
		$dbt = array_get_default($this->fields[$field_name], 'dbType');
		if (!$dbt)
			$dbt = $this->fields[$field_name]['type'];
		if ($dbt === 'custom' && isset($this->fields[$field_name]['custom_type']))
			$dbt = $this->fields[$field_name]['custom_type'];
		return $dbt;
	}

	function getFieldDefinition($field_name, $check_display=false) {
		if($check_display)
			return $this->model->getDisplayField($field_name, true);
		if(isset($this->fields[$field_name]))
			return $this->fields[$field_name];
	}

	function getFieldNames() {
		return array_keys($this->fields);
	}

	function getFieldDefaultValue($field_name) {
		if (!isset($this->fields[$field_name]))
			return null;
		$def = $this->fields[$field_name];
		$val = array_get_default($def, 'default');
		if(isset($val)) {
			if(array_get_default($def, 'type') == 'date')
				$val = ListQuery::date_default_value($val);
		}
		return $val;
	}


	function insertRow() {
		$this->new_record = true;
		return $this->save('insert');
	}

	function updateRow($optional = false) {
		return $this->save('update', $optional);
	}

	function putRow($optional = false) {
		return $this->save('put', $optional);
	}

	function beforeSave() {
		$this->fillDefaults();

		try {
			RowUpdate_check_duplicates($this);
			if($this->new_record) {
				foreach($this->fields as $f => $fdef) {
					if($fdef['type'] == 'sequence_prefix' && ! empty($fdef['sequence_name'])) {
						if (empty($this->updates[$f]))
							$this->updates[$f] = AppConfig::get_sequence_prefix($fdef['sequence_name']);
					}
					else if($fdef['type'] == 'sequence_number' && ! empty($fdef['sequence_name'])) {
						if (empty($this->updates[$f]))
							$this->updates[$f] = AppConfig::next_sequence_value($fdef['sequence_name']);
					}
				}
			}

			// may raise IAHActionAbort/IAHActionComplete or adjust update parameters
			$this->execHooks('before_save', array(&$this));
			$this->unformatPhoneNumbers();
		} catch (IAHActionCompleted $e) {
			return true;
		} catch (IAHActionAbort $e) {
			$this->addValidationError('before_save_aborted', $e->getMessage(), true);
			return false;
		}
	}

    function checkDuplicates($redirect = true, $type = '') {
        require_bean($this->model_name);
        $duplicates = null;
        $dup_checked = array_get_default($_REQUEST, 'dup_checked', false);
        if ($this->new_record && (! $dup_checked && method_exists($this->model_name, 'find_duplicates')) ) {
            $duplicates = call_user_func_array(array($this->model_name, 'find_duplicates'), array(&$this, $redirect, $type));
            //$this->model->execSpecialHook('find_duplicates', array(&$this, $redirect));
        }
        return $duplicates;
    }

	function save($insert_mode = null, $optional = false) {
		/** @var $db DBManager */
		global $db, $log;

		$stat = $this->beforeSave();
		if(isset($stat)) return $stat;

		$upd = $this->getChanges();

		if($insert_mode != 'insert' && $optional && ! $upd)
			return true;

		if ($this->new_record && !@$upd['id']) unset($upd['id']);
		$upd = array_merge($this->getMetaChanges(), $upd);

		foreach ($this->primary_key as $k) {
			if (!isset($upd[$k]))
				return false;
		}
		$this->saved = $upd;

		if (!isset($insert_mode)) {
			$insert_mode = $this->new_record ? 'insert' : 'update';
		}

		$updRow = $updCustom = array();
		foreach ($upd as $k => $v) {
			$def = array_get_default($this->fields, $k);
			if (is_null($def) || ($def['type'] != 'ref' && empty($def['is_related_value']))) {
				if (isset($this->column_fields[$k])) {
					$updRow[$k] = $v;
				} elseif (isset($this->custom_fields[$k])) {
					$updCustom[$k] = $v;
				}
			}
		}

		// may throw IAHQueryError
		$db->insertRow($this->model, $updRow, $insert_mode);

		$pkVal = $this->getPrimaryKeyValue();
		$performCustUpdate = (is_countable($this->custom_fields) && count($this->custom_fields) && ($insert_mode == 'insert' || count($updCustom)));

		if ($performCustUpdate) {
			if($insert_mode == 'insert') {
				// cannot use putRow with no values. assign default values for all custom fields
				foreach($this->custom_fields as $f => $idx) {
					if(! array_key_exists($f, $updCustom)) {
						$def = array_get_default($this->fields, $f);
						if ($def['type'] != 'ref' && empty($def['is_related_value'])) {
							if($def && isset($def['default'])) {
								$updCustom[$f] = $def['default'];
								if ($def['type'] == 'date')
									$updCustom[$f] = ListQuery::date_default_value($updCustom[$f]);
							}
							else
								$updCustom[$f] = null;
						}
					}
				}
			}
			$updCustom['id_c'] = $pkVal;
			$lnk = $this->model->getLinkDefinition('_cstm');
			$custModel = new ModelDef($lnk['custom_table']);
			try {
				$db->putRow($custModel, $updCustom, true);
			} catch (IAHModelError $e) {
				$log->error("Error updating custom field values: " . $e->getMessage());
			}
		}

		if($this->related_files)
			$this->updateRelatedFiles();

		$this->updates = $upd;

		$this->afterSave();
		if($insert_mode == 'insert' || $insert_mode == 'put') {
			$this->new_record = false;
			$this->orig_row = array_merge($updRow, $updCustom);
		} else {
			if(isset($this->orig_row))
				array_extend($this->orig_row, array_merge($updRow, $updCustom));
		}
		if(isset($this->orig_row))
			$this->updates = array_diff_assoc($this->updates, $this->orig_row);
		else
			$this->clearChanges();

		return true;
	}


	function afterSave() {
		$erased_info = $this->recordErased();
		if($this->model->audit_enabled)
			$this->saveAuditRecords($erased_info);

		if($this->auto_add_teams)
			$this->updateInheritedTeams();

		$pkVal = $this->getPrimaryKeyValue();
		AppConfig::db_invalidate($this->getModelName(), $pkVal);

        if ($this->model->activity_log_enabled && AppConfig::setting('site.performance.activity_log'))
            $this->addActivity();

		$this->execHooks('after_save', array(&$this));

		$targets = $this->checkNotify();
		try {
			$this->execHooks('notify', array(&$this, &$targets));
		} catch (IAHActionCompleted $e) {
			$targets = null;
		} catch (IAHActionAbort $e) {
			$targets = null;
		}
		if($targets)
			$this->sendNotify($targets);
	}

	function checkNotify() {
		$ret = array();
        $module = $this->getModuleDir();
		if($this->model->type == 'bean' && AppConfig::setting('notify.enabled') && ! $this->disable_notify && !AppConfig::setting("notify.disabled.$module")) {
			if($this->new_record || $this->getFieldUpdated('assigned_user_id')) {
				$aid = $this->getField('assigned_user_id');
				if($aid != $this->getUserId())
					$ret['assigned'][] = array('bean_name' => 'User', 'id' => $aid);
			}
		}
		return $ret;
	}

	function sendNotify(array $tg_acts) {
		foreach($tg_acts as $act => $ts) {
			$model = $this->model->getModelName();
			$targets = NotifyTargets::target_list($ts);
			NotifyManager::record_notify_targets($model, $act, $this->getPrimaryKeyValue(), $targets, $this->getUserId());
		}
	}

	function recordErased() {
		$info = [];
		$lq = new ListQuery('erase_personal');
		$lq->addSimpleFilter('parent_type', $this->getModuleDir());
		$lq->addSimpleFilter('parent_id', $this->getPrimaryKeyValue());
		$existing = $lq->runQuerySingle();
		if (!$existing || $existing->failed) {
			if (empty($this->erased)) return [];
			$upd = RowUpdate::blank_for_model('erase_personal');
			$upd->set('parent_id', $this->getPrimaryKeyValue());
			$upd->set('parent_type', $this->getModuleDir());
			$upd->set('assigned_user_id', AppConfig::current_user_id());
		} else {
			$upd = RowUpdate::for_result($existing);
		}
		$fields = array_filter(explode(',', $upd->getField('erased_fields')));
		foreach (array_keys($this->updates) as $f) {
			$pos = array_search($f, $fields);
			if ($pos !== false) {
				$info[$f] = 2;
				unset($fields[$pos]);
			}
		}
		foreach (array_keys($this->erased) as $f) {
			if (!in_array($f, $fields)) {
				$info[$f] = 1;
				$fields[] = $f;
			} else {
				unset($info[$f]);
			}
		}
		$upd->set('erased_fields', join(',', $fields));
		$upd->save();
		$this->erased = [];
		return $info;
	}

	function saveAuditRecords($erased_info = []) {
		/** @var $db DBManager */
		global $db, $log;

		$audit_table = $this->model->getAuditTableName();
		if (!$audit_table) {
			$log->error("Audit table not found for {$this->model->name}");
			return false;
		}

		list($rows, $toerase) = $this->getAuditRecords($erased_info);
		if (!is_array($rows))
			return $rows;

		if (!$rows && !$toerase)
			return true;

		$auditModel = new ModelDef('audit');
		$auditModel->table_name = $audit_table;

		foreach ($rows as $r) {
			$db->insertRow($auditModel, $r);
		}
		foreach ($toerase as $r) {
			$r['before_value_text'] = $r['before_value_string'] = null;
			$r['after_value_text'] = $r['after_value_string'] = null;
			$r['erased'] = 1;
			$db->updateRow($auditModel, $r);
		}
	}

	function getAuditRecords($erased_info = []) {
		global $current_user, $log;

		$auditFields = $this->model->getAuditedFields();
		if (!$auditFields)
			return true;

		$orig = $this->orig_row;
		if (!$orig) $orig = array();

		$pk = $this->primary_key[0];
		if (empty($this->updates[$pk])) {
			$log->error("Primary key not found for audit record {$this->model->name}");
			return false;
		}

		$rows = $existing = [];
		$source = $this->getField('personal_info_source');

		foreach ($auditFields as $f) {
			$erased = array_get_default($erased_info, $f, 0);
			if (array_key_exists($f, $this->updates)) {
				$coltype = $this->model->getFieldType($f);
				$upd = array(
					'id' => create_guid(),
					'parent_id' => $this->updates[$pk],
					'field_name' => $f,
					'data_type' => $coltype,
					'date_created' => gmdate("Y-m-d H:i:s"),
					'created_by' => $current_user->id,
					'erased' => $erased,
					'source' => $source,
				);
				if ($coltype == 'text' || $coltype == 'mediumtext') {
					$upd['before_value_text'] = array_get_default($orig, $f);
					$upd['after_value_text'] = $this->updates[$f];
				} else {
					$upd['before_value_string'] = array_get_default($orig, $f);
					$upd['after_value_string'] = $this->updates[$f];
				}
				if ($erased == 1) {
					$upd['before_value_text'] = $upd['before_value_string'] = null;
				}
				$rows[] = $upd;
			}
		}

		$erased_names = array_keys(array_filter($erased_info, function($e) {return $e == 1;}));
		if ($erased_names) {
			$lq = new ListQuery('audit');
			$lq->base_model->table_name = $this->model->audit_table_name;
			$lq->addSimpleFilter('parent_id', $this->updates[$pk]);
			$lq->addSimpleFilter('field_name', $erased_names);
			$existing = $lq->fetchAllRows();
		}

		return [$rows, $existing];
	}


	function getUserInheritedTeams($detail=false) {
		$teams = array();

		if($this->new_record && AppConfig::setting('aclconfig.inherit_creator')) {
			$uid = $this->getUserId();
			if($uid) {
				$found = AppConfig::setting("acl.user_teams.$uid", array());
				foreach($found as $t)
					if(empty($t['noninheritable'])) {
						if($detail) $teams[$t['id']] = $t;
						else $teams[] = $t['id'];
					}
			}
		}

		if(AppConfig::setting('aclconfig.inherit_assigned') && $this->model->haveFieldDefinition('assigned_user_id')) {
			$uid = $this->getField('assigned_user_id');
			if($uid) {
				$found = AppConfig::setting("acl.user_teams.$uid", array());
				foreach($found as $t)
					if(empty($t['noninheritable'])) {
						if($detail) $teams[$t['id']] = $t;
						else $teams[] = $t['id'];
					}
			}
		}

		if(! $detail) $teams = array_unique($teams);
		return $teams;
	}


	function updateInheritedTeams() {
		$teams = $this->getInheritedTeams();
		if($teams) {
			extract($teams);
			if($remove) $this->removeLinks('securitygroups', $remove);
			if($add) $this->addUpdateLinks('securitygroups', $add);
		}
		return $teams;
	}


	function getInheritedTeams($detail=false) {
		if(! $this->model || ! $this->model->haveLinkDefinition('securitygroups'))
			return false;
		if($this->model->haveFieldDefinition('shared_with'))
			return false;
		if($this->model_name == 'User' || $this->model_name == 'ACLRole')
			return false;
		$add = $remove = array();

		if($this->new_record) {
			$add = $this->getUserInheritedTeams();
		}

		if(AppConfig::setting('aclconfig.inherit_account')) {
			$acc_f = AppConfig::setting("model.detail.{$this->model_name}.primary_account");
			if($acc_f) {
				$fdef = AppConfig::setting("model.fields.{$this->model_name}.$acc_f");
				if($fdef && ! empty($fdef['id_name'])) {
					$rel_acc = $fdef['id_name'];
					$acc_new = $this->getField($rel_acc);
					$acc_orig = $this->getField($rel_acc, null, true);

					$acc_replace = AppConfig::setting('aclconfig.inherit_account_replace');

					if(($this->new_record && $acc_new) || ($acc_replace && $acc_new != $acc_orig)) {
						if($acc_new) {
							$teams = ListQuery::quick_fetch_linked('Account', $acc_new, 'securitygroups',
								array('id'), null, true);
							if($teams)
								foreach($teams as $t)
									$add[] = $t['id'];
						}

						if(! $this->new_record && $acc_orig) {
							$teams = ListQuery::quick_fetch_linked('Account', $acc_orig, 'securitygroups',
								array('id'), null, true);
							if($teams)
								foreach($teams as $t)
									$remove[] = $t['id'];

							// don't remove teams inherited from the created by / assigned user
							$user = $this->getUserInheritedTeams();
							$remove = array_diff($remove, $user);
						}
					}
				}
			}
		}

		$remove = array_diff($remove, $add);
		$add = array_diff($add, $remove);
		if($detail) {
			$add = $this->getTeamsDetail($add);
			$remove = $this->getTeamsDetail($remove);
		}
		return compact('add', 'remove');
	}

	function getTeamsDetail($list) {
		$teams = AppConfig::setting('acl.all_teams');
		$ret = array();
		foreach($list as $id) {
			if(isset($teams[$id]))
				$ret[$id] = $teams[$id];
		}
		return $ret;
	}


	function updateAccountInheritedTeams($team_id, $added=true) {
		global $db, $log;
		if(! AppConfig::setting('aclconfig.inherit_account_replace')) return;
		if(! $this->model || $this->model->getModelName() != 'Account') return;
		$pid = $this->getPrimaryKeyValue();
		if(! $pid) return;
		$seen = array();
		foreach($this->model->getLinks() as $l) {
			if(empty($l['bean_name'])) continue;
			$rmodel = $l['bean_name'];
			if(! empty($seen[$rmodel]) || $rmodel == 'Account' || $rmodel == 'SecurityGroup') continue;
			$pacc = AppConfig::setting("model.detail.$rmodel.primary_account");
			if($pacc) {
				$pacc_id = AppConfig::setting("model.fields.$rmodel.$pacc.id_name");
				$tbl = AppConfig::setting("model.detail.$rmodel.table_name");
				$ts = AppConfig::setting("model.links.$rmodel.securitygroups");
				if($ts && $tbl && $pacc_id) {
					$mod = AppConfig::module_for_model($rmodel);
					$tnow = gmdate('Y-m-d H:i:s');
					$qid = $db->quote($team_id);
					if($added)
						$q = "INSERT INTO securitygroups_records (date_modified, securitygroup_id, module, record_id, deleted) "
							."SELECT '$tnow', '$qid', '$mod', t.id, 0 FROM `$tbl` t WHERE t.$pacc_id='$pid' AND NOT t.deleted "
							."ON DUPLICATE KEY UPDATE date_modified='$tnow', deleted=0";
					else
						$q = "UPDATE securitygroups_records SET date_modified='$tnow', deleted=1 "
							."WHERE securitygroup_id='$qid' AND module='$mod' AND NOT deleted AND record_id IN "
							."(SELECT id FROM `$tbl` t WHERE t.$pacc_id='$pid' AND NOT t.deleted)";
					$db->query($q, false);
				}
			}
			$seen[$rmodel] = 1;
		}
	}


    function addActivity() {
        $status = null;
        if ($this->new_record) {
            $status = 'created';
			$cb = array($this->model_name, 'get_new_activity_status');
			if (is_callable($cb))
	            $status = call_user_func_array($cb, array(&$this));
        } else {
            require_bean($this->model_name);
			$cb = array($this->model_name, 'get_activity_status');
			if (is_callable($cb))
	            $status = call_user_func_array($cb, array(&$this));
        }

        if ($status) {
            require_once('modules/ActivityLog/ActivityLog.php');
            $log = new ActivityLog();
            $log->addActivity($this, $status);
            $log->deleteOldActivities();
        }
    }

	private static function unformat_phone_number ($phone) {
		return ListQuery::unformat_phone_number($phone);
	}

	private function unformatPhoneNumbers() {
		foreach ($this->fields as $f) {
			if ($f['type'] == 'phone' || $f['type'] == 'fax') {
				$fraw = $f['name'] . '__raw';
				if (isset($this->fields[$fraw])) {
					$val = $this->getField($f['name']);
					if (isset($val)) {
						$this->set($fraw, self::unformat_phone_number($val));
					}
				}
			}
		}
	}

    function clearChanges() {
		$this->updates = array();
	}


	function markDeleted($deleted = true) {
		/** @var $db DBManager */
		global $db;

		if (($this->new_record || !isset($this->orig_row) || !isset($this->orig_row['deleted'])) && $this->model->type != 'link')
			return false;

		$action_name = $deleted ? 'delete' : 'undelete';

		$this->updates['deleted'] = $deleted ? 1 : 0;

		try {
			$this->execHooks('before_' . $action_name, array(&$this));
		} catch (IAHActionCompleted $e) {
			return true;
		} catch (IAHActionAbort $e) {
			return false;
		}

		// also sets date_modified & modified_user_id
		$this->saved = $this->getChanges(true);
		$db->insertRow($this->model, $this->saved, 'update');
		$ret = $db->getAffectedRowCount();
		$primary = $this->getPrimaryKeyValue();
		if($primary && is_string($primary)) {
			Tracker::delete_item_history($this->getModuleDir(), $primary);
			AppConfig::db_invalidate($this->getModelName(), $primary);
		}
		$this->orig_row['deleted'] = false;

		$this->execHooks('after_' . $action_name, array(&$this));

		if ($deleted && $ret) {
        	if ($this->model->activity_log_enabled && AppConfig::setting('site.performance.activity_log')) {
        	    require_once('modules/ActivityLog/ActivityLog.php');
	            $log = new ActivityLog();
    	        $log->addActivity($this, 'deleted');
        	    $log->deleteOldActivities();
			}
		}
		return $ret;
	}


	function deleteRow() {
		global $db;
		$key = $this->getPrimaryKeyValue(true);
		foreach ($this->primary_key as $k) {
			if (!isset($key[$k]))
				return false;
		}
		return $db->deleteRow($this->model, $key);
	}


	function linkDefined($link_name) {
		$ltarget = $this->getLinkTargetModel($link_name);
		return !! $ltarget;
	}

	function getLinkDefinition($link_name) {
		return $this->model->getLinkDefinition($link_name);
	}

	function addUpdateLink($link_name, $rel_id, $params = null, $formatted = false, $validate = false) {
		if($this->addUpdateLinks($link_name, array($rel_id), $params, $formatted, $validate))
			return true;
		return false;
	}

	function addUpdateLinks($link_name, $rel_ids, $params = null, $formatted = false, $validate = false, $no_replace=false) {
		if (!is_array($params)) $params = array();

		if (!is_array($rel_ids) || !count($rel_ids))
			return true;
		$pkey = $this->getPrimaryKeyValue();
		if (!$pkey) {
			return false;
		}
		$ltarget = $this->getLinkTargetModel($link_name);
		if (!$ltarget) {
			return false; // raise exception?
		}

		$action_name = 'add_link';
		$join = $ltarget->getJoinModel();
		$added = 0;

		if ($join) {
            $link_model = $join;
			// handle insert using a join table (probably many-to-many)
			$lspec = $join->getLinkSpec();
			$lkey = $lspec['relationship_spec']['left']['join_key'];
			$rkey = $lspec['relationship_spec']['right']['join_key'];
            $rel_key = $rkey;
			$row_tpl = array(
				$lkey => $pkey,
				'deleted' => 0,
			);
			$jspec = $lspec['relationship_spec']['join'];
			if (isset($jspec['role_field']) && isset($jspec['role_value'])) {
				$row_tpl[$jspec['role_field']] = $jspec['role_value'];
			}

		} else {
            //updating the related bean
            $link_model = $ltarget;
            $lspec = $ltarget->getLinkSpec();
            $rkey = $lspec['relationship_spec']['right']['key'];
            $lkey = $lspec['relationship_spec']['left']['key'];
            $rel_key = $lkey;
            $row_tpl = array($rkey => $pkey,);
		}

		$this->link_update_key = $rel_key;

        foreach ($rel_ids as $rel_row) {
            $this->link_update = RowUpdate::for_model($link_model, $this->as_user_id);
            $this->link_update->prohibit_workflow = $this->prohibit_workflow;

            if (is_array($rel_row)) {
                if (!isset($rel_row[$rel_key])) {
                    if(isset($rel_row['id'])) {
                        $rel_row[$rel_key] = $rel_row['id'];
                        unset($rel_row['id']);
                    } else
                        continue; // no row ID given
                }
            } else if ($rel_row) {
                $rel_row = array($rel_key => $rel_row);
            } else
                continue; // no row ID given
            $rel_row = array_merge($row_tpl, $rel_row);

            if (isset($params[$rel_row[$rel_key]])) {
                $additional = $params[$rel_row[$rel_key]];
                $rel_row[$additional['name']] = $additional['value'];
            }

            if (! $join) {
                $result = ListQuery::quick_fetch_row($this->link_update->model_name, $rel_row[$rel_key]);
                if($no_replace && ! empty($result[$rkey]))
                	continue;
                if (! $this->link_update->setOriginal($result))
                    return false;
                $this->link_update->loadInput($rel_row, $formatted);
            }
            else // not fetching original row, so allow blank values to be loaded
                $this->link_update->loadInput($rel_row, $formatted, false);

            try {
				$this->execHooks(
					'before_' . $action_name,
					array(&$this, $link_name, $this->link_update, false),
					$ltarget,
					array(&$this, $link_name, $this->link_update, true)
				);
            } catch (IAHActionCompleted $e) {
                $this->link_update->clearChanges();
                $added ++;
                continue;
            } catch (IAHActionAbort $e) {
                $this->link_update->clearChanges();
                continue;
            }

            if ($join) {
                $join_mod_pk = $ltarget->join_model->primary_key;
                //if (! is_array($join_mod_pk) && ! isset($rel_row[$join_mod_pk]))
                    $this->link_update->new_record = true;
                if((! $validate || $this->link_update->validate()) && $this->link_update->putRow())
					$added ++;
            } else {
                if((! $validate || $this->link_update->validate()) && $this->link_update->save())
                	$added ++;
            }

			$this->execHooks(
				'after_' . $action_name,
				array(&$this, $link_name, $this->link_update, false),
				$ltarget,
				array(&$this, $link_name, $this->link_update, true)
			);

            if($link_model->getModelName() == 'securitygroups_records' && $this->model->getModelName() == 'Account') {
            	$this->updateAccountInheritedTeams($rel_row[$rel_key], true);
            }

            $this->link_update->clearChanges();
        }

        return $added;
	}

	function removeLink($link_name, $rel_id) {
		$this->removeLinks($link_name, array($rel_id));
	}

	function removeLinks($link_name, $rel_ids) {
		if (!is_array($rel_ids) || !count($rel_ids))
			return true;
		$pkey = $this->getPrimaryKeyValue();
		if (!$pkey) {
			return false;
		}
		$ltarget = $this->getLinkTargetModel($link_name);
		if (!$ltarget) {
			return false; // raise exception?
		}
		if($ltarget->is_union) {
			if(! empty($ltarget->link_spec['relationship_spec'])) {
				$rspec = $ltarget->link_spec['relationship_spec'];
				if(! empty($rspec['sources'])) {
					foreach($rspec['sources'] as $k => $src) {
						$ret = $this->removeLinks($k, $rel_ids);
					}
				}
			}
			return true;
		}

		$action_name = 'remove_link';
		$join = $ltarget->getJoinModel();

        if ($join) {
            $link_model = $join;
            $lspec = $join->getLinkSpec();
            $lkey = $lspec['relationship_spec']['left']['join_key'];
            $rkey = $lspec['relationship_spec']['right']['join_key'];
            $rel_key = $rkey;
            $row_tpl = array(
                $lkey => $pkey,
            );
            $jspec = $lspec['relationship_spec']['join'];
            if (isset($jspec['role_field']) && isset($jspec['role_value'])) {
                $row_tpl[$jspec['role_field']] = $jspec['role_value'];
            }
        } else {
            //updating the related bean
            $link_model = $ltarget;
            $lspec = $ltarget->getLinkSpec();
            $rkey = $lspec['relationship_spec']['right']['key'];
            $lkey = $lspec['relationship_spec']['left']['key'];
            $rel_key = $lkey;
            $row_tpl = array($rkey => $pkey,);
        }

        $this->link_update_key = $lkey;
        $found = 0;

        foreach ($rel_ids as $rel_row) {
            $this->link_update = RowUpdate::for_model($link_model, $this->as_user_id);
            $this->link_update->prohibit_workflow = $this->prohibit_workflow;

            if (is_array($rel_row)) {
                if (!isset($rel_row[$rel_key]) && isset($rel_row['id'])) {
                    $rel_row[$rel_key] = $rel_row['id'];
                    unset($rel_row['id']);
                } else
                    continue; // no row ID given
            } else if ($rel_row) {
                $rel_row = array($rel_key => $rel_row);
            } else
                continue; // no row ID given
            $rel_row = array_merge($row_tpl, $rel_row);

            if (! $join) {
                $result = ListQuery::quick_fetch_row($this->link_update->model_name, $rel_row[$rel_key]);
                if (! $this->link_update->setOriginal($result))
                    return false;
            }
            $this->link_update->set($rel_row);

            try {
                $this->execHooks('before_' . $action_name, array(&$this, $link_name, false), $ltarget, array(&$this, $link_name, true));
            } catch (IAHActionCompleted $e) {
                $this->link_update->clearChanges();
                continue;
            } catch (IAHActionAbort $e) {
                $this->link_update->clearChanges();
                continue;
            }

            if ($join) {
                if($this->link_update->markDeleted())
                    $found ++;
            } else {
                $required = false;
                if (isset($this->link_update->fields[$rkey]) && isset($this->link_update->fields[$rkey]['required']))
                    $required = $this->link_update->fields[$rkey]['required'];
                if (! $required) {
                    $this->link_update->set($rkey, null);
                    if ($this->link_update->save())
                        $found ++;
                }
            }

            $this->execHooks('after_' . $action_name, array(&$this, $link_name, false), $ltarget, array(&$this, $link_name, true));

            if($link_model->getModelName() == 'securitygroups_records' && $this->model->getModelName() == 'Account') {
				$this->updateAccountInheritedTeams($rel_row[$rel_key], false);
			}

            $this->link_update->clearChanges();
        }

        return $found;
	}

	function removeAllLinks($link_name, $except = null) {
		$pkey = $this->getPrimaryKeyValue();
		if (!$pkey) {
			return false;
		}
		$ltarget = $this->getLinkTargetModel($link_name);
		if (!$ltarget) {
			return false; // raise exception?
		}

		$join = $ltarget->getJoinModel();

		if ($join) {
			$lspec = $join->getLinkSpec();
			$lkey = $lspec['relationship_spec']['left']['join_key'];
			$rkey = $lspec['relationship_spec']['right']['join_key'];

			$lq = new ListQuery($join, array('id', $rkey, 'date_entered', 'date_modified', 'deleted'), array('plain' => true));
			$lq->addSimpleFilter($lkey, $pkey, null, 'primary');

			$jspec = $lspec['relationship_spec']['join'];
			if (isset($jspec['role_field']) && isset($jspec['role_value'])) {
				$lq->addSimpleFilter($jspec['role_field'], $jspec['role_value'], null, 'join_role');
			}

			if($except) {
				if(! is_array($except))
					$except = array($except);
				foreach($except as $e) {
					if(is_array($e)) {
						if(isset($e[$rkey]))
							$id = $e[$rkey];
						else if(isset($e['id']))
							$id = $e['id'];
						else
							continue;
					} else
						$id = $e;
					$except_ids[$id] = 1;
				}
				$except_ids = array_keys($except_ids);
			} else
				$except_ids = array();

			$result = $lq->fetchAll();
			if ($result) {
				$rows = array();
				foreach ($result->rows as &$r) {
					if(! in_array($r[$rkey], $except_ids))
						$rows[] = $r[$rkey];
				}

				$this->removeLinks($link_name, $rows);
			}
		} else {
			// not implemented
		}
	}


	function replaceLinks($link_name, $rel_ids, $params = null) {
		$this->removeAllLinks($link_name, $rel_ids);
		$this->addUpdateLinks($link_name, $rel_ids, $params);
	}


	/**
	 * @param string $link_name
	 * @return ModelDef
	 */
	function getLinkTargetModel($link_name) {
		if (!isset($this->link_models[$link_name])) {
			$link = $this->model->getLinkTargetModel($link_name);
			if (!isset($link)) $link = false;
			$this->link_models[$link_name] = $link;
		}

		return $this->link_models[$link_name];
	}

	function getUserId() {
		if ($this->as_user_id)
			return $this->as_user_id;
		return AppConfig::current_user_id();
	}

	function setUserId($uid) {
		$this->as_user_id = $uid;
	}


	function clearErrors() {
		$this->errors = array();
	}

	function addValidationError($code, $info = null, $fatal = true, $reason = null) {
		$this->errors[] = array($code, $info, $fatal, $reason);
		if ($fatal)
			$this->failed = true;
	}

	static function format_error_reason($r) {
		switch ($r) {
		case self::ERROR_INVALID_DATE:
			return translate('LBL_VALIDATE_ERROR_INVALID_DATE');
		case self::ERROR_INVALID_TIME:
			return translate('LBL_VALIDATE_ERROR_INVALID_TIME');
		case self::ERROR_INVALID_DATE_TIME:
			return translate('LBL_VALIDATE_ERROR_INVALID_DATE_TIME');
		case self::ERROR_INVALID_INT:
			return translate('LBL_VALIDATE_ERROR_INVALID_INT');
		case self::ERROR_INVALID_CURRENCY:
			return translate('LBL_VALIDATE_ERROR_INVALID_CURRENCY');
		case self::ERROR_INVALID_FLOAT:
			return translate('LBL_VALIDATE_ERROR_INVALID_FLOAT');
		case self::ERROR_STRING_TOO_LONG:
			return translate('LBL_VALIDATE_ERROR_STRING_TOO_LONG');
		}
		return $r;
	}

	function format_error_code($e) {
		$str = $e[0];
		switch ($e[0]) {
		case 'invalid_value':
			$str = translate('LBL_VALIDATE_ERROR_INVALID_VALUE') . ' ' . $this->translateFieldName($e[1]);
			break;
		case 'missing_primary':
			$str = translate('LBL_VALIDATE_ERROR_MISSING_PRIMARY');
			break;
		case 'primary_not_retrieved':
			$str = translate('LBL_VALIDATE_ERROR_PRIMARY_NOT_RETRIEVED');
			break;
		case 'no_update_primary':
			$str = translate('LBL_VALIDATE_ERROR_NO_UPDATE_PRIMARY');
			break;
		case 'missing_required':
			$str = translate('LBL_VALIDATE_ERROR_MISSING_REQUIRED') . ' ' . $this->translateFieldName($e[1]);
			break;
		case 'before_save_aborted':
			$str = translate('LBL_VALIDATE_ERROR_BEFORE_SAVE') . ' : ' . $e[1];
			break;
		case 'invalid_formula':
			$str = translate('LBL_VALIDATE_ERROR_INCORRECT_PRICEBOOK_FORMAT');
			break;
		}
		if ($e[3]) {
			$str .= ' : ' . self::format_error_reason($e[3]);
		}
		return $str;
	}

	function translateFieldName($f) {
		$def = $this->getFieldDefinition($f);
		if (!$def)
			return $f;
		if (!isset($def['vname']))
			return $f;
		return translate($def['vname'], array_get_default($def, 'vname_module', $this->getModuleDir()));
	}

	function getErrors($as_string=false, $message_only = false) {
		$ret = $this->errors;
		if($as_string) {
			if(is_array($ret)) {
				$r2 = array();
				foreach($ret as $r) {
					if(is_array($r)) {
						$e = $this->format_error_code($r);
						$e .= $r[2] ? ' (fatal)' : ' (non-fatal)';
						$r2[] = $e;
					} else {
						$r2[] = $r;
					}
				}
				if (is_numeric($as_string))
					$ret = $r2;
				else
					$ret = implode("\n", $r2);
			}
			else
				$ret = '';
		}
		return $ret;
	}

	function getFailed() {
		return $this->failed;
	}

	function getModel() {
		return $this->model;
	}

	function getModelName() {
		return $this->model_name;
	}

	function getModuleDir() {
		return $this->model->getModuleDir();
	}

	function getPrimaryKey() {
		return $this->primary_key;
	}

	function getTableName() {
		return $this->model->getTableName();
	}

	function getPrimaryKeyValue($force_array = false) {
		$val = array();
		foreach ($this->primary_key as $k) {
			if (isset($this->orig_row[$k])) {
				$val[$k] = $this->orig_row[$k];
			}
			else if ($this->new_record && isset($this->saved[$k])) {
				$val[$k] = $this->saved[$k];
			}
		}
		if (! $force_array) {
			if (count($val) == 1)
				return current($val);
			else if(! $val)
				return null;
		}
		return $val;
	}


	function loadRequest($prefix = '', $formatted = true, $ignore_blank = false) {
		if (isset($this->request))
			$req =& $this->request;
		else
			$req =& $_REQUEST;

		try {
			$this->execHooks('load_request', array(&$this, $req, $ignore_blank));
		} catch (IAHActionCompleted $e) {
			$skip = true;
		} catch (IAHActionAbort $e) {
			$skip = true;
		}

		if(empty($skip)) {
			$inp = array();
			$input_fields = array_merge($this->column_fields, $this->custom_fields);
			foreach($this->fields as $f)
				if(! empty($f['updateable']) && ! empty($f['name'])) $input_fields[$f['name']] = 1;
			foreach ($input_fields as $f => $_idx) {
				if (isset($req[$prefix . $f])) {
					if ($ignore_blank && $req[$prefix . $f] === '')
						continue;
					$inp[$f] = $req[$prefix . $f];
				}
			}

			$this->loadInput($inp, $formatted);
			$ffs = $this->model->getFileFields();
			if($ffs)
				$this->loadInputFiles($ffs);
			$ifs = $this->model->getImageFields();
			if($ifs)
				$this->loadInputImages($ifs);
		}

		if($this->new_record) {
			$aid = 'assigned_user_id';
			if (isset($this->fields[$aid]) && empty($this->updates[$aid])) {
				$this->updates[$aid] = AppConfig::current_user_id();
			}
			foreach($this->fields as $f => $fdef) {
				if($fdef['type'] == 'sequence_prefix' && ! empty($fdef['sequence_name']))
					$this->updates[$f] = AppConfig::get_sequence_prefix($fdef['sequence_name']);
			}
			if(empty($req[$prefix . 'record_inited']))
				$this->execHooks('new_record', array(&$this, $req));
		}
	}

	function loadInput($input, $formatted = true, $require_orig=true) {
		$this->input = $input;
		if($this->input_charset != 'utf-8') {
			foreach($input as $k => $v)
				$input[$k] = $GLOBALS['locale']->translateCharset($v, $this->input_charset, 'utf-8');
		}
		if ($formatted) {
			require_once('include/layout/FieldFormatter.php');
			$fmt = new FieldFormatter($this->input_format, 'editview');
			if($this->input_gmt) $fmt->offset_dates = false;
			if(isset($this->remove_whitespace))
				$fmt->remove_whitespace = $this->remove_whitespace;
			else if($this->source == 'editview' || $this->source == 'leadcapture')
				$fmt->remove_whitespace = true;
			$input = $fmt->unformatRow($this->fields, $input);
		}
		$this->input_unformat = $input;
		try {
			$this->execHooks('load_input', array(&$this, $input, $formatted));
			if ($input)
				$this->performSet($input, $require_orig);
		} catch (IAHActionCompleted $e) {
		} catch (IAHActionAbort $e) {
			return;
		}
	}

	function loadInputFiles(array $fields) {
		if(! is_array($fields))
			return;
		foreach($fields as $f) {
			$this->loadInputFile($f);
		}
	}

	function loadInputFile($field_name, $input_name=null) {
		require_once('include/upload_file.php');
		if(! $input_name) $input_name = $field_name;
		$fdef = $this->getFieldDefinition($field_name);
		if(! $fdef)
			return;
		$fdef['name'] = $input_name;
		$upload = new UploadFile($fdef);
		if(! empty($_REQUEST[$input_name . '_dupe']) && $this->new_record && $this->duplicate_of_id) {
			$mime = array_get_default($fdef, 'file_mime_type');
			$ext = array_get_default($fdef, 'file_ext');
			$src = ListQuery::quick_fetch_row($this->getModelName(), $this->duplicate_of_id, array($input_name, $mime, $ext), array('acl_filters' => 'view'));
			if($src && ! empty($src[$input_name]))
				return $this->addDuplicateOfFile($input_name, $src[$input_name], array_get_default($src, $mime), array_get_default($src, $ext));
			return; // original attachment not found. ignore any uploaded data
		}
		if(! empty($_REQUEST[$input_name . '_clear'])) {
			return $this->addInputFile($field_name, $upload, true, true);
		} else {
			if($upload->confirm_upload(true))
				return $this->addInputFile($field_name, $upload);
		}
	}

	function addInputFile($field_name, $upload, $remove_old=true, $no_move = false) {
		$fdef = $this->getFieldDefinition($field_name);
		if(! $fdef)
			return;
		$old = array();
		if($remove_old) $old[] = $this->getField($field_name, null, true);
		$this->related_files[$field_name] = array(&$upload, $old, $no_move);
		$input = array(
			$field_name => $upload->get_stored_file_name(),
		);
		if(! empty($fdef['file_mime_type']))
			$input[$fdef['file_mime_type']] = $upload->mime_type;
		if(! empty($fdef['file_ext']))
			$input[$fdef['file_ext']] = $upload->file_ext;
		$this->set($input);
		return $this->getField($field_name);
	}

	function addDuplicateOfFile($field_name, $file_name, $mime=null, $ext=null) {
		$fdef = $this->getFieldDefinition($field_name);
		if(! $fdef || ! $file_name)
			return;
		$dupe_path = UploadFile::duplicate_file2($file_name);
		if(! $dupe_path)
			return;
		$input = array(
			$field_name => $dupe_path,
		);
		if(! empty($fdef['file_mime_type']))
			$input[$fdef['file_mime_type']] = $mime;
		if(! empty($fdef['file_ext']))
			$input[$fdef['file_ext']] = $ext;
		$this->set($input);
		return $dupe_path;
	}

	function getRelatedFile($name, $ext=false) {
		if(isset($this->related_files[$name])) {
			$ret = $this->related_files[$name];
			if(! $ext) return $ret[0];
			return $ret;
		}
	}

	function loadInputImages(array $fields) {
		if(! is_array($fields))
			return;
		foreach($fields as $f) {
			$this->loadInputImage($f);
		}
	}

	function loadInputImage($field_name, $input_name=null) {
		if(! $input_name) $input_name = $field_name;
		$fdef = $this->getFieldDefinition($field_name);
		if(! $fdef)
			return;
		if(empty($fdef['filename']) || ! ($fname = $this->getFieldDefinition($fdef['filename'])) || empty($fname['name']))
			return;
		if(! empty($fdef['thumbnail_name']))
			$thumb = $this->getFieldDefinition($fdef['thumbnail_name']);
		if(! empty($_REQUEST[$input_name . '_clear'])) {
			$input = array(
				$fname['name'] => '',
			);
			if(! empty($thumb)) $input[$thumb['name']] = '';
			$this->set($input);
			return '';
		}
		require_once('include/upload_file.php');
		$upload = new UploadFile($fdef);
		if(! isset($_REQUEST[$input_name]) || substr($_REQUEST[$input_name], 0, 9) != 'INCOMING~')
			return;
		if($upload->confirm_upload(true)) {
			$input = array(
				$fname['name'] => $upload->get_stored_file_name(),
			);
			if(! empty($fdef['file_mime_type']))
				$input[$fdef['file_mime_type']] = $upload->mime_type;
			$old = array($this->getField($fname['name'], null, true));
			if(! empty($thumb)) {
				$input[$thumb['name']] = $upload->get_thumbnail_file_name();
				$old[] = $this->getField($thumb['name'], null, true);
			}
			$this->related_files[$field_name] = array(&$upload, $old);
			$this->set($input);
			return $input[$fname['name']];
		}
	}

	function updateRelatedFiles() {
		foreach($this->related_files as $field => &$rel) {
			foreach($rel[1] as $old_fname) {
				if($old_fname)
					$rel[0]->unlink_file('', $old_fname);
			}
			if (empty($rel[2]))
				$rel[0]->final_move();
		}
	}

	function checkValue(&$value, $field_name, &$reason) {
		$reason = null;
		$dbType = $this->getFieldDbType($field_name);
		$fType = $this->getFieldType($field_name);
		if ($dbType == 'date') {
			$value = trim($value);
			if(strlen($value)) {
				if(! preg_match('~^\d{4}-\d{2}-\d{2}$~', $value)) {
					$reason = self::ERROR_INVALID_DATE;
					return false;
				}
			} else
				$value = null;
		}
		else if ($dbType == 'time') {
			$value = trim($value);
			if(strlen($value)) {
				if(! preg_match('~^\d{1,2}:\d{2}(:\d{2})?$~', $value)) {
					$reason = self::ERROR_INVALID_TIME;
					return false;
				}
			} else
				$value = null;
		}
		else if ($dbType == 'datetime') {
			$value = trim($value);
			if(strlen($value)) {
				if(! preg_match('~^\d{4}-\d{2}-\d{2}\s+\d{1,2}:\d{2}(:\d{2})?$~', $value)) {
					$reason = self::ERROR_INVALID_DATE_TIME;
					return false;
				}
			} else
				$value = null;
		}
		else if ($dbType == 'int' || $dbType == 'short' || $dbType == 'tinyint') {
			$value = trim($value);
			if (strlen($value)) {
				if(((int) $value) . '' !== $value) {
					$reason = self::ERROR_INVALID_INT;
					return false;
				}
			} else
				$value = null;
		}
		else if ($fType == 'currency' || $fType == 'base_currency' || $fType == 'raw_currency') {
			// rounding and conversion done elsewhere
			$value = trim($value);
			if(strlen($value)) {
				if (!preg_match('~^-?0*(\d*(\.\d*)?)$~', $value, $m)) {
					$reason = self::ERROR_INVALID_CURRENCY;
					return false;
				}
			} else
				$value = null;
		}
		else if ($dbType == 'float' || $dbType == 'double') {
			$value = trim($value);
			if(strlen($value)) {
				if (!preg_match('~^-?0*(\d*(\.\d*)?)$~', $value, $m)) {
					$reason = self::ERROR_INVALID_FLOAT;
					return false;
				}
				$def_dec = 2;
				$decimals = array_get_default($this->fields[$field_name], 'decimal_places', $def_dec);
				// applies banker's rounding
				// FIXME - use system-configurable rounding, added to 6.7
				$value = round($value, $decimals, PHP_ROUND_HALF_EVEN) . '';
			} else
				$value = null;
		} else if ($dbType == 'varchar' && $fType == 'user_name') {
			$reason = User::check_user_name_duplication($this->getFieldValue('id'), $value);
			if ($reason) return false;
		}
		else if ($dbType == 'char' || $dbType == 'varchar') {
			if (isset($this->fields[$field_name])) {
				$def =& $this->fields[$field_name];
				$charset = array_get_default($def, 'charset');
				$len = array_get_default($def, 'len');
				if (isset($def['source']) && $def['source']['type'] != 'db') {
					// no matter
				} else if ($charset == 'ascii' && $len) {
					if (strlen($value) > $len) {
						if($this->auto_trim_values)
							$value = substr($value, 0, $len);
						else {
							$reason = self::ERROR_STRING_TOO_LONG;
							return false;
						}
					}
				} else if ($len) {
					if (mb_strlen($value) > $len) {
						if($this->auto_trim_values)
							$value = mb_substr($value, 0, $len);
						else {
							$reason = self::ERROR_STRING_TOO_LONG;
							return false;
						}
					}
				}
			}
		}
		return true;
	}


	function getInput($unformat=false) {
		if($unformat)
			return $this->input_unformat;
		return $this->input;
	}


	/**
	 * Basically same as getField(), but better support
	 * for null values. Allows to distinguish a field with null value
	 * from field that was neither fetched nor updated (by passing $default
	 * that is not likely to be used as field value, objects of predefined
	 * class work well)
	 */

	function getFieldValue($name, $default = null, $orig = false) {
		if (!$orig && array_key_exists($name, $this->updates))
			return $this->updates[$name];
		if (isset($this->orig_row))
			if (array_key_exists($name, $this->orig_row))
				return $this->orig_row[$name];
		return $default;
	}

	function getField($name, $default = null, $orig = false) {
		if (!$orig && isset($this->updates[$name]))
			return $this->updates[$name];
		if (isset($this->orig_row))
			return array_get_default($this->orig_row, $name, $default);
		return $default;
	}

	function getRow() {
		$row = isset($this->orig_row) ? $this->orig_row : [];
		if(isset($this->updates))
			array_extend($row, $this->updates);
		return $row;
	}

	function getFieldRetrieved($name) {
		if(isset($this->orig_row)) return array_key_exists($name, $this->orig_row);
		return false;
	}

	function getFieldUpdated($name) {
		if(isset($this->updates) && array_key_exists($name, $this->updates)) {
			$val = $this->getField($name, null, true);
			if($val !== $this->updates[$name])
				return true;
		}
		return false;
	}

	function isFieldEmpty($name) {
		$val = $this->getField($name);
		return empty($val);
	}

	function isFieldBlank($name) {
		$val = $this->getField($name);
		return (! isset($val) || $val === '');
	}

	function getChanges($update_meta = false) {
		global $db;
		if ($this->new_updates) {
			$this->cascadeChanges();
		}

		$orig =& $this->orig_row;
		if (!isset($orig))
			$changes = $this->updates;
		else
			$changes = array_diff_assoc($this->updates, $orig);

		// filter unnecessary updates
		if(isset($orig)) {
			foreach($changes as $k => $v) {
				if(isset($orig[$k])) {
					$dbType = $this->getFieldDbType($k);
					// FIXME - better solution would be to format floats coming out of the database
					if($dbType == 'float' || $dbType == 'double') {
						if((float)$orig[$k] == (float)$v)
							unset($changes[$k]);
					}
				}
			}
		}
		// enforce character set
		$dbCharset = $db->getConnectionCharset();
		foreach($changes as $k => $v) {
			$dbType = $this->getFieldDbType($k);
			if($dbType == 'char' || $dbType == 'varchar' || $dbType == 'text' || $dbType == 'mediumtext') {
				$charset = array_get_default($this->fields[$k], 'charset');
				if(! $charset) $charset = 'utf8';
				if($charset == 'ascii') {
					if(! mb_detect_encoding($v ?? '', 'ascii', true)) {
						$changes[$k] = mb_convert_encoding($v ?? '', 'ascii', 'utf-8');
					}
				}
				else if($charset != 'utf8mb4' || $dbCharset != 'utf8mb4') {
					// strip astral characters (not supported)
					$v = mb_convert_encoding($v ?? '', 'utf-8', 'utf-8');
					$v = preg_replace('/[\x{10000}-\x{10FFFF}]/u', "\xEF\xBF\xBD", $v);
					$changes[$k] = $v;
				}
			}
		}

		if ($update_meta) {
			if ($this->new_record && !@$changes['id']) unset($changes['id']);
			$changes = array_merge($this->getMetaChanges(), $changes);
		}
		foreach (array_keys($this->erased) as $f) {
			$changes[$f] = null;
		}

		return $changes;
	}

	function getMetaChanges() {
		global $timedate;

		$changes = $this->getPrimaryKeyValue(true);
		$defaults = array(
			'date_modified' => $timedate->get_gmt_db_datetime(),
			'modified_user_id' => $this->getUserId(),
			'deleted' => 0,
		);
		if(! $this->update_date_modified) {
			unset($defaults['date_modified']);
			unset($defaults['modified_user_id']);
		}
		if ($this->new_record) {
			$defaults['id'] = create_guid();
			if($this->update_date_entered)
				$defaults['date_entered'] = $timedate->get_gmt_db_datetime();
			$defaults['created_by'] = $this->getUserId();
		}

		foreach ($defaults as $k => $v)
			if (isset($this->column_fields[$k]))
				$changes[$k] = $v;
		return $changes;
	}

	function debugGetChanges() {
		$changes = $this->getChanges();
		if ($this->orig_row)
			$original = array_intersect_key($this->orig_row, $changes);
		else
			$original = null;
		$input = array_intersect_key($this->input, $changes);
		return compact('original', 'input', 'changes');
	}


	function cascadeChanges() {
		$this->doCurrencyUpdates();
		$this->doAddressUpdates();
		$this->new_updates = false;
	}

	function doCurrencyUpdates() {
		if (!is_array($this->fields))
			return null;
		foreach ($this->fields as $f => &$fdef) {
			if ($fdef['type'] == 'currency' && array_key_exists($f, $this->updates)) {
				$base = array_get_default($fdef, 'base_field');
				$cid = array_get_default($fdef, 'currency_id');
				$exrt = array_get_default($fdef, 'exchange_rate');
				if ($cid && $exrt) {
					$cid_val = $this->getField($cid);
					if (!$this->new_record && $this->orig_row && !array_key_exists($cid, $this->orig_row))
						// currency_id was not retrieved (should be fetched automatically, however)
						continue;
					if (!$cid_val)
						$cid_val = $this->updates[$cid] = '-99';
					$old_cid = $this->getField($cid, null, true);
					if($old_cid != $cid_val) {
						if(empty($this->updates[$exrt]) || $this->updates[$exrt] === 1) {
							if ($cid_val == '-99') {
								$rate_val = 1.0;
							} else {
								$currency = AppConfig::db_row('Currency', $cid_val);
								if (!$currency) {
									$cid_val = $this->updates[$cid] = '-99';
									$rate_val = 1.0;
								} else {
									$rate_val = $currency['conversion_rate'];
								}
							}
							$this->updates[$exrt] = $rate_val;
						} else
							$rate_val = (float)$this->updates[$exrt];
					} else
						$rate_val = (float)$this->getField($exrt);
					if(! $rate_val) $rate_val = 1;
					if($base) {
						$f_val = $this->getField($f);
						if (!isset($f_val))
							$this->updates[$base] = 0;
						else
							$this->updates[$base] = round((float) $f_val / $rate_val, 5);
					}
				}
			}
		}
	}


	function set($update, $val = null) {
		if (is_string($update))
			$update = array($update => $val);
		if (!is_array($update))
			return false;
		return $this->performSet($update);
	}

	function eraseField($name) {
		if (!AppConfig::setting("model.detail.{$this->model_name}.personal_data"))
			return;
		$def = $this->getFieldDefinition($name);
		if (empty($def['personal_info'])) return;
		$this->erased[$name] = true;
	}

	function performSet(array $update, $as_user=false) {
		foreach ($update as $k => $v) {
			$set = false;
			if(! $as_user)
				$set = true;
			else if($v !== '' || isset($this->orig_row[$k])) {
				if (isset($this->column_fields[$k]) || isset($this->custom_fields[$k]))
					$set = true;
				else if (isset($this->fields[$k]) && (! $as_user || ! empty($this->fields[$k]['updateable'])))
					$set = true;
				else if (isset($this->display_fields[$k]) && (! $as_user || ! empty($this->display_fields[$k]['updateable'])))
					$set = true;
			}
			if($set) {
				if($v === '') {
					$dbType = $this->getFieldDbType($k);
					if($dbType && in_array($dbType, self::$null_empty_string_types)) {
						if(isset($this->orig_row[$k]))
							$v = null;
						else
							continue;
					}
				}
				$this->updates[$k] = $v;
				$this->new_updates = true;
			}
		}
		return true;
	}

	function resetField($name) {
		unset($this->updates[$name]);
	}

	function setRelatedData($key, $value) {
		$this->related_data[$key] = $value;
	}

	function clearRelatedData() {
		$this->related_data[] = [];
	}

	function getRelatedData($key, $default_value=null) {
		return array_get_default($this->related_data, $key, $default_value);
	}

	function updateResult(RowResult &$row_result) {
		if ($this->failed)
			$row_result->addUpdateErrors($this->getErrors());
		if ($this->updates) {
			$changes = $this->getChanges();
			foreach ($changes as $k => $v)
				$row_result->row[$k] = $v;
			$this->customUpdateResult($row_result);

			if ($row_result->secondary_queries) {
				// fetch related values
				$lq = new ListQuery($row_result->base_model, $row_result->fields);
				$lq->populateSecondary($row_result, false);
			}
			// recalculate derived values
			ListQuery::process_result($row_result);

			require_once('include/layout/FieldFormatter.php');
			$fmt = new FieldFormatter($this->input_format, 'editview');
			$fmt->formatRowResult($row_result);
			if (is_array($this->invalid))
				$row_result->formatted = array_merge($row_result->formatted, $this->invalid);
		}
		if($this->new_record)
			$row_result->new_record = true;
	}

	function customUpdateResult(RowResult &$row_result) {
	}

	function copyAddress(RowUpdate &$from, $from_field, $to_field) {
		$base_f = $from->getFieldDefinition($from_field, true);
		$my_f = $this->getFieldDefinition($to_field, true);
		if($base_f && $my_f) {
			$base_pfx = $base_f['source']['prefix'];
			$my_pfx = $my_f['source']['prefix'];
			$copy = array('street', 'city', 'state', 'country', 'postalcode', 'statecode', 'countrycode');
			foreach($copy as $f) {
				$this->set($my_pfx . 'address_' . $f, $from->getField($base_pfx . 'address_' . $f));
			}
		}
	}

	function doAddressUpdates() {
		foreach($this->display_fields as $d) {
			if(isset($d['type']) && $d['type'] == 'address') {
				if(isset($d['source']['prefix']))
					$pfx = $d['source']['prefix'];
				else
					$pfx = '';
				$this->checkAddressField($pfx.'address');
			}
		}
	}

	function checkAddressField($addrname) {
		static $scodes;
		if(! isset($scodes)) {
			$scodes = include('include/language/country_states.php');
		}
		if($this->getFieldUpdated($addrname.'_state') || $this->getFieldUpdated($addrname.'_country')) {
			$cdef = $this->getFieldDefinition($addrname.'_countrycode');
			$ccode = null;
			if($cdef) {
				$country = strtolower(trim($this->getField($addrname.'_country')));
				if($country) {
					$ccodes = AppConfig::setting('lang.lists.current.app.country_codes', array());
					$countries = array();
					foreach($ccodes as $c => $name) {
						if(strtolower($name) == $country) {
							$ccode = $c;
							break;
						}
					}
				}
				$this->set($cdef['name'], $ccode);
			}
			$sdef = $this->getFieldDefinition($addrname.'_statecode');
			if($sdef) {
				$scode = null;
				$state = strtolower(trim($this->getField($addrname.'_state')));
				if($ccode && $state && isset($scodes[$ccode])) {
					foreach($scodes[$ccode] as $c => $name) {
						if(strtolower($name) == $state) {
							$scode = $c;
							break;
						}
					}
				}
				$this->set($sdef['name'], $scode);
			}
		}
	}

	function getLinkUpdate() {
		return $this->link_update;
	}

	function getLinkTargetId() {
		if($this->link_update && $this->link_update_key) {
			return $this->link_update->getField($this->link_update_key);
		}
	}

	function getLinkQuery($link_name, $fields=null, $params=null) {
		if(! is_array($params)) $params = array();
		$params['link_name'] = $link_name;
		$params['parent_key'] = $this->getPrimaryKeyValue();
		$lq = new ListQuery($this->model, $fields, $params);
		return $lq;
	}

	function getLinkResults($link_name, $fields = null, $params = null) {
		$lq = $this->getLinkQuery($link_name, $fields, $params);
		return $lq->fetchAll();
	}

	function getLinkCount($link_name, $distinct=false, $params=null) {
		$lq = $this->getLinkQuery($link_name, null, $params);
		return $lq->fetchResultCount($distinct);
	}

	function getLinkIds($link_name, $ext_params=null) {
		$params = array('auto_add_fields' => false, 'filter_deleted' => false, 'no_default_order_by' => true);
		if($ext_params) array_extend($params, $ext_params);
		$lq = $this->getLinkQuery($link_name, false, $params);
		$j = $lq->getJoinInfo('~join');
		if($j && ! empty($j['self_key'])) {
			$k = $j['self_key'];
			if($lq->addField("~join.$k", $k)) {
				$lq->addSimpleFilter("~join.deleted", 0);
				$result = $lq->fetchAllRows();
				return array_column($result, $k);
			}
		}
		return array();
	}

	function fetchRelated($name, $fields=true, $params=null, $update=false) {
		$def = $this->getFieldDefinition($name);
		if($def && array_get_default($def, 'type') == 'ref' && ! empty($def['id_name']) && ! empty($def['bean_name'])) {
			$id = $this->getField($def['id_name']);
			if($id) {
				$ret = ListQuery::quick_fetch($def['bean_name'], $id, $fields, $params);
				if($ret && $update) return RowUpdate::for_result($ret);
				return $ret;
			}
		}
	}

	function addHook($name, $hook) {
		$this->hooks[$name][] = $hook;
	}

	function clearHooks($name=null) {
		if(isset($name))
			unset($this->hooks[$name]);
		else
			$this->hooks = array();
	}

	function execHooks($name, $params=null, $alt_model=null, $alt_params = null) {
		if (!is_array($alt_params)) $alt_params = $params;
		if($this->model)
			$this->model->execHooks($name, $params, false);
		if($alt_model) {
			$alt_model->execHooks($name, $alt_params, false);
		}
		if($this->hooks && isset($this->hooks[$name])) {
			foreach($this->hooks[$name] as $hook)
				call_hook_function($hook, $this->getModelName(), $params);
		}
	}
}

?>
