<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/


function copy_recursive( $source, $dest ){
    if( is_file( $source ) ){
    	$pd = dirname($dest);
    	if(! file_exists($pd))
    		mkdir($pd, 0777, true);
        return( copy( $source, $dest ) );
    }
    if( !is_dir($dest) ){
        mkdir( $dest );
    }

    $status = true;

	$d = dir( $source );
	if (!$d) return false;
    while( $f = $d->read() ){
        if( $f == "." || $f == ".." ){
            continue;
        }
        $status &= copy_recursive( "$source/$f", "$dest/$f" );
    }
    $d->close();
    return( $status );
}

function mkdir_recursive($path, $check_is_parent_dir = false) {
	$path = clean_path($path);
	if(is_dir($path)) {
		return(true);
	}
	if(is_file($path)) {
		$GLOBALS['log']->error("ERROR: mkdir_recursive(): argument $path is already a file.\n");
		return false;
	}
	if (is_absolute_path($path)) {
		if (strpos($path, clean_path(getcwd()) . '/') !== 0) {
			return false;
		}
	} else {
		$path = clean_path(getcwd()) . '/' . $path;
	}
    return mkdir($path, 0775, true);
}

function is_absolute_path($path) {
	if (strtoupper(substr(PHP_OS, 0, 3)) === 'WIN') {
		$path = strtr( $path, '\\/', '\\\\' );
		// by drive letter: X:\
		if (preg_match( '@^[A-Z]:\\\\@i', $path ) )
			return true;
		// by network paths: \\server_name\public\
		if (preg_match( '@^\\\\\\\\[A-Z]+\\\\[^\\\\]@i', $path ) )
			return true;
	} else {
		if (preg_match('~^(/|[a-z]:[/\\\\])~', $path))
			return true;
    }
    return false;
}

function findAllFiles( $the_dir, $the_array, $include_dirs=false ){


	// longreach - start added - far more efficient version
	findAllFilesUnsorted($the_dir, $the_array, $include_dirs);
	rsort($the_array);
	return $the_array;
	// longreach - end added
	
	
    $d = dir($the_dir);
    while( $f = $d->read() ) {
        if( $f == "." || $f == ".." ){
            continue;
        }

        if( is_dir( "$the_dir/$f" ) ) {
			if($include_dirs) {
				$the_array[] = clean_path("$the_dir/$f");
			}
            $the_array = findAllFiles( "$the_dir/$f/", $the_array, $include_dirs );
        } else {
            $the_array[] = clean_path("$the_dir/$f");
        }


    }
    rsort($the_array);
    
    return $the_array;
}


// longreach - start added
function findAllFilesUnsorted($the_dir, &$the_array, $include_dirs=false) {
	$d = dir($the_dir);
    while( ($f = $d->read()) !== false ) {
        if( $f == "." || $f == ".." ){
            continue;
        }
        $child = clean_path("$the_dir/$f");
        if( is_dir($child) ) {
			if($include_dirs) {
				$the_array[] = $child;
			}
            findAllFilesUnsorted($child, $the_array, $include_dirs);
        } else {
            $the_array[] = $child;
        }
    }
}
// longreach - end added



?>
