<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/
/*********************************************************************************

 * Description:
 * Portions created by SugarCRM are Copyright (C) SugarCRM, Inc. All Rights
 * Reserved. Contributor(s): ______________________________________..
 * *******************************************************************************/

require_once 'include/config/SessionManager.php';
session_set_save_handler(new SessionManager);

if(empty($startTime))
    $startTime = microtime(true);

//define('CONFIG_CACHE_DISABLED', 1);

if (!defined('ONECRM_CSRF_PROTECT')) {
	define('ONECRM_CSRF_PROTECT', 'strict');
}
require_once('include/config/AppConfig.php');

if( ($log_path = AppConfig::setting('site.log.php_log_file')) ) {
	ini_set('log_errors', 1);
	ini_set('error_log', $log_path);
}
if(AppConfig::setting('site.performance.force_display_errors')) {
	error_reporting(E_ALL);
	ini_set('display_errors', 1);
} else {
	if(AppConfig::setting('site.performance.suppress_display_errors')) {
		ini_set('display_errors', 0);
	}
	if(AppConfig::setting('site.performance.suppress_deprecation_warnings')) {
		error_reporting(error_reporting() & ~ constant('E_DEPRECATED'));
	}
	if(AppConfig::setting('site.performance.suppress_strict_warnings')) {
		// hide strictnesss warnings in PHP 5.4+
		error_reporting(error_reporting() & ~ constant('E_STRICT'));
	}
}

// set default timezone
AppConfig::set_system_timezone();

// initialize global logger
$GLOBALS['log'] = AppConfig::create_logger();

// fail here - caller will likely redirect
if(! AppConfig::installed())
	return;

require_once('vendor/autoload.php');
require_once('sugar_version.php'); // provides $sugar_version, $sugar_flavor
require_once('include/utils.php');
require_once('include/utils/CacheManager.php');

if(! defined('DISABLE_CLEAN_INCOMING_DATA') || ! constant('DISABLE_CLEAN_INCOMING_DATA')) {
	clean_special_arguments();
	clean_incoming_data();
}

// set php.ini settings at entry points
setPhpIniSettings();

require_once('include/paths.php');
require_once('include/TimeDate.php');
require_once('include/Localization/Localization.php');
$GLOBALS['timedate'] = new TimeDate();
$GLOBALS['locale'] = new Localization();
$GLOBALS['use_current_user_login'] = false;
$GLOBALS['current_user'] = null;

// IIS does not set this parameter but it's often referenced
if (!isset ($_SERVER['REQUEST_URI'])) {
	$_SERVER['REQUEST_URI'] = '';
}

// disable sendmail if set by environment variable
$sendmail_disabled = ! empty($_ENV['DISABLE_IAH_SENDMAIL']);
if(function_exists('apache_getenv'))
	$sendmail_disabled |= apache_getenv('DISABLE_IAH_SENDMAIL');
AppConfig::set_local('site.sendmail_disabled', $sendmail_disabled);

if(defined('IAH_LIGHTWEIGHT')) {
	// no database connection - run entryInitDatabase to connect later
	return;
}

function entryInitDatabase() {
	require_once ('include/database/PearDatabase.php');
	require_once ('include/database/DBManager.php');
	require_once ('include/database/ListQuery.php');
	require_once ('include/dir_inc.php');
	require_once ('include/utils/misc_functions.php');
	require_once ('modules/ACL/ACLController.php');
	require_once ('modules/Administration/updater_utils.php');
	require_once ('modules/Users/User.php');
	require_once ('modules/Users/authentication/AuthenticationController.php');

	if(! defined('IAH_NO_DATABASE')) {
		try {
			$GLOBALS['db'] =& DBManager::getInstance();
		} catch(IAHDatabaseError $e) {
			$GLOBALS['log']->fatal('Error connecting to database: ' . $e->getMessage());
			sugar_die('Error connecting to database');
		}
	}

	AppConfig::load_module_index();
	$GLOBALS['current_user'] = new User();
}

entryInitDatabase();

?>
