<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');

if (!function_exists("ListView_php_main")) {
	function ListView_php_main() {
		require_once 'include/utils/misc_functions.php';
		global $pageInstance, $forceFlashMessages, $module, $record;
		$forceFlashMessages = true;
		$panel_name = array_get_default($_REQUEST, 'subpanel');
		if(! $panel_name) $panel_name = null;

		if($pageInstance->action == 'index' && ! $panel_name) {
			if (!iml($pageInstance->module))
				indexRedirect(array('path' => 'index.php'));

			$cust_lv = "modules/{$pageInstance->module}/ListView.php";
			if(file_exists($cust_lv)) {
				include($cust_lv);
				return;
			}
		}


		if (!iml($module))
			ACLController::displayNoAccess();

		if($panel_name) {
			if(! ACLController::checkAccess($module, 'view'))
				ACLController::displayNoAccess();
		}
		else if(! ACLController::checkAccess($module, 'list'))
			ACLController::displayNoAccess();

		if(AppConfig::setting('site.performance.defer_load_list') && ! defined('ASYNC_ENTRY')) {
			$keys = array('module', 'action', 'layout', 'report_id', 'list_id', 'list_offset', 'list_limit', 'debug');
			$params = array();
			foreach($keys as $k) {
				if(isset($_REQUEST[$k]))
					$params[$k] = $_REQUEST[$k];
			}
			$json = getJSONobj();
			$p = $params ? $json->encode($params) : '{}';
			$pageInstance->add_js_literal("sListView.deferredLoad($p);", null, LOAD_PRIORITY_FOOT);
			return;
		}

		require_once('include/ListView/ListViewManager.php');
		$context = array_get_default($_REQUEST, 'list_context');
		if(! $context) $context = 'listview';

		$params = array(
			'is_primary' => ($context == 'listview'),
		);

		$manager = ListViewManager::create($context, $params);

		if(AppConfig::setting('site.allow_debug'))
			$manager->debug = array_get_default($_REQUEST, 'debug', false);
		$manager->loadRequest();

		$layout = array_get_default($_REQUEST, 'layout');
		$report = array_get_default($_REQUEST, 'report_id');
		$data_id = array_get_default($_REQUEST, 'data_id');

		if($panel_name) {
			$parent_layout = array_get_default($_REQUEST, 'parent_layout');
			$parent_tab = array_get_default($_REQUEST, 'parent_tab');
			if(! $manager->initSubpanelView($module, $record, $panel_name, $parent_layout, $parent_tab))
				ACLController::displayNoAccess();
			if (!iml($manager->layout_module))
				return '';
		}
		else if(! $manager->initModuleView($module, $layout, null, $report, $data_id)) {
			if(AppConfig::is_admin())
				throw new IAHError('LBL_VIEW_NOT_FOUND');
			else
				ACLController::displayNoAccess();
		}

		return $manager->render();
	}
}

return ListView_php_main();
