// general utility and compatibility methods

var isTouchBrowser = ('createTouch' in document);
var isIPhone = navigator.userAgent.match(/iPhone|iPod|iPad/);

if(! window.console) {
	console = {is_stub: true, log: function() {}, error: function() {}, trace: function() {}};
}

function isNode(e) {
	return e instanceof Node;
}
function isElement(e) {
	return e instanceof Element;
}
function isString(o) {
	return (typeof(o) === 'string' || o instanceof String);
}
function isNumeric(s) {
	return (typeof(s) === 'number') || (typeof(s) === 'string' && (s - 0) == s && s.length);
}
function isArray(o) {
	return Array.isArray(o);
}
function isObject(o) {
	return (typeof(o) === 'object');
}
function isPlainObject(o) {
	if(typeof(o) !== 'object' || o === null) return false;
	var key;
	// find last property name, inherited properties are seen last
    for (key in o) {}
    return ! key || o.hasOwnProperty(key);
}
function isFunction(f) {
	return (typeof(f) === 'function');
}
function isWindow(o) {
	return !! (o === window || (o && o.document && o.location && o.alert && o.setInterval));
}
function isDocument(o) {
	if(o && o.nodeType === 9) return true;
}

if(! window.$) {
	function $(sel) {
		var t = typeof(sel);
		if(t == 'string')
			return document.getElementById(sel);
		else if(sel !== null && t == 'object' && sel.toElement)
			return sel.toElement();
		return sel;
	}
}


function resolveVar(path) {
	var ret, i;
	if(path && typeof(path) == 'string' && path.match(/^[a-zA-Z0-9_\.]+$/)) {
		ret = window;
		path = path.split('.');
		for(i = 0; i < path.length && ret; i++) {
			if(! path[i].length || ! path[i] in ret) {
				ret = undefined;
				break;
			}
			ret = ret[path[i]];
		}
	}
	return ret;
}

// this method is used to duplicate simple objects returned from popups, and leave no dangling references
function deep_clone(obj) {
	if(! isset(obj) || typeof(obj) != 'object')
		return obj;
	if(Array.isArray(obj))
		return obj.map(deep_clone);
	var clone = {};
	for (var property in obj) {
		if (typeof obj[property] == 'object' && obj[property] != null)
			clone[property] = deep_clone(obj[property]);
		else
			clone[property] = obj[property];
	}
	return clone;
}

function isdef(obj) {
	return typeof(obj) != 'undefined';
}
function isset(obj) {
	return (typeof(obj) != 'undefined' && obj !== null) ? true : null;
}
function blank(obj) {
	return !isset(obj) || obj == '';
}
function get_default(obj, def) {
	return isset(obj) ? obj : def;
}

// copy properties from one object to another
function extendObject(base, extend, override, restrict, debug) {
	if(! base || ! extend) return; // FIXME - throw exception?
	if(! isset(override)) override = true;
	var idx, arg, keys;
	if(override instanceof Array) {
		keys = override;
		override = true;
	}
	else {
		keys = [];
		if(restrict instanceof Array) {
			for(idx = 0; idx < restrict.length; idx++) {
				arg = restrict[idx];
				if(arg in extend)
					// hasOwnProperty not checked for named properties
					keys.push(arg);
			}
		} else {
			for(arg in extend)
				if(extend.hasOwnProperty(arg))
					keys.push(arg);
		}
	}
	for(idx = 0; idx < keys.length; idx++) {
		arg = keys[idx];
		if(override === true || ! (arg in base))
			base[arg] = extend[arg];
	}
	return base;
}
// class inheritance
function extendClass(subcls, base, custom) {
	if(! base || ! subcls)
		throw new TypeError("Cannot extend class: not defined");
	subcls.prototype = Object.create(base.prototype);
	subcls.prototype.constructor = subcls;
	subcls.superclass = base.prototype;
	if(custom)
		extendObject(subcls.prototype, custom);
	return subcls;
}
// mixin inheritance
function extendProto(base, subcls, custom, override) {
	if(! base || ! subcls)
		throw new TypeError("Cannot extend class prototype: not defined");
	if(! isset(override)) override = false;
	extendObject(base.prototype, subcls.prototype, override);
	if(custom) extendObject(base.prototype, custom);
}

function setAttrs(elt, attrs) {
	if(! attrs)
		return;
	for(var idx in attrs)
		setAttr(elt, idx, attrs[idx]);
}
function setAttr(elt, name, val) {
	if(isObject(val) && ! isFunction(val) && isset(elt[name]))
		setAttrs(elt[name], val);
	else if(val === null && elt.removeAttribute)
		elt.removeAttribute(name);
	else if(elt.setAttribute && elt.hasAttribute && (elt.hasAttribute(name) || name == 'type' || name == 'class' || name == 'style' || name == 'action' || name == 'target'))
		elt.setAttribute(name, val);
	else if(val !== undefined)
		elt[name] = val;
}
function setStyle(elt, name, val) {
	if(isElement(elt)) elt.style[name] = val;
}

function createElement2(name, attrs, content, parent) {
	var elt = document.createElement(name);
	setAttrs(elt, attrs);
	if(isset(content))
		AppBase.ui.setElementContent(elt, content);
	if(isset(parent))
		parent.appendChild(elt);
	return elt;
}
function nbsp(count) {
	if(!isset(count)) count = 1;
	str = '';
	while(count-- > 0) str += '\u00a0';
	return document.createTextNode(str);
}

function html_escape(str) {
	return (''+str).replace(/&/gi,'&amp;').replace(/</gi,'&lt;').replace(/>/gi,'&gt;').replace(/'/gi,'&#039;').replace(/"/gi,'&quot;');
}
function html_unescape(str) {
	return (''+str).replace(/&lt;/gi,'<').replace(/&gt;/gi,'>').replace(/&#0?39;/gi,'\'').replace(/&quot;/gi,'"').replace(/&amp;/gi,'&');
}

function encodeQueryString(params, prefix, joiner) {
	var ret = prefix || '';
	if(typeof(params) == 'string')
		return ret + params;
	if(params) {
		var args = [];
		for(var argName in params) {
			if(! params.hasOwnProperty(argName) || ! isset(params[argName]))
				continue;
			var argValue = encodeURIComponent(params[argName]);
			args.push(encodeURIComponent(argName) + '=' + argValue);
		}
		ret += args.join(joiner || '&');
	}
	return ret;
}
function parseQueryString(arg, joiner) {
	var ret = {};
	var params = arg.split(joiner || '&');
	for(var i = 0; i < params.length; i++) {
		var pair = params[i].split('=', 2);
		ret[decodeURIComponent(pair[0])] = decodeURIComponent(pair[1]);
	}
	return ret;
}

// simple default parameter for array filter
function identityFn(v) {
	return v;
}
// default sort method for numeric arrays
function cmpFn(a, b) {
	return a - b;
}
// default sort method for string arrays
function cmpStringFn(a, b) {
	if(typeof(a) === 'string')
		return a.localeCompare(b);
	return 0;
}
// sort method for ordering objects by a given property
function cmpProperty(prop, cmp) {
	return function(a, b) {
		return cmp(a[prop], b[prop]);
	}
}
// trim which supports null and non-string values
function trim(s) {
	if(! isset(s)) return '';
	return s.toString().trim();
}

function app_string(str) {
	// default implementation before sugar_3.js is included
	if(window.strings) return window.strings[str] || str ;
	return str;
}

function defaultFocus() {
	var test = $('defaultFocus');
	if(test) return test.focus();
	test = document.getElementsByClassName('default-focus');
	if(test.length) return test[0].focus();
}

function viewportSize() {
	return {
		width: window.innerWidth,
		height: window.innerHeight
	};
}

function onLoadPage(fn) { // used by login currently, misleading name
	AppBase.onDomReady(fn);
}

// escapes regular expression characters
if(! RegExp.escape)
	(function() {
		var sREpat = '[' + '/.*+?|()\\[]{}'.replace(/./g, '\\$&') + ']',
			sRE = new RegExp(sREpat, 'g');
		RegExp.escape = function(text) {
			return text.replace(sRE, '\\$&');
		}
	})();
// polyfills
if(! Array.flatten)
	Array.flatten = function(arr) {
		var ret = [], i = 0;
		for(; i < arr.length; i++) {
			if(Array.isArray(arr[i]))
				ret.pushArray(arr[i]);
			else
				ret.push(arr[i]);
		}
		return ret;
	}
if(! Array.prototype.pushArray)
	Array.prototype.pushArray = function(snd) {
		for(var i = 0; i < snd.length; i++)
			this.push(snd[i]);
		return this;
	}
if(! Array.prototype.unique)
	Array.prototype.unique = function(cmp) {
		var ret = [], i, j;
		for(i = 0; i < this.length; i++) {
			if(cmp) {
				for(j = 0; j < ret.length; j++)
					if(cmp(ret[j], this[i]))
						break;
				if(j == ret.length)
					ret.push(this[i]);
			} else {
				if(i === 0 || ret.indexOf(this[i]) < 0)
					ret.push(this[i]);
			}
		}
		return ret;
	}
if(! Array.prototype.remove)
	Array.prototype.remove = function(val) {
		var ret = 0;
		for(var i = 0; i < this.length; i++)
			if(this[i] === val) {
				this.splice(i --, 1);
				ret ++;
			}
		return ret;
	}
if(! Array.prototype.clear)
	Array.prototype.clear = function(val) {
		this.splice(0, this.length);
	}
if(! Array.prototype.filter) // JS 1.6
	Array.prototype.filter = function(cmp) {
		var ret = [], i;
		for(i = 0; i < this.length; i++) {
			if(cmp(this[i]))
				ret.push(this[i]);
		}
		return ret;
	}
if(! Array.prototype.map) // JS 1.6
	Array.prototype.map = function(fn, thisp) {
		var ret = [], i = 0;
		for(i = 0; i < this.length; i++)
			ret.push(fn.call(thisp, this[i], i, this));
		return ret;
	}
if(! Array.prototype.forEach) // JS 1.6
	Array.prototype.forEach = function(fn, thisp) {
		for(var i = 0; i < this.length; i++)
			fn.call(thisp, this[i], i, this);
	}
if(! Array.prototype.indexOf) // JS 1.6
	Array.prototype.indexOf = function(val) {
		for(var i = 0; i < this.length; i++)
			if(this[i] === val) return i;
		return -1;
	}
if(! Array.prototype.lastIndexOf) // JS 1.6
	Array.prototype.lastIndexOf = function(val) {
		for(var i = this.length-1; i >= 0; i--)
			if(this[i] === val) return i;
		return -1;
	}
if(! Array.prototype.getIndexes) {
	Array.prototype.getIndexes = function() {
		var ret = [], i = 0;
		for(; i < this.length; i++) ret.push(i);
		return ret;
	}
}
if(! Array.prototype.compareTo) {
	Array.prototype.compareTo = function(arr) {
		if(this.length != arr.length) return false;
		for(var i = 0; i < this.length; i++) {
			if(this[i] !== arr[i]) return false;
		}
		return true;
	}
}
if(! Object.assign) {
	Object.assign = function(target, args) {
		var idx, next;
		target = Object(target);
		for(idx = 1; idx < arguments.length; idx++) {
			next = arguments[idx];
			if(isset(next))
				extendObject(target, next);
		}
		return target;
	}
}
if(! Array.range) {
	Array.range = function(a, b, skip) {
		if(a === undefined) { a = 0; }
		if(b === undefined) { b = a; a = 0; }
		if(skip === undefined) { skip = a > b ? -1 : 1; }
		var ret = [];
		if(skip) {
			do {
				if(skip > 0 ? a >= b : a <= b) break;
				ret.push(a);
				a += skip;
			} while(1);
		}
		return ret;
	}
}
// supports negative numbers as opposed to % operator
function modulo(n, m) {
	return ((n % m) + m) % m;
}
if(! Math.sign) {
	Math.sign = function(v) {
		if(isNaN(v))
			return NaN;
		if(v === -0)
			return -0;
		else if(v === 0)
			return 0;
		else if(v > 0)
			return 1;
		else if(v < 0)
			return -1;
		return Math.sign(parseFloat(v));
	}
}
if(! String.prototype.startsWith) {
	String.prototype.startsWith = function(search, pos) {
		return this.substr(!pos || pos < 0 ? 0 : +pos, search.length) === search;
	}
}


function DomState(App) {
	this.dom_ready = false;
	this.dom_ready_hooks = [];
	this.dom_loaded = false;
	this.dom_load_hooks = [];
	
	this.onInit(function() {
		var wrap = App.dom.extNode;
		wrap(document).on('DOMContentLoaded', this.domReadyHandler.bind(this));
		wrap(window).on('load', this.domLoadHandler.bind(this));
		
		var st = document.readyState, i;
		if(st == 'loaded' || st == 'complete') {
			this.domReadyHandler();
			if(st == 'complete') this.domLoadHandler();
		}
	});
	
	this.onDomReady = function(hook) {
		this.dom_ready_hooks.push(hook);
		if(this.dom_ready) hook.call(this);		
	}
	this.domReadyHandler = function() {
		if(this.dom_ready) return;
		for(var i = 0; i < this.dom_ready_hooks.length; i++)
			this.dom_ready_hooks[i].call(this);
		this.dom_ready = true;
	}
	this.onDomLoad = function(hook) {
		this.dom_load_hooks.push(hook);
		if(this.dom_loaded) hook.call(this);
	}
	this.domLoadHandler = function() {
		if(this.dom_loaded) return;
		for(var i = 0; i < this.dom_load_hooks.length; i++)
			this.dom_load_hooks[i].call(this);
		this.dom_loaded = true;
	}
}


function AppModule(name, parent) {
	this.module_name = name;
	this.module_parent = parent || this;
	this.module_app = this.module_parent.module_app || this;
	this.module_init = [];
	this.module_load = [];
	this.module_requires = {};
	return this;
}
AppModule.prototype = {
	onLoad: function(hook) {
		this.module_load.push(hook);
		if(this.loaded && ! this.module_extending) hook.call(this);
		return this;
	},
	onInit: function(hook) {
		this.module_init.push(hook);
		if(this.inited && ! this.module_extending) hook.call(this);
		return this;
	},
	init: function() {
		if(arguments.length > 0)
			this.extendBase.apply(this, Array.prototype.slice.call(arguments));
		this.moduleCheckInit();
		return this;
	},
	moduleCheckInit: function() {
		var k, m, fail;
		if(this.inited) return;
		for(k in this.module_requires) {
			if(this.module_requires[k]) continue;
			m = this.module_parent.module(k);
			if(m && ! m.inited) {
				if(this.module_requires[k] !== false) {
					this.module_requires[k] = false;
					m.onLoad(this.moduleRequireLoaded.bind(this, k));
				}
				fail = true;
			}
			else if(! m)
				fail = true;
		}
		if(fail) return false;
		this.inited = true;
		this.moduleRunHooks(this.module_init);
		this.loaded = true;
		this.moduleRunHooks(this.module_load);
		return this;
	},
	moduleRunHooks: function(hooks, from) {
		for(i = from || 0; i < hooks.length; i++)
			hooks[i].call(this);
	},
	moduleRequireLoaded: function(name) {
		var m = this.module_app.module(name);
		if(m && m.inited) {
			this.module_requires[name] = true;
			this.moduleCheckInit();
		}
		return this;
	},
	onDomReady: function(hook) {
		var nm = this.module_name;
		this.module_parent.onDomReady(function() {
			// look up module again in case it was replaced (could be an alias)
			this.module(nm).onInit(hook);
		});
		return this;
	},
	onDomLoad: function(hook) {
		var nm = this.module_name;
		this.module_parent.onDomLoad(function() {
			this.module(nm).onInit(hook);
		});
		return this;
	},
	extendBase: function() {
		var ctor, hooks = [], i,
			ilen = this.module_init.length, llen = this.module_load.length;
		this.module_extending = true;
		for(i = 0; i < arguments.length; i++) {
			ctor = arguments[i];
			if(typeof(ctor) == 'function') {
				ctor = ctor.call(this, this.module_app);
				// constructor function may return an object
				if(ctor) extendObject(this, ctor);
			}
			else if(ctor)
				extendObject(this, ctor);
		}
		delete this.module_extending;
		if(this.inited)
			this.moduleRunHooks(this.module_init, ilen);
		if(this.loaded)
			this.moduleRunHooks(this.module_load, llen);		
		return this;
	},
	extendModule: function(name, alias) {
		var m = this.module(name), init;
		if(m) {
			init = Array.prototype.slice.call(arguments, 2);
			//m.extendBase.apply(m, init);
			m.onInit(function() {
				if(init.length)
					this.extendBase.apply(this, init);
				this.moduleInitAlias(alias);
			});
		}
		return m;
	},
	moduleAlias: function(name) {
		return this.moduleInitAlias(name);
	},
	moduleExists: function(name) {
		return (name in this && this[name] instanceof AppModule);
	},
	moduleLoaded: function(name) {
		return this.moduleExists(name) && this[name].inited;
	},
	module: function(name) {
		if(! this.moduleExists(name)) {
			if(name in this)
				throw new Error('invalid module name: '+name); // don't clobber other attributes
			this[name] = new AppModule(name, this.module_app || this);
		}
		return this[name];
	},
	moduleParent: function() {
		if(this.module_parent !== this) return this.module_parent;
	},
	moduleRequires: function(name) {
		var i, m, name, chk;
		for(i = 0; i < arguments.length; i++) {
			name = arguments[i];
			m = this.module_parent.module(name);
			chk = (m && m.inited ? true : null);
			this.module_requires[name] = chk;
		}
		return this;
	},
	moduleInitAlias: function(name) {
		var i, m, ihooks = [], lhooks = [];
		if(this.module_parent.moduleExists(name)) {
			m = this.module_parent[name];
			if(m !== this) {
				if(! m.inited) ihooks = m.module_init;
				if(! m.loaded) lhooks = m.module_load;
			}
		}
		this.module_parent[name] = this;
		// reattach any hooks previously added under this name
		for(i = 0; i < ihooks.length; i++)
			this.onInit(ihooks[i]);
		for(i = 0; i < lhooks.length; i++)
			this.onLoad(lhooks[i]);
		return this;
	}
};

AppBase = new AppModule();
AppBase.moduleRequires('base').init(DomState);

AppBase.module('browser').init(function(App) {
	var Caps = this;
	
	this.onInit(function() {
		Caps.mac = !! navigator.platform.match(/(Mac|iPhone|iPod|iPad)/i);
		Caps.windows = navigator.platform.substring(0, 3) === 'Win';
		
		var testElt = document.createElement('_');
	
		// check support for options parameter to addEventListener and for passive events
		Caps.eventListenerOptions = false;
		try {
		  var opts = Object.defineProperty({}, 'passive', {
			get: function() {
			  Caps.eventListenerOptions = true;
			}
		  });
		  window.addEventListener('test', null, opts);
		  window.removeEventListener('test', null, opts);
		} catch (e) {}

		// check for Element.classList support and add second parameter to toggle if necessary
		Caps.elementClassList = ('classList' in Element.prototype);
		if(Caps.elementClassList) {
			testElt.classList.toggle('test', true);
			if(!testElt.classList.contains('test')) {
				var _toggle = DOMTokenList.prototype.toggle;
				DOMTokenList.prototype.toggle = function(token, force) {
					if(force !== undefined && !this.contains(token) === !force) {
						return force;
					}
				}
				return _toggle.call(this, token);
			};
		}
	
		// find method for matching selectors
		Caps.selectorMatchMethod = null;
		var testMethod = ['matches', 'mozMatchesSelector', 'msMatchesSelector', 'oMatchesSelector', 'webkitMatchesSelector'];
		for(var i = 0; i < testMethod.length; i++)
			if(testMethod[i] in Element.prototype)
				Caps.selectorMatchMethod = testMethod[i];
		
		// find style property for CSS transformation
		var test = ['transform', 'webkitTransform', 'mozTransform'];
		Caps.transformProperty = '';
		for(var i = 0; i < test.length; i++) {
			if(testElt.style[test[i]] !== undefined) {
				Caps.transformProperty = test[i];
				break;
			}
		}
	
		// find style property for CSS transition
		var test = ['transition', 'webkitTransition', 'mozTransition'];
		Caps.transitionProperty = '';
		for(var i = 0; i < test.length; i++) {
			if(testElt.style[test[i]] !== undefined) {
				Caps.transitionProperty = test[i];
				break;
			}
		}
		
		// check for CSS animation properties
		var testPfx = ['', 'webkit', 'Moz'],
			animStart = 'animationstart',
			animIter = 'animationiteration',
			animEnd = 'animationend',
			animName = 'animationName',
			animFound = null;
		for(var i = 0; i < testPfx.length; i++) {
			if(testPfx[i]) animName = testPfx[i] + 'AnimationName';
			if(animName in testElt.style) {
				animFound = {supported: true, prefix: testPfx[i], name: animName};
				break;
			}
		}
		if(animFound) {
			if(! animFound.prefix || animFound.prefix === 'Moz') {
				animFound.start = 'animationstart';
				animFound.iter = 'animationiteration';
				animFound.end = 'animationend';
			} else {
				animFound.start = animFound.prefix + 'AnimationStart';
				animFound.iter = animFound.prefix + 'AnimationIteration';
				animFound.end = animFound.prefix + 'AnimationEnd';
			}
		}
		Caps.animation = animFound || {supported: false};
		
		Caps.insetSizes = {left: 0, right: 0, top: 0, bottom: 0};
		function testInsets() {
			var div = document.createElement('div'),
				s = div.style,
				sizes = Caps.insetSizes;
			s.paddingLeft = 'constant(safe-area-inset-left)';
			s.paddingRight = 'constant(safe-area-inset-right)';
			s.paddingTop = 'constant(safe-area-inset-top)';
			s.paddingBottom = 'constant(safe-area-inset-bottom)';
			document.body.appendChild(div);
			var cstyle = window.getComputedStyle(div);
			sizes.left = parseInt(cstyle.paddingLeft, 10) || 0;
			sizes.right = parseInt(cstyle.paddingRight, 10) || 0;
			sizes.top = parseInt(cstyle.paddingTop, 10) || 0;
			sizes.bottom = parseInt(cstyle.paddingBottom, 10) || 0;
			document.body.removeChild(div);
		}
		AppBase.onDomReady(testInsets);
		window.addEventListener('orientationchange', function() { requestAnimationFrame(testInsets); });
	});
});


AppBase.module('base')
.moduleRequires('browser')
.moduleAlias('event') // to be split up
.moduleAlias('dom')
.init(function(App) {

	var Base = this,
		Dom = this,
		Evt = this;

	/*
	Design notes:

	For DOM Nodes, attached event handlers and listeners are stored as properties on
	the Node itself itself instead of in a separate cache like jQuery. It seems
	that jQuery uses that method because complex properties would cause memory
	leaks when elements were removed from the document in IE versions before 9.

	Similarly, jQuery does not support useCapture on events as that was only
	implemented for IE in version 9, but this framework does support capturing events.
	Event listeners may also be marked as passive for improved performance when handling
	scroll events, for example.

	CSS Selectors are limited to the subset implemented by the browser, we don't
	evaluate those manually.

	While jQuery supports namespaces for identifying event handlers, this framework
	adds priorities to ensure that important handlers are executed first.

	Event namespace rules:
		click.x.y responds to click, click.x, click.y, click.x.y, click.y.x and nothing else
		event.ns is set to the matched part of the trigger with components sorted alphabetically
	*/
	
	// TODO - watch html5 online, offline events?
	// async request, web socket as components?

	Base.dataNamespace = '_1crm';

	Evt.keyAliases = {
		backspace: 8,
		'delete': 46,
		down: 40,
		enter: 13,
		left: 37,
		right: 39,
		space: 32,
		tab: 9,
		up: 39
	};

	Evt.priorityLevels = {
		debug: 20,
		custom: 18,		// user/site custom override behaviour
		override: 12,	// higher-priority layout hooks
		ext: 10,		// app replacing default behaviour
		field: 8,		// input field default behaviour
		app: 6,			// app default behaviour
		layout: 4,		// normal layout hooks
		'default': 0
	};
	Evt.getPriorityLevel = function(prio) {
		if(typeof(prio) === 'string') {
			if(prio in Evt.priorityLevels)
				return Evt.priorityLevels[prio];
			// throw error?
		}
		else if(typeof(prio) === 'number')
			return prio;
		return Evt.priorityLevels['default'];
	}

	Evt.compareByPriority = function(a, b) {
		if(a.priority != b.priority) return b.priority - a.priority;
		if(a.order != b.order) return a.order - b.order;
	}

	Dom.splitSelectors = function(type) {
		var ret = [], prefix = '', t, ts, p;
		if(typeof(type) === 'string') {
			ts = type.split(',');
			for(var i = 0; i < ts.length; i++) {
				t = ts[i].trim();
				if(! t.length) continue;
				// don't split on commas inside [] or ()
				if(t.lastIndexOf('[') > t.lastIndexOf(']') || t.lastIndexOf('(') > t.lastIndexOf(')')) {
					prefix += t + ',';
				}
				else {
					ret.push(prefix + t);
					prefix = '';
				}
			}
		}
		if(prefix.length) {
			console.error("Error parsing event type, unclosed brackets", type);
			return false;
		}
		return ret;
	}

	Dom.parseSimpleSelector = function(sel) {
		var ret = {}, p, m, ext;
		if(typeof(sel) === 'string') {
			while( (m = sel.match(/(.*)\[(.*)\](.*)/)) ) {
				var fs = m[2].split(','), fname, fval;
				for(var i = 0; i < fs.length; i++) {
					fname = fs[i].trim();
					if(! fname.length) continue;
					if(! ret.attrs) ret.attrs = {};
					p = fname.indexOf('=');
					if(p > 0) {
						fval = fname.substring(p+1);
						if(fval === 'true') fval = true;
						ret.attrs[fname.substring(0, p)] = fval;
					} else {
						ret.attrs[fname] = true;
					}
				}
				sel = m[1] + m[3];
			}
			while( (m = sel.match(/(.*)([#\.!])([a-zA-Z0-9\-_]*)(.*)/)) ) {
				ext = m[3].trim();
				if(ext.length) {
					if(m[2] === '#') {
						ret.id = ext;
					} else if(m[2] === '.') {
						if(ret.className) ret.className += '.' + ext;
						else ret.className = ext;
					}
					else {
						ret.priority = ext;
					}
				}
				sel = m[1] + m[4];
			}
			while( (p = sel.lastIndexOf(':')) >= 0) {
				if(! ret.params) ret.params = {};
				ext = sel.substring(p+1).trim();
				sel = sel.substring(0, p);
				m = ext.match(/(.+)\((.*)\)/);
				if(m)
					ret.params[m[1]] = m[2]; // debounce(100), throttle, delay etc
				else
					ret.params[ext] = true; // stop, prevent, once, passive, capture, etc
			}
			if(sel) {
				ret.tagName = sel.trim();
				return ret;
			}
		}
	}

	Dom.splitCssClasses = function(cls) {
		if(typeof(cls) === 'string')
			return cls.split(' ').map(trim).filter(identityFn);
		else if(Array.isArray(cls))
			return cls;
		return [];
	}
	
	Evt.fixListenerOptions = function(opts, asObject) {
		if(! opts) opts = asObject ? {} : false;
		else {
			if(typeof(opts) === 'object') {
				if(! asObject && ! App.browser.eventListenerOptions)
					opts = opts.capture ? true : false;
			} else if(asObject) {
				opts = opts ? {capture: true} : {};
			}
		}
		return opts;
	}
	
	Dom.adjustRect = function(bounds, left, top, width, height) {
		bounds.left += left;
		bounds.top += top;
		bounds.width += width;
		bounds.height += height;
		bounds.x = bounds.left;
		bounds.y = bounds.top;
		bounds.right = bounds.left + bounds.width;
		bounds.bottom = bounds.top + bounds.height;
	}
	
	Dom.formatSize = function(size, def_unit) {
		if(isset(size)) {
			var m = (''+size).match(/^(-?[0-9]*\.?[0-9]+|[0-9]+\.[0-9]*)\s*(px|em|ex|pt|%)?$/);
			if(m) {
				return m[1] + (m[2] || def_unit || 'px');
			}
		}
	}
	
	// match a node against a selector
	// accepts a CSS selector string, function, Node, ExtNode, or ExtNodeList
	Dom.matchSelector = function(node, sel, scope, extData) {
		if(node instanceof Node) {
			if(typeof(sel) === 'string') {
				if(App.browser.selectorMatchMethod) {
					try {
						return node[App.browser.selectorMatchMethod](sel);
					} catch(e) { // may throw SyntaxError for unsupported selector
						console.error(e);
						return false;
					}
				}
			} else if(typeof(sel) === 'function') {
				return sel.call(scope, node, extData);
			} else if(sel instanceof Node) {
				return node === sel;
			} else if(sel instanceof Dom.ExtNode) {
				return node === sel._node;
			} else if(sel instanceof Dom.ExtNodeList) {
				return sel.indexOf(node) >= 0;
			}
		} else if(node instanceof Dom.ExtNode)
			return node.matches(sel);
	}
	
	
	// get current event object
	Evt.getEvent = function(evt) {
		if(evt) return Evt.ExtEvent(evt);
		if(window.event) return Evt.ExtEvent(window.event);
	}
	
	// set global drag and drop data
	Evt.setDragData = function(evt, transfer, values, container_elt) {
		var dragData = Evt.globalDragData || {transfer: {}, values: {}};
		if(transfer) {
			for(var k in transfer) {
				if(evt) evt.dataTransfer.setData(k, transfer[k]);
				dragData.transfer[k] = transfer[k];
			}
		}
		if(values && typeof(values) === 'object') {
			extendObject(dragData.values, values);
		}
		if(! dragData.initial_event && evt) dragData.initial_event = evt;
		if(evt) dragData.target_elt = Dom.ExtNode(evt.target);
		if(evt && evt.type == 'dragstart')
			dragData.source_elt = dragData.target_elt;
		if(container_elt)
			dragData.container_elt = Dom.ExtNode(container_elt);
		// may want to track position, container, etc
		Evt.globalDragData = dragData;
	}
	
	// get current global drag and drop data
	Evt.getDragData = function(evt) {
		var gData = Evt.globalDragData,
			is_self = false;
		if(! gData) {
			if(evt) {
				gData = {initial_event: evt, target_elt: evt.currentTarget, transfer: null, values: {}};
				if(evt.dataTransfer.types) {
					gData.transfer = {};
					for(var i = 0; i < evt.dataTransfer.types.length; i++) {
						var t = evt.dataTransfer.types[i];
						gData.transfer[t] = evt.dataTransfer.getData(t);
					}
				}
				// add files etc
			}
		}
		if(evt && evt.target && gData.source_elt) {
			var container = gData.container_elt || gData.source_elt.parentNode;
			if(container && container.contains(evt.target))
				is_self = true;
		}
		if(gData && gData.state !== 'complete')
			gData.self = is_self;
		Evt.globalDragData = gData;
		return gData;
	}
	
	// set the drag and drop state, which is emitted on the source element
	// and stored in the drag data
	Evt.setDragState = function(evt, state, params) {
		var gData = this.getDragData(evt);
		if(gData) {
			gData.state = state;
			gData.state_params = (params = params || {});
			if(gData.source_elt)
				gData.source_elt.emit('dragstate', {value: state, params: params});
		}
	}

	Evt.clearDragData = function(evt) {
		try {
			if(evt) evt.dataTransfer.clearData();
		} catch(e) {
			// not-modifable error, don't care
		}
		delete Evt.globalDragData;
	}
	
	
	/**
		Event wrapper class. Provides standard event attributes
		alongside some custom attributes and methods (cmdKey, position()).
		Properties copied from the EventListener include
			- the capture and passive flags
			- the matched namespace, priority, and selector
			- data attached to the listener
		The event may also be marked 'delayed' if throttle, debounce or delay
		are specified for the listener.
	*/
	Evt.ExtEvent = function(params) {
		if(! (this instanceof Evt.ExtEvent)) {
			var ret = new Evt.ExtEvent();
			if(isset(params))
				ret = ret.initFrom(params);
			return ret;
		}
		this.capture = null;		// triggered in capture phase
		this.currentTarget = null;	// = target unless event is bubbling
		this.data = null;			// data param set on event hook
		this.delayed = false;		// if the event has been delayed by a timer (too late to stop)
		this.ns = '';				// alphabetized, dot-separated namespaces matched by trigger
		this.originalEvent = null;	// native event object
		this.passive = null;		// triggered by event handler with passive option set
		this.priority = null;		// priority set on event hook (custom, default, etc)
		this.result = null;			// value produced by earlier handler, to be returned
		this.selector = null;		// selector used on event listener
		this.target = null;			// element/component producing the event
		this.timeStamp = Date.now();
		this.type = '';				// string identifying the event type
		this.view = null;			// reference to the containing window
		this.which = null;			// normalized keyCode / charCode / button
		this.initFrom(params);
		return this;
	};
	Evt.ExtEvent.initProperties = [
		'capture', 'data', 'delayed', 'ns', 'originalEvent',
		'passive', 'priority', 'result', 'selector', 'timeStamp'
	];
	Evt.ExtEvent.prototype = {
		initFrom: function(params) {
			if(typeof(params) === 'string') {
				this._setType(params);
			}
			else if(params instanceof Event) {
				this._initFromEvent(params);
			}
			else if(params instanceof Evt.ExtEvent) {
				extendObject(this, params);
			}
			else if(isPlainObject(params)) {
				if(params.event) {
					this._initFromEvent(params.event);
					delete params.event;
				}
				else if(params.originalEvent) {
					this._initFromEvent(params.originalEvent);
					delete params.originalEvent;
				}
				if(params.type) {
					this._setType(params.type);
					delete params.type;
				}
				extendObject(this, params, true, Evt.ExtEvent.initProperties);
			}
			else
				return null;
			return this;
		},
		_initFromEvent: function(evt) {
			var k, v;
			for(k in evt) {
				v = evt[k];
				if(Evt.ExtEvent.initProperties.indexOf(k) >= 0
				|| typeof(v) === 'function'
				|| k.toUpperCase() === k) // skip constants
						continue;
				this[k] = v;
			}
			this.originalEvent = evt;
			if(this.eventPhase === Event.CAPTURING_PHASE)
				this.capture = true;
			// key alias for cross-platform modifier key
			// Cmd key on mac, Ctrl otherwise
			// note metaKey is generally the Windows key on Windows/Linux platforms
			this.cmdKey = (this.metaKey && App.browser.mac) || (this.ctrlKey && ! App.browser.mac);
			// copy back to event object for code not using the ExtEvent wrapper..
			evt.cmdKey = this.cmdKey;
			this.which = evt.which || (evt.charCode != null ? evt.charCode : evt.keyCode);
		},
		_setType: function(type) {
			// parse 'type' for priority, filters and ns
			type = Dom.parseSimpleSelector(type);
			if(type) {
				if(type.className)
					this.ns = type.className;
				if(type.priority)
					this.priority = type.priority;
				if(type.tagName)
					this.type = type.tagName;
			}
		},
		isDefaultPrevented: function() {
			var e = this.originalEvent;
			return e.defaultPrevented || this._defaultPrevented;
		},
		isImmediatePropagationStopped: function() {
			return this._immedPropagationStopped;
		},
		isPropagationStopped: function() {
			return this._propagationStopped;	
		},
		stopPropagation: function() {
			if(this.delayed) {
				console.error("Cannot stop propagation on delayed event");
				return;
			}
			var e = this.originalEvent;
			if(e) e.stopPropagation();
			this._propagationStopped = true;
		},
		stopImmediatePropagation: function() {
			if(this.delayed) {
				console.error("Cannot stop propagation on delayed event");
				return;
			}
			this._immedPropagationStopped = true;
		},
		preventDefault: function() {
			if(this.delayed || this.passive) {
				console.error("Cannot prevent default behaviour on delayed or passive event");
				return;
			}
			var e = this.originalEvent;
			if(e) e.preventDefault();
			this._defaultPrevented = true;
		},
		stopEvent: function() {
			if(this.delayed) {
				console.error("Cannot stop delayed event");
				return;
			}
			this.stopPropagation();
			this.preventDefault();
			return false;
		},
		makeDelayed: function() {
			var evt = new Evt.ExtEvent(this);
			evt.delayed = true;
			return evt;
		},
		position: function() {
			return {x: this.pageX, y: this.pageY};
		},
		inside: function(elt) {
			var pos = this.position();
			if (!pos)
				return;
			var x = pos.x, y = pos.y,
				region = SUGAR.ui.getEltRegion(elt);
			return region && x >= region.left && x <= region.right  && y >= region.top && y <= region.bottom;
		}
	};


	Evt.EventListener = function(params) {
		this.attached = false;		// if the listener has been attached to a component
		this.capture = false;		// catch events in the capture phase
		this.data = null;			// data param set on event hook
		this.extEvent = true;		// pass ExtEvent (true) or native event object (false)
		this.extTarget = false;		// wrap current target for use as handler scope
		this.filter = {};			// key, button, which, selector, ctrlKey, shiftKey, ..
		this.handler = null;		// handler called when listener is triggered
		this.ns = '';				// alphabetized, dot-separated namespaces matched by trigger
		this.passive = false;		// promises not to call preventDefault (better performance)
		this.passData = false;		// pass event.data as second argument to handler (old behaviour)
		this.priority = null;		// priority set on event hook (custom, default, etc)
		this.scope = null;			// scope for the event handler
		this.selector = null;		// selector for descendant node (delegate event handler)
		this.type = '';				// string identifying the event type
		this.unq = Evt.EventListener.unq ++;
		this.initFrom(params);
	};
	Evt.EventListener.unq = 0;
	Evt.EventListener.prototype = {
		initFrom: function(params) {
			if(params instanceof Evt.EventListener) {
				extendObject(this, params);
			}
			else if(isPlainObject(params)) {
				if(params.type) {
					this._setType(params.type);
				}
				this._extendParams(params);
			}
			if((this.throttle || this.debounce || this.delay) && this.handler !== false && ! this.prevent)
				this.passive = true; // cannot call preventDefault if handler is delayed
		},
		_extendParams: function(params) {
			extendObject(this, params, true,
				[
					'data', 'extEvent', 'extTarget', 'handler', 'ns',
					'origHandler', 'passData', 'priority', 'scope', 'selector',
					'throttle', 'debounce', 'delay', 'stop', 'prevent'
				]
			);
			if(params.filter)
				this._extendFilter(params.filter);
			if(isdef(params.self))
				this.filter.self = params.self;
		},
		_extendFilter: function(params) {
			if(! this.filter) this.filter = {};
			if(isPlainObject(params)) {
				extendObject(this.filter, params, true,
					['button', 'className', 'cmdKey', 'ctrlKey', 'key', 'metaKey',
					 'self', 'shiftKey', 'tagName', 'which']);
			}
			this._splitFilter('key');
		},
		_splitFilter: function(name) {
			if(! this.filter) return;
			var f = this.filter[name];
			if(typeof(f) === 'string') {
				var fs = f.split('|').map(trim).filter(identityFn);
				this.filter[name] = fs.length ? (fs.length === 1 ? fs[0] : fs) : null;
			}
		},
		_setType: function(type) {
			// parse 'type' for priority, filters and ns
			type = Dom.parseSimpleSelector(type);
			if(type) {
				if(type.className)
					this.ns = type.className;
				if(type.priority)
					this.priority = type.priority;
				if(type.tagName)
					this.type = type.tagName;
				if(type.attrs)
					this._extendFilter(type.attrs);
				if(type.params)
					this._extendParams(type.params);
			}
		},
		_matchFilter: function(evt) {
			var element = evt ? evt.target : null,
				filter = this.filter;
			if(! element || ! filter) {
				// if no target then probably matching for removal
				return true;
			}
			if(isset(filter.button)) {
				var b = '' + filter.button,
					mb = evt.buttons || 0;
				if((b === 'left' || b === '0') && ! (mb & 1))
					return false;
				if((b === 'right' || b === '2') && ! (mb & 2))
					return false;
				if((b === 'middle' || b === '1') && ! (mb & 4))
					return false;
			}
			if(filter.cmdKey && ! evt.cmdKey) {
				return false;
			}
			if(filter.ctrlKey && ! evt.ctrlKey) {
				return false;
			}
			if(filter.className && ! this._matchClass(element, filter.className)) {
				return false;
			}
			if(filter.key) {
				var key = filter.key, found = false;
				if(Array.isArray(key)) {
					for(var i = 0; i < key.length; i++) {
						var k = key[i];
						if(k in Evt.keyAliases) k = Evt.keyAliases[k];
						if(k && evt.which == k) {
							found = true;
							break;
						}
					}
				} else {
					if(key in Evt.keyAliases) key = Evt.keyAliases[key];
					if(evt.which == key)
						found = true;
				}
				if(! found) return false;
			}
			if(filter.metaKey && ! evt.metaKey) {
				return false;
			}
			if(filter.self && evt.target !== evt.currentTarget) {
				return false;
			}
			if(filter.shiftKey && ! evt.shiftKey) {
				return false;
			}
			if(filter.tagName && ! this._matchTagName(element, filter.tagName)) {
				return false;
			}
			if(isset(filter.which) && evt.which != filter.which) {
				return false;
			}
			return true;
		},
		_matchClass: function(elt, cls) {
			if(! elt) return;
			var match = Dom.splitCssClasses(cls);
			for(var i = 0; i < match.length; i++)
				if(! elt.classList.contains(match[i]))
					return false;
			return true;
		},
		_matchSelector: function(evt) {
			var element = evt ? evt.target : null;
			if(! evt) return;
			if(! element) {
				// matching an event with no target (probably for removal)
				// so check that selectors are identical (may be fn or string)
				if(evt.selector ? evt.selector !== this.selector : this.selector)
					return false;
				return true;
			}
			if(! this.selector)
				return true;
			if(element === evt.currentTarget) {
				// selectors only apply to descendant nodes
				return false;
			}
			if(Dom.matchSelector(element, this.selector, this, evt)) {
				return true;
			}
			return false;
		},
		_matchTagName: function(elt, tag) {
			var eltTag = elt.tagName.toUpperCase();
			if(typeof(tag) === 'string') {
				tag = tag.toUpperCase();
				return (eltTag === tag);
			}
			else if(Array.isArray(tag)) {
				for(var i = 0; i < tag.length; i++)
					if(tag.toUpperCase() === eltTag)
						return true;
				return false;
			}
			else {
				return true;
			}
		},
		_matchNamespace: function(evtNs) {
			if(evtNs) {
				if(! this.ns) return false;
				var ns = evtNs.split('.').map(trim).filter(identityFn),
					myNs = this.ns.split('.'),
					matchNs = [];
				for(var i = 0; i < ns.length; i++) {
					if(! myNs.contains(ns[i])) return false;
					matchNs.push(ns[i]);
				}
				matchNs.sort();
				return matchNs.join('.');
			}
			return '';
		},
		matchEvent: function(evt, handler, params) {
			if(! evt || ! evt.type || (this.type && this.type !== evt.type))
				return false;
			if(handler && handler !== this.handler && handler !== this.origHandler)
				return false;
			// skip capture events in bubble phase and vice versa
			if(isset(evt.capture) && (this.capture ? !evt.capture : evt.capture))
				return false;
			// match passive listeners for removeListener
			if(isset(evt.passive) && (this.passive ? !evt.passive : evt.passive))
				return false;
			// skip events with lower priority than listener
			var level = Evt.getPriorityLevel(this.priority);
			if(level < Evt.getPriorityLevel(evt.priority))
				return false;
			// each event namespace must also be defined by the listener
			var ret = {
				priority: level,
				ns: this._matchNamespace(evt.ns),
				selector: this.selector
			};
			if(ret.ns === false)
				return false;
			// match event listener selector
			if(! this._matchSelector(evt))
				return false;
			// match listener filters (class, tag, key, etc)
			if(! this._matchFilter(evt))
				return false;
			return ret;
		},
		getEventScope: function(evt) {
			if(this.scope) return this.scope;
			if(! evt) return;
			if(! this.extTarget) return evt.currentTarget;
			if(! evt.extCurrentTarget || evt.extCurrentTarget._node !== evt.currentTarget)
				evt.extCurrentTarget = Dom.ExtNode(evt.currentTarget);
			return evt.extCurrentTarget;
		},
		callHandler: function(evt, extraParams) {
			var evtArg = evt, args;
			if(! this.extEvent) evtArg = evt.originalEvent;
			if(this.passData) extraParams = evt.data;
			args = [evtArg];
			if(Array.isArray(extraParams))
				args.pushArray(extraParams);
			else if(extraParams !== undefined)
				args.push(extraParams);
			return this.handler.apply(this.getEventScope(evt), args);
		},
		getDelayedHandler: function(evt, extraParams) {
			var delayEvt = evt.makeDelayed(),
				self = this;
			return function() {
				if(self.attached && self.handler)
					return self.callHandler(delayEvt, extraParams);
			}
		},
		cancelDelayedHandler: function() {
			if(this.handlerTimeout)
				clearTimeout(this.handlerTimeout);
		},
		handleEvent: function(evt, extraParams) {
			var ret;
			if(! this.attached)
				return;
			if(! this.handler) {
				ret = false;
			} else {
				var call = true, now = Date.now(), delayEvt;
				evt.data = this.data;
				if(this.throttle) {
					this.cancelDelayedHandler();
					if(this.lastInvoke && now - this.lastInvoke < this.throttle) {
						this.handlerTimeout = setTimeout(this.getDelayedHandler(evt, extraParams), this.throttle);
						call = false;
					} else {
						this.lastInvoke = now;
					}
				}
				else if(this.debounce) {
					this.cancelDelayedHandler();
					this.handlerTimeout = setTimeout(this.getDelayedHandler(evt, extraParams), this.debounce);
					call = false;
				}
				else if(this.delay) {
					this.handlerTimeout = setTimeout(this.getDelayedHandler(evt, extraParams), this.delay);
					call = false;
				}
			
				if(call) {
					ret = this.callHandler(evt, extraParams);
				}
			}
			if(ret !== undefined) {
				evt.result = ret;
			}
			if(ret === false) {
				evt.stopEvent();
				evt.stopImmediatePropagation();
			}
			else {
				if(this.stop)
					evt.stopPropagation();
				if(this.stop_immed)
					evt.stopImmediatePropagation();
				if(this.prevent)
					evt.preventDefault();
			}
			if(this.once) {
				return true;
			}
		},
		setAttached: function(flag) {
			if(! isset(flag)) flag = true;
			this.attached = !! flag;
		}
	};


	Evt.EventManager = function(target) {
		if(target instanceof Node)
			return Dom.ExtNode(target);
		if(! (this instanceof Evt.EventManager))
			return new Evt.EventManager(target);
		this.target = target;
	}
	Evt.EventManager.prototype = {
		$getListeners: function() {
			if(! this.listeners)
				this.listeners = [];
			return this.listeners;
		},
		$addListener: function(listen) {
			listen.setAttached();
			this.$getListeners().push(listen);
		},
		$removeListener: function(listen) {
			listen.setAttached(false);
			this.$getListeners().remove(listen);
		},
		$clearListeners: function() {
			var listen = this.$getListeners();
			if(listen) {
				for(var i = 0; i < listen.length; i++)
					this.$removeListener(listen[i]);
			}
		},
		$findListeners: function(evt, handler, matchParams) {
			var ret = [],
				src = this.$getListeners(),
				match;
			if(src) {
				if(evt instanceof Evt.ExtEvent) {
					for(var i = 0; i < src.length; i++) {
						match = src[i].matchEvent(evt, handler, matchParams);
						if(match) {
							match.order = i;
							match.listener = src[i];
							ret.push(match);
						}
					}
				}
				else if(Array.isArray(evt)) {
					for(var i = 0; i < evt.length; i++)
						ret.pushArray(this.$findListeners(evt[i], handler, matchParams));
					ret = ret.unique();
				}
			}
			if(ret.length)
				ret.sort(Evt.compareByPriority);
			return ret;
		},
		$handleEvent: function(evt, extraParams) {
			evt = Evt.ExtEvent(evt);
			if(! evt) return;
			// identify listeners, ordered by priority and pre-filtered
			var found = this.$findListeners(evt),
				prev_ns = evt.ns;
			evt.result = undefined;
			if(found.length) {
				for(var i = 0; i < found.length; i++) {
					var match = found[i],
						listen = match.listener;
					evt.listener = listen;
					evt.ns = match.ns;
					evt.selector = match.selector;
					if(listen.handleEvent(evt, extraParams))
						this.$removeListener(listen);
					if(evt._immedPropagationStopped)
						break;
				}
			}
			return evt.result;
		},
	
		/**
		 * Attach a listener. Multiple signatures:
		 * on(event : event.EventListener)
		 * on(eventType : string,
		 *    [selector : string | function],
		 *    [data : object],
		 *    [handler : function | false],
		 *    [params : object])
		 *    eventType may combine multiple event matchers separated by commas
		 * on(eventMap : object)
		 */
		on: function(evt, selector, data, handler, params) {
			var listeners = this.$getListeners(), nl, ret;
			// attach handler (function or event manager)
			if(evt instanceof Evt.EventListener) {
				if(! listeners.contains(evt)) {
					this.$addListener(evt);
				}
				return evt;
			}
			else if(Array.isArray(evt)) {
				for(var i = 0; i < evt.length; i++)
					this.on(evt[i], selector, data, handler, params);
			}
			else if(typeof(evt) === 'string') {
				var types = Dom.splitSelectors(evt);
				if(types && types.length) {
					var init = {}, selType = typeof(selector);
					if(selType === 'function'
					&& (typeof(data) === 'function' || typeof(handler) === 'function'
					|| data === false || handler === false)) {
						// first function arg could be the handler or the selector
						// assume handler if only one is found
						init.selector = selector;
					} else if(selType === 'string') {
						init.selector = selector;
					} else if(selector !== null) {
						params = handler;
						handler = data;
						data = selector;
					}
					if(isPlainObject(data)) {
						// unlike jQuery, data must be a plain object, or omitted
						init.data = data;
						init.handler = handler;
					} else {
						init.handler = data;
						params = handler;
					}
					ret = [];
					for(var i = 0; i < types.length; i++) {
						init.type = types[i];
						nl = new Evt.EventListener(init);
						if(nl) {
							if(isPlainObject(params)) nl.initFrom(params);
							this.$addListener(nl);
							ret.push(nl);
						}
					}
					if(types.length === 1) {
						ret = ret[0];
					}
				}
			}
			else if(isPlainObject(evt)) {
				// add set of listeners (key = evt name, value = handler)
				var args;
				for(var k in evt) {
					// selector is actually optional params
					this.on(k, evt[k], selector);
				}
			}
			else {
				throw new TypeError("First parameter must be a string, array, object, or EventListener");
			}
			return this;
		},
		off: function(evt, selector, handler, params) {
			var found;
			if(evt instanceof Evt.EventListener) {
				this.$removeListener(evt);
			}
			else if(Array.isArray(evt)) {
				for(var i = 0; i < evt.length; i++)
					this.off(evt[i], selector, handler, params);
			}
			else {
				if(typeof(evt) === 'string') {
					var types = Dom.splitSelectors(evt);
					evt = [];
					if(types && types.length) {
						var init = {type: evt}, selType = typeof(selector);
						if(selType === 'function'
						&& (typeof(handler) === 'function' || handler === false)) {
							// first function arg could be the handler or the selector
							// assume handler if only one is found
							init.selector = selector;
						} else if(selType === 'string') {
							init.selector = selector;
						} else if(selector !== null) {
							params = handler;
							handler = selector;
						}
						params = Evt.fixListenerOptions(params, true);
						init.capture = !! params.capture;
						init.passive = !! params.passive;
						for(var i = 0; i < types.length; i++) {
							init.type = types[i];
							nl = new Evt.ExtEvent(init);
							if(nl) evt.push(nl);
						}
					}
				}
				else if(! (evt instanceof Evt.ExtEvent)) {
					throw new TypeError("First parameter must be a string, array, Event or EventListener");
				}

				params = {remove: true};
				found = this.$findListeners(evt, handler, params);
				for(var i = 0; i < found.length; i++)
					this.$removeListener(found[i].listener);
			}
			return this;
		},
		emit: function(evt, extraParams) {
			this.$handleEvent(evt, extraParams);
			return this;
		}	
	};


	Base.Component = function(params) {
		this.$init(params);
	}
	extendProto(Base.Component, Evt.EventManager, {
		$getListeners: function() {
			return this.$getData('_listeners');
		},
	
		// -- Property storage - abstracted to allow components to share data --
	
		$init: function(params) {
			Object.defineProperty(this, '_owner', {value: this, configurable: true});
			this.$initData('_listeners', []);
			if(isPlainObject(params))
				extendObject(this, params);
		},
		$initData: function(name, val) {
			if(! isset(name)) return;
			if(typeof(name) === 'object') {
				for(var k in name)
					this.$initData(k, name[k]);
			}
			else if(this.$getData(name) === undefined) {
				this.$setData(name, val);
			}
		},
		$getData: function(name, no_init) {
			var owner = this._owner,
				prop = Base.dataNamespace;
			if(! owner[prop] && ! no_init) {
				if(owner instanceof Node)
					owner[prop] = {};
				else
					Object.defineProperty(owner, prop, {value: {}, enumerable: false, configurable: true});
			}
			if(name === undefined) {
				return owner[prop];
			}
			return owner[prop] ? owner[prop][name] : null;
		},
		$setData: function(name, value) {
			if(isPlainObject(name)) {
				extendObject(this.$getData(), name);
			} else if(isset(name)) {
				this.$getData()[name] = value;
			}
		},
		$removeData: function(name) {
			var data = this.$getData(undefined, true),
				ret = data[name];
			delete data[name];
			return ret;
		},
		$clearData: function() {
			var owner = this._owner,
				prop = Base.dataNamespace;
			delete owner[prop];
		},
		$defineProperty: function(name, desc, value) {
			if(! name) throw new TypeError("Expected string: property name");
			var mapProp = name;
			if(typeof(desc) === 'string')
				mapProp = desc;
			if(! isPlainObject(desc)) desc = {};
			if('prop' in desc) {
				mapProp = desc.prop;
				delete desc.prop;
			}
			if(value !== undefined) {
				this.$setData(mapProp, value);
			}
			var defDesc = {configurable: true, enumerable: true};
			if(! ('get' in desc))
				defDesc['get'] = function() { return this.$getData(mapProp); }
			if(! ('set' in desc)) {
				var updated = desc.updated;
				delete desc.updated;
				defDesc['set'] = function(val) {
					this.$setData(mapProp, val);
					if(updated) updated.call(this, val, name);
				}
			}
			extendObject(desc, defDesc, false);
			Object.defineProperty(this, name, desc);
		}
		// $defineChild  - to tear down nested components?
	});
	

	/**
	 * A regular DOM node with accessors for adjusting properties and attaching listeners.
	 * Create with a Node argument or (string tagName, attrs, content, parent)
	 **/
	Dom.ExtNode = function(params) {
		if(! (this instanceof Dom.ExtNode)) {
			if(params instanceof Dom.ExtNode)
				return params;
			else if(params instanceof Dom.ExtNodeList)
				return params.first();
			var args = [null].pushArray(arguments),
				ctor = Dom.ExtNode.bind.apply(Dom.ExtNode, args),
				ret = new ctor();
			return ret._node ? ret : null;
		}
		Base.Component.call(this);
		if(params instanceof Node || isWindow(params)) {
			// check second argument for params
			this.$initEl(params);
			params = arguments[1];
		} else if(typeof(params) === 'string' && params.length) {
			try {
				params = document.querySelector(params);
			} catch(e) { // may throw SyntaxError for unsupported selector
				console.error(e);
				return;
			}
			return Dom.ExtNode(params, Array.prototype.slice.call(arguments, 1));
		} else if(params instanceof Dom.ElementSpec) {
			return params.create();
		} else if(params !== null && typeof(params) === 'object' && params.toElement) {
			// create from an object with a toElement method
			return Dom.ExtNode(params.toElement(), Array.prototype.slice.call(arguments, 1));
		}
		if(isPlainObject(params))
			this.$init(params);
	}
	extendClass(Dom.ExtNode, Base.Component, {
		/*
		note: must be an Element (nodeType = 1), DocumentNode (nodeType = 9) or other object
		to store properties. some elements (like canvas) do not allow custom properties
		*/

		$initEl: function(el) {
			var self = this,
				oldData = this.$getData();
			Object.defineProperty(this, '_node', {value: el, configurable: true});
			Object.defineProperty(this, '_owner', {value: el, configurable: true});
			if(this.$getData('_default_handler') === undefined) {
				this.$setData('_default_handler', this.$handleEvent.bind(this));
			}
			this.$defineProperty('_default_handler', '_default_handler');
			this.$initData('_handlers', []);
			this.$initData('_handler_idx', 0);
			this.$initData(oldData);
			Object.defineProperty(this, 'nodeType', {value: el.nodeType, configurable: true});
			Object.defineProperty(this, 'tagName', {value: el.tagName ? el.tagName.toUpperCase() : el.tagName, configurable: true});
			this.$mapEltProperty('parentNode', false);
			if(this.$getData('_focus_tracked')) {
				this.$defineProperty('focused', '_focused');
			}
			if(this.$getData('_hover_tracked')) {
				this.$defineProperty('hovered', '_hovered');
			}
		},
	
		$mapEltProperty: function(name, writable, propName) {
			if(! name) throw new TypeError();
			if(! isset(writable)) writable = true;
			if(! isset(propName)) propName = name;
			var desc = {get: function() { return this._node[propName]; },
						configurable: true, enumerable: true};
			if(writable) {
				desc.set = function(val) { this._node[propName] = val; return this._node[propName]; };		
			}
			Object.defineProperty(this, name, desc);
		},
	
		$addHandler: function(name, handler, options) {
			if(! this._node) return; // may wish to queue events until mounted
			var fn,
				self = this,
				handlers = this.$getData('_handlers'),
				idx = this.$getData('_handler_idx');
			if(this.$findHandler(name, handler, options) !== undefined)
				return;
			options = Evt.fixListenerOptions(options);
			if(! handler) {
				fn = this._default_handler;
			} else {
				fn = function(evt, extraParams) {
					var hook = (typeof(handler) === 'string' ? self[handler].bind(self) : handler);
					var ret;
					if(hook) ret = hook(evt, extraParams);
					if(ret !== false)
						ret = self._default_handler(evt, extraParams);
					return ret;
				}
			}
			this._node.addEventListener(name, fn, options);
			handlers.push({idx: idx, name: name, fn: fn, origFn: handler, options: options});
			this.$setData('_handler_idx', ++ idx);
			return idx;
		},
		$findHandler: function(ident, fn, options) {
			var i, handler,
				handlers = this.$getData('_handlers');
			options = Evt.fixListenerOptions(options, true);
			for(i = 0; i < handlers.length; i++) {
				handler = handlers[i];
				if(handler.idx === ident || (handler.name === ident && handler.origFn === fn)) {
					if(handler.options.capture === options.capture
					&& handler.options.passive === options.passive) {
						return i;
					}
				}
			}
		},
		$removeHandler: function(ident, fn, options) {
			var idx = this.$findHandler(ident, fn, options),
				handler,
				handlers = this.$getData('_handlers');
			if(idx !== undefined) {
				handler = handlers[idx];
				handlers.splice(idx, 1);
				this._node.removeEventListener(handler.name, handler.fn, handler.options);
				return true;
			} else {
				this._node.removeEventListener(ident, fn, options);
			}
		},
		$clearHandlers: function() {
			var handler, handlers = this.$getData('_handlers');
			if(handlers) {
				for(var i = 0; i < handlers.length; i++) {
					handler = handlers[i];
					this._node.removeEventListener(handler.name, handler.fn, handler.options);
				}
				this.$setData('_handlers', []);
			}
		},
	
		// override base $addListener to attach native handlers
		$addListener: function(listen) {
			if(! listen) return;
			if(listen.type === 'focus' || listen.type === 'blur') {
				this.trackFocus();
			}
			else if(listen.type === 'mouseenter' || listen.type === 'mouseleave') {
				this.trackHover();
			}
			this.$addHandler(listen.type, null, {capture: listen.capture, passive: listen.passive});
			Base.Component.prototype.$addListener.call(this, listen);
		},

		// trigger not implemented - use dispatchEvent(new CustomEvent(..)) ?
		// waiting for use cases
	
		$handleFocus: function() {
			this.$setData('_focused', true);
		},
		$handleBlur: function() {
			this.$setData('_focused', false);
		},
		trackFocus: function(flag) {
			if(! isset(flag)) flag = true;
			if(flag && ! this.$getData('_focus_tracked')) {
				this.$defineProperty('focused', '_focused');
				this.$addHandler('focus', '$handleFocus', {passive: true});
				this.$addHandler('blur', '$handleBlur', {passive: true});
				this.$setData('_focus_tracked', true);
			}
			// call automatically if a focus/blur event listener is attached
		},
		$handleMouseEnter: function() {
			this.$setData('_hovered', true);
		},
		$handleMouseLeave: function() {
			this.$setData('_hovered', false);
		},
		trackHover: function(flag) {
			// attach/remove hover hooks
			// call automatically if a mouseover/out event listener is attached
			if(flag && ! this.$getData('_hover_tracked')) {
				this.$defineProperty('hovered', '_hovered');
				this.$addHandler('mouseenter', '$handleMouseEnter', {passive: true});
				this.$addHandler('mouseleave', '$handleMouseLeave', {passive: true});
				this.$setData('_hover_tracked', true);
			}
		},
	
		makeDraggable: function(flag) {
		/* attach handlers
			ondrag	script	Triggers when an element is dragged
			ondragend	script	Triggers at the end of a drag operation
			ondragstart	script	Triggers at the start of a drag operation
		*/
		},
		makeDragTarget: function() {
		/* attach handlers
			ondragenter	script	Triggers when an element has been dragged to a valid drop target
			ondragleave	script	Triggers when an element leaves a valid drop target
			ondragover	script	Triggers when an element is being dragged over a valid drop target
			ondrop	script	Triggers when dragged element is being dropped
		*/
		},
			
		// fetch wrapped element
		get: function() {
			return this._node;
		},
		// check node equality
		equals: function(b) {
			if(! (b instanceof Dom.ExtNode))
				b = Dom.ExtNode(b);
			return b && b.get() === this.get();
		},
		// clone element - attached data and listeners are optional
		clone: function(copyDataAndEvents /*, deepCopyDataAndEvents*/) {
			var ret = new Dom.ExtNode(this._node.cloneNode(true));
			if(copyDataAndEvents) {
				var data = deep_clone(this.$getData()),
					handlers = this.$getData('_handlers');
				delete data.handlers;
				ret.$setData(data);
				if(handlers) {
					for(var i = 0; i < handlers.length; i++) {
						ret.$addHandler(handlers[i].name, handlers[i].origFn, handlers[i].options);
					}
				}
			}
			return ret;
		},
		// append element to a given parent node
		appendTo: function(elt) {
			if(elt) {
				if(elt instanceof Dom.ExtNode)
					elt._node.appendChild(this._node);
				else if(elt instanceof Dom.ExtNodeList) {
					var fst = elt.get(0);
					if(fst) (fst.append || fst.appendChild).bind(fst)(this._node);
				}
				else if(elt instanceof Node)
					elt.appendChild(this._node);
				else if(elt.push)
					elt.push(this._node);
			}
			return this;
		},
		// append content to the element
		append: function(content) {
			for(var i = 0; i < arguments.length; i++) {
				this.$insert(arguments[i]);
			}
			return this;
		},
		// prepend element to a given parent node
		prependTo: function(elt) {
			if(elt) {
				if(elt instanceof Dom.ExtNode)
					elt._node.insertBefore(this._node, elt._node.firstChild);
				else if(elt instanceof Node)
					elt.insertBefore(this._node, elt.firstChild);
				else if(elt.unshift)
					elt.unshift(this._node);
			}
			return this;
		},
		// append content to the element
		prepend: function(content) {
			var first = this._node.firstChild;
			for(var i = 0; i < arguments.length; i++) {
				this.$insert(arguments[i], null, first);
			}
			return this;
		},
		// insert content after this node
		after: function(content) {
			this.$insert(content, this.parentNode, this._node.nextSibling);
			return this;
		},
		// insert content before this node
		before: function(content) {
			this.$insert(content, this.parentNode, this._node);
			return this;
		},
		// insert this node after a given target
		insertAfter: function(elt) {
			if(elt) {
				Dom.ExtNode(elt).after(this._node);
			}
			return this;
		},
		// insert this node before a given target
		insertBefore: function(elt) {
			if(elt) {
				Dom.ExtNode(elt).before(this._node);
			}
			return this;
		},
	
		// utility method for inserting content
		$insert: function(content, parent, beforeElt) {
			if(! beforeElt) beforeElt = null;
			if(! parent) parent = this._node;
			if(parent) {
				// support DocumentFragment?
				if(typeof(content) === 'string') {
					parent.insertBefore(document.createTextNode(content), beforeElt);
				} else if(typeof(content) === 'number') {
					parent.insertBefore(document.createTextNode(content.toString()), beforeElt);
				} else if(content instanceof Dom.ExtNode) {
					parent.insertBefore(content.get(), beforeElt);
				} else if(content instanceof Node) {
					parent.insertBefore(content, beforeElt);
				} else if(content instanceof Dom.ElementSpec) {
					parent.insertBefore(content.toElement({ns: parent.namespaceURI}), beforeElt);
				} else if(Array.isArray(content) || content instanceof Dom.ExtNodeList) {
					for(var i = 0; i < content.length; i++)
						this.$insert(content[i], parent, beforeElt);
				}
			}
			return this;
		},

		// detach from the DOM without removing event handlers
		detach: function() {
			if(this._node.parentNode)
				this._node.parentNode.removeChild(this._node);
			var swap = this.$removeData('_swap');
			if(swap && swap.parentNode)
				swap.parentNode.removeChild(swap);
			return this;
		},
		// detach from DOM and remove handlers and data
		remove: function() {
			this.detach();
			this.$clearHandlers();
			this.$clearData();
			return this;
		},
		// remove handlers and data (including listeners) and reinit
		reinit: function() {
			var el = this._node;
			this.$clearHandlers();
			this.$clearData();
			this.$init();
			this.$initEl(el);
			return this;
		},
		// replace another node with this element
		replace: function(node) {
			if(node instanceof Dom.ExtNode)
				node = node.get();
			if(node && node.parentNode) {
				node.parentNode.replaceChild(this._node, node);
			}
			return this;
		},
		// replace this element with another node (support complex content?)
		replaceWith: function(node) {
			if(node instanceof Dom.ExtNode)
				node = node.get();
			if(this.parentNode && node instanceof Node) {
				this.parentNode.replaceChild(node, this._node);
			}
			return this;
		},
		// re-insert swapped-out node
		swapIn: function() {
			var swap = this.$removeData('_swap');
			if(! this.parentNode && swap && swap.parentNode) {
				swap.parentNode.replaceChild(this._node, swap);
			}
			return this;
		},
		// remove node from DOM leaving a placeholder node
		swapOut: function() {
			var swap = this.$getData('_swap');
			if(this.parentNode) {
				if(! swap) {
					swap = document.createComment('');
					this.$setData('_swap', swap);
				}
				this.replaceWith(swap);
			}
			return this;
		},
		// swap two elements in DOM
		swapWith: function(elt) {
			if(elt) {
				elt = Dom.ExtNode(elt);
				if(elt.parentNode) {
					this.swapOut();
					elt.replaceWith(this);
					elt.replace(this.$removeData('_swap'));
				}
			}
			return this;
		},
		// get document for element (if any)
		document: function() {
			if(isDocument(this._node))
				return this;
			if(this._node.ownerDocument)
				return Dom.ExtNode(this._node.ownerDocument);
			if(isWindow(this._node))
				return Dom.ExtNode(this._node.document);
		},
		// get window for element
		window: function() {
			if(isDocument(this._node))
				return Dom.ExtNode(this._node.defaultView);
			if(this._node.ownerDocument)
				return Dom.ExtNode(this._node.ownerDocument.defaultView);
			if(isWindow(this._node))
				return this;
		},
		// check if this element contains another element
		contains: function(elt) {
			var thisNode = this._node,
				otherNode = (elt instanceof Dom.ExtNode ? elt.get() : elt);
			return thisNode !== otherNode && thisNode.contains(otherNode);
		},
		// check if element is inside a given document (or any document)
		inDocument: function(doc) {
			if(! (this._node instanceof Node)) return false;
			var eltDoc = this._node.ownerDocument;
			if(! eltDoc || (doc && doc !== eltDoc) || ! eltDoc.body.contains(this._node))
				return false;
			return true;
		},
		// check if node is an element
		isElement: function() {
			return this.nodeType === 1;
		},
		// check if node is a window
		isWindow: function() {
			return isWindow(this._node);
		},
	
		// get/set text content
		text: function(text) {
			if(text === undefined) {
				return this._node.textContent;
			} else {
				if(text === null) text = '';
				this._node.textContent = text;
			}
			return this;
		},
		// get/set html content
		html: function(content) {
			if(content === undefined) {
				return this._node.innerHTML;
			} else {
				if(content === null) content = '';
				if(typeof(content) === 'string')
					this._node.innerHTML = content;
				else
					this.content(content);
			}
			return this;
		},
		// get/set node content
		content: function(content) {
			if(content === undefined) {
				return Array.prototype.slice.call(this._node.children);
			} else {
				this.empty();
				this.append(content);
			}
			return this;
		},
		// clear all child nodes
		empty: function() {
			var node = this._node;
			while(node.lastChild) node.removeChild(node.lastChild);
			return this;
		},
		// get/set value of input controls
		// note: different from jQuery in that the checked state is used for checkboxes
		val: function(value) {
			var type;
			if(this.tagName === 'INPUT' || this.tagName === 'TEXTAREA' || this.tagName === 'SELECT') {
				type = this._node.getAttribute('type');
				if(type === 'checkbox' || type === 'radio') {
					if(value === undefined) {
						return this.prop('checked');
					} else {
						this.prop('checked', !!value);
						return this;
					}
				}
			}
			return this.prop('value', value);
		},
	
		// set or remove an attribute on an element
		attr: function(name, value) {
			if(isPlainObject(name)) {
				for(var k in name)
					this.attr(k, name[k]);
			} else if(typeof(name) === 'string') {
				if(value === undefined)
					return this._node.getAttribute(name);
				else
					this._node.setAttribute(name, value);
			} else {
				throw new TypeError();
			}
			return this;
		},
		removeAttr: function(name) {
			this._node.removeAttribute(name);
			return this;
		},
	
		// set or remove a property on an element
		prop: function(name, value) {
			if(isPlainObject(name)) {
				for(var k in name)
					this.prop(k, name[k]);
			} else if(typeof(name) === 'string') {
				if(value === undefined)
					return this._node[name];
				else
					this._node[name] = value;
			} else {
				throw new TypeError();
			}
			return this;
		},
		removeProp: function(name) {
			try {
				this._node[name] = undefined;
				delete this._node[name];
			} catch(e) {
				console.error(e);
			}
			return this;
		},
	
		// assuming Element.classList support (IE 10 + possible shims for functionality)
		// not assuming multiple argument support for classList.add/remove at the moment
		addClass: function(cls) {
			var lst = this._node.classList;
			if(lst && cls) {
				cls = Dom.splitCssClasses(cls);
				for(var i = 0; i < cls.length; i++)
					lst.add(cls[i]);
			}
			return this;
		},
		removeClass: function(cls) {
			var lst = this._node.classList;
			if(lst && cls) {
				cls = Dom.splitCssClasses(cls);
				for(var i = 0; i < cls.length; i++)
					lst.remove(cls[i]);
			}
			return this;
		},
		hasClass: function(cls) {
			var lst = this._node.classList;
			if(lst && cls) {
				cls = Dom.splitCssClasses(cls);
				for(var i = 0; i < cls.length; i++)
					if(! lst.contains(cls[i])) return false;
				return true;
			}
			return false;
		},
		toggleClass: function(cls, force) {
			var lst = this._node.classList;
			if(lst && cls) {
				cls = Dom.splitCssClasses(cls);
				for(var i = 0; i < cls.length; i++)
					lst.toggle(cls[i], force);
			}
			return this;
		},
	
		// get/set CSS properties
		css: function(name, value) {
			if(isPlainObject(name)) {
				for(var k in name)
					this.css(k, name[k]);
			} else if(typeof(name) === 'string') {
				if(! this._node.style)
					;
				else if(value === undefined)
					return getComputedStyle(this._node).getPropertyValue(name);
				else if(value === null)
					this._node.style.removeProperty(name);
				else
					this._node.style[name] = value;
			} else if(Array.isArray(name)) {
				var ret = [];
				for(var i = 0; i < name.length; i++)
					ret.push(this.css(name[i]));
				return ret;
			} else {
				throw new TypeError();
			}
			return this;
		},
		// like css() but accepts and returns numeric values (pixels)
		cssSize: function(name, val) {
			if(isPlainObject(name)) {
				for(var k in name)
					this.cssSize(k, name[k]);
			} else if(val === undefined) {
				var ret = parseFloat(this.css(name));
				if(typeof(ret) !== 'number' || isNaN(ret)) ret = null;
				return ret;
			} else {
				this.css(name, val === null ? val : Dom.formatSize(val));
			}
			return this;
		},
		
		// fetch parent, skipping over table / table cell / body elements
		// that might be returned otherwise
		offsetParent: function() {
			var ret = Dom.ExtNode(this._node.offsetParent);
			if(ret && ret.css('position') === 'static') {
				// skip over table / table cell / body returned as offset parent
				ret = ret.offsetParent();
			}
			if(! ret && this._node)
				ret = Dom.ExtNode(this._node.ownerDocument.documentElement);
			return ret;
		},
		// fetch or set position relative to the document
		offset: function(val) {
			return this.offsetFrom(this.document(), val);
		},
		// fetch or set position relative to another element
		offsetFrom: function(other, val) {
			if(! other)
				// offset from offset parent
				other = this.offsetParent();
			else if(other === true)
				// offset from current position, not useful without a new value
				other = this;
			var offset = this.bounds(other), posStyle, pos, newPos;
			if(val === undefined || val === null) {
				if(! offset) return; // ??
				return {left: offset.left, top: offset.top};
			} else if(offset) {
				posStyle = this.css('position');
				if(posStyle === 'static') {
					this.css('position', (posStyle = 'relative'));
				}
				pos = {
					left: this.cssSize('left') || 0,
					top: this.cssSize('top') || 0
				};
				if(typeof(val) === 'function') {
					newPos = val(pos);
				} else {
					newPos = {left: pos.left, top: pos.top};
					if('left' in val)
						newPos.left += val.left - offset.left;
					if('top' in val)
						newPos.top += val.top - offset.top;
				}
				if(isPlainObject(newPos)) {
					if('left' in newPos)
						this.cssSize('left', newPos.left);
					if('top' in newPos)
						this.cssSize('top', newPos.top);
				}
			}
			return this;
		},
		// fetch position relative to offset parent, not settable
		position: function(val) {
			if(val === undefined) {
				if(isWindow(this._node)) {
					return {left: 0-window.pageXOffset, top: 0-window.pageYOffset};
				} else if(isDocument(this._node)) {
					return {left: 0, top: 0};
				} else {
					return {left: this._node.offsetLeft, top: this._node.offsetTop};
				}
			} else {
				return this.offsetFrom(this.offsetParent(), val);
			}
		},
		// fetch outer bounds rectangle relative to the document (default), viewport, or another element
		bounds: function(relative, scrollRelative) {
			var rect = {left:0, top: 0, width: 0, height: 0, bottom: 0, right: 0}, ob;
			if(relative) {
				// true means relative to viewport (window)
				if(relative !== true) {
					var relQ = Dom.query(relative);
					ob = relQ.bounds(true); // change to offset
					if(! ob) {
						// return position relative to document
						relative = false;
					}
					else if(scrollRelative) {
						Dom.adjustRect(ob, 0-relQ.scrollLeft(), 0-relQ.scrollTop(), 0, 0);
					}
				}
			}
			if(! relative)
				ob = {left: 0-window.pageXOffset, top: 0-window.pageYOffset};

			// assemble relative bounds
			if(isWindow(this._node)) {
				rect.width = window.innerWidth;
				rect.height = window.innerHeight;
			} else if(isDocument(this._node)) {
				rect.left = 0-window.pageXOffset;
				rect.top = 0-window.pageYOffset;
				rect.width = this._node.documentElement.clientWidth;
				rect.height = this._node.documentElement.clientHeight;
			} else {
				if(! this._node.ownerDocument)
					// cannot fetch bounds for disconnected node
					return;
				extendObject(rect, this._node.getBoundingClientRect(),
					'left top width height bottom right'.split(' '));
			}
			
			// absolutize bounds
			if(typeof(rect.left) === 'number' && ob)
				Dom.adjustRect(rect, 0-ob.left, 0-ob.top, 0, 0);
			
			rect.x = rect.left;
			rect.y = rect.top;
			return rect;
		},
	
		// set display: none and store old display style
		hide: function() {
			var disp = this.css('display');
			if(disp && disp !== 'none')
				this.$setData('_prev_display', disp);
			this.css('display', 'none');
			return this;
		},
		// show a previous hidden element. sets display to 'block' if hidden by CSS
		show: function() {
			var disp = this.$removeData('_prev_display');
			if(this.css('display') === 'none' || this._node.style.display === 'none') {
				if(disp) this.css('display', disp);
				else {
					this.css('display', null);
					if(this.css('display') === 'none') {
						// inherited from stylesheet
						this.css('display', 'block');
					}
				}
			}
			return this;
		},
		// toggle display state
		toggle: function(state) {
			if(state === undefined) {
				state = (this.css('display') === 'none');
			}
			if(state) this.show();
			else this.hide();
			return this;
		},
		
		// offset - relative to document (top, left)
		//   sets position to relative if static, presumably leaves absolute
		// height(value / function)
		// width
		// innerHeight(value / function)
		// innerWidth
		// outerHeight(includeMargin:boolean=false / value / function)
		// outerWidth

		// get/set horizontal scroll position
		scrollLeft: function(val) {
			if(val === undefined || val === null) {
				if(isWindow(this._node))
					return this._node.pageXOffset;
				return this._node.scrollLeft;
			} else {
				if(isWindow(this._node))
					this._node.pageXOffset = val;
				else
					this.node.scrollLeft = val;
				return this;
			}
		},
		// get/set vertical scroll position
		scrollTop: function(val) {
			if(val === undefined || val === null) {
				if(isWindow(this._node))
					return this._node.pageYOffset;
				return this._node.scrollTop;
			} else {
				if(isWindow(this._node))
					this._node.pageYOffset = val;
				else
					this.node.scrollTop = val;
				return this;
			}
		},
		// get horizontal scrollable width (with support for window)
		scrollWidth: function() {
			if(isWindow(this._node))
				return this._node.document.clientWidth;
			else if(isDocument(this._node))
				return this._node.documentElement.scrollWidth;
			return this._node.scrollWidth;
		},
		// get horizontal scrollable height (with support for window)
		scrollHeight: function() {
			if(isWindow(this._node))
				return this._node.document.clientHeight;
			else if(isDocument(this._node))
				return this._node.documentElement.scrollHeight;
			return this._node.scrollHeight;
		},
		
		// check if element matches a given selector
		matches: function(sel) {
			return Dom.matchSelector(this._node, sel, this);
		},
		// find descendant nodes using selector
		find: function(sel) {
			var nodes = new Dom.ExtNodeList();
			if(! sel) {
				// no match
			} else if(typeof(sel) === 'string') {
				try {
					nodes.push(this._node.querySelectorAll(sel));
				} catch(e) { // may throw SyntaxError for unsupported selector
					console.error(e, sel);
				}
			} else if(typeof(sel) === 'function') {
				nodes = this.find('*').filter(sel);
			} else if(typeof(sel) === Node || sel instanceof Dom.ExtNode) {
				if(this.contains(sel))
					nodes.push(sel);
			} else if(sel instanceof Dom.ExtNodeList) {
				for(var i = 0; i < sel.length; i++) {
					if(this.contains(sel[i]))
						nodes.push(sel[i]);
				}
			}
			return nodes;
		},
		// wrap first descendant given a selector
		findFirst: function(sel) {
			var nodes = new Dom.ExtNodeList();
			if(sel) {
				if(typeof(sel) === 'string') {
					try {
						nodes.push(this._node.querySelector(sel));
					} catch(e) { // may throw SyntaxError for unsupported selector
						console.error(e);
					}
				} else if(typeof(sel) === 'function') {
					nodes.push(this.find('*').first(sel));
				} else if(typeof(sel) === Node || sel instanceof Dom.ExtNode) {
					if(this.contains(sel))
						nodes.push(sel);
				} else if(sel instanceof Dom.ExtNodeList) {
					for(var i = 0; i < sel.length; i++) {
						if(this.contains(sel[i])) {
							nodes.push(sel[i]);
							break;
						}
					}
				}
			}
			return nodes;
		},
		// get child nodes (with an optional selector)
		children: function(sel) {
			var nodes = new Dom.ExtNodeList(this._node.children);
			if(sel) nodes = nodes.filter(sel);
			return nodes;
		},
		// get list of parent nodes (with optional selector)
		parents: function(sel) {
			var ret = new Dom.ExtNodeList();
			var node = this._node.parentNode;
			while(node && node.nodeType === 1) {
				if(! sel || Dom.matchSelector(node, sel, this))
					ret.push(node);
				node = node.parentNode;
			}
			return ret;
		},
		// find the closest ancestor node matching optional selector (starting with the current node)
		closest: function(sel) {
			var ret = new Dom.ExtNodeList();
			var node = this._node;
			while(node && node.nodeType === 1) {
				if(! sel || Dom.matchSelector(node, sel, this)) {
					ret.push(node);
					break;
				}
				node = node.parentNode;
			}
			return ret;
		}
	});
	
	
	/* Different from createElement2 in that:
		- name may be a selector in the format  tagName#id.cls.cls[attrTrue,attr=val]:param(val)
		- name may also be an object like {id, className, tagName, attrs, params, props, content, parent}
		- attrs is optional and sets element attributes, not properties
		- optional props parameter follows attrs, must be an object
		- no parent parameter
		- any number of content parameters (string, node, element, array)
	*/
	Dom.makeElement = function(name, attrs, props, content) {
		var init, i, el, cidx;
		if(isPlainObject(name) || name instanceof Dom.ElementSpec)
			init = name;
		else
			// split name using selector syntax
			init = Dom.parseSimpleSelector(name);
		if(! init || ! init.tagName) {
			console.error("Missing element tag name");
			return null;
		}
		try {
			if(init.ns)
				el = Dom.ExtNode(document.createElementNS(init.ns, init.tagName));
			else
				el = Dom.ExtNode(document.createElement(init.tagName));
		} catch(e) {
			console.error('Error creating element: ' + init.tagName);
			throw e;
		}
		if(init.id)
			el.prop('id', init.id);
		if(init.className)
			el.addClass(init.className.replace(/\./g, ' '));
		if(init.attrs)
			el.attr(init.attrs);
		if(init.props)
			el.prop(init.props);
		if(init.content)
			el.content(init.content);
		if(init.params) {
			// more of a test of params, might have other uses
			if(init.params.checked) el.prop('checked', true);
			if(init.params.hidden) el.hide();
		}
		cidx = 1;
		if(isPlainObject(attrs)) {
			el.attr(attrs);
			cidx = 2;
		}
		if(isPlainObject(props)) {
			el.prop(props);
			cidx = 3;
		}
		for(i = cidx; i < arguments.length; i++)
			el.append(arguments[i]);
		return el;
	}
	
	
	// same arguments as makeElement but returns a spec object instead of creating the node
	// main benefit is that the namespace can be inherited
	Dom.ElementSpec = function(init, attrs, props, content) {
		if(! (this instanceof Dom.ElementSpec)) {
			if(init instanceof Dom.ElementSpec)
				return init;
			var args = [null].pushArray(arguments),
				ctor = Dom.ElementSpec.bind.apply(Dom.ElementSpec, args);
			return new ctor();
		}
		if(isPlainObject(init) || init instanceof Dom.ElementSpec)
			extendObject(this, init);
		else if(typeof(init) === 'string') {
			init = Dom.parseSimpleSelector(init);
			if(init) extendObject(this, init);
		}
		var cidx = 1;
		if(isPlainObject(attrs)) {
			this.attr(attrs);
			cidx = 2;
		}
		if(isPlainObject(props)) {
			this.prop(props);
			cidx = 3;
		}
		for(i = cidx; i < arguments.length; i++)
			this.append(arguments[i]);
	}
	Dom.ElementSpec.prototype = {
		attr: function(name, val) {
			if(! name) return;
			if(! this.attrs) {
				this.attrs = {};
			}
			if(isPlainObject(name))
				extendObject(this.attrs, name);
			else if(isset(val))
				this.attrs[name] = val;
			else
				delete this.attrs[name];
			return this;
		},
		prop: function(name, val) {
			if(! name) return;
			if(! this.props) {
				this.props = {};
			}
			if(isPlainObject(name))
				extendObject(this.props, name);
			else if(isset(val))
				this.props[name] = val;
			else
				delete this.props[name];
			return this;
		},
		append: function(data) {
			if(isset(this.content)) {
				if(! Array.isArray(this.content))
					this.content = [this.content];
			} else
				this.content = [];
			if(Array.isArray(data)) {
				for(var elt in data)
					this.append(elt);
				return;
			}
			this.content.push(data);
			return this;
		},
		create: function(params) {
			var from = this;
			if(params && params.ns) {
				from = new Dom.ElementSpec(this);
				from.ns = params.ns;
			}
			return Dom.makeElement(from);
		},
		toElement: function(params) {
			var elt = this.create(params);
			if(elt) return elt.get();
		}
	};


	Dom.ExtNodeList = function(sel) {
		if(! (this instanceof Dom.ExtNodeList)) {
			if(sel instanceof Dom.ExtNodeList)
				return sel;
			return new Dom.ExtNodeList(sel);
		}
		Array.call(this);
		this.$init(sel);
	}
	Dom.htmlTempElt = document.createElement('div');
	extendClass(Dom.ExtNodeList, Array, {
		$init: function(sel) {
			if(typeof(sel) === 'string' && sel.length) {
				if(sel[0] === '<') {
					// treat as HTML
					var div = Dom.htmlTempElt;
					div.innerHTML = sel;
					this.pushArray(div.childNodes);
				} else {
					try {
						this.pushArray(document.querySelectorAll(sel));
					} catch(e) { // may throw SyntaxError for unsupported selector
						console.error(e);
					}
				}
			}
			else
				this.push(sel);
		},
		clone: function(dataAndEvents) {
			return this.map(function(node) { return Dom.ExtNode(node).clone(dataAndEvents); });
		},
		push: function(elt) {
			if(elt instanceof Node || isWindow(elt))
				Array.prototype.push.call(this, elt);
			else if(elt instanceof Dom.ExtNode && elt._node)
				Array.prototype.push.call(this, elt._node);
			else if(elt && elt.toElement)
				this.push(elt.toElement());
			else if(Array.isArray(elt) || elt instanceof Dom.ExtNodeList || elt instanceof NodeList || elt instanceof HTMLCollection)
				this.pushArray(elt);
			return this;
		},
		pushUnique: function(elt) {
			if(elt instanceof Node || isWindow(elt)) {
				if(this.indexOf(elt) < 0)
					Array.prototype.push.call(this, elt);
			}
			else if(elt instanceof Dom.ExtNode && elt._node)
				this.pushUnique(elt._node);
			else if(elt && elt.toElement)
				this.pushUnique(elt.toElement());
			else
				this.pushArrayUnique(elt);
			return this;
		},
		pushArray: function(elts) {
			if(! elts) return;
			if(elts instanceof NodeList || elts instanceof HTMLCollection)
				return Array.prototype.pushArray.call(this, elts);
			else {
				for(var i = 0; i < elts.length; i++)
					this.push(elts[i]);
			}
			return this;
		},
		pushArrayUnique: function(elts) {
			if(! elts) return;
			for(var i = 0; i < elts.length; i++)
				this.pushUnique(elts[i]);
			return this;
		},
		// fetch the Node for given index or all results as an array
		get: function(index) {
			if(! isset(index))
				return Array.prototype.slice.call(this);
			return this[index];
		},
		// fetch an index or all elements wrapped in ExtNode
		ext: function(index) {
			if(! isset(index))
				return this.map(Dom.ExtNode);
			if(! (index in this))
				return null;
			return Dom.ExtNode(this[index]);
		},
		// reduce results to a specific index
		eq: function(index) {
			var ret = new Dom.ExtNodeList();
			if(index < 0) index = this.length + index;
			ret.push(this[index]);
			return ret;
		},
		// forEach with each element converted to a ExtNode
		visit: function(method, thisp) {
			var node;
			for(var i = 0; i < this.length; i++) {
				node = Dom.ExtNode(this[i]);
				method.call(thisp || node, node, i, this);
			}
			return this;
		},
		// apply method to each element
		applyEach: function(method /*, extraArgs */) {
			var args = Array.prototype.slice.call(arguments, 1);
			if(typeof(method) === 'string')
				method = Dom.ExtNode.prototype[method];
			if(method)
			for(var i = 0; i < this.length; i++) {
				method.apply(Dom.ExtNode(this[i]), args);
			}
			return this;
		},
		
		// find if any or all elements have the class
		hasClass: function(cls, all) {
			var ret = null;
			cls = Dom.splitCssClasses(cls);
			for(var i = 0; i < this.length; i++) {
				ret = Dom.ExtNode(this[i]).hasClass(cls);
				if(all && ! ret) break;
				if(! all && ret) break;
			}
			return ret;
		},
		// combines text from all elements
		text: function(content) {
			var ret = this;
			if(content === undefined) {
				ret = [];
				for(var i = 0; i < this.length; i++) {
					ret.push(Dom.ExtNode(this[i]).text());
				}
				ret = ret.join();
			} else {
				this.applyEach(Dom.ExtNode.prototype.text, content);
			}
			return ret;
		},
		// combines content from all elements
		content: function(content) {
			var ret;
			if(content === undefined) {
				ret = [];
				for(var i = 0; i < this.length; i++) {
					ret.pushArray(Dom.ExtNode(this[i]).content());
				}
			} else {
				this.applyEach(Dom.ExtNode.prototype.content, content);
				ret = this;
			}
			return ret;
		},
		// filter currently selected nodes by a selector
		filter: function(sel) {
			var ret = new Dom.ExtNodeList();
			for(var i = 0; i < this.length; i++) {
				if(Dom.matchSelector(this[i], sel, this, i))
					ret.push(this[i]);
			}
			return ret;
		},
		// return first element with an optional selector
		// note - applies to currently selected nodes, not descendants like findFirst
		first: function(sel) {
			var ret = new Dom.ExtNodeList();
			if(sel !== undefined) {
				for(var i = 0; i < this.length; i++) {
					if(Dom.matchSelector(this[i], sel, this, i)) {
						ret.push(this[i]);
						break;
					}
				}
			} else
				ret.push(this[0]);
			return ret;
		},
		// find all descendant nodes matching a selector
		find: function(sel) {
			var ret = new Dom.ExtNodeList();
			for(var i = 0; i < this.length; i++) {
				ret.pushArrayUnique(Dom.ExtNode(this[i]).find(sel));
			}
			return ret;
		},
		// find first descendant node matching a selector
		findFirst: function(sel) {
			var ret;
			for(var i = 0; i < this.length; i++) {
				ret = Dom.ExtNode(this[i]).findFirst(sel);
				if(ret.length) break;
			}
			if(! ret) ret = new Dom.ExtNodeList();
			return ret;
		},
		// combine children from all nodes (no duplicates)
		children: function(sel) {
			var ret = new Dom.ExtNodeList();
			for(i = 0; i < this.length; i++) {
				ret.pushArrayUnique(Dom.ExtNode(this[i]).children(sel));
			}
			return ret;
		},
		// combine parents from all nodes (no duplicates)
		parents: function(sel) {
			var ret = new Dom.ExtNodeList();
			for(var i = 0; i < this.length; i++) {
				ret.pushArrayUnique(Dom.ExtNode(this[i]).parents(sel));
			}
			return ret;
		},
		// combine closest from all nodes (no duplicates)
		closest: function(sel) {
			var ret = new Dom.ExtNodeList();
			for(var i = 0; i < this.length; i++) {
				ret.pushArrayUnique(Dom.ExtNode(this[i]).closest(sel));
			}
			return ret;
		},
		
		// return a new ExtNodeList with any duplicate nodes removed
		unique: function() {
			var ret = new Dom.ExtNodeList();
			return ret.pushArrayUnique(this);
		},
		// remove duplicate elements in-place
		$makeUnique: function() {
			for(var i = this.length-1; i >= 0; i--) {
				if(this.indexOf(this[i]) !== i)
					this.splice(i, 1);
			}
			return this;
		}
		
		// bounds: maybe combine bounds rect for all children?
		// find: may also want to define ExtNodeList(sel, sel2) = ExtNodeList(sel).find(sel2)
		// replace: collect elements into DocumentFragment and perform replace
		// swapWith: probably not implemented (swap to where?)
	});
	var nodeListInherit = [
			'on', 'off', 'emit',
			'append', 'appendTo', 'prepend', 'prependTo',
			'before', 'after', 'insertBefore', 'insertAfter',
			'detach', 'remove', 'swapIn', 'swapOut', 'replaceWith',
			'removeAttr', 'removeProp', 'empty',
			'addClass', 'removeClass', 'toggleClass',
			'hide', 'show'
		],
		// applies method to each node ignoring results
		mapperAll = function(method) {
			if(typeof(method) === 'string')
				method = Dom.ExtNode.prototype[method];
			return function(args) {
				var i, node;
				for(i = 0; i < this.length; i++) {
					node = Dom.ExtNode(this[i]);
					method.apply(node, arguments);
				}
				return this;
			}
		},
		// getter returns result of applying method to the first element
		mapperFirst = function(method, checkIdx, objFirst) {
			if(typeof(method) === 'string')
				method = Dom.ExtNode.prototype[method];
			return function(args) {
				var i, node;
				if(checkIdx === undefined || arguments[checkIdx] === undefined
				// first argument to offset, scrollLeft etc can be null
				|| (checkIdx == 0 && arguments[checkIdx] === null)
				// first argument to css/attr/prop/size may be an object
				|| (objFirst && arguments[0] !== null && typeof(arguments[0]) === 'object')) {
					if(this.length) {
						node = Dom.ExtNode(this[0]);
						return method.apply(node, arguments);
					}
					return;
				} else {
					for(i = 0; i < this.length; i++) {
						node = Dom.ExtNode(this[i]);
						method.apply(node, arguments);
					}
				}
				return this;
			}
		};
	for(var i = 0; i < nodeListInherit.length; i++) {
		var method = nodeListInherit[i];
		Dom.ExtNodeList.prototype[method] = mapperAll(method);
	}
	Dom.ExtNodeList.prototype.attr			= mapperFirst('attr', 1, true);
	Dom.ExtNodeList.prototype.bounds		= mapperFirst('bounds');
	Dom.ExtNodeList.prototype.css			= mapperFirst('css', 1, true);
	Dom.ExtNodeList.prototype.cssSize		= mapperFirst('cssSize', 1, true);
	Dom.ExtNodeList.prototype.html			= mapperFirst('html', 0);
	Dom.ExtNodeList.prototype.inDocument	= mapperFirst('inDocument');
	Dom.ExtNodeList.prototype.isElement		= mapperFirst('isElement');
	Dom.ExtNodeList.prototype.offset		= mapperFirst('offset', 0);
	Dom.ExtNodeList.prototype.offsetFrom	= mapperFirst('offsetFrom', 1);
	Dom.ExtNodeList.prototype.offsetParent	= mapperFirst('offsetParent');
	Dom.ExtNodeList.prototype.position		= mapperFirst('position');
	Dom.ExtNodeList.prototype.prop			= mapperFirst('prop', 1, true);
	Dom.ExtNodeList.prototype.scrollLeft	= mapperFirst('scrollLeft', 0);
	Dom.ExtNodeList.prototype.scrollTop		= mapperFirst('scrollTop', 0);
	Dom.ExtNodeList.prototype.scrollWidth	= mapperFirst('scrollWidth');
	Dom.ExtNodeList.prototype.scrollHeight	= mapperFirst('scrollHeight');
	Dom.ExtNodeList.prototype.toggle		= mapperFirst('toggle');
	Dom.ExtNodeList.prototype.val			= mapperFirst('val', 0);
	
	
	// accepts a string ID, element, array, etc. and may return an ExtNode
	Dom.elementOrId = function(elt) {
		if(typeof(elt) === 'string' && elt.length) elt = document.getElementById(elt);
		elt = Dom.ExtNode(elt);
		if(! elt || (! elt.isElement() && ! elt.isWindow())) return;
		return elt;
	}
	// accepts a string selector, element, array, etc. and returns an ExtNodeList
	// (may be empty)
	Dom.query = Dom.ExtNodeList;
	// accepts a string selector, element, array, etc. and returns an ExtNodeList
	// (with one or zero elements)
	// allows method chaining when the element may not be found
	Dom.queryFirst = function(sel) {
		return Dom.ExtNodeList(Dom.ExtNode(sel));
	}
	// accepts a string ID, element, array, etc. and returns a collection
	// (may be empty)
	Dom.queryId = function(elt) {
		if(typeof(elt) === 'string' && elt.length) elt = document.getElementById(elt);
		return Dom.query(elt);
	}
	// convert to ExtNode from an existing node (same as queryFirst)
	// may want to reject anything but a Node or ExtNode
	Dom.extNode = Dom.ExtNode;
});


AppBase.extendModule('base', 'ui', function(App) {
	var UI = this,
		wrap = App.dom.extNode,
		queryId = App.dom.queryId,
		makeElt = App.dom.makeElement,
		$window = wrap(window),
		$document = wrap(document);
	
	UI.onInit(function() {
		$window.on('resize:throttle(50)', this.resizeHandler.bind(this));
		$window.on('scroll:throttle(50)', this.scrollHandler.bind(this));
		this.onDomLoad(this.layoutUpdated);
	});
	
	// trigger manually to force resize-safe, scroll-safe events (debounced)
	this.layoutUpdated = function() {
		clearTimeout(UI.layoutTimer);
		UI.layoutTimer = setTimeout(function() {
			UI.layoutTimer = null;
			UI.resizeHandler();
			UI.scrollHandler();
		}, 150);
	}
	
	this.resizeHandler = function(evt) {
		if(! document.body) return;
		var sz = {w: window.innerWidth, h: window.innerHeight},
			last = $window.$getData('_last_size');
		if(evt && last && last.w === sz.w && last.h === sz.h)
			// ignore repeated resize event if called with event parameter
			return;
		if(! last) {
			last = deep_clone(sz);
			last.first = true;
		}
		$window.emit('resize-safe', [sz, last]);
		$window.$setData('_last_size', sz);
	}
	this.addResizeHook = function(callb, scope, data) {
		$window.on('resize-safe', callb, {scope: scope, data: data});
	}
	
	this.scrollHandler = function(evt) {
		if(! document.body) return;
		var pos = {x: window.pageXOffset, y: window.pageYOffset},
			last = $window.$getData('_last_scroll');
		if(evt && last && last.x === pos.x && last.y === pos.y)
			// ignore repeated scroll event if called with event parameter
			return;
		if(! last) {
			last = deep_clone(pos);
			last.first = true;
		}
		$window.emit('scroll-safe', [pos, last]);
		$window.$setData('_last_scroll', pos);
	}
	this.addScrollHook = function(callb, scope, data) {
		$window.on('scroll-safe', callb, {scope: scope, data: data});
	}
	
	this.getViewport = function(margin, modal) {
		var ret = $window.bounds();
		var insets = App.browser.insetSizes;
		if(margin === true) {
			// default margin
			if(ret.width < 1000) margin = 5;
			else margin = 12;
		}
		ret.margin = margin = margin || 0;
		ret.doc_width = $document.scrollWidth() - 2*margin;
		ret.doc_height = $document.scrollHeight() - 2*margin;
		ret.window_width = ret.width;
		ret.window_height = ret.height;
		// page size (meta viewport) - note, can be different from window size on mobile
		var view_size = {
			width: document.documentElement.clientWidth - insets.left - insets.right,
			height: document.documentElement.clientHeight - insets.top - insets.bottom
		};
		ret.view_width = view_size.width - 2*margin;
		ret.view_height = view_size.height - 2*margin;
		var menuvis = (margin && !modal) ? 1 : 0,
			offset = (App.themes && App.themes.topMenuSize) ? App.themes.topMenuSize(menuvis) : null,
			header = queryId('tbarDiv').bounds();
		if(header && menuvis) {
			var hoffs = header.bottom + margin - ret.y;
			offset = Math.max(0, isset(offset) ? Math.min(offset, hoffs) : hoffs);
		}
		ret.offset = offset || 0;
		if(modal && isIPhone) {
			ret.width = view_size.width - insets.left - insets.right;
			ret.height = view_size.height - insets.top - insets.bottom;
		}
		ret.y += ret.offset + insets.top;
		ret.top += ret.offset + insets.top;
		ret.x += insets.left;
		ret.left += insets.left;
		ret.height -= ret.offset + insets.left - insets.right;
		ret.body_width = document.body.offsetWidth;
		ret.body_height = document.body.offsetHeight;
		UI.adjustRect(ret, margin, margin, -2*margin, -2*margin);
		return ret;
	}
	
	this.getEltRegion = function(elt, outer) {
		// NOTE: old method checked that outer actually contains elt
		// and has a position of relative or absolute
		return queryId(elt).bounds(queryId(outer));
	}

	this.getTransformProperty = function() {
		return App.browser.transformProperty;
	}
	
	this.getTransitionProperty = function() {
		return App.browser.transitionProperty;
	}

	this.addClass = function(elt, className) {
		queryId(elt).addClass(className);
	}
	this.removeClass = function(elt, className) {
		queryId(elt).removeClass(className);
	}
	this.hasClass = function(elt, className) {
		return queryId(elt).hasClass(className);
	}
	this.toggleClass = this.addRemoveClass = function(elt, className, state) {
		var node = queryId(elt);
		node.toggleClass(className, state);
		return node.hasClass(className);
	}
	this.setDisplayState = function(elt, value) {
		queryId(elt).toggle(!! value);
	}
	
	// old method for managing event listeners - prefer ExtNode.on(...)
	this.addEventListener = function(elt, name, callb, scope, data, options) {
		var opts = {scope: scope, data: data, passData: true, extEvent: true, extTarget: false};
		if(! elt || ! callb) return;
		if(isPlainObject(options)) {
			opts.capture = options.capture;
			opts.passive = options.passive;
		} else if(options) {
			opts.capture = true;
		}
		queryId(elt).on(name, callb, opts);
		return {name: name, listener: callb, options: options, compat: true};
	}
	this.addEventListeners = function(elt, evts, scope, data, options) {
		var ret = [];
		if(elt && evts) {
			for(name in evts) {
				ret.push(this.addEventListener(elt, name, evts[name], scope, data, options));
			}
		}
		return ret;
	}
	this.removeEventListener = function(elt, evt, fn, options) {
		if(elt && evt) {
			if(fn) {
				options = options ? Base.fixListenerOptions(options, true) : null;
				queryId(elt).off(evt, fn, options);
			}
			else if(evt.name)
				queryId(elt).off(evt.name, evt.listener, evt.options);
		}
	}
	this.removeEventListeners = function(elt, evts) {
		if(evts) {
			for(i = 0; i < evts.length; i++)
				this.removeEventListener(elt, evts[i]);
		}
	}
	
	this.stopPropagation = function(evt) {
		if(evt) evt.stopPropagation();
	}
	this.preventDefault = function(evt) {
		if(evt) evt.preventDefault();
	}
	this.stopEvent = function(evt) {
		if(evt) {
			evt.stopPropagation();
			evt.preventDefault();
		}
		return false;
	}

	this.clearChildNodes = function(elt) {
		queryId(elt).empty();
	}
	this.removeNode = function(elt, fullRemove) {
		if(fullRemove) queryId(elt).remove();
		else queryId(elt).detach();
	}
	this.inDocument = function(elt, doc) {
		return queryId(elt).inDocument(doc);
	}
	this.isAncestor = function(parent, child) {
		var p = wrap(parent), c = wrap(child);
		if(! parent || ! child) return false;
		return p.contains(c);
	}
	
	this.getElementText = function(elt) {
		return queryId(elt).text();
	}
	this.setElementText = function(elt, text) {
		if(text === undefined) text = null;
		queryId(elt).text(text);
	}
	
	this.addElementContent = function(elt, content) {
		queryId(elt).append(content);
	}
	
	this.setElementContent = function(elt, content) {
		if(content === undefined) content = null;
		queryId(elt).content(content);
	}

	this.setWidth = function(elt, width, def_unit) {
		width = UI.formatSize(width, def_unit);
		if(width)
			elt.style.width = width;
	}
	
	this.createIcon = function(icon_name, parent) {
		if(icon_name && icon_name.indexOf('-') < 0) icon_name = 'theme-icon module-' + icon_name;
		return makeElt({tagName: 'div', className: 'input-icon '+icon_name, parent: parent}).get();
	}
	
});

AppBase.module('animation').moduleRequires('dom').init(function(App) {

	var fromArray = function(arr) {
		return function(percent, index) {
			return arr[index];
		}
	}

	var setter = function(name) {
		return function(value) {
			this.animationProps[name] = value;
			return this;
		}
	}

	var addStyle = function(animation) {
		if (animation.styleAdded)
			return;
		var styleTag = document.createElement("style");
		var prefix = App.browser.animation.prefix ? '-' + App.browser.animation.prefix + '-' : '';

		var text = '@' + prefix + 'keyframes '  + animation.name + ' {\n';
		Object.keys(animation.keyframes).map(parseFloat).sort(function(a,b){return a - b}).forEach(function(percent) {
			text += percent + '% {\n';
			Object.keys(animation.keyframes[percent]).forEach(function(key) {
				text += key + ': ' + animation.keyframes[percent][key] + ';\n';
			});
			text += '}\n';
		});
		text += '}\n';
		styleTag.textContent = text;
		document.head.appendChild(styleTag);
		animation.styleAdded = true;
	}

	var Animation = function() {
		this.name = 'anim' + (Math.random() + '').substring(2);
		this.keyframes = {
			'0' : {},
			'100' : {}
		};
		this.animationProps = {
			delay: 0,
			direction: 'normal',
			duration: 0,
			'fill-mode' : 'none',
			'iteration-count': 1,
			'timing-function': 'linear'
		};
		this.styleAdded = false;
		this.handlers = {};
	}


	/**
	 * Sets 0% keyframe
	 **/
	Animation.prototype.from = function(rules, replace) {
		this.add(0, rules, replace);
		return this;
	}

	/**
	 * Sets 100% keyframe
	 **/
	Animation.prototype.to = function(rules, replace) {
		this.add(100, rules, replace);
		return this;
	}

	/**
	 * Adds keyframe
	 **/
	Animation.prototype.add = function(percent, rules, replace) {
		var pc = percent === 'from' ? 0 : ( percent === 'to' ? 100 : parseFloat(percent));
		if (isNaN(pc) || pc < 0 || pc > 100 || typeof rules !== 'object')
			return;
		if (replace || !isset(this.keyframes[pc]))
			this.keyframes[pc] = rules;
		else
			Object.assign(this.keyframes[pc], rules);
		return this;
	}

	/**
	 * Creates a series of keyframes. This function can be called multiple ways:
	 *
	 *
	 * distribute(0, 100, [{color: red}, {color: green}, {color: blue}])
	 *   creates 3 keyframes, at 0%, 50% and 100%, with red, green and blue
	 *
	 * distribute(20, 70, [{color: red}, {color: green}, {color: blue}])
	 *   creates 3 keyframes, at 20%, 45% and 70%, with red, green and blue
	 *
	 * distribute(0, 100, 20, function(percent, index) {return {opacity: percent + '%'} } )
	 *   creates 20 keyframes, from 0% to 100%, with opacity changing from 0% to 100%
	 *   index changes from 0 to 20
	 *
	 * in all cases, percent limits can be omited, and will default to 0 and 100
	 *
	 *
	 **/
	
	Animation.prototype.distribute = function() {
		var count, create, from, to;
		var args = Array.prototype.slice.call(arguments);
		args = args.map(function(arg) {
			var num = parseFloat(arg);
			if (!isNaN(num)) return num;
			return arg;
		});
		var signature = args.map(function(x) {return Array.isArray(x) ? 'array' : typeof x;}).join('|');
		switch (signature) {
			case 'number|number|array':
				from = args[0]; to = args[1];
				count = args[2].length - 1;
				create = fromArray(args[2]);
				break;
			case 'array':
				from = 0; to = 100;
				count = args[0].length - 1;
				create = fromArray(args[0]);
				break;
			case 'number|number|number|function':
				from = args[0]; to = args[1];
				count = args[2];
				create = args[3];
				break;
			case 'number|function':
				from = 0; to = 100;
				count = args[0];
				create = args[1];
				break;
			default:
				return this;
		}
		count = parseInt(count);
		if (count < 1)
			return this;
		if (from < 0 || from > 100 || to < 0 || to > 100 || from > to)
			return this;
		var percent = from;
		var step = (to - from) / count;
		for (var i = 0; i <= count; i++) {
			this.add(percent, create(percent, i));
			percent += step;
		}
		return this;

	}

	/**
	 * Sets animation delay
	 */

	Animation.prototype.delay = setter('delay');

	/**
	 * Sets animation direction
	 */
	Animation.prototype.direction = setter('direction');

	/**
	 * Sets animation duration
	 */
	Animation.prototype.duration = function(duration) {
		if (typeof duration === 'string')
			this.animationProps.duration = duration;
		else
			this.animationProps.duration = duration + 'ms';
		return this;
	}

	/**
	 * Sets animation fill mode
	 */
	Animation.prototype.fillMode = setter('fill-mode');

	/**
	 * Sets animation iterations count
	 */
	Animation.prototype.iterations = setter('iteration-count');

	/**
	 * Sets animation timing function
	 */
	Animation.prototype.timing = setter('timing-function');


	Animation.prototype.onStart = function(handler) {
		this.handlers.start = handler;
		return this;
	}

	Animation.prototype.onEnd = function(handler) {
		this.handlers.end = handler;
		return this;
	}
	Animation.prototype.onIteration = function(handler) {
		this.handlers.iteration = handler;
		return this;
	}

	/**
	 * Runs the animation
	 *
	 * node can be any argument accepted by ExtNodeList constructor
	 *
	*/
	Animation.prototype.run = function(node) {
		var self = this;
		addStyle(this);
		var nodes = new App.dom.ExtNodeList(node);
		var css = {};
		Object.keys(this.animationProps).forEach(function(key) {
			css['animation-' + key] = this.animationProps[key];
		}.bind(this));
		css['animationName'] = this.name;

		var handleStart = function(evt) {
			var args = Array.prototype.slice.call(arguments);
			if (typeof self.handlers.start === 'function')
				self.handlers.start.apply(this, args);
		}

		var handleEnd = function() {
			var args = Array.prototype.slice.call(arguments);
			if (typeof self.handlers.end === 'function')
				self.handlers.end.apply(this, args);
		}
		var handleIteration = function() {
			var args = Array.prototype.slice.call(arguments);
			if (typeof self.handlers.iteration === 'function')
				self.handlers.iteration.apply(this, args);
		}

		nodes.visit(function(node) {
			// TODO: consider onanimationcancel (currently in draft, not supported by major browsers)
			node.off('animationstart')
			node.off('animationend');
			node.off('animationiteration');
			node.css(css);
			var start = handleStart.bind(this);
			var end = handleEnd.bind(this);
			var iter = handleIteration.bind(this);
			node.on('animationstart', start);
			node.on('animationend', function(ev) {
				end(ev);
				node.off('animationstart', start);
				node.off('animationend', end);
				node.off('animationiteration', iter);
			});
			node.on('animationiteration', iter);
		});
	}

	this.Animation = Animation;

});
