/*  Copyright Mihai Bazon, 2002, 2003  |  http://dynarch.com/mishoo/
 * ------------------------------------------------------------------
 *
 * The DHTML Calendar, version 0.9.6 "Keep cool but don't freeze"
 *
 * Details and latest version at:
 * http://dynarch.com/mishoo/calendar.epl
 *
 * This script is distributed under the GNU Lesser General Public License.
 * Read the entire license text here: http://www.gnu.org/licenses/lgpl.html
 */



/** The Calendar object constructor. */
Calendar = function(params) {
	if(! params) params = {};
	
	for(var idx in Calendar.defaults) {
		if(! (idx in params))
			params[idx] = Calendar.defaults[idx];
	}
	extendObject(this, params);

	// one-time initializations
	if (! isset(Calendar._SDN)) {
		// table of short day names
		if (! isset(Calendar._SDN_len))
			Calendar._SDN_len = 3;
		var ar = [];
		for (var i = 8; i > 0;) {
			ar[--i] = Calendar._DN[i].substr(0, Calendar._SDN_len);
		}
		Calendar._SDN = ar;
		// table of short month names
		if (! isset(Calendar._SMN_len))
			Calendar._SMN_len = 3;
		ar = [];
		for (var i = 12; i > 0;) {
			ar[--i] = Calendar._MN[i].substr(0, Calendar._SMN_len);
		}
		Calendar._SMN = ar;
	}
};

// ** constants

// language
Calendar._TT = {};


Calendar.defaults = {
	dateFormat: SUGAR.session.formats.date,
	ttDateFormat: SUGAR.session.formats.date,
	firstDayOfWeek: SUGAR.session.week_start_day,
	minYear: 1970,
	maxYear: 2500,
	singleClick: true,
	weekNumbers: true,
	showsTime: false,
	electric: true,
	yearStep: 1,
	showsOtherMonths: false,
	updateOnCloseOnly: false,
	enableReadOnly: false,
	time24: true,
	dateToday: SUGAR.session.jscal_today
};


// BEGIN: UTILITY FUNCTIONS; beware that these might be moved into a separate
//        library, at some point.

Calendar.getAbsolutePos = function(el) {
	var SL = 0, ST = 0;
	var is_div = /^div$/i.test(el.tagName);
	if (is_div && el.scrollLeft)
		SL = el.scrollLeft;
	if (is_div && el.scrollTop)
		ST = el.scrollTop;
	var r = { x: el.offsetLeft - SL, y: el.offsetTop - ST };
	if (el.offsetParent) {
		var tmp = this.getAbsolutePos(el.offsetParent);
		r.x += tmp.x;
		r.y += tmp.y;
	}
	return r;
};

Calendar.isRelated = function (el, evt) {
	var related = evt.relatedTarget;
	if (!related) {
		var type = evt.type;
		if (type == "mouseover") {
			related = evt.fromElement;
		} else if (type == "mouseout") {
			related = evt.toElement;
		}
	}
	while (related) {
		if (related == el) {
			return true;
		}
		related = related.parentNode;
	}
	return false;
};

// END: UTILITY FUNCTIONS

// BEGIN: CALENDAR STATIC FUNCTIONS

/** Internal -- adds a set of events to make some element behave like a button. */
Calendar._add_evs = function(el, scope) {
	SUGAR.ui.addEventListeners(el, {
		mouseover: this.dayMouseOver,
		mousedown: this.dayMouseDown,
		mouseout: this.dayMouseOut,
		mouseup: this.dayMouseUp}, scope);
};

Calendar.showMonthsCombo = function () {
	if (!cal) {
		return false;
	}
	var cal = cal;
	var cd = cal.activeDiv;
	var mc = cal.monthsCombo;
	if (cal.hilitedMonth) {
		SUGAR.ui.removeClass(cal.hilitedMonth, 'hilite');
	}
	if (cal.activeMonth) {
		SUGAR.ui.removeClass(cal.activeMonth, 'active');
	}
	var mon = cal.monthsCombo.getElementsByTagName("div")[cal.date.getMonth()];
	SUGAR.ui.addClass(mon, 'active');
	cal.activeMonth = mon;
	var s = mc.style;
	s.display = "block";
	if (cd.navtype < 0)
		s.left = cd.offsetLeft + "px";
	else {
		var mcw = mc.offsetWidth;
		s.left = (cd.offsetLeft + cd.offsetWidth - mcw) + "px";
	}
	s.top = (cd.offsetTop + cd.offsetHeight) + "px";
};

Calendar.showYearsCombo = function (fwd) {
	if (!cal) {
		return false;
	}
	var cal = cal;
	var cd = cal.activeDiv;
	var yc = cal.yearsCombo;
	if (cal.hilitedYear) {
		SUGAR.ui.removeClass(cal.hilitedYear, 'hilite');
	}
	if (cal.activeYear) {
		SUGAR.ui.removeClass(cal.activeYear, 'active');
	}
	cal.activeYear = null;
	var Y = cal.date.getFullYear() + (fwd ? 1 : -1);
	var yr = yc.firstChild;
	var show = false;
	for (var i = 12; i > 0; --i) {
		if (Y >= cal.minYear && Y <= cal.maxYear) {
			yr.firstChild.data = Y;
			yr.year = Y;
			yr.style.display = "block";
			show = true;
		} else {
			yr.style.display = "none";
		}
		yr = yr.nextSibling;
		Y += fwd ? cal.yearStep : -cal.yearStep;
	}
	if (show) {
		var s = yc.style;
		s.display = "block";
		if (cd.navtype < 0)
			s.left = cd.offsetLeft + "px";
		else {
			var ycw = yc.offsetWidth;
			s.left = (cd.offsetLeft + cd.offsetWidth - ycw) + "px";
		}
		s.top = (cd.offsetTop + cd.offsetHeight) + "px";
	}
};

// event handlers

Calendar.dayMouseDown = function(ev) {
	var el = ev.currentTarget;
	if(ev.which != 1) return;
	if (! el || el.disabled) {
		return false;
	}
	var cal = this;
	cal.activeDiv = el;
	if (el.navtype != 300) {
		SUGAR.ui.addClass(el, 'hilite');
		SUGAR.ui.addClass(el, 'active');
	}
	if (el.navtype == -1 || el.navtype == 1) {
		cal.nav(el.navtype, 'month');
	} else if (el.navtype == -2 || el.navtype == 2) {
		cal.nav(Math.round(el.navtype / 2), 'year');
	} else if(el.navtype == 0) {
		cal.nav(0, 0, cal.today);
	}
	return SUGAR.ui.stopEvent(ev);
};

Calendar.dayMouseUp = function(ev) {
	var target = ev.target;
	if(ev.which != 1) return;
	if(isset(target.dayindex)) {
		var date = this.month_info.days[target.dayindex].date;
		this.setDate(date);
		this.callUpdateHandler();
	}
	return SUGAR.ui.stopEvent(ev);
}

Calendar.dayMouseOver = function(ev) {
	var el = ev.currentTarget;
	if (Calendar.isRelated(el, ev) || el.disabled) {
		return false;
	}
	if (el.ttip) {
		if (el.ttip.substr(0, 1) == "_") {
			var date = this.month_info.days[el.dayindex].date;
			el.ttip = date.print(this.ttDateFormat) + el.ttip.substr(1);
		}
		this.showStatus(el.ttip);
	}
	if (! isset(el.navtype)) {
		if(! this.month_info.days[el.dayindex].disabled) {
			SUGAR.ui.addClass(el, "hilite");
			SUGAR.ui.addClass(el.parentNode, "rowhilite");
		}
	}
	return SUGAR.ui.stopEvent(ev);
};

Calendar.dayMouseOut = function(ev) {
	var el = ev.currentTarget;
	if (Calendar.isRelated(el, ev) || el.disabled) {
		return false;
	}
	SUGAR.ui.removeClass(el, 'hilite');
	SUGAR.ui.removeClass(el.parentNode, 'rowhilite');
	this.showStatus(Calendar._TT["SEL_DATE"], true);
	return SUGAR.ui.stopEvent(ev);
};

// END: CALENDAR STATIC FUNCTIONS

// BEGIN: CALENDAR OBJECT FUNCTIONS

Calendar.prototype.getElement = function () {
	return this.outer;
}

Calendar.prototype.getDate = function () {
	return this.date;
}

/**
 *  This function creates the calendar inside the given parent.  If _par is
 *  null than it creates a popup calendar inside the BODY element.  If _par is
 *  an element, be it BODY, then it creates a non-popup calendar (still
 *  hidden).  Some properties need to be set before calling this function.
 */
Calendar.prototype.render = function () {
	if(! this.outer)
		this.outer = createElement2('div', {className: 'card-outer panel-outer panel-default calendar-outer input-calendar'});
	
	var cal = this,
		row, cell,
		nav_icon = {},
		hh = function (text, navtype, tip, target) {
			var ret, cls;
			if(navtype >= -2 && navtype <= 2) {
				cls = 'uii ' + nav_icon[navtype] + (navtype < 0 ? ' nav-left' : ' nav-right');
				ret = createElement2('span', {className: cls, navtype: navtype, ttip: tip}, text, target);
				Calendar._add_evs(ret, cal);
			} else {
				ret = createElement2('span', {className: 'nav-title', navtype: navtype, ttip: tip}, text, target);
				Calendar._add_evs(ret, cal);
			}
			return ret;
		};
	nav_icon[-2] = 'uii-fprevious';
	nav_icon[-1] = 'uii-previous';
	nav_icon[1] = 'uii-next';
	nav_icon[2] = 'uii-fnext';

	var head = createElement2('header', {className: 'card-header panel-header no-pad'}, null, this.outer),
		head_row = createElement2('h4', {className: 'text-label nav-row'}, null, head),
		subhead = createElement2('header', {className: 'card-header panel-subheader no-pad grid-body'}, null, this.outer),
		day_names = createElement2('div', {className: 'grid-row number-row dates'}, null, subhead);
	
	hh(null, -2, Calendar._TT['PREV_YEAR'], head_row);
	hh(null, -1, Calendar._TT['PREV_MONTH'], head_row);

	//this.title = hh(Calendar._TT["TODAY"], this.weekNumbers ? 4 : 3, 0, Calendar._TT["GO_TODAY"]);
	this.title = hh('', 300, '', head_row);

	hh(null, 1, Calendar._TT['NEXT_MONTH'], head_row);
	hh(null, 2, Calendar._TT['NEXT_YEAR'], head_row);
	
	// day names
	if (this.weekNumbers) {
		createElement2('div', {className: 'grid-cell number-cell day-head label week-number'}, Calendar._TT['WK'], day_names);
	}
	for (var i = 7; i > 0; --i) {
		cell = createElement2('div', {className: 'grid-cell number-cell day-head'}, '', day_names);
		if(i == 7)
			this.firstdayname = cell;
		if (!i) {
			cell.navtype = 100;
			Calendar._add_evs(cell, this);
		}
	}
	
	this._displayWeekdays();

	this.body = createElement2('div', {className: 'card-body panel-body grid-body'}, null, this.outer);
	this.rows = [];

	for (i = 6; i > 0; --i) {
		row = createElement2('div', {className: 'grid-row number-row'}, null, this.body);
		if (this.weekNumbers) {
			cell = createElement2('div', {className: 'grid-cell number-cell day label week-number'}, null, row);
		}
		for (var j = 7; j > 0; --j) {
			cell = createElement2('div', {className: 'grid-cell number-cell text-right day'}, null, row);
			Calendar._add_evs(cell, this);
		}
		this.rows.push(row);
	}
	
	if(! this.hide_status) {
		var foot = createElement2('footer', {className: 'card-footer panel-footer'}, null, this.outer);
		
		row = createElement2('div', {className: 'text-right ttip'}, null, foot);
		cell = hh(Calendar._TT['SEL_DATE'], 300, '', row);
		
		this.tooltips = cell;
	}

	this._init();		
}


/** keyboard navigation, only for popup calendars */
Calendar.prototype.keyDown = function(ev) {
	var act = true, //(isIE || ev.type == "keypress"),
		info = this.month_info;
	if (ev.shiftKey) {
		switch (ev.keyCode) {
		    case 37: // KEY left
				act && this.nav(-1, 'month');
				break;
		    case 38: // KEY up
				act && this.nav(-1, 'year');
				break;
		    case 39: // KEY right
				act && this.nav(1, 'month');
				break;
		    case 40: // KEY down
				act && this.nav(1, 'year');
				break;
			default:
				return false;
		}
	} else switch (ev.keyCode) {
		case 13: // KEY enter
		case 32: // KEY space (now)
			act && this.callUpdateHandler();
			break;
		case 27: // KEY esc
			act && this.callCloseHandler();
			break;
		case 37: // KEY left
		case 38: // KEY up
		case 39: // KEY right
		case 40: // KEY down
			if (act) {
				switch (ev.keyCode) {
					case 38: // KEY up
						this.nav(-1, 'week');
						break;
					case 37: // KEY left
						this.nav(-1, 'day');
						break;
					case 40: // KEY down
						this.nav(1, 'week');
						break;
					case 39: // KEY right
						this.nav(1, 'day');
						break;
				}
			}
			break;
		default:
			return false;
	}
	return SUGAR.ui.stopEvent(ev);
}


Calendar.prototype.nav = function(offset, step, from_date) {
	if(! offset) offset = 0;
	var date = new Date(from_date || this.date),
		sgn = offset < 0 ? -1 : 1;
	if(step == 'week')
		return this.nav(offset * 7, 'day', date);
	else if(step == 'month') {
		if(offset) {
			var d = date.getDate(),
				m = date.getMonth(),
				y = date.getFullYear();
			m += offset;
			while(m >= 12) {
				y ++;
				m -= 12;
			}
			while(m < 0) {
				y --;
				m += 12;
			}
			date.setDate(1);
			date.setFullYear(y);
			date.setMonth(m);
			date.setDate(Math.min(d, date.getMonthDays()));
			offset = 0;
		}
	}
	else if(step == 'year') {
		if(offset) {
			date.setFullYear(date.getFullYear() + offset);
			offset = 0;
		}
	}
	var info = this.calcMonthInfo(date),
		idx = info.currentIndex;
	idx += offset;
	while(1) {
		if(idx < info.startOffset) {
			idx = idx - info.startOffset + 1;
			date.setDate(0);
			info = this.calcMonthInfo(date);
			idx += info.currentIndex;
			continue;
		}
		if(idx >= info.endOffset) {
			idx -= info.endOffset;
			date.setDate(info.no_days+1);
			info = this.calcMonthInfo(date);
			idx += info.startOffset;
			continue;
		}
		if(! info.days[idx].disabled)
			break;
		idx += sgn;
	}
	if(this.setDate(info.days[idx].date))
		this.callSelectHandler();
}


/**
 *  (RE)Initializes the calendar to the given date and firstDayOfWeek
 */
Calendar.prototype._init = function (date, firstDayOfWeek) {
	if(isset(Calendar.dateToday))
		this.today = new Date(parseFloat(Calendar.dateToday));
	else
		this.today = new Date();
	if(date)
		this.date = new Date(date);
	else if(! this.date)
		this.date = new Date(this.today);
	if(isset(firstDayOfWeek)) this.firstDayOfWeek = firstDayOfWeek;
	var info = this.month_info = this.calcMonthInfo();
	if(! this.body)
		return;
	
	var row, cell, idx = 0, dayinfo;
	SUGAR.ui.clearChildNodes(this.body);
	for (var i = 0; i < this.rows.length; i++) {
		row = this.rows[i];
		date = info.days[i*7].date;
		cell = row.firstChild;
		if (this.weekNumbers) {
			SUGAR.ui.setElementText(cell, '' + app_string('LBL_WEEK_PREFIX') + date.getWeekNumber());
			cell = cell.nextSibling;
		}
		var hasdays = false;
		for (var j = 0; j < 7; ++j, cell = cell.nextSibling) {
			cell.dayindex = idx;
			dayinfo = info.days[idx ++];

			SUGAR.ui.addRemoveClass(cell, 'outside', dayinfo.otherMonth);
			SUGAR.ui.addRemoveClass(cell, 'inside', ! dayinfo.otherMonth);
			SUGAR.ui.addRemoveClass(cell, 'current', dayinfo.today);
			SUGAR.ui.addRemoveClass(cell, 'selected', dayinfo.selected);
			SUGAR.ui.addRemoveClass(cell, 'quiet', dayinfo.weekend);

			if(cell.disabled) {
				SUGAR.ui.setElementText(cell, '');
				cell.ttip = '';
				SUGAR.ui.removeClass(cell, 'responsive');
			} else {
				SUGAR.ui.setElementText(cell, dayinfo.date.getDate());
				if(! dayinfo.otherMonth) hasdays = true;
				cell.ttip = "_";
				if(dayinfo.today)
					cell.ttip += Calendar._TT["PART_TODAY"];
				SUGAR.ui.addClass(cell, 'responsive');
			}
		}
		if(hasdays) this.body.appendChild(row);
	}
	this.updateTitle();
	//this.onSetTime(); // FIXME
};


Calendar.prototype.calcMonthInfo = function(sel_date) {
	if(! sel_date) sel_date = this.date;
	var date = new Date(sel_date),
		year = sel_date.getFullYear(),
		month = sel_date.getMonth(),
		mday = sel_date.getDate(),
		no_days = sel_date.getMonthDays(),
		info = {
			year: year, month: month, mday: mday, no_days: no_days
		},
		weekend = this.weekend || Calendar._TT["WEEKEND"] || "0,6",
		dayinfo,
		status_func = (typeof(this.getDateStatus) == "function") ? this.getDateStatus : null;
	// calendar voodoo for computing the first day that would actually be
	// displayed in the calendar, even if it's from the previous month.
	// WARNING: this is magic. ;-)
	date.setDate(1);
	var day1 = (date.getDay() - this.firstDayOfWeek) % 7, start_offs;
	if (day1 < 0)
		day1 += 7;
	start_offs = (day1 ? -day1 : 0) + 1;
	// longreach - modified - setDate(-0) raises illegal access error in Chrome
	if(start_offs != 1)
		date.setDate(start_offs);
	info.startOffset = day1;
	info.endOffset = day1 + info.no_days;

	info.days = [];
	for(var i = 0; i < 42; i++) {
		dayinfo = {otherMonth: false, today: false, weekend: false, selected: false};
		dayinfo.date = new Date(date);
		var cls = 'day',
			iday = date.getDate();
		if(date.getMonth() != month) {
			dayinfo.otherMonth = true;
			if (! this.showsOtherMonths)
				dayinfo.disabled = true;
		}
		if(! dayinfo.disabled) {
			if(status_func) {
				var status = status_func(date, year, month, iday);
				if (status === true) {
					dayinfo.disabled = true;
				} else {
					if (/disabled/i.test(status))
						dayinfo.disabled = true;
					cls += ' ' + trim(status);
				}
			}
		}
		if(weekend.indexOf(date.getDay().toString()) != -1) {
			dayinfo.weekend = true;
		}
		if(date.equalsTo(this.today, false)) {
			dayinfo.today = true;
		}
		if(date.equalsTo(sel_date, false)) {
			info.currentIndex = i;
			dayinfo.selected = true;
		}
		info.days.push(dayinfo);
		date.setDate(date.getDate() + 1);
	}
	return info;
}

Calendar.prototype.updateTitle = function() {
	SUGAR.ui.setElementText(this.title, Calendar._MN[this.date.getMonth()] + ' ' + this.date.getFullYear());
}

/**
 *  Calls _init function above for going to a certain date (but only if the
 *  date is different than the currently selected one).
 */
Calendar.prototype.setDate = function (date) {
	if(!date) date = new Date();
	if (!date.equalsTo(this.date, false)) {
		this._init(date);
		return true;
	}
};

/**
 *  Refreshes the calendar.  Useful if the "disabledHandler" function is
 *  dynamic, meaning that the list of disabled date can change at runtime.
 *  Just * call this function if you think that the list of disabled dates
 *  should * change.
 */
Calendar.prototype.refresh = function () {
	this._init();
};

/** Modifies the "firstDayOfWeek" parameter (pass 0 for Synday, 1 for Monday, etc.). */
Calendar.prototype.setFirstDayOfWeek = function (firstDayOfWeek) {
	this._init(null, firstDayOfWeek);
	this._displayWeekdays();
};

/**
 *  Allows customization of what dates are enabled.  The "unaryFunction"
 *  parameter must be a function object that receives the date (as a JS Date
 *  object) and returns a boolean value.  If the returned value is true then
 *  the passed date will be marked as disabled.
 */
Calendar.prototype.setDateStatusHandler = Calendar.prototype.setDisabledHandler = function (unaryFunction) {
	this.getDateStatus = unaryFunction;
};

/** Customization of allowed year range for the calendar. */
Calendar.prototype.setRange = function (a, z) {
	this.minYear = a;
	this.maxYear = z;
};

/** Calls the first user handler (selectedHandler). */
Calendar.prototype.callSelectHandler = function () {
	if (this.onselect) {
		this.onselect(this, this.printDate(), this.date);
	}
};

/** Calls the first user handler (selectedHandler). */
Calendar.prototype.callUpdateHandler = function () {
	if (this.onchange) {
		this.onchange(this, this.printDate(), this.date);
	}
};

/** Calls the second user handler (closeHandler). */
Calendar.prototype.callCloseHandler = function () {
	if (this.onclose) {
		this.onclose(this);
	}
};

Calendar.prototype.callStatusHandler = function(msg, is_default) {
	if(this.onstatus) {
		this.onstatus(this, msg, is_default);
	}
};

Calendar.prototype.showStatus = function(msg, is_default) {
	this.callStatusHandler(msg, is_default);
	if(this.tooltips)
		SUGAR.ui.setElementText(this.tooltips, msg || '');
}

/** Removes the calendar object from the DOM tree and destroys it. */
Calendar.prototype.destroy = function () {
	var o = this.outer, el;
	if(o && (el = o.parentNode))
		el.removeChild(o);
};

/**
 *  Moves the calendar element to a different section in the DOM tree (changes
 *  its parent).
 */
Calendar.prototype.reparent = function (new_parent) {
	if(this.outer)
		new_parent.appendChild(this.outer);
};


/** Clear highlight attribute on rows and cells. */
Calendar.prototype.clearHighlight = function () {
	var rows = this.rows, row, cell, i;
	for (i = 0; i < this.rows.length; i++) {
		row = rows[i];
		SUGAR.ui.removeClass(row, 'rowhilite');
		for (cell = row.firstChild; cell; cell = cell.nextSibling) {
			SUGAR.ui.removeClass(cell, 'hilite');
			SUGAR.ui.removeClass(cell, 'active');
		}
	}
};

/**
 *  Return selected calendar date in native calendar format or custom format
 */
Calendar.prototype.printDate = function (format) {
	return this.date.print(format || this.dateFormat);
};


/** Customizes the date format. */
Calendar.prototype.setDateFormat = function (str) {
	this.dateFormat = str;
};

/** Customizes the tooltip date format. */
Calendar.prototype.setTtDateFormat = function (str) {
	this.ttDateFormat = str;
};

Calendar.parseDate = function(str, fmt) {
	var y = 0;
	var m = -1;
	var d = 0;
	var a = str.split(/\W+/);
	var b = fmt.match(/%./g);
	var i = 0, j = 0;
	var hr = 0;
	var min = 0;
	for (i = 0; i < a.length; ++i) {
		if (!a[i])
			continue;
		switch (b[i]) {
			case "%d":
			case "%e":
			d = parseInt(a[i], 10);
			break;

			case "%m":
			m = parseInt(a[i], 10) - 1;
			break;

			case "%Y":
			case "%y":
			y = parseInt(a[i], 10);
			(y < 100) && (y += (y > 29) ? 1900 : 2000);
			break;

			case "%b":
			case "%B":
			for (j = 0; j < 12; ++j) {
				if (Calendar._MN[j].substr(0, a[i].length).toLowerCase() == a[i].toLowerCase()) { m = j; break; }
			}
			break;

			case "%H":
			case "%I":
			case "%k":
			case "%l":
			hr = parseInt(a[i], 10);
			break;

			case "%P":
			case "%p":
			if (/pm/i.test(a[i]) && hr < 12)
				hr += 12;
			break;

			case "%M":
			min = parseInt(a[i], 10);
			break;
		}
	}
	if (y != 0 && m != -1 && d != 0) {
		return new Date(y, m, d, hr, min, 0);
	}
	y = 0; m = -1; d = 0;
	for (i = 0; i < a.length; ++i) {
		if (a[i].search(/[a-zA-Z]+/) != -1) {
			var t = -1;
			for (j = 0; j < 12; ++j) {
				if (Calendar._MN[j].substr(0, a[i].length).toLowerCase() == a[i].toLowerCase()) { t = j; break; }
			}
			if (t != -1) {
				if (m != -1) {
					d = m+1;
				}
				m = t;
			}
		} else if (parseInt(a[i], 10) <= 12 && m == -1) {
			m = a[i]-1;
		} else if (parseInt(a[i], 10) > 31 && y == 0) {
			y = parseInt(a[i], 10);
			(y < 100) && (y += (y > 29) ? 1900 : 2000);
		} else if (d == 0) {
			d = a[i];
		}
	}
	if (y == 0) {
		var today = new Date();
		y = today.getFullYear();
	}
	if (m != -1 && d != 0) {
		return new Date(y, m, d, hr, min, 0);
	}
	return null;
}

/**
 *  Tries to identify the date represented in a string.  If successful it also
 *  calls this.setDate which moves the calendar to the given date.
 */
Calendar.prototype.parseDate = function (str, fmt) {
	var d = Calendar.parseDate(str, fmt || this.dateFormat);
	if(d) this.setDate(d);
};


/** Internal function; it displays the bar with the names of the weekday. */
Calendar.prototype._displayWeekdays = function () {
	var fdow = this.firstDayOfWeek;
	var cell = this.firstdayname;
	var weekend = Calendar._TT["WEEKEND"];
	if(! weekend) weekend = "0,6";
	for (var i = 0; i < 7; ++i) {
		var realday = (i + fdow) % 7;
		if (i) {
			var lbl = Calendar._TT["DAY_FIRST"];
			if(! lbl) lbl = "%s";
			cell.ttip = lbl.replace("%s", Calendar._DN[realday]);
			cell.navtype = 100;
			cell.fdow = realday;
			Calendar._add_evs(cell, this);
		}
		SUGAR.ui.addRemoveClass(cell, 'weekend', (weekend.indexOf(realday.toString()) != -1));
		SUGAR.ui.setElementText(cell, Calendar._SDN[(i + fdow) % 7]);
		cell = cell.nextSibling;
	}
};

/** Internal function.  Hides all combo boxes that might be displayed. */
Calendar.prototype._hideCombos = function () {
	this.monthsCombo.style.display = "none";
	this.yearsCombo.style.display = "none";
};


// BEGIN: DATE OBJECT PATCHES

/** Adds the number of days array to the Date object. */
Date._MD = [31,28,31,30,31,30,31,31,30,31,30,31];

/** Constants used for time computations */
Date.SECOND = 1000 /* milliseconds */;
Date.MINUTE = 60 * Date.SECOND;
Date.HOUR   = 60 * Date.MINUTE;
Date.DAY    = 24 * Date.HOUR;
Date.WEEK   =  7 * Date.DAY;

Date.prototype.isLeapYear = function(year) {
	if (! isset(year)) {
		year = this.getFullYear();
	}
	if ((0 == (year%4)) && ( (0 != (year%100)) || (0 == (year%400)))) {
		return 1;
	}
	return 0;
}

/** Returns the number of days in the current month */
Date.prototype.getMonthDays = function(month, year) {
	if (! isset(year)) {
		year = this.getFullYear();
	}
	if (! isset(month)) {
		month = this.getMonth();
	}
	if (this.isLeapYear(year) && month == 1) {
		return 29;
	} else {
		return Date._MD[month];
	}
};

/** Returns the number of day in the year. */
Date.prototype.getDayOfYear = function() {
	var now = new Date(this.getFullYear(), this.getMonth(), this.getDate(), 0, 0, 0);
	var then = new Date(this.getFullYear(), 0, 0, 0, 0, 0);
	var time = now - then;
	return Math.floor(time / Date.DAY);
};

/** Returns the number of the week in year, as defined in ISO 8601. */
Date.prototype.getWeekNumber = function() {
	var d = new Date(this.getFullYear(), this.getMonth(), this.getDate(), 0, 0, 0);
	var DoW = d.getDay();
	d.setDate(d.getDate() - (DoW + 6) % 7 + 3); // Nearest Thu
	var ms = d.valueOf(); // GMT
	d.setMonth(0);
	d.setDate(4); // Thu in Week 1
	return Math.round((ms - d.valueOf()) / (7 * 864e5)) + 1;
};

/** Checks dates equality */
Date.prototype.equalsTo = function(date, check_time) {
	if (!date) return false;
	var ret = ((this.getFullYear() == date.getFullYear()) &&
		(this.getMonth() == date.getMonth()) &&
		(this.getDate() == date.getDate()));
	if(ret && check_time) ret &=
		((this.getHours() == date.getHours()) &&
		(this.getMinutes() == date.getMinutes()));
	return ret;
};

/** Prints the date in a string according to the given format. */
Date.prototype.print = function (str) {
	SUGAR.ui.CalendarInput.initLanguage();
	var m = this.getMonth(),
		d = this.getDate(),
		y = this.getFullYear(),
		wn = this.getWeekNumber(),
		w = this.getDay(),
		hr = this.getHours(),
		pm = (hr >= 12),
		ir = ((pm) ? (hr - 12) : hr) || 12,
		dy = this.getDayOfYear(),
		min = this.getMinutes(),
		sec = this.getSeconds(),
		s = {
			"%a": Calendar._SDN[w], // abbreviated weekday name [FIXME: I18N]
			"%A": Calendar._DN[w], // full weekday name
			'%b': Calendar._SMN[m], // abbreviated month name [FIXME: I18N]
			'%B': Calendar._MN[m], // full month name
			// FIXME: %c : preferred date and time representation for the current locale
			'%C': 1 + Math.floor(y / 100), // the century number
			'%d': (d < 10) ? ("0" + d) : d, // the day of the month (range 01 to 31)
			'%e': d, // the day of the month (range 1 to 31)
			// FIXME: %D : american date style: %m/%d/%y
			// FIXME: %E, %F, %G, %g, %h (man strftime)
			'%H': (hr < 10) ? ("0" + hr) : hr, // hour, range 00 to 23 (24h format)
			'%I': (ir < 10) ? ("0" + ir) : ir, // hour, range 01 to 12 (12h format)
			'%j': (dy < 100) ? ((dy < 10) ? ("00" + dy) : ("0" + dy)) : dy, // day of the year (range 001 to 366)
			'%k': hr,		// hour, range 0 to 23 (24h format)
			'%l': ir,		// hour, range 1 to 12 (12h format)
			'%m': (m < 9) ? ("0" + (1+m)) : (1+m), // month, range 01 to 12
			'%M': (min < 10) ? ("0" + min) : min, // minute, range 00 to 59
			'%n': "\n",		// a newline character
			'%p': pm ? "PM" : "AM",
			'%P': pm ? "pm" : "am",
			// FIXME: %r : the time in am/pm notation %I:%M:%S %p
			// FIXME: %R : the time in 24-hour notation %H:%M
			'%s': Math.floor(this.getTime() / 1000),
			'%S': (sec < 10) ? ("0" + sec) : sec, // seconds, range 00 to 59
			'%t': "\t",		// a tab character
			// FIXME: %T : the time in 24-hour notation (%H:%M:%S)
			'%u': w + 1,	// the day of the week (range 1 to 7, 1 = MON)
			'%U': (wn < 10) ? ("0" + wn) : wn,
			// %V, %W below
			'%w': w,		// the day of the week (range 0 to 6, 0 = SUN)
			// FIXME: %x : preferred date representation for the current locale without the time
			// FIXME: %X : preferred time representation for the current locale without the date
			'%y': ('' + y).substr(2, 2), // year without the century (range 00 to 99)
			'%Y': y,		// year with the century
			'%%': "%"		// a literal '%' character
		};
	s['%W'] = s['%V'] = s['%U'];

	var re = /%./g;
	var isSafari=navigator.userAgent.toLowerCase().indexOf ("safari")!=-1;
	if (!isSafari)
		return str.replace(re, function (par) { return s[par] || par; })

	var a = str.match(re);
	for (var i = 0; i < a.length; i++) {
		var tmp = s[a[i]];
		if (tmp) {
			re = new RegExp(a[i], 'g');
			str = str.replace(re, tmp);
		}
	}

	return str;
};

Date.prototype.__msh_oldSetFullYear = Date.prototype.setFullYear;
Date.prototype.setFullYear = function(y) {
	var d = new Date(this);
	d.__msh_oldSetFullYear(y);
	if (d.getMonth() != this.getMonth())
		this.setDate(28);
	this.__msh_oldSetFullYear(y);
};

// END: DATE OBJECT PATCHES
