GalleryLightbox = function (images, conn, totalCount, parentType, parentId, limit) {
	this.images = images || [];
	this.conn = conn;
	this.totalCount = totalCount;
	this.parentType = parentType;
	this.parentId = parentId;
	this.limit = limit;

	this.overlay = document.createElement('div');
	this.overlay.id = 'lightbox-overlay';
	this.overlay.style.display = 'none';
	this.overlay.style.position = 'fixed';
	this.overlay.style.top = '0';
	this.overlay.style.left = '0';
	this.overlay.style.width = '100%';
	this.overlay.style.height = '100%';
	this.overlay.style.backgroundColor = 'rgba(0,0,0,0.8)';
	this.overlay.style.zIndex = '1000';
	document.body.appendChild(this.overlay);

	this.container = document.createElement('div');
		this.container.id = 'lightbox-container';
	this.container.className = 'card-outer dialog-outer popup-dialog carousel-dialog resizable';
	this.container.style.position = 'fixed';
	this.container.style.top = '50%';
	this.container.style.left = '50%';
	this.container.style.transform = 'translate(-50%, -50%)';
	this.container.style.zIndex = '1001';
	this.container.style.textAlign = 'center';
	this.container.style.minHeight = '30%';
	this.container.style.minWidth = '90%';
	document.body.appendChild(this.container);

	const header = document.createElement('div');
	header.className = 'card-header dialog-header tools-row align-top';
	header.style.borderBottom = '1px solid #b9cddc';
	header.style.paddingBottom = '1px';

	const title = document.createElement('span');
	title.className = 'dialog-title';
	title.style.textAlign = 'left';
	title.innerHTML = '';
	header.appendChild(title);

	const closeButton = document.createElement('div');
	closeButton.className = 'uii uii-cancel uii-lg active-icon dialog-close';
	closeButton.addEventListener('click', () => {
		this.hide();
	});
	header.appendChild(closeButton);

	this.container.appendChild(header);

	this.prevButton = document.createElement('button');
	this.prevButton.innerHTML = '&larr;';
	this.prevButton.style.position = 'absolute';
	this.prevButton.style.left = '10px';
	this.prevButton.style.top = '50%';
	this.prevButton.style.transform = 'translateY(-50%)';
	this.prevButton.style.zIndex = '1002';
	this.prevButton.style.background = 'none';
	this.prevButton.style.border = 'none';
	this.prevButton.style.color = '#fff';
	this.prevButton.style.fontSize = '2rem';
	this.prevButton.style.cursor = 'pointer';
	this.prevButton.onclick = (e) => {
		e.stopPropagation();
		this.showPrevImage();
	};
	this.overlay.appendChild(this.prevButton);

	this.nextButton = document.createElement('button');
	this.nextButton.innerHTML = '&rarr;';
	this.nextButton.style.position = 'absolute';
	this.nextButton.style.right = '10px';
	this.nextButton.style.top = '50%';
	this.nextButton.style.transform = 'translateY(-50%)';
	this.nextButton.style.zIndex = '1002';
	this.nextButton.style.background = 'none';
	this.nextButton.style.border = 'none';
	this.nextButton.style.color = '#fff';
	this.nextButton.style.fontSize = '2rem';
	this.nextButton.style.cursor = 'pointer';
	this.nextButton.onclick = (e) => {
		e.stopPropagation();
		this.showNextImage();
	};
	this.overlay.appendChild(this.nextButton);

	this.image = document.createElement('img');
	this.image.style.maxWidth = '40%';
	this.image.style.maxHeight = '30%';
	this.image.style.paddingTop = '5px';
	this.image.style.className = 'img-carousel';
	this.container.appendChild(this.image);

	this.overlay.onclick = () => {
		this.hide();
	};

	document.addEventListener('keydown', (e) => {
		if (e.key === 'Escape') {
			this.hide();
		} else if (e.key === 'ArrowLeft') {
			this.showPrevImage();
		} else if (e.key === 'ArrowRight') {
			this.showNextImage();
		}
	});

	this.overlay.appendChild(this.container);
};

GalleryLightbox.prototype.showImage = function (index) {
	this.currentIndex = index;
	const id = this.images[this.currentIndex].id;
	const img = this.images[this.currentIndex].src;
	let title = this.images[this.currentIndex].title || '';

	if (!title) {
		title = decodeURIComponent(img.substring(img.lastIndexOf('/') + 1));
	}

	const titleElement = this.container.querySelector('.dialog-title');
	if (titleElement) {
		titleElement.innerHTML = title;
	}

	this.image.src = img;

	const existingLink = this.container.querySelector('.open-in-new-window');
	if (existingLink) {
		this.container.removeChild(existingLink);
	}

	const openInNewWindow = document.createElement('div');
	openInNewWindow.className = 'open-in-new-window';
	openInNewWindow.style.margin = '10px';
	openInNewWindow.innerHTML = '<div class="input-icon icon-tlink"></div>&nbsp;' +
		'<a href="download.php?type=Notes&id=' + id + '&field=filename&inline=1" target="_blank" class="tabDetailViewDFLink">' + app_string('LNK_OPEN_IN_NEW_WINDOW') + '</a>';

	this.container.insertBefore(openInNewWindow, this.image);

	this.overlay.style.display = 'block';
	this.container.style.display = 'block';
};

GalleryLightbox.prototype.showNextImage = function () {
	if (this.images.length > 1) {
		if (this.currentIndex >= this.images.length - 1 && this.images.length < this.totalCount) {
			this.loadMoreImages(() => {
				this.currentIndex++;
				this.showImage(this.currentIndex);
			});
		} else {
			this.currentIndex = (this.currentIndex + 1) % this.images.length;
			this.showImage(this.currentIndex);
		}
	}
};

GalleryLightbox.prototype.showPrevImage = function () {
	if (this.currentIndex === 0 && this.images.length < this.totalCount) {
		this.loadMoreImages((newImageCount) => {
			this.currentIndex = newImageCount - 1;
			this.showImage(this.currentIndex);
		}, true);
	} else {
		this.currentIndex = (this.currentIndex - 1 + this.images.length) % this.images.length;
		this.showImage(this.currentIndex);
	}
};

GalleryLightbox.prototype.hide = function () {
	this.overlay.style.display = 'none';
	if (this.overlay.parentNode) {
		this.overlay.parentNode.removeChild(this.overlay);
	}
	if (this.container.parentNode) {
		this.container.parentNode.removeChild(this.container);
	}
};

GalleryLightbox.prototype.loadMoreImages = function (callback, loadPrevious = false) {
	const self = this;
	const listLimit = this.limit;
	let currentPage;

	if (loadPrevious) {
		currentPage = Math.ceil(this.images.length / listLimit);
	} else {
		currentPage = Math.ceil(this.images.length / listLimit) + 1;
	}

	if (this.images.length >= this.totalCount) {
		return;
	}

	const params = {
		module: "Activities",
		action: "ShowImages",
		parent_type: this.parentType,
		parent_id: this.parentId,
		list_offset: loadPrevious ? Math.max(0, this.images.length - listLimit).toString() : (listLimit * (currentPage - 1)).toString(),
		list_limit: listLimit.toString(),
	};

	const req = new this.conn.HTTPRequest('async.php');
	req.setMethod('POST');
	req.setArgs(params);

	req.fetch(
		function (response) {
			const parser = new DOMParser();
			const doc = parser.parseFromString(response.responseText, 'text/html');

			const carouselImages = doc.querySelectorAll('img.img-carousel');
			const newImages = [];

			carouselImages.forEach(img => {
				newImages.push({
					id: img.getAttribute('data-id'),
					src: img.src,
					title: img.getAttribute('data-name'),
				});
			});

			if (loadPrevious) {
				self.images = newImages.concat(self.images);
				callback(newImages.length);
			} else {
				self.images = self.images.concat(newImages);
				callback(self.images.length - newImages.length);
			}
		},
	);
};
