
AppBase.module('colors').moduleRequires('ui-controls').init(function(App) {
	var UI = App.ui,
		Dom = App.dom,
		Wrap = Dom.extNode,
		MakeElt = Dom.makeElement;
	
	UI.fetchColorMap = function(callb) {
		if(UI.colorMap) {
			callb(UI.colorMap);
			return;
		}
		if(! UI.onRetrieveColorMap) {
			var hooks = UI.onRetrieveColorMap = [];
			var req = new SUGAR.conn.HTTPRequest('include/javascript/colors/colors.json');
			req.fetch(function() {
				UI.colorMap = JSON.parse(this.responseText);
				for(var i = 0; i < hooks.length; i++)
					hooks[i](UI.colorMap);
			});
		}
		UI.onRetrieveColorMap.push(callb);
	}

	UI.ColorTextInput = function(id, params) {
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		if(params) extendObject(this, params);
	};
	UI.ColorTextInput.prototype = {
		setup: function() {
			if(this.set_up) return;
			if(! this.elt && ! (this.elt = $(this.id)) )
				return;				
			if(! this.field) {
				if(! (this.field = Wrap(this.elt).findFirst('.input-text').get(0)) )
					return;
			}
			if(! this.icon_elt) {
				if(! (this.icon_elt = Wrap(this.elt).findFirst('.input-color-icon').get(0)) )
					return;
			}
			UI.fetchColorMap(function(val) {
				this.colorMap = val;
				this.updateDisplay();
			}.bind(this));
			this.addHooks();
			this.set_up = true;
		},
		addHooks: function() {
			var events = {
				focus: this.handleFocus,
				blur: this.handleBlur,
				change: this.handleChange,
				input: this.handleInput
			};
			this.events = UI.addEventListeners(this.field, events, this);
		},
		getValue: function(asNative) {
			var ret = this.field && this.field.value;
			// if asNative convert to [r,g,b,a]
			return ret;
		},
		focus: function() {
			this.field && this.field.focus();
		},
		handleFocus: function() {
			if(! this.focused) {
				this.focused = true;
				UI.callEvent(this, this.onfocus);
				this.updateDisplay();
			}
		},
		handleBlur: function() {
			if(this.focused) {
				this.focused = false;
				UI.callEvent(this, this.onblur);
				this.updateDisplay();
			}
		},
		handleChange: function() {
			var c = this.parseColor(this.getValue());
			if(c) this.field.value = c.clean;
			this.validate();
		},
		validate: function() {
			var val = this.getValue(),
				c = this.parseColor(val);
			this.invalid = false;
			if(! val) {
				if(this.required) {
					this.invalid = true;
					this.invalidMsg = 'required';
				}
				return;
			}
			if(! c) {
				this.invalid = true;
				this.invalidMsg = 'invalid';
			}
			this.updateDisplay();
			return ! this.invalid;
		},
		makeHex: function(parts) {
			if(! parts) return '';
			var ret = '#', hex;
			for(var i = 0; i < parts.length && i < 4; i++) {
				var part = parts[i];
				if(i == 3) {
					if(part === undefined || part == 1) continue;
					part = Math.round(part * 255);
				}
				hex = part.toString(16);
				ret += hex.length == 1 ? '0' + hex : hex;
			}
			return ret;
		},
		parseColor: function(color) {
			if(! color) return false;
			var c = trim(color).toLowerCase();
			if(this.colorMap && c in this.colorMap)
				return {clean: c, hex: this.makeHex(this.colorMap[c]), rgba: this.colorMap[c]};
			if(c[0] === '#')
				c = c.substring(1);
			if(c.length === 3) {
				// duplicate parts
				c = c[0] + c[0] + c[1] + c[1] + c[2] + c[2];
			}
			var alpha = 255;
			if(c.length == 8 && this.allow_alpha) {
				alpha = parseInt(c.substring(6, 8), 16);
				c = c.substring(0, 6);
			}
			if(c.length != 6) return false;
			var r = parseInt(c.substring(0, 2), 16),
				g = parseInt(c.substring(2, 4), 16),
				b = parseInt(c.substring(4, 6), 16);
			if(isNaN(r) || isNaN(g) || isNaN(b) || isNaN(alpha)) return false;
			var rgba = [r, g, b, alpha / 255];
			var hex = this.makeHex(rgba);
			return {clean: hex, hex: hex, rgba: rgba};
		},
		handleInput: function() {
			this.updateDisplay();
		},
		updateDisplay: function(val) {
			var val = this.parseColor(this.getValue());
			val = val && val.hex;
			UI.addRemoveClass(this.elt, 'active', !! this.focused);
			UI.addRemoveClass(this.elt, 'invalid', !! this.invalid);
			setAttr(this.icon_elt.style, 'backgroundColor', val ? val : null);
			UI.addRemoveClass(this.icon_elt, 'disabled', ! val);
		}
	};

});
