AppBase.module('conn').init(function(App) {
	var Conn = this;
	this.loaded = true;
	this.reqCount = 0;
	this.pendingConns = {};
	
	this.asyncRequest = function(url, params, callback, errback) {
		var req = new HTTPRequest(url || 'async.php', params);
		return req.fetch(callback, errback);
	}
	
	this.sendForm = function(form, conn_params, callback, errback, custom_values) {
		var req = new HTTPRequest(null, conn_params);
		req.setUrl(null);
		if(req.loadForm(form, custom_values)) {
			if(form.name) {
				for(var k in this.pendingConns) {
					if(! this.pendingConns[k].loaded && this.pendingConns[k].requireForm === form.name)
						this.pendingConns[k].abort();
				}
			}
			if(conn_params && conn_params.url)
				req.setUrl(conn_params.url);
			else if(! req.getUrl())
				req.setUrl('async.php');
			req.sentForm = form;
			req.fetch(callback, errback);
			return req;
		}
	}
	
	this.abortAll = function(criteria) {
		Conn.abortingAll = true;
		var conns = Conn.pendingConns;
		if(conns) {
			for(var k in conns)
				if(conns[k] && (! criteria || criteria(conns[k])))
					conns[k].abort();
		}
		Conn.abortingAll = false;
	}
	this.abortPageRequests = function() {
		Conn.abortAll(function(c) { return !c.global; });
	}
	this.abortRequest = function(id) {
		var conn = Conn.pendingConns[id];
		if(conn) conn.abort();
	}
	
	var FormArgs = function(classic) {
		this.classic = classic;
		if(window.FormData && ! classic)
			this.fdata = new FormData();
		else
			this.args = [];
	}
	FormArgs.prototype = {
		append: function(key, value) {
			if(! isset(key)) return;
			value = get_default(value, '');
			if(this.fdata) {
				if(value.push) {
					for(var v=0; v < value.length; v++)
						this.fdata.append(key, value[v]);
				} else
					this.fdata.append(key, value);
			} else {
				key = encodeURIComponent(key);
				if(value.push) {
					for(var v=0; v < value.length; v++)
						this.args.push(key + '=' + encodeURIComponent(value[v]));
				} else {
					this.args.push(key + '=' + encodeURIComponent(value));
				}
			}
		},
		getData: function() {
			if(this.fdata)
				return this.fdata;
			return this.args.join('&');
		},
		addFile: function(f) {
			if(this.classic) {
				if(! this.files) this.files = [];
				this.files.push(f);
			} else if(f.files && f.files[0])
				this.append(f.name, f.files[0]);
		},
		getFiles: function() {
			return (this.files && this.files.length) ? this.files : null;
		}
	}
	this.FormArgs = FormArgs;
	
	
	var HTTPRequest = function(url, params) {
		this.url = url || 'index.php';
		this.method = 'GET';
		this.args = null;
		this.postData = null;
		this.silent = false;
		this.status_msg = app_string('LBL_LOADING');
		this.content_type = 'application/x-www-form-urlencoded';
		this.timeout = 60000;
		this.max_retries = 0;
		this.async = true;
		this.request_id = (++ Conn.reqCount);
		if(params) {
			if(params.postData) {
				this.loadPostData(params.postData);
				delete params.postData;
			}
			extendObject(this, params);
		}
		this.retries = 0;
		this.progressPercent = null;
	}
	HTTPRequest.prototype = {
		setMethod: function(method) {
			this.method = method;
		},
		setArgs: function(args) {
			this.args = args;
		},
		setPostData: function(data) {
			this.method = 'POST';
			this.postData = data;
		},
		loadPostData: function(data) {
			if(this.method == 'GET')
				this.method = 'POST';
			if(typeof(data) == 'object')
				this.setPostData(encodeQueryString(data));
			else
				this.setPostData(data);
		},
		setUrl: function(url) {
			this.url = url;
		},
		getMethod: function() {
			return this.method;
		},
		getArgs: function() {
			return this.args;
		},
		getPostData: function() {
			if(! isset(this.postData) || this.postData === '') return null;
			return this.postData;
		},
		getUrl: function() {
			return this.url;
		},
		loadForm: function(form, custom_values) {
			var form = $(form) || document.forms[form];
			if(! form)
				return false;
			var method = (form.getAttribute('method') || '').toUpperCase(),
				action = form.getAttribute('action'),
				args = new FormArgs(true);
			this.loadFormValues(form, args, custom_values);
			if(method == 'GET' || method == 'POST')
				this.setMethod(method);
			if(isset(action))
				this.setUrl(action);
			if(this.method == 'GET')
				this.setArgs(args.getData());
			else
				this.setPostData(args.getData());
			if(method == 'POST')
				this.setFiles(args.getFiles());
			return true;
		},
		loadFormValues: function(form, args, custom_values) {
			var skip_custom = {};
			if(! custom_values)
				custom_values = {};
			for(var i = 0; i < form.elements.length; i++) {
				var elt = form.elements[i],
					key = get_default(elt.name, ''),
					value;
				if(! key.length)
					continue;
				if(isset(custom_values[key])) {
					args.append(key, custom_values[key]);
					skip_custom[key] = 1;
					continue;
				}
				if( ((elt.type == 'radio' || elt.type == 'checkbox') && ! elt.checked)
					|| (elt.tagName == 'BUTTON' || elt.type == 'button' || elt.type == 'submit' || elt.type == 'reset'))
						continue;
				if(elt.type == 'file') {
					if(elt.value)
						args.addFile(elt);
					continue;
				}
				if(elt.type == 'select-one' && elt.selectedIndex < 0)
					continue;
				if(elt.type == 'select-multiple') {
					for(var j = 0; j < elt.options.length; j++) {
						var opt = elt.options[j];
						if(opt.selected)
							args.append(key, opt.value);
					}
				} else {
					args.append(key, elt.value);
				}
			}
			for(var key in custom_values) {
				if(! skip_custom[key]) {
					args.append(key, custom_values[key]);
				}
			}
			return args;
		},
		addFile: function(f, params, file_list) {
			if(! this.files) this.files = [];
			this.files.push({input: f, name: f.name, params: params, files: file_list});
		},
		setFiles: function(fs) {
			this.files = [];
			if(fs) {
				for(var i = 0; i < fs.length; i++) {
					this.files.push({input: fs[i], name: fs[i].name});
				}
			}
		},
		revertFileInputs: function(clear) {
			if(this.files) {
				for(var i = 0; i < this.files.length; i++)
					this.fileReplaceStub(this.files[i], true, clear);
				this.files = [];
			}
		},
		fileReplaceStub: function(f, revert, clear) {
			if(! f.input) return;
			if(revert) {
				if(f.stub.parentNode)
					f.stub.parentNode.replaceChild(f.input, f.stub);
				f.input.name = f.name;
				if (clear)
					f.input.value = '';
				return f.input;
			} else {
				var fname = ((f.files && f.files.length) ? f.files[0].name : f.input.value), m = fname.match(/([^\/\\]+)$/);
				if(m) fname = m[1];
				f.stub = createElement2('div', {className: 'input-label'}, fname);
				f.save = createElement2('input', {type: 'hidden', name: f.name}, null, f.stub);
				if(f.input.parentNode)
					f.input.parentNode.replaceChild(f.stub, f.input);
				return f.stub;
			}
		},
		fail: function(msg) {
			this.setDuration();
			this.success = false;
			this.failure_msg = msg;
			this.setReadyState(0);
			this.hideStatus();
			this.oncomplete();
			this.onerror.apply(this.callback_scope || this, [null, this]);
			this.cleanup();
		},
		getRequest: function() {
			var url = this.getUrl();
			var args = encodeQueryString(this.getArgs());
			if(args !== '') {
				if(url.lastIndexOf('?') != -1)
					url += '&';
				else
					url += '?';
				url += args;
			}
			return url;
		},
		setReadyState: function(state) {
			this.readyState = state;
			this.onreadystatechange(state);
		},
		reset: function() {
			this.resetConnection();
		},
		resetConnection: function() {
			this.conn = null;
			this.success = null;
			this.setReadyState(0);
			this.responseText = '';
			this.responseXML = null;
			this.status = 0;
			this.statusText = '';
			this.progressPercent = null;
		},
		open: function() {
			this.reset();
			this.openStart = new Date();
			var c = new XMLHttpRequest(),
				method = this.getMethod(),
				action = this.getRequest(),
				csrf_cookie,
				self = this;
			csrf_cookie = 
				window.document.cookie.split(";")
					.map(function(str) {return str.trim()})
					.map(function(str) {return str.split('=')})
					.find(function(parts) {return parts[0] === 'UNKNOWN'});
			try {
				c.open(method, action, this.async ? true : false);
			} catch(e) {
				return this.fail("error opening connection");
			}
			if(this.async && isset(c.ontimeout) && isset(this.timeout)) {
				c.timeout = this.timeout;
				c.ontimeout = function() { self.ontimeout(); }
			}
			if(this.async) {
				c.onreadystatechange = function() { self._onreadystatechange(); }
				if(this.timeout && ! c.timeout) {
					// set timer to abort (NYI)
				}
				if(c.upload)
					c.upload.addEventListener("progress", function(evt) { self.uploadProgress(evt) }, false);
			}
			if(this.content_type) {
				if(! window.FormData || ! (this.postData instanceof FormData))
					c.setRequestHeader('Content-Type', this.content_type);
			}
			if (csrf_cookie) {
				c.setRequestHeader('X-OneCRM-CSRF-Token', csrf_cookie[1]);
			}
			this.conn = c;
			if(c.readyState != 1)
				return this.fail("connection not ready");
			this.setReadyState(1);
			return true;
		},
		fetchSync: function() {
			// sync transfer does not support timeout (in older browsers)
			this.async = false;
			return this.fetch();
		},
		fetch: function(callback, errback) {
			this.fetchStart = new Date();
			if(callback)
				this.onresponse = callback;
			if(errback)
				this.onerror = errback;
			if(this.async && this.files && this.files.length)
				return this.uploadFiles();
			return this.performFetch();
		},
		performFetch: function() {
			if(! this.silent && this.status_msg)
				this.showStatus(this.status_msg);
			if(this.open()) {
				Conn.pendingConns[this.request_id] = this;
				var data = this.getPostData();
				this.conn.send(data);
			}
			if(! this.async) {
				this._onreadystatechange();
				if(! this.success) {
					if(this.success === null)
						return this.fail(this.failure_msg || "unspecified connection failure");
					return;
				}
				return this.getResult();
			}
			return this;
		},
		uploadFiles: function() {
			var me = this, f, newname, i, j;
			var req, data = new FormData();
			data.append('user_id', SUGAR.session.user_id);
			for(i = 0; i < this.files.length; i++) {
				newname = 'file' + i;
				f = this.files[i];
				if(! (f.files && f.files.length) && ! (f.input && f.input.files && f.input.files.length)) continue;
				data.append(newname + '_meta', f.name);
				if(f.params) {
					for(j = 0; j < f.params.length; j++)
						data.append(newname + '_' + f.params[j].name, f.params[j].value);
				}
				this.fileReplaceStub(f);
				f.upload_name = newname;
				if(f.files)
					data.append(newname, f.files[0]);
				else
					data.append(newname, f.input.files[0]);
			}
			req = new JSONRequest(null, {url: 'upload.php', content_type: 'multipart/form-data'}, data)
			req.fetch(function() {
				me.checkLoad = true;
				me.completeUpload(this.result);
			});
		},
		completeUpload: function(result) {
			if(! this.checkLoad)
				return;
			var i, f;
			this.hideStatus();
			if(this.iframe) {
				result = this.iframe.contentWindow.upload_result;
				document.body.removeChild(this.iframe);
				this.iframe = null;
			}
			if(! result || ! result.files) {
				for(i = 0; i < this.files.length; i++) {
					this.fileReplaceStub(this.files[i], true);
					this.files[i].result = {error: 1};
				}
				return this.fail("File upload failed");
			} else {
				var revert = this.force_revert;
				var clear = this.clear_on_revert;
				for(i = 0; i < this.files.length; i++) {
					f = this.files[i];
					if(result.errors[f.upload_name]) {
						clear = false;
						revert = true;
					}
				}
				if (revert) {
					for(i = 0; i < this.files.length; i++) {
						this.fileReplaceStub(this.files[i], true, clear);
						this.files[i].result = {error: 1};
					}
				}
				var addp = {}, inc_id;
				for(i = 0; i < this.files.length; i++) {
					f = this.files[i];
					if(result.errors[f.upload_name])
						return this.fail(result.errors[f.upload_name]);
					if(! result.files[f.upload_name].id)
						return this.fail("Missing ID for uploaded file");
					inc_id = 'INCOMING~' + result.files[f.upload_name].id;
					if(f.save) f.save.value = inc_id;
					addp[f.name] = inc_id;
					f.result = result.files[f.upload_name];
				}
				if(! this.upload_manual_add)
					this.postData += '&' + encodeQueryString(addp);
			}
			if(this.onupload)
				this.onupload.apply(this, [this.files]);
			else
				this.performFetch();
		},
		cleanup: function() {
			if(this.conn) {
				this.conn.onreadystatechange = null;
				this.conn = null;
			}
			this.setReadyState(0);
			Conn.pendingConns[this.request_id] = false;
		},
		pending: function() {
			return (this.readyState != 0);
		},
		setDuration: function() {
			var now = new Date();
			if(this.fetchStart)
				this.fetchTime = now.getTime() - this.fetchStart.getTime();
			if(this.openStart)
				this.openTime = now.getTime() - this.openStart.getTime();
		},
		onreadystatechange: function(state) {},
		_onreadystatechange: function() {
			if(! this.conn || this.success === false || ! this.readyState) return;
			var state = this.conn.readyState;
			if(! state) {
				// likely timed out
				this.setDuration();
				this.setReadyState(state);
				this.ontimeout();
				return;
			}
			if(this.readyState < state)
				this.setReadyState(state);
			if(state < 4)
				return;
			try {
				this.status = this.conn.status;
				this.statusText = this.conn.statusText
			} catch(e) {
				// FF may throw NS_ERROR_NOT_AVAILABLE
				this.status = 0;
				this.statusText = '';
			}
			if(this.status == 200) {
				this.responseText = this.conn.responseText;
				this.responseXML = this.conn.responseXML;
				if(this.handleResponse()) {
					this.setDuration();
					this.success = true;
					this.hideStatus();
					this.oncomplete();
					this.onresponse.apply(this.callback_scope || this, [this]);
					this.cleanup();
				}
			}
			else if(this.status == 401) {
				this.setDuration();
				window.not_logged_in = true;
                this.hideStatus();
                App.asyncLogin.showLoginPopup(this);
				//this.fail(app_string('NTC_JS_SESSION_ENDED'));
			} else if(this.status == 403) {
				this.fail(app_string('NTC_JS_FORBIDDEN'));
			} else if(this.status == 400) {
				this.fail("Bad JSON request (status 400): " + this.conn.responseText);
			} else if(this.status == 0) {
				// likely aborted, may also be lost connection
				this.fail(''); // ('Network connection lost');
			} else
				this.fail("There was a problem with the connection:\n" + this.statusText + " (" + this.status + ")");
		},
		uploadProgress: function(evt) {
			if(! evt.lengthComputable)
				this.progressPercent = -1;
			else
				this.progressPercent = Math.round(evt.loaded * 100 / evt.total);
			if(this.onprogress)
				this.onprogress();
		},
		handleResponse: function() {
			return true;
		},
		getResult: function() {
			return this;
		},
		attemptRetry: function(fail_msg) {
			if(this.retries < this.max_retries) {
				this.retries ++;
				// fixme - do in setTimeout
				this.fetch();
			} else
				this.fail(fail_msg);
		},
		showStatus: function(msg, timeout) {
			this.hideStatus();
			if(App.moduleLoaded('ui-controls'))
				return (this.status_id = App.ui.Status.show(msg, timeout));
		},
		hideStatus: function() {
			if(this.status_id) App.ui.Status.remove(this.status_id);
			this.status_id = null;
		},
		abort: function() {
			if(this.conn) {
				this.conn.abort();
				this.aborted = true;
			}
			Conn.pendingConns[this.request_id] = false;
		},
		getError: function() {
			return this.failure_msg;
		},
		oncomplete: function() {},
		onresponse: function() {},
		ontimeout: function() {
			this.abort(); // may be called by setTimeout
			this.aborted = false;
			this.attemptRetry("connection timed out");
		},
		onerror: function(zero, self) {
			if(! this.silent && self.failure_msg) {
				this.showStatus(self.failure_msg, 4000);
			}
		}
	};
	this.HTTPRequest = HTTPRequest;


	var JSONRequest = function(action, params, postData) {
		var defaults = {
			method: 'POST',
			content_type: 'application/json',
			action: action,
			postData: postData};
		if(! params) params = {};
		extendObject(params, defaults);
		HTTPRequest.call(this, 'json.php', params);
		this.upload_manual_add = true;
	}
	extendClass(JSONRequest, HTTPRequest, {
		getUrl: function() {
			var url = this.url;
			if(this.action) {
				if(url.lastIndexOf('?') != -1)
					url += '&';
				else
					url += '?';
				url += 'action='+encodeURIComponent(this.action);
			}
			return url;
		},
		loadPostData: function(data) {
			this.setPostData(data);
		},
		getPostData: function() {
			if(! isset(this.postData) || this.postData === '') return null;
			if(typeof(this.postData) === 'string' || (window.FormData && this.postData instanceof FormData))
				return this.postData;
			return JSON.stringify(this.postData);
		},
		handleResponse: function() {
			try {
				this.result = this.responseText.length ? JSON.parse(this.responseText) : null;
			} catch(e) {
				console.error(e, this.responseText);
				return this.fail(e.toString());
			}
			if(! this.result) return this.fail('No result');
			return true;
		},
		fetchJSON: function(callb, errb) {
			return this.fetch(function() {
				var r = this.getResult();
				if(this.success && r) {
					callb.apply(this.callback_scope || this, [r, this]);
				} else {
					this.fail(this.failure_msg || 'Error decoding response');
				}
			}, errb);
		},
		loadForm: function(form, custom_values) {
			this.json_args = new Conn.JSONArgs();
			this.json_args.skip_action = true;
			this.loadFormValues(form, this.json_args, custom_values);
			this.setPostData(this.json_args.getData());
			this.setFiles(this.json_args.getFiles());
		},
		getResult: function() {
			return this.result;
		},
		reset: function() {
			this.resetConnection();
			this.result = undefined;
		},
		onupload: function(files) {
			if(files && this.json_args) {
				for(var i = 0; i < files.length; i++) {
					if(files[i].save)
						this.json_args.append(files[i].name, files[i].save.value);
				}
				this.setPostData(this.json_args.getData());
			}
			this.performFetch();
		}
	});
	this.JSONRequest = JSONRequest;
	
	
	this.JSONArgs = function() {
		this.args = {};
		this.skip_action = true;
	}
	this.JSONArgs.prototype = {
		append: function(key, value) {
			if(! isset(key)) return;
			if(key == 'action' && this.skip_action) return;
			if(key.match(/^[a-zA-Z_][a-zA-Z0-9_]*$/))
				this.args[key] = value;
		},
		getData: function() {
			return JSON.stringify(this.args);
		},
		addFile: function(f) {
			if(! this.files) this.files = [];
			this.files.push(f);
		},
		getFiles: function() {
			return (this.files && this.files.length) ? this.files : null;
		}
	};
	
	
	this.ProgressChecker = function(hooks) {
		this.hooks = hooks || {};
	}
	this.ProgressChecker.prototype = {
		start: function(start_action, poll_action, interval) {
			var self = this, url;
			this.start_action = start_action;
			this.start_time = new Date();
			this.poll_action = poll_action || start_action;
			this.interval = interval || 300;
			this.last_poll = null;
			this.percent = 0;
			this.process = '';
			this.timestamp = '';
			this.completed = false;
			this.history = [];
			this.poll_count = 0;
			this.in_progress = true;
			if(this.hooks.start) this.hooks.start.call(this);
			if(! this.start_url)
				this.start_url = this.hooks.start_url || ('json.php?action={ACTION}' + (this.hooks.module ? '&module='+this.hooks.module : ''));
			if(! this.poll_url)
				this.poll_url = this.hooks.poll_url || ('json.php?action={ACTION}&timestamp={TIMESTAMP}&progress_id={PROGRESS_ID}' + (this.hooks.module ? '&module='+this.hooks.module : ''));
			url = this.start_url.replace('{ACTION}', this.start_action);
			if(this.hooks.start_http)
				this.run_req = new Conn.HTTPRequest(url, {status_msg: null});
			else
				this.run_req = new Conn.JSONRequest(null, {method: 'GET', url: url, status_msg: null});
			this.run_req.fetch(
				function(r) { self.run_response(r); },
				function() { self.error_response(this, true); });
			if(this.hooks.sync_run)
				this.poll(300);
		},
		get_duration: function(millis) {
			var ms = (new Date()).getTime() - this.start_time.getTime(), mins, ret = '';
			if(millis) return ms;
			ms = Math.round(ms / 1000);
			mins = Math.floor(ms / 60);
			if(mins) {
				ret = mins + 'm ';
				ms -= mins * 60;
			}
			return ret + ms + 's';
		},
		poll: function(wait) {
			var self = this,
				now = new Date(),
				callb = function(r) { self.poll_response(r); },
				errb = function() { self.error_response(this); },
				url = this.poll_url;
			url = url.replace('{ACTION}', this.poll_action).replace('{TIMESTAMP}', encodeURIComponent(this.timestamp||'')).replace('{PROGRESS_ID}', encodeURIComponent(this.progress_id||''));
			this.poll_req = new Conn.JSONRequest(null, {method: 'GET', url: url, status_msg: null});
			if(! wait) wait = this.interval || 0;
			if(wait && this.last_poll) wait = Math.max(0, wait - now.getTime() + this.last_poll.getTime());
			setTimeout(function() { self.poll_count ++; self.last_poll = new Date(); self.poll_req.fetch(callb, errb); }, wait);
		},
		error_response: function(req, initial) {
			this.failure(req.getError());
		},
		run_response: function(req) {
			var data;
			if(this.hooks.sync_run)
				this.in_progress = false;
			if(! this.hooks.start_http && ! this.hooks.sync_run) {
				data = req.getResult();
				if(data)
					this.progress_id = data['progress_id'];
				if(! this.progress_id) {
					this.failure("Task not started");
					return;
				}
			}
			if(this.hooks.start_complete) this.hooks.start_complete.call(this);
			if(! this.hooks.sync_run)
				this.poll(300);
		},
		poll_response: function(req) {
			var data, perc, proc;
			if(! req) return this.failure('lost_conn');
			data = req.getResult();
			if(! data) return this.failure(req.getError());
			this.timestamp = data.timestamp;
			if (data.history) {
				for(var i = 0; i < data.history.length; i++) {
					this.history.push(data.history[i]);
					if(this.hooks.history) this.hooks.history.call(this, data.history[i]);
				}
			}
			if(data.finished) {
				this.in_progress = false;
				this.status = data.status || 'success';
				this.status_msg = data.status_message || '';
				if(this.status == 'success') {
					this.progress(100);
					return this.complete();
				}
				else
					return this.failure(this.status_msg);
			}
			if(data.current_process) {
				proc = data.current_process;
				perc = parseFloat(data.current_progress);
				if(isNaN(perc) || perc > 100) perc = 100;
				perc = Math.floor(perc / 100 * 98);
			} else {
				perc = 0;
			}
			this.progress(perc, proc);
			if(this.in_progress) {
				this.poll();
			} else {
				this.failure(this.hooks.std_failure_msg || '');
			}
		},
		progress: function(perc, current) {
			this.percent = perc;
			this.process = current;
			if(this.hooks.progress) this.hooks.progress.call(this, this.percent, this.process);
		},
		complete: function() {
			this.completed = true;
			if(this.run_req && this.in_progress) this.run_req.abort();
			if(this.hooks.complete) this.hooks.complete.call(this, this.status, this.status_msg);
		},
		failure: function(msg) {
			this.error_msg = msg || this.hooks.std_failure_msg || '';
			if(this.hooks.error) this.hooks.error.call(this, this.error_msg);
			else if(this.hooks.complete) this.hooks.complete.call(this, this.status, this.status_msg);
		}
	};
});


// simple test:
//var j = new SUGAR.conn.JSONRequest('get_version');
//alert(JSON.stringify(j.fetchSync()));


// for compatibility only

var json_objects = {};
function call_json_method(module,action,vars,variable_name,callback,options) {
	if(isset(window.not_logged_in))
		return false;
	if(! options) options = {};
	var silent = false;
	if(options.all_silent || options.hide_status_msg)
		silent = true;
	var async = (callback && ! options.sync) ? true : false; // not redundant, need false not undefined
	if(! variable_name) variable_name = module + ':' + action;
	json_objects[variable_name] = null;
	var args = encodeQueryString({module: module}) + '&' + (vars || '');
	var J = new AppBase.conn.JSONRequest(action, {args: args, async: async, silent: silent});
	J.oncomplete = function() { json_objects[variable_name] = this.getResult(); }
	return J.fetch(callback);
}


function getXMLHTTPinstance() {
	return new XMLHttpRequest();
}

function http_fetch_sync(url, post_data) {
	var method = isdef(post_data) ? 'POST' : 'GET';
	var req = new AppBase.conn.HTTPRequest(url, {method: method, postData: post_data});
	return req.fetchSync();
}

function http_fetch_async(url,callback,request_id,post_data) {
	var method = isdef(post_data) ? 'POST' : 'GET';
	return AppBase.conn.asyncRequest(url, {method: method, request_id: request_id, postData: post_data}, callback);
}

// not used
var global_xmlhttp = null;
