AppBase.extendModule('ui-controls', 'ui-dynlist', function(App) {

	var Wrap = App.dom.extNode,
		UI = this,
		MakeElt = App.dom.makeElement;
		
	var DynamicList = UI.DynamicList = function(id, params) {
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		this.cols = [];
		this.rows = [];
		this.sections = null;
		this.active_section = -1;
		this.active_indexes = [];
		this.hover_section = -1;
		this.hover_index = -1;
		this.depth_attrib = 'depth';
		this.fixed_attrib = '_fixed';
		this.group_attrib = '_group';
		this.require_group_attrib = false;
		this.strip_attrib = '_init';
		this.max_depth = 1;
		this.depth_column = null;
		this.show_header = 1;
		this.show_handles = 1;
		this.show_action = 0;
		this.show_delete = 1;
		this.renderer = null;
		if(params) extendObject(this, params);
	}
	DynamicList.prototype = {
		setup: function() {
			if(this.set_up) return;
			if(! this.elt)
				this.elt = $(this.id);
			if(! this.renderer) {
				if(this.renderer_class) {
					var c = resolveVar(this.renderer_class);
					if(c) this.renderer = new c();
				}
			}
			if(! this.field) {
				if(this.name && this.form)
					this.field = this.form.elements[this.name];
				else if(this.field_id)
					this.field = $(this.field_id);
			}
			this.set_up = true;
			if(this.elt && ! this.defer_render)
				this.render();
		},
		getRow: function(sect_index, index) {
			var rows = this.getRows(sect_index);
			return rows[index];
		},
		getRows: function(sect_idx) {
			if(isset(sect_idx)) {
				if(this.sections[sect_idx])
					return this.sections[sect_idx].rows;
				else
					return [];
			} else if(this.sections) {
				var ret = [];
				for(var i = 0; i < this.sections.length; i++) {
					ret.pushArray(this.sections[i].rows);
				}
				return ret;
			}
			return this.rows || [];
		},
		getSectionCount: function() {
			return this.sections ? this.sections.length : 0;
		},
		initSections: function() {
			var s, i, rows = this.getRows();
			if(this.renderer && this.renderer.getListSections)
				s = this.renderer.getListSections(rows, this);
			if(! s || ! s.length)
				s = [{name: 'default', rows: rows}];
			for(i = 0; i < s.length; i++)
				s[i].show_label = isset(s[i].label);
			this.sections = s;
			var a = [];
			for(var i = 0; i < this.sections.length; i++)
				a.push(this.sections[i].rows.length);
		},
		insertRow: function(row, sect_index, index, no_render, silent) {
			if(typeof(row) != 'object')
				return;
			if(! isset(sect_index)) {
				if(this.renderer && this.renderer.getInsertSection) {
					sect_index = this.renderer.getInsertSection(row, this.active_section, this);
				} else if(this.active_section >= 0) {
					sect_index = this.active_section;
				} else
					sect_index = this.sections.length - 1;
			}
			var rows = this.getRows(sect_index), ma, da = this.depth_attrib, pr;
			if(! isset(index) && this.active_section == sect_index && this.active_indexes.length) {
				ma = Math.max.apply(0, this.active_indexes);
				if(ma >= 0 && ma < rows.length) {
					if(da && ! isset(row[da])) {
						if(this.active_indexes.length == 1 && this.group_attrib && (pr = rows[this.active_indexes[0]])[this.group_attrib]) {
							// if currently selected row is a group, nest underneath it
							row[da] = pr[da] + 1;
						} else {
							row[da] = Math.min(rows[ma][da], this.max_depth);
							if(ma < rows.length-1)
								row[da] = Math.max(row[da], rows[ma+1][da]);
						}
					}
					index = ma+1;
					this.active_indexes = [index];
				}
			}
			if(! isset(index) || index > rows.length || index < 0) {
				index = rows.length;
				if(index > 0 && da && rows[index-1].type == 'filtergroup') {
					// nest underneath existing filter group if it's the last entry in the list
					row[da] = rows[index-1][da] + 1;
				}
			}
			if(da && ! isset(row[da])) row[da] = 0;
			this.processNewRow(row);
			rows.splice(index, 0, row);
			this.initSections();
			if(! silent)
				UI.callEvent(this, this.onupdate, sect_index, index);
			UI.markFormDirty(this.form);
			if(! no_render)
				this.renderInsertRow(sect_index, index);
			return index;
		},
		processNewRow: function(row) {
		},
		removeRow: function(sect_index, row_index, no_render, silent) {
			var rows = this.getRows(sect_index);
			if(! rows) return;
			if(row_index < 0 || row_index >= rows.length) {
				return;
			}
			var i = row_index+1, j, row = rows[row_index], c, na=[], rem;
			if(this.depth_attrib) {
				for(; i < rows.length; i++) {
					if(! isset(rows[i][this.depth_attrib]) || rows[i][this.depth_attrib] <= get_default(row[this.depth_attrib], 0))
						break;
				}
			}
			c = i-row_index;
			rem = rows.splice(row_index, c);
			if(rem) {
				for(i = 0; i < rem.length; i++) {
					if(rem[i]._init && rem[i]._init.ctls)
						UI.unregisterInputs(this.form, rem[i]._init.ctls);
				}
			}
			this.initSections();
			if(this.hover_section == sect_index) {
				for(i = 0; i < this.active_indexes.length; i++) {
					j = this.active_indexes[i];
					if(j < row_index) na.push(j);
					else if(j >= row_index+c) na.push(j-c);
				}
				this.active_indexes = na;
			}
			if(this.hover_section == sect_index && this.hover_index >= row_index && this.hover_index < row_index + c)
				this.hover_index = -1;
			if(! silent)
				UI.callEvent(this, this.onupdate, sect_index, row_index);
			UI.markFormDirty(this.form);
			if(! no_render) {
				//for(i = rpos+c-1; i >= rpos; i--)
				//	this.elt.deleteRow(i);
				//for(i = index; i < this.rows.length; i++)
				//	this.renderRowIndex(i);
				this.renderSections();
			}
		},
		updateRow: function(sect_index, row_index, values, no_render, silent) {
			var rows = this.getRows(sect_index);
			if(! rows || ! rows[row_index]) return;
			for(var k in values)
				rows[row_index][k] = values[k];
			this.initSections();
			if(! silent)
				UI.callEvent(this, this.onupdate, sect_index, row_index);
			UI.markFormDirty(this.form);
			if(! no_render)
				//this.renderRowIndex(index);
				this.renderSections();
		},
		moveRow: function(sect_index, index, new_index, depth, no_render, silent) {
			this.moveRows(sect_index, [index], new_index, depth, no_render, silent);
		},
		moveRows: function(sect_index, indexes, new_index, depth, no_render, silent) {
			var rows = this.getRows(sect_index),
				i = 0, j = 0,
				newact = [], upd = [],
				r = [], m = [], mi = [], md = [], d, di, li;
			if(new_index < 0) new_index = 0;
			if(new_index > rows.length) new_index = rows.length;
			indexes = Array.flatten(indexes).unique();
			if(! indexes.length) return;
			indexes.sort(function(a, b) { return a - b; });
			
			for(i = 0; i <= indexes.length; i++) {
				if(indexes[i] < rows.length) {
					m.push(rows[indexes[i]]);
					mi.push(indexes[i]);
					if(this.depth_attrib) {
						di = rows[indexes[i]][this.depth_attrib];
						if(isset(li) && li == indexes[i] - 1) {
							// this row is adjacent to the previous one, we can nest
							if(di < d) d = di;
						}
						else
							d = di;
						md.push(di - d);
					}
					li = indexes[i];
				}
			}

			for(i = 0; i <= rows.length; i++) {
				if(i == new_index) {
					for(j = 0; j < m.length; j++) {
						if(this.isActiveIndex(sect_index, mi[j]))
							newact.push(r.length);
						if(r.length != mi[j])
							upd.push(r.length);
						if(this.depth_attrib && isset(depth))
							m[j][this.depth_attrib] = Math.min(md[j] + depth, this.max_depth);
						r.push(m[j]);
					}
				}
				if(i == rows.length || mi.indexOf(i) >= 0)
					continue;
				else {
					if(this.isActiveIndex(sect_index, i))
						newact.push(r.length);
					if(r.length != i)
						upd.push(r.length);
					r.push(rows[i]);
				}
			}
			
			this.sections[sect_index].rows = r;
			this.active_indexes = newact;
			if(! silent)
				UI.callEvent(this, this.onupdate, sect_index, new_index);
			UI.markFormDirty(this.form);
			if(! no_render)
				this.renderSections();
			return upd;
		},
		_clearTableRows: function(parent, from) {
			if(! from) from = 0;
			for(var i = parent.rows.length-1; i >= from; i--)
				parent.deleteRow(i);
		},
		renderSections: function() {
			if(this.sections) {
				var i, j;
				for(i = 0; i < this.sections.length; i++)
					this.renderSection(i);
				for(j = this.elt.tBodies.length-1; j >= i && j > 0; j--)
					this.elt.removeChild(this.elt.tBodies[j]);
				if(! this.sections.length)
					this._clearTableRows(this.elt.tBodies[0]);
			}
			var popup = SUGAR.ui.PopupManager.currentModal();
			if(popup && Wrap(popup.elt).contains(this.elt))
				popup.reposition();
		},
		renderSection: function(index) {
			var sect = this.sections[index], tbody = this.elt.tBodies[index], tr, i;
			if(! sect)
				return;
			if(! tbody) {
				while(index >= this.elt.tBodies.length) {
					tbody = createElement2('tbody');
					this.elt.appendChild(tbody);
				}
			}
			if(sect.show_label) {
				tr = tbody.rows[0];
				if(! tr || tr.className != 'listSectionHead') {
					this._clearTableRows(tbody);
					tr = createElement2('tr', {className: 'listSectionHead'});
					tbody.appendChild(tr);
					tr.appendChild(createElement2('th'));
				}
				tr.cells[0].colSpan = this.getColSpan();
				UI.setElementText(tr.cells[0], sect.label);
			}
			this.renderSectionRows(index);
		},
		renderInsertRow: function(sect_index, index, no_update) {
			//this.elt.insertRow(this.getRenderIndex(index));
			//this.renderRow(index);
			this.renderSections();
		},
		renderSectionRows: function(sect_index) {
			if(! this.sections || ! this.sections[sect_index]) return;
			var sect = this.sections[sect_index],
				tbody = this.elt.tBodies[sect_index],
				offs = 0, tr, i;
			if(sect.show_label) offs ++;
			for(i = 0; i < sect.rows.length; i++) {
				tr = tbody.rows[offs+i];
				if(! tr) {
					tr = tbody.insertRow(offs+i);
					tr.className = 'listViewRow';
				}
				tr.section = sect_index;
				tr.sectRowIndex = i;
				this.renderRow(tr, sect_index, i);
			}
			this._clearTableRows(tbody, offs+i);
		},
		renderRow: function(elt, sect_index, index) {
			var i = 0, offs = 0, cell, c, value, rval, done,
				row = this.getRow(sect_index, index);
			if(this.show_handles) {
				cell = elt.cells[offs];
				if(! cell) cell = elt.insertCell(offs);
				else UI.clearChildNodes(cell);
				if(! this.fixed_attrib || ! row[this.fixed_attrib]) {
					var div = createElement2('div', {'class': 'uii uii-md uii-hamburger drag-handle active-icon'});
					cell.appendChild(div);
					this.makeDragTarget(div, {section: elt.section, index: elt.sectRowIndex});
				} else
					cell.appendChild(nbsp());
				cell.style.verticalAlign = 'middle';
				offs ++;
			}
			for(; i < this.cols.length; i++) {
				c = this.cols[i];
				cell = elt.cells[i+offs];
				if(! cell) cell = elt.insertCell(i+offs);
				else UI.clearChildNodes(cell);
				UI.addRemoveClass(cell, 'depthColumn', (this.depth_column ? this.depth_column == c.name : i == 0));
				value = row[c.name];
				done = false;
				if(c.render_method && this.renderer) {
					rval = this.renderer[c.render_method](row, c.name, sect_index, index, this, cell);
					if(! isset(rval))
						done = true;
					else if(rval !== false) {
						UI.setElementContent(cell, rval);
						done = true;
					}
				}
				
				if(done) {
					// rendered
				} else if(c.format == 'html') {
					cell.innerHTML = value;
				} else if(c.format == 'icon') {
					if(c.name == 'module') value = 'theme-icon module-' + value;
					cell.appendChild(createElement2('div', {'class': 'input-icon '+value}));
				} else if(c.format == 'check' || c.format == 'hidden') {
					var show = 1, edit = 1;
					if(c.format == 'hidden')
						if(this.depth_attrib && row[this.depth_attrib])
							show = 0;
					//if(this.fixed_attrib && row[this.fixed_attrib])
					//	edit = 0;
					if(show) {
						(function() {
						var chk = value ? '.checked' : '',
							div = MakeElt('div.uii.uii-checkbox.active-icon'+chk),
							upd = {};
						upd[c.name] = value ? 0 : 1;
						div.on('mousedown', function(evt) { evt.stopPropagation(); });
						div.on('click', function(evt) { 
							this.updateRow(sect_index, index, upd);
							return false;
						}, {scope: this});
						div.appendTo(cell);
						}.bind(this))();
					} else
						cell.appendChild(nbsp());
				} else if(c.editable) {
					var editor = this,
						v = (value === '') ? createElement2('em', null, app_string('LBL_MISSING_VALUE')) : value,
						span = createElement2('div', {className: 'input-field flat'},
							createElement2('div', {className: 'input-label'}, v));
					span.c = c;
					span.idx = [sect_index, index];
					ml = c.multiline;
					span.size = c.size || 20;
					var fn = function(val) {
					span.onclick = function(evt) {
						var edit = new UI.QuickText({size: this.c.size || 20, show_reset: false, multiline: this.c.multiline, format: this.c.format, label: this.c.label}),
							key = this.c.name,
							idx = this.idx;
						edit.init(val, function() {
							var upd = {}; upd[key] = this.getValue();
							editor.updateRow(idx[0], idx[1], upd);
						});
						edit.showPopup(0, this);
						UI.stopEvent(evt);
					}
					};
					fn(value);
					cell.appendChild(span);
				} else
					UI.setElementText(cell, value);
			}
			if(this.show_action) {
				cell = elt.cells[i+offs];
				if(! cell) cell = elt.insertCell(i+offs);
				else UI.clearChildNodes(cell);
				cell.className = 'utilsCell';
				if(! this.fixed_attrib || ! row[this.fixed_attrib]) {
					var acts = this.getMenuActions(row, sect_index, index);
					if(acts) {
						var div = createElement2('div', {'class': 'uii uii-gear active-icon'}),
							src = new UI.MenuSource(div, {options: acts}),
							self = this;
						src.onchange = function(key, val) { self.performMenuAction(sect_index, index, key, val, div); }
						cell.appendChild(div);
						src.setup();
					} else
						cell.appendChild(nbsp());
				}
				offs ++;
			}
			if(this.show_delete) {
				cell = elt.cells[i+offs];
				if(! cell) cell = elt.insertCell(i+offs);
				else UI.clearChildNodes(cell);
				cell.className = 'utilsCell';
				var showDelete = ! this.fixed_attrib || ! row[this.fixed_attrib];
				if (this.renderer && this.renderer.canDelete) {
					showDelete = this.renderer.canDelete(showDelete, row, sect_index, index, this);
				}
				if(showDelete) {
					var div = MakeElt('div.uii.uii-remove.active-icon')
						.on('mousedown', function(evt) { evt.stopPropagation(); })
						.on('click', function() { this.removeRow(elt.section, elt.sectRowIndex); return false;}, {scope: this})
						.appendTo(cell);
				}
				offs ++;
			}
			this.addRowHooks(elt, index);
			while(elt.cells.length > i+offs)
				elt.deleteCell(elt.cells.length-1);
			this.updateSectionRowStyle(elt.section, elt.sectRowIndex);
			if(this.renderer && this.renderer.postFormatRow) {
				this.renderer.postFormatRow(elt, row, sect_index, index, this);
			}
		},
		addRowHooks: function(row, index) {
			Wrap(row)
				.on('mouseover', function() { this.setHoverIndex(row.section, row.sectRowIndex); }, {scope: this})
				.on('mouseout', function() { this.setHoverIndex(null, null); }, {scope: this})
				.on('mousedown', function(evt) { this.handleClickRow(evt, row, index); }, {scope: this});
		},
		handleClickRow: function(evt, row, index) {
			var targ = evt.target;
			while(targ && targ.tagName) {
				var tag = targ.tagName.toLowerCase();
				if(tag == 'tr')
					break;
				if(tag == 'a' || tag == 'input' || tag == 'button' || tag == 'select' || tag == 'textarea')
					return; // ignore event - generated by child element
				if(UI.hasClass(targ, 'active-icon') || UI.hasClass(targ, 'input-field'))
					return;
				targ = targ.parentNode;
			}
		
			if(evt.metaKey || evt.shiftKey)
				this.setActiveIndex(row.section, row.sectRowIndex, -1);
			else
				this.setActiveIndex(row.section, row.sectRowIndex);
			UI.stopEvent(evt);
		},
		isActiveIndex: function(sect_index, row_index) {
			return (this.active_section == sect_index) && (this.active_indexes.indexOf(row_index) >= 0);
		},
		clearActive: function(no_update) {
			this.active_indexes.clear();
			if(! no_update) this.updateDisplay();
		},
		isHoverIndex: function(sect_index, row_index) {
			return (this.hover_section == sect_index) && (this.hover_index == row_index);
		},
		setActiveIndex: function(sect_index, row_index, state, no_update) {
			if(isArray(row_index))
				row_index = row_index.unique();
			else if(isset(row_index))
				row_index = [row_index];
			else
				return;
			var ai = this.active_indexes, rows = this.getRows(sect_index), row, fnd, i, upd = [];
			if(this.active_section != sect_index || ! isset(state)) {
				for(i = 0; i < ai.length; i++)
					upd.push([this.active_section, ai[i]]);
				ai.clear();
			}
			this.active_section = sect_index;
			for(i = 0; i < row_index.length; i++) {
				fnd = ai.remove(row_index[i]);
				row = rows[row_index[i]];
				if(! row)
					continue;
				if(this.fixed_attrib && row[this.fixed_attrib])
					continue;
				if(! isset(state) || (state === -1 && ! fnd) || state === true) {
					ai.push(row_index[i]);
					fnd = 1;
				}
				upd.push([sect_index, row_index[i]]);
			}
			if(! no_update) {
				for(i = 0; i < upd.length; i++)
					this.updateSectionRowStyle.apply(this, upd[i]);
			}
		},
		setHoverIndex: function(sect_index, row_index) {
			if(! isset(sect_index)) sect_index = -1;
			if(! isset(row_index)) row_index = -1;
			var olds = this.hover_section, oldi = this.hover_index;
			if(sect_index == olds && row_index == oldi)
				return;
			this.hover_section = sect_index;
			this.hover_index = row_index;
			if(oldi >= 0) this.updateSectionRowStyle(olds, oldi);
			if(row_index >= 0) this.updateSectionRowStyle(sect_index, row_index);
		},
		getSectionRowElt: function(sect_index, row_index) {
			if(! this.sections || ! this.sections[sect_index]) return;
			if(this.sections[sect_index].show_label)
				row_index ++;
			return this.elt.tBodies[sect_index].rows[row_index];
		},
		updateSectionRowStyle: function(sect_index, row_index) {
			var tr = this.getSectionRowElt(sect_index, row_index),
				row = this.getRow(sect_index, row_index);
			if(! row)
				return;
			if(tr) {
				UI.addRemoveClass(tr, 'evenListRowS1', row_index % 2 == 1);
				UI.addRemoveClass(tr, 'oddListRowS1', row_index % 2 == 0);
				UI.addRemoveClass(tr, 'markRow', this.isActiveIndex(sect_index, row_index));
				UI.addRemoveClass(tr, 'hoverRow', this.isHoverIndex(sect_index, row_index) && ! this.draginfo);
				tr.className = tr.className.replace(/\s*depth(Row|\d)\b/g, '');
				if(this.depth_attrib && row[this.depth_attrib]) {
					UI.addRemoveClass(tr, 'depthRow', true);
					UI.addRemoveClass(tr, 'depth'+row[this.depth_attrib], true);
				}
			}
		},
		updateDisplay: function() {
			var offs, ks, i, j;
			for(i = 0; i < this.sections.length; i++) {
				ks = this.sections[i].keys;
				for(j = 0; j < ks.length; j++)
					this.updateSectionRowStyle(i, j);
			}
		},
		getColSpan: function() {
			var span = this.cols.length;
			if(this.show_handles) span ++;
			if(this.show_action) span ++;
			if(this.show_delete) span ++;
			return span;
		},
		renderHeader: function() {
			if(! this.show_header || ! this.header_elt) return;
			var row, cm, cell, span, c, i, offs = 0;
			for(var i = this.header_elt.rows.length - 1; i >= 0; i--)
				this.header_elt.deleteRow(i);
			row = this.header_elt.insertRow(0);
			row.className = 'listEditorHead';
			cm = row.cells.length;
			if(this.show_handles)
				offs ++;
			for(i = 0; i < offs; i++) {
				cell = row.cells[i];
				if(! cell) {
					cell = document.createElement('th');
					row.appendChild(cell);
				}
				else UI.clearChildNodes(cell);
				cell.appendChild(nbsp());
			}
			for(i = 0; i < this.cols.length; i++) {
				if(i+offs >= cm) {
					cell = document.createElement('th');
					row.appendChild(cell);
				} else {
					cell = row.cells[i+offs];
					UI.clearChildNodes(cell);
				}
				c = this.cols[i];
				UI.setWidth(cell, c.width, '%');
				span = document.createElement('span');
				UI.setElementText(span, c.label);
				cell.appendChild(span);
			}
			var add_c = this.getColSpan() - i - offs;
			while(add_c > 0) {
				if(i+offs >= cm) {
					cell = document.createElement('th');
					row.appendChild(cell);
				} else {
					cell = row.cells[i+offs];
					UI.clearChildNodes(cell);
				}
				cell.appendChild(nbsp());
				offs++;
				add_c --;
			}
		},
		render: function() {
			if(! this.elt)
				this.elt = createElement2('table', {'class': 'listEditor panel-outer panel-default panel-border', cellpadding: 0, cellspacing: 0});
			for(var i = this.elt.rows.length - 1; i >= 0; i--)
				this.elt.deleteRow(i);
			if(this.show_header)
				this.header_elt = this.elt.tHead || this.elt.createTHead();
			this.initSections();			
			this.renderHeader();
			this.renderSections();
		},
		ondragstart: function(evt, info) {
			this.setActiveIndex(info.tag.section, info.tag.index);
			if(this.depth_attrib) {
				// mark inactive child nodes as active
				var sect_idx = info.tag.section,
					rows = this.getRows(sect_idx),
					i, d = 0;
				for(i = 0; i < rows.length; i++) {
					if(this.isActiveIndex(sect_idx, i) && ! d)
						d = rows[i][this.depth_attrib] + 1;
					else if(d && rows[i][this.depth_attrib] >= d)
						this.setActiveIndex(sect_idx, i, true);
					else
						d = 0;
				}
			}
			info.marker = -1;
			this.ondrag(evt, info);
		},
		ondrag: function(evt, info) {
			if(info.scrolled) {
				info.startX += info.scrolled.dx;
				info.startY += info.scrolled.dy;
			}
			var i, tr, left, rows,
				x = info.startX + info.dragX,
				y = info.startY + info.dragY,
				di, dd = info.depth, dp, dn,
				maybe = -1,
				sect_idx = info.tag.section,
				outer = this.elt.tBodies[sect_idx],
				pos = UI.getEltRegion(outer);
			di = -1;
			if((pos.left > x && pos.left - x > 35) || (pos.right < x && x - pos.right > 35))
				; // outside bounds
			else {
				left = pos.left;
				offs = (this.sections[sect_idx].show_label ? 1 : 0);
				rows = this.getRows(sect_idx);
				for(i = 0; i < rows.length; i++) {
					tr = this.getSectionRowElt(sect_idx, i);
					pos = UI.getEltRegion(tr);
					if(pos.top <= y && pos.bottom > y) {
						if(y > (pos.top + pos.bottom) / 2)
							di = i+1;
						else
							di = i;
						break;
					}
					else if(y < pos.top && pos.top - y < 25)
						maybe = i;
					else if(y >= pos.bottom && y - pos.bottom < 25)
						maybe = i+1;
				}
				if(di < 0 && maybe >= 0)
					di = maybe;
				while(di > 0 && this.isActiveIndex(sect_idx, di) && this.isActiveIndex(sect_idx, di-1))
					di --;
				// calculate depth for dragdrop
				dd = 0;
				if(di >= 0 && this.depth_attrib) {
					dp = di - 1;
					while(dp >= 0 && this.isActiveIndex(sect_idx, dp))
						dp --;
					if(dp >= 0) {
						dd = rows[dp][this.depth_attrib] + 1;
						if(this.require_group_attrib && ! rows[dp][this.group_attrib])
							dd --;
						dd = Math.min(Math.round(Math.max(x - left - 20, 0) / 25), dd);
						for(dn = di; dn < rows.length && this.isActiveIndex(sect_idx, dn); dn ++) {}
						if(dn < rows.length)
							dd = Math.max(rows[dn][this.depth_attrib], dd);
					}
					dd = Math.min(dd, this.max_depth);
				}
			}
			if(info.marker != di || info.depth != dd || info.scrolled) {
				info.marker = di;
				info.depth = dd;
				this.showDragEndTarget(info);
			}
		},
		ondragend: function(evt, info) {
			if(! info.canceled && info.marker >= 0 && (info.marker != info.tag.index || this.depth_attrib)) {
				this.moveRows(info.tag.section, this.active_indexes, info.marker, info.depth);
			}
			info.marker = -1;
			this.showDragEndTarget(info);
		},
		showDragEndTarget: function(info) {
			var dm = this.drag_marker;
			if(! dm) {
				dm = this.drag_marker = createElement2('div', {'class': 'drag-row-marker'});
				dm.appendChild(createElement2('div', {'class': 'uii uii-lg uii-next icon-mark-left'}));
				dm.appendChild(createElement2('div', {'class': 'uii uii-lg uii-previous icon-mark-right'}));
				document.body.appendChild(dm);
			}
			if(info.marker >= 0) {
				var di = info.marker,
					rows = this.getRows(info.tag.section),
					last = (di == rows.length), pos, y;
				if(last) di --;
				pos = UI.getEltRegion(this.getSectionRowElt(info.tag.section, di));
				if(pos) {
					y = last ? pos.bottom : pos.top;
					y -= 8;
					setAttrs(dm.style, {
						left: pos.x + 'px',
						top: y + 'px',
						width: pos.width + 'px'});
					dm.firstChild.style.left = (info.depth * 25) + 20 + 'px';
					toggleDisplay(dm, true);
					return true;
				}
			}
			toggleDisplay(dm, false);
		},
		getMenuActions: function(row, sect_index, index) {
			if(this.renderer && this.renderer.getMenuActions)
				return this.renderer.getMenuActions(row, sect_index, index, this);
		},
		performMenuAction: function(sect_index, index, action, action_ext, elt) {
			if(this.renderer && this.renderer.performMenuAction)
				return this.renderer.performMenuAction(this.getRow(sect_index, index), action, sect_index, index, this, action_ext, elt);
		},
		beforeSubmitForm: function() {
			if(this.field) {
				var rows = this.getRows(), rs = [], r, i, k;
				for(i = 0; i < rows.length; i++) {
					r = {};
					for(k in rows[i]) {
						if(k === this.fixed_attrib || k === this.strip_attrib)
							continue;
						r[k] = rows[i][k];
					}
					rs.push(r);
				}
				this.field.value = JSON.stringify(rs);
			}
		},
		clear: function() {
			if(this.sections) {
				for(var i = 0; i < this.sections.length; i++)
					this.sections[i].rows = [];
			}
			this.initSections();
			if(this.elt) this.render();
		}
	}
	extendProto(DynamicList, UI.DragHandler);
	
	
	var ListLayoutRender = UI.ListLayoutRender = function() {
	};
	ListLayoutRender.typeToFormats = {
		date : ['month', 'qtr', 'year', 'yearmonth', 'fqtr', 'fy'],
		datetime : ['date_only', 'time_only', 'month', 'qtr', 'year', 'yearmonth', 'fqtr', 'fy'],
		currency : ['rounded', 'thousands', 'millions'],
		decimal : ['rounded', 'thousands', 'millions'],
		'int' : ['thousands', 'millions'],
		'duration' : ['hours', 'minutes']
	};
	ListLayoutRender.format_options = {};
	ListLayoutRender.prototype = {
		renderName: function(row, field, sect_index, row_index, editor) {
			var value = row[field], extlbl = '', refl,
				elt = createElement2('div', {className: 'input-field flat'},
						createElement2('div', {className: 'input-label'}, value));
			if(this.no_edit_label)
				elt.className = '';
			else
			elt.onclick = function(evt) {
				var edit = new UI.QuickText({size: 20, show_reset: (row.std_label && row.std_label != value)});
				edit.init(value, function() {
					editor.updateRow(sect_index, row_index, {label: this.getValue()});
				});
				edit.onreset = function() {
					editor.updateRow(sect_index, row_index, {label: row.std_label});
				}
				edit.showPopup(0, this);
				UI.stopEvent(evt);
			}
			if(row.std_label && (row.std_label != value || row.ref_label))
				extlbl = row.std_label;
			if(row.ref_label)
				extlbl = row.ref_label + ': ' + extlbl;
			if(extlbl)
				elt = [elt, createElement2('div', {className: 'listAddLine input-field-pad-horz text-muted'}, extlbl)];
			return elt;
		},
		renderType: function(row, field, sect_index, row_index, editor) {
			var str = app_list_strings('field_type_dom');
			if(str) return get_default(str[row[field]], '');
			return row[field];
		},
		renderWidth: function(row, field, sect_index, row_index, editor) {
			var value = trim(row[field]);
			var elt = createElement2('div', {className: 'input-field flat'},
						createElement2('div', {className: 'input-label'}, value || '–'));
			elt.onclick = function(evt) {
				var edit = new UI.QuickText({format: 'int'});
				edit.init(value, function() {
					editor.updateRow(sect_index, row_index, {width: this.getValue()});
				});
				edit.showPopup(0, this);
				UI.stopEvent(evt);
			}
			return elt;
		},
		translateFormat : function(key) {
			if (!key) return app_list_strings('field_format_dom')['none'];
			return get_default(app_list_strings('field_format_dom')[key], key);
		},
		makeFormatOptions : function(type) {
			if (ListLayoutRender.format_options[type])
				return ListLayoutRender.format_options[type];
			var options = {keys: [''], values: [app_list_strings('field_format_dom')['none']]};
			if (ListLayoutRender.typeToFormats[type]) {
				for (var i=0; i < ListLayoutRender.typeToFormats[type].length; i++) {
					var fmt = ListLayoutRender.typeToFormats[type][i];
					options.keys.push(fmt);
					options.values.push(app_list_strings('field_format_dom')[fmt]);
				}
			}
			return ListLayoutRender.format_options[type] = options;
		},
		renderFormat: function(row, field, sect_index, row_index, editor) {
			if (!ListLayoutRender.typeToFormats[row['type']])
				return '';
			var lbl = createElement2('div', {className: 'input-field input-sm flat rbullet'},
							createElement2('div', {className: 'input-label single'}, this.translateFormat(row[field])));
			var render = this;
			lbl.onclick = function(evt) {
				var edit = new UI.QuickSelect({options: render.makeFormatOptions(row['type'])});
				edit.init(get_default(row[field], ''), function(key, val) {
					editor.updateRow(sect_index, row_index, {format: key});
				});
				edit.showPopup(0, this);
				UI.stopEvent(evt);
			}
			return lbl;
		},

		renderSort: function(row, field, sect_index, row_index, editor) {
			if (!row.total)
				return '';
			var iconClass = '';
			var sort = ('' + row.sort).toLowerCase();
			if (sort == 'asc') 
				iconClass = '-up';
			if (sort == 'desc') 
				iconClass = '-down';
			var lbl = createElement2('div', {className: 'input-icon icon-sort' + iconClass, style: {cursor: 'pointer'}});
			var render = this;
			lbl.onclick = function(evt) {
				if (sort == 'asc')
					sort = 'desc';
				else if (sort == 'desc')
					sort = '';
				else
					sort = 'asc';
				editor.updateRow(sect_index, row_index, {sort: sort});
			}
			var outer = createElement2('div');
			var inner = createElement2('div', {className: 'listColLabel with-arrow'});
			outer.appendChild(lbl);
			//outer.appendChild(inner);
			return outer;
		},
		getMenuActions: function(row, sect_index, row_index, editor) {
			if(row.total || ! row.field) return false;
			var opts = {keys: [], values: {'group': app_string('LBL_MAKE_GROUPED'), 'ungroup': app_string('LBL_MAKE_UNGROUPED')}};
			opts.keys.push(row.grouped ? 'ungroup' : 'group');
			return opts;
		},
		performMenuAction: function(row, action, sect_index, row_index, editor, act_ext) {
			if(action == 'group')
				editor.updateRow(sect_index, row_index, {grouped: 1, depth: 0});
			else if(action == 'ungroup')
				editor.updateRow(sect_index, row_index, {grouped: 0});
			else
				return;
		},
		getListSections: function(rows, editor) {
			var srclist = UI.getFormInput(editor.form, 'sources'),
				srcrows, seen = {}, grps = [], ret = [], row, gidx, i;
			if(editor.name == 'totals') return;
			if(! srclist || ! (srcrows = srclist.getRows())) return;
			for(i = 0; i < srcrows.length; i++) {
				seen[srcrows[i].name] = {pos: i};
				grps.push({label: 'LBL_GROUPED_FIELDS', srcpos: i, name: 'grouped-'+srcrows[i].name, rows: []});
				grps.push({label: 'LBL_REPORT_FIELDS', srcpos: i, name: 'ungrouped-'+srcrows[i].name, rows: []});
				grps.push({label: 'LBL_TOTALS_FIELDS', srcpos: i, name: 'totals-'+srcrows[i].name, rows: []});
			}
			// sects contains the default section
			for(i = 0; i < rows.length; i++) {
				row = rows[i];
				if(isset(row.source) && row.source in seen) {
					gidx = seen[row.source].pos;
				} else {
					gidx = 0;
				}
				grps[gidx*3 + (row.total ? 2 : (row.grouped ? 0 : 1))].rows.push(row);
			}
			for(i = 0; i < grps.length; i++) {
				if(grps[i].rows.length) {
					grps[i].label = app_string(grps[i].label) + srcrows[grps[i].srcpos].label;
					ret.push(grps[i]);
				}
			}
			return ret;
		}
	}

	var FilterListRender = UI.FilterListRender = function() {
	};
	FilterListRender.uniq_index = 1000;
	FilterListRender.group_ops = {
		'and': 'LBL_FILTER_GROUP_AND',
		'or': 'LBL_FILTER_GROUP_OR',
		'not': 'LBL_FILTER_GROUP_NOT',
		'not_and': 'LBL_FILTER_GROUP_NOT_AND'
	};
	FilterListRender.prototype = {
		getGroupOperator: function(group) {
			var op = group.operator;
			if(! FilterListRender.group_ops[op])
				op = group.operator = 'and';
			return app_string(FilterListRender.group_ops[op]);
		},
		getGroupOptions: function() {
			if(! FilterListRender.sel_opts) {
				var opts = {keys: ['and', 'or', 'not', 'not_and'], values: [], width: '200px'};
				for(var i = 0; i < opts.keys.length; i++)
					opts.values.push(app_string(FilterListRender.group_ops[opts.keys[i]]));
				FilterListRender.sel_opts = new UI.SelectOptions(opts);
			}
			return FilterListRender.sel_opts;
		},
		renderName: function(row, field, sect_index, row_index, editor, cell) {
			if(! row._init) {
				row._init = { filter: '' };
			}
			if(row.type == 'filtergroup') {			
				var lbl1 = createElement2('span', {className: 'list-group-label'}, app_string('LBL_LAYOUT_GROUP_PREFIX'));
				lbl1.appendChild(sepLabel());
				var lbl2 = createElement2('div', {className: 'input-field flat rbullet'},
							createElement2('div', {className: 'input-label'}, this.getGroupOperator(row)));
				var render = this;
				lbl2.onclick = function(evt) {
					var edit = new UI.QuickSelect({options: render.getGroupOptions()});
					edit.init(row.operator, function(key, val) {
						editor.updateRow(sect_index, row_index, {operator: key});
					});
					edit.showPopup(0, this);
					UI.stopEvent(evt);
				}
				return [lbl1, lbl2];
			}
			return ListLayoutRender.prototype.renderName.call(this, row, field, sect_index, row_index, editor, cell);
		},
		renderFilter: function(row, field, sect_index, row_index, editor, cell) {
			var init, d;
			if(! row._init) {
				row._init = { filter: '' };
			}
			if(! row._init.div) {
				row._iprefix = FilterListRender.uniq_index ++;
				d = row._init.div = createElement2('div');
				var fixPfx = function(val) {
					var re = new RegExp('\\$UNIQ\\$' + row.field + '\\b', 'g'), ret;
					ret = (''+val).replace(re, row._iprefix + row.name);
					if(row.id_name && row._uniq) {
						re = new RegExp('\\$UNIQ\\$' + row.id_name + '\\b', 'g');
						ret = ret.replace(re, row._iprefix + row.name + '_id');
					}
					return ret.replace(/\$UNIQ\$/g, row._iprefix);
				}
				d.innerHTML = fixPfx(row._init.filter);
				cell.appendChild(d);
				if(isArray(row._init.ext)) {
					eval.call(window, row._init.ext.map(fixPfx).join("\n"));
				}
				if(isArray(row._init.ctor)) {
					row._init.ctls = row._init.ctor.map(fixPfx).map(eval, window);
					init = function() {
						UI.registerInputs(editor.form, row._init.ctls);
					}
				}
			} else
				return row._init.div;
			if(init)
				setTimeout(init, 100);
		}
	}
	
	UI.FieldListRender = function() {};
	UI.FieldListRender.uniq_index = 1000;
	UI.FieldListRender.prototype = {
		renderName: function(row, field, sect_index, row_index, editor, cell) {
			if(! row._init) {
				row._init = { rendered: '' };
			}
			UI.setElementContent(cell, row[field]);
			if(row.required)
				cell.appendChild(createElement2('span', {className: 'requiredStar'}, '*'));
		},
		renderValue: function(row, field, sect_index, row_index, editor, cell) {
			var init, d;
			if(! row._init) {
				row._init = { rendered: '' };
			}
			if(! row._init.div) {
				row._iprefix = UI.FieldListRender.uniq_index ++;
				d = row._init.div = createElement2('div');
				var fixPfx = function(val) {
					var re = new RegExp('\\$UNIQ\\$' + row.field + '\\b', 'g'), ret;
					ret = (''+val).replace(re, row._iprefix + row.name);
					if(row.id_name && row._uniq) {
						re = new RegExp('\\$UNIQ\\$' + row.id_name + '\\b', 'g');
						ret = ret.replace(re, row._iprefix + row.name + '_id');
					}
					return ret.replace(/\$UNIQ\$/g, row._iprefix);
				}
				d.innerHTML = fixPfx(row._init.rendered);
				cell.appendChild(d);
				if(isArray(row._init.ext)) {
					eval.call(window, row._init.ext.map(fixPfx).join("\n"));
				}
				if(isArray(row._init.ctor)) {
					row._init.ctls = row._init.ctor.map(fixPfx).map(eval, window);
					init = function() {
						UI.registerInputs(editor.form, row._init.ctls);
					}
				}
			} else
				return row._init.div;
			if(init)
				setTimeout(init, 100);
		}
	}
	
	var SourceListRender = UI.SourceListRender = function() {
	};
	SourceListRender.uniq_index = 1000;
	SourceListRender.prototype = {
		translateDisplay: function(row) {
			var d = row.display || 'joined';
			this.getDisplayOptions(); // set map_opts
			return SourceListRender.map_opts[d] || '';
		},
		getDisplayOptions: function() {
			if(! SourceListRender.sel_opts) {
				var opts = {keys: ['joined', 'nested'], values: [], width: '200px'},
					list = app_list_strings('source_display_dom');
				for(var i = 0; i < opts.keys.length; i++)
					opts.values.push(list[opts.keys[i]]);
				SourceListRender.sel_opts = new UI.SelectOptions(opts);
				SourceListRender.map_opts = list;
			}
			return SourceListRender.sel_opts;
		},
		renderDisplay: function(row, field, sect_index, row_index, editor, cell) {
			if(! row._init) {
				row._init = {};
			}
			if(row.type == 'primary')
				row[field] = 'primary';
			var lbl, self = this;
			if(row[field] == 'primary') {
				lbl = createElement2('div', {className: 'input-field-pad'},
						createElement2('div', {className: 'input-label'}, this.translateDisplay(row)));
			} else {
				lbl = createElement2('div', {className: 'input-field flat rbullet'},
						createElement2('div', {className: 'input-label'}, this.translateDisplay(row)));
				lbl.onclick = function(evt) {
					var edit = new UI.QuickSelect({options: self.getDisplayOptions()});
					edit.init(row[field], function(key, val) {
						editor.updateRow(sect_index, row_index, {display: key});
					});
					edit.showPopup(0, this);
					UI.stopEvent(evt);
				}
			}
			return lbl;
		},
		renderName: function(row, field, sect_index, row_index, editor) {
			var value = row[field];
			if(row.std_label && row.std_label != value) {
				value += ' <div class="listAddLine input-field-pad-horz text-muted">' + row.std_label + '</div>';
			}
			return value;
		},
		renderRequired: function(row, field, sect_index, row_index, editor) {
			if(row.display == 'primary') {
				// non-editable required
				return createElement2('div', {'class': 'uii uii-checkbox active-icon checked disabled'});
			} else
				return false;
		}
	}
	
	
	var FieldSourceSelect = UI.FieldSourceSelect = function(id, params) {
		UI.SelectInput.call(this, id, params);
	}
	extendClass(FieldSourceSelect, UI.SelectInput, {
		getSourceRows: function() {
			var src = UI.getFormInput(this.form, 'sources');
			return src ? src.getRows() : null;
		},
		reload: function() {
			var i, k = '',
				rows = this.getSourceRows(),
				opts = {keys: [], values: {}, width: '250px', columns: [
					{name: 'icon', width: 25, type: 'icon'},
					{name: 'label', width: 225}
				]};
			if(rows)
			for(i = 0; i < rows.length; i++) {
				if(rows[i].field_name) {
					if(k) k += '.';
					k += rows[i].field_name;
				}
				opts.keys.push(k);
				opts.values[k] = {icon: rows[i].icon, label: rows[i].label, link_name: k, name: rows[i].name};
			}
			this.menu.setOptions(new UI.SelectOptions(opts));
			return opts.keys.length;
		}
	});
	
	
	var FieldSelectInput = UI.FieldSelectInput = function(id, params) {
		if(! params) params = {};
		params.show_checks = false;
		params.close_on_exit = false;
		UI.SelectInput.call(this, id, params);
		this.fields = null;
		this.always_search = true;
		this.show_search_above = true;
		this.search_width = 450;
	};
	extendClass(FieldSelectInput, UI.SelectInput, {
		setup: function() {
			if(this.set_up) return;
			UI.SelectInput.prototype.setup.call(this);
			if(! this.list_field) {
				if(this.form && this.list_name)
					this.list_field = UI.getFormInput(this.form, this.list_name);
			}
			var self = this;
			if(this.mode == 'filters') {
				this.addGroupButton = UI.getFormInput(this.form, this.name + '-addgroup');
				var group = {depth: 0, _group: true, type: 'filtergroup', operator: 'and'};
				UI.attachInputEvent(this.addGroupButton, 'onclick', function() { self.list_field.insertRow(deep_clone(group)); });
			} else {
				this.addEmptyButton = UI.getFormInput(this.form, this.name + '-addempty');
				var empty = {depth: 0, label: app_string('LBL_EMPTY_COLUMN')};
				UI.attachInputEvent(this.addEmptyButton, 'onclick', function() { self.list_field.insertRow(deep_clone(empty)); });
			}
			if(this.mode != 'sources') {
				this.source_select = new FieldSourceSelect(this.id+'-srcsel');
				this.source_select.onchange = function(k,v) { self.handleSelectSource(k,v); }
				UI.registerInput(this.form, this.source_select);
			}
		},
		getTargetOffset: function() {
			return {over: true, above: true};
		},
		handleClick: function(evt) {
			this.source_name = this.link_name = this.link_ref = this.link_ref_label = null;
			UI.stopEvent(evt);
			if(this.source_select && this.source_select.reload() > 1) {
				this.source_select.showPopup(null, this.elt);
				return;
			}
			this.showFields(evt);
		},
		showFields: function(evt) {
			if(this.fields && this.fields_module == this.module && this.fields_link == this.getLinkName())
				this.showDefault(evt);
			else
				this.fetchOptions();
		},
		handleSelectSource: function(k, src) {
			this.source_name = src.name;
			this.link_name = src.link_name;
			this.link_ref = this.link_ref_label = null;
			this.showFields();
		},
		getLinkName: function() {
			var src = this.custom_link_name || this.link_name;
			if(this.mode == 'sources' && this.list_field) {
				var i, p = [], rows = this.list_field.getRows();
				this.source_name = '';
				for(i = 1; i < rows.length; i++) {
					p.push(rows[i].field_name);
					this.source_name = rows[i].name;
				}
				src = p.join('.');
			}
			if(this.link_ref) {
				if(src) src += '.' + this.link_ref;
				else src = this.link_ref;
			}
			return src;
		},
		fetchOptions: function() {
			if(this.req)
				return;
			var list_id = (this.form && this.form.list_id) ? this.form.list_id.value : null,
				form_name = (this.form) ? this.form.getAttribute('name') : null,
				data = {module: this.model ? null : this.module, model: this.model, list_id: list_id, form_name: form_name,
					link_name: this.getLinkName(), link_alias: this.source_name, context: this.context, wf_module: this.wf_module},
				std_methods = {order: 'get_list_fields'},
				method = this.json_method || std_methods[this.mode] || 'get_list_'+this.mode,
				r = this.req = new App.conn.JSONRequest(method, {}, data),
				self = this,
				callback = function(data) { self.receiveOptions(data); };
			this.fields_module = this.module;
			this.fields_link = data.link_name;
			r.fetch(callback);
		},
		receiveOptions: function(req) {
			if(req && this.req) {
				var fs = req.getResult().result, fs2 = [], ks = [], os, cp, i = 0;
				for(; i < fs.length; i++) {
					ks.push(fs[i].name || fs[i].field_name);
					if(this.link_ref) {
						if(this.mode != 'filters')
							fs[i].name = this.link_ref + '.' + fs[i].name;
						if(fs[i].field)
							fs[i].field = this.link_ref + '.' + fs[i].field;
					}
					fs2.push(fs[i]);
					if((this.mode == 'fields' || this.mode == 'filters') && fs[i].type == 'ref' && ! fs[i].dynamic_module && this.context != 'workflow') {
						cp = deep_clone(fs[i]);
						cp.sel_related = true;
						cp.orig_label = cp.std_label;
						cp.std_label = app_string('LBL_LIST_RELATED_PREFIX') + cp.std_label;
						cp.label = cp.std_label;
						ks.push(cp.name + ':related');
						fs2.push(cp);
					}
				}
				this.fields = fs;
				var lbl = (this.mode == 'filters' || this.mode == 'updates') ? 'label' : 'std_label',
					cols = [], opts;
				if(this.mode == 'sources')
					cols.push({name: 'icon', width: 25, type: 'icon'});
				cols.push({name: lbl, width: (this.mode == 'sources' ? 200 : 280)});
				if(this.mode != 'sources')
					cols.push({name: 'type_label', width: 150, expand_property: 'sel_related'});
				opts = new UI.SelectOptions(
					{width: (this.mode == 'sources' ? 225 : 430) + 'px', keys: ks, values: fs2, columns: cols}
				);
				this.menu.setOptions(opts);
			}
			this.req = null;
			this.afterReceive();
		},
		afterReceive: function() {
			var delay = 0;
			if(this.link_ref_time) {
				// hacky way to prevent previous closing popup from stopping next popup
				// from showing
				delay = Math.max(100, 500 + this.link_ref_time - new Date().getTime());
			}
			setTimeout(this.showDefault.bind(this), delay);
		},
		update: function(key, value) {
			if(! value) return;
			if(value.sel_related) {
				this.link_ref = value.field ? value.field : value.name;
				this.link_ref_label = value.orig_label;
				this.link_ref_time = new Date().getTime();
				this.showFields();
				return;
			}
			else if(this.link_ref_label) {
				value.ref_label = this.link_ref_label;
			}
			if(this.list_field) {
				if(this.mode != 'sources')
					delete value.depth;
				var uniq = false;
				var clone = deep_clone(value);
				var name = clone.name;
				var rows = this.list_field.getRows();
				while (true) {
					dup = false;
					for (var i = 0; i < rows.length; i++) {
						if (rows[i].name  == name + (uniq || '') ) {
							dup = true;
							break;
						}
					}
					if (!dup)
						break;
					uniq = (uniq || 0) + 1;
				}
				if (uniq) {
					clone._uniq = uniq;
					clone.name += (uniq || '');
				}
				this.list_field.insertRow(clone);
				var rows = this.list_field.getRows();
			}
		},
		initSearcher: function() {
			var ret = UI.SelectInput.prototype.initSearcher.call(this);
			this.searcher.popup_return_multiple = true;
			return ret;
		}
	});
	
	
	var FormatInput = UI.FormatInput = function(id, params) {
		if(! params) params = {};
		UI.DynamicList.call(this, id, params);
	};
	extendClass(FormatInput, UI.DynamicList, {
		processNewRow: function(row) {
			if(! row.format) row.format = '';
		}
	});
	
	var SortOrderInput = UI.SortOrderInput = function(id, params) {
		if(! params) params = {};
		UI.DynamicList.call(this, id, params);
	};
	extendClass(SortOrderInput, UI.DynamicList, {
		processNewRow: function(row) {
			if(! row.order) row.order = 'asc';
		}
	});
	
	
	var SortOrderRender = UI.SortOrderRender = function() {
	};
	SortOrderRender.opts = {keys: ['asc', 'desc'], values: [app_string('LBL_ASCENDING'), app_string('LBL_DESCENDING')]};
	SortOrderRender.typeToFormats = {
		date : ['month', 'qtr', 'year', 'yearmonth', 'fqtr', 'fy'],
		datetime : ['date_only', 'time_only', 'month', 'qtr', 'year', 'yearmonth', 'fqtr', 'fy'],
		currency : ['rounded', 'thousands', 'millions'],
		decimal : ['rounded', 'thousands', 'millions'],
		'int' : ['thousands', 'millions']
	};
	SortOrderRender.format_options = {};
	SortOrderRender.prototype = {
		translateOrder: function(ord) {
			return SortOrderRender.opts.values[ord == 'desc' ? 1 : 0];
		},
		renderOrder: function(row, field, sect_index, row_index, editor, cell) {
			var lbl = createElement2('div', {className: 'input-field flat rbullet'},
							createElement2('div', {className: 'input-label'}, this.translateOrder(row[field])));
			var render = this;
			lbl.onclick = function(evt) {
				var edit = new UI.QuickSelect({options: SortOrderRender.opts});
				edit.init(row[field], function(key, val) {
					editor.updateRow(sect_index, row_index, {order: key});
				});
				edit.showPopup(0, this);
				UI.stopEvent(evt);
			}
			return lbl;
		},
		translateFormat : function(key) {
			if (!key) return app_list_strings('field_format_dom')['none'];
			return get_default(app_list_strings('field_format_dom')[key], key);
		},
		makeFormatOptions : function(type) {
			if (SortOrderRender.format_options[type])
				return SortOrderRender.format_options[type];
			var options = {keys: [''], values: [app_list_strings('field_format_dom')['none']]};
			if (SortOrderRender.typeToFormats[type]) {
				for (var i=0; i < SortOrderRender.typeToFormats[type].length; i++) {
					var fmt = SortOrderRender.typeToFormats[type][i];
					options.keys.push(fmt);
					options.values.push(app_list_strings('field_format_dom')[fmt]);
				}
			}
			return SortOrderRender.format_options[type] = options;
		},
		renderFormat: function(row, field, sect_index, row_index, editor) {
			if (!SortOrderRender.typeToFormats[row['type']])
				return '';
			var lbl = createElement2('div', {className: 'input-field flat rbullet'},
							createElement2('div', {className: 'input-label'}, this.translateFormat(row[field])));
			var render = this;
			lbl.onclick = function(evt) {
				var edit = new UI.QuickSelect({options: render.makeFormatOptions(row['type'])});
				edit.init(get_default(row[field], ''), function(key, val) {
					editor.updateRow(sect_index, row_index, {format: key});
				});
				edit.showPopup(0, this);
				UI.stopEvent(evt);
			}
			return lbl;
		}
	};
	
	
	var RelatedListRender = UI.RelatedListRender = function() {
	}
	RelatedListRender.prototype = {
		renderName: function(row, field, sect_index, row_index, editor, cell) {
			return row[field];
		}
	};
	
});



function reparentLayoutButtons(form) {
	// move button sections to the bottom of the form
	var Wrap = SUGAR.dom.extNode,
		MakeElt = SUGAR.dom.makeElement,
		form = Wrap(form),
		body = form.findFirst('.form-body').get(0),
		buttons = MakeElt('div.card-footer.form-body');
	if(! body) return;
	form.append(buttons);
	form.find('.form-body > .form-section').forEach(function(sect, idx) {
		sect = Wrap(sect);
		var id = sect.attr('id');
		if(id && id.match(/-buttons$/))
			sect.appendTo(buttons);
	});
}

function onUpdateFields() {
	updateChartSettings(this.form);
}

function onUpdateType() {
	updateChartSettings(this.form);
}

function onUpdateSources() {
	var rows = this.getRows(0);
	var c = rows.length - 1, i = 0, j, base = '', srcs = {}, fs, s, rem;
	for(; i < rows.length; i++) {
		// mark all but last row fixed (can't move or remove)
		rows[i]._fixed = (i && i == c) ? 0 : 1;
		if(i) {
			base += rows[i].link_name;
			srcs[base] = 1;
			base += '.';
		}
	}
	// remove fields, filters, sort orders dependent upon this source
	var lists = ['columns', 'filters', 'sort_order'];
	for(i = 0; i < lists.length; i++) {
		fs = SUGAR.ui.getFormInput(this.form, lists[i]);
		if(! fs)
			continue;
		for(s = fs.getSectionCount()-1; s >= 0; s--) {
			rows = fs.getRows(s);
			rem = [];
			for(j = 0; j < rows.length; j++) {
				if(rows[j].source && ! srcs[rows[j].source])
					rem.push(j);
			}
			for(j = rem.length-1; j >= 0; j--) {
				fs.removeRow(s, rem[j], j > 0, true);
			}
		}
	}
	updateChartSettings(this.form);
}

function updateChartSettings(form) {
	var fs = SUGAR.ui.getFormInput(form, 'columns'),
		sources = SUGAR.ui.getFormInput(form, 'sources'),
		type = SUGAR.ui.getFormInput(form, 'chart_type'),
		series = SUGAR.ui.getFormInput(form, 'chart_series');
	if(! fs || ! type || ! series) return;
	var rows = fs.getRows(),
		sources = sources.getRows(),
		grouped = [],
		totals = [],
		type_outer = $('chart_type_panel'),
		opt_outer = $('chart_options_panel'),
		warn = $('chart_disabled_msg');
	for(var i = 0; i < rows.length; i++) {
		if(rows[i].grouped) grouped.push(i);
		if(rows[i].total) totals.push(i);
	}
	if(grouped.length && totals.length) {
		toggleDisplay(warn, false);
		toggleDisplay(type_outer, true);
		toggleDisplay(opt_outer, type.getValue() != '');
		var k, found,
			cur = series.getValue(),
			o = {keys: [''], values: {'': app_string('LBL_NONE')}, width: 300};
		for(var i = 0; i < rows.length; i++) {
			if(! rows[i].total) continue;
			k = rows[i].field;
			for(var z = 0; z < sources.length; z++) {
				var s = sources[z];
				if (s.name == rows[i].source) {
					if (s.display != 'primary')
						k = rows[i].source + '.' + k;
					break;
				}
			}
			if (rows[i].format)
				k += '!' + rows[i].format;
			k += ':' + rows[i].total;
			if(k == cur) found = true;
			o.keys.push(k);
			o.values[k] = rows[i].label;
			if (rows[i].format)
				o.values[k] += ' (' + app_list_strings('field_format_dom')[rows[i].format] + ')';
		}
		if(! found) cur = '';
		series.menu.setOptions(o);
		series.setValue(cur);
		
		k = type.getValue();
		var inp, show, chks = ['exploded', 'stacked', '3d', 'percent'];
		for(var i = 0; i < chks.length; i++) {
			if(! (inp = SUGAR.ui.getFormInput(form, 'chart_'+chks[i])))
				continue;
			show = false;
			if(chks[i] == 'exploded' || chks[i] == 'percent')
				show = (k == 'pie');
			else if(chks[i] == 'stacked')
				show = (k == 'hbar' || k == 'vbar');
			else if(chks[i] == '3d')
				show = (k != 'line' && k != 'area');
			inp.setDisabled(! show);
		}
	} else {
		if(! warn) {
			warn = createElement2('tbody', {id: 'chart_disabled_msg'});
			var r = warn.insertRow(0),
				c = r.insertCell(0);
			c.colspan = 2;
			c.className = 'dataLabel';
			c.appendChild(document.createTextNode(mod_string('LBL_NOT_CHARTABLE', 'Reports')));
			type_outer.parentNode.insertBefore(warn, type_outer);
		}
		toggleDisplay(type_outer, false);
		toggleDisplay(opt_outer, false);
		type.setValue('', true);
		series.menu.setOptions({keys: [''], values: ["none"]});
		series.setValue('');
	}	
}


function onUpdateRelated() {
	var rows = this.getRows(0),
		seen = {}, i = 0, id, remove = [];
	for(; i < rows.length; i++) {
		id = rows[i].module + '~' + rows[i].id;
		if(seen[id]) remove.push(i);
		else seen[id] = 1;
	}
	remove.reverse();
	for(i = 0; i < remove.length; i++) {
		this.removeRow(0, remove[i], null, true);
	}
}

