// vim: set foldmethod=marker:

IAHMap = function() {
	return {
		script_url: '',
		inited: false,
		address_icon: 'https://maps.google.com/mapfiles/ms/icons/green-dot.png',
		company_icon: 'https://maps.google.com/mapfiles/ms/icons/blue-dot.png',
		shadow_icon: 'https://maps.google.com/mapfiles/ms/icons/msmarker.shadow.png',
		
		init: function(callback) {
			if(! this.inited && ! this.script_url) return;
			this.status = SUGAR.ui.showStatus(app_string('LBL_LOADING'));
			this.init_callback = callback;
			if(this.inited)
				this.loaded();
			else {
				this.inited = true;
				SUGAR.util.addScriptInclude(this.script_url+'&callback=IAHMap.loaded');
			}
		},
		
		loaded: function() {
			SUGAR.ui.hideStatus(this.status);
			if(this.init_callback) setTimeout(this.init_callback, 1);
		},
		
		initMap: function() {
			this.map = new google.maps.Map($(this.div_id), {zoom: 14});
			if(! this.geoCoder)
				this.geoCoder = new google.maps.Geocoder();
			if(! this.infoWindow)
				this.infoWindow = new google.maps.InfoWindow();
			if(! this.svcDirections)
				this.svcDirections = new google.maps.DirectionsService();
			this.showDirections = new google.maps.DirectionsRenderer({map: this.map, panel: $('gmaps_directions'), infoWindow: this.infoWindow, suppressMarkers: true});
		},
		
		setAddress: function(objectName, address) {
			this.objectName = objectName;
			this.address = address;
			this.address_result = null;
		},
		
		setCompanyAddress: function(address) {
			this.company_address = address;
			this.company_result = null;
		},
		
		showInfoWindow: function(info, marker) {
			this.infoWindow.setContent(info);
			this.infoWindow.open(this.map, marker);
		},

		onshow: function()/*{{{*/
		{
			this.initMap();
			var addr_sel = SUGAR.ui.getFormInput('gmaps', 'address_select');
			if(addr_sel) {
				SUGAR.ui.attachInputEvent(addr_sel, 'onchange', function() { IAHMap.setCompanyAddress(this.getOptionValue()); });
				this.setCompanyAddress(addr_sel.getOptionValue());
			}
			this.onresize();
			this.showAddress();
		},/*}}}*/
		
		onresize: function() {
			if(this.map) {
				var div = this.map.getDiv();
				var size = this.dlg.getContentSize();
				/*div.style.width = size.width - 314 + 'px';
				div.style.height = size.height - 10 + 'px';*/
				div.style.height = Math.max(size.height - 10, 400) + 'px';
				google.maps.event.trigger(this.map, 'resize');
				//$('gmaps_sidebar').style.height = size.height - 12 + 'px';
			}
		},
		
		fetch: function(module, id, prefix)
		{
			if(isIPhone) {
				var req = new SUGAR.conn.JSONRequest('get_entity_address', {silent: true}, {module: module, record: id, prefix: prefix});
				req.fetch(function() {
					var r = this.getResult();
					var base_url = isIPhone ? 'maps:' : 'https://maps.google.com/maps?';
					if(r && r.address) {
						document.location.href = base_url + 'saddr=' + encodeURIComponent(r.current.replace(/\n/g, ' ')) + '&daddr=' + encodeURIComponent(r.address.replace(/\n/g, ' '));
					}
				});
			} else
				this.init(function() { IAHMap.do_fetch(module, id, prefix); });
		},

		do_fetch: function(module, id, prefix)/*{{{*/
		{
			this.div_id = 'map_' + parseInt(Math.random()* 10000000);
			var dlg = IAHMap.dlg = new SUGAR.ui.Dialog('gmaps_dialog', {title_text: app_string('LBL_MAP'), width: 920, height: 555, min_width: 450, min_height: 300});
			var query =  'async.php?module=CompanyAddress&action=Map&type=' + module + '&id=' + id + '&div_id=' + this.div_id;
			if (prefix) query += '&prefix=' + prefix;
			dlg.onshow = function() { IAHMap.onshow(); }
			dlg.onresize = function() { IAHMap.onresize(); }
			dlg.ondestroy = function() { IAHMap.map = null; }
			dlg.fetchContent(query, null, null, true);
		}, /*}}}*/
		
		showAddress: function() {
			this.fetchAddressLocation(null, true, true);
		},
		
		showCompany: function()
		{
			// FIXME - clear result when selected company address is changed
			this.fetchCompanyLocation(null, true, true);
		},

		fetchCompanyLocation: function(callback, show_marker, show_info) {
			var address = this.company_address;
			if (!address) return;
			this.fetchLocation('company', address.plain, address.name, callback, show_marker, show_info);
		},

		fetchAddressLocation: function(callback, show_marker, show_info) {
			this.fetchLocation('address', this.address, this.objectName, callback, show_marker, show_info);
		},
		
		fetchLocation: function(varname, address, title, callback, show_marker, show_info) {
			var result_name = varname + '_result';
			function success() {
				if(show_marker) IAHMap.displayLocation(varname, show_info);
				if(callback) callback(IAHMap[result_name]);				
			}
			if(this[result_name]) {
				success();
				return;
			}
			this.locateAddress(address, function(result) {
				result.title = title;
				IAHMap[result_name] = result;
				success();
			});
		},
		
		displayLocation: function(varname, show_info) {
			var result_name = varname + '_result';
			var marker_name = varname + '_marker';
			var icon = varname + '_icon';
			if(! this[result_name])
				return;
			var loc = this[result_name].geometry.location;
			var marker = this[marker_name];
			if(! marker) {
				marker = this[marker_name] = new google.maps.Marker({icon: this[icon], shadow: this.shadow_icon});
				google.maps.event.addListener(marker, "click", function() {
					IAHMap.showInfoWindow(this.info, this);
				});
			}
			marker.info = '<b>' + this[result_name].title + '</b><br>' + this[result_name].formatted_address;
			marker.setMap(this.map);
			marker.setPosition(loc);
			marker.setVisible(true);
			this.map.setCenter(loc);
			if(show_info)
				google.maps.event.trigger(marker, 'click');
			
			var other = (varname == 'company' ? this.address_result : this.company_result);
			if (other) {
				var aloc = other.geometry.location;
				var lt1 = loc.lat();
				var lg1 = loc.lng();
				var lt2 = aloc.lat();
				var lg2 = aloc.lng();
				var sw = new google.maps.LatLng(Math.min(lt1, lt2), Math.min(lg1, lg2));
				var ne = new google.maps.LatLng(Math.max(lt1, lt2), Math.max(lg1, lg2));
				var bounds = new google.maps.LatLngBounds(sw, ne);
				this.map.fitBounds(bounds);
			}
		},

		driveFromTo : function(is_from)
		{
			this.fetchAddressLocation(function(result) {
				var from = result.geometry.location;
				IAHMap.fetchCompanyLocation(function(result2) {
					var to = result2.geometry.location;
					if (is_from) {
						var query = {origin:from, destination:to};
					} else {
						var query = {origin:to, destination:from};
					}
					query.travelMode = google.maps.TravelMode.DRIVING;
					IAHMap.svcDirections.route(query, function(response, status) {
						if (status == google.maps.DirectionsStatus.OK) {
							IAHMap.maximizeDiv('gmaps_directions');
							IAHMap.showDirections.setDirections(response);
						} else {
							alert(IAHMap.gcodeDescription(status));
						}
					});
				}, true);
			}, true);
		},

		locateAddress : function (address, callback)
		{
			this.geoCoder.geocode({address: address}, function(results, status) {
				if (status == google.maps.GeocoderStatus.OK) {
					var el = $('gmaps_suggestions');
					if (results.length == 1) {
						IAHMap.maximizeDiv('');
						callback(results[0]);
					} else {
						$('gmaps_didyoumean_address').innerHTML = '';
						$('gmaps_didyoumean_address').appendChild(document.createTextNode(address));
						el.innerHTML = '';
						for (var i = 0; i < results.length; i++) {
							var a = document.createElement('a');
							a.href="javascript:void(0);";
							a.result = results[i];
							a.onclick = function() {
								IAHMap.maximizeDiv('');
								callback(this.result);
							}
							a.appendChild(document.createTextNode((i+1)+": " + results[i].formatted_address));
							el.appendChild(a);
							el.appendChild(document.createElement('br'));
						}
						IAHMap.maximizeDiv('gmaps_didyoumean');
					}
				} else {
					alert(IAHMap.gcodeDescription(status));
				}
			});
		},
			
			
		maximizeDiv : function(id) {
			var divs = ['gmaps_directions', 'gmaps_didyoumean'];
			for (var i=0; i < divs.length; i++) {
				var divId = divs[i];
				var div = $(divId);
				if (!div) continue;
				if (divId != id) {
					div.style.display = 'none';
				} else {
					div.style.display = '';
				}
			}
		},

		gcodeDescription : function (code) {
			var str = mod_string('LBL_GSTATUS_' + code, 'CompanyAddress');
			if (!str) str = 'Error ' + code;
			return str;
		}
	};
}();
