window.ONECRM_IPC = (function () {

  const STATE_LOCK_NAME = "ONECRM_STATE_LOCK";
  const CHANNEL_NAME = "ONECRM_IPC";
  const MY_ID = (new Date()).toISOString() + Math.random();

  let channel = null;
  const handlers = {};
  let lock = null;
  let resolveLock = null;

  const peers = {};

  const available = function() {
    return typeof window.BroadcastChannel === "function"
  }
  
  const messageHandler = function(topic, message) {
    const topicHandlers = handlers[topic];
    if (!topicHandlers) return;
    topicHandlers.forEach(function(h) {
      h(message);
    })
  }

  const eventHandler = function(event) {
    const data = event.data;
    messageHandler(data.topic, data.message);
  }

  const promise = new Promise(function(resolve) {
    resolveLock = resolve;
  })

  if (available()) {
    channel = new BroadcastChannel(CHANNEL_NAME);
    channel.addEventListener("message", eventHandler);
    navigator.locks.request(STATE_LOCK_NAME, function(theLock) {
      lock = theLock;
      resolveLock();
      // hold lock forever, until the page is unloaded.
      // after the lock is released, another window will get
      // a chance to become the master
      return new Promise(function(){});
    })
  }

  const runOnce = function(callback, onlyImmediate) {
    if (!lock && onlyImmediate) return;
    return promise.then(callback);
  }

  const notifyAll = function(topic, message) {
    messageHandler(topic, message);
    channel && channel.postMessage({
      topic: topic,
      message: message,
    });
  }

  const notifyOthers = function(topic, message) {
    channel && channel.postMessage({
      topic: topic,
      message: message,
    });
  }

  const on = function(topic, handler) {
    const topicHandlers = handlers[topic] ? handlers[topic] : [];
    if (!topicHandlers.includes(handler)) {
      topicHandlers.push(handler);
      handlers[topic] = topicHandlers;
    }
  }

  const off = function(topic, handler) {
    if (!handler) {
      delete handlers[topic];
    } else {
      const topicHandlers = handlers[topic];
      if (topicHandlers) {
        handlers[topic] = topicHandlers.filter(function(h) {
          return h !== handler;
        })
      }
    }
  }

  return {
    available: available,
    on: on,
    off: off,
    notifyAll: notifyAll,
    notifyOthers: notifyOthers,
    runOnce: runOnce,
  }
  
})()