<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/

class jsAlerts {
	var $alerts = array();
	
	function __construct() {
	}
	
	function addAlert($id, $type, $name, $subtitle, $description, $time, $redirect='', $icon=''){
		$this->alerts[] = compact('id', 'type', 'name', 'subtitle', 'description', 'time', 'redirect', 'icon');
	}
	
	function getAlerts() {
		return $this->alerts;
	}
	
	function getScript(){
		$script = array();
		$json = getJSONobj();
		foreach($this->alerts as $alert) {
			$row = $json->encode($alert);
			$script[] = "SUGAR.notify.addAlert($row);";
		}
		return '<script type="text/javascript">' . implode("\n", $script) . '</script>';
	}
	
	function addDefault() {
		if(AppConfig::setting('site.features.enable_timeout_alerts')){
			$this->addAlert('timeout_warn', translate('ERROR_JS_ALERT_SYSTEM_CLASS', 'app'), translate('ERROR_JS_ALERT_TIMEOUT_TITLE', 'app'),'', translate('ERROR_JS_ALERT_TIMEOUT_MSG_1', 'app'), (session_cache_expire() - 2) * 60 );
			$this->addAlert('timeout_session', translate('ERROR_JS_ALERT_SYSTEM_CLASS', 'app'), translate('ERROR_JS_ALERT_TIMEOUT_TITLE', 'app'), '', translate('ERROR_JS_ALERT_TIMEOUT_MSG_2', 'app'), (session_cache_expire()) * 60 , 'index.php');
		}
	}
	
	function addActivities(){
		global $db, $timedate;
		$uid = AppConfig::current_user_id();
		if(! $uid)
			return;
		$show_remind = AppConfig::user_preference('show_reminders');
		if(! $show_remind)
			return;
		
		// cn: get a boundary limiter
		$reminder_max_time = AppConfig::setting('site.reminders.lookahead_seconds', 3600);
		$now = time();
		$dateTimeMax = gmdate('Y-m-d H:i:s', $now + $reminder_max_time);
		$dateTimeNow = gmdate('Y-m-d H:i:s', $now);
		
		require_once('modules/vCals/vCal.php');
		$lines = vCal::get_freebusy_lines($uid, true);
		if(! $lines)
			return;
		$vcMax = gmdate('Ymd\THis\Z', $now + $reminder_max_time);
		$vcNow = gmdate('Ymd\THis\Z', $now);
		$skip = false;
		if(count($lines) < 100) {
			// if more than 100 events, just perform the query, don't bother parsing.
			// can't assume that lines are ordered by start date
			$skip = true;
			foreach($lines as $l) {
				$times = explode('/', substr($l, 9));
				if($times[0] < $vcMax && $times[1] > $vcNow) {
					$skip = false;
					break;
				}
			}
		}
				
		$qid = $db->quote($uid);
		$taskQuery = "SELECT 'Tasks' module, m.id, m.name, m.reminder_time, m.description, '' location,
				m.date_due date_start, m.assigned_user_id FROM tasks m WHERE m.assigned_user_id='{$qid}' 
				AND NOT m.deleted AND IFNULL(m.reminder_time, 0) > 0
				AND m.date_due BETWEEN '$dateTimeNow' AND '$dateTimeMax'";
		$query_tpl = "
			SELECT '%s' module, m.id, m.name, m.reminder_time, m.description, %s, m.date_start, m.assigned_user_id
			FROM %s m LEFT JOIN %s mu ON m.id = mu.%s 
			WHERE mu.user_id ='{$qid}' 
				AND NOT m.deleted
				AND IFNULL(m.reminder_time, 0) > 0
				AND NOT IFNULL(mu.deleted, 1)
				AND date_start BETWEEN '$dateTimeNow' AND '$dateTimeMax'";
		if($skip) {
			$query = $taskQuery;
		} else {
			$query = '('
				. sprintf($query_tpl, 'Meetings', 'm.location', 'meetings', 'meetings_users', 'meeting_id')
				. ') UNION ('
				. sprintf($query_tpl, 'Calls', "'' location", 'calls', 'calls_users', 'call_id')
				. ") UNION ($taskQuery)";
		}
		$result = $db->query($query, false);
		if(! $result) {
			$GLOBALS['log']->error("Error performing jsalerts meetings query: ".$query);
			return;
		}
		
		while($row = $db->fetchByAssoc($result, -1, false)) {
			$timeStart = strtotime($row['date_start'] . ' GMT') - $row['reminder_time'];
			$timeRemind = $timeStart - time();
			if($timeRemind < 0) continue;

			$url = 'index.php?module=' . $row['module'] . '&action=DetailView&record=' . $row['id'];
			$instructions = translate('MSG_JS_ALERT_MTG_REMINDER_MSG', 'app');

			$meetingName = $row['name'];
			$desc = $row['description'];
			$sep = translate('LBL_SEPARATOR', 'app');
			$description = empty($desc) ? '' : translate('MSG_JS_ALERT_MTG_REMINDER_AGENDA', 'app') . $sep . $desc."\n";
			$location = $row['location'] ? translate('MSG_JS_ALERT_MTG_REMINDER_LOC', 'app') . $sep . $row['location'] . "\n" : '';
			
			// standard functionality
			$lbl = $row['module'] == 'Calls' ? 'MSG_JS_ALERT_MTG_REMINDER_CALL' :
					($row['module'] == 'Tasks' ? 'MSG_JS_ALERT_MTG_REMINDER_TASK' : 'MSG_JS_ALERT_MTG_REMINDER_MEETING');

			$this->addAlert($row['id'], translate($lbl, 'app'), $meetingName,
				translate('MSG_JS_ALERT_MTG_REMINDER_TIME', 'app') . $sep .$timedate->to_display_date_time($row['date_start']),
				$location. $description,
				$timeRemind,
				$url,
				$row['module']
			);
		}
	}
	
}



?>
