/**
 *
 * Popup helper functions
 *
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 */


function init_popup(form) {
	if(! form) form = document.forms['popup_query_form'];
	var inline = form.inline;
	if(! inline) {
		inline = createElement2('input', {type: 'hidden', name: 'inline'});
		if(popup_dialog)
			inline.value = 'true';
		form.appendChild(inline);
	}
	if(inline.value == 'true') {
		form.onsubmit = function() {return update_inline_popup(this)};
		var mu = document.PopupMassUpdate;
		if(mu)
			mu.onsubmit = form.onsubmit;
		var addform = $('addform');
		if(addform) {
			var forms = addform.getElementsByTagName('form');
			if(forms && forms.length && ! forms[0]._custom_name /* custom product */) {
				forms[0].onsubmit = function() {
					return check_form(this.getAttribute('name')) && update_inline_popup(this);
				}
			}
		}
	}
	if(form.request_data && form.request_data.value == "") {
		if(window.opener && inline.value != 'true') {
			var rdata = deep_clone(window.opener.get_popup_request_data());
			form.request_data_val = rdata;
			form.request_data.value = JSON.stringify(rdata);
		}
		else if(window.get_popup_request_data) {
			var rdata = get_popup_request_data();
			form.request_data_val = rdata;
			form.request_data.value = JSON.stringify(rdata);
		}
	}
}

function update_popup(form) {
	if(! form) form = document.forms.popup_query_form;
	if(form.inline && form.inline.value)
		return update_inline_popup(form);
	return form.submit();
}

function update_inline_popup(form, form_values, conn_params) {
	if(popup_dialog)
		popup_dialog.fetchFormContent(form, form_values, conn_params, fetched_popup, true);
	return false;
}

function submit_popup(form) {
	if(! form) form = document.forms['popup_query_form'];
	if(form.onsubmit)
		form.onsubmit();
	else
		form.submit();
}

function do_close_popup(force) {
	var form = document.forms['popup_query_form'];
	var inline = form && form.inline && form.inline.value == 'true';
	var close_popup = (window.opener && ! inline) ? window.opener.get_close_popup() : window.get_close_popup();

	if(close_popup || force)
	{
		if(close_popup && close_popup.apply)
			close_popup();
		else
			window.close();
	}
}

function popup_assoc_data(module, id, field_map) {
	var row_data = associated_javascript_data[id];
	if(! row_data)
		row_data = {};
	var ret = {};
	for(var the_key in field_map)
	{
		if(the_key == 'toJSON') continue;
		var the_name = field_map[the_key];
		var lookup = the_key.toUpperCase();
		if(the_key == 'user_name' && get_default(row_data.USE_FULL_NAME))
			lookup = 'FULL_NAME';
		ret[field_map[the_key]] = get_default(row_data[lookup], '');
	}
	return ret;
}

function fetch_request_data() {
	var form = document.forms.popup_query_form;
	if(form && isset(form.request_data_val))
		return form.request_data_val;
	if(form && form.request_data && form.request_data.value) {
		form.request_data_val = JSON.parse(form.request_data.value);
	}
	if(! isset(form.request_data_val)) {
		if(window.console) console.log("missing popup request data");
		return null;
	}
	return form.request_data_val;
}

function send_back(module, id)
{
	var request_data = fetch_request_data();
	
	var passthru_data = {};
	if(typeof(request_data.passthru_data) != 'undefined')
	{
		passthru_data = request_data.passthru_data;
	}

	// longreach - added -for adding an Asset/SupportedAssembly to a ServiceContract
	if(passthru_data['link_field_name'] == '') passthru_data['link_field_name'] = module.toLowerCase();
		
	var form_name = request_data.form_name;
	var field_to_name_array = request_data.field_to_name_array;
	
	var form = document.forms['popup_query_form'];
	var inline = form && form.inline && form.inline.value == 'true';
	
	if(window.opener && ! inline)
		var call_back_function = eval("window.opener." + request_data.call_back_function);
	else
		var call_back_function = eval("window." + request_data.call_back_function);

	var row_data = popup_assoc_data(module, id, field_to_name_array);

	var result_data = {"form_name":form_name, "module":module, "name_to_value_array":row_data, "passthru_data":passthru_data};
	
	if(request_data.return_result_array) {
		result_data.result_array = {order: [id], data: {}};
		result_data.result_array.data[id] = row_data;
	}
	
	call_back_function(result_data);

	do_close_popup();
}

function send_back_selected(module, form, field, error_message)
{
	var request_data = fetch_request_data();
	
	var sel_list = {};
	var assoc_data = {};
	var ids = [];
	var j=0;
	for (i = 0; i < form.elements.length; i++){
		if(form.elements[i].name == field && form.elements[i].checked) { 
			++j;
			sel_list["ID_" + j] = form.elements[i].value;
			ids.push(form.elements[i].value);
		}
	}
	if(! j) {
		window.alert(error_message);	
		return;
	}
	
	var passthru_data = {};
	if(typeof(request_data.passthru_data) != 'undefined')
	{
		passthru_data = request_data.passthru_data;
	}
	var form_name = request_data.form_name;
	var field_to_name_array = request_data.field_to_name_array;
	var inline = form.inline && form.inline.value == 'true';
	
	if(window.opener && ! inline)
		var call_back_function = eval("window.opener." + request_data.call_back_function);
	else
		var call_back_function = eval("window." + request_data.call_back_function);

	var result_data = {
		form_name: form_name,
		module: module,
		selection_list: sel_list,
		selection_count: sel_count,
		passthru_data: passthru_data,
		popup_window: window};
	
	if(request_data.return_result_array || request_data.get_multiple_associated_data) {
		if(request_data.get_multiple_associated_data)
			result_data.associated_row_data = {};
		if(request_data.return_result_array)
			result_data.result_array = {order: ids, data: {}};
		for(var k in sel_list) {
			var id = sel_list[k];
			var rowdata = popup_assoc_data(module, id, field_to_name_array);
			if(request_data.get_multiple_associated_data)
				result_data.associated_row_data[k] = rowdata;
			if(request_data.return_result_array)
				result_data.result_array.data[id] = rowdata;
		}
	}
	
	call_back_function(result_data);

	do_close_popup();
}

function popup_order_by(fields) {
	var form = document.PopupMassUpdate;
	if(! form || ! fields) return false;
	fields = fields.split('&');
	for(var i = 0; i < fields.length; i++) {
		if(! fields[i]) continue;
		var v = fields[i].split('=');
		if(v.length == 2) {
			if(! form.elements[v[0]]) {
				var inp = createElement2('input', {type: 'hidden', name: v[0], value: v[1]});
				form.appendChild(inp);
			} else
				form.elements[v[0]].value = v[1];
		}
	}
	return sListView.save_checks(0);
}
