/**
 * Popup parent helper functions in javascript
 *
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 */



var popup_dialog;

function get_popup_request_data()
{
	return document.popup_request_data;
}

function get_close_popup()
{
	return document.close_popup;
}

function open_popup_inline(title)
{
	var args = [];
	for(var i = 1; i < Math.max(arguments.length, 11); i++)
		if(i < arguments.length)
			args[i-1] = arguments[i];
		else
			args[i-1] = undefined;
	var custom;
	if(args.length > 10)
		custom = args[10];
	else
		custom = {};
	custom['inline'] = true;
	if(title)
		custom['title'] = title;
	args[10] = custom;
	return open_popup.apply(this, args);
}

function old_open_popup()
{
	var args = [];
	for(var i = 0; i < Math.max(arguments.length, 10); i++)
		if(i < arguments.length)
			args[i] = arguments[i];
		else
			args[i] = undefined;
	var custom;
	if(args.length > 10)
		custom = args[10];
	else
		custom = {};
	if(! isset(custom.inline))
		custom.inline = false;
	args[10] = custom;
	return open_popup.apply(this, args);
}

function open_popup(module_name, width, height, initial_filter, close_popup, hide_clear_button, popup_request_data, popup_mode, create, metadata, custom) {
    var options = {
		module: module_name, min_width: width, min_height: height, filter: initial_filter,
		close_popup: close_popup, hide_clear_button: hide_clear_button,
		request_data: popup_request_data, popup_mode: popup_mode,
		create: create, metadata: metadata
	};
	if(custom) {
		options.inline = custom.inline;
		options.title = custom.title;
	}
	if(! isset(options.inline)) // can be set to false for old-style popup
		options.inline = true;
	return open_popup_window(options);
}

function open_popup_window(options, dialog_params) {
	document.popup_request_data = options.request_data;
	document.close_popup = options.close_popup;
	
	var URL = options.url;
	if(! URL) {
		if(! options.module)
			return;
		var URL = 'async.php?'
			+ 'module=' + options.module
			+ '&action=' + get_default(options.action, 'Popup');
		if(options.hide_clear_button)
			URL += '&hide_clear_button=true';
		var popup_mode = options.popup_mode || (options.multiple ? 'MultiSelect' : 'single');
		URL += '&mode='+popup_mode;
		var create = options.create || 'false';
		URL += '&create='+create;
		if(options.layout)
			URL += '&layout='+options.layout;
		if(options.metadata)
			URL += '&metadata='+options.metadata;
		if(options.add_popup_fields)
			URL += '&add_popup_fields='+options.add_popup_fields;
	}
	if(options.filter) {
		var filter = options.filter;
		if(typeof(filter) == 'object')
			filter = encodeQueryString(filter);
		if(filter.length && filter[0] != '&')
			filter = '&' + filter;
		if(filter.indexOf("reassign") != -1 && options.request_data)
			filter += '&request_data=' + options.request_data;
		URL += '&query=true' + filter;
	}
	if(options.query_string) {
		URL += '&' + options.query_string;
	}
	
	var width = options.width,
		viewp = SUGAR.ui.getViewport();
	if(! width) {
		width = parseInt(get_default(options.min_width, 700));
		if(options.inline) {
			var scrw = viewp.width;
			if(scrw > 1024)
				width += Math.round(Math.min(150, (scrw - 1024) / 2));
		}
		if(options.max_width)
			width = Math.min(width, options.max_width);
	}
	var height = options.height;
	if(! height) {
		height = parseInt(get_default(options.min_height, 450));
		if(options.inline) {
			var scrh = viewp.height;
			if(scrh > 600)
				height += Math.round(Math.min(200, (scrh - 600) / 2));
		}
		if(options.max_height)
			height = Math.min(height, options.max_height);
	}
		
	if(options.inline) {
		URL += '&in_popup=true';
		var min_width = '400px';
		if (screen && screen.width < 390)
			min_width = '' + (screen.width - 10) + 'px';
		var params = {min_width: min_width, min_height: '300px'};
		if(options.min_width) params.min_width = options.min_width + 'px';
		if(options.min_height) params.min_height = options.min_height + 'px';
		if(options.width) params.width = options.width + 'px';
		if(options.height) params.height = options.height + 'px';
		if (typeof dialog_params === 'object') {
			params = Object.assign({}, dialog_params, params);
		}
		popup_dialog = new SUGAR.ui.Dialog(options.popup_name, params);
		if(options.onclose)
			popup_dialog.onclose = options.onclose;
		popup_dialog.ondestroy = function() {
			popup_dialog = null;
		}
		popup_dialog.setTitle(options.title || '&nbsp;');
		if(options.title)
			popup_dialog.ignore_init_title = true;
		document.close_popup = function() {
			if(! window.popup_dialog) return;
			popup_dialog.close();
		}
		popup_dialog.fetchContent(URL, null, fetched_popup, true);
		return true;
	}
	
	var windowFeatures = 'width=' + width
		+ ',height=' + height
		+ ',resizable=1,scrollbars=1';
	var win = window.open(URL, options.popup_name || 'popup_window', windowFeatures);

	if(window.focus)
		// put the focus on the popup if the browser supports the focus() method
		win.focus();

	return win;
}


function fetched_popup() {
	// called after popup is retrieved and rendered
}


/**
 * The reply data must be a JSON array structured with the following information:
 *  1) form name to populate
 *  2) associative array of input names to values for populating the form
 */
var from_popup_return  = false;
function set_return(popup_reply_data)
{
	from_popup_return = true;
	var form_name = popup_reply_data.form_name;
    var name_to_value_array = popup_reply_data.name_to_value_array;

	for (var the_key in name_to_value_array)
	{

		if(the_key == 'toJSON')
		{
			/* just ignore */
		}
		else
		{
			var displayValue=name_to_value_array[the_key];
			if ( (typeof displayValue) == "string")
				displayValue = displayValue.replace(/&amp;/gi,'&').replace(/&lt;/gi,'<').replace(/&gt;/gi,'>').replace(/&#039;/gi,'\'').replace(/&quot;/gi,'"');
			else
				displayValue = '';
			/* longreach - modified - call onchange after programmatic
					update of dropdown list
			window.document.forms[form_name].elements[the_key].value = displayValue;
			*/		


			if (window.document.forms[form_name] && (o = window.document.forms[form_name].elements[the_key])) {
				if (o.options) {
					for (var i=0,b=null; i<o.options.length; i++) {
						if (displayValue != o.options[i].value) continue;
						b = o.options[i].selected = true;
						o.selectAfterRedyState = displayValue;
						break;
					}
					o.optionAfterReady = displayValue;//patch
					if (b && (typeof o.onchange) == "function") {
						o.saveSubSelectedOutset = true;//patch
						o.onchange();
						o.saveSubSelectedOutset = false;//patch
					}
				} else o.value = displayValue;
			}
			
		}
	}
}

function set_return_and_save(popup_reply_data)
{
	var form_name = popup_reply_data.form_name;
	var name_to_value_array = popup_reply_data.name_to_value_array;
	
	for (var the_key in name_to_value_array)
	{
		if(the_key == 'toJSON')
		{
			/* just ignore */
		}
		else
		{
			window.document.forms[form_name].elements[the_key].value = name_to_value_array[the_key];
		}
	}
	
	window.document.forms[form_name].return_module.value = window.document.forms[form_name].module.value;
	window.document.forms[form_name].return_action.value = 'DetailView';
	window.document.forms[form_name].return_id.value = window.document.forms[form_name].record.value;
	window.document.forms[form_name].action.value = 'Save';
	window.document.forms[form_name].submit();
}

/**
 * This is a helper function to construct the initial filter that can be
 * passed into the open_popup() function.  It assumes that there is an
 * account_id and account_name field in the given form_name to use to
 * construct the intial filter string.
 */
function get_initial_filter_by_account(form_name)
{
	var account_id = window.document.forms[form_name].account_id.value;
	var account_name = escape(window.document.forms[form_name].account_name.value);
	var initial_filter = "&account_id=" + account_id + "&account_name=" + account_name;

	return initial_filter;
}


// new for 7.0
function update_subpanel(popup_reply_data)
{
	var form_name = popup_reply_data.form_name;
	var form_params = deep_clone(popup_reply_data.name_to_value_array);
	var passthru_data = popup_reply_data.passthru_data;
	// construct the POST request
	var query_array = [];
	if(! isset(form_params))
		form_params = {};
  	//construct the muulti select list
	var selection_list = popup_reply_data.selection_list;
	if(isset(selection_list)) {
		var recs = [], row;
		for(var the_key in selection_list) {
			row = selection_list[the_key];
			if(typeof row === 'object') row = row.id;
			if(! row) continue;
			recs.push(row);
		}
		form_params['selected_id[]'] = recs;
	}

	if (popup_reply_data.hasOwnProperty('filter')) {
        for(var name in popup_reply_data.filter) {
            form_params[name] = popup_reply_data.filter[name];
        }
    }

    for(var prop in passthru_data) {
		form_params[prop] = passthru_data[prop];
	}
	form_params.record_perform = 'subpanel_insert';
	
	var conn_params = {};
	if(passthru_data['reload_page'] != 'true') {
		conn_params.resultDiv = '' + form_params['list_id'] + '-outer';
	}
	SUGAR.ui.sendForm(form_name, form_params, conn_params);
}
