var RingCentralOneCRM = new (function () {

	var EVENT_NEW_CALL = '1CRM_RINGCENTRAL_EVENT_NEW_CALL';
	var EVENT_STATE_UPDATED = '1CRM_RINGCENTRAL_EVENT_STATE_UPDATED';
	var EVENT_NEWTAB = '1CRM_RINGCENTRAL_EVENT_NEWTAB';
	var EVENT_LOGIN = '1CRM_RINGCENTRAL_EVENT_LOGIN';
	var EVENT_LOGOUT = '1CRM_RINGCENTRAL_EVENT_LOGOUT';
	var Wrap = SUGAR.dom.extNode;

	var dom = {};

	var ID = (new Date()).toString() + Math.random().toString();

	var errorCount = 0;
	var currentIdx = 0;

	var Collection = function () {
		Array.call(this);
	};

	Collection.prototype = new Array();
	Collection.prototype.constructor = Collection;

	Collection.prototype.first = function (callback) {
		var elt = this.length ? this[0] : null;
		if (callback) {
			if (elt)
				callback(elt);
			return this;
		} else {
			return this.length ? this[0] : null;
		}
	};

	var $$ = function (cls, parent) {
		var coll = new Collection();
		Wrap(parent || document.body).find('.' + cls).forEach(function (elt) {
			coll.push(elt);
		});
		return coll;
	};

	var state = {
		authenticated: false,
		calls: [],
		errors: [],
	}

	var updateState = function (updater) {
		var newState = updater(state);
		state = newState
		ONECRM_IPC.notifyAll(EVENT_STATE_UPDATED, newState);
	}

	var setAuthenticated = function (value, error) {
		updateState(function (state) {
			return Object.assign(state, { authenticated: value, connectError: error });
		})
	}

	var authenticationFailed = function () {
		updateState(function (state) {
			return Object.assign(state, { authenticated: value, connectError: undefined });
		})
	}

	ONECRM_IPC.on(EVENT_STATE_UPDATED, function (newState) { 
		state = newState
		renderUI(state) 
	});

	ONECRM_IPC.on(EVENT_NEW_CALL, function() {
		var c = state.calls[0];
		if (c.direction == 'Inbound' && auto_expand)
			RC.showHide(null, true);
		currentIdx = 0;
		RC.updateNavigation();
	})


	var sdk, platform, Subscriptions, subscriptions,
		subscription, lock,
		callerId, sandbox, pressOne, auto_expand, unsubscribe,
		clientId;
	var RC = this;
	var RCSDK = RingCentral.SDK;
	var Subscriptions = RingCentral.Subscriptions;

	this.initialize = function (params) {
		clientId = params.clientId;
		callerId = params.callerId;
		sandbox = params.sandbox;
		pressOne = !!params.pressOne;
		auto_expand = !!params.auto_expand;
		sdk = new RCSDK({
			server: sandbox ? RCSDK.server.sandbox : RCSDK.server.production,
			appKey: 'x',
			appSecret: 'y',
			clientId: params.clientId
		});
		platform = sdk.platform();
		subscriptions = new Subscriptions({ sdk: sdk });

		SUGAR.onDomReady(function () {
			RC.initializeUI();
			RC.initializeMessages();
			RC.connect();
		});
	};

	this.refreshCalls = function () {
		var query_params = { module: 'Calls', type: 'RingCentral' };
		var req = new SUGAR.conn.JSONRequest('cti_history', {}, query_params);
		req.fetch(function (ret) {
			updateState(function(state) {
				state.calls = ret.result;
				return state;
			})
		});
	};

	this.initiateCall = function (number) {
		var rq = {
			from: { phoneNumber: callerId },
			callerId: { phoneNumber: callerId },
			to: { phoneNumber: number },
			playPrompt: pressOne
		};
		platform.post('/restapi/v1.0/account/~/extension/~/ring-out', rq)
			.then(function (response) {
			}).catch(function (err) {
				RC.addError(err.message);
			});
	};

	this.makeTranslate = function (eventName) {
		platform.on(eventName, function () {
			RC.translate([eventName].concat(Array.prototype.slice.call(arguments)));
		});
	};

	this.translate = function (args) {
		ONECRM_IPC.notifyAll(args);
	};

	this.unload = function () {
		unsubscribe && unsubscribe();
		unsubscribe = undefined;
	};

	RC.logout = function (ev) {
		ev.stopPropagation();
		RC.performLogout();
	};

	this.onConnect = function () {
		RC.displayLoader(false);
		subscription = subscriptions.createSubscription();
		subscription.on(subscription.events.notification, function (msg) {
			var newCall = null;
			var finishedCall = null;
			for (var i = 0; i < msg.body.activeCalls.length; i++) {
				var found = false;
				var call = msg.body.activeCalls[i];
				for (var j = 0; j < state.calls.length; j++) {
					if (state.calls[j].cti_id === call.sessionId) {
						found = true;
					}
				}
				if (!finishedCall && call.telephonyStatus == 'NoCall')
					finishedCall = call;
				if (!found)
					newCall = call;
			}
			if (newCall) {
				var c = newCall;
				var number = c.direction == 'Inbound' ? c.from : c.to;
				var query_params = { module: 'Calls', type: 'RingCentral', id: c.sessionId, number: number, direction: c.direction };
				var req = new SUGAR.conn.JSONRequest('start_cti_call', {}, query_params);
				req.fetch(function (ret) {
					updateState(function(state) {
						state.calls.unshift(ret.result)
						return state;
					})
					ONECRM_IPC.notifyAll(EVENT_NEW_CALL);
				});
			}
			if (finishedCall) {
				var c = finishedCall;
				var query_params = { module: 'Calls', type: 'RingCentral', id: c.sessionId };
				var req = new SUGAR.conn.JSONRequest('finish_cti_call', {}, query_params);
				req.fetch(function (ret) {});
			}
		});
		subscription.setEventFilters(['/restapi/v1.0/account/~/extension/~/presence?detailedTelephonyState=true'])
		.register().then(function (response) {
				unsubscribe = response.remove.bind(response);
			}).catch(function (e) {
				RC.addError(e.message);
			});
			RC.refreshCalls();
	};

	this.connect = function () {
		RC.makeTranslate(platform.events.refreshError);
		RC.makeTranslate(platform.events.logoutSuccess);
		RC.makeTranslate(platform.events.logoutError);
		ONECRM_IPC.runOnce(function () {
			platform.loggedIn().then(function (result) {
				RC.onConnect();
				setAuthenticated(!!result);
			}).catch(function (e) {
				setAuthenticated(false, e.message);
			});
		});
		SUGAR.ui.addEventListener(window, 'beforeunload', RC.unload);
	};

	this.performLogout = function () {
		platform.auth().cancelAccessToken()
			.then(function () {
				setAuthenticated(false);
			})
			.catch(function () {
				setAuthenticated(false);
			});
	};

	this.performLogin = function () {
		platform.loginWindow({ url: platform.loginUrl({ "state": "1234567890", "client_id": clientId }) })
			.then(function (authData) {
				platform.auth().setData(authData).then(function () {
					platform.loggedIn().then(function (loggedIn) {
						setAuthenticated(!!loggedIn);
					});
				});
			});
		return false;
	};

	this.initializeMessages = function () {
		ONECRM_IPC.on(EVENT_NEWTAB, function (id) {
			ONECRM_IPC.notifyOthers(EVENT_STATE_UPDATED, state);
		});
		ONECRM_IPC.on(platform.events.refreshError, authenticationFailed);
		ONECRM_IPC.on(platform.events.logoutSuccess, authenticationFailed);
		ONECRM_IPC.on(platform.events.logoutError, authenticationFailed);
	};

	this.callsWithNumber = function (call) {
		return !!RC.callNumber(call);
	};

	this.decorateCalls = function (update) {
		updateState(function(state) {
			state.calls.forEach(function (call) {
				if (update.hasOwnProperty(call.sessionId))
					call._fetched = update[call.sessionId];
			});
			return state;
		})
	};

	this.updateCalls = function (update) {
		RC.displayLoader(false);
		calls = update.filter(RC.callsWithNumber);
		RC.checkMissedCalls();
		var ids = {}, empty = true;
		calls.forEach(function (item) {
			if (!item.hasOwnProperty('_fetched')) {
				ids[item.sessionId] = RC.callNumber(item);
				empty = false;
			}
		});
		if (!empty) {
			var query_params = { module: 'Calls', type: 'RingCentral', ids: ids };
			var req = new SUGAR.conn.JSONRequest('cti_history', {}, query_params);
			req.fetch(function (ret) {
				RC.decorateCalls(ret.result);
			});
		}
		RC.renderCalls();
	};

	this.checkMissedCalls = function (calls) {
		calls.forEach(function (call) {
			if (call.telephonyStatus === "Ringing" && call.direction == "Inbound")
				call.wasRingIn = true;
			if (call.telephonyStatus === 'CallConnected')
				call.wasRingIn = false;
			if (call.telephonyStatus === 'NoCall' && call.wasRingIn)
				call.result = "Missed";
		});
		return calls;
	};

	var renderUI = function(state) {
		RC.displayLoader(false);
		if (state.authenticated) {
			RC.renderCalls(state.calls);
		} else {
			RC.displayLogin()
		}
		RC.renderErrors();
	}

	this.renderErrors = function() {

		state.errors.forEach(function(err) {
			var id = "ringcentral-error-"  + err.id;
			if (document.getElementById(id)) return;
			var p = createElement2('p');
			p.setAttribute('id', id);
			p.innerText = err.error;
			dom.errors.appendChild(p);
		})
		
		dom.error_count.innerText = state.errors.length;
		dom.error_count.style.display = state.errors.length > 0 ? '' : 'none';

	}

	this.toggleErrors = function (ev) {
		ev.stopPropagation();
		dom.errors.style.display = dom.errors.style.display == 'none' ? 'block' : 'none';
		dom.main.style.display = dom.errors.style.display == 'none' ? 'block' : 'none';
	};

	this.addError = function (err) {
		updateState(function(state) {
			var id = Math.random();
			state.errors.push({id: id, error: err})
			return state;
		});
	};

	this.displayView = function (name) {
		dom.main.setAttribute('data-active-view', name);
	};

	this.displayErrorList = function (show, initial) {
		dom.errors.style.display = show ? '' : 'none';
		dom.main.style.display = show ? 'none' : '';
		if (initial) {
			dom.error_count.style.display = 'none';
		}
	};

	this.displayLogin = function () {
		authenticated = false;
		unsubscribe && unsubscribe();
		unsubscribe = undefined;
		calls = [];
		dom.list.innerHTML = '';
		RC.displayView('login');
	};

	this.displayLoader = function (show, message) {
		RC.displayView(show ? 'loader' : 'calls');
		if (message) {
			dom.login_error.style.display = '';
			dom.error_message.innerText = message;
			dom.loading.style.display = 'none';
		} else {
			dom.login_error.style.display = 'none';
			dom.loading.style.display = '';
		}
	};

	this.initializeUI = function () {
		dom.container = document.getElementById("ringcentral-container");
		dom.errors = document.getElementById("ringcentral-errors");
		dom.main = document.getElementById("ringcentral-main");
		dom.header = document.getElementById('ringcentral-header');
		dom.square = document.getElementById('ringcentral-square');
		dom.content = document.getElementById('ringcentral-content');
		dom.error_count = document.getElementById("ringcentral-error-counter");
		dom.list = document.getElementById("ringcentral-calls-list");
		dom.status = document.getElementById("ringcentral-connection-status");
		dom.login_error = document.getElementById("ringcentral-login-error");
		dom.loading = document.getElementById("ringcentral-loading");
		dom.error_message = document.getElementById("ringcentral-login-error-message");
		dom.login_form = document.getElementById("ringcentral-login-form");
		dom.retry_button = document.getElementById("ringcentral-retry-button");
		dom.logout = document.getElementById("ringcentral-logout-icon");
		dom.prevButton = document.getElementById("ringcentral-prev-button");
		dom.prevIcon = document.getElementById("ringcentral-prev-icon");
		dom.nextButton = document.getElementById("ringcentral-next-button");
		dom.nextIcon = document.getElementById("ringcentral-next-icon");
		dom.callsCount = document.getElementById("ringcentral-calls-count");
		dom.login_form.onsubmit = RC.performLogin;
		dom.retry_button.onclick = function () { RC.displayLogin(); };
		dom.logout.onclick = RC.logout;
		dom.prevButton.onclick = RC.navigatePrev;
		dom.nextButton.onclick = RC.navigateNext;

		RC.showHide(null, Get_Cookie('ringcentral_expanded') == 'block');
		SUGAR.ui.addEventListener(dom.header, 'click', RC.showHide);
		SUGAR.ui.addEventListener(dom.square, 'click', RC.showHide);
		RC.displayLoader(true);
		RC.displayErrorList(false, true);
		dom.error_count.onclick = RC.toggleErrors;
		ONECRM_IPC.notifyOthers(EVENT_NEWTAB, ID);
	};

	this.showHide = function (_, state) {
		if (typeof state == 'undefined')
			state = dom.container.style.display == 'none';
		dom.container.style.display = state ? 'block' : 'none';
		dom.square.style.display = state ? 'none' : 'block';
		Set_Cookie('ringcentral_expanded', state ? 'block' : 'none', 1000);
	};

	this.createButton = function (label, icon, attrs) {
		attrs = attrs || {};
		attrs.className = (attrs.className || '') + ' input-button group-left';
		var b = createElement2('button', attrs);
		var i = createElement2('div', { className: 'input-icon left ' + icon });
		var l = createElement2('span', { className: 'input-label' });
		l.innerText = label;
		b.appendChild(i);
		b.appendChild(l);
		return b;
	};

	this.createCallWidget = function (id) {
		var table = createElement2('table', { id: id, className: 'ringcentral-call-info tabDetailView', width: '100%', style: { height: '315px' } });

		var tr = createElement2('tr', { style: 'vertical-align: top' });
		var td = createElement2('td', { className: 'tabDetailViewDF' });
		var icon = createElement2('img', { className: 'ringcentral-call-icon' });
		var number = createElement2('span', { className: 'js-call-number ', style: { cursor: 'pointer' } });
		var date = createElement2('span', { className: 'js-call-date', style: { marginLeft: '15px' } });
		td.appendChild(icon);
		td.appendChild(number);
		td.appendChild(date);
		tr.appendChild(td);
		table.appendChild(tr);

		var tr = createElement2('tr', { style: 'vertical-align: top', className: 'js-contact-info' });
		var td = createElement2('td', { className: 'tabDetailViewDF' });
		var dataDiv = createElement2('div', { className: 'ringcentral-contact-data' });
		var imgDiv = createElement2('div', { className: 'ringcentral-contact-image' });
		var name = createElement2('a', { className: 'js-contact-name', href: '#', style: { fontSize: '120%' } });
		var icon = createElement2('div', { className: 'js-contact-icon input-icon theme-icon theme-icon' });
		var phonesList = createElement2('div', { className: 'ringcentral-phones-list' });
		imgDiv.appendChild(createElement2('img', { className: 'js-contact-photo' }));
		dataDiv.appendChild(icon);
		dataDiv.appendChild(document.createTextNode(' '));
		dataDiv.appendChild(name);
		dataDiv.appendChild(phonesList);
		td.appendChild(dataDiv);
		td.appendChild(imgDiv);
		tr.appendChild(td);
		table.appendChild(tr);

		var tr = createElement2('tr', { style: 'vertical-align: top', className: 'js-contact-create' });
		var td = createElement2('td', { className: 'tabDetailViewDF' });
		var contactButton = RC.createButton('Create Contact', 'theme-icon create-Contact', { className: 'js-create-contact', style: { width: '100%', marginBottom: '4px' } });
		var accountButton = RC.createButton('Create Account', 'theme-icon create-Account', { className: 'js-create-account', style: { width: '100%', marginBottom: '4px' } });
		var leadButton = RC.createButton('Create Lead', 'theme-icon create-Lead', { className: 'js-create-lead', style: { width: '100%', marginBottom: '4px' } });

		td.appendChild(contactButton);
		td.appendChild(accountButton);
		td.appendChild(leadButton);
		tr.appendChild(td);
		table.appendChild(tr);

		var tr = createElement2('tr', { style: 'vertical-align: top' });
		var td = createElement2('td', { className: 'tabDetailViewDF' });
		var name = createElement2('h4', { className: 'js-call-name-wrapper' });
		var a = createElement2('a', { className: 'js-call-name', href: '#' });
		var editButton = createElement2('div', { className: 'js-edit-button input-icon active-icon icon-edit' });
		var icon = createElement2('div', { className: 'input-icon theme-icon theme-icon bean-Call' });

		var form = createElement2('form', { className: 'js-title-form', style: { display: 'none', marginBottom: '10px' } });
		var input = createElement2('input', { type: 'text', className: 'js-title-input', style: { width: '100%' } });
		form.appendChild(input);

		name.appendChild(icon);
		name.appendChild(document.createTextNode(' '));
		name.appendChild(a);
		name.appendChild(document.createTextNode(' '));
		name.appendChild(editButton);
		td.appendChild(name);
		td.appendChild(createElement2('hr'));
		td.appendChild(form);

		var form = createElement2('form', { className: 'js-notes-form' });
		var textarea = createElement2('textarea', { className: 'js-notes-text', style: { width: '100%' } });
		var button = createElement2('button', { type: 'submit', className: "input-button group-left" });
		var icon = createElement2('div', { className: "input-icon left icon-edit" });
		var text = createElement2('span', { className: "input-label" });
		text.innerText = 'Add Notes';
		button.appendChild(icon);
		button.appendChild(text);
		form.appendChild(textarea);
		form.appendChild(button);
		td.appendChild(form);
		tr.appendChild(td);
		table.appendChild(tr);

		return table;
	};

	this.callNumber = function (call) {
		return call.phone_number;
	};

	this.phoneClickHandler = function (number) {
		return function (ev) {
			RingCentralOneCRM.initiateCall(number);
			return false;
		}
	}

	this.notesFormHandler = function (notes, id) {
		return function () {
			var query_params = { module: 'Calls', id: id, notes: notes.value };
			var req = new SUGAR.conn.JSONRequest('cti_call_notes', {}, query_params);
			req.fetch(function (ret) {
			});
			notes.value = '';
			return false;
		};
	};

	this.titleSubmitHandler = function (input, id, wrapper) {
		return function (ev) {
			input.form.style.display = 'none';
			wrapper.style.display = '';
			var query_params = { module: 'Calls', id: id, name: input.value };
			var req = new SUGAR.conn.JSONRequest('cti_call_name', {}, query_params);
			req.fetch(function (ret) {
				updateState(function(state) {
					state.calls.forEach(function (call, idx) {
						if (ret.result.id == call.id) {
							state.calls[idx] = ret.result;
						}
					});
					return state;
				})
			});
			return false;
		}
	};

	this.titleKeyHandler = function (form, wrapper) {
		return function (ev) {
			if (ev.keyCode === 27) {
				form.style.display = 'none';
				wrapper.style.display = '';
			}
		}
	};

	this.editClickHandler = function (form, input, wrapper) {
		return function (ev) {
			form.style.display = '';
			wrapper.style.display = 'none';
			input.focus();
			return false;
		}
	};

	this.quickCreateCallback = function (number, module) {
		return function (id, name) {
			var ids = [];
			state.calls.forEach(function (call) {
				if (call.phone_number === number)
					ids.push(call.id);
			});
			var query_params = { module: 'Calls', ids: ids };
			var req = new SUGAR.conn.JSONRequest('cti_refresh_calls', {}, query_params);
			req.fetch(function (ret) {
				updateState(function(state) {
					for (var i = 0; i < ret.result.length; i++) {
						var item = ret.result[i];
						for (var j = 0; j < state.calls.length; j++) {
							if (state.calls[j].id === item.id) {
								state.calls[j] = item;
								break;
							}
						}
					}
					return state;
				})
			});
			RC.recordClickHandler(module, id)();
		};
	};

	this.recordCreateHandler = function (module, params, callback) {
		return function () {
			SUGAR.ui.openQuickCreate(module, params, callback);
		};
	};

	this.recordClickHandler = function (module, id, params) {
		params = params || {};
		params.action = params.action || 'DetailView';
		params.module = module;
		params.record = id;
		var args = {};
		for (var i in params) {
			if (params.hasOwnProperty(i))
				args[i] = params[i];
		}
		return function (ev) {
			var q_params = encodeQueryString(args);
			SUGAR.util.fetchContentMain('async.php?' + q_params);
			return false;
		};
	};

	this.renderCall = function (call) {
		var id = 'ringcentral-call-' + call.cti_id;
		var number = RC.callNumber(call);
		if (!number)
			return;
		var widget = document.getElementById(id);
		if (!widget) {
			widget = RC.createCallWidget(id);
			dom.list.insertBefore(widget, dom.list.firstChild);
		}
		var dt = new Date(call.date_start.replace(/(\d+)-(\d+)-(\d+)/, '$2/$3/$1') + ' GMT');
		var time = dt.print(getDateFormat().print_format) + ' ' + dt.print(getTimeFormat().print_format);
		var icon = call.direction == "Inbound" ? 'themes/Default/images/call_incoming.png' : 'themes/Default/images/call_out.png';
		if (call.result == 'Missed')
			icon = 'themes/Default/images/call_missed.png';

		$$('js-call-date', widget).first().innerText = time;
		$$('ringcentral-call-icon', widget).first().src = icon;
		$$('js-call-number', widget).first(function (elt) {
			elt.innerText = number;
			elt.onclick = RC.phoneClickHandler(number);
		});

		$$('js-call-name', widget).first(function (elt) {
			elt.innerText = call.name;
			elt.onclick = RC.recordClickHandler('Calls', call.id);
		});

		var wrapper = $$('js-call-name-wrapper', widget).first();
		var form = $$('js-title-form', widget).first();
		var input = $$('js-title-input', widget).first();
		var button = $$('js-edit-button', widget).first();
		form.onsubmit = RC.titleSubmitHandler(input, call.id, wrapper);
		input.onkeydown = RC.titleKeyHandler(form, wrapper);
		input.value = call.name;
		button.onclick = RC.editClickHandler(form, input, wrapper);

		var form = $$('js-notes-form', widget).first();
		var elt = $$('js-notes-text', widget).first();
		form.onsubmit = RC.notesFormHandler(elt, call.id);

		var contactInfo = $$('js-contact-info', widget).first();
		var contactCreate = $$('js-contact-create', widget).first();
		if (call._contact) {
			contactInfo.style.display = '';
			contactCreate.style.display = 'none';
			$$('js-contact-icon', widget).first(function (elt) {
				Wrap(elt).addClass('bean-' + call._contact._model);
			});
			$$('ringcentral-phones-list', widget).first(function (elt) {
				RC.renderContactPhones(elt, call._contact);
			});
			$$('js-contact-name', widget).first(function (elt) {
				elt.innerText = call._contact && call._contact.name || '';
				elt.onclick = RC.recordClickHandler(call._contact._module, call._contact.id);
			});
			$$('ringcentral-contact-image', widget).first(function (elt) {
				elt.style.display = call._contact._photo ? '' : 'none';
				if (call._contact._photo) {
					$$('js-contact-photo', widget).first(function (elt) {
						elt.src = call._contact._photo;
					});
				}
			});
		} else {
			contactInfo.style.display = 'none';
			contactCreate.style.display = '';
			$$('js-create-contact').first().onclick = RC.recordCreateHandler('Contacts', { phone_work: number }, RC.quickCreateCallback(number, 'Contacts'));
			$$('js-create-account').first().onclick = RC.recordCreateHandler('Accounts', { phone_office: number }, RC.quickCreateCallback(number, 'Accounts'));
			$$('js-create-lead').first().onclick = RC.recordCreateHandler('Leads', { phone_work: number }, RC.quickCreateCallback(number, 'Leads'));
		}
	};

	this.renderContactPhone = function (number, type, selected) {
		var attrs = { className: "summaryContact" };
		attrs.style = { border: 'solid 1px transparent' };
		if (type === selected) {
			attrs.style = { backgroundColor: '#FBFBFF', border: 'solid 1px #CCC' };
		}
		var div = createElement2('div', attrs);
		var div2 = createElement2('div', { className: "nobr" });
		var icon = createElement2('span', { className: "uii uii-sm" }, 'p');
		var a = createElement2('a', { className: "tabDetailViewDFLink", href: "#" });
		var sep = createElement2('span');
		var label = createElement2('span', { className: "inlineLabel" });
		sep.innerHTML = '&nbsp;&nbsp;';
		label.innerText = type;
		a.innerText = ' ' + number;
		div2.appendChild(icon);
		div2.appendChild(a);
		div.appendChild(div2);
		div.appendChild(sep);
		div.appendChild(label);
		a.onclick = RC.phoneClickHandler(number);
		return div;
	};

	this.renderContactPhones = function (container, contact) {
		container.innerHTML = '';
		for (var k in contact) {
			if ((k.indexOf('phone_') !== 0) || ~k.indexOf('__raw'))
				continue;
			var type = k.substr(6);
			var number = contact[k];
			if (!number || !number.trim().length)
				continue;
			var elt = RC.renderContactPhone(contact[k], type, contact._match);
			container.appendChild(elt);
		}
	};

	this.navigatePrev = function () {
		RC.navigate(-1);
	};

	this.navigateNext = function () {
		RC.navigate(1);
	};

	this.navigate = function (offset) {
		currentIdx += offset;
		RC.updateNavigation();
	};

	this.updateNavigation = function () {
		var prevEnabled = true;
		var nextEnabled = true;
		var calls = state.calls;
		if (calls.length) {
			if (currentIdx >= calls.length)
				currentIdx = calls.length - 1;
			if (currentIdx < 0)
				currentIdx = 0;
			prevEnabled = currentIdx > 0;
			nextEnabled = currentIdx < calls.length - 1;
			dom.callsCount.innerText = '' + (currentIdx + 1) + ' / ' + calls.length;
		}
		if (!prevEnabled) {
			dom.prevButton.disabled = true;
			Wrap(dom.prevIcon).addClass('off');
		} else {
			dom.prevButton.disabled = false;
			Wrap(dom.prevIcon).removeClass('off');
		}
		if (!nextEnabled) {
			dom.nextButton.disabled = true;
			Wrap(dom.nextIcon).addClass('off');
		} else {
			dom.nextButton.disabled = false;
			Wrap(dom.nextIcon).removeClass('off');
		}
		$$('ringcentral-call-info', dom.list).forEach(function (elt) {
			elt.style.display = 'none';
		});
		if (calls.length) {
			var id = 'ringcentral-call-' + calls[currentIdx].cti_id;
			var widget = document.getElementById(id);
			widget.style.display = '';
		}
	};

	this.renderCalls = function (calls) {
		for (var i = calls.length - 1; i >= 0; i--) {
			RC.renderCall(calls[i]);
		}
		RC.updateNavigation();
	};

})();

