/**
 * Javascript file for Sugar
 *
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 */


/**
 * Namespace for Sugar Objects
 */
var SUGAR = AppBase;
// Namespace for Homepage (dashlet management)
SUGAR.module('sugarHome');
// General namespace for utils
SUGAR.module('util');
// Theme data
SUGAR.module('themes');
// Connection manager
SUGAR.module('conn');
// UI library
SUGAR.module('ui');
// Database row cache
SUGAR.module('modelCache');
// Language strings and dropdown lists
SUGAR.module('language');
// Session info (display preferences and settings)
SUGAR.module('session');
// Timed notifications (upcoming meetings etc)
SUGAR.module('notify');
// Studio (module designer and settings)
SUGAR.module('Studio');


/* Utility functions */

function toggleDisplay(id, state, title) {
	var elt = $(id), elt_link, elt_title;
	id = elt ? elt.id : '';
	if(! elt) return;
	elt_link = $(id + 'link');
	elt_title = title ? $('sectionHeader-'+id) : null;
	if(! isset(state))
		state = (elt.style.display == 'none');
	if(state) {
		SUGAR.ui.setDisplayState(elt, true);
		SUGAR.ui.setDisplayState(elt_link, false);
		SUGAR.ui.setDisplayState(elt_title, true);
	}
	else {
		SUGAR.ui.setDisplayState(elt, false);
		SUGAR.ui.setDisplayState(elt_link, true);
		SUGAR.ui.setDisplayState(elt_title, false);
		return 'none';
	}
	return true;
}

function toggleSection(id) {
	var shown = toggleDisplay(id) != 'none',
		header = $('sectionHeader-' + id);
	if(header) {
		SUGAR.ui.addRemoveClass(header, 'sectionClosed', ! shown);
		SUGAR.ui.addRemoveClass(header, 'sectionOpen', shown);
	}
	return false;
}

function checkAll(form, field, value) {
	for (var i = 0; i < form.elements.length; i++) {
		if(form.elements[i].name == field)
			form.elements[i].checked = value;
	}
}

function replaceAll(text, src, rep) {
	offset = text.toLowerCase().indexOf(src.toLowerCase());
	while(offset != -1) {
		text = text.substring(0, offset) + rep + text.substring(offset + src.length ,text.length);
		offset = text.indexOf( src, offset + rep.length + 1);
	}
	return text;
}

function toDecimal(original) {
	temp = Math.round(original*100)/100;
	if((original * 100) % 100 == 0)
		return temp + '.00';
	if((original * 10) % 10 == 0)
		return temp + '0';
	return temp
}

function formatPhoneNumber(input) {
	if (! SUGAR.session.telephony.format_phones) return;
	var phone = input.value,
		stripped = phone,
		ext = null,
		extPattern = /[^0-9+\-.()*#\s][^0-9+\-()*#\s]*\s*(\d+)\s*$/,
		match;

	if (extPattern.test(phone)) {
		match = phone.match(extPattern);
		ext = match[1];
		stripped = stripped.replace(extPattern, '');
	}

	stripped = stripped.replace(/[^0-9()+]/g, '');

	if (/^[0-9]+$/.test(stripped) && stripped.length == SUGAR.session.telephony.local_digits) {
		phone = '+' + SUGAR.session.telephony.country_code + ' (' + SUGAR.session.telephony.local_code + ') ' + formatLocalPhone(stripped, ext);
	}
	if (/^\([0-9]+\)[0-9]+$/.test(stripped)) {
		match = stripped.match(/^\(([0-9]+)\)([0-9]+)$/);
		phone = '+' + SUGAR.session.telephony.country_code + ' (' + match[1] + ') ' + formatLocalPhone(match[2], ext);
	}
	if (/^[0-9]+$/.test(stripped) && (stripped.length == SUGAR.session.telephony.fixed_number_length) && (SUGAR.session.telephony.fixed_area_code_length > 0) && (SUGAR.session.telephony.fixed_number_length > 0)) {
		phone = '+' + SUGAR.session.telephony.country_code + ' (' + stripped.substr(0, SUGAR.session.telephony.fixed_area_code_length) + ') ' + formatLocalPhone(stripped.substring(SUGAR.session.telephony.fixed_area_code_length, 9999), ext);
	}
	if (/^\+([0-9]+)\([0-9]+\)[0-9]+$/.test(stripped)) {
		match = stripped.match(/^\+([0-9]+)\(([0-9]+)\)([0-9]+)$/);
		phone = '+' + match[1] + ' (' + match[2] + ') ' + formatLocalPhone(match[3], ext);
	}
	input.value = phone;
}

function formatUrl(input) {
	var url = trim(input.value), tail, sch;
	// fix duplicate schema
	while( (tail = url.match(/(.*:\/\/.*?)(\w+:\/\/.*)$/)) )
		url = tail[2];
	// fix issues like a double paste by looking for common schemas
	// http://www.example.comhttp://www.example.com -> comhttp://www.example.com -> http://www.example.com
	sch = url.match(/^(.*?(file|http|https|ftp|spdy))?(:\/\/.*)/i);
	if(sch && sch[2])
		url = sch[2] + sch[3];
	// fix :/ instead of ://
	url = url.replace(/:\/([^\/])/, '://$1');
	// fix missing schema
	if(! /:\/\//.test(url) && url.length)
		url = 'http://' + url;
	input.value = url;
}

function formatLocalPhone(phone, ext) {
	var patterns = [
			[],
			[1],
			[2],
			[3],
			[2, 2],
			[2, 3],
			[3, 3],
			[3, 4],
			[3, 3, 2]
		],
		pattern = patterns[phone.length],
		start = 0,
		ret = '',
		i;
	if (typeof(pattern) == 'undefined') return phone;
	for (i=0; i < pattern.length; i++) {
		if (i > 0) ret += '-';
		ret += phone.substring(start, start + pattern[i]);
		start += pattern[i];
	}
	if (ext) {
		ret += ' ' + SUGAR.session.telephony.ext_string + ' ' + ext;
	}
	return ret;
}

function setPointer(row, id, action) {
	return sListView.row_action(row, action, id);
}

// sends theForm via AJAX and fills in the theDiv
function sendAndRetrieve(theForm, theDiv, loadingStr, onUpdate, noScroll) {
	var done = function() {
		SUGAR.session.setLoadingRequest(null);
		if(onUpdate) onUpdate();
	}
	function success(data) {
		var body = data ? data.responseText : '', div  = $(theDiv);
		if (typeof theDiv === 'function') {
			theDiv(SUGAR.util.disableInlineScripts(body));
		} else if (div) {
			div.innerHTML = SUGAR.util.disableInlineScripts(body);
		}
		SUGAR.util.evalScript(body, onUpdate);
		if(! noScroll)
			SUGAR.util.scrollToElement(theDiv);
	}
	if(typeof loadingStr == 'undefined')
		loadingStr = app_string('LBL_LOADING');
	var req = SUGAR.conn.sendForm(theForm, {status_msg: loadingStr}, success);
	if(req) SUGAR.session.setLoadingRequest(req);
	return false;
}


// used to manage selected rows in listviews
var UidList = function(field) {
	this.allUids = false;
	this.uids = [];

	this.setUidString = function(uidStr) {
		this.uids = [];
		if(uidStr === 'all')
			this.allUids = true;
		else if(uidStr) {
			this.allUids = false;
			var uidList = uidStr.split(';');
			for(var i = 0; i < uidList.length; i++)
				if(uidList[i])
					this.uids.push(trim(uidList[i]));
		}
	}
	this.addUid = function(uid) {
		this.addUids([uid]);
	}
	this.addUids = function(uidsIn) {
		if(! uidsIn || ! uidsIn.length || this.allUids)
			return;
		var uidSet = {};
		for(var i = 0; i < uidsIn.length; i++) {
			uidsIn[i] = trim(uidsIn[i]);
			uidSet[uidsIn[i]] = 1;
		}
		for(var i = 0; i < this.uids.length; i++) {
			if(uidSet[this.uids[i]])
				uidSet[this.uids[i]] = 0;
		}
		for(var i = 0; i < uidsIn.length; i++)
			if(uidsIn[i] && uidSet[uidsIn[i]])
				this.uids.push(uidsIn[i]);
	}
	this.removeUid = function(uid) {
		this.removeUids([uid]);
	}
	this.removeUids = function(uidsIn) {
		if(! uidsIn || ! uidsIn.length || this.allUids)
			return;
		var uidSet = {};
		for(var i = 0; i < uidsIn.length; i++) {
			uidsIn[i] = trim(uidsIn[i]);
			uidSet[uidsIn[i]] = 1;
		}
		for(var i = 0; i < this.uids.length; i++) {
			if(uidSet[this.uids[i]])
				this.uids.splice(i --, 1);
		}
	}
    this.clear = function() {
        this.allUids = false;
        this.uids = [];
    }
	this.getCount = function() {
		if(this.allUids)
			return 'all';
		return this.uids.length;
	}
	this.getList = function() {
		if(this.allUids)
			return 'all';
		return this.uids.slice(0);
	},
	this.getString = function() {
		if(this.allUids) return 'all';
		else return this.uids.join(';');
	},
	this.save = function() {
		if(! this.field) return;
		this.field.value = this.getString();
	}

	this.field = field;
	if(field)
		this.setUidString(field.value);
}


// -- start sugarListView class
// js functions used for ListView
function sugarListView() {
	this.lists = {};
    this.offsetPopup = null;
    this.primary_id = null;
	return this;
}
sugarListView.prototype = {
	init: function(list_id, params) {
		if(! list_id) return false;
		if(! params) params = {};
		var persisted = {}, dl,
			outer_id = list_id+'-outer';
		if(params.type == 'dashlet')
			outer_id = 'dashlet-' + list_id;
		else if(params.is_sidebar)
			outer_id = 'sidebar-list-body';
		if(this.lists[list_id]) {
			persisted = this.lists[list_id].persist_data;
		}
		var list = this.lists[list_id] = {
			elt: $(list_id + '-outer'),
			type: params.type,
			table: $(list_id+'-main'),
			outer: $(outer_id),
			form: $(list_id+'-ListUpdate'),
			filter: $(list_id+'-FilterForm'),
			persist_data: persisted,
			is_primary: params.is_primary,
			is_sidebar: params.is_sidebar,
			count_options: params.count_options,
			query: params.query,
			mass_perform: params.mass_perform,
			result_count: params.result_count,
			total_count: params.total_count,
			sidebar_limit: params.sidebar_limit,
			subpanel_name: params.subpanel_name,
			subpanel_tab: params.subpanel_tab,
			parent_module: params.parent_module,
			parent_record: params.parent_record,
			drag: false,
			lastY: 0
		};
		if(list.is_primary) {
			if(list.query)
				SUGAR.session.loadState(list.query);
			this.primary_id = list_id;
		}
		if(list.is_sidebar)
			this.sidebar_id = list_id;
		var buttons = [list_id + '-ActionButtonHead', list_id + '-ActionButtonFoot'];
		var opts = list_id + '-MenuActions';
		if($(buttons[0]))
			list.top_menu = SUGAR.ui.registerMenuSource(buttons[0], {options: opts, icon_key: 'icon'}, list.form);
		if($(buttons[1]))
			list.bottom_menu = SUGAR.ui.registerMenuSource(buttons[1], {options: opts, icon_key: 'icon'}, list.form);
		var sel_limit = $(list_id + '-sel-limit');
		if(sel_limit) {
			opts = this.getCountOptions(list);
			var onch = function(key) {
				sListView.performSearch(list_id, {list_limit: key});
			}
			list.limit_menu = SUGAR.ui.registerMenuSource(sel_limit, {options: opts, onchange: onch}, list.form);
		}
		list.have_hr = !! (list.table && list.table.getElementsByClassName('listViewHRS1').length);
		list.row_span = list.have_hr ? 2 : 1;
		sListView.checkCurrentIndex(list_id);
        if (list.mass_perform == 'delete' || list.mass_perform == 'update' || list.mass_perform == 'trash') {
            sListView.setSelected(list_id, 'none');
            sListView.resetSelected(list_id);
            sListView.updateSelectedCount(list_id);
        }
		if(list.is_sidebar)
			this.setSidebarTitle();
		if (params.type == 'subpanel') {
			new SUGAR.ui.Draggable('#' + list_id + '-outer', {
				sourceType: 'reorderSubpanels',
				handle: '#' + list_id + '-title',
				values: {subpanel:  list_id + '-outer', list_id: list_id},
				onEnd: function(evt) {
					this.elt.closest('.subpanel-list').children('.subpanel-entry').css('opacity',  1);
					SUGAR.dom.query('.subpanel-entry').removeClass('subpanel-reorder').removeClass('dz-drag-hover');
				}
			});
		}
		return true;
	},

	deferredLoad: function(params) {
		SUGAR.onDomReady(function() {
			sListView.restoreLayout(null, params);
		});
	},

	getPrimaryId: function() {
		if(this.primary_id) {
			var list = this.getListView(this.primary_id);
			if(list && list.outer && SUGAR.ui.inDocument(list.outer)) return this.primary_id;
		}
	},
	getSidebarId: function() {
		if(this.sidebar_id) {
			var list = this.getListView(this.sidebar_id);
			if(list && list.outer && SUGAR.ui.inDocument(list.outer)) return this.sidebar_id;
		}
	},
	getListView: function(list_id) {
		return this.lists[list_id];
	},
	getForm: function(list_id) {
		if(this.lists[list_id])
			return this.lists[list_id].form;
	},
	getUids: function(list_id, nocreate) {
		var list = this.getListView(list_id);
		if(! list)
			return null;
		if(! isset(list.uids) && ! nocreate)
			list.uids = new UidList(list.form ? list.form.list_uids : null);
		return list.uids;
	},

	setCheckedAll: function(list_id, checked) {
		var list = this.getListView(list_id);
		if(! list)
			return null;
		var check_all_elm = $(list_id + '-check-all');
		if (check_all_elm) {
			if (checked == 'all') {
				check_all_elm.disabled = true;
				check_all_elm.checked = true;
			} else {
				check_all_elm.disabled = false;
				check_all_elm.checked = checked;
			}
		}
	},

	checkItem: function(list_id, elt, nocurrent) {
		var uidList = sListView.getUids(list_id);
		if(uidList) {
			if(elt.checked)
				uidList.addUid(elt.value);
			else
				uidList.removeUid(elt.value);
		}
		sListView.updateSelectedCount(list_id);
		sListView.row_action(elt, nocurrent ? 'mark' : 'click');
	},

	checkAllItems: function(list_id, uncheck) {
		var uidList = sListView.getUids(list_id);
		if(uidList) {
			if (!uncheck)
				uidList.setUidString('all');
			else
				uidList.clear();
		}
		sListView.updateSelectedCount(list_id);
	},

	setSelected: function(list_id, mode, check_status) {
		var field_name = 'mass[]'
		var elms = document.getElementsByName(field_name);
		var all_check = true;

		if(mode == 'none') {
			sListView.checkAllItems(list_id, true);
			check_status = false;
			all_check = false;
		} else if(mode == 'page') {
			var list = sListView.getUids(list_id).getList();
			if (list === 'all') {
				sListView.checkAllItems(list_id, true);
				check_status = true;
			}
			if (! isset(check_status) && list != 'all')
				check_status = !$(list_id + '-check-all').checked;
			all_check = check_status;
		} else if(mode == 'all') {
			sListView.checkAllItems(list_id, false);
			check_status = true;
			all_check = 'all';
		}
		sListView.setCheckedAll(list_id, all_check);

		for (var i = 0; i < elms.length; i++) {
			if(elms[i].name == field_name) {
				elms[i].checked = check_status;
				if (mode == 'page') {
					elms[i].disabled = false;
				} else {
					if (mode == 'all') {
						elms[i].disabled = check_status;
                    } else {
                        elms[i].disabled = false;
                    }
				}
				sListView.checkItem(list_id, elms[i], true);
			}
		}
	},

	updateSelectedCount: function(list_id) {
		var uidList = sListView.getUids(list_id),
			c = uidList ? uidList.getCount() : 0,
			sHead = $(list_id + '-SelectCountHead'),
			sFoot = $(list_id + '-SelectCountFoot');
		if(c == 'all') c = app_string('LBL_SELECTED_ALL');
		if(sHead) sHead.innerHTML = c;
		if(sFoot) sFoot.innerHTML = c;
	},

    resetSelected: function(list_id) {
        var uidList = sListView.getUids(list_id);
        if(uidList)
            uidList.clear();
    },

	performSearch: function(list_id, form_params, push_state) {
		var list = this.getListView(list_id);
		if(! list || ! list.filter) {
			console.error("missing listview search: "+list_id);
			return false;
		}
		SUGAR.util.maskDiv(list.outer);
		if(form_params)
			SUGAR.ui.updateForm(list.filter, form_params);
		SUGAR.session.setLoadingState('sListView', {method: 'restoreLayout', id: list_id, params: this.getLayoutState(list)}, push_state);
		sendAndRetrieve(list.filter, list.outer);
		return false;
	},

	clearSearch: function(list_id) {
		return this.performSearch(list_id, {filter_clear: 1});
	},

	getLayout: function(list_id) {
		var list = this.getListView(list_id);
		if(list) {
			var form = list.filter || list.form;
			if(form)
				return form.layout.value;
		}
	},

	restoreLayout: function(list_id, vars) {
		if(! vars) return;
		var list = this.getListView(list_id);
		if(list && list.filter && SUGAR.ui.inDocument(list.filter)) {
			if(! vars.report_id) vars.report_id = '';
			if(! vars.data_id) vars.data_id = '';
			if(! vars.filter_layout) vars.filter_layout = '';
			vars.filter_clear = 1;
			this.performSearch(list_id, vars);
		} else {
			var s = SUGAR.session.currentState;
			if(s && s.context == 'sListView' && s.id && s.id != list_id)
				this.restoreLayout(s.id, vars); // changed list_id
			else if(vars && vars.module) {
				vars.initial_load = true;
				SUGAR.conn.asyncRequest('async.php', {postData: vars},
					function(data) {
						SUGAR.util.loadContent('content-main', data.responseText);
					}
				);
			}
		}
	},

	setLayout: function(list_id, layout_name, report_id, data_id) {
		var list = this.getListView(list_id);
		if(! list || ! list.filter)
			return false;
		var vars = {layout: layout_name, filter_layout: '', report_id: report_id || '', data_id: data_id || '', filter_clear: 1};
		if(list.form && list.form.mass_panel_open)
			vars.mass_panel_open = list.form.mass_panel_open.value;
		this.performSearch(list_id, vars, true);
		return false;
	},

    getReportId: function(list_id) {
		var list = this.getListView(list_id);
		if(! list || ! list.filter)
			return false;
		return list.filter.report_id.value;
	},

	getReportFilter: function(list_id) {
		var filter = {},
			list = this.getListView(list_id),
			i, elt;
		if(list && list.filter) {
			for(i = 0; i < list.filter.elements.length; i++) {
				elt = list.filter.elements[i];
				if(elt.name && elt.type != 'button' && elt.type != 'submit' && elt.type != 'file') {
					filter[elt.name] = elt.value;
				}
			}
		}
		return filter;
	},

	runReport: function(list_id, button, add_filter) {
		var rid = sListView.getReportId(list_id),
			filter = add_filter ? sListView.getReportFilter(list_id) : '',
			req = new SUGAR.conn.JSONRequest('run_report', {}, {module: 'Reports', report_id: rid, filter: filter}),
			returnReport = function(data) {
				if(data)
					sListView.setLayout(list_id, 'Reports', rid, data.getResult());
			};
		req.button = button;
		if(button)
			button.disabled = true;
		req.fetch(returnReport);
	},

	returnToReport: function(list_id) {
		var rid = sListView.getReportId(list_id);
		return sListView.setLayout(list_id, 'Reports', rid);
	},

	getFilterLayout: function(list_id) {
		var list = this.getListView(list_id);
		if(! list || ! list.filter)
			return false;
		return list.filter.filter_layout.value;
	},

	setFilterLayout: function(list_id, layout_name) {
		var list = this.getListView(list_id);
		if(! list || ! list.filter) {
			console.log("missing listview search: "+list_id);
			return false;
		}
		this.performSearch(list_id, {filter_layout: layout_name || ''});
	},

	getLayoutState: function(list) {
		var frm = list.filter || list.form;
		if(frm) {
			var state = {module: frm.module.value, action: frm.action.value},
				add_f = ['layout', 'subpanel', 'link_name', 'parent_layout', 'record', 'record_perform', 'list_format'];
			for(var i = 0; i < add_f.length; i++) {
				if(isset(frm.elements[add_f[i]])) {
					v = frm.elements[add_f[i]].value;
					state[add_f[i]] = v;
				}
			}
			if(frm.report_id) {
				state.report_id = frm.report_id.value;
				if(list.form.data_id)
					state.data_id = frm.data_id.value;
			}
			return state;
		}
	},
	getExtendedState: function(list) {
		var state = {params: this.getLayoutState(list)},
			add_f = ['list_id', 'list_filter', 'list_offset', 'list_order_by', 'list_limit', 'list_context'],
			frm = list.form, v;
		if(! state.params) return null;
		if(frm) {
			state.action = frm.getAttribute('action');
			for(var i = 0; i < add_f.length; i++) {
				if(isset(frm.elements[add_f[i]])) {
					v = frm.elements[add_f[i]].value;
					if(add_f[i] == 'list_offset' || add_f[i] == 'list_limit') v = parseInt(v, 10);
					state.params[add_f[i]] = v;
				}
			}
		}
		return state;
	},

	navigate: function(list_id, offset, limit, order_by, parent_list) {
		var list = this.getListView(list_id);
		if(! list || ! list.form || ! list.outer) {
			console.log("missing listview objects: "+list_id);
			return false;
		}
		list.push_state = false;
		list.state_vars = {};
		if(isset(offset)) list.state_vars.list_offset = offset;
		if(isset(limit)) list.state_vars.list_limit = limit;
		if(isset(order_by)) list.state_vars.list_order_by = order_by;
		SUGAR.ui.updateForm(list.form, list.state_vars);
		if (parent_list)
			SUGAR.ui.updateForm(list.form, {parent_list: parent_list});
		else
			SUGAR.ui.updateForm(list.form, {parent_list: ''});
		var uidList = sListView.getUids(list_id, true);
		if(uidList) {
			uidList.save();
			if(list.is_popup) {
				this.persistRowData(list, uidList.getList());
			}
		}
		SUGAR.util.maskDiv(list.outer);
		sendAndRetrieve(list.form, list.outer);
	},

	showDetail: function(list_id, params, evt) {
		var list = sListView.getListView(list_id), list_state, sidebar;
		if(list) list_state = sListView.getExtendedState(list);
		q_params = encodeQueryString(params);
		if(! history.pushState || window.metaKeyPressed || (evt && (evt.which == 2 || (! evt.which && evt.button == 4)))) {
			SUGAR.util.loadUrl('index.php?' + q_params, null, true, evt);
		} else {
			SUGAR.session.setLoadingState(null, null, true, q_params);
			sidebar = list_state && list_state.params.list_context == 'sidebar';
			if(list_state) {
				if(list && list.sidebar_limit) list_state.params.list_limit = list.sidebar_limit;
				SUGAR.themes.saveDetailSidebar(list_state, params.module, params.record);
			}
			SUGAR.util.fetchContentMain('async.php?' + q_params);
			if(! list_state || ! sidebar)
				this.destroy(list_id);
			else
				sListView.setCurrentId(list_id, params.record, true);
		}
	},

	orderBy: function(list_id, order_by) {
		this.navigate(list_id, 0, null, order_by);
		return false;
	},

	getCountOptions: function(list) {
		var cstr = (list.count_options || []).map(function(c) { return c.toString(); });
		return {keys: cstr, values: cstr, width: '70px'};
	},

	showOffsetPopup: function(list_id, tg) {
		var list = this.getListView(list_id), popup;
		if(! list) return;
		if(! list.offsetPopup) {
			popup = $(list_id + '-offset-edit');
			if(! popup) return;
			list.offsetPopup = new SUGAR.ui.PopupSlidePanel(popup, {
				target_offset: {below: true},
				onshow: function() { var inp = $(list_id + '-offset-input'); inp.focus(); inp.select(); },
				onhide: function() { SUGAR.ui.removeNode(popup); },
				destroy_on_close: false}
			);
			//list.offsetPopup.render();
			var copts = this.getCountOptions(list),
				offs = $(list_id + '-offset-input'),
				csel = SUGAR.ui.registerInput(list.form,
					new SUGAR.ui.SelectInput(list_id + '-count-input', {name: 'list_limit', options: copts, show_stacked: true}));
			csel.setup();
			csel.updateLabel(list.form.list_limit.value, list.form.list_limit.value);
			if(offs) {
				offs.value = parseInt(list.form.list_offset.value, 10) + 1;
				offs.onkeypress = function(event) {
					if(event.keyCode==13) return sListView.loadCustomOffset(list_id);
				}
				offs.onkeyup = offs.oninput = function() {
					sListView.updateCustomOffset(list_id, this.value);
				}
			}
		}
		document.body.appendChild(list.offsetPopup.elt); // reparent
		list.offsetPopup.target = tg;
		list.offsetPopup.show();
		return false;
	},

	loadCustomOffset: function(list_id) {
		var list = this.getListView(list_id);
		if(! list || ! list.form) {
			console.log("missing listview objects: "+list_id);
			return false;
		}
		if (list.offsetPopup) list.offsetPopup.close();
		var offset = list.custom_list_offset;
		if(! isNumeric(offset) || offset <= 0) {
			offset = list.form.list_offset.value;
		} else {
			offset --;
		}
		this.navigate(list_id, offset);
	},

	updateCustomOffset: function(list_id, new_value) {
		var list = this.getListView(list_id);
		if(! list) {
			console.log("missing listview objects: "+list_id);
			return false;
		}
		list.custom_list_offset = new_value;
	},

	persistRowData: function(list, uids) {
		if(! list.persist_data.prev_rows)
			list.persist_data.prev_rows = {};
		if(uids.push && list.row_data) {
			for(var i = 0; i < uids.length; i++) {
				var data = list.row_data[uids[i]];
				if(data)
					list.persist_data.prev_rows[uids[i]] = data;
			}
		}
	},

	checkSelected: function(list_id, action, show_msg) {
		var uidList = sListView.getUids(list_id),
			c = uidList ? uidList.getCount() : 0;
		if(action == 'duplicate_merge' && c < 2) {
			alert(app_string('LBL_LISTVIEW_TWO_REQUIRED'));
		} else if(! c) {
			alert(app_string('LBL_LISTVIEW_NO_SELECTED'));
		} else
			return true;
		return false;
	},

    addToFavorites: function(list_id, elt, module, record_id) {
        var list = this.getListView(list_id);
        if(! list || ! list.form || ! list.outer) {
            console.log("missing listview objects: "+list_id);
            return false;
        }
        return SUGAR.ui.addToFavorites(elt, module, record_id);
    },

	massPrint: function(list_id, action) {
		var list = this.getListView(list_id), vresult;
		if(! list || ! list.form || ! list.outer) {
			console.log("missing listview objects: "+list_id);
			return false;
		}
        var uidList = sListView.getUids(list_id);
        if(! uidList || ! uidList.getCount()) {
            alert(app_string('LBL_LISTVIEW_NO_SELECTED'));
            return false;
		}

		var gotContent = function(html) {
			popup.loadContent(html, null, true);
		}

		var form_params = {module: list.form.module.value, list_id: list_id, config: true, mass_action: action};
        form_params.list_uids = uidList.getString();
        var popup = new SUGAR.ui.Dialog('massprint', {modal: true});
		//popup.fetchContent('async.php', {postData: form_params}, null, true);
		this.sendMassUpdate(list_id, 'PrintTemplate', gotContent, form_params);
	},

	sendMassUpdate: function(list_id, action, target, form_params, allow_none) {
		var list = this.getListView(list_id), vresult;
		if(! list || ! list.form || ! list.outer) {
			console.log("missing listview objects: "+list_id);
			return false;
		}
		if(! (vresult = SUGAR.ui.validateForm(list.form)).status) {
			SUGAR.ui.alertInvalidForm(vresult);
			return false;
		}
		if(! allow_none && ! sListView.checkSelected(list_id, action, true)) {
            return false;
		}
		var uidList = sListView.getUids(list_id);
		if(uidList) uidList.save();
		var old_perform = list.form.mass_perform.value, old_format = list.form.list_format.value;
		list.form.mass_perform.value = action;
		if(isset(form_params))
			SUGAR.ui.updateForm(list.form, form_params);
		if(target && typeof target !== 'function') {
			var old_act = list.form.getAttribute('action');
			var old_tg = list.form.getAttribute('target');
			setAttr(list.form, 'action', 'async.php');
			if(target === 'export') target = true;
			if(target === true) target = '_self';
			if(target) setAttr(list.form, 'target', target);
			list.form.submit();
			setAttr(list.form, 'action', old_act);
			setAttr(list.form, 'target', old_tg);
			list.form.mass_perform.value = old_perform;
			list.form.list_format.value = old_format;
		} else {
			if (!target) SUGAR.util.maskDiv(list.outer);
			var old_act = list.form.getAttribute('action');
			var old_tg = list.form.getAttribute('target');
			sendAndRetrieve(list.form, target || list.outer);
			setAttr(list.form, 'action', old_act);
			setAttr(list.form, 'target', old_tg);
			list.form.mass_perform.value = old_perform;
			list.form.list_format.value = old_format;
		}
		return false;
	},

	showActionConfig: function(list_id, wf_action) {
		var list = this.getListView(list_id), mod;
		if(! sListView.checkSelected(list_id, 'wfaction', true)) {
            return false;
		}
		if(! list || ! list.form || ! list.form.module) return;
		mod = list.form.module.value;
		SUGAR.popups.openUrl('async.php?module=Workflow&action=RenderActionConfig&context=listview&trigger_module='+mod+'&operation_type='+wf_action+'&entry_id='+list_id);
		return false;
	},

    emailEntries: function(list_id, module, statement) {
		if (statement) {
			sListView.sendMassUpdate(list_id, 'SendPDFStatements');
		} else {
			SUGAR.session.setLoadingState(null, null, true);
			sListView.sendMassUpdate(list_id, 'EmailMultiple');
		}
    },

    clearCategories: function(list_id) {
		SUGAR.session.setLoadingState(null, null, true);
		sListView.sendMassUpdate(list_id, 'ClearCategories');
    },

    displayOnMap: function(list_id, module) {
        var list = sListView.getListView(list_id);
		var vp = SUGAR.ui.getViewport(10);
        var form_params = {module: list.form.module.value, action: 'MultiMap', target_module: module, list_id: list_id, fixedSize: isIPhone ? vp.height : 0};
        if(! list || ! list.form || ! list.outer) {
            return false;
        }
        var uidList = sListView.getUids(list_id);
        if(! uidList || ! uidList.getCount()) {
            alert(app_string('LBL_LISTVIEW_NO_SELECTED'));
            return false;
        }
        var popup = new SUGAR.ui.Dialog('multimap', {width: 800, min_height: 600, modal: true, title: SUGAR.language.getString('LBL_MULTI_MAP_TITLE', module)});
        form_params.list_uids = uidList.getString();
        popup.fetchContent('async.php', {postData: form_params}, null, true);
    },

    MultiMapDisplay: function(list_id, module) {
		var list = this.getListView(list_id), vresult;
		var vp = SUGAR.ui.getViewport(10);
		if(! list || ! list.form || ! list.outer) {
			console.log("missing listview objects: "+list_id);
			return false;
		}
        var uidList = this.getUids(list_id);
        if(! uidList || ! uidList.getCount()) {
            alert(app_string('LBL_LISTVIEW_NO_SELECTED'));
            return false;
		}

		var gotContent = function(html) {
			popup.loadContent(html, null, true);
		}

		var form_params = {module: module, target_module: module, list_id: list_id, fixedSize: isIPhone ? vp.height : 0};
        form_params.list_uids = uidList.getString();
        var popup = new SUGAR.ui.Dialog('multimap', {width: 800, min_height: 600, modal: true, title: SUGAR.language.getString('LBL_MULTI_MAP_TITLE', module)});
		//popup.fetchContent('async.php', {postData: form_params}, null, true);
		this.sendMassUpdate(list_id, 'GetMultiMap', gotContent, form_params);
	},

    quickCampaign: function(list_id, module) {
        var list = sListView.getListView(list_id);
        var form_params = {module: 'Campaigns', action: 'QuickCampaignDialog', target_module: module, list_id: list_id};
        if(! list || ! list.form || ! list.outer) {
            return false;
        }
        var uidList = sListView.getUids(list_id);
        if(! uidList || ! uidList.getCount()) {
            alert(app_string('LBL_LISTVIEW_NO_SELECTED'));
            return false;
        }
        var popup = new SUGAR.ui.Dialog('quickcampaign', {modal: true, title: SUGAR.language.getString('QCAMPAIGN', module)});
        form_params.list_uids = uidList.getString();
        popup.fetchContent('async.php', {postData: form_params}, null, true);
    },

    convertToTargetList: function(list_id, module, report_id, data_id) {
        var list = sListView.getListView(list_id);
        var form_params = {module: 'ProspectLists', action: 'PopupConvertContacts', target_module: module};
        if(! list || ! list.form || ! list.outer) {
            return false;
        }
        var uidList = sListView.getUids(list_id);
        if(! uidList || ! uidList.getCount()) {
            alert(app_string('LBL_LISTVIEW_NO_SELECTED'));
            return false;
        }
        var popup = new SUGAR.ui.Dialog('convertprospectlist', {modal: true, title: SUGAR.language.getString('LBL_CONVERT_TO_TARGET_LIST', module)});
        form_params.list_id = list_id;
		if (report_id)
			form_params.report_id = report_id;
		if (data_id)
			form_params.data_id = data_id;
        form_params.list_uids = uidList.getString();
        popup.fetchContent('async.php', {postData: form_params}, null, true);
    },

    mailMerge: function(list_id, module) {
        var list = sListView.getListView(list_id);
        var form_params = {module: 'MailMerge', action: 'MergeDialog', target_module: module, list_id: list_id};
        if(! list || ! list.form || ! list.outer) {
            return false;
        }
        var uidList = sListView.getUids(list_id);
        if(! uidList || ! uidList.getCount()) {
            alert(app_string('LBL_LISTVIEW_NO_SELECTED'));
            return false;
        }
        var popup = new SUGAR.ui.Dialog('mailmerge', {modal: true, title: SUGAR.language.getString('MAIL_MERGE', module)});
        form_params.list_uids = uidList.getString();
        popup.fetchContent('async.php', {postData: form_params}, null, true);
    },

    exportVcards: function(list_id, mod) {
        var list = sListView.getListView(list_id);
        var form_params = {module: mod, action: 'ExportVCards'};
        if(! list || ! list.form || ! list.outer) {
            return false;
        }
        var uidList = sListView.getUids(list_id);
        if(! uidList || ! uidList.getCount()) {
            alert(app_string('LBL_LISTVIEW_NO_SELECTED'));
            return false;
        }
        uidList.save();
        var old_perform = list.form.mass_perform.value, old_format = list.form.list_format.value;
        list.form.mass_perform.value = '';
        SUGAR.ui.updateForm(list.form, form_params);
        var old_act = list.form.getAttribute('action');
        var old_tg = list.form.getAttribute('target');
        list.form.action = 'index.php';
        list.form.target = '_self';
        list.form.submit();
        list.form.action = old_act;
        list.form.target = old_tg;
        list.form.mass_perform.value = old_perform;
        list.form.list_format.value = old_format;
    },

    setInviteStatus: function(list_id, model, record_id, status) {
		var list = sListView.getListView(list_id),
			target = list ? list.type : 'dashlet';

        function ready() {
            var row = this.getResult();
            if (row && row.result == 'ok') {
                if (target == 'dashlet')
                    SUGAR.sugarHome.retrieveDashlet(list_id, null, null, null, true);
                else
                	sListView.navigate(list_id);
            }
        }

        var query_params = {model: model, record_id: record_id, accept_status: status};
        var req = new SUGAR.conn.JSONRequest('set_activity_status', {}, query_params);
        req.fetch(ready);

    },

	initPopup: function(list_id, params) {
		var list = this.getListView(list_id);
		if(! list || ! list.form)
			return;
		list.is_popup = true;
		if(params.first_load) {
			list.request_data = get_popup_request_data();
		} else {
			list.request_data = params.request_data;
		}
		var upd = {request_data: JSON.stringify(list.request_data)};
		SUGAR.ui.updateForm(list.form, upd);
		if(list.filter)
			SUGAR.ui.updateForm(list.filter, upd);
		list.row_data = get_default(params.row_data, {});
		// check if only ID and name are needed
		var rdata = list.request_data;
		if(rdata) {
			var fields = rdata.field_to_name_array;
			var simple = true;
			for(var k in fields) {
				if(k != 'id' && k != 'name') {
					simple = false;
					break;
				}
			}
			list.simple_response = simple;
		}
		params.list_id = list_id;
		params.content_id = '' + list_id + '-outer';
		params.fill = true;
		SUGAR.popups.initContent(params);
		list.outer = $(params.content_id);
	},

	popupReturn: function(list_id, module, record) {
		var list = this.getListView(list_id);
		if(! list.request_data) {
			console.error("missing popup request data");
			return false;
		}
		if(! record) record = '';

		var result_data = {
			form_name: list.request_data.form_name,
			module: module,
			passthru_data: list.request_data.passthru_data,
			primary_key: 'id' };

		if(list.simple_response) {
			var field_map = list.request_data.field_to_name_array;
			if(record.push) {
				result_data.associated_row_data = {};
				var row, key;
				var sel_list = {};
				if(list.request_data.return_result_array)
					result_data.result_array = {order: record, data: {}};
				for(var i = 0; i < record.length; i++) {
					key = "ID_" + i;
					sel_list[key] = record[i];
					row = this.getRowValue(list, record[i], field_map);
					result_data.associated_row_data[key] = row;
					if(result_data.result_array)
						result_data.result_array[record[i]] = row;
				}
				result_data.selection_list = sel_list;
				result_data.selection_count = record.length;
			} else {
				var row = this.getRowValue(list, record, field_map);
				result_data.name_to_value_array = row;
				if(list.request_data.return_result_array) {
					result_data.result_array = {order: [record], data: {}};
					result_data.result_array.data[record] = row;
				}
                result_data.filter = this.getFilters(list_id);
            }

			this.popupReturnResult(list_id, result_data);
		} else {
			var req = sListView.getJsonRequest(list_id, record, list.request_data.field_to_name_array, list.request_data.passthru_data);
			req.fetch(function(response) {
				var json_data = response.getResult(), pK,
					rindex = {}, rindex_row = {}, sel_list = {}, c = 0, row, unformat_row, key, keys = [], i;
				if(! json_data || json_data.failed) {
					console.error('bad response to popupReturn request');
					return;
				}
				pK = result_data.primary_key = json_data.primary_key;
				result_data.associated_row_data = {};
				if(list.request_data.return_result_array)
					result_data.result_array = {order: record, data: {}};
				for(i = 0; i < json_data.formatted_rows.length; i++)
					rindex[json_data.formatted_rows[i][pK]] = json_data.formatted_rows[i];
                for(i = 0; i < json_data.rows.length; i++) {
                    keys.push(json_data.rows[i][pK]);
                    rindex_row[json_data.rows[i][pK]] = json_data.rows[i];
				}
				if(! record.push) record = [record];
				if (record.length != 1 || record[0] !== 'all')
					keys = record;
				for(i = 0; i < keys.length; i++) {
					row = rindex[keys[i]];
					if(! row) continue;
                    unformat_row = rindex_row[keys[i]];
					key = "ID_" + (c ++);
					sel_list[key] = row;
					result_data.associated_row_data[key] = row;
                    result_data.name_to_value_array = row;
                    result_data.unformat_name_to_value_array = unformat_row;
					if(result_data.result_array)
						result_data.result_array[keys[i]] = row;
				}
				result_data.selection_list = sel_list;
				result_data.selection_count = keys.length;

				sListView.popupReturnResult(list_id, result_data);
			});
		}
		return false;
	},

    getFilters: function(list_id) {
        var filters = {},
            list = this.getListView(list_id),
            i, elt, value;
        if(list && list.filter) {
            var form_data = ['action', 'id', 'layout', 'filter_layout', 'list_context',
                'list_id', 'list_limit', 'module', 'multi_select', 'request_data'];

            for(i = 0; i < list.filter.elements.length; i++) {
                elt = list.filter.elements[i];
                if(elt.name && elt.type != 'button' && elt.type != 'submit' && elt.type != 'file') {
                    value = elt.value;

                    if (value == '' || form_data.indexOf(elt.name) != -1)
                        continue;

                    if (elt.type == 'checkbox' && !elt.checked)
                        value = 0;

                    filters[elt.name] = value;
                }
            }
        }
        return filters;
    },

    getRowValue: function(list, record, field_map) {
		var found_val = '';
		var row = {};
		if(list.persist_data.prev_rows)
			found_val = get_default(list.persist_data.prev_rows[record], found_val);
		if(list.row_data)
			found_val = get_default(list.row_data[record], found_val);
		var row = {};
		for(var k in field_map) {
			if(k == 'id')
				row[field_map[k]] = record;
			else
				row[field_map[k]] = found_val;
		}
		return row;
	},

	popupReturnSelected: function(list_id, module) {
		var ids = this.getUids(list_id).getList();
		if(ids.length) this.popupReturn(list_id, module, ids);
	},

	popupReturnResult: function(list_id, result_data) {
		var list = this.getListView(list_id);
		var cb = eval("window." + list.request_data.call_back_function);
		if(cb) {
			this.closePopup(list_id);
			cb(result_data);
		} else
			console.error("callback not defined: " + list.request_data.call_back_function);
		return false;
	},

	closePopup: function(list_id) {
		var close_f = get_close_popup();
		if(close_f)
			close_f();
		delete this.lists[list_id]; // free references to popup elements and data
	},

	getJsonRequest: function(list_id, rows, fields, additional_data) {
		var list = sListView.getListView(list_id);
		if(! list || ! list.form) {
			console.log("missing listview objects: "+list_id);
			return false;
		}
		var module = list.form.module.value;
		var layout = sListView.getLayout(list_id);
		//var method = list.is_popup ? 'popup_related_data' : 'list_related_data';
		var req_data = {list_id: list_id, module: module, layout: layout, rows: rows};
		if(fields)
			req_data.fields = fields;
		if(additional_data)
			req_data.additional_data = additional_data;
		return new SUGAR.conn.JSONRequest('popup_related_data', {}, req_data);
	},

	showArrangeLayouts: function(list_id, layout_name, module, from_admin_panel) {
		var list = sListView.getListView(list_id);
		if(! list || ! list.form) {
			console.log("missing listview objects: "+list_id);
			return false;
		}
		list.layout_dlg = new SUGAR.ui.Dialog('layoutEditDlg');
		var vars = {list_layout_perform: 'arrange'};
        if(module) vars.module = module;
        if (from_admin_panel) vars.from_saved_search = true;
		list.layout_dlg.fetchFormContent(list.form, vars, {url: 'async.php'}, null, true);
		return false;
	},

	showCreateLayout: function(list_id, layout_name, module, from_admin_panel) {
		this.showEditLayout(list_id, layout_name, 'create', module, from_admin_panel);
	},

    showDuplicateLayout: function(list_id, layout_name) {
        this.showEditLayout(list_id, layout_name, 'duplicate');
    },

    showEditLayout: function(list_id, layout_name, perform, module, from_admin_panel) {
		var list = sListView.getListView(list_id);
		if(! list || ! list.form) {
			console.log("missing listview objects: "+list_id);
			return false;
		}
		list.layout_dlg = new SUGAR.ui.Dialog('layoutEditDlg');
		var vars = {list_layout_perform: perform || 'edit'};
		if(layout_name) vars.layout = layout_name;
        if(module) vars.module = module;
        if (from_admin_panel) vars.from_saved_search = true;
		list.layout_dlg.fetchFormContent(list.form, vars, {url: 'async.php'}, null, true);
		return false;
	},

	showCreateReport: function(module) {
		var p = SUGAR.ui.PopupManager.currentModal();
		if(! p || p.id != 'layoutEditDlg')
			p = new SUGAR.ui.Dialog('layoutEditDlg');
		var vars = {list_layout_perform: 'create', layout: 'Reports', module: module, action: 'index'};
		var req = new SUGAR.conn.HTTPRequest('async.php', {postData: vars});
		p.fetchContent(req, null, null, true);
		return false;
	},

    adminCreateLayout: function(form) {
        var search_module = SUGAR.ui.getFormInput(form, 'search_module');
        if (search_module && search_module.getValue()) {
            var id_input = document.FilterForm.list_id;
            if (id_input) {
                var list_id = id_input.value;
                var module = search_module.getValue();
                SUGAR.popups.close();
                this.showEditLayout(list_id, null, 'create', module, true);
            }
            return true;
        } else {
            alert(app_string('ERR_CREATE_LIST_LAYOUT_MODULE'));
            return false;
        }
    },

    adminEditLayout: function(layout_id, module) {
        var id_input = document.FilterForm.list_id;
        if (id_input) {
            var list_id = id_input.value;
            var list = sListView.getListView(list_id);
            if(! list || ! list.form) {
                console.log("missing listview objects: "+list_id);
                return false;
            }
            list.layout_dlg = new SUGAR.ui.Dialog('layoutEditDlg');
            var vars = {list_layout_perform: 'edit', layout: layout_id, module: module, from_saved_search: true};
            list.layout_dlg.fetchFormContent(list.form, vars, {url: 'async.php'}, null, true);
        }
        return false;
    },

    deleteLayout: function(list_id, layout_name) {
		var list = sListView.getListView(list_id);
		if(! list || ! list.form) {
			console.log("missing listview objects: "+list_id);
			return false;
		}
		if(! confirm(app_string('MSG_CONFIRM_DELETE_LAYOUT')))
			return false;
		var vars = {list_layout_perform: 'delete', filter_clear: 1};
		if(layout_name) vars.layout = layout_name;
		return SUGAR.ui.sendForm(list.form, vars, {resultDiv: list.outer});
	},

	deleteRun: function(list_id) {
		var list = sListView.getListView(list_id);
		if(! list || ! list.form) {
			console.log("missing listview objects: "+list_id);
			return false;
		}
		if(! confirm(app_string('MSG_CONFIRM_DELETE_RUN')))
			return false;
		var vars = {list_layout_perform: 'delete_run', filter_clear: 1};
		return SUGAR.ui.sendForm(list.form, vars, {resultDiv: list.outer});
	},

	saveLayout: function(form) {
		var list_id = form.list_id.value, list = sListView.getListView(list_id),
			vars = {list_layout_perform: 'save', filter_clear: 1}, params, target;
		if(! list_id) {
			vars.new_from_ext = 1;
			target = true;
			params = {url: 'index.php'};
		} else if(! list || ! list.form) {
			console.log("missing listview objects: "+form.list_id.value);
			return false;
		} else {
			params = {resultDiv: list.outer, receiveCallback: function() { list.layout_dlg.close(); }};
		}
		var ret = SUGAR.ui.sendForm(form, vars, params, target);
		if(! isset(ret)) { // possible validation failed
			var nm = SUGAR.ui.getFormInput(form, 'name'),
				tabs = SUGAR.ui.getFormInput(form, 'form-tabs');
			if(nm && nm.invalid && tabs)
				tabs.setValue('general');
		}
		return ret;
	},

	cancelEditLayout: function(list_id) {
		var list = sListView.getListView(list_id), p;
		if(list && list.layout_dlg) {
			list.layout_dlg.close();
		} else {
			p = SUGAR.ui.PopupManager.currentModal();
			if(p && p.id == 'layoutEditDlg') p.close();
		}
	},

	showReportSql: function(list_id) {
		var list = sListView.getListView(list_id), rid;
		if(list && list.form) {
			rid = list.form.report_id.value;
			if(rid) {
				var dlg = new SUGAR.ui.Dialog('showSqlDlg');
				var vars = {list_layout_perform: 'show_sql', layout: rid, module: list.form.module.value};
				dlg.fetchFormContent(list.form, vars, {url: 'async.php'}, null, true);
			}
		}
	},

	handleKeyDown: function(list_id, evt) {
		var list = sListView.getListView(list_id), key = evt.keyCode, elt, offs, limit;
		if(list && key) {
			if(key == 78 || key == 80 || key == 187 || key == 189) { // n/p/+/-
				if(list.form) {
					offs = parseInt(list.form.list_offset.value, 10);
					limit = parseInt(list.form.list_limit.value, 10);
					if(key == 78) {
						if(evt.shiftKey) offs = list.total_count - limit;
						else offs = Math.min(offs + limit, list.total_count - limit);
					}
					else if(key == 80) {
						if(evt.shiftKey) offs = 0;
						else offs = Math.max(0, offs - limit);
					}
					else if(key == 187) {
						if(limit <= 5) limit = 10;
						else limit += 10;
					}
					else if(key == 189 && limit > 5) {
						if(limit <= 10) limit = 5;
						else limit -= 10;
					}
					else return;
					sListView.navigate(list_id, offs, limit);
					return true;
				}
			}
			else if(key == 67) { // c
				elt = $(list_id + '-create');
				if(elt) elt.onclick();
				return true;
			}
			else if(key == 191) { // forward-slash
				if(list.filter && ! evt.shiftKey) {
					elt = SUGAR.ui.getFormInput(list.filter, 'filter_text');
					if(elt) elt.focus();
					else list.form.focus();
					return true;
				}
			}
			else if(key >= 49 && key <= 57) { // 1-9
				sListView.setCurrentIndex(list_id, key - 49);
				return true;
			}
			else if(key == 77) { // m
				if(evt.shiftKey)
					sListView.setSelected(list_id, 'page');
				else {
					elt = sListView.getCurrentRow(list_id);
					if(elt) sListView.row_action(elt, 'toggle');
				}
				return true;
			}
			else if(key == 86) { // v
				elt = sListView.getCurrentRow(list_id);
				if(elt)
					sListView.clickRowLink(elt);
				return true;
			}
			else if(key == 65 || key == 90) { // a/z
				if(list.form) {
					offs = list.current_index;
					limit = list.result_count;
					if(isset(offs)) {
						if(evt.shiftKey) {
							elt = sListView.getCurrentRow(list_id);
							if(elt) sListView.row_action(elt, 'toggle', null, true);
						}
						if(key == 65) offs --;
						else offs ++;
						if(offs < 0) offs = limit-1;
						if(offs >= limit) offs = 0;
					} else if(key == 65)
						offs = limit-1;
					else
						offs = 0;
					sListView.setCurrentIndex(list_id, offs);
					return true;
				}
			}
			else if(key == 85) { // u
				elt = $('formContent-' + list_id + '-ListUpdate');
				if(elt) {
					if(!evt.shiftKey) {
						sugarListView.toggleUpdate(elt.id);
					}
					if(elt.style.display != 'none' || evt.shiftKey) {
						elt = list.form.ListUpdate_update;
						if(elt) elt.focus();
						//SUGAR.ui.focusForm(list.form);
					}
				}
				return true;
			}
			else if(key == 88) { // x
				if(list.top_menu) {
					if(list.top_menu.hasKey('trash')) list.top_menu.performActionKey('trash');
					else list.top_menu.performActionKey('delete');
				}
				return true;
			}
			else if(key == 83) { // s
				if(list.is_popup) {
					elt = $(list_id + '-accept');
					if(elt) elt.onclick();
					else {
						elt = sListView.getCurrentRow(list_id);
						if(elt)
							sListView.clickRowLink(elt);
					}
					return true;
				}
			}
		}
	},

	destroy: function(list_id) {
		var list = sListView.getListView(list_id);
		if(list) {
			list.form = list.outer = list.filter = list.table = list.top_menu = list.bottom_menu = null;
		}
	},


	/**
	 * Sets/unsets the pointer and marker on listview/subpanel rows
	 *
	 * @param   object    the table row
	 * @param   string    the action calling this script (over, out or click)
	 * @param   string    the ID of the corresponding row
	 * @param   boolean   flag to highlight marked rows (default false)
	 *
	 * @return  boolean
	 */
	row_action: function(theRow, theAction, rowId, newState, ev) {
		var ret = true,
			massChk, clickLink, found, list_id, targ, list, span, tag, ev;
		if(newState === '') newState = null;
		if(theRow.tagName.toLowerCase() == 'input') {
			massChk = theRow;
			theRow = theRow.parentNode;
			while(theRow && theRow.tagName) {
				if(theRow.tagName.toLowerCase() == 'tr') {
					found = true;
					break;
				}
				theRow = theRow.parentNode;
			}
			if(! found)
				return false;
			if(theAction == 'click') theAction = 'ext_click';
			newState = massChk.checked;
		}
		if(! rowId) rowId = theRow.getAttribute('data-id');

		var hoverCls = 'hoverRow',
			markCls = 'markRow',
			hovered = SUGAR.ui.hasClass(theRow, hoverCls),
			marked = SUGAR.ui.hasClass(theRow, markCls),
			removeCls = [],
			addCls = '',
			selRange, ridx, orow, oidx;
		if(! theRow.cells || ! theRow.cells.length)
			return ret;
		list_id = theRow.parentNode.parentNode.id.replace(/-main$/, '');
		list = sListView.getListView(list_id);
		span = list ? list.row_span : 2;

		if(theAction == 'click') {
			ev = SUGAR.event.getEvent(ev);
			if(! ev)
				return true;
			targ = ev.target;
			while(targ && targ.tagName) {
				if(targ == theRow)
					break;
				tag = targ.tagName.toLowerCase();
				if(tag == 'a' || tag == 'input' || tag == 'button' || (tag == 'div' && (SUGAR.ui.hasClass(targ, 'active-icon') || SUGAR.ui.hasClass(targ, 'input-check'))))
					return true; // ignore event
				targ = targ.parentNode;
			}

			if((list && list.is_popup && (! list.form.multi_select || ! list.form.multi_select.value) && list.form.module)
				|| SUGAR.ui.hasClass(document.body, 'mobile')) {
					clickLink = true;
					newState = true;
			}

			if(ev.shiftKey)
				selRange = true;
		}

		if(theAction == 'click' || theAction == 'mark' || theAction == 'toggle') {
			var elts = theRow.cells[0].childNodes;
			for(var i = 0; i < elts.length; i++) {
				if(elts[i].name == 'mass[]') {
					massChk = elts[i];
					break;
				} else if(elts[i].tagName == 'DIV' && SUGAR.ui.hasClass(elts[i], 'input-check') && elts[i].firstChild.name == 'mass[]') {
					massChk = elts[i].firstChild;
					break;
				}
			}
			if(massChk && ! isset(newState)) {
				newState = (theAction == 'mark') ? true : (! massChk.checked);
			}
			if(theAction == 'toggle') theAction = 'click';
		}

		if(theAction == 'clear') {
			 if(hovered)
				removeCls.push(hoverCls);
			 if(marked) {
				removeCls.push(markCls);
				theRow.marked = false;
			 }
		}
		else if(theAction == 'click' || theAction == 'ext_click' || theAction == 'mark') {
			if(marked && ! newState) {
				removeCls.push(markCls);
				theRow.marked = false;
			}
			else if(! marked && newState) {
				addCls = markCls;
				theRow.marked = true;
			}
			else
				theRow.marked = marked;
			ridx = Math.round(theRow.sectionRowIndex / span);

			if(selRange) {
				orow = sListView.getCurrentRow(list_id);
				if(orow) {
					oidx = Math.round(orow.sectionRowIndex / span);
					if(oidx != ridx)
						oidx += (ridx > oidx ? 1 : -1);
					while(oidx != ridx) {
						orow = sListView.getRow(list_id, oidx);
						if(orow) sListView.row_action(orow, 'mark');
						oidx += (ridx > oidx ? 1 : -1);
					}
				}
			}

			if(theAction != 'ext_click' && massChk && massChk.checked == marked) {
				massChk.checked = newState;
				if(massChk.onclick)
					massChk.onclick(null);
			}
			if(theAction != 'mark') {
				list_id = theRow.parentNode.parentNode.id.replace(/-main$/, '');
				sListView.setCurrentIndex(list_id, ridx);
			}
			ret = false; // cancel bubble
		}
		else {
			if(theAction == 'over')
				addCls = hoverCls;
			else if(theAction == 'out')
				removeCls.push(hoverCls);
		}

		for(var i = 0; i < removeCls.length; i++)
			SUGAR.ui.removeClass(theRow, removeCls[i]);
		if(addCls)
			SUGAR.ui.addClass(theRow, addCls);

		if(clickLink) setTimeout(function() { sListView.clickRowLink(theRow); }, 0);
		return ret;
	},

	clickRowLink: function(theRow, evt) {
		var elts = theRow.getElementsByTagName('a');
		// find 'name' link and click it
		for(var i = 0; i < elts.length; i++) {
			if(SUGAR.ui.hasClass(elts[i], 'listViewNameLink')) {
				if(elts[i].onclick && elts[i].onclick() === false) {
					return true;
				}
				if(elts[i].href) {
					SUGAR.util.loadUrl(elts[i].href, null, true, evt, true);
					return true;
				}
			}
		}
	},

	setCurrentIndex: function(list_id, index, silent) {
		var list = sListView.getListView(list_id), prev, row;
		if(! list || (! list.is_primary && ! list.is_popup && ! list.is_sidebar)) return;
		if(isset(index))
			row = list.table.tBodies[0].rows[index*list.row_span];
		if(isset(list.current_index))
			prev = list.table.tBodies[0].rows[list.current_index*list.row_span];
		if(prev) SUGAR.ui.removeClass(prev, 'currentRow');
		if(row) {
			list.current_index = index;
			SUGAR.ui.addClass(row, 'currentRow');
			if(! list.is_popup && ! silent)
				SUGAR.util.scrollToElement(row);
			if(list.is_sidebar && ! silent)
				this.clickRowLink(row);
		} else {
			list.current_index = null;
		}
	},

	setCurrentId: function(list_id, id, silent) {
		var list = sListView.getListView(list_id), rc = sListView.getRowCount(list_id), i, r, offs, state;
		if(list) {
			for(i = 0; i < rc; i++) {
				r = sListView.getRow(list_id, i);
				if(r && r.getAttribute('data-id') == id) {
					if(list.form && list.form.list_current_id) list.form.list_current_id.value = id;
					if(list.filter && list.filter.list_current_id) list.filter.list_current_id.value = id;
					if(list.form.module && list.form.list_context && list.form.list_context.value == 'sidebar') {
						state = sListView.getExtendedState(list);
						offs = parseInt(list.form.list_offset.value, 10) + i;
						state.params.list_offset = offs;
						state.params.list_limit = list.sidebar_limit;
						SUGAR.themes.saveDetailSidebar(state, list.form.module.value, id, $('sidebar-list-head'), $('sidebar-list-body'));
					}
					return sListView.setCurrentIndex(list_id, i, silent);
				}
			}
		}
	},

	checkCurrentIndex: function(list_id) {
		var list = sListView.getListView(list_id), rows, i;
		if(list && list.table && list.table.tBodies.length) {
			if(! isset(list.current_index)) {
				rows = list.table.tBodies[0].rows;
				for(i = 0; i < rows.length; i++) {
					if(rows[i].cells.length && SUGAR.ui.hasClass(rows[i], 'currentRow')) {
						list.current_index = Math.round(i / list.row_span);
						break;
					}
				}
			}
			return list.current_index;
		}
	},

	// return 'current' row (from keyboard navigation/mouse click), or find single checked row and make it current
	getCurrentRow: function(list_id) {
		var list = sListView.getListView(list_id), rows, row, i, found;
		sListView.checkCurrentIndex(list_id);
		if(isset(list.current_index)) {
			row = sListView.getRow(list_id, list.current_index);
		} else {
			rows = list.table.tBodies[0].rows;
			for(i = 0; i < rows.length; i++) {
				if(rows[i].cells.length && SUGAR.ui.hasClass(rows[i], 'markRow')) {
					if(row) return false;
					row = rows[i];
					found = Math.round(i / list.row_span);
				}
				i++; // skip dividers
			}
			if(row)
				sListView.setCurrentIndex(list_id, found, true);
		}
		return row;
	},

	getRow: function(list_id, idx) {
		var list = sListView.getListView(list_id);
		if(list && list.table) return list.table.tBodies[0].rows[idx*list.row_span];
	},

	getRowCount: function(list_id) {
		var list = sListView.getListView(list_id);
		if(list && list.table) return list.table.tBodies[0].rows.length / list.row_span;
	},

	setSidebarTitle: function() {
		var list, offs, t, lbl = $('sidebar-list-title');
		if(this.sidebar_id && (list = this.getListView(this.sidebar_id)) && list.form && lbl) {
			offs = parseInt(list.form.list_offset.value, 10) + 1;
			t = app_string('LBL_LIST_RESULTS') + ' (' + offs + '\u2014' + (offs + list.result_count - 1) + ' '+app_string('LBL_LIST_OF')+' ' + list.total_count + ')'
			SUGAR.ui.setElementText(lbl, t);
		}
	}
};

sugarListView.setupSubpanelsList = function(container_id) {
	new SUGAR.ui.DragTarget('#' + container_id, sugarListView.drag_config);
};

sugarListView.drag_config = {
	acceptedSourceTypes: ['reorderSubpanels'],
	onAcceptSource: function(evt, data, source) {
		return !source ? 'bubble' : null;
	},
	drop_effect: 'move',
	onEnter: function(evt, data) {
		if (!data.values || !data.values.subpanel) return;
		SUGAR.dom.query('.subpanel-entry').addClass('subpanel-reorder');
		var Wrap = SUGAR.dom.elementOrId;
		this.draggingElt = Wrap(data.values.subpanel);
		requestAnimationFrame(function() {
			this.elt.closest().children('.subpanel-entry').css('opacity',  0.5);
			evt.target.style.opacity =  1;
		}.bind(this));
	},
	onDrop: function(evt, data) {
		this.elt.closest().children('.subpanel-entry').css('opacity',  1);
		var order = [];
		var list = sListView.getListView(data.values.list_id);
		list.drag = false;
		var module = list.parent_module;
		var tab = list.subpanel_tab;
		var srcEl = this.elt;
		var els = srcEl.children('.subpanel-entry');
		for (var i = 0; i < els.length; i++) {
			var id = els[i].id.replace(/-outer$/, '');
			var list = sListView.getListView(id);
			order.push(list.subpanel_name);
		}
		var args = {
			module: module,
			tab: tab,
			order: order
		};
		var success = function(data) {
		};
		var req = new SUGAR.conn.JSONRequest('subpanel_order', null, args);
		req.fetch(success);
	},
	onMove: function(evt, data) {
		if (!data.values || !data.values.subpanel) {
			return;
		}
		SUGAR.dom.query('.subpanel-entry').removeClass('dz-drag-hover');
		var Wrap = SUGAR.dom.elementOrId;
		var pos = evt.position();
		var p = evt.currentTarget;
		var elt = this.draggingElt;
		var target = Wrap(evt.target).closest('.subpanel-entry').get(0);
		if (!target)
			return;
		var region = SUGAR.ui.getEltRegion(target);
		if(pos.y > region.top + (region.height / 2))
			target = target.nextSibling;
		if (target) {
			p.insertBefore(elt.get(), target);
		} else {
			p.appendChild(elt.get());
		}
	},
	checkAccept: function(evt, data) {
		return data.values && data.values.subpanel;
	}
};


sugarListView.row_action = sugarListView.prototype.row_action;

// used by subpanel framework
sugarListView.toggleShrink = function(list_id) {
	var list = sListView.getListView(list_id);
	if (list) {
		if (list.drag) {
			list.drag = false;
			return false;
		}
	}
	var shown = toggleDisplay(list_id+'-body') != 'none',
		title = $(list_id+'-title');
	if(title) {
		SUGAR.ui.addRemoveClass(title, 'sectionClosed', ! shown);
		SUGAR.ui.addRemoveClass(title, 'sectionOpen', shown);
	}
	return false;
}

sugarListView.registerSubpanelContainer = function(id) {

};

sugarListView.toggleUpdate = function(outer_id) {
	var shown = toggleDisplay(outer_id) != 'none',
		fid = outer_id.replace(/^formContent-/, ''),
		form = $(fid),
		head = $('formHeader-' + fid);
	if(form && form.mass_panel_open) form.mass_panel_open.value = shown ? 1 : 0;
	if(head) {
		SUGAR.ui.addRemoveClass(head, 'sectionClosed', ! shown);
		SUGAR.ui.addRemoveClass(head, 'sectionOpen', shown);
	}
	return false;
}

sugarListView.loadSubpanels = function(form) {
	var copy = ['module', 'action', 'record', 'layout', 'layout_tab'], vars = {}, e, i, outer;
	form = SUGAR.ui.getForm(form);
	if(! form) {
		console.error('subpanels form not found');
		return;
	}
	for(i = 0; i < copy.length; i++) {
		e = form.elements[copy[i]];
		if(e) vars[e.name] = e.value;
	}
	outer = $(form.getAttribute('name')+'-subpanels');
	if(vars.record && outer) {
		vars.record_perform = 'load_subpanels';
		SUGAR.ui.clearChildNodes(outer);
		outer.appendChild(createElement2('p', {style: 'margin-top: 1em; text-align: center'}, SUGAR.util.waitIcon()));
		SUGAR.conn.asyncRequest('async.php', {postData: vars, requireForm: form.name},
			function(data) {
				SUGAR.util.loadContent(outer, data.responseText);
				sugarListView.setupSubpanelsList(outer.id);
			}
		);
	}
}

sugarListView.reloadSubpanel = function(name) {
	var f, i;
	for(i = 0; i < document.forms.length; i++) {
		f = document.forms[i];
		if((f.name == 'ListUpdate' || f.name == 'PanelUpdate-'+name) && f.list_id && f.link_name && f.link_name.value == name)
			return SUGAR.ui.refreshTarget('subpanel', f.list_id.value);
	}
	return false;
}

sugarListView.moveSubpanelRow = function(list_id, row_id, dir) {
	var list = sListView.getListView(list_id) || {},
		name = list.name,
		reload = function() { sugarListView.reloadSubpanel(list.subpanel_name); },
		params = {module: list.parent_module, record: list.parent_record, link_name: list.subpanel_name, target_id: row_id, offset: dir},
		req = new SUGAR.conn.JSONRequest('move_subpanel_row', null, params);
	if(list.type == 'subpanel')
		req.fetch(reload);
}

sListView = new sugarListView();
// -- end sugarListView class


// format and unformat numbers
(function() {
	var re_cache = {}, num_pat = /(\d)(?=(\d\d\d)+(?!\d))/g;

	unformatNumber = function(n, num_grp_sep, dec_sep) {
		if(! isset(num_grp_sep) || ! isset(dec_sep)) return n;
		n = n.toString();
		if(n.length > 0) {
			nsplit = n.split(dec_sep);
			// longreach - modified - use >= not =
			if(num_grp_sep.length) {
				if(! re_cache[num_grp_sep])
					re_cache[num_grp_sep] = new RegExp(RegExp.escape(num_grp_sep), 'g');
				n = nsplit[0].replace(re_cache[num_grp_sep], '');
			}
			else {
				n = nsplit[0];
			}
			if(nsplit.length >= 2) {
				n = '' + n + '.' + nsplit[1];
			}
			return parseFloat(n);
		}
		return '';
	}

	// round parameter can be negative for decimal, precision has to be postive
	formatNumber = function(n, grp_sep, dec_sep, round, precision) {
		if(! isset(grp_sep) || ! isset(dec_sep)) return n;
		n = n.toString();
		if(n.split) n = n.split('.');
		else return n;

		if(n.length > 2) return n.join('.'); // that's not a num!
		// round
		if(isset(round)) {
			if(round > 0 && n.length > 1) { // round to decimal
				n[1] = parseFloat('0.' + n[1]);
				n[1] = Math.round(n[1] * Math.pow(10, round)) / Math.pow(10, round);
				n[1] = n[1].toString().split('.')[1];
			}
			if(round <= 0) { // round to whole number
				n[0] = Math.round(parseInt(n[0], 10) * Math.pow(10, round)) / Math.pow(10, round);
				n[0] = n[0].toString();
				n[1] = '';
			}
		}

		if(isset(precision) && precision >= 0) {
			if(n.length > 1 && isset(n[1])) n[1] = n[1].substring(0, precision); // cut off precision
			else n[1] = '';
			if(n[1].length < precision) {
				if(! n[0].length) n[0] = '0';
				for(var wp = n[1].length; wp < precision; wp++) n[1] += '0';
			}
		}

		if(grp_sep.length)
			n[0] = n[0].replace(num_pat, '$1' + grp_sep);
		return n[0] + (n.length > 1 && n[1] != '' ? dec_sep + n[1] : '');
	}
})();


/* Pop-up notification management */
SUGAR.module('notify').init(function() {
	var loadTime, alertTimeout, notify = this;

	this.onInit(function() {
		this.reset();
		window.SM2_DEFER = true; // prevent autoload of soundmanager
	});

	return {
		alerts: [],
		html5: {
			supported: function() {
				return isdef(window.webkitNotifications);
			},
			allowed: function() {
				return (webkitNotifications.checkPermission() === 0);
			},
			enable: function(callback) {
				// must be called in response to a user action
				if(! notify.html5.allowed()) webkitNotifications.requestPermission(callback);
				else if(callback) callback();
			},
			create: function(params) {
				if(! params || ! params.title || ! notify.html5.allowed()) return false;
				var n = webkitNotifications.createNotification(params.icon, params.title, params.body);
				if(n) {
					n.onclick = params.onclick;
					n.ondisplay = params.ondisplay;
					if(params.expiry) {
						n.ondisplay = function(evt) {
							setTimeout(function() {
								if(n) n.close();
							}, params.expiry);
							if(params.ondisplay) params.ondisplay(evt);
						}
					}
					n.onclose = params.onclose;
					n.show();
				}
				return n;
			}
		},
		getSecondsLoaded: function() {
			return ((new Date()).getTime() - loadTime) / 1000;
		},
		addAlert: function(alert) {
			if(! isPlainObject(alert)) return;
			notify.alerts.push(alert);
		},
		checkAlerts: function() {
			var secondsSinceLoad = notify.getSecondsLoaded(),
				mj = 0, msg, title = '', body = '', clickevt;
			for(mj = 0 ; mj < notify.alerts.length; mj++) {
				msg = notify.alerts[mj];
				if(! msg.done) {
					if(msg.time < secondsSinceLoad && msg.time > -1 ) {
						title = msg.type + app_string('LBL_SEPARATOR') + msg.name;
						body = (msg.subtitle ? msg.subtitle + '\n' : '') + msg.description;
						msg.done = 1;
						if(SUGAR.session.html5_notify && notify.html5.allowed()) {
							if(msg.redirect) clickevt = function() {
								SUGAR.util.loadUrl(msg.redirect);
							}
							notify.html5.create(
								{title: title, body: body, onclick: clickevt}
							);
						}
						else {
							title = html_escape(msg.name);
							if(msg.icon)
								title = '<div class="input-icon theme-icon module-' + msg.icon + '"></div> ' + title;
							SUGAR.popups.showMessage(body, {
								title_html: title,
								cancel_button: !! msg.redirect,
								cancel_button_label: app_string('LBL_IGNORE_BUTTON_LABEL'),
								url: msg.redirect});
						}
					}
				}
			}
		},
		createSoundPlayer: function(playerId, url) {
			var opts = {id: playerId, url: url}, mgr;
			if(! notify.soundManager) {
				notify.soundManager = mgr = new SoundManager();
				mgr.setup({
					url: 'include/images/player/',
					useHTML5Audio: true,
					debugMode: false,
					onready: function() {
						notify.soundManager.createSound(opts);
					},
					defaultOptions: {
						autoLoad: true,
						multiShot: true,
						stream: false,
						volume: 100
					}
				});
				if(mgr.audioFormats.mp3)
					mgr.audioFormats.mp3.required = false; // allows HTML5-only mode
				mgr.beginDelayedInit();
			} else
				notify.soundManager.createSound(opts);
			return playerId;
		},
		playSound: function(sound) {
			notify.soundManager.play(sound);
		},
		initStatusCheck: function(timestamp, interval, mp3_url, no_sound) {
			notify.status_check_timestamp = timestamp;
			notify.status_check_interval = interval;
			var init_player = function() {
				if(! notify.mail_sound_player)
					notify.mail_sound_player = notify.createSoundPlayer('new_email_player', mp3_url);
			}, init_interval = 12000;
			if(! no_sound)
				setTimeout(init_player, init_interval - 10000);
			notify.newEmailAlert = function() {
				if(! no_sound) {
					init_player();
					if(notify.mail_sound_player) notify.playSound(notify.mail_sound_player);
				}
			}
			notify.status_check_timer = setTimeout(SUGAR.notify.checkStatus, init_interval);
		},
		checkStatus: function(no_sound) {
			if (! isset(notify.status_check_timestamp)) {
				notify.status_check_timestamp = 0;
			}
			if (! isset(notify.status_check_interval)) {
				notify.status_check_interval = 300000;
			}
			var p = SUGAR.ui.PopupManager.currentModal();
			if(p && p.id == 'asyncLogin') {
				// defer check
				notify.status_check_timer = setTimeout(SUGAR.notify.checkStatus, notify.status_check_interval);
				return;
			}
			var params = {silent: true, global: true, timeout: Math.min(notify.status_check_interval - 5000, 120000)},
				req = new SUGAR.conn.JSONRequest('status_check', params, {timestamp: notify.status_check_timestamp, user_id: SUGAR.session.user_id});
			req.fetch(SUGAR.notify.returnStatusCheck);
		},
		returnStatusCheck: function(data) {
			clearTimeout(notify.status_check_timer);
			notify.status_check_timer = setTimeout(SUGAR.notify.checkStatus, notify.status_check_interval);
			if(! data)
				return;
			try {
				var result = data.getResult();
				notify.status_check_timestamp = result.timestamp;
				var count_new = parseInt(result.new_emails);
				if (count_new > 0) {
					var msg = app_string(count_new == 1 ? 'NTC_NEW_EMAIL_MESSAGE' : 'NTC_NEW_EMAIL_MESSAGES');
					msg = msg.replace('%d', count_new);
					SUGAR.ui.showStatus(msg, 3000);
					notify.newEmailAlert();
					var pid = sListView.getPrimaryId(), list, form;
					if(pid) {
						list = sListView.getListView(pid);
						form = list.form;
						if(form && form.module && form.module.value == 'Emails' && form.list_offset && form.list_offset.value == 0
						  && form.list_order_by && form.list_order_by.value == 'date_start DESC') {
							sListView.navigate(pid);
						}
					}
				}
				var span = $('new_emails_counter');
				if (span) {
					span.innerHTML = parseInt(result['unread_emails']);
				}
				if(isset(result.alerts)) {
					notify.reset();
					for(var i = 0; i < result.alerts.length; i++)
						notify.addAlert(result.alerts[i]);
				}
			} catch(e) {
				console.error(e);
			}
		},
		reset: function() {
			this.alerts = [];
			loadTime = (new Date()).getTime();
			if(alertTimeout)
				clearTimeout(alertTimeout);
			alertTimeout = setInterval(this.checkAlerts, 5000);
		}
	};
});



/**
 * General Sugar Utils
 */
SUGAR.module('util').init(function(App) {
	var UT = this,
		TH = App.themes,
		Dom = App.dom,
		loadedScripts = {},
		checkedPreloaded = false,
		pendingScripts = [],
		waitingOnLoad = 0,
		runScriptsInterval,
		checkWindowEval = null,
		isIE = (navigator.userAgent.match(/MSIE 9\b/i)),
		isSafari = (navigator.userAgent.match(/WebKit\/(.*?)\b/i)),
		isSafari3 = (isSafari && isSafari[1] >= 420),
		isGecko = (navigator.product == 'Gecko'),
		detailsPopup = null,
		scrollInfo = null,
		keyHandlers = [];

	this.onDomReady(function() {
		window.addEventListener('beforeunload', UT.handleBeforeUnload); // note - breaks forward/back caching in IE10+
		UT.initKeyHandler();
		UT.initDragHandler();
		UT.initResizeHandler();
		UT.initScrollHandler();
	});

	return {
		additionalDetailsCache: {},
		additionalDetailsCalls: {},

		loadUrl: function(addr, status, target, evt, delay) {
			if(! isset(status)) status = app_string('LBL_LOADING');
			evt = evt || window.event;
			if(target === true) {
				// open new window if meta key pressed
				if(window.metaKeyPressed || (evt && (evt.which == 2 || (! evt.which && evt.button == 4)))) {
					if(!Get_Cookie('ANDROID_APP'))
						target = '_blank';
					window.metaKeyPressed = false;
				}
				else target = null;
			}
			if(status && ! target) SUGAR.ui.showStatus(status);
			if(target) window.open(addr, target);
			else {
				if(delay) {
					if(delay === true) delay = 200; // minimal delay to allow redraw
					setTimeout(function() { location.href = addr; }, delay);
				}
				else location.href = addr;
			}
			SUGAR.ui.stopEvent(evt);
			return false;
		},
		handleLink: function(evt, opts) {
			if(! evt || ! evt.target) return;
			var link = Dom.elementOrId(evt.target).closest('a');
			if(! link.length) return;
			var href = link.attr('href');
			if(! href || (! href.startsWith('index.php') && ! href.startsWith('?'))) return;
			if(link.attr('data-async') == 'false') {
				if(! opts) opts = {};
				opts.async = false;
			}
			if(link.attr('data-ignorechanges')) {
				if(! opts) opts = {};
				opts.ignore_changes = true;
			}
			UT.navigateTo(href, evt, opts);
			return false;
		},
		navigateTo: function(url, evt, opts) {
			if(! opts) opts = {};
			if(isPlainObject(url)) url = 'index.php?' + encodeQueryString(url);
			var async = true;
			if(! url || (! url.startsWith('index.php') && ! url.startsWith('async.php') && ! url.startsWith('?')))
				async = false;
			if(! history.pushState || window.metaKeyPressed || (evt && (evt.which == 2 || (! evt.which && evt.button == 4))))
				async = false;
			if(SUGAR.session.request_method == 'POST' || ! get_default(opts.async, true))
				async = false;
			if(! async) {
				// checkDirty automatically handled
				SUGAR.util.loadUrl(url, null, true, evt, opts.delay);
			} else {
				if(! opts.ignoredirty && (m = UT.checkDirty()) && ! confirm(m))
					return false;
				var query, p;
				if(url.startsWith('index.php'))
					url = 'async.php?' + (query = url.substring(10));
				else {
					p = url.indexOf('?');
					query = (p >= 0) ? url.substring(p + 1) : '';
					if(p == 0) url = 'async.php' + url;
				}
				SUGAR.ui.PopupManager.closeAll();
				SUGAR.ui.addClass(document.body, 'navigating');
				SUGAR.session.setLoadingState(null, null, true, query);
				TH.toggleSlideMenu(false);
				SUGAR.ui.stopEvent(evt);
				UT.fetchContentMain(url, {
					callback: function() { SUGAR.ui.removeClass(document.body, 'navigating'); }
				});
			}
			return false;
		},
		handleBeforeUnload: function(evt) {
			if(SUGAR.conn.abortAll) SUGAR.conn.abortAll();
			var msg = UT.checkDirty();
			if(msg) evt.returnValue = msg;
			return msg;
		},
		checkDirty: function() {
			var f = SUGAR.ui.getForm('DetailForm');
			if(SUGAR.ui.isPrimaryForm(f) && SUGAR.ui.checkFormDirty(f)) {
				return app_string('LBL_CHANGES_WARNING');
			}
		},
		useWindowEval:function() {
			if(checkWindowEval === null) {
				checkWindowEval = false;
				if(window.eval) {
					var chk = '___useWindowEval___';
					window.eval('function ' + chk + '() {}');
					if(window[chk]) {
						delete window[chk];
						checkWindowEval = true;
					}
				}
			}
			return checkWindowEval;
		},
		finishedScriptLoad:function(src, failed) {
			waitingOnLoad --;
		},
		getPendingScripts:function() {
			return pendingScripts;
		},
		appendScriptBlock:function(text, source) {
			/*if(window.execScript) { //this case for IE but some scripts don't exec
				window.execScript(script.text);
			}*/
			/*else if(UT.useWindowEval()) { // seems to diverge in FF3 Mac?
				window.eval(script.text);
			}*/
			if(isIE) {
				var el = document.createElement('script');
				el.type = "text/javascript";
				if (null == el.canHaveChildren || el.canHaveChildren) {
					el.appendChild(document.createTextNode(text));
				} else {
					el.text = text;
				}
				var head = document.getElementsByTagName('head')[0];
				head.appendChild(el);
				head.removeChild(el); // evaluation has already happened
			} else {
				try {
					// http://www.blog.highub.com/javascript/decoding-jquery-evaluates-a-script-in-a-global-context/
					// work around browser issues to make sure window is the context
					(function(data) { window["eval"].call(window, text); })(text);
				} catch(e) {
					if(source)
						console.error("Error parsing "+source);
					else {
						console.error("Error parsing script literal", text);
						console.trace();}
					throw e;
				}
			}
		},
		runPendingScripts:function() {
			if(isIE) {
				// force delay to allow DOM updates
				runScriptsInterval = setInterval("SUGAR.util.runPendingScriptsImmed()", 50);
            } else
				UT.runPendingScriptsImmed();
		},
		runPendingScriptsImmed:function() {
			var head;
			while(pendingScripts.length) {
				if(waitingOnLoad || ((isIE || isSafari) && document.readyState && ! /loaded|complete/.test(document.readyState))) {
					if(! runScriptsInterval)
						runScriptsInterval = setInterval("SUGAR.util.runPendingScriptsImmed()", 50);
					return;
				}
				if(runScriptsInterval) {
					clearInterval(runScriptsInterval);
					runScriptsInterval = null;
				}
				if(! head) head = document.getElementsByTagName('head')[0];

				// include external files
				var added = false;
				while(pendingScripts.length && pendingScripts[0].src) {
					var script = pendingScripts.shift();

					var urlparts = script.src.match(/^(\w+:)\/\/(.*?)\/(.*)$/);
					if(! urlparts || (urlparts[1] == location.protocol && urlparts[2] == location.host)) {
						waitingOnLoad ++;
						// FIXME - enforce script loading order
						var req = SUGAR.conn.asyncRequest(script.src, {callback_scope: script},
							function(data) {
								UT.appendScriptBlock(data.responseText, script.src);
								UT.finishedScriptLoad(this.src); },
							function() { UT.finishedScriptLoad(this.src, true); });
					}
					else {
						var el = document.createElement('script');
						el.type = "text/javascript";
						el.src = script.src;
						if(isSafari3 || isGecko || isIE >= 8) {
							waitingOnLoad ++;
							el.onload = el.onreadystatechange = function() { UT.finishedScriptLoad(this.src); };
							el.onerror = function() { UT.finishedScriptLoad(this.src, true); };
						}
						head.appendChild(el);
					}
					added = true;
				}
				if(added)
					continue;

				// run callbacks
				if(pendingScripts[0].fn) {
					var script = pendingScripts.shift();
					script.fn();
					continue;
				}

				// run script blocks
				var script = pendingScripts.shift();
				if(! script.text)
					continue;

				UT.appendScriptBlock(script.text, script.src);
			}
		},
		getBaseHref:function() {
			var h = document.location.href;
			var p;
			if( (p = h.indexOf('?')) != -1 )
				h = h.substring(0, p);
			if( (p = h.lastIndexOf('/')) != -1 )
				h = h.substring(0, p+1);
			return h;
		},
		checkPreloaded:function() {
			if(checkedPreloaded) return;
			var pfx = this.getBaseHref();
			var elts = document.getElementsByTagName('script');
			for(var i = 0; i < elts.length; i++) {
				if(elts[i].src) {
					var src = elts[i].src;
					if(src.substring(0, pfx.length) == pfx)
						src = src.substring(pfx.length);
					loadedScripts[src] = true;
				}
			}
			checkedPreloaded = true;
		},
		evalScript:function(text, runAfter, defer) {
			var dummyEl = document.createElement("div");
			var scriptText;
			dummyEl.innerHTML = text;
            this.checkPreloaded();
			dummyEl.querySelectorAll("script").forEach((function(scriptEl) {
				if (scriptEl.src) {
					this.addScriptInclude(scriptEl.src, null, {deferExec: true});
				} else {
					scriptText = scriptEl.innerHTML;
					pendingScripts.push({text: scriptText});
				}
			}).bind(this))
			if(runAfter)
				pendingScripts.push({fn: runAfter});
			if(! defer)
				UT.runPendingScripts();
		},
		addScriptInclude:function(src, runAfter, params) {
			if(! params) params = {};
			this.checkPreloaded();
			var pfx = this.getBaseHref();
			if(src.substring(0, pfx.length) == pfx)
				src = src.substring(pfx.length);
			if(params.reload || ! loadedScripts[src]) {
				// load only once for the same URL
				pendingScripts.push({src: src});
				loadedScripts[src] = true;
			}
			if(runAfter)
				pendingScripts.push({fn: runAfter});
			if(! params.deferExec)
				UT.runPendingScripts();
		},
		disableInlineScripts:function(text) {
			text = text.replace(/<script\b[\s\S]*?<\/script\s*>/ig, '');
			return text;
		},

		/**
		 * Make an AJAX request.
		 *
		 * @param	url				string	resource to load
		 * @param	theDiv			string	id of element to insert loaded data into
		 * @param	postForm		string	if set, a POST request will be made to resource specified by url using the form named by postForm
		 * @param	callback		string	name of function to invoke after HTTP response is recieved
		 * @param	callbackParam	any		parameter to pass to callback when invoked
		 * @param	appendMode		bool	if true, HTTP response will be appended to the contents of theDiv, or else contents will be overriten.
		 */
	    retrieveAndFill: function(url, theDiv, postForm, callback, callbackParam, appendMode, loadingMsg, scrollTo) {
			var done = function() {
					SUGAR.session.setLoadingRequest(null);
					if(callback) callback(callbackParam);
					if(scrollTo) UT.scrollToElement(theDiv);
				},
				success = function(data) {
					UT.loadContent(theDiv, data.responseText, appendMode, done);
				}, req;
			if(isset(postForm))
				req = SUGAR.conn.sendForm(postForm, {url: url, status_msg: loadingMsg}, success);
			else
				req = SUGAR.conn.asyncRequest(url, {status_msg: loadingMsg}, success);
			SUGAR.session.setLoadingRequest(req);
		},
		fetchContentMain:function(url, params) {
			if(! isset(params)) params = {};
			UT.maskDiv('content-main');
			if(! params.skipAbortReqs)
				SUGAR.conn.abortPageRequests();
			var loadingMsg = params.loadingMessage || app_string('LBL_LOADING'),
				scrollTo = get_default(params.scrollTo, true);
			this.retrieveAndFill(url, 'content-main', params.postForm, params.callback, params.callbackParam, params.appendMode, loadingMsg, scrollTo);
		},
		loadContent: function(elt, html, append, done) {
			elt = $(elt);
			if(! elt) {
				if(done) done();
				return null;
			}
			if(! isset(html)) html = '';
			if(! isString(html)) {
				SUGAR.ui.clearChildNodes(elt);
				SUGAR.ui.setElementContent(elt, html);
				if(done) done();
				return;
			}
			try {
				var text = UT.disableInlineScripts(html);
				if(append)
					elt.innerHTML += text;
				else
					elt.innerHTML = text;
				UT.evalScript(html, done);
			}
			catch (e) {
				console.error(e);
			}
		},
		scrollToElement: function(elt) {
			var info, t, p;
			if(scrollInfo && scrollInfo.timer)
				clearTimeout(scrollInfo.timer);
			if(isString(elt)) elt = $(elt);
			if(elt && elt.id === 'content-main' && (t = $('main-title'))) // show title as well when content-main is requested
				elt = t;
			for(p = elt; p; p = p.parentNode) {
				if(p.tagName && p.style.position == 'absolute')
					return; // ignore inside of a popup dialog or frame decoration
			}
			info = scrollInfo = {count: 0, elt: elt, first: window.pageYOffset};
			info.timer = setInterval(function() { UT.scrollStep(info); }, 50);
		},
		scrollStep: function(info) {
			var abort = false, curtop = window.pageYOffset;
			if(info !== scrollInfo || ! isElement(info.elt) || ! SUGAR.ui.inDocument(info.elt))
				abort = true;
			else if(info.count > 10000) // extra safety
				abort = true;
			else if(isset(info.last) && Math.abs(info.last - curtop) > 10)
				abort = true;
			if(! abort) {
				var viewp = SUGAR.ui.getViewport(10),
					pos = SUGAR.ui.getEltRegion(info.elt),
					sgn, diff, bottom, newpos;
				if(! viewp || ! pos) return;
				diff = viewp.y - pos.y;
				// just checking top 300px are on screen, to avoid hiding menu bar on small screens
				bottom = pos.y + Math.min(pos.height, 300);
				if(diff > 0) {
					// element top above top of viewport
					sgn = -1;
				} else if(bottom > viewp.y + viewp.height && diff < 0) {
					// element bottom off screen
					sgn = 1;
					diff = Math.min(bottom - viewp.y - viewp.height, -diff);
				} else
					abort = true;
				if(sgn) {
					step = Math.min(Math.max(15, Math.min(Math.round(diff / 3), 80)), diff);
					newpos = curtop + step * sgn;
					window.scrollTo(window.pageXOffset, newpos);
					info.last = newpos;
				}
			}
			if(abort) {
				if(info.timer)
					clearTimeout(info.timer);
			}
		},
		maskDiv:function(theDiv, maskId) {
			if(! maskId) maskId = 'mask';
			var parentId = maskId + 'Parent',
				mask = this[maskId];
			if(! mask)
				mask = this[maskId] = createElement2('div', {className: 'divMask'});
			theDiv = $(theDiv);
			if(! theDiv)
				return;
			this[parentId] = theDiv;
			theDiv.oldPos = theDiv.style.position;
			theDiv.style.position = 'relative';
			theDiv.appendChild(mask);
			mask.style.opacity = 0.0;
			mask.style.display = 'block';
			mask.style.cursor = 'wait';
			mask.onselectstart = mask.onmousedown = mask.onclick = SUGAR.ui.stopEvent;
			setTimeout(function() { mask.style.opacity = null; /* revert to CSS value */ }, 50);
			this.testHideMask(maskId);
			return maskId;
		},
		hideMask: function(maskId, parentId) {
			if(! isset(parentId)) parentId = maskId + 'Parent';
			if(! this[parentId] || ! this[maskId]) return;
			this[parentId].style.position = this[parentId].oldPos;
			this[maskId].style.display = 'none';
			this[parentId] = null;
		},
		testHideMask:function(maskId) {
			var parentId = maskId + 'Parent';
			if(! this[maskId] || ! this[parentId])
				return;
			if(! this[maskId].parentNode || this[maskId].parentNode != this[parentId]) {
				this.hideMask(maskId, parentId);
			} else
				setTimeout("SUGAR.util.testHideMask('"+maskId+"')", 500);
		},
		getPageTitle:function() {
			return document.title;
		},
		setPageTitle:function(str) {
			document.title = str;
		},

		/**
		 * Retrieves additional details dynamically
		 */
		getAdditionalDetails: function(bean, id, spanId, extraArgs, context, delay) {
			if (!extraArgs) {
				extraArgs = {};
			}
			var go = function() {
				var oReturn;
				if(spanId.apply)
					oReturn = spanId;
				else
					oReturn = function(body, caption, width) {
						var dlg = new SUGAR.ui.MiniDialog(null, {target: spanId, target_offset: {left: 13}, width: (width||300)+'px'});
						dlg.loadContent(body, function() { SUGAR.popups.initContent({title: caption, fill: true}); });
						dlg.show();
						return dlg;
					}

				var cachekey = bean + ':' + id;
				if(context) cachekey += ':' + context;
				var success = function(data) {
					delete(UT.additionalDetailsCalls[spanId]);
					if(data) {
						UT.additionalDetailsCache[cachekey] = data.getResult();
					}
					var cache = UT.additionalDetailsCache[cachekey];
					if(cache)
						return oReturn(cache['body'], cache['caption'], cache['width']);
					return false;
				}

				var cache = UT.additionalDetailsCache[cachekey];
				if(isset(cache))
					return oReturn(cache['body'], cache['caption'], cache['width']);

				if(typeof UT.additionalDetailsCalls[spanId] != 'undefined') // call already in progress
					return;
				extraArgs.bean_name = bean;
				extraArgs.record = id;
				extraArgs.context = context;
				var args = (context == 'refinput' ? {silent: true} : null);
				var req = UT.additionalDetailsCalls[spanId] = new SUGAR.conn.JSONRequest('additional_details', args, extraArgs);
				req.fetch(success);
				UT.additionalDetailsLastCall = spanId;
				return false;
			}
			UT.additionalDetailsRpcCall = window.setTimeout(go, get_default(delay, 250));
		},
		clearAdditionalDetailsCall: function() {
			if(typeof UT.additionalDetailsRpcCall == 'number')
				window.clearTimeout(UT.additionalDetailsRpcCall);
			if(UT.additionalDetailsLastCall) {
				var call = UT.additionalDetailsCalls[UT.additionalDetailsLastCall];
				if(call) {
					call.abort();
					delete(UT.additionalDetailsCalls[UT.additionalDetailsLastCall]);
				}
				UT.additionalDetailsLastCall = null;
			}
		},
		getPos: function(elt) {
			var pos = {'left': 0, 'top': 0, 'width': 0, 'height': 0};
			if(elt) {
				pos.width = elt.offsetWidth;
				pos.height = elt.offsetHeight;
			} else
				pos.failed = true;
			var foundCell = false;
			while(elt && elt.nodeName != "BODY") {
				pos.left += elt.offsetLeft;
				pos.top += elt.offsetTop;
				if(! foundCell && elt.nodeName == 'TD') {
					foundCell = true;
					pos.cellTop = pos.top;
				}
				elt = elt.offsetParent;
			}
			pos.right = pos.left + pos.width;
			pos.bottom = pos.top + pos.height;
			return pos;
		},
		checkNavigationTag: function(category) {
			var tag = document.location.hash || '';
			var key = '#nav';
			if(category) key += '-' + category;
			if(tag.substring(0, key.length+1) == key + ':')
				return tag.substring(key.length + 1);
			return false;
		},
		setNavigationTag: function(tag, category) {
			var key = '#nav';
			if(category) key += '-' + category;
			document.location.hash = key + ':' + tag;
		},

		fadeTimers: {},
		fadeOutAfter: function(elt, timeout, duration) {
			if(this.fadeTimers[elt]) {
				clearTimeout(this.fadeTimers[elt].timer);
			}
			if(! isset(duration)) duration = 0.25;
			var stat = {
				anim: new App.animation.Animation()
					.distribute(20, function(percent) { return {opacity: (100 - percent) / 100}})
					.duration(duration * 1000)
				.timing('ease-out')
				.onEnd( function() { App.dom.extNode(elt).hide()})
			};
			this.fadeTimers[elt] = stat;
			stat.timer = setTimeout(function() {
				stat.anim.run(elt);
			}, timeout || 0);
		},

		showPrintConfig: function(button, pdf_type) {
			if(! button.form) return;
			var popup = new SUGAR.ui.MiniDialog('print-config', {modal: true, target: button.elt}),
				vars = {action: 'PDF', config: true};
			if(pdf_type) vars.pdf_type = pdf_type;
			popup.fetchFormContent(button.form, vars, {url: 'async.php'}, null, true);
			//popup.fetchContent('async.php?module='+mod+'&action=PDF&record='+rec+'&config=1', null, null, true);
		},

		showTemplatePrintConfig: function(button, pdf_type) {
			if(! button.form) return;
			var popup = new SUGAR.ui.MiniDialog('print-template-config', {modal: true, target: button.elt||button, target_active_class: 'active opened', close_on_ext_click: false}),
				vars = {action: 'PrintTemplate', config: true};
			popup.fetchFormContent(button.form, vars, {url: 'async.php'}, null, true);
			//popup.fetchContent('async.php?module='+mod+'&action=PDF&record='+rec+'&config=1', null, null, true);
		},

		initKeyHandler: function() {
			Dom.extNode(document)
				.on('keydown', UT.keyDownHandler)
				.on('keyup', UT.keyUpHandler);
			Dom.extNode(window)
				.on('blur', UT.winBlurHandler);
		},
		addKeyHandler: function(handler) {
			keyHandlers.push(handler);
		},
		keyUpHandler: function(evt) {
			window.metaKeyPressed = evt.cmdKey;
		},
		keyDownHandler: function(evt) {
			var tgt = evt.target, tag, id;
			if(evt.ctrlKey || evt.metaKey) { window.metaKeyPressed = true; return; } // ignore key events when ctrl/cmd is pressed
			while(tgt && tgt.tagName) {
				tag = tgt.tagName.toLowerCase();
				if((tag == 'input' || tag == 'textarea' || tag == 'select' || (SUGAR.ui.hasClass(tgt, 'input-field') && evt.keyCode !== 192 )))
					return; // ignore event
				tgt = tgt.parentNode;
			}
			for(var i = 0; i < keyHandlers.length; i++) {
				if(keyHandlers[i](evt.originalEvent)) {
					return;
				}
			}
			var popup = SUGAR.popups.getCurrent();
			if(popup && popup.modal) {
				if(popup.list_id && sListView.handleKeyDown(popup.list_id, evt)) {
					SUGAR.ui.stopEvent(evt);
					return;
				}
			}
			if( (id = sListView.getPrimaryId()) || (id = sListView.getSidebarId()) ) {
				if(sListView.handleKeyDown(id, evt)) {
					SUGAR.ui.stopEvent(evt);
					return;
				}
			}
			if(evt.keyCode == 191 && evt.shiftKey) { // ?
				tgt = SUGAR.ui.getFormInput('UnifiedSearch', 'query_string') || $('search-field');
				if(tgt) {
					tgt.focus();
					SUGAR.ui.stopEvent(evt);
					return;
				} else {
					tgt = $('tbar-search');
					if(tgt) {
						tgt.click();
						SUGAR.ui.stopEvent(evt);
						return;
					}
				}
			}
			tgt = null;
			if(evt.keyCode == 69) // e
				tgt = $('DetailForm_edit');
			else if(evt.keyCode == 68) // d
				tgt = $('DetailForm_duplicate');
			else if(evt.keyCode == 76) // l
				SUGAR.ui.returnToList('DetailForm');
			else if(evt.keyCode == 84) // t
				SUGAR.util.scrollToElement('content-main');
			else if(evt.keyCode == 88) // x
				tgt = $('DetailForm_delete') || $('DetailForm_trash');
			else if(evt.keyCode == 112) // f1
				tgt = $('module-help');
			else if(evt.keyCode == 192) { // ` or ~
				tgt = SUGAR.ui.getMenuSource(null, evt.shiftKey ? 'tbar-profile' : 'grouptab-0');
				if(tgt) {
					if (tgt.popup && tgt.popup.visible) {
						tgt.closePopup();
						document.getElementById("top-app-name").parentElement.focus()
					} else {
						tgt.showPopup();
					}
					SUGAR.ui.stopEvent(evt);
					return;
				}
			}
			if(tgt && tgt.onclick) {
				tgt.onclick();
				SUGAR.ui.stopEvent(evt);
				return;
			}
		},

		drag_elt_count: 0,
		initDragHandler: function() {
			Dom.extNode(document).on('dragenter, dragover, dragleave, drop', UT.handleDrag);
		},
		handleDrag: function(e) {
			var targ = e.target,
				tag = targ && targ.tagName.toLowerCase();
			if(tag == 'input' || tag == 'textarea' || tag == 'button')
				return;
			if(e.dataTransfer.files || e.dataTransfer.mozItemCount) {
				e.stopEvent();
				if(e.type == 'drop') {
					if(e.dataTransfer.files.length)
						;//console.log('drop ' + e.dataTransfer.files[0].name);
					UT.endDrag();
				}
				else if(e.type == 'dragleave') {
					setTimeout(function() {
						UT.drag_elt_count --;
						if(UT.drag_elt_count <= 0)
							UT.endDrag();
					}, 1);
				}
				else if(e.type == 'dragenter') {
					if(! UT.drag_elt_count) {
						/*var dlg = new SUGAR.ui.Dialog('iah_drag_drop', {
							have_close_button: false,
							align_center: true,
							title: "Upload files",
						});
						var h3 = createElement2('h3', {}, "Upload files", dlg.getContentElement());
						UT.drag_dlg = dlg;
						dlg.render() && dlg.show();*/
					}
					UT.drag_elt_count ++;
				}
			}
		},
		endDrag: function() {
			UT.drag_elt_count = 0;
			if(UT.drag_dlg) UT.drag_dlg.close();
		},

		initResizeHandler: function() {
			SUGAR.ui.addResizeHook(UT.handleResize, UT);
		},
		handleResize: function(evt, sz, osz) {
			var tog, pts = TH.page_breakpts,
				md = pts[2].width,
				lg = pts[3].width;
			if(osz) {
				if(osz.w >= md && sz.w < md)
					tog = false;
				else if(osz.w < md && sz.w >= md) {
					tog = true;
					TH.toggleSlideMenu(false);
				}
				if(isset(tog)) TH.resizeToggleSidebar(tog);
			}
			// determine breakpoint
			var i, pgsz, cls, remcls = [];
			for(i = 0; i < pts.length; i++) {
				if(pts[i].width && sz.w < pts[i].width) break;
				pgsz = pts[i].name;
			}
			for(i = 0; i < document.body.classList.length; i++) {
				cls = document.body.classList.item(i);
				if(cls.match(/^pgsz-/)) remcls.push(cls);
			}
			if(pgsz)
				remcls.remove('pgsz-' + pgsz);
			for(i = 0; i < remcls.length; i++)
				document.body.classList.remove(remcls[i]);
			if(pgsz)
				document.body.classList.add('pgsz-' + pgsz);
			if(TH.pageSizeClass !== pgsz) {
				TH.pageSizeClass = pgsz;
				Dom.extNode(window).emit('page-size', pgsz);
			}
		},
		getPageSize: function() {
			return TH.pageSizeClass;
		},

		move_sidebar: null,
		move_timeout: null,
		initScrollHandler: function() {
			SUGAR.ui.addScrollHook(UT.handleScroll, UT);
		},
		handleScroll: function(evt, pos, oldpos) {
			var floatm = TH.topMenuFloat(),
				m = $('tab-bar'),
				s = $('left-sidebar'), s2,
				main = $('content-main'), mt, diff,
				ssz = SUGAR.ui.getEltRegion(s), siz, top, btm, a, b,
				view = SUGAR.ui.getViewport(),
				menusz = TH.topMenuSize(1); // - TH.topMenuSize();
			if(floatm && m) {
				if(pos.y > 10 && (oldpos.y <= 10 || oldpos.first)) {
					SUGAR.ui.addClass(m, 'compact');
				}
				else if(pos.y < 10 && (oldpos.y >= 10 || oldpos.first)) {
					SUGAR.ui.removeClass(m, 'compact');
				}
			}
			if(floatm && main && s && (s2 = s.firstChild) && ! isIPhone && TH.sidebarOpen()) {
				siz = SUGAR.ui.getEltRegion(s2);

				// aligned to bottom of sidebar
				a = Math.max(ssz.top, ssz.bottom - siz.height) - pos.y;
				// aligned to bottom of viewport
				b = Math.min(view.height + view.offset - siz.height, ssz.bottom - siz.height - pos.y);
				btm = Math.min(a, b);
				// aligned to top of sidebar
				a = ssz.top - pos.y;
				// aligned to top of viewport
				b = menusz;
				if(btm > a && b > a) {
					s.style.minHeight = '' + siz.height + 'px';
					UT.move_sidebar = [
						'' + (ssz.left - pos.x) + 'px',
						'' + Math.min(btm, b) + 'px'
					];
				} else {
					UT.move_sidebar = false;
				}
				UT.moveSidebar();
			}
		},
		moveSidebar: function() {
			var s = $('left-sidebar').firstChild;
			if(UT.move_sidebar) {
				SUGAR.ui.addClass(s, 'fixed');
				s.style.left = UT.move_sidebar[0];
				s.style.top = UT.move_sidebar[1];
			} else if(UT.move_sidebar === false) {
				SUGAR.ui.removeClass(s, 'fixed');
				s.style.top = s.style.left = '';
			}
			UT.move_sidebar = null;
			UT.move_timeout = null;
		},

		winBlurHandler: function() {
			window.metaKeyPressed = false;
		},

		showHelp: function(module) {
			var url = 'docs/index.php';
			if(module) url += '?module=' + encodeURIComponent(module);
			UT.loadUrl(url, null, '_blank');
			return false;
		},

		waitIcon: function(attrs) {
			var lbl = app_string('LBL_LOADING'), ret, i;
			if(! attrs) attrs = {};
			if(! attrs.img && (isdef(document.body.style.animation) || isdef(document.body.style.WebkitAnimation))) {
				ret = createElement2('div', {className: 'wait-icon dynamic' + (attrs.small ? ' small' : ''), title: lbl});
				for(i = 1; i <= 3; i++)
					createElement2('div', {className: 'bar'+i}, null, ret);
				return ret;
			}
			return createElement2('img', {src: 'themes/Default/images/sqsWait.gif', className: 'wait-icon', width: 16, height: 16, title: lbl});
		}
	};
}); // end util


SUGAR.module('language').init(function() {
	var LANG = this, langs = {};

	var LangList = function(keys, vals) {
		this.keys = keys;
		this.values = vals;
		this.getKeys = function() {
			return this.keys;
		}
		this.getValues = function() {
			return this.values;
		}
		this.getObject = function() {
			var ret = {};
			if(this.keys) {
				for(var i = 0; i < this.keys.length; i++)
					ret[this.keys[i]] = this.values[i];
			}
			return ret;
		}
		this.getString = function(key) {
			if(this.keys && isset(key)) {
				var p = this.keys.indexOf(key);
				if(p >= 0) return this.values[p];
			}
		}
		this.length = function() {
			return this.keys ? this.keys.length : 0;
		}
		this.getOptions = function() {
			return {keys: this.keys, values: this.values};
		}
	}

	return {
		changeTo: function(lang) {
			var s = location.search.replace(/[&?]user_language=\w+/, '');
			location.href = (s ? s+'&' : '?') + 'user_language=' + lang;
		},
		populate: function(data) {
			if(typeof(data) != 'object' || ! data.language || ! data.name) return;
			if(! langs[data.language]) langs[data.language] = {app: {strings: {}, lists: {keys: {}, values: {}}}};
			if(! data.strings) data.strings = {};
			if(! data.lists) data.lists = {keys: {}, values: {}};
			langs[data.language][data.name] = data;
		},
		getModule: function(module, language) {
			if(! language) language = SUGAR.session.language;
			if(! module || module == 'current') module = SUGAR.session.module;
			if(! langs[language]) return null;
			if(! langs[language][module]) module = 'app';
			return langs[language][module];
		},
		getString: function(str, module, language) {
			if(! str) return str;
			while(1) {
				var mod = LANG.getModule(module, language);
				if(! mod)
					return str;
				var ret = mod.strings[str];
				if(isset(ret))
					return ret;
				inh = mod.inherit_from || 'app';
				if(inh == module || inh == mod.name || inh == module)
					return str;
				module = inh;
			}
		},
		getList: function(str, module, language) {
			if(! str) return str;
			while(1) {
				var mod = LANG.getModule(module, language);
				if(! mod)
					return false;
				if(mod.lists.keys) {
					var keys = mod.lists.keys[str];
					if(isset(keys))
						return new LangList(keys, mod.lists.values[str]);
				}
				inh = mod.inherit_from || 'app';
				if(inh == mod.name || inh == module)
					return false;
				module = inh;
			}
		},
		getListString: function(dom, val, module, language) {
			var list = this.getList(dom, module, language);
			if(list) return list.getString(val);
		},
		addList: function(str, value, language) {
			var mod = LANG.getModule('app', language), p;
			if(mod && value && value.keys) {
				mod.lists.keys[str] = value.keys;
				mod.lists.values[str] = value.values;
				return true;
			}
			return false;
		},
		getLoaded: function() {
			var mods = [], ms = langs[SUGAR.session.language], m;
			if(ms)
				for(m in ms) mods.push(m);
			return mods;
		},
		/* old interface */
		get: function(lang_list, str) {
			if(lang_list == 'mod_strings')
				lang_list = null;
			else if(lang_list == 'app_strings')
				lang_list = 'app';
			else if(lang_list == 'app_list_strings') {
				var list = LANG.getList(str, 'app'), ret = {};
				if(list) ret = list.getObject();
				return ret;
			}
			return LANG.getString(str, lang_list);
		}
	};
}); // end language


SUGAR.module('themes').init(function(App) {
	var TH = this,
		Wrap = App.dom.elementOrId,
		makeElt = App.dom.makeElement;
	this.onDomReady(function() {
		TH.detectFeatures();
		TH.addListeners();
		TH.initSidebar();
	});

	return {
		side_arrow_width: 10,
		menu_offset: {left: 8, top: -2},
		left_col_div: 'left-sidebar',
		show_left_col: null,
		auto_hide_menu: '',
		font_styles: {},
		color_styles: {},
		prev_link_elt: {},
		sidebar_list: null,
		init_page_hooks: [],
		page_breakpts: [
			{name: 'xs'},
			{name: 'sm', width: 768},
			{name: 'md', width: 992},
			{name: 'lg', width: 1200}
		],
		changeTo: function(theme) {
			var s = location.search.replace(/[&?]user_theme=\w+/, '');
			location.href = (s ? s+'&' : '?') + 'user_theme=' + encodeURIComponent(theme);
		},
		changeCss: function(style) {
			var s = location.search.replace(/[&?]theme_style=\w+/, '');
			location.href = (s ? s+'&' : '?') + 'theme_style=' + encodeURIComponent(style);
		},
		changeColor: function(colorName) {
			for(var c in this.color_styles)
				SUGAR.ui.removeClass(document.body, 'color-'+c);
			if(isset(this.color_styles[colorName]))
				return this.changeStyle('color', colorName, this.color_styles[colorName]);
			return false;
		},
		changeFont: function(fontName) {
			for(var c in this.font_styles)
				SUGAR.ui.removeClass(document.body, 'font-'+c);
			if(isset(this.font_styles[fontName]))
				return this.changeStyle('font', fontName, this.font_styles[fontName]);
			return false;
		},
		changeStyle: function(styleType, styleName, newStyle) {
			if(newStyle.link_elt && newStyle.link_elt == this.prev_link_elt[styleType])
				return true;
			if(! this.prev_link_elt[styleType])
				this.prev_link_elt[styleType] = $('theme:'+styleType);
			var prev = this.prev_link_elt[styleType];
			var nstyle = null;
			if(newStyle.link_elt) {
				nstyle = newStyle.link_elt;
			} else if(newStyle.href) {
				nstyle = document.createElement('link');
				nstyle.type = 'text/css';
				nstyle.title = newStyle.title;
				nstyle.href = newStyle.href;
				nstyle.disabled = true;
				nstyle.setAttribute('rel', 'stylesheet');
				nstyle.id = '';
				document.getElementsByTagName('head')[0].appendChild(nstyle);
				newStyle.link_elt = nstyle;
			}
			if(nstyle)
				nstyle.disabled = false;
			this.prev_link_elt[styleType] = nstyle;

			SUGAR.ui.addClass(document.body, styleType+'-'+styleName);
			if(prev)
				prev.disabled = true;
			Set_Cookie(''+SUGAR.session.theme+'_'+styleType+'_style', styleName, 365, '/', '', '');
			return true;
		},
		initLeftCol: function(override_show) {
			var showLeftCol = Get_Cookie('showLeftCol');
			if(isset(TH.show_left_col))
				showLeftCol = TH.show_left_col ? 'true' : 'false';
			if (showLeftCol === null) {
				showLeftCol = 'true';
				Set_Cookie('showLeftCol', showLeftCol, 30, '/', '', '');
			}
			TH.toggleLeftCol(showLeftCol == 'true');
		},
		toggleLeftCol: function(show) {
			var leftCol = $(TH.left_col_div);
			var hideHandle = $('HideHandle');
			if(! leftCol) return;
			if(typeof(show) == 'undefined')
				show = (leftCol.style.display == 'none' || leftCol.oldParent);
			var modulesMenu = $('modulesMenu');
			var modulesSrc = $('modulesMenuSource');
			var autoMenu;
			if(TH.auto_hide_menu)
				autoMenu = $(TH.auto_hide_menu);
			if(show) {
				if(leftCol.oldParent) {
					leftCol.oldParent.appendChild(leftCol);
					leftCol.oldParent = null;
				}
				if(modulesMenu) {
					if(modulesSrc)
						modulesSrc.style.display = 'none';
					if(modulesMenu.oldParent) {
						modulesMenu.oldParent.appendChild(modulesMenu);
						modulesMenu.oldParent.style.display = '';
						modulesMenu.oldParent = null;
					}
				}
				leftCol.style.display = 'block';
				if(! isset(TH.show_left_col))
					Set_Cookie('showLeftCol', 'true', 30, '/', '', '');
				if(hideHandle) {
					if(hideHandle.src)
						hideHandle.src = TH.imagePath+'hide.gif';
					else
						hideHandle.style.backgroundPosition = '0 0';
				}
				if(autoMenu)
					autoMenu.style.display = 'none';
			} else {
				leftCol.style.display = 'none';
				if(modulesMenu) {
					var modulesPopup = $('modulesMenuPopup');
					if(modulesPopup && modulesSrc) {
						modulesMenu.oldParent = modulesMenu.parentNode;
						modulesMenu.parentNode.style.display = 'none';
						modulesPopup.appendChild(modulesMenu);
						modulesSrc.style.display = '';
					}
				}
				if(! isset(TH.show_left_col))
					Set_Cookie('showLeftCol','false',30,'/','','');
				if(hideHandle) {
					if(hideHandle.src)
						hideHandle.src = TH.imagePath+'show.gif';
					else
						hideHandle.style.backgroundPosition = '-10px 0';
				}
				if(autoMenu)
					autoMenu.style.display = '';
			}
		},
		showHiddenMenu: function(targ, offset) {
			if(! isset(offset)) offset = {left: TH.side_arrow_width};
			var hideMenu = $('HideMenu');
			var leftCol = $(TH.left_col_div);
			if(! leftCol || ! hideMenu) return;
			if(leftCol.style.display == 'none') {
				if(! leftCol.oldParent)
					leftCol.oldParent = leftCol.parentNode;
				hideMenu.appendChild(leftCol);
				leftCol.style.display = 'block';
			}
			SUGAR.popups.showPopup(targ, hideMenu, offset);
		},
		chooseTab: function(tab, evt) {
			Set_Cookie('parentTab', tab, 0, '/','','');
			SUGAR.util.handleLink(evt);
		},
		findGroupTab: function() {
			var pg = TH.page_info || {},
				target = pg.module,
				cookieSel = Get_Cookie('parentTab'),
				i, j, grp, tab, active, target;
			if(pg.state) {
				if(pg.state.module) target = pg.state.module;
				if(pg.state.context === 'dashboard')
					target = pg.state.layout;
			}
			if(! TH.menuStructure || ! TH.menuStructure.grouped)
				return;
			for(i = 0; i < TH.menuStructure.grouped.length; i++) {
				grp = TH.menuStructure.grouped[i];
				for(j = 0; j < grp.modules.length; j++) {
					tab = grp.modules[j];
					if(grp.id === cookieSel && (tab.module === target || tab.id === target)) {
						active = {key: grp.key, index: i, active_index: j};
						break;
					}
					if(! active && (tab.module === target || tab.id === target)) {
						active = {key: grp.key, index: i, active_index: j};
					}
				}
			}
			return active;
		},
		updateCurrentTab: function(index) {
			if(! isset(index)) return;
			var i = 0, current = [], UI = SUGAR.ui, tab, found;
			while(true) {
				tab = $('grouptab-' + i);
				if(! tab)
					break;
				if(UI.hasClass(tab, 'current'))
					current.push(tab);
				if(i === index)
					found = tab;
				i ++;
			}
			if(found) {
				for(i = 0; i < current.length; i++)
					UI.removeClass(current[i], 'current');
				UI.addClass(found, 'current');
			}
		},
		initTopMenu: function() {
			var i, j, e, req, o, m, ms, plus, grp;
			if(! $('grouptab-0'))
				return;
			if(! isset(TH.menuStructure)) {
				req = new SUGAR.conn.JSONRequest('get_menu_structure', {silent: true, global: true});
				req.fetch(function() {
					TH.menuStructure = this.getResult();
					TH.initTopMenu();
				});
				return;
			}
			if(TH.menuStructure.grouped && ! TH.disableGroupTabs) {
				TH.groupMenus = {};
				TH.groupOptions = {};
				var topOpts = {keys: [], values: []};
				for(i = 0; i < TH.menuStructure.grouped.length; i++) {
					grp = TH.menuStructure.grouped[i];
					topOpts.keys.push(i);
					topOpts.values.push(grp.label);
					e = $('grouptab-' + i);
					if(e) {
						e.grpIndex = i;
						ms = grp.modules;
						o = {keys: [], values: []};
						for(j = 0; j < ms.length; j++) {
							o.keys.push(ms[j].id || ms[j].module);
							ms[j].icon = (ms[j].icon || 'theme-icon module-' + ms[j].module);
							o.values.push(ms[j]);
						}
						TH.groupOptions[i] = o;
						m = TH.groupMenus[i] = SUGAR.ui.registerMenuSource(
							e.firstChild, {
								id: e.id, options: o, hover_show: true, icon_key: 'icon',
								delay: 250, width: 200, ext_class: 'popup-global moduleMenu',
								target_offset: TH.menu_offset}, null, 'group-toolbar');
						m.setup();
						m.menu.oncurrent = function(idx, old) {
							if(this.touch_opened) return;
							var r = this.getOptionRow(idx);
							var v = this.getValue(this.getKey(idx));
							if(r && v && v.create) {
								if(! plus) {
									plus = createElement2('span', {className: 'uii uii-add active-icon'});
									plus = createElement2('div', {className: 'option-cell input-label option-icon'}, plus);
								}
								plus.onmousedown = function(e) { e.stopPropagation(); }
								plus.onclick = function(e) { SUGAR.util.navigateTo({module: v.module, action: 'EditView'}); SUGAR.ui.stopEvent(e); SUGAR.popups.close(); }
								r.appendChild(plus);
							} else if(plus)
								SUGAR.ui.removeNode(plus);
						}
					}
				}
				TH.topOpts = topOpts;
				TH.addSlideAppMenu();
				TH.updateModuleMenu();
			}
			TH.initTopSearch();
		},
		updateTopMenu: function() {
			var found = TH.findGroupTab();
			TH.updateCurrentTab(found && found.index);
		},
		updateModuleMenu: function() {
			var found = TH.findGroupTab(),
				mods = found && TH.menuStructure.grouped[found.index].modules,
				outer = $('sidebar-modules');
			TH.displayModuleMenu(mods, found, outer);
		},
		displayModuleMenu: function(mods, found, outer) {
			if(TH.enable_module_sidebar) {
				var opts;
				if(found) {
					opts = {keys: [], values: []};
					for(var i = 0; i < mods.length; i++) {
						opts.keys.push(mods[i].id || mods[i].module);
						opts.values.push(mods[i]);
					}
				}
				TH.renderSidebarMenu(outer, opts, 'modules', null, opts && opts.keys[found.active_index]);
			}
		},
		addSlideAppMenu: function() {
			var toggle = Wrap('tbar-mobile');
			if(! toggle) return;
			toggle.on('click', function() { TH.toggleSlideMenu('main'); return false; });
			TH.sideMask = makeElt('div.appside-menu-mask').appendTo(document.body)
				.on('click', function() { TH.toggleSlideMenu(false); });
		},
		renderAppSearch: function(side, target) {
			side = !!side;
			var searcher = new App.ui.SearchSelect('app-search',
				{flat: side,
				 search_title: app_string('LBL_SEARCH_QUICK_VIEW'),
				 ext_class: side ? 'card-outer appside-menu-panel' : 'popup-global',
				 disable_auto_select: true,
				 destroy_on_close: true,
				 dynamic_module: true,
				 global_search: true,
				 icon_key: 'icon',
				 show_checks: !side,
				 search_clear: false,
				 allow_popup: true,
				 search_blank: true,
				 matching_only: true,
				 modules: SUGAR.session.search_modules
				 });
			if(! side) {
				searcher.target = target;
				searcher.target_offset = {top: -2, outer_margin: 0};
				searcher.target_active_class = 'active';
			}
			searcher.onchange = function(k, v, mod, keepOpen, evt) {
				if(! v) return;
				if(v.query_source) mod = v.query_source;
				var url = 'index.php?module=' + encodeURIComponent(mod) + '&action=DetailView&record=' + encodeURIComponent(k);
				SUGAR.util.navigateTo(url, evt, {delay: true});
			}
			searcher.onaccept = searcher.onpopup = function(val, evt) {
				var q = trim(this.getQuery()),
					m = this.getModule(),
					url;
				if(m)
					url = 'index.php?module=' + encodeURIComponent(m) + '&action=ListView&layout=Browse&query=true&filter_text=' + encodeURIComponent(q);
				else
					url = 'index.php?module=Home&action=UnifiedSearch&query_string=' + encodeURIComponent(q);
				SUGAR.util.navigateTo(url, evt, {delay: true});
				this.close();
				return true;
			}
			searcher.setup();
			searcher.render();
			searcher.showDefault();
			return searcher;
		},
		toggleSlideMenu: function(mode, keepOpen) {
			var state, $body = Wrap(document.body);
			if(mode) {
				if(mode !== 'search') mode = 'main';
				var open = $body.hasClass('appside-open-' + mode);
				state = keepOpen || ! open;
			} else {
				state = false;
			}
			$body.toggleClass('appside-open', state);
			$body.toggleClass('appside-open-main', mode === 'main' && state);
			$body.toggleClass('appside-open-search', mode === 'search' && state);
			if(state) {
				SUGAR.popups.hidePopup();
				var menu = Wrap('app-side');
				menu.cssSize('top', document.body.scrollTop);
				if(mode === 'search') {
					var inner = menu.findFirst('.appside-search');
					TH.appMenuSearch = inner.ext(0) || makeElt('div.appside-search').appendTo(menu);
					var searcher = TH.renderAppSearch(true);
					TH.appMenuSearch.empty().append(searcher.elt);
					requestAnimationFrame(function() { searcher.focus(); });
				}
				else if(! TH.appMenuMain) {
					var inner = menu.findFirst('.appside-main');
					inner = inner.ext(0) || makeElt('div.appside-main').appendTo(menu);
					var mainInput = new SUGAR.ui.SelectList('app-menu-main', {flat: true, expand: true, ext_class: 'appside-menu', options: TH.topOpts});
					mainInput.onchange = function() {
						var idx = this.getSelectedIndex(), val = this.getSelectedValue();
						var opts = deep_clone(TH.groupOptions[idx]);
						TH.showAltSlideMenu(val, opts);
						this.setSelectedKey(null, true);
					}
					inner.append(mainInput.render());
					mainInput.setup();
					TH.appMenuMain = inner;
					TH.appMenu = mainInput;
					var scOuter = makeElt('div.app-sc-outer');
					var scInput = new SUGAR.ui.SelectList('app-menu-sc', {flat: true, ext_class: 'appside-menu', icon_key: 'icon'});
					scInput.onchange = function(old, evt) {
						var v = this.getSelectedValue();
						SUGAR.util.navigateTo(v.url, evt, {delay: true, async: get_default(v.async, true)});
						this.setSelectedKey(null, true);
					}
					scOuter.append(makeElt('label.appside-menu-label', app_string('LBL_SHORTCUTS')));
					scOuter.append(scInput.render());
					inner.append(scOuter);
					scInput.setup();
					TH.appScOuter = scOuter;
					TH.appScInput = scInput;
					var alt = menu.findFirst('.appside-alt');
					alt = alt.ext(0) || makeElt('div.appside-alt:hidden').appendTo(menu);
					var altReturn = makeElt('a.appside-menu-link.tools-row', {href: '#'})
						.on('click', function() { TH.toggleSlideMenu('main', true); return false; })
						.append(makeElt('span.no-flex').append(makeElt('span.uii.uii-previous')))
						.append(makeElt('span', nbsp(), app_string('LBL_BACK')))
						.appendTo(alt);
					var altInput = new SUGAR.ui.SelectList('app-menu-alt', {flat: true, ext_class: 'appside-menu', icon_key: 'icon'});
					altInput.onchange = scInput.onchange;
					alt.append(TH.altTitle = makeElt('label.appside-menu-label'));
					alt.append(altInput.render());
					altInput.setup();
					TH.appMenuAlt = alt;
					TH.altMenu = altInput;
				}
				if(TH.appMenuSearch) TH.appMenuSearch.toggle(mode === 'search');
				if(TH.appMenuMain) {
					TH.appMenuAlt.hide();
					if(mode === 'search') {
						TH.appMenuMain.hide();
					} else {
						TH.appScInput.setOptions((TH.page_info || {}).shortcuts);
						TH.appScOuter.toggle(TH.appScInput.getOptions().getCount());
						TH.appMenuMain.show();
					}
				}
			}
		},
		showAltSlideMenu: function(label, opts) {
			var menu = Wrap('app-side');
			TH.appMenuMain.hide();
			TH.altTitle.text(label);
			TH.altMenu.setOptions(opts);
			//TH.altMenu.renderOptions();
			TH.appMenuAlt.show();
		},
		addPopupAppMenu: function() {
			var nextMenu = new SUGAR.ui.MenuSource(
				'mobile-next-menu', {
					elt: $('tbar-mobile'), icon_key: 'icon',
					width: 200, ext_class: 'popup-global siteMenu',
					target_offset: {align: 'center', outer_margin: 0}}, null, 'system-toolbar');
				topOpts = TH.topOpts;
			nextMenu.initMenu();
			topOpts.keys.push('shortcuts');
			topOpts.values.push(app_string('LBL_SHORTCUTS'));
			m = SUGAR.ui.registerMenuSource(
				$('tbar-mobile'), {
					id: 'mobile-menu', options: topOpts, hover_show: true, expand: true,
					delay: 250, width: 200, ext_class: 'popup-global siteMenu',
					target_offset: {align: 'center', outer_margin: 0}}, null, 'system-toolbar');
			m.setup();
			m.onchange = function(idx, val) {
				var opts;
				if(typeof(idx) == 'number') {
					opts = deep_clone(TH.groupOptions[idx]);
					opts.keys.unshift('_');
					opts.values.unshift({_header: true, label: val});
				}
				else if(idx === 'shortcuts') {
					var opts = (TH.page_info || {}).shortcuts;
					opts.keys.unshift('_');
					opts.values['_'] = {_header: true, label: app_string('LBL_SHORTCUTS')};
				}
				if(opts) {
					nextMenu.menu.setOptions(opts);
					nextMenu.showPopup();
				}
			}
		},
		initTopSearch: function() {
			var srch = Wrap('tbar-search');
			if(srch) srch.on('click', TH.showTopSearch, {scope: TH});
		},
		showTopSearch: function(evt) {
			evt.stopEvent();
			if(window.innerWidth < TH.page_breakpts[2].width && $('app-side')) {
				TH.toggleSlideMenu('search');
				return;
			}
			if(TH.appSearcher) {
				TH.appSearcher.close();
				return;
			}
			var searcher = TH.renderAppSearch(false, evt.currentTarget);
			searcher.onshow = function() {
				TH.toggleSlideMenu(false);
				searcher.focus();
			}
			searcher.onhide = function() {
				TH.appSearcher = null;
			}
			TH.appSearcher = searcher;
		},
		toggleTextures: function() {
			var body = document.body;
			var cls = 'textured';
			if(SUGAR.ui.hasClass(body, cls)) {
				SUGAR.ui.removeClass(body, cls);
				Set_Cookie(''+SUGAR.session.theme+'_textured', 'false', 365, '/','','');
				return false;
			} else {
				SUGAR.ui.addClass(body, cls);
				Set_Cookie(''+SUGAR.session.theme+'_textured', 'true', 365, '/','','');
				return true;
			}
		},
		detectFullScreenSupport: function() {
			var fs = {hooks: []};
			if(isset(document.fullScreen)) {
				fs.prefix = '';
				fs.property = 'fullScreen';
			} else if(isset(document.mozFullScreen)) {
				fs.prefix = 'moz';
				fs.property = 'mozFullScreen';
			} else if(isset(document.webkitIsFullScreen)) {
				fs.prefix = 'webkit';
				fs.property = 'webkitIsFullScreen';
			}
			if(fs.property) {
				fs.request = fs.prefix + (fs.prefix ? 'R' : 'r') + 'equestFullScreen';
				if(document[fs.request + 'WithKeys']) fs.request += 'WithKeys';
				fs.cancel = fs.prefix + (fs.prefix ? 'C' : 'c') + 'ancelFullScreen';
				fs.event = fs.prefix + 'fullscreenchange';
			}
			this.fullscreen = fs;
		},
		getFullScreenState: function() {
			if(this.fullscreen.property) return document[this.fullscreen.property];
		},
		toggleFullScreen: function(elt) {
			var state = this.getFullScreenState(), method;
			if(isset(state)) {
				if(state) return document[this.fullscreen.cancel]();
				else {
					elt = ($(elt) || document.documentElement), method = this.fullscreen.request;
					if(Element.ALLOW_KEYBOARD_INPUT) return elt[method](Element.ALLOW_KEYBOARD_INPUT);
					else return elt[method]();
				}
			}
		},
		addListeners: function() {
			return; // DISABLED (loses state after clicking links)
			/*var button = $('fullscreen-button'), state;
			if(TH.fullscreen.event) {
				toggleDisplay(button, true);
				state = TH.getFullScreenState();
				SUGAR.ui.addRemoveClass(button, 'active', state);
				document.addEventListener(TH.fullscreen.event, function(evt) {
					state = TH.getFullScreenState();
					SUGAR.ui.addRemoveClass(button, 'active', state);
					for(var i = 0; i < TH.fullscreen.hooks.length; i++)
						TH.fullscreen.hooks[i](evt, state);
				}, false);
			}*/
		},
		addFullScreenHandler: function(evt) {
			TH.fullscreen.hooks.push(evt);
		},
		setMainTitle: function(title, params) {
			var t = $('main-title-text'), name = $('main-title-module'), help = $('module-help'), mod, icon;
			if(! isset(title)) title = '';
			if(params) {
				if(params.module) {
					icon = params.icon || 'input-icon theme-icon module-' + params.module;
					icon = '<div class="' + icon + '"></div>';
					mod = params.module_title || module_name(params.module);
					if(mod && params.module_link) {
						mod = makeElt('a', {href: params.module_link}).html(mod);
						mod.on('click', SUGAR.util.handleLink);
					} else {
						mod = makeElt('span').html(mod);
					}
					mod = [makeElt('span.module-icon').html(icon).append(nbsp()), mod];
					if(help) help.onclick = function() { return SUGAR.util.showHelp(params.module); }
				} else if(help) {
					help.onclick = function() { return SUGAR.util.showHelp(); }
				}
			}
			if(name) {
				if(! name.firstChild) createElement2('span', null, null, name);
				Wrap(name.firstChild).content(mod);
			}
			if(t) t.innerHTML = title;
			SUGAR.ui.addRemoveClass(name, 'visible', !! mod);
			SUGAR.ui.addRemoveClass(name, 'with-sep', (title && mod && ! params.title_merge));
			SUGAR.ui.removeClass(document.body, 'no-title');
		},
		hideMainTitle: function() {
			SUGAR.ui.addClass(document.body, 'no-title');
		},
		// v8 themes
		setPageInfo: function(params) {
			if(! params) params = {};
			this.page_info = params;
		},
		setToolsMenu: function(opts, form) {
			var b = TH.tools_btn;
			if(! b)
				b = TH.tools_btn = SUGAR.ui.registerMenuSource('module-tools', {icon_key: 'icon', label_key: 'label'}, form);
			if(opts && b.menu)
				b.menu.setOptions(opts);
			b.setDisabled(! opts);
		},
		initPage: function(params) {
			var pg = this.page_info, stats;
			if(! pg || pg.loaded) pg = {};
			if(params) extendObject(pg, params);
			this.page_info = pg;
			if(pg.state) {
				if(pg.state.query)
					SUGAR.session.loadState(pg.state.query);
				if(pg.state.context == 'detail' && pg.state.show_list)
					this.initDetailSidebar(pg.state.module, pg.state.record);
				if(SUGAR.session.track_page_stats) {
					stats = SUGAR.session.getPageStats(pg.state, pg.stats);
					if(stats) {
						setTimeout(function() {
							SUGAR.conn.asyncRequest('stats.php', {postData: stats, status_msg: null});
						}, 2000);
					}
				}
			}
			if((! pg.state || ! pg.state.query) && SUGAR.session.loadingQuery) {
				SUGAR.session.loadState(SUGAR.session.loadingQuery);
			}
			this.setBodyClass(pg.state);
			if(! pg.no_decoration) {
				this.setMainTitle(pg.title, pg);
				this.setToolsMenu(pg.tools, pg.tools_form);
			} else
				this.hideMainTitle();
			this.renderSidebar();
			this.updateTopMenu();
			for(var i = 0; i < TH.init_page_hooks.length; i++)
				TH.init_page_hooks[i](pg);
			TH.updateFavicon(pg.favicon_href, pg.favicon_hr_href);
			if(params.custom_init) {
				var cb = resolveVar(params.custom_init);
				if(cb) cb.apply(cb, params);
			}
			App.dom.extNode(window).emit('page-init', pg, params);
		},
		updateFavicon: function(icon, hires) {
			if(! icon) return;
			if(! hires) hires = icon;
			var links = Wrap(document.head).find('link'),
				rel, sizes;
			for(var i = 0; i < links.length; i++) {
				rel = links[i].getAttribute('rel');
				if(rel === 'shortcut icon' || rel === 'icon') {
					sizes = links[i].getAttribute('sizes');
					links[i].setAttribute('href', sizes === '32x32' ? hires : icon);
				}
			}
		},
		setBodyClass: function(state) {
			var bd = document.body, c, cs = [], i;
			if(! bd) return;
			for(i = 0; i < bd.classList.length; i++) {
				c = bd.classList[i];
				if(! c.match(/^(mod|ctx)-/))
					cs.push(c);
			}
			if(state) {
				if(state.module)
					cs.push('mod-' + state.module);
				if(state.context)
					cs.push('ctx-' + state.context);
			}
			bd.className = cs.join(' ');
		},
		onInitPage: function(f) {
			this.init_page_hooks.push(f);
		},
		initSidebar: function(c) {
			var out = $('outer-body'),
				hide = Get_Cookie(''+SUGAR.session.theme+'_hide_sidebar'),
				elt = SUGAR.dom.elementOrId('left-sidebar');
			if(elt && elt.hasClass('inited')) return;
			if(! elt || (! out && ! $('all-page'))) {
				if((c||0) < 100)
					setTimeout(function() { TH.initSidebar(c+1); }, 50);
				return;
			}
			if(out && ! TH.sidebarRendered) {
				// guess sidebar visiblity before page finishes rendering
				TH.toggleSidebar((window.innerWidth >= TH.page_breakpts[2].width && !hide) ? true : false);
			}
			if(TH.page_info)
				elt.addClass('inited');
			TH.sidebarInited = true;
			elt.on(SUGAR.ui.getTransitionProperty + 'end', SUGAR.ui.layoutUpdated);
		},
		toggleSidebar: function(state, by_user) {
			var outer = $('outer-body'),
				button = $('sidebar-toggle'),
				open = SUGAR.ui.hasClass(outer, 'sidebar-open'),
				pg = TH.page_info,
				wide = pg && (pg.sidebar_list_wide || pg.sidebar_wide);
			if(isset(state)) open = !state;
			if(outer) {
				SUGAR.ui.addRemoveClass(outer, 'sidebar-open', !open);
				SUGAR.ui.addRemoveClass(outer, 'sidebar-wide', !!(!open && wide));
			}
			if(button)
				SUGAR.ui.addRemoveClass(button, 'checked', !open);
			if(by_user)
				Set_Cookie(''+SUGAR.session.theme+'_hide_sidebar', open?1:'', 365, '/', '', '');
			SUGAR.ui.layoutUpdated();
			return open;
		},
		resizeToggleSidebar: function(state) {
			if(state && TH.page_info && TH.page_info.hide_side_menu) return;
			TH.toggleSidebar(state);
		},
		sidebarOpen: function() {
			return SUGAR.ui.hasClass($('outer-body'), 'sidebar-open');
		},
		saveDetailSidebar: function(list_state, module, record, head, body) {
			if(list_state && list_state.params)
				this.sidebar_list = {module: module, record: record, state: list_state, head: head, body: body};
			else
				this.sidebar_list = null;
		},
		expandDetailSidebar: function(state, record) {
			var body = $('sidebar-list-body'), pg = TH.page_info, srch;
			if(state && body && record && pg) {
				state.params.list_context = 'sidebar';
				state.params.list_current_id = record;
				state.params.initial_load = 1;
				SUGAR.conn.asyncRequest('async.php', {postData: state.params},
					function(data) {
						SUGAR.util.loadContent(body, data.responseText);
						pg.sidebar_list_wide = true;
						TH.toggleSidebar(true);
						if( (srch = $('sidebar-list-search')) && $('sidebar-list-filter')) {
							srch.onclick = function(e) { e.stopPropagation(); return TH.toggleSidebarFilter(); }
							toggleDisplay(srch, true);
						}
					}
				);
			}
		},
		initDetailSidebar: function(module, record) {
			var list = this.sidebar_list, head, c, pg = this.page_info, state;
			if(list && (! list.state || list.module != module || list.record != record)) list = null;
			if(pg && ! TH.disable_detail_sidebar) {
				if(list)
					state = list.state;
				else
					state = {params: {module: module, action: 'ListView'}};
				if(list && list.head && list.body) {
					c = [list.head, list.body];
					pg.sidebar_list_wide = true;
				} else {
					c = [
						createElement2('div', {id: 'sidebar-list-head', className: 'card-header panel-header sectionClosed'},
							createElement2('div', {className: 'tools-row'}, [
								createElement2('h5', {id: 'sidebar-list-title', className: 'no-margin'}, app_string('LBL_LIST_RESULTS')),
								createElement2('span', {id: 'sidebar-list-search', className: 'uii uii-search uii-lg active-icon', style: 'display: none'}),
								createElement2('span', {className: 'no-flex rbullet rbullet-inline'}, '\u00a0')
							])),
						createElement2('div', {id: 'sidebar-list-body', className: 'card-body panel-body', style: 'display:none'})];
				}
				if(c) {
					c[0].onclick = function() {
						TH.toggleSidebarList(state, module, record); return false;
					}
					pg.sidebar_list = c;
				}
			}
		},
		toggleSidebarList: function(state, module, record) {
			var head = $('sidebar-list-head'), body = $('sidebar-list-body'), show = (body && body.style.display == 'none'), srch;
			if(show) {
				SUGAR.ui.clearChildNodes(body);
				body.appendChild(createElement2('div', {className: 'card-body text-center'}, SUGAR.util.waitIcon()));
				this.expandDetailSidebar(state, record);
			} else {
				this.page_info.sidebar_list_wide = false;
				TH.toggleSidebar(true);
				toggleDisplay('sidebar-list-search', false);
			}
			SUGAR.ui.setDisplayState(body, show);
			SUGAR.ui.addRemoveClass(head, 'sectionOpen', show);
			SUGAR.ui.addRemoveClass(head, 'sectionClosed', !show);
		},
		toggleSidebarFilter: function(show) {
			SUGAR.ui.setDisplayState('sidebar-list-filter', show);
		},
		setSidebarContent: function(content) {
			if(this.page_info) this.page_info.sidebar_content = content;
		},
		renderSidebar: function() {
			var side = SUGAR.dom.elementOrId('left-sidebar'),
				panel_cls = 'panel-outer panel-border panel-default sidebar-panel',
				inner, c, c2, pg = TH.page_info || {}, hide, minw, main;
			if(side) {
				if(TH.sidebarInited)
					side.addClass('inited');
				side.empty();
				inner = createElement2('div', {id: 'sidebar-inner', className: 'sidebar-inner'}, null, side.get());
				c = createElement2('div', {id: 'sidebar-content', className: panel_cls}, null, inner);
				c2 = createElement2('div', {id: 'sidebar-list', className: panel_cls}, null, inner);
				c3 = createElement2('div', {id: 'sidebar-modules', className: panel_cls + ' hidden'}, null, inner);
				if(pg.sidebar_content)
					SUGAR.util.loadContent(c, pg.sidebar_content);
				else
					SUGAR.ui.addClass(c, 'hidden');
				if(pg.sidebar_list)
					SUGAR.util.loadContent(c2, pg.sidebar_list);
				else
					SUGAR.ui.addClass(c2, 'hidden');
				TH.renderSidebarMenu(inner, pg.shortcuts, 'shortcuts', app_string('LBL_SHORTCUTS'));
				TH.renderSidebarMenu(inner, pg.recent, 'recent', app_string('LBL_LAST_VIEWED'));
			}
			if(pg.no_decoration || Get_Cookie(''+SUGAR.session.theme+'_hide_sidebar') || pg.hide_side_menu) hide = true;
			main = $('content-main');
			if(pg.state && pg.state.context == 'dashboard') minw = TH.page_breakpts[3].width;
			else minw = TH.page_breakpts[2].width;
			if(main && main.offsetWidth) minw = Math.max(minw, main.offsetWidth+10);
			if(window.innerWidth && window.innerWidth < minw) hide = true;
			TH.sidebarRendered = true;
			TH.updateModuleMenu();
			TH.toggleSidebar(! hide);
		},
		renderSidebarMenu: function(outer, opts, id, title, current) {
			var elt = null;
			if(SUGAR.ui.hasClass(outer, 'sidebar-panel')) {
				elt = outer;
				outer = null;
			}
			if(opts && opts.keys && opts.keys.length) {
				if(! elt) {
					elt = createElement2('div', {id: 'sidebar-'+id, className: 'panel-outer panel-border panel-default sidebar-panel'});
					outer.appendChild(elt);
				}
				if(title) {
					createElement2('div', {className: 'card-header panel-header'},
						createElement2('h5', {className: 'no-margin'}, title), elt);
				}
				SUGAR.ui.registerSelectOptions('page-'+id, opts);
				var sel = new SUGAR.ui.SelectList('page-'+id,
					{options: 'page-'+id, icon_key: 'icon', no_scroll: 1, require_focus: 0, flat: 1, className: 'select-menu flat-top sidebar-menu'});
				if(TH.enable_module_sidebar)
					sel.options_active = true;
				sel.onchange = function(old, evt) {
					var v = this.getSelectedValue();
					SUGAR.util.navigateTo(v.url, evt, {delay: true, async: get_default(v.async, true)});
					this.setSelectedKey(null, true);
				}
				if(isset(current))
					sel.setSelectedKey(current, true);
				elt.appendChild(sel.render());
				sel.setup();
				SUGAR.ui.removeClass(elt, 'hidden');
				return sel;
			} else {
				SUGAR.ui.addClass(elt, 'hidden');
			}
		},
		version: function(chk) {
			var v = SUGAR.session.layout_version;
			if(isset(chk)) return v >= chk;
			return v;
		},
		detectFeatures: function() {
			this.detectFullScreenSupport();
			var e = createElement2('div');
			e.style.position = 'sticky';
			if(e.style.position.indexOf('sticky') >= 0) {
				SUGAR.ui.addClass(document.body, 'sticky');
				this.sticky = true;
			}
		},
		topMenuSize: function(modules) {
			return null;
			/*
			var sz = 0, menu = $('tab-bar');
			if($('tbarDiv') && ! this.sticky)
				sz += 32;  // fixed application bar
			if(modules && menu && this.topMenuFloat() && ! this.sticky)
				sz += 52;  // fixed module bar
			return sz;*/
		},
		topMenuFloat: function() {
			return document.body && SUGAR.ui.hasClass(document.body, 'float-menu');
		}
	};
});


SUGAR.module('modelCache').init(function() {
	var cache = {};
	return {
		resetCache: function() {
			cache = {};
		},
		addModelData: function(name, data) {
			cache[name] = new SUGAR.ui.SelectOptions(data);
		},
		getModelData: function(name) {
			return cache[name];
		},
		getModelKeys: function() {
			var ret = [];
			for(var k in cache)
				ret.push(k);
			return ret;
		}
	}
});


SUGAR.module('session').init(function(App) {
	var SESS = this;

	this.onInit(function() {
		this.update(window._session_init);
		this.initLocation = location.href;
		window.addEventListener('popstate', this.handleRestoreState.bind(this));
	});

	return {
		update: function(vars) {
			if(isObject(vars)) {
				for(var i in vars)
					this[i] = vars[i];
			}
		},
		getState: function() {
			return this.currentState;
		},
		setLoadingState: function(context, vars, push, query) {
			this.loadingState = {context: context, vars: vars, push: push};
			this.loadingQuery = query;
		},
		loadState: function(query) {
			var st = this.loadingState || {};
			this.pushState(query, st.context, st.vars, ! st.push);
		},
		clearLoadingState: function() {
			this.loadingState = null;
			this.loadingQuery = null;
		},
		pushState: function(query, context, vars, replace) {
			if(! isdef(context)) context = null;
			if(! isdef(vars)) vars = null;
			this.currentState = {query: query, context: context, vars: vars};
			if(! this.initialState)
				this.initialState = this.currentState;
			if(! history.pushState) return; // html5 required
			history[replace ? 'replaceState' : 'pushState'](
				this.currentState,
				null, '?'+query);
			this.clearLoadingState();
		},
		replaceState: function(query, context, vars) {
			this.pushState(query, context, vars, true);
		},
		handleRestoreState: function(evt) {
			var s = evt.state, m,
				initial = ! ('state' in history) && this.initLocation == location.href;
			// ignore when the browser is trying to load a cached state on the login page
			if(! SUGAR.session.user_id) return;
			if(isset(s)) {
				if( (m = SUGAR.util.checkDirty()) && ! confirm(m))
					return false;
				this.restoreState(s, initial);
			}
		},
		restoreState: function(s, initial) {
			if(! s) return;
			if(initial)
				s = this.initialState;
			if(this.currentState && this.compareStates(this.currentState, s))
				return;
			if(s.context == 'sListView' && s.vars)
				sListView[s.vars.method](s.vars.id, s.vars.params, s.vars.ext);
			else
				SUGAR.util.fetchContentMain('async.php?' + s.query, {scrollTo: false});
		},
		compareStates: function(s1, s2) {
			var k;
			if(! isObject(s1) || ! isObject(s2)) return s1 === s2;
			for(k in s1)
				if(s1.hasOwnProperty(k) && ! this.compareStates(s1[k], s2[k]))
					return false;
			for(k in s2)
				if(s2.hasOwnProperty(k) && ! this.compareStates(s1[k], s2[k]))
					return false;
			return true;
		},
		setLoadingRequest: function(req) {
			this.loadingRequest = req;
		},
		getPageStats: function(state, stats) {
			var req = this.loadingRequest, time, async, ret;
			if(req) {
				time = req.openTime;
				async = true;
			}
			else if(window.performance && ! this.timeSeen) {
				time = window.performance.timing.responseEnd - window.performance.timing.connectStart;
				this.timeSeen = true;
			}
			if(time) {
				ret = {duration: time, async: async};
				if(stats) for(k in stats)
					ret[k] = stats[k];
				if(state) {
					ret.context = state.context;
					ret.module = state.module;
					ret.record = state.record;
					ret.layout = state.layout;
					ret.user_id = SUGAR.session.user_id;
				}
			}
			return ret;
		}
	};
});


// usually attached to an input field and called as a method
function validateNumberInput(field, params) {
	if(! isset(field))
		field = this;
	if(! isset(params)) {
		if(isset(field.numFormat))
			params = field.numFormat;
		else
			params = field;
	}
	else {
		if(field.numFormat) {
			for(var idx in field.numFormat)
				if(! isset(params[idx])) params[idx] = field.numFormat[idx];
		}
	}
	var numval = validateNumber(field.value, params);
	field.value = params.noFormat ? numval : stdFormatNumber(numval);
	return parseFloat(numval);
}
function validateNumber(input, params) {
	if(! params) params = {};
	var numval, decimals = get_default(params.decimalPlaces, 2), user_seps = SUGAR.session.separators;
	if(typeof(input) == 'number')
		numval = input;
	else if(typeof(input) == 'string') {
		if(params.noFormat) {
			numval = parseFloat(input);
		} else {
			var seps = RegExp.escape(''+user_seps.thousands+user_seps.decimal);
			if(! isset(user_seps.match_pattern))
				user_seps.match_pattern = new RegExp('([+\-]?[0-9'+seps+']*[0-9][0-9'+seps+']*)', 'gi');
			var match = input.match(user_seps.match_pattern);
			if(match)
				numval = parseFloat(stdUnformatNumber(match[0]));
		}
	}
	if(isset(numval) && ! isNaN(numval)) {
		if(! isset(params.minimumDecimals))
			numval = numval.toFixed(decimals);
		else if(isNumeric(params.minimumDecimals)) {
			numval = restrictDecimals(numval, params.minimumDecimals, decimals);
		}
		else if(params.minimumDecimals && params.decimalPlaces)
			numval = restrictDecimals(numval, 0, params.decimalPlaces);
		if(isset(params.restrictPositive) && numval < 0)
			numval = -numval;
	}
	else if(isset(params.defaultNumValue))
		numval = params.defaultNumValue;
	else if(! isset(params.blankAllowed)) {
		if(! isset(params.minimumDecimals))
			numval = '0.00';
		else if(isNumeric(params.minimumDecimals))
			numval = '0'.toFixed(params.minimumDecimals);
		else
			numval = '0';
	}
	else
		numval = '';
	return numval;
}
function countDecimals(numval) {
	var s = numval.toString().split('.');
	if(s.length == 1) return 0;
	return s[1].length;
}
function restrictDecimals(numval, minDecs, maxDecs) {
	if(! isset(minDecs)) minDecs = 0;
	if(! isset(maxDecs)) maxDecs = 5;
	if(! maxDecs) return Math.round(numval).toString();
	var s = numval.toString().split('.');
	if(s.length == 1) s.push('');
	if(s[1].length > maxDecs) {
		s[1] = Math.round(parseInt(s[1], 10) / Math.pow(10, s[1].length - maxDecs)).toString();
		while(s[1].length < maxDecs) s[1] = '0' + s[1];
	}
	// rounded up to 1.(0)
	if (s[1].length > maxDecs) {
		s[0] = '' + (parseInt(s[0], 10) + 1);
		s[1] = s[1].substring(1);
	}
	s[1] = s[1].replace(/0+$/, '');
	if(s[1].length || minDecs) {
		while(s[1].length < minDecs)
			s[1] += '0';
		numval = s[0] + '.' + s[1];
	} else
		numval = s[0];
	return numval;
}
function stdFormatNumber(val, round, precision) {
	var seps = SUGAR.session.separators;
	if(! isset(val))  val = '';
	return formatNumber(val, seps.thousands, seps.decimal, round, precision);
}
function stdUnformatNumber(val) {
	var seps = SUGAR.session.separators;
	if(! isset(val))  val = '';
	return unformatNumber(val, seps.thousands, seps.decimal);
}
function adjustExchangeRate(old_rate, new_rate, fields, decimals) {
	for(var i = 0; i < fields.length; i++) {
		var value = 0;
		if(isset(fields[i].numvalue)) // used by Quotes
			value = fields[i].numvalue;
		else
			value = stdUnformatNumber(fields[i].value);
		value = ConvertFromDollar(ConvertToDollar(value, old_rate), new_rate).toFixed(decimals);
		fields[i].numvalue = value;
		fields[i].value = stdFormatNumber(value, decimals, decimals);
	}
}
function stdConvertRate(id, fields) {
	var decimals = CurrencyDecimalPlaces(id);
	adjustExchangeRate(lastRate, ConversionRates[id], fields, decimals);
	lastRate = ConversionRates[id];
}

function mod_string(idx, mod) {
	return SUGAR.language.getString(idx, mod);
}
function mod_list_strings(idx, mod) {
	return SUGAR.language.getList(idx, mod);
}
function app_string(idx) {
	return SUGAR.language.getString(idx, 'app');
}
function app_list_strings(idx) {
	return SUGAR.language.get('app_list_strings', idx);
}
function module_name(module, singular) {
	var lst = app_list_strings('moduleList' + (singular ? 'Singular' : ''));
	return get_default(lst[module], module);
}

function sepLabel(text) {
	var lbl = app_string('LBL_SEPARATOR');
	lbl = lbl.replace(' ', '\u00a0');
	if(text) return lbl;
	return document.createTextNode(lbl);
}
// --
var img_cache = {};
var ImageMeta = {};
function get_icon_image(id) {
	if(id == 'remove') {
		return createElement2('span', {className: 'uii uii-remove'});
	} else if(id == 'insert') {
		return createElement2('span', {className: 'uii uii-add'});
	}
	if(isset(img_cache[id]))
		return img_cache[id].cloneNode(false);
	if(isset(ImageMeta) && isset(ImageMeta[id])) {
		var img = document.createElement('img');
		setAttrs(img, ImageMeta[id]);
		img.border = 0;
		img.style.verticalAlign = 'middle';
		img_cache[id] = img.cloneNode(false);
		return img;
	}
	return document.createTextNode(id);
}
function get_image_url(id) {
	if(isset(ImageMeta[id]))
		return ImageMeta[id].src;
	return 'themes/'+SUGAR.session.theme+'/images/'+id+'.gif';
}

function getTimeFormat(params) {
	if(! params) params = {};
	var fmt = {};
	fmt.print_format = '' + get_default(params.format, SUGAR.session.formats.time);
	fmt.print = function(hr, min) {
		var d = new Date();
		d.setHours(hr);
		d.setMinutes(min);
		return d.print(this.print_format);
	}
	fmt.sep = params.time_separator || SUGAR.session.separators.hour_minute;
	if(fmt.print_format.indexOf('%p') > 0)
		fmt.meridiem = 2;
	else if(fmt.print_format.indexOf('%P') > 0)
		fmt.meridiem = 1;
	else
		fmt.meridiem = 0;
	return fmt;
}

function parseTimeString(str, params) {
	var fmt = getTimeFormat(params);
	var inp = {val: trim(str), format: fmt};
	var merid = inp.val.match(/^(.*?)([ap]m?)$/i);
	if(merid) {
		inp.merid = merid[2][0].toLowerCase();
		inp.no_merid = merid[1];
	} else {
		inp.merid = '';
		inp.no_merid = inp.val;
	}
	var hr = inp.no_merid.match(/^(\d+)(.*?)$/);
	if(hr) {
		if(inp.no_merid == '0') {
			var d = new Date();
			inp.hour = d.getHours();
			inp.minute = d.getMinutes();
		}
		else if(hr[1].length == 4) {
			inp.hour = parseInt(hr[1].substring(0, 2), 10);
			inp.minute = parseInt(hr[1].substring(2), 10);
		} else {
			inp.hour = parseInt(hr[1], 10);
			inp.minute = 0;
		}
		if(inp.merid == 'p' && inp.hour < 12)
			inp.hour += 12;
		else if(inp.merid == 'a' && inp.hour == 12)
			inp.hour -= 12;
		else if(inp.merid != 'a' && fmt.meridiem) {
			if(inp.hour < 8) inp.hour += 12;
		}
		var minsec = hr[2];
		if(minsec) {
			if(minsec.substring(0, fmt.sep.length) == fmt.sep) {
				var mm = minsec.substring(fmt.sep.length).match(/^(\d+)(.*?)$/);
				if(mm) {
					inp.minute = parseInt(mm[1], 10);
					var sec = mm[2];
				}
			} else if(minsec[0] == '.') {
				var mm = minsec.substring(fmt.sep.length).match(/^(\d+)(.*?)$/);
				if(mm) {
					inp.minute = 60 * ('0.'+mm[1]);
					var sec = mm[2];
				}
			}
		}
		inp.display = fmt.print(inp.hour, inp.minute);
		inp.d = new Date();
		inp.d.setHours(inp.hour, inp.minute);
		return inp;
	}
	return false;
}

function validateTimeInput(field, params) {
	if(! isset(field))
		field = this;
	if(! isset(params)) {
		if(isset(field.timeFormat))
			params = field.timeFormat;
		else
			params = field;
	}
	else {
		if(field.timeFormat) {
			for(var idx in field.timeFormat)
				if(! isset(params[idx])) params[idx] = field.timeFormat[idx];
		}
	}

	var ret = parseTimeString(field.value, params);
	if(ret) field.value = inp.display;
	return ret;
}

function getDateFormat(params) {
	if(! params) params = {};
	var fmt = {}, i, pats, p;
	fmt.print_format = '' + get_default(params.format, SUGAR.session.formats.date);
	fmt.makeDate = function(val) {
		var d = new Date(val.year, val.month - 1, val.day);
		return d;
	}
	fmt.print = function(val) {
		if(! val.d) val.d = this.makeDate(val);
		return val.d.print(this.print_format);
	}
	pats = fmt.print_format.match(/%[Ymd]/gi);
	for(i = 0; i < pats.length; i++) {
		switch(pats[i]) {
		case '%Y':
			fmt.year_position = i+1; break;
		case '%m':
			fmt.month_position = i+1; break;
		case '%d':
			fmt.day_position = i+1; break;
		}
	}
	return fmt;
}

function parseDateString(str, params) {
	var fmt = getDateFormat(params),
		inp = {val: trim(str), format: fmt},
		split = inp.val.match(/^(.+)\s+(\d+[:\.hap].*)$/i);
	if(split) {
		inp.val = split[1].trim();
		inp.time = split[2].trim();
	}
	var std = inp.val.match(/^(\d+)\D+(\d+)\D+(\d+)$/),
		retdate = function(d) {
			if(inp.time) {
				inp.time = parseTimeString(inp.time);
				if(inp.time) {
					d.setHours(inp.time.hour, inp.time.minute);
				}
			}
			inp.d = d;
			inp.year = d.getFullYear();
			inp.month = d.getMonth()+1;
			inp.day = d.getDate();
			inp.display = fmt.print(d);
			return inp;
		};
	// handle full three-part date
	if(std) {
		var y, m, d;
		if(std[1] > 1000) {
			// assume year, month, day, ignoring user date format
			y = parseInt(std[1], 10);
			m = parseInt(std[2], 10);
			d = parseInt(std[3], 10);
		}
		else {
			// assume ordering in user date format
			y = parseInt(std[fmt.year_position], 10);
			m = parseInt(std[fmt.month_position], 10);
			d = parseInt(std[fmt.day_position], 10);
		}
		if(! y) y = (new Date().getFullYear());
		else if(y < 100) y += 2000;
		return retdate(new Date(y, m-1, d));
	}

	// handle date offset (in days, months or years)
	var offset = inp.val.match(/^([+-])\s*(\.?\d+|\d+\.\d*)\s*([dmy]?)$/i);
	if(offset) {
		var d = new Date(), offs = parseInt(offset[1] + offset[2], 10), otype = offset[3].toLowerCase();
		if(otype == 'y') {
			d.setFullYear(d.getFullYear() + offs);
		}
		else if(otype == 'm') {
			var y = d.getFullYear(), m = d.getMonth() + 1 + offs;
			while(m <= 0) { y --; m += 12; }
			while(m > 12) { y ++; m -= 12; }
			d = new Date(y, m-1, d.getDate());
			if(! d)
				return false;
		}
		else {
			var s = d.getTime() + (offs * 24 * 3600 * 1000);
			d.setTime(s);
		}
		if(! d)
			return false;
		return retdate(d);
	}

	// handle one or two-part date (year and maybe month assumed)
	var pair = inp.val.match(/^(\d+)(?:\D+(\d+))?$/);
	if(pair) {
		var d = new Date(), p1 = parseInt(pair[1], 10), p2, tmp, y;
		if(isset(pair[2])) {
			p2 = parseInt(pair[2], 10);
			if(fmt.month_position > fmt.day_position) {
				tmp = p1; p1 = p2; p2 = tmp;
			}
			if(p1 > 1000) { y = p1; p1 = p2; p2 = 0; }
			else y = d.getFullYear();
			if(! p1) p1 = d.getMonth()+1;
			if(! p2) p2 = d.getDate();
			d = new Date(y, p1 - 1, p2);
		} else {
			if(p1 > 1000000)
				d.setTime(p1 / 1000); // timestamp, why not
			else if(p1 > 1000)
				d.setFullYear(p1);
			else if(p1)
				d = new Date(d.getFullYear(), d.getMonth(), p1);
			// else: zero-value = current date
		}
		if(! d)
			return false;
		return retdate(d);
	}

	return false;
}

function validateDateInput(field, params) {
	if(! isset(field))
		field = this;
	if(! isset(params)) {
		if(isset(field.dateFormat))
			params = field.dateFormat;
		else
			params = field;
	}
	else {
		if(field.dateFormat) {
			for(var idx in field.dateFormat)
				if(! isset(params[idx])) params[idx] = field.dateFormat[idx];
		}
	}

	var ret = parseDateString(field.value, params);
	if(ret) field.value = ret.display;
	return ret;
}

function formatDuration(minutes, hide_zero) {
	var min = Math.round(minutes);
	var hr = Math.floor(min / 60);
	min = min % 60;
	var ts = SUGAR.session.separators.duration_hr;
	return '' + hr + ((min || ! hide_zero) ? ts + (min < 10 ? '0'+min : min) : '');
}


SUGAR.module('kc').init(function(App) {
	var code = [], idx = 0;
	for(var i = 0; i < 11; i++) code.push('IIKKHJHJed0'.charCodeAt(i)-35);
	this.onDomReady(function() {
		SUGAR.util.addKeyHandler(SUGAR.kc.hit);
	});
	return {
		hit: function(e) {
			var c = e.keyCode;
			if(code[idx] == c) idx++; else idx = 0;
			if(idx == code.length) { SUGAR.kc.hook(); idx = 0; }
			if(idx) return true;
		},
		hook: function() {
			SUGAR.util.addScriptInclude('include/javascript/kc.js?'+(Math.random()*100));
		}
	};
});

// longreach - end added

// TEMP STUBS
addForm = addToValidate = removeFromValidate = check_form = function() {};

