<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


require_once('include/json/jsrd.php');

class IAH_JSRD extends JSRD {

	function load_result(&$spec, &$data) {
		if(! $this->load_schemata($spec))
			return false;
		$this->data = $data;
		return true;
	}

	function to_jsrd_field_type($type) {
		if($type == 'int' || $type == 'integer')
			return 'integer';
		if($type == 'float' || $type == 'double' || $type == 'currency')
			return 'float';
		if($type == 'date' || $type == 'time' || $type == 'datetime')
			return $type;
		if($type == 'bool' || $type == 'boolean')
			return 'boolean';
		return 'string';
	}
	
	function to_jsrd_field_format($type) {
		if($type == 'id' || $type == 'url' || $type == 'currency')
			return $type;
		if($type == 'enum' || $type == 'multienum' || $type == 'status')
			return $type;
		if($type == 'email_address')
			return 'email';
		if($type == 'text')
			return 'multiline';
		return null;
	}
	
	function jsrd_format_field_defs(&$seed, $jsrd_compat=true) {
		if(method_exists($seed, 'getFieldDefinitions'))
			$defs = $seed->getFieldDefinitions();
		else
			$defs = array();
		$ret = array();
		foreach($defs as $f => $def) {
			if(! isset($def['name']))
				continue;
			$fld = IAH_JSRD::from_iah_field_def($seed->module_dir, $def, $jsrd_compat);
			$ret[$fld['name']] = $fld;
		}
		return $ret;
	}
	
	function from_iah_field_def($mod, $def, $jsrd_compat=true) {
		$typ = array_get_default($def, 'type');
		if(isset($def['custom_type']))
			$typ = $def['custom_type'];
		$fld = array(
			'name' => $def['name'],
			'type' => $jsrd_compat ? IAH_JSRD::to_jsrd_field_type($typ) : $typ,
		);
		$format = IAH_JSRD::to_jsrd_field_format($typ);
		if(! is_null($format))
			$fld['format'] = $format;
		if(array_get_default($def, 'required')) {
			if($jsrd_compat)
				$fld['nullable'] = false;
			else
				$fld['required'] = true;
		}
		$lbl = array_get_default($def, 'vname');
		if($lbl)
			$fld['label'] = json_translate($lbl, $mod);
		$len = array_get_default($def, 'len');
		if($typ == 'id' && ! $len)
			$len = 36;
		if(isset($len))
			$fld[$jsrd_compat ? 'maxlen' : 'len'] = (int)$len;
		$custom = array();
		if(isset($def['options'])) {
			if($jsrd_compat)
				$custom['options'] = $def['options'];
			else
				$fld['options'] = json_get_option_list($def['options'], $mod);
		}
		if(count($custom))
			$fld['custom'] = $custom;
		return $fld;
	}
	
	function restored_db_values(&$bean) {
		global $_jsrd_restore_fields;
		if(! isset($_jsrd_restore_fields))
			$_jsrd_restore_fields = array();
		if(! isset($_jsrd_restore_fields[$bean->object_name])) {
			$restore = array();
			foreach($bean->getFieldDefinitions() as $f => $def) {
				$t = array_get_default($def, 'type');
				if($t == 'date' || $t == 'datetime' || $t == 'time' || $t == 'float' || $t == 'currency')
					$restore[$f] = true;
			}
			$_jsrd_restore_fields[$bean->object_name] = $restore;
		}
		if(isset($bean->fetched_row) && is_array($bean->fetched_row)) // set by retrieve
			$orig =& $bean->fetched_row;
		else if(isset($bean->_db_values) && is_array($bean->_db_values)) // set by process_union_list_query
			$orig =& $bean->_db_values;
		else
			return false;
		$ret = array();
		foreach($_jsrd_restore_fields[$bean->object_name] as $f => $v)
			if(isset($orig[$f])) $ret[$f] = $orig[$f];
		return $ret;
	}
	
	function set_bean_value(&$bean) {
		if($this->set_bean_schema($bean)) {
			$sch =& $this->schema[$bean->module_dir];
			$sch->type = 'record';
			$this->data =& $this->extract_bean_fields($sch, $bean);
			return true;
		}
		return false;
	}
	
	function set_bean_schema(&$seed) {
		if(! $this->add_bean_fields($seed)) {
			return false;
		}
		$this->set_root_schema($seed->module_dir);
		return true;
	}
	
	function add_bean_fields(&$seed, $fields=null, $extra=false) {
		if(method_exists($seed, 'getFieldDefinitions'))
			$defs = $seed->getFieldDefinitions();
		else
			$defs = array();
		if(! is_array($fields))
			$fields = array_merge($seed->column_fields, $seed->additional_column_fields);

		$add_fields = array();
		foreach($defs as $f => $def) {
			if(! isset($def['name']) || ! in_array($def['name'], $fields))
				continue;
			$fld = $this->from_iah_field_def($seed->module_dir, $def);
			$add_fields[] = $fld;
		}

		$schema_name = $seed->module_dir;
		if($this->has_schema($schema_name)) {
			foreach($add_fields as $fld)
				$this->schema[$schema_name]->add_field($fld, $extra);
		}
		else {
			$sch = array(
				'name' => $schema_name,
				'type' => 'sequence',
				'element_type' => 'record',
				'fields' => $add_fields);
			if(! $this->add_schema($sch))
				return false;
		}
		return true;
	}
	
	function &extract_bean_fields(&$schema, &$bean) {
		$row = array();
		foreach($schema->standard as $f) {
			$row[$f] = isset($bean->$f) ? $bean->$f : null;
		}
		foreach($schema->extra as $f) {
			if(isset($bean->$f))
				$row[$f] = $bean->$f;
		}
		return $row;
	}
	
	function add_bean_rows(&$list) {
		if(! $this->has_schema())
			return false;
		$sch =& $this->schema[$this->root_schema];
		foreach($list as $bean) {
			$this->data[] =& $this->extract_bean_fields($sch, $bean);
		}
	}

}

?>
