<?php
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/

// Utils related to Firebase Cloud Messaging and Android app notifications.
// We do not use API in Android app, so we need JSON methods.

if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');

define ('APNS_EXPIRATION_TIME', 2592000); //30 * 24 * 3600;

$json_supported_actions['apns_register'] = array(
	'login_required' => true,
);

function json_apns_register() {
	global $current_user;
	$token = $_REQUEST['token'];
	$device_id = $_REQUEST['device_id'];
	$type = 'apns';
	$user_id = AppConfig::current_user_id();
	$expires = gmdate('Y-m-d H:i:s', time() + APNS_EXPIRATION_TIME);

	$result = ListQuery::quick_fetch_key('push_tokens', 'device_id', $device_id);

	if ($result)
		$upd = RowUpdate::for_result($result);
	else
		$upd = RowUpdate::blank_for_model('push_tokens');

	$upd->set('token', $token);
	$upd->set('device_id', $device_id);
	$upd->set('token_type', $type);
	$upd->set('app_name', 'com.1crm.iosnew.-CRM');
	$upd->set('user_id', $user_id);
	$upd->set('expires', $expires);
	$upd->save();

	$ret = [
		'expires' => $expires,
	];
	return json_return_value($ret);
}

// we do not require login because when device fetches
// reminders, the session may have been expired.
// we consider that combination of device ID and token is
// enough to authenticate the user

$json_supported_actions['apns_reminders'] = array(
	'login_required' => false,
);

function json_apns_reminders() {
	$token = $_REQUEST['token'];
	$device_id = $_REQUEST['device_id'];
	$type = 'apns';
	$lq = new ListQuery('push_tokens');
	$lq->addSimpleFilter('device_id', $device_id);
	$lq->addSimpleFilter('token', $token);
	$lq->addSimpleFilter('token_type', 'apns');
	$row = $lq->runQuerySingle();
	if (!$row || $row->failed) {
		$ret = [];
		return json_return_value($ret);
	}
	$user_id = $row->getField('user_id');

	$meta = [
		'Call' => [],
		'Meeting' => [],
		'Task' => ['has_due' => true],
	];

	$data = [];
	foreach ($meta as $model => $m) {
		$lq = new ListQuery($model);
		$lq->addFields(['name', 'date_start', 'reminder_time']);
		$lq->addSimpleFilter('assigned_user_id', $user_id);
		$now = gmdate("Y-m-d H:i:s");
		$then = gmdate("Y-m-d H:i:s", time() + 180 * 24 * 3600);
		$clause = [
			'field' => 'date_start',
			'operator' => 'between',
			'value' => $now,
			'end' => $then,
		];
		$lq->addFilterClause($clause);
		$lq->addSimpleFilter('reminder_time', "CAST(IFNULL(reminder_time, '-1') AS SIGNED) > 0", 'literal');
		$result = $lq->runQuery(0, 200, false, null, 200);
		if (!$result || $result->failed) {
			continue;
		}
		foreach ($result->getRowIndexes() as $i) {
			$row = $result->getRowResult($i);
			$data[] = [
				'name' => $row->getField('name'),
				'date_start' => $row->getField('date_start'),
				'id' => $row->getField('id'),
				'reminder_time' => (int)$row->getField('reminder_time'),
				'type' => $model . 's',
			];
		}
	}
	return json_return_value($data);
}

