<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


$json_supported_actions['login'] = array(
	'jsrd' => 'auto',
	'login_required' => false,
	'schema' => array(
		'input' => array(
			'fields' => array(
				array('name' => 'username', 'nullable' => false),
				'password',
			),
			'extra_fields' => array(
				'language', 'theme', 'remember',
				'login_module', 'login_action', 'login_record',
				'encoded_password', 'mobile', 'captcha',
			),
		),
		'output' => array(
			'fields' => array(
				array('name' => 'result', 'type' => 'string'),
				array('name' => 'json_session_id', 'type' => 'string'),
				array('name' => 'user_id', 'type' => 'string', 'format' => 'id'),
			),
		),
	),
);

$json_supported_actions['captcha'] = array(
	'login_required' => false,
	'schema' => array(
		'input' => array(
		),
		'output' => array(
		),
	),
);

function json_captcha() {
	require_once 'include/Captcha/captcha.php';
	$captcha = new Captcha(AppConfig::captcha_complexity());
	$_SESSION['captcha'] = sha1($captcha->getText());
	header('Content-Type: image/svg+xml');
	echo $captcha->generate();
	exit;
}

function json_login() {
	global $authController, $app_strings;
	$username = array_get_default($_REQUEST, 'username');
	$pass = array_get_default($_REQUEST, 'password');

	$params = array(
		'encoded_password' => array_get_default($_REQUEST, 'encoded_password'),
		'remember' => array_get_default($_REQUEST, 'remember'),
		'language' => array_get_default($_REQUEST, 'language'),
		'theme' =>  array_get_default($_REQUEST, 'theme'),
		'no_persist_theme' =>  array_get_default($_REQUEST, 'no_persist_theme'),
	);

	if (AppConfig::captcha_complexity() >= 0) {
		$params['expected_captcha'] = array_get_default($_SESSION, 'captcha', true);;
		$params['captcha'] = sha1(array_get_default($_REQUEST, 'captcha', ''));
	}

	if(! $username) {
		$ret = array('result' => 'no_username', 'message' => $app_strings['LBL_LOGIN_NO_USERNAME']);
		json_auto_return($ret);
		return;
	}

	$authController->setSettings($_REQUEST);
	$authController->setRedirectParams($_REQUEST);

	// Check If Two-Factor Auth Enabled
	$user_id = $authController->getUserId($username, $pass);

	if ($user_id) {
		require_once('include/MultiFA/MultiFactorAuthLogin.php');
		require_bean('User');
		$user = new User();
		$user->id = $user_id;

		$multiFALogin = new \OneCRM\MultiFA\MultiFactorAuthLogin();

		if ($multiFALogin->setLoginCodeVerification($user, $GLOBALS['log'])) {

			$loginDetails = [
				'username' => $username,
				'password' => $pass,
				'params' => $params,
				'settings' => $authController->getSettings(),
				'redirect_params' => $authController->getRedirectParams()
			];

			$multiFALogin->storeLoginDetails($loginDetails);

			$ret = array('result' => 'show_multi_fa_code');
			json_auto_return($ret);
			return;
		}
	}

	$ret = $authController->getLoginResult($username, $pass, $params);
	json_auto_return($ret);
}

$json_supported_actions['mfa_verify_code'] = array(
	'jsrd' => 'auto',
	'login_required' => false,
	'schema' => array(
		'input' => array(
			'fields' => array(
				array('name' => 'otp', 'nullable' => false),
			)
		),
		'output' => array(
			'fields' => array(
				array('name' => 'result', 'type' => 'string'),
				array('name' => 'json_session_id', 'type' => 'string'),
				array('name' => 'user_id', 'type' => 'string', 'format' => 'id'),
			),
		),
	),
);
function json_mfa_verify_code() {
	global $authController;
	$code = array_get_default($_REQUEST, 'otp');

	require_once('include/MultiFA/MultiFactorAuthLogin.php');
	$multiFALogin = new \OneCRM\MultiFA\MultiFactorAuthLogin();
	$loginDetails = $multiFALogin->getLoginDetails();

	if (empty($loginDetails) || empty($loginDetails['username']) || empty($loginDetails['password'])) {
		$ret = ['result' => 'error'];
		$multiFALogin->hideLoginCodeVerification();
		json_auto_return($ret);
		return;
	}

	$user_id = $authController->getUserId($loginDetails['username'], $loginDetails['password']);

	if (! $user_id) {
		$ret = ['result' => 'error'];
		$multiFALogin->hideLoginCodeVerification();
		json_auto_return($ret);
		return;
	}

	require_bean('User');
	$user = new User();
	$user->id = $user_id;

	$result = $multiFALogin->verifyCode($user, $GLOBALS['log'], $code);

	//if wrong code show error
	if (! $result) {
		$ret = ['result' => 'wrong_code', 'message' => translate('LBL_MULTI_FA_VERIFY_CODE_ERROR', 'Users')];
	//if code correct then authenticate user
	} else {
		$multiFALogin->hideLoginCodeVerification();

		$authController->setSettings($loginDetails['settings']);
		$authController->setRedirectParams($loginDetails['redirect_params']);

		$ret = $authController->getLoginResult(
			$loginDetails['username'],
			$loginDetails['password'],
			$loginDetails['params']
		);
	}

	json_auto_return($ret);
}

$json_supported_actions['check_authenticated'] = array(
	'jsrd' => 'auto',
	'login_required' => false,
	'schema' => array(
		'output' => array(
			'fields' => array(
				array('name' => 'result', 'type' => 'integer'),
			),
		),
	),
);
function json_check_authenticated() {
	global $current_user;
	$ret = array('result' => (empty($current_user->id) ? 0 : 1));
	json_auto_return($ret);
}


$json_supported_actions['logout'] = array(
	'jsrd' => 'auto',
	'login_required' => false,
	'schema' => array(
		'input' => array(
		),
		'output' => array(
			'fields' => array(
				array('name' => 'result', 'type' => 'string'),
			),
		),
	),
);
function json_logout() {
	global $current_user;
	if($current_user && ! empty($current_user->id))
		$current_user->logged_out_session();
	foreach($_SESSION as $key => $val) {
		$_SESSION[$key] = '';
	}
	if(isset($_COOKIE[session_name()])) {
		if(AppConfig::detect_https() == 'https')
	       	setcookie(session_name(), '', time()-42000, '/', AppConfig::request_host_name(), FALSE, TRUE);
	    else
	       	setcookie(session_name(), '', time()-42000, '/', AppConfig::request_host_name(), FALSE, FALSE);
	}
	session_destroy();
	$ret = array('result' => 'ok');
	json_auto_return($ret);
}


$json_supported_actions['switch_user'] = array(
	'admin_only' => true,
	'jsrd' => 'auto',
	'schema' => array(
		'input' => array(
			'fields' => array(
				array('name' => 'user_name', 'nullable' => false)
			),
		),
		'output' => array(
			'fields' => array(
				array('name' => 'result', 'type' => 'string'),
				array('name' => 'user_id', 'type' => 'string', 'format' => 'id'),
			),
		),
	),
);
function json_switch_user() {
	global $current_user;
	if(empty($_REQUEST['user_name'])) {
		return json_bad_request(array('result' => 'missing_user_name'));
	}
	$sudo_user = new User();
	$sudo_id = $sudo_user->retrieve_user_id($_REQUEST['user_name']);
	if(!$sudo_id) {
		return json_bad_request(array('result' => 'invalid_login'));
	}
	else {
		$sudo_user->retrieve($sudo_id);			
		if($sudo_user->status != 'Active') {
			return json_bad_request(array('result' => 'not_active'));
		}
		else {
			$current_user = $sudo_user;
			// we now destroy and recreate the session to prevent user
			// preferences or any other session variables from leaking over
			$sid = session_id();
			$sess_copy_names = array(
				'server_unique_key', 'login_type', 'client_ip_address',
				'authenticated_user_language', 'json_session_language',
			);
			$sess_copy = array();
			foreach($sess_copy_names as $nm) 
				$sess_copy[$nm] = array_get_default($_SESSION, $nm);
			$GLOBALS['json_restore_prefs'] = null;
			session_destroy();
			session_unset();
			session_id($sid);
			AppConfig::session_start(null, true, true);
			$_SESSION['user_id'] = $sudo_id;
			$_SESSION['authenticated_user_id'] = $sudo_id;
			foreach($sess_copy as $nm => $val)
				if(isset($val))
					$_SESSION[$nm] = $val;
			AppConfig::session_init();
		}
	}
	$ret = array('result' => 'ok', 'user_id' => $sudo_id);
	json_auto_return($ret);
}


function json_action_supported($action) {
	global $current_user, $json_supported_actions;
	if(! isset($json_supported_actions[$action]))
		return false;
	$spec = $json_supported_actions[$action];
	// Check if the function is public
	$login_req = array_get_default($spec, 'login_required', true);
	if ($login_req === false) {
		// Public function
		return true;
	}
	if(array_get_default($spec, 'admin_only') && ! is_admin($current_user))
		return false;
	// Determine the login type
	if (array_get_default($_SESSION, 'login_type') === 'portal') {
		// Only allow public & portal functions
		return ((!empty($current_user->id)) && ($login_req === 'portal'));
	}
	return ((!empty($current_user->id)) && ($login_req !== 'portal'));
}

?>
