<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


$json_supported_actions['get_version'] = array(
	'jsrd' => 'explicit',
	'login_required' => false,
	'schema' => array(
		'output' => array(
			'fields' => array(
				array('name' => 'version', 'type' => 'string', 'nullable' => false),
			),
		),
	),
);
function json_get_version() {
	$ret = array('version' => AppConfig::version());
	json_auto_return($ret);
}


$json_supported_actions['get_supported_actions'] = array(
	'jsrd' => 'explicit',
	'login_required' => false,
	'schema' => array(
		'params' => array(
			array('name' => 'module', 'type' => 'string', 'optional' => true),
		),
		'output' => array(
			'type' => 'sequence',
			'fields' => array(
				array('name' => 'name', 'type' => 'string'),
				array('name' => 'jsrd', 'type' => 'boolean'),
			),
		),
	),
);
function json_get_supported_actions() {
	global $json_supported_actions, $output_format;
	$funs = array_filter(array_keys($json_supported_actions), 'json_action_supported');
	if($output_format == 'jsrd') {
		$ret = array();
		foreach($funs as $fn) {
			$info = array('name' => $fn, 'jsrd' => false);
			$spec =& $json_supported_actions[$fn];
			if(! empty($spec['jsrd']))
				$info['jsrd'] = true;
			$ret[] = $info;
		}
		$funs = $ret;
	}
	sort($funs);
	json_auto_return($funs);
}


$json_supported_actions['get_available_languages'] = array(
	'jsrd' => 'explicit',
	'login_required' => false,
	'schema' => array(
		'output' => array(
			'type' => 'sequence',
			'fields' => array(
				array('name' => 'code', 'type' => 'string'),
				array('name' => 'name', 'type' => 'string'),
			),
		),
	),
);
function json_get_available_languages() {
	$langs = AppConfig::get_languages();
	$ret = array();
	foreach($langs as $k => $v)
		$ret[] = array('code' => $k, 'name' => $v);
	json_auto_return($ret);
}


$json_supported_actions['get_user_locale_settings'] = array(
);
function json_get_user_locale_settings() {
	global $locale, $timedate;
	require_once('modules/Currencies/Currency.php');
	$currency = new Currency();
	$currency->retrieve($locale->getPrecedentPreference('currency'));
	$tz = $locale->getPrecedentPreference('timezone');
	$settings = array(
		'date_format' => $locale->getPrecedentPreference('default_date_format'),
		'time_format' => $locale->getPrecedentPreference('default_time_format'),
		'name_format' => $locale->getPrecedentPreference('default_locale_name_format'),
		'number_format' => $locale->getNumberFormat(),
		'address_format' => array(
			'fields' => $locale->getLocaleAddressShortFields(),
			'template' => $locale->getLocaleAddressTemplate(),
		),
		'currency' => array(
			'id' => $currency->id,
			'name' => $currency->name,
			'symbol' => $currency->symbol,
			'iso4217' => $currency->iso4217,
			'decimal_places' => $currency->decimal_places,
			'conversion_rate' => $currency->conversion_rate,
			'decimal_places' => $currency->decimal_places,
			'symbol_place_after' => $currency->symbol_place_after,
			'space_separate' => $locale->getLocaleSpaceSeparateCurrency(),
		),
		'timezone' => array(
			'name' => $tz,
			'offset' => ($tz ? $timedate->adjustmentForUserTimeZone($tz) : 0),
		),
	);
	json_auto_return($settings);
}


$json_supported_actions['retrieve'] = array(
	'jsrd' => 'auto',
	'schema' => array(
		'params' => array(
			array('name' => 'module', 'type' => 'string'),
			array('name' => 'record', 'type' => 'string', 'format' => 'id'),
		),
		'output' => array(
			'type' => 'record',
			'record_type' => 'field_def',
			'record_types' => array(
				//$jsrd_field_def_spec,
			),
		),
	),
);
function json_retrieve() {
	global $output_format;
	if(! empty($_REQUEST['bean_name'])) {
		$model = $_REQUEST['bean_name'];
		$module = AppConfig::module_for_model($model);
	} else {
		$module = $GLOBALS['module'];
		$model = AppConfig::module_primary_bean($module);
	}
	if(! $module || ! $model) {
		return json_bad_request('module_error');
	}
	$record = array_get_default($_REQUEST, 'record');
	
	$result = ListQuery::quick_fetch($model, $record, true, array('acl_filters' => array('view')));
	if($result) {
		require_once('include/layout/FieldFormatter.php');
		$fmt = new FieldFormatter('json');
		$row = $fmt->formatRow($result->fields, $result->row);
	} else {
		return json_forbidden();
	}

	$js_fields_arr = array('fields' => $row);
	json_return_value($js_fields_arr);
}


$json_supported_actions['retrieve_list'] = array(
	'jsrd' => 'auto',
	'schema' => array(
		'params' => array(
			array('name' => 'module', 'type' => 'string'),
			array('name' => 'records', 'type' => 'sequence', 'element_type' => 'string', 'format' => 'id'),
		),
		'output' => array(
			'fields' => array(
				array(
					'name' => 'fields', 'nullable' => false,
					'type' => 'sequence', 'record_type' => 'field_def'
				),
			),
			'record_types' => array(
				//$jsrd_field_def_spec,
			),
		),
	),
);
function json_retrieve_list() {
	global $output_format;
	if(! empty($_REQUEST['bean_name'])) {
		$model = $_REQUEST['bean_name'];
		$module = AppConfig::module_for_model($model);
	} else {
		$module = $GLOBALS['module'];
		$model = AppConfig::module_primary_bean($module);
	}
	if(! $module || ! $model) {
		return json_bad_request('module_error');
	}
	
	$records = array_get_default($_REQUEST, 'records');
	if(! is_array($records) || ! count($records))
		return json_bad_request('missing_records');
	$lq = new ListQuery($model, true, array('acl_filters' => array('view')));
	$lq->addFilterPrimaryKey($records);
	$result = $lq->runQuery();
	
	if($result && ! $result->failed) {
		require_once('include/layout/FieldFormatter.php');
		$fmt = new FieldFormatter('json');
		$list = $fmt->formatRows($result->fields, $result->rows);
	} else {
		$list = array();
	}
	
	$js_fields_arr = array('fields' => $list);
	json_return_value($js_fields_arr);
}


$json_supported_actions['change_password'] = array(
);
function json_change_password() {
	global $authController;
	$values = array_get_default($_REQUEST, 'values');
	if(! is_array($values) || empty($values['id'])) {
		return json_bad_request('missing_values');
	}

	$result = ListQuery::quick_fetch('User', $values['id'], array('id', 'user_name', 'need_new_password'));
	if (! $result)
		return json_bad_request('wrong_id');
	$uname = $result->getField('user_name');

	//Save error code
	$error_msg = 1;
	$error = false;
	$save = false;

	if (! AppConfig::is_admin() || $result->getField('id') == AppConfig::current_user_id()) {
		if ($result->getField('id') != AppConfig::current_user_id())
			return json_bad_request('unauthorized_access');
	
		if(! $authController->testAuthenticate($uname, $values['old_password'])) {
			$error = true;
			// Wrong old password error code
			$error_msg = 2;
		}
	}

	if ($values['new_password'] != $values['confirm_password'] || ! strlen($values['new_password'])) {
		$error = true;
		//Mismatch passwords error code
		$error_msg = 3;
	}

    if (! $error) {
    	$params = array();
    	if(isset($values['need_new_password']))
    		$params['pw_change'] = $values['need_new_password'];
        $save = $authController->updatePassword($uname, $values['new_password'], $params);
        if(! $save) $error_msg = 4;
    }

	if($save === false)
		$ret = array('result' => 'fail', 'msg' => $error_msg);
	else
		$ret = array('result' => 'ok', 'msg' => 0);
	json_auto_return($ret);
}


function _create_or_update($module, $model, $values) {
	json_clean_input_array($values);
	
	if(! empty($values['id']) && empty($values['new_with_id'])) {
		$result = ListQuery::quick_fetch($model, $values['id'], true, array('acl_filters' => array('edit')));
		if($result) $upd = RowUpdate::for_result($result);
		unset($values['id']);
	} else {
		if(ACLController::checkAccess($module, 'create')) {
			$upd = RowUpdate::blank_for_model($model);
		}
	}
	if(empty($upd))
		return false;
	
	if(!empty($values['new_with_id'])) {
		$upd->new_with_id = true;
	}
	
	$upd->request = $values;
	$upd->setInputFormat('json');
	$upd->setSource('json', 'create_or_update');
	$upd->loadRequest();
	if(! $upd->validate() || ! $upd->save())
		return null;
	return $upd->getPrimaryKeyValue();
}


$json_supported_actions['create_or_update'] = array(
);
// add check_result parameter to immediately do a retrieve?
function json_create_or_update() {
	if(! empty($_REQUEST['bean_name'])) {
		$model = $_REQUEST['bean_name'];
		$module = AppConfig::module_for_model($model);
	} else {
		$module = $GLOBALS['module'];
		$model = AppConfig::module_primary_bean($module);
	}
	if(! $module || ! $model) {
		return json_bad_request('module_error');
	}
	$values = array_get_default($_REQUEST, 'values');
	if(! is_array($values) || ! count($values)) {
		return json_bad_request('missing_values');
	}
	
	$return_id = _create_or_update($module, $model, $values);
	if(! $return_id)
		$ret = array('result' => 'fail', 'id' => '', 'module' => $module);
	else
		$ret = array('result' => 'ok', 'id' => $return_id, 'module' => $module);
	json_auto_return($ret);
}


$json_supported_actions['create_or_update_list'] = array(
);
// add check_result parameter to immediately do a retrieve?
function json_create_or_update_list() {
	if(! empty($_REQUEST['bean_name'])) {
		$model = $_REQUEST['bean_name'];
		$module = AppConfig::module_for_model($model);
	} else {
		$module = $GLOBALS['module'];
		$model = AppConfig::module_primary_bean($module);
	}
	if(! $module || ! $model) {
		return json_bad_request('module_error');
	}
	$values_list = array_get_default($_REQUEST, 'values_list');
	if(! is_array($values_list) || ! count($values_list)) {
		return json_bad_request('missing_values_list');
	}
	
	$ret_rows = array();
	$succ = 0; $fail = 0;
	foreach($values_list as $values) {
		$return_id = _create_or_update($module, $model, $values);
		if(! $return_id) {
			$ret_rows[] = array('result' => 'fail', 'id' => array_get_default($values, 'id', ''));
			$fail ++;
		}
		else {
			$ret_rows[] = array('result' => 'ok', 'id' => $return_id);
			$succ ++;
		}
	}
	
	$ret = array('ok' => $succ, 'fail' => $fail, 'rows' => $ret_rows);
	json_auto_return($ret);
}


/* legacy JSON method */
$json_supported_actions['get_full_list'] = array();

function json_get_full_list() {
	if(! empty($_REQUEST['bean_name'])) {
		$model = $_REQUEST['bean_name'];
		$module = AppConfig::module_for_model($model);
	} else {
		$module = $GLOBALS['module'];
		$model = AppConfig::module_primary_bean($module);
	}
	if(! $module || ! $model) {
		return json_bad_request('module_error');
	}

	$formatted = array_get_default($_REQUEST, 'formatted', false);
	$where = array_get_default($_REQUEST, 'where');
	$order = str_replace('\\','', rawurldecode(@$_REQUEST['order']));
	if(is_array($where)) {
		$where = _create_where_clause($where);
	} else {
		$where = str_replace('\\','', rawurldecode($where));
	}
	
	if(! ACLController::checkAccess($module, 'list'))
		return json_forbidden();
	$lq = new ListQuery($model, true, array('where' => $where, 'order_by' => $order, 'use_key_index' => false, 'acl_filters' => 'list'));
	$result = $lq->runQuery();

	if($result && ! $result->failed) {
		if($formatted) {
			require_once('include/layout/FieldFormatter.php');
			$fmt = new FieldFormatter('json');
			$rows = $fmt->formatRows($result->fields, $result->rows);
		} else {
			$rows = $result->rows;
		}
	} else {
		$rows = array();
	}
	
	json_return_value($rows);
}


function _create_where_clause($query) {
	$clauses = array();
	foreach ($query as $condition) {
		if (!isset($condition['operator'])) {
			$condition['operator'] = 'LIKE';
		}
		if (isset($condition['name']) && preg_match('~^[a-z0-9_`.]+$~i', $condition['name'])) {
			switch ($condition['operator']) {
				case '=':
				case '<':
				case '>':
				case '<=':
				case '>=':
				case '<>':
				case '!=':
					$clauses[] = $condition['name'] . $condition['operator'] . ' \'' . PearDatabase::quote($condition['value']) . '\'';
					break;
				case 'IN':
				case 'NOT IN':
					$in = array();
					foreach($condition['value'] as $val) {
						$in[] =  PearDatabase::quote($val);
					}
					$clauses[] = $condition['name'] . ' ' . $condition['operator'] . ' (\'' . join("','", $in)  . '\')';
					break;
                case 'starts_with':
                    $clauses[] = $condition['name'] . ' LIKE \'' . PearDatabase::quote($condition['value']) . '%\'';
                    break;
				default:
					$clauses[] = $condition['name'] . ' LIKE \'' . PearDatabase::quote($condition['value']) . '\'';
			}
		}
	}
	return implode(' AND ', $clauses);
}


$json_supported_actions['invoice_add_components'] = array();

function json_invoice_add_components()
{
	require_once 'modules/Assemblies/Assembly.php';
	require_once 'modules/Invoice/InvoiceLine.php';
	require_once 'modules/Invoice/InvoiceAdjustment.php';

	$parent_line_id = array_get_default($_REQUEST, 'parent_line_id');
	$assembly_id = array_get_default($_REQUEST, 'assembly_id');
	$position = array_get_default($_REQUEST, 'position', 0);

	$assembly = new Assembly;
	$assembly->retrieve($assembly_id);
	$products = Assembly::get_products_list($assembly->id);
	$assemblyLine = new InvoiceLine;
	$assemblyLine->retrieve($parent_line_id);
	$i = 0;
	foreach ($products as $product) {
		$adjId = create_guid();
		$line = new InvoiceLine;
		$line->invoice_id = $assemblyLine->invoice_id;
		$line->line_group_id = $assemblyLine->line_group_id;
		$line->name = $product['name'];
		$line->position = $i + $position;
		$i++;
		$line->related_id = $product['id'];
		$line->related_type = 'ProductCatalog';
		$line->quantity = $product['quantity'];
		$line->ext_quantity = $line->quantity;
		$line->tax_class_id =  $assembly->tax_code_id;
		$line->cost_price_usd = (float)$product['cost_usdollar'];
		$line->cost_price = (float)$product['cost_usdollar'];
		$line->list_price_usd = (float)$price['list_usdollar'];
		$line->list_price = (float)$price['list_usdollar'];
		$line->unit_price = (float)$product['purchase_usdollar'];
		$line->unit_price_usd = (float)$product['purchase_usdollar'];
		$line->ext_price = (float)$product['purchase_usdollar'] * $product['quantity'];
		$line->ext_price_usd = (float)$product['purchase_usdollar'] * $product['quantity'];
		$line->pricing_adjust_id = $adjId;
		$line->parent_id = $parent_line_id;
		$line->save();

		$adj = new InvoiceAdjustment;
		$adj->id = $adjId;
		$adj->new_with_id = 1;
		$adj->invoice_id = $assemblyLine->invoice_id;
		$adj->line_group_id = $assemblyLine->line_group_id;
		$adj->line_id = $line->id;
		$adj->position = $i + $position;
		$i++;
		$adj->type = 'Inherit';
		$adj->save();

	}
	$ret = $i;
	json_return_value($ret);
}

$json_supported_actions['get_relationships'] = array();

function json_get_relationships()
{
	global $module;
	$ids = array();
	global  $beanList, $beanFiles;

	$module_id = array_get_default($_REQUEST, 'module_id');
	$related_module = array_get_default($_REQUEST, 'related_module');

	if(empty($beanList[$related_module])){
		json_bad_request('no_module');
	}
	$class_name = $beanList[$module];
	require_once($beanFiles[$class_name]);
	$mod = new $class_name();
	$mod->retrieve($module_id);
	if(!$mod->ACLAccess('DetailView')){
		json_bad_request('no_access');
	}

	$id_list = get_linked_records($related_module, $module, $module_id, 0);

	if ($id_list === FALSE) {
		json_bad_request('no_relationship_support');
	}
	elseif (count($id_list) == 0) {
		$ret = array('ids'=>$ids );
		json_return_value($ret);
		return;
	}

	$list = array();

	$id_list_quoted = array_map("add_squotes", $id_list);
	$in = implode(", ", $id_list_quoted);

	$related_class_name = $beanList[$related_module];
	$related_mod = new $related_class_name();

	$sql = "SELECT {$related_mod->table_name}.id FROM {$related_mod->table_name} ";





	$sql .= " WHERE {$related_mod->table_name}.id IN ({$in}) ";

	$result = $related_mod->db->query($sql);
	while ($row = $related_mod->db->fetchByAssoc($result)) {
		$list[] = $row['id'];
	}

	$return_list = array();

	foreach($list as $id) {
		$related_class_name = $beanList[$related_module];
		$related_mod = new $related_class_name();
		$related_mod->retrieve($id);

		$return_list[] = array(
			'id' => $id,
			'date_modified' => $related_mod->date_modified,
			'deleted' => $related_mod->deleted
		);
	}

	$ret = array('ids' => $return_list);
	json_return_value($ret);
}
function add_squotes($str) {
	return "'" . $str . "'";
}


$json_supported_actions['allocate_payment'] = array();

function json_allocate_payment()
{
	$invoice_id = array_get_default($_REQUEST, 'invoice_id');
	$payment_id = array_get_default($_REQUEST, 'payment_id');
	$amount = array_get_default($_REQUEST, 'amount');

	require_once 'modules/Payments/Payment.php';
	require_once 'include/database/RowUpdate.php';
	global $db, $current_user;
	if(!is_admin($current_user)){
		json_bad_request('no_access');	
	}

	$id = create_guid();
	$query = "INSERT INTO invoices_payments SET "
		. "  `invoice_id`='" . PearDatabase::quote($invoice_id) . "'"
		. ", `payment_id`='" . PearDatabase::quote($payment_id) . "'"
		. ", `amount`='" . PearDatabase::quote($amount) . "'"
		. ", `exchange_rate`=1"
		. ", `amount_usdollar`='" . PearDatabase::quote($amount) . "'"
		. ", `date_modified`='" . gmdate('Y-m-d H:i:s') . "'";
	$db->query($query, true);

	$upd = RowUpdate::blank_for_model('Payment');
	$upd->setSource('json', 'allocate_payment');
	$related = array('invoices' => array($invoice_id => $invoice_id));
	Payment::update_related_amounts($upd, $related);
	$ret = array('id' => $id);
	json_return_value($ret);
}

$json_supported_actions['convert_lead'] = array();

function json_convert_lead()
{
	global $db, $current_user;
	if(!is_admin($current_user)){
		return json_forbidden();
	}

	$lead_id = array_get_default($_REQUEST, 'lead_id');

	require_once 'modules/Accounts/Account.php';
	require_once 'modules/Contacts/Contact.php';
	require_once 'modules/Leads/Lead.php';
	$lead = new Lead;
	if (!$lead->retrieve($lead_id)) {
		return json_bad_request('no_lead');
	}

	$contact = new Contact;
	$contact->id = create_guid();
	$contact->new_with_id = true;

	$account = new Account;
	$account->id = create_guid();
	$account->new_with_id = true;

	$transfer = array(
		'first_name',
		'last_name',
		'salutation',
		'email1',
		'email2',

		'primary_address_street',
		'primary_address_city',
		'primary_address_state',
		'primary_address_postalcode',
		'primary_address_country',

		'alt_address_street',
		'alt_address_city',
		'alt_address_state',
		'alt_address_postalcode',
		'alt_address_country',

		'phone_home',
		'phone_work',
		'phone_mobile',
		'phone_fax',
		'phone_other',
		'portal_name',
		'portal_active',
		'assigned_user_id',
	);
	foreach ($transfer as $f) {
		if (isset($lead->$f)) {
			$contact->$f = $lead->$f;
		}
	}
	$contact->primary_account_id = $account->id;

	if(! $contact->save()) {
		$GLOBALS['log']->error("Error converting lead to contact");
		return;
	}

	if ($GLOBALS['B2C']) {
		$account->createForPrimaryContact($contact);
		$account->name = $contact->getB2CAccountName();
	} else {
		if (!empty($lead->account_name)) {
			$account->name = $lead->account_name;
		} else {
			$account->name = $contact->getB2CAccountName();
		}
		$account->primary_contact_id = $contact->id;
		$account->assigned_user_id = $contact->assigned_user_id;
	}
	$account->save();
	$account->load_relationship('contacts');
	$account->contacts->add($contact->id);
	$lead->converted_lead("'$lead->id'", "'$contact->id'", "'$account->id'", "NULL");

	if (AppConfig::setting('portal.conversion_email')) {
		global $current_user;
		if(empty($current_user->email1)) {
			$GLOBALS['log']->warn("Notifications: no e-mail address set for user {$current_user->user_name}, cancelling send");
		} else {
			$notify_address = from_html($current_user->email2);
			$notify_name = (empty($current_user->first_name)) ? from_html($current_user->user_name) : from_html($current_user->first_name . " " . $current_user->last_name);

			require_once("include/Notify/RecordActionMessage.php");
			$message = RecordActionMessage::for_action('Lead', 'converted', $current_user->id);
			if($message) {
				$message->addRecordId($lead->id);
				$mailer = $message->getMailer();
				if(AppConfig::setting('notify.send_from_assigning_user'))
					$mailer->setFromUserId($current_user->id);
				$mailer->addTargetId('User', $current_user->id);
				$mailer->sendMessage($message);
			}
		}
	}
	$ret = array(
		'ids' => array(
			$contact->id,
			$account->id
		),
	);
	json_return_value($ret);
}

$json_supported_actions['quicksearch'] = array();

function json_quicksearch() {
	$module = array_get_default($_REQUEST, 'module');
	$bean_name = array_get_default($_REQUEST, 'bean_name');
	$filter_text = array_get_default($_REQUEST, 'filter_text', '');
	$offset = (int)array_get_default($_REQUEST, 'offset', 0);
	$limit = (int)array_get_default($_REQUEST, 'limit', 10);
	if($limit > 100) $limit = 100;
	$include = array_get_default($_REQUEST, 'include_ids');
	if($include && ! is_array($include)) {
		$include = array_filter(array_map('trim', explode(',', $include)));
	}
	$exclude = array_get_default($_REQUEST, 'exclude_ids');
	if($exclude && ! is_array($exclude)) {
		$exclude = array_filter(array_map('trim', explode(',', $exclude)));
	}

	if(! $bean_name) {
		if(! $module)
			return json_bad_request('no_bean_name');
		$bean_name = AppConfig::setting("modinfo.primary_beans.$module");
		if(! $bean_name)
			return json_bad_request('unsupported_module');
	} else {
		$module = AppConfig::setting("model.detail.$bean_name.module_dir");
	}

	//if(! strlen($filter_text))
	//	return json_bad_request('no_query');
		
	if(! ACLController::checkAccess($module, 'List'))
		return json_forbidden();

	require_once('include/ListView/ListFormatter.php');
	$fmt = new ListFormatter($bean_name);
	$fmt->setFormat('json', 'quicksearch');
	$lq =& $fmt->getQuery();
	
	if(isset($_REQUEST['fields']) && is_array($_REQUEST['fields'])) {
		$add_f = array();
		foreach($_REQUEST['fields'] as $k => $v) {
			if($v == '_display')
				$dname_alias = $lq->addDisplayName(is_numeric($k) ? null : $k);
			else if(is_string($v))
				$add_f[$k] = $v;
		}
		if($add_f)
			$added_f = $lq->addFields($add_f, true, false, true);
	}

	$lq->addPrimaryKey('id');
	if(empty($dname_alias))
		$dname_alias = $lq->addDisplayName('_display');
	if($include)
		$lq->addSimpleFilter('id', $include, 'in');
	if($exclude)
		$lq->addSimpleFilter('id', $exclude, 'not_in');

	$filter =& $fmt->getFilterForm();
	$filter->addBasicFilters();
	$filter->addAutoFilters();
	if($filter->filterDefined('filter_text')) {
		$fmt->loadFilter($_REQUEST);
	} else {
		if($dname_alias)
			$fmt->getQuery()->addFilterClause(array('field' => $dname_alias, 'value' => $filter_text, 'operator' => 'like', 'match' => 'prefix'));
		else {
			// cannot filter, return no results
			$limit = 0;
		}
	}
	if($dname_alias) {
		$lq->setOrderBy(
			array(
				array('field' => $dname_alias, 'type' => 'quicksearch', 'prefix' => $filter_text),
				array('field' => $dname_alias),
			));
	}

	// $fmt->list_filter will pull in filter_text from request
	$result = $fmt->getFormattedResult($offset, $limit);
	$enc = json_convert_list_result($result);
	$enc['display_name'] = $dname_alias;
    json_return_value($enc);
}


$json_supported_actions['email_quicksearch'] = array();

function json_email_quicksearch() {
	$filter_text = array_get_default($_REQUEST, 'filter_text', '');
	$offset = (int)array_get_default($_REQUEST, 'offset', 0);
	$limit = (int)array_get_default($_REQUEST, 'limit', 10);
	if($limit > 100) $limit = 100;

	if(! strlen($filter_text))
		;; //return json_bad_request('no_query');
	
	$module = array_get_default($_REQUEST, 'module', 'Contacts');
	$bean_name = AppConfig::module_primary_bean($module);
	if(! ACLController::checkAccess($module, 'List'))
		return json_forbidden();
	
	$listQuery = new ListQuery($bean_name);
	
	require_once('include/ListView/ListFormatter.php');
	$fmt = new ListFormatter($listQuery);
	$fmt->setFormat('json', 'quicksearch');
	
	if(preg_match('~^\s*(.*)\s*<([^>]+)>~', $filter_text, $m)) {
		$filter_name = $m[1];
		$filter_text = $m[2];
	} else {
		$filter_name = $filter_text;
	}
	if(preg_match('/^["\'](.*?)["\']$/', $filter_name, $m))
		$filter_name = str_replace('\"', '"', trim($m[1]));
	
	$listQuery->addPrimaryKey('id');
	$listQuery->addModelNameField();
	$listQuery->addSourceNameField();
	$dname_alias = $listQuery->addDisplayName('name', 'email_quicksearch');
	/*
	if ($module == 'Leads' || $module == 'Contacts') {
		$listQuery->fields['name']['source'] = array(
			'type' => 'person_name',
			'fields' => array('first_name', 'last_name'),
			'alias_fields' => array('first_name', 'last_name'),
		);
	}
	 */
	$listQuery->addFields(array('email1', 'email2', 'first_name', 'last_name', 'salutation', 'name'));
	
	$filter = array(
		'operator' => 'OR',
		'multiple' => array(
			array('field' => $dname_alias, 'value' => $filter_name, 'operator' => 'like'),
			array('field' => 'email1', 'value' => $filter_text, 'operator' => 'like'),
			array('field' => 'email2', 'value' => $filter_text, 'operator' => 'like'),
		),
	);
	$listQuery->addFilterClause($filter);
	$filter = array(
		'operator' => 'OR',
		'multiple' => array(
			array('field' => 'email1', 'operator' => 'length'),
			array('field' => 'email2', 'operator' => 'length'),
		),
	);
	$listQuery->addFilterClause($filter);
	
	if($dname_alias) {
		$listQuery->setOrderBy(
			array(
				array('field' => $dname_alias, 'type' => 'quicksearch', 'prefix' => $filter_text),
				array('field' => $dname_alias),
			));
	}

	require_once('modules/Emails/utils.php');
	$result = $fmt->getFormattedResult($offset, $limit);
	if(! $result->failed) {
		foreach($result->formatted_rows as &$row) {
			if(empty($row['email1']) || (! empty($row['email2']) && strpos($row['email2'], $filter_text) !== false)) {
				$email = $row['email2'];
			} else
				$email = $row['email1'];
			$display = format_email_row($email, $row[$dname_alias]);
			$row['_display'] = $display['entry'];
		}
	}
	
	$enc = json_convert_list_result($result);
	$enc['display_name'] = '_display';
    json_return_value($enc);
}

$json_supported_actions['list_recent'] = array();

function json_list_recent() {
	$module = array_get_default($_REQUEST, 'module');
	$bean_name = array_get_default($_REQUEST, 'bean_name');
	$limit = (int)array_get_default($_REQUEST, 'limit', 10);
	$limit = min(100, $limit);
	$exclude = array_get_default($_REQUEST, 'exclude_ids');
	if($exclude && is_string($exclude)) {
		$exclude = array_filter(array_map('trim', explode(',', $exclude)));
	}

	if(! $bean_name) {
		if($module) {
			$bean_name = AppConfig::module_primary_bean($module);
			if(! $bean_name)
				return json_bad_request('unsupported_module');
		}
	} else {
		$module = AppConfig::module_for_model($bean_name);
	}
	
	if($module) {
		if(! ACLController::checkAccess($module, 'List'))
			return json_forbidden();

		require_once('data/Tracker.php');
		require_once('include/ListView/ListFormatter.php');
		$fmt = new ListFormatter($bean_name);
		$fmt->setFormat('json', 'quicksearch');
		$lq =& $fmt->getQuery();
	
		if(isset($_REQUEST['fields']) && is_array($_REQUEST['fields'])) {
			$add_f = array();
			foreach($_REQUEST['fields'] as $k => $v) {
				if($v == '_display')
					$dname_alias = $lq->addDisplayName(is_numeric($k) ? null : $k);
				else if(is_string($v))
					$add_f[$k] = $v;
			}
			if($add_f)
				$added_f = $lq->addFields($add_f, true, false, true);
		}

		$lq->addPrimaryKey('id');
		if(empty($dname_alias))
			$dname_alias = $lq->addDisplayName('_display');
		$lq->addAclFilter('view');
		if($exclude)
			$lq->addSimpleFilter('id', $exclude, 'not_in');

		Tracker::add_tracker_query_join($lq, 'trk', true, true);

		$filter =& $fmt->getFilterForm();
		$filter->addBasicFilters();
		$filter->addAutoFilters();

		if(isset($_REQUEST['filters']) && is_array($_REQUEST['filters'])) {
			$fmt->loadFilter($_REQUEST['filters']);
		}
	
		$result = $fmt->getFormattedResult(0, $limit);
	}
	else {
		require_once('data/Tracker.php');
		$history = Tracker::get_recently_viewed(AppConfig::current_user_id(), null, $exclude);
		$result = new ListResult();
		$result->module_dirs = [];
		$result->base_model = 'Tracker';
		$result->primary_key = 'id';
		$result->limit = $limit;
		$result->total_count = 0;
		$dname_alias = '_display';
		if(count($history) > 0) {
			$idx = 0;
			foreach($history as $row) {
				if($idx >= $limit)
					break;
				$row = [
					'_display' => $row['item_summary'],
					'query_source' => $row['module_name'],
					'icon' => 'theme-icon module-'.$row['module_name'],
					'id' => $row['item_id'],
				];
				$result->rows[] = $row;
				$result->total_count ++;
				$idx ++;
			}
		}
		$result->formatted_rows = $result->rows;
	}

	/*if(! $result->failed) {
		$flip = array_flip($recent_ids);
		$rids = array();
		foreach(array_keys($result->rows) as $idx => $k) {
			$rids[$k] = array_get_default($flip, $k, 100+$idx);
		}
		array_multisort($rids, $result->rows, $result->formatted_rows);
	}*/
    
	$enc = json_convert_list_result($result);
	$enc['display_name'] = $dname_alias;
	json_return_value($enc);
}


$json_supported_actions['add_recent'] = array();

function json_add_recent() {
	$module = array_get_default($_REQUEST, 'module');
	$bean_name = array_get_default($_REQUEST, 'bean_name');
	$record = array_get_default($_REQUEST, 'record');
	$source = array_get_default($_REQUEST, 'source');

	if(! $bean_name) {
		if(! $module)
			return json_bad_request('no_bean_name');
		$bean_name = AppConfig::module_primary_bean($module);
		if(! $bean_name)
			return json_bad_request('unsupported_module');
	} else {
		$module = AppConfig::module_for_model($bean_name);
	}
	if(! $record) {
		return json_bad_request('no_record');
	}

	if(! ACLController::checkAccess($module, 'view'))
		return json_forbidden();
	$row = ListQuery::quick_fetch($bean_name, $record, array('id', '_display'), array('acl_checks' => array('view', 'owner')));
	if(! $row || ! $row->getAclAllowed('view'))
		return json_forbidden();
	
	// FIXME - check view permission on individual record
    require_once('data/Tracker.php');
	Tracker::track_view(AppConfig::current_user_id(), $module, $bean_name, $record, $row->getField('_display'), $row->getOwnerId());
	$ret = array('result' => 'ok');
	json_return_value($ret);
}


$json_supported_actions['popup_related_data'] = array();

function json_popup_related_data() {
	global $module;
	require_once('include/Popups/PopupManager.php');
	$mgr = new PopupManager();
	if(empty($_REQUEST['show_favorites']))
		$mgr->show_favorites = false;
	$mgr->loadRequest();
	
	$layout = array_get_default($_REQUEST, 'layout');
	
	if($layout) {
		if(! $mgr->initModuleView($module, $layout)) {
			return json_forbidden();
		}
	} else {
		if(! $mgr->initModule($module)) {
			return json_forbidden();
		}
	}

	$rows = array_get_default($_REQUEST, 'rows');

	if(! $rows)
		return json_bad_request('no_rows');
	if(! is_array($rows) && $rows !== 'all')
		$rows = array($rows);

	$fields = array_get_default($_REQUEST, 'fields');
	if(! $fields)
		return json_bad_request('no_fields');
	if(! is_array($fields))
		$fields = array_map('trim', explode(',', $fields));

	$filters = array_get_default($_REQUEST, 'filters');
	
	$result = $mgr->fetchJsonData($rows, $fields, 'list', $filters);
	$enc = json_convert_list_result($result);
	json_return_value($enc);
}


$json_supported_actions['subpanel_order'] = array();
function json_subpanel_order() {
	if (AppConfig::current_user_id()) {
		$tab = array_get_default($_REQUEST, 'tab');
		if (empty($tab))
			$tab = 'Standard';
		$key = 'subpanels_order_' . $_REQUEST['module'] . '_' . $tab;
		$current = UserPreference::getPreference($key, 'global');
		if (!$current)
			$current = array();
		$positions = array();
		foreach ($current as &$p) {
			if (in_array($p['name'], $_REQUEST['order']))
				$p['visible'] = true;
		}
		foreach ($current as $i => $p) {
			if (array_get_default($p, 'visible', true))
				$positions[] = $i;
		}
		foreach ($_REQUEST['order'] as $i =>$n) {
			if (isset($positions[$i]))
				$current[$positions[$i]] = array('name' => $n);
			else
				$current[] = array('name' => $n);
		}
		UserPreference::setPreference($key, $current, 0, 'global');
	}
	$result = true;
	json_return_value($result);
}


$json_supported_actions['move_subpanel_row'] = array();
function json_move_subpanel_row() {
	$module = array_get_default($_REQUEST, 'module');
	$bean = AppConfig::module_primary_bean($module);
	$record = array_get_default($_REQUEST, 'record');
	$link_name = array_get_default($_REQUEST, 'link_name');
	$target = array_get_default($_REQUEST, 'target_id');
	$offset = (int)array_get_default($_REQUEST, 'offset', 0);
	if(! $bean || ! $record || ! $link_name || ! $target)
		return json_bad_request('missing_param');
	$parent = ListQuery::quick_fetch($bean, $record, array('id'), array('acl_filter' => 'view'));
	if(! $parent)
		return json_bad_request('not_found');
	$def = $parent->getLinkModel($link_name);
	if($def) {
		$posn_f = $def->default_order_by;
		if($posn_f) {
			$posn = '~join.'.$posn_f;
			$q = $parent->getLinkQuery($link_name, array('id', $posn));
			$q->setOrderBy($posn);
			$all = array();
			$found = null;
			$idx = 0;
			foreach($q->runQuery() as $row) {
				$id = $row->getField('id');
				if($id === $target)
					$found = $idx;
				$all[$id] = $row->getField($posn);
				$idx ++;
			}
			if(isset($found)) {
				$newpos = $found + $offset;
				if($newpos < 0) $newpos = 0;
				if($newpos >= count($all))
					$newpos = count($all)-1;
				if($newpos != $found) {
					$ids = array_keys($all);
					array_splice($ids, $found, 1);
					array_splice($ids, $newpos, 0, array($target));
					$nall = array();
					foreach($ids as $id)
						$nall[$id] = $all[$id];
					$all = $nall;
				}
				$lastpos = 0;
				$updates = array();
				foreach($all as $id => $pos) {
					if($pos <= $lastpos)
						$updates[$id] = ++ $lastpos;
					else
						$lastpos = $pos;
				}
				if($updates) {
					$upd = $parent->getRowUpdate();
					foreach($updates as $id => $pos) {
						$param = array('id' => $id, $posn_f => $pos);
						$upd->addUpdateLink($link_name, $param);
					}
				}
			}
		}
	}
	$ret = array('status' => 'ok');
	json_return_value($ret);
}


$json_supported_actions['additional_details'] = array();
function json_additional_details() {
	global $pageInstance, $pageController, $theme, $themeInstance;
	
	AppConfig::init_page_controller();
	$themeInstance = $pageController->create_theme_instance();
	require_once('include/layout/IncludeManager.php');
	$pageInstance = new IncludeManager();

	$model = array_get_default($_REQUEST, 'bean_name');
	if(! $model)
		return json_bad_request('no_model');
	$primary = AppConfig::module_primary_bean($model);
	if($primary) $model = $primary;

	require_once('include/DetailView/DetailFactory.php');
	/** @var MiniDetailManager  */
	$mgr = DetailFactory::getManager('mini');

	$record = array_get_default($_REQUEST, 'record');
	if(! $record)
		return json_bad_request('no_record');

	$layout = array_get_default($_REQUEST, 'layout');
	$context = array_get_default($_REQUEST, 'context');

	$result = $mgr->fetchJsonData($model, $record, $layout, $context);
	json_return_value($result);
}


$json_supported_actions['get_list_fields'] = array();
function json_get_list_fields() {
	$model = array_get_default($_REQUEST, 'model');
	$link_name = array_get_default($_REQUEST, 'link_name');
	$link_alias = array_get_default($_REQUEST, 'link_alias');
	$form_name = array_get_default($_REQUEST, 'form_name');
	$context = array_get_default($_REQUEST, 'context');
	
	if(! $model) {
		$module = array_get_default($_REQUEST, 'module');
		if($module)
			$model = AppConfig::module_primary_bean($module);
			if(! $model)
				return json_bad_request('no_module');
	}
	
	require_once('include/ListView/ListLayoutEditor.php');
	$layout = new ListLayoutEditor();
	if(! $layout->initFromModel($model, null, $form_name, $context))
		return json_bad_request('invalid_model');
	$data = ['result' => $layout->getFieldsJson($link_name, $link_alias)];
	json_return_value($data);
}


$json_supported_actions['get_list_updates'] = array();
function json_get_list_updates() {
	$model = array_get_default($_REQUEST, 'model');
	$link_name = array_get_default($_REQUEST, 'link_name');
	$link_alias = array_get_default($_REQUEST, 'link_alias');
	$form_name = array_get_default($_REQUEST, 'form_name');
	$context = array_get_default($_REQUEST, 'context');
	$wf_mod = array_get_default($_REQUEST, 'wf_module');
	
	if(! $model) {
		$module = array_get_default($_REQUEST, 'module');
		if($module)
			$model = AppConfig::module_primary_bean($module);
			if(! $model)
				return json_bad_request('no_module');
	}
	
	require_once('include/ListView/ListLayoutEditor.php');
	$layout = new ListLayoutEditor();
	if(! $layout->initFromModel($model, null, $form_name, $context))
		return json_bad_request('invalid_model');
	$layout->wf_module = $wf_mod;
	$data = ['result' => $layout->getFieldsForUpdate($link_name, $link_alias)];
	json_return_value($data);
}


$json_supported_actions['get_list_totals'] = array();
function json_get_list_totals() {
	$model = array_get_default($_REQUEST, 'model');
	$link_name = array_get_default($_REQUEST, 'link_name');
	$link_alias = array_get_default($_REQUEST, 'link_alias');
	$form_name = array_get_default($_REQUEST, 'form_name');
	$context = array_get_default($_REQUEST, 'context');
	
	if(! $model) {
		$module = array_get_default($_REQUEST, 'module');
		if($module)
			$model = AppConfig::module_primary_bean($module);
			if(! $model)
				return json_bad_request('no_module');
	}
	
	require_once('include/ListView/ListLayoutEditor.php');
	$layout = new ListLayoutEditor();
	if(! $layout->initFromModel($model, null, $form_name, $context))
		return json_bad_request('invalid_model');
	$data = ['result' => $layout->getTotalsJson($link_name, $link_alias)];
	json_return_value($data);
}


$json_supported_actions['get_list_filters'] = array();
function json_get_list_filters() {
	$model = array_get_default($_REQUEST, 'model');
	$link_name = array_get_default($_REQUEST, 'link_name');
	$link_alias = array_get_default($_REQUEST, 'link_alias');
	$form_name = array_get_default($_REQUEST, 'form_name');
	$context = array_get_default($_REQUEST, 'context');

	if(! $model) {
		$module = array_get_default($_REQUEST, 'module');
		if($module)
			$model = AppConfig::module_primary_bean($module);
			if(! $model)
				return json_bad_request('no_module');
	}
	
	require_once('include/ListView/ListLayoutEditor.php');
	$layout = new ListLayoutEditor();
	if(! $layout->initFromModel($model, null, $form_name, $context))
		return json_bad_request('invalid_model');
	$data = ['result' => $layout->getFiltersJson($link_name, $link_alias)];
	json_return_value($data);
}


$json_supported_actions['get_list_sources'] = array();
function json_get_list_sources() {
	$model = array_get_default($_REQUEST, 'model');
	$link_name = array_get_default($_REQUEST, 'link_name');
	$link_alias = array_get_default($_REQUEST, 'link_alias');
	$context = array_get_default($_REQUEST, 'context');
	$single = array_get_default($_REQUEST, 'single');
	
	if(! $model) {
		$module = array_get_default($_REQUEST, 'module');
		if($module)
			$model = AppConfig::module_primary_bean($module);
			if(! $model)
				return json_bad_request('no_module');
	}
	
	require_once('include/ListView/ListLayoutEditor.php');
	$layout = new ListLayoutEditor();
	if(! $layout->initFromModel($model, null, null, $context))
		return json_bad_request('invalid_model');
	$data = ['result' => $layout->getSourcesJson($link_name, $link_alias, $single)];
	json_return_value($data);
}


$json_supported_actions['status_check'] = array();
function json_status_check() {
	global $current_user;
	$result = array('timestamp' => time(), 'unread_emails' => 0, 'new_emails' => 0, 'alerts' => array());
	
	if(! empty($_REQUEST['user_id']) && $_REQUEST['user_id'] != $current_user->id)
		json_auth_die(null, false);
	
	if(ACLController::checkAccess('Emails', 'list')) {
		$current_user->loadPreferences('email_import', null, true);
		$count = $current_user->getPreference('new_email_count', 'email_import');
		$stamp = $current_user->getPreference('new_email_timestamp', 'email_import');
		$last_stamp = array_get_default($_REQUEST, 'timestamp', 0);
		$new_email_messages = 0;

		require_once 'modules/EmailFolders/EmailFolder.php';
		$folders = EmailFolder::get_folders_list($current_user->id);
		foreach ($folders as $folder) {
			if ($folder['reserved'] == STD_FOLDER_INBOX && $folder['newmessages'] > 0) {
				$new_email_messages = $folder['newmessages'];
				break;
			}
		}
		$result['unread_emails'] = $new_email_messages;
		$result['new_emails'] = $last_stamp < $stamp ? (int)$count : 0;
	}
	
	require_once('include/javascript/jsAlerts.php');
	$alerts = new jsAlerts();
	$alerts->addDefault();
	$alerts->addActivities();
	$result['alerts'] = $alerts->getAlerts();
	
	json_return_value($result);
}



function json_convert_list_result(&$listResult) {
	$enc = array(
		'failed' => $listResult->failed,
		'module_dirs' => $listResult->module_dirs,
		'base_model' => $listResult->base_model,
		'primary_key' => $listResult->primary_key,
		'fields' => json_restrict_fields($listResult->fields),
		'is_union' => $listResult->is_union,
		'offset' => $listResult->offset,
		'limit' => $listResult->limit,
		'result_count' => $listResult->result_count,
		'total_count' => $listResult->total_counted ? null : $listResult->total_count,
	);
	if(! $listResult->failed) {
		$enc['rows'] = array_values($listResult->rows);
		$enc['formatted_rows'] = array_values($listResult->formatted_rows);
	}
	return $enc;
}

function json_restrict_fields($fields) {
	if(is_array($fields)) {
		$fs = array();
		foreach($fields as $k => $v) {
			unset($v['source']);
			unset($v['union_source']);
			$fs[$k] = $v;
		}
		return $fs;
	}
	return $fields;
}

$json_supported_actions['save_assigned_resource_data'] = array();

function json_save_assigned_resource_data() {
    $user_id = array_get_default($_REQUEST, 'target_id', '');
    $project_task_id = array_get_default($_REQUEST, 'record', '');
    $list_id = array_get_default($_REQUEST, 'list_id', '');
    $result = 'failure';
    $task = ListQuery::quick_fetch('ProjectTask', $project_task_id);
	$reset = array_get_default($_REQUEST, 'reset');

    if ($task && ! $task->failed) {
        $update = RowUpdate::for_result($task);
        $update->setSource('json', 'save_assigned_resource_data');
        $params = array();
        $field_name = array_get_default($_REQUEST, 'field');
        $field_value = array_get_default($_REQUEST, 'field_value');
        
        if (! empty($field_name)) {
            $params[$user_id] = array('name' => $field_name, 'value' => $field_value);
            $update->addUpdateLink('booking_users', $user_id, $params, true);
            $result = 'ok';

			if ($field_name == 'estim_hours') {
                $task_hours = calc_task_hours($task->getField('date_start'), $task->getField('date_due'), $task->getField('include_weekends'));
                $utilization = unformat_number($field_value) / $task_hours * 100;
                $params[$user_id] = array('name' => 'utilization', 'value' => $utilization);
                $update->addUpdateLink('booking_users', $user_id, $params);
            }

            if ($field_name == 'hourly_cost') {
                $params[$user_id] = array('name' => 'hourly_cost_custom', 'value' => $reset ? 0 : 1);
                $update->addUpdateLink('booking_users', $user_id, $params);
                require_once('modules/Currencies/Currency.php');
                $currency = new Currency();
                $currency->retrieve($task->getField('currency_id'));
                $params[$user_id] = array('name' => 'hourly_cost_usdollar', 'value' => $currency->convertToDollar(unformat_number($field_value)));
                $update->addUpdateLink('booking_users', $user_id, $params);
            }
        }
    }

	$enc['list_id'] = $list_id;
    $enc['result'] = $result;
    json_return_value($enc);
}

$json_supported_actions['save_quick_edit_result'] = array();

function json_save_quick_edit_result() {
    $list_id = array_get_default($_REQUEST, 'list_id', '');
    $module = array_get_default($_REQUEST, 'module', '');
    $model = array_get_default($_REQUEST, 'bean_name', '');
	$parent_key = array_get_default($_REQUEST, 'record', '');
	$target_id = array_get_default($_REQUEST, 'target_id', '');
	$link_name = array_get_default($_REQUEST, 'link_name', '');
	$field = array_get_default($_REQUEST, 'field', '');
	$value = array_get_default($_REQUEST, 'field_value');
    $result = 'failure';
    $model = null;
    
    if ($module && ! $model)
        $model = AppConfig::module_primary_bean($module);
    
    if(substr($field, 0, 6) == '~join.') {
    	$field = substr($field, 6);
    	$update_join = true;
    }
    else if(strpos($field, '.') !== false) {
    	// link fields not generally supported
    	$field = null;
    }

    if ($model && $link_name && $parent_key && $target_id && $field) {
        $parent = ListQuery::quick_fetch($model, $parent_key, null, array('acl_filters' => 'edit'));

        if ($parent && ! $parent->failed) {
        	if(! empty($update_join) || $parent->getLinkModel($link_name)) {
        		$upd = RowUpdate::for_result($parent);
        		$upd->setSource('json', 'save_quick_edit_result');
				$params = array($target_id => array('name' => $field, 'value' => $value));
				if($upd->addUpdateLink($link_name, $target_id, $params, true, true))
					$result = 'ok';
        	}
        	else {
				$lq = $parent->getLinkQuery($link_name, null, array('acl_filters' => 'edit'));
        		$target = $lq->queryRecord($target_id);
        		if($target) {
        			$upd = RowUpdate::for_result($target);
        			$upd->setSource('json', 'save_quick_edit_result');
					$upd->loadInput(array($field => $value), true);
					if($upd->validate() && $upd->save())
						$result = 'ok';
				}
			}
        }
    } else if ($model && $target_id && $field && isset($value)){
        // this one is the one, if quickedit is called from a standard ListView, without Link and Relation
        $rowResultForPrimary = ListQuery::quick_fetch($model, $target_id, null, array('acl_filters' => 'edit'));
        if ($rowResultForPrimary && ! $rowResultForPrimary->failed) {
            $upd = RowUpdate::for_result($rowResultForPrimary);
            $upd->setSource('json', 'save_quick_edit_result');
            $upd->loadInput(array($field => $value), true);
            if($upd->validate() && $upd->save()) {
                $result = 'ok';
            }
        }
    }

	$enc['list_id'] = $list_id;
    $enc['result'] = $result;

    json_return_value($enc);
}

$json_supported_actions['get_user_task_rates'] = array();
function json_get_user_task_rates()
{
    $task_id = array_get_default($_REQUEST, 'task_id');
    $user_id = array_get_default($_REQUEST, 'user_id');
    $result = 'failure';
	$return = array();
	$task = ListQuery::quick_fetch('ProjectTask', $task_id);
	if ($task && !$task->failed) {
		$lq = new ListQuery('ProjectTask', array('~join.hourly_cost_custom', '~join.hourly_cost'), array('link_name' => 'booking_users', 'parent_key' => $task_id));
		$lq->addSimpleFilter('~join.user_id', $user_id);
		$user = $lq->runQuerySingle();
		if ($user && !$user->failed && $user->getField('~join.hourly_cost_custom')) {
			$return['hourly_cost'] = $user->getField('~join.hourly_cost');
			$return['currency_id'] = $task->getField('currency_id');
			$return['exchange_rate'] = $task->getField('exchange_rate');
			$result = 'ok';
		}
	}
	if ($result != 'ok') {
		$lq = new ListQuery('Employee');
		$lq->addSimpleFilter('user_id', $user_id);
		$emp = $lq->runQuerySingle();

		if ($emp && !$emp->failed) {
			$return['hourly_cost'] = $emp->getField('std_hourly_rate');
			$return['currency_id'] = $emp->getField('salary_currency_id');
			$return['exchange_rate'] = $emp->getField('salary_exchange_rate');
			$result = 'ok';
		}
	}

    $return['result'] = $result;

    json_return_value($return);
}

$json_supported_actions['get_rates_by_cat'] = array();

function json_get_rates_by_cat() {
    $category_id = array_get_default($_REQUEST, 'category_id');
    $result = 'failure';
    $return = array();

    $fields = array('billing_rate', 'billing_currency', 'billing_currency_id', 'paid_rate',
        'paid_currency', 'paid_currency_id', 'tax_code', 'tax_code_id');
    $category = ListQuery::quick_fetch_row('BookingCategory', $category_id, $fields);

    if ($category) {
        for ($i = 0; $i < sizeof($fields); $i++) {
            $index = $fields[$i];

            if (isset($category[$index]))
                $return[$index] = $category[$index];
        }
        $result = 'ok';
    }

    $return['result'] = $result;

    json_return_value($return);
}

$json_supported_actions['get_freebusy_lines'] = array();

function json_get_freebusy_lines() {
    $user_id = array_get_default($_REQUEST, 'user_id');
    $resource_id = array_get_default($_REQUEST, 'resource_id');
    $date_format = array_get_default($_REQUEST, 'date_format', 'js');
    $gmt = array_get_default($_REQUEST, 'gmt', false);

    $return = array();
    $lines = '';

    if (! empty($user_id)) {
		require_once('modules/vCals/vCal.php');
		$lines = vCal::get_freebusy_array($user_id, $gmt, $date_format);
    } elseif (! empty($resource_id)) {
        require_once('modules/Resources/Resource.php');
		$lines = Resource::get_free_busy_array($resource_id, $gmt, $date_format);
    }

    $return['result'] = $lines;

    json_return_value($return);
}

$json_supported_actions['set_activity_status'] = array();

function json_set_activity_status() {
    $model = array_get_default($_REQUEST, 'model');
    $record_id = array_get_default($_REQUEST, 'record_id');
    $status = array_get_default($_REQUEST, 'accept_status');
    $return = array();
    $result = 'ok';

    $user = ListQuery::quick_fetch('User', AppConfig::current_user_id());
    $bean = ListQuery::quick_fetch($model, $record_id);

    if ($user && $bean) {
        require_bean('Meeting');
        $upd = RowUpdate::for_result($bean);
        $upd->setSource('json', 'set_activity_status');
        Meeting::set_invite_status($upd, $user, $status);
    } else {
        $result = 'failure';
    }

    $return['result'] = $result;

    json_return_value($return);
}

$json_supported_actions['check_for_updates'] = array();

function json_check_for_updates() {
    require_once('modules/LicenseInfo/check_updates.php');
    require_once('modules/LicenseInfo/LicenseHistory.php');
    require_once('modules/LicenseInfo/SystemUpdateManager.php');

	if(! AppConfig::is_admin() || ! SystemUpdateManager::is_enabled()){
		json_bad_request('no_access');
	}

    global $mod_strings;
	global $pageController, $pageInstance, $themeInstance;
	AppConfig::init_page_controller();
	$themeInstance = $pageController->create_theme_instance();
	require_once('include/layout/IncludeManager.php');
	$pageInstance = new IncludeManager();

    $mod_strings = return_module_language(AppConfig::language(), 'LicenseInfo');
    $error = 0;
    $error_message = null;
    $license = new LicenseHistory();

    //clear system_updates table
    SystemUpdateManager::reset_updates();

    if ($license->retrieve_latest() === null) {
        $error = 1;
        $error_message = $mod_strings['LBL_LICENSE_MISSING'];
    } else {
    	$license->get_extensions();
        $status = $license->get_support_status();

        if (! empty($status['error'])) {
            $error = 1;
            $error_message = strip_tags($status ['text']);
        } else {
            $updates = check_for_updates();

            if (! is_array($updates)) {
                $error = 1;

                if ($updates == '96: No Updates' || $updates == 'No Updates') {
                    $error_message = $mod_strings['LBL_NO_UPDATES_ERROR'];
                } else {
                    $error_message = $updates;
                }

            }
        }
    }

    $updates_manager = new SystemUpdateManager();
    $result = $updates_manager->getUpdatesTable(true);

    $return = compact('error', 'error_message', 'result');
    json_return_value($return);
}

$json_supported_actions['move_calendar_activity'] = array();

function json_move_calendar_activity() {
	global $timedate;
    $module = array_get_default($_REQUEST, 'module');
    $record_id = array_get_default($_REQUEST, 'record');
    $date_start = array_get_default($_REQUEST, 'date_start');
	$time_start = array_get_default($_REQUEST, 'time_start');
    $all_day = array_get_default($_REQUEST, 'is_daylong');
    $duration = array_get_default($_REQUEST, 'duration');
    $result = 'failure';
    $update = [];

    $model = AppConfig::module_primary_bean($module);
    
	if($model == 'Call' || $model == 'Meeting' || $model == 'Task' || $model == 'BookedHours')
		$activity = ListQuery::quick_fetch($model, $record_id, true, array('acl_filters' => 'edit'));
	$field = ($model == 'Task' ? 'date_due' : 'date_start');
	if($model == 'BookedHours') {
		$dur_field = 'quantity';
		$bstatus = $activity ? $activity->getField('status') : null;
		if($all_day || $bstatus == 'approved' || $bstatus == 'submitted') {
			// skip
			$result = 'ok';
		}
	}
	else if($model == 'Meeting' || $model == 'Call')
		$dur_field = 'duration';
	else
		$dur_field = null;
	
    if (! empty($activity) && ($date_start || $time_start || $duration)) {
		$upd = RowUpdate::for_result($activity);
		$upd->setSource('json', 'move_calendar_activity');
		
		$local_from = $timedate->handle_offset($upd->getField($field), $timedate->get_db_date_time_format(), true);
		list($date_from, $time_from) = explode($local_from, ' ', 2);
		$have_time = false;
		if($time_start && preg_match('~^(\d{1,2}:\d{1,2})~', $time_start, $m)) {
			$time_start = $m[1].':00';
			$have_time = true;
		}
		if($date_start && preg_match('~^(\d{4}-\d{1,2}-\d{1,2} \d{1,2}:\d{1,2})~', $date_start, $m))
			$date_start = $m[1];
		else if(preg_match('~^(\d{4}-\d{1,2}-\d{1,2})~', $date_start, $m))
			$date_start = $m[1] . ' ' . ($have_time ? $time_start : $time_from);
		else if($have_time)
			$date_start = $date_from . ' ' . $time_start;
		else
			$date_start = null;

		if($date_start) {
			if ($all_day) {
				require_bean('Meeting');
				$daylong_param = Meeting::make_day_long_duration(null, $date_start);
				if($daylong_param) {
					$update[$field] = $daylong_param['date_start'];
					if($dur_field)
						$update[$dur_field] = $daylong_param['duration'];
					$update['is_daylong'] = 1;
				}
			} else {
				$conv = $timedate->handle_offset($date_start, $timedate->get_db_date_time_format(), false);
				if($conv) {
					$update[$field] = $conv;
					if($dur_field && $upd->getField('is_daylong'))
						$update[$dur_field] = 30;
					$update['is_daylong'] = 0;
				}
			}
        }
        if($dur_field) {
        	if($duration) {
				$update[$dur_field] = $duration;
				if(! $date_start) $date_start = $local_from; // for return value
			} else {
				$duration = $upd->getField($dur_field);
			}
		}
        
		if($update) {
			$upd->set($update);
			if($upd->validate() && $upd->save())
				$result = 'ok';
		}        
    } else
    	$date_start = null;

    $return = array(
        'result' => $result,
        'activity_id' => $record_id,
        'date_start' => $date_start,
        'duration' => $duration,
    );

    json_return_value($return);
}


$json_supported_actions['get_entity_address'] = array();

function json_get_entity_address() {
	$bean_name = array_get_default($_REQUEST, 'bean_name');
	if(! $bean_name) {
		$module = array_get_default($_REQUEST, 'module');
		$bean_name = AppConfig::module_primary_bean($module);
	}
	if(! $bean_name)
		return json_bad_request(array('error' => 'no_module'));
	$record = array_get_default($_REQUEST, 'record');
	if(! $record)
		return json_bad_request(array('error' => 'no_record'));
	$prefix = array_get_default($_REQUEST, 'prefix', '');
	
	$result = ListQuery::quick_fetch($bean_name, $record, array('id', $prefix . 'address'), array('acl_filters' => 'view', 'formatted' => 'json'));
	$ret = array('address' => '', 'current' => '');
	if($result) {
		$ret['address'] = $result->formatted[$prefix . 'address'];
	}
	
	require_once('modules/CompanyAddress/CompanyAddress.php');
	$result = ListQuery::quick_fetch_key('CompanyAddress', 'main', 1, array('id', 'address'), array('acl_filters' => 'view', 'formatted' => 'json'));
	if($result) {
		$ret['current'] = $result->formatted['address'];
	}
	
	json_return_value($ret);
}


$json_supported_actions['get_updated_teams'] = array();

function json_get_updated_teams() {
	$bean_name = array_get_default($_REQUEST, 'bean_name');
	if(! $bean_name) {
		$module = array_get_default($_REQUEST, 'module');
		$bean_name = AppConfig::module_primary_bean($module);
	}
	if(! $bean_name)
		return json_bad_request(array('error' => 'no_module'));
	$record = array_get_default($_REQUEST, 'record');
	if($record) {
		$row = ListQuery::quick_fetch($bean_name, $record);
		if($row)
			$rec = $row->getRowUpdate();
	} else {
		$rec = RowUpdate::blank_for_model($bean_name);
	}
	if($rec) {
		$rec->loadRequest();
		$upd = $rec->getInheritedTeams(true);
	} else
		$upd = null;
	if($upd) {
		$upd['add'] = array_values($upd['add']);
		$upd['remove'] = array_values($upd['remove']);
		foreach($upd['add'] as &$t)
			$t['_display'] = $t['name'];
		foreach($upd['remove'] as &$t)
			$t['_display'] = $t['name'];
	}
	$ret = array('teams' => $upd);
	json_return_value($ret);
}


$json_supported_actions['get_preload_files'] = array('login_required' => false, 'no_session' => true);

function json_get_preload_files() {
	require_once('include/layout/pages/BasePage.php');
	$page = new BasePage();
	$page->language = AppConfig::language();
	$page->add_standard_headers();
	$page->format_includes('head');
	$ret = array('includes' => $page->preload_includes);
	json_return_value($ret);
}


$json_supported_actions['get_country_states'] = array();
function json_get_country_states() {
	$states = include('include/language/country_states.php');
	if(! $states) $states = array();
	$ret = array();
	foreach($states as $c => $ss) {
		$ret[$c] = array('keys' => array_keys($ss), 'values' => array_values($ss));
	}
	json_return_value($ret);
}


function handle_date_filter($compact) {
	$module = array_get_default($_REQUEST, 'module');
	$bean = array_get_default($_REQUEST, 'bean_name');
	$field_name = array_get_default($_REQUEST, 'field_name');
	$filter = array_get_default($_REQUEST, 'filter');
	$context = array_get_default($_REQUEST, 'context');
	if($module && ! $bean) $bean = AppConfig::module_primary_bean($module);
	if(! $bean) return json_bad_request(array('error' => 'no_module'));
	require_once('include/database/DateRange.php');
	$range = new IAHDateRange($bean, $context);
	try {
		$ret = ['result' => $range->expandFilter($field_name, $filter, false, $compact)];
	} catch(IAHDateParseError $e) {
		$ret = ['error' => $e->getMessage()];
	}
	json_return_value($ret);
}

$json_supported_actions['expand_date_filter'] = array('login_required' => false);
function json_expand_date_filter() {
	return handle_date_filter(false);
}
$json_supported_actions['compact_date_filter'] = array('login_required' => false);
function json_compact_date_filter() {
	return handle_date_filter(true);
}

