<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');



$json_supported_actions['get_available_modules'] = array(
	'jsrd' => 'auto',
);
function json_get_available_modules() {
	global $jsonModuleList, $beanList;
	$mods = array();
	foreach($jsonModuleList as $m)
		if(isset($beanList[$m]))
			$mods[] = $m;
	sort($mods);
	json_auto_return($mods);
}


$json_supported_actions['get_module_fields'] = array(
	'jsrd' => 'auto',
);
function json_get_module_fields() {
	global $module, $beanList, $beanFiles, $output_format;
	if(empty($module))
		json_bad_request('missing_module');
	require_once($beanFiles[$beanList[$module]]);
	$seed = new $beanList[$module];
	$field_defs = IAH_JSRD::jsrd_format_field_defs($seed, $output_format == 'jsrd');
	json_return_value($field_defs);
}


$json_supported_actions['get_menu_structure'] = array(
	'jsrd' => 'auto',
);
function json_get_menu_structure() {
	require_once('include/GroupedTabs/GroupedTabStructure.php');
	$gt = new GroupedTabStructure();
	$grp = $gt->get_grouped_tabs();
	$flat = $gt->get_flat_tabs();
	$show_menu = AppConfig::setting('display.index.show_menu_create_button');
	foreach($grp as &$g) {
		$ms = array();
		foreach($g['modules'] as $k => $v) {
			if($v['module'] != $k) $v['id'] = $k;
			if(! empty($v['icon'])) $v['icon'] = 'theme-icon module-' . $v['icon'];
			if( ($b = AppConfig::module_primary_bean($v['module']))
			  && ACLController::checkAccess($v['module'], 'create', true)
			  && ! empty($show_menu[$b]))
				  $v['create'] = true;

			$ms[] = $v;
		}
		$g['modules'] = $ms;
	}
	$ret = array(
		'grouped' => array_values($grp),
		'flat' => array_values($flat),
	);
	json_return_value($ret);
}

/*
$json_supported_actions['get_search_fields'] = array();
function json_get_search_fields() {
	global $module;
	if(empty($module))
		json_bad_request('missing_module');
	require_once('include/SearchForm/SearchForm.php');
	$form = new SearchForm($module);
	$ret = array();
	$relevant = array('input_type', 'options_add_blank');
	foreach($form->searchFields as $k => $def) {
		$f = array('name' => $k);
		foreach($relevant as $r)
			if(isset($def[$r])) $f[$r] = $def[$r];
		$ret[] = $f;
	}
	json_return_value($ret);
}


function &_get_listview_column_defs($module) {
	global $beanList, $beanFiles;
	global $listViewDefs;
	if(! isset($beanList[$module]))
		return false;
	require_once($beanFiles[$beanList[$module]]);
	$seed = new $beanList[$module];
	$ret = array();
	$field_defs = IAH_JSRD::jsrd_format_field_defs($seed);
	if(count($field_defs))
		$ret[] = array('name' => 'id', 'label' => json_translate('LBL_ID'), 'type' => 'string', 'format' => 'id');
	require_once('include/ListView/ListViewDisplay.php');
	ListViewDisplay::loadListViewDefs($module);
	$relevant = array('width', 'link', 'module', 'id', 'default', 'sortable', 'dynamic_module', 'align');
	// 'icon' - used by Emails
	foreach($listViewDefs[$module] as $k => $def) {
		$name = strtolower($k);
		if($name == 'id')
			continue;
		$lbl = array_get_default($def, 'label');
		if($lbl) $lbl = json_translate($lbl, $module);
		$f = array('name' => $name, 'label' => $lbl);
		if(isset($field_defs[$name])) {
			$fdef = $field_defs[$name];
			$f['type'] = $fdef['type'];
			if(isset($fdef['format']))
				$f['format'] = $fdef['format'];
			if(isset($fdef['maxlen']))
				$f['maxlen'] = $fdef['maxlen'];
		}
		else {
			$f['type'] = 'string'; // TODO - check for missing field defs like created_by_name and modified_user_name
		}
		foreach($relevant as $r)
			if(isset($def[$r]))
				$f[$r] = $def[$r];
		$ret[] = $f;
	}
	return $ret;
}


$json_supported_actions['get_listview_columns'] = array(
	'jsrd' => 'explicit',
	'schema' => array(
		'params' => array(
			array('name' => 'module', 'type' => 'string', 'optional' => true),
		),
		'output' => array(
			'type' => 'sequence',
			'fields' => array(
				array('name' => 'name', 'type' => 'string', 'nullable' => false),
				array('name' => 'label', 'type' => 'string', 'nullable' => false),
				array('name' => 'type', 'type' => 'string', 'nullable' => false),
			),
			'extra_fields' => array(
				array('name' => 'format', 'type' => 'string'),
				array('name' => 'maxlen', 'type' => 'integer'),
				array('name' => 'width', 'type' => 'string'),
				array('name' => 'link', 'type' => 'boolean'),
				array('name' => 'module', 'type' => 'string'),
				array('name' => 'id', 'type' => 'string'),
				array('name' => 'default', 'type' => 'boolean'),
				array('name' => 'sortable', 'type' => 'boolean'),
				array('name' => 'dynamic_module', 'type' => 'string'),
				array('name' => 'align', 'type' => 'string'),
			),
		),
	),
);
function json_get_listview_columns() {
	global $module;
	$ret =& _get_listview_column_defs($module);
	json_auto_return($ret);
}*/


/*
function _all_subpanel_field_attrs() {
	require_once('include/SubPanel/SubPanelDefinitions.php');
	global $beanList, $beanFiles, $jsonModuleList;
	$fs = array();
	foreach($jsonModuleList as $m) {
		if(! isset($beanList[$m]) || ! isset($beanFiles[$beanList[$m]]))
			continue;
		require_once($beanFiles[$beanList[$m]]);
		$seed = new $beanList[$m];
		$subpanel_defs = new SubPanelDefinitions($seed);
		$tabs = $subpanel_defs->get_available_tabs();
		foreach($tabs as $ord => $panel_name) {
			$panel = $subpanel_defs->load_subpanel($panel_name);
			$fields = $panel->get_list_fields();
			foreach($fields as $fid => $fdef) {
				foreach($fdef as $k => $v)
					$fs[$k] = 1;
			}
		}
	}
	json_return_value($fs);
}
*/


function &_get_subpanel_list_fields(&$panel, $extended=true, $collection=false) {
	static $fcopy = array('sortable', 'length_limit');
	static $mcopy = array('format', 'maxlen', 'nullable');

	$ret = array();
	$panel_module = $panel->get_module_name();
	$list_fields = $panel->get_list_fields();
	if(is_countable($list_fields) && count($list_fields)) {
		$ret[] = array('name' => 'id', 'label' => json_translate('LBL_ID'), 'type' => 'string', 'format' => 'id');
		if($collection)
			$ret[] = array('name' => 'module_name', 'type' => 'string');
	}
	if(! $extended) {
		$ret = array_keys($list_fields);
		return $ret;
	}
	$mod_defs = IAH_JSRD::jsrd_format_field_defs($panel->template_instance);
	foreach($list_fields as $fid => $fdef) {
		if($fid == 'id')
			continue;
		$mod_def = array_get_default($mod_defs, $fid);
		$f = array(
			'name' => $fid,
			'type' => $mod_def ? $mod_def['type'] : 'string'
		);
		if($mod_def) {
			foreach($mcopy as $fc)
				if(isset($mod_def[$fc]))
					$f[$fc] = $mod_def[$fc];
		}
		$lbl = array_get_default($fdef, 'vname');
		if($lbl)
			$f['label'] = json_translate($lbl, $panel_module);
		else if(isset($mod_def['label']))
			$f['label'] = $mod_def['label'];
		$width = array_get_default($fdef, 'width');
		if($width && preg_match('~^(\d+)~', $width, $m))
			$f['width'] = (int)$m[1];
		$usage = array_get_default($fdef, 'usage');
		if($usage == 'query_only')
			$f['display'] = false;
		foreach($fcopy as $fc) {
			if(isset($fdef[$fc]))
				$f[$fc] = $fdef[$fc];
		}
		$widget = array_get_default($fdef, 'widget_class');
		if($widget == 'SubPanelDetailViewLink') {
			$f['link'] = true;
			$v = array_get_default($fdef, 'target_module');
			if($v)
				$f['module'] = $v;
			$v = array_get_default($fdef, 'target_module_key');
			if($v)
				$f['dynamic_module'] = $v;
			$v = array_get_default($fdef, 'target_record_key');
			if($v)
				$f['id'] = $v;
		}
		else if($widget == 'SubPanelEmailLink')
			$f['format'] = 'email';
		else if($widget == 'SubPanelEditButton' || $widget == 'SubPanelRemoveButton' || $widget == 'SubPanelDeleteButton'
		|| $widget == 'SubPanelIcon')
			continue; // ?
		else if($widget) $f['class'] = $widget;
		$ret[] = $f;
	}
	return $ret;
}


function _get_subpanel(&$seed, $panel_name) {
	require_once('include/SubPanel/SubPanelDefinitions.php');
	$subpanel_defs = new SubPanelDefinitions($seed);
	$tabs = $subpanel_defs->get_available_tabs();	
	if(! in_array($panel_name, $tabs))
		return false;
	$ret = array();	
	$panel = $subpanel_defs->load_subpanel($panel_name);
	return $panel;
}


// private
function _get_subpanels(&$seed, $extended=true) {
	require_once('include/SubPanel/SubPanelDefinitions.php');
	$subpanel_defs = new SubPanelDefinitions($seed);
	$tabs = $subpanel_defs->get_available_tabs();	
	$ret = array();
	
	foreach($tabs as $ord => $panel_name) {
		$panel = $subpanel_defs->load_subpanel($panel_name);
		$panel_module = $panel->get_module_name();
		$def = array(
			'name' => $panel_name,
			'module_name' => $panel_module,
			'order' => $ord,
			'title' => $panel->get_title(),
			//'force_display' => $panel->get_force_display(),
		);
		$buttons = $panel->get_buttons();
		if( ($collect = $panel->isCollection()) ) {
			if(! is_countable($panel->sub_subpanels) || ! count($panel->sub_subpanels))
				continue;
			$panel = current($panel->sub_subpanels);
		}
		$def['fields'] =& _get_subpanel_list_fields($panel, $extended, $collect);
		//$def['fields'] = $fields;
		//$def['buttons'] = $buttons;
		$ret[$panel_name] = $def;
	}
	return $ret;
}


$json_supported_actions['get_subpanel_info'] = array();
function json_get_subpanel_info() {
	global $module, $beanList, $beanFiles, $jsrd_minimal;
	$ret = array();
	if(! isset($beanList[$module]) || ! isset($beanFiles[$beanList[$module]])) {
		json_return_value($ret);
		sugar_cleanup(true);
		return;
	}
	require_once($beanFiles[$beanList[$module]]);
	$seed = new $beanList[$module];
	$ret = array_values(_get_subpanels($seed, ! $jsrd_minimal));
	json_return_value($ret);
}


$json_supported_actions['get_dropdown_list'] = array(
	'login_required' => false,
);
function json_get_dropdown_list() {
	$mod = array_get_default($_REQUEST, 'module', '');
	$include_blank = array_get_default($_REQUEST, 'include_blank', false);
	$dd_name = array_get_default($_REQUEST, 'dropdown_name');
	if(! empty($dd_name))
		$optlist = json_get_option_list($dd_name, $mod, true, ! $include_blank);
	else
		$optlist = null;
	if(empty($optlist))
		json_bad_request('invalid_dropdown');
	json_return_value($optlist);
}


$json_supported_actions['get_dropdown_lists'] = array(
	'login_required' => false,
);
function json_get_dropdown_lists() {
	$mod = array_get_default($_REQUEST, 'module', '');
	$include_blank = array_get_default($_REQUEST, 'include_blank', false);
	$dd_names = array_get_default($_REQUEST, 'dropdown_names');
	$ret = array();
	foreach($dd_names as $dd_name) {
		if(! empty($dd_name))
			$optlist = json_get_option_list($dd_name, $mod, true, ! $include_blank);
		else
			$optlist = null;
		if(empty($optlist))
			$optlist = array();
		$ret[] = array('name' => $dd_name, 'values' => $optlist);
	}
	json_return_value($ret);
}


$json_supported_actions['get_currency_list'] = array(
);
function json_get_currency_list() {
	require_once('modules/Currencies/ListCurrency.php');
	$lst = new ListCurrency();
	$lst->lookupCurrencies();
	$ret = array();
	if(!empty($lst->list)) {
		$rows = array();
		foreach($lst->list as $currency)
			if($currency->status == 'Active')
				$rows[] = $currency;
		$ret = json_serialize_list($lst->list[0], $rows);
	}
	json_return_value($ret);
}


?>
