<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/
require_once('include/json/portal_utils.php');
require_once('modules/Accounts/Account.php');
require_once('modules/Contacts/Contact.php');
require_once('modules/Leads/Lead.php');
require_once('modules/Users/User.php');
require_once('modules/Opportunities/Opportunity.php');
include_once('modules/Notes/Note.php');
include_once('include/upload_file.php');
include_once('include/utils/html_utils.php');


global $invalid_contact_fields, $valid_modules_for_contact;
$invalid_contact_fields = array(
	'portal_name'=>1, 'portal_password'=>1, 'portal_active'=>1,
);
$valid_modules_for_contact = array(
	'Contacts'=>1, 'Cases'=>1, 'Notes'=>1, 'Bugs'=>1, 'Accounts'=>1,
	'Project' => 1, 'Documents' => 1, 'SalesOrders' =>1, 'Invoice' => 1, 'Quotes' => 1, 'SalesOrders' => 1,
	'EventSessions' => 1,
	'ProjectTask' => 1,
);


$valid_partner_modules = array(
	'Opportunities' => 1,
	'Leads' => 1,
);

$json_supported_actions['portal_login'] = array(
	'login_required' => false,
	//array('portal_auth'=>'tns:user_auth','user_name'=>'xsd:string', 'application_name'=>'xsd:string'),
);

function json_portal_login() {
    global $current_user;
    $portal_auth = array_get_default($_REQUEST, 'portal_auth');
    if(! is_array($portal_auth))
    	return json_bad_request(array('error' => 'missing_portal_auth'));
    $user_name = array_get_default($_REQUEST, 'user_name', '');
    $application_name = array_get_default($_REQUEST, 'application_name', '');

	$result = portal_login_user($portal_auth);
    if($result != 'success'){
        if($result == 'sessions_exceeded')
            return json_bad_request(array('error' => 'sessions_exceeded'));
        else
            return json_bad_request(array('error' => 'no_portal'));
    }
    $ok = false;
    // Initialize the first name / last name
    $arExtraDetails = array (
		'first_name' => '',    
		'last_name' => '',    
		'email_address' => '',    
		'contact_id' => '',    
		'account_id' => '',    
		'lead_id' => '',
		'email_accounts' => 0,
    );

	if($user_name == 'lead' || $user_name == 'portal') {
		$ok = true;
	} else {
		$contact = new Contact();
		$contact = $contact->retrieve_by_string_fields(array('portal_name'=>$user_name, 'portal_active'=>'1', 'deleted'=>0) );
		
		if($contact != null){
			$_SESSION['contact_id'] = $contact->id;
			$_SESSION['assigned_user_id'] = $contact->assigned_user_id;
			$_SESSION['first_name'] = $arExtraDetails['first_name'] = $contact->first_name;
			$_SESSION['last_name'] = $arExtraDetails['last_name'] = $contact->last_name;
			$_SESSION['account_id'] = $contact->primary_account_id;
			$arExtraDetails['email_address'] = $contact->email1;
			$arExtraDetails['account_id'] = $contact->primary_account_id;
			$arExtraDetails['contact_id'] = $contact->id;
			$arExtraDetails['user_id'] = $current_user->id;
			$arExtraDetails['email_accounts'] = $contact->email_accounts;
			$user_name = 'contact';
			portal_build_relationship_tree($contact);
			$ok = true;
		}
		else {
			require_once('modules/Leads/Lead.php');
			$lead = new Lead;
			$lead = $lead->retrieve_by_string_fields(array('portal_name'=>$user_name, 'portal_active'=>'1', 'deleted'=>0) );
			if($lead != null){
				$_SESSION['first_name'] = $arExtraDetails['first_name'] = $lead->first_name;
				$_SESSION['last_name'] = $arExtraDetails['last_name'] = $lead->last_name;
				$arExtraDetails['email_address'] = $lead->email1;
				$arExtraDetails['lead_id'] = $lead->id;
				$arExtraDetails['user_id'] = $current_user->id;
				$_SESSION['lead_id'] = $lead->id;
				$_SESSION['registered_lead'] = true;
				$_SESSION['assigned_user_id'] = $lead->assigned_user_id;
				$user_name = 'lead';
				$ok = true;
			}
		}
	}
	if($ok) {
		$_SESSION['is_valid_session'] = true;
		$_SESSION['ip_address'] = $_SERVER['REMOTE_ADDR'];
		$_SESSION['portal_id'] = $current_user->id;
		$_SESSION['type'] = $user_name;
		$_SESSION['login_type'] = 'portal';
		$ret = array('result' => 'ok', 'json_session_id' => session_id(), 'type' => $_SESSION['type'], 'extra_details' => $arExtraDetails);
		return json_return_value($ret);
	}
	else
		return json_bad_request(array('error'=> 'invalid_login'));
}


$json_supported_actions['portal_set_entry'] = array(
	'login_required' => 'portal',
);
function json_portal_set_entry() {
	// Get access to globals
	global $module;

	$get_ids = get_portal_module_ids($module);
	if($get_ids === false) {
		return json_bad_request('No access');
	}

    $update = [];
    $request = array_get_default($_REQUEST, 'name_value_list', array());

    foreach($request as $strKey => $strValue) {
        $update[$strKey] = json_clean_input_value($strValue);
    }
    
    if(! empty($update['id'])) {
    	if($get_ids !== true && ! in_array($update['id'], $get_ids))
    		$update = [];
    }

    $strId = null;
    $bIsUpdate = false;
    $upd = null;

    if (sizeof($update) > 0) {
        $model = AppConfig::module_primary_bean($module);

        if(! empty($update['id'])) {
            $result = ListQuery::quick_fetch($model, $update['id'], true, array('acl_filters' => array('edit')));
            if($result) $upd = RowUpdate::for_result($result);
            unset($update['id']);
            $bIsUpdate = true;
        } else {
            if(ACLController::checkAccess($module, 'create')) {
                $upd = RowUpdate::blank_for_model($model);
            }
        }

        $strId = $upd->orig_rows;
        if(! empty($upd)) {
        	$upd->setSource('json', 'portal_set_entry');

            if (! $bIsUpdate) {
                if(isset($_SESSION['assigned_user_id']) && empty($update['assigned_user_id']))
                    $update['assigned_user_id'] = $_SESSION['assigned_user_id'];
                if(isset($_SESSION['account_id']) && ! array_key_exists('account_id', $update))
                    $update['account_id'] = $_SESSION['account_id'];
                if ($module == 'Cases')
                    $update['cust_contact_id'] = $_SESSION['contact_id'];

                // Indicate that this was created via the portal
                $update['portal_flag'] = 1;

                if($module == 'Cases') {
                    $user_id = AppConfig::setting('company.case_queue_user');
                    $user_name = get_assigned_user_name($user_id);
                    if (! empty($user_name)) {
                        $update['assigned_user_id'] = $user_id;
                    }
                }

                if($_SESSION['type'] == 'contact' && $module != 'Contacts'){
                    if(isset($_SESSION['contact_id']))
                        $update['contact_id'] = $_SESSION['contact_id'];
                }

                if (empty($update['assigned_user_id']))
                    $update['assigned_user_id'] = AppConfig::current_user_id();
            }

            $upd->set($update);

            if($upd->validate() && $upd->save()) {
                $strId = $upd->getPrimaryKeyValue();
                if (! $bIsUpdate && isset($_SESSION['contact_id']))
                    $upd->addUpdateLinks('contacts', $_SESSION['contact_id']);
            }
        }
    }

    // If a new record, add the id to the list
    if (! $bIsUpdate && $strId) {
        if (! empty($_SESSION['viewable'][$module])) {
            // Add to the list
            $_SESSION['viewable'][$module][$strId] = $strId;
        } else {
            $_SESSION['viewable'][$module] = array(
                $strId => $strId,
            );
        }
    }

    $project_id = $upd ? $upd->getField('project_id', '') : '';

    // Set the return array
	$arReturn = array (
		'id' => $strId,
		'project_id' => $project_id,
	);
	// Return result
	json_return_value($arReturn);
}

$json_supported_actions['portal_get_entry'] = array(
	'login_required' => 'portal',
);
function json_portal_get_entry(){
    // Get access to globals
	global  $valid_modules_for_contact, $module, $db;
    $record = array_get_default($_REQUEST, 'id');

	// Make sure the module is valid
	if(empty($record)) {
		// Indicate the error
		return json_bad_request("No record id specified");
	}
	
	$get_ids = get_portal_module_ids($module);
	if($get_ids === false){
		// Indicate the error
		return json_bad_request("No access");
	}

	$model = AppConfig::module_primary_bean($module);
    $field_list = array_get_default($_REQUEST, 'field_list');
    if (! $field_list)
        $field_list = true;
    
	$result = null;
    if($get_ids && ($get_ids === true || in_array($record, $get_ids))) {
		$params = array('acl_filters' => array('view'));
		$lq = new ListQuery($model, $field_list, $params);
		if ($field_list === true) {
			$lq->addFields($lq->getAvailableFields());
		}
		$result = $lq->queryRecord($record);
		if (!$result || $result->failed)
			$result = null;
	}

    if($result) {
        require_once('include/layout/FieldFormatter.php');
        $fmt = new FieldFormatter('json');
        $arReturn = $fmt->formatRow($result->fields, $result->row);
    } else {
        return json_forbidden();
    }

	$arReturn['history'] = array(
		'emails' => array(),
		'notes' => array(),		
	);

	// Get the email history
	$strQuery = "
		SELECT
			*,
			UNIX_TIMESTAMP(date_entered) as note_date_entered
		FROM
			notes
		WHERE
			parent_id = '" .$db->quote($record). "'
		AND
			portal_flag = 1
		ORDER BY
			date_entered
	";

	$objResult = $db->query($strQuery);

	while ($arRow = $db->fetchByAssoc($objResult)) {
		// Add the case record
		$arReturn['history']['notes'][] = $arRow;
	}

	// If this is the cases module, return the notes and emails history
	if ($module == 'Cases') {
		// Get the email history
		$strQuery = "
			SELECT
				E.id,
				E.name,
				E.to_addrs,
				E.from_addr,
				B.description,
				B.description_html,
				UNIX_TIMESTAMP(E.date_start) as email_start_date
			FROM
				emails E
			LEFT JOIN
				emails_bodies B
			ON
				E.id = B.email_id
			LEFT JOIN
				emails_cases L
			ON
				E.id = L.email_id
			WHERE
				L.case_id = '" .$db->quote($record). "'
			ORDER BY
				E.date_start
		";
		// Execute the query
		$objResult = $db->query($strQuery);
		// Loop through and process the results
		while ($arRow = $db->fetchByAssoc($objResult)) {
			// Description set?
			if (empty($arRow['description'])) {
				// Generate from html version
				$arRow['description'] = html2plaintext(html_entity_decode($arRow['description_html']));
			}
			// Convert newlines to breaks
			$arRow['description'] = str_replace("\n", "<br/>", $arRow['description']);
			// Don't pass back the html
			unset($arRow['description_html']);
			// Get the list of attachments
			$arRow['attachments'] = array();
			// Get the email history
			$strQuery = "
				SELECT
					*,
					UNIX_TIMESTAMP(date_entered) as note_date_entered
				FROM
					notes
				WHERE
					parent_id = '" .$db->quote($arRow['id']). "'
				ORDER BY
					date_entered
			";
			// Execute the query
			$objAttachments = $db->query($strQuery);
			// Loop through and process the results
			while ($arAttachment = $db->fetchByAssoc($objAttachments)) {
				// Add the case record
				$arRow['attachments'][] = $arAttachment;
			}
			// Add the case record
			$arReturn['history']['emails'][] = $arRow;
		}
	}
	// Return the data
	json_return_value($arReturn);
}

$json_supported_actions['portal_get_related_notes'] = array(
	'login_required' => 'portal',
);
function json_portal_get_related_notes(){
	// Get access to globals
	global  $db, $beanList, $beanFiles, $valid_modules_for_contact, $module;
	global $valid_partner_modules;
	// Make sure the module is valid
	if(empty($beanList[$_REQUEST['parent_type']])) {
		// Indicate the error
		json_server_error("No matching module");
		// Nothing else to do
		return false;
	}
	// If a lead, only allow lead access
	if($_SESSION['type'] == 'lead' && $module != 'Leads'){
		// Indicate the error
		json_server_error("No access allowed");
		// Nothing else to do
		return false;
	}
	// Ensure that we're accessing a valid module
	if (($_SESSION['type'] == 'contact') && !key_exists($_REQUEST['parent_type'], $valid_modules_for_contact)) {
		if (empty($_SESSION['is_partner']) || !isset($valid_partner_modules[$module])) {
			// Indicate the error
			json_server_error("No access allowed");
			// Nothing else to do
			return false;
		}
	}
	// Generate the query
	$strQuery = "
		SELECT
			n.id,
			n.date_entered,
			n.name,
			n.filename,
			n.file_mime_type,
			n.description,
			u.first_name,
			u.last_name
		FROM
			notes n
		LEFT JOIN users u
			ON 
				u.id = n.created_by
		WHERE
			parent_type = '{$_REQUEST['parent_type']}'
		AND
			parent_id = '{$_REQUEST['parent_id']}'
		AND
			portal_flag = 1
	";
	// Initialize the notes array
	$arNotes = array();
	// Execute the query
	$objResult = $db->query($strQuery);
	// Loop through and process the results
	while (($arRow = $db->fetchByAssoc($objResult)) != null) {
		$arRow['created_by_name'] = $GLOBALS['locale']->getLocaleFormattedName($arRow['first_name'], $arRow['last_name'], '');
		// Add the case record
		$arNotes[] = $arRow;
	}
    // Generate the return array
    $arReturn = array (
    	'total_count' => count($arNotes),
    	'list' => $arNotes,
    );
	// Return the data
	json_return_value($arReturn);
}

$json_supported_actions['portal_get_file_list'] = array(
	'login_required' => 'portal',
);
function json_portal_get_file_list() {
	// Get access to the global variables
	global $db;
    // Set the start record
    if (empty($_REQUEST['start_record'])) {
    	// Start record not specified - set to 0
    	$_REQUEST['start_record'] = 0;
    }
    // Set the number of records
    if (empty($_REQUEST['num_records'])) {
    	// Filter not specified - set to 10
    	$_REQUEST['num_records'] = 10;
    }
	// Combine the contacts and accounts ids
	$arIds = $_SESSION['viewable']['Accounts'];
	// Add the contact id
	$arIds[] = $_SESSION['contact_id'];
	// Generate the query
	$strQuery = "
		SELECT
			D.id,
			D.document_name as name,
			D.active_date,
			D.exp_date,
			D.description,
			D.category_id,
			D.subcategory_id,
			R.filename,
			R.revision,
			DR.relation_id
		FROM
			document_relations DR
		LEFT JOIN
			documents D
		ON
			DR.document_id = D.id
		LEFT JOIN
			document_revisions R
		ON
			D.document_revision_id = R.id
		WHERE
			D.deleted = 0
		AND
			DR.relation_id IN ('" . implode("','", $arIds) . "')
		LIMIT
			{$_REQUEST['start_record']}, {$_REQUEST['num_records']}
	";
	// Initialize the releases array
	$arFiles = array();
	// Execute the query
	$objResult = $db->query($strQuery);
	// Loop through and process the results
	while (($arRow = $db->fetchByAssoc($objResult)) != null) {
		// Add the case record
		$arFiles[] = $arRow;
	}
	// Generate the query
	$strQuery = "
		SELECT
			count(*) as record_count
		FROM
			document_relations DR
		LEFT JOIN
			documents D
		ON
			DR.document_id = D.id
		LEFT JOIN
			document_revisions R
		ON
			D.document_revision_id = R.id
		WHERE
			D.deleted = 0
		AND
			DR.relation_id IN ('" . implode("','", $arIds) . "')
	";
	// Execute the query
	$objResult = $db->query($strQuery);
	// Loop through and process the results
	if ($arRow = $db->fetchByAssoc($objResult)) {
		// Add the case record
		$intRecordCount = $arRow['record_count'];
	}
    // Generate the return array
    $arReturn = array (
    	'total_count' => $intRecordCount,
    	'list' => $arFiles,
    );
	// Return the value
	return json_return_value($arReturn);
}

$json_supported_actions['portal_get_entry_list'] = array(
	'login_required' => 'portal',
);
function json_portal_get_entry_list() {
	// Get access to the global variables
	global $module;

    // Lead users not allowed access
	if($_SESSION['type'] == 'lead' && $module != 'EventSessions') {
		return json_bad_request("No access allowed");
	}
	
	$get_ids = get_portal_module_ids($module);
	if($get_ids === false) {
		return json_bad_request("No matching module");
    }
    
    $where = array_get_default($_REQUEST, 'where');
    $order_by = array_get_default($_REQUEST, 'order_by');
    $start_record = array_get_default($_REQUEST, 'start_record', 0);
    $num_records = array_get_default($_REQUEST, 'num_records', 10);
    $select_fields = array_get_default($_REQUEST, 'select_fields');
    if (! $select_fields)
        $select_fields = true;
    $list = array();
    $total = 0;

	if ($get_ids) {
		$lq = new ListQuery(AppConfig::module_primary_bean($module), $select_fields, array('acl_filters' => array('view')));
		if ($select_fields === true)
			$lq->addFields($lq->getAvailableFields());
		if ($get_ids !== true)
			$lq->addFilterPrimaryKey($get_ids);
		if ($where) {
			if (is_array($where))
				$lq->addFilterClause($where);
			else
				$lq->where = $where;
		}

		$result = $lq->runQuery($start_record, $num_records, true, $order_by);

		if($result && ! $result->failed) {
			require_once('include/layout/FieldFormatter.php');
			$fmt = new FieldFormatter('json');
			$formatted_list = $fmt->formatRows($result->fields, $result->rows);

			if (! $get_ids || $get_ids === true)
				$get_ids = array_keys($formatted_list);

			if (sizeof($formatted_list > 0)) {
				foreach ($formatted_list as $key => $row) {
					$list[] = $row;
				}
			}

			$total = $result->total_count;
		}
	}

    if ($module == 'EventSessions') {
        require_once('modules/EventSessions/json.php');
        portal_add_registration_info($list);
    }

    $arReturn = array (
    	'total_count' => $total,
		'list' => $list,
		'get_ids' => "('" .implode("', '", $get_ids). "')",
    );
    json_return_value($arReturn);
}

$json_supported_actions['portal_check_registration'] = array(
	'login_required' => false,
);
function json_portal_check_registration() {
	// Ensure that the admin user is valid
	$objAdminUser = new User();
	// Do we have a valid user?
	if (!$objAdminUser->retrieve_by_string_fields(array('user_name' => $_REQUEST['admin_user'], 'user_hash' => $_REQUEST['admin_pass'], 'is_admin' => 1, 'status' => 'Active', 'deleted' => 0))) {
		// Indicate the error
		return json_bad_request(array('error'=> 'invalid_admin_user'));
	}
	// Create the contact object
	$objContact = new Contact();
	// Check for a matching contact that isn't yet active
	if($objContact = $objContact->retrieve_by_string_fields(array('email1' => $_REQUEST['email_address'], 'portal_active' => '0', 'deleted' => 0))) {
		// Set the return array
		$arReturn = array (
			'id' => $objContact->id,
			'first_name' => $objContact->first_name,
			'last_name' => $objContact->last_name,
			'email_address' => $objContact->email1,
		);
		// Return the contact details
		return json_return_value($arReturn);
	}
	// No matching contact found
	return json_bad_request(array('error'=> 'no_matching_email'));
}

$json_supported_actions['portal_create_user'] = array(
	'login_required' => false,
);
function json_portal_create_user() {
	// Get access to globals
	global $db;
	// Ensure that the admin user is valid
	$objAdminUser = new User();
	// Do we have a valid user?
	if (!$objAdminUser->retrieve_by_string_fields(array('user_name' => $_REQUEST['admin_user'], 'user_hash' => $_REQUEST['admin_pass'], 'is_admin' => 1, 'status' => 'Active', 'deleted' => 0))) {
		// Indicate the error
		return json_bad_request(array('error'=> 'invalid_admin_user'));
	}
	// Determine the user type
	switch ($_REQUEST['type']) {
		// Lead  convert and create contact
		case 'lead':
			// Retrieve the lead
			$objLead = new Lead();
			// Retrieve record
			if (!$objLead->retrieve($_REQUEST['id'])) {
				// Error retrieving lead
				return json_bad_request(array('error'=> 'error_finding_lead'));
			}
			// First of all, create the user account
			$objUser = new User();
			// Create the password
			$strPassword = generateRandomPassword();
			// Create the username
			if (!$strUsername = getUniqueUsername($objLead->first_name, $objLead->last_name)) {
				// Error getting username
				return json_bad_request(array('error'=> 'error_finding_unique_user'));
			}
			// Set the user info
			$objUser->first_name = $objLead->first_name;
			$objUser->last_name = $objLead->last_name;
			$objUser->user_name = "_p_" . $strUsername;
			$objUser->status = "Active";
			$objUser->portal_only = "1";
			$objUser->user_hash = md5($strPassword);
			$objUser->email1 = $objLead->email1;
			// Save the user record
			if (!$objUser->save()) {
				// Error creating account
				return json_bad_request(array('error'=> 'error_creating_user'));
			}
			// Initialize account id
			$strAccountId = '';
			// Is there an account associated with the lead?
			if (($_REQUEST['account_id'] == '-1') && !empty($objLead->account_name)) {
				// Create the account first
				$objAccount = new Account();
				// Set the account info
				$objAccount->assigned_user_id = $objAdminUser->id;
				$objAccount->name = $objLead->account_name;
				$objAccount->phone_office = $objLead->phone_work;
				$objAccount->phone_fax = $objLead->phone_fax;
				$objAccount->email1 = $objLead->email1;
				$objAccount->billing_address_street = $objAccount->shipping_address_street = $objLead->primary_address_street;
				$objAccount->billing_address_city = $objAccount->shipping_address_city = $objLead->primary_address_city;
				$objAccount->billing_address_state = $objAccount->shipping_address_state = $objLead->primary_address_state;
				$objAccount->billing_address_country = $objAccount->shipping_address_country = $objLead->primary_address_country;
				// Create the account
				if (!$strAccountId = $objAccount->save()) {
					// Error creating account
					return json_bad_request(array('error'=> 'error_creating_account'));
				}
			} elseif (!empty($_REQUEST['account_id']) && ($_REQUEST['account_id'] != '-1')) {
				// User associated lead with existing account
				$strAccountId = $_REQUEST['account_id'];
			}
			// Create the contact object
			$objContact = new Contact();
			// Create the contact
			$objContact->assigned_user_id = $objAdminUser->id;
			$objContact->primary_account_id = $strAccountId;
			$objContact->first_name = $objLead->first_name;
			$objContact->last_name = $objLead->last_name;
			$objContact->email1 = $objLead->email1;
			$objContact->phone_work = $objLead->phone_work;
			$objContact->phone_fax = $objLead->phone_fax;
			$objContact->portal_name = '_p_' . $strUsername;
			$objContact->portal_active = 1;
			$objContact->primary_address_street = $objContact->alt_address_street = $objLead->primary_address_street;
			$objContact->primary_address_city = $objContact->alt_address_city = $objLead->primary_address_city;
			$objContact->primary_address_state = $objContact->alt_address_state = $objLead->primary_address_state;
			$objContact->primary_address_country = $objContact->alt_address_country = $objLead->primary_address_country;
			// Save the contact
			if (!$strContactId = $objContact->save()) {
				// Error creating account
				return json_bad_request(array('error'=> 'error_creating_contact'));
			}
			// Update and save the lead
			$objLead->contact_id = $strContactId;
			$objLead->account_id = $strAccountId;
			$objLead->status = 'Converted';
			$objLead->converted = 1;
			// Save the lead
			$objLead->save();
			// Create the return array
			$arReturn = array (
				'first_name' => $objLead->first_name,
				'last_name' => $objLead->last_name,
				'email_address' => $objLead->email1,
				'username' => $strUsername,
				'password' => $strPassword,
			);
			// Return the value
			return json_return_value($arReturn);
			break;
			
		// Contact
		case 'contact':
			// Retrieve the lead
			$objContact = new Contact();
			// Retrieve record
			if (!$objContact->retrieve($_REQUEST['id'])) {
				// Error retrieving lead
				return json_bad_request(array('error'=> 'error_finding_contact'));
			}
			// First of all, create the user account
			$objUser = new User();
			// Create the password
			$strPassword = generateRandomPassword();
			// Create the username
			if (!$strUsername = getUniqueUsername($objContact->first_name, $objContact->last_name)) {
				// Error getting username
				return json_bad_request(array('error'=> 'error_finding_unique_user'));
			}
			// Set the user info
			$objUser->first_name = $objContact->first_name;
			$objUser->last_name = $objContact->last_name;
			$objUser->user_name = "_p_" . $strUsername;
			$objUser->status = "Active";
			$objUser->portal_only = "1";
			$objUser->user_hash = md5($strPassword);
			$objUser->email1 = $objContact->email1;
			// Save the user record
			if (!$objUser->save()) {
				// Error creating account
				return json_bad_request(array('error'=> 'error_creating_user'));
			}
			if (!empty($_REQUEST['account_id']) && ($_REQUEST['account_id'] != '-1')) {
				// User associated lead with existing account
				$objContact->primary_account_id = $_REQUEST['account_id'];
			}
			// Update and save the contact
			$objContact->portal_active = 1;
			$objContact->portal_name = "_p_{$strUsername}";
			// Save the contact
			$objContact->save();
			// Create the return array
			$arReturn = array (
				'first_name' => $objContact->first_name,
				'last_name' => $objContact->last_name,
				'email_address' => $objContact->email1,
				'username' => $strUsername,
				'password' => $strPassword,
				'id' => $objUser->id,
			);
			// Return the value
			return json_return_value($arReturn);
			break;
	}
	// Invalid type
	return json_bad_request(array('error'=> 'invalid_record_type'));
}

$json_supported_actions['portal_retrieve_password'] = array(
	'login_required' => false,
);
function json_portal_retrieve_password() {
	// Get access to globals
	global $db;
	// Ensure that the admin user is valid
	$objAdminUser = new User();
	// Do we have a valid user?
	if (!$objAdminUser->retrieve_by_string_fields(array('user_name' => $_REQUEST['admin_user'], 'user_hash' => $_REQUEST['admin_pass'], 'is_admin' => 1, 'status' => 'Active', 'deleted' => 0))) {
		// Indicate the error
		return json_bad_request(array('error'=> 'invalid_admin_user'));
	}
	// First of all, create the user object
	$objUser = new User();
	// Attempt to find a matching user
	if (!$objUser->retrieve_by_string_fields(array('email1' => $_REQUEST['email_address'], 'portal_only' => '1', 'status' => 'Active', 'deleted' => 0))) {
		// Indicate the error
		return json_bad_request(array('error'=> 'no_matching_email_address'));
	}
	// Create the password
	$strPassword = generateRandomPassword();
	// Update the password
	$objUser->user_hash = md5($strPassword);
	// Save the user
	if (!$objUser->save()) {
		// Indicate the error
		return json_bad_request(array('error'=> 'error_updating_user'));
	}
	// Create the return array
	$arReturn = array (
		'first_name' => $objUser->first_name,
		'username' => $objUser->user_name,
		'password' => $strPassword,
	);
	// Return the value
	return json_return_value($arReturn);
}

$json_supported_actions['portal_create_lead'] = array(
	'login_required' => false,
);
function json_portal_create_lead() {
	// Ensure that the admin user is valid
	$objAdminUser = new User();
	// Do we have a valid user?
	if (!$objAdminUser->retrieve_by_string_fields(array('user_name' => $_REQUEST['admin_user'], 'user_hash' => $_REQUEST['admin_pass'], 'is_admin' => 1, 'status' => 'Active', 'deleted' => 0))) {
		// Indicate the error
		return json_bad_request(array('error'=> 'invalid_admin_user'));
	}
	json_clean_incoming_data();
	// Create the contact object
	$objLead = new Lead();
	// Set the lead info
	$objLead->first_name = $_REQUEST['first_name'];
	$objLead->last_name = $_REQUEST['last_name'];
	$objLead->email1 = $_REQUEST['email_address'];
	$objLead->account_name = $_REQUEST['account_name'];
	$objLead->phone_work = $_REQUEST['office_phone'];
	$objLead->phone_mobile = $_REQUEST['mobile_phone'];
	$objLead->phone_fax = $_REQUEST['office_fax'];
	$objLead->primary_address_street = $objLead->alt_address_street = $_REQUEST['address_street'];
	$objLead->primary_address_city = $objLead->alt_address_city = $_REQUEST['address_city'];
	$objLead->primary_address_state = $objLead->alt_address_state = $_REQUEST['address_state'];
	$objLead->primary_address_country = $objLead->alt_address_country = $_REQUEST['address_country'];
	// Save the lead
	if (!$strLeadId = $objLead->save()) {
		// Error creating account
		return json_bad_request(array('error'=> 'error_creating_lead'));
	}
	// Create the return array
	$arReturn = array (
		'id' => $strLeadId,
	);
	// Return the value
	return json_return_value($arReturn);

}

$json_supported_actions['portal_create_opportunity'] = array(
	'login_required' => false,
);
function json_portal_create_opportunity() {
	// Ensure that the admin user is valid
	$objAdminUser = new User();
	// Do we have a valid user?
	if (!$objAdminUser->retrieve_by_string_fields(array('user_name' => $_REQUEST['admin_user'], 'user_hash' => $_REQUEST['admin_pass'], 'is_admin' => 1, 'status' => 'Active', 'deleted' => 0))) {
		// Indicate the error
		return json_bad_request(array('error'=> 'invalid_admin_user'));
	}
	json_clean_incoming_data();
	// Create the account first
	$objAccount = new Account();
	// Set the account info
	$objAccount->assigned_user_id = $objAdminUser->id;
	$objAccount->name = $_REQUEST['account_name'];
	$objAccount->phone_office = $_REQUEST['office_phone'];
	$objAccount->email1 = $_REQUEST['email_address'];
	$objAccount->billing_address_street = $objAccount->shipping_address_street = $_REQUEST['address_street'];
	$objAccount->billing_address_city = $objAccount->shipping_address_city = $_REQUEST['address_city'];
	$objAccount->billing_address_state = $objAccount->shipping_address_state = $_REQUEST['address_state'];
	$objAccount->billing_address_country = $objAccount->shipping_address_country = $_REQUEST['address_country'];
	// Create the account
	if (!$strAccountId = $objAccount->save()) {
		// Error creating account
		return json_bad_request(array('error'=> 'error_creating_account'));
	}
	// Create the contact object
	$objContact = new Contact();
	// Create the contact
	$objContact->assigned_user_id = $objAdminUser->id;
	$objContact->primary_account_id = $strAccountId;
	$objContact->first_name = $_REQUEST['first_name'];
	$objContact->last_name = $_REQUEST['last_name'];
	$objContact->email1 = $_REQUEST['email_address'];
	$objContact->phone_work = $_REQUEST['office_phone'];
	$objContact->primary_address_street = $objContact->alt_address_street = $_REQUEST['address_street'];
	$objContact->primary_address_city = $objContact->alt_address_city = $_REQUEST['address_city'];
	$objContact->primary_address_state = $objContact->alt_address_state = $_REQUEST['address_state'];
	$objContact->primary_address_country = $objContact->alt_address_country = $_REQUEST['address_country'];
	// Save the contact
	if (!$strContactId = $objContact->save()) {
		// Error creating account
		return json_bad_request(array('error'=> 'error_creating_contact'));
	}
	// Create the opportunity
	$objOpportunity = new Opportunity();
	// Create the opp
	$objOpportunity->assigned_user_id = $objAdminUser->id;
	$objOpportunity->account_id = $strAccountId;
	$objOpportunity->account_name = $_REQUEST['account_name'];
	$objOpportunity->product_name_c = $_REQUEST['opportunity_name'];
	$objOpportunity->referred_by_c = $_REQUEST['opportunity_referred_by'];
	$objOpportunity->sales_stage = $_REQUEST['opportunity_status'];
	$objOpportunity->date_closed = $_REQUEST['opportunity_appointment_date'];
	$objOpportunity->description = $_REQUEST['opportunity_notes'];
	$objOpportunity->contact_id = $strContactId;
	// Save the contact
	if (!$strOpportunityId = $objOpportunity->save()) {
		// Error creating account
		return json_bad_request(array('error'=> 'error_creating_opportunity'));
	}
	// Save the relationship change
	$objOpportunity->save_relationship_changes();
	// Create the return array
	$arReturn = array (
		'account_id' => $strAccountId,
		'contact_id' => $strContactId,
		'opportunity_id' => $strOpportunityId,
	);
	// Return the value
	return json_return_value($arReturn);
}

$json_supported_actions['portal_get_accounts_list'] = array(
	'login_required' => false,
);
function json_portal_get_accounts_list() {
	// Ensure that the admin user is valid
	$objAdminUser = new User();
	// Do we have a valid user?
	if (!$objAdminUser->retrieve_by_string_fields(array('user_name' => $_REQUEST['admin_user'], 'user_hash' => $_REQUEST['admin_pass'], 'is_admin' => 1, 'status' => 'Active', 'deleted' => 0))) {
		// Indicate the error
		return json_bad_request(array('error'=> 'invalid_admin_user'));
	}
	// Get access to db
	global $db;
	// Get list of accounts
	$strQuery = "
		SELECT
			id,
			name
		FROM
			accounts
		WHERE
			deleted = 0
		ORDER BY
			name
	";
	// Execute the query
	$objResult = $db->query($strQuery);
	// Initialize accounts
	$arAccounts = array();
	// Loop through and process the results
	while (($arRow = $db->fetchByAssoc($objResult)) != null) {
		// Add the case record
		$arAccounts[$arRow['id']] = $arRow['name'];
	}
	// Return the value
	return json_return_value($arAccounts);
}

$json_supported_actions['portal_get_quote_details'] = array(
	'login_required' => 'portal',
);
function json_portal_get_quote_details() {
	// Include the required functions
	require_once 'modules/Quotes/utils.php';
	
	// Determine the module
	switch ($_REQUEST['module_type']) {
		// Quote
		case 'Quotes':
			require_once 'modules/Quotes/Quote.php';
			$objSeed = new Quote();
			break;
		
		// Invoice
		case 'Invoice':
			require_once 'modules/Invoice/Invoice.php';
			$objSeed = new Invoice();
			break;
		
		// Sales Order
		case 'SalesOrders':
			require_once 'modules/SalesOrders/SalesOrder.php';
			$objSeed = new SalesOrder();
			break;
			
		// Unsupported module
		default:
			return json_bad_request(array('error'=> 'unsupported_module'));
	}
	// Retrieve the record
	$objSeed->retrieve($_REQUEST['id']);
	// Get the details
	$arDetails = BuildSoapItemsList($objSeed);
	// Return the result
	$arReturn = array(
		'groups' => $arDetails['groups'],
		'extra' => $arDetails['extra'],
	);
	// Return the value
	return json_return_value($arReturn);
}


function json_portal_save_attachment() {
    // Initialize the result
    $bReturn = false;
	// Create a new note
	if (!empty($_REQUEST['file_contents'])) {
		// Create the new note
		$objNote = new Note();
		// Create the file upload
		$objFileUpload = new UploadFile();
		// Decode the file content
        $strContentDecoded = base64_decode($_REQUEST['file_contents']);
        // Set the file content
        $objFileUpload->set_for_soap($_REQUEST['file_name'], $strContentDecoded);
		// longreach - added
        $objFileUpload->create_stored_filename();
		// Get the extension position
        $inExtensionPosition = strrpos($objFileUpload->stored_file_name, ".");
        // Set the extention
        $objFileUpload->file_ext = substr($objFileUpload->stored_file_name, $inExtensionPosition + 1);
        // Valid extension?
        if (in_array($objFileUpload->file_ext, AppConfig::setting('site.upload.badext'))) {
            // Set to text file    
    		$objFileUpload->stored_file_name .= ".txt";
            $objFileUpload->file_ext = "txt";
        }
		// Set the filename
        $objNote->filename = $objFileUpload->get_stored_file_name();
        // Set the mime type
        $objNote->file_mime_type = $objFileUpload->getMimeSoap($objNote->filename);
        // Set the name
        $objNote->name = $_REQUEST['file_name'];
        // Set the parent id
        $objNote->parent_id = $_REQUEST['parent_id'];
        // Set the parent name
        $objNote->parent_type = $_REQUEST['parent_module'];
        // Set the portal flag
        $objNote->portal_flag = 1;
        // Save the note
        $strReturnId = $objNote->save();
        // Do final move
        $objFileUpload->final_move($strReturnId);
        // Set the return value
        $bReturn = true;
	}
	// Set the return array
	$arReturn = array (
		'return' => $bReturn,
	);
	// Return encoded data
	json_return_value($arReturn);
}

$json_supported_actions['portal_save_attachment'] = array('login_required' => 'portal');



function json_portal_set_related_note() {
    // Initialize the result
    $bReturn = false;
	// Create the new note
	$objNote = new Note();
    // Set the name
    $objNote->name = $_REQUEST['name'];
    // Set the description
    $objNote->description = $_REQUEST['description'];
    // Set the parent id
    $objNote->parent_id = $_REQUEST['parent_id'];
    // Set the parent name
    $objNote->parent_type = $_REQUEST['parent_type'];
    // Set the portal flag
    $objNote->portal_flag = 1;
    // Save the note
    $strReturnId = $objNote->save();
    // Set the return array
	$arReturn = array (
		'id' => $strReturnId,
	);
	// Return encoded data
	json_return_value($arReturn);
}

$json_supported_actions['portal_set_related_note'] = array('login_required' => 'portal');

$json_supported_actions['portal_get_bug_releases'] = array('login_required' => 'portal');

function json_portal_get_bug_releases() {
	// Get access to globals
	global $db;
	// Generate the query
	$strQuery = "
		SELECT
			id,
			name
		FROM
			releases
		WHERE
			status = 'Active'
		ORDER BY
			list_order
	";
	// Initialize the releases array
	$arReleases = array();
	// Execute the query
	$objResult = $db->query($strQuery);
	// Loop through and process the results
	while (($arRow = $db->fetchByAssoc($objResult)) != null) {
		// Add the case record
		$arReleases[$arRow['id']] = $arRow['name'];
	}
	// Return the value
	return json_return_value($arReleases);
}

$json_supported_actions['portal_get_default_account'] = array('login_required' => false);

function json_portal_get_default_account() {
	// Get access to globals
	global $db;
	// Ensure that the admin user is valid
	$objAdminUser = new User();
	// Do we have a valid user?
	if (!$objAdminUser->retrieve_by_string_fields(array('user_name' => $_REQUEST['admin_user'], 'user_hash' => $_REQUEST['admin_pass'], 'is_admin' => 1, 'status' => 'Active', 'deleted' => 0))) {
		// Indicate the error
		return json_bad_request(array('error'=> 'invalid_admin_user'));
	}
	// Initialize the default account id
	$strAccountId = $strAccountName = '';
	// Determine the type of user
	switch ($_REQUEST['user_type']) {
		// Lead
		case 'lead':
			$lead = ListQuery::quick_fetch('Lead', $_REQUEST['user_id'], array('account_name'));
			if($lead) {
				$strAccountName = $lead->getField('account_name');
				// Account name set?
				if (!empty($strAccountName)) {
					$acc = ListQuery::quick_fetch_key('Account', 'name', $strAccountName);
					if($acc) {
						$strAccountId = $acc->getField('id');
					} else {
						// Indicate that we're to use a blank account (create a new account)
						$strAccountId = '-1';
					}
				}
			}			
			break;
			
		case 'contact':
			$ctc = ListQuery::quick_fetch('Contact', $_REQUEST['user_id'], array('primary_account', 'primary_account_id'));
			if($ctc) {
				$strAccountId = $ctc->getField('primary_account_id');
				$strAccountName = $ctc->getField('primary_account');
			}
			break;

		default:
			// Invalid type
			return json_bad_request(array('error'=> 'invalid_user_type'));	
	}
	// Set up return array
	$arReturn = array(
		'id' => $strAccountId,
		'name' => $strAccountName,
	);
	return json_return_value($arReturn);
}

/**
* generateRandomPassword; Generates a random password
*
* @access protected
* @return string
*/
function generateRandomPassword() {
	// Start with a blank password
	$strPassword = "";	
	// Define possible characters
	$strPossible = "0123456789bcdfghjkmnpqrstvwxyz"; 
	// Set up a counter
	$i = 0;
	// add random characters to $password until $length is reached
	while ($i < 8) { 
		// pick a random character from the possible characters
		$charTemp = substr($strPossible, mt_rand(0, strlen($strPossible)-1), 1);    
		// We don't want this character if it's already in the password
		if (!strstr($strPassword, $charTemp)) { 
			// Add this character to the password
			$strPassword .= $charTemp;
			// Increment the counter
			$i++;
		}
	}
	// Return the generated password
	return $strPassword;
}

/**
* getUniqueUsername; Generates a unique username
*
* @access protected
* @return string
*/
function getUniqueUsername($strFirstName, $strLastName) {
	// Get access to globals
	global $db;
	// Create the username
	$strUsername = strtolower(substr($strFirstName, 0, 1)) . strtolower($strLastName);
	// Initialize the counter
	$intCounter = 0;
	// Loop until we find a unique username
	while ($intCounter < 100) {
		// Update the username
		$strTemp = ($intCounter ? "_p_{$strUsername}{$intCounter}" : "_p_{$strUsername}");
		// Is this unique?
		$strQuery = "
			SELECT
				id
			FROM
				users
			WHERE
				user_name = '{$strTemp}'
		";
		// Execute the query
		$objResult = $db->query($strQuery);
		// Loop through and process the results
		if (!($arRow = $db->fetchByAssoc($objResult))) {
			// No existing user - return the username
			return substr($strTemp, 3);
		}
		// Inrement the counter
		$intCounter++;
	}
	// Error finding unique user
	return false;
}


$json_supported_actions['portal_get_enabled_items'] = array('login_required' => false);

function json_portal_get_enabled_items()
{
	global $current_user;
	$portal_perm_options = array('quotes', 'projects', 'cases', 'bugs', 'events', 'kb', 'faq', 'forum', 'contactus', 'mycontact', 'password', 'request', 'register',);
	$defaults = array(
		'lead' => array(
			'quotes' => '',
			'projects' => '',
			'bugs' => '',
			'cases' => '',
		),
	);
	$portal_perm = array();
	if (empty($current_user->id)) {
		$type = 'unreg';
	} elseif (!empty($_SESSION['registered_lead'])) {
		$type = 'lead';
		$portal_perm_options[] = 'subscribe';
	} else {
		$type = 'contact';
		$portal_perm_options[] = 'subscribe';
	}
	foreach ($portal_perm_options as $opt) {
		$key = 'portal.perm_' . $type. '_' . $opt;
		$val = AppConfig::setting($key);
		if (! isset($val) && isset($defaults[$type][$opt])) {
		} elseif($val) {
			$portal_perm[] = $opt;
		}
	}
	json_return_value($portal_perm);
}

$json_supported_actions['portal_is_registered_lead'] = array('login_required' => 'portal');

function json_portal_is_registered_lead()
{
	$ret = empty($_SESSION['registered_lead']) ? '' : $_SESSION['lead_id'];
	json_return_value($ret);
}

$json_supported_actions['portal_is_partner'] = array('login_required' => 'portal');

function json_portal_is_partner()
{
	$ret = empty($_SESSION['is_partner']) ? '' : $_SESSION['is_partner'];
	json_return_value($ret);
}



$json_supported_actions['portal_reset_cache'] = array('login_required' => 'portal');

function json_portal_reset_cache()
{
	global  $beanList, $beanFiles;
	if($_SESSION['type'] == 'lead' ) {
		json_bad_request('no_access');
	}
	unset($_SESSION['viewable'][$module]);;
	$ret = 1;
	json_return_value($ret);
}



$json_supported_actions['portal_check_admin_login'] = array(
	'login_required' => false,
);
function json_portal_check_admin_login() {
	// Ensure that the admin user is valid
	$objAdminUser = new User();
	// Do we have a valid user?
	if (!$objAdminUser->retrieve_by_string_fields(array('user_name' => $_REQUEST['admin_user'], 'user_hash' => $_REQUEST['admin_pass'], 'is_admin' => 1, 'status' => 'Active', 'deleted' => 0))) {
		// Indicate the error
		return json_bad_request(array('error'=> 'invalid_admin_user'));
	}
	// Set the return array
	$arReturn = array (
		'result' => true,
	);
	// Return the contact details
	return json_return_value($arReturn);
}
