<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2019 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');

$json_supported_actions['get_stripe_public_key'] = array('login_required' => true);
function json_get_stripe_public_key() {
	$ret = array('publishableKey' => AppConfig::setting('company.merchant_public_key_stripe'));
	json_auto_return($ret);
}

function generateResponse($intent)
{
  switch($intent->status) {
	case "requires_action":
	case "requires_source_action":
	  // Card requires authentication
	  return [
		'requiresAction'=> true,
		'paymentIntentId'=> $intent->id,
		'clientSecret'=> $intent->client_secret
	  ];
	case "requires_payment_method":
	case "requires_source":
	  // Card was not properly authenticated, suggest a new payment method
	  return [
		error => "Your card was denied, please provide a new payment method"
	  ];
	case "succeeded":
	  // Payment is complete, authentication not required
	  // To cancel the payment after capture you will need to issue a Refund (https://stripe.com/docs/api/refunds)
	  return [
		  'clientSecret' => $intent->client_secret,
		  'paymentIntentId'=> $intent->id,
		];
  }
}

$json_supported_actions['pay_by_stripe'] = array('login_required' => true);
function json_pay_by_stripe() {
	try {
	  \Stripe\Stripe::setApiKey(AppConfig::setting('company.merchant_login_stripe'));
	  if($_REQUEST['paymentMethodId'] != null) {
		// Create new PaymentIntent with a PaymentMethod ID from the client.
		$merchant_currency = new Currency();
		$merchant_currency->retrieve(AppConfig::setting('company.merchant_currency'));
		$intent = \Stripe\PaymentIntent::create([
		  "amount" => (int)sprintf("%d", str_replace(",","", $_REQUEST['amount']) * 100),
		  "currency" => $merchant_currency->iso4217,
		  "payment_method" => $_REQUEST['paymentMethodId'],
		  "confirmation_method" => "manual",
		  "confirm" => true,
		  "off_session" => true,
		  // If a mobile client passes `useStripeSdk`, set `use_stripe_sdk=true`
		  // to take advantage of new authentication features in mobile SDKs
		  "use_stripe_sdk" => $_REQUEST['useStripeSdk'],
		  'metadata' => ['invoice_ID' => $_REQUEST['invoiceId']]
		]);
		// After create, if the PaymentIntent's status is succeeded, fulfill the order.
		} else if ($_REQUEST['paymentIntentId'] != null) {
		// Confirm the PaymentIntent to finalize payment after handling a required action
		// on the client.
		$intent = \Stripe\PaymentIntent::retrieve($_REQUEST['paymentIntentId']);
		$intent->confirm();
		// After confirm, if the PaymentIntent's status is succeeded, fulfill the order.
	  }
	  $output = generateResponse($intent);

	  echo json_encode($output);
	} catch (\Stripe\Exception\ApiErrorException $e) {
	  echo json_encode([
		'error' => $e->getMessage()
	  ]);
	}
}

$json_supported_actions['stripe_webhook'] = array('login_required' => false);
function json_stripe_webhook() {
	try {
		global $current_user;
		$u = array_get_default($_GET, 'u');
		if ($u !== sha1(AppConfig::unique_key())) {
			header("HTTP/1.1 200 OK");
			echo 'ok';
			exit;
		}
		$payload = @file_get_contents('php://input');
		$data = json_decode($payload);

		if (!$current_user || !$current_user->id) {
			$current_user = new User;
			$current_user->getSystemUser();
		}

		$chargeId = $data->data->object->charge;
		
		if (ListQuery::quick_fetch_key('Payment', 'customer_reference', $chargeId)) {
			header("HTTP/1.1 200 OK");
			echo 'ok';
			exit;
		}

		$inv_id = $data->data->object->metadata->invoice_id;
		$invoice = ListQuery::quick_fetch('Invoice', $inv_id);
		if (empty($invoice))
			sugar_die('Invalid invoice ID');

		$currency = new Currency();
		$merchant_currency = new Currency();
		$merchant_currency->retrieve(AppConfig::setting('company.merchant_currency'));
		$invoice_amount_due = $invoice->getField('amount_due');
		$invoice_amount_due_usd = $invoice->getField('amount_due_usdollar');

		$paymentUpd = RowUpdate::blank_for_model('Payment', $invoice->getField('assigned_user_id'));
		$payment = array(
			'direction' => 'incoming',
			'assigned_user_id' => $invoice->getField('assigned_user_id'),
			'modified_user_id' => $invoice->getField('assigned_user_id'),
			'created_by' => $invoice->getField('assigned_user_id'),
			'amount' => $data->data->object->amount_paid / 100,
			'currency_id' => $currency->retrieveIDByIsoCode(strtoupper($data->data->object->currency)),
			'account_id' => $invoice->getField('billing_account_id'),
			'payment_date' => gmdate('Y-m-d H:i:s'),
			'customer_reference' => $chargeId,
			'payment_type' => 'Credit Card',
			'applied_currency_id' => $invoice->getField('currency_id'),
			'applied_amount' => $data->data->object->amount_paid / 100,
		);
		$paymentUpd->set($payment);
		$paymentUpd->save();
		$paymentId = $paymentUpd->getField('id');

		$items = array(
			array(
				'invoice_id' => $inv_id,
				'amount' => $data->data->object->amount_paid / 100,
				'assigned_user_id' => $invoice->getField('assigned_user_id'),
				'modified_user_id' => $invoice->getField('assigned_user_id'),
				'created_by' => $invoice->getField('assigned_user_id'),
				'exchange_rate' => $invoice->getField('exchange_rate'),
				'deleted' => 0,
				'date_modified' => gmdate('Y-m-d H:i:s'),
			),
		);

		$paymentUpd->addUpdateLinks('invoices', $items);
		header("HTTP/1.1 200 OK");
		echo 'ok';
		exit;
	} catch (\Stripe\Exception\ApiErrorException $e) {
	  echo json_encode([
		'error' => $e->getMessage()
	  ]);
	}
}
