<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


function set_json_ctype() {
	if(! headers_sent())
		header('Content-Type: application/json; charset=UTF-8');
}

function json_log($msg, $level=null) {
	global $log;
	if(! isset($level))
		$level = AppConfig::setting("json.log_level");
	if(! $level) return;
	$log->$level($msg);
}

function json_auth_die($msg=null, $kill_session=true) {
	end_clean_all_buffers();
	global $authController;
	if(is_null($msg)) {
		if($authController->auth_error)
			$msg = $authController->auth_error;
		else
			$msg = "An active session is required to perform this action";
	}
	header("HTTP/1.1 401 Not Authorized");
	if($kill_session) {
		if(isset($_COOKIE[session_name()])) {
			if(AppConfig::detect_https() == 'https')
	       		setcookie(session_name(), '', time()-42000, '/', AppConfig::request_host_name(), FALSE, TRUE);
	       	else
	       		setcookie(session_name(), '', time()-42000, '/', AppConfig::request_host_name(), FALSE, FALSE);
		}
		@session_destroy();
	}
	echo $msg."\n";
	sugar_cleanup();
	exit(1);
}

function json_forbidden($msg=null) {
	end_clean_all_buffers();
	if(is_null($msg)) {
		$msg = "The requested action is not permitted";
	}
	header("HTTP/1.1 403 Forbidden");
	set_json_ctype();
	echo $msg."\n";
	sugar_cleanup();
	exit(2);
}

function json_bad_request($msg=null) {
	end_clean_all_buffers();
	if(is_null($msg))
		$msg = "Unsupported action";
	header("HTTP/1.1 400 Bad Request");
	if(is_array($msg))
		json_return_value($msg);
	else
		echo $msg."\n";
	json_log("Unsupported action: {$GLOBALS['action']}");
	sugar_cleanup();
	exit(3);
}

function json_server_error($msg=null) {
	end_clean_all_buffers();
	if(is_null($msg))
		$msg = "Server Error";
	header("HTTP/1.1 500 Server Error");
	echo $msg."\n";
	sugar_cleanup();
	exit(4);
}

function json_return_value(&$value, $setvar=null, $close=false) {
	end_clean_all_buffers();
	$json = getJSONobj();
	$out = $json->encode($value, true);
	if($setvar && preg_match('~^[a-zA-Z_]\w*$~', $setvar)) {
		$out = '<script type="text/javascript">'
			. $setvar . '=' . $out . ';</script>';
	} else {
		set_json_ctype();
		$out .= "\n";
	}
	if($close) {
		end_clean_all_buffers(true);
		//error_reporting(0);
		ini_set('zlib.output_compression', 0);
		header('Content-Length: '.mb_strlen($out, '8bit'));
		header('Connection: close');
		header('Content-Encoding: none');
		ignore_user_abort();
	}
	echo $out;
	if($close) {
		flush();
		@session_write_close();
	}
}

function json_auto_return(&$value) {
	global $output_format, $action, $json_supported_actions;
	if($output_format == 'jsrd') {
		$done = false;
		if(! empty($json_supported_actions[$action])
		&& ! empty($json_supported_actions[$action]['schema'])
		&& array_get_default($json_supported_actions[$action],'jsrd') == 'explicit' ) {
			$spec = $json_supported_actions[$action]['schema']['output'];
			if(empty($spec['name']))
				$spec['name'] = $action;
			$jsrd = new IAH_JSRD();
			if($jsrd->load_result($spec, $value)) {
				$done = true;
				jsrd_return_value($jsrd);
			}
			else {
				global $log;
				$log->fatal("JSRD load error: ".$jsrd->last_load_error());
			}
		}
		if(! $done) {
			$jsrd =& jsrd_wrap_value($value, $action);
			if($jsrd)
				jsrd_return_value($jsrd);
		}
	}
	else
		json_return_value($value);
}

function json_perform_action($action) {
	global $json_supported_actions;
	$spec = $json_supported_actions[$action];
	$fn = array_get_default($spec, 'function_name', 'json_'.$action);
	$cls = array_get_default($spec, 'class_name');
	$inc = array_get_default($spec, 'includes');
	if($inc && is_array($inc)) {
		foreach($inc as $f) {
			if(! file_exists($f)) {
				json_server_error();
				return false;
			}
			require_once($f);
		}
	}
	if($cls) {
		if(class_exists($cls)) {
			$obj = new $cls();
			if(method_exists($obj, $fn)) {
				$obj->$fn();
				return true;
			}
		}
	}
	else if(function_exists($fn)) {
		call_user_func($fn);
		return true;
	}
	json_server_error();
	return false;
}

function json_translate($key, $mod='', $sel='', $returnOriginal=false) {
	if(! empty($_REQUEST['skip_translation']))
		return $key;
	return translate($key, $mod, $sel, $returnOriginal);
}

function json_get_option_list($name, $mod='', $pair=true, $remove_blank=false) {
	global $app_strings, $app_list_strings;
	if(!empty($mod)) {
		global $current_language;
		$mod_strings = return_module_language($current_language, $mod);
	} else {
		global $mod_strings;
	}
	$opts = array();
	if(isset($mod_strings[$name]))
		$opts = $mod_strings[$name];
	else if(isset($app_strings[$name]))
		$opts = $app_strings[$name];
	else if(isset($app_list_strings[$name]))
		$opts = $app_list_strings[$name];
	if(! is_array($opts))
		return null;
	if($remove_blank && isset($opts['']))
		unset($opts['']);
	if($pair) {
		$ret = array();
		foreach($opts as $k => $v)
			$ret[] = array('name' => $k, 'value' => $v);
		$opts = $ret;
	}
	return $opts;
}

function json_clean_input_value($value) {
	return securexss($value);
}

function json_clean_input_array(&$values) {
	if(is_array($values))
		$values = array_map('securexss', $values);
}

function json_clean_incoming_data() {
	json_filter_array($_REQUEST, 'securexss');
	json_filter_array($_POST, 'securexss');
	json_filter_array($_GET, 'securexss');
}

function json_filter_array(&$input, $fn, $recurse=false) {
	foreach($input as $k => $v) {
		if(is_array($v)) {
			if($recurse)
				json_filter_array($input[$k], $fn);
		} else
			$input[$k] = $fn($v);
	}
}

function json_handle_magic_quotes_gpc() {
	json_filter_array($_REQUEST, 'stripslashes', true);
	json_filter_array($_POST, 'stripslashes', true);
	json_filter_array($_GET, 'stripslashes', true);
}

function _count_query($query) {
	global $db;
	$result = $db->query($query);
	if($row = $db->fetchByAssoc($result)) {
		return $row['c'];
	}
	return 0;
}



function get_module_link_field($module_1, $module_2) {
	global $beanList, $beanFiles;

	// check to make sure both modules exist
	if (empty($beanList[$module_1]) || empty($beanList[$module_2])) {
		return FALSE;
	}

	$class_1 = $beanList[$module_1];
	require_once($beanFiles[$class_1]);

	$obj_1 = new $class_1();

	// loop through link fields of $module_1, checking for a link to $module_2
	foreach ($obj_1->get_linked_fields() as $linked_field) {
		$obj_1->load_relationship($linked_field['name']);
		$field = $linked_field['name'];

		if (empty($obj_1->$field)) {
			continue;
		}

		if ($obj_1->$field->getRelatedModuleName() == $module_2) {
			return $field;
		}
	}

	return FALSE;
}

// Retrieves array of ids for records of $get_module linked to $from_module by $get_id
// Example: to retrieve list of Contacts associated to Account X: $return = get_linked_records("Contacts", "Accounts", "X");
function get_linked_records($get_module, $from_module, $get_id, $deleted = 0) {
	global $beanList, $beanFiles;

	// instantiate and retrieve $from_module
	$from_class = $beanList[$from_module];
	require_once($beanFiles[$from_class]);
	$from_mod = new $from_class();
	$from_mod->retrieve($get_id);

	$field = get_module_link_field($from_module, $get_module);
	if ($field === FALSE) {
		return FALSE;
	}

	$from_mod->load_relationship($field);
	$id_arr = array();
	$query = $from_mod->$field->getQuery(false, array(), $deleted);
	$result = $from_mod->db->query($query, true);
	while($row = $from_mod->db->fetchByAssoc($result))
	{
		$id_arr[] = $row['id'];
	}
	return $id_arr;
}


?>
