<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

class BaseTheme {
	var $theme_name;
	var $theme_path;
	var $image_path;
	var $base_theme;
	var $config = array();
	var $hide_side_menu = '';
	var $custom_max_tabs = null;
	var $custom_max_subtabs = null;
	var $hide_dupe_list_shortcuts = false;
	var $texture_support = false;
	var $texture_enabled = true;
	var $support_group_tabs = true;
	var $support_flat_tabs = true;
	var $support_color_styles = false;
	var $support_font_styles = false;
	var $css_styles;
	var $mobile_support = false;
	var $mobile_mode = false;
	var $module_list;
	var $tabGroups = array();
	var $assign_shortcuts = true;
	var $shortcuts_module;
	var $assign_recent_links = true;
	var $layout_version = 7;
	var $title_loaded;
	var $smarty_templates = true;
	var $graph_colors = array();
	var $default_label_pos;
	var $custom_page_init;
	var $classic_title = false;
	var $override_css;
	var $can_set_background = false;
	var $current_font_style;
	var $current_color_style;
	
	function __construct($name, $params=null) {
		$this->theme_name = $name;
		if(! $params) $params = array();
		if(isset($params['mobile_mode']))
			$this->mobile_mode = $params['mobile_mode'];
		$this->theme_path = "themes/$this->theme_name/";
		
		if($name == 'Default')
			return;
		
		$this->config = AppConfig::get_theme_config($this->theme_name);
		if($this->texture_support) {
			if(isset($this->config['texture_enabled']))
				$this->texture_enabled = $this->config['texture_enabled'];
			if(isset($_COOKIE[$this->theme_name . '_textured']))
				$this->texture_enabled = ($_COOKIE[$this->theme_name . '_textured'] == 'true');
		}
		$this->base_theme = array_get_default($this->config, 'base_theme');
		if($this->base_theme)
			$this->image_path = "themes/{$this->base_theme}/images/";
		else
			$this->image_path = "{$this->theme_path}images/";
		foreach(array(
				'mobile_support', 'assign_shortcuts', 'assign_recent_links',
				'support_group_tabs', 'support_flat_tabs', 'css_styles', 'classic_title') as $k)
			if(isset($this->config[$k]))
				$this->$k = $this->config[$k];
		$this->layout_version = array_get_default($this->config, 'layout_version', 7);
		$this->smarty_templates = array_get_default($this->config, 'smarty_templates', true);
		
		global $even_bg, $odd_bg, $hilite_bg, $barChartColors, $pieChartColors;
		if(! function_exists('insert_popup_header'))
			include_once($this->theme_path.'layout_utils.php');
		$this->graph_colors = AppConfig::setting('layout.graph_colors');
		
		$this->default_label_pos = array_get_default($this->config, 'default_label_pos');
	}
	
	function version($min=null) {
		if(isset($min)) return ($this->layout_version >= $min);
		return $this->layout_version;
	}
	
	function get_config($param=null, $default=null) {
		if(! isset($param)) return $this->config;
		return array_get_default($this->config, $param, $default);
	}
	
	function init_render() {
		$GLOBALS['modListHeader'] = $this->get_module_list();
	}
	
	function finish_render() {
		global $pageInstance;
		if(! $pageInstance->update_page_state)
			return;
		$json = getJSONobj();
		$params = array();
		if($this->version(8)) {
			$sc = $this->get_shortcuts($this->shortcuts_module);
			$sce = array('keys' => array(), 'values' => array());
			$ord = array('create', 'list', 'other');
			$j = 0;
			foreach($ord as $o) {
				if(isset($sc[$o]))
					foreach($sc[$o] as $s) {
						unset($s['image']);
						$sce['keys'][] = ($k = 'sc_' . ($j++));
						$sce['values'][$k] = $s;
					}
			}
			$params['shortcuts'] = $sce;
			
			$rec = $this->get_recent_links(null, false);
			$ree = array('keys' => array(), 'values' => array());
			$j = 0;
			foreach($rec as $r) {
				$r['label'] = $r['label_full'];
				unset($r['label_full']);
				unset($r['image']);
				$ree['keys'][] = ($k = 're_' . ($j++));
				$ree['values'][$k] = $r;
			}
			$params['recent'] = $ree;
		}
		if($pageInstance->no_decoration)
			$params['no_decoration'] = 1;
		if($pageInstance->page_state)
			$params['state'] = $pageInstance->page_state;
		if($pageInstance->tools_options) {
			$params['tools'] = $pageInstance->tools_options;
			if(! isset($params['tools']['width']) && isset($params['tools']['maxlen']))
				$params['tools']['width'] = round(max($params['tools']['maxlen'], 15) * 0.7) +2.5 .'em';
			if($pageInstance->tools_form)
				$params['tools_form'] = $pageInstance->tools_form;
		}
		if($this->hide_side_menu)
			$params['hide_side_menu'] = 1;
			
		if(AppConfig::setting('site.performance.track_page_stats')) {
			$stats = [];
			if(! empty($GLOBALS['startTime']))
				$stats['render_duration'] = round( (microtime(true) - $GLOBALS['startTime']) * 1000);
			if(! empty($GLOBALS['db']))
				$stats['db_duration'] = round($GLOBALS['db']->getTotalQueryTime() * 1000);
			$stats['config_duration'] = round(AppConfig::$_config->total_request_time * 1000);
			$params['stats'] = $stats;
		}
		
		$favicon = $pageInstance->favicon;
		$params['favicon_href'] = array_get_default($favicon, 'href');
		$params['favicon_hr_href'] = array_get_default($favicon, 'hr_href');
		
		if($this->custom_page_init)
			$params['custom_init'] = $this->custom_page_init;
		
		$params = $json->encode($params);
		$pageInstance->add_js_literal("SUGAR.themes.initPage($params);", null, LOAD_PRIORITY_FOOT);
	}
	
	function finalize_page() {
		$this->add_theme_styles();
	}
	
	function hide_side_menu($hide, $complete) {
		$this->hide_side_menu = $hide ? ($complete ? 'complete' : 'hide') : '';
	}
	
	
	function get_global_links() {
		global $global_control_links, $current_user, $app_strings;
		require_once("include/globalControlLinks.php");
		$global_links = array();
		foreach($global_control_links as $key => $value) {
			if(isset($value['linkinfo'])) {
				foreach ($value['linkinfo'] as $label => $url) {
					$global_links[$key] = compact('label', 'url');
				}
				if(isset($value['icon']))
					$global_links[$key]['icon'] = $value['icon'];
			}
		}
		return $global_links;
	}
	
	
	function get_shortcuts($currentModule=null) {
		global $app_list_strings, $app_strings, $pageController, $pageInstance;
		
		$module_menu = $pageController->get_menu_shortcuts($currentModule);
		$user_tabs = AppConfig::setting('modinfo.current_user');

		$lastActive = false;
		$all = array();
		foreach($module_menu as $menu_item) {
			list($url, $label, $icon) = $menu_item;
			$module = array_get_default($menu_item, 3, '');
			$title = to_html(array_get_default($app_list_strings['moduleList'], $module, ''));
			$icon = get_image_info($icon, false, "title=\"$title\" border=\"0\"");
			$image = $icon['html'];
			$icon = $icon['class'];
			$category = '';
			$purl = parse_url($url);
			$args = array();
			if($purl && array_get_default($purl, 'path') == 'index.php' && isset($purl['query'])) {
				parse_str($purl['query'], $args);
			}
			if(! $module)
				$module = array_get_default($args, 'module', '');
			$item = compact('url', 'label', 'module', 'icon', 'image', 'category');
			
			if(count($args)) {
				$act = array_get_default($args, 'action');
				if($act == 'index' || $act == 'ListView' || ($module == 'iFrames' && $act == 'DetailView'))
					$item['category'] = 'list';
				else if($act == 'EditView')
					$item['category'] = 'create';
				if(! $item['category']) {
					if(substr($icon, 0, 6) == 'Create')
						$item['category'] = 'create';
					else
						$item['category'] = 'other';
				}
				if($item['category'] == 'create' || $item['category'] == 'list') {
					if($module && $module != $currentModule && $module != $pageController->module && isset($user_tabs[$module]) && ($user_tabs[$module]['hidden'] == 'user' || $user_tabs[$module]['hidden'] == 'acl')) {
						continue;
					}
				}

				$active = 0;
				$signif = 0;
				foreach($args as $k => $v) {
					if($k == 'return_module' || $k == 'return_action')
						continue;
					$signif ++;
					if($k == 'module' || $k == 'action')
						$chk = ($pageInstance->$k == $v);
					else
						$chk = (isset($_REQUEST[$k]) && $_REQUEST[$k] == $v);
					if(! $chk)
						$active = -1;
					else if($active >= 0)
						$active ++;
				}
				if($active > 0 && (! $lastActive || $lastActive[1] < $active))
					$lastActive = array(count($all), $active);
				if($signif == 2 && ! empty($args['module']) && array_get_default($args, 'action') == 'index')
					$item['simple_module_link'] = $args['module'];
			}
			$all[] = $item;
		}
		if($lastActive)
			$all[$lastActive[0]]['active'] = true;
		$shortcuts = array();
		foreach($all as $item) {
			$shortcuts[$item['category']][] = $item;
			if($item['category'] != 'list' && $item['category'] != 'non_list')
				$shortcuts['non_list'][] = $item;
		}

		return $shortcuts;
	}
	
	
	function get_recent_links($module_name=null, $escape=true) {
		global $app_list_strings;
		$user_id = AppConfig::current_user_id();
		$recent_links = array();
		if(! empty($user_id)) {
			require_once('data/Tracker.php');
			$history = Tracker::get_recently_viewed($user_id, $module_name);
			if(count($history) > 0) {
				$idx = 0;
				foreach($history as $row) {
					if($idx >= 10)
						break;
					$module = $row['module_name'];
					$record = $row['item_id'];
					$title = to_html(array_get_default($app_list_strings['moduleList'], $module, ''));
					$icon = get_image_info($module, 'border="0" align="absmiddle" alt="" title="'.$title.'"');
					$image = $icon['html'];
					$icon = $icon['class'];
					$label = getTrackerSubstring($row['item_summary'], 30, true);
					$label_full = $row['item_summary'];
					if($escape) {
						$label = to_html($label);
						$label_full = to_html($label_full);
					}
					$url = "index.php?module={$module}&action=DetailView&record={$record}";
					$recent_links[] = compact('image', 'icon', 'label', 'label_full', 'url', 'module', 'record');
					$idx ++;
				}
			}
		}
		return $recent_links;
	}
	
	function populate_footer(&$tpl)
	{
	}
	
	function populate_header(&$tpl, $currentModule=null) {
		global $app_strings, $pageInstance, $current_user;
		if(! isset($currentModule))
			$currentModule = $pageInstance->module;
		$user_id = AppConfig::current_user_id();
		require_once('include/upload_file.php');
		
		$tpl->assign("is_demo_site", is_demo_site());
		$tpl->assign("THEME", $this->theme_name);
		$static_url = AppConfig::site_static_url();
		$tpl->assign("IMAGE_PATH", $static_url . $this->image_path);
		$tpl->assign("MODULE_NAME", $currentModule);
		$tpl->assign("ACTION", $pageInstance->action);
		$tpl->assign("USER_ICON", AppConfig::is_admin() ? 'icon-adminuser' : 'icon-user');
		$tpl->assign("DEV_BRANCH", AppConfig::dev_branch());
		if(! empty($current_user->photo_thumb)) {
			$img_path = $current_user->photo_thumb;
			$fdef = $current_user->getFieldDefinition('photo');
			$img_dir = array_get_default($fdef, 'upload_dir');
			$img_path = UploadFile::get_path($img_path, '', $img_dir);
			$img_size = array(20, 20);
			$thumb = resolve_image($img_path, $img_size);
			if($thumb) {
				$attr = "width=\"{$thumb[1]}\" height=\"{$thumb[2]}\"";
				$value = "<img src=\"$img_path\" border=\"0\" alt=\"\" $attr style=\"vertical-align: baseline\">";
				$s = '';
				if($thumb[1] != $img_size[0]) {
					$s = "width: {$img_size[0]}px";
				}
				if($thumb[2] < $img_size[1]) {
					$pad = floor(($img_size[1] - $thumb[2])/2);
					$s = "padding-top: {$pad}px; padding-bottom: {$pad}px";
				} 
				$value = "<div class=\"image-wrap\" style=\"$s; line-height: 8px; vertical-align: middle; margin-top: -2px\">$value</div>";
				$tpl->assign("USER_PHOTO", $value);
			}
		}
		$tpl->assign("USER_ID", $user_id);
		$tpl->assign("texture_enabled", ($this->texture_support && $this->texture_enabled));
		if($this->support_color_styles) {
			if(! empty($this->current_color_style))
				$tpl->assign("current_color_style", $this->current_color_style);
			$tpl->assign('color_styles', $this->config['color_styles']);
		}
		if($this->support_font_styles) {
			if(! empty($this->current_font_style))
				$tpl->assign("current_font_style", $this->current_font_style);
			$tpl->assign('font_styles', $this->config['font_styles']);
		}
		
		$pageInstance->theme_template_setup($tpl);
		$tpl->assign('ONLOAD', $pageInstance->get_onload());
		
		$global_links = $this->get_global_links();
		$tpl->assign('global_links', $global_links);
		
		$modList = $this->get_module_list();
		if(! is_countable($modList) || ! count($modList))
			$tpl->assign('hide_tab_bar', true);
		
		$tabs = $this->get_tabs($currentModule);
		$tpl->assign('module_tabs', $tabs);
		
		if( ($sel_mod = $tabs['active']) ) {
			if(isset($tabs['tabs'][$sel_mod]['tabs'])) {
				$submods = $tabs['tabs'][$sel_mod];
				$sel_mod = $submods['tabs'][$submods['active']]['module'];
			} else
				$sel_mod = $tabs['tabs'][$sel_mod]['module'];
		}
		if($sel_mod == 'Administration' && $currentModule != $sel_mod)
			$sel_mod = '';
		$this->shortcuts_module = $sel_mod;

		if($this->assign_shortcuts) {
			$tpl->assign("SHORTCUTS", $app_strings['LBL_SHORTCUTS']);
			$shortcuts = $this->get_shortcuts($sel_mod);
			if($tabs['use_group_tabs'] && $tabs['active'] && ! empty($shortcuts['list'])) {
				$sel_tab = $tabs['tabs'][$tabs['active']]; // tabs tabs tabs tabs.. tabs
				$tab_mods = array();
				foreach($sel_tab['tabs'] as $k => $subt) {
					if($sel_tab['active'] != $k)
						$tab_mods[$subt['module']] = 1;
				}
				$sc = array();
				foreach($shortcuts['list'] as $s) {
					if(! empty($s['simple_module_link']) && ! empty($tab_mods[$s['simple_module_link']]))
						continue;
					$sc[] = $s;
				}			
				if(count($sc) == 1 && ! empty($sc[0]['simple_module_link']))
					$sc = array();
				$shortcuts['list'] = $sc;
			}
			if(! $tabs['use_group_tabs']) {
				// suppress single module link to the current module
				if(! empty($shortcuts['list'])) {
					if(count($shortcuts['list']) == 1 && array_get_default($shortcuts['list'][0], 'simple_module_link') == $sel_mod)
						$shortcuts['list'] = array();
				}
			}
			$tpl->assign('shortcuts', $shortcuts);
		}
		
		$hide_menu = $this->hide_side_menu;
		if(! $hide_menu && array_get_default($_COOKIE, 'showLeftCol', 'true') == 'false')
			$hide_menu = 'guess';
		if($user_id) {
			if($this->assign_recent_links) {
				$tpl->assign("TITLE_LAST_VIEW", $app_strings['LBL_LAST_VIEWED']);
				$recent_links = $this->get_recent_links();
				$tpl->assign('recent_links', $recent_links);
			}

			if($hide_menu != 'complete') {
				//if(($shortcuts_swap || ! $shortcuts) && (! $last_view_swap || ! $recent_links) && ! $new_record_form)
				//	$hide_menu = 'complete';
			}
		} else
			$hide_menu = 'complete';

		$tpl->assign('hide_side_menu', $hide_menu);
		
		$shortcuts_swap = $current_user->getPreference('swap_shortcuts');
		if(!isset($shortcuts_swap)) $shortcuts_swap = AppConfig::setting('layout.defaults.swap_shortcuts');
		$last_view_swap = $current_user->getPreference('swap_last_viewed');
		if(!isset($last_view_swap)) $last_view_swap = AppConfig::setting('layout.defaults.swap_last_viewed');
		if($hide_menu == 'complete') {
			$last_view_swap = true;
			$shortcuts_swap = false;
		}
		$tpl->assign('recent_on_side', $last_view_swap);
		$tpl->assign('shortcuts_on_top', $shortcuts_swap || ! $this->assign_shortcuts);
		
		$this->add_toolbar_menus($tpl);
	}
	
	
	function get_module_list() {
		if(! isset($this->module_list))
			$this->module_list = get_user_module_list();
		return $this->module_list;
	}

	
	function get_use_group_tabs() {
		global $current_user;
		if(! $this->support_group_tabs)
			return false;
		if(! $this->support_flat_tabs)
			return true;
		$nav_style = $current_user->getPreference('navigation_paradigm');
		if(!isset($nav_style))
			$nav_style = AppConfig::setting('layout.defaults.navigation_paradigm');
		return $nav_style == 'gm';
	}

	
	function get_tabs($module, $grpModule=null) {
		require_once('include/GroupedTabs/GroupedTabStructure.php');
		$tabManager = new GroupedTabStructure();
		$tabs = $tabManager->get_theme_tabs($this->get_use_group_tabs(), $module, $grpModule, $this->custom_max_tabs, $this->custom_max_subtabs);
		return $tabs;
	}
	
	
	function render_header() {
		if($this->smarty_templates) {
			require_once('include/Sugar_Smarty.php');
			$tpl = new Sugar_Smarty();
			$tpl_path = $this->theme_path.'header.tpl';
		} else {
			require_once('include/layout/TemplateStub.php');
			$tpl = new TemplateStub();
			$tpl_path = $this->theme_path.'render_header.php';
		}
		
		$this->populate_header($tpl);
		
		$str = $tpl->fetch($tpl_path);
		$tpl->export_includes();
		return $str;
	}
	
	
	function render_footer($footer_copyright=null, $footer_etc=null) {
		global $pageInstance, $locale;
		
		$max_tabs = array_get_default($this->config, 'max_footer_tabs', 10);
		
		if($this->smarty_templates) {
			require_once('include/Sugar_Smarty.php');
			$tpl = new Sugar_Smarty();
			$tpl_path = $this->theme_path.'footer.tpl';
		} else {
			require_once('include/layout/TemplateStub.php');
			$tpl = new TemplateStub();
			$tpl_path = $this->theme_path.'render_footer.php';
		}
		
		$pageInstance->theme_template_setup($tpl, true);
		$this->populate_footer($tpl);
		
		if(isset($pageInstance) && AppConfig::current_user_id() && $locale->getPrecedentPreference('show_footer_module_links')) {
			$s = '';
			$tabs = $this->get_module_list();
			$numTabs = count($tabs);
			$i=1;
			foreach($tabs as $module_name) {
				$s .= '<a href="index.php?module='.$module_name.'&action=index" class="footerLink" onclick="return SUGAR.util.handleLink(event);">'
					. translate('LBL_MODULE_TITLE', $module_name) . '</a>';
				if ($max_tabs && ($i > $numTabs-1 || $i % $max_tabs == 0)) {}
				else $s .= ' | ';
				if ($max_tabs && ($i % $max_tabs == 0))
				$s .= '<br>';
				$i++;
			}
			$tpl->assign('FOOTER_LINKS', $s);
		}
		$tpl->assign('MODULE_NAME', $pageInstance->module);
		$tpl->assign('FOOTER_COPYRIGHT', $footer_copyright);
		$tpl->assign('FOOTER_ETC', $footer_etc);

		$ret = $tpl->fetch($tpl_path);
		$tpl->export_includes();
		return $ret;
	}
	
	
	function add_toolbar_menus($tpl) {
		global $pageInstance, $app_strings;
		
		assign_calendar_links($tpl);
		$links = $tpl->getTemplateVars();
		$menus = array();
		
		if(! empty($links['IAH_DAY_LINK'])) {
			$menus['menu_calendar'] = array(
				'day' => array(
					'icon' => 'theme-icon ticon-CalendarDay',
					'label' => $app_strings['LBL_DAY_CAL_LINK'],
					'url' => $links['IAH_DAY_LINK'],
				),
				'week' => array(
					'icon' => 'theme-icon ticon-CalendarWeek',
					'label' => $app_strings['LBL_WEEK_CAL_LINK'],
					'url' => $links['IAH_WEEK_LINK'],
				),
				'month' => array(
					'icon' => 'theme-icon module-Calendar',
					'label' => $app_strings['LBL_MONTH_CAL_LINK'],
					'url' => $links['IAH_MONTH_LINK'],
				),
			);
		}

		if(! empty($links['IAH_CHECK_EMAIL_LINK'])) {
			$menus['menu_email'] = array(
				'email' => array(
					'icon' => 'icon-email',
					'label' => $app_strings['LBL_GOTO_EMAIL'],
					'url' => $links['IAH_EMAIL_LINK'],
				),
				'compose' => array(
					'icon' => 'icon-send',
					'label' => $app_strings['LBL_COMPOSE_EMAIL'],
					'url' => $links['IAH_COMPOSE_EMAIL_LINK'],
				),
				'check' => array(
					'icon' => 'icon-recur',
					'label' => $links['CHECK_EMAILS_LABEL'],
					'url' => $links['IAH_CHECK_EMAIL_LINK'],
				),
			);
		}
		
		$menus['menu_system'] = array();

		if(! empty($links['IAH_PROFILE_LINK']))
			$menus['menu_system']['profile'] = array(
				'icon' => 'input-icon ' . (AppConfig::is_admin() ? 'icon-adminuser' : 'icon-user'),
				'label' => $app_strings['LBL_MY_ACCOUNT'],
				'url' => $links['IAH_PROFILE_LINK'],
			);
		if(! empty($links['IAH_ADMIN_LINK']))
			$menus['menu_system']['admin'] = array(
				'icon' => 'icon-action',
				'label' => $app_strings['LBL_ADMIN'],
				'url' => $links['IAH_ADMIN_LINK'],
			);
		if(! empty($links['IAH_DIRECTORY_LINK']))
			$menus['menu_system']['directory'] = array(
				'icon' => 'theme-icon bean-Employee',
				'label' => $app_strings['LBL_DIRECTORY'],
				'url' => $links['IAH_DIRECTORY_LINK'],
			);
		if(! empty($links['IAH_ABOUT_LINK']))
			$menus['menu_system']['about'] = array(
				'icon' => 'input-icon icon-help',
				'label' => $app_strings['LNK_ABOUT'],
				'url' => $links['IAH_ABOUT_LINK'],
			);
		if(! empty($links['IAH_LOGOUT_LINK']))
			$menus['menu_system']['logout'] = array(
				'icon' => 'icon-cancel',
				'label' => $app_strings['LBL_LOGOUT'],
				'url' => $links['IAH_LOGOUT_LINK'],
				'async' => false,
			);
		if(! empty($links['IAH_LOGIN_LINK']))
			$menus['menu_system']['login'] = array(
				'icon' => 'icon-user',
				'label' => $app_strings['LBL_LOGIN_BUTTON_LABEL'],
				'url' => $links['IAH_LOGIN_LINK'],
				'async' => false,
			);
		
		$json = getJSONobj();
		foreach($menus as $mid => $opts) {
			$opts = $json->encode(array(
				'keys' => array_keys($opts), 'values' => $opts, 'icon_key' => 'icon',
			));
			$tpl->assign($mid, $opts);
		}
	}
	
	function theme_style_defined($style) {
		return preg_match('/^[a-z]*$/i', $style) && isset($this->css_styles[$style])
			&& file_exists("themes/{$this->theme_name}/less/style-{$style}.less");
	}
	
	function get_theme_css_styles() {
		if(! empty($this->css_styles)) {
			$styles = $this->css_styles;
			foreach($styles as $k => &$s)
				$s['id'] = $k;
			return $styles;
		}
	}
	
	function get_theme_css_style() {
		if(! empty($this->css_styles)) {
			$saved_style = array_get_default($_COOKIE, 'ck_' . $this->theme_name . '_style');
			$style = array_get_default($_GET, 'theme_style', $saved_style);
			if($style && $this->theme_style_defined($style))
				return $style;
			return key($this->css_styles);
		}
	}
	
	function get_theme_css_params() {
		$params = ['theme' => $this->theme_name];
		if(isset($this->config['less_vars']))
			array_extend($params, $this->config['less_vars']);
		if(! empty($this->css_styles)) {
			$saved_style = array_get_default($_COOKIE, 'ck_' . $this->theme_name . '_style');
			$style = $this->get_theme_css_style();
			if($style) {
				$params['theme-style'] = $style;
				if($saved_style != $style) {
					$ck_expire = time() + 86400 * 90;
					if(AppConfig::detect_https() == 'https')
			       		setcookie('ck_' . $this->theme_name . '_style', $style, $ck_expire, '/', AppConfig::request_host_name(), FALSE, TRUE);
			       	else
			       		setcookie('ck_' . $this->theme_name . '_style', $style, $ck_expire, '/', AppConfig::request_host_name(), FALSE, FALSE);
				}
			}
		}
		return $params;
	}
	
	function set_override_css($css) {
		// used to test precompiled theme CSS
		$this->override_css = $css;
	}
		
	function get_theme_css() {
		if($this->override_css) {
			return [
				['type' => 'literal', 'value' => $this->override_css],
			];
		}
		
		require_once('include/utils/theme_utils.php');
		$compiled = cached_theme_css($this->theme_name);
		
		$ret = [[
			'type' => 'include',
			'value' => $compiled['include_path'],
			'params' => ['no_min' => true, 'no_ver' => true],
		]];
		if(empty($theme_less_found)) {
			$def_style = "themes/{$this->theme_name}/style.css";
			if(file_exists($def_style))
				$ret[] = $def_style;
		}
		
		return $ret;
	}
	
	function add_theme_styles() {
		global $pageInstance;
		$base = $this->base_theme ? $this->base_theme : $this->theme_name;
		$theme = $this->theme_name;
		
		$theme_css = $this->get_theme_css();
		if($theme_css) {
			foreach($theme_css as $inc_path) {
				if(is_array($inc_path)) {
					$tpars = array_get_default($inc_path, 'params');
					if(array_get_default($inc_path, 'type', 'include') == 'include') {
						$pageInstance->add_css_include($inc_path['value'], $tpars, LOAD_PRIORITY_THEME);
					} else {
						$pageInstance->add_css_literal($inc_path['value'], $tpars, LOAD_PRIORITY_THEME);
					}
				} else {
					$pageInstance->add_css_include($inc_path, null, LOAD_PRIORITY_THEME);
				}
			}
		}
		
		$pageInstance->add_css_include("themes/Default/theme-icons.css", null, LOAD_PRIORITY_THEME);
		$pageInstance->add_css_include("themes/Default/print-icons.css", array('media' => 'print'), LOAD_PRIORITY_THEME);
		$custom = AppConfig::custom_dir();
		if(file_exists($custom . 'themes/Default/icons.css'))
			$pageInstance->add_css_include($custom . 'themes/Default/icons.css', null, LOAD_PRIORITY_THEME);
		$static_url = AppConfig::site_static_url();
		$pfx = $static_url . css_prefix();
		$sfx = css_suffix();

		if($this->support_color_styles) {
			$colors = array();
			foreach($this->config['color_styles'] as $c) {
				$href = "themes/{$theme}/colors.{$c['name']}.css";
				if(! file_exists($href)) $href = '';
				$c['href'] = $href;
				$colors[$c['name']] = $c;
			}

			$selColor = array_get_default($_COOKIE, $theme.'_color_style');
			if(! $selColor || ! isset($colors[$selColor]))
				$selColor = key($colors);
			if($colors[$selColor]['href'])
				$pageInstance->add_css_include($colors[$selColor]['href'], array('attrs' => 'id="theme:color"'), LOAD_PRIORITY_THEME);
			$this->current_color_style = $selColor;

			foreach($colors as $i => $c) {
				if($colors[$i]['href'])
					$colors[$i]['href'] = $pfx.$colors[$i]['href'].$sfx;
			}
			$pageInstance->set('color_styles', $colors);
		}
		if($this->support_font_styles) {
			$fonts = array();
			foreach($this->config['font_styles'] as $f) {
				$href = "themes/{$theme}/fonts.{$f['name']}.css";
				if(! file_exists($href)) $href = '';
				$f['href'] = $href;
				$fonts[$f['name']] = $f;
			}
			
			$selFont = array_get_default($_COOKIE, $theme.'_font_style');
			if(! $selFont || ! isset($fonts[$selFont]))
				$selFont = key($fonts);
			if($fonts[$selFont]['href'])
				$pageInstance->add_css_include($fonts[$selFont]['href'], array('attrs' => 'id="theme:font"'), LOAD_PRIORITY_THEME);
			$this->current_font_style = $selFont;
			
			foreach($fonts as $i => $f) {
				if($fonts[$i]['href'])
					$fonts[$i]['href'] = $pfx.$fonts[$i]['href'].$sfx;
			}
			$pageInstance->set('font_styles', $fonts);
		}
	}

	function mobileSupport()
	{
		return $this->mobile_support;
	}

	function isMobile()
	{
		return $this->mobile_mode;
	}

	function addTabGroup($id)
	{
		$this->tabGroups[] = $id;
	}

	function canSetBackground()
	{
		return $this->can_set_background;
	}

	function get_module_title($icon, $title, $show_help=true) {
		global $pageInstance;
		
		if(! $this->classic_title && ! $this->title_loaded) {
			$mod = $pageInstance->get_title_module();
			$mod_name = $pageInstance->get_title_module_label();
			$mod_html = to_html($mod_name);
			if($mod_html == $title)
				$title = '';
			$params = array(
				'module' => $mod,
				'module_title' => $mod_html,
				'title_merge' => $pageInstance->title_module_merge ? 1 : 0,
				'title' => $title,
				'show_help' => $show_help,
			);
			if($icon) {
				if(! preg_match('/^(module|bean|ticon|input)-/', $icon)) {
					$icon = get_image_info($icon);
					$icon = $icon['found'] ? $icon['class'] : '';
				}
				if($icon && strpos($icon, 'input-icon') === false)
					$icon = 'input-icon theme-icon ' . $icon;
				$params['icon'] = $icon;
			}
			if($pageInstance->action != 'index' && ACLController::checkAccess($mod, 'list'))
				$params['module_link'] = 'index.php?module=' . $mod . '&action=index';
			$this->title_loaded = true;
			$js = getJSONobj();
			return '<script type="text/javascript">SUGAR.themes.setPageInfo(' . $js->encode($params) . ');</script>';
		}
		if($this->title_loaded && ! $title)
			return;
		
		$the_title = "<table width='100%' cellpadding='0' cellspacing='0' border='0' class='moduleTitle'><tr valign='top'><td>";
		$icon = preg_replace("/ /","",$icon);
		$icon = get_module_header_icon($icon, $title);
		if($icon)  $icon .= '&nbsp;';
		
		$the_title .= "<td><h2>". $icon . $title."</h2></td>\n";

		if ($show_help) {
			$the_title .= "<td valign='top' align='right' nowrap style='padding-top:3px; padding-left: 5px;'>";
			$help_link = get_help_link();
			$icon = '<div class="input-icon icon-help active-icon" style="vertical-align: text-bottom"></div>';
			$help_label = translate('LNK_HELP', 'app');
			$the_title .= "&nbsp;<a href='$help_link' target='_blank' class='utilsLink'>{$icon}&nbsp;<span class=\"utilsLabel\">$help_label</span></a></td>\n";
		}

		$the_title .= "</tr></table>\n";
		
		return $the_title;
	}
	
	function get_form_header($form_title, $other_text, $show_help, $icon='') {
		$the_form = '';
		$the_form .= '<table width="100%" cellpadding="0" cellspacing="0" border="0" class="h3Row"><tr>';
		$help_label = translate('LNK_HELP', 'app');

		if($icon) {
			if(strpos($icon, '<') === false)
				$icon = get_image($icon, ' alt="" title="" border="0" style="margin-top: -2px"');
			$form_title = $icon .'&nbsp;'. $form_title;
		}

		$the_form .= '<td nowrap><h3>'.$form_title.'</h3></td>';

		if ($other_text && preg_match('~<form\b~', $other_text)) {
			$the_form .= "<td colspan='10' width='100%'><span style=\"line-height:1px\">&nbsp;</span></td>\n";
			$the_form .= "</tr><tr>\n";
			$the_form .= "<td align='left' valign='middle' nowrap style='padding-bottom: 2px;'>$other_text</td>\n";
			$the_form .= "<td width='100%'><span style=\"line-height:1px\">&nbsp;</span></td>\n";

			if ($show_help) {
				$the_form .= "<td align='right' nowrap>";
				$help_link = get_help_link();
				$icon = '<div class="input-icon icon-help active-icon" style="vertical-align: text-bottom"></div>';
				$the_form .= "&nbsp;<a href='$help_link' target='_blank' class='utilsLink'>{$icon}&nbsp;<span class=\"utilsLabel\">{$help_label}</span></a></td>\n";
			}
		} else {
			if ($other_text) {
				$the_form .= "<td width='20'><span style=\"line-height:1px\">&nbsp;</span></td>\n";
				$the_form .= "<td valign='middle' nowrap width='100%'>$other_text</td>\n";
			}
			else {
				$the_form .= "<td width='100%'><span style=\"line-height:1px\">&nbsp;</span></td>\n";
			}

			if ($show_help) {
				$the_form .= "<td align='right' nowrap>";
				$help_link = get_help_link();
				$icon = '<div class="input-icon icon-help active-icon" style="vertical-align: text-bottom"></div>';
				$the_form .= "&nbsp;<a href='$help_link' target='_blank' class='utilsLink'>{$icon}&nbsp;<span class=\"utilsLabel\">{$help_label}</span></a></td>\n";
			}

		}

		$the_form .= '</tr></table>';

		return $the_form;
	}
	
	function get_form_footer() {
	}
	
	function get_popup_header() {
		$charset = AppConfig::charset();
		$out  = '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Strict//EN">'."\n";
		$out .=	'<html><head>';
		$out .= get_favicon();
		$out .=	'<meta http-equiv="Content-Type" content="text/html; charset='.$charset.'">';
		$out .=	'<title>'.translate('LBL_BROWSER_TITLE', 'app').'</title>';
		$out .= get_header_includes('popup');
		$out .=	'</head><body style="margin: 10px">';
		return $out;
	}
	
	function get_popup_footer() {
		return '</body></html>';
	}
	
	function generate_graph_color($input, $instance, $html=false) {
		$instance = (int)$instance;
		$out = $this->graph_colors[$instance % count($this->graph_colors)];
		return ($html ? '#' : '0x') . $out;
	}
}

?>
