<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('modules/Dashboard/Dashboard.php');

class DashletManager {
	var $seedDashboard;

	function __construct() {
		$this->seedDashboard = new Dashboard();
	}

	static function get_dashlet_files() {
		$cache_file = CacheManager::get_location('system/') . 'dashlet_cache.php';
		if(! is_file($cache_file)) {
			require_once('include/Dashlets/DashletCacheBuilder.php');
			$dc = new DashletCacheBuilder();
			$dc->buildCache();
		}
		global $dashletsFiles;
		if(! isset($dashletsFiles))
			$dashletsFiles = array();
		include($cache_file);
		return $dashletsFiles;
	}

	static function get_chart_files() {
		static $predefined_charts;
		if(! isset($predefined_charts)) {
			$predefined_charts = array();
			include('modules/Charts/code/predefined_charts.php');
		}
		return $predefined_charts;
	}
	
	static function get_all_dashlet_info() {
		$dlet_files = self::get_dashlet_files();
		$chart_files = self::get_chart_files();
		$ret = array();
		foreach($dlet_files as $cls => $info) {
			$info['type'] = 'standard';
			$ret[$cls] = $info;
		}
		foreach($chart_files as $cls => $info) {
			$ret[$cls] = array(
				'type' => 'chart',
				'class' => $cls,
				'chart_type' => $info['type'],
				'title' => translate($info['label'], 'Charts'),
				'icon' => array_get_default($info, 'icon', 'Forecasts'),
				'category' => 'Charts',
			);
			if($info['type'] == 'code')
				$ret[$cls]['file'] = "modules/Charts/code/{$cls}.php";
			else {
				$ret[$cls]['file'] = "modules/Charts/code/ReportChartDashlet.php";
				$ret[$cls]['properties']['report_id'] = $chart_files[$cls]['id'];
			}
		}
		return $ret;
	}
	
	static function lookup_dashlet_files($dashlet_list) {
		$dlet_files = self::get_dashlet_files();
		$chart_files = self::get_chart_files();
		$ret = array();
		foreach($dashlet_list as $id => $dlet) {
			if(! is_array($dlet))
				$dlet = array('className' => $dlet);
			$cls = $dlet['className'];
			if(isset($dlet_files[$cls]))
				$dlet['fileLocation'] = $dlet_files[$cls]['file'];
			else if(isset($chart_files[$cls])) {
				if($chart_files[$cls]['type'] == 'code')
					$dlet['fileLocation'] = "modules/Charts/code/{$cls}.php";
				else if($chart_files[$cls]['type'] == 'report'){
					$dlet['className'] = 'ReportChartDashlet';
					$dlet['fileLocation'] = "modules/Charts/code/ReportChartDashlet.php";
					$dlet['properties']['report_id'] = $chart_files[$cls]['id'];
				}
			} else if(preg_match('~^report_(.*)$~', $cls, $m)) {
				$dlet['className'] = 'ReportChartDashlet';
				$dlet['fileLocation'] = "modules/Charts/code/ReportChartDashlet.php";
				$dlet['properties']['report_id'] = $m[1];
			} else
				continue;
			$ret[$id] = $dlet;
		}
		return $ret;
	}
	
	function get_default_pages() {
		$section_pages = array();
		include('modules/Home/default_pages.php');
		return $section_pages;
	}
	
	function store_default_pages() {
		global $current_user;
		$data = $this->get_default_pages();
		if(! is_array($data) || empty($current_user) || empty($current_user->id))
			$data = array();
		$pages = array();
		foreach($data as $section => $sdata) {
			$dboard = new Dashboard();
			if(isset($sdata['section']))
				$dboard->section = $sdata['section'];
			if(isset($sdata['title']))
				$dboard->name = $sdata['title'];
			if(isset($sdata['icon']))
				$dboard->icon = $sdata['icon'];
			if(isset($sdata['position']))
				$dboard->position = ($sdata['position'] == 'start' ? 0 : 10);
			if(isset($sdata['tab_order']))
				$dboard->tab_order = $sdata['tab_order'];
			$dboard->shared_with = 'all';
			$content = array();
			$ncols = array();
			if(! isset($sdata['columns']))
				$sdata['columns'] = array();
			foreach($sdata['columns'] as $colk => $col) {
				$ncols[$colk] = $col;
				$ncols[$colk]['dashlets'] = array();
				foreach($col['dashlets'] as $dlet) {
					if(! is_array($dlet))
						$dlet = array('className' => $dlet);
					$dlet_id = create_guid();
					if(isset($dlet['options'])) {
						$dboard->set_dashlet_options($dlet_id, $dlet['options']);
						unset($dlet['options']);
					}
					$ncols[$colk]['dashlets'][$dlet_id] = $dlet;
				}
			}
			$content['columns'] = $ncols;
			$dboard->assigned_user_id = '0';
			$dboard->modified_user_id = '1';
			$dboard->set_content($content);
			$dboard->published = array_get_default($sdata, 'published', true) ? 1 : 0;
			$dboard->flat_tab = array_get_default($sdata, 'flat_tab', false) ? 1 : 0;
			$dboard->save() && $dboard->retrieve();
			if($dboard->section && $dboard->id)
				$pages[$dboard->section][$dboard->id] = $dboard;
		}
		return $pages;
	}
	
	
	function get_user_pages($uid=null, $reload=false) {
		static $pages = array();
		if(! isset($uid))
			$uid = AppConfig::current_user_id();
		if(! isset($pages[$uid]) || $reload)
			$pages[$uid] = $this->seedDashboard->fetch_grouped($uid, true);
		return $pages[$uid];
	}
	
	function get_page($id) {
		$page = new Dashboard();
		if($page->retrieve($id))
			// check permissions here?
			return $page;
	}
	
	static function get_section_options() {
		require_once('include/GroupedTabs/GroupedTabStructure.php');
		$top = GroupedTabStructure::get_top_level();
		return $top;
	}
	
	function get_defined_sections() {
		global $app_strings;
		$tabs = $this->get_user_pages();
		if(! $tabs)
			$tabs = array();		
		$top = $this->get_section_options();
		$ret = array();
		foreach($top as $t) {
			if(isset($tabs[$t])) {
				$ret[$t] = $this->sort_tabs($tabs[$t]);
				unset($tabs[$t]);
			}
		}
		foreach($tabs as $lst) {
			$lst = $this->sort_tabs($lst);
			foreach($lst as $k => $tab)
				$ret['LBL_TABGROUP_OTHER'][$k] = $tab;
		}
		return $ret;
	}
	
	function sort_tabs($tabs) {
		$inter = array();
		foreach($tabs as $idx => $t) {
			$inter[$t->tab_order][$idx] = $t;
		}
		ksort($inter);
		$ret = array();
		foreach($inter as $l => $tabs)
			foreach($tabs as $idx => $t)
				$ret[$idx] = $t;
		return $ret;
	}
	
	function get_dashboard_page_info($flat=false) {
		// use session cache here?
		$sects = $this->get_defined_sections();
		$ret = array();
		foreach($sects as $k => $pages) {
			foreach($pages as $pid => $dboard) {
				if($flat && ! $dboard->flat_tab)
					continue;
				$row = array(
					'title' => $dboard->get_title(),
					'position' => $dboard->position,
					'flat_tab' => $dboard->flat_tab,
					'tab_order' => $dboard->tab_order,
					'icon' => $dboard->get_icon(),
				);
				if($flat)
					$ret[$pid] = $row;
				else
					$ret[$k][$pid] = $row;
			}
		}
		return $ret;
	}

}

?>