<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */



class FieldFormatter {
	var $format = 'html';
	var $context = null;
	var $list_context, $edit_context;
	var $primary_key;
	var $module_dirs;
	var $disable_relative_time;
    var $show_detail_in_popup = false;
    var $show_favorite_star = false;
    var $show_social_accounts = true;
    var $listview_primary = null;
    var $list_id;
    var $detail_link_url;
    var $detail_link_action;
	var $display_currency_id;
	var $limit_text_length;
	var $limit_line_length = 120;
	var $offset_dates = true;
	var $remove_whitespace = false;
	var $add_detail_links;
	var $parent_module, $parent_action, $parent_record, $parent_record_dynamic, $parent_layout, $parent_panel;
	var $user;
    
    static $settable_params = array(
    	'primary_key', 'module_dirs', 'listview_primary', 'list_id',
		'show_detail_in_popup', 'display_currency_id', 
		'disable_relative_time',
		'add_detail_links', 'detail_link_url', 'detail_link_action',
		'parent_module', 'parent_action', 'parent_record', 'parent_layout', 'parent_panel',
    );
	
	function __construct($format=null, $context=null, $params=null) {
		$trace = debug_backtrace();
		$this->disable_relative_time = AppConfig::setting('layout.disable_relative_dates_times');
		$this->disable_relative_time |= $format == 'pdf';
		if(isset($format))
			$this->setFormat($format, $context);
		$this->init($params);
	}
	
	function init($params=null) {
		if(is_array($params)) {
			if(isset($params['format']))
				$this->setFormat($params['format'], array_get_default($params, 'context'));
			foreach(self::$settable_params as $p)
				if(isset($params[$p]))
					$this->$p = $params[$p];
		}
		$this->initCurrency();
	}
	
	function initCurrency($currency_id=null) {
		if(! isset($currency_id) && isset($this->display_currency_id))
			return;
		global $current_user;
		if(isset($current_user))
			$this->display_currency_id = $current_user->getPreference('currency');
	}
	
	function getFormat() {
		return $this->format;
	}
	function getContext() {
		return $this->context;
	}
	function setFormat($fmt, $context=null) {
		$this->format = $fmt;
		$this->setContext($context);
	}

	function setUser($user) {
		$this->user = $user;
	}

	function setContext($ctx) {
		$this->context = $ctx;
		$this->list_context = ($ctx == 'listview' || $ctx == 'subpanel' || $ctx == 'popup' || $ctx == 'dashlet' || $ctx == 'unified_search' || $ctx == 'export');
		$this->edit_context = ($ctx == 'editview' || $ctx == 'massupdate' || ($this->format == 'json' || $this->format == 'soap'));
	}
	function setPrimaryKey($key) {
		$this->primary_key = $key;
	}
	
	
	function getFieldWidgets($model, $fields) {
		$wpop = array();
		foreach($fields as $f => $def) {
			if(! empty($def['widget'])) {
				$spec = AppConfig::get_widget_spec($def['widget']);
				if($spec && $spec['type'] == 'field') {
					$mobj = (AppConfig::setting("model.detail.$model.type") == 'bean') ? new ModelDef($model) : null;
					$wobj = new $spec['class']($def, $mobj);
					$wpop[$f] = array('widget' => $wobj);
				}
			}
		}
		return $wpop;
	}
	
	function formatListResult(ListResult &$result) {
		$this->setPrimaryKey($result->primary_key);
		$this->module_dirs = $result->module_dirs;
		if($result->is_union && ! $result->union_key)
			$result->union_key = 'query_source';
		$result->formatted_rows = $this->formatRows($result->fields, $result->rows, $result->union_key);
		$wpop = $this->getFieldWidgets($result->base_model, $result->fields);
		if($wpop) {
			foreach($wpop as $f => $w_spec) {
				$w_spec['widget']->formatList($this, $result);
			}
		}
		return $result->formatted_rows;
	}

	function formatRowResult(RowResult &$result) {
		$this->setPrimaryKey($result->primary_key);
		$this->module_dirs = $result->module_dirs;
		if(! is_array($result->row)) $result->row = array();
		$result->formatted = $this->formatRow($result->fields, $result->row, $result->union_key);
		$wpop = $this->getFieldWidgets($result->base_model, $result->fields);
		if($wpop) {
			foreach($wpop as $f => $w_spec) {
				$result->formatted[$f] = $w_spec['widget']->formatField($this, $result);
			}
		}
		return $result->formatted;
	}
	

	function formatRow(array &$model_fields, array $row) {
		$input = array(&$row);
		$result = $this->formatRows($model_fields, $input);
		return $result[0];
	}
	
	
	function formatRows(array &$model_fields, array &$rows, $union_key=null) {
		$names = $defer = array();
		foreach($model_fields as $name => &$spec) {
			if(isset($spec['formula']))
				// reorder formula fields to end - they may have dependencies
				$defer[] = $name;
			else
				$names[] = $name;
			$t = array_get_default($spec, 'type');
			if($t == 'enum' || $t == 'multienum' || $t == 'status') {
				if(!isset($spec['options']) && isset($spec['options_function']) && empty($spec['options_function_dynamic'])) {
					$cb = $spec['options_function'];
					$spec['options'] = call_hook_function($cb, null);
					if(! isset($spec['options']))
						$spec['options'] = array();
				}
			}
		}
		unset($spec);
		$names = array_merge($names, $defer);
		$result = array();

		foreach($rows as $k => &$row) {
			$erased = array_filter(explode(',', array_get_default($row, '___erased_fields___', '')));
			$fmt = array();
			foreach($names as $name) {
				$spec =& $model_fields[$name];
				if(isset($union_key) && ! empty($row[$union_key])) {
					$spec =& $spec['union_source'][$row[$union_key]];
				}
				if (!$spec) $spec = [];
				$value = array_get_default($row, $name);
				$t = array_get_default($spec, 'type');
				if(! empty($spec['protected'])) {
					$value = null;
				}
				else if(isset($spec['formula'])) {
					$value = $this->formatFormulaValue($spec, $row, $fmt, $value);
				}
				else if(isset($spec['options_function']) && !empty($spec['options_function_dynamic'])) {
					$cb = $spec['options_function'];
					$spec['options'] = call_hook_function($cb, null, array($name, $row, $this->edit_context));
					if(! isset($spec['options']))
						$spec['options'] = array();
					$value = $this->formatRowValue($spec, $row, $value);
				}
				else if($this->context == 'workflow') {
					// render field name when selected as related value in WF editor
					$value = $this->formatRowValue($spec, $row, $value);
				}
				else if(($t != 'id' && $t != 'char' && $t != 'varchar') || (is_array($spec) && ! empty($spec['detail_link']))) {
					if (!is_array($spec)) $spec = [];
					$value = $this->formatRowValue($spec, $row, $value);
				}
				else if($this->format == 'html' && ! $this->edit_context) {
					if (in_array(array_get_default($spec, 'name', ''), $erased))
						$value = '<div class="badge state-error">' . translate('LBL_VALUE_ERASED', 'app') . '</div>';
					else
						$value = to_html($value);
				}
				else if($this->format == 'pdf' && ! $this->edit_context) {
					if (in_array(array_get_default($spec, 'name', ''), $erased))
						$value = strtoupper(translate('LBL_VALUE_ERASED', 'app'));
				}
				$fmt[$name] = $value;
			}
			$result[$k] = $fmt;
		}
		return $result;
	}
	
	
	function getAddDetailLinks() {
		if(isset($this->add_detail_links))
			return $this->add_detail_links;
		else if($this->context == 'popup')
			return 'primary';
		else if($this->list_context && $this->context != 'export')
			return true;
		else if($this->context == 'email')
			return false;
		else if($this->context == 'notify')
			return true;
		else
			return 'not_primary';
		return false;
	}

	function isCSV() {
		return $this->format == 'csv' || $this->format == 'tsv' || $this->format == 'scsv' || $this->format == 'xlscsv';
	}

	function formatRowValue(array &$spec, array &$row, $value) {
		global $locale, $timedate, $image_path, $app_strings, $app_list_strings;
		static $currency_types = array('currency', 'base_currency', 'raw_currency');
		$erased = array_filter(explode(',', array_get_default($row, '___erased_fields___', '')));

		if (in_array(array_get_default($spec, 'name', ''), $erased) && $this->format == 'html')
			return '<div class="badge state-error">' . translate('LBL_VALUE_ERASED', 'app') . '</div>';
		elseif (in_array(array_get_default($spec, 'name', ''), $erased) && $this->format == 'pdf')
			return strtoupper(translate('LBL_VALUE_ERASED', 'app'));

		$is_mobile = AppConfig::is_mobile();
		$formatting_done = false;
		$orig_value = $value;
		$add_detail_link = ! empty($spec['detail_link']) ? $this->getAddDetailLinks() : false;
		
		$editable = $this->edit_context;
		if ($editable) {
			if (isset($spec['editable']) && $spec['editable'] === false) {
				$editable = false;
			}
		}
		$translate_vals = ($this->format == 'html' || $this->format == 'pdf' || $this->format == 'text');
		if ($this->isCSV()) {
			$translate_vals = AppConfig::setting('export.dropdowns') == 'values';
		}
        if(isset($spec['source']) && is_array($spec['source']) && isset($spec['source']['calc_formula'])) {
            $value = $this->calculateValue($spec, $row);
        }		

		$type = array_get_default($spec, 'type');
		if(isset($spec['display_type']))
			$type = $spec['display_type'];
		$format = array_get_default($spec, 'format');
		if (!empty($spec['is_total'])) {
			if ($spec['total'] == 'count') {
				$type = 'int';
			}
		}
		if($type == 'date' || $type == 'datetime') {
			$date_type = $type;
			if($format == 'date_only') {
				$type = 'date';
			}
			else if($format == 'time_only')
				$type = 'time';
			else if($format) {
				$type = $format;
			}
		}

		if(isset($spec['alt_options'])) {
			$kv = array_get_default($row, array_get_default($spec, 'alt_name', $spec['name'].'_alt'));
			if(isset($spec['alt_options'][$kv])) {
				$value = $spec['alt_options'][$kv];
				return '['.$value.']';
			}
		}
		
		if($type == 'ref') {
			if(isset($spec['ref_display_type'])) {
				$spec2 = $spec;
				$spec2['type'] = $spec['ref_display_type'];
				$value = $this->formatRowValue($spec2, $row, $value);
				$formatting_done = true;
			}
		}
		else if($type == 'email' && $this->format == 'html' && $this->context != 'popup') {
			$recip_name = '';
			$parent_type = $this->getModuleDir(array_get_default($spec, 'link', 0));
			$parent_id = array_get_default($row, $this->primary_key, '');

			if (isset($spec['parent_type_field']))
				$parent_type = array_get_default($row, $spec['parent_type_field']);
			if (isset($spec['parent_id_field']))
				$parent_id = array_get_default($row, $spec['parent_id_field']);

			$max_len = array_get_default($spec, 'max_length', $this->list_context ? array_get_default($spec, 'width', 25) : null);
			if($this->context == 'view')
				$link_cls = 'tabDetailViewDFLink';
			else
				$link_cls = 'listViewExtLink';
			$value = $locale->getUserFormatEmail($value, $recip_name, $parent_type, $parent_id, array('link_class' => $link_cls, 'max_length' => $max_len, 'context' => $this->context));
			if($value && ($this->context != 'email' && $this->context != 'notify')) {
				$icon = '<span class="uii uii-sm">e</span>';
				$value = '<span class="nobr">'.$icon . ' ' . $value.'</span>';
			}
			return $value;
		}
		else if($type == 'skype' && $this->format == 'html'
			 && $this->context != 'email' && $this->context != 'notify' && $this->context != 'popup') {

			if($value) {
				$link_cls = 'tabDetailViewDFLink';
				$icon = '<div class="input-icon icon-skype"></div>';
				$quoted_value = to_html(urlencode($value));
				$value = to_html($value);
				$idd = uniqid('skype-');
				$opts = array(
					'call' => array(
						'label' => translate('LBL_SKYPE_CALL'),
						'url' => 'skype:' . $quoted_value . '?call',
					),
					'chat' => array(
						'label' => translate('LBL_SKYPE_CHAT'),
						'url' => 'skype:' . $quoted_value . '?chat',
					),
					'video' => array(
						'label' => translate('LBL_SKYPE_VIDEO'),
						'url' => 'skype:' . $quoted_value . '?call&video=true',
					),
				);

				global $pageInstance;
				$json = getJSONObj();
		    	$s_opts = array('keys' => array_keys($opts), 'values' => $opts, 'width' => '20em');
		    	$jso = $json->encode(array('options' => $s_opts, 'hover_show' => 1, 'delay' => 100));
				$pageInstance->add_js_literal("SUGAR.ui.registerMenuSource('$idd', $jso);", null, LOAD_PRIORITY_FOOT);

				return '<div id="' . $idd . '" class="nobr">' . $icon . ' ' . '<a href="#" onclick="return false" class="tabDetailViewDFLink">' . $value . '</a></div>';
			}
		} else if(($type == 'phone' || $type == 'extension_number' || $type == 'fax') && $this->format == 'html' && $this->context != 'popup') {
			if($this->context == 'view')
				$link_cls = 'tabDetailViewDFLink';
			else
				$link_cls = 'listViewExtLink';
            $parent_type = $this->getModuleDir(array_get_default($spec, 'link', 0));
            $parent_id = array_get_default($row, $this->primary_key, '');
            $max_len = array_get_default($spec, 'max_length', $this->list_context ? array_get_default($spec, 'width', 20) : null);
            if($type == 'phone')
				$value = $locale->getUserFormatPhone($value, array('link_class' => $link_cls, 'parent_id' => $parent_id, 'parent_type' => $parent_type, 'context' => $this->context, 'max_length' => $max_len));
			if($value && ($this->context != 'email' && $this->context != 'notify')) {
				$name = array_get_default($spec, 'name', '');
				$ict = (substr($name, -7) == '_mobile') ? 'm' : 'p';
				$icon = '<span class="uii uii-sm">'.$ict.'</span>';
				$value = '<span class="text-number">' . $value . '</span>';
				$value = '<span class="nobr">' . $icon . ' ' . $value . '</span>';
			}
			return $value;
		}
		else if($type == 'url' && $this->format == 'html' && $this->context != 'popup') {
			if($this->context == 'view')
				$link_cls = 'tabDetailViewDFLink';
			else
				$link_cls = 'listViewExtLink';
			$max_len = array_get_default($spec, 'max_length', $this->list_context ? array_get_default($spec, 'width', 25) : null);
			$value = $locale->getUserFormatExtUrl($value, array('link_class' => $link_cls, 'max_length' => $max_len));
			if($value && ($this->context != 'email' && $this->context != 'notify')) {
				$icon = '<span class="uii uii-sm">u</span>';
				$value = '<span class="nobr">' . $icon . ' ' . $value . '</span>';
			}
			return $value;
		}
		else if(in_array($type, $currency_types) && $this->format != 'json') {
			if($value == 0 && ! $this->edit_context && $this->context != 'chart') {
				if($this->format == 'html')
					return '&mdash;';
				if($this->list_context && $this->format == 'pdf')
					return html_entity_decode('&ndash;', ENT_QUOTES, 'UTF-8');
				if (!($this->format === 'plain' && $this->context === 'email_template'))
					return '';
			}
			if($this->edit_context && ! strlen($value ?? ''))
				return '';
			$dcid = ($this->list_context && $this->context != 'subpanel') ? $this->display_currency_id : null;
			$disable_symbol = 
				$this->edit_context 
				|| $type == 'raw_currency' 
				|| ($this->format != 'html' && $this->format != 'pdf' && !($this->format === 'plain' && $this->context === 'email_template'))
			;
			$fmt = array();
			if($this->format == 'pdf')
				$fmt['type'] = 'pdf';
			if($disable_symbol)
				$fmt['currency_symbol'] = false;
			if(! empty($spec['show_symbol'])) // override for credit card popup
				$fmt['currency_symbol'] = true;
			if(isset($spec['dbType']) && $spec['dbType'] == 'int')
				$fmt['decimals'] = 0;
			if(! empty($spec['extra_precision']) && ! $this->list_context)
				$fmt['round'] = -1;
			if(! empty($spec['exchange_rate'])) {
				$exid = array_get_default($row, $spec['exchange_rate']);
				$fmt['exchange_rate'] = $exid;
			}
			if ($this->format === 'plain' && $this->context === 'email_template')
				$fmt['symbol_space'] = ' ';

			$do_conv = array_get_default($spec, 'display_converted', false);
						
			if($type == 'base_currency') {
				if(! $do_conv)
					$dcid = '-99'; // always display in base currency
				
				if(isset($spec['display_currency_id'])) {
					$dcfn = $spec['display_currency_id'];
					if(isset($row[$dcfn]))
						$dcid = $row[$dcfn];
				}
				if($dcid && $dcid != '-99') {
					// convert base currency to display
					$fmt['convert'] = true;
					$fmt['currency_id'] = $dcid;
				}
				else
					$fmt['currency_id'] = '-99';
			}
			else {
				if(! empty($spec['currency_id']))
					$cid = array_get_default($row, $spec['currency_id'], '-99');
				else
					$cid = '-99'; // assume default currency
				if($do_conv && $dcid && $type == 'currency' && ! empty($spec['base_field'])) {
					$fmt['currency_id'] = $dcid;
					if($cid != $dcid) {
						$value = array_get_default($row, $spec['base_field']);
						$fmt['convert'] = true;
					}
				}
				else {
					// no conversion, display given value in given currency
					$fmt['entered_currency_id'] = $cid;
					$fmt['currency_id'] = $cid;
				}
			}
			if (! empty($spec['decimal_places']))
				$fmt['decimals'] = $spec['decimal_places'];

			$value = currency_format_number($value, $fmt); // HTML-encoded
			if($this->format == 'html') {
				if($this->list_context)
					$value = str_replace(' ', '&nbsp;', $value); // prevent wrapping when thousands separator is space
				if(! $this->edit_context) {
					$value = '<span class="text-number">' . $value . '</span>';
					$formatting_done = true;
				}
			}
			return $value;
		}
		else if($type == 'status' && $this->format == 'html') {
			$dom = array_get_default($spec, 'options');
			$status_dom = array_get_default($spec, 'status_options', $dom);
			if(is_string($status_dom))
				$status_dom = AppConfig::setting('statuses.'.$status_dom);
			$lbl = $dom ? translate_dd($dom, $value) : $value;
			$cls = is_array($status_dom) ? array_get_default($status_dom, $value, 'neutral') : 'neutral';
			if(! strlen($lbl ?? ''))
				return '';
			if($this->context == 'email' || $this->context == 'notify') {
				return '<table class="badge state-'.$cls.'"><tr><td>'.to_html($lbl).'</td></tr></table>';
			} else {
				return '<div class="badge state-'.$cls.'">'.to_html($lbl).'</div>';
			}
		}
		else if(($type == 'enum' || $type == 'status') && $translate_vals) {
			if(isset($spec['options']))
				$value = translate_dd($spec['options'], $value);
			if(is_array($value))
				$value = array_get_default($value, 'name', '');
			if(! strlen($value ?? '') && $this->format == 'pdf' && ! empty($spec['grouped']))
				$value = translate('LBL_MISSING_VALUE', 'app');
		}
		else if($type == 'multienum' && $translate_vals) {
			$val = $value;
			if (!is_array($val)) {
				if (!is_string($val))
					$val = [];
				else
					$val = explode('^,^', $val);
			}

			if(isset($spec['options'])) {
				$v2 = array();
				foreach($val as $v)
					$v2[] = translate_dd($spec['options'], $v);
			} else {
				$v2 = $val;
			}
			natcasesort($v2);
			$value = implode(', ', $v2);
			if(! strlen($value ?? '') && $this->format == 'pdf' && ! empty($spec['grouped']))
				$value = translate('LBL_MISSING_VALUE', 'app');
		}
		else if($type == 'module_name' && $translate_vals) {
			$dom = $app_list_strings['moduleList'];
			$value = array_get_default($app_list_strings['moduleList'], $value, $value);
		}
		else if($type == 'date' && $this->format != 'soap') {
			$value = $timedate->to_display_date($value, $date_type == 'datetime' && $format != 'date_only' && $this->offset_dates, $this->user);
			if($this->format == 'html' && ! $this->edit_context) {
				if(strlen($value ?? ''))
					$value = '<nobr class="text-number">' . to_html($value) . '</nobr>';
				$formatting_done = true;
			}
		}
		else if($type == 'time' && $this->format != 'soap') {
			$value = $timedate->to_display_time($value, true, false, $this->user);
			if($this->format == 'html' && ! $this->edit_context) {
				if(strlen($value ?? ''))
					$value = '<nobr class="text-number">' . to_html($value) . '</nobr>';
				$formatting_done = true;
			}
		}
		else if($type == 'datetime' && $this->format != 'soap') {
			if(! $this->disable_relative_time && empty($spec['disable_relative_format']) && ! $this->edit_context && $this->context != 'email' && $this->context != 'notify') {
				if(isset($value)) {
					$value = $timedate->to_relative_date_time($value, $this->offset_dates ? null : array('no_offset' => true), $this->user);
				}
			} else
                $value = $timedate->to_display_date_time($value, true, $this->offset_dates, $this->user);
            if(strlen($value ?? '') && $this->format == 'html' && ! $this->edit_context) {
				$value = '<span class="text-number">' . $value . '</span>';
				$formatting_done = true;
            }

			$name = array_get_default($spec, 'name', '');
            if (($name == 'date_modified' || $name == 'date_entered') && ($this->format == 'html' && ! $this->edit_context && ! $this->list_context))
                return $this->addMetaChangesTooltip($name, $value, $row);
		}
		else if($type == 'month' && ($translate_vals || $this->isCSV())) {
			$value = array_get_default($app_list_strings['months_long_dom'], $value, $value);
		}
		else if($type == 'duration') {
			if ($format == 'hours' || $format == 'minutes' || ! $translate_vals) {
				if($format != 'hours' && $format != 'minutes' && isset($value))
					$value = (int)$value / 60;
				if(isset($value)) $value = format_number($value, 2);
			} else {
	        	$all_day = array_get_default($spec, 'all_day_flag');
    	    	if($all_day && array_get_default($row, $all_day))
        			$value = $app_strings['LBL_DURATION_ALL_DAY'];
	        	else {
					if(empty($value) || $value == 0) {
						if($this->format == 'html' && $this->context != 'chart')
							return '&mdash;';
						return '';
					}
					$hours = ($value < 60) ? 0 : floor($value / 60);
					$minutes = ($value < 60) ? $value : ($value - ($hours * 60));
					if ($minutes < 10) $minutes = '0' . $minutes;
					if(($this->context == 'export' && $this->format != 'pdf') || $this->format == 'json')
						$value = $hours . ':' . $minutes;
					else {
						$value = $hours . $app_strings['LBL_HOURS_ABBREV'] ." ". $minutes . $app_strings['LBL_MINS_ABBREV'];
						if($this->format == 'html' && ! $this->edit_context) {
							$value = '<span class="text-number">' . $value . '</span>';
							$formatting_done = true;
						}
					}
				}
			}
        }
        else if($type == 'reminder' && $translate_vals) {
        	$dom = array_get_default($spec, 'options', 'reminder_time_options');
			$opts = array_get_default($app_list_strings, $dom, array());
			$value = array_get_default($opts, $value, $value);
        }
		else if(($type == 'item_number' || $type == 'sequence_number') && $this->format == 'html') {
			if(! $value) {
				if($type == 'sequence_number')
					return '<em>' . to_html(translate('LBL_UNDETERMINED_VALUE', 'app')) . '</em>';
				if($spec['source']['type'] == 'sequence' && ! empty($spec['source']['fields'])) {
					$sp = $spec['source']['fields'][0];
					$spval = array_get_default($row, $sp);
					if($spval)
						return '<em class="text-number">' . to_html($spval) . '?</em>';
					else
						return '<em>' . to_html(translate('LBL_UNDETERMINED_VALUE', 'app')) . '</em>';
				}
			}
			else if(! $this->edit_context) {
				$value = '<span class="itemNumber text-number" style="white-space: nowrap">'.to_html($value).'</span>';
				$formatting_done = true;
			}
		}
		else if(($type == 'rate_percent' || $type == 'percentage') && $this->format != 'json') {
			$pfp = array('percentage' => true);
			$dec = array_get_default($spec, 'decimal_places', 2);
			$value = format_number($value, $dec, -1, $pfp);
			if(! $this->edit_context && $this->format == 'html') {
				$value = '<span class="text-number">' . $value . '</span>';
				$formatting_done = true;
			}
		}
		else if(($type == 'float' || $type == 'double' || $type == 'decimal') && $this->format != 'json') {
			if(($this->edit_context || ! empty($spec['no_format_blank'])) && ! isset($value))
				return '';
			if(empty($spec['disable_num_format'])) {
				$decimals = 2;
				if(isset($spec['dbType']) && $spec['dbType'] == 'int')
					$decimals = 0;
				else if(isset($spec['decimal_places']))
					$decimals = $spec['decimal_places'];
				$precision = $decimals;
				if(! empty($spec['extra_precision']))
					$decimals = -1;
				$value = format_number($value, $decimals, $precision);
				if(! $this->edit_context && $this->format == 'html') {
					$value = '<span class="text-number">' . $value . '</span>';
					$formatting_done = true;
				}
			}
		}
		else if($type == 'bool') {
			if($this->format == 'html') {
				if($this->context == 'email' || $this->context == 'notify') {
					return translate_dd('dom_int_bool', $value ? 1 : 0);
				}
				if($this->context == 'editview' || $this->context == 'view') {
					$subclass = $value ? ' checked' : '';
					return '<div class="input-toggle readonly' . $subclass . '"><label><div class="switch"></div></label></div>';
				}
				return '<span class="text-disabled uii uii-' . ($value ? 'checked' : 'checkbox').'"></span>';
			}
			else if($this->format == 'pdf') {
				return translate_dd('dom_int_bool', $value ? 1 : 0);
			}
		}
        else if($type == 'password') {
            return '********';
        }
		else if($type == 'age') {
			if(empty($value)) return '';
			if(array_get_default($spec['formula'] ?? [], 'format') == 'months') {
				list($year, $month, $day) = sscanf($value, "%d-%d-%d");
				list($cday, $cmonth, $cyear) = array_slice(localtime(), 3, 6);
				$cyear += 1900; $cmonth ++;
				$value = self::date_difference($year, $month, $day, $cyear, $cmonth, $cday);
			} else {
				$hourlen = 3600;
				$daylen = $hourlen*24;
				$now = strtotime($timedate->get_gmt_db_datetime());
				$age = $now - strtotime($value);
				$days = floor($age / $daylen);
				$age -= $days * $daylen;
				$hours = floor($age / $hourlen);
				$age -= $hours * $hourlen;
				$lbl = $app_strings['LBL_TIME_PERIOD_DAYS_HOURS'];
				$value = str_replace(array('%1', '%2'), array((int)$days, (int)$hours), $lbl);
			}
		}
		else if($type == 'status_color' && $this->format == 'html') {
			if($value)
				return '<span class="status-icon uii uii-led led-' . $value . '"></span>';
			return "";
		}
		else if($type == 'attach_icon') {
			if($this->format == 'html' && $value)
				return get_image('attachment', 'alt="" title="' . translate('LBL_HAS_ATTACH', 'app') . '"');
			else
				return '';
		}
		else if($type == 'replied_icon') {
			if($this->format == 'html' && $value) {
				return get_image('replied', '');
			}
			else
				return '';
		}
		else if($type == 'address') {
            $fs = array();
			if (isset($spec['source']['alias_fields']))
                $fs = $spec['source']['alias_fields'];
            else if(isset($spec['source']['fields']))
            	$fs = $spec['source']['fields'];
            else
            	$fs = array();
            if(isset($spec['source']['prefix']))
            	$prefix = $spec['source']['prefix'];
            else
            	$prefix = '';
			$addr = array($prefix.'module' => $this->getModuleDir(), $prefix.'record' => array_get_default($row, 'id'));
			foreach($fs as $part => $field) {
				if(is_int($part)) $part = $field;
				$p = array_get_default($row, $field);
				if(isset($p) && $this->format == 'html')
					$p = to_html($p);
				if($part == 'account' || $part == 'contact') {
					$addr[$prefix.$part.'_field'] = $field;
					$part .= '_name';
				}
				$addr[$prefix.$part] = $p;
			}
			if($this->format == 'pdf')
				$dformat = 'display_pdf';
			elseif($this->format == 'html' && $this->context == 'listview')
				$dformat = 'display_list';
			elseif ($this->context == 'export' || $this->format == 'json')
				$dformat = $this->format;
			else
				$dformat = 'display';
			$value = $locale->getLocaleFormattedAddress($addr, $prefix, $dformat, null, true);
			return $value;
		}
        else if ($type == 'html' && ! $this->edit_context) {
            if ($this->format != 'html')
                $value = strip_tags($value);
            return $value;
        }
		else if ($type == 'image' && $this->format == 'html') {
			$row_id = array_get_default($row, 'id');
			require_once 'include/upload_file.php';
			if(empty($spec['store_full_path']))
				$img_dir = array_get_default($spec, 'upload_dir');
			else
				$img_dir = '';
			$alt_img_dir = array_get_default($spec, 'alt_upload_dir', '');

			$filename = array_get_default($spec, 'filename');
			$alt_filename = array_get_default($spec, 'alt_filename');
			$thumb = array_get_default($spec, 'thumbnail_name');
			$alt_thumb = array_get_default($spec, 'alt_thumb');
			$thumb_file = $value = null;
			if($filename) $value = array_get_default($row, $filename);
			if($thumb && $value) $thumb_file = array_get_default($row, $thumb);
			$value = UploadFile::get_path($value, '', $img_dir);
			if(! $thumb_file)
				$thumb_file = $value;
			else
				$thumb_file = UploadFile::get_path($thumb_file, '', $img_dir);

			$placeholder_setting = array_get_default($spec, 'placeholder_config');
			if ($placeholder_setting)
				$default_image = AppConfig::setting($placeholder_setting, '', true);
			else
				$default_image = array_get_default($spec, 'placeholder', AppConfig::setting('model.supported_types.image.placeholder'));

			$dsize = array_get_default($spec, 'display_size', array(80, 80));
			if($this->list_context)
				$dsize = array_get_default($spec, 'list_display_size', array(40, 40));
			else if(AppConfig::is_mobile())
				$dsize = array(60, 60);
			$thumb = resolve_image($thumb_file, $dsize);
			$image = resolve_image($value, $dsize, $default_image);
			if(! $thumb) {
				if((! $image || ! empty($image[4])) && $alt_filename && $alt_thumb) {
					$alt_path = UploadFile::get_path(array_get_default($row, $alt_filename), '', $alt_img_dir);
					$alt_tpath = UploadFile::get_path(array_get_default($row, $alt_thumb), '', $alt_img_dir);
					if($alt_path) {
						$alt_image = resolve_image($alt_path, $dsize);
						if($alt_image) {
							$image = $alt_image;
							$image[4] = 1; // placeholder
							$thumb = resolve_image($alt_tpath, $dsize);
							if(! $thumb) $thumb = $image;
						} else
							return '';
					} else {
						if(! $image) return '';
						$thumb = $image;
					}
				} else {
					if(! $image) return '';
					$thumb = $image;
				}
			}
			
			$path = to_html($thumb[0]);
			$attr = "width=\"{$thumb[1]}\" height=\"{$thumb[2]}\"";
			if(! empty($thumb[4])) $attr .= " data-placeholder=\"1\"";
			$value = "<img src=\"$path\" border=\"0\" alt=\"\" $attr>";
			$s = '';
			if($thumb[1] != $dsize[0]) {
				$s = "width: {$dsize[0]}px";
			}
			if($thumb[2] < $dsize[1]) {
				$pad = floor(($dsize[1] - $thumb[2])/2);
				$s = "padding-top: {$pad}px; padding-bottom: {$pad}px";
			} 
			if($this->context != 'email' && $this->context != 'notify') {
				$value = "<div class=\"image-wrap\" style=\"$s\">$value</div>";
				if($image) {
					if(AppConfig::is_mobile())
						$value = '<a href="' . to_html($image[0]) . '">' . $value . '</a>';
					else {
						if(isset($spec['label']))
							$label = $spec['label'];
						else if(isset($spec['vname']) && isset($spec['vname_module']))
							$label = translate($spec['vname'], $spec['vname_module']);
						else
							$label = '';
						$href = "SUGAR.ui.Lightbox.show('" . javascript_escape($image[0]) . "', '" . javascript_escape($label) . "'); return false;";
						$value = '<a href="#" onclick="' . to_html($href) . '">' . $value . '</a>';
					}
				}
			}
			return $value;
        }
		else if(($type == 'file_ref') && $this->format == 'html' && $value) {
			$row_id = array_get_default($row, 'id');
			require_once 'include/upload_file.php';
			$file_path = UploadFile::get_path($value, '', array_get_default($spec, 'upload_dir'));

			if(file_exists($file_path)) {
				if(($p = strrpos($value, '/')) !== false)
					$fname = to_html(substr($value, $p+1));
				else
					$fname = to_html($value);
				$mod = $this->getModuleDir();
				if($this->context == 'view')
					$link_cls = 'tabDetailViewDFLink';
				else
					$link_cls = 'listViewExtLink';

				$ver = substr(md5(filemtime($file_path)), -5);
                $dl_link = "download.php?type=$mod&id={$row_id}&ver={$ver}&field=".urlencode($spec['name']);
                $mime_field = array_get_default($spec, 'file_mime_type');
                $mime_type = '';
                if ($mime_field && isset($row[$mime_field]))
                    $mime_type = $row[$mime_field];

                $value = $this->getFileLink($fname, $dl_link, $mime_type, $link_cls);
				$size = format_file_size(filesize($file_path));
				$value .= " ($size)";
			} else {
				$value = '<ss>'.to_html($value).'</ss>';
			}
			return $value;
		}
		else if($type == 'file_size' && $this->format != 'json') {
			$value = format_file_size($value);
		}
        else if($type == 'weight') {
            if (! isset($spec['source']['alias_fields']))
                $value = '';
            else {
                $fv = array();
                foreach($spec['source']['alias_fields'] as $f)
                    $fv[] = array_get_default($row, $f);

                $value = $locale->getUserFormatWeighty($fv);
            }
            return $value;
        }
		else if($type == 'location_city') {
			if (! isset($spec['source']['alias_fields']))
				$value = '';
			else {
				$fv = array();
				foreach($spec['source']['alias_fields'] as $f)
					$fv[] = array_get_default($row, $f);
				$value = $locale->getUserFormatCity($fv);
			}
		}
		else if($type == 'progress') {
			if($this->format == 'html') {
				if($this->context == 'email' || $this->context == 'notify')
					return strlen($value ?? '') ? $value . '%' : '';
				$sz = (int)$value;
				$w = 150;
				$cls = '';
				if($this->list_context) {
					$w = 60;
					$cls = 'input-sm';
				}
				$minlw = 30;
				$ip = round(min(max($sz, 0), 100), 2);
				$iw = round($w * $sz / 100);
				$lbl = '<label class="input-label"><span class="text-number">'.$sz.'</span><small>&#8202;%</small></label>';
				$bar1 = '<div class="input-progress-bar" style="width: '.$ip.'%">';
				$bar2 = '<div class="input-progress-bar empty">';
				if($sz < 50 && $w-$iw >= $minlw)
					$bar2 .= $lbl;
				else
					$bar1 .= $lbl;
				$value = '<div class="input-field input-progress-outer '.$cls.' readonly" style="width: '.$w.'px"><div class="input-progress">'.$bar1.'</div>'.$bar2.'</div></div></div>';
			} else if($this->format == 'pdf') {
				$value = (int)$value . '%';
			}
			return $value;
		}
		else if($type == 'color' && $this->format == 'html' && ($this->context != 'email' && $this->context != 'notify')) {
			$s = $value ? '" style="background-color: '.to_html($value).'"' : ' disabled"';
			$ret = '<div class="input-icon input-color-icon'.$s.'></div>&nbsp;'; //</div>';
			$ret .= ($value ? to_html($value) : '<span class="text-placeholder">' . translate('LBL_NONE') . '</span>');
			return $ret;
		}

		if($this->format == 'html') {
			if(! $formatting_done && ! empty($spec['multiline'])) {
				$maxlen = array_get_default($spec, 'max_length', $this->list_context ? array_get_default($spec, 'width') : null);
				$max_lines = 0;
				if ($this->list_context)
					$max_lines = array_get_default($spec, 'list_max_lines', $max_lines);
				$value = $this->formatMultilineText($value, ($type == 'text' && $this->list_context), $maxlen, $max_lines);
				$formatting_done = true;
			}
			
			// add link if needed
			if($add_detail_link) {
				$primary = ($type != 'ref' && empty($spec['link']));
				if(($add_detail_link !== 'primary' || $primary) && ($add_detail_link !== 'not_primary' || ! $primary)) {
					$link_val = ($type == 'enum' ? $value : $orig_value);
					$value = $this->formatDetailLink($spec, $row, $value, $formatting_done, $link_val, (! $primary || $add_detail_link === 'all'));
					$formatting_done = true;
				}
			}
			
			if($type == 'name' && $this->context == 'view' && empty($spec['link'])) {
				$idv = array_get_default($row, 'id');
				if($idv) {
					if ($this->show_favorite_star) {
						if(! $formatting_done) $value = to_html($value);
						$value = $this->addFavoriteStar($row, $value, $idv);
						$formatting_done = true;
					}

					if($this->show_social_accounts) {
						require_once('modules/SocialAccounts/SocialAccount.php');
						$social_accounts = new SocialAccount(array('related_type' => $this->getModuleDir(), 'related_id' => $idv));
						$social_icons = $social_accounts->getIconsChain();
						if (strlen($social_icons)) {
							if(! $formatting_done) $value = to_html($value);
							$value .= ' ' . $social_icons;
							$formatting_done = true;
						}
					}
				}
			}

			if(! $this->edit_context && ! $formatting_done)
				$value = to_html($value);
		}
		
		if($this->format == 'pdf' && ! empty($spec['grouped']) && ! strlen($value ?? ''))
			$value = translate('LBL_MISSING_VALUE', 'app');
		
		return $value;
	}
	
	function formatDetailLink($spec, $row, $display_value, $formatted=false, $value=null, $add_info=false) {
		global $themeInstance;
		if(! $formatted) {
			if(! isset($value))
				$value = $display_value;
			$display_value = to_html($value);
		}
		$type = array_get_default($spec, 'type');
		
		$icon = '';
		$cls = '';
		if ($this->context == 'view') {
			$cls = 'tabDetailViewDFLink';
		} else if ($type == 'name' && $this->context != 'dashlet') {
			$cls = 'listViewNameLink';
		} else /*if($this->context != 'popup' && $this->context != 'export')*/ {
			$cls = 'listViewExtLink';
		}
		
		if(isset($spec['dynamic_module']))
			$detail_mod = array_get_default($row, $spec['dynamic_module']);
		else if($type == 'ref' && isset($spec['bean_name']))
			$detail_mod = AppConfig::module_for_model($spec['bean_name']);
		else {
			$link = array_get_default($spec, 'link', 0);
			$detail_mod = array_get_default($spec, 'module_dir', $this->getModuleDir($link));
		}
		$primary_bean = AppConfig::module_primary_bean($detail_mod);
		
		
		if($type == 'ref') {
			$detail_id_key = array_get_default($spec, 'id_name', $spec['name'].'_id');
			if(! $this->list_context || AppConfig::is_mobile() || ! empty($spec['detail_icon'])) {
				if($detail_mod == 'Users') $icon = 'bean-User';
				else $icon = 'module-' . $detail_mod;
				$icon = get_image($icon, '');
				$icon = $icon ? $icon . '&nbsp;' : '';
			}
		} else {
			$link = array_get_default($spec, 'link');
			if ($link)
				$detail_id_key = $link . '.id';
			else if(is_array($this->primary_key) && count($this->primary_key) == 1)
				$detail_id_key = $this->primary_key[0];
			else
				$detail_id_key = $this->primary_key;
		}
		if(! empty($detail_id_key))
			$detail_id = array_get_default($row, $detail_id_key);
		
		if($type == 'name') {
			if(! strlen(trim($display_value ?? '')) && ! $this->edit_context) {
				$display_value = translate('LBL_MISSING_VALUE', 'app');
				if($cls) $cls .= ' missing';
			}
			
			if($this->list_context && empty($spec['link']) && $primary_bean && strlen($display_value ?? '') && ! empty($detail_id)) {
				if($this->context == 'listview') {
					$std = AppConfig::setting("model.detail.$primary_bean.standard_records");
					if(is_array($std) && in_array($detail_id, $std)) {
						$display_value .= ' <span class="requiredStar">*</span>';
					}
				}
				if(AppConfig::is_mobile()) {
					if($detail_mod == 'Users') $icon = 'bean-User';
					else $icon = 'module-' . $detail_mod;
					$icon = get_image($icon, '');
					$icon = $icon ? $icon . '&nbsp;' : '';
				}
			}
		}
		
		if(! strlen($display_value ?? ''))
			return '';
		else if(empty($detail_mod) || empty($detail_id) || ! ACLController::checkAccess($detail_mod, 'view'))
			return '<span class="detailLink">' . $icon . $display_value . '</span>';
		
		$relate_arr = array();
		$relate_str = '';
		$parent_id = $this->parent_record;
		if($this->parent_record_dynamic) {
			$parent_id = array_get_default($row, $this->parent_record_dynamic);
		}
		if($this->parent_module && $parent_id) {
			$relate_arr['return_module'] = $this->parent_module;
			$relate_arr['return_action'] = $this->parent_action ? $this->parent_action : 'DetailView';
			$relate_arr['return_record'] = $parent_id;
			$relate_arr['return_layout'] = $this->parent_layout;
			if($this->parent_panel) $relate_arr['return_panel'] = $this->parent_panel;
			$relate_str = encode_query_string($relate_arr);
		}
		
		$no_link_mobile = AppConfig::is_mobile() && ! AppConfig::is_mobile_module($detail_mod);
		$subpanel_async = ($this->context == 'subpanel' && $themeInstance->layout_version >= 8);
		
		if($no_link_mobile) {
			$detail_link = $display_value;
		}
		else if ($this->show_detail_in_popup) {
			$detail_link = get_popup_detail_link($detail_id, $detail_mod, $display_value, $this->show_detail_in_popup, $this->list_id);
			$detail_link = '<a class="'.$cls.'" href="#" onclick="'.to_html($detail_link).'">'.$display_value.'</a>';
		}
		else if(isset($this->detail_link_action) || isset($this->detail_link_url)) {
			$detail_link = isset($this->detail_link_url) ? $this->detail_link_url : '#';
			$detail_action = isset($this->detail_link_action) ? $this->detail_link_action : '';
			$repl = array(
				'{MODULE}' => $detail_mod,
				'{NAME}' => addcslashes($value, '"\''),
				'{ID}' => addcslashes($detail_id, '"\''),
				'{RETURN}' => $relate_str,
			);
			$detail_link = str_replace(array_keys($repl), $repl, $detail_link);
			if($detail_action) {
				$repl['{URL}'] = addcslashes($detail_link, '\'"');
				$detail_action = str_replace(array_keys($repl), $repl, $detail_action);
				$onclick = " onclick=\"$detail_action\"";
			} else
				$onclick = " onclick=\"SUGAR.util.handleLink(event)\"";
			$detail_link = '<a class="'.$cls.'" href="'.to_html($detail_link).'"'.$onclick.'>'.$display_value.'</a>';
		}
		else if(($this->listview_primary || $this->context == 'sidebar' /* || $subpanel_async*/ || $this->context == 'dashlet') && $type != 'ref' && empty($spec['link'])) {
			$link_info = get_detail_link_info($detail_id, $detail_mod, $this->list_id, $relate_arr);
			$detail_link = '<a class="'.$cls.'" href="'.$link_info['url'].'" onclick="'.$link_info['async'].'">'.$display_value.'</a>';
		}
		else {
			$detail_link = get_detail_link($detail_id, $detail_mod, $relate_arr);
			$detail_link = '<a class="'.$cls.'" href="'.$detail_link.'" onclick="SUGAR.util.handleLink(event)">'.$display_value.'</a>';
		}
		
		$detail_link = $icon . $detail_link;
		$scls = 'detailLink';
		if($detail_mod == 'Users' && $detail_id == AppConfig::current_user_id())
			$scls .= ' detailCurrent';
		else if($detail_mod == 'SecurityGroups') {
			$teams = AppConfig::setting('acl.user_teams.'.AppConfig::current_user_id(), array());
			if(isset($teams[$detail_id]))
				$scls .= ' detailCurrent';
		}
		
		if($this->context == 'email' || $this->context == 'notify') $add_info = false;
		if($add_info && ! AppConfig::is_mobile() && (! isset($spec['bean_name']) || $primary_bean == $spec['bean_name'])) {
			$info_id = ($this->list_id ? $this->list_id : 'DetailForm_') . array_get_default($spec, 'name', '') . '_info';
			if($this->list_id) $info_id .= '_' . array_get_default($row, 'id');
			$detail_link = '<span id="' . $info_id . '" class="'.$scls.'" '
				. 'onmouseout="SUGAR.util.clearAdditionalDetailsCall()" '
				. 'onmousedown="SUGAR.util.clearAdditionalDetailsCall()" '
				. "onmouseover=\"return SUGAR.util.getAdditionalDetails('$primary_bean', '$detail_id', '$info_id', null, '{$this->context}', 750)\" "
				. '>' . $detail_link . '</span>';
		} else
			$detail_link = '<span class="'.$scls.'">' . $detail_link . '</span>';
		
		return $detail_link;
	}
	
	function addFavoriteStar($row, $value, $id) {
        require_once('modules/Favorites/Favorite.php');
        $favorite = array_get_default($row, 'favorite');
        $favorite_link = Favorite::get_star($this->getModuleDir(), $id, null, $favorite);
        if($this->show_favorite_star === 'after')
        	return $value . '&nbsp;' . $favorite_link;
        return $favorite_link .'&nbsp;'. $value;
    }

    function getFileLink($name, $download_link, $image_mime_type, $css_class) {
        $mime_type = explode('/', $image_mime_type);
		$click = '';
        if (isset($mime_type[0]) && $mime_type[0] == 'image') {
        	$dl_title = javascript_escape($name);
            $href = "#";
            $click = "onclick=\"SUGAR.ui.Lightbox.show('{$download_link}', '{$name}'); return false;\"";
            $target = '';
        } else {
            $href = $download_link;
            $target = "target=\"_blank\"";
        }
        $link = "<a href=\"{$href}\" $click {$target} class=\"$css_class\">$name</a>";

        return $link;
    }

	function formatFormulaValue(array &$spec, array &$row, array &$formatted, $value) {
		$raw_vals = array();
		$fmt_vals = array();
		$html = '';

		if(isset($spec['formula']['alias_fields']))
			$fields = $spec['formula']['alias_fields'];
		elseif(isset($spec['source']['alias_fields']))
			$fields = array_flip($spec['source']['alias_fields']);
		else if(isset($spec['formula']['fields'])) {
			$fields = array();
			foreach($spec['formula']['fields'] as $f)
				$fields[$f] = $f;
		} else
			$fields = array();

		foreach($fields as $f => $orig_f) {
			$html .= array_get_default($row, $f);
			$raw_vals[$orig_f] = array_get_default($row, $f);
			$fmt_vals[$orig_f] = array_get_default($formatted, $f);
		}

		if($spec['formula']['type'] == 'function') {
			$spec['values'] = $fmt_vals;
			$spec['raw_values'] = $raw_vals;
			return call_hook_function($spec['formula'], null, array($spec, $this, 'display'));
		} else if ($spec['formula']['type'] == 'html2plaintext') {
			require_once 'include/utils/html_utils.php';
			return html2plaintext($html, array_get_default($spec['formula'], 'wrap', true));
		}
		else
			return $this->formatRowValue($spec, $row, $value);
	}
	
	
	function formatMultilineText($value, $smaller=true, $maxlen=null, $max_lines = 0) {
		if($this->format == 'html') {
			if($this->limit_line_length) $maxline = $this->limit_line_length;
			else if($this->list_context) $maxline = 30;
			else $maxline = 0;
			if($maxline) {
				// break long lines
				$value = preg_replace('~(\S{'.$maxline.'})(?=\S)~u', "\${1}\n", $value ?? '');
			}
			if(! isset($maxlen)) {
				if($this->limit_text_length) $maxlen = $this->limit_text_length;
				else if($this->list_context) $maxlen = AppConfig::setting('layout.list.long_text_limit', 150);
				else $maxlen = 0;
			}
			if($maxlen) {
				$value = ltrim($value ?? '');
				$value = str_limit_length($value, $maxlen, true, true);
			}
			$display = to_html($value ?? '');
			if($this->list_context)
				// limit newlines
				$display = preg_replace('~(\r?\n)(\r?\n)+~u', '\1', $display);
			if (!$max_lines && $this->list_context) {
				$max_lines = 5;
			}
			$max_lines_style = '';
			if ($max_lines) {
				// assume themes have line-height:12pt and oveflow:hidden for listLongText
				$max_lines_style="max-height: " . (12 * $max_lines) . 'pt;';
			}
			$display = nl2br($display ?? '');
			if (strlen($display)) {
				if($smaller)
					$display = '<span style="' . $max_lines_style . '" class="listLongText">'.$display.'</span>';
				else {
					// preserve leading spaces
					$display = preg_replace_callback('~^[ \t]+~um', function($m) {
						$ret = '';
						foreach(str_split($m[0]) as $c) {
							if($c == ' ') $ret .= '&nbsp;';
							else $ret .= '&nbsp;&nbsp;&nbsp;&nbsp;';
						}
						return $ret;
					}, $display);
				}
				return $display;
			} else
				return "";
		}
		return $value;
	}	
	
	function unformatRow(array &$model_fields, array $input) {
		global $timedate;
		static $numtypes = array(
			'int', 'float', 'double', 'uint', 'ulong', 'long',
			'short', 'tinyint', 'duration',
			'currency', 'base_currency', 'raw_currency', 'decimal',
			'percentage', 'rate_percent',
		);
		
		$ftypes = $defer = array();
		foreach($model_fields as $name => &$spec) {
			if(isset($spec['source']) && is_array($spec['source']) && $spec['source']['type'] !== 'db' && $spec['source']['type'] !== 'custom_field')
				continue;
			$t = $spec['type'];
			$ftypes[$name] = $t;
		}
		unset($spec);

		$ret = array();
		foreach($input as $name => $value) {
			if(isset($ftypes[$name])) {
				$type = $ftypes[$name];
				$spec =& $model_fields[$name];
				if ($type == 'rate_percent' || $type == 'percentage') {
					$value = rtrim($value, " \t\r\n%");
				}

				if($type == 'date' && $this->format != 'soap') {
					$value = $timedate->to_db_date($value, false);
				}
				else if($type == 'time' && $this->format != 'soap') {
					$value = $timedate->to_db_time($value, false);
				}
				else if($type == 'datetime' && $this->format != 'soap') {
					if(array_get_default($spec, 'format') == 'date_only') {
						if(strlen($value ?? '') && strpos($value, ':') === false)
							$value .= ' ' . gmdate($timedate->get_time_format(), strtotime('today 00:00 GMT'));
                        $value = $timedate->to_db($value, false);
                    } else {
                        $value = $timedate->to_db($value, $this->offset_dates);
                    }
				}
				else if(in_array($type, $numtypes) && empty($spec['disable_num_format'])) {
					// round decimal places here? probably better done in RowUpdate
					$value = unformat_number($value);
				}
				// temporary fix for html output from CKeditor 3.6.3
				else if($type == 'html') {
					$value = preg_replace('~<body id="cke_pastebin".*?</body>~s', '', $value);
				}
				else if($this->remove_whitespace && is_string($value) && $type != 'text' && $type != 'blob') {
					$value = trim($value);
				}
			}
			$ret[$name] = $value;
		}
		return $ret;
	}

	function getModuleDir($link=0) {
		return array_get_default($this->module_dirs, $link, '');
	}

    function calculateValue(array $spec, array $row) {
        require_once('include/layout/FormulaParser.php');
        $parser = new FormulaParser($spec['source']['calc_formula']);
        return $parser->calculate($row);
    }

    /**
     * Add Modified / Created user's name tooltip
     *
     * @param string $date_field
     * @param string $date_value
     * @param array $row
     * @return string
     */
    function addMetaChangesTooltip($date_field, $date_value, array $row) {
        $user_field = '';
        $label = 'LBL_BY';
        if ($date_field == 'date_modified') {
            $user_field = 'modified_user';
            $label = 'LBL_MODIFIED';
        } elseif ($date_field == 'date_entered') {
            $user_field = 'created_by_user';
            $label = 'LBL_CREATED';
        }

        $value = $date_value;

        if ($user_field != '' && ! empty($row[$user_field])) {
            global $app_strings;
            $by = javascript_escape($app_strings[$label] .' <strong>'. $row[$user_field] .'</strong>');
            $value = '<span class="uii uii-info active-icon" onmousedown="this.onmouseover()" onmouseover="return SUGAR.popups.tooltip(\'' .$by. '\', this);"></span>&nbsp;' .$value;
        }

        return $value;
    }

	static function date_difference($y1, $m1, $d1, $y2, $m2, $d2) {
		$d1_earlier_in_year = ($m2 > $m1 || ($m2 == $m1 && $d2 >= $d1));
		if($y1 > $y2 || ($y1 == $y2 && !$d1_earlier_in_year))
			return '';
		$years = $y2 - $y1;
		if($d1_earlier_in_year) {
			$months = $m2 - $m1;
		}
		else {
			$years --;
			$months = 12 - $m1 + $m2;
		}
		if($d2 < $d1)
			$months --;
		$lbl = $GLOBALS['app_strings']['LBL_TIME_PERIOD_YEARS_MONTHS'];
		$lbl = str_replace(array('%1', '%2'), array((int)$years, (int)$months), $lbl);
		return $lbl;
	}
	
}
