<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

require_once('include/layout/forms/FilterForm.php');


class FilterEditor {

	var $module;
	var $primary_source = '';
	var $list_query;
	var $filter_form;
	var $form_name;
	var $form_id;
	var $filter_options;
	var $filter_values;
	var $distinguish = 'fvalue_$UNIQ$';
	var $sources = [];

	function __construct(ListQuery $lq, $form_name, $form_id, $context) {
		$this->form_name = $form_name;
		$this->form_id = $form_id;
		$this->list_query = $lq;
		$this->filter_form = new FilterForm($lq, 
			$form_name, $form_id, $this->distinguish,
			$context);
		$this->filter_form->show_text_filter = true;
		$this->filter_form->expand_select_filters = true;
	}
	
	function loadRequestFilter($input, $name, $is_report=false) {
		$options = array_get_default($input, $name);
		$json = getJSONobj();
		$options = $json->decode($options);
		if(! is_array($options))
			return false;
		self::clean_filter_input($options);
		$this->loadFilterOptions($options, $is_report);
		$this->filter_form->loadFilter($input, 'editor', 'fvalue_');
		$this->filter_values = $this->filter_form->getArray();
		self::clean_filter_prefix($options);
		$this->filter_options = $options;
		return true;
	}
	
	function loadFilterLayout($name) {
		$inp = array_get_default($_POST, 'list_filter');
		if($inp)
			$inp = getJSONobj()->decode($inp);
		if(! $inp) $inp = array();
		$this->filter_form->loadFilterLayout($name);
		$this->filter_form->loadFilter($inp);
		$this->filter_values = $this->filter_form->getArray();
	}
	
	function loadFilterSpec(&$filters) {
		foreach($filters as &$f) {
			if(isset($f['source'])) {
				if($f['source'] != $this->primary_source) {
					if(isset($this->sources[$f['source']]))
						$f['link'] = $this->sources[$f['source']]['link_name'];
					else
						$f['link'] = $f['source'];
				}
				unset($f['source']);
			}
			if(! empty($f['multiple']))
				$this->loadFilterSpec($f['multiple']);
		}
	}
	
	function loadFilterOptions(array $filters, $is_report=false) {
		$this->loadFilterSpec($filters);
		$this->filter_form->setIsReport($is_report, $is_report ? 'interactive' : null);
		$this->filter_form->loadFilterLayout($filters);
	}
	
	function loadFilterValues($vals) {
		if(! is_array($vals))
			$vals = array();
		$this->filter_form->loadArrayFilter($vals);
		array_extend($vals, $this->filter_form->getFilterValues());
		$this->filter_values = $vals;
	}

	function encodeFilters($edit=true) {
		//$this->filter_form->loadArrayFilter($this->filter_values);
		$filters = $this->filter_form->getDisplayFilterOptions(false, true);
		foreach($filters as &$f) {
			if(! empty($f['link']) && ! empty($f['field'])) {
				$f['field'] = $f['link'].'.'.$f['field'];
				unset($f['link']);
			}
		}
		$render = $this->filter_form->formatFilters($filters, $this->filter_values, $edit);
		foreach($render as &$filt) {
			// do reverse translation of link name to source name
			if(! empty($filt['source'])) {
				foreach($this->sources as $src) {
					if(isset($src['link_name']) && $src['link_name'] == $filt['source']) {
						$filt['source'] = $src['name'];
						break;
					}
				}
			}
		}
		$this->translateParentRefs($render);
		$ret = array();
		foreach($render as $f) {
			if(! empty($f['basic']))
				$f['fixed'] = true;
			$ret[] = $f;
		}
		return $ret;
	}

	function getClauses($forWorkflow = false) {
		$clauses = $this->filter_form->getFilterClauses($forWorkflow);
		return $clauses;
	}
	
	function getClauseFields($clauses) {
		return ListQuery::get_filter_clause_fields($clauses);
	}
	
	function translateParentRefs(array &$cols) {
		if($this->list_query) {
			foreach($cols as $k => &$c) {
				$field_name = array_get_default($c, 'field', '');
				$src = array_get_default($c, 'source');
				$link = array_get_default($c, 'link');
				if($link) $field_name = $link . '.' . $field_name;
				if( ($p = strrpos($field_name, '.')) !== false) {
					$path = substr($field_name, 0, $p);
					$field_name = substr($field_name, $p+1);
					$f = $this->list_query->getFieldDefinition($path, $err);
					$lnk = $this->list_query->getLink($path);
					if($f && isset($f['vname'])) {
						if($lnk) $vmod = $lnk->getModuleDir();
						else $vmod = $this->list_query->getModuleDir();
						$c['ref_label'] = translate($f['vname'], array_get_default($f, 'vname_module', $vmod));
					}
				}
			}
		}
	}
	
	function translateTypes(array &$cols) {
		foreach($cols as $k => &$c) {
			$t = array_get_default($c, 'type', '');
			if($t == 'ref' && ! empty($c['bean_label']))
				$c['type_label'] = $c['bean_label'];
			else
				$c['type_label'] = translate_dd('field_type_dom', $t, '');
		}
	}
	
	function formatFilters($filters) {
		$ret = array();
		if($filters)
		foreach($filters as $filt) {
			$pfx = str_repeat('&nbsp;&nbsp;&nbsp;', $filt['depth']);
			if($filt['type'] != 'filtergroup')
				$lbl = '<b>'.to_html($filt['label']).'</b> – ';
			else
				$lbl = '';
			$ret[] = $pfx.$lbl.$filt['_init']['filter'];
		}
		if($ret)
			$ret = '<div style="padding: 5px">'.implode('<br>', $ret).'</div>';
		else
			$ret = '';
		return $ret;
	}

	static function clean_filter_input(&$filters) {
		static $keep = array('name', 'field', 'source', 'label', 'hidden', 'when', '_iprefix');
		$nc = array();
		$ldepth = -1;
		$group_idx = array();
		$gidx = 0;
		$names = array();
		foreach($filters as $c) {
			$c2 = array();
			foreach($keep as $k) {
				if(! empty($c[$k]) && ! is_array($c[$k]))
					$c2[$k] = $c[$k];
			}
			$type = array_get_default($c, 'type');
			if($type == 'filtergroup') {
				$op = array_get_default($c, 'operator');
				if($op != 'and' && $op != 'or' && $op != 'not' && $op != 'not_and')
					$op = 'and';
				$c2['type'] = $type;
				$c2['operator'] = $op;
				$c2['multiple'] = array();
				$c2['name'] = 'group-'.($gidx ++);
			}
			else {
				if(isset($c2['name'])) {
					if(! empty($names[$c2['name']]))
						$c2['unique'] = $names[$c2['name']] ++;
					else
						$names[$c2['name']] = 1;
				}
			}
		
			$d = array_get_default($c, 'depth', 0);
			if($d > $ldepth + 1)
				$d = 0;
			while($ldepth > $d-1) {
				$ldepth --;
				array_pop($group_idx);
			}
			if($d) {
				$parent =& $nc[$group_idx[0]];
				for($i = 1; $i < $d; $i++)
					$parent =& $parent['multiple'][$group_idx[$i]];
				$idx = count($parent['multiple']);
				$parent['multiple'][] = $c2;
			}
			else {
				$idx = count($nc);
				$nc[] = $c2;
			}
			if($type == 'filtergroup') {
				$ldepth = $d;
				array_push($group_idx, $idx);
			}
		}
		$filters = $nc;
	}
	
	static function clean_filter_prefix(array &$filters) {
		foreach($filters as &$f) {
			unset($f['_iprefix']);
			if(isset($f['multiple']) && is_array($f['multiple']))
				self::clean_filter_prefix($f['multiple']);
		}
	}

}

