<?php

require_once 'include/ListView/ListFormatter.php';

class HighlightedListFieldFormatter extends FieldFormatter {

    var $order_modules = array('Bills', 'Invoice');

	function formatRows(array &$model_fields, array &$rows, $union_key=null) {
        $today = date('Y-m-d H:i:s', time());
        $field = 'date_start';
        $additional_fields = array();

        if ($this->getModuleDir() == 'Activities') {
            $field = 'activity_date';
        } elseif ($this->getModuleDir() == 'Tasks') {
            $field = 'date_due';
        } elseif ($this->getModuleDir() == 'ProjectTask') {
            $today = date('Y-m-d', time());
            $field = 'date_due!date_only';
        } elseif (in_array($this->getModuleDir(), $this->order_modules)) {
            $field = 'due_date';
            $additional_fields = array('amount', 'amount_due', 'amount_usdollar', 'amount_due_usdollar');
        } elseif ($this->getModuleDir() == 'SalesOrders') {
            $field = 'due_date';
            $additional_fields = array('amount', 'amount_usdollar');
        }

        $unformatted = $rows;
        $ret = parent::formatRows($model_fields, $rows, $union_key);

        if ($this->format == 'html' && $this->getModuleDir() != 'Notes') {
            foreach ($ret as $key => &$row) {
                if (empty($row[$field]) || ! isset($unformatted[$key][$field]))
                    continue;

                if ($this->isHighlighted($unformatted[$key], $field, $today)) {
                    $additional_fields[] = $field;
                    $this->highlightField($row, $additional_fields);
                }
            }
        }

        return $ret;
    }

    function highlightField(&$row, $fields) {
        if (! is_array($fields))
            $fields = array($fields);

        if (sizeof($fields) > 0) {
            for ($i = 0; $i < sizeof($fields); $i++) {
                $name = $fields[$i];
                if (isset($row[$name]))
                    $row[$name] = "<span class='overdueTask'>".$row[$name]."</span>";
            }
        }
    }

    function isHighlighted($row, $date_field, $today) {
        $status = false;
        if ($row[$date_field] < $today)
            $status = true;
        if (in_array($this->getModuleDir(), $this->order_modules) && abs($row['amount_due'] ?? 0) < 0.005)
            $status = false;
        if ($this->getModuleDir() == 'SalesOrders' && $row['so_stage'] == 'Closed - Shipped and Invoiced')
            $status = false;

        return $status;
    }
}

