<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


define('LOAD_PRIORITY_PRE_INIT', 10); /* first thing loaded, normally empty */
define('LOAD_PRIORITY_APP_INIT', 9); /* sugar_3.js, language files, settings */
define('LOAD_PRIORITY_CUSTOM_INIT', 8); /* override app settings */
define('LOAD_PRIORITY_THEME', 7); /* theme includes */
define('LOAD_PRIORITY_HEAD', 5); /* insert in head tag */
define('LOAD_PRIORITY_INCLUDE', 4); /* head for full page, body for async load */
define('LOAD_PRIORITY_FORM', 4); /* same as include */
define('LOAD_PRIORITY_BODY', 3); /* insert before content */
define('LOAD_PRIORITY_FOOT', 1); /* insert after content */
define('LOAD_PRIORITY_END', 0); /* insert after content */
define('LOAD_PRIORITY_DEFER', -1); /* on page load */
define('LOAD_PRIORITY_READY', -3); /* after DOM is ready */

require_once('include/layout/ResourceLoader.php');

class IncludeManager extends ResourceLoader {
	var $includes = array();
	var $js_model_cached = array();
	var $include_count = 0;
	var $jsmin = false;
	var $jsmin_require_length = 500;
	var $cssmin = false;
	var $include_grouping_disabled = false;
	var $last_normalize_count = 0;
	var $http2_preload = false;
	var $preload_includes;
	var $update_page_state;
	var $module = '';
	var $action = '';
	var $layout = '';

	static $include_types_ordered = array(
		'css', // type, path/value, rel, media, id, name, attrs
		'javascript', // type, path/value/language, attrs
		'data',
		'json', // name, type, value/params, callback
		'image', // name, type, path/value, params
		'html', // name, type, path/value
	);

	function __construct() {
		parent::__construct();
		if(AppConfig::setting('layout.jsmin_enabled')) {
			$this->jsmin = true;
		}
		if($this->jsmin && ! AppConfig::setting('layout.jsmin_css_disabled')) {
			$this->cssmin = true;
			require_once('include/javascript/cssmin.php');
		}
		if(AppConfig::setting('layout.include_grouping_disabled'))
			$this->include_grouping_disabled = true;
		if(1||(AppConfig::detect_http2() && AppConfig::setting('site.performance.http2_preload'))) {
			/*$this->include_grouping_disabled = true;
			$this->jsmin = false;
			$this->cssmin = false;*/
			$this->http2_preload = true;
		}
		if(getenv('PAGESPEED') // env variable set by .htaccess (on apache)
		|| (function_exists('apache_get_modules') && ($mods = apache_get_modules())
			&& (in_array('mod_instaweb', $mods) || in_array('mod_pagespeed', $mods)))) {
			$this->include_grouping_disabled = true;
			$this->jsmin = false;
			$this->cssmin = false;
			$this->http2_preload = false;
		}
		if($this->jsmin)
			require_once('include/javascript/jsminplus.php');
		if($this->cssmin)
			require_once('include/javascript/cssmin.php');
		if($this->http2_preload) {
			// check session for recent preload (could be smart and remember individual includes)
			if(isset($_SESSION['http2_preload'])) {
				$prev = explode(':', $_SESSION['http2_preload']);
				if(count($prev) == 2 && $prev[0] == js_version_params() && time() - $prev[1] < 3600) {
					// skip preloading
					$this->http2_preload = false;
				}
			}
		}
	}

	function add_include($type, $details, $priority) {
		if($priority == LOAD_PRIORITY_INCLUDE) {
			if(defined('ASYNC_ENTRY'))
				$priority = LOAD_PRIORITY_BODY;
			else
				$priority = LOAD_PRIORITY_HEAD;
		}
		$inc = array($type, $details, $this->include_count ++);
		if(! empty($details['prepend'])) {
			if(! isset($this->includes[$priority])) $this->includes[$priority] = [];
			array_unshift($this->includes[$priority], $inc);
		}
		else {
			$this->includes[$priority][] = $inc;
		}
	}

	function add_js_include($path, $params=null, $priority=null) {
		if(! isset($priority)) $priority = LOAD_PRIORITY_INCLUDE;
		if(! is_array($params)) $params = array();
		if(isset($params['type']))
			$params['mimetype'] = $params['type'];
		if (!empty($params['ext'])) {
			$params['type'] = 'ext';
		} else {
			$params['type'] = 'file';
		}
		$cpath = 'custom/' . $path;
		if(substr($path, 0, 8) == 'modules/' && is_readable($cpath))
			$path = $cpath;
		$params['path'] = $path;
		$this->add_include('javascript', $params, $priority);
	}

	function add_js_literal($source, $params=null, $priority=null) {
		if(! isset($priority)) $priority = LOAD_PRIORITY_END;
		if(! is_array($params)) $params = array();
		if(isset($params['type']))
			$params['mimetype'] = $params['type'];
		$params['type'] = 'literal';
		$params['value'] = $source;
		$this->add_include('javascript', $params, $priority);
	}

	function add_css_include($path, $params=null, $priority=null) {
		if(! isset($priority)) $priority = LOAD_PRIORITY_INCLUDE;
		if(! is_array($params)) $params = array();
		if(isset($params['type']))
			$params['mimetype'] = $params['type'];
		$params['type'] = 'file';
		$params['path'] = $path;
		$this->add_include('css', $params, $priority);
	}

	function add_css_literal($source, $params=null, $priority=null) {
		if(! isset($priority)) $priority = LOAD_PRIORITY_INCLUDE;
		if(! is_array($params)) $params = array();
		if(isset($params['type']))
			$params['mimetype'] = $params['type'];
		$params['type'] = 'literal';
		$params['value'] = $source;
		$this->add_include('css', $params, $priority);
	}

	function add_include_group($group, $params=null, $priority=null) {
		if(! isset($priority)) $priority = LOAD_PRIORITY_INCLUDE;
		if(! is_array($params)) $params = array();
		$params['type'] = 'group';
		$params['group_name'] = $group;
		$this->add_include('javascript', $params, $priority);
		$this->add_include('css', $params, $priority);
	}

	function add_js_include_group($group, $params=null, $priority=null) {
		if(! isset($priority)) $priority = LOAD_PRIORITY_INCLUDE;
		if(! is_array($params)) $params = array();
		$params['type'] = 'group';
		$params['group_name'] = $group;
		$this->add_include('javascript', $params, $priority);
	}

	function add_css_include_group($group, $params=null, $priority=null) {
		if(! isset($priority)) $priority = LOAD_PRIORITY_INCLUDE;
		if(! is_array($params)) $params = array();
		$params['type'] = 'group';
		$params['group_name'] = $group;
		$this->add_include('css', $params, $priority);
	}

	function add_js_language($module, $params=null, $priority=null) {
		if(! isset($priority)) $priority = LOAD_PRIORITY_INCLUDE;
		if(! is_array($params)) $params = array();
		if(! isset($params['language']))
			$params['language'] = $GLOBALS['current_language'];
		$params['type'] = 'language';
		$params['module'] = $module;
		$this->add_include('javascript', $params, $priority);
		$parent = AppConfig::setting("lang.index.current.inherits.$module");
		if($parent && $parent != 'app')
			$this->add_js_language($parent, null, $priority);
	}

	function add_json_request($name, $arguments, $params=null, $priority=null) {
		if(! isset($priority)) $priority = LOAD_PRIORITY_DEFER;
		if(! is_array($params)) $params = array();
		$params['type'] = 'request';
		$params['name'] = $name;
		$params['arguments'] = $arguments;
		$this->add_include('json', $params, $priority);
	}

	function add_json_literal($name, $value, $params=null, $priority=null) {
		if(! isset($priority)) $priority = LOAD_PRIORITY_END;
		if(! is_array($params)) $params = array();
		$params['type'] = 'literal';
		$params['name'] = $name;
		$params['value'] = $value;
		$this->add_include('json', $params, $priority);
	}

	function add_html_literal($value, $params=null, $priority=null) {
		if(! isset($priority)) $priority = LOAD_PRIORITY_BODY;
		if(! is_array($params)) $params = array();
		$params['type'] = 'literal';
		$params['value'] = $value;
		$this->add_include('html', $params, $priority);
	}


	function add_js_model_cache($model_name) {
		if(empty($this->js_model_cached[$model_name])) {
			$c = array();
			$rows = $this->load_model_cache($model_name);
			// trim is actually used to convert numeric keys into strings for javascript:
			$data = array('keys' => array_map('trim', array_keys($rows)), 'values' => $rows, 'display_name' => 'name');
			$json = getJSONobj();
			$js = 'SUGAR.modelCache.addModelData("'.$model_name.'", '.$json->encode($data).');';
			$this->add_js_literal($js, null, LOAD_PRIORITY_BODY);
			$this->js_model_cached[$model_name] = 1;
		}
	}

	function load_model_cache($model_name) {
		$rows = AppConfig::db_all_rows($model_name);
		$fields = AppConfig::setting("model.detail.$model_name.js_cache.fields");
		if($fields) {
			// filter row fields
		}
		return $rows;
	}

	function add_image_preload($path, $params=null, $priority=null) {
		if(! isset($priority)) $priority = LOAD_PRIORITY_APP_INIT;
		if(! is_array($params)) $params = array();
		$params['type'] = 'file';
		$params['path'] = $path;
		$this->add_include('image', $params, $priority);
	}


	function cmp_includes($a, $b) {
		if($a[0] != $b[0]) { // sort by type
			$ord_a = array_search($a[0], self::$include_types_ordered);
			$ord_b = array_search($b[0], self::$include_types_ordered);
			if($ord_a === false && $ord_b !== false)
				return 1;
			if($ord_b === false && $ord_a !== false)
				return -1;
			return $ord_a < $ord_b ? -1 : 1;
		}
		return $a[2] < $b[2] ? -1 : 1; // sort by include order
	}


	function normalize_includes() {
		krsort($this->includes);
		foreach($this->includes as $prio => $incs) {
			// remove duplicate file includes _at the same priority level_
			$seen_files = array();
			$incsCount = is_countable($incs) ? count($incs) : 0;
			for($k = 0; $k < $incsCount; $k++) {
				list($type, $details, $order) = $incs[$k];
				// load group info
				if($details['type'] == 'group' && ! isset($details['group_info'])) {
					$info = $this->load_include_group($details['group_name'], $details, $type);
					$inc_types = $info[3] ?? [];
					if(isset($inc_types[$type])) {
						foreach($info[1] as $inc) {
							if($inc['inc_type'] == $type) {
								if($inc['type'] == 'file')
									$seen_files[$inc['path']] = true;
							}
						}
					}
					$incs[$k][1]['group_info'] = $info;
				}
			}
			$nodups = array();
			for($k = 0; $k < $incsCount; $k++) {
				list($type, $details, $order) = $incs[$k];
				if($details['type'] == 'group') {
					list($groupname, $all_incs, $inc_params, $inc_types) = $details['group_info'];
					if(empty($inc_types[$type]))
						continue;
					// expand group into individual includes if minimizer is not enabled
					$expand_all = $this->include_grouping_disabled || ! empty($details['no_min'])
						|| ($type == 'javascript' && ! $this->jsmin)
						|| ($type == 'css' && ! $this->cssmin);
					$c = $inc_types[$type];
					$i = 0;
					$keep = $append = array();
					foreach($all_incs as $row) {
						if($row['inc_type'] == $type) {
							if($expand_all || ! empty($row['cond_comment'])) {
								unset($row['inc_type']);
								$norder = round($order + ($i / $c), 2);
								$append[] = array($type, $row, $norder);
								$i ++;
							} else
								$keep[] = $row;
						}
					}
					if(count($keep)) {
						if(count($keep) == 1) {
							$row = $keep[0];
							unset($row['inc_type']);
							$nodups[] = array($type, $row, $order);
						} else {
							$incs[$k][1]['group_info'][1] = $keep;
							$nodups[] = $incs[$k];
						}
					}
					foreach($append as $row)
						$nodups[] = $row;
					continue;
				}
				if(! isset($details['path']))
					$nodups[] = $incs[$k];
				else if(! empty($seen_files[$details['path']]))
					continue;
				else {
					$seen_files[$details['path']] = 1;
					$nodups[] = $incs[$k];
				}
			}
			usort($nodups, array($this, 'cmp_includes'));
			$this->includes[$prio] = $nodups;
		}
		$this->last_normalize_count = $this->include_count;
	}


	function get_includes($section=null, $type_filter=null, $remove_after=false) {
		if(! isset($section))
			return $this->includes;
		if($this->last_normalize_count < $this->include_count)
			$this->normalize_includes();
		$inclist = array();
		$added = array();
		foreach($this->includes as $prio => $incs) {
			if(is_int($section) && $prio != $section)
				continue;
			else if($section == 'head' && $prio < LOAD_PRIORITY_HEAD)
				break;
			else if($section == 'body') {
				if($prio > LOAD_PRIORITY_BODY)
					continue;
				if($prio <= LOAD_PRIORITY_FOOT)
					break;
			}
			else if($section == 'foot' && $prio != LOAD_PRIORITY_FOOT)
				continue;
			else if($section == 'end' && $prio != LOAD_PRIORITY_END)
				continue;
			$added[] = $prio;
			foreach($incs as $idx => $i) {
				$incs[$idx][2] = $idx;
				$incs[$idx][3] = $prio;
			}
			$inclist = array_merge($inclist, $incs);
		}
		if(! isset($type_filter)) {
			if($remove_after) {
				foreach($added as $prio)
					$this->includes[$prio] = array();
			}
			if($section == 'head' || $section == 'body') {
				// move CSS files to the top, to allow downloading while we wait for JS
				$css = array(); $rest = array();
				foreach($inclist as $inc)
					if($inc[0] == 'css' && ($inc[1]['type'] == 'file' || $inc[1]['type'] == 'group' || $inc[3] <= LOAD_PRIORITY_THEME)) $css[] = $inc;
					else $rest[] = $inc;
				$inclist = array_merge($css, $rest);
			}
			return $inclist;
		}
		$ret = array();
		foreach($inclist as $inc)
			if($inc[0] == $type_filter) {
				if($remove_after)
					unset($this->includes[$inc[3]][$inc[2]]);
				$ret[] = $inc;
			}
		return $ret;
	}


	function format_include_block($block, &$ret) {
		$out = $block['head'];
		$value = $block['content'];
		if($block['type'] == 'javascript' && $this->jsmin && (! $this->jsmin_require_length || strlen($value) > $this->jsmin_require_length))
			$value = JSMinPlus::minify($value);
		else if($block['type'] == 'css' && $this->cssmin)
			$value = cssmin::minify($value);
		$out .= trim($value) . $block['foot'];
		if($block['cond_comment'])
			$out = "<!--[if {$block['cond_comment']}]>$out<![endif]-->";
		if(strlen($out))
			$ret[] = $out;
		return 1;
	}

	function format_includes($section, $type_filter=null, $remove_after=false, $dynamic_inc=false) {
		$incs = $this->get_includes($section, $type_filter, $remove_after);
		$out = array();
		$block_def = array('type' => '', 'head' => '', 'foot' => '', 'content' => '', 'prio' => 0, 'special' => false, 'cond_comment' => null);
		$block = null;
		$static_url = AppConfig::site_static_url();
		$preload = array();
		$image_preload = array();
		header('X-Frame-Options: SAMEORIGIN');
		$csp = AppConfig::setting('site.csp');
		$csp_value = join(";", array_map(function($directive, $values) {
			return $directive . " " . join(" ", $values);
		}, array_keys($csp), array_values($csp)));
		header("Content-Security-Policy: " . $csp_value);

		foreach($incs as $inc) {
			list($type, $detail, $order, $priority) = $inc;
			$cond_comment = array_get_default($detail, 'cond_comment');
			if($type == 'javascript' && $detail['type'] == 'language')
				$detail = $this->get_js_language_include($detail);
			$ext = '';
			if(isset($detail['path']) && ($p = strpos($detail['path'], '?')) !== false) {
				$ext = substr($detail['path'], $p + 1);
				$detail['path'] = substr($detail['path'], 0, $p);
			}
			$no_merge = isset($detail['no_merge']) || isset($detail['attrs']) || isset($detail['mimetype']) || $cond_comment || $dynamic_inc;
			if($block && ($block['type'] != $type || $detail['type'] != 'literal' || $block['no_merge'] || $no_merge)) {
				$this->format_include_block($block, $out);
				$block = null;
			}
			if($detail['type'] == 'group') {
				$group_params = array();
				foreach($detail['group_info'][2] as $k => $v)
					$group_params[] = urlencode($k).'='.urlencode($v);
				$type_ext = ($type == 'javascript') ? '.js' : '.css';
				$detail['path'] = 'mingroup/'.$detail['group_name'].$type_ext;
				$ext = '';
				if(count($group_params))
					$ext = implode('&', $group_params);
				$detail['type'] = 'file';
			}
			if($type == 'css') {
				$mimetype = array_get_default($detail, 'mimetype', 'text/css');
				$attrs = array_get_default($detail, 'attrs', '');
				$no_min = array_get_default($detail, 'no_min');
				$no_ver = array_get_default($detail, 'no_ver');
				if($attrs) $attrs = ' '.$attrs;
				if($detail['type'] == 'file') {
					$do_min = $this->cssmin && !$no_min;
					$path = css_prefix($do_min, $ext, $no_ver) . $detail['path'] . css_suffix($do_min, $ext, $no_ver);
					$media = array_get_default($detail, 'media', 'all');
					$rel = array_get_default($detail, 'rel', 'stylesheet');
					if(isset($detail['title']))
						$attrs .= ' title="'.$detail['title'].'"';
					$chkpre = array_get_default($detail, 'preload', ($priority == LOAD_PRIORITY_APP_INIT || $priority == LOAD_PRIORITY_THEME) && $media == 'all');
					if($chkpre) $preload[] = array('url' => $path, 'as' => 'style');
					if(strpos($path, '://') === false) $path = $static_url . $path;
					$out[] = "<link type=\"{$mimetype}\" rel=\"{$rel}\" media=\"{$media}\" href=\"".htmlspecialchars($path, ENT_COMPAT, 'UTF-8')."\"{$attrs} />";
				}
				else if(! empty($detail['value'])) {
					if(! $block) {
						$block = $block_def;
						$block['no_merge'] = $no_merge;
						$block['prio'] = $priority;
						$block['type'] = 'css';
						$block['head'] = "<style type=\"{$mimetype}\"{$attrs}>";
						$block['foot'] = '</style>';
						$block['cond_comment'] = $cond_comment;
					}
					$val = preg_replace('~^\s*<style\b[^>]*>(\s*<!--[^\n]*)?|(-->\s*)?</style>\s*$~', '', $detail['value']);
					$block['content'] .= "\n".$val;
				}
				continue;
			}
			if($type == 'javascript') {
				$mimetype = array_get_default($detail, 'mimetype', 'text/javascript');
				$attrs = array_get_default($detail, 'attrs', '');
				$no_min = array_get_default($detail, 'no_min');
				$no_ver = array_get_default($detail, 'no_ver');
				if($attrs) $attrs = ' '.$attrs;
				if($detail['type'] == 'file') {
					$do_min = $this->jsmin && !$no_min;
					$inc_type = empty($detail['language']) ? 'js' : 'lang';
					$path = js_prefix($do_min, $inc_type, $ext, $no_ver) . $detail['path'] . js_suffix($do_min, $inc_type, $ext, $no_ver);
					$chkpre = array_get_default($detail, 'preload', $priority == LOAD_PRIORITY_APP_INIT);
					if($chkpre) $preload[] = array('url' => $path, 'as' => 'script');
					if(strpos($path, '://') === false) $path = $static_url . $path;
					$scr = "<script type=\"{$mimetype}\" src=\"".htmlspecialchars($path, ENT_COMPAT, 'UTF-8')."\"{$attrs}></script>";
					if($detail['path'] == 'include/JSON.js') // use browser native JSON if available
						$scr = '<script type="text/javascript">if(!window.JSON||!JSON.stringify)document.write("'.addcslashes($scr,'"/').'");</script>';
					$out[] = $scr;
				} else 	if($detail['type'] == 'ext') {
					$path = $detail['path'];
					$out[] = "<script type=\"{$mimetype}\" src=\"".htmlspecialchars($path, ENT_COMPAT, 'UTF-8')."\"{$attrs}></script>";
				}
				else if(! empty($detail['value'])) {
					if(! $block) {
						$block = $block_def;
						$block['no_merge'] = $no_merge;
						$block['type'] = 'javascript';
						$block['head'] = "<script type=\"{$mimetype}\"{$attrs}>";
						$block['foot'] = '</script>';
						$block['cond_comment'] = $cond_comment;
					}
					$val = preg_replace('~^\s*<script\b[^>]*>(\s*<!--[^\n]*)?|(-->\s*)?</script>\s*$~', '', $detail['value']);
					$block['content'] .= "\n".$val;
				}
				continue;
			}
			if($type == 'html') {
				if(! empty($detail['value'])) {
					$out[] = $detail['value'];
				}
				continue;
			}
			if($type == 'image') {
				if($detail['type'] == 'file') {
					$no_ver = array_get_default($detail, 'no_ver');
					$path = $detail['path'];
					if(! $no_ver) $path .= '?' . js_version_params();
					$preload[] = array('url' => $path, 'as' => 'image');
					$image_preload[] = $path;
				}
				continue;
			}
		}
		if($block) {
			$this->format_include_block($block, $out);
			$block = null;
		}

		$this->preload_includes = $preload;
		if($this->http2_preload && $preload) {
			$this->send_preload_headers($preload);
		}
		else if($image_preload) {
			$static = AppConfig::site_static_url();
			$preload = array();
			foreach($image_preload as $i) {
				$preload[] = '"' . javascript_escape($static_url . $i) . '"';
			}
			$preload_json = implode(',', $preload);
			$preload = <<<EOS
				<script type="text/javascript">
				var _img = [$preload_json], _pre = [];
				for(var i = 0; i < _img.length; i++){ _pre[i] = new Image(); _pre[i].src = _img[i]; }
				</script>
EOS;
			$out[] = $preload;
		}

		return implode("\n", $out);
	}

	function send_preload_headers(array $preload) {
		$base_path = '';
		$site = AppConfig::site_url();
		$p = strpos($site, '://');
		if($p) {
			$p = strpos($site, '/', $p+3);
			if($p)
				$base_path = substr($site, $p);
		}
		foreach($preload as $p) {
			header("Link: <{$base_path}{$p['url']}>; rel=preload; as={$p['as']}", false);
		}
		$_SESSION['http2_preload'] = js_version_params() .':'. time();
	}

	function merge_includes($other) {
		foreach($other->includes as $prio => $incs) {
			foreach($incs as $inc) {
				$this->add_include($inc[0], $inc[1], $prio);
			}
		}
	}

	function clear_includes($priority=null) {
		if(isset($priority))
			$this->includes[$priority] = array();
		else
			$this->includes = array();
		$this->js_model_cached = array();
		$this->last_normalize_count = 0;
		$this->include_count = 0;
	}
}

?>
