<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/utils/basic_utils.php');
require_once('include/utils/external_cache.php');

class ResourceLoader {	
	var $language;
	var $jsmin;

	function __construct() {
	}

	function resolve_language($language=null) {
		$base_language = AppConfig::setting('locale.base_language', 'en_us');
		$default_language = AppConfig::setting('locale.defaults.language', $base_language);
		if(empty($language)) {
			if(! empty($this->language))
				$language = $this->language;
			else if(isset($GLOBALS['current_language']))
				$language = $GLOBALS['current_language'];
		}
		$language_used = empty($language) ? $default_language : $language;
		return compact('base_language', 'default_language', 'language_used');
	}
	
	function reload_module_language($module, $language=null) {
		$lang = $this->resolve_language();
		$key = "module_language.".$lang['language_used'].$module;
		sugar_cache_clear($key);

		$js_cache_dir = CacheManager::get_location('javascript/');
		$sugar_version = AppConfig::version();
		$js_version = AppConfig::js_custom_version();
		$js_lang_version = AppConfig::language_version();

		$jslang = $js_cache_dir . 'jslanguage_'.$module.'_'.$lang['language_used']
			. "_{$js_lang_version}_{$sugar_version}_{$js_version}.js";
		if(file_exists($jslang))
			@unlink($jslang);
	}
	
	function get_mod_strings($module, $language=null, $resolve=false) {
		if(! $language) $language = 'current';
		
		$ret = AppConfig::setting("lang.strings.$language.$module");
		if($resolve && ($inh = AppConfig::setting("lang.index.$language.inherits.$module")) ) {
			$s2 = AppConfig::setting("lang.strings.$language.$inh");
			$ret = array_replace($s2, $ret);
		}
		return $ret;
	}
	
	function get_app_list_strings($language, $merge = false) {
		if(! $language) $language = 'current';	
		return AppConfig::setting("lang.lists.$language.app");
	}
	
	function get_app_strings($language) {
		if(! $language) $language = 'current';	
		return AppConfig::setting("lang.strings.$language.app");
	}
	
	function get_js_language_include($detail, $language=null) {
		if(! is_array($detail))
			$detail = array('module' => $detail, 'language' => $language, 'no_min' => true);
		$detail['type'] = 'file';
		if(! $this->jsmin || ! empty($detail['no_min'])) {
			$detail['no_min'] = true;
			$detail['path'] = $this->get_js_language_path($detail);
			if(! $detail['path'])
				return false;
		} else {
			$mod = array_get_default($detail, 'module');
			if(! $mod || $mod == 'app')
				$path = 'jslanguage/app/'.$detail['language'].'.js';
			else
				$path = "jslanguage/modules/$mod/".$detail['language'].'.js';
			$detail['lang'] = true;
			$detail['path'] = $path;
		}
		return $detail;
	}
	
	function get_js_language_path($params) {
		$module = array_get_default($params, 'module', '');
		if(! $module) $module = 'app';
		$lang = array_get_default($params, 'language', '');

		return AppConfig::setting("lang.jscache.$lang.$module");
	}


	static function _replace_params($value, $params, &$effect) {
		if(! is_countable($params) || ! count($params))
			return $value;
		$replace = $with = array();
		foreach($params as $k => $v) {
			$value = str_replace('{'.$k.'}', $v, $value, $c);
			if($c) $effect[$k] = $v;
		}
		return $value;
	}
	
	static function get_include_groups($reload=false) {
		return AppConfig::setting('include_groups');
	}
	
	static function load_include_group($name, $params=null, $restrict_type=null) {
		static $cats = array(
			'js_include' => array('inc_type' => 'javascript', 'type' => 'file', 'tpl_path' => ''),
			'css_include' => array('inc_type' => 'css', 'type' => 'file', 'tpl_path' => ''),
			'js_literal' => array('inc_type' => 'javascript', 'type' => 'literal', 'value' => ''),
			'css_literal' => array('inc_type' => 'css', 'type' => 'literal', 'value' => ''),
			'language' => array('inc_type' => 'javascript', 'type' => 'language', 'language' => '', 'module' => ''),
		);
		if(! is_array($params)) $params = array();
		if(isset($restrict_type) && ! is_array($restrict_type))
			$restrict_type = array($restrict_type);
		$groups = self::get_include_groups();
		if(! isset($groups[$name]))
			return null;
		$grp = $groups[$name];
		if(isset($grp['groups'])) {
			$subgrps = array();
			foreach($grp['groups'] as $v) {
				$subg = self::load_include_group($v, $params, $restrict_type);
				if($subg)
					$subgrps[] = $subg;
			}
			$ret = array($name, array(), array(), array());
			foreach($subgrps as $g) {
				$ret[1] = array_merge($ret[1], $g[1]); // incs
				$ret[2] = array_merge($ret[2], $g[2]); // params
				foreach($g[3] as $k => $v)
					$ret[3][$k] = @$ret[3][$k] + $v;
			}
			return $ret;
		}
		$param_values = array();
		$param_defaults = array();
		$test_params = array();
		$inc_types = array();
		if(isset($grp['params'])) {
			foreach($grp['params'] as $k => $param) {
				$param_defaults[$k] = isset($param['default']) ? $param['default'] : '';
				if(isset($params[$k])) {
					$param_values[$k] = $params[$k];
					$test_params[] = $k;
				} else
					$param_values[$k] = $param_defaults[$k];
			}
		}
		$incs = array();
		foreach($cats as $cat => $spec) {
			if(! isset($grp[$cat]) || (isset($restrict_type) && ! in_array($spec['inc_type'], $restrict_type)))
				continue;
			foreach($grp[$cat] as $row) {
				$inc = $spec;
				if(is_array($row)) {
					$inc = array_merge($inc, $row);
					if(isset($inc['path']))
						$inc['tpl_path'] = $inc['path'];
				}
				else {
					if($inc['type'] == 'file')
						$inc['tpl_path'] = $row;
					else if($inc['type'] == 'language')
						$inc['language'] = $row;
					else
						$inc['value'] = $row;
				}
				@$inc_types[$inc['inc_type']] ++;
				$incs[] = $inc;
			}
		}
		do {
			$missing = false;
			$set_params = array();
			foreach($incs as $idx => $inc) {
				if($inc['type'] == 'file') {
					$path = self::_replace_params($inc['tpl_path'], $param_values, $set_params);
					if(! file_exists($path)) {
						$missing = true;
						break;
					}
					$incs[$idx]['path'] = $path;
				}
				else if($inc['type'] == 'language') {
					$incs[$idx]['language'] = self::_replace_params($inc['language'], $param_values, $set_params);
				}
			}
			if($missing) {
				do {
					if(! is_countable($test_params) || ! count($test_params))
						return false;
					$p = array_pop($test_params);
					if($param_values[$p] != $param_defaults[$p]) {
						$param_values[$p] = $param_defaults[$p];
						break;
					}
				} while(1);
				continue;
			}
			return array($name, $incs, $set_params, $inc_types);
		} while(1);
	}

}

?>
