<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FilterForm.php');

class EditableForm {
    /**
     * @var ListQuery
     */
	var $list_query;
	var $form_name;
	var $form_id;
	var $form_class = 'card-outer form-outer default-form';
	var $form_style;
	var $form_action;
	var $method;
	var $have_attachments;
	var $style;
	var $hooks;
	var $hidden_fields;
	var $field_name_prefix;
	var $classic = false;
	var $js;
	var $js_controls;
	var $js_models;
	var $js_includes;
	var $max_ref_width = 20;
	var $default_ref_width = 18;
	var $max_text_width = 26;
	var $autofocus;
	var $autofocus_fields;
	var $show_text_filter = false;
	var $expand_select_filters = false;
	var $duplicate_of_id;
	var $context;
	var $add_field_classes;
	var $base_model;

	static $number_types = array(
		'int', 'short', 'float', 'double', 'currency', 'base_currency', 'raw_currency',
		'number', 'percentage', 'rate_percent', 'star_rating', 'progress', 'sequence_number',
	);

	function __construct($style, $form_name, $form_id=null, $prefix='', $context=null) {
		$this->setFormName($form_name, $form_id);
		$this->style = $style;
		$this->js = array();
		$this->hooks = array();
		$this->hidden_fields = array();
		$this->field_name_prefix = $prefix;
		$this->context = $context;
		if (AppConfig::is_mobile()) {
			$this->max_text_width = 25;
		}
	}

	function setFormName($form_name, $form_id=null) {
		$this->form_name = $form_name;
		$this->form_id = isset($form_id) ? $form_id : $form_name;
	}

	function setFormId($id) {
		$this->form_id = $id;
	}

	function setFormClass($cls) {
		$this->form_class = $cls;
	}

	function getFormClass() {
		$cls = $this->form_class;
		return $cls;
	}

	function setFormStyle($sty) {
		$this->form_style = $sty;
	}

	function getFormStyle() {
		$sty = $this->form_style;
		return $sty;
	}

	function setFormAction($action) {
		$this->form_action = $action;
	}

	function getFieldIdPrefix() {
		return $this->form_id;
	}

	function getFieldNamePrefix() {
		return $this->field_name_prefix;
	}

	function setFieldNamePrefix($pfx) {
		$this->field_name_prefix = $pfx;
	}

	function setAutoFocus($flag=true) {
		$this->autofocus = $flag;
	}

	function setAutoFocusFields($fields) {
		if(! $fields) $fields = array();
		else if(! is_array($fields)) $fields = array($fields);
		$this->autofocus_fields = $fields;
	}

	function exportIncludes() {
		global $pageInstance;
		if($this->js_models) {
			foreach($this->js_models as $bean => $_)
				$pageInstance->add_js_model_cache($bean);
		}
		if($this->js_includes) {
			foreach($this->js_includes as $inc)
				$pageInstance->add_js_include($inc, null, LOAD_PRIORITY_INCLUDE);
		}
		$js = $this->js;
		$init = $this->getFormInit();
		if($init) $js[] = $init;
		if($js) {
			$pageInstance->add_js_literal(implode('', $js));
		}
	}

	function getFormInit() {
		$jsc = $this->getControlsInit();
		if($jsc)
			$ctl = ", [\n" . implode(",\n", $jsc) . ']';
		else
			$ctl = ', null';
		if(isset($this->hooks['oninit']))
			$oninit = $this->hooks['oninit'];
		else
			$oninit = array();
		if($this->autofocus) {
			if(is_array($this->autofocus_fields))
				$fields = ', ' . getJSONobj()->encode(array_values($this->autofocus_fields));
			else
				$fields = ', null';
            if(isset($this->hooks['focus']))
                $focus_fn = $this->prepareHooks($this->hooks['focus']);
            else
                $focus_fn = "null";
            $postInit = ", function() { SUGAR.ui.focusForm(this{$fields}, {$focus_fn}); }";
		} else
			$postInit = '';
		$ret = '';
		if($oninit) {
            $fn = $this->prepareHooks($oninit);
			$ret .= "SUGAR.ui.onInitForm('$this->form_id', $fn);";
		}
		$ret .= "SUGAR.ui.initForm('$this->form_id'{$ctl}{$postInit});";
		return $ret;
	}

	function onInitForm($event) {
		$oninit = array_get_default($this->hooks, 'oninit');
		if(! $oninit) $oninit = array();
		else if(! is_array($oninit)) $oninit = array($oninit);
		$oninit[] = $event;
		$this->hooks['oninit'] = $oninit;
	}

    function prepareHooks($functions) {
        $fn = 'function() {' . self::combine_hooks($functions) . '}';
        return str_replace('{FORM}', 'this', $fn);
    }

	function getControlsInit($clear=false) {
		$c = array();
		if($this->js_controls) {
			foreach($this->js_controls as $k => $v)
				$c[] = "new SUGAR.ui.$v[0]('$k', $v[1])";
		}
		if($clear) $this->js_controls = array();
		return $c;
	}

	function getExtraInit($clear=false) {
		$ret = $this->js;
		if($clear) $this->js = array();
		return $ret;
	}

	function addControl($cls, $id, $params=null) {
		if(is_array($params))
			$params = getJSONobj()->encode($params);
		else if(! isset($params))
			$params = 'null';
		$this->js_controls[$id] = array($cls, $params);
	}

	function addInit($init) {
		$this->js[] = $init;
	}

	function addScriptInclude($script) {
		$this->js_includes[] = $script;
	}

	function open($cls=null, $style=null) {
		// FIXME - need to set encoding if form has attachments
		$attrs = array(
			'name' => $this->form_name,
			'id' => $this->form_id,
			'autocomplete' => 'off',
			'method' => isset($this->method) ? $this->method : 'POST',
			'class' => isset($cls) ? $cls : $this->getFormClass(),
			'style' => isset($style) ? $style : $this->getFormStyle(),
		);
		if ($this->have_attachments)
			$attrs['enctype'] = 'multipart/form-data';
		if(! empty($this->form_action))
			$attrs['action'] = $this->form_action;
		if(is_array($this->hooks))
			foreach($this->hooks as $k => $v) {
				if($k !== 'oninit' && $k != 'focus')
					$attrs[$k] = self::combine_hooks($v);
			}
		$ret = self::get_tag('form', $attrs) . "\n";
		foreach($this->hidden_fields as $k => $v)
			$ret .= self::get_tag('input', array('type' => 'hidden', 'name' => $k, 'value' => $v)) . "\n";
		$ret .= create_csrf_input();
		return $ret;
	}

	function close() {
		return "</form>\n";
	}

	function addHooks($hooks) {
		if(is_array($hooks)) {
			foreach($hooks as $k => $v) {
				if(! is_array($v)) $v = array($v);
				if(isset($this->hooks[$k]))
					array_extend($this->hooks[$k], $v);
				else
					$this->hooks[$k] = $v;
			}
		}
	}

	function addHiddenFields($hidden) {
		if(is_array($hidden)) {
			foreach($hidden as $k => $v)
				$this->hidden_fields[$k] = $v;
		}
	}

	function getFormName() {
		return $this->form_name;
	}

	function getFormId() {
		return $this->form_id;
	}

	function getFieldClass($base='', $spec=null, $field_mods=true, $check_disabled=false) {
		$cls = [$base];
		if($this->add_field_classes) $cls[] = $this->add_field_classes;
		if(isset($spec['class']))
			$cls[] = $spec['class'];
		if(isset($spec['ext_class'])) // prefer ext_class - same as in javascript
			$cls[] = $spec['ext_class'];
		if($check_disabled) {
			if(! empty($spec['readonly']))
				$cls[] = 'readonly';
			if(! empty($spec['disabled']))
				$cls[] = 'disabled';
		}
		if($field_mods) {
			if(! empty($spec['compact']))
				$cls[] = 'compact';
			if(! empty($spec['flat']))
				$cls[] = 'flat';
			if(! empty($spec['quick']))
				$cls[] = 'flat quick';
		}
		return implode(' ', $cls);
	}

	function getDefaultPlaceholder($type=null) {
		if($type == 'module_name' && $this->style == 'searchform')
			return translate('LBL_FILTER_ANY_MODULE', 'app');
		else if($this->style == 'searchform')
			return translate('LBL_FILTER_ANY_VALUE', 'app');
		else if($this->style == 'massupdate')
			return translate('LBL_NO_CHANGE', 'app');
		else
			return translate('LBL_NONE', 'app');
	}

	function renderField(RowResult $result, $name, $form_field=null) {
		global $app_list_strings;

		$search_form = $this->style == 'searchform';

		if (is_array($name)) {
			$def = $name;
			$name = array_get_default($def, 'name');
		}
		else if(isset($result))
			$def = array_get_default($result->fields, $name, array());
		else
			$def = array();

		if($form_field instanceof FormField) {
			$override = $form_field->getOverrideSpec();
			if($override) $def = array_merge($def, $override);
		}

		$t = array_get_default($def, 'type', 'varchar');
		if(! isset($def['name']))
			$def['name'] = $name;

		if(! isset($def['label']))
			$def['label'] = $name;
		if($t == 'bool' && ($this->style == 'massupdate' || ($search_form && empty($def['required'])))) {
			$t = 'enum';
			$def['expanded'] = false;
			if(empty($def['options']))
				$def['options'] = 'dom_int_bool';
		}
		if(! empty($def['alt_options'])) {
			$alt_name = array_get_default($def, 'alt_name', $name.'_alt');
			$def['alt_value'] = $result->getField($alt_name);
		}

		if($t == 'ref') {
			$raw_val = $result->getField($name, null, false);
			$def['value'] = $raw_val;
			return $this->renderRef($result, $def);
		}
		else if($t == 'address') {
			return $this->renderAddress($def, $result);
		}
		else if($t == 'bool') {
			$raw_val = $result->getField($name, '', false);
			return $this->renderCheck($def, $raw_val);
		}
		else if($t == 'enum' || $t == 'status') {
			$raw_val = $result->getField($name, '', false);
			if(! isset($def['name']))
				$def['name'] = $name;
			if($search_form && empty($def['no_search'])) {
				$op = array_get_default($def, 'operator');
                if (array_get_default($def, 'expanded', true)) {
                    return $this->renderSelectFilter($def, $raw_val);
                } elseif (is_array($raw_val) || $op == 'any_of' || $op == 'not_any_of') {
                    return $this->renderMultiSelect($def, $raw_val);
                } else if($op == 'empty') {
                	return $app_list_strings['select_filter_operators_dom']['empty'];
                } else if($op == 'not_empty') {
                	return $app_list_strings['select_filter_operators_dom']['not_empty'];
                }
			}
			return $this->renderSelect($def, $raw_val);
		}
		else if($t == 'multienum') {
			$raw_val = $result->getField($name, '', false);
			if(! isset($def['name']))
				$def['name'] = $name;
			if($search_form && empty($def['no_search'])) {
                if (array_get_default($def, 'expanded', true))
                    return $this->renderSelectFilter($def, $raw_val);
            }
			return $this->renderMultiSelect($def, $raw_val);
		}
		else if($t == 'module_name') {
			$raw_val = $result->getField($name, '', false);
			if(! isset($def['name']))
				$def['name'] = $name;
			return $this->renderModuleName($def, $raw_val);
		}
		else if($t == 'date' || $t == 'datetime' || $t == 'time') {
			$value = $result->getField($name, '', true);
			if($search_form && $t != 'time' && empty($def['no_search'])) {
				return $this->renderDateFilter($def, $value);
			} else {
                if (isset($def['subtype']) && $def['subtype'] == 'period') {
				    return $this->renderPeriodDate($def, $value);
                } else {
                    return $this->renderDateTime($def, $value);
                }
			}
		}
		else if($t == 'duration') {
            $value = $result->getField($name, '', false);
            $is_all_day = isset($def['all_day_flag']) ? $result->getField($def['all_day_flag'], 0) : 0;
            return $this->renderDuration($def, $value, $is_all_day);
        }
        else if($t == 'reminder') {
            $value = $result->getField($name, '', false);
            return $this->renderReminder($def, $value);
        }
        else if($t == 'html') {
            $value = $result->getField($name, '', false);
			if($search_form) {
				if(array_get_default($def, 'filter', $this->show_text_filter))
					return $this->renderTextFilter($def, $value);
				return $this->renderText($def, $value);
			}
			else if($this->context == 'workflow' || $this->context == 'listview') {
				return $this->renderTextArea($def, $value);
			}
			else {
				return $this->renderHtmlEditor($def, $value);
			}
        }
		else if($t == 'text' || $t == 'blob' || (($t == 'name' || $t == 'varchar') && ! empty($def['multiline']))) {
			$value = $result->getField($name, '', false);
            if($search_form) {
				if(array_get_default($def, 'filter', $this->show_text_filter))
					return $this->renderTextFilter($def, $value);
				return $this->renderText($def, $value);
			} else {
				return $this->renderTextArea($def, $value);
			}
		}
		else if($t == 'unified_search') {
			$mod = $result->module_dirs[0];
			if(! isset($def['autosave']) && $mod) {
				$def['autosave'] = "com.1crm.search.$mod";
			}
			$value = $result->getField($name, '', false);
			if($mod && ($mod == 'Reports' || $mod == 'Home'))
				$mod = null;
			return $this->renderSearch($def, $value, $mod);
		}
		else if($t == 'email_recips') {
			return $this->renderEmailRecips($result, $def);
		}
		else if($t == 'file_ref') {
			$value = $result->getField($name, '', false);
			if(isset($def['file_mime_type']))
				$mime = $result->getField($def['file_mime_type'], '', false);
			else $mime = null;
			$dupe = !! $result->for_duplicate;
			return $this->renderFileRef($def, $value, $mime, $dupe);
		}
		else if($t == 'image') {
			$vs = array('value' => $result->getField($name, '', true));
			$ps = array('filename', 'thumbnail_name');
			$lnk = array_get_default($def, 'link');
			if($lnk) {
				$m = $result->getModel();
				if($m) {
					$lm = $result->getLinkModel($lnk);
					if($lm) $vs['model'] = $lm->getModelName();
				}
			} else
				$vs['model'] = $result->getModelName();
			$vs['field'] = $def['name'];
			foreach($ps as $p) {
				$pname = array_get_default($def, $p);
				$vs[$p] = $pname ? $result->getField($pname, null, false) : null;
			}
			return $this->renderImage($def, $vs);
		}
        else if($t == 'alphabet' && $search_form) {
            $value = $result->getField($name, '', true);
            return $this->renderAlphabet($def, $value);
        }
        else if($t == 'button') {
			return $this->renderButton($def);
        }
        else if($t == 'color') {
        	$value = $result->getField($name, '');
			return $this->renderColor($def, $value);
        }
		else if(in_array($t, self::$number_types) && $search_form && empty($def['no_search'])) {
			$value = $result->getField($name, null, false);
			$ext = array('end' => $result->getField($name . '-end'));
			return $this->renderNumberFilter($def, $value, $ext);
		}
		else if(in_array($t, self::$number_types) && !$search_form && empty($def['no_search'])) {
			$value = $result->getField($name, null, $t != 'star_rating' && $t != 'progress');
			return $this->renderText($def, $value);
		}
		else if($t == 'stripe-element') {
        	$value = $result->getField($name, '');
			return $this->renderStripeElement($def, $value);
        }
		else {
			if(! isset($def['name']))
				return '';
			$value = $result->getField($name, '', false);
			$errs = $result->getFieldUpdateErrors($name);
			if($search_form && empty($def['no_search']) && array_get_default($def, 'filter', $this->show_text_filter))
				return $this->renderTextFilter($def, $value);
			return $this->renderText($def, $value, $errs);
		}
	}

	function formatFieldOptions($model, array $fields, $values=null, $edit=true) {
		require_once('include/database/RowResult.php');
		$result = new RowResult;
        $result->base_model = $model;
        $result->module_dirs[] = AppConfig::module_for_model($model);
        if(! isset($values))
        	$values = array();
        $fdefs = array();
        foreach($fields as $k => $f)
        	$fdefs[$f['name']] = $f;

        $fmt = new FieldFormatter('html', $edit ? 'editview' : ($this->context ? $this->context : 'listview'));
		$result->row = $values;
		$result->formatted = $fmt->formatRow($fdefs, $values);
		$result->fields = $fdefs;

		$ret = array();
		foreach($fields as $k => $f) {
			$type = array_get_default($f, 'type', '');
			if($edit)
				$body = $this->renderField($result, $f);
			else
				$body = array_get_default($result->formatted, $f['name']);
			$init = $this->getControlsInit(true);
			$preinit = $this->getExtraInit(true);
			$row = array(
				'name' => $f['name'],
				'field' => array_get_default($f, 'field', $f['name']),
				'id_name' => array_get_default($f, 'id_name'),
				'source' => array_get_default($f, 'link', ''),
				'label' => $f['label'],
				'type' => $type,
				'type_label' => array_get_default($f, 'type_label', ''),
				'required' => ! empty($f['required']),
				'_init' => array('rendered' => $body, 'ctor' => $init, 'ext' => $preinit),
			);
			$ret[] = $row;
		}
		return $ret;
	}

	function renderText(array $spec, $value, $errs=null) {
		$t = array_get_default($spec, 'type', 'varchar');
		$dbt = array_get_default($spec, 'dbType');
		if ($t === 'custom' && isset($spec['custom_type'])) {
			$t = $spec['custom_type'];
		}
		$name = $this->getFieldNamePrefix() . $spec['name'];
		$sel_id = $this->getFieldIdPrefix() . $name.'-input';
		$ret = '';

		if(! empty($spec['alt_options'])) {
			$ret .= $this->renderAltOptions($spec);
		}

		$attrib = array(
			'type' => 'text',
			'name' => $name,
			'id' => $sel_id,
			'value' => $value,
			'class' => $this->getFieldClass('input-text', $spec),
		);
		$min = array_get_default($spec, 'min');
		$max = array_get_default($spec, 'max');

		// html5 types
		if($t == 'email' || $t == 'url' || $t == 'password')
			$attrib['type'] = $t;
		if($t == 'phone' || $t == 'fax')
			$attrib['type'] = 'tel';

		// other formats
		if($t == 'int' || $t == 'short' || $t == 'number' || $t == 'progress'
				|| $t == 'file_size'
				|| $dbt == 'int') {
			$format = 'int';
			if($t == 'file_size' && $value)
				$attrib['value'] = format_number($value, -1);
		}
		else if($t == 'sequence_number')
			$format = 'sequence_number';
		else if($t == 'float' || $t == 'double') {
			$format = 'float';
			$decimals = array_get_default($spec, 'decimal_places', 2);
		} else if($t == 'currency' || $t == 'base_currency' || $t == 'raw_currency') {
			$format = 'currency';
			if(isset($spec['decimal_places']))
				$decimals = $spec['decimal_places'];
		}
		else if($t == 'percentage' || $t == 'rate_percent') {
			if(! isset($min)) $min = 0;
			if(! isset($max)) $max = 100;
			$dfmt = array_get_default($spec, 'display_format');
			$attrib['value'] = rtrim($attrib['value'], " %");
			if($dfmt != 'rounded' && ($dbt == 'float' || $dbt == 'double')) {
				$format = 'float';
				$decimals = array_get_default($spec, 'decimal_places', 2);
				$min_decimals = 0;
			}
			else
				$format = 'int';
			if(isset($value))
				$attrib['value'] = trim(str_replace('%', '', $value));
		}
		else if(($t == 'phone' || $t == 'fax') && AppConfig::setting('telephony.format_phones'))
			$format = 'phone';
		else if( ($t == 'url' || $t == 'email' || $t == 'hostname' || $t == 'password') && $this->style != 'searchform')
			$format = $t;
		else if(isset($spec['format']))
			$format = $spec['format'];
		else
			$format = 'text';
		if($t == 'user_name' || ! array_get_default($attrib, 'autocorrect', true)) {
			$attrib['autocapitalize'] = "off";
			$attrib['autocorrect'] = "off";
			$attrib['spellcheck'] = "false";
		}
		$w = '';
		if(isset($spec['len'])) {
			$attrib['maxlength'] = $spec['len'];
		}
		if(isset($spec['width'])) {
			$w = $spec['width'];
			$custom_w = true;
		}
		if(! $w) {
			if($format == 'int' || $format == 'float' || $format == 'currency') {
				$w = 10;
				$custom_w = true;
			}
			else if(! empty($spec['len']))
				$w = $spec['len'];
			else
				$w = 20;
		}
		if($w) {
			if($w > $this->max_text_width && empty($custom_w)) $w = $this->max_text_width;
			if(AppConfig::is_mobile()) $w = min(40, $w);
			$attrib['size'] = $w;
			if(! empty($custom_w))
				$w_style = "width: {$w}em; ";
		}
		foreach(array('style', 'disabled', 'readonly', 'tabindex', 'autocomplete',
				'onkeypress', 'onkeydown', 'onkeyup', 'placeholder') as $f)
			if(isset($spec[$f]) && ($f != 'disabled' || $spec[$f]))
				$attrib[$f] = $spec[$f];
		if(! empty($w_style))
			$attrib['style'] = $w_style . (isset($attrib['style']) ? $attrib['style'] : '');
		if(! empty($spec['align']))
			$attrib['style'] = (isset($attrib['style']) ? $attrib['style'] : '') . '; text-align: ' . $spec['align'];
		$format_regexp = array_get_default($spec, 'format_regexp', '');
		if(isset($spec['min_decimals']))
			$min_decimals = $spec['min_decimals'];
		$params = @compact('name', 'format', 'decimals', 'min_decimals', 'min', 'max', 'format_regexp');
		$params['label'] = $this->getLabel($spec);
		if(! is_null($params['label']))
			$attrib['aria-label'] = $params['label'];
		if(! empty($spec['required']))
			$params['required'] = true;
		if(! empty($spec['onchange']))
			$params['onchange'] = $spec['onchange'];
        if(! empty($spec['disable_num_format']))
			$params['no_format'] = true;
        if(! empty($spec['currency_id']))
			$params['currency_id'] = $spec['currency_id'];
		if(! empty($spec['alt_name']))
			$params['alt_name'] = $spec['alt_name'];
        if(! empty($spec['check_strength']))
			$params['check_strength'] = $spec['check_strength'];
		if($errs)
			$params['invalid'] = true;
		if(! empty($spec['search_dom'])) {
			$params['search_dom'] = $spec['search_dom'];
			$params['show_search'] = true;
		}
		if(! empty($spec['key_name']))
			$params['key_name'] = $spec['key_name'];
		if (!empty($spec['prevent_autofill'])) {
			if ($t == 'password') {
				$attrib['autocomplete'] = 'new-password';
			} else {
				$attrib['autocomplete'] = 'nope';
			}
			$ret .= '<input type="' . $attrib['type']
				. '" name="fake' . uniqid() . '"'
			   . '	style="width: 0px; margin: 0; padding: 0; opacity: 0; border: solid 1px transparent" disabled>';
		}
		$this->addControl('TextInput', $sel_id, $params);
		$ret .= self::get_tag('input', $attrib, true);
		return $ret;
	}

	function renderStripeElement(array &$spec, $value, $errs=null) {
		$name = $this->getFieldNamePrefix() . $spec['name'];
		$sel_id = $this->getFieldIdPrefix() . $name.'-input';
		$ret = '';
		$attrib = array(
			'id' => $sel_id,
		);

		//$this->addControl('TextInput', $sel_id, $params);
		$ret .= self::get_tag('div', $attrib, true);
		$ret .= self::get_tag('div', array('id' => 'card-errors', 'role' => 'alert'), true);
		return $ret;
	}

    function renderHtmlEditor(array $spec, $value, $extra_tags = false) {
        require_once("include/CKeditor_IAH/CKeditor_IAH.php");

		$name = $this->getFieldNamePrefix() . $spec['name'];
		$sel_id = $this->getFieldIdPrefix() . $name . '-input';
        $oCKeditor = new CKeditor_IAH();
        $cfg = array();

        if(isset($spec['height']))
        	$cfg['height'] = $spec['height'].'px';
        else
        	$cfg['height'] = $oCKeditor->config['height'];
        if (isset($spec['fullPage']) && $spec['fullPage'] == true) {
            $cfg['fullPage'] = true;
        } else {
            $value = to_html2($value);
        }

        $ret = $oCKeditor->editor2($sel_id, $name, $value, $extra_tags);
        $attrs = array(
        	'name' => $name,
        	'label' => $this->getLabel($spec),
        	'ck_config' => $cfg,
        );
        $this->addControl('HtmlInput', $sel_id, $attrs);

        return $ret;
    }

	function renderTextArea(array $spec, $value) {
		$name = $this->getFieldNamePrefix() . $spec['name'];
		$sel_id = $this->getFieldIdPrefix() . $name.'-input';
		$attr = array('name' => $name, 'id' => $sel_id,
			'class' => $this->getFieldClass('input-text', $spec),
			'aria-label' => $this->getLabel($spec)
		);
		$rows = $attr['rows'] = array_get_default($spec, 'height', 4);
		$cols = $attr['cols'] = array_get_default($spec, 'width', 60);
		if (AppConfig::is_mobile())
			$cols = $attr['cols'] = 25;
		// created_by_module_designer is checked here to fix #3006 for existing data
		if(! empty($spec['len']) && empty($spec['created_by_module_designer']))
			$attr['maxlength'] = $spec['len'];
		$params = array(
			'name' => $name,
			'label' => $this->getLabel($spec),
		);
		if(! empty($spec['required']))
			$params['required'] = true;
		if(! empty($spec['onchange']))
			$params['onchange'] = $spec['onchange'];
		$this->addControl('TextAreaInput', $sel_id, $params);
		return self::get_tag('textarea', $attr) . to_html($value) . '</textarea>';
	}

	function renderDateTime(array $spec, $value) {
		$name = $this->getFieldNamePrefix() . $spec['name'];
		$inp_id = $this->getFieldIdPrefix() . $name;
		$sel_id = $inp_id . '-input';
		$ret = self::get_tag('input', array(
			'type' => 'hidden', 'name' => $name, 'id' => $inp_id . '-hidden',
			'value' => $value));
		if(! empty($spec['alt_options'])) {
			$ret .= $this->renderAltOptions($spec);
		}
		$cls = $this->getFieldClass('input-field input-field-group rbullet datetime-input', $spec);
		$span_cls = 'input-label datetime-label text-number';
		$icon_outer_cls = 'input-label input-field-icon lg';
		//$icon_cls = 'input-icon icon-calendar';
		$icon_cls = 'uii uii-lg uii-date';
		$format = array_get_default($spec, 'type', 'datetime');
		if(array_get_default($spec, 'format') == 'date_only')
			$format = 'date';
		if($format === 'time')
			$icon_cls = 'uii uii-lg uii-time';

		$ph = array_get_default($spec, 'placeholder', $this->getDefaultPlaceholder($format));
		$style = '';
		if(empty($spec['var_width']) && empty($spec['quick']))
			$style = 'min-width: 10rem; ';
		$style .= array_get_default($spec, 'style', '');

		$ret .= self::get_tag('div', ['class' => $cls, 'id' => $sel_id,
			'style' => $style,
			'tabindex' => array_get_default($spec, 'tabindex', '0')]);
		$ret .= self::get_tag('div', ['class' => $icon_outer_cls]);

		$ret .= self::get_tag('div', ['class' => $icon_cls], true);
		$ret .= '</div>';
		$ret .= self::get_tag('div', ['class' => $span_cls]) . $value . '</div>';
		$ret .= '</div>';

		$ret = self::get_tag('div', ['class' => 'input-wrapper', 'id' => $inp_id]) . $ret . '</div>';

		global $timedate;
		$attrs = array(
			'name' => $name,
			'format' => $format,
			'label' => $this->getLabel($spec),
			'required' => ! empty($spec['required']),
			'placeholder' => $ph,
		);
		if(! empty($spec['alt_name']))
			$attrs['alt_name'] = $spec['alt_name'];
		if(! empty($spec['onchange']))
			$attrs['onchange'] = $spec['onchange'];
		if(! empty($spec['disable_date']))
			$attrs['disable_date'] = $spec['disable_date'];
		$this->addControl('DateTimeInput', $inp_id, $attrs);

		return $ret;
	}

    /**
     * Using in MonthlyServices only for now
     */
    function renderPeriodDate(array $spec, $value) {
        global $timedate;

        $name = $this->getFieldNamePrefix() . $spec['name'];
        $sel_id = $this->getFieldIdPrefix() . $name;
        $display = empty($value) ? "display: none;" : "";
        $prev_icon = self::get_tag('div', array('class' => 'input-icon icon-prev'), true);
        $next_icon = self::get_tag('div', array('class' => 'input-icon icon-next'), true);;

        $ret = '<span>';
        $ret .= self::get_tag('a', array('id' => $name.'_prev', 'href' => '#', 'class' => 'listViewTdToolsS1',
            'onclick' => "changeDate('subtract', '{$this->form_name}'); return false;", 'style' => $display) );
        $ret .= $prev_icon . '&nbsp;</a><span id="'.$name.'_view">'.$value.'</span>';
        $ret .= self::get_tag('a', array('id' => $name.'_next', 'href' => '#', 'class' => 'listViewTdToolsS1',
            'onclick' => "changeDate('add', '{$this->form_name}'); return false;", 'style' => $display) );
        $ret .= '&nbsp' . $next_icon . '</a>';
        $ret .= self::get_tag('input', array('type' => 'hidden', 'id' => $sel_id, 'name' => $name, 'value' => $value));
        $ret .= "</span>";

        return $ret;
    }

    function resolveModelName(array $spec) {
		$model = $this->base_model;

		if($spec) {

			if (isset($spec['full_link'])) {
				if ($this->list_query) {
					$link_model = $this->list_query->getModelName($spec['full_link']);
					if ($link_model)
						return $link_model;
				}
			}

			$link_name = array_get_default($spec, 'link');
			$field = array_get_default($spec, 'field');
			$parts = explode('.', $field);
			array_pop($parts);
			if(count($parts)) {
				$parts = implode('.', $parts);
				$link_name = $link_name ? ($link_name . '.' . $parts) : $parts;
			}
			if($model && $link_name) {
				$parts = explode('.', $link_name);
				foreach($parts as $link) {
					$model = $model->getLinkTargetModel($link);
					if(! $model) break;
				}
			}
		}
		return $model ? $model->getModelName() : null;
    }

    function renderDateFilter(array $spec, $value, $ext_values=null) {
		$basename = $this->getFieldNamePrefix() . $spec['name'];
		$name = $basename . '-filter';
		$sel_id = $this->getFieldIdPrefix() . $name;
		$attrs = ['id' => $sel_id, 'class' => $this->getFieldClass('input-field rbullet', $spec, true, true)];
		if(isset($spec['tabindex'])) $attrs['tabindex'] = $spec['tabindex'];
		if(! empty($spec['var_width'])) {
			$w = $spec['width'];
			if(is_numeric($w)) {
				$w = max(10, min($this->max_ref_width, $w));
				$w = round($w + 1, 2);
				$w .= 'em';
			}
			$attrs['style'] = "width: {$w}";
		}

		$filt_ops = FilterForm::$date_filter_ops;
		$filt_pds = FilterForm::$date_filter_periods;
		if(! is_array($ext_values))
			$ext_values = array();

		$operator = array_get_default($spec, 'operator');
		if(! $operator)
			$operator = array_get_default($ext_values, 'operator');
		if(! in_array($operator, $filt_ops))
			$operator = $filt_ops[0];
		$period = array_get_default($spec, 'period');
		if(! $period)
			$period = array_get_default($ext_values, 'period');
		if(! in_array($period, $filt_pds))
			$period = $filt_pds[0];
		$end = array_get_default($spec, 'end');
		if(! $end)
			$end = array_get_default($ext_values, 'end');
		if(! empty($end)) $end = $GLOBALS['timedate']->to_display_date($end, false);

		if($value == 'Today' && ($operator == 'before_date' || $operator == 'after_date')) {
			$value = translate('LBL_DAY_CAL_LINK', 'app');
		}
		$ret = self::get_tag('input', ['type' => 'hidden', 'name' => $basename, 'value' => $value]);

		$oper_name = $basename.'-operator';
		$ret .= self::get_tag('input', ['type' => 'hidden', 'name' => $oper_name, 'value' => $operator]);

		$period_name = $basename.'-period';
		$ret .= self::get_tag('input', ['type' => 'hidden', 'name' => $period_name, 'value' => $period]);

		$end_name = $basename.'-end';
		$ret .= self::get_tag('input', ['type' => 'hidden', 'name' => $end_name, 'value' => $end]);

		$dom = 'date_range_operators_dom';
		if(! $operator || $operator == 'any') {
			$lbl = translate_dd($dom, 'any');
		}
		else if($operator == 'empty' || $operator == 'not_empty')
			$lbl = translate_dd($dom, $operator);
		else if($operator == 'today' || $operator == 'tomorrow' || $operator == 'yesterday' || $operator == 'after_today' || $operator == 'before_today')
			$lbl = translate_dd('date_filter_operators_dom', $operator);
		else if($operator == 'on_date' || $operator == 'not_on_date') {
			$lbl = $value;
			if($operator == 'not_on_date')
				$lbl = translate_dd($dom, 'not') . ' ' . $lbl;
		}
		else if($operator == 'between_dates') {
			$lbl = translate_dd($dom, 'from_pfx');
			$lbl .= ' ' . $value . ' ';
			if($end) {
				$lbl .= translate_dd('date_range_embed_dom', 'to');
				$lbl .= ' ' . $end;
			}
		}
		else if($operator == 'before_date' || $operator == 'after_date') {
			$lbl = translate_dd($dom, $operator == 'before_date' ? 'before_pfx': 'after_pfx');
			$lbl .= ' ' . $value;
		}
		else if($operator == 'period_current' || $operator == 'period_next' || $operator == 'period_prev') {
			$pfx = str_replace('period_', '', $operator);
			if($pfx == 'prev') $pfx = 'previous';
			$lbl = translate_dd($dom, $pfx);
			$pd = str_replace('fiscal_', 'fiscal', $period);
			if(substr($pd, 0, 5) == 'days_')
				$lbl .= ' ' . substr($pd, 5) . ' ' . translate_dd('interval_units_dom', 'days');
			else
				$lbl .= ' ' . translate_dd('interval_unit_dom', $pd . 's');
		}
		$ret .= self::get_tag('div', $attrs) . '<span class="input-label">' . to_html($lbl) . '</span></div>';
		$params = array(
			'field_name' => array_get_default($spec, 'field', $basename),
			'name' => $basename,
			'base_name' => $spec['name'],
			'bean_name' => $this->resolveModelName($spec),
			'label' => $this->getLabel($spec),
			'required' => false,
			'compat' => true,
		);
		if(! empty($spec['auto_submit'])) $params['auto_submit'] = true;
		if(! empty($spec['disabled'])) $params['disabled'] = true;
		$this->addControl('DateRangeInput', $sel_id, $params);
		return $ret;
	}

	function renderDateRange($spec, $value='') {
		require_once('include/database/DateRange.php');
		$range = new IAHDateRange();
		$range->setModel($this->base_model);

		$expand = $range->expandFilter($spec, $value, true);
		$basename = $this->getFieldNamePrefix() . $expand['name'];
		$name = $basename . '-filter';
		$sel_id = $this->getFieldIdPrefix() . $name;
		$attrs = ['id' => $sel_id, 'class' => $this->getFieldClass('input-field rbullet', $spec, true, true)];
		if(isset($spec['tabindex'])) $attrs['tabindex'] = $spec['tabindex'];
		if(! empty($spec['var_width'])) {
			$w = $spec['width'];
			if(is_numeric($w)) {
				$w = max(10, min($this->max_ref_width, $w));
				$w = round($w + 1, 2);
				$w .= 'em';
			}
			$attrs['style'] = "width: {$w}";
		}

		$ret = self::get_tag('input', ['type' => 'hidden', 'name' => $basename, 'value' => $expand['encoded']]);
		$ret .= self::get_tag('div', $attrs) . '<span class="input-label">' . to_html($expand['text']) . '</span></div>';
		$params = array(
			'name' => $basename,
			'base_name' => $expand['name'],
			'base_module' => $expand['module'],
			'bean_name' => $expand['bean_name'],
			'label' => $this->getLabel($spec),
			'required' => false,
			'parsed' => $expand['filter'],
		);
		if(! empty($spec['auto_submit'])) $params['auto_submit'] = true;
		if(! empty($spec['disabled'])) $params['disabled'] = true;
		$this->addControl('DateRangeInput', $sel_id, $params);
		return $ret;
	}

    function renderTextFilter(array $spec, $value, $ext_values=null) {
		$filt_ops = FilterForm::$text_filter_ops;
		if(! is_array($ext_values))
			$ext_values = array();

		$operator = array_get_default($spec, 'match', array_get_default($spec, 'operator'));
		if(! isset($operator))
			$operator = array_get_default($ext_values, 'operator');
		if(! isset($operator) || ! in_array($operator, $filt_ops))
			$operator = 'like';

		$basename = $this->getFieldNamePrefix() . $spec['name'];
		$sel_id = $this->getFieldIdPrefix() . $basename;
		$oper_name = $spec['name'].'-operator';
		$blank = ! strlen(trim($value));

		$render_op = $operator;
		if($render_op && $blank && $render_op != 'empty' && $render_op != 'not_empty')
			$render_op = '';

		$oper_lbl = translate_dd('text_filter_operators_dom', $render_op);
		if($render_op == 'eq' && ! $blank) {
			$oper_lbl = $value;
		}
		else if($render_op && $render_op != 'empty' && $render_op != 'not_empty') {
			if(! $blank)
				$oper_lbl .= ': ' . $value;
			else
				$oper_lbl .= ': ' . translate('LBL_ANY', 'app');
		}

		$ret = self::get_tag('div', ['class' => 'input-wrapper', 'id' => $sel_id]);
		$ret .= self::get_tag('input', ['type' => 'hidden', 'name' => $basename.'-operator', 'value' => $operator], true);
		$ret .= self::get_tag('input', ['type' => 'hidden', 'name' => $basename, 'value' => $value], true);
		$cls = 'input-field rbullet';
		$cls = $this->getFieldClass($cls, $spec);
		$lbl_cls = 'input-label filter-label';
		if(! $render_op) $lbl_cls .= ' text-placeholder';
		$attrs = ['class' => $cls, 'style' => '', 'tabindex' => '0', 'id' => $sel_id.'-input'];
		foreach(array('style', 'tabindex') as $f)
			if(isset($spec[$f]))
				$attrs[$f] = $spec[$f];
		$ret .= self::get_tag('div', $attrs);
		$ret .= self::get_tag('div', ['class' => $lbl_cls]);
		$ret .= $oper_lbl . '</div></div>';
		$ret .= '</div>';

		$this->addControl('TextFilterInput', $sel_id, "{name: '$basename'}");
		return $ret;
	}

    function renderSelectFilter(array $spec, $value) {
        $filt_ops = FilterForm::$select_filter_ops;

        $operator = array_get_default($spec, 'operator');
        if(! in_array($operator, $filt_ops)) {
			$operator = FilterForm::DEFAULT_SELECT_FILTER_OPERATOR;
        }
        if($operator == 'eq' && ! is_array($value) && ! strlen($value))
			$operator = '';

		$basename = $this->getFieldNamePrefix() . $spec['name'];
		$sel_id = $this->getFieldIdPrefix() . $basename;
		$oper_name = $spec['name'].'-operator';
		$opts = $this->getSelectOptions($dom, $spec, $basename);

		$render_op = $operator;
		if($render_op && ! isset($value))
			$render_op = '';

		$oper_lbl = translate_dd('select_filter_operators_dom', $render_op);
		if(is_array($value)) {
			$value = implode('^,^', $value);
			$oper_lbl = ''; // JS only currently
		} else if($render_op && $render_op != 'empty' && $render_op != 'not_empty') {
			$selidx = self::get_selected_index($opts, $value);
			$label_key = array_get_default($spec, 'label_key', 'name');
			if(isset($selidx))
				$oper_val = array_get_default($opts['values'], $selidx, '');
			else
				$oper_val = null;
			if(is_array($oper_val))
				$oper_val = array_get_default($oper_val, $label_key);

			if(! strlen($oper_val)) $oper_val = translate('LBL_MISSING_VALUE', 'app');
			if($render_op == 'eq')
				$oper_lbl = $oper_val;
			else
				$oper_lbl .= ' ' . $oper_val;
		}

		$ret = self::get_tag('div', ['class' => 'input-wrapper', 'id' => $sel_id]);
		$ret .= self::get_tag('input', ['type' => 'hidden', 'name' => $basename.'-operator', 'value' => $operator], true);
		$ret .= self::get_tag('input', ['type' => 'hidden', 'name' => $basename, 'value' => $value], true);
		$cls = 'input-field rbullet';
		$cls = $this->getFieldClass($cls, $spec);
		$lbl_cls = 'input-label filter-label';
		if(! $render_op) $lbl_cls .= ' text-placeholder';
		$attrs = ['class' => $cls, 'style' => '', 'tabindex' => '0', 'id' => $sel_id.'-input'];
		foreach(array('style', 'tabindex') as $f)
			if(isset($spec[$f]))
				$attrs[$f] = $spec[$f];
		$ret .= self::get_tag('div', $attrs);
		$ret .= self::get_tag('div', ['class' => $lbl_cls]);
		$ret .= $oper_lbl . '</div></div>';
		$ret .= '</div>';

		$attrs = [
			'name' => $basename,
			'options' => $opts,
			'add_blank' => false, //array_get_default($spec, 'options_add_blank', true),
			'auto_submit' => ! empty($spec['auto_submit']),
			'fixed_operator' => array_get_default($spec, 'fixed_operator'),
		];
        $init_value = is_array($value) ? $value[0] : $value;
        if (empty($init_value) && isset($spec['default_value']))
            $attrs['init_value'] = $spec['default_value'];

        $this->addControl('SelectFilterInput', $sel_id, $attrs);
        return $ret;
    }

    function renderNumberFilter(array $spec, $value, $ext_values=null) {
		$filt_ops = FilterForm::$number_filter_ops;
		if(! is_array($ext_values))
			$ext_values = array();
		if($spec['type'] == 'currency' || $spec['type'] == 'base_currency')
			$fparams = array('use_currency_decimals' => 1);
		else
			$fparams = null;
		if($spec['type'] != 'duration') {
			if(isset($value)) $value = format_number($value, -1, null, $fparams);
		}

		$operator = array_get_default($spec, 'operator');
		if(! $operator)
			$operator = array_get_default($ext_values, 'operator');
		if(! in_array($operator, $filt_ops))
			$operator = 'eq';
		$end = array_get_default($spec, 'end');
		if(! isset($end))
			$end = array_get_default($ext_values, 'end');
		if($spec['type'] != 'duration') {
			if(isset($end)) $end = format_number($end, -1, null, $fparams);
		}

		$basename = $this->getFieldNamePrefix() . $spec['name'];
		$sel_id = $this->getFieldIdPrefix() . $basename;
		$oper_name = $spec['name'].'-operator';

		$render_op = $operator;
		if($render_op && ! isset($value))
			$render_op = '';

		$oper_lbl = translate_dd('number_filter_operators_dom', $render_op);
		if($render_op && $render_op != 'empty' && $render_op != 'not_empty') {
			if(isset($value))
				$oper_lbl .= ' ' . $value;
			else
				$oper_lbl .= ' ' . translate('LBL_ANY', 'app');
			if($render_op == 'between') {
				$oper_lbl .= translate('LBL_FILTER_VALUE_AND', 'app');
				if(isset($end))
					$oper_lbl .= ' ' . $end;
				else
					$oper_lbl .= ' ' . translate('LBL_ANY', 'app');
			}
		}

		$ret = self::get_tag('div', ['class' => 'input-wrapper', 'id' => $sel_id]);
		$ret .= self::get_tag('input', ['type' => 'hidden', 'name' => $basename.'-operator', 'value' => $operator], true);
		$ret .= self::get_tag('input', ['type' => 'hidden', 'name' => $basename, 'value' => $value], true);
		$ret .= self::get_tag('input', ['type' => 'hidden', 'name' => $basename.'-end', 'value' => $end]);
		$cls = 'input-field rbullet';
		$cls = $this->getFieldClass($cls, $spec);
		$lbl_cls = 'input-label filter-label';
		if(! $render_op) $lbl_cls .= ' text-placeholder';
		$attrs = ['class' => $cls, 'style' => '', 'tabindex' => '0', 'id' => $sel_id.'-input'];
		foreach(array('style', 'tabindex') as $f)
			if(isset($spec[$f]))
				$attrs[$f] = $spec[$f];
		$ret .= self::get_tag('div', $attrs);
		$ret .= self::get_tag('div', ['class' => $lbl_cls]);
		$ret .= $oper_lbl . '</div></div>';
		$ret .= '</div>';

		$this->addControl('NumberFilterInput', $sel_id, "{name: '$basename'}");
		return $ret;
	}


	function renderDuration(array $spec, $value, $is_all_day) {
		global $app_strings;

		$duration = $this->format_duration($value);
		$name = $this->getFieldNamePrefix() . $spec['name'];
		$sel_id = $this->getFieldIdPrefix() . $name;
		$disabled = false;

		$all_day = array_get_default($spec, 'all_day_flag');
		if($all_day) {
			$check_spec = array('type' => 'bool', 'name' => $all_day, 'label' => $app_strings['LBL_DURATION_ALL_DAY'], 'show_label' => true, 'class' => 'border-left');
			$all_day_body = $this->renderCheck($check_spec, $is_all_day);
			$all_day_field = $all_day;
			if($is_all_day) $disabled = true;
		} else
			$all_day_body = $all_day_field = '';

		$ret = self::get_tag('input', array('type' => 'hidden', 'id' => $sel_id, 'name' => $name, 'value' => $value));
		$attribs = array(
			'type' => 'text', 'class' => 'input-text', 'id' => $sel_id.'-time',
			'size' => 11, 'value' => $duration);
		if($disabled)
			$attribs['disabled'] = 'disabled';
		$ret .= self::get_tag('input', $attribs, true);
		if($all_day_body) {
			$ret = self::get_tag('div', ['class' => 'input-field input-field-group', 'id' => $sel_id.'-input'])
				. $ret . $all_day_body . '</div>';
		}

        $hours_abbrev = $app_strings['LBL_HOURS_ABBREV'];
        $min_abbrev = $app_strings['LBL_MINS_ABBREV'];

		$attrs = array(
			'name' => $name,
			'label' => $this->getLabel($spec),
			'all_day_name' => $all_day_field,
			'hours_abbrev' => $hours_abbrev,
			'mins_abbrev' => $min_abbrev,
			'required' => ! empty($spec['required']),
		);
		if(! empty($spec['onchange']))
			$attrs['onchange'] = $spec['onchange'];
		$this->addControl('DurationInput', $sel_id, $attrs);

		return $ret;
	}

	function renderReminder(array $spec, $value) {
		if(! isset($spec['options'])) {
			$spec['options'] = 'reminder_time_options';
			$spec['options_add_blank'] = false;
		}
		return $this->renderSelect($spec, $value);
	}

	function renderSelect(array $spec, $selected) {
		$simple = false; //AppConfig::is_mobile();
		$name = $this->getFieldNamePrefix() . $spec['name'];
		$type = array_get_default($spec, 'type');
		if(empty($spec['required']) && ! isset($spec['options_add_blank']))
			$spec['options_add_blank'] = true;
		if(! isset($spec['options_blank_label']) && ($this->style == 'massupdate' || $this->style == 'searchform')) {
			$spec['options_blank_label'] = $this->getDefaultPlaceholder($type);
		}
		$opts =& $this->getSelectOptions($dom, $spec, $name);
		$selidx = self::get_selected_index($opts, $selected);

		$attrs = array('name' => $name);

		if ($simple) {
			$ret = self::get_tag('select', array('name' => $name), false);
			foreach ($opts['keys'] as $i => $k) {
				$optAttrs = array('value' => $k);
				if ($selected == $k) $optAttrs['selected'] = 'selected';
				$ret .= self::get_tag('option', $optAttrs, false);
				$text = $opts['values'][$i];
				if (is_array($text))
					$text = array_get_default($text, 'name', '');
				$ret .= $text;
				$ret .= '</option>';
			}
			$ret .= '</select>';
		} else {
			$ret = '';

			if(! empty($spec['alt_options'])) {
				$ret .= $this->renderAltOptions($spec);
			}

			if(empty($spec['no_hidden']))
				$ret .= self::get_tag('input', array('type' => 'hidden', 'name' => $name, 'value' => $selected));
			$sel_id = $this->getFieldIdPrefix() . $name . '-input';
			$label_key = array_get_default($spec, 'label_key', 'name');
			$opt_icon = array_get_default($spec, 'options_icon', '');
			if($type == 'module_name' && ! $opt_icon)
				$opt_icon = 'key';
			$icon = array_get_default($spec, 'icon', '');
			$compact = array_get_default($spec, 'compact', false);
			if($this->style == 'searchform' || $this->style == 'massupdate') $compact = true;
			$var_width = array_get_default($spec, 'var_width', $compact || array_get_default($spec, 'quick'));
			$dyn_mod = array_get_default($spec, 'dynamic_module');
			if($dyn_mod) {
				$dval = array_get_default($spec, 'dynamic_module_value', '');
				$ret .= self::get_tag('input', array('type' => 'hidden', 'name' => $dyn_mod, 'value' => $dval));
			}

			if(isset($selidx))
				$selopt = array_get_default($opts['values'], $selidx, '');
			else
				$selopt = null;
			if(is_array($selopt))
				$sellbl = array_get_default($selopt, $label_key);
			else
				$sellbl = $selopt;
			if(! isset($sellbl))
				$sellbl = array_get_default($spec, 'placeholder', '');

			$cls = 'input-field input-field-group rbullet';
			$cls = $this->getFieldClass($cls, $spec, true, true);
			$attrs = array('class' => $cls, 'style' => '', 'tabindex' => '0', 'id' => $sel_id, 'aria-label' => $this->getLabel($spec));
			foreach(array('style', 'tabindex') as $f)
				if(isset($spec[$f]))
					$attrs[$f] = $spec[$f];
			if(isset($spec['align']))
				$attrs['style'] .= '; float: '.$spec['align'];
			if(empty($spec['no_label']) && ! $var_width) {
				$w = array_get_default($spec, 'width');
				if(! $w && $opts['maxlen'] > 2) $w = $opts['maxlen'] - 1;
				if($w) {
					if(is_numeric($w)) {
						$w = max(8, min($this->max_ref_width, $w));
						$w .= 'em';
					}
					$attrs['style'] .= "; width: {$w}";
				}
			}
			$ret .= self::get_tag('div', $attrs);

			if($opt_icon || $icon) {
				if($opt_icon == 'key') $ico_val = $selected;
				else $ico_val = $opt_icon ? (is_array($selopt) ? array_get_default($selopt, $opt_icon, '') : '') : $icon;
				$lg = '';
				if($ico_val && strpos($ico_val, '-') === false) $ico_val = 'input-icon theme-icon module-' . $ico_val;
				else if($ico_val && strpos($ico_val, 'uii') === false) $ico_val = 'input-icon '.$ico_val;
				else if(strpos($ico_val, 'uii-lg') !== false) $lg = ' lg';
				if($ico_val && array_get_default($spec, 'icon_active', true)) $ico_val = 'active '.$ico_val;
				$ret .= self::get_tag('div', array('class' => 'input-label input-field-icon '.$lg,
					'style' => $ico_val ? '' : 'display: none'));
				$ret .= self::get_tag('div', array('class' => $ico_val, 'id' => $sel_id.'-icon'), true);
				$ret .= '</div>';
			}

			if(empty($spec['no_label'])) {
				$lbl_cls = 'input-label single';
				$blank_val = array_get_default($opts, 'blank_key', '');
				if($selected === $blank_val) $lbl_cls .= ' text-placeholder';
				$attrs = array('class' => $lbl_cls, 'id' => $sel_id.'-label');
				$ret .= self::get_tag('div', $attrs);
				$ret .= to_html($sellbl);
				$ret .= '</div>';
			}

			$ret .= '</div>';

			$l = max(15, min(35, $opts['maxlen']));
			if(! isset($opts['width']))
				$opts['width'] = $l.'em';
			$attribs = array(
				'name' => $name,
				'label' => $this->getLabel($spec),
				'label_key' => $label_key,
				'options' => $dom,
				'no_label' => ! empty($spec['no_label']),
				'required' => ! empty($spec['required']),
				'auto_submit' => ! empty($spec['auto_submit']),
				'ignore_dirty' => ! empty($spec['ignore_dirty']),
			);
			if($opt_icon)
				$attribs['icon_key'] = $opt_icon;
			else if($icon)
				$attribs['static_icon'] = true;
			if(isset($spec['outer_id']))
				$attribs['outer_id'] = $spec['outer_id'];
			if(! empty($spec['onchange']))
				$attribs['onchange'] = $spec['onchange'];
			if(! empty($spec['alt_name']))
				$attribs['alt_name'] = $spec['alt_name'];
			if(isset($spec['mark_blank']))
				$attribs['mark_blank'] = $spec['mark_blank'];
			if(! empty($dyn_mod))
				$attribs['dynamic_module'] = $dyn_mod;
			if(! empty($opts['dom_name'])) {
				unset($opts['keys']);
				unset($opts['values']);
				$attribs['options'] = $opts;
			}
			if(! empty($spec['placeholder']))
				$attribs['placeholder'] = $spec['placeholder'];
			if(! empty($spec['exchange_rate']))
				$attribs['exchange_rate'] = $spec['exchange_rate'];
			else {
				$json = getJSONobj();
				$opts_jso = $json->encode($opts);
				$this->js[] = "SUGAR.ui.registerSelectOptions('$dom', $opts_jso);";
			}
			$cls = array_get_default($spec, 'input_class', 'SelectInput');
			$this->addControl($cls, $sel_id, $attribs);
		}

		return $ret;
	}


	function &getSelectOptions(&$dom, $spec, $elt_id) {
		$icon = array_get_default($spec, 'options_icon', '');
		$group_levels = array_get_default($spec, 'options_group_levels', 0);
		$label_key = array_get_default($spec, 'label_key');
		$type = array_get_default($spec, 'type');
		$ret = array('maxlen' => 0);
		if(! empty($spec['options_processed'])) {
			$dom = $spec['options_name'];
			return $spec['options'];
		}
		if(isset($spec['options_add_blank'])) {
			$ret['add_blank'] = $spec['options_add_blank'];
			if(isset($spec['options_blank_label']))
				$ret['blank_label'] = $spec['options_blank_label'];
		}
		if(isset($spec['options_blank_key'])) {
			$ret['blank_key'] = $spec['options_blank_key'];
		}

		if(! empty($spec['options_function'])) {
			$cb = $spec['options_function'];
			$dom = call_hook_function($cb, null);
		} else if(isset($spec['options']) && $spec['options'] !== '')
			$dom = $spec['options'];
		else
			$dom = null;
		if(is_array($dom)) {
			if($type == 'module_name' && isset($dom[0])) {
				$opts = array();
				$mod_dom = AppConfig::setting('lang.lists.current.app.moduleList', array());
				foreach($dom as $m)
					$opts[$m] = array_get_default($mod_dom, $m, $m);
			} else
				$opts = $dom;
			$dom = $elt_id.'-options';
		} else if(is_string($dom) && substr($dom, 0, 6) == 'model.') {
			if(! $label_key) $label_key = '_display';
			$bean = substr($dom, 6);
			$this->js_models[$bean] = 1;
			$db_rows = AppConfig::db_all_rows(substr($dom, 6));
			$opts = array();
			if(is_array($db_rows)) {
				foreach($db_rows as $k => $v)
					$opts[$k] = $v[$label_key];
			}
			$ret['dom_name'] = $dom;
		} else if($dom) {
			$opts = AppConfig::setting('lang.lists.current.' . $GLOBALS['currentModule'] . '.' . $dom);
			if (!is_array($opts)) {
				$opts = isset($GLOBALS['app_list_strings'][$dom]) ? $GLOBALS['app_list_strings'][$dom] : array();
			} else {
				$ret['dom_module'] = $GLOBALS['currentModule'];
			}
			$ret['dom_name'] = $dom;
			if($this->style == 'massupdate')
				$dom = 'mu_' . $dom;
			else if($this->style == 'searchform')
				$dom = 'srch_' . $dom;
		} else
			$opts = array();
		if(isset($spec['options_limit_keys'])) {
			$limit = array_flip($spec['options_limit_keys']);
			$opts = array_intersect_key($opts, $limit);
			$ret['limit_keys'] = $spec['options_limit_keys'];
		}
		if(! $label_key) $label_key = 'name';
		$ret['display_name'] = $label_key;

		self::process_opts($opts, $ret, $label_key, $group_levels);

		if(isset($spec['options_columns']))
			$ret['columns'] = $spec['options_columns'];
		else if($icon)
			$ret['columns'] = array(array('name' => $label_key, 'icon' => $icon));

		return $ret;
	}

	static function get_selected_index(array $opts, &$selected) {
		if(! isset($opts['keys'])) return null;
		if(is_int($selected)) $selected .= ''; // convert to string
		foreach($opts['keys'] as $i => $k) {
			if(! isset($selected) && ($k === '' || $k === '-99'))
				$selected = $k;
			if($k === $selected) return $i;
		}
		return null;
	}

	static function process_opts(array &$opts, array &$ret, $label_key, $group_levels=0, $idx=0) {
		$len = 0;
		if(! empty($ret['add_blank'])) {
			$lbl = array_get_default($ret, 'blank_label', 'LBL_NONE');
			if($lbl) $lbl = translate($lbl, 'app');
			if(! isset($opts[''])) {
				$ret['keys'][] = '';
				$ret['values'][] = $lbl;
			}
			if(empty($ret['dom_name']))
				unset($ret['add_blank']);
		}
		foreach($opts as $k => &$o) {
			if($group_levels > 0 && is_array($o))
				$idx = max($len, self::max_opt_length($o, $ret, $label_key, $group_levels-1, $idx));
			else {
				if(is_array($o)) {
					if(isset($o[$label_key]))
						$len = max($len, mb_strlen($o[$label_key]));
				} else
					$len = max($len, mb_strlen($o));
				$ret['keys'][] = $k.'';
				$ret['values'][] = is_int($o) ? $o.'' : $o;
				$idx ++;
			}
		}
		$ret['maxlen'] = max($ret['maxlen'], $len);
		return $idx;
	}


	function renderMultiSelect(array $spec, $selected) {
		$label_key = array_get_default($spec, 'label_key', 'name');
		$name = $this->getFieldNamePrefix() . $spec['name'];
		$opts =& $this->getSelectOptions($dom, $spec, $name);
		if(is_array($selected)) {
			$selected = implode('^,^', $selected);
		}

		$attrs = array('name' => $name);

		$ret = self::get_tag('input', array('type' => 'hidden', 'name' => $name, 'value' => $selected));
		$sel_id = $this->getFieldIdPrefix() . $name . '-input';
		$rows = array_get_default($spec, 'display_rows', $this->style == 'editview' ? 4 : 3);
        $compact = array_get_default($spec, 'compact', false);
		$lookup = array_get_default($spec, 'options_lookup');
		$add_remove = array_get_default($spec, 'add_remove', true);
		$full_height = array_get_default($spec, 'full_height', false);

        $cls = 'input-field select-list';
        if($add_remove) $cls .= ' rbullet';
        $cls = $this->getFieldClass($cls, $spec);
        $attrs = array('class' => $cls, 'id' => $sel_id, 'style' => '');
		foreach(array('style') as $f)
			if(isset($spec[$f]))
				$attrs[$f] = $spec[$f];
		if(isset($spec['mode_select'])) { // used by teams massupdate
			$attrs['style'] .= '; display: none';
		}
		$attrs['tabindex'] = array_get_default($spec, 'tabindex', '0');

		$w = array_get_default($spec, 'width', max($opts['maxlen'], $this->default_ref_width));
		if($w) {
			if(is_numeric($w)) {
				$w = max(6, min($this->max_ref_width, $w));
				if($this->context == 'workflow') $w += 4;
				$w .= 'em';
			}
			$st = array_get_default($attrs, 'style', '');
			$attrs['style'] = "$st; width: $w";
		}

		$ret .= self::get_tag('div', $attrs);
		$ret .= self::get_tag('div', array('id' => $sel_id.'-body', 'class' => 'menu-outer input-scroll', 'style' => 'display: none'), true);
		$ret .= self::get_tag('div', array('class' => 'input-label', 'aria-label' => $this->getLabel($spec))) . '&nbsp;' . '</div>';

		$attribs = array(
			'name' => $name,
			'label' => $this->getLabel($spec),
			'options' => $dom,
			'multi_select' => true,
			'rows' => $rows,
			'required' => ! empty($spec['required']),
			'disable_current' => true,
			'full_height' => !!$full_height,
			'label_key' => $label_key,
		);
		if(! empty($spec['options_icon']))
			$attribs['icon_key'] = $spec['options_icon'];
		if(! empty($spec['placeholder']))
			$attribs['placeholder'] = $spec['placeholder'];
		else
			$attribs['placeholder'] = $this->getDefaultPlaceholder('multienum');
		if(! empty($opts['dom_name'])) {
			unset($opts['keys']);
			unset($opts['values']);
			$attribs['options'] = $opts;
		}
		else {
			$json = getJSONobj();
			$opts_jso = $json->encode($opts);
			$this->js[] = "SUGAR.ui.registerSelectOptions('$dom', $opts_jso);";
		}

		if($add_remove) {
			$attribs['add_remove'] = true;
			if(isset($spec['options_sorted']))
				$attribs['add_remove_sorted'] = $spec['options_sorted'];
			if(isset($lookup)) {
				if(! isset($lookup['module']) && isset($lookup['bean_name']))
					$lookup['module'] = AppConfig::module_for_model($lookup['bean_name']);
				$attribs['icon'] = 'theme-icon module-SecurityGroups';
				$attribs['options_lookup'] = $lookup;
			}
		}

		$ret .= '</div>';

		/*$selopt = array_get_default($opts, $selected, '');
		if($icon) {
			$ico_val = is_array($selopt) ? array_get_default($selopt, $icon, '') : '';
			$ret .= self::get_tag('div', array('class' => 'input-icon left '.$ico_val, 'id' => $sel_id.'-icon'));
			$ret .= '</div>';
		}*/

		if(isset($spec['mode_select'])) { // used by teams massupdate
			$mspec = $spec['mode_select'];
			//$mspec['compact'] = true;
			//$mspec['class'] = 'input-outer';
			$mspec['var_width'] = true;
			if(empty($mspec['name']))
				$mspec['name'] = $spec['name'] . '-mode';
			$attribs['mode_name'] = $this->getFieldNamePrefix() . $mspec['name'];
			$mode = $this->renderSelect($mspec, null);
			$ret = $mode . $ret;
		}

		$cls = array_get_default($spec, 'input_class', 'SelectList');
		$this->addControl($cls, $sel_id, $attribs);

		return $ret;
	}


	function renderCheck(array $spec, $value=false) {
		$simple = array_get_default($spec, 'simple'); //AppConfig::is_mobile();
		$name = $this->getFieldNamePrefix() . $spec['name'];
		$sel_id = $this->getFieldIdPrefix() . $name . '-input';

		$check = self::get_tag('input', array(
			'type' => 'hidden',
			'name' => $name,
			'value' => 0,
			'id' => $sel_id.'-hidden'), true);
		$attrs = array(
			'type' => 'checkbox',
			'name' => $name,
			'value' => 1,
			'id' => $sel_id.'-checkbox',
			'tabindex' => array_get_default($spec, 'tabindex', 0),
		);
		if($value)
			$attrs['checked'] = 'checked';
		if(! empty($spec['disabled']))
			$attrs['disabled'] = 'disabled';
		$check .= self::get_tag('input', $attrs, true);

		if($this->classic) {
			$ret = self::get_tag('div', array('id' => $sel_id, 'class' => 'input-wrapper'));
			$ret .= $check . '</div>';
		} else if ($simple) {
			$ret = self::get_tag('div', array('id' => $sel_id, 'class' => $this->getFieldClass('input-check', $spec, false)));
			$ret .= $check;
			$ret .= self::get_tag('label', array('for' => $sel_id.'-checkbox'), true);
			$ret .= '</div>';
		} else {
			$lbl = $this->getLabel($spec);
			$show_label = ! empty($spec['show_label']) && strlen($lbl);
			$toggle = isset($spec['toggle']) ? $spec['toggle'] : ($show_label ? false : true);
			if(! empty($spec['basic'])) // basic filter, like favourites star
				$toggle = false;

			$cls = $this->getFieldClass($toggle ? 'input-toggle' : 'input-field input-check', $spec);
			$attrs = array(
				'id' => $sel_id,
				'class' => $cls,
			);
			$ret = self::get_tag('div', $attrs);
			$ret .= $check;
			$ret .= self::get_tag('label', array('class' => 'input-label', 'for' => $sel_id.'-checkbox'));
			if($toggle) $ret .= '<div class="switch"></div>';
			if($show_label) {
				$ret .= ' ' . to_html($lbl);
			}
			$ret .= '</label></div>';

			if(! empty($spec['suffix_label']) && strlen($lbl)) {
				$ret .= ' '.to_html($lbl);
			}
			$attrs = array(
				'name' => $name,
				'label' => $lbl,
			);
			if(! empty($spec['auto_submit']))
				$attrs['auto_submit'] = $spec['auto_submit'];
			if(! empty($spec['onchange']))
				$attrs['onchange'] = $spec['onchange'];
			$cls = array_get_default($spec, 'input_class', 'CheckInput');
			$this->addControl($cls, $sel_id, $attrs);
		}
		return $ret;
	}


	function renderCurrencySelect(RowResult $result, array $spec, $id) {
		$exrate = array_get_default($spec, 'exchange_rate');
		$exdef = $exrate ? array_get_default($result->fields, $exrate) : null;
		$ret = '<div class="input-button-group">';
		$spec['input_class'] = 'CurrencySelect';
		$estyle = '';
		if($exdef) {
			if($id != '-99')
				$spec['class'] = 'hard-right';
			else
				$estyle = 'style="display: none"';
		}
		$ret .= $this->renderSelect($spec, $id);
		$name = $this->getFieldNamePrefix() . $spec['name'];
		$exb_id = $this->getFieldIdPrefix() . $name.'-input-editrate';
		if($exdef) {
			$ex_name = $this->getFieldNamePrefix() . $exrate;
			$ex_id = $this->getFieldIdPrefix() . $ex_name;
			$ret .= '<input type="hidden" id="'.$ex_id.'" name="'.$ex_name.'" value="'.to_html($result->getField($exrate, 1, true)).'" />';
			$this->addControl('HiddenInput', $ex_id, "{name: '$ex_name'}");
			$exc_label = translate('LBL_EXCHANGE_RATE', 'app');
			$ret .= '<button class="input-button input-outer hard-left dark" type="button" title="'.$exc_label.'" id="'.$exb_id.'" '.$estyle.'><div class="input-icon icon-exchangerate"></div></button>';
		}
		$ret .= '</div>';
		return $ret;
	}


	function renderAltOptions($spec, $result=null, $value=null) {
		$name = $spec['name'];
		$aopts = $spec['alt_options'];
		if(! isset($aopts[''])) {
			$aopts = array_merge( array('' => translate('LBL_SELECT_OTHER', 'app')), $aopts);
		}
		$alt_name = array_get_default($spec, 'alt_name', $name.'_alt');
		$spec['alt_name'] = $this->getFieldNamePrefix() . $alt_name;
		$alt_spec = array(
			'name' => $alt_name,
			'vname' => $spec['vname'],
			'vname_module' => array_get_default($spec, 'vname_module'),
			'type' => 'enum',
			'options' => $aopts,
			'var_width' => true,
		);
		if(isset($spec['alt_value']))
			$alt_v = $spec['alt_value'];
		else if($result)
			$alt_v = $result->getField($alt_name);
		else
			$alt_v = $value;
		return $this->renderSelect($alt_spec, $alt_v) . ' ';
	}


	function renderRef(RowResult $result, array $spec) {
		$name = $this->getFieldNamePrefix() . $spec['name'];
		if((isset($spec['field']) && $spec['field'] != $spec['name']) || ! isset($spec['id_name']))
			$id_name = $spec['name'] . '_id';
		else
			$id_name = $spec['id_name'];
		$pid_name = $this->getFieldNamePrefix() . $id_name;
		if(array_key_exists('value', $spec))
			$value = $spec['value'];
		else
			$value = $result->getField($spec['name'], '', false);
		if(array_key_exists('id_value', $spec))
			$id = $spec['id_value'];
		else
			$id = $result->getField($pid_name, '', false);
		$union_mod = array_get_default($spec, 'union_module');
        if (empty($id))
            $id = $result->getField($id_name, '', false);
		if(isset($spec['dynamic_module'])) {
			$dyn_mod_f = $spec['dynamic_module'] . array_get_default($spec, 'unique', '');
			$sel_type = array_get_default($result->fields, $dyn_mod_f, array());
			$dyn_mod = $sel_type['value'] = $result->getField($dyn_mod_f);
			$bean = AppConfig::module_primary_bean($dyn_mod);
		} else {
			$dyn_mod = $dyn_mod_f = $sel_type = null;
			$bean = array_get_default($spec, 'bean_name');
		}

		if($bean && empty($spec['options']) && AppConfig::setting("model.detail.$bean.js_cache.enabled")) {
			$spec['options'] = 'model.'.$bean;
			if($bean == 'Currency' && empty($spec['input_class']))
				$spec['input_class'] = 'CurrencySelect';
		}
		if(isset($spec['options_function']) || isset($spec['options'])) {
			$opts =& $this->getSelectOptions($dom, $spec, $name);
			if(isset($opts['keys']))
			foreach($opts['keys'] as $idx => $k) {
				if($k == $id) {
					$oval = array_get_default($opts['values'], $idx);
					if($oval) {
						if(is_string($oval)) $value = $oval;
						else $value = array_get_default($oval, $opts['display_name']);
					}
				}
			}
			if(! empty($opts['dom_name'])) {
				// use full objects instead of strings on javascript side
				unset($opts['keys']);
				unset($opts['values']);
			}
			$spec['options'] = $opts;
		}

		$sel_id = $this->getFieldIdPrefix() . $name;
		$ret = self::get_tag('div', ['id' => $sel_id, 'class' => 'input-wrapper']);
		$ret .= self::get_tag('input', array('type' => 'hidden', 'name' => $pid_name, 'value' => $id), true);
		$ret .= self::get_tag('input', array('type' => 'hidden', 'name' => $name, 'value' => $value), true);
		if($dyn_mod_f) {
			$dyn_ref = $this->getFieldNamePrefix() . $dyn_mod_f;
			$dyn_ref_id = $this->getFieldIdPrefix(). $dyn_mod_f.'-input';
			$ret .= self::get_tag('input', array('type' => 'hidden', 'name' => $dyn_ref, 'id' => $dyn_ref_id, 'value' => $dyn_mod), true);
			$this->addControl('HiddenInput', $dyn_ref_id, "{name: '$dyn_mod_f'}");
		}
		$hide = false;

		if(! empty($spec['alt_options'])) {
			$ret .= $this->renderAltOptions($spec, $result);
		}

		$icon = ''; // bean-BEANNAME
		$bean_label = '';
		$base_cls = 'input-field input-field-group rbullet ref-input';
		$span_cls = 'input-label ref-label';
		$icon_cls = 'input-label input-field-icon';
		$ph = array_get_default($spec, 'placeholder', $this->getDefaultPlaceholder('ref'));
		if(! $id) {
			$value = $ph;
			$span_cls .= ' text-placeholder';
		}

		$cls = $this->getFieldClass($base_cls, $spec, true, true);
		$attrs = array('class' => $cls, 'id' => $sel_id.'-input', 'style' => '');
		$w = array_get_default($spec, 'width', $this->default_ref_width);
		$var_width = array_get_default($spec, 'var_width');
		if($this->style == 'searchform' || $this->style == 'massupdate') $var_width = true;
		if($var_width || (isset($spec['class']) && $spec['class'] == 'multi-ref'))
			$w = '';
		if($w) {
			if(is_numeric($w)) {
				$w = max(6, min($this->max_ref_width, $w));
				$w .= 'em';
			}
			$attrs['style'] .= "width: {$w}";
		}
		if(isset($spec['style']))
			$attrs['style'] .= '; ' . $spec['style'];
		$attrs['tabindex'] = array_get_default($spec, 'tabindex', 0);
		$ret .= self::get_tag('div', $attrs);

		if($sel_type && ! $union_mod) {
			if($dyn_mod) {
				if($dyn_mod == 'Users') $icon = "theme-icon bean-User";
				else $icon = "theme-icon module-$dyn_mod";
			} else {
				$icon = '';
			}
		} else {
			$mod = $bean ? AppConfig::module_for_model($bean) : '';
            if (! empty($spec['icon'])) {
				$icon = $spec['icon'];
				if(strpos($icon, '-') === false) {
					$icon_info = get_image_info($icon);
					if($icon_info && $icon_info['found']) $icon = $icon_info['class'];
				}
            } else if ($mod) {
            	if($mod == 'Users') $icon = "theme-icon bean-User";
                else $icon = "theme-icon module-$mod";
            }
		}
		$icon_style = $icon ? '' : 'display: none';
		$ret .= self::get_tag('div', array('class' => $icon_cls, 'title' => $bean_label, 'style' => $icon_style));
		$ret .= self::get_tag('div', array('class' => 'input-icon ref-icon '.$icon), true);
		$ret .= '</div>';
		$ret .= self::get_tag('div', array('class' => $span_cls, 'aria-label' => $this->getLabel($spec))) . to_html($value) . '</div>';
		$ret .= '</div>';
		$ret .= '</div>'; // field-outer

		$attribs = array(
			'name' => $name,
			'label' => $this->getLabel($spec),
			'key_name' => $pid_name,
			'required' => ! empty($spec['required']),
			'placeholder' => $ph,
		);
		if(! empty($spec['disabled']))
			$attribs['disabled'] = true;
		else if(! empty($spec['readonly']))
			$attribs['disabled'] = 'readonly';
		foreach(array('allow_custom', 'allow_rename', 'auto_submit') as $f)
			if(! empty($spec[$f])) $attribs[$f] = true;
		if(! empty($attribs['auto_submit']))
			$attribs['submit_empty'] = true;
		if($union_mod)
			$attribs['union_module'] = $union_mod;
		if($dyn_mod_f) {
			$attribs['module_field_name'] = $this->getFieldNamePrefix() . $dyn_mod_f;
			$dyn_f = $result->getFieldDefinition($dyn_mod_f);
			if($dyn_f && isset($dyn_f['options']) && is_array($dyn_f['options'])) {
				$attribs['modules'] = $dyn_f['options'];
			} elseif($dyn_f && !empty($dyn_f['options_function'])) {
				$cb = $dyn_f['options_function'];
				$dom = call_hook_function($cb, null);
				if(is_array($dom)) {
					$attribs['modules'] = $dom;
				}
			}
		}
		else {
			$attribs['module'] = $mod;
			if(array_get_default($spec, 'quick_create', true) && empty($spec['add_filters'])) {
				if($bean && AppConfig::setting("display.edit.$bean.quick_create.via_ref_input")) {
					$qclayout = AppConfig::setting("views.detail.{$mod}.edit.QuickCreate");
					if(ACLController::checkAccess($mod, 'create') && $qclayout) {
						$attribs['quick_create'] = true;
					}
				}
			}
		}
		foreach(array('onchange', 'add_filters', 'popup_callback', 'popup_return_multiple',
				'list_name', 'alt_name', 'options', 'exchange_rate') as $f)
			if(! empty($spec[$f])) $attribs[$f] = $spec[$f];
		if (AppConfig::is_mobile())
			$attribs['show_select'] = true;
		$cls = array_get_default($spec, 'input_class', 'RefInput');
		$this->addControl($cls, $sel_id, $attribs);
		return $ret;
	}


	function renderEmailRecips(RowResult $result, array $spec) {
		$name = $this->getFieldNamePrefix() . $spec['name'];
		$value = $result->getField($name, '', false);

		require_once('modules/Emails/utils.php');
		$arr = @parse_addrs($value);
		if($arr)
			$value = implode('; ', array_column($arr, 'entry'));

		$sel_id = $this->getFieldIdPrefix() . $name.'-input';
		$attrs = array('class' => '', 'id' => $sel_id);
		$attrs['style'] = "width: 100%";
		if(isset($spec['style']))
			$attrs['style'] .= '; ' . $spec['style'];
		$ret = self::get_tag('div', $attrs);
		$attrs = array('type' => 'hidden', 'class' => 'email-recips-input', 'value' => $value, 'name' => $name);
		$ret .= self::get_tag('input', $attrs, true);
		$ret .= '<div class="email-recips-container"></div>';

		$ret .= '</div>';
        $attrs = array(
        	'name' => $name,
        	'label' => $this->getLabel($spec),
        	'required' => ! empty($spec['required']),
			'value' => $value,
			'recips_array' => $arr,
		);
		if(! empty($spec['add_filters']))
			$attrs['add_filters'] = $spec['add_filters'];
        if(! empty($spec['onchange']))
        	$attrs['onchange'] = $spec['onchange'];
		$this->addControl('EmailRecipsInput', $sel_id, $attrs);

		return $ret;
	}

	function renderSearch(array $spec, $value='', $module=null) {
		$autosave = array_get_default($spec, 'autosave', '');
		//$mod = $this->list_query->getModuleDir();
		//$autosave = "com.1crm.search.$mod";
		$name = $this->getFieldNamePrefix() . $spec['name'];
		$base_id = array_get_default($spec, 'override_id', $name);
		$sel_id = $this->getFieldIdPrefix() . $base_id . '-input';
		$cls = $this->getFieldClass('input-field input-field-group input-search', $spec, true, true);
		if(strpos($cls, 'input-lg') !== false)
			$ext_class = ' input-lg';
		else
			$ext_class = '';
		$attrs = array('class' => $cls, 'id' => $sel_id);
		$fulltext_search = array_get_default($spec, 'fulltext_search', false);
		$w = array_get_default($spec, 'width', $fulltext_search ? 18 : 15);
		$style = 'width: '.$w.'em; ';
		if(isset($spec['style']))
			$style = $spec['style'];
		$attrs['style'] = $style;

		$ret = '';
		if($fulltext_search)
			$ret .= self::get_tag('input', array('type' => 'hidden', 'name' => $name.'_what', 'value' => $spec['search_what']));

		$ret .= self::get_tag('div', $attrs);
		if(! $fulltext_search) {
			$ret .= self::get_tag('div', ['class' => 'input-label input-field-icon' . ($ext_class ? '' : ' lg')]);
			$ret .= self::get_tag('div', array('class' => 'uii uii-lg uii-search'), true);
			$ret .= '</div>';
		}
        else {
            $select_spec = array('name' => $name.'_what', 'options' => 'search_what_dom', 'class' => 'border-right',
                'options_add_blank' => false, 'var_width' => true, 'no_hidden' => true, 'icon' => 'uii uii-lg uii-search');
            $ret .= $this->renderSelect($select_spec, $spec['search_what']);
        }

		$attrs = array('type' => 'text', 'class' => 'input-text input-entry '.$ext_class,
			'autocapitalize' => 'off', //'placeholder' => translate('LBL_SEARCH'),
			'results' => 9, 'incremental' => false, 'size' => $w, 'name' => $name,
			'id' => $base_id, 'value' => $value);
		foreach(array('autosave', 'placeholder', 'tabindex') as $f)
			if(isset($spec[$f]))
				$attrs[$f] = $spec[$f];
		$ret .= self::get_tag('input', $attrs);
        $clear_show = 'display: none';
		$ret .= self::get_tag('div', array('style' => $clear_show, 'class' => 'uii uii-clear search-clear'), true);
		$ret .= '</div>';
		$attrs = array(
			'name' => $name,
			'label' => $this->getLabel($spec),
		);
		if(! empty($spec['auto_submit']))
			$attrs['auto_submit'] = true;
		if(! empty($spec['quick_view'])) {
			$attrs['quick_view'] = true;
			if(! empty($module)) $attrs['module'] = $module;
		}
		$this->addControl('SearchInput', $sel_id, $attrs);
		return $ret;
	}

	function renderAddress(array $spec, RowResult $result) {
		global $locale, $themeInstance;
 		static $parts = array();
 		$format = $locale->getLocaleAddressFormatName();
 		if($format == 'Germany' || $format == 'Switzerland')
 			$parts = array('account', 'contact');
 		else
 			$parts = array('contact', 'account');

		if(isset($spec['source']['alias_fields']))
			$names = $spec['source']['alias_fields'];
		else {
			if(isset($spec['source']['fields']))
				$fields = $spec['source']['fields'];
			else
				$fields = [];
			$pfx = array_get_default($spec['source'], 'prefix');
			$names = [];
			foreach($fields as $f)
				$names[$f] = $pfx.$f;
		}

 		$parts[] = 'address_street';
 		if($format == 'France' || $format == 'Germany' || $format == 'Switzerland') {
 			$parts[] = array('address_postalcode', 'address_city');
			$parts[] = array('address_state', 'address_country');
 		} else {
 			$parts[] = array('address_city', 'address_state');
			$parts[] = array('address_country', 'address_postalcode');
		}
		if(isset($names['address_email']))
			$parts[] = 'address_email';
		if(isset($names['address_phone']))
			$parts[] = 'address_phone';
		$sel_id = $this->getFieldIdPrefix() . $spec['name'] . '-input';
		$lbls = array(
			'address_street' => 'LBL_ADDRESS_STREET_FIELD',
			'address_city' => 'LBL_ADDRESS_CITY_FIELD',
			'address_state' => 'LBL_ADDRESS_STATE_FIELD',
			'address_statecode' => 'LBL_ADDRESS_STATECODE_FIELD',
			'address_country' => 'LBL_ADDRESS_COUNTRY_FIELD',
			'address_countrycode' => 'LBL_ADDRESS_COUNTRYCODE_FIELD',
			'address_postalcode' => 'LBL_ADDRESS_POSTALCODE_FIELD',
			'address_email' => 'LBL_ADDRESS_EMAIL_FIELD',
			'address_phone' => 'LBL_ADDRESS_PHONE_FIELD',
		);
		$widths = array(
			'address_city' => 15,
			'address_state' => 12,
			'address_country' => 15,
			'address_postalcode' => ($format == 'France' || $format == 'Germany' || $format == 'Switzerland') ? 12 : 8,
			'address_email' => 20,
			'address_phone' => 15,
		);
		$ftypes = array(
			'address_email' => 'email',
			'address_phone' => 'phone',
		);

		$addr = array();
        $row = $result->row;
		foreach($names as $part => $field) {
			$p = array_get_default($row, $field);
			$addr[$part] = $p;
		}
		$ret = '<div id="'.$sel_id.'" class="address-edit">';
		$star = '<span class="requiredStar">*</span>';

		// FIXME - adapt to support alternate address formats
		foreach($parts as $n) {
			if(is_array($n) || isset($names[$n])) {
				if($n === 'account' || $n === 'contact') {
					$lbl = translate($n == 'account' ? 'LBL_ACCOUNT' : 'LBL_CONTACT', 'app');
					if(isset($result->fields[$names[$n]]))
						$ref_spec = $result->fields[$names[$n]];
					else
						$ref_spec = array(
							'type' => 'ref',
							'vname' => $n == 'account' ? 'LBL_ACCOUNT' : 'LBL_CONTACT',
							'name' => $names[$n],
							'id_name' => $names[$n].'_id',
							'bean_name' => ($n == 'account' ? 'Account' : 'Contact'),
						);
					$req = '<span class="requiredStar">*</span>';
					if(! empty($ref_spec['required']) && $themeInstance->layout_version < 8)
						$lbl .= $req;
					if($n == 'contact' && isset($names['account'])) {
						$ref_spec['add_filters'][] = array(
							'param' => 'primary_account',
							'field_name' => $names['account'],
						);
					}
					if($themeInstance->layout_version >= 8) {
						$vmod = array_get_default($ref_spec, 'vname_module', $result->getModuleDir());
						$lbl = translate($ref_spec['vname'], $vmod);
						$ref_spec['placeholder'] = '('.$lbl.')';
						$lbl = null;
					}
					$ret .= '<div class="row">';
					$ret .= '<div class="col-12 address-edit-cell form-entry">';
					if($lbl) $ret .= '<p class="form-label sub-label">'.$lbl.'</p>';
					$ret .= '<div class="form-value">';
					$ret .= $this->renderRef($result, $ref_spec);
					if(! empty($ref_spec['required']) && $themeInstance->layout_version >= 8)
						$ret .= $req;
                    $ret .= '</div></div></div>';
					$show_street_label = true;
					continue;
				} else if($n == 'address_street' || $n == 'address_email' || $n == 'address_phone') {
					$req = ! empty($result->fields[$names[$n]]['required']) ? $star : '';
					$ret .= '<div class="row">';
					$ret .= '<div class="col-12 address-edit-cell form-entry">';
					if($n != 'address_street' || ! empty($show_street_label)) {
						$lbl = to_html(translate($lbls[$n], 'app'));
						$ret .= '<p class="form-label sub-label">'.$lbl.$req.'</p>';
					}
					$ret .= '<div class="form-value">';
					if($n == 'address_street') {
						$textarea_spec = array('vname' => $lbls[$n], 'vname_module' => 'app', 'name' => $names[$n], 'height' => 2, 'width' => 50);
						if($req) $textarea_spec['required'] = true;
						$ret .= $this->renderTextArea($textarea_spec, $addr[$n]);
					}
					else {
						$etc_spec = array('vname' => $lbls[$n], 'name' => $names[$n]);
						if(isset($widths[$n])) $etc_spec['width'] = $widths[$n];
						if(isset($ftypes[$n])) $etc_spec['type'] = $ftypes[$n];
						if($req) $etc_spec['required'] = true;
						$ret .= $this->renderText($etc_spec, $addr[$n]);
					}
					$ret .= '</div></div></div>';
					continue;
				} else if(is_array($n)) {
					$ret .= '<div class="row">';
					foreach($n as $np) {
						$req = ! empty($result->fields[$names[$np]]['required']) ? $star : '';
						$ret .= '<div class="col address-edit-cell form-entry">';
						$ret .= '<p class="form-label sub-label">' . to_html(translate($lbls[$np], 'app')) . $req . '</p>';

						$ret .= '<div class="form-value">';
						$etc_spec = array('vname' => $lbls[$np], 'name' => $names[$np]);
						if(isset($widths[$np])) $etc_spec['width'] = $widths[$np];
						if(isset($ftypes[$np])) $etc_spec['type'] = $ftypes[$np];
						if(! empty($result->fields[$names[$np]]['required']))
							$etc_spec['required'] = true;
						if($np == 'address_country') {
							$etc_spec['search_dom'] = 'country_codes';
							$etc_spec['autocomplete'] = 'off';
							if(isset($names['address_countrycode'])) {
								$codef = $names['address_countrycode'];
								$codenm = $this->getFieldNamePrefix() . $codef;
								$codeid = $this->getFieldIdPrefix() . $codenm . '-input';
								$codeval = $result->getField($codef);
								$ret .= self::get_tag('input', [
									'type' => 'hidden',
									'id' => $codeid,
									'name' => $codenm,
									'value' => $codeval,
								], true);
								$this->addControl('HiddenInput', $codeid, ['name' => $codenm]);
								$etc_spec['key_name'] = $codenm;
							}
						}
						else if($np == 'address_state') {
							if(isset($names['address_statecode'])) {
								$codef = $names['address_statecode'];
								$codenm = $this->getFieldNamePrefix() . $codef;
								$codeid = $this->getFieldIdPrefix() . $codenm . '-input';
								$codeval = $result->getField($codef);
								$ret .= self::get_tag('input', [
									'type' => 'hidden',
									'id' => $codeid,
									'name' => $codenm,
									'value' => $codeval,
								], true);
								$this->addControl('HiddenInput', $codeid, ['name' => $codenm]);
								$etc_spec['key_name'] = $codenm;
							}
						}
						$ret .= $this->renderText($etc_spec, $addr[$np]);
						$ret .= '</div></div>';
					}
					$ret .= '</div>';
				}
			}
		}

		if(! empty($spec['copy_to'])) {
			$copy = $spec['copy_to'];
            $copy_rel_contacts = array_get_default($spec, 'copy_to_rel_contacts', false);
			if(! is_array($copy)) $copy = array($copy);
			$ret .= '<div class="row"><div class="col-12 address-edit-cell edit-menu">';
			$from = javascript_escape($spec['name']);
			$copy_id = $sel_id . '-copy';
			$link = '<div class="input-field flat" id="'.$copy_id.'">';
			$link .= '<span class="input-label"><span class="uii uii-next"></span> ' . translate('LBL_ADDRESS_COPY_LABEL', 'app') . '</span>';
            if ($copy_rel_contacts)
				$link .= '<span class="input-label" id="'.$sel_id.'-message" style="display: none"></span>';
			$link .= '</div>';
            $ret .= $link;
            $ret .= '</div></div>';
		}

		$ret .= '</div>';

		$attrs = array('name' => $spec['name'], 'parts' => $names, 'label' => $this->getLabel($spec));
		if(! empty($copy)) $attrs['copy_to'] = $copy;
        if (! empty($copy_rel_contacts)) $attrs['copy_to_rel_contacts'] = $copy_rel_contacts;
		$this->addControl('AddressInput', $sel_id, $attrs);

		return $ret;
	}


	function renderImage(array $spec, array $values) {
		$name = $this->getFieldNamePrefix() . $spec['name'];
		$sel_id = $this->getFieldIdPrefix() . $name . '-input';
		$disabled = array_get_default($spec, 'disabled');
		$v = array_get_default($values, 'value');
		$w = $h = 0;
		if(preg_match('~<img\b[^>]+>~', $v, $m)) {
			$v = $m[0];
			if(preg_match('~width="(\d+)" height="(\d+)"~', $m[0], $m)) {
				$w = $m[1];
				$h = $m[2];
			}
		} else
			$v = '';
		$dsize = array_get_default($spec, 'display_size', array(80, 80));
		if(AppConfig::is_mobile())
			$dsize = array(60, 60);
		$st = "width: {$dsize[0]}px; height: {$dsize[1]}px";
		$attrs = array('id' => $sel_id, 'class' => $this->getFieldClass('image-input', $spec, true, true));
		if(isset($spec['tabindex']))
			$attrs['tabindex'] = $spec['tabindex'];
		$ret = self::get_tag('div', $attrs);
		$ret .= self::get_tag('div', array('class' => 'image-inner', 'style' => $st));
		if($v && $h && $h < $dsize[1]) {
			$pad = floor(($dsize[1] - $h) / 2);
			$v = substr($v, 0, -1) . " style=\"margin-top: {$pad}px\">";
		}
		$ret .= $v;
		$ret .= '</div>';
		$attrs = array('type' => 'hidden', 'name' => $name, 'value' => '');
		$ret .= self::get_tag('input', $attrs, true);
		$attrs = array('type' => 'hidden', 'name' => $name.'_clear', 'value' => '');
		$ret .= self::get_tag('input', $attrs, true);

		$path = array_get_default($values, 'filename');
		if ($path && isset($spec['filename'])) {
			$attrs = array('type' => 'hidden', 'name' => $spec['filename'], 'value' => $path);
			$ret .= self::get_tag('input', $attrs, true);
		}

		$ret .= '</div>';

		$clear_path = '';
		if(isset($spec['placeholder_config']))
			$clear_path = AppConfig::setting($spec['placeholder_config'], '', true);
		else
			$clear_path = array_get_default($spec, 'placeholder');
		if(! $path)
			$path = $clear_path;
		else if(empty($spec['store_full_path'])) {
			require_once('include/upload_file.php');
			$upload = new UploadFile($spec);
			$path = $path ? $upload->get_file_path($path) : '';
		}
		$attrs = array('name' => $name, 'label' => $this->getLabel($spec), 'path' => $path, 'clear_path' => $clear_path, 'width' => $dsize[0], 'height' => $dsize[1]);
		if(isset($values['model'])) $attrs['model'] = $values['model'];
		if(isset($values['field'])) $attrs['field_name'] = $values['field'];
		if($disabled) $attrs['disabled'] = 1;
		$this->addControl('ImageInput', $sel_id, $attrs);

		return $ret;
	}


	function renderFileRef(array $spec, $filename, $mime_type='', $dupe=false) {
		$name = $this->getFieldNamePrefix() . $spec['name'];
		$sel_id = $this->getFieldIdPrefix() . $name . '-input';

		require_once('include/upload_file.php');
		$upload = new UploadFile($spec);
		$file_path = $filename ? $upload->get_file_path($filename) : '';
		$found = $file_path && file_exists($file_path);
		$attrs = array(
			'type' => 'file', 'name' => $name, 'id' => $sel_id,
			'size' => 40, 'class' => $this->getFieldClass('input-field input-file', $spec),
		);
		if(isset($spec['tabindex']))
			$attrs['tabindex'] = $spec['tabindex'];
		if($found && $dupe)
			$attrs['disabled'] = true;
		$ret = self::get_tag('input', $attrs, true);

		if($found) {
			$ret .= '<div style="margin: 0.2em 0" class="input-file-info">';
			if($dupe) {
				$check_spec = array('type' => 'bool', 'name' => $name.'_dupe', 'label' => translate('LBL_DUPE_FILE', 'app'), 'quick' => true, 'show_label' => true);
				$ret .= $this->renderCheck($check_spec, 1) . ' ';
			}
			else if(empty($spec['no_clear'])) {
				$check_spec = array('type' => 'bool', 'name' => $name.'_clear', 'label' => translate('LBL_REMOVE', 'app'), 'quick' => true, 'show_label' => true);
				$ret .= $this->renderCheck($check_spec, 0) . ' ';
			}
			$filename = preg_replace('~^[^/]*/~', '', $filename);
			$ret .= '<span class="input-field-pad input-file-curname">' . to_html($filename);
			$size = format_file_size(filesize($file_path));
			$ret .= " ($size";
			if($mime_type)
				$ret .= ', '.to_html($mime_type);
			$ret .= ')</span>';
			$ret .= '</div>';
		}

		$attrs = array('name' => $name, 'label' => $this->getLabel($spec));
		if(! empty($spec['required'])) $attrs['required'] = true;
		$this->addControl('FileInput', $sel_id, $attrs);
		return $ret;
	}

    function renderAlphabet(array $spec, $value) {
        $name = $this->getFieldNamePrefix() . $spec['name'];
        $sel_id = $this->getFieldIdPrefix() . $name . '-input';

        $ret = self::get_tag('input', array('type' => 'hidden', 'name' => $name, 'value' => $value), true);
        $ret .= '<span>';
        $letters = range('A', 'Z');
        $letters_num = sizeof($letters);
        $counter = 0;

        foreach ($letters as $letter)  {
            $ret .= self::get_tag('a', array('id' => $sel_id .'-'. $letter, 'href' => '#', 'class' => 'listViewTdToolsS1'));
            $ret .= $letter . '</a>';
            if ($counter + 1 != $letters_num)
                $ret .= '&nbsp;&middot;&nbsp;';

            $counter ++;
        }
        $ret .= "</span>";

        $auto_sub = empty($spec['auto_submit']) ? 'false' : 'true';
        $onchange = empty($spec['onchange']) ? 'null' : addcslashes($spec['onchange'], "'");
        $this->addControl('AlphabetInput', $sel_id, "{name: '$name', auto_submit: $auto_sub, onchange: '$onchange'}");

        return $ret;
    }

	function renderModuleName(array $spec, $value) {
		return $this->renderSelect($spec, $value);
	}


	static function render_button(array $spec) {
		if(isset($spec['icon'])) {
			$icon = $spec['icon'];
			unset($spec['icon']);
		} else
			$icon = '';
		if(strpos($icon, '-') !== false) {
			if(strpos($icon, 'uii') !== false)
				$icon = "<span class=\"uii $icon\"></span> ";
			else
				$icon = "<div class=\"input-icon left $icon\"></div>";
		}
		else if($icon) {
			$icon = get_image($icon, 'alt=""');
			$icon = str_replace('class="', 'class="left ', $icon);
		}

		if(isset($spec['label'])) {
			$label = $spec['label'];
			unset($spec['label']);
		} else
			$label = '';

		if(! isset($spec['class']))
			$spec['class'] = 'input-button';
		if(! empty($spec['show_arrow'])) {
			$spec['class'] .= ' rbullet';
			unset($spec['show_arrow']);
		}
		if(! empty($spec['flat'])) {
			$spec['class'] .= ' flat';
			unset($spec['flat']);
		}
		if(! isset($spec['type']))
			$spec['type'] = 'button';
		if(empty($spec['html_label']))
			$label = to_html($label);
		else
			unset($spec['html_label']);

		if(strlen($label))
			$label = "<span class=\"input-label\" id=\"{$spec['id']}-label\">$label</span>";
		$label = $icon . $label;

		$ret = self::get_tag('button', $spec) . $label . '</button>';
		return $ret;
	}


	function renderButton(array $spec) {
		$spec['name'] = $this->getFieldNamePrefix() . $spec['name'];
		if(! isset($spec['id']))
			$spec['id'] = $this->getFieldIdPrefix() . $spec['name'] . '-input';
		if(! isset($spec['onclick'])) {
			$onclick = array_get_default($spec, 'perform');
			unset($spec['perform']);
		} else
			$onclick = $spec['onclick'];
		unset($spec['onclick']);

		$ret = self::render_button($spec);
		$attribs = array('name' => $spec['name']);
		if(isset($onclick))
			$attribs['onclick'] = $onclick;
		$this->addControl('ButtonInput', $spec['id'], $attribs);
		return $ret;
	}

	function renderFormButton(FormButton $button, $in_popup = false, $disabled = false) {
		global $themeInstance, $pageInstance;

		if(isset($button->static_html)) {
			return $button->static_html;
		}

		$cls = 'input-button';
		if($button->class) {
			if(strpos($button->class, 'input-button') !== false)
				$cls = $button->class;
			else
				$cls .= ' '.$button->class;
		}

		if($button->type == 'select') {
			$attribs = array(
				'id' => $button->id ? $button->id : $this->form_id . '_' . $button->name,
				'name' => $button->field_name ? $button->field_name : $this->form_name . '_' . $button->name,
				'class' => $cls,
				'style' => $button->style,
				'tabindex' => $button->tabindex,
				'options' => $button->getOptions($this->form_name),
				'label_key' => $button->label_key,
				'align' => $button->align,
				'onchange' => $button->onchange,
				'icon' => $button->getIcon(),
				'var_width' => $button->var_width,
			);
			return $this->renderSelect($attribs, $button->value);
		}

		if($button->type == 'group') {
			$attribs = array(
				'id' => $button->id ? $button->id : $this->form_id . '_' . $button->name,
				'name' => $button->field_name ? $button->field_name : $this->form_name . '_' . $button->name,
				'class' => $cls . ' menu-source rbullet-ddown',
				'style' => $button->style,
				'tabindex' => $button->tabindex,
				'icon' => $button->getIcon(),
				'type' => 'button',
				'label' => $button->getLabel(),
				'show_arrow' => true,
			);
			if($button->html_label)
				$attribs['html_label'] = $button->html_label;

			$opts = $button->getOptions($this->form_name, $in_popup, true);
			if($opts) {
				$sel_opts = array('maxlen' => 0);
				self::process_opts($opts, $sel_opts, 'label');
				$sel_opts['width'] = round(max($sel_opts['maxlen'], 15) * 0.7) +2.5 .'em';

				if($button->promote == 'tools' && ! $themeInstance->classic_title) {
					$pageInstance->set_tools_options($sel_opts, $this->getFormId());
					return '';
				}

				$json = getJSONobj();
				$js_opts = $json->encode($sel_opts);
				$this->addInit("SUGAR.ui.registerSelectOptions('{$attribs['id']}-Menu', $js_opts);");
				$this->addInit("SUGAR.ui.registerMenuSource('{$attribs['id']}', {options: '{$attribs['id']}-Menu', icon_key: 'icon'}, '$this->form_name');");
			}

			$ret = self::render_button($attribs);
			return $ret;
		}

		$name = $button->name;
		$attribs = array(
			'id' => $button->id ? $button->id : $this->form_id . '_' . $name,
			'name' => $this->form_name . '_' . $name,
			'type' => $button->type,
			'class' => $cls,
			'style' => $button->style,
			'tabindex' => $button->tabindex,
		);
		if(isset($button->align))
			$attribs['style'] .= '; float: '.$button->align;
		foreach($button->getHooks($this->form_name, $in_popup) as $k => $v)
			$attribs[$k] = $v;
		$attribs['icon'] = $button->getIcon();
		$attribs['label'] = $button->getLabel();
		if($button->html_label)
			$attribs['html_label'] = $button->html_label;
		if ($disabled) {
			$attribs['disabled'] = 'disabled';
		}
		return self::render_button($attribs);
	}


	function renderTabGroup(array $spec, $active) {
		$name = $this->getFieldNamePrefix() . $spec['name'];
		$sel_id = $this->getFieldIdPrefix() . $name.'-input';
		$ret = self::get_tag('input', array('type' => 'hidden', 'name' => $name, 'value' => $active));
		$attribs = array(
			'id' => $sel_id,
			'class' => $this->getFieldClass('input-tabgroup nowrap', $spec, false),
		);
		$ret .= self::get_tag('div', $attribs);
		$ret .= self::get_tag('div', array('id' => $sel_id . '-inner', 'class' => 'input-tabgroup-inner'));

		$tabs_count = 0;
		$tabs_tot = is_countable($spec['tabs']) ? count($spec['tabs']) : 0;
		foreach($spec['tabs'] as $idx => $t) {
			if(empty($t['name']) || ! empty($t['hidden']) || ! empty($t['separator'])) continue;
			$outer_id = $sel_id.'-tabouter-'.$t['name'];
			$tab_id = $sel_id.'-tab-'.$t['name'];
			$icon = array_get_default($t, 'icon', '');
			$bclass = array_get_default($t, 'class', '');
			$label = array_get_default($t, 'label');
			if(! empty($t['hide_label'])) $label = null;
			if($icon) {
				if(! empty($label))
					$icon = 'left '.$icon;
				else
					$bclass = ' no-label '.$bclass;
			}
			$tclass = ($t['name'] == $active) ? 'current' : '';
			$prev = array_get_default($spec['tabs'], $idx - 1);
			$next = array_get_default($spec['tabs'], $idx + 1);
			if($prev && ! empty($prev['separator'])) $tclass .= ' postsep';
			if($next && ! empty($next['separator'])) $tclass .= ' presep';
			$zi = $tabs_tot - $tabs_count;
			$ret .= self::get_tag('div', array('id' => $outer_id, 'class' => 'tab-outer normal '.$tclass, 'style' => "z-index: $zi"));
			$ret .= self::get_tag('div', array('id' => $tab_id, 'class' => 'tab-button '.$bclass, 'tabindex' => '0'));
			$ret .= self::get_tag('div', array('class' => 'input-icon tab-icon '.$icon, 'style' => ($icon ? '' : 'display: none')), true);
			$ret .= self::get_tag('span', array('class' => 'input-label tab-label'));
			if(! empty($label))
				$ret .= to_html($label);
			$ret .= '</span></div>';
			$ret .= '</div>';
			$tabs_count++;
		}

		$ret .= '</div></div>';

		$attribs = array('name' => $name, 'tabs' => $spec['tabs'], 'current_tab' => $active);
		if(isset($spec['manage_sections']))
			$attribs['manage_sections'] = $spec['manage_sections'];
		if(isset($spec['onchange']))
			$attribs['onchange'] = $spec['onchange'];
		$this->addControl('TabGroup', $sel_id, $attribs);
		return $ret;
	}


	function renderColor(array $spec, $value='') {
		$name = $this->getFieldNamePrefix() . $spec['name'];
		$inp_id = $this->getFieldIdPrefix() . $name;
		$sel_id = $inp_id . '-input';
		$cls = $this->getFieldClass('input-field input-field-group input-color', $spec, true, true);
		$attrs = array('class' => $cls, 'id' => $sel_id);

		$ret = self::get_tag('div', $attrs);
		$ret .= self::get_tag('div', ['class' => 'input-label input-field-icon']);
		$ret .= self::get_tag('div', ['class' => 'input-icon input-color-icon'], true);
		$ret .= '</div>';

		$attrs = ['type' => 'text', 'class' => 'input-text input-entry',
			'size' => 10, // 'placeholder' => translate('LBL_COLOR_PLACEHOLDER'),
			'name' => $name, 'id' => $inp_id, 'value' => $value];
		foreach(array('autosave', 'placeholder', 'tabindex') as $f)
			if(isset($spec[$f]))
				$attrs[$f] = $spec[$f];
		$ret .= self::get_tag('input', $attrs, true);
		$ret .= '</div>';
		$attrs = array(
			'name' => $name,
			'label' => $this->getLabel($spec),
		);
		if(isset($spec['allow_alpha']))
			$attrs['allow_alpha'] = $spec['allow_alpha'];
		$this->addScriptInclude('include/javascript/color.js');
		$this->addControl('ColorTextInput', $sel_id, $attrs);
		return $ret;
	}


	static function get_tag($name, $attribs, $close=false, $no_enc=false) {
		$ret = "<$name";
		foreach($attribs as $k => $v) {
			if(is_array($v) || is_object($v)) continue;
			if(! $no_enc) $v = to_html($v);
			$ret .= " $k=\"$v\"";
		}
		if($close)
			return "$ret></$name>";
		return "$ret>";
	}

    static function format_duration($value) {
        global $app_strings;

		if(! strlen($value)) return '';
        if (! $value) {
            $hours = '0';
            $minutes = '00';
        } else {
            $hours = ($value < 60) ? 0 : floor($value / 60);
            $minutes = ($value < 60) ? $value : ($value - ($hours * 60));
            if ($minutes < 10) $minutes = '0' . $minutes;
        }

        $hours_abbrev = $app_strings['LBL_HOURS_ABBREV'];
        $min_abbrev = $app_strings['LBL_MINS_ABBREV'];
        $duration = $hours . $hours_abbrev ." ". $minutes . $min_abbrev;

        return $duration;
    }

    static function combine_hooks($hooks) {
    	if(! is_array($hooks))
    		return $hooks;
    	$ret = array();
    	foreach($hooks as $h) {
    		if(! strlen($h)) continue;
    		$h = preg_replace('~;\s*$~', '', $h);
    		if(preg_match('~^\w+$~', $h)) $h = "SUGAR.ui.callEvent(this, $h, this);";
    		$ret[] = $h;
    	}
    	return implode(';', $ret);
    }

    function getLabel($spec) {
    	if(isset($spec['label']))
    		return $spec['label'];
    	else if(isset($spec['vname'])) {
    		$vmod = array_get_default($spec, 'vname_module');
    		return translate($spec['vname'], $vmod);
    	}
    	return null;
    }
}
?>
