<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/EditableForm.php');
require_once('include/layout/forms/FilterElement.php');

class FilterForm  extends EditableForm {
	var $base_model;
	var $basic_options;
	var $auto_options;
	var $filter_options;
	var $filter_source;
	var $session_filter;
	var $filter;
	var $layouts;
	var $layout_name;
    var $display_layout_name;
	var $default_layout;
	var $default_filter;
	var $blank_filter;
	var $add_hidden_fields;
	var $show_advanced;
	var $show_labels;
    var $show_clear = true;
    var $show_text_filter = true;
	var $apply_acl = true;
    var $edit_layout = true;
	var $context;
	var $title, $title_desc;
	var $icon;
	var $is_custom;
	var $is_report, $is_report_data, $is_report_owner, $report_run_method;
	var $unified_search_match;
    var $search_what;
    var $max_ref_width = 15;
    var $max_text_width = 22;
    var $widgets = array();
    var $add_field_classes = 'filter';
	var $config_filter_options;
	var $override_filters;
	var $override_user_id;

	static $date_filter_ops = array(
		'',
		'before_date',
		'after_date',
		'before_today',
		'after_today',
		'on_date',
		'between_dates',
		'not_on_date',
		'empty',
		'not_empty',
		'yesterday',
		'today',
		'tomorrow',
		'period_current',
			// week, month, quarter, year, fiscal quarter, fiscal year
		'period_next',
			// week, month, quarter, year, fiscal quarter, fiscal year, 7 days, 30 days
		'period_prev',
			// week, month, quarter, year, fiscal quarter, fiscal year, 7 days, 30 days
		/*'month',
		'year',
		'fiscal_year',*/
		//'range',
			// [n] [days/weeks/months/quarters/fiscal quarters/years/fiscal years]
			// starting n [before/after] today
	);
	static $date_filter_date_ops = array(
		'before_date', 'after_date', 'before_today', 'after_today', 'on_date', 'not_on_date', 'between_dates',
	);
	static $date_filter_periods = array(
		'week', 'month', 'quarter', 'year', 'fiscal_quarter', 'fiscal_year', 'days_7', 'days_14', 'days_30', 'days_60', 'days_90', 'days_180', 'days_365',
	);

	static $number_filter_ops = array(
		'',
		'gt',
		'lt',
		'eq',
		'not_eq',
		'gte',
		'lte',
		'between',
		'empty',
		'not_empty',
	);

	static $text_filter_ops = array(
		'',
		'like',
		'eq',
		'prefix',
		'suffix',
		'not_like',
		'not_eq',
		'empty',
		'not_empty',
	);

    static $select_filter_ops = array(
        '',
        'eq',
        'not_eq',
        'any_of',
        'not_any_of',
        'empty',
        'not_empty',
    );

    const DEFAULT_OWNER_FIELD = 'assigned_user';
    const DEFAULT_SELECT_FILTER_OPERATOR = 'eq';

	function __construct($listQuery, $form_name = null, $form_id = null, $prefix = '', $context = 'listview', $show_advanced=null) {
		parent::__construct('searchform', $form_name, $form_id, $prefix);
		$this->list_query =& $listQuery;
		$this->base_model = $listQuery->getBaseModel();
		$this->basic_options = array();
		$this->auto_options = array();
		$this->filter_options = array();
		$this->config_filter_options = array();
		$this->filter_source = '';
		$this->filter = array();
		$this->override_filters = array();
		$this->add_hidden_fields = array();
        $this->context = $context;
		$this->show_advanced = $show_advanced;
        $this->loadLayoutIndex();
		$this->loadBasicFilterOptions();
		$this->loadAutoFilterOptions();
		//$this->getAvailableFilters();
		if($context == 'workflow') {
			$this->max_ref_width = 20;
			$this->max_text_width = 25;
		}
	}

	function setDefaultLayout($l) {
		$this->default_layout = $l;
	}

	function getDefaultFilter() {
		return isset($this->default_filter) ? $this->default_filter : array();
	}

	function setDefaultFilter($f) {
		$this->default_filter = $f;
	}

	function setTitle($t, $desc=null) {
		$this->title = $t;
		$this->title_desc = $desc;
	}

	function getTitle() {
		return $this->title;
	}

	function getTitleDesc() {
		return $this->title_desc;
	}

	function setIcon($t) {
		$this->icon = $t;
	}

	function getIcon() {
		return $this->icon;
	}

	function setIsReport($is_report, $run_method, $is_report_data=false, $is_report_owner=false) {
		$this->is_report = $is_report;
		$this->is_report_data = $is_report_data;
		$this->is_report_owner = $is_report_owner;
		$this->report_run_method = $run_method;
	}

    function setEditStatus($assigned_user_id, $created_by_id) {
        if (! AppConfig::is_admin()) {
            $current_user_id = AppConfig::current_user_id();
            if ($created_by_id != $current_user_id && $assigned_user_id != $current_user_id) {
                $this->edit_layout = false;
            }
        }
    }

	function loadLayoutIndex() {
		$mod = $this->list_query->getModuleDir();
		$this->layouts = self::load_layout_index($mod);
	}

	static function load_layout_index($module) {
		$layouts = array();
		if (!AppConfig::setting("modinfo.by_name.$module.detail.disable_browse_all"))
			$layouts['Browse'] = array('name' => 'Browse', 'title' => 'LBL_LAYOUT_BROWSE_ALL', 'priority' => -10);
		if (!AppConfig::is_mobile()) {
			$prios = array_column($layouts, 'priority');
			$details = AppConfig::setting("views.detail.{$module}.search");
			if(is_array($details)) {
				foreach($details as $name => $d) {
					if($name == 'Standard') {
						if(! isset($d['priority']))
							$d['priority'] = 0;
						if(! isset($d['title']))
							$d['title'] = 'LBL_LAYOUT_QUICK_FILTER';
					}
					else if(! isset($d['priority'])) {
						$d['priority'] = 10;
					}
					$prios[] = $d['priority'];
					$layouts[$name] = $d;
				}
				$names = array_keys($layouts);
				array_multisort($prios, SORT_ASC, $names, SORT_ASC, $layouts);
			}
		}
		return $layouts;
	}

    function loadOwnerFilterOptions($link_name = null) {
		$spec = $this->getOwnerFilterSpec($link_name);
        if (sizeof($spec) > 0) {
            $spec['basic'] = true;
            $link = '';
            if (! empty($link_name))
                $link = $link_name.'.';
            if($this->context == 'popup')
            	$spec['var_width'] = true;
            $this->basic_options[$link.'filter_owner'] = $spec;
        }
    }

	function getOwnerFilterSpec($link_name = null, $owner_field = '') {
		if(! empty($link_name)) {
			$lnk = $this->list_query->addLink($link_name);
			if(! $lnk)
				return;
			$model = $this->list_query->getModelName($lnk);
			$module = $this->list_query->getModuleDir($lnk);
			$link = $link_name.'.';
		}
		else {
			$model = $this->list_query->getModelName();
			$module = $this->list_query->getModuleDir();
			$link = '';
		}
		if(isset($this->override_user_id))
			$uid = $this->override_user_id;
		else
			$uid = AppConfig::current_user_id();
		$level = ACLAction::getUserAccessLevel($uid, $module, 'list');

        if ($owner_field != '' && $owner_field != 'filter_owner') {
            $field = $owner_field;
        } else {
            $field = self::DEFAULT_OWNER_FIELD;
        }
		$auser = $this->list_query->getFieldDefinition($link.$field, $err);
		$opts_lables = array_get_default($auser, 'labels', array());
		$have_teams = in_array($model, AppConfig::setting('model.index.securitygroups'));

        $owner_opts = array(
			'all' => array(
				'vname' => ($auser || ! $have_teams) ? 'LBL_FILTER_OWNER_ANY' : 'LBL_FILTER_OWNER_ANY_TEAM',
            	'icon' => ($auser || ! $have_teams) ? 'icon-users' : 'icon-teams',
            ),
		);

        if($auser) {
			$owner_opts['mine'] = array(
				'field' => $auser['id_name'],
				'link' => $link,
				'vname' => array_get_default($opts_lables, 'mine', 'LBL_FILTER_OWNER_MINE'),
				'value' => $uid,
				'icon' => AppConfig::is_admin() ? 'icon-adminuser' : 'icon-user',
			);
			if($level == ACL_ALLOW_OWNER && ! AppConfig::is_admin()) {
				unset($owner_opts['all']);
			} else {
				$owner_opts['not_mine'] = array(
					'field' => $auser['id_name'],
					'link' => $link,
					'vname' => array_get_default($opts_lables, 'not_mine', 'LBL_FILTER_OWNER_NOT_MINE'),
					'operator' => '!=',
					'value' => $uid,
					'icon' => 'icon-users',
				);
				$owner_opts['select'] = array(
					'field' => $auser['id_name'],
					'link' => $link,
					'vname' => array_get_default($opts_lables, 'select', 'LBL_FILTER_OWNER_SELECT'),
					'icon' => 'icon-popup',
				);
			}
		}

		$choose_team = ! ACLController::requireOwner($module, 'view', $uid);
		$my_teams_only = ACLController::requireSecurityGroup($module, 'view', $uid);
		if($have_teams && $choose_team) {
			$myGroups = AppConfig::setting("acl.user_teams.$uid", array());
			if ($myGroups) {
				if(! $my_teams_only)
					$owner_opts['my_team'] = array(
						'vname' => array_get_default($opts_lables, 'my_team', 'LBL_FILTER_OWNER_MY_TEAM'),
						'operator' => 'team',
						'value' => array_keys($myGroups),
						'icon' => 'icon-teams',
					);
				if(! $my_teams_only || count($myGroups) > 1) {
					foreach ($myGroups as $grp) {
						$owner_opts['team_' . $grp['id']] = array(
							'vname' => $grp['name'],
							'operator' => 'team',
							'value' => $grp['id'] ,
							'icon' => 'icon-teams',
						);
					}
				}
			}
			if(! $my_teams_only) {
				$teams = AppConfig::setting("acl.all_teams", array());
				foreach($teams as $grp) {
					if(! isset($owner_opts['team_' . $grp['id']])) {
						$owner_opts['team_' . $grp['id']] = array(
							'vname' => $grp['name'],
							'operator' => 'team',
							'value' => $grp['id'] ,
							'icon' => 'icon-teams',
						);
					}
				}
			}
		}

		if(($this->context == 'dashlet' || $level == ACL_ALLOW_OWNER) && isset($owner_opts['mine']))
			$default_owner = 'mine';
		else
			$default_owner = 'all';

		$label = 'LBL_FILTER_OWNER';
		if ($field != self::DEFAULT_OWNER_FIELD && isset($auser['vname']))
			$label = $auser['vname'];

		$spec = array(
			'vname' => $label,
			'type' => 'owner',
			'default_value' => $default_owner,
			'options' => $owner_opts,
			'options_icon' => 'icon',
			'options_blank_key' => 'all',
			'input_class' => 'OwnerSelect',
			'report_filter' => true,
			'var_width' => true,
		);
		if(count($owner_opts) <= 1)
			$spec['hidden'] = true;

		$override = AppConfig::setting("display.filters.$model.filter_owner");
		if($override)
			array_update_recursive($spec, $override);
        return $spec;
    }

	function addOwnerFilterOption($value, $field='filter_owner') {
		if(isset($this->filter_options[$field])) {
			$old_opts = $this->filter_options[$field]['options'];
			if(isset($old_opts[$value]))
				return true;
			if(! isset($old_opts['select']))
				return false;
			$names = array();
			if (strpos($value, '#') !== false) {
				$ids = array_filter(explode('#', $value));
				$count = 0;
				foreach ($ids as $uid) {
					$user_name = get_assigned_user_name($uid);
					if ($user_name) {
						$names[] = $user_name;
						$count++;
					}
				}
				$user_name = translate('LBL_OWNER_MULTIPLE_USERS') . ' (' . $count . ')';
			} else {
				$user_name = get_assigned_user_name($value);
			}
			if(! $user_name)
				return false;
			$opts = array();
			foreach($old_opts as $k => $v) {
				if($k == 'select') {
					$v2 = $v;
					$v2['label'] = $user_name;
					$v2['title'] = empty($names) ? $user_name : join(', ', $names);
					$v2['value'] = empty($names) ? $value : $ids;
					$v2['icon'] = 'icon-user';
					$opts[$value] = $v2;
				}
				$opts[$k] = $v;
			}
			$this->filter_options[$field]['options'] = $opts;
			return true;
		}
	}

	function loadFavoritesFilterOption() {
        if (AppConfig::setting("display.list.{$this->list_query->getModelName()}.show_favorites")) {
            $this->basic_options['filter_favorites'] = array(
                'name' => 'filter_favorites',
                'vname' => 'LBL_FILTER_FAVORITES',
                'type' => 'flag',
                'class' => 'check-star',
                'basic' => true,
                'field' => 'favorites.deleted',
                'default_value' => false,
                'operator' => 'eq',
                'value' => '0'
            );
            if($this->context == 'popup')
            	$this->basic_options['filter_favorites']['show_label'] = 0;
        }
    }

	/** Load filter info. from user input (ListLayoutEditor) */
	function loadUserFilterOptions($input) {
		// FIXME add more sanitizing of inputs
		$fs =& $this->filter_options;
		if($this->is_report && $this->is_report_data != 'archived')
			$fs = array();
		else
			$fs = $this->basic_options;
		$this->loadFilterDefs($this->filter_options, $input, true);
	}


	function getFilterDefinition($spec) {
		if(! is_array($spec)) {
			if( ($p = strrpos($spec, '.')) !== false) {
				$lnk = substr($spec, 0, $p);
				$spec = array('name' => substr($spec, $p+1), 'link' => $lnk);
			} else
				$spec = array('name' => $spec);
		}
		$name = array_get_default($spec, 'name');
		$field = array_get_default($spec, 'field');
		if(! $name) return false;
		$link_name = array_get_default($spec, 'link', '');
		if($field && ($pp = strrpos($field, '.')) !== false) {
			$pfx = substr($field, 0, $pp);
			if($link_name) $link_name .= '.'.$pfx;
			else $link_name = $pfx;
			$field = substr($field, $pp+1);
		}
		if(! $link_name) {
			// basic filters first
			if(isset($this->basic_options[$name])) {
				$opt = $this->basic_options[$name];
				$opt['basic'] = true;
				return $opt;
			}
		}
		// look up model-defined filters
		if($this->list_query) {
            if ($this->display_layout_name) {
                $model = $this->display_layout_name;
            } elseif ($link_name) {
				$lnk = $this->list_query->addLink($link_name);
				if(! $lnk)
					return;
				$model = $this->list_query->getModelName($lnk);
			} else {
				$model = $this->base_model->getModelName();
			}
			$def = AppConfig::setting("display.filters.$model.$name");
			if(! $def && $field)
				$def = AppConfig::setting("display.filters.$model.$field");
			if($def) {
				// take display filter definition and merge field definition
				$type = array_get_default($def, 'type');
				if(empty($def['no_inherit']) && empty($def['fields']) && $type != 'flag' && $type != 'section') {
					$field2 = array_get_default($def, 'field', $name);
					if($field2 && $def2 = $this->getFieldDefinition($field2, $link_name)) {
						unset($def2['name']);
						if(isset($def['options_function']))
							unset($def2['options']);
						if(isset($def['vname']))
							unset($def2['vname_module']);
						$def = array_merge($def2, $def);
					}
				}
				if($link_name && ! isset($def['vname_module'])) {
					$def['vname_module'] = AppConfig::module_for_model($model);
				}
				if($field) $def['field'] = $field;
				if($link_name) $def['link'] = $link_name;
				return $def;
			}
			else if($field) {
				$def = $this->getFieldDefinition($field, $link_name);
				if($def) {
					unset($def['name']);
					if($field) $def['field'] = $field;
					if($link_name) $def['link'] = $link_name;
					return $def;
				}
			}
		}
		// look up query columns / model fields
		$def = $this->getFieldDefinition($name, $link_name);
		if($def) {
			unset($def['name']);
			if($field) $def['field'] = $field;
			if($link_name) $def['link'] = $link_name;
		}
		return $def;
	}

	function _stripFieldDefinition(array &$def) {
		$def['editable'] = true;
		$def['updateable'] = true;
		unset($def['required']);
		unset($def['default']);
		unset($def['alias']);
		unset($def['detail_link']);
	}

	function getFieldDefinition($name, $link_name=null) {
		if($this->list_query) {
			$field = ($link_name ? $link_name . '.' : '').$name;
			if( ($p = strrpos($field, '.')) !== false) {
				$link_name = substr($field, 0, $p);
				$name = substr($field, $p+1);
			}
			$def = $this->list_query->getFieldDefinition($field, $err);
			if($def) {
				if(! isset($def['field'])) $def['field'] = $name;
				if($link_name)
					$def['link'] = $link_name;
				else
					unset($def['link']);
				$this->_stripFieldDefinition($def);
				return $def;
			}
		}
		return false;
	}

	function getBasicFilterOptions() {
		return $this->basic_options;
	}

	function getFilterOptions() {
		return $this->filter_options;
	}

	function getFilterValues() {
		return $this->filter;
	}


	function loadFilterDefs(&$target, $defs, $basic=false, $assume_value=false) {
		$ret = 0;
		$extra = array();

		if(is_array($defs)) {
			foreach($defs as $k => $filt) {
				if(! is_array($filt)) {
					if($assume_value && ! is_int($k))
						$filt = array('value' => $filt);
					else
						$filt = array('name' => $filt);
				} else {
					unset($filt['source']);
					// fix redundant link (from old bug)
					$fn = array_get_default($filt, 'field');
					$lnk = array_get_default($filt, 'link');
					if($lnk && ($p = strrpos($fn, '.')) !== false) {
						if(substr($fn, 0, $p) == $lnk)
							$filt['field'] = substr($fn, $p+1);
					}
				}
				if(! empty($filt['ignore']))
					continue;

				$type = array_get_default($filt, 'type');
				if(! $type && isset($filt['multiple']) && is_array($filt['multiple']))
					$type = 'filtergroup';
				$filt['type'] = $type;
				if($type == 'hidden')
					$filt['hidden'] = true;

				if(! isset($filt['name'])) {
					if(is_int($k)) {
						if(isset($filt['field']))
							$k = $filt['field'];
						else if($type == 'filtergroup')
							$k = 'group-' . (is_countable($target) ? count($target) : 0);
						else
							continue;
					}
					$filt['name'] = $k;
				}
				else
					$k = $filt['name'];

				if(isset($filt['unique']))
					$k .= $filt['unique'];

				if(isset($target[$k])) {
					$this->_updateFilter($target[$k], $filt);
					continue;
				}

				if($type == 'filtergroup') {
					$multi = array_get_default($filt, 'multiple', array());
					$filt['multiple'] = array();
					$ret += $this->loadFilterDefs($filt['multiple'], $multi);
				} else {
					$def = $this->getFilterDefinition($filt);
					if($def) {
						unset($def['source']);
                        if (isset($def['type']) && ($def['type'] == 'owner' && $k != 'filter_owner'))
							$def = $this->getOwnerFilterSpec(null, $k);
                        $this->_updateFilter($def, $filt);
						$filt = $def;
                        $type = $filt['type'];
                    }
					else if(! isset($filt['fields'])) {
						if(AppConfig::setting('site.allow_debug'))
							pr2($filt, 'unknown filter');
						continue;
					}
				}

				if($type == 'ref') {
					$filt['allow_custom'] = true;
					$dynm = array_get_default($filt, 'dynamic_module');
					if($dynm && ! isset($target[$dynm])) $extra[] = array('name' => $dynm, 'hidden' => true, 'filter_internal' => true, 'unique' => array_get_default($filt, 'unique', ''));
				}

				if($basic) $filt['basic'] = true;

				$target[$k] = $filt;
				$ret ++;
			}
		}

		if($extra) {
			$ret += $this->loadFilterDefs($target, $extra);
		}
		return $ret;
	}

	function _updateFilter(&$target, $spec, $override=false) {
		// override allowed properties
		foreach(array('label', 'vname', 'vname_module', 'hidden', 'disabled',
			'value', 'default_value', '_iprefix', 'unique', 'basic_type',
			'filter_internal', 'add_filters', 'operator', 'expanded', 'when') as $f)
			if(isset($spec[$f])) $target[$f] = $spec[$f];
	}


	function loadFilterTextOption($link_name=null) {
		if($this->list_query->getUnifiedSearchFields($link_name)) {
			$model = $this->list_query->getModelName();
			$quick_view = false;
			if($model && ! AppConfig::setting("display.list.{$model}.view_in_popup") && $this->form_name != 'LayoutEdit')
				$quick_view = true;
			if($quick_view && ! AppConfig::setting("site.features.filter_quick_view", true))
				$quick_view = false;
			$opt = array(
				'name' => 'filter_text', 'vname' => 'LBL_FILTER_TEXT',
				'type' => 'unified_search', 'basic' => true,
                'fulltext_search' => $this->list_query->fulltext_search,
                'search_what' => 'all',
				'report_filter' => true,
				'quick_view' => $quick_view,
			);
			if($this->context == 'sidebar') {
				if($opt['fulltext_search'])
					$opt['width'] = 16; // make room for clear button
			}
			else if($this->context == 'popup') {
				$opt['width'] = 14;
			}
			$this->basic_options['filter_text'] = $opt;
			return true;
		}
	}

	function loadBasicFilterOptions() {
		if($this->context == 'workflow') return;
		$this->basic_options = array();
		$this->loadFilterTextOption();
		$this->loadOwnerFilterOptions();
        $this->loadFavoritesFilterOption();
		if(! $this->base_model) return;
        if ($this->display_layout_name) {
            $model = $this->display_layout_name;
        } else {
            $model = $this->base_model->name;
		}
		$basic = null;
		if ($this->context == 'popup')
			$basic = AppConfig::setting("display.popup_filters.$model");
		if(!isset($basic))
			$basic = AppConfig::setting("display.basic_filters.$model");
		if(isset($basic))
			$this->loadFilterDefs($this->basic_options, $basic, true);
	}

	function addBasicFilters() {
		foreach($this->basic_options as $k => $v) {
			if($this->is_report && empty($v['report_filter']))
				continue;
			$this->filter_options[$k] = $v;
		}
	}

	function loadAutoFilterOptions() {
		$this->auto_options = array();
		if(! $this->base_model) return;
        if ($this->display_layout_name) {
            $model = $this->display_layout_name;
        } else {
            $model = $this->base_model->name;
        }
		$auto = AppConfig::setting("display.auto_filters.$model");
		if(isset($auto))
			$this->loadFilterDefs($this->auto_options, $auto);
	}

	function addAutoFilters() {
		foreach($this->auto_options as $k => $v) {
			if(! isset($this->filter_options[$k])) {
				$v['hidden'] = true;
				$this->filter_options[$k] = $v;
			}
		}
	}

	function loadLayoutSpec($name) {
		$mod = $this->list_query->getModuleDir();
		if(! isset($this->layouts[$name]))
			return false;
		$spec = $this->layouts[$name];
		if(empty($spec['type']))
			return false;
		$cfg = AppConfig::setting("views.layout.{$mod}.{$spec['type']}.$name");
		return $cfg;
	}


	function loadFilterLayout($layout) {
		$fs =& $this->filter_options;
		$fs = array();
		if($this->is_report && $this->is_report_data)
			return true;
		$this->addBasicFilters();

		if(is_array($layout)) {
			$this->is_custom = true;
			$this->loadFilterDefs($fs, $layout);
		}
		else {
			$this->layout_name = 'Browse';

			$cfg = $this->loadLayoutSpec($layout);
			if($cfg) {
				$this->layout_name = $layout;
				// need to check [$model_name] not ['primary'] if not using primary bean
				if(isset($cfg['elements'])) {
					$this->loadFilterDefs($fs, $cfg['elements']);
				}
			}
		}

		if(! $this->is_report && ! $this->is_custom)
			$this->addAutoFilters();

		return true;
	}


	function addOverrideFilters(array $fs) {
		$this->loadFilterDefs($this->override_filters, $fs, false, true);
	}

	function applyOverrideFilters(&$opts) {
		foreach($this->override_filters as $k => $v) {
			if(isset($opts[$k])) {
				$this->_updateFilter($opts[$k], $v, true);
			} else {
				$v['hidden'] = true;
				$opts[$k] = $v;
			}
			$opts[$k]['override'] = true;
		}
	}


	function applyAclFilter($user_id=null) {
		if(! isset($user_id)) $user_id = AppConfig::current_user_id();
		$level = ACLAction::getUserAccessLevel($user_id, $this->list_query->getModuleDir(), 'list');
		$ret = array();
		if($level == ACL_ALLOW_OWNER && isset($this->filter_options['filter_owner'])) {
			//$this->filter_options['filter_owner']['disabled'] = true;
			//$this->filter['filter_owner'] = 'mine';
		}
		else if($level == ACL_ALLOW_GROUP) {
			// show rows which are assigned to this user OR are assigned to one of their teams
		}
		else if($level == ACL_ALLOW_NONE) {
			// should not really get this far
		}
	}

	function getFilterNames() {
		$ret = array();
		foreach($this->filter_options as $k => $o) {
			if(empty($o['override']))
				$ret[] = $k;
		}
		return $ret;
	}

	function filterDefined($name, $link=null) {
		if(! is_array($name))
			$name = array('name' => $name, 'link' => $link);
		return $this->getFilterDefinition($name) ? true : false;
	}

	function getFilterValue($name) {
		if(isset($this->override_filters[$name]))
			return array_get_default($this->override_filters[$name], 'value');
		return array_get_default($this->filter, $name);
	}

	function setSessionFilter($filter) {
		$this->session_filter = $filter;
	}


	function loadRequestFilter($list_id, $source='listview', $prefix='') {
        $req =& $_REQUEST;
		if($source === 'listview') {
			$req_list_id = array_get_default($req, 'list_id');
			if($list_id === $req_list_id)
				$source = 'encoded';
			else if(! $req_list_id)
				$source = 'query';
			else
				$source = 'none';
		}
		if($source === 'encoded') {
			if(isset($req['list_filter'])) {
				$input = $req['list_filter'];
				$json = getJSONobj();
				$input = $json->decode($input);
				$this->loadRequestLayout($prefix);
				return $this->loadFilter($input, 'encoded', '', $this->default_filter);
			}
			else
				$source = 'query';
		}
		if($source === 'query') {
			if(! empty($req['query']))
				$source = 'request';
			else if(is_array($this->session_filter) && empty($req['filter_clear']))
				$source = 'session';
			else
				$source = 'none';
		}
		if($source === 'request' && empty($req['filter_clear'])) {
			$this->loadRequestLayout($prefix);
			return $this->loadFilter($req, 'request', $prefix, $this->default_filter);
		}
		if($source == 'session') {
			$this->loadRequestLayout('', true);
			return $this->loadFilter($this->session_filter, 'session', '', $this->default_filter);
		}
		$this->loadRequestLayout($prefix);
		return $this->loadFilter($this->default_filter, 'none', $prefix, $this->default_filter);
	}


	function loadRequestLayout($prefix='', $session=false) {
		if($session)
			$input = $this->session_filter;
		else
			$input =& $_REQUEST;
		$advanced = array_get_default($input, $prefix.'filter_advanced');
		if($advanced && ! isset($this->show_advanced))
			$this->show_advanced = $advanced;
		$layout = array_get_default($input, $prefix.'filter_layout', '');
        if($this->show_advanced && ! $layout) {
        	if(array_key_exists('Standard', $this->layouts))
        		$layout = 'Standard';
        }
        else if(! $layout && isset($this->default_layout))
        	$layout = $this->default_layout;
		$this->search_what = array_get_default($input, $prefix.'filter_text_what', 'all');
		if(isset($this->basic_options['filter_text'])) {
			$this->basic_options['filter_text']['search_what'] = $this->search_what;
		}

		return $this->loadFilterLayout($layout);
	}


	function loadArrayFilter($input, $input_prefix='', $defaults=null) {
		if(! isset($defaults)) $defaults = $this->default_filter;
		return $this->loadFilter($input, 'encoded', '', $defaults);
	}


	function loadFilter($input, $source=null, $input_prefix='', $defaults=null) {
		if(isset($source))
			$this->filter_source = $source;
		if(! is_array($input))
			$input = array();
		if(! is_array($defaults))
			$defaults = array();

		$filter = array();
		$names = $this->getFilterNames();
		$opts =& $this->filter_options;
		if(! $opts) $opts = array();
		$this->loadFilter2($filter, $input, $opts, $names, $input_prefix, '', '', $defaults);
		$this->filter = $filter;
		if($this->apply_acl)
			$this->applyAclFilter();

        if(! $source) $source = true;
        return $source;
	}

	function loadFilter2(array &$filter, array $input, array &$opts, $limit_keys, $input_prefix='', $target_prefix='', $group_prefix='', $defaults=null) {
		global $timedate;

		$unformat_values = ($this->filter_source == 'request' || $this->filter_source == 'editor');
		$this->blank_filter = array();

		if(! $limit_keys)
			$limit_keys = array_keys($opts);
		foreach($limit_keys as $k) {
			$opt =& $opts[$k];
			$base = array_get_default($opt, 'name', $k);
			$uniq = array_get_default($opt, 'unique', '');
			$n = $base . $uniq;
			$prefix = $input_prefix;
			if(isset($opt['_iprefix'])) {
				$prefix .= $opt['_iprefix'];
				$inp_uniq = '';
			} else {
				$prefix .= $group_prefix;
				$inp_uniq = $uniq;
			}
			$tg_prefix = $target_prefix . $group_prefix;
			$gn = $tg_prefix . $n;
			if(isset($this->override_filters[$gn]))
				$filter[$gn] = array_get_default($this->override_filters[$gn], 'value');

			$filter_type = array_get_default($opt, 'type');
			$widget = array_get_default($opt, 'widget');
			if($filter_type == 'filtergroup') {
				if(isset($opt['multiple'])) {
					$this->loadFilter2($filter, $input, $opt['multiple'], null, $input_prefix, $target_prefix, $group_prefix.$opt['name'].'-', $defaults);
				}
				continue;
			}

			if(isset($input[$prefix.$base.$inp_uniq])) {
				if($input[$prefix.$base.$inp_uniq] !== '')
					$filter[$gn] = $input[$prefix.$base.$inp_uniq];
				else {
					// we need to remember blank filter values to override defaults
					// when persisting search via the massupdate form or in the session
					$this->blank_filter[$gn] = '';
				}
			} else if(! isset($filter[$gn])) {
				if(isset($defaults[$tg_prefix.$n]))
					$filter[$gn] = $defaults[$tg_prefix.$n];
				else if(isset($opt['default_value']))
					$filter[$gn] = $opt['default_value'];
			}
			if($filter_type == 'flag' && isset($filter[$gn]))
				$filter[$gn] = $filter[$gn] ? 1 : 0;

			if($filter_type == 'ref' || $filter_type == 'section_ref') {
				if($uniq || (isset($opt['field']) && $base != $opt['field']))
					$idn = $base . $uniq . '_id';
				else
					$idn = array_get_default($opt, 'id_name', $base.'_id');
				if(isset($input[$prefix.$idn])) {
					if($input[$prefix.$idn] !== '')
						$filter[$tg_prefix.$idn] = $input[$prefix.$idn];
				} else if(isset($defaults[$tg_prefix.$idn])) {
					if($defaults[$tg_prefix.$idn] !== '')
						$filter[$tg_prefix.$idn] = $defaults[$tg_prefix.$idn];
				}
				if($filter_type == 'section_ref') {
					if(isset($opt['options_function']))
						$this->expandFilterOptions($opt, $filter);
					$id_val = array_get_default($filter, $tg_prefix.$idn);
					if(isset($opt['options']) && ! isset($opt['options'][$id_val]))
						$filter[$tg_prefix.$idn] = key($opt['options']);
				}
				$dynm = array_get_default($opt, 'dynamic_module');
				if($dynm) {
					$dynm .= $uniq;
					if(isset($input[$prefix.$dynm]) && $input[$prefix.$dynm] !== '')
						$filter[$tg_prefix.$dynm] = $input[$prefix.$dynm];
					else if(isset($defaults[$tg_prefix.$dynm]) && $defaults[$tg_prefix.$dynm] !== '')
						$filter[$tg_prefix.$dynm] = $defaults[$tg_prefix.$dynm];
				}
			}

			else if($filter_type == 'section' || $filter_type == 'owner') {
				$v = array_get_default($filter, $gn);
				if(isset($opt['options_function']))
					$this->expandFilterOptions($opt, $filter);
				if($filter_type == 'owner' && $v == 'select')
					$v = null; // not selectable
				if(! isset($opt['options'][$v])) {
					if($filter_type != 'owner' || ! $this->addOwnerFilterOption($v, $gn))
						$filter[$gn] = array_get_default($opt, 'default_value');
				}
			}

			else if(($filter_type == 'date' || $filter_type == 'datetime')) {
				$db_pat = ($filter_type == 'date') ? '~^(\d{4}-\d{1,2}-\d{1,2})$~' : '~^(\d{4}-\d{1,2}-\d{1,2})( \d{1,2}:\d{1,2}:\d{1,2})?$~';
				$op_filter = $n.'-operator';
				$end_filter = $n.'-end';
				$gop = $tg_prefix.$op_filter;
				if(isset($input[$prefix.$op_filter]))
					$filter[$gop] = $input[$prefix.$op_filter];
				else if(isset($defaults[$gop]))
					$filter[$gop] = $defaults[$gop];
				else if(isset($opt['operator']))
					$filter[$gop] = $opt['operator'];
				else if(! empty($filter[$n]))
					$filter[$gop] = 'on_date';
				else
					$filter[$gop] = '';

				if($filter[$gop] == 'month') {
					$filter[$gop] = 'between_dates';
					if(preg_match('~^(\d{4})-(\d{1,2})$~', $filter[$gn], $m)) {
						$filter[$gn] = $m[1] . '-' . $m[2] . '-1';
						$end_day = date('t', strtotime($filter[$gn]));
						$input[$prefix.$end_filter] = $m[1] . '-' . $m[2] . '-' . $end_day;
					}
				}

				if(! in_array($filter[$gop], self::$date_filter_ops))
					$filter[$gop] = self::$date_filter_ops[0];
				if(! in_array($filter[$gop], self::$date_filter_date_ops))
					unset($filter[$gn]);
				if($filter[$gop] == 'between_dates') {
					if(isset($input[$prefix.$end_filter]) && $input[$prefix.$end_filter] !== '') {
						$filter[$tg_prefix.$end_filter] = $input[$prefix.$end_filter];
						if($unformat_values && ! preg_match($db_pat, $filter[$tg_prefix.$end_filter])) {
							$filter[$tg_prefix.$end_filter] = $timedate->to_db_date($filter[$tg_prefix.$end_filter], false, false);
						}
					}
				}

				$pd_filter = $n.'-period';
				$gpd = $tg_prefix.$pd_filter;
				if(substr($filter[$gop], 0, 7) == 'period_') {
					if(isset($input[$prefix.$pd_filter]))
						$filter[$gpd] = $input[$prefix.$pd_filter];
					else if(isset($defaults[$gpd]))
						$filter[$gpd] = $defaults[$gpd];
					else if(isset($opt['period']))
						$filter[$gpd] = $opt['period'];
					else
						$filter[$gpd] = '';
					if(! in_array($filter[$gpd], self::$date_filter_periods))
						$filter[$gpd] = self::$date_filter_periods[0];
				}

				if(! empty($filter[$gn]) && $unformat_values && ! preg_match($db_pat, $filter[$gn]) ) {

					if($filter[$gn] != 'Today')
						$filter[$gn] = $timedate->to_db_date($filter[$gn], false, false);
				}
			}

			else if($filter_type == 'enum' || $filter_type == 'multienum' || $filter_type == 'status') {
                if(isset($filter[$gn]) && is_string($filter[$gn]) && strpos($filter[$gn], '^,^') !== false)
                    $filter[$gn] = explode('^,^', $filter[$gn]);

				$op_filter = $n.'-operator';
                $gop = $tg_prefix.$op_filter;

                if(isset($input[$prefix.$op_filter]))
                    $filter[$gop] = $input[$prefix.$op_filter];
                else if(isset($defaults[$gop])) {
                    $filter[$gop] = $defaults[$gop];
					if(! $filter[$gop] && isset($filter[$gn])) {
						if(is_array($filter[$gn]))
							$filter[$gop] = 'any_of';
						else if(strlen($filter[$gn]))
							$filter[$gop] = self::DEFAULT_SELECT_FILTER_OPERATOR;
					}
				}
                else if(isset($opt['operator']))
                    $filter[$gop] = $opt['operator'];
                else {
                	if(isset($filter[$gn])) {
                		if(is_array($filter[$gn]))
                			$filter[$gop] = 'any_of';
                		else
							$filter[$gop] = self::DEFAULT_SELECT_FILTER_OPERATOR;
					} else
						$filter[$gop] = '';
                }
                if(! in_array($filter[$gop], self::$select_filter_ops))
                    $filter[$gop] = self::DEFAULT_SELECT_FILTER_OPERATOR;
                if(! $filter[$gop] || $filter[$gop] == 'empty' || $filter[$gop] == 'not_empty')
                    unset($filter[$gn]);
			}

			else if(in_array($filter_type, EditableForm::$number_types)) {
				$op_filter = $n.'-operator';
				$gop = $tg_prefix.$op_filter;
				if(isset($input[$prefix.$op_filter]))
					$filter[$gop] = $input[$prefix.$op_filter];
				else if(isset($defaults[$gop]))
					$filter[$gop] = $defaults[$gop];
				else if(isset($opt['operator']))
					$filter[$gop] = $opt['operator'];
				else
					$filter[$gop] = '';
				if(! in_array($filter[$gop], self::$number_filter_ops))
					$filter[$gop] = '';
				if(! $filter[$gop] || $filter[$gop] == 'empty' || $filter[$gop] == 'not_empty')
					unset($filter[$gn]);
				$end_filter = $n.'-end';
				if($filter[$gop] == 'between') {
					if(isset($input[$prefix.$end_filter])) {
						if($input[$prefix.$end_filter] !== '') {
							$fev = $input[$prefix.$end_filter];
							if($unformat_values)
								$fev = unformat_number($fev);
							$filter[$tg_prefix.$end_filter] = $fev;
						}
					}
					else if(isset($defaults[$tg_prefix.$end_filter])) {
						$filter[$tg_prefix.$end_filter] = $defaults[$tg_prefix.$end_filter];
					}
				}
				if(isset($filter[$gn]) && $unformat_values)
					$filter[$gn] = unformat_number($filter[$gn]);
			}

			else if(! $widget && $filter_type != 'unified_search' && $filter_type != 'flag' && $filter_type != 'bool') {
				$op_filter = $n.'-operator';
				$gop = $tg_prefix.$op_filter;
				if(isset($input[$prefix.$op_filter]))
					$op = $input[$prefix.$op_filter];
				else if(isset($defaults[$gop]))
					$op = $defaults[$gop];
				else if(isset($opt['operator']))
					$op = $opt['operator'];
				else
					$op = '';
				if($op && in_array($op, self::$text_filter_ops))
					$filter[$gop] = $op;
			}

			if ($widget) {
				$field = $this->loadWidget($widget, $opt);
				if($field) {
					$field->loadFilter($filter, $input, $prefix);
				}
			}
		}
	}


	function loadWidget($widget, $def) {
		if(! is_array($def)) $def = array();
		$name = array_get_default($def, 'name');
		if($name && isset($this->widgets[$name]))
			return $this->widgets[$name];
		if($this->form_id && empty($def['form_id']))
			$def['form_id'] = $this->form_id;
		$wspec = AppConfig::get_widget_spec($widget);
		if($wspec) {
			$name = array_get_default($def, 'name');
			if(isset($wspec['params']))
				$def = array_merge($wspec['params'], $def);
			$field = new $wspec['class']($def, $this->base_model);
			if ($field instanceof FilterElement) {
				if($name) $this->widgets[$name] = $field;
				return $field;
			}
		}
	}


	function getFilterClauses($forWorkflow = false) {
		$opts = $this->filter_options;
		$this->applyOverrideFilters($opts);
		$this->expandFilters($opts);
		return $this->getFilterClausesFromOpts($opts, $this->filter, '', $forWorkflow);
	}

	function getFilterClausesFromOpts(array $opts, array $filter, $prefix='', $forWorkflow = false) {
		$ret = array();
		foreach($opts as $f => $sparam) {
			$wfield = $widget = null;
        	$field = array_get_default($sparam, 'field', '');
        	if(isset($sparam['link']) && $field && strpos($field, '.') === false)
        		$field = $sparam['link'].'.'.$field;
        	$rkey = $prefix.$f;
        	if(isset($sparam['override']))
        		$value = array_get_default($sparam, 'value');
        	else
				$value = array_get_default($filter, $rkey);
			$t = array_get_default($sparam, 'type', '');
			if(! $t && isset($sparam['multiple']))
				$t = 'filtergroup';
			$when = array_get_default($sparam, 'when');
			$ignore_val = ($when == 'updated' || $when == 'not_updated') || $forWorkflow;
			$oper = array_get_default($filter, $rkey.'-operator');

			if (isset($sparam['widget'])) {
				$widget = $sparam['widget'];
				$wfield = $this->loadWidget($widget, $sparam);
			}

			if(! empty($wfield)) {
				$clause = $wfield->getFilterClause($filter);
				if (isset($clause)) $ret[$rkey] = $clause;
			}
			else if (isset($sparam['filter_clause_source'])) {
				$src = $sparam['filter_clause_source'];
				$farg = $sparam;
				$farg['value'] = $value;
				$farg['name'] = $field; // FIXME - ensure that $farg['name'] maps to the quoted database field name for sql generation
				$farg['table'] = $this->base_model->getTableAlias(true);
				$clause = call_hook_function($src, $this->base_model->getModelName(), array($farg, $filter, $prefix));
				if (isset($clause)) $ret[$rkey] = $clause;
			}
			else if($t == 'filtergroup') {
				$op = array_get_default($sparam, 'operator', 'and');
				$multi = array_get_default($sparam, 'multiple', array());
				$clauses = $this->getFilterClausesFromOpts($multi, $filter, $prefix.$sparam['name'].'-', $forWorkflow);
				$ret[$rkey] = array('multiple' => $clauses, 'operator' => $op);
			}
			else if($t == 'unified_search' && strlen($value ?? '')) {
				$q = $this->list_query->getUnifiedSearchFilter($value, $this->unified_search_match, null, $this->search_what);
				if($q !== false) {
					$hooks = AppConfig::setting('model.hooks.' . $this->base_model->getModelName() . '.unified_search', array());
					foreach($hooks as $hook)
						call_hook_function($hook, $this->base_model->getModelName(), array(&$this, &$this->list_query, &$q, $value));

					$q = array('multiple' => $q, 'operator' => 'or');
					$ret[$rkey] = $q;
				}
			}
			else if($t == 'ref' || $t == 'section_ref') {
				$idn = array_get_default($sparam, 'id_name');
				$dynn = array_get_default($sparam, 'dynamic_module');
				if(! $idn)
					$idn = $field . '_id';
				else {
					// add link
					if( ($p = strrpos($field, '.')) !== false)
						$idn = substr($field, 0, $p+1) . $idn;
				}
				if($field && $field == $f)
					$inp_idn = $idn;
				else
					$inp_idn = $f . '_id';
				$value = array_get_default($filter, $prefix.$inp_idn);
				if(isset($value) || $ignore_val) {
					$ret[$rkey] = array('field' => $idn, 'value' => $value, 'when' => $when);
				}
				else if($field) {
					$value = array_get_default($filter, $rkey);
					if(isset($value)) {
						if($dynn) {
							$dyn_val = array_get_default($filter, $prefix.$dynn);
							if($dyn_val)
								$ret[$rkey] = array('field' => $field, 'value' => $value, 'operator' => 'like', 'filter_module' => $dyn_val, 'when' => $when);
						} else
							$ret[$rkey] = array('field' => $field, 'value' => $value, 'operator' => 'like', 'when' => $when);
					}
				}
			}
			else if($t == 'date' || $t == 'datetime') {
				if($this->context == 'workflow' && $field) {
					require_once('include/database/DateRange.php');
					$dr = new IAHDateRange($this->base_model, $this->context);
					try {
						$range = [];
						foreach ($filter as $fk => $fv) {
							if (strpos($fk, $rkey . '-') === 0) {
								$fk = substr($fk, strlen($rkey)+1);
									$range[$fk] = $fv;
							}
						}
						if(!isset($range['value']) && isset($value))
							$range['value'] = $value;
						$range = $this->normalizeRange($range);
						$val = $dr->encodeRange($range);
						$filt = $dr->parseRange($val);
						$ret[$rkey] = array('field' => $field, 'value' => $filt, 'operator' => 'date_range', 'when' => $when);
					} catch(IAHDateParseError $e) {
						$GLOBALS['log']->error("Error saving workflow filter: ".$e->getMessage());
					}
				}
				else if($field) {
					$chk = isset($value);
					if($oper == 'between_dates')
						$chk |= isset($filter[$rkey.'-end']);
					if(($oper && (! in_array($oper, self::$date_filter_date_ops) || $chk)) || $ignore_val) {
						if($value == 'Today') {
							if($oper == 'before_date')
								$day = ' -1 day';
							else
								$day = ' +1 day';

							$date = $GLOBALS['timedate']->get_gmt_db_date();

							$value = date('Y-m-d', strtotime($date.$day));
						}
						$ret[$rkey] = array('field' => $field, 'value' => $value, 'operator' => $oper, 'when' => $when);
						if(isset($filter[$rkey.'-period'])) {
							$ret[$rkey]['period'] = $filter[$rkey.'-period'];
						}
						if($oper == 'between_dates' && isset($filter[$rkey.'-end'])) {
							$ret[$rkey]['end'] = $filter[$rkey.'-end'];
						}
					}
				}
			}
            else if($t == 'enum' || $t == 'multienum' || $t == 'status')  {
                if($field) {
                    if(! $oper) $oper = 'eq';
                    $chk = isset($value);
                    if(($oper && (! in_array($oper, self::$select_filter_ops) || ($oper == 'empty' || $oper == 'not_empty') || $chk)) || $ignore_val) {
                        $ret[$rkey] = array('field' => $field, 'value' => $value, 'operator' => $oper, 'when' => $when);
                    }
                }
            }
			else if(in_array($t, EditableForm::$number_types)) {
				if($field) {
					$chk = isset($value);
					if($oper == 'between')
						$chk |= isset($filter[$rkey.'-end']);
					if(($oper && (($oper == 'empty' || $oper == 'not_empty') || $chk)) || $ignore_val) {
						$ret[$rkey] = array('field' => $field, 'value' => $value, 'operator' => $oper, 'when' => $when);
						if($oper == 'between' && isset($filter[$rkey.'-end'])) {
							$ret[$rkey]['end'] = $filter[$rkey.'-end'];
						}
					}
				}
			}
			else if(isset($sparam['multiple'])) {
				if($t == 'flag') {
					$negate = array_get_default($sparam, 'negate_flag', false);
					if(! $value ^ $negate)
						continue;
				}
				$spec = array('multiple' => $sparam['multiple']);
				if(isset($sparam['operator']))
					$spec['operator'] = $sparam['operator'];
				$ret[$rkey] = $spec;
			}
			else if($oper == 'empty' || $oper == 'not_empty') {
				$ret[$rkey] = array('field' => $field, 'operator' => $oper, 'when' => $when);
			}
			else if(isset($value) || $ignore_val) {
				$param = $sparam;
				if($t == 'flag') {
					$negate = array_get_default($param, 'negate_flag', false);
					if(! $value ^ $negate)
						continue;
				}
				else if($t == 'section' || $t == 'owner') {
					$param = $param['options'][$value];
					$field = array_get_default($param, 'field', '');
				}
				else if($t == 'section_ref') {
					$field = $param['id_name'];
				}
				if(isset($param['value']))
					$value = $param['value'];
				$spec = array('value' => $value);
				$fpfx = '';
				if(! empty($sparam['field'])) {
					if( ($pp = strrpos($sparam['field'], '.')) !== false && strpos($field, '.') === false)
						$fpfx = substr($sparam['field'], 0, $pp+1);
				}
				if($field)
					$spec['field'] = $fpfx.$field;
				if(isset($param['match']))
					$spec['match'] = $param['match'];
				if(isset($param['link_name'])) // 'exists' filter
					$spec['link_name'] = $param['link_name'];
				if(isset($param['operator']))
					$spec['operator'] = $param['operator'];
				if(isset($param['period']))
					$spec['period'] = $param['period'];
				if(isset($param['end']))
					$spec['end'] = $param['end'];
				if($when)
					$spec['when'] = $when;
				if(! empty($param['fields'])) {
					$ror = array(
						'operator' => 'or',
						'multiple' => array(),
					);
					foreach($param['fields'] as $f2) {
						$spec['field'] = $fpfx.$f2;
						$ror['multiple'][] = $spec;
					}
					$ret[$rkey] = $ror;
				} else
					$ret[$rkey] = $spec;
			}
		}
		return $ret;
	}


	function addHiddenFields($fields) {
		if(is_array($fields))
		foreach($fields as $k => $f)
			$this->add_hidden_fields[$k] = $f;
		parent::addHiddenFields($fields);
	}


	function formatFilterValues(array $filters, array $values) {
		global $timedate;
		foreach($filters as $k => $f) {
			$t = array_get_default($f, 'type');
			if(in_array($t, EditableForm::$number_types)) {
				if(isset($values[$k])) {
					$values[$k] = format_number($values[$k], -1);
				}
				if(isset($values[$k.'-end'])) {
					$values[$k.'-end'] = format_number($values[$k.'-end'], -1);
				}
			}
			else if(($t == 'date' || $t == 'datetime')) {
				if(isset($values[$k]) && $values[$k] != 'Today') {
					$values[$k] = $timedate->to_display_date($values[$k], false);
				}
				if(isset($values[$k.'-end'])) {
					$values[$k.'-end'] = $timedate->to_display_date($values[$k.'-end'], false);
				}
			}
		}
		return $values;
	}


	function renderField(RowResult $result, $name, $form_field=null) {
		if(is_string($name)) {
			if( ($p = strpos($name, '!')) !== false)
				$name = substr($name, 0, $p);
			$def = $result->fields[$name];
		} else
			$def = $name;
		if($def && isset($def['widget'])) {
			$wobj = $this->loadWidget($def['widget'], $def);
			if($wobj)
				return $wobj->render($this, $result);
		}
		return parent::renderField($result, $name, $form_field);
	}


	/** Format filters using EditableForm directly, for layout editor
	*/
	function formatFilters(array $filters, $values=null, $edit=true) {
		$module = $this->list_query->getModuleDir();
		require_once('include/database/RowResult.php');
		$result = new RowResult;
        $result->base_model = $this->base_model->name;
        $result->module_dirs[] = $module;
        if(! isset($values))
        	$values = $this->filter;

		$f2 = array();
		foreach($filters as $k => $f) {
			if(! empty($f['override'])) {
				$values[$k] = array_get_default($f, 'value');
				if(! isset($f['hidden']))
					$f['hidden'] = true;
			}
			$f2[$k] = $f;
		}
		$result->row = $values;
		$result->formatted = $this->formatFilterValues($filters, $values);
		$result->fields = $f2;

		$ret = array();
		foreach($f2 as $k => $f) {
			if(! empty($f['filter_internal']))
				continue;
			$type = array_get_default($f, 'type', '');
			if($type == 'filtergroup') {
				$body = $init = $preinit = '';
				if(! $edit) {
					$gop = strtoupper(array_get_default($f, 'operator', 'and'));
					$body = translate('LBL_FILTER_GROUP_'.$gop, 'app').translate('LBL_SEPARATOR', 'app');
				}
			} else {
				if($edit) {
					$body = $this->renderField($result, $k);
					$init = $this->getControlsInit(true);
					$preinit = $this->getExtraInit(true);
				} else {
					$body = $this->previewFilter($result, $k);
					$init = $preinit = null;
				}
			}
			$row = array(
				'depth' => array_get_default($f, 'depth', 0),
				'name' => $f['name'],
				'field' => array_get_default($f, 'field', $f['name']),
				'id_name' => array_get_default($f, 'id_name'),
				'source' => array_get_default($f, 'link', ''),
				'label' => $f['label'],
				'std_label' => array_get_default($f, 'std_label'),
				'type' => $type,
				'type_label' => array_get_default($f, 'type_label', ''),
				'hidden' => array_get_default($f, 'hidden', 0),
				'when' => array_get_default($f, 'when'),
				'_init' => array('filter' => $body, 'ctor' => $init, 'ext' => $preinit),
				'_fixed' => empty($f['basic']) ? 0 : 1,
			);
			if($type == 'filtergroup') {
				$row['_group'] = 1; // allow nesting in DynamicList
				$row['operator'] = array_get_default($f, 'operator', 'and');
			}
			$ret[] = $row;
		}
		return $ret;
	}

	function isBasicLayout() {
		return (empty($this->layout_name) || $this->layout_name === 'Browse') && empty($this->show_advanced) && ! $this->is_custom;
	}


	/** Format filters within a search form (using FormGenerator)
	*/
	function formatSearch($list_id, $module, $action, $limit=null, $mode=null, $params=null) {
		global $themeInstance, $pageInstance;

		if(! is_array($params)) $params = [];

		$basic_search = $this->isBasicLayout();
		$render_basic = ($mode === 'basic') || $basic_search;
		$basic_show_labels = isset($this->show_labels) ? $this->show_labels : false;
		if($this->is_report && ($this->is_report_data || $this->report_run_method != 'interactive'))
			$all_opts = array();
		else
			$all_opts = $this->getDisplayFilterOptions($basic_search, true);

		$sorted_opts = array_filter($all_opts, function($opt) {
			return empty($opt['hidden']);
		});

		$hidden = array(
			'list_id' => $list_id, 'id' => $list_id, 'module' => $module, 'action' => $action,
			'query' => true, 'filter_clear' => '', 'filter_layout' => $this->layout_name);
		if(isset($limit)) $hidden['list_limit'] = $limit;
		$hidden = array_merge($this->add_hidden_fields, $hidden);

		$filter = $this->filter;
		$opt_keys = array();
		foreach($sorted_opts as $k => &$opt) {
			if(! empty($opt['override'])) {
				$filter[$k] = array_get_default($opt, 'value');
			}
			$opt['name'] = $k; // adds group prefix to nested fields
			if(empty($opt['hidden'])) {
				if($opt['type'] == 'hidden')
					$hidden[$k] = array_get_default($filter, $k);
				else
					$opt_keys[] = $k;
			}
			$opt['force_editable'] = true;
			if(! isset($opt['required']))
				$opt['required'] = false;
		}
		unset($opt);

		require_once('include/layout/forms/HtmlFormGenerator.php');
		require_once('include/database/RowResult.php');
		$result = new RowResult;
        $result->base_model = $this->base_model->name;
        $result->module_dirs[] = $this->list_query->getModuleDir();
		$result->row = $filter; // FIXME will need to unformat values like dates
		$result->formatted = $this->formatFilterValues($sorted_opts, $filter);
		$result->fields = $all_opts;

		foreach($this->auto_options as $k => $f) {
			if(isset($this->filter_options[$k]) && ! empty($this->filter_options[$k]['hidden']) && isset($filter[$k]))
				$hidden[$k] = $filter[$k];
		}
		$controls = array();

		$layout = array(
			'indicate_required' => false,
			'editable' => true,
			'form_hidden_fields' => $hidden,
			'icon' => $this->getIcon(),
			'title' => $this->getTitle(),
			'title_desc' => $this->getTitleDesc(),
			'type' => 'searchform',
			'base_module' => $module,
			'buttons_position' => 'bottom',
			'show_title_links' => false,
		);
		if (!empty($list_id)) {
			$layout['form_hooks'] = array('onsubmit' => "return sListView.performSearch('{$list_id}');");
		}

		$menu_title = translate('LBL_FILTER_CHOOSE');
		$edit_layout = $delete_layout = $duplicate_layout = null;
		$tools_btn = '';

		if (!AppConfig::is_mobile()) {
			if($render_basic) {
				$menu = array(
					'type' => 'button',
					'id' => $list_id.'-FilterMenu',
					'label' => '<div class="input-arrow input-bounded">'.$menu_title.'</div>',
					'html_label' => true,
					'class' => 'input-button menu-source',
					'tabindex' => 4,
				);
				if (!empty($list_id)) {
					$menu['perform'] = "return sListView.performSearch('{$list_id}', {filter_advanced: 1});";
				}
			} else {
				$menu_opts = array();
				$alts = 0;
				foreach($this->layouts as $lid => $l) {
					if($lid == 'Browse' && $lid !== $this->layout_name) continue;
					$menu_opts[$lid] = array(
						'label' => translate(array_get_default($l, 'title'), $module),
						'perform' => "sListView.setSearchLayout('$list_id', '$lid');",
					);
					if($this->layout_name === $lid)
						$menu_title = $menu_opts[$lid]['label'];
					else
						$alts ++;
				}
				if($alts) {
					//$json = getJSONobj(); $opts_js = $json->encode(array('keys' => array_keys($menu_opts), 'values' => $menu_opts, 'label_key' => 'label'));
					//$GLOBALS['pageInstance']->add_js_literal("SUGAR.ui.registerSelectOptions('{$list_id}-FilterOptions', $opts_js);", null, LOAD_PRIORITY_FOOT);
					//$GLOBALS['pageInstance']->add_js_literal("SUGAR.ui.registerInput(new SUGAR.ui.SelectInput('{$list_id}-FilterMenu', {options: '{$list_id}-FilterOptions'}));", null, LOAD_PRIORITY_FOOT);
					$menu = array(
						'id' => $list_id.'-menu',
						'tabindex' => 4,
						'type' => 'select',
						'icon' => 'icon-filter',
						'options' => $menu_opts,
						'align' => 'right',
						'value' => $this->layout_name,
						'label_key' => 'label',
						'onchange' => "return sListView.setFilterLayout('{$list_id}', this.getValue());",
						'var_width' => true,
					);
					/*$menu = '<button type="button" tabindex="4" id="'.$list_id.'-FilterMenu" class="input-select"><div class="input-arrow input-bounded">'.$menu_title.'</div></button>';
					$menu = array(
						'static_html' => "<div style=\"float: right\">{$menu}</div>",
					);*/
				} else
					$menu = '';

				if($this->is_custom && ($this->edit_layout || $this->is_report)) {
					$edit_layout = array(
						'icon' => 'icon-editlayout',
						'label' => translate('LBL_EDIT_LAYOUT_BUTTON_LABEL', 'app'),
						'align' => 'right',
						'style' => 'margin-left: 0.25em',
						'type' => 'button',
						'perform' => "return sListView.showEditLayout('$list_id');",
						'tabindex' => 5,
					);
					$delete_layout = array(
						'icon' => 'icon-delete',
						'label' => translate('LBL_DELETE_LAYOUT_BUTTON_LABEL', 'app'),
						'align' => 'right',
						'style' => 'margin-left: 0.25em',
						'type' => 'button',
						'perform' => "return sListView.deleteLayout('$list_id');",
						'tabindex' => 6,
					);
					if($this->is_report) {
						$edit_layout['label'] = translate('LBL_EDIT_REPORT_BUTTON_LABEL', 'app');
						$delete_layout['label'] = translate('LBL_DELETE_REPORT_BUTTON_LABEL', 'app');
					}
				}
                if ($this->is_custom && ! $this->is_report) {
                    $duplicate_layout = array(
                        'icon' => 'icon-duplicate',
                        'label' => translate('LBL_DUPLICATE_LAYOUT_BUTTON_LABEL', 'app'),
                        'align' => 'right',
                        'style' => 'margin-left: 0.25em',
                        'type' => 'button',
                        'perform' => "return sListView.showDuplicateLayout('$list_id');",
                        'tabindex' => 5,
                    );
                }
				if($this->is_report_data /* == 'archived'*/) {
					$layout['form_buttons']['return'] = array(
						'icon' => 'icon-return',
						'label' => translate('LBL_RETURN_TO_REPORT_BUTTON_LABEL', 'app'),
						'type' => 'button',
						'perform' => "return sListView.returnToReport('$list_id');",
						'tabindex' => 5,
					);
				}
			}

			$clear_button = array();
			if ($this->show_clear) {
				$clear_button = array(
					'name' => 'clearButton',
					'icon' => 'uii-clear',
					'label' => translate('LBL_CLEAR_BUTTON_LABEL', 'app'),
					'perform' => "return sListView.clearSearch('{$list_id}');",
					'tabindex' => $render_basic ? 5 : 3,
					'type' => 'button',
				);
			}

			$tools_options = array();
			if($themeInstance->version(8) && ! AppConfig::read_only() && ! $themeInstance->classic_title) {
				$tools_options['create_layout'] = array(
					'icon' => 'icon-add',
					'label' => translate('LBL_CREATE_LAYOUT_FORM_TITLE', 'app'),
					'perform' => "return sListView.showCreateLayout('$list_id');",
				);
				$lname = array_get_default($this->hidden_fields, 'layout');
				if($this->is_report || $lname == 'Reports') {
					$tools_options['create_layout']['label'] = translate('LBL_CREATE_REPORT_BUTTON_LABEL', 'app');
				}
			}

			if(! $render_basic) {
				$layout_lbl = translate('LBL_FILTER_LAYOUT', 'app');
				$buttons = array();
				if($themeInstance->version(8)) {
					$tools_options['delete_layout'] = $delete_layout;
					$tools_options['edit_layout'] = $edit_layout;
					$tools_options['duplicate_layout'] = $duplicate_layout;
				} else {
					$buttons['delete_layout'] = $delete_layout;
					$buttons['edit_layout'] = $edit_layout;
                    $buttons['duplicate_layout'] = $duplicate_layout;
                }
                $buttons['apply'] = array(
					'name' => 'applyButton',
					'type' => 'submit',
					'icon' => 'uii-accept',
					'label' => translate('LBL_APPLY_FILTER_BUTTON_LABEL', 'app'),
					'tabindex' => 3,
				);
				$buttons['archive'] = array(
					'name' => 'archiveButton',
					'type' => 'button',
					'icon' => 'uii-accept',
					'label' => translate('LBL_RUN_AND_ARCHIVE_BUTTON_LABEL', 'app'),
					'tabindex' => 4,
				);
				$buttons['clear'] = $clear_button;
				if (!empty($list_id)) {
					$buttons['apply']['perform'] = "return sListView.performSearch('{$list_id}');";
				}
				if($this->is_report) {
					$buttons['apply']['label'] = translate('LBL_RUN_REPORT_BUTTON_LABEL', 'app');

					if(! $this->is_report_data && AppConfig::is_admin()) {
						$buttons['sql'] = array(
							'type' => 'button',
							'icon' => 'theme-icon ticon-Log',
							'label' => translate('LBL_SHOW_SQL_BUTTON_LABEL', 'app'),
							'perform' => "return sListView.showReportSql('{$list_id}', this);",
						);
						if($themeInstance->version(8)) {
							$tools_options['sql'] = $buttons['sql'];
							unset($buttons['sql']);
						}
					}

					if($this->report_run_method != 'interactive') {
						$buttons['apply']['perform'] = "return sListView.runReport('{$list_id}', this);";
						unset($buttons['clear']);
						unset($buttons['archive']);

						/*if($this->is_report_data != 'archived') {
							$buttons['archived'] = array(
								'type' => 'button',
								'icon' => 'icon-return',
								'label' => translate('LBL_ARCHIVED_RUNS_BUTTON_LABEL', 'app'),
								'perform' => "return sListView.showArchivedReports('{$list_id}');",
								'tabindex' => $render_basic ? 5 : 3,
							);
						}*/
					} else {
						if($this->is_report_data) {
							unset($buttons['clear']);
							unset($buttons['archive']);
						} else
							$buttons['archive']['perform'] = "return sListView.runReport('{$list_id}', this, true);";
					}

					if($this->is_report_data)
						unset($buttons['apply']);
					else {
						$buttons['return'] = array(
							'icon' => 'icon-return',
							'label' => translate('LBL_RETURN_TO_LIST_BUTTON_LABEL', 'app'),
							'type' => 'button',
							'perform' => "return sListView.setLayout('$list_id', 'Reports');",
							'tabindex' => 6,
						);
						if(! $this->is_report_owner && ! AppConfig::is_admin()) {
							unset($buttons['edit_layout']);
							unset($buttons['delete_layout']);
						}
					}
					if($this->is_report_data == 'data') {
						/*$layout['form_buttons']['edit'] = array(
							'icon' => 'icon-edit',
							'label' => translate('LBL_EDIT_RUN_BUTTON_LABEL', 'app'),
							'align' => 'right',
							'style' => 'margin-left: 0.25em',
							'type' => 'button',
							'perform' => "return sListView.showEditRun('$list_id');",
							'tabindex' => 5,
						);*/
						$buttons['delete'] = array(
							'icon' => 'icon-delete',
							'label' => translate('LBL_DELETE_RUN_BUTTON_LABEL', 'app'),
							'align' => 'right',
							'style' => 'margin-left: 0.25em',
							'type' => 'button',
							'perform' => "return sListView.deleteRun('$list_id');",
							'tabindex' => 6,
						);
					}
					if($this->is_report_data /*== 'archived'*/) {
						$buttons['return'] = array(
							'icon' => 'icon-return',
							'label' => translate('LBL_RETURN_TO_REPORT_BUTTON_LABEL', 'app'),
							'type' => 'button',
							'perform' => "return sListView.returnToReport('$list_id');",
							'tabindex' => 5,
						);
					}
				} else {
					unset($buttons['archive']);
				}

				$layout['form_buttons'] = $buttons;

				$cols = min(max(count($opt_keys), 3), 4);
				$layout['sections'] = array(
					array(
						'label_position' => 'top',
						'columns' => $cols,
						'widths' => array('25%'),
						'label_widths' => array('0%'),
						'elements' => $opt_keys,
					),
				);
			}

			$tools_options = array_filter($tools_options);
			if($tools_options) {
				$sel_opts = array('maxlen' => 0);
				if($themeInstance->classic_title) {
					$layout['form_buttons']['tools'] = [
						'name' => 'tools',
						'type' => 'group',
						'icon' => 'uii uii-gear',
						'group_options' => $tools_options,
						'tabindex' => 7,
					];
				} else {
					self::process_opts($tools_options, $sel_opts, 'label');
					$pageInstance->set_tools_options($sel_opts, $this->form_id);
				}
			}
		}

		$layout['form_id'] = $this->form_id;
		$model = new ModelDef($this->base_model);
		$model->addDisplayFieldDefinitions($sorted_opts);
		$gen = new HtmlFormGenerator('html', $model, $layout, $this->form_name, $this->form_id);
		$this->setFormClass('card-outer panel-outer listview-filter');
		//if(! empty($params['hide']))
		//	$this->setFormStyle('display: none');
		$gen->setFormObject($this);
		$frm =& $this;

		$out = '';
		if($render_basic && !AppConfig::is_mobile()) {
			if(! $opt_keys && empty($layout['form_buttons']))
				return $out;
            $clear = '';
            if ($this->show_clear && $clear_button)
			    $clear = $this->renderButton($clear_button);
			//$menu = $frm->renderButton($menu);
				$menu = '';
			// hidden search button needed to enable automatic form submission when enter is pressed
			$search_button = array(
				'name' => 'searchButton',
				'icon' => 'icon-accept',
				'label' => translate('LBL_SEARCH_BUTTON_LABEL', 'app'),
				'tabindex' => 5,
				'type' => 'submit',
				'style' => 'display: none',
			);
			$menu = $this->renderButton($search_button);
			$out = '<div class="card-outer form-outer search-panel">';
			$out .= '<div class="card-body form-body"><div class="form-section"><div class="row form-row">';
			$out .= '<div class="filter-basic-outer column form-cell">';
			$out .= '<div class="filter-basic-inner form-inline">';
			foreach($opt_keys as $f) {
				$opt = $sorted_opts[$f];
				$def = array_get_default($result->fields, $f);
				if (is_array($def)) {
					$label = '';
					$ft = array_get_default($def, 'type');
					if($basic_show_labels || !empty($def['force_label']))
						$label = '<div class="form-label">' . $def['label'] . '</div>';
					else if(($ft == 'ref' || $ft == 'varchar') && ! empty($def['label']))
						$result->fields[$f]['placeholder'] = '(' . $def['label'] . ')';
				}
				$out .= '<div class="form-entry">' . $label . '<div class="form-value">' . $frm->renderField($result, $f) . '</div></div>';
			}
			$out .= '</div>';
			if($menu || $clear || $tools_btn)
				$out .= "<div class=\"filter-basic-controls\">$menu $tools_btn $clear</div>";
			$out .= '</div></div></div></div></div>';
		}

		$gen->wrap_form = false;
		$gen->setHeader($out);
		$gen->renderForm($result);
		$out = $gen->getResult();
		$gen->exportIncludes();
		/*if($render_basic) {
			$id = $list_id . '-filter';
			$s = '';
			if(! empty($params['hide']))
				$s = 'style="display: none"';
			$out = '<div class="listview-filter" id="'.$id.'" '.$s.'>'.$out.'</div>';
		}*/
		return $out;
	}


	function flattenFilters(array $opts, array &$into, $depth=0, $prefix='') {
		foreach($opts as $key => $opt) {
			$t = array_get_default($opt, 'type');
			if(! $t && ! empty($opt['multiple']))
				$t = $opt['type'] = 'filtergroup';
			$opt['depth'] = $depth;
			$into[$prefix.$key] = $opt;
			if($t == 'filtergroup') {
				$this->flattenFilters($opt['multiple'], $into, $depth+1, $prefix.$opt['name'].'-');
			}
		}
	}


	function getSortedFilterOptions(array $opts) {
		$cat_sort = array('unified_search' => 0, 'section' => 1, 'section_ref' => 2, 'owner' => 3, 'flag' => 4);
		$cats = $pos = array();
		$i = 0;
		foreach($opts as $key => $row) {
			$pos[$key] = $i ++;
			$cats[$key] = isset($row['type']) ? array_get_default($cat_sort, $row['type'], 10) : 10;
		}
		array_multisort($cats, SORT_ASC, $pos, SORT_ASC, $opts);
		$ret = array();
		$this->flattenFilters($opts, $ret);
		return $ret;
	}


	function expandFilterOptions(&$opt, $filter=null, $cleanup=false) {
		$fn_args = array();
		if(isset($opt['filter_args'])) {
			foreach($opt['filter_args'] as $fa) {
				if(isset($filter) && isset($filter[$fa]))
					$fn_args[] = $filter[$fa];
				else
					$fn_args[] = $this->getFilterValue($fa);
			}
		}
		$opt['options'] = call_hook_function($opt['options_function'], $this->base_model->name, $fn_args);
		if($cleanup)
			unset($opt['options_function']);
	}


	function getDisplayFilterOptions($basic=false, $hidden=false, $overrides=true) {
		$opts = $this->filter_options;
		if($overrides)
			$this->applyOverrideFilters($opts);
		$this->expandFilters($opts);
		$opts = $this->getSortedFilterOptions($opts);
		$module = $this->list_query->getModuleDir();
		$basic_show_labels = isset($this->show_labels) ? $this->show_labels : false;

		$sorted_opts = $extra = array();
		foreach($opts as $f => $opt) {
			if($basic && empty($opt['basic']))
				continue;
			$t = array_get_default($opt, 'type', '');
			if(! $hidden && ! empty($opt['hidden']))
				continue;
			if($t == 'filtergroup' && ! $hidden)
				continue;
			$uniq = array_get_default($opt, 'unique', '');

			if($t == 'flag') {
				$t = $opt['type'] = 'bool';
				$opt['required'] = true;
				if($basic && ! isset($opt['show_label']) && ! $basic_show_labels)
					$opt['show_label'] = true;
			}
			if(($t == 'enum' || $t == 'multienum' || $t == 'status') && $this->expand_select_filters) {
				$opt['expanded'] = true;
			}
			if($t == 'section' || $t == 'owner') {
				$selopts = array();
				$ico = array_get_default($opt, 'options_icon');
				unset($opt['options_function']);
				foreach($opt['options'] as $k => $o) {
					// FIXME do potential per-option ACL check here
					// may want to copy icon, width, other params
					$newopt = array('label' => $o['label']);
					if($ico) $newopt[$ico] = array_get_default($o, $ico, '');
					$newopt['title'] = array_get_default($o, 'title');
					$selopts[$k] = $newopt;
				}
				$opt['options'] = $selopts;
				$t = $opt['type'] = 'enum';
				if(! isset($opt['required']))
					$opt['required'] = true;
				if(empty($opt['label_key']))
					$opt['label_key'] = 'label';
				$opt['expanded'] = false;
			}
			if($t == 'section_ref') {
				$opt['type'] = 'ref';
				if(isset($opt['options_function']))
					$this->expandFilterOptions($opt, null, true);
			}
			if($t == 'module_name') {
				if(isset($opt['options_function']))
					$this->expandFilterOptions($opt, null, true);
			}
			if($t == 'ref') {
				$opt['quick_create'] = false;
				if(! empty($opt['dynamic_module']))
					$extra[] = array(
						'name' => $opt['dynamic_module'],
						'unique' => $uniq,
					);
			}

			if($basic) {
				$opt['auto_submit'] = true;
                if (isset($opt['basic_type']))
                    $opt['type'] = $opt['basic_type'];
            } else if(isset($opt['label'])) {
				$opt['vname'] = $opt['label'];
				$opt['vname_module'] = $module;
			}

			if(! isset($opt['tabindex']))
				$opt['tabindex'] = 1; // may leave this up to HtmlFormGenerator later
			$sorted_opts[$f] = $opt;
		}

		foreach($extra as $fspec) {
			$lspec = $this->getFilterDefinition($fspec);
			if($lspec) {
				$lspec['unique'] = $fspec['unique'];
				$lspec['hidden'] = $lspec['filter_internal'] = true;
				$index = $lspec['name'] ?? null;
				if (isset($lspec['unique']))
					$index .= $lspec['unique'];
				if ($index)
					$sorted_opts[$index] = $lspec;
			}
		}

		return $sorted_opts;
	}


	// fill in filter operators, plus other attributes which may be based on previous user input ($this->filter)
	function expandFilters(array &$opts, $prefix='') {
		static $default_filter_ops = array(
			'time' => 'eq', 'enum' => 'eq', 'multienum' => 'eq', 'status' => 'eq',
			'bool' => 'eq', 'filtergroup' => 'and', 'module_name' => 'eq',
		);

		$defmod = $this->list_query->getModuleDir();
		if(is_array($opts)) {
			foreach($opts as $f => &$opt) {
				if(! isset($opt['name']))
					$opt['name'] = $f;
				$pf = $prefix.$f;

				$t = array_get_default($opt, 'type');
				if(! $t && ! empty($opt['multiple']))
					$t = $opt['type'] = 'filtergroup';
				if($t == 'filtergroup') {
					if(empty($opt['multiple']))
						$opt['multiple'] = array();
					$this->expandFilters($opt['multiple'], $prefix.$opt['name'].'-');
				}

				if(($t == 'date' || $t == 'datetime')) {
					$op_filter = $pf.'-operator';
					if(isset($this->filter[$op_filter])) {
						$opt['operator'] = $this->filter[$op_filter];
					}
					if(empty($opt['operator']) && ! empty($this->filter[$pf]))
						$opt['operator'] = 'on_date';

					$pd_filter = $pf.'-period';
					if(isset($this->filter[$pd_filter])) {
						$opt['period'] = $this->filter[$pd_filter];
					}
					$end_filter = $pf.'-end';
					if(isset($this->filter[$end_filter])) {
						$opt['end'] = $this->filter[$end_filter];
					}
				}
				else if($t == 'ref') {
					if(isset($opt['field']) && $opt['field'] == $f)
						$id_name = $prefix . array_get_default($opt, 'id_name', '');
					else
						$id_name = $prefix . $f . '_id';
					if(isset($this->filter[$id_name])) {
						$opt['id_value'] = $this->filter[$id_name];
					}
				}
                else if($t == 'enum' || $t == 'status' || $t == 'multienum') {
                    $op_filter = $pf.'-operator';
                    if(isset($this->filter[$op_filter])) {
                        $opt['operator'] = $this->filter[$op_filter];
                    }
                    if(empty($opt['operator']) && ! empty($this->filter[$pf]))
                        $opt['operator'] = self::DEFAULT_SELECT_FILTER_OPERATOR;
                }
                else if(in_array($t, EditableForm::$number_types)) {
					$op_filter = $pf.'-operator';
					if(isset($this->filter[$op_filter])) {
						$opt['operator'] = $this->filter[$op_filter];
					}
					if(empty($opt['operator']) && ! empty($this->filter[$pf]))
						$opt['operator'] = 'eq';

					$end_filter = $pf.'-end';
					if(isset($this->filter[$end_filter])) {
						$opt['end'] = $this->filter[$end_filter];
					}
				}
				else if(! isset($opt['operator'])) {
					if(isset($default_filter_ops[$t]))
						$opt['operator'] = $default_filter_ops[$t];
					else if($t == 'owner' || $t == 'unified_search' || ! empty($opt['multiple']))
						; // pass
					else {
						$op_filter = $pf.'-operator';
						$op = null;
						if(isset($this->filter[$op_filter]))
							$op = $this->filter[$op_filter];
						if(! $op || ! in_array($op, self::$text_filter_ops))
							$op = 'like';
						if($op == 'prefix' || $op == 'suffix' || $op == 'extended') {
							$opt['match'] = $op;
							$op = 'like';
						}
						$opt['operator'] = $op;
					}
				}

				if(isset($opt['vname'])) {
					$mod = array_get_default($opt, 'vname_module', $defmod);
					$opt['std_label'] = translate($opt['vname'], $mod);
				}
				if(! isset($opt['label'])) {
					$opt['label'] = array_get_default($opt, 'std_label', $f);
				}
				if(($t == 'section' || $t == 'section_ref' || $t == 'owner') && isset($opt['options'])) {
					foreach($opt['options'] as $kf => &$o) {
						if(! is_array($o)) $o = array('vname' => $o);
						if(! isset($o['label'])) {
							if(isset($o['vname'])) {
								$mod = array_get_default($o, 'vname_module', $defmod);
								$o['label'] = translate($o['vname'], $mod);
							}
							else
								$o['label'] = '';
						}
					}
				}
			}
		}
	}


	function getArray($add_blank=false) {
		$ret = $this->filter;
		if($add_blank && $this->blank_filter) array_extend($ret, $this->blank_filter);
		return $ret;
	}

	function getFilterSource() {
		return $this->filter_source;
	}

	function getFilterLayoutName() {
		return $this->layout_name;
	}

	function getAvailableFilters($link_name=null, $extend=false) {
		if(! $this->list_query || ! $this->base_model)
			return array();
        if ($this->display_layout_name) {
            $model = $this->display_layout_name;
        } elseif (! empty($link_name)) {
			$lnk = $this->list_query->addLink($link_name);
			if(! $lnk)
				return;
			$model = $this->list_query->getModelName($lnk);
			$link = $link_name.'.';
		}
		else {
			$model = $this->base_model->getModelName();
			$link = '';
		}
		if($this->context == 'workflow')
			$display = array();
		else
			$display = AppConfig::setting("display.filters.$model", array());
		$fields = array_keys($display);
		if($link_name) {
			// remove basic filters from select list - they are mandatory
			//$basic = AppConfig::setting("display.basic_filters.$model", array());
			foreach(array('filter_text', 'filter_owner') as $f)
				unset($fields[$f]);
		}
		$list = $this->list_query->getAvailableFields($link_name);
		if($list)
			$fields = array_unique(array_merge($fields, $list));
		if(! $extend)
			return $fields;

		$ret = array();
		foreach($fields as $k) {
			$def = $this->getFilterDefinition(array('name' => $k, 'link' => $link_name));
			if($def) {
				if(isset($def['source'])) {
					$stype = $def['source']['type'];
					if($stype != 'db' && $stype != 'custom_field' && $stype != 'concat' && $stype != 'subselect'
							// ref fields with dynamic module currently disabled - end up with ["" LIKE 'Name'] clause because name is normally fetched in secondary query
							&& ($def['type'] != 'ref' /* || ! empty($def['dynamic_module'])*/))
						continue;
					if($this->context == 'workflow' && ($stype == 'concat' || $stype == 'subselect')) {
						continue;
					}
				} else if(empty($def['fields']) && empty($def['field']) && empty($def['multiple']))
					continue;
				if(! isset($def['name']))
					$def['name'] = $k;
				if(($def['type'] == 'enum' || $def['type'] == 'multienum' || $def['type'] == 'status') && $this->expand_select_filters) {
					if(! isset($def['expanded']))
						$def['expanded'] = true;
				}
				unset($def['width']);
				$ret[$k] = $def;
			}
		}
		return $ret;
	}

	function previewFilter($result, $name) {
		$spec = $result->getFieldDefinition($name);
		if(! $spec) return '';

		$type = array_get_default($spec, 'type');
		$operator = array_get_default($spec, 'operator');
		$when = array_get_default($spec, 'when');
		$value = $result->getField($name, null, true);

		if($when == 'updated' || $when == 'not_updated') {
			$lbl = translate('LBL_FILTER_WHEN_'.strtoupper($when), 'Workflow');
			return to_html($lbl);
		}

		$whenText = translate('LBL_FILTER_WHEN_' . strtoupper($when));
		if ($whenText) {
			$whenText = $whenText . ' - ';
		}

		if($type == 'filtergroup') {
			return translate('LBL_FILTER_GROUP_'.strtoupper($operator), 'app');
		}
		if($type == 'bool') {
			if($value || $operator == 'true') return translate_dd('dom_int_bool', 1);
			else return $whenText . translate_dd('dom_int_bool', 0);
		}
		if($type == 'date' || $type == 'datetime') {
			if($this->context == 'workflow') {
				require_once('include/database/DateRange.php');
				if (empty($spec['operator']))
					$ret = translate_dd('date_filter_operators_dom', '');
				else {
					$dr = new IAHDateRange($this->base_model, $this->context);
					try {
						$range = $spec;
						$range['value'] = $value;
						$range = $this->normalizeRange($range);
						$val = $dr->encodeRange($range);
						$filt = $dr->parseRange($val);
						$ret = $dr->formatRange($filt);
					} catch(IAHDateRangeError $e) {
						$ret = '&lt;error&gt;';
					}
				}
			}
			else {
				$op = translate_dd('date_filter_operators_dom', $operator);
				$ret = $op . ' ' . $value;
				if($operator == 'between_dates') {
					$ret .= ', '.$result->getField($name.'-end', '', true);
				}
			}
			return $whenText . $ret;
		}
		if($type == 'enum' || $type == 'multienum' || $type == 'status') {
			if($operator == 'eq') $op = '';
			else $op = translate_dd('select_filter_operators_dom', $operator);
			if(! is_array($value)) $value = explode('^,^', $value);
			$opts = array_get_default($spec, 'options', array());
			$vs = array();
			foreach($value as $v) $vs[] = translate_dd($opts, $v);
        	return $whenText . $op . ' ' . to_html(implode(', ', $vs));
		}
		if(in_array($type, EditableForm::$number_types)) {
			$op = translate_dd('number_filter_operators_dom', $operator);
			if(! $operator || $operator == 'empty' || $operator == 'not_empty')
				return $whenText . $op;
			$ret = $op . ' ' . $value;
			if($operator == 'between') {
				$ret .= ', '.$result->getField($name.'-end', '', true);
			}
			return $whenText . $ret;
		}
		if($type == 'ref') {
			// make a link?
			return $whenText . $value;
		}

		$operator = array_get_default($spec, 'match', $operator);
		if($operator == 'like') $operator = '';
		$op = translate_dd('text_filter_operators_dom', $operator);

		if(! strlen($value ?? ''))
			return $whenText . $op . ' <i>' . translate('LBL_EMPTY_STRING', 'app'). '</i>';
		return $whenText . $op . ' ' . to_html($value);
	}

	function normalizeRange($range) {
		$negate = !empty($range['negate']);
		$op = $range['operator'];
		if (strpos($op, 'not_') === 0) {
			$op = substr($op, 4);
			$negate = !$negate;
		}
		if (isset($range['period'])) {
			if(preg_match('~^days_\d+$~', $range['period'])) {
				$range['count'] = substr($range['period'], 5);
				$range['period'] = 'day';
			}
			switch ($range['period']) {
			case 'fiscal_year':
				$range['period'] = 'fiscalyear';
				break;
			case 'fiscal_quarter':
				$range['period'] = 'fiscalquarter';
				break;
			}
		}
		switch ($op) {
		case 'between_dates':
			$op = 'range';
			if (isset($range['value']))
				$range['start'] = ['operator' => 'date', 'value' => $range['value']];
			if (isset($range['end']))
				$range['end'] = ['operator' => 'date', 'value' => $range['end']];
			break;
		case 'before_date':
		case 'after_date':
			$op = str_replace('_date', '', $op);
			$range['value'] = ['operator' => 'date', 'value' => $range['value']];
			break;
		case 'period_current':
			$op = 'current';
			break;
		case 'period_prev':
			$op = 'previous';
			break;
		case 'period_next':
			$op = 'next';
			break;
		case 'yesterday':
		case 'today':
		case 'tomorrow':
			$range['value'] = $op;
			$op = 'special';
			break;
		}
		$range['negate'] = $negate;
		$range['operator'] = $op;
		return $range;
	}

}

