<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormElement.php');

class FormButton extends FormElement {
	var $html_label;
	var $icon;
	var $order;
	var $hooks;
	var $async;
	var $target;
	var $pre_check, $confirm;
	var $perform, $pre_perform, $post_perform;
	var $url;
	var $params;
	var $conn_params;
	var $query_params;
	var $static_html;
	var $align;
	var $tabindex;
	var $group;
	var $group_options;
	var $auto_ungroup;
	var $acl;
	var $acl_module;
	var $no_duplicate; // when buttons are shown at top and bottom of form - only render once
	var $promote;
	var $mode; // limit to view or edit
    /**
     * @var RowResult
     */
    var $record;
    /**
     * @var FormLayout
     */
    var $layout;

	// dropdown button
	var $options;
	var $label_key;
	var $value;
	var $onchange;
	var $var_width;
	var $field_name;

	// loaded from result
	var $result_params = array();

	static $opt_params = array(
		'order', 'list_id', 'confirm', 'static_html', 'perform', 'target', 'url', 'icon',
		'align', 'tabindex', 'conn_params', 'html_label', 'group', 'auto_ungroup',
		'options', 'value', 'label_key', 'onchange', 'var_width', 'field_name', 'acl', 'acl_module',
		'no_duplicate', 'pre_perform', 'post_perform', 'query_params', 'replace', 'promote', 'mode',
	);

	function __construct($params=null, $model=null) {
		$this->init($params, $model);
	}

	function init($params=null, $model=null) {
		parent::init($params, $model);
		foreach(self::$opt_params as $k)
			if(isset($params[$k])) $this->$k = $params[$k];
		$this->async = array_get_default($params, 'async', true);
		$this->hooks = array();
		$this->params = array_get_default($params, 'params', array());
		$this->promoteParams();
		foreach(array('onclick') as $k)
			if(isset($params[$k]))
				$this->hooks[$k] = $params[$k];
		if(isset($params['group_options']) && is_array($params['group_options'])) {
			foreach($params['group_options'] as $k => $b) {
				$this->addGroupOption($k, $b);
			}
		}
		$this->type = array_get_default($params, 'type', 'submit');
	}

	function updateParams(array $params) {
		parent::updateParams($params);
		foreach(self::$opt_params as $k)
			if(array_key_exists($k, $params)) $this->$k = $params[$k];
		if(isset($params['async'])) $this->async = $params['async'];
		foreach(array('onclick') as $k)
			if(array_key_exists($k, $params))
				$this->hooks[$k] = $params[$k];
		if(isset($params['params']))
			array_extend($this->params, $params['params']);
		$this->promoteParams();
		if(isset($params['conn_params'])) {
			if(! is_array($this->conn_params))
				$this->conn_params = array();
			array_extend($this->conn_params, $params['conn_params']);
		}
	}

	function promoteParams() {
		foreach(self::$opt_params as $k) {
			// promote attributes from params array
			if(isset($this->params[$k])) {
				$this->$k = $this->params[$k];
				unset($this->params[$k]);
			}
		}
	}

	function setLayout(FormLayout $layout) {
		$this->layout = $layout;
	}

	static function sort_buttons(&$buttons, $row_result) {
		if(! is_array($buttons) || ! $buttons) return;
		$prio_order = array();
		$groups = array();
		$group_join = array();
		foreach($buttons as $k => &$b) {
			if($b->type == 'group') {
				$groups[] = $k;
				if(! isset($group_join[$k]))
					$group_join[$k] = array();
			} else if($b->group) {
				$group_join[$b->group][] = $k;
			}
		}
		foreach($groups as $k) {
			foreach($group_join[$k] as $k2) {
				$buttons[$k]->addGroupOption($k2, $buttons[$k2]);
				unset($buttons[$k2]);
			}
			$opts = $buttons[$k]->getGroupOptions();
			if ($opts) {
				foreach ($opts as $o) {
					if($row_result)
						$o->loadResult($row_result, array());
				}
			}
			reset($opts);
			if(is_countable($opts) && count($opts) == 1 && current($opts)->auto_ungroup) {
				current($opts)->order = $buttons[$k]->order;
				$buttons[current($opts)->name] = current($opts);
				unset($buttons[$k]);
			}
			else if(! $opts) {
				unset($buttons[$k]);
			}
		}
		foreach($buttons as $k => &$b) {
			$prio_order[] = isset($b->order) ? $b->order : 50;
		}
		$nat_order = range(0, count($buttons)-1);
		array_multisort($prio_order, $nat_order, $buttons);
	}

    function getAdditionalHtml(FormGenerator $gen) {
    }

    function loadResult(RowResult $result, array $context) {
    	parent::loadResult($result, $context);
    	if($result && ! $result->failed) {
    		if(! empty($result->module_dirs[0]))
	    		$result_module = $result->module_dirs[0];
    		$model = $result->base_model;
    		$record = $result->getField('id');
			$this->record = $result;
			if($result->row)
			foreach ($result->row as $k => $v) {
				if(! is_array($v) && ! is_object($v))
					$this->result_params['field.' . $k] = $v;
			}
    	}
    	foreach(array('result_module', 'model', 'record') as $k)
			$this->result_params[$k] = isset($$k) ? $$k : null;
		if($this->acl && ! $result->getAclAllowed($this->acl)) {
			$this->hidden_flag = true;
			$this->hidden = true;
		}
    }

    /**
     * Load system type flags - config, mode, mobile
     * (not bean flags using RowResult, helpful for ListView Actions)
     *
     */
    function loadSystemFlags() {
        foreach(self::$flag_fields as $k) {
            $kflag = $k.'_flag';
            if(isset($this->$kflag))
                $this->$k = eval_filter_flag($this->$kflag);
            else
                $this->$k = null;
        }
    }

    function replacePerformParams($perform, $form_name=null) {
    	global $pageInstance;
    	$record = array_get_default($this->result_params, 'record', '');
    	$replace = array(
    		'{FORM_NAME}' => $form_name,
			'{FORM}' => "document.forms['$form_name']",
    		'{RECORD}' => $record,
    		'{USER}' => AppConfig::current_user_id(),
    	);
    	foreach(array('module', 'action', 'layout') as $k) {
    		$$k = $replace['{'.strtoupper($k).'}'] = $pageInstance->$k;
    	}
    	if($record) {
    		if(! $layout) $layout = 'Standard';
			$replace['{RETURN}'] = "return_module=$module&return_action=$action&return_layout=$layout&return_record=$record";
		} else {
			$replace['{RETURN}'] = '';
		}
		if($this->result_params) {
			foreach($this->result_params as $k => $v)
				$replace['{'.strtoupper($k).'}'] = $v;
		}
    	return str_replace(array_keys($replace), array_values($replace), $perform);
    }

    function getIcon() {
    	return $this->icon;
    }

    function getAcl() {
    	return $this->acl;
    }

	function getHooks($form_name, $in_popup=false) {
		$hooks = $this->hooks;
		if(! isset($hooks['onclick']))
			$hooks['onclick'] = $this->getPerform($in_popup);
		$hooks['onclick'] = $this->replacePerformParams($hooks['onclick'], $form_name);
		return $hooks;
	}

	function getParam($name) {
		if(isset($this->$name)) return $this->name;
		return array_get_default($this->params, $name);
	}

	function getArray() {
		$arr = array();
		foreach(self::$elt_copy_attribs as $k)
			if(isset($this->$k))
				$arr[$k] = $this->$k;
		foreach(self::$opt_params as $k)
			if(isset($this->$k))
				$arr[$k] = $this->$k;
		$arr = array_merge($arr, $this->params, $this->hooks);
		if($this->list_id) {
			foreach(array('pre_check', 'perform', 'pre_perform', 'post_perform') as $k) {
				if(isset($arr[$k]))
					$arr[$k] = str_replace('{LIST_ID}', $this->list_id, $arr[$k]);
			}
		}
		$arr['icon'] = $this->getIcon();
		return $arr;
	}

	function getGroupOptions() {
		$r = $this->group_options;
		if(! $r) $r = array();
		return $r;
	}

	function addGroupOption($k, $b) {
		if(is_array($b)) $b = new FormButton($b);
		$this->group_options[$k] = $b;
	}

	function getOptions($form_name, $in_popup=false, $visible_only=false) {
		if($this->type == 'group') {
			$ret = array();
			$opts = $this->getGroupOptions();
			foreach($opts as $k => $b) {
				if($visible_only && ! $b->getVisible())
					continue;
				$perform = $b->getPerform($in_popup);
				$perform = $this->replacePerformParams($perform, $form_name);
				$ret[$k] = array(
					'icon' => $b->getIcon(),
					'label' => $b->getLabel(),
					'perform' => $perform,
				);
			}
			return $ret;
		}
		return $this->options;
	}

	function getPerform($in_popup=false) {
		$json = getJSONobj();
		if($this->params)
			$params = $this->params;
		else {
			if($this->style == 'massupdate')
				$k = 'mass_perform';
			else
				$k = 'record_perform';
			$params = array($k => $this->name);
		}
		if(isset($this->confirm)) {
			$confirm = 'if(! confirm(\''.javascript_escape($this->confirm).'\')) return false; ';
		} else {
			$confirm = '';
		}

		$pre_perform = $post_perform = '';
		if(isset($this->pre_perform)) {
			$pre_perform = $this->pre_perform;
			if($pre_perform && ! preg_match('~;\s*$~', $pre_perform))
				$pre_perform .= ';';
		}
		$pre_perform = $confirm . $pre_perform;
		if(isset($this->post_perform)) {
			$post_perform = $this->post_perform;
			if($post_perform && ! preg_match('~;\s*$~', $post_perform))
				$post_perform .= ';';
		}

		if(is_string($this->perform)) {
			$perform = $pre_perform . $this->perform . $post_perform;
		}
		else if(! empty($this->url)) {
			$tg = ($this->target) ? ", null, '{$this->target}'" : '';
			$perform = "{$pre_perform} SUGAR.util.navigateTo('".javascript_escape($this->url)."'{$tg}); {$post_perform} return false;";
		}
		else if(isset($params['mass_perform'])) {
			$act = $params['mass_perform'];
			$tg = array_get_default($params, 'mass_target');
			if($tg) $tg = ", '$tg'";
			else if(! empty($params['mass_full_page'])) $tg = ', true';
			else $tg = '';
			$perform = "{$pre_perform} var ret = sListView.sendMassUpdate('{LIST_ID}', '$act'$tg); {$post_perform} return ret;";
		}
		else {
			if ($in_popup) {
				$no_async = ", false, true";
                if (!isset($params['reload_after']))
	                $params['reload_after'] = 1;
                if(!isset($params['close_popup']))
	                $params['close_popup'] = 1;
			} else {
				$no_async = $this->async ? '' : ', true';
			}
			if($this->target)
				$no_async = ", '$this->target'" . ($in_popup ? ', true' : '');
			if(isset($this->conn_params)) {
				$conn_json = ', '.$json->encode($this->conn_params);
			} else {
				$conn_json = ', null';
			}
            $param_json = $json->encode($params);
            $perform = "try { {$pre_perform} var ret = SUGAR.ui.sendForm({FORM}, {$param_json}{$conn_json}{$no_async}); {$post_perform} return ret; } catch(e) { console.error(e); return false; }";
		}
		if($this->list_id) {
			$perform = str_replace('{LIST_ID}', $this->list_id, $perform);
		}
		return $perform;
	}

	function getNoDuplicate() {
		global $themeInstance;
		if($this->promote == 'tools' && ! $themeInstance->classic_title)
			return true;
		return $this->no_duplicate;
	}

	function getVisible() {
		$vis = parent::getVisible();
		if($vis && $this->type == 'group') {
			$vc = 0;
			$opts = $this->getGroupOptions();
			foreach($opts as $k => $b) {
				if($b->getVisible()) {
					$vc ++;
					break;
				}
			}
			if(! $vc) $vis = false;
		}
		return $vis;
	}

	function translate($module) {
		$vmod = empty($this->vname_module) ? $module : $this->vname_module;
		$vconf = $this->confirm;
		if($vconf) {
			$conf = translate($vconf, $vmod);
			if ((string)$conf === '') {
				$conf = translate($vconf, 'app');
			}
			$this->confirm = $conf;
		}
		return parent::translate($module);
	}
}

?>
