<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


class FormElement {
	var $id;
	var $name;
	var $value = '';
    /**
     * @var ModelDef
     */
	var $model;
	var $vname;
	var $vdesc;
	var $vhelp;
	var $vname_module;
	var $label;
	var $desc;
	var $width;
	var $type;
	var $format;
	var $help;
	var $class;
	var $style;
	var $colspan;
	var $rowspan;
	var $tabindex;
    var $onchange;
    var $disable_date;
    var $hide_label;
	var $required;
	var $form_id;
	var $list_id;
	var $widget;
	var $copy_to;
	var $copy_to_rel_contacts;
	var $expanded;
	var $mode = 'any';
	var $format_regexp;
	var $required_fields;
	var $display_rows;
	var $mobile_align = 'right';

	var $hidden, $hidden_flag;
	var $disabled, $disabled_flag;
	var $readonly, $readonly_flag;
	var $editable, $editable_flag;
	var $updateable, $updateable_flag;
	var $explicitly_hidden;
	
	static $elt_copy_attribs = array(
		'type', 'vname', 'vdesc', 'vhelp', 'vname_module', 'label', 'value', 'desc', 'help',
		'format', 'style', 'colspan', 'rowspan', 'tabindex', 'disable_date', 'required',
		'copy_to', 'copy_to_rel_contacts', 'form_id', 'list_id', 'width', 'widget', 'expanded',
		'format_regexp', 'required_fields', 'display_rows', 'mode', 'hide_label', 'explicitly_hidden',
	);

	static $flag_fields = array('hidden', 'disabled', 'readonly', 'editable', 'updateable');
	
    var $elements = array();
    
	function __construct($params=null, $model=null) {
		$this->init($params, $model);
	}
	
	function init($params=null, $model=null) {
		if(! is_array($params)) {
			$name = $params;
			$params = array();
			if(isset($name)) {
				$params['name'] = $name;
            }
		}
		foreach(array('id', 'name', 'class', 'onchange') as $k)
			$this->$k = array_get_default($params, $k, '');
		foreach(self::$elt_copy_attribs as $k)
			if(array_key_exists($k, $params))
				$this->$k = $params[$k];
		foreach(self::$flag_fields as $k) {
			$v = array_get_default($params, $k);
			$kflag = $k.'_flag';
			if(isset($v))
				$this->$kflag = expand_filter_flag($v);
		}
		if(isset($model))
			$this->setModel($model);
	}
	
	function setModel(ModelDef $model, $recurse=false) {
		$this->model = $model;
		if($recurse) {
			$es = $this->getElements();
			if($es)
				foreach($es as &$elt)
					$elt->setModel($model, $recurse);
		}
	}
	
	function updateParams(array $params) {
		foreach(array('id', 'name', 'class', 'onchange') as $k)
			if(array_key_exists($k, $params))
				$this->$k = $params[$k];
		foreach(self::$elt_copy_attribs as $k)
			if(array_key_exists($k, $params))
				$this->$k = $params[$k];
		foreach(self::$flag_fields as $k) {
			if(! array_key_exists($k, $params)) continue;
			$v = $params[$k];
			$kflag = $k.'_flag';
			if(isset($v))
				$this->$kflag = expand_filter_flag($v);
		}
	}

	function loadElement($elt) {
		if(! ($elt instanceof FormElement)) {
			if(is_string($elt))
				$elt = array('name' => $elt);
			if(is_array($elt)) {
				if(! empty($elt['name']) && $this->model) {
					$mod_field = $this->model->getDisplayField($elt['name'], true);
					if($mod_field) {
						unset($mod_field['source']);
						if(isset($mod_field['type']) && $mod_field['type'] == 'widget')
							unset($mod_field['type']);
						$elt = array_merge($mod_field, $elt);
					}
				}
				$type = array_get_default($elt, 'type');
				if(! $type && ! empty($elt['section']))
					$type = 'section';
				$widget = array_get_default($elt, 'widget');
				if($widget)
					$elt = $this->createElementWidget($widget, $elt);
				else
					$elt = $this->createElementType($type, $elt);
			}
			else
				return false;
		}
		return $elt;
	}
	
	function addElements(array $elts) {
		$ret = array();
		foreach($elts as &$e)
			$ret[] = $this->addElement($e);
		return $ret;
	}
	
	function createElementType($type, $spec=null) {
		if($spec)
			return new FormField($spec, $this->model);
	}
	
	function addElement($elt) {
		$elt = $this->loadElement($elt);
		if(! $elt) {
			return null;
		} else if(! empty($elt->id)) {
			$this->elements[$elt->id] =& $elt;
			return $elt->id;
		} else {
			$idx = is_countable($this->elements) ? count($this->elements) : 0;
			$this->elements[$idx] =& $elt;
			return $idx;
		}
	}
	
	function removeElement($elt_path) {
		if(is_array($elt_path)) {
			$elt_id = array_shift($elt_path);
			if(count($elt_path)) {
				if(isset($this->elements[$elt_id]))
					return $this->elements[$elt_id]->removeElement($elt_path);
				return null;
			}
		} else
			$elt_id = $elt_path;
		if(isset($this->elements[$elt_id])) {
			unset($this->elements[$elt_id]);
			return true;
		}
	}
	
	function insertElements($elt_path, array $elts, $pos='replace', $span=0) {
		if(is_array($elt_path)) {
			$elt_id = array_shift($elt_path);
			if(count($elt_path)) {
				if(isset($this->elements[$elt_id])) {
					$found_id = $this->elements[$elt_id]->insertElements($elt_path, $elts, $pos, $span);
					if(! is_array($found_id)) $found_id = array($found_id);
					array_unshift($found_id, $elt_id);
					return $found_id;
				}
				else
					return null;
			}
		} else
			$elt_id = $elt_path;
		$new_id = null;
		if(isset($this->elements[$elt_id])) {
			$old_elts = $this->elements;
			$this->elements = array();
			$skip = 0;
			foreach(array_keys($old_elts) as $k) {
				if(! isset($new_id) && $k == $elt_id && $pos != 'after') {
					$new_id = $this->addElements($elts);
					if($span > 0) $skip = $span;
					if($new_id && $pos == 'replace')
						$skip++;
				}
				if($skip) {
					$skip --;
					continue;
				}
				if(is_int($k))
					$this->elements[] = $old_elts[$k];
				else
					$this->elements[$k] = $old_elts[$k];
				if(! isset($new_id) && $k == $elt_id && $pos == 'after') {
					$new_id = $this->addElements($elts);
					if($span > 0) $skip = $span;
				}
			}
		}
		return $new_id;
	}
	
	function findElementPath($elt_name) {
		foreach($this->elements as $id => &$e) {
			if($e->getName() === $elt_name)
				return $id;
			$sub_id = $e->findElementPath($elt_name);
			if(isset($sub_id)) {
				if(! is_array($sub_id)) $sub_id = array($sub_id);
				array_unshift($sub_id, $id);
				return $sub_id;
			}
		}
	}
	
	function &getElements() {
		return $this->elements;
	}
	
	function getElement($idx) {
		return array_get_default($this->elements, $idx);
	}
	
	function getRequiredFields() {
		if(isset($this->required_fields))
			$ret = $this->required_fields;
		else
			$ret = array();
		foreach(self::$flag_fields as $k) {
			$kflag = $k.'_flag';
			if(isset($this->$kflag))
			$ret = array_merge($ret, get_filter_flag_fields($this->$kflag));
		}
		$es =& $this->getElements();
		if($es)
			foreach($es as &$e)
				$ret = array_merge($ret, $e->getRequiredFields());
		return $ret;
	}
	
	function getId() {
		return $this->id;
	}

	function getName() {
		return $this->name;
	}
	
	function getLabel($context=null) {
		return $this->label;
	}

	function getDescription() {
		return $this->desc;
	}
	
	function getHelp() {
		return $this->help;
	}
	
	function getWidth() {
		return $this->width;
	}
	
	function getClass() {
		return $this->class;
	}
	
	function setClass($cls) {
		$this->class = $cls;
	}

	function getType() {
		return $this->type;
	}
	
	function getVisible() {
		return !$this->explicitly_hidden && !$this->hidden;
	}

	function getDisabled() {
		return $this->disabled;
	}

	function getReadOnly() {
		return $this->readonly;
	}

	function getMode() {
		return $this->mode;
	}
	
	function getEditable($context=null) {
		if(is_array($context)) {
			if(isset($context['editable']) && ! $context['editable'])
				return false;
			if(! isset($this->editable) && ! empty($context['editable']))
				return true;
		}
		return $this->editable;
	}
	
	function getRequired() {
		return $this->required;
	}
		
	function beforeRender(FormGenerator $gen) {
		$es =& $this->getElements();
		if($es)
			foreach($es as &$elt)
				$elt->beforeRender($gen);
	}
	
	function translate($module) {
		$vmod = empty ($this->vname_module) ? $module : $this->vname_module;
		if(isset($this->vname) && ! isset($this->label)) {
			$this->label = translate($this->vname, $vmod);
			if ((string)$this->label === '') {
				$this->label = translate($this->vname, 'app');
			}
		}
		if(isset($this->vdesc) && ! isset($this->desc))
			$this->desc = translate($this->vdesc, $vmod);
		if(isset($this->vhelp) && ! isset($this->help))
			$this->help = translate($this->vhelp, $vmod);
		$es =& $this->getElements();
		if($es)
			foreach($es as &$elt)
				$elt->translate($module);
	}
	
	function loadResult(RowResult $result, array $context) {
		foreach(self::$flag_fields as $k) {
			$kflag = $k.'_flag';
			if(isset($this->$kflag))
				$this->$k = eval_filter_flag($this->$kflag, $result);
			else
				$this->$k = null;
		}
		$edit_layout = ! empty($context['edit_layout']);
		if($edit_layout)
			$this->editable = false;
		else if(! $result->new_record && isset($this->updateable) && ! $this->updateable)
			$this->editable = false;
		$es =& $this->getElements();
		if($es)
			foreach($es as &$elt)
				$elt->loadResult($result, $context);
	}
	
	// for normal field rendering that doesn't depend on a form
	function formatField(FieldFormatter $fmt, RowResult $result, $row_id=null) {
		return $this->getFormattedValue($result);
	}
	
	function formatList(FieldFormatter $fmt, ListResult $result) {
		return;
	}
	
	function getResultFieldName() {
		return $this->name;
	}
		
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		return $this->getFormattedValue($row_result);
	}

	function getRawValue($row_result) {
		if($this->type == 'static' || ! $row_result)
			return '';
		return $row_result->getField($this->getResultFieldName(), '');
	}
	
	function getFormattedValue($row_result) {
		if($this->type == 'static' || ! $row_result)
			return $this->value;
		return $row_result->getField($this->getResultFieldName(), $this->value, true);
	}
	
	function getHtmlCell(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		$this->beforeRender($gen);
		$visib = $this->getVisible();
		return array(
			// render value first - may alter value of label
			'type' => $this->type,
			'name' => $this->getName(),
			'value' => $visib ? $this->renderHtml($gen, $row_result, $parents, $context) : '',
			'raw_value' => $this->getRawValue($row_result),
			'label' => ($visib && ! $this->hide_label) ? $this->getLabel() : '',
			'desc' => $visib ? $this->getDescription() : '',
			'id' => $this->id,
			'colspan' => $this->colspan,
			'rowspan' => $this->rowspan,
			'editable' => $this->getEditable($context),
			'required' => $this->getRequired(),
			'visible' => $visib,
		);
	}


	function getPlainCell($row_result, array $parents, array $context) {
		//$this->beforeRender($gen);
		$visib = $this->getVisible();
		return array(
			'type' => $this->type,
			'name' => $this->name,
			'value' => $visib ? $this->getFormattedValue($row_result) : '',
			'label' => $visib ? $this->getLabel() : '',
			'desc' => $visib ? $this->getDescription() : '',
			'id' => $this->id,
			'colspan' => $this->colspan,
			'rowspan' => $this->rowspan,
			'required' => $this->getRequired(),
			'visible' => $visib,
		);
	}
	
	function loadUpdateRequest(RowUpdate $update, array $input) {
		$es =& $this->getElements();
		if($es) {
			foreach($es as &$e)
				$e->loadUpdateRequest($update, $input);
		}
	}
	
	function validateInput(RowUpdate $update) {
		$es =& $this->getElements();
		if($es) {
			foreach($es as &$e) {
				$ret = $e->validateInput($update);
				if(isset($ret) && ! $ret)
					return false;
			}
		}
	}
	
	function beforeUpdate(RowUpdate $update) {
		$es =& $this->getElements();
		if($es)
			foreach($es as &$e)
				$e->beforeUpdate($update);
	}
	
	function afterUpdate(RowUpdate $update) {
		$es =& $this->getElements();
		if($es)
			foreach($es as &$e)
				$e->afterUpdate($update);
	}

	function createElementWidget($widget, $field) {
		static $widgets = array();
		$form_id = $this->form_id;
		if(! $form_id) $form_id = '';
		if($form_id && ! isset($field['form_id']))
			$field['form_id'] = $form_id;
		$name = array_get_default($field, 'name');
		if (isset($widgets[$form_id][$name]))
			return $widgets[$form_id][$name];
		$wspec = $this->loadWidgetSpec($widget);
		if(! $wspec) {
			$ret = null;
			return $ret;
		}
		if(isset($wspec['params']))
			$field = array_merge($wspec['params'], $field);
		$field = new $wspec['class']($field, $this->model);
		
		// ajrw - Disabled cache. Widget should only be looked up once per form
		//$widgets[$form_id][$name] =& $field;
		
		return $field;
	}
	
	function loadWidgetSpec($widget) {
		return AppConfig::get_widget_spec($widget);
	}
}

?>
