<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormElement.php');

class FormField extends FormElement {
	var $id_field;
	var $type_field;
	var $max_length;
	var $size, $height;
	var $show_label;
	var $missing_field;
	var $skip_field_lookup;
	var $list_source;
	var $list_format;
	var $list_position;
	var $placeholder;
	var $link;
	
	static $field_copy_attribs = array(
		'max_length', 'size', 'id_field', 'type_field', 'height', 'show_label', 'list_source',
		'onkeypress', 'onkeyup', 'onkeydown', 'disabled', 'readonly',
		'list_format', 'list_position', 'placeholder',
	);
    
    static $editable_source_types = array(
    	'ref', 'address', 'image',
    );

	function init($params=null, $model=null) {
		if(! is_array($params)) $params = array();
		$this->skip_field_lookup = array_get_default($params, 'skip_field_lookup');
		parent::init($params, $model);
		foreach(self::$field_copy_attribs as $k)
			if(! isset($this->$k) && isset($params[$k]))
				$this->$k = $params[$k];
	}
	
	function getRequiredFields() {
		$ret = parent::getRequiredFields();
		$name = empty($this->format) ? $this->name
			: array('name' => $this->name, 'format' => $this->format);
		$ret = array_merge($ret, array($name, $this->id_field, $this->type_field));
		return $ret;
	}
	
	function getVisible() {
		$ret = parent::getVisible();
		if($ret && ! $this->name)
			return false;
		return $ret;
	}
	
	function getOverrideFields() {
		static $f;
		if(! isset($f)) {
			$f = array_merge(self::$elt_copy_attribs, self::$field_copy_attribs);
			$f[] = 'class';
			$f[] = 'onchange';
		}
		return $f;
	}
	
	function getOverrideSpec() {
		$ret = array();
		foreach($this->getOverrideFields() as $k)
			if(! empty($this->$k))
				$ret[$k] = $this->$k;
		return $ret;
	}
	
	function setModel(ModelDef $model, $recurse=false) {
		parent::setModel($model, $recurse);
		if(! $this->skip_field_lookup) {
			$this->loadModelFields();
		}
	}
	
	function loadModelFields() {
		$this->missing_field = true;
		if($this->name) {
			$def = $this->model->getDisplayField($this->name, true);
			if($def) {
				$this->missing_field = false;
				$this->loadFieldDefinition($def);
			}
		}
	}
	
	function loadFieldDefinition(array $def) {
		foreach(array('type', 'vname', 'vdesc', 'vhelp', 'vname_module', 'format', 'required', 'format_regexp') as $f)
			if(! isset($this->$f) && isset($def[$f]))
				$this->$f = $def[$f];

		if(isset($def['editable']) && ! isset($this->editable_flag))
			$this->editable_flag = $def['editable'];
		if(isset($def['updateable']) && ! isset($this->updateable_flag))
			$this->updateable_flag = $def['updateable'];
		
		if(isset($def['len']) && ! isset($this->max_length))
			$this->max_length = $def['len'];
		if($this->type == 'ref') {
			if(! empty($def['id_name']))
				$this->id_field = $def['id_name'];
			if(! empty($def['dynamic_module']))
				$this->type_field = $def['dynamic_module'];
		}
		
		// FIXME
		// make calculated fields non-editable: this may need to be refined or checked elsewhere
		// example is full_number for Quotes or ExpenseReports
		$src = array_get_default($def, 'source');
		if(! isset($this->editable_flag) && is_array($src)
				&& ! in_array($this->type, self::$editable_source_types)
				&& ($this->type != 'multienum' && $src['type'] != 'db' && $src['type'] != 'field' && $src['type'] != 'custom_field')) {
			$this->editable_flag = false;
		}
		if(! empty($def['force_editable'])) {
			$this->editable_flag = true;
		}
	}
	
	function getResultFieldName() {
		$name = parent::getResultFieldName();
		if($this->format) $name .= '!'.$this->format;
		return $name;
	}
	
	// for rendering involving a form (detailview/editview)
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if(empty($this->name) || $this->missing_field || ! $row_result)
			return '';
		if($this->getEditable($context))
			$value = $this->standardRenderEdit($gen->getFormObject(), $row_result, $parents, $context);
		else
			$value = $this->standardRenderView($row_result, $parents, $context);
		return $value;
	}
	
	// override in order to reduce SQL queries
	function formatList(FieldFormatter $fmt, ListResult $result) {
		$name = $this->getResultFieldName();
		foreach($result->getRowIndexes() as $idx) {
			$result->formatted_rows[$idx][$name] = $this->formatField($fmt, $result->getRowResult($idx), $idx);
		}
	}

	// custom for listview
	function renderListValue(ListFormatter $list_fmt, ListResult $result, $row_id, $list_params=null) {
		$row = $result->getRowResult($row_id);
		if($row) {
			$fmt = $list_fmt->getFieldFormatter();
			return $this->formatField($fmt, $row);
		}
		return '';
	}
	
	function standardRenderEdit($form, RowResult $result, array $parents, array $context) {
		return $form->renderField($result, $this->getResultFieldName(), $this);
	}
	
	function getRawValue($result) {
		return array_get_default($result->row, $this->getResultFieldName(), '');
	}

	function standardRenderView(RowResult $result, array $parents, array $context) {
		$ret = $this->getFormattedValue($result);
		// show refs as a dash when rendered as part of an editable form
		if(! strlen($ret) && ! empty($context['editable']) && $this->type == 'ref') {
			$ret = '&mdash;';
		}
		return $ret;
	}
}

?>
