<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormLayout.php');

abstract class FormGenerator {
	var $name;
	var $format;
	var $model;
	var $layout;
	var $field_map;
	var $form_obj;
	var $debug;
	var $perform;
	var $acl_require;
	var $acl_check;
	var $require_display_name, $display_name;
	var $link_parent_info;
	var $form_id;
	var $before_update_called = false;
	var $after_update_called = false;
	var $extra_required = [];
	var $header;
	var $footer;


	static function new_form($format=null, $model=null, $layout=null, $form_name=null, $form_id=null) {
		if(! isset($format)) $format = 'html';
		if($format == 'html') {
			require_once('include/layout/forms/HtmlFormGenerator.php');
			$form = new HtmlFormGenerator($format, $model, $layout, $form_name, $form_id);
			return $form;
		}
		if($format == 'pdf') {
			require_once 'include/layout/forms/PdfFormGenerator.php';
			$form = new PdfFormGenerator($format, $model, $layout, $form_name, $form_id);
			return $form;
		}

	}

	static function html_form($model=null, $layout=null, $form_name=null, $form_id=null) {
		return self::new_form('html', $model, $layout, $form_name, $form_id);
	}


	function __construct($format=null, $model=null, $layout=null, $form_name=null, $form_id=null) {
		$this->format = $format;
		if(is_string($model)) $model = new ModelDef($model);
		$this->model = $model;
		$this->form_id = $form_id;
		if(isset($layout))
			$this->setLayout($layout);
		if(isset($form_name)) {
			require_once('include/layout/forms/EditableForm.php');
			$this->setFormObject(new EditableForm($this->layout->getType(), $form_name, $form_id));
		}
		$this->init();
	}

	function setLayout($layout) {
		if(is_array($layout)) {
			if (!isset($layout['form_id']))
				$layout['form_id'] = $this->form_id;
			$layout = new FormLayout($layout, $this->model);
		}
		if($layout instanceof FormLayout)
			$this->layout = $layout;
		else
			$this->layout = new FormLayout(array('form_id' => $this->form_id), $this->model);
	}

	function setFormObject($form) {
		$this->form_obj = $form;
	}

	function &getFormObject() {
		return $this->form_obj;
	}

	function getFormClass() {
		return $this->getFormObject()->getFormClass();
	}

	function setFormClass($cls) {
		$this->getFormObject()->setFormClass($cls);
	}

	function setFieldNamePrefix($prefix) {
		$this->getFormObject()->setFieldNamePrefix($prefix);
	}

	function getModel() {
		return $this->model;
	}

	function getLayout() {
		return $this->layout;
	}

	function setTitle($t, $icon='') {
		$this->getLayout()->setTitle($t, $icon);
	}

	function setDefaultEditable($edit, $override=true) {
		$this->getLayout()->setDefaultEditable($edit, $override);
	}

	function getDefaultEditable() {
		return $this->getLayout()->getDefaultEditable();
	}

	function setDefaultLabelPosition($pos) {
		return $this->getLayout()->setDefaultLabelPosition($pos);
	}

	function getDefaultLabelPosition() {
		return $this->getLayout()->getDefaultLabelPosition();
	}

	abstract function exportIncludes();

	function requireAclAction($name) {
		$this->acl_require[$name] = 1;
	}

	function addAclChecks($checks) {
		if(is_string($checks)) $checks = explode(',', $checks);
		foreach($checks as $acl)
			$this->acl_check[$acl] = 1;
	}

	function setPerform($perform) {
		$this->perform = $perform;
	}

	function setLinkParentRecord($module, $action, $record, $layout=null) {
		$this->link_parent_info = compact('module', 'action', 'record', 'layout');
	}


	function renderForm($row_result=null, $module=null) {
		if(! isset($module) && ($row_result && isset($row_result->module_dirs[0])))
			$module = $row_result->module_dirs[0];
		$this->layout->translate($module);
		$this->layout->processFormButtons($row_result);
		$this->layout->processTabs($row_result);

		if($this->form_obj) {
			if(isset($this->layout->form_hooks)) {
				$this->form_obj->addHooks($this->layout->form_hooks);
				$this->layout->form_hooks = null; // prevent duplicate addition
			}
			if(isset($this->layout->form_hidden_fields))
				$this->form_obj->addHiddenFields($this->layout->form_hidden_fields);
			if(isset($this->layout->rec_hidden_fields)) {
				$hidden = array();
				foreach($this->layout->rec_hidden_fields as $f) {
					$hidden[$f] = $row_result->getField($f);
				}
				$this->form_obj->addHiddenFields($hidden);
			}
		}

		$this->renderStart();
		$this->renderTitle();
		$this->renderHeader($row_result, $module);
		$this->renderSections($row_result, $module);
		$this->renderFooter();
		$this->renderFinish();
	}

	function renderBodyOnly($row_result=null, $module=null) {
		if(! isset($module))
			$module = $row_result->module_dirs[0];
		$this->layout->translate($module);
		$this->renderSections($row_result, $module);
	}

	function renderSections($row_result=null, $module=null) {
		$context = array();
		$tabs_spec = $this->layout->getTabs();
		$std_tab = null;
		$ban_tabs = array();
		if($tabs_spec) {
			foreach($tabs_spec as $t) {
				if(! isset($std_tab) && empty($t['hidden']) && isset($t['name']))
					$std_tab = $t['name'];
				if(! empty($t['restricted']) && isset($t['name']))
					$ban_tabs[$t['name']] = 1;
			}
		}
		$ids = $this->layout->getSectionIds();
		$editable = $this->layout->getDefaultEditable();
		$sect_tab = array();
		$tab_sect = array();
		$tab_prev = array('' => null);
		$visib_ids = array();
		foreach($ids as $index => $id) {
			$sect = $this->layout->getSection($id);
			if($row_result)
				$sect->loadResult($row_result, $context);
			if (! $this->layout->getEditingLayout()) {
				if(! $sect->getVisible())
					continue;
				$mode = $sect->getMode();
				if($editable && $mode != 'edit' && $mode != 'any')
					continue;
				if(!$editable && $mode != 'view' && $mode != 'any')
					continue;
			}
			$tab = $sect->tab;
			if(! $tab)
				$tab = '';
			$chk_tab = ($tab ? $tab : $std_tab);
			if($chk_tab && ! empty($ban_tabs[$chk_tab]))
				continue;
			$visib_ids[] = $id;
			$sect_tab[$id] = $tab;
			$tab_sect[$tab][] = $id;
		}

		foreach($visib_ids as $index => $id) {
			$tab = $sect_tab[$id];
			array_pop($tab_sect[$tab]);
			$context['next'] = array_get_default($tab_sect[$tab], 0);
			$context['prev'] = array_get_default($tab_prev, $tab);
			if(isset($this->layout->show_empty_rows))
				$context['show_empty_rows'] = $this->layout->show_empty_rows;
			$this->renderSection($id, $row_result, array(), $context);
			$tab_prev[$tab] = $id;
		}
	}

	function renderSection($id, $row_result, array $parents, array $context) {
		$sect = $this->layout->getSection($id);
		if($sect) {
			if(! isset($context))
				$context = array();
			$this->_renderSection($sect, $row_result, $parents, $context);
		}
	}

	function renderStart() { }
	function renderHeader($row_result=null, $module=null) {
		$this->renderSummary($row_result, $module);
	}
	function renderFooter() { }
	function renderFinish() { }

	abstract function renderTitle();

	function renderSummary($row_result=null, $module=null) {
		$summary = $this->layout->getSummary();
		$context = [];
		if($summary) {
			if($row_result)
				$summary->loadResult($row_result, $context);
			if($summary->getVisible()) {
				$this->_renderSection($summary, $row_result, array(), $context);
			}
		}
	}

	abstract function _renderSection(FormSection $section, $row_result, array $parents, array $context);

	abstract function getResult();
	abstract function init();

	function setHeader($text) {
		$this->header = $text;
	}

	function setFooter($text) {
		$this->footer = $text;
	}

	function getRequiredFields() {
		if($this->layout)
			return $this->layout->getRequiredFields();
	}

	function getExtraRequiredFields() {
		// used to add fields required by subpanels on the same page
		return $this->extra_required;
	}

	function addRequiredFields($fs) {
		array_extend($this->extra_required, $fs);
	}

	function getCheckAcl() {
		// only checking ACL on buttons for now - maybe fields later
		$acl_check = array();
		$btns = $this->layout->getFormButtons();
		if($btns) {
			foreach($btns as $b) {
				$acl = $b->getAcl();
				if($acl) $acl_check[$acl] = 1;
			}
		}
		$tabs = $this->layout->getTabs();
		if($tabs) {
			foreach($tabs as $t) {
				if(! empty($t['acl']))
					$acl_check[$t['acl']] = 1;
			}
		}
		if($this->acl_check)
			array_extend($acl_check, $this->acl_check);
		return array_keys($acl_check);
	}

	function getRequireAcl() {
		return $this->acl_require;
	}

	function &getQuery($row_key=null) {
		require_once('include/database/ListQuery.php');

		$q = new ListQuery($this->model);
		$layout_fields = $this->getRequiredFields();
		if($this->require_display_name)
			$this->display_name = $q->addDisplayName();
		$field_aliases = $q->addFields($layout_fields);
		if(isset($row_key))
			$q->addFilterPrimaryKey($row_key);
		$acl_check = $this->getCheckAcl();
		$q->addAclChecks($acl_check);
		$acl_req = $this->getRequireAcl();
		if($acl_req) $q->addAclFilters($acl_req);

		switch($this->perform) {
			case 'save':
			case 'edit':
				$hooks = array('edit', 'before_save', 'save', 'after_save');
                $this->addSubpanelFields($q);
				break;
			case 'delete':
				$q->addFields(array('deleted', 'date_modified', 'modified_user_id'));
				$hooks = array('before_delete', 'delete', 'after_delete');
				break;
			default:
                $this->addMetaChangesFields($layout_fields, $q);
                $this->addSubpanelFields($q);
                $hooks = array('view');
		}
		if (!$this->layout->getEditingLayout()) {
			$hooks[] = 'edit_layout';
		}
		// add fields required by custom logic hooks we might encounter
		$hook_fields = $this->model->getHookRequiredFields($hooks);
		if($hook_fields)
			$q->addFields($hook_fields);
		$ext_req = $this->getExtraRequiredFields();
		if($ext_req)
			$q->addFields($ext_req);

        if ($this->perform == 'view' && AppConfig::setting("display.list.{$this->model->name}.show_favorites")) {
            require_once('modules/Favorites/Favorite.php');
            Favorite::add_join($q);
        }

        return $q;
	}

	function requireDisplayName($req = true) {
		$this->require_display_name = $req;
	}

	function getDisplayNameField() {
		return $this->display_name;
	}

	function getFormName() {
		return null;
	}

	function addSubpanelFields(ListQuery $q) {
		$model = $q->getModelName();
		$links = AppConfig::setting("model.links.$model");
		foreach ($links as $link) {
			if (!empty($link['initial_filter_fields'])) {
				$q->addFields(array_keys($link['initial_filter_fields']));
			}
		}
	}

    /**
     * Add modified_user and created_user fields to query
     *
     * @param array $layout_fields
     * @param ListQuery $query
     */
    function addMetaChangesFields($layout_fields, ListQuery $query) {
        if (is_array($layout_fields) && sizeof($layout_fields) > 0) {
            $fields = array();

            if (in_array('date_modified', $layout_fields))
                $fields[] = 'modified_user';
            if (in_array('date_entered', $layout_fields))
                $fields[] = 'created_by_user';

            if (sizeof($fields) > 0)
                $query->addFields($fields);
        }
    }

    function getQueryResult(ListQuery $list_query, $do_format=true, $for_duplicate=false) {
		global $db;

		$result = $list_query->runQuerySingle(null, $for_duplicate);
		if($this->debug)
			prq($result->query, 'row query', true);
		if($this->debug > 1)
			pr2($list_query->added_field_names, 'added field names');
		if(! $result->failed && $do_format)
			$this->formatResult($result);

		if($for_duplicate && $this->model->name == 'Task') {
			$result->row['status'] = $result->fields['status']['default'];
		}

		return $result;
	}

	function formatResult(RowResult $result) {
		require_once('include/layout/FieldFormatter.php');
		// fixme - create fieldformatter from query
		// query->getformatter? query->formatrows?
		// or better maybe, use listresult
		$lt = $this->layout->getType();
		if(! $lt || $lt == 'record') {
			if($this->layout->getDefaultEditable() && $this->getFormObject()) $lt = 'editview';
			else $lt = 'view';
			$new_style = true;
		}
		$fmt = new FieldFormatter($this->format, $lt);
		$fmt->show_favorite_star = AppConfig::setting("display.list.{$result->base_model}.show_favorites");
        if(! empty($new_style)) {
        	$fmt->show_social_accounts = false;
        	if($fmt->show_favorite_star)
				$fmt->show_favorite_star = 'after';
        }
		if($this->getFormName() == 'MiniDetailForm')
			$fmt->limit_text_length = 300;
		if($this->link_parent_info && ! empty($this->link_parent_info['module'])) {
			$fmt->parent_module = $this->link_parent_info['module'];
			$fmt->parent_action = $this->link_parent_info['action'];
			$fmt->parent_record = $this->link_parent_info['record'];
			$fmt->parent_layout = $this->link_parent_info['layout'];
		}
        $fmt->formatRowResult($result);
	}

	function getRowResult($row_key, $do_format=true, $for_duplicate=false) {

		// ajrw: this method is explicitly separate from DBCache, which should do the same thing
		// does DBCache need to use RowResult as well?

		$query = $this->getQuery($row_key);

        return $this->getQueryResult($query, $do_format, $for_duplicate);
	}

	function getBlankResult() {
		$query = $this->getQuery(null);
		$result = $query->getBlankResult();
		$this->formatResult($result);
		return $result;
	}

	function loadUpdateRequest(RowUpdate $update, array $input) {
		return $this->layout->loadUpdateRequest($update, $input);
	}

	function validateInput(RowUpdate $update) {
		return $this->layout->validateInput($update);
	}

	function beforeUpdate(RowUpdate $update) {
		if (!$this->before_update_called) {
			$this->before_update_called = true;
			return $this->layout->beforeUpdate($update);
		}
	}

	function afterUpdate(RowUpdate $update) {
		if (!$this->after_update_called) {
			$this->after_update_called = true;
			return $this->layout->afterUpdate($update);
		}
	}

	function addUpdateHooks(RowUpdate $upd) {
		$upd->addHook('before_save', array(&$this, 'beforeUpdate'));
		$upd->addHook('after_save', array(&$this, 'afterUpdate'));
		$this->before_update_called = $this->after_update_called = false;
	}

	static function calculateConditionallyHidden($row_result) {
		$hidden = array();
		foreach ($row_result->fields as $f) {
			if (!empty($f['depends_on']) && isset($row_result->fields[$f['depends_on']])) {
				if (in_array($f['depends_on'], $hidden)) {
					$hidden[] = $f['name'];
					continue;
				}
				$parent_f = $row_result->fields[$f['depends_on']];
				$parent_value = $row_result->getField($f['depends_on']);
				if ($parent_f['type'] == 'bool') {
					if ($parent_value && $f['depends_on_value'] === '0')
						$hidden[] = $f['name'];
					else if (!$parent_value && $f['depends_on_value'] === '1')
						$hidden[] = $f['name'];
				} else {
					if (is_array($f['depends_on_value']) && !empty($f['depends_on_value'])) {
						if ($parent_f['type'] == 'multienum') {
							$parent_value_arr = array_filter(explode('^,^', $parent_value));
							if (!count(array_intersect($parent_value_arr, $f['depends_on_value'])))
								$hidden[] = $f['name'];
						} else {
							if (!in_array($parent_value, $f['depends_on_value']))
								$hidden[] = $f['name'];
						}
					}
				}
			}
		}
		return $hidden;
	}
}
