<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormTableSection.php');
require_once('include/layout/forms/FormButton.php');
require_once('include/layout/forms/FormSummary.php');

class FormLayout extends FormElement {
	var $name;
	var $title;
	var $icon;
	var $title_desc;
	var $type;
	var $model;
	var $base_module;
	var $class;
	var $form_hooks;
	var $form_hidden_fields;
	var $rec_hidden_fields;
	var $form_buttons;
	var $buttons_position;
	var $form_autofocus_fields;
	var $styles;
	var $scripts;
	var $tabs;
	var $sections = [];
	var $section_index;
	var $show_title_links;
    var $in_popup = false;
	var $indicate_required = true;
	var $skip_field_lookup = false;
    var $current_tab, $default_tab;
    var $summary;
    var $css_classes;
    var $show_empty_rows;

    var $default_editable = false;
    var $default_label_pos = 'left';
	var $editing_layout = false;
	var $editable_sections = array();
	var $drop_targets = array();
	var $disabled_layout_fields = array();
	var $drag_sources = array();
	var $availableFields = array();
	var $editElements = array();


	function __construct($params=null, $model=null) {
		$this->init($params, $model);
	}

	function init($params=null, $model=null) {
		global $themeInstance;
		if(! is_array($params))
			$params = array();
		if (isset($params['form_id']))
			$this->form_id = $params['form_id'];
		if($model)
			$this->setModel($model);
		$this->editing_layout = !empty($this->editing_layout) || !empty($params['editing_layout']);
		foreach(array('name', 'title', 'icon', 'title_desc', 'class', 'base_module', 'buttons_position') as $k)
			$this->$k = array_get_default($params, $k, '');
		if(isset($params['editable']))
			$this->default_editable = $params['editable'];
		$this->default_label_pos =
			($themeInstance && $themeInstance->default_label_pos) ? $themeInstance->default_label_pos :
			($themeInstance && $themeInstance->version(8) ? 'top' : 'left');
		if(isset($params['label_position']))
			$this->default_label_pos = $params['label_position'];
		if(! empty($params['type']))
			$this->setType($params['type']);
        if(! empty($params['in_popup']))
            $this->setInPopup($params['in_popup']);
        if(isset($params['indicate_required']))
            $this->setIndicateRequired($params['indicate_required']);
        if(isset($params['skip_field_lookup']))
            $this->skip_field_lookup = $params['skip_field_lookup'];
		if(! empty($params['form_hooks']))
			$this->addFormHooks($params['form_hooks'], true);
		if(! empty($params['form_hidden_fields']))
			$this->addFormHiddenFields($params['form_hidden_fields'], true);
		if(! isset($this->form_buttons))
			$this->form_buttons = array();
		if(! empty($params['mobile_form_buttons']) && AppConfig::is_mobile())
			$this->addFormButtons($params['mobile_form_buttons'], false);
		if(! empty($params['form_buttons']) && !$this->editing_layout && !AppConfig::is_mobile())
			$this->addFormButtons($params['form_buttons'], false);
		if(! empty($params['form_autofocus_fields']))
			$this->form_autofocus_fields = $params['form_autofocus_fields'];
		$this->sections = array();
		$this->section_index = 0;
		$this->styles = array();
		$this->scripts = array();
		if(isset($params['summary']))
			$this->setSummary($params['summary']);
		if(isset($params['tabs']))
			$this->addTabs($params['tabs']);
		if(! empty($params['current_tab']))
			$this->setCurrentTab($params['current_tab']);
		if(! empty($params['default_tab']))
			$this->setDefaultTab($params['default_tab']);
		if(isset($params['sections']))
			$this->addSections($params['sections']);
		if(isset($params['ext_sections']))
			$this->extendSections($params['ext_sections']);
		if(isset($params['styles']))
			$this->addStyles($params['styles']);
		if(isset($params['scripts']))
			$this->addScripts($params['scripts']);
		if(isset($params['ext_elements']))
			$this->extendElements($params['ext_elements']);
		if(isset($params['show_empty_rows']))
			$this->show_empty_rows = $params['show_empty_rows'];
	}

	function initCssClasses() {
		$edit_layout = $this->getEditingLayout();
		$lstyle = $this->getType();
		$lcls = $this->getClass();
		$editable = ($this->getDefaultEditable() !== false && ($lstyle != 'view' || strpos($lcls, 'tabForm') !== false));
		if($edit_layout) {
			$lstyle = 'editview';
			$lcls = '';
			$editable = true;
		}
		$fcls = 'default-form';
		if($lstyle == 'configform')
			$fcls = 'config-form';
		else if($lstyle == 'searchform')
			$fcls = 'config-form search-form';
		else if($lstyle == 'massupdate')
			$fcls = 'config-form massupdate-form';
		else if($lstyle == 'adminform')
			$fcls = 'admin-form';

		$outer = 'card-outer form-outer '.$fcls.($editable ? ' editable' : '');
		if($lcls)
			$outer .= ' '.$lcls;

		/*if ($edit_layout) {
			$lblCls .= ' edit_layout';
		}*/

		$this->css_classes = [
			'form-outer' => $outer,
			'form-header' => 'card-header form-header',
			'form-footer' => 'card-footer form-footer',
			'form-title' => 'row form-title-row',
			'form-title-section' => 'row form-row form-title-row',
			'form-title-entry' => 'form-entry form-title',
			'form-buttons-top' => 'card-header form-buttons',
			'form-buttons-bottom' => 'card-footer form-buttons',
			'form-buttons-row' => 'row buttons-row',
			'form-body' => 'card-body form-body',
			'form-section' => 'form-section',
			'form-row' => 'row form-row',
			'form-cell' => 'column form-cell',
			'form-spacer' => 'column form-cell form-spacer',
			'form-value' => 'form-value',
			'form-entry' => 'form-entry',
			'form-label' => 'form-label',
			'form-desc' => 'form-desc',
		];
	}

	function getCssClasses() {
		if(! $this->css_classes) $this->initCssClasses();
		return $this->css_classes;
	}

	function setTitle($title, $icon='', $desc=null) {
		$this->title = $title;
		$this->icon = $icon;
		$this->title_desc = $desc;
	}

	function getTitle() {
		return translate($this->title, $this->base_module);
	}

	function getIcon() {
		if($this->show_title_links && ! $this->icon)
			return $this->base_module;
		return $this->icon;
	}

	function getTitleDesc() {
		return $this->title_desc;
	}

	function setBaseModule($module) {
		$this->base_module = $module;
	}

	function getBaseModule() {
		return $this->base_module;
	}

	function setModel(ModelDef $model, $recurse=false) {
		$this->model = $model;
	}

	function getModel() {
		return $this->model;
	}

	function setDefaultEditable($edit, $override=true) {
		if($override || ! isset($this->default_editable))
			$this->default_editable = $edit;
	}

	function getDefaultEditable() {
		return $this->default_editable;
	}

	function setDefaultLabelPosition($pos) {
		$this->default_label_pos = $pos;
	}

	function getDefaultLabelPosition() {
		return $this->default_label_pos;
	}

	function setType($type) {
		if($type == 'edit') $type = 'editview';
		$this->type = $type;
		$this->show_title_links = ($type != 'listupdate' && !$this->in_popup);
	}

    function getInPopup() {
    	return $this->in_popup;
    }

    function setInPopup($view_mode) {
        $this->in_popup = $view_mode;
        $this->show_title_links = ($view_mode) ? false : true;
    }

    function setIndicateRequired($indicate=true) {
    	$this->indicate_required = $indicate;
    }

	function getCurrentTab() {
		return $this->current_tab;
	}

	function setCurrentTab($tab) {
		$this->current_tab = $tab;
	}

	function getDefaultTab() {
		return $this->default_tab;
	}

	function setDefaultTab($tab) {
		$this->default_tab = $tab;
	}

	function setSummary($summary) {
		if(is_array($summary))
			$this->summary = new FormSummary($summary, $this->model);
		else if($summary instanceof FormSummary)
			$this->summary = $summary;
	}

	function getSummary() {
		return $this->summary;
	}


	function setFormHooks($hooks) {
		$this->form_hooks = array();
		if(! is_array($hooks))
			$hooks = array();
		$this->addFormHooks($hooks);
	}

	function addFormHooks($hooks, $override=false) {
		if(! $hooks) return;
		foreach($hooks as $k => $v)
			$this->addFormHook($k, $v, $override);
	}

	function addFormHook($name, $hook, $override=false) {
		if(! is_array($hook)) $hook = array($hook);
		if($override || ! isset($this->form_hooks[$name]))
			$this->form_hooks[$name] = $hook;
		else
			array_extend($this->form_hooks[$name], $hook);
	}


	function setFormHiddenFields($fields) {
		if(! is_array($fields))
			$fields = array();
		$this->form_hidden_fields = $fields;
	}

	function addFormHiddenFields($fields, $override=true) {
		if(! $fields) return;
		foreach($fields as $k => $v)
			if($override || ! isset($this->form_hidden_fields[$k]))
				$this->form_hidden_fields[$k] = $v;
	}

	function addRecordHiddenFields($fields, $override=true) {
		if(! $fields) return;
		if($override) $this->rec_hidden_fields = array();
		array_extend($this->rec_hidden_fields, $fields);
	}


	function setFormButtons($buttons) {
		$this->form_buttons = array();
		$this->addFormButtons($buttons);
	}

	function addFormButtons($buttons, $override=true) {
		if(! $buttons) return;
		foreach($buttons as $k => $spec) {
			unset($button);
			if($spec instanceof FormButton) {
				$k = $spec->name;
				$button = $spec;
			}
			else if(is_array($spec)) {
				$spec['form_id'] = $this->form_id;
				if(! is_int($k) && empty($spec['name'])) $spec['name'] = $k;
				$widget = array_get_default($spec, 'widget');
				if($widget) {
					$wspec = AppConfig::get_widget_spec($widget);
					if(! $wspec || $wspec['type'] != 'form_button')
						continue;
					if(isset($wspec['params']))
						$spec = array_merge($wspec['params'], $spec);
					$button = new $wspec['class']($spec, $this->model);
				}
				else {
					if(! $override && isset($this->form_buttons[$k])) {
						$this->form_buttons[$k]->updateParams($spec);
						continue;
					}
					$button = new FormButton($spec, $this->model);
				}
				$button->setLayout($this);
			}
			if(( ($override || !empty($button->replace) ) || ! isset($this->form_buttons[$k])) && isset($button))
				$this->form_buttons[$k] = $button;
		}
	}

	function processFormButtons(RowResult $result=null) {
		if($this->form_buttons) {
			foreach($this->form_buttons as $button) {
				if($result)
					$button->loadResult($result, array());
			}
			FormButton::sort_buttons($this->form_buttons, $result);
		}
	}

	function processTabs(RowResult $result=null) {
		$ltype = $this->getType();
		$currentHidden = false;
		$view_mode = ($ltype == 'view' || ($ltype == 'record' && ! $this->getDefaultEditable()));
		if($this->tabs) {
			foreach($this->tabs as &$tab) {
				if($result && $result->acl && ! empty($tab['acl'])) {
					if(empty($result->acl[$tab['acl']])) {
						$tab['hidden'] = true;
						$tab['restricted'] = true;
					}
				}
				$tab_mode = array_get_default($tab, 'mode', 'any');
				if ($view_mode && ($tab_mode !== 'view' && $tab_mode !== 'any'))
					$tab['hidden'] = true;
				if (!$view_mode && ($tab_mode !== 'edit' && $tab_mode !== 'any'))
					$tab['hidden'] = true;
				if (!$this->getEditingLayout() && !empty($tab['explicitly_hidden']))
					$tab['hidden'] = true;
				if (array_get_default($tab, 'hidden') === true && $tab['name'] == $this->current_tab)
					$currentHidden = true;
			}
			if ($currentHidden && $this->default_tab)
				$this->current_tab = $this->default_tab;
		}
	}

	function &getFormButtons() {
		return $this->form_buttons;
	}


	function addStyles($styles) {
		if(! is_array($styles))
			return;
		foreach($styles as $sty)
			$this->styles[] = $sty;
	}

	function getStyles() {
		return $this->styles;
	}

	function addScripts($scripts) {
		if(! is_array($scripts))
			return;
		foreach($scripts as $scr)
			$this->scripts[] = $scr;
	}

	function addScriptInclude($script, $priority=LOAD_PRIORITY_INCLUDE) {
		$this->scripts[] = array('file' => $script, 'priority' => $priority);
	}

	function addExtScriptInclude($script, $priority=LOAD_PRIORITY_INCLUDE) {
		$this->scripts[] = array('url' => $script, 'priority' => $priority);
	}

	function addScriptLiteral($script, $priority=LOAD_PRIORITY_FOOT) {
		$this->scripts[] = array('literal' => $script, 'priority' => $priority);
	}

	function addCSSInclude($script, $priority=LOAD_PRIORITY_INCLUDE) {
		$this->styles[] = array('file' => $script, 'priority' => $priority);
	}

	function addCSSLiteral($script, $priority=LOAD_PRIORITY_INCLUDE) {
		$this->styles[] = array('literal' => $script, 'priority' => $priority);
	}

	function addJSLanguage($module, $priority=LOAD_PRIORITY_INCLUDE) {
		$this->scripts[] = array('language' => $module, 'priority' => $priority);
	}

	function addJSModelCache($model) {
		$this->scripts[] = array('model' => $model);
	}

	function addFormInitHook($script) {
		$this->addFormHook('oninit', $script);
	}

	function addScript($script) {
		$this->scripts[] = $script;
	}

	function getScripts() {
		return $this->scripts;
	}


	function addTabs($tabs) {
		if(! is_array($tabs))
			return;
		foreach($tabs as $k => &$tab) {
			if(! isset($tab['name'])) {
				if(is_string($k))
					$tab['name'] = $k;
				else
					continue;
			}
			$this->addTab($tab);
		}
	}

	function addTab($tab) {
		$this->tabs[] = $tab;
	}

	function getTabs() {
		return $this->tabs;
	}


	function addSections($sects) {
		if(! is_array($sects))
			return;
		foreach($sects as &$sect)
			$this->addSection($sect);
	}

	function addSection($section) {
		if(is_string($section))
			$section = array('name' => $section, 'type' => 'section');
		if(is_array($section) && ! isset($section['name']) && isset($section['id']))
			$section['name'] = $section['id'];
		if(is_array($section) && ! isset($section['type']))
			$section['type'] = 'section';
		$section = $this->loadElement($section);
		if($section instanceof FormSection) {
			if(empty($section->id))
				$section->id = 'section-'.($this->section_index ++);
			return $this->addElement($section);
		}
	}

	function extendSections($ext_sections) {
		$sections = $this->elements;

		if (is_array($ext_sections))
			foreach ($ext_sections as $section) {
				if (is_string($section))
					$section = array('name' => $section, 'type' => 'section');
				if (is_array($section) && !isset($section['name']) && isset($section['id']))
					$section['name'] = $section['id'];
				if (is_array($section) && !isset($section['type']))
					$section['type'] = 'section';
				$el = $this->loadElement($section);

				$mode = $find = null;
				foreach (array('before', 'after', 'replace', 'remove') as $m) {
					if (isset($section[$m])) {
						$mode = $m;
						$find = $section[$m];
						break;
					}
				}
				if ($find) {
					$pos = 0;
					$new_sections = $sections;
					foreach ($sections as $k => $s) {
						if ($k == $find) {
							if ($mode == 'before' || $mode == 'after'){
								if($mode == 'after') $pos++;

								$new_sections = array_slice($sections, 0, $pos, true) +
									array($el->id => $el) +
									array_slice($sections, $pos, NULL, true);
							}
							else if ($mode == 'replace'){
								$new_sections[$k] = $el;
							}
							else if ($mode == 'remove')
								unset($new_sections[$k]);
							$find = null;
							break;
						}
						$pos++;
					}
					$sections = $new_sections;
				} else {
					$sections[$el->id] = $el;
				}
			}
		$this->elements = $sections;
	}

	function extendElements(array $elts) {
		foreach($elts as $elt) {
			if(! is_array($elt)) {
				$elt = array('name' => $elt);
			}
			$find = $mode = null;
			$span = 0;
			foreach(array('before', 'after', 'replace', 'remove') as $m) {
				if(isset($elt[$m])) {
					$mode = $m;
					$find = $elt[$m];
					unset($elt[$m]);
				}
			}
			if(isset($elt['span'])) {
				$span = $elt['span'];
				unset($elt['span']);
			}
			if($find) {
				$fpath = $this->findElementPath($find);
				if($fpath) {
					if($mode == 'remove')
						$this->removeElement($fpath);
					else {
						if(isset($elt['elements']))
							$this->insertElements($fpath, $elt['elements'], $mode, $span);
						else
							$this->insertElements($fpath, array($elt), $mode, $span);
					}
				}
			} else
				$this->addElement($elt);
		}
	}

	// stub to allow findElementPath usage in DetailManager to work for summary fields
	// we should probably add the summary elt to elements instead, but that would require more changes
	function findElementPath($elt_name) {
		$id = parent::findElementPath($elt_name);
		if(! $id && $this->summary && ($sub_id = $this->summary->findElementPath($elt_name))) {
			if(! is_array($sub_id)) $sub_id = array($sub_id);
			array_unshift($sub_id, $this->summary->id);
			$id = $sub_id;
		}
		return $id;
	}

	function loadWidgetSpec($widget) {
		$wspec = parent::loadWidgetSpec($widget);
		if($wspec['type'] != 'section')
			return false;
		return $wspec;
	}

	function createElementType($type, $spec=null) {
		if(! $spec) $spec = array();
		if($this->form_id && ! isset($spec['form_id']))
			$spec['form_id'] = $this->form_id;
		if(! empty($this->skip_field_lookup))
			$spec['skip_field_lookup'] = $this->skip_field_lookup;
		if($type == 'section') {
			return new FormTableSection($spec, $this->model);
		}
	}


	function getSections() {
		return $this->elements;
	}

	function getSection($id) {
		return array_get_default($this->elements, $id);
	}

	function getSectionIds() {
		return array_keys($this->elements);
	}


	function getRequiredFields() {
		$ret = parent::getRequiredFields();
		foreach($this->form_buttons as $b) {
			$rf = $b->getRequiredFields();
			if(is_array($rf))
				array_extend($ret, $rf);
		}
		if($this->summary)
			array_extend($ret, $this->summary->getRequiredFields());
		return array_filter($ret);
	}

	function translate($module) {
		parent::translate($module);
		foreach($this->form_buttons as $button)
			$button->translate($module);
		if($this->summary)
			$this->summary->translate($module);
	}

	// handle updates to complex widgets

	function loadUpdateRequest(RowUpdate $update, array $input) {
		parent::loadUpdateRequest($update, $input);
		foreach($this->form_buttons as &$b)
			$b->loadUpdateRequest($update, $input);
		if($this->summary)
			$this->summary->loadUpdateRequest($update, $input);
	}

	function validateInput(RowUpdate $update) {
		$ret = parent::validateInput($update);
		if(isset($ret) && ! $ret)
			return false;
		foreach($this->form_buttons as &$b) {
			$ret = $b->validateInput($update);
			if(isset($ret) && ! $ret)
				return false;
		}
		if($this->summary) {
			$ret = $this->summary->validateInput($update);
			if(isset($ret) && ! $ret)
				return false;
		}
	}

	function beforeUpdate(RowUpdate $update) {
		parent::beforeUpdate($update);
		foreach($this->form_buttons as &$b)
			$b->beforeUpdate($update);
		if($this->summary)
			$this->summary->beforeUpdate($update);
	}

	function afterUpdate(RowUpdate $update) {
		parent::afterUpdate($update);
		foreach($this->form_buttons as &$b)
			$b->afterUpdate($update);
		if($this->summary)
			$this->summary->afterUpdate($update);
	}

	// form layout editor (could move to a HtmlFormLayout subclass)
	function getEditingLayout() {
		return $this->editing_layout;
	}

	function addDropTarget($id) {
		$this->drop_targets[] = $id;
	}

	function addDisabledLayoutField($id) {
		$this->disabled_layout_fields[] = $id;
	}

	function getDisabledLayoutFields() {
		return $this->disabled_layout_fields;
	}

	function getDropTargets() {
		return $this->drop_targets;
	}

	function addEditableSection($section) {
		$this->editable_sections[$section['id']] = $section;
	}

	function getEditableSections() {
		return $this->editable_sections;
	}

	function expandFormDef(array $def) {
		$ret = array('sections' => array(), 'fields' => array());
		if(isset($def['summary']) && is_array($def['summary'])) {
			foreach($def['summary'] as $sub => $sect) {
				if(is_array($sect) && isset($sect['fields']) && is_array($sect['fields'])) {
					foreach($sect['fields'] as $k => $v) {
						if(is_string($v) && $v) {
							$name = $v;
							$v = array();
						}
						else if(is_array($v) && array_get_default($v, 'type') == 'section') {
							//$this->expandSectionDef($v, $ret);
							continue;
						}
						else if(is_array($v) && ! empty($v['name'])) {
							$name = $v['name'];
							unset($v['name']);
						}
						else
							continue;
						if(! isset($ret['fields'][$name]))
							$ret['fields'][$name] = $v;
					}
				}
			}
		}
		if(isset($def['sections'])) {
			foreach($def['sections'] as $idx => $sect) {
				if(is_array($sect))
					$this->expandSectionDef($sect, $ret);
			}
		}
		return $ret;
	}

	function expandSectionDef(array $def, &$ret) {
		$id = array_get_default($def, 'id');
		if(! $id) $id = is_countable($ret['sections']) ? count($ret['sections']) : 0;
		if(isset($def['elements'])) {
			foreach($def['elements'] as $k => $v) {
				if(is_string($v) && $v) {
					$name = $v;
					$v = array();
				}
				else if(is_array($v) && (array_get_default($v, 'type') == 'section' || !empty($v['section']))) {
					$this->expandSectionDef($v, $ret);
					continue;
				}
				else if(is_array($v) && ! empty($v['name'])) {
					$name = $v['name'];
					unset($v['name']);
				}
				else
					continue;
				//if(! isset($ret['fields'][$name]))
					$ret['fields'][$name] = $v;
			}
		}
		unset($def['elements']);
		$ret['sections'][$id] = $def;
	}

	function updateFromRequest(&$form_def, &$lang, $request)
	{
		$json = getJSONObj();
		$request = $json->decode($request['le_encoded_data']);
		$new_def = array();
		$specs = $this->expandFormDef($form_def);

		$header = $subheader = $notes = null;
		$stubs = array();

		foreach ($request['sections_order'] as $sId) {
			$section = $request['sections'][$sId];
			$isHeader = ($sId == '_form_header' || $sId == '_form_subheader' || $sId == '_form_notes');
			if ($isHeader) {
				$header_v = substr($sId, 6);
				if(! $$header_v) {
					if (isset($form_def['summary'][$header_v]))
						$$header_v = $form_def['summary'][$header_v];
					else
						$$header_v = array();
				}
				$hdr =& $$header_v;
				$hdr['fields'] = array();
				foreach ($section['rows']['row_' . $sId] as $cId => $elt) {
					$lId  = 'LBL_ ' . str_replace('-', '_', $sId) . '_' . $cId;
					$eltRef = $request['elements'][$elt['id']];
					$name = $eltRef['name'];
					if($name && isset($specs['fields'][$name])) {
						$newElt = $specs['fields'][$name];
					} else
						$newElt = array();
					if(! empty($eltRef['label_modified']) && $name) {
						$fdef = $this->model->getFieldDefinition($name);
						if (isset($fdef['vname'])) {
							unset($newElt['label']);
							$lang[$fdef['vname']] = $eltRef['label'];
						} else {
							$newElt['label'] = $eltRef['label'];
						}
					}
					if($newElt && $name) {
						$newElt['name'] = $name;
					} else
						$newElt = $name;
					if($newElt)
						$hdr['fields'][] = $newElt;
				}
			} else {
				if(isset($specs['sections'][$sId]))
					$newSection = $specs['sections'][$sId];
				else
					$newSection = array('id' => $sId);
				$newSection['elements'] = array();
				if(isset($section['columns']))
					$newSection['columns'] = $section['columns'];
				if (! empty($section['title_modified']) && isset($section['title']))
					$newSection['label'] = $section['title'];
				if (isset($section['widget']))
					$newSection['widget'] = $section['widget'];
				if (isset($section['tab']))
					$newSection['tab'] = $section['tab'];
				foreach ($section['rows_order'] as $rId) {
					$row = $section['rows'][$rId];
					foreach ($row as $cId => $elt) {
						$lId  = 'LBL_ ' . str_replace('-', '_', $sId) . '_' . $rId .  '_' . $cId;
						$eltRef = $request['elements'][$elt['id']];
						$name = $eltRef['name'];
						if($name && isset($specs['fields'][$name])) {
							$newElt = $specs['fields'][$name];
						} else
							$newElt = array();
						if (array_get_default($elt, 'colspan', 1) != 1) {
							$newElt['colspan'] = $elt['colspan'];
						} else
							unset($newElt['colspan']);
						if(! empty($eltRef['label_modified']) && $name) {
							$fdef = $this->model->getFieldDefinition($name);
							if (isset($fdef['vname'])) {
								unset($newElt['label']);
								$lang[$fdef['vname']] = $eltRef['label'];
							} else {
								$newElt['label'] = $eltRef['label'];
							}
						}
						if($newElt && $name)
							$newElt['name'] = $name;
						else
							$newElt = $name;
						$newSection['elements'][] = $newElt;
						if(! empty($elt['is_section']) || array_get_default($elt, 'type') === 'section')
							$stubs[] = array($sId, count($newSection['elements'])-1, $eltRef['name']);
					}
				}
				$new_def[$sId] = $newSection;
			}
		}

		// replace stubs in order to allow nested sections
		foreach($stubs as $stub) {
			list($sId, $idx, $subId) = $stub;
			if(isset($new_def[$sId]['elements'][$idx])) {
				if(isset($new_def[$subId])) {
					$new_def[$subId]['type'] = 'section';
					$new_def[$sId]['elements'][$idx] = $new_def[$subId];
					unset($new_def[$subId]);
				} else {
					unset($new_def[$sId]['elements'][$idx]);
				}
			}
		}

		$form_def['sections'] = array_values($new_def);
		$id_prefix = uniqid('');
		$id_idx = 0;
		foreach ($form_def['sections'] as &$s) {
			if (isset($s['id'])) {
				if (substr($s['id'], 0, 7) == 'nEwUiD-')
					$s['id'] = $id_prefix . ($id_idx++);
			}
		}
		unset($s);
		if ($header)
			$form_def['summary']['header'] = $header;
		if ($subheader)
			$form_def['summary']['subheader'] = $subheader;
		if ($notes)
			$form_def['summary']['notes'] = $notes;
		unset($form_def['ext_elements']);
		unset($form_def['ext_sections']);
	}

	function addAllFields($row_result=null)
	{
		if(! $row_result || ! $this->model) return;
		$fields = $this->model->getFieldDefinitions();
		$dfields = $this->model->getDisplayFieldDefinitions();
		foreach ($dfields as $f => $def) {
			if (isset($def['type']) && $def['type'] == 'address')
				$fields[$f] = $def;
		}
		foreach ($fields as $name => $def) {
			if (isset($def['multi_select_group'])) continue;
			if ($name == 'deleted') continue;
			if ($def['type'] == 'id') continue;
			if ($def['type'] == 'base_currency') continue;
			$def['form_id'] = $this->form_id;
			$f = new FormField($def, $this->model);
			$f->translate($row_result->module_dirs[0]);
			if (! isset($f->label)) continue;
			$source = isset($def['related_value_orig']) ? $def['related_value_orig']['source']['type'] : $def['source']['type'];
			$this->availableFields[] = array('name' => $f->name, 'label' => $f->label, 'source' => $source, 'ui' => array_get_default($def, 'ui', true));
		}
		usort($this->availableFields, array($this, 'sortByCustom'));
	}

	function sortByCustom($a, $b)
	{
		$ta = $a['source'];
		$tb = $b['source'];
		if ($ta == 'custom_field' && $tb != 'custom_field') return -1;
		if ($ta != 'custom_field' && $tb == 'custom_field') return 1;
		return strcmp($a['label'], $b['label']);
	}

	function getAvailableFields()
	{
		return $this->availableFields;
	}

	function addEditElement($id, $elt)
	{
		$this->editElements[$id] = $elt;
	}

	function getEditElements()
	{
		return $this->editElements;
	}

}

?>
