<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormElement.php');

class FormSection extends FormElement {
	var $tab;
	var $toggle_display;
	var $skip_field_lookup;
	var $collapsible;
	var $collapsed;

	var $section_index = 0;
	var $element_index = 0;

	function init($params=null, $model=null) {
		parent::init($params, $model);
		if(isset($params['id']))
			$this->id = $params['id'];
		else
			$this->id = $this->name;
		foreach(array('tab', 'toggle_display', 'skip_field_lookup', 'collapsible', 'collapsed') as $k)
			$this->$k = array_get_default($params, $k, $this->$k);
		if(isset($params['elements']))
			$this->addElements($params['elements']);
	}

	function getTab() {
		return $this->tab;
	}

	function getToggleDisplay() {
		return $this->toggle_display;
	}

	function getCollapsible() {
		return $this->collapsible;
	}

	function getCollapsed() {
		return $this->collapsed;
	}

	function getRequiredFields() {
		$fields = parent::getRequiredFields();
		if($this->toggle_display && is_array($this->toggle_display))
			$fields[] = $this->toggle_display['name'];
		return $fields;
	}

	function loadElement($elt) {
		$elt = parent::loadElement($elt);
		if($elt instanceof FormSection) {
			if(empty($elt->id))
				$elt->id = 'section-' . ($this->section_index ++);
			$ok = 1;
		}
		else if($elt) {
			if(empty($elt->id))
				$elt->id = 'field-' . ($this->element_index ++);
			$ok = 1;
		}
		else
			$elt = null;
		return $elt;
	}

	function addSection($section) {
		if(is_string($section))
			$section = array('name' => $section, 'type' => 'section');
		$section = $this->loadElement($section);
		if($section instanceof FormSection)
			return $this->addElement($section);
	}

	function renderPdf(PdfFormGenerator $gen, $row_result, array $parents, array $context)
	{
	}

}

?>
