<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormSection.php');

class FormSummary extends FormSection {
	var $collapsible = false;
	
	var $image;
	var $header;
	var $subheader;
	var $meta;
	var $owner;
	var $activity_date, $activity_date_name;
	var $notes;
	var $actions = array();
	var $teams;
	var $social;
	var $email_status;
	var $phone_status;
	
	function init($params=null, $model=null) {
		if(! $params || empty($params['name']))
			$params['name'] = '_summary';
		parent::init($params, $model);
		if(isset($params['image']))
			$this->loadImage($params['image']);
		if(isset($params['header']))
			$this->loadHeader($params['header']);
		if(isset($params['subheader']))
			$this->loadSubHeader($params['subheader']);
		if(isset($params['owner']))
			$this->loadOwner($params['owner']);
		if(isset($params['meta']))
			$this->loadMeta($params['meta']);
		if(isset($params['teams']))
			$this->loadTeams($params['teams']);
		if(isset($params['actions']))
			$this->loadActions($params['actions']);
		if(isset($params['notes']))
			$this->loadNotes($params['notes']);
		if(isset($params['activity_date']))
			$this->activity_date_name = $params['activity_date'];
		$this->initDefaults();
	}
	
	function loadImage($info) {
		$this->image = $this->loadElement($info);
		if($this->image->missing_field)
			$this->image = null;
		if($this->image) $this->addElement($this->image);
	}
	
	function loadFieldDefs($info) {
		$ret = array();
		if(is_array($info)) {
			foreach($info as $cat => $f) {
				$idx = $this->addElement($f);
				if(isset($idx)) $ret[$cat] = $idx;
			}
		} else if($info) {
			$idx = $this->addElement($info);
			if(isset($idx)) $ret[] = $idx;
		}
		return $ret;
	}
	
	function loadHeader($info) {
		if(! is_array($info)) $info = array();
		if(isset($info['fields']))
			$info['fields'] = $this->loadFieldDefs($info['fields']);
		$this->header = $info;
	}

	function loadSubHeader($info) {
		if(! is_array($info)) $info = array();
		if(isset($info['fields']))
			$info['fields'] = $this->loadFieldDefs($info['fields']);
		$this->subheader = $info;
	}
	
	function loadMeta($info) {
		$meta = array('fields' => array());
		if(! is_array($info)) $info = array();
		if(isset($info['fields'])) {
			$info['fields'] = $this->loadFieldDefs($info['fields']);
			array_extend($meta['fields'], $info['fields']);
		}
		foreach(array('date_entered', 'date_modified') as $d) {
			$idx = $this->addElement($d);
			if(isset($idx)) {
				$meta['fields'][] = $idx;
				$f = $this->getElement($idx);
				if($f) {
					if($f->vname == 'LBL_DATE_ENTERED')
						$f->vname = 'LBL_DATE_ENTERED_SHORT';
					if($f->vname == 'LBL_DATE_MODIFIED')
						$f->vname = 'LBL_DATE_MODIFIED_SHORT';
				}
			}
		}
		$this->meta = $meta;
	}
	
	function loadOwner($info) {
		$owner = array('fields' => array());
		if(! is_array($info)) $info = array();
		if(empty($info['disabled']))
		foreach(array('assigned_user') as $u) {
			$spec = ['name' => $u];
			if(isset($info['fields'][$u]))
				array_extend($spec, $info['fields'][$u]);
			$idx = $this->addElement($spec);
			if(isset($idx)) {
				$owner['fields'][] = $idx;
			}
		}
		$this->owner = $owner;
	}
	
	function loadTeams($info) {
		if(is_array($info) && ! empty($info['disabled']))
			$this->teams = false;
		else
			$this->teams = $this->addElement(array('name' => 'teams', 'widget' => 'EditTeamsWidget'));
	}
	
	function loadNotes($info) {
		if(! is_array($info)) $info = array();
		if(isset($info['fields']))
			$info['fields'] = $this->loadFieldDefs($info['fields']);
		$this->notes = $info;
	}
	
	function loadActions($info) {
		$this->actions = array();
		if(is_array($info)) {
			foreach($info as $cat => $acts) {
				if(! is_array($acts)) $acts = array();
				if(isset($acts['fields']))
					$acts['fields'] = $this->loadFieldDefs($acts['fields']);
				$this->actions[$cat] = $acts;
			}
		}
		if(isset($this->actions['social'])) {
			$this->social = $this->addElement(array('name' => 'social', 'widget' => 'SocialAccountsWidget'));
		}
		if(! empty($this->actions['email'])) {
			$this->email_status = $this->addElement(array('name' => 'email_status', 'widget' => 'ContactStatusInput'));
		}
		if(! empty($this->actions['phone'])) {
			$this->phone_status = $this->addElement(array('name' => 'phone_status', 'widget' => 'ContactStatusInput'));
		}
	}
	
	function initDefaults() {
		if((! $this->header || empty($this->header['fields'])) && $this->model) {
			$dname = $this->model->getDisplayName();
			if(! $dname) $dname = $this->model->getDisplayField('name', true);
			if($dname) {
				if(array_get_default($dname, 'type') == 'prefixed' && isset($dname['fields'])) {
					$this->header['fields'] = $this->loadFieldDefs($dname['fields']);
				} else {
					$def = $this->addElement($dname);
					if(isset($def)) {
						$this->header['fields'] = array($def);
						$e = $this->getElement($def);
						if($e->type == 'name' && ! $e->width) $e->width = 30;
					}
				}
			}
		} else if($this->model && is_countable($this->header['fields']) && count($this->header['fields']) == 1) {
			$f = $this->getElement($this->header['fields'][0]);
			if($f && $f instanceof FormField && ! $f->width)
				$f->width = 30;
		} else if($this->model && is_countable($this->header['fields']) && count($this->header['fields']) == 2) {
			$f1 = $this->getElement($this->header['fields'][0]);
			$f2 = $this->getElement($this->header['fields'][1]);
			if($f1 && $f1 instanceof FormField && $f1->type == 'item_number') {
				if($f2 && $f2 instanceof FormField && ! $f2->width)
					$f2->width = 30;
			}
		}

		if($this->subheader && ! empty($this->subheader['fields'])) {
			foreach($this->subheader['fields'] as $f) {
				$e = $this->getElement($f);
				if($e && $e instanceof FormField && $e->getType() == 'varchar')
					if(! $e->width && $e->max_length >= 35)
						$e->width = 18; // $e->style = 'width: 15em';
			}
		}
		if(! $this->meta)
			$this->loadMeta(null);
		if($this->meta && isset($this->meta['fields'])) {
			foreach($this->meta['fields'] as $f) {
				$e = $this->getElement($f);
				if($e && $e->getType() == 'multienum') {
					if(! $e->width) $e->width = 18;
					if(empty($e->display_rows)) $e->display_rows = 4;
				}
				else if($e && $e->getType() == 'ref') {
					if(! $e->width) $e->width = 18;
				}
			}
		}
		if(! $this->owner)
			$this->loadOwner(null);
		if($this->owner && isset($this->owner['fields'])) {
			foreach($this->owner['fields'] as $f) {
				$e = $this->getElement($f);
				if($e && $e->getType() == 'multienum') {
					if(! $e->width) $e->width = 18;
					if(empty($e->display_rows)) $e->display_rows = 4;
				}
				else if($e && $e->getType() == 'ref') {
					if(! $e->width) $e->width = 18;
				}
			}
		}
		if($this->activity_date_name)
			$this->activity_date = $this->addElement($this->activity_date_name);
		else
			$this->activity_date = $this->addElement(array('name' => 'last_activity_date', 'vname' => 'LBL_LIST_LAST_ACTIVITY'));

		if(! isset($this->teams))
			$this->loadTeams(null);
		
		foreach($this->elements as &$e) {
			if(! $e->tabindex) $e->tabindex = $this->tabindex;
		}
	}
	
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {		
		global $current_user;
		$form_obj = $gen->getFormObject();
		$style = array();
		$style = implode('; ', $style);
		$editable = $gen->getLayout()->getDefaultEditable();
		$edit_layout = $gen->getLayout()->getEditingLayout();
		$layout = $gen->getLayout();

		$ctxedit = $context;
		$ctxedit['editable'] = $editable;
		$ctxedit['summary'] = true;
		$subpath = $parents;
		$subpath[] =& $this;
		$cls = ($editable && !$edit_layout) ? 'editable' : '';
		$module = $row_result->getModuleDir();
		$pk = $row_result->getPrimaryKeyValue();
		$json = getJSONobj();
		$sep = translate('LBL_SEPARATOR', 'app');
		
		$html = "<div class=\"form-section summary-body $cls\" style=\"$style\" id=\"le_section_{$this->id}\">\n";
		$html .= "<div class=\"row form-row summary-row\">\n";
		
		if($this->image && $this->checkVisible($this->image, $editable)) {
			$irs = 2;
			if(AppConfig::is_mobile()) $irs++;
			$html .= "<div class=\"column summary-image\" id=\"le_section_{$this->id}_image\">";
			$imgctx = $ctxedit;
			if($edit_layout) {
				$row_result->assign($this->image->name, null, true);
				$imgctx['editable'] = 1;
				$this->image->disabled = true;
			}
			$html .= $this->image->renderHtml($gen, $row_result, $subpath, $imgctx);
			$html .= "</div>";
		}
		
		$html .= '<div class="column summary-main">';
		$html .= '<div class="row">';
		$html .= '<div class="column summary-left">';
		if($this->header && ! empty($this->header['fields'])) {
			$h = $this->joinHeaderLine($gen, $row_result, $subpath, $ctxedit, $this->header, '_form_header');
			$line = implode('', $h);
			if ($edit_layout) {
				$onclick = ' onclick="SUGAR.le.editField(this)" ';
				$dtId = 'hdt__form_header'; 
				$innerId = 'hlw__form_header'; 
				$line = '<div ' . $onclick . ' class="input-field input-field-group state-label"><div id="' . $innerId . '" class="input-label">' . $line . '</div></div>';
				$layout->addDropTarget($dtId);
				$line = "<div id=\"hdd__form_header\" style=\"display:inline-block\" class=\"edit_layout_wrapper\" >$line</div>";
			}
			if(strlen($line)) {
				if(! $editable || $edit_layout) $line = '<h3>' . $line . '</h3>';
				$html .= '<div class="summary-header" id="_form_header">';
				$html .= $line;
				$html .= '</div>';
			}
		}

		if($this->subheader && ! empty($this->subheader['fields'])) {
			$h = $this->joinHeaderLine($gen, $row_result, $subpath, $ctxedit, $this->subheader, '_form_subheader');
			$line = implode('', $h);
			if ($edit_layout) {
				$dtId = 'hdt__form_subheader'; 
				$innerId = 'hlw__form_subheader'; 
				$layout->addDropTarget($dtId);
				$onclick = ' onclick="SUGAR.le.editField(this)" ';
				$line = '<div ' . $onclick . ' class="input-field input-field-group state-label"><div id="' . $innerId . '"class="input-label">' . $line . '</div></div>';
				$line = "<div id=\"hdd__form_subheader\" style=\"display:inline-block\" class=\"edit_layout_wrapper\" >$line</div>";
			}
			if(strlen($line)) {
				if(! $editable || $edit_layout) $line = '<h4>' . $line . '</h4>';
				$html .= '<div class="summary-subheader" id="_form_subheader">';
				$html .= $line;
				$html .= '</div>';
			}
		}

		$header_fields = array();
		if (!empty($this->header['fields']))
			foreach ($this->header['fields'] as $f) {
				$e = $this->getElement($f);
				$header_fields[] = $e->getName();
			}
		if (!empty($this->subheader['fields']))
			foreach ($this->subheader['fields'] as $f) {
				$e = $this->getElement($f);
				$header_fields[] = $e->getName();
			}
		if (! empty($this->notes['fields']))
			foreach ($this->notes['fields'] as $f) {
				$e = $this->getElement($f);
				$header_fields[] = $e->getName();
			}

		foreach ($this->elements as $el) {
			$f = $el->getName();
			if (!in_array($f, $header_fields))
				$gen->getLayout()->addDisabledLayoutField($f);
		}
		

		$rel_acts = $this->actions;
		$check = array(array(
			'email' => translate('LBL_EMAIL', 'Accounts'),
			'phone' => translate('LBL_PHONE', 'Accounts'),
			'email_status' => translate('LBL_EMAIL_STATUS', 'Contacts'),
			'phone_status' => translate('LBL_PHONE_STATUS', 'Contacts'),			
			'web' => translate('LBL_WEBSITE', 'Accounts'),
			'fax' => translate('LBL_PHONE_FAX', 'Accounts'),
			'skype' => translate('LBL_SKYPE_ID', 'Contacts'),
		));
		$seen = array();
		$editrows = array();
		foreach($check as $check_cats) {
			$contact = array();
			foreach($check_cats as $check_cat => $check_label) {
				if(! empty($rel_acts[$check_cat]) && ! empty($rel_acts[$check_cat]['fields'])) {
					foreach($rel_acts[$check_cat]['fields'] as $cat => $f) {
						$e = $this->getElement($f);
						if(! $e || ! $this->checkVisible($e, $editable)) continue;
						if ($edit_layout)
							continue;
						if(! is_int($cat) && ($cat == 'primary' || $cat == 'alternate' || $cat == 'fax'))
							$e->placeholder = $this->categoryLabel($cat);
						if($e->getType() == 'email' || $e->getType() == 'url')
							$e->style = 'width: 18em';
						else if($e->getType() == 'phone' || $e->getType() == 'fax')
							$e->style = 'width: 14em';
						$value = $e->renderHtml($gen, $row_result, $subpath, $ctxedit);
						if($value) {
							if(is_int($cat) && $check_cat == 'fax' && !$editable) $cat = 'fax';
							if(is_int($cat) && $check_cat == 'web' && !$editable) $cat = 'web';
							if(is_int($cat)) $contact[][] = $value;
							else $contact[$cat][] = $value;
							$seen[$check_cat] = 1;
						}
					}
					$stat = $check_cat . '_status';
					if(($check_cat == 'email' || $check_cat == 'phone') && isset($this->$stat)) {
						$e = $this->getElement($this->$stat);
						if($e && ! empty($seen[$check_cat])) {
							$value = $e->renderHtml($gen, $row_result, $subpath, $ctxedit);
							$contact[][] = $value;
						}
					}
				}
				if($editable && $contact) {
					$cell = '<p class="form-label">' . $check_label . '</p>';
					foreach($contact as $cat => $vals) {
						$lbl = '';
						if(is_string($cat) && $cat !== 'primary' && $cat !== 'alternate' && $cat !== 'fax')
							$lbl = ' &nbsp;<span class="form-label inline-label">' . $this->categoryLabel($cat) . '</span>';
						$cell .= '<div class="summary-contact form-value">' . $vals[0] . "$lbl</div>";
					}
					$contact = array();
					$editrows[] = $cell;
				}
			}
			foreach($contact as $cat => $vals) {
				$vals = array_filter($vals);
				if(! $vals) continue;
				$row = implode(' &nbsp;&nbsp; ', $vals);
				if(! is_int($cat) && $cat != 'primary' && (count($vals) != 1 || $cat != 'web')) {
					$lbl = $this->categoryLabel($cat);
					$row .= " &nbsp; <span class=\"form-label inline-label\">$lbl</span>";
				}
				$html .= '<div class="summary-contact form-value">' . $row . '</div>';
			}
		}
		if($editrows) {
			foreach($editrows as $i => $r) {
				if($i && $i % 2 == 0) $html .= '<br>';
				$html .= ' <div class="summary-header-entry form-entry contact-info">' . $r . '</div>';
			}
		}
		
		if($this->notes && ! empty($this->notes['fields'])) {
			$html .= '<div class="summary-notes" id="_form_notes">';
			$h = $this->joinHeaderLine($gen, $row_result, $subpath, $ctxedit, $this->notes, '_form_notes');
			$html .= implode('', $h);
			$html .= '</div>';
		}

		$buttons = array();
		if(! $editable) {
			$user_mods = get_user_module_list(null, true);
			$qc = array(
				'email' => array('link_name' => 'emails', 'module' => 'Emails', 'vname' => 'LNK_COMPOSE_EMAIL'),
				'meeting' => array('link_name' => 'meetings', 'module' => 'Meetings', 'vname' => 'LBL_SCHEDULE_MEETING_BUTTON_LABEL'),
				'call' => array('link_name' => 'calls', 'module' => 'Calls', 'vname' => 'LBL_SCHEDULE_CALL_BUTTON_LABEL'),
				'task' => array('link_name' => 'tasks', 'module' => 'Tasks', 'vname' => 'LNK_NEW_TASK'),
				'note' => array('link_name' => 'notes', 'module' => 'Notes', 'vname' => 'LNK_NEW_NOTE'),
			);
			foreach($qc as $cat => $qc_info) {
				$qc_spec = array_get_default($rel_acts, $cat, array());
				if(! empty($qc_spec['disabled'])) continue;
				if(! ACLController::checkAccess($qc_info['module'], 'create')) continue;
				if(AppConfig::is_mobile() && ! AppConfig::is_mobile_module($qc_info['module'])) continue;
				if(! isset($user_mods[$qc_info['module']])) continue;
				$ldef = $this->model->getLinkRelationship($qc_info['link_name']);
				$qcl_name = array_get_default($qc_info, 'layout', 'QuickCreate');
				$qc_layout = AppConfig::setting("views.detail.{$qc_info['module']}.edit.$qcl_name");
				$pbean = AppConfig::module_primary_bean($qc_info['module']);
				if($ldef && $ldef['right']['model'] == $pbean) {
					if($qc_layout && ! AppConfig::is_mobile()) {
						$qparams = array(
							'target_name' => 'subpanel_by_name',
							'target_id' => array('history', 'activities'),
							'primary_name' => 'related_id',
							'return_panel' => 'activities',
							'parent_type' => $module,
							'parent_id' => $pk,
							'return_panel' => 'activities',
							'return_module' => $module,
							'return_action' => 'DetailView',
							'return_record' => $pk,
						);
						if($qc_info['module'] == 'Tasks' && $module == 'Users') {
							$qparams['parent_type'] = null;
							$qparams['parent_id'] = null;
							$qparams['assigned_user_id'] = $pk;
						}
						if($qc_info['module'] == 'Emails') {
							$qparams['dialog_params']['keep_in_document'] = true;
						}
						$qc_params = $json->encode($qparams);
						$perform = "SUGAR.ui.openQuickCreate('{$qc_info['module']}', $qc_params)";
					}
					else {
						$qparams = array(
							'module' => $qc_info['module'],
							'action' => 'EditView',
							'parent_type' => $module,
							'parent_id' => $pk,
							'return_panel' => 'activities',
							'return_module' => $module,
							'return_action' => 'DetailView',
							'return_record' => $pk,
						);

						if($qc_info['module'] == 'Emails' && $module == 'Users') {
							$qparams['parent_type'] = null;
							$qparams['parent_id'] = null;
							$qparams['user_id'] = $pk;
						}
						$url = javascript_escape('index.php?' . encode_query_string($qparams));
						$perform = "SUGAR.util.loadUrl('$url');";
					}

					if($qc_info['module'] == 'Emails') {
						$client = $current_user->getPreference('email_link_type');
						if (!$client)
							$client = AppConfig::setting('email.default_client');
						if (AppConfig::is_mobile())
							if(!AppConfig::is_mobile_module('Emails'))
								$client = '';
						if ($client != 'sugar') {
							require_bean('Email');
							$addr = Email::load_to_addr($qparams);
							$perform = "window.location.href=" . $json->encode('mailto:' . $addr['to']) . ";";
						}
					}

					$btitle = translate($qc_info['vname'], 'Activities');
					$btn = array('icon' => 'input-icon theme-icon module-'.$qc_info['module'], 'perform' => $perform, 'title' => $btitle);
					if($cat == 'call' && isset($this->phone_status)) {
						$e = $this->getElement($this->phone_status);
						if($e->opt_out_name && $row_result->getField($e->opt_out_name)) {
							$btn['disabled'] = true;
						}
					}
					if($cat == 'email' && isset($this->email_status)) {
						$e = $this->getElement($this->email_status);
						if($e->opt_out_name && $row_result->getField($e->opt_out_name)) {
							$btn['disabled'] = true;
						}
					}
					$buttons[] = $btn;
				}
			}
		}
		$brow = '';
		if (!$edit_layout) {
			foreach($buttons as $b) {
				$dis = (! empty($b['disabled']) ? ' disabled="disabled"' : '');
				$icon = $b['icon'];
				if(! $dis) $icon .= ' active';
				$brow .= '<div class="inline-elt"><button class="input-button activity-button muted" type="button" title="'.to_html($b['title']).'" onclick="' . to_html($b['perform']) . '"' . $dis . '><div class="' . $icon . '"></div></button></div>';
			}
			if(isset($rel_acts['social']) && $this->social && ! $editable) {
				require_once('modules/SocialAccounts/SocialAccount.php');
				$sa = new SocialAccount(array('related_type' => $row_result->getModuleDir(), 'related_id' => $row_result->getPrimaryKeyValue()));
				$icons = $sa->getIconsList(false, '');
				foreach($icons as $i)
					$brow .= '<div class="inline-elt">'.$i.'</div>';
			}

			if($brow) {
				$brow = '<div class="buttons-inline">' . $brow . '</div>';
				$brow = '<div class="buttons-bar summary-buttons">' . $brow . '</div>';
			}
			if(isset($rel_acts['social']) && $editable && $this->social) {
				$sw = $this->getElement($this->social);
				if($sw) {
					$brow .= '<div class="form-entry">' . $sw->renderHtmlCell($gen, $row_result) . '</div>';
				}
			}
		}

		if($brow) $html .= $brow;

		$html .= '</div>';
				
		$rhs = '';
		if(!$edit_layout && $this->owner && ! empty($this->owner['fields'])) {
			foreach($this->owner['fields'] as $f) {
				$e = $this->getElement($f);
				if(! $e || ! $this->checkVisible($e, $editable)) continue;
				$v = $e->renderHtml($gen, $row_result, $subpath, $ctxedit);
				if(strlen($v ?? '')) {
					if($editable) {
						$l = $e->getLabel(true);
						if($e->getRequired()) $l .= '<span class="requiredStar">*</span>';
						$v = '<p class="form-label">' . $l . '</p>' . $v;
					}
					$cls = $editable ? '' : 'class="withIcon"';
					$rhs .= "<li $cls>$v</li>";
				}
			}
		}

		if(!$edit_layout && ! empty($this->teams)) {
			$t_elt = $this->getElement($this->teams);
			if($t_elt)
				$teams = $t_elt->renderHtml($gen, $row_result, $subpath, $ctxedit);
		}
		if(! empty($teams)) {
			$lbl = translate('LBL_SECURITYGROUPS', 'app');
			if($editable)
				$teams = '<li><p class="form-label">' . to_html($lbl) . '</p>' . $teams . '</li>';
			$rhs .= $teams;
		}
		/*if(! $editable && ! $edit_layout && isset($this->activity_date) && ($f = $this->getElement($this->activity_date))) {
			if($f->getVisible()) {
				$v = $f->renderHtml($gen, $row_result, $subpath, $ctxedit);
				if(strlen($v)) {
					//$brow .= ' &nbsp;<div class="summary-buttons-meta inline-elt"><span class="form-label inline-label">' . $f->getLabel() . $sep . '</span>';
					//$brow .= $v . '</div>';
				}
			}
		}*/

		if(($this->meta /* || isset($this->activity_date)*/) && $editable && ! $edit_layout) {
			$meta_fs = $this->meta ? $this->meta['fields'] : [];
			//if(! empty($this->activity_date))
			//	$meta_fs[] = $this->activity_date;
			foreach($meta_fs as $f) {
				$e = $this->getElement($f);
				if(! $e || ! $this->checkVisible($e, $editable)) continue;
				$t = $e->getType();
				$v = $e->renderHtml($gen, $row_result, $subpath, $ctxedit);
				if(strlen($v ?? '')) {
					$li_c = '';
					if($editable) {
						$l = $e->getLabel(true);
						if(! $e->getEditable($context) && ($t == 'date' || $t == 'datetime' || strlen($v ?? '') < 20)) {
							$v = '<span class="form-label summary-inline">' . $l . $sep . '</span>' . $v;
							$li_c = 'withLabel';
						}
						else {
							if($e->getEditable($context) && $e->getRequired()) $l .= '<span class="requiredStar">*</span>';
							$v = '<p class="form-label">' . $l . '</p>' . $v;
						}
					}
					$rhs .= "<li class=\"$li_c\">$v</li>";
				}
			}
		}
		
		if($rhs) {
			$rhs = '<div class="summary-owner"><ul class="summary-list expand-list compact" id="summaryOwner">'
				 . $rhs
				 . '</ul></div>';
		}
		
		if(! $editable && ! $edit_layout && (($this->meta && ! empty($this->meta['fields'])) || ! empty($this->activity_date))) {
			$rhs .= '<div class="summary-meta"><ul class="summary-list">';
			$meta_fs = $this->meta ? $this->meta['fields'] : [];
			if(! empty($this->activity_date))
				$meta_fs[] = $this->activity_date;
			foreach($meta_fs as $f) {
				$e = $this->getElement($f);
				if(! $e || ! $this->checkVisible($e, $editable)) continue;
				$l = $e->getLabel(true);
				if($e->getEditable($context) && $e->getRequired()) $l .= '<span class="requiredStar">*</span>';
				$show_label = isset($e->show_label) ? $e->show_label : true;
				$v = $e->renderHtml($gen, $row_result, $subpath, $ctxedit);
				$lic = '';
				$c = 'form-label';
				$elt = 'p';
				$t = $e->getType();
				if($t == 'date' || $t == 'datetime' || $t == 'enum' || $t == 'status' || strlen($v ?? '') < 20) {
					$c = 'form-label summary-inline';
					$elt = 'span';
					$lic = 'withLabel';
					$l .= $sep;
				}
				$lbl = $show_label ? "<$elt class=\"$c\">$l</$elt>" : '';
				if(strlen($v ?? ''))
					$rhs .= "<li class=\"$lic\">{$lbl}{$v}</li>";
			}
			$rhs .= '</ul></div>';
		}
		
		if($rhs) {
			$html .= '<div class="column summary-right">' . $rhs . '</div>';
		}
		
		$html .= '</div>'; // row
		$html .= '</div>'; // main
 		
		$html .= '</div>'; // row
		
		$html .= '</div>';
		return $html;
	}
	
	
	function categoryLabel($cat) {
		$key = 'LBL_SUMMARY_HINT_'.strtoupper($cat);
		$lbl = translate($key, 'Accounts');
		return ($lbl != $key) ? $lbl : $cat;
	}

	
	function joinHeaderLine($gen, $row_result, $parents, $context, $line, $sectionId) {
		static $eltIndex = 10000;
		$all = array();
		$editable = $gen->getLayout()->getDefaultEditable();
		$layout = $gen->getLayout();
		$form_obj = $gen->getFormObject();
		$jsRows = array();
		$hidden = HtmlFormGenerator::calculateConditionallyHidden($row_result);

		$edit_layout = $layout->getEditingLayout();

		if(! empty($line['fields'])) {
			foreach($line['fields'] as $cellidx => $f) {
				$e = $this->getElement($f);
				if(! $e) continue;
				$eltId = 'elt_' . $eltIndex++;
				$cellName = $e->getName();
				if (!$editable && in_array($cellName, $hidden))
					continue;
				if ($edit_layout) {
					$lbl = $rawLbl = $e->getLabel();
					$layout->addEditElement($eltId, array(
						'name' => $cellName,
						'section' => $sectionId,
						'label' => $rawLbl,
						'widget' => $e->widget,
						'list_format' => $e->list_format,
					));
					$onclick = '';
					$lblId = 'lbl_' . $eltId;
					if (! strlen($cellName ?? '')) {
						$lbl = '&nbsp;';
					} else {
						$onclick = ' onclick="SUGAR.le.editField(this)" ';
						//$lbl = '<span ' . $onclick . ' id="' . $lblId . '">' . $lbl . '</span>';
						//$lbl .= '&nbsp;<span id="del_' . $eltId . '" style="visibility:hidden" class="mini-close" onclick="SUGAR.le.clearField(this)"></span>';
					}
					$lbl = "<span class=\"edit-layout-header-field\">$lbl</span>";
					$jsRows['row_' . $sectionId][$cellidx] = array(
						'id' => $eltId,
					);
					$all[] = $lbl;
				} else {
					$e->loadResult($row_result, $context);
					if(! $this->checkVisible($e, $editable)) continue;
					$v = $e->renderHtml($gen, $row_result, $parents, $context);
					if(strlen($v ?? '')) {
						$n = $e->getName();
						$list_format = isset($e->list_format) ? $e->list_format : null;
						if($context['editable']) {
							//$e->translate($this->model->getModuleDir());
							$l = $e->getLabel(true);
							if($e->getEditable($context) && $e->getRequired())
								$l .= '<span class="requiredStar">*</span>';
							$l = strlen($l ?? '') ? '<p class="form-label">' . $l . '</p>' : '';
							$v = ' <div class="summary-header-entry form-entry cell-' . $n . '">' . $l . '<div class="form-value">' . $v . '</div></div>';
						} else {
							if(! empty($e->show_label)) {
								$l = $e->getLabel(true);
								$sep = translate('LBL_SEPARATOR', 'app');
								$v = '<div class="summary-header-entry form-entry cell-' . $n . '"><span class="form-label">' . $l . $sep . '</span><div class="form-value">' . $v . '</div></div>';
							}
						}
						if($list_format == 'parenth') {
							if(! $context['editable'] /*|| ! $e->getEditable($context)*/)
								$v = " ($v)";
						}
						if($list_format == 'separate') {
							if(! $context['editable'] /*|| ! $e->getEditable($context)*/)
								$v = translate('LBL_SEPARATOR', 'app') . $v;
						}
						else if($list_format == 'bracket') {
							if(! $context['editable'] /*|| ! $e->getEditable($context)*/)
								$v = " [$v]";
						}
						else if($list_format == 'none') {
							$v = ' ' . $v;
						}
						else if(count($all) && ! $context['editable'])
							$all[] = " &mdash; ";
						$all[] = $v;
					}
				}
			}
		}
		$jsSection = array(
			'id' => $sectionId,
			'rows' => $jsRows,
			'rows_order' => array_keys($jsRows),
		);
		$gen->getLayout()->addEditableSection($jsSection);
		return $all;
	}

	function renderPdf(PdfFormGenerator $gen, $row_result, array $parents, array $context) {
		$def = array(
			'id' => '_summary_section',
			'elements' => array(
			),
			'type' => 'section',
		);
		$header_fields = array();
		if($this->header && ! empty($this->header['fields'])) {
			foreach($this->header['fields'] as $cellidx => $f) {
				$e = $this->getElement($f);
				if(! $e) continue;
				$cellName = $e->getName();
				$def['elements'][] = $cellName;
				$header_fields[] = $cellName;
			}
		}
		if($this->subheader && ! empty($this->subheader['fields'])) {
			foreach($this->subheader['fields'] as $cellidx => $f) {
				$e = $this->getElement($f);
				if(! $e) continue;
				$cellName = $e->getName();
				$def['elements'][] = $cellName;
				$header_fields[] = $cellName;
			}
		}
		foreach ($this->elements as $el) {
			$f = $el->getName();
			if (!in_array($f, $header_fields))
				$def['elements'][] = $f;
		}
		$section = new FormTableSection($def, $gen->layout->model);
		$section->loadResult($row_result, $context);
		$section->translate($gen->layout->model->getModuleDir());
		return $section->renderPdf($gen, $row_result, $parents, $context);
	}
	
	function checkVisible($element, $editable) {
		if(! $element || ! $element->getVisible())
			return false;
		$el_mode = $element->getMode();
		if($editable && $el_mode != 'edit' && $el_mode != 'any')
			return false;
		if(!$editable && $el_mode != 'view' && $el_mode != 'any')
			return false;
		return true;
	}
}

?>
