<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormSection.php');
require_once('include/layout/forms/FormField.php');
require_once('include/layout/forms/FormTbodySection.php');

class FormTableSection extends FormSection {
	var $columns;
	var $column_titles;
	var $show_descriptions;
	var $label_position;
	var $label_widths;
	var $desc_widths;
	var $widths;
	var $table_width;
	var $toggle_hidden;
	var $debug_render;
	
	function init($params=null, $model=null) {
		parent::init($params, $model);
		$this->columns = array_get_default($params, 'columns', 2);
		if (AppConfig::is_mobile()) {
			$mobi_fixed = array_get_default($params, 'mobile_columns');
			if($mobi_fixed) $this->columns = $mobi_fixed;
			else {
				$mobi_columns = AppConfig::get_mobile_columns();
				if(! $this->columns || $this->columns > $mobi_columns)
					$this->columns = $mobi_columns;
			}
		}
		$this->column_titles = array_get_default($params, 'column_titles');
		$this->show_descriptions = array_get_default($params, 'show_descriptions');
		$this->label_position = array_get_default($params, 'label_position', 'default');

		if (AppConfig::is_mobile()) $this->label_position = 'top';
		$this->label_widths = array('20%');
		$w = ($this->columns == 1 ? 80 : 30);
		if($this->show_descriptions && $this->columns == 1)
			$w /= 2;
		$this->widths = array($w.'%');
		if(! empty($params['label_widths'])) {
			if(is_array($params['label_widths']))
				$this->label_widths = $params['label_widths'];
			else
				$this->label_widths = explode(' ', $params['label_widths']);
		}
		if(! empty($params['widths'])) {
			if(is_array($params['widths']))
				$this->widths = $params['widths'];
			else
				$this->widths = explode(' ', $params['widths']);
		}
		if(! empty($params['desc_widths'])) {
			if(is_array($params['desc_widths']))
				$this->desc_widths = $params['desc_widths'];
			else
				$this->desc_widths = explode(' ', $params['desc_widths']);
		}
		if(isset($params['width']))
			$this->table_width = $params['width'];
	}
	
	function getColWidth($colidx, $label=false, $desc=false) {
		if($label)
			$w =& $this->label_widths;
		else {
			if($desc && isset($this->desc_widths))
				$w =& $this->desc_widths;
			else
				$w =& $this->widths;
		}
		if($w && is_countable($w))
			return $w[$colidx % count($w)];
		return '';
	}
	

	function translate($module) {
		parent::translate($module);
		if(is_array($this->column_titles)) {
			foreach($this->column_titles as $i => $c)
				$this->column_titles[$i] = translate($c, $module);
		}
	}
	
	
	function createElementType($type, $spec=null) {
		if(! $spec) $spec = array();
		if($this->form_id && ! isset($spec['form_id']))
			$spec['form_id'] = $this->form_id;
		if(! empty($this->skip_field_lookup))
			$spec['skip_field_lookup'] = true;
		if($type == 'section') {
			return new FormTbodySection($spec, $this->model);
		}
		else if($type == 'static') {
			return new FormElement($spec, $this->model);
		}
		else {
			return new FormField($spec, $this->model);
		}
	}
	
	function getColumnTitles() {
		return $this->column_titles;
	}
	
	function getShowDescriptions() {
		return $this->show_descriptions;
	}
	
	
	function renderTbodies(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		$ret = array();
		$body_idx = 0;
		$body = null;
        $elts =& $this->getElements();
        $editable = array_get_default($context, 'editable', 0);
        $edit_layout = array_get_default($context, 'edit_layout', 0);
        if($elts)
		foreach($elts as $id => $elt) {
			if($elt instanceof FormTbodySection) {
				if(! $body) {
					$body = array(
						'id' => $this->id.'-'.($body_idx ++),
						'cells' => array(),
						'label' => null,
						'titles' => $this->getColumnTitles(),
						'show_desc' => $this->getShowDescriptions(),
						'toggle_disp' => $edit_layout ? 0 : $this->getToggleDisplay(),
						'collapsible' => $this->getCollapsible(),
						'collapsed' => $this->getCollapsed(),
						'section_id' => $this->id,
					);
					if($body_idx) {
						$body['titles'] = null; // first tbody only
					}
				}
				if($edit_layout) {
					// insert stub for nested section
					$body['cells'][] = array(
						'name' => $elt->id,
						'label' => $elt->getLabel(),
						'colspan' => 10,
						'rowspan' => 1,
						'section_id' => $this->id,
						'is_section' => 1,
						'visible' => 1,
						'value' => '.',
					);
				}
				if($body && $body['cells']) {
					$ret[] = $body;
				}
				if($elt->getVisible())
					$cells = $elt->renderCells($gen, $row_result, $parents, $context);
				else
					$cells = null;
				if($cells) $ret[] = array(
					'id' => $elt->id,
					'cells' => $cells,
					'label' => $elt->getLabel(),
					'titles' => $elt->getColumnTitles(),
					'show_desc' => $elt->getShowDescriptions(),
					'toggle_disp' => $edit_layout ? 0 : $elt->getToggleDisplay(),
					'collapsible' => $elt->getCollapsible(),
					'collapsed' => $elt->getCollapsed(),
					'section_id' => $elt->id,
					'parent_id' => $this->id,
				);
				$body = null;
				$body_idx ++;
			}
			else if($elt instanceof FormElement) {
				$mode = $elt->getMode();
				if(($editable && $mode != 'edit' && $mode != 'any') || (! $editable && $mode == 'edit'))
					continue;
				if(! $body) {
					$body = array(
						'id' => $this->id.'-'.($body_idx ++),
						'cells' => array(),
						'label' => null,
						'titles' => $this->getColumnTitles(),
						'show_desc' => $this->getShowDescriptions(),
						'toggle_disp' => $edit_layout ? 0 : $this->getToggleDisplay(),
						'collapsible' => $this->getCollapsible(),
						'collapsed' => $this->getCollapsed(),
						'section_id' => $this->id,
					);
					if($body_idx) {
						$body['titles'] = null; // first tbody only
					}
				}
				$body['cells'][] = $elt->getHtmlCell($gen, $row_result, $parents, $context);
			}
		}
		if($body && $body['cells']) $ret[] = $body;
		return $ret;
	}
	
	
	function renderOuterTable(HtmlFormGenerator $gen, array $parents, array $context, $params=null) {
		if(! is_array($params)) $params = array();
		$width = array_get_default($params, 'width', $this->table_width);
		$body = array_get_default($params, 'body');
		$columns = array_get_default($params, 'columns', $this->columns);
		$labelPos = array_get_default($params, 'label_position', $this->label_position);
		$elements = array_get_default($params, 'elements');
		$style = array_get_default($params, 'style', '');
		if(isset($params['title']))
			$title = $params['title'];
		else
			$title = $this->getLabel();
		$css_class = $gen->getLayout()->getCssClasses();
		$ext_class = array_get_default($params, 'css');
		if($ext_class) array_extend($css_class, $ext_class);
		$edit_layout = $gen->getLayout()->getEditingLayout();
		
		if(! is_array($style)) {
			$s = array();
			if($style) $s[] = $style;
			$style = $s;
		}
		
		$hideTable = false;
		if($this->tab && $gen->getLayout()->getCurrentTab() !== $this->tab)
			$hideTable = true;
		if($this->toggle_hidden)
			$hideTable = true;
		$tcls = $css_class['form-section'];
		if(! $edit_layout && ($hideTable || ! empty($params['hidden'])))
			$tcls .= ' hidden';
		if($width)
			$style[] = 'width: $width';
		$style = implode('; ', $style);
		

		// print table header
		$html = "<div class=\"$tcls\" style=\"$style\" id=\"le_section_{$this->id}\">\n";
		if($title || $edit_layout) {
			$title_html = to_html($title);
			if ($edit_layout) {
				$title_html = "<span style=\"padding-left: 4px; padding-right: 4px; border: 1px dotted green; cursor:pointer\" id=\"section-title-{$this->id}\" onclick=\"SUGAR.le.editSectionTitle('{$this->id}');\">$title_html</span>";
			}
			if ($edit_layout) {
				$iconUp = '<div class="uii uii-arrow-up uii-md active-icon" onclick="SUGAR.le.moveSection(\'' . $this->id . '\', -1); return false;"></div>';
				$iconDown = '&nbsp;<div class="uii uii-arrow-down uii-md active-icon" onclick="SUGAR.le.moveSection(\'' . $this->id . '\', 1); return false;"></div>';
				if(! empty($params['editable_rows'])) {
					$add_lbl = to_html(translate('LBL_LAYOUT_ADD_ROW', 'NewStudio'));
					$iconAddRow = '&nbsp;<a href="#" class="uii uii-add uii-md active-icon" onclick="SUGAR.le.addRow(' . "'{$this->id}', '{$this->id}'" . '); return false;"><label>'.$add_lbl.'</label></a>';
				} else
					$iconAddRow = '';
				$rem_lbl = to_html(translate('LBL_LAYOUT_REM_SECTION', 'NewStudio'));
				$iconDelSection = '&nbsp;<a href="#" class="uii uii-remove uii-md active-icon" onclick="SUGAR.le.removeSection(' . "'{$this->id}'" . '); return false;"><label>'.$rem_lbl.'</label></a>';

				$title_html = $iconUp . $iconDown . $iconAddRow . $iconDelSection . '&nbsp;' . $title_html;
			}
			$title_extra = array_get_default($params, 'title_extra');
			if ($title_html) {
				$html .= "<header class=\"{$css_class['form-title-section']}\">\n";
				$html .= "<div class=\"{$css_class['form-cell']}\">\n";
				$html .= "<div class=\"{$css_class['form-title-entry']}\">\n";
				$html .= "<h4>{$title_html}{$title_extra}</h4></div></div>\n";
				$html .= "</header>\n";
			}
		}

		if(isset($params['description']))
			$d = $params['description'];
		else
			$d = $this->getDescription() ?? '';
		if (strlen($d)) {
			$html .= "<header class=\"{$css_class['form-row']} form-help-row\">";
			$html .= "<div class=\"{$css_class['form-cell']}\">";
			$html .= "<div class=\"{$css_class['form-entry']} form-help\">";
			$html .= "<div class=\"{$css_class['form-value']} text-help\">";
			$html .= $d;
			$html .= "</div></div></div></header>\n";
		}
		
		// for simple table contents at the moment (single section, no column titles)
		if(isset($elements)) {
			$rows = $this->arrangeSectionRows($elements, $columns, !$edit_layout);
			if($rows) {
				$html .= "<div class=\"{$css_class['form-section']}\" id=\"{$this->id}-body\">\n";
				$html .= $this->renderRows($gen, $rows, $columns, $labelPos, 0, $body);
				$html .= '</div>';
			}
		}

		$html .= $body . '</div>';

		if($edit_layout) {
			$jsRows = array_get_default($params, 'js_rows');
			if(! $jsRows) $jsRows = array($this->id => array());
			foreach($jsRows as $sid => $subRows) {
				$jsSection = array(
					'id' => $sid,
					'title' => $sid == $this->id ? $title : null,
					'columns' => $this->columns,
					'rows' => $subRows,
					'rows_order' => array_keys($subRows),
					'widget' => $sid == $this->id ? $this->widget : null,
				);
				$gen->getLayout()->addEditableSection($jsSection);
			}
		} else {
			$toggle_js = array_get_default($params, 'toggle_js');
			if($toggle_js) {
				if(is_array($toggle_js)) $toggle_js = implode(';', $toggle_js);
				$gen->getLayout()->addScriptLiteral("SUGAR.ui.onInitForm('{$gen->getFormName()}', function() { $toggle_js });");
			}
		}

		return $html;
	}
	
	
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {		
		$layout = $gen->getLayout();
		$form_obj = $gen->getFormObject();
		$edit_layout = $layout->getEditingLayout();
		$editable = $edit_layout ? false : ($layout->getDefaultEditable() && isset($form_obj));
		$subpath = $parents;
		$subpath[] =& $this;
		$ncontext = array_merge($context, compact('edit_layout', 'editable'));
		$show_empty = array_get_default($context, 'show_empty_rows',
			AppConfig::setting('layout.show_empty_detail_rows', false));
		
		$tbodies = $this->renderTbodies($gen, $row_result, $subpath, $ncontext);
		$this->arrangeTbodyRows($tbodies, $row_result, !$edit_layout && !$show_empty);
		$html = $this->outputTbodyRows($gen, $tbodies, $subpath, $ncontext);

		return $html;
	}
	

	function arrangeSection(array $elements, $columns=null, $type='section') {
		if(! isset($columns)) $columns = 2;
		if($columns < 1) $columns = 1;
		$sections = [
			[
				'type' => 'section',
				'columns' => $columns,
				'colspan' => $columns,
				'elements' => [],
			],
		];
		$rows = [0 => []];		
		$sect_idx = array_fill(0, $columns, 0);
		$sect_row = [0 => 0];
		$sect_col = [0 => 0];
		$sect_closed = [0 => false];
		$hfill = [0 => 0];
		$vfill = [0 => 0];
		$cidx = 0;
		$ridx = 0;
		$lclear = $rclear = false;
		$rclear_idx = 0;
	
		foreach($elements as $e) {
			if(is_array($e)) {
				$cspan = min(array_get_default($e, 'colspan', 1), $columns);
				$rspan = array_get_default($e, 'rowspan', 1);
				$ecols = array_get_default($e, 'columns', $cspan);
				$type = array_get_default($e, 'type');
				$clear = array_get_default($e, 'clear', 'none');
				if(! $cspan || ! $rspan) continue;
				if(isset($e['elements']) && is_array($e['elements'])) {
					if(! $type) $type = $e['type'] = 'section';
					if($type == 'row')
						$e['elements'] = arrangeForm($e['elements'], $ecols)['elements'][0]['elements'];
					else
						$e = arrangeForm($e['elements'], $ecols, $type);
					$e['colspan'] = $cspan;
					$e['rowspan'] = $rspan;
					// fixme - copy any other properties (title etc)
				}
				else if(! $type) {
					if(! empty($e['name'])) {
						$type = $e['type'] = 'field';
					}
					else if(! empty($e['widget'])) {
						$type = $e['type'] = 'widget';
					}
					else if(isset($e['value'])) {
						$type = $e['type'] = 'static';
					}
					else
						$type = $e['type'] = 'spacer';
				}
				$fill = $e;
			}
			else {
				$cspan = $rspan = 1;
				$clear = 'none';
				if($e)
					$fill = ['type' => 'field', 'name' => $e];
				else
					$fill = ['type' => 'spacer'];
			}
			$lclear = ($clear == 'left' || $clear == 'both' || $rclear);
			$rclear = ($clear == 'right' || $clear == 'both');
		
			$placed = false;
			while(! $placed) {
				$sidx = $sect_idx[$cidx];
				if(empty($sect_closed[$sidx])) {
					$rmax = $rspan; //($sidx == 0 ? $rspan : $sections[$sidx]['rowspan'] - $vfill[$sidx]);
					if((! $lclear || $cidx == 0) && ($ridx >= $rclear_idx) && $rspan <= $rmax && $cspan + $hfill[$sidx] <= $sections[$sidx]['columns']) {
						// can place here
						if($rspan > 1) {
							// have a cell with rowspan - create new sections around it
							if($hfill[$sidx]) {
								$sections[] = [
									'type' => 'section',
									'columns' => $hfill[$sidx],
									'colspan' => $hfill[$sidx],
									'rowspan' => $rspan,
									'elements' => [
										['type' => 'row', 'elements' => $rows[$sidx]],
									],
									'parent_idx' => $sidx,
								];
								$new_sidx = count($sections) - 1;
								$sect_col[$new_sidx] = $sect_col[$sidx];
								$sect_row[$new_sidx] = $sect_row[$sidx] + $vfill[$sidx];
								$hfill[$new_sidx] = 0;
								$vfill[$new_sidx] = 1;
								$rows[$new_sidx] = [];
								$rows[$sidx] = [];
								for($i = $sect_col[$sidx]; $i < $cidx; $i ++) {
									$sect_idx[$i] = $new_sidx;
								}
							}
							$fill['colspan'] = $cspan;
							$fill['parent_idx'] = $sidx;
							$sections[] = $fill;
							$fill_sidx = count($sections) - 1;
							$sect_col[$fill_sidx] = $cidx;
							$sect_row[$fill_sidx] = $sect_row[$sidx] + $vfill[$sidx];
							$sect_closed[$fill_sidx] = true;
							$hfill[$fill_sidx] = 0;
							$vfill[$fill_sidx] = $rspan;
							for($i = 0; $i < $cspan; $i ++) {
								$sect_idx[$cidx + $i] = $fill_sidx;
							}
							if($hfill[$sidx] + $cspan < $sections[$sidx]['columns']) {
								$sections[] = [
									'type' => 'section',
									'columns' => $sections[$sidx]['columns'] - $hfill[$sidx] - $cspan,
									'colspan' => $sections[$sidx]['columns'] - $hfill[$sidx] - $cspan,
									'rowspan' => $rspan,
									'elements' => [],
									'parent_idx' => $sidx,
								];
								$new_sidx = count($sections) - 1;
								$sect_col[$new_sidx] = $cidx + $cspan;
								$sect_row[$new_sidx] = $sect_row[$sidx] + $vfill[$sidx];
								$hfill[$new_sidx] = 0;
								$vfill[$new_sidx] = 0;
								$rows[$new_sidx] = [];
								for($i = 0; $i < $sections[$new_sidx]['colspan']; $i ++) {
									$sect_idx[$cidx + $cspan + $i] = $new_sidx;
								}
							}
							$sidx = $fill_sidx;
						} else {
							$rows[$sidx][] = $fill;
							$hfill[$sidx] += $cspan;
						}
						if($rclear)
							$rclear_idx = $ridx + $rspan;
						$cidx += $cspan;
						$placed = true;
					}
					else {
						// fill space
						$fill_c = min($cspan, $sections[$sidx]['columns'] - $hfill[$sidx]);
						if($fill_c) {
							$rows[$sidx][] = ['type' => 'spacer', 'colspan' => $fill_c];
							$hfill[$sidx] += $fill_c;
							$cidx += $fill_c;
						}
					}
				}
				else
					$cidx ++;

				if(empty($sect_closed[$sidx]) && $hfill[$sidx] == $sections[$sidx]['columns']) {
					// append current row
					$sections[$sidx]['elements'][] = ['type' => 'row', 'elements' => $rows[$sidx]];
					$hfill[$sidx] = 0;
					$vfill[$sidx] ++;
					$rows[$sidx] = [];
					if($sidx != 0 && $vfill[$sidx] >= $sections[$sidx]['rowspan']) {
						// finish section
						$sections[$sidx]['rows'] = $vfill[$sidx];
						$sect_closed[$sidx] = true;
					}
				}
			
				if($cidx == $columns) {
					$cidx = 0;
					$ridx ++;

					// close sections
					while(1) {
						$close_idx = [];
						$sub_idx = [];
						for($i = 0; $i < $columns; $i++) {
							$sidx = $sect_idx[$i];
							if(! $sidx) continue;
							$pidx = $sections[$sidx]['parent_idx'];
							if(empty($sect_closed[$sidx])) {
								$close_idx[$pidx] = 0;
							}
							else if(! isset($close_idx[$pidx]) || $close_idx[$pidx]) {
								$close_idx[$pidx] = 1;
								$sub_idx[$pidx][$sidx] = 1;
							}
							while($pidx != 0) {
								$sidx = $pidx;
								$pidx = $sections[$sidx]['parent_idx'];
								$close_idx[$pidx] = 0;
							}
						}
						if(! array_filter($close_idx)) break;
					
						foreach($close_idx as $pidx => $_) {
							if(! $_) continue;
							$row = [];
							$max_rspan = 0;
							$s_vfill = [];
							foreach($sub_idx[$pidx] as $sidx => $_) {
								$max_rspan = max($max_rspan, $sections[$sidx]['rowspan']);
								$row[] = $sections[$sidx];
								$s_vfill[] = $vfill[$sidx];
							}
							for($i = 0; $i < count($row); $i++) {
								while($s_vfill[$i] < $max_rspan) {
									$row[$i]['elements'][] = ['type' => 'row', 'elements' => ['type' => 'spacer', 'colspan' => $row[$i]['columns']]];
									$s_vfill[$i] ++;
								}
							}
							$sections[$pidx]['elements'][] = ['type' => 'row', 'elements' => $row];
							$hfill[$pidx] = 0;
							$vfill[$pidx] += $s_vfill[0];
							$sect_cspan = $sections[$pidx]['colspan'];
							for($i = 0; $i < $sect_cspan; $i ++) {
								$sect_idx[$sect_col[$pidx] + $i] = $pidx;
							}
							if($pidx && $vfill[$pidx] >= $sections[$pidx]['rowspan']) {
								$sections[$pidx]['rows'] = $vfill[$pidx];
								$sect_closed[$pidx] = true;
							}
						}
					}
				}
			}
		}
	
		// append all rows
		for($i = 0; $i < $columns; $i++) {
			$sidx = $sect_idx[$i];
			if(empty($sect_closed[$sidx]) && $hfill[$sidx]) {
				if($hfill[$sidx] < $sections[$sidx]['columns'])
					$rows[$sidx][] = ['type' => 'spacer', 'colspan' => $sections[$sidx]['columns'] - $hfill[$sidx]];
				$sections[$sidx]['elements'][] = ['type' => 'row', 'elements' => $rows[$sidx]];
				$hfill[$sidx] = 0;
				$vfill[$sidx] ++;
				$sections[$sidx]['rows'] = $vfill[$sidx];
				$sect_closed[$sidx] = true;
			}
		}
	
		return $sections[0];
	}
	
	
	function checkRowEmpty($row) {
		if(isset($row['visible']) && ! $row['visible'])
			return true;
		$empty = true;
		foreach($row['elements'] as $col) {
			$visib = array_get_default($col, 'visible', true);
			$type = array_get_default($col, 'type');
			if($type == 'spacer' || ! $visib)
				continue;
			else if($type == 'section') {
				if(! empty($col['value'])) {
					$empty = false;
					break;
				}
				if(! empty($col['elements'])) {
					foreach($col['elements'] as $r) {
						if(! $this->checkRowEmpty($r)) {
							$empty = false;
							break 2;
						}
					}
				}
			}
			else {
				$cell = array_get_default($col, 'value');
				if(strlen(trim($cell ?? ''))) {
					$empty = false;
					break;
				}
			}
		}
		return $empty;
	}


	function renderRows(HtmlFormGenerator $gen, $rows, $columns=2, $lbl_pos=null, $offset=0, $body = array()) {
		static $eltIndex = 0;
		$layout = $gen->getLayout();
		$edit_layout = $layout->getEditingLayout();
		if($columns < 1) $columns = 1;
		$ret = '';
		$indRequired = $gen->getLayout()->indicate_required;
		$css_class = $gen->getLayout()->getCssClasses();
		$entry_cls = ($lbl_pos == 'left' ? ' label-left' : '');
		$idx = $offset;
		if (isset($body['js_rows'])) {
			$jsRows =& $body['js_rows'];
		} else {
			$jsRows = array();
		}
		
		$roffset = 0;
		if (isset($body['section_id']) && isset($jsRows[$body['section_id']])) {
			$roffset = is_countable($jsRows[$body['section_id']]) ? count($jsRows[$body['section_id']]) : 0;
		}
		foreach($rows as $ridx => $row) {
			$rowidx = $ridx + $roffset;
			$cls = ($idx ++ % 2 == 0) ? ' row-odd' : ' row-even';
			if ($edit_layout) {
				$tableRowId = 'id="' . $body['section_id'] . '_row_' . $rowidx . '"';
			} else {
				$tableRowId = '';
			}
			$ret .= '<div ' . $tableRowId . ' class="'.$css_class['form-row'].$cls.' col-unstack">';
			foreach($row['elements'] as $cellidx => $col) {
				if($edit_layout) {
					$col['visible'] = true;
					$col['editable'] = true;
				}

				$eltId = 'elt_' . $eltIndex++;
				$cspan = array_get_default($col, 'colspan', 1);
				$rspan = array_get_default($col, 'rowspan', 1);
				$type = array_get_default($col, 'type');
				$visible = array_get_default($col, 'visible', true);
				$sty = '';
				$perc = $cspan / $columns;
				$pcols = $perc * 12;
				if ($edit_layout) {
					$perc = $perc * 0.9;
				}
				
				if(! empty($col['required']) && ! empty($col['editable']) && $visible && $indRequired && $type != 'bool')
					$star_required = '<span class="requiredStar">*</span>';
				else
					$star_required = '';
				
				$lbl = $rawLbl = array_get_default($col, 'label', '');
				if (!strlen(trim($lbl))) {
					$lbl = '&nbsp;';
				}

				$dtId = 'dt_' . $this->id . '_' . $rowidx . '_' . $cellidx;
				$lblId = 'lbl_' . $eltId;
				$onclick = "";
				$cellName = array_get_default($col, 'name', '');
				$name_class = $cellName ? 'cell-' . $cellName : '';
				if ($edit_layout)  {
					$onclick = ' onclick="SUGAR.le.editField(this)" ';
				}

				$lbl = '<div id="' . $lblId . '" class="input-field input-field-group state-label" style="display:block"><div ' . $onclick . ' class="input-label">' . $lbl . '</div><div id="del_' .$eltId . '" class="input-label input-remove" style="visibility:hidden" onclick="SUGAR.le.clearField(this)"></div></div>';
				//$lbl = '<span ' . $onclick . ' id="' . $lblId . '">' . $lbl . '</span>';

				if($type == 'spacer' || ! $visible) {
					$cls = $css_class['form-spacer'];
					$cell = '&nbsp;';
				}
				else if($type == 'section') {
					if(! empty($col['value'])) {
						$cell = $col['value'];
						$cls = $css_class['form-section'];
					}
					else if(isset($col['elements'])) {
						$cls = $css_class['form-section'];
						$cell = $this->renderRows($gen, $col['elements'], $col['columns'], $lbl_pos, $idx, $body);
					} else
						$cell = '';
				}
				else if($type == 'field' && ! isset($col['value'])) {
					$cls = $css_class['form-cell'];
					$cell = $col['name'];
					if(! empty($this->debug_render))
						$cell .= " <span class=\"text-muted small\"> $type ($cspan x $rspan)</span>";
				}
				else {
					$cls = $css_class['form-cell'];
					$cell = array_get_default($col, 'value');
					if(! strlen($cell)) $cell = '&nbsp;';
					if($type != 'spacer' && $type != 'section') {
						$cell = '<div class="'.$css_class['form-value'].'">'.$cell.'</div>';
						if(! empty($col['label']))
							$cell = '<p class="'.$css_class['form-label'].'">'.$col['label'].$star_required.'</p>' . $cell;
						$cell = '<div class="'.$css_class['form-entry'].$entry_cls.'">'.$cell.'</div>';
					}
					if(! empty($col['desc']))
						$cell .= '<div class="'.$css_class['form-desc'].'">'.$col['desc'].'</div>';
				}

				if(round($pcols) == $pcols && !$edit_layout)
					$cls .= ' sm-'.$pcols;
				else
					$sty = ' style="flex-basis: '.($perc*100).'%"';
				
				if ($edit_layout) {
					if (! empty($col['is_section']) || array_get_default($col, 'type') === 'section') {
						$layout->addEditElement($eltId, array(
							'name' => $cellName,
							'section' => $body['section_id'],
							'row' => $rowidx,
							'col' => $cellidx,
							'label' => $rawLbl,
							'is_section' => 1,
						));
						$jsRows[$body['section_id']]['row_' . $rowidx][$cellidx] = array(
							'id' => $eltId,
							'colspan' => $col['colspan'],
							'is_section' => 1,
						);
						continue;
					}					
					$layout->addEditElement($eltId, array(
						'name' => $cellName,
						'section' => $body['section_id'],
						'row' => $rowidx,
						'col' => $cellidx,
						'label' => $rawLbl,
					));
					$ddId = 'dd_' . $eltId;
					$lbl = "<div id=\"$ddId\" class=\"edit_layout_wrapper\" onmouseover=\"SUGAR.le.showDelete(this)\" onmouseout=\"SUGAR.le.hideDelete(this)\" >$lbl</div>";
					$jsRows[$body['section_id']]['row_' . $rowidx][$cellidx] = array(
						'id' => $eltId,
						'colspan' => $col['colspan'],
					);
					$dtId = 'dt_' . $eltId;
					$layout->addDropTarget($dtId);
					$cell = $lbl;
				} else {
				}
				$ret .= "<div class=\"$cls $name_class span-$cspan\"$sty>$cell</div>";
			}
			if ($edit_layout ) {
				if (empty($col['is_section']) && array_get_default($col, 'type') !== 'section') {
					$ret .= '<div class="column form-cell sm-1" style="flex-basis:10%">';
					$ret .= '<div class="form-entry">';
					$ret .= '<span class="uii uii-add uii-md active-icon" onclick="SUGAR.le.addRow(' . "'{$body['section_id']}', '{$body['section_id']}_row_{$rowidx}'" . '); return false;"></span>';
					$ret.= '&nbsp;<span class="uii uii-remove uii-md active-icon" onclick="SUGAR.le.deleteRow(' . "'{$this->id}', 'row_{$rowidx}', '{$body['id']}'" . '); return false;"></span>';
					$ret .= '</div>';
					$ret .= '</div>';
				}
			}
			$ret .= '</div>';
		}
		return $ret;
	}
	
	
	function arrangeSectionRows(array $elements, $columns=null, $remove_blank=true) {
		$sect = $this->arrangeSection($elements, $columns);
		$rows = [];
		foreach($sect['elements'] as $row) {
			if(! $remove_blank || ! $this->checkRowEmpty($row))
				$rows[] = $row;
		}
		return $rows;
	}
	
	
	function arrangeTbodyRows(array &$tbodies, $row_result, $remove_blank = true)
	{
		$eltPerRow = max(1, $this->columns);

		// arrange list of cells into separate rows
		foreach($tbodies as $k => &$body) {
			$cells = $body['cells'];
			unset($body['cells']);
			$nextspan = array();
			$body['toggle_js'] = array();
			
			$rows = $this->arrangeSectionRows($cells, $eltPerRow, $remove_blank);
			$body['rows'] = $rows;
			if(! $rows)
				$body['visible'] = false;

			$toggle = $body['toggle_disp'];
			if($toggle) {
				if(! is_array($toggle)) $toggle = array('name' => $toggle);
				$value = array_get_default($row_result->row, $toggle['name']);
				if(! isset($toggle['value'])) {
					$body_hide = (! $value);
					$hook = 'this.getValue()';
				} elseif(is_array($toggle['value'])) {
					$toggle_values = $toggle['value'];
					$values = [];
					foreach ($toggle_values as $k => $v) {
						if (is_array($v)) $values[] = $k;
						else $values[] = $v;
					}
					$json = getJSONobj();
					$body_hide = !in_array($value, $values);
					$hook =  '~' . $json->encode($values) . '.indexOf(this.getValue())';
				} else {
					$body_hide = ($toggle['value'] != $value);
					$jvalue = addcslashes($toggle['value'], "'");
					$hook = "this.getValue() == '$jvalue'";
				}
				$post = array_get_default($toggle, 'onchange', '');
				if($body['section_id'] == $this->id) {
					if($body_hide) $this->toggle_hidden = true;
					$hide_elt = 'le_section_'.$this->id;
				} else {
					if($body_hide) $body['hidden'] = true;
					$hide_elt = $body['id'];
				}
				$hook = "function() { toggleDisplay('{$hide_elt}', $hook, true); $post }";
				$body['toggle_js'][] = "SUGAR.ui.attachFormInputEvent(this, '{$toggle['name']}', 'onchange', $hook);";
			}
		}
	}
	
	
	function resolveLabelPosition(HtmlFormGenerator $gen, array $context) {
		$edit_layout = array_get_default($context, 'edit_layout');
		$lblPos = $this->label_position;
		if($edit_layout)
			$lblPos = 'top';
		if($lblPos == 'default')
			$lblPos = $gen->getDefaultLabelPosition();
		return $lblPos;
	}
	
	
	function outputTbodyRows(HtmlFormGenerator $gen, array $tbodies, array $parents, array $context) {
		global $themeInstance;
		
		$layout = $gen->getLayout();
		$edit_layout = array_get_default($context, 'edit_layout');
		$css_class = $layout->getCssClasses();
		$lblPos = $this->resolveLabelPosition($gen, $context);
		$lblSide = ($lblPos == 'above' || $lblPos == 'top') ? false : $lblPos;

        $no_row_border = "";
        $no_empty = false;
        $show_empty_message = false;
        $style = array();
        if ($this->id == 'mini_details' && ! $edit_layout) {
            $style[] = 'border: 0';
            $no_row_border = 'style="border-bottom: 0;"';
            $no_empty = true;
            $show_empty_message = true;
        }

		// pre-render setup
		$eltPerRow = max(1, $this->columns);
		$cellPerCol = 1;
		if($lblSide)
			$cellPerCol ++;
		$anyDesc = false;
		foreach($tbodies as $k => &$body) {
			if(isset($body['show_desc']) && (! empty($body['rows']) || $k == 0))
				$anyDesc |= $body['show_desc'];
		}
		unset($body);
		if($anyDesc)
			$cellPerCol ++;
		$filledElems = 0;
		$indRequired = $layout->indicate_required;
		$toggleJs = array();
		$jsRows = array();
		$html = '';
		
		// print out tbodies and contained rows
		foreach($tbodies as $body) {
			if(empty($body['rows']) || ! array_get_default($body, 'visible', true))
				continue;
			$body['js_rows'] =& $jsRows;

			$showDesc = $body['show_desc'];
			if(! isset($showDesc) && $this->show_descriptions)
				$showDesc = 1;
			$s = '';
			if(! empty($body['hidden']))
				$s .= '; display: none';
			if(! empty($body['toggle_js']))
				array_extend($toggleJs, $body['toggle_js']);

			$label = $body['label'];
			if($label) {
				$label = to_html($label);
				$colCount = $eltPerRow * $cellPerCol;
				if($body['collapsible'] && ! $edit_layout) {
					$icon = '<div class="input-arrow" style="margin-left: 1.5em">&nbsp;</div>';
					$label .= $icon;
					$onclick = "onclick=\"return toggleSection('{$body['id']}')\"";
					$ls = 'cursor: pointer;';
					$extcls = $body['collapsed'] ? 'sectionClosed' : 'sectionOpen';
				}
				else
					$extcls = $onclick = $ls = '';

				if ($edit_layout) {
					$add_lbl = to_html(translate('LBL_LAYOUT_ADD_ROW', 'NewStudio'));
					$iconAddRow = '&nbsp;<a href="#" class="uii uii-add uii-md active-icon" onclick="SUGAR.le.addRow(' . "'{$body['id']}'" . '); return false;"><label>'.$add_lbl.'</label></a>';
				} else {
					$iconAddRow = '';
				}

				$html .= "<header class=\"{$css_class['form-title-section']}\" id=\"sectionHeader-{$body['id']}\" style=\"$s\" $onclick>\n";
				$html .= "<div class=\"{$css_class['form-cell']}\">\n";
				$html .= "<div class=\"{$css_class['form-title-entry']}\">\n";
				$html .= "<h4>{$label}$iconAddRow</h4>\n";
				$html .= "</div></div></header>\n";
				if(! $edit_layout && $body['collapsible'] && $body['collapsed'] && empty($body['hidden']))
					$s .= '; display: none';
			}

			$html .= "<div class=\"{$css_class['form-section']}\" id=\"{$body['id']}\" style=\"$s\">\n";
			
			$titles = $body['titles'];
			if(is_array($titles) && $titles) {
				$html .= "<div class=\"{$css_class['form-title-section']}\">\n";
				$c = count($titles);
				$i = 0;
				foreach($titles as $label) {
					if($i >= $eltPerRow) break;
					if($i == $c - 1)
						$cspan = $cellPerCol * ($eltPerRow - $i);
					else
						$cspan = $cellPerCol;
					if ($edit_layout) {
						$cspan++;
					}
					$label = to_html($label);
					$html .= "<div class=\"{$css_class['form-cell']}\">\n";
					$html .= "<div class=\"{$css_class['form-title-entry']}\">\n";
					$html .= "<h4>$label</h4>\n";
					$html .= "</div></div>\n";
				}
				$html .= "</div>\n";
			}

			/*
				$tbody .= $append;
				if(strlen($tbody)) {
					$html .= "<tr id=\"{$body['id']}_row_{$rowidx}\">\n" . $tbody;
					
					if ($edit_layout) {
						$sid = isset($body['parent_id']) ? $body['id'] : $this->id;
						$html .= '<td nowrap class="dataLabel">';
						$html .= '<div class="input-icon icon-add active-icon" onclick="SUGAR.le.addRow(' . "'{$this->id}', '{$body['id']}_row_{$rowidx}'" . '); return false;"></div>';
						$html .= '&nbsp;<div class="input-icon icon-delete active-icon" style="cursor:pointer" onclick="SUGAR.le.deleteRow(' . "'{$this->id}', 'row_{$rowidx}', '{$body['id']}'" . '); return false;"></div>';
						$html .= '</td>';
					}
	
					$html .= "</tr>\n";
				}
			}*/
			
			$html .= $this->renderRows($gen, $body['rows'], $eltPerRow, $lblPos, 0, $body);
			
			$html .= "</div>\n";
		}

        if ($show_empty_message && ! $html) {
        	$html .= "<header class=\"{$css_class['form-header']}\" id=\"sectionHeader-{$body['id']}\" style=\"$s\" $onclick>\n";
			$html .= "<div class=\"{$css_class['form-row']}\">";
			$html .= "<div class=\"{$css_class['form-cell']}\">\n";
			$html .= "<div class=\"{$css_class['form-entry']} form-help\">\n";
			$html .= "<div class=\"{$css_class['form-value']}\">";
			$html .= '<h5 class="text-help">' . translate('LBL_NO_ADDITIONAL_DETAILS')."</h5>\n";
			$html .= "</div></div></div></div></header>\n";
        }
        else if(! $html)
        	return;
        
		$colCount = $eltPerRow * $cellPerCol;
		if ($edit_layout) $colCount++;
        $params = array(
        	'css' => $css_class,
        	'style' => $style,
        	'body' => $html,
        	'colspan' => $colCount,
        	'hidden' => ! empty($hideTable),
        	'js_rows' => $jsRows,
        	'toggle_js' => $toggleJs,
        	'editable_rows' => true,
        );
        return $this->renderOuterTable($gen, $parents, $context, $params);
	}
	
	
	function renderPdf(PdfFormGenerator $gen, $row_result, array $parents, array $context)
	{
		$elts =& $this->getElements();
		$need_title = true;
		if($elts)  {
			$row = 0;
			$backgrounds = array(
				array(
					array(245, 245, 245),
					array(255, 255, 255),
				),
				array(
					array(240, 240, 240),
					array(250, 250, 250),
				),
			);
			$cols = array();
			$data = array();
			$cur_col = 0;
			$opts = array(
				'border' => 0,
				'padding' => 2,
			);
			foreach($elts as $id => $elt) {
				$cell = $elt->getPlainCell($row_result, $parents, $context);
				$cs = array_get_default($cell, 'colspan', 1);
				if ($cs + $cur_col > $this->columns && !empty($data)) {
					$title = $need_title ? $this->getLabel() : '';
					$need_title = false;
					$gen->pdf->DrawTable($data, $cols, $title, false, $opts);
					$cols = array();
					$data = array();
					$cur_col = 0;
					$row++;
				}
				$cols['label_'.$id] = array(
					'width' => $this->getColWidth($cur_col, true),
					'background-color' => $backgrounds[$row%2][0],
				);
				$w = $this->getColWidth($cur_col, false);
				while ($cs > 1) {
					$cur_col++;
					$w = add_perc_width($w, $this->getColWidth($cur_col, true));
					$w = add_perc_width($w, $this->getColWidth($cur_col, false));
					$cs--;
				}
				$cols['text_'.$id] = array(
					'width' => $w,
					'background-color' => $backgrounds[$row%2][1],
				);
				$data[0]['label_' . $id] = $cell['label'];
				$data[0]['text_' . $id] = $cell['value'];
				$cur_col++;
			}
			if (!empty($data)) {
				$title = $need_title ? $this->getLabel() : '';
				$gen->pdf->DrawTable($data, $cols, '', false, $opts);
			}
		}
	}
}
function add_perc_width($a, $b) {
	return ((float)$a + (float)$b) . '%';
}

?>
