<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormGenerator.php');
require_once('include/layout/forms/EditableForm.php');

class HtmlFormGenerator extends FormGenerator {
    /**
     * @var EditableForm
     */
	var $form_obj;
	var $result;
	var $default_buttons_pos = 'top';
	var $wrap_form = true;
	var $section_render_count = 0;
	var $open_pos;

	function init() {
		$this->result = array();
	}


	function exportIncludes() {
		global $pageInstance;
		$styles = $this->layout->getStyles();
		foreach($styles as $sty) {
			if(! is_array($sty))
				continue;
			$prio = LOAD_PRIORITY_BODY;
			if(isset($sty['priority']))
				$prio = $sty['priority'];
			if(isset($sty['file']))
				$pageInstance->add_css_include($sty['file'], null, $prio);
			else if(isset($sty['literal']))
				$pageInstance->add_css_literal($sty['literal'], null, $prio);
		}
		$scripts = $this->layout->getScripts();
		foreach($scripts as $scr) {
			if(! is_array($scr))
				continue;
			$prio = LOAD_PRIORITY_FORM;
			if(isset($scr['priority']))
				$prio = $scr['priority'];
			if(isset($scr['file']))
				$pageInstance->add_js_include($scr['file'], null, $prio);
			else if(isset($scr['url']))
				$pageInstance->add_js_include($scr['url'], array('ext' => true), $prio);
			else if(isset($scr['literal']))
				$pageInstance->add_js_literal($scr['literal'], null, $prio);
			else if(isset($scr['language']))
				$pageInstance->add_js_language($scr['language'], null, $prio);
			else if(isset($scr['model']))
				$pageInstance->add_js_model_cache($scr['model']);
		}
		$prio = LOAD_PRIORITY_FOOT;

		if($this->layout->getInPopup()) {
			$this->addPopupInit($prio);
		}

		if($this->layout->getEditingLayout()) {
			$pageInstance->add_js_include('include/javascript/dynamic_list.js', null, LOAD_PRIORITY_BODY);
			$pageInstance->add_js_include('include/javascript/layout_editor.js', null, LOAD_PRIORITY_BODY);
			$edit_sects = $this->layout->getEditableSections();
			$all_elts = $this->layout->getEditElements();
			if (!empty($edit_sects)) {
				$json = getJSONobj();
				$encodedSections = $json->encode(array(
					'sections' => $edit_sects,
					'elements' => $all_elts,
				));
				$disabledFields = $json->encode($this->layout->getDisabledLayoutFields());
				$encodedTargets = $json->encode($this->layout->getDropTargets());
				$encodedFields = $json->encode($this->layout->getAvailableFields());
				$default_columns = $this->layout->name == 'Mobile' ? 1 : 2;
				$pageInstance->add_js_literal("SUGAR.le.init(" . $encodedSections . "," . $encodedTargets . ", " . $encodedFields . ", " . $default_columns . ", " . $disabledFields . ");", null, $prio);
			}
		}

		if($this->form_obj) {
			if(isset($this->layout->form_hooks))
				$this->form_obj->addHooks($this->layout->form_hooks);
			$this->form_obj->exportIncludes();
		}
	}

	function renderForm($row_result=null, $module=null)
	{
		if ($this->layout->getEditingLayout()) {
			$this->layout->addAllFields($row_result);
		} else {
			$this->exportConditionalFields($row_result);
		}
		return parent::renderForm($row_result, $module);
	}

	function exportConditionalFields($row_result) {
		if(! $row_result) return;
		$ltype = $this->layout->getType();
		$disable = ($ltype == 'searchform') || ($ltype == 'massupdate');
		if ($ltype == 'searchform' || $ltype == 'massupdate') return;
		$model_name = $row_result->getModelName();
		if(! $model_name || $model_name == 'Configurator')
			return;
		$model = new ModelDef($model_name);
		$cols = $model->getColumnFields();
		foreach($cols as $fname) {
			$def = $model->getFieldDefinition($fname);
			if(isset($def['for_ref'])) {
				$cols[] = $def['for_ref'];
			}
		}
		$fields = array_merge($cols, $model->getCustomFields());
		$conditional = array();
		if (!$disable) {
			// FIXME - index conditional fields in ModelManager for speed
			foreach ($fields as $fname) {
				$def = $model->getFieldDefinition($fname);
				if (!empty($def['depends_on'])) {
					$conditional[] = $def;
				}
			}
		}
		$json = getJSONobj();
		$conditional = $json->encode($conditional);
		$GLOBALS['pageInstance']->add_js_literal("SUGAR.ui.initConditionalFields($conditional);", null, LOAD_PRIORITY_FOOT);
		$hidden = $json->encode(self::calculateConditionallyHidden($row_result));
		$GLOBALS['pageInstance']->add_js_literal("SUGAR.ui.hideConditionals($hidden);", null, LOAD_PRIORITY_FOOT);
	}
	function getCustomFields() {
		return AppConfig::setting("model.custom_fields.{$this->name}", array());
	}

	function addPopupInit($prio=LOAD_PRIORITY_FOOT) {
		$icon = $this->layout->getIcon();
		if(! $icon)
			$icon = $GLOBALS['currentModule'];
		$title = $this->layout->getTitle();
		if($icon) $title = get_image($icon, '') . '&nbsp;' . $title;
		$params = array('title' => $title, 'fill' => true);
		$json = getJSONobj();
		$params = $json->encode($params);
		$fname = $this->getFormName();
		if($fname) $params = substr($params, 0, -1) . ", onshow: function() {SUGAR.ui.focusForm(document.forms.$fname);}}";
		$GLOBALS['pageInstance']->add_js_literal("SUGAR.popups.initContent($params);", null, $prio);
	}

	function renderTitle() {
		global $pageInstance, $themeInstance;
		if($this->layout->getInPopup())
			return;
		$icon = $this->layout->getIcon();
		$title = $this->layout->getTitle();
		$desc = $this->layout->getTitleDesc();
		$ltype = $this->layout->getType();
		if(! strlen($title)) {
			if($ltype == 'massupdate' || $ltype == 'searchform' || $themeInstance->layout_version < 8 || ! $pageInstance->update_page_state)
				return;
		}
		if($ltype == 'massupdate') {
			$open = array_get_default($this->layout->form_hidden_fields, 'mass_panel_open');
			$cls = $open ? 'sectionOpen' : 'sectionClosed';
			$onclick = 'return sugarListView.toggleUpdate(\'formContent-' . $this->form_obj->form_id . '\');';
			$this->result[] = "<div class=\"card-header panel-header tools-row {$cls}\" id=\"formHeader-{$this->form_obj->form_id}\" style=\"cursor:pointer\" onclick=\"$onclick\">";
			$this->result[] = '<h4><span class="rbullet rbullet-inline">' . $title . '</span></h4>';
			$this->result[] = '</div>';
		}
		else if($ltype == 'searchform') {
			if($icon) $title = get_image($icon, '') . '&nbsp;' . $title;
			$this->result[] = '<div class="card-header form-header panel-header">';
			$this->result[] = '<h4 class="form-title search-title">' . $title . '</h4>';
			if(strlen($desc)) {
				$this->result[] = '<div class="form-desc">' . nl2br($desc) . '</div>';
			}
			$this->result[] = '</div>';
		} else {
			if(! $icon && $this->model) $icon = $this->model->name;
			$this->result[] = get_module_title($icon, $title, $this->layout->show_title_links);
		}
	}

	function renderStart() {
		if($this->layout->form_autofocus_fields) {
			$this->form_obj->setAutoFocusFields($this->layout->form_autofocus_fields);
		}
		if(! $this->layout->buttons_position) {
			$this->layout->buttons_position = $this->default_buttons_pos;
		}
		$this->section_render_count = 0;
	}

	function renderHeader($row_result=null, $module=null) {
		if ($this->form_obj) {
			if ($this->layout->getType() == 'massupdate') {
				$open = array_get_default($this->layout->form_hidden_fields, 'mass_panel_open');
				$hidden = $open ? '' : 'style="display:none"';
				$this->result[] = "<div id=\"formContent-{$this->form_obj->form_id}\" class=\"card-outer form-outer default-form\" $hidden>";
			}
		}
		if(isset($this->header))
			$this->result[] = $this->header;
		//$this->renderTabs();
		if($this->wrap_form && ! $this->form_obj) {
			$this->result[] = '<div class="'.$this->getFormClass().'">';
			$this->open_pos = count($this->result);
		}
		if($this->layout->buttons_position == 'top' || $this->layout->buttons_position == 'topbottom')
			$this->renderButtons('top');
		parent::renderHeader($row_result, $module);
		$this->renderTabs();
	}

	function renderSummary($row_result=null, $module=null) {
		$pos = is_array($this->result) ? count($this->result) : 0;
		parent::renderSummary($row_result, $module);
		if(is_array($this->result) && count($this->result) > $pos) {
			array_splice($this->result, $pos, 0,
				'<div class="card-header form-summary">');
			$this->result[] = '</div>';
		}
	}

	function renderFooter() {
		if($this->layout->buttons_position == 'bottom' || $this->layout->buttons_position == 'topbottom')
			$this->renderButtons('bottom', ($this->layout->buttons_position == 'topbottom'));
		if(isset($this->open_pos)) {
			if(is_array($this->result) && count($this->result) == $this->open_pos)
				array_pop($this->result);
			else
				$this->result[] = '</div>';
		}
		if(isset($this->footer))
			$this->result[] = $this->footer;
		if ($this->form_obj) {
			if ($this->layout->getType() == 'massupdate') {
				$this->result[] = "</div>";
			}
		}
	}

	function renderTabs() {
		$tabs_spec = $this->layout->getTabs();
		$active = $this->layout->getCurrentTab();
		if($tabs_spec) {
			$mod = $this->layout->getBaseModule();
			$ltype = $this->layout->getType();
			$view_mode = ($ltype == 'view' || ($ltype == 'record' && ! $this->layout->getDefaultEditable()));
			$tabs = array();
			foreach($tabs_spec as $k => $tab) {
				if(! is_array($tab) || empty($tab['name'])) continue;
				$name = $tab['name'];
				if($ltype == 'view' && empty($tab['perform']) && $name) {
					$tab['perform'] = "SUGAR.ui.changeLayout(this.form, '$name');";
				}
				if(isset($tab['vname'])) {
					$tab['label'] = translate($tab['vname'], $mod);
				}
				else if(isset($tab['label'])) {
					$tab['label'] = translate($tab['label'], $mod);
				}
				if($ltype == 'view' && $name == $this->layout->getName())
					$active = $name;
				if($this->layout->getEditingLayout())
					//$tab['disabled'] = 1;
					$tab['editing'] = 1;
				$tabs[] = $tab;
			}
			if($tabs) {
				$spec = array(
					'name' => ($ltype == 'record' ? 'layout_tab' : 'form-tabs'),
					'tabs' => $tabs,
				);
				foreach($this->layout->getSectionIds() as $sid) {
					$sect = $this->layout->getSection($sid);
					if($sect) {
						$tabname = $sect->getTab();
						if($ltype == 'record' && ! $tabname) $tabname = $tabs[0]['name'];
						if($tabname)
							$spec['manage_sections']['le_section_'.$sid][] = $tabname;
					}
				}
				if($ltype == 'record' && $view_mode)
					$spec['onchange'] = "SUGAR.ui.layoutTabUpdated(this);";
				$this->result[] = '<div class="card-header form-tabs">';
				$this->result[] = $this->form_obj->renderTabGroup($spec, $active);
				$this->result[] = '</div>';
			}
		}
	}

	function renderButtons($position, $duplicate=false) {
		$buttons = $this->layout->getFormButtons();
		$edit_layout = $this->layout->getEditingLayout();
		$editable = $this->layout->getDefaultEditable();
		if($buttons) {
			$html = array('left' => array(), 'center' => array(), 'right' => array());
            $additional_html = array();
			$ltype = $this->layout->getType();
			foreach($buttons as $b) {
				if(! $b->getVisible()) continue;
				if($ltype == 'record') {
					if($editable && $b->mode != 'edit' && $b->mode != 'any') continue;
					if(! $editable && $b->mode == 'edit') continue;
				}
				if($duplicate && $b->getNoDuplicate()) continue;
				$isSaveLayout = ($b->name == 'save_layout_edit' || $b->name == 'save_layout_view');
				$isSaveLayout = $isSaveLayout || ($this->layout->getEditingLayout() && ($b->name == 'edit' || $b->name == 'view'));
				$align = $b->align;
                if(! array_key_exists($align, $html))
                	$align = 'left';
				$name = $b->name;
				$b->align = null;
				if($duplicate) $b->name .= '2';
				$b->beforeRender($this);
				$b->class = ($b->class ? $b->class.' ' : '').'group-'.$align;
				if($align == 'left' && ! count($html[$align]))
					$b->class .= ' first';
				$button_html = $this->form_obj->renderFormButton($b, $this->layout->in_popup, $edit_layout && !$isSaveLayout);
				$b->align = $align;
				if(! $duplicate)
					$additional_html[] = $b->getAdditionalHtml($this);
                if(strlen($button_html)) {
					$html[$align][] = [$button_html, $b->id];
                }
				if($b->type == 'group' && ! $duplicate) {
					foreach($b->getGroupOptions() as $b2)
						if($b2->getVisible())
							$additional_html[] = $b2->getAdditionalHtml($this);
				}
				$b->name = $name;
			}
			$display = $this->layout->getType();
			$dcls = [];
			if($position == 'bottom')
				$dcls[] = 'card-footer';
			//else if($display == 'massupdate' || $display == 'distribute')
			//	$dcls[] = 'form-mid';
			else if($display != 'searchform')
				$dcls[] = 'card-header';
			$dcls = implode(' ', $dcls);
			$this->result[] = "<div class=\"$dcls form-buttons\">";
			$this->result[] = "<div class=\"row buttons-row\">";
			foreach($html as $k => $bs) {
				$this->result[] = "<div class=\"column buttons-inline align-$k\">";
				foreach($bs as $b) {
					$idstr = ($b[1] ? ' id="'.to_html($b[1] . '-div') .'"' : '');
					$this->result[] = '<div class="inline-elt"'.$idstr.'>'.$b[0].'</div>';
				}
				$this->result[] = '</div>';
			}
			if ($edit_layout) {
				$add_lbl = to_html(translate('LBL_LAYOUT_ADD_SECTION', 'NewStudio'));
				$this->result[] = '<div><a href="#" class="uii uii-add uii-md active-icon" onclick="SUGAR.le.addSection(); return false;"><label>'.$add_lbl.'</label></a></div>';
			}
			$this->result[] = '</div>';
			$this->result[] = '</div>' . implode(' ', $additional_html);
		}
	}

	function renderSections($row_result=null, $module=null) {
		$this->result[] = '<div class="card-body form-body">';
		parent::renderSections($row_result, $module);
		$this->result[] = '</div>';
	}

	function _renderSection(FormSection $section, $row_result, array $parents, array $context) {
		$this->result[] = $section->renderHtml($this, $row_result, array(), $context);
		$this->section_render_count ++;
	}

	function renderFinish() {
		if($this->form_obj) {
			$cls = $this->section_render_count ? null : '';
			array_unshift($this->result, $this->form_obj->open());
			$this->result[] = $this->form_obj->close();
		}
		if ($this->layout->getEditingLayout()) {
			$this->result[] = $this->makeFieldsBox();
		}
	}

	function getResult() {
		return implode('', $this->result);
	}

	function getEditableForm() {
		return $this->form_obj;
	}

	function getFormName() {
		if($this->form_obj)
			return $this->form_obj->getFormName();
	}

	function makeFieldsBox()
	{
		global $theme;
		$theme_path = "themes/" . $theme . "/";
		$image_path = $theme_path . "images/";
		$avail_lbl = to_html(translate('LBL_LAYOUT_AVAILABLE_FIELDS', 'NewStudio'));
		return <<<BOX
<div id="le_fields_box" class="card-outer dialog-outer dialog-popup compact draggable" style="position:fixed; width:180px; top:4000px; min-height: 10px; left:4px; margin-bottom: 10px; display: block; z-index: 1000;">
<div class="card-header dialog-header"><div class="dialog-title tools-row align-top sectionClosed" id="le_fields_title"><h4>{$avail_lbl}</h4><span class="rbullet rbullet-inline no-flex"></span></div></div>
<div class="card-body dialog-body" style="max-height:300px; min-height:10px !important">
<div id="le_fields_content" class="form-section">
</div>
</div>
</div>
BOX;
	}
}

?>
