<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once 'include/layout/forms/EditableForm.php';
require_once 'include/pdf/PDFManager.php';

class PdfFormGenerator extends FormGenerator {
    /**
     * @var EditableForm
     */
	var $form_obj;
	var $result;
	var $pdf;
	var $conditionally_hidden_els;
	
	function init() {
		$this->conditionally_hidden_els = array();
		$this->result = array();
		$mgr = AppConfig::setting("display.view." . $this->model->name . ".pdf_manager");
		if ($mgr) {
			require_once $mgr['file'];
			$this->pdf = new $mgr['class'];
		} else {
			$this->pdf = new PDFManager;
		}
		$this->pdf->new_page();
	}

	function renderTitle() {
		$title = $this->layout->getTitle();
		$cols = $data = array();
		$this->pdf->DrawTable($data, $cols, $title);
	}

	function renderSummary($row_result=null, $module=null) {
		$this->conditionally_hidden_els = self::calculateConditionallyHidden($row_result);
		parent::renderSummary($row_result, $module);
	}

	function renderSections($row_result=null, $module=null) {
		if (empty($this->conditionally_hidden_els))
			$this->conditionally_hidden_els = self::calculateConditionallyHidden($row_result);
		parent::renderSections($row_result, $module);
	}

	function _renderSection(FormSection $section, $row_result, array $parents, array $context) {
		$section->loadResult($row_result, $context);
		$elements = $section->getElements();

		if (is_array($elements) && !empty($this->conditionally_hidden_els)) {
			foreach ($elements as $element) {
				if (in_array($element->getName(), $this->conditionally_hidden_els)) {
					$element->hidden = true;
				}
			}
		}

		$section->renderPdf($this, $row_result, array(), $context);
	}
	
	function getResult() {
		ob_clean();
		$this->pdf->filename = $this->layout->model->name . '.pdf';
		$this->pdf->serve_dynamic();
	}

	function exportIncludes() {}
}
