<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/DashletManager.php');
require_once('include/layout/pages/ModulePage.php');

class DashletPage extends ModulePage {
	var $dashlet_ids = array();
	var $system_page = false;
	var $section = '';
	var $dashboard_id = '';
	var $dashboard;
	var $dmgr = null;
	var $edit_layout;
	
	function init_context() {
		parent::init_context();
		if($this->module == 'Home' && $this->action != 'About' && $this->action != 'UnifiedSearch') {
			$this->dmgr = new DashletManager();
			if(! $this->init_dashlets())
				$this->redirect_no_dashboards();
		}
	}
	
	function redirect_no_dashboards() {
		global $themeInstance, $current_user;
		if(isset($themeInstance)) {
			$tabs = $themeInstance->get_tabs('Home');
		} else if($current_user && $current_user->id) {
			require_once('include/GroupedTabs/GroupedTabStructure.php');
			$tabManager = new GroupedTabStructure();
			$nav_style = $current_user->getPreference('navigation_paradigm');
			if(!isset($nav_style))
				$nav_style = AppConfig::setting('layout.defaults.navigation_paradigm');
			$tabs = $tabManager->get_theme_tabs($nav_style == 'gm', 'Home');
		}
		if($tabs) {
			if($tabs['use_group_tabs']) $opts = current($tabs['tabs']);
			else $opts = $tabs;
			foreach($opts['tabs'] as $o) {
				if(! empty($o['url']) && $o['module'] != 'Home') {
					header('Location: ' . $o['url']);
					exit;
				}
			}
		}
	}

	function load_dashboard($dashId) {
		if(! $this->dmgr) $this->dmgr = new DashletManager();
		$this->dashboard = $this->dmgr->get_page($dashId);
		if($this->dashboard) {
			$this->section = $this->dashboard->section;
			$this->dashboard_id = $this->dashboard->id;
			$set_ptab = true;
			$save_page = true;
			return true;
		}
	}
	
	function get_dashboard() {
		return $this->dashboard;
	}
	
	function get_dashboard_title() {
		if($this->dashboard) {
			$st = ''; //$this->dashboard->get_section_title();
			$t = $this->dashboard->get_title();
			if($st && $t)
				return ($st == $t) ? $t : "$st: $t";
			else if($st)
				return $st;
			return $t;
		}
		return '';
	}
	
	function get_dashboard_icon() {
		if($this->dashboard)
			return $this->dashboard->get_icon();
		return '';
	}
	
	function get_module_name($mod=null) {
		$t = from_html($this->get_dashboard_title());
		if($t)
			return $t;
		else if($this->action == 'UnifiedSearch')
			return translate('LBL_SEARCH');
		return parent::get_module_name($mod);
	}
	
	
	function init_dashlets() {
		global $current_user;
		
		if(! empty($_REQUEST['dashboard_id'])) {
			$this->load_dashboard($_REQUEST['dashboard_id']);
		}
		else if(! empty($_REQUEST['layout'])) {
			$this->load_dashboard($_REQUEST['layout']);
		}
		if(! $this->dashboard) {
			if(isset($_GET['parentTab']))
				$this->section = $_GET['parentTab'];
			else if(isset($_POST['parentTab']))
				$this->section = $_POST['parentTab'];
			else if(isset($_SESSION['home_selected_section'])) {
				// ignore cookie if this is the first load for the session
				if(isset($_COOKIE['parentTab']))
					$this->section = $_COOKIE['parentTab'];
				else {
					$this->section = $_SESSION['home_selected_section'];
					$set_ptab = true;
				}
			} else
				$this->section = '';
			
			$sects = $this->dmgr->get_defined_sections();
			if(! isset($sects[$this->section])) {
				$found = false;
				foreach($sects as $k => $sect) {
					$lbl = translate($k);
					if($lbl == $this->section) {
						$this->section = $k;
						$found = true;
						break;
					}
				}
				if(! $found) {
					reset($sects);
					$this->section = key($sects);
				}
				$set_ptab = true;
			}
			
			if(! $this->section)
				return false;
				
			$this->dashboard_id = $current_user->getPreference('selected_page~'.$this->section, 'home');
			$_REQUEST['layout'] = $this->dashboard_id;
				
			if(! $this->dashboard_id || ! isset($sects[$this->section][$this->dashboard_id])) {
				$this->dashboard_id = key($sects[$this->section]);
				$_REQUEST['layout'] = $this->dashboard_id;
				$save_page = true;
			}
			
			$this->dashboard = $sects[$this->section][$this->dashboard_id];
		}

		$_SESSION['home_selected_section'] = $this->section;
		if(! empty($set_ptab)) {
			$_REQUEST['parentTab'] = $_COOKIE['parentTab'] = $this->section;
			if(AppConfig::detect_https() == 'https')
	       		setcookie('parentTab', $this->section, time()-42000, '/', AppConfig::request_host_name(), FALSE, TRUE);
	       	else
	       		setcookie('parentTab', $this->section, time()-42000, '/', AppConfig::request_host_name(), FALSE, FALSE);
		}
		if(! empty($save_page))
			$current_user->setPreference('selected_page~'.$this->section, $this->dashboard_id, 0, 'home');
		
		$this->dashboard->init_from_content();
		return true;
	}
	
	function add_edit_javascript($async=false) {
		$maxCount = AppConfig::setting('layout.max_dashlets_homepage', 15);
		$ids = $this->dashboard->get_dashlet_ids();
		$col_count = $this->dashboard->col_count;
		$dashletIds = '["' . implode('","', $ids) . '"]';
		$col_opts = array('keys' => array(), 'values' => array());
		for($i = 1; $i < 5; $i++) {
			$col_opts['keys'][] = $i;
			$col_opts['values'][] = array('label' => translate('LBL_COLUMNS_'.$i, 'Home'));
		}
		$json = getJSONobj();
		$col_json = $json->encode($col_opts);
		
		if(!$this->edit_layout)
			return;
		$js = "
			SUGAR.sugarHome.maxCount = {$maxCount};
			SUGAR.sugarHome.layoutMode = true;
			SUGAR.sugarHome.init = function () {
				var dashletIds = {$dashletIds};
				SUGAR.ui.registerMenuSource('set_columns',
					{options: {$col_json},
					 onchange: function(key) { if(key) SUGAR.sugarHome.setColumns(key); } });
			}
		";
		if($async)
			$js .= "SUGAR.sugarHome.init();";
		else
			$js .= "SUGAR.onDomReady(SUGAR.sugarHome.init);";
		$this->add_js_literal($js, null, LOAD_PRIORITY_FOOT);
	}

	function after_render() {
		parent::after_render();
		
		if(! $this->dashboard)
			return;

		$this->add_include_group('home');
	}
}

