<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/pages/BasePage.php');

class ModulePage extends BasePage {
	var $header_row;
	var $module_menu;

	var $module = '';
	var $action = '';
	var $record = '';
	var $layout = '';
	var $title_module, $title_module_label, $title_module_merge;

	function init_context() {
		parent::init_context();
		$this->module = isset($GLOBALS['currentModule']) ? $GLOBALS['currentModule'] : '';
		$this->action = isset($GLOBALS['action']) ? $GLOBALS['action'] : 'index'; 
		$this->record = isset($GLOBALS['record']) ? $GLOBALS['record'] : '';
		$this->layout = isset($GLOBALS['layout']) ? $GLOBALS['layout'] : '';
	}

	function module_title($title) {
		global $app_strings;
		if($title)
			return $app_strings['LBL_SHORT_BROWSER_TITLE']. ': '. $title;
		return $app_strings['LBL_BROWSER_TITLE'];
	}

	function get_module_name($mod=null) {
		if(! isset($mod)) $mod = $this->get_title_module();
		global $app_list_strings, $subModuleList;
		if(isset($app_list_strings['moduleList'][$mod]))
			$title = $app_list_strings['moduleList'][$mod];
		else if(isset($subModuleList[$mod])
				&& isset($app_list_strings['moduleList'][$subModuleList[$mod]]))
			$title = $app_list_strings['moduleList'][$subModuleList[$mod]];
		else
			$title = $app_list_strings['moduleList']['Administration'];
		return $title;
	}

	function init_title($module_label=null) {
		parent::init_title();
		if(empty($this->user->id))
			return;
		if(isset($module_label))
			$this->title = $this->module_title($module_label);
		else
			$this->title = $this->module_title($this->get_module_name($this->get_title_module()));
		$this->title_suffix = ' | '.$this->title;
	}

	function init_favicon($custom=null) {
		$std_icon_href = 'favicon.ico?s='.AppConfig::version();
		$std_icon_type = 'image/vnd.microsoft.icon';
		$icon_href = $std_icon_href;
		$icon_type = $std_icon_type;
		$hr_href = null;
		$user_agent = array_get_default($_SERVER, 'HTTP_USER_AGENT', '');
		if(! AppConfig::setting('layout.disable_module_favicon') && strpos($user_agent, ' MSIE ') === false) {
			$icon_name = isset($custom) ? $custom : $this->get_title_module();
			$icon_paths = array(
				"themes/{$this->theme}/images/{$icon_name}.gif",
				"themes/Default/images/{$icon_name}.gif",
			);
			$hr_paths = array(
				"themes/Default/images32/{$icon_name}.png",
			);
			$action = isset($GLOBALS['action']) ? $GLOBALS['action'] : '';
			$record = isset($GLOBALS['record']) ? $GLOBALS['record'] : '';
			if($action == 'EditView' && ! $record && ! isset($custom)) {
				array_unshift($icon_paths, "themes/Default/images/Create{$icon_name}.gif");
				array_unshift($icon_paths, "themes/{$this->theme}/images/Create{$icon_name}.gif");
				array_unshift($hr_paths, "themes/Default/images32/Create{$icon_name}.png");
			}
			else if($action == 'About' || $action == 'UnifiedSearch') {
				$icon_paths = $hr_paths = array();
			}
			foreach($icon_paths as $path) {
				if(file_exists($path)) {
					$icon_href = to_html($path);
					$icon_type = 'image/gif';
					break;
				}
			}
			foreach($hr_paths as $path) {
				if(file_exists($path)) {
					$hr_href = to_html($path);
					break;
				}
			}
		}
		$icon = "<link rel=\"shortcut icon\" href=\"$icon_href\" type=\"$icon_type\">".
			"\n<link rel=\"icon\" href=\"$icon_href\" type=\"$icon_type\" sizes=\"16x16\">";
		if($hr_href)
			$icon .= "\n<link rel=\"icon\" href=\"$hr_href\" type=\"image/png\" sizes=\"32x32\">";
		$this->favicon = [
			'href' => $icon_href,
			'hr_href' => $hr_href,
			'html' => $icon,
		];
	}

	// in translations, LBL_MODULE_NAME is usually translated, which breaks the icon in the module header
	// unfortunately this field is sometimes used for other purposes, so we can't just override it with currentModule
	// hopefully 4.5 fixes this
	function get_module_header_icon($module, $title, $ext='.gif') {
		$safe_module = preg_replace("/[^A-Za-z_]/","",$module);
		$path0 = "modules/$safe_module/icon";
		$path1 = $this->image_path.$safe_module;
		$path2 = "themes/Default/images/$safe_module";
		$found = file_exists($path0.$ext) ? $path0 : (file_exists($path1.$ext) ? $path1 : (file_exists($path2.$ext) ? $path2 : ''));
		if($found)
			return get_image($found, 'style="margin-top: 3px;"');
		if($this->module && $this->module != $module)
			return get_module_header_icon($this->module, $title, $ext);
		return '';
	}

	function is_pdf_printable() {
		static $no_pdf = array('Dashboard', 'Activities', 'Calendar', 'Forums');
		if(! $this->module || in_array($this->module, $no_pdf))
			return false;
		return true;
	}

	function get_printable_link() {
		$ident = "module={$this->module}&action={$this->action}&record={$this->record}";
		if(!$this->is_pdf_printable()) {
			$link = "index.php?print=true&{$ident}";
		} else {
			$link = "index.php?pdf=true&pdf_print=true&{$ident}";
			if(isset($_REQUEST['forecast_time_period']))
				$link .= "&forecast_time_period=".$_REQUEST['forecast_time_period'];
		}
		return $link;
	}

	// move to renderer class
	function make_skype_link($phone, $link_class='tabDetailViewDFLink') {
		$phone = format_phone($phone);
		if(AppConfig::setting('telephony.integration') == 'skypeout') {
			if(skype_formatted($phone))
				$phone = '<a class="'.$link_class.'" href="skype://' . $phone. '">'.$phone. '</a>' ;
		}
		return $phone;
	}

	// move to renderer class
	function format_email_link($address, $name='', $link_params='') {
		$userPref = empty($this->user->id) ? '' : $this->user->getPreference('email_link_type');
		$defaultPref = AppConfig::setting('email.default_client', 'sugar');
		if($userPref != '') {
			$client = $userPref;
		} else {
			$client = $defaultPref;
		}

		if($client == 'sugar') {
			if($name) $address = "$name <$address>";
			$link = '<a href="index.php?module=Emails&action=EditView&type=out'.
				'&to_email_addrs='.htmlentities($address).$link_params.'" class="listViewTdLinkS1">';

		} else {
			$link = '<a href="mailto:' . $address .'" class="listViewTdLinkS1">';
		}
		return  $link . ($name ? $name : $address) . '</a>';
	}

	function add_status_check_javascript($immed = false) {
		global $current_user;
		if (!$immed) {
			$interval = (int)AppConfig::setting('site.performance.status_check_interval', 300000);
			$timestamp = time();
			$no_sound = $current_user ? ($current_user->getPreference('email_sound_disable') ? 1 : 0) : 0;
			$static_root = javascript_escape(AppConfig::site_static_url());
			$s = "SUGAR.notify.initStatusCheck($timestamp, $interval, '{$static_root}include/images/player/mail.mp3', $no_sound)";
		} else {
			$s = "SUGAR.notify.checkStatus();";
		}
		$this->add_js_literal($s, null, LOAD_PRIORITY_END);
		return $s;
	}
	
	function format_includes($section, $type_filter=null, $remove_after=false, $dynamic_inc=false) {
		$ret = parent::format_includes($section, $type_filter, $remove_after, $dynamic_inc);
		if ($section == 'end') {
			$ringcentral_data = $this->ringcentral_data();
			if ($ringcentral_data) {
				require_once 'include/layout/widgets/RingCentralWidget.php';
				$w = new RingCentralWidget;
				$ret .= $w->render($ringcentral_data);
			}
			$chat_provider = $this->chat_provider();
			if ($chat_provider) {
				$ret .= $chat_provider->render();
			}
		}
		return $ret;
	}

	function after_render() {

		global $app_strings;
		parent::after_render();
		
		// do not replace session when inside a popup
		if($this->partial_page) return;

		global $timedate;
		global $locale;
		global $themeInstance;
		global $current_user;

		$today = (1000 * strtotime($timedate->handle_offset(gmdate('Y-m-d H:i:s', time()), 'Y-m-d H:i:s')));
		$hour_offset = $timedate->get_hour_offset() * 60 * 60;
		$start_day = 0;
		$begin_hour = AppConfig::setting('company.day_begin_hour');
		$end_hour = AppConfig::setting('company.day_end_hour');
		$seps = get_number_seperators();
		$display_name = '';
		$user_name = '';
		$full_name = '';
		$real_names = 1;

		if($this->user->id) {
			$cid = $this->user->getPreference('currency');
			if(! isset($cid)) $cid = '-99';
			if(! isset($_SESSION['user_currency_id']) || $_SESSION['user_currency_id'] != $cid) {
				require_once('modules/Currencies/Currency.php');
				$cur = new Currency();
				$cur->retrieve($cid);
				$_SESSION['user_currency_id'] = $cid;
				$_SESSION['user_currency_decimals'] = $cur->decimal_places;
			}
			$decimals = $_SESSION['user_currency_decimals'];
			$real_names = $locale->getUseRealNames() ? 1 : 0;
			$user_name = $this->user->user_name;
			$full_name = $this->user->full_name;
			$display_name = $real_names ? $full_name : $user_name;
			$start_day = (int)($this->user->week_start_day);
			$begin_hour = (int)($this->user->day_begin_hour);
			$end_hour = (int)($this->user->day_end_hour);
		}
		if(! isset($decimals))
			$decimals = $locale->getPrecedentPreference('default_currency_significant_digits');

		$site_url = AppConfig::site_url();
		$static_url = AppConfig::site_static_url();
		if(! $static_url) $static_url = $site_url;
		
		require_once('modules/Home/UnifiedSearchAdvanced.php');
		$adv = new UnifiedSearchAdvanced();
		$search_info = $adv->getModelIndex();
		$search_mods = [];
		foreach($search_info['allowed'] as $model)
			$search_mods[] = AppConfig::module_for_model($model);
		
		$sess_vars = array(
			'is_admin' => is_admin() ? 1 : 0,
			'module' => $this->module,
			'language' => $this->language,
			'user_id' => $this->user->id,
			'user_name' => $user_name,
			'full_name' => $full_name,
			'display_name' => $display_name,
			'use_real_names' => $real_names,
			'site_url' => $site_url,
			'static_url' => $static_url,
			'theme' => $this->theme,
			'layout_version' => $themeInstance->version(),
			'separators' => array(
				'thousands' => $seps[0],
				'decimal' => $seps[1],
				'duration_hr' => array_get_default($app_strings, 'LBL_DURATION_SUFFIX_HOUR', 'h'),
				'hour_minute' => $timedate->get_time_separator(),
			),
			'formats' => array(
				'date' => $timedate->get_cal_date_format(),
				'time' => $timedate->get_cal_time_format(),
			),
			'currency' => array(
				'significant_digits' => $decimals,
			),
			'telephony' => get_phone_format_options(),
			'week_start_day' => $start_day,
			'day_begin_hour' => $begin_hour,
			'day_end_hour' => $end_hour,
			'jscal_today' => $today,
			'time_offset' => $hour_offset,
			'track_page_stats' => AppConfig::setting('site.performance.track_page_stats') ? 1 : 0,
			'search_modules' => $search_mods,
		);
		if(! $this->async_page)
			$sess_vars['request_method'] = strtoupper($_SERVER['REQUEST_METHOD']);
		$json = getJSONobj();
		$sess_init = $json->encode($sess_vars);

		if($this->async_page) {
			$this->add_js_literal("SUGAR.session.update($sess_init);", ['prepend' => true], LOAD_PRIORITY_BODY);
		} else {
			// add this AFTER the page is rendered, because we might change some of the settings
			$inc = <<<EOS
				if(typeof window.app_strings == "undefined") app_strings = {};
				_session_init = $sess_init;
EOS;
			$this->add_js_literal($inc, null, LOAD_PRIORITY_PRE_INIT);
		}
	}
	
	function finalize_popup_header() {
		$inc = "var popup_module = '{$this->module}';";
		$this->add_js_literal($inc, null, LOAD_PRIORITY_PRE_INIT);
	}

	function finalize_page() {
		parent::finalize_page();

		$ringcentral_data = $this->ringcentral_data();
		if ($ringcentral_data) {
			$json = getJSONobj();
			extract($ringcentral_data);
			$args = array(
				'sandbox' => !!AppConfig::setting('telephony.ringcentral_sandbox', 0),
				'callerId' => $number_direct,
				'pressOne' => (bool)$press_one,
				'auto_expand' => (bool)$expand,
				'clientId' => $clientId,
			);
			$js_args = $json->encode($args);
			$this->add_js_include('include/javascript/rcsdk/ringcentral.min.js', ['no_min' => true], LOAD_PRIORITY_HEAD);
			$this->add_js_include('include/javascript/rcsdk/ringcentral-subscriptions.min.js', ['no_min' => true], LOAD_PRIORITY_HEAD);
			$this->add_js_include('include/javascript/ipc.js', null, LOAD_PRIORITY_HEAD);
			$this->add_js_include('include/javascript/ringcentral.js', null, LOAD_PRIORITY_HEAD);
			$this->add_js_literal('RingCentralOneCRM.initialize(' . $js_args . ');');
			$path = 'themes/Default/ringcentral.css';
			$this->add_css_include($path, null, LOAD_PRIORITY_THEME);
		}
		
		$chat_provider = $this->chat_provider();
		if ($chat_provider) {
			$chat_provider->finalize_page($this);
		}
	}

	function ringcentral_data() {
		global $current_user;
		if (!AppConfig::is_mobile() 
				&& !$this->async_page 
				&& AppConfig::setting('telephony.integration') == 'ringcentral' 
				&& AppConfig::current_user_id()) {
			$number_direct = $current_user->getPreference('ringcentral_number_direct');
			$press_one = $current_user->getPreference('ringcentral_press_one');
			$expand = $current_user->getPreference('ringcentral_expand');
			$clientId = AppConfig::setting('telephony.ringcentral_client_id');
			if ($number_direct) {
				return compact('number_direct', 'press_one', 'expand', 'clientId');
			}
		}
	}

	function chat_provider() {
		global $current_user;
		if (!AppConfig::is_mobile() 
				&& !$this->async_page 
				&& AppConfig::current_user_id()) {
			require_once 'include/chat/ChatProvider.php';
			$p = ChatProvider::get();
			if ($p && $p->isGuiEnabled())
				return $p;
		}
	}

	function get_onload() {
		if($this->action == "EditView")
			return "if(window.set_focus) set_focus();";
		return '';
	}
	
	function set_title_module($mod, $label=null, $merge=false) {
		$this->title_module = $mod;
		$this->title_module_label = $label;
		$this->title_module_merge = $merge;
		$this->init_title($label);
		$this->init_favicon();
	}
	
	function get_title_module() {
		if($this->title_module) return $this->title_module;
		return $this->module;
	}
	
	function get_title_module_label() {
		if($this->title_module_label) return $this->title_module_label;
		return $this->get_module_name();
	}

}

?>
