<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
// included by index.php and async.php


if(! isset($nextDirective))
	$nextDirective = $pageController->get_directive();
if($nextDirective && $nextDirective[0] == 'redirect')
	indexRedirect($nextDirective[1]);

loadLicense();

if($nextDirective && isset($nextDirective[1]))
	indexCheckRawOutput($nextDirective[1]);


// for printing
$GLOBALS['request_string'] = '';
foreach ($_GET as $key => $val) {
	if(is_array($val)) {
		foreach ($val as $k => $v) {
			$GLOBALS['request_string'] .= $val[$k].'='.urlencode($v).'&';
		}
	} else {
		$GLOBALS['request_string'] .= $key.'='.urlencode($val).'&';
	}
}
$GLOBALS['request_string_noprint'] = $GLOBALS['request_string'];
$GLOBALS['request_string'] .= 'print=true';
// end printing


global $theme, $themeInstance;
$themeInstance = $pageController->create_theme_instance();
$GLOBALS['log']->debug('current theme is: '.$themeInstance->theme_name);

// set user theme and language cookies as defaults for login screen
if(isset($_SESSION['authenticated_user_id'])) {
	$ck_expire = time() + 86400 * 90;
	if(isset($_SESSION['authenticated_user_language'])) {
		if(AppConfig::detect_https() == 'https')
	       	setcookie('ck_login_language', $_SESSION['authenticated_user_language'], $ck_expire, '/', AppConfig::request_host_name(), FALSE, TRUE);
	    else
	       	setcookie('ck_login_language', $_SESSION['authenticated_user_language'], $ck_expire, '/', AppConfig::request_host_name(), FALSE, FALSE);
	}
	if($themeInstance->theme_name) {
		if(AppConfig::is_mobile()) {
			if(AppConfig::detect_https() == 'https')
		       	setcookie('ck_mobile_theme', $themeInstance->theme_name, $ck_expire, '/', AppConfig::request_host_name(), FALSE, TRUE);
		    else
		       	setcookie('ck_mobile_theme', $themeInstance->theme_name, $ck_expire, '/', AppConfig::request_host_name(), FALSE, FALSE);
		}
		else if(empty($_SESSION['no_persist_theme'])) {
			if(AppConfig::detect_https() == 'https')
		       	setcookie('ck_login_theme', $themeInstance->theme_name, $ck_expire, '/', AppConfig::request_host_name(), FALSE, TRUE);
		    else
		       	setcookie('ck_login_theme', $themeInstance->theme_name, $ck_expire, '/', AppConfig::request_host_name(), FALSE, FALSE);
		}
	}
}


$themeInstance->init_render();

global $pageInstance;
$pageInstance = $pageController->create_page_instance();
$pageInstance->std_init();
$pageInstance->add_standard_headers($skipHeaders);


// Set module shortcuts menu to an empty array, in case
// we don't have access to the module
if(!isset($module_menu)) $module_menu = array();


// longreach - added - more descriptive title
if(AppConfig::current_user_id()) {
	if(isset($app_list_strings['moduleList'][$currentModule]))
		$title = $app_list_strings['moduleList'][$currentModule];
	else if(isset($subModuleList[$currentModule])
			&& isset($app_list_strings['moduleList'][$subModuleList[$currentModule]]))
		$title = $app_list_strings['moduleList'][$subModuleList[$currentModule]];
	else
		$title = $app_list_strings['moduleList']['Administration'];
	$app_strings['LBL_BROWSER_TITLE'] =
		$app_strings['LBL_SHORT_BROWSER_TITLE']. ': '. $title;
}
// longreach - end added

if(!empty($_SESSION['HomeOnly']))
	$moduleList = array('Home');
if(!isset($modListHeader))
	$modListHeader = get_user_module_list();


$tasks = AppConfig::setting('pre_render_tasks');
foreach ($tasks as $task) {
	if (array_get_default($task, 'enabled')) {
		require_once $task['file'];
		call_user_func($task['function']);
	}
}

try {
while(isset($nextDirective)) {
	if(! $pageController->check_access())
		throw new IAHPermissionError('LBL_MODULE_NO_ACCESS');

	if(! is_array($nextDirective)) {
		if(! $nextDirective)
			$nextDirective = array('not_supported', array());
		else
			$nextDirective = array($nextDirective, array());
	}
	if(count($nextDirective) < 2 || ! is_array($nextDirective[1])) $nextDirective[1] = array();
	if($nextDirective[0] === 'not_supported')
		throw new IAHPermissionError('LBL_ACTION_NOT_SUPPORTED');

	if($nextDirective[0] === 'redirect')
		indexRedirect($nextDirective[1]);
	indexCheckRawOutput($nextDirective[1]);

	if(isset($_GET['pdf_print'])) {
		handlePDFPrint();
	}

	if($nextDirective[0] != 'location_replace')
		$pageInstance->before_render();

	$GLOBALS['sugar_die_exception'] = 'IAHError';
	ob_start();
	if($nextDirective[0] === 'call') {
		$m = $nextDirective[1]['method'];
		$nextDirective = $pageController->$m();
	}
	else if($nextDirective[0] === 'perform') {
		AppConfig::init_page_controller($nextDirective[1]);
		$pageInstance->module = $pageController->get_module();
		$pageInstance->action = $pageController->get_action();
		$nextDirective = $pageController->get_directive();
	}
	else if($nextDirective[0] === 'include') {
		$currentModuleFile = array_get_default($nextDirective[1], 'path');
		if(! $currentModuleFile || ! file_exists($currentModuleFile))
			throw new IAHPermissionError('LBL_ACTION_NOT_SUPPORTED');
		$GLOBALS['log']->debug('--------->  BEGING INCLUDING REQUESTED PAGE: ['.$currentModuleFile.']  <------------');
		$nextDirective = include($currentModuleFile);
		$GLOBALS['log']->debug('--------->  END INCLUDING REQUESTED PAGE: ['.$currentModuleFile.']  <------------');
	}
	else if($nextDirective[0] === 'location_replace') {
		indexRedirect($nextDirective[1], true);
	}
	else
		$nextDirective = null;
	unset($GLOBALS['sugar_die_exception']);
	$content = ob_get_clean();

	if (!empty($_SESSION['FLASH_MESSAGES']) && (!$skipHeaders || !empty($GLOBALS['forceFlashMessages']))) {
		if (function_exists('display_flash_messages')) {
			// silly trick to silence Intellephence
			$display_flash_messages = 'display_flash_messages';
			$display_flash_messages($_SESSION['FLASH_MESSAGES']);
		} else {
			display_flash_messages_default($_SESSION['FLASH_MESSAGES']);
		}
		$_SESSION['FLASH_MESSAGES'] = array();
	}

	echo $content;

	$pageInstance->after_render();

	$hdrs = headers_list();
	if($hdrs) foreach($hdrs as $h) {
		if(preg_match('/^Location:/i', $h))
			sugar_cleanup(true);
	}

	if(isset($focus) && is_subclass_of($focus, 'SugarBean') && $focus->bean_implements('ACL')) {
		if(!$doRawOutput) {
			if (isset($focus->assigned_user_id)) $_uid = $focus->assigned_user_id;
			elseif (isset($focus->created_by)) $_uid = $focus->created_by;
			else $_uid = true;

			require_once("modules/SecurityGroups/SecurityGroup.php");
			$in_group = SecurityGroup::groupHasAccess($focus->module_dir,$record);
			ACLController::addJavascript($focus->module_dir, '', $_uid, $in_group);
		}
	}

	if(! is_string($nextDirective) && ! is_array($nextDirective))
		$nextDirective = null;

} // end while
} catch(IAHInternalError $e) {
	end_clean_all_buffers();
	$log->fatal($e->getErrorName().': '.$e->getMessage().' ('.$e->getFile().':'.$e->getLine().')');
	if(AppConfig::setting('site.log.detail_internal_errors')) {
		pr2($e->getTraceAsString(), $e->getErrorName().': '.$e->getMessage());
	}
	exit;
} catch(IAHError $e) {
	$log->error($e->getErrorName().': '.$e->getMessage().' ('.$e->getFile().':'.$e->getLine().')');
	$msg_w = '60%';
	theme_hide_side_menu(true, true);
	$handlers = array_reverse(ob_list_handlers());
	end_clean_all_buffers();
	ob_start();
	if(! defined('ASYNC_ENTRY')) {
		$pageInstance->finalize_page();
		if($skipFooters) {
			insert_popup_header($theme);
			$msg_w = '90%';
		} else if($skipHeaders)
			include ('themes/'.$theme.'/header.php');
	}
	require_once('XTemplate/xtpl.php');
	$tpl = new XTemplate('include/utils/FlashMessages.html');
	$tpl->assign('WIDTH', $msg_w);
	$tpl->assign('ICON', '<span class="uii uii-danger"></span>');
	$tpl->assign('PANEL_CLASS', 'panel-danger');
	$_msg = $e->getMessage();
	if(AppConfig::setting('site.log.stack_trace_errors'))
		$_msg .= "\n<pre style='width: 60em; overflow: auto'>".htmlentities($e->getTraceAsString(), ENT_QUOTES, 'UTF-8').'</pre>';
	else if(AppConfig::setting('site.log.detail_errors')) {
		$_msg .= "\n<br><br>".htmlentities("module: $module, action: $action");
		$_msg .= "\n<br><br>".htmlentities($e->getErrorName() . ': ' . $e->getFile() . ':' . $e->getLine(), ENT_QUOTES, 'UTF-8');
	}
	$tpl->assign('MESSAGE', $_msg);
	$redir = $e->redirect_home;
	if($redir && ($pageInstance->action == 'Popup' || ! empty($_REQUEST['in_popup']))) $redir = false;
	if($redir) {
		$seconds = $e->seconds;
		$redir = str_replace('{TIME}', "<span id=\"seconds_left\">$seconds</span>", translate('LBL_REDIRECT_COUNTDOWN'));
		$tpl->assign('ETC', '<br /><br />'.$redir.'<script> function redirect_countdown(left){left--;var elt = $("seconds_left"); if(! elt) return; elt.innerHTML = left; if(left == 0){document.location.href = "index.php"; return;}else{setTimeout("redirect_countdown("+ left+")", 1000)}};setTimeout("redirect_countdown('.$seconds.')", 1000)</script>');
	}
	$tpl->parse('static.message');
	$tpl->parse('static');
	$tpl->out('static');
	define('IAH_FATAL_ERR', 1);
}


?>
