<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormSection.php');

class ActivityButtonsWidget extends FormSection {
	var $module;
	var $actions;
	var $check_fields = array();
	var $autoload_actions = true;
	
	function init($params=null, $model=null) {
		parent::init($params, $model);
	}

	function getRequiredFields() {
		$fields = parent::getRequiredFields();
		foreach($this->check_fields as $cf) {
			if($cf) $fields[] = $cf['name'];
		}
		return $fields;
	}
	
	function setModel(ModelDef $model, $recurse=false) {
		parent::setModel($model, $recurse);
		if(! $this->model) return;
		$mname = $this->model->getModelName();
		$this->module = $this->model->getModuleDir();
		if(AppConfig::module_primary_bean($this->module) != $mname)
			return;
		$this->check_fields['do_not_call'] = $this->model->getFieldDefinition('do_not_call');
		$this->check_fields['email_opt_out'] = $this->model->getFieldDefinition('email_opt_out');
		$this->check_fields['invalid_email'] = $this->model->getFieldDefinition('invalid_email');
		if($this->autoload_actions) {
			$layout = AppConfig::setting("views.layout.{$this->module}.record.Standard");
			if($layout && ! empty($layout['summary']) && ! empty($layout['summary']['actions']))
				$this->loadActions($layout['summary']['actions']);
		}
	}
	
	function checkField($name, $record) {
		if(empty($this->check_fields[$name]))
			return;
		$nm = $this->check_fields[$name]['name'];
		return $record->getField($nm);
	}
	
	function loadActions($info) {
		// loaded from form layout
		$this->actions = array();
		if(is_array($info)) {
			foreach($info as $cat => $acts) {
				if(! is_array($acts)) $acts = array();
				$this->actions[$cat] = $acts;
			}
		}
	}
	
	function quickCreateOptions() {
		return array(
			'email' => array('link_name' => 'emails', 'module' => 'Emails', 'vname' => 'LNK_COMPOSE_EMAIL'),
			'meeting' => array('link_name' => 'meetings', 'module' => 'Meetings', 'vname' => 'LBL_SCHEDULE_MEETING_BUTTON_LABEL'),
			'call' => array('link_name' => 'calls', 'module' => 'Calls', 'vname' => 'LBL_SCHEDULE_CALL_BUTTON_LABEL'),
			'task' => array('link_name' => 'tasks', 'module' => 'Tasks', 'vname' => 'LNK_NEW_TASK', 'layout' => 'Standard'),
			'note' => array('link_name' => 'notes', 'module' => 'Notes', 'vname' => 'LNK_NEW_NOTE'),
		);
	}

	function getButtons(RowResult $row_result) {
		global $current_user;
		$rel_acts = $this->actions;
		$user_mods = get_user_module_list(null, true);
		$qc = $this->quickCreateOptions();
		$json = getJSONobj();
		$record_id = $row_result->getPrimaryKeyValue();
		$buttons = array();
		foreach($qc as $cat => $qc_info) {
			$qc_spec = array_get_default($rel_acts, $cat, array());
			if(! empty($qc_spec['disabled'])) continue;
			if(! ACLController::checkAccess($qc_info['module'], 'create')) continue;
			if(AppConfig::is_mobile() && ! AppConfig::is_mobile_module($qc_info['module'])) continue;
			if(! isset($user_mods[$qc_info['module']])) continue;
			$ldef = $this->model->getLinkRelationship($qc_info['link_name']);
			$qcl_name = array_get_default($qc_info, 'layout', 'QuickCreate');
			$qc_layout = AppConfig::setting("views.detail.{$qc_info['module']}.edit.$qcl_name");
			$pbean = AppConfig::module_primary_bean($qc_info['module']);
			if($ldef && $ldef['right']['model'] == $pbean) {
				if($qc_layout && ! AppConfig::is_mobile()) {
					$qparams = array(
						'target_name' => 'subpanel_by_name',
						'target_id' => array('history', 'activities'),
						'primary_name' => 'related_id',
						'return_panel' => 'activities',
						'parent_type' => $this->module,
						'parent_id' => $record_id,
						'return_panel' => 'activities',
						'return_module' => $this->module,
						'return_action' => 'DetailView',
						'return_record' => $record_id,
					);
					if($qc_info['module'] == 'Tasks' && $this->module == 'Users') {
						$qparams['parent_type'] = null;
						$qparams['parent_id'] = null;
						$qparams['assigned_user_id'] = $record_id;
					}
					$qc_params = $json->encode($qparams);
					$perform = "SUGAR.ui.openQuickCreate('{$qc_info['module']}', $qc_params)";
				}
				else {
					$qparams = array(
						'module' => $qc_info['module'],
						'action' => 'EditView',
						'parent_type' => $this->module,
						'parent_id' => $record_id,
						'return_panel' => 'activities',
						'return_module' => $this->module,
						'return_action' => 'DetailView',
						'return_record' => $record_id,
					);

					if($qc_info['module'] == 'Emails' && $this->module == 'Users') {
						$qparams['parent_type'] = null;
						$qparams['parent_id'] = null;
						$qparams['user_id'] = $record_id;
					}
					$url = javascript_escape('index.php?' . encode_query_string($qparams));
					$perform = "SUGAR.util.loadUrl('$url');";
				}

				if($qc_info['module'] == 'Emails') {
					$client = $current_user->getPreference('email_link_type');
					if (!$client)
						$client = AppConfig::setting('email.default_client');
					if (AppConfig::is_mobile())
						if(!AppConfig::is_mobile_module('Emails'))
							$client = '';
					if ($client != 'sugar') {
						require_bean('Email');
						$addr = Email::load_to_addr($qparams);
						$perform = "window.location.href=" . $json->encode('mailto:' . $addr['to']) . ";";
					}
				}

				$btitle = translate($qc_info['vname'], 'Activities');
				$btn = array('icon' => 'input-icon theme-icon module-'.$qc_info['module'], 'perform' => $perform, 'title' => $btitle);
				
				if($cat == 'call' && $this->checkField('do_not_call', $row_result)) {
					$btn['disabled'] = true;
				}
				if($cat == 'email' && ($this->checkField('email_opt_out', $row_result) || $this->checkField('invalid_email', $row_result))) {
					$btn['disabled'] = true;
				}
				$buttons[] = $btn;
			}
		}
		return $buttons;
	}
	
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if($this->getEditable($context) || $gen->getLayout()->getEditingLayout())
			return '';
		$buttons = $this->getButtons($row_result);
		$ret = '';
		if($buttons) {
			foreach($buttons as $b) {
				$dis = (! empty($b['disabled']) ? ' disabled="disabled"' : '');
				$icon = $b['icon'];
				if(! $dis) $icon .= ' active';
				$ret .= '<div class="inline-elt"><button class="input-button activity-button muted" type="button" title="'.to_html($b['title']).'" onclick="' . to_html($b['perform']) . '"' . $dis . '><div class="' . $icon . '"></div></button></div>';
			}
		}
		if(isset($this->actions['social'])) {
			/*if($editable && $this->social) {
				$sw = $this->getElement($this->social);
				if($sw) {
					$brow .= $sw->renderHtmlCell($gen, $row_result);
				}
			}
			else {*/
				require_once('modules/SocialAccounts/SocialAccount.php');
				$sa = new SocialAccount(array('related_type' => $row_result->getModuleDir(), 'related_id' => $row_result->getPrimaryKeyValue()));
				$icons = $sa->getIconsList(false, '');
				foreach($icons as $i)
					$ret .= '<div class="inline-elt">'.$i.'</div>';
		}
		/*if(! $editable && ! $edit_layout && isset($this->activity_date) && ($f = $this->getElement($this->activity_date))) {
			if($f->getVisible()) {
				$v = $f->renderHtml($gen, $row_result, $subpath, $ctxedit);
				if(strlen($v)) {
					$brow .= ' &nbsp;<div class="summaryButtonsMeta"><span class="inlineLabel">' . $f->getLabel() . $sep . '</span>';
					$brow .= $v . '</div>';
				}
			}
		}*/
		if($ret) {
			$ret = '<div class="form-buttons">'
				 . '<div class="row buttons-row">'
				 . '<div class="column buttons-inline">'
				 . $ret
				 . '</div></div></div>';
		}
		return $ret;
	}
		
    function renderListValue(ListFormatter $fmt, ListResult $result, $row_id, $list_params=null) {
    }	
}

