<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormField.php');

class AddressFormatInput extends FormField {	
	var $params;
	var $address_format = 'display';
	var $address_templates = null;
	
	function init($params=null, $model=null) {
		parent::init($params, $model);
		if(is_array($params))
			$this->params = $params;
	}
	
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if(empty($this->name))
			return '';
		if($this->getEditable($context))
			$value = $this->renderHtmlEdit($gen, $row_result, $context);
		else
			$value = $this->renderHtmlView($gen, $row_result, $context);
		return $value;
	}
	
	function getRequiredFields() {
		return array($this->name);
	}
	
	function renderHtmlEdit(HtmlFormGenerator $gen, RowResult $row_result, array $context) {
		$form =& $gen->getFormObject();
		$gen->getLayout()->addScriptInclude('include/javascript/locale.js');

		$name = $form->getFieldNamePrefix() . $this->name;
		$sel_id = $form->getFieldIdPrefix() . $name.'-input';
		
		$ret = EditableForm::get_tag('div', array('id' => $sel_id));
		global $locale;
		$info = $locale->getLocaleAddressFormatInfo($this->address_format, $this->address_templates);
		$opts = array();
		foreach($info['options'] as $k => $v) {
			$opts[$k] = array('name' => $v, 'sample' => $info['samples'][$k]);
		}
		$spec = array(
			'name' => $name,
			'type' => 'enum',
			'options' => $opts,
			'input_class' => 'AddressFormatInput',
			'options_add_blank' => false,
		);
		$value = array_get_default($row_result->row, $this->name);
		if(! isset($opts[$value])) {
			$value = $locale->getLocaleDefaultAddressFormat();
		}
		$ret .= $form->renderSelect($spec, $value);
				
		$disp_id = $form->getFieldIdPrefix() . $name . '-sample';
		$sample = array_get_default($info['samples'], $value, '');
		$ret .= "<div id=\"$disp_id\" class=\"sampleValue\">$sample</div>";
		$ret .= '</div>';
		
		return $ret;
	}
	
	function renderHtmlView(HtmlFormGenerator $gen, RowResult $row_result, array $context) {
		global $locale;
		$value = array_get_default($row_result->row, $this->name);
		$format = $this->address_format;
		if($value) {
			$tpl = $locale->getLocaleAddressTemplate($format, $value);
			if($tpl) {
				$addr = $locale->getLocaleSampleAddress($format);
				$name = $locale->translateAddressFormatName($value);
				$sample = $locale->getLocaleFormattedAddress($addr, '', $format, $tpl);
				return to_html($name) . '<div class="sampleValue">' . $sample . '</div>';
			}
		}
	}
	
}
