<?php

require_once('include/layout/forms/FormField.php');


class ContactStatusInput extends FormField {
	var $opt_out_spec, $opt_out_name;
	var $opt_in_spec, $opt_in_name;
	var $opt_in_date_spec, $opt_in_date_name;
	var $invalid_email_spec, $invalid_email_name;

	function init($params=null, $model=null) {
		parent::init($params, $model);
		if(! $this->vname) {
			$this->vname = ($this->name == 'email_status' ? 'LBL_EMAIL_STATUS' : 'LBL_PHONE_STATUS');
			$this->vname_module = 'Contacts';
		}
	}
	
	function getRequiredFields() {
		return array($this->opt_out_name, $this->opt_in_name, $this->opt_in_date_name, $this->invalid_email_name);
	}
	
	function loadModelFields() {
		if($this->name == 'phone_status') {
			$test = array('opt_out' => 'do_not_call');
		} else {
			$test = array('opt_out' => 'email_opt_out', 'opt_in' => 'email_opt_in', 
				'opt_in_date' => 'email_opt_in_date', 'invalid_email' => 'invalid_email');
		}
		foreach($test as $f => $v) {
			$spec_name = $f . '_spec';
			$name = $f . '_name';
			$spec = $this->$spec_name = $this->model->getDisplayField($v, true);
			if($spec)
				$this->$name = $spec['name'];
		}
	}

	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if($this->getEditable($context))
			$value = $this->renderHtmlEdit($gen, $row_result, $context);
		else
			$value = $this->renderHtmlView($row_result, $context);
		return $value;
	}
	
	function renderHtmlEdit(HtmlFormGenerator $gen, RowResult $row_result, array $context) {
		$form =& $gen->getFormObject();

		$ret = '';
		$opts = array();
		$sel_opt = '';
		$date = $date_id = '';
		if($this->opt_out_spec) {
			if($this->name == 'phone_status') {
				$chk = $row_result->getField($this->opt_out_name);
				$spec = $this->opt_out_spec;
				$spec['show_label'] = true;
				$spec['quick'] = true;
				$ret .= $form->renderCheck($spec, $chk);
			} else {
				$opts[''] = translate('LBL_EMAIL_STATUS_DEFAULT', 'Contacts');
				$opts['opt_out'] = translate('LBL_EMAIL_STATUS_OPT_OUT', 'Contacts');
				if($row_result->getField($this->opt_out_name))
					$sel_opt = 'opt_out';
			}
		}
		if($this->opt_in_spec && $opts) {
			$opts['opt_in'] = translate('LBL_EMAIL_STATUS_OPT_IN', 'Contacts');
			if($this->opt_in_date_spec) {
				$spec = $this->opt_in_date_spec;
				if($sel_opt !== 'opt_in')
					$spec['style'] = 'display: none';
				$date_val = $row_result->getField($this->opt_in_date_name, null, true);
				$spec['quick'] = true;
				$spec['placeholder'] = translate('LBL_EMAIL_STATUS_OPT_IN_ON', 'Contacts');
				$date = $form->renderDateTime($spec, $date_val);
				$date_id = $form->getFieldIdPrefix() . $this->opt_in_date_name . '-input';
			}
			if($sel_opt != 'opt_out' && $row_result->getField($this->opt_in_name))
				$sel_opt = 'opt_in';
		}
		if($opts) {
			$spec = array('name' => $this->name, 'type' => 'enum', 'options' => $opts);
			$js = 'toggleDisplay("' . javascript_escape($date_id) . '", this.getValue() == \'opt_in\');';
			$js .= 'var f = SUGAR.ui.getFormInput(this.form, "' . $this->opt_in_date_name . '"); if(f && ! f.getValue()) f.setValue(new Date());';
			$spec['onchange'] = $js;
			$spec['quick'] = true;
			//$spec['mark_blank'] = false;
			$ret .= $form->renderSelect($spec, $sel_opt);
		}
		if($date) {
			$dstyle = ($sel_opt === 'opt_in') ? '' : 'display: none';
			$ret .= ' ' . $date;
		}

		if($this->invalid_email_spec) {
			$chk = $row_result->getField($this->invalid_email_name);
			$spec = $this->invalid_email_spec;
			$spec['show_label'] = true;
			$spec['quick'] = true;
			$ret .= '<div style="margin-top: 0.25em">' . $form->renderCheck($spec, $chk) . '</div>';
		}
		
		return $ret;
	}
	
	function renderHtmlView(RowResult $row_result, array $context) {
		$opt_in = $opt_out = $invalid = $stat = false;
		if($this->opt_out_name && $row_result->getField($this->opt_out_name)) {
			if($this->name == 'phone_status' || $row_result->getField('email1') || $row_result->getField('email2'))
				$opt_out = true;
		}
		if($this->opt_in_name && $row_result->getField($this->opt_in_name))
			$opt_in = true;
		if($this->invalid_email_name && $row_result->getField($this->invalid_email_name))
			$invalid = true;

		if($opt_out && ! $opt_in) {
			if($this->name == 'phone_status') 
				$stat .= '<div><em>' . translate('LBL_PHONE_STATUS_DO_NOT_CALL', 'Contacts') . '</em></div>';
			else
				$stat .= '<div><em>' . translate('LBL_EMAIL_STATUS_OPT_OUT_WARN', 'Contacts') . '</em></div>';
		}
		if($opt_in) {
			$opt_in_date = $row_result->getField($this->opt_in_date_name, null, true);
			$stat .= '<div><em>' . translate('LBL_EMAIL_OPT_IN_DATE', 'Contacts') . ': ' . $opt_in_date . '</em></div>';
		}
		if($invalid) {
			$stat .= '<div><em>' . translate('LBL_EMAIL_STATUS_INVALID_WARN', 'Contacts') . '</em></div>';
		}
		if($stat) {
			return $stat;
		}
	}
	
	function loadUpdateRequest(RowUpdate $update, array $input) {
		if($this->name == 'email_status' && isset($input[$this->name])) {
			$v = $input[$this->name];
			if($this->opt_out_name) {
				if($v === '') {
					$update->set($this->opt_out_name, 0);
					if($this->opt_in_name)
						$update->set($this->opt_in_name, 0);
				}
				else if($v === 'opt_out') {
					$update->set($this->opt_out_name, 1);
					if($this->opt_in_name)
						$update->set($this->opt_in_name, 0);
				}
				else if($v == 'opt_in') {
					$update->set($this->opt_out_name, 0);
					if($this->opt_in_name)
						$update->set($this->opt_in_name, 1);
				}
			}
		}
	}
}
