<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormElement.php');

// TODO: add ways to dynamically configure list input from function or link data source

class DynamicListInput extends FormElement {
	var $width = '100%';
	var $show_header = true;
	var $renderer_class;
	var $depth_attrib;
	var $group_attrib;
	var $strip_attrib;
	var $fixed_attrib;
	var $max_depth;
	var $depth_column;
	var $show_handles;
	var $show_action;
	var $show_delete;
	var $require_group_attrib;
	var $onupdate;
	var $cols = array(), $rows = array();
	
	static $opt_attribs = array(
		'show_header', 'renderer_class', 'show_handles', 'show_action', 'show_delete',
		'depth_attrib', 'group_attrib', 'require_group_attrib',
		'strip_attrib', 'fixed_attrib', 'max_depth', 'depth_column', 'onupdate',
	);
	
	function init($params=null, $model=null) {
		parent::init($params, $model);
		if(is_array($params))
		$attrs = array_merge(self::$opt_attribs, array('cols', 'rows'));
		foreach($attrs as $k)
			if(isset($params[$k]))
				$this->$k = $params[$k];
	}
	
	function getRequiredFields() {
		$ret = parent::getRequiredFields();
		return $ret;
	}
		
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if(empty($this->name))
			return '';
		if($this->getEditable($context))
			$value = $this->renderHtmlEdit($gen, $row_result, $context);
		else
			$value = '';
		return $value;
	}
	
	function exportIncludes($layout=null) {
		$script = 'include/javascript/dynamic_list.js';
		if($layout)
			$layout->addScriptInclude($script);
		else
			$GLOBALS['pageInstance']->add_js_include($script, null, LOAD_PRIORITY_BODY);
	}
	
	function renderHtmlEdit(HtmlFormGenerator $gen, RowResult $row_result, array $context) {
		$form =& $gen->getFormObject();
		$this->exportIncludes($gen->getLayout());
		$rows = array_get_default($row_result->row, $this->name, $this->rows);
		return $this->render($gen->getFormObject(), $rows);
	}
	
	function render(EditableForm $form, $rows) {
		$name = $form->getFieldNamePrefix() . $this->name;
		$sel_id = $form->getFieldIdPrefix() . $name.'-input';
		$attribs = array(
			'id' => $sel_id,
			'width' => $this->width,
			'class' => 'listEditor panel-outer panel-default panel-border',
			'cellpadding' => 0,
			'cellspacing' => 0,
		);
		if(isset($this->class))
			$attribs['class'] .= ' '.$this->class;
		$ret = EditableForm::get_tag('input', array('type' => 'hidden', 'name' => $name));
		$ret .= EditableForm::get_tag('table', $attribs);
		$ret .= '<tr><td>' . to_html(translate('LBL_LOADING', 'app')) . '</td></tr>';
		$ret .= '</table>';
		
		$cols = array();
		if(is_array($this->cols)) {
			foreach($this->cols as $k => $c) {
				if(isset($c['label'])) $c['label'] = translate($c['label'], array_get_default($c, 'label_module', 'app'));
				$cols[$k] = $c;
			}
		}

		$attribs = array('name' => $name, 'cols' => $cols, 'rows' => $rows);
		foreach(self::$opt_attribs as $k)
			if(isset($this->$k))
				$attribs[$k] = $this->$k;
		$attribs = array_merge($attribs, $this->getOptionalAttributes());
		$form->addControl('DynamicList', $sel_id, $attribs);
		return $ret;
	}

	protected function getOptionalAttributes() {
		return [];
	}

	
}
