<?php

require_once('include/layout/forms/FormField.php');

class EditTeamsWidget extends FormField {
	var $teams;
	var $new_teams;

	function init($params=null, $model=null) {
		parent::init($params, $model);
	}
	
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		$editable = $gen->getLayout()->getDefaultEditable();
		if($editable && $row_result->base_model == 'User' && ! is_admin()) {
			$editable = false;
		}

		if($this->supportTeams()) {
			$ts = $this->getTeams($row_result, $editable);
			if($ts)
				$t_opts = $ts->getRows();
			else {
				$upd = RowUpdate::for_result($row_result);
				$t_opts = $upd->getUserInheritedTeams(true);
				$t_ids = array_keys($t_opts);
			}
			foreach($t_opts as &$t) {
				$t['_display'] = $t['name'];
				$t['icon'] = 'icon-teams';
			}
			$pk = $row_result->getPrimaryKeyValue();
			$json = getJSONobj();

			if($editable) {
				$form = $gen->getFormObject();
				$spec = array(
					'type' => 'multienum',
					'name' => 'teams',
					'label' => 'teams',
					'options_icon' => 'icon',
					'options' => $t_opts,
					'label_key' => '_display',
					'display_rows' => 4,
					'tabindex' => $this->tabindex,
					'add_remove' => true,
					'options_lookup' => [
						'bean_name' => 'SecurityGroup',
					],
				);
				$k = implode('^,^', array_keys($t_opts));
				$html = $form->renderMultiSelect($spec, $k);
				
				if($this->model && AppConfig::setting('aclconfig.inherit_account') && AppConfig::setting('aclconfig.inherit_account_replace')) {
					$mname = $this->model->getModelName();
					$pacct = AppConfig::setting("model.detail.$mname.primary_account");
					if($pacct) {
						$pacct = javascript_escape($pacct);
						$form->addInit("SUGAR.ui.onInitForm('{$form->getFormName()}', function(frm) { SUGAR.ui.attachTeamsUpdate(frm, ['$pacct', 'assigned_user']); });");
					}
				}
				return $html;
			}
			
			$html = '';
			if($ts) {
				$rows = $this->sortRows($ts->getRows(true));
				$ri = 0;
				foreach($rows as $idx => $t) {
					$ri ++;
					$c = ($ri > 4) ? 'expand-show' : '';
					$html .= "<li class=\"withIcon $c\"><div class=\"input-icon icon-teams active\"></div>&nbsp;{$t['name']}</li>";
				}
				if($ri > 4) {
					$lbl = to_html(translate('LBL_LIST_SHOW_MORE', 'app'));
					$html .= "<li><a href=\"#\" class=\"utilsLink\" onclick=\"SUGAR.ui.toggleExpandList('summaryOwner', this);\">$lbl</a></li>";
				}
			}
			if(0) {
				$lbl = translate('LBL_ADD_TEAM', 'app');
				$popup_request_data = array(
					'call_back_function' => 'update_subpanel',
					'form_name' => $gen->getFormName(),
					'field_to_name_array' => array('record_id' => $pk),
					'passthru_data' => array(
						'reload_page' => 1,
						'bean_name' => 'SecurityGroup',
						'link_name' => 'securitygroups',
					),
				);
				$params = $json->encode($popup_request_data);
				$lnk = to_html("return open_popup(\"SecurityGroups\",undefined,undefined,\"\",true,true,$params,\"MultiSelect\");");
				$html .= "<li><a href='#' class=\"utilsLink\" onclick=\"$lnk\"><div class=\"input-icon icon-add\"></div>&nbsp;$lbl</a></li>";
			}
			return $html;
		}
	}
	
	function supportTeams() {
		if($this->model) return $this->model->haveLinkDefinition('securitygroups');
	}
	
	function sortRows($rows) {
		$mine = $other = array();
		$teams = array_keys(AppConfig::setting('acl.user_teams.'.AppConfig::current_user_id(), array()));
		foreach($rows as $pk => $t) {
			if(in_array($pk, $teams)) {
				$t['mine'] = true;
				$mine[$pk] = $t;
			}
			else
				$other[$pk] = $t;
		}
		return array_merge($mine, $other);
	}
	
	function getTeams($row_result, $editable=false) {
		if(! isset($this->teams)) {
			$this->teams = false;
			$id = $row_result->getPrimaryKeyValue();
			if($id) {
				$teams = $row_result->getLinkResults('securitygroups', array('id', 'name'), array('order_by' => 'name'));
				if($teams->getResultCount()) {
					$fmt = new FieldFormatter('html', $editable ? 'editview' : 'view');
					$fmt->list_id = '_'; // unique element IDs
					$fmt->primary_key = $teams->primary_key;
					$fmt->module_dirs = $teams->module_dirs;
					if(! $editable) $fmt->add_detail_links = 'all';
					$fmt->parent_module = $row_result->getModuleDir();
					$fmt->parent_action = 'DetailView';
					$fmt->parent_record = $row_result->getPrimaryKeyValue();
					//$fmt->parent_layout = $this->link_parent_info['layout'];
					$teams->formatted_rows = $fmt->formatRows($teams->fields, $teams->rows);
				} else
					$teams->rows = $teams->formatted_rows = array();
				$this->teams = $teams;
			}
		}
		return $this->teams;
	}
	
	function loadUpdateRequest(RowUpdate $update, array $input) {
		$ret = parent::loadUpdateRequest($update, $input);
		if(isset($input['teams'])) {
			$t = $input['teams'];
			if(! is_array($t))
				$t = explode('^,^', $t);
			$this->new_teams = $t;
		}
		return $ret;
	}
	
	function afterUpdate(RowUpdate $update) {
		$ret = parent::afterUpdate($update);
		$disable_edit = false;
		if($update->getModelName() == 'User' && ! is_admin()) {
			$disable_edit = true;
		}
		if($this->supportTeams() && ! $disable_edit) {
			$ts = $this->getTeams($update);
			if($ts && isset($this->new_teams)) {
				global $db;
				$old_ids = array_keys($ts->getRows());
				$add = array_diff($this->new_teams, $old_ids);
				$remove = array_diff($old_ids, $this->new_teams);
				if($add)
					$update->addUpdateLinks('securitygroups', $add);
				if($remove && !$update->new_record) {
					$update->removeLinks('securitygroups', $remove);
				}

				if ($update->model->audit_enabled && AppConfig::setting('site.audit_teams'))
					do {
						$audit_table = $update->model->getAuditTableName();
						if (!$audit_table) {
							break;
						}

						$audit_row = [
							'id' => create_guid(),
							'parent_id' => $update->getPrimaryKeyValue(),
							'field_name' => '!teams!',
							'data_type' => 'teams',
							'before_value_text' => join(',', $old_ids),
							'after_value_text' => join(',', $this->new_teams),
							'date_created' => gmdate("Y-m-d H:i:s"),
							'created_by' => AppConfig::current_user_id(),
						];


						$auditModel = new ModelDef('audit');
						$auditModel->table_name = $audit_table;
						$db->insertRow($auditModel, $audit_row);
					} while(false);
			}
		}
		return $ret;
	}
}
/*
 */
?>
