<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormField.php');
require_once('include/utils/oauth_utils.php');

use League\OAuth2\Client\Token\AccessToken;

class EmailAuthInput extends FormField {
	private $token_input_id;
	private $service_field_name;

	function init($params=null, $model=null) {
		$this->token_input_id = uniqid("o365_token_input_");
		$this->service_field_name = $params['email_service_field'];
		return parent::init($params, $model);
	}

	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		$html = parent::renderHtml($gen, $row_result, $parents, $context);
		if($this->getEditable($context))
			$value = $this->renderHtmlEdit($gen, $row_result, $context);
		else
			$value = $this->renderHtmlView($gen, $row_result, $context);

		return $value;
	}

	public function renderHtmlEdit(HtmlFormGenerator $gen, RowResult $row_result, array $context) {
		static $script_exported;

		$encoded_token = $row_result->getField($this->name);
		$status = $this->tokenStatus($encoded_token);
		$success_message = to_html($status['success'] ?? "");
		$error_message = to_html($status['error'] ?? "");
		$button_label = to_html(translate("LBL_O365_AUTH_BUTTON_LABEL", 'Users'));

		$urls = [];
		foreach (["o365", "gmail"] as $s) {
			$urls[$s] = buildOAuth2Uri($s, ['input' => $this->token_input_id]);
		}

		$html = '<input type="hidden" id="' . $this->token_input_id . '" name="' . $this->name . '" value="' . to_html($encoded_token)  .  '">';
		$html .= '&nbsp;<button id="o365_auth_btn" type="button" onclick="start_email_oauth(this.form, ' . to_html(json_encode($urls)) . ', \'' . $this->service_field_name . '\'); return false;" class="input-button">' . $button_label . '</button>';
		$html .= '<div style="color: green" id="' . $this->token_input_id . '_success_message">' . $success_message . '</div>';
		$html .= '<div style="color: red" id="' . $this->token_input_id . '_error_message">' . $error_message . '</div>';
		if (!$script_exported) {
			$html .= <<<SCRIPT
<script type="text/javascript">
function o365_auth_result(result, input_id) {
	var error_message, success_message, success_elt, error_elt, token_elt;
	if (result && result.success) {
		success_message = result.message
	} else if (result && !result.success) {
		error_message = result.message;
	} else {
		error_message = SUGAR.language.getString('LBL_O365_AUTH_UNEXPECTED_ERROR', 'Users');
	}
	token_elt = document.getElementById(input_id);
	success_elt = document.getElementById(input_id + '_success_message');
	error_elt = document.getElementById(input_id + '_error_message');
	if (success_message) {
		success_elt.innerText = success_message;
		success_elt.style.display = '';
		error_elt.style.display = 'none';
		var old_token = {};
		try{
			old_token = JSON.parse(token_elt.value);
		} catch (e) {}
		var new_token =  result.token;
		if (!new_token.refresh_token && old_token.refresh_token)
			new_token.refresh_token = old_token.refresh_token;
		token_elt.value = JSON.stringify(new_token);
	} else {
		error_elt.innerText = error_message;
		error_elt.style.display = '';
		success_elt.style.display = 'none';
	}
}

function start_email_oauth(form, urls, service_field_name) {
	var input = SUGAR.ui.getFormInput(form, service_field_name);
	var service = input.getValue();
	var url = urls[service];
	open_popup_window({url: url});
}
</script>
SCRIPT;
			$script_exported = true;
		}
		return $html;
	}

	public function renderHtmlView(HtmlFormGenerator $gen, RowResult $row_result, array $context) {
		$encoded_token = $row_result->getField($this->name);
		$status = $this->tokenStatus($encoded_token);
		if (!empty($status['error'])) {
			$html = '<div style="color: red">' . to_html($status['error']) . '</div>';
		} else {
			$html = '<div style="color: green">' . to_html($status['success']) .  ' </div>';
		}
		return $html;
	}

	protected function tokenStatus($encoded_token) {
		$token = null;
		$token_array = @json_decode($encoded_token, true);
		if ($token_array) {
			try {
				$t = new AccessToken($token_array);
				if ($t->hasExpired()) {
					$expires = $t->getExpires();
					// Assuming access token expiration is 1 hour, and refresh token expiration is 90 days
					if ($expires + 89 * 24 * 60 * 60 <= time())  {
						return ['error' => translate('LBL_O365_AUTH_EXPIRED_TOKEN', 'Users')];
					}
				}
				$token = $t;
			} catch (Exception $e) {
				pr2($e->getMessage());
			}
		}
		if (!$token) {
			return ['error' => translate('LBL_O365_AUTH_INVALID_TOKEN', 'Users')];
		}
		return ['success' => translate('LBL_O365_AUTH_VALID_TOKEN', 'Users')];
	}

}
?>
