<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormElement.php');

// TODO: add ways to dynamically configure list input from function or link data source

class FieldSelectInput extends FormElement {
	var $icon = 'uii-add', $width, $button_label, $field_model, $list_name, $sel_mode, $context, $custom_source, $wf_module;
	
	function init($params=null, $model=null) {
		parent::init($params, $model);
		if(is_array($params))
		foreach(array('icon', 'width', 'button_label', 'field_model', 'list_name', 'sel_mode', 'context', 'custom_source', 'wf_module') as $k)
			if(isset($params[$k]))
				$this->$k = $params[$k];
		if(! $this->sel_mode) $this->sel_mode = 'fields';
	}
	
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if(empty($this->name))
			return '';
		if($this->getEditable($context))
			$value = $this->renderHtmlEdit($gen, $row_result, $context);
		else
			$value = '';
		return $value;
	}
	
	function renderHtmlEdit(HtmlFormGenerator $gen, RowResult $row_result, array $context) {
		$form =& $gen->getFormObject();
		$gen->getLayout()->addScriptInclude('include/javascript/dynamic_list.js');

		$name = $form->getFieldNamePrefix() . $this->name;
		$sel_id = $form->getFieldIdPrefix() . $name.'-input';
		
		$attrs = array(
			'type' => 'button', 'class' => 'input-button rbullet rbullet-ddown', 'id' => $sel_id);
		foreach(array('style', 'disabled', 'tabindex') as $f)
			if(isset($this->$f) && ($f != 'disabled' || $this->$f))
				$attrs[$f] = $spec[$f];
		if(isset($this->class))
			$attrs['class'] .= ' '.$this->class;
		$ret = EditableForm::get_tag('button', $attrs);
		
		$attrs = array('class' => 'input-label');
		if(isset($this->width))
			$width = $this->width;
		else
			$width = '10em';
		if($width)
			$attrs['style'] = "width: {$width}";
		$ret .= EditableForm::get_tag('div', $attrs);

		if($this->icon) {
			$icon = EditableForm::get_tag('span', array('class' => 'uii '.$this->icon, 'id' => $sel_id.'-icon'), true);
			$ret .= $icon . '&nbsp;';
		}
		
		$labels = array(
			'sources' => 'LBL_SELECT_SOURCE',
			'fields' => 'LBL_SELECT_FIELD',
			'order' => 'LBL_SELECT_FIELD',
			'filters' => 'LBL_SELECT_FILTER',
			'totals' => 'LBL_SELECT_TOTAL',
			'updates' => 'LBL_SELECT_FIELD',
		);
		
		$label = $this->button_label;
		if(! isset($label))
			$label = translate($labels[$this->sel_mode]);
		if(isset($label)) {
			$ret .= EditableForm::get_tag('span', array('class' => 'input-label', 'id' => $sel_id.'-label'));
			$ret .= to_html($label);
			$ret .= '</span>';
		}
		$ret .= '</div></button>';
		
		if($this->sel_mode == 'filters') {
			$b1 = array(
				'name' => $this->name.'-addgroup',
				'label' => translate('LBL_ADD_FILTER_GROUP'),
				'icon' => $this->icon,
				'register' => true,
			);
			$ret .= '&nbsp;';
			$ret .= $form->renderButton($b1);
		}
		else if($this->sel_mode == 'fields') {
			$b1 = array(
				'name' => $this->name.'-addempty',
				'label' => translate('LBL_ADD_EMPTY_COLUMN'),
				'icon' => $this->icon,
				'register' => true,
			);
			$ret .= '&nbsp;';
			$ret .= $form->renderButton($b1);
		}

		$attribs = array('name' => $name, 'model' => $this->field_model,
			'list_name' => $this->list_name, 'mode' => $this->sel_mode,
			'context' => $this->context, 'custom_source' => $this->custom_source,
			'wf_module' => $this->wf_module);
		$form->addControl('FieldSelectInput', $sel_id, $attribs);
		return $ret;
	}
	
}
