<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

require_once 'include/layout/forms/FormField.php';

class InviteesList extends FormField
{
	var $type_name;
	var $link_name;

	// could be optimized along the lines of ListTeamsWidget

	function renderVal(RowResult $result, $format, $context) {
		$rows = $this->getRows($result, $format, $context);
		return $this->renderRows($rows, $format, $context);
	}
	
	function renderRows($rows, $format, $context) {
		$ret = '';
		foreach ($rows as $row) {
			if ($ret) $ret .= ', ';
			$display = $row['_display'];
			if($format == 'html' && $context != 'email') {
				if ($row['query_source'] == 'users') {
					$model = 'User';
					$module = 'Users';
				} else if($row['query_source'] == 'leads') {
					$model = 'Lead';
					$module = 'Leads';
				} else {
					$model = 'Contact';
					$module = 'Contacts';
				}
				$display = '<div class="input-icon theme-icon bean-' . $model . '" style="vertical-align: text-top"></div>&nbsp;' . $display;
				$display = '<a href="index.php?module=' . $module . '&action=DetailView&record=' . $row['id'] . '" class="tabDetailViewDFLink">'
					. $display . '</a>';

			}
			$ret .= $display;
		}
		return $ret;
	}

	function init($params=null, $model=null) {
		if(isset($params['type_name']))
			$this->type_name = $params['type_name'];
		if(isset($params['link_name']))
			$this->link_name = $params['link_name'];
		parent::init($params, $model);
	}
	
	function loadModelFields() {
		$mname = $this->model->getModelName();
		if(! $this->link_name && ($mname == 'Meeting' || $mname == 'Call'))
			$this->link_name = 'invitees';
	}
	
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if($this->getEditable($context))
			$value = $this->renderHtmlEdit($gen, $row_result, $context);
		else
			$value = $this->renderHtmlView($row_result, $context);
		return $value;
	}
	
	function loadLinkedRows($id, $format, $context) {
		$rows = null;
		if($id) {
			$lq = new ListQuery($this->model->getModelName(), array('_display', 'first_name', 'last_name', 'salutation'), array('link_name' => $this->link_name));
			$lq->setFormatted($format, $context);
			$lq->setParentKey($id);
			$lq->setOrderBy('');
			$res = $lq->runQuery();
			$rows = $res->getRows();
		}
		if (!$rows) $rows = array();
		return $rows;
	}
	
	function getRows(RowResult $result, $format, $context) {
		if($this->link_name && $this->model) {
			$rows = $this->loadLinkedRows($result->getField('id'), $format, $context);
		} else {
			$ids = $result->getField($this->name);
			$types = $result->getField($this->type_name);
			$rows = array();
			if($ids && $types) {
				$ids = preg_split('/\s*;\s*/', $ids);
				$types = preg_split('/\s*;\s*/', $types);
				$qsrc = array();
				foreach($ids as $idx => $id)
					$qsrc[$types[$idx]][] = $id;
				foreach($qsrc as $mod => $mids) {
					$mname = AppConfig::module_primary_bean($mod);
					if($mname) {
						$lq = new ListQuery($mname, array('id', '_display'));
						$lq->addFieldStatic('query_source', strtolower($mod));
						$lq->addSimpleFilter('id', $mids);
						foreach($lq as $ctc)
							$rows[] = $ctc->row;
					}
				}
			}
		}
		return $rows;
	}
	
	function renderHtmlEdit(HtmlFormGenerator $gen, $row_result, array $context) {
		//	$row_result->assign('invitee_ids', '1;67df14e1-e365-5123-499a-4f091699ba06');
		//	$row_result->assign('invitee_types', 'Users;Contacts');

		$rows = $this->getRows($row_result, 'html', 'edit');
		$form = $gen->getFormObject();
		$id = $form->getFieldIdPrefix().'-'.$this->name;
		$refspec = array(
			'name' => 'add_'.$this->name,
			'type' => 'ref',
			'bean_name' => 'Invitee',
			'union_module' => 'Invitees',
			'placeholder' => translate('LBL_ADD_INVITEE', 'Workflow'),
		);
		$ret = $form->renderRef($row_result, $refspec);
		$ids = $types = array();
		foreach($rows as $r) {
			$ids[] = $r['id'];
			$types[] = $r['query_source'] == 'users' ? 'Users' : 'Contacts';
		}
		$ids = implode(';', $ids);
		$types = implode(';', $types);
		$ret .= EditableForm::get_tag('input', array('type' => 'hidden', 'name' => $this->name, 'value' => $ids));
		$ret .= EditableForm::get_tag('input', array('type' => 'hidden', 'name' => $this->type_name, 'value' => $types));
		$ret .= '<div id="'.$id.'" style="width: 25em">';
		foreach($rows as $row)
			$ret .= '<div>'.$this->formatRow($row, false).'</div>';
		$ret .= '</div>';
		$form->addControl('InviteesList', $id, "{name: '{$this->name}', type_name: '{$this->type_name}'}");
		return $ret;
	}
	
	function formatRow($row, $link=true) {
		if ($row['query_source'] == 'users') {
			$model = 'User';
			$module = 'Users';
		} else {
			$model = 'Contact';
			$module = 'Contacts';
		}
		$ret = '<div class="input-icon theme-icon bean-' . $model . '" style="vertical-align: text-top"></div>&nbsp;';
		$label = to_html($row['_display']);
		if($link)
			$label = '<a href="index.php?module=' . $module . '&action=DetailView&record=' . $row['id'] . '" class="tabDetailViewDFLink">'
			. $label . '</a>';
		$ret .= $label;
		return $ret;
	}

	function renderHtmlView(RowResult $row_result, array $context)
	{
		return $this->renderVal($row_result, 'html', 'view');
	}
	
	function formatField(FieldFormatter $fmt, RowResult $result, $row_id=null) {
		$id = $result->getField('id');
		return $this->renderVal($result, $fmt->getFormat(), $fmt->getContext());
	}
}
