<?php

require_once('include/layout/forms/FormField.php');

class ListTeamsWidget extends FormField {

	private $list_cache = [];

	function getLabel($context=null) {
		$l = parent::getLabel($context);
		$default = translate('LBL_SECURITYGROUPS');
		if(! $l) $l = $default;
		return $l;
	}

	function getWidth() {
		return 50;
	}
	
	function fetchTeams(FieldFormatter $ofmt, $ids) {
		if(! $this->model)
			return;
		foreach($ids as $id)
			if($id) $this->list_cache[$id] = array();
		$mod = $this->model->getModuleDir();
		$model = $this->model->getModelName();
		$lq_params = array(
			'link_name' => 'securitygroups',
			'use_key_index' => false,
		);
		$lq = new ListQuery($model, array('id', 'name', '~join.record_id'), $lq_params);
		$lq->addSimpleFilter('~join.module', $mod);
		$lq->addSimpleFilter('~join.record_id', $ids);
		$teams = $lq->runQuery();
		if($teams->getResultCount()) {
			$fmt = new FieldFormatter($ofmt->getFormat(), $ofmt->getContext());
			$fmt->primary_key = $teams->primary_key;
			$fmt->module_dirs = $teams->module_dirs;
			$fmt->parent_module = $mod;
			$fmt->parent_action = 'DetailView';
			$fmt->parent_record_dynamic = '~join.record_id';
			//$fmt->parent_layout = $this->link_parent_info['layout'];
			$fmt->formatListResult($teams);
			return $teams;
		}
	}
	
	function getIdName() {
		$id_name = 'id';
		if($this->model && $this->model->link_spec)
			$id_name = $this->model->link_spec['name'].'.id';
		return $id_name;
	}

	function formatList(FieldFormatter $fmt, ListResult $result) {
		if($result->rows && $this->model) {
			$pids = array();
			$cache = array();
			$id_name = $this->getIdName();
			foreach($result->rows as $row) {
				$pid = array_get_default($row, $id_name);
				$id = array_get_default($row, 'id');
				if($pid)
					$pids[$pid][] = $id;
				$cache[$id] = array();
			}
			$teams = $this->fetchTeams($fmt, array_keys($pids));
			if($teams) {
				foreach($teams->formatted_rows as $row) {
					$pid = $row['~join.record_id'];
					if($pid && isset($pids[$pid])) {
						foreach($pids[$pid] as $rid)
							$cache[$rid][] = $row;
					}
				}
				$this->list_cache = $cache;
			}
		}
		parent::formatList($fmt, $result);
	}
	
	function formatField(FieldFormatter $fmt, RowResult $result, $row_id=null) {
		$ret = '';
		if(isset($row_id) && isset($this->list_cache[$row_id])) {
			$rows = $this->list_cache[$row_id];
		} else {
			$id_name = $this->getIdName();
			$ids = array($result->getField($id_name));
			$teams = $this->fetchTeams($fmt, $ids);
			if($teams)
				$rows = $teams->formatted_rows;
			else
				$rows = array();
		}
		$idx = 0;
		foreach($rows as $team) {
			if($idx ++) {
				if($fmt->getFormat() == 'html') $ret .= '<br>';
				$ret .= "\n";
			}
			$name = $team['name'];
			if($fmt->getFormat() == 'html') {
				$name = '<div class="input-icon theme-icon module-SecurityGroups"></div>&nbsp;' . $name;
				if($fmt->getContext() == 'view')
					$name = '<div style="padding-left: 20px; text-indent: -20px">' . $name . '</div>';
			}
			$ret .= $name;
		}
		return $ret;
	}
}

?>
