<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormField.php');

class QuickEditListElt extends FormField {

	var $link_name;
	var $target_key;
	var $join_key;

	function init($params=null, $model=null) {
        parent::init($params, $model);
	}
	
	function loadFieldDefinition(array $def) {
		parent::loadFieldDefinition($def);
		$join = $this->model->getJoinModel();
		if($join) {
			$lspec = $join->getLinkSpec();
			$this->link_name = $lspec['name'];
			$this->target_key = $lspec['relationship_spec']['right']['key'];
			if($lspec && ! empty($lspec['relationship_spec']['right']['join_key']))
				$this->join_key = $lspec['relationship_spec']['right']['join_key'];
		} else {
			$lspec = $this->model->getLinkSpec();
			if($lspec) {
				$this->link_name = $lspec['name'];
				$this->target_key = $lspec['relationship_spec']['left']['key'];
			}
		}
	}

	function getRequiredFields() {
        $fields = parent::getRequiredFields();
		$fields[] = $this->target_key;
        return $fields;
	}

    function renderListValue(ListFormatter $fmt, ListResult $result, $row_id, $list_params=null) {
		$field = $this->name;
		if($this->list_source) $field = $this->list_source . '.' . $field;
        $field_spec = array_get_default($result->fields, $field, array());
        
        $params = array(
            'id' => $this->name . '-' . $row_id,
            'module' => $list_params['module'],
            'record' => array_get_default($list_params, 'parent_key'),
            'link_name' => array_get_default($list_params, 'link_name'),
            'target_id' => $row_id,
            'field' => $field,
            'list_id' => $this->list_id,
            'type' => array_get_default($field_spec, 'type'),
            'callback' => 1,
        );
        if($fmt->report_id && $fmt->report_id == $params['record']) {
        	$params['module'] = 'Reports';
        }

        $label = array_get_default($result->formatted_rows[$row_id], $field);
        if (! strlen($label)) $label = '--';

        if (isset($field_spec['type']) && ($field_spec['type'] == 'enum' || $field_spec['type'] == 'status')) {
            $form = new EditableForm('SubPanel', 'QuickEdit');
            $add_blank = array_get_default($field_spec, 'options_add_blank', true);
            $spec = array('options' => $field_spec['options'], 'options_add_blank' => $add_blank);
            $opts = $form->getSelectOptions($dom, $spec, '', '');
            $params['options'] = $opts;
            $params['default_val'] = array_get_default($field_spec, 'default');
            $params['selected'] = array_get_default($result->rows[$row_id], $field);
            $arrow = true;
        }
        else if(isset($field_spec['type']) && $field_spec['type'] == 'bool') {
        	$params['value'] = array_get_default($result->rows[$row_id], $field);
        	$chk = $params['value'] ? 'checked' : '';
        	$label = '<span class="uii uii-checkbox '.$chk.'"></span>';
        }
        else {
        	$fval = array_get_default($result->rows[$row_id], $field);
			$params['value'] = $fval;
			if(! empty($field_spec['is_override'])) {
				$params['show_reset'] = 1;
			}
		}
        
        $json = getJSONobj();
        $json_params = $json->encode($params);
        $attrs = [
        	'class' => 'input-field flat' . (! empty($arrow) ? ' rbullet' : ''),
        	'id' => $params['id'],
        	'onclick' => 'SUGAR.ui.quickEdit('.$json_params.', event);',
        ];
        $html = EditableForm::get_tag('div', $attrs);
        $attrs = ['class' => 'input-label', 'id' => $params['id'].'-label'];
        $html .= EditableForm::get_tag('div', $attrs);
		$html .= $label . '</div></div>';
        return $html;
    }
    
}
