<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once 'include/layout/forms/FormButton.php';
require_once 'modules/Recurrence/RecurrenceRule.php';

class RecurrenceButton extends FormButton {

	var $module, $record, $copy_rules;
	var $recurrence_rule_id;
	
	function getRequiredFields() {
		return array('recurrence_rule_id', 'can_edit_recurrence');
	}
	
	function getAdditionalHtml(FormGenerator $gen)
	{
		global $app_strings, $pageInstance;
        $layout = $gen->getLayout();
		$layout->addScriptInclude('modules/Recurrence/recur.js', LOAD_PRIORITY_FOOT);
		$layout->addCSSLiteral('@import url(modules/Recurrence/style.css);');
		$layout->addScriptInclude('modules/Recurrence/recur.js');
		$layout->addJSLanguage('Recurrence');

		$recur = new RecurrenceRule;
		$rule = null;
		if ($this->copy_rules) {
			$rule = RecurrenceRule::retrieve_rule($this->recurrence_rule_id);
		}
		$recur_rules = RecurrenceRule::rule_to_JSON($rule);
		return '<input type="hidden" name="recurrence_rules" id="recurrence_rules" value="' . htmlspecialchars($recur_rules) . '">';
	}
	
	function init($params=null, $model=null) {
		global $app_strings, $pageInstance;
		parent::init($params, $model);
		$this->type = 'button';
		$this->icon = 'icon-recur';
		$this->label = $app_strings['LBL_RECUR_BUTTON_LABEL'];
		$this->perform = "recur_schedule.display($('recurrence_rules'));";
		$this->module = array_get_default($params, 'module', 'Meetings');
	}
	
    function loadResult(RowResult $result, array $context) {
		parent::loadResult($result, $context);
		$this->record = $result->getField('id');
		$this->recurrence_rule_id = $result->getField('recurrence_rule_id');
		$this->copy_rules = empty($result->for_duplicate);
	}
	
}

