<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormField.php');

class RelativeTimeInput extends FormField {	
	var $params;
	var $have_time = true;
	
	function init($params=null, $model=null) {
		parent::init($params, $model);
		if(is_array($params))
			$this->params = $params;
	}
	
	function loadFieldDefinition(array $def) {
		parent::loadFieldDefinition($def);
		if(array_get_default($def, 'type') == 'date')
			$this->have_time = false;
	}
	
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if(empty($this->name))
			return '';
		if($this->getEditable($context))
			$value = $this->renderHtmlEdit($gen, $row_result, $context);
		else
			$value = $this->renderHtmlView($gen, $row_result, $context);
		return $value;
	}
	
	function getRequiredFields() {
		return array($this->name);
	}

	function renderHtmlView(HtmlFormGenerator $gen, RowResult $row_result, array $context) {
		$type = $row_result->getField($this->name . '_type');
		$count = $row_result->getField($this->name . '_count');
		if($type == 'same_day')
			$date = translate('LBL_RELTIME_SAME_DAY', 'Workflow');
		else if($type == 'next_day')
			$date = translate('LBL_RELTIME_NEXT_DAY', 'Workflow');
		else {
			$date = translate('LBL_RELTIME_AFTER', 'Workflow');
			$date = str_replace('{COUNT}', $count, $date);
		}
		if($this->have_time) {
			$time = $row_result->getField($this->name . '_time', null, true);
			$date = str_replace(array('{DATE}', '{TIME}'), array($date, $time), translate('LBL_RELTIME_FORMAT', 'Workflow'));
		}
		return $date;
	}
	
	function renderHtmlEdit(HtmlFormGenerator $gen, RowResult $row_result, array $context) {
		$form =& $gen->getFormObject();

		$name = $form->getFieldNamePrefix() . $this->name;
		$sel_id = $form->getFieldIdPrefix() . $this->name;
		$type_name = $name.'_type';
		$count_name = $name.'_count';
		$time_name = $name.'_time';
		
		$ret = '<div id="'.$sel_id.'">';
		$type_spec = array(
			'name' => $type_name,
			'type' => 'enum',
			'options' => array(
				'same_day' => translate('LBL_RELTIME_SAME_DAY', 'Workflow'),
				'next_day' => translate('LBL_RELTIME_NEXT_DAY', 'Workflow'),
				'days_of' => translate('LBL_RELTIME_DAYS_OF', 'Workflow'),
			),
		);
		if($this->required)
			$type_spec['options_add_blank'] = false;
		$render = $form->renderSelect($type_spec, $row_result->getField($type_name));
		$count_spec = array(
			'name' => $count_name,
			'type' => 'int',
			'placeholder' => translate('LBL_RELTIME_DAYS_PLACEHOLDER', 'Workflow'),
			'len' => 5,
		);
		$render .= ' ' . $form->renderText($count_spec, $row_result->getField($count_name));
		if($this->have_time) {
			$time_spec = array(
				'name' => $time_name,
				'type' => 'time',
				'placeholder' => translate('LBL_RELTIME_TIME_PLACEHOLDER', 'Workflow'),
			);
			$time_render = $form->renderDateTime($time_spec, $row_result->getField($time_name, null, true));
			$render = str_replace(array('{DATE}', '{TIME}'), array($render, $time_render), translate('LBL_RELTIME_FORMAT', 'Workflow'));
		}
		$ret .= $render;
		$ret .= '</div>';
		
        $attrs = array(
        	'name' => $name,
        	'label' => $this->getLabel(),
        	'have_time' => $this->have_time ? 1 : 0,
        	'required' => ! empty($this->required),
        );
        $form->addControl('RelativeTimeInput', $sel_id, $attrs);

		return $ret;
	}
	
}
