<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormField.php');

class RunIntervalInput extends FormField {	
	var $qty_spec;
	var $qty_name;
	var $unit_spec;
	var $unit_name;
	
	function init($params=null, $model=null) {
		if(isset($params['qty_name']))
			$this->qty_name = $params['qty_name'];
		else
			$this->qty_name = array_get_default($params, 'name');
		if(isset($params['unit_name']))
			$this->unit_name = $params['unit_name'];
		else
			$this->unit_name = $this->qty_name . '_unit';
		parent::init($params, $model);
	}
	
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if(empty($this->qty_name))
			return '';
		if($this->getEditable($context))
			$value = $this->renderHtmlEdit($gen, $row_result, $context);
		else
			$value = $this->renderHtmlView($row_result, $context);
		return $value;
	}
	
	function getRequiredFields() {
		return array($this->qty_name, $this->unit_name);
	}
	
	function loadModelFields() {
		$this->qty_spec = $this->model->getDisplayField($this->qty_name, true);
		$this->unit_spec = $this->model->getDisplayField($this->unit_name, true);
		$q =& $this->qty_spec;
		if($q && ! $this->vname) {
			$this->vname = $q['vname'];
			$this->vname_module = $q['vname_module'];
		}
	}
	
	function renderHtmlEdit(HtmlFormGenerator $gen, RowResult $row_result, array $context) {
		$form =& $gen->getFormObject();

		$name = $form->getFieldNamePrefix() . $this->qty_name;
		$sel_id = $form->getFieldIdPrefix() . $name.'-input';
		
		$value = array_get_default($row_result->row, $this->qty_name);
		$ret = $form->renderText($this->qty_spec, $value);
		$ret .= '&nbsp;';
		$unit = array_get_default($row_result->row, $this->unit_name);
		if(! $unit && isset($this->unit_spec['default']))
			$unit = $this->unit_spec['default'];
		$ret .= $form->renderSelect($this->unit_spec, $unit);
		return $ret;
	}
	
	function renderHtmlView(RowResult $row_result, array $context) {
		$value = $row_result->getField($this->qty_name, 0, true);
		$unit = $row_result->getField($this->unit_name, null, true);
		if(! $unit && isset($this->unit_spec['default']))
			$unit = $this->unit_spec['default'];
		return $value . ' ' . $unit;
	}
	
    function renderListValue(ListFormatter $fmt, ListResult $result, $row_id, $list_params=null) {
		$row = $result->getRowResult($row_id);
		if($row)
			$html = $this->renderHtmlView($row, array());
		else
			$html = '';
		if($html && $result->base_model == 'EmailMarketing') {
			$onclick = "SUGAR.popups.openUrl('async.php?module=EmailMarketing&action=DelayPopup&email_id=".$row_id."&list_id=".$this->list_id."', null, {width: '300px', title_text: '".$this->label."', resizable: false}); return false;";
			$html = '<div class="input-field flat rbullet" onclick="'.$onclick.'"><span class="input-label">'.$html.'</span></div>';
		}

        return $html;
    }
	
}
