<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormField.php');

class StarRatingWidget extends FormField {
	var $base_url;
	var $instant;
	
	function init($params=null, $model=null) {
		parent::init($params, $model);
		if(is_array($params)) {
			if(isset($params['base_url']))
				$this->base_url = $params['base_url'];
			if(isset($params['instant']))
				$this->instant = $params['instant'];
		}
	}
	
	function renderForEmail($val) {
		$basic_stars = 1;
		
		$star_on = 'themes/Default/images32/Star.png';
		$star_off = 'themes/Default/images32/StarOff.png';
		$star_grey = 'themes/Default/images32/StarGrey.png';
		$star_blue = 'themes/Default/images32/StarBlue.png';

		$ret = $tail = array();
		if($this->base_url && ! $this->instant)
			$ret[] = '<input type="hidden" name="rating" value="'.$val.'">';
		for($i = 0; $i < 5; $i++) {
			$w = (5-$i)*20;
			if($this->base_url && $this->instant) {
				$href = $this->base_url.'&rating='.($i+1);
				$onclick = '';
			} else {
				$href = '#';
				$onclick = 'show_rating('.($i+1).'); return false;';
			}
			$active = !! $this->base_url;
			
			if($active && $basic_stars && $this->instant) {
				$ret[] = '<a href="'.to_html($href).'" onclick="'.to_html($onclick).'" style="display: inline-block; width: 20px; height: 20px; overflow: hidden">';
				$ret[] = '<img src="'.to_html($star_off).'" class="star_off" width="20" height="20" style="width: 20px; height: 20px; float: left" border="0" alt="'.($i+1).'">';
				$ret[] = '</a>';
				continue;
			}
			
			$ret[] = '<div class="rating_box" style="display: inline-block; width: '.$w.'px; height: 20px; overflow: hidden">';
			$star_style = 'display: inline-block; width: 20px; height: 20px; line-height: 20px; float: left; overflow: hidden';
			if($active)
				$ret[] = '<a style="'.$star_style.'" href="'.to_html($href).'" onclick="'.to_html($onclick).'">';
			else
				$ret[] = '<div style="'.$star_style.'">';
			if(! $this->instant)
				$ret[] = '<img src="'.to_html($star_blue).'" class="star_blue" width="20" height="20" style="width: 20px; height: 20px; float: left; display: none" border="0" alt="">';
			$show_grey = ($i < $val) ? '' : 'display: none';
			$ret[] = '<img src="'.to_html($star_grey).'" class="star_grey" width="20" height="20" style="width: 20px; height: 20px; float: left; '.$show_grey.'" border="0" alt="">';
			$ret[] = '<img src="'.to_html($star_off).'" class="star_off" width="20" height="20" style="width: 20px; height: 20px; float: left" border="0" alt="'.($i+1).'">';
			$ret[] = '<img src="'.to_html($star_on).'" class="star_on" width="20" height="20" style="width: 20px; height: 20px; float: left" border="0" alt="">';
			if($active)
				$ret[] = '</a>';
			else
				$ret[] = '</div>';
			$tail[] = '</div>';
		}
		array_extend($ret, $tail);
		return '<div id="rating_input" class="rating_outer" style="display: inline-block; width: 100px; height: 20px">' . implode('', $ret) . '</div>';
	}

	function formatField(FieldFormatter $fmt, RowResult $result, $row_id=null) {
		$value = round($result->getField($this->getResultFieldName()));
		if($fmt->getFormat() == 'html') {
			if($fmt->getContext() == 'email')
				return $this->renderForEmail($value);
			else if($value) {
				$ret = '<nobr>';
				for ($i = 0; $i < 5; $i++) {
					$cls = $value <= $i ? '' : 'active';
					$ret .= '<div class="input-icon icon-rating ' . $cls . '"></div>';
				}
				$value = $ret . '</nobr>';
			}
		}
		if(! $value) {
			if($fmt->getContext() == 'email' || $fmt->getContext() == 'notify' || $fmt->getContext() == 'view')
				$value = '';
			else
				$value = 0;
		}
		return $value;
	}

}

?>
