<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormButton.php');

class SubpanelCreateButton extends FormButton {

	var $module;
    var $action;
    var $acl = 'create';
    var $link_name;
    var $panel_name;
    var $parent_module;
    var $parent_id;
    var $model_name;
	
	function init($params=null, $model=null) {
		parent::init($params, $model);

        $this->module = array_get_default($params, 'module', 'Meetings');
		$this->action = array_get_default($params, 'action', 'EditView');
        $this->acl_module = $this->module;
		$this->parent_module = array_get_default($this->params, 'parent_module', '');
		$this->parent_id = array_get_default($this->params, 'parent_id', '');
		$this->panel_name = array_get_default($this->params, 'panel_name');
		$this->link_name = array_get_default($this->params, 'link_name', $this->panel_name);
		$this->type = 'button';
		$this->model_name = AppConfig::module_primary_bean($this->module);
		
        if (! empty($params['icon'])) {
            $this->icon = $params['icon'];
        } else {
            $this->icon = 'theme-icon create-'.$this->model_name;
        }
        
		$qc_layout = array_get_default($params, 'quick_create_layout', 'QuickCreate');
		$qc_layout = AppConfig::setting("views.detail.{$this->module}.edit.$qc_layout");

        if ($qc_layout) {
			$quick_create_params = $this->getQuickCreateParams();
            $json = getJSONobj();
            $qc_params = $json->encode($quick_create_params);
            $this->perform = $this->possibleMailto("SUGAR.ui.openQuickCreate('{$this->module}', $qc_params)", $quick_create_params);
        } else {
        	$qparams = $this->getQueryParams();
            $qparams['module'] = $this->module;
            $qparams['action'] = $this->action;

            $url = javascript_escape('index.php?' . encode_query_string($qparams));
            $this->perform = $this->possibleMailto("SUGAR.util.loadUrl('$url');", $qparams);
        }
	}
	
	function getQueryParams() {
		$type_field = array_get_default($this->params, 'type_field');
		$id_field = array_get_default($this->params, 'id_field');
		$pmod = array_get_default($this->params, 'parent_module', '');
		$pid = array_get_default($this->params, 'parent_id', '');
		
		if($this->parent_module == 'Accounts' && ($this->module == 'Emails' || $this->module == 'Notes')) {
			$type_field = null;
			$id_field = 'account_id';
		}
		
		if(! $id_field) {
			$found = false;
			$pm = AppConfig::module_primary_bean($this->parent_module);
			if($pm && $this->link_name) {
				$pm = new ModelDef($pm);
				$link = $pm->getLinkRelationship($this->link_name);
				if($link && empty($link['sources'])) {
					$id_field = $link['right']['key'];
					if(! empty($link['right']['role_field']) && ! empty($link['right']['role_value'])) {
						$type_field = $link['right']['role_field'];
						$pmod = $link['right']['role_value'];
					}
					$found = true;
				}
			}
		
			if(! $found) {
				$id_field = 'related_id';
				if(! $type_field)
					$type_field = 'related_type';
			}
		}

		$qparams = array(
			$id_field => $pid,
			'return_module' => $pmod,
			'return_action' => 'DetailView',
			'return_record' => $pid,
		);
        if (isset($this->params['layout']))
            $qparams['layout'] = $this->params['layout'];
		if($type_field)
			$qparams[$type_field] = $pmod;
		if(! empty($this->query_params))
			array_extend($qparams, $this->query_params);
		return $qparams;
	}
	
	function getQuickCreateParams() {
		$panel_name = array_get_default($this->params, 'panel_name', '');
		$id_field = array_get_default($this->params, 'id_field', 'related_id');

		$qparams = $this->getQueryParams();
		$quick_create_params = array(
			'target_id' => $this->list_id,
			'target_name' => 'subpanel',
			'primary_name' => $id_field,
			'return_panel' => $panel_name,
		);
		array_extend($qparams, $quick_create_params);
		return $qparams;
	}

	function possibleMailto($perform, $params) {
		global $current_user;
		if ($this->module == 'Emails') {
			if (array_get_default($params, 'type') != 'archived') {
				$client = $current_user->getPreference('email_link_type');
				if (!$client)
					$client = AppConfig::setting('email.default_client');
				if (AppConfig::is_mobile())
					if(!AppConfig::is_mobile_module('Emails'))
						$client = '';
				if ($client != 'sugar') {
					$json = getJSONobj();
					$p = $params;
					require_bean('Email');
					$addr = Email::load_to_addr($params);
					if (!empty($addr['to']))
						$perform = "window.location.href=" . $json->encode('mailto:' . $addr['to']) . ";";
				}
			}
		}
		return $perform;
	}
}

