<?php

require_once  'include/DLManager.php';

use GeoIp2\Database\Reader;

class MaxMindGeoIP {

	public static $instance;

	protected $dbFile;

	protected function __construct() {
		$this->dbFile = 'geoip/GeoLite2-City.mmdb';
	}

	public static function instance() {
		if (!self::$instance)
			self::$instance = new self;
		return self::$instance;
	}

	public static function available() {
		return self::instance()->isAvailable();
	}

	protected function isAvailable() {
		$filename = AppConfig::cache_dir() . $this->dbFile;
		return is_readable($filename);
	}

	public static function lookup($ip) {
		return self::instance()->lookupIP($ip);
	}

	protected function lookupIP($ip) {
		if (!$this->isAvailable())
			return false;
		$filename = AppConfig::cache_dir() . $this->dbFile;
		try {
			$reader = new Reader($filename);
			$record = $reader->city($ip);
			return $record;
		} catch (Exception $e) {
			$GLOBALS['log']->fatal($e->getMessage());
			return false;
		}
	}

	public static function download() {
		$key = AppConfig::setting('company.maxmind_key');
		if (empty($key)) return;
		$temp_directory = AppConfig::cache_dir() . 'incoming/';
		$temp_filename = uniqid($temp_directory) . '.tar.gz';
		$response_file_handle = fopen($temp_filename, 'wb');

		$finishDownload = function(&$dl) use ($temp_filename) {
			if(!$dl->failed) {
				$pd = new PharData($temp_filename);
				$iterator = new RecursiveIteratorIterator($pd);
				$filename = AppConfig::cache_dir() . self::instance()->dbFile;

				foreach ($iterator as $k => $v) {
					if ((substr($k, -5)) === '.mmdb') {
						create_cache_directory('geoip/');
						// cannot use rename() becase source and destination
						// use diffrent wrappers
						copy($k, $filename);
						@unlink($temp_filename);
						break;
					}
				}
			} else {
				$GLOBALS['log']->error('cannot download MaxMind database');
			}
		};

		$dl_params = [
			'url' => 'https://download.maxmind.com/app/geoip_download?edition_id=GeoLite2-City&license_key=' 
				. urlencode($key) 
				. '&suffix=tar.gz',
			'timeout' => 30,
			'max_redirects' => 2,
			'callback' => $finishDownload,
			'response_file_handle' => $response_file_handle,
		];

		DLManager::add_download($dl_params);
		DLManager::process_downloads();
	}
}

