<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/pdf/TCPDFHelper.php');

class PDFManager extends TCPDFHelper {

	var $addressId = null;

	var $filename;
	var $header_options;
	var $footer_options;
	var $in_table = false;
	var $safe_filename;
	var $preferred_font_mono;
	var $focus_type;
	var $pdf_type;
	var $alt_lang_module;

	
	var $all_fields = array('page', 'status');
	var $drawing_fields = array(
		'LineWidth', 'ColorFlag',
		'FontFamily', 'FontStyle', 'CurrentFont', 'FontSize', 'FontSizePt',
		'TextColor', 'FillColor', 'DrawColor');
	
	var $default_orientation = 'P';
	var $default_paper_size = 'Letter';
	var $default_font_face = 'freesans';
	var $default_font_mono = 'courier';
	var $default_font_size = '9';
	var $default_line_width = '1pt';
	var $table_status;
	var $use_language = 'current';

// Public interface

	function __construct($filename='', $orientation=null, $unit='pt', $format='') {
		$this->filename = $filename;
		$paper_size = $format ? $format : strtoupper(AppConfig::setting('company.pdf_paper_size', $this->default_paper_size));
		if(! $orientation)
			$orientation = $this->default_orientation;
		parent::__construct($orientation, $unit, $paper_size, true, 'UTF-8');
		$this->Open();
		$this->SetCreator('1CRM System');

		$font_list = AppConfig::setting('pdf_fonts.index');
		if(($sys_font = AppConfig::setting('company.pdf_font_face')) && isset($font_list[$sys_font]))
			$this->default_font_face = $sys_font;
		if(($user_font = array_get_default($_REQUEST, 'pdf_font_face')) && isset($font_list[$user_font]))
			$this->default_font_face = $user_font;
		$langs = AppConfig::get_languages();
		if(($user_lang = array_get_default($_REQUEST, 'pdf_language')) && isset($langs[$user_lang]))
			$this->use_language = $user_lang;

		$this->SetHeaderMargin(20);
		$margins = AppConfig::setting('pdf.margins');
		$this->SetMargins($margins['left'], $margins['top']);
		$this->setCellPaddings($this->lv('1mm'), $this->lv('1mm'), $this->lv('1mm'), $this->lv('1mm'));
		$this->SetFooterMargin(20);
		$this->SetAutoPageBreak(true, 40);
		$this->setImageScale(4);
		$this->init();
	}
	
	/* set deployment-specific options in this extension file */
	function init() {
		$ext_f = AppConfig::custom_dir() . 'include/pdf/ext.php';
		if(file_exists($ext_f)) {
			global $custom_pdf_init;
			require_once($ext_f);
			if(isset($custom_pdf_init) && is_array($custom_pdf_init))
				foreach($custom_pdf_init as $f)
					$f($this);
		}		
	}
	
	function new_page($orientation=null) {
		if(! isset($orientation)) $orientation = $this->CurOrientation;
		$this->AddPage($orientation);
	}
	
	function save_output($redirect=false) {
		session_write_close(); // prevent session locking while file is generated

		$this->safe_filename = filename_safe_string($this->filename);
		$this->Output($this->safe_filename, 'F');
		if($redirect)
			header("Location: ".htmlentities($this->safe_filename));
		return $this->safe_filename;
	}
	
	function serve_dynamic($force_download=false) {
		session_write_close(); // prevent session locking while file is generated

		$this->Output($this->filename, $force_download ? 'D' : 'I');
		sugar_cleanup(true);
	}
	
	function get_output() {
		session_write_close(); // prevent session locking while file is generated

		return $this->Output($this->filename, 'S');
	}
	
/*	not used
	function &get_document_state() {
		static $clsvars;
		if(! isset($clsvars))
			$clsvars = array_keys(get_class_vars(get_class(parent)));
		$status = array();
		foreach($clsvars as $f)
			$status[$f] = $this->$f;
		return $status;
	}
	
	function &get_drawing_state() {
		$status = array();
		foreach($this->color_fields as $f)
			$status[$f] = $this->$f;
		return $status;
	}
	
	function revert_state(&$old_state) {
		foreach($old_state as $k => $v)
			$this->$k = $v;
	}
*/

	function get_company_header_text($reload=false, $one_line = true) {
		static $header_text;
		global $locale;
		if(! isset($header_text) || $reload) {
			$settings = $this->getCompanyAddress();
			$settings['account_name'] = preg_replace('/\r\n|\n/', ' ', $settings['name']);
			$tpl = $locale->getLocaleAddressTemplate('pdf', $settings['address_format']);
			$addr = $locale->getLocaleFormattedAddress($settings, '', 'pdf', $tpl);
			if ($one_line) {
				$header_text = preg_replace('/\r\n|\n/', ' • ', $addr);
			} else {
				$header_text = preg_replace('/(\r?\n)+/', "\n", $addr);
			}
			$phones = array();
			if (!empty($settings['phone'])) {
				$phones[] = AppConfig::setting("lang.strings.{$this->use_language}.app.LBL_PDF_PHONE") . ': ' . $settings['phone'];
			}
			if (!empty($settings['fax'])) {
				$phones[] = AppConfig::setting("lang.strings.{$this->use_language}.app.LBL_PDF_FAX") . ': ' . $settings['fax'];
			}
			if (!empty($phones)) {
				$header_text .= "\n" . implode(" • ", $phones);
			}
		}
		return $header_text;
	}
	
	function get_company_logo_info() {
		$address = $this->getCompanyAddress();
        $base_dir = AppConfig::files_dir() . 'images/logos/';
        $logo = array_get_default($address, 'logo', '');
        $file = '';

        if (! empty($logo)) {
            if (file_exists($logo)) {
                $file = $logo;
            } else if (file_exists($base_dir . $logo)) {
                $file = $base_dir . $logo;
            }
        }

        if (empty($file))
            $file = get_company_logo_file();

		$info = array('path' => $file);
		if(file_exists($file)) {
			if(($size = @getimagesize($file)) !== null) {
				list($info['width'], $info['height'], $info['type']) = $size;
			}
		}
		return $info;
	}
	
	function get_company_paper_size() {
		return AppConfig::setting('company.pdf_paper_size');
	}
	
	function DrawTable(&$data, $cols, $title='', $header=true, $body=array()) {

		$left = $this->x;
		$status = array();
		$this->table_status =& $status;
		$defs = $this->get_default_table_options();
		
		if(! is_array($title)) {
			$text = $title;
			$title = $defs['title'];
			$title['text'] = $text;
		}
		else
			$title = array_merge($defs['title'], $title);
		
		if(! is_array($header)) {
			$show = $header;
			$header = $defs['header'];
			if(! $show)
				$header['enabled'] = false;
		}
		else
			$header = array_merge($defs['header'], $header);
		
		$body = array_merge($defs['body'], $body);
				
		if($header['enabled']) {
			$color = $header['color'];
			$this->SetTextColor($color[0], $color[1], $color[2], true);
			$color = $header['border-color'];
			$this->SetDrawColor($color[0], $color[1], $color[2], true);
			$bgcolor = $header['background-color'];
			$this->SetFillColor($bgcolor[0], $bgcolor[1], $bgcolor[2], true);
		}
		$hfont = $this->get_font_from_css($header);
		$this->SetFont($hfont[0], $hfont[1], $hfont[2]);
		$availW = $this->getPageWidth() - $this->rMargin - $this->getX();
		$maxW = $this->get_width($body, '', '100%', $availW);
		$minColW = $this->FontSize * 1.5;
		$cell_paddings = $this->getCellPaddings();
		$margin = $cell_paddings['L'];
		
		$hrow = array();
		$colwidths = array();
		$canshrink = array();
		$minwidths = array();
		$maxwidths = array();
		
		foreach($cols as $idx => $col) {
			$coltitle = (string)array_get_default($col, 'title', '');
			$cols[$idx]['text'] = $coltitle;
			$minwidths[$idx] = max($minColW, $this->MinTextWidth($coltitle, true, $margin));
			$maxwidths[$idx] = 0;
			$wrapped = $this->WrapText($coltitle, -1, 0);
			foreach($wrapped as $wrapline)
				$maxwidths[$idx] = max($maxwidths[$idx], $wrapline[0] + 2*$margin);
		}
		$titlewidths = $minwidths;
		
		$bfont = $this->get_font_from_css($body);
		$this->SetFont($bfont[0], $bfont[1], $bfont[2]);
		$colids = array_keys($cols);
		foreach(array_keys($data) as $rowidx) {
			$rowdata =& $data[$rowidx];
			$row = array();
			foreach($colids as $idx) {
				$text = (string)array_get_default($rowdata, $idx, '');
				$col = array();
				$minwidths[$idx] = max($minwidths[$idx], $this->MinTextWidth($text, false, $margin, 25));
				$wrapped = $this->WrapText($text, -1, 0);
				foreach($wrapped as $wrapline)
					$maxwidths[$idx] = max($maxwidths[$idx], $wrapline[0] + 2*$margin);
			}
		}
		
		// adjust min widths to below maxW if possible
		if(array_sum($minwidths) > $maxW) {
			$availW = array();
			$minSum = array_sum($minwidths);
			foreach($minwidths as $idx => $mw) {
				$availW[$idx] = $minwidths[$idx] - $titlewidths[$idx];
			}
			$availSum = array_sum($availW);
			if($availSum > $minSum - $maxW) {
				foreach($availW as $idx => $as)
					$availW[$idx] = $as * ($minSum - $maxW) / $availSum;
				$availSum = array_sum($availW);
			}
			if($availSum) {
				foreach($minwidths as $idx => $mw) {
					$minwidths[$idx] -= $availW[$idx];
				}
			}
		}
		
		foreach($cols as $idx => $col) {
			$nowrap = (bool)array_get_default($col, 'nowrap', 0);
			$cols[$idx]['nowrap'] = $nowrap;
			$canshrink[$idx] = ! $nowrap;
			$w = $this->get_width($col, '', 0, $maxW);
			if(! $nowrap && ! $w) {
				$colwidths[$idx] = $maxwidths[$idx];
			}
			else {
				$minwidths[$idx] = min($maxW / 2, $minwidths[$idx]);
				if($w < $minwidths[$idx])
					$w = $minwidths[$idx];
				if($nowrap)
					$w = max($w, $maxwidths[$idx]);
				$colwidths[$idx] = $w;
			}
		}
		
		$sumW = array_sum($colwidths);
		$doresize = true;
		if($sumW < $maxW) {
			$expand = array();
			foreach($cols as $idx => $col) {
				if(empty($col['width'])) {
					$expand[$idx] = log(2 * $maxwidths[$idx] - $colwidths[$idx]);
				}
			}
			$diff = $maxW - $sumW;
			$totexp = array_sum($expand);
			foreach($expand as $idx => $val) {
				$colwidths[$idx] += $val * $diff / $totexp;
			}
			$sumW = array_sum($colwidths);
		}
		else
		while($sumW > $maxW && $doresize) {
			$avail = 0;
			$totprop = 0;
			$doresize = false;
			foreach($cols as $idx => $col) {
				if($canshrink[$idx]) {
					$avail += $colwidths[$idx] - $minwidths[$idx];
					$totprop += sqrt($maxwidths[$idx]);
					$doresize = true;
				}
			}
			if(! $doresize)
				break;
			$diff = min($sumW - $maxW, $avail);
			$newwidths = $colwidths;
			$doresize = false;
			
			if($diff > 10) {
				foreach($cols as $idx => $col) {
					if($canshrink[$idx]) {
						$newwidths[$idx] -= sqrt($maxwidths[$idx]) * $diff / $totprop;
						if($newwidths[$idx] < $minwidths[$idx]) {
							$newwidths[$idx] = $minwidths[$idx];
							$canshrink[$idx] = false;
						}
						$doresize = true;
					}
				}
			}
			$colwidths = $newwidths;
			$sumW = array_sum($colwidths);
		}
		foreach($cols as $idx => $col) {
			$col['width'] = $cols[$idx]['width'] = $colwidths[$idx];
			$col['text-align'] = array_get_default($col, 'header-text-align', 'L');
			$hrow[] = $col;
		}
		
		// FIXME - calc height of each row to ensure we stay on one page
		// need to break up CellRow into two stages
		$fail = 0;
		$tryrow = 0;
		$lastrow = -1;
		$allowspan = array_get_default($body, 'allow-page-span', true);
		$isaltrow = array_get_default($body, 'start-alt-row', false);
		$newpage = false;
		while(1) {
			$reset_point = strlen($this->pages[$this->page]);
			$wrap = false;
			$wrap2 = false;
			$undo = false;
			$this->setX($this->lv(array_get_default($title, 'left', $left)));
			if($title['text'] && (empty($title['new-page-only']) || $newpage)) {
				$align = $this->get_text_align($title, '', 'C');
				$trow = array(
					array('text' => $title['text'], 'width' => $maxW, 'text-align' => $align),
				);
				$wrap = ! $this->CellRow($trow, $title);
				$repeat = array_get_default($title, 'repeat', true);
				if(! $repeat)
					$title['text'] = '';
			}
			$this->setX($left);
			if(! $wrap && $header['enabled']) {
				$wrap = ! $this->CellRow($hrow, $header);
			}
			if(! $wrap) {
				$nr = is_countable($data) ? count($data) : 0;
				$colids = array_keys($cols);
				$ck = array_keys($data);
				for(; $tryrow < $nr; $tryrow++) {
					$rowdata =& $data[$ck[$tryrow]];
					$nl = 0;
					$row = array();
					foreach($colids as $id) {
						$text = (string)array_get_default($rowdata, $id, '');
						$align = $this->get_text_align($cols[$id]);
						$col = array();
						$col['text'] = $text;
						$col['width'] = $cols[$id]['width'];
						$col['text-align'] = $align;
						if($isaltrow && isset($cols[$id]['alt-background-color']))
							$col['background-color'] = $cols[$id]['alt-background-color'];
						elseif (isset($cols[$id]['background-color']))
							$col['background-color'] = $cols[$id]['background-color'];
						$row[] = $col;
					}
					$parms = $body;
					if($isaltrow && isset($body['alt-background-color']))
						$parms['background-color'] = $body['alt-background-color'];
					//additional wrap check to avoid double wraps
					$wrap2 = ! $this->CellRow($row, $parms);
					if($wrap2)
						break;
					$isaltrow = ! $isaltrow;
				}
				if($tryrow == $lastrow || $tryrow >= $nr || $fail)
					break;
			}
			if(! $allowspan) {
				$tryrow = 0;
			}
			if($tryrow == 0) {
				// undo table header - bit of a hack
				$this->pages[$this->page] = substr($this->pages[$this->page], 0, $reset_point);
			}
			if ($wrap2 || $wrap) {
				$this->WrapPage();
				$newpage = true;
				$this->setY($this->tMargin);
			}
			$lastrow = $tryrow;
		}
	}
	
	function getCompanyAddress($id = null)
	{
		require_bean('CompanyAddress');
		return CompanyAddress::get_address_array($id ? $id : $this->addressId, false);
	}

	function useCompanyAddress($id)
	{
		$this->addressId = $id;
	}
	
	function getCompanyAddressId() {
		return $this->addressId;
	}

// Private methods

	function Header() {

		$this->header_options = array(
			'content' => $this->get_company_header_text(),
			'text-align' => 'center',
			'font-size' => 8,
			'border' => 'TB',
			'border-width' => '0.3pt',
			'margin' => 18,
			'padding' => '1mm',
		);

		if(! empty($this->header_options['content'])) {
			$this->print_header = true;
			$opts = $this->header_options;
			$m = $this->getMargins();
			list($orig_lm, $orig_rm, $orig_tm) = array($m['left'], $m['right'], $m['top']);
			if(isset($opts['margin']))
				$margin = $this->lv($opts['margin']);
			else
				$margin = $this->header_margin;
			$this->setMargins($margin, $margin);
			$this->setXY($margin, $margin);
			$data = array(array('text' => $opts['content']));
			$cols = array('text' => array('width' => '100%', 'text-align' => 'center'));
			$this->DrawTable($data, $cols, '', false, $opts);
			$this->setMargins($orig_lm, $orig_tm, $orig_rm);
			$this->setXY($this->lMargin, $this->tMargin);// + $this->header_margin /*+ 10*/);
		}
	}
	
	function Footer() {
		if(! $this->print_footer) {
			return;
		}

		$this->footer_options = array(
			'link-hide' => AppConfig::setting('layout.pdf.footer_link_hidden'),
			'link-url' => AppConfig::setting('layout.pdf.footer_link_url'),
			'link-title' => AppConfig::setting('layout.pdf.footer_link_title'),
			'font-size' => 8,
			'padding' => 4,
		);
		$pg_text = AppConfig::setting("lang.strings.{$this->use_language}.app.LBL_PDF_PAGE_NUMBER_STRING");
		if($pg_text)
			$this->footer_options['page'] = $pg_text;

		$this->InFooter = true;
		
		$this->AliasNbPages(); // set page number template field
		
		$left = $this->lMargin;
		$right = $this->rMargin;
		$top = $this->tMargin;
		
		$margin = $this->header_margin;
		$this->setMargins($margin, $margin);

		$font = $this->get_font_from_css($this->footer_options);
		$this->SetFont($font[0], $font[1], $font[2]);
		$line_width = 0.3;
		$this->SetLineWidth($line_width);
		$this->SetDrawColor(0, 0, 0);
		$this->SetTextColor(0, 0, 0);
		
		$pad = $this->get_padding($this->footer_options, '', 0);
		$footer_height = 2*$pad + $this->FontSize;
		$footer_y = $this->h - $this->footer_margin - $footer_height;
		$this->SetXY($margin, $footer_y); 
		
		$center_w = $this->w - $margin - $margin - $this->lv('80pt');
		$this->Cell(40, $footer_height, '', 'T', 0, '', 0, '');
		$link = $this->footer_options['link-url'];
		$text = $this->footer_options['link-title'];
		if(! empty($this->footer_options['link-hide']))
			$text = $link = '';
		$this->Cell($center_w, $footer_height, $text, 'T', 0, 'C', 0, $link);
		$text = array_get_default($this->footer_options, 'page', '{pg} / {nb}'); // '{Page} {pg} / {nb}'
		$w_page = is_array($this->l) && isset($this->l['w_page']) ? $this->l['w_page'] : '';
		$text = str_replace('{Page}', $w_page, $text);
		$text = str_replace('{nb}', $this->getAliasNbPages(), $text);
		$text = str_replace('{pg}', $this->PageNo(), $text);
		$this->Cell(40, $footer_height, $text, 'T', 0, 'R', 0, '');

		$this->InFooter = false;
		
		$this->setMargins($left, $top, $right);
	}
	
	function get_default_table_options() {
		$def_opts = array(
			'title' => array(
				'color' => array(0, 0, 0),
				'font-weight' => 'bold',
				'font-size' => 14,
				'border' => 0,
				'line-height' => '150%',
			),
			'header' => array(
				'enabled' => true,
				//'background-color' => array(0, 0, 0),
				//'color' => array(255, 255, 255),
				//'border-color' => array(255, 255, 255),
				'border' => 1,
				'background-color' => array(200, 200, 200),
				'color' => array(0, 0, 0),
				'border-color' => array(0, 0, 0),
				'font-weight' => 'bold',
			),
			'body' => array(
				'border' => 1,
			),
		);
		return $def_opts;
	}
	
	function get_font_from_css(&$spec, $pfx='') {
		$face = array_get_default($spec, $pfx.'font-face', $this->default_font_face);
		$bold = array_get_default($spec, $pfx.'font-weight') == 'bold';
		$italic = array_get_default($spec, $pfx.'font-style') == 'italic';
		$size = array_get_default($spec, $pfx.'font-size', $this->default_font_size);
		$size = str_replace('pt', '', $size);
		$underline = array_get_default($spec, $pfx.'text-decoration') == 'underline';
		return array($face, ($bold ? 'B' : '').($italic ? 'I' : ''), $size, $underline);
	}
	
	function get_text_align(&$spec, $pfx='', $default='J') {
		$align = array_get_default($spec, $pfx.'text-align', $default);
		switch($align) {
			case 'L': case 'left':   return 'L';
			case 'C': case 'center': return 'C';
			case 'R': case 'right':  return 'R';
			default: return 'J';
		}
		return $align;
	}
	
	function get_width(&$spec, $pfx='', $default='', $max=null) {
		$w = array_get_default($spec, $pfx.'width', $default);
		return $this->lv($w, $max);
	}

	//TODO rewrite without pfx - or replace with default method getCellPaddings()? lv() might be needed to get pixel values, but could also be used after getting the padding values
	function get_padding(&$spec, $pfx='', $default='') {
		$w = array_get_default($spec, $pfx.'padding', $default);
		return $this->lv($w);
	}
	
	function lv($l, $max=null) {
		if($max === null)
			$max = $this->getPageWidth() - $this->rMargin - $this->lMargin;
		if(preg_match('/(-?[\d\.]+)(%|pt|px|mm|cm|in)/', $l, $m)) {
			switch($m[2]) {
				case '%':
					$l = $m[1] * $max / 100;
					break;
				case 'px': $l = $m[1] * 25.4 / 72; break;
				case 'mm': $l = $m[1] * 72 / 25.4; break;
				case 'cm': $l = $m[1] * 72 / 2.54; break;
				case 'in': $l = $m[1] * 72; break;
				case 'pt': $l = $m[1]; break;
			}
		}
		if($l < 0) $l += $max + $this->lMargin;
		return $l / $this->k;
	}
	
	function get_line_height(&$spec, $pfx='') {
		$ht = array_get_default($spec, $pfx.'line-height');
		if($ht && preg_match('/(\d+)%/', $ht, $m))
			$ht = $m[1] * $this->FontSize / 100;
		else if(is_numeric($ht))
			$ht = $ht * $this->FontSize;
		else if($ht)
			$ht = $this->lv($ht);
		else
			$ht = $this->FontSize;
		return $ht;
	}
	
	function get_default_font() {
		$s = array();
		return $this->get_font_from_css($s);
	}
	
	function CellRow($cells, $options=array()) {
		$fill_h = array_get_default($options, 'fill-height', -1);
		$color = array_get_default($options, 'color', array(0,0,0));
		$this->SetTextColor($color[0], $color[1], $color[2], true);
		$color = array_get_default($options, 'border-color', array(0,0,0));
		$this->SetDrawColor($color[0], $color[1], $color[2], true);
		$bwidth = $this->lv(array_get_default($options, 'border-width', $this->default_line_width));
		$this->SetLineWidth($bwidth);
		$this->SetDrawColor($color[0], $color[1], $color[2], true);
		$url = array_get_default($options, 'link-url', '');

		$bgcolor = array_get_default($options, 'background-color', 'transparent');
		if($bgcolor == 'transparent') {
			$fill = false;
			$bgcolor = array(255,255,255);
		}
		else
			$fill = true;
		$font = $this->get_font_from_css($options);
		$this->SetFont($font[0], $font[1], $font[2]);
		$lineh = $this->get_line_height($options);
		$cell_paddings = $this->getCellPaddings();
		$oldmargins = $cell_paddings;
		$cell_padding_left = $this->get_padding($options, '', $cell_paddings['L']);
		$this->setCellPaddings($cell_padding_left, $cell_padding_left, $cell_padding_left, $cell_padding_left);
		$border = array_get_default($options, 'border', 0);
		$defalign = $this->get_text_align($options);
		
		$nl = 0;
		$ck = array_keys($cells);
		foreach($ck as $idx) {
			$cell =& $cells[$idx];
			$text = $cell['text'];
			$w = array_get_default($cell, 'width', $this->getPageWidth() - $this->getX() - $this->rMargin);
			$cell['width'] = $this->lv($w);
			$cell['lines'] = $this->WrapText($text, $cell['width']);
			$nl = max($nl, count($cell['lines']));
		}
		
		$allow_pgbrk = array_get_default($options, 'allow-page-break', false);

		$endPage = $origPage = $this->getPage();
		$origX = $endX = $this->getX();
		$origY = $endY = $this->getY();
		if($fill_h > 0)
			$rowH = $fill_h;
		else
			$rowH = $lineh * $nl + 2*$cell_paddings['L'];
		$skip = false;
		if(($origY + min($rowH, $lineh + $cell_paddings['T'] + $cell_paddings['B'])) > $this->PageBreakTrigger && empty($this->InFooter)) {
			if(! $allow_pgbrk || ! $this->AcceptPageBreak())
				$skip = true;
			else {
				$this->WrapPage();
				$origY = $endY = $this->tMargin;
				$endPage = $origPage = $this->getPage();
				$this->setXY($origX, $origY);
			}
		}
		if(! $skip) {
			foreach($ck as $idx) {
				$cell =& $cells[$idx];
				$h = $nl * $lineh + 2 * $cell_paddings['L'];
				$lines =& $cell['lines'];
				for($cnl = count($lines); $cnl < $nl; $cnl ++)
					$lines[] = array(0, 0, '');
				$align = $this->get_text_align($cell, '', $defalign);

				$bgcolor = array_get_default($cell, 'background-color', array_get_default($options, 'background-color', 'transparent'));
				if($bgcolor == 'transparent') {
					$fill = false;
					$bgcolor = array(255,255,255);
				}
				else
					$fill = true;
				$this->SetFillColor($bgcolor[0], $bgcolor[1], $bgcolor[2], true);

				$this->MultiCell2($cell['width'], $lineh, $lines, $border, $align, $fill, $url, $fill_h);
				$endX += $cell['width'];
				$endY = $this->getY();
				$newPage = $this->getPage();
				if ($newPage >= $endPage) {
					$endPage = $newPage;
				}

				$break_margin = $this->h - $this->PageBreakTrigger;
				$this->setPage($origPage);
				//only reset Y on the same page to avoid double page breaks
				if ($newPage == $endPage)
					$this->setXY($endX, $origY);
				$this->SetAutoPageBreak($this->AutoPageBreak, $break_margin);
			}
            if (empty($newPage) || $newPage == $endPage) {
				$break_margin = $this->h - $this->PageBreakTrigger;
				$this->setPage($endPage);
				$this->setXY($origX, $endY);
				$this->SetAutoPageBreak($this->AutoPageBreak, $break_margin);
			} else {
				$this->new_page();
				$this->setX($origX);
			}

		}
		$this->setCellPaddings($oldmargins['L'], $oldmargins['T'], $oldmargins['R'], $oldmargins['B']);
		return !$skip;
	}
	
	function CalcCellHeight($cell, $options=array()) 
	{
		$font = $this->get_font_from_css($options);
		$prev_FontFamily = $this->FontFamily;
		$prev_FontStyle = $this->FontStyle;
		$prev_FontSizePt = $this->FontSizePt;
		if($prev_FontFamily)
			$this->setNullOutput();
		$this->SetFont($font[0], $font[1], $font[2]);
		$lineh = $this->get_line_height($options);
		$cell_paddings = $this->getCellPaddings();
		$margin = $this->get_padding($options, '', $cell_paddings['L']);
		$border = array_get_default($options, 'border', 0);
		$defalign = $this->get_text_align($options);

		$oldmargins = $cell_paddings;
		$cell_padding_left = $this->get_padding($options, '', $cell_paddings['L']);
		$this->setCellPaddings($cell_padding_left, $cell_padding_left, $cell_padding_left, $cell_padding_left);

		$text = $cell['text'];
		$w = array_get_default($cell, 'width', $this->getPageWidth() - $this->getX() - $this->rMargin);
		$lines = $this->WrapText($text, $this->lv($w));
		$nl = count($lines);
		if($prev_FontFamily) {
			$this->setFont($prev_FontFamily, $prev_FontStyle, $prev_FontSizePt);
			$this->setNullOutput(false);
		}

		$rowH = $lineh * $nl + 2*$margin;
		$this->setCellPaddings($oldmargins['L'], $oldmargins['T'], $oldmargins['R'], $oldmargins['B']);
		return $rowH;
	}
	
	function moveX($offs) {
		$this->setX($this->getX() + $this->lv($offs));
	}
	
	function moveY($offs, $reset_x=true) {
		$x = $reset_x ? $this->lMargin : $this->getX();
		$this->setXY($x, $this->getY() + $this->lv($offs));
	}
	
	function RuleLine($options=array()) {
		$width = $this->get_width($options, '', '100%');
		$ht = $this->lv(array_get_default($options, 'height', $this->LineWidth));
		$prevht = $this->LineWidth;
		$this->setLineWidth($ht);
		$prevcolor = $this->DrawColor;
		if(isset($options['color'])) {
			$c = $options['color'];
			$this->setDrawColor($c[0], $c[1], $c[2]);
		}
		
		$padding = $this->get_padding($options, '', '3pt');
		$this->moveY($padding);
		$m = $this->getMargins();
		$maxw = $this->getPageWidth();
		$x = ($maxw - $m['left'] - $m['right'] - $width) / 2 + $m['left'];
		$x2 = $x - $m['left'] + $m['right'];
		$this->Line($x, $this->getY(), $maxw - $x2, $this->getY());
		$this->setLineWidth($prevht);
		$this->DrawColor = $prevcolor;
		if($this->page>0) {
			$this->_out($this->DrawColor);
		}
		$this->moveY($padding);
	}
	
	function get_monospace_font_face() {
		global $pdf_mono_fonts;
		if(isset($this->preferred_font_mono))
			return $this->preferred_font_mono;
		if(! isset($pdf_mono_fonts) || ! is_array($pdf_mono_fonts))
			return $this->default_font_mono;
		$def = $this->default_font_face;
		if(in_array($def, $pdf_mono_fonts))
			return $def;
		if($def == 'vera' || $def == 'verase')
			return 'veramo';
		return $this->default_font_mono;
	}
	
	function render_config() {
		if(empty($_REQUEST['record']))
			sugar_die('No record ID provided');

		$fields = $this->get_config_fields();
		$layout = $this->get_config_layout($fields);		
		$result = new RowResult;
		$result->row = array();
		foreach($fields as $k => $f) {
			if(isset($f['default']))
				$result->row[$k] = $f['default'];
			if(isset($f['default_id']))
				$result->row[$f['id_name']] = $f['default_id'];
		}
		$result->fields = $fields;
		
		require_once('include/layout/forms/FormGenerator.php');
		$model = new ModelDef($this->focus_type);
		$model->addDisplayFieldDefinitions($result->fields);
		$gen = FormGenerator::html_form($model, $layout, 'printForm');
		$gen->getFormObject()->max_ref_width = 50;
		$gen->renderForm($result);
		echo $gen->getResult();
		$gen->exportIncludes();
		$t = javascript_escape($this->get_config_title());
		$GLOBALS['pageInstance']->add_js_literal("SUGAR.popups.initContent({title: '$t'});", null, LOAD_PRIORITY_FOOT);
	}
	
	function default_company_address() {
		$rs = AppConfig::db_all_rows('CompanyAddress');
		$val = null;
		foreach($rs as $a) {
			if(! isset($val) || $a['main'])
				$val = $a;
		}
		return $val;
	}
	
	function get_config_fields() {
		$addr = $this->default_company_address();
		$fields = array(
			'pdf_use_address' => array(
				'vname' => 'LBL_USE_ADDRESS',
				'vname_module' => 'app',
				'type' => 'ref',
				'bean_name' => 'CompanyAddress',
				'id_name' => 'use_address',
				'required' => true,
				'default_id' => $addr['id'],
				'default' => $addr['name'],
				'width' => 45,
				'editable' => true,
			),
		);
		$langs = AppConfig::get_languages();
		if(count($langs) > 1) {
			$fields['pdf_language'] = array(
				'vname' => 'LBL_USE_LANGUAGE',
				'vname_module' => 'app',
				'type' => 'enum',
				'required' => true,
				'options' => $langs,
				'default' => AppConfig::language(),
				'width' => 45,
				'editable' => true,
			);
		}
		return $fields;
	}
	
	function get_config_title() {
		return '';
	}
	
	function get_config_buttons() {
		$buttons = array(
			'print' => array(
				'icon' => 'icon-print',
				'label' => translate('LBL_OPEN_AS_PDF', 'app'),
				'params' => array(),
				'async' => false,
				'align' => 'right',
				'target' => '_blank',
				'post_perform' => "SUGAR.popups.close()",
			),
			'email' => array(
				'icon' => 'theme-icon module-Emails',
				'label' => translate('LBL_EMAIL_AS_PDF', 'app'),
				'params' => array(
					'attach_to_email' => true,
				),
				'async' => false,
				'align' => 'right',
			)
		);
		return $buttons;
	}
	
	function get_config_hidden() {
		return array(
			'module' => AppConfig::module_for_model($this->focus_type),
			'action' => 'PDF',
			'record' => $_REQUEST['record'],
			'pdf_type' => $this->pdf_type,
		);
	}
	
	function get_config_layout($fields) {
		$layout = array(
			'type' => 'config',
			'editable' => true,
			'buttons_position' => 'bottom',
			'form_hidden_fields' => $this->get_config_hidden(),
			'form_buttons' => $this->get_config_buttons(),
			'sections' => array(
				array(
					'label_position' => 'top',
					'columns' => 1,
					'elements' => array_keys($fields),
				),
			),
		);
		return $layout;
	}

	public function getPageBreakTrigger()
	{
		return $this->PageBreakTrigger;
	}

}
