<?php

class TCPDFHelper extends TCPDF
{

    protected $AliasNbPages;
	protected $ws = 0;
	protected $null_output;

    function __construct($orientation='P', $unit='mm', $format='A4', $unicode=true, $encoding='UTF-8', $diskcache=false, $pdfa=false) {
        parent::__construct($orientation, $unit, $format, $unicode, $encoding,$diskcache,$pdfa);
    }

	public function Image($file, $x=null, $y=null, $w=0, $h=0, $type='', $link='', $align='', $resize=false, $dpi=300, $palign='', $ismask=false, $imgmask=false, $border=0, $fitbox=false, $hidden=false, $fitonpage=false, $alt=false, $altimgs=array()) {
		if (is_string($file) && strpos($file, '://') === false && strpos($file, '/') !== 0)
			$file = getcwd() . '/' . $file;
		return parent::Image($file, $x, $y, $w, $h, $type, $link, $align, $resize, $dpi, $palign, $ismask, $imgmask, $border, $fitbox, $hidden, $fitonpage, $alt, $altimgs);
	}

	public function Output($name='doc.pdf', $dest='I') {
		if (is_string($name) && strpos($name, '://') === false && strpos($name, '/') !== 0)
			$name = getcwd() . '/' . $name;
		return parent::Output($name, $dest);
	}

    public function AliasNbPages($alias='{nb}') {
        //Define an alias for total number of pages //TODO add brackets if font isn't unicode?
        TCPDF_STATIC::$alias_tot_pages = $this->AliasNbPages = $this->_escapetext($alias);
    }

    // Disable Pagegroups
    public function getPageGroupAlias() {
        return $this->getNumPages();
    }
    // Disable Pagegroups
    public function getGroupPageNo() {
        return $this->getPage();
    }

    // wrap text, returning an array of line widths and text
    // use $w=-1 to only wrap at newlines
    // adapted from MultiCell
    public function WrapText(&$txt, $w=0, $margin=-1) {
        $lines = array();

		$cell_paddings = $this->getCellPaddings();
        if($margin < 0) $margin = $cell_paddings['L'];

        if($w == 0)
            $wmax = $this->w - $this->rMargin - $this->x - 2 * $margin;
        else
            $wmax = $w - 2 * $margin;

		require_once 'include/utils/html_utils.php';
        $s = str_replace("\r", '', $txt); // remove carriage returns (UTF-8 safe)
        $nb = strlen($s);

        $sep=-1;
        $i=0;
        $j=0;
        $l=0;
        $ns=0;

        while($i<$nb) {
            //Get next character
            $c = mb_substr($s, $i, 1); //$s{$i};
            if(preg_match("/[\n]/u", $c)) {
                $t = mb_substr($s, $j, $i-$j);
                $l = $this->GetStringWidth($t);
                $lines[] = array($l, 0, $t);
                $i++;
                $sep=-1;
                $j=$i;
                $l=0;
                $ns=0;
                continue;
            }
            if($isspace = preg_match("/[ ]/u", $c) ? 1 : 0) {
                $sep = $i;
                //$ls = $l;
                $ns++;
            }

            $l = $this->GetStringWidth(mb_substr($s, $j, $i-$j));

            if($wmax > 0 && $l > $wmax) {
                //Automatic line break
                if($sep == -1) {
                    if($i == $j) {
                        $i++;
                    }
                    $t = mb_substr($s, $j, $i-$j);
                    $l = $this->GetStringWidth($t);
                    $lines[] = array($l, 0, $t);
                }
                else {
                    $t = mb_substr($s, $j, $sep-$j);
                    $l = $this->GetStringWidth($t);
                    $ws = ($ns>1) ? ($wmax-$l)/($ns-1) : 0; // used in justifying text
                    $lines[] = array($l, $ws, $t);
                    $i = $sep + 1;
                }
                $sep=-1;
                $j=$i;
                $l=0;
                $ns=0;
            }
            else {
                $i++;
            }
        }
        //Last chunk
        if($i > $j) {
            $t = mb_substr($s, $j, $i-$j);
            $l = $this->GetStringWidth($t);
            $lines[] = array($l, 0, $t);
        }
        return $lines;
    }

    // calculate the minimum printed width of a string
    public function MinTextWidth(&$txt, $nowrap=false, $margin=-1, $breaklen=0) {
    	$cell_paddings = $this->getCellPaddings();
        if($margin < 0) $margin = $cell_paddings['L'];
        require_once 'include/utils/html_utils.php';
		$txt = html2plaintext((htmlentities ($txt)));
        $s = str_replace("\r", '', $txt); // remove carriage returns (UTF-8 safe)
        $nb = strlen($s);
        $head = -1;
        $minw = 0;
        $pat = $nowrap ? "/[\n]/u" : "/[\n| ]/u";

        for($i = 0; $i < $nb; $i++) {
            $c = mb_substr($s, $i, 1);
            if(preg_match($pat, $c)) {
                $head = -1;
            }
            else {
                if($head < 0)
                    $head = $i;
                if(! $nowrap && $breaklen && $i-$head > $breaklen)
                    $head = $i - $breaklen;
                $w = $this->GetStringWidth(mb_substr($s, $head, $i-$head+1));
                if($w > $minw)
                    $minw = $w;
            }
        }
        return $minw + 2*$margin;
    }

    // draw a list of pre-wrapped lines
    public function MultiCell2($w, $h, $lines, $border=0, $align='J', $fill=0, $link='', $fill_h = -1) {
        $k = $this->k;
        if($w == 0) {
            $w = $this->w - $this->rMargin - $this->x;
        }
        $cell_paddings = $this->getCellPaddings();
        $wmax = ($w - 2 * $cell_paddings['L']);
        $b=0;
        if($border) {
            if(is_numeric($border))
                $border = 'LTRB';
        }
        $origWs = $this->ws;
        if($align != 'J') {
            $this->ws = 0;
            $this->_out('0 Tw');
        }
        $nl = is_countable($lines) ? count($lines) : 0;
        if(! $nl) {
            $lines[] = array(0, 0, '');
            $nl ++;
        }
        $origX = $this->getX();
        $endY = $this->getY();
        $def_margins = array($cell_paddings['L'], $cell_paddings['T'], $cell_paddings['R'], $cell_paddings['B']);
        $i = 0;
        $lastwritten = -1;
        $x = $this->x;
        $top = $y = $this->y;
        $texty = array();

        $last_h = $h;
        $total_h = $nl * $h;
        $fill_h -= $cell_paddings['T'];
        $fill_h -= $cell_paddings['B'];
        if($fill_h > 0) {
            if($fill_h < $total_h) {
                // shrink cell to fill_h?
                //$last_h = max(0, $last_h - $total_h + $fill_h);
            } else if($fill_h > $total_h) {
                $fill_h -= $total_h;
                while ($fill_h > 0) {
                    $lines[] = array($w, 0, '');
                    $nl++;
                    $fill_h -= $h;
                }
                $last_h = $fill_h + $h;
            }
        }

        $save_h = $h;
        while($i < $nl) {
            $h = ($i == $nl -1) ? $last_h : $save_h;
            $margins = $def_margins;
            if($i > 0)
                $margins[1] = 0;
            if($i < $nl-1)
                $margins[3] = 0;

            $ty = $y + $margins[1];
            $texty[$i] = $ty;
            $y2 = $ty + $h + $margins[3];
            $pageBreak = ($y2 - $margins[3] + $def_margins[3] > $this->PageBreakTrigger && empty($this->InFooter) && $this->AcceptPageBreak());

            // Draw block of lines
            if(++ $i == $nl || $pageBreak) {
                if($pageBreak)
                    $i--;
                $nw = $i - $lastwritten;
                $b = $border;
                if($lastwritten >= 0) {
                    $b = str_replace('T', '', $b);
                }
                if($pageBreak) {
                    $b = str_replace('B', '', $b);
                }
                $s = '';
                if($nw) {
                    if($fill) {
                        $op = 'f';
                        $s = sprintf('%.2f %.2f %.2f %.2f re %s ', $x * $k, ($this->h - $top) * $k, $w * $k, ($top-$y2) * $k, $op);
                    }
                    if($border) {
                        if(strpos($border,'L') !== false) {
                            $s .= sprintf('%.2f %.2f m %.2f %.2f l S ',$x*$k,($this->h-$top)*$k,$x*$k,($this->h-$y2)*$k);
                        }
                        if(strpos($border,'T') !== false) {
                            $s .= sprintf('%.2f %.2f m %.2f %.2f l S ',$x*$k,($this->h-$top)*$k,($x+$w)*$k,($this->h-$top)*$k);
                        }
                        if(strpos($border,'R') !== false) {
                            $s .= sprintf('%.2f %.2f m %.2f %.2f l S ',($x+$w)*$k,($this->h-$top)*$k,($x+$w)*$k,($this->h-$y2)*$k);
                        }
                        if(strpos($border,'B') !== false) {
                            $s .= sprintf('%.2f %.2f m %.2f %.2f l S ',$x*$k,($this->h-$y2)*$k,($x+$w)*$k,($this->h-$y2)*$k);
                        }
                    }
                    $addsp = false;
                    for($j = $lastwritten + 1; $j < $i; $j++) {
                        $h = ($j == $nl -1) ? $last_h : $save_h;
                        list($width, $ws, $text) = $lines[$j];
                        if($text === '')
                            continue;
                        $ty = $texty[$j];
                        if($addsp)
                            $s .= ' ';
                        if($align == 'R') {
                            $dx = $w - $margins[2] - $width;
                        }
                        elseif($align=='C') {
                            $dx = ($w - $width)/2;
                        }
                        else {
                            $dx = $margins[0];
                            if($align=='J' && $ws != $this->ws) {
                                $this->ws = $ws;
                                $s .= sprintf('%.3f Tw ', $this->ws * $k);
                            }
                        }
                        if($this->ColorFlag) {
                            $s .= 'q '.$this->TextColor.' ';
                        }

                        $txt2 = $this->_escapetext($text);
                        //$basefonty = $ty + (($h + $this->FontAscent - $this->FontDescent)/2);
                        $basefonty = $ty + ($h/2) + ($this->FontSize/3);
                        $s .= sprintf('BT %.2f %.2f Td (%s) Tj ET', ($x + $dx) * $k, ($this->h - $basefonty) * $k, $txt2);
                        if($this->underline) {
                            $s.=' '.$this->_dounderline($x + $dx, $ty + 0.5 * $h + 0.3 * $this->FontSize, $text);
                        }
                        if($this->ColorFlag) {
                            $s.=' Q';
                        }
                        // can we call Link once per block?
                        if($link) {
                            $this->Link($x + $dx, $ty + 0.5 * $h - 0.5 * $this->FontSize, $width, $this->FontSize, $link);
                        }
                        $addsp = true;
                    }
                }
                if($s) {
                    $this->_out($s);
                }
                $lastwritten = $i-1;

                if($pageBreak) {
                    $this->WrapPage();
                    $top = $y = $this->tMargin;
                    $this->setXY($x, $y);
                }
                else {
                    $this->setXY($x, $y2);
                }
            }
            else {
                $y = $y2;
            }
        }
        if($this->ws != $origWs) {
            $this->ws = $origWs;
            $this->_out(sprintf('%.3f Tw', $this->ws * $k));
        }
    }

    //
    public function WrapPage() {
        $ws = $this->ws;
        if($ws > 0) {
            $this->ws = 0;
            $this->_out('0 Tw');
        }
        $this->AddPage($this->CurOrientation);
        if($ws > 0) {
            $this->ws = $ws;
            $this->_out(sprintf('%.3f Tw',$ws * $this->k));
        }
    }

    // CID font support
    protected function _putcidfont0($font) {
        $longname = $name = $font['name'];
        $enc = $font['enc'];
        if($enc)
            $longname .= "-$enc";

        $this->_newobj();
        $this->_out('<</Type /Font');
        $this->_out('/BaseFont /'.$longname);
        $this->_out('/Subtype /Type0');
        if($enc)
            $this->_out('/Encoding /'.$enc);
        $this->_out('/DescendantFonts ['.($this->n+1).' 0 R]');
        $this->_out('>>');
        $this->_out('endobj');

        $this->_newobj();
        $this->_out('<</Type /Font');
        $this->_out('/BaseFont /'.$name);
        $this->_out('/Subtype /CIDFontType0');
        $cidinfo = '/Registry ('.$font['cidinfo']['Registry'].') ';
        $cidinfo .= '/Ordering ('.$font['cidinfo']['Ordering'].') ';
        $cidinfo .= '/Supplement '.$font['cidinfo']['Supplement'];
        $this->_out('/CIDSystemInfo <<'.$cidinfo.'>>');
        $this->_out('/FontDescriptor '.($this->n+1).' 0 R');
        $codes = array_keys($font['cw']);
        $first = current($codes);
        $last = end($codes);
        $this->_out('/DW '.$font['dw']);
        $w = '/W [';
        $ranges = array();
        $currange = 0;
        for($i = $first; $i <= $last; $i++) {
            if(isset($font['cw'][$i]) && ! $currange)
                $currange = $i-31;
            else if(! isset($font['cw'][$i]))
                $currange = 0;
            if($currange)
                $ranges[$currange][] = $font['cw'][$i];
        }
        foreach($ranges as $k => $ws)
            $w .= ' '.$k.' [ '.implode(' ', $ws).' ]';
        $w .= ' ]';
        $this->_out($w);
        $this->_out('>>');
        $this->_out('endobj');

        $this->_newobj();
        $s='<</Type /FontDescriptor /FontName /'.$name;
        foreach($font['desc'] as $k=>$v) {
            $s.=' /'.$k.' '.$v;
        }
        $this->_out($s.'>>');
        $this->_out('endobj');
    }

    public function getLeftMargin() {
        return $this->lMargin;
    }

    public function getTopMargin() {
        return $this->tMargin;
    }

    public function getMargins() {
    	$cell_paddings = $this->getCellPaddings();
        $ret = array(
            'left' => $this->lMargin,
            'top' => $this->tMargin,
            'right' => $this->rMargin,
            'bottom' => $this->bMargin,
            'header' => $this->header_margin,
            'footer' => $this->footer_margin,
            'cell' => $cell_paddings['L'],
        );
        return $ret;
    }

    public function getLoadedImage($filename) {
        if(! isset($this->images[$filename]))
            return null;
        return $this->images[$filename];
    }

    public function setNullOutput($flag=true) {
        $this->null_output = $flag;
    }

    /**
    * Format a text string
    * @param string $s string to escape.
    * @return string escaped string.
    * @access protected
    */
    protected function _escapetext($s) {
        if($this->isunicode) {

            // 1crm - start added
            if($this->CurrentFont['type'] == 'core' || $this->CurrentFont['type'] == 'TrueType')
                $s = TCPDF_FONTS::UTF8ToLatin1($s, true, $this->CurrentFont);
            else
            // 1crm - end added


            //Convert string to UTF-16BE and reverse RTL language
            $s = TCPDF_FONTS::utf8StrRev($s, false, $this->tmprtl, true, $this->CurrentFont);
        }
        return $this->_escape($s);
    }

    /**
     * Add \ before \, ( and )
     * @param string $s string to escape.
     * @return string escaped string.
     * @access protected
     */
    protected function _escape($s) {
        // the chr(13) substitution fixes the Bugs item #1421290.
        return strtr($s, array(')' => '\\)', '(' => '\\(', '\\' => '\\\\', chr(13) => '\r', chr(160) => ' '));
    }
}
