<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');

/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/

require_once("include/templates/Template.php");

class TemplateDragDropChooser extends Template {
    var $args;
/*
 * This function creates the html and uses the args parameter to call the class file
 * ideally, you would want to call the displayScriptTags() function, 
 * followed by the displayDefinitionScript();
 * and lastly call the display function 
 */
    function display(){

  /*   valid entries for expected arguments array are as follow:
   *   args['left_header'] = value of left table header
   *   args['mid_header'] = value of middle table header
   *   args['right_header'] = value of right table header
   *   args['left_data'] = array to use in left data.
   *   args['mid_data'] = array to use in middle data.   
   *   args['right_data'] =array to use in right data.
   *   args['title'] =  title (if any) to be used.
   *   args['classname'] =  name to be used as class. This helps when defining multiple templates on one screen.
   *   args['left_div_name'] = Name to be used for left div (should be unique)
   *   args['mid_div_name'] = Name to be used for middle div (should be unique)
   *   args['right_div_name'] = Name to be used for right div (should be unique)
   *   args['gridcount'] = Number of grids to show.  Acceptable Values are 'one','two' and 'three'
   *                       The string is converted to numeric values, so you could also set these directly  
   *                       The values are Zero Based, so to display one column, set to '0'
   *                       To display two columns set to '1', to display three columns set to '2'.
   * $this->args['return_array']= if this is set to true, then 'display()' function returns html and javascript
   *                              in array format.  This will allow granular control of html so that you can
   *                              seperate the tables and customize the grid   
   */    
   	
		$dd_id = array_get_default($this->args, 'classname', 'DragDropGrid');
		
		if(isset($this->args['columns'])) {
			// IAH-added interface
			$columns = $this->args['columns'];
			$columnsCount = is_countable($columns) ? count($columns) : 0;
			$this->args['gridcount'] = $columnsCount - 1;
		} else {
			// old, more limited interface
			//convert values for gridcount in case they are in string form
			if($this->args['gridcount'] == 'one'){
				$this->args['gridcount'] = 0;
			}elseif($this->args['gridcount'] == 'two'){
				$this->args['gridcount'] = 1;
			}elseif($this->args['gridcount'] == 'three'){
				$this->args['gridcount'] = 2;
			}

			$columns = array();
			$columns[] = array('data' => $this->args['left_data'], 'name' => $this->args['left_div_name'], 'title' => $this->args['left_header']);
			if($this->args['gridcount'] == 2)
				$columns[] = array('data' => $this->args['mid_data'], 'name' => $this->args['mid_div_name'], 'title' => $this->args['mid_header']);
			if($this->args['gridcount'] > 0)
				$columns[] = array('data' => $this->args['right_data'], 'name' => $this->args['right_div_name'], 'title' => $this->args['right_header']);
		}
		
		$height = array_get_default($this->args, 'height', '270px');
		
		$script = "<script type='text/javascript'> TemplateDragDropChooser.{$dd_id}_cols = [];";
		$script .= 'TemplateDragDropChooser.initialize("' . $dd_id . '");';
		$html_str_arr['begin'] = '<div style="width: 600px;" id="' . $dd_id . '"><div>';
		$j = 0;

		foreach($columns as $i => $col) {
			$w = array_get_default($col, 'width', '180px');
			$editable = array_get_default($col, 'editable', true);

			$style = 'float: left; width:'.$w.'; height:'.$height.'; overflow: hidden;' . array_get_default($col, 'style', '');
			$data = "<div id='{$col['name']}' style='{$style}'>";
			$title = array_get_default($col, 'title', '');

			if($title) {
				$title_style = 'cursor: default; display: block; text-align: center; font-size: 10px; '
					. 'margin: 0; margin-bottom: 0.5em; border-bottom: 1px solid gray; ' . array_get_default($col, 'title_style', '');
				$data .= '<h4 class="dragDropTitle" style="'.$title_style.'">'.$title.'</h4>';
			}

			$ul_style = 'margin: 0; padding: 0; height: 245px; overflow-x: hidden; overflow-y: auto; '. array_get_default($col, 'ul_style', '');

			$ul_class = '';
			if ($editable)
				$ul_class = 'dragDropList';

			$data .= "<ul id='{$col['name']}_list' class='{$ul_class}' style='$ul_style'>";

			$li_style = 'margin: 2px; padding: 0.25em; ';
			if($editable) $li_style .= 'background-color: #ddd; cursor: move; ';
			$li_style .= array_get_default($col, 'li_style', '');

			if(isset($col['data'])) {
				foreach($col['data'] as $val) {
					$lbl = $val[1];
					$title = '';
					if(empty($col['no_escape_labels'])) {
						$lbl = htmlspecialchars($val[1], ENT_COMPAT, 'UTF-8');
						$title = $lbl;
					}
					$reqd = ! empty($val[2]);
					$etc = $reqd ? ' isRequired="true"' : '';
					$data .= '<li id="'.$val[0].'" class="noBullet dragDropItem" style="'.$li_style.'" title="'.$title.'"'.$etc.'><div>'.$lbl.'</div></li>';

					if($editable) {
						$script .= "TemplateDragDropChooser.makeDraggable('{$val[0]}');";
						$j ++;
					}

				}
			}

			if($editable) {
				$upd_var = array_get_default($col, 'update_field', $col['name'].'_values');
				$script .= "TemplateDragDropChooser.{$dd_id}_cols[$i] = {name: '{$col['name']}', field: '{$upd_var}'};";
			}

			$data .= '</ul></div>';
			$html_str_arr['col_'.$col['name']] = $data;
		}
		$html_str_arr['end'] = "</div></div>";

		$script .= '</script>';

		$html_str_arr['script'] = $script;

		 if(isset($this->args['return_array']) && $this->args['return_array']){
			return $html_str_arr;    
		 }else{
			return implode('', $html_str_arr);
		 }
    }

/*
 * This script is the javascript class definition for the template drag drop object.  This
 * makes use of the args['classname'] parameter to name the class and to prefix variables with.  This is done
 * dynamically so that multiple template dragdrop objects can be defined on the same page if needed
 * without having the variables mix up as you drag rows around.
 */
    function displayDefinitionScript() {
        return '<script type="text/javascript">' . file_get_contents(dirname(__FILE__) . '/TemplateDragDropChooser.js') . '</script>';
    }
      
}

?>
