<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/
/*********************************************************************************

 * Description:
 ********************************************************************************/

require_once('include/utils/file_utils.php');

class UploadFile {

	var $field_name;
	var $stored_file_name;
	var $thumbnail_file_name;
	var $original_file_name;
	var $mime_type;
	var $temp_file_location;
	var $temp_thumbnail_location;
	var $source_file_path;
	var $use_soap = false;
	var $incoming_id = false;
	var $incoming_result = null;
	var $file;
	var $file_ext;
	var $upload_dir = null;
	var $old_upload_dir = null;
	var $bad_exts = array();

	// longreach - added default for $field_name - not always used
	function __construct ($field_name='') {
		if(is_array($field_name)) {
			$spec = $field_name;
			if(! empty($spec['upload_dir'])) {
				$dir = $spec['upload_dir'];
				$dir = self::replace_placeholders($dir);
				$this->set_upload_dir($dir);
			}
			$field_name = $spec['name'];
		}
		// $field_name is the name of your passed file selector field in your form
		// i.e., for Emails, it is "email_attachmentX" where X is 0-9
		$this->field_name = $field_name;
		$this->bad_exts = self::get_bad_exts();
	}

	static function get_bad_exts() {
		return array_map('strtolower', AppConfig::setting('site.upload.badext', array()));
	}

	static public function replace_placeholders($dir) {
		$dir = str_replace('{CACHE}', AppConfig::cache_dir(), $dir);
		$dir = str_replace('{FILES}', AppConfig::files_dir(), $dir);
		$dir = str_replace('//', '/', $dir);
		return $dir;
	}

	static function get_upload_dir(&$self) {
		if(isset($self) && $self instanceof UploadFile)
			return $self->upload_dir();
		return AppConfig::upload_dir();
	}

	function upload_dir() {
		if(isset($this->upload_dir))
			return $this->upload_dir;
		return AppConfig::upload_dir();
	}
	function set_upload_dir($p) {
		$this->old_upload_dir = $this->upload_dir;
		$this->upload_dir = $p;
	}
	function revert_upload_dir() {
		$this->upload_dir = $this->old_upload_dir;
		$this->old_upload_dir = null;
	}

	function set_for_soap($filename, $file) {
		$this->stored_file_name = $filename;
		$this->use_soap = true;
		$this->file = $file;
		$this->mime_type = $this->getMimeSoap($filename);
	}


	/**
	 * wrapper for this::get_file_path()
	 * @param string stored_file_name File name in filesystem
	 * @param string bean_id note bean ID
	 * @return string path with file name
	 */
	function get_url($stored_file_name, $bean_id='', $upload_dir=null) {
		if(! isset($upload_dir))
			$upload_dir = self::get_upload_dir($this);
		/* longreach - removed
		return UploadFile::get_file_path($stored_file_name,$bean_id);
		*/
		// longreach - start added

		if(($p = strrpos($stored_file_name, '/')) !== false) {
			$bean_id = substr($stored_file_name, 0, $p+1);
			$stored_file_name = substr($stored_file_name, $p+1);

			if(substr($bean_id, 0, 6) == 'email/')
				$upload_dir = AppConfig::files_dir();
        } else {
            //$bean_id = '1/';
        }
		return $upload_dir . $bean_id . rawurlencode($stored_file_name);
		// longreach - end added
	}

	/**
	 * builds a URL path for an anchor tag
	 * @param string stored_file_name File name in filesystem
	 * @param string bean_id note bean ID
	 * @return string path with file name
	 */
	function get_file_path($stored_file_name, $bean_id='') {
		$upload_dir = self::get_upload_dir($this);
		return self::get_path($stored_file_name, $bean_id, $upload_dir);
	}

	static function get_path($stored_file_name, $bean_id='', $upload_dir=null) {
		if(! isset($upload_dir)) $upload_dir = AppConfig::upload_dir();
		$upload_dir = self::replace_placeholders($upload_dir);
		if(($p = strrpos($stored_file_name ?? '', '/')) !== false) {
			$bean_id = substr($stored_file_name, 0, $p+1);
			$stored_file_name = substr($stored_file_name, $p+1);

			if(substr($bean_id, 0, 6) == 'email/')
				$upload_dir = AppConfig::files_dir();
        } else {
            if(! strlen($stored_file_name ?? '')) return '';
        }
		return $upload_dir . $bean_id . $stored_file_name;
	}


	// longreach - start added
	static function get_display_filename($stored_file_name) {
		$fname = $stored_file_name;
		if(($p = strrpos($fname, '/')) !== false)
			$fname = substr($fname, $p+1);
		return $fname;
	}

	function get_file_size($stored_file_name, $bean_id, $formatted=true) {
		$fpath = UploadFile::get_file_path($stored_file_name, $bean_id);
		$sz = filesize($fpath);
		if($formatted)
			$sz = format_file_size($sz);
		return $sz;
	}

	function get_download_url(&$bean) {
		return 'download.php?type='.$bean->module_dir.'&id='.$bean->id;
	}
	// longreach - end added


	/**
	 * duplicates an already uploaded file in the filesystem.
	 * @deprecated Does nothing
	 * @param string old_id ID of original note
	 * @param string new_id ID of new (copied) note
	 * @param string filename Filename of file (deprecated)
	 */
	function duplicate_file($file_name, $new_filename) {
		// $destination = $this->upload_dir() . $new_id;
		// if(!copy($source, $destination)) {
		// 	$GLOBALS['log']->warn("upload_file could not copy [ {$source} ] to [ {$destination} ]");
		// }
		// $source = $this->upload_dir() . $old_id . $file_name;
		// $destination = $this->upload_dir() . $new_id . $file_name;


		// // longreach - start added
		// $p = strrpos($stored_file_name, '/');
		// if($p !== false) {
		// 	$source = $this->upload_dir() . $file_name;
		// 	$this->stored_file_name = substr($stored_file_name, $p+1);
		// 	$destination = $this->get_upload_path($new_id);
		// }
		// // longreach - end added


		// copy($source, $destination);
	}

	static function duplicate_file2($file_name) {
		$file = new UploadFile;
		$file->use_soap = true;
		$file->stored_file_name = $file_name;
		$src = $file->get_upload_path('');

		if (is_file($src)) {
			$basename = basename($file_name);
			$file->stored_file_name = $basename;
			$file->create_stored_filename();
			$dst = $file->get_upload_path('');
			copy($src, $dst);
			return $file->stored_file_name;
		} else {
			return '';
		}
	}

	/**
	 * standard PHP file-upload security measures. all variables accessed in a global context
	 * @return bool True on success
	 */
	function confirm_upload($check_incoming=null) {
		if($check_incoming) {
			$reqval = array_get_default($_REQUEST, $this->field_name);
			if(isset($reqval) && substr($reqval, 0, 9) == 'INCOMING~') {
				$inc_id = substr($reqval, 9);
				$result = ListQuery::quick_fetch('incoming_files', $inc_id);
				if($result && $result->row['user_id'] == AppConfig::current_user_id()) {
					$this->incoming_result = $result;
					$this->incoming_id = $inc_id;
					$this->mime_type = $result->row['file_mime_type'];
					$thumb = (! empty($result->row['thumbnail_filename'])) ? $result->row['thumbnail_filename'] : null;
					$this->create_stored_filename($result->row['upload_filename'], $thumb);
					$this->temp_file_location = CacheManager::get_location('incoming/') . $result->row['filename'];
					if($thumb)
						$this->temp_thumbnail_location = CacheManager::get_location('incoming/') . $thumb;
					unset($_REQUEST[$this->field_name]);
					return true;
				}
				throw new IAHUploadError("Incoming file ID not recognized");
			}
		}

		if(! isset($_FILES[$this->field_name]))
			return false;

		// longreach - start added - handle upload errors
		global $log;
		if(! empty($_FILES[$this->field_name]['error'])) {
			switch($_FILES[$this->field_name]['error']) {
				case UPLOAD_ERR_INI_SIZE:
					$log->fatal('Uploaded file size exceeded PHP setting upload_max_filesize');
					throw new IAHUploadError("The uploaded file exceeds the server's maximum upload size.");
				case UPLOAD_ERR_FORM_SIZE: // not going to happen
					throw new IAHUploadError("The uploaded file could not be processed.");
				case UPLOAD_ERR_PARTIAL:
					$log->fatal('Detected partial file upload (possibly trying to upload a MacOS resource fork)');
					throw new IAHUploadError("The uploaded file could not be processed. If using MacOS, try adding the file to a zip archive first.");
				case UPLOAD_ERR_NO_FILE:
					return false;
				default:
					// NO_TMP_DIR or CANT_WRITE
					$log->fatal('Upload error - missing upload temp directory, or file could not be written to');
					throw new IAHUploadError("The uploaded file could not be processed. Please check your server settings.");
			}
		}
		// longreach - end added

		$max_upload = (int)AppConfig::setting('site.upload.maxsize');

		if(!is_uploaded_file($_FILES[$this->field_name]['tmp_name'])) {
			return false;
		} elseif($_FILES[$this->field_name]['size'] > $max_upload) {
			throw new IAHUploadError("ERROR: uploaded file was too big: max filesize: $max_upload");
		}
		// longreach - start added
		else if ($_FILES[$this->field_name]['size'] == 0) {
			$log->fatal("Received 0 bytes of uploaded file ({$_FILES[$this->field_name]['name']})");
			throw new IAHUploadError("ERROR: uploaded file ({$_FILES[$this->field_name]['name']}) was empty - this may indicate a problem with your browser or platform.");
		}
		// longreach - end added

		if(!is_writable($this->upload_dir())) {
			throw new IAHUploadError("ERROR: cannot write to directory: {$this->upload_dir()} for uploads");
		}

		$this->mime_type =$this->getMime($_FILES[$this->field_name]);
		$this->stored_file_name = $this->create_stored_filename();
		$this->temp_file_location = $_FILES[$this->field_name]['tmp_name'];

		return true;
	}

	function getMimeSoap($filename){

		if( function_exists( 'ext2mime' ) )
		{
			// Just to silence Intelephence
			$ext2mime = 'ext2mime';
			$mime = $ext2mime($filename);
		}
		else
		{
			$mime = 'application/octet-stream';
		}
		return $mime;

	}
	function getMime($file)
	{

		if( $file['type'] )
		{
			$mime = $file['type'];
		}
		elseif( function_exists( 'mime_content_type' ) )
		{
			$mime = mime_content_type( $file['tmp_name'] );
		}
		elseif( function_exists( 'ext2mime' ) )
		{
			$ext2mime = 'ext2mime';
			$mime = $ext2mime($file['name']);
		}
		else
		{
			$mime = 'application/octet-stream';
		}
		return $mime;
	}

	/**
	 * gets note's filename
	 * @return string
	 */
	function get_stored_file_name() {
		return $this->stored_file_name;
	}

	function get_thumbnail_file_name() {
		return $this->thumbnail_file_name;
	}

	/**
	 * creates a file's name for preparation for saving
	 * @return string
	 */
	function create_stored_filename($from_name=null, $thumbnail=false) {
		if(isset($from_name)) {
			$stored_file_name = $this->original_file_name = $from_name;
		}
		else if(!$this->use_soap) {
			$stored_file_name = urldecode(stripslashes($_FILES[$this->field_name]['name']));
			$this->original_file_name = $stored_file_name;

			/**
			 * cn: bug 8056 - windows filesystems and IIS do not like utf8.  we are forced to urlencode() to ensure that
			 * the file is linkable from the browser.  this will stay broken until we move to a db-storage system
			 */
			if(is_windows()) {
				// create a non UTF-8 name encoding
				// 176 + 36 char guid = windows' maximum filename length
				$end = (strlen($stored_file_name) > 176) ? 176 : strlen($stored_file_name);
				$stored_file_name = substr($stored_file_name, 0, $end);
				$this->original_file_name = urldecode(stripslashes($_FILES[$this->field_name]['name']));
			}
		} else {
			$stored_file_name = $this->stored_file_name;
			$this->original_file_name = $stored_file_name;
		}

		// no leading dots (hidden files)
		$stored_file_name = preg_replace('~^\.+~', '_', $stored_file_name);
		if($stored_file_name == 'index.html' || $stored_file_name == 'index.htm')
			$stored_file_name = '_' . $stored_file_name;

        $ext_pos = strrpos($stored_file_name, '.');
        if($ext_pos) {
			$file_ext = trim(strtolower(substr($stored_file_name, $ext_pos + 1)));
			if(in_array($file_ext, $this->bad_exts))
				$ext_pos = false;
		}
		if(! $ext_pos) {
			if(! $stored_file_name)
				$stored_file_name = md5(time() . rand());
			$ext_pos = strlen($stored_file_name);
			$stored_file_name .= ".txt";
			$file_ext = "txt";
		}
		$base_name = substr($stored_file_name, 0, $ext_pos);
		$this->file_ext = $file_ext;

		$upload_dir = $this->upload_dir();
		$prefix = '';
		for($i = 1; ; $i++) {
			$dirpath = $upload_dir . $i;
			$prefix = $i . '/';
			$file_name = $prefix . $stored_file_name;
			if(!file_exists($dirpath)) {
				mkdir($dirpath); // need to set permissions?
				// disable directory indexing
				$idxhtml = fopen($dirpath . '/' . 'index.html', 'w');
				fclose($idxhtml);
				break;
			}
			if(!file_exists($upload_dir . $file_name))
				break;
		}
		$this->stored_file_name = $stored_file_name = $file_name;

		if($thumbnail)
			$this->thumbnail_file_name = $prefix . $base_name . '_t.' . $this->file_ext;
		else
			$this->thumbnail_file_name = null;

		return $stored_file_name;
	}

	/**
	 * moves uploaded temp file to permanent save location
	 * @param string bean_id ID of parent bean
	 * @return bool True on success
	 */
	function final_move($bean_id='', $explicit_filename = false) {
		if ($explicit_filename)
			$destination = $bean_id;
		else
	        $destination = clean_path($this->get_upload_path($bean_id));
        if($this->use_soap) {
        	$fp = fopen($destination, 'wb');
        	if(!fwrite($fp, $this->file)){
        		throw new IAHUploadError("ERROR: can't save file to $destination");
        	}
        	fclose($fp);
		} else if($this->incoming_id && $this->temp_file_location) {
			if(! rename($this->temp_file_location, $destination)) {
				throw new IAHUploadError("ERROR: can't move uploaded file to $destination. Please ensure the directory is writable by the web server");
			}
			if($this->temp_thumbnail_location) {
				$t_path = clean_path($this->get_thumbnail_path());
				if($t_path)
					rename($this->temp_thumbnail_location, $t_path);
			}
			$this->clean_incoming();
		} else {
			if(!move_uploaded_file($_FILES[$this->field_name]['tmp_name'], $destination)) {
				throw new IAHUploadError("ERROR: can't move uploaded file to $destination. Please ensure the directory is writable by the web server");
			}
		}
		return true;
	}

	function remove() {
		if($this->clean_incoming()) return true;
		if($this->temp_file_location) {
			@unlink($this->temp_file_location);
			return true;
		}
	}

	function clean_incoming()
	{
		if($this->incoming_result) {
			self::remove_incoming($this->incoming_result);
			$this->incoming_result = null;
			return true;
		}
	}

	/**
	 * moves uploaded temp file into 'incoming' directory
	 * @return bool True on success
	 */
	function save_incoming($user_id, $params=null) {
		if(! is_array($params)) $params = array();

		$real_name = $this->get_stored_file_name();
		if( ($rpos = strrpos($real_name, '/')) !== false)
			$real_name = substr($real_name, $rpos+1);
		$tmp_name = md5($real_name . time() . rand());
		$tmp_dir = CacheManager::get_location('incoming/');
		$tmp_path = $tmp_dir . $tmp_name;
		if($this->use_soap) {
			$fp = fopen($tmp_path, 'wb');
			if(fwrite($fp, $this->file) === false){
				throw new IAHUploadError("Can't save file to $tmp_path");
			}
			fclose($fp);
		} elseif($this->source_file_path) {
			if(! rename($this->source_file_path, $tmp_path)) {
				throw new IAHUploadError("Can't rename {$this->source_file_path} to $tmp_path");
			}
		} else {
			if(! move_uploaded_file($_FILES[$this->field_name]['tmp_name'], $tmp_path)) {
				throw new IAHUploadError("Can't move uploaded file to $tmp_dir. Please ensure the directory is writable by the web server");
			}
		}

		$is_image = array_get_default($params, 'is_image') ? 1 : 0;
		$thumb_file = $thumb_size = $thumb_pos = $thumb_mime = null;
		if($is_image) {
			list($img_res, $img) = open_image_file($tmp_path, $real_name);

			if((! is_resource($img_res) && ! $img_res instanceof \GdImage) || ! $img['width'] || ! $img['height']) {
				@unlink($tmp_path);
				throw new IAHUploadError("Unrecognized image file");
			}
			$this->mime_type = $img['mime'];

			if(! empty($params['thumbnail'])) {
				$size = array_get_default($params, 'thumbnail_size');
				if(! is_array($size)) $size = array(100, 100);
				$square = array_get_default($size, 2, false);
				list($thumb_res, $thumb) = thumbnail_image($img_res, $img, $size[0], $size[1], $square);
				if(! is_resource($thumb_res) && ! $thumb_res instanceof \GdImage) {
					@unlink($tmp_path);
					throw new IAHUploadError("Error resampling image file");
				}
				$thumb_name = md5($real_name . 'thumb' . time() . rand());
				$thumb_file = $tmp_dir . $thumb_name;
				$thumb_save = $thumb['save'];
				if(! $thumb_save($thumb_res, $thumb_file)) {
					@unlink($tmp_path);
					throw new IAHUploadError("Error resampling image file");
				}
				$thumb_size = filesize($thumb_file);
				$thumb_mime = $thumb['mime'];
			}

			$max_size = array_get_default($params, 'max_size');
			if($max_size && ($max_size < $img['width'] || $max_size < $img['height'])) {
				$ratio = $max_size / max($img['width'], $img['height']);
				$newWidth = round($img['width'] * $ratio);
				$newHeight = round($img['height'] * $ratio);
				list($img2, $info2) = resample_image($img_res, $img, array('w' => $newWidth, 'h' => $newHeight));
				if(! is_resource($img2) && ! $img2 instanceof \GdImage) {
					@unlink($tmp_path);
					if($thumb_file) @unlink($thumb_file);
					throw new IAHUploadError("Error saving image file");
				}
				$img_save = $info2['save'];
				if(! $img_save($img2, $tmp_path)) {
					@unlink($tmp_path);
					if($thumb_file) @unlink($thumb_file);
					throw new IAHUploadError("Error saving image file");
				}
			}
		}

		$file_size = filesize($tmp_path);
		$upd = RowUpdate::blank_for_model('incoming_files');
		$upd->set(array(
			'id' => create_guid(),
			'date_entered' => $GLOBALS['timedate']->get_gmt_db_datetime(),
			'user_id' => $user_id,
			'filename' => $tmp_name,
			'upload_filename' => $real_name,
			'file_mime_type' => $this->mime_type,
			'file_size' => $file_size,
			'module_name' => array_get_default($params, 'module'),
			'model_name' => array_get_default($params, 'model'),
			'field_name' => array_get_default($params, 'field'),
			'thumbnail_filename' => $thumb_name ?? "",
			'thumbnail_position' => $thumb_pos,
			'thumbnail_size' => $thumb_size,
			'thumbnail_mime_type' => $thumb_mime,
			'is_image' => $is_image,
		));
		if(! $upd->insertRow()) {
			@unlink($tmp_path);
			if($thumb_file) @unlink($thumb_file);
			throw new IAHUploadError("Error storing record for incoming file");
		}

		// FIXME - need to do periodic cleanup of incoming files

		$this->incoming_id = $upd->getPrimaryKeyValue();
		return $this->incoming_id;
	}


	static function clean_expired_incoming() {
		global $db;
		$expiry = (int)AppConfig::setting('site.performance.upload_expiry_time');
		if(! $expiry) return;
		$lq = new ListQuery('incoming_files', true);
		$lq->addFilterClause("date_entered < DATE_SUB(NOW(), INTERVAL $expiry MINUTE)");
		$result = $lq->runQuery(0, 10, false, 'date_entered');
		if($result && ! $result->failed) {
			foreach($result->getRowIndexes() as $idx)
				self::remove_incoming($result->getRowResult($idx));
		}
	}

	static function remove_incoming(RowResult $row) {
		$fname = $row->getField('filename');
		$thumb = $row->getField('thumbnail_filename');
		$cache = CacheManager::get_location('incoming/');
		if(! empty($fname)) {
			$path = $cache . $fname;
			if(file_exists($path)) @unlink($path);
		}
		if(! empty($thumb)) {
			$path = $cache . $thumb;
			if(file_exists($path)) @unlink($path);
		}
		$upd = RowUpdate::for_result($row);
		$upd->deleteRow();
	}


	/**
	 * returns the path with file name to save an uploaded file
	 * @param string bean_id ID of the parent bean
	 * @return string
	 */
	function get_upload_path($bean_id='') {
		if(strpos($this->stored_file_name, '/') !== false)
			$bean_id = '';
		 $file_name = $bean_id.$this->stored_file_name;
		 return $this->upload_dir().$file_name;
	}

	function get_thumbnail_path($bean_id='') {
		if($this->thumbnail_file_name)
			return $this->upload_dir().$this->thumbnail_file_name;
	}

	/**
	 * deletes a file
	 * @param string bean_id ID of the parent bean
	 * @param string file_name File's name
	 */
	function unlink_file($bean_id,$file_name) {
		if(strpos($file_name, '/') !== false)
			$bean_id = '';

        if (is_file($this->upload_dir().$bean_id.$file_name)) {
            return unlink($this->upload_dir().$bean_id.$file_name);
        } else {
            return false;
        }
    }
}
