<?php
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/
/*********************************************************************************

 * Description:  Includes generic helper functions used throughout the application.
 * Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
 * All Rights Reserved.
 * Contributor(s): ______________________________________..
 ********************************************************************************/


require_once('include/utils/basic_utils.php');
require_once('include/utils/longreach_utils.php');
require_once('include/utils/external_cache.php');
require_once('include/utils/security_utils.php');
require_once('include/utils/user_utils.php');
@include_once('custom/include/custom_utils.php');


//longreach - modified - $is_group is not used
function get_assigned_user_name($assigned_user_id, $is_group = '' /*' AND is_group=0 '*/) {
	static $saved_user_list = null;
	$blankvalue = '';

	if(empty($saved_user_list)) {
		$saved_user_list = get_user_array(false, '', '', false, null, $is_group);
	}
	if(! $assigned_user_id)
		return '';
	else if($assigned_user_id == '-99')
		return translate('LBL_SYSTEM_USER');
	else if(isset($saved_user_list[$assigned_user_id])) {
		return $saved_user_list[$assigned_user_id];
	}
	// Get access to db
	global $db;
	// Not in the saved user list - include portal users
	$strQuery = "
		SELECT
			CONCAT_WS(' ', first_name, last_name) as name
		FROM
			users
		WHERE
			id = '{$assigned_user_id}'
	";
	// Execute
	if (($objResult = $db->query($strQuery)) && ($arRow = $db->fetchByAssoc($objResult))) {
		// Got a match
		return $arRow['name'];
	}

	return $blankvalue;
}


function get_user_info($id, $reload=false) {
	static $info = array();
	global $db, $current_user;
	if($id && (! array_key_exists($id, $info) || $reload)) {
		if(isset($current_user) && $current_user->id == $id)
			$info[$id] = $current_user->fetched_row;
		else
			$info[$id] = AppConfig::db_row('User', $id);
	}
	return array_get_default($info, $id);
}


/**
 * retrieves the user_name column value (login)
 * @param string id GUID of user
 * @return string
 */
function get_user_name($id, $reload=false) {
	$info = get_user_info($id, $reload);
	return $info ? $info['user_name'] : '';
}

/**
 * retrieves the first_name and last_name column value (login)
 * @param string id GUID of user
 * @return string
 */
function get_user_full_name($id, $reload=false) {
	$info = get_user_info($id, $reload);
	return $info ? trim($info['first_name'] . ' ' . $info['last_name']) : '';
}


/* ajrw - remove this section once get_user_array is updated */
if(empty($register))$register = array();

function set_register_value($category, $name, $value){
	global $register;
	if(empty($register[$category]))
		$register[$category] = array();
	$register[$category][$name] = $value;
}

function get_register_value($category,$name){
	global $register;
	if(empty($register[$category]) || empty($register[$category][$name])){
		return false;
	}
	return $register[$category][$name];
}

function get_register_values($category){
	global $register;
	if(empty($register[$category])){
		return false;
	}
	return $register[$category];
}

function clear_register($category, $name){
	global $register;
	if(empty($name)){
		unset($register[$category]);
	}else{
		if(!empty($register[$category]))
			unset($register[$category][$name]);
	}
}
/* end section */


//TODO Update to use global cache
//longreach - modified - $is_group is not used
function get_user_array($add_blank=true, $status="Active", $assigned_user="", $use_real_name=false, $user_name_begins = null, $is_group='' /* ' AND is_group=0 '*/ ) {
	global $locale, $db;

	if(empty($locale)) {
		require_once('include/Localization/Localization.php');
		$locale = new Localization();
	}
	if(is_array($status))
		$stat_key = implode(',', $status);
	else
		$stat_key = $status;

	$user_array = get_register_value('user_array', $add_blank. $stat_key . $assigned_user);

	if(!$user_array) {
		$temp_result = Array();
		// Including deleted users for now.
		if (empty($status)) {
			$query = "SELECT id, first_name, last_name, user_name from users WHERE 1=1";
		}
		else if(is_array($status)) {
			$status_q = implode("','", array_map(array(&$db, 'quote'), $status));
			$query = "SELECT id, first_name, last_name, user_name from users WHERE status IN ('$status_q')";
		}
		else {
			$status = $db->quote($status);
			$query = "SELECT id, first_name, last_name, user_name from users WHERE status='$status'";
		}

		if (!empty($user_name_begins)) {
			$user_name_begins = $db->quote($user_name_begins);
			$query .= " AND user_name LIKE '$user_name_begins%' ";
		}
		// longreach - start added
		$query .= " AND (!portal_only OR portal_only = 'off') ";
		// longreach - end added
		if (!empty($assigned_user)) {
			$assigned_user = $db->quote($assigned_user);
			$query .= " OR id='$assigned_user'";
		}
		$query = $query.' AND NOT deleted ORDER BY user_name ASC';

		$GLOBALS['log']->debug("get_user_array query: $query");
		$result = $db->query($query, true, "Error filling in user array: ");

		if ($add_blank==true) {
			// Add in a blank row
			$temp_result[''] = '';
		}

		// Get the id and the name.
		while($row = $db->fetchByAssoc($result, -1, false)) {
			if($use_real_name == true || showFullName()) {
				if(isset($row['last_name'])) { // cn: we will ALWAYS have both first_name and last_name (empty value if blank in db)
					$temp_result[$row['id']] = $locale->getLocaleFormattedName($row['first_name'],$row['last_name']);
				} else {
					$temp_result[$row['id']] = $row['user_name'];
				}
			} else {
				$temp_result[$row['id']] = $row['user_name'];
			}
		}

		$user_array = $temp_result;
		set_register_value('user_array', $add_blank. $stat_key . $assigned_user, $temp_result);
	}


	return $user_array;
}


/**
 *
 * based on user pref then system pref
 */
function showFullName() {
	global $locale;
	return $locale->getUseRealNames();
}


/** This function retrieves an application language file and returns the array of strings included in the $app_list_strings var.
 * Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
 * All Rights Reserved.
 * Contributor(s): ______________________________________..
 * If you are using the current language, do not call this function unless you are loading it for the first time */
function return_app_list_strings_language($language, $merge = false) {
	if(empty($language))
		$language = 'current';

	return AppConfig::setting("lang.lists.$language.app");
}


/** This function retrieves an application language file and returns the array of strings included.
 * Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
 * All Rights Reserved.
 * Contributor(s): ______________________________________..
 * If you are using the current language, do not call this function unless you are loading it for the first time */
function return_application_language($language) {
	if(empty($language))
		$language = 'current';

	return AppConfig::setting("lang.strings.$language.app");
}

/** This function retrieves a module's language file and returns the array of strings included.
 * Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
 * All Rights Reserved.
 * Contributor(s): ______________________________________..
 * If you are in the current module, do not call this function unless you are loading it for the first time */
function return_module_language($language, $module) {
    // Jenny - Bug 8119: Need to check if $module is not empty
    if (empty($module)) {
        $stack  = debug_backtrace();
        $GLOBALS['log']->warn("Variable module is not in return_module_language ". var_export($stack, true));
        return array();
    }

	if(empty($language))
		$language = 'current';

	$ret = AppConfig::setting("lang.strings.$language.$module");
	while( ($inh = AppConfig::setting("lang.index.$language.inherits.$module")) ) {
		$s2 = AppConfig::setting("lang.strings.$language.$inh");
		$ret = array_replace($s2, $ret);
		$module = $inh;
	}
	return $ret;
}

function return_dashlet_language($language, $module, $dashlet) {
    if (empty($dashlet)) {
        $stack  = debug_backtrace();
        $GLOBALS['log']->warn("Variable dashlet is not in return_dashlet_language ". var_export($stack, true));
        return array();
    }

    if(empty($language))
        $language = 'current';

    $ret = AppConfig::setting("lang.dashlet_strings.$language.$module.$dashlet");

    return $ret;
}


/** This function retrieves an application language file and returns the array of strings included in the $mod_list_strings var.
 * Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
 * All Rights Reserved.
 * Contributor(s): ______________________________________..
 * If you are using the current language, do not call this function unless you are loading it for the first time */
function return_mod_list_strings_language($language, $module='') {
	global $currentModule;

	if(empty($language))
		$language = 'current';
	if(! $module)
		$module = $currentModule;

	return AppConfig::setting("lang.lists.$language.$module");
}


/**
 * A temporary method of generating GUIDs of the correct format for our DB.
 * @return String contianing a GUID in the format: aaaaaaaa-bbbb-cccc-dddd-eeeeeeeeeeee
 *
 * Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
 * All Rights Reserved.
 * Contributor(s): ______________________________________..
*/
function create_guid()
{
    $microTime = microtime();
	list($a_dec, $a_sec) = explode(" ", $microTime);

	$dec_hex = sprintf("%x", $a_dec* 1000000);
	$sec_hex = sprintf("%x", $a_sec);

	ensure_length($dec_hex, 5);
	ensure_length($sec_hex, 6);

	$guid = "";
	$guid .= $dec_hex;
	$guid .= create_guid_section(3);
	$guid .= '-';
	$guid .= create_guid_section(4);
	$guid .= '-';
	$guid .= create_guid_section(4);
	$guid .= '-';
	$guid .= create_guid_section(4);
	$guid .= '-';
	$guid .= $sec_hex;
	$guid .= create_guid_section(6);

	return $guid;

}

function create_guid_section($characters)
{
	$return = "";
	for($i=0; $i<$characters; $i++)
	{
		$return .= sprintf("%x", mt_rand(0,15));
	}
	return $return;
}

function ensure_length(&$string, $length)
{
	$strlen = strlen($string);
	if($strlen < $length)
	{
		$string = str_pad($string,$length,"0");
	}
	else if($strlen > $length)
	{
		$string = substr($string, 0, $length);
	}
}

/**
 * Check if user id belongs to a system admin.
 * Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
 * All Rights Reserved.
 * Contributor(s): ______________________________________..
 */
function is_admin($user=null) {
	global $current_user;
	static $checked = array();
	if(is_object($user))
		return $user->is_admin;
	if(! isset($user) || ($user && $user == AppConfig::current_user_id())) {
		if(empty($current_user)) return false;
		return $current_user->is_admin;
	}
	if(! isset($checked[$user])) {
		$row = AppConfig::db_row('User', $user);
		$checked[$user] = isset($row) ? array_get_default($row, 'is_admin', false) : false;
	}
	return $checked[$user];
}


/**
 * Return an array of directory names.
 * Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
 * All Rights Reserved.
 * Contributor(s): ______________________________________..
 */
function get_themes() {
	return AppConfig::get_themes();
}

/**
 * THIS FUNCTION IS DEPRECATED AND SHOULD NOT BE USED; USE get_select_options_with_id()
 * Create HTML to display select options in a dropdown list.  To be used inside
 * of a select statement in a form.
 * param $option_list - the array of strings to that contains the option list
 * param $selected - the string which contains the default value
 * Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
 * All Rights Reserved.
 * Contributor(s): ______________________________________..
 */
function get_select_options ($option_list, $selected) {
	return get_select_options_with_id($option_list, $selected);
}

/**
 * Create HTML to display select options in a dropdown list.  To be used inside
 * of a select statement in a form.   This method expects the option list to have keys and values.  The keys are the ids.  The values are the display strings.
 * param $option_list - the array of strings to that contains the option list
 * param $selected - the string which contains the default value
 * Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
 * All Rights Reserved.
 * Contributor(s): ______________________________________..
 */
function get_select_options_with_id ($option_list, $selected_key) {
	return get_select_options_with_id_separate_key($option_list, $option_list, $selected_key);
}


/**
 * Create HTML to display select options in a dropdown list.  To be used inside
 * of a select statement in a form.   This method expects the option list to have keys and values.  The keys are the ids.  The values are the display strings.
 * param $label_list - the array of strings to that contains the option list
 * param $key_list - the array of strings to that contains the values list
 * param $selected - the string which contains the default value
 * Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
 * All Rights Reserved.
 * Contributor(s): ______________________________________..
 */
function get_select_options_with_id_separate_key ($label_list, $key_list, $selected_key) {
	global $app_strings;
	$select_options = "";

	//for setting null selection values to human readable --None--
	$pattern = "/'0?'></";
	$lbl_none = isset($app_strings) ? $app_strings['LBL_NONE'] ?? '' : '';
	$replacement = "''>". $lbl_none ."<";

	if (empty($key_list)) $key_list = array();
	//create the type dropdown domain and set the selected value if $opp value already exists
	foreach ($key_list as $option_key=>$option_value) {
		if(is_array($option_value)) {
			$select_options .= "\n<optgroup label=\"$option_key\">";
			$select_options .= get_select_options_with_id_separate_key($option_value, $option_value, $selected_key);
			$select_options .= "\n</optgroup>";
			continue;
		}

		$selected_string = '';
		// the system is evaluating $selected_key == 0 || '' to true.  Be very careful when changing this.  Test all cases.
		// The bug was only happening with one of the users in the drop down.  It was being replaced by none.
		if ((is_array($selected_key) && in_array($option_key, $selected_key)) || (! is_array($selected_key) && (($option_key != '' && $selected_key == $option_key) || ((string)$selected_key === (string)$option_key))))
		{
			$selected_string = 'selected="selected" ';
		}

		$html_value = $option_key;

		$select_options .= "\n<option ".$selected_string."value='$html_value'>$label_list[$option_key]</option>";
	}
	$select_options = preg_replace($pattern, $replacement, $select_options);
	return $select_options;
}


/**
 * Call this method instead of die().
 * Then we call the die method with the error message that is passed in.
 */
function sugar_die($error_message)
{
	global $sugar_die_exception;
	if(! empty($sugar_die_exception))
		throw new $sugar_die_exception($error_message);
	sugar_cleanup(true, $error_message);
}


function translate($string, $mod='', $selectedValue='', $returnOriginal = false){
	if(! $string)
		return $string;
	if(! $mod)
		$mod = isset($GLOBALS['currentModule']) ? $GLOBALS['currentModule'] : 'app';
	$returnValue = AppConfig::setting("lang.strings.current.$mod.$string");

	if(! isset($returnValue) && $mod != 'app')
		$returnValue = AppConfig::setting("lang.lists.current.app.$string");

	if(is_array($returnValue)) {
		if(!empty($selectedValue) && isset($returnValue[$selectedValue])) {
			return $returnValue[$selectedValue];
		} elseif ($returnOriginal) {
			return $selectedValue;
		}
	}

	if(! isset($returnValue)){
		return $string;
	}

	return $returnValue;
}

function translate_dd($dropdown, $value, $default=null, $lang=null, $mod=null) {
	if(! $lang) $lang = 'current';
	if(! $mod) $mod = 'app';
	if(is_array($dropdown))
		$list = $dropdown;
	else
		$list = AppConfig::setting("lang.lists.$lang.$mod.$dropdown", array());
	if(! isset($default)) $default = $value;
	return array_get_default($list, $value, $default);
}


// Designed to take a string passed in the URL as a parameter and clean all "bad" data from it
// The second argument is a string, "filter," which corresponds to a regular expression
function clean_string($str, $filter = "STANDARD") {
	static $filters = Array(
		"STANDARD"        => "#[^A-Z0-9\-_\.\@]#i",
		"STANDARDSPACE"   => "#[^A-Z0-9\-_\.\@\ ]#i",
		"FILE"            => "#[^A-Z0-9\-_\.]#i",
		"NUMBER"          => "#[^0-9\-]#i",
		"SQL_COLUMN_LIST" => "#[^A-Z0-9,_\.]#i",
		"PATH_NO_URL"     => "#://#i",
		"SAFED_GET"		  => "#[^A-Z0-9\@\=\&\?\.\/\-_~\+]#i", /* range of allowed characters in a GET string */
		"UNIFIED_SEARCH"	=> "#[\\x00]#", /* cn: bug 3356 & 9236 - MBCS search strings */
		"AUTO_INCREMENT"	=> "#[^0-9\-,\ ]#i",
		"ALPHANUM"        => "#[^A-Z0-9\-]#i",
	);

	if (preg_match($filters[$filter], $str)) {
		if (isset($GLOBALS['log']) && is_object($GLOBALS['log'])) {
			$GLOBALS['log']->fatal("SECURITY: bad data passed in; string: {$str}");
		}
		die("Bad data passed in; <a href=\"".AppConfig::site_url()."\">Return to Home</a>");
	}
	else {
		return $str;
	}
}

function clean_special_arguments() {
	if(isset($_SERVER['PHP_SELF'])) {
		if (!empty($_SERVER['PHP_SELF'])) clean_string($_SERVER['PHP_SELF'], 'SAFED_GET');
	}
    clean_superglobals('return_action');
    clean_superglobals('return_module');
	return true;
}

/**
 * cleans the given key in superglobals $_GET, $_POST, $_REQUEST
 */
function clean_superglobals($key, $filter = 'STANDARD') {
    if(isset($_GET[$key])) clean_string($_GET[$key], $filter);
    if(isset($_POST[$key])) clean_string($_POST[$key], $filter);
    if(isset($_REQUEST[$key])) clean_string($_REQUEST[$key], $filter);
}

// Works in conjunction with clean_string() to defeat SQL injection, file inclusion attacks, and XSS
function clean_incoming_data() {
	// longreach - start added
	$magic_q = function_exists('get_magic_quotes_gpc') && get_magic_quotes_gpc();
	if (isset($_REQUEST['action']) && $_REQUEST['action'] == 'Save' && $_REQUEST['module'] =='Emails') {
		$description_html = $_POST['description_html'];
		if ($magic_q) {
			$description_html = stripslashes($description_html);
		}
		require_once('include/utils/html_utils.php');
		$description_html = fix_fck_html($description_html);
	}
	// longreach - end added

	secure_xss($_REQUEST, $magic_q);
	secure_xss($_POST, $magic_q);
	secure_xss($_GET, $magic_q);


	// longreach - start added
	if (isset($_REQUEST['action']) && $_REQUEST['action'] == 'Save' && $_REQUEST['module'] =='Emails') {
		$_POST['description_html'] = $_REQUEST['description_html'] = $description_html;
	}
	// longreach - end added


	// Any additional variables that need to be cleaned should be added here
	if (isset($_REQUEST['action'])) clean_string($_REQUEST['action']);
	if (isset($_REQUEST['module'])) clean_string($_REQUEST['module']);
	if (isset($_REQUEST['record'])) clean_string($_REQUEST['record'], 'STANDARDSPACE');
	if (isset($_SESSION['authenticated_user_theme'])) clean_string($_SESSION['authenticated_user_theme']);
	if (isset($_SESSION['authenticated_user_language'])) clean_string($_SESSION['authenticated_user_language']);
	if (isset($_REQUEST['language'])) clean_string($_REQUEST['language']);
	if (isset($_REQUEST['offset'])) clean_string($_REQUEST['offset']);
	if (isset($_REQUEST['stamp'])) clean_string($_REQUEST['stamp']);

	// Clean "offset" and "order_by" parameters in URL
	foreach ($_GET as $key => $val) {
		if (str_end($key, "_offset")) {
			clean_string($_GET[$key], "ALPHANUM"); // keep this ALPHANUM for disable_count_query
		}
		elseif (str_end($key, "_ORDER_BY")) {
			clean_string($_GET[$key], "SQL_COLUMN_LIST");
		}
	}

	return 0;
}

// Returns TRUE if $str begins with $begin
function str_begin($str, $begin) {
	return (substr($str, 0, strlen($begin)) == $begin);
}

// Returns TRUE if $str ends with $end
function str_end($str, $end) {
	return (substr($str, strlen($str) - strlen($end)) == $end);
}

function securexss($value) {
	$xss_cleanup=  array('"' =>'&quot;', "'" =>  '&#039;' , '<' =>'&lt;' , '>'=>'&gt;');
	// longreach - added
	if(is_array($value)) return $value;
	$value = preg_replace('/javascript:/i', 'java script:', $value);
	return str_replace(array_keys($xss_cleanup), array_values($xss_cleanup), $value);
}

function secure_xss(&$target, $strip_slashes=false) {
	if(! is_array($target))
		return;
	foreach($target as $k => $v) {
		if (is_array($v)) {
			secure_xss($v, $strip_slashes);
		} else {
			if($strip_slashes)
				$v = stripslashes($v);
			$v = preg_replace('/javascript:/i', 'java script:', $v);
		}
		$target[$k] = $v;
	}
}


function get_image($image,$other_attributes='',$width='',$height='',$no_div=false){
    if($width === '') $width = null;
    if($height === '') $height = null;
    $img = get_image_info($image, $no_div, $other_attributes, $width, $height);
    if($no_div && $img['found']) {
		if(isset($img['img_html']))
			return $img['img_html'];
		else if($img['src'])
			return $img['html'];
		else
			return '';
	}
	return $img['html'];
}

function number_empty($value){
	return empty($value) && $value != '0';
}

/* checks if it is a number that atleast has the plus at the beggining
 */
function skype_formatted($number){
	return substr($number, 0, 1) == '+' || substr($number, 0, 2) == '00' || substr($number, 0, 2) == '011';
}


function insert_charset_header() {
	header('Content-Type: text/html; charset='.AppConfig::charset());
}

function getCurrentURL($maintain_async=false)
{
	$href = "http:";
	if(AppConfig::detect_https())
		$href = 'https:';

	// longreach - start added - fix for 'Open in New Window', 'Plain Text'/'HTML' links in email detailview
	if($_SERVER['REQUEST_METHOD'] == 'POST') {
		global $module, $action, $record;
		$query_string = 'module='.urlencode($module).'&action='.urlencode($action);
		if(!empty($record))
			$query_string .= '&record='.urlencode($record);
	} else
		$query_string = $_SERVER['QUERY_STRING'];
	// longreach - end added

	$script = $_SERVER['SCRIPT_NAME'];
	if(! $maintain_async && substr($script, -10) == '/async.php')
		$script = substr($script, 0, -10) . '/index.php';

	$href.= "//".$_SERVER['HTTP_HOST'].$script.'?'.preg_replace('~&?asyncListView=1~', '', $query_string);
	return $href;
}

function javascript_escape($str) {
	return addcslashes($str, "/\\'\"\r\n");
}

function js_escape($str, $keep=true){
	$str = html_entity_decode(str_replace("\\", "", $str), ENT_QUOTES);

	if($keep){
		$str = javascript_escape($str);
	}
	else {
		$str = str_replace("'", " ", $str);
		$str = str_replace('"', " ", $str);
	}

	return $str;
}


function encode_query_string(array $params, $auto_prefix=false) {
	$qs = array();
	foreach($params as $k => $v) {
		if(! isset($v)) continue;
		$qs[] = urlencode($k) .'='. urlencode($v);
	}
	$ret = implode('&', $qs);
	if($ret && $auto_prefix) $ret = '?'.$ret;
	return $ret;
}


function br2nl($str) {
	$regex = "#br.+?>#";
	preg_match_all($regex, $str, $matches);

	foreach($matches[0] as $match) {
		$str = str_replace($match, "br>", $str);
	}

	$brs = array('<br>','<br/>', '<br />');
	$str = str_replace("\r\n", "\n", $str); // make from windows-returns, *nix-returns
	$str = str_replace($brs, "\n", $str); // to retrieve it

	return $str;
}


function register_cleanup_handler($fn) {
	global $_pre_cleanup_hooks;
	if(! isset($_pre_cleanup_hooks) || ! is_array($_pre_cleanup_hooks))
		$_pre_cleanup_hooks = array();
	if(! in_array($fn, $_pre_cleanup_hooks))
		$_pre_cleanup_hooks[] = $fn;
	return true;
}


function sugar_cleanup($exit = false, $msg = null) {
	AppConfig::save_local('userpref'); // save user preferences

	// longreach - added
	global $_pre_cleanup_hooks;
	if(! empty($_pre_cleanup_hooks) && is_array($_pre_cleanup_hooks)) {
		foreach($_pre_cleanup_hooks as $fn) {
			if(is_string($fn) && function_exists($fn)) $fn();
			else if(is_array($fn)) call_user_func($fn);
		}
	}

	if(isset($msg)) {
		echo $msg;
	}

	if(class_exists('DBManager')) {
		DBManager::disconnectAll();
	}
	if($exit) {
		//while(ob_get_level() && @ob_end_clean()) {}
		exit();
	}
}


function display_stack_trace(){
	$stack  = debug_backtrace();
	echo '<br>';
	$first = true;

	foreach($stack as $item){
			$file  = '';if(isset($item['file']))$file = $item['file'];
			$class  = '';if(isset($item['class']))$class = $item['class'];
			$line  = '';if(isset($item['line']))$line = $item['line'];
			$function  = '';if(isset($item['function']))$function = $item['function'];
			if(!$first){
				echo '<font color="black"><b>' . $file . '</b></font>' . '<font color="blue">[L:' . $line . ']</font>'.'<font color="red">(' . $class .':'. $function .  ')</font><br>';
			}else{
				$first = false;
			}
	}
}

function StackTraceErrorHandler($errno, $errstr, $errfile,$errline) {

	// longreach - added - don't report intentionally suppressed errors
	$e = error_reporting();
	if($e == 0)  return;

	$halt_script = true;
	switch($errno){
			case E_STRICT:
				if(! ($e & E_STRICT))
					return;
				$halt_script = false;
				$type = 'Strict';
				break;
			case E_USER_NOTICE:
			case E_NOTICE:
				if(! ($e & E_NOTICE))
					return;
				$halt_script = false;
				$type = 'Notice';
				break;
			case E_USER_WARNING:
   			case E_COMPILE_WARNING:
   			case E_CORE_WARNING:
  		 	case E_WARNING:

		      $halt_script = false;
		       $type = "Warning";
		       break;

		    case E_RECOVERABLE_ERROR:
		    	$halt_script = false;
		    	$type = "Recoverable Error";
		    	break;

		   	case E_USER_ERROR:
		   	case E_COMPILE_ERROR:
		   	case E_CORE_ERROR:
		  	case E_ERROR:

		       $type = "Fatal Error";
		       break;

		   	case E_PARSE:

		       $type = "Parse Error";
		       break;

		   default:
		   //don't know what it is might not be so bad
		      	$halt_script = false;
		        $type = "Unknown Error ($errno)";
		       break;
	}
	@$error_disp = "<b>$type</b>: $errstr occured in <b>$errfile</b> on line $errline [" . date("Y-m-d H:i:s") . ']';
	echo $error_disp;
	display_stack_trace();
	if(ini_get('log_errors')) {
		@$log_msg = "PHP $type:  $errstr in $errfile on line $errline";
		error_log($log_msg);
	}
	if($halt_script){
		exit -1;
	}
}


/**
 * For translating the php.ini memory values into bytes.  e.g. input value of '8M' will return 8388608.
 */
function return_bytes($val)
{
	if (is_numeric($val)) return $val;
	$val = trim($val);
	$last = strtolower(substr($val, -1));
	$numeric = substr($val, 0, -1);
	if (!is_numeric($numeric)) return 0;
	switch($last)
	{
		case 'g':
			$numeric *= 1024;
		case 'm':
			$numeric *= 1024;
		case 'k':
			$numeric *= 1024;
	}
	return $numeric;
}
/**
 * Adds the href HTML tags around any URL in the $string
 */
function url2html($string) {
	//
	// longreach - modified
	$return_string = preg_replace_callback(
		'/(\w+:\/\/|www\d*\.)([^\s<>&]+(&(?!(lt|gt)+;)[^\s<>&]*)*)(?<![\.,\?\!\]])/i',
		function($matches) {
			return _url2html_link($matches[1].$matches[2]);
		},
		$string);
	return $return_string;
}
function _url2html_link($link, $escaped=true) {
	if(! $escaped) $link = to_html($link);
	$display = $link;
	if(! preg_match('~^\w+:/~', $link)) $link = 'http://'.$link;
	return "<a href=\"$link\" target=\"_new\" class=\"tabDetailViewDFLink\" style=\"font-weight: normal;\">$display</a>";
}

/**
 * tries to determine whether the Host machine is a Windows machine
 */
function is_windows() {
	return ! strncasecmp(PHP_OS, 'WIN', 3);
}

function is_os_writable($file)
{
	return is_windows() ? is_writable_windows($file) : is_writable($file);
}


/**
 * equivalent for windows filesystem for PHP's is_writable()
 * @param string file Full path to the file/dir
 * @return bool true if writable
 */
function is_writable_windows($file) {
	if($file[strlen($file)-1] =='/') {
		return is_writable_windows($file.uniqid(mt_rand()).'.tmp');
	}

	// the assumption here is that Windows has an inherited permissions scheme
	// any file that is a descendant of an unwritable directory will inherit
	// that property and will trigger a failure below.
	if(is_dir($file)) {
		return true;
	}

	$file = str_replace("/", '\\', $file);

	if(file_exists($file)) {
	   if (!($f = @fopen($file, 'r+')))
	       return false;
	   fclose($f);
	   return true;
	}

	if(!($f = @fopen($file, 'w')))
	   return false;
	fclose($f);
	unlink($file);
	return true;
}


function checkAuthUserStatus(){
	authUserStatus();
}
function checkLoginUserStatus(){
	getLoginUserStatus();
}


/**
 * This function will take a number and system_id and format
 * @param	$url URL containing host to append port
 * @param	$port the port number - if '' is passed, no change to url
 * @return	$resulturl the new URL with the port appended to the host
 */
function appendPortToHost($url, $port)
{
	$resulturl = $url;

	// if no port, don't change the url
	if($port != '')
	{
		$split = explode("/", $url);
		//check if it starts with http, in case they didn't include that in url
		if(str_begin($url, 'http'))
		{
			//third index ($split[2]) will be the host
			$split[2] .= ":".$port;
		}
		else // otherwise assumed to start with host name
		{
			//first index ($split[0]) will be the host
			$split[0] .= ":".$port;
		}

		$resulturl = implode("/", $split);
	}

	return $resulturl;
}


function str_limit_length($value, $len=30, $dots=true, $unicode=true, $word=5) {
	$strlen = mb_strlen($value);
	if($strlen > $len) {
		$chop_len = $len - ($dots ? 1 : 0);
		if(preg_match('~\s~', mb_substr($value, $chop_len-1, 2)))
			// if there's a space right before or after the break, safe to just trim
			$chopped = rtrim(mb_substr($value, 0, $chop_len));
		else {
			$chopped = mb_substr($value, 0, $chop_len);
			if($word && preg_match("~^(.+)\s\S{1,$word}\$~u", $chopped, $m)) {
				$chopped = $m[1];
			}
		}
		if($dots) $chopped .= $unicode ? '…' : '..';
	} else {
		$chopped = $value;
	}
	return $chopped;
}


function getTrackerSubstring($name, $len=20, $dots=false) {
	return str_limit_length($name, $len, $dots);
}

function check_site_flags() {
	$args = func_get_args();
	if(count($args) == 1 && is_array($args[0]))
		$args = $args[0];
	$match = array();
	foreach($args as $t) {
		if($t == 'demo' && is_demo_site())
			$match[] = $t;
		else if($t == 'readonly' && AppConfig::read_only())
			$match[] = $t;
		else if($t == 'all')
			$match[] = $t;
	}
	return $match;
}

function action_restricted_for() {
	$args = func_get_args();
	if(count($args) == 1) $args = $args[0];
	$check = check_site_flags($args);
	if(! $check)
		return false;
	if($check[0] == 'demo')
		$fail = 'LBL_DISALLOWED_DEMO_SITE';
	else if($check[0] == 'readonly')
		$fail = 'LBL_DISALLOWED_READ_ONLY';
	else
		$fail = 'LBL_DISALLOWED';
	throw new IAHPermissionError($fail);
}

function action_restrict_params($test, $params) {
	$check = check_site_flags($test);
	if(! $check)
		return false;
	foreach($params as $p) {
		if(! empty($_REQUEST[$p]))
			action_restricted_for($test);
	}
	return $check;
}

function action_filter_params($test, $params) {
	$cut = false;
	$check = check_site_flags($test);
	if(! $check)
		return false;
	foreach($params as $p => $v) {
		if(isset($_REQUEST[$p])) {
			if($v === null) {
				unset($_REQUEST[$p]);
				if(isset($_GET[$p])) unset($_GET[$p]);
				if(isset($_POST[$p])) unset($_POST[$p]);
				if(isset($_COOKIE[$p])) unset($_COOKIE[$p]);
			} else {
				$_REQUEST[$p] = $v;
				if(isset($_GET[$p])) $_GET[$p] = $v;
				if(isset($_POST[$p])) $_POST[$p] = $v;
				if(isset($_COOKIE[$p])) $_COOKIE[$p] = $v;
			}
		}
	}
	return $check;
}

function theme_hide_side_menu($hide=true, $complete=false) {
	global $themeInstance;
	if(isset($themeInstance)) {
		$themeInstance->hide_side_menu($hide, $complete);
	}
	$GLOBALS['theme_hide_side_menu'] = $hide;
	if($complete)
		$GLOBALS['theme_hide_side_complete'] = $hide;
}

function ical_escape_value($val, $from_html=true) {
	if($from_html) $val = from_html($val);
	return addcslashes(str_replace("\r", '', $val), "\":;,\n");
}

function parse_accept_language() {
	$lang = explode(',', array_get_default($_SERVER, 'HTTP_ACCEPT_LANGUAGE', ''));
	$qs = array();
	$ret = array();
	foreach($lang as $l) {
		$l = explode(';', trim($l));
		$q = 1.0;
		$name = array_shift($l);
		foreach($l as $lpart) {
			if(substr($lpart, 0, 2) == 'q=') {
				$q = substr($lpart, 2);
				break;
			}
		}
		$name = strtolower(str_replace('-', '_', $name));
		$ret[] = compact('name', 'q');
		$qs[] = $q;
	}
	array_multisort($qs, SORT_DESC, $ret);
	return $ret;
}


class IAHPermissionError extends IAHError {
	var $redirect_home = true;
	public function __construct($msg='') {
		if(is_array($msg)) $msg = translate($msg[0], $msg[1]);
		else $msg = translate($msg, 'ACL');
		parent::__construct($msg);
	}
	public static function &withParams($msg, $p) {
		$e = new self($msg);
		$e->setParams($p);
		return $e;
	}
}


if(AppConfig::setting('site.log.stack_trace_errors'))
	set_error_handler('StackTraceErrorHandler');


// for debugging only
function _ppd($mixed) {
	throw new IAHError('function removed: '.__FUNCTION__);
}
function _ppl($mixed, $die=false, $displayStackTrace=false, $loglevel="fatal") {
	throw new IAHError('function removed: '.__FUNCTION__);
}
function _ppf($bean, $die=false) {
	throw new IAHError('function removed: '.__FUNCTION__);
}
function _pp($mixed) {
	throw new IAHError('function removed: '.__FUNCTION__);
}
function _ppt($mixed){
	throw new IAHError('function removed: '.__FUNCTION__);
}
function _pptd($mixed){
	throw new IAHError('function removed: '.__FUNCTION__);
}

function create_csrf_input() {
	$cookie_value = $_SESSION['csrf_cookie_value'] ?? null;
	$input_name = $_SESSION['csrf_input_name'] ?? null;
	if (!empty($cookie_value) && !empty($input_name)) {
		return sprintf('<input type="hidden" name="%s" value="%s">', $input_name, $cookie_value);
	}
	return '';
}

