<?php
/**
 * Class for CVS management.
 *
 * @example
 * <code>
 * <?php
 * $manager = new CSVManager($filePath);			
 * $manager->export( $data);
 * ?>
 * </code>
 * @access public
 * @package CP
 */
class CSVManager {

	/**
	 * CSV delimiter character.
	 *
	 * @var string
	 */
	private $delimiter;
	
	/**
	 * CSV quote character.
	 *
	 * @var string
	 */
	private $quote;
	
	/**
	 * Path to CSV file
	 *
	 * @var string
	 */
	private $filePath;
		
	/**
	 * Constructor.
	 *
	 * @param string $filePath - path to CSV file
	 * @param string $delimiter - [optional][default=','] CSV delimiter character.
	 * @param string $quote - [optional][default='"'] CSV quote character.
	 * @return CSVManager
	 */
	public function __construct($filePath, $delimiter = ',', $quote='"'){
		$this->delimiter = $delimiter;
		$this->quote = $quote;
		$this->filePath = $filePath;	
	}
	
	/**
	 * Export data to file in CSV format
	 *
	 * @param array $data - information.
	 * @param string $head - head string for CSV
	 */
	public function export( $data, $head = ""){
		$fhandle = fopen( $this->filePath, 'w');
		if ($head != "") {
			$head .= "\n";
			fputs( $fhandle, $head);					
		}
		$dataCount = is_countable($data) ? count($data) : 0;
		for ($i = 0; $i < $dataCount; $i++){
			$this->fputcsv($fhandle, $data[ $i ], $this->delimiter, $this->quote);
		}
        fclose($fhandle);
	}

	/**
	 * Add data to file in CSV format
	 *
	 * @param array $data - information.
	 */
	public function addDataToFile( $data) {
		$fhandle = fopen( $this->filePath, 'a');
		$dataCount = is_countable($data) ? count($data) : 0;
		for ($i = 0; $i < $dataCount; $i++){
			$this->fputcsv($fhandle, $data[$i], $this->delimiter, $this->quote);
		}
	}
	
	/**
	 * Method for CSV put (for PHP4, 'cause in PHP5 there is 'fputcsv')
	 *
	 * @param fileHandle $handle - file handle
	 * @param array $row - array with data
	 * @param string $fd - [optional][default=','] CSV delimiter character.
	 * @param string $quote - [optional][default='"'] CSV quote character.
	 * @return int - file counter after CVS information added. 
	 **/
	private function fputcsv( $handle, $row, $fd=',', $quote='"') {
		if (function_exists('fputcsv')) {
			//return fputcsv($handle, split($fd, $row),$fd,$quot);
			return fputcsv($handle, $row, $fd, $quote);
		}
		$str = '';
		foreach ($row as $cell) {
			str_replace(Array( $quote, "\n" ), Array($quote.$quote,  ''), $cell);
			$str .= $quote . $cell . $quote . $fd;
		}
		fputs( $handle, substr( $str, 0, -1 ) . "\n");
		return $str - 1;
	}
}
?>
