<?php
require_once('include/utils/file_utils.php');

class CacheManager {

    public function __construct() {}

    /**
     * Get cache location
     *
     * @param string $dir_name - target dir name
     * @param bool $in_files_store - if true location in 'files' dir by default in 'cache' dir
     * @return string
     */
    static function get_location($dir_name, $in_files_store = false) {
        if (! $in_files_store) {
            $base_dir = AppConfig::cache_dir();
        } else {
            $base_dir = AppConfig::files_dir();
        }

        clean_path($dir_name);
        $dir_name = ltrim($dir_name, '/');
        if (substr($dir_name, -1) != '/')
            $dir_name .= '/';
        $path = $base_dir . $dir_name;

        if (! is_dir($path)) {
            $manager = new self();
            if (! $manager->createDir($path)) {
                $manager->logError($path);
                $path = $base_dir;
            }
        }

        return $path;
    }

    /**
     * Check cache folders structure and create default folders if not exist
     *
     */
    public function createFoldersStructure() {
        //check 'cache' dirs
        $this->checkAndCreateDirs($this->getDefaultCacheFolders(), AppConfig::cache_dir());
        //check 'files' dirs
        $this->checkAndCreateDirs($this->getDefaultFilesFolders(), AppConfig::files_dir());
    }

    /**
     * @param array $dirs - dirs for checking
     * @param string $base_dir - root dir
     */
    private function checkAndCreateDirs($dirs, $base_dir) {
        for ($i = 0; $i < sizeof($dirs); $i++) {
            $path = $base_dir . $dirs[$i];
            if (! is_dir($path)) {
                if (! $this->createDir($path))
                    $this->logError($path);
            }
        }
    }

    /**
     * @param string $path
     * @return bool
     */
    private function createDir($path) {
        $status = mkdir($path, 0775, true);

        if ($status) {
            $empty_index = fopen($path . '/' . 'index.html', 'w');
            fclose($empty_index);
        }

        return $status;
    }

    /**
     * @param string $path
     */
    private function logError($path) {
        if (isset($GLOBALS['log']))
            $GLOBALS['log']->error('Cache Error: '.$path.' was not created');
    }

    /**
     * Get default 'cache' folders list
     *
     * @return array
     */
    private function getDefaultCacheFolders() {
        return array(
            'csv', 'feeds', 'html_files',
            'import', 'incoming', 'javascript', 'modules',
            'pdf', 'stocks', 'system', 'temp', 'upgrades',
            'weather', 'wsdl', 'xml', 'smarty', 'smarty/cache',
            'smarty/configs', 'smarty/template_c'
        );
    }

    /**
     * Get default 'files' folders list
     *
     * @return array
     */
    private function getDefaultFilesFolders() {
        return array(
            'backups', 'email', 'images',
            'reports', 'upload'
        );
    }
}
