<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

class IAHError extends Exception {
	var $error_name = '1CRM Error';
	var $redirect_home = false;
	var $seconds = 5;
	var $icon;
	var $no_translate;

	public function __construct($msg='') {
		if(function_exists('translate') && empty($this->no_translate)) {
			if(is_array($msg)) $msg = translate($msg[0], $msg[1]);
			else $msg = translate($msg);
		}
		parent::__construct($msg);
	}
	public function setParams($p) {
		foreach($p as $k => $v)
			$this->$k = $v;
	}
	public static function &withParams($msg, $p) {
		$e = new self($msg);
		$e->setParams($p);
		return $e;
	}
	public function getErrorName() {
		return $this->error_name;
	}
}

class IAHInternalError extends IAHError {
	var $error_name = 'Internal Error';
	// not to be displayed
}

class IAHDatabaseError extends IAHInternalError {
	var $error_name = 'Database Error';
	// general database query error
}
class IAHQueryError extends IAHDatabaseError {
	var $error_name = 'Query Error';
	// general database query error
}

class IAHActionAbort extends IAHInternalError {
	// used by hooks to indicate that the default behaviour is unnecessary (see RowUpdate)
}
class IAHActionCompleted extends IAHInternalError {
	// used by hooks to indicate that the default behaviour is unnecessary (see RowUpdate)
}

class IAHRecordLoadError extends IAHQueryError {
	var $error_name = 'Record Load Error';
	// used to indicate that a specified record could not be found (DetailManager mainly)
}
class IAHLayoutLoadError extends IAHInternalError {
	var $error_name = 'Layout Load Error';
	// used to indicate that a specified form layout could not be loaded
}

class IAHUploadError extends IAHError {
	var $error_name = 'Upload Error';
	// used to indicate that an error occurred in handling an uploaded file (see UploadFile)
}



/// a simple wrapper around print_r to make it more readable in html.  Useful for debugging.
///
function pr2($caca, $title=null, $wrap=false, $hide=false, $escape=true)
{
	static $count = 0;
	echo '<div class="pr2" style="min-width: 20%; max-width: 80em; border: 1px solid orange; background-color: #ffd; margin: 10px 2px;">';
	if($hide) {
		$title .= ' <a href="#" onclick="toggleDisplay(\'pr2-body-'.$count.'\'); return false" style="text-decoration: none">[show]</a>';
	}
	if(isset($title)) {
		echo '<div class="pr2-title" style="font: 8pt Verdana; padding: 2px; font-weight: bold; color: #444; border-bottom: 1px solid orange;">'.$title.'</div>';
	}
	$hidden = $hide ? 'display: none;' : '';
	echo '<div class="pr2-body" id="pr2-body-'.($count++).'" style="overflow: auto; font-family: monospace; color: #222; white-space: pre; padding: 2px;'.$hidden.'">';
	if($caca instanceof SugarBean) {
		$caca = clone($caca);
		// reduce output
		unset($caca->db);
		unset($caca->custom_fields);
		unset($caca->field_name_map);
		unset($caca->field_defs);
	}
	if($escape || ! is_string($caca))
		$caca = htmlspecialchars(print_r($caca, true), ENT_QUOTES|ENT_IGNORE, 'UTF-8');
	if($wrap) $caca = wordwrap($caca, 100);
	echo $caca;
	echo '</div></div>';
}

function format_backtrace($trace=null, $html=false, $skip=1) {
	$ret = [];
	if(! $trace) $trace = debug_backtrace();
	$last_f = null;
	$base_path = realpath('.');
	$inc_funcs = array('include', 'require', 'include_once', 'require_once');
	$idx = 0;
	foreach($trace as $frame) {
		if($idx ++ < $skip) continue;
		foreach(array('function', 'line', 'file', 'class', 'type') as $f)
			$$f = array_get_default($frame, $f);
		if($html)
			$fn = "<span style='color: #222'>$type</span><span style='color: blue'>$function</span>";
		else
			$fn = $type.$function;
		if(in_array($function, $inc_funcs)) {
			$inc = $frame['args'][0];
			if(! strcmp(substr($inc, 0, strlen($base_path)), $base_path))
				$inc = '.' . substr($inc, strlen($base_path));
			if($html) $inc = '<span style="color: #0a0">'.to_html($inc).'</span>';
			$fn .= ' '.$inc;
		} else if(isset($frame['args'])) {
			$args = array();
			foreach($frame['args'] as &$a) {
				if(is_null($a))
					$fa = 'null';
				else if(is_bool($a))
					$fa = $a ? 'true' : 'false';
				else if(is_int($a) || is_float($a))
					$fa = $a;
				else if(is_string($a)) {
					if(strlen($a) > 36) {
						$a = substr($a, 0, 34);
						if($html) $a = to_html($a);
						$a .= '…';
					} else if($html) $a = to_html($a);
					$fa = '"' . addcslashes($a, '"') . '"';
				}
				else if(is_array($a))
					$fa = 'array['.count($a).']';
				else if(is_object($a))
					$fa = get_class($a);
				else
					$fa = gettype($a);
				if($html) $fa = "<span style='color: #0a0'>$fa</span>";
				$args[] = $fa;
			}
			$args = implode(', ', $args);
			if($html)
				$args = "<span style='color: #555'>($args)</span>";
			else
				$args = "($args)";
			$fn .= $args;
		}
		if(isset($line)) {
			if($html)
				$fn .= "<span style='color: #222'>";
			$fn .= ' [';
			if($file && $file != $last_f) {
				if(! strcmp(substr($file, 0, strlen($base_path)), $base_path))
					$file = '.' . substr($file, strlen($base_path));
				if($html)
					$fn .= '<span style="color: #555">'.to_html($file).'</span>';
				else
					$fn .= $file;
			}
			$fn .= ":$line]";
			if($html) $fn .= '</span>';
		}
		if(isset($class)) {
			if($class == 'ModelDef' && isset($frame['object'])) $class .= '('.$frame['object']->name.')';
			if($html)
				$fn = "<span style='color: #a00'>$class</span>$fn";
			else
				$fn = $class.$fn;
		}
		$ret[] = $fn;
		$last_f = $file;
	}
	return $ret;
}

// render/output a simple stack trace
function tr2($format=false, $title=null, $html=null, $hide=false) {
	if(! isset($html)) $html = (php_sapi_name() != 'cli');
	$ret = format_backtrace(null, $html);
	if($format === 2)
		return $ret;
	$ret = implode("\n", $ret);
	if($format)
		return $ret;
	if(! $title) $title = 'stack trace';
	if(! $html)
		echo "$title:\n$ret\n";
	else
		pr2($ret, $title, false, $hide, false);
}

function prq($query, $title=null, $hide=false) {
	$hl = SqlFormatter::format($query, true);
	pr2($hl, $title, true, $hide, false);
}


/**
 * Set default php.ini settings for entry points
 */
function setPhpIniSettings($enable_compress=true) {
	// check and enable compression
	if(! headers_sent()) {
		$compress = AppConfig::get_server_compression();
		$zlib_flush_err = (substr(phpversion(), 0, 4) == '5.3.');

		if($compress == 'zlib') {
			if($zlib_flush_err) {
				ini_set('zlib.output_compression', '0'); // avoid PHP bug 49816 when flush() is called
			}
			else {
				ini_set('zlib.output_compression_level', '9');
			}
		}
		else if(! $compress && $enable_compress) {
			ob_start('ob_gzhandler');
			AppConfig::$override_compression = 'ob_gzhandler';
		}
	}
	
	// mbstring module
	if(function_exists('mb_strlen')) {
		mb_internal_encoding('UTF-8');
	}
}

/**
 * Will check if a given PHP version string is supported (tested on this ver),
 * unsupported (results unknown), or invalid (something will break on this
 * ver).  Do not pass in any pararameter to default to a check against the
 * current environment's PHP version.
 *
 * @return 3 implies supported, 2 is probably okay but not tested,
 *   1 is old but probably supported, 0 is too old, -1 is specifically unsupported
 */
function check_php_version($sys_php_version=null) {
	if(empty($sys_php_version))
		$sys_php_version = phpversion();
	if(($p = strpos($sys_php_version, '-')) !== false) // strip platform identifier (present on ubuntu)
		$sys_php_version = substr($sys_php_version, 0, $p);
	
	$unsupported = AppConfig::setting('install.php_versions.unsupported', array());
	if(in_array($sys_php_version, $unsupported))
		return -1;

	$max_known_safe = AppConfig::setting('install.php_versions.max_known_safe');
	if(version_compare($sys_php_version, $max_known_safe, '>'))
		return 2;
	
	$min_known_safe = AppConfig::setting('install.php_versions.min_known_safe');
	if(version_compare($sys_php_version, $min_known_safe, '>='))
		return 3;
	
	// deprecated versions
	$old_versions = AppConfig::setting('install.php_versions.old_versions', array());
	if(in_array($sys_php_version, $old_versions))
		return 1;
	
	return 0;
}


/**
 * Singleton to return JSON object
 * @return	JSON object
 */
function getJSONobj($reload=false) {
	static $json;
	if(isset($json) && ! $reload) return $json;
	require_once('include/JSON.php');
	$json = new JSON(JSON_LOOSE_TYPE);
	return $json;
}

/**
 * Fetch a given $key from the array, returning $default if is missing or set to null.
 */
function array_get_default($arr, $key, $default=null) {
	if(! isset($arr[$key])) return $default;
	return $arr[$key];
}

if(! function_exists('array_replace')) {
	/* New in PHP 5.3 - see standard documentation */
	function array_replace(array $gimp, array $dom /*, array ...*/) {
		foreach($dom as $k => &$v) {
			$gimp[$k] = $v;
		}
		$na = func_num_args();
		for($i = 2; $i < $na; $i++)
			$gimp = array_replace($gimp, func_get_arg($i));
		return $gimp;
	}
}

if(! function_exists('array_replace_recursive')) {
	/* New in PHP 5.3 - see standard documentation */
	function array_replace_recursive(array $gimp, array $dom /*, array ...*/) {
		foreach($dom as $k => &$v) {
			if(isset($gimp[$k]) && is_array($gimp[$k]) && is_array($v))
				$gimp[$k] = array_replace_recursive($gimp[$k], $v);
			else
				$gimp[$k] = $v;
		}
		$na = func_num_args();
		for($i = 2; $i < $na; $i++)
			$gimp = array_replace_recursive($gimp, func_get_arg($i));
		return $gimp;
	}
}

/** For compatibility - use array_replace_recursive in future */
function sugarArrayMerge($gimp, $dom) {
	return array_replace_recursive($gimp, $dom);
}

/** In-place version of array_merge.
 *  Also like array_update, but appends values with numeric keys.
 */
if(! function_exists('array_extend')) {
	function &array_extend(array &$gimp, array $dom /*, array ... */) {
		foreach($dom as $k => &$v) {
			if(is_int($k))
				$gimp[] = $v;
			else
				$gimp[$k] = $v;
		}
		$na = func_num_args();
		for($i = 2; $i < $na; $i++)
			array_extend($gimp, func_get_arg($i));
		return $gimp;
	}
}

if(! function_exists('array_extend_recursive')) {
	function &array_extend_recursive(array &$gimp, array $dom /*, array ... */) {
		foreach($dom as $k => &$v) {
			if(is_int($k))
				$gimp[] = $v;
			else if(isset($gimp[$k]) && is_array($gimp[$k]) && is_array($v))
				array_extend_recursive($gimp[$k], $v);
			else
				$gimp[$k] = $v;
		}
		$na = func_num_args();
		for($i = 2; $i < $na; $i++)
			array_extend_recursive($gimp, func_get_arg($i));
		return $gimp;
	}
}

/** In-place version of array_replace.
 */
if(! function_exists('array_update')) {
	function &array_update(array &$gimp, array $dom /*, array ... */) {
		foreach($dom as $k => &$v) {
			$gimp[$k] = $v;
		}
		$na = func_num_args();
		for($i = 2; $i < $na; $i++)
			array_update($gimp, func_get_arg($i));
		return $gimp;
	}
}

if(! function_exists('array_update_recursive')) {
	function &array_update_recursive(array &$gimp, array $dom /*, array ... */) {
		foreach($dom as $k => &$v) {
			if(isset($gimp[$k]) && is_array($gimp[$k]) && is_array($v))
				array_update_recursive($gimp[$k], $v);
			else
				$gimp[$k] = $v;
		}
		$na = func_num_args();
		for($i = 2; $i < $na; $i++)
			array_update_recursive($gimp, func_get_arg($i));
		return $gimp;
	}
}

if(! function_exists('array_fill_keys')) {
	// PHP 5.2.0+
	function array_fill_keys(array $keys, $value) {
		$vals = array_fill(0, count($keys), $value);
		return array_combine($keys, $vals);
	}
}

function array_flatten(array $a) {
	// naive version
	$ret = array();
	foreach($a as $k => $v) {
		if(is_array($v)) array_extend($ret, array_flatten($v));
		else $ret[] = $v;
	}
	return $ret;
}

/**
 * Very cool algorithm for sorting multi-dimensional arrays.  Found at http://us2.php.net/manual/en/function.array-multisort.php
 * Syntax: array_csort($array [, 'col1' [, SORT_FLAG [, SORT_FLAG]]]...);
 * Explanation: $array is the array you want to sort, 'col1' is the name of the column
 * you want to sort, SORT_FLAGS are : SORT_ASC, SORT_DESC, SORT_REGULAR, SORT_NUMERIC, SORT_STRING
 * you can repeat the 'col',FLAG,FLAG, as often you want, the highest prioritiy is given to
 * the first - so the array is sorted by the last given column first, then the one before ...
 * Example: $array = array_csort($array,'town','age',SORT_DESC,'name');
 * Portions created by SugarCRM are Copyright (C) SugarCRM, Inc. All Rights Reserved.
 * (refactored by ajrw - 1crm)
 */
function array_csort(&$arr) {
	$num_args = func_num_args();
	$params = array();
	for($i = 1; $i < $num_args; $i++) {
		$arg = func_get_arg($i);
		if (is_string($arg)) {
			$sortarr = array();
			foreach ($arr as $k => &$row) {
				$sortarr[$k] = $row[$arg];
			}
			$params[] =& $sortarr;
			unset($sortarr);
		} else {
			$params[] = $arg;
		}
	}
	$params[] =& $arr;
	call_user_func_array('array_multisort', $params);
	return $arr;
}

// define array_column if missing (added in php 5.5)
if(! function_exists('array_column'))
	require_once('include/utils/array_column.php');

/** Access environment variables defined in $_ENV or in Apache configuration (.htaccess) */
function custom_getenv($name) {
	$ret = array_get_default($_ENV, $name);
	if(function_exists('apache_getenv')) {
		$val = apache_getenv($name);
		if($val !== false)
			$ret = $val;
	}
	return $ret;
}

function is_demo_site($recheck=false, $ignore_unlock=false) {
	static $check;
	if(! isset($check) || $recheck || $ignore_unlock) {
		if(custom_getenv('IAH_MAKE_DEMO_SITE')) {
			global $current_user;
			$users = custom_getenv('IAH_UNLOCK_DEMO_USERS');
			if(! empty($users) && ! empty($current_user) && ! $ignore_unlock) {
				$u = preg_split('~\W+~', $users);
				if(in_array($current_user->user_name, $u))
					return ($check = false);
			}
			if($ignore_unlock)
				return true;
			else
				$check = true;
		}
		if(! isset($check))
			$check = false;
		if(AppConfig::setting('config.is_demo_site'))
			$check = true;
	}
	return $check;
}

function is_read_only_db($recheck=false) {
	static $check;
	if(! isset($check) || $recheck) {
		$check = false;
		if(custom_getenv('IAH_MAKE_READ_ONLY'))
			$check = true;
		if(AppConfig('site.db_options.readonly'))
			$check = true;
	}
	return $check;
}


function to_html($string, $mode=null) {
	$mode = $mode ?? (ENT_QUOTES | ENT_SUBSTITUTE);
	if(is_string($string))
		return htmlspecialchars($string, $mode, 'UTF-8');
	return $string;
}

// only used in a couple places, should probably go away
function to_html2($string) {
	return to_html($string, ENT_COMPAT);
}

function from_html($string, $mode=ENT_QUOTES) {
	if(is_string($string))
		return htmlspecialchars_decode($string, $mode);
	return $string;
}


function call_hook_function($hook, $default_class=null, $params=null) {
	if(! isset($params)) $params = array();
	if(is_array($hook)) {
		if(isset($hook['file'])) {
			require_once($hook['file']);
			unset($hook['file']);
		}
		if(isset($hook['class_function'])) {
			if(! isset($hook['class'])) {
				if(! $default_class)
					return;
				$hook['class'] = $default_class;
			}
			$hook = array($hook['class'], $hook['class_function']);
		}
		else if(isset($hook['function'])) {
			if(! empty($hook['class'])) {
				if(! class_exists($hook['class']))
					require_bean($hook['class']);
				if (!method_exists($hook['class'], $hook['function'])) return;
				$hook = array($hook['class'], $hook['function']);
			} else if(is_array($hook['function']))
				$hook = $hook['function'];
			else
				$hook = array($hook['function']);
		}
		else if(! isset($hook[0]))
			return;
		if(count($hook) == 2) {
			if (is_string($hook[0]) && !class_exists($hook[0])) {
				require_bean($hook[0]);
			}
			if (is_array($hook) && is_string($hook[0]) && !method_exists($hook[0], $hook[1])) {
				return;
			}
		} else
			$hook = $hook[0];
	}
	else if(is_string($hook)) {
		if (!function_exists($hook)) {
			if ($default_class) {
				$hook = array($default_class, $hook);

				if (!class_exists($hook[0]))
					require_bean($hook[0]);

				if (!method_exists($hook[0], $hook[1])) {
					return;
				}
			} else {
				return;
			}
		}
	}
	else
		return;
	return call_user_func_array($hook, $params);
}


function glob_unsorted() {
	// may want to try combining paths using GLOB_BRACE (try benchmark)
	$ret = array();
	$a = func_get_args();
	foreach($a as $path) {
		$fs = glob($path, GLOB_NOSORT);
		if($fs) array_extend($ret, $fs);
	}
	return $ret;
}

function glob_sorted() {
   // may want to try combining paths using GLOB_BRACE (try benchmark)
   $ret = array();
   $a = func_get_args();
   foreach($a as $path) {
      $fs = glob($path);
      if($fs) array_extend($ret, $fs);
   }
   return $ret;
}

function html_tag($name, $attribs, $content=null, $no_enc=false) {
	$ret = "<$name";
	foreach($attribs as $k => $v) {
		if(is_array($v) || is_object($v)) continue;
		if(! $no_enc) $v = to_html($v);
		$ret .= " $k=\"$v\"";
	}
	if(isset($content) && $content !== false) {
		if($content === true) { // close with empty content
			if($name == 'div' || $name == 'script' || $name == 'table')
				$content = '';
			else
				return "$ret/>";
		}
		return "$ret>$content</$name>";
	}
	return "$ret>";
}

function enumerate_and_end_buffers($clean, $nondefault = false) {
	for(;;) {
		$ob_status = ob_get_status();
		$ob_level = ob_get_level();
		if (empty($ob_status))
			break;
		if ($ob_status['name'] !== 'default output handler' && !$nondefault) {
			break;
		}
		yield $ob_level => $clean ? ob_get_contents() : "";
		$result = $clean ? @ob_end_clean() : @ob_end_flush();
		if (!$result) {
			break;
		}
	}
}

function end_flush_all_buffers($nondefault = false) {
	foreach (enumerate_and_end_buffers(false, $nondefault) as $unused);
}

function end_clean_all_buffers($nondefault = false) {
	foreach (enumerate_and_end_buffers(true, $nondefault) as $unused);
}

