<?php

/**
 * Class DavToken
 * Manage CardDAV / CalDAV sync token
 */
class DavToken {

	/**
	 * @var string - carddav/caldav
	 */
	private $type;

	/**
	 * DavToken constructor.
	 *
	 * @param string $type - carddav / caldav
	 */
	public function __construct($type) {
		$this->type = $type;
	}

	/**
	 * Returns current CardDAV/CalDAV sync-token
	 * (if token doesn't isset the method will create new one)
	 *
	 * @return int|null - timestamp
	 */
	public function getCurrent() {
		$token = UserPreference::getPreference("{$this->type}_sync_token");

		if (! $token) {
			if (! AppConfig::current_user_id())
				return null;

			$token = $this->generate();

			UserPreference::setPreference(
				"{$this->type}_sync_token",
				$token
			);
			UserPreference::savePreferencesToDB();
		}

		return $token;
	}

	/**
	 * Update sync-token with current timestamp value
	 */
	public function update() {
		UserPreference::setPreference(
			"{$this->type}_sync_token",
			$this->generate()
		);
		UserPreference::savePreferencesToDB();
	}

	/**
	 * Convert timestamp token value to date time string
	 *
	 * @param string|int $token - timestamp
	 *
	 * @return false|string
	 */
	public function convertToDate($token) {
		return gmdate('Y-m-d H:i:s', (int)$token);
	}

	/**
	 * Generate new token value
	 *
	 * @return int
	 */
	private function generate() {
		return strtotime(gmdate('Y-m-d H:i:s') .' GMT');
	}
}
