<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/

/**
 * Internal -- Has the external cache been checked to determine if it is available and configured.
 */
$GLOBALS['external_cache_checked'] = false;

/**
 * Internal -- Was an external cache detected - Cache still may be specifically disabled.
 */
$GLOBALS['external_cache_detected'] = false;

/**
 * Internal -- Is the external cache available.  This setting is determined by checking for the availability
 * of the external cache functions.  It can be overridden by adding a config variable 
 * (external_cache_disabled=true).
 */
$GLOBALS['external_cache_enabled'] = false;

/**
 * Internal -- This is controlled by a config setting (external_cache_reset) that will update the cache
 * with new values, but not read from the cache.
 */
$GLOBALS['external_cache_overwrite'] = false;

/**
 * Internal -- The number of hits on the external out of process cache in this request
 */
$GLOBALS['external_cache_request_external_hits'] = 0;

/**
 * Internal -- The number of total requests to the external out of process cache in this request
 */
$GLOBALS['external_cache_request_external_total'] = 0;

/**
 * Internal -- The number of hits on the external local cache in this request
 */
$GLOBALS['external_cache_request_local_hits'] = 0;

/**
 * Internal -- The number of total requests to the external local cache in this request
 */
$GLOBALS['external_cache_request_local_total'] = 0;

/**
 * Internal -- The data structure for the local cache.
 */
static $cache_local_store = array();

/**
 * Internal -- The type of external store available.
 */
static $external_cache_type = null;

/**
 * The interval in seconds that an external cache entry is valid.
 */
define('EXTERNAL_CACHE_INTERVAL_SECONDS', 300 );

/**
 * This constant is provided as a convenience for users that want to store null values
 * in the cache.  If your function frequently has null results that take a long time to
 * calculate, store those results in the cache.  On retrieval, substitue the value you 
 * stored for null.
 */
define('EXTERNAL_CACHE_NULL_VALUE', "SUGAR_CACHE_NULL_ZZ");
define('EXTERNAL_CACHE_FALSE_VALUE', "SUGAR_CACHE_FALSE_ZZ");

/**
 * Set this to true to see cache debugging messages in the end user UI.
 * This is a quick way to determine how well the cache is working and find out what
 * records are not being cached effectively.
 */
define('EXTERNAL_CACHE_DEBUG', false);

/**
 * Internal -- Determine if there is an external cache available for use.  
 * Currently only Zend Platform is supported.
 */
function check_cache()
{
    if(EXTERNAL_CACHE_DEBUG) echo "<HR>Checking cache<HR>";
    
    $default_order = "zend,apcu,apc,memcached";
    $ext_cache_order = AppConfig::setting('cache.detect_order', $default_order);
    $ext_cache_order = array_map('trim', explode(',', $ext_cache_order));
    
    if(! $GLOBALS['external_cache_checked'])
    {
        $GLOBALS['external_cache_checked'] = true;
        
        // Disable during install and if unique_key is not set
        if(AppConfig::in_installer() || ! AppConfig::unique_key()) {
        	$cache_disabled = true;
        }
        // If the cache is manually disabled, turn it off.
        if(AppConfig::setting('cache.disabled')) {
            $cache_disabled = true;
        }
        
        foreach($ext_cache_order as $check) {
        	if($check == 'zend' && function_exists("zend_shm_cache_fetch") && ini_get('zend_datacache.enable'))
			{
				$GLOBALS['external_cache_detected'] = true;
				$GLOBALS['external_cache_type'] = "zend";
				break;
			}
	        elseif($check == 'apc' && function_exists("apc_store") && ini_get('apc.enabled'))
			{
				$GLOBALS['external_cache_detected'] = true;
				$GLOBALS['external_cache_type'] = "apc";
				break;
			}
			elseif($check == 'apcu' && function_exists("apcu_store") && ini_get('apc.enabled'))
			{
				$GLOBALS['external_cache_detected'] = true;
				$GLOBALS['external_cache_type'] = "apcu";
				break;				
			}
			else if($check == 'memcached' && class_exists('MemCached') && AppConfig::setting('cache.memcached.enabled')) {
				$mc = new MemCached();
				$mc->setOption(Memcached::OPT_COMPRESSION, true);
				$mc->setOption(Memcached::OPT_BINARY_PROTOCOL, true);
				//$mc->setOption(Memcached::OPT_PREFIX_KEY, AppConfig::unique_key());
				foreach(AppConfig::setting('cache.memcached.servers') as $s)
					if(! $mc->addServer($s, 0))
						pr2("failed to add server $s");
				$GLOBALS['external_cache_mc'] =& $mc;
				$GLOBALS['external_cache_detected'] = true;
				$GLOBALS['external_cache_type'] = 'memcached';
				break;
			}
		}
		if(! $GLOBALS['external_cache_detected'] || ! empty($cache_disabled)) {
			$GLOBALS['external_cache_enabled'] = false;
			return;
		}
        
        // If the cache is being reset, turn it off for this round trip
        $sep = ($GLOBALS['external_cache_type'] == 'zend') ? '::' : '';
        $value = external_cache_retrieve_helper(sugar_cache_prefix().$sep.'EXTERNAL_CACHE_RESET', $found);
        if($found && $value) {
            // We are in a cache reset, do not use the cache.
            $GLOBALS['external_cache_enabled'] = false;
            return;
        }
        
        $GLOBALS['external_cache_enabled'] = true;
    }
    
    if(EXTERNAL_CACHE_DEBUG && $GLOBALS['external_cache_enabled']) echo "<HR>Cache enabled: {$GLOBALS['external_cache_type']}<HR>";
}


// added for consistency with old API - sugar_cache_get preferred
function sugar_cache_retrieve($key) {
	$ret = sugar_cache_get($key, $found);
	if($found && $ret === null)
		$ret = EXTERNAL_CACHE_NULL_VALUE;
	return $ret;
}


function sugar_cache_prefix() {
	return substr(sha1(AppConfig::unique_key()), 10) . AppConfig::dev_branch();
}


/**
 * Retrieve a key from cache.  For the Zend Platform, a maximum age of 5 minutes is assumed.
 *
 * @param String $key -- The item to retrieve.
 * @return The item unserialized
 */
function sugar_cache_get($key, &$found, $no_local_cache=false)
{
    if(! $GLOBALS['external_cache_checked'])
    {
        check_cache();
    }

	if(! $no_local_cache) {
		if (isset($GLOBALS['external_cache_request_local_total']))
			$GLOBALS['external_cache_request_local_total']++;
		if(isset($GLOBALS['cache_local_store'][$key]))
		{
			$GLOBALS['external_cache_request_local_hits']++;    
			$found = true;
			return $GLOBALS['cache_local_store'][$key];
		}
	}
    
    if(!$GLOBALS['external_cache_enabled'])
    {   
	    $found = false;
        return null;
    }

	// If it is not in memory, but is in cache, copy it to memory and use it
	$sep = ($GLOBALS['external_cache_type'] == 'zend') ? '::' : '';
    $value = external_cache_retrieve_helper(sugar_cache_prefix().$sep.$key, $found);
    if($found)
    {
    	if($value === EXTERNAL_CACHE_NULL_VALUE)
    		$value = null;
    	if(! $no_local_cache)
			$GLOBALS['cache_local_store'][$key] = $value;
		$GLOBALS['external_cache_request_external_hits']++;
    }
	else
	{
		if(EXTERNAL_CACHE_DEBUG) echo "<HR>External cache retrieve failed: $key <HR>";    
	}

	return $value;
}

/**
 * Internal -- This function actually retrieves information from the caches.
 * It is a helper function that provides that actual cache API abstraction.
 *
 * @param unknown_type $key
 * @return unknown
 */
function external_cache_retrieve_helper($key, &$found)
{
	$GLOBALS['external_cache_request_external_total']++;    
	
	$value = null;
    if($GLOBALS['external_cache_type'] == 'zend')
    {
        $value = @zend_shm_cache_fetch($key);
        if($value === false || is_null($value)) {
        	// documentation says this returns NULL, but in practice it returns FALSE
        	$value = null;
        	$found = false;
        } else {
        	if($value === EXTERNAL_CACHE_FALSE_VALUE)
        		$value = false;
        	$found = true;
        }
    }
    elseif ($GLOBALS['external_cache_type'] == 'apc')
    {
        $value = apc_fetch(array($key));
        if(is_array($value) && array_key_exists($key, $value)) {
        	$value = $value[$key];
        	$found = true;
        }
        else {
        	$value = null;
        	$found = false;
        }
    }
    elseif ($GLOBALS['external_cache_type'] == 'apcu')
    {
        $value = @apcu_fetch($key, $found);
    }
    else if($GLOBALS['external_cache_type'] == 'memcached')
    {
    	$mc = $GLOBALS['external_cache_mc'];
    	if( ($value = $mc->get($key)) !== false || $mc->getResultCode() == MemCached::RES_SUCCESS) {
    		$found = true;
    	} else {
    		$value = null;
    		$found = false;
    	}
    }
    if(EXTERNAL_CACHE_DEBUG) echo "<HR>retrieved key from cache ($key) value: ($value)<HR>";
    
    return $value;
}

/**
 * Put a value in the cache under a key
 *
 * @param String $key -- Global namespace cache.  Key for the data.
 * @param Serializable $value -- The value to store in the cache.
 */
function sugar_cache_put($key, $value, $no_local_cache=false, $replace_value=false)
{
    if(!$GLOBALS['external_cache_checked'])
    {
        check_cache();
    }

    if(EXTERNAL_CACHE_DEBUG) echo "<HR>Adding key to local cache $key with value ($value)<HR>";
        
    if(! $no_local_cache)
    	$GLOBALS['cache_local_store'][$key] = $value;
    else
    	unset($GLOBALS['cache_local_store'][$key]);
    
    if($GLOBALS['external_cache_enabled'])
    {
		if(is_null($value))
			$value = EXTERNAL_CACHE_NULL_VALUE;

		$sep = ($GLOBALS['external_cache_type'] == 'zend') ? '::' : '';
	    $external_key = sugar_cache_prefix().$sep.$key;
	    $test_time = constant('EXTERNAL_CACHE_INTERVAL_SECONDS');
		if($GLOBALS['external_cache_type'] == 'zend')
		{
			if($value === false)
				$value = EXTERNAL_CACHE_FALSE_VALUE;
			zend_shm_cache_store($external_key, $value, $test_time);
		    if(EXTERNAL_CACHE_DEBUG) echo "<HR>Added key to Zend cache $external_key with value ($value)<HR>";
		}
		elseif ($GLOBALS['external_cache_type'] == 'apc')
		{
			if($replace_value)
				$return = @apc_store($external_key, $value, $test_time);
			else
				$return = @apc_add($external_key, $value, $test_time);
		    
		    if(EXTERNAL_CACHE_DEBUG) echo "<HR>Added key to APC cache $external_key with value ($value) to be stored for $test_time seconds<HR>";
		}
		elseif ($GLOBALS['external_cache_type'] == 'apcu')
		{
			if($replace_value)
				$return = @apcu_store($external_key, $value, $test_time);
			else
				$return = @apcu_add($external_key, $value, $test_time);
		    
		    if(EXTERNAL_CACHE_DEBUG) echo "<HR>Added key to APCu cache $external_key with value ($value) to be stored for $test_time seconds<HR>";
		}
		else if($GLOBALS['external_cache_type'] == 'memcached')
		{
			if($replace_value)
				$return = $GLOBALS['external_cache_mc']->set($external_key, $value, $test_time);
			else
				$return = $GLOBALS['external_cache_mc']->add($external_key, $value, $test_time);
			
			if(EXTERNAL_CACHE_DEBUG) echo "<HR>Added key to Memcached $external_key with value ($value) to be stored for $test_time seconds<HR>";
		}
    }
}

/**
 * Clear a key from the cache.  This is used to invalidate a single key.
 *
 * @param String $key -- Key from global namespace
 */
function sugar_cache_clear($key)
{
    unset($GLOBALS['cache_local_store'][$key]);

    if($GLOBALS['external_cache_enabled'])
    {
    	$sep = ($GLOBALS['external_cache_type'] == 'zend') ? '::' : '';
        $external_key = sugar_cache_prefix().$sep.$key;
	    if($GLOBALS['external_cache_type'] == 'zend')
	    {
	        zend_shm_cache_delete($external_key);
	    }
	    elseif ($GLOBALS['external_cache_type'] == 'apc')
	    {
	        apc_delete($external_key);
	    }
	    elseif ($GLOBALS['external_cache_type'] == 'apcu')
	    {
	        apcu_delete($external_key);
	    }
	    elseif ($GLOBALS['external_cache_type'] == 'memcached')
	    {
	        $GLOBALS['external_cache_mc']->delete($external_key);
	    }
        
    }
}

/**
 * Turn off external caching for the rest of this round trip and for all round 
 * trips for the next cache timeout.  This function should be called when global arrays
 * are affected (studio, module loader, upgrade wizard, ... ) and it is not ok to 
 * wait for the cache to expire in order to see the change.
 */
function sugar_cache_reset()
{
	if(EXTERNAL_CACHE_DEBUG) echo "<HR>Clearing local/external cache<HR>";
	
    // Set a flag to clear the code.
    sugar_cache_put('EXTERNAL_CACHE_RESET', true);
    
    // Clear the local cache
    $GLOBALS['cache_local_store'] = array();

	// Clear the external cache    
	if($GLOBALS['external_cache_enabled']) {
	    if ($GLOBALS['external_cache_type'] == 'zend') {
	    	zend_shm_cache_clear(sugar_cache_prefix());
		} elseif ($GLOBALS['external_cache_type'] == 'apc') {
			apc_clear_cache();
		} elseif ($GLOBALS['external_cache_type'] == 'apcu') {
			apcu_clear_cache();
		} elseif ($GLOBALS['external_cache_type'] == 'memcached') {	
	        $GLOBALS['external_cache_mc']->flush();
		}
    }
    
    // Disable the external cache for the rest of the round trip
    $GLOBALS['external_cache_enabled'] = false;
}


function sugar_cache_enabled() {
    if(! $GLOBALS['external_cache_checked'])
        check_cache();
	return $GLOBALS['external_cache_enabled'];
}

function sugar_cache_detected() {
    if(! $GLOBALS['external_cache_checked'])
        check_cache();
	return $GLOBALS['external_cache_detected'];
}

function sugar_cache_type() {
    if(! sugar_cache_detected())
    	return null;
	return $GLOBALS['external_cache_type'];
}
