<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/**
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 */

require_once('include/utils/array_utils.php');

function clean_path( $path )
{
    // clean directory/file path with a functional equivalent
	$path = str_replace(array("\\", "//", "/./"), "/", $path );
	$path = preg_replace('~/+~', '/', $path);
    return $path;
}

/* note: last component of path is treated as a filename, so terminate directory paths with an extra '/' */
function create_cache_directory($path, $keep_final=false)
{
	$path = clean_path(AppConfig::cache_dir().$path);
	if($keep_final || substr($path, -1) == '/') {
		$dir = rtrim($path, '/');
		$file = '';
	} else {
		if( ($p = strrpos($path, '/')) !== false) {
			$dir = substr($path, 0, $p);
			$file = substr($path, $p+1);
			if($file == '.') $file = '';
		} else {
			$dir = $path;
			$file = '';
		}
	}
	if(! is_dir($dir)) {
		mkdir($dir, 0775, true);
	}
    $path = $dir . '/' . $file;
    if(strlen($file) && is_dir($path))
    	// remove existing directory if it's blocking a temp file
    	rmdir_recursive($path);
    return $path;
}

/* note: last component of path is treated as a filename, so terminate directory paths with an extra '/' */
function create_files_directory($path, $keep_final=false)
{
	$path = clean_path(AppConfig::files_dir().$path);
	if($keep_final || substr($path, -1) == '/') {
		$dir = rtrim($path, '/');
		$file = '';
	} else {
		if( ($p = strrpos($path, '/')) !== false) {
			$dir = substr($path, 0, $p);
			$file = substr($path, $p+1);
			if($file == '.') $file = '';
		} else {
			$dir = $path;
			$file = '';
		}
	}
	if(! is_dir($dir)) {
		mkdir($dir, 0775, true);
	}
    $path = $dir . '/' . $file;
    if(strlen($file) && is_dir($path))
    	// remove existing directory if it's blocking a temp file
    	rmdir_recursive($path);
    return $path;
}


function rmdir_recursive( $path ){
    if( is_file( $path ) ){
        return unlink( $path );
    }
    if( !is_dir( $path ) ){
        $GLOBALS['log']->error("ERROR: rmdir_recursive(): argument $path is not a file or a dir.\n");
        return false;
    }

    $status = true;

    $d = dir( $path );
    $dirs = array();
    while( $f = $d->read() ){
        if( $f == "." || $f == ".." )
            continue;
        $f = "$path/$f";
        if(is_file($f))
        	$status &= unlink($f);
        else
        	$dirs[] = $f;
    }
    $d->close();
    foreach($dirs as $f)
		$status &= rmdir_recursive($f);
	if($status)
		$status = rmdir($path);
		
    return $status;
}


function get_module_dir_list()
{
	$modules = array();
	$path = 'modules';
	$d = dir($path);
	while($entry = $d->read())
	{	
		if($entry != '..' && $entry != '.')
		{
			if(is_dir($path. '/'. $entry))
			{
				$modules[$entry] = $entry;
			}
		}
	}
	return $modules;
}

function mk_temp_dir( $base_dir, $prefix="" )
{
    $temp_dir = tempnam( getcwd() .'/'. $base_dir, $prefix );
    if( !$temp_dir || !unlink( $temp_dir ) )
    {
        return( false );
    }

    if( mkdir( $temp_dir ) ){
        return( $temp_dir );
    }

    return( false );
}

function remove_file_extension( $filename )
{
	$p = strrpos($filename, ".");
	if($p !== false)
		return substr($filename, 0, $p);
	return $filename;
}

function write_array_to_file( $the_name, $the_array, $the_file )
{
    $the_string =   "<?php\n" .
                    '// created: ' . date('Y-m-d H:i:s') . "\n" .
                    "\$$the_name = " .
                    var_export_helper( $the_array ) .
                    ";\n?>\n";

    if( $fh = @fopen( $the_file, "w" ) )
    {
        fputs( $fh, $the_string, strlen($the_string) );
        fclose( $fh );
        return( true );
    }
    else
    {
        return( false );
    }
}

function write_encoded_file( $soap_result, $write_to_dir, $write_to_file="" )
{
    // this function dies when encountering an error -- use with caution!
    // the path/file is returned upon success

    require_once('include/dir_inc.php');

    if( $write_to_file == "" )
    {
        $write_to_file = $write_to_dir . "/" . $soap_result['filename'];
    }

    $file = $soap_result['data'];
    $write_to_file = str_replace( "\\", "/", $write_to_file );

    $dir_to_make = dirname( $write_to_file );
    if( !is_dir( $dir_to_make ) )
    {
        mkdir_recursive( $dir_to_make );
    }
    $fh = fopen( $write_to_file, "wb" );
    fwrite( $fh, base64_decode( $file ) );
    fclose( $fh );

    if( md5_file( $write_to_file ) != $soap_result['md5'] )
    {
        die( "MD5 error after writing file $write_to_file" );
    }
    return( $write_to_file );
}

function create_custom_directory($file)
{
    $paths = explode('/',$file);
    $dir = 'custom';
    if(!file_exists($dir))
    {
        mkdir($dir, 0755);
    }
    for($i = 0; $i < sizeof($paths) - 1; $i++)
    {
        $dir .= '/' . $paths[$i];
        if(!file_exists($dir))
        {
            mkdir($dir, 0755);
        }
    }
    return $dir . '/'. $paths[sizeof($paths) - 1];
}

/**
 * This function will recursively generates md5s of files and returns an array of all md5s. 
 * 
 * @param	$path The path of the root directory to scan - must end with '/'
 * @param	$ignore_dirs array of filenames/directory names to ignore running md5 on - default 'cache' and 'upload'
 * @result	$md5_array an array containing path as key and md5 as value
 */
function generateMD5array($path, $ignore_dirs = array('cache', 'files', 'upload'))
{
	$dh  = opendir($path);
	while (false !== ($filename = readdir($dh)))
	{
		$current_dir_content[] = $filename;
	}
	
	// removes the ignored directories
	$current_dir_content = array_diff($current_dir_content, $ignore_dirs);
	
	sort($current_dir_content);
	$md5_array = array();
	
	foreach($current_dir_content as $file)
	{
		// make sure that it's not dir '.' or '..'
		if(strcmp($file, ".") && strcmp($file, ".."))
		{
			if(is_dir($path.$file))
			{
				// For testing purposes - uncomment to see all files and md5s
				//echo "<BR>Dir:  ".$path.$file."<br>";
				//generateMD5array($path.$file."/");
				
				$md5_array += generateMD5array($path.$file."/", $ignore_dirs);
			}
			else
			{
				// For testing purposes - uncomment to see all files and md5s
				//echo "   File: ".$path.$file."<br>";
				//echo md5_file($path.$file)."<BR>";
				
				$md5_array[$path.$file] = md5_file($path.$file);
			}
		}
	}
	
	return $md5_array;
	
}

/**
 * Function to compare two directory structures and return the items in path_a that didn't match in path_b
 * 
 * @param	$path_a The path of the first root directory to scan - must end with '/'
 * @param	$path_b The path of the second root directory to scan - must end with '/'
 * @param	$ignore_dirs array of filenames/directory names to ignore running md5 on - default 'cache' and 'upload'
 * @result	array containing all the md5s of everything in $path_a that didn't have a match in $path_b
 */
function md5DirCompare($path_a, $path_b, $ignore_dirs = array('cache', 'files', 'upload'))
{
	$md5array_a = generateMD5array($path_a, $ignore_dirs);
	$md5array_b = generateMD5array($path_b, $ignore_dirs);
	
	$result = array_diff($md5array_a, $md5array_b);
	
	return $result;
}

/**
 * Function to retrieve all file names of matching pattern in a directory (and it's subdirectories)
 * example: getFiles($arr, './modules', '.+/EditView.php/'); // grabs all EditView.phps
 * @param array $arr return array to populate matches
 * @param string $dir directory to look in [ USE ./ in front of the $dir! ]
 * @param string $pattern optional pattern to match against
 */
function getFiles(&$arr, $dir, $pattern = null) { 
    $contents = glob("{$dir}/*"); 
    if(is_array($contents)) {
        foreach($contents as $file) { 
            if(is_dir($file)) {
                getFiles($arr, $file, $pattern);
            }
            else {
                if(empty($pattern)) $arr[] = $file;
                else if(preg_match($pattern, $file)) 
                $arr[] = $file;
            }
        }
    }
}

/**
 * Function to split up large files for download
 * used in download.php
 * @param string $filename
 * @param int $retbytes
 */
function readfile_chunked($filename,$retbytes=true) 
{
   	$chunksize = 1*(1024*1024); // how many bytes per chunk
	$buffer = '';
	$cnt = 0;
	$handle = fopen($filename, 'rb');
	if ($handle === false) 
	{
	    return false;
	}
	while (!feof($handle)) 
	{
	    $buffer = fread($handle, $chunksize);
	    echo $buffer;
	    flush();
	    if ($retbytes) 
	    {
	        $cnt += strlen($buffer);
	    }
	}
	    $status = fclose($handle);
	if ($retbytes && $status) 
	{
	    return $cnt; // return num. bytes delivered like readfile() does.
	}
	return $status;
}


?>
