<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/


function query_module_access_list(&$user, $semi_invis=false)
{
	return get_user_module_list($user->id, $semi_invis);
}

function get_user_module_list($user_id=null, $semi_invis=false)
{
	if(! isset($user_id)) $user_id = AppConfig::current_user_id();
	if($user_id)
		$mods = AppConfig::setting("modinfo.by_user.$user_id");
	else
		$mods = array();
	$ret = array();
	foreach($mods as $mod => $info) {
		if($info['hidden'] && ($info['hidden'] != 'manual' || ! $semi_invis)) {
			continue;
		}
		$ret[$mod] = $mod;
	}
	return $ret;
}


// grabs client ip address and returns its value
function query_client_ip()
{
	global $_SERVER;
	$clientIP = false;
	if(isset($_SERVER['HTTP_CLIENT_IP']))
	{
		$clientIP = $_SERVER['HTTP_CLIENT_IP'];
	}
	elseif(isset($_SERVER['HTTP_X_FORWARDED_FOR']) AND preg_match_all('#\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}#s', $_SERVER['HTTP_X_FORWARDED_FOR'], $matches))
	{
		// check for internal ips by looking at the first octet
		foreach($matches[0] AS $ip)
		{
			if(!preg_match("#^(10|172\.16|192\.168)\.#", $ip))
			{
				$clientIP = $ip;
				break;
			}
		}

	}
	elseif(isset($_SERVER['HTTP_FROM']))
	{
		$clientIP = $_SERVER['HTTP_FROM'];
	}
	else
	{
		$clientIP = array_get_default($_SERVER, 'REMOTE_ADDR', '127.0.0.1');
	}
	
	// longreach - added - ipv6 support
	if( ($ipv6 = normalize_ipv6($clientIP)) !== false)
		$clientIP = $ipv6;

	return $clientIP;
}

// longreach - added
function normalize_ipv6($ip) {
	$ip = strtolower(trim($ip));
	$ipp = explode('::', $ip);
	if(count($ipp) == 1) {
		$left = $ipp[0];
		$right = '';
	}
	else if(count($ipp) > 2)
		return false;
	else
		list($left, $right) = $ipp;
	if(! preg_match('/^[0-9a-f:]*$/', $left))
		return false;
	if(! preg_match('/^[0-9a-f:]*$/', $right))
		return false;
	$pre = array();
	foreach(explode(':', $left) as $p)
		if(strlen($p)) $pre[] = hexdec($p);
	$post = array();
	foreach(explode(':', $right) as $p)
		if(strlen($p)) $post[] = hexdec($p);
	$c = count($pre) + count($post);
	if($c > 8)
		return false;
	for($i = $c; $i < 8; $i++)
		$pre[] = 0;
	$ip = implode(':', array_map('dechex', array_merge($pre, $post)));
	return $ip;
}


// moved from SugarAuthenticate class
function validate_client_ip(&$mismatch) {
	// grab client ip address
	$clientIP = query_client_ip();
	$classCheck = 0;

	// check to see if we've got a current ip address in $_SESSION
	// and check to see if the session has been hijacked by a foreign ip
	if (AppConfig::setting('site.security.verify_client_ip') && isset($_SESSION['client_ip_address'])) {
		$sessionIP = $_SESSION['client_ip_address'];
		$session_parts = explode('.', $sessionIP);
		$client_parts = explode('.', $clientIP);
		
		if(count($client_parts) != 4)
			$classCheck = 0; // ipv6
		else {
			// match class C IP addresses
			for ($i = 0; $i < 3; $i ++) {
				if ($session_parts[$i] == $client_parts[$i]) {
					$classCheck = 1;
					continue;
				} else {
					$classCheck = 0;
					break;
				}
			}
		}

		if($sessionIP != $clientIP && empty($classCheck)) {
			$mismatch = array($sessionIP, $clientIP);
			return false;
		}
	} else {
		$_SESSION['client_ip_address'] = $clientIP;
	}
	return true;
}

function get_session_end_eror_msg() {
    $error = '';

    if (isset($_SESSION['display_session_ended'])) {
		$error = translate($_SESSION['display_session_ended'], 'app');
        unset($_SESSION['display_session_ended']);
    }

    return $error;
}

// sets value to key value
function get_val_array($arr){
	$new = array();
	if(!empty($arr)){
	foreach($arr as $key=>$val){
		$new[$key] = $key;
	}
	}
	return $new;
}

