<?php		

require_once('include/less/Less.php');

function theme_config_params($theme_name='Default') {
	global $pageController;

	$root_url = AppConfig::site_url();
	$root_path = preg_replace('~^\w+://[^/]*~', '', $root_url);
	$root_local = getcwd();
	$options = [
		'compress' => true,
		'sourceMap' => AppConfig::setting('site.features.css_source_maps', false),
		'relativeUrls' => true,
		'sourceMapRootpath' => $root_path,
		'sourceMapBasepath' => $root_local,
		'strictUnits' => true,
		'urlArgs' => js_version_params('css'),
	];

	$files = [
		'themes/Default/less/origin.less' => $root_url.'themes/',
	];

	if(! preg_match('~^[\w_\-]+$~', $theme_name) || ! is_dir('themes/'.$theme_name))
		$theme_name = 'Default';
	$theme_custom = "themes/{$theme_name}/less/theme.less";		
	$site_custom = "custom/$theme_custom";
	if($theme_name != 'Default' && file_exists($theme_custom)) {
		$files[$theme_custom] = $root_url.'themes/';
		$theme_less_found = true;
		if(file_exists($site_custom))
			$files[$site_custom] = $root_url.'themes/';
	}
	if($theme_name != 'Default') {
		$themeInstance = $pageController->create_theme_instance($theme_name);
		$variables = $themeInstance->get_theme_css_params();
	} else {
		$variables = ['theme' => 'Default'];
	}
	$variables['include-demo'] = 'true';
	
	$cache_dir = AppConfig::cache_dir()  . 'less/';
	
	return compact('files', 'options', 'variables', 'cache_dir');
}

function cached_theme_css($theme_name='Default', $vars = null) {
	$config = theme_config_params($theme_name);
	if($vars) {
		array_extend($config['variables'], $vars);
	}

	Less_Cache::$cache_dir = $config['cache_dir'];
	try {
		$compiled = Less_Cache::Get($config['files'], $config['options'], $config['variables']);
	} catch(Exception $e) {
		pr2("Error compiling CSS: ".$e->getMessage());
		return;
	}

	$include_path = Less_Cache::$cache_dir . $compiled;
	return compact('include_path');
}

class VariableVisitor extends Less_Visitor {
	var $standard_vars = [];
	var $custom_vars = [];
	var $visit_custom = false;
	var $isPreVisitor = true;

	public function run($root) {
		return $this->visitObj($root);
	}

	public function visitRule( $ruleNode ){
		if( $ruleNode->variable ){
			if($ruleNode->name == '@custom-vars') {
				$this->visit_custom = true;
			} else {
				$output = new Less_Output();
				$ruleNode->value->genCSS($output);
				$key = substr($ruleNode->name, 1);
				$val = $output->toString();
				if($this->visit_custom)
					$this->custom_vars[$key] = $val;
				else
					$this->standard_vars[$key] = $val;
			}
		}
		// return $ruleNode;
	}
}

function custom_theme_css($theme_name='Default', $vars = null) {
	$config = theme_config_params($theme_name);
	$config['options']['sourceMap'] = true;
	$collector = new VariableVisitor();
	$config['options']['plugins'][] = $collector;
	if($vars) {
		array_extend($config['variables'], $vars);
	}

	$parser = new Less_Parser($config['options']);
	foreach($config['files'] as $file_path => $uri_root) {
		$parser->ParseFile($file_path, $uri_root);
	}
	$parser->ModifyVars(['custom-vars' => 1]); // placeholder
	$parser->ModifyVars($config['variables']);
	
	return [
		'css' => $parser->getCss(),
		'standard_vars' => $collector->standard_vars,
		'custom_vars' => $collector->custom_vars,
	];
}
