<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/
/*********************************************************************************

 * Description:
 ********************************************************************************/

class vCard{
	var $properties = array();
	var $name = 'no_name';
	var $charset = 'CHARSET=UTF-8';

	function clear(){
		$this->properties = array();
	}

	function loadRow($result) {
		global $app_list_strings;
		$this->clear();
		// cn: bug 8504 - CF/LB break Outlook's vCard import
		$street = $result->getField('primary_address_street');

		$salutation = $result->getField('salutation');
		$salutation = array_get_default($app_list_strings['salutation_dom'], $salutation, $salutation);
		$this->setName($result->getField('first_name'), $result->getField('last_name'), $salutation);
		$this->setBirthDate($result->getField('birthdate'));
		$this->setPhoneNumber($result->getField('phone_fax'), 'FAX');
		$this->setPhoneNumber($result->getField('phone_home'), 'HOME');
		$this->setPhoneNumber($result->getField('phone_mobile'), 'CELL');
		$this->setPhoneNumber($result->getField('phone_work'), 'WORK');
		$this->setEmail($result->getField('email1'));
		$this->setWebsite($result->getField('website'));
		$this->setAddress($street,
			$result->getField('primary_address_city'),
			$result->getField('primary_address_state'),
			$result->getField('primary_address_postalcode'),
			$result->getField('primary_address_country'), 'WORK');
		$this->setORG($result->getField('primary_account'), $result->getField('department'));
		$this->setTitle($result->getField('title'));
		$this->setProperty('UID', $this->escape($result->getField('id')));
		$this->setProperty('NOTE', $this->escape($result->getField('description')));
	}

	function escape($str) {
		return str_replace(array("\r", "\n", ";", ","), array("\\r", "\\n", "\\;", "\\,"), $str);
	}

	function loadRecord($record_id, $bean='Contact') {
		require_once('include/database/ListQuery.php');
		try {
			$model = new ModelDef($bean);
		} catch(Exception $e) {
			return false;
		}

		$fields = array(
			'first_name', 'last_name', 'salutation',
			'birthdate', 'phone_fax', 'phone_home', 'phone_mobile', 'phone_work',
			'email1', 'email2',
			'primary_address',
			'primary_account', 'department', 'title', 'website'
		);
		$lq = new ListQuery($model, $fields);

		// FIXME - ACL

		$result = $lq->queryRecord($record_id);
		if($result) {
			$this->loadRow($result);
			return true;
		}
	}

	function setTitle($title){
		$this->setProperty("TITLE", $this->escape($title));
	}
	function setORG($org, $dep){
		$this->setProperty("ORG", $this->escape($org) . ';' . $this->escape($dep) );
	}
	function setAddress($address, $city, $state,$postal, $country, $type){
		$this->setProperty("ADR;$type",";;" . $this->escape($address) . ';' . $this->escape($city) . ';' . $this->escape($state) . ';' . $this->escape($postal) . ';' . $this->escape($country));
	}

	function setName($first_name, $last_name, $prefix){
		$this->name = trim($first_name.' '.$last_name);
		$this->setProperty('N', $this->escape($last_name).';'.$this->escape($first_name).';;'.$this->escape($prefix) );
		$this->setProperty('FN', $this->escape("$prefix $first_name $last_name"));
	}

	function setEmail($address){
		$this->setProperty('EMAIL;INTERNET', $this->escape($address), false);
	}

	function setWebsite($address){
		$this->setProperty('URL', $this->escape($address));
	}

	function setPhoneNumber( $number, $type){
		$this->setProperty("TEL;$type", $this->escape($number));
	}
	function setBirthDate($date){
			$this->setProperty('BDAY', $this->escape($date));
	}
	function getProperty($name){
		if(isset($this->properties[$name]))
			return $this->properties[$name];
		return null;
	}

	function setProperty($name, $value, $addCharset = true){
		if ($this->charset && $addCharset) {
			$name .= ';' . $this->charset;
		}
		$this->properties[$name] = $value;
	}

	function toString(){
		$temp = "BEGIN:VCARD\n";
		$temp .= "VERSION:4.0\n";
		foreach($this->properties as $key=>$value){
			$temp .= $key. ':'.$value."\n";
		}
		$temp.= "END:VCARD\n";


		return $temp;
	}

	function saveVCard(){
		global $locale;

		$content = $this->toString();
		$fname = download_safe_filename($this->name) . '.vcf';
		header("Content-Disposition: attachment; filename=\"{$fname}\"");
		header("Content-Type: text/x-vcard; charset=" . $locale->getExportCharset());
		header("Expires: Mon, 26 Jul 1997 05:00:00 GMT");
		header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT");
		header("Cache-Control: max-age=0");
		header("Pragma: public");
		header("Content-Length: " . strlen($content));

		print $content;
	}

	function importFromString($str, $module='Contact', $save = true) {
		return $this->importFromArray(preg_split("~[\\r\\n]+~", $str), $module, $save);
	}

	function importFromArray($lines, $module='Contact', $save = true) {
		global $current_user;
		global $locale;
		$start = false;
		require_once("modules/".$module."s/$module.php");
		$contact = new $module();
		$contact->title = 'imported';
		$contact->assigned_user_id = $current_user->id;
		$fullname = '';
		$linesCount = is_countable($lines) ? sizeof($lines) : 0;
		for($index = 0; $index < $linesCount; $index++){
			$line = $lines[$index];

			$line = trim($line);
			if($start){
				//VCARD is done
				if(substr_count(strtoupper($line), 'END:VCARD')){
					if(!isset($contact->last_name)){
						$contact->last_name = $fullname;
					}
					if(empty($contact->last_name))
						return '';
					return $save ? $contact->save() : $contact;

				}
				$keyvalue = explode(':',$line,2);
				if(sizeof($keyvalue)==2){
					$value = str_replace('\:', ':', $keyvalue[1]);
					for($newindex= $index + 1;  $newindex < sizeof($lines), substr_count($lines[$newindex], ':') == 0; $newindex++){
							$value .= $lines[$newindex];
							$index = $newindex;
					}
					$key = strtoupper($keyvalue[0]);
					$key = strtr($key, '=', '');
					$key = strtr($key, ',',';');
					$keys = explode(';' ,$key);

					$charset = 'utf-8';
					for($i = 1; $i < count($keys); $i++) {
						if(preg_match('/CHARSET=(.*)/', $keys[$i], $km))
							$charset = strtolower($km[1]);
					}
					if($charset != 'utf-8')
						$value = $locale->translateCharset($value, $charset);
					$values = explode(';',$value );

					if($keys[0] == 'TEL'){
						if(substr_count($key, 'WORK') > 0){
								if(substr_count($key, 'FAX') > 0){
										if(!isset($contact->phone_fax)){
											$contact->phone_fax = $value;
										}
								}else{
									if(!isset($contact->phone_work)){
											$contact->phone_work = $value;
									}
								}
						}
						if(substr_count($key, 'HOME') > 0){
								if(substr_count($key, 'FAX') > 0){
										if(!isset($contact->phone_fax)){
											$contact->phone_fax = $value;
										}
								}else{
									if(!isset($contact->phone_home)){
											$contact->phone_home = $value;
									}
								}
						}
						if(substr_count($key, 'CELL') > 0){
								if(!isset($contact->phone_mobile)){
										$contact->phone_mobile = $value;
								}

						}
						if(substr_count($key, 'FAX') > 0){
										if(!isset($contact->phone_fax)){
											$contact->phone_fax = $value;
						}

						}

					}
					if($keys[0] == 'N'){
						if(sizeof($values) > 0)
							$contact->last_name = $values[0];
						if(sizeof($values) > 1)
							$contact->first_name = $values[1];




					}
					if($keys[0] == 'FN'){
						$fullname = $value;


					}

	}
					if($keys[0] == 'ADR'){
						if(substr_count($key, 'WORK') > 0 && (substr_count($key, 'POSTAL') > 0|| substr_count($key, 'PARCEL') == 0)){

								if(!isset($contact->primary_address_street) && sizeof($values) > 2){
                                        $textBreaks = array("\n", "\r");
                                        $vcardBreaks = array("=0A", "=0D");
										$contact->primary_address_street = str_replace($vcardBreaks, $textBreaks, $values[2]);
								}
								if(!isset($contact->primary_address_city) && sizeof($values) > 3){
										$contact->primary_address_city = $values[3];
								}
								if(!isset($contact->primary_address_state) && sizeof($values) > 4){
										$contact->primary_address_state = $values[4];
								}
								if(!isset($contact->primary_address_postalcode) && sizeof($values) > 5){
										$contact->primary_address_postalcode = $values[5];
								}
								if(!isset($contact->primary_address_country) && sizeof($values) > 6){
										$contact->primary_address_country = $values[6];
								}
						}
						else if(empty($contact->alt_address_street)){
								if(!isset($contact->alt_address_street) && sizeof($values) > 2){
                                        $textBreaks = array("\n", "\r");
                                        $vcardBreaks = array("=0A", "=0D");
										$contact->alt_address_street = str_replace($vcardBreaks, $textBreaks, $values[2]);
								}
								if(!isset($contact->alt_address_city) && sizeof($values) > 3){
										$contact->alt_address_city = $values[3];
								}
								if(!isset($contact->alt_address_state) && sizeof($values) > 4){
										$contact->alt_address_state = $values[4];
								}
								if(!isset($contact->alt_address_postalcode) && sizeof($values) > 5){
										$contact->alt_address_postalcode = $values[5];
								}
								if(!isset($contact->alt_address_country) && sizeof($values) > 6){
										$contact->alt_address_country = $values[6];
								}
						}
					}

					if($keys[0] == 'TITLE'){
						$contact->title = $value;

					}
					if($keys[0] == 'EMAIL'){
						if(!isset($contact->email1)){
							$contact->email1 = $value;
						}else if(!isset($contact->email2)){
							$contact->email2 = $value;
						}

					}

					if($keys[0] == 'URL'){
						$contact->website = $value;

					}

					if($keys[0] == 'ORG'){
						if(sizeof($values) > 1){
								$contact->department = $values[1];
						}

					}

					if($keys[0] == 'NOTE'){
						$contact->description = $value;
					}

				}




			//FOUND THE BEGINING OF THE VCARD
			if(!$start && substr_count(strtoupper($line), 'BEGIN:VCARD')){
				$start = true;
			}

		}
		if(!isset($contact->last_name) || empty($contact->last_name))
			return '';
		return $save ? $contact->save() : $contact;
	}

	function importVCard($filename, $module='Contact', $save = true){
		$lines =	file($filename);
		return $this->importFromArray($lines, $module, $save);
	}
}

?>
