<?php
if(!defined('sugarEntry'))define('sugarEntry', true);

/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/

define('ONECRM_CSRF_PROTECT', 'strict');

$startTime = microtime(true);

//xdebug_start_trace('1crm');
//error_reporting(E_ALL); ini_set('display_errors', 1);


// Display license if requested
if(isset($_REQUEST['page']) && $_REQUEST['page'] == 'licensePrint'){
	include('install/licensePrint.php');
	exit;
}

if(! empty($_REQUEST['entryPoint'])) {
	if($_REQUEST['entryPoint'] == 'getImage') {
		include('soap/getImage.php');
		return;
	}
}


require_once('include/entryPoint.php');
// Prevent access via alternate site urls
if(AppConfig::is_extern_url())
	return;
if( ($redir = AppConfig::get_auto_redirect()) ) {
	if(array_get_default($_SERVER, 'REQUEST_METHOD') == 'GET') {
		indexRedirect(array('path' => $redir[0]));
	} else
		return;
}
// Redirect to install.php if not yet installed
if(! AppConfig::installed()) {
	if(AppConfig::can_install())
		indexRedirect(array('path' => 'install.php'));
	else {
		$GLOBALS['log']->fatal('application not installed, and installation explicitly disabled');
		exit; // install would have to be explicitly disabled in standard config
	}
}
// Redirect to upgrade.php if upgrade in progress
if (AppConfig::upgrade_in_progress()) {
	indexRedirect(array('path' => 'upgrade.php'));
}

insert_charset_header();
// Adjust server and request variables
if(!empty($_REQUEST['cancel_redirect'])) {
	if(!empty($_REQUEST['return_action'])) {
		$_REQUEST['action'] = $_REQUEST['return_action'];
		$_POST['action'] = $_REQUEST['return_action'];
		$_GET['action'] = $_REQUEST['return_action'];
	}
	if(!empty($_REQUEST['return_module'])) {
		$_REQUEST['module'] = $_REQUEST['return_module'];
		$_POST['module'] = $_REQUEST['return_module'];
		$_GET['module'] = $_REQUEST['return_module'];
	}
	if(!empty($_REQUEST['return_id'])) {
		$_REQUEST['id'] = $_REQUEST['return_id'];
		$_POST['id'] = $_REQUEST['return_id'];
		$_GET['id'] = $_REQUEST['return_id'];
	}
}

$module = array_get_default($_REQUEST, 'module', '');
$action = array_get_default($_REQUEST, 'action', '');
$record = array_get_default($_REQUEST, 'record', '');
// Used for current record focus
$focus = null;

///////////////////////////////////////////////////////////////////////////////
////	USER LOGIN AUTHENTICATION
//FIRST PLACE YOU CAN INSTANTIATE A SUGARBEAN;

if($module == 'Users' && $action == 'Login') {
	// preserve old links
	indexRedirect('login.php');
}

$authController = new AuthenticationController();

// for Disconnected Client
if(isset($_REQUEST['MSID'])) {
	session_id($_REQUEST['MSID']);
	$do_seamless = true;
}

if(! AppConfig::session_start()) {
	header('HTTP/1.1 503 Service Unavailable');
	sugar_die('Error starting session');
}

if(! empty($do_seamless)) {
	if(isset($_SESSION['user_id']) && isset($_SESSION['seamless_login'])) {
		unset ($_SESSION['seamless_login']);
		global $current_user;
		$authController->sessionAuthenticate($_SESSION['user_id']);
		$current_user->authenticated = true;
		$use_current_user_login = true;
	} else {
	    if(isset($_COOKIE[session_name()])) {
	    	if(AppConfig::detect_https() == 'https')
	       		setcookie(session_name(), '', time()-42000, '/', AppConfig::request_host_name(), FALSE, TRUE);
	       	else
	       		setcookie(session_name(), '', time()-42000, '/', AppConfig::request_host_name(), FALSE, FALSE);
        }
	    sugar_cleanup(false);
	    session_destroy();
	    die('Not a valid entry method');
	}
}

// If recording is available, call the recorder to record this round trip.
if(is_file("include/recorder.php"))
	include("include/recorder.php");

// Initialize $current_user
if(! $use_current_user_login) {
	$current_user = new User();
	if(isset($_SESSION['authenticated_user_id'])) {
		if(! $authController->sessionAuthenticate()) {
			session_destroy();
			indexRedirect('login.php');
		}
		$GLOBALS['log']->debug('Current user is: '.$current_user->user_name);
		$current_user->update_access_time();
	}
}
////	END USER LOGIN AUTHENTICATION
///////////////////////////////////////////////////////////////////////////////

//Redirect to upgrade.php if update in progress?
if(! AppConfig::check_database_version($info)) {
	header('HTTP/1.1 503 Service Unavailable');
	$GLOBALS['log']->fatal('Database version (config/sugar_version in config table) does not match application version.');
	sugar_die($info['message']);
}


if(isset($_REQUEST[session_name()]))
	$GLOBALS['log']->debug("*** Starting Application for session ".session_name());
else
	$GLOBALS['log']->debug("*** Starting Application for new session");

ACLController::filterModuleList($moduleList);
ACLController::filterModuleList($modSemiInvisList);


// Initialize page controller
AppConfig::init_page_controller();

// Redirect to login page if user is not logged in and action is not allowed
if(empty($_SESSION['authenticated_user_id']) && ! $pageController->action_allowed_without_user()) {
	$login_nav = array();

	// preserve a timed-out user's click choice
	if(AppConfig::started_new_session() || empty($_SERVER['HTTP_REFERER'])) {
		if($_SERVER['REQUEST_METHOD'] == 'GET' && !empty($record) && !empty($action) && !empty($module)) {
			$login_nav['login_module'] = $module;
			$login_nav['login_action'] = $action;
			$login_nav['login_record'] = $record;
		}
		else if($_SERVER['REQUEST_METHOD'] == 'GET' && empty($record) && $action == 'index' && !empty($module)) {
			$login_nav['login_module'] = $module;
			$login_nav['login_action'] = $action;
			if(! empty($_REQUEST['layout']))
				$login_nav['login_layout'] = $_REQUEST['layout'];
		}
	}
	if(isset($_REQUEST['mobile'])) $login_nav['mobile'] = $_REQUEST['mobile'];

	$GLOBALS['log']->debug('User not authorized, going to the login page');
	$login_nav = 'login.php' . encode_query_string($login_nav, true);
	$nextDirective = array('redirect', $login_nav);
}

// Check user's timezone is set, otherwise redirect
if(! AppConfig::check_init_user()) {
	$nextDirective = array('perform',
		array('module' => 'Users', 'action' => 'InitUser', 'record' => $current_user->id));
}

// Check if 2FA should be set immediately
if(AppConfig::check_2fa()) {
	$nextDirective = array('perform',
		array('module' => 'Users', 'action' => 'InitUser', 'record' => $current_user->id));
}


/// Utility methods used by above

function indexRedirect($params, $async=false) {
	if(is_string($params))
		$path = $params;
	else if(isset($params['path']))
		$path = $params['path'];
	else
		$path = 'index.php' . encode_query_string($params, true);
	if($async) {
		$path = javascript_escape($path);
		echo "<script nodisplay>SUGAR.util.loadUrl('$path');</script>";
	} else
		header('Location: '.$path);
	if(function_exists('sugar_cleanup'))
		sugar_cleanup(true);
	else
		exit;
}

function indexCheckRawOutput($params) {
	global $doRawOutput, $skipHeaders, $skipFooters;
	if(isset($params['raw_output'])) {
		$doRawOutput = $params['raw_output'];
		if($doRawOutput)
			$skipHeaders = $skipFooters = true;
	}
	if(isset($params['skip_headers']))
		$skipHeaders = $params['skip_headers'];
	if(isset($params['skip_footers']))
		$skipFooters = $params['skip_footers'];
}

function handlePDFPrint() {}

$doRawOutput = $skipHeaders = $skipFooters = false;


ob_start();
include('include/layout/render.php');
$pageData = ob_get_clean();


if(isset($_GET['print'])) {
	include ("phprint.php");
	sugar_cleanup(true);
}

if(!$skipHeaders)
	$pageInstance->finalize_page();
if(!$skipFooters) {
	if(! empty($current_user->id) && ! AppConfig::setting('site.performance.disable_status_check')) {
		$pageInstance->add_status_check_javascript();
	}
}

//skip headers for popups, deleting, saving, importing and other actions
if(!$skipHeaders) {

	// reload mod_strings - should not be necessary, but seems to be wiped out occasionally
	if(empty($mod_strings))
		$mod_strings = return_module_language($current_language, $currentModule);

	// longreach - added - get new messages count
	$new_email_messages = 0;
	if(! empty($current_user->id))
		$new_email_messages = $current_user->get_new_emails_count();

	$GLOBALS['log']->debug("rendering header");
	$head_p = "themes/$theme/header.php";
	if(file_exists($head_p))
		include($head_p);
	else if(isset($themeInstance))
		echo $themeInstance->render_header();

	echo '<div id="content-main">'; // end content div for async replace

	// Only print the errors for admin users.
	include_once ('modules/Administration/DisplayWarnings.php');

	if(AppConfig::read_only()) {
		echo '<p class="error">' . translate('WARN_READ_ONLY_MODE', 'Administration') . '</p>';
	}

} else {
	$GLOBALS['log']->debug("skipping headers");
}


echo $pageInstance->format_includes('body');
echo $pageData;
echo $pageInstance->format_includes('foot');


if(!$skipFooters) {
	echo '</div>'; // end content div for async replace

	$footer_settings = '';
	if(! empty($current_user->id) && ($_SERVER['REQUEST_METHOD'] == 'GET' || $action == 'DetailView') && !isset($_COOKIE['ANDROID_APP']) && !isset($_COOKIE['IOS_APP'])) {
	    // theme and language setters
	    $footer_settings .= '<form name="render_settings"><table class="footer_settings" border="0">';
		$ctls = array();
		$footer_settings .= '<tr><td><div class="buttons-inline">';

		if (!AppConfig::is_mobile()) {
			$all_themes = AppConfig::get_themes();
			$theme_title = to_html(translate('LBL_THEME', 'Users'));
			$footer_settings .=
				'<div class="inline-elt">'.
				'<input type="hidden" name="user_theme" value="'.to_html($theme).'" />'.
				'<div class="input-field input-field-group rbullet flatter" id="user_theme-input" title="'.$theme_title.'" style="width: 12em">'.
				'<div class="input-label input-field-icon"><div class="input-icon theme-icon ticon-Themes"></div></div><span class="input-label single">' .
				to_html($all_themes[$theme]) . '</span></div></div>';
			$theme_opts = encode_select_options($all_themes, true);
			$ctls[] = "new SUGAR.ui.SelectInput('user_theme-input', {name: 'user_theme', options: $theme_opts, onchange: function(k) { SUGAR.themes.changeTo(k); }})";

			if(isset($themeInstance) && ($css_styles = $themeInstance->get_theme_css_styles()) ) {
				$theme_style = $themeInstance->get_theme_css_style();
				if($theme_style) {
					$style_info = $css_styles[$theme_style];
					$footer_settings .=
						'<div class="inline-elt">'.
						'<input type="hidden" name="theme_style" value="'.to_html($theme_style).'" />'.
						'<div class="input-field input-field-group rbullet flatter" id="theme_style-input" title="'.$style_info['name'].'">'.
						'<div class="input-label input-field-icon"><div class="input-icon input-color-icon" style="background-color: '.$style_info['color'].'"></div></div></div></div>';
					$style_opts = array_column($css_styles, 'name', 'id');
					foreach($style_opts as $k => $v)
						$style_opts[$k] = ['color' => $css_styles[$k]['color'], '_display' => $v];
					$style_opts = encode_select_options($style_opts, true, ['columns' => [['type' => 'color'], ['name' => '_display']], 'display_name' => '_display']);
					$ctls[] = "new SUGAR.ui.SelectInput('theme_style-input', {name: 'theme_style', options: $style_opts, options_width: 200, color_icon: true, onchange: function(k) { SUGAR.themes.changeCss(k); }})";
				}

				if ($themeInstance->canSetBackground()) {
					$footer_settings .=
						'<div class="inline-elt">'.
						'<div class="input-field input-field-group rbullet flatter" onclick="themeCustomizeDialog();" id="theme_background-input" title="">'.
						'<div class="input-label input-field-icon">'.
						'<div class="input-icon icon-edit"></div>'.
						'</div></div></div>';
				}
			}
		}

	    //set language
	    $all_langs = AppConfig::get_languages();
		$lang_opts = encode_select_options($all_langs, true);
		$lang_title = to_html(translate('LBL_LANGUAGE', 'Users'));
		$disabled = !empty($_COOKIE['ANDROID_APP']) ? 'true' : 'false';
		$footer_settings .=
			'<div class="inline-elt">'.
			'<input type="hidden" name="user_language" value="'.to_html($current_language).'" />'.
			'<div class="input-field input-field-group rbullet flatter" id="user_language-input" title="' . $lang_title . '" style="width: 12em">'.
			'<div class="input-label input-field-icon"><div class="input-icon theme-icon ticon-LanguagePacks"></div></div><span class="input-label single">' .
			to_html($all_langs[$current_language]) . '</span></div></div>';
		$ctls[] = "new SUGAR.ui.SelectInput('user_language-input', {name: 'user_language', disabled: $disabled, options: $lang_opts, onchange: function(k) { SUGAR.language.changeTo(k); }})";
		$footer_settings .= '</div></td></tr>';
		$pageInstance->add_js_literal('SUGAR.ui.initForm("render_settings", ['.implode(', ', $ctls).']);', null, LOAD_PRIORITY_END);
		$footer_settings .= '</table></form>';
	}

	$footer_debug = '';
	$calc_response = AppConfig::setting('site.performance.calculate_response_time');
	$calc_response_flag = AppConfig::setting('site.performance.calculate_response_time_flag');
	if($calc_response_flag && ! empty($_REQUEST[$calc_response_flag])) {
		$calc_response = true;
	}
	if($calc_response && (! AppConfig::is_mobile() || ! empty($_REQUEST['debug']))) {
		$deltaTime = microtime(true) - $startTime;
		$dbTime = (! empty($db) ? $db->getTotalQueryTime() : 0);
		$footer_debug .= $app_strings['LBL_SERVER_RESPONSE_TIME'].' '.format_number(round($deltaTime,2),2).' '.$app_strings['LBL_SERVER_RESPONSE_TIME_SECONDS'].'<br />';
		$footer_debug .= $app_strings['LBL_SERVER_RESPONSE_TIME_DB'].' '.format_number(round($dbTime,2),2).' '.$app_strings['LBL_SERVER_RESPONSE_TIME_SECONDS'].'<br />';
		if(AppConfig::setting('config.log_all_requests')) {
			$cfgTime = AppConfig::$_config->total_request_time;
			$footer_debug .= $app_strings['LBL_SERVER_RESPONSE_TIME_CONFIG'].' '.format_number(round($cfgTime,2),2).' '.$app_strings['LBL_SERVER_RESPONSE_TIME_SECONDS'].'<br />';
		}

		if(AppConfig::setting('site.performance.show_page_resources')) {
			// Print out the resources used in constructing the page.
			$included_files = get_included_files();
			$all_parsed = ConfigParser::get_all_parsed();

			$footer_qcount = (! empty($db) ? $db->getQueryCount() : 0);
			$footer_debug .= ($app_strings['LBL_SERVER_RESPONSE_RESOURCES'].' ('.$footer_qcount.', '.sizeof($included_files).', '.sizeof($all_parsed).')<br>');
			// Display performance of the internal and external caches....
			$cache_hits = $GLOBALS['external_cache_request_local_hits'];
			$cache_total = $GLOBALS['external_cache_request_local_total'];
			$footer_debug .= "Data cache (hits/total=ratio) local ($cache_hits/$cache_total=".($cache_total ? round($cache_hits*100/$cache_total,0) : 0)."%)";
			if(function_exists('opcache_get_status')) {
				$op_stat = opcache_get_status();
				$footer_debug .= ', opcache ' . ($op_stat && ! empty($op_stat['opcache_enabled']) ? 'enabled' : 'disabled');
			}
			if($GLOBALS['external_cache_request_external_total'])
				// Only display out of process cache results if there was at least one attempt to retrieve from the out of process cache (this signifies that it was enabled).
				$footer_debug .= ", external ({$GLOBALS['external_cache_request_external_hits']}/{$GLOBALS['external_cache_request_external_total']}=".round($GLOBALS['external_cache_request_external_hits']*100/$GLOBALS['external_cache_request_external_total'],0)."%)";
			else
				$footer_debug .= ", no external";
			if($GLOBALS['external_cache_type'] && is_admin())
				$footer_debug .= ' <a href="index.php?module=Administration&action=ResetCache" class="footerLink">[reset cache]</a>';
			$footer_debug .= "<br>";
		}

		if(AppConfig::setting('site.performance.show_memory_usage')) {
			$footer_debug .= 'Memory usage peak: '. format_file_size(memory_get_peak_usage());
			$footer_debug .= ', current: ' . format_file_size(memory_get_usage()) . '<br>';
		}
	}

	// Under the Sugar Public License referenced above, you are required to leave in all copyright statements in both
	// the code and end-user application as well as the the powered by image. You can not change the url or the image below  .
	$footer_copyright = '<div class="footerLeft">&copy; 2004-' . date('Y') . ' <a href="http://www.1crm.com" target="_blank"  class="footerLink copyRightLink">1CRM Systems Corp.</a> All Rights Reserved.<br />';
	$footer_copyright .= '&copy; 2004-2008 <a href="http://www.sugarcrm.com" target="_blank" class="footerLink copyRightLink" rel="nofollow">SugarCRM Inc.</a> All Rights Reserved.</div>';
	$static = AppConfig::site_static_url();
	$footer_copyright .= '<div class="footerRight"><a href="http://www.sugarforge.org" target="_blank" rel="nofollow" title="Nostalgia"><img style="margin-top: 2px" border="0" width="106" height="23" src="'.$static.'include/images/poweredby_sugarcrm.png" alt="Powered By SugarCRM"></a>';
	$footer_copyright .= ' <a href="http://www.1crm.com/1crm-engine/" target="_blank" title="Modernity"><img style="margin-top: 2px" border="0" width="106" height="23" src="'.$static.'include/images/iah/1crm-powered.png" alt="Powered by 1CRM Engine"></a></div>';

	if(isset($themeInstance))
		echo $themeInstance->render_footer($footer_copyright, array('settings' => $footer_settings, 'debug' => $footer_debug));

	echo $pageInstance->format_includes('end');

	echo "\n</body></html>";
}

$pageInstance->post_output();

sugar_cleanup();
// finished with $_SESSION variable, avoid lock contention with other requests
session_write_close();
if(defined('IAH_FATAL_ERR'))
	exit;

// page output is finished
ignore_user_abort(true);
if (class_exists('Webhook') && Webhook::has_pending_calls()) {
	$content = '';
	foreach (enumerate_and_end_buffers(true, true) as $buf_content) {
		$content = $buf_content . $content;
	}
	ini_set('zlib.output_compression', '0');
	header('Content-Encoding: none');
	header('Content-Length: ' . mb_strlen($content, '8bit'));
	echo $content;
} else {
	end_flush_all_buffers();
}
flush();

include('include/layout/post_request.php');


