<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
define('sugarEntry', 1);


//define('OPEN_INSPECT', 1); // uncomment to allow unrestricted access (this is a security risk)

$inspector_version = '1.2';
$script_name = basename(__FILE__);
function array_update(&$arr, $fn, $recurse=false) {
	foreach($arr as $k => $v) {
		if(is_array($v)) {
			if($recurse)
				array_update($arr[$k], $fn);
		} else
			$arr[$k] = $fn($v);
	}
}

$magic_q = function_exists('get_magic_quotes_gpc') && get_magic_quotes_gpc();
if ($magic_q) {
	array_update($_REQUEST, 'stripslashes', true);
	array_update($_POST, 'stripslashes', true);
	array_update($_GET, 'stripslashes', true);
	array_update($_COOKIE, 'stripslashes', true);
}

function include_config() {
	global $sugar_version;
	require_once('sugar_version.php');
	require_once('include/config/AppConfig.php');
	require_once('include/utils/CacheManager.php');
	require_once('include/paths.php');
	AppConfig::set_system_timezone();
}
function fix_setting($v) {
	if(is_array($v))
		return '';
	else if($v === true)
		$v = 'true';
	else if($v === false)
		$v = 'false';
	else if($v === null)
		$v = 'null';
	return $v;
}
function config_flatten($c, $pfx='') {
	$ret = array();
	if(is_array($c)) {
		foreach($c as $k => $v)
			if(is_array($v))
				array_extend($ret, config_flatten($v, $pfx . $k . '.'));
			else {
				$t = gettype($v);
				if($t == 'string')
					$t = 'text';
				$v = fix_setting($v);
				$ret[$pfx . $k] = array('value' => $v, 'type' => $t);
			}
	}
	return $ret;
}
function init_log() {
	require_once('include/log/Logger.php');
	$GLOBALS['log'] = new Katzgrau\KLogger\Logger('inspector', false);
}
function lock_inspector() {
	AppConfig::set_local('inspector.enabled', false);
	AppConfig::save_local();
}


init_log();

if(php_sapi_name() == 'cli' && isset($argv)) {

	// switch to owner of the current file to preserve file permissions
	if(function_exists('posix_getpwuid')) {
		$puser = posix_getpwuid(posix_geteuid());
		if($puser && $puser['name'] == 'root') {
			$uid = getmyuid();
			$gid = getmygid();
			if($uid !== false && $uid != $puser['uid']) {
				posix_setegid($gid);
				posix_seteuid($uid);
			}
		}
	}

	// command line options
	if(is_countable($argv) && count($argv) > 1) {
		$cmd = $argv[1];
		if($cmd == 'lock') {
			include_config();
			lock_inspector();
		}
		else if($cmd == 'reset') {
			include_config();
			$db = db_connect();
			if($db) $db->query('delete from config where category="info" and name="sugar_version"');
			$cs = glob('cache/system/*.php', GLOB_NOSORT);
			if($cs) foreach($cs as $c)
				unlink($c);
			echo "Database version reset and system caches cleared\n";
		}
		else if($cmd == 'db-update' || $cmd == 'db-check-update' || $cmd == 'db-repair' || $cmd == 'db-check-repair') {
			include_config();
			$db = db_connect();
			if(! $db) die('Error connecting to database');
			require_once('include/database/DBChecker.php');
			$checker = new DBChecker();
			$checker->reloadModels();
			if($cmd == 'db-update' || $cmd == 'db-check-update') {
				$actions = array('columns' => 1, 'audit' => 1, 'indices' => 1);
				if($cmd == 'db-update') $actions['execute'] = 1;
			} else {
				$actions = array('check' => 1);
				if($cmd == 'db-repair') $actions['repair'] = $actions['execute'] = 1;
			}
			try {
				$sql = $checker->checkTables($actions, 'text');
				if(! $sql) echo "No changes.\n";
				else if(! empty($actions['execute'])) echo "/* Done.*/\n";
			} catch(Exception $e) {
				die('Error checking database schema: '.$e->getMessage()."\n");
			}
		}
		else if($cmd == 'mysql') {
			$path = '';//`which mysql`;
			if(! $path) {
				foreach(array('/usr/local/mysql/bin/mysql', '/usr/bin/mysql') as $p) {
					if(file_exists($p)) {
						$path = $p;
						break;
					}
				}
			}
			if(! $path)
				die('Could not locate mysql binary');
			include_config();
			$db_info = AppConfig::setting('database.primary');
			if(! $db_info)
				die('Could not locate database information');
			$n = escapeshellarg($db_info['name']);
			$u = escapeshellarg($db_info['user_name']);
			$p = escapeshellarg($db_info['password']);
			echo "Connecting to database $n ...\n";
			$cmd = "$path -u $u --password=$p $n";
			$descriptors = array(
				array('file', '/dev/tty', 'r'),
				array('file', '/dev/tty', 'w'),
				array('file', '/dev/tty', 'w')
			);
			$proc = proc_open($cmd, $descriptors, $pipes);
			if(is_resource($proc)) {
				do {
					$status = proc_get_status($proc);
					usleep(50);
				} while($status['running']);
			}
		}
		else if($cmd == 'cleanup') {
			if(count($argv) > 2) {
				$cpath = $argv[2];
				$base = realpath('');
				if(file_exists($cpath)) {
					$remove = file($cpath);
					require_once('include/utils/file_utils.php');
					foreach($remove as $f) {
						$f = realpath(trim($f));
						if(! $f || substr($f, 0, strlen($base)) != $base)
							continue;
						if(is_dir($f))
							rmdir_recursive($f);
						else
							unlink($f);
						echo "removed: $f\n";
					}
				} else
					die("File not found\n");
			} else
				die("Please provide input file name\n");
		}
	}
	exit;
}


if(! defined('OPEN_INSPECT') || ! constant('OPEN_INSPECT')) {
	$server_cfg_disabled = ! empty($_ENV['DISABLE_IAH_INSPECT']);
	if(function_exists('apache_getenv'))
		$server_cfg_disabled |= apache_getenv('DISABLE_IAH_INSPECT');
	if(function_exists('getenv'))
		$server_cfg_disabled |= getenv('DISABLE_IAH_INSPECT');
	if($server_cfg_disabled)
		die('1CRM remote assist is disabled by server configuration.');

    include_config();

	if(! AppConfig::setting('inspector.enabled') || is_demo_site())
		die('1CRM remote assist is not enabled. (<a href="index.php">Return</a>)');

    $unique_key = AppConfig::unique_key('i@h');
    $access_pass = AppConfig::setting('inspector.password');
    if (! $access_pass)
        $access_pass = md5($unique_key);
	$ip = array_get_default($_SERVER, 'REMOTE_ADDR', '127.0.0.1');
    $limit_ip = AppConfig::setting('inspector.limit_ip');
	if(trim($limit_ip)) {
		$limit_ip = explode(' ', $limit_ip);
		foreach($limit_ip as $test_ip) {
			if(trim($test_ip) == $ip)
				$ip_okay = true;
		}
		if(empty($ip_okay))
			die('1CRM remote assist: access denied based on IP address ('.$ip.')');
	}
	$validated = md5($ip.$access_pass);
	if(empty($_COOKIE['inspect_validated']) || $_COOKIE['inspect_validated'] != $validated) {
		$user_pass = array_get_default($_REQUEST, 'user_pass', '');
		$check_valid = md5($ip.md5($unique_key.$user_pass));
		if($check_valid != $validated) {
?>
	<form action="<?php echo $script_name; ?>" method="POST">
	<p align="center">Enter password: <input type="password" name="user_pass" autocomplete="off" autofocus="true" />
	<input type="submit" value="Submit" /></p>
	</form>
<?php
			return;
		}
		if(AppConfig::detect_https() == 'https')
	       	setcookie('inspect_validated', $validated, time()+42000, '/', AppConfig::request_host_name(), FALSE, TRUE);
	    else
	       	setcookie('inspect_validated', $validated, time()+42000, '/', AppConfig::request_host_name(), FALSE, FALSE);
		if(strlen($user_pass)) {
			$_COOKIE['show_logout'] = '1';
			if(AppConfig::detect_https() == 'https')
		       	setcookie('show_logout', '1', time()+42000, '/', AppConfig::request_host_name(), FALSE, TRUE);
		    else
		       	setcookie('show_logout', '1', time()+42000, '/', AppConfig::request_host_name(), FALSE, FALSE);
		}
	}

	if(param('confirm_lock', false, '')) {
		lock_inspector();
		header('Location: inspect.php');
		return;
	}
	$show_lock_option = true;
} else {
	include_config();
}

$mode = param('mode', true, 'files');
$button_defs = array(
	'files' => 'File Browser',
	'db' => 'Table Browser',
	'sql' => 'SQL Console',
	'config' => 'Config',
	'phpinfo' => 'PHP Info',
	'cacheinfo' => 'Cache Info',
);
if(! empty($_COOKIE['show_logout']))
	$button_defs['logout'] = 'Logout';
$sel_mode = isset($button_defs[$mode]) ? $mode : 'files';
$buttons = '';
foreach($button_defs as $m => $title) {
	$cls = 'btn' . ($m == $sel_mode ? ' btn-current' : '');
	$buttons .= "<button type=\"button\" onclick=\"form_submit_new({mode: '{$m}'});\" class=\"{$cls}\"><div>{$title}</div></button>\n";
}

ob_start();
mb_internal_encoding('utf-8');
header('Content-type: text/html; charset=UTF-8');
ob_start();
?>

<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
<html>
<head>
<title>1CRM remote assist</title>
<style type="text/css">
body {
	font-family: sans-serif;
	margin: 0 0 2em 0;
	padding: 0;
	background-color: white;
	color: black;
}
form, input, select, textarea, button {
	margin: 0;
	padding: 0;
}
.checkbox {
	margin: 0;
	padding: 0;
	position: relative;
	top: 2px;
}
.textbox, select, .input {
	border: 1px inset #568;
	font-family: Monaco, "Bitstream Vera Mono", Consolas, monospace;
	font-size: 9pt;
}
.textbox, .input {
	padding: 2px;
}
a {
	color: #23a;
}
.main {
	margin-left: 5%;
	width: 90%;
}
.bar {
	background-color: #dde6ff;
	border-bottom: 1px solid #555;
	padding: 0.5em 1em;
	margin-bottom: 1em;
	line-height: 25pt;
}
.bar-info {
	color: #348;
	font: 14pt Georgia;
	float: right;
	line-height: 25pt;
}
.bar-info sub {
	font-size: 60%;
}
.btn {
	border: 2pt outset #459;
	background-color: white;
	color: #333;
	text-decoration: none;
	font: 9pt Verdana;
	font-weight: bold;
	padding: 3pt;
	border-radius: 3pt;
}
.btn div {
	border: 1px solid white;
	padding: 2pt;
	border-radius: 2pt;
}
.btn-current div {
	text-decoration: underline;
}
.bar .btn {
	margin-right: 0.5em;
	padding: 2pt;
}
.btn:hover {
	color: #459;
}
.btn:focus {
	border-style: inset;
}
.list-outer, .list-long-outer {
	border: 1px inset gray;
	overflow: auto;
}
.list-outer {
	height: 250px;
}
.list {
	background-color: #ddd;
	width: 100%;
}
.list th {
	background-color: #bbb;
	color: black;
	font: 10pt sans-serif;
	font-weight: bold;
	padding: 2pt 4pt 0 6pt;
	text-align: left;
	border-bottom: 1px solid #999;
}
.list td {
	background-color: white;
	padding: 2pt 6pt;
}
.list .alt td {
	background-color: #f0f0f0;
}
.list-item {
	font-family: Monaco, "Bitstream Vera Mono", Consolas, monospace;
	font-size: 10pt;
	padding: 0.1em;
	color: black;
	white-space: pre;
}
.list-long-outer th {
	font-size: 9pt;
}
.list-long-outer .list-item {
	font-size: 8pt;
}
.linked {
	cursor: pointer;
}
.dir {
	color: blue;
}
.no-access {
	color: red;
}
.list-head {
	font-family: Monaco, "Bitstream Vera Mono", Consolas, monospace;
	font-size: 10pt;
	color: white;
	background-color: #346;
	border: 1px solid #abd;
	padding: 0.2em 0.5em;
}
.list .hover td {
	background-color: #dfeeff;
}
.list .active td {
	background-color: #579;
	color: white;
}

.editor {
	width: 100%;
	height: 40em;
	font-family: Monaco, "Bitstream Vera Mono", Consolas, monospace;
	font-size: 9pt;
}

.code {
	border: 1px inset gray;
	font-family: Monaco, "Bitstream Vera Mono", Consolas, monospace;
	font-size: 9pt;
	overflow: auto;
	height: 30em;
	white-space: pre;
	line-height: 100%;
	padding-left: 4pt;
	margin-top: 0.5em;
}
.image {
	border: 1px inset gray;
	overflow: auto;
	padding: 1em;
	max-height: 30em;
	margin-top: 0.5em;
	text-align: center;
}
.info-box {
	border: 1px solid gray;
	font-size: 11pt;
	overflow: auto;
	margin: 0.5em 0;
	padding: 1em;
	text-align: center;
	background-color: #dde6ef;
}
.sql-outer {
	border: 1px inset gray;
	padding: 0.25em;
}
#replace_file {
	padding: 0.5em;
}
.ln {
	color: gray;
	border-right: 1px solid gray;
	margin-right: 4pt;
	font-size: 6pt;
}
.sizer {
	border: 1px solid #999;
	background-color: #ddd;
	font-size: 14px;
	height: 5px;
	line-height: 5px;
	text-align: center;
	overflow: hidden;
	cursor: move;
}
.error-box {
	background-color: #f99;
	border: 1px solid red;
	padding: 0.2em;
	text-align: center;
	margin-bottom: 1em;
}
h4 {
	text-align: left;
	margin: 6px 0;
}
ul.plain li {
	list-style: none;
	line-height: 120%;
	margin: 6px 0;
}
</style>
<script type="text/javascript">
	function show_replace_file() {
		var elt = document.getElementById('replace_file');
		if(elt) elt.style.display = (elt.style.display == 'none' ? '' : 'none');
	}

	var hovered_row;
	function hover_row(item) {
		if(hovered_row && hovered_row != item)
			unhover_row();
		add_remove_class(item, 'hover', true);
		hovered_row = item;
	}

	function unhover_row(item) {
		if(! item || item == hovered_row) {
			add_remove_class(item, 'hover');
			hovered_row = null;
		}
	}

	function add_remove_class(item, cls, add) {
		var clsName = item.className;
		var rem_pat = new RegExp('\\s?\\b'+cls+'\\b', 'g');
		clsName = clsName.replace(rem_pat, '');
		if(add)
			clsName += ' ' + cls;
		item.className = clsName;
	}

	function form_set(key, val) {
		var form = document.forms[0];
		if(form.elements[key])
			form.elements[key].value = val;
		else {
			var inp = document.createElement('input');
			inp.type = 'hidden';
			inp.name = key;
			inp.value = val;
			form.appendChild(inp);
		}
	}

	function form_submit(append_action) {
		if(! document.forms[0].std_action)
			document.forms[0].std_action = document.forms[0].action;
		else
			document.forms[0].action = document.forms[0].std_action;
		if(append_action)
			document.forms[0].action += append_action;
		document.forms[0].submit();
		return false;
	}

	function form_submit_new(args) {
		var form = document.createElement('form'), input, k;
		form.method = document.forms[0].method;
		form.action = document.forms[0].std_action || document.forms[0].action;
		if(args) {
			for(k in args) {
				input = document.createElement('input');
				input.type = 'hidden';
				input.name = k;
				input.value = args[k];
				form.appendChild(input);
			}
		}
		document.body.appendChild(form);
		form.submit();
	}

	function check(e, elt) {
		if(typeof(elt) == 'string') {
			elt = document.getElementById(elt);
			elt.checked = ! elt.checked;
		}
		if(elt) {
			var checked = elt.checked;
			if(e) {
				if(e.stopPropagation) e.stopPropagation();
				else e.cancelBubble = true;
			}
			while(elt && elt.nodeName != 'TR')
				elt = elt.parentNode;
			if(elt)
				add_remove_class(elt, 'active', checked);
		}
	}

	function check_all(elt, nm) {
		var chks = document.getElementsByName(nm);
		for(var i in chks) {
			if(chks[i].checked != elt.checked) {
				chks[i].checked = elt.checked;
				check(null, chks[i]);
			}
		}
	}

	function catch_enter(elt, call) {
		elt.onkeypress = function(e) {
			if(! e) e = window.event;
			if(e.keyCode == 13)
				call();
		}
	}

	function mouse_y(evt) {
		if(window.event)
			return window.event.clientY + document.documentElement.scrollTop + document.body.scrollTop;
		else
			return evt.clientY + window.scrollY;
	}

	function move_sizer(evt) {
		var start_y = mouse_y(evt);
		var elt = document.getElementById('resize_list');
		var start_h = elt.offsetHeight - 2;
		document.onmouseup = function() {
			document.onmouseup = document.onmousemove = null;
		}
		document.onmousemove = function(e) {
			elt.style.height = '' + Math.max(start_h + mouse_y(e) - start_y, 0) + 'px';
		}
	}
</script>
</head>
<body>
<form action="<?php echo $script_name; ?>" method="POST" enctype="multipart/form-data" id="form">
<div class="bar">
<div class="bar-info">1<span style="font-size:11pt">CRM</span> remote assist<sub><?php echo $inspector_version; ?></sub></div>
<?php echo $buttons; ?>
</div>
<div class="main">

<?php

$page_header = ob_get_contents();
ob_end_clean();

function param($name, $persist=false, $default=null) {
	global $keep_params;
	if(! isset($keep_params)) $keep_params = array();
	if($persist && array_key_exists($name, $keep_params))
		return $keep_params[$name];
	if(isset($_GET[$name]))
		$val = $_GET[$name];
	else
		$val = array_get_default($_REQUEST, $name, $default);
	if($persist)
		$keep_params[$name] = $val;
	return $val;
}
function display_size(&$bytes) {
	$b = (int)$bytes;
	$s = array('B', 'K', 'M', 'GB', 'TB');
	if($b <= 0)
		return "0 ".$s[0];
	$con = 1024;
	$e = (int)(log($b,$con));
	return number_format($b/pow($con, $e), $e ? 2 : 0, '.', ',') . ' ' . $s[$e];
}
function display_file_date($f, $show) {
	$info = stat($f);
	if($info && isset($info[$show])) {
		$ts = $info[$show];
		return date('Y-m-d H:i:s', $ts);
	}
	return '?';
}
function attr_string($attrs) {
	$s = '';
	foreach($attrs as $k => $v)
		$s .= " $k=\"".to_html($v).'"';
	return $s;
}
function table_select($cols, &$rows, $styles, $show_header=false, $long=false) {
	static $elt_attrs = array('form', 'href', 'onclick', 'class', 'style');
	$ret = '<div class="'.($long ? 'list-long-outer' : 'list-outer" id="resize_list').'">';
	$ret .= "<table border=\"0\" cellpadding=\"0\" cellspacing=\"1\" class=\"list\">\n";
	if($show_header) {
		$ret .= '<thead><tr>';
		foreach($cols as $k) {
			if(isset($styles[$k]) && isset($styles[$k]['title'])) {
				$title = $styles[$k]['title'];
				unset($styles[$k]['title']);
			}
			else $title = to_html($k);
			$ret .= '<th>'.$title.'</th>';
		}
		$ret .= "</tr></thead>\n";
	}
	$ret .= "<tbody>\n";
	$widths = array();
	$no_escape = array();
	foreach($cols as $k) {
		if(isset($styles[$k])) {
			if(isset($styles[$k]['width'])) {
				$widths[$k] = $styles[$k]['width'];
				unset($styles[$k]['width']);
			}
			if(isset($styles[$k]['no_escape'])) {
				$no_escape[$k] = 1;
				unset($styles[$k]['no_escape']);
			}
		}
	}
	foreach($styles as $k => $v)
		$styles[$k] = attr_string($v);
	$rowidx = 0;
	foreach($rows as $row) {
		$linked = false;
		$attrs = array();
		$attrs['onmouseover'] = 'hover_row(this);';
		$attrs['onmouseout'] = 'unhover_row(this);';
		foreach($elt_attrs as $k) {
			if(isset($row['@'.$k])) {
				if($k == 'form') {
					$upd = '';
					foreach($row['@'.$k] as $k2 => $v2) {
						$k2 = to_html(addcslashes($k2, "'\\"));
						$v2 = to_html(addcslashes($v2, "'\\"));
						$upd .= "form_set('$k2','$v2'); ";
					}
					$anchor = isset($row['@anchor']) ? "'".$row['@anchor']."'" : '';
					$row['@onclick'] = $upd."return form_submit($anchor);";
				}
				else if($k == 'onclick' || $k == 'href') {
					$attrs[$k] = $row['@'.$k];
					$attrs['ondblclick'] = "return false;";
					$linked = true;
				} else
					$attrs[$k] = array_get_default($attrs, $k, '') . $row['@'.$k];
			}
		}
		$cls = 'list-item'.($rowidx ++ % 2 == 0 ? '' : ' alt').($linked ? ' linked' : '');
		$attrs['class'] = $cls . (isset($attrs['class']) ? ' '.$attrs['class'] : '');
		$ret .= '<tr'.attr_string($attrs).'>';
		$have_link = false;
		foreach($cols as $k) {
			$attrs = array_get_default($styles, $k, '');
			if($rowidx == 1 && isset($widths[$k])) {
				$attrs .= attr_string(array('width' => $widths[$k]));
			}
			$val = isset($row[$k]) ? (empty($no_escape[$k]) ? to_html($row[$k]) : $row[$k]) : '&nbsp;';
			if(isset($row['@name']) && ! $have_link) {
				$have_link = true;
				$val = '<a name="'.to_html($row['@name']).'" style="position: relative; top: -3pt"></a>'.$val;
			}
			$ret .= "<td{$attrs}>{$val}</td>";
		}
		$ret .= "</tr>\n";
	}
	$ret .= "</tbody></table></div>\n";
	if(! $long) {
		$ret .= '<div class="sizer" onmousedown="move_sizer(event); return false;">&#9662;</div>';
	}
	return $ret;
}



function db_connect($params=null) {
	global $dbcfg;
	include_config();
    $dbcfg = AppConfig::setting('database.primary');
	if(empty($dbcfg))
		die('Database settings not defined');
	if(is_array($params)) {
		if(isset($params['type'])) {
			$dbcfg['type'] = $params['type'];
			$dbcfg['force_type'] = true;
		}
		if(isset($params['persistent'])) $dbcfg['persistent'] = $params['persistent'];
	}
	require_once('include/database/DBManager.php');
	$mgr_cls = DBManagerFactory::getManagerFromSpec($dbcfg);
	if(! $mgr_cls)
		die('Could not determine database manager');
	$mgr = new $mgr_cls();
	try {
		$mgr->connect($dbcfg);
	} catch(IAHDatabaseError $e) {
		die("Error connecting to database: {$e->getMessage()}");
	}
	return $mgr;
}
function get_filename($p='filename') {
	global $filename, $path, $extname;
	$filename = param($p);
	$extname = null;
	if(isset($filename)) {
		$root = realpath('.');
		$filename = realpath($filename);
		if(substr($filename, 0, strlen($root)) != $root)
			$filename = '.';
		else {
			$filename = substr($filename, strlen($root)+1);
			if(! strlen($filename))
				$filename = '.';
		}
		if(is_dir($filename)) {
			$path = $filename;
			$filename = null;
		}
		else {
			$path = dirname($filename);
			$filename = basename($filename);
			$extname = $path . '/' . $filename;
		}
	} else {
		$path = '.';
	}
}
function show_error($msg) {
	echo '<div class="error-box">'.$msg.'</div>';
}
function dbq($f) {
	return "`$f`";
}
function scan_files($path, $extensions=array(), &$matches=array())
{
	if ($handle = opendir($path)) {
 		while (false !== ($file = readdir($handle))) {
    		if ($file != "." && $file != "..") {
				if (preg_match("/^.+\.((".implode(")|(",$extensions)."))$/", $file))
	        	    $matches[] = $path."/".$file;
				if (is_dir($path."/".$file))
					scan_files($path."/".$file, $extensions, $matches);
			}
	    }
    	closedir($handle);
	}
}
function scan_dupes($n)
{
	if ($n == 1)
		return true;
	else
		return false;
}
function format_query_time($s) {
	return round($s, 4) . ' seconds';
}
function format_seconds($s) {
	$ret = array();
	$chk = array(60, 60, 24);
	foreach($chk as $idx => $len) {
		$cnt = floor($s / $len);
		array_unshift($ret, sprintf('%02d', $s % $len));
		$s = $cnt;
		if(! $s && $idx > 0) break;
	}
	return implode(':', $ret);
}
$highlight_types = array('php', 'xml', 'html', 'txt', 'tpl', 'js', 'htaccess');
$show_reversed = array('log');
$show_text_formats = array('log', 'csv', 'tmp', 'patch');
$image_types = array('png', 'jpg', 'jpeg', 'gif', 'ico');

if($mode == 'download') {
	get_filename();
	if(isset($extname) && is_readable($extname)) {
		ob_end_clean();
		header('Content-Type: application/octet-stream');
		header('Content-Disposition: attachment; filename='.$filename);
		header('Expires: 0');
		header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
		$sz = filesize($extname);
		header('Content-Length: '.$sz);
		flush();
		readfile($extname);
		return;
	}
	$keep_params['mode'] = $mode = 'files';
}
else if($mode == 'delete') {
	get_filename();
	if(isset($extname) && is_writable($extname)) {
		if(! empty($_REQUEST['confirm'])) {
			unlink($extname);
			$_REQUEST['filename'] = $path;
			$keep_params['mode'] = $mode = 'files';
		}
		else {
			echo "<div class=\"list-head\">Delete File</div>\n";
			$show_info = 'Removing file: '.to_html($extname).'<br /><br />';
			$show_info .= '<input type="hidden" name="filename" value="'.to_html($extname).'" />';
			$show_info .= '<button type="submit" class="btn" onclick="form_set(\'confirm\', \'1\');">Confirm Delete</button>';
			$show_info .= ' <button type="button" class="btn" name="cancel" onclick="form_set(\'mode\', \'files\'); return form_submit(\'#active-file\');">Cancel</button>';
		}
	} else
		$keep_params['mode'] = $mode = 'files';
}
else if($mode == 'rmdir') {
	get_filename();
	if(isset($path) && is_writable($path) && $path != '.') {
		if(! empty($_REQUEST['confirm'])) {
			rmdir($path);
			$_REQUEST['filename'] = dirname($path);
			$keep_params['mode'] = $mode = 'files';
		}
		else {
			echo "<div class=\"list-head\">Remove Directory</div>\n";
			$show_info = 'Removing directory: '.to_html($path).'/<br /><br />';
			$show_info .= '<input type="hidden" name="filename" value="'.to_html($path).'/" />';
			$show_info .= '<button type="submit" class="btn" onclick="form_set(\'confirm\', \'1\');">Confirm</button>';
			$show_info .= ' <button type="button" class="btn" name="cancel" onclick="form_set(\'mode\', \'files\'); return form_submit(\'#active-file\');">Cancel</button>';
		}
	} else
		$keep_params['mode'] = $mode = 'files';
}
else if($mode == 'copy-move') {
	get_filename();
	if(isset($extname) && is_readable($extname)) {
		if(! empty($_REQUEST['confirm'])) {
			$source = $extname;
			$target = array_get_default($_REQUEST, 'target', '');
			if(empty($target))
				show_error('Missing target filename');
			else {
				$t_path = realpath(dirname($target));
				$t_file = basename($target);
				$do_copy = array_get_default($_REQUEST, 'copy', '0');
				if($t_path === false || ! is_writable($t_path))
					show_error('Cannot write to target directory');
				else {
					$t_ext = $t_path .'/'. $t_file;
					if(file_exists($t_ext) && ! is_writable($t_ext))
						show_error('Target file is not writable');
					else if($t_ext == $extname)
						show_error('Source and target files are the same');
					else {
						if($do_copy)
							copy($extname, $t_ext);
						else
							rename($extname, $t_ext);
						$_REQUEST['filename'] = $t_ext;
					}
				}
			}
			$keep_params['mode'] = $mode = 'files';
		}
		else {
			echo "<div class=\"list-head\">Copy/Move File</div>\n";
			$show_info = 'Source file: '.to_html($extname).'<br /><br />';
			$show_info .= '<label><input type="radio" class="radio" name="copy" value="1" autocomplete="off" /> Copy</label>';
			$show_info .= ' <label><input type="radio" class="radio" name="copy" value="0" checked="checked" autocomplete="off" /> Move</label>';
			$targ = ($path == '.' ? '' : to_html($path).'/');
			$show_info .= '<br /><br />Target file: <input type="text" class="input" name="target" size="35" value="'.$targ.'" autocomplete="off" />';
			$show_info .= '<input type="hidden" name="filename" value="'.to_html($extname).'" />';
			$show_info .= '<br /><br /><button type="submit" class="btn" onclick="form_set(\'confirm\', \'1\');">Confirm</button>';
			$show_info .= ' <button type="button" class="btn" name="cancel" onclick="form_set(\'mode\', \'files\'); return form_submit(\'#active-file\');">Cancel</button>';
		}
	} else
		$keep_params['mode'] = $mode = 'files';
}
else if($mode == 'upload') {
	get_filename();
	if(isset($_FILES['upload_file'])) {
		if($_FILES['upload_file']['error']) {
			show_error('The uploaded file could not be processed, check the server\'s max upload_max_filesize and post_max_size.');
		}
		else if(isset($extname)) {
			if(is_writable($extname))
				move_uploaded_file($_FILES['upload_file']['tmp_name'], $extname);
			else
				show_error('The target file is not writable');
		}
		else if(is_writable($path)) {
			$target = $path.'/'.$_FILES['upload_file']['name'];
			if(! file_exists($target) || is_writable($target)) {
				move_uploaded_file($_FILES['upload_file']['tmp_name'], $target);
				$_REQUEST['filename'] = $target;
			}
			else
				show_error('The target file is not writable');
		}
		$keep_params['mode'] = $mode = 'files';
	}
	else {
		echo "<div class=\"list-head\">Upload File</div>\n";
		$show_info = '';
		if(isset($extname) && is_writable($extname)) {
			$show_info .= '<b>Replacing file:</b> '.to_html($extname).'<br /><br />';
			$show_info .= '<input type="hidden" name="filename" value="'.to_html($extname).'" />';
		}
		else {
			$show_info .= '<b>Target directory:</b> '.to_html($path).'/<br /><br />';
			$show_info .= '<input type="hidden" name="filename" value="'.to_html($path).'" />';
		}
		$show_info .= '<input type="file" name="upload_file" size="30" />';
		$show_info .= '<br /><br /><button class="btn" type="submit" name="replace">Upload</button>';
		$show_info .= ' <button class="btn" type="button" onclick="form_set(\'mode\', \'files\'); return form_submit(\'#active-file\');">Cancel</button>';
	}
}
else if($mode == 'create-dir') {
	get_filename();
	$target = array_get_default($_REQUEST, 'target', '');
	if(! empty($target)) {
		$target = basename(trim($target, '/'));
		mkdir($path . '/' . $target);
		$keep_params['mode'] = $mode = 'files';
	}
	else {
		echo "<div class=\"list-head\">Create Directory</div>\n";
		$show_info = '<b>Parent directory:</b> '.to_html($path).'/<br /><br />';
		$show_info .= 'Directory name: <input type="text" class="input" name="target" size="30" value="" autocomplete="off" />';
		$show_info .= '<input type="hidden" name="filename" value="'.to_html($path).'" />';
		$show_info .= '<br /><br /><button type="submit" class="btn">Create</button>';
		$show_info .= ' <button type="button" class="btn" name="cancel" onclick="form_set(\'mode\', \'files\'); return form_submit(\'#active-file\');">Cancel</button>';
	}
}
else if($mode == 'edit') {
	get_filename();
	if (!empty($_POST['confirm'])) {
		$fp = fopen($extname, 'w');
		if (!$fp) {
			show_error('Cannot open target file for writing');
		} else {
			fwrite($fp, $_POST['edit_content']);
			fclose($fp);
		}
		$keep_params['mode'] = $mode = 'files';
	} else {
		$fp = fopen($extname, 'r');
		if($fp) {
			$edit = fread($fp, filesize($extname));
			fclose($fp);
		} else
			$edit = '';
		echo "<div class=\"list-head\">Edit File</div>\n";
		$show_info = 'File: '.to_html($extname).'<br /><br />';
		echo '<input type="hidden" name="filename" value="'.to_html($extname).'" />';
		$show_info .= '<button type="submit" class="btn" onclick="form_set(\'confirm\', \'1\');">Save</button>';
		$show_info .= ' <button type="button" class="btn" name="cancel" onclick="form_set(\'mode\', \'files\'); return form_submit(\'#active-file\');">Cancel</button>';
	}
}

if($mode == 'logout') {
	if(AppConfig::detect_https() == 'https')
	    setcookie('inspect_validated', '', time()-42000, '/', AppConfig::request_host_name(), FALSE, TRUE);
	else
	    setcookie('inspect_validated', '', time()-42000, '/', AppConfig::request_host_name(), FALSE, FALSE);
	header('Location: '.$script_name);
	return;
}
else if($mode == 'phpinfo') {
	ob_start();
	phpinfo();
	$tail = preg_replace('~</?(!DOCTYPE[^>]*|html|head|body)>~', '', ob_get_contents());
	ob_end_clean();
}
else if($mode == 'cacheinfo') {
    include_config();
	check_cache();

	if(! empty($_REQUEST['clear_cache'])) {
		$clear = $_REQUEST['clear_cache'];
		AppConfig::invalidate_cache($clear == 'all' ? '' : $clear);
		if ($clear == 'all' || $clear == 'lang') {
			AppConfig::promote_js_custom_version();
		}
		/*if($clear == 'all') {
			AppConfig::cache_reset();
			check_cache();
		}*/
	}
	$clear_opts = array(
		'all' => 'All system caches',
		'lang' => 'Language cache (forces CSS/javascript refresh)',
		'modinfo' => 'Module metadata cache',
		'model' => 'Model metadata cache',
		'display' => 'Display metadata cache',
		'views' => 'Layout cache',
		'acl' => 'ACL cache',
		'ext' => 'Extension cache',
	);
	$clear_body = '';
	foreach($clear_opts as $k => $v) {
		$checked = (empty($clear) ? 'all' : $clear) == $k ? ' checked="checked"' : '';
		$cleared = (! empty($clear) && $clear == $k) ? "&nbsp; <em>Cleared</em>" : '';
		$clear_body .= "<li><label><input type='radio' name='clear_cache' value='$k'$checked> $v$cleared</label></li>\n";
	}
	echo <<<EOF
		<div class="info-box"><h4>System Cache</h4>
		<ul class="plain" style="text-align: left">
			$clear_body
		</ul>
		<button type="submit" class="btn">Clear Cache</button>
		</div>
EOF;

	if(! $GLOBALS['external_cache_enabled']) {
		if(AppConfig::setting('cache.disabled'))
			$show_info = "External cache support is disabled.";
		else
			$show_info = "No recognized external cache is enabled.";
	}
	else {
		$cache_type = $GLOBALS['external_cache_type'];
		if($cache_type == 'apc' || $cache_type == 'apcu') {
			if($cache_type == 'apc') {
				echo '<div class="list-head">APC (Alternative PHP Cache) Status Information</div>';
				$cache_info = apc_cache_info();
				$mem_info = apc_sma_info(true);
			} else {
				echo '<div class="list-head">APCu (Alternative PHP Cache) Status Information</div>';
				$cache_info = apcu_cache_info();
				$mem_info = apcu_sma_info(true);
			}
			$show = array('num_slots' => 'Slots', 'ttl' => 'TTL', 'num_hits' => 'Hits',
				'num_misses' => 'Misses', 'ratio' => 'Ratio', 'start_time' => 'Start Time');
			$show2 = array('num_seg' => 'Memory Segments', 'seg_size' => 'Segment Size',
				'avail_mem' => 'Available Memory');
			$row_times = array('mtime', 'creation_time', 'deletion_time', 'access_time');
			$cache_info['ratio'] = sprintf('%0.1f:1', $cache_info['num_hits'] / $cache_info['num_misses']);
			$info = array();
			foreach($show as $k => $v) {
				$val = $cache_info[$k];
				if($k == 'start_time')
					$val = date('Y/m/d H:i:s', $val);
				$info[] = "$v: ".to_html($val);
			}
			$show_info = implode(' &middot; ', $info);
			$info = array();
			foreach($show2 as $k => $v) {
				$val = $mem_info[$k];
				$info[] = "$v: ".to_html($val);
			}
			$show_info .= '<br />'.implode(' &middot; ', $info);
			$total_mem_size = 0;
			$cols = array('filename', 'num_hits', 'ref_count', 'mem_size');
			$rows = array();
			$sort_rows = array();
			$root = realpath('.');
			$sel_file = param('filename', true, '');
			$sel_file_info = null;
			foreach($cache_info['cache_list'] as $row) {
				if($row['type'] != 'file')
					continue;
				$nm = $row['filename'];
				if(substr($nm, 0, strlen($root)) != $root)
					continue;
				$nm = substr($nm, strlen($root)+1);
				if('./'.$nm == $sel_file) {
					$sel_file_info = $row;
					foreach($row_times as $f) {
						if(isset($sel_file_info[$f]) && $sel_file_info[$f]) {
							$v = ' ('.$sel_file_info[$f].')';
							$sel_file_info[$f] = date('Y/m/d H:i:s', $sel_file_info[$f]).$v;
						}
					}
					$row['@class'] = 'active';
					$row['@name'] = 'active-file';
				}
				$row['filename'] = $nm;
				$row['@form'] = array('filename' => './'.$nm);
				$row['@anchor'] = '#active-file';
				$rows[] = $row;
				$total_mem_size += $row['mem_size'];
				$sort_rows[] = strtolower($nm); //- $row['num_hits'];
			}
			array_multisort($sort_rows, $rows);
			$attrs = array();
			echo table_select($cols, $rows, $attrs, true);
			$show_info .= '<br />Total Cached Files: '.count($rows).' &middot; Total Memory: '.$total_mem_size;
			if($sel_file_info) {
				$show_text = print_r($sel_file_info, true);
			}
		}
		else if($GLOBALS['external_cache_type'] == 'zend') {
			$show_info = 'Zend Cache is enabled';
		}
	}
}
else if($mode == 'files') {
	get_filename();
	if(isset($extname)) {
		if(! file_exists($extname) || ! is_readable($extname))
			$show_info = 'File is not readable by the webserver user';
		else {
			if(preg_match('/\.([0-9a-z_\-]+)(\.\d)?$/i', $filename, $m))
				$file_ext = strtolower($m[1]);
			else
				$file_ext = '';
			$file_size = filesize($extname);
			$window_size = 102400;
			$max_image_size = 3*102400;
			if(in_array($file_ext, $highlight_types)) {
				if($file_size > $window_size && empty($_REQUEST['show_all'])) {
					$fp = fopen($extname, 'r');
					if($fp) {
						$highlight = fread($fp, $window_size);
						fclose($fp);
						$showing_size = $window_size;
					} else
						$highlight = '';
				} else
					$highlight = file_get_contents($extname);
				$hl_type = $file_ext;
				$show_link = $extname;
			} else if(in_array($file_ext, $show_text_formats)) {
				if($file_size > $window_size && empty($_REQUEST['show_all'])) {
					$fp = fopen($extname, 'r');
					if($fp) {
						if(in_array($file_ext, $show_reversed)) {
							fseek($fp, $file_size - $window_size);
							$showing_reversed = true;
						}
						$show_text = fread($fp, $window_size);
						fclose($fp);
						if(! empty($showing_reversed)) {
							$lines = array_reverse(explode("\n", trim($show_text, "\n")));
							$show_text = implode("\n", $lines);
						}
						$showing_size = $window_size;
					} else
						$show_text = '';
				} else {
					$show_text = file_get_contents($extname);
					if(in_array($file_ext, $show_reversed)) {
						$lines = array_reverse(explode("\n", trim($show_text, "\n")));
						$show_text = implode("\n", $lines);
						$showing_reversed = true;
					}
				}
				$show_link = $extname;
			} else if(in_array($file_ext, $image_types)) {
				if($file_size < $max_image_size)
					$show_image = $extname;
				$show_link = $extname;
			} else {
				$show_link = $extname;
			}
		}
	}

	echo '<div class="list-head">Path: '.preg_replace('/^\./', '', $path).'/</div>';
	if(! isset($path))
		$path = '.';
	$show_hidden = param('show_hidden', true, true);
	$cols = array('name', 'size', 'perm', 'user', 'group');
	$attrs = array('name' => array('width' => '90%'));
	$rows = $sort_names = array();
	$index = dir($path);
	$num_files = 0;
	$num_dirs = 0;
	$total_size = 0;
	if($index) {
		$user_cache = $grp_cache = array();
		while(($f = $index->read()) !== false) {
			if(($f == '.' && $path != '.') || ($f == '..' && $path == '.'))
				continue;
			else if(! $show_hidden && $f[0] == '.' && $f != '.' && $f != '..')
				continue;
			$row = array();
			$relname = "$path/$f";
			$t = filetype($relname);
			$meta = stat($relname);
			if($meta && function_exists('posix_getpwuid')) {
				if(isset($user_cache[$meta['uid']]))
					$uid = $user_cache[$meta['uid']];
				else {
					$lookup = posix_getpwuid($meta['uid']);
					$uid = $user_cache[$meta['uid']] = $lookup['name'];
				}
				$row['user'] = $uid;
				if(isset($grp_cache[$meta['gid']]))
					$gid = $grp_cache[$meta['gid']];
				else {
					$lookup = posix_getgrgid($meta['gid']);
					$gid = $grp_cache[$meta['gid']] = $lookup['name'];
				}
				$row['group'] = $uid;
				$filemode = octdec(substr(decoct(fileperms($relname)), -3));
				$perms = '';
				$perms .= (($filemode & 0x0100) ? 'r' : '-'); // user
				$perms .= (($filemode & 0x0080) ? 'w' : '-');
				$perms .= (($filemode & 0x0020) ? 'r' : '-'); // group
				$perms .= (($filemode & 0x0010) ? 'w' : '-');
				$row['perm'] = $perms;
			}
			if($t == 'dir') {
				if($f != '.' && $f != '..')
					$num_dirs ++;
				$f .= '/';
			}
			else {
				$t = 'file';
				if($meta) {
					$row['size'] = display_size($meta['size']);
					$total_size += $meta['size'];
				}
				if(! is_readable($relname) || ! is_writable($relname))
					$t .= ' no-access';
				$num_files ++;
			}
			if($f == $filename) {
				$t .= ' active';
				$row['@name'] = 'active-file';
			}
			$row['name'] = $f;
			$row['@class'] = $t;
			$row['@form'] = array('filename' => $relname);
			$row['@anchor'] = '#active-file';
			$prefix = '1';
			if (is_dir($relname)) {
				$prefix = '0';
			}
			$sort_names[] = $prefix . strtolower($f); // for mixed-case sorting
			$rows[] = $row;
		}
		$index->close();
	}
	array_multisort($sort_names, $rows);
	echo table_select($cols, $rows, $attrs);

	if(! isset($extname)) {
		$show_info = "$num_files file".($num_files != 1 ? 's' : '').' ('.display_size($total_size).' total)'
			.", $num_dirs sub-director".($num_dirs != 1 ? 'ies' : 'y');
		if(is_writable($path)) {
			$set_file = "form_set('filename', '".addcslashes(to_html($path),"'\\")."/'); ";
			$show_info .= '<br /><br /><button class="btn" type="button" onclick="'.$set_file.'form_set(\'mode\', \'create-dir\'); return form_submit();">Create sub-directory</button> ';
			$show_info .= ' <button class="btn" type="button" onclick="'.$set_file.'form_set(\'mode\', \'upload\'); return form_submit();">Upload file</button> ';
			if(! $num_files && ! $num_dirs)
				$show_info .= ' <button class="btn" type="button" onclick="'.$set_file.'form_set(\'mode\', \'rmdir\'); return form_submit();">Remove directory</button> ';
		}
	}
}
else if($mode == 'db') {
    $db = db_connect();
	if(! empty($_REQUEST['show_procs'])) {
		$result = $db->query("SHOW FULL PROCESSLIST", false);
		$cols = array('Id', 'Host', 'db', 'Command', 'Time', 'State', 'Info');
		$attrs = array();
		if($result) {
			$procs = $db->fetchRows($result, true, false);
			$times = array();
			$sleeping = 0;
			foreach($procs as $idx => $p) {
				if($p['Command'] == 'Sleep') {
					unset($procs[$idx]);
					$sleeping ++;
				}
				else {
					$times[] = $p['Time'];
					$procs[$idx]['Time'] = format_seconds($p['Time']);
				}
			}
			array_multisort($times, SORT_DESC, $procs);
			echo '<div class="list-head">MySQL Processes</div>';
			echo table_select($cols, $procs, $attrs, true, true);
			$show_info = "$sleeping process(es) sleeping";
		}
	}
	else if(! empty($_REQUEST['show_table'])) {
		$table_name = $_REQUEST['show_table'];
		$result = $db->query('SHOW COLUMNS FROM '.$table_name, false);
		if(! $result)
			die("Database error, could not list fields:\n" . $db->getLastError());
		if(isset($_REQUEST['columns']) && is_array($_REQUEST['columns']))
			$checked = $_REQUEST['columns'];
		else
			$checked = array('id', 'name', 'date_modified', 'assigned_user_id');
		$cols = array('check', 'Field', 'Type', 'Null', 'Key', 'Default', 'Extra');
		$fields = array();
		$check_all = '<input type="checkbox" class="checkbox" onclick="check_all(this, \'columns[]\');" />';
		$attrs = array('check' => array('width' => '1%', 'no_escape' => true, 'title' => $check_all));
		$sort_names = array();
		$sort_options = array();
		while($row = $db->fetchByAssoc($result, -1, false)) {
			$nm = to_html($row['Field']);
			$chk = in_array($nm, $checked) ? ' checked="checked"' : '';
			$row['check'] = '<input name="columns[]" id="col_'.$nm.'" type="checkbox" class="checkbox" value="'.$nm.'"'.$chk.' onclick="check(event, this);" autocomplete="off" />';
			if($chk)
				$row['@class'] = 'active';
			$row['@onclick'] = "check(event, 'col_{$nm}'); return false;"; // interferes with checkbox clicking, need workaround
			$sort_names[] = ($chk ? 0 : 1) . strtolower($row['Field']);
			$fields[] = $row;
		}
		array_multisort($sort_names, $fields);
		echo '<div class="list-head">MySQL Table: '.to_html($dbcfg['name'].'.'.$table_name).'</div>';
		echo table_select($cols, $fields, $attrs, true);

		$set_table = "form_set('show_table', '".to_html($table_name)."')";
		$where = param('where', false);
		$order_by = param('order_by', false);
		$sort_order = param('sort_order', false, 'ASC');
		$offset = param('offset', false, 0);
		$show_info = '<button class="btn" type="submit" onclick="'.$set_table.'; form_set(\'offset\', '.$offset.'); form_set(\'query_columns\', 1); form_submit(\'#tail\');">Query selected columns</button>';
		$show_info .= ' where <input type="text" class="textbox" name="where" size="40" value="'.to_html($where).'" onfocus="catch_enter(this, function(){'.$set_table.'});"/>';
		$show_info .= ' order by <select name="order_by" style="width: 10em"><option></option>';
		$columns = array();
		foreach($fields as $f) {
			if(in_array($f['Field'], $checked))
				$columns[] = $f['Field'];
			$sel = ($order_by == $f['Field']) ? ' selected="selected"' : '';
			$show_info .= '<option'.$sel.'>'.to_html($f['Field']).'</option>';
		}
		$show_info .= '</select><select name="sort_order">';
		$show_info .= '<option'.($sort_order == 'ASC' ? ' selected="selected"' : '').'>ASC</option>';
		$show_info .= '<option'.($sort_order == 'DESC' ? ' selected="selected"' : '').'>DESC</option>';
		$show_info .= '</select>';
		if(count($columns)) {
			$where_clause = $where ? "WHERE $where " : '';
			$order_clause = $order_by ? "ORDER BY `$order_by` $sort_order " : '';
			$limit = 30;
			$sql = 'SELECT '.implode(', ', array_map('dbq', $columns))." FROM `$table_name` {$where_clause}{$order_clause}LIMIT $offset,$limit";
			$sql_c = "SELECT COUNT(*) FROM `$table_name` $where_clause";
			$result = $db->query($sql_c, false);
			if(! $result)
				$tail = 'Error performing count query: '.$db->getLastError();
			else {
				$row = $db->fetchByRow($result, -1, false);
				$row_count = $row[0];
				$edit_query = '<input type="hidden" name="query" value="'.to_html($sql).'" />';
				$edit_query .= '<a href="#" onclick="form_set(\'mode\', \'sql\'); return form_submit();">edit query</a>';
				if(! $row_count)
					$show_info .= "<br />No results &middot; $edit_query";
				else {
					$endpos = min($row_count, $offset + $limit);
					$startpos = min($offset + 1, $endpos);
					$show_info .= "<br />showing $startpos to $endpos ($row_count total results) &middot; $edit_query";

					$result = $db->query($sql, false);
					if(! $result)
						$tail = 'Error performing query: '.$db->getLastError();
					else {
						$rows = $db->fetchRows($result, true, false);
						$attrs = array();
						$show_info .= ' &middot; ' . format_query_time($db->query_time);
						$tail = table_select($columns, $rows, $attrs, true, true);
					}

					$nav = '';
					if($offset) {
						$prev = max($offset - $limit, 0);
						$nav .= "<a href=\"#\" onclick=\"$set_table; form_set('offset', 0); return form_submit('#tail');\">&lt;&lt; start</a> ";
						$nav .= "<a href=\"#\" onclick=\"$set_table; form_set('offset', $prev); return form_submit('#tail');\">&lt; prev</a> ";
					} else {
						$nav .= '&lt;&lt; start &lt; prev ';
					}
					$final = max($row_count - $limit - 1, 0);
					if($final && $endpos <= $final) {
						$next = $endpos;
						$nav .= " <a href=\"#\" onclick=\"$set_table; form_set('offset', $next); return form_submit('#tail');\">next &gt;</a>";
						$nav .= " <a href=\"#\" onclick=\"$set_table; form_set('offset', $final); return form_submit('#tail');\">end &gt;&gt;</a>";
					} else {
						$nav .= ' next &gt; end &gt;&gt;';
					}
					$show_info .= "<br />$nav\n";
				}
			}
		}
	}
	else {
		$cli_ver = $db->getClientInfo();
		$srv_ver = $db->getServerInfo();
		$show_info = "MySQL client version: $cli_ver ({$db->dbLib})<br />";
		$show_info .= "MySQL server version: $srv_ver<br />";
		$show_info .= '<a href="#" onclick="form_set(\'show_procs\', 1); return form_submit();">Show processes</a><br />';
		$show_info .= preg_replace('/(Flush)/', '<br />$1', $db->getStats());
		$ver_five = version_compare($srv_ver, '5.0') >= 0;

		$result = $db->query("SHOW GLOBAL STATUS", false);
		if($result) {
			$rows = $db->fetchRows($result, true, false);
			$stats = array();
			foreach($rows as $r) $stats[$r['Variable_name']] = $r['Value'];
			$show_info .= "<br />Connections: {$stats['Connections']}, ";
			$show_info .= "Max used: {$stats['Max_used_connections']}";
		}

		if($ver_five) {
			$sql = 'SELECT table_name `name`, table_rows `rows`, engine `type`'
				. ' FROM INFORMATION_SCHEMA.tables'
				. ' WHERE table_type = "BASE TABLE" AND table_schema = "'.$dbcfg['name'].'" ORDER BY table_name';
			$cols = array('name', 'rows', 'type');
		}
		else {
			$sql = 'SHOW TABLES';
			$cols = array('name');
		}
		$result = $db->query($sql, false);
		if(! $result)
			die("Database error, could not list tables:\n".$db->getLastError());
		$tables = array();
		$attrs = array('name' => array('width' => '90%'));
		while($row = $db->fetchByAssoc($result, -1, false)) {
			if(! $ver_five)
				$row = array('name' => current($row));
			$row['@form'] = array('show_table' => $row['name']);
			$tables[] = $row;
		}
		echo '<div class="list-head">MySQL Database: '.to_html($dbcfg['name']).'</div>';
		echo table_select($cols, $tables, $attrs, $ver_five);
	}
}
else if($mode == 'sql') {
	$db = db_connect();
	$query = param('query', false, '');
	$execute = param('execute', false, 0);
	echo '<div class="list-head">MySQL Console</div>';
	echo '<div class="sql-outer">';
	echo '<textarea name="query" rows="4" cols="50" style="width: 99.7%;" autocapitalize="off" autocorrect="off" spellcheck="false">'.to_html($query).'</textarea>';
	echo '<button type="submit" class="btn" onclick="form_set(\'execute\', 1);">Execute Query</button>';
	echo ' <button type="button" class="btn" onclick="form_set(\'execute\', 1); form_set(\'csv\', 1); return form_submit();">Export CSV</button>';
	echo '</div>';
	if(! empty($query) && $execute) {
		$result = $db->query($query, false);
		if(! $result)
			$show_info = 'MySQL Error: '.$db->getLastError();
		else {
			if($result === true) {
				$row_count = $db->getAffectedRowCount();
				$show_info = "query executed, $row_count rows affected (" . format_query_time($db->query_time) . ')';
			}
			else {
				$row_count = $db->getRowCount($result);
				$show_info = "displaying $row_count results (" . format_query_time($db->query_time) . ')';
				$field_count = $db->getFieldCount($result);
				$cols = array();
				$attrs = array();
				$titles = array();
				for($i = 0; $i < $field_count; $i++) {
					$col = $db->fetchField($result, $i);
					$titles[$i] = $col ? $col->name : $i;
    				$attrs[$i]['title'] = $titles[$i];
    				$cols[] = $i;
    			}
				$rows = $db->fetchRows($result, false, false);
				$csv = param('csv', false, 0);
				if($csv) {
					$csv_f = tmpfile();
					if(! $csv_f)
						die("Error opening temp file");

					fputcsv($csv_f, $titles);
					foreach($rows as $row)
						fputcsv($csv_f, $row);

					if(preg_match('~\bfrom\s+(\w+)\s~i', $query, $m))
						$csv_name = $m[1].'.csv';
					else
						$csv_name = 'query.csv';

					ob_end_clean();
					header('Content-type: text/csv');
					header("Content-disposition: attachment; filename=\"$csv_name\"");
					header('Content-length: '.ftell($csv_f));

					fseek($csv_f, 0);
					fpassthru($csv_f);
					exit;
				}
				$tail = table_select($cols, $rows, $attrs, true, true);
			}
		}
	}
}
else if($mode == 'config') {
	$db = db_connect();
	$cats = ConfigManager::get_all_categories();
	$category = param('category', false, '');
	$setting = param('setting', false, '');
	if($setting && param('confirm', false, '')) {
		$v = param('new_value', false, '');
		$t = param('value_type', false, 'auto');
		if($v === 'false' && ($t == 'bool' || $t == 'auto'))
			$v = false;
		else if($v === 'true' && ($t == 'bool' || $t == 'auto'))
			$v = true;
		else if($v === 'null' && $t == 'auto')
			$v = null;
		else if($t == 'number')
			$v = (float)$v;
		else if($t == 'auto') {
			$old_v = AppConfig::setting($setting);
			$old_t = gettype($old_v);
			if($old_t == 'boolean' && ($v === '0' || $v === '1')) {
				$v = !! $v;
			}
			else if(is_numeric($v))
				$v = (float)$v;
		}
		AppConfig::set_local($setting, $v);
		AppConfig::save_local();
	}

	echo '<p>';
	if($category) echo '<a href="#" onclick="form_set(\'category\', \'\'); return form_submit();">&lt; Back</a> &middot; ';
	echo '<a href="#" onclick="form_set(\'confirm_lock\', \'1\'); return form_submit();">Lock Inspector</a></p>';
	echo '<div class="list-head">Config Settings'.($category ? ": $category" : '').'</div>';
	$cols = array('name', 'type');
	$attrs = array('name' => array('width' => '90%'));
	$rows = $names = array();
	if(! $category) {
		foreach($cats as $cat => $info) {
			$row = array('name' => $cat, 'type' => $info['type']);
			if($info['type'] == 'local' || $info['type'] == 'db') {
				$row['@class'] = 'file';
				if($cat == $category) {
					$row['@class'] .= ' active';
					$row['@name'] = 'active-row';
				}
				$row['@anchor'] = '#active-row';
				$row['@form'] = array('category' => $cat);
			}
			$rows[] = $row;
			$names[] = strtolower($cat);
		}
		array_multisort($names, $rows);
		echo table_select($cols, $rows, $attrs);
	}
	else {
		$c = explode('.', $category);
		$s = AppConfig::setting($c[0]);
		$vals = config_flatten($s, $category . '.');
		ksort($vals);
		$cols = array('name', 'value', 'type');
		$attrs = array('name' => array('width' => '40%'), 'type' => array('width' => '15%'));
		$rows = array();
		foreach($vals as $k => $v) {
			$v['name'] = $k;
			$v['@class'] = 'file';
			if($k === $setting) {
				$v['@class'] .= ' active';
				$v['@name'] = 'active-row';
			}
			$v['@anchor'] = '#active-row';
			$v['@form'] = array('category' => $category, 'setting' => $k);
			$rows[] = $v;
		}
		echo table_select($cols, $rows, $attrs);
	}
	if($setting) {
		$show_info = '<input type="text" class="input" name="setting" size="40" value="'.to_html($setting).'">&nbsp;';
		$show_info .= '<select name="value_type" class="input">';
		$show_info .= '<option value="auto">Auto</option>';
		$show_info .= '<option value="text">Text</option>';
		$show_info .= '<option value="bool">Boolean</option>';
		$show_info .= '<option value="number">Number</option>';
		$show_info .= '</select>&nbsp;';
		$val = fix_setting(AppConfig::setting($setting));
		$show_info .= '<input type="text" class="input" name="new_value" size="40" value="'.to_html($val).'">&nbsp;';
		$show_info .= '<button type="submit" class="btn" onclick="form_set(\'category\', \''.to_html($category).'\'); form_set(\'confirm\', \'1\');">Update</button>';
	}
}

if(isset($show_link)) {
	echo '<div class="info-box">';
	echo '<input type="hidden" name="filename" value="'.to_html($show_link).'" />';
	echo 'Download file: <a href="#" onclick="return form_submit(\'?mode=download\');">'.to_html($show_link).'</a>';
	echo ' ('.display_size($file_size).')';
	echo '<br />Created '.display_file_date($show_link, 'ctime').', modified '.display_file_date($show_link, 'mtime');
	if(isset($showing_size) || isset($showing_reversed)) {
		$rev = ! empty($showing_reversed);
		echo '<br />Displaying '.(isset($showing_size) ? ($rev ? 'last ' : 'first ').display_size($showing_size).',' : '').($rev ? ' reversed' : '');
	}
	echo '<br /><br />';
	if(is_writable($show_link)) {
		if(in_array($file_ext, $highlight_types))
			echo '<button class="btn" type="button" onclick="form_set(\'mode\', \'edit\'); return form_submit();">Edit</button> ';
		echo '<button class="btn" type="button" onclick="form_set(\'mode\', \'upload\'); return form_submit();">Replace</button> ';
		echo '<button class="btn" type="button" onclick="form_set(\'mode\', \'copy-move\'); return form_submit();">Copy/Move</button> ';
		echo '<button class="btn" type="button" onclick="form_set(\'mode\', \'delete\'); return form_submit();">Delete</button> ';
	}
	echo '<button class="btn" type="submit" onclick="return form_submit(\'#active-file\');">Refresh</button>';
	if(isset($highlight)) {
		$chk = param('show_line_numbers', false, true) ? ' checked="checked"' : '';
		echo '<input type="hidden" name="show_line_numbers" value="0" />';
		echo '&nbsp; <label><input type="checkbox" class="checkbox" name="show_line_numbers" autocomplete="off" value="1"'.$chk.' /> Show line numbers</label>';
	}
	echo '</div>';
}
else if(isset($show_info)) {
	echo '<div class="info-box">'.$show_info.'</div>';
}

if(isset($edit)) {
	echo '<textarea name="edit_content" class="editor">' . to_html($edit) . '</textarea>';
}



if(isset($highlight)) {
	$line_numbers = param('show_line_numbers', false, true);
	echo '<div class="code">';
	if($hl_type == 'php') {
		$body = highlight_string($highlight, true);
		$body = preg_replace('/(<\/?code>|\n)/', '', $body);
		if($line_numbers) {
			$lines = explode('<br />', $body);
			$linesCount = is_countable($lines) ? count($lines) : 0;
			$digits = ceil(log10($linesCount+1));
			$pat = '%'.$digits.'u';
			for($i = 0; $i < $linesCount; $i++) {
				$lines[$i] = '<span class="ln">'.sprintf($pat, $i+1) . ' </span>' . $lines[$i];
			}
			$body = implode("\n", $lines);
		}
		else {
			$body = preg_replace('/<br \/>/', "\n", $body);
		}
	} else {
		$body = to_html($highlight);
		if($line_numbers) {
			$lines = explode("\n", $body);
			$linesCount = is_countable($lines) ? count($lines) : 0;
			$digits = ceil(log10($linesCount+1));
			$pat = '%'.$digits.'u';
			for($i = 0; $i < $linesCount; $i++) {
				$lines[$i] = '<span class="ln">'.sprintf($pat, $i+1) . ' </span>' . $lines[$i];
			}
			$body = implode("\n", $lines);
		}
	}
	echo $body;
	echo '</div>';
}
else if(isset($show_text)) {
	echo '<div class="code">';
	echo to_html($show_text);
	echo '</div>';
}
else if(isset($show_image)) {
	echo '<div class="image">';
	echo '<img src="?mode=download&filename='.to_html($show_image).'" title="'.to_html($show_image).'" border="0">';
	echo '</div>';
}

if(isset($tail))
	echo '<a name="tail"></a>'.$tail;

$content = ob_get_contents();
ob_end_clean();

echo $page_header;

if(isset($keep_params)) {
	foreach($keep_params as $k => $v) {
		$k = to_html($k);
		$v = to_html($v);
		echo "\n<input type=\"hidden\" name=\"$k\" value=\"$v\" />";
	}
}
echo $content;

?>
</div>
</form>
</body>
</html>
