<?php

class InstallerCmdLine
{
	static $format = 'human';

	static $options = array(
		'setup_db_database_name' => array('Database Name', 'string', true, '-d', '--db'),
		'setup_db_host_name' => array('Database Host Name', 'string', false, '-h', '--host'),
		'setup_db_sugarsales_user' => array('Database Username', 'string', true, '-u', '--user'),
		'setup_db_sugarsales_password' => array('Database Password', 'string', true, '-p', '--pass', '--password'),
		'setup_db_create_sugarsales_user' => array('Create Database User', 'flag', false, '-c', '--create', '--create-user'),
		'setup_db_create_database' => array('Create Database', 'flag', false, '-cd', '--create-db'),
		'setup_db_drop_tables' => array('Drop and Recreate Database Tables', 'flag', false, '--drop'),
		'setup_db_admin_user_name' => array('Database Admin Username', 'string', false, '-a', '--admin'),
		'setup_db_admin_password' => array('Database Admin Password', 'string', false, '-w', '--admin-password'),
		'setup_site_url' => array('Site URL', 'string', true, '--url'),
		'setup_site_admin_password' => array('Site Administrator Password', 'string', false, '--ap'),
		'default_language' => array('Default Language', 'string', false, '--lang'),
		'installer_config_file' => array('Path to Installer Config file', 'string', false, '--cfg'),
		'setup_db_pop_demo_data' => array('Install Demo Data', 'flag', false, '--demo'),
		'setup_lock_demo_site' => array('Lock Demo Site After Install', 'flag', false, '--lockdown'),
		'setup_enable_portal' => array('Enable Portal Access', 'flag', false, '--portal'),
        'installer_no_confirm' => array('Run Without Options Confirm', 'flag', false, '--wc'),
		'help' => array('Print This Message', 'flag', false, '--help'),
		'output_format' => array('Output format (serialize, json, human)', 'string', false, '-m'),
	);

	static public function parseOptions()
	{
		global $argc, $argv, $locale;
		require_once('include/TimeDate.php');
		$timedate = new TimeDate();
		$tz = $timedate->guessTimeZone();
		if(! $tz)
			$tz = AppConfig::setting('locale.defaults.timezone');

		$ret = array(
			'setup_db_type' => 'mysql',
			'default_language' => AppConfig::setting('locale.defaults.language'),
			'setup_db_drop_tables' => 0,
			'setup_db_host_instance' => '',
			'setup_db_use_mb_demo_data' => 0,
			'setup_lock_demo_site' => 0,
			'setup_server_timezone' => $tz,
			'default_date_format' => AppConfig::setting('locale.defaults.date_format'),
			'default_time_format' => AppConfig::setting('locale.defaults.time_format'),
			'default_locale_name_format' => AppConfig::setting('locale.defaults.name_format'),
			'default_number_format' => $locale->getLocaleDefaultNumberFormat(),
			'default_address_format' => $locale->getLocaleDefaultAddressFormat(),
			'default_currency_name' => AppConfig::setting('locale.base_currency.name'),
			'default_currency_symbol' => AppConfig::setting('locale.base_currency.symbol'),
			'default_currency_iso4217' => AppConfig::setting('locale.base_currency.iso4217'),
			'default_email_charset' => AppConfig::setting('email.default_charset'),
			'default_export_charset' => AppConfig::setting('export.charset'),
			'default_export_format' => AppConfig::setting('export.list_format'),
			'default_currency_significant_digits' => AppConfig::setting('locale.base_currency.significant_digits'),
		);
			
		for ($i = 1; $i < $argc; $i++) {
			$found = false;
			$switch = $argv[$i];
			foreach (self::$options as $option => $def) {
				$idx = array_search($switch, $def, true);
				if ($idx > 1) {
					$type = $def[1];
					if ($type == 'flag') {
						$ret[$option] = 1;
					} else {
						if ($i >= $argc-1) {
							self::usage("Missing argument for $switch option");
						}
						$ret[$option] = $argv[$i+1];
						$i++;
					}
					$found = true;
					break;
				}
			}
			if (!$found) {
				self::usage("Unknown command-line option: {$switch}");
			}
		}

		if (isset($ret['help']))
			self::usage();

		if (isset($ret['output_format'])) {
			$f = $ret['output_format'];
			if ($f !== 'serialize' && $f !== 'json' && $f !== 'human') 
				self::usage("Invalid output_format");
			self::$format = $f;
			unset($ret['output_format']);
		}

		define('INSTALLER_CLI_OUTPUT', self::$format);

		foreach (self::$options as $option => $def) {
			if ($def[2] && !isset($ret[$option])) {
				self::usage("Option {$def[3]} is required\n");
			}
		}
		if (!isset($ret['setup_db_host_name']))
			$ret['setup_db_host_name'] = 'localhost';
		if (isset($ret['setup_db_admin_user_name']) && !isset($ret['setup_db_admin_password'])) {
			self::usage("Admin database user password not specified");
		}
		if (!isset($ret['setup_db_admin_user_name']) && isset($ret['setup_db_admin_password'])) {
			self::usage("Admin database user name not specified");
		}
		$ret['setup_db_username_is_privileged'] = isset($ret['setup_db_admin_user_name']) ? 0 : 1;
		if ($ret['setup_db_username_is_privileged']) {
			$ret['setup_db_admin_user_name'] = $ret['setup_db_sugarsales_user'];
			$ret['setup_db_admin_password'] = $ret['setup_db_sugarsales_password'];
		}
		if (!isset($ret['setup_site_admin_password']) || !strlen($ret['setup_site_admin_password'])) {
			$ret['setup_site_admin_password'] = 'admin';
		}
		$ret['setup_site_admin_password_retype'] = $ret['setup_site_admin_password'];

		if (!empty($ret['setup_db_pop_demo_data'])) {
			$demo_tree = AppConfig::setting('install_demo.options');
			$demo_options = array();
			flat_demo_options($demo_options, $demo_tree);
			foreach ($demo_options as $opt) {
				$ret['demo_options'][$opt['name']] = 1;
			}
		}

        if (! isset($ret['installer_no_confirm']) || $ret['installer_no_confirm'] == 0) {
            self::printOptions($ret);
            if (!self::confirm())
                exit;
        }

		return $ret;
	}


	private static function printOptions($opt)
	{
		print("\nInstallation options\n");
		printf("  Database Host             :  %s\n", $opt['setup_db_host_name']);
		printf("  Database Name             :  %s\n", $opt['setup_db_database_name']);
		printf("  Database Username         :  %s\n", $opt['setup_db_sugarsales_user']);
		printf("  Database Password         :  %s\n", $opt['setup_db_sugarsales_password']);
		printf("  Create Database User      :  %s\n", empty($opt['setup_db_create_sugarsales_user']) ? 'No' : 'Yes');
		printf("  Create Database           :  %s\n", empty($opt['setup_db_create_database']) ? 'No' : 'Yes');
		printf("  Drop and Recreate Tables  :  %s\n", empty($opt['setup_db_drop_tables']) ? 'No' : 'Yes');
		printf("  Database Admin Username   :  %s\n", $opt['setup_db_admin_user_name']);
		printf("  Database Admin Password   :  %s\n", $opt['setup_db_admin_password']);
		printf("  Site URL                  :  %s\n", $opt['setup_site_url']);
		printf("  Site Admin Password       :  %s\n", $opt['setup_site_admin_password']);
		printf("  Default Language          :  %s\n", $opt['default_language']);
		printf("  Server Timezone           :  %s\n", $opt['setup_server_timezone']);
		printf("  Install Demo Data         :  %s\n", empty($opt['setup_db_pop_demo_data']) ? 'No' : 'Yes');
		printf("  Lock Demo Site            :  %s\n", empty($opt['setup_lock_demo_site']) ? 'No' : 'Yes');
		printf("  Use Installer Config File :  %s\n", empty($opt['installer_config_file']) ? '<none>' : $opt['installer_config_file']);
		printf("  Path to php.ini file      :  %s\n", get_cfg_var('cfg_file_path'));
		print("\n");
	}

	private static function confirm()
	{
		print("\nProceed with installation? (yes/no) > ");
		for (;;) {
			@ob_flush();
			$reply = strtolower(trim(fgets(STDIN)));
			if ($reply === 'yes')
				return true;
			if ($reply === 'no')
				return false;
			print("Please, answer yes or no  > ");
			@ob_flush();
		}
	}

	private static function usage($message = null)
	{
		global $argv;
		if ($message) {
			printf("\n%s\n", $message);
		}
		print("\nUsage: {$argv[0]} options\n");
		print("\n==========================================================================\n");
		print("  Options                        Function                          Type   \n");
		print("==========================================================================\n");
		foreach (self::$options as $name => $def) {
			printf("%s %-30s %-33s %s\n", $def[2] ? '*' : ' ', join(', ', array_slice($def, 3)), $def[0], $def[1]);
		}
		print("\n* - required options. String option require additional argument\n\n");
		exit;
	}

	public static function reportStatus($message, $status = 'error')
	{
		$ret = array(
			'status' => $status,
			'message' => $message,
		);

		if (self::$format == 'json') {
			$json = getJSONobj();
			echo $json->encode($ret);
		} else if (self::$format == 'serialize') {
			echo serialize($ret);
		} else {
			echo "\n";
			$msgs = $ret['message'];
			if (!is_array($msgs))
				$msgs = array($msgs);
			foreach ($msgs as $msg) {
				echo $msg . "\n";
			}
		}
		if($status == 'success')
			installer_cleanup();
		exit($status === 'success' ? 0 : 1);
	}

	public static function message($msg)
	{
		if (self::$format === 'human')
			echo $msg;
	}

}

