<?php

require_once 'include/config/format/ConfigParser.php';
require_once 'modules/UWizard/UWizard.php';

class InstallerConfigParser
{
	static $CONFIG_ZIPFILE = 'cache/temp/___installer_config.zip';
	static $CONFIG_DESTDIR = 'cache/temp/___installer_config';
	static $CONFIG_FILE = 'cache/temp/___installer_config/installer_config.php';

	static public function parseConfigFile()
	{
		$defaults = self::defaults();
		if (!empty($_SESSION['INSTALLER_CONFIG_AVAILABLE'])) {
			$config = ConfigParser::load_file(self::$CONFIG_FILE);
			foreach ($config['session'] as $k => $v) {
				$defaults[$k] = $v;
			}
		}
		return $defaults;
	}

	private static function defaults()
	{
		global $locale;
		require_once('include/TimeDate.php');
		$timedate = new TimeDate();
		$tz = $timedate->guessTimeZone();
		if(! $tz)
			$tz = AppConfig::setting('locale.defaults.timezone');
		return array(
			'setup_server_timezone' => $tz,
			'default_date_format' => AppConfig::setting('locale.defaults.date_format'),
			'default_time_format' => AppConfig::setting('locale.defaults.time_format'),
			'default_locale_name_format' => AppConfig::setting('locale.defaults.name_format'),
			'default_number_format' => $locale->getLocaleDefaultNumberFormat(),
			'default_address_format' => $locale->getLocaleDefaultAddressFormat(),
			'default_currency_name' => AppConfig::setting('locale.base_currency.name'),
			'default_currency_symbol' => AppConfig::setting('locale.base_currency.symbol'),
			'default_currency_iso4217' => AppConfig::setting('locale.base_currency.iso4217'),
			'default_email_charset' => AppConfig::setting('email.default_charset'),
			'default_export_charset' => AppConfig::setting('export.charset'),
			'default_export_format' => AppConfig::setting('export.list_format'),
			'default_currency_significant_digits' => AppConfig::setting('locale.base_currency.significant_digits'),
			'default_language' => AppConfig::setting('locale.defaults.language'),
		);
	}


	static public function upload($inputName)
	{
		if (!isset($_FILES[$inputName])) return '';
		$error = array_get_default($_FILES[$inputName], 'error');
		switch ($error) {
			case UPLOAD_ERR_OK: break;
			case UPLOAD_ERR_INI_SIZE: return 'UPLOAD_ERR_INI_SIZE';
			case UPLOAD_ERR_FORM_SIZE: return 'UPLOAD_ERR_FORM_SIZE';
			case UPLOAD_ERR_PARTIAL: return 'UPLOAD_ERR_PARTIAL';
			case UPLOAD_ERR_NO_FILE: return '';
			case UPLOAD_ERR_NO_TMP_DIR: return 'UPLOAD_ERR_NO_TMP_DIR';
			case UPLOAD_ERR_CANT_WRITE: return 'UPLOAD_ERR_CANT_WRITE';
			case UPLOAD_ERR_EXTENSION: return 'UPLOAD_ERR_EXTENSION';
			default: return 'UPLOAD_ERR_NO_FILE';
		}
		$fname = basename(clean_path($_FILES[$inputName]['name']));
		if (substr(strtolower($fname), -4) != '.zip')
			return 'ERR_NOT_ZIP';


		if (!move_uploaded_file($_FILES[$inputName]['tmp_name'], self::$CONFIG_ZIPFILE)) {
			return 'ERR_CANT_MOVE';
		}

		return self::processConfigZip();
	}
	
	public static function finalConfigPath()
	{
		return AppConfig::setting('install.save_config_path', 'cache/installer_config.zip');
	}

	public static function processConfigZip($filename = null)
	{
		if (!$filename)
			$filename = self::$CONFIG_ZIPFILE;

		$_SESSION['INSTALLER_CONFIG_AVAILABLE'] = false;
		$zip = new ZipArchive;
		$result = $zip->open($filename);
		if ($result !== true) {
			return 'ERR_CANT_OPEN_ZIP';
		}
		if (!mkdir_recursive(self::$CONFIG_DESTDIR))
			return 'ERR_CANT_CREATE_ZIP_CONTENT_DIR';
		if (!$zip->extractTo(self::$CONFIG_DESTDIR))
			return 'ERR_CANT_UNZIP_CONFIG';

		try {
			$config = ConfigParser::load_file(self::$CONFIG_FILE);
		} catch (IAHConfigFileError $e) {
			return 'ERR_CANT_PARSE_CONFIG';
		}

		if (!is_array($config) || !isset($config['session']) || !is_array($config['session']))
			return 'ERR_INVALID_CONFIG';

		foreach (array_get_default($config, 'langpacks', array()) as $langpack) {
			$result = UWizard::quickCheckInstall(self::$CONFIG_DESTDIR . '/' . $langpack);
			if($result['status'] != 'success') {
				if (isset($result['message']))
					return $result['message'];
				return $result['errors'][0];
			}
		}

		$_SESSION['INSTALLER_CONFIG_AVAILABLE'] = true;
		return '';
	}

	static public function installLanguagePacks()
	{
		if (!empty($_SESSION['INSTALLER_CONFIG_AVAILABLE'])) {
			$config = ConfigParser::load_file(self::$CONFIG_FILE);
			foreach (array_get_default($config, 'langpacks', array()) as $langpack) {
				$result = UWizard::quickInstall(self::$CONFIG_DESTDIR . '/' . $langpack);
			}
		}
	}
	
	static public function cleanup() {
		$final = self::finalConfigPath();
		if(file_exists($final)) unlink($final);
		$destdir = self::$CONFIG_DESTDIR;
		if(file_exists($destdir)) rmdir_recursive($destdir);
	}
}

