<?php
if (!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');

if (!isset($install_script) || !$install_script) {
    die($mod_strings['ERR_NO_DIRECT_SCRIPT']);
}
set_time_limit(3600);

// flush after each output so the user can see the progress in real-time

require_once ('include/utils/basic_utils.php');
if (CLI_INSTALLER)
	end_flush_all_buffers();
else
	end_clean_all_buffers();
ob_implicit_flush();

require_once ('include/utils.php');
require_once ('data/Tracker.php');
require_once ('include/database/PearDatabase.php');
require_once ('include/database/DBManager.php');
require_once ('include/utils/file_utils.php');
require_once ('include/ProgressIndicator.php');
require_once ('modules/UWizard/UWizard.php');
require_once ('modules/Users/User.php');


// switch to selected default language for standard and demo data
// does not affect installer-based language ($mod_strings)
$current_language = $_SESSION['default_language'];

$progressIndicator = new ProgressIndicator(AppConfig::setting('install.progress_log'), true, CLI_INSTALLER);
$progressIndicator->logProgress($mod_strings['LBL_PERFORM_PRE_INSTALL'], 50, 0);

$setup_db_drop_tables = $_SESSION['setup_db_drop_tables'];
$setup_db_host_instance = $_SESSION['setup_db_host_instance'];
$setup_db_host_name = $_SESSION['setup_db_host_name'];
$setup_db_sugarsales_password = $_SESSION['setup_db_sugarsales_password'];
$setup_db_sugarsales_user = $_SESSION['setup_db_sugarsales_user'];
$setup_db_use_mb_demo_data = $_SESSION['setup_db_use_mb_demo_data'];
$setup_site_admin_password = $_SESSION['setup_site_admin_password'];
$setup_site_guid = (! empty($_SESSION['setup_site_specify_guid'])) ? $_SESSION['setup_site_guid'] : '';
$setup_site_url = $_SESSION['setup_site_url'];
$parsed_url = parse_url($setup_site_url);
$setup_site_host_name = $parsed_url['host'];
$setup_site_log_dir = array_get_default($_SESSION, 'setup_site_custom_log_dir');
$setup_site_log_file = array_get_default($_SESSION, 'setup_site_custom_log_file');
$setup_site_session_path = array_get_default($_SESSION, 'setup_site_custom_session_path');

session_write_close();

$bottle = handleLocalConfig();
handleHtaccess();

$install_currencies = AppConfig::setting('locale.currencies');
$default_currency_iso4217 =  $_SESSION['default_currency_iso4217'];
if ($default_currency_iso4217 != 'USD') {
	$install_currencies['USD'] = array(
		'symbol' => '$',
		'name' => 'US Dollar',
		'rate' =>  "1.0",
		'significant_digits' => 2,
	);
}

$all_tables = AppConfig::setting('model.index.by_type');
$install_tbl_cats = array('table', 'bean');
$install_tables = array();
foreach($install_tbl_cats as $t) {
	if(! isset($all_tables[$t]))
		continue;
	$install_tables = array_merge($install_tables, $all_tables[$t]);
}
sort($install_tables);
$install_rels = array_get_default($all_tables, 'link', array());
sort($install_rels);


$tasks = array(
	'create_tables' => array(
		'name'=> $mod_strings['LBL_PERFORM_TABLES'],
		'steps' => count($install_tables),
	),
	'create_relationships' => array(
		'name'=> $mod_strings['LBL_PERFORM_CREATE_RELATIONSHIPS'],
		'steps' => count($install_rels),
	),

	'create_default_settings' => array(
		'name'=> $mod_strings['LBL_PERFORM_DEFAULT_SETTINGS'],
		'steps' => 1,
	),

	'create_default_users' => array(
		'name'=> $mod_strings['LBL_PERFORM_DEFAULT_USERS'],
		'steps' => 1,
	),

	'create_default_currencies' => array(
		'name'=> $mod_strings['LBL_CREATE_DEFAULT_CURRENCIES'],
		'steps' => 1,
	),

	'create_default_booking' => array(
		'name'=> $mod_strings['LBL_CREATE_DEFAULT_BOOKING'],
		'steps' => 1,
	),

	'create_default_schedules' => array(
		'name'=> $mod_strings['LBL_CREATE_DEFAULT_SCHEDULES'],
		'steps' => 1,
	),
	
	'create_default_dashboards' => array(
		'name'=> $mod_strings['LBL_CREATE_DEFAULT_DASHBOARDS'],
		'steps' => 1,
	),

	'create_default_reports' => array(
		'name'=> $mod_strings['LBL_CREATE_DEFAULT_REPORTS'],
		'steps' => 1,
	),

	'create_default_folders' => array(
		'name'=> $mod_strings['LBL_GROUP_EMAIL_FOLDERS'],
		'steps' => 1,
	),

	'create_default_feeds' => array(
		'name'=> $mod_strings['LBL_POPULATE_FEEDS'],
		'steps' => 1, //count(file('modules/Feeds/feeds_os.sql')),
	),

	'create_default_pdf_templates' => array(
		'name'=> $mod_strings['LBL_CREATE_PDF_TEMPLATES'],
		'steps' => 1,
	),

	'enable_portal_access' => array(
		'name'=> $mod_strings['LBL_ENABLE_PORTAL_ACCESS'],
		'steps' => 1,
	),

);


try { // catch database error


// load the database config and manager
$dbCfg = getDatabaseConfig(true);
$mgr_cls = DBManagerFactory::getManagerFromSpec($dbCfg);
$db = new $mgr_cls();

if ($dbCfg['create_user']) {
	// create the database user
	if (CLI_INSTALLER) InstallerCmdLine::message($mod_strings['LBL_PERFORM_CREATE_DB_USER']);
	$db->createDatabaseUser($dbCfg);
	if (CLI_INSTALLER) InstallerCmdLine::message($mod_strings['LBL_PERFORM_DONE'] . "\n");
}

if ($dbCfg['create_database']) {
	// create the database
	if (CLI_INSTALLER) InstallerCmdLine::message("{$mod_strings['LBL_PERFORM_CREATE_DB_1']} {$dbCfg['name']} {$mod_strings['LBL_PERFORM_CREATE_DB_2']} {$dbCfg['host_name']}...");
	$db->createDatabase($dbCfg);
	if (CLI_INSTALLER) InstallerCmdLine::message($mod_strings['LBL_PERFORM_DONE'] . "\n");
} else {
    // ensure the charset and collation are utf8
    $db->setDatabaseCollation($dbCfg);
}

// clear system caches so we're working with current data
AppConfig::invalidate_cache('ext');
AppConfig::invalidate_cache('model');
AppConfig::invalidate_cache('modinfo');
AppConfig::invalidate_cache('lang');
AppConfig::invalidate_cache('display');
AppConfig::invalidate_cache('view');
AppConfig::invalidate_cache('acl');

$db->connect($dbCfg);

if(AppConfig::setting('install.debug.show_memory_usage'))
	$_show_memory_usage = true;
$startTime = microtime(true);
$focus = 0;
$processed_tables = array(); // for keeping track of the tables we have worked on

$empty = '';
$new_tables = 1; // is there ever a scenario where we DON'T create the admin user?

$new_config = 0;
$new_report = 1;


showMemoryUsage("Start");

$total_steps = 0;
foreach ($tasks as $task) {
	$total_steps += $task['steps'];
}

$progressIndicator->logProgress($mod_strings['LBL_PERFORM_PRE_INSTALL'], $mod_strings['LBL_DONE'], 0);

foreach ($tasks as $task_id=> $task) {
	switch ($task_id) {
	case 'create_tables': {
		/**
		 * loop through all the Beans and create their tables
		 */
		$s = 0;
		foreach($install_tables as $model_name) {
			$table = AppConfig::setting("model.detail.$model_name.table_name");
			$source = AppConfig::setting("model.detail.$model_name.module_dir", '');
			if($source)
				$source .= '/' . AppConfig::setting("model.detail.$model_name.bean_name", '');
			if(! $table)
				continue;
			
			$fields = AppConfig::setting("model.fields.$model_name");
			$indices = AppConfig::setting("model.indices.$model_name");

			$tableExists = $db->tableExists($table);
			if ($setup_db_drop_tables && $tableExists) {
				$db->dropTableName($table);
				$tableExists = false;
			}
			if(! $tableExists) {
				if($db->createTableParams($table, $fields, $indices)) {
					if($model_name == 'config')
						$new_config = 1;
				}
			}
			
			if(AppConfig::setting("model.detail.$model_name.audit_enabled")) {
				$audit_table = AppConfig::setting("model.detail.$model_name.audit_table_name");
				$auditTableExists = $db->tableExists($audit_table);
				if ($setup_db_drop_tables && $auditTableExists) {
					$db->dropTableName($audit_table);
					$auditTableExists = false;
				}
				if (!$auditTableExists) {
					if(! isset($audit_table_fields)) {
						$audit_table_fields = AppConfig::setting("model.fields.audit");
						$audit_table_indices = AppConfig::setting("model.indices.audit");
					}
					$db->createTableParams($audit_table, $audit_table_fields, $audit_table_indices);
				}
			}

			$s++;
			update_progress($task_id, $s);
		}
		
		showMemoryUsage("Created tables");

		break;
	}

	case 'create_relationships': {
		$s = 0;
		foreach($install_rels as $model_name) {
			$table = AppConfig::setting("model.detail.$model_name.table_name");
			$source = AppConfig::setting("model.detail.$model_name.module_dir", '');
			if($source)
				$source .= '/' . AppConfig::setting("model.detail.$model_name.bean_name", '');
			if(! $table)
				continue;
			
			$fields = AppConfig::setting("model.fields.$model_name");
			$indices = AppConfig::setting("model.indices.$model_name");

			$tableExists = $db->tableExists($table);
			if ($setup_db_drop_tables && $tableExists) {
				$db->dropTableName($table);
				$tableExists = false;
			}
			if(! $tableExists) {
				$db->createTableParams($table, $fields, $indices);
			}
			$s++;
			update_progress($task_id, $s);
		}
		
		showMemoryUsage("Created relationships");
		
		break;
	}

	case 'create_default_settings': {
		if($new_tables) {
			// TODO: move into optional step
			global $vendor_info;
			require_once('vendor_info.php');
			$addr = RowUpdate::blank_for_model('CompanyAddress');
			$company_info = array('name', 'address_street', 'address_city', 'address_state', 'address_postalcode', 'address_country');
			foreach($company_info as $field) {
				$val = isset($vendor_info[$field]) ? $vendor_info[$field] : '';
				$addr->set($field, $val);
			}
			$addr->set(array(
				'main' => 1,
				'is_warehouse' => 1,
				'main_warehouse' => 1,
				'assigned_user_id' => 1,
				'modified_user_id' => 1,
			));
			$addr->save();
			$vendor_info['id'] = $addr->getPrimaryKeyValue();
		}
		update_progress($task_id, 1);

		break;
	}

	case 'create_default_users': {
		if ($new_tables) {
			create_default_users();
		} else {
			set_admin_password($setup_site_admin_password);
		}

		// longreach - added - moved before demo data creation
		$current_user = new User();
		$current_user->retrieve(1);

		AppConfig::db_check_standard_records('ACLRole');
		require_once 'modules/DataPrivacy/PrivacyActivity.php';
		PrivacyActivity::create_default_role();
		
		// longreach - added
		if (!defined('inScheduler')) define('inScheduler', 1);
		include_once ('modules/Forecasts/RunScheduled.php');

		update_progress($task_id, 1);
		break;
	}

	case 'create_default_currencies': {
		AppConfig::db_check_standard_records('Currency');
		
		// longreach - start added - insert default currencies
		if ($new_tables) {
			$base_rate = 1.0;
			if ($default_currency_iso4217 != 'USD') {
				if (isset($install_currencies[$default_currency_iso4217])) {
					$base_rate = (float)$install_currencies[$default_currency_iso4217]['rate'];
				}
			}
			require_once ('modules/Currencies/Currency.php');
			foreach($install_currencies as $iso4217 => $row) {
				if ($default_currency_iso4217 == $iso4217)
					continue;
				$cur = RowUpdate::blank_for_model('Currency');
				$rate = (float)$row['rate'];
				$cur->set(array(
					'iso4217' => $iso4217,
					'name' => $row['name'],
					'symbol' => $row['symbol'],
					'conversion_rate' => $rate / $base_rate,
					'decimal_places' => $row['significant_digits'],
					'status' => 'Active',
					'created_by' => 1,
				));
				$cur->save();
			}
		}
		update_progress($task_id, 1);

		break;
	}

	case 'create_default_booking': {
		AppConfig::db_check_standard_records('TaxRate');
		AppConfig::db_check_standard_records('TaxCode');

		if ($new_tables) {
			require_once ('modules/BookingCategories/install_defaults.php');
			populateBookingCategories();
		}

		update_progress($task_id, 1);
		break;
	}

	case 'create_default_schedules': {
		if ($new_tables) {
			create_default_schedules();
		}

		update_progress($task_id, 1);
		break;
	}
	
	case 'create_default_dashboards': {
		if ($new_tables) {
			require_once('include/layout/DashletManager.php');
			$mgr = new DashletManager();
			$mgr->store_default_pages();
			
			require_once('modules/SystemMessages/utils.php');
			register_standard_messages();
		}
		
		update_progress($task_id, 1);
		break;
	}

	case 'create_default_reports': {
		if ($new_tables) {
			require_once ('modules/Reports/ReportTemplate.php');			
			$status = ReportTemplate::scan_template_dir();

			require_once('modules/Reports/Report.php');
			$created = array();
			$errored = array();
			$idx = 0;
			foreach(ReportTemplate::all_by_filename() as $tpl) {
				try {
					$report = Report::load_template($tpl, true);
				} catch(IAHInternalError $e) {
					$errored[] = $tpl->name . ': ' . $e->getMessage();
					continue;
				}
				$report->set('shared_with', 'all');
				if($report->save())
					$created[] = $report->getField('name');
				else
					$errored[] = $tpl->name . ': ' . 'Error saving report';
			}
		}
	
		update_progress($task_id, 1);
		break;
	}

	case 'create_default_folders': {
        require_once ('modules/EmailFolders/EmailFolder.php');
		EmailFolder::create_default_folders_for_user(null);

        //check and create cache folders if not exist
        $cacheManager = new CacheManager();
        $cacheManager->createFoldersStructure();

		update_progress($task_id, 1);
		break;
	}

	case 'create_default_feeds': {
		require_once ('modules/Feeds/Feed.php');
		Feed::populate_feeds();

		update_progress($task_id, 1);
		break;
	}

	case 'create_default_pdf_templates': {
		require_once ('modules/PDFTemplates/PDFTemplate.php');
		PDFTemplate::update_defaults();

		update_progress($task_id, 1);
		break;
	}

	case 'enable_portal_access': {
		if (!empty($_SESSION['setup_enable_portal'])) {
			setup_enable_portal_access();
		}
		update_progress($task_id, 1);
		break;
	}

	}
	update_progress($task_id, $mod_strings['LBL_DONE']);
}



$progressIndicator->logProgress($mod_strings['LBL_PERFORM_POST_INSTALL'], 50);

showMemoryUsage("Basic install data");

$deltaTime = microtime(true) - $startTime;

InstallerConfigParser::installLanguagePacks();

if (isset($_SESSION['INSTALLED_LANG_PACKS']) && is_array($_SESSION['INSTALLED_LANG_PACKS']) && !empty($_SESSION['INSTALLED_LANG_PACKS'])) {
    updateUpgradeHistory();
}

///////////////////////////////////////////////////////////////////////////
////	HANDLE SUGAR VERSIONS

require_once ('modules/Versions/InstallDefaultVersions.php');


} catch(IAHDatabaseError $e) {
	$progressIndicator->logMessage($e->getMessage(), 'error');
	$msg = $mod_strings['ERR_INIT_DB'];
	$progressIndicator->finish('crash', $msg);
	throw new IAHError($msg);
}


///////////////////////////////////////////////////////////////////////////////
////	SETUP DONE

$memoryUsed = '';
if (function_exists('memory_get_usage')) {
    $memoryUsed = $mod_strings['LBL_PERFORM_OUTRO_5'] . memory_get_usage() . $mod_strings['LBL_PERFORM_OUTRO_6'];
}
$fpResult = '';
$errTcpip = '';


$bottleMsg = '';

if (count($bottle) > 0) {
    foreach($bottle as $bottle_message) {
        $bottleMsg.= "{$bottle_message}\n";
    }
} else {
    $bottleMsg = $mod_strings['LBL_PERFORM_SUCCESS'];
}
$progressIndicator->logProgress($mod_strings['LBL_PERFORM_POST_INSTALL'], $mod_strings['LBL_DONE'], 100);

$progressIndicator->finish();

if(empty($_SESSION['setup_db_pop_demo_data']))
	AppConfig::lock_installer(! empty($_SESSION['setup_lock_demo_site']));

UWizard::copyInstallationPackages();

if (empty($_SESSION['setup_db_pop_demo_data'])) {
	saveInstallerConfig();
}


function update_progress($task_id, $step)
{
	static $performed = 0;

	global $total_steps, $tasks;
	global $progressIndicator;
	
	$task = $tasks[$task_id];

	$overall = null;

	if (is_numeric($step)) {
		$performed++;
		$percent = 100 * $step / $task['steps'];
		$overall = 100 * $performed / $total_steps;

		if ($percent > 100) $percent = 100;
		if ($overall > 100) $overall = 100;
	} else {
		$percent = $step;
	}

	$progressIndicator->logProgress($task['name'], $percent, $overall);
}

