<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/


if( !isset( $install_script ) || !$install_script ){
    die($mod_strings['ERR_NO_DIRECT_SCRIPT']);
}

// for keeping track of whether to enable/disable the 'Next' button
$error_found = false;

//
// Check to see if session variables are working properly
//
if(! AppConfig::test_session()) {
	$phpIniLocation = get_cfg_var("cfg_file_path");
	die("<p>{$mod_strings['LBL_SESSION_ERR_DESCRIPTION']}</p>\n".htmlspecialchars($phpIniLocation));
}


// PHP VERSION
$php_version = constant('PHP_VERSION');
$check_php_version_result = check_php_version();

$systemCheckResults = array(
);

switch($check_php_version_result) {
	case -1:
		$systemCheckResults['phpVersion'] = array('stop', $mod_strings['ERR_CHECKSYS_PHP_UNSUPPORTED_BUG'] . ' ' . $php_version . ')');
		$error_found = true;
		break;
	case 0:
		$systemCheckResults['phpVersion'] = array('stop', $mod_strings['ERR_CHECKSYS_PHP_UNSUPPORTED'] . ' ' . $php_version . ')');
		$error_found = true;
		break;
	case 1:
		$systemCheckResults['phpVersion'] = array('optional', $mod_strings['ERR_CHECKSYS_PHP_DEPRECATED'] . ' ' . $php_version . ')');
		break;
	case 2:
		$systemCheckResults['phpVersion'] = array('optional', $mod_strings['ERR_CHECKSYS_PHP_UNTESTED'] . ' ' . $php_version . ')');
		break;
	case 3:
		$systemCheckResults['phpVersion'] = array('go', $mod_strings['LBL_CHECKSYS_PHP_OK'] . ' ' . $php_version . ')');
		break;
}


if(! isset($_SESSION['setup_db_type']))
	$_SESSION['setup_db_type'] = AppConfig::setting('install.defaults.database.type', 'mysql');

$dbVersion = '';
switch($_SESSION['setup_db_type']) {
    case 'mysql':
        $db_name = $mod_strings['LBL_CHECKSYS_DB_MYSQL'];
        $dbCheck = extension_loaded('mysql') || extension_loaded('mysqli');
        break;
}

if(! empty($dbCheck)){
	$systemCheckResults['dbStatus'] = array('go', $mod_strings['LBL_CHECKSYS_OK']);
} else {
	$systemCheckResults['dbStatus'] = array('stop', $mod_strings['LBL_CHECKSYS_NOT_AVAILABLE']);
	$error_found = true;
}

// XML Parsing
if(function_exists('xml_parser_create')) {
	$systemCheckResults['xmlStatus'] = array('go', $mod_strings['LBL_CHECKSYS_OK']);
} else {
	$systemCheckResults['xmlStatus'] = array('stop', $mod_strings['LBL_CHECKSYS_NOT_AVAILABLE']);
	$error_found = true;
}

// cURL
if(function_exists('curl_init')) {
	$systemCheckResults['curlStatus'] = array('go', $mod_strings['LBL_CHECKSYS_OK']);
} else {
	$systemCheckResults['curlStatus'] = array('stop', $mod_strings['ERR_CHECKSYS_CURL']);
}

// mbstrings
if(function_exists('mb_strlen')) {
	$systemCheckResults['mbstringStatus'] = array('go', $mod_strings['LBL_CHECKSYS_OK']);
} else {
	$systemCheckResults['mbstringStatus'] = array('stop', $mod_strings['ERR_CHECKSYS_MBSTRING']);
	$error_found = true;
}

// zlib
if(function_exists('gzclose')) {
	$systemCheckResults['zlibStatus'] = array('go', $mod_strings['LBL_CHECKSYS_OK']);
} else {
	$systemCheckResults['zlibStatus'] = array('optional', $mod_strings['ERR_CHECKSYS_ZLIB']);
}

// tls
$ch = curl_init('https://www.howsmyssl.com/a/check');
curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
$data = curl_exec($ch);
curl_close($ch);

$json = json_decode($data);
$tls = str_replace('TLS ','',$json->tls_version);
$tls = floatval($tls);

if($tls < 1.2) {
	$systemCheckResults['tlsStatus'] = array('optional', $mod_strings['LBL_LOW_TLS_VERSION']);
}
else {
	$systemCheckResults['tlsStatus'] = array('go', $mod_strings['LBL_CHECKSYS_OK'].' ('.$json->tls_version.')');
}

// zip
if(function_exists('zip_open')) {
	$systemCheckResults['zipStatus'] = array('go', $mod_strings['LBL_CHECKSYS_OK']);
} else {
	$systemCheckResults['zipStatus'] = array('stop', $mod_strings['ERR_CHECKSYS_ZIP']);
	$error_found = true;
}


// openssl
if(function_exists('openssl_pkey_get_private')) {
	$systemCheckResults['opensslStatus'] = array('go', $mod_strings['LBL_CHECKSYS_OK']);
} else {
	$systemCheckResults['opensslStatus'] = array('optional', $mod_strings['ERR_CHECKSYS_OPENSSL']);
}

// imap
if(function_exists('imap_open')) {
	$systemCheckResults['imapStatus'] = array('go', $mod_strings['LBL_CHECKSYS_OK']);
} else {
	$systemCheckResults['imapStatus'] = array('optional', $mod_strings['ERR_CHECKSYS_IMAP']);
}

// soap
if(function_exists('is_soap_fault')) {
	$systemCheckResults['soapStatus'] = array('go', $mod_strings['LBL_CHECKSYS_OK']);
} else {
	$systemCheckResults['soapStatus'] = array('optional', $mod_strings['ERR_CHECKSYS_SOAP']);
}

// longreach - added - DOM for PDFprint
if(class_exists("DOMDocument")) {
	$systemCheckResults['domStatus'] = array('go', $mod_strings['LBL_CHECKSYS_OK']);
} else {
	$systemCheckResults['domStatus'] = array('stop', $mod_strings['ERR_CHECKSYS_DOM']);
	$error_found = true;
}

// GD
if(function_exists('gd_info')) {
	if(! function_exists('imageftbbox'))
		$systemCheckResults['gdStatus'] = array('optional', $mod_strings['ERR_CHECKSYS_GD_FT']);
	else
		$systemCheckResults['gdStatus'] = array('go', $mod_strings['LBL_CHECKSYS_OK']);
} else {
	$systemCheckResults['gdStatus'] = array('optional', $mod_strings['ERR_CHECKSYS_GD']);
}

// config.php
$cfg_path = AppConfig::local_config_path();
if(make_writable($cfg_path)) {
	$systemCheckResults['configStatus'] = array('go', $mod_strings['LBL_CHECKSYS_OK']);
} else {
	$systemCheckResults['configStatus'] = array('stop', $mod_strings['ERR_CHECKSYS_NOT_WRITABLE']);
	$error_found = true;
}

// custom dir
if(make_writable('./custom', 'dir')) {
	$systemCheckResults['customStatus'] = array('go', $mod_strings['LBL_CHECKSYS_OK']);
} else {
	$systemCheckResults['customStatus'] = array('stop', $mod_strings['ERR_CHECKSYS_NOT_WRITABLE']);
	$error_found = true;
}

$check_paths = array(
	'site.paths.cache_dir',
	'site.paths.upload_dir',
	'site.paths.pdf_dir',
	'site.paths.xml_dir',
);

$systemCheckResults['cacheStatus'] = array('go', $mod_strings['LBL_CHECKSYS_OK']);
foreach($check_paths as $p) {
	$check_path = AppConfig::setting($p);
	if(! make_writable($check_path, 'dir')) {
		$systemCheckResults['cacheStatus'] = array('stop', $mod_strings['ERR_CHECKSYS_NOT_WRITABLE']);
		$error_found = true;
	}
}

// memory limit
$memory_msg     = "";
// CL - fix for 9183 (if memory_limit is enabled we will honor it and check it; otherwise use unlimited)
$memory_limit = ini_get('memory_limit');
if(empty($memory_limit)){
	$memory_limit = "-1";
}
$sugarMinMem = AppConfig::setting('install.memory_minimum', 120);
// logic based on: http://us2.php.net/manual/en/ini.core.php#ini.memory-limit
if( $memory_limit == "" ){          // memory_limit disabled at compile time, no memory limit
	$systemCheckResults['memory_msg'] = array('go', $mod_strings['LBL_CHECKSYS_MEM_OK']);
} elseif( $memory_limit == "-1" ){   // memory_limit enabled, but set to unlimited
	$systemCheckResults['memory_msg'] = array('go', $mod_strings['LBL_CHECKSYS_MEM_UNLIMITED']);
} else {
	$mem_display = $memory_limit;
    rtrim($memory_limit, 'M');
    $memory_limit_int = (int) $memory_limit;
    if( $memory_limit_int < $sugarMinMem){
        $memory_msg = "{$mod_strings['ERR_CHECKSYS_MEM_LIMIT_1']}{$sugarMinMem}{$mod_strings['ERR_CHECKSYS_MEM_LIMIT_2']}";
        $memory_msg = str_replace('$memory_limit', $mem_display, $memory_msg);
		$systemCheckResults['memory_msg'] = array('stop', $memory_msg);
    } else {
		$systemCheckResults['memory_msg'] = array('go', "{$mod_strings['LBL_CHECKSYS_OK']} ({$memory_limit})");
    }
}

// external cache
$cache_type = sugar_cache_type();
if($cache_type) {
	$systemCheckResults['memCacheStatus'] = array('go', "{$mod_strings['LBL_CHECKSYS_OK']} ($cache_type)");
} else {
	$systemCheckResults['memCacheStatus'] = array('optional', $mod_strings['ERR_CHECKSYS_EXT_CACHE']);
}

$Components = array(
	$mod_strings['LBL_CHECKSYS_COMPONENT'] => array(
		array($mod_strings['LBL_CHECKSYS_PHPVER'], 'phpVersion'),
		array($db_name, 'dbStatus'),
		array($mod_strings['LBL_CHECKSYS_MBSTRING'], 'mbstringStatus'),
		array($mod_strings['LBL_CHECKSYS_XML'], 'xmlStatus'),
		array($mod_strings['LBL_CHECKSYS_DOM'], 'domStatus'),
		array($mod_strings['LBL_CHECKSYS_ZIP'], 'zipStatus'),
		array($mod_strings['LBL_CHECKSYS_SOAP'], 'soapStatus'),
		//array($mod_strings['LBL_CHECKSYS_SAFE_MODE'], 'safeModeStatus'),
		array("{$mod_strings['LBL_CHECKSYS_MEM']} {$sugarMinMem}M", 'memory_msg'),
		array("{$mod_strings['LBL_CHECKSYS_CONFIG']} ($cfg_path)", 'configStatus'),
		array($mod_strings['LBL_CHECKSYS_CUSTOM'], 'customStatus'),
		array($mod_strings['LBL_CHECKSYS_CACHE'], 'cacheStatus'),
		//array("{$mod_strings['LBL_CHECKSYS_SESSION']}{$session_save_path})", 'sessionStatus'),
	),
	$mod_strings['LBL_CHECKSYS_COMPONENT_OPTIONAL'] => array(
		array($mod_strings['LBL_CHECKSYS_EXT_CACHE'], 'memCacheStatus'),
		array($mod_strings['LBL_CHECKSYS_GD'], 'gdStatus'),
		array($mod_strings['LBL_CHECKSYS_IMAP'], 'imapStatus'),
		array($mod_strings['LBL_CHECKSYS_OPENSSL'], 'opensslStatus'),
		array($mod_strings['LBL_CHECKSYS_CURL'], 'curlStatus'),
		array($mod_strings['LBL_CHECKSYS_ZLIB'], 'zlibStatus'),
		array($mod_strings['LBL_TLS_VERSION'], 'tlsStatus'),
	),
);


if (AUTO_INSTALLER) {
	$systemCheckErrors = array();
	foreach ($Components as $SectionTitle => $Comps) {
		foreach ($Comps as $Comp) {
			list($CompName, $CompStatusName) = $Comp;
			if ($systemCheckResults[$CompStatusName][0] == 'stop') 
				$systemCheckErrors[] = $CompName . ' : ' . $systemCheckResults[$CompStatusName][1];
		}
	}
	return;
}

// PHP.ini
$phpIniLocation = get_cfg_var("cfg_file_path");

// disable form if error found
$disabled = $error_found ? 'disabled="disabled"' : '';

// longreach - start added - template support
require_once('XTemplate/xtpl.php');
$xtpl = new XTemplate('install/template.html');

installer_init_template($xtpl);
$xtpl->assign('PAGE_TITLE', $mod_strings['LBL_WIZARD_TITLE'] .  $next_step);
$xtpl->assign('HEADER_TITLE', $mod_strings['LBL_STEP'] . $next_step . ': ' . $mod_strings['LBL_CHECKSYS_TITLE']);
$xtpl->assign('JAVASCRIPT', '');
$xtpl->assign('ONLOAD', "disableButtons();");
$xtpl->parse('head.start_form');
$xtpl->parse('head.row');
$xtpl->parse('head');
$out = $xtpl->text('head');
// longreach - end added

// longreach - modified
$out .= <<<EOQ
<div class="row page-row">
	<div class="column page-column">
        <p>{$mod_strings['LBL_CHECKSYS_1']}</p>

		<table cellpadding="0" cellspacing="0" border="0" width="100%" class="install-summary system-checks">
EOQ;

foreach ($Components as $SectionTitle => $Comps) {
	$out .= <<<EOQ
		<thead>
          <tr>
            <th width="60%">{$SectionTitle}</th>
            <th width="40%" class="status-td">{$mod_strings['LBL_CHECKSYS_STATUS']}</th>
          </tr>
        </thead>
        <tbody>
EOQ;
	foreach ($Comps as $Comp) {
		list($CompName, $CompStatusName) = $Comp;
		$out .= <<<EOQ
          <tr>
            <td>{$CompName}</td>
			<td class="status-td"><span class="{$systemCheckResults[$CompStatusName][0]}">{$systemCheckResults[$CompStatusName][1]}</span></td>
		  </tr>
EOQ;
	}
	$out .= '<tr class="spacer"><td colspan="2">&nbsp;</td></tr></tbody>';
}

$out .= <<<EOQ
        </table>

        <div align="center" style="margin: 5px;">
          <i>{$mod_strings['LBL_CHECKSYS_PHP_INI']}<br>{$phpIniLocation}</i>
        </div>
    </div>
</div>
EOQ;


// longreach - start added
$buttons = array(
	array('TITLE' => $mod_strings['LBL_CHECKSYS_RECHECK'], 'NAME' => 'Re-check', 'TYPE' => 'button',
		'ONCLICK' => "this.form.goto.value='Re-check'; this.form.submit();", 'ICON' => 'refresh'),
	array('TITLE' => $mod_strings['LBL_BACK'], 'NAME' => 'Back', 'TYPE' => 'button',
		'ONCLICK' => "this.form.goto.value='Back'; this.form.submit();", 'ICON' => 'arrow-left'),
	array('TITLE' => $mod_strings['LBL_NEXT'], 'NAME' => 'Next', 'TYPE' => 'submit', 'ID'=> 'defaultFocus',
		'ONCLICK' => "this.form.goto.value='Next';", 'ICON' => 'arrow-right', 'CLASS' => 'lwib-accept',
		'ATTRS' => ($error_found ? 'disabled="disabled"' : '')),
	);
installer_show_buttons($xtpl, $buttons, $next_step);
$xtpl->parse('foot');
$out .= $xtpl->text('foot');

// longreach - end added
echo $out;

////	END PAGEOUTPUT
///////////////////////////////////////////////////////////////////////////////
?>
