<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/
/*********************************************************************************

 * $Description: TODO: To be written. Portions created by SugarCRM are Copyright
 * (C) SugarCRM, Inc. All Rights Reserved. Contributor(s):
 * ______________________________________..
 * *******************************************************************************/

require_once('include/utils/zip_utils.php');
require_once('include/utils/file_utils.php');
require_once('include/upload_file.php');
require_once('include/dir_inc.php');

///////////////////////////////////////////////////////////////////////////////
////	FROM welcome.php


/**
 * creates UpgradeHistory entries
 * @param mode string Install or Uninstall
 */
function updateUpgradeHistory() {
	require_once('modules/Administration/UpgradeHistory.php');
	if(isset($_SESSION['INSTALLED_LANG_PACKS']) && is_countable($_SESSION['INSTALLED_LANG_PACKS']) && count($_SESSION['INSTALLED_LANG_PACKS']) > 0) {
		foreach($_SESSION['INSTALLED_LANG_PACKS'] as $k => $zipFile) {
		    $new_upgrade = new UpgradeHistory();
		    $new_upgrade->filename      = $zipFile;
		    $new_upgrade->md5sum        = md5_file($zipFile);
		    $new_upgrade->type          = 'langpack';
		    $new_upgrade->version       = $_SESSION['INSTALLED_LANG_PACKS_VERSION'][$k];
		    $new_upgrade->status        = "installed";
		    $new_upgrade->manifest   	= $_SESSION['INSTALLED_LANG_PACKS_MANIFEST'][$k];
		    $new_upgrade->save();
		}
	}
}


/**
 * retrieves the name of the language
 */
function getLanguagePackName($the_file) {
    require_once( "$the_file" );
    if( isset( $app_list_strings["language_pack_name"] ) ){
        return( $app_list_strings["language_pack_name"] );
    }
    return( "" );
}


/**
 * takes session vars and creates config.php
 * @return array bottle collection of error messages
 */
function handleLocalConfig() {
	global $bottle;
	global $mod_strings;
	global $setup_site_host_name;
	global $setup_site_log_dir;
	global $setup_site_log_file;
	global $setup_site_session_path;
	global $setup_site_guid;
	global $setup_site_url;
	global $setup_sugar_version;
	global $progressIndicator;
	$unique_key = $setup_site_guid;
	if(empty($unique_key))
		$unique_key = md5(create_guid());

	$bottle = [];

	$pubkey = $privkey = '';
	if(function_exists('openssl_pkey_get_private')) {
		$ssl_config = array(
			"digest_alg" => "sha512",
			"private_key_bits" => 4096,
			"private_key_type" => OPENSSL_KEYTYPE_RSA,
		);
		if( ($cfg_path = getenv('OPENSSL_CONF')) ) {
			$ssl_config["config"] = $cfg_path;
		}
		while($msg = openssl_error_string())
			; // clear any previous errors
		$res = openssl_pkey_new($ssl_config);
		if($res) {
			openssl_pkey_export($res, $privkey);
			$pubkey = openssl_pkey_get_details($res);
			$pubkey = $pubkey["key"];
		} else {
			$openssl_err = [];
			while($msg = openssl_error_string())
				$openssl_err[] = $msg;
			if(isset($progressIndicator))
				$progressIndicator->logMessage('OpenSSL: Error creating private key');
			$GLOBALS['log']->error("OpenSSL: Error creating private key: ".implode("\n", $openssl_err));
		}
	}


	$updates = array(
		'config.unique_key' => $unique_key,
		'database.primary' => getDatabaseConfig(),
		'site.install_date' => date('Y-m-d H:i:s'),
		'site.base_url' => $_SESSION['setup_site_url'],
		'site.host_name' => $setup_site_host_name,
		'site.timezone' => $_SESSION['setup_server_timezone'],
		'locale.defaults.date_format' => $_SESSION['default_date_format'],
		'locale.defaults.time_format' => $_SESSION['default_time_format'],
		'locale.defaults.language' => $_SESSION['default_language'],
		'locale.defaults.name_format' => $_SESSION['default_locale_name_format'],
		'locale.defaults.number_format' => $_SESSION['default_number_format'],
		'locale.defaults.address_format' => $_SESSION['default_address_format'],
        'locale.base_currency.name' => $_SESSION['default_currency_name'],
        'locale.base_currency.symbol' => $_SESSION['default_currency_symbol'],
        'locale.base_currency.iso4217' => $_SESSION['default_currency_iso4217'],
        'locale.base_currency.significant_digits' => $_SESSION['default_currency_significant_digits'],
		'email.default_charset' => $_SESSION['default_email_charset'],
		'export.charset' => $_SESSION['default_export_charset'],
		'export.list_format' => $_SESSION['default_export_format'],
		'oauth.private_key' => $privkey,
		'oauth.public_key' => $pubkey,
	);
	
	if (!empty($_SESSION['setup_lock_demo_site'])) {
		$updates['site.security.enforce_login_delay'] = false;
	}
	
	if(! empty($_SESSION['setup_site_custom_log'])) {
		$updates['site.log.dir'] = $setup_site_log_dir;
		$updates['site.log.file'] = $setup_site_log_file;
	}
	if(! empty($_SESSION['setup_site_custom_session_path'])) {
		$updates['site.session.dir'] = $setup_site_session_path;
	}

	AppConfig::update_local($updates);

	if(AppConfig::save_local()) {
		// was 'Done'
	} else {
		$bottle[] = $mod_strings['ERR_PERFORM_CONFIG_PHP_4'];
	}
	
	return $bottle;	
}


/**
 * (re)write the .htaccess file to prevent browser access to the log file
 */
function handleHtaccess() {
	global $mod_strings;
	require_once('modules/Administration/upgrade_utils.php');
	$update = update_htaccess();
	if( $update['failed'] ){
		echo '<p>' . $mod_strings['ERR_PERFORM_HTACCESS_1'] . '<span class=stop>' . htmlspecialchars($update['file']) . '</span> ' . $mod_strings['ERR_PERFORM_HTACCESS_2'] . "</p>\n";
		echo '<p>' . $mod_strings['ERR_PERFORM_HTACCESS_3'] . "</p>\n";
		echo '<textarea class="input-textarea" cols="100" rows="20">' . htmlspecialchars($update['contents']) . '</textarea><br>';
	}
	$GLOBALS['log']->info("Updated .htaccess file");
}


function create_default_users(){
    global $setup_site_admin_password;
	
	$exist = ListQuery::quick_fetch('User', 1);
	if(! $exist) {
		$new_pass = (empty($_SESSION['setup_lock_demo_site'])
			&& $setup_site_admin_password == 'admin') ? 1 : 0;
		//Create default admin user
    	$user = RowUpdate::blank_for_model('User');
		$user->set(array(
			'id' => 1,
			'last_name' => 'Administrator',
			'user_name' => 'admin',
			'title' => 'Administrator',
			'status' => 'Active',
			'is_admin' => true,
			'user_hash' => strtolower(md5($setup_site_admin_password)),
			'email' => '', // FIXME - add admin email to installer settings
			'modified_user_id' => 1,
			'need_new_password' => $new_pass,
		));
		$user->save();
		$GLOBALS['log']->info("Created admin user, ID ". $user->getPrimaryKeyValue());

		// echo 'Creating RSS Feeds';
		/*require_once('modules/Feeds/Feed.php');
		$feed = new Feed();
		$feed->createRSSHomePage($user->id);*/
	}


    // We need to change the admin user to a fixed id of 1.
    // $query = "update users set id='1' where user_name='$user->user_name'";
    // $result = $db->query($query, true, "Error updating admin user ID: ");

	// FIXME - check install.default_user.create, use install.default_user.*
    /*if( $create_default_user ){
        $default_user = new User();
        $default_user->last_name = $sugar_config['default_user_name'];
        $default_user->user_name = $sugar_config['default_user_name'];
        $default_user->status = 'Active';
        if( isset($sugar_config['default_user_is_admin']) && $sugar_config['default_user_is_admin'] ){
            $default_user->is_admin = true;
        }
        //$default_user->user_password = $default_user->encrypt_password($sugar_config['default_password']);
		$default_user->user_hash = strtolower(md5($sugar_config['default_password']));
        $default_user->save();
        $feed->createRSSHomePage($user->id);
    }*/
}

function set_admin_password( $password ) {
	$user = ListQuery::quick_fetch('User', 1);
	if($user) {
		$upd = RowUpdate::for_result($user);
		$upd->set('user_hash', strtolower(md5($password)));
		$upd->save();
	}
}


function main_vendor_id()
{
	global $db;
	static $id = null;
	if ($id === null) {
		$query = "SELECT id FROM company_addresses WHERE main = 1";
		$res = $db->query($query, true);
		if ($row = $db->fetchByAssoc($res)) {
			$id = $row['id'];
		} else {
			$id = '';
		}
	}
	return $id;
}


// Returns true if the given file/dir has been made writable (or is already
// writable).
function make_writable($file, $type='file', $no_create=false)
{
	$ret_val = false;
	if(($type == 'file' && is_file($file)) || ($type == 'dir' && is_dir($file)))
	{
		if(is_writable($file))
			return true;
		else {
			$original_fileperms = fileperms($file);

			// add user writable permission
			$new_fileperms = $original_fileperms | 0x0080;
			@chmod($file, $new_fileperms);

			if(is_writable($file))
				return true;
			else {
				// add group writable permission
				$new_fileperms = $original_fileperms | 0x0010;
				@chmod($file, $new_fileperms);

				if(is_writable($file))
					return true;
				else {
					// add world writable permission
					$new_fileperms = $original_fileperms | 0x0002;
					@chmod($file, $new_fileperms);

					if(is_writable($file))
					{
						return true;
					}
				}
			}
		}
	}
	else if(! $no_create) {
		if($type == 'dir') {
			if(@ mkdir($file, $mode=0660, true))
				return true;
		}
		else if($type == 'file') {
			if(@ touch($file))
				return make_writable($file, 'file', true);
		}
	}
	return false;
}

function recursive_make_writable($start_file)
{
	$ret_val = make_writable($start_file);

	if($ret_val && is_dir($start_file))
	{
		// PHP 4 alternative to scandir()
		$files = array();
		$dh = opendir($start_file);
		$filename = readdir($dh);
		while(!empty($filename))
		{
			if($filename != '.' && $filename != '..')
			{
				$files[] = $filename;
			}

			$filename = readdir($dh);
		}

		foreach($files as $file)
		{
			$ret_val = recursive_make_writable($start_file . '/' . $file);

			if(!$ret_val)
			{
				break;
			}
		}
	}

	return $ret_val;
}

function recursive_is_writable($start_file)
{
	$ret_val = is_writable($start_file);

	if($ret_val && is_dir($start_file))
	{
		// PHP 4 alternative to scandir()
		$files = array();
		$dh = opendir($start_file);
		$filename = readdir($dh);
		while(!empty($filename))
		{
			if($filename != '.' && $filename != '..')
			{
				$files[] = $filename;
			}

			$filename = readdir($dh);
		}

		foreach($files as $file)
		{
			$ret_val = recursive_is_writable($start_file . '/' . $file);

			if(!$ret_val)
			{
				break;
			}
		}
	}

	return $ret_val;
}


// one place for form validation/conversion to boolean
function get_boolean_from_request( $field ){
    if( !isset($_REQUEST[$field]) ){
        return( false );
    }

    if( ($_REQUEST[$field] == 'on') || ($_REQUEST[$field] == 'yes') ){
        return(true);
    }
    else {
        return(false);
    }
}

function stripslashes_checkstrings($value){
   if(is_string($value)){
      return stripslashes($value);
   }
   return $value;
}


function print_debug_array( $name, $debug_array ){
    ksort( $debug_array );

    print( "$name vars:\n" );
    print( "(\n" );

    foreach( $debug_array as $key => $value ){
        if( stristr( $key, "password" ) ){
            $value = "WAS SET";
        }
        print( "    [$key] => $value\n" );
    }

    print( ")\n" );
}


function validate_localization() {
	global $mod_strings;
	
	$errors = array();
	
	foreach($_REQUEST as $k => $v) {
		if($v == "'") {
			$errors[] = "<span class='error'>{$mod_strings['ERR_NO_SINGLE_QUOTE']}{$k}</span>";
		}
	}
	return $errors;		
}


function getDatabaseConfig($for_setup=false) {
	$db_spec = array(
		'type' => $_SESSION['setup_db_type'],
		'host_name' => $_SESSION['setup_db_host_name'],
		'name' => $_SESSION['setup_db_database_name'],
		'user_name' => $_SESSION['setup_db_sugarsales_user'],
		'password' => $_SESSION['setup_db_sugarsales_password'],
	);
	if($for_setup) {
		$db_spec['create_user'] = ! empty($_SESSION['setup_db_create_sugarsales_user']);
		$db_spec['create_database'] = ! empty($_SESSION['setup_db_create_database']);
		if(! empty($_SESSION['setup_db_username_is_privileged']) && empty($db_spec['create_user'])) {
			$db_spec['admin_user'] = $db_spec['user_name'];
			$db_spec['admin_password'] = $db_spec['password'];
		} else {
			$db_spec['admin_user'] = $_SESSION['setup_db_admin_user_name'];
			$db_spec['admin_password'] = $_SESSION['setup_db_admin_password'];
		}
		$db_spec['password2'] = array_get_default($_SESSION, 'setup_db_sugarsales_password_retype');
	}
	return $db_spec;
}


function validate_dbConfig() {
	global $mod_strings;
    $errors = array();
    
    require_once('include/database/DBManager.php');
    $db_spec = getDatabaseConfig(true);
    
    do {
		if(! strlen($db_spec['name']))
			$errors[] = $mod_strings['ERR_DB_NAME'];
		if(! strlen($db_spec['user_name']))
			$errors[] = $mod_strings['ERR_DB_USER'];
		if(! empty($db_spec['create_user']) && isset($db_spec['password2']) && $db_spec['password2'] != $db_spec['password'])
			$errors[] = $mod_strings['ERR_DB_PASSWORD'];
		try {
			$mgr_cls = DBManagerFactory::getManagerFromSpec($db_spec);
		} catch(IAHDatabaseError $e) {
			$errors[] = $e->getMessage(); // not translated
		}
		if($errors) break;

		$mgr = new $mgr_cls();
		$err_msg = $mgr->validateConfig($db_spec);
		switch($err_msg) {
			case 'err_db_name':
				$errors[] = $mod_strings['ERR_DB_NAME2'];
				break;
			case 'err_admin_req':
				$errors[] = $mod_strings['ERR_DB_PRIV_USER'];
				break;
			case 'err_db_exists':
				$errors[] = $mod_strings['ERR_DB_EXISTS'];
				break;
			case 'err_db_exists_not':
				$errors[] = $mod_strings['ERR_DB_EXISTS_NOT'];
				break;
			case 'err_user_exists':
				$errors[] = $mod_strings['ERR_DB_USER_EXISTS'];
				break;
			case 'err_old_version':
				$min_ver = MysqlManager::$min_server_version;
				$errors[] = str_replace('{VERSION}', $min_ver, $mod_strings['ERR_DB_MYSQL_VERSION1'].$mgr->getServerVersion().$mod_strings['ERR_DB_MYSQL_VERSION2']);
				break;
			case 'ok':
				break;
			default:
				$errors[] = $mod_strings['ERR_DB_ADMIN'] . $mgr->getLastError() . ')';
				break;
		}
		$mgr->disconnect();
	} while(0);

    return $errors;
}


function validate_siteConfig(){
	global $mod_strings;
   $errors = array();
   if($_SESSION['setup_site_url'] == ''){
      $errors[] = $mod_strings['ERR_URL_BLANK'];
   }

   if($_SESSION['setup_site_admin_password'] == ''){
      $errors[] = $mod_strings['ERR_ADMIN_PASS_BLANK'];
   }

   if($_SESSION['setup_site_admin_password'] != $_SESSION['setup_site_admin_password_retype']){
      $errors[] = $mod_strings['ERR_PASSWORD_MISMATCH'];
   }

   if(!empty($_SESSION['setup_site_custom_session_path']) && $_SESSION['setup_site_session_path'] == ''){
      $errors[] = $mod_strings['ERR_SESSION_PATH'];
   }

   if(!empty($_SESSION['setup_site_custom_session_path']) && $_SESSION['setup_site_session_path'] != ''){
      if(is_dir($_SESSION['setup_site_session_path'])){
         if(!is_writable($_SESSION['setup_site_session_path'])){
            $errors[] = $mod_strings['ERR_SESSION_DIRECTORY'];
         }
      }
      else {
         $errors[] = $mod_strings['ERR_SESSION_DIRECTORY_NOT_EXISTS'];
      }
   }

   if(!empty($_SESSION['setup_site_custom_log_dir']) && $_SESSION['setup_site_log_dir'] == ''){
      $errors[] = $mod_strings['ERR_LOG_DIRECTORY_NOT_EXISTS'];
   }

   if(!empty($_SESSION['setup_site_custom_log_dir']) && $_SESSION['setup_site_log_dir'] != ''){
      if(is_dir($_SESSION['setup_site_log_dir'])){
         if(!is_writable($_SESSION['setup_site_log_dir'])) {
            $errors[] = $mod_strings['ERR_LOG_DIRECTORY_NOT_WRITABLE'];
         }
      }
      else {
         $errors[] = $mod_strings['ERR_LOG_DIRECTORY_NOT_EXISTS'];
      }
   }

   if(!empty($_SESSION['setup_site_specify_guid']) && $_SESSION['setup_site_guid'] == ''){
      $errors[] = $mod_strings['ERR_SITE_GUID'];
   }

   return $errors;
}

/**
 * creates the remove/delete form for langpack page
 * @param string type commit/remove
 * @param string manifest path to manifest file
 * @param string zipFile path to uploaded zip file
 * @param int nextstep current step
 * @return string ret <form> for this package
 */
function getPackButton($type, $manifest, $zipFile, $next_step, $uninstallable='Yes', $showButtons=true) {
    global $mod_strings;
    
    $button = $mod_strings['LBL_LANG_BUTTON_COMMIT'];
    if($type == 'remove') {
        $button = $mod_strings['LBL_LANG_BUTTON_REMOVE'];
    } elseif($type == 'uninstall') {
    	$button = $mod_strings['LBL_LANG_BUTTON_UNINSTALL'];
    }
    
    $disabled = ($uninstallable == 'Yes') ? false : true;
    
    $ret = "<form name='delete{$zipFile}' action='install.php' method='POST'>
                <input type='hidden' name='current_step' value='{$next_step}'>
                <input type='hidden' name='goto' value='{$mod_strings['LBL_CHECKSYS_RECHECK']}'>
                <input type='hidden' name='languagePackAction' value='{$type}'>
                <input type='hidden' name='manifest' value='".urlencode($manifest)."'>
                <input type='hidden' name='zipFile' value='".urlencode($zipFile)."'>";
    if(!$disabled && $showButtons) {
		$ret .=	"<input type='submit' value='{$button}'>";
    }
    $ret .= "</form>";
    return $ret;
}

/**
 * finds all installed languages and returns an array with the names
 * @return array langs array of installed languages
 */
function getInstalledLanguages() {
	$langDir = 'include/language/';
	$dh = opendir($langDir);
	
	$langs = array();
	while($file = readdir($dh)) {
		if(substr($file, -3) == 'php') {
			
		}
	}
}


function extractFile( $zip_file, $file_in_zip, $base_tmp_upgrade_dir){
    $my_zip_dir = mk_temp_dir( $base_tmp_upgrade_dir );
    unzip_file( $zip_file, $file_in_zip, $my_zip_dir );
    return( "$my_zip_dir/$file_in_zip" );
}

function extractManifest( $zip_file,$base_tmp_upgrade_dir ) {
    return( extractFile( $zip_file, "manifest.php",$base_tmp_upgrade_dir ) );
}


function getLicenseContents($filename)
{
	$fh = fopen( $filename, 'r' ) or die( "License file not found!" );
	$license_file = fread( $fh, filesize( $filename ) );
	fclose( $fh );
	
	return $license_file;
}



///////////////////////////////////////////////////////////////////////////////
////	FROM POPULATE SEED DATA
$seed = array(
	'qa', 		'dev',			'code',
	'pro',		'sales',		'support',
	'pal',		'my',			'depot',
	'crbm', 	'hr',			'mobile',
	'books',	'paleo',		'fitness',
);
$tlds = array(
	".com", ".org", ".net",
	".edu", ".info", ".biz",
	".ca", ".com.au", ".us",
	".eu", ".es", ".de", ".it", ".co.uk",
	".cn", ".co.jp", 
);

/**
 * creates a random, DNS-clean webaddress
 */
function createWebAddress() {
	global $seed;
	global $tlds;
	$seedCount = is_countable($seed) ? count($seed) : 0;
	$tldsCount = is_countable($tlds) ? count($tlds) : 0;
	
	$one = $seed[rand(0, $seedCount - 1)];
	$two = $seed[rand(0, $seedCount - 1)];
	$tld = $tlds[rand(0, $tldsCount - 1)];
	
	return "http://www.{$one}{$two}{$tld}";
}

/**
 * creates a random email address
 * @return string
 */
function createEmailAddress() {
	global $seed;
	global $tlds;
	$seedCount = is_countable($seed) ? count($seed) : 0;
	
	$part[0] = $seed[rand(0, $seedCount - 1)];
	$part[1] = $seed[rand(0, $seedCount - 1)];
	$part[2] = $seed[rand(0, $seedCount - 1)];

	$tldsCount = is_countable($tlds) ? count($tlds) : 0;
	
	$tld = $tlds[rand(0, $tldsCount - 1)];
	
	$len = rand(1,3);
	
	$ret = '';
	for($i=0; $i<$len; $i++) {
		$ret .= (empty($ret)) ? '' : '.'; 
		$ret .= $part[$i];
	}
	
	if($len == 1) {
		$ret .= rand(10, 99); 
	}

	return "{$ret}@example{$tld}";
}


function add_digits($quantity, &$string, $min = 0, $max = 9) {
	for($i=0; $i < $quantity; $i++) {
		$string .= mt_rand($min,$max);
	}
}

function create_phone_number() {
	$phone = "+1 (";
	add_digits(3, $phone);
	$phone .= ") ";
	add_digits(3, $phone);
	$phone .= "-";
	add_digits(4, $phone);

	return $phone;
}

function create_date($db=true) {
	global $timedate;
	list($m,$d,$y) = explode(' ', date('m d Y'));
	$fmt = $db ? $timedate->get_db_date_format() : $timedate->get_date_format();
	$date = date($fmt, mktime(0, 0, 0, $m, $d+mt_rand(0,365), $y));
	return $date;
}

function create_time($db=true) {
	global $timedate;
	$tm = mktime(mt_rand(6,19), mt_rand(0,3)*15, 0);
	$fmt = $db ? $timedate->get_db_time_format() : $timedate->get_time_format();
	$tm = date($timedate->get_time_format(), $tm);
	return $tm;
}

function create_past_date($db=true) {
	global $timedate;
	list($m,$d,$y) = explode(' ', date('m d Y'));
	$fmt = $db ? $timedate->get_db_date_format() : $timedate->get_date_format();
	$date = date($fmt, mktime(0, 0, 0, $m, $d+mt_rand(-365,-1), $y));
	return $date;
}

function create_date_time($db=true) {
	global $timedate;
	list($m,$d,$y) = explode(' ', date('m d Y'));
	$tm = mktime(mt_rand(6,19), mt_rand(0,3)*15, 0, $m, $d+mt_rand(0, 365), $y);
	$fmt = $db ? $timedate->get_db_date_time_format() : $timedate->get_date_time_format();
	$date = date($fmt, $tm);
	return $date;
}

function create_past_date_time($db=true) {
	global $timedate;
	list($m,$d,$y) = explode(' ', date('m d Y'));
	$tm = mktime(mt_rand(6,19), mt_rand(0,3)*15, 0, $m, $d+mt_rand(-365,-1), $y);
	$fmt = $db ? $timedate->get_db_date_time_format() : $timedate->get_date_time_format();
	$date = date($fmt, $tm);
	return $date;
}


// longreach - start added
function create_default_schedules()
{
	require_once('modules/Scheduler/utils.php');
	scheduler_rebuild_tasks(true);
}

function installRetrieveBean($cls, $id, $reload=false) {
	static $cache;
	if(! isset($cache)) $cache = array();
	if(! isset($cache[$cls])) $cache[$cls] = array();
	if(! isset($cache[$cls][$id]) || $reload) {
		$cache[$cls][$id] = new $cls();
		if(! $cache[$cls][$id]->retrieve($id))
			return null;
	}
	return $cache[$cls][$id];
}

function &installBean($class, $name, $extra = array())
{
	static $map = array();
	if (!isset($map[$class][$name])) {
		$bean = new $class;
		$bean->name = $name;
		foreach ($extra as $k => $v) {
			$bean->$k = $v;
		}
		if (!empty($bean->id))
			$bean->new_with_id = true;
		$bean->save();
		$bean->retrieve($bean->id, false);
		$map[$class][$name] =& $bean;
	}
	return $map[$class][$name];
}

function showMemoryUsage($stage='') {
	if(! empty($GLOBALS['_show_memory_usage'])) {
		if($stage) $stage .= ' - ';
		echo '<p>'.$stage.'Memory: ' . number_format(memory_get_usage(), 0, '.', ',') . " bytes";
		if(function_exists('memory_get_peak_usage'))
			echo ', Peak: ' . number_format(memory_get_peak_usage(), 0, '.', ',') . " bytes";
		echo '</p>';
	}
	flush();
}

function installOpenCSV($filename)
{
	$fh = fopen($filename, 'r');
	$header = fgetcsv($fh);
	return array($fh, $header);
}

function installCloseCSV($handle)
{
	fclose($handle[0]);
}

function installGetCSV($handle)
{
	$row = fgetcsv($handle[0]);
	if ($row === false) {
		return false;
	}
	$ret = array();
	foreach ($handle[1] as $idx => $name) {
		$ret[$name] = @$row[$idx];
	}
	return $ret;
}

function relToDocRoot($path)
{
	$doc_root_reg = '/^' . preg_quote($_SERVER['DOCUMENT_ROOT'], '/') . '/';
	return preg_replace($doc_root_reg,  '', realpath($path));
}

function array_get_random($array)
{
	if(is_array($array) && count($array) > 0) {
		$key = array_rand($array);
		if (isset($array[$key])) return $array[$key];
	}
	return null;
}

function check_install_permissions(&$reason) {
	/*$lconfig = AppConfig::local_config_path();
	if((file_exists($lconfig) && ! is_writable($lconfig))
	|| (! file_exists($lconfig) && ! is_writable(dirname($lconfig)))) {
		$reason = 'config';
		return false;
	}*/
	if(AppConfig::setting('config.installer_locked')) {
		$reason = 'locked';
		return false;
	}
	$check_dir = AppConfig::cache_dir() . 'system';
	if(! is_writable($check_dir)) {
		$reason = 'cachedir';
		return false;
	}
	if(! AppConfig::test_session()) {
		$reason = 'session';
		return false;
	}
	return true;
}


function installer_load_supported_languages($reload=false) {
	global $supportedLanguages;
	if(isset($_SESSION['supportedLanguages']) && ! $reload)
		$supportedLanguages = $_SESSION['supportedLanguages'];
	$ret = array();
	$langs = glob("install/language/lang.*.meta.php", GLOB_NOSORT);
	if($langs) {
		foreach($langs as $l) {
			$parts = explode('.', $l);
			if(count($parts) != 4) continue;
			$name = $parts[1];
			if(! is_readable($l) || ! is_readable("install/language/lang.{$name}.strings.php"))
				continue;
			try {
				$meta = ConfigParser::load_file($l);
			} catch(IAHConfigFileError $e) {
				$errmsg = "Error loading installer language file:" . $e->getMessage();
				$GLOBALS['log']->fatal($errmsg);
				continue;
			}
			if(is_array($meta) && ! empty($meta['detail']['label'])) {
				$ret[$name] = $meta['detail']['label'];
			}
		}
	}
	asort($ret);
	$supportedLanguages = $_SESSION['supportedLanguages'] = $ret;
	return $ret;
}


function installer_load_language($input_lang=null, $use_session=false, $reload_supported=false) {
	global $supportedLanguages;
	if(empty($supportedLanguages) || $reload_supported)
		installer_load_supported_languages($reload_supported);
	$default_lang = AppConfig::setting('install.defaults.language');
	
	$check = array();
	if($input_lang) {
		$input_lang = preg_replace('~[^a-zA-Z_]~','_',$input_lang);
		$check[] = $input_lang;
	}

	if($use_session && ! empty($_SESSION['language']))
		$check[] = $_SESSION['language'];
	
	if(! $check) {
		if(! empty($_SERVER['HTTP_ACCEPT_LANGUAGE'])) {
			foreach(parse_accept_language() as $l)
				$check[] = $l['name'];
		}
	}
	
	$check[] = $default_lang;
	
	$found = null;
	$errmsg = null;
	$strings = [];
	foreach($check as $lang) {
		if(isset($supportedLanguages[$lang])) {
			$lang_path = "install/language/lang.{$lang}.strings.php";
			try {
				$strings = ConfigParser::load_file($lang_path);
			} catch(Exception $e) {
				$errmsg = "Error loading installer language file:" . $e->getMessage();
				$GLOBALS['log']->fatal($errmsg);
				continue;
			}
			$found = $lang;
			break;
		}
	}
	
	if(! $found) {
		if($errmsg) die($errmsg);
		die('Installation language could not be loaded');
	}
	
	if($found != 'en_us') {
		$lang_base = ConfigParser::load_file('install/language/lang.en_us.strings.php');
		$strings = sugarArrayMerge($lang_base, $strings);
	}
	
	if($use_session)
		$_SESSION['language'] = $found;
	$GLOBALS['current_language'] = $found;
	$GLOBALS['mod_strings'] = $strings;
}

function saveInstallerConfig()
{
	if (empty($_SESSION['setup_save_config']))
		return;

	require_once 'include/config/format/ConfigWriter.php';
	$cw = new ConfigWriter;
	$cfg = array();
	$keys = array(
		'default_date_format',
		'default_time_format',
		'default_language',
		'default_locale_name_format',
		'default_email_charset',
		'default_export_charset',
		'default_export_format',
		'default_currency_name',
		'default_currency_symbol',
		'default_currency_iso4217',
		'default_currency_significant_digits',
		'default_number_format',
		'default_address_format',
		'demo_options',
		'setup_db_pop_demo_data',
		'setup_db_use_mb_demo_data',
	);
	foreach ($keys as $k) {
		if (isset($_SESSION[$k]))
			$cfg['session'][$k] = $_SESSION[$k];
	}
	$zip = new ZipArchive;
	$path = InstallerConfigParser::finalConfigPath();
	$zip->open($path, ZipArchive::OVERWRITE | ZipArchive::CREATE);
	require_once 'include/database/ListQuery.php';
	$lq = new ListQuery('UpgradeHistory');
	$lq->addSimpleFilter('type', 'langpack');
	$result = $lq->fetchAll();
	if($result && ! $result->failed) {
		foreach($result->getRows() as $idx => $row) {
			$zip->addFile($row['filename'], basename($row['filename']));
			$cfg['langpacks'][] = basename($row['filename']);
		}
	}
	$cw->writeFile("cache/temp/installer_config.php", $cfg);
	$zip->addFile("cache/temp/installer_config.php", 'installer_config.php');
	$zip->close();
	unlink("cache/temp/installer_config.php");
}

function flat_demo_options(&$options, $tree, $parent = null, $level = 0) {
	foreach($tree as $name => $opts) {
		$opts['name'] = $name;
		$opts['parent'] = $parent;
		$opts['level'] = $level;
		$options[$name] = $opts;
		if (!empty($opts['kids'])) {
			$kids = $opts['kids'];
			flat_demo_options($options, $kids, $name, $level+1);
			$options[$name]['kids'] = array_keys($kids);
		}
	}
}

function install_photo($path, $dir='/images/photos/') {
	if(! $path || ! is_file($path) || ! ($fname = basename($path)) )
		return;
	$dir = CacheManager::get_location($dir, true);
	for($i = 1; ; $i++) {
		$p = $dir . $i;
		if(! is_dir($p)) mkdir_recursive($p);
		$p .= '/' . $fname;
		if(! file_exists($p)) {
			copy($path, $p);
			return "$i/$fname";
		}
	}
}

function setup_enable_portal_access() {
	$options = array(
		'enabled',
		'perm_unreg_request',
		'perm_unreg_contactus',
		'perm_unreg_register',
		'perm_unreg_faq',
		'perm_lead_quotes',
		'perm_lead_bugs',
		'perm_lead_cases',
		'perm_lead_projects',
		'perm_lead_events',
		'perm_lead_kb',
		'perm_lead_faq',
		'perm_lead_contactus',
		'perm_lead_mycontact',
		'perm_lead_password',
		'perm_lead_request',
		'perm_lead_subscribe',
		'perm_contact_quotes',
		'perm_contact_bugs',
		'perm_contact_cases',
		'perm_contact_projects',
		'perm_contact_events',
		'perm_contact_kb',
		'perm_contact_faq',
		'perm_contact_contactus',
		'perm_contact_mycontact',
		'perm_contact_password',
		'perm_contact_request',
		'perm_contact_subscribe',

	);
	$opts = array();
	foreach ($options as $idx => $o) {
		$opts['portal'][$o] = true;
	}
	AppConfig::update_local($opts);
}

function installer_init_template($tpl) {
	global $mod_strings;
	$l = '';
	$doc_path = 'docs/InstallGuide.pdf';
	if(file_exists($doc_path))
		$l = '<a href="#" onclick="window.open($doc_path);" class="lw-head-link"><span class="uii uii-info"></span> '.$mod_strings['LBL_HELP'].'</a>';
	$tpl->assign('HEAD_LINK', $l);
	$tpl->assign('LANG', $mod_strings);
	// debug
	//global $the_file;
	//if(isset($the_file)) $tpl->assign('DEBUG', $the_file);
}

function installer_show_buttons($tpl, $buttons, $next=null) {
	if(isset($next))
		$tpl->assign('CURRENT_STEP', $next);
	if(! $buttons) return;
	foreach($buttons as $btn) {
		foreach($btn as $f=>$v) {
			$tpl->assign($f, $v);
		}
		if(! empty($btn['ICON']))
			$tpl->parse('foot.button_row.icon_button');
		else
			$tpl->parse('foot.button_row.button');
	}
	$tpl->parse('foot.button_row');
}


function install_kb_category($cat) {
	$subcategories = array_get_default($cat, 'subcategories', array());
	$articles = array_get_default($cat, 'articles', array());
	unset($cat['subcategories']);
	unset($cat['articles']);
	if (!empty($cat['image_filename'])) {
		$cat['image_filename'] = install_photo($cat['image_filename'], 'images/photos/');
	}
	$upd = RowUpdate::blank_for_model('KBCategory');
	$upd->set($cat);
	$upd->set('assigned_user_id', '1');
	$upd->save();
	$id = $upd->getPrimaryKeyValue();
	foreach ($subcategories as $sc) {
		$sc['parent_id'] = $id;
		install_kb_category($sc);
	}
	foreach ($articles as $art) {
		$art['category_id'] = $id;
		$upd = RowUpdate::blank_for_model('KBArticle');
		$upd->set($art);
		$upd->set('assigned_user_id', '1');
		$upd->save();
	}
}

function installer_cleanup() {
	global $progressIndicator;
	InstallerConfigParser::cleanup();
	$log = AppConfig::setting('install.progress_log');
	if(file_exists($log)) unlink($log);
	$log = AppConfig::setting('install_demo.progress_log');
	if(file_exists($log)) unlink($log);
}

// longreach - end added
