<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
define('sugarEntry', 1);


define('CACHE_MAX_AGE', 31536000);

require_once('sugar_version.php');
require_once('include/utils/basic_utils.php');
require_once('include/utils/longreach_utils.php');
require_once('include/config/AppConfig.php');
require_once('include/utils/CacheManager.php');

setPhpIniSettings(false);

// initialize global logger
$GLOBALS['log'] = AppConfig::create_logger();

$js_cache_dir = CacheManager::get_location('javascript/');
$sugar_version = AppConfig::version();
$js_version = AppConfig::js_custom_version();
$js_lang_version = AppConfig::language_version();
$gz_handler = AppConfig::get_server_compression();


function fail() {
	@header('HTTP/1.0 404 Not Found');
}
// duplicated from utils.php
function clean_min_cache() {
	global $sugar_version, $js_version, $js_cache_dir;
	$valid_ext = preg_quote("_{$sugar_version}_{$js_version}.", '~');
	$s = scandir($js_cache_dir);
	$t = time();
	foreach($s as $cache_file) {
		$path = $js_cache_dir.$cache_file;
		if(! is_file($path) || ! preg_match('~\.(js|css)$~', $cache_file))
			continue;
		if(preg_match("~$valid_ext(js|css)$~", $cache_file))
			continue;
		$mt = @filemtime($path);
		// remove expired cache files after 72 hours
		if($mt !== false && ($t - $mt) / 3600 > 72)
			@unlink($path);
	}
}
if(rand(1, 100) > 95)
	clean_min_cache();

$allowed = array('include/', 'modules/', 'themes/', 'custom/include/', 'custom/modules/', 'custom/themes/');

// check path argument, ensure it is in the current directory, readable, allowed, and a javascript file
if(! empty($_REQUEST['f']))
	$path_arg = $_REQUEST['f'];
else if(! empty($_SERVER['PATH_INFO'])) {
	$path_arg = $_SERVER['PATH_INFO'];
	if(preg_match('~^/ver\$([^/]*)(.*)$~', $path_arg, $m)) {
		$ver_arg = $m[1];
		$path_arg = $m[2];
		parse_str($ver_arg, $_REQUEST);
	}
}
else
	return fail();
$script = basename($_SERVER['SCRIPT_NAME']);
$root = strtr(realpath(dirname(__FILE__)), '\\', '/');
if(substr($root, -1) != '/') $root .= '/';
$path_arg = preg_replace('~^(/'.preg_quote($script).')?[/\\\\]*~', '', $path_arg);

if(substr($path_arg, 0, 9) == 'mingroup/') {
	if(! preg_match('~^(.*?)\.(js|css)$~', substr($path_arg, 9), $m))
		return fail();
	$groupname = $m[1];
	$ext = $m[2];
	require_once('include/layout/ResourceLoader.php');
	$inc_group = ResourceLoader::load_include_group($groupname, $_REQUEST);
	if(! $inc_group)
		return fail();
	return return_group($inc_group, $ext);
}

if(substr($path_arg, 0, 11) == 'jslanguage/') {
	global $sugar_version, $js_version, $js_cache_dir;
	if(! preg_match('~^(.*?)\.js$~', substr($path_arg, 11), $m))
		return fail();
	$langpath = explode('/', $m[1]);
	if(count($langpath) < 2)
		fail();
	if($langpath[0] == 'app')
		$p = array('module' => 'app', 'language' => $langpath[1]);
	else if($langpath[0] == 'modules' && count($langpath) >= 3)
		$p = array('module' => $langpath[1], 'language' => $langpath[2]);
	else
		return fail();
	if(! preg_match('~^[a-z\-_]+$~i', $p['language']) || ! preg_match('~^[a-z\-_][a-z0-9\-_]*$~i', $p['module']))
		return fail();
	require_once('include/layout/ResourceLoader.php');
	return handle_js_language($p, true);
}

$path = strtr(realpath($path_arg), '\\', '/');
if($path === false || substr($path, 0, strlen($root)) != $root)
	return fail();
$path = substr($path, strlen($root));
if(! $path || ! file_exists($path) || ! is_readable($path))
	return fail();
$path_allowed = false;
foreach($allowed as $base_path) {
	if(substr($path, 0, strlen($base_path)) == $base_path) {
		$path_allowed = true;
		break;
	}
}
if(! $path_allowed)
	return fail();
return_min_file($path);


function return_group($group, $ext) {
	global $sugar_version, $js_version, $js_cache_dir, $js_lang_version;
	list($groupname, $all_incs, $inc_params, $inc_types) = $group;
	$param_md5 = '_'.md5(serialize($group));
	$add_chk = '';
	$type = ($ext == 'js' ? 'javascript' : 'css');
	if(empty($inc_types[$type]))
		return fail();
	
	$source_mtime = 0;
	$incs = array();
	foreach($all_incs as $row) {
		if($row['inc_type'] != $type)
			continue;
		if($row['type'] == 'language') {
			$row['type'] = 'file';
			$row['path'] = handle_js_language($row);
			$add_chk = '_'.$js_lang_version;
		}
		if($row['type'] == 'file') {
			$row_mtime = @filemtime($row['path']);
			if($row_mtime > $source_mtime)
				$source_mtime = $row_mtime;
		}			
		$incs[] = $row;
	}
	$cache_fname = $js_cache_dir . "mingroup_{$groupname}{$param_md5}{$add_chk}_{$sugar_version}_{$js_version}.$ext";
	list($use_cache, $update_cache) = check_min_cache($cache_fname, $source_mtime);
	
	if($use_cache && ! $update_cache)
		return_cache($cache_fname, $ext);
	else {
		$src = '';
		foreach($incs as $row) {
			if($row['type'] == 'file') {
				$contents = file_get_contents($row['path']);
				if($type == 'css')
					$contents = prefix_css_paths(dirname($row['path']), $contents);
				$src .= $contents;
			} else {
				if(! empty($row['cond_comment']))
					continue;
				$src .= $row['value'];
			}
			$src .= "\n";
		}
		$min_source = minify_source($src, $ext);
		if($use_cache) {
			//cache_log("updating cache $cache_fname");
			write_cache($cache_fname, $min_source, $ext, true);
		}
		else {
			start_output($ext);
			echo $min_source;
			end_output();
		}
	}
}

function handle_js_language($p, $echo=false) {
	$loader = new ResourceLoader();
	$inc = $loader->get_js_language_path($p);
	if($inc && $echo)
		return_min_file($inc);
	return $inc;
}

function check_min_cache($cache_fname, $source_mtime=0) {
	global $etag, $cache_mtime, $js_cache_dir;
	$etag = md5($cache_fname);
	$use_cache = is_writable($js_cache_dir);
	$update_cache = false;
	if(file_exists($cache_fname)) {
		$cache_mtime = @filemtime($cache_fname);
		if($cache_mtime < $source_mtime)
			$update_cache = true;
	}
	else {
		$cache_mtime = time();
		$update_cache = true;
	}
	calc_expires($cache_mtime);
	return array($use_cache, $update_cache);
}


function return_min_file($path) {
	global $js_cache_dir, $sugar_version, $js_version, $etag;
	if(! preg_match('~^(.*)\\.(js|css)$~', $path, $m))
		return fail();
	$noext_path = $m[1];
	$ext = $m[2];
	$dir_path = dirname($path);
	$source_mtime = @filemtime($path);
	$cache_fname = $js_cache_dir . preg_replace('~[/\\\\]~', '_', $noext_path) . "_{$sugar_version}_{$js_version}.$ext";
	list($use_cache, $update_cache) = check_min_cache($cache_fname, $source_mtime);
	if(preg_match('~\-min.js$~', $path))
		$use_cache = false; // already minified
	
	if($use_cache && ! $update_cache)
		return_cache($cache_fname, $ext);
	else if($use_cache) {
		//cache_log("updating cache $cache_fname");
		$source = file_get_contents($path);
		$min_source = minify_source($source, $ext);
		if($ext == 'css')
			$min_source = prefix_css_paths($dir_path, $min_source);
		write_cache($cache_fname, $min_source, $ext, true);
	}
	else {
		// don't bother with minify if we can't save the cache file
		return_file($path, $ext);
	}
}

function add_content_type($ext) {
	if($ext == 'js')
		header('Content-Type: text/javascript; charset=utf-8');
	else
		header('Content-Type: text/css');
}

function return_file($path, $ext) {
	start_output($ext);
	if($ext == 'css') {
		$dir_path = dirname($path);
		$source = file_get_contents($path);
		$source = prefix_css_paths($dir_path, $source);
		echo $source;
	} else {
		@readfile($path);
	}
	end_output();
}

function minify_source($source, $ext) {
	if($ext == 'js') {
		require_once('include/javascript/jsminplus.php');
		$min_source = trim(JSMinPlus::minify($source));
	} else {
		require_once('include/javascript/cssmin.php');
		$min_source = cssmin::minify($source);
	}
	return $min_source;
}


function return_cache($cache_fname, $ext) {
	global $etag, $last_mod, $expires, $gz_handler;
	if(! $gz_handler) {
		// cannot do this if ob_gzhandler is enabled in php.ini
		// even removing the handler produces some output, which is invalid with 304 header
		if($etag && strpos(array_get_default($_SERVER, 'HTTP_IF_NONE_MATCH', ''), $etag) !== false)
			$cache_ok = true;
		else if(array_get_default($_SERVER, 'HTTP_IF_MODIFIED_SINCE', '') == $last_mod)
			$cache_ok = true;
		if(isset($_SERVER['HTTP_CACHE_CONTROL'])) {
			if(strpos($_SERVER['HTTP_CACHE_CONTROL'], 'no-cache') !== false)
				$client_no_cache = true;
			//else if(strpos($_SERVER['HTTP_CACHE_CONTROL'], 'max-age=0') !== false)
			//	$client_no_cache = true;
		}
		if(! empty($cache_ok) && empty($client_no_cache)) {
			//cache_log("sending 304 header $cache_fname");
			while(ob_get_level()>0 && @ob_get_clean() !== false);
			header('HTTP/1.1 304 Not Modified');
			add_content_type($ext);
			add_cache_headers();
			exit;
		}
	}
	start_output($ext);
	@readfile($cache_fname);
	end_output();
	exit;
}

function write_cache($cache_fname, $min_source, $ext, $echo=false) {
	if($fp = fopen($cache_fname, 'wb')) {
		fwrite($fp, $min_source);
		fclose($fp);
		$cache_mtime = @filemtime($cache_fname);
		calc_expires($cache_mtime);
	}
	if($echo) {
		start_output($ext);
		echo $min_source;
		end_output();
	}
}


function calc_expires($cache_mtime) {
	global $last_mod, $expires;
	$last_mod = gmdate('D, d M Y H:i:s \G\M\T', $cache_mtime);
	$expires = gmdate('D, d M Y H:i:s \G\M\T', $cache_mtime+constant('CACHE_MAX_AGE'));	
}

function add_cache_headers() {
	global $etag, $expires, $last_mod;
	header("Last-Modified: $last_mod");
	header("Expires: $expires");
	header("Cache-Control: public, max-age=".constant('CACHE_MAX_AGE'));
	if($etag)
		header("ETag: \"$etag\"");
}

function start_output($ext) {
	if(headers_sent()) return;
	
	global $gz_handler, $gz_enc;
	if(! $gz_handler) {
		/*if(function_exists('ob_gzhandler')) {
			ob_start('ob_gzhandler');
			$gz_handler = 'ob_gzhandler';
		}*/
		// prefer to install our own handler, to support 304 header return
		$accept_encs = array_map('trim', explode(',', array_get_default($_SERVER, 'HTTP_ACCEPT_ENCODING', '')));
		$enc_map = array();
		$ord = count($accept_encs);
		foreach($accept_encs as $enc) {
			$vals = explode(';', $enc);
			$enc = array_shift($vals);
			$qval = 0.5;
			foreach($vals as $p) {
				if(substr($p, 0, 2) == 'q=')
					$qval = (float)substr($p, 2);
			}
			$enc_map[$enc] = array($qval, $ord--);
		}
		arsort($enc_map); // sort by reverse qval, highest first
		$gz_enc = '';
		foreach($enc_map as $enc => $ord) {
			if($enc == 'gzip' || $enc == 'x-gzip')
				$gz_enc = 'gzip';
			else if($enc == 'deflate' || $enc == 'zlib')
				$gz_enc = 'deflate';
			if($gz_enc)
				break;
		}
		if($gz_enc) {
			$gz_handler = 'gzip';
			ob_start('gzip_handler', 0);
			//cache_log($gz_enc);
		}
	}
	add_content_type($ext);
	add_cache_headers();
}

function gzip_handler($buffer, $flags) {
	global $gz_enc;
	if($gz_enc == 'gzip')
		$buffer = gzencode($buffer, 9, FORCE_GZIP);
	else if($gz_enc == 'deflate')
		$buffer = gzcompress($buffer, 9);
	else
		$buffer = false;
	if($buffer !== false) {
		header("Content-Encoding: $gz_enc", true);
		header('Content-Length: '.strlen($buffer), true);
	}
	return $buffer;
}

function end_output() {
}

function prefix_css_paths($prefix, $source) {
	$site_pfx = AppConfig::site_static_url();
	if(! $site_pfx) $site_pfx = AppConfig::site_url();
	$site_pfx = preg_replace('~^\w+://[^/]*~', '', $site_pfx);
	if(substr($site_pfx, -1) != '/') $site_pfx .= '/';
	$prefix = $site_pfx . $prefix;
	$source = preg_replace_callback(
		'~url\((["\']?)([^\)]+?)(["\']?)\)~',
		function($matches) use ($prefix) {
			return adjust_css_path($prefix, $matches[2]);
		},
		$source);
	$source = preg_replace('~@import\s+["\']~', '\\0'.$prefix.'/', $source);
	return $source;
}

function adjust_css_path($prefix, $path) {
	if(($sfx = substr($path, -4)) == '.png' || $sfx == '.svg')
		$path .= '?' . js_version_params();
	$path = $prefix.'/'.$path;
	$parent_pat = '~[^/]+/\.\./~';
	while(preg_match($parent_pat, $path))
		$path = preg_replace($parent_pat, '', $path, 1);
	return 'url("'.$path.'")';
}

function cache_log($msg) {
	$fp = fopen("cache_log.txt", "a");
	fwrite($fp, $msg."\n");
	fclose($fp);
}

?>
