<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


define('sugarEntry', 1);
define('loginEntry', 1);
// note: this entry point does not automatically have a database connection
define('IAH_LIGHTWEIGHT', 1);
require_once('include/entryPoint.php');

// Prevent access via alternate site urls
if(AppConfig::is_extern_url())
	return;
if( ($redir = AppConfig::get_auto_redirect()) ) {
	if(array_get_default($_SERVER, 'REQUEST_METHOD') == 'GET') {
		loginRedirect($redir[0]);
	} else
		return;
}
// Redirect to install.php if not yet installed
if(! AppConfig::installed()) {
	if(AppConfig::can_install()) {
		loginRedirect('install.php');
	} else {
		$GLOBALS['log']->fatal('application not installed, and installation explicitly disabled');
		exit; // install would have to be explicitly disabled in standard config
	}
}
// Redirect to upgrade.php if upgrade in progress
if (AppConfig::upgrade_in_progress()) {
	loginRedirect('upgrade.php');
}

function loginRedirect($params) {
	if(is_string($params))
		$path = $params;
	else if(isset($params['path']))
		$path = $params['path'];
	else
		$path = 'index.php' . encode_query_string($params, true);
	header('Location: '.$path);
	if(function_exists('sugar_cleanup'))
		sugar_cleanup(true);
	else
		exit;
}

AppConfig::set_login_params();

if(session_status() == PHP_SESSION_NONE)
	AppConfig::session_start(null, true);

SessionManager::keepSession();

$login_error = get_session_end_eror_msg();
if(! empty($_SESSION['login_error'])) {
	$login_error = $_SESSION['login_error'];
	unset($_SESSION['login_error']);
}
if(isset($_SESSION['mobile_detected']))
	$detect_mobile = $_SESSION['mobile_detected'];
else
	$detect_mobile = $_SESSION['mobile_detected'] = AppConfig::detect_mobile();
$detect_mobile = false; // must pass ?mobile=1 to enable old mobile theme

global $current_language;
$check_lang = $guess_lang = array();
if(! empty($_REQUEST['user_language']))
	$check_lang[] = $_REQUEST['user_language'];
if(! empty($_COOKIE['ck_login_language']))
	$guess_lang[] = $_COOKIE['ck_login_language'];
AppConfig::init_user_language($check_lang, $guess_lang);

$action = array_get_default($_REQUEST, 'action', 'login');
$allow_recover = check_pw_recovery();
$recover = ($allow_recover && $action == 'recover') ? 1 : 0;
if($action == 'recover' && ! $recover) $action = 'login';
$allow_remember = AppConfig::setting('site.security.remember_user');
$remember_user = $allow_remember && AppConfig::setting('site.security.default_remember_user') ? 1 : 0;
$default_user = AppConfig::setting('site.login.default_user', '');
$default_password = AppConfig::setting('site.login.default_password', '');
$static_url = AppConfig::site_static_url();
$jsver = js_version_params();
$show_mobile = (! empty($_REQUEST['mobile']) || $detect_mobile) ? 1 : 0;
$default_mobile = $detect_mobile && ! $detect_mobile['tablet'];
$mobile_themes = AppConfig::get_themes(true);


$supported_actions = array(
	'recover' => translate('LBL_RECOVER_TITLE', 'Users'),
	'about' => translate('LNK_ABOUT', 'app'),
	'login' => translate('LBL_LOGIN_TITLE', 'Users'),
);
if(! isset($supported_actions[$action]))
	$action = 'login';
$with_form = ($action == 'login' || $action == 'recover');


// request and complete password recovery
function check_pw_recovery() {
	if(! AppConfig::setting('site.security.email_password_recovery'))
		return false;
	$send_type = AppConfig::setting('email.send_type');
	if($send_type == 'sendmail') {
		if(AppConfig::setting('site.sendmail_disabled'))
			return false;
	} else if($send_type == 'SMTP') {
		if(! AppConfig::setting('email.smtp_server'))
			return false;
	}
	return true;
}

if($recover && ! empty($_REQUEST['user_name'])) {
	// recovery limited to POST so that external links can't trigger it
	if(! empty($_REQUEST['key']) || $_SERVER['REQUEST_METHOD'] == 'POST') {
		entryInitDatabase();
		loadLicense();
		$GLOBALS['authController'] = new AuthenticationController();
		require_once('modules/Users/utils.php');
		if(! empty($_REQUEST['key'])) {
			if(user_password_recovery($_REQUEST['user_name'], $_REQUEST['key']))
				loginRedirect('index.php');
			else
				$recover = 'fail';
		} else {
			$captcha_passed = true;
			if (AppConfig::captcha_complexity() >= 0) {
				$expected_captcha = array_get_default($_SESSION, 'captcha', true);
				if ($expected_captcha !== sha1(array_get_default($_POST, 'captcha', ''))) {
					$captcha_passed = false;
				}
			}
			if ($captcha_passed) {
				user_password_recovery($_REQUEST['user_name']);
				// show success message either way so we don't confirm user/email existence
				$recover = 'done';
			} else {
				AppConfig::track_login(false);
				$recover = 'captcha';
			}
		}
	}
}


$captcha_complexity = AppConfig::captcha_complexity();
$page_title = $supported_actions[$action];
$page_title .= ' | ' . translate('LBL_BROWSER_TITLE', 'app');

require_once('include/layout/IncludeManager.php');
$inc_mgr = new IncludeManager();
require_once('include/layout/BaseTheme.php');
$themeInstance = new BaseTheme('Default'); // temporary - should probably init correct theme

$theme_css = $themeInstance->get_theme_css();
foreach($theme_css as $href) {
	if(is_array($href)) {
		$tpars = array_get_default($href, 'params');
		$href = $href['value'];
	} else {
		$tpars = null;
	}
	$inc_mgr->add_css_include($href, $tpars, LOAD_PRIORITY_THEME);
}
$login_css = 'themes/Default/login.css';
$inc_mgr->add_css_include($login_css, null, LOAD_PRIORITY_THEME);
if(file_exists('custom/' . $login_css))
	$inc_mgr->add_css_include('custom/' . $login_css, null, LOAD_PRIORITY_THEME);

$inc_mgr->add_js_include('include/javascript/base.js', null, LOAD_PRIORITY_THEME);
$inc_mgr->add_js_include('include/javascript/conn.js', null, LOAD_PRIORITY_THEME);
$inc_mgr->add_js_include('include/javascript/lightweight.js', null, LOAD_PRIORITY_THEME);
$inc_mgr->add_js_include('include/javascript/login.js', null, LOAD_PRIORITY_THEME);

$theme_css = $inc_mgr->format_includes('head', 'css');
$theme_js = $inc_mgr->format_includes('head', 'javascript');

$icon_href = 'favicon.ico?s='.AppConfig::version();
$icon_type = 'image/vnd.microsoft.icon';
$favicon = "<link rel=\"shortcut icon\" href=\"$icon_href\" type=\"$icon_type\">\n".
	"<link rel=\"icon\" href=\"$icon_href\" type=\"$icon_type\" sizes=\"16x16\">\n";
$meta = get_mobile_page_headers(! $recover);

$login_user_name = $login_password = '';
if(! empty($_REQUEST['default_user_name']))
	$login_user_name = $_REQUEST['default_user_name'];
if($recover && ! empty($_REQUEST['user_name']))
	$login_user_name = $_REQUEST['user_name'];
else {
	if(! empty($_COOKIE['ck_login_forget'])) {
		$remember_user = 0;
	}
	else if($allow_remember && ! empty($_COOKIE['ck_login_username'])) {
		$login_user_name = $_COOKIE['ck_login_username'];
		$remember_user = 1;
	}
	if(! $login_user_name && ! $recover) {
		$login_user_name = $default_user;
		$login_password = $default_password;
	}
}


$login_message = '';
if(AppConfig::setting('site.login.use_custom_message')) {
	$custom_msg = AppConfig::setting('site.login.custom_message_file');
	if($custom_msg)
		$login_message = @file_get_contents($custom_msg);
	else
		$login_message = AppConfig::setting('site.login.custom_message', '');
}
if(empty($login_message)) {
	$msg_paths = array(); $msg_lang = array();
	$base_lang = AppConfig::setting('locale.base_language', 'en_us');
	$default_lang = AppConfig::setting('locale.defaults.language');
	foreach(array($current_language, $default_lang, $base_lang) as $l) {
		if(! empty($msg_lang[$l])) continue;
		$msg_lang[$l] = true;
		if(is_demo_site())
			$msg_paths[] = "include/language/{$l}.login_message_demo.html";
		$msg_paths[] = "custom/include/language/{$l}.login_message.html";
		$msg_paths[] = "include/language/{$l}.login_message.html";
	}
	$login_message = '';
	foreach($msg_paths as $p) {
		if(file_exists($p) && is_readable($p)) {
			$login_message = file_get_contents($p);
			break;
		}
	}
}
$login_message = str_replace('{SITE_URL}', AppConfig::site_url(), $login_message);

$login_logo = AppConfig::setting('site.top_logo.login', '');
$login_std = AppConfig::setting('site.top_logo.login', '', true);
if(! $login_logo || ! file_exists($login_logo)) {
	$top_logo = AppConfig::setting('site.top_logo.primary', '');
	$top_std = AppConfig::setting('site.top_logo.primary', '', true);
	if($top_logo != $top_std)
		$login_logo = $top_logo;
	else
		$login_logo = $login_std;
}
$logo_h = AppConfig::setting('site.top_logo.login_height', 50);
$size = custom_get_image_size($login_logo);
if($size) {
	if($size[1] >= $logo_h * 1.5) {
		$size[0] = round($size[0] * $logo_h / $size[1]);
		$size[1] = round($logo_h);
	}
	$login_logo = '<div class="lw-logo"><img src="'.to_html($static_url.$login_logo.'?'.$jsver).'" width="'.$size[0].'" height="'.$size[1].'" alt="1CRM"></div>';
}
else {
	$login_logo = '';
}

// lightweight app_string suppoort (not including full language file)
$need_strings = array(
	'LBL_LOADING',
	'LBL_LOGGING_IN',
	'LBL_REDIRECTING',
	'LBL_CONNECTION_FAILED',
	'NTC_LOGIN_MESSAGE',
	'NTC_JS_SESSION_ENDED',
	'NTC_JS_FORBIDDEN',
);
$appstr = array();
foreach($need_strings as $s)
	$appstr[] = "\"$s\":\"".javascript_escape(translate($s, 'app')).'"';
$appstr = '{'.implode(',', $appstr).'}';

require_once('include/MultiFA/MultiFactorAuthLogin.php');
$mFALogin = new \OneCRM\MultiFA\MultiFactorAuthLogin();

?>
<!DOCTYPE html>
<html>
<head>
<base href="<?php echo to_html(AppConfig::site_url()); ?>">
<meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1">
<meta charset="<?php echo to_html(AppConfig::charset()); ?>">
<meta name="viewport" content="width=device-width, initial-scale=1<?php if($with_form) echo ', maximum-scale=1, user-scalable=no'; ?>">
<?php echo $meta; ?>
<?php echo $favicon; ?>
<title><?php echo $page_title; ?></title>
<?php echo $theme_css . $theme_js; ?>
</head>
<body class="login <?php if($show_mobile) echo 'mobile '; if($action == 'about') echo 'about '; if($with_form) echo 'with-form '; ?>">
<section class="container lw-outer">
<header class="row page-row lw-head">
	<?php echo $login_logo; ?>
	<?php
	if($action != 'login')
		echo '<a href="login.php" class="lw-head-link"><span class="uii uii-login"></span> '.translate('LBL_LOGIN_BUTTON_LABEL', 'Users').'</a>';
	else
		echo '<a href="login.php?action=about" class="lw-head-link"><span class="uii uii-info"></span> '.translate('LNK_ABOUT', 'app').'</a>';
	?>
</header>
<main class="row page-row lw-main">

<?php if($action == 'about') {

require('modules/Home/About.php');

} else if($action == 'recover') { ?>

<div class="row page-row">
<div class="column page-column sm-8 sm-offset-2 recover-form">
<p class="lead"><?php echo translate('LBL_RECOVER_TITLE', 'Users'); ?></p>

<?php if($recover === 'fail') { ?>

	<p><?php echo translate('LBL_RECOVERY_FAILED', 'Users'); ?></p>
	<div class="lw-form-buttons text-right">
		<button id="return_button" class="lw-iconbutton" type="button" onclick="loginReturn();" tabindex="2">
			<span class="uii uii-arrow-left"></span>
			<span class="btn-label"><?php echo translate('LBL_LOGIN_RETURN_LABEL', 'Users'); ?></span>
		</button>
	</div>

<?php } else if($recover === 'done') { ?>

	<p><?php echo translate('LBL_RECOVERY_SENT', 'Users'); ?></p>
	<div class="lw-form-buttons text-right">
		<button id="return_button" class="lw-iconbutton" type="button" onclick="loginReturn();" tabindex="2">
			<span class="uii uii-arrow-left"></span>
			<span class="btn-label"><?php echo translate('LBL_LOGIN_RETURN_LABEL', 'Users'); ?></span>
		</button>
	</div>

<?php } else { ?>

<form name="LoginForm" action="login.php" method="POST" onsubmit="return checkRecoverForm();" role="form">
	<input type="hidden" name="action" value="recover">
	<?php echo create_csrf_input() ?>
	<div class="form-group">
		<label class="control-label" for="login_user"><?php echo translate('LBL_USER_NAME', 'Users'); ?></label>
		<div class="input-field input-field-group input-block">
			<input id="login_user" name="user_name" class="input-text input-lg" type="text" tabindex="1" autocapitalize="off" autocorrect="off" spellcheck="false" value="<?php echo to_html($login_user_name); ?>">
			<span class="input-label input-field-icon lg"><span class="uii uii-user uii-lg"></span></span>
		</div>
	</div>
	<div id="captcha-outer" class="form-group">
		<label class="control-label" for="login_captcha"><?php echo translate('LBL_CAPTCHA', 'Users'); ?></label>
		<img class="captcha" id="captcha-img" onclick="showCaptcha()" />
		<div class="input-field input-field-group input-block">
			<input id="login_captcha" name="captcha" class="input-text input-lg" type="text" tabindex="1" value="">
			<span class="input-label input-field-icon lg"><span class="uii uii-lock uii-lg"></span></span>
		</div>
	</div>
	<section class="card-outer panel-outer panel-border panel-round panel-alert form-group login-error" role="alert" id="error-outer" <?php if($recover !== 'captcha') echo 'style="display: none"'; ?>>
		<div class="card-body panel-body panel-pad-body">
			<span class="uii uii-alert"></span>
			<span id="login-error"><?php echo translate('ERR_CAPTCHA_FAIL', 'Users') ?></span>
		</div>
	</section>
	<div class="lw-form-buttons text-right">
		<button id="return_button" class="lw-iconbutton" type="button" onclick="loginReturn();" tabindex="2">
			<span class="uii uii-arrow-left"></span>
			<span class="btn-label"><?php echo translate('LBL_LOGIN_RETURN_LABEL', 'Users'); ?></span>
		</button>
		<button id="recover_button" class="lw-iconbutton lwib-accept" type="submit" tabindex="2">
			<span class="uii uii-email"></span>
			<span class="btn-label"><?php echo translate('LBL_RECOVER_BUTTON_LABEL', 'Users'); ?></span>
		</button>
	</div>
</form>

<?php } ?>

</div>
</div>

<?php } elseif ( $mFALogin->isDisplayCodeVerification() ) { /* Two FA Code verification */

    echo $mFALogin->getCodeVerificationForm($login_message);

?>

<?php } else { /* not password recovery */ ?>

<div class="row page-row sm-unstack">
<div class="column page-column md-7 xs-hidden login-message">
	<?php echo $login_message; ?>
</div>
<div class="column page-column md-5 md-offset-0 sm-8 sm-offset-2 lw-form login-form">
<div class="form-outer">
<form name="LoginForm" action="index.php" method="POST" onsubmit="return doLogin();" role="form">
	<input type="hidden" name="module" value="Users">
	<input type="hidden" name="action" value="Authenticate">
	<input type="hidden" name="mobile" value="0">
	<input type="hidden" name="user_remember" value="0">
	<noscript><input type="hidden" name="no_js" value="1"></noscript>
<?php
	foreach(array('login_module','login_action','login_record','login_layout') as $lf) {
		echo '<input type="hidden" name="'.$lf.'" value="'.to_html(array_get_default($_REQUEST, $lf, '')).'">';
	}
?>

	<div class="form-group">
		<label class="control-label" for="login_user"><?php echo translate('LBL_USER_NAME', 'Users'); ?></label>
		<div class="input-field input-field-group input-block">
			<input id="login_user" name="user_name" class="input-text input-lg" type="text" tabindex="1" autocapitalize="off" autocorrect="off" spellcheck="false" value="<?php echo to_html($login_user_name); ?>">
			<span class="input-label input-field-icon lg"><span class="uii uii-user uii-lg"></span></span>
		</div>
	</div>
	<div class="form-group">
		<label class="control-label" for="login_pass"><?php echo translate('LBL_PASSWORD', 'Users'); ?></label>
		<div class="input-field input-field-group input-block">
			<input id="login_pass" name="user_password" class="input-text input-lg" type="password" tabindex="1" value="<?php echo to_html($login_password); ?>">
			<span class="input-label input-field-icon lg"><span class="uii uii-lock uii-lg"></span></span>
		</div>
	</div>
	<div id="captcha-outer" class="form-group">
		<label class="control-label" for="login_lang"><?php echo translate('LBL_CAPTCHA', 'Users'); ?></label>
		<img class="captcha" id="captcha-img" onclick="showCaptcha()" />
		<div class="input-field input-field-group input-block">
			<input id="login_captcha" name="user_captcha" class="input-text input-lg" type="text" tabindex="1" value="">
			<span class="input-label input-field-icon lg"><span class="uii uii-lock uii-lg"></span></span>
		</div>
	</div>
	<div id="lang-outer" class="form-group">
		<label class="control-label" for="login_lang"><?php echo translate('LBL_LANGUAGE', 'Users'); ?></label>
		<select id="login_lang" name="user_language" class="input-field input-block input-lg" tabindex="1">
			<?php
				$langs = AppConfig::get_languages();
				echo get_select_options_with_id($langs, $current_language);
			?>
		</select>
	</div>
	<div id="theme-outer" class="form-group" <?php if($default_mobile) echo 'style="display: none"'; ?>>
		<label class="control-label" for="login_theme"><?php echo translate('LBL_THEME', 'Users'); ?></label>
		<select id="login_theme" name="user_theme" class="input-field input-block input-lg" tabindex="1">
			<?php
				$themes = AppConfig::get_themes(false);
				$theme = null;
				if(isset($_REQUEST['user_theme']))
					$theme = $_REQUEST['user_theme'];
				else if(isset($_COOKIE['ck_login_theme']))
					$theme = $_COOKIE['ck_login_theme'];
				if(! $theme || ! isset($themes[$theme]))
					$theme = AppConfig::theme();
				echo get_select_options_with_id($themes, $theme);
			?>
		</select>
	</div>
	<section class="card-outer panel-outer panel-border panel-round panel-alert form-group login-error" role="alert" id="error-outer" <?php if(empty($login_error)) echo 'style="display: none"'; ?>>
		<div class="card-body panel-body panel-pad-body">
			<span class="uii uii-alert"></span>
			<span id="login-error"><?php if(! empty($login_error)) echo to_html($login_error); ?></span>
		</div>
	</section>
	<div class="login-foot">
		<div class="login-checks">
			<?php if($show_mobile) { ?>
			<div class="form-group mobile">
				<div class="input-check">
					<input type="checkbox" name="mobile" value="1" <?php if($default_mobile) echo 'checked="checked"'; ?> tabindex="1"><label> <?php echo to_html(translate('LBL_LOGIN_MOBILE', 'Users')); ?></label>
				</div>
			</div>
			<?php } ?>
			<?php if($allow_remember) { ?>
			<div class="form-group remember">
				<div class="input-check">
					<input type="checkbox" name="user_remember" value="1" <?php if($remember_user) echo 'checked="checked"'; ?> tabindex="1"><label> <?php echo to_html(translate('LBL_REMEMBER_ME', 'Users')); ?></label>
				</div>
			</div>
			<?php } ?>
		</div>
		<div id="login-opts" class="login-opts">
			<?php if($allow_recover) { ?>
				<a href="login.php?action=recover" class="login-util" tabindex="1"><?php echo to_html(translate('LBL_RECOVER_PASS_LINK', 'Users')); ?></a>
			<?php } ?>
		</div>
		<section class="card-outer card-inline panel-outer panel-border panel-round panel-info login-status" role="alert" id="status-outer" style="display: none">
			<div class="card-body panel-body panel-pad-body">
				<span class="uii uii-info"></span>
				<span id="login-status"></span>
			</div>
		</section>
		<button id="login_button" class="lw-iconbutton lwib-accept" type="submit" tabindex="2">
			<span class="uii uii-arrow-right"></span>
			<span class="btn-label"><?php echo translate('LBL_LOGIN_BUTTON_LABEL', 'Users'); ?></span>
		</button>
		<div class="clearfix"></div>
	</div>
</form>
</div>
</div>
</div>

<?php } ?>

</main>

<?php

	if(! function_exists('format_number')) {
		function format_number($val, $dec) {
			return number_format($val, $dec);
		}
	}

	// Under the Sugar Public License, you are required to leave in all copyright statements in both
	// the code and end-user application as well as the the powered by image. You can not change the url or the image below.
	$footer_copyright = '<div class="column page-column footerLeft">&copy; 2004-' . date('Y') . ' <a href="http://www.1crm.com" target="_blank"  class="footerLink copyRightLink">1CRM Corp.</a> All Rights Reserved.<br />';
	$footer_copyright .= '&copy; 2004-2008 <a href="http://www.sugarcrm.com" target="_blank" class="footerLink copyRightLink" rel="nofollow">SugarCRM Inc.</a> All Rights Reserved.</div>';
	$footer_copyright .= '<div class="column page-column footerRight"><a href="http://www.sugarforge.org" target="_blank" rel="nofollow" title="Nostalgia"><img style="margin-top: 2px" border="0" width="106" height="23" src="'.$static_url.'include/images/poweredby_sugarcrm.png" alt="Powered By SugarCRM"></a>';
	$footer_copyright .= ' <a href="http://www.1crm.com/1crm-engine/" target="_blank" title="Modernity"><img style="margin-top: 2px" border="0" width="106" height="23" src="'.$static_url.'include/images/iah/1crm-powered.png" alt="Powered by 1CRM Engine"></a></div>';

	$footer_debug = '';
	$calc_response = AppConfig::setting('site.performance.calculate_response_time');
	$calc_response_flag = AppConfig::setting('site.performance.calculate_response_time_flag');
	if($calc_response_flag && ! empty($_REQUEST[$calc_response_flag])) {
		$calc_response = true;
	}
	if($calc_response && (! AppConfig::is_mobile() || ! empty($_REQUEST['debug']))) {
		$deltaTime = microtime(true) - $startTime;
		$footer_debug .= translate('LBL_SERVER_RESPONSE_TIME', 'app').' '.format_number(round($deltaTime,2),2).' '.translate('LBL_SERVER_RESPONSE_TIME_SECONDS', 'app').'<br />';
		$dbTime = (! empty($db) ? $db->getTotalQueryTime() : 0);
		$footer_debug .= translate('LBL_SERVER_RESPONSE_TIME_DB', 'app').' '.format_number(round($dbTime,2),2).' '.translate('LBL_SERVER_RESPONSE_TIME_SECONDS', 'app').'<br />';
		if(AppConfig::setting('config.log_all_requests')) {
			$cfgTime = AppConfig::$_config->total_request_time;
			$footer_debug .= translate('LBL_SERVER_RESPONSE_TIME_CONFIG', 'app').' '.format_number(round($cfgTime,2),2).' '.translate('LBL_SERVER_RESPONSE_TIME_SECONDS', 'app').'<br />';
		}

		if(AppConfig::setting('site.performance.show_page_resources')) {
			// Print out the resources used in constructing the page.
			$included_files = get_included_files();
			$all_parsed = ConfigParser::get_all_parsed();

			$footer_qcount = (! empty($db) ? $db->getQueryCount() : 0);
			$footer_debug .= (translate('LBL_SERVER_RESPONSE_RESOURCES', 'app').' ('.$footer_qcount.', '.sizeof($included_files).', '.sizeof($all_parsed).')<br>');
			// Display performance of the internal and external caches....
			$cache_hits = $GLOBALS['external_cache_request_local_hits'];
			$cache_total = $GLOBALS['external_cache_request_local_total'];
			$footer_debug .= "Data cache (hits/total=ratio) local ($cache_hits/$cache_total=".($cache_total ? round($cache_hits*100/$cache_total,0) : 0)."%)";
			if($GLOBALS['external_cache_request_external_total'])
				// Only display out of process cache results if there was at least one attempt to retrieve from the out of process cache (this signifies that it was enabled).
				$footer_debug .= " external ({$GLOBALS['external_cache_request_external_hits']}/{$GLOBALS['external_cache_request_external_total']}=".round($GLOBALS['external_cache_request_external_hits']*100/$GLOBALS['external_cache_request_external_total'],0)."%)";
			else
				$footer_debug .= ", no external";
			if($GLOBALS['external_cache_type'] && is_admin())
				$footer_debug .= ' <a href="index.php?module=Administration&action=ResetCache" class="footerLink">[reset cache]</a>';
			$footer_debug .= "<br>";
		}

		if(AppConfig::setting('site.performance.show_memory_usage')) {
			$footer_debug .= 'Memory usage peak: '. format_file_size(memory_get_peak_usage());
			$footer_debug .= ', current: ' . format_file_size(memory_get_usage()) . '<br>';
		}
	}

?>
<footer class="lw-footer">
<div class="row page-row col-unstack">
	<?php echo $footer_copyright; ?>
<?php if($footer_debug) {
	echo '<div class="lw-debug column page-column col-12">'. $footer_debug . '</div>';
} ?>
</div>
</footer>
</section>

<script type="text/javascript">
strings = <?php echo $appstr; ?>;
mobile_themes = <?php
	echo "['" . implode("','", array_map('javascript_escape', array_keys($mobile_themes))) . "']";
?>;
onLoadPage(function() {initLoginForm(<?php echo $captcha_complexity ?>)});
<?php if($action == 'login') { echo "onLoadPage(doPreload);\n"; } ?>
</script>

</body>
</html>
<?php
	end_flush_all_buffers();
	if (function_exists("fastcgi_finish_request")) {
		fastcgi_finish_request();
	}
