<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormTableSection.php');

class RolesWidget extends FormTableSection {
	var $model_name;
	
	static $preferred_order = array('access', 'view', 'list', 'edit', 'create', 'delete', 'import', 'export', 'report', 'approve', 'manage_personal');
	static $hide_actions = array('create');

	function init($params=null, $model=null) {
		parent::init($params, $model);
		if(! $this->id)
			$this->id = 'acl_roles';
		if($model) $this->model_name = $model->name;
	}
	
	function getInstructions($context=null) {
		if($context && ! empty($context['editable'])) {
			$lbl = 'LBL_EDIT_VIEW_DIRECTIONS';
			$trans = translate($lbl);
			if($trans != $lbl) return $trans;
		}
	}
	
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if($gen->getLayout()->getEditingLayout()) {
			$params = array();
			return $this->renderOuterTable($gen, $parents, $context, $params);		
		}
		
		$role_id = $row_result->getPrimaryKeyValue();
        $editable = $gen->getLayout()->getDefaultEditable();
        $ctxedit = $context;
        $ctxedit['editable'] = $editable;

        if ($this->model_name == 'User') {
            $editable = false;
            $categories = ACLAction::getUserActions($role_id);
        } else {
		    $categories = ACLRole::getRoleActions($role_id, 'module', true);
        }

		foreach ($categories as $mod => $acl) {
			$bean = AppConfig::module_primary_bean($mod);
			if (!$bean || !AppConfig::setting("model.detail.{$bean}.importable"))
				if (isset($categories[$mod]['module']['import']))
					unset($categories[$mod]['module']['import']);
			if (!$bean || !AppConfig::setting("model.detail.{$bean}.personal_data"))
					unset($categories[$mod]['module']['manage_personal']);
		}


		$no_edit = AppConfig::setting('acl.default_role.no_edit', array());
		foreach($no_edit as $key=>$value){
			if($value) {
				unset($categories[$key]);
			}
		}
        
        self::reorder_actions($categories);

        $action_names = ACLAction::setupCategoriesMatrix($categories);
        $action_names = self::reorder_names($action_names);
        $groups = ACLAction::groupCategories($categories);
		$body = '<div id="'.$this->id.'-body" class="acl-roles-body">';
		$options = [];
		$panel_edit = $editable ? '' : 'panel-readonly';

        foreach ($groups as $index => $group) {
			$body .= '<div class="row form-row"><div class="column form-cell expand"><div class="form-entry">';
			$body .= '<section class="card-outer panel-outer panel-default acl-outer panel-border panel-round '.$panel_edit.'">';
			$body .= '<div class="card-header panel-header"><h3 class="no-margin">' . $group['label'] . '</h3></div>';

            if (sizeof($action_names) > 0) {
				$body .= '<div class="card-header panel-subheader grid-body">';
				$body .= '<div class="grid-row">';
				$body .= '<div class="grid-cell acl-row-head acl-col-module">&nbsp;</div>';
                foreach ($action_names as $action_id => $action_name) {
                    $body .= '<div class="grid-cell acl-row-head">' . $action_name . '</div>';
                }
				$body .= '</div>';
				$body .= '</div>';
            }
            
            $body .= '<div class="card-body panel-body panel-pad-body grid-body">';

            foreach ($group['modules'] as $module_id => $module) {
            	
                $body .= '<div class="grid-row">';
                $icon = '<div class="input-icon theme-icon module-' . $module . '"></div> ';
                $body .= '<div class="grid-cell acl-col-module">' . $icon . $module_id . '</div>';

                foreach ($categories[$module] as $type => $actions) {
                	foreach ($action_names as $action_id => $action_name) {
                		$action = array_get_default($actions, $action_id);
                    	if($action) {
                    		$name = $action['accessName'];
							if (!empty($action['inactive'])) {
								$class = "aclInactive";
							} else {
								$class = "acl" . $action['cssclass'];
							}
						} else {
							$name = '';
							$class = 'aclInactive';
						}

                        if (! $editable || !$action || !empty($action['fixed']) || empty($action['accessName'])) {
                    		$accessName = $action['accessName'] ?? '';
                            $body .= '<div class="grid-cell acl-cell '.$class.'"><span class="acl-label">'.$accessName.'</span></div>';
                        } else {
                            $cursor = '';
                            $onclick = '';
                            $id = $module .'-'. $action_id;
							$options[$id] = $action['accessOptions'];
	                        $body .= '<div class="grid-cell acl-cell responsive '.$class.'">'
	                        	. '<input type="hidden" name="acl_value['.$id.']" id="acl-'.$id.'" value="'.$action['aclaccess'].'" />'
	                        	. '<span class="acl-label">' . $action['accessName'].'</span></div>';
                        }

                    }
                }

                $body .= '</div>';
            }

            $body .= '</div>';
            $body .= '</section>';
            $body .= '</div></div></div>';
        }
        
        if($editable) {
        	$gen->getLayout()->addScriptInclude('modules/ACLRoles/roles.js');
        	$params = ['options' => $options, 'tabIndex' => 0];
        	$gen->getFormObject()->addControl('RolesEditor', $this->id, $params);
        }

        $body = $this->renderOuterTable($gen, $parents, $context,
        	['body' => $body . '</div>',
        	 'description' => $this->getInstructions($ctxedit)]);

        return $body;
    }
    
    
    static function reorder_actions(&$cats) {
		foreach($cats as $mod => $acts) {
        	$new_acts = array();
        	foreach(self::$preferred_order as $act) {
        		if(isset($acts['module'][$act]))
        			$new_acts[$act] = $acts['module'][$act];
        	}
        	array_extend($new_acts, $acts['module']);
        	foreach(self::$hide_actions as $act)
				unset($new_acts[$act]);
        	$cats[$mod]['module'] = $new_acts;
		}
    }
    
	static function reorder_names($names) {
		$new_names = [];
       	foreach(self::$preferred_order as $act) {
       		if(isset($names[$act]))
        		$new_names[$act] = $names[$act];
		}
		array_extend($new_names, $names);
		return $new_names;
    }
    

	function loadUpdateRequest(RowUpdate $update, array $input) {
        $upd_access = array();
        if(isset($input['acl_value']) && is_array($input['acl_value'])) {
            foreach($input['acl_value'] as $action_id => $value) {
                if(! strpos($action_id, '-'))
                    continue;
                list($mod, $action) = explode('-', $action_id, 2);
                $upd_access[$mod][$action] = array('aclaccess' => $value);
            }

            $update->setRelatedData($this->id.'_rows', $upd_access);
        }
	}
	
	function validateInput(RowUpdate $update) {
		return true;
	}
	
	function afterUpdate(RowUpdate $update) {
		$row_updates = $update->getRelatedData($this->id.'_rows');
		if(! $row_updates)
			return;

		$reload = false;
		$role_id = $update->getPrimaryKeyValue();

		if($role_id != null) {
			foreach ($row_updates as $module => $actions) {
				ACLManager::make_consistent($actions);
				foreach($actions as $act_id => $row) {
					ACLRole::setAction($role_id, $module, $act_id, $row['aclaccess']);
					$reload = true;
				}
			}
		}

        if($reload)
            AppConfig::invalidate_cache('acl');
	}
}
?>
