<?php

function fetch_api_clients() {
	global $vendor_info;
	require_once('vendor_info.php');
	require_once('include/utils/dl_utils.php');
	$base = preg_replace('~/soap.php$~', '/', $vendor_info['license_server']);
	$url = $base.'json.php?module=PublicAPIClients&action=fetch_api_clients';
	$req = new RemoteRequest($url);
	$req->setProgressCallback(function($perc) { scheduler_task_progress(10.0 + $perc * 0.8); });
	$data = $req->fetch();
	$errmsg = $req->getError();
	if($errmsg) {
		scheduler_task_finish('error', "Error fetching API clients: $errmsg");
		return false;
	}
	$json = getJSONObj();
	if($data) $data = $json->decode($data);
	if (is_array($data) && array_get_default($data, 'status') == 'success') {
		scheduler_task_message('Downloaded client list');
		foreach ($data['clients'] as $cl) {
			$existing = ListQuery::quick_fetch('ClientEntity', $cl['id'], true, ['filter_deleted' => false]);
			if (!$existing && !$cl['enabled'])
				continue;
			if ($existing) {
				$update = RowUpdate::for_result($existing);
			} else {
				$update = RowUpdate::blank_for_model('ClientEntity');
			}
			$upd = [
				'id' => $cl['id'],
				'deleted' => (int)(!$cl['enabled']),
				'name' => $cl['name'],
				'publisher' => $cl['publisher'],
				'description' => $cl['description'],
				'grant_types' => $cl['grant_types'],
				'is_public' => 1,
				'secret' => $cl['secret'],
			];

			$local_redirects = $existing ? array_filter(preg_split("~[\\r\\n]+~", $existing->getField('redirect'))) : [];
			$redirects = array_filter(preg_split("~[\\r\\n]+~", $cl['redirect']));
			foreach ($redirects as $redir) {
				if (!in_array($redir, $local_redirects)) {
					$local_redirects[] = $redir;
				}
			}
			$upd['redirect'] = implode("\n", $local_redirects);

			$update->set($upd);
			$update->save();
		}
		scheduler_task_message('Saved client list');
	}
}

