<?php

require_once 'include/ListView/ListFormatter.php';

class AccountListFieldFormatter extends FieldFormatter {

	function formatRows(array &$model_fields, array &$rows, $union_key=null) {
		$unformatted = $rows;
        $ret = parent::formatRows($model_fields, $rows, $union_key);

		$fields = array('balance', 'balance_payable');

		$overdue = array();

		$load_ids = array_keys($ret);
		
		$lq1 = new ListQuery('Invoice');
		$lq1->addField(array('name' => 'billing_account_id', 'grouped' => true));
		$lq1->addTotal(array('name' => 'id', 'total' => 'count'));
		$lq1->addSimpleFilter('billing_account_id', $load_ids);
		$lq1->addSimpleFilter('due_date', gmdate('Y-m-d'), '<');
		$lq1->addSimpleFilter('amount_due', 0.005, '>');
		$lq1->addSimpleFilter('cancelled', 0);
		$lq1->setGroupBy('billing_account_id');

		$lq2 = new ListQuery('Bill');
		$lq2->addField(array('name' => 'supplier_id', 'grouped' => true));
		$lq2->addTotal(array('name' => 'id', 'total' => 'count'));
		$lq2->addSimpleFilter('supplier_id', $load_ids);
		$lq2->addSimpleFilter('due_date', gmdate('Y-m-d'), '<');
		$lq2->addSimpleFilter('amount_due', 0.005, '>');
		$lq2->addSimpleFilter('cancelled', 0);
		$lq2->setGroupBy('supplier_id');

		$lq = new ListQuery;
		$lq->addUnionQuery($lq1);
		$lq->addUnionQuery($lq2);

		$res = $lq->runQuery();
		foreach($res->getRows() as $row) {
			$overdue[$row['billing_account_id']] = true;
		}

        if ($this->format == 'html') {
			foreach ($ret as $key => &$row) {
				foreach ($fields as $f) {
					if ($this->isHighlighted($unformatted[$key], $f, $overdue)) {
						$this->highlightField($row, $f);
					}
				}
            }
        }

        return $ret;
    }

    function highlightField(&$row, $f) {
            if (isset($row[$f]))
	            $row[$f] = "<span class='overdueTask'>".$row[$f]."</span>";
    }

	function isHighlighted($row, $f, $overdue) {
		if ($f == 'balance' && !$row['is_supplier'])
			$check = 'credit_limit';
		else
			$check = 'purchase_credit_limit';
		return ($row[$check] > 0 && $row[$f] > $row[$check]) || isset($overdue[$row['id']]);
	}

}

