<?php

class MultiMap {
	static function listupdate_perform($mu, $handler_name, $fmt, $listResult, $uids, $no_edit) {
		if ($handler_name != 'GetMultiMap') return;
		$GLOBALS['log']->fatal($uids);
		self::displayMap($uids);
		exit;
	}

	private static function displayMap($uids) {
		require_once 'include/layout/FieldFormatter.php';

		$divId = uniqid('map_');

		$limit = 1000;

		$key = AppConfig::setting('company.google_maps_key');

		if (!$key) {
			$message = translate('LBL_MISSING_GMAPS_KEY', 'Accounts');
			echo '<div style="padding: 20px; padding-top: 40px;">'.$message.'</div>';
			die();
		}

		$models = ['Account', 'Contact', 'Lead'];

		$model = AppConfig::module_primary_bean($_REQUEST['target_module']);
		if (!in_array($model, $models)) return;

		$modelDef = new ModelDef($model);
		$lq = new ListQuery($model);
		$fields = $modelDef->getFieldDefinitions();
		$lq->addFields(array_keys($fields));
		$lq->addAclFilter('view');

		if ($uids != 'all') {
			$lq->addSimpleFilter('id', $uids);
		}

		$count = $lq->fetchResultCount();
		if ($count > $limit) {
			die("Too many records (no more than $limit allowed)");
		}

		global $locale;

		$prefix = $model == 'Account' ? 'billing_' : 'primary_';

		$cfg_key = 'company.google_maps_' . strtolower($model) . '_fields';
		$extra_fields = array_filter(explode('^,^', AppConfig::setting($cfg_key)));
		$fmt = new FieldFormatter('html', null, ['add_detail_links' => true]);
		$addresses = [];
		$result = $lq->fetchAll();
		foreach ($result->getRowIndexes() as $idx) {
			$row = $result->getRowResult($idx);
			$formatted = $fmt->formatRowResult($row);
			$addr = $locale->getLocaleBeanFormattedAddress($row, $prefix, 'csv');
			if (!empty($addr)) {
				$address = [
					'address' => $addr,
					'name' => $row->getField('name', '', true),
					'name_raw' => $row->getField('name'),
					'extra' => [],
				];
				$n = 0;
				foreach ($extra_fields as $fname) {
					$n++;
					if ($n > 3) break;
					$value = array_get_default($formatted, $fname);
					if (!strlen($value)) continue;
					$def = $row->getFieldDefinition($fname);
					$label = translate($def['vname'], array_get_default($def, 'vname_module', $row->getModuleDir()));

					$address['extra'][] = [
						'label' => $label,
						'value' => $value,
					];
				}
				$addresses[] = $address;
			}
		}

		$json = getJSONobj();
		$addresses_json = $json->encode($addresses);



		if (empty($_REQUEST['fixedSize'])) {
			echo '<div id="'.$divId.'" style="position: absolute; top: 0; bottom: 0; left:0; right: 0;"></div>';
		} else {
			echo '<div id="'.$divId.'" style="min-height: '.$_REQUEST['fixedSize'].'px"></div>';
		}

		?>

		<script>
			multiMapInit(<?php echo "'$divId', ", $addresses_json, ", '$key'" ?>);
		</script>
		<?php
	}
}
