<?php

require_once 'modules/PDFTemplates/PDFTemplateProviderBase.php';

class AccountPDFTemplateProvider extends PDFTemplateProviderBase
{

	protected function __construct($type) {	}


	public static function template_types()
	{
		return array(
			'statement' => translate('LBL_PDF_TYPE_STATEMENT', 'Accounts'),
		);
	}

	public static function template_skeleton($type)
	{
		$ret = array('sections' => array(
			array(
				'name' => 'header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_HEADER',
				'height' => 150,
				'elements' => array (),
			),
			array(
				'name' => 'invoice_info',
				'type' => 'table',
				'title' => 'LBL_SKEL_INVOICE_INFO',
				'columns' => array (),
			),
			array(
				'name' => 'invoice_payments_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_INVOICE_PAYMENTS_HEADER',
				'height' => 50,
				'elements' => array (),
			),
			array(
				'name' => 'payments_list',
				'type' => 'table',
				'title' => 'LBL_SKEL_PAYMENTS_LIST',
				'offset' => 40,
				'columns' => array (),
			),
			array(
				'name' => 'credit_payments_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_CREDIT_PAYMENTS_HEADER',
				'height' => 50,
				'elements' => array (),
			),
			array(
				'name' => 'credits_list',
				'type' => 'table',
				'title' => 'LBL_SKEL_CREDITS_LIST',
				'offset' => 40,
				'columns' => array (),
			),
			array(
				'name' => 'invoice_footer',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_INVOICE_FOOTER',
				'height' => 50,
				'columns' => array (),
			),
			array(
				'name' => 'footer',
				'data_id' => 'header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_FOOTER',
				'height' => 40,
				'elements' => array (),
			),
		));
		foreach ($ret['sections'] as &$sec) $sec['title'] = translate($sec['title'], 'Accounts');
		return $ret;
	}
	public static function template_sample_data($type)
	{
		$datefmt = AppConfig::setting('locale.defaults.date_format');
		$data = array(
			'header' => array(
				'todays_date' => array(translate('LBL_TODAYS_DATE'), date($datefmt), ),
				'name' => array(translate('LBL_SAMPLE_HEADER_NAME', 'Accounts'), translate('LBL_SAMPLE_HEADER_NAME_VALUE', 'Accounts'), ),
				'tax_information' => array(translate('LBL_SAMPLE_HEADER_TAX', 'Accounts'), translate('LBL_SAMPLE_HEADER_TAX_VALUE', 'Accounts'), ),
				'billing_address' => array(translate('LBL_SAMPLE_HEADER_BILLING_ADDRESS_LABEL', 'Accounts'), translate('LBL_SAMPLE_HEADER_BILLING_ADDRESS_VALUE', 'Accounts'), ),
				'shipping_address' => array(translate('LBL_SAMPLE_HEADER_SHIPPING_ADDRESS_LABEL', 'Accounts'), translate('LBL_SAMPLE_HEADER_SHIPPING_ADDRESS_VALUE', 'Accounts'), ),
				'balance' => array(translate('LBL_SAMPLE_HEADER_BALANCE', 'Accounts'), translate('LBL_SAMPLE_HEADER_BALANCE_VALUE', 'Accounts'), ),
			),
			'invoice_info' => array(
				'number' => array(translate('LBL_SAMPLE_INVOICE_NUMBER', 'Accounts'), translate('LBL_SAMPLE_INVOICE_NUMBER_VALUE', 'Accounts'), ),
				'due_date' => array(translate('LBL_SAMPLE_DUE_DATE', 'Accounts'), translate('LBL_SAMPLE_DUE_DATE_VALUE', 'Accounts'), ),
				'subject' => array(translate('LBL_SAMPLE_INVOICE_SUBJECT', 'Accounts'), translate('LBL_SAMPLE_INVOICE_SUBJECT_VALUE', 'Accounts'), ),
				'amount' => array(translate('LBL_SAMPLE_INVOICE_AMOUNT', 'Accounts'), translate('LBL_SAMPLE_INVOICE_AMOUNT_VALUE', 'Accounts'), ),
			),
			'invoice_footer' => array(
				'amount_due' => array(translate('LBL_SAMPLE_INVOICE_AMOUNT_DUE', 'Accounts'), translate('LBL_SAMPLE_INVOICE_AMOUNT_DUE_VALUE', 'Accounts'), ),
			),
			'payments_list' => array(
				'date' => array(translate('LBL_SAMPLE_PAYMENT_DATE', 'Accounts'), translate('LBL_SAMPLE_PAYMENY_DATE_VALUE', 'Accounts'), ),
				'payment_id' => array(translate('LBL_SAMPLE_PAYMENT_ID', 'Accounts'), translate('LBL_SAMPLE_PAYMENT_ID_VALUE', 'Accounts'), ),
				'payment_ref' => array(translate('LBL_SAMPLE_PAYMENT_REF', 'Accounts'), translate('LBL_SAMPLE_PAYMENT_REF_VALUE', 'Accounts'), ),
				'type' => array(translate('LBL_SAMPLE_PAYMENT_TYPE', 'Accounts'), translate('LBL_SAMPLE_PAYMENT_TYPE_VALUE', 'Accounts'), ),
				'amount' => array(translate('LBL_SAMPLE_PAYMENT_AMOUNT', 'Accounts'), translate('LBL_SAMPLE_PAYMENT_AMOUNT_VALUE', 'Accounts'), ),
			),
			'credits_list' => array(
				'date' => array(translate('LBL_SAMPLE_CREDIT_DATE', 'Accounts'), translate('LBL_SAMPLE_CREDIT_DATE_VALUE', 'Accounts'), ),
				'number' => array(translate('LBL_SAMPLE_CREDIT_NUMBER', 'Accounts'), translate('LBL_SAMPLE_CREDIT_NUMBER_VALUE', 'Accounts'), ),
				'subject' => array(translate('LBL_SAMPLE_CREDIT_SUBJECT', 'Accounts'), translate('LBL_SAMPLE_CREDIT_SUBJECT_VALUE', 'Accounts'), ),
				'amount' => array(translate('LBL_SAMPLE_CREDIT_AMOUNT', 'Accounts'), translate('LBL_SAMPLE_CREDIT_AMOUNT_VALUE', 'Accounts'), ),
			),
		);
		$ret = array();
		foreach ($data as $k => $v) {
			$ret[$k] = array();
			foreach ($v as $kk => $vv) {
				$ret[$k][$kk] = array('label' => $vv[0], 'value' => $vv[1]);
			}
		}
		return $ret;
	}

	public static function create($type)
	{
		return new AccountPDFTemplateProvider($type);
	}

	public function begin()
	{
	}

	public function end()
	{
	}

	public function getFileName()
	{
		$filename = translate('LBL_PDF_STATEMENT', 'Accounts');
		$nm = $this->row->getField('name');
		$filename .= '_' . $nm;
		$filename = preg_replace('~\.+$~', '', trim($filename));
		$filename .= '.pdf';
		return $filename;
	}

	public function iterate(PDFTemplatePrinter $p)
	{
		$this->iterateHeaderFooter($p, 'header');
		$this->iterateInvoices($p);
		$this->iterateHeaderFooter($p, 'footer');
	}

	protected function iterateInvoices(PDFTemplatePrinter $p) {
		$items = $this->getData('invoice_info');
		$footers = $this->getData('invoice_footer');
		$payments = $this->getData('payments_list');
		$credits = $this->getData('credits_list');
		foreach ($items as $idx => $item) {
			$p->renderSection($this->findSection('invoice_info'), array($item));
			if (!empty($payments[$idx])) {
				$p->renderSection($this->findSection('invoice_payments_header'), array());
				$p->renderSection($this->findSection('payments_list'), $payments[$idx]);
			}
			if (!empty($credits[$idx])) {
				$p->renderSection($this->findSection('credit_payments_header'), array());
				$p->renderSection($this->findSection('credits_list'), $credits[$idx]);
			}
			$p->renderSection($this->findSection('invoice_footer'), $footers[$idx]);
		}
	}

	protected function iterateHeaderFooter(PDFTemplatePrinter $p, $name)
	{
		$p->renderSection($this->findSection($name), $this->getData('header'));
	}

	public function getRow()
	{
		return $this->row;
	}
	public function getData($section = null)
	{
		if (empty($this->data))
			$this->createData();
		if (!$section)
			return $this->data;
		return array_get_default($this->data, $section, array());
	}

	protected function createData() {
		$row =& $this->row;
		$datefmt = AppConfig::setting('locale.defaults.date_format');
		$data = array(
			'todays_date' => date($datefmt),
			'name' => $row->getField('name'),
			'tax_information' => $row->getField('tax_information'),
			'billing_address' => $this->_address('billing_') ,
			'shipping_address' => $this->_address('shipping_'),
		);

		$data = array_merge($this->header_data, $data);
		$this->data['header'] = $data;
		$this->createInvoiceData();
	}

	protected function createInvoiceData() {
		$rows = array();
		$row = $this->row;
        $lq = new ListQuery('Account', null, array('link_name' => 'invoice', 'parent_key' => $row->getField('id')));
        $lq->addSimpleFilter('cancelled', null, 'false');
        $lq->addSimpleFilter('amount_due', null, 'non_zero');
        $invoices = $lq->fetchAllObjects();

        $lq = new ListQuery('Account', null, array('link_name' => 'credits', 'parent_key' => $row->getField('id')));
        $lq->addSimpleFilter('cancelled', null, 'false');
        $lq->addSimpleFilter('amount_due', null, 'non_zero');
		$lq->addSimpleFilter('apply_credit_note', null, 'false');
		$credits = $lq->fetchAllObjects();
        
        $invoices += $credits;
		$last_currency = 0;
		$single_currency = false;
        $balance = $balance_usdollar = 0;
		$currency = new Currency;
        $currency->retrieve(-99);
		$currency_id = $default_currency_id = $currency->id;

		$idx = 0;
		foreach ($invoices as $invoice) {
            $currency = new Currency();
            $currency->retrieve($invoice->currency_id, false);
            if ($last_currency && ($last_currency !== $currency->id)) {
                $single_currency = false;
            }
            $last_currency = $currency->id;
			$currency_id = $currency->id;
			$number_field = $invoice->object_name == 'CreditNote' ? 'credit_number' : 'invoice_number';
            $rows[] = array(
                'due_date' => $invoice->due_date,
                'number' => $invoice->prefix.$invoice->$number_field,
                'subject' => $invoice->name,
                'amount' => $this->currency_format($invoice->amount, $currency->id),
			);
			$total_paid = $this->createPaymentsData($idx, $invoice, $currency);
			$amount_due = $invoice->amount - $total_paid;
			if ($invoice->object_name == 'CreditNote') {
				$amount_due = -$amount_due;
			}
            $balance += $amount_due;
            $balance_usdollar += $currency->convertToDollar($amount_due);
			$idx++;
			$this->data['invoice_footer'][] = array(
				'amount_due' => $this->currency_format($amount_due, $currency->id)
			);
		}
		$this->data['invoice_info'] = $rows;
		$cid = $row->getField('currency_id', '-99');
		$account_currency = new Currency();
		$account_currency->retrieve($cid);
		$account_balance = $account_currency->convertFromDollar($balance_usdollar);
		$this->data['header']['balance'] = $single_currency ? $this->currency_format($balance, $currency_id) : $this->currency_format($balance_usdollar, $default_currency_id);
		$this->data['header']['balance'] = $this->currency_format($account_balance, $account_currency->id);
	}

	protected function createPaymentsData($idx, $invoice, $currency) {
		global $timedate;
		$total_paid = 0;
		if ($invoice->object_name == 'CreditNote') {
			$payments = CreditNote::get_payments($invoice->id, $invoice->currency_id);
			$notes = array();
		} else {
			$notes = Invoice::get_attached_credit_notes($invoice->id, null, null);
			$payments = Invoice::get_payments($invoice->id, $invoice->currency_id);
		}

		if (count($payments) || count($notes)) {
			if (count($payments)) {
				foreach ($payments as $i => $payment) {
					$total_paid += $payment['allocated'];
					$row = array(
						'date' => $timedate->to_display_date($payment['payment_date'], false),
						'amount' => $this->currency_format($payment['allocated'], $currency->id),
						'type' => translate('payment_type_dom', '', $payment['payment_type'], true),
						'payment_id' => $payment['prefix'] . $payment['payment_id'],
						'payment_ref' => $payment['customer_reference'],
					);
					$this->data['payments_list'][$idx][] = $row;
				}
			}

			if (count($notes)) {
				$data = array();
				foreach ($notes as $note) {
					$row = array(
						'date' => $timedate->to_display_date($note['due_date'], false),
						'number' => $note['prefix'].$note['credit_number'],
						'subject' => $note['name'],
						'amount' => $this->currency_format($note['amount'], $currency->id),
					);
					$this->data['credits_list'][$idx][] = $row;
					$total_paid += $note['amount'];
				}
			}
		}
		return $total_paid;
	}
	
	protected function currency_format($amount, $currency_id = false, $no_round = false) {
		$params = array('currency_symbol' => false, 'type' => 'pdf');
		if ($currency_id) {
			$params['currency_symbol'] = true;
			$params['currency_id'] = $currency_id;
		}
		if($no_round)
			$params['round'] = null;
		return currency_format_number($amount, $params);
	}

	protected function _address($pfx, $row = null)
	{
		$focus = $row ? $row : $this->row;
		global $locale;
		$ret = $locale->getLocaleBeanFormattedAddress($focus, $pfx, false);
		return $ret;

	}

	public function getNoteTitle() {
		return translate('LBL_PDF_STATEMENT', 'Accounts');
	}

	public function getSignedDocumentTitle() {
		$prefix =
			$this->row
			? '[' . $this->row->getField('name') . '] '
			: '';

		return $prefix . $this->getNoteTitle();
	}

	public function getEmailTitle()
	{
		return translate('LBL_EMAILED_STATEMENT', 'Accounts');
	}

	public static function get_extra_sources() {
		return array(
			array(
				'module' => 'Accounts',
				'model' => 'Account',
				'prefix' => 'member_of',
				'label' => 'LBL_MEMBER_OF',
			),
			array(
				'module' => 'Contacts',
				'model' => 'Contact',
				'prefix' => 'primary_contact',
				'label' => 'LBL_PRIMARY_CONTACT',
			),
		);
	}
}

