<?php

class StripeAuthorization {
	private $request = array();
	private $account = null;
	private $contact = null;
	private $error = null;
	public function __construct($req) {
		$this->request = $req;
		$this->account = ListQuery::quick_fetch('Account', $this->input('account_id'));
		$this->contact = ListQuery::quick_fetch('Contact', $this->account->getField('primary_contact_id'));
	}

	public function processRequest() {
		if ($this->input('process'))
			if ($this->process()) {
				$GLOBALS['pageInstance']->add_js_literal("SUGAR.popups.close(); SUGAR.ui.refreshTarget('DetailForm');", null, LOAD_PRIORITY_BODY);
				return;
			}
		$this->render();
	}

	private function process() {
		$params = array(
			'source' => array(
				'object' => 'card',
				'number' => $this->input('authorize_card_number'),
				'exp_month' => $this->input('authorize_card_month'),
				'exp_year' => $this->input('authorize_card_year'),
				'cvc' => $this->input('authorize_card_code'),
				'name' => $this->input('authorize_card_first_name') . ' ' . $this->input('authorize_card_last_name'),
				'address1' => $this->input('authorize_card_address'),
				'city' => $this->input('authorize_card_city'),
				'state' => $this->input('authorize_card_state'),
				'country' => $this->input('authorize_card_country'),
				'zip' => $this->input('authorize_card_zip'),
			),
		);

		$customer_id = $this->account->getField('stripe_customer_id');
		\Stripe\Stripe::setApiKey(AppConfig::setting('company.merchant_login_stripe'));
		try {
			if ($customer_id) {
				$cu = \Stripe\Customer::retrieve($customer_id);
				$cu->source = $params['source'];
				$result = $cu->save();
			} else {
				$result = \Stripe\Customer::create($params);
				$upd = RowUpdate::for_result($this->account);
				$upd->set('stripe_customer_id', $result->id);
				$upd->save();
			}
			return true;
		} catch (\Stripe\Exception\ApiErrorException $e) {
			$this->error = $e->getMessage();
		}
	}

	private function render() {
		if ($this->error) {
			$msgs = [ [to_html($this->error), 'error'] ];
			echo display_flash_messages_default($msgs, false);
		}
		
		list($fields_card, $fields_customer) = $this->getFields();
		$layout = $this->getLayout($fields_card, $fields_customer);
		$result = new RowResult;
		$result->row = array();
		$fields = array_merge($fields_card, $fields_customer);
		$result->fields = $fields;
		foreach ($fields as $f => $spec) {
			$def = null;
			if (isset($spec['contact_field']))
				$def = $this->contactField($spec['contact_field']);
			$result->row[$f] = $this->input($f, $def);
		}
		
		require_once('include/layout/FieldFormatter.php');
		require_once('include/layout/forms/FormGenerator.php');
		$model = new ModelDef('Account');
		$model->addDisplayFieldDefinitions($result->fields);
		$gen = FormGenerator::html_form($model, $layout, 'stripeAuthForm');
		$gen->getFormObject()->max_ref_width = 50;
		$gen->getFormObject()->addHiddenFields(
			array(
				'account_id' => $this->input('account_id'),
				'module' => 'Accounts',
				'action' => 'StripeAuthorize',
				'process' => '1',
			)
		);
		$result->formatted = $result->row;
		$gen->renderForm($result);
		echo $gen->getResult();
		$gen->exportIncludes();
	}

	private function input($name, $default = null) {
		return array_get_default($this->request, $name, $default);
	}

	private function getFields() {
		$fields = array(
			array(
				'authorize_card_number' => array(
					'vname' => 'LBL_AUTHORIZE_CARD_NUMBER',
					'type' => 'varchar',
					'required' => true,
					'width' => 45,
					'editable' => true,
				),
				'authorize_card_code' => array(
					'vname' => 'LBL_AUTHORIZE_CARD_CODE',
					'type' => 'varchar',
					'required' => true,
					'width' => 6,
					'editable' => true,
				),
				'authorize_card_month' => array(
					'vname' => 'LBL_AUTHORIZE_CARD_MONTH',
					'type' => 'varchar',
					'required' => true,
					'width' => 6,
					'editable' => true,
				),
				'authorize_card_year' => array(
					'vname' => 'LBL_AUTHORIZE_CARD_YEAR',
					'type' => 'varchar',
					'required' => true,
					'width' => 6,
					'editable' => true,
				),
			),
			array(
				'authorize_card_first_name' => array(
					'vname' => 'LBL_AUTHORIZE_CARD_FIRST_NAME',
					'type' => 'varchar',
					'editable' => true,
					'contact_field' => 'first_name',
				),
				'authorize_card_last_name' => array(
					'vname' => 'LBL_AUTHORIZE_CARD_LAST_NAME',
					'type' => 'varchar',
					'editable' => true,
					'contact_field' => 'last_name',
				),
				'authorize_card_address' => array(
					'vname' => 'LBL_AUTHORIZE_CARD_ADDRESS',
					'type' => 'varchar',
					'editable' => true,
					'contact_field' => 'primary_address_street',
				),
				'authorize_card_city' => array(
					'vname' => 'LBL_AUTHORIZE_CARD_CITY',
					'type' => 'varchar',
					'editable' => true,
					'contact_field' => 'primary_address_city',
				),
				'authorize_card_state' => array(
					'vname' => 'LBL_AUTHORIZE_CARD_STATE',
					'type' => 'varchar',
					'editable' => true,
					'contact_field' => 'primary_address_state',
				),
				'authorize_card_country' => array(
					'vname' => 'LBL_AUTHORIZE_CARD_COUNTRY',
					'type' => 'varchar',
					'editable' => true,
					'contact_field' => 'primary_address_country',
				),
				'authorize_card_zip' => array(
					'vname' => 'LBL_AUTHORIZE_CARD_ZIP',
					'type' => 'varchar',
					'editable' => true,
					'contact_field' => 'primary_address_postalcode',
				),
			),
		);
		return $fields;
	}

	private function getLayout($fields_card, $fields_customer) {
		$layout = array(
			'type' => 'config',
			'editable' => true,
			'buttons_position' => 'bottom',
			'form_buttons' => $this->getFormButtons(),
			'sections' => array(
				array(
					'vname' => 'LBL_AUTHORIZE_CARD_DETAILS',
					'label_position' => 'top',
					'columns' => 2,
					'elements' => array_keys($fields_card),
				),
				array(
					'vname' => 'LBL_AUTHORIZE_CUSTOMER_DETAILS',
					'label_position' => 'top',
					'columns' => 2,
					'elements' => array_keys($fields_customer),
				),
			),
		);
		return $layout;
	}

	private function getFormButtons() {
		$buttons = array(
			'authorize' => array(
				'icon' => 'icon-accept',
				'label' => translate('LBL_AUTHORIZE_BUTTON_LABEL', 'Accounts'),
				'params' => array(),
				'async' => true,
				'align' => 'right',
				'perform' => 'SUGAR.ui.authorizeCard(this.form); return false;',
			),
		);
		return $buttons;
	}

	private function contactField($name) {
		if ($this->contact)
			return $this->contact->getField($name);
	}
}

$auth = new StripeAuthorization($_REQUEST);
$auth->processRequest();
