<?php
class ActivityUpdater {

    /**
     * @var RowUpdate
     */
    private $activity_update;

    static $updated = array();

    /**
     * @var array
     */
    private $parent_modules = array(
        'Leads', 'Contacts', 'Prospects', 'Cases', 'Bugs', 'Project'
    );

    /**
     * @param RowUpdate $activity_update
     */
	public function __construct($activity_update) {
        $this->activity_update = $activity_update;
    }

    /**
     * Update activity's parent record
     *
     * @return void
     */
	public function update() {
       	$parent_type = $this->activity_update->getField('parent_type');
        $parent_id = $this->activity_update->getField('parent_id');

        if ( ($parent_type && $parent_id) && $this->needUpdate($parent_type) )
        	$this->fetchAndUpdate(AppConfig::module_primary_bean($parent_type), $parent_id);

        $account_id = $this->activity_update->getField('account_id');
        if($account_id)
        	$this->fetchAndUpdate('Account', $account_id);

        $contact_id = $this->activity_update->getField('contact_id');
        if($contact_id)
        	$this->fetchAndUpdate('Contact', $contact_id);
	}

	public function fetchAndUpdate($model, $id) {
		if(! empty(self::$updated["$model:$id"]))
			return;
		$target = ListQuery::quick_fetch($model, $id);
		if ($target) {
			$target = RowUpdate::for_result($target);
			$this->updateRecord($target);
			self::$updated["$model:$id"] = 1;
		}
	}

	public function updateRecord(RowUpdate $target) {
		$target->set(
			array(
                'last_activity_date' => gmdate('Y-m-d H:i:s'),
				'date_modified' => gmdate('Y-m-d H:i:s'),
			)
		);
		$target->setSource('last_activity');
		$target->prohibit_workflow = $this->activity_update->prohibit_workflow;
		$target->as_user_id = $this->activity_update->as_user_id;
		return $target->validate() && $target->save();
	}

    /**
     *
     * @param string $parent_type
     * @return bool
     */
    private function needUpdate($parent_type) {
        return in_array($parent_type, $this->parent_modules);
    }
}
?>
