<?php

require_once 'include/ListView/ListFormatter.php';

class ActivityListFieldFormatter extends FieldFormatter {

	function formatRows(array &$model_fields, array &$rows, $union_key=null) {
        $unformatted = $rows;
        $ret = parent::formatRows($model_fields, $rows, $union_key);
        $lang_strings = array_merge(
			AppConfig::setting("lang.strings.current.ActivityLog"),
			AppConfig::setting("lang.dashlet_strings.current.ActivityLog.ActivityLogDashlet")
		);

        foreach ($ret as $key => &$row) {
            $template = $this->formatStatus($row, $unformatted[$key], $lang_strings);
            $this->formatActivity($row, $unformatted[$key], $model_fields, $template);
        }

        return $ret;
    }

	function formatStatus(&$row, $unformatted_row, $lang_strings) {
        $status = $unformatted_row['status'];
        $date_start = $unformatted_row['moved_to'];
        $module = $row['module_name'];
        switch ($status) {
            case '';
                $formatted = '';
                break;
			case 'moved':
				$tpl = $lang_strings['LBL_MOVED'];
				$params = array($row['moved_to']);
				break;
			case 'closed':
			case 'reopened':
				$tpl = $lang_strings['LBL_'.strtoupper($status)];
				$params = array();
				break;
			case 'locked':
			case 'unlocked':
				$tpl = $lang_strings['LBL_'.strtoupper($status)];
				$params = array();
				break;
			case 'delete':
			case 'deleted':
				$tpl = $lang_strings['LBL_DELETED'];
				$params = array();
				break;
            case 'converted':
				$tpl = $lang_strings['LBL_CONVERTED_TO'];
				$params = array($this->formatConverted($row, $unformatted_row));
                break;
            case 'created':
				if (!empty($row['converted_to_id'])) {
					$tpl = $lang_strings['LBL_CONVERTED_FROM'];
					$params  = array($this->formatConverted($row, $unformatted_row));
				} elseif ($date_start){
                    global $timedate;
                    $date_start = $timedate->to_display_date_time($date_start);
                    if ($module == 'ProjectTask')
                        $date_start = $timedate->to_display_date($date_start);
					$tpl = $lang_strings['LBL_SCHEDULED'];
					$params = array($date_start);
                } else {
					$tpl = $lang_strings['LBL_CREATED'];
					$params = array();
                }
				break;
			case 'expire_warn':
				$tpl = $lang_strings['LBL_EXPIRE_WARN'];
				$params = array();
				break;
            default:
				$tpl = $lang_strings['LBL_STATUS_DEFAULT'];
				$params = array();
                break;
        }
		return array($tpl, $params);
    }

	function formatConverted(&$row, $unformatted_row) {
        $icon = '';

        if ($this->format == 'html') {
            global $app_list_strings;
            $icon_name = $unformatted_row['converted_to_type'];
            $mod_name = array_get_default($app_list_strings['moduleListSingular'], $icon_name, '');
            $icon = get_image($icon_name, 'title="' . to_html($mod_name) . '" alt="" border=0');
            $icon .= ' ';
        }

        return $icon . $row['converted_to'];
	}

	function formatActivity(&$row, $unformatted_row, $model_fields, $template) {
		$icon = '';
		$model_name = AppConfig::module_primary_bean($unformatted_row['module_name']);
		global $app_list_strings;
        $mod_strings = array_merge(
			AppConfig::setting("lang.strings.current.ActivityLog"),
			AppConfig::setting("lang.dashlet_strings.current.ActivityLog.ActivityLogDashlet")
		);

        if ($this->format == 'html') {
            $module_field = $model_fields['record_item']['dynamic_module'];
            $icon_name = $unformatted_row[$module_field];
            $mod_name = array_get_default($app_list_strings['moduleListSingular'], $icon_name, '');
            $icon = get_image($icon_name, 'title="' . to_html($mod_name) . '" alt="" border=0');
            $icon .= ' ';
        }

		$params = $template[1];
		array_unshift($params, $icon . $row['record_item']);
		array_push($params, $row['assigned_user']);
		array_unshift($params, $template[0]);
		$record = $row['record_item_id'];
		$log_record = $row['id'];
		$row['activity_text'] = call_user_func_array('sprintf', $params);
		$restore_confirm_lbl = $mod_strings['LBL_CONFIRM_RESTORE'];
		$restore_lbl = $mod_strings['LBL_RESTORE'];
		$row['restore_record'] = '<button name="restore_record" type="button" class="input-button first" style="" tabindex="" onclick="restore(\''.$model_name.'\',\''.$record.'\',\''.$log_record.'\')">
									<div class="input-icon left icon-return"></div>
									<span class="input-label" id="listView-a208-create-label">'.$restore_lbl.' '.$mod_name.'</span>
								</button>
								
								<script>
									function restore(mod, id, logid) {
									    if (confirm("'.$restore_confirm_lbl.'") == true) {
										  SUGAR.util.navigateTo(\'index.php?module=ActivityLog&action=RestoreRecord&restoreon=\'+mod+\'&restore=\'+id+\'&logid=\'+logid+\'\');
										} 
									}
								</script>';
    }
}
