<?php

action_restricted_for('demo');

global $db;

$models = AppConfig::setting('modinfo.primary_beans');

$fields = ListQuery::quick_fetch_all('DynField', true, null, true);

$defined = [];
$undefined = [];

foreach ($fields as $f) {
        $defined[$f['custom_module']][] = $f['name'];
}

foreach ($models as $modelName) {
	try {
		$modelDef = new ModelDef($modelName);
	} catch (\IAHModelError $e) {
		continue;
	}
	$module = AppConfig::module_for_model($modelName);
	$table_name = AppConfig::setting("model.detail.$modelName.table_name") . "_cstm";
	try {
        $res = $db->query("explain $table_name");
		while ($row = $db->fetchByAssoc($res)) {
			$fname = $row['Field'];
            if ($fname == 'id_c') continue;
			$def = $modelDef->getFieldDefinition($fname);
			if (!empty($def['inferred'])) {
				continue;
			}
			if (!isset($defined[$module]) || !in_array($fname, $defined[$module])) {
				$undefined[$modelName]['model'] = $modelName;
				$undefined[$modelName]['label'] = translate('LBL_MODULE_TITLE', $module);
				$undefined[$modelName]['custom_table'] = $table_name;
				$undefined[$modelName]['fields'][] = $fname;
            }
		}
	} catch (\Exception $e) {
	}
}
echo get_module_title('Repair', $mod_strings['LBL_CLEANUP_CUSTOM'], false);

echo '<div class="detailOuter">';
echo '<p>' . $mod_strings['LBL_CLEANUP_CUSTOM_DETAILS'] . '</p>';

if (empty($undefined)) {
	require_once('XTemplate/xtpl.php');
	$tpl = new XTemplate('include/utils/FlashMessages.html');
	$tpl->assign('ICON', '<span class="uii uii-final"></span>');
	$tpl->assign('PANEL_CLASS', 'panel-final');
	$tpl->assign('MESSAGE', $mod_strings['LBL_CLEANUP_CUSTOM_NO_ACTION']);
	$tpl->parse('static.message');
	$tpl->parse('static');
	echo $tpl->text('static');
	echo '</div>';
	return;
}


$queries = [];

if (!empty($_POST)) {
	foreach ($undefined as $model => $data) {
		$toRemove = array_filter($data['fields'], function($f) use ($model) {
			return !empty($_POST["{$model}_{$f}"]);
		});
		if (!empty($toRemove)) {
			$queries[] = "ALTER TABLE {$data['custom_table']} DROP " . join(", DROP ", $toRemove) . ";";
		}
	}
}

if (!empty($queries)) {
	if (empty($_POST['__dry_run__'])) {
		foreach ($queries as $q) {
			$db->query($q);
		}
		require_once('XTemplate/xtpl.php');
		$tpl = new XTemplate('include/utils/FlashMessages.html');
		$tpl->assign('ICON', '<span class="uii uii-info"></span>');
		$tpl->assign('PANEL_CLASS', 'panel-info');
		$tpl->assign('MESSAGE', $mod_strings['LBL_CLEANUP_CUSTOM_DONE']);
		$tpl->parse('static.message');
		$tpl->parse('static');
		echo $tpl->text('static');
		echo '</div>';
		return;
	}
	pr2(join("\n", $queries));
}

?>
	<form id="cleanupcustom" name="cleanupcustom" method="post" autocomplete="off" onsubmit="toggleSendForm()">
		<?php echo create_csrf_input(); ?>
		<input type='hidden' name='action' value='CleanupCustom'>
		<input type='hidden' name='module' value='Administration'>
	<table class='tabForm' cellpadding="0" cellspacing="0" width="500">
	<tr><th class="dataLabel" colspan="2">
	<h4 class="dataLabel"><?php echo $mod_strings['LBL_CLEANUP_CUSTOM_SELECT'];?></h4>
	</th></tr>
	<tr><td>
		<table cellpadding="0" cellspacing="0" border="0" width="100%" style="margin: 0.5em 0">
<?php foreach ($undefined as $model => $data) : ?>
		<tr><td class="dataLabel" colspan="2">
			<p><label><input type="checkbox" value="1" checked="checked" onchange="toggleCustomFieldForDeletion('<?php echo $model;?>', this.checked)">
			<b><?php echo $data['label']; ?></b></label></p>
		</td></tr>
<?php foreach ($data['fields'] as $f) : ?>
		<tr><td class="dataLabel" width="10">&nbsp;</td>
			<td class="dataLabel">
			<p><label><input type="checkbox" class="customCleanup_<?php echo $model;?>" id="<?php echo $model . '.' . $f ?>" name="<?php echo $model . '_' . $f ?>" value="1" <?php if ($_SERVER['REQUEST_METHOD'] !== 'POST' || !empty($_POST[$model . '_' . $f])) echo 'checked="checked"';?>>
<?php echo $f; ?></label></p>
			</td>
		</tr>
<?php endforeach; ?>
<?php endforeach; ?>
		<tr><td class="dataLabel" colspan="2">
<hr />
			<p><label><input type="checkbox" name="__dry_run__" value="1" checked="checked">Dry run (display SQL)</label></p>
			</td>
		</tr>
</table>
		<tr><td class="dataLabel" colspan="2">
		<button type="submit" class="input-button input-outer" id="sendform"><div class="input-icon icon-accept left"></div><span class="input-label"> <?php echo $mod_strings['LBL_GO']; ?> </span></button>
		</td></tr>
</table>
</form>
</div>
<script>
function toggleCustomFieldForDeletion(model, checked) {
	var fields = document.getElementsByClassName("customCleanup_" + model);
	for (var i = 0; i < fields.length; i++) {
		fields[i].checked = checked;
	}
}
</script>
