<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');

action_restricted_for('demo');
ini_set( "max_execution_time", "0" );

global $current_user, $mod_strings;
if(! is_admin($current_user))
	die('Admin Only Section');

require_once 'include/database/ListQuery.php';

$contacts = getRecordsWithInvalidEmails('Contact');
$accounts = getRecordsWithInvalidEmails('Account');
$leads = getRecordsWithInvalidEmails('Lead');

$contactsView = getIncorrectRecordsHtml($contacts, 'Contacts');
$accountsView = getIncorrectRecordsHtml($accounts, 'Accounts');
$leadsView = getIncorrectRecordsHtml($leads, 'Leads');

echo <<<EOF
<div style="font-size: 120%; max-width: 700px;">
<table class="tabForm" cellpadding="0" cellspacing="0" width="100%">
	<tr>
		<th class="dataLabel" colspan="2">
			<h2 style="text-align:center"><b>{$mod_strings['LBL_VALIDATE_EMAILS_TITLE']}</b></h2>
		</th>
	</tr>
	{$contactsView}
	{$accountsView}
	{$leadsView}
</table>
</div>
EOF;

function getIncorrectRecordsHtml($records, $module) {
	global $mod_strings;
	$module_name = translate_dd('moduleList', $module);

	$result = "<tr>
			<th class='dataLabel' colspan='2' style='text-align: center;'>
				<h4><div class='input-icon theme-icon module-{$module}'></div> {$module_name}</h4>
			</th>
		</tr>
		<tr>
		<td class='dataLabel' colspan='2'>
			<table width='100%' cellspacing='1' cellpadding='10' border='1' style='border-collapse: collapse;'>
				<tbody>
					<tr>
						<th style='padding: 6px;'>{$mod_strings['LBL_VALIDATE_EMAILS_NAME_COLUMN']}</th>
						<th style='padding: 6px;'>{$mod_strings['LBL_VALIDATE_EMAILS_EMAIL_COLUMN']}</th>
						<th style='padding: 6px;'>{$mod_strings['LBL_VALIDATE_EMAILS_ALT_EMAIL_COLUMN']}</th>
					</tr>";

	if (is_countable($records) && sizeof($records) == 0) {
		$result .= "<tr><td style='padding: 6px; text-align: center;' colspan='3'>{$mod_strings['LBL_VALIDATE_EMAILS_NO_RECORDS_MSG']}</td></tr>";
	} else {
		foreach ($records as $record) {
			if ($module == 'Accounts') {
				$name = $record['name'];
			} else {
				$name = "{$record['first_name']} {$record['last_name']}";
			}

			$result .= "<tr>
			<td style='padding: 6px;'><a href='?module={$module}&action=DetailView&record={$record['id']}' target='_blank'>{$name}</a></td>
			<td style='padding: 6px;'>{$record['email1']}</td>
			<td style='padding: 6px;'>{$record['email2']}</td>
			</tr>
		";
		}
	}


	$result .=	"</td></tbody></table></td></tr>";

	return $result;
}

function getRecordsWithInvalidEmails($model) {
	$fields = ['email1', 'email2'];

	if ($model == 'Account') {
		$fields[] = 'name';
	} else {
		$fields[] = 'first_name';
		$fields[] = 'last_name';
	}

	$lq = new ListQuery($model, $fields);
	$result = $lq->runQuery(0, null, false, null, 100);
	$ret = [];

	while(! $result->failed) {
		foreach ($result->getRowIndexes() as $idx) {
			$rowResult = $result->getRowResult($idx);
			$row = $rowResult->getRow();
			$add = false;

			if (! empty($row['email1']) && ! filter_var($row['email1'], FILTER_VALIDATE_EMAIL)) {
				$row['email1'] = "<span style='color: red;'>{$row['email1']}</span>";
				$add = true;
			}

			if (! empty($row['email2']) && ! filter_var($row['email2'], FILTER_VALIDATE_EMAIL)) {
				$row['email2'] = "<span style='color: red;'>{$row['email2']}</span>";
				$add = true;
			}

			if ($add)
				$ret[] = $row;
		}

		if($result->page_finished)
			break;

		$lq->pageResult($result);
	}

	return $ret;
}
