<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/
require_once('include/dir_inc.php');
require_once('modules/Administration/remote_backup.php');
require_once('modules/Administration/zip_helper.php');


function failBackup($message) {
	$GLOBALS['log']->fatal($message);
	if (defined('inScheduler')) {
		trigger_error($message, E_USER_ERROR);
	}
}

function runBackup() {
	ignore_user_abort(true);
	set_time_limit(0);
	ini_set('memory_limit', '-1');

	$sqlFile = null;
	$now = date("Ymd_His");
	$backupZip = "backup_{$now}.zip";
	$backupDir = rtrim(realpath(AppConfig::setting('site.paths.backup_dir')), '/');
	if (empty($backupDir) || !is_dir($backupDir)) {
		failBackup("Backup directory does not exist");
		return false;
	}
	if (!is_writable($backupDir)) {
		failBackup("Backup directory is not writable");
		return false;
	}
	
	$rsyncTargetDir = AppConfig::setting("backup.rsync_target_dir");
	$fileFilter = AppConfig::setting("backup.zip_files_filter");
	$doRsyncFilesDir = false;
	$doDatabaseDump = false;
	if (isRsyncSupported() && AppConfig::setting('backup.rsync_enabled') && $rsyncTargetDir)
		$doRsyncFilesDir = true;
	if($doRsyncFilesDir && $fileFilter == 'complete')
		$fileFilter = 'maximal';
	if (isDumpSupported() && AppConfig::setting('backup.dump_enabled'))
		$doDatabaseDump = true;

	$zipFailed = false;	
    $zip = ZipHelper::create(isBackupEncryptionSupported() === true && AppConfig::setting('backup.crypto_enabled'));
    $zipPath = $backupDir .'/'. $backupZip;
	$zip->setCwd(getcwd());
	if(! $zip->open($zipPath)) {
		$zipFailed = "Error reopening backup zip";
	}
	if($doDatabaseDump && !$zipFailed) {
		$sqlName = "1crmdb_{$now}.sql";
		$sqlFile = createBackupTempFile($sqlName);
		if (!$sqlFile) {
			$zipFailed = "Error creating temp file for database backup: no target filename for export";
		} else {
			if (backupDatabase($zip, $sqlName, $sqlFile)) {
				$zip->addFile('.backup_dump_location');
				// close and reopen the zip so we have the SQL backup in case the next step fails
				// only works for ZipHelperPHP
				if(! $zip->flush()) {
					$zipFailed = "Error saving database SQL to zip";
				}
			}
		}
	}

	if(empty($zipFailed)) {
		if($fileFilter && $fileFilter != 'none') {
			backupApplication($zip, $fileFilter);
		}
	}
	
	if(! $zip->close()) {
		$zipFailed = "Error closing backup zip";
	}
	
	if ($zipFailed) {
		$zip->removeFile($zipPath);
	}

	if (!empty($sqlFile)) {
		@unlink($sqlFile);
	}
	
	if($doRsyncFilesDir) {
		rsyncBackupFiles(AppConfig::files_dir(), $rsyncTargetDir);
	}
	
	$dir = rtrim(realpath(AppConfig::setting('site.paths.backup_dir')), '/') . '/';
	$files = glob($dir . 'backup_*_*.{zip,zip.enc}', GLOB_BRACE);
	$files = array_filter($files, 'filterBackupFiles');
	usort($files, 'sortBackupFiles');
	$max = AppConfig::setting('backup.num_files_to_keep');
	if (!is_int($max) || $max < 1)
		$max = 5;
	$count = count($files);
	for ($i = $max ; $i < $count ; $i++) {
		@unlink($files[$i]);
	}

	$rb = RemoteBackup::create();
	if ($rb)
		$rb->sync();

	if ($zipFailed) {
		failBackup($zipFailed);
	}
	return empty($zipFailed);
}


function sortBackupFiles($a, $b) {
	return strcmp($b, $a);
}

function filterBackupFiles($f) {
	return preg_match('~/backup_\\d{8}_\\d{6}\\.zip(\\.enc)?$~', $f);
}

function backupDatabase(ZipHelper $zip, $sqlName, $sqlFile) {
	$dbOpts = AppConfig::setting('database.primary');
	if(! $dbOpts) return false;
	$dbhost = escapeshellarg($dbOpts['host_name']);
	$dbuser = escapeshellarg($dbOpts['user_name']);
	$dbpass = escapeshellarg($dbOpts['password']);
	$dbname = escapeshellarg($dbOpts['name']);

	$non_admin_options = "--skip-lock-tables";
	$sqlEscaped = escapeshellarg($sqlFile);
	$execPath = escapeshellarg(AppConfig::setting('backup.mysqldump_path'));
	$command = "$execPath $non_admin_options -h $dbhost -u $dbuser --password=$dbpass $dbname > $sqlEscaped";
	$result = system($command);
	if ($zip instanceof ZipHelperCmd) {
		return $zip->addFile($sqlFile);
	} else {
		return $zip->addFile($sqlFile, $sqlName);
	}
}


function createBackupTempFile($filename) {
	$tmpdirs = [
		AppConfig::setting('backup.mysqldump_temp_path'),
		sys_get_temp_dir(),
		 CacheManager::get_location('temp/')
	];
	foreach ($tmpdirs as $tmpdir) {
		if (!$tmpdir) {
			continue;
		}
		if(substr($tmpdir, -1) != DIRECTORY_SEPARATOR) $tmpdir .= DIRECTORY_SEPARATOR;
		$path = $tmpdir . $filename;
		if(@touch($path)) {
			if (file_put_contents('.backup_dump_location', $path) === false) {
				return false;
			}
			return $path;
		}
	}
	return false;
}


function rsyncBackupFiles($filesDir, $targetDir) {	
	if (!file_exists($targetDir)) {
		if(! mkdir_recursive($targetDir)) {
			$GLOBALS['log']->fatal("Error creating backup directory for rsync backup: $targetDir");
			return false;
		}
	}
	if (!is_dir($targetDir)) {
		$GLOBALS['log']->fatal("Backup directory path is an existing file: $targetDir");
		return false;
	}
	
	$filesDir = escapeshellcmd($filesDir);
	$targetDir = escapeshellcmd($targetDir);
	$execPath = escapeshellarg(AppConfig::setting('backup.rsync_path'));
	$command = "$execPath --recursive --delete --exclude='files/backups/*.zip' ".$filesDir." ".$targetDir;
	system($command);
}


function backupApplication(ZipHelper $zip, $fileFilter=true) {
	if($fileFilter == 'minimal') {
		$cfgPath = AppConfig::local_config_path();
		if(file_exists($cfgPath)) $zip->addFile($cfgPath);
		$zip->addDir('custom', false);
		$zip->addDir('modules', false);
	} else {
		$systemCacheDir = rtrim(AppConfig::cache_dir(), '/');
		$systemFilesDir = rtrim(AppConfig::files_dir(), '/');
		$except = array($systemCacheDir, $systemFilesDir, 'restore.php');
		
		$zip->addDir('.', false, $except);
		
		if ($fileFilter != 'complete') {
			$zip->addDir($systemFilesDir, true);
		} else {
			$zip->addDir($systemFilesDir, false, array($systemFilesDir . '/backups'));
		}

		$zip->addDir($systemFilesDir, true);
	}
}


function getBackupsList() {
    $backupFiles = array();
	$backupDir = rtrim(AppConfig::setting('site.paths.backup_dir'), '/');
    
	$h = opendir($backupDir);
	
	if ($h !== false) {
		while ($file = readdir($h)) {
			$path = $backupDir . '/' . $file;
			if ($file !== false && is_file($path) && substr($file, 0, 7) == 'backup_') {
				$modified = filemtime($path);
				$backupFiles[] = array (
					"name" => $path,
					"size" => filesize($path),
					"modified" => $modified,
				);
			} 
		}
		
		closedir($h);
	}

	SortByDate($backupFiles);	
	
	return $backupFiles;
}

function comparar($a, $b) {
	//reversed to order from newest to oldest, change to ($a["1"], $b["1"]) to do oldest to newest
	return strnatcasecmp($b['modified'], $a['modified']); 
}

function SortByDate(&$Files) {
  usort($Files, 'comparar');
}

function isRsyncSupported($path=null) {
	if(empty($path)) $path = AppConfig::setting('backup.rsync_path');
	if(! preg_match('~rsync(.exe)?$~', $path)) return false;
	$path = escapeshellarg($path);
	if($path)
		$out = @shell_exec("$path --help");

	if (! empty($out) && strpos($out, "rsync") !== false) {
		return true;
	} else {
		return false;
	}
}

function isDumpSupported($path=null) {
	if(empty($path)) $path = AppConfig::setting('backup.mysqldump_path');
	if(! preg_match('~mysqldump(.exe)?$~', $path)) return false;
	$path = escapeshellarg($path);
	if($path)
		$out = @shell_exec("$path --help");

	if (! empty($out) && strpos($out, "mysqldump") !== false) {
		return true;
	} else {
		return false;
	}
}

function describeEncryptionSupportErrors($errors) {
	if (empty($errors)) return '';
	$ret = translate('LBL_BACKUP_ENCRYPTION_UNAVAILABLE', 'Administration') . '<ul>';
	foreach ($errors as $code) {
		$ret .= '<li>' . translate('LBL_BACKUP_ENCRYPTION_UNAVAILABLE_' . strtoupper($code), 'Administration') . '</li>';

	}
	return $ret;
}

function isBackupEncryptionSupported() {
	require_once 'modules/Administration/zip_helper.php';
	$errors = [];
	if (!ZipHelper::can_use_zip()) {
		$errors[] = 'no_zip';
	}
	$descriptorspec = [
		0 => ["pipe", "r"],
		1 => ["pipe", "w"],
		2 => ["pipe", "w"]
	];

	$process = proc_open('openssl ciphers', $descriptorspec, $pipes);
	if (is_resource($process)) {
		fclose($pipes[0]);

		stream_get_contents($pipes[1]);
		fclose($pipes[1]);

		stream_get_contents($pipes[2]);
		fclose($pipes[2]);

		$return_value = proc_close($process);
		if ($return_value) {
			$errors[] = 'no_openssl';
		}
	}
	return empty($errors) ? true : $errors;
}


