<?php

require_once 'include/RemoteFS/RemoteFSFactory.php';

class RemoteBackup {

	private $backend;
	private $config;

	private function __construct($backend, $cfg) {
		$this->backend = $backend;
		$this->config = $cfg;
	}

	public static function create() {
		$cfg = AppConfig::setting('backup');
		$backend = RemoteFSFactory::create($cfg['remote_type'], $cfg);
		if (!$backend)
			return null;
		return new self($backend, $cfg);
	}

	/**
	 *
	 * Copies latest backup to remote server, then makes sure
	 * that remote server has no more that confugured number
	 * of backups.
	 *
	 * @throws RemoteFSException
	 */

	public function sync() {
		$this->backend->connect();
		try {
			$this->copyLatest();
			$this->removeOld();
		} catch (Exception $e) {
			$this->backend->disconnect();
			throw $e;
		}
		$this->backend->disconnect();
	}

	private function copyLatest() {
		$dir = rtrim(realpath(AppConfig::setting('site.paths.backup_dir')), '/') . '/';
		$files = glob($dir . 'backup_*_*.zip');
		if($this->config['crypto_enabled'] == 1)
			$files = glob($dir . 'backup_*_*.zip.enc');
		$files = array_filter($files, array($this, 'filterFiles'));
		$latest = end($files);
		if (!$latest)
			return;
		$this->backend->copyToRemote($latest, basename($latest));
	}

	private function removeOld() {
		try {
			$files = $this->backend->ls('');
		} catch (RemoteFSException $e) {
			return;
		}
		$files = array_filter($files, array($this, 'filterRemoteFiles'));
		usort($files, array($this, 'sortRemoteFiles'));
		$max = array_get_default($this->config, 'num_remote_files_to_keep');
		if (!is_int($max) || $max < 1)
			$max = 5;
		$count = count($files);
		for ($i = $max ; $i < $count ; $i++) {
			try {
				$this->backend->rm($files[$i][0]);
			} catch (RemoteFSException $e) {
			}
		}
	}

	private function filterFiles($f) {
		if($this->config['crypto_enabled'] == 1)
			return preg_match('~/backup_\\d{8}_\\d{6}\\.zip\\.enc$~', $f);
		return preg_match('~/backup_\\d{8}_\\d{6}\\.zip$~', $f);
	}
	private function filterRemoteFiles($f) {
		if($this->config['crypto_enabled'] == 1)
			return preg_match('~/backup_\\d{8}_\\d{6}\\.zip\\.enc$~', '/' . $f[0]);
		return preg_match('~/backup_\\d{8}_\\d{6}\\.zip$~', '/' . $f[0]);
	}

	private function sortRemoteFiles($a, $b) {
		// reserse order !!
		return strcmp($b[0], $a[0]);
	}
}

