<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');

require_once('data/SugarBean.php');

class Assembly extends SugarBean
{
	var $id;
	var $date_entered;
	var $created_by;
	var $date_modified;
	var $modified_user_id;
	var $deleted;
	var $name;
	var $purchase_name;
	var $product_category_id;
	var $product_type_id;
	var $supplier_id;
	var $manufacturer_id;
	var $model_id;
	var $product_url;
	var $manufacturers_part_no;
	var $vendor_part_no;
	var $description;
	var $eshop;
	var $image_url;
	var $thumbnail_url;
	var $is_available;
	var $date_available;
	
	// not stored
	var $product_category_name;
	var $product_type_name;
	var $supplier_name;
	var $manufacturer_name;
	var $model_name;
	var $created_by_name;
	var $modified_user_name;
	
	var $table_name = 'assemblies';
	var $object_name = 'Assembly';
	var $module_dir = 'Assemblies';
	
	var $new_schema = true;
	
	var $additional_column_fields = array(
		'product_category_name',
		'product_type_name',
		'supplier_name',
		'manufacturer_name',
		'model_name',
		'created_by_name',
		'modified_user_name',
	);
	
	var $relationship_fields = array(
		'supplier_id' => 'suppliers',
	);
	
	
	static function get_products_list($id, $fields=null)
	{
		$lq = new ListQuery('Assembly', null, array('link_name' => 'products', 'parent_key' => $id, 'use_key_index' => false, 'acl_filters' => 'view'));
		$lq->addPrimaryKey('id');
		$lq->addFields(array(
			'discount_id' => '~join.discount_id',
			'discount_type' => '~join.discount_type',
			'discount_name' => '~join.discount_name',
			'discount_value' => '~join.discount_value',
			'quantity' => '~join.quantity',
		));
		$lq->setOrderBy('~join.position');
		if(! $id) $lq->addFilterClause('0'); // don't return everything
		if(is_array($fields))
			$lq->addFields($fields);
		$result = $lq->runQuery();
		require_once('include/layout/FieldFormatter.php');
		$fmt = new FieldFormatter('json');
		if(! empty($result->rows)) {
			$result->formatted_rows = array();
			foreach($result->rows as $idx => $row) {
				// formatting rows manually in order to allow discounts to be applied
				$GLOBALS['pb_discount_vars'] = array(
					'discount_id' => $row['discount_id'],
					'discount_type' => $row['discount_type'],
					'discount_value' => $row['discount_value'],
				);
				$result->formatted_rows[$idx] = $fmt->formatRow($result->fields, $row);
				$result->formatted_rows[$idx]['related_type'] = 'ProductCatalog';
				unset($GLOBALS['pb_discount_vars']);
			}
		}
		return array(
			'fields' => $result->formatted_rows,
		);
	}
	
	function get_search_categories()
	{
		require_once 'modules/ProductCategories/ProductCategory.php';
		return array('' => '') + get_product_categories_list(true);
	}

	function get_search_types($param, $searchFields)
	{
		require_once 'modules/ProductTypes/ProductType.php';
		$pType = new ProductType;
		$arPT = $pType->get_for_category(@$searchFields['product_category_id']['value'], true);
		$pType->cleanup();
		return array('' => '') + $arPT;
	}

  static function init_record(RowUpdate &$upd, $input) {
        $update = array();

        if (! empty($input['model_id'])) {
            $model = ListQuery::quick_fetch_row('Model', $input['model_id']);

            if ($model != null) {
                $update['model_id'] = $input['model_id'];
                $manuf = ListQuery::quick_fetch_row('Account', $model['manufacturer_id']);
                if ($manuf != null) {
                    $update['manufacturer_id'] = $manuf['id'];
                    $update['manufacturer_name'] = $manuf['name'];
                }
            }
        }

        $upd->set($update);
    }
    
    static function updated_supplier(RowUpdate &$upd) {
    	if($upd->getFieldUpdated('supplier_id') && ($supp = $upd->getField('supplier_id'))) {
    		$upd->addUpdateLink('suppliers', $supp);
    	}
    }
    
    static function recalc_prices(RowUpdate &$upd) {
		$list_usdollar = 0;
		$cost_usdollar = 0;
		$purchase_usdollar = 0;
		$tax_code_id = '';
		$lq = $upd->getLinkQuery('products');
		
		$lq->addFields(array(
			'cost_usdollar',
			'list_usdollar',
			'purchase_usdollar',
			'tax_code_id',
			'quantity' => '~join.quantity',
			'discount_type' => '~join.discount_type',
			'discount_value' => '~join.discount_value',
		));
		$products = $lq->fetchAll();
		foreach($products->getRowIndexes() as $idx) {
			$product = $products->getRowResult($idx);
			$purchase = $product->getField('purchase_usdollar', 0);
			$quantity = $product->getField('quantity', 0);

			if (! $tax_code_id)
				$tax_code_id = $product->getField('tax_code_id');
			if ($product->getField('discount_type') == 'fixed')
				$purchase -= $product->getField('discount_value');
			else
				$purchase *= (1 - $product->getField('discount_value') / 100.00);
			
			$list_usdollar += $quantity * $product->getField('list_usdollar', 0);
			$cost_usdollar += $quantity * $product->getField('cost_usdollar', 0);
			$purchase_usdollar += $quantity * $purchase;
		}
		
		$upd->set(compact('cost_usdollar', 'list_usdollar', 'purchase_usdollar', 'tax_code_id'));
	}

	static function update_image_url($sname, $dname, $img_dir, RowUpdate $upd) {
		$fname = $upd->getField($sname);
		if ($fname) {
			require_once('include/upload_file.php');
			$value = AppConfig::site_url() . UploadFile::get_path($fname, '', $img_dir);
			$upd->set($dname, $value);
		}
	}

	static function update_image_urls(RowUpdate &$upd) {
		$spec = $upd->getFieldDefinition('image');
		$img_dir = array_get_default($spec, 'upload_dir', '');
		self::update_image_url('image_filename', 'image_url', $img_dir, $upd);
		self::update_image_url('image_thumb', 'thumbnail_url', $img_dir, $upd);
	}

	static function update_assembly_qty(RowUpdate &$upd, $link_name, $link_update) {
		if ($link_name == 'products') {
			$qty = $link_update->getField('assembly_quantity', '__NOT_SET__');
			if ($qty !== '__NOT_SET__')
				$link_update->set('quantity', unformat_number($qty));
		}
	}
}

?>
