<?php

require_once 'modules/Quotes/PDFTemplatesProvider.php';

class BillPDFTemplateProvider extends QuotePDFTemplateProvider
{
	
	protected function __construct($type)
	{
		parent::__construct($type);
	}

	public static function template_types()
	{
		return array(
			'bill' => translate('LBL_PDF_BILL_TITLE', 'Bills'),
		);
	}

	public static function template_skeleton($type)
	{
		$ret = array('sections' => array(
			array(
				'name' => 'header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_HEADER',
				'height' => 150,
				'elements' => array (),
			),
			array(
				'name' => 'group_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_GROUP_HEADER',
				'height' => 150,
				'elements' => array (),
			),
			array(
				'name' => 'group_items',
				'type' => 'table',
				'title' => 'LBL_SKEL_GROUP_ITEMS',
				'columns' => array (),
			),
			array(
				'name' => 'group_totals_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_GROUP_TOTALS_HEADER',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'group_totals',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_GROUP_TOTALS',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'group_footer',
				'data_id' => 'group_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_GROUP_FOOTER',
				'height' => 150,
				'elements' => array (),
			),
			array(
				'name' => 'quote_totals_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_QUOTE_TOTALS_HEADER',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'quote_totals',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_QUOTE_TOTALS',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'notes',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_NOTES',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'footer',
				'data_id' => 'header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_FOOTER',
				'height' => 40,
				'elements' => array (),
			),
		));
		foreach ($ret['sections'] as &$sec) $sec['title'] = translate($sec['title'], 'Bills');
		$ret['sections'] = array_values($ret['sections']);
		return $ret;
	}

	public static function template_sample_data($type)
	{
		$datefmt = AppConfig::setting('locale.defaults.date_format');
		$data = array(
			'header' => array(
				'todays_date' => array(translate('LBL_TODAYS_DATE'), date($datefmt), ),
				'number' => array(translate('LBL_SAMPLE_HEADER_NUMBER_LABEL', 'Bills'), 'SO2013-001'),
				'terms' => array(translate('LBL_SAMPLE_HEADER_TERMS_LABEL', 'Bills'), translate('LBL_SAMPLE_HEADER_TERMS_VALUE', 'Bills'), ),
				'date' => array(translate('LBL_SAMPLE_HEADER_DATE_LABEL', 'Bills'), '2014-11-05'),
				'date_due' => array(translate('LBL_SAMPLE_HEADER_DATE_DUE_LABEL', 'Bills'), '2014-11-05'),
				'buyer' => array(translate('LBL_SAMPLE_HEADER_BUYER_LABEL', 'Bills'), translate('LBL_SAMPLE_HEADER_BUYER_VALUE', 'Bills'), ),
				'tax_info' => array(translate('LBL_SAMPLE_HEADER_TAX_INFO_LABEL', 'Bills'), ''),
				'billing_address' => array(translate('LBL_SAMPLE_HEADER_BILLING_ADDRESS_LABEL', 'Bills'), translate('LBL_SAMPLE_HEADER_BILLING_ADDRESS_VALUE', 'Bills'), ),
				'description' => array(translate('LBL_SAMPLE_HEADER_DESCRIPTION_LABEL', 'Bills'), translate('LBL_SAMPLE_HEADER_DESCRIPTION_VALUE', 'Bills'), ),
				'supplier_invoice' => array(translate('LBL_SAMPLE_HEADER_SUPPLIER_INVOICE_LABEL', 'Bills'), 'I-201358'),
			),
			'group_header' => array(
				'group_name' => array(translate('LBL_SAMPLE_GROUP_HEADER_GROUP_NAME_LABEL', 'Bills'), translate('LBL_SAMPLE_GROUP_HEADER_GROUP_NAME_VALUE', 'Bills'), ),
				'status' => array(translate('LBL_SAMPLE_GROUP_HEADER_STATUS_LABEL', 'Bills'), translate('LBL_SAMPLE_GROUP_HEADER_STATUS_VALUE', 'Bills'), ),
			),
			'group_totals' => array(
				'total_name' => array(translate('LBL_SAMPLE_GROUP_TOTALS_TOTAL_NAME_LABEL', 'Bills'), translate('LBL_SAMPLE_GROUP_TOTALS_TOTAL_NAME_VALUE', 'Bills'), ),
				'total_value' => array(translate('LBL_SAMPLE_GROUP_TOTALS_TOTAL_VALUE_LABEL', 'Bills'), translate('LBL_SAMPLE_GROUP_TOTALS_TOTAL_VALUE_VALUE', 'Bills'), ),
			),
			'quote_totals' => array(
				'total_name' => array(translate('LBL_SAMPLE_QUOTE_TOTALS_TOTAL_NAME_LABEL', 'Bills'), translate('LBL_SAMPLE_QUOTE_TOTALS_TOTAL_NAME_VALUE', 'Bills'), ),
				'total_value' => array(translate('LBL_SAMPLE_QUOTE_TOTALS_TOTAL_VALUE_LABEL', 'Bills'), translate('LBL_SAMPLE_QUOTE_TOTALS_TOTAL_VALUE_VALUE', 'Bills'), ),
			),
			'group_items' => array(
				'number' => array(translate('LBL_SAMPLE_GROUP_ITEMS_NUMBER_LABEL', 'Bills'), '1'),
				'product' => array(translate('LBL_SAMPLE_GROUP_ITEMS_PRODUCT_LABEL', 'Bills'), translate('LBL_SAMPLE_GROUP_ITEMS_PRODUCT_VALUE', 'Bills'), ),
				'mfr_part_no' => array(translate('LBL_SAMPLE_GROUP_ITEMS_MFR_PART_NO_LABEL', 'Bills'), '12345'),
				'quantity' => array(translate('LBL_SAMPLE_GROUP_ITEMS_QUANTITY_LABEL', 'Bills'), '2'),
				'listprice' => array(translate('LBL_SAMPLE_GROUP_ITEMS_LISTPRICE_LABEL', 'Bills'), '10.0'),
				'unitprice' => array(translate('LBL_SAMPLE_GROUP_ITEMS_UNITPRICE_LABEL', 'Bills'), '20.0'),
				'extprice' => array(translate('LBL_SAMPLE_GROUP_ITEMS_EXTPRICE_LABEL', 'Bills'), '40.0'),
			),
		);
		self::injectProductSampleData($data['group_items']);
		$ret = array();
		foreach ($data as $k => $v) {
			$ret[$k] = array();
			foreach ($v as $kk => $vv) {
				$ret[$k][$kk] = array('label' => $vv[0], 'value' => $vv[1]);
			}
		}
		return $ret;
	}


	public static function create($type)
	{
		return new BillPDFTemplateProvider($type);
	}

	protected function _address($pfx, $row=null)
	{
		$focus = $row ? $row : $this->row;

		if ($pfx == 'billing_') {
			$acc = ListQuery::quick_fetch('Account', $focus->getField('supplier_id'));
			if ($acc) {
				$acc->assign('billing_contact_name', $focus->getField('supplier_contact'));
				$acc->assign('billing_account_name', $acc->getField('name'));
				return parent::_address('billing_', $acc);
			}
		}
	}
	
	public function initialize($layout, ListQuery $lq)
	{
		$lq->addField('supplier_contact');
		return parent::initialize($layout, $lq);
	}

	public function getFileName() {
		$filename_pfx = translate('LBL_PDF_BILL_TITLE', 'Bills');
		$filename = $filename_pfx . '_' . $this->row->getField('prefix');
		$filename .= $this->row->getField('bill_number') . '_' . $this->row->getField('supplier.name');
		$filename = preg_replace('~\.+$~', '', trim($filename));
		$filename = $filename . '.pdf';
		return $filename;
	}
	
	public function getNoteTitle()
	{
		return translate('LBL_PREPARED_BILL', 'Bills');
	}

	public function getEmailTitle()
	{
		return translate('LBL_EMAILED_BILL', 'Bills');
	}
	
	public function getSignedDocumentTitle() {
		$prefix =
			$this->row
			? '[' . $this->row->getField('prefix') . $this->row->getField('bill_number') . '] '
			: '';

		return $prefix . $this->getNoteTitle();
	}
	
	protected function createData()
	{
		global $timedate;
		$row =& $this->row;
		$user = new User();
		$user->retrieve($row->getField('assigned_user_id'), false);
		$terms = AppConfig::setting("lang.lists.current.app.terms_dom", array());
		$currency =& $this->get_currency();
		$tax_information = $this->row->getField('tax_information');
		$datefmt = AppConfig::setting('locale.defaults.date_format');
		$data = array(
			'todays_date' => date($datefmt),
			'number' => $row->getField('prefix') . $row->getField('bill_number'),
			'terms' => array_get_default($terms, $row->getField('terms'), $row->getField('terms')),
			'date' => $timedate->to_display_date($row->getField('date_entered')),
			'date_due' => $row->getField('due_date', '', true),
			'buyer' => $row->getField('assigned_user_name'),
			'tax_info' => from_html(AppConfig::setting('company.tax_information', '')),
			'billing_address' => $this->_address('billing_') . "\n" . $tax_information,
			'description' => $row->getField('description'),
			'supplier_invoice' => $row->getField('invoice_reference'),
		);
		$data = array_merge($this->header_data, $data);
		$data = array_merge($this->header_data, $data);
		$this->data['header'] = $data;
		$this->createGroupData();
	}

	public static function get_extra_sources() {
		return array(
			array(
				'module' => 'PurchaseOrders',
				'model' => 'PurchaseOrder',
				'prefix' => 'related_purchase_order',
				'label' => 'LBL_RELATED_PURCHASE',
			),
			array(
				'module' => 'Accounts',
				'model' => 'Account',
				'prefix' => 'supplier',
				'label' => 'LBL_SUPPLIER',
			),
			array(
				'module' => 'Contacts',
				'model' => 'Contact',
				'prefix' => 'supplier_contact',
				'label' => 'LBL_SUPPLIER_CONTACT',
			),
			array(
				'module' => 'ShippingProviders',
				'model' => 'ShippingProvider',
				'prefix' => 'shipping_provider',
				'label' => 'LBL_SHIPPING_PROVIDER',
			),
		);
	}
}
	
