<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/
/*********************************************************************************

 * Description:  TODO: To be written.
 * Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
 * All Rights Reserved.
 * Contributor(s): ______________________________________..
 ********************************************************************************/




require_once('data/SugarBean.php');


// Bug is used to store customer information.
class Bug extends SugarBean {
	// Stored fields
	var $id;
	var $date_entered;
	var $date_modified;
	var $modified_user_id;
	var $assigned_user_id;



	var $bug_number;
	var $description;
	var $name;
	var $status;
	var $priority;

	// These are related
	var $resolution;
	var $found_in_release;
	var $release_name;
	var $fixed_in_release_name;
	var $created_by;
	var $created_by_name;
	var $modified_by_name;
	var $account_id;
	var $contact_id;
	var $case_id;
	var $task_id;
	var $note_id;
	var $meeting_id;
	var $call_id;
	var $email_id;
	var $assigned_user_name;
	var $type;



	// longreach - added
	var $planned_for_release;
	var $planned_for_release_name;
	var $is_private;
	var $product_id;
	var $product_name;



	//BEGIN Additional fields being added to Bug Tracker
	
	var $fixed_in_release;
	var $work_log;
	var $source;
	var $product_category;
	//END Additional fields being added to Bug Tracker
	
	var $module_dir = 'Bugs';
	var $table_name = "bugs";
	var $rel_account_table = "accounts_bugs";
	var $rel_contact_table = "contacts_bugs";
	var $rel_case_table = "cases_bugs";

	var $object_name = "Bug";


	var $new_schema = true;

	
	function save($check_notify = FALSE){
		// longreach - start added
		if (empty($this->id) || !empty($this->new_with_id)) {
			$this->bug_number = $this->getNextSequenceValue();
		}
		// longreach - end adedd
		return parent::save($check_notify);
	}

	// longreach - start added
	function getCurrentSequenceValue()
	{
		return AppConfig::current_sequence_value('bug_number_sequence');
	}
	
	function getNextSequenceValue()
	{
		return AppConfig::next_sequence_value('bug_number_sequence');
	}

	function get_view_closed_where_basic($params)
	{
		return empty($params['value']) ? "(bugs.status != 'Closed' AND bugs.status != 'Rejected')" : '1';
	}
	function get_view_closed_where_advanced($params)
	{
		return '1';
	}

	function getDefaultListWhereClause()
	{
		return "(bugs.status != 'Closed' AND bugs.status != 'Rejected')";
	}

    static function init_record(RowUpdate &$upd, $input) {
        $update = array();

        if (! empty($input['acase_id']) && $upd->getSourceAction() != 'editview_save') {
            $case_fields = array('name', 'description', 'account_id', 'cust_contact_id');
            $case = ListQuery::quick_fetch_row('aCase', $input['acase_id'], $case_fields);

            if ($case) {
                for ($i = 0; $i < sizeof($case_fields); $i++) {
                    $field = $case_fields[$i];
                    if ($field == 'cust_contact_id') {
                        $case['contact_id'] = $case['cust_contact_id'];
                        $field = 'contact_id';
                    }
                    if (! empty($case[$field]) && empty($update[$field]))
                        $update[$field] = $case[$field];
                }
				copy_custom_fields($case['id'], $upd, 'aCase');
            }
        }

        $upd->set($update);
    }

    static function init_from_email(RowUpdate &$upd, $input) {
    	$email_id = array_get_default($input, 'email_id');
        if ($email_id) {
            $email = ListQuery::quick_fetch('Email', $email_id, array('name', 'description', 'contact_id', 'parent_type', 'parent_id', 'account_id'));

            if ($email) {
                $update = array('name' => $email->getField('name'),
                    'description' => $email->getField('description'),
                    'source' => 'Email', 'contact_id' => $email->getField('contact_id'));
                if($email->getField('parent_type') == 'Accounts' && $email->getField('parent_id'))
                	$update['account_id'] = $email->getField('parent_id');
                else
                	$update['account_id'] = $email->getField('account_id');

                $upd->set($update);
            }
        }
    }

    static function before_save(RowUpdate $upd) {
    }
    
	static function after_save(RowUpdate $upd) {
		$acc_id = $upd->getField('account_id');
		if($acc_id && $upd->getFieldUpdated('account_id')) {
			$upd->addUpdateLink('accounts', $acc_id);
		}

		$ctc_id = $upd->getField('contact_id');
		if($ctc_id && $upd->getFieldUpdated('contact_id')) {
			$upd->addUpdateLink('contacts', $ctc_id);
		}

        if (array_get_default($_REQUEST, 'return_module') == 'Emails')
            self::set_email_parent($upd);
	}

    static function get_activity_status(RowUpdate $upd) {
        $status = null;
        $new_bug_status = $upd->getField('status');
        $orig_bug_status = $upd->getField('status', null, true);

        $closed_bug_statuses = array('Closed', 'Rejected');

        if (in_array($new_bug_status, $closed_bug_statuses) && ! in_array($orig_bug_status, $closed_bug_statuses)) {
            $status = 'closed';
        } elseif ($new_bug_status == 'Assigned' && in_array($orig_bug_status, $closed_bug_statuses)) {
            $status = 'reopened';
        }

        return $status;
    }
    
	static function portal_filter($lq, $contact_id, $account_id) {
		$clause = [
			'operator' => 'OR',
			'multiple' => [
				['operator' => 'literal', 'value' => '0']
			],
		];
		if ($account_id) $clause['multiple'][] = ['field' => 'account_id', 'value' => $account_id];
		if ($contact_id) $clause['multiple'][] = ['field' => 'contact_id', 'value' => $contact_id];
		$lq->addFilterClause($clause);
		$lq->addSimpleFilter('is_private', 0);
	}
}
?>
