/******************************************************************************
* The contents of this file are subject to the CareBrains Software End User
* License Agreement ('License') which can be viewed at
* http://www.sugarforum.jp/download/cbieula.shtml
* By installing or using this file, You have unconditionally agreed to the
* terms and conditions of the License, and You may not use this file except in
* compliance with the License.  Under the terms of the license, You shall not,
* among other things: 1) sublicense, resell, rent, lease, redistribute, assign
* or otherwise transfer Your rights to the Software, and 2) use the Software
* for timesharing or service bureau purposes such as hosting the Software for
* commercial gain and/or for the benefit of a third party.  Use of the Software
* may be subject to applicable fees and any use of the Software without first
* paying applicable fees is strictly prohibited.
* Your Warranty, Limitations of liability and Indemnity are expressly stated
* in the License.  Please refer to the License for the specific language
* governing these rights and limitations under the License.
*****************************************************************************/

var CalendarCtrl = function() {
	return {
		isDashlet: false,
		dashletId: '',
		form_name: 'form_calendar',
		currentCls: '',
		
		globalInitCalendar: function(calendarCls, async, dashletId, params) {
			this.currentCls = calendarCls;
			if(! blank(dashletId)) {
				CalendarCtrl.isDashlet = true;
				CalendarCtrl.dashletId = dashletId;
			}
			if(! async && ! CalendarCtrl.isDashlet) {
				var navTag = SUGAR.util.checkNavigationTag('Calendar');
				if(navTag) {
					var params = parseQueryString(navTag);
					CalendarCtrl.reloadFormWithParams(params, CalendarCtrl.form_name);
					return;
				}
			}
            var self = this;
            window.setTimeout(function() {self.initRefInputs();}, 300);
			if(calendarCls && window[calendarCls])
				window[calendarCls].initCalendar(params);
		},
		
		initRefInputs: function() {
			var sel_user = SUGAR.ui.getFormInput('settings_form', 'user_name_selected');
			if(sel_user) {
				sel_user.onchange = function(k, v) { if(k) CalendarCtrlBase.setUserId(k); }
				sel_user.popup_callback = 'CalendarCtrlBase.popupReturnUser';
				sel_user.popup_return_multiple = true;
			}
            var sel_team = SUGAR.ui.getFormInput('settings_form', 'team_name_selected');
            if(sel_team) {
                sel_team.onchange = function(k, v) { if(k) CalendarCtrlBase.setTeamId(k); }
                sel_team.popup_callback = 'CalendarCtrlBase.popupReturnTeam';
                sel_team.popup_return_multiple = true;
            }
			var sel_res = SUGAR.ui.getFormInput('settings_form', 'res_name_selected');
			if(sel_res) {
				sel_res.onchange = function(k, v) { if(k) CalendarCtrlBase.setResourceId(k); }
				sel_res.popup_callback = 'CalendarCtrlBase.popupReturnResource';
				sel_res.popup_return_multiple = true;
			}
		},
		
		showCalendarBody: function(data, callbackFunc) {
			window.setTimeout('ajaxStatus.hideStatus()', 500);
			if(! data)
				return;
			try {
				$("calendar_body").innerHTML = SUGAR.util.disableInlineScripts(data.responseText);
				SUGAR.util.evalScript(data.responseText, callbackFunc);
			} catch (err) {
				console.error(err.message);
			}
		},
		
		reloadWithParams: function(params, callbackFunc) {
			params.forDashlet = CalendarCtrl.isDashlet;
			params.dashletId = CalendarCtrl.dashletId;
			var qstr = encodeQueryString(params);
			var success = function(data) {
				CalendarCtrl.showCalendarBody(data, callbackFunc);
			}
			SUGAR.util.maskDiv('calendar_body');
			if(! CalendarCtrl.isDashlet)
				SUGAR.util.setNavigationTag(qstr, 'Calendar');
			var postData = "module=Calendar&action=asyncCalendarBody&"+qstr;
			SUGAR.conn.asyncRequest(null, {method: 'POST', postData: postData}, success);
		},
		asyncCalendarBody:function(viewType, targetDate, callbackFunc) {
			return this.reloadWithParams({view_type: viewType, target_date: targetDate}, callbackFunc);
		},
		
		reloadFormWithParams: function(params, formName, callbackFunc) {
			var form = document.forms[formName];
			if(! form) return;
			var tabsForm = SUGAR.ui.getForm('calendar_tabs');
			var tabs = SUGAR.ui.getFormInput(tabsForm, 'view_type_sel');
			if(! params) params = {};
			for(var argName in params)
				if(isset(params[argName])) {
					if(form.elements[argName])
						form.elements[argName].value = params[argName];
					else {
						var elt = createElement2('input',
							{type: 'hidden', name: argName, value: params[argName]});
						form.appendChild(elt);
					}
				}
			if(! CalendarCtrl.isDashlet) {
				var qstr = encodeQueryString(params);
				SUGAR.util.setNavigationTag(qstr, 'Calendar');
			}
			if (tabs) {
				tabs.setValue(params.view_type, true);
			}
            return this.asyncCalendarBodyWithForm(formName, callbackFunc);
		},
		asyncCalendarBodyWithForm:function(formName, callbackFunc) {
			var success = function(data) {
				CalendarCtrl.showCalendarBody(data, callbackFunc);
			}
			SUGAR.util.maskDiv('calendar_body');
			SUGAR.conn.sendForm(document.forms[formName], null, success);
		},
		
		reload:function(viewType, params) {
			if(viewType == 'day') {
				DayCalendarCtrl.reloadCalendar(params);
			} else if(viewType == 'week') {
				WeekCalendarCtrl.reloadCalendar(params);
			} else if(viewType == 'month') {
				MonthCalendarCtrl.reloadCalendar(params);
			} else if(viewType == 'year') {
				YearCalendarCtrl.reloadCalendar(params);
			} else if(viewType == 'team_day') {
				TeamDayCalendarCtrl.reloadCalendar(params);
			} else if(viewType == 'team_week') {
				TeamWeekCalendarCtrl.reloadCalendar(params);
			} else if(viewType == 'resource_day') {
				ResourceDayCalendarCtrl.reloadCalendar(params);
			} else if(viewType == 'resource_week') {
				ResourceWeekCalendarCtrl.reloadCalendar(params);
			}
		}
	};
}();


var CalendarCtrlBase = new function() {
	this.view_type = '';
	this.timer = null;
	this.inited = false;	
	this.lastHover = null;
	this.waitRemoveHover = true;
	
	this.addActivity = function(act) {
		if(act && act.id) {
			this.activity_ids.push(act.id);
			this.activities[act.id] = act;
		}
	}
	this.addControl = function(ctl) {
		ctl.cal = this;
		ctl.setup();
		SUGAR.ui.registerInput(CalendarCtrl.form_name, ctl);
	}
	this.endHover = function(elt) {
		SUGAR.ui.removeClass(elt, 'hover expand');
	}
	this.checkEndHover = function() {
		if(! this.waitRemoveHover || ! this.lastHover)
			return;
		this.endHover(this.lastHover);
		this.lastHover = null;
		this.waitRemoveHover = false;
	}
	this.ov = function(elt) {
		var activity = SUGAR.dom.ExtNode(elt).closest('.panel-activity');
		if(activity.length) elt = activity[0];
		if(this.lastHover == elt) {
			if(this.waitRemoveHover)
				this.waitRemoveHover = false;
		}
		else if(this.lastHover) {
			this.endHover(this.lastHover);
			this.lastHover = null;
		}
		SUGAR.ui.addClass(elt, 'hover');
		setTimeout(function() {
			if(SUGAR.ui.hasClass(elt, 'hover'))
				SUGAR.ui.addClass(elt, 'expand');
		}, 400);
	}
	this.ot = function(elt, timeout) {
		var activity = SUGAR.dom.ExtNode(elt).closest('.panel-activity');
		if(activity.length) elt = activity[0];
		if(timeout === undefined)
			timeout = 300;
		if(! timeout)
			this.endHover(elt);
		else {
			this.checkEndHover();
			this.lastHover = elt;
			this.waitRemoveHover = true;
			var self = this;
			window.setTimeout(function() {self.checkEndHover();}, timeout);
		}
	}
	this.initCalendar = function(params) {
		this.activity_ids = [];
		this.activities = {};
		if(params) {
			this.start_hour = params.start_hour;
			this.end_hour = params.end_hour;
			if(params.activities && this.addActivity) {
				for(var i = 0; i < params.activities.length; i++)
					this.addActivity(params.activities[i]);
			}
		}
		this.form = document.forms[CalendarCtrl.form_name];
	}
	this.initJsCalendar = function(callback) {
		var init_val = this.form.target_date ? Calendar.parseDate(this.form.target_date.value, '%Y-%m-%d') : null;
		var cal_inp = new SUGAR.ui.CalendarInput('', {
			form: this.form,
			name: 'target_date',
			date_format: '%Y-%m-%d',
			attach_target: 'select_day',
			onaccept: callback,
			target_offset: {top: -1, align: 'right'},
			dateVal: init_val
		});
		cal_inp.setup();
	}
	this.reloadCalendar = function(params) {
		CalendarCtrl.reloadFormWithParams(params, CalendarCtrl.form_name);
	}
	this.moveCalendar = function(targetDate) {
		this.reloadCalendar({view_type: this.view_type, target_date: targetDate});
	}
	this.applyMode = function(mode) {
		if (! mode) mode = "activities";

		var baseMode = this.form.view_mode.value;
		var userType = 'user';
		
		var targetType = 'user';
		if (mode == 'resources') targetType = 'resource';
		var params = {view_mode: mode, target_type: targetType};
		this.reloadCalendar(params);
	}
	this.showMoreList = function(targ) {
		//var width = $("div_main_settings").offsetWidth;
		//$("div_more_list").style.width = width + "px";		
		SUGAR.popups.showPopup(targ, 'div_more_list');
	}
	this.hideMoreList = function(targ) {
		SUGAR.popups.hidePopup('div_more_list');
	}
	this.toggleFlag = function(settingsForm, name) {
		var elt = settingsForm.elements[name];
		if(elt) {
			elt.value = elt.value ? 0 : 1;
			this.applySettings(settingsForm);
		}
	}
	this.applySettings = function(settingsForm) {
		var i, elt, params = {},
			inputs = ['display_meetings', 'display_calls', 'display_tasks',
				'display_project_tasks', 'display_events', 'display_leave',
				'display_bookings'];
		for(i = 0; i < inputs.length; i++) {
			elt = settingsForm.elements[inputs[i]];
			if(elt) {
				if(elt.length == 2 && elt[1].type == 'checkbox')
					params[inputs[i]] = elt[1].checked ? 1 : 0;
				else if(elt.type == 'checkbox')
					params[inputs[i]] = elt.checked ? 1 : 0;
				else
					params[inputs[i]] = elt.value;
			}
		}
		this.reloadCalendar(params);
	}
	this.showUserForm = function(targ) {
		SUGAR.popups.showPopup(targ, 'div_select_user');
	}
	this.hideUserForm = function() {
		SUGAR.popups.hidePopup('div_select_user');
	}
	this.setCurrentUser = function(settingsForm) {
		var params = {
			target_type: 'user',
			target_user_type: 'user',
			user_id: '',
			target_user_id: '',
			selected_targets: ''				
		};
		this.reloadCalendar(params);
	}
	this.popupReturnUser = function(popup_data) {
		//set_return(popup_data);
		CalendarCtrlBase.applyUser($('settings_form'), popup_data);
	}
	this.applyUser = function(settingsForm, popupData) {
        if(popupData) {
            if(popupData.selection_list) {
                var uids = [];
                var pK = popupData.primary_key || 'id';
                for(var k in popupData.selection_list) {
                    uids.push(popupData.selection_list[k][pK]);
                }
                if(! uids.length)
                    return;
                SUGAR.ui.updateForm(settingsForm, {'user_id_selected': uids.join(',')});
            }
        }
        this.setUserId(settingsForm.user_id_selected && settingsForm.user_id_selected.value || '');
	}
	this.setUserId = function(userId, targetUserType) {
        var params = {
            target_type: 'user',
            target_user_type: targetUserType || 'user',
            user_id: userId,
            target_user_id: userId,
            selected_targets: userId
        };
        this.reloadCalendar(params);
	}
	this.showResourceForm = function(targ) {
		SUGAR.popups.showPopup(targ, 'div_select_resource');
	}	
	this.hideResourceForm = function() {
		SUGAR.popups.hidePopup('div_select_resource');
	}
	this.popupReturnResource = function(popup_data) {
		set_return(popup_data);
		CalendarCtrlBase.applyResource($('settings_form'), popup_data);
	}
	this.applyResource = function(settingsForm, popupData) {
        if(popupData) {
            if(popupData.selection_list) {
                var uids = [];
                var pK = popupData.primary_key || 'id';
                for(var k in popupData.selection_list) {
                    uids.push(popupData.selection_list[k][pK]);
                }
                if(! uids.length)
                    return;
                SUGAR.ui.updateForm(settingsForm, {res_id_selected: uids.join(',')});
            }
        }
        this.setResourceId(settingsForm.res_id_selected && settingsForm.res_id_selected.value || '');
	}
	this.setResourceId = function(resourceId) {
		var targetType = 'resource';
		var form_calendar = document.forms[CalendarCtrl.form_name];
		var baseTargetType = form_calendar.target_type.value;
		var baseTargetId = form_calendar.target_id.value;

		var params = {
			selected_targets: resourceId,
			resource_id: resourceId,
			target_type: targetType
		};
		this.reloadCalendar(params);
	}
    this.popupReturnTeam = function(popup_data) {
        set_return(popup_data);
        CalendarCtrlBase.applyTeam($('settings_form'), popup_data);
    }
    this.applyTeam = function(settingsForm, popupData) {
        if(popupData) {
            if(popupData.selection_list) {
                var uids = [];
                var pK = popupData.primary_key || 'id';
                for(var k in popupData.selection_list) {
                    uids.push(popupData.selection_list[k][pK]);
                }
                if(! uids.length)
                    return;
                SUGAR.ui.updateForm(settingsForm, {team_id_selected: uids.join(',')});
            }
        }
        this.setTeamId(settingsForm.team_id_selected && settingsForm.team_id_selected.value || '');
    }
    this.setTeamId = function(teamId) {
        var params = {
            selected_targets: teamId,
            team_id: teamId,
            target_type: 'team'
        };
        this.reloadCalendar(params);
    }
    this.showProjectForm = function(targ) {
		SUGAR.popups.showPopup(targ, 'div_select_project');		
	}
	this.hideProjectForm = function() {
		SUGAR.popups.hidePopup('div_select_project');
	}
	this.applyProject = function(settingsForm) {
		this.setProjectId(settingsForm.project_id_selected.value);
	}
	this.setProjectId = function(projectId, projectType) {
		var targetType = "user";
		var form_calendar = document.forms[CalendarCtrl.form_name];
		if(! projectId)
			projectType = 'all';
		
		var params = {
			target_type: targetType,
			project_id: projectId,
			project_type: projectType || 'project'
		};
		this.reloadCalendar(params);		
	}
    this.showTimesheetForm = function(targ) {
        SUGAR.popups.showPopup(targ, 'div_select_timesheet');
    }
    this.hideTimesheetForm = function() {
        SUGAR.popups.hidePopup('div_select_timesheet');
    }
    this.applyTimesheet = function(filterForm) {
        var id = '';
        var timesheetsRadios = filterForm.select_timesheets;
        if (timesheetsRadios.length == undefined) {
            id = timesheetsRadios.value;
        } else {
            for (var i=0; i < timesheetsRadios.length; i++) {
                if (timesheetsRadios[i].checked) {
                    id = timesheetsRadios[i].value;
                    break;
                }
            }
        }
        var params = {timesheet_id: id};
        this.reloadCalendar(params);
    }
    this.submitTimesheet = function(id, message) {
        if (id != null && id != '') {
            var _confirm = confirm(message);
            if (_confirm) {
                document.location.href = 'index.php?module=Timesheets&action=Save&record=' +id+ '&submit_timesheet=1';
                return true;
            } else {
                return false;
            }
        }
    }
    this.approveHours = function(id, operation, message) {
        if (id != null && id != '') {
            var _confirm = confirm(message);
            if (_confirm) {
                var appAction = 'all';
                if (operation == 'reject') appAction = 'reject-all';
                document.location.href = 'index.php?module=Timesheets&action=ApproveHours&record=' +id+ '&approve_action=' + appAction;
                return true;
            } else {
                return false;
            }
        }
    }
	this.toggleSelectTargetType = function(viewMode, entry) {
		var divMain = $("div_"+entry);
		var divLabel = $("div_"+entry+"_label");
		var view = (viewMode == 'all') ? "none" : "";
		divMain.style.display = view;
		divLabel.style.display = view;		
	}
    this.addGridEntry = function(settingsForm, type) {        
        var params = {};
        if (type == "resource") {
            var resId = settingsForm.res_id_selected.value;
            params = {grid_res_id: resId};
        } else {
            var userId = settingsForm.user_id_selected.value;
            params = {grid_user_id: userId};
        }
        this.reloadCalendar(params);
    }
    this.deleteGridEntry = function(id, type) {
        var params = {};
        if (type == "resource") {
            params = {grid_res_id_del: id};
        } else {
            params = {grid_user_id_del: id};
        }
        this.reloadCalendar(params);
    }
    this.showEditor = function(elt, module, record, params) {
    	if(! params) params = {};
    	if(record) params.record = record;
		return SUGAR.ui.openQuickCreate(module, params, this.afterQuickCreate(this));
    }
	this.afterQuickCreate = function(self) {
		return function() {
			self.reloadCalendar({view_type: self.view_type});
		};
	}
    this.getMode = function() {
		var form = document.forms[CalendarCtrl.form_name];
		if(form && form.view_mode) return form.view_mode.value;
    }
	this.showNew = function(elt, user_id, res_id, date, hour, minute, durHour, durMin, is_daylong, user_ids, res_ids) {
		var params = {
				assigned_user_id: user_id,
				resource_id: res_id,
				date_start: date,
				date_is_gmt: 1,
				time_hour_start: hour,
				time_minute_start: minute,
				duration_hours: durHour,
				duration_minutes: durMin,
				is_daylong: is_daylong,
				user_ids: user_ids,
				resource_ids: res_ids},
			mod = 'Meetings',
			mode = this.getMode();
		if(mode == 'projects')
			return;
		if(mode == 'timesheets') {
			mod = 'Booking';
			params.timesheet_id = document.forms[CalendarCtrl.form_name].timesheet_id.value;
		}
		return this.showEditor(elt, mod, null, params);
	}
	this.getCellPos = function(table, cell){
		var x, y, skip = 0, pos = {}, rows, cells, allday;
		rows = table.getElementsByClassName('number-row');
		for(y=0; y < rows.length; y++){
			if(SUGAR.ui.hasClass(rows[y], 'week-head')) {
				skip ++;
				continue;
			}
			allday = SUGAR.ui.hasClass(rows[y], 'all-day');
			if(allday) skip ++;
			cells = rows[y].getElementsByClassName('hour');
			for(x=0; x < cells.length; x++) {
				if(cells[x] === cell) {
					pos.is_daylong = allday;
					pos.col = x;
					pos.row = y;
					pos.skip = skip;
					return pos;
				}
			}
		}
	}
	this.resolveMouseCell = function(px, py, outer) {
		var p = SUGAR.ui.getEltRegion(outer),
			x, y, skip = 0, pos = {}, rows, cells, allday, rgn, hpos = -1;
		if(! p || ! px) return;
		if(px < p.left || px > p.right || py < p.top || py > p.bottom) return;
		rows = outer.getElementsByClassName('number-row');
		for(y = 0; y < rows.length; y++){
			if(SUGAR.ui.hasClass(rows[y], 'week-head')) {
				skip ++;
				continue;
			}
			allday = SUGAR.ui.hasClass(rows[y], 'all-day');
			if(allday) skip ++;
			cells = rows[y].getElementsByClassName('hour');
			for(x = 0; x < cells.length; x++) {
				if(hpos >= 0 && x != hpos) continue;
				if(! (rgn = SUGAR.ui.getEltRegion(cells[x])) )
					continue;
				if(hpos < 0) {
					if(rgn.left <= px && rgn.right >= px)
						hpos = x;
					else
						continue;
				}
				if(rgn.top > py)
					break;
				if(rgn.bottom >= py) {
					pos.startTime = 
					pos.cell = cells[x];
					pos.is_daylong = allday;
					pos.col = x;
					pos.row = y;
					pos.skip = skip;
					pos.offsetX = px - rgn.left;
					pos.offsetY = py - rgn.top;
					return pos;
				}
			}
		}
	}
	this.resolveMouseRange = function(px, py, rgn, outer) {
		var pos = this.resolveMouseCell(px, py, outer), pos2, pos3, rowh;
		if(pos && rgn) {
			if(! pos.is_daylong)
				pos2 = this.resolveMouseCell(px, py - rgn.y, outer);
			if(pos2 && pos2.is_daylong)
				pos2.row ++; // keep below all-day row
			if(rgn.rows)
				rowh = rgn.rows - 1;
			else {
				if( (pos3 = this.resolveMouseCell(px, py - rgn.y + rgn.h, outer)) )
					rowh = pos3.row - pos2.row;
			}
			if(pos2 && rowh) {
				pos.range = true;
				pos.row1 = pos2.row;
				pos.row2 = pos2.row + rowh;
			}
		}
		return pos;
	}
	this.getTimeRange = function(pos1, pos2, rows) {
		// pos2, rows optional
		var time = {}, top_row, bottom_row, diff;
		if(! pos1) return;
		if(pos1.is_daylong) {
			time.is_daylong = 1;
			time.start_hour = this.start_hour;
			time.start_min = 0;
			time.end_hour = this.end_hour;
			time.end_min = 0;
		}
		else {
			if(pos2) {
				top_row = Math.min(pos1.row, pos2.row) - pos1.skip;
				bottom_row = Math.max(pos1.row, pos2.row) - pos1.skip + 1;
			} else if(pos1.range) {
				top_row = pos1.row1 - pos1.skip;
				bottom_row = pos1.row2 - pos1.skip;
			} else {
				top_row = bottom_row = pos1.row - pos1.skip;
				if(rows) bottom_row += rows;
				else bottom_row ++;
			}
			time.is_daylong = 0;
			time.start_hour = Math.floor(this.start_hour + top_row / 2);
			time.start_min = (top_row % 2 == 1) ? 30 : 0;
			time.end_hour = Math.floor(this.start_hour + bottom_row / 2);
			time.end_min = (bottom_row % 2 == 1) ? 30 : 0;
		}
		time.dur_total = time.end_hour * 60 + time.end_min - time.start_hour * 60 - time.start_min;
		time.dur_hour = Math.floor(time.dur_total / 60);
		time.dur_min = time.dur_total - time.dur_hour * 60;
		time.date = this.weekDays ? this.weekDays[pos1.col] : this.form.target_date.value;
		return time;
	}
	this.markCellRange = function(table, x1, y1, x2, y2) {
		if(! table) return;
		var x, y, rows, cells, cell;
		rows = table.getElementsByClassName('number-row');
		for(y=0; y < rows.length; y++) {
			cells = rows[y].getElementsByClassName('hour');
			for(x=0; x < cells.length; x++) {
				cell = cells[x];
				if(! this.isResponsiveCell(cell)) continue;
				SUGAR.ui.addRemoveClass(cell, 'selected', (y1-y)*(y-y2)>=0 && (x1-x)*(x-x2)>=0);
				SUGAR.ui.removeClass(cell, 'active');
			}
		}
	}
	this.isResponsiveCell = function(cell) {
		return SUGAR.ui.hasClass(cell, 'hour') && SUGAR.ui.hasClass(cell, 'responsive');
	}
	this.extendTo = function(cls, map, map2) {
		var k, i;
		cls.prototype = this;
		for(k in this)
			cls[k] = this[k];
		for(i = 1; i < arguments.length; i++)
			for(k in arguments[i])
				cls[k] = arguments[i][k];
	}
};


var CalHourGridEvents = {
	mouseDown:function(e, table) {
		this.startCell = e.target;
		if(! this.isResponsiveCell(this.startCell)) {
			this.startCell = null;
			return;
		}
		SUGAR.ui.stopEvent(e);
		this.mouseMove(e, table);
	},
	
	mouseUp:function(e, table) {
		var from, to, endCell, range;
		if(! this.startCell || ! this.endCell){
			return false;
		}
		
		from = this.getCellPos(table, this.startCell);
		to = this.getCellPos(table, this.endCell);
		endCell = this.endCell;
		this.startCell = this.endCell = null;
		this.markCellRange(table);
		if(!from || !to) {
			return false;
		}

		var target_id = this.form.target_id.value,
			target_type = this.form.target_type.value,
			mode = this.getMode(),
			timesheet_id = this.form.timesheet_id.value;
			target_user_id = '',
			target_res_id = '';
		if(target_type == 'user') {
			target_user_id = target_id;
		} else if (target_type == 'resource') {
			target_res_id = target_id;
		}
		
		if (mode != "projects") {			
			range = this.getTimeRange(from, to);
			var qcp = {
					assigned_user_id: target_user_id,
					resource_id: target_res_id,
					timesheet_id: timesheet_id,
					date_start: range.date,
					date_is_gmt: 1,
					time_hour_start: range.start_hour,
					time_minute_start: range.start_min,
					duration_hours: range.dur_hour,
					duration_minutes: range.dur_min,
					is_daylong: range.is_daylong},
				m = (mode == 'timesheets' ? 'Booking' : 'Meetings');
			this.showEditor(endCell, m, null, qcp);
		}
	},
	
	mouseMove:function(e, table){
		var from, to;
		this.endCell = e.target;
		if(! this.isResponsiveCell(this.endCell) || ! this.startCell) {
			return;
		}
		
		from = this.getCellPos(table, this.startCell);
		to = this.getCellPos(table, this.endCell);
		if(!from || !to)
			return;
		if(from.col != to.col)
			to.col = from.col;
		this.markCellRange(table, from.col, from.row, to.col, to.row);
	},
	
	mouseLeave: function(e, table) {
		if(this.startCell) {
			this.startCell = null;
			this.endCell = null;
			this.markCellRange(table);
		}
	},
			
	dragOver: function(e, outer) {
		var px = e.clientX + window.scrollX, py = e.clientY + window.scrollY,
			pos;
		if(this.drag_x == px && this.drag_y == py)
			return; // ignore duplicate events
		this.drag_x = px;
		this.drag_y = py;
		if( (pos = this.resolveMouseRange(px, py, this.drag_rgn, outer)) ) {
			if(pos.range)
				this.markCellRange(outer, pos.col, pos.row1, pos.col, pos.row2);
			else
				this.markCellRange(outer, pos.col, pos.row, pos.col, pos.row);
			// allow drop
			e.preventDefault();
		}
	},

	drop: function(e, outer) {
		var px = e.clientX + window.scrollX, py = e.clientY + window.scrollY,
			pos, range, self = this;
		this.markCellRange(outer);
		if( (pos = this.resolveMouseRange(px, py, this.drag_rgn, outer)) ) {
			range = this.getTimeRange(pos);
			if(this.drag_activity_id) {
				var query_params = {
					module: this.activities[this.drag_activity_id].module,
					record: this.drag_activity_id,
					date_start: range.date,
					time_start: range.start_hour + ':' + range.start_min,
					is_daylong: range.is_daylong};
				var req = new SUGAR.conn.JSONRequest(
					'move_calendar_activity',
					{status_msg: app_string('LBL_LOADING')},
					query_params);
				req.fetch(function() { 
					var row = this.getResult();
					if(row && row.result == 'ok')
						self.reloadCalendar();
				});
			}
		}
	},
	
	eventDragStart: function(e, act_id) {
		var mod = this.activities[act_id].module,
			url = 'index.php?' + encodeQueryString({module: mod, action: 'DetailView', record: act_id}),
			div = $('div_act_' + act_id),
			rgn = SUGAR.ui.getEltRegion(div),
			rows = 1, act;
		if(! div || ! rgn) return;
		this.drag_activity_id = act_id;
		this.drag_url = url;
		if( (act = this.activities[act_id]) ) {
			if(act.isDaylong || ! act.durationMin)
				rows = 1;
			else
				rows = Math.ceil((act.durationMin - 1) / 30);
		}
		this.drag_rgn = {
			x: e.clientX + window.scrollX - rgn.left,
			y: e.clientY + window.scrollY - rgn.top,
			w: rgn.width,
			h: rgn.height,
			rows: rows
		};
		e.dataTransfer.setData('text/plain', url);
		e.dataTransfer.setData('text/uri-list', url);
		e.dataTransfer.setData('url', url);
		e.dataTransfer.dropEffect = 'move';
	},

	eventDragEnd: function(e, act_id) {
		this.drag_activity_id = null;
		this.drag_url = null;
		this.drag_rgn = null;
		this.markCellRange($('calendar-body'));
	},
	
	eventMouseOver: function(e, act_id) {
		// set timer to show additional details
		this.ov(e.target);
	},

	eventMouseOut: function(e, act_id) {
		// hide details
		this.ot(e.target);
	},
	
	eventClick: function(e, act_id) {
		if(e.target.closest('a')) {
			return;
		}
		var mod = this.activities[act_id].module;
		if(mod && act_id) {
			this.showEditor(this, mod, act_id);
			SUGAR.ui.stopEvent(e);
		}
	},
	
	/*eventDblClick: function(e, act_id) {
		var mod = this.activities[act_id].module;
		if(mod && act_id)
			this.showEditor(this, mod, act_id);
	},*/

	attachEvents: function() {
		var table = $('calendar-body'), act_id, div, div_detail;
		SUGAR.ui.addEventListeners(table, {
			mousedown: this.mouseDown,
			mouseup: this.mouseUp,
			mousemove: this.mouseMove,
			mouseleave: this.mouseLeave,
			dragover: this.dragOver,
			drop: this.drop}, this, table);
		if(this.activity_ids) {
			for(var i = 0; i < this.activity_ids.length; i++) {
				act_id = this.activity_ids[i];
				div = $('div_act_' + act_id);
				div_detail = $('div_detail_' + act_id);
				if(div && this.activities[act_id].canEdit && ! div.draggable) {
					SUGAR.ui.addEventListeners(div, {
						dragstart: this.eventDragStart,
						dragend: this.eventDragEnd,
						mouseover: this.eventMouseOver,
						mouseout: this.eventMouseOut,
						click: this.eventClick,
						dblclick: this.eventDblClick
					}, this, act_id);
					SUGAR.ui.addEventListeners(div, {
						// event pass thru
						dragover: this.dragOver,
						drop: this.drop
					}, this, table);
					div.draggable = true;
				} else if (div && this.activities[act_id].isViewAble) {
                    SUGAR.ui.addEventListeners(div, {
                        mouseover: this.eventMouseOver,
                        mouseout: this.eventMouseOut
                    }, this, act_id);
                }
				if (div_detail) {
					SUGAR.ui.addEventListeners(div_detail, {
						click: function(e) {
							e.stopPropagation();
						}
					}, this);
				}
			}
		}
	}
}


//////////////////////////////////////////////////////////////////
// for Day calendar
//////////////////////////////////////////////////////////////////
var DayCalendarCtrl = new function() {
	var startCell = null;

	CalendarCtrlBase.extendTo(this, CalHourGridEvents, {
		view_type: 'day',
		
		initCalendar: function(acts) {
			CalendarCtrlBase.initCalendar.call(this, acts);
			this.initJsCalendar(function() {
				DayCalendarCtrl.moveCalendar(this.getValue());
			});
			this.attachEvents();
        },

		asyncCalendarBody:function(targetDate, targetId, targetType, teamId) {
			var params = {
				view_type: this.view_type,
				target_date: targetDate, target_id: targetId,
				target_type: targetType, target_team_id: teamId};
			DayCalendarCtrl.reloadCalendar(params);
        },

		toggleTaskList:function(isViewList) {
			var div_task_count = $("div_task_count");
			var div_task_list = $("div_task_list");
			if(isViewList) {
				div_task_count.style.display = "none";
				div_task_list.style.display = "";
			} else {
				div_task_count.style.display = "";
				div_task_list.style.display = "none";
			}
		}
	});
}();

//////////////////////////////////////////////////////////////////
// for week calendar
//////////////////////////////////////////////////////////////////
var WeekCalendarCtrl = new function() {
	var dds;
	var toDayWeekIndex = -1;
	var firstDate = "";
	var weekDays;
	var summaryTimeout;

    CalendarCtrlBase.extendTo(this, CalHourGridEvents, {
		view_type: 'week',

        displaySummary : function(offset) {
			var target_id = 'summary_details_' + offset;
			var divs = document.getElementsByName('summary_details');
			for (var i in divs) {
				var div = divs[i];
				try {
					if (div.id == target_id) {
						div.style.zIndex = 1000;
						div.style.display = '';
					} else {
						div.style.display = 'none';
					}
				} catch(e) {
				}
			}
			if (this.summaryTimeout) {
				window.clearTimeout(this.summaryTimeout);
				this.summaryTimeout = null;
			}
		},

		summaryLeaved : function() {
			this.summaryTimeout = window.setTimeout('WeekCalendarCtrl.displaySummary(-1)', 500);
		},
		
		initCalendar:function(acts) {
			CalendarCtrlBase.initCalendar.call(this, acts);
			
			this.toDayWeekIndex = parseInt(this.form.today_weekindex.value);
			this.firstDate = this.form.week_first_date.value;

			this.weekDays = Array();
			for(var i = 0; i < 7; i++) {
				var elt = 'week_date_' + i;
				if(this.form[elt])
					this.weekDays[i] = this.form[elt].value;
			}

            this.initJsCalendar(function() {
				WeekCalendarCtrl.moveCalendar(this.getValue());
            });
            
            this.attachEvents();
        },
		
		asyncCalendarBody:function(targetDate, targetId, targetType, teamId) {
			var params = {
				view_type: this.view_type, target_date: targetDate,
				target_id: targetId, selected_targets: targetId,
				target_type: targetType, target_team_id: teamId};
			this.reloadCalendar(params);
		}
	});
}();


//////////////////////////////////////////////////////////////////
// for Month calendar
//////////////////////////////////////////////////////////////////
var MonthCalendarCtrl = new function() {
	CalendarCtrlBase.extendTo(this, {
		view_type: 'month',

		initCalendar:function() {
			CalendarCtrlBase.initCalendar.call(this);
			this.initJsCalendar(function() {
				MonthCalendarCtrl.moveCalendar(this.getValue());
			});
		},
		
		asyncCalendarBody:function(targetDate, userId, teamId) {
			var params = {
				view_type: this.view_type, target_date: targetDate,
				target_user_id: userId, target_team_id: teamId};
			MonthCalendarCtrl.reloadCalendar(params);
		}
	});
}();

//////////////////////////////////////////////////////////////////
// for Year calendar
//////////////////////////////////////////////////////////////////
var YearCalendarCtrl = new function() {	
	CalendarCtrlBase.extendTo(this, {
		view_type: 'year',

		initCalendar:function() {
			CalendarCtrlBase.initCalendar.call(this);
			this.initJsCalendar(function() {
				YearCalendarCtrl.moveCalendar(this.getValue());
			});
		},
		
		asyncCalendarBody:function(targetDate, targetId, targetType, teamId) {
			var params = {
				view_type: this.view_type, target_date: targetDate,
				target_id: targetId, target_type: targetType,
				target_team_id: teamId};
			YearCalendarCtrl.reloadCalendar(params);
		}
	});
}();

//////////////////////////////////////////////////////////////////
// for Team Day calendar
//////////////////////////////////////////////////////////////////
var TeamDayCalendarCtrl = new function() {
	CalendarCtrlBase.extendTo(this, {
		view_type: 'team_day',

		initCalendar:function() {
			CalendarCtrlBase.initCalendar.call(this);
			this.initJsCalendar(function() {
				CalendarCtrl.asyncCalendarBody('team_day', this.getValue(), TeamDayCalendarCtrl.initCalendar);
			});
		},
		
		asyncCalendarBody:function(targetDate, teamId) {
			var params = {
				view_type: this.view_type,
				target_date: targetDate,
				target_team_id: teamId};
			TeamDayCalendarCtrl.reloadCalendar(params);
		},

		teamOnChange:function(teamId) {
			TeamDayCalendarCtrl.reloadCalendar({target_team_id: teamId});
		}
		
	});
}();



//////////////////////////////////////////////////////////////////
// for Team Week calendar
//////////////////////////////////////////////////////////////////
var TeamWeekCalendarCtrl = new function() {
	CalendarCtrlBase.extendTo(this, {
		view_type: 'team_week',

		initCalendar:function() {
			CalendarCtrlBase.initCalendar.call(this);
			this.initJsCalendar(function() {
				TeamWeekCalendarCtrl.moveCalendar(this.getValue());
			});
		},
		
		teamPopup:function() {
			var popupdata = {
				'call_back_function':'TeamWeekCalendarCtrl.setTeamReturn',
				'form_name':'form_calendar',
				'field_to_name_array':{
					'id':'target_team_id',
					'name':'span_team_name'
				}
			};
		},
		
		setTeamReturn:function(popup_reply_data) {
			var form_name = popup_reply_data.form_name;
			var name_to_value_array = popup_reply_data.name_to_value_array;
			
			for (var the_key in name_to_value_array) {
				if(the_key == 'toJSON') {
					/* just ignore */
				} else if(the_key == 'target_team_id') {
					var element = $(the_key);
					element.value = name_to_value_array[the_key];
				} else if(the_key == 'span_team_name') {
					var element = $(the_key);
					element.innerHTML = name_to_value_array[the_key];
				}
			}
			TeamWeekCalendarCtrl.reloadCalendar();
		},
		
		asyncCalendarBody:function(targetDate, teamId) {
			var params = {
				view_type: this.view_type,
				target_date: targetDate,
				target_team_id: teamId};
			TeamWeekCalendarCtrl.reloadCalendar(params);
		},
		
		teamOnChange:function(teamId) {
			document.forms[CalendarCtrl.form_name].target_team_id.value = teamId;
			TeamWeekCalendarCtrl.reloadCalendar();
		}
	});
}();


//////////////////////////////////////////////////////////////////
// for Resource Day calendar
//////////////////////////////////////////////////////////////////
var ResourceDayCalendarCtrl = new function() {
	CalendarCtrlBase.extendTo(this, {
		view_type: 'resource_day',

		initCalendar:function() {
			CalendarCtrlBase.initCalendar.call(this);
			this.initJsCalendar(function() {
				ResourceDayCalendarCtrl.moveCalendar(this.getValue());
			});
		},
		
		asyncCalendarBody:function(targetDate) {
			var params = {view_type: this.view_type, target_date: targetDate};
			ResourceDayCalendarCtrl.reloadCalendar(params);
		},
		
		resTypeOnChange:function(resType) {
			ResourceDayCalendarCtrl.reloadCalendar({target_res_type: resType});
		}
	});
}();

//////////////////////////////////////////////////////////////////
// for Resource Week calendar
//////////////////////////////////////////////////////////////////
var ResourceWeekCalendarCtrl = new function() {
	CalendarCtrlBase.extendTo(this, {
		view_type: 'resource_week',

		initCalendar:function() {
			CalendarCtrlBase.initCalendar.call(this);
			this.initJsCalendar(function() {
				ResourceWeekCalendarCtrl.moveCalendar(this.getValue());
			});
		},
		
		resTypeOnChange:function(resType) {
			ResourceDayCalendarCtrl.reloadCalendar({target_res_type: resType});
		},
		
		asyncCalendarBody:function(targetDate, teamId) {
			var params = {
				view_type: this.view_type,
				target_date: targetDate,
				target_team_id: teamId};
			ResourceWeekCalendarCtrl.reloadCalendar(params);
		}
	});
}();



var UsersCalendarCtrl = function() {
	return {
		ov:function(e) {
			SUGAR.ui.addClass(e, 'hover');
		},
		ot:function(e) {
			SUGAR.ui.removeClass(e, 'hover');
		}
	};
}();


var setPrintLink = function(elt) {
	var printUrl = $('print_url');
	if (printUrl) {
		if(elt && (elt = $(elt)))
			elt.href = printUrl.value;
		else {
			var printLinks = document.getElementsByName('print_link');
			for  (var i in printLinks) {
				printLinks[i].href =  printUrl.value;
			}
		}
	}
};


var CalUserSelect = function(id, params) {
	if(! params) params = {};
	params.icon_key = 'icon';
	params.options = {
		width: 220,
		keys: ['', 'select'],
		values: [
			{label: mod_string('LBL_ME', 'Calendar'), icon: 'icon-user'},
			{label: app_string('LBL_FILTER_OWNER_SELECT'), icon: 'icon-popup'}]};
	if(params.all_users) {
		params.options.keys.splice(0, 0, 'all');
		params.options.values.splice(0, 0, {label: mod_string('LBL_ALL_USERS_BUTTON_LABEL', 'Calendar'), icon: 'icon-users'});
	}
	if(params.user_id == 'all')
		params.init_value = 'all';
	else if(params.user_id && params.user_name && params.user_name != 'Me') {
		params.options.keys.splice(1, 0, params.user_id);
		params.options.values.splice(1, 0, {label: params.user_name, icon: 'icon-user'});
		params.init_value = params.user_id;
	} else
		params.init_value = '';
	SUGAR.ui.SelectInput.call(this, id, params);
};
extendClass(CalUserSelect, SUGAR.ui.SelectInput, {
	userUpdate: function(key, value) {
		this.update(key, value);
	},
	update: function(key, value, silent) {
		if(key == 'select')
			this.showSelectUser();
		else if(key == 'all')
			this.cal.setUserId('', 'all');
		else
			this.cal.setUserId(key);
	},
	showSelectUser: function() {
		var request_data = {
				form_name: this.form.getAttribute('id') || this.form.getAttribute('name'),
				call_back_function: this.popup_callback || 'ui_popup_return',
				passthru_data: {ui_input_name: this.name || this.id},
				field_to_name_array: this.field_to_name || {id: 'id', label: 'name'}
			},
			params = {module: 'Users', request_data: request_data, inline: true};
		open_popup_window(params);
	}
});


var CalResSelect = function(id, params) {
	if(! params) params = {};
	params.icon_key = 'icon';
	params.options = {
		width: 220,
		keys: ['', 'select'],
		values: [
			{label: mod_string('LBL_RESOURCE_ALL', 'Calendar'), icon: 'theme-icon bean-Resource'},
			{label: mod_string('LBL_SELECT_RESOURCE', 'Calendar'), icon: 'icon-popup'}]};
	if(params.res_id && params.res_name) {
		params.options.keys.splice(1, 0, params.res_id);
		params.options.values.splice(1, 0, {label: params.res_name, icon: 'theme-icon bean-Resource'});
		params.init_value = params.res_id;
	} else
		params.init_value = '';
	SUGAR.ui.SelectInput.call(this, id, params);
};
extendClass(CalResSelect, SUGAR.ui.SelectInput, {
	userUpdate: function(key, value) {
		this.update(key, value);
	},
	update: function(key, value, silent) {
		if(key == 'select')
			this.showSelectRes();
		else {
			this.cal.setResourceId(key || '');
		}
	},
	showSelectRes: function() {
		var request_data = {
				form_name: this.form.getAttribute('id') || this.form.getAttribute('name'),
				call_back_function: this.popup_callback || 'ui_popup_return',
				passthru_data: {ui_input_name: this.name || this.id},
				field_to_name_array: this.field_to_name || {id: 'id', label: 'name'}
			},
			params = {module: 'Resources', request_data: request_data, inline: true, min_width: 700};
		open_popup_window(params);
	}
});


var CalProjectSelect = function(id, params) {
	if(! params) params = {};
	params.icon_key = 'icon';
	params.options = {
		width: 220,
		keys: ['', 'select'],
		values: [
			{label: mod_string('LBL_PROJECT_ALL', 'Calendar'), icon: 'theme-icon bean-Project'},
			{label: mod_string('LBL_SELECT_PROJECT', 'Calendar'), icon: 'icon-popup'}]};
	if(params.proj_id && params.proj_name) {
		params.options.keys.splice(1, 0, params.proj_id);
		params.options.values.splice(1, 0, {label: params.proj_name, icon: 'theme-icon bean-Project'});
		params.init_value = params.proj_id;
	} else
		params.init_value = '';
	SUGAR.ui.SelectInput.call(this, id, params);
};
extendClass(CalProjectSelect, SUGAR.ui.SelectInput, {
	userUpdate: function(key, value) {
		this.update(key, value);
	},
	update: function(key, value, silent) {
		if(key == 'select')
			this.showSelectProject();
		else {
			this.cal.setProjectId(key || '');
		}
	},
	showSelectProject: function() {
		var request_data = {
				form_name: this.form.getAttribute('id') || this.form.getAttribute('name'),
				call_back_function: this.popup_callback || 'ui_popup_return',
				passthru_data: {ui_input_name: this.name || this.id},
				field_to_name_array: this.field_to_name || {id: 'id', label: 'name'}
			},
			params = {module: 'Project', request_data: request_data, inline: true, min_width: 700};
		open_popup_window(params);
	}
});

