<?php
if (!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/******************************************************************************
* The contents of this file are subject to the CareBrains Software End User
* License Agreement ('License') which can be viewed at
* http://www.sugarforum.jp/download/cbieula.shtml
* By installing or using this file, You have unconditionally agreed to the
* terms and conditions of the License, and You may not use this file except in
* compliance with the License.  Under the terms of the license, You shall not,
* among other things: 1) sublicense, resell, rent, lease, redistribute, assign
* or otherwise transfer Your rights to the Software, and 2) use the Software
* for timesharing or service bureau purposes such as hosting the Software for
* commercial gain and/or for the benefit of a third party.  Use of the Software
* may be subject to applicable fees and any use of the Software without first
* paying applicable fees is strictly prohibited.
* Your Warranty, Limitations of liability and Indemnity are expressly stated
* in the License.  Please refer to the License for the specific language
* governing these rights and limitations under the License.
*****************************************************************************/
global $current_language;

require_once('include/layout/FieldFormatter.php');
require_once('include/layout/widgets/InviteesList.php');
require_once 'modules/Calendar/utils.php';
load_holidays();

require_once('modules/Calendar/ExtClassForCalendar/UserEx.php');
require_once('modules/Calendar/ExtClassForCalendar/MeetingEx.php');
require_once('modules/Calendar/ExtClassForCalendar/CallEx.php');

define('CAL_HEIGHT_PER_HOUR', 50);
define('CAL_HEIGHT_TUNE_OFFSET', -2);

define('CAL_DAY_LEFT_OFFSET', 63);
define('CAL_DAY_ACTIVITY_WIDTH', 200);
define('CAL_DAY_WIDTH', 475);
define('CAL_DUP_OFFSET', 42);
define('CAL_MIN_DUP_OFFSET', 18);
define('CAL_WEEK_DAY_WIDTH', 120);
define('CAL_WEEK_HEADER_HEIGHT', 32);

class CalendarDataUtil {

	static function param($name, $value = null)
	{
		static $params = array();
		if (!is_null($value)) {
			$params[$name] = $value;
		}

		return isset($params[$name]) ? $params[$name] : null;
	}

	static function getLocalToDay() {
		global $timedate;
		$now = $timedate->to_display_date_time(gmdate("Y-m-d H:i:s"));
		return $timedate->to_display($now, $timedate->get_date_time_format(), 'Y-m-d');
	}

	static function getLocalCurrentHour() {
		global $timedate;
		$now = $timedate->to_display_date_time(gmdate("Y-m-d H:i:s"));
		return $timedate->to_display($now, $timedate->get_date_time_format(), 'H');
	}

	static function getHourArray() {
		global $current_user;
		$startHour = $current_user->day_begin_hour;
		$endHour = $current_user->day_end_hour;
		$dateFormats = CalendarDateTime::userDateTimeFormats();
		if(empty($startHour) && empty($endHour)) {
			$startHour = 9;
			$endHour = 18;
		}
		$ret = array();
		for ($i = $startHour; $i < $endHour; $i++) {
			if($i == $startHour || $i == 12 || $i == 24 || $i == $endHour-1) {
				$merid = $dateFormats['meridiem'];
				$fmt = $merid ? $dateFormats['time_hour'] . $merid : $dateFormats['time_no_meridiem'];
			} else
				$fmt = $dateFormats['time_no_meridiem'];
			$ret[$i] = array('display' => date($fmt, strtotime(sprintf("%02d:00", $i))));
		}
		return $ret;
	}

	static function formatActivityTimes($startDT, $endTS=null) {
		$showStartMinutes = ($startDT->localMin != 0 || ! isset($endTS));
		$dateFormats = CalendarDateTime::userDateTimeFormats();
		if(isset($endTS)) {
			$duration = $endTS - $startDT->localDateTime_t;
			$showEndMinutes = (($startDT->localMin + $duration / 60) % 60 != 0);
			$max_dur = ((12 - $startDT->localHour % 12) * 60 - $startDT->localMin) * 60;
			$endFmt = $showEndMinutes
						? $dateFormats['time_no_zero']
						: $dateFormats['time_hour'] . $dateFormats['meridiem'];
			$endStr = date($endFmt, $endTS);
			$showStartMerid = ($duration >= $max_dur);
		} else {
			$endStr = '';
			$showStartMerid = true;
		}
		$startFmt = $showStartMerid
					? ($showStartMinutes
						? $dateFormats['time_no_zero']
						: $dateFormats['time_hour'] . $dateFormats['meridiem'])
					: ($showStartMinutes
						? $dateFormats['time_no_meridiem']
						: $dateFormats['time_hour']);
		return array(date($startFmt, $startDT->localDateTime_t), $endStr);
	}

	static function getWorkHourArray($format = "", $isPre7 = false, $isAfter22 = false) {
		global $current_user;

		$result = array ();

		$start = isset($current_user->day_begin_hour) ? $current_user->day_begin_hour : 7;
		$end = isset($current_user->day_end_hour) ? $current_user->day_end_hour : 22;

		if(empty($start) && empty($end)) {
			$start = 9;
			$end = 18;
		}

		if ($isPre7) {
			$start = 0;
		}
		if ($isAfter22) {
			$end = 24;
		}

		if($format === 'user' || $format === 'user_short') {
			$dateFormats = CalendarDateTime::userDateTimeFormats();
			for ($i = $start; $i < $end; $i++) {
				if($format == 'user_short') {
					$fmt = $dateFormats['time_hour'];
					//if(($i == $start || $i == $end-1) && ($i != 12 && $i != 24))
					//	$fmt .= $dateFormats['meridiem'];
				} else if($i == $start || $i == 12 || $i == 24 || $i == $end-1) {
					$merid = $dateFormats['meridiem'];
					$fmt = $merid ? $dateFormats['time_hour'] . $merid : $dateFormats['time_no_meridiem'];
				} else
					$fmt = $dateFormats['time_no_meridiem'];
				$result[$i] = array (
					'display' => date($fmt, strtotime(sprintf("%02d:00", $i)))
				);
			}
		} else if (!empty ($format)) {
			for ($i = $start; $i < $end; $i++) {
				$result[$i] = array (
					'display' => sprintf($format,
					$i
				));
			}
		} else {
			for ($i = $start; $i < $end; $i++) {
				$result[$i] = array (
					'display' => $i
				);
			}
		}
		return $result;
	}

	static function getWeekDays($calDateTime, $checkMonth=null, $weekends=true) {
		global $current_language, $app_list_strings;

		$localToDay = CalendarDataUtil::getLocalToDay();
		$lang = return_module_language($current_language, 'Calendar');
		$baseDate_t = strtotime($calDateTime->localWeekFirstDate);
		$dateFormats = CalendarDateTime::userDateTimeFormats();

		$weekDays = array ();
		$i = $calDateTime->systemWeekFirstDay - 1;
		for ($idx = 0; $idx < 7; $idx++) {
			$i++;
			if ($i > 6) {
				$i = 0;
			}
			if(! $weekends && ($i == 0 || $i == 6))
				continue;
			$timestamp = strtotime("+{$idx} day", $baseDate_t);
			$targetDate = date("Y-m-d", $timestamp);

			$holiday = "";
			$rokuyou = "";

			$matches = array ();
			$selMonth = false;
			if (preg_match('/(\d{4})-(\d{2})-(\d{2})/', $targetDate, $matches) > 0) {
				$Year = $matches[1];
				$Month = $matches[2];
				$Day = $matches[3];

				if (function_exists('holiday')) {
					$holiday = holiday($Year, $Month, $Day);
				}
				if (function_exists('get_rokuyou')) {
					$rokuyou = get_rokuyou($Year, $Month, $Day);
				}
				if(isset($checkMonth) && $Month == $checkMonth)
					$selMonth = true;
			}

			if ($i == 0)
				$colClass = "quiet sunday";
			else if ($i == 6)
				$colClass = "quiet saturday";
			else
				$colClass = "";
			$weekdayCssClass = $colClass;

			$dateCssClass = ' '.$colClass;
			if ($holiday != "")
				$dateCssClass .= " holiday";
			if ( ($isToday = ($targetDate == $localToDay)) )
				$dateCssClass .= " current";
			if(isset($checkMonth) && ! $selMonth)
				$dateCssClass .= " outside";
			else
				$dateCssClass .= " inside";

			$weekDays[$i] = array (
				'weekDayIndex' => $i,
				'date' => $targetDate,
				'day' => ltrim($Day, '0'),
				'date_short' => date($dateFormats['date_short'], $timestamp), //custom_strftime($lang['LBL_FORMAT_SHORT_DATE'], $gmt_timestamp),
				'weekday' => $app_list_strings['weekdays_dom'][$i],
				'holiday' => $holiday,
				'rokuyo' => $rokuyou,
				'dateCssClass' => $dateCssClass,
				'weekdayCssClass' => $weekdayCssClass,
				'isToday' => $isToday
			);
		}


		return $weekDays;
	}

	static function getWeekDayFontCss() {
		return array (
			0 => 'quiet sunday',
			1 => '',
			2 => '',
			3 => '',
			4 => '',
			5 => '',
			6 => 'quiet saturday',
		);
	}

	static function getRecurrenceFieldName()
	{
		return 'recurrence_of_id';
	}

	static function isModuleEnabled($module, $access)
	{
		return !ACLController::moduleSupportsACL($module) || ACLController::checkAccess($module, $access, true);
	}

	static function getLeaveOnDatesByUserSql($userIds, $datesClauses) {
		global $current_user;
		$strIds = join("','", $userIds);

		$lq = new ListQuery('Vacation', [], ['auto_add_fields' => false, 'skip_acl_groups_filter' => self::skip_acl_groups_filter()]);
		$lq->addField('id');
		$lq->addField('leave_type');
		$lq->addField('assigned_user_id');
		$lq->addFieldLiteral('is_daylong', "'1'");
		$lq->addFieldLiteral('is_private', "'0'");
		$lq->addField('days', 'name');
		$lq->addFieldLiteral('duration_hours', '(TO_DAYS(m.date_end) - TO_DAYS(m.date_start) ) * 24 + 1');
		$lq->addFieldLiteral('duration_minutes', "'0'");
		$lq->addField('date_start');
		$lq->addFieldLiteral('time_start', "''");
		$lq->addFieldLiteral('recurrence_id', "''");
		$lq->addFieldLiteral('module', "'Vacations'");
		$lq->addFieldLiteral('calendar_color', "''");
		$lq->addFieldLiteral('status', "''");
		$lq->addFieldLiteral('status_color', "''");
		$lq->addFieldLiteral('is_publish', "'1'");
		$lq->addFieldLiteral('accept_status', "'accept'");
		$lq->addField('assigned_user_id', 'user_id');

		$lq->addFilterClause("((m.leave_type='vacation' AND m.status IN('planned', 'approved')) OR (m.leave_type='sick' AND m.status = 'days_taken'))");

		foreach ($datesClauses as $clause) {
			$lq->addFilterClause($clause);
		}

		if (!empty($userIds) && !AppConfig::setting('company.show_all_vacations')) {
			$lq->addFilterClause("m.assigned_user_id IN ('{$strIds}')");
		}
		$lq->addAclFilter();
		return $lq->getSql(false, false, 'm');
	}

	static function getLeaveOnDayByUserSql(&$calDateTime, $userIds) {
		return self::getLeaveOnDatesByUserSql(
			$userIds,
			[
				"'{$calDateTime->localDate}' BETWEEN m.date_start AND m.date_end "
			]
		);
	}

	static function getMeetingsOnDatesByUserSql($userIds, $onlyBookings, $datesClauses, $innerJoin = null) {
        global $timedate;

		$lq = new ListQuery('Meeting', [], ['auto_add_fields' => false, 'skip_acl_groups_filter' => self::skip_acl_groups_filter()]);
		$lq->addField('id');
		$lq->addFieldLiteral('leave_type', "''");
		$lq->addField('assigned_user_id');
		$lq->addField('is_daylong');
		$lq->addField('is_private');
		$lq->addField('name');
		$lq->addFieldLiteral('duration_hours', "'0'");
		$lq->addField('duration', 'duration_minutes');
		$lq->addFieldLiteral('date_start', 'DATE(m.date_start)');
		$lq->addFieldLiteral('time_start', 'TIME(m.date_start)');
		$lq->addFieldLiteral('recurrence_id', "IFNULL("
                . "(SELECT id FROM recurrence_rules rr WHERE rr.parent_type='Meetings' AND rr.parent_id=m.id AND !rr.deleted LIMIT 1)"
                . ", m.".CalendarDataUtil::getRecurrenceFieldName() . ")");
		$lq->addFieldLiteral('module', "'Meetings'");
		$lq->addFieldLiteral('calendar_color', "''");
		$lq->addField('status');
		$lq->addFieldLiteral('status_color', "IF(m.status = 'Held' || m.status = 'Not Held', 'grey', IF(m.date_start < '".$timedate->get_gmt_db_datetime()."', 'red', 'green'))");
		$lq->addField('is_publish');

		if (!empty($userIds)) {
			$strUserIds = join("','", $userIds);
			$link = $lq->getLink('users', 'mu', $innerJoin);
			$lq->addSimpleFilter("mu~join.user_id", $userIds, 'in');
			$lq->addField('mu~join.accept_status', 'accept_status');
			$lq->addField('mu~join.user_id', 'user_id');
		} else {
			$lq->addFieldLiteral('accept_status', "''");
			$lq->addField('assigned_user_id', 'user_id');
		}

		foreach ($datesClauses as $clause) {
			$lq->addFilterClause($clause);
		}
		if ($onlyBookings) {
			$lq->addSimpleFilter("booking_app", '1');
		}
		$lq->addAclFilter();
		$sql = $lq->getSql(false, false, 'm');
		return $sql;
	}

	static function getMeetingsOnDayByUserSql(&$calDateTime, $userIds, $onlyBookings = false) {
		global $timedate;

		return self::getMeetingsOnDatesByUserSql(
			$userIds,
			$onlyBookings,
			[
				"(m.date_start <= '{$calDateTime->gmtNextDateTime}' AND DATE_ADD(m.date_end, INTERVAL m.duration MINUTE) >= '{$calDateTime->gmtPrevDateTime}')",
			]
		);
	}

	static function getEventsOnDatesByUserSql($userIds, $datesClauses) {
		$lq = new ListQuery('EventSession', [], ['auto_add_fields' => false, 'skip_acl_groups_filter' => self::skip_acl_groups_filter()]);
		$lq->addField('id');
		$lq->addFieldLiteral('leave_type', "''");
		$lq->addField('assigned_user_id');
		$lq->addFieldLiteral('is_daylong', "'0'");
		$lq->addFieldLiteral('is_private', "'0'");
		$lq->addField('name');
		$lq->addFieldLiteral('duration_hours', "IF(!m.no_date_start && !m.no_date_end, TIMEDIFF(m.date_end, m.date_start), '01:00')");
		$lq->addFieldLiteral('duration_minutes', "'###'");
		$lq->addFieldLiteral('date_start', 'DATE(IF(!m.no_date_start, m.date_start, m.date_end))');
		$lq->addFieldLiteral('time_start', 'TIME(IF(!m.no_date_start, m.date_start, m.date_end))');
		$lq->addFieldLiteral('recurrence_id', "''");
		$lq->addFieldLiteral('module', "'EventSessions'");
		$lq->addField('calendar_color');
		$lq->addFieldLiteral('status', "''");
		$lq->addFieldLiteral('status_color', "''");
		$lq->addFieldLiteral('is_publish', "'1'");
		$lq->addFieldLiteral('accept_status', "'accept'");
		$lq->addField('assigned_user_id', 'user_id');

		$lq->addFilterClause("(!m.no_date_start OR !m.no_date_end)");
		foreach ($datesClauses as $clause) {
			$lq->addFilterClause($clause);
		}
		$lq->addAclFilter();
		return $lq->getSql(false, false, 'm');
	}

	static function getEventsOnDayByUserSql(&$calDateTime, $userIds) {
		return self::getEventsOnDatesByUserSql(
			$userIds,
			[
				"((IF(!m.no_date_start, m.date_start, m.date_end) >= '{$calDateTime->gmtPrevDate} 00:00:00' AND IF(!m.no_date_start, m.date_start, m.date_end) <= '{$calDateTime->gmtNextDate} 23:59:59') "
				. " OR (IF(!m.no_date_end, m.date_end, m.date_start) >= '{$calDateTime->gmtPrevDate} 00:00:00' AND IF(!m.no_date_end, m.date_end, m.date_start) <= '{$calDateTime->gmtNextDate} 23:59:59'))"
			]
		);
	}

	static function getCallsOnDatesByUserSql($userIds, $datesClauses, $innerJoin = null) {
        global $timedate;

		$lq = new ListQuery('Call', [], ['auto_add_fields' => false, 'skip_acl_groups_filter' => self::skip_acl_groups_filter()]);
		$lq->addField('id');
		$lq->addFieldLiteral('leave_type', "''");
		$lq->addField('assigned_user_id');
		$lq->addFieldLiteral('is_daylong', "0");
		$lq->addField('is_private');
		$lq->addField('name');
		$lq->addFieldLiteral('duration_hours', "'0'");
		$lq->addField('duration', 'duration_minutes');
		$lq->addFieldLiteral('date_start', 'DATE(m.date_start)');
		$lq->addFieldLiteral('time_start', 'TIME(m.date_start)');
		$lq->addFieldLiteral('recurrence_id', "''");
		$lq->addFieldLiteral('module', "'Calls'");
		$lq->addFieldLiteral('calendar_color', "''");
		$lq->addField('status');
		$lq->addFieldLiteral('status_color', "IF(m.status = 'Held' || m.status = 'Not Held', 'grey', IF(m.date_start < '".$timedate->get_gmt_db_datetime()."', 'red', 'green'))");
		$lq->addField('is_publish');

		if (!empty($userIds)) {
			$strUserIds = join("','", $userIds);
			$link = $lq->getLink('users', 'mu', $innerJoin);
			$lq->addSimpleFilter("mu~join.user_id", $userIds, 'in');
			$lq->addField('mu~join.accept_status', 'accept_status');
			$lq->addField('mu~join.user_id', 'user_id');
		} else {
			$lq->addFieldLiteral('accept_status', "''");
			$lq->addField('assigned_user_id', 'user_id');
		}
		foreach ($datesClauses as $clause) {
			$lq->addFilterClause($clause);
		}
		$lq->addAclFilter();

		return $lq->getSql(false, false, 'm');
	}

	static function getCallsOnDayByUserSql(&$calDateTime, $userIds) {
		return self::getCallsOnDatesByUserSql(
			$userIds,
			[
				"(m.date_start <= '{$calDateTime->gmtNextDateTime}' AND DATE_ADD(m.date_end, INTERVAL m.duration MINUTE) >= '{$calDateTime->gmtPrevDateTime}')"
			]
		);
	}

	static function getProjectTasksOnDatesByUserSql($userIds, $datesClauses, $innerJoin = null) {
		global $timedate;

		$projectType = CalendarDataUtil::param('project_type');
		$projectId = CalendarDataUtil::param('project_id');


		$lq = new ListQuery('ProjectTask', [], ['auto_add_fields' => false, 'skip_acl_groups_filter' => self::skip_acl_groups_filter()]);
		$lq->addField('id');
		$lq->addFieldLiteral('leave_type', "''");
		$lq->addField('assigned_user_id');
		$lq->addFieldLiteral('is_daylong', "1");
		$lq->addFieldLiteral('is_private', "0");
		$lq->addField('name');
		$lq->addFieldLiteral('duration_hours', "TIMEDIFF(CONCAT(CONCAT(m.date_due, ' '), '23:59:59'), CONCAT(CONCAT(m.date_start, ' '), '00:00:00'))");
		$lq->addFieldLiteral('duration_minutes', "'###'");
		$lq->addFieldLiteral('date_start', 'DATE(m.date_start)');
		$lq->addFieldLiteral('time_start', "'00:00:00'");
		$lq->addFieldLiteral('recurrence_id', "''");
		$lq->addFieldLiteral('module', "'ProjectTask'");
		$lq->addFieldLiteral('calendar_color', "''");
		$lq->addField('status');
		$lq->addFieldLiteral('status_color', "IF(SUBSTR(m.status, 1, 6) = 'Closed', 'grey', IF(m.date_due < '".$timedate->get_gmt_db_date()."', 'red', 'green'))");
		$lq->addFieldLiteral('is_publish', "1");
		$lq->addFieldLiteral('accept_status', "'accept'");
		if (!empty($userIds)) {
			$strUserIds = join("','", $userIds);
			$link = $lq->getLink('booking_users', 'ptu', $innerJoin);
			$lq->addFilterClause([
				'operator' => 'OR',
				'multiple' => [
					[
						"field" => "ptu~join.user_id",
						'value' => $userIds,
						'operator' => 'in'
					],
					[
						"field" => "assigned_user_id",
						'value' => $userIds,
						'operator' => 'in'
					],
				],
			]);
			$lq->addField('ptu~join.user_id', 'user_id');
		} else {
			$lq->addField('assigned_user_id', 'user_id');
		}
		if (($projectType != null && $projectType != 'all') && $projectId != null) {
			$lq->addSimpleFilter('parent_id', $projectId);
		}
		foreach ($datesClauses as $clause) {
			$lq->addFilterClause($clause);
		}

		$lq->addAclFilter();
		$sql = $lq->getSql(false, false, 'm');
		return $sql;

	}

	static function getProjectTasksOnDayByUserSql(&$calDateTime, $userIds) {
		return self::getProjectTasksOnDatesByUserSql(
			$userIds,
			[
				"(DATE(m.date_start) <= '{$calDateTime->gmtNextDate}' AND DATE(m.date_due) >= '{$calDateTime->gmtPrevDate}')",
			]
		);

	}

	static function getBookedHoursOnDatesByUserSql($userIds, $datesClauses) {
		$timesheetId = CalendarDataUtil::param('timesheet_id');

		$lq = new ListQuery('BookedHours', [], ['auto_add_fields' => false, 'skip_acl_groups_filter' => self::skip_acl_groups_filter()]);
		$lq->addField('id');
		$lq->addFieldLiteral('leave_type', "''");
		$lq->addField('assigned_user_id');
		$lq->addFieldLiteral('is_daylong', "0");
		$lq->addFieldLiteral('is_private', "0");
		$lq->addField('name');
		$lq->addFieldLiteral('duration_hours', "0");
		$lq->addField('quantity', 'duration_minutes');
		$lq->addFieldLiteral('date_start', 'DATE(m.date_start)');
		$lq->addFieldLiteral('time_start', "TIME(m.date_start)");
		$lq->addFieldLiteral('recurrence_id', "''");
		$lq->addFieldLiteral('module', "'Booking'");
		$lq->addFieldLiteral('calendar_color', "''");
		$lq->addField('status');
		$lq->addFieldLiteral('status_color', "IF(m.status = 'draft', 'grey', IF(m.status = 'pending' || m.status = 'submitted', 'yellow', IF(m.status = 'approved', 'green', 'red')))");
		$lq->addFieldLiteral('is_publish', "1");
		$lq->addField('status', 'accept_status');
		$lq->addField('assigned_user_id', 'user_id');
		foreach ($datesClauses as $clause) {
			$lq->addFilterClause($clause);
		}
		if ($timesheetId != null) {
			$lq->addSimpleFilter('timesheet_id', $timesheetId);
		}
		if (!empty($userIds)) {
			$strUserIds = join("','", $userIds);
			$lq->addSimpleFilter("assigned_user_id", $userIds, 'in');
		}

		$lq->addAclFilter();
		$sql = $lq->getSql(false, false, 'm');
		return $sql;
	}

	static function getBookedHoursOnDayByUserSql(&$calDateTime, $userIds) {
		return self::getBookedHoursOnDatesByUserSql(
			$userIds,
			[
				"(DATE(m.date_start) >= '{$calDateTime->gmtDate}' AND DATE(m.date_start) <= '{$calDateTime->gmtNextDate}')",
			]
		);
	}

	static function getTasksOnDatesByUserSql($userIds, $datesClauses) {
        global $timedate;

		$lq = new ListQuery('Task', [], ['auto_add_fields' => false, 'skip_acl_groups_filter' => self::skip_acl_groups_filter()]);
		$lq->addField('id');
		$lq->addFieldLiteral('leave_type', "''");
		$lq->addField('assigned_user_id');
		$lq->addFieldLiteral('is_daylong', "0");
		$lq->addField('is_private');
		$lq->addField('name');
		$lq->addFieldLiteral('duration_hours', "0");
		$lq->addFieldLiteral('duration_minutes', "IF(effort_actual, effort_actual, effort_estim)");
		$lq->addFieldLiteral('date_start', 'DATE(IFNULL(m.date_due, m.date_start))');
		$lq->addFieldLiteral('time_start', "TIME(IFNULL(m.date_due, m.date_start))");
		$lq->addFieldLiteral('recurrence_id', "''");
		$lq->addFieldLiteral('module', "'Tasks'");
		$lq->addFieldLiteral('calendar_color', "''");
		$lq->addField('status');
		$lq->addFieldLiteral('status_color', "IF(m.status = 'Completed' || m.status = 'Pending Input' || m.status = 'Deferred', 'grey', IF(IFNULL(m.date_due, m.date_start) < '".$timedate->get_gmt_db_datetime()."', 'red', 'green'))");
		$lq->addFieldLiteral('is_publish', "1");
		$lq->addFieldLiteral('accept_status', "'accept'");
		$lq->addField('assigned_user_id', 'user_id');

		if (!empty($userIds)) {
			$strUserIds = join("','", $userIds);
			$lq->addSimpleFilter("assigned_user_id", $userIds, 'in');
		}
		foreach ($datesClauses as $clause) {
			$lq->addFilterClause($clause);
		}

		$lq->addAclFilter();
		$sql = $lq->getSql(false, false, 'm');
		return $sql;

	}

	static function getTasksOnDayByUserSql(&$calDateTime, $userIds) {
		return self:: getTasksOnDatesByUserSql(
			$userIds,
			[
				"DATE(IFNULL(m.date_due, m.date_start)) <= '{$calDateTime->gmtNextDate}'",
				"DATE(IFNULL(m.date_due, m.date_start)) >= '{$calDateTime->gmtPrevDate}'",
			]
		);
	}

	static function getMeetingsOnDatesByResourceSql($resourceIds, $datesClauses, $innerJoin = null) {
        global $timedate;

		if (is_array($resourceIds)) {
			$resourceIds = array_filter($resourceIds);
		}

		$lq = new ListQuery('Meeting', [], ['auto_add_fields' => false, 'skip_acl_groups_filter' => self::skip_acl_groups_filter()]);
		$lq->addField('id');
		$lq->addField('assigned_user_id');
		$lq->addFieldLiteral('leave_type', "''");
		$lq->addField('is_daylong');
		$lq->addField('name');
		$lq->addFieldLiteral('duration_hours', "'0'");
		$lq->addField('duration', 'duration_minutes');
		$lq->addFieldLiteral('date_start', 'DATE(m.date_start)');
		$lq->addFieldLiteral('time_start', 'TIME(m.date_start)');
		$lq->addFieldLiteral('module', "'Meetings'");
		$lq->addFieldLiteral('calendar_color', "''");
		$lq->addField('status');
		$lq->addFieldLiteral('status_color', "IF(m.status = 'Held' || m.status = 'Not Held', 'grey', IF(m.date_start < '".$timedate->get_gmt_db_datetime()."', 'red', 'green'))");
		$lq->addField('is_publish');

		$link = $lq->getLink('resources', 'mr', $innerJoin);
		$lq->addField('mr~join.resource_id', 'resource_id');
		$lq->addSimpleFilter("mr~join.deleted", 0); // filter out meetings that do not use resources
		foreach ($datesClauses as $clause) {
			$lq->addFilterClause($clause);
		}
		if (!empty($resourceIds)) {
			$lq->addSimpleFilter('mr~join.resource_id', $resourceIds);
		}
		$lq->addAclFilter();
		$lq->setGroupBy("m.id");
		$lq->setOrderBy("m.date_start");

		$sql = $lq->getSql(false, false, 'm');
		return $sql;
	}

	static function getMeetingsOnDayByResourceSql(&$calDateTime, $resourceId) {
		return self::getMeetingsOnDatesByResourceSql(
			$resourceId,
			[
				"(DATE(m.date_start) <= '{$calDateTime->gmtNextDate}' AND DATE(m.date_end) >= '{$calDateTime->gmtPrevDate}')",
			]
		);
	}

	static function formatEventStatus($row) {
		static $fdefs = [];
		static $fmt;
		if(! $fmt) {
			$fmt = new FieldFormatter('html', 'view');
		}
		if(! empty($row['status']) && ! empty($row['module'])) {
			if(! isset($fdefs[$row['module']])) {
				$model = AppConfig::module_primary_bean($row['module']);
				$fdefs[$row['module']] = AppConfig::setting("model.fields.$model.status", false);
			}
			$fdef = $fdefs[$row['module']];
			if($fdef) {
				return $fmt->formatRowValue($fdef, $row, $row['status']);
			}
		}
		return '';
	}

	static function skip_acl_groups_filter() {
		return !AppConfig::setting("site.calendar_group_acl", false);
	}

	static function formatRelatedFields(&$activity, $primaryModule, $primaryId, $fields) {
		$model = AppConfig::module_primary_bean($primaryModule);
		$result = ListQuery::quick_fetch($model, $primaryId, $fields);

		foreach ($fields as $field) {
			$fieldValue = $result->getField($field);

			if (! $fieldValue)
				continue;

			$fDef = $result->getFieldDefinition($field);

			if (isset($fDef['dynamic_module'])) {
				$fieldModule = $result->getField($fDef['dynamic_module']);
				$fieldBean = AppConfig::module_primary_bean($fieldModule);
			} else {
				$fieldModule = $fDef['module_dir'];
				$fieldBean = $fDef['bean_name'];
			}

			$inner = '<div class="input-icon theme-icon bean-' .$fieldBean.
				'" style="vertical-align: text-top"></div>&nbsp;' .$fieldValue;

			$formatted = '<a href="index.php?module=' .$fieldModule. '&action=DetailView&record=' .
				$result->getField($fDef['id_name']). '" class="tabDetailViewDFLink">'.$inner.'</a>';

			if (! isset($activity['inviteesHtml'])) {
				$activity['inviteesHtml'] = $formatted;
			} else {
				$activity['inviteesHtml'] .= '<br>' . $formatted;
			}

		}
	}

	static function populateExtendedInfo(&$activities) {
		$related = [
			'Calls' => 'invitees',
			'Meetings' => 'invitees',
			'Tasks' => ['account', 'contact', 'parent']
		];
		$populate = [];
		foreach($activities as $idx => &$act) {
			$act['statusHtml'] = self::formatEventStatus($act);
			$mod = $act['module'];
			if(isset($related[$mod])) $populate[$mod][$idx] = $act['id'];
		}

		foreach($populate as $mod => $ids) {
			if (! is_array($related[$mod])) {
				$widget = new InviteesList(
					['link_name' => $related[$mod]],
					new ModelDef(AppConfig::module_primary_bean($mod)));
				foreach($ids as $idx => $id) {
					$rows = $widget->loadLinkedRows($id, 'html', []);
					$value = $widget->renderRows($rows, 'html', []);
					$activities[$idx]['inviteesHtml'] = $value;
				}
			} else {
				foreach($ids as $idx => $id) {
					self::formatRelatedFields($activities[$idx], $mod, $id, $related[$mod]);
				}
			}

		}

	}

	static function getActivitiesOnDay(&$calDateTime, $targetType, $targets, $teamSecurity = false,
		$activityWidth = CAL_DAY_ACTIVITY_WIDTH, $leftOffset = 0, $dupOffset = -1, $isMerge = false,
		$exceptMeetingIds = array(), $holidays=false, $daylongDupe=false, $debug=false) {

		global $timedate;
		global $current_user, $current_language;
		global $db;

		$dateFormats = CalendarDateTime::userDateTimeFormats();
		$strings = return_module_language($current_language, "Calendar");
		if(! is_array($activityWidth)) $activityWidth = array($activityWidth, $activityWidth);

		$activities = array ();
		if($holidays && function_exists('holiday')) {
			$holiday = holiday($calDateTime->localYear, $calDateTime->localMonth, $calDateTime->localDay);
			if($holiday)
				$activities[] = array(
					'module' => 'Holidays',
					'isDuplicate' => false,
					'is_daylong' => true,
					'startTime' => '',
					'endTime' => '',
					'height' => CAL_HEIGHT_PER_HOUR + CAL_HEIGHT_TUNE_OFFSET,
					'left' => $leftOffset + CAL_DAY_LEFT_OFFSET,
					'width' => $activityWidth[0],
					'forDate' => $calDateTime->localDate,
					'subject' => $holiday);
		}

		if ($targetType == 'user' || $targetType == 'all') {
			if ($targetType == 'all') {
				$targetId = '';
			}
			$sqlClauses = [];
			if (CalendarDataUtil::param('display_leave') && CalendarDataUtil::isModuleEnabled('Vacations', 'list')) {
				$sqlClauses[] = CalendarDataUtil::getLeaveOnDayByUserSql($calDateTime, $targets);
			}
			if (CalendarDataUtil::param('display_meetings') && CalendarDataUtil::isModuleEnabled('Meetings', 'list')) {
				$sqlClauses[] = CalendarDataUtil::getMeetingsOnDayByUserSql($calDateTime, $targets);
			}
			if (CalendarDataUtil::param('display_bookings') && CalendarDataUtil::isModuleEnabled('Meetings', 'list')) {
				$sqlClauses[] = CalendarDataUtil::getMeetingsOnDayByUserSql($calDateTime, $targets, true);
			}
			if (CalendarDataUtil::param('display_calls') && CalendarDataUtil::isModuleEnabled('Calls', 'list')) {
				$sqlClauses[] =  CalendarDataUtil::getCallsOnDayByUserSql($calDateTime, $targets);
			}
			if (CalendarDataUtil::param('display_events') && CalendarDataUtil::isModuleEnabled('EventSessions', 'list')) {
				$sqlClauses[] =  CalendarDataUtil::getEventsOnDayByUserSql($calDateTime, $targets);
			}
			if (CalendarDataUtil::param('display_tasks') && CalendarDataUtil::isModuleEnabled('Tasks', 'list')) {
				$sqlClauses[] =  CalendarDataUtil::getTasksOnDayByUserSql($calDateTime, $targets);
			}
			if (CalendarDataUtil::param('display_project_tasks') && CalendarDataUtil::isModuleEnabled('ProjectTask', 'list')) {
				$sqlClauses[] =  CalendarDataUtil::getProjectTasksOnDayByUserSql($calDateTime, $targets);
			}
			if (CalendarDataUtil::param('display_booked_hours') && CalendarDataUtil::isModuleEnabled('Booking', 'list')) {
				$sqlClauses[] =  CalendarDataUtil::getBookedHoursOnDayByUserSql($calDateTime, $targets);
			}
			$sql = join(") UNION (", $sqlClauses);
			if (!empty ($sql)) {
				$sql = "($sql)  ORDER BY date_start, time_start";
			}
		} elseif (CalendarDataUtil::isModuleEnabled('Meetings', 'list')) {
			$sql = CalendarDataUtil::getMeetingsOnDayByResourceSql($calDateTime, $targets);
		}

		if (empty($sql)) {
			return $activities;
		}

		if($debug)
			pr2($sql, 'calendar sql', true);

		$resultSet = $db->query($sql);
		if ($resultSet === false) {
			return $activities;
		}

		$recurrenceImg = '<div class="input-icon icon-recur"></div>';
		$meeting_strings = return_module_language($current_language, "Meetings");

		$maxDupLevel = 0;

		while ($row = $db->fetchByAssoc($resultSet)) {
			if ($row['date_start'] == '0000-00-00') {
				continue;
			}
			if(in_array($row['id'], $exceptMeetingIds)) {
				continue;
			}
			if ($row['module'] == 'Vacations') {
				if($row['leave_type'] == 'sick')
					$row['name'] = get_assigned_user_name($row['assigned_user_id']) . translate('LBL_SEPARATOR', 'app') .  sprintf($strings['LBL_VACATION_SICK_DAYS'], $row['name']);
				else
					$row['name'] = get_assigned_user_name($row['assigned_user_id']) . translate('LBL_SEPARATOR', 'app') .  sprintf($strings['LBL_VACATION_LEAVE_DAYS'], $row['name']);
			}
			if ($row['duration_minutes'] == '###') {
				$s = explode(':', $row['duration_hours']);
				$row['duration_hours'] = $s[0];
				$row['duration_minutes'] = count($s) > 1 ? $s[1] : '00';
			}
			$tz = ($row['module'] == 'ProjectTask' ? CAL_BASE_LOCAL : CAL_BASE_GMT);
			$startDT = new CalendarDateTime("{$row['date_start']} {$row['time_start']}", false, $tz, false);

			$durationMin = intval($row['duration_hours']) * 60 + intval($row['duration_minutes']);
			if ($row['module'] == 'Tasks')
				$durationMin = 1;
			$localEnd_ts = strtotime("{$startDT->localDateTime} +{$durationMin} minutes");
			$localEndDateTime = date("Y-m-d H:i:s", $localEnd_ts);
			if(substr($localEndDateTime, -9) == ' 00:00:00' && $startDT->localDateTime_t != $localEnd_ts && $row['module'] != 'ProjectTask') {
				// for events ending at midnight, don't carry over into next day
				$localEnd_ts --;
				$localEndDateTime = date("Y-m-d H:i:s", $localEnd_ts);
			}

			//check time zone
			if ( $row['module'] != 'Vacations' && ($localEnd_ts < $calDateTime->localDateTime_t || $startDT->localDateTime > $calDateTime->localNextDateTime)) {
				continue;
			}
			//...check time zone

			if ($row['module'] == 'Tasks') {
				if ($localEndDateTime < $calDateTime->localDateTime || $localEndDateTime > $calDateTime->localNextDateTime) {
					continue;
				}
			}

			global $current_user;
			$startHour = $current_user->day_begin_hour;
			$endHour = $current_user->day_end_hour;
			if(empty($startHour) && empty($endHour)) {
				$startHour = 9;
				$endHour = 18;
			}

			$fromPrevDay = $toNextDay = false;
			$startMin = ($startHour - floor($startHour)) * 60;
			$startHour = floor($startHour);
			//Start Time and End Time
			$localDateTime = date(CAL_DATE_FORMAT, $calDateTime->localDateTime_t) . sprintf(" %02d:00:00", $startHour);
			if ($startDT->localDateTime < $localDateTime) {
				$fromPrevDay = true;
			//if ($startDT->localDateTime <= $calDateTime->localDateTime) {
				$top = 0;
				//$diffSec = strtotime($calDateTime->localDateTime) - strtotime($startDT->localDateTime);
				$diffSec = strtotime($localDateTime) - $startDT->localDateTime_t;
				$height = CAL_HEIGHT_PER_HOUR * (intval($row['duration_hours']) + intval($row['duration_minutes']) / 60 - $diffSec / 3600);
				//$displayStartTime = date('(n/j)G:i', $startDT->localDateTime_t);
				$startOffsetMinutes = 0;
				$durationMin = $durationMin - ($diffSec/60);
			} else {
				$top = CAL_HEIGHT_PER_HOUR * (($startDT->localHour + $startDT->localMin / 60) - $startHour);
				$height = CAL_HEIGHT_PER_HOUR * (intval($row['duration_hours']) + intval($row['duration_minutes']) / 60);
				$startOffsetMinutes = $startDT->localHour * 60 + $startDT->localMin;
			}
			//check step over the day
			$top = min($top, CAL_HEIGHT_PER_HOUR * ($endHour - $startHour) - CAL_HEIGHT_PER_HOUR / 2);

			/*$displayStartTime = date($dateFormats['time_no_zero'], $startDT->localDateTime_t);
			$displayEndTime = date($dateFormats['time_no_zero'], $localEnd_ts);*/
			$showEndTs = ($row['module'] == 'Tasks') ? null : $localEnd_ts;
			list($displayStartTime, $displayEndTime) = CalendarDataUtil::formatActivityTimes($startDT, $showEndTs);

			if (($top + $height) > CAL_HEIGHT_PER_HOUR * ($endHour - $startHour)) {
				$toNextDay = true;
				$height = CAL_HEIGHT_PER_HOUR * ($endHour - $startHour) - $top;
			}

			if ($row['module'] == 'ProjectTask') {
				$displayEndTime = date($dateFormats['date_short'], $localEnd_ts);
				$displayStartTime = date($dateFormats['date_short'], $startDT->localDateTime_t);
			}
			/*
			if ($row['module'] == 'Tasks') {
				$displayEndTime = date("n/j", strtotime($localEndDateTime));
				$displayStartTime = date("n/j", $startDT->localDateTime_t);
			}
			*/

			$top += CAL_HEIGHT_PER_HOUR;

			if ($fromPrevDay && $toNextDay) {
				$row['is_daylong'] = 1;
			}

			if ($row['module'] == 'EventSessions') {
				$row['is_daylong'] = 1;
			}
			if ($row['module'] == 'ProjectTask') {
				$row['is_daylong'] = 1;
			}

			if (!empty($row['is_daylong'])) {
				$top = 0;
				$height = CAL_HEIGHT_PER_HOUR;
			}

			if ($height < CAL_HEIGHT_PER_HOUR/2) {
				$height = CAL_HEIGHT_PER_HOUR/2;
			}

			$row['contacts'] = array();

			if ($row['module'] == 'Meetings') {
				$row['contacts'] = MeetingEx::getParticipantsById($row['id']);
			}
			if ($row['module'] == 'Calls') {
				$row['contacts'] = CallEx::getParticipantsById($row['id']);
			}

			if (CalendarDataUtil::param('add_contacts')) {
				if ($row['module'] == 'Meetings') {
					foreach (MeetingEx::getContactsById($row['id']) as $cid => $cname) {
						$row['contacts'][$cid] = $cname;
					}
				}
				if ($row['module'] == 'Calls') {
					foreach (CallEx::getContactsById($row['id']) as $cid => $cname) {
						$row['contacts'][$cid] = $cname;
					}
				}
			}

			//check duplication...
			$dupLevel = 0;
			$isDuplicate = false;
			if(empty($row['is_daylong']) || $daylongDupe) {
				foreach ($activities as $id => $regActivity) {
					if($regActivity['is_daylong'] && !$daylongDupe)
						continue;

					if ($startDT->localDateTime < $regActivity['endDateTime'] && $localEndDateTime > $regActivity['startDateTime'])
					{
						$isDuplicate = $activities[$id]['isDuplicate'] = true;
						$dupLevel = max($dupLevel, $regActivity['duplicateLevel'] + 1);
						$maxDupLevel = max($dupLevel, $maxDupLevel);

						if($isMerge) {
							$isModified = false;
							if($startDT->localDateTime < $regActivity['startDateTime']) {
								$activities[$id]['startDateTime'] = $startDT->localDateTime;
								$activities[$id]['startTime'] = $displayStartTime;
								$activities[$id]['startOffsetMinutes'] = $startOffsetMinutes;
								$isModified = true;
							}
							if($localEndDateTime > $regActivity['endDateTime']) {
								$activities[$id]['endDateTime'] = $localEndDateTime;
								$activities[$id]['endTime'] = $displayEndTime;
								$isModified = true;
							}
							if($isModified) {
								if ($startDT->localDateTime <= $calDateTime->localDateTime) {
									$startDateTime = $calDateTime->localDate . ' 00:00:00';
								} else {
									$startDateTime = $calDateTime->localDate . ' ' . $activities[$id]['startTime'];
								}
								$endDateTime = $localEndDateTime;
								$diffSec = strtotime($endDateTime) - strtotime($startDateTime);
								$activities[$id]['durationMin'] = $diffSec / 60;
							}
							//break;
						}
					}
				}
			}

			if($isMerge) {
				if($isDuplicate) {
					continue;
				}
			}
			$row['is_duplicate'] = $isDuplicate;

			$workRecurrenceImg = "";
			if (!empty ($row['recurrence_id'])) {
				$workRecurrenceImg = $recurrenceImg;
			}

			$isViewAble = true;
			$subject = $row['name'];

			//if non Publish...
			if($row['is_publish'] == 0) {
				$cls = substr($row['module'], 0, -1) . 'Ex';
				$participants = call_user_func(array($cls, 'getParticipantsById'), $row['id']);
				if(!array_key_exists($current_user->id, $participants)) {
					$subject = "({$meeting_strings['LBL_NON_PUBLISH']})";
					$isViewAble = false;
				} else {
					$subject .= "({$meeting_strings['LBL_NON_PUBLISH']})";
				}
			}
			//...if non Publish

			if (@$row['is_private'] && $current_user->id != $row['assigned_user_id']) {
				if (!isset($row['contacts'][$current_user->id])) {
					$subject = $strings['LBL_PRIVATE_' . strtoupper($row['module'])];
					$isViewAble = false;
				}
			}
			if (!CalendarDataUtil::param('add_contacts')) {
				$row['contact'] = [];
			}

			$assigned_user = array_get_default($row, 'assigned_user_id');
			$isViewAble = $isViewAble &&
				CalendarDataUtil::hasAccess($row['module'], $row['id'], $assigned_user, 'view');
			$canEdit = $isViewAble &&
				CalendarDataUtil::hasAccess($row['module'], $row['id'], $assigned_user, 'edit');
			if($row['module'] == 'Booking' && ($row['accept_status'] == 'submitted' || $row['accept_status'] == 'approved'))
				$canEdit = false;

			$activities[] = array (
				'id' => $row['id'],
				'top' => $top,
				'left' => $leftOffset + CAL_DAY_LEFT_OFFSET,
				'width' => $activityWidth[0],
				'height' => $height + CAL_HEIGHT_TUNE_OFFSET,
				'startTime' => $displayStartTime,
				'endTime' => $displayEndTime,
				'startDateTime' => $startDT->localDateTime,
				'endDateTime' => $localEndDateTime,
				'startOffsetMinutes' => $startOffsetMinutes,
				'subject' => $subject,
				'imgHTML' => get_image($row['module'], "border='0' hspace='0' vspace='0'"),
				'recurrenceImgHTML' => $workRecurrenceImg,
				'durationMin' => $durationMin,
				'duplicateLevel' => $dupLevel,
				'isDuplicate' => $isDuplicate,
				'isPublish' => $row['is_publish'],
				'is_daylong' => $row['is_daylong'],
				'isViewAble' => $isViewAble,
				'canEdit' => $canEdit,
				'module' => $row['module'],
				'forDate' => $calDateTime->localDate,
				'contacts' => $row['contacts'],
				'is_private' => array_get_default($row, 'is_private', 0),
				'status' => $row['status'],
                'status_color' => $row['status_color'],
                'calendar_color' => (! empty($row['calendar_color'])) ? 'background-color: '.$row['calendar_color'].';' : '',
                'cssClass' => self::getCssClass($row),
			);
		}

		// adjust left position for overlapping events
		if($dupOffset == -1 && $maxDupLevel) {
			$dupOffset = (CAL_DAY_WIDTH - CAL_DAY_ACTIVITY_WIDTH - CAL_MIN_DUP_OFFSET) / $maxDupLevel;
			$dupOffset = round(max(CAL_MIN_DUP_OFFSET, min(CAL_DAY_ACTIVITY_WIDTH+3, $dupOffset)));
		}
		self::populateExtendedInfo($activities);
		foreach($activities as $id => $regActivity) {
			if(! $maxDupLevel)
				$activities[$id]['width'] = $activityWidth[1];
			if($regActivity['isDuplicate'])
				$activities[$id]['left'] += $regActivity['duplicateLevel'] * $dupOffset;
		}

		if (count($activities) > 1) {
			global $current_language;
			$counts = array();
			$texts = array();
			foreach ($activities as $activity) {
				if ($activity['is_daylong']) {
					@$counts[$activity['module']]++;
				}
			}
			if (array_sum($counts) > 1) {
				foreach($counts as $module => $n) {
					$texts[] = sprintf($strings['LBL_ACTIVITY_COUNT_' . strtoupper($module)], $n);
				}
				$activities['summary'] = join(', ', $texts);
			}
		}
		//$isDuplicate
		//$dupOffset
		//$maxDupLevel

		return $activities;
	}

	static function getCssClass($activity) {
		$cls = '';
		if($activity && ! empty($activity['status_color'])) {
			if($activity['status_color'] == 'red') {
				$cls = 'panel-danger activity-overdue';
			}
			else if($activity['status_color'] == 'yellow') {
				$cls = 'panel-alert activity-special';
			}
			else if($activity['status_color'] == 'grey') {
				$cls = 'panel-muted activity-ignore';
			}
			else
				$cls = 'panel-info activity-normal';
		}
		if(! empty($activity['is_duplicate']))
			$cls .= ' activity-dupe';
		return $cls;
	}

	static function getWeekActivitiesEveryUser(& $calDateTime, & $users, $teamSecurity = false, $weekends=true, $removeEmpty = true) {
		global $timedate;
		global $current_user, $current_language;
		global $db;
		$strings = return_module_language($current_language, "Calendar");
		$dateFormats = CalendarDateTime::userDateTimeFormats();
		$firstDay = $calDateTime->systemWeekFirstDay;

		$activitiesEveryUserArray = array();
		if (in_array($current_user->id, $users)) {
			$activitiesEveryUserArray[$current_user->id] = "";
		}
		if (empty($users)) {
			$users = array($current_user->id);
		}

		$quoteUserIds = array();
		foreach ($users as $userId) {
			$quoteUserIds[] = "'{$userId}'";
			if(! isset($stdDayArray)) {
				$stdDayArray = array();
				for ($i = 0; $i < 7; $i++) {
					if(! $weekends && ($i == 0 || $i == 6))
						continue;
					$stdDayArray[$i] = array(
						'activities' => array()
					);
				}
			}
			$activitiesEveryUserArray[$userId] = array (
				'user_full_name' => get_user_full_name($userId),
				'activities_of_day_array' => $stdDayArray,
			);
		}
		$strUserIds = implode(",", $quoteUserIds);

		$gmtEndDateTime = date("Y-m-d H:i:s", strtotime("+7 day", strtotime($calDateTime->gmtWeekFirstDateTime)));
		$gmtEndDate = date("Y-m-d", strtotime($gmtEndDateTime));

		$sqlClauses = [];

		if (CalendarDataUtil::param('display_leave') && CalendarDataUtil::isModuleEnabled('Vacations', 'list')) {
			$sqlClauses[] = self::getLeaveOnDatesByUserSql(
				$users,
				[
					"m.date_start <= '{$gmtEndDate}'",
					"m.date_end >= '{$calDateTime->gmtWeekFirstDate}'"
				]
			);
		}

		if ( (CalendarDataUtil::param('display_meetings') || CalendarDataUtil::param('display_bookings')) && CalendarDataUtil::isModuleEnabled('Meetings', 'list')) {
			$where = '';

			$whereClauses = [
				"jt0.user_id IS NOT NULL",
				"(DATE(m.date_start) <= '{$gmtEndDate}' AND DATE(m.date_end) >= '{$calDateTime->gmtWeekFirstDate}')"
			];

			if (CalendarDataUtil::param('display_bookings'))
				$whereClauses[] = "m.booking_app = 1";
			$sqlClauses[] = self::getMeetingsOnDatesByUserSql($users, false, $whereClauses, true);
		}

		if (CalendarDataUtil::param('display_calls') && CalendarDataUtil::isModuleEnabled('Calls', 'list')) {
			$sqlClauses[] = self::getCallsOnDatesByUserSql(
				$users,
				[
					"(DATE(m.date_start) <= '{$gmtEndDate}' AND DATE(m.date_end) >= '{$calDateTime->gmtWeekFirstDate}')"
				],
				true
			);
		}

		if (CalendarDataUtil::param('display_tasks') && CalendarDataUtil::isModuleEnabled('Tasks', 'list')) {
			$sqlClauses[] = self::getTasksOnDatesByUserSql(
				$users,
				[
					"DATE(IFNULL(m.date_due, m.date_start)) >= '{$calDateTime->gmtWeekFirstDate}'",
					"DATE(IFNULL(m.date_due, m.date_start)) <= '{$gmtEndDate}'",
					"(m.date_due IS NOT NULL OR m.date_start IS NOT NULL)",
				]
			);
		}

		if (CalendarDataUtil::param('display_project_tasks') && CalendarDataUtil::isModuleEnabled('ProjectTask', 'list')) {
			$sqlClauses[] = self::getProjectTasksOnDatesByUserSql(
				$users,
				[
					" ((DATE(m.date_start) >= '{$calDateTime->gmtWeekFirstDate}' AND DATE(m.date_start) <= '{$gmtEndDate}') "
					. " OR (DATE(m.date_due) >= '{$calDateTime->gmtWeekFirstDate}' AND DATE(m.date_due) <= '{$gmtEndDate}') "
					. " OR (DATE(m.date_start) <= '{$calDateTime->gmtWeekFirstDate}' AND DATE(m.date_due) >= '{$gmtEndDate}'))"
				],
				true
			);

		}

		if (CalendarDataUtil::param('display_booked_hours') && CalendarDataUtil::isModuleEnabled('Booking', 'list')) {
			$sqlClauses[] = self::getBookedHoursOnDatesByUserSql(
				$users,
				[
					"(DATE(m.date_start) >= '{$calDateTime->gmtWeekFirstDate}' AND DATE(m.date_start) <= '{$gmtEndDate}') "
				]
			);

		}

		if (CalendarDataUtil::param('display_events') && CalendarDataUtil::isModuleEnabled('EventSessions', 'list')) {
			$sqlClauses[] = self::getEventsOnDatesByUserSql(
				$users,
				[
					"("
					. " (IF(!m.no_date_start, m.date_start, m.date_end) >= '{$calDateTime->gmtWeekFirstDate} 00:00:00' AND IF(!m.no_date_start, m.date_start, m.date_end) <= '{$gmtEndDate} 23:59:59') "
					. " OR (IF(!m.no_date_end, m.date_end, m.date_start) >= '{$calDateTime->gmtWeekFirstDate} 00:00:00' AND IF(!m.no_date_end, m.date_end, m.date_start) <= '{$gmtEndDate} 23:59:59') "
					. " OR (IF(!m.no_date_start, m.date_start, m.date_end) <= '{$calDateTime->gmtWeekFirstDate} 23:59:00' AND IF(!m.no_date_end, m.date_end, m.date_start) >= '{$gmtEndDate} 00:00:00')"
					. ") "
				]
			);
		}

		$sql = join(") UNION (", $sqlClauses);
		if (!empty($sql)) {
			$sql = "($sql) ORDER BY date_start, time_start ";
		}

		if (empty($sql)) {
			return $activitiesEveryUserArray;
		}

		$resultSet = $db->query($sql);
		if ($resultSet === false) {
			return $activitiesEveryUserArray;
		}

		$meetingImg = get_image('Meetings', "border='0' hspace='0' vspace='0'");
		$recurrenceImg = '<div class="input-icon icon-recur"></div>';
		$meeting_strings = return_module_language($current_language, "Meetings");

		while ($row = $db->fetchByAssoc($resultSet)) {
			if ($row['date_start'] == '0000-00-00') {
				continue;
			}
			if ($row['module'] == 'Meetings') {
				$row['contacts'] = MeetingEx::getParticipantsById($row['id']);
			}
			if ($row['module'] == 'Calls') {
				$row['contacts'] = CallEx::getParticipantsById($row['id']);
			}
			if ($row['duration_minutes'] == '###') {
				$s = explode(':', $row['duration_hours']);
				$row['duration_hours'] = $s[0];
				$row['duration_minutes'] = count($s) > 1 ? $s[1] : '00';
			}

			if ($row['module'] == 'Vacations') {
				if($row['leave_type'] == 'sick')
					$row['name'] = get_assigned_user_name($row['assigned_user_id']) . translate('LBL_SEPARATOR', 'app') .  sprintf($strings['LBL_VACATION_SICK_DAYS'], $row['name']);
				else
					$row['name'] = get_assigned_user_name($row['assigned_user_id']) . translate('LBL_SEPARATOR', 'app') .  sprintf($strings['LBL_VACATION_LEAVE_DAYS'], $row['name']);

			}

			$tz = CAL_BASE_GMT;
			//$tz = ( ($row['module'] == 'ProjectTask' || $row['module'] == 'Vacations') ? CAL_BASE_LOCAL : CAL_BASE_GMT);
            if (empty($row['time_start'])) {
                $row['time_start'] = '00:00:00';
                $tz = CAL_BASE_LOCAL;
            }
			$startDT = new CalendarDateTime("{$row['date_start']} {$row['time_start']}", false, $tz, false);


			$durationMin = intval($row['duration_hours']) * 60 + intval($row['duration_minutes']);
			if($row['module'] == 'Tasks')
				$localEndDateTime_t = $startDT->localDateTime_t;
			else
				$localEndDateTime_t = strtotime("+{$durationMin} minutes", $startDT->localDateTime_t);
			$localEndDateTime = date("Y-m-d H:i:s", $localEndDateTime_t);
			if(substr($localEndDateTime, -9) == ' 00:00:00' && $localEndDateTime_t != $startDT->localDateTime_t && $row['module'] != 'ProjectTask') {
				// for events ending at midnight, don't carry over into next day
				$localEndDateTime_t --;
				$localEndDateTime = date("Y-m-d H:i:s", $localEndDateTime_t);
			}

			//check time zone
			if ($localEndDateTime < $calDateTime->localWeekFirstDate.' 00:00:00' || $startDT->gmtDateTime >= $gmtEndDateTime) {
				continue;
			}

			$startWeekDayIndex = date("w", $startDT->localDateTime_t);
			$endWeekDayIndex = date("w", $localEndDateTime_t);

			$activityImg = $meetingImg;

			$workRecurrenceImg = "";
			if (!empty ($row['recurrence_id'])) {
				$workRecurrenceImg = $recurrenceImg;
			}

			if ($row['module'] == 'ProjectTask' /* || $row['module'] == 'Tasks'*/) {
				$displayEndTime = date($dateFormats['date_short'], $localEndDateTime_t);
				if($row['module'] == 'Tasks') {
					$displayEndTime .= ' '.date($dateFormats['time_no_zero'], $localEndDateTime_t);
					$displayStartTime = '';
				} else
					$displayStartTime = date($dateFormats['date_short'], $startDT->localDateTime_t);
			}
			else {
				/*$displayStartTime = date($dateFormats['time_no_zero'], $startDT->localDateTime_t);
				$displayEndTime = date($dateFormats['time_no_zero'], $localEndDateTime_t);*/
				$showEnd = ($row['module'] == 'Tasks') ? null : $localEndDateTime_t;
				list($displayStartTime, $displayEndTime) = CalendarDataUtil::formatActivityTimes($startDT, $showEnd);
			}

			//Adjust loop variable.
			$startIndex = $startWeekDayIndex;
			$endIndex = $endWeekDayIndex;

			if($startDT->localDateTime_t < strtotime($calDateTime->localWeekFirstDate)) {
				$startIndex = $firstDay;
			}

			if($localEndDateTime_t >= strtotime($calDateTime->localNextWeekDate)) {
				$endIndex = $firstDay - 1;

				if($endIndex < 0) $endIndex = 6;

			}
			//...Adjust

			$weekIndex = $startIndex;
			for ($weekIndex = $startIndex; ; $weekIndex++) {

				if($weekIndex > 6) $weekIndex = 0;
				if(! $weekends && ($weekIndex == 0 || $weekIndex == 6)) {
					if($weekIndex == $endIndex) break;
					continue;
				}

				//check duplication...
				$is_duplicate = 0;
				if ($row['module'] != 'EventSessions' && $row['module'] != 'ProjectTask' && $row['module'] != 'Booking')
				foreach ((array)@$activitiesEveryUserArray[$row['user_id']]['activities_of_day_array'][$weekIndex]['activities'] as $id => $regActivity) {
					if ($regActivity['module'] == 'EventSessions' || $regActivity['module'] == 'ProjectTask' || $regActivity['module'] == 'Booking') continue;
					if (($startDT->localDateTime >= $regActivity['startDateTime'] && $startDT->localDateTime < $regActivity['endDateTime']) || ($localEndDateTime >= $regActivity['startDateTime'] && $localEndDateTime < $regActivity['endDateTime']) || ($startDT->localDateTime < $regActivity['startDateTime'] && $localEndDateTime > $regActivity['endDateTime'])) {
						$activitiesEveryUserArray[$row['user_id']]['activities_of_day_array'][$weekIndex]['activities'][$id]['is_duplicate'] = 1;
						$is_duplicate = 1;
					}
				}
				$row['is_duplicate'] = $is_duplicate;
				//...check duplication

				$isViewAble = true;
				$subject = $row['name'];

				//if non Publish...
				if($row['is_publish'] == 0) {
					$cls = substr($row['module'], 0, -1) . 'Ex';
					$participants =  call_user_func(array($cls, 'getParticipantsById') , $row['id']);
					if(!array_key_exists($current_user->id, $participants)) {
						$subject = "({$meeting_strings['LBL_NON_PUBLISH']})";
						$isViewAble = false;
					} else {
						$subject .= "({$meeting_strings['LBL_NON_PUBLISH']})";
					}
				}
				//...if non Publish

				if ($row['is_private'] && $current_user->id != $row['assigned_user_id']) {
					if (!isset($row['contacts'][$current_user->id])) {
						$subject = $strings['LBL_PRIVATE_' . strtoupper($row['module'])];
						$isViewAble = false;
					}
				}

				$assigned_user = array_get_default($row, 'assigned_user_id');
				$isViewAble = $isViewAble &&
					CalendarDataUtil::hasAccess($row['module'], $row['id'], $assigned_user, 'view');
				$canEdit = $isViewAble &&
					CalendarDataUtil::hasAccess($row['module'], $row['id'], $assigned_user, 'edit');

				if ($row['module'] == 'Vacations') {
					$displayStartTime = '';
					$displayEndTime = '';
				}

				$activitiesEveryUserArray[$row['user_id']]['activities_of_day_array'][$weekIndex]['activities'][$row['id']] = array (
					'startTime' => $displayStartTime,
					'endTime' => $displayEndTime,
					'startDateTime' => $startDT->localDateTime,
					'endDateTime' => $localEndDateTime,
					'subject' => $subject,
					'imgHTML' => get_image($row['module'], "border='0' hspace='0' vspace='0'"),
					'recurrenceImgHTML' => $workRecurrenceImg,
					'isDuplicate' => $is_duplicate,
					'module' => $row['module'],
					'isPublish' => $row['is_publish'],
					'isAllDay' => $row['is_daylong'],
					'isViewAble' => $isViewAble,
					'canEdit' => $canEdit,
					'status' => $row['status'] . '..',
                    'status_color' => $row['status_color'],
                    'calendar_color' => (! empty($row['calendar_color'])) ? 'background-color: '.$row['calendar_color'].';' : '',
                    'cssClass' => self::getCssClass($row),
				);

				if($weekIndex == $endIndex) break;
			}
		}

		foreach ($activitiesEveryUserArray as $user_id => $acts) {
			$sum = 0;
			foreach($acts['activities_of_day_array'] as $perday) {
				if (!empty($perday['activities'])) {
					$sum = 1;
					break;
				}
			}
			if (!$sum && $removeEmpty) {
				unset($activitiesEveryUserArray[$user_id]);
			} else {
				$replace = array();
				for ($i = $firstDay; $i < 7; $i++) {
					if(! $weekends && ($i == 0 || $i == 6))
						continue;
					$replace[$i] = array_get_default($acts['activities_of_day_array'], $i, array());
				}
				for ($i = 0; $i < $firstDay; $i++) {
					if(! $weekends && ($i == 0 || $i == 6))
						continue;
					$replace[$i] = array_get_default($acts['activities_of_day_array'], $i, array());
				}
				$activitiesEveryUserArray[$user_id]['activities_of_day_array'] = $replace;
			}
		}
		return $activitiesEveryUserArray;
	}

	static function getWeekActivitiesEveryResource(& $calDateTime, &$resources, $teamSecurity = false, $weekends=true) {
		global $timedate;
		global $current_user, $current_language;
		global $db;

		$dateFormats = CalendarDateTime::userDateTimeFormats();
		$activitiesEveryResourceArray = array ();

		if (array_key_exists($current_user->id, $resources)) {
			$activitiesEveryResourceArray[$current_user->id] = "";
		}

		foreach ($resources as $resourceId => $resourceName) {
			$activitiesEveryResourceArray[$resourceId] = array (
				'user_full_name' => $resourceName,
				'activities_of_day_array' => array ()
			);

			for ($i = 0; $i < 7; $i++) {
				if(! $weekends && ($i == 0 || $i == 6))
					continue;
				$activitiesEveryResourceArray[$resourceId]['activities_of_day_array'][$i] = array (
					'activities' => array ()
				);
			}
		}

		$gmtEndDateTime_t = strtotime("{$calDateTime->gmtWeekFirstDateTime} +7 day");
		$gmtEndDateTime = date("Y-m-d H:i:s", $gmtEndDateTime_t);
		$gmtEndDate = substr($gmtEndDateTime, 0, 10);

		$sql = self::getMeetingsOnDatesByResourceSql(
			array_keys($resources),
			[
				[
					'operator' => 'or',
					'multiple' => [
						"(m.date_start >= '{$calDateTime->gmtWeekFirstDate}' AND m.date_start <= '{$gmtEndDate}')",
						"(m.date_end >= '{$calDateTime->gmtWeekFirstDate}' AND m.date_end <= '{$gmtEndDate}')",
						"(m.date_start <= '{$calDateTime->gmtWeekFirstDate}' AND m.date_end >= '{$gmtEndDate}')",
					],
				],
			],
			true
		);

		$resultSet = $db->query($sql);
		if ($resultSet === false) {
			return $activitiesEveryResourceArray;
		}

		$meetingImg = get_image('Meetings', "border='0' hspace='0' vspace='0'");
		$recurrenceImg = '<div class="input-icon icon-recur"></div>';
		$meeting_strings = return_module_language($current_language, "Meetings");

		while ($row = $db->fetchByAssoc($resultSet)) {
			if ($row['date_start'] == '0000-00-00') {
				continue;
			}
			$startDT = new CalendarDateTime("{$row['date_start']} {$row['time_start']}", true, CAL_BASE_GMT, false);

			$durationMin = intval($row['duration_hours']) * 60 + intval($row['duration_minutes']);
			$localEndDateTime_t = strtotime("+{$durationMin} minutes", $startDT->localDateTime_t);
			$localEndDateTime = date("Y-m-d H:i:s", $localEndDateTime_t);

			//check time zone
			if ($localEndDateTime < $calDateTime->localWeekFirstDate || $startDT->gmtDateTime > $gmtEndDateTime) {
				continue;
			}

			$startWeekDayIndex = $startDT->localWeekDayIndex;
			$endWeekDayIndex = date("w", $localEndDateTime_t);

			$activityImg = $meetingImg;

			$workRecurrenceImg = "";
			if (!empty ($row['recurrence_id'])) {
				$workRecurrenceImg = $recurrenceImg;
			}

			//Adjust loop variable.
			$startIndex = $startWeekDayIndex;
			$endIndex = $endWeekDayIndex;

			if($startDT->localDateTime_t < strtotime($calDateTime->localWeekFirstDate)) {
				$startIndex = $weekends ? 0 : 1;
			} else if($localEndDateTime_t > strtotime($calDateTime->localNextWeekDate)) {
				$endIndex = $weekends ? 6 : 5;
			}
			//...Adjust

			for ($weekIndex = $startIndex; $weekIndex <= $endIndex; $weekIndex++) {
				if(! $weekends && ($weekIndex == 0 || $weekIndex == 6))
					continue;

				list($displayStartTime, $displayEndTime) = CalendarDataUtil::formatActivityTimes($startDT, $localEndDateTime_t);

				//check duplication...
				$is_duplicate = 0;
				if (isset($activitiesEveryResourceArray[$row['resource_id']])) {
					foreach ($activitiesEveryResourceArray[$row['resource_id']]['activities_of_day_array'][$weekIndex]['activities'] as $id => $regActivity) {
						if (($startDT->localDateTime >= $regActivity['startDateTime'] && $startDT->localDateTime < $regActivity['endDateTime']) || ($localEndDateTime >= $regActivity['startDateTime'] && $localEndDateTime < $regActivity['endDateTime']) || ($startDT->localDateTime < $regActivity['startDateTime'] && $localEndDateTime > $regActivity['endDateTime'])) {
							$activitiesEveryResourceArray[$row['resource_id']]['activities_of_day_array'][$weekIndex]['activities'][$id]['is_duplicate'] = 1;
							$is_duplicate = 1;
						}
					}
				}
				$row['is_duplicate'] = $is_duplicate;
				//...check duplication

				$isViewAble = true;
				$subject = $row['name'];

				//if non Publish...
				if($row['is_publish'] == 0) {
					$cls = substr($row['module'], 0, -1) . 'Ex';
					$participants =  call_user_func(array($cls, 'getParticipantsById'), $row['id']);
					if(!array_key_exists($current_user->id, $participants)) {
						$subject = "({$meeting_strings['LBL_NON_PUBLISH']})";
						$isViewAble = false;
					} else {
						$subject .= "({$meeting_strings['LBL_NON_PUBLISH']})";
					}
				}
				//...if non Publish

				$assigned_user = array_get_default($row, 'assigned_user_id');
				$isViewAble = $isViewAble &&
					CalendarDataUtil::hasAccess($row['module'], $row['id'], $assigned_user, 'view');
				$canEdit = $isViewAble &&
					CalendarDataUtil::hasAccess($row['module'], $row['id'], $assigned_user, 'edit');

				$activitiesEveryResourceArray[$row['resource_id']]['activities_of_day_array'][$weekIndex]['activities'][$row['id']] = array (
					'startTime' => $displayStartTime,
					'endTime' => $displayEndTime,
					'startDateTime' => $startDT->localDateTime,
					'endDateTime' => $localEndDateTime,
					'subject' => $subject,
					'imgHTML' => get_image($row['module'], "border='0' hspace='0' vspace='0'"),
					'recurrenceImgHTML' => $workRecurrenceImg,
					'isDuplicate' => $is_duplicate,
					'module' => $row['module'],
					'isPublish' => $row['is_publish'],
					'isViewAble' => $isViewAble,
					'canEdit' => $canEdit,
					'cssClass' => self::getCssClass($row),
				);
			}
		};

		foreach ($activitiesEveryResourceArray as $user_id => $acts) {
				$replace = [];
				for ($i = $calDateTime->systemWeekFirstDay; $i < 7; $i++) {
					if(! $weekends && ($i == 0 || $i == 6))
						continue;
					$replace[$i] = array_get_default($acts['activities_of_day_array'], $i, []);
				}
				for ($i = 0; $i < $calDateTime->systemWeekFirstDay; $i++) {
					if(! $weekends && ($i == 0 || $i == 6))
						continue;
					$replace[$i] = array_get_default($acts['activities_of_day_array'], $i, []);
				}
				$activitiesEveryResourceArray[$user_id]['activities_of_day_array'] = $replace;
		}
		return $activitiesEveryResourceArray;
	}


	static function ownerWhere($module, $table = 'm', $field = 'assigned_user_id')
	{
		global $current_user, $beanList, $beanFiles;
		static $clauses = array();
		if (isset($clauses[$module])) {
			return $clauses[$module];
		}
		$clsname = $beanList[$module];
		require_once $beanFiles[$clsname];
		if ( ACLController::requireOwner($module, 'list') ){
			$bean = new $clsname;
			$level = ACLAction::getUserActions($current_user->id, false, $module, 'module', 'list');
			$ownerWhere = $bean->getOwnerWhere($current_user->id, $level['aclaccess'], $table, $field);
			$clause = 'AND ' . $ownerWhere;
		} else {
			$clause = '';
		}
		$clauses[$module] = $clause;
		return $clause;
	}

	static function hasAccess($module, $id, $assigned_user_id, $action)
	{

		if(($module == 'Tasks' || $module == 'ProjectTask') && $action == 'edit')
			return false; // cannot edit using meeting editor

		require_once("modules/SecurityGroups/SecurityGroup.php");
		$in_group = SecurityGroup::groupHasAccess($module, $id);

		return ACLController::checkAccess($module, $action, $assigned_user_id, $in_group);
	}
}
?>
