<?php

require_once 'include/upload_file.php';
require_once 'include/GoogleSync.php';

class GoogleCalendarSync extends GoogleSync
{

	protected function getConfigKey()
	{
		return 'google_calendar';
	}

	protected function getDirection()
	{
		return $this->user->getPreference('google_calendar_direction');
	}

	function stringEndsWith($haystack,$needle,$case=true) {
		$expectedPosition = strlen($haystack) - strlen($needle);
		if ($case){
			return strrpos($haystack, $needle, 0) === $expectedPosition;
		}
		return strripos($haystack, $needle, 0) === $expectedPosition;
	}

	protected function getRestrictSince()
	{
		$since = $this->user->getPreference('google_restrict_since');

		if($this->stringEndsWith($since, 'd')) {
			$since = str_replace('d', ' days', $since);
		} else if($this->stringEndsWith($since, 'm')) {
			$since = str_replace('m', ' months', $since);
		} else if($this->stringEndsWith($since, 'y')) {
			$since = str_replace('y', ' years', $since);
		} else {
			$since = 'all';
		}

		if($since != 'all')
			$since = date('Y-m-d\TH:i:s\Z', strtotime(date('Y-m-d 00:00:00'). ' - '.$since));

		return $since;
	}

	protected function getService()
	{
		$service = new Google_Service_Calendar($this->client);
		return $service;
	}

	protected function getUpdated($item) {
		return $item->getUpdated();
	}

	protected function getQuery()
	{
		$query = array(
			'showDeleted' => true,
			'timeZone' => 'UTC',
			'orderBy' => 'updated'
		);
		$res_since = $this->getRestrictSince();
		if($res_since != 'all') {
			$query['timeMin'] = $res_since;
		}
		return $query;
	}

	protected function createIAHItem($sql_row = null)
	{
		$id = null;
		if (is_array($sql_row))
			$id = array_get_default($sql_row, 'id');
		$module = array_get_default($sql_row, 'module', 'Meetings');
		switch ($module) {
			case 'Calls':
				$model = 'Call';
				break;
			case 'Tasks':
				$model = 'Task';
				break;
			case 'Recurrence':
				$model = 'RecurrenceRule';
				break;
			default:
				$model = 'Meeting';
				break;
		}
		$row = null;
		if ($id)
			$row = ListQuery::quick_fetch($model, $id, true, array('filter_deleted' => false));
		if ($row)
			return RowUpdate::for_result($row);
		return RowUpdate::blank_for_model($model);

	}

	protected function setIAHItemFields($iahItem, $item)
	{
		global $db;
		$googleRerurId = $item->getRecurringEventId();
		$origStartTime = $item->getOriginalStartTime();

		$update = [];

		$update['name'] =  $item->getSummary();
		if (!strlen($update['name'])) {
			$update['name'] = translate('LBL_EMPTY_MEETING_TITLE', 'Calendar');
		}
		$meeting_params = json_decode($iahItem->getField('online_meeting_params'));
		$desc = $item->getDescription();
		if($meeting_params->join_url) {
			require_once('include/OnlineMeetings/OnlineMeetingProcess.php');
			$iahItem->setRelatedData('online_meeting_create', 1);
        	$online_meeting_process = new OnlineMeetingProcess();
        	$online_meeting_process->run($iahItem);
			$desc = str_replace('

'.$meeting_params->join_url, '', $item->getDescription());
		}
		if($iahItem->getField('gtm_id') != '') {
			require_once('include/OnlineMeetings/OnlineMeetingProcess.php');
			$iahItem->setRelatedData('online_meeting_create', 1);
        	$online_meeting_process = new OnlineMeetingProcess();
        	$online_meeting_process->run($iahItem);
			$desc = str_replace('

https://www.gotomeeting.com/join/' . $iahItem->getField('gtm_id'), '', $item->getDescription());
		}
		$update['description'] = $desc;

		$times = $this->format_itime($iahItem, $item);
		$update['date_start'] = $times['start_primary'] . ' ' . $times['start_secondary'];
		$update['is_private'] = $item->getVisibility() == 'private' ? 1 : 0;

		switch($iahItem->getModelName())
		{
			case "Task":
				$update['date_due'] = $times['end_primary'] . ' ' . $times['end_secondary'];
				if ($iahItem->new_record) {
					$update['priority'] = "P1";
					$update['status'] = "Not Started";
				}
				break;
			// for recurrence,  prepare the same data as for meeting,
			// it will be used as recurrence template
			case "RecurrenceRule":
			case "Meeting":
				if ($googleRerurId) {
					$update['recurrence_exception'] = 1;
				}
				$update['location'] = $item->getLocation();
				if ($iahItem->new_record) {
					$update['parent_type'] = "Accounts";
					$update['status'] = "Planned";
				}
				$update['duration'] = $times['end_primary'] * 60 + $times['end_secondary'];
				$update['is_daylong'] = $times['is_daylong'];
				$update['reminder_time'] = $times['reminder_time'];
				$update['date_end'] = $times['date_end'];
				break;

			default:
				$update['duration'] = $times['end_primary'] * 60 + $times['end_secondary'];
				$update['is_daylong'] = $times['is_daylong'];
				$update['reminder_time'] = $times['reminder_time'];
				$update['date_end'] = $times['date_end'];
				break;
		}

		if ($iahItem->new_record)
			$update['assigned_user_id'] = $this->user->id;

		list($origRule, $newRule) = $this->getRules($times, $iahItem);

		if (!$iahItem->new_record) {
			if ($newRule && !$origRule) {
				$update['assigned_user_id'] = $this->user->id;
				$newRule['template'] = json_encode($update);
				return $this->convertNormalToRecurring($newRule, $iahItem, $item);
			} elseif (!$newRule && $origRule) {
				$update['assigned_user_id'] = $this->user->id;
				return $this->convertRecurringToNormal($update, $iahItem, $item);
			} elseif ($newRule && $origRule) {
				$update['assigned_user_id'] = $this->user->id;
				$newRule['template'] = json_encode($update);
				return $this->updateRecurring($newRule, $iahItem);
			}
		} elseif ($iahItem->getModelName() == 'RecurrenceRule') {
			$update['assigned_user_id'] = $this->user->id;
			$newRule['template'] = json_encode($update);
			$update = $newRule;
			$update['parent_type'] = 'Meetings';
		}

		$iahItem->set($update);
		$iahItem->setRelatedData('skip_first_instance', false);
	}

	private function getRules($times, $iahItem) {
		global $timedate;
		$origRule = $newRule = null;
		if ($iahItem->getModelName() == 'RecurrenceRule') {
			$origRule = $iahItem->getRow();
		}

		if (!empty($times['rule'])) {
			$newRule = [];
			$rules = $times['rule'];
			$until = null;
			if (isset($rules['UNTIL'])) {
				$until = $rules['UNTIL'];
				if (strpos($until, 'T') !== false)
					$until = gmdate('Y-m-d H:i:s', strtotime(str_replace("Z", "", $rules['UNTIL']) . ' GMT'));
				else {
					$untilDate = gmdate('Y-m-d', strtotime($rules['UNTIL'] . ' GMT'));
					$until = $timedate->handle_offset($untilDate . " 23:59:59", "Y-m-d H:i:s", false, $this->user->id);
				}
			}
			$newRule['freq'] = $rules['FREQ'];
			$newRule['freq_interval'] = array_get_default($rules, 'INTERVAL', 1);
			$newRule['until'] =  $until;
			unset($rules['UNTIL']);
			unset($rules['FREQ']);
			unset($rules['INTERVAL']);

			$limit_count = array_get_default($rules, 'COUNT', 0);

			$ruleStr = '';
			foreach ($rules as $k => $v) {
				if (!empty($ruleStr)) $ruleStr .= ';';
				$ruleStr .= "$k=$v";
			}
			$newRule['rule'] =  $ruleStr;
		}
		return [$origRule, $newRule];
	}

	private function convertNormalToRecurring($rule, $iahItem, $item) {
		$actions = [
			['perform' => 'delete', 'item' => $iahItem],
			['perform' => 'delete_sync', 'item' => $iahItem],
		];
		$update = RowUpdate::blank_for_model('RecurrenceRule');
		$update->set($rule);
		$update->set('parent_type', 'Meetings');
		$update->setRelatedData('skip_first_instance', false);
		$update->_google_fields['google_id'] = $this->getItemId($item);
		$update->set('date_entered', gmdate('Y-m-d H:i:s', strtotime($this->getUpdated($item))));
		$update->set('date_modified', $update->_google_fields['google_last_sync'] = gmdate('Y-m-d H:i:s'));
		$update->_google_fields['google_timestamp'] = $this->getUpdated($item);
		$actions[] = ['perform' => 'save', 'item' => $update];
		return $actions;
	}

	private function convertRecurringToNormal($upd, $iahItem, $item) {
		$actions = [
			['perform' => 'delete', 'item' => $iahItem],
			['perform' => 'delete_sync', 'item' => $iahItem],
		];
		$update = RowUpdate::blank_for_model('Meeting');
		$update->set($upd);
		$update->set('assigned_user_id', $this->user->id);
		$update->_google_fields['google_id'] = $this->getItemId($item);
		$update->set('date_entered', gmdate('Y-m-d H:i:s', strtotime($this->getUpdated($item))));
		$update->set('date_modified', $update->_google_fields['google_last_sync'] = gmdate('Y-m-d H:i:s'));
		$update->_google_fields['google_timestamp'] = $this->getUpdated($item);
		$actions[] = ['perform' => 'save', 'item' => $update];
		return $actions;
	}

	private function updateRecurring($rule, $iahItem) {
		$iahItem->setRelatedData('skip_first_instance', false);
		$iahItem->_google_fields['google_last_sync'] = gmdate('Y-m-d H:i:s');
		$iahItem->set($rule);
	}


	protected function getSQLQuery($lastSyncTime)
	{
		$limit = (int)((int)array_get_default($this->params, 'max_items', 500)/3);
		$limit = 200;
		if ($limit < 1) {
			$limit = 1;
		}
		$tables = array('meetings' => 'Meetings', 'calls' => 'Calls', 'tasks' => 'Tasks');
		$queries = array();
		foreach ($tables as $table => $module) {
			if (empty($this->options['sync'][$module])) continue;
			$query = "SELECT id, '{$module}' as module FROM {$table} c "
				." LEFT JOIN google_sync s ON s.related_id=c.id AND s.user_id='".$this->user->id . "' AND s.related_type='{$module}'";
			if ($module == 'Meetings' && $this->options['sync']['Meetings'] == 2) {
				$query .= " LEFT JOIN meetings_users mu ON mu.meeting_id=c.id AND mu.deleted=0 AND mu.user_id='{$this->user->id}'";
			}
			if ($module == 'Calls' && $this->options['sync']['Calls'] == 2) {
				$query .= " LEFT JOIN calls_users mu ON mu.call_id=c.id AND mu.deleted=0 AND mu.user_id='{$this->user->id}'";
			}
			$exceptionClause = 
				$module == 'Meetings'
				? 'OR c.recurrence_exception '
				: ''
			;
			$query .= " WHERE (IFNULL(s.google_id,'')!='' OR NOT c.deleted $exceptionClause) ";
			$query .= " AND (c.date_modified > s.google_last_sync OR s.google_last_sync IS NULL)";
			if (($module == 'Meetings' || $module == 'Calls') && $this->options['sync'][$module] == 2) {
				$query .= " AND (assigned_user_id = '{$this->user->id}' OR mu.user_id='{$this->user->id}') ";
				if ($module == 'Meetings')
					$query .= " AND (IFNULL(recurrence_rule_id, '') = '' OR recurrence_exception) ";
			} else {
				$query .= " AND assigned_user_id = '{$this->user->id}' ";
			}
			$query .= ' ORDER BY c.date_modified ';
			$query .= ' LIMIT ' . $limit;
			$queries[] = $query;

			if ($module == 'Meetings') {
				$query = "SELECT id, 'Recurrence' as module FROM recurrence_rules c "
					." LEFT JOIN google_sync s ON s.related_id=c.id AND s.user_id='".$this->user->id . "' AND s.related_type='Recurrence'";
				$query .= " WHERE (IFNULL(s.google_id,'')!='' OR NOT c.deleted) ";
				$query .= " AND (c.date_modified > s.google_last_sync OR s.google_last_sync IS NULL)";
				$queries[] = $query;
			}

		}
		if (!empty($queries)) {
			return '(' . join(') UNION (', $queries) . ')';
		} else {
			return	"SELECT id FROM meetings WHERE 0";
		}
	}

	protected function setGoogleItemFields($iahItem, $item)
	{
		if ($iahItem->getModelName() == 'RecurrenceRule') {
			$data = @json_decode($iahItem->getField('template'), true);
			if (!is_array($data)) $data = [];
		} else {
			$data = $iahItem->getRow();
		}

		$item->setSummary(array_get_default($data, 'name'));
		$item->setLocation(array_get_default($data, 'location'));
		$meeting_params = json_decode(array_get_default($data, 'online_meeting_params'));
		$link = '';
		if($meeting_params->join_url)
			$link .= '

'.$meeting_params->join_url;
		else if(array_get_default($data, 'gtm_id') != '') {
			$link .= "

https://www.gotomeeting.com/join/" . array_get_default($data, 'gtm_id');
		}
		$item->setDescription(array_get_default($data, 'description').$link);
		$item->visibility = (array_get_default($data, 'is_private')? 'private' : 'public');

		$gtime = $this->format_gtime($iahItem);
		if ($gtime['start'] > $gtime['end']) {
			$gtime['end'] = $gtime['start'];
		}

		$key = strlen($gtime['start']) > 10 ? 'dateTime' : 'date';
		$start = new Google_Service_Calendar_EventDateTime(array($key => $gtime['start'], 'timeZone' => 'UTC'));

		$key = strlen($gtime['end']) > 10 ? 'dateTime' : 'date';
		$end = new Google_Service_Calendar_EventDateTime(array($key => $gtime['end'], 'timeZone' => 'UTC'));

		$item->setStart($start);
		$item->setEnd($end);

		$rem_array = array();

		if (array_get_default($data, 'reminder_time') > 0) {
			$rem = new Google_Service_Calendar_EventReminder(array('method' => 'popup', 'minutes' => ceil($iahItem->getField('reminder_time') / 60)));
			$rem_array[] = $rem;
		}

		if ($iahItem->getField('email_reminder_time') > 0) {
			$rem = new Google_Service_Calendar_EventReminder(array('method' => 'email', 'minutes' => ceil($iahItem->getField('email_reminder_time') / 60)));
			$rem_array[] = $rem;
		}

		if (!empty($rem_array)) {
			$reminders = new Google_Service_Calendar_EventReminders(array('overrides' => $rem_array, 'useDefault' => false));
			$item->setReminders($reminders);
		} else {
			$reminders = new Google_Service_Calendar_EventReminders(array('useDefault' => true));
		}

		if ($iahItem->getModelName() == "RecurrenceRule") {
				$parsedRule = $this->parseRule($iahItem->getField('rule'));
				$parsedRule['FREQ'] = $iahItem->getField('freq');
				$parsedRule['INTERVAL'] = $iahItem->getField('freq_interval');
				$parsedRule['COUNT'] = $iahItem->getField('limit_count');
				if ($iahItem->getField('until')) {
					$parsedRule['UNTIL'] = gmdate('Ymd\\THis\\Z', strtotime($iahItem->getField('until')));
				}
				$start = gmdate('Ymd\\THis\\Z', strtotime($iahItem->getField('date_start')));
				$m = (int)$iahItem->getField('duration');

				$end = gmdate('Ymd\\THis\\Z', strtotime($start . " + $m minutes" ));
				if ($iahItem->getField('is_daylong')) {
					$ruleText = 'DTSTART;VALUE=DATE:' . $start . "\n";
					$ruleText .= 'DTEND;VALUE=DATE:' . $end . "\n";
				} else {
					$ruleText = 'DTSTART:' . $start . "\n";
					$ruleText .= 'DTEND:' . $end . "\n";
				}
				if (isset($parsedRule['UNTIL']) && $parsedRule['UNTIL'] < $start) {
					unset($parsedRule['UNTIL']);
				}
				if (!empty($parsedRule['COUNT'])) {
					unset($parsedRule['UNTIL']);
				} else {
					unset($parsedRule['COUNT']);
				}
				// $ruleText .= "RRULE:" ; !!!!!!!!!!
				$ruleText = "RRULE:" ;
				$i = 0;
				foreach ($parsedRule as $k => $v) {
					if ($i) $ruleText .= ';';
					$ruleText .= "$k=$v";
					$i++;
				}
				$item->setRecurrence(array($ruleText));
		}
	}

	protected function realCreateGoogleItem($iahItem)
	{
		$id = $this->user->getPreference('google_calendar_id');
		if (empty($id)) {
			$id = 'primary';
		}
		$item = new Google_Service_Calendar_Event;
		try	{
			$this->setGoogleItemFields($iahItem, $item);
			$item = $this->service->events->insert($id, $item);
		} catch (Exception $e) {
			$item_id = $iahItem ? $iahItem->getField('id') : '';
			$GLOBALS['log']->fatal("Failed to create new Google event ($item_id): ".$e->getMessage());
			$item = null;
		}
		return $item;
	}

	protected function deleteItem($item) {
		$calid = $this->user->getPreference('google_calendar_id');
		if (empty($calid)) {
			$calid = 'primary';
		}
		try	{
			$this->service->events->delete($calid, $item->getId());
		} catch (Exception $e) {}
	}

	protected function realUpdateGoogleItem($item, $iahItem)
	{
		$this->setGoogleItemFields($iahItem, $item);
		return $item;
	}

	protected function getListEntry($googleId) {
		$calid = $this->user->getPreference('google_calendar_id');
		if (empty($calid)) {
			$calid = 'primary';
		}
		$ret =  $this->service->events->get($calid, $googleId);
		return $ret;
	}

	function createGoogleItem($iahItem) {
		global $timedate;
		$calid = $this->user->getPreference('google_calendar_id');
		if (empty($calid)) {
			$calid = 'primary';
		}
		$recurOfId = $iahItem->getField('recurrence_rule_id');
		$origStartTime = $iahItem->getField('recurrence_start_time');
		$isException = $iahItem->getField('recurrence_exception');
		$isAllDay = $iahItem->getField('is_daylong');

		if ($isException && $recurOfId) {
			$lq = new ListQuery('google_sync');
			$lq->addSimpleFilter('related_id', $recurOfId);
			$lq->addSimpleFilter('user_id', $this->user->id);
			$sync_data = $lq->runQuerySingle();
			if ($sync_data && !$sync_datia->failed) {
				$originalStart = gmdate('Y-m-d\TH:i:s\Z', strtotime($origStartTime . ' GMT'));
				if ($isAllDay) {
					$originalStart = $timedate->handle_offset($origStartTime, "Y-m-d", true, $this->user->id);
				}
				$params = [
					'originalStart' => $originalStart,
					'showDeleted' => true,
				];
				try {
					$items = $this->service->events->instances($calid, $sync_data->getField('google_id'), $params);
					$itemsList = $items->getItems();
					if (!empty($itemsList)) {
						$item = $itemsList[0];
						$iahItem->_google_fields['google_id'] = $item->getId();
						$iahItem->_google_fields['google_timestamp'] = $this->getUpdated($item);
						$iahItem->_google_fields['google_last_sync'] = $iahItem->getField('date_modified');
						return parent::updateGoogleItem($iahItem);
					}
				} catch (Exception $e) {
					$GLOBALS['log']->fatal($e->getMessage());
				}
			}
			return;
		}
		return parent::createGoogleItem($iahItem);
	}

	function get_event_time($event) {
		$start = $event->getStart();
		$end = $event->getEnd();
		$allday = 0;

		if (!empty($start['date'])) {
			$allday = 1;
			$start_epoch = strtotime($start['date'] . ' GMT');
			$end_epoch = strtotime($end['date'] . ' GMT');
		} else {
			$start_epoch = strtotime($start['dateTime']);
			$end_epoch = strtotime($end['dateTime']);
		}
		return compact('start_epoch', 'end_epoch', 'allday');
	}

	function format_itime($focus, $event)
	{
		global $timedate;
		$reminder = 0;
		$rule = null;

		extract($this->get_event_time($event));

		$rec = $event->getRecurrence();
		if (!empty($rec)) {
			foreach ($rec as $rl) {
				$m = array();
				if (preg_match('/^RRULE:(.*)$/m', $rl, $m)) {
					$rule = $m[1];
				}
			}
		}

		$rule = $this->parseRule($rule);

		$start_date = gmdate("Y-m-d", $start_epoch);
		$start_time = gmdate("H:i:s", $start_epoch);
		$date_end = gmdate("Y-m-d", $end_epoch);

		switch ($focus->getModelName())
		{
			case "Task":
				$end_primary = gmdate("Y-m-d", $end_epoch);
				$end_secondary = gmdate("H:i:s", $end_epoch);
				break;

			default:
				$duration = $end_epoch - $start_epoch;
				$end_primary = floor($duration / 3600);
				$end_secondary = floor(($duration % 3600) / 60);
				break;
		}



		return	array(
			"start_primary" => $start_date,
			"start_secondary" => $start_time,
			"end_primary" => $end_primary,
			"end_secondary" => $end_secondary,
			"date_end" => $date_end,
			"is_daylong" => $allday,
			"reminder_time" => $reminder,
			"rule" => $rule,
		);
	}

	function format_gtime($focus)
	{
		global $timedate;

		$data = $focus->getRow();
		if ($focus->getModelName() == 'RecurrenceRule') {
			$data = json_decode($focus->getField('template'), true);
		}
		$start  = array_get_default($data, 'date_start');
		$start_epoch = strtotime($start . " GMT ");
		$start_date = gmdate("Y-m-d", $start_epoch);
		$start_time = gmdate("H:i:s", $start_epoch);

		switch ($focus->getModelName())
		{
			case "Task":
				$end_epoch = strtotime(array_get_default($data, 'date_due') . ' GMT');
				if (!array_get_default($data, 'date_due') || array_get_default($data, 'date_due') == '0000-00-00 00:00:00') {
					$end_date = $start_date;
					$end_time = $start_time;
				} else {
					$end_date = gmdate("Y-m-d", $end_epoch);
					$end_time = gmdate("H:i:s", $end_epoch);
					if (!$start) {
						$start_date = gmdate("Y-m-d", $end_epoch - 60);
						$start_time = gmdate("H:i:s", $end_epoch - 60);
					}
				}
				break;

			default:
				$end_epoch =  $start_epoch + (array_get_default($data, 'duration') * 60);
				$end_date = gmdate("Y-m-d", $end_epoch);
				$end_time = gmdate("H:i:s", $end_epoch);
				break;
		}

		if (array_get_default($data, 'is_daylong'))
		{
			$gstart = $timedate->handle_offset(array_get_default($data, 'date_start'), 'Y-m-d', true, $this->user);
			//$gstart = "{$start_date}";
			$gend = "";
		}
		else
		{
			$gstart = "{$start_date}T{$start_time}.000+00:00";
			$gend = "{$end_date}T{$end_time}.000+00:00";
		}

		return array("start" => $gstart, "end" => $gend);
	}

	protected function canCreateInIAH($item, $iahItem)
	{
		return $iahItem->getModelName() == 'Meeting' || $iahItem->getModelName() == 'RecurrenceRule';
	}

	protected function parseRule($rule)
	{
		$ret = array();
		$m = array();
		if (preg_match_all('/([^=;]*)=([^=;]*)($|;)/u', $rule, $m)) {
			foreach($m[1] as $i => $k) {
				$ret[$k] = $m[2][$i];
			}
		}
		return $ret;
	}

	protected function loadIahItemForGoogleItem($item)
	{
		$data = [];
		$googleId = $this->getItemId($item);
		$googleRerurId = $item->getRecurringEventId();
		$googleRerurRule = $item->getRecurrence();
		$origStartTime = $item->getOriginalStartTime();

		$clause = $this->getGoogleIdClause($googleId);
		$lq = new ListQuery('google_sync', array('related_type', 'related_id', 'google_id'));
		$lq->addFilterClause($clause);
		$result = $lq->runQuerySingle();
		if ($result && !$result->failed) {
			$data = array(
				'module' => $result->getField('related_type'),
				'id' => $result->getField('related_id'),
			);
		} else if ($googleRerurRule) {
			$data = [
				'module' => 'Recurrence',
				'id' => null,
			];
		} else if ($googleRerurId) {
			$lq = new ListQuery('google_sync');
			$lq->addSimpleFilter('google_id', $googleRerurId);
			$sync = $lq->runQuerySingle();
			if ($sync && !$sync->failed) {
				if (isset($origStartTime['dateTime']))
					$timestamp = strtotime($origStartTime['dateTime']);
				else
					$timestamp = strtotime($origStartTime['date'] . ' 00:00:00 GMT');
				$originalStart = gmdate('Y-m-d H:i:s', $timestamp);
				$lq = new ListQuery('Meeting');
				$lq->addSimpleFilter('recurrence_rule_id', $sync->getField('related_id'));
				$lq->addSimpleFilter('recurrence_start_time', $originalStart);
				$result = $lq->runQuerySingle();
				if ($result && !$result->failed) {
					$data = array(
						'module' => 'Meetings',
						'id' => $result->getField('id'),
					);

				}
			}
			
		}
		return $this->createIAHItem($data);
	}

	protected function canSyncToGoogle($iahItem)
	{
		if ($iahItem->getModelName() == 'Task'
			&& $iahItem->getField('date_start') == null
			&& $iahItem->getField('date_due') == null) {
				return false;
		}
		return true;
	}

	protected function isItemDeleted($item)
	{
		return $item->getStatus() == 'cancelled';
	}

	protected function saveItem($item, $iahItem) {
		$calid = $this->user->getPreference('google_calendar_id');
		$calid = $this->user->getPreference('google_calendar_id');
		if (empty($calid)) {
			$calid = 'primary';
		}
		if ($item->getId()) {
			$this->service->events->update($calid, $item->getId(), $item);
		} else {
			$this->service->events->insert($calid, $item);
		}
	}

	protected function getItems($query) {
		$id = $this->user->getPreference('google_calendar_id');
		if (empty($id)) {
			$id = 'primary';
		}
		return $this->service->events->listEvents($id, $query);
	}

	protected function mapLegacyGoogleId($googleId) {
		if (!preg_match('~https://~', $googleId))
			return $googleId;
		$parts = parse_url($googleId);
		$id = urldecode(basename($parts['path']));
		return $id;
	}

	protected function getGoogleIdClause($googleId) {
		$clause = array('multiple' => array(), 'operator' => 'or');
		$clause['multiple'][] = array(
			'field' => 'google_id',
			'value' => $googleId,
		);
		$clause['multiple'][] = array(
			'field' => 'google_id',
			'value' => 'http%://%/' . $googleId,
			'operator' => 'like',
			'match' => 'custom',
		);
		return $clause;
	}

	protected function setQueryUpdatedMin(&$query, $min) {
		if ($min)
			$query['updatedMin'] = $min;
	}
	protected function supportsSyncToken() {
		return true;
	}
}

/*
 *
 * FREQ=WEEKLY;WKST=MO;BYDAY=TU,TH
 *
 * FREQ=WEEKLY;WKST=MO;INTERVAL=3;BYDAY=TU,TH
 * FREQ=MONTHLY;WKST=MO;INTERVAL=4;BYDAY=4TU
 * FREQ=MONTHLY;WKST=MO;INTERVAL=4;BYMONTHDAY=23
 *
 */
