<?php

require_once 'include/config/format/ConfigParser.php';

function get_holidays_params()
{
	global $current_user, $current_language, $locale;
	global $HOLIDAYS;
	
	if (!empty($HOLIDAYS)) {
		return array($HOLIDAYS['country'], $HOLIDAYS['language'], $HOLIDAYS['variant']);
	}
	
	$country =  $current_user->getPreference('user_holidays');
	if (empty($country)) {
		$country = $locale->getPrecedentPreference('default_holidays');
	}
	$variant = null;
	if (strpos($country, '|') !== false) {
		list($country, $variant) = explode('|', $country);
	}

	$holidays_file = "modules/Calendar/holidays/$country.holidays.php";
	$default_h = AppConfig::setting('locale.defaults.holidays', 'usa');

	if (file_exists($holidays_file)) {
	} elseif (file_exists("modules/Calendar/holidays/{$default_h}.holidays.php")) {
		$country = $default_h;
	} else {
		return array('', '', '');
	}
	
	$holidays_file = "modules/Calendar/holidays/$country.holidays.php";
	$HOLIDAYS = ConfigParser::load_file($holidays_file);
	$HOLIDAYS['mtime'] = filemtime($holidays_file);
	
	$language_used = $current_language;
	$HOLIDAYS['variant'] = $variant;
	$HOLIDAYS['country'] = $country;
	$HOLIDAYS['language'] = $language_used;

	$holidays_file = "modules/Calendar/holidays/$country.holidays_calc.php";
	if (file_exists($holidays_file))
		require_once $holidays_file;
	
	return array($country, $language_used, $variant);
}

function get_holidays_mtime() {
	return isset($GLOBALS['HOLIDAYS']['mtime']) ? : false;
}

function load_holidays($reload=false)
{
	global $HOLIDAYS;
	if($reload) unset($HOLIDAYS);

	if(empty($HOLIDAYS) || ! isset($HOLIDAYS['lang'])) {
		list ($country, $language, $variant) = get_holidays_params();
	
		$base_language = AppConfig::setting('locale.base_language', 'en_us');
		$holiday_base = array_get_default($HOLIDAYS, 'language_default', $base_language);
	
		$lang_paths = array_unique(array(
			"modules/Calendar/holidays/$country.$holiday_base.php",
			"custom/include/language/$country.$holiday_base.php",
			"modules/Calendar/holidays/$country.$language.php",
			"custom/include/language/$country.$language.php",
		));

		foreach($lang_paths as $path)
			if (file_exists($path))
				$HOLIDAYS['mtime'] = max($HOLIDAYS['mtime'], filemtime($path));
		$HOLIDAYS['lang'] = ConfigParser::load_files($lang_paths);
	}
	
	if(AppConfig::language() == 'ja') {
		include_once("modules/Calendar/language/lunacalendar.php");
	}
}

function holiday($year, $month, $day)
{
	global $HOLIDAYS;
	list (,, $variant) = get_holidays_params();
	if (!function_exists('calculate_holiday')) {
		return '';
	}
	$str = calculate_holiday($year, $month, $day, $variant);
	if (isset($HOLIDAYS['lang'][$str])) {
		return $HOLIDAYS['lang'][$str];
	}
	return '';
}

function easter_relative($year, $month, $day, $diff) {
	static $easter = [
		1980 => [4, 6],
		1981 => [4, 19],
		1982 => [4, 11],
		1983 => [4, 3],
		1984 => [4, 22],
		1985 => [4, 7],
		1986 => [3, 30],
		1987 => [4, 19],
		1988 => [4, 3],
		1989 => [3, 26],
		1990 => [4, 15],
		1991 => [3, 31],
		1992 => [4, 19],
		1993 => [4, 11],
		1994 => [4, 3],
		1995 => [4, 16],
		1996 => [4, 7],
		1997 => [3, 30],
		1998 => [4, 12],
		1999 => [4, 4],
		2000 => [4, 23],
		2001 => [4, 15],
		2002 => [3, 31],
		2003 => [4, 20],
		2004 => [4, 11],
		2005 => [3, 27],
		2006 => [4, 16],
		2007 => [4, 8],
		2008 => [3, 23],
		2009 => [4, 12],
		2010 => [4, 4],
		2011 => [4, 24],
		2012 => [4, 8],
		2013 => [3, 31],
		2014 => [4, 20],
		2015 => [4, 5],
		2016 => [3, 27],
		2017 => [4, 16],
		2018 => [4, 1],
		2019 => [4, 21],
		2020 => [4, 12],
		2021 => [4, 4],
		2022 => [4, 17],
		2023 => [4, 9],
		2024 => [3, 31],
		2025 => [4, 20],
		2026 => [4, 5],
		2027 => [3, 28],
		2028 => [4, 16],
		2029 => [4, 1],
		2030 => [4, 21],
		2031 => [4, 13],
		2032 => [3, 28],
		2033 => [4, 17],
		2034 => [4, 9],
		2035 => [3, 25],
		2036 => [4, 13],
		2037 => [4, 5],
		2038 => [4, 25],
		2039 => [4, 10],
		2040 => [4, 1],
		2041 => [4, 21],
		2042 => [4, 6],
		2043 => [3, 29],
		2044 => [4, 17],
		2045 => [4, 9],
		2046 => [3, 25],
		2047 => [4, 14],
		2048 => [4, 5],
		2049 => [4, 18],
		2050 => [4, 10],
	];
	
	$easter_time = mktime(0,0,0, $month, $day - $diff, $year);
	$easter_year = date('Y', $easter_time);
	$easter_month = date('m', $easter_time);
	$easter_day = date('d', $easter_time);
	if (!isset($easter[$easter_year])) {
		return false;
	}
	return $easter[$easter_year][0] == $easter_month && $easter[$easter_year][1] == $easter_day;
}

