<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
********************************************************************************/
/*********************************************************************************

* Description:  Includes the functions for Customer module specific charts.
********************************************************************************/

require_once('modules/Campaigns/Campaign.php');
require_once('include/charts/Charts.php');
require_once('include/utils.php');


class CampaignCharts {
	var $default_width = 900;
	var $default_height = 450;
	var $strings_module = 'Campaigns';

	function svg_charts_enabled() {
		return AppConfig::setting('layout.svg_charts_enabled');
	}
	
	function get_cache_filename($type, $params=array()) {
		global $current_user;
		array_unshift($params, $current_user->getUserPrivGuid());
		$entropy = abs(crc32(implode(':', $params)));
		$cache_file = filename_safe_string("{$entropy}_{$type}") . ($this->svg_charts_enabled() ? '.svg' : '.xml');
		return AppConfig::setting('site.paths.xml_dir') . $cache_file;
	}

	/**
	* Creates opportunity pipeline image as a VERTICAL accumlated bar graph for multiple users.
	* param $datax- the month data to display in the x-axis
	* Portions created by SugarCRM are Copyright (C) SugarCRM, Inc..
	* All Rights Reserved..
	* Contributor(s): ______________________________________..
	*/	
	function campaign_response_by_activity_type($datay= array(), $targets=array(), $campaign_id = '', $marketing_id='', $refresh=false) {
		global $barChartColors, $app_list_strings, $db;
		$cache_file_name = $this->get_cache_filename('_campaign_response_by_activity_type_', array($campaign_id, $marketing_id));
		$svg = $this->svg_charts_enabled();

		if (!file_exists($cache_file_name) || $refresh == true) {
			$hits = array();
			$by_tg_type = array();

			$lq = new ListQuery('CampaignLog', null, array('no_default_order_by' => true));
			$lq->primary_added = true;
			$lq->setTableAlias('log');
			$lq->addField('activity_type');
			$lq->addField('target_type');
			$lq->addFieldLiteral('hits', 'count(*)');
			$lq->addSimpleFilter('campaign_id', $campaign_id);
			if (!empty($marketing_id))
				$lq->addSimpleFilter('marketing_id', $marketing_id);
			$lq->addSimpleFilter('archived', 0);
			$lq->setOrderBy('activity_type, target_type');
			$lq->setGroupBy('activity_type, target_type');
			foreach($lq->fetchAllRows() as $row) {
				$act_type = $row['activity_type'];
				$tg_type = $row['target_type'];
				$hits[$act_type][$tg_type] = $row['hits'];
				if(! isset($by_type[$tg_type]))
					$by_tg_type[$tg_type] = 0;
				$by_tg_type[$tg_type] += $row['hits'];
			}
			
			if($svg) {
				require_once 'include/SVGCharts/Bar.php';
				require_once('include/SVGCharts/impl/SVGChartData.php');
				$cdata = new SVGChartData;
				$chart = new SVGChartBar($this->default_width, $this->default_height);
				$chart->setTitle(translate('LBL_CAMPAIGN_RESPONSE_BY_RECIPIENT_ACTIVITY', $this->strings_module));
				$chart->setHorizontal();

				$series = array();
				foreach($targets as $k => $v)
					$series[$k] = array();
				
				foreach ($datay as $key=>$translation) {
					if ($key == '')
						continue;
					
					if(isset($hits[$key])) {
						foreach($hits[$key] as $rel_type => $type_hits) {
							$series[$rel_type]['points'][$key] = array(
								'n' => $type_hits,
								'c' => 1,
							);
						}
					}
				}
				
				foreach($series as $k => $v)
					$cdata->addSeries($k, $v);

				$cdata->setLabels($datay);
				$cdata->setSeriesLabels($targets);
				$chart->setData($cdata);
				
				$fileContents = $chart->render();
			} else {
				$fileContents = '     <yData defaultAltText="'.to_html(translate('LBL_ROLLOVER_VIEW', $this->strings_module)).'">'."\n";
				$totalArr = array();
				foreach ($datay as $key=>$translation) {
					if ($key == '')
						continue;
				
					$rowTotal = (int)array_sum(array_get_default($hits, $key, array()));
					$totalArr[] = $rowTotal;
					$fileContents .= '          <dataRow title="'.to_html($translation).'" endLabel="'.to_html(format_number($rowTotal, -1)).'">'."\n";
					if(isset($hits[$key])) {
						foreach($hits[$key] as $rel_type => $type_hits) {
							//create alternate text.
							$alttext = ' ';
							if(!empty($targeted)){
								$alttext=$targets[$outcome].': '.$mod_strings['LBL_TARGETED'].' '.$targeted[$outcome]. ', '.$mod_strings['LBL_TOTAL_TARGETED'].' '. $total_targeted. ".";
							}
							//if ($key != 'targeted'){
								$alttext.=" $translation: ". $type_hits;
							//}
							$fileContents .= '               <bar id="'.$rel_type.'" totalSize="'.$type_hits.'" altText="'.to_html($alttext).'" url="#'.$key.'"/>'."\n";
						}
					}
					$fileContents .= '          </dataRow>'."\n";
				}
				$fileContents .= '     </yData>'."\n";
				$max = get_max($totalArr);
				$fileContents .= '     <xData min="0" max="'.$max.'" length="10" prefix="'.''.'" suffix=""/>'."\n";
				$fileContents .= '     <colorLegend status="on">'."\n";

				$i=0;
				foreach ($targets as $tg_type=>$translation) {
					$color = generate_graphcolor($tg_type, $i);
					$fileContents .= '          <mapping id="'.$tg_type.'" name="'.to_html($translation).'" color="'.$color.'"/>'."\n";
					$i++;
				}
			
				$fileContents .= '     </colorLegend>'."\n";
				$fileContents .= '     <graphInfo>'."\n";
				$fileContents .= '          <![CDATA['.' '.']]>'."\n";
				$fileContents .= '     </graphInfo>'."\n";
				$fileContents .= '     <chartColors ';
				foreach ($barChartColors as $key => $value) {
					$fileContents .= ' '.$key.'='.'"'.$value.'" ';
				}
				$fileContents .= ' />'."\n";
				$fileContents .= '</graphData>'."\n";
			
				$title = '<graphData title="'.to_html(translate('LBL_CAMPAIGN_RESPONSE_BY_RECIPIENT_ACTIVITY', $this->strings_module)).'">'."\n";
				$fileContents = $title.$fileContents;
			}

			save_xml_file($cache_file_name, $fileContents);
		}
		
		return $this->display_chart('hBarF', $cache_file_name);
	}
	
	
	//campaign roi compputations
	function campaign_response_roi($datay=array(), $targets=array(), $campaign_id = '', $marketing_id='', $refresh=false) {
		global $barChartColors, $app_list_strings, $db;
		$cache_file_name = $this->get_cache_filename('_campaign_response_by_roi_', array($campaign_id, $marketing_id));
		$svg = $this->svg_charts_enabled();
		
		if (!file_exists($cache_file_name) || $refresh == true) {
			$keys = array(
				'Budget', 'Investment', 'Expected_Revenue', 'Weighted_Revenue', 'Revenue',
			);
			
			$campaign = ListQuery::quick_fetch_row('Campaign', $campaign_id, array(
				'Investment' => 'actual_cost_usd', 'Budget' => 'budget_usd', 'Expected_Revenue' => 'expected_revenue_usd',
			), array('acl_checks' => 'view'));
			if(! $campaign) return;
			
			$data = array();
			foreach($keys as $key) {
				if(isset($datay[$key]))
					$data[$key] = isset($campaign[$key]) ? $campaign[$key] : 0;
			}
			
			$lq = new ListQuery('Opportunity', null, array('no_default_order_by' => true));
			$lq->primary_added = true;
			$lq->setTableAlias('opp');
			$lq->addFieldLiteral('c', 'count(*)');
			$lq->addField('sales_stage');
			$lq->addFieldLiteral('amount', "SUM(opp.amount_usdollar)");
			$lq->addFieldLiteral('weighted', "SUM(opp.amount_usdollar*opp.probability/100)");
			$lq->addSimpleFilter('campaign_id', $campaign_id);
			$lq->setGroupBy('sales_stage');
			foreach($lq->fetchAllRows() as $row) {
				if($row['sales_stage'] == 'Closed Won') {
					if(isset($data['Revenue']))
						$data['Revenue'] = $row['amount'];
					if(isset($data['Weighted_Revenue']))
						$data['Weighted_Revenue'] += $row['amount'];
				} else {
					if(isset($data['Weighted_Revenue']))
						$data['Weighted_Revenue'] += $row['weighted'];
				}
			}
			
			if($svg) {
				require_once 'include/SVGCharts/Bar.php';
				require_once('include/SVGCharts/impl/SVGChartData.php');
				$cdata = new SVGChartData;
				$chart = new SVGChartBar($this->default_width, $this->default_height);
				$chart->setTitle(translate('LBL_CAMPAIGN_RETURN_ON_INVESTMENT', $this->strings_module));
				$chart->removeLegend();
				
				foreach ($keys as $key) {
					if(! $key || ! isset($datay[$key]))
						continue;
					$value = $data[$key];
					if(! isset($value)) $value = 0;
					if($value > 10) $value = round($value);
					$series = array(
						'points' => array(
							$key => array(
								'n' => $value,
								'c' => 1,
							),
						),
					);
					$cdata->addSeries($key, $series);
				}

				$cdata->setLabels($datay);
				$chart->setData($cdata);
				
				$fileContents = $chart->render();
			}
			else {
				$max = max(80, get_max($data));
				$maxy = 20;
				$idx = 0;
				$colors = array();
			
				$fileContents = '     <yData min="0" max="'.$maxy.'" length="20" defaultAltText="'.to_html(translate('LBL_ROLLOVER_VIEW', $this->strings_module)).'">'."\n";			
				foreach ($keys as $key) {
					if(! $key || ! isset($datay[$key]))
						continue;
					$value = $data[$key];
					$translation = $datay[$key];
					if(! isset($value)) $value = 0;
					if($value > 10) $value = round($value);
				
					$fileContents .= '          <dataRow title="'.to_html($translation).'" endLabel="'.to_html(format_number($value, -1)).'">'."\n";
					$fileContents .= '               <bar id="'.$key.'" totalSize="'.$value.'" altText="" url="#'.$key.'"/>'."\n";
					$fileContents .= '          </dataRow>'."\n";
				
					$colors[$key] = array($translation, generate_graphcolor($key, $idx ++));
				}
			
				$fileContents .= '     </yData>'."\n";
			
				$fileContents .= '     <xData min="0" max="'.$max.'" length="12" prefix="'.''.'" suffix=""/>'."\n";
					
				$fileContents .= '     <colorLegend status="off">'."\n";

				foreach ($colors as $key => $c)
					$fileContents .= '          <mapping id="'.$key.'" name="'.to_html($c[0]).'" color="'.$c[1].'"/>'."\n";
			
				$fileContents .= '     </colorLegend>'."\n";
			
				$fileContents .= '     <graphInfo>'."\n";
				$fileContents .= '          <![CDATA['.' '.']]>'."\n";
				$fileContents .= '     </graphInfo>'."\n";
				$fileContents .= '     <chartColors ';
			
				foreach ($barChartColors as $key => $value) {
					$fileContents .= ' '.$key.'='.'"'.$value.'" ';
				}
			
				$fileContents .= ' />'."\n";
				$fileContents .= '</graphData>'."\n";
			
				$title = '<graphData title="'.to_html(translate('LBL_CAMPAIGN_RETURN_ON_INVESTMENT', $this->strings_module)).'">'."\n";
				$fileContents = $title.$fileContents;
			}

			save_xml_file($cache_file_name, $fileContents);
		}
		
		return $this->display_chart('vBarF', $cache_file_name);
	}
	
	function display_chart($chartName, $xmlFile, $width=null, $height=null) {
		if(! file_exists($xmlFile) || ! $chartName)
			return '';
		if(! $width) $width = $this->default_width;
		if(! $height) $height = $this->default_height;
		$svg = preg_match('/\.svg$/', $xmlFile);
		if (!$svg) {
			$chartPath = "include/charts/$chartName.swf";
			$chartUrl = $chartPath . '?filename=' . htmlentities($xmlFile, ENT_QUOTES, 'UTF-8');
			$html = <<<EOH
				<object classid="clsid:D27CDB6E-AE6D-11cf-96B8-444553540000"
					codebase="https://download.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=6,0,0,0"
					width="$width" height="$height" id="$chartName" align="">
					<param name="movie" value="$chartUrl">
					<param name="bgcolor" value="#FFFFFF">
					<param name="wmode" value="transparent">
					<param name="quality" value="high">
					<embed src="$chartUrl" wmode="transparent" quality="high" bgcolor="#FFFFFF"
						width="$width" height="$height" name="$chartName" align=""
						type="application/x-shockwave-flash" pluginspage="https://www.macromedia.com/go/getflashplayer">
				</object>
EOH;
		} else {
			$tm = substr(md5(@filemtime($xmlFile).'0'), 0, 5);
			$height = '';
			$f = @fopen($xmlFile, 'r');
			if ($f) {
				$str = fread($f, 1024);
				fclose($f);
				$m = array();
				if (preg_match('~<svg[^>]*?width="(\d+)"[^>]*?height="(\d+)"[^>]*>~', $str, $m)) {
					$width = $m[1];
					$height = 'height="' . $m[2] . '"';
				}
			}
			$html =  '<object data="' . $xmlFile.'?'.$tm . '" type="image/svg+xml" width="'.$width .'" ' . $height . ' title="" wmode="transparent"></object>';
		}
		return $html;
	}

}// end charts class
?>
