<?php

class WebToLeadForm {

	/**
	 * Validate Web To Lead form (form_token and CAPTCHA)
	 *
	 * @param string $form_token
	 * @param array $input
	 *
	 * @return array
	 */
	public function validate($form_token, $input) {
		$query = new ListQuery('web_to_lead_forms');
		$query->addSimpleFilter('token', $form_token);
		$result = $query->runQuerySingle();

		$success = true;
		$error = '';

		if (! $result->failed) {
			if ($result->getField('with_captcha')) {
				$captcha_token = $input['token'] ?? '';
				$action = $input['c_action'] ?? '';

				return $this->validateCaptcha($captcha_token, $action);
			}
		} else {
			$success = false;
			$error = translate('LBL_WEB_TO_LEAD_FORM_TOKEN_ERROR', 'Leads');
		}

		return ['success' => $success, 'error' => $error];
	}

	/**
	 * Validate form CAPTCHA
	 *
	 * @param string $token
	 * @param string $action
	 *
	 * @return array
	 */
	private function validateCaptcha($token, $action) {
		require_once('modules/Campaigns/FormCaptcha.php');

		$captcha = new FormCaptcha();
		$success = true;
		$error = '';

		if (! $captcha->isValid($token, $action)) {
			$success = false;
			$error = translate('LBL_WEB_TO_LEAD_FORM_CAPTCHA_ERROR', 'Leads');
		}

		return ['success' => $success, 'error' => $error];
	}

	/**
	 * Add new Web To Lead form
	 *
	 * @param bool $with_captcha
	 *
	 * @return null|string - form token if added or null otherwise
	 */
	public function add($with_captcha = false) {
		$upd = RowUpdate::blank_for_model('web_to_lead_forms');
		$token = $this->generateToken();

		$upd->set([
			'token' => $token,
			'with_captcha' => (int)$with_captcha
		]);

		if ($upd->save()) {
			return $token;
		} else {
			return null;
		}
	}

	/**
	 * Generate new from token
	 *
	 * @return string
	 */
	private function generateToken() {
		$bytes = openssl_random_pseudo_bytes(20);
		return bin2hex($bytes);
	}
}