<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

class WordpressFormGen {

	private $columns;
	private $extra;
	private $model;

	public function __construct($columns, $extra) {
		$this->columns = $columns;
		$this->extra = $extra;
		$this->model = new ModelDef('Lead');
	}

	public function generateCode() {
		$ret = array(
			'fields' => array(),
		);
		$first_col = array_get_default($this->columns, 0, array());
		$second_col = array_get_default($this->columns, 1, array());
		$row_count = max(count($first_col), count($second_col));

		$ret['form'] = '';
		
		if (isset($this->extra['web_header']))
			$ret['form'] .= '<h2>' . to_html($this->extra['web_header']) . "</h2>\n" ;
		if (isset($this->extra['web_description']))
			$ret['form'] .= '<p>' . to_html($this->extra['web_description']) . "</p>\n" ;

		$ret['form'] .= "<table>\n";
		for ($i = 0; $i < $row_count; $i++) {
			$field_first = array_get_default($first_col, $i, '');
			$field_second = array_get_default($second_col, $i, '');
			$ret['form'] .= "  <tr>\n";
			$this->renderField($field_first, $ret);
			$this->renderField($field_second, $ret);
			$ret['form'] .= "  </tr>\n";
		}

		$submit = array('name' => 'form_submit', 'type' => 'submit');
		if (isset($this->extra['web_submit']))
			$submit['label'] = $this->extra['web_submit'];
		$ret['fields'][] = $submit;

		if (isset($this->extra['web_footer']))
			$ret['form'] .= "<tr><td colspan=\"2\">" . to_html($this->extra['web_footer']) . "</td></tr>\n";
		$ret['form'] .= "<tr><td colspan=\"2\" class=\"ocrmlf-submit\">{form_submit}</td></tr>\n";

		if (isset($this->extra['captcha_script']))
			$ret['form'] .= "<tr><td colspan=\"2\" style='display: none;'>".$this->extra['captcha_script']."</td></tr>\n";

		$ret['form'] .= "</table>\n";

        $ret['url'] = AppConfig::site_url() . 'WebToLeadHandler.php';
		if (isset($this->extra['redirect_url']) && $this->extra['redirect_url'] != 'http://')
			$ret['redirect'] = $this->extra['redirect_url'];

		if (isset($this->extra['form_token'])) {
			$ret['fields'][] = $this->addHiddenField('form_token', $this->extra['form_token']);
			$ret['form'] .= "{form_token}\n";
		}

		if (isset($this->extra['token'])) {
			$ret['fields'][] = $this->addHiddenField('token', $this->extra['token']);
			$ret['form'] .= "{token}\n";
		}

		if (isset($this->extra['c_action'])) {
			$ret['fields'][] = $this->addHiddenField('c_action', $this->extra['c_action']);
			$ret['form'] .= "{c_action}\n";
		}

		if (isset($this->extra['assigned_user_id'])) {
			$ret['fields'][] = $this->addHiddenField('assigned_user_id', $this->extra['assigned_user_id']);
			$ret['form'] .= "{assigned_user_id}\n";
		}
		if (isset($this->extra['campaign_id'])) {
			$ret['fields'][] = $this->addHiddenField('campaign_id', $this->extra['campaign_id']);
			$ret['form'] .= "{campaign_id}\n";
		}
		if (!empty($this->extra['subscriptions'])) {
			foreach ($this->extra['subscriptions'] as $sid) {
				$ret['form'] .= '<input type="hidden" name="subscriptions[]" value="' . $sid . '">';
			}
		}
		
		$ret['fields'][] = $this->addHiddenField('wtl_json', '1');
		$ret['form'] .= "{wtl_json}\n";

		return $ret;
	}

	private function renderField($name, &$ret) {
		if (    $f = $this->addField($name, $ret)     ) {
			$extra = array_get_default($f, 'extra');
			$fdef = $this->model->getFieldDefinition($name);
			$ret['form'] .= '    <td class="ocrmlf-field-label">';
			$ret['form'] .= to_html(translate(array_get_default($fdef,'vname'), array_get_default($fdef, 'vname_module', 'Leads')));
			$ret['form'] .= "</td>\n";
			$ret['form'] .= '    <td>{' . $name . "}" . $extra ."</td>\n";
		}
	}

	private function addField($name, &$ret) {
		foreach ($ret['fields'] as $f) {
			if ($f['name'] == $name)
				return true;
		}
		$fdef = $this->model->getFieldDefinition($name);
		if (!$fdef)
			return false;
		$type = array_get_default($fdef, 'custom_type', array_get_default($fdef, 'type'));
		switch ($type) {
			case 'enum':
			case 'multienum':
				$f = $this->addEnumField($fdef, $type);
				break;
			case 'bool':
				$f = $this->addCheckboxField($fdef, $type);
				break;
			case 'text':
				$f = $this->addTextareaField($fdef, $type);
				break;
			case 'date':
				$f = $this->addDateField($fdef, $type);
				break;
			case 'varchar':
			case 'name':
			case 'phone':
			case 'email':
			case 'url':
			case 'int':
			case 'double':
			case 'currency':
			case 'base_currency':
				$f = $this->addTextField($fdef, $type);
				break;
			default:
				return false;
		}
		$ret['fields'][] = $f;
		return $f;
	}

	private function addEnumField($fdef, $type) {
		global $app_list_strings;
		$f = array(
			'name' => $fdef['name'],
			'type' => 'select',
			'multiple' => $type == 'multienum',
			'required' => !empty($fdef['required']),
		);
		$f['choices'] = '';
		foreach ($app_list_strings[$fdef['options']] as $k => $v)
			$f['choices'] .= $k . '|' . $v . "\n";
		return $f;
	}

	private function addCheckboxField($fdef, $type) {
		$f = array(
			'name' => $fdef['name'],
			'type' => 'checkbox',
		);
		return $f;
	}

	private function addTextareaField($fdef, $type) {
		$f = array(
			'name' => $fdef['name'],
			'type' => 'textarea',
			'cols' => 25,
			'rows' => 5,
			'required' => !empty($fdef['required']),
		);
		return $f;
	}
	
	private function addDateField($fdef, $type) {
		$f = array(
			'name' => $fdef['name'],
			'type' => 'date',
			'required' => !empty($fdef['required']),
			'extra' => '<br><span style="font-size:85%">' . translate('LBL_WEBLEAD_DATE_FORMAT', 'app') . '</span>',
		);
		return $f;
	}
	
	private function addTextField($fdef, $type) {
		static $types_map = array(
			'int' => 'spinbox',
			'email' => 'email',
			'url' => 'url',
			'phone' => 'phone',
		);
		$f = array(
			'name' => $fdef['name'],
			'type' => array_get_default($types_map, $type, 'text'),
			'required' => !empty($fdef['required']),
		);
		return $f;
	}

	private function addHiddenField($name, $value) {
		$f = array(
			'name' => $name,
			'id' => $name,
			'type' => 'hidden',
			'value' => $value,
		);
		return $f;
	}
}
