<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormTableSection.php');
require_once('modules/Campaigns/Charts.php');

class CampaignWidget extends FormTableSection {

	/**
	 * Used to build chart
	 *
	 * @var string
	 */
	var $latest_marketing_id = 'all';

	function init($params=null, $model=null) {
		parent::init($params, $model);
	}

	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if($gen->getLayout()->getEditingLayout()) {
			$params = array();
			return $this->renderOuterTable($gen, $parents, $context, $params);		
		}

		$html = '';

		if (!$gen->getLayout()->getDefaultEditable()) {
			if ($this->id == 'roi_chart') {
				$html = $this->renderRoiChart($row_result);
			} elseif ($this->id == 'track_chart') {
				$html = $this->renderTrackChart($gen, $row_result);
			} elseif ($this->id == 'diagnostic') {
				$html = $this->renderDiagnosticView();
			} elseif ($this->id == 'subscriptions') {
				$html = $this->renderSubscriptionsView($gen, $row_result);
			}
		}

		if (!$html)
			return '';
		$params = array(
			'body' => $html,
		);
		$html = $this->renderOuterTable($gen, $parents, $context, $params);
		return $html;
	}
	
	function getRequiredFields() {
		$fields = array();

		return $fields;
	}
	
	/**
	 * Render Track chart
	 *
	 * @param HtmlFormGenerator $gen
	 * @param RowResult $row_result
	 * @return string
	 */
	function renderTrackChart(HtmlFormGenerator $gen, RowResult $row_result) {
		global $mod_strings, $app_list_strings, $current_user;
		
		if($row_result->base_model == 'EmailMarketing') {
			$record = $row_result->getField('campaign_id');
			$this->latest_marketing_id = $row_result->getField('id');
		} else
			$record = $row_result->getField('id');
		$type = $row_result->getField('campaign_type');

		$filter_label = $filter_select = $filter = '';

		if ($type == 'NewsLetter') {
			$filter_label = $mod_strings['LBL_FILTER_CHART_BY'] . ':';
			$filter_select = $this->renderFilterSelect($gen, $_REQUEST, $record);
		}
		
		if($filter_label) {
			$filter = <<<EOQ
				<div class="row form-row">
				<div class="column form-cell">
				<div class="form-value text-center">
				<span class="form-label">{$filter_label}</span> {$filter_select}
				</div></div></div>
EOQ;
		}

		$chart = $this->getChart($record, 'track');

		$body = <<<EOQ
			<div class="row form-row">
			<div class="column form-cell">
			<div class="form-value text-center">
				{$filter}
				{$chart}
			</div></div></div>
EOQ;

		return $body;
	}

	/**
	 * Render ROI chart
	 *
	 * @param RowResult $row_result
	 * @return string
	 */
	function renderRoiChart(RowResult $row_result) {
		$record = $row_result->getField('id');
		$chart = $this->getChart($record, 'roi');

		$body = <<<EOQ
			<div class="row form-row">
			<div class="column form-cell">
			<div class="form-value text-center">
			{$chart}
			</div></div></div>
EOQ;

		return $body;
	}

	/**
	 * Get chart by type
	 *
	 * @param string $record
	 * @param string $type: track or roi
	 * @return string
	 */
	function getChart($record, $type = 'track') {
		global $app_list_strings, $current_user;

		$chart = new CampaignCharts();

		$marketing_id = $this->latest_marketing_id;
		if ($marketing_id == 'all')
			$marketing_id = '';

		if ($type == 'track') {
			$types_k = array('Contacts', 'Leads', 'Prospects', 'Users');
			$types = array();
			foreach($types_k as $t)
				$types[$t] = array_get_default($app_list_strings['moduleList'], $t, $t);
			$chart_image = $chart->campaign_response_by_activity_type($app_list_strings['campainglog_activity_type_dom'], $types, $record, $marketing_id, true);

		} elseif ($type == 'roi') {
			$chart_image = $chart->campaign_response_roi($app_list_strings['roi_type_dom'], $app_list_strings['roi_type_dom'], $record, $marketing_id, true);
		} else {
			$chart_image = '&nbsp;';
		}

		return $chart_image;
	}

	/**
	 * Render Diagnostic panels
	 *
	 * @return string
	 */
	function renderDiagnosticView() {
		$email_components = $this->getEmailComponents();
		$schedule_components = $this->getScheduleComponents();

		$body = <<<EOQ
<div id="diagnose" class="contentdiv">
	<div class="row form-row">
		<div class="column sm-12 form-cell">
			<h4>{$email_components['led']} Email Components</h4>
		</div>
	</div>
			<div id="email" >
			{$email_components['conf_msg']}
			{$email_components['mbox_table']}
			</div>
			<div class="row form-row">
				<div class="column form-cell sm-12">
					{$email_components['setup_link']}
				</div>
			</div>


		</div>

		<div class="form-section">
			<div class="row form-row">
				<div class="column sm-12 form-cell">
					<h4>{$schedule_components['led']}  Scheduler Components</h4>
				</div>
			</div>

			<div id="schedule">
			{$schedule_components['emails_message']}
			</div>

			<div class="row form-row" style="padding-top: 8px">
				<div class="column sm-12 form-cell">
					<button type="submit" class="input-button group-left" onclick="return SUGAR.ui.sendForm(this);"><span class="uii uii-refresh"></span> <span class="input-label" id="DetailForm_-label">Re-Check</span></button>
				</div>
			</div>

			</div>
EOQ;

		return $body;
	}

	/**
	 * Get Email Components diagnostic block
	 *
	 * @return array: ['image', 'setup_link', 'mbox_table', 'conf_msg']
	 */
	function getEmailComponents() {
		global $mod_strings, $current_user;

		$email_health = 0;

		$lq = new ListQuery('EmailPOP3');

		$clauses = array(
			"type" => array(
				"value" => 'bounce',
				"field" => "mailbox_type",
			),
			"active" => array(
				"value" => 1,
				"field" => "active"
			),
		);

		$lq->addFilterClauses($clauses);
		$result = $lq->fetchAll();

		$mbox_table = '';

		if (! $result->failed) {
			$boxes = $result->rows;

			if (sizeof($boxes) > 0) {

				$mbox_table .= '<div class="row form-row"><div class="column form-cell sm-12"><h4> ' . $mod_strings['LBL_MAILBOX_CHECK1_GOOD'] . ' </h4></div></div>';
				$mbox_table .= '<div class="row form-row">';

				$mbox_table .= '<div class="column form-cell sm-3"><span class="form-label">' . $mod_strings['LBL_MAILBOX_NAME'] . ' </span></div>';
				$mbox_table .= '<div class="column form-cell sm-3"><span class="form-label">' . $mod_strings['LBL_LOGIN'] . ' </span></div>';
				$mbox_table .= '<div class="column form-cell sm-3"><span class="form-label">' . $mod_strings['LBL_MAILBOX'] . ' </span></div>';
				$mbox_table .= '<div class="column form-cell sm-3"><span class="form-label">' . $mod_strings['LBL_SERVER_URL'] . ' </span></div>';
				$mbox_table .= '</div>';

				foreach ($boxes as $id => $details) {
					$mbox_table .= '<div class="row form-row">';

					$mbox_table .= '<div class="column form-cell sm-3">' . $details['name'] . '</div>';
					$mbox_table .= '<div class="column form-cell sm-3">' . $details['username'] . '</div>';
					$mbox_table .= '<div class="column form-cell sm-3">' . $details['imap_folder'] . '</div>';
					$mbox_table .= '<div class="column form-cell sm-3">' . $details['host'] . '</div>';
					$mbox_table .= '</div>';
				}
			} else {
				$mbox_table .= '<div class="row form-row"><div class="column form-cell sm-12"><h4 class="error"> ' . $mod_strings['LBL_MAILBOX_CHECK1_BAD'] . ' </h4></div></div>';
				$email_health ++;
			}
		}


		//email settings configured
		$conf_msg = '';

		if ( strstr(AppConfig::setting('notify.from_address'), 'example.com') ){
			//if from address is the default, then set "bad" message and increment health counter
			$conf_msg .= '<div class="row form-row"><div class="column form-cell sm-12"><h4 class="error"> ' . $mod_strings['LBL_MAILBOX_CHECK2_BAD'] . ' </h4></div></div>';
			$email_health ++;
		} else {
			$conf_msg .= '<div class="row form-row"><div class="column form-cell sm-12"><h4> ' . $mod_strings['LBL_MAILBOX_CHECK2_GOOD'] . ' </h4></div></div>';
			$conf_msg .= '<div class="row form-row">';

			$conf_msg .= '<div class="column form-cell sm-2"><span class="form-label">' . $mod_strings['LBL_WIZ_FROM_NAME'] . ' </span></div>';
			$conf_msg .= '<div class="column form-cell sm-2"><span class="form-label">' . $mod_strings['LBL_WIZ_FROM_ADDRESS'] . ' </span></div>';
			$conf_msg .= '<div class="column form-cell sm-2"><span class="form-label">' . $mod_strings['LBL_MAIL_SENDTYPE'] . ' </span></div>';

			if(AppConfig::setting('email.send_type') == 'SMTP'){
				$conf_msg .= '<div class="column form-cell sm-2"><span class="form-label">' . $mod_strings['LBL_MAIL_SMTPSERVER'] . ' </span></div>';
				$conf_msg .= '<div class="column form-cell sm-2"><span class="form-label">' . $mod_strings['LBL_MAIL_SMTPUSER'] . ' </span></div>';
			}
			$conf_msg .= "</div>";

			$conf_msg .= '<div class="row form-row">';
			$conf_msg .= '<div class="column form-cell sm-2">' . AppConfig::setting('notify.from_name') . '</div>';
			$conf_msg .= '<div class="column form-cell sm-2">' . AppConfig::setting('notify.from_address') . '</div>';
			$conf_msg .= '<div class="column form-cell sm-2">' . AppConfig::setting('email.send_type') . '</div>';

			if(AppConfig::setting('email.send_type') == 'SMTP'){
				$conf_msg .= '<div class="column form-cell sm-2">' . AppConfig::setting('email.smtp_server') . '</div>';
				$conf_msg .= '<div class="column form-cell sm-2">' . AppConfig::setting('email.smtp_user') . '</div>';
			}
			$conf_msg .= "</div>";

		}

		$email_setup_wiz_link = '';

		if ($email_health > 0) {

			if (is_admin($current_user)) {
				$email_setup_wiz_link = "<a href='index.php?module=Campaigns&action=WizardEmailSetup'>".$mod_strings['LBL_EMAIL_SETUP_WIZ']."</a>";
			} else {
				$email_setup_wiz_link = $mod_strings['LBL_NON_ADMIN_ERROR_MSG'];
			}

		}

		$emailLed = $this->defineDiagnosticLED($email_health, 2);

		return array('led' => $emailLed, 'setup_link' => $email_setup_wiz_link, 'mbox_table' => $mbox_table, 'conf_msg' => $conf_msg);
	}

	/**
	 * Get Schedule Components diagnostic block
	 *
	 * @return array: ['image', 'emails_message']
	 */
	function getScheduleComponents() {
		require_once('modules/Scheduler/Schedule.php');
		global $mod_strings, $current_user;

		$check_types = [
			'emailmandelivery',
			'emaildripfeed',
			'processbounces',
			'process_workflow',
		];

		$lq = new ListQuery('Schedule', array('type', 'enabled'));
		$lq->addSimpleFilter('type', $check_types);
		$lq->addSimpleFilter('hidden', 0);
		$result = $lq->fetchAll();

		$sched_health = 0;
		$sched_mes = '';
		$sched_mes_body = '';
		$found = [];

		if (! $result->failed) {
			foreach ($result->rows as $id => $details) {
				$found[$details['type']] = true;
			}
		}

		$sched_health = count($check_types) - count($found);

		//determine which table header to use, based on whether or not schedulers were found
		$sched_mes = '<div class="row form-row">';
		$sched_mes .= '<div class="column form-cell sm-12">';
		if($sched_health == 0)
			$sched_mes .= '<h5>' . $mod_strings['LBL_SCHEDULER_CHECK_GOOD'] .'</h5>';
		else
			$sched_mes .= '<h5>' . $mod_strings['LBL_SCHEDULER_CHECK_BAD'] .'</h5>';
		$sched_mes .= '</div></div>';

		$sched_mes = '<div class="row form-row">';
		$sched_mes .= '<div class="column form-cell sm-4"><span class="form-label">' . $mod_strings['LBL_SCHEDULER_NAME'] . '</span></div>';
		$sched_mes .= '<div class="column form-cell sm-4"><span class="form-label">' . $mod_strings['LBL_SCHEDULER_STATUS'] . '</span></div>';
		$sched_mes .= '</div>';

		foreach($check_types as $type) {
			$ok = ! empty($found[$type]);
			$status = $mod_strings[$ok ? 'LBL_ACTIVE' : 'LBL_INACTIVE'];
			$sched_mes .= '<div class="row form-row">';
			$sched_mes .= '<div class="column form-cell sm-4">' . Schedule::translate_type($type) . '</div>';
			$sched_mes .= '<div class="column form-cell sm-4">' . $status . '</div>';
			$sched_mes .= '</div>';
		}

		$admin_sched_link = '';

		if (! $sched_health) {

			if (is_admin($current_user)) {
				$admin_sched_link = "<a href='index.php?module=Scheduler&action=index'>".$mod_strings['LBL_SCHEDULER_LINK']."</a>";
			} else {
			 $admin_sched_link = $mod_strings['LBL_NON_ADMIN_ERROR_MSG'];
			}

		}

		$final_sched_msg = $sched_mes;

		if($admin_sched_link) {
			$final_sched_msg .= '<div class="row form-row">';
			$final_sched_msg .= '<div class="column form-cell">';
			$final_sched_msg .= $admin_sched_link;
			$final_sched_msg .= '</div></div>';
		}

		$scheduleLED = $this->defineDiagnosticLED($sched_health, count($check_types));

		return array('led' => $scheduleLED, 'emails_message' => $final_sched_msg);
	}

	/**
	 * Render Manage Subscriptions page
	 *
	 * @param HtmlFormGenerator $gen
	 * @param RowResult $row_result
	 * @return string
	 */
	function renderSubscriptionsView(HtmlFormGenerator $gen, RowResult $row_result) {
		require_bean($this->model->name);
		global $app_strings;
		$input = $_REQUEST;
		$classname = "SUGAR_GRID";
		$record = $row_result->getField('id');
		$focus = new $this->model->name();

		//if subsaction has been set, then process subscriptions
		if(isset($input['subs_action'])) {
			$this->manageSubscriptions($focus, $record, $input);

			if(isset($input['return_module']) && isset($input['return_record']) && isset($input['return_action'])) {
				header("Location: async.php?module=".$input['return_module']."&action=".$input['return_action']."&record=".$input['return_record']);
			}
		}

		$inputs = array();
		$subscriptions_list = $this->constructDDSubscriptionList($gen, $focus, $record, $classname, $inputs);
		extract($inputs);

		$body = <<<EOQ
			<input type="hidden" name="subs_action" value="process" />
			<input type="hidden" name="subscribe_lists" id="ddgrid0_values" value="{$subs_value}" />
			<input type="hidden" name="remove_lists" id="ddgrid1_values" value="{$unsubs_value}" />									        
			<table width="100%" border="0" cellspacing="0" cellpadding="0" class="tabForm">
				<tr><td>
					<table  border="0" cellspacing="0" cellpadding="0">
					<tr>
					<th align="left" class="dataLabel" colspan="4"><h4 class="">{$app_strings['LBL_MANAGE_SUBSCRIPTIONS_FOR']}{$row_result->getField('name')}</h4></th>
					</tr>
					<tr><td>&nbsp;</td></tr>
					<td>
					<table align="center" border="0" cellpadding="0" cellspacing="0" width='350'>
					<tbody><tr><td align="center">{$subscriptions_list}</td></tr></tbody>
					</table>
					</td>
					</tr>
					</table>
				</td></tr>
				<tr><td>&nbsp;</td></tr>
				<tr><td>
					<button type='submit' class='input-button input-outer' onclick='return SUGAR.ui.sendForm(document.forms.DetailForm, {"record_perform":"save"}, null);'><div class="input-icon icon-accept left"></div><span class="input-label">{$app_strings['LBL_SAVE_BUTTON_LABEL']}</span></button>
					<button title="{$app_strings['LBL_CANCEL_BUTTON_TITLE']}" accessKey="{$app_strings['LBL_CANCEL_BUTTON_KEY']}" class="input-button input-outer" onclick="return SUGAR.ui.cancelEdit(document.forms.DetailForm);" type="button" name="button"><div class="input-icon icon-cancel left"></div><span class="input-label">{$app_strings['LBL_CANCEL_BUTTON_LABEL']}</span></button>
				</td></tr>
			</table>
EOQ;

		return $body;
	}

	/**
	 * This method constructs Drag and Drop multiselect box of subscriptions
	 * for display in manage subscription form
	 *
	 * @param HtmlFormGenerator $gen
	 * @param SugarBean $focus
	 * @param string $record
	 * @param string $classname
	 * @return string
	 */
	function constructDDSubscriptionList(HtmlFormGenerator $gen, $focus, $record, $classname, &$input_values) {
		$str = '';

		if ($focus->retrieve($record)) {
			require_once('modules/Campaigns/utils.php');
			require_once("include/templates/TemplateDragDropChooser.php");
			global $mod_strings;

			// Lets start by creating the subscription and unsubscription arrays
			$subscription_arrays = CampaignUtils::get_subscription_lists($focus);
			$unsubs_arr = $subscription_arrays['unsubscribed'];
			$subs_arr =  $subscription_arrays['subscribed'];
			$exempt =  $subscription_arrays['exempt'];
			$unsubs_arr_rev = array();
			$subs_arr_rev = array();
			$exempt_rev = array();

			$subs_value="";
			$unsubs_value="";
			$exempt_value="";


			foreach ($unsubs_arr as $key=>$val) {
				$unsubs_arr_rev[] = array($key, $val);
				if (!empty($unsubs_value))
					$unsubs_value .= ", ";
				$unsubs_value .= $key;
			}

			foreach ($subs_arr as $key=>$val) {
				$subs_arr_rev[] = array($key, $val);
				if (!empty($subs_value))
					$subs_value .= ", ";
				$subs_value .= $key;
			}
			foreach ($exempt as $key=>$val) {
				$exempt_rev[] = array($key, '<a href="index.php?module=Campaigns&action=DetailView&record='.$key.'">'.htmlentities($val, ENT_QUOTES, 'UTF-8').'</a>');
				if (!empty($exempt_value))
					$exempt_value .= ", ";
				$exempt_value .= $key;
			}

			$input_values = compact('subs_value', 'unsubs_value', 'exempt_value');

			//now call function that creates javascript for invoking DDChooser object
			$dd_chooser = new TemplateDragDropChooser();
			$dd_chooser->args['classname']  = $classname;
			$dd_chooser->args['columns'] = array(
				array(
					'title' => $mod_strings['LBL_ALREADY_SUBSCRIBED_HEADER'],
					'data' => $subs_arr_rev,
					'name' => 'ddgrid0',
				),
				array(
					'title' => $mod_strings['LBL_UNSUBSCRIBED_HEADER'],
					'data' => $unsubs_arr_rev,
					'name' => 'ddgrid1',
				),
				array(
					'title' => $mod_strings['LBL_OPTED_OUT_HEADER'],
					'data' => $exempt_rev,
					'name' => 'optout',
					'editable' => false,
					'no_escape_labels' => true,
				),
			);
			$dd_chooser->args['title']        =  ' ';

			$str .= $dd_chooser->displayDefinitionScript();
			$str .= $dd_chooser->display();
		}

		return $str;
	}

	/**
	 * Perform Subscription management work.
	 * This method processes selected subscriptions and calls the
	 * right methods to subscribe or unsubscribe the user
	 *
	 * @param SugarBean $focus
	 * @param string $record
	 * @param array $input - user input ($_REQUEST)
	 * @return void
	 */
	function manageSubscriptions($focus, $record, $input){
		if ($focus->retrieve($record)) {
			require_once('modules/Campaigns/utils.php');            
			$all_lists = CampaignUtils::get_subscription_lists($focus);

			$sel_subs = array();
			if(! empty($input['subscribe_lists'])) {
				$sel_subs = array_map('trim', explode(",", $input['subscribe_lists']));
				$sel_subs = array_filter($sel_subs);
			}

			$cur_subs = array_keys($all_lists['subscribed']);
			$add_subs = array_diff($sel_subs, $cur_subs);
			$remove_subs = array_diff($cur_subs, $sel_subs);

			foreach($add_subs as $list_id)
				CampaignUtils::subscribe_to_list($focus, $list_id);

			foreach($remove_subs as $list_id)
				CampaignUtils::remove_from_list($focus, $list_id);
		}            
	}

	/**
	 * Render filter dropdown for NewsLetters
	 *
	 * @param HtmlFormGenerator $gen
	 * @param array $input - $_REQUEST
	 * @param string $record
	 * @return null|string
	 */
	function renderFilterSelect(HtmlFormGenerator $gen, $input, $record) {
		$selected_marketing_id = null;
		
		if (!empty($input['mkt_id'])) {
			$selected_marketing_id = $input['mkt_id'];
			$this->latest_marketing_id = $selected_marketing_id;
		}

		$options = array();

		$lq = new ListQuery('EmailMarketing', array('id', 'name', 'date_modified'));
		$lq->addSimpleFilter('campaign_id', $record);
		$lq->setOrderBy('date_modified desc');

		$result = $lq->fetchAll();

		if (! $result->failed) {

			foreach ($result->rows as $id => $details) {
				$options[$id] = $details['name'];
			}
		}

		$spec = array('name' => 'mkt_id', 'options' => $options, 'onchange' => 'return SUGAR.ui.sendForm(document.forms.DetailForm);');
		$filter_select = $gen->form_obj->renderSelect($spec, $selected_marketing_id);

		return $filter_select;
	}

	/**
	 * Determines the appropriate image source
	 *
	 * @param int $num - "health" parameter being tracked whenever there is something wrong (higher number = bad)
	 * @param int $total - the total number things being checked
	 * @return string
	 */
	function defineDiagnosticLED($num, $total) {
		//if health number is equal to total number then all checks failed, set red image
		if ($num == $total) {
			$color = 'red';
		} elseif ($num == 0) {
			$color = 'green';
		} else {
			$color = 'yellow';
		}

		return '<span class="uii uii-led led-' . $color . '"></span>';
	}
}

?>
