<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormTableSection.php');
require_once('modules/Campaigns/utils.php');
require_once('modules/Administration/Administration.php');

class EmailSetupWidget extends FormTableSection {

    const PREFERENCES_CATEGORY = 'global';

    const CAMPAIGN_EMAILS_PER_RUN = 500;

    /**
     * @var Administration
     */
    var $administration;

    function init($params=null, $model=null) {
        $administration = new Administration();
        $administration->retrieveSettings('notify');
        $administration->retrieveSettings('mail');
        $administration->retrieveSettings('massemailer');
        $this->administration = $administration;
        parent::init($params, $model);
    }

	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if($gen->getLayout()->getEditingLayout()) {
			$params = array();
			return $this->renderOuterTable($gen, $parents, $context, $params);		
		}

        return $this->renderSetupView($gen);
	}
	
	function getRequiredFields() {
        $fields = array();

		return $fields;
	}

    /**
     * Render Email Setup wizard
     *
     * @param HtmlFormGenerator $gen
     * @return string
     */
    function renderSetupView(HtmlFormGenerator $gen) {
        global $mod_strings, $app_strings, $app_list_strings;

        $layout = $gen->getLayout();
        $layout->addScriptInclude('modules/Campaigns/wizard.js');
        $layout->addScriptInclude('modules/Campaigns/email.js');
        $layout->addScriptLiteral("var LOCATION_DEFAULT='".$mod_strings['TRACKING_ENTRIES_LOCATION_DEFAULT_VALUE']."'", LOAD_PRIORITY_FOOT);

        $notify_fromaddress = $this->getPreference('notify_from_address');
        $notify_fromname = $this->getPreference('notify_from_name');
        $mail_smtpserver = $this->getPreference('mail_smtpserver');
        $mail_smtpport = $this->getPreference('mail_smtpport');
        $mail_smtpuser = $this->getPreference('mail_smtpuser');
        $mail_smtppass = $this->getPreference('mail_smtppass');

        $spec = array('name' => 'mail_sendtype', 'options' => $app_list_strings['notifymail_sendtype'],
            'onchange' => "notify_setrequired();", 'options_add_blank' => false, 'width' => 12);
        $selected_sendtype = $this->getPreference('mail_sendtype');
        if (! $selected_sendtype)
            $selected_sendtype = 'sendmail';
        $sendtype_select = $gen->form_obj->renderSelect($spec, $selected_sendtype);

        $smtp_auth_req_check = $this->renderCheckBox($gen, 'mail_smtpauth_req', $this->getPreference('mail_smtpauth_req'), "notify_setrequired();");

        $location_type = $this->getPreference('massemailer_tracking_entities_location_type');

        if (empty($location_type) || $location_type == '1') {
            $default_checked = "checked";
            $location_state = "disabled";
            $userdefined_checked = "";
            $location = $mod_strings['TRACKING_ENTRIES_LOCATION_DEFAULT_VALUE'];
        } else  {
            $default_checked = "";
            $location_state = "";
            $userdefined_checked = "checked";
            $location = $this->getPreference('massemailer_tracking_entities_location');
        }

        $email_copy = $this->getPreference('massemailer_email_copy');

        if (empty($email_copy) || $email_copy == '1') {
            $yes_checked = "checked='checked'";
            $no_checked = "";
        } else  {
            $yes_checked = "";                        
            $no_checked = "checked='checked'";
        }

        $emails_per_run = $this->getPreference('massemailer_campaign_emails_per_run');
        if (empty($emails_per_run))
            $emails_per_run = EmailSetupWidget::CAMPAIGN_EMAILS_PER_RUN;

        $user_mailboxes = $this->renderMailboxesList();
        $create_mbox_check = $this->renderCheckBox($gen, 'create_mbox', $user_mailboxes['mbox_needed'], "notify_setrequired();");

        $mailbox_edit = $this->renderMailboxEdit($gen);

        $body = <<<EOQ
            <input type='hidden' name='campaign_type' value="{$mod_strings['LBL_NEWSLETTER']}" />
            <input type="hidden" id="wiz_total_steps" name="totalsteps" value="3" />
            <input type="hidden" id="wiz_current_step" name="currentstep" value="0" />
            <input type="hidden" id="wiz_summary_step" name="wiz_summary_step" value="create_email_setup_summary" />

            <div id='wizard' class="form-section">
            	<div id="wiz_message"></div>
				<div id='step1' class="form-section">
					<div class="row form-row form-title-row">
						<div class="column form-cell">
                            <div class="form-entry form-title">
							<h4 class="dataLabel">{$mod_strings['LBL_NAVIGATION_MENU_SETUP']}</h4>
                            </div>
						</div>
					</div>
					<div class="row form-row form-title-row">
						<div class="column form-cell">
                            <div class="form-entry">
							{$mod_strings['LBL_EMAIL_SETUP_DESC']}
                            </div>
						</div>
					</div>
					<div class="row form-row">
						<div class="column form-cell sm-6">
							<div class="form-entry">
								<div class="form-label">{$mod_strings['LBL_WIZ_FROM_NAME']} <span class="requiredStar">{$app_strings['LBL_REQUIRED_SYMBOL']}</div>
								<div class="form-value"><input name="notify_from_name" tabindex='1' size='25' maxlength='128' type="text" id="notify_fromname" value="{$notify_fromname}" title="{$mod_strings['LBL_WIZ_FROM_NAME']}" class="input-text" /></div>
							</div>
						</div>
						<div class="column form-cell sm-6">
							<div class="form-entry">
								<div class="form-label">{$mod_strings['LBL_MAIL_SENDTYPE']}</div>
								<div class="form-value">{$sendtype_select}</div>
							</div>
						</div>
					</div>
					<div class="row form-row">
						<div class="column form-cell sm-6">
							<div class="form-entry">
								<div class="form-label">{$mod_strings['LBL_WIZ_FROM_ADDRESS']} <span class="requiredStar">{$app_strings['LBL_REQUIRED_SYMBOL']}</div>
								<div class="form-value"><input name='notify_from_address' id='notify_fromaddress' tabindex='1' size='25' maxlength='128' type="text" value="{$notify_fromaddress}" title="{$mod_strings['LBL_WIZ_FROM_ADDRESS']}"  class="input-text" /></div>
							</div>
						</div>
					</div>
					<div id="smtp_settings">
						<div class="row form-row">
							<div class="column form-cell sm-6">
								<div class="form-entry">
									<div class="form-label">{$mod_strings['LBL_MAIL_SMTPSERVER']} <span class="requiredStar">{$app_strings['LBL_REQUIRED_SYMBOL']}</div>
									<div class="form-value">
                    					<input type="text"  title="{$mod_strings['LBL_MAIL_SMTPSERVER']}"  name="mail_smtpserver"  id='mail_smtpserver'  tabindex="1" size="25" maxlength="64" value="{$mail_smtpserver}"  class="input-text">
									</div>
								</div>
							</div>
							<div class="column form-cell sm-6">
								<div class="form-entry">
									<div class="form-label">{$mod_strings['LBL_MAIL_SMTPPORT']} <span class="requiredStar">{$app_strings['LBL_REQUIRED_SYMBOL']}</div>
									<div class="form-value">
                    					<input type="text" id='mail_smtpport' name="mail_smtpport" title="{$mod_strings['LBL_MAIL_SMTPPORT']}" tabindex="1" size="5" maxlength="5" value="{$mail_smtpport}"  class="input-text">
									</div>
								</div>
							</div>
						</div>
						<div class="row form-row">
							<div class="column form-cell sm-6">
								<div class="form-entry">
									<div class="form-label">{$mod_strings['LBL_MAIL_SMTPAUTH_REQ']}</div>
									<div class="form-value">{$smtp_auth_req_check}</div>
								</div>
							</div>
						</div>
					</div>
					<div id="smtp_auth">
						<div class="row form-row">
							<div class="column form-cell sm-6">
								<div class="form-entry">
									<div class="form-label">{$mod_strings['LBL_MAIL_SMTPUSER']} <span class="requiredStar">{$app_strings['LBL_REQUIRED_SYMBOL']}</div>
									<div class="form-value">
										<input type="text" name="mail_smtpuser" id='mail_smtpuser' title='{$mod_strings['LBL_MAIL_SMTPUSER']}' size="25" maxlength="64" value="{$mail_smtpuser}" tabindex='1'  class="input-text">
									</div>
								</div>
							</div>
							<div class="column form-cell sm-6">
								<div class="form-entry">
									<div class="form-label">{$mod_strings['LBL_MAIL_SMTPPASS']} <span class="requiredStar">{$app_strings['LBL_REQUIRED_SYMBOL']}</div>
									<div class="form-value">
										<input type="password" title="{$mod_strings['LBL_MAIL_SMTPPASS']}" name="mail_smtppass" id='mail_smtppass' size="25" maxlength="64" value="{$mail_smtppass}" tabindex='1' class="input-text">
									</div>
								</div>
							</div>
						</div>
					</div>
					<div class="row form-row form-title-row">
						<div class="column form-cell">
                            <div class="form-entry form-title">
							<h4 class="dataLabel">{$mod_strings['LBL_NOTIFY_TITLE']}</h4>
                            </div>
						</div>
					</div>
					<div class="row form-row">
						<div class="column form-cell sm-6">
							<div class="form-entry">
								<div class="form-label">{$mod_strings['LBL_EMAILS_PER_RUN']} <span class="requiredStar">{$app_strings['LBL_REQUIRED_SYMBOL']}</div>
								<div class="form-value">
									<input name='massemailer_campaign_emails_per_run' id='massemailer_campaign_emails_per_run' title="{$mod_strings['LBL_EMAILS_PER_RUN']}" tabindex='1' maxlength='128' size="5" type="text" value="{$emails_per_run}" class="input-text">
								</div>
							</div>
						</div>
					</div>
					<div class="row form-row">
						<div class="column form-cell sm-6 vertical">
							<div class="form-entry">
							<div class="form-label">{$mod_strings['LBL_LOCATION_TRACK']} <span class="requiredStar">{$app_strings['LBL_REQUIRED_SYMBOL']}</div>
							<div class="form-value">
								<div class="input-field input-check"><input type='radio' onclick="change_state(this);" name='massemailer_tracking_entities_location_type' id='massemailer_tracking_entities_location_type' title="{$mod_strings['LBL_DEFAULT_LOCATION']}" value="1" {$default_checked}> <label class="input-label"> {$mod_strings['LBL_DEFAULT_LOCATION']}</label></div>
								<div class="input-field input-check"><input type='radio' {$userdefined_checked} onclick="change_state(this);" name='massemailer_tracking_entities_location_type' id='massemailer_tracking_entities_location_type' title="{$mod_strings['LBL_CUSTOM_LOCATION']}" value="2"> <label class="input-label"> {$mod_strings['LBL_CUSTOM_LOCATION']}</label></div>
							</div>
							</div>
							<div class="form-entry">
								<input name='massemailer_tracking_entities_location' id='massemailer_tracking_entities_location' title="{$mod_strings['LBL_LOCATION_TRACK']}" {$location_state} maxlength='128' size='40' type="text" value="{$location}" class="input-text">
							</div>
						</div>
					</div>
					<div class="row form-row">
						<div class="column form-cell sm-6 vertical">
							<div class="form-entry">
							<div class="form-label">{$mod_strings['LBL_CAMP_MESSAGE_COPY']} <span class="requiredStar">{$app_strings['LBL_REQUIRED_SYMBOL']}</div>
							<div class="form-value">
								<div class="input-field input-check"><input type='radio' name='massemailer_email_copy'value="1" {$yes_checked}> <label class="input-label"> {$mod_strings['LBL_YES']}</label></div>

								<div class="input-field input-check"><input type='radio' name='massemailer_email_copy' value="2" {$no_checked}> <label class="input-label"> {$mod_strings['LBL_NO']}</label></div>
							</div>
							</div>
						</div>
					</div>
				</div>
				<div id='step2' class="form-section">
					<div class="row form-row form-title-row">
						<div class="column form-cell">
                            <div class="form-entry form-title">
							<h4 class="dataLabel">{$mod_strings['LBL_NAVIGATION_MENU_NEW_MAILBOX']}</h4>
                            </div>
						</div>
					</div>
					<div class="row form-row form-title-row">
						<div class="column form-cell sm-12">
							{$user_mailboxes['mailboxes']}
						</div>
					</div>
					<div class="row form-row">
						<div class="column form-cell sm-12">
                            <div class="form-entry">
							<input type='hidden' id='wiz_new_mbox' name='wiz_new_mbox' value='0'>
            					{$create_mbox_check}&nbsp;{$mod_strings['LBL_CREATE_MAILBOX']}
                            </div>
						</div>
					</div>
					<div class="row form-row" id="new_mbox">
						<div class="column form-cell sm-12">
							<div class="form-entry">
								{$mailbox_edit}
                            </div>
						</div>
					</div>
				</div>

				<div id='step3' class="form-section">
					<div class="row form-row form-title-row">
						<div class="column form-cell">
							<div class="form-entry form-title">
								<h3>{$mod_strings['LBL_NAVIGATION_MENU_SUMMARY']}</h3>
							</div>
						</div>
					</div>
					<div id='wiz_summ'></div>
				</div>
			</div>


EOQ;

        return $body;
    }

    /**
     * Render user mailboxes list
     *
     * @return array
     */
    function renderMailboxesList() {
        global $mod_strings, $even_bg, $odd_bg, $hilite_bg;
        $mbox = CampaignUtils::get_user_mailboxes();
        $need_mbox = 0;

        $mbox_table = "<table class='listView' width='100%' border='0' cellspacing='0' cellpadding='1'>";

        if (count($mbox) > 0) {
            $mbox_table .= "<tr><td colspan='5'><b>" .count($mbox) ." ". $mod_strings['LBL_MAILBOX_CHECK_WIZ_GOOD']." </b>.</td></tr>";
            $mbox_table .= "<tr><td width='20%' class='listViewThS1'><b>".$mod_strings['LBL_MAILBOX_NAME']."</b></td>"
                .  " <td width='20%' class='listViewThS1'><b>".$mod_strings['LBL_LOGIN']."</b></td>"
                .  " <td width='20%' class='listViewThS1'><b>".$mod_strings['LBL_MAILBOX']."</b></td>"
                .  " <td width='20%' class='listViewThS1'><b>".$mod_strings['LBL_SERVER_URL']."</b></td>"
                .  " <td width='20%' class='listViewThS1'><b>".$mod_strings['LBL_LIST_STATUS']."</b></td></tr>";

            $colorclass = ' ';

            foreach ($mbox as $details) {

                if ( $colorclass == "class='evenListRowS1' bgColor='$even_bg' ") {
                    $colorclass = "class='oddListRowS1' bgColor='$odd_bg' ";
                    $bgColor = $odd_bg;
                } else {
                    $colorclass = "class='evenListRowS1' bgColor='$even_bg' ";
                    $bgColor = $even_bg;
                }

                $mbox_table .= "<tr  onmouseover=\"setPointer(this, '', 'over', '$bgColor', '$hilite_bg', '');\" onmouseout=\"setPointer(this, '', 'out', '$bgColor', '$hilite_bg', '');\" onmousedown=\"setPointer(this, '', 'click', '$bgColor', '$hilite_bg', '');\">";
                $mbox_table .= "<td $colorclass>".$details['name']."</td>";
                $mbox_table .= "<td $colorclass>".$details['username']."</td>";
                $mbox_table .= "<td $colorclass>".$details['imap_folder']."</td>";
                $mbox_table .= "<td $colorclass>".$details['host']."</td>";
                $mbox_table .= "<td $colorclass>".array_get_default($details, 'status', '')."</td></tr>";
            }
        } else {
            $need_mbox = 1;
            $mbox_table .= "<tr><td colspan='5'><b>".$mod_strings['LBL_MAILBOX_CHECK_WIZ_BAD']." </b>.</td></tr>";
        }

        $mbox_table .= "</table>";
        
        return array('mbox_needed' => $need_mbox, 'mailboxes' => $mbox_table);
    }

    /**
     * Render block for creating new mailbox
     *
     * @param HtmlFormGenerator $gen
     * @return string
     */
    function renderMailboxEdit(HtmlFormGenerator $gen) {
        global $mod_strings, $app_strings;

        $protocols = array('IMAP'=>'IMAP', 'POP3'=>'POP3');
        $spec = array('name' => 'protocol', 'options' => $protocols, 'onchange' => "hide_show_imap_folder();set_default_port();");
        $protocol_select = $gen->form_obj->renderSelect($spec, 'IMAP');

        $active_check = $this->renderCheckBox($gen, 'active', 1);
        $leave_check = $this->renderCheckBox($gen, 'leave_on_server', 1);
        $use_ssl_check = $this->renderCheckBox($gen, 'use_ssl', 0, "set_default_port();");

        $body = <<<EOQ
			<div class="row form-row">
				<div class="column form-cell sm-6">
					<div class="form-entry">
						<div class="form-label">{$mod_strings['LBL_NAME']} <span class="requiredStar">{$app_strings['LBL_REQUIRED_SYMBOL']}</div>
						<div class="form-value">
							<input id="name" type="text" name='name' tabindex='1' size="30" value="" title="{$mod_strings['LBL_NAME']}" class="input-text">
						</div>
					</div>
				</div>
				<div class="column form-cell sm-6">
					<div class="form-entry">
						<div class="form-label">{$mod_strings['LBL_ACTIVE']}</div>
						<div class="form-value">
							{$active_check}
						</div>
					</div>
				</div>
			</div>
			<div class="row form-row">
				<div class="column form-cell sm-6">
					<div class="form-entry">
						<div class="form-label">{$mod_strings['LBL_FROM_NAME']}</div>
						<div class="form-value">
                			<input type="text" name='from_name' tabindex='1' size="30" value="" class="input-text">
						</div>
					</div>
				</div>
				<div class="column form-cell sm-6">
					<div class="form-entry">
						<div class="form-label">{$mod_strings['LBL_EMAIL']} <span class="requiredStar">{$app_strings['LBL_REQUIRED_SYMBOL']}</div>
						<div class="form-value">
                			<input id="from_addr" type="text" name='email' tabindex='2' size="30" value="" title="{$mod_strings['LBL_EMAIL']}" class="input-text">
						</div>
					</div>
				</div>
			</div>
			<div class="row form-row">
				<div class="column form-cell sm-6">
					<div class="form-entry">
						<div class="form-label">{$mod_strings['LBL_HOST']} <span class="requiredStar">{$app_strings['LBL_REQUIRED_SYMBOL']}</div>
						<div class="form-value">
                			<input id="server_url" type="text" name='host' tabindex='1' size="30" value="" title="{$mod_strings['LBL_HOST']}" class="input-text">
						</div>
					</div>
				</div>
				<div class="column form-cell sm-6">
					<div class="form-entry">
						<div class="form-label">{$mod_strings['LBL_USERNAME']} <span class="requiredStar">{$app_strings['LBL_REQUIRED_SYMBOL']}</div>
						<div class="form-value">
                			<input id="email_user" type="text" name='username' tabindex='2' size="25" value="" title="{$mod_strings['LBL_USERNAME']}" class="input-text">
						</div>
					</div>
				</div>
			</div>
			<div class="row form-row">
				<div class="column form-cell sm-6">
					<div class="form-entry">
						<div class="form-label">{$mod_strings['LBL_PASSWORD']} <span class="requiredStar">{$app_strings['LBL_REQUIRED_SYMBOL']}</div>
						<div class="form-value">
                			<input id="email_password" type="password" name='password' tabindex='1' size="25" value="" autocomplete="off" title="{$mod_strings['LBL_PASSWORD']}" class="input-text">
						</div>
					</div>
				</div>
				<div class="column form-cell sm-6">
					<div class="form-entry">
						<div class="form-label">{$mod_strings['LBL_LEAVE_ON_SERVER']}</div>
						<div class="form-value">
							{$leave_check}
						</div>
					</div>
				</div>
			</div>
			<div class="row form-row">
				<div class="column form-cell sm-6">
					<div class="form-entry">
						<div class="form-label">{$mod_strings['LBL_PROTOCOL']}</div>
						<div class="form-value">
							{$protocol_select}
						</div>
					</div>
				</div>
				<div class="column form-cell sm-6">
					<div class="form-entry">
						<div id="imap_folder_label" class="form-label">{$mod_strings['LBL_IMAP_FOLDER']}</div>
						<div id="imap_folder_name" class="form-value">
							<input id="mailbox" type="text" name='imap_folder' tabindex='2' size="25" value="INBOX" title="{$mod_strings['LBL_IMAP_FOLDER']}" class="input-text">
						</div>
					</div>
				</div>
			</div>
			<div class="row form-row">
				<div class="column form-cell sm-6">
					<div class="form-entry">
						<div class="form-label">{$mod_strings['LBL_PORT']}</div>
						<div class="form-value">
                			<input type="text" name='port' id="port" tabindex='1' size="8" value="143" title="{$mod_strings['LBL_PORT']}" class="input-text">
						</div>
					</div>
				</div>
				<div class="column form-cell sm-6">
					<div class="form-entry">
						<div class="form-label">{$mod_strings['LBL_USE_SSL']}</div>
						<div class="form-value">
							{$use_ssl_check}
						</div>
					</div>
				</div>
			</div>
EOQ;

        return $body;
    }

    /**
     * Get mail preference value
     *
     * @param  string $name - preference name
     * @return mixed
     */
    function getPreference($name) {
        if (isset($this->administration->settings[$name])) {
            return $this->administration->settings[$name];
        } else {
            return null;
        }
    }

    /**
     * Set new value for mail preference
     *
     * @param string $name - preference name
     * @param mixed $value
     * @return void
     */
    function setPreference($name, $value) {
		$prefix = Administration::get_config_prefix($name);
		AppConfig::set_local(join('.', $prefix), $value);
    }

    /**
     * Get users's email preferencies list
     *
     * @return array
     */
    function getPreferencesList() {
        $prefs = array('notify_from_address', 'notify_from_name', 'mail_smtpserver', 'mail_smtpport', 'mail_smtpuser',
            'mail_smtppass', 'mail_sendtype', 'mail_smtpauth_req', 'massemailer_tracking_entities_location_type',
            'massemailer_tracking_entities_location', 'massemailer_email_copy', 'massemailer_campaign_emails_per_run'
        );

        return $prefs;
    }

    /**
     * Render check box
     *
     * @param HtmlFormGenerator $gen
     * @param string $name
     * @param mixed $value
     * @param string $onchange
     * @return string
     */
    function renderCheckBox(HtmlFormGenerator $gen, $name, $value, $onchange = null) {
        $spec = array('name' => $name);
        if ($onchange) $spec['onchange'] = $onchange;

        if (is_string($value) && $value == 'on') {
            $checked = 1;
        } elseif (is_string($value) && $value == 'off') {
            $checked = 0;
        } elseif (is_bool($value) && $value === true) {
            $checked = 1;
        } elseif (is_bool($value) && $value === false) {
            $checked = 0;
        } elseif (empty($value)) {
            $checked = 0;
        } else {
            $checked = $value;
        }

        return $gen->form_obj->renderCheck($spec, $checked);
    }

	function loadUpdateRequest(RowUpdate $update, array $input) {
        $preferences = $this->getPreferencesList();
        $updated_prefs = array();

        for ($i = 0; $i < sizeof($preferences); $i++) {
            if (isset($input[$preferences[$i]]))
                $updated_prefs[$preferences[$i]] = $input[$preferences[$i]];
		}

        $update->setRelatedData($this->id.'_pref_rows', $updated_prefs);

        $mailbox_fields = array('name', 'active', 'from_name', 'email', 'host', 'username',
            'password', 'leave_on_server', 'protocol', 'imap_folder', 'port', 'use_ssl');

        $mailbox_data = array();

        if(isset($input['wiz_new_mbox']) && ($input['wiz_new_mbox'] == '1')) {

            for ($i = 0; $i < sizeof($mailbox_fields); $i++) {
                if (isset($input[$mailbox_fields[$i]]))
                    $mailbox_data[$mailbox_fields[$i]] = $input[$mailbox_fields[$i]]; 
            }

            if (sizeof($mailbox_data) > 0) {
                require_once('modules/EmailFolders/EmailFolder.php');
                $mailbox_data['scheduler'] = 1;
                $mailbox_data['mailbox_type'] = 'bounce';
                $mailbox_data['active'] = 1;
                $mailbox_data['user_id'] = '-1';
                $mailbox_data['email_folder_id'] = EmailFolder::get_std_folder_id('-1', STD_FOLDER_CAMPAIGN);

                $update->setRelatedData($this->id.'_mbox_rows', $mailbox_data);
            }
        }

    }
	
	function validateInput(RowUpdate $update) {
		return true;
	}
	
	function afterUpdate(RowUpdate $update) {
        $pref_updates = $update->getRelatedData($this->id.'_pref_rows');

        if (sizeof($pref_updates) > 0) {
            $preferences = $this->getPreferencesList();
            for ($i = 0; $i < sizeof($preferences); $i++) {
                if (isset($pref_updates[$preferences[$i]])) {
                    $this->setPreference($preferences[$i], $pref_updates[$preferences[$i]]);
				}
			}
			AppConfig::save_local();
            AppConfig::invalidate_cache('notify');
            AppConfig::invalidate_cache('mail');
            AppConfig::invalidate_cache('massemailer');
        }

        $mbox_data = $update->getRelatedData($this->id.'_mbox_rows');

        if (is_array($mbox_data) && sizeof($mbox_data) > 0) {
            require_once('modules/EmailPOP3/EmailPOP3.php');
            EmailPOP3::create_mbox($mbox_data);
        }
    }
}
?>
