<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormTableSection.php');
require_once('modules/Campaigns/FormCaptcha.php');

class WebToLeadWidget extends FormTableSection {

    /**
     * Form required fields
     *
     * @var array
     */
    var $required_fields;

	/**
	 * @var FormCaptcha
	 */
    var $captcha;

    const CLASSNAME = 'SUGAR_GRID';

    const DEFAULT_POST_URL = 'WebToLeadHandler.php';

    function init($params=null, $model=null) {
        parent::init($params, $model);
        $this->captcha = new FormCaptcha();
    }

	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if($gen->getLayout()->getEditingLayout()) {
			$params = array();
			return $this->renderOuterTable($gen, $parents, $context, $params);
		}

        if ($this->id == 'webtolead_generation') {
            return $this->renderFormEditor($gen);
		} elseif ($this->id == 'wordpress_export') {
            return $this->renderWordpressCode($gen);
        } elseif ($this->id == 'webtolead_save') {
            return $this->renderDownloadView();
        } else {
            return $this->renderSelectListView($gen, $row_result);
        }
	}

	function getRequiredFields() {
        $fields = array('assigned_user', 'campaign', 'name');

		return $fields;
	}

    /**
     * Render Track chart
     *
     * @param HtmlFormGenerator $gen
     * @param RowResult $row_result
     * @return string
     */
    function renderSelectListView(HtmlFormGenerator $gen, RowResult $row_result) {
        global $mod_strings, $app_strings;

		$fields = $this->getLeadFields();
		$required = array_values(array_filter($fields, function($f) { return !empty($f[2]);}));
		$json = getJSONobj();
		$options = [];
		foreach ($fields as $f) {
			$options[$f[0]] = $f[1];
		}

		$def = [
			'name' => 'left',
			'type' => 'multienum',
			'options' => $options,
			'full_height' => true,
		];

		$left = $gen->form_obj->renderField($row_result, $def);
		$def = [
			'name' => 'right',
			'type' => 'multienum',
			'options' => $options,
			'full_height' => true,
		];

		$right = $gen->form_obj->renderField($row_result, $def);

        $body = <<<EOQ
<div id='grid_Div' class="form-section">
	<div class="row form-row">
		<div class="column form-cell">
			<b>{$mod_strings['LBL_DRAG_DROP_COLUMNS1']}</b>
		</div>
	</div><div class="row form-row">
		<div class="column form-cell sm-12">
			<b>{$mod_strings['LBL_DRAG_DROP_COLUMNS2']}</b>
		</div>
	</div><div class="row form-row">
		<div class="column form-cell sm-12">&nbsp;</div>
	</div>
	<div id="webformfields"></div>
	<div class="row form-row col-unstack">
		<div class="column form-cell sm-6 vertical">
			<div class="form-label">{$mod_strings['LBL_LEFT_COLUMN']}</div>
			<div class="form-entry">
				{$left}
			</div>
		</div>
		<div class="column form-cell sm-6 vertical">
			<div class="form-label">{$mod_strings['LBL_RIGHT_COLUMN']}</div>
			<div class="form-entry">
				{$right}
			</div>
		</div>
	</div>
</div>
EOQ;

        $body .= $this->renderFormProperties($gen, $row_result);

        $layout = $gen->getLayout();
        $layout->addScriptInclude('modules/Campaigns/WebToLead.js');
        $layout->addScriptLiteral('initLeftRight(' . $json->encode($required) . ');', LOAD_PRIORITY_END);

        return $body;
    }

    /**
     * Render Lead form properties block
     *
     * @param HtmlFormGenerator $gen
     * @param RowResult $row_result
     * @return string
     */
    function renderFormProperties(HtmlFormGenerator $gen, RowResult $row_result) {
        global $mod_strings, $app_strings;

		$def = array(
			'name' => 'add_target_list',
			'type' => 'ref',
			'bean_name' => 'ProspectList',
			'onchange' => 'addTargetListToForm(this);',
		);

		$target_list = $gen->form_obj->renderField($row_result, $def);

		$campaign = $gen->form_obj->renderField($row_result, 'campaign');
        $assigned_user = $gen->form_obj->renderField($row_result, 'assigned_user');
        $spec = array('name' => 'chk_edit_url', 'onchange' => 'editUrl();');
        $edit_url_check = $gen->form_obj->renderCheck($spec, 0);

        $web_post_url = AppConfig::site_url() . self::DEFAULT_POST_URL;
        $redirect_url = 'http://';

	    $body = <<<EOQ
            <div id='lead_queries_Div' style="display: none" class="form-section" >
				<div class="row form-row">
					<div class="column form-cell sm-12 vertical">
						<div class="form-label">{$mod_strings['LBL_DEFINE_LEAD_HEADER']}</div>
						<div class="form-entry">
                			<input id="web_header" name="web_header" title="Name" size="60" value="{$mod_strings['LBL_LEAD_DEFAULT_HEADER']}" type="text" class="input-text">
						</div>
					</div>
				</div>
				<div class="row form-row">
					<div class="column form-cell sm-12 vertical">
						<div class="form-label">{$mod_strings['LBL_DESCRIPTION_LEAD_FORM']}</div>
						<div class="form-entry">
                			<textarea tabindex='1' name='web_description' rows='2' cols='55' class="input-text">{$mod_strings['LBL_DESCRIPTION_TEXT_LEAD_FORM']}</textarea>
						</div>
					</div>
				</div>
				<div class="row form-row">
					<div class="column form-cell sm-12 vertical">
						<div class="form-label">{$mod_strings['LBL_DEFINE_LEAD_SUBMIT']}</div>
						<div class="form-entry">
            				<input class="input-text" id="web_submit" name="web_submit" title="Name" size="60" value="{$mod_strings['LBL_DEFAULT_LEAD_SUBMIT']}" type="text">
						</div>
					</div>
				</div>
				<div class="row form-row">
					<div class="column form-cell sm-12 vertical">
						<div class="form-label">{$mod_strings['LBL_DEFINE_LEAD_DUP_ERROR']}</div>
						<div class="form-entry">
                			<input class="input-text" id="dup_message" name="dup_message" size="60" value="{$mod_strings['LBL_DEFAULT_LEAD_DUP_ERROR']}" type="text" >
						</div>
					</div>
				</div>
				<div class="row form-row">
					<div class="column form-cell sm-12 vertical">
						<div class="form-label">{$mod_strings['LBL_DEFINE_LEAD_POST_URL']}</div>
						<div class="form-entry">
                			<input class="input-text" id="post_url" name="post_url" size="60" disabled='true' value="{$web_post_url}" type="text">
						</div>
						<div class="form-entry">
                			{$edit_url_check}&nbsp;{$mod_strings['LBL_EDIT_LEAD_POST_URL']}
						</div>
					</div>
				</div>
				<div class="row form-row">
					<div class="column form-cell sm-12 vertical">
						<div class="form-label">{$mod_strings['LBL_DEFINE_LEAD_REDIRECT_URL']}</div>
						<div class="form-entry">
                			<input class="input-text" id="redirect_url" name="redirect_url" size="60" value="{$redirect_url}" type="text">
						</div>
					</div>
				</div>
				<div class="row form-row">
					<div class="column form-cell sm-12 vertical">
						<div class="form-label">{$mod_strings['LBL_LEAD_NOTIFY_CAMPAIGN']} <span class="required">{$app_strings['LBL_REQUIRED_SYMBOL']}</span></div>
						<div class="form-entry">
							{$campaign}
						</div>
					</div>
				</div>
				<div class="row form-row">
					<div class="column form-cell sm-12 vertical">
						<div class="form-label">{$mod_strings['LBL_LEAD_NOTIFY_SUBSCRIPTIONS']}</div>
						<div class="form-entry">
							{$target_list}
						</div>
						<div class="form-entry" id="selected_target_lists">
						</div>
					</div>
				</div>
				<div class="row form-row">
					<div class="column form-cell sm-12 vertical">
						<div class="form-label">{$app_strings['LBL_ASSIGNED_TO']} <span class="required">{$app_strings['LBL_REQUIRED_SYMBOL']}</span></div>
						<div class="form-entry">
                			{$assigned_user}
						</div>
					</div>
				</div>
				{$this->captcha->getSettingsHtml($gen, $mod_strings)}
				<div class="row form-row">
					<div class="column form-cell sm-12 vertical">
						<div class="form-label">{$mod_strings['LBL_LEAD_FOOTER']}</div>
						<div class="form-entry">
                			<textarea class="input-text" tabindex='1' name='web_footer' rows='2' cols='55'></textarea>
						</div>
					</div>
				</div>

            <table width="100%" cellpadding="0" cellspacing="0" border="0">
            <tr><td>&nbsp;</td></tr>
            </table>
            <table width="100%" cellpadding="0" cellspacing="0" border="0" class="tabForm">
            <tr><td>
            </td></tr>
            </table>
            </div>
EOQ;
        return $body;
    }

	/**
     * Render Web-to-Lead form editor
     *
     * @param HtmlFormGenerator $gen
     * @return string
     */
    function renderFormEditor(HtmlFormGenerator $gen) {
		global $mod_strings, $app_strings;
		$subscriptions = array_unique(array_get_default($_POST, 'subscriptions', array()));

        $web_form_header = $mod_strings['LBL_LEAD_DEFAULT_HEADER'];
        $web_form_description = $mod_strings['LBL_DESCRIPTION_TEXT_LEAD_FORM'];
        $web_form_submit_label = $mod_strings['LBL_DEFAULT_LEAD_SUBMIT'];
        $web_post_url = AppConfig::site_url() .'/'. self::DEFAULT_POST_URL;

        $web_redirect_url = '';
        $web_assigned_user = '';
        $web_form_footer = '';
        $dup_message = '';
		$add_captcha = $_REQUEST['add_captcha'] ?? false;

        if(! empty($_REQUEST['web_header']))
            $web_form_header = $_REQUEST['web_header'];
        if(! empty($_REQUEST['web_description']))
            $web_form_description = $_REQUEST['web_description'];
        if(! empty($_REQUEST['web_submit']))
            $web_form_submit_label = to_html($_REQUEST['web_submit']);
        if(! empty($_REQUEST['post_url']))
            $web_post_url = $_REQUEST['post_url'];
        if(! empty($_REQUEST['redirect_url']) && $_REQUEST['redirect_url'] != "http://")
            $web_redirect_url = $_REQUEST['redirect_url'];
        if(! empty($_REQUEST['web_footer']))
            $web_form_footer = $_REQUEST['web_footer'];
        if(! empty($_REQUEST['campaign_id']))
            $web_form_campaign = $_REQUEST['campaign_id'];
        if(! empty($_REQUEST['assigned_user_id']))
            $web_assigned_user = $_REQUEST['assigned_user_id'];
        if(! empty($_REQUEST['dup_message']))
            $dup_message = $_REQUEST['dup_message'];

        $Web_To_Lead_Form_html = "<form action='$web_post_url' name='WebToLeadForm' method='POST'>";
        $Web_To_Lead_Form_html .= "<table width='100%' style='border-top: 1px solid; border-bottom: 1px solid; padding: 10px 6px 12px 10px; background-color: rgb(233, 243, 255); font-size: 12px; background-repeat: repeat-x; background-position: center top;'>";
        $Web_To_Lead_Form_html .= "<tr align='center' style='color: rgb(0, 105, 225); font-family: Arial,Verdana,Helvetica,sans-serif; font-size: 18px; font-weight: bold; margin-bottom: 0px; margin-top: 0px;'><td COLSPAN='4'><b><h2>$web_form_header</h2></b></td></tr>";
        $Web_To_Lead_Form_html .= "<tr align='center' style='color: rgb(0, 105, 225); font-family: Arial,Verdana,Helvetica,sans-serif; font-size: 2px; font-weight: normal; margin-bottom: 0px; margin-top: 0px;'><td COLSPAN='4'>&nbsp</td></tr>";
        $Web_To_Lead_Form_html .= "<tr id='error_message_row' align='center' style='display: none;color: rgb(255, 0, 0); font-family: Arial,Verdana,Helvetica,sans-serif; font-size: 12px; font-weight: bold; margin-bottom: 0px; margin-top: 0px;'><td id='error_message' COLSPAN='4'></td></tr>";
        $Web_To_Lead_Form_html .= "<tr align='center' style='color: rgb(0, 105, 225); font-family: Arial,Verdana,Helvetica,sans-serif; font-size: 2px; font-weight: normal; margin-bottom: 0px; margin-top: 0px;'><td COLSPAN='4'>&nbsp</td></tr>";
        $Web_To_Lead_Form_html .= "<tr align='left' style='color: rgb(0, 105, 225); font-family: Arial,Verdana,Helvetica,sans-serif; font-size: 12px; font-weight: normal; margin-bottom: 0px; margin-top: 0px;'><td COLSPAN='4'>$web_form_description</td></tr>";
        $Web_To_Lead_Form_html .= "<tr align='center' style='color: rgb(0, 105, 225); font-family: Arial,Verdana,Helvetica,sans-serif; font-size: 8px; font-weight: normal; margin-bottom: 0px; margin-top: 0px;'><td COLSPAN='4'>&nbsp</td></tr>";

		$statusAdded = false;
        $Web_To_Lead_Form_html .= $this->generateForm($statusAdded);

        $Web_To_Lead_Form_html .= "<tr align='center' style='color: rgb(0, 105, 225); font-family: Arial,Verdana,Helvetica,sans-serif; font-size: 18px; font-weight: bold; margin-bottom: 0px; margin-top: 0px;'><td COLSPAN='4'>&nbsp</td></tr>";

        if(! empty($web_form_footer)) {
            $Web_To_Lead_Form_html .= "<tr align='center' style='color: rgb(0, 105, 225); font-family: Arial,Verdana,Helvetica,sans-serif; font-size: 18px; font-weight: bold; margin-bottom: 0px; margin-top: 0px;'><td COLSPAN='4'>&nbsp</td></tr>";
            $Web_To_Lead_Form_html .= "<tr align='left' style='color: rgb(0, 105, 225); font-family: Arial,Verdana,Helvetica,sans-serif; font-size: 12px; font-weight: normal; margin-bottom: 0px; margin-top: 0px;'><td COLSPAN='4'>$web_form_footer</td></tr>";
        }

	    if ($add_captcha) {
            $this->captcha->refreshKeys($_REQUEST);
		    $Web_To_Lead_Form_html .= "<tr align='center'><td colspan='10' style='display: none;'>_CAPTCHA_</td></tr>";
	    }

	    $Web_To_Lead_Form_html .= "<tr align='center'><td colspan='10'><input type='button' onclick='check_webtolead_fields(this.form);' class='button' name='Submit' value='$web_form_submit_label'/></td></tr>";

		if (!empty($subscriptions)) {
			$Web_To_Lead_Form_html .= "<tr><td style='display: none'>";
			foreach ($subscriptions as $sid) {
        		$Web_To_Lead_Form_html .= '<input type="hidden" name="subscriptions[]" value="' . $sid . '">';
			}
        	$Web_To_Lead_Form_html .= "</td></tr>";
		}
        if (!$statusAdded)
           $Web_To_Lead_Form_html .= "<tr><td style='display: none'><input type='hidden' id='status' name='status' value='New'></td></tr>";
        if(! empty($web_form_campaign))
           $Web_To_Lead_Form_html .= "<tr><td style='display: none'><input type='hidden' id='campaign_id' name='campaign_id' value='$web_form_campaign'></td></tr>";

        if(! empty($web_redirect_url))
            $Web_To_Lead_Form_html .= "<tr><td style='display: none'><input type='hidden' id='redirect_url' name='redirect_url' value='$web_redirect_url'></td></tr>";

        if(! empty($web_assigned_user))
            $Web_To_Lead_Form_html .= "<tr><td style='display: none'><input type='hidden' id='assigned_user_id' name='assigned_user_id' value='$web_assigned_user'></td></tr>";

        $Web_To_Lead_Form_html .= "<tr><td style='display: none'><input type='hidden' id='error_message_text' name='error_message_text' value='{$dup_message}'></td></tr>";
	    $Web_To_Lead_Form_html .= "<tr><td style='display: none'><input type='hidden' id='form_token' name='form_token' value='_TOKEN_'></td></tr>";

        $req_fields = '';

        if($this->required_fields != null) {
            foreach($this->required_fields as $req) {
                 $req_fields = $req_fields . $req . ';';
            }
        }

        if(! empty($req_fields)) {
            $Web_To_Lead_Form_html .= "<tr><td style='display: none'><input type='hidden' id='req_id' name='req_id' value='$req_fields'></td></tr>";
        }

        $Web_To_Lead_Form_html .= "</table >";
        $Web_To_Lead_Form_html .= "</form>";
        //$Web_To_Lead_Form_html .= $this->getFormJs();

        $spec = array('name' => 'body_html', 'fullPage' => true);
        $editor = $gen->form_obj->renderHtmlEditor($spec, $Web_To_Lead_Form_html, true);

		$body = <<<EOQ
<div class="form-section">
	<div class="text-right"><span class="required">{$app_strings['LBL_REQUIRED_SYMBOL']}</span> {$app_strings['NTC_REQUIRED']}</div>
	<div id="html_div">{$editor}</div>
</div>
EOQ;

        return $body;
    }

    /**
     * Generate Web-to-Lead form for Campaign
     *
     * @return void
     */
    function generateForm(&$statusAdded) {
		$left_fields = array_filter(explode('^,^', $_POST['left']));
		$right_fields = array_filter(explode('^,^', $_POST['right']));
		$columns = max(count($left_fields), count($right_fields));
        $Web_To_Lead_Form_html = '';
        $required_fields = array();

        for ($i = 0; $i < $columns; $i++) {
            $colsFirstField = '';
            $colsSecondField = '';

            if (! empty($left_fields[$i]))
                $colsFirstField = $left_fields[$i];
            if (! empty($right_fields[$i]))
                $colsSecondField = $right_fields[$i];

            $Web_To_Lead_Form_html .= "<tr>";
            $Web_To_Lead_Form_html .= $this->renderFormCol($colsFirstField, $required_fields);
            $Web_To_Lead_Form_html .= $this->renderFormCol($colsSecondField, $required_fields);
            $Web_To_Lead_Form_html .= "</tr>";
        }

		$statusAdded = in_array('status', $required_fields);
		if (!$statusAdded)
			$required_fields[] = 'status';

		$this->required_fields = $required_fields;

        return $Web_To_Lead_Form_html;
    }

	function renderWordpressCode(HtmlFormGenerator $gen) {
		require_once('modules/Campaigns/WebToLeadForm.php');
		require_once 'modules/Campaigns/WordpressFormGen.php';

		global $app_strings;
		$subscriptions = array_unique(array_get_default($_POST, 'subscriptions', array()));
		$left_fields = array_filter(explode('^,^', $_POST['left']));
		$right_fields = array_filter(explode('^,^', $_POST['right']));

		$columns = array(
			$left_fields,
			$right_fields,
		);

		$extra_fields = array(
			'web_header',
			'web_description',
			'web_submit',
			'redirect_url',
			'web_footer',
			'campaign_id',
			'assigned_user_id',
			'dup_message',
		);

		$extra = array();
		foreach ($extra_fields as $f)
			if (isset($_REQUEST[$f]) && strlen($_REQUEST[$f]))
				$extra[$f] = $_REQUEST[$f];
		if (!empty($subscriptions))
			$extra['subscriptions'] = $subscriptions;

		$with_captcha = (bool)$_REQUEST['add_captcha'] ?? false;
		$form = new WebToLeadForm();
		$extra['form_token'] = $form->add($with_captcha);

		if ($with_captcha) {
			$this->captcha->refreshKeys($_REQUEST);
			$extra = array_merge($extra, $this->captcha->getFormFields());
		}

		$wp = new WordpressFormGen($columns, $extra);
		$code = $wp->generateCode();
		$json = getJSONObj();

		$ret = '<p style="margin-left: 1em;">' . $app_strings['LBL_COPY_TO_WORDPRESS'] . '</p>';
		$ret .= '<textarea style="margin-left: 1em; width:90%; height : 300px;">' . to_html($json->encode($code)) . '</textarea>';
		return $ret;
	}

    /**
     * Render Web-to-Lead form columns
     *
     * @param string $colsField
     * @param array $required_fields
     * @return string
     */
    function renderFormCol($colsField, &$required_fields) {
        global $app_strings, $app_list_strings;

        $web_required_symbol = $app_strings['LBL_REQUIRED_SYMBOL'];
        $weblead_date_format = $app_strings['LBL_WEBLEAD_DATE_FORMAT'];
        $field_type = null;
        $field_name = null;
        $field_options = null;
        $field_required = '';

        $lead = new ModelDef('Lead');
        $field_defs = $lead->getFieldDefinitions();

        if (isset($field_defs[$colsField]) && $field_defs[$colsField] != null) {

            $field_vname = preg_replace('/:$/','',translate($field_defs[$colsField]['vname'], 'Leads'));
            $field_name  = $colsField;
            $field_label = $field_vname . ": ";

            if (isset($field_defs[$colsField]['custom_type']) && $field_defs[$colsField]['custom_type'] != null) {
                $field_type = $field_defs[$colsField]['custom_type'];
            } else {
                $field_type = $field_defs[$colsField]['type'];
            }

            if (isset($field_defs[$colsField]['required']) && $field_defs[$colsField]['required'] != null) {
                $field_required = $field_defs[$colsField]['required'];

                if (! in_array($field_defs[$colsField]['name'], $required_fields)) {
                    array_push($required_fields, $field_defs[$colsField]['name']);
                }
            }

            if ($field_defs[$colsField]['name'] == 'last_name') {
                if (! in_array($field_defs[$colsField]['name'], $required_fields)) {
                    array_push($required_fields, $field_defs[$colsField]['name']);
                }
            }

            if ($field_type == 'enum' || $field_type == 'multienum') $field_options = $field_defs[$colsField]['options'];
        }

        $Web_To_Lead_Form_html = '';

		$text_types = array('text', 'varchar', 'name', 'phone', 'email', 'url', 'int', 'double', 'currency', 'base_currency');

        if (isset($field_defs[$colsField]) && $field_defs[$colsField] != null) {
            if ($field_type=='enum' || $field_type == 'multienum') {

                if (isset($field_defs[$colsField]['default'])) {
                    $lead_options = get_select_options_with_id($app_list_strings[$field_options], $field_defs[$colsField]['default']);
                } else {
                    $lead_options = get_select_options_with_id($app_list_strings[$field_options], '');
                }
                if ($field_required) {
                    $Web_To_Lead_Form_html .= "<td width='15%' style='text-align: left; font-size: 12px; font-weight: normal;'>$field_label<span class='required' style='color: rgb(255, 0, 0);'>$web_required_symbol</span></td>";
                } else {
                    $Web_To_Lead_Form_html .= "<td width='15%' style='text-align: left; font-size: 12px; font-weight: normal;'>$field_label</td>";
                }

                if (isset($field_defs[$colsField]['isMultiSelect']) && $field_defs[$colsField]['isMultiSelect'] == 1) {
                    $Web_To_Lead_Form_html .= "<td width='35%' style='font-size: 12px; font-weight: normal;'><select id=$field_name multiple='true' name=$field_name tabindex='1'>$lead_options</select></td>";
                } elseif ($this->ifRadioButton($field_defs[$colsField]['name'])) {
                    $Web_To_Lead_Form_html .="<td width='35%' style='font-size: 12px; font-weight: normal;'>";

                    foreach($app_list_strings[$field_options] as $field_option){
                        $Web_To_Lead_Form_html .="<input id='$colsField"."_$field_option' name='radio_grp_$colsField' value='$field_option' type='radio'>";
                        $Web_To_Lead_Form_html .="<span ='document.getElementById('".$field_defs[$colsField]."_$field_option').checked =true style='cursor:default'; onmousedown='return false;'>$field_option</span><br>";
                    }
                    $Web_To_Lead_Form_html .="</td>";
                } else {
                    $Web_To_Lead_Form_html .= "<td width='35%' style='font-size: 12px; font-weight: normal;'><select id=$field_name name=$field_name tabindex='1'>$lead_options</select></td>";
                }
            }

            if ($field_type == 'bool') {
                if ($field_required){
                    $Web_To_Lead_Form_html .= "<td width='15%' style='text-align: left; font-size: 12px; font-weight: normal;'>$field_label<span class='required' style='color: rgb(255, 0, 0);'>$web_required_symbol</span></td>";
                } else {
                    $Web_To_Lead_Form_html .= "<td width='15%' style='text-align: left; font-size: 12px; font-weight: normal;'>$field_label</td>";
                }

                $Web_To_Lead_Form_html .= "<td width='35%' style='font-size: 12px; font-weight: normal;'><input type='checkbox' id=$field_name name=$field_name></td>";
            }

            if (in_array($field_type, $text_types)) {
				if ($field_name=='last_name' ||   $field_required){
                    $Web_To_Lead_Form_html .= "<td width='15%' style='text-align: left; font-size: 12px; font-weight: normal;'>$field_label<span class='required' style='color: rgb(255, 0, 0);'>$web_required_symbol</span></td>";
                } else {
                    $Web_To_Lead_Form_html .= "<td width='15%' style='text-align: left; font-size: 12px; font-weight: normal;'>$field_label</td>";
                }

                $Web_To_Lead_Form_html .= "<td width='35%' style='font-size: 12px; font-weight: normal;'><input id=$field_name name=$field_name type='text'></td>";
			}

			if ($field_type == 'date') {
				$Web_To_Lead_Form_html .= "<td width='15%' style='text-align: left; font-size: 12px; font-weight: normal;'>$field_label</td>";
				$Web_To_Lead_Form_html .= "<td width='35%' style='font-size: 12px; font-weight: normal;'><input id=$field_name name=$field_name type='text'> <br><span style='font-size:85%'>$weblead_date_format</span></td>";
			}

        } else {
            $Web_To_Lead_Form_html .= "<td width='15%' style='text-align: left; font-size: 12px; font-weight: normal;'>&nbsp</td>";
            $Web_To_Lead_Form_html .= "<td width='35%' style='font-size: 12px; font-weight: normal;'>&nbsp</td>";
        }

        return $Web_To_Lead_Form_html;
    }

    /**
     * Save generated form and render Download Web-to-Lead form page
     *
     * @return string
     */
    function renderDownloadView() {
    	require_once('modules/Campaigns/WebToLeadForm.php');
        global $mod_strings;

        $savetime = time();

        if(! empty($_REQUEST['body_html'])) {
            $dir_path = CacheManager::get_location("generated_forms/", true);

            if(!file_exists($dir_path)) {
                mkdir($dir_path, 0777);
            }

            $file = $dir_path .'WebToLeadForm_'. $savetime .'.html';
            $fp = fopen($file, 'wb');

            $html = from_html($_REQUEST['body_html']);
	        $with_captcha = strpos($html, '_CAPTCHA_') !== false;

	        if ($with_captcha)
		        $html = str_replace('_CAPTCHA_', $this->captcha->getPageCode(), $html);

	        //Add form to DB and add token
	        $form = new WebToLeadForm();
	        $token = $form->add($with_captcha);
	        $html = str_replace('_TOKEN_', $token, $html);

	        $html = str_replace('</body>', $this->getFormJs() . '</body>', $html);

			fwrite($fp, $html);
            fclose($fp);
        }

        $webformlink = "<b>{$mod_strings['LBL_DOWNLOAD_TEXT_WEB_TO_LEAD_FORM']}</b><br/><br />";
        $webformlink .= "<a href='{$dir_path}WebToLeadForm_{$savetime}.html' target='_blank'>{$mod_strings['LBL_DOWNLOAD_WEB_TO_LEAD_FORM']}</a>";

		$body = <<<EOQ
<div class="form-section">
	<div class="form-row">
		<div class="form-cell column sm-12">
			{$webformlink}
		</div>
	</div>
</div>
EOQ;

        return $body;
    }

    /**
     * Get Web-to-Lead Form javascript
     *
     * @return string
     */
    function getFormJs() {
        global $mod_strings;
        $web_form_required_fileds_msg = $mod_strings['LBL_PROVIDE_WEB_TO_LEAD_FORM_FIELDS'];

        $js = "<script type='text/javascript'>
			var OCRM_EX1 = '_1crm_ex1', OCRM_EX2 = '_1crm_ex2';
			function check_webtolead_fields(form) {
                if(document.getElementById('req_id') != null) {
                    var reqs = document.getElementById('req_id').value;
                    reqs = reqs.substring(0,reqs.lastIndexOf(';'));
                    var req_fields = reqs.split(';');
                    nbr_fields = req_fields.length;
                    var req = true;
                    for(var i=0;i<nbr_fields;i++){
                        if (document.getElementById(req_fields[i]).value.length <= 0) {
                            req = false;
                            break;
                        }
                    }
                    if(req) {
                        if (typeof captchaSubmit == 'function') { 
                            captchaSubmit(); 
                        } else {
                        	form.submit();                            
                        }
                        return true;
                    } else {
                        alert('$web_form_required_fileds_msg');
                        return false;
                    }
                } else {
                    if (typeof captchaSubmit == 'function') { 
                        captchaSubmit(); 
                    } else {
                    	form.submit();                        
                    }
                }
            }
;
            function assignFormFields() {
				var params = window.location.search.substr(1).split('&');
				var ex1 = 'NaN', ex2 = 'NaN';
				console.debug(params);
				for (var i= 0; i < params.length; i++) {
					if (params[i] === '')
						continue;
                    var param = params[i].split('=');
					var name = decodeURIComponent(param[0]);
					if (param.length == 1) {
						var extra_parts = param[0].split('_');
						ex1 = extra_parts[0];
						ex2 = '';
						if (extra_parts.length > 1)
							ex2 = extra_parts[1];
						ex1 = '' + parseInt(ex1);
						ex2 = '' + parseInt(ex2);
					} else if (param.length != 2) {
					}
                    var value = decodeURIComponent(param[1]).replace(/\+/g, ' ');
                    if (name === 'isDuplicate') {
                        document.getElementById('error_message_row').style.display = '';
                        document.getElementById('error_message').innerHTML = document.getElementById('error_message_text').value;
                    }
                    var input = document.getElementsByName(name)[0];
                    if (!input) continue;
                    input.value = value;
                }
				track_ex(ex1, ex2);
			};

			function track_ex(ex1, ex2) {
				var d = new Date();
				d.setTime(d.getTime() + 365*24*60*60*1000 );
				if (ex1 == 'NaN')
					ex1 = '' + parseInt(getCookie(OCRM_EX1));
				if (ex2 == 'NaN')
					ex2 = '' + parseInt(getCookie(OCRM_EX2));
				if (ex1 != 'NaN') {
					addInput(OCRM_EX1, '_ex1', ex1);
					document.cookie = OCRM_EX1 + '=' + ex1 + '; path=/; expires='+d.toUTCString();
				}
				if (ex2 != 'NaN') {
					addInput(OCRM_EX2, '_ex2', ex2);
					document.cookie = OCRM_EX2 + '=' + ex2 + '; path=/; expires='+d.toUTCString();
				}
			};

			function addInput(id, name, value) {
				var input = document.getElementById(id);
				if (!input) {
					var form = document.forms['WebToLeadForm'];
					input = document.createElement('input');
					input.setAttribute('type', 'hidden');
					input.setAttribute('value', value);
					input.setAttribute('name', name);
					form.appendChild(input);
				}
				input.setAttribute('value', value);
			};

			function getCookie(name) {
				var matches = document.cookie.match(new RegExp('(?:^|; )' + name.replace(/([\.$?*|{}\(\)\[\]\\\/\+^])/g, '\\$1') + '=([^;]*)'));
				return matches ? decodeURIComponent(matches[1]) : undefined;
			}

            assignFormFields();
        </script>";

        return $js;
    }


    /**
     * Get Lead fields list
     *
     * @return array
     */
    function getLeadFields() {
        global $app_strings;

        $lead = new ModelDef('Lead');
        $field_defs = $lead->getFieldDefinitions();
        $fields = array();

        foreach ($field_defs as $field_def) {

            if (preg_match('/category(\d+)/', $field_def['name']) || $field_def['name'] == 'category') {
                continue;
            }

            if( ( $field_def['type'] == 'ref' && empty($field_def['custom_type']) )
                || $field_def['type'] == 'assigned_user' || $field_def['type'] == 'link'
                || (isset($field_def['source'])  && $field_def['source'] == 'non-db') || $field_def['type'] == 'id') {

                continue;
            }

            if($field_def['name'] == 'deleted' || $field_def['name'] == 'converted' || $field_def['name'] == 'date_entered'
                || $field_def['name'] == 'date_modified' || $field_def['name'] == 'modified_user_id'
                || $field_def['name'] == 'assigned_user_id' || $field_def['name'] == 'created_by'
                || $field_def['name'] == 'team_id' || $field_def['name'] == 'name') {

                continue;
            }

            if(isset($field_def['vname']))
                $field_def['vname'] = preg_replace('/:$/','',translate($field_def['vname'],'Leads'));
            else
                $field_def['vname'] = $field_def['name'];

            $col_arr = array();
            if((isset($field_def['required']) && $field_def['required'] != null && $field_def['name'] != 'status') || $field_def['name'] == 'last_name') {
                $cols_name = $field_def['vname'] .' '. $app_strings['LBL_REQUIRED_SYMBOL'];
                $col_arr[0] = $field_def['name'];
                $col_arr[1] = $cols_name;
                $col_arr[2] = true;
            } else {
                $cols_name = $field_def['vname'];
                $col_arr[0] = $field_def['name'];
                $col_arr[1] = $cols_name;
            }

            if (! in_array($cols_name, $fields))
                array_push($fields, $col_arr);
        }

        return $fields;
    }

    /**
     * Check if custom field radiobutton or not
     *
     * @param string $custom_field_name
     * @return null|array
     */
    function ifRadioButton($custom_field_name) {
        $cust_row = null;
        $lq = new ListQuery('DynField', array('id', 'data_type'));
        $lq->addSimpleFilter('name', $custom_field_name);
        $result = $lq->runQuerySingle();

        if (! $result->failed) {
            if($result->row != null && $result->row['data_type'] == 'radioenum') {
                $cust_row = $result->row;
            }
        }

        return $cust_row;
	}

	static function buttons($layout) {
		if ($layout == 'WordpressExport' || $layout == 'WebToLeadSave')
			return [];
		global $app_strings, $mod_strings;
		if ($layout == 'WebToLeadGen') {
			return [
				[
					'perform' => "this.form.layout.value='WebToLeadSave';",
					'vname' => $app_strings['LBL_SAVE_WEB_TO_LEAD_FORM'],
				],
				[
                	'perform' => "return SUGAR.ui.cancelEdit(document.forms.DetailForm);",
					'vname' => $app_strings['LBL_CANCEL_BUTTON_LABEL'],
				]
			];
		}
		return [
			[
				'perform' => "dragDropAllFields('{$app_strings['LBL_ADD_ALL_LEAD_FIELDS']}','{$app_strings['LBL_REMOVE_ALL_LEAD_FIELDS']}'); return false;",
				'id' => 'lead_add_remove_button',
				'vname' => $app_strings['LBL_ADD_ALL_LEAD_FIELDS'],
				'icon' => 'icon-add',
			],
			[
	            'perform' => 'return SUGAR.ui.cancelEdit(document.forms.DetailForm);',
				'vname' => $app_strings['LBL_CANCEL_BUTTON_LABEL'],
				'icon' => 'icon-cancel',
			],
			[
				'id' => 'lead_next_button',
	            'perform' => "askLeadQ('next','{$mod_strings['LBL_SELECT_REQUIRED_LEAD_FIELDS']}','{$mod_strings['LBL_SELECT_LEAD_FIELDS']}'); return false;",
				'vname' => $app_strings['LBL_NEXT_BUTTON_LABEL'],
				'icon' => 'icon-right',
			],
			[
				'id' => 'lead_back_button',
	            'perform' => "askLeadQ('back'); return false;",
				'vname' => $app_strings['LBL_BACK'],
				'style' => 'display: none',
				'icon' => 'icon-left',
			],
			[
				'id' => 'lead_form_button',
	            'perform' => "this.form.layout.value='WebToLeadGen';",
				'vname' => $app_strings['LBL_GENERATE_WEB_TO_LEAD_FORM'],
				'style' => 'display: none',
				'icon' => 'icon-accept',
			],
			[
				'id' => 'lead_export_button',
	            'perform' => "this.form.layout.value='WordpressExport';",
				'vname' => $app_strings['LBL_EXPORT_TO_WORDPRESS'],
				'style' => 'display: none',
				'icon' => 'icon-accept',
			]
		];
		/*
            <input title="{$app_strings['LBL_GENERATE_WEB_TO_LEAD_FORM']}" class="input-button" onclick="this.form.layout.value='WebToLeadGen';return addGrids('DetailForm');" type="submit" name="button" value="{$app_strings['LBL_GENERATE_WEB_TO_LEAD_FORM']}" />
            <input title="{$app_strings['LBL_EXPORT_TO_WORDPRESS']}" class="input-button" onclick="this.form.layout.value='WordpressExport';return addGrids('DetailForm');" type="submit" name="button" value="{$app_strings['LBL_EXPORT_TO_WORDPRESS']}" />
		 */
	}
}

?>
