<?php

require_once 'modules/Quotes/PDFTemplatesProvider.php';

class WorkOrderPDFTemplateProvider extends QuotePDFTemplateProvider
{
	
	protected function __construct($type)
	{
		parent::__construct($type);
	}

	public static function template_types()
	{
		return array(
			'work_order' => translate('LBL_PDF_ORDER', 'Cases'),
		);
	}

	public static function template_skeleton($type)
	{
		$ret = array('sections' => array(
			array(
				'name' => 'header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_HEADER',
				'height' => 150,
				'elements' => array (),
			),
			array(
				'name' => 'hours_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_HOURS_HEADER',
				'height' => 150,
				'elements' => array (),
			),
			array(
				'name' => 'hours',
				'type' => 'table',
				'title' => 'LBL_SKEL_HOURS',
				'columns' => array (),
			),
			array(
				'name' => 'parts_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_PARTS_HEADER',
				'height' => 150,
				'elements' => array (),
			),
			array(
				'name' => 'parts',
				'type' => 'table',
				'title' => 'LBL_SKEL_PARTS',
				'columns' => array (),
			),
			array(
				'name' => 'footer',
				'data_id' => 'header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_FOOTER',
				'height' => 40,
				'elements' => array (),
			),
		));
		foreach ($ret['sections'] as &$sec) $sec['title'] = translate($sec['title'], 'Cases');
		$ret['sections'] = array_values($ret['sections']);
		return $ret;
	}

	public static function template_sample_data($type)
	{
		$datefmt = AppConfig::setting('locale.defaults.date_format');
		$data = array(
			'header' => array(
				'todays_date' => array(translate('LBL_TODAYS_DATE'), date($datefmt), ),
				'number' => array(translate('LBL_SAMPLE_HEADER_NUMBER_LABEL', 'Cases'), 'P2013-001'),
				'date' => array(translate('LBL_SAMPLE_HEADER_DATE_LABEL', 'Cases'), '2014-11-05'),
				'contact' => array(translate('LBL_SAMPLE_HEADER_CONTACT_LABEL', 'Cases'), translate('LBL_SAMPLE_HEADER_CONTACT_VALUE', 'Cases'), ),
				'contact_phone' => array(translate('LBL_SAMPLE_HEADER_CONTACT_PHONE_LABEL', 'Cases'), '123-45-898'),
				'cust_req_number' => array(translate('LBL_SAMPLE_HEADER_CUST_REQ_NUMBER_LABEL', 'Cases'), '123456789'),
				'description' => array(translate('LBL_SAMPLE_HEADER_DESCRIPTION_LABEL', 'Cases'), translate('LBL_SAMPLE_HEADER_DESCRIPTION_VALUE', 'Cases'), ),
				'standard_terms' => array(translate('LBL_SAMPLE_HEADER_STANDARD_TERMS_LABEL', 'Cases'), ''),
				'address' => array(translate('LBL_SAMPLE_HEADER_ADDRESS_LABEL', 'Cases'), translate('LBL_SAMPLE_HEADER_ADDRESS_VALUE', 'Cases'), ),
				'case_subject' => array(translate('LBL_SAMPLE_HEADER_CASE_SUBJECT_LABEL', 'Cases'), translate('LBL_SAMPLE_HEADER_CASE_SUBJECT_VALUE', 'Cases'), ),
			),
			'hours' => array(
				'number' => array(translate('LBL_SAMPLE_HOURS_NUMBER_LABEL', 'Cases'), '1'),
				'subject' => array(translate('LBL_SAMPLE_HOURS_SUBJECT_LABEL', 'Cases'), translate('LBL_SAMPLE_HOURS_SUBJECT_VALUE', 'Cases'), ),
				'hours' => array(translate('LBL_SAMPLE_HOURS_HOURS_LABEL', 'Cases'), '3'),
				'status' => array(translate('LBL_SAMPLE_HOURS_STATUS_LABEL', 'Cases'), translate('LBL_SAMPLE_HOURS_STATUS_VALUE', 'Cases'), ),
				'booking_class' => array(translate('LBL_SAMPLE_HOURS_BOOKING_CLASS_LABEL', 'Cases'), translate('LBL_SAMPLE_HOURS_BOOKING_CLASS_VALUE', 'Cases'), ),
				'date' => array(translate('LBL_SAMPLE_HOURS_DATE_LABEL', 'Cases'), '2014-11-05'),
				'user' => array(translate('LBL_SAMPLE_HOURS_USER_LABEL', 'Cases'), translate('LBL_SAMPLE_HOURS_USER_VALUE', 'Cases'), ),
			),
			'parts' => array(
				'number' => array(translate('LBL_SAMPLE_PARTS_NUMBER_LABEL', 'Cases'), '1'),
				'name' => array(translate('LBL_SAMPLE_PARTS_NAME_LABEL', 'Cases'), translate('LBL_SAMPLE_PARTS_NAME_VALUE', 'Cases'), ),
				'category' => array(translate('LBL_SAMPLE_PARTS_CATEGORY_LABEL', 'Cases'), translate('LBL_SAMPLE_PARTS_CATEGORY_VALUE', 'Cases'), ),
				'type' => array(translate('LBL_SAMPLE_PARTS_TYPE_LABEL', 'Cases'), translate('LBL_SAMPLE_PARTS_TYPE_VALUE', 'Cases'), ),
				'price' => array(translate('LBL_SAMPLE_PARTS_PRICE_LABEL', 'Cases'), '125.00'),
			),
			'hours_header' => array(
			),
			'parts_header' => array(
			),
		);
		$ret = array();
		foreach ($data as $k => $v) {
			$ret[$k] = array();
			foreach ($v as $kk => $vv) {
				$ret[$k][$kk] = array('label' => $vv[0], 'value' => $vv[1]);
			}
		}
		return $ret;
	}


	public static function create($type)
	{
		return new WorkOrderPDFTemplateProvider($type);
	}
	
	public function iterate(PDFTemplatePrinter $p)
	{
		$this->iterateHeaderFooter($p, 'header');
		$p->renderSection($this->findSection('hours_header'), array());
		$p->renderSection($this->findSection('hours'), $this->data['hours']);
		$p->renderSection($this->findSection('parts_header'), array());
		$p->renderSection($this->findSection('parts'), $this->data['parts']);
		$this->iterateHeaderFooter($p, 'footer');
	}

	protected function get_service_parts_lines(&$items)
	{
		$data = array();
		$num = 1;

		foreach($items->getRowResults() as $row) {
			$datarow = array(
				'number' => $num ++,
				'name' => $row->getField('name', '', true),
				'category' => $row->getField('product_category.name', '', true),
				'type' => $row->getField('product_type.name', '', true),
				'price' => $this->currency_format($row->getField('purchase_price'), $row->getField('currency_id'), true),
			);
			
			$data[] = $datarow;
		}
		
		return $data;
	}	
	protected function get_booked_lines(&$items)
	{
		$data = array();
		$num = 1;
		$bk_cls = AppConfig::setting("lang.lists.{$this->use_language}.app.booked_hours_related_dom", array());

		foreach($items->getRowResults() as $row) {
			$datarow = array(
				'number' => $num ++,
				'subject' => $row->getField('name', '', true),
				'hours' => $row->getField('quantity', '-', true),
				'status' => ucwords($row->getField('status')),
				'booking_class' => translate('booked_hours_related_dom', '',  $row->getField('related_type'), true),
				'date' => $row->getField('date_start', '', true),
				'user' => $row->getField('assigned_user.name', '', true),
			);
			
			$data[] = $datarow;
		}
		
		return $data;
	}

	public function initialize($layout, ListQuery $lq)
	{
		$lq->addField('assigned_user.name');
		$lq->addField('product_category.name');
		$lq->addField('product_type.name');

		$lq->addField('assigned_user.name');
		$lq->addField('cust_contact.name');
		$lq->addField('account.name', 'shipping_account_name');
		$lq->addfield('account.shipping_address_street', 'shipping_address_street');
		$lq->addfield('account.shipping_address_city', 'shipping_address_city');
		$lq->addfield('account.shipping_address_state', 'shipping_address_state');
		$lq->addfield('account.shipping_address_postalcode', 'shipping_address_postalcode');
		$lq->addfield('account.shipping_address_country', 'shipping_address_country');

		return parent::initialize($layout, $lq);
	}
	
	public function getFileName() {
		$filename_pfx = translate('LBL_FILENAME_WORK_ORDER', 'Cases');
		$filename = $filename_pfx . "_";
		$filename .= $this->row->getField('case_number') . '_' . $this->row->getField('assigned_user.name');
		$filename = preg_replace('~\.+$~', '', trim($filename));
		$filename = $filename . '.pdf';
		return $filename;

	}


	public function getNoteTitle()
	{
		return translate('LBL_PREPARED_ORDER', 'Cases');
	}

	public function getEmailTitle()
	{
		return translate('LBL_EMAILED_ORDER', 'Cases');
	}

	public function getSignedDocumentTitle() {
		$prefix =
			$this->row
			? '[' . $this->row->getField('case_number') . '] '
			: '';

		return $prefix . $this->getNoteTitle();
	}

	protected function createData()
	{
		global $app_list_strings, $timedate;
		$row =& $this->row;
		$datefmt = AppConfig::setting('locale.defaults.date_format');
		$data = array(
			'todays_date' => date($datefmt),
			'number' => $row->getField('case_number'),
			'date' => $timedate->to_display_date($row->getField('date_entered')),
			'contact' => $row->getField('cust_contact.name'),
			'cust_req_number' => $row->getField('cust_req_no'),
			'contact_phone' => $row->getField('cust_phone_no'),
			'description' => $row->getField('description'),
			'standard_terms' => trim(AppConfig::setting('company.std_case_terms', '')),
			'case_subject' => $row->getField('name'),
			'address' => $this->_address('shipping_'),
		);
		$data = array_merge($this->header_data, $data);
		$this->data['header'] = $data;
		$this->createHoursData();
		$this->createPartsData();
	}

	protected function createHoursData()
	{
		$lq = new ListQuery('aCase', true, array('link_name' => 'booked_hours', 'formatted' => 'pdf', 'use_real_names' => true));
		$lq->setParentKey($this->row->getField('id'));
		$lq->addField('assigned_user.name');
		$res = $lq->runQuery();

		$booked_lines = $this->get_booked_lines($res);
		foreach(range(1,5) as $i)
			$booked_lines[] = array();

		$this->data['hours'] =  $booked_lines;
	}

	protected function createPartsData()
	{
		$data = array();

		$lq = new ListQuery('aCase', true, array('link_name' => 'products', 'formatted' => 'pdf', 'use_real_names' => true));
		$lq->setParentKey($this->row->getField('id'));
		$lq->addField('assigned_user.name');
		$lq->addField('product_category.name');
		$lq->addField('product_type.name');
		$res = $lq->runQuery();

		$parts_lines = $this->get_service_parts_lines($res);
		foreach(range(1,5) as $i)
			$parts_lines[] = array();

		$this->data['parts'] = $parts_lines;
	}

	public static function get_extra_sources() {
		return array(
			array(
				'module' => 'Contacts',
				'model' => 'Contact',
				'prefix' => 'cust_contact',
				'label' => 'LBL_CUST_CONTACT',
			),
			array(
				'module' => 'SubContracts',
				'model' => 'SubContract',
				'prefix' => 'contract',
				'label' => 'LBL_CONTRACT_NAME',
			),
			array(
				'module' => 'Assets',
				'model' => 'Asset',
				'prefix' => 'asset',
				'label' => 'LBL_ASSET',
			),
			array(
				'module' => 'Accounts',
				'model' => 'Account',
				'prefix' => 'account',
				'label' => 'LBL_PRIMARY_ACCOUNT',
			),
		);
	}}
	
