<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
if (!defined('inScheduler')) die('Unauthorized access');


$days_warn = (int)AppConfig::setting('company.case_auto_close_warn');
$days_close = (int)AppConfig::setting('company.case_auto_close_close');

$do_close = $days_close > 0;
$do_warn = ($days_warn > 0) && (($days_close < 1) || ($days_close > $days_warn));

if (!$do_warn && !$do_close)
	return;

$date_warn = gmdate('Y-m-d H:i:s', strtotime("now - {$days_warn} days"));
$date_close = gmdate('Y-m-d H:i:s', strtotime("now - {$days_close} days"));

$lq = new ListQuery('aCase');
$lq->addFields(array('id', 'status', 'assigned_user_id', 'inactivity_start_date', 'inactivity_warn_date'));
$lq->addSimpleFilter('status', 'Pending');
$lq->addFilterClause('inactivity_start_date IS NOT NULL');

if (!$do_warn)
	$lq->addSimpleFilter('inactivity_start_date', $date_close, '<=');
else
	$lq->addSimpleFilter('inactivity_start_date', $date_warn, '<=');

$result = $lq->runQuery(0, null, false, null, 20);

while(! $result->failed) {
	foreach($result->getRowIndexes() as $idx) {
		$row = $result->getRowResult($idx);
		$last_activity = $row->getField('inactivity_start_date');
		$warned = $row->getField('inactivity_warn_date');
		if ($do_close && $last_activity <= $date_close)
			close_expired_case($row);
		elseif ($do_warn && ! $warned && $last_activity <= $date_warn)
			warn_expired_case($row);
	}
	if($result->page_finished)
		break;
	$lq->pageResult($result);
}


function warn_expired_case($row) {
	require_once('include/Notify/NotifyManager.php');

	$upd = RowUpdate::for_result($row);
	$upd->set('inactivity_warn_date', gmdate('Y-m-d H:i:s'));
	$upd->save();
	
	$targets = new NotifyTargets();
	$cs = $row->getLinkResults('contacts', true);
	if($cs) {
		foreach($cs->rows as $contact) {
			$contact['bean_name'] = 'Contact';
			$targets->addTargetInfo($contact);
		}
	}
	NotifyManager::record_notify_targets($row->base_model, 'expire_warn', $row->getPrimaryKeyValue(), $targets);

	if (1) {
		require_once('modules/ActivityLog/ActivityLog.php');
		$log = new ActivityLog();
		$log->addActivity($upd, 'expire_warn');
		$log->deleteOldActivities();
	}
}

function close_expired_case($row) {
	$upd = RowUpdate::for_result($row);
	$upd->set('status', 'Closed - Expired');
	$upd->save();
}

