<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/
/*********************************************************************************

 * Description:  returns HTML for client-side image map.
 * Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
 * All Rights Reserved.
 * Contributor(s): ______________________________________..
 ********************************************************************************/

require_once('include/charts/BaseChart.php');

class Chart_funnel_opportunities extends BaseChart
// almost exact copy of Chart_won_opportunities
{
	var $is_sidebar = false;
	var $modules = array('Opportunities');
	var $default_title = 'LBL_FUNNEL_OPPS_TITLE';
	var $chart_type = 'pie';
	var $dashletIcon = 'Leads';

	var $user_ids;
	var $date_start;
	var $date_end;
	var $lead_sources;

	var $chart_select_options = array('user_ids');
	var $chart_date_options = array('date_start', 'date_end');

	function init_chart_options($options) {
		global $current_user, $timedate;
		$this->user_ids = array_get_default($options, 'user_ids');

		if($this->is_sidebar)
			$this->user_ids = array($current_user->id);
		else
			$this->user_ids = array_get_default($options, 'user_ids');

		$this->date_start = array_get_default($options, 'date_start');
		if(! $this->date_start)
			$this->date_start = date($timedate->dbDayFormat, time());

		$this->date_end = array_get_default($options, 'date_end');
		if(! $this->date_end)
			$this->date_end = date($timedate->dbDayFormat, strtotime('now +1 year'));
		$entropy = $this->create_options_hash($options);
		$this->cache_filename = filename_safe_string(implode('_', array(
			$entropy, 'pipeline_funnel',
			date('Y-m-d'),
		)));
		$svg = $this->svg_charts_enabled();
		$this->cache_filename .= ($svg ? '.svg' : '.xml');
		$GLOBALS['log']->debug("cache file name is: $this->cache_filename");
	}


	function chart_display_options() {
		$lbl_users = translate('LBL_USERS', 'Charts');

        return array(
            'labels' => array(
                'user_ids' => $lbl_users
            ),
            'spec' => array(
                'user_ids' => array(
                	'type' => 'multienum',
                	'options' => get_user_array(false),
                	'width' => 25,
					'ref_spec' => array(
						'icon' => 'theme-icon module-Users',
						'placeholder' => translate('LBL_SELECT_USER_PLACEHOLDER', 'app'),
						'bean_name' => 'User',
						'popup_return_multiple' => 1,
					),
				),
				'date_start' => array('type' => 'date'),
				'date_end' => array('type' => 'date'),
            ),
            'values' => array(
				'user_ids' => $this->user_ids,
				'date_start' => $this->date_start,
				'date_end' => $this->date_end,
            )
        );
	}


	function get_footer_text() {
		global $timedate;
		$start = $timedate->to_display_date($this->date_start);
		$end = $timedate->to_display_date($this->date_end);
		return translate('LBL_OPP_BY_SALES_STAGE_FORM_DESC', 'Charts') . ' [' . $start . ' - ' . $end . ']';
	}

	function gen_data()
	{
		global $db, $app_strings, $current_user, $app_list_strings;

		$vars = array(
			'legends',
			'total',
			'subtitle',
			'leadSourceArr',
			'selectedTotal',
		);
		$cache_file_name = $this->get_cache_file();

		$sources = $this->lead_sources ? $this->get_selected_values($this->lead_sources, 'lead_source_dom') : array();

		require_once('modules/Currencies/Currency.php');
		$legends = $app_list_strings['sales_stage_dom'];
		unset($legends['Closed Lost']);

		$user_id = $this->user_ids ? $this->get_selected_values($this->user_ids, get_user_array(false)) : array();

		$lq = new ListQuery('Opportunity');
		$lq->primary_added = true;
		$lq->addField('sales_stage');
		$lq->addFieldLiteral('opp_count', 'count(*)', 'int');
		$lq->addFieldLiteral('total', 'sum(amount_usdollar/1000)', 'double');
		if(is_countable($sources) && count($sources))
			$lq->addSimpleFilter('lead_source', array_keys($sources));
		if(is_countable($user_id) && count($user_id))
			$lq->addSimpleFilter('assigned_user_id', array_keys($user_id));

		$lq->addFilterClause(array(
			'field' => 'date_closed',
			'operator' => 'between',
			'value' => $this->date_start,
			'end' => $this->date_end,
		));

		$lq->addAclFilters('report');
		$lq->setGroupBy('sales_stage');
		$lq->setOrderBy('total');
		$query = $lq->getSql();

		$selectedTotal = 0;

		//build pipeline by lead source data
		$total = 0;
		$div = 1;
		$symbol = AppConfig::setting('locale.base_currency.symbol');
		if($current_user->getPreference('currency') ) {
			require_once('modules/Currencies/Currency.php');
			$currency = new Currency();
			$currency->retrieve($current_user->getPreference('currency'));
			$div = $currency->conversion_rate;
			$symbol = $currency->symbol;
		}
		$subtitle = translate('LBL_OPP_SIZE', 'Charts').' '.$symbol.'1'.translate('LBL_OPP_THOUSANDS', 'Charts');

		$result = $db->query($query) or sugar_die("Error performing query: " . $db->last_error);
		$leadSourceArr =  array();
		while($row = $db->fetchByAssoc($result, -1, false))
		{
			$leadSource = $row['sales_stage'];
			if(! isset($legends[$leadSource]))
				continue;
			if($row['total']*$div<=100){
				$sum = round($row['total']*$div, 2);
			} else {
				$sum = round($row['total']*$div);
			}

			$leadSourceArr[$leadSource]['opp_count'] = $row['opp_count'];
			$leadSourceArr[$leadSource]['sum'] = $sum;
		}

		return compact($vars);
	}

	function gen_xml() {
		return false;
	}

	function gen_svg()
	{
		global $app_strings, $current_user, $app_list_strings;
		$cache_file_name = $this->get_cache_file();
		if (!file_exists($cache_file_name) || $this->refresh) {
			extract($this->gen_data());
			require_once 'include/SVGCharts/Funnel.php';
			require_once('include/SVGCharts/impl/SVGChartData.php');
			$data = new SVGChartData;
			$size = $this->get_chart_info();
			$w = $size[1];
			$chart = new SVGChartFunnel($w, $w/2);
			$this->init_svg_chart($chart);

			$series = array();
			$seriesLabels = array();
			$labels = array();
			$seriesVars = array();


			foreach ($legends as $lead_source_key=>$translation) {
				if ($lead_source_key == '') {
					$lead_source_key = translate('NTC_NO_LEGENDS', 'Charts');
					$translation = translate('NTC_NO_LEGENDS', 'Charts');
				}
				if(!isset($leadSourceArr[$lead_source_key])) {
					$leadSourceArr[$lead_source_key] = array('opp_count' => 0, 'sum' => 0);
				}
				$series['points'][$lead_source_key] = array(
					'n' => $leadSourceArr[$lead_source_key]['sum'],
					'c' => format_number($leadSourceArr[$lead_source_key]['opp_count'], 0, 0),
					//currency_format_number($leadSourceArr[$lead_source_key]['sum'], array('currency_symbol' => false))

				);
				$labels[$lead_source_key] = $translation;
				if(isset($leadSourceArr[$lead_source_key])){$total += $leadSourceArr[$lead_source_key]['sum'];}
			}

			$title = translate('LBL_TOTAL_PIPELINE', 'Charts').currency_format_number($total, array('currency_symbol' => true, 'symbol_space'=>'')).$app_strings['LBL_THOUSANDS_SYMBOL'];

			$data->addSeries('', $series);
			$data->setLabels($labels);

			$valueRollover = '$count$ '.translate('LBL_OPPS_WORTH', 'Charts'). ' $value$' . translate('LBL_OPP_THOUSANDS', 'Charts') . ' ' . translate('LBL_OPPS_IN_STAGE', 'Charts').' $group1$';
			$chart->setStatus(translate('LBL_ROLLOVER_WEDGE_DETAILS', 'Charts'), $valueRollover, $valueRollover);

			$url = AppConfig::site_url()
				. 'index.php?module=Opportunities&action=index&layout=Standard&view_closed=1&sales_stage=$series$&query=true&searchFormTab=advanced_search';

			if (!empty($this->user_ids)) {
				$url .= '&filter_owner=' . urlencode(implode('#', $this->user_ids));
			} else {
				$url .= '&filter_owner=all';
			}

			$url .= '&date_closed-operator=between_dates&date_closed=' . $this->date_start . '&date_closed-end=' . $this->date_end;

			$chart->setData($data);
			$chart->setTitle($title);
			$chart->setLinkTemplate($url);
			$fileContents = $chart->render();
			$this->save_xml_file($cache_file_name, $fileContents);
		}
		return $cache_file_name;
	}

	function display_chart()
	{
		if (!$this->svg_charts_enabled())
			return '<div style="text-align:center;font-weight:bold">' . translate('LBL_SVG_ONLY', 'Charts') . '</div>';
		return parent::display_chart();
	}
}


?>
