<?php
require_once 'data/SugarBean.php';
class Commission extends SugarBean {
	var $new_schema = true;
	var $object_name = "Commission";
	var $module_dir = "Commissions";

	var $additional_column_fields = Array('sales_order_id');

	static function init_record(RowUpdate &$upd) {
		$lq = new ListQuery('Commission');
		if(AppConfig::setting("company.commission_module") == 'SalesOrders')
			$lq->addSimpleFilter('sales_order_id', $upd->getField('sales_order_id'), '=');
		else if(AppConfig::setting("company.commission_module") == 'Invoices')
			$lq->addSimpleFilter('invoice_id', $upd->getField('invoice_id'), '=');
		$res = $lq->fetchAll();

		$split = 100;
		foreach($res as $commsplit) {
			$split = $split - (float)$commsplit->getField('percentage');
		}

		$upd->set('percentage', $split);
 
		return $split;
	}

	static function CalcCommissions(RowUpdate &$upd) {

		if ($upd->getRelatedData('updating_so_comission')) return;

		switch (AppConfig::setting("company.commission_module")) {
		case 'Invoices':
			$id_field = 'invoice_id';
			$model_name = 'Invoice';
			$profit_field = 'gross_profit';
			break;
		case 'SalesOrders':
			$id_field = 'sales_order_id';
			$model_name = 'SalesOrder';
			$profit_field = 'gross_profit_so';
			break;
		default:
			return;
		}

		$updin = ListQuery::quick_fetch($model_name, $upd->getField($id_field));
		$id = $updin->getPrimaryKeyValue();

		$commperc = (float)AppConfig::setting("company.commission_percentage");
		if (AppConfig::setting("company.commission_type") == 'GP') {
			$grossprofit = $updin->getField($profit_field);
		} else {
			$grossprofit = $updin->getField('pretax');
		}
		$grossprofit = (float)$grossprofit;
		$commval = ($commperc * $grossprofit) / 100;

		$lq = new ListQuery('Commission');
		$lq->addSimpleFilter($id_field, $id, '=');
		$res = $lq->fetchAll();
		$total_perc = 0;
		foreach ($res as $comm_user) {
			$total_perc = $total_perc + (float)$comm_user->getField('percentage');
		}
		foreach($res as $comm_user) {
			$update = array();
			$update['commission'] = (float)($comm_user->getField('percentage') * $commval) / 100;
			$update['commission_total'] = $commval;
			$update['percentage_total'] = $total_perc;
			$comm_user_update = RowUpdate::for_result($comm_user);
            $comm_user_update->set($update);
            $comm_user_update->setRelatedData('updating_so_comission', true);
            $comm_user_update->save();
		}
	}

	static function before_save(RowUpdate &$upd) {
		if ($upd->getRelatedData('updating_so_comission')) return;
		$update = array();
		$user = ListQuery::quick_fetch_row('User', $upd->getField('assigned_user_id'));
		if(AppConfig::setting("company.commission_module") == 'SalesOrders') {
			$so = ListQuery::quick_fetch_row('SalesOrder', $upd->getField('sales_order_id'));
			$update['name'] = $user['user_name'].' - '.$so['prefix'].$so['so_number'];
		} else if(AppConfig::setting("company.commission_module") == 'Invoices') {
			$so = ListQuery::quick_fetch_row('Invoice', $upd->getField('invoice_id'));
			$update['name'] = $user['user_name'].' - '.$so['prefix'].$so['invoice_number'];
		}
		
		$upd->set($update);
	}
}
