<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/**
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 */



require_once 'include/utils/file_utils.php';
require_once 'include/Dropbox/AuthProvider.php';

class Configurator {
	const DROPBOX_KEY = 'qf4h1wb4fdt5qzv';
	const DROPBOX_SECRET = 'jpwfjzq8q6rhrbd';

	var $settings = array();
	var $errors = array('main' => '');

	function loadConfig($standard=false, $flat=true) {
		$cfg = AppConfig::extract_settings(array_values($this->settings), $standard);
		$ret = array();
		if(isset($cfg['backup'])) {
			require_once('modules/Administration/backup_utils.php');
			if(isset($_REQUEST['backup_rsync_path']))
				$cfg['backup']['rsync_path'] = $_REQUEST['backup_rsync_path'];
			if(isset($_REQUEST['backup_mysqldump_path']))
				$cfg['backup']['mysqldump_path'] = $_REQUEST['backup_mysqldump_path'];
			$cfg['backup']['rsync_available'] = isRsyncSupported(array_get_default($cfg['backup'], 'rsync_path')) ? 1 : 0;
			$cfg['backup']['dump_available'] = isDumpSupported(array_get_default($cfg['backup'], 'mysqldump_path')) ? 1 : 0;
			$encSupported =  isBackupEncryptionSupported();
			$cfg['backup']['crypto_available'] = $encSupported === true ? 1 : 0;
			$cfg['backup']['crypto_reason'] = '';
			if ($encSupported !== true)
				$cfg['backup']['crypto_reason'] = describeEncryptionSupportErrors($encSupported);
			if($flat) $ret['backup_rsync_available'] = $cfg['backup']['rsync_available'];
			if($flat) $ret['backup_dump_available'] = $cfg['backup']['dump_available'];
			if($flat) $ret['backup_crypto_available'] = $cfg['backup']['crypto_available'];
			if($flat) $ret['backup_crypto_reason'] = $cfg['backup']['crypto_reason'];
			if (!is_int($cfg['backup']['num_files_to_keep']) || $cfg['backup']['num_files_to_keep'] < 1)
				$cfg['backup']['num_files_to_keep'] = 5;
			if (!is_int($cfg['backup']['num_remote_files_to_keep']) || $cfg['backup']['num_remote_files_to_keep'] < 1)
				$cfg['backup']['num_remote_files_to_keep'] = 5;
		}
		if($flat) {
			$this->flattenConfig($cfg, $cfg2);
			foreach($this->settings as $k => $v)
				$ret[$k] = array_get_default($cfg2, $v);
			$hooks = AppConfig::setting('configurator.hooks.load');
			foreach ($hooks as $hook) {
				call_hook_function($hook, null, [&$ret]);
			}
			return $ret;
		}
		return $cfg;
	}

	function populateDefaults() {
		$cfg = $this->loadConfig(true, true); // load standard, flat
		$upd = array();
		foreach($this->settings as $k => $f) {
			$upd[$f] = array_get_default($cfg, $k);
		}
		return $upd;
	}

	function populateFromPost($fields = array()) {
		$cfg = $this->loadConfig(true, true); // load standard, flat
		$mod = new ConfiguratorModel();
		$defs = $mod->getDisplayFieldDefinitions();

		require_once('include/layout/FieldFormatter.php');
		$fmt = new FieldFormatter('html', 'editview');
		$input = $fmt->unformatRow($fields, $_POST);

		$upd = array();

		if (! empty($_POST['gtm_forget_token']) && ! empty($_POST['online_meeting_app'])) {
			require_once('include/OnlineMeetings/OnlineMeetingsFactory.php');
			try {
				$onlineMeeting = OnlineMeetingsFactory::getInstance($_POST['online_meeting_app']);
				$onlineMeeting->resetAccessToken(true);
			} catch (OnlineMeetingException $e) {
				$GLOBALS['log']->error("Can't instance Online Meeting object {$_POST['online_meeting_app']}: {$e->getMessage()}");
			}
		}

		if (!empty($_POST['module_sequences'])) {
			$seqs = json_decode($_POST['module_sequences'], true);
			if(is_array($seqs)) {
				foreach($seqs as $seq_row) {
					if(! empty($seq_row['updated'])) {
						if(! empty($seq_row['prefix_name']))
							$input[$seq_row['prefix_name']] = $seq_row['prefix'];
						if(! empty($seq_row['sequence_name']))
							$input[$seq_row['sequence_name']] = $seq_row['sequence'];
					}
				}
			}
		}
		foreach($this->settings as $k => $f) {
			$fk = $k;
			$ft = isset($defs[$k]) ? array_get_default($defs[$k], 'type') : null;
			$default = array_get_default($cfg, $k);
			if(isset($input[$fk])) {
				if($f == 'inspector.new_password') {
					if(strlen($input[$fk]))
						$upd['inspector.password'] = md5(AppConfig::unique_key('i@h').$input[$fk]);
				}
				//else if($k == 'company_logo_file' || $k == 'site_top_logo_primary')
				//	continue;
				else if(is_bool($default) || $ft == 'bool')
					$upd[$f] = ! empty($input[$fk]) ? true : false;
				else if(is_int($default) || $ft == 'int')
					$upd[$f] = (int)$input[$fk];
				else
					$upd[$f] = $input[$fk];
			}
		}
		return $upd;
	}

	function saveConfig($fields = array()) {
		$hooks = AppConfig::setting('configurator.hooks.save');
		$upd = $this->populateFromPost($fields);
		$coll = AppConfig::setting('database.primary.collation');
		$fc_period = AppConfig::setting('company.forecast_period');
		$fy_start = AppConfig::setting('company.fiscal_year_start');

		$this->saveImages($upd);
		$this->updateDropbox($upd);
		$hooks = AppConfig::setting('configurator.hooks.save');
		foreach ($hooks as $hook) {
			call_hook_function($hook, null, [&$upd]);
		}


		AppConfig::update_local($upd, true);
		AppConfig::save_local();

		if($coll != AppConfig::setting('database.primary.collation')) {
			// reconnect to use new collation setting (managed on Locale page)
			DBManager::disconnectInstance();
			$GLOBALS['db'] =& DBManager::getInstance();
		}
		if($fc_period != AppConfig::setting('company.forecast_period')
		|| $fy_start != AppConfig::setting('company.fiscal_year_start')) {
			require_once('modules/Forecasts/Forecast.php');
			$forecast = new Forecast;
			$forecast->recreatePull();
		}
	}

	function restoreConfig() {
		$orig = $this->populateDefaults();
		AppConfig::update_local($orig, true);
		AppConfig::save_local();
	}

	function flattenConfig($cfg, &$ret, $prefix='') {
		if(! isset($ret))
			$ret = array();
		foreach($cfg as $cat => $vals) {
			if(! is_array($vals))
				$ret[$prefix.$cat] = $vals;
			else
				$this->flattenConfig($vals, $ret, $prefix.$cat.'.');
		}
	}

	function updateDropbox(&$updates) {
		$authToken = array_get_default($updates, 'backup.dropbox_auth_token');
		if (!$authToken || dropbox_lib_disabled())
			return;
		$updates['backup.dropbox_auth_token'] = '';
		$provider = new DropboxAuthProvider(true);
		$accessToken = $provider->getAccessToken('authorization_code', ['code' => $authToken]);
		$token = $accessToken->getToken();
		$updates['backup.dropbox_token'] = $accessToken->jsonSerialize();
	}

	function saveImages(&$updates) {
		require_once('include/upload_file.php');
		$upload_dir = CacheManager::get_location('images/logos/', true);

		if(! empty($_REQUEST['company_logo_image_clear'])) {
			$updates['company.logo_file'] = AppConfig::setting('company.logo_file', '', true);
		} else {
			$upload_logo = new UploadFile('company_logo_image');
			$upload_logo->set_upload_dir($upload_dir);
			if ($upload_logo->confirm_upload(true)) {
				//$upload_logo->unlink_file('', basename($focus->getLogoPath()));
				$upload_logo->final_move('');
				$updates['company.logo_file'] = $upload_logo->get_upload_path();
			}
		}

		if(! empty($_REQUEST['site_top_logo_image_clear'])) {
			$updates['site.top_logo.primary'] = AppConfig::setting('site.top_logo.primary', '', true);
		} else {
			$upload_logo = new UploadFile('site_top_logo_image');
			$upload_logo->set_upload_dir($upload_dir);
			if ($upload_logo->confirm_upload(true)) {
				//$upload_logo->unlink_file('', basename($focus->getLogoPath()));
				$upload_logo->final_move('');
				$updates['site.top_logo.primary'] = $upload_logo->get_upload_path();
			}
		}
	}

	public static function oauthServerOptions() {
		$strings = AppConfig::setting("lang.lists.current.app.oauth_server_dom");
		$strings['custom'] = AppConfig::host_name();
		return $strings;
	}

}


?>
