<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');

require_once('include/layout/forms/FormGenerator.php');

class SystemURLsEditor {
	public function render() {
		$layout = [
			'type' => 'configform',
			'editable' => true,
			'form_buttons' => [
				'save' => [
					'icon' => 'icon-accept',
					'vname' => 'LBL_SAVE_BUTTON_LABEL',
				],
				'cancel' => [
					'icon' => 'icon-cancel',
					'type' => 'button',
					'vname' => 'LBL_CANCEL_BUTTON_LABEL',
					'perform' => 'return SUGAR.util.loadUrl("index.php?module=Administration&action=index");',
				],
			],
			'sections' => [
				[
					'id' => 'main',
					'vdesc' => 'LBL_URLS_WARNING_DESC',
					'label_position' => 'top',
					'columns' => 1,
					'elements' => [],
				],
			],
		];

		$fields = [
			'default_url' => [
				'vname' => 'LBL_DEFAULT_URL',
				'type' => 'varchar',
				'len' => 255,
				'width' => 30,
				'required' => true,
			],
			'extra_urls' => [
				'vname' => 'LBL_EXTRA_URLS',
				'type' => 'text',
			],
		];

		$result = new RowResult();
		$result->row = [
			'default_url' => AppConfig::setting('site.base_url'),
			'extra_urls' => join("\n", AppConfig::setting('site.extra_urls', [])),
		];
		$result->fields = [];
		foreach($fields as $k => $f) {
			$result->fields[$k] = $f;
			$f['name'] = $k;
			$layout['sections'][0]['elements'][] = $f;
		}

		$gen = FormGenerator::html_form(null, $layout, 'SystemURLsForm');
		$gen->getFormObject()->addHiddenFields(
			['module' => 'Configurator', 'action' => 'SystemURLs', 'save' => '1']
		);
		$gen->renderForm($result);
		$out = $gen->getResult();
		$gen->exportIncludes();
		echo '<div class="detailOuter">' . $out . '</div>';
	}

	public function save() {
		$current = AppConfig::setting("site.base_url");
		$base = $_POST['default_url'];
		$extra = $_POST['extra_urls'];
		$used = [];
		if (!$this->validateURL($base)) {
			$base = $current;
		}
		$used[$base] = true;
		$extra = array_filter(array_map("trim", explode("\n", $extra)));
		$extra = array_filter($extra, function ($url) use (&$used) {
			if ($this->validateURL($url)) {
				$used[$url] = true;
				return true;
			}
			return false;
		});
		if (empty($used[$current])) {
			$extra[] = $current;
		}
		AppConfig::update_local([
			'site.base_url' => $base,
			'site.extra_urls' => $extra,
		]);
		AppConfig::save_local();
		$upd_status = javascript_escape(translate('LBL_SETTINGS_UPDATED'));
		ob_clean();
		echo $upd_status;
		echo '<script nodisplay>SUGAR.util.loadUrl("index.php?module=Administration&action=index", "'.$upd_status.'");</script>';
	}

	private function validateURL($url) {
		$parts = parse_url($url);
		if (!is_array($parts)) return false;
		$scheme = $parts['scheme'] ?? '';
		if ($scheme !== 'http' && $scheme !== 'https') return false;
		if (empty($parts['host'])) return false;
		return true;
	}

	public function process() {
		if (!empty($_POST['save'])) $this->save();
		else $this->render();
	}
}

$editor = new SystemURLsEditor();
$editor->process();
