<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

require_once('include/layout/widgets/DynamicListInput.php');

class ModuleSequenceInput extends DynamicListInput {
	var $data;
	var $seq_mods = [];
	var $seq_opts = [];

	function init($params=null, $model=null) {
		$this->show_delete = false;
		$this->show_handles = false;
		$this->onupdate = 'sequenceUpdated';
		$this->cols = array(
			array(
				'name' => 'module',
				'label' => 'LBL_MODULE',
				'width' => '25%',
				'render_method' => 'renderModule',
			),
			array(
				'name' => 'label',
				'label' => 'LBL_FIELD',
				'width' => '35%',
			),
			array(
				'name' => 'prefix',
				'label' => 'LBL_SEQUENCE_PREFIX',
				'width' => '20%',
				'editable' => true,
				'render_method' => 'renderPrefix',
			),
			array(
				'name' => 'sequence',
				'label' => 'LBL_SEQUENCE_NUMBER',
				'width' => '20%',
				'editable' => true,
				'format' => 'sequence_number',
				//'render_method' => 'renderFilter',
			)
		);
		$this->renderer_class = 'SUGAR.ui.ModuleSequenceRender';
		parent::init($params, $model);
	}
	
	function getSequences() {
		if(! $this->seq_opts) {
			$this->seq_mods = AppConfig::setting('model.index.sequences');
			foreach($this->seq_mods as $mod) {
				$fs = AppConfig::setting('model.fields.'.$mod);
				foreach($fs as $f) {
					$t = array_get_default($f, 'type');
					if($t == 'sequence_number' && ! empty($f['sequence_name']) && array_get_default($f, 'edit_sequence', true)) {
						$module = AppConfig::module_for_model($mod);
						$lbl_key = array_get_default($f, 'vname_sequence', $f['vname']);
						$lbl = translate($lbl_key, array_get_default($f, 'vname_module', $module));
						$pfx = array_get_default($f, 'prefix_name');
						$pfx_seq = ($pfx && isset($fs[$pfx]['sequence_name'])) ? $fs[$pfx]['sequence_name'] : null;
						$this->seq_opts[] = [
							'module' => $module,
							'name' => $f['sequence_name'],
							'label' => $lbl,
							'prefix_name' => $pfx_seq ? 'company_' . $pfx_seq : '',
							'sequence_name' => 'company_' . $f['sequence_name'],
						];
					}
				}
			}
		}
		return $this->seq_opts;
	}
	
	function getRequiredFields() {
		$req = parent::getRequiredFields();
		foreach($this->getSequences() as $row) {
			if($row['prefix_name']) $req[] = $row['prefix_name'];
			$req[] = $row['sequence_name'];
		}
		return $req;
	}
	
	function loadResult(RowResult $row_result, array $context) {
		foreach($this->getSequences() as $row) {
			$row['module_label'] = translate_dd('moduleList', $row['module']);
			$row['prefix'] = $row['prefix_name'] ? $row_result->getField($row['prefix_name']) : '';
			$row['sequence'] = $row_result->getField($row['sequence_name']);
			if(! $row['sequence']) $row['sequence'] = '1';
			$this->data[] = $row;
		}
		array_csort($this->data, 'module_label', 'label');
	}
	
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		$editable = $gen->getLayout()->getDefaultEditable();		
		$form = $gen->getFormObject();

		if($editable) {
			$row_result->assign($this->name, $this->data);
			$ret = parent::renderHtml($gen, $row_result, $parents, $context);
		} else {
			/*$ret = $editor->formatFilters($filters);
			if(! $ret)
				$ret = translate('LBL_NONE', 'app');*/
			$ret = '...';
		}
		
		$layout = $gen->getLayout();
		$layout->addScriptInclude('modules/Configurator/sequences.js');
		$this->exportIncludes($layout);
		return $ret;
	}
}

?>
