<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

require_once('include/layout/forms/FormField.php');

class StorageUtilW extends FormField {
	function GetDirectorySize($path){
	    $bytestotal = 0;
	    $path = realpath($path);
	    if($path!==false && $path!='' && file_exists($path)){
	        foreach(new RecursiveIteratorIterator(new RecursiveDirectoryIterator($path, FilesystemIterator::SKIP_DOTS)) as $object){
	            $bytestotal += $object->getSize();
	        }
	    }
	    return $bytestotal;
	}

	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		require_once('modules/LicenseInfo/LicenseHistory.php');
		$license = new LicenseHistory();
		$hosting_type = $license->retrieve_latest()->hosting_type;
		global $vendor_info, $mod_strings;
		require_once('vendor_info.php');

		$env = PHP_SAPI == 'cli' ? getenv() : $_SERVER;
		$storage_type = array_get_default($env, 'STORAGE_TYPE',  $vendor_info['storage_type']);
		$onecrm_cloud = getenv("ONECRM_CLOUD");
		$storage_total_startup = array_get_default($env, 'STORAGE_TOTAL_STARTUP', $vendor_info['storage_total_startup']);
		if($onecrm_cloud == 1)
			$storage_total_startup = 1;
		$storage_total_startup_plus = array_get_default($env, 'STORAGE_TOTAL_STARTUPP', $vendor_info['storage_total_startupp']);
		if($onecrm_cloud == 1)
			$storage_total_startup_plus = 1;
		$storage_total_professional = array_get_default($env, 'STORAGE_TOTAL_PRO', $vendor_info['storage_total_pro']);
		if($onecrm_cloud == 1)
			$storage_total_professional = 2;
		$storage_total_enterprise = array_get_default($env, 'STORAGE_TOTAL_ENT', $vendor_info['storage_total_ent']);
		if($onecrm_cloud == 1)
			$storage_total_enterprise = 2.5;


		$p = LicenseHistory::weather('PRODUCTS');

		if(in_array('startup', $p))
			$storage_total = $storage_total_startup;
		else if(in_array('startup_plus', $p))
			$storage_total = $storage_total_startup_plus;
		else if(in_array('infoathand', $p))
			$storage_total = $storage_total_professional;
		else if(in_array('enterprise', $p))
			$storage_total = $storage_total_enterprise;
		else
			$storage_total = $storage_total_startup;

		$cloud = '';

		if($onecrm_cloud == 1) {
			$cloud = $mod_strings['LBL_CLOUD_USER'];
			$limit = LicenseHistory::weather('USERS');
			$totalspace = $limit * $storage_total;
			$usedspace = AppConfig::setting('misc.current_disk_space_usage');
			$freespace = $totalspace-$usedspace;
		} else {
			if($storage_type == 'Per User') {
				$cloud = $mod_strings['LBL_CLOUD_USER'];
				$totalspace = $storage_total*LicenseHistory::weather('USERS');
				$usedspace = AppConfig::setting('misc.current_disk_space_usage');
				$freespace = $totalspace-$usedspace;
			} else if($storage_type == 'Per Instance') {
				$cloud = $mod_strings['LBL_CLOUD_USER'];
				$totalspace = $storage_total;

				$usedspace = AppConfig::setting('misc.current_disk_space_usage');
				$freespace = $totalspace-$usedspace;
			} else {
				$freespace = disk_free_space("./");
				$totalspace = disk_total_space("./");

				$freespace = round($freespace/1024/1024/1024,2);
				$totalspace = round($totalspace/1024/1024/1024,2);
				$usedspace = $totalspace-$freespace;
			}
		}

		if($this->name == 'total_storage_display')
			$html = $totalspace.' GB '.$cloud;
		else if($this->name == 'used_storage_display')
			$html = $usedspace.' GB'.' / '.round(($usedspace/$totalspace*100),2).'%';
		else if($this->name == 'free_storage_display')
			$html = $freespace.' GB';
		else if($this->name == 'backups_storage_display')
			$html = AppConfig::setting('misc.current_disk_space_usage_backups').' MB';
		else if($this->name == 'email_storage_display')
			$html = AppConfig::setting('misc.current_disk_space_usage_email').' MB';
		else if($this->name == 'images_storage_display')
			$html = AppConfig::setting('misc.current_disk_space_usage_images').' MB';
		else if($this->name == 'reports_storage_display')
			$html = AppConfig::setting('misc.current_disk_space_usage_reports').' MB';
		else if($this->name == 'upload_storage_display')
			$html = AppConfig::setting('misc.current_disk_space_usage_upload').' MB';
		else if($this->name == 'db_storage_display')
			$html = AppConfig::setting('misc.current_db_usage_upload').' MB';
		else
			$html = '';

		return $html;
	}
}

?>
