<?php

require_once 'include/upload_file.php';
require_once 'include/GoogleSync.php';
require_once 'modules/Contacts/Contact.php';
require_once 'modules/Accounts/Account.php';

class GoogleContactsSync extends GoogleSync
{
	
	protected function getConfigKey()
	{
		return 'google_contacts';
	}

	protected function getDirection()
	{
		$dir = $this->getPreference('direction');
		return $dir === '1' ? 'to_google' : $dir;
	}
	
	protected function getService()
	{
		$service = new \Google\Service\PeopleService($this->client);
		return $service;
	}

	protected function getUpdated($item) {
		/* @var $item \Google\Service\PeopleService\Person */
		$metadata = $this->getContactMetadata($item->getMetadata());
		return $metadata->getUpdateTime();
	}

	protected function getQuery()
	{
		$query = array(
			'showDeleted' => true,
			'timeZone' => 'UTC',
			'orderBy' => 'updated'
		);
		return $query;
	}

	protected function createIAHItem($sql_row = null)
	{
		$id = null;
		if (is_array($sql_row))
			$id = array_get_default($sql_row, 'id');
		if ($id) {
			$lq = new ListQuery('Contact', true, array('filter_deleted' => false));
			$lq->addField('primary_account');
			$row = $lq->queryRecord($id);
			if (!$row || $row->failed)
				$row = null;
			if ($row)
				return RowUpdate::for_result($row);
		}
		$ret = RowUpdate::blank_for_model('Contact');
		$ret->set('assigned_user_id', AppConfig::current_user_id());
		return $ret;
	}

	protected function setIAHItemFields($iahItem, $item)
	{
		/* @var $item \Google\Service\PeopleService\Person */

		// Names
		$first = $last = '';
		if (count($item->getNames()) > 0) {
			$names = $item->getNames();
			$name = $names[0];
			/* @var $name Google\Service\PeopleService\Name */
			$first = $name->getGivenName();
			$last = $name->getFamilyName();
		}
		if (!strlen($last)) {
			$last = '[empty]';
		}
		$iahItem->set(array(
			'first_name' => $first,
			'last_name' => $last
		));

		// Addresses
		$addresses = array();
		if(count($item->getAddresses()) > 0) {
			foreach ($item->getAddresses() as $a) {
				/* @var $a Google\Service\PeopleService\Address */

				$metadata = $a->getMetadata();
				$prefix = $metadata->getPrimary() ? 'primary' : 'alt';

				if (!isset($addresses[$prefix])) {
					$addresses[$prefix][$prefix . '_address_street'] = $a->getStreetAddress();
					$addresses[$prefix][$prefix . '_address_city'] = $a->getCity();
					$addresses[$prefix][$prefix . '_address_state'] = $a->getRegion();
					$addresses[$prefix][$prefix . '_address_postalcode'] = $a->getPostalCode();
					$addresses[$prefix][$prefix . '_address_country'] = $a->getCountry();
					$addresses[$prefix][$prefix . '_address_countrycode'] = $a->getCountryCode();
				}
			}
		}
		foreach ($addresses as $a) {
			$iahItem->set($a);
		}

		// Emails
		$emails = array();
		if(count($item->getEmailAddresses()) > 0) {
			foreach ($item->getEmailAddresses() as $e) {
				/* @var $e Google\Service\PeopleService\EmailAddress */
				$metadata = $e->getMetadata();
				if($metadata->getPrimary() && !isset($emails['email1'] )) {
					$emails['email1'] = $e->getValue();
				} elseif (!isset($emails['email2'])) {
					$emails['email2'] = $e->getValue();
				}
			}
		}
		$iahItem->set($emails);

		// Phones
		$phones = array();
		if(count($item->getPhoneNumbers()) > 0) {
			$phoneFields = array('phone_home', 'phone_mobile', 'phone_other', 'phone_fax');
			foreach($item->getPhoneNumbers() as $phone) {
				$metadata = $phone->getMetadata();
				$field = 'phone_' . $phone->getType();
				if($metadata->getPrimary() && !isset($phones['phone_work'])) {
					$phones['phone_work'] = $phone->getValue();
				} elseif(in_array($field, $phoneFields) && !isset($phones[$field])) {
					$phones[$field] = $phone->getValue();
				}
			}
		}
		$iahItem->set($phones);

		// Organization 
		if(count($item->getOrganizations()) > 0) {
			$organization = $item->getOrganizations()[0];
			$iahItem->set('title', $organization->getTitle());
		}
	}

	protected function getSQLQuery($lastSyncTime)
	{
		global $db;
		$all = $this->getPreference('which');

		$query = "SELECT c.id FROM contacts c "
			." LEFT JOIN google_sync s ON s.related_id=c.id AND s.user_id='".$this->user->id . "' AND s.related_type='Contacts'"
			." WHERE  1 ";
		$query .= " AND (c.date_modified > s.google_last_sync OR s.google_last_sync IS NULL)";
		if ($all) {
			if (ACLAction::userNeedsOwnership($this->user->id, 'Contacts', 'view','module')) {
				$level = ACLAction::getUserActions($this->user->id, false, 'Contacts', 'module', 'view');
				$c = new Contact;
				$owner_where = $c->getOwnerWhere($this->user->id, $level['aclaccess'], 'c');
				$query .= ' AND ' . $owner_where;
			}
		} else {
			$query .= " AND c.assigned_user_id='" . $db->quote($this->user->id) . "'";
		}
		$query .= ' ORDER BY c.date_modified ';
		$query .= ' LIMIT ' . (int)array_get_default($this->params, 'max_docs', 50);
		return $query;
	}

	protected function setGoogleItemFields($iahItem){
		$person = new \Google\Service\PeopleService\Person();

		// Name
		$personName = new \Google\Service\PeopleService\Name();
		$personName->setGivenName($iahItem->getField('first_name'));
		$personName->setFamilyName($iahItem->getField('last_name'));
		$person->setNames($personName);

		// Address
		$addresses = [];
		$prefixes = ['primary', 'alt'];
		foreach ($prefixes as $prefix) {
			$address = new Google\Service\PeopleService\Address();
			$address->setStreetAddress($iahItem->getField($prefix . '_address_street'));
			$address->setCity($iahItem->getField($prefix . '_address_city'));
			$address->setRegion($iahItem->getField($prefix . '_address_state'));
			$address->setPostalCode($iahItem->getField($prefix . '_address_postalcode'));
			$address->setCountry($iahItem->getField($prefix . '_address_country'));
			$address->setCountryCode($iahItem->getField($prefix . '_address_countrycode'));
			$addresses[] = $address;
		}
		$person->setAddresses($addresses);

		// Email
		$emails = [];
		$emailFields = ['email1', 'email2'];
		foreach ($emailFields as $field) {
			$email = new Google\Service\PeopleService\EmailAddress();
			$email->setValue($iahItem->getField($field));
			if($field == 'email1') {
				$metadata = new Google\Service\PeopleService\FieldMetadata();
				$metadata->setPrimary(true);
				$email->setMetadata($metadata);
			}
			$emails[] = $email;
		}
		$person->setEmailAddresses($emails);

		// Phone
		$phones = [];
		$phoneTypes = array('work', 'home', 'mobile', 'other', 'fax');
		foreach ($phoneTypes as $type) {
			$phone = new \Google\Service\PeopleService\PhoneNumber();
			$phone->setValue($iahItem->getField('phone_' . $type));
			$phone->setType($type);
			if($type == 'work') {
				$metadata = new Google\Service\PeopleService\FieldMetadata();
				$metadata->setPrimary(true);
				$phone->setMetadata($metadata);
			}
			$phones[] = $phone;
		}
		$person->setPhoneNumbers($phones);

		// Organization 
		$organization = new \Google\Service\PeopleService\Organization();
		$organization->setTitle($iahItem->getField('title'));
		$person->setOrganizations([$organization]);

		return $person;
	}

	protected function realCreateGoogleItem($iahItem) {
		try	{
			$person = $this->setGoogleItemFields($iahItem);
			$item = $this->service->people->createContact($person);
		} catch (Exception $e) {
			$item_id = $iahItem ? $iahItem->getField('id') : '';
			$GLOBALS['log']->fatal("Failed to create new Google contact ($item_id): ".$e->getMessage());
			$item = null;
		}
		return $item;
	}

	protected function deleteItem($item) {}
	
	protected function realUpdateGoogleItem($item, $iahItem)
	{
		/* @var $item \Google\Service\PeopleService\Person */
		$person = $this->setGoogleItemFields($iahItem);
		$person->setEtag($item->getEtag());
		return $this->service->people->updateContact(
			'people/' . $iahItem->_google_fields['google_id'],
			$person,
			['updatePersonFields' => 'names,addresses,emailAddresses,phoneNumbers,organizations']
		);
	}

	protected function getListEntry($googleId)
	{
		return $this->service->people->get('people/' . $googleId, ['personFields' => 'metadata']);
	}

	protected function saveItem($item, $iahItem){}
	
	protected function getItems($query)
	{
		$optParams = array(
			'pageSize' => $query['maxResults'],
			'personFields' =>  'metadata,names,addresses,emailAddresses,phoneNumbers,organizations',
			'requestSyncToken' => true,
			'syncToken' => array_get_default($query, 'syncToken'),
			'pageToken' =>array_get_default($query, 'pageToken'),
		);

		try {
			return $this->service->people_connections->listPeopleConnections('people/me', $optParams);
		} catch (\Google\Exception $e) {
			$this->logException($e);
		}
	}

	protected function mapLegacyGoogleId($googleId) {
		return $googleId;
	}

	protected function getGoogleIdClause($googleId) {
		$clause = array('multiple' => array(), 'operator' => 'or');
		$clause['multiple'][] = array(
			'field' => 'google_id',
			'value' => $googleId,
		);
		$clause['multiple'][] = array(
			'field' => 'google_id',
			'value' => 'http%://%/' . $googleId,
			'operator' => 'like',
			'match' => 'custom',
		);
		return $clause;
	}

	protected function getItemId($item)
	{
		/* @var $item \Google\Service\PeopleService\Person */
		$metadata = $this->getContactMetadata($item->getMetadata());
		$resourceName = explode('/', $item->getResourceName());
		return $resourceName[1];
	}

	/**
	 * @return Google\Service\PeopleService\Source
	 */
	protected function getContactMetadata($metaData) {
		if($metaData && is_array($metaData->getSources())) {
			foreach ($metaData->getSources() as $source) {
				if($source['type'] == 'CONTACT') {
					return $source;
				}
			}
		}
	}
}
