<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormTableSection.php');

class RelatedRecordsWidget extends FormTableSection {
	var $no_sel_account;
	var $rel_data = array();

    function init($params=null, $model=null) {
		parent::init($params, $model);
		if(! $this->id)
			$this->id = 'related_records';
		if(! empty($params['no_sel_account']))
			$this->no_sel_account = $params['no_sel_account'];
	}

	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if($gen->getLayout()->getEditingLayout()) {
			$params = array();
			return $this->renderOuterTable($gen, $parents, $context, $params);
		}

		if($gen->getLayout()->getDefaultEditable()) {
			return $this->renderHtmlEdit($gen, $row_result, $parents, $context);
		}
		return '';
	}

	function getRequiredFields() {
		if($this->no_sel_account) return array();
		return array('primary_account');
	}

	function renderPanel($gen, $name, $label, $body, $open = false, $row_result = false) {
		if ($row_result) $primaryAccountId = $row_result->getField('primary_account_id');

		$panel = '<div class="form-row"><div class="form-cell"><div class="form-entry">';
		$panel .= '<div class="card-outer panel-outer panel-default panel-border panel-round">';
		$panel .= '<div class="card-header panel-header">';
		$label = translate($label, 'Contacts');

		if ($name === 'newopp' && !$primaryAccountId) {
			$label = translate($label, 'Contacts') . ' (' . translate('LBL_ACCOUNT_IS_REQUIRED', 'Contacts') . ')';
			$disabled = true;
		}

		$spec = [
			'name' => $name,
			'label' => $label,
			'onchange' => "toggleDisplayContacts('{$name}div')",
			'toggle' => true,
			'show_label' => true,
			'disabled' => $disabled ?? false,
		];
		$check = $gen->form_obj->renderCheck($spec, $open);
		$panel .= $check;
		$panel .= '</div>';
		$display = !$open ? 'display: none' : '';
		$panel .= '<div class="card-body panel-body panel-pad-body" id="' . $name . 'div" style="' . $display . '">';

		if(is_array($body)) {
			$rr = new RowResult('Account');
			$vals = [];
			foreach($body as $e) {
				if(isset($e['vname']))
					$e['label'] = translate($e['vname'], $e['vname_module']);
				if(! array_key_exists('value', $e)) {
					if(isset($e['init_value']))
						$rr->assign($e['name'], $e['init_value']);
					$e2 = $e;
					$e2['required'] = false;
					$e['value'] = $gen->getFormObject()->renderField($rr, $e2);
					$e['editable'] = true;
				}
				$vals[] = $e;
			}
			$body = '';
			$rows = $this->arrangeSectionRows($vals, 2);
			if($rows) {
				//$body .= '<div class="form-section">';
				$body .= $this->renderRows($gen, $rows, 2);
				//$body .= '</div>';
			}
		}
		$panel .= $body;

		$panel .= '</div>';
		$panel .= '</div>';
		$panel .= '</div></div></div>';
		return $panel;
	}

	function renderHtmlEdit(HtmlFormGenerator $gen, RowResult $row_result, array $parents, array $context) {

		$rel_lead_id = array_get_default($this->rel_data, 'rel_lead_id');
		if($rel_lead_id)
			$gen->getFormObject()->addHiddenFields(['rel_lead_id' => $rel_lead_id]);

		$acct = array_get_default($this->rel_data, 'account', []);
		$newacct = empty($acct['disabled']) ? 1 : 0;
		$body = '';

		$elements = [
			[
				'type' => 'varchar',
				'name' => 'account_name',
				'vname' => 'LBL_LIST_ACCOUNT_NAME',
				'vname_module' => 'Accounts',
				'width' => 30,
				'required' => true,
				'init_value' => array_get_default($acct, 'name'),
			],
			[
				'type' => 'bool',
				'name' => 'account_is_supplier',
				'vname' => 'LBL_IS_SUPPLIER',
				'vname_module' => 'Accounts',
				'required' => true,
				'init_value' => array_get_default($acct, 'is_supplier'),
			],
			[
				'type' => 'phone',
				'name' => 'account_phone_office',
				'vname' => 'LBL_LIST_PHONE',
				'vname_module' => 'Accounts',
				'init_value' => array_get_default($acct, 'phone_office'),
			],
			[
				'type' => 'url',
				'name' => 'account_website',
				'vname' => 'LBL_WEBSITE',
				'vname_module' => 'Accounts',
				'width' => 30,
				'init_value' => array_get_default($acct, 'website'),
			],
			[
				'type' => 'text',
				'name' => 'account_description',
				'vname' => 'LBL_DESCRIPTION',
				'vname_module' => 'Accounts',
				'colspan' => 2,
				'init_value' => array_get_default($acct, 'description'),
			],
			// FIXME - restore create note
		];
		if($newacct) {
			$open = strlen(array_get_default($acct, 'name')) != 0;
			$body .= $this->renderPanel($gen, 'newaccount', 'LNK_NEW_ACCOUNT', $elements, $open);
		}


		$elements = [
			[
				'type' => 'varchar',
				'name' => 'opportunity_name',
				'vname' => 'LBL_OPPORTUNITY_NAME',
				'vname_module' => 'Accounts',
				'width' => 30,
				'required' => true,
			],
			[
				'type' => 'date',
				'name' => 'opportunity_date_closed',
				'vname' => 'LBL_DATE_CLOSED',
				'vname_module' => 'Opportunities',
				'required' => true,
			],
			[
				'type' => 'enum',
				'name' => 'opportunity_sales_stage',
				'options' => 'sales_stage_dom',
				'vname' => 'LBL_SALES_STAGE',
				'vname_module' => 'Opportunities',
				'required' => true,
			],
			[
				'type' => 'currency',
				'name' => 'opportunity_amount',
				'vname' => 'LBL_AMOUNT',
				'vname_module' => 'Opportunities',
				'required' => true,
			],
			[
				'type' => 'text',
				'name' => 'opportunity_description',
				'vname' => 'LBL_DESCRIPTION',
				'vname_module' => 'Opportunities',
				'colspan' => 2,
			],
			// FIXME - restore create note
		];
		$body .= $this->renderPanel($gen, 'newopp', 'LNK_NEW_OPPORTUNITY', $elements, false, $row_result);


		$elements = [
			[
				'value' => '<div class="input-check input-field">'
					. '<input type="radio" value="Call" name="appointment" id="appt_call">'
					. '<label for="appt_call" class="input-label text-label"> ' . translate('LNK_NEW_CALL', 'Contacts') . '</label>'
					. '</div>'
					. ' <div class="input-check input-field">'
					. '<input type="radio" value="Meeting" name="appointment" id="appt_meeting" checked="checked">'
					. '<label for="appt_meeting" class="input-label text-label"> ' . translate('LNK_NEW_MEETING', 'Contacts') . '</label>'
					. '</div>',
				'colspan' => 2,
				'required' => true,
			],
			[
				'type' => 'varchar',
				'name' => 'appointment_name',
				'vname' => 'LBL_LIST_SUBJECT',
				'vname_module' => 'Calls',
				'width' => 30,
				'required' => true,
			],
			[
				'type' => 'datetime',
				'name' => 'appointment_date_start',
				'vname' => 'LBL_DATE',
				'vname_module' => 'Calls',
				'required' => true,
			],
			[
				'type' => 'text',
				'name' => 'appointment_description',
				'vname' => 'LBL_DESCRIPTION',
				'vname_module' => 'Accounts',
				'colspan' => 2,
			],
		];
		$body .= $this->renderPanel($gen, 'newappointment', 'LNK_NEW_APPOINTMENT', $elements);

		$params = ['body' => $body];
		return $this->renderOuterTable($gen, $context, $parents, $params);
	}


	function loadUpdateRequest(RowUpdate $update, array $input) {
        $upd_relates = array();

        $lead_id = array_get_default($input, 'lead_id');
        if($lead_id && ($lead = ListQuery::quick_fetch('Lead', $lead_id))) {
        	$upd_relates['rel_lead_id'] = $lead_id;
        	$acct = $lead->getField('account_name');
        	if($acct) {
        		$upd_relates['account'] = array(
        			'name' => $acct,
        			'phone_office' => $lead->getField('phone_work'),
        			'website' => $lead->getField('website'),
        		);
				if($update->getField('primary_account_id'))
					$upd_relates['account']['disabled'] = 1;
        	}
        } else {
        	if(! empty($input['rel_lead_id']))
				$upd_relates['rel_lead_id'] = $input['rel_lead_id'];

			if (! empty($input['newaccount'])) {
				$upd_relates['account'] = self::fill_data('account', $input);
				$upd_relates['account_notes'] = self::fill_notes('account', $input);
			}

			if (! empty($input['newopp'])) {
				$upd_relates['opportunity'] = self::fill_data('opportunity', $input);
				$upd_relates['opportunity_notes'] = self::fill_notes('opportunity', $input);
			}

			if (! empty($input['newappointment'])) {
				if (! empty($input['appointment'])) {
					$upd_relates['appointment'] = self::fill_data('appointment', $input);
					$upd_relates['appointment']['model'] = $input['appointment'];
				}
			}
		}

        $update->setRelatedData($this->id.'_rows', $upd_relates);
        $this->rel_data = $upd_relates;
	}

	function validateInput(RowUpdate $update) {
		return true;
	}

	function afterUpdate(RowUpdate $update) {
		$row_updates = $update->getRelatedData($this->id.'_rows');
		if(! $row_updates)
			return;

        $lead_id = array_get_default($row_updates, 'rel_lead_id');
        $account_id = null;
		$partner_id = null;
		$campaign_id = null;
        $lead = null;

        if($lead_id) {
            $lead = ListQuery::quick_fetch('Lead', $lead_id);
            if ($lead) {
				$partner_id = $lead->getField('partner_id');
				$campaign_id = $lead->getField('campaign_id');
			}
        }

        if (isset($row_updates['account']) && is_countable($row_updates['account']) && sizeof($row_updates['account']) > 0) {
            if ($partner_id)
                $row_updates['account']['partner_id'] = $partner_id;
            $account_id = self::addAccount($update, $row_updates['account'], $row_updates['account_notes'], $lead);
        }

        if(! $account_id) {
        	$account_id = $update->getField('primary_account_id');

            if(AppConfig::is_B2C()) {
                $account_id = $update->getField('primary_contact_for');
            }

			$this->setAccountPrimaryContact($account_id, $update);
		}

        if (isset($row_updates['opportunity']) && is_countable($row_updates['opportunity']) && sizeof($row_updates['opportunity']) > 0) {
            if ($partner_id)
                $row_updates['opportunity']['partner_id'] = $partner_id;
            $opp_id = self::addOpportunity($update, $row_updates['opportunity'], $row_updates['opportunity_notes'], $account_id, $lead);
        } else {
            $opp_id = null;
        }

		if($lead) {
			$contact_id = $update->getField('id');
            $this->moveLeadRelationships($update, $lead_id, $account_id);

            $lead_up = RowUpdate::for_result($lead);
            $lead_up->set('contact_id', $contact_id);
            $lead_up->set('status', 'Converted');
            $date = gmdate('Y-m-d H:i:s');
            $lead_up->set('date_converted', $date);
            $lead_up->set('converted', '1');
            if($account_id)
                $lead_up->set('account_id', $account_id);
            if($opp_id)
                $lead_up->set('opportunity_id', $opp_id);
			$lead_up->validate() && $lead_up->save();
			if ($campaign_id) {
				require_once 'modules/Campaigns/utils.php';
				CampaignActivityTracker::campaign_log_lead_entry($campaign_id, $lead_id, $contact_id, 'Contacts', $update->getField('email1'), 'contact');
			}
        }

        if (isset($row_updates['appointment']) && is_countable($row_updates['appointment']) && sizeof($row_updates['appointment']) > 0) {
            self::addAppointment($update, $row_updates['appointment'], $account_id);
        }
	}

    function saveAfterDuplicates(RowUpdate $update, $selected = array()) {
        $row_updates = $update->getRelatedData($this->id.'_rows');
	    $lead_id = array_get_default($row_updates, 'rel_lead_id');

        $account_id = null;
	    $partner_id = null;
	    $lead = null;

        $account_notes = array_get_default($row_updates, 'account_notes', array());
        $opportunity_notes = array_get_default($row_updates, 'opportunity_notes', array());

	    if($lead_id) {
		    $lead = ListQuery::quick_fetch('Lead', $lead_id);

		    if ($lead)
			    $partner_id = $lead->getField('partner_id');
	    }

	    if (isset($selected['account_id'])) {
            $account_id = $selected['account_id'];
            $acc_result = ListQuery::quick_fetch('Account', $account_id);
            $acc_upd = RowUpdate::for_result($acc_result);
            self::addAccountLink($update, $acc_upd, $account_notes);
        } elseif (isset($row_updates['account']) && is_countable($row_updates['account']) && sizeof($row_updates['account']) > 0) {
	        if ($partner_id)
		        $row_updates['account']['partner_id'] = $partner_id;
	        $account_id = self::addAccount($update, $row_updates['account'], $account_notes);
        }

        if(! $account_id) {
            $account_id = $update->getField('primary_account_id');

            if(AppConfig::is_B2C()) {
                $account_id = $update->getField('primary_contact_for');
            }
        }

        if (isset($selected['opportunity_id'])) {
            $opp_result = ListQuery::quick_fetch('Opportunity', $selected['opportunity_id']);
            $opp_upd = RowUpdate::for_result($opp_result);
            self::addOpportunityLink($update, $opp_upd, $opportunity_notes, $account_id);
        } elseif (isset($row_updates['opportunity']) && is_countable($row_updates['opportunity']) && sizeof($row_updates['opportunity']) > 0) {
            self::addOpportunity($update, $row_updates['opportunity'], $opportunity_notes, $account_id);
        }

        if (isset($row_updates['appointment']) && is_countable($row_updates['appointment']) && sizeof($row_updates['appointment']) > 0)
            self::addAppointment($update, $row_updates['appointment'], $account_id);

	    if($lead && $account_id) {
		    $lead_up = RowUpdate::for_result($lead);
		    $lead_up->set('account_id', $account_id);
		    $lead_up->validate() && $lead_up->save();
	    }
    }

    function moveLeadRelationships(RowUpdate $contact_update, $lead_id, $account_id) {
        $activities = $this->getRelationships('activities', $lead_id);
        $this->reassignLinks($activities, $contact_update, $account_id);
        $history = $this->getRelationships('history', $lead_id);
        $this->reassignLinks($history, $contact_update, $account_id);
        $documents = $this->getRelationships('documents', $lead_id);
        $this->reassignLinks($documents, $contact_update, $account_id);
    }

    function reassignLinks($links, RowUpdate $to_upd, $account_id) {
        $links_num = is_countable($links) ? sizeof($links) : 0;
        $link_models = array();
        if (is_array($links) && $links_num > 0) {

            for ($i = 0; $i < $links_num; $i++) {
                $link_name = $links[$i]['query_source'];
                if(!$link_name) {
                	$link_name = 'documents';
				}

                if ( ! isset($link_models[$link_name])) {
                    $link_def = $to_upd->model->getLinkDefinition($link_name);
                    $link_models[$link_name] = $link_def['bean_name'];
                }

                $link_model_name = $link_models[$link_name];
                $link_model_result = ListQuery::quick_fetch($link_model_name, $links[$i]['id']);

                if (! $link_model_result->failed) {
                    $link_upd = RowUpdate::for_result($link_model_result);
                    $need_save = false;
					$contact_id = $to_upd->getField('id');

					switch ($link_name) {
					case 'calls':
					case 'meetings':
						if ($account_id) {
						$link_upd->set(array('account_id' => $account_id));
						}
						$link_upd->set(array('parent_type' => 'Contacts', 'parent_id' => $contact_id));
						$need_save = true;
						$link_upd->addUpdateLink('contacts', $contact_id);
						break;
					case 'tasks':
                        $link_upd->set(array('account_id' => $account_id));
                        $link_upd->set('contact_id', $contact_id);
                        $need_save = true;
						break;
					case 'emails':
						if ($account_id) {
							$link_upd->set(array('account_id' => $account_id));
						}
						$link_upd->set(array('parent_type' => 'Contacts', 'parent_id' => $contact_id));
						$need_save = true;
						break;
					case 'notes':
                        $link_upd->set(array('account_id' => $account_id, 'contact_id' => $contact_id));
                        $need_save = true;
						break;
					case 'documents':
						$link_upd->addUpdateLink('contacts', $contact_id);
						if($account_id) {
							$link_upd->addUpdateLink('accounts', $account_id);
						}
						break;
					default:
                        $link_upd->addUpdateLink('contacts', $contact_id);
					}
                    if ($need_save)
                        $link_upd->save();
                }
            }

        }
    }

    /**
     * Get relationships data from parent record
     *
     * @param string $rel_name - link name
     * @param string $parent_id
     * @return array
     */
    function getRelationships($rel_name, $parent_id) {
        $query = new ListQuery('Lead', null, array('link_name' => $rel_name));
        $query->setParentKey($parent_id);
        $query->setOrderBy('id');
        $result = $query->fetchAll();
        $relationships = array();

        if (! $result->failed && sizeof($result->rows) > 0) {
            foreach ($result->rows as $rel) {
                array_push($relationships, $rel);
            }
        }

        return $relationships;
    }

    function setAccountPrimaryContact($account_id, RowUpdate $contact_upd) {
        if ($account_id) {
            $acc_result = ListQuery::quick_fetch('Account', $account_id);
            if ($acc_result && ! $acc_result->getField('primary_contact_id')) {
                $acc_upd = RowUpdate::for_result($acc_result);
                $acc_upd->set('primary_contact_id', $contact_upd->getField('id'));
                $acc_upd->save();
            }
        }
    }

    /**
     * Add related account
     *
     * @static
     * @param RowUpdate $contact_upd
     * @param array $data
     * @param array $note_data
     * @param RowResult|null $lead
     * @return array|mixed|null
     */
    static function addAccount(RowUpdate $contact_upd, $data, $note_data, $lead = null) {
        global $current_user;
        $acc_upd = RowUpdate::blank_for_model('Account');

        $addr_fields = array('address_street', 'address_city',
            'address_state', 'address_postalcode', 'address_country');

        for ($i = 0; $i < sizeof($addr_fields); $i++) {
            $contact_field = 'primary_' . $addr_fields[$i];
            $billing =  'billing_' . $addr_fields[$i];
            $shipping =  'shipping_' . $addr_fields[$i];

            $data[$billing] = $contact_upd->getField($contact_field);
            $data[$shipping] = $contact_upd->getField($contact_field);
        }

        $data['primary_contact_id'] = $contact_upd->getField('id');
        $data['assigned_user_id'] = AppConfig::current_user_id();
        $data['currency_id'] = $current_user->getPreference('currency');

		$skip = array(
			'id', 'id_c', 'portal_name',
			'date_entered', 'date_modified', 'deleted',
			'modified_user_id', 'assigned_user_id', 'created_by',
			'name', 'temperature', 'phone_fax', 'email1', 'email2',
			'email_opt_out', 'website', 'description', 'invalid_email',
			'partner_id', 'last_activity_date',
		);
		if($lead) {
			$fs_extend = array_intersect($lead->getFieldNames(), $acc_upd->getFieldNames());
			// copy any additional matching fields
			foreach($fs_extend as $k) {
				if(in_array($k, $skip) || isset($data[$k]) || strpos($k, '_address_') !== false) continue;
				$data[$k] = $lead->getField($k);
			}
		}

        $acc_upd->set($data);
		if ($lead)
			copy_custom_fields($lead, $acc_upd);

        if ($acc_upd->save()) {
            self::addAccountLink($contact_upd, $acc_upd, $note_data);


            // copy lead social links to new account
            if(isset($_POST['social_accounts'])) {
				$social_accounts = json_decode($_POST['social_accounts']);

				require_once 'modules/SocialAccounts/SocialAccount.php';

				foreach($social_accounts as $sa) {
					$ins_row = \SocialAccount::quick_create_row($sa->url, 'Accounts', $acc_upd->getField('id'));
		  			if ($ins_row) {
		  				$ins_row->save();
					}
				}
			}

            return $acc_upd->getField('id');
        }

        return null;
    }

    /**
     * Link contact with account and create notes
     *
     * @param RowUpdate $contact_upd
     * @param RowUpdate $account_upd
     * @param array $note_data
     */
	static function addAccountLink(RowUpdate $contact_upd, RowUpdate $account_upd, $note_data) {
		$res = ListQuery::quick_fetch('Contact', $contact_upd->getField('id'));
		if ($res) {
			$upd = RowUpdate::for_result($res);
			$upd->prohibit_workflow = true;
	        $upd->set(array('primary_account_id' => $account_upd->getField('id')));
			$upd->setRelatedData('force_email_account', null);
        	$upd->save();
		}

        if (is_array($note_data) && sizeof($note_data) > 0) {
            $note_id = self::addNote('Accounts', $account_upd->getField('id'), $note_data);

            if ($note_id)
                $account_upd->addUpdateLink('notes', $note_id);
        }
    }

    /**
     * Add related opportunity
     *
     * @static
     * @param RowUpdate $contact_upd
     * @param array $data
     * @param array $note_data
     * @param string $account_id
     * @param RowResult|null $lead
     * @return array|mixed|null
     */
    static function addOpportunity(RowUpdate $contact_upd, $data, $note_data, $account_id, $lead = null) {
        global $current_user, $timedate;
        $opp_upd = RowUpdate::blank_for_model('Opportunity');

        if (! $account_id)
        	return false;

		$data['account_id'] = $account_id;
        $data['lead_source'] = $contact_upd->getField('lead_source');
        $data['assigned_user_id'] = AppConfig::current_user_id();
        $data['currency_id'] = $current_user->getPreference('currency');
        $data['amount'] = unformat_number($data['amount']);
        $data['date_closed'] = $timedate->to_db_date($data['date_closed'], false);

        if (isset($data['sales_stage'])) {
            global $app_list_strings;
            $sales_stage = $data['sales_stage'];
            $forecast_cats = $app_list_strings['sales_forecast_dom'];
            $probabilities = $app_list_strings['sales_probability_dom'];

            $probability = 0;
            if (isset($probabilities[$sales_stage]))
                $probability = $probabilities[$sales_stage];

            $data['probability'] = $probability;

            if (isset($forecast_cats[$sales_stage]))
                $data['forecast_category'] = $forecast_cats[$sales_stage];

        }

		$skip = array(
			'id', 'id_c',
			'date_entered', 'date_modified', 'deleted',
			'modified_user_id', 'assigned_user_id', 'created_by',
			'lead_source', 'description',
			'account_id', 'partner_id',
		);
		if($lead) {
			$fs_extend = array_intersect($lead->getFieldNames(), $opp_upd->getFieldNames());
			// copy any additional matching fields
			foreach($fs_extend as $k) {
				if(in_array($k, $skip) || isset($data[$k]) || strpos($k, '_address_') !== false) continue;
				$data[$k] = $lead->getField($k);
			}
		}

        $opp_upd->set($data);

		if (! $opp_upd->validate()) {
			$GLOBALS['log']->error('opp validate failed');
			$GLOBALS['log']->error($opp_upd->errors);
			return null;
		}
        if ($opp_upd->save()) {
            self::addOpportunityLink($contact_upd, $opp_upd, $note_data, $account_id);
            return $opp_upd->getField('id');
        }

        return null;
    }

    /**
     * Link contact with opportunity and create notes
     *
     * @static
     * @param RowUpdate $contact_upd
     * @param RowUpdate $opportunity_upd
     * @param array $note_data
     * @param string $account_id
     */
    static function addOpportunityLink(RowUpdate $contact_upd, RowUpdate $opportunity_upd, $note_data, $account_id) {
        if ($account_id)
            $opportunity_upd->addUpdateLink('accounts', $account_id);

        $opportunity_upd->addUpdateLink('contacts', $contact_upd->getField('id'));

        if (is_array($note_data) && sizeof($note_data) > 0) {
            $note_id = self::addNote('Opportunities', $opportunity_upd->getField('id'), $note_data);

            if ($note_id)
                $opportunity_upd->addUpdateLink('notes', $note_id);
        }
    }

    /**
     * Add related appointment (Call or Meeting)
     *
     * @static
     * @param RowUpdate $contact_upd
     * @param array $data
     * @param string $account_id
     * @return array|mixed|null
     */
    static function addAppointment(RowUpdate $contact_upd, $data, $account_id) {
    	global $timedate;
        $app_upd = RowUpdate::for_model($data['model']);
        $app_upd->loadBlankRecord();

        $data['direction'] = 'Outbound';
        $data['status'] = 'Planned';
        $data['duration'] = 60;
        $data['assigned_user_id'] = AppConfig::current_user_id();
        $data['date_start'] = $timedate->to_db($data['date_start']);

        if ($account_id) {
            $data['parent_type'] = 'Accounts';
            $data['parent_id'] = $account_id;
        } else {
            $data['parent_type'] = 'Contacts';
            $data['parent_id'] = $contact_upd->getField('id');
        }

        $app_upd->set($data);

		if (! $app_upd->validate()) {
			$GLOBALS['log']->error('appointment validate failed');
			$GLOBALS['log']->error($app_upd->errors);
			return null;
		}
        if ($app_upd->save()) {
        	$app_upd->addUpdateLink('users', array('id' => $data['assigned_user_id'], 'accept_status' => 'accept'));
            $app_upd->addUpdateLink('contacts', $contact_upd->getField('id'));

            return $app_upd->getField('id');
        }

        return null;
    }

    /**
     * Add note
     *
     * @static
     * @param string $parent_type
     * @param string $parent_id
     * @param array $data
     * @return array|mixed|null
     */
    static function addNote($parent_type, $parent_id, $data) {
        $note_upd = RowUpdate::for_model('Note');
        $note_upd->loadBlankRecord();

        $data['parent_type'] = $parent_type;
        $data['parent_id'] = $parent_id;
        $data['assigned_user_id'] = AppConfig::current_user_id();

        $note_upd->set($data);

        if ($note_upd->save())
            return $note_upd->getField('id');

        return null;
    }

    /**
     * Fill related records data from input
     *
     * @static
     * @param string $type - type of related records
     * (account, opportunity, appointment)
     * @param array $input
     * @return array
     */
    static function fill_data($type, $input) {
        $data = array();
        $fields = array(
            'account' => array('name', 'phone_office', 'website', 'description', 'is_supplier'),
            'opportunity' => array('name', 'date_closed', 'sales_stage', 'amount', 'description'),
            'appointment' => array('name', 'date_start', 'description')
        );

        if (! empty($input[$type . '_name'])) {
			$typeCount = is_countable($fields[$type]) ? sizeof($fields[$type]) : 0;
            for ($i = 0; $i < $typeCount; $i++) {
                $field_name = $fields[$type][$i];
                if (isset($input[$type .'_'. $field_name]))
                    $data[$field_name] = $input[$type .'_'. $field_name];
            }
        }

        return $data;
    }

    /**
     * Fill related record'd notes data
     *
     * @static
     * @param string $type - type of related records
     * (account, opportunity, appointment)
     * @param array $input
     * @return array
     */
    static function fill_notes($type, $input) {
        $notes = array();

        if (! empty($input[$type . '_notes_name'])) {
            $notes['name'] = $input[$type . '_notes_name'];
            if (isset($input[$type . '_notes_description']))
                $notes['description'] = $input[$type . '_notes_description'];
        }

        return $notes;
    }
}
?>
