<?php

require_once 'modules/Quotes/PDFTemplatesProvider.php';

class CreditNotePDFTemplateProvider extends QuotePDFTemplateProvider
{

	public $state;
	public $iteration = array();


	public static function template_types()
	{
		return array(
			'credit_note' => translate('LBL_PDF_CREDIT', 'CreditNotes'),
		);
	}

	protected function __construct($type)
	{
		return parent::__construct($type);
	}

	public static function template_skeleton($type)
	{
		$ret = array('sections' => array(
			array(
				'name' => 'header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_HEADER',
				'height' => 150,
				'elements' => array (),
			),
			array(
				'name' => 'group_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_GROUP_HEADER',
				'height' => 150,
				'elements' => array (),
			),
			array(
				'name' => 'group_items',
				'type' => 'table',
				'title' => 'LBL_SKEL_GROUP_ITEMS',
				'columns' => array (),
			),
			array(
				'name' => 'group_totals_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_GROUP_TOTALS_HEADER',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'group_totals',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_GROUP_TOTALS',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'group_footer',
				'data_id' => 'group_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_GROUP_FOOTER',
				'height' => 150,
				'elements' => array (),
			),
			array(
				'name' => 'quote_totals_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_QUOTE_TOTALS_HEADER',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'quote_totals',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_QUOTE_TOTALS',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'notes',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_NOTES',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'footer',
				'data_id' => 'header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_FOOTER',
				'height' => 40,
				'elements' => array (),
			),
		));
		foreach ($ret['sections'] as &$sec) $sec['title'] = translate($sec['title'], 'CreditNotes');
		return $ret;
	}

	public static function template_sample_data($type)
	{
		$datefmt = AppConfig::setting('locale.defaults.date_format');
		$data = array(
			'header' => array(
				'todays_date' => array(translate('LBL_TODAYS_DATE'), date($datefmt), ),
				'number' => array(translate('LBL_SAMPLE_HEADER_NUMBER_LABEL', 'CreditNotes'), 'Q2013-001'),
				'date' => array(translate('LBL_SAMPLE_HEADER_DATE_LABEL', 'CreditNotes'), '2014-11-05'),
				'date_due' => array(translate('LBL_SAMPLE_HEADER_DATE_DUE_LABEL', 'CreditNotes'), '2014-11-05'),
				'tax_info' => array(translate('LBL_SAMPLE_HEADER_TAX_INFO_LABEL', 'CreditNotes'), ''),
				'billing_address' => array(translate('LBL_SAMPLE_HEADER_BILLING_ADDRESS_LABEL', 'CreditNotes'), translate('LBL_SAMPLE_HEADER_BILLING_ADDRESS_VALUE', 'CreditNotes'), ),
				'billing_address_contact_last' => array(translate('LBL_SAMPLE_HEADER_BILLING_ADDRESS_CONTACT_LAST_LABEL', 'Quotes'), translate('LBL_SAMPLE_HEADER_BILLING_ADDRESS_VALUE', 'Quotes'), ),
				'description' => array(translate('LBL_SAMPLE_HEADER_DESCRIPTION_LABEL', 'CreditNotes'), translate('LBL_SAMPLE_HEADER_DESCRIPTION_VALUE', 'CreditNotes'), ),
				'invoice' => array(translate('LBL_SAMPLE_HEADER_INVOICE_LABEL', 'CreditNotes'), translate('LBL_SAMPLE_HEADER_INVOICE_VALUE', 'CreditNotes'), ),
			),
			'group_header' => array(
				'group_name' => array(translate('LBL_SAMPLE_GROUP_HEADER_GROUP_NAME_LABEL', 'CreditNotes'), translate('LBL_SAMPLE_GROUP_HEADER_GROUP_NAME_VALUE', 'CreditNotes'), ),
				'status' => array(translate('LBL_SAMPLE_GROUP_HEADER_STATUS_LABEL', 'CreditNotes'), translate('LBL_SAMPLE_GROUP_HEADER_STATUS_VALUE', 'CreditNotes'), ),
			),
			'group_totals' => array(
				'total_name' => array(translate('LBL_SAMPLE_GROUP_TOTALS_TOTAL_NAME_LABEL', 'CreditNotes'), translate('LBL_SAMPLE_GROUP_TOTALS_TOTAL_NAME_VALUE', 'CreditNotes'), ),
				'total_value' => array(translate('LBL_SAMPLE_GROUP_TOTALS_TOTAL_VALUE_LABEL', 'CreditNotes'), translate('LBL_SAMPLE_GROUP_TOTALS_TOTAL_VALUE_VALUE', 'CreditNotes'), ),
			),
			'quote_totals' => array(
				'total_name' => array(translate('LBL_SAMPLE_QUOTE_TOTALS_TOTAL_NAME_LABEL', 'CreditNotes'), translate('LBL_SAMPLE_QUOTE_TOTALS_TOTAL_NAME_VALUE', 'CreditNotes'), ),
				'total_value' => array(translate('LBL_SAMPLE_QUOTE_TOTALS_TOTAL_VALUE_LABEL', 'CreditNotes'), translate('LBL_SAMPLE_QUOTE_TOTALS_TOTAL_VALUE_VALUE', 'CreditNotes'), ),
			),
			'group_items' => array(
				'number' => array(translate('LBL_SAMPLE_GROUP_ITEMS_NUMBER_LABEL', 'CreditNotes'), '1'),
				'product' => array(translate('LBL_SAMPLE_GROUP_ITEMS_PRODUCT_LABEL', 'CreditNotes'), translate('LBL_SAMPLE_GROUP_ITEMS_PRODUCT_VALUE', 'CreditNotes'), ),
				'quantity' => array(translate('LBL_SAMPLE_GROUP_ITEMS_QUANTITY_LABEL', 'CreditNotes'), '2'),
				'listprice' => array(translate('LBL_SAMPLE_GROUP_ITEMS_LISTPRICE_LABEL', 'CreditNotes'), '10.0'),
				'unitprice' => array(translate('LBL_SAMPLE_GROUP_ITEMS_UNITPRICE_LABEL', 'CreditNotes'), '20.0'),
				'extprice' => array(translate('LBL_SAMPLE_GROUP_ITEMS_EXTPRICE_LABEL', 'CreditNotes'), '40.0'),
			),
		);
		self::injectProductSampleData($data['group_items']);
		$ret = array();
		foreach ($data as $k => $v) {
			$ret[$k] = array();
			foreach ($v as $kk => $vv) {
				$ret[$k][$kk] = array('label' => $vv[0], 'value' => $vv[1]);
			}
		}
		return $ret;
	}


	public static function create($type)
	{
		return new CreditNotePDFTemplateProvider($type);
	}

	protected function iterateHeaderFooter(PDFTemplatePrinter $p, $name)
	{
		global $timedate;
		$row =& $this->row;
		$user = new User();
		$user->retrieve($row->getField('assigned_user_id'), false);
		$terms = AppConfig::setting("lang.lists.current.app.terms_dom", array());
		$tax_information = $this->row->getField('tax_information');
		$data = array(
			'number' => $row->getField('prefix') . $row->getField('credit_number'),
			'date' => $timedate->to_display_date($row->getField('date_entered')),
			'date_due' => $row->getField('due_date', '', true),
			'tax_info' => from_html(AppConfig::setting('company.tax_information', '')),
			'billing_address' => $this->_address('billing_') . "\n" . $tax_information,
			'billing_address_contact_last' => $this->_address('billing_c') . "\n" . $tax_information,
			'description' => $row->getField('description'),
			'invoice' => $row->getField('invoice'),
		);
		$data = array_merge($this->header_data, $data);
		$p->renderSection($this->findSection($name), $data);
	}

	function getFileName() {
		$filename_pfx = translate('LBL_PDF_CREDIT', 'CreditNotes');
		$filename = $filename_pfx . '_' . $this->row->getField('prefix');
		$filename .= $this->row->getField('credit_number') . '_' . $this->row->getField('billing_account_name');
		$filename = preg_replace('~\.+$~', '', trim($filename));
		$filename = $filename . '.pdf';
		return $filename;
	}

	public function getNoteTitle()
	{
		return translate('LBL_PREPARED_CREDIT', 'CreditNotes');
	}

	public function getEmailTitle()
	{
		return translate('LBL_EMAILED_CREDIT', 'CreditNotes');
	}

	public function getSignedDocumentTitle() {
		$prefix =
			$this->row
			? '[' . $this->row->getField('prefix') . $this->row->getField('credit_number') . '] '
			: '';

		return $prefix . $this->getNoteTitle();
	}

	protected function createData()
	{
		global $timedate;
		$row =& $this->row;
		$user = new User();
		$user->retrieve($row->getField('assigned_user_id'), false);
		$terms = AppConfig::setting("lang.lists.current.app.terms_dom", array());
		$tax_information = $this->row->getField('tax_information');
		$datefmt = AppConfig::setting('locale.defaults.date_format');
		$data = array(
			'todays_date' => date($datefmt),
			'number' => $row->getField('prefix') . $row->getField('credit_number'),
			'date' => $timedate->to_display_date($row->getField('date_entered')),
			'date_due' => $row->getField('due_date', '', true),
			'tax_info' => from_html(AppConfig::setting('company.tax_information', '')),
			'billing_address' => $this->_address('billing_') . "\n" . $tax_information,
			'description' => $row->getField('description'),
			'invoice' => $row->getField('invoice'),
		);
		$data = array_merge($this->header_data, $data);
		$this->data['header'] = $data;
		$this->createGroupData();
	}

	public function initialize($layout, ListQuery $lq)
	{
		$lq->addField('invoice');
		parent::initialize($layout, $lq);
	}

	public static function get_extra_sources() {
		return array(
			array(
				'module' => 'Invoice',
				'model' => 'Invoice',
				'prefix' => 'invoice',
				'label' => 'LBL_CREDIT_FOR_INVOICE',
			),
			array(
				'module' => 'Opportunities',
				'model' => 'Opportunity',
				'prefix' => 'opportunity',
				'label' => 'LBL_OPPORTUNITY_NAME',
			),
			array(
				'module' => 'Accounts',
				'model' => 'Account',
				'prefix' => 'billing_account',
				'label' => 'LBL_BILLING_ACCOUNT',
			),
			array(
				'module' => 'Contacts',
				'model' => 'Contact',
				'prefix' => 'billing_contact',
				'label' => 'LBL_BILLING_CONTACT',
			),
		);
	}
}

