<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*
	Copyright 2005 Sveta Smirnova
	$Id: CurrencyUtils.php 7531 2010-10-13 20:03:33Z andrew $
*/

require_once('modules/Currencies/Currency.php');

class CurrencyUtils
{
	static function get_list($reload=false)
	{
		static $list;
		
		if (! isset($list) || $reload) {
			$focus = new Currency();
			$temp_list = $focus->get_full_list('name');
			$focus->retrieve('-99');
	  		if(is_array($temp_list)){
				$temp_list = array_merge(array($focus), $temp_list);
			}else{
				$temp_list = array($focus);	
			}
			foreach ($temp_list as $value) {
				$list[$value->id] = $value;
			}
		}
		
		return $list;
	}
	
	static function get_currency_by_id($currency_id)
	{
		$list = CurrencyUtils::get_list();
		return (isset($list[$currency_id]) ? $list[$currency_id] : $list['-99']);
	}
	
	static function get_currency_name($currency_id, $short=false)
	{
		$currency = CurrencyUtils::get_currency_by_id($currency_id);
		if( $currency->deleted != 1)
			return ($short ? $currency->name : $currency->iso4217) . ' : ' . $currency->symbol;
		else 
			return ($short ? Currency::getDefaultCurrencyName() : Currency::getDefaultISO4217()) . ' : ' . Currency::getDefaultCurrencySymbol();
	}
	
	function get_pdf_currency_name($currency_id)
	{
		$currency = CurrencyUtils::get_currency_by_id($currency_id);
		return $currency->name . ' : ' . $currency->getPdfCurrencySymbol();
	}
	
	function get_pdf_currency_symbol($currency_id)
	{
		$currency = CurrencyUtils::get_currency_by_id($currency_id);
		return $currency->getPdfCurrencySymbol();
	}
	
	function get_currency_symbol($currency_id)
	{
		$currency = CurrencyUtils::get_currency_by_id($currency_id);
		if( $currency->deleted != 1)
			return $currency->symbol;
		else 
			return Currency::getDefaultCurrencySymbol();
	}

	static function update_rates()
	{
		global $log;
		
		$src = AppConfig::setting("locale.defaults.currency_source", 'ECB');
		$requestUrl = AppConfig::setting("locale.currency_sources.$src.url");
		if(! $requestUrl) {
			$src = 'ECB';
			$requestUrl = AppConfig::setting("locale.currency_sources.$src.url");
		}
		if(! $requestUrl) {
			$log->error("Error determining currency data source URL");
			return;
		}
		
		require_once('include/utils/dl_utils.php');
		$req = new RemoteRequest($requestUrl);
		$req->connect_timeout = 30;
		$req->timeout = 30;
		$data = $req->fetch();
		if(! $data) {
			$log->error("Error downloading currency data ($requestUrl)");
			return;
		}		
		self::process_rates($data, $src, $requestUrl);
	}
	
	static function process_rates($data, $src, $requestUrl) {
		global $log, $timedate, $disable_date_format;

		libxml_use_internal_errors(true);
		$xml = simplexml_load_string($data);
		
		if (!$xml) {
			$log->error("Error parsing currency data as XML ($requestUrl)");
			return;
		}
		
		$map = array();
		$dt_start = gmdate('Y-m-d H:i:s');
		
		if($src == 'ECB') {
			$xml->registerXPathNamespace('ns1', 'http://www.ecb.int/vocabulary/2002-08-01/eurofxref');
			$rates = $xml->xpath("/gesmes:Envelope/ns1:Cube/ns1:Cube/ns1:Cube");
			foreach($rates as $rate) {
				$attrs = $rate->attributes();
				if(empty($attrs['currency']) || ! isset($attrs['rate']))
					continue;
				$symbol = (string)$attrs['currency'];
				$theRate = (float)$attrs['rate'];
				if (!empty($symbol))
					$map[$symbol] = $theRate;
			}
			$map['EUR'] = 1;
		}
		else if($src == 'BOC') {
			$by_date = [];
			foreach($xml->xpath("/data/observations/o") as $obsv) {
				$by_date[(string)$obsv->attributes()['d']] = $obsv;
			}
			krsort($by_date);
			if($by_date) {
				$chk_date = key($by_date);
				$obsv = current($by_date);
				foreach($obsv->children() as $rate) {
					if($rate->getName() == 'v') {
						$name = $rate->attributes()['s'];
						if(substr($name, 0, 2) == 'FX') {
							$symbol = substr($name, 2, 3);
							$theRate = (float)((string)$rate);
							if(! empty($symbol) && ! empty($theRate))
								$map[$symbol] = 1/$theRate;
						}
					}
				}
			}
			$map['CAD'] = 1;
		}
		
		$defSymbol = AppConfig::setting('locale.base_currency.iso4217');
		if (!isset($map[$defSymbol])) {
			$log->error("Rate not defined for default currency ($defSymbol)");
			return;
		}
		
		$all = CurrencyUtils::get_list();
		$missing = [];
		foreach ($all as $currency) {
			if (isset($map[$currency->iso4217])) {
				$currency->conversion_rate = 1.0000 / $map[$defSymbol] * $map[$currency->iso4217];
				$currency->last_retrieved = $disable_date_format ? $dt_start : $timedate->to_display_date_time($dt_start);
				$currency->save();
			} else {
				if($currency->status == 'Active') {
					$last_ret = $currency->last_retrieved;
					if(isset($last_ret)) {
						$ts = strtotime($last_ret . ' UTC');
						if($ts) {
							$diff = time() - $ts;
							$missing[] = [$currency->id, $currency->name, $diff];
						}
					}
				}
			}
		}
	}
}

?>
