<?php

require_once 'data/SugarBean.php';

class  CustomerLevel extends SugarBean {

	/**
	 * Formats Customer Level information for signup.
	 */

	public static function formatLevel($level) {
		if ($level instanceof RowResult) {
			$level = $level->row;
		}
		if (!is_array($level)) {
			return [
				'discounts' => ['per_plan' => []],
			];
		}
		$ret = ['discounts' => ['per_plan' => []]];
		if (!empty($level['default_coupon_id'])) {
			if ($coupon = ListQuery::quick_fetch_row('Coupon', $level['default_coupon_id'])) {
				$ret['discounts']['default'] = self::formatCoupon($coupon);
			}
		}
		if (!empty($level['coupons'])) {
			$json = getJSONobj();
			$coupons = $json->decode($level['coupons']);
			if (is_array($coupons)) {
				foreach ($coupons as $c) {
					$plan = ListQuery::quick_fetch_row('Plan', $c['plan_id']);
					if ($plan) {
						$coupon = ListQuery::quick_fetch_row('Coupon', $c['coupon_id']);
						if ($coupon) {
							$row = [
								'plan_id' => $plan['id'],
								'pp_plan_id' => $plan['pp_plan_id'],
								'coupon' => self::formatCoupon($coupon),
								'keep_default' => $c['keep_default'],
							];
							$ret['discounts']['per_plan'][] = $row;
						}
					}
				}
			}
		}
		return $ret;
	}

	private static function formatCoupon($coupon) {
		$copy = ['name', 'id', 'pp_coupon_id', 'discount_mode', 'discount_amount', 'discount_type', 'addon_mode', 'plan_mode', 'discount_percentage', 'discount_type'];
		$ret = [];
		foreach ($copy as $k) $ret[$k] = array_get_default($coupon, $k);

		$lq = new ListQuery('Coupon', null, array('link_name' => 'plans', 'parent_key' => $coupon['id']));
		$plans = $lq->fetchAllRows();
		$ret['plans'] = array_map(function($plan) {
			return [
				'id' => $plan['id'],
				'pp_plan_id' => $plan['pp_plan_id'],
			];
		}, array_values($plans));

		$lq = new ListQuery('Coupon', null, array('link_name' => 'addons', 'parent_key' => $coupon['id']));
		$plans = $lq->fetchAllRows();
		$ret['addons'] = array_map(function($plan) {
			return [
				'id' => $plan['id'],
				'pp_addon_id' => $plan['pp_addon_id'],
			];
		}, array_values($plans));

		return $ret;
	}

}

