<?php

require_once('include/layout/forms/FormTableSection.php');

class CustomerLevelWidget extends FormTableSection {

	function init($params=null, $model=null) {
		parent::init($params, $model);
		if(! $this->id)
			$this->id = 'levels';
	}
	
	function getRequiredFields() {
		return [$this->name];
	}
	
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if($gen->getLayout()->getEditingLayout()) {
			$params = array();
			return $this->renderOuterTable($gen, $parents, $context, $params);		
		}
		return $this->renderHtmlImpl($gen, $row_result, $context, $gen->getLayout()->getDefaultEditable());
    }
	

	function renderHtmlImpl(HtmlFormGenerator $gen, RowResult $row_result, $context, $editable) {
		$rows = $this->fillRows($row_result);
		$dynamic = $this->getDynamicInput($rows, $editable);
	
		if (!$editable) {
			$context['editable'] = true;
		}

		$body = $dynamic->renderHtml($gen, $row_result, $rows, $context);
		if (!$editable) {
			$context['editable'] = false;
		}
		$button = <<<HTML
	<div class="buttons-inline">
		<div class="inline-elt" onclick="var input=SUGAR.ui.getFormInput('DetailForm', 'coupons'); input.insertRow({})" >
			<div id="coupons-coupon-add" class="uii uii-add uii-md active-icon"><label>Add Per-Plan Coupon</label></div>
		</div>
	</div>
HTML;
		if (!$editable) $button = '';
		$body =  <<<HTML
<div class="form-section">
	$body
	$button
</div>
HTML;
        return $body;
	}

	function renderHtmlView(HtmlFormGenerator $gen, RowResult $row_result) {
		$rows = $this->fillRows($row_result);
		$body = 'View';
		$params = [
			'columns' => 1,
			'elements' => [
				[
					'type' => 'static',
					'label' => translate('LBL_BODY', 'Emails'),
					'value' => $body,
				],
			]
		];
		return parent::renderOuterTable($gen, [], [], $params);
	}

	function fillRows(RowResult $row_result) {
		$json = getJSONobj();
		$rows = $json->decode($row_result->getField($this->name));
		unset($row_result->row[$this->name]);
		if (!is_array($rows)) $rows = [
			[
				'coupon_id' => 'wesrfwewe',
				'plan_id' => 'wesrfwewe',
			]
		];

		$coupon_ids = array_column($rows, 'coupon_id');
		$plan_ids = array_column($rows, 'plan_id');

		$lq = new ListQuery('Coupon', ['id', 'name']);
		$lq->addSimpleFilter('id', $coupon_ids);
		$all_coupons = $lq->fetchAllRows();

		$lq = new ListQuery('Plan', ['id', 'name']);
		$lq->addSimpleFilter('id', $plan_ids);
		$all_plans = $lq->fetchAllRows();

		$rows = array_map(function($row) use ($all_coupons, $all_plans) {
			$coupon_id = array_get_default($row, 'coupon_id');
			$plan_id = array_get_default($row, 'plan_id');
			if ($coupon_id && $plan_id) {
				if (isset($all_coupons[$coupon_id]) && isset($all_plans[$plan_id])) {
					return [
						'coupon_id' => $coupon_id,
						'coupon_name' => $all_coupons[$coupon_id]['name'],
						'plan_id' => $plan_id,
						'plan_name' => $all_plans[$plan_id]['name'],
						'keep_default' => empty($row['keep_default']) ? 0 : 1,
					];
				}
			}
		}, $rows);
		$rows = array_filter($rows);
		return $rows;
	}

    function getDynamicInput($rows, $editable) {
        $input_params = array(
            'name' => 'coupons',
			'class' => 'listLayoutConfig',
            'renderer_class' => $editable ? 'CustomerLevelRenderer' : 'CustomerLevelRendererView',
			'label' => translate('LBL_FILTERS', $this->model->module_dir),
			'show_handles' => $editable,
			'show_delete' => $editable,
			'rows' => $rows,
            'cols' => array(
                array(
                    'name' => 'plan_id',
                    'label' => translate('LBL_PLAN'),
                    'width' => '20%',
                    'render_method' => 'renderPlan'
                ),
                array(
                    'name' => 'coupon_id',
                    'label' => translate('LBL_COUPON'),
                    'width' => '20%',
                    'render_method' => 'renderCoupon'
                ),
            )
		);
		if ($editable) {
			$input_params['cols'][] = array(
                'name' => 'keep_default',
                'label' => translate('LBL_KEEP_DEFAULT_COUPON'),
                'width' => '20%',
                'format' => 'check'
            );
		} else {
			$input_params['cols'][] = array(
                'name' => 'keep_default',
                'label' => translate('LBL_KEEP_DEFAULT_COUPON'),
                'width' => '20%',
                'render_method' => 'renderCheckbox',
            );
		}

        $input = $this->getInputWidget('DynamicListInput', $input_params);

        return $input;
    }
	
	function loadUpdateRequest(RowUpdate $update, array $input) {
		$ret = parent::loadUpdateRequest($update, $input);
		if (!empty($input[$this->name])) {
			$json = getJSONobj();
			$rows = $json->decode($input[$this->name]);
			if (!is_array($rows)) {
				$rows = [];
			}
			$rows = array_map(function($row) {
				return [
					'coupon_id' => array_get_default($row, 'coupon_id'),
					'plan_id' => array_get_default($row, 'plan_id'),
					'keep_default' => array_get_default($row, 'keep_default') ? '1' : 0,
				];
			}, $rows);
			$update->set($this->name, $json->encode($rows));
		}
		return $ret;
	}

    function getInputWidget($name, $params) {
        $wspec = AppConfig::get_widget_spec($name);
        if(isset($wspec['params']))
            $params = array_merge($wspec['params'], $params);
        $input = new $wspec['class']($params, $this->model);
        $input->editable = true;

        return $input;
    }

}

