<?php

return (function() {
	global $mod_strings;
	$sep = translate('LBL_SEPARATOR', 'app');


	require_once 'include/layout/forms/FormGenerator.php';
	require_once 'include/config/ModelDef.php';
	require_once 'include/database/ListQuery.php';
	require_once 'include/database/RowUpdate.php';
	require_once 'include/layout/FieldFormatter.php';
	require_once 'include/formbase.php';

	$module = $_REQUEST['module'];
	$model = AppConfig::module_primary_bean($module);
	$return_params = array('return_module', 'return_action', 'return_record', 'return_layout');

	if (!$model || !AppConfig::setting("model.detail.{$model}.personal_data")) {
		ACLController::displayNoAccess();
		return;
	}
	
	if (!ACLController::checkAccess($module, 'manage_personal')) {
		ACLController::displayNoAccess();
		return;
	}

	$row = ListQuery::quick_fetch($model, $_REQUEST['record']);

	if (!$row) {
		ACLController::displayNoAccess();
		return;
	}


	if (!empty($_POST['perform_erase'])) {
		$json = getJSONobj();
		$fields = $json->decode($_POST['fields'], 1);
		if (is_array($fields)) {
			$fields = array_values(array_map(function($f) {return $f['field'];},
				array_filter($fields, function($f) {return $f['erase'];})
			));
			$upd = RowUpdate::for_result($row);
			foreach ($fields as $f) {
				$upd->eraseField($f);
			}
			$upd->save();
			if (!empty($_REQUEST['return_module']) && !empty($_REQUEST['return_action'])) {
				return [
					'redirect',
					[
						'module' => $_REQUEST['return_module'],
						'action' => $_REQUEST['return_action'],
						'record' => array_get_default($_REQUEST, 'return_record'),
						'layout' => array_get_default($_REQUEST, 'return_layout'),
					]
				];
			}
		}
	}


	$fmt = new FieldFormatter;
	$fmt->formatRowResult($row);

	$spec = array(
		'name' => 'fields',
		'vname' => 'LBL_SELECT_FIELDS_ERASE',
		'vname_module' => 'DataPrivacy',
		'widget' => 'DynamicListInput',
		'show_handles' => false,
		'show_delete' => false,
		'max_depth' => 0,
		'width' => '100%',
		'colspan' => 2,
		'cols' => array(
			array(
				'name' => 'label',
				'label' => 'LBL_FIELD',
				'width' => '45%',
			),
			array(
				'name' => 'value',
				'label' => translate('LBL_CURRENT_VALUE', 'DataPrivacy'),
				'width' => '50%',
				'format' => 'html',
			),
			array(
				'name' => 'erase',
				'label' => translate('LBL_ERASE_CHECK', 'DataPrivacy'),
				'width' => '2%',
				'editable' => 1,
				'format' => 'check',
			),
		),
	);
	$checkbox_spec = [
		'name' => 'check_all',
		'vname' => 'LBL_CHECK_ALL',
		'vname_module' => 'DataPrivacy',
		'type' => 'bool',
		'colspan' => 2,
	];
	$layout = array(
		'type' => 'configform',
		'editable' => true,
		'form_buttons' => array(
			'save' => array(
				'icon' => 'icon-accept',
				'vname' => 'LBL_SAVE_BUTTON_LABEL',
			),
			'cancel' => array(
				'icon' => 'icon-cancel',
				'vname' => 'LBL_CANCEL_BUTTON_LABEL',
				'url' => 'index.php?module=DataPrivacy',
			),
		),
		'sections' => array(
			array(
				'id' => 'main',
				'label_position' => 'top',
				'elements' => [$checkbox_spec, $spec],
			),
		),
	);
	$result = new RowResult();
	$result->fields = ['field' => ['type' => 'varchar']];
	$result->row = [];

	$modelDef = new ModelDef($model);
	$fields = $modelDef->getFieldDefinitions();
	$result->row['check_all'] = false;

	foreach($fields as $name => $field) {
		if (empty($field['personal_info'])) continue;
		$result->row['fields'][] = [
			'label' => translate(array_get_default($field, 'vname', array_get_default($field, 'vname_module', $module))),
			'value' => $row->getField($name, null, true),
			'erase' => !!strlen($row->getField($name)),
			'field' => $name,
		];
	}

	$gen = FormGenerator::html_form(null, $layout, 'ErasePersonal');
	$gen->layout->addScriptInclude('modules/DataPrivacy/dataprivacy.js', LOAD_PRIORITY_FOOT);
	$gen->layout->addScriptLiteral('data_privacy_erase_init("ErasePersonal");');
	$hidden = array('module' => $_REQUEST['module'], 'action' => $_REQUEST['action'], 'record' => $_REQUEST['record'], 'perform_erase' => 1);
	foreach ($return_params as $p) {
		$hidden[$p] = array_get_default($_REQUEST, $p);
	}
	$gen->getFormObject()->addHiddenFields($hidden);
	$gen->renderForm($result);
	echo '<div class="detailOuter">' . $gen->getResult() . '</div>';
	$gen->exportIncludes();
})();
