<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/
require_once 'data/SugarBean.php';

class PrivacyActivity extends SugarBean {

	const DEFAULT_ROLE_ID = 'defa6149-01e1-2345-6789-abcdef012345';

	static function get_privacy_enabled_modules() {
		$models = AppConfig::setting('model.index.personal_data');
		$modules = array_map(['AppConfig', 'module_for_model'], $models);
		return $modules;
	}

	static function create_default_role() {
		require_once 'include/database/ListQuery.php';
		require_once 'include/database/RowUpdate.php';
		require_once 'modules/ACLActions/actiondefs.php';
		$existing = ListQuery::quick_fetch('ACLRole', self::DEFAULT_ROLE_ID, ['id'], ['filter_deleted' => false]);
		if ($existing) return;
		$role = RowUpdate::blank_for_model('ACLRole');
		$role->set([
			'id' => self::DEFAULT_ROLE_ID,
			'name' => translate('LBL_DEFAULT_ROLE_NAME', 'DataPrivacy'),
			'description' => translate('LBL_DEFAULT_ROLE_DESC', 'DataPrivacy'),
		]);
		$role->save();
		foreach (['Contacts', 'Leads', 'Prospects'] as $module) {
			$action = RowUpdate::blank_for_model('ACLAction');
			$action->set([
				'name' => 'manage_personal',
				'category' => $module,
				'acltype' => 'module',
				'aclaccess' => ACL_ALLOW_ALL,
				'role_id' => self::DEFAULT_ROLE_ID,
			]);
			$action->save();
		}
		$action = RowUpdate::blank_for_model('ACLAction');
		$action->set([
			'name' => 'access',
			'category' => 'DataPrivacy',
			'acltype' => 'module',
			'aclaccess' => ACL_ALLOW_ENABLED,
			'role_id' => self::DEFAULT_ROLE_ID,
		]);
			$action->save();
		AppConfig::invalidate_cache('acl');
	}
}

