<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');

 
 class ListDiscounts {
	var $focus = null;
	var $list = null;
	var $javascript = '';
	
	function lookupDiscounts($status='Active', $type = 'percentage'){
		require_once('modules/Discounts/Discount.php');
		$this->focus = new Discount();
		if(!empty($type)) {
			$where = "discount_type = '$type'";
			$rows = $this->focus->get_full_list('name', $where);
			$this->focus->retrieve('-99');
			if(is_array($rows)){
				$rows = array_merge(array($this->focus), $rows);
			}else{
				$rows = array($this->focus);	
			}
		} else {
			$rows = AppConfig::db_all_objects('Discount');
		}
		if($status) {
			foreach(array_keys($rows) as $k)
				if($rows[$k]->status != $status)
					unset($rows[$k]);
		}
		foreach($rows as &$row) {
			if ($row->applies_to_selected) {
				$products = ListQuery::quick_fetch_all('Discount', ['id'], ['link_name' => 'products', 'parent_key' => $row->id], true);
				$products = array_map(function($p) {return $p['id'];}, array_values($products));
				$row->products = $products;
			}
		}
		$this->list = $rows;
	}
	
	function handleAdd(){
			global $current_user;
			if($current_user->is_admin){
			if(isset($_POST['edit']) && $_POST['edit'] == 'true' && isset($_POST['name']) && !empty($_POST['name']) && isset($_POST['rate']) && !empty($_POST['rate']) ){
				require_once('modules/Discounts/Discount.php');
				$discount = new Discount();
				if(isset($_POST['record']) && !empty($_POST['record'])){
					$discount->retrieve($_POST['record']);
					$discount->format_all_fields();
				}
				$discount->name = $_POST['name'];
				$discount->status = $_POST['status'];
				$discount->rate = $_POST['rate'];
				$discount->save();
				$this->focus = $discount;
			}
			}
		
	}
	
	function getJavascript() {
		require_once('include/JSON.php');
		$json = new JSON(JSON_LOOSE_TYPE);
		$order = array('-99');
		$all = array();
		$all['-99'] = array(
			'id' => '-99',
			'name' => translate('LBL_NONE', 'app'),
			'rate' => 0,
			'amount' => 0,
			'type' => 'StdPercentDiscount',
			'related_type' => 'Discounts',
		);
		if(!empty($this->list)) {
            foreach($this->list as $data) {
				$order[] = $data->id;
                $all[$data->id] = array(
                	'id' => $data->id,
                	'name' => $data->name,
                	'rate' => $data->rate,
                	'amount' => $data->fixed_amount,
                	'type' => ($data->discount_type == 'percentage' ? 'StdPercentDiscount' : 'StdFixedDiscount'),
					'related_type' => 'Discounts',
					'applies_to_selected' => !!$data->applies_to_selected,
					'products' => $data->applies_to_selected ? $data->products : [],
                );
		    }
		}
		$this->javascript = '<script type="text/javascript">';
		$this->javascript .= 'SysData.discounts_order = ' . (count($order) ? $json->encode($order) : '[]') . ";\n";
		$this->javascript .= 'SysData.discounts = ' . $json->encode($all) . ";\n";
		
	    $this->javascript .= <<<EOS
			function get_discount (id) {
				if(typeof(Discounts[id]) != 'undefined')
					return Discounts[id].rate;
			}
EOS;
		      
		return $this->javascript . "</script>";

        
	}
	
	
	function getSelectOptions($id = ''){
		global $current_user;
		$options = '';
		$this->lookupDiscounts();
		
		if(isset($this->list ) && !empty($this->list )){
			foreach ($this->list as $data){
				if($data->status == "Active"){
					$selected = ($id == $data->id) ? ' selected' : '';
					$rate = str_replace('.00', '', number_format($data->rate, 2));
					$label = $data->name;
					if($data->id != -99)
						$label .= ' ('.$rate.'%)';
					$options .= '<option value="' .$data->id. '"' .$selected. '>' .$label. '</option>';
				}
			}
		}
		return $options;
	}
}

