<?php

require_once 'data/SugarBean.php';


class DynField extends SugarBean
{
	var $table_name = 'fields_meta_data';
	var $object_name = 'DynField';
	var $module_dir = 'DynFields';
	var $new_schema = true;
	

    static function &load_available_fields($module=null, $base_bean_name=null) {
		$rows = AppConfig::db_all_rows('DynField');
		if(isset($module) || isset($base_bean_name)) {
			$ret = array();
			foreach($rows as $r) {
				if(isset($module) && $r['custom_module'] != $module)
					continue;
				if(isset($base_bean_name) && $r['custom_bean'] != $base_bean_name)
					continue;
				$ret[$r['id']] = $r;
			}
			return $ret;
		}
		return $rows;
    }
    
    static function get_custom_field_definitions(array $fields) {
		$defs = array();
		foreach($fields as $spec) {
			$dt = $spec['data_type'];
			$ret = array(
				'name' => $spec['name'],
				'vname' => $spec['label'],
				'type' => $dt,
				'required' => ($spec['required_option'] === 'required'),
				'len' => $spec['max_size'],
				'massupdate' => $spec['mass_update'],
				'audited' => $spec['audited'],
				'personal_info' => array_get_default($spec, 'personal_info'),
				'privacy_consent' => array_get_default($spec, 'privacy_consent'),
				'default' => $spec['default_value'],
				'duplicate_merge' => $spec['duplicate_merge'],
				'mobile_app' => array_get_default($spec, 'mobile_app'),
				'unified_search' => array_get_default($spec, 'unified_search'),
				'help' => $spec['help'],
			);
            if ($dt == 'calculated') {
                $ret['editable'] = false;
                $ret['source']['calc_formula'] = $spec['formula'];
                $ret['type'] = ! empty($spec['data_format']) ? $spec['data_format'] : 'varchar';
            }
			foreach($ret as $k => $v)
				if(! isset($v))
					unset($ret[$k]);
			if($dt === 'enum' || $dt == 'status' || $dt === 'multienum') {
				$ret['options'] = $spec['ext1'];
				// FIXME for multienum, ext2 represents the number of displayed rows - NYI
			}
			if(($dt === 'float' || $dt == 'double') && is_numeric($spec['ext1'])) {
				$ret['decimal_places'] = $spec['ext1'];
			}
			if($dt === 'related_value') {
				$ret['ref'] = $spec['ext1'];
				$ret['field'] = $spec['ext3'];
			}
			if($dt === 'int') {
				$ret['min'] = $spec['ext1'];
				$ret['max'] = $spec['ext2'];
			}
			if($dt === 'ref') {
				// use given name for ID field
				$ret['id_name'] = $ret['name'] . '_id';
				$ret['rname'] = 'id';
				$ret['module'] = $spec['ext1'];
				$ret['bean_name'] = AppConfig::module_primary_bean($spec['ext1']);

				unset($ret['len']);
				unset($ret['help']);
				unset($ret['default']);
			}
			if($dt === 'text' || $dt === 'html') {
				$ret['default'] = $spec['ext4'];
			}
			if($dt == 'date') {
				$ret['default'] = $spec['ext1'];
			}
			if(! empty($spec['ext5'])) {
				$ret['depends_on'] = $spec['ext5'];
				$json = getJSONObj();
				if (strpos($spec['ext6'], '^,^') !== false)
					$ret['depends_on_value'] = explode('^,^', $spec['ext6']);
				else {
					$values = $json->decode($spec['ext6']);
					if ($values === null)
						$values = $spec['ext6'];
					$ret['depends_on_value'] = $values;
				}
				$ext7 = $json->decode($spec['ext7']);
				if ($ext7 && is_array($ext7))
					$ret['conditional_options'] = $ext7;
			}
			
			$defs[$ret['name']] = $ret;
		}
		return $defs;
    }

	static public function translate_label($spec)
	{
		return translate($spec['raw_values']['label'], $spec['raw_values']['custom_module']);
	}
    
	static function after_delete(RowUpdate $upd)
	{
		$bean_name = $upd->getField('custom_bean');
		AppConfig::db_invalidate('DynField');
		AppConfig::invalidate_cache('model');
		if($bean_name) {
			AppConfig::setting("model.custom_field_defs.$bean_name");
			$model = new ModelDef($bean_name);
			$model->getLinkDefinition('_cstm');
		}
	}
	
	static function show_default_value($type, $default, $ext4)
	{
		if($type == 'text' || $type == 'html') return $ext4;
		return $default;
	}
}

