<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/

/* GROUPWARE - modified: 2008-04-06
 * based on IAH 5.2.1 - 4636
 * Quick Campaign feature added, tapping into regular campaign handling
 * 					 Quick Campaigns have special name / title prefix "***"
 * fixed bug: duplicate campaign log bounce/opt out entries were created for recipients,
 *            that had opted out/invalid email property set;
 * changed:		for recipients, that had opted out/invalid email property set,
 *            no log entry at all will now be created, to be consistent with no-log like for suppression handling
 *
*/


require_once('data/SugarBean.php');

class EmailMan extends SugarBean{
	var $id;
	var $deleted;
	var $date_created;
	var $date_modified;
	var $module;
	var $module_id;
	var $marketing_id;
	var $campaign_id;
	var $user_id;
	var $list_id;
	var $invalid_email;
	var $from_name;
	var $from_email;
	var $in_queue;
	var $in_queue_date;
	var $template_id;
	var $send_date_time;
	var $table_name = "emailman";
	var $object_name = "EmailMan";
	var $module_dir = "EmailMan";
	var $send_attempts;
	var $related_id;
	var $related_type;
	var $test=false;
	var $notes_array = array();
	var $restricted_addresses;
	var $restricted_domains;

	var $tracking_url;
	var $tracker_urls;
	var $store_emails;
	var $user_sig;
	var $target_tracker_key;
	var $addedCustomHeaderIdentifier;


	function toString(){
		return "EmailMan:\nid = $this->id ,user_id= $this->user_id module = $this->module , related_id = $this->related_id , related_type = $this->related_type ,list_id = $this->list_id, send_date_time= $this->send_date_time\n";
	}

	var $new_schema = true;


	function remove_queued() {
		global $db;
		$this_id = $db->quote($this->id);
		$query = 'DELETE FROM '. $db->quoteField($this->table_name) . " WHERE id='$this_id'";
		return $db->query($query);
	}

	function set_as_sent($email_address, $success=true, $delete=true, $related_id=null, $related_type=null, $activity_type=null, $no_log=false){
		global $db;

		// no_log is true when this process encounters an emailaddress from the suppression list.
		if ($no_log) {
			$this->remove_queued();
			return;
		}

		$this->send_attempts++;
		if($delete || $this->send_attempts > 5) {
			$log = RowUpdate::blank_for_model('CampaignLog');
			$log->set(array(
				'campaign_id' => $this->campaign_id,
				'marketing_id' => $this->marketing_id,
				'target_tracker_key' => $this->target_tracker_key,
				'activity_type' => $activity_type ? $activity_type : 'send error',
				'activity_date' => gmdate('Y-m-d H:i:s'),
				'list_id' => $this->list_id,
				'more_information' => $email_address,
				'target_type' => $this->related_type,
				'target_id' => $this->related_id,
			));
			if($related_type && $related_id) {
				$log->set('related_type', $related_type);
				$log->set('related_id', $related_id);
			}
			if($this->test)
				$log->set('is_test', 1);
			$ret = $log->validate() && $log->save();
			$this->remove_queued();
			return $ret;
		} else {
			//try to send the email again at a later date. currently this timeperiod is set to one day.
			$this_id = $db->quote($this->id);
			$query = 'UPDATE ' . $db->quoteField($this->table_name) . " SET in_queue='1', send_attempts='$this->send_attempts', in_queue_date='". gmdate('Y-m-d H:i:s') ."' WHERE id = '$this_id'";
			$db->query($query);
		}
	}

	function sendEmail($mail, $testmode=false) {
	    $this->test = $testmode;

		global $current_user, $timedate, $db;
		require_once('modules/Emails/Email.php');
		require_once('modules/EmailFolders/EmailFolder.php');
		require_once('modules/EmailTemplates/EmailTemplate.php');
		require_once('modules/EmailTemplates/TemplateParser.php');
		require_once('modules/EmailMarketing/EmailMarketing.php');
		require_once('modules/Campaigns/Campaign.php');
		require_once('include/utils/html_utils.php');

		$beanName = AppConfig::module_primary_bean($this->related_type);
		if (!$beanName) {
			$this->set_as_sent(null, false, true, null, null, null, true);
			return false;
		}
		$record = ListQuery::quick_fetch($beanName, $this->related_id);
		// FIXME - fetch all related records at once for efficiency when sending campaign
		if(! $record) {
			$this->set_as_sent(null, false, true, null, null, null, true);
			return false;
		}
		$lower_email_address = trim(strtolower($record->getField('email1')));

		$at_pos = strrpos($lower_email_address, '@');
		if(! strlen($lower_email_address) || $at_pos === false) {
			$this->set_as_sent($lower_email_address, false, true, null, null, null, true);
			return false;
		}

		if($record->getField('email_opt_out')) {
			$mail->ErrorInfo .= "\nRecipient Email Opt Out";
			$this->set_as_sent($lower_email_address, false, true, null, null, 'removed', true);
			return false;
		}
		if($record->getField('invalid_email')) {
			$mail->ErrorInfo .= "\nRecipient Invalid Email";
			$this->set_as_sent($lower_email_address, false, true, null, null, 'invalid email', true);
			return false;
		}

		// get tracking entities locations.
		if (!isset($this->tracking_url) || ! isset($this->store_emails)) {
		    if (AppConfig::setting('massemailer.tracking_entities_location_type') == '2')
		    	$this->tracking_url = AppConfig::setting('massemailer.tracking_entities_location');
		    if(! $this->tracking_url)
				$this->tracking_url = AppConfig::site_url();
			$this->tracking_url = preg_replace('~/$~', '', $this->tracking_url);
			if(AppConfig::setting('massemailer.email_copy') == '2') {
				$this->store_emails = false;
			} else {
				$this->store_emails = true;
			}
		}

		// test against individual address.
		if ($this->restricted_addresses and isset($this->restricted_addresses[$lower_email_address])) {
			$this->set_as_sent($lower_email_address, true, true, null, null, null, true);
			return true;
		}
		// test against restricted domains
		if($this->restricted_domains) {
			foreach ($this->restricted_domains as $domain=>$value) {
				$pos = strrpos($lower_email_address, $domain);
				if ($pos !== false && $pos > $at_pos) {
					//found
					$this->set_as_sent($lower_email_address, true, true, null, null, null, true);
					return true;
				}
			}
		}
		// test for duplicate email address.
		if (!empty($this->campaign_id)) {
			$dup_query = "SELECT id FROM campaign_log WHERE more_information='".$db->quote($lower_email_address)."' AND campaign_id='".$db->quote($this->campaign_id)."' AND marketing_id != '' AND marketing_id = '" . $db->quote($this->marketing_id) . "' AND NOT is_test";
			$dup = $db->query($dup_query, true);
			$dup_row = $db->fetchByAssoc($dup);
			if (!empty($dup_row)) {
				//email address was processed //silent delete this entry.
				$this->set_as_sent($lower_email_address, true, true, null, null, null, true);
				return true;
			}
		}

		$start = microtime();
		$this->target_tracker_key = create_guid();

		// fetch email marketing.
		if (empty($this->current_emailmarketing) || $this->current_emailmarketing->getPrimaryKeyValue() != $this->marketing_id) {
			$this->current_emailmarketing = ListQuery::quick_fetch('EmailMarketing', $this->marketing_id);
			if(! $this->current_emailmarketing) {
				$this->set_as_sent($lower_email_address, false, true, null, null, null, true);
				return false;
			}
		}
		$template_id = $this->current_emailmarketing->getField('template_id');
		$inbound_email_id = $this->current_emailmarketing->getField('inbound_email_id');
		$campaign_id = $this->current_emailmarketing->getField('campaign_id');
		$from_name = $this->current_emailmarketing->getField('from_name');
		$no_auto_optout = $this->current_emailmarketing->getField('no_auto_optout');
		$no_auto_optin = $this->current_emailmarketing->getField('no_auto_optin');

		// fetch campaign details..
		if (empty($this->current_campaign) || $this->current_campaign->getPrimaryKeyValue() != $campaign_id) {
			$this->current_campaign = ListQuery::quick_fetch('Campaign', $campaign_id);
			if(! $this->current_campaign) {
				$this->set_as_sent($lower_email_address, false, true, null, null, null, true);
				return false;
			}
			$track_query = $this->current_campaign->getLinkQuery('tracked_urls');
			$this->tracked_urls = $track_query->fetchAllRows();
			$this->has_optout_links=false;
			foreach($this->tracked_urls as $url) {
				if($url['optout_type'] == 'optout' || $url['optout_type'] == 'optout-all')
					$this->has_optout_links = true;
			}
		}

		// fetch email template associate with the marketing message.
		if (empty($this->current_emailtemplate) or $this->current_emailtemplate->getPrimaryKeyValue() !== $template_id) {
			$this->current_emailtemplate = ListQuery::quick_fetch('EmailTemplate', $template_id);
			if(! $this->current_emailtemplate) {
				$this->set_as_sent($lower_email_address, false, true, null, null, null, true);
				return false;
			}

			$this->notes_array = $this->current_emailtemplate->getLinkQuery('notes')->fetchAllObjects(null, false);
		}

		if (empty($this->current_mailbox) || $this->current_mailbox->getPrimaryKeyValue() != $inbound_email_id) {
			$this->current_mailbox = ListQuery::quick_fetch('EmailPOP3', $inbound_email_id);
			if(! $this->current_mailbox) {
				$this->set_as_sent($lower_email_address, false, true, null, null, null, true);
				return false;
			}
		}
		$from_addr = trim($this->current_mailbox->getField('email'));
		if (! strlen($from_addr))
			$from_addr = AppConfig::setting('notify.from_address');
		$return_path = trim($this->current_mailbox->getField('return_path'));
		$reply_to = trim($this->current_mailbox->getField('reply_to'));
		if (! strlen($reply_to))
			$reply_to = $from_addr;
		if (! strlen($return_path))
			$return_path = $from_addr;

		if (!$this->addedCustomHeaderIdentifier) {
			$mail->addCustomHeader('1CRM-Tracker-Identifier', $this->target_tracker_key);
			$this->addedCustomHeaderIdentifier = true;
		}


		$mail->ClearAllRecipients();
		$mail->ClearReplyTos();
		$mail->Sender	= $return_path;
		$mail->From     = $from_addr;
		$mail->FromName = $from_name;
		$mail->AddReplyTo($reply_to, $from_name);

		$subject = $this->current_emailtemplate->getField('subject');
		$body = $this->current_emailtemplate->getField('body');
		$body_html = $this->current_emailtemplate->getField('body_html');
		if(! strlen($body))
			$body = html2plaintext($body_html);
		else if(! strlen($body_html))
			$body_html = nl2br($body);

		// parse and replace bean variables.
		$template_data = TemplateParser::parse_generic(
			array(
				'subject' => $subject,
				'body_html' => $body_html,
				'body' => $body,
			),
			array(
				$this->related_type => $this->related_id,
			),
			array(
				'aliases' => array($this->related_type => 'Contacts'),
			)
		);

		// GROUPWARE - begin modified: Quick Campaign
		//    - introduce special email template variables for [SIGNATURE], [SENDER_FIRSTNAME], [SENDER_LASTNAME],[SENDER_TITLE].
		if(! isset($this->user_sig))
			$this->user_sig = $current_user->getDefaultSignature(false);
		$sig = '';
		if($this->user_sig) {
			if(! empty($user_sig['signature_html']))
				$sig = $user_sig['signature_html'];
			else if(! empty($user_sig['signature']))
				$sig = $user_sig['signature'];
		}
		$template_data['body_html'] = str_replace( '[SIGNATURE]' , $sig, $template_data['body_html'] );
		$template_data['body_html'] = str_replace( '[SENDER_FIRSTNAME]' , $current_user->first_name, $template_data['body_html'] );
		$template_data['body_html'] = str_replace( '[SENDER_LASTNAME]' , $current_user->last_name, $template_data['body_html'] );
		$template_data['body_html'] = str_replace( '[SENDER_TITLE]' , $current_user->title, $template_data['body_html'] );
		// GROUPWARE - end modified: Quick Campaign

		// parse and replace urls.
		// this is new style of adding tracked urls to a campaign.
		$tracker_url_template = $this->tracking_url . '/campaign_trackerv2.php?track=%s'.'&identifier='.$this->target_tracker_key;
		$removeme_url_template = $this->tracking_url . '/removeme.php?track=%s&identifier='.$this->target_tracker_key;
		$optin_url_template = $this->tracking_url . '/optin.php?track=%s&identifier='.$this->target_tracker_key;

		$lq = new ListQuery('Campaign', null, array('link_name' => 'tracked_urls'));
		$lq->setParentKey($this->campaign_id);
		$tracker_urls =  $lq->fetchAllRows();
		$urls_replaced = EmailTemplate::parse_tracker_urls($template_data,$tracker_url_template,$tracker_urls,$removeme_url_template, $optin_url_template);
		$template_data = $urls_replaced['parsed'];
		$replaced_optout = $urls_replaced['optout'];
		$replaced_optin = $urls_replaced['optin'];
		$contact_name = trim($record->getField('first_name') . ' ' . $record->getField('last_name'));
		$mail->AddAddress($lower_email_address, $contact_name);
		$mail->Subject = $template_data['subject'];

		$mail->Body = wordwrap($template_data['body_html'], 900);
		$mail->AltBody = $template_data['body'];

		// do not add the default remove me link if the campaign has a trackerurl of the opotout link
		if (!$replaced_optout && !$no_auto_optout) {
			$optout_text = to_html(translate('TXT_REMOVE_ME', 'EmailMan'));
			$optout_label = to_html(translate('TXT_REMOVE_ME_CLICK', 'EmailMan'));
			$optout_text_alt = translate('TXT_REMOVE_ME_ALT', 'EmailMan');
			$mail->Body .= "<br><br><font size='2'>{$optout_text}<a href='". $this->tracking_url . "/removeme.php?identifier={$this->target_tracker_key}'>{$optout_label}</a></font>";
			$mail->AltBody .= "\n\n\n{$optout_text_alt} ". $this->tracking_url . "/removeme.php?identifier={$this->target_tracker_key}";
		}

		if (!$replaced_optin && !$no_auto_optin) {
			$optout_text = to_html(translate('TXT_OPTIN', 'EmailMan'));
			$optout_label = to_html(translate('TXT_OPTIN_CLICK', 'EmailMan'));
			$optout_text_alt = translate('TXT_OPTIN_ALT', 'EmailMan');
			$mail->Body .= "<br><br><font size='2'>{$optout_text}<a href='". $this->tracking_url . "/optin.php?identifier={$this->target_tracker_key}'>{$optout_label}</a></font>";
			$mail->AltBody .= "\n\n\n{$optout_text_alt} ". $this->tracking_url . "/optin.php?identifier={$this->target_tracker_key}";
		}
		// add image reference to track opening of html emails.
		$mail->Body .= '<img height="1" width="1" src="' . to_html("{$this->tracking_url}/image.php?identifier={$this->target_tracker_key}") . '">';

		// cn: bug 4684, handle attachments in email templates.
		$mail->handleAttachments($this->notes_array);

		if($this->store_emails) {
			$mailData = array(
				'subject' => $mail->Subject,
				'body' => $mail->Body,
				'alt_body' => $mail->AltBody,
				'from' => $mail->From,
			);
		}

		$hooks = AppConfig::setting("model.hooks.EmailMan.before_send", []);
		
		foreach($hooks as $hook) {
			call_hook_function($hook, null, [$this, $mail]);
		}

		$track_id = create_guid();
		$mail->prepForOutbound(true, $track_id);

		$success = $mail->send();
		$related_email_id = null;
		if($success && $this->store_emails) {
			$email = new Email();
			$email->id = $track_id;
			$email->new_with_id = true;

			// GROUPWARE - begin modified: Quick Campaign - remove full name from recipient of email
			$email->to_addrs = $lower_email_address;
			// GROUPWARE - end modified: Quick Campaign

			$email->to_addrs_ids = $this->related_id .';';
			$email->to_addrs_names = $contact_name . ';';
			$email->to_addrs_emails = $lower_email_address . ';';
			$email->type = 'archived';
			$email->deleted = '0';

			// GROUPWARE - begin modified: Quick Campaign - set campaign message name
			$campaign_name = $this->current_campaign->getField('name');
			if ( (substr($campaign_name, 0, 3) ) == '***') {
				$email->name = '*** Quick Campaign: '.$mailData['subject'];
			}
			else {
				$email->name = $campaign_name.': '.$mailData['subject'];
			}
			// GROUPWARE - end modified: Quick Campaign

			$email->setDescriptionHTML($mailData['body']);
			$email->setDescription($mailData['alt_body']);
			$email->folder = EmailFolder::get_std_folder_id($this->user_id, STD_FOLDER_SENT);
			$email->isread = 1;
			$email->from_addr = $mailData['from'];
			$email->assigned_user_id = $this->user_id;
			$email->parent_type = $this->related_type;
			$email->parent_id = $this->related_id;
			$email->campaign_id = $campaign_id;
			$email->date_start = $timedate->to_display_date_time(gmdate('Y-m-d H:i:s'));
			$email->status = 'sent';
			$related_email_id = $email->save();

			foreach($this->notes_array as $note) {
				// create "audit" email without duping off the file to save on disk space
				$noteAudit = new Note();
				$noteAudit->parent_id = $related_email_id;
				$noteAudit->parent_type = $email->module_dir;
				$noteAudit->name = translate('LBL_ATTACHMENT', 'EmailMan');
				$noteAudit->description = "[".$note->filename."] ".translate('LBL_ATTACHMENT_AUDIT', 'EmailMan');
				$noteAudit->assigned_user_id = $this->user_id;
				$noteAudit->save();
				$noteAudit->cleanup();
			}

			$email->cleanup();
		}

		if ($success) {
			$this->set_as_sent($lower_email_address, $success, $success, $related_email_id, $related_email_id ? 'Emails' : null, 'targeted');
		} else {
			//log send error.
			$this->set_as_sent($lower_email_address, $success, $this->test);
		}

		return $success;
	}
}
