<?php

require_once 'include/layout/FieldFormatter.php';

class TemplateParser
{
	static $fields = array();
	static $row_cache = array();

	static function parse_generic($text, array $objects, $params = array(), $html_idx = array())
	{
		$is_array = is_array($text);
		if ($is_array) {
			$texts =& $text;
			$ret = array();
		} else {
			$texts = array($text);
		}
		$pattern = '~\\$([A-Za-z][A-Za-z0-9_]*\\b)~';
		$disable_cache = array_get_default($params, 'disable_cache');
		$aliases = array_get_default($params, 'aliases', array());
		$model_aliases = array_get_default($params, 'model_aliases', array());
		$primary = array_get_default($params, 'primary', false);
		$user = array_get_default($params, 'user', $GLOBALS['current_user']);
		if($disable_cache) self::$row_cache = array(); // clear previous, but use cache during this call
		
		$prefixes = array();
		$fields = array();
		$row = array();

		foreach ($objects as $module => $id) {
			if (isset($model_aliases[$module])) {
				list($module, $alias) = $model_aliases[$module];
			} else {
				$alias = strtolower(AppConfig::module_primary_bean(array_get_default($aliases, $module, $module)));
			}
			$model = AppConfig::module_primary_bean($module);
			$obj_prefixes = [];
			$prefix = $alias . '_';
			$obj_prefixes[] = $prefix;
			$fields[$prefix] = self::getFieldDefs($model);
			if ($alias == 'acase') {
				$alias = 'case';
				$obj_prefixes[] = 'case_';
				$fields['case_'] = $fields[$prefix];
			}
			array_extend($prefixes, $obj_prefixes);
			$r = self::fetch($model, $id, $user);
			if ($r) {
				foreach ($obj_prefixes as $prefix) {
					foreach ($r as $k => $v) {
						$row[$prefix . $k] = $v;
					}
					$row[$prefix . 'iah_url'] = AppConfig::site_url() . 'index.php?module=' . $module . '&action=DetailView&record=' . $r['id'];
					$row[($prefix == 'record_' ? '' : $prefix) . 'record_url'] = $row[$prefix . 'iah_url'];
				}

				if ((!$primary || $primary == $module) && $module != 'Accounts' && empty($objects['Accounts'])) {
					$acc_id = '';
					$acc_primary = AppConfig::setting("model.detail.$model.primary_account");
					if($acc_primary && isset($fields[$prefix][$acc_primary])) {
						$acc_id = array_get_default($fields[$prefix][$acc_primary], 'id_name', $acc_primary.'_id');
					} else {
						foreach ($fields[$prefix] as $k => $def) {
							if (array_get_default($def, 'type') == 'ref' && array_get_default($def, 'bean_name') == 'Account') {
								$acc_id = array_get_default($def, 'id_name', $k . '_id');
								break;
							}
						}
					}
					if ($acc_id) {
						$acc_row = self::fetch('Account', $r[$acc_id], $user);
						if ($acc_row) {
							foreach ($acc_row as $k => $v) {
								$row['account_' . $k] = $v;
								foreach ($obj_prefixes as $prefix) {
									$row[$prefix . 'account_' . $k] = $v;
								}
							}
							$row['account_record_url'] = AppConfig::site_url() . 'index.php?module=Accounts&action=DetailView&record=' . $acc_row['id'];
							$row['account_iah_url'] = $row['account_record_url'];
							foreach ($obj_prefixes as $prefix) {
								$row[$prefix . 'account_record_url'] = $row['account_record_url'];
								$row[$prefix . 'account_iah_url'] = $row['account_iah_url'];
							}
						}
					}
				}
			}
		}

		foreach ($texts as $key => $txt) {
			$matches = array();
			preg_match_all($pattern, $txt, $matches, PREG_OFFSET_CAPTURE);
			$offset = 0;
			foreach ($matches[1] as $match) {
				$field_name = $match[0];
				$default = false;
				foreach($prefixes as $prefix) {
					if(substr($field_name, 0, strlen($prefix)) == $prefix && substr($field_name, -5) == '_name') {
						$alt_name = substr($field_name, strlen($prefix)+1, -5);
						if(isset($fields[$prefix][$alt_name]) && $fields[$prefix][$alt_name]['type'] == 'ref')
							$default = array_get_default($row, $prefix.$alt_name, $default);
					}
				}
				$value = array_get_default($row, $field_name, $default);
				if (in_array($key, $html_idx)) {
					$value = nl2br(url2html(htmlspecialchars($value)));
				}

				if ($value !== false) {
					$match_len = strlen($match[0]);
					$txt = substr_replace($txt, $value, $offset + $match[1] - 1, $match_len + 1);
					$offset += strlen($value) - $match_len - 1;
				}
			}

			if (empty($params['preserve_placeholders']))
				$txt = preg_replace($pattern, '', $txt);
			if ($is_array)
				$ret[$key] = $txt;
			else
				$ret = $txt;
		}
		
		return $ret;
	}

	private static function fetch($model, $id, $user, $disable_cache = false)
	{
		static $formatters = array();
		$key = $model . '_' . $id . '_' . $user->id;
		if (!$disable_cache && isset(self::$row_cache[$key])) {
			return self::$row_cache[$key];
		}
		$fields = self::getFieldDefs($model);
		foreach($fields as $idx => $f)
			if(! empty($f['protected']))
				unset($fields[$idx]);
		$result = ListQuery::quick_fetch($model, $id, array_keys($fields), array('process_results' => true));
		$row = array();
		if ($result) {
			$row = $result->row;
			foreach ($row as $k => $v) {
				if (!isset($fields[$k]))
					continue;
				$def = $fields[$k];
				if ($def['type'] == 'enum' || $def['type'] == 'status') {
					$row[$k] = translate($def['options'], $model, (string)$v);
					if (is_array($row[$k])) $row[$k] = '';
				}
			}
			$result->row = $row;
		}
		if (empty($formatters[$user->id])) {
			$formatter = new FieldFormatter('plain', 'email_template');
			$formatter->disable_relative_time = true;
			$formatter->setUser($user);
			$formatters[$user->id] = $formatter;
		} else {
			$formatter = $formatters[$user->id];
		}
		if ($result) {
			$row = $formatter->formatRowResult($result);
		}
		if(! $disable_cache)
			self::$row_cache[$key] = $row;
		return $row;
	}

	private static function getFieldDefs($model)
	{
		if (!isset(self::$fields[$model])) {
			$def = new ModelDef($model);
			self::$fields[$model] = $def->getFieldDefinitions();
		}
		return self::$fields[$model];
	}
}

