<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('modules/Emails/widgets/EmailWidget.php');
require_once 'include/database/ListQuery.php';

class EmailTemplateWidget extends EmailWidget {

	const DEFAULT_INSERT_MODULE = 'Accounts';

	var $campaign_id;

	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {

		if ($gen->getLayout()->getDefaultEditable())
            return $this->renderHtmlEdit($gen, $row_result, $context);

        return $this->renderHtmlView($gen, $row_result);
	}
	
	function getRequiredFields() {
        $fields = array('body', 'body_html', 'subject', 'campaign_id');

		return $fields;
	}
	
    /**
     * Render HTML for Email Template plain text body and attachments
     *
     * @param HtmlFormGenerator $gen
     * @param RowResult $row_result
     * @return string
     */
    function renderHtmlView(HtmlFormGenerator $gen, RowResult $row_result) {
        global $mod_strings;

        $attachments = '&nbsp;';
        $record = $row_result->getField('id');
		$new_style = ($gen->getLayout()->getType() == 'record');
		$lpos = $gen->getLayout()->getDefaultLabelPosition();

        if ($record)
            $attachments = $this->renderAttachmentsView($row_result->getModuleDir(), $record, $mod_strings['LBL_ATTACHMENT_VIEW']);

        $body = <<<EOQ
            <table width="100%" border="0" cellspacing="0" cellpadding="0" class="tabDetailView" style="margin-top: 0.5em">
            {$this->renderPlainRow($gen, $row_result)}
EOQ;
		if($lpos == 'top') {
		$body .= <<<EOQ
            <tr>
            <td class="tabDetailViewDF">
            	<p class="topLabel">{$mod_strings['LBL_ATTACHMENTS']}</p>
            	<div class="fieldValue">{$attachments}</div>
			</td>
			</tr>
EOQ;
		} else {
		$body .= <<<EOQ
            <tr>
            <td class="tabDetailViewDL" width="20%">{$mod_strings['LBL_ATTACHMENTS']}</td>
            <td class="tabDetailViewDF" colspan="3" width="80%">{$attachments}</td>
            </tr>
EOQ;
		}

		$body .= '</table>';

        $layout = $gen->getLayout();
        $layout->addScriptInclude('modules/EmailTemplates/template.js', LOAD_PRIORITY_FOOT);

        return $body;
    }

	function renderHtmlEdit(HtmlFormGenerator $gen, RowResult $row_result, $context) {
        global $mod_strings, $app_list_strings, $app_strings;

        $record = $row_result->getField('id');

        $subject = $gen->form_obj->renderField($row_result, 'subject');
        $template_body_html = $gen->form_obj->renderField($row_result, 'body_html');
        $input = $_REQUEST;
		$attachments = $this->renderAttachmentsEdit($gen, $row_result, $record, $input);
        $upload = $this->renderAttachmentUpload($gen);
		$new_style = ($gen->getLayout()->getType() == 'record');
		$lpos = $gen->getLayout()->getDefaultLabelPosition();
        $body = "";

        $modules_opts = array(
            'Accounts' => $app_list_strings['moduleList']['Accounts'],
            'Contacts' => $mod_strings['LBL_CONTACT_AND_OTHERS'],
            'Project' => $mod_strings['LBL_PROJECT'],
            'ProjectTask' => $mod_strings['LBL_PROJECT_TASK'],
            'Cases' => $mod_strings['LBL_CASE'],
            'Bugs' => $mod_strings['LBL_BUG'],
        );
        $spec = array('name' => 'variable_module', 'options' => $modules_opts, 'options_add_blank' => false,  'onchange' => 'add_variables();');
        $modules_select = $gen->form_obj->renderSelect($spec, EmailTemplateWidget::DEFAULT_INSERT_MODULE);

        $modules_fields = $this->getModulesFields();
        $default_fields = $modules_fields[EmailTemplateWidget::DEFAULT_INSERT_MODULE];
        $fields_opts = array();
        
		$fieldsCount = is_countable($default_fields) ? sizeof($default_fields) : 0;
        for ($i = 0; $i < $fieldsCount; $i++) {
            $key = '$' . $default_fields[$i]['name'];
            $value = $default_fields[$i]['value'];
            $fields_opts[$key] = $value;
		}

        $spec = array('name' => 'variable_name', 'options' => $fields_opts, 'onchange' => 'show_variable();', 'width' => '12em');
        $fields_select = $gen->form_obj->renderSelect($spec, '');

        $spec = array('name' => 'variable_text', 'width' => 30);
		$variable_input = $gen->form_obj->renderText($spec, '');

        $tracker_urls = '';
        if (! empty($this->campaign_id))
            $tracker_urls = $this->renderTrackerUrls($gen);

        $body .= <<<EOQ
            <input type="hidden" name="remove_attachments" value="" id="remove_attachments" />
            <table width="100%" border="0" cellspacing="0" cellpadding="0" class="tabForm" style="margin-top: 0.5em">
EOQ;

		if($lpos == 'top') {
		$body .= <<<EOQ
		    <tr>
            <td class="dataField">
            	<p class="topLabel">{$mod_strings['LBL_SUBJECT']}<span class="requiredStar">*</span></p>
            	{$subject}
            </td>
            <tr>
            <td class="dataField">
				<p class="topLabel">{$mod_strings['LBL_INSERT_VARIABLE']}</p>
				{$modules_select}
				{$fields_select}
				<span class="dataLabel">:</span>
				{$variable_input}
				<button type='button' tabindex="70" onclick='insert_variable();' class='input-button input-outer'><span class="input-label">{$mod_strings['LBL_INSERT']}</span></button>
			</td>
            </tr>
            {$tracker_urls}
            <tr>
            <td class="dataField">
            	<p class="topLabel">{$mod_strings['LBL_BODY']}</p>
            	{$template_body_html}
			</td>
            </tr>
            {$this->renderPlainRow($gen, $row_result, true)}
            <tr>
            <td class="dataField">
            	<p class="topLabel">{$mod_strings['LBL_ATTACHMENTS']}</p>
				{$attachments}{$upload}
            </td>
            </tr>
            </table>
EOQ;
		}
		else {
		$body .= <<<EOQ
		    <tr>
            <td class="dataLabel" width="20%">{$mod_strings['LBL_SUBJECT']}<span class="requiredStar">*</span></td>
            <td class="dataField" colspan="3" width="80%">{$subject}</td>
            </tr>
            <tr>
                <td class="dataLabel" align="left">{$mod_strings['LBL_INSERT_VARIABLE']}</td>
                <td class="dataField" colspan="3">
                    {$modules_select}
                    {$fields_select}
                    <span class="dataLabel">:</span>
                    {$variable_input}
                    <button type='button' tabindex="70" onclick='insert_variable();' class='input-button input-outer'><span class="input-label">{$mod_strings['LBL_INSERT']}</span></button>
                </td>
            </tr>
            {$tracker_urls}
            <tr>
            <td class="dataLabel" width="20%">{$mod_strings['LBL_BODY']}</td>
            <td class="dataField" colspan="3" width="80%">{$template_body_html}</td>
            </tr>
            {$this->renderPlainRow($gen, $row_result, true)}
            <tr>
            <td class="dataLabel" width="20%">{$mod_strings['LBL_ATTACHMENTS']}</td>
            <td class="dataField" colspan="3" width="80%">{$attachments}{$upload}</td>
            </tr>
            </table>
EOQ;
		}

        $layout = $gen->getLayout();
        $layout->addScriptInclude('modules/Emails/Email.js');
		$layout->addJSLanguage('Emails');
        $this->addAttachmentsJs($layout);

        $json = getJSONObj();
        $fields_js = "var field_defs = " . $json->encode($modules_fields) . ';';
        $layout->addScriptLiteral($fields_js, LOAD_PRIORITY_FOOT);

        $layout->addScriptInclude('modules/EmailTemplates/template.js', LOAD_PRIORITY_FOOT);

        return $body;
	}


    /**
     * Render Tracker URLs row
     *
     * @param HtmlFormGenerator $gen
     * @return string
     */
    function renderTrackerUrls(HtmlFormGenerator $gen) {
        global $app_strings, $mod_strings;

		$new_style = ($gen->getLayout()->getType() == 'record');
		$lpos = $gen->getLayout()->getDefaultLabelPosition();
        $trackers = $this->getTrackerURLs();
        $tracker_opts = array('' => $app_strings['LBL_NONE']);
        foreach ($trackers as $tracker) {
            $tracker_opts[$tracker['id']] = $tracker['tracker_name'];
        }

        $spec = array('name' => 'tracker_id', 'options' => $tracker_opts, 'width' => '25em', 'onchange' => 'fill_tracker();');
        $tracker_select = $gen->form_obj->renderSelect($spec, '');

        $spec = array('name' => 'tracker_url', 'width' => 30);
        $tracker_url = $gen->form_obj->renderText($spec, '');

        $spec = array('name' => 'tracker_name', 'width' => 30);
        $tracker_name = $gen->form_obj->renderText($spec, '');

		if($lpos == 'top') {
        $html =<<<EOQ
            <tr>
                <td class="dataField">
                	<p class="topLabel">{$mod_strings['LBL_INSERT_TRACKER_URL']}</p>
                	<div class="fieldValue">
                    {$tracker_select}
                    {$tracker_url}
					{$tracker_name}
                    <button type='button' tabindex="70" onclick='insert_tracker();' class='input-button input-outer'><span class="input-label">{$mod_strings['LBL_INSERT_URL_REF']}</span></button>
                    </div>
                </td>
            </tr>
EOQ;
		} else {
        $html =<<<EOQ
            <tr>
                <td class="dataLabel" align="left">{$mod_strings['LBL_INSERT_TRACKER_URL']}</td>
                <td class="dataField" colspan="3">
                    {$tracker_select}
                    {$tracker_url}
					{$tracker_name}
                    <button type='button' tabindex="70" onclick='insert_tracker();' class='input-button input-outer'><span class="input-label">{$mod_strings['LBL_INSERT_URL_REF']}</span></button>
                </td>
            </tr>
EOQ;
		}
		
        $layout = $gen->getLayout();
        $json = getJSONObj();
        $trackers_js = "var trackers = " . $json->encode($trackers) . ';';
        $layout->addScriptLiteral($trackers_js, LOAD_PRIORITY_FOOT);

        return $html;
    }

    /**
     * Render html for view / edit plain text row
     * 
     * @param HtmlFormGenerator $gen
     * @param RowResult $row_result
     * @param bool $edit
     * @return string
     */
    function renderPlainRow(HtmlFormGenerator $gen, RowResult $row_result, $edit = false) {
        global $mod_strings;

        $checked = 0;
        $style = 'style = "display: none;padding: 5px;"';
		$new_style = ($gen->getLayout()->getType() == 'record');
		$lpos = $gen->getLayout()->getDefaultLabelPosition();

        if (! $edit) {
            $body = $row_result->getField('body');

            if (! empty($body)) {
                $checked = 1;
                $style = 'style = "padding: 5px;"';
            }

            $row_value = $body;
            $label_class = 'tabDetailViewDL';
            $value_class = 'tabDetailViewDF';
            $label = $mod_strings['LBL_SHOW_ALT_TEXT'];
        } else {
            $row_value =  $gen->form_obj->renderField($row_result, 'body');
            $label_class = 'dataLabel';
            $value_class = 'dataField';
            $label = $mod_strings['LBL_EDIT_ALT_TEXT'];
        }

        $spec = array('name' => 'show_plain', 'onchange' => 'show_plain();');
        $show_plain = $gen->form_obj->renderCheck($spec, $checked);

		if($lpos == 'top') {
        $row = <<<EOQ
            <tr>
            <td class="{$value_class}">
                <label>{$show_plain}&nbsp;&nbsp;{$label}</label><br>
                <div id="plain_text_div" {$style}><pre>{$row_value}</pre></div>
            </td></tr>
EOQ;
		} else {
        $row = <<<EOQ
            <tr>
            <td valign="top" valign="top" class="{$label_class}">&nbsp;</td>
            <td colspan="3" class="{$value_class}">
                <label>{$show_plain}&nbsp;&nbsp;{$label}</label><br>
                <div id="plain_text_div" {$style}><pre>{$row_value}</pre></div>
            </td></tr>
EOQ;
		}
		
        return $row;
    }

    /**
     * Get modules fields list for Insert variable feature
     *
     * @return array
     */
    function getModulesFields() {
        $contact = new ModelDef('Contact');
        $lead = new ModelDef('Lead');
        $prospect = new ModelDef('Prospect');
        $account = new ModelDef('Account');
        $project = new ModelDef('Project');
        $case = new ModelDef('aCase');
        $bug = new ModelDef('Bug');
        $project_task = new ModelDef('ProjectTask');

        // add fields here that would not make sense in an email template
        $badFields = array(
        	'id_c',
            'account_description',
            'contact_id',
            'lead_id',
            'opportunity_amount',
            'opportunity_id',
            'opportunity_name',
            'opportunity_role_id',
            'opportunity_role_fields',
            'opportunity_role',
            'campaign_id',
        );

        $field_defs = array(
            'Contacts' => array($contact->getFieldDefinitions(), 'contact_'),
            'Leads' => array($lead->getFieldDefinitions(), 'contact_'),
            'Prospects' => array($prospect->getFieldDefinitions(), 'contact_'),
            'Accounts' => array($account->getFieldDefinitions(), 'account_'),
            'Project' => array($project->getFieldDefinitions(), 'project_'),
            'ProjectTask' => array($project_task->getFieldDefinitions(), 'projecttask_'),
            'Cases' => array($case->getFieldDefinitions(), 'case_'),
            'Bugs' => array($bug->getFieldDefinitions(), 'bug_'),
        );

        $field_defs_array = array();

        foreach ($field_defs as $module => $data) {
            $fields = array();
            foreach($data[0] as $field_def) {
                if(	($field_def['type'] == 'assigned_user' || $field_def['type'] =='link') ||
                    ($field_def['type'] == 'bool') ||
                    (in_array($field_def['name'], $badFields)) ) {

                    continue;
                }
                if( $field_def['type'] == 'enum' && ! empty($field_def['multi_select_group']))
                	continue; // skip category fields
                if(empty($field_def['vname']))
                	continue;

                $lang_module = $module;
                if (isset($field_def['original_module'])) {
                    $lang_module = $field_def['original_module'];
                }

				$vname_prefix = '';
				if (!empty($field_def['for_ref'])) {
					$ref_def = array_get_default($data[0], $field_def['for_ref']);
					if ($ref_def) {
						$lang_module = array_get_default($ref_def, 'vname_module', $lang_module);
					}
					$field_def['vname'] = $ref_def['vname'];
					$vname_prefix = '[ID] ';
				}
                $field_def['vname'] = $vname_prefix . preg_replace('/:$/','',translate($field_def['vname'], $lang_module));
                $temp_Value = array('name' => $data[1] . $field_def['name'], 'value' => $field_def['vname']);

				$field_defs_array[$module][$temp_Value['name']] = $temp_Value;
            }
        }

        $field_defs_array['Contacts'] += $field_defs_array['Leads'];
        $field_defs_array['Contacts'] += $field_defs_array['Prospects'];
        unset($field_defs_array['Prospects']);
        unset($field_defs_array['Leads']);

		foreach($field_defs_array as $k => $v)
			$field_defs_array[$k] = array_values(array_csort($v, 'value'));

        return $field_defs_array;
	}

	function getTrackerURLs()
	{
		$lq = new ListQuery('Campaign', null, array('link_name' => 'tracked_urls'));
		$lq->setParentKey($this->campaign_id);
		return $lq->fetchAllRows();
	}

	function loadUpdateRequest(RowUpdate $update, array $input) {
        $related_update = array();
        $related_update['attachments'] = $related_update['attachments'] = $this->loadUpdateAttachments($input);

		$update->setRelatedData($this->id.'_rows', $related_update);

		$this->campaign_id = array_get_default($input, 'campaign_id');

	}
	
	function validateInput(RowUpdate $update) {
		return true;
	}
	
	function beforeUpdate(RowUpdate $update) {
		// to shade EmailWidget::beforeUpdate()
	}

	function afterUpdate(RowUpdate $update) {
        global $mod_strings, $timedate;

		$row_updates = $update->getRelatedData($this->id.'_rows');
		if(! $row_updates)
			return;

        $this->updateAttachments($update, $row_updates);
    }
}
?>
