<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


 /*********************************************************************************
 * $Id: $
 ********************************************************************************/

require_once('include/MailClient/EmailClient.php');
require_once('include/MailClient/IAHEmailParser.php');
require_once('modules/EmailPOP3/EmailPOP3.php');
require_once('modules/EmailFolders/EmailFolder.php');
require_once('modules/Campaigns/ProcessBouncedEmails.php');
require_once 'modules/Emails/utils.php';


class IAHEmailImportError extends IAHInternalError {
	var $error_name = 'Email Import Error';
}


// called by scheduler
function run_email_import() {
	$import = new EmailImport();
	$import->run();
}

function process_campaign_bounces() {
	$import = new EmailImport();
	$import->process_campaign_bounces = true;
	$import->run();
}


class EmailImport
{
	var $db;
	var $accounts;
	var $current_account;
	var $current_account_idx;
	var $state;

	// This is used to pass data between states.
	// Reset in state_process_next_message
	var $message_state = array();

	var $boxhash;
	var $client;

	var $messages_count = 0;
	var $dl_start_time;
	var $break_on = '';
	var $break_state= '';
	var $max_duration;
	var $max_messages;

	var $email_counts = array();
	var $header_cache = array();
	var $seen_cache = array();

	var $archive = false;
	var $check_for_user = false;
	var $limit_scheduled = true;
	var $markPointReached;
	var $markPoint;
	var $nMessages;
	var $msg_number;
	var $messageId;
	var $message;


	var $process_campaign_bounces;
	var $logger;
	var $log;

	var $mailbox_skills;

	function __construct($archive = false, $for_user = null, $limit_scheduled = true)
	{
		global $db, $log;
		$this->db =& $db;
		$this->logger =& $log;
		$this->log =& $this;
		$this->archive = $archive;
		if($for_user === true)
			$for_user = AppConfig::current_user_id();
		$this->check_for_user = $for_user;
		$this->limit_scheduled = $limit_scheduled;
		$this->max_duration = AppConfig::setting('email.import.max_session_duration');
		$this->max_messages = AppConfig::setting('email.import.max_session_messages');
		if (!$archive) {
			$this->setup();
		}
	}

	function run()
	{
		global $current_user;
		$saved_user_id = AppConfig::current_user_id();
		$GLOBALS['NEW_EMAILS_COUNT'] = 0;
		$this->dl_start_time = time();
		try {
			while ($this->state) {
				$this->log->debug('State: ' . $this->state);
				if (!$this->break_on && ($this->messages_count > $this->max_messages || time() - $this->dl_start_time >= $this->max_duration)) {
					if ($this->messages_count > $this->max_messages) {
						$this->log->warn(
							'Hit maximum message download limit ('
							. $this->max_messages . ' messages), finishing.'
						);
					} else {
						$this->log->warn(
							'Hit message download time limit ('
							. $this->max_duration . ' seconds), finishing.'
						);
					}
					if ($this->break_state) {
						$this->state = $this->break_state;
					}
					$this->break_on = 'get_account';
				}

				if ($this->break_on == $this->state) {
					break;
				}

				$method = 'state_' . $this->state;
				if (method_exists($this, $method)) {
					$this->$method();
				} else {
					$this->log->fatal('Unknown state : ' . $this->state);
				}
				if ($this->check_for_user) {
					$this->update_mail_counts();
				}
				$this->logProgress();
			}
		} catch (Exception $e) {
			if ($saved_user_id) {
				$current_user->retrieve($saved_user_id);
			}
			throw new Exception($e->getMessage(), $e->getCode(), $e);
		}
		if ($saved_user_id) {
			$current_user->retrieve($saved_user_id);
		}
	}

	function connection_test($mailbox_id) {
		if(! $mailbox_id || ! isset($this->accounts[$mailbox_id]))
			throw new IAHEmailImportError('Mailbox not found');
		$this->current_account = $this->accounts[$mailbox_id];
		return $this->connection_test_perform();
	}

	function connection_test2($account_data) {
		$this->current_account = $account_data;
		return $this->connection_test_perform();
	}

	protected function connection_test_perform() {
		$result = [
			'connect' => true,
			'login' => true,
			'open' => true,
			'folders' => false,
		];
		$status = $this->do_connect();
		if(! $status) {
			$result['connect'] = $this->client->lastError;
			@$this->client->disconnect();
			return $result;
		} else {
			$status = $this->do_login();
			if(!$status) {
				@$this->client->disconnect();
				$result['login'] = $this->client->lastError;
				return $result;
			} else {
				$status = $this->do_select_folder($folder_name);
				if(! $status) {
					$result['open'] = "Error selecting folder '$folder_name'";
				}
				try {
					$result['folders'] = $this->client->getFolders();
				} catch (Exception $e) {
				}
				@$this->client->disconnect();
				return $result;
			}
		}
	}

	function setup()
	{
		$query = "SELECT emails_pop3.*, IFNULL(users.user_name,'') iah_user_name, users.email1, users.email2 FROM emails_pop3 LEFT JOIN users ON emails_pop3.user_id=users.id WHERE (!users.deleted OR emails_pop3.user_id='-1') AND !emails_pop3.deleted AND emails_pop3.active";
		if ($this->check_for_user) {
			$query .= " AND emails_pop3.user_id = '" . $this->check_for_user . "'";
		} else if($this->limit_scheduled) {
			$query .= ' AND emails_pop3.scheduler';
		}
		$query .= ' ORDER BY IFNULL(emails_pop3.last_check, 0)';
		$result = $this->db->query($query, true, "Error getting POP accounts list: ");
		$this->accounts = array();
		while ($account = $this->db->fetchByAssoc($result, -1, false)) {
			$addrs = array($account['email'], $account['email1'], $account['email2']);
			$addrs = array_map('strtolower', array_map('trim', $addrs));
			$account['all_addresses'] = array_filter($addrs);
			$this->accounts[$account['id']] = $account;
		}
		reset($this->accounts);
		$this->current_account_idx = 0;
		$this->state = 'get_account';
		$this->logProgress();
	}

	function state_get_account()
	{
		global $current_user;
		$this->break_state = '';
		$this->current_account = current($this->accounts);
		while ($this->current_account && (($this->current_account['mailbox_type'] == 'bounce') ^ !empty($this->process_campaign_bounces))) {
			$this->current_account = next($this->accounts);
			$this->current_account_idx ++;
		}
		next($this->accounts);
		if ($this->current_account) {
			$this->current_account_idx ++;
			$this->log->debug('Next account: ' . $this->current_account['host'] . ':' . $this->current_account['username']);
			$this->markPointReached = $this->current_account['last_id'] == '!';
			$this->markPoint = $this->current_account['last_id'];
			if ($this->markPointReached) {
				$this->debug('No markpoint from last run');
			} else {
				$this->debug('Markpoint from last run : ' . $this->markPoint);
			}
	        $current_user = new User;
    	    if ($this->current_account['user_id'] != -1) {
        	    $current_user->retrieve($this->current_account['user_id']);
	        } else {
    	        $current_user->id = -1;
	        }
    	    $this->boxhash = md5($this->current_account['host'] . "\n" . $this->current_account['username']);
			$this->state = 'connect';

		} else {
			$this->log->debug('No more accounts to process');
			$this->state = '';
		}
	}

	function do_connect() {
		$params = array(
			'mail_account_id' => $this->current_account['id'] ?? null,
			'host' => $this->current_account['host'],
			'username' => $this->current_account['username'],
			'password' => $this->current_account['password'],
			'imap_auth_type' => $this->current_account['imap_auth_type'],
			'o365_auth_token' => @json_decode($this->current_account['o365_auth_token'], true),
			'port' => $this->current_account['port'],
			'login_method' => 'USER',
			'last_check' => array_get_default($this->current_account, 'last_check'),
			'use_ssl' => $this->current_account['use_ssl'],
			'disable_gssapi' => $this->current_account['disable_gssapi'],
			'folder' => $this->current_account['imap_folder'],
			'mark_read' => $this->current_account['mark_read'],
		);
		$this->client =& EmailClient::create($this->current_account['protocol'], $params);
		$this->log->debug("Email client class: ".get_class($this->client));
        return $this->client->connect();
	}

	function state_connect()
	{
		$this->break_state = 'save_markpoint';
		$status = $this->do_connect();
        if (!$status) {
			$this->log->error(
				'Error connecting to mail server '
					. $this->current_account['host']
					. ':' . $this->current_account['port']
					. ' (' . $this->client->lastError
					. ') for user '
					. $this->current_account['iah_user_name']
					. ', account ' . $this->current_account['id']
			);
            $this->state = 'get_account';
		} else {
	    	$this->log->debug('Connected to ' . $this->current_account['host']);
            $this->state = 'login';
		}
		$dt = gmdate('Y-m-d H:i:s');
		$query = "UPDATE emails_pop3 SET last_check='$dt' WHERE id='{$this->current_account['id']}'";
		$this->db->query($query, false);
	}

	function do_login()
	{
		$status = $this->client->login();
		return $status;
	}

	function state_login()
	{
        $status = $this->do_login();
        if (!$status) {
			$this->log->error(
				'Error logging into '
				. $this->current_account['host']
				. ' as ' . $this->current_account['username']
				. ' for user ' . $this->current_account['iah_user_name']
			);
            $this->state = 'disconnect';
        } else {
			$this->log->debug(
				'Logged into ' . $this->current_account['host']
				. ' as ' . $this->current_account['username']
			);
            $this->state = 'select_folder';
		}
	}

	function do_select_folder(&$folder_name) {
		$folder_name = $this->current_account['imap_folder'];
        return $this->client->selectBox($this->current_account['imap_folder']);
	}

	function state_select_folder()
	{
		$status = $this->do_select_folder($folder_name);
        if (!$status) {
			$this->log->error('Error selecting folder ' . $folder_name);
			$this->state = 'disconnect';
		} else {
			$this->log->debug('Selected folder ' . $folder_name);
			$this->state = 'num_messages';
		}
	}

	function state_num_messages()
	{
		$since = null;
		if(! empty($this->current_account['restrict_since'])) {
			$since_str = EmailPOP3::restrict_since_time($this->current_account['restrict_since']);
			if($since_str)
				$since = date('j-M-Y', strtotime($since_str));
		}
        $this->nMessages = $this->client->numMsg($since);
        if ($this->nMessages === false) {
			$this->log->warn(
				'Error getting number of messages from '
				. $this->current_account['host'] . ' as '
				. $this->current_account['username'] . ' for user ' . $this->current_account['iah_user_name']
				. ': ' . $this->client->lastError
			);
            $this->state = 'disconnect';
        } else {
			$this->log->info(
				$this->nMessages . ' messages on ' .$this->current_account['host']
				. ' for ' . $this->current_account['username']
			);
			$this->msg_number = $this->nMessages + 1;
			$this->state = 'process_next_message';
		}
	}

	function state_process_next_message()
	{
		$this->message_state = array();
		$this->msg_number --;
		$this->markPoint = '!';
		if ($this->msg_number < 1) {
			if(! $this->markPointReached && $this->nMessages > 0) {
				$this->log->debug('Previous markpoint not found, downloading all messages');
				$this->markPointReached = true;
				$this->msg_number = $this->nMessages + 1;
			} else {
				$this->log->debug('No more messages to process');
				$this->messageId = '!';
				$this->state = 'save_markpoint';
			}
		} else {
			$this->log->debug('Starting processing message # ' . $this->msg_number);
			$this->state = 'get_msg_id';
		}
	}

	function state_disconnect()
	{
		$this->nMessages = 0;
		$this->client->disconnect();
		$this->log->debug('Disconnected');
		$this->state = 'get_account';
	}

	function state_get_msg_id()
	{
		$headers = $this->getMessageHeaders($this->msg_number);
		$messageId = '';
		$m = array();
		if (preg_match('/^message-id:(.*)$/mi', $headers, $m)) {
			$messageId = trim($m[1]);
		}
		if (empty($messageId)) {
			$messageId = 'IAH-' . md5($headers);
		}
		$this->messageId  = $messageId;
		$this->debug('Message ID : ' . $this->messageId);

		$this->state = 'check_markpoint';
	}

	function state_check_markpoint()
	{
		if (!$this->markPointReached && $this->current_account['last_id'] == $this->messageId) {
			$this->markPointReached = true;
			$this->log->debug('Reached markpoint');
		} elseif (!$this->markPointReached) {
			$this->log->debug('Markpoint not reached');
		}
		$this->state = 'check_seen_status';
	}

	function state_check_seen_status()
	{
		if ($this->seenMessageID()) {
			if (!$this->markPointReached) {
				$this->log->debug('Message already seen - skipping');
				$this->state = 'process_next_message';
			} else {
				$this->log->debug('Message already seen - account check finished');
				$this->messageId = '!';
				$this->state = 'save_markpoint';
			}
		} else {
			$this->log->debug('Message not seen before');
			$this->state = 'fetch_message';
			$this->messages_count++;
		}
	}

	function state_fetch_message()
	{
		$this->message = $this->client->getMsg($this->msg_number);
		if ($this->message === false) {
			$this->log->warn(
				'Error getting message ' . $this->msg_number
				. ' from ' . $this->current_account['host']
				. ' as ' . $this->current_account['username']
				. ($this->client->lastError !== false) ? ": {$this->client->lastError}" : ''
			);
			$this->state = 'save_markpoint';
		} else {
			$this->log->debug(
				'Recieved message ' . $this->msg_number
				. ' from ' . $this->current_account['host']
				. ' as ' . $this->current_account['username']
			);
			$this->state = 'import_message';
		}
	}

	function state_import_message()
	{
		if (!$this->import_message()) {
			$this->log->warn('Error importing message');
			$this->message_state['import_failed'] = true;
		} else {
			$this->log->debug('Message imported');
		}
		$this->state = 'store_id';
	}

	function state_store_id()
	{
		$query = sprintf(
        	'INSERT INTO emails_uidl SET boxhash=\'%s\', uidl=\'%s\', message_id=\'%s\'',
			PearDatabase::quote($this->boxhash), uniqid(''), PearDatabase::quote($this->messageId)
		);
		$this->db->query($query);
		$this->seen_cache[$this->boxhash . $this->messageId] = true;
		$this->state = 'delete_message';
	}

	function state_delete_message()
	{
		if (!$this->current_account['leave_on_server'] && empty($this->message_state['import_failed'])) {
			if (!$this->client->deleteMsg($this->msg_number)) {
				$this->log->warn(
					'Error marking message ' . $this->msg_number
					. ' as deleted at ' . $this->current_account['host']
					. ' as ' . $this->current_account['username']
				);
				$this->state = 'save_markpoint';
			} else {
				$this->log->debug('Marked message for deletion');
				$this->state = 'process_next_message';
			}
		} else {
			$this->log->debug('Message left on server');
			$this->state = 'process_next_message';
		}
	}

	function state_save_markpoint()
	{
		$this->db->query("UPDATE emails_pop3 SET last_id='".PearDatabase::quote($this->messageId)."' WHERE id='{$this->current_account['id']}'");
		$this->state = 'disconnect';
	}




	function getMessageHeaders($msgNum)
	{
		$key = $this->boxhash . $msgNum;
		if(! isset($this->header_cache[$key]))
			$this->header_cache[$key] = $this->client->getHeaders($msgNum);
		return $this->header_cache[$key];
	}

	function seenMessageID()
	{
		$key = $this->boxhash . $this->messageId;
		if (empty($this->messageId)) return false;
		if(! isset($this->seen_cache[$key])) {
			$mid = PearDatabase::quote($this->messageId);
			$query = "SELECT COUNT(*) AS n FROM emails_uidl WHERE message_id='" . $mid . "' AND boxhash='$this->boxhash'";
			$res = $this->db->query($query);
			$row = $this->db->fetchByAssoc($res);
			$this->seen_cache[$key] = !empty($row['n']);
		}
		return $this->seen_cache[$key];
	}

	function import_message($message = null, $user = null) /* {{{ */
	{
		$relate_accounts = !AppConfig::setting('email.no_account_relations');
		$case = null;
		$emailsAreSent = !empty($this->current_account['imap_is_sent']) && $this->current_account['protocol'] == 'IMAP';
		$parse = new IAHEmailParser();
		try {
			$parse->loadSource($this->archive ? $message : $this->message);
		}
		catch(IAHEmailParseError $e) {
			$this->log->warn('Email parse error: '.$e->getMessage());
			return false;
		}

		if ($this->archive) {
        	$user_id = $user->id;
	        $mailbox_id = null;
	        $folder_id = EmailFolder::get_std_folder_id($user_id, STD_FOLDER_INBOX);
	        $status = null; // ?
	        $intent = null;
		} else {
			$from = strtolower(array_get_default($parse->from , 0, ''));
			$user_id = $this->current_account['user_id'];
			$mailbox_id = $this->current_account['id'];
			$folder_id = null;
			if ($this->current_account['detect_bcc']) {
				if ($from && $from == strtolower($this->current_account['email'])) {
					$folder_id = EmailFolder::get_std_folder_id($this->current_account['user_id'], STD_FOLDER_SENT);
					if ($folder_id)
						$emailsAreSent = true;
				}
			}
			if (!$folder_id)
				$folder_id = $this->current_account['email_folder_id'];
			if(! $folder_id && $this->current_account['mailbox_type'] == 'bounce')
				$folder_id = EmailFolder::get_std_folder_id($this->current_account['user_id'], STD_FOLDER_CAMPAIGN);
			if(! $folder_id)
				$folder_id = EmailFolder::get_std_folder_id($this->current_account['user_id'], STD_FOLDER_INBOX);
			$status = $emailsAreSent ? 'sent' : 'received';
			$intent = $this->current_account['mailbox_type'];
			if ($this->current_account['mailbox_type'] == 'bounce') {
				if(campaign_process_bounced_emails($parse)) {
					// email not saved
					$this->log->debug("Processed bounce email ({$parse->from[2]})");
					return true;
				}
			}
		}

		$is_auto_reply = match_autoreply_email($parse);

		$email = $parse->getEmailRecord($user_id, $mailbox_id, $folder_id);
		if($status) $email->set('status', $status);
		$email->set('intent', $intent ? $intent : 'pick');

		$related = $parse->findRelatedRecords($emailsAreSent);
		$add_relate = array();
		$acc = $contact = null;
		$from_this_user = false;
		foreach($related as $addr => $rels) {
			if(in_array($addr, $this->current_account['all_addresses'])) {
				continue; // never add things related to the user's address
			}
			else {
				if(isset($rels['Accounts'])) {
					$add_relate = array_merge($add_relate, $rels['Accounts']);
					if (!isset($acc))
						$acc = $rels['Accounts'][0];
				}

				if(isset($rels['Contacts'])) {
					$add_relate = array_merge($add_relate, $rels['Contacts']);
					if(! isset($contact)) {
						$contact = $rels['Contacts'][0];
						$email->set('contact_id', $contact['id']);
					}
				}
				if(isset($rels['Leads']))
					$add_relate = array_merge($add_relate, $rels['Leads']);
			}
			if(isset($rels['Users'])) {
				foreach($rels['Users'] as $k => $v) {
					$user = ListQuery::quick_fetch_row('User', $v['id']);
					if($user['portal_only'])
						unset($rels['Users'][$k]);
				}
				$add_relate = array_merge($add_relate, $rels['Users']);
			}

		}

		if(! $parse->fromNoRelateDomain()) {
			$thread_id = $parse->findEmailThread();
			if(! $thread_id) $thread_id = create_guid();
			$email->set('thread_id', $thread_id);
		}

		$case_matched = $case_found = null;
		$bug_matched = null;

		if ($email->getField('in_reply_to')) {
			$prev_email = ListQuery::quick_fetch_key('Email', 'message_id', $email->getField('in_reply_to'));
			if ($prev_email && !$prev_email->failed) {
				$prev_ptype = $prev_email->getField('parent_type');
				$prev_pid = $prev_email->getField('parent_id');
				if($prev_ptype && $prev_pid) {
					$email->set(array(
						'parent_id' => $prev_pid,
						'parent_type' => $prev_ptype,
					));
					if($prev_ptype == 'Cases')
						$case_matched = true;
					else if($prev_ptype == 'Bugs')
						$bug_matched = true;
				}
			}
		}

		if (!$this->archive && $contact && !$case_matched && !empty($this->current_account['one_case'])) {
			$clq = new ListQuery('aCase');
			$clq->addSimpleFilter('cust_contact_id', $contact['id']);
			$clq->addSimpleFilter('status', 'Closed', 'not_like', null, 'prefix');
			$maybe_case  = $clq->runQuerySingle();
			$case_matched = $maybe_case && !$maybe_case->failed;
			if ($case_matched) {
				$email->set(array(
					'parent_id' => $maybe_case->getField('id'),
					'parent_type' => 'Cases',
				));
			}
		}

		if (!$this->archive) {
			if(! $case_matched) {
				$test_match = match_case_email_subject($parse->getSubject());
				if ($test_match[0]) {
					$case_matched = true;
					$case_number = $test_match[1];

					$lq = new ListQuery('aCase');
					$lq->addSimpleFilter('case_number', $case_number);
					$case_result = $lq->runQuerySingle();
					if ($case_result && ! $case_result->failed) {
						$add_relate[] = array(
							'type' => 'Cases',
							'id' => $case_result->getField('id'),
							'rel_name' => 'cases',
						);
						$case_found = true;
					}
				}
			} else {
				$matched_case = ListQuery::quick_fetch('aCase', $email->getField('parent_id'));
				if ($matched_case) {
					if ($matched_case->getField('status') == 'Pending') {
						$mcu = RowUpdate::for_result($matched_case);
						$mcu->set('status', 'Active - Assigned');
						$mcu->save();
					}
				}
			}

        	if(! isset($queue_user_loaded)) {
				$queue_user_id = AppConfig::setting('company.case_queue_user');
				if(! $queue_user_id) {
					// select first admin user
					$query = "SELECT id FROM users WHERE is_admin AND status='Active' LIMIT 1";
					$result = $this->db->query($query, true);
					$row = $this->db->fetchByAssoc($result);
					$queue_user_id = $row['id'];
				}
				$queue_user_loaded = true;
			}
			$create_for_user =
				($this->current_account['user_id'] != '-1') ? $this->current_account['user_id'] :
				$queue_user_id;

			// hooks for custom email parsers - contributed by visual4
			$hooks = AppConfig::setting('model.hooks.Email.before_import', array());
			try {
				$hook_params = compact('case_matched', 'bug_matched', 'create_for_user', 'is_auto_reply');
				foreach($hooks as $hook)
					call_hook_function($hook, 'Email', array(&$email, &$parse, $hook_params));
			} catch(IAHActionAbort $e) {
				$this->log->warn('Email import aborted: '.$e->getMessage());
				return false;
			}

			if($is_auto_reply) {
				// skip case/bug creation
			}
			else if($this->current_account['mailbox_type'] == 'support' && ! $case_matched) {

				$case = RowUpdate::blank_for_model('aCase', $create_for_user);
				$from = strtolower(array_get_default($parse->from , 0, ''));
				if ($from)
					$case->setRelatedData('email_received_from', $from);
				if ($from && !$contact && $this->current_account['auto_create_contact']) {
					$name = trim(array_get_default($parse->from, 1));
					if (strlen($name)) {
						$parts = preg_split('~\s+~', $name, 2);
						if (is_countable($parts) && count($parts) > 1) {
							list($fname, $lname) = $parts;
						} else {
							$fname = '';
							$lname = $parts[1];
						}
					} else {
						$fname = '';
						$lname = $from;
					}
					$cont = RowUpdate::blank_for_model('Contact');
					$user_id = $this->current_account['user_id'] == '-1' ? AppConfig::current_user_id() : $this->current_account['user_id'];
					$cont->set('assigned_user_id', $user_id);
					$cont->set('first_name', $fname);
					$cont->set('last_name', $lname);
					$cont->set('email1', $from);
					$cont->save();
					$contact = array('id' => $cont->getPrimaryKeyValue(), 'phone_work' => '');
				}
				if($contact) {
					$email->set('contact_id', $contact['id']);
					$case->set('cust_contact_id', $contact['id']);
					$case->set('cust_phone_no', $contact['phone_work']);
				}
				if($acc)
					$case->set('account_id', $acc['id']);
				$case->set(array(
					'name' => $parse->getSubject(),
					'description' => $parse->getTextBody(),
					'assigned_user_id' => $create_for_user,
					'created_by' => $queue_user_id,
				));
				if($case->validate() && $case->save()) {
					$add_relate[] = array(
						'type' => 'Cases',
						'id' => $case->getPrimaryKeyValue(),
						'rel_name' => 'cases',
					);


					$skill_params = array();
					$skills = $this->getCurrentMailboxSkills();
					if ($case->getField('account_id')) {
						$a_skills = $this->getSkills('Account', $case->getField('account_id'));
						$skills = $this->combineSkills($skills, $a_skills);
					}
					foreach ($skills as $skill_id => $skill_rating) {
						$skill_params[$skill_id] = array('name' => 'rating', 'value' => $skill_rating);
					}
					$case->addUpdateLinks('skills', array_keys($skill_params), $skill_params);
				} else {
					$case = null;
				}
			} else if($this->current_account['mailbox_type'] == 'bug' && ! $bug_matched) {
				$bug = RowUpdate::blank_for_model('Bug', $create_for_user);
				if($contact)
					$bug->set('contact_id', $contact['id']);
				if($acc)
					$bug->set('account_id', $acc['id']);
				$bug->set(array(
					'name' => $parse->getSubject(),
					'description' => $parse->getTextBody(),
					'assigned_user_id' => $create_for_user,
					'created_by' => $queue_user_id,
				));
				if($bug->validate() && $bug->save()) {
					$add_relate[] = array(
						'type' => 'Bugs',
						'id' => $bug->getPrimaryKeyValue(),
						'rel_name' => 'bugs',
					);
				}
			}
		}

		if($acc)
			$email->set('account_id', $acc['id']);
		if (! $email->getField('parent_id') && is_countable($add_relate) && count($add_relate)) {
			$i = 0;
			usort($add_relate, function($a1, $a2) {
				$t = $a1['type'];
				$w1 = $t == 'Leads' ? 3 : ($t == 'Accounts' ? 2 : ($t == 'Contacts' ? 1 : 4));
				$t = $a2['type'];
				$w2 = $t == 'Leads' ? 3 : ($t == 'Accounts' ? 2 : ($t == 'Contacts' ? 1 : 0));
				return $w1 - $w2;
			});
			foreach($add_relate as $rel) {
				$relType = $rel['type'];
				if($relType == 'Users')
					continue;
				if(($i == 0 && $relate_accounts) || ($relType == 'Cases' || $relType == 'Bugs')) {
					$email->set(array(
						'parent_type' => $relType,
						'parent_id' => $rel['id'],
					));
				}
				$i++;
			}
		}

		try {
			if(! $email->validate() || ! $email->save()) {
				$this->log->error('Error saving imported email: '.$email->getErrors(true));
				return false;
			}
		} catch (IAHQueryError $e) {
			$this->log->error('Error saving imported email: '.$e->getMessage());
			return false;
		}

		$this->assignTeams($email);
		if ($case) {
			$this->assignTeams($case);
		}
		$email_id = $email->getPrimaryKeyValue();
		$parse->saveAttachments($email_id, $user_id);

		if (! $this->archive && AppConfig::setting('email.keep_raw')) {
			$this->storeRawMessage($parse->message_id, $parse->source);
		}

		foreach ($add_relate as $rel) {
			$email->addUpdateLink($rel['rel_name'], $rel['id']);
		}

		if (! $this->archive) {
			$GLOBALS['NEW_EMAILS_COUNT']++;
			if (!isset($this->email_counts[$this->current_account['user_id']])) {
				$this->email_counts[$this->current_account['user_id']] = 0;
			}
			$this->email_counts[$this->current_account['user_id']]++;
		}


		///////////////////////////////////////////////////////////
		//
		//         Auto-reply
		//
		///////////////////////////////////////////////////////////
		if (!$this->archive && ! $is_auto_reply) {
			global $current_user;
			$out_of_office = $current_user->getPreference('out_of_office');
			if((!$out_of_office && $this->current_account['template_id']) || ($out_of_office && $this->current_account['ooo_template_id'])) {
				if($parse->reply_to[0]) {
					$contactAddr = $parse->reply_to[0];
					$contactName = $parse->reply_to[1];
				} else {
					$contactAddr = $parse->from[0];
					$contactName = $parse->from[1];
				}
				if($this->getAutoreplyStatus($contactAddr)
				&& $this->checkOutOfOffice($parse->getSubject())
				&& $this->checkFilterDomain(array($parse->from[0], $parse->reply_to[0]))) { // if we haven't sent this guy 10 replies in 24hours

					if(!empty($this->current_account['from_name'])) {
						$from_name = $this->current_account['from_name'];
					} else { // use system default
						$from_name = AppConfig::setting('notify.from_name', '');
					}
					if(!empty($this->current_account['email'])) {
						$from_addr = $this->current_account['email'];
					} else {
						$from_addr = AppConfig::setting('notify.from_address', '');
					}

					$to = array(
						array(
							'email' => $contactAddr,
							'display' => $contactName,
							'entry' => IAHEmailParser::join_email_name($contactAddr, $contactName),
						),
					);
					require_once('modules/EmailTemplates/EmailTemplate.php');
					$et = new EmailTemplate();
					$et->retrieve($out_of_office ? $this->current_account['ooo_template_id'] : $this->current_account['template_id']);
					if(empty($et->body))		{ $et->body = ''; }
					if(empty($et->body_html))	{ $et->body_html = ''; }

					$reply = new Email();
					$reply->type				= 'out';
					$reply->status				= 'sent';
					$reply->to_addrs			= to_html($to[0]['entry']);
					$reply->to_addrs_arr		= $to;
					$reply->cc_addrs_arr		= array();
					$reply->bcc_addrs_arr		= array();
					$reply->from_name			= $from_name;
					$reply->from_addr			= $from_addr;
					$reply->name				= from_html($et->subject);
					$reply->description			= from_html($et->body);
					$reply->description_html	= from_html($et->body_html);
					$reply->in_reply_to			= $parse->message_id;
					$reply->custom_headers['Auto-Submitted'] = 'auto-replied';
					$reply->custom_headers['Precedence'] = 'bulk';

					$reply->send();
					$reply->cleanup();
					$et->cleanup();
					$this->setAutoreplyStatus($contactAddr);
				} else {
					$this->log->debug('InboundEmail: auto-reply threshold reached for email ('.$contactAddr.') - not sending auto-reply');
				}
			}

			$hooks = AppConfig::setting('model.hooks.Email.after_import', array());
			$hook_params = compact('case_matched', 'bug_matched', 'create_for_user', 'add_relate');
			foreach($hooks as $hook)
				call_hook_function($hook, 'Email', array(&$email, &$parse, $hook_params));
		}

		return $email_id;
    }

	function assignTeams(RowUpdate $item) {
		$mailbox_id = $this->current_account['id'];
		$lq = new ListQuery('EmailPOP3', ['id'], ['link_name' => 'teams', 'parent_key' => $mailbox_id]);
		$teams = $lq->fetchAllRows();
		foreach ($teams as $t) {
			$item->addUpdateLink('securitygroups', $t['id']);
		}
	}

	function combineSkills($s1, $s2) {
		$skills = $s1;
		foreach ($s2 as $id => $s) {
			if (!isset($skills[$id]) || ($skills[$id] < $s))
				$skills[$id] = $s;
		}
		return $skills;
	}

	function getSkills($model_name, $id) {
		$linked = ListQuery::quick_fetch_linked($model_name, $id, 'skills', array('~join.rating'), null, true);
		$skills = array();
		foreach ($linked as $l) {
			$skills[$l['~join.skill_id']] = $l['~join.rating'];
		}
		return $skills;
	}

	function getCurrentMailboxSkills() {
		if (!isset($this->mailbox_skills[$this->current_account['id']])) {
			$this->mailbox_skills[$this->current_account['id']] = $this->getSkills('EmailPOP3', $this->current_account['id']);
		}
		return $this->mailbox_skills[$this->current_account['id']];
	}

	function setAutoreplyStatus($addr) {
		$this->db->query('INSERT INTO inbound_email_autoreply (id, deleted, date_entered, date_modified, autoreplied_to) VALUES (
							\''.create_guid().'\',
							0,
							\''.gmdate('Y-m-d H:i:s', strtotime('now')).'\',
							\''.gmdate('Y-m-d H:i:s', strtotime('now')).'\',
							\''.$addr.'\') ');
	}

	function getAutoreplyStatus($from)
	{
		$q_clean = 'UPDATE inbound_email_autoreply SET deleted = 1 WHERE date_entered < \''.gmdate('Y-m-d H:i:s', strtotime('now -24 hours')).'\'';
		$r_clean = $this->db->query($q_clean);

		$q = 'SELECT count(*) AS c FROM inbound_email_autoreply WHERE deleted = 0 AND autoreplied_to = \''.PearDatabase::quote($from).'\'';
		$r = $this->db->query($q);
		$a = $this->db->fetchByAssoc($r);
		if($a['c'] >= 10) {
			$this->log->debug('Autoreply cancelled - more than 10 replies sent in 24 hours.');
			return false;
		} else {
			return true;
		}
	}

	function checkOutOfOffice($subject) {
		$ooto = array("Out of the Office", "Out of Office");

		foreach($ooto as $str) {
			if(preg_match("~{$str}~i", $subject)) {
				$this->log->debug('Autoreply cancelled - found "Out of Office" type of subject.');
				return false;
			}
		}
		return true; // no matches to ooto strings
	}

	function checkFilterDomain(array $addrs)
	{
		$filterDomain = $this->current_account['filter_domain'];
		if(!isset($filterDomain) || empty($filterDomain)) {
			return true; // nothing set for this
		} else {
			$filterDomain = '@'.strtolower($filterDomain);
			foreach($addrs as $addr) {
				if($addr && strpos($addr, $filterDomain) !== false) {
					$this->log->debug('Autoreply cancelled - email address domain matches filter domain.');
					return false;
				}
			}
			return true; // no match
		}
	}

	function warn($msg)
	{
		if(function_exists('scheduler_task_message'))
			scheduler_task_message($msg, 'warn');
		$this->logger->warn('*EmailImport* : ' . $msg);
	}

	function info($msg)
	{
		$this->logger->info('*EmailImport* : ' . $msg);
	}

	function debug($msg)
	{
		$this->logger->debug('*EmailImport* : ' . $msg);
    }

	function error($msg)
	{
		if(function_exists('scheduler_task_message'))
			scheduler_task_message($msg, 'error');
		$this->logger->error('*EmailImport* : ' . $msg);
    }

	function fatal($msg)
	{
		if(function_exists('scheduler_task_message'))
			scheduler_task_message($msg, 'error');
		$this->logger->fatal('*EmailImport* : ' . $msg);
	}

	function logProgress() {
		static $init_states = ['connect', 'login', 'select_folder', 'num_messages'];
		if(! function_exists('scheduler_task_progress'))
			return;
		if(empty($this->accounts)) $progress = 0;
		else {
			$state = $this->state;
			$acct_idx = $this->current_account_idx-1;
			if($state == 'get_account') {
				$acct_pg = 0;
				$acct_idx ++;
			}
			else if( ($idx = array_search($state, $init_states)) !== false)
				$acct_pg = 25 * ($idx + 1) / (count($init_states) + 1);
			else if($state == 'disconnect')
				$acct_pg = 90;
			else if(! empty($this->nMessages) && isset($this->msg_number))
				$acct_pg = 25 + 75 * ($this->nMessages - $this->msg_number + 1) / $this->nMessages;
			else if(! $state)
				$acct_pg = 100;
			else
				$acct_pg = 50;
			$accountsCount = is_countable($this->accounts) ? count($this->accounts) : 1;
			$progress = (100 * $acct_idx + $acct_pg) / $accountsCount;
		}
		scheduler_task_progress($progress);
	}

	function update_mail_counts()
	{
		global $current_user;
		foreach ($this->email_counts as $id => $count) {
			$user = new User();
			if ($user->retrieve($id)) {
				$user->setPreference('new_email_count', $count, 0, 'email_import');
				$user->setPreference('new_email_timestamp', time(), 0, 'email_import');
				if(empty($current_user) || $current_user->id != $user->id)
					$user->savePreferencesToDB();
			}
			$user->cleanup();
		}
	}


	function storeRawMessage($messageId, $message)
	{
		$destination = get_raw_message_filename($messageId);
		$fp = @fopen($destination, 'wb');
		if ($fp) {
			@fwrite($fp, $message);
			@fclose($fp);
		}
	}
}

