<?php

require_once 'include/ListView/MobileListFormatter.php';
require_once 'include/database/RowResult.php';
require_once 'modules/Emails/EmailDistributor.php';

class EmailsListFormatter extends ListFormatter {

	var $delegate;

	static function create($model = null, $link_name = null, $list_id = null, $context = 'listview', $params = null)
	{
		if (AppConfig::is_mobile())
			return new MobileEmailsListFormatter($model, $link_name, $list_id, $context, $params);
		return new self($model, $link_name, $list_id, $context, $params);
	}
	
	function __construct($model = null, $link_name = null, $list_id = null, $context = 'listview', $params = null) {
		parent::__construct($model, $link_name, $list_id, $context, $params);
		$this->delegate = new EmailsListFormatterDelegate($params['listview_manager']);
	}

	function outputPostHeader()
	{
		$this->delegate->outputPostHeader();
	}
}

class MobileEmailsListFormatter extends MobileListFormatter {

	var $delegate;

	static function create($model = null, $link_name = null, $list_id = null, $context = 'listview', $params = null)
	{
		return new self($model, $link_name, $list_id, $context, $params);
	}
	
	function __construct($model = null, $link_name = null, $list_id = null, $context = 'listview', $params = null) {
		parent::__construct($model, $link_name, $list_id, $context, $params);
		$this->delegate = new EmailsListFormatterDelegate($params['listview_manager']);
	}
}


class EmailsListFormatterDelegate {
	private $listView;
	public function __construct($listView) {
		$this->listView = $listView;
	}
	
	function outputPostHeader()
	{
		global $app_strings;
		$fmt = $this->listView->formatter;
		$form = $fmt->getFilterForm();

		if($form->getFilterValue('section') != 'group')
			return '';
		$open = array_get_default($_REQUEST, 'mass_perform') == 'groupEmailAssign';

		$hidden_fields = array(
			'list_id' => $fmt->list_id,
            'module' => 'Emails',
			'action' => $fmt->action,
			'list_offset' => $this->listView->list_result->offset,
			'list_limit' => $this->listView->list_result->limit,
            'list_order_by' => $this->listView->list_result->order_by,
			'list_uids' => '',
			'mu_hash' => ListMassUpdate::getHash(),
			'mass_perform' => 'groupEmailAssign',
			'list_format' => 'html',
		);

		if (!empty($fmt->massupdate)) {
			$hidden_fields['list_uids'] = $fmt->massupdate->getUids();
		}

		if($form) {
			$json = getJSONObj();
			$hidden_fields['list_filter'] = $json->encode($form->getArray(true));
		}
		$user_ids = explode('^,^', array_get_default($_REQUEST, 'assign_to', ''));
		$users = self::users_list($user_ids);

		$fields = array(
			'assign_to' => array(
				'name' => 'assign_to',
				'type' => 'multienum',
				'editable' => true,
				'updatable' => true,
				'vname' => 'LBL_USER_SELECT',
				'options_icon' => 'icon',
				'options' => $users,
				'options_sorted' => false,
				'label_key' => '_display',
				'placeholder' => translate('LBL_FILTER_ADD_USER', 'Emails'),
				'add_remove' => true,
				'options_lookup' => [
					'bean_name' => 'User',
				],
			),
			'dist_method' => array(
				'name' => 'dist_method',
				'type' => 'enum',
				'editable' => true,
				'updatable' => true,
				'vname' => 'LBL_USING_RULES',
				'options' => 'dom_email_distribution',
				'options_add_blank' => false,
				'source' => array(
					'type' => 'db',
				),
			),
		);

		$layout = array(
			'type' => 'distribute',
			'editable' => true,
			'collapsible' => true,
			'collapsed' => ! $open,
			'form_hidden_fields' => $hidden_fields,
			'buttons_position' => 'bottom',
			'form_buttons' => array(
				'assign' => array(
					'list_id' => $fmt->list_id,
					'icon' => 'icon-accept',
					'vname' => 'LBL_BUTTON_DISTRIBUTE',
					'vname_module' => 'Emails',
					'perform' => "perform_email_distribution(this.form); return false;",
					'params' => array(
						'mass_perform' => 'groupEmailAssign',
					),
				),
			),
			'sections' => array(
				array(
					'id' => 'assignment',
					'label_position' => 'top',
					'type' => 'section',
					'columns' => 2,
					'widths' => array('12%', '35%'),
					'label_widths' => array('12%', '35%'),
					'elements' => array(
						array(
							'name' => 'assign_to',
							'width' => 30,
							'display_rows' => 5,
						),
						'dist_method',
					)
				),
			),
		);

		$method = array_get_default($_REQUEST, 'dist_method');
		if(! $method) $method = array_get_default($_SESSION, 'emails_distribution_method');
		if(! $method) $method = AppConfig::setting('email.default_dist_method');
		$_SESSION['emails_distribution_method'] = $method;
		$result = new RowResult;
		$result->row = array(
			'dist_method' => $method,
			'assign_to' => implode('^,^', array_keys($users)),
		);
		$result->fields = $fields;
		$model = new ModelDef('Email');
		$model->addDisplayFieldDefinitions($fields);
		$gen = FormGenerator::html_form($model, $layout, $fmt->form_name . '_distribute', $fmt->list_id.'-' . $fmt->form_name . '_distribute');
		$gen->setFormClass('listUpdate');
		$gen->renderForm($result, $model->module_dir);
		$body = $gen->getResult();
		$gen->layout->addScriptLiteral($this->getDistributionJS($fmt->list_id));
		$gen->exportIncludes();

		$error = '';

		if (isset($_SESSION['_EMAIL_DISTRIBUTION_ERROR'])) {
			$error = EmailDistributor::error_text($_SESSION['_EMAIL_DISTRIBUTION_ERROR']);
			unset($_SESSION['_EMAIL_DISTRIBUTION_ERROR']);
			$open = true;
		}

		$arrow = '<span class="rbullet rbullet-inline"></span>';
		$onclick = 'return sugarListView.toggleUpdate(\'formContent-' . $gen->form_obj->form_id . '_distribute\');';
		$openClosed = $open ? 'sectionOpen' : 'sectionClosed';
		$bodyStyle = $open ? '' : 'style="display:none"';
		echo '<div class="card-header panel-header">';
		echo "<div class=\"listUpdateHead listUpdateMid $openClosed\" id=\"formHeader-{$gen->form_obj->form_id}_distribute\" style=\"cursor:pointer\" onclick=\"$onclick\">";
		echo '<h4 class="panel-title">' . translate('LBL_DIST_TITLE', 'Emails') . '&nbsp' . $arrow . '&nbsp;<span class="error">' . $error  .  '</span></h4>';
		echo '</div>';
		echo "<div id=\"formContent-{$gen->form_obj->form_id}_distribute\" $bodyStyle>";
		echo $body;
		echo '</div>';
		echo '</div>';

	}

	public static function users_list($ids=null)
	{
		$ret = array();
		if($ids)
			$ids = array_filter(array_unique($ids));
		else
			return $ret;
		$users = ListQuery::quick_fetch_all('User', array('id','_display'),
			array('filters' => array(
				array('field'=> 'id', 'value' => $ids, 'operator' => 'in'),
				array('field'=> 'status', 'value' => 'Active', 'operator' => '='),
				array('field'=> 'portal_only', 'value' => '0'))));
		if ($users) {
			foreach ($users->rows as $row) {
				$ret[$row['id']] = array(
					'id' => $row['id'],
					'_display' => $row['_display'],
					'icon' => 'theme-icon module-Users',
				);
			}
			if($ids) {
				// use previous order
				$sort = array();
				foreach($ids as $id) {
					if(isset($ret[$id])) {
						$sort[$id] = $ret[$id];
						unset($ret[$id]);
					}
				}
				foreach($ret as $id => $u)
					$sort[$id] = $u;
				$ret = $sort;
			}
		}
		return $ret;
	}

	private function getDistributionJS($list_id)
	{
		return <<<JS
		function perform_email_distribution(form) {
			var list = sListView.getListView('{$list_id}');
			var uidList = sListView.getUids('{$list_id}');
			var uids = '';
			if(uidList) 
				uids = uidList.getString();
			form.list_uids.value = uids;
			SUGAR.util.maskDiv(list.outer);	
			sendAndRetrieve(form, list.outer);
		}
JS;
	}
}

