<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


use Spatie\Dropbox\Client;

require_once 'include/Dropbox/AuthProvider.php';

if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


$json_supported_actions['fetch_template_attachments'] = array();
$json_supported_actions['dropbox_download'] = array();
$json_supported_actions['dropbox_create_link'] = array();

function json_fetch_template_attachments() {
	$record = array_get_default($_REQUEST, 'record');

	require_once('modules/Emails/widgets/EmailWidget.php');
	$w = new EmailWidget();

	if($record) {
		$result = $w->fetchAttachments('EmailTemplates', $record);
		$note_list = $result->getRows();
	}
	else
		$note_list = array();

	$result = $w->renderAttachmentsEditList(null, null, $note_list, 'template');
	json_return_value($result);
}

function json_dropbox_download() {
	session_write_close();
	$result = array(
		'success' => false,
	);
	$fname = sys_get_temp_dir() . '/' . uniqid('');

	try {
		$provider = new DropboxAuthProvider(false);
	    $client = new Client($provider);
		$body = $client->download($_REQUEST['path'], $fname);
		$f = fopen($fname, "w");
		stream_copy_to_stream($body, $f);
		fclose($f);
	} catch (Exception $e) {
		$result['message'] = $e->getMessage();
		json_return_value($result);
		return;
	}

	$result['success'] = true;
	$result['filename'] = $fname;
	json_return_value($result);
}

function json_dropbox_create_link() {
	session_write_close();
	$result = array(
		'success' => false,
	);
	$get_existing = false;
	$link = null;
	$pathToFile = $_REQUEST['path'];
	try {
		$provider = new DropboxAuthProvider(false);
	    $client = new Client($provider);
		$link = $client->createSharedLinkWithSettings($pathToFile);
	} catch (Exception $e) {
		$msg = $e->getMessage();
		if (strpos($msg, "shared_link_already_exists") === 0) {
			$get_existing = true;
		}
		if (!$get_existing) {
			$result['message'] = $msg;;
			json_return_value($result);
			return;
		}
	}

	if ($get_existing) {
		$links = $client->listSharedLinks($pathToFile, true);
		foreach ($links as $item) {
			if ($item['link_permissions']['resolved_visibility']['.tag'] === 'public') {
				$link = $item;
				break;
			}
		}
	}

	if (!is_array($link) || !isset($link['url'])) {
		json_return_value($result);
		exit;
	}

	$result['success'] = true;
	$result['url'] = $link['url'];
	json_return_value($result);
}

