<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

require_once('modules/Workflow/WFProcessorBase.php');

class SendEmailProc extends WFProcessorBase {
	var $related_model = 'Email';

	static $primary_email_options = [
		[
			'modules' => ['Contacts'],
			'primary' => 'LBL_INCLUDE_CONTACT_EMAIL'
		],
		[
			'modules' => ['Leads'],
			'primary' => 'LBL_INCLUDE_LEAD_EMAIL',
		],
		[
			'modules' => ['Cases'],
			'primary' => 'LBL_INCLUDE_CONTACTS_EMAIL'
		],
		[
			'modules' => ['Notes'],
			'primary' => 'LBL_INCLUDE_CONTACT_EMAIL'
		],
		[
			'modules' => ['Accounts', 'Invoice', 'Quotes', 'SalesOrders', 'PurchaseOrders', 'Bills', 'Shipping', 'Receiving', 'SubContracts'],
			'primary' => 'LBL_INCLUDE_ACCOUNT_EMAIL',
			'secondary' => 'LBL_INCLUDE_CONTACT_EMAIL'
		],
		[
			'modules' => ['LicenseServer'],
			'primary' => 'LBL_INCLUDE_CONTACT_EMAIL',
		],
		[
			'modules' => ['Opportunities'],
			'primary' => 'LBL_INCLUDE_ACCOUNT_EMAIL',
			'secondary' => 'LBL_INCLUDE_CONTACT_EMAIL'
		],
	];


	function process(RowUpdate $rowUpdate) {
		global $timedate;
		require_once('include/utils/html_utils.php');
		require_once('modules/EmailTemplates/TemplateParser.php');
		require_once('modules/Emails/utils.php');

		$send_from_system = ! AppConfig::setting('notify.send_from_assigning_user');
		$system_from_addr = AppConfig::setting('notify.from_address', '');
		$system_from_name = AppConfig::setting('notify.from_name', '');

		$module = $rowUpdate->getModuleDir();
		$rowId = $rowUpdate->getPrimaryKeyValue();

		$ids = preg_split('/\s*;\s*/', $this->getParam('invitee_ids', ''));
		$types = preg_split('/\s*;\s*/', $this->getParam('invitee_types', ''));
		$addrs = $this->getParam('to_addrs');

		$initiator = new User;
		$initiator->retrieve($this->getUserId());

		if ($this->getParam('include_initiator')) {
			array_unshift($ids, $initiator->id);
			array_unshift($types, 'Users');
		}

		if (empty($initiator->id)) {
			$initiator->getSystemUser();
		}

		if ($this->getParam('include_primary_email')) {
			if ($module == 'Accounts' || $module == 'Contacts' || $module == 'Leads') {
				array_unshift($ids, $rowId);
				array_unshift($types, $module);
			} elseif ($module == 'SubContracts' || $module == 'Opportunities') {
				array_unshift($ids, $rowUpdate->getField('account_id'));
				array_unshift($types, 'Accounts');
			} elseif ($module == 'Invoice' || $module == 'Quotes' || $module == 'SalesOrders') {
				array_unshift($ids, $rowUpdate->getField('billing_account_id'));
				array_unshift($types, 'Accounts');
			} elseif ($module == 'PurchaseOrders' || $module == 'Bills' || $module == 'Receiving') {
				array_unshift($ids, $rowUpdate->getField('supplier_id'));
				array_unshift($types, 'Accounts');
			} elseif ($module == 'Shipping') {
				array_unshift($ids, $rowUpdate->getField('shipping_account_id'));
				array_unshift($types, 'Accounts');
			} elseif ($module == 'LicenseServer') {
				array_unshift($ids, $rowUpdate->getField('contact_id'));
				array_unshift($types, 'Contacts');
			} elseif ($module == 'Cases') {
				$cust_contact_id = $rowUpdate->getField('cust_contact_id');
				if (!empty($cust_contact_id)) {
					array_unshift($ids, $cust_contact_id);
					array_unshift($types, 'Contacts');
				}
				$lq = new ListQuery('aCase', null, array('link_name' => 'contacts'));
				$lq->addFields(array('id'));
				$lq->setParentKey($rowId);
				$result = $lq->runQuery();
				foreach ($lq->getResultIds($result) as $cid) {
					array_unshift($ids, $cid);
					array_unshift($types, 'Contacts');
				}
			} elseif($module == 'Notes') {
				array_unshift($ids, $rowUpdate->getField('contact_id'));
				array_unshift($types, 'Contacts');
			}
		}

		if ($this->getParam('include_secondary_email')) {
			if ($module == 'Accounts') {
				array_unshift($ids, $rowUpdate->getField('primary_contact_id'));
				array_unshift($types, 'Contacts');
			} elseif ($module == 'SubContracts') {
				array_unshift($ids, $rowUpdate->getField('customer_contact_id'));
				array_unshift($types, 'Contacts');
			} elseif ($module == 'Invoice' || $module == 'Quotes' || $module == 'SalesOrders') {
				array_unshift($ids, $rowUpdate->getField('billing_contact_id'));
				array_unshift($types, 'Contacts');
			} elseif ($module == 'PurchaseOrders' || $module == 'Bills' || $module == 'Receiving') {
				array_unshift($ids, $rowUpdate->getField('supplier_contact_id'));
				array_unshift($types, 'Contacts');
			} elseif ($module == 'Shipping') {
				array_unshift($ids, $rowUpdate->getField('shipping_contact_id'));
				array_unshift($types, 'Contacts');
			} elseif ($module == 'Opportunities') {
				$account = ListQuery::quick_fetch('Account', $rowUpdate->getField('account_id'));
				array_unshift($ids, $account->getField('primary_contact_id'));
				array_unshift($types, 'Contacts');
			}
		}

		$seen = array();
		$recips = array();
		foreach($ids as $i => $id) {
			$t = $types[$i];
			$key = $t . '_' . $id;
			if (empty($seen[$key]) && $t && $id) {
				$recips[] = array(
					'type' => $t,
					'record' => $id,
				);
			}
			$seen[$key] = true;
		}

		$from_name = $this->getParam('from_name');
		$from_addr = $this->getParam('from_addr');
		if (empty($from_addr)) {
			if($send_from_system || empty($initiator->email1)) {
				if(empty($from_name)) $from_name = $system_from_name;
				$from_addr = $system_from_addr;
			}
			else {
				if(empty($from_name)) $from_name = $initiator->name;
				$from_addr = $initiator->email1;
			}
		}
		$cc_mbox_id = $this->getParam('cc_mailbox_id');
		$mbox = null;
		if (!empty($cc_mbox_id)) {
			$mbox = ListQuery::quick_fetch_row('EmailPOP3', $cc_mbox_id, array('email'));
		}

		if($addrs) {
			$to = normalize_emails_array(@parse_addrs($addrs), null, true, true);
			foreach($to as $addr) {
				if(! empty($addr['contact_id'])) {
					$addr['type'] = 'Contacts';
					$addr['record'] = $addr['contact_id'];
				}
				$recips[] = $addr;
			}
		}

		$ret = [];
		foreach ($recips as $recip) {
			require_bean('Email');
			$email = new Email();
			$email->from_name = $from_name;
			$email->from_addr = $from_addr;
			$email->type = 'sent';
			$email->isread = 'read';
			$email->parent_type = $module;
			$email->parent_id = $rowId;
			$email->assigned_user_id = $initiator->id;
			$email->set_created_by = $email->update_modified_by = false;
			$email->created_by = $email->assigned_user_id;
			$email->modified_user_id = $email->assigned_user_id;
			$email->send_from_system = $send_from_system;
			$email->date_start = $timedate->to_display_date_time(gmdate('Y-m-d H:i:s'));

			$objects = array(
				$module => $rowId,
				'Records' => $rowId,
			);
			$params = array(
				'disable_cache' => true,
				'user' => $initiator,
				'aliases' => array(),
				'model_aliases' => array('Records' => array($module, 'record')),
			);

			if(isset($recip['type'])) {
				if(empty($recip['email'])) {
					$contact_model = AppConfig::module_primary_bean($recip['type']);
					$id = $recip['record'];
					if (empty($id) || !$contact_model) {
						continue;
					}
					$contact = ListQuery::quick_fetch_row($contact_model, $id, ['id', 'email1', '_display']);
					if (!$contact) continue;
					if (empty($contact['email1'])) continue;
					$recip['email'] = $contact['email1'];
				}
				$email->to_addrs_arr[] = array('email' => $recip['email'], 'display' => array_get_default($recip, 'display'));
				$objects[$recip['type']] = $recip['record'];
				if($recip['type'] != 'Contacts' && $module != 'Contacts') {
					$objects['Contacts'] = $recip['record'];
					$params['model_aliases']['Contacts'] = array($recip['type'], 'contact');
				}
				$objects['Recipients'] = $recip['record'];
				$params['model_aliases']['Recipients'] = array($recip['type'], 'recipient');
			} else if(! empty($recip['email'])) {
				$email->to_addrs_arr[] = $recip;
			} else
				continue;

			if($mbox && ! empty($mbox['email']))
				$email->cc_addrs_arr[] = array('email' => $mbox['email']);

			$subject = $this->getParam('name');
			$body = $this->getParam('description_html');

			$tpl_id = $this->getParam('template_id');
			if($tpl_id) {
				$tpl = ListQuery::quick_fetch('EmailTemplate', $tpl_id);

				if(! $tpl)
					throw new WFProcessingError("Email template not found");
				$subject = $tpl->getField('subject');
				$body = $tpl->getField('body_html');

				require_once('modules/Emails/widgets/EmailWidget.php');
				$w = new EmailWidget();

				$result = $w->fetchAttachments('EmailTemplates', $tpl_id);
				$note_list = $result->getRows();

				$email->attachment_rows = $note_list;
			}

			list ($email->name, $description_html) = TemplateParser::parse_generic(
				array($subject, $body),
				$objects,
				$params
			);
			if(! strlen(trim($description_html)))
				throw new WFProcessingError("Empty email body");
			$email->setDescriptionHTML($description_html);

			if ($email->send()) {
				self::debug_msg($email->name, 'Email sent');

				// Save the email
				$email->prohibit_workflow = true;
				$email->save();

				$emlResult = ListQuery::quick_fetch('Email', $email->id);
				$emlUpdate = RowUpdate::for_result($emlResult);
				$type = array_get_default($recip, 'type');
				if(! empty($recip['record'])) {
					if ($type == 'Accounts') {
						$emlUpdate->addUpdateLink('accounts', $recip['record']);
					}
					else if ($type == 'Leads') {
						$emlUpdate->addUpdateLink('leads', $recip['record']);
					}
					else if ($type == 'Contacts') {
						$emlUpdate->addUpdateLink('contacts', $recip['record']);
					}
				}
				$ret[] = $email->id;
			}
			else {
				self::debug_msg($email->send_error_msg, 'Error sending email');
				$GLOBALS['log']->error('Error sending email: '.$email->send_error_msg);
			}
		}

		return $ret;
	}

	function getConfigFields() {
		$fields = parent::getConfigFields();
		$model = new ModelDef('Email');

		$fields['to_addrs'] = $model->getFieldDefinition('to_addrs');
		$fields['to_addrs']['required'] = false;
		$fields['include_initiator'] = [
			'type' => 'bool',
			'vname' => 'LBL_INCLUDE_INITIATOR',
			'default' => 0,
			'suffix_label' => true,
			'hide_label' => true,
		];
		foreach(self::$primary_email_options as $opt) {
			if(in_array($this->module, $opt['modules'])) {
				foreach($opt as $k => $v) {
					if($k == 'primary' || $k == 'secondary') {
						$fields['include_' . $k . '_email'] = [
							'type' => 'bool',
							'vname' => $v,
							'default' => 0,
							'suffix_label' => true,
							'hide_label' => true,
						];
					}
				}
				break;
			}
		}
		$fields['cc_mailbox'] = array(
			'type' => 'ref',
			'vname' => 'LBL_CC_MAILBOX',
			'bean_name' => 'EmailPOP3',
			'id_name' => 'cc_mailbox_id',
		);
		$fields['cc_mailbox_id'] = array('type' => 'ref_id', 'hidden' => true);
		$fields['from_name'] = $model->getFieldDefinition('from_name');
		$fields['from_addr'] = $model->getFieldDefinition('from_addr');
		$fields['template'] = array(
			'type' => 'ref',
			'vname' => 'LBL_NOTIFICATION_TEMPLATE',
			'bean_name' => 'EmailTemplate',
			'id_name' => 'template_id',
		);
		$fields['template_id'] = array('type' => 'ref_id', 'hidden' => true);
		$fields['name'] = $model->getFieldDefinition('name');
		$fields['name']['required'] = false;
		$fields['description_html'] = array(
			'vname' => 'LBL_BODY',
			'vname_module' => 'Emails',
			'type' => 'html',
			'height' => 200,
		);

		return $fields;
	}

	function beforeRenderConfigForm(HtmlFormGenerator $gen, $context, $entry_id) {
		global $pageInstance;
		$pageInstance->add_js_language('Emails');
		$gen->getLayout()->addScriptInclude('modules/Emails/Email.js');
		$gen->getFormObject()->onInitForm('initWorkflowEmail({FORM});');
	}

}

