<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

require_once('include/layout/forms/FormField.php');
require_once( 'modules/Emails/widgets/EmailWidget.php');

class EmailHeadersWidget extends FormField {
	var $to_addrs, $cc_addrs, $bcc_addrs, $from_name, $from_addr;

	function init($params=null, $model=null) {
        parent::init($params, $model);
        $fields = array('to_addrs', 'cc_addrs', 'bcc_addrs', 'from_name', 'from_addr');
        foreach($fields as $f) {
        	$this->$f = $this->loadElement($f);
        	if($this->$f) $this->addElement($this->$f);
        }
	}
	
	function getRequiredFields() {
		$fs = parent::getRequiredFields();
		$fs[] = 'status';
		$fs[] = 'to_addrs_names';
		$fs[] = 'cc_addrs_names';
		$fs[] = 'bcc_addrs_names';
		return $fs;
	}
	
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if($gen->getLayout()->getType() == 'editview' || $gen->getLayout()->getDefaultEditable()) {
			return $this->renderHtmlEdit($gen, $row_result, $parents, $context);
		}
		return $this->renderHtmlView($gen, $row_result, $parents, $context);
	}

	function renderHtmlEdit(HtmlFormGenerator $gen, RowResult $row_result, array $parents, array $context) {
		global $current_user;
		$ret = '';
		$summary = ! empty($context['summary']);
		$frm = $gen->getFormObject();
		
		$all_bcc = EmailWidget::addAdditionalBcc($_REQUEST, $row_result->getField('bcc_addrs', '', false));
		$row_result->assign('bcc_addrs', $all_bcc);

		$show_buttons = [];
		foreach(array('to_addrs', 'cc_addrs', 'bcc_addrs') as $f) {
			if($this->$f) {
				if ($f != 'to_addrs' && !$row_result->getField($f)) {
					$display = 'none';
					$show_buttons[] = $f;
				} else {
					$display = 'block';
				}
				$lbl = $this->$f->getLabel($context);
				$this->$f->style = 'margin-bottom: 3px';
				$fh = $this->$f->renderHtml($gen, $row_result, $parents, $context);
				if($this->$f->getRequired())
					$lbl .= '<span class="requiredStar">*</span>';
				$ret .= "<div id=\"addr_input_$f\" class=\"align-middle\" style=\"display: $display\"><span class=\"form-label\">$lbl</span>$fh</div>";
			}
		}
		if (!empty($show_buttons)) {
			$ret .= '<div class="buttons-inline" style="margin-bottom: 4px;">';
			foreach ($show_buttons as $f) {
				$lbl = translate('LBL_SHOW_' . strtoupper($f));
				$ret .= '<div class="inline-elt"><div tabindex="0" onclick="showAddressField(this)" class="uii uii-add active-icon" data-field="' . $f . '"><label>'. $lbl . '</label></div></div>';
			}
			$ret .= '</div>';
		}

		if($this->from_name && $this->from_addr) {
			$smtp_opts = $this->getSMTPOptions();
			if (empty($smtp_opts)) {
				$spec = array(
					'name' => 'from_sel',
					'type' => 'enum',
					'options' => array(),
					'options_add_blank' => false,
					'input_class' => 'EmailFromSelect',
					'var_width' => true,
				);
				$pfx = $frm->getFieldIdPrefix();
				$this->from_name->width = 28;
				$this->from_name->placeholder = translate('LBL_FROM_NAME', 'Emails');
				$fh = $this->from_name->renderHtml($gen, $row_result, $parents, $context);
				$this->from_addr->required = true;
				$this->from_addr->placeholder = translate('LBL_FROM_EMAIL', 'Emails');
				$fh .= '&nbsp;' . $this->from_addr->renderHtml($gen, $row_result, $parents, $context);
				if($row_result->getField('status') == 'draft' && $row_result->new_record) {
					$opts = array();
					$from = $row_result->getField('from_name');
					$addr = $row_result->getField('from_addr');
					$label = join_name_email('', $from, $addr);
					if($addr)
						$opts[] = array('email' => $addr, 'from' => $from, 'name' => $label);
					if ($current_user->email1) {
						if ($current_user->email1 != $addr || $current_user->name != $from) {
							$opts[] = array('email' => $current_user->email1, 'from' => $current_user->name, 'name' => join_name_email('', $current_user->name, $current_user->email1));
						}
					}
					if ($current_user->email2) {
						if ($current_user->email2 != $addr || $current_user->name != $from) {
							$opts[] = array('email' => $current_user->email2, 'from' => $current_user->name, 'name' => join_name_email('', $current_user->name, $current_user->email2));
						}
					}
					if($opts) {
						$opts[] = array('custom' => 1, 'name' => translate('LBL_CUSTOM_OPTION', 'app'));
						$row_result->assign('from_sel', 0);
						$spec['options'] = $opts;
						$fh = $frm->renderField($row_result, $spec, 'from_sel')
							. "&nbsp;<div style=\"display: none; margin-top: 0.5em\" id=\"{$pfx}from_sel-custom\">$fh</div>";
					}
				}
				$lbl = translate('LBL_FROM', 'Emails');
				$ret .= "<div class=\"align-middle\"><span class=\"form-label no-flex\" style=\"min-width: 4em\">$lbl</span><span>$fh</span></div>";
			} else {
				$spec = array(
					'name' => 'smtp_account_id',
					'type' => 'enum',
					'options' => $smtp_opts,
					'options_add_blank' => false,
					'var_width' => true,
				);
				if($row_result->getField('status') == 'draft' && $row_result->new_record) {
					if (!$row_result->getField('smtp_account_id'))
						$row_result->assign('smtp_account_id', key($smtp_opts));
					$fh = $frm->renderField($row_result, $spec, 'smtp_account_id');
				}
				$lbl = translate('LBL_FROM', 'Emails');
				$ret .= "<div class=\"align-middle\"><span class=\"form-label no-flex\" style=\"min-width: 4em\">$lbl</span><span>$fh</span></div>";			
			}
		}
		//$ret = "<div style=\"padding: 3px 0\">$ret</div>";
		return $ret;
	}

	function getSMTPOptions() {
		if (!AppConfig::setting('company.smtp_accounts_enabled')) return [];
		$lq = new ListQuery('EmailPOP3');
		$lq->addSimpleFilter('user_id', AppConfig::current_user_id());
		$lq->addSimpleFilter('smtp_on', 1);
		$rows = $lq->fetchAllRows();
		$opts = [];
		foreach ($rows as $row) {
			$label = '<' . $row['email'] . '>';
			if ($row['from_name']) $label = $row['from_name'] . ' ' . $label;
			$opts[$row['id']] = $label;
		}
		return $opts;
	}
	
	function renderHtmlView(HtmlFormGenerator $gen, RowResult $row_result, array $parents, array $context) {
		$ret = '';
		$fs = array('to_addrs', 'cc_addrs', 'bcc_addrs');
		$sep = translate('LBL_SEPARATOR', 'app');
		foreach($fs as $f) {
			$lbl = $this->$f->getLabel($context);
			$addr = $row_result->getField($f);
			if($addr)
			$addrs[$f] = array('label' => $lbl, 'value' => $addr);
		}
		$from_name = $row_result->getField('from_name');
		$from_addr = $row_result->getField('from_addr');
		if(strlen($from_addr)) {
			$addr = "<$from_addr>";
			if(strlen($from_name)) {
				$nm = addcslashes($from_name, '"');
				$addr = "\"$nm\" $addr";
			}
			$lbl = translate('LBL_FROM', 'Emails');
			$addrs['from'] = array('label' => $lbl, 'value' => $addr);
		}
		foreach ($addrs as $f => $addr) {
			$who = array();
			$arr = @parse_addrs($addr['value']);
			$arr = merge_email_names($arr, $row_result->getField($f . '_names'));
			foreach($arr as $entry) {
				$val = format_email_link($entry['email'], $entry['display'], '', 'tabDetailViewDFLink', true);
				if($val) $who[] = $val;
			}
			if($who)
				$ret .= "<div class=\"tools-row\"><span class=\"form-label no-flex\" style=\"min-width: 5em\">{$addr['label']}{$sep}</span><span class=\"form-value\">".implode('; ', $who) . '</span></div>';
		}
		return $ret;
	}

}
