<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


use Sabre\VObject;
use OneCRM\OAuth\SMTPOAuth;

require_once('include/layout/forms/FormTableSection.php');
require_once('modules/Emails/Email.php');
require_once('modules/EmailTemplates/EmailTemplate.php');
require_once('modules/EmailTemplates/TemplateParser.php');
require_once('modules/EmailFolders/EmailFolder.php');
require_once('modules/Emails/Forms.php');
require_once('modules/Emails/utils.php');
require_once('include/utils/html_utils.php');
require_once 'include/OAuth/SMTPOAuth.php';

class EmailWidget extends FormTableSection {
	const MAX_ATTACHMENTS = 10;

	function init($params=null, $model=null) {
		parent::init($params, $model);
		if(! $this->id)
			$this->id = 'email_form';
	}

	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if($gen->getLayout()->getEditingLayout()) {
			$params = array();
			return $this->renderOuterTable($gen, $parents, $context, $params);
		}

		if($gen->getLayout()->getDefaultEditable()) {
			return $this->renderHtmlEdit($gen, $row_result, $context);
		}
		return $this->renderHtmlView($gen, $row_result);
	}

	function getRequiredFields() {
		$fields = ['type', 'mailbox', 'message_id', 'thread_id', 'date_start',
			'contact', 'replace_fields', 'in_reply_to', 'isread', 'status',
			'assigned_user_id', 'description', 'description_html', 'replied'];
		return $fields;
	}

	function renderHtmlView(HtmlFormGenerator $gen, RowResult $row_result) {
		global $mod_strings;
		$input = $_REQUEST;

		$attachments = '&nbsp;';
		$email_body = '&nbsp;';
		$record = $row_result->getField('id');

		if ($record) {
			$attachments = $this->renderAttachmentsView($row_result->getModuleDir(), $record, $mod_strings['LBL_ATTACHMENT_VIEW']);
			$email_body = $this->renderBody($input, $row_result);
		}

		$params = [
			'columns' => 1,
			'elements' => [
				[
					'type' => 'static',
					'label' => translate('LBL_BODY', 'Emails'),
					'value' => $email_body,
				],
				[
					'type' => 'static',
					'label' => translate('LBL_ATTACHMENTS', 'Emails'),
					'value' => $attachments,
				]
			]
		];
		return parent::renderOuterTable($gen, [], [], $params);
	}

	function renderHtmlEdit(HtmlFormGenerator $gen, RowResult $row_result, $context) {
		global $current_user, $mod_strings, $app_strings;
		$input = $_REQUEST;
		$sep = $app_strings['LBL_SEPARATOR'];

		$record = $row_result->getField('id');
		$reply = '';
		$forward = array_get_default($input, 'forward', '');
		$type = $row_result->getField('type');

		$descriptions = array(
			'plain' => $row_result->getField('description'),
			'html' => $row_result->getField('description_html'),
		);

		if (! $row_result->new_record) {
			$descriptions['html'] = show_inline_images(cleanHTML($descriptions['html'], true, true));
		} elseif (array_get_default($input,'description_html') === null) {
			$descriptions['html'] = $this->addSignature($descriptions['html'], $type);
		}

		$email_body = $this->renderHtmlEditor($gen, $descriptions);
		$attachments = $this->renderAttachmentsEdit($gen, $row_result, $record, $input);
		$upload = $this->renderAttachmentUpload($gen);

		$hidden = [
			'contact_id' => $row_result->getField('contact_id'),
			//'user_id' => $current_user->id,
			'send' => '',
			'type' => $type,
			'reply' => $reply,
			'forward' => $forward,
			'message_id' => $row_result->getField('message_id'),
			'thread_id' => $row_result->getField('thread_id'),
			'in_reply_to' => $row_result->getField('in_reply_to'),
		];
		/*$return_module_hidden = '';
		$return_action_hidden = '';
		if (empty($input['return_module']))
			$return_module_hidden = '<input type="hidden" name="return_module" value="Emails" />';
		if (empty($input['return_action']))
			$return_action_hidden = '<input type="hidden" name="return_action" value="DetailView" />';
*/
		$gen->getFormObject()->addHiddenFields($hidden);

		$params = [
			'columns' => 3,
			'elements' => [],
		];
		if (!Email::check_email_settings(false) && ($type == 'out' || $type == 'draft')) {
			$params['description'] = '<div class="panel-outer panel-danger panel-border panel-round">'
				. '<div class="panel-body panel-pad-body">' . $mod_strings['WARNING_SETTINGS_NOT_CONF']." <a href='index.php?module=Users&action=EditView&record=".$current_user->id."'>".$mod_strings['LBL_EDIT_MY_SETTINGS']
				. '</a></div></div>';
		}

		if ($type != 'archived') {
			$replace_fields = $gen->form_obj->renderField($row_result, 'replace_fields');
			$compose_format = $this->getCheckedComposeFormat();
			$send_as = <<<EOQ
				<div class="input-field input-check"><input type="radio" name="send_as" value="plain" {$compose_format['plain']}><label class="input-label"> {$mod_strings['LBL_SEND_AS_PLAIN']}</label></div>&nbsp;
				<div class="input-field input-check"><input type="radio" name="send_as" value="html" {$compose_format['html']}><label class="input-label"> {$mod_strings['LBL_SEND_AS_HTML']}</label></div>
EOQ;
			$params['elements'][] = [
				'type' => 'static',
				'label' => translate('LBL_SEND_AS', 'Emails'),
				'value' => $send_as,
			];
			$params['elements'][] = [
				'type' => 'static',
				'label' => translate('LBL_USE_TEMPLATE', 'Emails'),
				'value' => $this->renderTemplateSelect($gen, $row_result),
			];
			$params['elements'][] = [
				'type' => 'static',
				'label' => translate('LBL_REPLACE_TEMPLATE_FIELDS', 'Emails'),
				'value' => $replace_fields,
			];
		}
		$params['elements'][] = [
			'type' => 'static',
			'label' => translate('LBL_BODY', 'Emails'),
			'value' => $email_body,
			'colspan' => 3,
		];
		$params['elements'][] = [
			'type' => 'static',
			'label' => translate('LBL_ATTACHMENTS', 'Emails'),
			'value' => $attachments . $upload,
			'colspan' => 3,
		];

		$layout = $gen->getLayout();
		$layout->addJSLanguage('Emails');
		$layout->addScriptInclude('modules/Emails/Email.js');
		$this->addAttachmentsJs($layout);

		return parent::renderOuterTable($gen, [], [], $params);
	}


	/**
	 * Render email body for DetailView
	 *
	 * @param  array $input - user input ($_REQUEST)
	 * @param  RowResult $row_result
	 * @return string
	 */
	function renderBody($input, $row_result) {
		$format = $this->getShowFormat($input);
		$body = '';
		$links = '';
		$body_format = 'html';
		$has_remote = false;

		$description_html = $row_result->getField('description_html');
		$description = $row_result->getField('description');
		if ($description_html != '' && $format['show_html'] == true)  {
			$html = cleanHTML($description_html);
			if (has_remote_content($html)) $has_remote = true;

			if ($format['show_remote'] == true) {
				$html = show_inline_images($html);
				$html = remove_tracker_images($html);
			} else {
				$html = remove_images($html, true);
				$html = show_inline_images($html);
			}

			$body .= $html;
		} else {
			if ($description != '') {
				$desc = nl2br(url2html(to_html($description)));
				$desc = preg_replace('~\t~', '    ', $desc);
				$desc = preg_replace('~  ~', '&nbsp; ', $desc);
				if($description_html != '')
					$body_format = 'plain_from_html';
				else
					$body_format = 'plain';
			} else {
				$desc = nl2br(html2plaintext(to_html($description_html), false));
				$body_format = 'plain_from_html';
			}
			if ($desc != '')
				$body .= '<div style="font-family: monospace" id="email_body">' . $desc . '</div>';
		}

		if ($body != '')
			$links = $this->renderBodyLinks($row_result->getField('id'), $body_format, $format['show_remote'], $has_remote);
		else
			$body = '&nbsp;';

		return $links . $body;
	}

	/**
	 * Get email body show format
	 *
	 * @param  array $input - user input ($_REQUEST)
	 * @return array - ['show_html'] => true/false, ['show_images'] => true/false
	 */
	function getShowFormat($input) {
		global $current_user;

		$show_html = $current_user->getPreference('email_display_format') == 'html';

		if (! empty($input['display_format']) && $input['display_format'] == 'html') {
			$show_html = true;
		} elseif (! empty($input['display_format']) && $input['display_format'] == 'plain') {
			$show_html = false;
		}

		$show_images = $current_user->getPreference('show_images');
		$show_remote = $show_images;

		if (! empty($input['show_images'])) {
			$show_images = true;
		} elseif (! empty($input['hide_images'])) {
			$show_images = false;
		}
		if (! empty($input['show_remote'])) {
			$show_remote = true;
		}

		return compact('show_html', 'show_images', 'show_remote');
	}

	/**
	 * Render additional email body links for DetailView
	 *
	 * @param string $record
	 * @param string $format
	 * @param bool $show_images
	 * @param bool $has_remote - has remote content in body or not
	 * @return string
	 */
	function renderBodyLinks($record, $format = 'html', $show_remote = true, $has_remote = false) {
		global $mod_strings;

		$links = array();
		$lbl = array_get_default($mod_strings, 'LNK_OPEN_IN_NEW_WINDOW', 'Open in New Window');
		if (empty($_REQUEST['hide_theme_headers'])) {
	        $links[] = '<div class="input-icon icon-tlink"></div>&nbsp;<a href="index.php?module=Emails&action=DetailView&record='.$record.'&hide_theme_headers=1" class="tabDetailViewDFLink" target="_blank">'.$lbl.'</a>';
		}

		if ($format == 'html')  {

			if ($has_remote) {
				$url = preg_replace('~&?(show_remote)=[a-z0-9]*~i', '', getCurrentURL());
				if (! $show_remote) {
					$lbl = array_get_default($mod_strings, 'LNK_SHOW_REMOTE', 'Show Remote Content');
					$links[] = '<a href="'.$url.'&show_remote=true" class="tabDetailViewDFLink">'.$lbl.'</a>';
				}
			}

			$url = preg_replace('~&?display_format=[a-z]*~i', '', getCurrentURL());
			$lbl = array_get_default($mod_strings, 'LNK_PLAIN_TEXT', 'Plain Text');
			$links[] = '<a href="'.$url.'&display_format=plain" class="tabDetailViewDFLink">'.$lbl.'</a>';

		} else {
			$url = preg_replace('~&?display_format=[a-z]*~i', '', getCurrentURL());

			if ($format == 'plain_from_html') {
				$lbl = array_get_default($mod_strings, 'LNK_HTML_FORMAT', 'HTML Format');
				$links[] = '<a href="'.$url.'&display_format=html" class="tabDetailViewDFLink">'.$lbl.'</a>';
			}

			$lbl = array_get_default($mod_strings, 'LNK_WRAP_DISABLE', 'Disable Text Wrapping');
			$links[] = '<a href="#" onclick="toggle_email_wrap(); return false;" id="email_wrap_link" class="tabDetailViewDFLink">'.$lbl.'</a>';

			$GLOBALS['pageInstance']->add_js_literal(nowrap_email_js(), null, LOAD_PRIORITY_BODY);
		}

		$links = implode('&nbsp;&nbsp;&nbsp;', $links) . '<br><br>';

		return $links;
	}

	/**
	 * Render email attachments html for DetailView
	 *
	 * @param string $module
	 * @param string $record
	 * @param string $label
	 * @return string
	 */
	function renderAttachmentsView($module, $record, $label) {
		global $current_user, $timedate;
		$fields = array('name', 'filename', 'file_mime_type');
		$lq = new ListQuery('Note', $fields);
		$lq->addSimpleFilter('parent_type', $module, '=');
		$lq->addSimpleFilter('parent_id', $record, '=');
		$lq->addSimpleFilter('email_attachment', 1, '=');
		$notes_list = $lq->fetchAllRows('name');

		if($notes_list) {
			$attachments = '';
			foreach($notes_list as $id => $note) {
				$dl_link = "download.php?type=Notes&id={$id}";
				$dl_title = javascript_escape($note['name']);
				$attachments .= "<a href=\"{$dl_link}&inline=0\" target=\"_blank\" class=\"tabDetailViewDFLink\">". $note['name'] ."</a>";
				$array = explode('.', $note['name']);
				$ext = end($array);
				if($ext == 'ics' || $ext == 'calendar') {
					$attachments .= $this->renderEventsButtons($note, $module, $record);
				}

				if(preg_match('/\.(jpe?g|gif|png|bmp|tiff?|txt|pdf|svg)$/i', $note['filename'])) {
					$mime_type = explode('/', $note['file_mime_type']);

					if (isset($mime_type[0]) && $mime_type[0] == 'image') {
						$href = "javascript:SUGAR.ui.Lightbox.show('{$dl_link}', '{$dl_title}');";
						$target = "";
					} else {
						$href = "{$dl_link}&inline=1";
						$target = "target=\"_blank\"";
					}

					$attachments .= "&nbsp; <div class=\"input-icon icon-tlink\"></div>&nbsp;<a //href=\"{$href}\" {$target} class=\"tabDetailViewDFLink\">". $label ."</a>";
				}

				$attachments .= '<br />';
			}

		} else {
			$attachments = '&nbsp;';
		}

		return $attachments;
	}

	/**
	 * Render email attachments hmtl for EditView
	 *
	 * @param  string $record
	 * @param  array $input - user input ($_REQUEST)
	 * @return string
	 */
	function renderAttachmentsEdit($gen, $row_result, $record, $input) {
		$forward = array_get_default($input, 'forward');
		$reattach_note_id = array_get_default($input, 'reattach_note_id');
		$relabel_note = array_get_default($input, 'relabel_note');

		$action = array_get_default($input, 'action');
		$perform = array_get_default($input, 'record_perform');
		$duplicate = ($action === 'Duplicate' || $perform === 'duplicate');

		$attachments = '';
		$hidden = array('new_notes' => array(), 'new_documents' => array(), 'remove_notes' => '');

		require_once('include/layout/FieldFormatter.php');
		$fmt = new FieldFormatter('html', 'view');
		$fmt->module_dirs = array('Notes');

		$notes_list = array();
		if ( ! empty($record) || $forward || ! empty($reattach_note_id) ) {
			$fields = array('id', 'name', 'filename');
			$lq = new ListQuery('Note', $fields);

			if ($forward) {
				$lq->addSimpleFilter('parent_id', $forward, '=');
			} elseif (! empty($reattach_note_id)) {
				$hidden['reattach_note_id'] = $reattach_note_id;
				$hidden['relabel_note'] = $relabel_note;
				$lq->addFilterPrimaryKey($reattach_note_id);
			} else {
				$lq->addSimpleFilter('parent_id', $record, '=');
			}
			$notes_list = $lq->fetchAllRows('name');
		}
		$render = $this->renderAttachmentsEditList($gen, $row_result, $notes_list);
		$attachments .= $render['body'];
		if ($forward || $duplicate)
			array_extend($hidden['new_notes'], $render['note_ids']);
		array_extend($hidden['new_documents'], $render['document_ids']);

		$hidden_str = '';
		foreach($hidden as $k => $v) {
			if(is_array($v)) $v = implode(',', $v);
			$v = to_html($v);
			$hidden_str .= "<input type=\"hidden\" name=\"$k\" value=\"$v\" />";
		}

		return $hidden_str . $attachments;
	}

	// used by json method
	function renderAttachmentsEditList($gen, $row_result, $notes_list, $prefix='oldfile') {
		 require_once('include/layout/FieldFormatter.php');
		$fmt = new FieldFormatter('html', 'view');
		$fmt->module_dirs = array('Notes');

		$document_ids = array();
		$note_ids = array();
		$body = '';
		$idx = 0;
		$file_spec = array(
			'type' => 'file_ref',
		);
		if ($notes_list) {
			foreach($notes_list as $id => $note) {
				if (empty($note['filename']))
					continue;

				$note_ids[] = $id;
				$cancel = "&nbsp;<div class=\"uii active-icon uii-remove uii-lg\" onclick=\"EmailAttachments.removeAttachment(this, '$id'); return false;\"></div>";

				$file_spec['name'] = $prefix.$idx;
				$display = $fmt->formatRowValue($file_spec, $note, $note['filename']);
				if($display) {
					$body .= '<div id="'.$prefix.'-outer'.$idx.'" style="padding-bottom: 0.3em">' . $display . $cancel . '</div>';
				}
				$idx ++;
			}
		}
		for ($inc_idx = 0; $inc_idx < self::MAX_ATTACHMENTS && $idx < self::MAX_ATTACHMENTS; $inc_idx++) {
			$field_name =  'email_attachment' . $inc_idx;
			$inc_id = array_get_default($_POST, $field_name);
			if ($inc_id && strpos($inc_id, 'INCOMING~') === 0) {
				$input = "<input type=\"hidden\" name=\"$field_name\" value=\"$inc_id\">";
				$inc_id = substr($inc_id, 9);
				$inc_row = ListQuery::quick_fetch_row('incoming_files', $inc_id);
				if ($inc_row) {
					$cancel = "&nbsp;<div class=\"uii uii-md uii-remove active-icon\" onclick=\"EmailAttachments.removeAttachment(this, '$inc_id'); return false;\"></div>";

					$file_spec['name'] = $prefix.$idx;
					$display = $fmt->formatRowValue($file_spec, $inc_row, $inc_row['upload_filename']);
					if($display) {
						$body .= '<div id="'.$prefix.'-outer'.$idx.'" style="padding-bottom: 0.3em">' . $input . $display . $cancel . '</div>';
					}
					$idx++;
				}
			}
		}
		$doc_spec = array(
			'type' => 'ref',
			'bean_name' => 'Document',
		);
		$new_document_ids = array_filter(explode(',', array_get_default($_POST, 'new_documents', '')));
		foreach ($new_document_ids as $did) {
			$doc = ListQuery::quick_fetch_row('Document', $did);
			if ($doc) {
				$cancel = "&nbsp;<div class=\"uii uii-md uii-remove active-icon\" onclick=\"EmailAttachments.removeAttachment(this, '$did'); return false;\"></div>";
				$fname = $prefix.$idx;
				$doc_spec['name'] = $fname;
				if ($gen) {
					$row_result->assign($fname, $doc['document_name']);
					$row_result->assign($fname . '_id', $doc['id']);
					$display = $gen->getFormObject()->renderRef($row_result, $doc_spec);
				} else {
					$display = $fmt->formatDetailLink($doc_spec, $doc, $doc['document_name']);
				}
				if($display) {
					$body .= '<div id="'.$prefix.'-outer'.$idx.'" style="padding-bottom: 0.3em">' . $input . $display . $cancel . '</div>';
				}
				$document_ids[] = $did;
				$idx++;
			}
		}
		return compact('note_ids', 'body', 'document_ids');
	}


	function renderEventsButtons($note, $module, $record) {
		require_once 'include/upload_file.php';
		global $current_user;
		global $timedate;

		$fname = UploadFile::get_path($note['filename']);
		$f = @fopen($fname, 'r');
		if (!$f) {
			return '';
		}
		try {
			$vcalendar = VObject\Reader::read($f, VObject\Reader::OPTION_FORGIVING);
		} catch (Exception $e) {
			return '';;
		} finally {
			fclose($f);
		}
		$buttons = '';
		foreach ($vcalendar->VEVENT as $event) {
			$start_formatted = '';
			$diff = '30';

			$user_preferences = $current_user->getUserDateTimePreferences();
			$offset = $user_preferences['userGmtOffset'];
			$offset_minutes = abs($offset) % 60;
			$offset_hours = (int)(abs($offset) / 60);
			$sign = $offset >= 0 ? "+" : "-";
			$tz = new DateTimeZone(sprintf("GMT%s%02d:%02d", $sign, $offset_hours, $offset_minutes));

			if ($event->DTSTART) {			
				$start = $event->DTSTART->getDateTime();
				$start = $start->setTimeZone($tz);
				$start_formatted = $start->format($user_preferences['date'] . ' ' . $user_preferences['time']);
				if ($event->DTEND) {
					$end = $event->DTEND->getDateTime();
					$end = $end->setTimeZone($tz);
					$diffValue = (int)(($end->getTimestamp() - $start->getTimestamp()) / 60);
					if ($diffValue >= 0)
						$diff = $diffValue;
				}
			}

			$name = (string)$event->SUMMARY;
			if($name == '')
				$name = (string)$event->DESCRIPTION;
			$desc = (string)$event->DESCRIPTION;
			$lbl_schedule = to_html(translate('LBL_NEW_FORM_TITLE', 'Meetings').' ('.$name.')');
			$loc = (string)$event->LOCATION;
			$email_record = ListQuery::quick_fetch_row('Email', $record);
			$contact_id = "";
			if($email_record['parent_type'] == 'Contacts')
				$contact_id = $email_record['parent_id'];

			$qc_params = [
				'name' => $name, 
				'description' => $desc,
				'date_start' => $start_formatted,
				'duration' => $diff, 
				'parent_type' => $module,
				'parent_id'  => $record,
				'location' =>  $loc,
				'contact_invitees' => $contact_id,
			];
			$encoded_qc_params = to_html(json_encode($qc_params));

			$buttons .= ' <button name="SubPanel_create_meeting" type="button" class="input-button" onclick="SUGAR.ui.openQuickCreate(\'Meetings\',' . $encoded_qc_params . ')"><div class="input-icon left theme-icon create-Meeting"></div><span class="input-label" id="listView-4a82-create_meeting-label">'.$lbl_schedule.'</span></button> ';
		}
		return $buttons;
	}

	/**
	 * Add attachments init jsavascript
	 *
	 * @param  FormLayout $layout
	 * @return void
	 */
	function addAttachmentsJs($layout) {
		$attJs = 'var file_path = "'.AppConfig::site_url().'/download.php?type=Notes&id=";';
		$attJs .= 'var lnk_remove = "'.translate('LNK_REMOVE', 'app').'";';
		$attJs .= 'var ERR_NOT_ADDRESSED = "'.translate('ERR_NOT_ADDRESSED', 'Emails').'";';
		$layout->addScriptLiteral($attJs, LOAD_PRIORITY_FOOT);
	}


	function previewTemplate() {
		return <<<EOF
<div style="display:none" id="email_dropzone_preview_template">
<div class="dz-preview dz-file-preview">
  <div class="dz-details">
    <div class="dz-filename"><span data-dz-name></span></div>
    <div class="dz-size" data-dz-size></div>
    <img data-dz-thumbnail />
  </div>
  <div class="dz-progress"><span class="dz-upload" data-dz-uploadprogress></span></div>
  <div class="dz-success-mark uii uii-md uii-accept active-icon"></div>
  <div class="dz-error-mark uii uii-md uii-cancel active-icon"></div>
  <div class="dz-delete uii uii-md uii-remove active-icon"></div>
  <div class="dz-error-message"><span data-dz-errormessage></span></div>
</div>
</div>
EOF;
	}

	/**
	 * Render html for attachments upload
	 *
	 * @param HtmlFormGenerator $gen
	 * @return string
	 */
	function renderAttachmentUpload(HtmlFormGenerator $gen) {
		global $mod_strings;

		$html = '<div id="template_attachments"></div>';
		$html .= '<div id="uploads_div"></div>';
		$html .= '<div id="documents_div"></div>';
		$html .= $this->previewTemplate();
		$html .= '<div id="email_dropzone" class="dropzone input-field border-dashed">';
		$html .= '<span class="dz-default dz-message input-label"><span class="uii uii-add"></span> ';
		$html .= '<span class="text-help">' . translate_dd('dropzone_messages', 'dictDefaultMessage') . '</span></span></div>';
		$html .= ' <button type="button" name="add_document_button" onclick="EmailAttachments.addDocument(this.form);" class="input-button"><span class="input-label"><span class="uii uii-add"></span> '.$mod_strings['LBL_ADD_DOCUMENT'].'</span></button>';
		$accessToken = AppConfig::setting("userpref.current_user.global.dropbox_token");
		if ($accessToken) {
			$html .= ' <button type="button" name="add_dropbox_file_button" onclick="addDropboxFile();" class="input-button"><div class="input-icon theme-icon ticon-dropbox left"></div><span class="input-label">'.translate('LBL_ADD_DROPBOX_FILE', 'Emails').'</span></button>';
			$html .= ' <button type="button" name="add_dropbox_link_button" onclick="addDropboxLink();" class="input-button"><div class="input-icon theme-icon ticon-dropbox left"></div><span class="input-label">'.translate('LBL_ADD_DROPBOX_LINK', 'Emails').'</span></button>';
			//$html .= ' <button type="button" name="dropbox_upload_button" onclick="dropboxUpload();" class="input-button input-outer"><div class="input-icon theme-icon ticon-dropbox left"></div><span class="input-label">'.$mod_strings['LBL_DROPBOX_UPLOAD'].'</span></button>';
			$html .= '<script type="text/javascript" src="https://www.dropbox.com/static/api/2/dropins.js" id="dropboxjs" data-app-key="498w0czfaio0g5t"></script>';
		}

		$gen->getFormObject()->addControl('EmailAttachments', 'attachments');

		return $html;
	}

	/**
	 * Render HTMLEditor for editing email body
	 *
	 * @param HtmlFormGenerator $gen
	 * @param  array $descriptions: ['plain'], ['html']
	 * @return string
	 */
	function renderHtmlEditor(HtmlFormGenerator $gen, $descriptions) {
		if ($descriptions['html'] != '') {
			$value = $descriptions['html'] ?? '';
		} else {
			$value = htmlentities($descriptions['plain'] ?? '', ENT_COMPAT, 'UTF-8');
			$value = preg_replace('~\t~', '    ', $value);
			$value = preg_replace('~  ~', '&nbsp; ', $value);
			$value = nl2br($value);
		}

		$spec = array('name' => 'description_html', 'height' => 300);
		$editor = $gen->form_obj->renderHtmlEditor($spec, $value);

		return $editor;
	}

	/**
	 * Add standard signature to email body
	 *
	 * @param  string $body
	 * @param  string $type - email type
	 * @return string
	 */
	function addSignature($body, $type) {
		global $current_user;
		$signed_body = $body;
		$sig = $current_user->getDefaultSignature(false);

		if($sig && $type != 'draft') {
			if($current_user->getPreference('signature_prepend')) {
				$signed_body = $sig['signature_html'].'<p>&nbsp;</p>' . $signed_body;
			} else {
				$signed_body .= '<p>&nbsp;</p>'.$sig['signature_html'];
			}
		}

		return $signed_body;
	}

	/**
	 * Render Email Templates select box
	 *
	 * @param HtmlFormGenerator $gen
	 * @return string
	 */
	function renderTemplateSelect(HtmlFormGenerator $gen, RowResult $record) {
		$onchange = 'fill_email(this.getKey(), this.form);';
		$spec = array(
			'name' => 'email_template', 'module' => 'EmailTemplates', 'bean_name' => 'EmailTemplate', 'onchange' => $onchange,
			'width' => 20,
		);
		$select = $gen->getFormObject()->renderRef($record, $spec);

		return $select;
	}

	/**
	 * Get checked format for Format radio buttons
	 *
	 * @return array
	 */
	function getCheckedComposeFormat() {
		global $current_user;

		$send_as = $current_user->getPreference('email_compose_format');
		$checked = array('plain' => '', 'html' => '');

		if ($send_as == 'html') {
			$checked['html'] = 'checked';
		} else {
			$checked['plain'] = 'checked';
		}

		return $checked;
	}

	/**
	 * Add additional bcc addresses
	 *
	 * @param  array $input - user input($_REQUEST)
	 * @param  string $bcc
	 * @return string
	 */
	static function addAdditionalBcc($input, $bcc) {
		$additional_addr = $bcc;
		if (!empty($input['email_multi']) && !empty($input['list_uids'])) {
			$mass_mod = $input['action_module'];
			$model = AppConfig::module_primary_bean($mass_mod);

			if($model)
			foreach (explode(';', $input['list_uids']) as $lead_id) {
				$obj = ListQuery::quick_fetch_row($model, $lead_id, array('_display', 'email_opt_out', 'invalid_email', 'email1', 'email2'));
				$email = array_get_default($obj, 'email1', $obj['email2']);
				if ($obj && !empty($email) && empty($obj['invalid_email'])) {
					$name = '';
					if (! empty($obj['_display']))
						$name = $obj['_display'];
					$additional_addr .= $name . ' <' . $email . '>; ';
				}

				/** @noinspection PhpUnusedLocalVariableInspection */
				$obj = null;
			}
		}

		return $additional_addr;
	}

	/**
	 * Replace template variables on values
	 *
	 * @static
	 * @param RowUpdate $update
	 * @param string $description
	 * @param string $description_html
	 * @return array
	 */
	static function replaceFields(RowUpdate $update, $description, $description_html) {
		$account_id = $update->getField('account_id');
		$parent_id = $update->getField('parent_id');
		$parent_type = $update->getField('parent_type');
		$parent = array();
		$params = array();

		if (! empty($parent_id)) {
			$parent[$parent_type] = $parent_id;
			$params['primary'] = $parent_type;
		}

		if (! empty($account_id)) {
			$parent['Accounts'] = $account_id;
		}

		$replace_fields = $update->getField('replace_fields');
		$name = $update->getField('name');

		if ($replace_fields) {

			if (!isset($parent['Contacts'])) {
		        $to = normalize_emails_array(parse_addrs($update->getField('to_addrs')), null, true, true);
			    $cc = normalize_emails_array(parse_addrs($update->getField('cc_addrs')), null, true, true);
			    $bcc = normalize_emails_array(parse_addrs($update->getField('bcc_addrs')), null, true, true);
				$all_contacts = array_merge($to, $cc, $bcc);
				$num_contacts = 0;
				foreach ($all_contacts as $contact) {
					if (isset($contact['contact_id'])) {
						if ($num_contacts < 1) {
							$contact_id = $contact['contact_id'];
							$num_contacts ++;
						}
						else
							break;
					}
				}
				if ($num_contacts == 1 && isset($contact_id)) {
					$parent['Contacts'] = $contact_id;
					if (empty($parent_id))
						$params['primary'] = 'Contacts';
				}
			}

			$params['aliases'] = array(
				'Leads'  => 'Contacts',
				'Prospects'  => 'Contacts',
			);

			list($name, $description, $description_html) = TemplateParser::parse_generic(
				array($name, $description, $description_html),
				$parent,
				$params,
				AppConfig::setting('site.templates.preserve_html') ? array(2) : array()
			);
			$update->set(array('replace_fields' => 0));
		}

		return array('name' => $name, 'html' => $description_html, 'plain' => $description);
	}


	function loadUpdateAttachments(array $input) {
		$ret = array(
			'new_notes' => array_filter(explode(',', array_get_default($input, 'new_notes', ''))),
			'new_documents' => array_filter(explode(',', array_get_default($input, 'new_documents', ''))),
			'remove_notes' => array_filter(explode(',', array_get_default($input, 'remove_notes', ''))),
			'reattach_note_id' => array_get_default($input, 'reattach_note_id', ''),
			'relabel_note' => array_get_default($input, 'relabel_note', ''),
		);
		foreach ($ret['remove_notes'] as $nid) {
			$idx = array_search($nid, $ret['new_notes']);
			if ($idx !== false) {
				unset($ret['new_notes'][$idx]);
			}
		}
		return $ret;
	}


	function loadUpdateRequest(RowUpdate $update, array $input) {
		$related_update = array();
		$main_update = array();
		if (isset($input['description_html'])) {
			$description_html = $input['description_html'];
			// temporary fix for html output from CKeditor 3.6.3
			$description_html = preg_replace('~<body id="cke_pastebin".*?</body>~s', '', $description_html);
			$description = html2plaintext($description_html);
		} else if (isset($input['description']))
			$description = $input['description'];

		$send = '0';
		if (isset($input['send'])) $send = $input['send'];
		$related_update['send'] = $send;
		$send_as = 'html';
		if (isset($input['send_as'])) $send_as = $input['send_as'];
		$related_update['send_as'] = $send_as;
		$related_update['attachments'] = $this->loadUpdateAttachments($input);

		$type = $update->getField('type');
		$parent_id = $update->getField('parent_id');
		$parent_type = $update->getField('parent_type');

		if ($parent_type == 'Contacts')
			$main_update['contact_id'] = $parent_id;

		if ($type != 'draft' && isset($description_html)) {
			$after_replace = EmailWidget::replaceFields($update, $description, $description_html);
			$main_update['name'] = $after_replace['name'];
			$description = $after_replace['plain'];
			$description_html = $after_replace['html'];
			$subject = $after_replace['name'];
		}

		if(isset($description_html))
			$main_update['description_html'] = $description_html;
		if(isset($description))
			$main_update['description'] = $description;
		if(isset($subject))
			$main_update['name'] = $subject;
		$main_update['isread'] = 1;

		if(! $update->getField('status') || $type == 'archived') {
			$status = 'draft';
			if ($type == 'archived') $status = 'archived';
			$main_update['status'] = $status;
		}

		$update->set($main_update);
		$update->setRelatedData($this->id.'_rows', $related_update);
	}

	function validateInput(RowUpdate $update) {
		return true;
	}

	function afterUpdate(RowUpdate $update) {
		$sent = $update->getRelatedData('sent_by_email_widget');
		$update->setRelatedData('sent_by_email_widget', false);
		if (!$sent) return;
		if ($update->getField('parent_type' != 'Cases' || !$update->getField('parent_id')))
			return;
		if (!$update->getField('case_status'))
			return;
		$case = ListQuery::quick_fetch('aCase', $update->getField('parent_id'));
		if (!$case)
			return;
		$cu = RowUpdate::for_result($case);
		$cu->set('status', $update->getField('case_status'));
		$res = trim($update->getField('case_resolution'));
		if (strlen($res))
			$cu->set('resolution', $res);
		$cu->save();
	}

	function beforeUpdate(RowUpdate $update) {
		global $mod_strings;

		$row_updates = $update->getRelatedData($this->id.'_rows');
		if(! $row_updates)
			return;

		$email_id = $update->getField('id');
		if (!$email_id) {
			$update->set('id', create_guid());
		}

		// set by Email before_save hook
		$recipients = $update->getRelatedData('recipients');
		$upd_after_send = array();
		$attachments = $this->updateAttachments($update, $row_updates);

		//Start SEND
		if ($recipients && $update->getField('type') == 'out' && $row_updates['send'] == '1') {
			$descriptions = array('html' => $update->getField('description_html'), 'plain' => $update->getField('description'));

			$use_smtp_account = null;
			if (!empty($update->getField('smtp_account_id'))) {
				$acc = ListQuery::quick_fetch_row('EmailPOP3', $update->getField('smtp_account_id'));
				if ($acc && $acc['user_id'] == AppConfig::current_user_id()) {
					$use_smtp_account = $acc;
				}
			}

			$update->setRelatedData('use_smtp_account', $use_smtp_account);
			if (EmailWidget::send($update, $recipients, $descriptions, $attachments, $row_updates['send_as'])) {
				$update->setRelatedData('sent_by_email_widget', true);
				$send_error = false;
				$upd_after_send['status'] = 'sent';
				$upd_after_send['date_start'] = gmdate('Y-m-d H:i:s');
			} else {
				$update->setRelatedData('sent_by_email_widget', true);
				$send_error = true;
				$upd_after_send['status'] = 'send_error';
			}

			$update->set($upd_after_send);

			if ($send_error) {
				$errmsg = empty($GLOBALS['send_error_msg']) ? '' : '<br>&nbsp;&nbsp;&nbsp;'.htmlentities($GLOBALS['send_error_msg']);
				if (isset($mod_strings['LBL_ERROR_SENDING_EMAIL']))
					add_flash_message($mod_strings['LBL_ERROR_SENDING_EMAIL'].$errmsg, 'error');
			} else {
				if (isset($mod_strings['LBL_MESSAGE_SENT']))
					add_flash_message($mod_strings['LBL_MESSAGE_SENT']);
			}
		}
		//End SEND
	}

	function updateAttachments(RowUpdate $update, $row_updates) {
		global $log;
		$email_id = $update->getField('id');
		$module = $update->getModuleDir();
		$remove_notes = $row_updates['attachments']['remove_notes'];

		if ($email_id && $remove_notes) {
			$result = self::fetchAttachments($module, $email_id);
			if($result && ! $result->failed) {
				foreach($result->getRowIndexes() as $idx) {
					$note = $result->getRowResult($idx);
					if(in_array($note->getField('id'), $remove_notes)) {
						$upd = RowUpdate::for_result($note);
						$upd->markDeleted();
					}
				}
			}
		}

		$new_note = RowUpdate::blank_for_model('Note');
		for ($i = 0; $i < EmailWidget::MAX_ATTACHMENTS; $i++) {
			if( ($filename = $new_note->loadInputFile('filename', 'email_attachment'.$i)) ) {
				$new_note->set(array(
					'name' => $new_note->getRelatedFile('filename')->get_display_filename($filename),
					'email_attachment' => 1,
					'assigned_user_id' => $update->getField('assigned_user_id'),
					'parent_type' => $module,
					'parent_id' => $email_id,
				));
				$new_note->save();
				$new_note = RowUpdate::blank_for_model('Note');
			}
		}

		for ($i = 0; $i < EmailWidget::MAX_ATTACHMENTS; $i++) {
			if ( ( $tmpFile = array_get_default($_REQUEST, 'dropbox_temp_file_' . $i)) ) {
		        $fname = $_REQUEST['dropbox_file_' . $i];
				$fl = new UploadFile('');
				$fl->use_soap = true;
				$fl->stored_file_name = $fname;
				$filename = $fl->create_stored_filename();
				$destination = AppConfig::upload_dir() . $filename;
				if (!@copy($tmpFile, $destination)) {
					$log->error("error saving email attachment to $destination");
					continue;
				}
				$note = RowUpdate::blank_for_model('Note');
				$note->set(array(
					'name' => $fname,
					'email_attachment' => 1,
					'parent_type' => $module,
					'parent_id' => $email_id,
					'file_mime_type' => $fl->getMimeSoap($fname),
					'filename' => $filename,
					'assigned_user_id' => $update->getField('assigned_user_id'),
				));
				$note->save();
			}
		}

		$new_documents = $row_updates['attachments']['new_documents'];
		foreach ($new_documents as $did) {
			$doc = ListQuery::quick_fetch('Document', $did);
			if($doc && ($rev_id = $doc->getField('document_revision_id')) ) {
				$rev = ListQuery::quick_fetch('DocumentRevision', $rev_id);
				$new_note->set(array(
					'name' => $doc->getField('document_name'),
					'email_attachment' => 1,
					'assigned_user_id' => $update->getField('assigned_user_id'),
					'parent_type' => $module,
					'parent_id' => $email_id,
					'filename' => UploadFile::duplicate_file2($rev->getField('filename')),
					'file_mime_type' => $rev->getField('file_mime_type'),
				));
				$new_note->save();
				$new_note = RowUpdate::blank_for_model('Note');
			}
		}


		$new_notes = $row_updates['attachments']['new_notes'];
		foreach ($new_notes as $did) {
			$note = ListQuery::quick_fetch_row('Note', $did);
			if($note) {
				$dupe_data = $note;
				unset($dupe_data['id']);
				$new_note->set($dupe_data);
				$new_note->set('filename', UploadFile::duplicate_file2($note['filename']));
				$new_note->set(array(
					'email_attachment' => 1,
	        		'assigned_user_id' => $update->getField('assigned_user_id'),
					'parent_type' => $module,
					'parent_id' => $email_id,
				));
				$new_note->save();
				$new_note = RowUpdate::blank_for_model('Note');
			}
		}


		$reattach_id = $row_updates['attachments']['reattach_note_id'];
		$relabel_note = $row_updates['attachments']['relabel_note'];
		if($reattach_id) {
			$note = ListQuery::quick_fetch('Note', $reattach_id);
			if($note) {
				$new_note = RowUpdate::for_result($note);
				$new_note->set(array(
					'parent_type' => $module,
					'parent_id' => $email_id,
					'email_attachment' => 1,
				));
				if($relabel_note)
					$new_note->set('name', $relabel_note);
				$new_note->save();
			}
		}

		$result = self::fetchAttachments($module, $email_id);
		if($result && ! $result->failed)
			return $result->getRows();
		return array();
	}

	static function fetchAttachments($module, $email_id) {
		$lq = new ListQuery('Note', array('id', 'name', 'filename', 'file_mime_type', 'deleted'));
		$lq->addSimpleFilter('parent_type', $module, '=');
		$lq->addSimpleFilter('parent_id', $email_id, '=');
		return $lq->fetchAll('name');
	}

	/**
	 * Replace Email->send() method for now, maybe later move this one
	 *
	 * @param RowUpdate $update
	 * @param array $recipients - ['to'],['cc'],['bcc']
	 * @param array $descriptions - ['html'], ['plain']
	 * @param array $attachments - Notes array
	 * @param string $send_as - html or plain
	 * @return bool
	 */
	static function send(RowUpdate $update, $recipients, $descriptions, $attachments, $send_as = 'html') {
		global $current_user;

		$smtp_account = $update->getRelatedData('use_smtp_account');
		if (!empty($smtp_account)) {
			$update->set('from_name', $smtp_account['from_name']);
			$update->set('from_addr', $smtp_account['email']);
		}

		$updated_fields = array();
		$email = new Email();
		$message_id = $update->getField('message_id');
		$in_reply_to = $update->getField('in_reply_to');
		$new = false;
		$email_id = $update->getField('id');
		if (empty($email_id)) {
			$email_id = create_guid();
			$updated_fields['id'] = $email_id;
		}

		if (empty($message_id)) {
			$updated_fields['message_id'] = Email::create_message_id();
			$message_id = $updated_fields['message_id'];
			$new = true;
		}

		if ( $new || (isset($update->saved['message_id']) || isset($update->saved['in_reply_to'])) ) {
			$email->message_id = $message_id;
			$email->thread_id = $update->getField('thread_id');
			$email->in_reply_to = $in_reply_to;
			$email->assign_thread_id();
			$updated_fields['thread_id'] = $email->thread_id;
		}

		require_once 'include/SugarPHPMailer.php';
		$mail = new SugarPHPMailer();

		if (defined('inScheduler')) {
			$mail->InitForSend(true);
		} else {
			if ($smtp_account) {
				$mail->Mailer = "smtp";
				$mail->Host = $smtp_account['smtp_host'];
				$mail->Port = $smtp_account['smtp_port'];

				$SMTPSecure = $smtp_account['smtp_security'];
				$mail->SMTPSecure = $SMTPSecure;
				if ($SMTPSecure) {
					$verify = $smtp_account['smtp_verification'];
					$mail->setSSLVerifyOptions($verify);
				}

				$smtp_auth_req = $smtp_account['smtp_auth_req'];
				if ($smtp_auth_req) {
					$mail->SMTPAuth = TRUE;
					if ($smtp_auth_req !== "o365" && $smtp_auth_req !== "gmail") {
						$mail->Username = $smtp_account['smtp_username'];
						$mail->Password = $smtp_account['smtp_password'];
					} else {
						$mail->AuthType = 'XOAUTH2';
						$oauthTokenProvider = new SMTPOAuth($smtp_account['smtp_username'], $smtp_auth_req, ['user_extra_smtp', $smtp_account['id']]);
						$mail->setOAuth($oauthTokenProvider);
					}
				}
			} elseif ($current_user->getPreference('mail_sendtype') == "SMTP" || AppConfig::setting('site.sendmail_disabled')) {
				$mail->Mailer = "smtp";
				$mail->Host = $current_user->getPreference('mail_smtpserver');
				$mail->Port = $current_user->getPreference('mail_smtpport');

				$SMTPSecure = $current_user->getPreference('mail_smtpsecurity');
				$mail->SMTPSecure = $SMTPSecure;
				if ($SMTPSecure) {
					$verify = $current_user->getPreference('mail_smtp_verification');
					$mail->setSSLVerifyOptions($verify);
				}

				$smtp_auth_req = $current_user->getPreference('mail_smtpauth_req');
				if ($smtp_auth_req) {
					$mail->SMTPAuth = TRUE;
					if ($smtp_auth_req !== "o365" && $smtp_auth_req !== "gmail") {
						$mail->Username = $current_user->getPreference('mail_smtpuser');
						$mail->Password = $current_user->getPreference('mail_smtppass');
					} else {
						$mail->AuthType = 'XOAUTH2';
						$oauthTokenProvider = new SMTPOAuth($current_user->getPreference('mail_smtpuser'), $smtp_auth_req, ['user_smtp', $current_user->id]);
						$mail->setOAuth($oauthTokenProvider);
					}
				}
			} elseif ($current_user->getPreference('mail_sendtype') == 'sendmail') {
				$mail->Mailer = "sendmail";
			}

		}

		if (is_array($recipients['to'])) {
			foreach ($recipients['to'] as $addr_arr) {
				if ( empty($addr_arr['display'])) {
					$mail->AddAddress($addr_arr['email'], "");
				} else {
					$mail->AddAddress($addr_arr['email'], $addr_arr['display']);
				}
			}
		}
		if (is_array($recipients['cc'])) {
			foreach ($recipients['cc'] as $addr_arr) {
				if ( empty($addr_arr['display'])) {
					$mail->AddCC($addr_arr['email'], "");
				} else {
					$mail->AddCC($addr_arr['email'], $addr_arr['display']);
				}
			}
		}
		if (is_array($recipients['bcc'])) {
			foreach ($recipients['bcc'] as $addr_arr) {
				if ( empty($addr_arr['display'])) {
					$mail->AddBCC($addr_arr['email'], "");
				} else {
					$mail->AddBCC($addr_arr['email'], $addr_arr['display']);
				}
			}
		}

		$from_addr = $update->getField('from_addr');
		$from_name = $update->getField('from_name');
		if(empty($from_addr)) {
			$mail->From = $current_user->getPreference('mail_fromaddress');
			$updated_fields['from_addr'] = $mail->From;
			$mail->FromName =  $current_user->getPreference('mail_fromname');
			$updated_fields['from_name'] = $mail->FromName;
			$from_name = $mail->FromName;
		} else {
			$mail->From = $from_addr;
		}

		$mail->Sender = $mail->From;
		$mail->FromName = from_html($from_name);
		$mail->AddReplyTo($mail->From, $mail->FromName);
		$mail->Subject = from_html($update->getField('name'));

		//Start ADD ATTACHMENTS
		foreach ($attachments as $note) {
			$file_location = rawurldecode(UploadFile::get_path($note['filename'], $note['id']));
			$filename = basename($note['filename']);
			$mime_type = array_get_default($note, 'file_mime_type', 'text/plain');
			$mail->AddAttachment($file_location, $filename, 'base64', $mime_type);
		}
		//End ADD ATTACHMENTS

		if (! empty($descriptions['html']) && $send_as == 'html') {
			// wp: if body is html, then insert new lines at 996 characters. no effect on client side
			// due to RFC 2822 which limits email lines to 998
			$cids = array();
			$html= show_inline_images($descriptions['html']);
			$html = make_inline_images($html, $cids);

			foreach ($cids as $cid => $filename) {
				$type = 'application/octet-stream';
				$m = array();

				if (preg_match('~\.([a-z]+)$~', $filename, $m)) {
					if (in_array(strtolower($m[1]), array('gif', 'jpeg', 'png', 'tiff'))) {
						$type = 'image/' . strtolower($m[1]);
					}
				}

				$mail->AddEmbeddedImage($filename, $cid, '', 'base64', $type);
			}

			$mail->Body = $html;
			$mail->AltBody = from_html($descriptions['plain']);
			$mail->IsHTML(true);
		} else {
			$mail->Body = from_html($descriptions['plain']);
		}

		$mail->MessageID = $message_id;
		$mail->prepForOutbound(false, $email_id);
		$mail->AllowEmpty = true;

		if (! empty($in_reply_to)) {
			$mail->AddCustomHeader('In-Reply-To: ' . from_html($in_reply_to));
		}

		$references = $email->get_references();
		if (! empty($references)) {
			$mail->AddCustomHeader('References: ' . from_html($references));
		}

		if ($mail->send()) {
			$updated_fields['folder'] = EmailFolder::get_std_folder_id($update->getField('assigned_user_id'), STD_FOLDER_SENT);
			$updated_fields['type'] = 'out';
			$updated_fields['send_error'] = 0;

			if($attachments) {
				if($update->getField('parent_type') == 'Quotes') {
					$quote = ListQuery::quick_fetch('Quote', $update->getField('parent_id'));
					if ($quote) {
						if($quote->getField('quote_stage') == 'Draft') {
							$qu = RowUpdate::for_result($quote);
							$qu->set('quote_stage', 'Delivered');
							$qu->save();
						}

					}
				}
			}

			$status = true;
			$update->setRelatedData('in_reply_to', $in_reply_to);
		} else {

			$GLOBALS['log']->warn("Error emailing:".$mail->ErrorInfo);
			$GLOBALS['send_error_msg'] = $mail->ErrorInfo;

			$updated_fields['folder'] = EmailFolder::get_std_folder_id($update->getField('assigned_user_id'), STD_FOLDER_DRAFTS);
			$updated_fields['type'] = 'draft';

			$status = false;
		}

		$update->set($updated_fields);

		return $status;
	}

	function renderPdf(PdfFormGenerator $gen, $row_result, array $parents, array $context) {
		global $mod_strings;
		$backgrounds = array(
			array(
				array(245, 245, 245),
				array(255, 255, 255),
			),
			array(
				array(240, 240, 240),
				array(250, 250, 250),
			),
		);
		$opts = array(
			'border' => 0,
		);
		$cols = array(
			'label' => array(
				'width' => '20%',
				'background-color' => $backgrounds[0][0],
				'alt-background-color' => $backgrounds[1][0],
			),
			'text' => array(
				'width' => '80%',
				'background-color' => $backgrounds[0][1],
				'alt-background-color' => $backgrounds[1][1],
			),
		);
		$data = array();
		$address_fields = array(
			'to_addrs' => 'LBL_TO',
			'cc_addrs' => 'LBL_CC',
			'bcc_addrs' => 'LBL_BCC',
		);
		$str = '';
		if (($s = $row_result->getField('from_name')) !== '') $str .= $s. ' ';
		$str .= '<' . $row_result->getField('from_addr') . ">\n";
		$data[] = array(
			'label' => $mod_strings['LBL_FROM'],
			'text' => $str,
		);
		foreach ($address_fields as $f => $label) {
			$str = '';
			$arr = @parse_addrs($row_result->row[$f]);
			$arr = merge_email_names($arr, $row_result->row[$f . '_names']);
			foreach($arr as $addr) {
				if ($addr['display'] !== '') $str .= $addr['display'] . ' ';
				$str .= '<' . $addr['email'] . ">\n";
			}
			$data[] = array(
				'label' => $mod_strings[$label],
				'text' => $str,
			);
		}
		$data[] = array(
			'label' => $mod_strings['LBL_SUBJECT'],
			'text' => array_get_default($row_result->formatted, 'name', ''),
		);

		$description_html = $row_result->getField('description_html');
		$description = $row_result->getField('description');
		if ($description != '') {
			$desc = $description;
			$desc = preg_replace('~\t~', '    ', $desc);
		} else {
			$desc = html2plaintext(to_html($description_html), false);
		}

		$data[] = array(
			'label' => $mod_strings['LBL_BODY'],
			'text' => $desc,
		);
		$gen->pdf->DrawTable($data, $cols, '   ', false, $opts);
	}
}
?>
