<?php

require_once 'modules/Quotes/PDFTemplatesProvider.php';
require_once 'modules/ExpenseReports/ExpenseReport.php';
require_once 'modules/BookingCategories/BookingCategory.php';

class ExpensePDFTemplateProvider extends QuotePDFTemplateProvider
{
	
	protected function __construct($type)
	{
		parent::__construct($type);
	}

	public static function template_types()
	{
		return array(
			'expense_report' => translate('LBL_PDF_REPORT_TITLE', 'ExpenseReports'),
		);
	}

	public static function template_skeleton($type)
	{
		$ret = array('sections' => array(
			array(
				'name' => 'header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_HEADER',
				'height' => 150,
				'elements' => array (),
			),
			array(
				'name' => 'expense_items',
				'type' => 'table',
				'title' => 'LBL_SKEL_EXPENSE_ITEMS',
				'columns' => array (),
			),
			array(
				'name' => 'expense_totals_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_EXPENSE_TOTALS_HEADER',
				'columns' => array (),
			),
			array(
				'name' => 'expense_totals',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_EXPENSE_TOTALS',
				'columns' => array (),
			),
			array(
				'name' => 'footer',
				'data_id' => 'header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_FOOTER',
				'height' => 40,
				'elements' => array (),
			),
		));
		foreach ($ret['sections'] as &$sec) $sec['title'] = translate($sec['title'], 'ExpenseReports');
		$ret['sections'] = array_values($ret['sections']);
		return $ret;
	}

	public static function template_sample_data($type)
	{
		$datefmt = AppConfig::setting('locale.defaults.date_format');
		$data = array(
			'header' => array(
				'todays_date' => array(translate('LBL_TODAYS_DATE'), date($datefmt), ),
				'number' => array(translate('LBL_SAMPLE_HEADER_NUMBER_LABEL', 'ExpenseReports'), '2013-001'),
				'date' => array(translate('LBL_SAMPLE_HEADER_DATE_LABEL', 'ExpenseReports'), '2014-11-05'),
				'description' => array(translate('LBL_SAMPLE_HEADER_DESCRIPTION_LABEL', 'ExpenseReports'), translate('LBL_SAMPLE_HEADER_DESCRIPTION_VALUE', 'ExpenseReports'), ),
				'employee' => array(translate('LBL_SAMPLE_HEADER_EMPLOYEE_LABEL', 'ExpenseReports'), translate('LBL_SAMPLE_HEADER_EMPLOYEE_VALUE', 'ExpenseReports'), ),
			),
			'expense_totals' => array(
				'total_name' => array(translate('LBL_SAMPLE_EXPENSE_TOTALS_TOTAL_NAME_LABEL', 'ExpenseReports'), translate('LBL_SAMPLE_EXPENSE_TOTALS_TOTAL_NAME_VALUE', 'ExpenseReports'), ),
				'advance_value' => array(translate('LBL_SAMPLE_EXPENSE_TOTALS_ADVANCE_VALUE_LABEL', 'ExpenseReports'), translate('LBL_SAMPLE_EXPENSE_TOTALS_ADVANCE_VALUE_VALUE', 'ExpenseReports'), ),
				'total_value' => array(translate('LBL_SAMPLE_EXPENSE_TOTALS_TOTAL_VALUE_LABEL', 'ExpenseReports'), translate('LBL_SAMPLE_EXPENSE_TOTALS_TOTAL_VALUE_VALUE', 'ExpenseReports'), ),
				'balance_value' => array(translate('LBL_SAMPLE_EXPENSE_TOTALS_BALANCE_VALUE_LABEL', 'ExpenseReports'), translate('LBL_SAMPLE_EXPENSE_TOTALS_BALANCE_VALUE_VALUE', 'ExpenseReports'), ),
			),
			'expense_items' => array(
				'number' => array(translate('LBL_SAMPLE_EXPENSE_ITEMS_NUMBER_LABEL', 'ExpenseReports'), '1'),
				'number2' => array(translate('LBL_SAMPLE_EXPENSE_ITEMS_NUMBER2_LABEL', 'ExpenseReports'), '1'),
				'date' => array(translate('LBL_SAMPLE_EXPENSE_ITEMS_DATE_LABEL', 'ExpenseReports'), '2014-11-05'),
				'category' => array(translate('LBL_SAMPLE_EXPENSE_ITEMS_CATEGORY_LABEL', 'ExpenseReports'), translate('LBL_SAMPLE_EXPENSE_ITEMS_CATEGORY_VALUE', 'ExpenseReports'), ),
				'quantity' => array(translate('LBL_SAMPLE_EXPENSE_ITEMS_QUANTITY_LABEL', 'ExpenseReports'), 2),
				'rate' => array(translate('LBL_SAMPLE_EXPENSE_ITEMS_RATE_LABEL', 'ExpenseReports'), '9.00'),
				'pretax' => array(translate('LBL_SAMPLE_EXPENSE_ITEMS_PRETAX_LABEL', 'ExpenseReports'), '18.00'),
				'amount' => array(translate('LBL_SAMPLE_EXPENSE_ITEMS_AMOUNT_LABEL', 'ExpenseReports'), '18.00'),
				'description' => array(translate('LBL_SAMPLE_EXPENSE_ITEMS_DESCRIPTION_LABEL', 'ExpenseReports'), translate('LBL_SAMPLE_EXPENSE_ITEMS_DESCRIPTION_VALUE', 'ExpenseReports'), ),
			),
		);
		$ret = array();
		foreach ($data as $k => $v) {
			$ret[$k] = array();
			foreach ($v as $kk => $vv) {
				$ret[$k][$kk] = array('label' => $vv[0], 'value' => $vv[1]);
			}
		}
		return $ret;
	}

	public static function create($type)
	{
		return new ExpensePDFTemplateProvider($type);
	}
	
	public function iterate(PDFTemplatePrinter $p)
	{
		$this->iterateHeaderFooter($p, 'header');
		$p->renderSection($this->findSection('expense_items'), $this->data['expense_items']);
		$this->iterateTotals($p);
		$this->iterateHeaderFooter($p, 'footer');
	}
	
	protected function iterateTotals(PDFTemplatePrinter $p)
	{
		$totals = $this->getData('expense_totals');
		if(! is_countable($totals) || ! count($totals)) return;
		$h = 0;
		$head = $this->findSection('expense_totals_header');
		if($head) {
			$p->adjustSectionHeight($head, array());
			$h += $head['height'];
		}
		$section = $this->findSection('expense_totals');
		$p->adjustSectionHeight($section, $totals[0]);
		$totalsCount = is_countable($totals) ? count($totals) : 0;
		$h += $totalsCount * $section['height'];
		
		$p->testWrapSection($h);
		$p->renderSection($head, array());
		foreach ($totals as $data) {
			$p->renderSection($section, $data);
		}
	}

	protected function get_lines() 
	{
		static $taxCodes;
		if (!$taxCodes) {
			$taxCodes = array();
			require_once 'modules/TaxCodes/TaxCode.php';
			$tc = new TaxCode;
			$codes = $tc->get_full_list('position asc');
			$tc->retrieve('-99');
			array_unshift($codes, $tc);
			foreach ($codes as $tc) {
				$taxCodes[$tc->id] = $tc->code;
				$tc->cleanup();
			}
			unset($codes);
		}
		

		$items = ExpenseReport::get_line_items($this->row->getField('id'));
		$currency =& $this->get_currency();
		$data = array();
		$num = 1;

		$cat = new BookingCategory;
		$categories = $cat->get_option_list('expenses');
		
		foreach($items as $row) {
			if ($row['split']) $num2 = 1;
			$desc = $row['description'];
			$tax_class = array_get_default($row, 'tax_class_id', '-99');
			if($tax_class != '-99') {
				$taxinfo = translate('LBL_PDF_TAX_CODE_PREFIX', 'ExpenseReports') . $taxCodes[$tax_class];
				if(strlen($desc)) $desc = $taxinfo . "\n" . $desc;
				else $desc = $taxinfo;
			}
			$datarow = array(
				'number' => $num ++,
				'category' => array_get_default($categories, $row['category'], ''),
				'quantity' => $row['quantity'] ? $row['quantity'] : '',
				'rate' => $row['paid_rate'] ? $this->currency_format($row['paid_rate'], $currency->id) : '',
				'pretax' => $this->currency_format($row['amount'], $currency->id),
				'amount' => $this->currency_format($row['total'], $currency->id),
				'raw_amount' => $row['amount'],
				'date' => $row['date'],
				'description' => $desc,
				'split' => $row['split'],
			);
			if ($row['parent_id']) {
				$datarow['category'] = '        ' . $datarow['category'];
				$datarow['number2'] = $num2 ++;
				unset($datarow['number']);
			}
			if ($row['split']) {
				unset($datarow['quantity']);
				unset($datarow['rate']);
			}
			$datarow['raw_amount'] = $row['amount'];
			$data[] = $datarow;
		}
		return $data;
	}

	public function initialize($layout, ListQuery $lq)
	{
		return parent::initialize($layout, $lq);
	}
	
	public function getFileName() {
		$filename_pfx = translate('LBL_FILENAME_REPORT', 'ExpenseReports');
		$filename = $filename_pfx . '_' . $this->row->getField('prefix');
		$filename .= $this->row->getField('report_number') . '_' . $this->row->getField('assigned_user_name');
		$filename = preg_replace('~\.+$~', '', trim($filename));
		$filename = $filename . '.pdf';
		return $filename;
	}

	public function getNoteTitle()
	{
		return translate('LBL_PREPARED_REPORT', 'ExpenseReports');
	}

	public function getEmailTitle()
	{
		return translate('LBL_EMAILED_REPORT', 'ExpenseReports');
	}

	public function getSignedDocumentTitle() {
		$prefix =
			$this->row
			? '[' . $this->row->getField('prefix') . $this->row->getField('report_number') . '] '
			: '';

		return $prefix . $this->getNoteTitle();
	}

	protected function createData()
	{
		global $app_list_strings, $timedate;
		$row =& $this->row;
		$user = new User();
		$user->retrieve($row->getField('assigned_user_id'), false);
		$terms = AppConfig::setting("lang.lists.current.app.terms_dom", array());
		$currency =& $this->get_currency();
		$tax_information = $this->row->getField('tax_information');
		$system_currency =& $this->get_system_currency();

		$user = new User();
		$user->retrieve($row->getField('assigned_user_id'));
		$datefmt = AppConfig::setting('locale.defaults.date_format');
		$data = array(
			'todays_date' => date($datefmt),
			'number' => $row->getField('prefix') . $row->getField('report_number'),
			'date' => $timedate->to_display_date($row->getField('date_entered')),
			'employee' => $user->name,
			'description' => $row->getField('description'),
		);
		$data = array_merge($this->header_data, $data);
		$this->data['header'] = $data;

		$this->data['expense_items'] = $this->get_lines();

		$this->createTotalsData();
	}

	private function createTotalsData() {
		$row =& $this->row;
		$currency_row = $exchange_row = $amount_row = $usd_row = array();
		
		$currency = $this->get_advance_currency();
		$advance = $this->currency_format($row->getField('advance'), $currency->id);

		$total = 0.0;
		foreach ($this->data['expense_items'] as $l) {
			$total += $l['raw_amount'];
		}

		if ($row->getField('advance_currency_id') != '-99') {
			$currency_row['advance_value'] = $currency->name;
			$exchange_row['advance_value'] = number_format($row->getField('advance_exchange_rate'), 5);
			$amount_row['advance_value'] = $advance;
		}

		$currency = $this->get_system_currency();
		$advance = $this->currency_format($row->getField('advance_usdollar'), $currency->id);
		$usd_row['advance_value'] = $advance;

		$currency =& $this->get_currency();
		$total = $this->currency_format($total + $row->getField('tax'), $currency->id);

		if ($row->getField('currency_id') != '-99') {
			$currency_row['total_value'] = $currency->name;
			$exchange_row['total_value'] = number_format($row->getField('exchange_rate'), 5);
			$amount_row['total_value'] = $total;
		}

		$currency =& $this->get_system_currency();
		$total = $this->currency_format($row->getField('total_amount_usdollar'), $currency->id);
		$usd_row['total_value'] = $total;


		$currency =& $this->get_currency();
		$balance = $this->currency_format($row->getField('balance'), $currency->id);
		if ($row->getField('currency_id') != '-99') {
			$currency_row['balance_value'] = $currency->name;
			$exchange_row['balance_value'] = number_format($row->getField('exchange_rate'), 5);
			$amount_row['balance_value'] = $balance;
		}

		$currency =& $this->get_system_currency();
		$balance = $this->currency_format($row->getField('balance_usdollar'), $currency->id);
		$usd_row['balance_value'] = $balance;


		if (!empty($currency_row)) {
			$currency =& $this->get_system_currency();
			$currency_row['total_name'] =  translate('LBL_PDF_CURRENCY', 'ExpenseReports');
			$exchange_row['total_name'] = translate('LBL_PDF_EXCHANGE_RATE', 'ExpenseReports');
			$amount_row['total_name'] = translate('LBL_PDF_TOTAL_AMOUNT', 'ExpenseReports');
			$totals = array(
				$currency_row,
				$exchange_row,
				$amount_row,
			);
		} else {
			$totals = array();
		}

		$usd_row['total_name'] = sprintf(translate('LBL_PDF_AMOUNT_USD', 'ExpenseReports'), $currency->name);
		$totals[] = $usd_row;
		$this->data['expense_totals'] = $totals;

	}

	function &get_system_currency() {
    	static $currency;
    	if(! isset($currency)) {
			require_once('modules/Currencies/Currency.php');
			$currency = new Currency();
			$currency->loadDefaultCurrency();;
		}
		return $currency;
    }
	
	function &get_advance_currency() {
		static $currency;
		if(! isset($currency)) {
			require_once('modules/Currencies/Currency.php');
			$currency = new Currency();
			$currency->retrieve($this->row->getField('advance_currency_id'), false);
		}
		return $currency;
    }

	public static function get_extra_sources() {
		return array(
			array(
				'module' => 'Accounts',
				'model' => 'Account',
				'prefix' => 'account',
				'label' => 'LBL_ACCOUNT_NAME',
			),
		);
	}}
	
