AppBase.module('expense-reports').moduleRequires('ui-controls').init(function(App) {
	var UI = App.ui,
		Wrap = App.dom.extNode,
		QueryId = App.dom.queryId,
		All = App.dom.ExtNodeList,
		MakeElt = App.dom.makeElement;

	UI.ExpenseEditor = function(id, params) {
		this.id = 'ExpenseEditor';
		this.readOnly = false;
		this.topItems = {};
		this.allItems = {};
		this.itemCnt = 1;
		this.elements = {};
		this.itemsOrder = [];
		this.dialogs = {};
		this.Categories = {};
		this.fileInputs = {};

		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else {
			this.id = id;
			this.elt = $(id + '-expense_groups');
		}
		if(params) extendObject(this, params);
	}

	UI.ExpenseEditor.prototype = {
		init: function()
		{
		},

		getDragConfig: function(ts) {
			return {
				as_source: true,
				as_target: true,
				drop_effect: 'move',
				onStart: function(evt) {
					ts.dragDirection = 0;
					ts.dragY = evt.clientY;
					var el = evt.target;
					var parentId = el.getAttribute('data-parent-id');
					setTimeout(function() {
						All('.timesheet-row-expenses-split').css('opacity', 0.1);
						if (parentId) {
							All('.timesheet-row-expenses').css('opacity', 0.1);
							All('.timesheet-row-expenses[data-parent-id="' + parentId + '"]').css('opacity', 1);
						} else {
							All('.timesheet-row-expenses').css('opacity', 1);
							All('.timesheet-row-expenses[data-parent-id]').css('opacity', 0.1);
						}
					}, 0);
					ts.draggingEl = el;
				},
				onMove: function(evt) {
					this.dragDirection = evt.clientY - this.dragY;
					if (this.dragDirection != 0) {
						this.dragDirection /= Math.abs(this.dragDirection);
					}
					var y = evt.clientY;
					this.dragY = y;
					var parents = Wrap(evt.target).parents('.timesheet-row-expenses');
					var row = parents[0];

					if (!row) return;

					var parentId = ts.draggingEl.getAttribute('data-parent-id');

					var filter = function(row) {
						if (parentId && row.getAttribute('data-parent-id') !== parentId)
							return false;
						else if (!parentId && row.getAttribute('data-parent-id'))
							return false;
						return true;
					}

					if (!filter(row)) return;

					var region = Wrap(row).bounds(true);
					var delta = (region.bottom - region.top) / 4;
					var top = region.top + delta;
					var bottom = region.top + delta * 3;
					var before, beforeId;
					if (y < top && this.dragDirection < 0) {
						before = row;
					} else if (y > bottom && this.dragDirection > 0) {
						before = row.nextSibling;
					} else {
						return;
					}

					if (before) {
						beforeId = before.getAttribute('data-id');
						row.parentNode.insertBefore(ts.draggingEl, before);
					} else {
						row.parentNode.appendChild(ts.draggingEl);
					}

					var order = parentId ? this.topItems[parentId].itemsOrder : ts.itemsOrder;
					order.splice(0, order.length);
					var args = All('.timesheet-row-expenses').filter(filter).map(function(row) {return row.getAttribute('data-id');});
					args.unshift(0);
					args.unshift(0);
					Array.prototype.splice.apply(order, args);
				},
				checkAccept: function(evt, data) {
					return true;
				},
				onEnd: function(evt) {
					All('.timesheet-row-expenses').css('opacity', 1);
						All('.timesheet-row-expenses-split').css('opacity', 1);
					ts.paint();
				}
			}
		},

		setup : function() {
			if(this.set_up) return;
			this.set_up = true;
			//new UI.DragTarget(this.elt, dragConfig);
			var self = this;
			SUGAR.ui.onInitForm(this.form, function() {
				self.paint();
				if(! self.readOnly)
					self.updateTotals();
			});
			this.currency_input = SUGAR.ui.getFormInput(this.form, 'currency') || this.currencyStub('currency', this.currency_id);
			SUGAR.ui.attachInputEvent(this.currency_input, 'onrateupdate', function(old_rate, new_rate) {
				self.updateExchangeRate(old_rate, new_rate);
			});
			this.advance_currency = SUGAR.ui.getFormInput(this.form, 'advance_currency') || this.currencyStub('advance_currency', this.advance_currency_id);
			SUGAR.ui.attachInputEvent(this.advance_currency, 'onrateupdate', function(old_rate, new_rate) {
				self.updateAdvanceExchangeRate(old_rate, new_rate);
			});
			SUGAR.ui.attachFormInputEvent(this.form, 'advance', 'onchange', function() {
				self.updateAdvanceAmount();
			});

			var button = $(this.id + '-expense-add');
			if (button) {
				button.onclick = function() {
					self.addItem();
					self.paint();
				};
			}
			if (this.add_items) {
				this.add_items.forEach(function(item) {
					this.addItem(item.parent_id, item, true);
				}.bind(this));
			}
			this.detectHTML5();
		},

		detectHTML5 : function()
		{
			var input = document.createElement("input");
			input.setAttribute("type", "email");
			this.html5 = (input.type !== "text");
			delete input;
		},

		currencyStub : function(id, value) {
			var sel = new SUGAR.ui.CurrencySelect(id, {init_key: value, form: this.form});
			sel.setup();
			return sel;
		},

		setItems : function(items) {
			this.topItems = items;
		},

		addItem : function(parentId, values, keepId) {
			var id ;
			if (!keepId) {
				id = '~newitem~' + this.itemCnt++;
			} else  {
				id = values.id;
			}
			var item = {
				category: '',
				total: 0.0,
				total_usdollar: 0.0,
				amount: 0.0,
				amount_usdollar: 0.0,
				tax_class_id : '-99',
				quantity: 0.0,
				paid_rate : 0.0,
				paid_rate_usd : 0.0,
				description : '',
				'date' : '',
				attachmentSrc : null,
				unit: ''
			};
			if (values) {
				for (var k in values) {
					item[k] = values[k];
				}
			}
			if (parentId) {
				delete item.attachmentSrc;
			}
			item.children = {};
			item.itemsOrder = [];
			item.split = 0;
			item.id = id;
			item.parent_id = parentId;

			if (blank(item.category)) {
				for (var i = 0; i < this.Categories.order.length; i++) {
					item.category = this.Categories.order[i];
					var category = this.Categories.values[item.category];
					item.unit = category.expenses_unit;
					item.paid_rate = category.paid_rate_usd * this.currencyRate();
					break;
				}
			}

			if (parentId) {
				var parentItem = this.topItems[parentId];
				if (!parentItem) {
					return;
				}
				parentItem.children[id] = item;
				parentItem.itemsOrder.push(id);
				parentItem.split = 1;
				parentItem.expanded = true;
				item['date'] = parentItem['date'];
			} else {
				this.topItems[id] = item;
				this.itemsOrder.push(id);
			}
			this.allItems[id] = item;
			return item;
		},

		// find item position:
		// -1 if idx == 0
		// 1 if idx >= count - 1
		// 0 otherwise
		itemPosition: function(idx, count) {
			return (idx == 0) ? -1 : ((idx >= count - 1) ? 1 : 0);
		},

		paint : function() {
			var self = this;
			if (!this.readOnly) {
				SUGAR.ui.getFormInput(this.form, 'total_pretax').setReadOnly(true);
				SUGAR.ui.getFormInput(this.form, 'total_amount').setReadOnly(true);
				SUGAR.ui.getFormInput(this.form, 'balance').setReadOnly(true);
			}
			for (var i in this.fileInputs) {
				this.form.appendChild(this.fileInputs[i]);
			}
			var groupsContainer = $(this.id + '-expense_groups');
			groupsContainer.innerHTML = '';
			var rCnt = 0;

			if(! this.itemsOrder.length) {
				groupsContainer.appendChild(
					createElement2('div', {className: 'row expense-items-blank'},
						createElement2('span', {className: 'input-field-pad text-help'},
							mod_string('LBL_NO_LINE_ITEMS', 'ExpenseReports'))));
			}

			for (var i = 0; i < this.itemsOrder.length; i++) {
				var iId = this.itemsOrder[i];
				var item = this.topItems[iId];
				this.addItemLine(item, groupsContainer, this.itemPosition(i, this.itemsOrder.length));

				if (item.split && item.expanded) {
					for (var j = 0; j < item.itemsOrder.length; j++) {
						var cId = item.itemsOrder[j];
						var child = item.children[cId];
						this.addItemLine(child, groupsContainer, this.itemPosition(j, item.itemsOrder.length));
					}

					if (!this.readOnly) {
						var row = createElement2('div', {className: 'col-space-sm row timesheet-row-expenses-split'}, null, groupsContainer);
						var cell = createElement2('div', {className: 'column expense-col-add-split'}, null, row);
						var button = this.createIconLink('add', 'LBL_ADD_SPLIT', null, null, 'uii-md');
						button.iId = iId;
						button.onclick = function()
						{
							var newItem = self.addItem(this.iId, self.allItems[this.iId]);
							if (self.hasUnits(newItem)) {
								newItem.amount = newItem.quantity * newItem.paid_rate;
							} else {
								newItem.amount = 0;
							}
							self.recalcTaxes(newItem);
							self.recalculateSplit(newItem.parent_id);
							self.paint();
							var categorySelect = self.getElement([newItem.id, 'category']);
							categorySelect.focus();
							return false;
						};
						cell.appendChild(button);
					}
				}
			}
		},

		createTableHeader : function(row) {
			var self = this;
			var headers = [
				['', {width: '20px'}],
				['LBL_ITEM_DATE', {}],
				['LBL_ITEM_CATEGORY', {colSpan : 2}],
				['LBL_ITEM_QUANTITY', {colSpan : 2}],
				['LBL_ITEM_RATE', {colSpan : 2}],
				['LBL_ITEM_PRETAX', {}],
				['LBL_ITEM_TAX', {}],
				['LBL_ITEM_AMOUNT', {}],
				['LBL_ITEM_DESCRIPTION', {}]
			];
			var cCnt = 0;
			var cell;
			for (var i = 0; i < headers.length; i++) {
				cell = row.insertCell(cCnt++);
				cell.appendChild(headers[i][0] ? document.createTextNode(mod_string(headers[i][0])) : nbsp());
				setAttrs(cell, {'class': 'dataLabel', style:{fontWeight:'bold', whiteSpace: 'nowrap'} });
				setAttrs(cell, headers[i][1]);
			}

		},

		addUploadLine : function(item, row) {
		},

		getExpenseDescription: function(item) {
			return function() {
				var description = '' + (item.description || '');
				var container = MakeElt('span');
				if (item.split) {
					container.append(this.formatCurrency(item.total));
				} else {
					var showTotal = false;
					if (this.hasUnits(item)) {
						container.append(item.quantity + ' ' + this.unitName(item) + ' x ' + this.formatCurrency(item.paid_rate));
						showTotal = true;
					} else {
						container.append(this.formatCurrency(item.amount));
					}
					var tid = item.tax_class_id;
					if (tid && tid != "-99") {
						if (SysData.taxcodes[tid]) {
							container.append(' +' + SysData.taxcodes[tid].code + '');
							showTotal = true;
						}
					}
					if (showTotal) {
						container.append(' = ' + this.formatCurrency(item.total));
					}
				}
				if (description.length) {
					container
						.append(' ')
						.append(MakeElt('i').append('(').append(description).append(')'))
				}
				if (!container.html().length) {
					container.html('&nbsp;');
				}
				return container;
			}
		},

		addItemLine : function(item, container, pos) {
			var self = this;
			var cCnt = 0;

			var parent = createElement2('div', {className: 'timesheet-row-expenses row-space-sm col-space-sm row'}, null, container);
			parent.setAttribute('data-id', item.id);
			if (item.parent_id) {
				parent.setAttribute('data-parent-id', item.parent_id);
			}

			var handleCol = createElement2('div', {className: "column expense-col-split"}, null, parent);
			if (!this.readOnly) {
				var handle = createElement2('div',  {className: "uii uii-hamburger uii-lg drag-handle active-icon"}, null, handleCol);
				new UI.Draggable(parent, this.getDragConfig(this));
				new UI.DragTarget(parent, this.getDragConfig(this));
			}

			var actionCell = createElement2('div', {className: 'column expense-col-split'}, null, parent);
			var button = null;
			var decimals = this.currencyDecimalPlaces();

			if (!item.split && !item.parent_id) {
				if (!this.readOnly) {
					button = this.createIconLink('split', '');
					var iId = item.id;
					button.onclick = function()
					{
						var newItem = self.addItem(iId, item);
						self.recalcTaxes(newItem);
						self.paint();
						return false;
					};
				} else {
					button = null;
				}
			} else {
				if (item.split) {
					if (item.expanded) {
						button = this.createIconLink('clear', '');
						button.onclick = function()
						{
							item.expanded = false;
							self.paint();
							return false;
						};
					} else {
						button = this.createIconLink('split', '');
						button.onclick = function()
						{
							item.expanded = true;
							self.paint();
							return false;
						};
					}
				}
			}

			if (button) {
				actionCell.appendChild(button);
			} else {
				actionCell.innerHTML = '&nbsp;';
			}

			var dateCell = createElement2('div', {className: 'column expense-col-date'}, null, parent);
			if (blank(item.parent_id)) {
				var date_id = 'calendar_' + this.itemCnt++;
				var dateInput = new SUGAR.ui.DateTimeInput(date_id, {
					init_value: item['date'],
					block: true,
					format: 'date',
					onchange: function() {
						item['date'] = this.getValue();
					},
					disabled: this.readOnly
				});
				var dateDiv = dateInput.render();
				dateCell.appendChild(dateDiv);
				dateInput.setup();
				dateInput.updateDisplay();
			} else {
				dateCell.innerHTML = '&nbsp;';
			}


			if (item.parent_id) {
				//var spacerCell = createElement2('div', {className: 'column expense-col-date'}, null, parent);
			}
			var categoryCell = createElement2('div', {className: 'column expense-col-category'}, null, parent);
			var options = this.Categories;
			var attrs = {
				onchange: function(value) {
					item.category = value;
					if (!item.split) {
						item.quantity = 0.0;
						item.amount = 0.0;
						item.unit = self.Categories.values[value].expenses_unit;
						var rate = self.currencyRate();
						item.paid_rate = self.paidRate(value) * rate;
						self.recalcTaxes(item);
						if (item.parent_id) {
							self.recalculateSplit(item.parent_id);
						}
						self.updateTotals();
					}
					self.paint();
				}
			}
			var categorySelect = this.createSelect(options.order, options.values, item.category, attrs, 'name');
			categoryCell.appendChild(categorySelect);
			this.addElement([item.id, 'category'], categorySelect);

			var quantityCell = createElement2('div', {className: 'column expense-col-amount'}, null, parent);

			if (!this.readOnly) {
				var name = 'pricing-' + item.id;
				var id = name + '-pricing';
				var wrapper = createElement2('div', {className: 'input-wrapper input-block', id: id});
				var qtyInput = createElement2('input', {type: 'hidden', name: name + '-qty'}, null, wrapper);
				var rateInput = createElement2('input', {type: 'hidden', name: name + '-rate'}, null, wrapper);
				var taxInput = createElement2('input', {type: 'hidden', name: name + '-tax'}, null, wrapper);
				var inputDiv = createElement2('div', {className: 'input-field input-field-group rbullet input-block'}, null, wrapper);
				var labelDiv = createElement2('div', {className: 'input-label'}, null, inputDiv);
				quantityCell.appendChild(wrapper);

				var useQty = this.hasUnits(item);

				var onchange = function(qty, rate, tax, description) {
					item.tax_class_id = tax;
					item.description = description;
					if (useQty) {
						item.quantity = parseFloat(qty) || 0;
						item.paid_rate = parseFloat(rate) || 0;
						item.amount = item.paid_rate * item.quantity;
					} else {
						item.amount = parseFloat(rate) || 0;
					}
					self.recalcTaxes(item);
					if (item.parent_id) {
						self.recalculateSplit(item.parent_id);
					}
					self.updateTotals();
					self.paint();
				}
				var input = new SUGAR.ui.ExpensePricingInput(id, {
					description: item.description, name: name, qty: item.quantity,
					rate: useQty ? item.paid_rate : item.amount, tax: item.tax_class_id || "-99",
					onchange: onchange,
					tax_label: mod_string('LBL_ITEM_TAX'),
					qty_label : mod_string('LBL_ITEM_QUANTITY'),
					description_label : mod_string('LBL_ITEM_DESCRIPTION'),
					rate_label: mod_string(useQty ? 'LBL_ITEM_RATE' : 'LBL_ITEM_AMOUNT'),
					disable_qty: !useQty || item.split,
					disable_rate: item.split,
					disable_tax: item.split,
					unit: this.unitName(item),
					input_elt: inputDiv,
					get_label: this.getExpenseDescription(item).bind(this)
				});
				SUGAR.ui.registerInput(this.form, input);
				input.setup();
			} else {
				Wrap(quantityCell).append(this.getExpenseDescription(item).bind(this)());
			}

			var cell = createElement2('div', {className: 'column expense-col-actions'}, null, parent);
			if (item.attachmentSrc || item.hasNewAttachment) {
				button = this.createIconLink('extern', '');
				if (item.hasNewAttachment)  {
					var a = createElement2('span');
					button.style.color = '#CCC';
				} else {
					var a = createElement2('a', {href: item.attachmentSrc, target: '_blank'});
				}
				a.appendChild(button);
				cell.appendChild(a);
			}
			if (!this.readOnly) {

				if (item.split || !item.parent_id) {

					button = this.createIconLink('paperclip', '');
					if (item.attachmentSrc || item.hasNewAttachment) {
						button.style.color = 'red';
						button.title = mod_string('LBL_REMOVE_ATTACHMENT', 'ExpenseReports');
					} else {
						button.title = mod_string('LBL_ADD_ATTACHMENT', 'ExpenseReports');
					}
					button.onclick = function()
					{
						if (item.hasNewAttachment || item.attachmentSrc) {
							if (confirm(mod_string('LBL_CONFIRM_REMOVE_ATTACHMENT', 'ExpenseReports'))) {
								self.removeAttachment(item);
							}
						} else {
							$('upload_' + item.id).click();
						}
						return false;
					};
					var fileInput = $('upload_' + item.id);
					if (!fileInput) {
						fileInput = createElement2('input', {type:'file', id: 'upload_' + item.id, name: 'upload_' + item.id, value: item.uploadSrc});
						fileInput.onchange = function() {
							if (self.html5) {
								button.title = this.value;
							} else {
								this.title = this.value;
							}
							item.hasNewAttachment = true;
							self.paint();
						};
						if (!this.html5) {
							fileInput.style.position = 'relative';
							fileInput.style.width = '20px';
							fileInput.style.right = '20px';
							fileInput.style.zIndex = 2;
							fileInput.style.filter = 'alpha(opacity: 0)';
							fileInput.style.opacity = 0;
							fileInput.style.cursor = 'pointer';
						} else {
							fileInput.setAttribute('style', "visibility:hidden;position:absolute;top:-50;left:-50");
							this.form.appendChild(fileInput);
						}
						this.fileInputs[item.id] = fileInput;
					}
					cell.appendChild(button);
					if (!this.html5) {
						var container2 = createElement2('div', {style: {position: 'relative'}});
						container2.style.width='19px';
						container2.style.height='19px';
						container2.style.overflow = 'hidden';
						container2.appendChild(fileInput);
						cell.appendChild(container2);
					}
				}

				button = this.createIconLink(
					'remove', '',
					function() {
						self.removeItem(item.id, item.parent_id);
						self.paint();
						return false;
					}
				);
				cell.appendChild(button);
			}
		},

		removeAttachment : function(item) {
			item.attachmentSrc = null;
			item.hasNewAttachment = false;
			item.removeOriginalAttachment = true;
			this.paint();
		},

		recalcTaxes : function(item) {
			var rates = [];
			var tid = item.tax_class_id;
			if (tid) {
				if (SysData.taxcodes[tid]) {
					rates = SysData.taxcodes[tid].rates;
				}
			}
			item.total = 0.0;
			var base = parseFloat(item.amount);
			for (var i = 0; i < rates.length; i++) {
				item.total += base * rates[i].rate / 100;
			}
			item.total += base;
		},

		addElement : function(id, elt) {
			if(typeof(id) == 'object')
				id = id.join(':');
			this.elements[id] = elt;
		},

		getElement : function(id) {
			if(typeof(id) == 'object')
				id = id.join(':');
			if(this.elements[id])
				return this.elements[id];
			return null;
		},

		createSelect : function(values, labels, selected, attrs, labelKey) {
			if(! attrs) attrs = {};
			if(! blank(attrs.readOnly)) {
				delete attrs.readOnly;
				attrs.disabled = true;
			}
			if(this.readOnly)
				attrs.disabled = true;

			var outer = createElement2('span', {className: 'input-field input-block input-field-group rbullet'});
			var opts = {keys: values, values: labels, display_name: labelKey || 'name'};
			var select_opts = new SUGAR.ui.SelectOptions(opts);
			if(attrs.options_width) select_opts.width = attrs.options_width;
			attrs.options = select_opts;

			var lbl = labels[selected];
			if (! isset(lbl)) lbl = '--';
			else if(isPlainObject(lbl)) lbl = lbl[opts.display_name];

			lbl = createElement2('span', {className: "input-label"}, lbl);
			outer.appendChild(lbl);

			if(! attrs.disabled) {
				var quicksel = new SUGAR.ui.QuickSelect(attrs);
				quicksel.initSource(outer, selected, attrs.onchange);
			} else {
				outer.className += ' disabled';
			}

			return outer;
		},

		checkAmount : function(value) {
			var ret = parseFloat(value);
			if (isNaN(ret)) ret = 0;
			return ret;
		},

		recalculateSplit : function(topId) {
			var amount = 0.0;
			var total = 0.0;
			for (var cId in this.topItems[topId].children) {
				var child = this.topItems[topId].children[cId];
				amount += parseFloat(child.amount);
				total += parseFloat(child.total);
			}
			this.topItems[topId].amount = amount;
			this.topItems[topId].total = total;

			this.updateTotals();
		},

		updateTotals : function() {
			var total = 0.00;
			var amount = 0.00;
			for (var iId in this.topItems) {
				amount += parseFloat(this.topItems[iId].amount);
				total += parseFloat(this.topItems[iId].total);
			}
			var taxes = this.roundNumber(total - amount);
			$(this.id + '-tax').value = this.formatCurrency(taxes);
			$('tax_display').value = '';
			SUGAR.ui.getFormInput(this.form, 'total_pretax').setValue(amount);
			SUGAR.ui.getFormInput(this.form, 'total_amount').setValue(total);
			$('tax_display').value += '' + this.formatCurrency(taxes);

			try {
				var totalUSD = total / this.currencyRate();
				var adv_inp = SUGAR.ui.getFormInput(this.form, 'advance');
				var advance = 0;
				if (adv_inp)
					advance = adv_inp.getValue(true);
				var advanceUSD = advance / this.advanceCurrencyRate();
				var balanceUSD = totalUSD - advanceUSD;

				var balance = balanceUSD * this.currencyRate();
				SUGAR.ui.getFormInput(this.form, 'balance').setValue(balance);
			} catch (e) {
				console.error(e);
			}
		},

		moveDown : function(iId, parentId) {
			var order;
			if (parentId) {
				order = this.topItems[parentId].itemsOrder;
			} else {
				order = this.itemsOrder;
			}
			for (var i = 0; i < order.length; i++) {
				if (order[i] == iId) {
					if (i < order.length - 1) {
						order[i] = order[i+1];
						order[i+1] = iId;
						this.paint();
					}
				}
			}
		},

		moveUp : function(iId, parentId) {
			var order;
			if (parentId) {
				order = this.topItems[parentId].itemsOrder;
			} else {
				order = this.itemsOrder;
			}
			for (var i = 0; i < order.length; i++) {
				if (order[i] == iId) {
					if (i > 0) {
						order[i] = order[i-1];
						order[i-1] = iId;
						this.paint();
					}
				}
			}
		},

		removeItem : function(iId, parentId) {
			var order;
			if (parentId) {
				order = this.topItems[parentId].itemsOrder;
			} else {
				order = this.itemsOrder;
			}
			for (var i = 0; i < order.length; i++) {
				if (order[i] == iId) {
					order.splice(i, 1);
					delete(this.allItems[iId]);
					if (parentId) {
						delete(this.allItems[parentId].children[iId]);
						if (order.length == 0) {
							this.allItems[parentId].split = 0;
						}
					} else {
						delete(this.topItems[iId]);
					}
					if (parentId) {
						this.recalculateSplit(parentId);
					}
					this.paint();
				}
			}
			if (this.fileInputs[iId]) {
				this.form.removeChild(this.fileInputs[iId]);
				this.fileInputs[iId] = null;
				delete this.fileInputs[iId];
			}
		},

		getItemsJSON : function() {
			var order = 1;
			var ret = [];
			var rate = this.currencyRate();
			for (var i = 0; i < this.itemsOrder.length; i++) {
				var item = this.topItems[this.itemsOrder[i]];
				var copy = deep_clone(item);
				copy.line_order = order++;
				delete(copy.children);
				delete(copy.itemsOrder);
				copy.total_usdollar = copy.total / rate;
				copy.amount_usdollar = copy.amount / rate;
				copy.paid_rate_usd = copy.paid_rate / rate;
				ret.push(copy);
				for (var j = 0; j < item.itemsOrder.length; j++) {
					var child = deep_clone(item.children[item.itemsOrder[j]]);
					child.line_order = order++;
					child['date'] = item['date'];
					delete child.children;
					delete child.itemsOrder;
					child.amount_usdollar = child.amount / rate;
					child.paid_rate_usd = child.paid_rate / rate;
					ret.push(child);
				}
			}
			return JSON.stringify(ret);
		},

		createInput : function(attrs) {
			if (typeof(attrs) != 'object') {
				attrs = {};
			}
			if (this.readOnly) {
				attrs.readOnly = 'readOnly';
			}
			var input = document.createElement('input');
			setAttrs(input, attrs);
			return input;

		},

		createTextInput : function(attrs) {
			if(typeof(attrs) != 'object')  attrs = {};
			if (this.readOnly)
				attrs.readOnly = true;
			var input;
			if(isset(attrs.rows))
				input = new SUGAR.ui.TextAreaInput(null, attrs);
			else
				input = new SUGAR.ui.TextInput(null, attrs);
			SUGAR.ui.registerInput(this.form, input);
			return input.render();
		},

		updateExchangeRate : function (old_rate, new_rate) {
			for (var i = 0; i < this.itemsOrder.length; i++) {
				var item = this.topItems[this.itemsOrder[i]];
				if (this.hasUnits(item) && !item.split) {
					item.paid_rate = item.paid_rate / old_rate * new_rate;
					item.amount = item.paid_rate * item.quantity;
				} else {
					item.amount = item.amount / old_rate * new_rate;
				}
				for (var j = 0; j < item.itemsOrder.length; j++) {
					var child = item.children[item.itemsOrder[j]];
					if (this.hasUnits(child)) {
						child.paid_rate = child.paid_rate / old_rate * new_rate;
						child.amount = child.paid_rate * child.quantity;
					} else {
						child.amount = child.amount / old_rate * new_rate;
					}
					this.recalcTaxes(child);
				}
				this.recalcTaxes(item);
			}
			this.updateTotals();
			this.paint();
		},

		updateAdvanceExchangeRate : function (old_rate, new_rate) {
			/*var form = document.DetailForm;
			var adv_inp = document.DetailForm.advance;
			var numVal = validateNumberInput(adv_inp);*/
			this.updateTotals();
			this.paint();
		},

		updateAdvanceAmount : function () {
			//var adv_inp = document.DetailForm.advance;
			//var numVal = validateNumberInput(adv_inp);
			this.updateTotals();
			this.paint();
		},

		isImage : function (src) {
			return src.match(/\.(gif|jpg|jpeg|png)$/i);
		},

		isPDF : function (src) {
			return src.match(/\.pdf$/i);
		},

		hasUnits : function(item) {
			if (!this.Categories.values[item.category]) return false;
			return !blank(this.Categories.values[item.category].expenses_unit);
		},

		unitName : function(item) {
			return SUGAR.language.get('app_list_strings', 'expense_units_dom')[item.unit];
		},

		paidRate : function(category) {
			if (!this.Categories.values[category]) return "";
			return this.Categories.values[category].paid_rate_usd;
		},

		formatCurrency : function(val, noRound) {
			if(! isset(val) || val === '') return '';
			var decimals = this.currencyDecimalPlaces();
			if(noRound)
				return stdFormatNumber(restrictDecimals(parseFloat(val), decimals));
			return stdFormatNumber(parseFloat(val * 100).toFixed(decimals) / 100, decimals, decimals);
		},

		roundNumber : function(val) {
			var decimals = this.currencyDecimalPlaces();
			return parseFloat(val * 100).toFixed(decimals) / 100;
		},

		currencyDecimalPlaces : function(currency_id) {
			return this.currency_input.getDecimals(currency_id);
		},

		currencyRate : function(currency_id) {
			return this.currency_input.getRate(true, currency_id);
		},

		advanceCurrencyRate : function(currency_id) {
			return this.advance_currency.getRate(true, currency_id);
		},

		beforeSubmitForm : function(form) {
			if (form) {
				if ($('tax'))
					$('tax').value = stdUnformatNumber($('tax').value);
				var currency_fields = ['balance', 'total_amount', 'total_pretax'];
				var field = '';
				var field_input = null;

				if(! this.readOnly) {
					var input = form[this.id + '-line_items'];
					if(! input) input = createElement2('input', {type: 'hidden', name: this.id + '-line_items'}, null, form);
					input.value = this.getItemsJSON();
				}
			}
		},

		createIconLink: function(icon, text, onclick, attrs, size) {
			if(typeof(attrs) != 'object')  attrs = {};
			if(! size) size = 'uii-lg';
			var elt = MakeElt('span.uii.active-icon.uii-' + icon, attrs);
			elt.addClass(size);
			if(onclick) elt.on('click', onclick);
			if(text) elt.append(MakeElt('label', ' ' + mod_string(text, 'ExpenseReports')));
			return elt.get();
		}
	}
	//extendProto(UI.ExpenseEditor, SUGAR.ui.DragHandler);


    UI.ExpensePricingInput = function(id, params) {
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		this.decimals = 2;
		this.invalid = false;
		if(params) extendObject(this, params);
	}
	UI.ExpensePricingInput.prototype = {
		setup: function() {
			if(this.set_up)
				return;
			if(! this.elt)
				this.elt = $(this.id);
			if(! this.elt)
				return;
			var basename = this.name;
			if(this.form && basename) {
				if(! this.input_elt)
					this.input_elt = Wrap(this.elt).findFirst('.input-field').get(0);
			}
			if (!window.SysData || !window.SysData.taxcodes_order || !window.SysData.taxcodes) {
				this.disable_tax = false;
			}
			this.addHooks();
			this.updateDisplay();
			this.set_up = true;
		},
		addHooks: function() {
			if(! this.input_elt) return;
			Wrap(this.input_elt)
				.on('mousedown', '.input-label', function(evt) { evt.preventDefault(); /* prevent click issues */})
				.on('mousedown', this.handleMouseDown, {scope: this})
				.on('click', this.handleClick, {scope: this})
				.on('keydown', this.handleKeyDown, {scope: this})
				.on('focus', this.handleFocus, {scope: this})
				.on('blur', this.handleBlur, {scope: this});
		},
		getDescriptionValue: function() {
			return this.description || '';
		},
		getQtyValue: function() {
			return this.qty;
		},
		getRateValue: function() {
			return this.rate;
		},
		focus: function() {
			this.input_elt.focus();
		},
		handleKeyDown: function(evt) {
			if(evt.keyCode == 13) {
				if(this.opened) {
					this.acceptInput();
				} else if(! this.disabled)
					this.showPopup();
				UI.stopEvent(evt);
			} else if(evt.keyCode == 27) {
				// esc
				if(this.opened) {
					this.hidePopup();
					this.focus();
					UI.stopEvent(evt);
				}
			} else if(evt.keyCode == 32) {
				if(! this.opened && ! this.disabled) {
					this.showPopup();
					UI.stopEvent(evt);
				}
			}
		},
		handleMouseDown: function(evt) {
			if(! this.disabled) {
				if(evt.currentTarget === this.input_elt)
					evt.stopPropagation();
			}
		},
		handleClick: function(evt) {
			if(evt.which == 1) {
				if(this.opened)
					this.hidePopup();
				else if(! this.disabled)
					this.showPopup();
				UI.stopEvent(evt);
			}
		},
		handleFocus: function(evt) {
			this.focused = true;
			this.updateDisplay();
		},
		handleBlur: function(evt) {
			this.focused = false;
			this.updateDisplay();
		},

		returnValue: function(name, val) {
			if(name === 'qty') {
				this.qty_next = val;
			} else if(name === 'rate') {
				this.rate_next = val;
			} else if(name === 'tax') {
				this.tax_next = val;
			} else if(name === 'description') {
				this.description_next = val;
			}
		},
		acceptInput: function(noclose) {
			if(this.opened) {
				this.description = get_default(this.description_next, this.descriptioni||'');
				this.qty = get_default(this.qty_next, this.qty);
				this.rate = get_default(this.rate_next, this.rate);
				this.tax = get_default(this.tax_next, this.tax);
				if (!noclose) {
					this.hidePopup();
					this.updateDisplay();
					this.focus();
				}
				UI.callEvent(this, this.onchange, this.qty, this.rate, this.tax, this.description);
			}
		},
		clear: function() {
			this.rate = '';
			this.tax = '';
			this.qty = '';
			this.description = '';
			if(this.opened) {
				this.hidePopup();
				this.focus();
			}
			this.updateDisplay();
			UI.callEvent(this, this.onchange, this.qty, this.rate, this.tax, this.description);
		},
		getPopupContent: function() {
			var self = this;
			if(! this.qty_input && !this.disable_qty) {
				this.qty_input = new UI.TextInput(this.id + '-qval',
					{format: 'float', min_decimals: 0, decimals: this.decimals, tabIndex: 0,
					 init_value: this.qty});
				this.qty_input.render();
				this.qty_input.onchange = function() {
					self.returnValue('qty', this.getValue(this.field.value));
				}
				this.qty_input.onreturn = function() {
					self.acceptInput();
					self.updateDisplay();
				}
			}
			if(! this.rate_input && !this.disable_rate) {
				this.rate_input = new UI.TextInput(this.id + '-rval',
					{format: 'float', min_decimals: 0, decimals: this.decimals, tabIndex: 0,
					 init_value: this.rate, size: 12});
				this.rate_input.render();
				this.rate_input.onchange = function() {
					self.returnValue('rate', this.getValue(this.field.value));
				}
				this.rate_input.onreturn = function() {
					self.acceptInput();
					self.updateDisplay();
				}
			}
			if (!this.tax_input && !this.disable_tax) {
				var attrs = {no_scroll: true, popup: true, show_stacked: true, init_value: this.tax};
				if(this.readOnly)
					attrs.disabled = true;
				var outer = createElement2('div', {className: 'input-field rbullet'});
				var opts = {keys: SysData.taxcodes_order, values: SysData.taxcodes, display_name: 'name'};
				var options = new UI.SelectOptions(opts);

				attrs.options = new UI.SelectList(this.id + '-tval-popup', {
					options: options, popup: true, show_checks: false});
				attrs.onchange = function(key, value) {
					self.returnValue('tax', key);
					self.acceptInput(true);
					self.updateDisplay();
				}
				this.tax_input = new UI.SelectInput(this.id + '-tval', attrs);
				this.tax_input.setup();
				this.tax_input.render();
			}
			if(! this.description_input && !this.disable_description) {
				this.description_input = new UI.TextAreaInput(this.id + '-dval',
					{tabIndex: 0,
					 cols: 40,
					 init_value: this.description || ''});
				this.description_input.render();
				this.description_input.onchange = function() {
					self.returnValue('description', this.getValue());
				}
				this.description_input.onreturn = function() {
					self.acceptInput();
					self.updateDisplay();
				}
			}
			if(! this.value_label) {
				this.value_label = MakeElt('h5.form-label.no-margin');
				this.tools_row = MakeElt('div.row.tools-row')
					.append(this.value_label)
					.append(MakeElt('div.uii.uii-lg.active-icon.uii-remove', {tabindex: 0})
						.on('click, keydown[key=enter|space]', function() {
							self.clear();
						}))
					.append(MakeElt('div.uii.uii-lg.active-icon.uii-accept', {tabindex: 0})
						.on('click, keydown[key=enter|space]', function() {
							self.acceptInput();
						}));
			}
			this.header_div = MakeElt('div.panel-header.card-header')
				.append(this.tools_row);
			this.value_div = MakeElt('div.card-body.panel-body');
			if (!this.disable_qty) {
				var unitLabel = '';
				if (this.unit) {
					unitLabel = ' (' + this.unit + ')';
				}
				this.value_div
					.append(MakeElt('div.row.form-row').append(MakeElt('div.form-cell.column.sm-12').append(MakeElt('div.form-entry')
						.append([
							MakeElt('div.form-label', this.qty_label + unitLabel),
							MakeElt('div.form-value', this.qty_input.field)]))));
			}
			if (!this.disable_rate || ! this.disable_tax) {
				var row = MakeElt('div.row.form-row');
				if (!this.disable_rate) {
					row.append(MakeElt('div.form-cell.column.sm-6').append(MakeElt('div.form-entry')
						.append([
							MakeElt('div.form-label', this.rate_label),
							MakeElt('div.form-value', this.rate_input.field)])));
				}
				if (!this.disable_tax) {
					row.append(MakeElt('div.form-cell.column.sm-6').append(MakeElt('div.form-entry')
						.append([
							MakeElt('div.form-label', this.tax_label),
							MakeElt('div.form-value', this.tax_input.elt)])));
				}
				this.value_div.append(row);
			}
			if (!this.disable_description) {
				this.value_div
					.append(MakeElt('div.row.form-row').append(MakeElt('div.form-cell.column.sm-12').append(MakeElt('div.form-entry')
						.append([
							MakeElt('div.form-label', this.description_label),
							MakeElt('div.form-value', this.description_input.field)]))));
			}
			return [this.header_div, this.value_div];
		},
		showPopup: function() {
			var self = this,
				content = this.getPopupContent(),
				p = new UI.PopupSlidePanel(this.id+'-popup',
					{target: this.input_elt, target_offset: {top: -1},
					 destroy_on_close: true,
					 min_width: 350});
			p.onshow = function() {
				self.opened = true;
				self.updateDisplay();
				if (self.qty_input)
					self.qty_input.select();
				else if (self.rate_input)
					self.rate_input.select();
				else if (self.description_input)
					self.description_input.select();
			}
			p.onclose = function() {
				self.opened = false;
				self.popup = null;
				self.updateDisplay();
			}
			p.render();
			Wrap(p.elt).content(content);
			if (this.qty_input) {
				this.qty_input.setup();
				this.qty_input.setValue(this.qty_next = this.getQtyValue(), true);
			}
			if (this.rate_input) {
				this.rate_input.setup();
				this.rate_input.setValue(this.rate_next = this.getRateValue(), true);
			}
			if (this.description_input) {
				this.description_input.setup();
				this.description_input.setValue(this.description_next = this.getDescriptionValue(), true);
			}
			this.popup = p;
			p.show();
		},
		hidePopup: function() {
			if(this.popup) this.popup.close();
		},
		updateDisplay: function() {
			UI.addRemoveClass(this.input_elt, 'active', !! (this.focused || this.opened));
			UI.addRemoveClass(this.input_elt, 'opened', !! this.opened);
			UI.addRemoveClass(this.input_elt, 'invalid', this.invalid);
			var val, lbl,
				label_elt = Wrap(this.elt).findFirst('.input-label').ext(0);
			if(label_elt) {
				if (this.get_label) {
					Wrap(label_elt)
						.content(
							this.get_label(this)
						);
				}
			}
		}

	};

});



function createTextLabel(lbl, no_translate) {
	if(blank(no_translate))
		lbl = mod_string(lbl);
	var elt = document.createTextNode(lbl);
	return elt;
}
