<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


require_once 'modules/Forecasts/Forecast.php';	
require_once 'modules/Forecasts/Forms.php';	

class ForecastCalculator
{
	var $total		= 0;
    var $weighted	= 0;
    var $best_case	= 0;
    var $commit		= 0;
    var $forecast	= 0;
	var $quota		= 0;
	var $actual		= 0;
	
    var $period;
	var $users;
	
	var $filled_ids = array();
	
	function __construct($period, $users = null)
	{
		static $all_users;
		$this->period = $period;
		if($users == null) {
			if(!isset($all_users))
				$all_users = $this->get_all_users();
			$users =& $all_users;
		}
		$this->users =& $users;
	}
	
	function get_all_users() {
		global $db;
		$lq = new ListQuery('User', ['id', 'user_name', 'reports_to_id', 'status']);
		$lq->addSimpleFilter('portal_only', 0);
		$users = [];
		foreach ($lq->getIterator() as $user) {
			$users[$user->getField('id')] = $user->row;
		}

		// we need to sort the users in such a way that when iterating the array, 
		// we move up the reports tree. That is a requirement for filling roll-up 
		// forecasts - lower levels must be filled first. To do so, we first sort
		// in reverse order, and then reverse the array

		$guard = 1000;
		$sorted = [];
		$bosses = ['' => true];
		while (count($users)) {
			$guard--;
			if (!$guard) break;
			$add = [];
			foreach ($users as $id => $u) {
				if (!empty($bosses[$u['reports_to_id']])) {
					$sorted[$id] = $u;
					unset($users[$id]);
					$add[$id] = true;
				}
			}
			$bosses += $add;
		}
		$sorted_keys = array_reverse(array_keys($sorted));
		$ret = [];
		foreach ($sorted_keys as $k) {
			$ret[$k] = $sorted[$k];
		}
		return $ret;
	}

	function which_totals($category)
	{
		static $map = array(
			'Actual' => 'actual',
			'Commit' => 'commit',
			'Best Case' => 'best_case',
			'Pipeline' => 'total',
		);
		static $hierarchy = array(
			'total',
			'best_case',
			'commit',
			'actual',
		);
		if (!isset($map[$category])) return array();
		$ret = array();
		$stop_on = $map[$category];
		foreach ($hierarchy as $total_name) {
			$ret[] = $total_name;
			if ($total_name == $stop_on) {
				break;
			}
		}
		return $ret;
	}	

	function fill_my($opp)
	{
		$totals = $this->which_totals($opp->getField('forecast_category'));
		foreach ($totals as $total) {
			$this->$total 	+= $opp->getField('amount_usdollar');
		}
		$this->weighted += $opp->getField('amount_usdollar') * $opp->getField('probability') / 100;
	}
	
	function getMonthlyData($offset, $fiscal_year_start, &$start_date, &$end_date, &$forecast_name)
	{
		$start_date = gmdate("Y-m-d", gmmktime(0, 0, 0, date('n') + $offset, 1));
		$end_date = gmdate("Y-m-d", gmmktime(0, 0, 0, date('n') + $offset + 1, 0));
		$forecast_name = gmdate('M Y', gmmktime(0, 0, 0, date('n') + $offset, 1));
	}
	
	function getQuarterlyData($offset, $fiscal_year_start, &$start_date, &$end_date, &$forecast_name)
	{
		$time = gmmktime(0, 0, 0, date('n') + $offset * 3);
		$month = date('n', $time);
		$year = date('Y', $time);
		$start_month = $fiscal_year_start + floor(($month - $fiscal_year_start) / 3) * 3;
		if (0 > $start_month)
			$start_month = 12 + $start_month;
		$start_date = gmdate("Y-m-d", gmmktime(0, 0, 0, $start_month, 1, $year));
		$end_date = gmdate("Y-m-d", gmmktime(0, 0, 0, $start_month + 3, 0, $year));
		if ($fiscal_year_start <= $start_month) {
			$quart_number = floor(($start_month - $fiscal_year_start) / 3) + 1;
			$fy_year = $year;
		} else {
			$quart_number = floor(($start_month + 12 - $fiscal_year_start) / 3) + 1;
			$fy_year = $year - 1;
		}
		$forecast_name = 'FY' . $fy_year . '-Q' . $quart_number;
	}
	
	function getLatestDate($fiscal_year_start, $forecast_periods) {
		return self::get_latest_date($this->period, $fiscal_year_start, $forecast_periods);
	}

	function getRemoveDate($fiscal_year_start, $history_periods) {
		return self::get_remove_date($this->period, $fiscal_year_start, $history_periods);
	}
	
	static function get_latest_date($period, $fiscal_year_start, $forecast_periods)
	{
		switch ($period) {
			case 'Monthly':
				return gmdate("Y-m-d", gmmktime(0, 0, 0, date('n') + $forecast_periods, 2));
			case 'Quarterly':
				$month = date('n');
				$start_month = $fiscal_year_start + floor(($month - $fiscal_year_start) / 3) * 3;
				if (0 > $start_month)
					$start_month = 12 + $start_month;
				return gmdate("Y-m-d", gmmktime(0, 0, 0, $start_month + $forecast_periods * 3, 2));
			default:
				return;
		}
	}
	
	static function get_remove_date($period, $fiscal_year_start, $history_periods)
	{
		switch ($period) {
			case 'Monthly':
				return gmdate("Y-m-d", gmmktime(0, 0, 0, date('n') - $history_periods, 1));
			case 'Quarterly':
				$month = date('n');
				$start_month = $fiscal_year_start + floor(($month - $fiscal_year_start) / 3) * 3;
				if (0 > $start_month)
					$start_month = 12 + $start_month;
				return gmdate("Y-m-d", gmmktime(0, 0, 0, $start_month - $history_periods * 3, 1));
			default:
				return;
		}
	}
	
	function fill_personal_forecast(RowUpdate &$forecast, $period=null) {
		require_once 'modules/Opportunities/Opportunity.php';
		$info = Forecast::getPeriodInfo($period);

		//check user has opportunities in the period

		$lq = new ListQuery('Opportunity', true);
		$lq -> addSimpleFilter('date_closed', $info['start_date'], '>=');
		$lq -> addSimpleFilter('date_closed', $info['end_date'], '<=');
		$lq -> addSimpleFilter('assigned_user_id', $forecast->getField('user_id'));
		$lq -> addSimpleFilter('forecast_category', 'Omitted', '!=');

		$update = ['opportunities' => 0];
		$temp = new self($this->period, $this->users);
		foreach ($lq->getIterator() as $opp) {
			$temp->fill_my($opp);
			$update['opportunities']++;
		}
		
		$forecast->set(array(
			'opportunities' => $update['opportunities'],
			'total' => $temp->total,
			'weighted' => $temp->weighted,
			'best_case' => $temp->best_case,
			'commit' => $temp->commit,
			'actual' => $temp->actual,
			'type' => 'personal',
		));
	}
	
	function fill_team_individual(RowUpdate &$forecast, $team, $period)
	{
		$team_list = (is_countable($team) && count($team)) ? ("'" . implode("','", $team) . "'") : '';
		$clause = [
			'multiple' => [
				['field' => 'user_id', 'value' => $team],
				['field' => 'start_date', 'value' => $period],
				['field' => 'type', 'value' => 'personal'],
			],
		];
		
		if(($row = Forecast::getTotals($clause)) !== null) {
			$forecast->set(array(
				'opportunities' => $row['opportunities'],
				'total' => $row['total'],
				'weighted' => $row['weighted'],
				'best_case' => $row['best_case'],
				'commit' => $row['commit'],
				'actual' => $row['actual'],
				'forecast' => $row['forecast'],
				'quota' => $row['quota'],
			));
		}
		$forecast->set('type', 'team_individual');
	}
	
	function fill_team_rollup(RowUpdate &$forecast, $period)
	{
		$user_id = $forecast->getField('user_id');
		$clause = [
			'operator' => 'AND',
			'multiple' => [
				['field' => 'start_date', 'value' => $period],
				[
					'operator' => 'OR',
					'multiple' => [
						[
							'operator' => 'AND',
							'multiple' => [
								['field' => 'type', 'value' => 'personal'],
								['field' => 'user_id', 'value' => $user_id],
							],
						],
						[
							'operator' => 'AND',
							'multiple' => [
								['field' => 'user.reports_to_id', 'value' => $user_id],
								['field' => 'type', 'value' => 'team_individual'],
								['field' => 'user.status', 'value' => 'Active'],
							],
						],
					],
				],
			],
		];

		if(($row = Forecast::getTotals($clause)) !== null) {
			$forecast->set(array(
				'opportunities' => $row['opportunities'] ?? 0,
				'total' => $row['total'] ?? 0,
				'weighted' => $row['weighted'] ?? 0,
				'best_case' => $row['best_case'] ?? 0,
				'commit' => $row['commit'] ?? 0,
				'actual' => $row['actual'] ?? 0,
			));
			if($row['user_count'] == 1) {
				$forecast->set([
					'forecast' => $row['forecast'],
					'quota' => $row['quota'],
				]);
			}
		}
		$forecast->set('type', 'team_rollup');
	}

	
	function fill_forecasts($offset, $fiscal_year_start, $force = false)
	{
		global $db;
		
		$start_date = 0;
		$end_date = 0;
		$forecast_name = 0;
		
		$funcname = 'get' . $this->period . 'Data';
		
		$this->$funcname($offset, $fiscal_year_start, $start_date, $end_date, $forecast_name);
		
		if (!$force) {
			$result = $db->query("select count(id) from forecasts where start_date='" . PearDatabase::quote($start_date) . "' and deleted != 1");
			if ($row = $db->fetchByAssoc($result)) {
				if (0 < current($row))
					return;
			}
		}
		
		foreach ($this->users as $id => $user) {
			$focus = RowUpdate::blank_for_model('Forecast');

			$focus->set(array(
				'name' => $forecast_name,
				'user_id' => $id,
				'start_date' => $start_date,
				'end_date' => $end_date,
			));
			
			$this->fill_personal_forecast($focus, $start_date);
			
			$focus->save();
			unset($focus);
			// remove and archive old forecasts by user
		}

		foreach ($this->users as $id => $user) {
			$team = getTeamList($this->users, $id, true);
			$team[] = $id;
			
			$focus = RowUpdate::blank_for_model('Forecast');

			$focus->set(array(
				'name' => $forecast_name,
				'user_id' => $id,
				'start_date' => $start_date,
				'end_date' => $end_date,
			));
			
			$this->fill_team_individual($focus, $team, $start_date);
			
			$focus->save();
			unset($focus);
		}
		foreach ($this->users as $id => $user) {
			$focus = RowUpdate::blank_for_model('Forecast');
			
			$focus->set(array(
				'name' => $forecast_name,
				'user_id' => $id,
				'start_date' => $start_date,
				'end_date' => $end_date,
			));
			
			$this->fill_team_rollup($focus, $start_date);
			
			$focus->save();
			unset($focus);
		}
	}
	
	function update_user_team_forecasts($user_id, $period_id=null) {
		global $db;
		
		if($period_id !== null && ! Forecast::retrieve_by_period($user_id, $period_id))
			return;
		$updateFor = getReportsToList($this->users, $user_id);

		$result = null;
		if (!empty($updateFor)) {
			$lq = new ListQuery('Forecast', true);
			$lq->addSimpleFilter('user_id', $updateFor);
			if($period_id)
				$lq->addSimpleFilter('start_date', $period_id);
			$lq->addSimpleFilter('type', array('team_individual', 'team_rollup'));
			$lq->setOrderBy('type');
			$result = $lq->fetchAll();
		}
		
		$teams = array();

		if($result && ! $result->failed) {
			foreach ($updateFor as $curUserId) {
				foreach($result->getRowIndexes() as $idx) {
					$base = $result->getRowResult($idx);
					$focus = RowUpdate::for_result($base);
					$uid = $focus->getField('user_id');
					if ($uid != $curUserId) continue;
					$period = $focus->getField('start_date');
					if ($focus->getField('type') == 'team_individual') {
						if (!isset($teams[$uid])) {
							$teams[$uid] = getTeamList($this->users, $uid, true);
							if(! in_array($uid, $teams[$uid]))
								$teams[$uid][] = $uid;
						}
						$this->fill_team_individual($focus, $teams[$uid], $period);
					} else {
						$this->fill_team_rollup($focus, $period);
					}
					$focus->save();
					unset($focus);
				}
			}
		}
	}
	
	function cleanup() {
		if(isset($this->users)) {
			foreach($this->users as $u) {
				if($u instanceof SugarBean)
					$u->cleanup();
			}
			unset($this->users);
		}
	}

}

?>
