<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');

require_once('include/Dashlets/Dashlet.php');
require_once('include/Sugar_Smarty.php');
require_once('modules/Home/Dashlets/WeatherDashlet/WeatherDashletConfigView.php');

class WeatherDashlet extends Dashlet {

	var $dashletIcon = 'Weather';

	var $cities;

	var $cityDisplayData;

	var $degreesUnits;
	var $windUnits;

	var $mostRecent;

	var $showTimes = 1;

	const CITIES_NUM = 10;

	const YAHOO_QUERY_URL = "https://weather-ydn-yql.media.yahoo.com/forecastrss";
	const OPENMAPS_URL = "http://api.openweathermap.org/data/3.0/onecall";
	const OPENMAPS_URL_CUR = "http://api.openweathermap.org/data/2.5/weather";

	const UNLIMITED_VISIBILITY = "999";

	const DEFAULT_DEGREES_UNITS = "f";
	const DEFAULT_WIND_UNITS = "mph";

	const FAHRENHEIT = "f";

	const CELSIUS = "c";

	const MPH = 'mph';
	const KMPH = 'kmph';
	const MPS = 'mps';

	const DEFAULT_CACHE_EXPIRY = 7200; // 2 hours

	const REQUEST_TIMEOUT = 20; // seconds

    /**
     * Constructor
     *
     * @global string current language
     * @param string $id id for the current dashlet (assigned from Home module)
     * @param array $def options saved for this dashlet
     */
    function __construct($id, $def=null) {
        parent::__construct($id, $def); // call parent constructor

        $this->loadLanguage('WeatherDashlet'); // load the language strings here

        $this->isConfigurable = true; // dashlet is configurable
        $this->hasScript = true;  // dashlet has javascript attached to it

        // if no custom settings, use default
        if(empty($def['title'])) {
        	$this->title = $this->dashletStrings['LBL_TITLE'];
        } else {
        	$this->title = $def['title'];
        }

        if (isset($def['auto_refresh_time']))
            $this->autoRefreshTime = $def['auto_refresh_time'];

        if (!empty($def['cities'])) {
			$this->cities = $def['cities'];
		} else {
			$this->cities = $this->getDefaultCities();
		}

		if(isset($def['show_times']))
			$this->showTimes = $def['show_times'];

		if (!empty($def['degrees_units'])) {
			$this->degreesUnits = $def['degrees_units'];
			// sanitize, mainly for cache filename
			if($this->degreesUnits != self::FAHRENHEIT && $this->degreesUnits != self::CELSIUS)
				$this->degreesUnits = self::DEFAULT_DEGREES_UNITS;
		} else {
			$this->degreesUnits = self::DEFAULT_DEGREES_UNITS;
		}

		if (!empty($def['wind_units'])) {
			$this->windUnits = $def['wind_units'];
			if($this->windUnits != self::MPH && $this->windUnits != self::KMPH && $this->windUnits != self::MPS)
				$this->windUnits = self::DEFAULT_WIND_UNITS;
		} else {
			$this->windUnits = self::DEFAULT_WIND_UNITS;
		}
    }

    function process() {
    	global $pageInstance, $timedate;
    	$pageInstance->add_js_include('modules/Home/Dashlets/WeatherDashlet/WeatherDashlet.js');
    	$this->cityDisplayData = array();

		$missing = array();

		foreach($this->cities as $city) {
			$woeid = $city["woeid"];

			if(! empty($city['timezone'])) {
				$city['current_time'] = date($timedate->get_time_format(), strtotime($timedate->get_gmt_db_datetime()) + intval($city['timezone']));
			} else
				$city['current_time'] = '--';


			$this->cityDisplayData[$woeid] = $city;

			$weather = $this->getWeather($woeid);
			if($weather === false) {
				if(! $this->asyncDisplay)
					return 'pending'; // defer display
				$missing[] = $woeid;
			}
			else
				$this->cityDisplayData[$woeid]['weather_data'] = $this->formatWeather($weather);
		}

		if(count($missing)) {
			$this->fetchWeather($missing);
			return 'download';
		}
    }

    function displaySample($text='') {
    	global $pageInstance;
    	$pageInstance->add_js_include('modules/Home/Dashlets/WeatherDashlet/WeatherDashlet.js');
    	return parent::displaySample($text);
    }

    /**
     * Displays the dashlet
     *
     * @return string html to display dashlet
     */
	function displayBody() {
        $ss = new Sugar_Smarty();
        $ss->assign('id', $this->id);
		$ss->assign('LANG', $this->dashletStrings);

		global $theme, $odd_bg, $even_bg, $hilite_bg;
		require_once("themes/$theme/layout_utils.php");

		$ss->assign("citiesList", $this->cityDisplayData);
		$ss->assign("showTimes", $this->showTimes);

		$ss->assign('bgHilite', $hilite_bg);
		$ss->assign('rowColor', array('oddListRow', 'evenListRow'));
		$ss->assign('bgColor', array($odd_bg, $even_bg));

		$json = getJSONObj();
		$ss->inc_mgr->add_js_literal('Weather.init("'.$this->id.'", ' . $json->encode($this->cityDisplayData) . ');', null, LOAD_PRIORITY_FOOT);
        if ($this->autoRefreshTime > 0)
            $ss->inc_mgr->add_js_literal("SUGAR.sugarHome.initDashletAutoRefresh('{$this->id}', {$this->autoRefreshTime});", null, LOAD_PRIORITY_FOOT);

		$str = $ss->fetch(
			'modules/Home/Dashlets/WeatherDashlet/WeatherDashlet.tpl'
		);

		$ss->export_includes();

        return $str;
    }

    function getFooterText($mode=null) {
    	if($mode == 'sample') return;
		global $timedate;
		$dt = gmdate('Y-m-d H:i:s', isset($this->mostRecent) ? $this->mostRecent : time());
		$dt = $timedate->to_relative_date_time($dt);
		return str_replace('{time}', '<nobr>'.$dt.'</nobr>', $this->dashletStrings['LBL_LAST_UPDATE']);
    }

    function getCacheFilename($woeid) {
    	$key = (int)$woeid . $this->degreesUnits;
    	$cache_fname = CacheManager::get_location('weather/') . $key . '.txt';
    	return $cache_fname;
    }

    /**
	 * Get weather from local cache, reverting to fetchWeather if not found.
	 *
	 * @param string $woeid
	 * @return array
	 */
    function getWeather($woeid) {
		if($this->forceRefresh)
			return false;

    	$cache_fname = $this->getCacheFilename($woeid);
    	$now = time();
    	$ret = null;
    	if(file_exists($cache_fname)) {
    		$mtime = @filemtime($cache_fname);
    		if($now - $mtime < self::DEFAULT_CACHE_EXPIRY) {
    			$ret = unserialize(file_get_contents($cache_fname));
				if(! isset($this->mostRecent) || $this->mostRecent < $mtime)
					$this->mostRecent = $mtime;
    		}
    	}
    	if(empty($ret))
    		return false;
		return $ret;
    }

	function formatWeather($data) {
    	global $timedate;
    	if($data) {
			if($data['pressure'])
				$data['pressure'] = format_number($data['pressure'], -1) .
					' ' . $data['pressure_units'];
			else
				$data['pressure'] = '--';

			if($data['humidity'])
				$data['humidity'] .= '%';
			else
				$data['humidity'] = '--';

			if($data['visibility'] >= self::UNLIMITED_VISIBILITY)
				$data['visibility'] = $this->dashletStrings['LBL_UNLIMITED'];
			else
				$data['visibility'] = format_number(round($data['visibility'], 1), -1) .
					' ' . $data['visibility_units'];

			if($data['feels_like']) {
				if($data['feels_like'] == $data['temp'])
					$data['feels_like'] = '';
				else
					$data['feels_like'] = '( ' . $this->dashletStrings['LBL_FEELS_LIKE'] . ' ' . $data['feels_like'] . ')';
			}

			$wind_speed = $this->convertWindSpeed($data['wind_speed'], $data['wind_units'], $this->windUnits);

			$data['wind'] = format_number($wind_speed, -1) .
				' ' . $this->dashletStrings['LBL_WIND_' . strtoupper($this->windUnits)] . ' ' . $this->getWindDirection($data['wind_direction']);

			$data['sunrise'] = $timedate->to_db_time($data['sunrise']);
			$data['sunset'] = $timedate->to_db_time($data['sunset']);

			if(empty($data['image'])) {
				$data['image'] = 'include/images/blank.gif';
			}
		}
		return $data;
    }

	function convertWindSpeed($speed, $units, $targetUnits) {
		$ret = $speed;
		if($targetUnits == self::MPH && $units == 'km/h')
			$ret = $speed / 1.609344;
		else if($targetUnits == self::KMPH && $units != 'km/h')
			$ret = $speed * 1.609344;
		else if($targetUnits == self::MPS) {
			if($units == 'km/h')
				$ret = $speed * 1000 / 3600;
			else
				$ret = $speed * 1.609344 * 1000 / 3600;
		}
		if($ret != $speed)
			$ret = round($ret);
		return $ret;
	}

	function buildBaseString($baseURI, $method, $params) {
	    $r = array();
	    ksort($params);
	    foreach($params as $key => $value) {
	        $r[] = "$key=" . rawurlencode($value);
	    }
	    return $method . "&" . rawurlencode($baseURI) . '&' . rawurlencode(implode('&', $r));
	}

	function buildAuthorizationHeader($oauth) {
	    $r = 'Authorization: OAuth ';
	    $values = array();
	    foreach($oauth as $key=>$value) {
	        $values[] = "$key=\"" . rawurlencode($value) . "\"";
	    }
	    $r .= implode(', ', $values);
	    return $r;
	}

	/**
	 * Get weather from Yahoo weather
	 *
	 * @param array $woeids - to find your WOEID,
	 * browse or search for your city from the Yahoo Weather home page
     * @param bool $save_cache
	 * @return array
	 */
	function fetchWeather($woeids, $save_cache=true) {
		if($this->degreesUnits == 'c')
			$units = 'metric';
		else if($this->degreesUnits == 'f')
			$units = 'imperial';
		else
			$units = '';
		foreach($woeids as $woeid) {
			$requestUrl = self::OPENMAPS_URL_CUR."?id=$woeid&appid=".AppConfig::setting('company.openweather_key');

			require_once('include/utils/dl_utils.php');

			$req = new RemoteRequest($requestUrl);
			$resultString = $req->fetch();

			$proceed = true;
			if($resultString) {
				$json = getJSONobj();
				$jsonData = $json->decode($resultString);
			} else {
				$proceed = false;
			}
			if (!is_array($jsonData)) $proceed = false;
			if (isset($jsonData['cod']) && $jsonData['cod'] != 200) $proceed = false;

			if (!$proceed) {
				$this->cityDisplayData[$woeid]['weather_data'] = null;
				continue;
			}

			$query = array(
			    'lat' => $jsonData['coord']['lat'],
			    'lon' => $jsonData['coord']['lon'],
			    'mode' => 'json',
			    'units' => $units,
				'appid' => AppConfig::setting('company.openweather_key')
			);

			DLManager::add_download(array(
				'woeid' => $woeid,
				'url' => self::OPENMAPS_URL . '?' . http_build_query($query),
				'request_method' => 'GET',
				'save_cache' => $save_cache,
				'callback' => array(&$this, 'finishDownload'),
			));
		}
	}

    function finishDownload(&$dl) {
    	$woeid = $dl->woeid;
    	if(! $dl->failed) {
    		$dlData = $dl->getResponse();
			$arrData = $this->parseWeatherResult($woeid, $dlData);
			if($dl->save_cache) {
				$cache_fname = $this->getCacheFilename($woeid);
				if( ($fp = @fopen($cache_fname, 'w')) ) {
					fwrite($fp, serialize($arrData));
					fclose($fp);
				}
			}

			$this->cityDisplayData[$woeid]['weather_data'] = $this->formatWeather($arrData);
		} else {
			$this->cityDisplayData[$woeid]['weather_data'] = null;
		}
    }


	function parseWeatherResult($woeid, &$data) {
		$json = getJSONobj();
		$result = $json->decode($data);

		if($result && !empty($result['current'])) {

			$row = array();

			$row['woeid'] = $woeid;

			if($this->degreesUnits == 'c') {
				$unit = 'C';
				$unit_dist = 'Km';
				$vis_calc = 1000;
				$pressure_unit = 'Hpa';
			}
			else {
				$unit = 'F';
				$unit_dist = 'M';
				$vis_calc = 1609;
				$pressure_unit = 'mb';
			}

			if($this->windUnits == 'kmph') {
				$wind_calc = 3.6;
				$wind_unit = 'km/h';
			}
			else if($this->windUnits == 'mph') {
				$wind_calc = 2.237;
				$wind_unit = 'mph';
			} else {
				$wind_calc = 1;
				$wind_unit = 'mps';
			}

			global $timedate, $current_user;

			$row['text'] = $result['current']['weather'][0]['description'];
			$row['temp'] = round($result['current']['temp']) . '&deg;' . $unit;
			$row['feels_like'] = round($result['current']['feels_like']) . '&deg;' . $unit;
			$row['humidity'] = $result['current']['humidity'];
			$row['pressure_units'] = $pressure_unit;
			$row['pressure'] = $result['current']['pressure'];
			$row['visibility'] = $result['current']['visibility']/$vis_calc;
			$row['visibility_units'] = $unit_dist;
			$row['wind_speed'] = number_format($result['current']['wind_speed']*$wind_calc,2);
			$row['wind_units'] = $wind_unit;
			$row['wind_direction'] = $result['current']['wind_deg'];
			$row['image'] = 'http://openweathermap.org/img/w/'.$result['current']['weather'][0]['icon'].'.png';

			$useroffset = $timedate->getTimeZoneOffset($timedate->getUserTimeZone($current_user));

			$row['sunrise'] = date("Y-m-d H:i:s", $result['current']['sunrise']+$result['timezone_offset']);
			$row['sunset'] = date("Y-m-d H:i:s", $result['current']['sunset']+$result['timezone_offset']);
			$row['forecast1_day'] = date("Y-m-d", $result['daily'][0]['dt']);
			$row['forecast1_date'] = date("Y-m-d", $result['daily'][0]['dt']);
			$row['forecast1_low'] = round($result['daily'][0]['temp']['min']).'&deg;';
			$row['forecast1_high'] = round($result['daily'][0]['temp']['max']).'&deg;';
			$row['forecast1_text'] = $result['daily'][0]['weather']['description'];
			$row['forecast2_date'] = date("Y-m-d",$result['daily'][1]['dt']);
			$row['forecast2_day'] = date("Y-m-d",$result['daily'][1]['dt']);
			$row['forecast2_low'] = round($result['daily'][1]['temp']['min']).'&deg;';
			$row['forecast2_high'] = round($result['daily'][1]['temp']['max']).'&deg;';
			$row['forecast2_text'] = $result['daily'][1]['weather']['description'];
			$row['forecast2_code'] = $result['daily'][1]['dt'];
		} else
			$row = false;

		return $row;
	}

	function reformat_time($d) {
		global $timedate;
		$native_fmt = 'h:i a';
		if($d && $timedate->check_matching_format($d, $native_fmt)) {
			$d = $timedate->swap_formats($d, $native_fmt, $timedate->get_time_format());
		}
		return $d;
	}

	/**
	 * Provide external access to _searchLocations via CallDashletMethod
	**/
	function searchLocations() {
		$locName = array_get_default($_REQUEST, 'location');
		$locId = array_get_default($_REQUEST, 'location_id', '');
		$json = self::_searchLocations($locName, $locId, true);
		return $json;
	}

	function fetchTimezone($woeid) {
		require_once('include/utils/dl_utils.php');
		$woeid = addcslashes($woeid, '"');
		$query = "select timezone from geo.places where id=\"$woeid\"";
		$requestUrl = self::OPENMAPS_URL_CUR . urlencode($query);

		$req = new RemoteRequest($requestUrl);
		$resultString = $req->fetch();

		if($resultString) {
			$json = getJSONobj();
			$jsonData = $json->decode($resultString);
			if($jsonData && isset($jsonData['timezone'])) {
				return $jsonData['timezone'];
			}
		}
		return false;
	}

	static function _searchLocations($locName, $locId, $as_string = false) {
		$ret = array('count' => 0, 'location_id' => $locId);
		$json = getJSONobj();
		if(! empty($locName)) {
			require_once('include/utils/dl_utils.php');
			$locName = addcslashes($locName, '"');
			$lang = addcslashes($GLOBALS['current_language'], '"');

			$requestUrl = self::OPENMAPS_URL_CUR."?q=$locName&appid=".AppConfig::setting('company.openweather_key');

			$req = new RemoteRequest($requestUrl);
			$resultString = $req->fetch();

			if($resultString) {
				$jsonData = $json->decode($resultString);
				if($jsonData && isset($jsonData['id'])) {
					$ret['place'][] = $jsonData;
				}
			}
		}
		if($as_string)
			$ret = $json->encode($ret);
		return $ret;
	}

    /**
     * Displays the javascript for the dashlet
     *
     * @return string javascript to use with this dashlet
     */
	function displayScript() {
		$json = getJSONObj();
		$script = '<script type="text/javascript">WeatherLang = ' . $json->encode($this->dashletStrings) . ';</script>';
		return $script;
    }

    /**
     * Displays the configuration form for the dashlet
     *
     * @return string|void html to display form
     */
    function displayOptions() {
        $view = new WeatherDashletConfigView($this);
        return $view->getTemplate();
    }

    /**
     * Called to filter out $_REQUEST object when the user submits the configure dropdown
     *
     * @param array $req $_REQUEST
     * @return array filtered options to save
     */
    function saveOptions($req) {
        global $current_user;

        $options = array();
        $options['title'] = $req['title'];
        $options['auto_refresh_time'] = $req['auto_refresh_time'];
        $options['degrees_units'] = $req['degrees_units'];
        $options['wind_units'] = $req['wind_units'];
        $options['show_times'] = $req['show_times'];

        $json = getJSONobj();
        $cities = $json->decode($req['cities']);
        if(! is_array($cities))
        	$cities = array();

        $old_cities = array();
        if(isset($options['cities'])) {
        	foreach($options['cities'] as $city) {
        		if(! empty($city['woeid']))
	        		$old_cities[$city['woeid']] = $city;
        	}
        }

        foreach($cities as $city_key => $city) {
        	$city['name'] = trim(array_get_default($city, 'name', ''));
        	$city['woeid'] = trim(array_get_default($city, 'woeid', ''));
        	if($city['name'] !== "" && $city['woeid']) {
        		if(isset($old_cities[$city['woeid']])) {
        			$old_city = $old_cities[$city['woeid']];
        			if(isset($old_city['timezone']))
        				$city['timezone'] = $old_city['timezone'];
        		}
				if(empty($city['timezone'])) {
					$city['timezone'] = $this->fetchTimezone($city['woeid']);

				}
        		$options['cities'][] = $city;
        	}
        }

		$current_user->setPreference('weather', null, 0, 'WeatherDashlet');

		return $options;
    }

	function getDefaultCities() {
		return AppConfig::setting('locale.defaults.weather_cities', array());
	}

	/**
	 * Get barometric pressure state by state code
	 *
	 * @param int $stateCode:
	 * steady (0), rising (1), or falling (2).
	 * @return string
	 */
	function getPressureState($stateCode) {

		$states = array(
			0 => $this->dashletStrings['LBL_PRESSURE_STEADY'],
			1 => $this->dashletStrings['LBL_PRESSURE_RISING'],
			2 => $this->dashletStrings['LBL_PRESSURE_FALLING'],
		);

		return $states[$stateCode];
	}

	function getWindDirection($dir) {
		$dir = (round($dir / 45) * 45) % 360;
		return array_get_default($this->dashletStrings, 'LBL_WIND_DEG'.$dir);
	}
}

?>
