<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/

require_once('modules/Reports/Report.php');

class DashletsPopup {

	public function display() {
		global $current_language, $mod_strings, $pageInstance;
		require_once('include/layout/DashletManager.php');
		require_once('include/layout/forms/EditableForm.php');

		$theCats = array(
        	'Module Views',
			'Charts',
			'External',
			'Tools',
			'Miscellaneous',
		);
		$default_category = 'Miscellaneous';
		
		$dashlets = array();
		$script = array();
		foreach($theCats as $cat) {
			$script[$cat] = '';
			$dashlets[$cat] = array();
		}
		
		$dashletStrings = [];
		$dashletMeta = [];

		$dashlets_info = DashletManager::get_all_dashlet_info();
		$this->addCustomCharts($dashlets_info);

		foreach($dashlets_info as $className => $info) {
			if(!empty($info['meta']) && is_file($info['meta'])) {
				require_once($info['meta']); // get meta file
				
				$directory = substr($info['meta'], 0, strrpos($info['meta'], '/') + 1);
				if(is_file($directory . $info['class'] . '.' . $current_language . '.lang.php')) 
					require_once($directory . $info['class'] . '.' . $current_language . '.lang.php');
				elseif(is_file($directory . $info['class'] . '.en_us.lang.php')) 
					require_once($directory . $info['class'] . '.en_us.lang.php');

				$lang = isset($dashletStrings[$info['class']]) ? $dashletStrings[$info['class']] : array();
				$title = $this->label($dashletMeta[$info['class']]['title'], $lang);
				$description = $this->label($dashletMeta[$info['class']]['description'], $lang);

				if(empty($dashletMeta[$info['class']]['icon'])) { // no icon defined in meta data
					if(empty($info['icon']))
						$icon = '';
					else // use one found in directory
						$icon = get_image($info['icon'], 'border="0" alt=""');
				}
				else { // use one defined in meta data
					$icon = get_image($dashletMeta[$info['class']]['icon'], 'border="0" alt=""');
					if(! $icon)
						$icon = get_image($dashletMeta[$info['class']]['icon'], 'border="0" alt=""');
				}
								
				$categ = $dashletMeta[$info['class']]['category'];
				$id = $title;
			}
			else if($info['type'] == 'chart') {
				$icon = get_image($info['icon'], 'border="0" alt=""');
				$description = '';
				$title = $info['title'];
				$categ = $info['category'];

				if (isset($info['file'])) {
					$id = $title;
				} else {
					$id = $info['class'];
				}

			} else {
				continue;
			}

			if(! $categ || ! isset($dashlets[$categ]))
				$categ = $default_category;
			
			$dlet_desc = addcslashes($description, "'");
			$dashlets[$categ][$id] = array($className, $icon, $title, $dlet_desc);
		}

		$this->sortCharts($dashlets);

		$tabs = array();
		$default_tab = '';
		$divs = array();
		$sects = array();

		foreach($theCats as $cat) {
			if(! is_countable($dashlets[$cat]) || ! count($dashlets[$cat]))
				continue;
			if(! $default_tab) $default_tab = $cat;
			$tabs[] = array(
				'name' => $cat,
				'label' => $mod_strings['dashlet_categories_dom'][$cat],
			);
			$sects[$cat.'_div'][] = $cat;
			ksort($dashlets[$cat]);
			$left = $right = '';
			$catCount = is_countable($dashlets[$cat]) ? count($dashlets[$cat]) : 0;
			$c = round($catCount / 2);
			$i = 0;
			foreach($dashlets[$cat] as $dlet) {
				list($className, $icon, $title, $desc) = $dlet;
				$elt_id = 'dashlet_'.$className;
				$link = '<div class="form-entry row"><a id="'.$elt_id.'" href="#" onclick="return SUGAR.sugarHome.addDashlet(\'' . $className . '\');" style="display: block; padding: 5px; text-decoration: none">'
					. $icon . '&nbsp;' . $title . '</a></div>';
				if($desc)
					$script[$cat] .= "var t = new SUGAR.ui.Tooltip('{$elt_id}_tt', {target_id: '$elt_id', target_offset: {left: 40}, text: '$desc', container_id: 'dashlet-add' } );";

				if($i >= $c)
					$right .= $link;
				else
					$left .= $link;
				$i ++;
			}
			$cls = 'form-section' . ($cat == $default_tab ? '' : ' hidden');
			$div = "<div id=\"{$cat}_div\" class=\"$cls\">";
			$div .= '<div class="row card-row form-row">';
			$div .= '<div class="column form-cell">' . $left . '</div>';
			$div .= '<div class="column form-cell">' . $right . '</div>';
			$div .= '</div></div>';
			$divs[$cat] = $div;
		}
		
		$spec = array(
			'name' => 'dashlet_tab',
			'tabs' => $tabs,
			'manage_sections' => $sects,
		);

		$frm = new EditableForm('tabgroup', 'TabForm');
		$str = $frm->open();
		$str .= '<div class="card-header form-tabs">' . $frm->renderTabGroup($spec, 'Module Views') . '</div>';
		$str .= '<div class="card-body form-body">';
		$str .= implode('', $divs);
		$str .= '</div>';
		$str .= $frm->close();
		$frm->exportIncludes();
		$script = implode('', $script);
		$pageInstance->add_js_literal($script, null, LOAD_PRIORITY_FOOT);
		
		return $str;
	}

	private function label($t, $strings) {
		if(is_array($t))
			return translate($t[0], $t[1]);
		else if(isset($strings[$t]))
			return $strings[$t];
		return $t;
	}

	private function addCustomCharts(&$dashlets_info) {
		$charts = Report::get_chartable_reports();

		foreach($charts as $chart_id => $chart_info) {

			if(isset($dashlets_info[$chart_id]) || isset($dashlets_info['report_' . $chart_id]))
				continue;

			$dashlets_info['report_' .$chart_id] = [
				'type' => 'chart',
				'class' => $chart_id,
				'chart_type' => $chart_info['chart_type'],
				'title' => $chart_info['name'],
				'icon' => 'Forecasts',
				'category' => 'Charts',
				'properties' => ['report_id' => $chart_info['id']]
			];
		}
	}

	private function sortCharts(&$dashlets) {
		function cmp($a, $b) {
			if ($a == $b)
				return 0;
			return ($a[2] < $b[2]) ? -1 : 1;
		}

		usort($dashlets['Charts'], 'cmp');
	}
}

$popup = new DashletsPopup();
$body = $popup->display();

echo $body;
