<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');

$json_supported_actions['start_import'] = array();
function json_start_import()
{
	require_once('modules/ImportDB/profiles/ImportDBProfileManager.php');
	require_once('modules/ImportDB/functions.php');
	require_once('include/ProgressIndicator.php');
	
	$module = array_get_default($_REQUEST, 'target_module');

	ImportDBProfileManager::init();

	$profile = getImportDBProfile($module);
	if(! $profile) {
		return json_bad_request('Import profile not found');
	}
	$mod_title = getImportDBModuleTitle($profile, $module);
	checkImportDBFile($profile, $module);

	$iter = array_get_default($_REQUEST, 'iter');
	if (is_null($iter)) {
		$limit = PHP_INT_MAX;
		$offset = 0;
	} else {
		$limit = IMPORTDB_CHUNK_SIZE;
		$offset = intval($iter) * $limit;
	}
	if(! $offset) $_SESSION['import_total'] = 0;
	
	$pid = importProgressId();
	$ppath = importProgressPath($pid);
	$progress = $ppath ? new ProgressIndicator($ppath, true) : null;
	
	$ret = ['progress_id' => $pid];
	// return result to user and effectively close connection
	json_return_value($ret, null, true);

	$csv_handle = fopen(IMPORTDB_DATA_FILE, 'r');
	set_time_limit(3600);
	try {
		$count = $profile->executeImport($module, $csv_handle, $ppath, $offset, $limit);
		$_SESSION['import_total'] += $count;
		
		if ($_SESSION['import_total'] == 1) {
			$msg = sprintf(translate('MSG_ITEM_IMPORTED_SINGLE', 'ImportDB'), $profile->getAvailableModules($module));
		} else {
			$msg = sprintf(translate('MSG_ITEM_IMPORTED_PLURAL', 'ImportDB'), $_SESSION['import_total'],
				$profile->getAvailableModules($module));
		}
		$progress->finish('success', $msg);
	} catch (Exception $e) {
		$progress->finish('error', $e->getMessage());
	}
	
	sugar_cleanup();
}

$json_supported_actions['get_import_progress'] = array();
function json_get_import_progress()
{
	require_once('modules/ImportDB/functions.php');
	$pid = array_get_default($_REQUEST, 'progress_id');
	if(! $pid)
		return json_bad_request('Missing progress ID');
	$timestamp = array_get_default($_REQUEST, 'timestamp');
	$ppath = importProgressPath($pid);
	if(! $ppath) $data = [];
	else {
		require_once('include/ProgressIndicator.php');
		$indicator = new ProgressIndicator($ppath);
		if(! empty($timestamp)) {
			$data = $indicator->pollHistory($timestamp, 5);
		} else {
			$data = $indicator->requestHistory();
		}
	}
	json_return_value($data);
}

$json_supported_actions['import_complete'] = array();
function json_import_complete()
{
	require_once('modules/ImportDB/functions.php');
	if(file_exists(IMPORTDB_DATA_FILE))
		@unlink(IMPORTDB_DATA_FILE);
	$ret = ['status' => 'ok'];
	json_return_value($ret);
}

