<?php
require_once('modules/Invoice/EmailStepsTemplate.php');
require_once('modules/Emails/Email.php');
require_once('include/layout/forms/EditableForm.php');
require_once 'modules/Invoice/EmailInvoice.php';
require_once 'modules/Invoice/Invoice.php';
require_once('modules/PDFTemplates/PDFTemplatePrinter.php');
require_once('modules/PDFTemplates/utils.php');

class EmailPDF {

	/**
	 * @var EditableForm
	 */
	var $from_obj;

	/**
	 * @var array - user input ($_REQUEST)
	 */
	var $input;

	/**
	 * @var int - current step
	 */
	var $step;

	const SESSION_INVOICE_IDS = 'invoice_ids';
	const SESSION_TEMPLATE_ID = 'template_id';
	const SESSION_PDF_TEMPLATE = 'pdf_template_id';

	/**
	 * @param array $input - user input ($_REQUEST)
	 */
	function __construct($input) {
		$this->input = $input;
		$this->step = (int)array_get_default($this->input, 'pdf_next_step', '1');
		$this->form_obj = new EditableForm('editview', 'EmailPDFInvoice');
	}

	/**
	 * Load current step page
	 *
	 * @return string
	 */
	function loadStep() {
		$step_page = '';

		$this->initInvoiceData();

		switch ($this->step) {
			case 1:
				$step_page = $this->initStep1();
				break;
			case 2:
				$step_page = $this->initStep2();
				break;
			case 3:
				$step_page = $this->initStep3();
				break;
			default:
				$this->destroyInvoiceData();
				header('Location: async.php?action=index&module=Invoice');
				break;
		}

		$this->form_obj->exportIncludes();
		return $step_page;
	}

	/**
	 * Initialize and render step 1
	 *
	 * @return string
	 */
	function initStep1() {
		if (! Email::check_email_settings()) {
			$str = return_module_language($GLOBALS['current_language'], 'Emails');
			return '<span class="error">'.$str['WARNING_SETTINGS_NOT_CONF'].'</span>';
		}

		$selected_template = $_SESSION['email_pdf_invoices'][EmailPDF::SESSION_TEMPLATE_ID];
		$selected_pdf_template = $_SESSION['email_pdf_invoices'][EmailPDF::SESSION_PDF_TEMPLATE];

		return EmailStepsTemplate::step1($this->form_obj, $selected_template, $selected_pdf_template);
	}

	/**
	 * Initialize and render step 2
	 *
	 * @return string
	 */
	function initStep2() {
		$objEmailInvoice = new CEmailInvoice();
		$created_emails = array();
		$errors = array();

		foreach ($_SESSION['email_pdf_invoices'][EmailPDF::SESSION_INVOICE_IDS] as $InvoiceId) {
			$invoice = ListQuery::quick_fetch_row('Invoice', $InvoiceId);

			$objAccount = new Account();
			$objAccount->retrieve($invoice['billing_account_id']);

			if ($arContactIds = $objAccount->getAccountingContacts()) {
				
				$arNotes = array();
				$printer = new PDFTemplatePrinter($_SESSION['email_pdf_invoices'][EmailPDF::SESSION_PDF_TEMPLATE], $InvoiceId);
				$printer->useCompanyAddress(array_get_default($_REQUEST, 'use_address'));
				$pdf = $printer->render();
				$pdf->filename = filename_safe_string($printer->getFileName());
				$filename = $pdf->filename;
				$mergePDF = $printer->handle_merge($pdf);
				$pdfId = $printer->create_note($mergePDF, $filename, false);
				$arNotes[] = $pdfId;

				$arEmails = $objEmailInvoice->createEmailDrafts($InvoiceId, $arContactIds, $arNotes, $_SESSION['email_pdf_invoices'][EmailPDF::SESSION_TEMPLATE_ID]);

				foreach ($arEmails as $strEmailId => $arEmailInfo) {
					$format = translate('LBL_MASS_PDF_DRAFT_EMAIL', 'Accounts');

					$string = sprintf(
						$format,
						"<a href='index.php?module=Emails&action=DetailView&record={$strEmailId}' class='tabDetailViewDFLink' target='_blank'>{$arEmailInfo['subject']}</a>",
						"<a href='index.php?module=Accounts&action=DetailView&record={$objAccount->id}' class='tabDetailViewDFLink' target='_blank'>{$objAccount->name}</a>",
						"<a href='index.php?module=Contacts&action=DetailView&record={$arEmailInfo['contact_id']}' class='tabDetailViewDFLink' target='_blank'>{$arEmailInfo['contact_name']}</a>"
					);

					$created_emails[] = array('ID' => $strEmailId, 'DESCRIPTION' => $string);
				}

			} else {
				$format = translate('LBL_MASS_PDF_NO_CONTACT', 'Accounts');

				$string = sprintf(
					$format,
					"<a href='index.php?module=Accounts&action=DetailView&record={$objAccount->id}' class='tabDetailViewDFLink' target='_blank'>{$objAccount->name}</a>"
				);

				$errors[] = $string;
			}
		}

		return EmailStepsTemplate::step2($this->form_obj, $created_emails, $errors);
	}

	/**
	 * Initialize and render step 3
	 *
	 * @return string
	 */
	function initStep3() {
		global $timedate;
		$emails = array();
		$errors = array();

		if (isset($this->input['email_ids'])) {
			foreach ($this->input['email_ids'] as $strId => $checked) {
				if (!$checked) continue;
				$objEmail = new Email();
				$objEmail->retrieve($strId);
				// Parse the address arrays
				$objEmail->regenerate_email_arrays();
				// Regenerate the attachment array
				$objEmail->regenerate_attachment_array();

				if ($objEmail->send2($errors)) {
					$objEmail->folder = EmailFolder::get_std_folder_id($objEmail->assigned_user_id, STD_FOLDER_SENT);
					$objEmail->type = 'out';
					$objEmail->status = 'sent';
					$objEmail->date_start = $timedate->to_display_date_time(date('Y-m-d H:i:s'), true, false);
					$objEmail->save();

					$format = translate('LBL_MASS_PDF_SENT_EMAIL', 'Invoice');
					$string = sprintf(
						$format,
						"<a href='index.php?module=Emails&action=DetailView&record={$objEmail->id}' class='tabDetailViewDFLink' target='_blank'>{$objEmail->name}</a>",
						"{$objEmail->to_addrs}"
					);
					$emails[] = $string;
				} else {
					$format = translate('LBL_MASS_PDF_ERROR_EMAIL', 'Invoice');
					$string = sprintf(
						$format,
						"<a href='index.php?module=Emails&action=DetailView&record={$objEmail->id}' class='tabDetailViewDFLink' target='_blank'>{$objEmail->name}</a>",
						"{$objEmail->to_addrs}"
					);
					$errors[] =  $string;
				}
			}
		}

		return EmailStepsTemplate::step3($emails, $errors);
	}

	/**
	 * Initialize the PDF Invoice temp data
	 *
	 * @return void
	 */
	function initInvoiceData() {
		if (!array_key_exists('email_pdf_invoices', $_SESSION)) {
			$_SESSION['email_pdf_invoices'] = array (
				EmailPDF::SESSION_INVOICE_IDS => array(),
				EmailPDF::SESSION_TEMPLATE_ID => '',
				EmailPDF::SESSION_PDF_TEMPLATE => '',
			);
		}

		if ($this->step == 1) {

			if(isset($this->input['list_uids']))
				$_SESSION['email_pdf_invoices'][EmailPDF::SESSION_INVOICE_IDS] = explode(';', $this->input['list_uids']);

		} else if ($this->step == 2) {
			if (isset($this->input['template_id'])) {
				$_SESSION['email_pdf_invoices'][EmailPDF::SESSION_TEMPLATE_ID] = $this->input['template_id'];
				$_SESSION['email_pdf_invoices'][EmailPDF::SESSION_PDF_TEMPLATE] = $this->input['pdf_template_id'];
			}
		}
	}

	/**
	 * Destroy PDF invoice temp data
	 *
	 * @return void
	 */
	function destroyInvoiceData() {
		unset($_SESSION['email_pdf_invoices']);
	}
}
?>
