<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


require_once('data/SugarBean.php');

class Invoice extends SugarBean {
	// Stored fields
	var $id;
	var $invoice_number;
	var $date_entered;
	var $date_modified;
	var $modified_user_id, $modified_by_name;
	var $created_by, $created_by_name;
	var $assigned_user_id, $assigned_user_name;
	//
	var $name;
	var $purchase_order_num;
	var $description;
	var $due_date;
	var $terms;
	var $prefix;
	//
	var $opportunity_id, $opportunity_name;
	var $event_id, $event_name;
	// These are related
	var $billing_account_name, $billing_account_id;
	var $billing_address_street, $billing_address_city, $billing_address_state, $billing_address_postalcode, $billing_address_country;
	var $billing_phone;
	var $billing_email;
	// Used by quick-creation form
	var $account_id;
	//
	var $shipping_account_name, $shipping_account_id;
	var $shipping_address_street, $shipping_address_city, $shipping_address_state, $shipping_address_postalcode, $shipping_address_country;
	var $shipping_phone;
	var $shipping_email;

	var $populate_addresses = true;

	//
	var $billing_contact_id, $billing_contact_name;
	var $shipping_contact_id, $shipping_contact_name;
	var $currency_id, $exchange_rate;
	var $shipping_provider_id;
	var $tax_information;
	var $tax_exempt;
	var $discount_before_taxes;
	var $shipping_stage;

	// line items
	var $line_items = array();
	var $grand_total;

	var $amount, $amount_usdollar;
	var $amount_due;
	var $amount_due_usdollar;
	var $cancelled;
	var $show_components;
	var $products_created;
	// ...

	var $from_quote_id;
	var $refunded_invoice_id, $refunded_invoice_number;
	var $affect_invoice_balance;
	var $from_so_id;
	var $from_so_number, $from_so_name; // not stored
	var $from_quote_number; // not stored
	var $allocated; // for subpanel on payments
	var $gross_profit, $gross_profit_usdollar;

	var $partner_id;
	var $partner_name;
	var $note_id;

	var $relationship_fields = array(
		'note_id' => 'notes',
		'billing_account_id' => 'billing_account_link',
		'shipping_account_id' => 'shipping_account_link',
		'project_id' => 'projects',
	);

	//
	var $table_name = "invoice";
	var $object_name = "Invoice";
	var $group_object_name = "InvoiceLineGroup";
	var $module_dir = "Invoice";
	var $new_schema = true;
	//
	var $account_table = "accounts";
	var $contact_table = "contacts";
	var $opportunity_table = "opportunities";


	static $inherit_quote_fields = array(
		'assigned_user_id',
		'deleted',
		'name',
		'opportunity_id',
		'partner_id',
		'purchase_order_num',
		'billing_account_id',
		'billing_contact_id',
		'billing_address_street',
		'billing_address_city',
		'billing_address_state',
		'billing_address_postalcode',
		'billing_address_country',
		'shipping_account_id',
		'shipping_contact_id',
		'shipping_address_street',
		'shipping_address_city',
		'shipping_address_state',
		'shipping_address_postalcode',
		'shipping_address_country',
		'currency_id',
		'exchange_rate',
		'shipping_provider_id',
		'description',
		'amount',
		'amount_usdollar',
		'terms',
		'show_components',
		'tax_information',
		'tax_exempt',
		'pricebook_id',
	);

	static $massupdate_restricted = [
		'terms', 'invoice_date', 'due_date', 'billing_account_id',
	   'tax_exempt', 'discount_before_taxes',
	];

    static $grp_idx = 0;

	static function init_from_tally(TallyUpdate &$self, TallyUpdate &$tally) {
		$map = self::$inherit_quote_fields;
		$type = $tally->getModelName();
		if ($type == 'Quote') {
			$map['id'] = 'from_quote_id';
		} elseif ($type == 'SalesOrder') {
			$map['id'] = 'from_so_id';
			$map['related_quote_id'] = 'from_quote_id';
		}

		$updates =  array();
		foreach ($map as $key => $value) {
			if(is_int($key)) $key = $value;
			$fv = $tally->getField($key);
			if(isset($fv)) {
				$updates[$value] = $fv;
			}
		}
		$self->set($updates);
		copy_custom_fields($tally, $self);
		$groups = $tally->getGroups(true);
		$self->replaceGroups($groups);
		return $updates;
	}

	static function add_intermediate_invoices(RowUpdate &$self, RowUpdate &$tally) {
		/** @var TallyUpdate $self */

		$int_invoices = $tally->getLinkResults('invoice');

		if ($int_invoices->getResultCount() > 0) {
			require_once('modules/Invoice/FinalMilestone.php');
			$rows = $int_invoices->getRows();
			$groups = $self->getGroups(true);

			$grp_idx = sizeof($groups) + 1;
			$grp_id = 'temp~'.($grp_idx);

			$finalMilestone = new FinalMilestone($rows);
			$invoices_group = $finalMilestone->createInvoicesGroup($grp_id);
			$groups = array_merge($groups, $invoices_group);

			$self->replaceGroups($groups);
		}
	}

	static function init_from_account($account_id) {

		$lq = new ListQuery('Account');
		$lq->addColumnFields();
		$lq->addField('default_pricebook.editable');
		$result = $lq->queryRecord($account_id);
		if ($result && !$result->failed)
			$account = $result->row;
		else
			$account = null;


		$data = array();

		if ($account != null) {
			init_invoice_from_account($data, $account);
        }

        return $data;
	}

	static function init_from_case($case_id) {
	    $case = ListQuery::quick_fetch_row('aCase', $case_id);
        $data = array();

        if ($case != null) {
            global $current_language;
            $case_mod_strings = return_module_language($current_language, 'Cases');

            if(! empty($case['account_id'])) {
            	$account_data = self::init_from_account($case['account_id']);
            	array_extend($data, $account_data);
            }
            if(! empty($case['cust_contact_id'])) {
            	$data['billing_contact_id'] = $data['shipping_contact_id'] = $case['cust_contact_id'];
            }
            if(! empty($case['name'])) {
                $data['name'] = $case['name'];
            }
            if(! empty($case['description'])) {
                $data['description'] = $case_mod_strings['LBL_DESCRIPTION']."\n ".$case['description'];
            }

            if(! empty($data['description'])) $data['description'] .= "\n\n";

            if(! empty($case['resolution']))
                $data['description'] .= $case_mod_strings['LBL_RESOLUTION']."\n ".$case['resolution'];
        }

        return $data;
	}

    function getCurrentPrefix()
	{
		return AppConfig::get_sequence_prefix('invoice_prefix');
	}


	function getCurrentSequenceValue()
	{
		return AppConfig::current_sequence_value('invoice_number_sequence');
	}

	function getNextSequenceValue()
	{
		return AppConfig::next_sequence_value('invoice_number_sequence');
	}

	function save($check_notify = FALSE, $update_balance = true)
	{
		// need an ID to save line items
		if(empty($this->id)) {
			$this->id = create_guid();
			$this->new_with_id = true;
		}

		if(! $this->due_date)
			$this->due_date = Invoice::calc_due_date($this->terms, true);

		// must do this before manipulating numeric fields
		$this->unformat_all_fields();

		$this->save_line_groups();

		$ret = parent::save($check_notify);

		return $ret;
	}

	static function create_supported_assembly(RowUpdate $upd, $line, $id)
	{
		if(! strlen($line['name']))
			return;
		$sa = RowUpdate::blank_for_model('SupportedAssembly');
		$a = ListQuery::quick_fetch('Assembly', $line['related_id']);
		$sa->set('id', $id);
		$transfer = array(
			'supplier_id',
			'manufacturer_id',
			'model_id',
			'vendor_part_no',
			'product_url',
			'product_category_id',
			'product_type_id',
		);

		if ($a && !$a->getField('deleted')) {
			foreach ($transfer as $f) {
				$sa->set($f, $a->getField($f));
			}
		}
		$sa->set(array(
			'manufacturers_part_no' => $line['mfr_part_no'],
			'name' => $line['name'],
			'quantity' => $line['quantity'],
			'account_id' => $upd->getField('billing_account_id'),
			'tax_code_id' => $line['tax_class_id'],
			'currency_id' => $upd->getField('currency_id'),
			'exchange_rate' => $upd->getField('exchange_rate'),
		));
		$sa->save();
		$serials = array_filter(preg_split('~[\r\n]+~', $line['serial_numbers']));
		if (! empty($line['serial_no']) && ! in_array($line['serial_no'], $serials))
			$serials[] = $line['serial_no'];
		foreach ($serials as $ser) {
			$ser_up = RowUpdate::blank_for_model('SerialNumber');
			$ser_up->set('serial_no', $ser);
			$ser_up->set('item_name', 'System');
			$ser_up->set('assembly_id', $sa->getPrimaryKeyValue());
			$ser_up->save();
		}
	}

	static function create_supported_product(RowUpdate &$upd, $line, $parent_id = '')
	{
		if($line['related_type'] != 'ProductCatalog')
			return;
		if(! strlen($line['name']))
			return;
		$sp = RowUpdate::blank_for_model('Asset');
		$p = ListQuery::quick_fetch('Product', $line['related_id']);
		$transfer = array(
			'supplier_id',
			'manufacturer_id',
			'model_id',
			'vendor_part_no',
			'url',
			'product_category_id',
			'product_type_id',
		);

		if ($p && !$p->getField('deleted')) {
			foreach ($transfer as $f) {
				$sp->set($f, $p->getField($f));
			}
		}
		$name = $line['name'];
		$pnum = $line['mfr_part_no'];
		if(! strlen($name))
			$name = $pnum;
		else if(! strlen($pnum))
			$pnum = $name;
		$sp->set(array(
			'name' => $name,
			'manufacturers_part_no' => $pnum,
			'quantity' => $line['quantity'],
			'account_id' => $upd->getField('billing_account_id'),
			'supported_assembly_id' => $parent_id,
			'tax_code_id' => $line['tax_class_id'],
			'currency_id' => $upd->getField('currency_id'),
			'exchange_rate' => $upd->getField('exchange_rate'),
			'purchase_price' => $line['unit_price'],
			'purchase_usdollar' => $line['unit_price_usd'],
		));
		$sp->save();
		$serials = array_filter(preg_split('~[\r\n]+~', $line['serial_numbers']));
		if (! empty($line['serial_no']) && ! in_array($line['serial_no'], $serials))
			$serials[] = $line['serial_no'];
		foreach ($serials as $ser) {
			$ser_up = RowUpdate::blank_for_model('SerialNumber');
			$ser_up->set('serial_no', $ser);
			$ser_up->set('item_name', 'System');
			$ser_up->set('asset_id', $sp->getPrimaryKeyValue());
			$ser_up->save();
		}
	}

	function save_line_groups() {
		if(isset($this->line_groups) && is_array($this->line_groups)) {
			foreach(array_keys($this->line_groups) as $k) {
				$this->line_groups[$k]->save();
			}
			$gt =& $this->line_groups['GRANDTOTAL'];
			$this->amount = $gt->total;
			$this->gross_profit = $gt->subtotal - $gt->discount - $gt->cost;
		}
	}

	function get_recurrence_forward_instances()
	{
		return 1;
	}

	function get_recurrence_scheduled_interval()
	{
		return 3600*24*30; // for one month by befault
	}

	function get_recurrence_date_field()
	{
		return 'due_date';
	}

	function get_recurrence_time_field()
	{
		return '';
	}

	function extractQuantities($line_items, &$products, &$assemblies, $sign)
	{
		foreach ($line_items as $group) {
			if (!empty($group->lines)) foreach ($group->lines as $row) {
				if($row['related_type'] == 'ProductCatalog') {
					$id = $row['related_id'];
					if(! isset($products[$id])) $products[$id] = 0;
					$products[$id] += $row['ext_quantity'] * $sign;
				}
				else if($row['related_type'] == 'Assemblies') {
					$id = $row['related_id'];
					if(! isset($assemblies[$id])) $assemblies[$id] = 0;
					$assemblies[$id] += $row['ext_quantity'] * $sign;
				}
			}
		}
	}

	static function calc_due_date($terms = null, $display = false) {
		global $timedate;
		$offset = 0;
		if($terms) {
			if(preg_match('/(\d+) Days/i', $terms, $m))
				$offset = $m[1];
		}

		$now = gmdate('Y-m-d H:i:s', strtotime(gmdate('Y-m-d H:i:s') . ' GMT + ' . $offset . ' days'));
		$now = $timedate->to_display_date_time($now, false, true);
		list($now, $unused) = explode(' ', $now);

		if(!$display)
			$now = $timedate->swap_formats($now, $timedate->get_date_format(), $timedate->get_db_date_format());
		return $now;
	}

	function get_view_closed_where_advanced($param)
	{
		return '1';
	}

	function get_view_closed_where_basic($param)
	{
		return empty($param['value']) ? "ABS(invoice.amount_due) > 0.005 AND ! invoice.cancelled" : '1';
	}

	function getDefaultListWhereClause()
	{
		return "(ABS(invoice.amount_due) > 0.005 AND ! invoice.cancelled)";
	}

	function search_by_number($param) {
		if (empty($param['value'])) {
			return 1;
		} else {
			return "(invoice.invoice_number = '" . PearDatabase::quote($param['value']) . "' OR CONCAT(invoice.prefix, invoice.invoice_number) = '" . PearDatabase::quote($param['value']) . "' ) ";
		}
	}

	function cleanup() {
		if(isset($this->line_groups)) {
			$this->cleanup_line_groups($this->line_groups);
			unset($this->line_groups);
		}
		parent::cleanup();
	}
	
	function cleanup_line_groups(&$groups) {
		$this->cleanup_list($groups);
	}

	function get_search_stage_options()
	{
		global $app_list_strings, $mod_strings;
		return array_merge(array(''=>''), $app_list_strings['shipping_stage_dom']);
	}

	function get_stage_where_advanced($param)
	{
		return "invoice.shipping_stage='" . PearDatabase::quote($param['value']) . "'";
	}

	function get_stage_where_basic($param)
	{
		return '1';
	}

	static function get_attached_credit_notes($id, $currency_id, $exchange_rate)
	{
		$lq = new ListQuery('CreditNote');
		$lq->addSimpleFilter('invoice_id', $id);
		$lq->addSimpleFilter('apply_credit_note', '', 'true');
		$lq->addSimpleFilter('cancelled', 0);
		return $lq->fetchAllRows();
	}

    static function payment_allocations($id, $currency_id, $include_refunded = true) {
    	$tbl = 'invoices_payments';
    	$rel_id = 'invoice_id';
        $query = sprintf(
            "SELECT
            payments.*, CONCAT(payments.prefix, payments.payment_id) AS full_number,
            '%s' as invoice_currency_id,
            link.amount AS allocated,
            link.amount_usdollar AS allocated_usdollar
            FROM `$tbl` link
            LEFT JOIN payments
            ON link.payment_id = payments.id
            WHERE
            !link.deleted
                AND
            !payments.deleted
            %s
                AND
            link.$rel_id = '%s'
            ",
            PearDatabase::quote($currency_id),
            $include_refunded ? '' : ' AND payments.refunded = 0',
            PearDatabase::quote($id)
        );

        return $query;
    }

    static function get_payments($id, $currency_id) {
        global $db;
		$payments = array();

        $query = self::payment_allocations($id, $currency_id);
        $res = $db->query($query);

        while ($row = $db->fetchByAssoc($res, -1, false)) {
            $payments[] = $row;
        }

		return $payments;
	}

    static function update_account_balance($billing_account_id, $user_id = null) {
        $account = ListQuery::quick_fetch('Account', $billing_account_id);

        if ($account) {
            $upd = RowUpdate::for_result($account, null, $user_id);
            //Call pre_update_balance through account's before_save hook
            $upd->save();
        }
    }

    static function after_delete(RowUpdate $upd) {
        global $db;
        $eid = $db->quote($upd->getPrimaryKeyValue());

        //$upd->removeAllLinks('payments');
        $db->query("UPDATE booked_hours bh SET bh.invoice_id = NULL WHERE bh.invoice_id = '$eid'");

        self::update_account_balance($upd->getField('billing_account_id'));
    }

    static function before_save(RowUpdate $upd) {
        if($upd->new_record) {
            $upd->set(array(
                'amount_due' => $upd->getField('amount')
            ));

            if(! $upd->getField('invoice_date'))
            	$upd->set('invoice_date', date('Y-m-d'));
        } else {
            $amount = (float)$upd->getField('amount');
            $id = $upd->getPrimaryKeyValue();
            $total_paid = 0;
			$amount_credited = 0;
			$credit_payments = 0;

			$payments = self::get_payments($id, $upd->getField('currency_id'));
			foreach ($payments as $payment) {
				$total_paid += $payment['allocated'];
				if ($payment['direction'] == 'credit') {
					$credit_payments += $payment['allocated'];
				}
			}

			$notes = self::get_attached_credit_notes($id, null, null);
			foreach ($notes as $note) {
				$amount_credited += $note['amount_usdollar'];
			}

            $total_paid += $amount_credited * $upd->getField('exchange_rate');

            $amount_due = $amount - $total_paid;
            $net_amount = $amount - $credit_payments;
            $upd->set('amount_due', $amount_due);
            $upd->set('net_amount', $net_amount);
        }

		$products_created = $upd->getField('products_created', 'none');
		if ($products_created === 'none') {
			if ($upd->new_record) {
				$products_created = 0;
			} else {
				$u = RowUpdate::blank_for_model($upd->getModelName());
				if ($u->retrieveRecord($upd->getPrimaryKeyValue())) {
					$products_created = $u->getField('products_created');
				} else {
					$products_created = 1;
				}
			}
		}
		if (! $products_created && $upd->getField('amount_due') <= 0) {
			if (AppConfig::setting('company.create_invoice_products') && $upd instanceof TallyUpdate) {
				$id_map = array();
				$children = array();
				$groups = $upd->getGroups();
				foreach ($groups as $gid => $group) {
					foreach ($group['lines'] as $line) {
						if (!empty($line['is_comment'])) continue;
						if (!empty($line['parent_id'])) {
							$children[] = $line;
						} else {
							if (!empty($line['sum_of_components'])) {
								$id_map[$line['id']] = create_guid();
								self::create_supported_assembly($upd, $line, $id_map[$line['id']]);
							} else {
								self::create_supported_product($upd, $line);
							}
						}
					}
				}
				foreach ($children as $child) {
					self::create_supported_product($upd, $child, array_get_default($id_map, $line['parent_id'], ''));
				}
				$upd->set('products_created', 1);
			}
		}
    }

    static function after_save(RowUpdate $upd) {
    	if($upd->new_record) {
    		$acc_id = $upd->getField('billing_account_id');

            if(! empty($acc_id) && ($base = ListQuery::quick_fetch('Account', $acc_id))) {
    			$acc_up = RowUpdate::for_result($base, null, $upd->getUserId());

                if(! $acc_up->getField('first_invoice_id'))
    				$acc_up->set('first_invoice_id', $upd->getField('id'));

                $acc_up->set('last_invoice_id', $upd->getField('id'));
				$acc_up->save();
    		}

    		$qt_id = $upd->getField('from_quote_id');
            if(! empty($qt_id) && ($base = ListQuery::quick_fetch('Quote', $qt_id))) {
    			$qt_up = RowUpdate::for_result($base, null, $upd->getUserId());
    			if($qt_up->getField('quote_stage') != 'Closed Accepted') {
    				$qt_up->set('quote_stage', 'Closed Accepted');
    				$qt_up->save();
    			}
    		}

    		$opp_id = $upd->getField('opportunity_id');
            if(! empty($opp_id) && ($base = ListQuery::quick_fetch('Opportunity', $opp_id))) {
    			$opp_up = RowUpdate::for_result($base, null, $upd->getUserId());
    			if(! preg_match('~^Closed ~', $opp_up->getField('sales_stage'))) {
					$stage =  'Closed Won';
					$opp_up->set('sales_stage', $stage);
					require_once 'modules/Opportunities/Opportunity.php';
                	$prob_map = Opportunity::probability_map();
	                if(isset($prob_map[$stage])) {
    	                $opp_up->set('probability', $prob_map[$stage]);
        	        }
    				$opp_up->save();
    			}
    		}
		}

		if (isset($_REQUEST['recurrence_rules'])) {
			require_once 'modules/Recurrence/RecurrenceRule.php';
			$rule = new RecurrenceRule;
			$rule->update_rules_from_JSON('Invoice', $upd->getPrimaryKeyValue(), $_REQUEST['recurrence_rules'], true);
		}

        self::update_account_balance($upd->getField('billing_account_id'), $upd->getUserId());

        $orig_value = $upd->getField('billing_account_id', null, true);
		$new_value = $upd->getField('billing_account_id');
		if($orig_value && $new_value && $orig_value != $new_value)
			self::update_account_balance($orig_value, $upd->getUserId());
		
		require_once 'modules/MonthlyServices/MonthlyService.php';
		MonthlyService::updateBalances($upd->getPrimaryKeyValue());

		if($upd instanceof TallyUpdate)
			self::update_booked_hours($upd);

		$so_id = $upd->getField('from_so_id');
		if ($so_id) {
			$row = ListQuery::quick_fetch_key('Shipping', 'so_id', $so_id);
			if($row) {
				$upd = RowUpdate::for_result($row, null, $upd->getUserId());
				$upd->setRelatedData('so_status_update', true);
				require_bean('Shipping');
				Shipping::update_stage($upd);
			}
		}


    }

    static function update_booked_hours(TallyUpdate $upd) {
    	$self_id = $upd->getPrimaryKeyValue();
    	$groups = $upd->getGroups();
    	if($groups && $self_id) {
			foreach ($groups as $gid => $group) {
				foreach ($group['lines'] as $line) {
					if (!empty($line['is_comment'])) continue;
					if (!empty($line['related_id']) && $line['related_type'] == 'Booking') {
						$row = ListQuery::quick_fetch('BookedHours', $line['related_id']);
						if($row && $row->getField('invoice_id') != $self_id) {
							$upd = RowUpdate::for_result($row);
							$upd->set('invoice_id', $self_id);
							$upd->save();
						}
					}
				}
			}
		}
    }

    static function show_sales_popup(DetailManager $mgr) {
        require_bean('Account');
		Account::add_account_popup($mgr->getRecord(), 'billing_account_id', 'sales');
		$mgr->getLayout()->addExtScriptInclude('https://js.stripe.com/v3/');
    }

	static function add_additional_hiddens(DetailManager $mgr) {
		if ($mgr->getRecord()->getField('final') == 1)
			$mgr->layout->addFormHiddenFields(array('final' => 1), false);
	}

	static function get_booked_hours_group($from_id, $from_model) {

		// for projects, we have to find hours booked against project tasks,
		// not the project itself

		$parent_model = $from_model;

		if ($from_model =='Project')
			$parent_model = 'ProjectTask';

        $hours = new ListQuery($from_model, true, array('link_name' => 'booked_hours', 'parent_key' => $from_id));
        $hours->addField('booking_category');
        $hours->addSimpleFilter('status', 'approved', '=');
        $hours->addSimpleFilter('booking_class', 'billable-work', '=');
        $hours->addSimpleFilter('invoice_id', '', 'null');
		$result = $hours->runQuery();
        $add_comments = AppConfig::setting('company.add_booked_hours');
		$new_groups = array();

		$from_row = ListQuery::quick_fetch($from_model, $from_id);

        if($result && $result->total_count) {
            $grp_id = 'temp~'.(self::$grp_idx ++);
            $new_groups = array(
                $grp_id => array(
                    'id' => $grp_id,
					'status' => 'Draft',
					'name' => $from_row ? $from_row->getField('name') : '',
                    'lines' => array(),
                    'lines_order' => array(),
                    'adjusts' => array(),
                    'adjusts_order' => array(),
                    'group_type' => 'service',
                ),
            );

            $fmap = array(
                'related_id' => 'id',
                'tax_class_id' => 'tax_code_id',
                'currency_id' => 'billing_currency',
                'exchange_rate' => 'billing_exchange_rate',
                'raw_cost_price' => 'paid_rate',
                'raw_list_price' => 'billing_rate',
                'raw_unit_price' => 'billing_rate',
                'cost_price' => 'paid_rate_usd',
                'list_price' => 'billing_rate_usd',
                'unit_price' => 'billing_rate_usd',
            );
            foreach($result->rows as $row) {
                $line = array('related_type' => 'Booking');
                $line['id'] = 'temp~' . rand(1000, 2000);
                $line['quantity'] = $row['quantity'] / 60;
                $line['name'] = $row['booking_category'];
                if(! $add_comments)
                    $line['name'] .= ': '.$row['name'];
                foreach($fmap as $k => $f) {
                    if(is_int($k)) $k = $f;
                    $line[$k] = $row[$f];
                }

                $new_groups[$grp_id]['lines'][$line['id']] = $line;
				$new_groups[$grp_id]['lines_order'][] = $line['id'];
                if($add_comments) {
                    $comment_id = 'comment~' . rand(1000, 2000);
                    $new_groups[$grp_id]['lines'][$comment_id] = array('related_type' => 'Notes', 'parent_id' => $line['id'], 'body' => $row['name'], 'is_comment' => 1);
                    $new_groups[$grp_id]['lines_order'][] = $comment_id;
				}
            }
        }

        return $new_groups;
    }

    static function get_service_parts_group($case_id) {
        $prods = new ListQuery('aCase', true, array('link_name' => 'products', 'parent_key' => $case_id));
        $prods->addField('~join.quantity', 'quantity');
        $result = $prods->runQuery();
        $new_groups = array();

        if($result || $result->total_count) {
            $grp_id = 'temp~'.(self::$grp_idx ++);
            $new_groups = array(
                $grp_id => array(
                    'id' => $grp_id,
                    'status' => 'Draft',
                    'lines' => array(),
                    'lines_order' => array(),
                    'adjusts' => array(),
                    'adjusts_order' => array(),
                    'group_type' => 'products',
                ),
            );

            $fmap = array(
                'quantity',
                'related_id' => 'id',
                'name',
                'purchase_name',
                'tax_class_id' => 'tax_code_id',
                'mfr_part_no' => 'manufacturers_part_no',
                'vendor_part_no',
                'currency_id',
                'exchange_rate',
                'description',
                'raw_cost_price' => 'cost',
                'raw_list_price' => 'list_price',
                'raw_unit_price' => 'purchase_price',
                'cost_price' => 'cost_usdollar',
                'list_price' => 'list_usdollar',
                'unit_price' => 'purchase_usdollar',
            );

            foreach($result->rows as $row) {
                $line = array('related_type' => 'ProductCatalog');
                $line['id'] = 'temp~' . rand(1000, 2000);
                foreach($fmap as $k => $f) {
                    if(is_int($k)) $k = $f;
                    $line[$k] = $row[$f];
                }
                $new_groups[$grp_id]['lines'][$line['id']] = $line;
                $new_groups[$grp_id]['lines_order'][] = $line['id'];
            }
        }

        return $new_groups;
    }

	static function init_record(RowUpdate &$upd, $input) {
		/** @var TallyUpdate $upd */

        $update = array();
        $inited = false;

        $from_quote_id = null;
        if(! empty($input['quote_id'])) {
            $from_quote_id = $input['quote_id'];
        } elseif (! empty($input['from_quote_id'])) {
            $from_quote_id = $input['from_quote_id'];
        }
        $from_so_id = null;
        if(! empty($input['salesorder_id'])) {
            $from_so_id = $input['salesorder_id'];
        } elseif (! empty($input['from_so_id'])) {
            $from_so_id = $input['from_so_id'];
        }

        if (empty($from_quote_id) && empty($from_so_id)) {

            $initFrom = array('opportunity_id' => 'Opportunity',
                'project_id' => 'Project', 'acase_id' => 'aCase');


            foreach($initFrom as $id_f => $bean_name) {
                if(! empty($input[$id_f])) {
                    $bean = ListQuery::quick_fetch_row($bean_name, $input[$id_f]);

                    if ($bean != null) {
                        if ($id_f == 'opportunity_id') {
                            $update['opportunity_id'] = $bean['id'];
                            $update['name'] = $bean['name'];
                            $update['partner_id'] = $bean['partner_id'];
                        }
						copy_custom_fields($id_f, $upd, $bean_name);
                    } else {
                        $update['name'] = $bean['name'];
                    }

					if (! empty($bean['billing_account_id'])) {
                        $account_data = self::init_from_account($bean['billing_account_id']);
                        $update = $update + $account_data;
                    } elseif (! empty($bean['account_id'])) {
                        $account_data = self::init_from_account($bean['account_id']);
                        $update = $update + $account_data;
                    }

                    if ($id_f == 'acase_id') {
                        $case_data = self::init_from_case($bean['id']);
                        $update = $update + $case_data;
                    }

                    if(isset($bean['currency_id'])) {
                        $update['currency_id'] = $bean['currency_id'];
                    }

					if ($bean_name == 'aCase' || $bean_name == 'Project') {
	                    //Update Line Items Groups
		                self::update_groups($upd, $bean['id'], $bean_name);
					}

                    $inited = true;
                    break;
                }
            }
        } else {
            $inited = true;
        }

        if(! $inited && ! empty($input['billing_account_id'])) {
            $account_data = self::init_from_account($input['billing_account_id']);
            array_extend($update, $account_data);
        }

        if (isset($input['parent_id']) && isset($input['parent_name'])
            && isset($input['parent_type']) && $input['parent_type'] == 'EventSessions') {

            $session = ListQuery::quick_fetch_row('EventSession', $input['parent_id']);

            if ($session != null) {
                $event = ListQuery::quick_fetch_row('Event', $session['event_id']);

                if ($event != null) {
                    $update['event_id'] = $event['id'];
                    $update['event_name'] = $event['name'];
                }
            }
        }

        if ($from_quote_id) {
			$quote = ListQuery::quick_fetch('Quote', $from_quote_id);
			if ($quote) {
	        	$quote_upd = RowUpdate::for_result($quote);
    	    	array_extend($update, self::init_from_tally($upd, $quote_upd));
			}
        } elseif($from_so_id) {
			$so = ListQuery::quick_fetch('SalesOrder', $from_so_id);
			if ($so) {
	        	$so_upd = RowUpdate::for_result($so);
    	        array_extend($update, self::init_from_tally($upd, $so_upd));

    	        if (! empty($input['final'])) {
		            require_once('modules/SalesOrders/SalesOrder.php');
    	        	$final_invoice = SalesOrder::get_final_invoice($so);

    	        	if (! $final_invoice) {
			            global $mod_strings;

			            $upd->set('final', 1);
			            $upd->set('name', $mod_strings['LBL_FINAL_MILESTONE_INVOICE_SUBJECT']);

			            self::add_intermediate_invoices($upd, $so_upd);
		            } else {
			            $upd->set('final', 0);
		            }
	            }

			}
        } else {
            $update['show_components'] = AppConfig::setting('company.quote_default_components', 'all');
        }

		if (!empty($upd->duplicate_of_id)) {
			$upd->set('invoice_date', null);
			$upd->set('due_date', null);
			$upd->set('cancelled', null);
			$upd->set('locked', null);
			$upd->set('shipping_stage', null);
		}

		foreach ($update as $k => $v) {
			if ($upd->isFieldEmpty($k) || $k == 'currency_id')
				$upd->set($k, $v);
		}

        if (! $upd->getField('terms'))
            $upd->set('terms', AppConfig::setting('company.invoice_default_terms', 'COD'));
        if (! $upd->getField('due_date'))
            $upd->set('due_date', Invoice::calc_due_date($upd->getField('terms')));
        if (! $upd->getField('invoice_date'))
			$upd->set('invoice_date', Invoice::calc_due_date());
		$so_stage = (!empty($so)) ? $so->getField('so_stage') : '';
		if (! $upd->getField('shipping_stage') || strpos($so_stage, 'Shipped and not Invoiced') !== false) {
			if (strpos($so_stage, 'Partially Shipped and not Invoiced') !== false)
				$shipping_stage = 'Partially Shipped';
			elseif (strpos($so_stage, 'Shipped and not Invoiced') !== false)
				$shipping_stage = 'Shipped';
			else
				$shipping_stage = $upd->getFieldDefaultValue('shipping_stage');
			$upd->set('shipping_stage', $shipping_stage);
		}
		if (!empty($_REQUEST['case_ids'])) {
			require_once 'modules/Booking/BookedHours.php';
			$ids = array_filter(explode(',', $_REQUEST['case_ids']));
			$groups = array();
			foreach ($ids as $cid) {
				$hours = self::get_booked_hours_group($cid, 'aCase');
				$groups = $groups + $hours;
			}
			$upd->replaceGroups($groups);
		}
    }

    static function update_groups(TallyUpdate &$invoice_upd, $from_id, $from_model) {
    	if($from_model == 'aCase' && ! AppConfig::setting('site.features.auto_add_case_line_items'))
    		return;
    	if($from_model == 'Project' && ! AppConfig::setting('site.features.auto_add_project_line_items'))
    		return;

        $hours = self::get_booked_hours_group($from_id, $from_model);
        $service_parts = array();
        if ($from_model == 'aCase')
            $service_parts = self::get_service_parts_group($from_id);

        $groups = array_merge($hours, $service_parts);
        if (sizeof($groups) > 0)
            $invoice_upd->replaceGroups($groups);
    }

    static function get_activity_status(RowUpdate $upd) {
        $status = null;

        if ($upd->getField('cancelled') &&  !$upd->getField('cancelled', null, true))
            $status = 'cancelled';

        if (!$upd->getField('locked') &&  $upd->getField('locked', null, true))
            $status = 'unlocked';

        return $status;
    }

	static function get_new_activity_status(RowUpdate $upd) {
        $status = 'created';

		if ($upd->getField('from_so_id')) {
			$status = array(
				'status' => 'created',
				'converted_to_type' => 'SalesOrders',
				'converted_to_id' => $upd->getField('from_so_id'),
			);
		} elseif ($upd->getField('from_quote_id')) {
			$status = array(
				'status' => 'created',
				'converted_to_type' => 'Quotes',
				'converted_to_id' => $upd->getField('from_quote_id'),
			);
		}


        return $status;
    }

	static function paid_status_subselect($colspec, $table, $for_order=false) {
		global $db, $timedate;
		$inv = AppConfig::setting("model.detail.Invoice.table_name");
		$canc_f = $db->quoteField('cancelled', $table);
		$amt_f = $db->quoteField('amount_due', $table);
		$date_f = $db->quoteField('due_date', $table);
		$db_dt = $timedate->get_gmt_db_datetime();
		$now = explode(' ', $timedate->handle_offset($db_dt, $timedate->get_db_date_time_format(), true));
		return "
			CASE WHEN $canc_f THEN 'cancelled'
				WHEN ABS($amt_f) <= 0.005 THEN 'paid'
				WHEN $date_f < '{$now[0]}' THEN 'overdue'
				ELSE 'due'
			END";
	}

	static function get_status_badge($cancelled, $amount) {
		if($cancelled) return 'cancelled';
	}

	static function portal_filter($lq, $contact_id, $account_id) {
		if ($account_id) {
			$clause  = ['field' => 'billing_account_id', 'value' => $account_id];
		} else {
			$clause = ['operator' => 'literal', 'value' => '0'];
		}
		$lq->addFilterClause($clause);
	}

	static function CalcCommissions(RowUpdate &$upd) {
		if(AppConfig::setting("company.commission_module") == 'Invoices') {
			$id = $upd->getPrimaryKeyValue();
			//Create Comm Record on new record
			require_bean('Commission');
			if($upd->new_record) {
				$comm = RowUpdate::blank_for_model('Commission');

				$comm->set([
					'invoice_id' => $id,
					'percentage' => 100,
					'assigned_user_id' => $upd->getField('assigned_user_id'),
					'modified_user_id' => $upd->getField('modified_user_id'),
				]);
				$GLOBALS['log']->fatal('sdfsdfsd');
				$comm->save();
			}
			$commperc = AppConfig::setting("company.commission_percentage");
			if (AppConfig::setting("company.commission_type") == 'GP') {
				$grossprofit = $upd->getField('gross_profit');
			} else {
				$grossprofit = $upd->getField('pretax');
			}
			if(!is_numeric($grossprofit))
				$grossprofit = 0;
			if(!is_numeric($commperc))
				$commperc = 0;
			$commval = ($commperc*$grossprofit)/100;

			$lq = new ListQuery('Commission');
			$lq->addSimpleFilter('invoice_id', $id, '=');
			$res = $lq->fetchAll();

			foreach($res as $comm_user) {
				$update = array();
				$update['commission'] = ($comm_user->getField('percentage')*$commval)/100;
				$update['commission_total'] = $commval;
				$comm_user_update = RowUpdate::for_result($comm_user);
	            $comm_user_update->set($update);
	            $comm_user_update->setRelatedData('updating_so_comission', true);
	            $comm_user_update->save();
			}
		}
	}

	static function listupdate_perform($mu, $perform, &$listFmt, &$list_result, $uids) {
		if ($perform == 'SendPDFInvoices') {
			$ids = array();
			while(! $list_result->failed) {
				foreach($list_result->getRowIndexes() as $idx) {
					$row = $list_result->getRowResult($idx);
					$ids[] = $module_id = $row->getField('id');
				}
				if($list_result->page_finished)
					break;
				$listFmt->pageResult($list_result, true);
			}
			$_REQUEST['list_uids'] = join(';', $ids);
			
			return array(
				'perform', 
				array(
					'module' => 'Invoice',
					'action' => 'SendPDFInvoices',
				),
			);
		}
	}

	static function cancel_editable(&$row, &$flag, &$val) {
		if ($row->getField('cancelled')) {
			$val = true;
			return;
		}
		$lq = new ListQuery('Invoice', ['id'], ['link_name' => 'payments', 'parent_key' => $row->getField('id')]);
		$n = $lq->fetchResultCount();
		$val = !$n;
	}
	
	static function restrict_locked_massupdate($mu, $perform, RowUpdate $upd, $input) {
		static $restrictCount = 0;
		if (!$upd->getField('locked')) return true;
		$restrictionApplied = false;
		foreach (self::$massupdate_restricted as $f) {
			if ($upd->getFieldUpdated($f)) {
				$upd->resetField($f);
				$restrictionApplied = true;
			}
		}
		if ($restrictionApplied) {
			$restrictCount++;
			$msg = sprintf(translate('LBL_MASSUPDATE_LOCK_WARNING', 'Invoice'), $restrictCount);
			add_flash_message($msg, 'error', 'invoice_massupdate_restriction');
		}
	}

}
?>
