<?php

require_once 'modules/Quotes/PDFTemplatesProvider.php';

class InvoicePDFTemplateProvider extends QuotePDFTemplateProvider
{

	protected function __construct($type)
	{
		parent::__construct($type);
	}

	public static function template_types()
	{
		return array(
			'invoice' => translate('LBL_PDF_INVOICE_TITLE', 'Invoice'),
			'statement' => translate('LBL_PDF_STATEMENT_TITLE', 'Invoice'),
		);
	}

	public static function template_skeleton($type)
	{
		$ret = array('sections' => array(
			array(
				'name' => 'header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_HEADER',
				'height' => 150,
				'elements' => array (),
			),
			array(
				'name' => 'group_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_GROUP_HEADER',
				'height' => 150,
				'elements' => array (),
			),
			array(
				'name' => 'group_items',
				'type' => 'table',
				'title' => 'LBL_SKEL_GROUP_ITEMS',
				'columns' => array (),
			),
			array(
				'name' => 'group_totals_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_GROUP_TOTALS_HEADER',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'group_totals',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_GROUP_TOTALS',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'group_footer',
				'data_id' => 'group_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_GROUP_FOOTER',
				'height' => 150,
				'elements' => array (),
			),
			array(
				'name' => 'quote_totals_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_QUOTE_TOTALS_HEADER',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'quote_totals',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_QUOTE_TOTALS',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'payments_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_PAYMENTS_HEADER',
				'height' => 20,
				'elements' => array (),
			),
			array(
				'name' => 'payments_table',
				'type' => 'table',
				'title' => 'LBL_SKEL_PAYMENTS_TABLE',
				'columns' => array (),
			),
			array(
				'name' => 'payments_footer',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_PAYMENTS_FOOTER',
				'height' => 20,
				'elements' => array (),
				'data_id' => 'payments_header',
			),
			array(
				'name' => 'notes',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_NOTES',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'terms',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_TERMS',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'footer',
				'data_id' => 'header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_FOOTER',
				'height' => 40,
				'elements' => array (),
			),
		));
		foreach ($ret['sections'] as &$sec) $sec['title'] = translate($sec['title'], 'Invoice');
		if ($type != 'statement') {
			unset($ret['sections']['payments_header']);
			unset($ret['sections']['payments_table']);
			unset($ret['sections']['payments_footer']);
		}
		$ret['sections'] = array_values($ret['sections']);
		return $ret;
	}

	public static function template_sample_data($type)
	{
		$datefmt = AppConfig::setting('locale.defaults.date_format');
		$data = array(
			'header' => array(
				'todays_date' => array(translate('LBL_TODAYS_DATE'), date($datefmt), ),
				'number' => array(translate('LBL_SAMPLE_HEADER_NUMBER_LABEL', 'Invoice'), 'I2013-001'),
				'terms' => array(translate('LBL_SAMPLE_HEADER_TERMS_LABEL', 'Invoice'), translate('LBL_SAMPLE_HEADER_TERMS_VALUE', 'Invoice'), ),
				'date' => array(translate('LBL_SAMPLE_HEADER_DATE_LABEL', 'Invoice'), '2014-11-05'),
				'due_date' => array(translate('LBL_SAMPLE_HEADER_DUE_DATE_LABEL', 'Invoice'), '2014-11-05'),
				'sales_person' => array(translate('LBL_SAMPLE_HEADER_SALES_PERSON_LABEL', 'Invoice'), translate('LBL_SAMPLE_HEADER_SALES_PERSON_VALUE', 'Invoice'), ),
				'tax_info' => array(translate('LBL_SAMPLE_HEADER_TAX_INFO_LABEL', 'Invoice'), ''),
				'billing_address' => array(translate('LBL_SAMPLE_HEADER_BILLING_ADDRESS_LABEL', 'Invoice'), translate('LBL_SAMPLE_HEADER_BILLING_ADDRESS_VALUE', 'Invoice'), ),
				'billing_address_contact_last' => array(translate('LBL_SAMPLE_HEADER_BILLING_ADDRESS_CONTACT_LAST_LABEL', 'Quotes'), translate('LBL_SAMPLE_HEADER_BILLING_ADDRESS_VALUE', 'Quotes'), ),
				'shipping_address' => array(translate('LBL_SAMPLE_HEADER_SHIPPING_ADDRESS_LABEL', 'Invoice'), translate('LBL_SAMPLE_HEADER_SHIPPING_ADDRESS_VALUE', 'Invoice'), ),
				'shipping_address_contact_last' => array(translate('LBL_SAMPLE_HEADER_SHIPPING_ADDRESS_CONTACT_LAST_LABEL', 'Quotes'), translate('LBL_SAMPLE_HEADER_SHIPPING_ADDRESS_VALUE', 'Quotes'), ),
				'purchase_order' => array(translate('LBL_SAMPLE_HEADER_PURCHASE_ORDER_LABEL', 'Invoice'), translate('LBL_SAMPLE_HEADER_PURCHASE_ORDER_VALUE', 'Invoice'), ),
				'description' => array(translate('LBL_SAMPLE_HEADER_DESCRIPTION_LABEL', 'Invoice'), translate('LBL_SAMPLE_HEADER_DESCRIPTION_VALUE', 'Invoice'), ),
				'standard_terms' => array(translate('LBL_SAMPLE_HEADER_STANDARD_TERMS_LABEL', 'Invoice'), ''),
				'paypal_button' => array(translate('LBL_SAMPLE_PAYPAL_BUTTON', 'Invoice'), 'PayPal'),
			),
			'group_header' => array(
				'group_name' => array(translate('LBL_SAMPLE_GROUP_HEADER_GROUP_NAME_LABEL', 'Invoice'), translate('LBL_SAMPLE_GROUP_HEADER_GROUP_NAME_VALUE', 'Invoice'), ),
				'status' => array(translate('LBL_SAMPLE_GROUP_HEADER_STATUS_LABEL', 'Invoice'), translate('LBL_SAMPLE_GROUP_HEADER_STATUS_VALUE', 'Invoice'), ),
			),
			'group_totals' => array(
				'total_name' => array(translate('LBL_SAMPLE_GROUP_TOTALS_TOTAL_NAME_LABEL', 'Invoice'), translate('LBL_SAMPLE_GROUP_TOTALS_TOTAL_NAME_VALUE', 'Invoice'), ),
				'total_value' => array(translate('LBL_SAMPLE_GROUP_TOTALS_TOTAL_VALUE_LABEL', 'Invoice'), translate('LBL_SAMPLE_GROUP_TOTALS_TOTAL_VALUE_VALUE', 'Invoice'), ),
			),
			'quote_totals' => array(
				'total_name' => array(translate('LBL_SAMPLE_QUOTE_TOTALS_TOTAL_NAME_LABEL', 'Invoice'), translate('LBL_SAMPLE_QUOTE_TOTALS_TOTAL_NAME_VALUE', 'Invoice'), ),
				'total_value' => array(translate('LBL_SAMPLE_QUOTE_TOTALS_TOTAL_VALUE_LABEL', 'Invoice'), translate('LBL_SAMPLE_QUOTE_TOTALS_TOTAL_VALUE_VALUE', 'Invoice'), ),
			),
			'group_items' => array(
				'number' => array(translate('LBL_SAMPLE_GROUP_ITEMS_NUMBER_LABEL', 'Invoice'), '1'),
				'product' => array(translate('LBL_SAMPLE_GROUP_ITEMS_PRODUCT_LABEL', 'Invoice'), translate('LBL_SAMPLE_GROUP_ITEMS_PRODUCT_VALUE', 'Invoice'), ),
				'quantity' => array(translate('LBL_SAMPLE_GROUP_ITEMS_QUANTITY_LABEL', 'Invoice'), '2'),
				'listprice' => array(translate('LBL_SAMPLE_GROUP_ITEMS_LISTPRICE_LABEL', 'Invoice'), '10.0'),
				'unitprice' => array(translate('LBL_SAMPLE_GROUP_ITEMS_UNITPRICE_LABEL', 'Invoice'), '20.0'),
				'extprice' => array(translate('LBL_SAMPLE_GROUP_ITEMS_EXTPRICE_LABEL', 'Invoice'), '40.0'),
				'discount' => array(translate('LBL_SAMPLE_GROUP_ITEMS_DISCOUNT_LABEL', 'Invoice'), '5%'),
				'mfr_part_no' => array(translate('LBL_SAMPLE_GROUP_ITEMS_MFR_PART_NO_LABEL', 'Invoice'), '123456'),
				'serial_numbers' => array(translate('LBL_SAMPLE_GROUP_ITEMS_SERIAL_NUMBERS_LABEL', 'Invoice'), '748905'),
			),
			'payments_header' => array(
				'payments_total' => array(translate('LBL_SAMPLE_PAYMENTS_HEADER_PAYMENTS_TOTAL_LABEL', 'Invoice'), '10.0'),
				'amount_due' => array(translate('LBL_SAMPLE_PAYMENTS_HEADER_AMOUNT_DUE_LABEL', 'Invoice'), '30.0'),
			),
			'payments_table' => array(
				'payment_date' => array(translate('LBL_SAMPLE_PAYMENTS_TABLE_PAYMENT_DATE_LABEL', 'Invoice'), '2014-11-05'),
				'payment_id' => array(translate('LBL_SAMPLE_PAYMENTS_TABLE_PAYMENT_ID_LABEL', 'Invoice'), 'P-1235'),
				'payment_type' => array(translate('LBL_SAMPLE_PAYMENTS_TABLE_PAYMENT_TYPE_LABEL', 'Invoice'), translate('LBL_SAMPLE_PAYMENTS_TABLE_PAYMENT_TYPE_VALUE', 'Invoice'), ),
				'customer_reference' => array(translate('LBL_SAMPLE_PAYMENTS_TABLE_CUSTOMER_REFERENCE_LABEL', 'Invoice'), '123456'),
				'amount' => array(translate('LBL_SAMPLE_PAYMENTS_TABLE_AMOUNT_LABEL', 'Invoice'), '20.0'),
				'allocated' => array(translate('LBL_SAMPLE_PAYMENTS_TABLE_ALLOCATED_LABEL', 'Invoice'), '10.0'),
			),
		);
		self::injectProductSampleData($data['group_items']);
		$ret = array();
		foreach ($data as $k => $v) {
			$ret[$k] = array();
			foreach ($v as $kk => $vv) {
				$ret[$k][$kk] = array('label' => $vv[0], 'value' => $vv[1]);
			}
		}
		return $ret;
	}


	public static function create($type)
	{
		return new InvoicePDFTemplateProvider($type);
	}

	protected function iterateExtraFooter(PDFTemplatePrinter $p)
	{

		$section = $this->findSection('payments_header');
		$p->renderSection($section, $this->data['payments_header']);

		$section = $this->findSection('payments_table');
		$p->renderSection($section, $this->data['payments_table']);

		$section = $this->findSection('payments_footer');
		$p->renderSection($section, $this->data['payments_header']);

		parent::iterateExtraFooter($p);
	}

	public function getFileName() {
		$filename_pfx = translate($this->type == 'statement' ? 'LBL_PDF_STATEMENT_TITLE' : 'LBL_PDF_INVOICE_TITLE', 'Invoice');
		$filename = $filename_pfx . '_' . $this->row->getField('prefix');
		$filename .= $this->row->getField('invoice_number') . '_' . $this->row->getField('billing_account_name');
		$filename = preg_replace('~\.+$~', '', trim($filename));
		$filename = $filename . '.pdf';
		return $filename;
	}

	public function getNoteTitle()
	{
		return translate('LBL_PREPARED_INVOICE', 'Invoice');
	}

	public function getEmailTitle()
	{
		return translate('LBL_EMAILED_INVOICE', 'Invoice');
	}
	
	public function getSignedDocumentTitle() {
		$prefix =
			$this->row
			? '[' . $this->row->getField('prefix') . $this->row->getField('invoice_number') . '] '
			: '';

		return $prefix . $this->getNoteTitle();
	}

	protected function createData()
	{
		$row =& $this->row;
		$user = new User();
		$user->retrieve($row->getField('assigned_user_id'), false);
		$terms = AppConfig::setting("lang.lists.current.app.terms_dom", array());
		$currency =& $this->get_currency();
		$tax_information = trim($this->row->getField('tax_information'));
		$datefmt = AppConfig::setting('locale.defaults.date_format');
		if($tax_information) $tax_information = "\n" . $tax_information;

		$data = array(
			'todays_date' => date($datefmt),
			'number' => $row->getField('prefix') . $row->getField('invoice_number'),
			'terms' => array_get_default($terms, $row->getField('terms'), $row->getField('terms')),
			'date' => $row->getField('invoice_date', '', true),
			'due_date' => $row->getField('due_date', '', true),
			'sales_person' => $row->getField('assigned_user_name'),
			'tax_info' => from_html(AppConfig::setting('company.tax_information', '')),
			'billing_address' => $this->_address('billing_') . $tax_information,
			'billing_address_contact_last' => $this->_address('billing_c') . "\n" . $tax_information,
			'shipping_address' => $this->_address('shipping_'),
			'shipping_address_contact_last' => $this->_address('shipping_c'),
			'description' => $row->getField('description'),
			'purchase_order' => $row->getField('purchase_order_num'),
			'amount_due' => $this->currency_format($this->row->getField('amount_due'), $currency->id),
			'standard_terms' => trim(AppConfig::setting('company.std_invoice_terms', '')),
			'paypal_button' => $this->getPayPalButtonCode()
		);

		$data = array_merge($this->header_data, $data);
		$this->data['header'] = $data;

		$this->changeFieldType('header', 'paypal_button', 'html');

		$this->createGroupData();
		$this->createPaymentsData();
	}

	private function getPayPalButtonCode() {
		$paypal_link = AppConfig::setting('company.paypal_link');
		$html = '';

		if ($this->row->getField('amount_due') > 0 && ! empty($paypal_link)) {
			$currency = ListQuery::quick_fetch('Currency', $this->row->getField('currency_id'), ['iso4217', 'symbol']);

			$currency_code = $currency->getField('iso4217');
			$currency_symbol = $currency->getField('symbol');

			$item_name = $this->row->getField('prefix') . $this->row->getField('invoice_number') .'-'. $this->row->getField('name') . ' (' . $currency_symbol . $this->row->getField('amount_due') .')';
			$item_name = urlencode($item_name);

			$link = $paypal_link .
				'&amount=' . $this->row->getField('amount_due') .
				'&currency_code='.$currency_code .
				'&item_name=' . $item_name;

			$html = '<a href="'.$link.'"  target="_blank" title="PayPal"><img border="0" width="195" height="74" src="include/images/paypal_button.png" alt="PayPal"></a>';
		}

		return $html;
	}

	private function changeFieldType($section, $field, $new_type) {
		$sections = $this->layout['sections'];

		foreach ($sections as $section_index => $item) {
			if ($item['name'] != $section)
				continue;

			foreach ($item['elements'] as $elem_index => $element) {
				if ($element['type'] != 'field')
					continue;

				if ($element['field'] == $field) {
					$this->layout['sections'][$section_index]['elements'][$elem_index]['type'] = $new_type;
				}
			}
		}
	}

	protected function createPaymentsData()
	{
		global $app_list_strings;
		$currency = $this->get_currency();
		$rate = $this->row->getField('exchange_rate');

		$lq = new ListQuery('Invoice', true, array('link_name' => 'payments', 'parent_key' => $this->row->getField('id')));
		$lq->addField('~join.amount', 'allocated');
		$res = $lq->runQuery();
		$payments = $res->getRows();

		$lq = new ListQuery('CreditNote');
		$lq->addSimpleFilter('invoice_id', $this->row->getField('id'));
		$lq->addSimpleFilter('apply_credit_note', '', 'true');
		$lq->addSimpleFilter('cancelled', 0);
		$res = $lq->runQuery();
		$notes = $res->getRows();

		$data = array();
		$payments_total = 0;
		global $timedate;
		foreach ($payments as $payment) {
			$data[] = array(
				'payment_date' => $timedate->to_display_date($payment['payment_date'], false),
				'payment_id' => $payment['prefix'] . $payment['payment_id'],
				'amount' => $this->currency_format($payment['amount'], $currency->id),
				'allocated' => $this->currency_format($payment['allocated'], $currency->id),
				'payment_type' => $app_list_strings['payment_type_dom'][$payment['payment_type']],
				'customer_reference' => $payment['customer_reference'],
			);
			$payments_total += $payment['allocated'];
		}

		foreach ($notes as $note) {
			$data[] = array(
				'payment_date' => $timedate->to_display_date($note['due_date'], false),
				'payment_id' => $note['prefix'] . $note['credit_number'],
				'amount' => $this->currency_format($note['amount'], $note['currency_id']),
				'allocated' => $this->currency_format($note['amount_usdollar'] * $rate, $currency->id),
				'payment_type' => translate('LBL_PAYMENT_TYPE_CREDIT', 'Invoice'),
				//'customer_reference' => $payment['customer_reference'],
			);
			$payments_total += $note['amount_usdollar'] * $rate;
		}

		$header_data = array(
			'payments_total' => $this->currency_format($payments_total, $currency->id),
			'amount_due' => $this->currency_format($this->row->getField('amount_due'), $currency->id),
		);

		$this->data['payments_header'] = $header_data;
		$this->data['payments_table'] = $data;

	}

	public static function get_extra_sources() {
		return array(
			array(
				'module' => 'Opportunities',
				'model' => 'Opportunity',
				'prefix' => 'opportunity',
				'label' => 'LBL_OPPORTUNITY',
			),
			array(
				'module' => 'SalesOrders',
				'model' => 'SalesOrder',
				'prefix' => 'from_so',
				'label' => 'LBL_FROM_SO_NUMBER',
			),
			array(
				'module' => 'Quotes',
				'model' => 'Quote',
				'prefix' => 'from_quote',
				'label' => 'LBL_FROM_QUOTE_NUMBER',
			),
			array(
				'module' => 'Accounts',
				'model' => 'Account',
				'prefix' => 'billing_account',
				'label' => 'LBL_BILLING_ACCOUNT',
			),
			array(
				'module' => 'Accounts',
				'model' => 'Account',
				'prefix' => 'shipping_account',
				'label' => 'LBL_SHIPPING_ACCOUNT',
			),
			array(
				'module' => 'Contacts',
				'model' => 'Contact',
				'prefix' => 'billing_contact',
				'label' => 'LBL_BILLING_CONTACT',
			),
			array(
				'module' => 'Contacts',
				'model' => 'Contact',
				'prefix' => 'shipping_contact',
				'label' => 'LBL_SHIPPING_CONTACT',
			),
			array(
				'module' => 'ShippingProviders',
				'model' => 'ShippingProvider',
				'prefix' => 'shipping_provider',
				'label' => 'LBL_SHIPPING_PROVIDER',
			),
		);
	}

	function get_custom_merge() {
		if( ($cid = $this->get_field('currency_id')) ) {
			$curr = ListQuery::quick_fetch_row('Currency', $cid, array('terms_pdf'));
			if($curr && ($fname = $curr['terms_pdf'])) {
				require_once('include/upload_file.php');
				$path = UploadFile::get_path($fname);
				if(file_exists($path) && substr($path, -4) == '.pdf')
					return array('append' => $path);
			}
		}
	}

}

