<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

require_once('modules/Currencies/Currency.php');
require_once('include/layout/FieldFormatter.php');


class CardPaymentForm {
	var $errors;
	var $invoice;
	var $gateway;
	var $processor;
	var $request;

	public function __construct($req) {
		$fmt = new FieldFormatter('html', 'editview');
		$all_fields = $this->getFields();
		$fields = call_user_func_array('array_merge', $all_fields);
		$unformatted = $fmt->unformatRow($fields, $req);
		$this->request = $unformatted;
	}

	function init() {
		$inv_id = $this->input('invoice_id');
		if ($inv_id)
			$this->invoice = ListQuery::quick_fetch('Invoice', $inv_id);
		if (empty($this->invoice))
			sugar_die('Invalid invoice ID');
		include_once('include/CreditCards/autoload.php');
		$this->gateway = AppConfig::setting('company.payment_gateway');
		$this->processor = CreditCardProcessor::factory($this->gateway);
		if (! $this->processor)
			sugar_die('Please check system settings for credit card processing');
	}

	public function processRequest() {
		$this->init();
		if ($this->input('process'))
			if ($this->process()) {
				return;
			}
		$this->render();
	}

	protected function process() {
		global $pageInstance;

		$currency = new Currency();
		$currency->retrieve($this->invoice->getField('currency_id'));
		$merchant_currency = new Currency();
		$merchant_currency->retrieve(AppConfig::setting('company.merchant_currency'));

		$process = $this->processor;
		$amount = $this->input('payment_amount');
		$invoice_id = $this->invoice->getField('id');
		$invoice_amount_due = $this->invoice->getField('amount_due');
		$invoice_amount_due_usd = $this->invoice->getField('amount_due_usdollar');
		$invoice_cid = $this->invoice->getField('currency_id');
		$invoice_exrt = $this->invoice->getField('exchange_rate');

		$process->param_login = AppConfig::setting('company.merchant_login_' . $this->gateway);
		$process->param_password = AppConfig::setting('company.merchant_password_' . $this->gateway);
		$account = ListQuery::quick_fetch('Account', $this->invoice->getField('billing_account_id'));
		$process->param_amount = $amount;
		$process->param_firstName = $this->input('contact_first_name');
		$process->param_lastName = $this->input('contact_last_name');
		$process->param_companyName = $account->getField('name');
		$process->param_email = $account->getField('email1');
		$process->param_phoneNumber = $this->input('contact_address_phone');
		$process->param_address = $this->input('contact_address_street');
		$process->param_city = $this->input('contact_address_city');
		$process->param_state = $this->input('contact_address_state');
		$process->param_zip = $this->input('contact_address_postalcode');
		$process->param_country = $this->input('contact_address_country');
		$process->param_invoiceEmail = $this->input('stripe_invoice_email');
		$process->param_currency = $merchant_currency->iso4217;
		$process->param_invoice_id = $invoice_id;
		$process->param_invoice_name = $this->invoice->getField('name');

		try {
			$data = $process->process();
			$invoiceId = $process->decodeResponse($data);
			$format = translate('LBL_CC_INVOICE_SENT', 'Invoice');
			add_flash_message(sprintf($format, $this->input('stripe_invoice_email')), 'info');
			$pageInstance->add_js_literal('SUGAR.popups.close(); SUGAR.util.navigateTo("index.php?module=Invoice&action=DetailView&record=' . $invoice_id . '");</script>');

			return true;
		} catch (CreditCardProcessorException $e) {
			$m = $e->getMessage();
			if ($m)
				$this->errors[] = $m;
			foreach ($e->getMessages() as $m) {
				$this->errors[] = $m;
			}
		}
	}

	protected function render() {
		if ($this->errors) {
			$msgs = [];
			foreach($this->errors as $error)
				$msgs[] = [to_html($error), 'error'];
			echo display_flash_messages_default($msgs, false);
			$this->errors = [];
		}

		$all_fields = $this->getFields();
		$layout = $this->getLayout($all_fields);
		$result = new RowResult;
		$result->row = array();
		$fields = call_user_func_array('array_merge', $all_fields);

		$account = ListQuery::quick_fetch('Account', $this->invoice->getField('billing_account_id'));
		if(! $account) $account = new RowResult();
		$contact = ListQuery::quick_fetch('Contact', $this->invoice->getField('billing_contact_id'));
		if(! $contact) $contact = new RowResult();

		$currency = new Currency();
		$currency->retrieve($this->invoice->getField('currency_id'));
		$merchant_currency = new Currency();
		$merchant_currency->retrieve(AppConfig::setting('company.merchant_currency'));

		$amount_due = $this->invoice->getField('amount_due');
		if ($merchant_currency->id != $currency->id) {
			$symbol = $merchant_currency->symbol;
			$total_usd = $this->invoice->getField('amount_due_usdollar');
			$due_usd = $this->invoice->getField('amount_due_usdollar');
			$merchant_total = $merchant_currency->convertFromDollar($total_usd);
			$merchant_due = $merchant_currency->convertFromDollar($due_usd);
			$result->row['converted_amount'] = $merchant_total;
			$result->row['converted_amount_due'] = $merchant_due;
			$amount_due = $merchant_due;
		}

		foreach ($fields as $f => $spec) {
			$fields[$f]['required'] = false;
			$def = null;
			if($f == 'invoice_currency_id')
				$def = $currency->id;
			else if($f == 'converted_currency')
				$def = $merchant_currency->name;
			else if($f == 'converted_currency_id')
				$def = $merchant_currency->id;
			else if(substr($f, 0, 8) == 'invoice_')
				$def = $this->invoice->getField(substr($f, 8));
			else if($f == 'stripe_invoice_email')
				$def = $account->getField('email1');
			else if($spec['type'] == 'address') {
				$pfx = array_get_default($spec['source'], 'prefix', '');
				foreach($spec['source']['fields'] as $f2) {
					$fn = $pfx.$f2;
					$def = $this->invoice->getField('billing_' . $f2);
					if($f2 == 'address_phone' && empty($def)) {
						$def = $account->getField('phone_office');
					}
					else if($f2 == 'address_email' && empty($def)) {
						$def = $account->getField('email1');
					}
					$def = array_get_default($result->row, $fn, $def);
					$result->row[$fn] = $this->input($fn, $def);
				}
				continue;
			}
			else if(substr($f, 0, 8) == 'contact_') {
				$def = $contact->getField(substr($f, 8));
			}
			else if($f == 'payment_amount') {
				// FIXME - decimal places?
				$def = $amount_due;
			}
			$def = array_get_default($result->row, $f, $def);
			$result->row[$f] = $this->input($f, $def);
		}
		$result->fields = $fields;

		require_once('include/layout/forms/FormGenerator.php');
		$model = new ModelDef('Invoice');
		$model->addDisplayFieldDefinitions($result->fields);
		$gen = FormGenerator::html_form($model, $layout, 'cardPaymentForm');
		$gen->getFormObject()->addHiddenFields(
			array(
				'invoice_id' => $this->input('invoice_id'),
				'module' => 'Invoice',
				'action' => 'PopupCardPayment',
				'process' => '1',
			)
		);
		$fmt = new FieldFormatter('html', 'editview');
		$fmt->formatRowResult($result);
		$gen->renderForm($result);
		echo $gen->getResult();
		$gen->getLayout()->addScriptLiteral(
			'SUGAR.popups.initContent({width: 600, fill: true});');
		//$gen->getLayout()->addScriptInclude('modules/Invoice/script.js');
		//$gen->getLayout()->addCSSInclude('modules/Invoice/styles.css');
		$gen->exportIncludes();
	}

	protected function input($name, $default = null) {
		return array_get_default($this->request, $name, $default);
	}

	protected function getFields() {
		$fields = array(
			array(
				'invoice_name' => array(
					'vname' => 'LBL_CC_INVOICE_SUBJECT',
					'type' => 'varchar',
					'colspan' => 2,
				),
				'invoice_amount' => array(
					'vname' => 'LBL_CC_INVOICE_TOTAL',
					'type' => 'currency',
					'currency_id' => 'invoice_currency_id',
					'show_symbol' => true,
				),
				'invoice_amount_due' => array(
					'vname' => 'LBL_CC_INVOICE_DUE',
					'type' => 'currency',
					'currency_id' => 'invoice_currency_id',
					'show_symbol' => true,
				),
				'converted_amount' => array(
					'vname' => 'LBL_CONVERTED_AMOUNT',
					'type' => 'currency',
					'currency_id' => 'converted_currency_id',
					'show_symbol' => true,
				),
				'converted_amount_due' => array(
					'vname' => 'LBL_CONVERTED_AMOUNT_DUE',
					'type' => 'currency',
					'currency_id' => 'converted_currency_id',
					'show_symbol' => true,
				),
				'converted_currency' => array(
					'vname' => 'LBL_CURRENCY',
					'type' => 'ref',
					'bean_name' => 'Currency',
					'editable' => false,
				),
				'converted_currency_id' => array(
					'type' => 'ref_id',
					'hidden' => true,
				),
				'invoice_currency_id' => array(
					'type' => 'ref_id',
					'hidden' => true,
				),
				'payment_amount' => array(
					'vname' => 'LBL_CC_PAYMENT_AMOUNT',
					'type' => 'currency',
					'required' => true,
					'editable' => true,
				),
			),
			array(
				'stripe_invoice_email' => array(
					'vname' => 'LBL_INVOICE_EMAIL',
					'type' => 'varchar',
					'editable' => true,
					'required' => true,
					'width' => 100,
				),
			),
			array(
				'contact_first_name' => array(
					'vname' => 'LBL_FIRST_NAME',
					'vname_module' => 'Contacts',
					'type' => 'varchar',
					'width' => 20,
					'editable' => true,
					'required' => true,
				),
				'contact_last_name' => array(
					'vname' => 'LBL_LAST_NAME',
					'vname_module' => 'Contacts',
					'type' => 'varchar',
					'width' => 20,
					'editable' => true,
					'required' => true,
				),
				'contact_address' => array(
					'vname' => 'LBL_BILLING_ADDRESS',
					'type' => 'address',
					'editable' => true,
					'required' => true,
					'source' => [
						'type' => 'address',
						'prefix' => 'contact_',
						'fields' => [
							'address_street',
							'address_city',
							'address_state',
							'address_country',
							'address_postalcode',
							'address_phone',
							'address_email',
						]
					],
					'colspan' => 2,
				),
			),
		);
		return $fields;
	}

	protected function getLayout($fields) {
		$layout = array(
			'type' => 'config',
			'editable' => true,
			'buttons_position' => 'bottom',
			'form_buttons' => $this->getFormButtons(),
			'sections' => array(
				array(
					'vname' => 'LBL_INVOICE_INFO_TITLE',
					'label_position' => 'top',
					'columns' => 2,
					'elements' => array_keys(array_filter($fields[0], function($f) { return empty($f['hidden']); })),
				),
				array(
					'vname' => 'LBL_INVOICE_EMAIL_TITILE',
					'label_position' => 'top',
					'columns' => 1,
					'elements' => array_keys($fields[1]),
				),
				array(
					'vname' => 'LBL_CUSTOMER_INFO_TITLE',
					'label_position' => 'top',
					'columns' => 2,
					'elements' => array_keys($fields[2]),
				),
			),
		);
		return $layout;
	}

	protected function getFormButtons() {
		$buttons = array(
			'submit' => array(
				'icon' => 'icon-accept',
				'label' => translate('LBL_SUBMIT_CC_PAYMENT', 'Invoice'),
				'params' => array(),
				'async' => true,
				'align' => 'right',
				'perform' => 'SUGAR.ui.sendForm({FORM}, {no_redirect: true}, {}, false, true); return false;',
			),
		);
		return $buttons;
	}

}


$form = new CardPaymentForm($_REQUEST);
$form->processRequest();
return;

