<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

require_once('modules/Workflow/WFProcessorBase.php');

class EmailInvoiceProc extends WFProcessorBase {
	var $related_model = 'Email';

	function process(RowUpdate $rowUpdate) {
		global $timedate;
		require_once('include/utils/html_utils.php');
		require_once('modules/EmailTemplates/TemplateParser.php');
		require_once('modules/Emails/utils.php');
		require_once('modules/PDFTemplates/PDFTemplatePrinter.php');
		require_once('modules/PDFTemplates/utils.php');

		$send_from_system = ! AppConfig::setting('notify.send_from_assigning_user');
		$system_from_addr = AppConfig::setting('notify.from_address', '');
		$system_from_name = AppConfig::setting('notify.from_name', '');
		
		$module = $rowUpdate->getModuleDir();
		$rowId = $rowUpdate->getPrimaryKeyValue();

		require_bean('Email');
		$seedEmail = RowUpdate::blank_for_model('Email');
		$recipChk = $this->combineValuesWithAlt($rowUpdate, $seedEmail, ['contact', 'contact_id'], $this->getParams());
		$recipId = array_get_default($recipChk, 'contact_id');
		if (! $recipId)
			$recipId = $rowUpdate->getField('billing_contact_id');

		if(! $recipId) {
			throw new WFProcessingError('Cannot email invoice: Contact not defined');
		}

		$acctId = $rowUpdate->getField('billing_account_id');

		$initiator = new User;
		$initiator->retrieve($this->getUserId());
		
		$from_name = $this->getParam('from_name');
		$from_addr = $this->getParam('from_addr');
		if (empty($from_addr)) {
			if($send_from_system || empty($initiator->email1)) {
				if(empty($from_name)) $from_name = $system_from_name;
				$from_addr = $system_from_addr;
			}
			else {
				if(empty($from_name)) $from_name = $initiator->name;
				$from_addr = $initiator->email1;
			}
		}
		$cc_mbox_id = $this->getParam('cc_mailbox_id');
		$mbox = null;
		if (!empty($cc_mbox_id)) {
			$mbox = ListQuery::quick_fetch('EmailPOP3', $cc_mbox_id, array('email'));
		}

		$pdftplId = $this->getParam('pdf_template_id');
		if(! $pdftplId) {
			throw new WFProcessingError('Cannot email invoice: PDF template not defined');
		}
		$printer = new PDFTemplatePrinter($pdftplId, $rowId);
		$pdf = $printer->render();
		$pdf->filename = filename_safe_string($printer->getFileName());
		$pdfId = $printer->create_note($pdf, $pdf->filename, false);

		$email = new Email();
		$email->from_name = $from_name;
		$email->from_addr = $from_addr;			
		$email->type = 'sent';
		$email->isread = 'read';
		$email->parent_type = $module;
		$email->parent_id = $rowId;
		$email->contact_id = $recipId;
		$email->account_id = $acctId;
		$email->assigned_user_id = $initiator->id;
		$email->set_created_by = $email->update_modified_by = false;
		$email->created_by = $email->assigned_user_id;
		$email->modified_user_id = $email->assigned_user_id;
		$email->send_from_system = $send_from_system;
		$email->date_start = $timedate->to_display_date_time(gmdate('Y-m-d H:i:s'));

		$objects = [
			$module => $rowId,
			'Records' => $rowId,
			'Contacts' => $recipId,
			'Recipients' => $recipId,
		];
		$params = [
			'disable_cache' => true,
			'user' => $initiator,
			'aliases' => [],
			'model_aliases' => [
				'Records' => [$module, 'record'],
				'Recipients' => ['Contacts', 'recipient'],
			],
		];

		$contact = ListQuery::quick_fetch_row('Contact', $recipId, ['id', 'email1', '_display']);
		if (!$contact || empty($contact['email1'])) {
			throw new WFProcessingError('Cannot email invoice: Contact not found or has no email address');
		}

		$email->to_addrs_arr = [
			['email' => $contact['email1'], 'display' => $contact['_display']],
		];
		

		if($mbox) {
			$mbox_email = $mbox->getField('email');
			if (!empty($mbox_email)) {
				$email->cc_addrs_arr[] = array('email' => $mbox_email);
			}
		}
			
		$tplId = $this->getParam('template_id');
		if(! $tplId) {
			throw new WFProcessingError('Cannot email invoice: Email template not defined');
		}
		$tpl = ListQuery::quick_fetch('EmailTemplate', $tplId);
		if(! $tpl) {
			throw new WFProcessingError("Cannot email invoice: Email template not found");
		}
		$subject = $tpl->getField('subject');
		$body = $tpl->getField('body_html');
		
		list ($email->name, $description_html) = TemplateParser::parse_generic(
			array($subject, $body),
			$objects,
			$params
		);
		if(! strlen(trim($description_html)))
			throw new WFProcessingError("Cannot email invoice: Empty email body");
		$email->setDescriptionHTML($description_html);

		$pdfNote = new Note();
		$pdfNote->retrieve($pdfId);
		$email->saved_attachments[] = $pdfNote;

		if ($email->send()) {
			self::debug_msg($email->name, 'Email sent');

			// Save the email
			$email->prohibit_workflow = true;
			$email->save();

			// reparent attachment
			$pdfNote->email_attachment = 1;
			$pdfNote->parent_type = 'Emails';
			$pdfNote->parent_id = $email->id;
			$pdfNote->save();

			$ret[] = $email->id;
		}
		else {
			self::debug_msg($email->send_error_msg, 'Error sending email');
			$GLOBALS['log']->error('Error sending email: '.$email->send_error_msg);
		}
		
		return $ret;
	}

	function getRequiredFields() {
		$fs = parent::getRequiredFields();
		$fs[] = 'billing_account_id';
		return $fs;
	}
	
	function getConfigFields() {
		require_once('modules/PDFTemplates/utils.php');
		$fields = parent::getConfigFields();
		$model = new ModelDef('Email');

		$fields['contact'] = [
			'name' => 'contact',
			'vname' => 'LBL_CONTACT_NAME',
			'vname_module' => 'Emails',
			'type' => 'ref',
			'bean_name' => 'Contact',
			'id_name' => 'contact_id',
			'editable' => false,
			'required' => true
		];
		$fields['contact_id'] = array('type' => 'ref_id', 'hidden' => true);
		$fields['contact']['alt_options'] = $this->getRelatedRefFields('Contact');
		$fields['contact_alt'] = array('type' => 'varchar', 'hidden' => true, 'default' => '=billing_contact');

		$fields['cc_mailbox'] = array(
			'type' => 'ref',
			'vname' => 'LBL_CC_MAILBOX',
			'bean_name' => 'EmailPOP3',
			'id_name' => 'cc_mailbox_id',
		);
		$fields['cc_mailbox_id'] = array('type' => 'ref_id', 'hidden' => true);
		$fields['from_name'] = $model->getFieldDefinition('from_name');
		$fields['from_addr'] = $model->getFieldDefinition('from_addr');
		$tpl_opts = get_config_templates('Invoice', false);
		$fields['pdf_template'] = array(
			'type' => 'ref',
			'vname' => 'LBL_PDF_TEMPLATE',
			'vname_module' => 'Invoice',
			'bean_name' => 'PDFTemplate',
			'id_name' => 'pdf_template_id',
			'required' => true,
			'options' => $tpl_opts,
		);
		$defTpl = PDFTemplate_get_default('Invoice', 'invoice');
		$fields['pdf_template_id'] = array('type' => 'ref_id', 'hidden' => true,
			'default' => $defTpl->getPrimaryKeyValue());
		$fields['template'] = array(
			'type' => 'ref',
			'vname' => 'LBL_NOTIFICATION_TEMPLATE',
			'vname_module' => 'Invoice',
			'bean_name' => 'EmailTemplate',
			'id_name' => 'template_id',
			'required' => true,
		);
		$fields['template_id'] = array('type' => 'ref_id', 'hidden' => true);

		return $fields;
	}

}

