<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


require_once('data/SugarBean.php');
require_once('include/utils.php');

class KBArticle extends SugarBean {	

	// Stored fields
  	var $id;
	var $name;
	var $summary;
	var $description;
	var $deleted;
	var $portal_active;

	var $table_name = 'kb_articles';
	var $object_name = 'KBArticle';
	var $module_dir = 'KBArticles';
	var $new_schema = true;


	// This is used to retrieve related fields from form posts.
	var $additional_column_fields = Array(
		'case_id',
	);
	var $relationship_fields = array(
		'case_id' => 'cases',
	);

	function get_summary_text()
	{
		return "$this->name";
	}

	/// This function fills in data for the list view only.
	function fill_in_additional_list_fields()
	{
	}


	/// This function fills in data for the detail view only.
	function fill_in_additional_detail_fields()
	{
	}


	function get_list_view_data()
	{
		$temp_array = $this->get_list_view_array();
		return $temp_array;
	}


	function save($check_notify = FALSE) {
	
		return parent::save($check_notify);
	}

	function bean_implements($interface) {
		switch($interface) {
			case 'ACL':return true;
		}
		return false;
	}

	function get_tags()
	{
		$ret = array();
		$query = "SELECT kb_tags.* FROM kb_tags LEFT JOIN kb_articles_tags ON kb_articles_tags.tag_id = kb_tags.id WHERE kb_articles_tags.article_id='" . PearDatabase::quote($this->id) . "' AND kb_articles_tags.deleted = 0";
		$res = $this->db->query($query, true);
		while($row = $this->db->fetchByAssoc($res)) {
			$ret[$row['id']] = $row;
		}
		return $ret;
	}

	function search_by_tag_dummy()
	{
		return '1';
	}

	function mark_deleted($id)
	{
		global $db;
		parent::mark_deleted($id);
		$db->query("UPDATE kb_articles_tags SET deleted=1 WHERE article_id='" . PearDatabase::quote($id) . "'", true);
	}

	function search_by_tag()
	{
		$include = array();
		if (!empty($_REQUEST['plus_tag'])) {
			$include = explode(',', $_REQUEST['plus_tag']);
		}
		$exclude = array();
		if (!empty($_REQUEST['minus_tag'])) {
			$exclude = explode(',', $_REQUEST['minus_tag']);
		}
		if (!empty($include) || !empty($exclude)) {
			$ids = $this->find_by_tags($include, $exclude);
			if (!empty($ids)) {
				return "(kb_articles.id IN ('" . join("', '", $ids) . "'))";
			} else {
				return ' 0 ';
			}
		} else {
			return '1';
		}
	}

	function find_by_tags($include = array(), $exclude = array(), $sort = '', $offset = 0, $limit=10000000, $for_portal = false)
	{
		$query = $this->create_tag_query($include, $exclude, $sort, $for_portal);
		$query .= " LIMIT " . (int)$offset . ", " . (int)$limit;
		$res = $this->db->query($query, true);
		$ret = array();
		while($row = $this->db->fetchByAssoc($res)) {
			$ret[] = $row['article_id'];
		}
		return $ret;
	}

	function count_by_tags($include = array(), $exclude = array(), $for_portal = false)
	{
		$query = $this->create_tag_query($include, $exclude, $for_portal);
		$res = $this->db->query($query, true);
		return mysql_num_rows($res);
	}
	
	function create_tag_query($include = array(), $exclude = array(), $sort = '', $for_portal = false)
	{
		$where = '';
		$count1 = is_countable($include) ? count($include) : 0;
		$having = "cnt1 = " . $count1 . " AND cnt2 = 0 ";
		if (!empty($include) || !empty($exclude)) {
			$where .= " AND tag_id IN('" . join("', '", array_merge($include, $exclude)) . "')";
		}
		if ($for_portal) {
			$where .= ' AND kb_articles.portal_active ';
		}
		$query = " SELECT kb_articles_tags.article_id , COUNT(IF(tag_id IN('" . join("', '", $include) . "'), 1, NULL)) AS cnt1, COUNT(IF(tag_id IN('" . join("', '", $exclude) . "'), 1, NULL)) AS cnt2 FROM kb_articles_tags LEFT JOIN kb_articles ON kb_articles.id=kb_articles_tags.article_id WHERE kb_articles_tags.deleted = 0 $where  GROUP BY kb_articles_tags.article_id " ;
		$query .= ' HAVING ' . $having;
		if ($sort) {
			$query .= " ORDER BY $sort ";
		}
		return $query;
	}

	function create_export_query(&$order_by, &$where) {
		$q = $this->_get_default_export_query($order_by, $where);
		return $q;
	}

    static function init_record(RowUpdate &$upd, $input) {
        $update = array();

        if (! empty($input['case_id'])) {
            $fields = array('name' => 'name', 'description' => 'summary', 'resolution' => 'description');
            $case = ListQuery::quick_fetch_row('aCase', $input['case_id'], array_keys($fields));

            if ($case != null) {
                foreach ($fields as $case_field => $kb_field) {
                    if (isset($case[$case_field]))
                        $update[$kb_field] = $case[$case_field];
                }
            }
        }

        $upd->set($update);
    }

    static function add_additional_hiddens(DetailManager $mgr) {
        if (! empty($_REQUEST['case_id']))
            $mgr->layout->addFormHiddenFields(array('case_id' => $_REQUEST['case_id']), false);
    }

    static function set_related(RowUpdate $upd) {
        if (! empty($_REQUEST['case_id']))
            $upd->addUpdateLink('cases', $_REQUEST['case_id']);
    }


	static function before_save(RowUpdate &$upd) 
	{
		$upd->set('ref_article_id', '');
	}

	static function update_references(RowUpdate &$upd) {
		$value = $upd->getField('description');
		$article_id = $upd->getPrimaryKeyValue();

		$ref_type = $upd->getModelName() == 'KBArticle' ? 'Article' : 'Category';

        $searchArray = function($arrays, $key, $search) {
		   $count = 0;

		   foreach($arrays as $object) {
		       if(is_object($object)) {
		          $object = get_object_vars($object);
		       }

		       if(array_key_exists($key, $object) && $object[$key] == $search) $count++;
		   }

		   return $count;
		};

		if($value != '') {
			$value = str_replace("<onecrm:","<onecrm_",$value);
			$value = str_replace("</onecrm:","</onecrm_",$value);
			$doc = new DOMDocument();
			libxml_use_internal_errors(true);
			$doc->loadHTML('<?xml encoding="utf-8" ?>' . $value);

			//Manage article references
            $result = $doc->getElementsByTagName('onecrm_article');

			$lq = new ListQuery('kb_articles_ref');
			$lq->addSimpleFilter('article_id', $article_id);
			$lq->addSimpleFilter('ref_type', 'Article');
			$exists = $lq->fetchAll()->rows;

			$exist_arr = array();

			foreach($exists as $ex_row) {
				$exist_arr[] = $ex_row['ref_id'];
			}

			$newa = array();

			foreach($result as $node) {
				$id = $node->getAttribute('id');
				$article = ListQuery::quick_fetch('KBArticle', $id);
				if($article) {
					$ref = $searchArray($exists,'ref_id', $id);
					$newa[] = $id;
					if ($ref == 0) {
						$ru = new RowUpdate('kb_articles_ref');
						$ru->prohibit_workflow = true;
						$ru->request = array(
							'article_id' => $article_id,
							'ref_type' => 'Article',
							'ref_id' => $id,
							'deleted' => 0,
						);
	
						$ru->loadRequest();
						$ru->insertRow();
					}
				}
			}

			$diff = array_diff($exist_arr,$newa);
			
			foreach($diff as $del_row){
				$lq = new ListQuery('kb_articles_ref');
				$lq->addSimpleFilter('ref_id', $del_row);
				$lq->addSimpleFilter('article_id', $article_id);
				$rows_d = $lq->runQuery();
				foreach($rows_d as $row_d) {
					$update_d = RowUpdate::for_result($row_d);
					$update_d->prohibit_workwlow = true;
	            	$update_d->deleteRow();
				}
	        }

	        //Manage category references

			$result = $doc->getElementsByTagName('onecrm_category');
			$lq = new ListQuery('kb_articles_ref');
			$lq->addSimpleFilter('article_id', $article_id);
			$lq->addSimpleFilter('ref_type', 'Category');
			$exists = $lq->fetchAll()->rows;

			$exist_arr = array();

			foreach($exists as $ex_row) {
				$exist_arr[] = $ex_row['ref_id'];
			}

			$newc = array();

			foreach($result as $node) {
				$id = $node->getAttribute('id');
				$category = ListQuery::quick_fetch('KBCategory', $id);

				$ref = $searchArray($exists,'ref_id', $id);
				if($category) {
					$newc[] = $id;
					if($ref == 0) {
						$id = $category->getField('id');
						$ru = new RowUpdate('kb_articles_ref');
						$ru->request = array(
							'article_id' => $article_id,
							'ref_type' => 'Category',
							'ref_id' => $id,
							'deleted' => 0,
						);
						$ru->loadRequest();
						$ru->insertRow();
					}
				}
			}

			$diff = array_diff($exist_arr,$newc);
			
			foreach($diff as $del_row){
				$lq = new ListQuery('kb_articles_ref');
				$lq->addSimpleFilter('ref_id', $del_row);
				$lq->addSimpleFilter('article_id', $article_id);
				$rows_d = $lq->runQuery();
				foreach($rows_d as $row_d) {
					$update_d = RowUpdate::for_result($row_d);
	            	$update_d->deleteRow();
				}
	        }
		}

		if(empty($newa)) {
	       	$lq = new ListQuery('kb_articles_ref');
			$lq->addSimpleFilter('article_id', $article_id);
			$lq->addSimpleFilter('ref_type', 'Article');
			$rows_d = $lq->runQuery();
			foreach($rows_d as $row_d) {
				$update_d = RowUpdate::for_result($row_d);
	            $update_d->deleteRow();
			}
	    }

	    if(empty($newc)) {
	       	$lq = new ListQuery('kb_articles_ref');
			$lq->addSimpleFilter('article_id', $article_id);
			$lq->addSimpleFilter('ref_type', 'Category');
			$rows_d = $lq->runQuery();
			foreach($rows_d as $row_d) {
				$update_d = RowUpdate::for_result($row_d);
	           	$update_d->deleteRow();
			}
	    }

	    //Update references

		$newname = $upd->getField('name');
		$oldname = $upd->getField('name',null,true);
		$id = $upd->getPrimaryKeyValue();

		if($newname != $oldname) {

			$lq = new ListQuery('kb_articles_ref');
			$lq->addSimpleFilter('ref_id', $article_id);
			$lq->addSimpleFilter('ref_type', $upd->getModelName() == 'KBArticle' ? 'Article' : 'Category');
			$tag_name = $upd->getModelName() == 'KBArticle' ? 'article' : 'category';
			$listResult = $lq->runQuery();
			$rowResults = $listResult->getRowResults();

			foreach($rowResults as $row) {
				$ref_article_id = $row->getField('article_id');

				$ref_article = ListQuery::quick_fetch('KBArticle', $ref_article_id);

				if($ref_article) {
					$value = $ref_article->getField('description');
					if($value != '') {
						$value = str_replace("<onecrm:","<onecrm_",$value);
						$value = str_replace("</onecrm:","</onecrm_",$value);
						$doc = new DOMDocument();
						libxml_use_internal_errors(true);
						$doc->loadHTML('<?xml encoding="utf-8" ?>' . $value);

						//Manage article references
						$result = $doc->getElementsByTagName("onecrm_$tag_name");
						$nodes = [];
						foreach ($result as $node) {
							$nodes[] = $node;
						}
						foreach ($nodes as $node) {
							if ($node->getAttribute('id') == $id) { 
								$new_node = new DOMElement("onecrm_$tag_name");
								$node->parentNode->replaceChild($new_node, $node);
								$new_node->setAttribute('contenteditable', 'false');
								$new_node->setAttribute('id', $id);
								$new_node->appendChild(new DOMText($newname));
								$new_node->parentNode->setAttribute('title', $newname);
							}
						}

						$newdesc = $doc->saveHTML();
						$newdesc = str_replace("<onecrm_","<onecrm:",$newdesc);
						$newdesc = str_replace("</onecrm_","</onecrm:",$newdesc);
						$update_ref = RowUpdate::for_result($ref_article);
						$update_ref->set('description', $newdesc);
						$update_ref->save();
					}
				}
			}
		}
	}


	static function do_update_counters($prev_parent, $new_parent) {
		$moved_up = false;
		$cat_id = $prev_parent;
		while ($cat_id) {
			if ($new_parent && $new_parent == $cat_id) {
				$moved_up = true;
				break;
			}
			$cat = ListQuery::quick_fetch('KBCategory', $cat_id, ['articles_count', 'parent_id']);
			if ($cat) {
				$cat_upd = RowUpdate::for_result($cat);
				$cat_upd->set('articles_count', $cat->getField('articles_count') - 1);
				$cat_upd->setRelatedData('updating_counters', true);
				$cat_upd->prohibit_workflow = true;
				$cat_upd->save();
				$cat_id = $cat->getField('parent_id');
			} else {
				break;
			}
		}
		if ($moved_up) return;

		if ($new_parent && $new_parent != $prev_parent) {
			$cat_id = $new_parent;
			while ($cat_id) {
				$cat = ListQuery::quick_fetch('KBCategory', $cat_id, ['articles_count', 'parent_id']);
				if ($cat) {
					$cat_upd = RowUpdate::for_result($cat);
					$cat_upd->set('articles_count', $cat->getField('articles_count') + 1);
					$cat_upd->setRelatedData('updating_counters', true);
					$cat_upd->prohibit_workflow = true;
					$cat_upd->save();
					$cat_id = $cat->getField('parent_id');
				} else {
					break;
				}
			}
		}
	}

	static function update_counters(RowUpdate &$upd) {
		// prevent infinite loop
		if ($upd->getRelatedData('updating_counters')) {
			return;
		}

		// first, determine what kind of update we have:
		// Insert, Update, Delete or Undelete
		if ($upd->new_record) {
			$type = 'insert';
		} elseif ((bool)$upd->getField('deleted', false, true) != (bool)$upd->getField('deleted', false)) {
			$type = $upd->getField('deleted') ? 'delete' : 'undelete';
		} else {
			$type = 'update';
		}

		// if undelete, hierarchy may be out of sync. Do complete update.
		if ($type == 'undelete') {
			KBCategory::update_all_counters();
			return;
		}

		$prev_parent = $upd->getField('category_id', null, true);
		$new_parent = $upd->getField('category_id');
		if ($type == 'delete') $new_parent = null;

		// if update and no parent change, do nothing
		if ($type == 'update' && $prev_parent == $new_parent) {
			return;
		}
		self::do_update_counters($prev_parent, $new_parent);

	}

	static function update_counters_unlink(RowUpdate $upd, $link_name=null, $right=null) {
		if ($link_name !== 'articles' || !$right) return;
		self::do_update_counters($upd->getField('id'), null);
	}

	static function add_full_path($model, &$record) {
		$cats =  AppConfig::db_all_rows('KBCategory');
		$path = [];
		$key = $model == 'KBArticle' ? 'category_id' : 'parent_id';
		$cat_id = array_get_default($record, $key);
		$modelDef = new ModelDef('KBCategory');
		$def = $modelDef->getFieldDefinition('image');
		require_once 'include/upload_file.php';
		while ($cat_id) {
			$cat = array_get_default($cats, $cat_id);
			if (!$cat) break;
			$row = ['id' => $cat['id'], 'name' => $cat['name'], 'category_icon' => $cat['category_icon']];
			$upload_dir = \UploadFile::replace_placeholders($def['upload_dir']);
			$filename_field = $def['filename'];
			$filename = array_get_default($cat, $filename_field);
			if ($filename) {
				$value = \UploadFile::get_path($filename, '', $upload_dir);
				$row[$filename_field] = $value;
				$row['__full_' . $filename_field] = AppConfig::site_url() . $value;
			}
			array_unshift($path, $row);
			$cat_id = $cat['parent_id'];
		}
		$json = getJsonObj();
		$record['__path'] = $json->encode($path);
	}
}

