<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once 'include/layout/forms/FormTableSection.php';

class KBTagsWidget extends FormTableSection {

	var $tags_text = null;
	var $tags_list = array();

	function init($params=null, $model=null) {
		parent::init($params, $model);
		if(! $this->id)
			$this->id = 'kb_tags_widget';
	}
	
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context)
	{
		if($gen->getLayout()->getEditingLayout()) {
			$params = array();
			return $this->renderOuterTable($gen, $parents, $context, $params);		
		}

		if ($gen->getLayout()->getDefaultEditable()) return $this->renderHtmlEdit($gen, $row_result, $parents, $context);
		else return $this->renderHtmlView($gen, $row_result, $parents, $context);
	}
	
	function renderHtmlView(HtmlFormGenerator $gen, RowResult $row_result, array $parents, array $context)
	{
		global $mod_strings;
		$tagIds = array();
		$tags = array();
		$id = $row_result->getField('id');
		$lq = new ListQuery('kb_articles_tags');
		$lq->addSimpleFilter('article_id', $id);
		$lr = $lq->runQuery();
		foreach ($lr->getRowIndexes() as $idx) {
			$rr = $lr->getRowResult($idx);
			$tagIds[] = $rr->getField('tag_id');
		}
		if (!empty($tagIds)) {
			$lq = new ListQuery('kb_tags');
			$lq->addSimpleFilter('id', $tagIds);
			$lr = $lq->runQuery();
			foreach ($lr->getRowIndexes() as $idx) {
				$rr = $lr->getRowResult($idx);
				$tags[] = array($rr->getField('id'), $rr->getField('tag'));
			}
		}
		$html = <<<EOH
<div class="form-section">
	<div class="row form-row form-title-row">
		<div class="column form-cell">
			<div class="form-entry form-title">
				<h4>{$mod_strings['LBL_TAGS']}</h4>
			</div>
		</div>
	</div>
	<div class="row form-row">
		<div class="column form-cell">
			<div class="form-entry"><div class="form-value">
EOH;

		foreach ($tags as $i => $tag) {
			if ($i) $html .= ', &nbsp; ';
			$html .= '<a href="index.php?module=KBArticles&action=index&query=true&include_tags=' . urlencode($tag[1]) . '">' . $tag[1] . '</a>';
		}
		$html .= <<<EOH
			</div></div>
		</div>
	</div>
</div>
EOH;
		return $html;
	}

	function renderHtmlEdit(HtmlFormGenerator $gen, RowResult $row_result, array $parents, array $context)
	{
		global $mod_strings;
		global $pageInstance;
		
		$filtering = $this->id == 'kb_tags_include' || $this->id == 'kb_tags_exclude';
		
		if ($filtering) {
			$this->loadTagsFilter();
		} else {
			if (empty($this->tags_text)) {
				$this->loadTagsText($row_result);
			}
		}
		$vname = $this->vname;
		if (!$vname)
			$vname = 'LBL_TAGS_EDIT';
		$html = <<<EOH
<div class="form-section">
	<div class="row form-row form-title-row">
		<div class="column form-cell">
			<div class="form-entry form-title">
				<h4>{$mod_strings[$vname]}</h4>
			</div>
		</div>
	</div>
	<div class="row form-row">
		<div class="column form-cell sm-12">
			<div class="form-entry"><div class="form-value">
				<div id="{$this->name}_tags_container"></div>
			</div></div>
		</div>
	</div>
</div>
EOH;
		$json = getJSONObj();
		$tags_list = $json->encode(array_filter(array_map('trim', explode(',', $this->tags_text)), function($str) {return strlen($str);}));
		$allow_new = AppConfig::setting('company.kb_allow_new_tags') ? 'true' : 'false';
		if ($filtering)
			$allow_new = 'false';
		$js = "new KBTagsAutocomplete().init('{$this->name}_tags_container', '{$this->name}_tags', '$this->name', $allow_new, $tags_list);";
		$pageInstance->add_js_literal($js, null, LOAD_PRIORITY_FOOT);
		return $html;
	}

	function loadTagsFilter() {
		if ($this->id == 'kb_tags_include') {
			$this->tags_text = array_get_default($_REQUEST, 'include_tags', '');
		} else {
			$this->tags_text = array_get_default($_REQUEST, 'exclude_tags', '');
		}
	}

	function loadTagsText(RowResult $row_result)
	{
		$tagIds = array();
		$id = $row_result->getField('id');
		$lq = new ListQuery('kb_articles_tags');
		$lq->addSimpleFilter('article_id', $id);
		$lr = $lq->runQuery();
		foreach ($lr->getRowIndexes() as $idx) {
			$rr = $lr->getRowResult($idx);
			$tagIds[] = $rr->getField('tag_id');
		}
		if (empty($tagIds)) {
			$this->tags_text = '';
			return;
		}
		$tags = array();
		$lq = new ListQuery('kb_tags');
		$lq->addSimpleFilter('id', $tagIds);
		$lr = $lq->runQuery();
		foreach ($lr->getRowIndexes() as $idx) {
			$rr = $lr->getRowResult($idx);
			$tags[] = $rr->getField('tag');
		}
		$this->tags_text = join(', ', $tags);
	}

	function loadUpdateRequest(RowUpdate $update, array $input) {
		$tagsNames = explode(',', array_get_default($input, 'kb_tags', ''));
		$tagsNames = array_map('trim', $tagsNames);
		$tagsNames = array_filter($tagsNames, 'strlen');
        $lq = new ListQuery('kb_tags');
		$lq->addSimpleFilter('tag', $tagsNames);
		$lr = $lq->runQuery();

        $existentTags = array();
		foreach ($lr->getRowIndexes() as $idx) {
			$rr = $lr->getRowResult($idx);
			$existentTags[] = $rr->getField('tag');
			$this->tags_list[] = $rr->getField('id');
		}

		$this->tags_text = join(', ', $existentTags);
        $update->setRelatedData($this->id.'_rows', $this->extractNewTags($tagsNames, $existentTags));
    }

	function validateInput(RowUpdate $update) {
		return true;
	}
	
	function afterUpdate(RowUpdate $update) {
        $this->addNewTags($update);
        $id = $update->getPrimaryKeyValue();

		$lq = new ListQuery('kb_articles_tags');
		$lq->addSimpleFilter('article_id', $id);
		$lq->filter_deleted = false;
		$lr = $lq->runQuery();
		$add_tags = $this->tags_list;

		foreach ($lr->getRowIndexes() as $idx) {
			$rr = $lr->getRowResult($idx);
			$tid = $rr->getField('tag_id');
			if (($idx = array_search($tid, $add_tags)) !== false) {
				unset($add_tags[$idx]);
				if ($rr->getField('deleted')) {
					$ru = new RowUpdate($rr);
					$ru->markDeleted(false);
				}
			} else {
				if (!$rr->getField('deleted')) {
					$ru = new RowUpdate($rr);
					$ru->markDeleted(true);
				}
			}
		}
		foreach ($add_tags as $tid) {
			$ru = new RowUpdate('kb_articles_tags');
			$ru->request = array(
				'article_id' => $id,
				'tag_id' => $tid,
			);
			$ru->loadRequest();
			$ru->insertRow();
		}

	}

    function addNewTags(RowUpdate $update) {
        $new_tags = $update->getRelatedData($this->id.'_rows');

        if (is_countable($new_tags) && sizeof($new_tags) > 0) {
            for ($i = 0; $i < sizeof($new_tags); $i++) {
                $tag_update = RowUpdate::blank_for_model('kb_tags');
                $tag_update->set('tag', $new_tags[$i]);
                if ($tag_update->save())
                    $this->tags_list[] = $tag_update->getPrimaryKeyValue();
            }
        }
    }

    function extractNewTags($inputTags, $existentTags) {
        $result = array();
        if (is_countable($inputTags) && sizeof($inputTags) > 0) {
            for ($i = 0; $i < sizeof($inputTags); $i++) {
                if (! empty($inputTags[$i]) && ! in_array($inputTags[$i], $existentTags))
                    $result[] = htmlspecialchars($inputTags[$i]);
            }
        }
        return $result;
    }
}
?>
