<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


require_once('data/SugarBean.php');

class KBCategory extends SugarBean {	

	// Stored fields
	var $id;
	var $name;
	var $description;
	var $deleted;
	var $parent_id;
	
	
	var $table_name = 'kb_categories';
	var $object_name = 'KBCategory';
	var $module_dir = 'KBCategories';
	var $new_schema = true;


	public static function disable_circular($upd) {
		$myId = $upd->getField('id');
		if (!$myId) return; // new record, nothing to do
		$parentId = $upd->getField('parent_id');
		if (!$parentId) return; // no parent, nothing to do
		while ($parentId) {
			if ($parentId == $myId)	throw new IAHActionAbort(translate('LBL_CIRCULAR_REF_ERR'));
			$parent = ListQuery::quick_fetch('KBCategory', $parentId);
			$parentId = $parent->getField('parent_id');
		}
	}

	static function before_save(RowUpdate &$upd) 
	{
		self::disable_circular($upd);
	}

	static function update_all_counters() {
		$counters = [];
		$lq = new ListQuery('KBArticle');
		$lq->addSimpleFilter('category_id', null, 'not_empty');
		$res = $lq->runQuery(0, null, false, null, 100);
		while (true) {
			foreach ($res->rows as $row_idx => $row) {
				@$counters[$row['category_id']]['articles']++;
			}
			if ($res->page_finished)
				break;
			$lq->pageResult($res);
		}
		$lq = new ListQuery('KBCategory');
		$lq->addSimpleFilter('parent_id', null, 'not_empty');
		$res = $lq->runQuery(0, null, false, null, 100);
		while (true) {
			foreach ($res->rows as $row_idx => $row) {
				@$counters[$row['parent_id']]['categories']++;
				@$counters[$row['parent_id']]['articles'] += @$counters[$row['id']]['articles'] ;
			}
			if ($res->page_finished)
				break;
			$lq->pageResult($res);
		}
		foreach ($counters as $id => $c) {
			$cat = ListQuery::quick_fetch('KBCategory', $id);
			if (!$cat) continue;
			$cat_upd = RowUpdate::for_result($cat);
			$cat_upd->set([
				'categories_count' => 0 + @$c['categories'],
				'articles_count' => 0 + @$c['articles'],
			]);
			$cat_upd->prohibit_workflow = true;
			$cat_upd->setRelatedData('updating_counters', true);
			$cat_upd->save();
		}
	}

	static function do_update_counters($prev_parent, $new_parent) {
		if ($prev_parent) {
			$cat = ListQuery::quick_fetch('KBCategory', $prev_parent, ['categories_count']);
			if ($cat) {
				$cat_upd = RowUpdate::for_result($cat);
				$cat_upd->set('categories_count', $cat->getField('categories_count') - 1);
				$cat_upd->setRelatedData('updating_counters', true);
				$cat_upd->prohibit_workflow = true;
				$cat_upd->save();
			}
		}
		if ($new_parent && $new_parent != $prev_parent) {
			$cat = ListQuery::quick_fetch('KBCategory', $new_parent, ['categories_count']);
			if ($cat) {
				$cat_upd = RowUpdate::for_result($cat);
				$cat_upd->set('categories_count', $cat->getField('categories_count') + 1);
				$cat_upd->setRelatedData('updating_counters', true);
				$cat_upd->prohibit_workflow = true;
				$cat_upd->save();
			}
		}
	}

	static function update_counters(RowUpdate &$upd) {
		// prevent infinite loop
		if ($upd->getRelatedData('updating_counters')) {
			return;
		}

		// first, determine what kind of update we have:
		// Insert, Update, Delete or Undelete
		if ($upd->new_record) {
			$type = 'insert';
		} elseif ((bool)$upd->getField('deleted', false, true) != (bool)$upd->getField('deleted', false)) {
			$type = $upd->getField('deleted') ? 'delete' : 'undelete';
		} else {
			$type = 'update';
		}

		// if undelete, hierarchy may be out of sync. Do complete update.
		if ($type == 'undelete') {
			self::update_all_counters();
			return;
		}

		$prev_parent = $upd->getField('parent_id', null, true);
		$new_parent = $upd->getField('parent_id');
		if ($type == 'delete') $new_parent = null;

		// if update and no parent change, do nothing
		if ($type == 'update' && $prev_parent == $new_parent) {
			return;
		}
		self::do_update_counters($prev_parent, $new_parent);

	}
	static function update_counters_unlink(RowUpdate $upd, $link_name=null, $right=null) {
		if ($link_name !== 'subcategories') return;
		if ($right) return;
		self::do_update_counters($upd->getField('id'), '');
	}
	
}

?>
