<?php

require_once('include/utils/theme_utils.php');

echo get_module_title('LayoutTest', 'Custom Settings', true);

$theme = AppConfig::theme();

function rgba_to_hex($val) {
	if(preg_match('~^rgba\(([^,]+),\s*([^,]+),\s*([^,]+),\s*([^,]+)\)~', $val, $m)) {
		if($m[4]*1 == 1) {
			$val = sprintf('#%02x%02x%02x', $m[1], $m[2], $m[3]);
		} else {
			$val = sprintf('#%02x%02x%02x%02x', $m[1], $m[2], $m[3], $m[4]*255);
		}
	}
	return $val;
}

$inp_vars = [];
if(array_key_exists('ConfigForm_update', $_POST)) {
	foreach($_POST as $k => $v) {
		if(strlen($v) && substr($k, 0, 6) == 'theme_') {
			$inp_vars[substr($k, 6)] = $v;
		}
	}
}

$std_vars = array_map('rgba_to_hex', $inp_vars);

$theme_vars = [];
foreach($inp_vars as $k => $v) {
	if(strlen($v) == 9 && $v[0] == '#') {
		list($r, $g, $b, $alpha) = sscanf($v, "#%02x%02x%02x%02x");
		$alpha = round($alpha / 255, 4);
		$v = "rgba($r,$g,$b,$alpha)";
	}
	$theme_vars[$k] = $v;
}

$custom = custom_theme_css($theme, $theme_vars);
$themeInstance->set_override_css($custom['css']);
//pr2($custom['standard_vars']);

$diff_vars = [];
foreach($std_vars as $k => $v) {
	$val = array_get_default($custom['standard_vars'], $k);
	if($val != $v && $val != $theme_vars[$k]) {
		$diff_vars[$k] = $val;
	}
}


require_once('include/layout/forms/FormGenerator.php');

$layout = array(
	'type' => 'configform',
	'editable' => true,
	'form_buttons' => array(
		'save' => array(
			'icon' => 'icon-accept',
			'name' => 'update',
			'label' => 'Test Settings',
			'perform' => '',
		),
		'reset' => array(
			'icon' => 'icon-cancel',
			'label' => 'Reset',
			'perform' => '',
		),
	),
	'sections' => array(
		array(
			'id' => 'main',
			'label_position' => 'top',
			'columns' => 4,
			'elements' => array(),
		),
		array(
			'id' => 'combined',
			'label_position' => 'top',
			'columns' => 1,
			'elements' => array(
				[
					'name' => 'combined',
					'type' => 'text',
					'vname' => 'Settings',
				]
			),
		),
	),	
);

$misc_colors = [
	'tint-color',
];

$text_colors = [
	'text-color-base',
	'text-color-link',
    'text-color-field',
    'text-color-muted',
    'text-color-placeholder',
    'text-color-hover',
    'text-color-label',
    'text-color-form-label',
];

$bg_colors = [
	'bg-color-body',
	'bg-color-light',
	//'bg-color-field',
	'bg-color-check',
	'bg-color-static',
	'bg-color-active',
	'bg-color-selected',
	'bg-color-disabled',
	'bg-color-muted',
	'bg-color-quiet',
	'bg-color-neutral',
	'bg-color-solid',
	//'bg-color-button',
	'bg-color-panel',
	//'bg-color-panel-header',
	'bg-color-panel-subheader',
	'bg-color-panel-footer',
	//'bg-color-panel-readonly-header',
	//'bg-color-panel-readonly-subheader',
	//'bg-color-panel-readonly-body',
	//'bg-color-panel-readonly-footer',
	'bg-color-system',
	'bg-color-system-header',
	'bg-color-system-subheader',
	'bg-color-system-footer',
	//'bg-color-dialog',
	//'bg-color-dialog-header',
	//'bg-color-dialog-footer',
	//'bg-color-form',
	//'bg-color-form-header',
	//'bg-color-tab',
	//'bg-color-tab-current',
	//'bg-color-label',
	//'bg-color-label-dark',
	//'bg-color-menu-header',
	//'bg-color-dark-header',
	//'bg-color-dark-subheader',
	//'bg-color-slider',
	//'bg-color-slider-indicator',
	'bg-color-invert',
	'bg-color-select',
	'border-color-default',
	//'border-color-field',
	'border-color-dark',
	//'border-color-button',
	//'border-color-tab',
	'border-color-tab-current',
	'border-color-soft',
	//'border-color-grid',
	'border-color-muted',
	//'border-color-button-muted',
	'border-color-form',
	//'border-color-panel',
	//'border-color-panel-readonly',
];

$result = new RowResult();
$result->row = array();

$colors = $misc_colors + $text_colors + $bg_colors;
foreach($colors as $color) {
	$fields['theme_'.$color] = array(
		'label' => $color,
		'type' => 'color',
		'allow_alpha' => true,
		'len' => 25,
	);
	$cval = $custom['standard_vars'][$color];
	if(isset($std_vars[$color]))
		$cval = $std_vars[$color];
	else
		$cval = rgba_to_hex($cval);
	$result->row['theme_' . $color] = $cval;
}

$result->row['combined'] = json_encode($diff_vars, JSON_PRETTY_PRINT);

$result->fields = array();
foreach($fields as $k => $f) {
	$result->fields[$k] = $f;
	$f['name'] = $k;
	$layout['sections'][0]['elements'][] = $f;
}

$gen = FormGenerator::html_form(null, $layout, 'ConfigForm');
$gen->getFormObject()->addHiddenFields(
	['module' => 'LayoutTest', 'action' => 'Custom']
);
$gen->renderForm($result);
$form = $gen->getResult();
$gen->exportIncludes();

?>

<div class="detailOuter">
	<?php echo $form; ?>

	<div class="demo-block" style="margin-top: 2em">
		<div class="demo-title">
			Test controls
		</div>
		<div class="row demo-row">
			<div class="demo-inline">
				<article class="panel-outer panel-border panel-default">
					<header class="panel-header">
						<h3 class="no-margin">
							Default Panel
						</h3>
					</header>
					<header class="panel-subheader">
						<h5 class="no-margin">
							Panel subheader
						</h5>
					</header>
					<section class="panel-body panel-pad-body">
						<span>Panel body</span>
					</section>
					<footer class="panel-footer">
						<span>Panel footer</span>
					</footer>
				</article>
			</div>
			<div class="demo-inline">
				<article class="panel-outer panel-border panel-system">
					<header class="panel-header">
						<h3 class="no-margin">
							System Panel
						</h3>
					</header>
					<header class="panel-subheader">
						<h5 class="no-margin">
							Panel subheader
						</h5>
					</header>
					<section class="panel-body panel-pad-body">
						<span>Panel body</span>
					</section>
					<footer class="panel-footer">
						<span>Panel footer</span>
					</footer>
				</article>
			</div>
			<div class="demo-inline">
				<article class="panel-outer panel-border panel-default panel-readonly">
					<header class="panel-header">
						<h3 class="no-margin">
							Read-only Panel
						</h3>
					</header>
					<header class="panel-subheader">
						<h5 class="no-margin">
							Panel subheader
						</h5>
					</header>
					<section class="panel-body panel-pad-body">
						<span>Panel body</span>
					</section>
					<footer class="panel-footer">
						<span>Panel footer</span>
					</footer>
				</article>
			</div>
		</div>
		<div class="row demo-row">
			<div class="column">
				<div class="input-tabgroup nowrap listview-tabs">
					<div class="input-tabgroup-inner">
						<div class="tab-outer normal current" style="z-index: 14">
							<div class="tab-button " tabindex="0">
								<div class="input-icon tab-icon " style="display: none"></div><span class="input-label tab-label">Browse All</span>
							</div>
						</div>
						<div class="tab-outer normal " style="z-index: 13">
							<div class="tab-button " tabindex="0">
								<div class="input-icon tab-icon " style="display: none"></div><span class="input-label tab-label">Quick Filter</span>
							</div>
						</div>
						<div class="tab-outer normal  postsep" style="z-index: 3">
							<div class="tab-button  no-label " tabindex="0">
								<div class="input-icon tab-icon icon-editlayout"></div><span class="input-label tab-label"></span>
							</div>
						</div>
					</div>
				</div>
				<div class="card-outer panel-outer listview-panel panel-system panel-border">
					<div class="card-header panel-header listview-header listview-search-header">
						<div class="card-outer panel-outer listview-filter">
							<div class="card-outer form-outer search-panel">
								<div class="card-body form-body">
									<div class="form-section">
										<div class="row form-row">
											<div class="filter-basic-outer column form-cell">
												<div class="filter-basic-inner form-inline">
													<div class="form-entry">
														<div class="form-value">
															<div class="input-field input-field-group input-search filter" style="width: 15em; ">
																<div class="input-label input-field-icon lg">
																	<div class="uii uii-lg uii-search"></div>
																</div>
																<input type="text" class="input-text input-entry " autocapitalize="off" results="9" incremental="" size="15" name="filter_text" id="filter_text" value="" tabindex="1">
															</div>
														</div>
													</div>
												</div>
												<div class="filter-basic-controls">
													<button name="clearButton" tabindex="5" type="button" id="listView-e895-FilterFormclearButton-input" class="input-button"><span class="uii uii-clear"></span> <span class="input-label" id="listView-e895-FilterFormclearButton-input-label">Clear</span></button>
												</div>
											</div>
										</div>
									</div>
								</div>
							</div>
						</div>
					</div>
					<div class="card-header panel-subheader listview-header">
						<div class="row listview-nav-block compact">
							<div class="column listview-actions buttons-inline align-left">
								<div class="inline-elt">
									<button name="SubPanel_create" type="button" class="input-button first">
										<div class="input-icon left icon-edit"></div><span class="input-label" id="listView-e895-create-label">Create</span>
									</button>
								</div>
								<div class="inline-elt">
									<button class="input-button menu-source">
										<div class="input-arrow">
											<span class="input-label"><span class="uii uii-gear"></span> Actions</span>
										</div>
									</button>
								</div>
							</div>
							<div class="column listview-status align-left">&nbsp;Selected: <span id="listView-e895-SelectCountHead" class="text-number">0</span> of <span class="text-number">18</span></div>
						</div>
					</div>
					<div class="card-body panel-body listview-body">
						<table class="listView" cellpadding="0" cellspacing="0" border="0" width="100%">
							<thead class="listViewColumns">
								<tr class="listHead">
									<th class="listViewThS1 listViewMeta complex">
										<div class="input-field compact muted listview-select rbullet" tabindex="1" role="button" style="margin: -1px">
											<div class="input-label">
												<div class="input-check"><input type="checkbox" class="checkbox"  name="check-all" value="">
													<label for="listView-e895-check-all"></label>
												</div>
											</div>
										</div>
									</th>
									<th class="listViewThS1">&nbsp;</th>
									<th class="listViewThS1 complex" width="80%">
										<a href="#" class="listViewThLinkS1 listColLink">
											<div class="listColHead responsive sort-active">
												<div class="listColHeadInner">
													<div class="listColLabel nowrap with-arrow">Account Name</div>
													<div class="listColArrow">
														<div class="input-icon list-sort icon-sort-down"></div>
													</div>
												</div>
											</div>
										</a>
									</th>
								</tr>
							</thead>
							<tbody>
								<tr data-id="53c6014d-3742-ae2a-d431-59d021e863a1" class="listViewRow oddListRowS1">
									<td class="listViewTd listViewMeta" width="1%"><div class="input-check"><input onclick="sListView.checkItem('listView-e895', this)" type="checkbox" class="checkbox" name="mass[]" value="53c6014d-3742-ae2a-d431-59d021e863a1" autocomplete="off"><label></label></div>&nbsp;<span id="adspan_listView-e895_53c6014d-3742-ae2a-d431-59d021e863a1" class="uii uii-info active-icon" onmousedown="SUGAR.ui.stopPropagation(event);" onmouseout="return SUGAR.util.clearAdditionalDetailsCall()" onmouseover="return SUGAR.util.getAdditionalDetails('Account', '53c6014d-3742-ae2a-d431-59d021e863a1', 'adspan_listView-e895_53c6014d-3742-ae2a-d431-59d021e863a1', null, 'listview')" ></span></td>
									<td class="listViewTd"><span class="uii uii-star active-icon " onmousedown="SUGAR.ui.stopPropagation(event);" onclick="sListView.addToFavorites('listView-e895', this, 'Accounts', '53c6014d-3742-ae2a-d431-59d021e863a1'); return false;" title="Add To Favorites"></span></td>
									<td class="listViewTd"><span class="detailLink"><a class="listViewNameLink" href="index.php?module=Accounts&action=DetailView&record=53c6014d-3742-ae2a-d431-59d021e863a1" onclick="sListView.showDetail('listView-e895', {module:'Accounts',action:'DetailView',record:'53c6014d-3742-ae2a-d431-59d021e863a1'}, event); return false;">A16</a></span></td>
								</tr>
								<tr data-id="5a81a914-ecc6-0c0e-b374-57339043811b" class="listViewRow evenListRowS1">
									<td class="listViewTd listViewMeta" width="1%"><div class="input-check"><input onclick="sListView.checkItem('listView-e895', this)" type="checkbox" class="checkbox" name="mass[]" value="5a81a914-ecc6-0c0e-b374-57339043811b" autocomplete="off"><label></label></div>&nbsp;<span id="adspan_listView-e895_5a81a914-ecc6-0c0e-b374-57339043811b" class="uii uii-info active-icon" onmousedown="SUGAR.ui.stopPropagation(event);" onmouseout="return SUGAR.util.clearAdditionalDetailsCall()" onmouseover="return SUGAR.util.getAdditionalDetails('Account', '5a81a914-ecc6-0c0e-b374-57339043811b', 'adspan_listView-e895_5a81a914-ecc6-0c0e-b374-57339043811b', null, 'listview')" ></span></td>
									<td class="listViewTd"><span class="uii uii-star active-icon " onmousedown="SUGAR.ui.stopPropagation(event);" onclick="sListView.addToFavorites('listView-e895', this, 'Accounts', '5a81a914-ecc6-0c0e-b374-57339043811b'); return false;" title="Add To Favorites"></span></td>
									<td class="listViewTd"><span class="detailLink"><a class="listViewNameLink" href="index.php?module=Accounts&action=DetailView&record=5a81a914-ecc6-0c0e-b374-57339043811b" onclick="sListView.showDetail('listView-e895', {module:'Accounts',action:'DetailView',record:'5a81a914-ecc6-0c0e-b374-57339043811b'}, event); return false;">Account</a></span><div class="listAddLine">Investor</div></td>
								</tr>
							</tbody>
						</table>
					</div>
				</div>
			</div>
		</div>
	</div>
</div>
