<?php

$pageInstance->set_no_decoration();
$skipHeaders = $skipFooters = 1;

require_once('include/utils/theme_utils.php');

$theme_name = array_get_default($_REQUEST, 'theme', 'Default');
$vars = [];
$css_info = cached_theme_css($theme_name, $vars);

function renderToggle($name='', $id='', $cls='', $body='', $type='checkbox', $etc='') {
	return <<<EOQ
	<div class="input-toggle $cls">
		<input type="$type" id="$id" name="$name" value="1" $etc>
		<label for="$id" class="input-label text-label">
			<div class="switch"></div>
			{$body}
		</label>
	</div>
EOQ;
}

function renderCheck($name='', $id='', $cls='', $body='', $type='checkbox', $etc='') {
	return <<<EOQ
	<div class="input-check $cls">
		<input type="$type" id="$id" name="$name" value="1" $etc>
		<label for="$id" class="input-label text-label">
			{$body}
		</label>
	</div>
EOQ;
}

function renderForm($title, $elements, $columns=2) {
	$ret = '<div class="card-outer form-outer default-form">';
	if($title)
		$ret .= '<div class="card-header form-header"><h3 class="form-row form-title">' . $title . '</h3></div>';
	$body = [];
	$cidx = 0;
	$row = null;
	foreach($elements as $e) {
		$span = array_get_default($e, 'colspan', 1);
		if($row && $cidx + $span > $columns) {
			$body[] = $row;
			$row = null;
			$cidx = 0;
		}
		if(! $row) {
			$row = [$e];
			if($span >= $columns) {
				$body[] = $row;
				$row = null;
				$cidx = 0;
			} else
				$cidx = $span;
		} else {
			$row[] = $e;
			$cidx += $span;
		}
	}
	if($row)
		$body[] = $row;
	$bodies = [$body];
	foreach($bodies as $rows) {
		$ret .= '<div class="card-body form-body">';
		foreach($rows as $row) {
			$ret .= '<div class="row form-row col-unstack">';
			foreach($row as $elt) {
				$ret .= '<div class="column form-cell">';
				if(! empty($elt['label']))
					$ret .= '<div class="topLabel">'.$elt['label'].'</div>';
				$ret .= '<div class="form-element">' . $elt['value'] . '</div>';
				$ret .= '</div>';
			}
			$ret .= '</div>';
		}
		$ret .= '</div>';
	}
	return $ret;
}

function renderCalendar() {
	$ret = '';
	$date = date('2015-m-d');
	$ts = strtotime($date . ' 12:00:00');
	list($year, $month, $day, $mlen) = explode(' ', date('Y n j t', $ts));
	$ret .= date('F', $ts);
	$first = strtotime("$year-$month-1 12:00:00");
	$first_dow = date('w', $first);
	$wday_ts = $first_dow ? strtotime("$year-$month-1 12:00:00 - $first_dow days") : $first;
	$rows = ceil(($mlen + $first_dow) / 7);
	
	$ret = '<div class="card-outer calendar-outer input-calendar panel-outer panel-border panel-default">';
	$ret .= '<header class="card-header panel-header no-pad">';
	$larr = '<span class="uii uii-fprevious nav-left"></span>';
	$larr .= '<span class="uii uii-previous nav-left"></span>';
	$rarr = '<span class="uii uii-next nav-right"></span>';
	$rarr .= '<span class="uii uii-fnext nav-right"></span>';
	$ret .= '<h4 class="text-label nav-row">';
	$ret .= $larr . ' <span class="nav-title">' . date('M Y', $ts) . '</span> ' . $rarr;
	$ret .= '</h4>';
	$ret .= '</header>';
	$ret .= '<header class="card-header panel-subheader no-pad grid-body">';
	$ret .= '<div class="grid-row dates">';
	for($j = 0; $j < 7; $j++) {
		$ret .= '<div class="grid-cell number-cell day-head">' . date('D', $wday_ts) . '</div>';
		$wday_ts += 24*3600;
	}
	$ret .= '</header>';
	$ret .= '<div class="card-body panel-body grid-body">';
	$idx = 1 - $first_dow;
	for($i = 0; $i < $rows; $i++) {
		$ret .= '<div class="grid-row">';
		for($j = 0; $j < 7; $j++) {
			$cls = ($idx < 1 || $idx > $mlen) ? 'outside' : 'inside';
			if($idx == $day) $cls .= ' current';
			$cls .= ($j == 0 || $j == 6) ? ' quiet' : '';
			$idxoff = $idx >= 1 ? '+'.($idx-1) : $idx-1;
			$dayn = date('j', strtotime("$idxoff days", $first));
			$ret .= '<div class="grid-cell number-cell day responsive '.$cls.'" tabindex="0"><span class="number">'.$dayn.'</span></div>';
			$idx ++;
		}
		$ret .= '</div>';
	}
	$ret .= '</div>';
	$ret .= '<footer class="card-footer panel-footer">';
	$ret .= '<div class="text-right"><a href="#" class="input-link text-error">cancel</a> &middot; <a href="#" class="input-link text-success">ok</a></div>';
	$ret .= '</footer>';
	$ret .= '</div>';
	return $ret;
}

function renderTimeSel($h24 = false) {
	$ret = '<div class="panel-outer panel-border panel-default" style="width: 250px">';
	$ret .= '<header class="panel-header">';
	$ret .= '<h5 class="text-label no-margin">Select time</h5>';
	$ret .= '</header>';
	$ret .= '<div class="panel-subheader">';
	$ret .= '<label class="text-label">hour</label>';
	$ret .= '</div>';
	$ret .= '<div class="panel-body grid-body">';
	$hour = 5;
	$min = 0;
	if($h24) {
		$tval = sprintf('%02d:%02d', $hour, $min);
		for($i = 0; $i < 24; $i++) {
			if($i == 12) $ret .= '</div>';
			if($i == 0 || $i == 12)
				$ret .= '<div class="grid-row">';
			$cls = ($i < 6 || $i >= 18) ? 'quiet' : '';
			if($i == $hour) $cls .= ' active';
			$ret .= '<div class="grid-cell number-cell responsive '.$cls.'" tabindex="0"><span class="number">'.sprintf('%02d', $i).'</span></div>';
		}
		$ret .= '</div>';
	} else {
		$tval = sprintf('%d:%02d pm', $hour, $min);
		for($i = 1; $i <= 12; $i++) {
			if($i == 7) $ret .= '</div>';
			if($i == 1 || $i == 7)
				$ret .= '<div class="grid-row">';
			$cls = ($i < 6 || $i >= 18) ? 'quiet' : '';
			if($i == $hour) $cls .= ' active';
			$ret .= '<div class="grid-cell number-cell responsive '.$cls.'" tabindex="0"><span class="number">'.$i.'</span></div>';
		}
		$ret .= '</div>';
	}
	$ret .= '</div>';
	$ret .= '<div class="panel-subheader">';
	$ret .= '<label class="text-label">minute</label>';
	$ret .= '</div>';
	$ret .= '<div class="panel-body grid-body">';
	$ret .= '<div class="grid-row">';
	for($i = 0; $i < 60; $i += 15) {
		$cls = $i == $min ? 'active' : '';
		$ret .= '<div class="grid-cell number-cell responsive '.$cls.'" tabindex="0"><span class="number">:'.sprintf('%02d', $i).'</span></div>';
	}
	$ret .= '</div>';
	$ret .= '</div>';
	$ret .= '<div class="panel-subheader">';
	$ret .= '<label class="text-label">enter time</label>';
	$ret .= '</div>';
	$ret .= '<div class="panel-body grid-body">';
	$ret .= '<div class="grid-row">';
	$ret .= '<input type="text" class="input-text input-block flat" size="12" value="'.$tval.'"></input>';
	if(! $h24) {
		$ret .= '<div class="input-button-group">';
		$ret .= renderCheck('meridiem', 'meridiem_am', 'box no-icon', 'AM', 'radio');
		$ret .= renderCheck('meridiem', 'meridiem_pm', 'box no-icon', 'PM', 'radio', 'checked=1');
		$ret .= '</div>';
	}
	$ret .= '</div>';
	$ret .= '</div>';
	$ret .= '<footer class="panel-footer">';
	$ret .= '<div class="text-right"><a href="#" class="input-link text-error">cancel</a> &middot; <a href="#" class="input-link text-success">ok</a></div>';
	$ret .= '</footer>';
	$ret .= '</div>';
	return $ret;
}

function arrangeTable($elements, $columns=2) {
	$depth = array_fill(0, $columns, 0);
	$ret = ['type' => 'section', 'rows' => []];
	$row = [];
	$cidx = 0;
	$ridx = 0;
	foreach($elements as $e) {
		if(is_array($e)) {
			$cspan = array_get_default($e, 'colspan', 1);
			$rspan = array_get_default($e, 'rowspan', 1);
			if(! $cspan || ! $rspan) continue;
			if(isset($e['elements'])) {
				$ecols = array_get_default($e, 'columns', $cspan);
				$e['elements'] = arrangeTable($e['elements'], $ecols);
			} else {
				$fill = $e;
			}
		}
		else {
			$cspan = $rspan = 1;
			$fill = $e;
		}
		
		do {
			$wrap = false;
			for($cidx = 0; ; $cidx ++) {
				if($cidx == $columns) {
					$wrap = true;
					break;
				}
				else if($depth[$cidx] <= $ridx)
					break;
			}
			if(! $wrap) {
				$cmax = min($cidx + $cspan, $columns);
				for($i = $cidx; $i < $cmax; $i ++) {
					if($depth[$i] > $ridx) {
						$wrap = true;
						break;
					}
				}
			}
			if(! $wrap) {
				for($i = $cidx; $i < $cmax; $i ++) {
					$depth[$i] += $rspan;
				}
				$row[] = $fill;
				$cidx = $cmax;
				if($cidx < $columns)
					// check next
					break;
			}
			$ret['rows'][] = ['elements' => $row];
			$row = [];
			$ridx ++;
			for($i = 0; $i < $columns; $i++) {
				if($depth[$i] < $ridx)
					$depth[$i] = $ridx;
			}
			if(! $wrap)
				break;
		}
		while(1);
	}
	if($row) {
		if($ret['rows']) {
			$ret['rows'][] = ['elements' => $row];
			return $ret;
		}
		return ['elements' => $row];
	}
	return $ret;
}



function arrangeForm(array $elements, $columns=2, $type='section') {
	if($columns < 1) $columns = 1;
	$sections = [
		[
			'type' => 'section',
			'columns' => $columns,
			'colspan' => $columns,
			'elements' => [],
		],
	];
	$rows = [0 => []];		
	$sect_idx = array_fill(0, $columns, 0);
	$sect_row = [0 => 0];
	$sect_col = [0 => 0];
	$sect_closed = [0 => false];
	$hfill = [0 => 0];
	$vfill = [0 => 0];
	$cidx = 0;
	$ridx = 0;
	$lclear = $rclear = false;
	$rclear_idx = 0;
	
	foreach($elements as $e) {
		if(is_array($e)) {
			$cspan = min(array_get_default($e, 'colspan', 1), $columns);
			$rspan = array_get_default($e, 'rowspan', 1);
			$ecols = array_get_default($e, 'columns', $cspan);
			$type = array_get_default($e, 'type');
			$clear = array_get_default($e, 'clear', 'none');
			if(! $cspan || ! $rspan) continue;
			if(isset($e['elements']) && is_array($e['elements'])) {
				if(! $type) $type = $e['type'] = 'section';
				if($type == 'row')
					$e['elements'] = arrangeForm($e['elements'], $ecols)['elements'][0]['elements'];
				else
					$e = arrangeForm($e['elements'], $ecols, $type);
				$e['colspan'] = $cspan;
				$e['rowspan'] = $rspan;
				// fixme - copy any other properties
			}
			else if(! $type) {
				if(! empty($e['name'])) {
					$type = $e['type'] = 'field';
				}
				else if(! empty($e['widget'])) {
					$type = $e['type'] = 'widget';
				}
				else if(isset($e['value'])) {
					$type = $e['type'] = 'static';
				}
				else
					$type = $e['type'] = 'spacer';
			}
			$fill = $e;
		}
		else {
			$cspan = $rspan = 1;
			$clear = 'none';
			if($e)
				$fill = ['type' => 'field', 'name' => $e];
			else
				$fill = ['type' => 'spacer'];
		}
		$lclear = ($clear == 'left' || $clear == 'both' || $rclear);
		$rclear = ($clear == 'right' || $clear == 'both');
		
		$placed = false;
		while(! $placed) {
			$sidx = $sect_idx[$cidx];
			if(empty($sect_closed[$sidx])) {
				$rmax = $rspan; //($sidx == 0 ? $rspan : $sections[$sidx]['rowspan'] - $vfill[$sidx]);
				if((! $lclear || $cidx == 0) && ($ridx >= $rclear_idx) && $rspan <= $rmax && $cspan + $hfill[$sidx] <= $sections[$sidx]['columns']) {
					// can place here
					if($rspan > 1) {
						// have a cell with rowspan - create new sections around it
						if($hfill[$sidx]) {
							$sections[] = [
								'type' => 'section',
								'columns' => $hfill[$sidx],
								'colspan' => $hfill[$sidx],
								'rowspan' => $rspan,
								'elements' => [
									['type' => 'row', 'elements' => $rows[$sidx]],
								],
								'parent_idx' => $sidx,
							];
							$new_sidx = count($sections) - 1;
							$sect_col[$new_sidx] = $sect_col[$sidx];
							$sect_row[$new_sidx] = $sect_row[$sidx] + $vfill[$sidx];
							$hfill[$new_sidx] = 0;
							$vfill[$new_sidx] = 1;
							$rows[$new_sidx] = [];
							$rows[$sidx] = [];
							for($i = $sect_col[$sidx]; $i < $cidx; $i ++) {
								$sect_idx[$i] = $new_sidx;
							}
						}
						$fill['colspan'] = $cspan;
						$fill['parent_idx'] = $sidx;
						$sections[] = $fill;
						$fill_sidx = count($sections) - 1;
						$sect_col[$fill_sidx] = $cidx;
						$sect_row[$fill_sidx] = $sect_row[$sidx] + $vfill[$sidx];
						$sect_closed[$fill_sidx] = true;
						$hfill[$fill_sidx] = 0;
						$vfill[$fill_sidx] = $rspan;
						for($i = 0; $i < $cspan; $i ++) {
							$sect_idx[$cidx + $i] = $fill_sidx;
						}
						if($hfill[$sidx] + $cspan < $sections[$sidx]['columns']) {
							$sections[] = [
								'type' => 'section',
								'columns' => $sections[$sidx]['columns'] - $hfill[$sidx] - $cspan,
								'colspan' => $sections[$sidx]['columns'] - $hfill[$sidx] - $cspan,
								'rowspan' => $rspan,
								'elements' => [],
								'parent_idx' => $sidx,
							];
							$new_sidx = count($sections) - 1;
							$sect_col[$new_sidx] = $cidx + $cspan;
							$sect_row[$new_sidx] = $sect_row[$sidx] + $vfill[$sidx];
							$hfill[$new_sidx] = 0;
							$vfill[$new_sidx] = 0;
							$rows[$new_sidx] = [];
							for($i = 0; $i < $sections[$new_sidx]['colspan']; $i ++) {
								$sect_idx[$cidx + $cspan + $i] = $new_sidx;
							}
						}
						$sidx = $fill_sidx;
					} else {
						$rows[$sidx][] = $fill;
						$hfill[$sidx] += $cspan;
					}
					if($rclear)
						$rclear_idx = $ridx + $rspan;
					$cidx += $cspan;
					$placed = true;
				}
				else {
					// fill space
					$fill_c = min($cspan, $sections[$sidx]['columns'] - $hfill[$sidx]);
					if($fill_c) {
						$rows[$sidx][] = ['type' => 'spacer', 'colspan' => $fill_c];
						$hfill[$sidx] += $fill_c;
						$cidx += $fill_c;
					}
				}
			}
			else
				$cidx ++;

			if(empty($sect_closed[$sidx]) && $hfill[$sidx] == $sections[$sidx]['columns']) {
				// append current row
				$sections[$sidx]['elements'][] = ['type' => 'row', 'elements' => $rows[$sidx]];
				$hfill[$sidx] = 0;
				$vfill[$sidx] ++;
				$rows[$sidx] = [];
				if($sidx != 0 && $vfill[$sidx] >= $sections[$sidx]['rowspan']) {
					// finish section
					$sections[$sidx]['rows'] = $vfill[$sidx];
					$sect_closed[$sidx] = true;
				}
			}
			
			if($cidx == $columns) {
				$cidx = 0;
				$ridx ++;

				// close sections
				while(1) {
					$close_idx = [];
					$sub_idx = [];
					for($i = 0; $i < $columns; $i++) {
						$sidx = $sect_idx[$i];
						if(! $sidx) continue;
						$pidx = $sections[$sidx]['parent_idx'];
						if(empty($sect_closed[$sidx])) {
							$close_idx[$pidx] = 0;
						}
						else if(! isset($close_idx[$pidx]) || $close_idx[$pidx]) {
							$close_idx[$pidx] = 1;
							$sub_idx[$pidx][$sidx] = 1;
						}
						while($pidx != 0) {
							$sidx = $pidx;
							$pidx = $sections[$sidx]['parent_idx'];
							$close_idx[$pidx] = 0;
						}
					}
					if(! array_filter($close_idx)) break;
					
					foreach($close_idx as $pidx => $_) {
						if(! $_) continue;
						$row = [];
						$max_rspan = 0;
						$s_vfill = [];
						foreach($sub_idx[$pidx] as $sidx => $_) {
							$max_rspan = max($max_rspan, $sections[$sidx]['rowspan']);
							$row[] = $sections[$sidx];
							$s_vfill[] = $vfill[$sidx];
						}
						for($i = 0; $i < count($row); $i++) {
							while($s_vfill[$i] < $max_rspan) {
								$row[$i]['elements'][] = ['type' => 'row', 'elements' => ['type' => 'spacer', 'colspan' => $row[$i]['columns']]];
								$s_vfill[$i] ++;
							}
						}
						$sections[$pidx]['elements'][] = ['type' => 'row', 'elements' => $row];
						$hfill[$pidx] = 0;
						$vfill[$pidx] += $s_vfill[0];
						$sect_cspan = $sections[$pidx]['colspan'];
						for($i = 0; $i < $sect_cspan; $i ++) {
							$sect_idx[$sect_col[$pidx] + $i] = $pidx;
						}
						if($pidx && $vfill[$pidx] >= $sections[$pidx]['rowspan']) {
							$sections[$pidx]['rows'] = $vfill[$pidx];
							$sect_closed[$pidx] = true;
						}
					}
				}
			}
		}
	}
	
	// append all rows
	for($i = 0; $i < $columns; $i++) {
		$sidx = $sect_idx[$i];
		if(empty($sect_closed[$sidx]) && $hfill[$sidx]) {
			if($hfill[$sidx] < $sections[$sidx]['columns'])
				$rows[$sidx][] = ['type' => 'spacer', 'colspan' => $sections[$sidx]['columns'] - $hfill[$sidx]];
			$sections[$sidx]['elements'][] = ['type' => 'row', 'elements' => $rows[$sidx]];
			$hfill[$sidx] = 0;
			$vfill[$sidx] ++;
			$sections[$sidx]['rows'] = $vfill[$sidx];
			$sect_closed[$sidx] = true;
		}
	}
	
	return $sections[0];
}


function renderRows($layout) {
	$ret = '';
	$cols = $layout['columns'];
	foreach($layout['elements'] as $row) {
		$ret .= '<div class="row form-row col-unstack">';
		foreach($row['elements'] as $col) {
			$cspan = array_get_default($col, 'colspan', 1);
			$rspan = array_get_default($col, 'rowspan', 1);
			$cls = 'column';
			$sty = '';
			$perc = $cspan / $cols;
			$pcols = $perc * 12;
			if(round($pcols) == $pcols)
				$cls .= ' sm-'.$pcols;
			else
				$sty = ' style="flex-basis: '.($perc*100).'%"';
			if($col['type'] == 'section') {
				$cls .= ' form-section';
				$cell = renderRows($col);
			}
			else if($col['type'] == 'field') {
				$cls .= ' form-cell';
				$cell = $col['name'];
				$cell .= " <span class=\"text-muted small\">($cspan x $rspan)</span>";
			}
			else if($col['type'] == 'spacer') {
				$cls .= ' form-cell form-spacer';
				$cell = "<em class=\"text-muted\">(spacer x $cspan)</em>";
			}
			else {
				$cell = '';
			}
			$ret .= "<div class=\"$cls\"$sty>$cell</div>";
		}
		$ret .= '</div>';
	}
	return $ret;
}


?>

<!DOCTYPE html>
<html>
<head>
<base href="<?php echo to_html(AppConfig::site_url()); ?>">
<meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1">
<meta charset="<?php echo to_html(AppConfig::charset()); ?>">
<meta name="viewport" content="width=device-width, initial-scale=1.0">
<title>UI Controls</title>
<link rel="stylesheet" type="text/css" href="<?php echo $css_info['include_path']; ?>">
<link rel="stylesheet" type="text/css" href="themes/Default/theme-icons.css">
<!-- link href="https://fonts.googleapis.com/css?family=Source+Sans+Pro" rel="stylesheet" -->
<!-- link href="https://fonts.googleapis.com/css?family=Overpass:300,700" rel="stylesheet" -->
<script src="include/javascript/base.js"></script>
<script src="include/javascript/lightweight.js"></script>
<script src="include/javascript/conn.js"></script>
</head>
<body>


<div class="theme-select text-center mrgn-bttm-lg">
	<form method="GET">
		<input type="hidden" name="module" value="LayoutTest">
		<input type="hidden" name="action" value="Less">
		Theme: <select class="input-field" name="theme" onchange="this.form.submit()">
		<?php
			$theme_list = ['Default' => 'Default'] + AppConfig::get_themes();
			foreach($theme_list as $tk => $tv) {
				$sel = $tk == $theme_name ? 'selected' : '';
				echo '<option value="'.$tk.'" '.$sel.'>'.to_html($tv).'</option>';
			}
		?>
		</select>
	</form>
</div>

<?php



$vals = [
'A',
'B',
'C',
'D',
['rowspan' => 2, 'name' => 'E'],
'F',
'G',
'H',
'I',
['colspan' => 2, 'name' => 'J'],
'K',
'L',
['colspan' => 2, 'rowspan' => 2, 'elements' => ['M', 'N', 'O', 'P'], 'clear' => 'right'],
['colspan' => 2, 'name' => 'Q'],
'R',
['colspan' => 2, 'elements' => ['S', 'T']],
'U',
'V',
'W',
'X',
'Y',
'Z',
];

/*$layout = arrangeForm($vals, 4);
echo <<<EOS
<section class="container">
<div class="demo-block">
	<div class="card-outer card-form form-panel expand">
		<div class="card-header"><h3 class="form-title">Form test</h3></div>
		<div class="card-body form-section">
EOS;
echo renderRows($layout);
echo '</div></div></section></div>';
echo '</body></html>';
return;*/


?>

<section class="container">
<div class="demo-block">

<div class="demo-title">
	Text formatting
</div>
<div class="row demo-row header">
<div class="demo-inline"><h1>Heading 1</h1></div>
<div class="demo-inline"><h2>Heading 2</h2></div>
<div class="demo-inline"><h3>Heading 3</h3></div>
<div class="demo-inline"><h4>Heading 4</h4></div>
<div class="demo-inline"><h5>Heading 5</h5></div>
<div class="demo-inline"><h6>Heading 6</h6></div>
</div>
<div class="row demo-row header">
<div class="demo-inline">
	<p class="lead">Lead paragraph</p>
	<p>Regular paragraph</p>
</div>
</div>
<div class="row demo-row">
<div class="demo-inline"><a href="#">Link</a></div>
<div class="demo-inline"><span class="text-muted">Muted</span></div>
<div class="demo-inline"><span class="text-active">Active</span></div>
<div class="demo-inline"><span class="text-success">Success</span></div>
<div class="demo-inline"><span class="text-error">Error</span></div>
<div class="demo-inline"><span class="text-info">Info</span></div>
<div class="demo-inline"><span class="text-draft">Draft</span></div>
<div class="demo-inline"><span class="text-special">Special</span></div>
<div class="demo-inline"><b>Bold</b></div>
<div class="demo-inline"><em>Italic</em></div>
<div class="demo-inline"><b><em>Bold/Italic</em></b></div>
<div class="demo-inline"><code>Monospace</code></div>
</div>
<div class="row demo-row">
<div class="demo-inline">
	<div class="badge">Badge</div>
	<div class="badge state-label">Label</div>
	<div class="badge state-active">Active</div>
	<div class="badge state-selected">Selected</div>
	<div class="badge state-success">Success</div>
	<div class="badge state-error">Error</div>
	<div class="badge state-info">Info</div>
	<div class="badge state-final">Final</div>
	<div class="badge state-draft">Draft</div>
	<div class="badge state-pending">Pending</div>
	<div class="badge state-deferred">Deferred</div>
	<div class="badge state-special">Special</div>
	<div class="badge state-closed">Closed</div>
	<div class="badge state-dead">Dead</div>
	<div class="badge state-alert">Alert</div>
	<div class="badge state-warning">Warning</div>
	<div class="badge state-danger">Danger</div>
</div>
</div>

<div class="demo-title">
	Icons
</div>
<div class="row demo-row">
<div class="demo-inline">
	<div class="demo-icon" title="1crm">1</div>
	<div class="demo-icon" title="required">*</div>
	<div class="demo-icon" title="previous">◂</div>
	<div class="demo-icon" title="up arrow">▴</div>
	<div class="demo-icon" title="next">▸</div>
	<div class="demo-icon" title="down arrow">▾</div>
	<div class="demo-icon" title="up chevron">⌃</div>
	<div class="demo-icon" title="down chevron">⌄</div>
	<div class="demo-icon" title="to top">⌅</div>
	<div class="demo-icon" title="up arrow">↑</div>
	<div class="demo-icon" title="down arrow">↓</div>
	<div class="demo-icon" title="left arrow">←</div>
	<div class="demo-icon" title="right arrow">→</div>
	<div class="demo-icon" title="plus">＋</div>
	<div class="demo-icon" title="attachment">⛓</div>
	<div class="demo-icon" title="accept">✔</div>
	<div class="demo-icon" title="cancel">❌</div>
	<div class="demo-icon" title="small check">✓</div>
	<div class="demo-icon" title="small remove">✕</div>
	<div class="demo-icon" title="check off">☐</div>
	<div class="demo-icon" title="check on">☑</div>
	<div class="demo-icon" title="radio off">○</div>
	<div class="demo-icon" title="radio on">☒</div>
	<div class="demo-icon" title="star off">☆</div>
	<div class="demo-icon" title="star on">★</div>
	<div class="demo-icon" title="first">⏮</div>
	<div class="demo-icon" title="fprevious">⏪</div>
	<div class="demo-icon" title="previous">◀</div>
	<div class="demo-icon" title="next">▶</div>
	<div class="demo-icon" title="fnext">⏩</div>
	<div class="demo-icon" title="last">⏭</div>
	<div class="demo-icon" title="info">ℹ</div>
	<div class="demo-icon" title="danger">❗</div>
	<div class="demo-icon" title="add">+</div>
	<div class="demo-icon" title="remove">⛔</div>
	<div class="demo-icon" title="ok">k</div>
	<div class="demo-icon" title="clear">ⓧ</div>
	<div class="demo-icon" title="help">?</div>
	<div class="demo-icon" title="split">s</div>
	<div class="demo-icon" title="email">e</div>
	<div class="demo-icon" title="extern">u</div>
	<div class="demo-icon" title="mobile">m</div>
	<div class="demo-icon" title="phone">p</div>
	<div class="demo-icon" title="user">♙</div>
	<div class="demo-icon" title="lock">⚷</div>
	<div class="demo-icon" title="sign in">&lt;</div>
	<div class="demo-icon" title="sign out">&gt;</div>
	<div class="demo-icon" title="hourglass">⌛</div>
	<div class="demo-icon" title="gear">⚙</div>
	<div class="demo-icon" title="refresh">↺</div>
	<div class="demo-icon" title="search">⚲</div>
	<div class="demo-icon" title="date">d</div>
	<div class="demo-icon" title="time">t</div>
	<div class="demo-icon" title="hamburger">≡</div>
	<div class="demo-icon" title="dots">:</div>
	<div class="demo-icon" title="odots">.</div>
	<div class="demo-icon" title="print">n</div>
</div>
<div class="demo-inline">
	<div class="demo-icon" title="grey led"><span class="uii uii-led led-grey"></span></div>
	<div class="demo-icon" title="green led"><span class="uii uii-led led-green"></span></div>
	<div class="demo-icon" title="red led"><span class="uii uii-led led-red"></span></div>
	<div class="demo-icon" title="yellow led"><span class="uii uii-led led-yellow"></span></div>
	<div class="demo-icon" title="violet led"><span class="uii uii-led led-violet"></span></div>
	<div class="demo-icon" title="blue led"><span class="uii uii-led led-blue"></span></div>
	<div class="demo-icon" title="orange led"><span class="uii uii-led led-orange"></span></div>
	<div class="demo-icon" title="last"><span class="glyphicon glyphicon-check"></span></div>
</div>
</div>

<div class="demo-title">
	Checkboxes
</div>
<div class="row demo-row">
<div class="demo-inline"><?php echo renderToggle('togtest', 'togtest_sm', 'input-sm', 'Small'); ?></div>
<div class="demo-inline"><?php echo renderToggle('togtest', 'togtest', '', 'Normal', 'checkbox', 'checked=1'); ?></div>
<div class="demo-inline"><?php echo renderToggle('togtest', 'togtest_lg', 'input-lg', 'Large'); ?></div>
<div class="demo-inline"><?php echo renderToggle('togtest', 'togtest_dis', '', 'Disabled', 'checkbox', 'disabled=1'); ?></div>
<div class="demo-inline">
	<div class="input-toggle readonly checked">
		<label class="input-label text-label">
			<div class="switch"></div>
			Read-only
		</label>
	</div>
</div>
</div>
<div class="row demo-row">
<div class="demo-inline"><?php echo renderCheck('checktest', 'checktest_sm', 'small', 'Small'); ?></div>
<div class="demo-inline"><?php echo renderCheck('checktest', 'checktest', '', 'Normal', 'checkbox', 'checked=1'); ?></div>
<div class="demo-inline"><?php echo renderCheck('checktest', 'checktest_lg', 'large', 'Large'); ?></div>
<div class="demo-inline"><?php echo renderCheck('checktest', 'checktest_dis', '', 'Disabled', 'checkbox', 'disabled=1'); ?></div>
<div class="demo-inline"><?php echo renderCheck('checktest', 'checktest_dis', 'input-button', 'Button', 'checkbox'); ?></div>
<div class="demo-inline"><?php echo renderCheck('checktest', 'checktest_dis', 'input-field', 'Field', 'checkbox'); ?></div>
</div>

<div class="demo-title">
	Radio inputs
</div>
<div class="row demo-row">
<div class="demo-inline"><?php echo renderToggle('radiotest', 'radiotest_sm', 'input-sm', 'Small', 'radio'); ?></div>
<div class="demo-inline"><?php echo renderToggle('radiotest', 'radiotest', '', 'Normal', 'radio', 'checked=1'); ?></div>
<div class="demo-inline"><?php echo renderToggle('radiotest', 'radiotest_lg', 'input-lg', 'Large', 'radio'); ?></div>
</div>
<div class="row demo-row">
<div class="demo-inline"><?php echo renderCheck('radiotest', 'radiotest_sm', 'small', 'Small', 'radio'); ?></div>
<div class="demo-inline"><?php echo renderCheck('radiotest', 'radiotest', '', 'Normal', 'radio', 'checked=1'); ?></div>
<div class="demo-inline"><?php echo renderCheck('radiotest', 'radiotest_lg', 'large', 'Large', 'radio'); ?></div>
<div class="demo-inline"><?php echo renderCheck('radiotest', 'radiotest_dis', '', 'Disabled', 'radio', 'disabled=1'); ?></div>
<div class="demo-inline"><?php echo renderCheck('radiotest', 'radiotest_dis', 'input-button', 'Button', 'radio'); ?></div>
<div class="demo-inline"><?php echo renderCheck('radiotest', 'radiotest_dis', 'input-field', 'Field', 'radio'); ?></div>
</div>

<div class="demo-title">
	Text inputs
</div>
<div class="row demo-row">
<div class="demo-inline"><input type="text" class="input-text input-sm" value="Small text input" style="width: 10em" /></div>
<div class="demo-inline"><input type="text" class="input-text" value="Normal text input" style="width: 10em" /></div>
<div class="demo-inline"><input type="text" class="input-text input-lg" value="Large text input" style="width: 10em" /></div>
<div class="demo-inline"><input type="text" class="input-text" readonly=1 value="Read-only text" style="width: 10em" /></div>
<div class="demo-inline"><input type="text" class="input-text" disabled=1 value="Disabled text" style="width: 10em" /></div>
<div class="demo-inline"><input type="text" class="input-text" placeholder="Placeholder" style="width: 10em" /></div>
</div>
<div class="row demo-row">
<div class="demo-inline"><textarea class="input-text input-sm" style="width: 10em">Small text area</textarea></div>
<div class="demo-inline"><textarea class="input-text" style="width: 10em">Normal text area</textarea></div>
<div class="demo-inline"><textarea class="input-text input-lg" style="width: 10em">Large text area</textarea></div>
<div class="demo-inline"><textarea class="input-text" readonly=1 style="width: 10em">Read-only text area</textarea></div>
<div class="demo-inline"><textarea class="input-text" disabled=1 style="width: 10em">Disabled text area</textarea></div>
<div class="demo-inline"><input type="text" class="input-text flat" value="Flat text" style="width: 10em" /></div>
</div>

<div class="demo-title">
	Button inputs  
</div>
<div class="row demo-row">
<div class="demo-inline"><button type="button" class="input-button input-sm"><span class="input-label">Small button</span></button></div>
<div class="demo-inline"><button type="button" class="input-button"><span class="input-label">Normal button</span></button></div>
<div class="demo-inline"><button type="button" class="input-button input-lg"><span class="input-label">Large button</span></button></div>
<div class="demo-inline"><button type="button" class="input-button"><div class="input-icon theme-icon module-Cases left"></div><span class="input-label">Icon button</span></button></div>
<div class="demo-inline"><button type="button" class="input-button" disabled=1><span class="input-label">Disabled button</span></button></div>
<div class="demo-inline">
	<button class="input-button rbullet" tabindex="0"><div class="input-label">Dropdown button</div></button>
</div>
<div class="demo-inline"><a href="#" class="input-button" tabindex="0"><div class="input-label">Link button</div></a></div>
<div class="demo-inline">
	<button type="button" class="input-button loading-test" tabindex="0">
		<div class="input-label">Loading button</div>
		<div class="wait-outer">
			<div class="wait-icon dynamic heavy">
				<div class="bar1"></div><div class="bar2"></div><div class="bar3"></div>
			</div>
		</div>
	</button>
</div>
<!-- div class="demo-inline">
	<button type="button" class="input-button glyph-test" tabindex="0">
		<span class="glyphicon glyphicon-plus"></span>
	</button>
</div -->

</div>

<div class="demo-title">
	Field inputs
</div>
<div class="row demo-row">
<div class="demo-inline">
	<div class="input-field rbullet input-sm" tabindex="0"><div class="input-label">Small dropdown</div></div>
</div>
<div class="demo-inline">
	<div class="input-field rbullet" tabindex="0"><div class="input-label">Normal dropdown</div></div>
</div>
<div class="demo-inline">
	<div class="input-field rbullet input-lg" tabindex="0"><div class="input-label">Large dropdown</div></div>
</div>
<div class="demo-inline">
	<div class="input-field" tabindex="0"><div class="input-label"><div class="input-icon theme-icon module-Cases left"></div>Field with icon</div></div>
</div>
<div class="demo-inline">
	<div class="input-field readonly" tabindex="0"><div class="input-label">Read-only field</div></div>
</div>
<div class="demo-inline">
	<div class="input-field rbullet disabled"><div class="input-label">Disabled dropdown</div></div>
</div>
<div class="demo-inline">
	<div class="input-field loading-test" tabindex="0">
		<div class="input-label">Loading field</div>
		<div class="wait-outer">
			<div class="wait-icon dynamic heavy">
				<div class="bar1"></div><div class="bar2"></div><div class="bar3"></div>
			</div>
		</div>
	</div>
</div>
<div class="demo-inline">
	<div class="input-field rbullet muted" tabindex="0">
		<div class="input-label">Muted dropdown</div>
	</div>
</div>
<div class="demo-inline">
	<div class="input-field rbullet flat" tabindex="0">
		<div class="input-label">Flat dropdown</div>
	</div>
</div>
</div>


<div class="demo-title">
	Long-text fields
</div>
<div class="row demo-row">
<div class="demo-inline">
	<div class="input-field rbullet" style="max-width: 15em" tabindex="0"><div class="input-label single">Single line field with a long long long long long input</div></div>
</div>
<div class="demo-inline">
	<div class="input-field rbullet" style="max-width: 12em" tabindex="0"><div class="input-label limit-height">Multi-line field limited to 3 lines which goes on and on here's some more text etc etc etc etc</div></div>
</div>
<div class="demo-inline">
	<div class="input-field rbullet" style="max-width: 12em" tabindex="0"><div class="input-label"><div class="input-icon theme-icon module-Cases left"></div>Multi-line field with an inline icon that goes on and on here's some more text etc etc etc etc</div></div>
</div>
<div class="demo-inline">
	<div class="input-field input-field-group rbullet" style="max-width: 9em" tabindex="0">
		<div class="input-label input-field-icon"><div class="input-icon theme-icon module-Cases"></div></div>
		<div class="input-label">Multi line text with a separated icon
			<?php echo renderToggle('checkinl', 'checkinl', 'input-sm', ''); ?>
		</div>
	</div>
</div>

</div>

<div class="demo-title">
	Tag inputs
</div>
<div class="row demo-row">
<div class="demo-inline">
	<div class="input-field state-alert input-sm"><div class="input-label">Inline</div></div>
	<div class="input-field input-field-group state-final"><div class="input-label">Tags</div><div class="input-label input-remove"></div></div>
	<div class="input-field state-danger input-lg rbullet"><div class="input-label">Test</div></div>
	<div class="input-field state-invert rbullet"><div class="input-label">Test</div></div>
</div>
<div class="demo-inline">
	<div class="input-field rbullet input-sm" tabindex="0">
		<div class="input-inline input-sm">
			<div class="input-field input-field-group state-label"><div class="input-label">Small</div><div class="input-label input-remove"></div></div>
			<div class="input-field input-field-group state-label"><div class="input-label">Tags</div><div class="input-label input-remove"></div></div>
			<div class="input-field input-field-group state-label"><div class="input-label">List</div><div class="input-label input-remove"></div></div>
		</div>
	</div>
</div>
<div class="demo-inline">
	<div class="input-field" style="max-width: 15em" tabindex="0">
		<div class="input-inline">
			<div class="input-field state-special"><div class="input-label">CSS</div></div>
			<div class="input-field input-field-group state-success"><div class="input-label">JavaScript</div><div class="input-label input-remove"></div></div>
			<div class="input-field state-warning"><div class="input-label">PHP</div></div>
			<div class="input-field state-active"><div class="input-label">Docker</div></div>
			<div class="input-field state-label"><div class="input-label">OpenShift</div></div>
			<div class="input-field state-info"><div class="input-label">Perl</div></div>
		</div>
	</div>
</div>
<div class="demo-inline">
	<div class="input-field rbullet input-lg" tabindex="0">
		<div class="input-inline input-lg">
			<div class="input-field state-label"><div class="input-label">Large</div></div>
			<div class="input-field state-label"><div class="input-label">Tags</div></div>
			<div class="input-field state-label"><div class="input-label">List</div></div>
		</div>
	</div>
</div>
<div class="demo-inline">
	<div class="input-field rbullet disabled">
		<div class="input-inline">
			<div class="input-field state-label"><div class="input-label">Disabled</div></div>
			<div class="input-field state-label"><div class="input-label">Tags</div></div>
			<div class="input-field state-label"><div class="input-label">List</div></div>
		</div>
	</div>
</div>
</div>


<div class="demo-title">
	Input groups
</div>
<div class="row demo-row">
<div class="demo-inline">
	<div class="input-field input-field-group">
		<div class="input-label input-addon state-label"><div class="input-icon theme-icon module-Cases"></div></div>
		<label class="input-label input-addon state-label" for="cell-group">Label</label>
		<div class="input-field rbullet border-left" id="cell-group" tabindex="0" style="max-width: 10em"><div class="input-label">Field over multiple lines</div></div>
	</div>
</div>
<div class="demo-inline">
	<div class="input-field input-field-group">
		<label class="input-label state-label input-addon" for="group-text">Input label</label>
		<input class="input-text border-left" value="Text input" id="group-text" />
	</div>
</div>
<div class="demo-inline">
	<div class="input-field input-field-group">
		<label class="input-label state-label input-addon" for="group-field">Input label</label>
		<div class="input-field rbullet border-left" id="group-field"><div class="input-label">Dropdown field</div></div>
	</div>
</div>
<div class="demo-inline">
	<div class="input-field input-field-group">
		<label class="input-label state-label input-addon" for="group-field">Input label</label>
		<button class="input-button rbullet border-left" id="group-field"><div class="input-label">Dropdown button</div></button>
	</div>
</div>
</div>
<div class="row demo-row">
<div class="demo-inline">
	<div class="input-button-group">
		<div class="input-button" tabindex="0"><div class="input-label">Sample</div></div>
		<div class="input-button" tabindex="0"><div class="input-label">Button</div></div>
		<div class="input-button" tabindex="0"><div class="input-label">Group</div></div>
	</div>
</div>
<div class="demo-inline">
	<div class="input-button-group">
		<div class="input-button rbullet" tabindex="0"><div class="input-label">Dropdown</div></div>
		<div class="input-button rbullet" tabindex="0"><div class="input-label">Button</div></div>
		<div class="input-button rbullet" tabindex="0"><div class="input-label">Group</div></div>
	</div>
</div>
<div class="demo-inline">
	<div class="input-button-group nav-button-group muted">
		<button type="button" title="Start" class="input-button nav-button" disabled="disabled">
			<i class="uii uii-first off"></i>
		</button>
		<button type="button" title="Previous" class="input-button nav-button" disabled="disabled">
			<i class="uii uii-previous off"></i>
		</button>
		<button type="button" class="input-button nav-button bar-insert">
			<span class="input-label rbullet rbullet-ddown no-sep">1–19 of 19</span>
		</button>
		<button type="button" title="Next" class="input-button nav-button">
			<i class="uii uii-next off"></i>
		</button>
		<button type="button" title="End" class="input-button nav-button">
			<i class="uii uii-last off"></i>
		</button>
	</div>
</div>
<div class="demo-inline">
	<div class="input-field-group">
		<div class="input-field state-final" tabindex="0"><div class="input-label">Sample</div></div>
		<div class="input-field state-info" tabindex="0"><div class="input-label">Tag</div></div>
		<div class="input-field state-danger" tabindex="0"><div class="input-label">Group</div></div>
	</div>
</div>
</div>


<div class="demo-title">
	Progress inputs
</div>
<div class="row demo-row">
<div class="demo-inline">
	<div class="input-field input-progress-outer input-sm state-success" style="width: 10em">
		<div class="input-progress">
			<div class="input-progress-bar state-active" style="width: 50%"><label>50<small>%</small></label></div>
			<div class="input-progress-bar state-final" style="width: 30%"></div>
		</div>
	</div>
</div>
<div class="demo-inline">
	<div class="input-field input-field-group input-success">
		<input type="text" class="input-text text-right" size="3" value="50"></input>
		<div class="input-label small border-right">%</div>
		<div class="input-field input-progress-outer" style="width: 15em">
			<div class="input-progress">
				<div class="input-progress-bar state-disabled" style="width: 50%"></div>
				<div class="input-progress-bar empty"></div>
			</div>
		</div>
	</div>
</div>
<div class="demo-inline">
	<div class="input-field input-progress-outer input-lg" style="width: 15em">
		<div class="input-progress">
			<div class="input-progress-bar state-alert" style="width: 30%"><label>30<small>%</small></label></div>
		</div>
	</div>
</div>
</div>


<div class="demo-title">
	File inputs
</div>
<div class="row demo-row">
<div class="demo-inline">
	<input type="file" data-placeholder="File input" class="input-field input-file" />
</div>
<div class="demo-inline">
	<div class="input-fileset">
		<input type="file" data-placeholder="File input" id="fileset" class="input-file" />
		<label for="fileset" class="input-field rbullet">
			<div class="input-label">
				<span class="uii">⛓</span>
				<span class="text-placeholder">Select a file</span>
			</div>
		</label>
	</div>
</div>
</div>


<div class="demo-title">
	Multi select
</div>
<div class="row demo-row">
<div class="demo-inline">
	<div class="input-field input-field-group rbullet" id="multisel-blank" tabindex="0" style="width: 10em">
		<div class="input-label text-placeholder">(placeholder)</div>
	</div>
	<div style="text-align: right; padding-right: 5px; margin-top: -1px">
		<div class="card-outer card-inline panel-outer panel-border panel-default popup-default flat-top hover-tools active">
			<div class="panel-header">
				<div class="tools-row">
					<span class="uii uii-search uii-lg active-icon"></span>
				</div>
			</div>
		</div>
	</div>
</div>
<div class="demo-inline">
	<div class="input-field input-sm rbullet" id="multisel-head" tabindex="0">
		<section class="input-scroll menu-outer" style="width: 150px; height: 100px">
			<p class="menu-option" tabindex="0">Small menu</p>
			<p class="menu-option" tabindex="0">Small menu</p>
			<p class="menu-option" tabindex="0">Small menu</p>
			<p class="menu-option drag-indicator"></p>
		</section>
	</div>
</div>
<div class="demo-inline">
	<div class="input-field rbullet" id="multisel" tabindex="0">
		<section class="input-scroll menu-outer" style="width: 200px; height: 100px">
			<div class="menu-header">Teams</div>
			<div class="menu-option input-label single" tabindex="0"><div class="input-icon theme-icon module-SecurityGroups left"></div>Team 1</div>
			<div class="menu-option input-label single" tabindex="0"><div class="input-icon theme-icon module-SecurityGroups left"></div>Team 2</div>
			<div class="menu-option input-label single" tabindex="0"><div class="input-icon theme-icon module-SecurityGroups left"></div>Team 3</div>
			<div class="menu-option input-label single" tabindex="0"><div class="input-icon theme-icon module-SecurityGroups left"></div>Team 4</div>
			<div class="menu-option input-label single" tabindex="0"><div class="input-icon theme-icon module-SecurityGroups left"></div>Team 5</div>
			<div class="menu-option input-label single" tabindex="0"><div class="input-icon theme-icon module-SecurityGroups left"></div>Team 6</div>
			<div class="menu-option input-label single" tabindex="0"><div class="input-icon theme-icon module-SecurityGroups left"></div>Team 7</div>
		</section>
	</div>
</div>
<div class="demo-inline">
	<div class="input-field input-lg rbullet" id="multisel-head" tabindex="0">
		<section class="input-scroll menu-outer" style="width: 150px">
			<p class="menu-option" tabindex="0">Large menu</p>
			<p class="menu-option" tabindex="0">Large menu</p>
		</section>
	</div>
</div>
</div>
<div class="row demo-row">
<div class="demo-inline">
	<div class="input-field input-field-group" id="to_addrs" tabindex="0">
		<label class="input-label input-addon" style="min-width: 3em">To:</label>
		<div class="input-inline border-left" style="width: 450px">
			<div class="input-field rbullet state-info">
				<div class="input-label">Andrew Whitehead <span class="text-muted">&lt;andrew@1crm.com&gt;</span></div>
			</div>
		</div>
		<div class="input-label input-addon">
			<button class="input-button">
				<i class="uii uii-add"></i> Add
			</button>
		</div>
	</div>
</div>
</div>


</div><!-- /block -->
<div class="demo-block">
<div class="demo-title">
	Panels
</div>
<div class="row demo-row">
<div class="demo-inline">
	<article class="panel-outer panel-border panel-default">
		<section class="panel-body panel-pad-body">
			<span>Default panel</span>
		</section>
	</article>
</div>
<div class="demo-inline">
	<article class="panel-outer panel-border panel-default">
		<header class="panel-header">
			<h3 class="no-margin">
				Panel header
			</h3>
		</header>
		<header class="panel-subheader">
			<h5 class="no-margin">
				Panel subheader
			</h5>
		</header>
		<section class="panel-body panel-pad-body">
			<span>Panel body</span>
		</section>
		<footer class="panel-footer">
			<span>Panel footer</span>
		</footer>
	</article>
</div>
<div class="demo-inline">
	<article class="panel-outer panel-border panel-default" style="max-width: 150px">
		<header class="panel-header">
			<h3 class="no-margin">
				Panel header
			</h3>
		</header>
		<article class="panel-body menu-outer">
			<h4 class="menu-header">Menu as panel-body</h4>
			<p class="menu-option rbullet rbullet-right no-sep" tabindex="0">Option 1</p>
			<p class="menu-option rbullet rbullet-right no-sep" tabindex="0">Option 2 with a longer label</p>
			<p class="menu-option rbullet rbullet-right no-sep" tabindex="0">Option 3</p>
		</article>
		<footer class="panel-footer">
			<p class="no-margin">Panel footer</p>
		</footer>
	</article>
</div>
<div class="demo-inline">
	<article class="card-outer popup-default panel-outer panel-border panel-default" style="width: 150px">
		<footer class="card-header panel-header">
			<p class="no-margin">Panel header</p>
		</footer>
		<ul class="card-body panel-body menu-outer">
			<li class="menu-option" tabindex="0">Menu option</li>
			<li class="menu-header rbullet">Menu header</li>
			<ul class="menu-outer nested">
				<li class="menu-option" tabindex="0">Menu option</li>
				<li class="menu-option" tabindex="0">Menu option</li>
				<li class="menu-header">Menu header</li>
				<li class="menu-option" tabindex="0">Menu option</li>
			</ul>
		</ul>
		<footer class="card-footer panel-footer">
			<p class="no-margin">Panel footer</p>
		</footer>
	</article>
</div>
<div class="demo-inline">
	<article class="card-outer panel-outer panel-border panel-default" style="max-width: 200px; max-height: 200px">
		<header class="card-header panel-header">
			<div class="input-field input-field-group input-search">
				<span class="uii uii-search uii-lg input-field-icon"></span>
				<input type="text" class="input-text"></input>
			</div>
		</header>
		<header class="card-header panel-subheader">
			<p class="no-margin">
				Recent items
			</p>
		</header>
		<article class="card-body panel-body menu-outer">
			<section class="card-body menu-outer">
				<div class="menu-option input-label" tabindex="0"><span class="input-icon theme-icon module-Accounts left"></span>Option 1</div>
				<div class="menu-option input-label" tabindex="0"><span class="input-icon theme-icon module-Cases left"></span>Option 2</div>
				<div class="menu-option input-label" tabindex="0"><span class="input-icon theme-icon module-Tasks left"></span>Option 3</div>
				<div class="menu-option input-label" tabindex="0"><span class="input-icon theme-icon module-Project left"></span>Option 4</div>
				<div class="menu-option input-label" tabindex="0"><span class="input-icon theme-icon module-Accounts left"></span>Option 5</div>
				<div class="menu-option input-label" tabindex="0"><span class="input-icon theme-icon module-Cases left"></span>Option 6</div>
				<div class="menu-option input-label" tabindex="0"><span class="input-icon theme-icon module-Tasks left"></span>Option 7</div>
				<div class="menu-option input-label" tabindex="0"><span class="input-icon theme-icon module-Project left"></span>Option 8</div>
				<div class="menu-option input-label" tabindex="0"><span class="input-icon theme-icon module-Accounts left"></span>Option 9</div>
				<div class="menu-option input-label" tabindex="0"><span class="input-icon theme-icon module-Cases left"></span>Option 10</div>
				<div class="menu-option input-label" tabindex="0"><span class="input-icon theme-icon module-Tasks left"></span>Option 11</div>
				<div class="menu-option input-label" tabindex="0"><span class="input-icon theme-icon module-Project left"></span>Option 12</div>
			</section>
		</article>
		<footer class="card-footer panel-footer">
			<p class="card-row">18 items found</p>
		</footer>
	</article>
</div>
</div> <!-- /row -->
<div class="row demo-row">
<div class="demo-inline">
<?php echo renderCalendar(); ?>
</div>
<div class="demo-inline">
<?php echo renderTimeSel(); ?>
</div>
</div> <!-- /row -->
<div class="row demo-row">
<div class="demo-inline">
	<article class="panel-outer panel-border panel-round panel-system">
		<header class="panel-header">
			<h4 class="no-margin">System panel</h4>
		</header>
		<header class="panel-subheader">
			<p class="no-margin">Subheader</p>
		</header>
		<section class="panel-body panel-pad-body">
			<p class="no-margin">Body</p>
		</section>
		<footer class="panel-footer">
			<p class="no-margin">Footer</h4>
		</footer>
	</article>
</div>
<div class="demo-inline">
	<article class="panel-outer panel-border panel-round panel-default compact panel-readonly">
		<header class="panel-header">
			<h5 class="no-margin">Readonly panel</h4>
		</header>
		<section class="panel-body panel-pad-body">
			<p class="no-margin">Body</p>
		</section>
		<footer class="panel-footer">
			<p class="no-margin">Footer</h4>
		</footer>
	</article>
</div>
<div class="demo-inline">
	<article class="panel-outer panel-border panel-round panel-muted compact">
		<header class="panel-header">
			<h5 class="no-margin">Muted panel</h4>
		</header>
		<section class="panel-body panel-pad-body">
			<p class="no-margin">Body</p>
		</section>
		<footer class="panel-footer">
			<p class="no-margin">Footer</h4>
		</footer>
	</article>
</div>
<div class="demo-inline">
	<article class="panel-outer panel-border panel-round panel-default panel-loading loading-test">
		<header class="panel-header">
			<h5 class="no-margin">Loading panel</h4>
		</header>
		<section class="panel-body panel-pad-body">
			<p class="no-margin">Body</p>
		</section>
		<footer class="panel-footer">
			<p class="no-margin">Footer</h4>
		</footer>
		<!-- div class="loading-mask" -->
		<div class="wait-outer">
			<div class="wait-icon dynamic heavy">
				<div class="bar1"></div><div class="bar2"></div><div class="bar3"></div>
			</div>
		</div>
	</article>
</div>
</div> <!-- /row -->
<div class="row demo-row">
<div class="demo-inline">
	<article class="panel-outer panel-border panel-round panel-dark">
		<header class="panel-header">
			<h4 class="no-margin">Dark panel</h4>
		</header>
		<section class="panel-body panel-pad-body">
			<p class="no-margin">Body</p>
		</section>
		<footer class="panel-footer">
			<p class="no-margin">Footer</h4>
		</footer>
	</article>
</div>
<div class="demo-inline">
	<article class="panel-outer panel-border panel-round panel-info">
		<header class="panel-header">
			<h4 class="no-margin">Info panel</h4>
		</header>
		<section class="panel-body panel-pad-body">
			<p class="no-margin"><span class="uii uii-info"></span> Body</p>
		</section>
		<footer class="panel-footer">
			<p class="no-margin">Footer</h4>
		</footer>
	</article>
</div>
<div class="demo-inline">
	<article class="panel-outer panel-border panel-round panel-final">
		<header class="panel-header">
			<h4 class="no-margin">Final panel</h4>
		</header>
		<section class="panel-body panel-pad-body">
			<p class="no-margin">Body</p>
		</section>
		<footer class="panel-footer">
			<p class="no-margin">Footer</h4>
		</footer>
	</article>
</div>
<div class="demo-inline">
	<article class="panel-outer panel-border panel-round panel-alert">
		<header class="panel-header">
			<h4 class="no-margin">Alert panel</h4>
		</header>
		<section class="panel-body panel-pad-body">
			<p class="no-margin">Body</p>
		</section>
		<footer class="panel-footer">
			<p class="no-margin">Footer</h4>
		</footer>
	</article>
</div>
<div class="demo-inline">
	<article class="panel-outer panel-border panel-round panel-danger">
		<header class="panel-header">
			<h4 class="no-margin">Danger panel</h5>
		</header>
		<section class="panel-body panel-pad-body">
			<p class="no-margin"><span class="uii uii-danger"></span> Body</p>
		</section>
		<footer class="panel-footer">
			<p class="no-margin">Footer</h4>
		</footer>
	</article>
</div>


</div> <!-- /row -->
</div>

</div><!-- /block -->
<div class="demo-block">
	<div class="row demo-row">
		<div class="column">
			<div class="input-tabgroup nowrap listview-tabs">
				<div class="input-tabgroup-inner">
					<div class="tab-outer normal current" style="z-index: 14">
						<div class="tab-button " tabindex="0">
							<div class="input-icon tab-icon " style="display: none"></div><span class="input-label tab-label">Browse All</span>
						</div>
					</div>
					<div class="tab-outer normal " style="z-index: 13">
						<div class="tab-button " tabindex="0">
							<div class="input-icon tab-icon " style="display: none"></div><span class="input-label tab-label">Quick Filter</span>
						</div>
					</div>
					<div class="tab-outer normal  postsep" style="z-index: 3">
						<div class="tab-button  no-label " tabindex="0">
							<div class="input-icon tab-icon icon-editlayout"></div><span class="input-label tab-label"></span>
						</div>
					</div>
				</div>
			</div>
			<div class="card-outer panel-outer listview-panel panel-system panel-border">
				<div class="card-header panel-header listview-header listview-search-header">
					<div class="card-outer panel-outer listview-filter">
						<div class="card-outer form-outer search-panel">
							<div class="card-body form-body">
								<div class="form-section">
									<div class="row form-row">
										<div class="filter-basic-outer column form-cell">
											<div class="filter-basic-inner form-inline">
												<div class="form-entry">
													<div class="form-value">
														<div class="input-field input-field-group input-search filter" style="width: 15em; ">
															<div class="input-label input-field-icon lg">
																<div class="uii uii-lg uii-search"></div>
															</div>
															<input type="text" class="input-text input-entry " autocapitalize="off" results="9" incremental="" size="15" name="filter_text" id="filter_text" value="" tabindex="1">
														</div>
													</div>
												</div>
											</div>
											<div class="filter-basic-controls">
												<button name="clearButton" tabindex="5" type="button" id="listView-e895-FilterFormclearButton-input" class="input-button"><span class="uii uii-clear"></span> <span class="input-label" id="listView-e895-FilterFormclearButton-input-label">Clear</span></button>
											</div>
										</div>
									</div>
								</div>
							</div>
						</div>
					</div>
				</div>
				<div class="card-header panel-subheader listview-header">
					<div class="row listview-nav-block compact">
						<div class="column listview-actions buttons-inline align-left">
							<div class="inline-elt">
								<button name="SubPanel_create" type="button" class="input-button first">
									<div class="input-icon left icon-edit"></div><span class="input-label" id="listView-e895-create-label">Create</span>
								</button>
							</div>
							<div class="inline-elt">
								<button class="input-button menu-source">
									<div class="input-arrow">
										<span class="input-label"><span class="uii uii-gear"></span> Actions</span>
									</div>
								</button>
							</div>
						</div>
						<div class="column listview-status align-left">&nbsp;Selected: <span id="listView-e895-SelectCountHead" class="text-number">0</span> of <span class="text-number">18</span></div>
					</div>
				</div>
				<div class="card-body panel-body listview-body">
					<table class="listView" cellpadding="0" cellspacing="0" border="0" width="100%">
						<thead class="listViewColumns">
							<tr class="listHead">
								<th class="listViewThS1 listViewMeta complex">
									<div class="input-field compact muted listview-select rbullet" tabindex="1" role="button" style="margin: -1px">
										<div class="input-label">
											<div class="input-check"><input type="checkbox" class="checkbox"  name="check-all" value="">
												<label for="listView-e895-check-all"></label>
											</div>
										</div>
									</div>
								</th>
								<th class="listViewThS1">&nbsp;</th>
								<th class="listViewThS1 complex" width="80%">
									<a href="#" class="listViewThLinkS1 listColLink">
										<div class="listColHead responsive sort-active">
											<div class="listColHeadInner">
												<div class="listColLabel nowrap with-arrow">Account Name</div>
												<div class="listColArrow">
													<div class="input-icon list-sort icon-sort-down"></div>
												</div>
											</div>
										</div>
									</a>
								</th>
							</tr>
						</thead>
						<tbody>
							<tr data-id="53c6014d-3742-ae2a-d431-59d021e863a1" class="listViewRow oddListRowS1">
								<td class="listViewTd listViewMeta" width="1%"><div class="input-check"><input onclick="sListView.checkItem('listView-e895', this)" type="checkbox" class="checkbox" name="mass[]" value="53c6014d-3742-ae2a-d431-59d021e863a1" autocomplete="off"><label></label></div>&nbsp;<span id="adspan_listView-e895_53c6014d-3742-ae2a-d431-59d021e863a1" class="uii uii-info active-icon" onmousedown="SUGAR.ui.stopPropagation(event);" onmouseout="return SUGAR.util.clearAdditionalDetailsCall()" onmouseover="return SUGAR.util.getAdditionalDetails('Account', '53c6014d-3742-ae2a-d431-59d021e863a1', 'adspan_listView-e895_53c6014d-3742-ae2a-d431-59d021e863a1', null, 'listview')" ></span></td>
								<td class="listViewTd"><span class="uii uii-star active-icon " onmousedown="SUGAR.ui.stopPropagation(event);" onclick="sListView.addToFavorites('listView-e895', this, 'Accounts', '53c6014d-3742-ae2a-d431-59d021e863a1'); return false;" title="Add To Favorites"></span></td>
								<td class="listViewTd"><span class="detailLink"><a class="listViewNameLink" href="index.php?module=Accounts&action=DetailView&record=53c6014d-3742-ae2a-d431-59d021e863a1" onclick="sListView.showDetail('listView-e895', {module:'Accounts',action:'DetailView',record:'53c6014d-3742-ae2a-d431-59d021e863a1'}, event); return false;">A16</a></span></td>
							</tr>
							<tr data-id="5a81a914-ecc6-0c0e-b374-57339043811b" class="listViewRow evenListRowS1">
								<td class="listViewTd listViewMeta" width="1%"><div class="input-check"><input onclick="sListView.checkItem('listView-e895', this)" type="checkbox" class="checkbox" name="mass[]" value="5a81a914-ecc6-0c0e-b374-57339043811b" autocomplete="off"><label></label></div>&nbsp;<span id="adspan_listView-e895_5a81a914-ecc6-0c0e-b374-57339043811b" class="uii uii-info active-icon" onmousedown="SUGAR.ui.stopPropagation(event);" onmouseout="return SUGAR.util.clearAdditionalDetailsCall()" onmouseover="return SUGAR.util.getAdditionalDetails('Account', '5a81a914-ecc6-0c0e-b374-57339043811b', 'adspan_listView-e895_5a81a914-ecc6-0c0e-b374-57339043811b', null, 'listview')" ></span></td>
								<td class="listViewTd"><span class="uii uii-star active-icon " onmousedown="SUGAR.ui.stopPropagation(event);" onclick="sListView.addToFavorites('listView-e895', this, 'Accounts', '5a81a914-ecc6-0c0e-b374-57339043811b'); return false;" title="Add To Favorites"></span></td>
								<td class="listViewTd"><span class="detailLink"><a class="listViewNameLink" href="index.php?module=Accounts&action=DetailView&record=5a81a914-ecc6-0c0e-b374-57339043811b" onclick="sListView.showDetail('listView-e895', {module:'Accounts',action:'DetailView',record:'5a81a914-ecc6-0c0e-b374-57339043811b'}, event); return false;">Account</a></span><div class="listAddLine">Investor</div></td>
							</tr>
						</tbody>
					</table>
				</div>
			</div>
		</div>
	</div>
</div><!-- /block -->
<div class="demo-block">
<div class="demo-title">
	Dialog test
</div>
<div class="row demo-row">
<div class="demo-inline">
	<article class="card-outer dialog-outer popup-dialog" style="width: 200px">
		<header class="card-header dialog-header tools-row align-top">
			<div class="dialog-title">
				<h5>Dialog title</h5>
			</div>
			<div class="spacer"></div>
			<div class="uii uii-cancel active-icon dialog-close"></div>
		</header>
		<header class="card-header dialog-buttons">
			<div class="row buttons-row">
				<div class="column buttons-inline align-left">
					<div class="inline-elt">
						<a class="input-button">Test</a>
					</div>
				</div>
			</div>
		</header>
		<section class="card-body dialog-body dialog-pad-body">
			<p>Dialog body</p>
		</section>
		<footer class="card-footer dialog-buttons">
			<div class="row buttons-row">
				<div class="column buttons-inline align-left">
					<div class="inline-elt">
						<a class="input-button">Test</a>
					</div>
				</div>
			</div>
		</footer>
		<footer class="card-footer dialog-footer tools-row align-bottom">
			<div class="dialog-status text-muted">status</div>
			<div class="dialog-resizer no-flex"></div>
		</footer>
	</article>
</div>
</div><!-- /row -->
</div><!-- /block -->


<div class="demo-block">
<div class="demo-title">
	Compatibility
</div>
<div class="row demo-row">
<div class="demo-inline">
	<button type="button" class="input-button">
		<div class="input-label"><span class="uii uii-accept"></span> Accept</div>
	</button>
</div>
<div class="demo-inline">
	<button type="button" class="input-button">
		<div class="input-label"><span class="uii uii-cancel"></span> Cancel</div>
	</button>
</div>
<div class="demo-inline">
	<input type="button" class="input-button" value="Classic button" />
</div>
<div class="demo-inline">
	<button type="button" class="input-select input-outer">
		<div class="input-icon theme-icon module-Cases left"></div>
		<div class="input-arrow select-label">
			<span class="input-label">Test input-select</span>
		</div>
	</button>
</div>
<div class="demo-inline">
	<select name="real_select" class="input-field">
		<option>Real select input</option>
	</select>
</div>
<div class="demo-inline">
	<select name="real_select" class="input-field" multiple>
		<option>Multi</option>
		<option>Select</option>
		<option>Input</option>
	</select>
</div>
</div>
<div class="row demo-row">
<div class="demo-inline">
	<div class="input-select-multi input-outer" id="listView-c863-ListUpdatereassign_objects-input" style="width: 10.25em" tabindex="0">
		<div id="listView-c863-ListUpdatereassign_objects-input-scroll" class="input-scroll select-inner" style="height: 60px;">
			<div class="scroll-content" style="top: 0;">
			<table class="select-rows" width="100%">
				<tr class="select-option-row"><td class="select-option "><div class="select-option-inner input-bounded"><span class="input-label">Contacts</span></div></td></tr><tr class="select-option-row"><td class="select-option "><div class="select-option-inner input-bounded"><span class="input-label">Activities</span></div></td></tr><tr class="select-option-row"><td class="select-option "><div class="select-option-inner input-bounded"><span class="input-label">History</span></div></td></tr><tr class="select-option-row"><td class="select-option "><div class="select-option-inner input-bounded"><span class="input-label">Quotes</span></div></td></tr><tr class="select-option-row"><td class="select-option "><div class="select-option-inner input-bounded"><span class="input-label">Invoices</span></div></td></tr><tr class="select-option-row"><td class="select-option "><div class="select-option-inner input-bounded"><span class="input-label">Opportunities</span></div></td></tr><tr class="select-option-row"><td class="select-option "><div class="select-option-inner input-bounded"><span class="input-label">Cases</span></div></td>
				</tr>
			</table>
			</div>
			<div class="scroll-button input-outer up"><div class="input-icon"></div></div>
			<div class="scroll-button input-outer down"><div class="input-icon"></div></div>
			<div class="scroll-button input-outer pos" style="height: 13px; top: 15px;"><div class="input-icon" style="top: -1px;"></div></div>
		</div>
	</div>
</div>
<div class="demo-inline">
	<div id="DetailFormphoto-input" class="image-input input-outer" tabindex="0"><div class="image-inner" style="width: 90px; height: 90px"><img src="themes/Default/images/no_user.png" border="0" alt="" width="90" height="90" data-placeholder="1" style="margin-top: 0px;"><div class="input-icon icon-popup image-popup-icon"></div></div><input type="hidden" name="photo" value=""><input type="hidden" name="photo_clear" value=""></div>
</div>
<div class="demo-inline">
	<button class="lw-iconbutton lwib-accept" type="button" tabindex="0">
		<span class="uii">✔</span>
		<span class="btn-label">Login</span>
	</button>
</div>
</div> <!-- /row -->
</div> <!-- /block -->


<div class="demo-block">
<div class="demo-title">
	Inline test
</div>
<div class="row demo-row">
<div class="demo-inline" style="border-bottom: 1px solid black">
	Test
	<span class="uii uii-lg uii-search"></span>
	<div class="input-field input-field-group input-search">
		<input type="text" class="input-text" value="Text"></input>
	</div>
	Test
	<div class="input-field input-field-group input-search">
		<span class="uii uii-lg uii-search input-field-icon"></span>
		<input type="text" class="input-text" value="Text"></input>
	</div>
	Test
</div>
</div>
</div> <!-- /block -->

<div class="demo-block">
<div class="demo-title">
	Line items test
</div>
<div class="row demo-row">
<div class="demo-inline" style="width: 100%">
	<section class="card-outer panel-outer panel-default line-items-outer panel-border panel-round">
		<div class="card-header panel-header">
			<div class="row form-row">
				<div class="column form-cell col-3">
					<div class="form-entry">
						<div class="form-label">Group Name</div>
						<div class="form-value">
							<input type="text" class="input-text flat" placeholder="(none)">
						</div>
					</div>
				</div>
				<div class="column form-cell col-3">
					<div class="form-entry">
						<div class="form-label">Group Stage</div>
						<div class="form-value">
							<div class="input-field flat rbullet"><div class="input-label">Draft</div></div>
						</div>
					</div>
				</div>
				<div class="column form-cell col-3">
					<div class="form-entry">
						<div class="form-label">Group Type</div>
						<div class="form-value">
							<div class="input-field flat rbullet"><div class="input-label">Products</div></div>
						</div>
					</div>
				</div>
				<div class="column form-cell col-3">
					<div class="form-entry">
						<div class="form-label">Default Pricing</div>
						<div class="form-value">
							<div class="input-field flat rbullet"><div class="input-label">Editable Prices</div></div>
						</div>
					</div>
				</div>
			</div>
		</div>
		<div class="card-header panel-subheader">
			<div class="row col-space-sm">
				<div class="column" style="flex: 0 0 30px"></div>
				<div class="column" style="font-weight: bold">Product or Assembly</div>
				<div class="column" style="font-weight: bold; flex: 0 0 16%">Pricing</div>
				<div class="column text-right" style="font-weight: bold; flex: 0 0 4rem">Qty</div>
				<div class="column text-right" style="font-weight: bold; flex: 0 0 12%">Cost</div>
				<div class="column text-right" style="font-weight: bold; flex: 0 0 12%">List Price</div>
				<div class="column text-right" style="font-weight: bold; flex: 0 0 12%">Unit Price</div>
				<div class="column text-right" style="font-weight: bold; flex: 0 0 12%">Extd. Price</div>
				<div class="column" style="flex: 0 0 40px"></div>
			</div>
		</div>
		<div class="card-body panel-body panel-pad-body line-items-body">
			<div class="row col-space-sm row-space-sm">
				<div class="column" style="flex: 0 0 30px">
					<div class="uii uii-lg uii-hamburger active-icon"></div>
				</div>
				<div class="column">
					<div class="input-field input-field-group input-block rbullet">
						<div class="input-label input-field-icon">
							<div class="input-icon theme-icon ref-icon module-Assemblies"></div>
						</div>
						<div class="input-label">
							Test Assembly Name
							<div class="itemNumber text-muted" style="font-size: 80%; line-height: 1.25">MFJ-925892i</div>
						</div>
					</div>
				</div>
				<div class="column" style="flex: 0 0 16%">
					<div class="input-field input-block readonly">
						<div class="input-label">
							Assembly
						</div>
					</div>
				</div>
				<div class="column" style="flex: 0 0 4rem">
					<input type="text" class="input-text text-right" size="3" value="1">
				</div>
				<div class="column text-right" style="flex: 0 0 12%"><input type="text" class="input-text text-right" value="40.00" size="12" readonly></div>
				<div class="column text-right" style="flex: 0 0 12%"><input type="text" class="input-text text-right" value="60.00" size="12" readonly></div>
				<div class="column text-right" style="flex: 0 0 12%"><input type="text" class="input-text text-right" value="44.00" size="12" readonly></div>
				<div class="column text-right" style="flex: 0 0 12%"><input type="text" class="input-text text-right" value="44.00" size="12" readonly></div>
				<div class="column text-right" style="flex: 0 0 40px"><div class="uii uii-lg uii-remove active-icon"></div></div>
			</div>
			<div class="row col-space-sm row-space-sm">
				<div class="column" style="flex: 0 0 30px"></div>
				<div class="column" style="flex: 0 0 30px">
					<div class="uii uii-lg uii-hamburger active-icon"></div>
				</div>
				<div class="column">
					<div class="input-field input-field-group input-block rbullet">
						<div class="input-label input-field-icon">
							<div class="input-icon theme-icon ref-icon module-ProductCatalog"></div>
						</div>
						<div class="input-label">
							Test Product Name
							<div class="itemNumber text-muted" style="font-size: 80%; line-height: 1.25">MFJ-925892i</div>
						</div>
					</div>
				</div>
				<div class="column" style="flex: 0 0 16%">
					<div class="input-field input-block rbullet">
						<div class="input-label">
							Markup: +10%
							<div class="itemNumber text-muted" style="font-size: 80%; line-height: 1.25">STD</div>
						</div>
					</div>
				</div>
				<div class="column" style="flex: 0 0 4rem">
					<input type="text" class="input-text text-right" size="3" value="2">
				</div>
				<div class="column text-right" style="flex: 0 0 12%"><input type="text" class="input-text text-right" value="20.00" size="12"></div>
				<div class="column text-right" style="flex: 0 0 12%"><input type="text" class="input-text text-right" value="30.00" size="12"></div>
				<div class="column text-right" style="flex: 0 0 12%"><input type="text" class="input-text text-right" value="22.00" size="12" readonly></div>
				<div class="column text-right" style="flex: 0 0 12%"><input type="text" class="input-text text-right" value="44.00" size="12" readonly></div>
				<div class="column text-right" style="flex: 0 0 40px"><div class="uii uii-lg uii-remove active-icon"></div></div>
			</div>
			<div class="row col-space-sm row-space-sm">
				<div class="column" style="flex: 0 0 30px"></div>
				<div class="column" style="flex: 0 0 30px">
					<div class="uii uii-lg uii-hamburger active-icon"></div>
				</div>
				<div class="column">
					<div class="input-field input-field-group input-block">
						<div class="input-label input-field-icon">
							<div class="input-icon theme-icon ref-icon module-Notes"></div>
						</div>
						<textarea class="input-text" rows="2">Comments</textarea>
					</div>
				</div>
				<div class="column text-right" style="flex: 0 0 48%"></div>
				<div class="column text-right" style="flex: 0 0 40px"><div class="uii uii-lg uii-remove active-icon"></div></div>
			</div>
			<div class="row col-space-sm">
				<div class="column" style="flex: 0 0 37px"></div>
				<div class="column">
					<div class="buttons-inline">
						<div class="inline-elt">
							<span class="uii uii-add active-icon uii-link"><label>add component</label></span>
						</div>
						<div class="inline-elt">
							<span class="uii uii-add active-icon uii-link"><label>add comment</label></span>
						</div>
					</div>
				</div>
			</div>
			<div class="row col-space-sm row-space-sm">
				<div class="column" style="flex: 0 0 30px">
					<div class="uii uii-lg uii-hamburger active-icon"></div>
				</div>
				<div class="column">
					<div class="input-field input-field-group input-block rbullet">
						<div class="input-label input-field-icon">
							<div class="input-icon theme-icon ref-icon module-ProductCatalog"></div>
						</div>
						<div class="input-label">
							Test Product Name
							<div class="itemNumber text-muted" style="font-size: 80%; line-height: 1.25">MFJ-925892i</div>
						</div>
					</div>
				</div>
				<div class="column" style="flex: 0 0 16%">
					<div class="input-field input-block rbullet">
						<div class="input-label text-muted">
							Custom
							<div class="itemNumber text-muted" style="font-size: 80%; line-height: 1.25">STD</div>
						</div>
					</div>
				</div>
				<div class="column" style="flex: 0 0 4rem">
					<input type="text" class="input-text text-right" size="3" value="1">
				</div>
				<div class="column text-right" style="flex: 0 0 12%"><input type="text" class="input-text text-right" value="20.00" size="12"></div>
				<div class="column text-right" style="flex: 0 0 12%"><input type="text" class="input-text text-right" value="30.00" size="12"></div>
				<div class="column text-right" style="flex: 0 0 12%"><input type="text" class="input-text text-right" value="54.00" size="12"></div>
				<div class="column text-right" style="flex: 0 0 12%"><input type="text" class="input-text text-right" value="54.00" size="12" readonly></div>
				<div class="column text-right" style="flex: 0 0 40px"><div class="uii uii-lg uii-remove active-icon"></div></div>
			</div>
		</div>
		<div class="card-footer panel-footer line-items-footer">
			<div class="row">
				<div class="column col-8">
					<div class="buttons-inline">
						<div class="inline-elt">
							<div class="input-button rbullet rbullet-ddown"><span class="uii uii-add"></span> Add Product/Assembly</div>
						</div>
						<div class="inline-elt">
							<div class="input-button"><span class="uii uii-add"></span> Add Comment</div>
						</div>
					</div>
					<div class="buttons-inline">
						<div class="inline-elt">
							<span class="uii uii-md uii-remove active-icon"><label>remove group</label></span>
						</div>
					</div>
				</div>
				<div class="column align-right">
					<div class="row col-space-sm">
						<div class="column">
							<div class="form-label input-field-pad">Subtotal</div>
						</div>
						<div class="column text-right">
							<div class="form-value"><input type="text" class="input-text text-right" value="98.00" size="12" readonly></div>
						</div>
					</div>
					<div class="row col-space-sm">
						<div class="column">
							<span class="uii uii-add active-icon uii-link"><label>add discount</label></span>
						</div>
					</div>
					<div class="row col-space-sm">
						<div class="column">
							<span class="uii uii-add active-icon uii-link"><label>add tax</label></span>
						</div>
					</div>
					<div class="row col-space-sm">
						<div class="column">
							<div class="form-label input-field-pad" style="display: inline-block">Shipping</div>
							<div class="input-field rbullet"><span class="input-label text-placeholder">&mdash;</span></div>
						</div>
						<div class="column text-right">
							<div class="form-value"><input type="text" class="input-text text-right" value="0.00" size="12"></div>
						</div>
					</div>
					<div class="row col-space-sm">
						<div class="column">
							<div class="form-label input-field-pad">Total</div>
						</div>
						<div class="column text-right">
							<div class="form-value"><input type="text" class="input-text text-right" value="98.00" size="12" readonly></div>
						</div>
					</div>
				</div>
				<div class="column" style="flex: 0 0 40px"></div>
			</div>
		</div>
	</section>
</div>
</div>
</div><!-- /block -->

</section>


<section class="container demo-block" style="border: 1px solid red">
	<div class="row demo-row" style="border: 1px solid red">
		<div class="column lg-4 lg-offset-2" style="border: 1px solid blue">
			Test
		</div>
		<div class="column" style="border: 1px solid blue">
			Test
		</div>
	</div>
</section>


<section class="container">
	<div class="card-outer form-outer default-form hl-form-cells">
		<header class="card-header form-header">
			<div class="row form-title">
				<h3 class="column">Form test</h3>
			</div>
		</header>
		<header class="card-header form-buttons">
			<div class="row buttons-row">
				<div class="column buttons-inline align-left">
					<div class="inline-elt">
						<button id="DetailForm_save" name="DetailForm_save" type="button" class="input-button group-left first">
							<div class="input-icon left icon-accept"></div>
							<span class="input-label" id="DetailForm_save-label">Save</span>
						</button>
					</div>
					<div class="inline-elt">
						<button id="DetailForm_cancel" name="DetailForm_cancel" type="button" class="input-button group-left">
							<div class="input-icon left icon-cancel"></div>
							<span class="input-label" id="DetailForm_cancel-label">Cancel</span>
						</button>
					</div>
				</div>
				<div class="column buttons-inline align-center">
				</div>
				<div class="column buttons-inline align-right">
				</div>
			</div>
		</header>
		<div class="card-body form-body">
			<section class="form-section">
				<div class="row form-row col-unstack">
					<div class="column form-cell col-6">
						<div class="topLabel">Single</div>
					</div>
					<div class="column form-cell">
						<div class="topLabel">Single</div>
					</div>
				</div>
			</section>
			<section class="form-section">
				<header class="row form-row form-title col-unstack">
					<h4 class="column form-cell">
						Sub-section title
					</h4>
				</header>
				<div class="row form-row col-unstack">
					<div class="column form-cell col-6">
						<div class="topLabel">Rowspan</div>
					</div>
					<div class="column form-section">
						<div class="row form-row">
							<div class="column form-cell">
								<div class="topLabel">Single</div>
							</div>
						</div>
						<div class="row form-row">
							<div class="column form-cell">
								<div class="topLabel">Single</div>
							</div>
						</div>
					</div>
				</div>
			</section>
			<section class="form-section">
				<header class="row form-row form-help-row col-unstack">
					<div class="column form-cell">
						<div class="form-entry form-help">
							<div class="form-value text-help">
								Section intro text here
							</div>
						</div>
					</div>
				</header>
				<div class="row form-row col-unstack">
					<div class="column form-section">
						<div class="row form-row">
							<div class="column form-cell">
								<div class="topLabel">Single</div>
							</div>
						</div>
						<div class="row form-row">
							<div class="column form-cell">
								<div class="topLabel">Single</div>
							</div>
						</div>
					</div>
					<div class="column form-cell col-6">
						<div class="topLabel">Rowspan</div>
					</div>
				</div>
			</section>
		</div>
		<footer class="card-footer form-buttons">
			<div class="row buttons-row">
				<div class="column buttons-inline align-left">
					<div class="inline-elt">
						<button id="DetailForm_save" name="DetailForm_save" type="button" class="input-button group-left first">
							<div class="input-icon left icon-accept"></div>
							<span class="input-label" id="DetailForm_save-label">Save</span>
						</button>
					</div>
					<div class="inline-elt">
						<button id="DetailForm_cancel" name="DetailForm_cancel" type="button" class="input-button group-left">
							<div class="input-icon left icon-cancel"></div>
							<span class="input-label" id="DetailForm_cancel-label">Cancel</span>
						</button>
					</div>
				</div>
				<div class="column buttons-inline align-center">
				</div>
				<div class="column buttons-inline align-right">
				</div>
			</div>
		</footer>
	</div>
</section>


<!--
<section class="container">
	<?php
		$title = 'Toggle test';
		$elts = [];
		$elts[] = ['value' => renderToggle('togtest', 'togtest_sm', 'input-sm', 'Small')];
		$elts[] = ['value' => renderToggle('togtest', 'togtest', '', 'Normal', 'checkbox', 'checked=1')];
		$elts[] = ['colspan' => 2, 'value' => renderToggle('togtest', 'togtest_lg', 'input-lg', 'Large')];
		$elts[] = ['value' => renderToggle('togtest', 'togtest_dis', '', 'Disabled', 'checkbox', 'disabled=1')];
		echo renderForm($title, $elts, $columns=2);
	?>
</section>
-->

</body>
</html>

