<?php

require_once 'include/database/RowUpdate.php';
require_once __DIR__ . '/mautic_api/autoload.php';  
require_once __DIR__ . '/RequestHack.php';  

use Mautic\Auth\ApiAuth;
use Mautic\MauticApi;

class LeadGuerrillaUpdateHandler {

	private static $fhandle;
	private static $LOCK_FILE = 'cache/.lead_guerrilla.lock';

	private static $map = [
		'title',
		'firstname' => 'first_name',
		'lastname' => 'last_name',
		'company' => ['account', 'account_name', 'primary_account'],
		'email' => ['email1', 'email2'],
		'phone' => 'phone_work',
		'mobile' => 'phone_mobile',
		'fax' => 'phone_fax',
		'address1' => 'primary_address_street',
		'city' => 'primary_address_city',
		'state' => 'primary_address_state',
		'zipcode' => 'primary_address_postalcode',
		'country' => 'primary_address_country',
		'website',
	];
	
	public static function get_contacts_options() {
		return [
			"" => translate('LBL_LEAD_GUERRILLA_SYNC_LEADS', 'LeadGuerrilla'),
			"contacts" => translate('LBL_LEAD_GUERRILLA_SYNC_CONTACTS', 'LeadGuerrilla'),
			"both" => translate('LBL_LEAD_GUERRILLA_SYNC_BOTH', 'LeadGuerrilla'),
		];
	}

	public static function afterSave(RowUpdate $upd) {
		self::handle_record($upd, true);
	}
	
	public static function handle_record(RowUpdate $upd, $modified=false) {
		$emptyValue = new StdClass;

		if (! AppConfig::setting('lead_guerrilla.enabled')) {
			return;
		}
		
		if (!defined('LEAD_GUERRILLA_INITIAL_PUSH_DONE')) {
			if (!defined('LEAD_GUERRILLA_INITIAL_PUSH') && AppConfig::setting('lead_guerrilla.initial_push') && !AppConfig::setting('lead_guerrilla.initial_push_finished')) {
				return;
			}
		}

		$allFields = defined('LEAD_GUERRILLA_INITIAL_PUSH') && !defined('LEAD_GUERRILLA_INITIAL_PUSH_DONE');

		$modelName =  $upd->getModelName();
		$sync_objects = AppConfig::setting('lead_guerrilla.contacts');
		if ($sync_objects === 'both' && $modelName === 'Lead' && $upd->getField('contact_id')) {
			return;
		} else if ($sync_objects == 'contacts' && $modelName != 'Contact') {
				return;
		}

		$app = $upd->getRelatedData('X_ONECRM_APPLICATION');
		if ($app === 'LeadGuerrilla') {
			return;
		}

		// RowUpdate may do not have mautic_id field if it was not fetched with original record
		// Here I check if the field was fetched, and if not, fetch it from DB directly
		if ($upd->getFieldRetrieved('mautic_id')) {
			$mautic_id = $upd->getFieldValue('mautic_id');
		} else {
			if ($upd->new_record) {
				$mautic_id = null;
			} else {
				$existing = ListQuery::quick_fetch_row($modelName, $upd->getField('id'), ['mautic_id']);
				if ($existing) {
					$mautic_id = array_get_default($existing, 'mautic_id');
				} else {
					$mautic_id = null;
				}
			}
		}

		$allFields = $allFields || !$mautic_id;
		$contactApi = self::authenticate();
		if (!$contactApi) {
			return;
		}
		try {
			$data = [];
			foreach (self::$map as $k => $v) {
				if (is_numeric($k))
					$k = $v;
				$v = (array)$v;
				foreach ($v as $vv) {
					if ($allFields || $upd->getFieldUpdated($vv)) {
						$value = $upd->getField($vv, $emptyValue);
						if ($value !== $emptyValue) {
							$data[$k] = $value;
							break;
						}
					}
				}
			}
			if ($allFields || $upd->getFieldUpdated('primary_address_street')) {
				$data['address2'] = '';
				$parts = explode("\n", $upd->getField('primary_address_street'), 2);
				$data['address1'] = $parts[0];
				if (count($parts) > 1) {
					$data['address2'] = $parts[1];
				}
			}

			foreach (self::get_tag_fields($upd->getModel()) as $k) {
				$v = substr($k, 4);
				if (strlen($v)) {
					if ($allFields || $upd->getFieldUpdated($k)) {
						$v = str_replace('_', ' ', $v);
						$value = $upd->getField($k);
						if ($value)
							$data['tags'][] = $v;
						else
							$data['tags'][] = '-' . $v;
					}
				}
			}
			$fields = self::getContactFields($contactApi);
			$vfields = [];
			foreach ($fields as $f) {
				if (strpos($f, '_onecrm_') === 0)
					$cf = substr($f, 8);
				elseif ($f == 'lead_source')
					$cf = $f;
				else
					$cf = 'lg_' . $f;
				$fdef = $upd->getFieldDefinition($cf);
				if($fdef && ($fsrc = array_get_default($fdef, 'source')) && array_get_default($fsrc, 'type', 'db') != 'db') {
					$vfields[$f] = $cf;
					$virtual = true;
				} else
					$virtual = false;
				if ($allFields || $upd->getFieldUpdated($cf) || $virtual) {
					$value = $upd->getField($cf, $emptyValue);
					if ($value !== $emptyValue)
						$data[$f] = $value;
				}
			}
			if($vfields) {
				$refetch = ListQuery::quick_fetch_row($upd->model, $upd->getField('id'), $vfields);
				if($refetch) {
					array_extend($data, array_intersect_key($refetch, $vfields));
				}
			}
                               
			$updateDNC = $allFields || $upd->getFieldUpdated('email_opt_out');
			if (empty($data) && !$updateDNC) {
				return;
			}
			if (!empty($data)) {
				$data['crmid'] = $upd->getField('id');
				$data['crmmodel'] = $modelName;
				$data['__from_onecrm'] = '1';
				if ($mautic_id) {
					$response = $contactApi->edit($mautic_id, $data);
				} else {
					$response = $contactApi->create($data);
				}
			} else {
				$response = array();
			}
			if (!isset($response['error'])) {
				if (!$mautic_id) {
					$mautic_id = $response['contact']['id'];
					$upd2 = clone($upd);
					$upd2->prohibit_workflow = true;
					$upd2->new_record = false;
					$upd2->set('mautic_id', $mautic_id);
					$upd->set('mautic_id', $mautic_id); // just in case
					$upd2->save();
				}
				if ($allFields || $upd->getFieldUpdated('email_opt_out')) {
					if ($upd->getField('email_opt_out')) {
						$contactApi->addDnc($mautic_id, 'email');
					} else {
						$contactApi->removeDnc($mautic_id, 'email');
					}
				}
			} else {
				$GLOBALS['log']->fatal($response['error']);
			}
		} catch (Exception $e) {
			$GLOBALS['log']->fatal($e->getMessage());
		}
	}

	private static function authenticate() {
		if (!AppConfig::setting('lead_guerrilla.enabled'))
			return null;
	    $baseUrl = AppConfig::setting('lead_guerrilla.url');
	    $clientKey = AppConfig::setting('lead_guerrilla.client_id');
		$clientSecret = AppConfig::setting('lead_guerrilla.client_secret');
		self::lockConfig();
		$token = self::getAccessToken();
		if (!$baseUrl || !$clientKey || !$clientSecret || !$token) {
			self::unlockConfig();
			return null;
		}
		$settings = array(
		    'baseUrl'          => $baseUrl,
		    'version'          => 'OAuth2',
		    'clientKey'        => $clientKey,
			'clientSecret'     => $clientSecret,
			'callback' =>  AppConfig::site_url() . 'index.php?module=LeadGuerrilla',
			'accessToken' => $token['access_token'],
			'accessTokenExpires' => $token['expires'],
			'refreshToken' => $token['refresh_token'],
		);
		$initAuth = new ApiAuth();
		$auth = $initAuth->newAuth($settings);
		try {
			if ($auth->validateAccessToken()) {
				if ($auth->accessTokenUpdated()) {
					$accessTokenData = $auth->getAccessTokenData();
					self::setAccessToken($accessTokenData);
				}
				self::unlockConfig();
				$api = new MauticApi();
				$contactApi = $api->newApi('ContactsHack', $auth, $baseUrl . '/api');
				return $contactApi;
			}
			self::unlockConfig();
		} catch (Exception $e) {
			self::unlockConfig();
			$GLOBALS['log']->fatal($e->getMessage());
		}
	}
	
	public static function get_hook_required_fields($model) {
		if(is_string($model))
			$model = new ModelDef($model);
		$fields = self::get_tag_fields($model);
		$fields[] = 'contact_id';
		$fields[] = 'mautic_id';
		return $fields;
	}

	public static function get_tag_fields($model) {
		$ret = [];
		$fields = $model->getFieldDefinitions();
		foreach ($fields as $f) {
			$k = $f['name'];
			if (strpos($k, 'tag_') === 0) {
				$ret[] = $k;
			}
		}
		return $ret;
	}

	private static function getContactFields($contactApi) {
		$key = 'leadguerrilla_lead_fields__';
		$found = false;
		$fields = AppConfig::cache_get_timed($key, time() - 3600, $found);
		if (!$found) {
			$fields = [];
			$response = $contactApi->getFieldList();
			foreach ($response as $f) {
				$fields[] = $f['alias'];
			}
			AppConfig::cache_put_timed($key, $fields);
		}
		return $fields;
	}

	private static function getAccessToken() {
		global $db;
		$query = "SELECT value FROM config WHERE category = 'lead_guerrilla' AND name = 'token'";
		$result = $db->query($query);
		$row = $db->fetchByAssoc($result, -1, false);
		if (!$row) {
			return AppConfig::setting("lead_guerrilla.token");
		}
		$token = @unserialize($row['value']);
		return $token;
	}

	private static function setAccessToken($token) {
		global $db;
		$query = sprintf(
			"INSERT INTO config (category, name, value) VALUES ('lead_guerrilla', 'token', '%s') "
			. " ON DUPLICATE KEY UPDATE value=VALUES(value)", $db->quote(serialize($token))
		);
		$db->query($query);
	}

	private static function lockConfig() {
		self::$fhandle = fopen(self::$LOCK_FILE, 'w');
		flock(self::$fhandle, LOCK_EX);
	}

	private static function unlockConfig() {
		if (self::$fhandle) {
			flock(self::$fhandle, LOCK_UN);
			fclose(self::$fhandle);
			self::$fhandle = null;
		}
	}
}
