<?php

require_once dirname(__FILE__) . '/APIClient.php';
require_once dirname(__FILE__) . '/WebhookHandler.php';
require_once dirname(__FILE__) . '/TranscriptRenderer.php';

class LiveChatIncProvider extends ChatProvider {
	public static function instance() {
		return new self();
	}

	public function logo() {
		return 'modules/LiveChatInc/images/logo.png';
	}
	
	public function getTabs() {
		return [
			[
				'id' => 'info',
				'icon' => 'include/chat/images/icon-info.png',
			],
			[
				'id' => 'queue',
				'icon' => 'include/chat/images/icon-queue.png',
			],
		];
	}
	public function jsImplName() {
		return 'LiveChatIncImpl';
	}

	public function finalize_page($page) {
		$page->add_js_include('modules/LiveChatInc/livechat.js', null, LOAD_PRIORITY_HEAD);
		$page->add_css_include('modules/LiveChatInc/css/livechat.css', null, LOAD_PRIORITY_THEME);
		parent::finalize_page($page);
	}
	
	public function isEnabled() {
		return AppConfig::setting('chat.integration') == 'livechat';
	}

	public function isGuiEnabled() {
		// GUI disabled until LiveChat resolves their API issues :(
		return false;
		if (!$this->isEnabled())
			return false;
		$username = AppConfig::setting('userpref.current_user.global.livechatinc_username');
		$api_key =  AppConfig::setting('userpref.current_user.global.livechatinc_api_key');
		return !!($username && $api_key);
	}

	public function registerEndpoints($server) {
		$server->registerEndpoint('/private/livechatinc/apirequest/*', 'POST', [$this, 'apiRequest']);
		$server->registerEndpoint('/private/livecahtinc/webhook/' . AppConfig::unique_key(), 'POST', function($server, $params, $post) {
			$handler = new LiveChatIncWebhookHandler($server, $params, $post);
			return $handler->handleRequest();
		}, false);
	}

	public function apiRequest($server, $params, $post) {
		$username = AppConfig::setting('userpref.current_user.global.livechatinc_username');
		$api_key =  AppConfig::setting('userpref.current_user.global.livechatinc_api_key');
		$client = new LiveChatIncAPIClient($username, $api_key);
		$url = '/' . $params['_'] . '?' . http_build_query(array_get_default($post, 'query', []));
		$method = array_get_default($post, 'method', 'GET');
		$postdata = array_get_default($post, 'postdata');
		$reply = $client->send($method, $url, $postdata);
		header('Content-Type: application/json');
		$server->reply($reply, 200, false);

	}
	
	public function extendOptions(&$options) {
		$options['username'] = AppConfig::setting('userpref.current_user.global.livechatinc_username');
	}
	
	public function renderTranscriptForSummary($text) {
		$renderer = new TranscriptRenderer;
		return $renderer->renderForSummary($text);
	}
	
	public function renderTranscript($text) {
		$renderer = new TranscriptRenderer;
		return $renderer->render($text);
	}
	
	public function getLocalizedName() {
		return translate('LBL_CHAT_PROVIDER_NAME', 'LiveChatInc');
	}

}

