<?php

class TranscriptRenderer {

	function renderVisitorInfo($data) {
		if (!isset($data['visitor']))
			return '';

		$fields = ['name', 'email', 'country', 'city', 'timezone', 'page_current'];

		$body = '<h3>' . translate('LBL_VISITOR_INFO', 'LiveChatInc') . '</h3>';
		$new_row = true;
		foreach ($fields as $f) {
			if (isset($data['visitor'][$f])) {
				if ($new_row) {
					$body .= '<div class="row form-row row-even col-unstack">';
				}
				$value = $data['visitor'][$f];
				if ($f == 'page_current') {
					$value = sprintf('<a href="%s" target="_blank">%s</a>', $value, $value);
				} else {
					$value = to_html($value);
				}
				$body .= sprintf(
					'<div class="column sm-6 form-cell span-1"><div class="form-entry"><p class="form-label">%s</p><div class="form-value">%s</div></div></div>',
					translate('LBL_VISITOR_' . strtoupper($f), 'LiveChatInc'),
					$value
				);
				$new_row = !$new_row;
				if ($new_row) {
					$body .= '</div>';
				}
			}
		}
		if (!$new_row) {
			$body .= '</div>';
		}
		return $body;
	}

	function inferAuthorName($data, $type, $login = null) {
		if ($type == 'visitor') {
			return $data['visitor']['name'];
		}
		foreach ($data['chat']['agents'] as $a) {
			if ($a['login'] == $login)
				return $a['name'];
		}
	}

	function preformatTranscript($data, $raw = false) {
		global $timedate;
		$ret = [];
		$messages = [];
		if (!empty($data['chat']['messages']))
			$messages = $data['chat']['messages'];
		if (!empty($data['chat']['attachments'])) {
			foreach ($data['chat']['attachments'] as $att) {
				if ($att['status'] == 'OK') {
					$url = $att['files'][0]['url'];
					$path = parse_url($url, PHP_URL_PATH);
					$filename = basename($path);
					$formatted_text = '<a target="_blank" href="' . $url . '">' . to_html($filename) . '</a>';
					if (!empty($att['files'][0]['thumbs']['url'])) {
						$formatted_text .= '<br>';
						$formatted_text .= '<img src="' . $att['files'][0]['thumbs']['url'] . '" style="max-width:90%">';
					}
					$messages[] = [
						'user_type' => $att['author']['type'],
						'author_name' => $this->inferAuthorName($data, $att['author']['type'], array_get_default($att['author'], 'login')),
						'timestamp' => $att['timestamp'],
						'formatted_text' => $formatted_text,
					];
				}
			}
		}
		if (!empty($data['chat']['events'])) {
			foreach($data['chat']['events'] as $ev) {
				$messages[] = [
					'timestamp' => $ev['timestamp'],
					'author_name' => $this->inferAuthorName($data, $ev['user_type'], array_get_default($ev, 'user_id')),
					'user_type' => $ev['user_type'],
					'formatted_text' => '<i>' . to_html($ev['text']) . '</i>',
					'text' => $ev['text'],
				];
			}
		}

		if (empty($messages))
			return $ret;
		usort($messages, function($a, $b) {
			return $a['timestamp'] - $b['timestamp'];
		});
		if ($raw)
			return $messages;
		$block = null;
		$type = null;
		foreach ($messages as $msg) {
			if (!$block || $msg['user_type'] != $type) {
				if ($block) {
					$ret[] = $block;
				}
				$block = [
					'user_type' => $msg['user_type'],
					'author_name' => to_html($msg['author_name']),
					'parts' => [],
				];
				$type = $msg['user_type'];
			}
			$time = $timedate->to_display_time(gmdate("Y-m-d H:i:s", $msg['timestamp']));
			$block['parts'][$time][] = array_get_default($msg, 'formatted_text', to_html(array_get_default($msg, 'text')));
		}
		if ($block) {
			$ret[] = $block;
		}
		return $ret;
	}

	function renderTranscript($data) {
		global $timedate;
		$preformatted = $this->preformatTranscript($data);
		$started_date = $timedate->to_display_date_time(gmdate("Y-m-d H:i:s", $data['chat']['started_timestamp']));
		$body = '<h3>' . translate('LBL_TRANSCRIPT', 'LiveChatInc') . '</h3>';
		$body .= "<div>" . translate('LBL_CHAT_STARTED', 'LiveChatInc') . " $started_date</div>";
		$body .= '<div style="width: 50%; margin-top: 20px; min-width: 400px">';
		foreach ($preformatted as $block) {
			$body .= '<div class="chat-message ' . $block['user_type'] . '-message">';
			$body .= '<div class="chat-message-author">' . $block['author_name'] . '</div>';
			foreach ($block['parts'] as $time => $parts) {
				$body .= '<div class="chat-message-time">' . $time . '</div>';
				foreach ($parts as $txt) {
					$body .= "<p>$txt</p>";
				}
			}
			$body .= '</div>';
		}
		$body .= '</div>';
		return $body;
	}

	function render($text) {
		$json = getJSONObj();
		$data = $json->decode($text);
		$body = $this->renderVisitorInfo($data);
		$body .= $this->renderTranscript($data);
		return $body;
	}
	
	function renderForSummary($text) {
		$json = getJSONObj();
		$data = $json->decode($text);
		$messages = $this->preformatTranscript($data, true);
		$ret = '';
		foreach ($messages as $msg) {
			if (isset($msg['text']))
				$ret .= $msg['text'] . "\r\n";
		}
		return $ret;
	}
}


