<?php

require_once 'include/database/ListQuery.php';
require_once 'include/database/RowUpdate.php';
require_once dirname(__FILE__) . '/APIClient.php';

class LiveChatIncWebhookHandler {

	var $display_fields = ['account_name', 'primary_account', 'title', 'department', 'email1', 'email2', 'phone_home', 'phone_mobile', 'phone_work', 'phone_other', 'phone_fax', 'website'];
	var $required_fields = ['first_name', 'last_name'];
	var $links = [
		'website' => true,
	   'primary_account' => ['index.php?module=Accounts&action=DetailView&record=%s', 'primary_account_id']
	];
	var $server;
	var $params;
	var $post;
	var $user_id;
	var $lead;
	var $query_fields;

	function __construct($server, $params, $post) {
		$this->server = $server;
		$this->params = $params;
		$this->post = $post;
		$this->user_id = '1';
		$this->lead = null;
		$this->query_fields = array_merge($this->display_fields, $this->required_fields);
	}

	function findUserId() {
		global $current_user;
		$current_user = new User;
		$agent_email = $this->post['chat']['agents'][0]['login'];
		$lq = new ListQuery('User');
		$users = $lq->fetchAll();
		foreach ($users->rows as $row) {
			$login = AppConfig::user_preference('livechatinc_username', 'global', $row['id']);
			if ($login == $agent_email) {
				$this->user_id = $row['id'];
				break;
			}
		}
		$current_user->retrieve($this->user_id);
	}

	function matchVisitor() {
		switch (AppConfig::setting('chat.livechatinc.match_mode')) {
		case 'Both':
			$models = ['Contact', 'Lead'];
			break;
		case 'Contact':
			$models = ['Contact'];
			break;
		default:
			$models = ['Lead'];
			break;
		}
		$visitor = $this->post['visitor'];
		foreach ($models as $model) {
			$clause = [
				'operator' => 'OR',
				'multiple' => [
					['field' => 'email1', 'value' => $visitor['email']],
					['field' => 'email2', 'value' => $visitor['email']],
				]
			];
			$lq = new ListQuery($model, $this->query_fields);
			$lq->addFilterClause($clause);
			$result = $lq->runQuerySingle();
			if ($result && !$result->failed) {
				$this->lead = $result;
				return;
			}
		}

		$model = AppConfig::setting('chat.livechatinc.create_mode') == 'Contacts' ? 'Contact' : 'Lead';
		$upd = RowUpdate::blank_for_model($model);
		list($first_name, $last_name) = preg_split('~\s+~', $visitor['name'], 2);
		if (!strlen($last_name)) {
			$last_name = $first_name;
			$first_name = '';
		}
		$upd->set([
			'first_name' => $first_name,
			'last_name' => $last_name,
			'email1' => $visitor['email'],
			'assigned_user_id' => AppConfig::current_user_id(),
		]);
		$upd->save();
		$this->lead = ListQuery::quick_fetch($model, $upd->getField('id'), $this->query_fields);
	}

	function findOrCreateChat() {
		$lq = new ListQuery('Chat');
		$lq->addSimpleFilter('provider_name', 'livechat');
		$lq->addSimpleFilter('external_id', $this->post['chat']['id']);
		$chat = $lq->runQuerySingle();
		if ($chat && !$chat->failed) {
			return RowUpdate::for_result($chat);
		}
		$chatName = $this->post['visitor']['name'];
		if (!empty($this->post['visitor']['country'])) {
			$chatName .= ' : ' . $this->post['visitor']['country'];
		}
		$upd = RowUpdate::blank_for_model('Chat');
		$upd->set([
			'external_id' => $this->post['chat']['id'],
			'provider_name' => 'livechat',
			'name' => $chatName,
			'date_started' => gmdate('Y-m-d H:i:s', $this->post['chat']['started_timestamp']),
			'assigned_user_id' => AppConfig::current_user_id(),
		]);
		return $upd;
	}

	function updateTranscript() {
		$data = $this->post;
		if (
			!isset($data['visitor']['name'])
			|| !isset($data['visitor']['email'])
			|| !isset($data['chat'])
		) return;
		$this->findUserId();
		$this->matchVisitor();
		if (!$this->lead) {
			return;
		}
		$chat = $this->findOrCreateChat();
		if (!$chat)
			return;
		$json = getJSONObj();
		$new_data = $json->decode($chat->getField('transcript'));
		if (!is_array($new_data))
			$new_data = [];
		$new_data['chat'] = $data['chat'];
		$new_data['visitor'] = $data['visitor'];
		if (isset($data['pre_chat_survey']))
			$new_data['pre_chat_survey'] = $data['pre_chat_survey'];
		$chat->set('transcript', $json->encode($new_data));
		$chat->set('parent_id', $this->lead->getField('id'));
		$chat->set('parent_type', $this->lead->getModuleDir());
		if ($data['event_type'] == 'chat_ended')
			$chat->set('status', 'Finished');
		$chat->save();
		$lead_upd = RowUpdate::for_result($this->lead);
		$lead_upd->set([
			'chat_activity' => 1,
			'livechat_activity' => 1,
		]);
		$lead_upd->save();
	}

	function updateVisitorDetails() {
		if (!$this->lead)
			return;
		$username = AppConfig::setting('userpref.current_user.global.livechatinc_username');
		$api_key =  AppConfig::setting('userpref.current_user.global.livechatinc_api_key');
		$client = new LiveChatIncAPIClient($username, $api_key);
		$module = $this->lead->getModuleDir();
		$logo_url = preg_replace('~^http(s)://~', '', AppConfig::site_url() . 'modules/LiveChatInc/images/1crm_logo.png');
		$url = AppConfig::site_url() 
			. 'index.php?module=' . $this->lead->getModuleDir()
			. '&record=' . $this->lead->getField('id')
			. '&action=DetailView'
			;
		$postdata = [
			'license_id' => $this->post['license_id'],
			'token' => $this->post['token'],
			'id' => 'onecrm',
			'icon' => $logo_url,
			'fields' => [
				[
					'name' => translate('LBL_MODEL_' . strtoupper($this->lead->getModuleDir()), 'LiveChatInc'),
					'value' => $this->lead->getField('first_name') . ' ' . $this->lead->getField('last_name'),
					'url' => $url,
				]
			],
		];
		foreach ($this->display_fields as $f) {
			$value = $this->lead->getField($f);
			if (strlen($value)) {
				$def = $this->lead->getFieldDefinition($f);
				$label = translate($def['vname'], array_get_default($def, 'vname_module', $module));
				$link = null;
				$link_info = array_get_default($this->links, $f);
				if ($link_info) {
					if (is_array($link_info)) {
						$link = AppConfig::site_url() . sprintf($link_info[0], $this->lead->getField($link_info[1]));
					} else {
						$link = $value;
					}
				}
				$postdata['fields'][] = [
					'name' => $label,
					'value' => $value,
					'url' => $link,
				];
			}
		}
		$endpoint = '/visitors/' . $this->post['visitor']['id'] . '/details';
		$client->send('POST', $endpoint, $postdata);

	}

	function handleRequest() {
		switch ($this->post['event_type']) {
			case 'chat_ended':
				$this->updateTranscript();
				break;
			case 'chat_started':
				$this->updateTranscript();
				$this->updateVisitorDetails();
				break;
		}
		$this->server->reply('OK', 200, false);
	}

}

