<?php
class ActivityInvite {

    /**
     * @var array
     */
    private $ids_list;

    /**
     * @var null|string
     */
    private $main_model;

    /**
     *
     * @param string $main_model_name
     * @param array $ids_list - list of invitee participants IDs:
     *['user_invitees'], ['contact_invitees'], ['lead_invitees'], ['resource_invitees']
     */
    public function __construct($main_model_name = null, $ids_list = null) {
        $this->main_model = $main_model_name;
    	if(! is_array($ids_list)) $ids_list = array();
        $this->ids_list = $ids_list;
    }

    /**
     * Get invitees list
     *
     * @param RowResult $row_result
     * @return array: ['id'], ['_display'], ['module']
     */
    public function getList(RowResult $row_result) {
        $invitees = array();
        $to_load = array();

        if (! empty($this->ids_list['user_invitees']) || ! empty($this->ids_list['contact_invitees']) || ! empty($this->ids_list['lead_invitees']) || ! empty($this->ids_list['resource_invitees'])) {
            $this->fillInviteesByIds($invitees);
        } elseif (! $row_result->new_record){
            $this->fillInvitees($row_result->getField('id'), $invitees);
        } else {
        	if(! empty($_REQUEST['contact_ids'])) {
				$to_load['Contacts'] = array_map('trim', explode(',', $_REQUEST['contact_ids']));
			} else if ($row_result->getField('parent_type') == 'Contacts' && $row_result->getField('parent_id')) {
				$to_load['Contacts'] = array($row_result->getField('parent_id'));
			} else if ($row_result->getField('parent_type') == 'Leads') {

				//If lead hac contact so add this contact to invitees list and add this Lead otherwise
        		$leadContact = $this->getLeadContactId($row_result->getField('parent_id'));

				if ($leadContact) {
					$to_load['Contacts'] = array($leadContact);
				} else {
					$to_load['Leads'] = array($row_result->getField('parent_id'));
				}
			}

			if(! empty($_REQUEST['user_ids'])) {
				$to_load['Users'] = array_map('trim', explode(',', $_REQUEST['user_ids']));
			}
			else if ($row_result->getField('parent_type') == 'Users' && $row_result->getField('parent_id')) {
				$to_load['Users'] = array($row_result->getField('parent_id'));
			}

			if(! empty($_REQUEST['resource_ids'])) {
				$to_load['Resources'] = array_map('trim', explode(',', $_REQUEST['resource_ids']));
			}
		}

        if ($to_load) {
        	foreach($to_load as $mod => $items) {
        		foreach($items as $item) {
        			if(! $item) continue;
					$targ = $this->loadParticipant($mod, $item);
					if ($targ) $invitees[] = $targ;
				}
			}
        }

        return $invitees;
    }

    public static function update_links(RowUpdate $upd, $links) {
        $inv = new ActivityInvite($upd->model_name);
        return $inv->updateLinks($upd, $links);
    }

	public function updateLinks(RowUpdate $upd, $ids_list=null) {
		if(! $upd->new_record) {
			if(! is_array($ids_list) && $upd->getFieldUpdated('assigned_user_id')) {
				// most likely mass-update, change invitees accordingly
				$reassigned = true;
				$prev_assigned = $upd->getField('assigned_user_id', null, true);
				$upd->removeLink('users', $prev_assigned);
				// new assigned user will be added later
			}
			if($upd->getFieldUpdated('date_start') || $upd->getFieldUpdated('duration')) {
				$clear_vcals = true;
			}
		}

		$linked = false;
		if($upd->new_record || is_array($ids_list) || ! empty($reassigned) || ! empty($clear_vcals)) {
			$this->ids_list = $ids_list;
        	$this->addStandardParticipants($upd);

        	$participants = array('user' => 'users', 'contact' => 'contacts', 'lead' => 'leads', 'resource' => 'resources');
        	foreach($participants as $name => $link) {
        		if(! $upd->getModel()->haveLinkDefinition($link))
        			continue;
				$old_ids = $upd->getLinkIds($link);
				$new_ids = $this->formatInvites($name, true);
				if(! isset($new_ids))
					$new_ids = [];
				else
					$linked = true;
				$removed = array_diff($old_ids, $new_ids);
				if (!is_array($ids_list)) {
					$removed = array();
				}
				if($removed)
					$upd->removeLinks($link, $removed);

				$added = array_diff($new_ids, $old_ids);
				foreach($added as $id) {
					if($name == 'user' && $id == AppConfig::current_user_id())
						$id = array('id' => $id, 'accept_status' => 'accept');
					$upd->addUpdateLink($link, $id);
				}
				if($name == 'user' && ! empty($clear_vcals)) {
					// clear vcal for users not added or removed
					$same = array_intersect($new_ids, $old_ids);
					if($same) self::update_vcals($same);
				}
			}
		}

		return $linked;
    }


    /**
     * Update users vcal data
     *
     * @static
     * @param array $user_ids
     */
    public static function update_vcals(array $user_ids) {
    	foreach($user_ids as $id) {
    		self::update_vcal($id);
    	}
    }

    /**
     * Update user vcal data
     *
     * @static
     * @param string $user_id
     */
    public static function update_vcal($user_id) {
    	require_once('modules/vCals/vCal.php');
		vCal::clear_sugar_vcal($user_id);
    }

    /**
     * @static
     * @param RowUpdate $upd
     */
    public static function handle_delete(RowUpdate &$upd) {
    	$model = $upd->getModelName();
    	$id = $upd->getField('id');
    	if($model && $id) {
			$ids = $upd->getLinkIds('users');
			if($ids)
				self::update_vcals($ids);
		}
    }

    /**
     * Fill invitee participants
     *
     * @param string $record_id
     * @param array $invitees
     */
    private function fillInvitees($record_id, &$invitees) {
    	if(! $record_id) return;
        $links = array('users', 'contacts', 'leads', 'resources');
        $model_def = new ModelDef($this->main_model);

        for ($i = 0; $i < sizeof($links); $i++) {
	        if(! $model_def->haveLinkDefinition($links[$i])) continue;
            $lq = new ListQuery($model_def, array('id', '_display'), array('link_name' => $links[$i], 'parent_key' => $record_id));

            foreach($lq->fetchAllRows() as $invite) {
                $invite['module'] = ucfirst($links[$i]);
                $invitees[] = $invite;
            }
        }
    }

    /**
     * Fill invite participants from user input
     *
     * @param array $invitees
     */
    private function fillInviteesByIds(&$invitees) {
        $participants = array('user' => 'Users', 'contact' => 'Contacts', 'lead' => 'Leads');
        if($this->main_model == 'Meeting')
            $participants['resource'] = 'Resources';

        foreach ($participants as $name => $module) {
			$ids = $this->formatInvites($name);
			foreach($ids as $id) {
				$invite = $this->loadParticipant($module, $id);
				if ($invite)
					$invitees[]= $invite;
			}
        }
    }

    /**
     * Load participant's data
     *
     * @param string $module
     * @param string $id
     * @return array|null
     */
    private function loadParticipant($module, $id) {
    	if(! $id) return null;
        $participant = ListQuery::quick_fetch_row(AppConfig::module_primary_bean($module), $id, array('id', '_display'));

        if ($participant) {
            $participant['module'] = $module;
            return $participant;
        } else {
            return null;
        }
    }

    /**
     * Get Lead's converted contact id
     *
     * @param string $lead_id
     * @return null|string
     */
    private function getLeadContactId($lead_id) {
        $lead = ListQuery::quick_fetch_row('Lead', $lead_id, array('contact_id'));
        if ($lead && ! empty($lead['contact_id']))
            return $lead['contact_id'];
        return null;
    }

    /**
     * Add standard invitee participants (assigned user, related contact)
     *
     * @param RowUpdate $update
     */
    private function addStandardParticipants(RowUpdate $update) {
        if (! is_array($this->ids_list))
            $this->ids_list = array();
        if(empty($this->ids_list['user_invitees'])) {
            $this->ids_list['user_invitees'] = $update->getField('assigned_user_id');
        }

        $parent_type = $update->getField('parent_type');

        if ($parent_type == 'Contacts') {
            if (! empty($this->ids_list['contact_invitees'])) {
                $this->ids_list['contact_invitees'] .= ','.$update->getField('parent_id');
            } else {
                $this->ids_list['contact_invitees'] = $update->getField('parent_id');
            }
        }
    }

    /**
     * Set accept status for current user
     *
     * @param RowUpdate $upd
     */
    private function acceptInviteForCurrentUser(RowUpdate $upd) {
        $user = ListQuery::quick_fetch('User', AppConfig::current_user_id());
        if ($user) {
            require_bean('Meeting');
            Meeting::set_invite_status($upd, $user, 'accept');
        }
    }

    /**
     * Format invites ids string and explode
     *
     * @static
     * @param string $category
     * @return array|mixed
     */
    private function formatInvites($category, $return_blank=false) {
		$inv = array_get_default($this->ids_list, $category . '_invitees');
		if(! isset($inv)) {
			if($return_blank) return null;
			$inv = array();
		}
		if(is_array($inv)) return $inv;
        else {
        	$formatted_invitees = preg_replace('/(\s|\,$)/', '', $inv);
			$formatted_invitees = explode(',', $formatted_invitees);
		}
        return $formatted_invitees;
    }
}
?>
