<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/
/*********************************************************************************

 * Description:	 TODO: To be written.
 * Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
 * All Rights Reserved.
 * Contributor(s): ______________________________________..
 ********************************************************************************/

require_once('data/SugarBean.php');
require_once('modules/Contacts/Contact.php');
require_once('modules/Users/User.php');
require_once 'include/utils/vcal_utils.php';

// Meeting is used to store customer information.
class Meeting extends SugarBean {
	// Stored fields
	var $id;
	var $date_entered;
	var $date_modified;
	var $assigned_user_id;
	var $modified_user_id;
	var $created_by;
	var $created_by_name;
	var $modified_by_name;


	// longreach - added
	var $is_private;
	var $is_daylong;
	// longreach - added
	var $email_reminder_time;


	var $description;
	var $name;
	var $location;
	var $status;
	var $date_start;
	var $date_end;
	var $parent_type;
	var $parent_id;
	var $field_name_map;
	var $contact_id;
	var $user_id;
	var $reminder_time;
	var $required;
	var $accept_status;
	var $parent_name;
	var $contact_name;
	var $contact_phone;
	var $contact_email;
	var $account_id;
	var $opportunity_id;
	var $case_id;
	var $assigned_user_name;
	var $outlook_id;

	var $recur_copy_relations = array('Users' => 'users', 'Contacts' => 'contacts');


	var $update_vcal = true;
	var $contacts_arr;
	var $users_arr;
		// when assoc w/ a user/contact:
	var $table_name = "meetings";
	var $rel_users_table = "meetings_users";
	var $rel_contacts_table = "meetings_contacts";
	var $module_dir = "Meetings";
	var $object_name = "Meeting";

	// This is used to retrieve related fields from form posts.
	var $additional_column_fields = array('assigned_user_name', 'assigned_user_id', 'contact_id', 'user_id', 'contact_name', 'accept_status');

	var $relationship_fields = array('account_id'=>'account','opportunity_id'=>'opportunity','case_id'=>'case',
									 'assigned_user_id'=>'users','contact_id'=>'contacts', 'user_id'=>'users');

	// so you can run get_users() twice and run query only once
	var $cached_get_users = null;
	var $new_schema = true;

	var $update_recurring = true;
	
	var $recur_chain_fields  = [
		"assigned_user_id",
		"description",
		"name",
		"status",
		"location",
		"duration",
		'reminder_time',
		'is_daylong',
		'date_start',
	];

	var $recur_date_fields  = [
		"duration",
		'is_daylong',
		'date_start',
	];

	function __construct() {
		parent::__construct();
		if (!in_array('recurrence_rules', $this->additional_column_fields)) {
			$this->additional_column_fields[] = 'recurrence_rules';
		}
	}


	/**
	 * Stub for integration
	 * @return bool
	 */
	function hasIntegratedMeeting() {
		return false;
	}

	function get_meeting_users() {
		$template = new User();
		// First, get the list of IDs.
		$query = "SELECT meetings_users.required, meetings_users.accept_status, meetings_users.user_id from meetings_users where meetings_users.meeting_id='$this->id' AND meetings_users.deleted=0";
		$GLOBALS['log']->debug("Finding linked records $this->object_name: ".$query);
		$result = $this->db->query($query, true);
		$list = Array();

		while($row = $this->db->fetchByAssoc($result)) {
			$template = new User(); // PHP 5 will retrieve by reference, always over-writing the "old" one
			$record = $template->retrieve($row['user_id']);
			$template->required = $row['required'];
			$template->accept_status = $row['accept_status'];

			if($record != null) {
				// this copies the object into the array
				$list[] = $template;
			}
		}
		return $list;
	}

	// longreach - start added
	function get_recurrence_forward_instances() {
		return AppConfig::setting('company.meetings_recurrence_max_instances', 30);
	}

	function get_recurrence_scheduled_interval() {
		return AppConfig::setting('company.meetings_recurrence_max_days', 30) * 24 * 3600 ;
	}

	function recur_pre_save()
	{
		$this->google_id = null;
	}
	
	static function make_day_long_duration($user, $date_start) {
		$dt_format_str = 'Y-m-d H:i:s';
		$d_format_str = 'Y-m-d';

		global $timedate;
		$date_start =  $timedate->swap_formats($date_start, $timedate->get_db_date_time_format(), $dt_format_str);
		if(! isset($user))
			$user = $GLOBALS['current_user'];
		if(isset($user->day_begin_hour)) {
			$hrs = $user->day_begin_hour;
			$startTime = sprintf('%02d:%02d', floor($hrs), ($hrs - floor($hrs)) * 60);
			$hrs = $user->day_end_hour;
			$endTime = sprintf('%02d:%02d', floor($hrs), ($hrs - floor($hrs)) * 60);
		}
		if(empty($startTime) || empty($endTime)) {
			$startTime = "09:00";
			$endTime = "18:00";
		}
		list($startHour, $startMin) = explode(":", $startTime);
		list($endHour, $endMin) = explode(":", $endTime);

		$user_date_time = $timedate->to_display_date_time($date_start, true, true, $user);
		$user_date_time = $timedate->swap_formats($user_date_time, $timedate->get_date_time_format(true, $user), $dt_format_str);
		list($user_date, $user_time) = explode(' ', $user_date_time);
		$d = $user_date . ' ' . $startTime . ':00';

        $d =  $timedate->swap_formats($d, $dt_format_str, $timedate->get_date_time_format());
        $d = $timedate->to_db($d);

        $duration = (($endHour - $startHour) * 60) + ($endMin - $startMin);

        return array('duration' => $duration, 'date_start' => $d);
	}

	static function mutateDetailButtons($detail, &$buttons) {
        $buttons['delete'] = array(
            'vname' => 'LBL_DELETE_BUTTON_LABEL',
            'accesskey' => 'LBL_DELETE_BUTTON_KEY',
            'order' => 30,
            'confirm' => 'NTC_DELETE_CONFIRMATION',
            'icon' => 'icon-delete',
            'acl' => 'delete',
            'params' => array(
                'record_perform' => 'delete',
            ),
            'type' => 'button',
            'hidden' => 'bean.is_recurring'
        );
        $buttons['delete_instance'] = array(
            'vname' => 'LBL_DELETE_BUTTON_LABEL',
            'accesskey' => 'LBL_DELETE_BUTTON_KEY',
            'order' => 30,
            'icon' => 'icon-delete',
            'type' => 'group',
            'hidden' => '! bean.is_recurring'
        );

		$delete_group = [
			'this' => 'LBL_DELETE_INSTANCE_SINGLE',
			'future' => 'LBL_DELETE_INSTANCE_FUTURE',
			'all' => 'LBL_DELETE_INSTANCE_ALL',
		];
		

        foreach ($delete_group as $name => $label) {
            $buttons['delete_' . $name] = array(
                'vname' => translate($label, 'Recurrence'),
                'confirm' => 'NTC_DELETE_CONFIRMATION',
                'icon' => 'icon-delete',
                'acl' => 'delete',
                'params' => array(
                    'record_perform' => 'delete',
                    'break_sequence' => $name
                ),
                'type' => 'button',
                'group' => 'delete_instance',
			);
        }
    }

    /**
     * @static
     * @param string $recurrence_rule_id
     * @return bool
     */
	static function is_recurring($recurrence_rule_id) {
		return !!self::load_recurrence_rule($recurrence_rule_id);
    }

	static function load_recurrence_rule($recurrence_rule_id) {
		require_once 'modules/Recurrence/RecurrenceRule.php';
		if (!$recurrence_rule_id) return false;
		$rule = ListQuery::quick_fetch('RecurrenceRule', $recurrence_rule_id);
		return $rule;
    }

	static function can_edit_recurrence($id, $recurrence_rule_id) {
		return !!self::load_recurrence_rule($recurrence_rule_id);
    }

    /**
     * @static
     * @param string $rule_id - recurrence rule ID
     * @param string $index - current recurring index
     * @param string $type: 'next' or 'prev'
     * @return null
     */
    static function get_recurring_instance($rule_id, $index, $type) {
        global $db;

        if ($type == 'next') {
			$operator = '>';
			$order = "ASC";
        } else {
            $operator = '<';
			$order = "DESC";
        }

        $query = "SELECT id FROM `meetings` WHERE deleted = 0 AND recurrence_rule_id = '$rule_id' AND recurrence_index {$operator} '$index' ORDER BY recurrence_index $order LIMIT 1";
        $result = $db->query($query, true, "Error retrieving recurring object");

        if ($row = $db->fetchByAssoc($result))
            return $row['id'];

        return null;
    }
	
	static function get_first_recurring_instance_id($rule_id) {
		global $db;
        $query = "SELECT id FROM `meetings` WHERE recurrence_rule_id = '$rule_id' AND deleted = 0 ORDER BY recurrence_index LIMIT 1";
        $result = $db->query($query, true, "Error retrieving recurring object");

		if ($row = $db->fetchByAssoc($result))
			return $row['id'];
		return null;
	}

    static function init_record(RowUpdate &$upd, $input) {
        global $current_user, $timedate;
        $update = array();

        if (! empty($input['duration_hours']) || ! empty($input['duration_minutes'])) {
            $duration = null;
            if (isset($input['duration_hours']))
                $duration += $input['duration_hours'] * 60;
            if (isset($input['duration_minutes']))
                $duration += $input['duration_minutes'];
            $update['duration'] = $duration;
        }
        if($upd->new_record && ! $upd->getField('duration') && empty($update['duration']))
			$update['duration'] = 30;

        if (! empty($input['date_start']) && (! empty($input['time_hour_start']) || ! empty($input['time_start'])) ) {
            $dt_start = $input['date_start'];
            if(! preg_match('/\d{4}-\d{2}-\d{2}/', $dt_start))
                $dt_start = gmdate('Y-m-d');

            if (! empty($input['time_start'])) {
                $tm_start = $input['time_start'];
                if(! preg_match('/\d{2}:\d{2}(:\d{2})/', $tm_start))
                    $tm_start = gmdate('H:i:s');
            } else if(isset($input['time_hour_start'])) {
                $h = $input['time_hour_start'];
                $m = array_get_default($input, 'time_minute_start', '00');
                $tm_start = sprintf('%02d:%02d:00', $h, $m);
            } else {
                $tm_start = '';
            }

            $update['date_start'] = $timedate->handle_offset($dt_start .' '.$tm_start, $timedate->get_db_date_time_format(), false);
        } /* elseif (! empty($input['date_start'])) {
            $update['date_start'] = $timedate->handle_offset($input['date_start'], $timedate->get_db_date_time_format(), false);
            $update['date_start'] = $input['date_start'];
        }*/
        else if (! $upd->getField('date_start')) {
			$tm = time();
            $tm = ceil($tm / 3600) * 3600;
            $update['date_start'] = gmdate('Y-m-d H:i:s', $tm);
        }
        
        self::set_default_reminder_time($upd);
        
        $parent_type = $upd->getField('parent_type');
        $parent_id = $upd->getField('parent_id');
        if($parent_type == 'Accounts' && $parent_id) {
        	$upd->set(array('parent_type' => null, 'parent_id' => null, 'account_id' => $parent_id));
        	$parent_type = $parent_id = null;
        }

        $acc_f = $parent_type ? get_standard_account_field($parent_type) : null;
        if($acc_f && $parent_id && ! $upd->getField('account_id')) {
        	$acc_m = AppConfig::module_primary_bean($parent_type);
			$acc_p = ListQuery::quick_fetch_row($acc_m, $parent_id, array($acc_f['id']));
			if($acc_p)
				$update['account_id'] = $acc_p[$acc_f['id']];
        }
        
        $upd->set($update);

        if(! $upd->getField('name')) {
			if(($parent_type == 'Contacts' || $parent_type == 'Leads') && $parent_id) {
				$pq = ListQuery::quick_fetch(AppConfig::module_primary_bean($parent_type), $parent_id, ['name']);
				$meet_with = $pq->getField('name');
			} else if( ($acc_id = $upd->getField('account_id')) ) {
				$pq = ListQuery::quick_fetch('Account', $acc_id, ['name']);
				$meet_with = $pq->getField('name');
			}
			if(! empty($meet_with))
				$upd->set('name', translate('LBL_MEETING_WITH_PREFIX', 'Meetings') . $meet_with);
		}
    }

	static function set_default_reminder_time(RowUpdate &$upd) {
		global $current_user;
		$remind = $current_user->getPreference('reminder_time');
		if($remind) $upd->set('reminder_time', $remind);
	}

	/**
     * Set Meeting/Call invite status
     *
     * @static
     * @param RowUpdate $upd
     * @param RowResult $entity
     * @param string $status: accept, decline, tentative
     * @return void
     */
    static function set_invite_status(RowUpdate &$upd, RowResult $entity, $status) {
        $link_name = null;

        if ($entity->base_model == 'User') {
            $link_name = 'users';
        } elseif ($entity->base_model == 'Contact') {
            $link_name = 'contacts';
        }

        if ($link_name != null) {
            $id = $entity->getField('id');
            $params[$id] = array('name' => 'accept_status', 'value' => $status);
            $upd->addUpdateLink($link_name, $id, $params);
            return true;
        }
    }

    static function get_invites($id, $type = 'users') {
        global $db;

        $id_name = 'user_id';
        if ($type == 'contacts')
            $id_name = 'contact_id';

        $query = "SELECT $type.id, $type.first_name, $type.last_name, $type.email1, $type.email2 FROM meetings_$type
            LEFT JOIN $type ON meetings_$type.$id_name = $type.id
            WHERE meetings_$type.meeting_id='$id' AND ( meetings_$type.accept_status IS NULL OR meetings_$type.accept_status = 'none') AND meetings_$type.deleted = 0";

        $result = $db->query($query, true);
        $list = array();

        while($row = $db->fetchByAssoc($result, -1, false)) {
            $invite = array(
                'id' => $row['id'],
                'full_name' => $row['first_name'] .' '. $row['last_name'],
                'receive_notifications' => 1,
                'email1' => $row['email1'],
                'email2' => $row['email2'],
                'user_name' => '',
                'type' => $type
            );

            $list[] = $invite;
        }

        return $list;
    }

	static function send_apns(RowUpdate $upd) {
		require_once 'include/utils/apns_utils.php';
		$prev_user_id = $upd->getField('assigned_user_id', null, true);
		$user_id = $upd->getField('assigned_user_id');
		$user_changed = $prev_user_id && ($prev_user_id != $user_id);

		$prev_reminder = $upd->getField('reminder_time', null, true);
		$reminder = $upd->getField('reminder_time');
		$reminder_changed = $prev_reminder && ($prev_reminder != $reminder);

		$prev_start = $upd->getField('date_start', null, true);
		$start = $upd->getField('date_start');
		$start_changed = $prev_start && ($prev_start != $start);

		$prev_name = $upd->getField('name', null, true);
		$name = $upd->getField('name');
		$name_changed = $prev_name && ($prev_name != $name);

		if ($user_changed) {
			apns_events_updated($user_id);
			apns_events_updated($prev_user_id);
		}
		if ((!$prev_user_id && $user_id) || $reminder_changed || $start_changed || $name_changed) {
			apns_events_updated($user_id);
		}
	}

    static function before_save(RowUpdate $upd) {
        require_once('modules/Calendar/DateTimeUtil.php');
        $start = $upd->getField('date_start');
        $dt_start = explode(' ', $start, 2);

        if ($upd->getField('is_daylong')) {
            $daylong_params = self::make_day_long_duration(null, $start);
            $duration = $daylong_params['duration'];
            $upd->set('duration', $duration);
            $upd->set('date_start', $daylong_params['date_start']);
            $dt_start = explode(' ', $daylong_params['date_start'], 2);
        } else {
            $duration = $upd->getField('duration');
        }

        if(count($dt_start) == 2) {
            $date_end = date('Y-m-d', strtotime(implode(' ', $dt_start) . ' GMT + ' . $duration . 'minutes'));;
            $upd->set('date_end', $date_end);
		}

		if (!$upd->new_record && $upd->getField('recurrence_rule_id')) {
			$update_mode = $upd->getRelatedData('break_sequence', 'this');
			if ($update_mode == 'this') {
				$upd->set('recurrence_exception', 1);
			}
		}

	    $input = $upd->getInput();
	    if (! isset($input['reminder_time']))
		    self::set_default_reminder_time($upd);

	    require_once('include/OnlineMeetings/OnlineMeetingProcess.php');
        $online_meeting_process = new OnlineMeetingProcess();
        $online_meeting_process->run($upd);
    }
	
	static function update_vcal(RowUpdate $upd) {
		if(!version_compare(phpversion(), "5.4", ">="))
			return;
		if (!$upd->getField('id')) {
			$id = create_guid();
			$upd->set('id', $id);
			$upd->set('vcal_uri', $id . '.ics');
		}
		$vcal = vCalUtils::vcal_update($upd, null, true);
		$upd->set('vcal', $vcal);
		$upd->set('vcal_size', strlen($vcal));
	}

	static function get_online_meeting_create($gtm_id, $online_meeting_params) {
		require_once('include/OnlineMeetings/OnlineMeetingsFactory.php');

		try {
			$onlineMeeting = OnlineMeetingsFactory::getInstance();
		} catch (OnlineMeetingException $e) {
			return 0;
		}

		if ($onlineMeeting->getAppName() == 'gotomeeting') {
			$result = empty($gtm_id) ? 0 : 1;
		} else {
			$params = $onlineMeeting->decodeStoredParams($online_meeting_params);
			$result = ! empty($params) ? 1 : 0;
		}

		return $result;
	}

	static function get_online_meeting_url($gtm_id, $online_meeting_params) {
		require_once('include/OnlineMeetings/OnlineMeetingsFactory.php');

		try {
			$onlineMeeting = OnlineMeetingsFactory::getInstance();
		} catch (OnlineMeetingException $e) {
			return '';
		}

		if ($onlineMeeting->getAppName() == 'gtm') {
			return $onlineMeeting->getJoinUrl(['gtm_id' => $gtm_id]);
		} else {
			return $onlineMeeting->getJoinUrl(
				$onlineMeeting->decodeStoredParams($online_meeting_params)
			);
		}
	}

	static function get_zoom_meeting_password($online_meeting_params) {
		require_once('include/OnlineMeetings/OnlineMeetingsFactory.php');

		try {
			$onlineMeeting = OnlineMeetingsFactory::getInstance();
		} catch (OnlineMeetingException $e) {
			return '';
		}

		if ($onlineMeeting->getAppName() == 'zoom') {
			$params = $onlineMeeting->decodeStoredParams($online_meeting_params);

			if (isset($params['extra_data']) && isset($params['extra_data']['password'])) {
				return $params['extra_data']['password'];
			}

		} else {
			return '';
		}
	}

	static function load_request(RowUpdate $upd, $req) {
		if (isset($req['recurrence_rules'])) {
			$upd->setRelatedData('recurrence_rules', $req['recurrence_rules']);
			$upd->setRelatedData('recurrence_rules_convert_date', true);
			if (!$upd->getField('recurrence_rule_id'))
				$upd->set('recurrence_rule_id', create_guid());
		}
		if(isset($req['break_sequence'])) {
			$upd->setRelatedData('break_sequence', $req['break_sequence']);
		}
		$upd->setRelatedData('online_meeting_create', array_get_default($req, 'online_meeting_create'));
    }

	static function reset_recurrence(RowUpdate $upd) {
		$rules = $upd->getRelatedData('recurrence_rules');
		$recurrence_rule_id = $upd->getField('recurrence_rule_id');
		$upd->setRelatedData('recurrence_rule_id', $recurrence_rule_id);
		if ($rules) {
			$rules = @json_decode($rules);
			if (empty($rules)) {
				$upd->set('recurrence_rule_id', null);
			}
		}
	}

	static function prepare_recurrence_update(RowUpdate $upd) {
		global $timedate;
		$recurrence_rule_id = $upd->getField('recurrence_rule_id');
		if (!$recurrence_rule_id) return;
		$update_mode = $upd->getRelatedData('break_sequence', 'this');
		if ($upd->getField('recurrence_exception')) {
			$update_mode = 'this';
		}
		if ($update_mode != 'all' && $update_mode != 'future') {
			if ($update_mode != 'none') {
				$upd->setRelatedData('break_sequence', 'this');
			}
			return;
		}
		$upd->set('recurrence_exception', 0);
		$rule = ListQuery::quick_fetch('RecurrenceRule', $recurrence_rule_id);
		if (!$rule) {
			$upd->setRelatedData('break_sequence', 'this');
			$upd->setRelatedData('recurrence_rule_id', '');
			return;
		}
		// collect changes and make sure the specified update mode can be applied
		$origDate = $upd->getField('date_start', null, true);
		$newDate = $upd->getField('date_start', null, false);
		$user_id = $upd->getField('assigned_user_id');
		$userOrigDate = $timedate->to_display_date($origDate, true, $user_id);
		$userNewDate = $timedate->to_display_date($newDate, true, $user_id);
		$userNewTime = $timedate->to_display_time($newDate, true, $user_id);

		$newTemplate = $oldTemplate = @json_decode($rule->getField('template'), true);
		foreach ($oldTemplate as $k => $v) {
			if ($k == 'date_start') continue;
			if ($upd->getFieldUpdated($k)) {
				$newTemplate[$k] = $upd->getField($k);
			}
		}

		if ($userOrigDate != $userNewDate || $update_mode == 'future') {
			$update_mode = 'future';
			$upd->setRelatedData('break_sequence', $update_mode);
			$newTemplate['date_start'] = $newDate;
		} else {
			$tplOrigDate = $timedate->to_display_date($oldTemplate['date_start'], true, $user_id);
			$newTemplate['date_start'] = $timedate->to_db($tplOrigDate . ' ' . $userNewTime, true, $user_id);
		}
		
		$ruleUpd = RowUpdate::for_result($rule);
		if ($update_mode == 'all') {
			$ruleUpd->set('template', json_encode($newTemplate));
		} else {
			$prevDay = gmdate("Y-m-d H:i:s", strtotime($upd->getField('recurrence_start_time') . ' GMT -1 day'));
			$userDateTime = $timedate->handle_offset($prevDay, "Y-m-d H:i:s", true, $user_id);
			list($userDate, $_) = explode(' ', $userDateTime, 2);
			$until = $timedate->handle_offset($userDate . " 23:59:59", "Y-m-d H:i:s", false, $user_id);

			$ruleUpd->set('until', $until);
			$newRuleId = create_guid();
			$upd->set('recurrence_rule_id', $newRuleId);
			$upd->set('recurrence_index', 0);
			$newRule = $ruleUpd::blank_for_model('RecurrenceRule');
			$newRule->set('id', $newRuleId);
			$newRule->set([
				'template' => json_encode($newTemplate),
				'parent_type' => 'Meetings',
				'freq' => $rule->getField('freq'),
				'freq_interval' => $rule->getField('freq_interval'),
				'until' => $rule->getField('until'),
				'rule' => $rule->getField('rule'),
			]);
			$upd->setRelatedData('new_recurring_rule', $newRule);
		}
		$upd->setRelatedData('recurring_rule_update', $ruleUpd);
	}

    static function update_recurrence(RowUpdate $upd) {
		$recurrence_rule_id = $upd->getField('recurrence_rule_id');
		if (!$recurrence_rule_id) return;
        require_once 'modules/Recurrence/RecurrenceRule.php';
		$rule = new RecurrenceRule;
		$new_rules = $upd->getRelatedData('recurrence_rules');
		$update_mode = $upd->getRelatedData('break_sequence', 'this');
		$rule_id = $upd->getRelatedData('recurrence_rule_id');
		if($rule_id && isset($new_rules) && $upd->new_record) { 
			// just creates new instances for a new meeting. For existing meetings, updates are handled differently
			$rule->update_rules_from_JSON($rule_id, 'Meeting', $upd->getRow(), $new_rules, true);
			return;
		}
		$recurring_rule_update = $upd->getRelatedData('recurring_rule_update');
		$new_recurring_rule = $upd->getRelatedData('new_recurring_rule');
		if ($update_mode == 'all') {
			$recurring_rule_update->save();
		}
		if ($update_mode == 'future') {
			$recurring_rule_update->save();
			$new_recurring_rule->save();
		}
		if (!$upd->getRelatedData('disable_vcal_rebuild') && $upd->getField('recurrence_exception')) {
			vCalUtils::rebuild_recur_vcal($upd->getField('recurrence_rule_id'));
		}
    }

    static function update_invitees(RowUpdate $upd) {
        require_once('modules/Meetings/ActivityInvite.php');
        $row_updates = $upd->getRelatedData('event_scheduler_rows');
		$linked = ActivityInvite::update_links($upd, $row_updates);
        $send = $upd->getRelatedData('send_invites');
        if ($send && $linked)
            self::send_invites($upd);
    }
    
    static function send_invites(RowUpdate $upd) {
    	require_once('include/Notify/RecordActionMessage.php');
    	$message = RecordActionMessage::for_action($upd->getModelName(), 'invited', $upd->getUserId());
    	if($message) {
			$message->addRecordId($upd->getPrimaryKeyValue());
			$inv = $message->getInvitees();
			if($inv) {
				$mailer = $message->getMailer();
				foreach($inv as $tgt)
					$mailer->addTargetInfo($tgt);
				$mailer->sendMessage($message);
			}
		}
    }

	static function before_delete(RowUpdate $upd) {
		global $timedate;
        $rule = self::load_recurrence_rule($upd->getField('recurrence_rule_id'));
        $user_id = $upd->getField('assigned_user_id');
        if ($rule) {
            require_once 'modules/Recurrence/RecurrenceRule.php';
			$mode = $upd->getRelatedData('break_sequence', 'this');
			if ($mode == 'this') {
				$upd->set('recurrence_exception', 1);
				return;
			}
			$upd->set('recurrence_exception', 0);
			$ruleUpd = RowUpdate::for_result($rule);
			if ($mode == 'all') {
				$upd->setRelatedData('after_delete_action', 'delete');
			} elseif ($mode == 'future') {
				$prevDay = gmdate("Y-m-d H:i:s", strtotime($upd->getField('recurrence_start_time') . ' GMT -1 day'));
				$userDateTime = $timedate->handle_offset($prevDay, "Y-m-d H:i:s", true, $user_id);
				list($userDate, $_) = explode(' ', $userDateTime, 2);
				$until = $timedate->handle_offset($userDate . " 23:59:59", "Y-m-d H:i:s", false, $user_id);
				$ruleUpd->set('until', $until);
				$upd->setRelatedData('after_delete_action', 'save');
			}
			$upd->setRelatedData('after_delete_rule', $ruleUpd);
        }
    }

	static function after_delete(RowUpdate $upd) {
        require_once('modules/Meetings/ActivityInvite.php');
        ActivityInvite::handle_delete($upd);
		$mode = $upd->getRelatedData('break_sequence', 'this');
		$rule = $upd->getRelatedData('after_delete_rule');
		if ($rule && $mode != 'this') {
			$action = $upd->getRelatedData('after_delete_action');
			switch ($action) {
			case 'delete':
				$rule->markDeleted();
				break;
			case 'save':
				$rule->save();
				break;
			}
		}
		if (!$upd->getRelatedData('disable_vcal_rebuild') && $upd->getField('recurrence_exception')) {
			vCalUtils::rebuild_recur_vcal($upd->getField('recurrence_rule_id'));
		}
    }
    
    static function update_user_vcal(RowUpdate $upd, $link_name=null) {
    	if($link_name == 'users') {
			require_once('modules/Meetings/ActivityInvite.php');
			$link = $upd->getLinkUpdate();
			ActivityInvite::update_vcal($link->getField('user_id'));
    	}
    }

    static function get_activity_status(RowUpdate $upd) {
        $status = null;

        if (! $upd->new_record) {
            if ($upd->getField('date_start') != $upd->getField('date_start', null, true))
                $status = 'moved';
        }

        return $status;
    }
	
	public static function additional_details_title($record, &$title)
	{
		if ($record) {
			if (AppConfig::current_user_id() != $record->getField('assigned_user_id')) {
				$is_private = $record->getField('is_really_private', '__UNSET__');
				if ($is_private === '__UNSET__') 
					$is_private = $record->getField('is_private');
				if ($is_private) {
					$title = translate('LBL_PRIVATE_MEETING', 'Meetings');
				}
			}
		}
	}
	
	public static function is_really_private($id, $is_private) {
		if (!$is_private) return false;
        $lq = new ListQuery('Meeting', null, array('link_name' => 'users', 'parent_key' => $id));
        $lq->addSimpleFilter('id', AppConfig::current_user_id());
		$count = $lq->fetchResultCount();
		return $count == 0;
	}
	
	function get_recurrence_exclude_template_fields() {
		return [
			'id',
			'date_entered',
			'date_modified',
			'modified_user_id',
			'created_by',
			'deleted',
			'outlook_id',
			'gtm_id',
			'gtm_email',
			'gtm_org_id',
			'vcal_size',
			'vcal',
			'vcal_uri',
			'sf_id',
			'recurrence_of_id',
			'recurrence_index',
			'recurrence_rules',
			'forward_times',
			'recurrence_exception',
			'recurrence_start_time',
		];
	}

}

