<?php 

class RefreshGTMToken {

	static $GTM_LOCK_FILE = 'cache/.gotomeeting.lock';
	var $lock_handle = null;


	static function do_refresh() {
		$inst = new RefreshGTMToken();
		$inst->doRefresh();
	}

	function lock() {
		$this->lock_handle = @fopen(self::$GTM_LOCK_FILE, 'w');
		if($this->lock_handle)
			flock($this->lock_handle, LOCK_EX);
		return $this->lock_handle;
	}

	function unlock() {
		if ($this->lock_handle) {
			flock($this->lock_handle, LOCK_UN);
			@fclose($this->lock_handle);
			$this->lock_handle = null;
		}
	}
		
	function checkStatus() {
		require_once 'modules/UserPreferences/UserPreference.php';

		$system_token = AppConfig::setting('company.gtm_auth_token');
		$user_token = UserPreference::getPreference('gtm_token');
		$system_refresh = false;
		$user_refresh = false;
		$now = time();

		if ($system_token) {
			$system_token = @unserialize($system_token);
			if (! $system_token) {
				$GLOBALS['log']->debug('System-wide GoToMeeting token could not be decoded, will not be refreshed');
			} else if ($system_token['refresh_after'] >= $now) {
				$GLOBALS['log']->debug('System-wide GoToMeeting token is new enough, no need to refresh');
			} else if ($system_token['refresh_before'] < $now) {
				$GLOBALS['log']->debug('System-wide GoToMeeting token expired, no refresh attempt needed');
			} else {
				$system_refresh = true;
			}
		}
		
		if ($user_token) {
			if ($user_token['refresh_after'] >= $now) {
				$GLOBALS['log']->debug('GoToMeeting token for user ' . AppConfig::current_user_id() . ' is new enough, no need to refresh');
			} else if ($user_token['refresh_before'] < $now) {
				$GLOBALS['log']->debug('GoToMeeting token for user ' . AppConfig::current_user_id() . ' expired, no refresh attempt needed');
			} else {
				$user_refresh = true;
			}
		}

		$any_refresh = $system_refresh || $user_refresh;
		return compact(
			'system_refresh', 'system_token',
			'user_refresh', 'user_token',
			'any_refresh');
	}

	function doRefresh() {
		$this->updateConfigSetting();

		$status = $this->checkStatus();
		if(! $status['any_refresh'])
			return;

		if(! $this->lock())
			return;
		
		// check again now that we have a lock
		$status = $this->checkStatus();
		if(! $status['any_refresh'])
			return;

		require_once 'include/OnlineMeetings/GoToMeeting/GoToMeeting.php';
		
		if ($status['system_refresh']) {
			try {
				$gtm = new GoToMeeting($status['system_token']);
				$newToken = $gtm->refreshToken();
				$info = $gtm->obtainOrganizerInfo();
				$newToken['organizer_email'] = $info[0]['email'];
				AppConfig::set_local('company.gtm_auth_token', serialize($newToken), true);
				AppConfig::save_local('company');
			} catch (Exception $e) {
				$GLOBALS['log']->fatal('Error refreshing system-wide GoToMeeting token  ' . AppConfig::current_user_id() . ': ' . $e->getMessage());
			}
		}
		
		if ($status['user_refresh']) {
			try {
				$gtm = new GoToMeeting($status['user_token']);
				$newToken = $gtm->refreshToken();
				$info = $gtm->obtainOrganizerInfo();
				$newToken['organizer_email'] = $info[0]['email'];
				UserPreference::setPreference('gtm_token', $newToken);
				UserPreference::savePreferencesToDB();
			} catch (Exception $e) {
				$GLOBALS['log']->fatal('Error refreshing GoToMeeting token for user ' . AppConfig::current_user_id() . ': ' . $e->getMessage());
			}
		}
		
		$this->unlock();
	}

	/**
	 * Need this method for backward compatibility.
	 * Set GoToMeeting as Online Meeting App
	 */
	function updateConfigSetting() {
		$meeting_app = AppConfig::setting('company.online_meeting_app');
		$gtm_enabled = AppConfig::setting('site.features.gotomeeting.enabled');
		$gtm_auth_token = AppConfig::setting('company.gtm_auth_token');

		if ( empty($meeting_app) && ($gtm_enabled && ! empty($gtm_auth_token)) ) {
			AppConfig::set_local('company.online_meeting_app', 'gotomeeting');
			AppConfig::save_local();
		}
	}
}

