<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

require_once('modules/Workflow/WFProcessorBase.php');

class ScheduleMeetingProc extends WFProcessorBase {
	var $related_model = 'Meeting';

	function initRelatedRecord(RowUpdate $rowUpdate) {
		$meeting = parent::initRelatedRecord($rowUpdate);
		$module = $rowUpdate->getModuleDir();
		$rowId = $rowUpdate->getPrimaryKeyValue();
		$upd = array(
			'status' => 'Planned',
			'date_start' => $this->getRelativeDate('date_start'),
		);
		if ($module == 'Accounts') {
			$upd['account_id'] = $rowId;
		} else {
			$upd['parent_type'] = $module;
			$upd['parent_id'] = $rowId;
		}
		$meeting->set($upd);
		return $meeting;
	}

	function processRelatedRecord(RowUpdate $upd, RowUpdate $meeting) {
		$this->addInvitees($upd, $meeting);
	}

	function addInvitees(&$rowUpdate, &$focus) {
		$ids = preg_split('/\s*;\s*/', $this->getParam('invitee_ids'));
		$types = preg_split('/\s*;\s*/', $this->getParam('invitee_types'));

		$module = $rowUpdate->getModuleDir();
		$rowId = $rowUpdate->getPrimaryKeyValue();

		$invitees = array();
		foreach($ids as $idx => $cid) {
			if(! isset($types[$idx])) break;
			$invitees[$types[$idx]][] = $cid;
		}

		if ($this->getParam('include_contact') && $module == 'Contacts') {
			$invitees['Contacts'][] = $rowId;
		}

		if(isset($invitees['Contacts'])) {
			foreach($invitees['Contacts'] as $contact_id) {
				$focus->addUpdateLink('contacts', $contact_id);
			}
		}
		if(isset($invitees['Leads'])) {
			foreach($invitees['Leads'] as $contact_id) {
				$focus->addUpdateLink('leads', $contact_id);
			}
		}
		if(isset($invitees['Users'])) {
			foreach($invitees['Users'] as $user_id) {
				$focus->addUpdateLink('users', $user_id);
			}
		}
		return $invitees;
	}

	function getConfigFields() {
		$fields = parent::getConfigFields();
		$model = new ModelDef('Meeting');

		$fields['name'] = $model->getFieldDefinition('name');
		$fields['name']['width'] = 60;
		$fields['invitee_ids'] = array(
			'vname' => 'LBL_INVITEE',
			'vname_module' => 'Meetings',
			'widget' => 'InviteesList',
			'type_name' => 'invitee_types',
		);
		if($this->module == 'Contacts')
			$fields['include_contact'] = array(
				'vname' => 'LBL_INCLUDE_CONTACT',
				'type' => 'bool',
			);
		$fields['invitee_types']['hidden'] = true;
		$fields['date_start'] = array(
			'vname' => 'LBL_DATE_TIME',
			'vname_module' => 'Meetings',
			'widget' => 'RelativeTimeInput',
			'check_hide' => 'date_start_type',
		);
		$fields['date_start_type']['hidden'] = true;
		$fields['date_start_count'] = array(
			'type' => 'int',
			'hidden' => true,
		);
		$fields['date_start_time'] = array(
			'type' => 'time',
			'hidden' => true,
		);
		$fields['duration'] = $model->getFieldDefinition('duration');
		$fields['duration']['default'] = 60;
		$fields['is_daylong'] = $model->getFieldDefinition('is_daylong');
		$fields['is_daylong']['hidden'] = true;
		$fields['description'] = $model->getFieldDefinition('description');

		return $fields;
	}

}

