// vim: autoindent:smartindent:
/**
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 */

//////////////////////////////////////////////////
// class: SugarWidgetScheduler
// widget to display the meeting scheduler
//
//////////////////////////////////////////////////


AppBase.module('schedule-viewer', 'ui-controls').init(function(App) {
	var UI = SUGAR.ui,
		Dom = SUGAR.dom,
		Wrap = Dom.elementOrId,
		MakeElt = Dom.makeElement,
		EltSpec = Dom.ElementSpec;

	UI.HourGrid = function(params) {
		this.hour_size = 60;
		this.hour_color = '#bbb';
		this.minute_color = '#ddd';
		this.hour_stroke = 1;
		this.minute_stroke = 0.5;
		if(params) extendObject(this, params);
	}
	UI.HourGrid.prototype = {
		render: function() {
			if(! this.elt) {
				var hsz = this.hour_size,
					msz = hsz / 4,
					path1 = '',
					path2 = 'M ' + (this.hour_stroke / 2) + ' 0 V 10';
				for(var i = 1; i < 4; i++)
					path1 += 'M ' + (msz * i + this.minute_stroke / 2) + ' 0 V 10 ';
				var svg = EltSpec({tagName: 'svg', ns: 'http://www.w3.org/2000/svg'}, {version: '1.1', width: hsz, height: '100'})
						.append(EltSpec('defs')
							.append(EltSpec('pattern', {id: 'grid', width: hsz, height: 10, patternUnits: 'userSpaceOnUse'})
								.append(EltSpec('path', {d: path1, fill: 'none', stroke: this.minute_color, 'stroke-width': this.minute_stroke}))
								.append(EltSpec('path', {d: path2, fill: 'none', stroke: this.hour_color, 'stroke-width': this.hour_stroke}))
							))
						.append(EltSpec('rect', {width: '100%', height: '100%', fill: 'url(#grid)'}));
				this.elt = svg.toElement();
			}
			return this.elt;
		},
		getDataUri: function() {
			var xml = (new XMLSerializer()).serializeToString(this.render());
			return 'data:image/svg+xml;base64,' + btoa(xml);
		},
		getImage: function() {
			return MakeElt('img', {src: this.getDataUri()});
		}
	};


	UI.ScheduleViewer = function(id, params) {
		this.hour_size = 60;
		this.hours = 12;
		this.segments = 4;
		this.attends = [];
		this.orig_attends_hash = {};
		this.timeslots = [];
		this.hash_format = '%Y%m%d%H%M%S';
		this.dialog = 0;
		this.basic = 0;
		this.show_attend = true;
		this.pad_inner = 4;
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		if(params) extendObject(this, params);
		if(! Array.isArray(this.attends)) this.attends = [];
	}
	UI.ScheduleViewer.prototype = {
		setup: function() {
			if(this.set_up) return;
			var self = this;
			if(! this.elt) this.elt = $(this.id + '-main');
			if(! this.elt) return;
			this.date_start_input = UI.getFormInput(this.form, 'date_start');
			this.user_input = UI.getFormInput(this.form, 'assigned_user');
			this.duration_input = UI.getFormInput(this.form, 'duration') || UI.getFormInput(this.form, 'quantity');
			if(! this.date_start_input || ! this.duration_input)
				return;
			if(this.basic || ! this.user_input)
				this.show_attend = false;
			this.hour_grid = new UI.HourGrid({hour_size: this.hour_size});
			this.addHooks();
			UI.onInitForm(this.form, function() { self.setOriginal(); self.render(); });
			this.set_up = true;
		},

		setOriginal: function() {
			this.orig_attends_hash = {};
			if(this.form.record.value) {
				for(var i = 0; i < this.attends.length; i++)
					this.orig_attends_hash[this.attends[i].id] = 1;
				this.orig_start = this.getStartDate();
				this.orig_start_ts = Math.floor(this.orig_start.getTime() / 1000);
				this.orig_duration = this.getDuration();
			}
		},

		getStartDate: function() {
			var dt = this.date_start_input.getValue(true);
			if(! dt) dt = new Date();
			return dt;
		},

		getDuration: function() {
			return this.duration_input.getValue();
		},

		initTimeSlots: function() {
			var segment_len = 3600/this.segments,
				dt_start = this.getStartDate(),
				duration = this.getDuration(),
				duration_minutes = duration * 60,
				hours_before = Math.min(5, Math.max(1, Math.floor((this.hours - duration/60) / 2))),
				begin_hour = SUGAR.session.day_begin_hour - 1,
				curdate, have_start, have_end, have_orig_start, have_orig_end, start_ts, ts, obj;

			start_ts = Math.floor(dt_start.getTime() / 1000);
			curdate = new Date(dt_start.getFullYear(), dt_start.getMonth(), dt_start.getDate(), dt_start.getHours(), 0, 0);
			hours_before = Math.max(1, Math.min(hours_before, curdate.getHours() - begin_hour));
			curdate.setHours(curdate.getHours() - hours_before);
			this.timeslots = [];

			var timef = getTimeFormat().print_format,
				timef_short = '',
				timef_long = '';
			if(timef.indexOf('%H') >= 0) {
				timef_short = '%H';
			}
			else if(timef.indexOf('%h') >= 0) {
				timef_short = '%h';
			}
			else {
				timef_short = '%l';
			}
			timef_long = timef_short;
			if(timef.indexOf('%P')) {
				timef_long += '%P';
			}
			else if(timef.indexOf('%p')) {
				timef_long += '%p';
			}

			for(var i=0; i < this.hours*this.segments; i++) {
				ts = Math.floor(curdate.getTime() / 1000);
				obj = {hash: curdate.print(this.hash_format), ts: ts, date_obj: curdate};
				if(i == 0 || curdate.getHours() === 0 || curdate.getHours() === 12)
					obj.format_hour = curdate.print(timef_long);
				else
					obj.format_hour = curdate.print(timef_short);

				if(! have_start && start_ts <= ts) {
					obj.is_start = true;
					have_start = true;
				}
				if(! have_end && start_ts + duration_minutes <= ts) {
					obj.is_end = true;
					have_end = true;
				}
				if(! have_orig_start && this.orig_start_ts && this.orig_start_ts <= ts) {
					obj.is_orig_start = true;
					have_orig_start = true;
				}
				if(have_orig_start && ! have_orig_end && this.orig_start_ts + this.orig_duration*60 <= ts) {
					obj.is_orig_end = true;
					have_orig_end = true;
				}
				this.timeslots.push(obj);

				curdate = new Date((ts + segment_len)*1000);
			}
		},

		addHooks: function() {
			var self = this,
				updfn = function() { self.timeUpdated(); };
			if(this.form) {
				UI.attachInputEvent(this.date_start_input, 'onchange', updfn);
				UI.attachInputEvent(this.duration_input, 'onchange', updfn);
				if(this.user_input)
					UI.attachInputEvent(this.user_input, 'onchange', function() { self.userUpdated(); });
			}
			Wrap(this.elt).find('.scheduler-add').on('click', this.showSearcher, {scope: this});
		},

		timeUpdated: function() {
			this.initTimeSlots();
			this.clearFreeBusy();
			this.renderBody();
		},

		userUpdated: function() {
			if(this.checkAssignedUser()) {
				this.loadFreeBusy();
				this.renderBody();
			}
		},

		checkAssignedUser: function() {
			var uid, display;
			if(this.user_input) {
				uid = this.user_input.getKey();
				display = this.user_input.getValue();
			} else {
				uid = SUGAR.session.user_id;
				display = SUGAR.session.display_name;
			}
			this.assigned_id = uid;
			if(! uid || ! display) return;
			for(var i = 0; i < this.attends.length; i++) {
				if(this.attends[i].module == 'Users' && this.attends[i].id == uid)
					return false;
			}
			this.attends.splice(0, 0, {module: 'Users', id: uid, _display: display});
			return true;
		},

		addRow: function(row, module) {
			if(! row || ! row.id) return;
			if(module) row.module = module;
			for(var i = 0; i < this.attends.length; i++) {
				if(this.attends[i].module == row.module && this.attends[i].id == row.id)
					return;
			}
			this.attends.push(row);
			this.loadFreeBusy();
			this.renderBody();
		},

		deleteRow: function(index) {
			// FIXME - enforce assigned user
			this.attends.splice(index, 1);
			this.renderBody();
		},

		render: function() {
			this.initTimeSlots();
			this.checkAssignedUser();
			this.loadFreeBusy();
			this.renderBody();
		},

		renderBody: function() {
			if(! this.elt)
				return;

			var i, j, img, row, tr, cell, self = this,
				top_date = this.getStartDate().print(getDateFormat().print_format),
				outer = Wrap(this.elt),
				date = outer.findFirst('.scheduler-date')
					.text(top_date),
				times = outer.findFirst('.scheduler-times').ext(0),
				body = outer.findFirst('.scheduler-body').ext(0);

			if(times) {
				times.empty();
				times.css('position', 'relative');
				for(j=0; j < this.timeslots.length; j += this.segments) {
					cell = MakeElt('div.scheduler-hour.input-label.single')
						.prop('ts', j)
						.css('position', 'absolute')
						.css('line-height', '1.25')
						.appendTo(times)
						.position({left: j * this.hour_size/this.segments + 4})
						.text(this.timeslots[j].format_hour);
				}
			}

			if(! body) return;
			body.empty();

			for(i = 0; i < this.attends.length; i++) {
				row = this.attends[i];
				tr = MakeElt('div.row.col-space-sm.scheduler-hours').appendTo(body);

				if(1 || this.show_attend) {
					img = (row.module == 'Users' ? 'bean-User' : 'module-'+row.module);
					img = MakeElt('div.input-icon.no-flex.theme-icon.'+img, {style: 'margin-right: 0.5em'});
					cell = MakeElt('div.column.scheduler-col-user.col-3.tools-row', img)
						.append(MakeElt('span', row._display)).appendTo(tr);
					if(! this.basic && row.id != this.assigned_id) {
						img = MakeElt('div.uii.uii-lg.uii-remove.active-icon')
							.prop('idx', i)
							.on('click', function() { self.deleteRow(this.idx); })
							.appendTo(cell);
					}
				}

				cell = MakeElt('div.column.scheduler-col-main').appendTo(tr);
				cell.css('background-image', 'url(' + this.hour_grid.getDataUri() + ')')
					.css('padding', 0)
					.css('position', 'relative')
					.on('mouseenter, mouseleave, mousemove, mousedown', this.rowEvent, {scope: this, data: {rowIndex: i}});

				this.renderFreeBusy(i);
			}
		},

		rowEvent: function(evt) {
			if(evt && evt.target) {
				if(evt.type == 'mousedown') {
					this.startDrag(evt.data.rowIndex, this.getEventTimeslot(evt));
					evt.stopEvent();
				}
				else if(isset(this.selStart)) {
					if(evt.type == 'mousemove') {
						this.selEnd = this.getEventTimeslot(evt);
						this.highlightRange();
					}
					else if(evt.type == 'mouseleave') {
						this.selOuter = true;
						this.highlightRange();
					}
					else if(evt.type == 'mouseenter') {
						this.selOuter = false;
						this.highlightRange();
					}
				}
			}
		},

		getEventTimeslot: function(evt) {
			var pos = evt.position(),
				bounds = Wrap(evt.target).closest('.scheduler-col-main').bounds();
				xoffs = pos.x - bounds.x,
				slot = Math.floor(xoffs * this.segments / this.hour_size);
			return slot;
		},

		startDrag: function(rowIndex, slotIndex) {
			this.selRow = rowIndex;
			this.selStart = slotIndex;
			this.selEnd = slotIndex;
			this.selOuter = false;
			this.highlightRange();
			this.drag_events = UI.addEventListeners(document,
				{mouseup: this.dragEvent,
				 keydown: this.dragEvent}, this);
		},

		dragEvent: function(evt) {
			if(evt.type == 'mouseup' || (evt.type == 'keydown' && evt.keyCode == 27)) {
				UI.removeEventListeners(document, this.drag_events);
				if(evt.type == 'keydown') {
					evt.stopEvent();
					this.cancelDrag();
				} else {
					this.completeDrag();
				}
			}
		},

		cancelDrag: function() {
			this.selStart = null;
			this.highlightRange();
		},

		completeDrag: function() {
			if(this.selOuter)
				return this.cancelDrag();
			var dur_slots = Math.abs(this.selStart - this.selEnd) + 1;
			var duration = dur_slots * 60 / this.segments;
			this.duration_input.setValue(duration, true);
			var date = this.timeslots[Math.min(this.selStart, this.selEnd)].date_obj;
			this.selStart = null;
			this.highlightRange();
			this.date_start_input.setValue(date);
		},

		highlightRange: function() {
			if(! isset(this.selStart)) {
				if(this.dragElt) {
					this.dragElt.remove();
					this.dragElt = null;
				}
				return;
			}
			var start = Math.min(this.selStart, this.selEnd),
				width = Math.abs(this.selStart - this.selEnd) + 1,
				outer = this.getDisplayRow(this.selRow || 0).childNodes[1];

			if(! this.dragElt) {
				this.dragElt = MakeElt('div.panel-outer.panel-border.panel-final')
					.css('position', 'absolute')
					.css('box-sizing', 'border-box');
			}

			this.dragElt.appendTo(outer)
				.position({left: start * this.hour_size / this.segments, top: this.pad_inner})
				.cssSize('width', width * this.hour_size / this.segments)
				.cssSize('bottom', this.pad_inner)
				.css('opacity', this.selOuter ? 0.5 : 1.0);
		},

		getDisplayRow: function(index) {
			var body = Wrap(this.elt).findFirst('.scheduler-body').get(0);
			return body.childNodes[index];
		},

		renderFreeBusy: function(index) {
			var row = this.getDisplayRow(index),
				attend = this.attends[index],
				states = [], prevst, prevfrom, st,
				in_current, in_orig, busy, cell, cls, blk;
			if(! row || ! attend) return;
			cell = Wrap(row.childNodes[1]);
			if(! cell) return;
			cell.empty();

			for(var i=0; i < this.timeslots.length; i++) {
				if(this.timeslots[i].is_start) {
					in_current = true;
				}
				if(this.timeslots[i].is_end) {
					in_current = false;
				}
				if(this.timeslots[i].is_orig_start) {
					in_orig = true;
				}
				if(this.timeslots[i].is_orig_end) {
					in_orig = false;
				}

				if(attend.busy) {
					busy = attend.busy[this.timeslots[i].hash];
					if(in_orig && this.orig_attends_hash[attend.id]) {
						if(busy && ! in_current) busy --;
						else if(busy == 1) busy = 0;
					}
					else if(in_current && busy)
						busy ++;
				} else
					busy = 0;

				if(busy > 1)
					st = 'panel-danger';
				else if(busy)
					st = 'panel-alert';
				else if(in_current)
					st = 'panel-info';
				else
					st = '';
				states.push(st);
			}

			prevst = '';
			for(var i = 0; i <= states.length; i++) {
				st = (i == states.length) ? '' : states[i];
				if(st != prevst) {
					if(prevst) {
						blk = MakeElt('div.panel-outer.panel-border.'+prevst)
							.css('position', 'absolute')
							.css('box-sizing', 'border-box')
							.css('opacity', 0.5)
							.appendTo(cell)
							.position({left: prevfrom * this.hour_size / this.segments, top: this.pad_inner})
							.cssSize('width', (i - prevfrom) * this.hour_size / this.segments)
							.cssSize('bottom', this.pad_inner);
					}
					prevst = st;
					prevfrom = i;
				}
			}

			var p = UI.PopupManager.currentModal();
			if(p) p.reposition();
		},

		clearFreeBusy: function(full) {
			for(var i = 0; i < this.attends.length; i++) {
				if(full) this.attends[i].busy_result = null;
				this.loadFreeBusySlots(i);
			}
		},

		loadFreeBusy: function() {
			if(this.basic) return;
			for(var i = 0; i < this.attends.length; i++) {
				if(! this.attends[i].busy)
					this.loadFreeBusyRow(i);
			}
		},

		loadFreeBusyRow: function(index) {
			var row = this.attends[index], params = {}, self = this;
			if(! row || ! row.module || ! row.id) return;
			var query_params = {};
			if (row.module == 'Resources') {
				query_params['resource_id'] = row.id;
			} else {
				query_params['user_id'] = row.id;
			}
			var req = new SUGAR.conn.JSONRequest('get_freebusy_lines', {status_msg: ''}, query_params);
			req.passthru = {module: row.module, id: row.id};
			req.fetch(function() { self.returnFreeBusy(this.passthru.module, this.passthru.id, this.getResult().result); });
		},

		returnFreeBusy: function(module, id, result) {
			var i, j, row, rowIndex, ts;
			for(i = 0; i < this.attends.length; i++) {
				if(this.attends[i].module == module && this.attends[i].id == id) {
					row = this.attends[i];
					rowIndex = i;
				}
			}
			if(! row || ! result) return;
			row.busy_result = result;
			this.loadFreeBusySlots(rowIndex);
			this.renderFreeBusy(rowIndex);
		},

		loadFreeBusySlots: function(rowIndex) {
			var row = this.attends[rowIndex], result, starts = [], ends = [];
			if(! row || ! (result = row.busy_result)) return;
			row.busy = {};
			for(i = 0; i < result.length; i++) {
				starts.push(Math.floor((new Date(result[i][0])).getTime() / 1000));
				ends.push(Math.floor((new Date(result[i][1])).getTime() / 1000));
			}
			for(i = 0; i < this.timeslots.length; i++) {
				ts = this.timeslots[i].ts;
				for(j = 0; j < starts.length; j++) {
					if(starts[j] <= ts && ends[j] > ts) {
						row.busy[this.timeslots[i].hash] = (row.busy[this.timeslots[i].hash] || 0) + 1;
					}
				}
			}
		},

		showSearcher: function(evt) {
			var tgt = evt.currentTarget,
				mod = Wrap(tgt).attr('data-module'),
				filters = {},
				parent_inp = UI.getFormInput(this.form, 'account'),
				self = this;
			if(! mod) return;
			evt.stopEvent();
			if(parent_inp && parent_inp.getKey()) {
				filters.primary_account_id = parent_inp.getKey();
				filters.primary_account = parent_inp.getValue();
			}

			var searcher = new App.ui.SearchSelect('app-search',
				{target: tgt, target_offset: {top: -1},
				 target_active_class: 'active opened',
				 popup_callback: 'scheduleReturnRows',
				 passthru: {field_id: this.id},
				 form: this.form,
				 filters: filters,
				 destroy_on_close: true,
				 module: mod,
				 icon_key: 'icon',
				 show_checks: true,
				 search_clear: false,
				 allow_popup: true,
				 search_blank: true,
				 matching_only: true});
			searcher.onshow = function() {
				searcher.focus();
			}
			searcher.onchange = function(k, v, mod) {
				if(! v) return;
				if(v._module) mod = v._module;
				self.addRow(v, mod);
			}
            searcher.setup();
			searcher.render();
			searcher.showDefault();
		},

		beforeSubmitForm: function() {
			if(! this.form || this.basic) return;
			var row, i, invites = {};
			for(var i=0; i < this.attends.length; i++) {
				row = this.attends[i];
				invites[row.module] = get_default(invites[row.module], '') + row.id + ',';
			}
			if(! this.form.user_invitees)
				MakeElt('input', {type: 'hidden', name: 'user_invitees'}).appendTo(this.form);
			this.form.user_invitees.value = invites['Users'] || '';
			if(! this.form.contact_invitees)
				MakeElt('input', {type: 'hidden', name: 'contact_invitees'}).appendTo(this.form);
			this.form.contact_invitees.value = invites['Contacts'] || '';
            if(! this.form.lead_invitees)
                MakeElt('input', {type: 'hidden', name: 'lead_invitees'}).appendTo(this.form);
            this.form.lead_invitees.value = invites['Leads'] || '';
			if(! this.form.resource_invitees)
				MakeElt('input', {type: 'hidden', name: 'resource_invitees'}).appendTo(this.form);
			this.form.resource_invitees.value = invites['Resources'] || '';
		}
	};


	window.scheduleReturnRows = function(data) {
		if(data) {
			var sched = SUGAR.ui.getFormInput(data.form_name, data.passthru_data.field_id),
				row;
			if(! sched) return;
			if(data.selection_count) {
				for(var i = 0; i < data.selection_count; i++) {
					row = deep_clone(data.selection_list['ID_' + i]);
					row.module = data.module;
					sched.addRow(row);
				}
			} else {
				row = deep_clone(data.name_to_value_array);
				row.module = data.module;
				sched.addRow(row);
			}
		}
	}


});

