<?php

// note: duplicates functionality from vCals module
function create_meeting_ical($record_id, $model='Meeting', $check_user_acl=false, $for_user=false) {
	$params = array();
	if($check_user_acl)
		$params['acl_filters'] = 'view';
	$row = ListQuery::quick_fetch($model, $record_id, true, $params);
	if($row)
		return create_result_ical($row, $for_user);
}

function create_result_ical($row, $for_user=false) {
	$model_name = $row->getModelName();
	$date_entered = $row->getField('date_entered');
	$start_time = $row->getField('date_start');
	$duration = $row->getField('duration');
	$end_time = $start_time;
	if(! $start_time) return;
	if(! $duration) $duration = 1;

	$start_ts = strtotime("$start_time UTC");
	if(! $start_ts) return;
	$start_ic = gmdate("Ymd\\THi00\\Z", $start_ts);
	$end_ts = strtotime("$start_time UTC + $duration minutes");
	$end_ic = gmdate("Ymd\\THi00\\Z", $end_ts);
	if($date_entered) {
		$enter_ts = strtotime("$date_entered UTC");
		$enter_ic = gmdate("Ymd\\THis\\Z", $enter_ts);
	} else {
		$enter_ic = $start_ic;
	}

	$user_id = $row->getField('assigned_user_id');
	$organizer = null;
	if($user_id) {
		$organizer = get_user_info($user_id);
	}

	$str = "BEGIN:VEVENT\n";
	if($organizer) {
		$full_name = ListQuery::format_person_name($organizer['first_name'], $organizer['last_name'], $organizer['salutation']);
		$str .= "ORGANIZER;CN=" . ical_escape_value($full_name, false) . ":MAILTO:" . ical_escape_value($organizer['email1'], false) . "\n";
	}
	$str .= "SUMMARY:".ical_escape_value($row->getField('name'), false). "\n";
	$str .= "DTSTAMP:$enter_ic\n";
	$str .= "DTSTART:$start_ic\n";
	$str .= "DTEND:$end_ic\n";
	$str .= "DESCRIPTION:" . ical_escape_value($row->getField('description'), false) . "\n";

	if(json_decode($row->getField('online_meeting_params'))->join_url)
		$str .= "URL;VALUE=URI:" . json_decode($row->getField('online_meeting_params'))->join_url. "\n";
	else if($row->getField('gtm_id') != '') {
		$str .= "URL;VALUE=URI:https://www.gotomeeting.com/join/" . $row->getField('gtm_id'). "\n";
	}
	else if($for_user)
		$str .= "URL;VALUE=URI:" . AppConfig::site_url().
			"index.php?module=".$row->getModuleDir()."&action=DetailView&record=". $row->getPrimaryKeyValue(). "\n";

	$str .= "UID:" . $row->getPrimaryKeyValue() . "\n";

	if($model_name == 'Meeting' && ($loc = $row->getField('location')))
		$str .= "LOCATION:" . ical_escape_value($loc, false) . "\n";
	if($model_name == 'Call' || $model_name == 'Meeting') {
		$users = $row->getLinkResults('users', array('id', 'full_name', 'email1', 'accept_status' => '~join.accept_status'));
		$contacts = $row->getLinkResults('contacts', array('id', 'full_name' => 'name', 'email1', 'accept_status' => '~join.accept_status'));
		$attend = array();
		if($users)
			array_extend($attend, $users->getRows());
		if($contacts)
			array_extend($attend, $contacts->getRows());
		foreach($attend as $attendee) {
			if(! strlen($attendee['full_name'])) continue;
			$str .= 'ATTENDEE;CN='.ical_escape_value($attendee['full_name'], false);
			$stat = '';
			if($attendee['accept_status'] == 'accept')
				$stat = 'ACCEPTED';
			else if($attendee['accept_status'] == 'tentative')
				$stat = 'TENTATIVE';
			else if($attendee['accept_status'] == 'decline')
				$stat = 'DECLINED';
			else
				$stat = 'NEEDS-ACTION;RSVP=TRUE';
			if($stat)
				$str .= ';PARTSTAT='.$stat;
			$email = $attendee['email1'];
			//if(strlen($email))
				$str .= ':MAILTO:'. ical_escape_value($email, false);
			$str .= "\n";
		}
	}

	$remind = $row->getField('reminder_time');
	if($remind > 0) {
		$str .= "BEGIN:VALARM\n";
		$str .= "TRIGGER:-PT" . $remind/60 . "M\n";
		$str .= "ACTION:DISPLAY\n";
		$str .= "DESCRIPTION:" . ical_escape_value($row->getField('name'), false) . "\n";
		$str .= "END:VALARM\n";
	}
	$str .= "END:VEVENT\n";
	return array('title' => $row->getField('name'), 'body' => $str);
}

function wrap_ical_result($str) {
	$ret =
		  "BEGIN:VCALENDAR\n"
		. "VERSION:2.0\n"
		. "METHOD:PUBLISH\n"
		//. "X-WR-CALNAME:$name (1CRM)\n"
		. "PRODID:-//1CRM//1CRM Calendar//EN\n"
		. "X-WR-TIMEZONE:".AppConfig::system_timezone()."\n"
		. "CALSCALE:GREGORIAN\n"
		. $str
		. "END:VCALENDAR\n";
	// ical must use CRLF
	$str = preg_replace('~\R~u', "\r\n", $str);
	return $ret;
}

function user_return_ical($ret, $title=null) {
	if(! $title) $title = 'calendar';
	end_clean_all_buffers();
	header('Content-Type: text/calendar');
	header('Content-Disposition: inline; filename=' . filename_safe_string($title) . '.ics');
	echo $ret;
	sugar_cleanup(true);
}

