<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
require_once('include/layout/forms/FormField.php');
require_once('include/OnlineMeetings/OnlineMeetingsFactory.php');

class MSTeamsAuthInput extends FormField {

	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		$html = '';
		$lstyle = $gen->getLayout()->getType();
		$for_company = $this->name == 'company_ms_teams_auth_token';
		$token = null;

		try {
			$onlineMeeting = OnlineMeetingsFactory::getInstance('ms_teams');
			$user_id = null;

			if (! $for_company)
				$user_id = $row_result->getField('id');

			$token = $onlineMeeting->loadStoredAccessToken($for_company, $user_id);
		} catch (OnlineMeetingException $e) {
			$onlineMeeting = null;
		}

		$error_display = $success_display = 'none';
		$error_message = '';

		if ($for_company) {
			$success_message = $this->addAppNameToLangString('LBL_ONLINE_MEETING_AUTH_MESSAGE');
		} else {
			$success_message = $this->addAppNameToLangString('LBL_ONLINE_MEETING_PERSONAL_AUTH_MESSAGE');
		}

		if (! $token) {
			$message_const = 'LBL_ONLINE_MEETING_AUTH_ERROR_MESSAGE';

			//If user token is empty but system authorised
			if (! $for_company && $onlineMeeting) {
				$system_token = $onlineMeeting->loadStoredAccessToken(true);

				if ($system_token)
					$message_const = 'LBL_ONLINE_MEETING_PERSONAL_SYSTEM_AUTH_MESSAGE';
			}

			$error_message = $this->addAppNameToLangString($message_const) ."<br>". $this->addAppNameToLangString('LBL_ONLINE_MEETING_AUTH_SSL');
			$error_display = 'block';
		} else {
			if ($token['refresh_before'] < time()) {
				$error_message = $this->addAppNameToLangString('LBL_ONLINE_MEETING_AUTH_EXPIRED');
				$error_display = 'block';
			} else {
				$success_display = 'block';
			}
		}

		$html .= '<div id="om_error_message" class="text-draft" style="display:' . $error_display . '">' . $error_message. '</div>';
		$html .= '<div id="om_success_message" class="text-success" style="display:' . $success_display . '">' . $success_message. '</div>';

		if($lstyle == 'editview' || ($lstyle == 'record' && $gen->getLayout()->getDefaultEditable())) {

			if ($for_company || (! $for_company && $row_result->getField('id') == AppConfig::current_user_id())) {

				$html .= '<button type="button" onclick="return msTeamsAuth(this.form, ' . (int) $for_company
					. ');" class="input-button" style="" tabindex=""  icon=""><span class="input-label">'
					. translate('LBL_BACKUP_DROPBOX_AUTHORIZE', 'Configurator') . '</span></button>';

			}

		}

		return $html;
	}

	function addAppNameToLangString($label) {
		$str = translate($label, 'Configurator');
		$str = str_replace('%MEETING_APP%', 'Microsoft Teams', $str);

		return $str;
	}
}
