<?php

require_once 'include/layout/forms/FormField.php';
require_once('include/OnlineMeetings/OnlineMeetingsFactory.php');

class OnlineMeetingUrlWidget extends FormField {

	/**
	 * @var OnlineMeeting
	 */
	private $onlineMeeting;

	function init($params=null, $model=null) {
		parent::init($params, $model);

		try {
			$this->onlineMeeting = OnlineMeetingsFactory::getInstance();
		} catch (OnlineMeetingException $e) {}
	}

	function getLabel($context=null) {
		if ($this->onlineMeeting) {
			return translate($this->onlineMeeting->getJoinUrlLabel(), 'Meetings');
		} else {
			return translate('LBL_ONLINE_MEETING_JOIN_URL', 'Meetings');
		}
	}

	function getRequiredFields() {
		return ['gtm_id', 'online_meeting_params', 'online_meeting_join_url'];
	}

	function getRawValue($row_result) {
		if($this->type == 'static' || ! $row_result)
			return '';

		return $row_result->getField('online_meeting_join_url');
	}

	function getFormattedValue($row_result) {
		if($this->type == 'static' || ! $row_result)
			return $this->value;

		$url = $this->getJoinUrl(
			$row_result->getField('gtm_id'),
			$row_result->getField('online_meeting_params')
		);

		if ($url == '')
			return '';

		return $row_result->getField('online_meeting_join_url', $url, true);
	}

	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if (! $this->onlineMeeting)
			return '';

		if($gen->getLayout()->getType() == 'editview')
			return '';

		return $this->getFormattedValue($row_result);
	}

	private function getJoinUrl($gtm_id, $online_meeting_params) {
		if (! $this->onlineMeeting)
			return '';

		if ($this->onlineMeeting->getAppName() == 'gtm') {
			return $this->onlineMeeting->getJoinUrl(['gtm_id' => $gtm_id]);
		} else {
			$params = json_decode($online_meeting_params);

			if (is_object($params))
				$params = (array)$params;

			if (! is_array($params))
				$params = [];

			return $this->onlineMeeting->getJoinUrl($params);
		}
	}
}
