<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormTableSection.php');
require_once('modules/Meetings/ActivityInvite.php');

class SchedulerWidget extends FormTableSection {

	var $model_name;
	var $dialog;
	var $basic;
	var $hours;

	function init($params=null, $model=null) {
		parent::init($params, $model);
		if(! $this->id)
			$this->id = 'event_scheduler';
		if($model) $this->model_name = $model->name;
		if(is_array($params) && isset($params['dialog']))
			$this->dialog = $params['dialog'];
		if(is_array($params) && isset($params['basic']))
			$this->basic = $params['basic'];
		if(is_array($params) && isset($params['hours']))
			$this->hours = $params['hours'];
	}
	
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if($gen->getLayout()->getEditingLayout()) {
			$params = array();
			return $this->renderOuterTable($gen, $parents, $context, $params);		
		}

		if($gen->getLayout()->getDefaultEditable()) {
			return $this->renderHtmlEdit($gen, $row_result, $parents, $context);
		}
		return $this->renderHtmlView($gen, $row_result);
	}
	
	function getRequiredFields() {
		return array();
	}
	
	function beforeRender(FormGenerator $gen) {
		if(! $gen->getLayout()->getDefaultEditable())
			$this->hidden = true;
	}
	
	function renderHtmlView(HtmlFormGenerator $gen, RowResult $row_result) {
        return "";
	}

	function renderHtmlEdit(HtmlFormGenerator $gen, RowResult $row_result, array $parents, array $context) {
        global $mod_strings, $themeInstance;
        $dialog = $this->dialog;

        $form_name = $gen->getFormName();
		$hidden = '';

		if ($this->model_name == 'Meeting' || $this->model_name == 'Call') {
			$add_invitee_label = translate('LBL_ADD_INVITEE_BUTTON', 'Meetings');
			$add_invitee = <<<EOQ
				<div class="inline-elt"><button name="addInvitee" type="button" class="input-button rbullet rbullet-ddown scheduler-add" data-module="Invitees"><span class="input-label"><span class="uii uii-add"></span> {$add_invitee_label}</span></button></div>
EOQ;
		} else
			$add_invitee = '';
		
		if ($this->model_name == 'Meeting') {
            $add_resource = <<<EOQ
            	<div class="inline-elt"><button name="addResource" type="button" class="input-button rbullet rbullet-ddown scheduler-add" data-module="Resources"><span class="input-label"><span class="uii uii-add"></span> {$mod_strings['LBL_ADD_RESOURCE_BUTTON']}</span></button></div>
EOQ;
		} else {
			$add_resource = '';
        }

		$send_check_cell = '';
		if($this->model_name == 'Meeting' || $this->model_name == 'Call') {
			$send_check_spec = [
				'name' => 'send_invites',
				'vname' => 'LBL_SEND_INVITATIONS',
				'vname_module' => 'Meetings',
				'show_label' => true];
			$send_default = AppConfig::setting('notify.send_invites_by_default');
			$send_checked = array_get_default($_REQUEST, 'send_invites', $send_default && ! $row_result->getField('id'));
			$send_check = $gen->form_obj->renderCheck($send_check_spec, $send_checked);
			$send_check_cell = '<div class="inline-elt">' . $send_check . '</div>';
			$hidden .= '<input type="hidden" name="user_invitees" value="" />';
            $hidden .= '<input type="hidden" name="contact_invitees" value="" />';
			$hidden .= '<input type="hidden" name="lead_invitees" value="" />';
            $hidden .= '<input type="hidden" name="resource_invitees" value="" />';
		}

		$om_create_cell = '';

		if($this->model_name == 'Meeting') {

			require_once('include/OnlineMeetings/OnlineMeetingsFactory.php');

			try {
				$onlineMeeting = OnlineMeetingsFactory::getInstance();

				$om_create_spec = [
					'name' => 'online_meeting_create',
					'vname' => $onlineMeeting->getCreateLabel(),
					'vname_module' => 'Meetings',
					'show_label' => true
				];

				$om_create_checked = $row_result->getField('online_meeting_create');
				$om_create = $gen->form_obj->renderCheck($om_create_spec, $om_create_checked);
				$om_create_cell = '<div class="inline-elt">' . $om_create . '</div>';

			} catch (OnlineMeetingException $e) {}
		}
		
		$body = <<<EOQ
			<div id="{$this->id}-main" class="scheduler-main">
				<div class="row form-row col-unstack" id="{$this->id}-nav">
					<div class="column form-cell col-6">
						<div class="form-entry">
						<div class="buttons-inline">
							{$add_invitee}{$add_resource}
						</div>
						</div>
					</div>
					<div class="column form-cell col-6">
						<div class="form-entry">
						<div class="buttons-inline">
							{$send_check_cell}{$om_create_cell}
						</div>
						</div>
					</div>
				</div>
				<div class="row form-row">
					<div class="column form-cell expand"><div class="form-entry">
						<section class="card-outer panel-outer panel-default scheduler-outer panel-border panel-round">
							<div class="card-header panel-subheader">
								<div class="row col-space-sm">
									<div class="column scheduler-row-head col-3 scheduler-col-user"><h5 class="no-margin scheduler-date"></h5></div>
									<div class="column scheduler-row-head scheduler-col-main scheduler-times">time</div>
								</div>
							</div>
							<div class="card-body panel-body panel-pad-body scheduler-body">
							</div>
						</section>
					</div></div>
				</div>
			</div>
EOQ;


        $activityInvite = new ActivityInvite($this->model_name, $_REQUEST);
        $invitees = $activityInvite->getList($row_result);
        $params = ['dialog' => (int)$dialog, 'attends' => $invitees];
        if(isset($this->basic))
        	$params['basic'] = $this->basic;
        if(isset($this->hours))
        	$params['hours'] = $this->hours;

        $layout = $gen->getLayout();
        $layout->addScriptInclude('modules/Meetings/scheduler.js');
        $gen->getFormObject()->addControl('ScheduleViewer', $this->id, $params);
        
        $params = ['body' => $body];
        //if($dialog) return $body;
        if($dialog) $params['title'] = '';

		return $this->renderOuterTable($gen, $parents, $context, $params);
	}

	function loadUpdateRequest(RowUpdate $update, array $input) {
        $upd_invites = array();
        $this->addInviteesIdsFromInput($input, $upd_invites);
        $update->setRelatedData('event_scheduler_rows', $upd_invites);
        $update->setRelatedData('send_invites', array_get_default($input, 'send_invites'));
    }


    /**
     * Add invitee participants IDs to array
     *
     * @param array $input
     * @param array $invitees
     */
    function addInviteesIdsFromInput($input, &$invitees) {
        $names = array('user', 'contact', 'lead', 'resource');

        for ($i = 0; $i < sizeof($names); $i++) {
            if(! empty($input[$names[$i] . '_invitees'])) {
                $invitees[$names[$i] . '_invitees'] = $input[$names[$i] . '_invitees'];
            }
        }
    }

}
?>
