<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point'); 
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

require_once 'include/Sugar_Smarty.php';
require_once 'include/layout/forms/EditableForm.php';

$frm = new EditableForm('editview', 'ModuleDesignerFiltersForm');

$tpl = new Sugar_Smarty;

$module_name = array_get_default($_REQUEST, 'mod_name', '');

$redirect = array(
		'perform', 
		array(
			'module' => 'ModuleDesigner',
			'action' => 'index',
			'layout' => '', 
		),
	);

if (!preg_match('/^[a-z][a-z_0-9]+$/i', $module_name))
	return $redirect;

if (!AppConfig::setting("modinfo.primary_beans.$module_name"))
	return $redirect;
if (!AppConfig::setting("modinfo.by_name.$module_name.detail.module_designer", true))
	return $redirect;

$modelName = AppConfig::setting("modinfo.primary_beans.$module_name");
$fields = AppConfig::setting("model.fields.$modelName");

$popup_filters = array();
$basic = AppConfig::setting("display.popup_filters.$modelName");
if (!isset($basic))
	$basic = AppConfig::setting("display.basic_filters.$modelName");
if (!isset($basic))
	$basic = array();

foreach ($basic as $k => $v) {
	if (is_array($v))
		$popup_filters[$k] = true;
	elseif (is_string($v))
		$popup_filters[$v] = true;
}

$details = AppConfig::setting("views.detail.{$module_name}.search");
if (isset($details['Standard']))  {
	$cfg = AppConfig::setting("views.layout.{$module_name}.search.Standard");
} else {
	$cfg = array('elements' => array());
}

$enabled_types = array(
	'varchar', 'ref', 'enum', 'multienum', 'name', 'number', 'date', 
	'time', 'datetime', 'bool', 'int', 'float', 'currency', 
	'base_currency', 'raw_currency', 'double', 'percentage', 
	'rate_percent', 'phone', 'extension_number', 'email',
);

$available_fields = array();

foreach ($fields as $f) {
	if (!in_array($f['type'], $enabled_types))
		continue;
	$label = isset($f['vname']) ? translate($f['vname'], array_get_default($f, 'vname_module', $module_name)) : '';
	if (empty($label))
		$label = $f['name'];
	$available_fields[] = array(
		'name' => $f['name'],
		'label' => $label,
		'visible' => in_array($f['name'], $cfg['elements']),
		'popup' => isset($popup_filters[$f['name']]),
	);
}


if (!empty($_POST['_save'])) {
	$json = getJSONobj();
	$all_filters = $json->decode($_POST['filters']);

	$filtered = array_values(array_filter(array_map(function($e) {return $e['visible'] ? $e['name'] : null;}, $all_filters)));
	$popup = array_values(array_filter(array_map(function($e) {return $e['popup'] ? $e['name'] : null;}, $all_filters)));
	$popup_filters = array();
	foreach ($basic as $k => $v) {
		if (is_array($v))
			$fname = $k;
		else
			$fname = $v;
		if (!isset($fields[$fname]))
			$popup_filters[$k] = $v;
	}
	foreach ($popup as $v)
		$popup_filters[] = $v;

	$search_def = array(
		'detail' => array('type' => 'search'),
		'layout' => array('elements' => $filtered),
	);

	$display = array('popup_filters' => $popup_filters);

	require_once 'include/config/format/ConfigWriter.php';
	$cw = new ConfigWriter;

	$filename =  AppConfig::custom_dir() . "/modules/$module_name/new_views/search.Standard.php";
	$cw->writeFile($filename, $search_def);
	
	$filename =  AppConfig::custom_dir() . "/modules/$module_name/display/display.$modelName.php";
	$cw->writeFile($filename, $display);

	if (AppConfig::setting("modinfo.by_name.$module_name.detail.created_by_module_designer")) {
		$filename =  "modules/$module_name/display/display.$modelName.php";
		if (!file_exists($filename)) {
			@mkdir(dirname($filename));
			@fclose(fopen($filename, 'w'));
		}
	}

	AppConfig::invalidate_cache('views');
	AppConfig::invalidate_cache('display');
	unset($_POST['_save']);
	return array(
		'perform', 
		array(
			'module' => 'ModuleDesigner',
			'action' => 'Filters',
			'mod_name' => $module_name,
			'layout' => '', 
		),
	);
}

$result = new RowResult();
$result->module_dirs = ['ModuleDesigner'];


require_once('include/layout/forms/FormGenerator.php');

$spec = array(
	'name' => 'filters',
	'vname' => '',
	'widget' => 'DynamicListInput',
	'show_handles' => false,
	'show_delete' => false,
	'max_depth' => 0,
	'width' => '100%',
	'colspan' => 2,
	'cols' => array(
		array(
			'name' => 'label',
			'label' => 'LBL_FILTER_FIELD_NAME',
			'width' => '60%',
		),
		array(
			'name' => 'visible',
			'format' => 'check',
			'label' => 'LBL_LIST_FILTER',
			'width' => '20%',
		),
		array(
			'name' => 'popup',
			'format' => 'check',
			'label' => 'LBL_POPUP_FILTER',
			'width' => '20%',
		),
	),
);
$layout = array(
	'type' => 'configform',
	'editable' => true,
	'form_buttons' => array(
		'save' => array(
			'icon' => 'icon-accept',
			'vname' => 'LBL_SAVE_BUTTON_LABEL',
		),
		'cancel' => array(
			'icon' => 'icon-cancel',
			'vname' => 'LBL_CANCEL_BUTTON_LABEL',
			'perform' => "this.form.action='index.php';this.form.action.value='index';",
		),
	),
	'sections' => array(
		array(
			'id' => 'main',
			'label_position' => 'top',
			'elements' => array(
				$spec,
			),
		),
	),
);
$result = new RowResult();
$result->fields = [
	'filters' => [
		'type' => 'varchar',
	],
];
$result->row = [];
foreach($available_fields as $f) {
	$result->row['filters'][] = [
		'name' => $f['name'],
		'label' => $f['label'],
		'visible' => $f['visible'],
		'popup' => $f['popup'],
	];
}
array_csort($result->row['filters'], 'name');

$title = $mod_strings['LBL_EDIT_FILTERS'];
$icon = get_module_header_icon($module_name, '');
$title .= ' &laquo; ' . $icon . ' ' . translate('LBL_MODULE_TITLE', $module_name);
echo get_module_title('ModuleDesigner', $title, true); 

$gen = FormGenerator::html_form(null, $layout, 'ModuleDesignerFiltersForm');
$gen->getFormObject()->addHiddenFields(
	[ 'module' => 'ModuleDesigner', 'action' => 'Filters', 'mod_name' => $module_name, '_save' => 1 ]
);
$gen->renderForm($result);
echo '<div class="detailOuter">' . $gen->getResult() . '</div>';
$gen->exportIncludes();



