<?php

class ModuleDesignerFields
{
	public static $settable = [
		'name', 'audited', 'massupdate',
	 	'unified_search',	
		'duplicate_merge', 'required', 
		'mobile_app', 'personal_info', 
		'privacy_consent', 'default'];

	public static $dataTypes = array(
		'varchar', 'text', 'int', 'double', 'bool', 'email',
		'enum', 'multienum', 'date', 'url', 'html', 'calculated',
		'ref', 'related_value', 'currency', 'name',
		'status', 'item_number',
		'module_name',
	);

	var $module_name;
	var $model_name;

	public static function data_type_options() {
		$types =  array_filter(self::$dataTypes, function($t) {return $t != 'module_name';});
		$ret = [];
		foreach ($types as $type) {
			$ret[$type] = translate('LBL_TYPE_' . strtoupper($type));
		}
		return $ret;
	}

	public function __construct($module_name)
	{
		$this->module_name = $module_name;
		$this->model_name = AppConfig::setting("modinfo.by_name.$module_name.detail.primary_bean");
	}

	public function save($fields)
	{
		if (!empty($fields)) {
			$updateLang = array();
			require_once 'include/config/format/ConfigParser.php';
			require_once 'include/config/format/ConfigWriter.php';
			$orig_filename = "modules/{$this->module_name}/models/bean.{$this->model_name}.php";
			$filename = AppConfig::custom_dir() . "modules/{$this->module_name}/models/bean.{$this->model_name}.php";
			$defs = AppConfig::setting("model.fields.{$this->model_name}");
			$orig = ConfigParser::load_file($orig_filename);
			$content = ConfigParser::load_files([$orig_filename, $filename], ['no_cache' => true]);

			$createLinks = array();
			foreach ($fields as $fid => $field) {
				$isNew = strpos($fid, 'newfield~') === 0;
				if ($isNew && $field['type'] == 'ref') {
					$createLinks[] = $field;
				}
				$name = $field['name'];
				unset($field['name']);
				$def = array_get_default($defs, $name, array());
				if (!empty($def['inferred'])) continue;
				if (strlen(array_get_default($field, 'label', ''))) {
					$langKey = 'MB_LABEL_' . strtoupper($name);
					$field['vname'] = $langKey;
					$updateLang[$langKey] = $field['label'];
					unset($field['label']);
				}
				$configName = array_get_default($def, 'from_app_field', $name);
				$configValue = array_get_default($content['fields'], $configName, null);
				$isStandard = !is_null($configValue);
				if (!$isStandard)
					$configValue = array();
				foreach ($field as $k => $v) {
					if ( (isset($def[$k]) && $def[$k] !== $v) || !$isStandard || in_array($k, self::$settable) )
						$configValue[$k] = $v;
				}
				if (isset($configValue['type']) && $configValue['type'] == 'bool')
					$configValue['default'] = $configValue['default'] ? 1 : 0;
				if (!$isStandard) {
					unset($configValue['source']);
					$configValue['created_by_module_designer'] = true;
				}
				$content['fields'][$configName] = $configValue;
			}

			$cw = new ConfigWriter;
			$cw->writeDiff($filename, $content, $orig);
			AppConfig::invalidate_cache('model');
			$this->reloadModels();

			if (!empty($updateLang)) {
				foreach ($updateLang as $k => $v) {
					AppConfig::set_local("lang.strings.base.{$this->module_name}.$k", $v);
				}
				AppConfig::save_local('lang');
				AppConfig::cache_reset();
			}

			AppConfig::cache_reset();
			$parser = new ConfigParser;
			$cMgr = $parser->getCacheManager();
			$cMgr->clearCache($filename);
			AppConfig::invalidate_cache('model');

			$links_rels =  $this->createLinksAndRels($createLinks);
			$this->addLinks($links_rels['links']);
			$this->addRelationships($links_rels['relationships']);

			return array(
				'redirect', 
				array(
					'module' => 'ModuleDesigner',
					'action' => 'index',
					'layout' => '', 
				)
			);
		}
	}

	private function addLinks($links)
	{
		foreach ($links as $k => $link) {
			$filename = AppConfig::custom_dir() . "modules/{$link['module']}/models/bean.{$link['_model']}.php";
			unset($link['_model']);
			$content = ConfigParser::load_files([$filename], ['no_cache' => true]);
			$link['module'] = $this->module_name;
			$link['bean_name'] = $this->model_name;
			$content['links'][$k] = $link;

			$cw = new ConfigWriter;
			$cw->writeFile($filename, $content);
		}

		$this->reloadModels();
	}

	private function addRelationships($relationships)
	{
		foreach ($relationships as $k => $rel) {
			$filename = AppConfig::custom_dir() . "modules/{$this->module_name}/models/bean.{$this->model_name}.php";
			unset($rel['_model']);
			$content = ConfigParser::load_files([$filename], ['no_cache' => true]);
			$content['relationships'][$k] = $rel;

			$cw = new ConfigWriter;
			$cw->writeFile($filename, $content);
		}

		$this->reloadModels();

	}

	private function reloadModels()
	{
		require_once 'include/database/DBChecker.php';
		$checker = new DBChecker();
		$checker->reloadModels();

		try {
			$checker->checkRepairModel($this->model_name, DB_CHECK_STANDARD | DB_CHECK_AUDIT, true, false);
		} catch(Exception $e) {
			$GLOBALS['log']->fatal('Error repairing database: '.$e->getMessage());
			pr2($e->getMessage(), 'Error repairing database', true);
		}
	}

	private function createLinksAndRels($fields)
	{
		$rels = $links = array();
		foreach ($fields as $f) {
			$modules = array();
			if (empty($f['bean_name'])) {
				foreach ($GLOBALS['app_list_strings']['moduleListSingular'] as $k => $v) {
					$bean = AppConfig::module_primary_bean($k);
					if ($bean) {
						$modules[] = $k;
					}
				}
			} else {
				$modules[] = '';
			}
			foreach ($modules as $mod) {
				if ($mod == $this->module_name) continue;
				$rel = array();
				$rel_name = 'mb_rel_' . $this->module_name . '_' . $f['name'];
				$link_name = 'mb_link_' . $this->module_name . '_' . $f['name'];
				if ($mod) {
					$rel_name .= '_' . $mod;
					$link_name .= '_' . $mod;
				}
				$rel['key'] = $f['name'] . '_id';
				$rel['target_key'] = 'id';
				if ($mod) {
					$rel['role_column'] = $f['name'] . '_mb_module_';
					$rel['role_value'] = $mod;
					$rel['target_bean'] = AppConfig::module_primary_bean($this->module_name);
					$rel['_model'] = AppConfig::module_primary_bean($mod);
				} else {
					$rel['target_bean'] = $f['bean_name'];
					$mod = AppConfig::module_for_model($f['bean_name']);
				}
				$rel['relationship_type'] = 'one-to-many';
				$rels[$rel_name] = $rel;

				AppConfig::set_local("lang.strings.base.{$mod}.".'MB_LINK_' . strtoupper($f['name']  . $mod), $this->module_name);
				AppConfig::save_local('lang');
				AppConfig::cache_reset();

				$link = array(
					'_model' => AppConfig::module_primary_bean($mod),
					'relationship' => $rel_name,
					'module' => $mod,
					'bean_name' =>  $rel['target_bean'],
					'vname' => 'MB_LINK_' . strtoupper($f['name']  . $mod),
				);
				$links[$link_name] = $link;
			}
		}
		return array(
			'links' => $links,
			'relationships' => $rels,
		);
	}
	
	public function getFieldListHtml() {
        $fields = $this->getModelFields();
        $html = '';

		$fieldsCount = is_countable($fields) ? sizeof($fields) : 0;
        for ($i = 0; $i < $fieldsCount; $i++) {
            $html .= '<div id="newfield_' .$fields[$i]. '" class="edit_layout_wrapper" onclick="ModuleDesignerFields.addToFormula(this);">$' .$fields[$i]. '</div>';
        }

        return $html;
    }

    public function getFunctionListHtml() {
        require_once 'include/layout/FormulaParser.php';
        $functions = FormulaParser::get_allowed_functions();
        $html = '';

        foreach ($functions as $func => $tooltip) {
            $html .= '<div id="newfunc_' .$func. '" class="edit_layout_wrapper" onclick="ModuleDesignerFields.addToFormula(this);" onmouseover="return SUGAR.popups.tooltip(\''.$tooltip.'\', this);">' .$func. '</div>';
        }

        return $html;
    }

    private function getModelFields() {
		$edit_model = new ModelDef(AppConfig::module_primary_bean($this->module_name));
		return $edit_model->getAllDisplayFields();
    }

}

