ModuleDesignerFields = new function()
{

	this.init = function(form, params)
	{
		this.form = form;
		for (var i in params) {
			this[i] = params[i];
		}
		this.nextId = 0;
	};

	this.render = function()
	{
		var table = $('fields_table');
		this.field_dialog = $('field_dialog');
		SUGAR.ui.unregisterInput(this.form, 'ext1');
		SUGAR.ui.unregisterInput(this.form, 'default');
		for (var fid in this.fields) {
			var field = this.fields[fid];
			if (field.inferred)
				continue;
			if (field.type == 'module_name')
				continue;
			var rid = 'row_' + fid;
			var row = $(rid);
			if (!row) {
				createElement2('div', {className: 'row form-row'},
					createElement2('div', {className: 'column form-cell sm-12', style: {height: '1px', backgroundColor: '#CCC' }}),
					table
				);
				row = createElement2('div', {id: rid, className: 'row form-row col-unstack', style: {cursor: 'pointer'}}, null, table);
				row.onclick = function() {
					ModuleDesignerFields.showForm(this.id.replace('row_', ''));
				}
				row.onmouseover = function() {
					SUGAR.ui.addClass(this, 'currentRow markRow');
				};
				row.onmouseout = function() {
					SUGAR.ui.removeClass(this, 'currentRow markRow');
				};
			}
			row.innerHTML = '';
			var values = [
				field.name,
				get_default(field.label, mod_string(field.vname, this.module)),
				app_list_strings('field_type_dom')[field.type]
			];
			values.forEach(function(value) {
				var p = createElement2('p', { className: 'form-value'}, value);
				var entry = createElement2('div', {className: 'form-entry'}, p);
				var col = createElement2('div', {className: 'column form-cell sm-4'}, entry);
				row.appendChild(col);
			});
		}
	
	};
	
	this.showForm = function(field_id)
	{
		var self = this;
		this.editingId = field_id;
		this.editingField = deep_clone(this.fields[field_id]);
		this.popup = new SUGAR.ui.Dialog('field_editor', {content_elt: this.field_dialog, width: '640px', destroy_on_close: false});
		this.popup.setTitle(mod_string('LBL_EDIT_DIALOG_TITLE', 'ModuleDesigner'));
		this.popup.render();
		this.popup.onshow = function() {
			self.resetErrors();
			self.renderForm();
			SUGAR.ui.setDisplayState(self.field_dialog, true);
			this.reposition();
		}
		this.popup.show();
	}
        
	this.checkDefaultValue = function() {
		switch (this.editingField.type) {
			case 'bool':
				this.editingField['default'] = this.editingField['default'] ? 1 : 0;
				break;
		}
	}

	this.renderForm = function()
	{
		table = $('custom_fields_body');
		table.innerHTML = '';
		this.checkDefaultValue();
		var isNew = this.isNewField(this.editingId);
		var typeInput = SUGAR.ui.getFormInput(this.form, 'type');
		typeInput.setValue(this.editingField.type, true);
		typeInput.setDisabled(!isNew);

		var nameInput = SUGAR.ui.getFormInput(this.form, 'name');
		nameInput.setValue(this.editingField.name, true);
		nameInput.setDisabled(!isNew);

		var labelInput = SUGAR.ui.getFormInput(this.form, 'label');
		labelInput.setValue(get_default(this.editingField.label, mod_string(this.editingField.vname, this.module)), true);

		var auditInput = SUGAR.ui.getFormInput(this.form, 'audited');
		auditInput.setValue(get_default(this.editingField.audited, false) ? true : false, true);

		var personalInput = SUGAR.ui.getFormInput(this.form, 'personal_info');
		personalInput.setValue(get_default(this.editingField.personal_info, false) ? true : false, true);

		var privacyInput = SUGAR.ui.getFormInput(this.form, 'privacy_consent');
		privacyInput.setValue(get_default(this.editingField.privacy_consent, false) ? true : false, true);

		var massInput = SUGAR.ui.getFormInput(this.form, 'massupdate');
		massInput.setValue(get_default(this.editingField.massupdate, false) ? true : false, true);

		var usearchInput = SUGAR.ui.getFormInput(this.form, 'unified_search');
		usearchInput.setValue(get_default(this.editingField.unified_search, false) ? true : false, true);

		var mobile = get_default(this.editingField.mobile_app, false);
		mobile = (mobile == 'enabled' || mobile == true || mobile == 1);
		var dup = get_default(this.editingField.duplicate_merge, true);
		dup = (dup == 'enabled' || dup == true || dup == 1);
		
		var dupInput = SUGAR.ui.getFormInput(this.form, 'duplicate_merge');
		dupInput.setValue(dup ? true : false, true);

		var mobileInput = SUGAR.ui.getFormInput(this.form, 'mobile_app');
		mobileInput.setValue(mobile ? true : false, true);

		this.displayElement('mobile_app_wrapper', isNew || this.editingField.created_by_module_designer);

		var reqInput = SUGAR.ui.getFormInput(this.form, 'required');
		reqInput.setValue(get_default(this.editingField.required, false) ? true : false, true);
		reqInput.setDisabled(this.editingField.module_designer == 'label_only');

		this.displayAudit(true);
		this.displayMassUpdate(true);
		this.displayFormula(false);
        this.displayRequired(true);

		switch (typeInput.getValue()) {
			case 'ref':
				this.renderModulesList(table, mod_string('LBL_RELATED_TYPE', 'ModuleDesigner'));
				break;
			case 'related_value':
				this.renderRefFields(table, mod_string('LBL_RELATED_VALUE', 'ModuleDesigner'));
				this.renderRefFieldFields(table, this.editingField.ref, SUGAR.ui.getFormInput(this.form, 'ext3'));
				break;
			case 'varchar':
			case 'item_number':
				this.renderDefaultText(table);
				if (!this.editingField.len) this.editingField.len = 50;
				this.renderSize(table);
				break;
			case 'text':
				this.renderDefaultTextArea(table);
				break;
			case 'int':
				this.renderDefaultText(table);
				if (!this.editingField.len) this.editingField.len = 11;
				if (this.editingField.len > 11) this.editingField.len = 11;
				this.renderSize(table);
				break;
			case 'double':
				this.renderDefaultText(table);
				break;
			case 'bool':
				this.renderDefaultBool(table);
				break;
			case 'email':
				this.renderDefaultText(table);
				break;
			case 'enum':
			case 'status':
				this.renderDropDownsList(table, mod_string('LBL_DROPDOWN_LIST', 'ModuleDesigner'));
				this.renderDefaultDropdown(table, SUGAR.ui.getFormInput(this.form, 'ext1').getValue());
				break;
			case 'multienum':
				this.renderDropDownsList(table, mod_string('LBL_MULTI_SELECT', 'ModuleDesigner'));
				this.renderDefaultDropdown(table, SUGAR.ui.getFormInput(this.form, 'ext1').getValue());
				break;
			case 'date':
				this.renderDefaultDate(table, mod_string('LBL_DEFAULT_VALUE', 'ModuleDesigner'));
				break;
			case 'url':
				this.renderDefaultText(table, mod_string('LBL_DEFAULT_URL', 'ModuleDesigner'));
				break;
			case 'html':
				this.renderDefaultTextArea(table, mod_string('LBL_HTML', 'ModuleDesigner'));
				break;
            case 'calculated':
                this.displayAudit(false);
                this.displayMassUpdate(false);
                this.displayRequired(false);
                this.displayFormula(true);
                this.renderFormulaInput(table, mod_string('LBL_FORMULA', 'ModuleDesigner'));
                break;
		}
		this.renderDependent(table);
	};

	this.renderDependent = function(table) {
		var elements = this.createRow(mod_string('LBL_DEPENDS_ON', 'ModuleDesigner'));
		table.appendChild(elements[0]);
		var cell = elements[2];

		var keys = [''];
		var values = [''];

		for (var fid in this.fields) {
			var f = this.fields[fid];
			if (f.name == this.editingField.name)
				continue;
			if (! (
				((f.type == 'enum' || f.type == 'status') && !f.multi_select_group) 
				|| (f.type == 'bool' )
				|| (f.type == 'multienum' )
			))
				continue;
			var vname_module;
			if (f.vname_module)
				vname_module = f.vname_module;
			else
				vname_module = this.module;
			keys.push(f['name']);
			values.push(mod_string(f.vname, vname_module));
		}
		
		var opts = {keys: keys, values: values, width: 300};
		
		var select =  new SUGAR.ui.SelectInput('', {
			width: 200,
			options: opts,
			init_value: this.editingField.depends_on
		});

		cell.appendChild(select.render());

		var depOptsElements = this.createRow('');
		table.appendChild(depOptsElements[0]);
		this.renderDependentOptions(depOptsElements[1], depOptsElements[2]);

		var condOptsElements = this.createRow('');
		table.appendChild(condOptsElements[0]);
		var t = createElement2('table', {'cellpadding': "10", 'width': '100%'});
		condOptsElements[2].appendChild(t);
		this.renderConditionalOptions(t);
		select.onchange = function() {
			ModuleDesignerFields.editingField.depends_on = this.getValue();
			ModuleDesignerFields.editingField.depends_on_value = '';
			ModuleDesignerFields.editingField.conditional_options = [];
			ModuleDesignerFields.renderDependentOptions(depOptsElements[1], depOptsElements[2]);
			ModuleDesignerFields.renderConditionalOptions(t);
		};
	};
    
	this.renderConditionalOptions = function(table) {

		while (table.rows.length) {
			table.deleteRow(0);
		}

		if ((this.editingField.type != 'enum' && this.editingField.type != 'status') || !this.editingField.depends_on) {
			return;
		}

		var fname = this.editingField.depends_on;
		var f = null;
		for (var i in this.fields) {
			if (this.fields[i].name == fname) {
				f = this.fields[i];
				break;
			}
		}
		if (!f || (f.type != 'enum' && f.type != 'status' && f.type != 'bool')) {
			return;
		}

		var row = table.insertRow(table.rows.length);
		var cell = this.renderLabel(row, mod_string('LBL_CONDITIONAL_OPTIONS', 'ModuleDesigner'));
		cell.colSpan = 3;
		cell.style.textAlign = 'center';
		cell.style.fontWeight = 'bold';

		var icon = createElement2('div',  {'className': 'input-icon icon-add active-icon'});
		icon.onclick = function() {
			ModuleDesignerFields.addConditionalOption();
			ModuleDesignerFields.renderConditionalOptions(table);
		}
		cell.appendChild(nbsp());
		cell.appendChild(icon);

		var f = null;
		for (var i in this.fields) {
			if (this.fields[i].name == fname) {
				f = this.fields[i];
				break;
			}
		}

		if (!this.editingField.conditional_options)
			this.editingField.conditional_options = [];
		for (var i = 0; i < this.editingField.conditional_options.length; i++) {
			var opt = this.editingField.conditional_options[i];
			var row = table.insertRow(table.rows.length);
			var cell = row.insertCell(0);
			var sel = this.renderDropdown(f.type == 'bool' ? 'custom_checkbox_dependent' : f.options, this.editingField.conditional_options[i][0], f.type != 'bool', f.type == 'bool');
			var fn = function(k) {
				sel.onchange = function() {
					ModuleDesignerFields.editingField.conditional_options[k][0] = this.getSelectedKeys();
				}
			};
			fn(i);
			cell.appendChild(sel.render());
			var cell = row.insertCell(1);
			var sel = this.renderDropdown(SUGAR.ui.getFormInput(this.form, 'ext1').getValue(), this.editingField.conditional_options[i][1], true);
			var fn = function(k) {
				sel.onchange = function() {
					ModuleDesignerFields.editingField.conditional_options[k][1] = this.getSelectedKeys();
				}
			};
			fn(i);
			cell.appendChild(sel.render());

			var icon = createElement2('div', {'className': "input-icon icon-delete active-icon"});
			var fn = function(k) {
				icon.onclick = function() {
					ModuleDesignerFields.editingField.conditional_options.splice(k, 1);
					ModuleDesignerFields.renderConditionalOptions(table);
				}
			};
			fn(i);
			var cell = row.insertCell(2);
			cell.appendChild(icon);
		}
	};

	this.addConditionalOption = function() {
		if (!this.editingField.conditional_options)
			this.editingField.conditional_options = [];
		this.editingField.conditional_options.push([ [], [] ]);
	}


	this.renderDependentOptions = function(labelEl, valueEl) {
		labelEl.innerHTML = '';
		valueEl.innerHTML = '';
		var fname = this.editingField.depends_on;
		var f = null;
		for (var i  in this.fields) {
			if (this.fields[i].name == fname) {
				f = this.fields[i];
				break;
			}
		}
		if (f) {
			labelEl.innerText = mod_string('LBL_VISIBLE_WHEN', 'ModuleDesigner');
			if (f.type == 'bool') {
				var sel = this.renderDropdown('custom_checkbox_dependent', this.editingField.depends_on_value, false);
				sel.onchange = function() {
					ModuleDesignerFields.editingField.depends_on_value = this.getValue();
				}
				valueEl.appendChild(sel.render());
			} else {
				var sel = this.renderDropdown(f.options, this.editingField.depends_on_value, true);
				sel.onchange = function() {
					var val = this.getSelectedKeys();
					ModuleDesignerFields.editingField.depends_on_value = val;
				}
				valueEl.appendChild(sel.render());

			}
		}
	};

	this.renderDropdown = function(dropdown_name, vals, multi, remove_first) {
		var options = {};
		var opt_name = null;
		var app_lang = SUGAR.language.getModule('app');
		for (var i=0; i < this.dom_opts.length; i++) {
			if (this.dom_opts[i] == dropdown_name) {
				opt_name = dropdown_name;
				break;
			}
		}
		var keys = [];
		var values = [];
		if (opt_name) {
			keys = app_lang.lists.keys[opt_name].filter(function(a,idx) {return i || !remove_first});
			values = app_lang.lists.values[opt_name].filter(function(a,idx) {return i || !remove_first});
		}

		var opts = {keys: keys, values: values};
		
		var sel;
		if (multi) {
			vals = Array.isArray(vals) ? vals : vals.split('^,^').filter(function(x) {return x.length});
			sel = new SUGAR.ui.SelectList('', {
				options: opts,
				multi_select: true,
				rows: 6,
				width: 200
			});
			sel.setSelectedKey(vals);
		} else {
			sel = new SUGAR.ui.SelectInput('', {
				options: opts,
				init_value: vals,
				width: 200
			});
		}
		sel.setup();
		sel.render().style.overflowY = 'auto';

		return sel;
	};	


	this.renderFormulaInput = function(table, label)
	{
		if (!label) label = SUGAR.language.getString('LBL_FORMULA');

		var elements = this.createRow(label);
		table.appendChild(elements[0]);
        var cell = elements[2];
        var input = document.createElement('textarea');
        input.name = 'formula';
        input.id = 'formula';
        input.size = 15;
        input.rows = 4;
        input.cols = 50;
		if (this.editingField.source) {
	        input.value = this.editingField.source.calc_formula || '';
		} else {
	        input.value = '';
		}
        input.onchange = function() {
			if (!ModuleDesignerFields.editingField.source) {
				ModuleDesignerFields.editingField.source = {};
			}
            ModuleDesignerFields.editingField.source.calc_formula = this.value;
        };
        cell.appendChild(input);
        return input;
    };
    
	this.displayFormula = function(show)
    {
        this.displayElement('custom_fields_formula', show);
    };

    this.addToFormula = function(formula_elt)
    {
        if ($('formula') && formula_elt.innerHTML != '') {
            var postfix = '';
            if (formula_elt.id.indexOf('newfunc') != -1)
                postfix = '(';
            $('formula').value = $('formula').value + formula_elt.innerHTML + postfix;
			if (!this.editingField.source)
				this.editingField.source = {};
            this.editingField.source.calc_formula = $('formula').value;
			$('formula').focus();
        }
    };

	this.defaultDefault = function(type)
	{
		switch (type) {
			case 'ref':
				break;
			case 'email':
			case 'varchar':
			case 'item_number':
			case 'text':
			case 'url':
			case 'html':
				return "";
				break;
			case 'int':
			case 'double':
				return 0;
				break;
			case 'bool':
				return false;
				break;
		}
	};
	
	this.renderDefaultBool = function(table)
	{
		this.renderCheckbox(table, mod_string('LBL_DEFAULT_VALUE', 'ModuleDesigner'), 'default');
	};
	
	this.renderCheckbox = function(table, label, name) {

		var elements = this.createRow(label);
		table.appendChild(elements[0]);

		var cell = elements[2];

		var input = new SUGAR.ui.CheckInput('default_check',
			{no_default: true,
				add_switch: true,
				className: 'input-toggle',
				label: '',
				init_value: get_default(this.editingField[name], false)});
		input.render();
		SUGAR.ui.registerInput(this.form, input);
		
		
		
		input.onchange = function() {
			ModuleDesignerFields.updateField(this, name);
		}
		cell.appendChild(input.elt.get());
		if (this.editingField.module_designer == 'label_only')
			input.setDisabled( true);
	};

	this.renderLabel = function(row, label)
	{
		var cell = row.insertCell(0);
		cell.className = 'dataLabel';
		cell.style.verticalAlign = 'top';
		cell.appendChild(document.createTextNode(label));
		return cell;
	};
	
	this.renderDefaultText = function(table, label)
	{
		if (!label) label = mod_string('LBL_DEFAULT_VALUE', 'ModuleDesigner');
		return this.renderTextField(table, label, 'default', 20);
	};

	this.renderSize = function(table)
	{
		return this.renderTextField(table, mod_string('LBL_MAX_SIZE', 'ModuleDesigner'), 'len', 5);
	};
	
	this.renderTextField = function(table, label, name, size) {

		var elements = this.createRow(label);
		table.appendChild(elements[0]);

		var cell = elements[2];
		var input = document.createElement('input');
		input.className = 'input-text';
		input.name = name;
		input.id = name;
		input.size = size;
		input.value = get_default(this.editingField[name], '');
		input.onchange = function() {
			ModuleDesignerFields.updateField(this, name);
		}
		if (this.editingField.module_designer == 'label_only')
			input.disabled = true;
		cell.appendChild(input);
		return input;
	};

	this.createRow = function(label) {
		var wrapper = createElement2('div', {className: 'row form-row col-unstack'});
		var labelEl = createElement2('p', {className: 'form-label'}, label);
		var valueEl = createElement2('div', {className: 'form-value'});
		var cell = createElement2('div', {className: 'form-entry'}, [labelEl, valueEl]);
		createElement2('div', {className: 'column form-cell sm-12 span-2'}, cell, wrapper);
		return [wrapper, labelEl, valueEl];
	};

	this.renderModulesList = function(table, label)
	{
		var elements = this.createRow(label);
		table.appendChild(elements[0]);
		var cell = elements[2];

		var keys = [];
		var values = [];

		for (var i in this.rel_modules) {
			keys.push(i);
			values.push(this.rel_modules[i]);
		}

		var opts = {keys: keys, values: values, width: 300};
		
		var select =  new SUGAR.ui.SelectInput('bean_name', {
			width: 200,
			options: opts,
			init_value: this.editingField.bean_name
		});

		
		select.onchange = function() {
			ModuleDesignerFields.updateField(this, 'bean_name');
		};
		cell.appendChild(select.render());
		if (this.editingField.module_designer == 'label_only')
			select.setDisabled(true);
		select.setDisabled(!this.isNewField(this.editingId));
	};
	
	this.renderRefFields = function(table, label) {
		var self = this;

		var elements = this.createRow(label);
		table.appendChild(elements[0]);
		var cell = elements[2];

		var keys = [];
		var values = [];
		var value = this.editingField.ref;
		var found = false;
		var first = null;
		this.ref_fields.forEach(function(rf) {
			var opt = document.createElement('option');
			keys.push(rf.name);
			values.push(rf.label);
			if (!first)
				first = rf.name;
			if (rf.name == self.editingField.ref) {
				found = true;
			}
		});
		if (!found) {
			value = first;
			this.editingField.ref = first;
		}
		var opts = {keys: keys, values: values, width: 300};
		
		var select =  new SUGAR.ui.SelectInput('ext1', {
			width: 200,
			options: opts,
			init_value: value
		});

		select.onchange = function() {
			self.editingField.ref = this.getValue();
			self.renderRefFieldFields(table, this.getValue(), SUGAR.ui.getFormInput(self.form, 'ext3'));
		};
		cell.appendChild(select.render());
		SUGAR.ui.unregisterInput(this.form, 'ext3');
	};

	this.renderRefFieldFields = function(table, field, select) {
		var self = this;
		if (!select) {
			var elements = this.createRow(mod_string('LBL_RELATED_FIELD', 'ModuleDesigner'));
			table.appendChild(elements[0]);
			var cell = elements[2];

			var opts = {keys: [], values: [], width: 300};
		
			var select =  new SUGAR.ui.SelectInput('ref_field_field', {
				name: 'ext3',
				width: 200,
				options: opts
			});
			select.onchange = function() {
				self.editingField.field = this.getValue();
			};
			cell.appendChild(select.render());
			SUGAR.ui.registerInput(this.form, select);
		}
		var model = null;
		this.ref_fields.forEach(function(rf) {
			if (rf.name == field) {
				model = rf.model;
			}
		});
		var value = null;
		var keys = [];
		var values = [];
		var found = false;
		var first = null;
		if (this.ref_fields_fields[model]) {
			this.ref_fields_fields[model].forEach(function(rf) {
				if (!first)
					first = rf.name;
				keys.push(rf.name);
				values.push(rf.label);
				if (rf.name == self.editingField.field) {
					found = true;
					value = rf.name;
				}
			});
		}
		if (!found) {
			self.editingField.field = first;
			value = first;
		}
		var opts = {keys: keys, values: values, width: 300};
		select.setOptions(new SUGAR.ui.SelectOptions(opts));
		select.setValue(value);
	}

	this.renderDropDownsList = function(table, label)
	{
		var self = this;
		var elements = this.createRow(label);
		table.appendChild(elements[0]);
		var cell = elements[2];

		var keys = [];
		var values = [];
		var first = null;
		var found = null;
		var value = this.editingField.options;
		for (var i=0; i < this.dom_opts.length; i++) {
			if (!first)
				first = this.dom_opts[i];
			if (this.dom_opts[i] == value)
				found = true;
			keys.push(this.dom_opts[i]);
			values.push(this.dom_opts[i]);
		}

		if (!found)
			value = this.editingField.options = first;
		var opts = {keys: keys, values: values, width: 300};
		var select =  new SUGAR.ui.SelectInput('ext1', {
			name: 'ext1',
			width: 200,
			options: opts,
			init_value: value
		});
		
		select.onchange = function() {
			ModuleDesignerFields.updateField(this, 'options');
			ModuleDesignerFields.renderDefaultDropdown(table, this.getValue(),  SUGAR.ui.getFormInput(self.form, 'default'));
		};
		if (this.editingField.module_designer == 'label_only')
			select.setDisabled(true);
		cell.appendChild(select.render());
		SUGAR.ui.registerInput(this.form, select);
	};
	
	this.renderDefaultDropdown = function(table, list, select)
	{

		var app_lang = SUGAR.language.getModule('app');
		var def = undefined;
		if (!select) {
			var elements = this.createRow(mod_string('LBL_DEFAULT_VALUE', 'ModuleDesigner'));
			table.appendChild(elements[0]);
			var cell = elements[2];

			var opts = {keys: [], values: [], width: 300};
			select =  new SUGAR.ui.SelectInput('default', {
				name: 'default',
				width: 200,
				options: opts
			});

			select.onchange = function() {
				ModuleDesignerFields.updateField(this, 'default');
			};
			cell.appendChild(select.render());
			SUGAR.ui.registerInput(this.form, select);
		}
		var keys = deep_clone(app_lang.lists.keys[list]);
		var values = deep_clone(app_lang.lists.values[list]);
		if(!~keys.indexOf('')) {
			keys.unshift('');
			values.unshift('');
		}
		for (var i =0; i < keys.length; i++) {
			var k = keys[i];
			if (typeof(def) == 'undefined') def = k;
			if(k == this.editingField['default']) {
				def = k;
			}
		}
		var opts = {keys: keys, values: values, width: 300};
		select.setOptions(new SUGAR.ui.SelectOptions(opts));
		if (this.editingField.module_designer == 'label_only')
			select.setDisabled(true);
		if (def.length) select.setValue(def, true);
		this.editingField['default'] = def;
	};
	
	this.renderDefaultTextArea = function(table, label)
	{
		if (!label) label = mod_string('LBL_DEFAULT_VALUE', 'ModuleDesigner');

		var elements = this.createRow(label);
		table.appendChild(elements[0]);

		var cell = elements[2];
		var input = document.createElement('textarea');
		input.size = 20;
		input.value = get_default(this.editingField['default'], '');
		input.rows = 8;
		input.cols = 40;
		input.onchange = function() {
			ModuleDesignerFields.updateField(this, 'default');
		};
		input.id = 'default';
		input.name = 'default';
		cell.appendChild(input);
		if (this.editingField.module_designer == 'label_only')
			input.disabled = true;
	};

	this.renderDefaultDate = function(table, label)
	{
		var elements = this.createRow(label);
		table.appendChild(elements[0]);
		var cell = elements[2];

		var keys = [];
		var values = [];

		for (var i in this.date_default_opts) {
			var opt = document.createElement('option');
			keys.push(i);
			values.push(this.date_default_opts[i]);
		}

		var opts = {keys: keys, values: values, width: 300};
		select =  new SUGAR.ui.SelectInput('default', {
			name: 'default',
			width: 200,
			options: opts,
			init_value: this.editingField['default']
		});

		select.onchange = function() {
			ModuleDesignerFields.updateField(this, 'default');
		};
		cell.appendChild(select.render());
		if (this.editingField.module_designer == 'label_only')
			select.setDisabled(true);
	};

	this.updateField = function(input, key)
	{
		var value;
		if (typeof input.getValue === 'function')
			value = input.getValue();
		else if (get_default(input.type) == 'checkbox')
			value = input.checked;
		else
			value = input.value;
		this.editingField[key] = value;
		if (key == 'type' && value == 'text')
			this.editingField.len = null;

		this.renderForm();
	};
    
	this.displayElement = function(id, show, input)
    {
        var display = '';
        if (! show)
            display = 'none';
        if ($(id))
            $(id).style.display = display;
		if ($(input))
			$(input).disabled = this.editingField.module_designer == 'label_only';
    };
    
	this.displayRequired = function(show)
    {
        this.displayElement('req', show, 'required');
    };
	
	this.displayAudit = function(show)
    {
        this.displayElement('audit', show, 'audited');
        this.displayElement('personal_info', show, 'personal_info');
        this.displayElement('privacy_consent', show, 'privacy_consent');
    };
	
	this.displayMassUpdate = function(show)
    {
        this.displayElement('mass_update', show, 'massupdate');
    };

	this.saveField = function()
	{
		if (!this.validate())
			return;
		this.popup.hide();
		this.fields[this.editingId] = this.editingField;
		if (typeof(this.fields[this.editingId]['default']) == "undefined")
			this.fields[this.editingId]['default'] = this.defaultDefault(this.editingField.type);
		if (this.editingField.type == 'ref' && !this.editingField.bean_name) {
			this.fields[this.editingId + '_module'] = {
				name: this.editingField.name + '_mb_module_',
				type: 'module_name'
			};
			this.fields[this.editingId].dynamic_module = this.editingField.name + '_mb_module_';
		} else {
			delete (this.fields[this.editingId].dynamic_module);
		}
		this.render();
	};

	this.newField = function()
	{
		this.editingId = 'newfield~' + this.nextId++;
		this.editingField = {name:'', vname: '', label: '', type: 'varchar'};
		this.popup = new SUGAR.ui.Dialog('field_editor', {content_elt: this.field_dialog, width: '640px', title: mod_string('LBL_CREATE_DIALOG_TITLE', 'ModuleDesigner'), destroy_on_close: false});
		this.popup.setTitle(mod_string('LBL_CREATE_DIALOG_TITLE', 'ModuleDesigner'));
		this.popup.render();
		this.popup.show();
		this.resetErrors();
		this.renderForm();
		this.field_dialog.style.display='';
	};

	this.isNewField = function(id)
	{
		return id.match(/^newfield~/);
	};

	this.resetErrors = function()
	{
		$('custom_errors').innerHTML = '';
        if ($('default'))
			$('default').className = '';
        if ($('len'))
			$('len').className = '';
	};

	this.validate = function()
	{
		this.errors = [];
		var ok = true;
		if (!this.validateFieldName()) ok = false;
		if (!this.validateSize()) ok = false;
		if (!this.validateDefaultValue()) ok = false;
		$('custom_errors').innerHTML = '';
		for (var i =0; i < this.errors.length; i++) {
			var div = document.createElement('div');
			div.className = 'error';
			div.appendChild(document.createTextNode(this.errors[i]));
			$('custom_errors').appendChild(div);
		}
		return ok;
	};

	this.validateFieldName = function()
	{
		var nameInput = SUGAR.ui.getFormInput(this.form, 'name');
		var value = nameInput.getValue();
		ok = value.match(/^[a-zA-Z]([A-Za-z0-9_])*$/);
		if (!ok) this.errors.push(mod_string('ERR_INVALID_NAME', 'ModuleDesigner'));

		if (ok) {
			var fields = Object.assign({}, this.all_fields, this.fields);
			for (var id in fields) {
				if ((fields[id].name || '').toLowerCase() === value.toLowerCase() && id != this.editingId) {
					this.errors.push(mod_string('ERR_DUPLICATE_FIELD', 'ModuleDesigner'));
					ok = false;
					break;
				}
			}
		}
		
		SUGAR.ui.addRemoveClass(nameInput.elt, 'state-error', !ok);

		return ok;
	};

	this.validateSize = function()
	{
		var ok = true;
		if (!$('len')) return true;
		var value = $('len').value;
        var val = '';
		var typeInput = SUGAR.ui.getFormInput(this.form, 'type');
		switch (typeInput.getValue()) {
			case 'int':
				val =  parseInt(value);
				if ('' + val == 'NaN') val = 0;
				ok = val > 0 && val < 12;
				if (!ok) this.errors.push(mod_string('ERR_SIZE_1', 'ModuleDesigner'));
				else this.editingField.len = val;
				break;
			case 'varchar':
			case 'item_number':
				val =  parseInt(value);
				if ('' + val == 'NaN') val = 0;
				ok = val > 0 && val < 256;
				if (!ok) this.errors.push(mod_string('ERR_SIZE_2', 'ModuleDesigner'));
				else this.editingField.len = val;
				break;
		}
		SUGAR.ui.addRemoveClass($('len'), 'state-error', !ok);
		return ok;
	};

	this.validateDefaultValue = function()
	{
		var ok = true;
		var max_size = 0x7FFFFFFF;
		if ($('len')) {
			var value = $('len').value;
			max_size = parseInt(value) || 0x7FFFFFFF;
		}
		
        if ($('default')) {
            var value = $('default').value;
			var typeInput = SUGAR.ui.getFormInput(this.form, 'type');
            switch (typeInput.getValue()) {
                case 'int':
                    ok = (value == '') || ('' + parseInt(value)) != 'NaN';
					if (!ok) this.errors.push(mod_string('ERR_INT_VALUE', 'ModuleDesigner'));
                    break;
                case 'double':
                    ok = (value == '') || ('' + parseFloat(value)) != 'NaN';
					if (!ok) this.errors.push(mod_string('ERR_DEC_VALUE', 'ModuleDesigner'));
                    break;
                case 'date':
                    ok = validateDateInput($('default'));
					ok = true;
					if (!ok) this.errors.push(mod_string('ERR_DATE_VALUE', 'ModuleDesigner'));
                    break;
                case 'varchar':
                case 'item_number':
					ok = value.length <= max_size;
					if (!ok) this.errors.push(mod_string('LBL_TOO_LONG_STRING', 'ModuleDesigner'));
                    break;
            }
			SUGAR.ui.addRemoveClass($('default'), 'state-error', !ok);
        }
		return ok;
	};

	this.saveAll = function(form)
	{
		var fields = JSON.stringify(this.fields);
		return SUGAR.ui.sendForm(form, {fields: fields}, null);
	};

}();

