<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


class MonthlyServiceUtil {
		
	/**
	 * Constructor
	 * 
	 * private because class has only static methods
	 */	
	private function __construct() {}


	/**
	 * Create Invoice for Recurring Service
	 * 
	 * @param array $serviceData
	 * @return string
	 */
	public static function createInvoice($serviceData) {
		global $timedate;
		
		require_bean('Invoice');
		$inv = RowUpdate::blank_for_model('Invoice', $serviceData['assigned_user_id']);
		Invoice::init_record($inv, array('quote_id' => $serviceData['quote_id']));
		$quote = ListQuery::quick_fetch_row('Quote', $serviceData['quote_id']);
		$inv->set('billing_contact_id', $serviceData['contact_id']);
		$inv->set('billing_account_id', $serviceData['account_id']);
		$inv->set('terms', $serviceData["invoice_terms"]);
		$inv->set('due_date', Invoice::calc_due_date($serviceData['invoice_terms']));
		$inv->set('purchase_order_num', $serviceData["purchase_order_num"]);
		$inv->set('description', $quote["description"]."\n\n". translate('LBL_RECURRING_SERVICE', 'MonthlyServices') .': '.$serviceData["booking_category_name"]. "\n".
				translate('LBL_START_DATE', 'MonthlyServices') .': '. $timedate->to_display_date($serviceData["invoice_start_date"], false). ', '.
				translate('LBL_END_DATE', 'MonthlyServices') .': '. $timedate->to_display_date($serviceData["invoice_end_date"], false));
		if(! $inv->validate() || ! $inv->save()) {
			$GLOBALS['log']->error("Error saving recurring invoice: " . $inv->getErrors(true));
		}
		$invoiceId = $inv->getPrimaryKeyValue();

	    return $invoiceId;
	}

	/**
	 * Create PDF by Invoice
	 *
	 * @param string $invoiceId
     * @param string $addressId
	 * @return string
	 */
	public static function createPdf($invoiceId, $addressId) {
		require_once('modules/PDFTemplates/PDFTemplatePrinter.php');
		require_once('modules/PDFTemplates/utils.php');

		$tpl = PDFTemplate_get_default('Invoice', 'invoice');
		$printer = new PDFTemplatePrinter($tpl->getField('id'), $invoiceId);
		$printer->useCompanyAddress($addressId);
		$pdf = $printer->render();
		$pdf->filename = filename_safe_string($printer->getFileName());
		$pdfId = $printer->create_note($pdf, $pdf->filename, false);
		return $pdfId;
	}
	
	/**
	 * Send Invoice to Contact and CC User
	 * 
	 * @param array $serviceData
	 * @param string $invoiceId
	 * @param string $pdfId
	 */
	public static function sendInvoice($serviceData, $invoiceId, $pdfId, $charge_message) {
		global $app_strings, $timedate;

		$dateFormat = array_get_default($app_strings, 'LBL_DATE_SHORT_FORMAT', '%b %e, %Y');
        $params = array(
			'{CATEGORY}' => $serviceData['booking_category_name'],
			'{ACCOUNT}' => $serviceData['account_name'],
			'{START_DATE}' => custom_strftime($dateFormat, strtotime($serviceData['invoice_start_date'])),
        	'{END_DATE}' => custom_strftime($dateFormat, strtotime($serviceData['invoice_end_date'])),
        	'{CHARGE_MESSAGE}' => $charge_message ? ("\n\n" . $charge_message) : '',
        );
        
		require_once('include/Notify/RecordActionMessage.php');
		$message = RecordActionMessage::for_action('MonthlyService', 'invoice');
		if($message) {
			$message->addRecordId($serviceData['id']);
			$message->display_name = 'account.name';
			$message->title = array('LBL_NOTIFY_INVOICE_TITLE', 'MonthlyServices');
			$message->subject = $message->title;
			$message->body = array('LBL_NOTIFY_INVOICE_BODY', 'MonthlyServices');
			$message->populateRecords();
			$message->replace_params = $params;
			$message->getIncludes()->addNoteAttachment($pdfId);
			$mailer = $message->getMailer();
			$ctc_target = $user_target = null;
			if(! empty($serviceData['contact_id'])) {
				$ctc_email = $mailer->addTargetId('Contact', $serviceData['contact_id']);
				$ctc_target = $mailer->getTarget($ctc_email);
			}
			if(! empty($serviceData['cc_user_id'])) {
				$user_email = $mailer->addTargetId('User', $serviceData['cc_user_id']);
				$user_target = $mailer->getTarget($user_email);
			}
			$mailer->sendMessage($message);
						
			if($ctc_target) {
				$fmt = NotifyFormatter::create_standard();
				$ctc_target['remove_images'] = true;
				$fmt->setTarget($ctc_target);
				$render = $mailer->renderEmail($fmt, array($message));
				if($render) {
					$sender = $mailer->getSender();
					
					require_bean('Email');
					$email = new Email();
					$email->id = create_guid();
					$email->new_with_id = true;
					$email->date_start = $timedate->to_display_date_time(gmdate('Y-m-d H:i:s'));
					$email->assigned_user_id = $serviceData['assigned_user_id'];
					$email->account_id = $serviceData['account_id'];
					$email->contact_id = $ctc_target['id'];
					$email->parent_type = 'Invoice';
					$email->parent_id = $invoiceId;
					$email->from_name = $sender['from_name'];
					$email->from_addr = $sender['from_addr'];
					$email->to_addrs_arr = array(
						array('email' => $ctc_target['email']),
					);
					$email->cc_addrs_arr = array();
					$email->status = 'sent';
					$email->isread = 1;
					$email->bcc_addrs_arr = array();
					
					$email->name = $render['subject'];
					$email->setDescription($render['text_body']);
					$email->setDescriptionHTML($render['body']);

					$note = ListQuery::quick_fetch('Note', $pdfId);
					if ($note) {
						$note_upd = $note->getRowUpdate();
						$note_upd->set(array(
							'email_attachment' => 1,
							'parent_type' => 'Emails',
							'parent_id' => $email->id,
						));
						$note_upd->prohibit_workflow = true;
						$note_upd->save();
						$email->regenerate_attachment_array();
					}
					
					$email->prohibit_workflow = true;
					$email->save();
				}
			}
		}
	}
	
	/**
	 * Send Recurring Service Expire Remainder to Contact and CC User
	 * 
	 * @param array $serviceData
	 */
	public static function sendServiceExpiryRemainder($serviceData) {
		require_once('include/Notify/RecordActionMessage.php');
		$message = RecordActionMessage::for_action('MonthlyService', 'expired');
		if($message) {
			$message->addRecordId($serviceData['id']);
			$message->display_name = 'account.name';
			$message->title = array('LBL_NOTIFY_EXPIRED_TITLE', 'MonthlyServices');
			$message->subject = $message->title;
			$message->body = array('LBL_NOTIFY_EXPIRED_BODY', 'MonthlyServices');
			$message->populateRecords();
			$message->replace_params = array('{CATEGORY}' => $message->getRecordField('booking_category.name'));
			$mailer = $message->getMailer();
			if(! empty($serviceData['contact_id']))
				$mailer->addTargetId('Contact', $serviceData['contact_id']);
			if(! empty($serviceData['cc_user_id']))
				$mailer->addTargetId('User', $serviceData['cc_user_id']);
			$mailer->sendMessage($message);
		}
	}

	public static function isFinished($id)
	{
		$service = ListQuery::quick_fetch('MonthlyService', $id);
		$res = true;
		if ($service) {
			if ($service->getField('next_invoice') <= $service->getField('end_date'))
				$res = false;
		}
		return $res;
	}


	private static function saveEmail($subject, $body, $to, $cc, $note, $account, $contact, $parent_type, $parent_id) {
		$from_addr = AppConfig::setting('notify.from_address', '');
		$from_name = AppConfig::setting('notify.from_name', '');
		$email = new Email();
		$email->id = create_guid();
		$email->new_with_id = true;
		$email->account_id = $account;
		$email->contact_id = $contact;
		$email->parent_type = $parent_type;
		$email->parent_id = $parent_id;

		$email->from_name = $from_name;
		$email->from_addr = $from_addr;

		$email->to_addrs_arr = array(
			array('email' => $to),
		);
		if ($cc) {
			$email->cc_addrs_arr = array(
				array('email' => $cc),
			);
		}

		$email->type = 'sent';
		$email->isread = 'read';
		$email->bcc_addrs_arr = array();
		$email->name = $subject;	
		$email->setDescription($body);
		if ($note) {
			$note->email_attachment = 1;
			$note->parent_type = 'Emails';
			$note->parent_id = $email->id;
			$note->save();
			$email->regenerate_attachment_array();
		}

		$exc = array();
		if ($email->send($exc)) {
			// Save the email
			$email->prohibit_workflow = true;
			$email->save();
		}
	}

}






?>
