NewStudio = new function() {
	this.data_type = '';
	this.field_name = '';
	this.default_value = '';
    this.formula = '';
	this.fieldLabel = '';
	this.max_size = 50;
	this.required = false;
	this.audited = false;
	this.ext1 = '';
	this.ext4 = '';
	this.ext5 = '';
	this.ext6 = '';
	this.all_fields = [];
	this.dom_opts = [];
	this.date_default_opts = {};
	this.errors = [];
	this.conditional_options = [];
	this.ref_fields = [];

	this.init = function(params) {
		for (var p in params) {
			this[p] = params[p];
		}
		if($('field_name').form.edit.value)
			$('field_name').setAttribute('readonly', true);;
	};

	this.checkDefaultValue = function() {
		var data_format = $('data_type').value;

		if (data_type == 'calculated') {
			data_format = $('data_format').value;
		}
		switch (data_format) {
			case 'bool':
				this.default_value = parseInt(this.default_value) ? 1 : 0;
				break;
		}
	}

	this.renderForm = function() {
		table = $('custom_fields_body');
		while (table.rows.length > 0) {
			table.deleteRow(0);
		}

		this.checkDefaultValue();

        this.displayAudit(true);
        this.displayMassUpdate(true);
        this.displayFormula(false);
        this.displayFormat(false);

		switch ($('data_type').value) {
			case 'ref':
				this.renderModulesList(table, mod_string('LBL_RELATED_TYPE', 'NewStudio'));
				break;
			case 'related_value':
				this.renderRefFields(table, mod_string('LBL_RELATED_VALUE', 'NewStudio'));
				this.renderRefFieldFields(table, this.ext1, $('ref_field_field'));
				break;
			case 'varchar':
			case 'item_number':
				this.renderDefaultText(table);
				if (!this.max_size) this.max_size = 50;
				this.renderSize(table);
				break;
			case 'text':
				this.renderDefaultTextArea(table);
				break;
			case 'int':
				this.renderDefaultText(table);
				if (!this.max_size) this.max_size = 11;
				if (this.max_size > 11) this.max_size = 11;
				this.renderSize(table);
				break;
			case 'double':
				this.renderDefaultText(table);
				break;
			case 'bool':
				this.renderDefaultBool(table);
				break;
			case 'email':
				this.renderDefaultText(table);
				break;
			case 'enum':
			case 'status':
				this.renderDropDownsList(table, SUGAR.language.getString('LBL_DROPDOWN_LIST'));
				this.renderDefaultDropdown(table, $('ext1').value);
				break;
			case 'multienum':
				this.renderDropDownsList(table, SUGAR.language.getString('LBL_MULTI_LIST'));
				this.renderDefaultDropdown(table, $('ext1').value);
				break;
			case 'date':
				this.renderDefaultDate(table, SUGAR.language.getString('LBL_DEFAULT_VALUE'));
				break;
			case 'url':
				this.renderDefaultText(table, SUGAR.language.getString('LBL_DEFAULT_URL'));
				break;
			case 'html':
				this.renderDefaultTextArea(table, SUGAR.language.getString('LBL_HTML'));
				break;
            case 'calculated':
				this.renderDefaultHidden(table);
                this.displayAudit(false);
                this.displayMassUpdate(false);
                this.displayFormula(true);
                this.displayFormat(true);
				this.renderFormulaInput(table);
                break;
		}
		this.renderDependent(table);
	};

	this.renderDependent = function(table) {
		var row = table.insertRow(table.rows.length);
		this.renderLabel(row, mod_string('LBL_DEPENDS_ON', 'NewStudio'));
		var cell = row.insertCell(1);
		cell.className = 'dataField';
		var select = document.createElement('select');
		select.name = 'ext5';
		select.className = 'input-field';

		var opt = document.createElement('option');
		opt.value = ''
		opt.text = '';
		try {
			select.add(opt, null);
		}
		catch(e) {
			select.add(opt);
		}

		for (var i = 0; i < this.control_fields.length; i++) {
			var f = this.control_fields[i];
			var opt = document.createElement('option');
			var vname_module;
			if (f.vname_module)
				vname_module = f.vname_module;
			else
				vname_module = this.module;
			opt.value = f['name'];
			opt.text = mod_string(f.vname, vname_module);
			if (f.name == this.ext5)
				opt.selected = true;
			try {
				select.add(opt, null);
			}
			catch(e) {
				select.add(opt);
			}
		}
		var row = table.insertRow(table.rows.length);
		cell.appendChild(select);
		NewStudio.renderDependentOptions(row);

		var row2 = table.insertRow(table.rows.length);
		var c = row2.insertCell(0);
		c.colSpan = 2;
		var t = createElement2('table', {'cellpadding': "10", 'width': '100%'});
		this.conditional_options_table = t;
		c.appendChild(t);
		this.renderConditionalOptions(t);
		select.onchange = function() {
			NewStudio.ext5 = this.value;
			NewStudio.ext6 = '';
			NewStudio.conditional_options = [];
			NewStudio.renderDependentOptions(row);
			NewStudio.renderConditionalOptions(t);
		};
	};

	this.renderConditionalOptions = function(table) {

		while (table.rows.length) {
			table.deleteRow(0);
		}

		if (($('data_type').value != 'enum' && $('data_type').value != 'status') || !this.ext5) {
			return;
		}

		var fname = this.ext5;
		var f = null;
		for (var i = 0; i < this.control_fields.length; i++) {
			if (this.control_fields[i].name == fname) {
				f = this.control_fields[i];
				break;
			}
		}
		if (!f || (f.type != 'enum' && f.type != 'status' && f.type != 'bool')) {
			return;
		}

		var row = table.insertRow(table.rows.length);
		var cell = this.renderLabel(row, mod_string('LBL_CONDITIONAL_OPTIONS', 'NewStudio'));
		cell.colSpan = 3;
		cell.style.textAlign = 'center';
		cell.style.fontWeight = 'bold';

		var icon = createElement2('div',  {'className': 'input-icon icon-add active-icon'});
		icon.onclick = function() {
			NewStudio.addConditionalOption();
			NewStudio.renderConditionalOptions(table);
		}
		cell.appendChild(nbsp());
		cell.appendChild(icon);

		var f = null;
		for (var i = 0; i < this.control_fields.length; i++) {
			if (this.control_fields[i].name == fname) {
				f = this.control_fields[i];
				break;
			}
		}

		for (var i = 0; i < this.conditional_options.length; i++) {
			var opt = this.conditional_options[i];
			var row = table.insertRow(table.rows.length);
			var cell = row.insertCell(0);
			var sel = this.renderDropdown(f.type == 'bool' ? 'custom_checkbox_dependent' : f.options, this.conditional_options[i][0], f.type != 'bool', f.type == 'bool');
			var fn = function(k) {
				sel.onchange = function() {
					var selected = [];
					for (var i = 0; i < this.options.length; i++) {
						if (this.options[i].selected)
							selected.push(this.options[i].value);
					}
					NewStudio.conditional_options[k][0] = selected;
				}
			};
			fn(i);
			cell.appendChild(sel);
			var cell = row.insertCell(1);
			cell.className = 'dataField';
			var sel = this.renderDropdown($('ext1').value, this.conditional_options[i][1], true);
			var fn = function(k) {
				sel.onchange = function() {
					var selected = [];
					for (var i = 0; i < this.options.length; i++) {
						if (this.options[i].selected)
							selected.push(this.options[i].value);
					}
					NewStudio.conditional_options[k][1] = selected;
				}
			};
			fn(i);
			cell.appendChild(sel);

			var icon = createElement2('div', {'className': "input-icon icon-delete active-icon"});
			var fn = function(k) {
				icon.onclick = function() {
					NewStudio.conditional_options.splice(k, 1);
					NewStudio.renderConditionalOptions(table);
				}
			};
			fn(i);
			var cell = row.insertCell(2);
			cell.appendChild(icon);
		}
	};

	this.renderDropdown = function(dropdown_name, vals, multi, remove_first, unrestricted) {
		var options = {};
		if (multi)
			options = {'multiple' : true, 'size': 6};
		var sel = createElement2('select', options);
		sel.className = 'input-field';
		sel.style.overflowY = 'auto';
		var opt_name = null;
		var opts = null;
		var app_lang = SUGAR.language.getModule('app');
		var dom_opts = unrestricted ? this.all_dom_opts : this.dom_opts;
		for (var i=0; i < dom_opts.length; i++) {
			if (dom_opts[i] == dropdown_name) {
				opt_name = dropdown_name;
				break;
			}
		}
		if (opt_name) {
			var keys = deep_clone(app_lang.lists.keys[opt_name]);
			var values = deep_clone(app_lang.lists.values[opt_name]);
			for (var i = 0; i < keys.length; i++) {
				if (!i && remove_first)
					continue;
				var opt = document.createElement('option');
				opt.value = keys[i];
				opt.text = values[i];
				if (multi) {
					for(var j = 0; j < vals.length; j++) {
						if (vals[j] == opt.value)
							opt.selected = true;
					}
				} else {
					opt.selected = vals == opt.value;
				}
				try {
					sel.add(opt, null);
				}
				catch(e) {
					sel.add(opt);
				}
			}
		}
		return sel;
	};

	this.addConditionalOption = function() {
		this.conditional_options.push([ [], [] ]);
	}

	this.renderDependentOptions = function(row) {
		row.innerHTML = '';
		var fname = this.ext5;
		var f = null;
		for (var i = 0; i < this.control_fields.length; i++) {
			if (this.control_fields[i].name == fname) {
				f = this.control_fields[i];
				break;
			}
		}
		if (f) {
			this.renderLabel(row, mod_string('LBL_VISIBLE_WHEN', 'NewStudio'));
			var cell = row.insertCell(1);
			cell.className = 'dataField';
			if (f.type == 'bool') {
				var sel = this.renderDropdown('custom_checkbox_dependent', this.ext6, false);
				sel.onchange = function() {
					NewStudio.ext6 = this.value;
				}
				cell.appendChild(sel);
			} else {
				var sel = this.renderDropdown(f.options, ext6, true, false, true);
				sel.onchange = function() {
					var val = [];
					for (var i = 0; i < this.options.length; i++) {
						if (this.options[i].selected) {
							val.push(this.options[i].value);
						}
					}
					NewStudio.ext6 = val;
				}
				cell.appendChild(sel);

			}
		}
	};

	this.renderModulesList = function(table, label)
	{
		var row = table.insertRow(table.rows.length);
		this.renderLabel(row, label);
		var cell = row.insertCell(1);
		cell.className = 'dataField';
		var select = document.createElement('select');
		select.name = 'ext1';
		select.id = 'bean_name';
		select.className = 'input-field';
		for (var i in this.rel_modules) {
			var opt = document.createElement('option');
			opt.value = i;
			opt.text = this.rel_modules[i];
			try {
				select.add(opt, null);
			}
			catch(e) {
				select.add(opt);
			}
			if (i == this.ext1)
				opt.selected = true;
		}
		select.onchange = function() {
			NewStudio.ext1 = this.value;
		};
		//select.disabled = !this.isNewField(this.editingId);
		cell.appendChild(select);
	};

	this.renderRefFields = function(table, label)
	{
		var row = table.insertRow(table.rows.length);
		this.renderLabel(row, label);
		var cell = row.insertCell(1);
		cell.className = 'dataField';
		var select = document.createElement('select');
		select.name = 'ext1';
		select.className = 'input-field';
		var found = false;
		var first = null;
		this.ref_fields.forEach(function(rf) {
			var opt = document.createElement('option');
			opt.value = rf.name;
			opt.text = rf.label;
			if (!first)
				first = rf.name;
			try {
				select.add(opt, null);
			}
			catch(e) {
				select.add(opt);
			}
			if (rf.name == NewStudio.ext1) {
				opt.selected = true;
				found = true;
			}
		});
		if (!found) {
			select.value = first;
			this.ext1 = first;
		}
		select.onchange = function() {
			NewStudio.ext1 = this.value;
			NewStudio.renderRefFieldFields(table, this.value, $('ref_field_field'));
		};
		cell.appendChild(select);
	};

	this.renderRefFieldFields = function(table, field, select) {
		if (!select) {
			var row = table.insertRow(table.rows.length);
			this.renderLabel(row, mod_string('LBL_RELATED_FIELD', 'NewStudio'));
			var cell = row.insertCell(1);
			cell.className = 'dataField';
			select = document.createElement('select');
			select.name = 'ext3';
			select.id= 'ref_field_field';
			select.className = 'input-field';
			select.onchange = function() {
				NewStudio.ext3 = this.value;
			};
			cell.appendChild(select);
		} else {
			select.innerHTML = '';
		}
		var model = null;
		this.ref_fields.forEach(function(rf) {
			if (rf.name == field) {
				model = rf.model;
			}
		});
		if (this.ref_fields_fields[model]) {
			this.ref_fields_fields[model].forEach(function(rf) {
				var opt = document.createElement('option');
				opt.value = rf.name;
				opt.text = rf.label;
				try {
					select.add(opt, null);
				}
				catch(e) {
					select.add(opt);
				}
				if (rf.name == NewStudio.ext3)
					opt.selected = true;
			});
		}
	}

    this.displayAudit = function(show)
    {
        this.displayElement('audit_row', show);
    };

    this.displayMassUpdate = function(show)
    {
        this.displayElement('mass_update_row', show);
    };

    this.displayFormula = function(show)
    {
        this.displayElement('custom_fields_formula', show);
    };

    this.displayFormat = function(show)
    {
        this.displayElement('data_format_row', show);
    };

	this.renderDefaultText = function(table, label)
	{
		if (!label) label = SUGAR.language.getString('LBL_DEFAULT_VALUE');
		return this.renderTextField(table, label, 'default_value', 20);
	};

	this.renderDefaultBool = function(table)
	{
		this.renderCheckbox(table,SUGAR.language.getString('LBL_DEFAULT_VALUE'), 'default_value', true);
	};

	this.renderDefaultTextArea = function(table, label)
	{
		if (!label) label = SUGAR.language.getString('LBL_DEFAULT_VALUE');
		var row = table.insertRow(table.rows.length);
		this.renderLabel(row, label);

		var cell = row.insertCell(1);
		cell.className = 'dataField';
		var input = document.createElement('textarea');
		input.size = 20;
		input.value = this.ext4 || this.default_value;
		input.rows = 8;
		input.cols = 60;
		input.onchange = function() {
			NewStudio.ext4 = this.value;
		};
		input.id = 'ext4';
		input.name = 'ext4';
		input.className = 'input-textarea input-outer';
		cell.appendChild(input);
	};

	this.renderDropDownsList = function(table, label)
	{
		var row = table.insertRow(table.rows.length);
		this.renderLabel(row, label);
		var cell = row.insertCell(1);
		cell.className = 'dataField';
		var select = document.createElement('select');
		select.name = 'ext1';
		select.id = 'ext1';
		select.className = 'input-field';
		for (var i=0; i < this.dom_opts.length; i++) {
			var opt = document.createElement('option');
			opt.value = this.dom_opts[i];
			opt.text = this.dom_opts[i];
			try {
				select.add(opt, null);
			}
			catch(e) {
				select.add(opt);
			}
			if(this.dom_opts[i] == this.ext1)
				opt.selected = true;
		}
		select.onchange = function() {
			NewStudio.ext1 = this.value;
			NewStudio.ext6 = '';
			NewStudio.conditional_options = [];
			NewStudio.renderDefaultDropdown(table, this.value, $('default_value'));
			NewStudio.renderConditionalOptions(NewStudio.conditional_options_table);
		};
		cell.appendChild(select);
	};

	this.renderDefaultDate = function(table, label)
	{
		var row = table.insertRow(table.rows.length);
		this.renderLabel(row, label);
		var cell = row.insertCell(1);
		cell.className = 'dataField';
		var select = document.createElement('select');
		select.name = 'ext1';
		select.id = 'ext1';
		select.className = 'input-field';
		for (var i in this.date_default_opts) {
			var opt = document.createElement('option');
			opt.value =i;
			opt.text = this.date_default_opts[i];
			if(i == this.ext1)
				opt.selected = true;
			try {
				select.add(opt, null);
			}
			catch(e) {
				select.add(opt);
			}
		}
		select.onchange = function() {
			NewStudio.default_value = this.value;
		};
		cell.appendChild(select);
	};

	this.renderDefaultDropdown = function(table, list, select)
	{

		var app_lang = SUGAR.language.getModule('app');
		var def = undefined;
		if (select) {
			select.innerHTML = '';
		} else {
			var row = table.insertRow(table.rows.length);
			this.renderLabel(row, SUGAR.language.getString('LBL_DEFAULT_VALUE'));
			var cell = row.insertCell(1);
			cell.className = 'dataField';
			select = document.createElement('select');
			select.name = 'default_value';
			select.id = 'default_value';
			select.className = 'input-field';
			select.onchange = function() {
				NewStudio.default_value = this.value;
			};
			cell.appendChild(select);
		}
		var keys = deep_clone(app_lang.lists.keys[list]);
		var values = deep_clone(app_lang.lists.values[list]);
		if(keys.indexOf('') < 0) {
			keys.unshift('');
			values.unshift('');
		}
		for (var i =0; i < keys.length; i++) {
			var k = keys[i];
			var v = values[i];
			var opt = document.createElement('option');
			opt.value = k;
			opt.text = v;
			try {
				select.add(opt, null);
			}
			catch(e) {
				select.add(opt);
			}
			if (! isset(def)) def = k;
			if(k == this.default_value) {
				opt.selected = true;
				def = k;
			}
		}
		this.default_value = def;
	};

	this.renderDefaultHidden = function(table)
	{
		var row = table.insertRow(table.rows.length);
		var cell = row.insertCell(0);
		var input = document.createElement('input');
		input.type = 'hidden';
		input.name = 'default_value';
		input.id = 'default_value';
		input.value = this.default_value;
		cell.appendChild(input);
	};


	this.renderSize = function(table)
	{
		return this.renderTextField(table, SUGAR.language.getString('LBL_MAX_SIZE'), 'max_size', 5);
	};

	this.renderTextField = function(table, label, name, size) {
		var row = table.insertRow(table.rows.length);
		this.renderLabel(row, label);

		var cell = row.insertCell(1);
		cell.className = 'dataField';
		var input = document.createElement('input');
		input.name = name;
		input.id = name;
		input.size = size;
		input.className = 'input-text';
		if (isset(this[name])) {
			input.value = this[name];
			input.onchange = function() {
				NewStudio[name] = this.value;
			}
		}
		cell.appendChild(input);
		return input;
	};

	this.renderCheckbox = function(table, label, name, add_default) {
		var row = table.insertRow(table.rows.length);
		this.renderLabel(row, label);

		var cell = row.insertCell(1);
		cell.className = 'dataField';
		var input = document.createElement('input');
		input.type = 'checkbox';
		input.id = name;
		input.name = name;
		input.value = 1;
		if (isset(this[name])) {
			input.checked = parseInt(this[name], 10);
			input.onclick = function() {
				NewStudio[name] = this.checked ? 1 : 0;
			}
		}
		if(add_default) {
			var input_h = createElement2('input', {type: 'hidden', name: name, value: 0});
			cell.appendChild(input_h);
		}
		cell.appendChild(input);
	};

	this.renderLabel = function(row, label)
	{
		var cell = row.insertCell(0);
		cell.className = 'dataLabel';
		cell.style.verticalAlign = 'top';
		cell.appendChild(document.createTextNode(label));
		return cell;
	};

    this.renderFormulaInput = function(table, label)
    {
        if (!label) label = SUGAR.language.getString('LBL_FORMULA');
        var row = table.insertRow(table.rows.length);
        this.renderLabel(row, label);

        var cell = row.insertCell(1);
        cell.className = 'dataField';
        var input = document.createElement('textarea');
        input.name = 'formula';
        input.id = 'formula';
        input.size = 15;
        input.rows = 4;
        input.cols = 50;
        input.className = 'input-text';
        input.value = this.formula;
        input.onchange = function() {
            NewStudio['formula'] = this.value;
        };
        cell.appendChild(input);
        return input;
    };

    this.addToFormula = function(formula_elt)
    {
        if ($('formula') && formula_elt.innerHTML != '') {
            var postfix = '';
            if (formula_elt.id.indexOf('newfunc') != -1)
                postfix = '(';
            $('formula').value = $('formula').value + formula_elt.innerHTML + postfix;
            this.formula = $('formula').value;
        }
    };

	this.save = function()
	{
		if (!this.validate()) return false;
		$('ext5').value = this.ext5;
		$('ext6').value = JSON.stringify(this.ext6);
		$('ext7').value = JSON.stringify(this.conditional_options);
	};

	this.validate = function()
	{
		this.errors = [];
		var ok = true;
		if (!this.validateFieldName()) ok = false;
		if (!this.validateSize()) ok = false;
		if (!this.validateDefaultValue()) ok = false;
        if (!this.validateFormula()) ok = false;
		$('custom_errors').innerHTML = '';
		for (var i =0; i < this.errors.length; i++) {
			var div = document.createElement('div');
			div.className = 'error';
			div.appendChild(document.createTextNode(this.errors[i]));
			$('custom_errors').appendChild(div);
		}
		return ok;
	};

	this.validateFieldName = function()
	{
		var value = $('field_name').value;
		ok = value.match(/^[a-zA-Z]([A-Za-z0-9_])*$/);
		if (!ok) this.errors.push(SUGAR.language.getString('LBL_INVALID_FIELD_NAME'));
		else {
			var existing = this.all_fields.find(function(name) {
				return name.toLowerCase() === value.toLowerCase();
			});
			if (existing && !$('field_name').form.edit.value) {
				ok = false;
				this.errors.push(SUGAR.language.getString('LBL_DUPLICATE_FIELD_NAME'));
			}
		}
		if (!ok) $('field_name').className = 'input-text input-outer invalid';
		else $('field_name').className = 'input-text input-outer';
		return ok;
	};

	this.validateSize = function()
	{
		var ok = true;
		if (!$('max_size')) return true;
		var value = $('max_size').value;
        var val = '';
		switch ($('data_type').value) {
			case 'int':
				val =  parseInt(value);
				if ('' + val == 'NaN') val = 0;
				ok = val > 0 && val < 12;
				if (!ok) this.errors.push(SUGAR.language.getString('LBL_INVALID_SIZE'));
				break;
			case 'varchar':
			case 'item_number':
				val =  parseInt(value);
				if ('' + val == 'NaN') val = 0;
				ok = val > 0 && val < 256;
				if (!ok) this.errors.push(SUGAR.language.getString('LBL_INVALID_SIZE2'));
				break;
		}
		if (!ok) $('max_size').className = 'input-text input-outer invalid';
		else $('max_size').className = '';
		return ok;
	};

	this.validateDefaultValue = function()
	{
		var ok = true;
		var max_size = 0x7FFFFFFF;
		if ($('max_size')) {
			var value = $('max_size').value;
			max_size = parseInt(value) || 0x7FFFFFFF;
		}

		if ($('default_value')) {
            var value = $('default_value').value;
            switch ($('data_type').value) {
                case 'int':
                    ok = (value == '') || ('' + parseInt(value)) != 'NaN';
					if (!ok) this.errors.push(SUGAR.language.getString('LBL_INVALID_INT'));
                    break;
                case 'double':
                    ok = (value == '') || ('' + parseFloat(value)) != 'NaN';
					if (!ok) this.errors.push(SUGAR.language.getString('LBL_INVALID_DEC'));
                    break;
                case 'date':
                    ok = validateDateInput($('default_value'));
					ok = true;
					if (!ok) this.errors.push(SUGAR.language.getString('LBL_INVALID_DATE'));
                    break;
                case 'varchar':
                case 'item_number':
					ok = value.length <= max_size;
					if (!ok) this.errors.push(SUGAR.language.getString('LBL_TOO_LONG_STRING'));
                    break;
            }
            if (!ok) $('default_value').className = 'input-text input-outer invalid';
            else $('default_value').className = '';
        }
		return ok;
	};

    this.validateFormula = function()
   	{
   		var ok = true;
        if ($('data_type').value == 'calculated') {
            var value = $('formula').value;
            ok = (value != '');
			if (!ok) this.errors.push(SUGAR.language.getString('LBL_INVALID_FORMULA'));
            if (!ok) $('formula').className = 'input-textarea input-outer invalid';
            else $('formula').className = '';
        }
   		return ok;
   	};

    this.displayElement = function(id, show)
    {
        var display = '';
        if (! show)
            display = 'none';
        if ($(id))
            $(id).style.display = display;
    };

}();
