<?php

require_once 'modules/NewStudio/wizards/StudioWizardBase.php';
require_once 'include/Sugar_Smarty.php';
require_once 'include/layout/forms/EditableForm.php';
require_once 'include/layout/widgets/DynamicListInput.php';

class ArrangeSubpanelsWizard extends StudioWizardBase {

	var $module;
	var $model;
	var $tab;
	var $record_id;

	public function __construct($params) {
		parent::__construct($params);
		$this->module = $_REQUEST['edit_module'];
		$this->tab = $_REQUEST['tab'];
		$this->model = new ModelDef(AppConfig::module_primary_bean($this->module));
		$this->record_id = $_REQUEST['record_id'];
	}

	public function render()
	{
		global $pageInstance;
		
		$pref = UserPreference::getPreference('subpanels_order_' . $_REQUEST['edit_module'] . '_' . $_REQUEST['tab'], 'global');
		if(! is_array($pref)) $pref = array();
		$layout = AppConfig::setting("views.layout.{$this->module}.record.Standard");
		$new_rows = array();
		foreach ($layout['subpanels'] as $sp) {
			if (is_string($sp))
				$sp = array('name' => $sp);
			$tab = array_get_default($sp, 'tab', 'Standard');
			if ($tab != $this->tab)
				continue;
            $link = $this->model->getLinkTargetModel($sp['name']);
            if(! $link) continue;
			$sp['module'] = $link->getModuleDir();
			$sp['bean_name'] = $link->name;
			$sp['model_type'] = $link->type;
			$joined = $link->getJoinModel();
			if($joined) {
				$lspec = $joined->getLinkSpec();
			} else {
				$lspec = $link->getLinkSpec();
			}
			if(isset($lspec['vname']) && !isset($p['title']))
				$sp['title'] = $lspec['vname'];
			if(isset($lspec['vname_module']))
				$sp['vname_module'] = $lspec['vname_module'];
			$visible = true;
			$new_rows[] = array('key' => $sp['name'], 'value' => translate($sp ['title'], array_get_default($sp, 'vname_module', $this->module)), 'visible' => true);
		}

		$rows = array();
		foreach ($pref as $p) {
			foreach ($new_rows as $i => $row) {
				if ($p['name'] == $row['key']) {
					$row['visible'] = array_get_default($p, 'visible', true);
					$rows[] = $row;
					unset($new_rows[$i]);
				}
			}
		}
		$rows = array_merge($rows, $new_rows);

		$frm = new EditableForm('configform', 'ConfigForm');
		$frm->addHiddenFields(array(
			'edit_module' => $this->module,
			'tab' => $this->tab,
			'record_id' => $this->record_id,
			'module' => 'NewStudio',
			'action' => 'index',
			'wizard' => 'ArrangeSubpanels',
			'subpanels_save' => '1',
		));

		$attrs = array(
			'name' => 'subpanel_data',
			'cols' => array(
				array(
					'name' => 'value',
					'label' => translate('LBL_ARRANGE_LAYOUTS_NAME', 'app'),
					'width' => '100%',
					'editable' => false,
					'multiline' => false,
				),
				array(
					'name' => 'visible',
					'label' => translate('LBL_ARRANGE_LAYOUTS_VISIBLE', 'app'),
					'width' => '1%',
					'format' => 'check',
				),
			),
			'width' => '100%',
			'depth_attrib' => '',
			'show_delete' => false,
		);
		$subpanel_data = new DynamicListInput($attrs);
		$subpanel_body = $subpanel_data->render($frm, $rows);
		
		$save_spec = array(
			'name' => 'save_subpanels',
			'label' => translate('LBL_SAVE_BUTTON_LABEL', 'app'),
			'icon' => 'input-icon icon-accept',
			'perform' => 'console.log(this.form); this.form.submit(); SUGAR.popups.close(); return ret;',
			'perform' => 'var ret = SUGAR.ui.sendForm(this.form, {}, null, true); SUGAR.popups.close(); return false;',
		);
		$save_btn = $frm->renderButton($save_spec);
		$reset_spec = array(
			'name' => 'reset_subpanels',
			'label' => translate('LBL_RESET_BUTTON_LABEL', 'app'),
			'icon' => 'input-icon icon-delete',
			'perform' => 'var ret = SUGAR.ui.sendForm(this.form, {subpanels_reset: 1}, null, true); SUGAR.popups.close(); return false;',
		);
		$reset_btn = $frm->renderButton($reset_spec);
		
		// ajrw - would be better to use a form generator here...
		
		echo $frm->open('card-outer form-outer default-form expand');
		echo '<div class="card-header form-buttons">';
		echo '<div class="row buttons-row"><div class="column buttons-inline">';
		echo '<div class="inline-elt">';
		echo $save_btn;
		echo '</div><div class="inline-elt">';
		echo $reset_btn;
		echo '</div>';
		echo '</div></div></div>';
		echo '<div class="card-body form-body">';
		echo '<div class="form-section">';
		echo '<div class="row form-row"><div class="column form-cell">';
		echo '<div class="form-entry"><div class="form-value">';
		echo $subpanel_body;
		echo '</div></div></div></div></div></div>';
		$subpanel_data->exportIncludes();
		echo $frm->close();
		$frm->exportIncludes();
		
		$form_lbl = translate('LBL_ARRANGE_SUBPANELS_FORM_TITLE', 'app');
		$form_icon = '<div class="input-icon icon-editlayout active"></div>&nbsp;';
		$pageInstance->add_js_literal(
			'SUGAR.popups.initContent({title: "'.addcslashes($form_icon.$form_lbl, '"').'", width: 500, fill: true});',
			null, LOAD_PRIORITY_FOOT);

		$pageInstance->set_partial_page();
	}

	public function process() {
		if (empty($_POST['subpanels_save']))
			return;
		$json = getJSONobj();
		$data = $json->decode($_POST['subpanel_data']);
		if (is_array($data)) {
			$pref = array();
			if (empty($_POST['subpanels_reset'])) {
				foreach ($data as $sp) {
					$pref[] = array('name' => $sp['key'], 'visible' => $sp['visible']);
				}
			}
			UserPreference::setPreference('subpanels_order_' . $this->module . '_' . $this->tab, $pref);
		}
		return array(
			'module' => $this->module,
			'action' => 'DetailView',
			'record' => $this->record_id,
			'layout_tab' => $this->tab,
			'external' => true,
		);
		
	}

	
}

