<?php

require_once 'modules/NewStudio/wizards/StudioWizardBase.php';
require_once 'include/Sugar_Smarty.php';
require_once 'include/layout/forms/EditableForm.php';
require_once 'include/layout/widgets/DynamicListInput.php';

class ArrangeTabsWizard extends StudioWizardBase {

	var $module;
	var $model;
	var $record_id;

	public function __construct($params) {
		parent::__construct($params);
		$this->module = $_REQUEST['layout_module'];
		$this->record_id = $_REQUEST['record'];
		$this->model = new ModelDef(AppConfig::module_primary_bean($this->module));
	}

	public function render()
	{
		global $pageInstance;
		
		$layout = AppConfig::setting("views.layout.{$this->module}.record.Standard");
		$rows = array();
		if (empty($layout['tabs'])) {
			$rows[] = array(
				'key' => 'Standard',
				'value' => translate('LBL_LAYOUT_STANDARD', 'app'),
				'visible' => true,
			);
		} else {
			foreach ($layout['tabs'] as $tab_id => $tab) {
				$rows[] = array(
					'key' => $tab_id, 
					'value' => translate($tab['vname'], array_get_default($tab, 'vname_module', $this->module)), 
					'visible' => !array_get_default($tab, 'explicitly_hidden', false),
					'non_standard' => array_get_default($tab, 'non_standard', false),
				);
			}
		}


		$frm = new EditableForm('configform', 'ConfigForm');
		$frm->addHiddenFields(array(
			'layout_module' => $this->module,
			'record' => $this->record_id,
			'module' => 'NewStudio',
			'action' => 'index',
			'wizard' => 'ArrangeTabs',
			'tabs_save' => '1',
		));

		$attrs = array(
			'name' => 'tabs_data',
			'cols' => array(
				array(
					'name' => 'value',
					'label' => translate('LBL_ARRANGE_LAYOUTS_NAME', 'app'),
					'width' => '100%',
					'editable' => false,
					'multiline' => false,
				),
				array(
					'name' => 'visible',
					'label' => translate('LBL_ARRANGE_LAYOUTS_VISIBLE', 'app'),
					'width' => '1%',
					'format' => 'check',
				),
			),
			'width' => '100%',
			'depth_attrib' => '',
			'show_delete' => 1,
			'show_action' => 1,
			'renderer_class' => 'StudioDetailTabsRenderer',
		);
		$tabs_data = new DynamicListInput($attrs);
		$tabs_body = $tabs_data->render($frm, $rows);
		
		$save_spec = array(
			'name' => 'save_tabs',
			'label' => translate('LBL_SAVE_BUTTON_LABEL', 'app'),
			'icon' => 'input-icon icon-accept',
			'perform' => 'var ret = SUGAR.ui.sendForm(this.form, {}, null, true); SUGAR.popups.close(); return false;',
		);
		$save_btn = $frm->renderButton($save_spec);
		$add_spec = array(
			'name' => 'add_tab',
			'label' => translate('LBL_ADD_DETAIL_TAB', 'app'),
			'icon' => 'input-icon icon-edit',
			'perform' => 'StudioDetailTabsRenderer.createTab(this); return false;',
		);
		$add_btn = $frm->renderButton($add_spec);
		$reset_spec = array(
			'name' => 'reset_subpanels',
			'label' => translate('LBL_RESET_BUTTON_LABEL', 'app'),
			'icon' => 'input-icon icon-delete',
			'perform' => 'var ret = SUGAR.ui.sendForm(this.form, {subpanels_reset: 1}, null, true); SUGAR.popups.close(); return false;',
		);
		$reset_btn = $frm->renderButton($reset_spec);
		
		// ajrw - would be better to use a form generator here...
		
		echo $frm->open('card-outer form-outer default-form expand');
		echo '<div class="card-header form-buttons">';
		echo '<div class="row buttons-row"><div class="column buttons-inline">';
		echo '<div class="inline-elt">';
		echo $save_btn;
		echo '</div><div class="inline-elt">';
		echo $add_btn;
		echo '</div><div class="inline-elt">';
		echo $reset_btn;
		echo '</div>';
		echo '</div></div></div>';
		echo '<div class="card-body form-body">';
		echo '<div class="form-section">';
		echo '<div class="row form-row"><div class="column form-cell">';
		echo '<div class="form-entry"><div class="form-value">';
		echo $tabs_body;
		echo '</div></div></div></div></div></div>';
		$tabs_data->exportIncludes();
		echo $frm->close();
		$frm->exportIncludes();
		
		$form_lbl = translate('LBL_ARRANGE_TABS', 'app');
		$form_icon = '<div class="input-icon icon-editlayout active"></div>&nbsp;';
		$pageInstance->add_js_include('modules/NewStudio/detail_tabs.js', null, LOAD_PRIORITY_BODY);
		$pageInstance->add_js_literal(
			'SUGAR.popups.initContent({title: "'.addcslashes($form_icon.$form_lbl, '"').'", width: 500});',
			null, LOAD_PRIORITY_FOOT);

		$pageInstance->set_partial_page();
	}

	public function process() {
		if (empty($_POST['tabs_save']))
			return;
		$json = getJSONobj();
		$data = $json->decode($_POST['tabs_data']);
		$layout = AppConfig::setting("views.layout.{$this->module}.record.Standard");
		$tabs = isset($layout['tabs']) ? $layout['tabs'] : array();
		if (is_array($data)) {
			$pref = array();
			foreach ($data as $tab) {
				$key = $tab['key'];
				if (!isset($tabs[$key])) {
					$prefs[$key]= array();
				} else {
					$prefs[$key] = $tabs[$key];
				}
				$prefs[$key]['vname'] = $tab['value'];
				$prefs[$key]['explicitly_hidden'] = !$tab['visible'];
				$prefs[$key]['non_standard'] = array_get_default($tab, 'non_standard', false);
			}


			$detail = AppConfig::setting("views.detail.{$this->module}.record.Standard");
			$layout = AppConfig::setting("views.layout.{$this->module}.record.Standard", array());
			$layout['tabs'] = $prefs;
			$form_def = array(
				'detail' => $detail,
				'layout' => $layout,
			);
			$lang = array();
			$filename = AppConfig::custom_dir()."modules/{$this->module}/new_views/record.Standard.php";
			require_once 'include/config/format/ConfigWriter.php';
			$cw = new ConfigWriter;
			$cw->writeFile($filename, $form_def);
			AppConfig::invalidate_cache('views');
			AppConfig::cache_reset();
		}
		return array(
			'module' => $this->module,
			'action' => 'DetailView',
			'record' => $this->record_id,
			'external' => true,
		);
		
	}

	
}

