<?php
require_once 'modules/NewStudio/wizards/StudioWizardBase.php';

class ConfigTabsWizard extends StudioWizardBase
{
	public function process()
	{
		$perform = array_get_default($this->params, 'record_perform');
		if($perform == 'reset') {
			$std = AppConfig::setting('module_order.hidden', array(), true);
			AppConfig::set_local('module_order.hidden', $std);
			$std = AppConfig::setting('module_order.disable_user_edit', null, true);
			AppConfig::set_local('module_order.disable_user_edit', $std);
			AppConfig::save_local();
		} else if($perform == 'save') {
			$tabs_input = array_get_default($this->params, 'tabs');
			if($tabs_input) {
				$json = getJSONobj();
				$tabs = $json->decode($tabs_input);
				if($tabs) {
					$hidden = array();
					foreach($tabs as $tab) {
						if(! empty($tab['hidden']))
							$hidden[] = $tab['module'];
					}
					AppConfig::set_local('module_order.hidden', $hidden);
				}
			}
			$can_override = array_get_default($this->params, 'allow_override');
			if(isset($can_override))
				AppConfig::set_local('module_order.disable_user_edit', ! $can_override);
			AppConfig::save_local();
		}
		AppConfig::invalidate_cache('modinfo');
	}

	public function render()
	{
		echo get_module_title('Administration', translate('LBL_CONFIGURE_TABS'), false);

		$cpath = AppConfig::local_config_path();
		if(! is_writable($cpath)) {
			echo '<p class="error">' . str_replace('%s', $cpath, translate('MSG_CONFIG_NOT_WRITABLE', 'Configurator')) . '</p>';
		}
		
		require_once('include/layout/forms/FormGenerator.php');
		$all_tabs = AppConfig::setting('modinfo.index.all');
		$tabs = array();
		$hidden = AppConfig::setting('modinfo.index.hidden');
		foreach($all_tabs as $mod) {
			$access = AppConfig::setting("modinfo.by_name.$mod.acl.fixed.access");
			if($access == 'admin' || in_array($mod, $hidden) || ! iml($mod)) continue;
			$tabs[] = $mod;
		}
		$custom_hidden = AppConfig::setting('module_order.hidden', array());
		
		$spec = array(
			'name' => 'tabs',
			'vname' => 'LBL_CONFIGURE_TABS',
			'widget' => 'DynamicListInput',
			'show_handles' => false,
			'show_delete' => false,
			'max_depth' => 0,
			'width' => '400px',
			'colspan' => 2,
			'cols' => array(
				array(
					'name' => 'icon',
					'label' => '',
					'width' => '5%',
					'format' => 'icon',
				),
				array(
					'name' => 'name',
					'label' => 'LBL_MODULE',
					'width' => '75%',
				),
				array(
					'name' => 'hidden',
					'format' => 'check',
					'label' => 'LBL_HIDDEN',
					'width' => '20%',
				),
			),
		);
		$layout = array(
			'type' => 'configform',
			'editable' => true,
			'form_buttons' => array(
				'save' => array(
					'icon' => 'icon-accept',
					'vname' => 'LBL_SAVE_BUTTON_LABEL',
				),
				'reset' => array(
					'icon' => 'icon-delete',
					'vname' => 'LBL_BTN_RESET_TABS',
					'confirm' => 'NTC_BTN_RESET_TABS',
				),
				'cancel' => array(
					'icon' => 'icon-cancel',
					'vname' => 'LBL_CANCEL_BUTTON_LABEL',
					'url' => 'index.php?module=Administration',
				),
			),
			'sections' => array(
				array(
					'id' => 'main',
					'label_position' => 'top',
					'elements' => array(
						array(
							'name' => 'allow_override',
							'vname' => 'LBL_ALLOW_USER_TABS',
						),
						$spec,
					),
				),
			),
		);
		$result = new RowResult();
		$result->fields = array(
			'tabs' => array(
				'type' => 'varchar',
			),
			'allow_override' => array(
				'type' => 'bool',
			),
		);
		$result->row = array(
			'allow_override' => ! AppConfig::setting('module_order.disable_user_edit'),
		);
		foreach($tabs as $m) {
			$result->row['tabs'][] = array(
				'icon' => 'theme-icon module-'.$m,
				'module' => $m,
				'name' => translate('LBL_MODULE_TITLE', $m),
				'hidden' => in_array($m, $custom_hidden),
			);
		}
		array_csort($result->row['tabs'], 'name');

		
		$gen = FormGenerator::html_form(null, $layout, 'TabsForm');
		$gen->getFormObject()->addHiddenFields(
			array('module' => $_REQUEST['module'], 'action' => $_REQUEST['action'], 'wizard' => $_REQUEST['wizard'])
		);
		$gen->renderForm($result);
		echo '<div class="detailOuter">' . $gen->getResult() . '</div>';
		$gen->exportIncludes();
	}
}

