<?php

require_once 'modules/NewStudio/wizards/StudioWizardBase.php';
require_once 'modules/NewStudio/utils.php';
require_once 'modules/DynFields/DynField.php';
require_once 'include/ListView/ListViewManager.php';

class EditCustomWizard extends StudioWizardBase
{
	var $errors = array();

	public function process()
	{
		$edit_mod = array_get_default($this->params, 'edit_module');
		if($edit_mod)
			$bean_name = AppConfig::module_primary_bean($edit_mod);
		else
			$bean_name = null;
		if (isset($this->params['DeleteCustom'])) {
			if (!empty($this->params['edit'])) {
				$base = ListQuery::quick_fetch('DynField', $this->params['edit']);
				$upd = RowUpdate::for_result($base);
				$upd->markDeleted();
				
				AppConfig::db_invalidate('DynField');
				AppConfig::invalidate_cache('model');
				AppConfig::setting("model.custom_field_defs.$bean_name");
				$model = new ModelDef($bean_name);
				$model->getLinkDefinition('_cstm');
				
				return array(
					'wizard' => 'EditCustom',
					'edit_module' => $this->params['edit_module'],
				);
			}
		} elseif (!empty($this->params['save'])) {
			$fields = array(
				'data_type',
				'data_format',
				'required_option',
				'max_size',
				'audited',
				'personal_info',
				'privacy_consent',
				'ext1',
				'ext3',
				'ext4',
				'ext5',
				'ext6',
				'ext7',
				'default_value',
				'mass_update',
	            'duplicate_merge',
				'mobile_app',
				'unified_search',
                'formula'
			);
						
			if (!empty($this->params['edit'])) {
				$base = ListQuery::quick_fetch('DynField', $this->params['edit']);
			}
			if(! empty($base))
				$upd = RowUpdate::for_result($base);
			else
				$upd = RowUpdate::blank_for_model('DynField');
			
			foreach ($fields as $f) {
				$upd->set($f, array_get_default($this->params, $f));
			}
			if ($this->params['data_type'] == 'date') {
				global $timedate;
				//$upd->set('default_value', $timedate->to_db_date($this->params['default_value'], false));
			}
			$upd->set(array(
				'name' => $this->params['field_name'],
				'custom_module' => $this->params['edit_module'],
				'custom_bean' => $bean_name,
			));

			$langKey = strtoupper('LBL_CSTM_' . $this->params['field_name']);
			AppConfig::set_local("lang.strings.current.{$this->params['edit_module']}.$langKey", $this->params['label']);
			AppConfig::save_local('lang');
			$upd->set('label', $langKey);

			try {
				$upd->save();
				AppConfig::db_invalidate('DynField');
				AppConfig::invalidate_cache('model');

				// re-load custom table def
				AppConfig::setting("model.custom_field_defs.$bean_name");
				$model = new ModelDef($bean_name);
				$model->getLinkDefinition('_cstm');

				require_once 'include/database/DBChecker.php';
				$checker = new DBChecker();
				$checker->reloadModels();
				$cust_table = AppConfig::setting("model.detail.$bean_name.table_name") . '_cstm';
				$checker->checkRepairModel($cust_table, DB_CHECK_STANDARD | DB_CHECK_AUDIT, true, false);
				return array(
					'wizard' => 'EditCustom',
					'edit_module' => $this->params['edit_module'],
				);
			} catch(Exception $e) {
				$this->errors[] = $e->getMessage();
				$GLOBALS['log']->error('Error updating custom fields: '.$e->getMessage());
				pr2($this->errors, 'Error updating custom fields', true);
				exit;
			}
		}
	}

	public function render()
	{
		return render_EditCustomWizard($this);
	}

	public function renderModules()
	{
		global $mod_strings, $app_list_strings;
		$sep = translate('LBL_SEPARATOR', 'app');
		echo get_module_title('Administration', $mod_strings['LBL_EDIT_CUSTOM'], false);
		
		$cust_count = array();
		$defs = AppConfig::setting('model.custom_field_defs');
		if($defs) {
			foreach($defs as $bean => $rows)
				$cust_count[$bean] = is_countable($rows) ? count($rows) : 0;
		}
		
		$modules = AppConfig::setting('modinfo.index.normal');
		$modules = array_merge($modules, AppConfig::setting('modinfo.index.manual'));
		$mods_trans = array();
		foreach($modules as $mod)
			$mods_trans[$mod] = array_get_default($app_list_strings['moduleList'], $mod, $mod);
		asort($mods_trans);
		$url_tpl = 'index.php?module=NewStudio&wizard=EditCustom&edit_module={MODULE}';
		echo '<div class="detailOuter">';
		echo parent::renderModuleChooser($mods_trans, $url_tpl,
			'<h4 class="no-margin">' . $mod_strings['LBL_SELECT_MODULE'] . '</h4>',
			$cust_count);
		echo '</div>';
	}

	public function renderFieldsList()
	{
		global $app_list_strings, $mod_strings;
		$mod = $this->params['edit_module'];
		$sep = translate('LBL_SEPARATOR', 'app');
		if (!defined('ASYNC_ENTRY')) {
			echo get_module_title('Administration', $mod_strings['LBL_EDIT_CUSTOM'] . $sep . $app_list_strings['moduleList'][$mod], false);
		}
		$_REQUEST['custom_module'] = $mod;
		$_REQUEST['query'] = 1;
		$lv = new ListViewManager('listview', ['is_primary' => true]);
        $lv->show_create_button = false;
        $lv->show_filter = false;
		$lv->show_tabs = false;
		//$lv->show_mass_update = false;
		$lv->show_title = false;
		$qparams = ['edit_module' => $mod, 'wizard' => 'EditCustom'];
		$lv->add_query_params = $qparams;
		$lv->addHiddenFields($qparams);
		$lv->loadRequest();
		if(! $lv->initModuleView('DynFields', 'Standard'))
			ACLController::displayNoAccess();
		$buttons = array();
		$buttons['create'] = array(
			'icon' => 'icon-edit',
			'label' => translate('LBL_CREATE_BUTTON_LABEL'),
			'perform' => "SUGAR.util.navigateTo('index.php?module=NewStudio&wizard=EditCustom&edit_module=$mod&edit=');",
			'type' => 'button',
			'params' => array(
				'toplevel' => 1,
			),
		);
		$fmt = $lv->getFormatter();
		$fmt->addButtons($buttons);
        $fmt->detail_link_url = 'index.php?module=NewStudio&action=index&wizard=EditCustom&edit={ID}&edit_module=' . $mod;
        echo '<div class="detailOuter">';
		$lv->render();
		echo '</div>';
	}

	public function renderEdit()
	{
		global $app_list_strings, $mod_strings, $app_strings;
		$id = $this->params['edit'];
		$mod = $this->params['edit_module'];
		$model_name = AppConfig::module_primary_bean($mod);
		$sep = translate('LBL_SEPARATOR', 'app');
		echo get_module_title('Administration', $mod_strings['LBL_EDIT_CUSTOM'] . $sep . $app_list_strings['moduleList'][$mod], false);
		$params = array();
		
		$fields = array(
			'label',
			'max_size',
			'data_type',
			'data_format',
			'audited',
			'personal_info',
			'privacy_consent',
			'ext1',
			'ext3',
			'ext4',
			'ext5',
			'default_value',
			'mass_update',
            'formula',
            'duplicate_merge',
			'mobile_app',
			'unified_search',
            'required_option',
		);
			
		if ($id && ($cf = ListQuery::quick_fetch('DynField', $id))) {
			$params['name'] = $cf->getField('name');
			foreach($fields as $f)
				$params[$f] = $cf->getField($f);
			$ext6 = $cf->getField('ext6');
			$ext7 = $cf->getField('ext7');
			$json = getJSONObj();
			$params['ext6'] = $json->decode($ext6);
			$ext7 = $json->decode($ext7);
			if ($ext7 && is_array($ext7))
				$params['conditional_options'] = $ext7;
			$params['label'] = translate($cf->getField('label'), $this->params['edit_module']);
		}
		else
			$cf = ListQuery::quick_fetch_blank('DynField');
		if (!empty($this->params['save'])) {
			foreach ($fields as $f) {
				$params[$f] = array_get_default($this->params, $f);
			}
		}
		$add_languages = array();
		$control_fields = array();
		$all_fields = AppConfig::setting("model.fields.$model_name");
		$params['all_fields'] = array_map('strtolower', array_keys($all_fields));
		foreach ($all_fields as $f) {
			if (! isset($params['name']) || $f['name'] != $params['name']) {
				if (!empty($f['multi_select_group']) && ($f['type'] == 'enum' || $f['type'] == 'status') )
					continue;
				if ($f['type'] == 'enum' || $f['type'] == 'status' || $f['type'] == 'bool' || $f['type'] == 'multienum') {
					$control_fields[] = $f;
					if (isset($f['vname_module']))
						$add_languages[$f['vname_module']] = true;
				}
			}
		}
		$params['control_fields'] = $control_fields;
		
		$all_dom_opts = array_keys(AppConfig::setting('lang.lists.current.app', array()));
		$exclude = array_merge(
			AppConfig::setting("lang.detail.current.app.lists_generated", array()),
			AppConfig::setting("lang.detail.current.app.lists_hidden", array())
		);
		$dom_opts = array_diff($all_dom_opts, $exclude);
		sort($dom_opts);
		sort($all_dom_opts);
		$params['dom_opts'] = $dom_opts;
		$params['all_dom_opts'] = $all_dom_opts;

		$date_default_opts = array('' => '');
		$date_default_opts += AppConfig::setting('lang.lists.current.NewStudio.date_default_values', array());
		$params['date_default_opts'] = $date_default_opts;
		
		$beans = AppConfig::setting('modinfo.primary_beans');
		foreach ($beans as $mod => $bean) {
			$name = isset($GLOBALS['app_list_strings']['moduleListSingular'][$mod]) ? $GLOBALS['app_list_strings']['moduleListSingular'][$mod] : $bean;
			$params['rel_modules'][$mod] = $name;
		}

		$scan_models = array();
		$params['ref_fields_fields'] = array();
		$mod = $this->params['edit_module'];
		$model_name = AppConfig::module_primary_bean($mod);
		$rel_fields = AppConfig::setting("model.fields.{$model_name}");
		$params['ref_fields'] = array();
		foreach ($rel_fields as $f) {
			if ($f['type'] == 'ref' && !empty($f['bean_name']) && empty($f['is_related_value'])) {
				$params['ref_fields'][] = array(
					'name' => $f['name'],
					'label' => translate($f['vname'], array_get_default($f, 'vname_module', $mod)),
					'model' => $f['bean_name'],
				);
				$scan_models[$f['bean_name']] = true;
			}
		}
		foreach ($scan_models as $model => $_) {
			$mod = AppConfig::module_for_model($model);
			$params['ref_fields_fields'][$model] = array();
			$fields = $rel_fields = AppConfig::setting("model.fields.{$model}");
			foreach ($fields as $f) {
				if (($f['type'] != 'ref' || empty($f['is_related_value'])) && !empty($f['vname'])) {
					$params['ref_fields_fields'][$model][] = array(
						'name' => $f['name'],
						'label' => translate($f['vname'], array_get_default($f, 'vname_module', $mod)),
					);
				}
			}
		}


		$params['module'] = $this->params['edit_module'];

		global $pageInstance;
		$pageInstance->add_js_include('modules/NewStudio/custom.js', null, LOAD_PRIORITY_BODY);
		$json = getJSONObj();
		$paramsJS = $json->encode($params);
		$pageInstance->add_js_literal('NewStudio.init(' . $paramsJS . '); NewStudio.renderForm();', null, LOAD_PRIORITY_FOOT);
	
		$pageInstance->add_js_language($this->params['edit_module'], null, LOAD_PRIORITY_BODY);
		foreach (array_keys($add_languages) as $al)
			$pageInstance->add_js_language($al, null, LOAD_PRIORITY_BODY);

        $dataTypes = array(
			'varchar', 'text', 'int', 'double', 'percentage', 'bool', 'email', 'phone',
			'item_number',
			'enum', 'status', 'multienum', 'date', 'datetime', 'url', 'html',
			'calculated', 'ref', 'related_value', 'currency', 'name',
		);

        $typeSelect = '<select id="data_type" name="data_type" onchange="NewStudio.renderForm();" class="input-field">';
		foreach ($dataTypes as $t) {
			$sel = ($t == @$params['data_type']) ? 'selected="selected"' : '';
			$typeSelect .= '<option value="' . $t . '" ' . $sel . ' >';
			$typeSelect .= to_html(translate('LBL_TYPE_' . strtoupper($t), 'NewStudio'));
			$typeSelect .= '</option>';
		}
		$typeSelect .= '</select>';
		
		$fdef = $cf->getFieldDefinition('data_format');
        $formatSelect = '<select id="data_format" name="data_format" onchange="NewStudio.renderForm();" class="input-field">';
        $fmts = $fdef['options_limit_keys'];
        $dfmt = ! empty($params['data_format']) ? $params['data_format'] : 'varchar';
		foreach ($fmts as $t) {
			$sel = ($t == $dfmt) ? 'selected="selected"' : '';
			$formatSelect .= '<option value="' . $t . '" ' . $sel . ' >';
			$formatSelect .= to_html(translate('LBL_TYPE_' . strtoupper($t), 'NewStudio'));
			$formatSelect .= '</option>';
		}
		$formatSelect .= '</select>';
		$formatLabel = translate('COLUMN_TITLE_DATA_FORMAT', 'DynFields');
		
		$nameVal = to_html(array_get_default($params, 'name'));
		$labelVal = to_html(array_get_default($params, 'label'));
		$auditChecked = array_get_default($params, 'audited') ? 'checked="checked"' : '';
		$personalChecked = array_get_default($params, 'personal_info') ? 'checked="checked"' : '';
		$consentChecked = array_get_default($params, 'privacy_consent') ? 'checked="checked"' : '';
		$massChecked = array_get_default($params, 'mass_update') ? 'checked="checked"' : '';
		$mergeDupChecked = array_get_default($params, 'duplicate_merge') ? 'checked="checked"' : '';
		$mobileChecked = array_get_default($params, 'mobile_app') ? 'checked="checked"' : '';
		$unifiedSearchChecked = array_get_default($params, 'unified_search') ? 'checked="checked"' : '';
		$reqSelect = '<select id="required_option" name="required_option" class="input-field">';
		$sel = (array_get_default($params, 'required_option') == 'optional') ? 'selected="selected"' : '';
		$reqSelect .= "<option value=\"optional\" $sel>Optional</option>";
		$sel = (array_get_default($params, 'required_option') == 'required') ? 'selected="selected"' : '';
		$reqSelect .= "<option value=\"required\" $sel>Required</option>";
		$reqSelect .= "</select>";

		$cancelParams = "{module: 'NewStudio', action: 'index', wizard: 'EditCustom', edit_module: '{$this->params['edit_module']}'}";

		$csrf_input = create_csrf_input();		
		$html = <<<HTML
<div class="detailOuter">
<form method="post" action="index.php" id="StudioCustomForm" autocomplete="off" class="card-outer form-outer default-form">
{$csrf_input}
<input type="hidden" name="module" value="NewStudio">
<input type="hidden" name="action" value="index">
<input type="hidden" name="wizard" value="EditCustom">
<input type="hidden" name="edit_module" value="{$this->params['edit_module']}">
<input type="hidden" name="edit" value="$id">
<input type="hidden" name="save" value="1">
<input type="hidden" name="ext5" id="ext5" value="">
<input type="hidden" name="ext6" id="ext6" value="">
<input type="hidden" name="ext7" id="ext7" value="">

<div class="card-header form-buttons">
<div class="row buttons-row">
<div class="column buttons-inline">
<div class="inline-elt">
	<button type="submit" onclick="return NewStudio.save();" class="input-button"><div class="input-icon icon-accept left"></div><span class="input-label">{$app_strings['LBL_SAVE_BUTTON_LABEL']}</span></button>
</div><div class="inline-elt">
	<button type="button" class="input-button" name="DeleteCustom"><div class="input-icon icon-delete left"></div><span class="input-label">{$app_strings['LBL_DELETE_BUTTON_LABEL']}</span></button>
</div><div class="inline-elt">
	<button type="button" class="input-button" name="CancelCustom" onclick="SUGAR.util.navigateTo($cancelParams);"><div class="input-icon icon-cancel left"></div><span class="input-label">{$app_strings['LBL_CANCEL_BUTTON_LABEL']}</span></button>
</div>
<div id="custom_errors"></div>
</div></div>
</div>

<div class="form-body">
<div class="form-section">
<div class="row form-row">
<div class="column form-cell">
<div class="form-entry">
<div class="form-value">
<table>
<tbody id="custom_fields_main">
<tr>
<td class="dataLabel">{$mod_strings['LBL_DATA_TYPE']}</td>
<td class="dataField">
$typeSelect
</td></tr>

<tr id="data_format_row" style="display:none">
<td class="dataLabel">{$formatLabel}</td>
<td class="dataField">
$formatSelect
</td></tr>

<tr>
<td class="dataLabel">{$mod_strings['LBL_FIELD_NAME']}</td>
<td class="dataField"><input type="text" name="field_name" id="field_name" value="{$nameVal}" class="input-text"></td>
</tr>

<tr>
<td class="dataLabel">{$mod_strings['LBL_FIELD_LABEL']}</td>
<td class="dataField"><input type="text" name="label" id="label" value="{$labelVal}" class="input-text"></td>
</tr>

<tr id="audit_row">
<td class="dataLabel">{$mod_strings['LBL_AUDIT']}</td>
<td class="dataField"><div class="input-toggle"><input type="checkbox" name="audited" id="audited" value="1" {$auditChecked}><label for="audited" class="input-label"><div class="switch"></div></label></div></td>
</tr>

<tr id="personal_row">
<td class="dataLabel">{$mod_strings['LBL_PERSONAL_DATA']}</td>
<td class="dataField"><div class="input-toggle"><input type="checkbox" name="personal_info" id="personal_info" value="1" {$personalChecked}><label for="personal_info" class="input-label"><div class="switch"></div></label></div></td>
</tr>

<tr id="consent_row">
<td class="dataLabel">{$mod_strings['LBL_PRIVACY_CONSENT']}</td>
<td class="dataField"><div class="input-toggle"><input type="checkbox" name="privacy_consent" id="privacy_consent" value="1" {$consentChecked}><label for="privacy_consent" class="input-label"><div class="switch"></div></label></div></td>
</tr>

<tr id="mass_update_row">
<td class="dataLabel">{$mod_strings['LBL_MASSUPDATE']}</td>
<td class="dataField"><div class="input-toggle"><input type="checkbox" name="mass_update" id="mass_update" value="1" {$massChecked}><label for="mass_update" class="input-label"><div class="switch"></div></label></div></td>
</tr>

<tr id="dup_merge_row">
<td class="dataLabel">{$mod_strings['LBL_DUP_MERGE']}</td>
<td class="dataField"><div class="input-toggle"><input type="checkbox" name="duplicate_merge" id="duplicate_merge" value="1" {$mergeDupChecked}><label for="duplicate_merge" class="input-label"><div class="switch"></div></label></div></td>
</tr>

<tr id="mobile_app_row">
<td class="dataLabel">{$mod_strings['LBL_MOBILE_APP']}</td>
<td class="dataField"><div class="input-toggle"><input type="checkbox" name="mobile_app" id="mobile_app" value="1" {$mobileChecked}><label for="mobile_app" class="input-label"><div class="switch"></div></label></div></td>
</tr>

<tr id="unified_search_row">
<td class="dataLabel">{$mod_strings['LBL_UNIFIED_SEARCH']}</td>
<td class="dataField"><div class="input-toggle"><input type="checkbox" name="unified_search" id="unified_search" value="1" {$unifiedSearchChecked}><label for="unified_search" class="input-label"><div class="switch"></div></label></div></td>
</tr>

<tr>
<td class="dataLabel">{$mod_strings['LBL_REQUIRED']}</td>
<td class="dataField">$reqSelect</td>
</tr>


</tbody>

<tbody id="custom_fields_body">
</tbody>

<tbody id="custom_fields_formula" style="display: none;">
<tr>
    <td class="dataLabel">&nbsp;</td>
    <td class="dataLabel">
        <table>
            <tr>
            <td class="dataLabel"><div style="font-weight: bold;">{$mod_strings['LBL_FORMULA_FUNTIONS']}</div></td>
            <td class="dataLabel"><div style="font-weight: bold;">{$mod_strings['LBL_FORMULA_FIELDS']}</div></td>
            </tr>
            <tr>
            <td class="dataField">
                <div style="max-height: 100px; min-height: 10px !important; background: none repeat scroll 0 0 #FAFAFA;  width: 200px; overflow-x: hidden;" class="dialog-content">
                {$this->getFunctionListHtml()}
                </div>
            </td>
            <td class="dataField">
                <div style="max-height: 100px; min-height: 10px !important; width: 300px; background: none repeat scroll 0 0 #FAFAFA; overflow-x: hidden;"  class="dialog-content">
                {$this->getFieldListHtml()}
                </div>
            </td>
            </tr>
        </table>
    </td>
</tr>
</tbody>
</table>
</div></div></div></div></div></div>
</form>
HTML;
		echo $html;
	}

    private function getFieldListHtml() {
        $fields = $this->getModelFields();
        $html = '';

		$fieldsCount = is_countable($fields) ? sizeof($fields) : 0;
        for ($i = 0; $i < $fieldsCount; $i++) {
            $html .= '<div id="newfield_' .$fields[$i]. '" class="edit_layout_wrapper" onclick="NewStudio.addToFormula(this);">$' .$fields[$i]. '</div>';
        }

        return $html;
    }

    private function getFunctionListHtml() {
        require_once('include/layout/FormulaParser.php');
        $functions = FormulaParser::get_allowed_functions();
        $html = '';

        foreach ($functions as $func => $tooltip) {
            $html .= '<div id="newfunc_' .$func. '" class="edit_layout_wrapper" onclick="NewStudio.addToFormula(this);" onmouseover="return SUGAR.popups.tooltip(\''.$tooltip.'\', this);">' .$func. '</div>';
        }

        return $html;
    }

    private function getModelFields() {
        $edit_model = new ModelDef(AppConfig::module_primary_bean($this->params['edit_module']));
        return $edit_model->getAllDisplayFields();
    }
}

