{*

/**
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 */

*}

<style type="text/css">
#trashcan {
	border-radius: 0;
	border-style: dashed;
}
</style>

<script>
{literal}

EditTabsManager = new function() {
	var Dom = SUGAR.dom,
		MakeElt = Dom.makeElement;
	
	this.init = function(allModules, initTabs) {
		var outer = this.modulesOuter = Dom.queryId('moduleListOuter');
		SUGAR.ui.registerSelectOptions('tabEditModules', allModules);
		this.modLabels = {};
		for(var i = 0; i < allModules.keys.length; i++) {
			this.modLabels[allModules.keys[i]] = allModules.values[i].label;
		}
		this.moduleSelect = new SUGAR.ui.SelectList('allMods', {
			options: 'tabEditModules', rows: 20, icon_key: 'module',
			className: 'card-outer card-body panel-body input-block', flat: true
		});
		this.moduleSelect.setDragConfig({
			as_source: true,
			position_target: false, // just for dumping modules, no position indicator
			move_cursor: true,
			checkAccept: function(evt, data) {
				if(data && ! data.self && data.values.option && data.values.option.module)
					return true;
			},
			onAccept: function(evt) {
				outer.addClass('panel-active');
			},
			onLeave: function(evt) {
				outer.removeClass('panel-active');
			}
		});

		new SUGAR.ui.DragTarget('#trashcan', {
			checkAccept: function(evt, data) {
				if(data && data.values.source && data.values.source.source_input && data.values.source.source_input.id == 'allMods')
					return false;
				if(data && ! data.self && data.values.option && data.values.option.module)
					return true;
			},
		});

		outer.findFirst('.panel-body')
			.replaceWith(this.moduleSelect.render());
		this.moduleSelect.setup();
		this.tabsOuter = Dom.queryId('tabsOuter');
		this.tabs = deep_clone(initTabs);
		this.renderTabs();
	}
	
	this.renderTabs = function() {
		var row, tabIdx, tabsPerRow = 3;
		this.tabsOuter.empty();
		for(tabIdx = 0; tabIdx < this.tabs.order.length; tabIdx++) {
			if(tabIdx % tabsPerRow === 0) {
				row = MakeElt('div.row.form-row.col-unstack').appendTo(this.tabsOuter);
			}
			this.renderTab(this.tabs.order[tabIdx], row);
		}
		while(row && row.get().childNodes.length < tabsPerRow) {
			MakeElt('div.column.form-cell.form-spacer').appendTo(row);
		}
	};
	
	this.renderTab = function(tabId, row) {
		var self = this, tab = this.tabs.values[tabId],
			opts = {keys: [], values: {}, display_name: 'label'},
			modLabels = this.modLabels,
			outer, modId, menu, opts, i, remIcon, tabLabel;
		if(tab.modules) {
			for(i = 0; i < tab.modules.length; i++) {
				modId = tab.modules[i];
				opts.keys.push(modId);
				opts.values[modId] = {
					label: get_default(modLabels[modId], modId),
					module: modId
				};
			}
		}
		SUGAR.ui.registerSelectOptions('tabModules_'+tabId, opts);
		menu = new SUGAR.ui.SelectList('tabModules_'+tabId, {
			options: 'tabModules_'+tabId, rows: 10, icon_key: 'module',
			className: 'card-outer card-body panel-body menu-lg input-block', flat: true,
			placeholder: 'No module(s)'
			/*,add_remove: true, add_remove_sorted: false*/
		});
		menu.setDragConfig({
			as_source: true,
			move_cursor: true,
			checkAccept: function(evt, data) {
				if(data && data.values.option && data.values.option.module)
					return true;
			},
			afterDragContent: function(evt, drag, data) {
				if(data.state === 'complete' && ! data.self) {
					menu.removeOption(data.values.key);
					EditTabsManager.updateTabs(tabId, menu.options.getKeys());
				}
			},
			onAccept: function(evt, data) {
				outer.addClass('panel-active');
			},
			onLeave: function(evt, data) {
				outer.removeClass('panel-active');
			},
			onDrop: function(evt, data) {
				var modId = data.values.option.module,
					value = {module: modId, label: get_default(modLabels[modId], modId)},
					oldIdx = menu.getKeyIndex(modId),
					index = data.target_index;
				if(oldIdx >= 0) {
					if(index === oldIdx) return;
					if(isset(index) && index > oldIdx) index --;
					menu.removeOption(modId);
				}
				menu.addOption(modId, value, index);
				menu.setCurrentIndex(menu.getKeyIndex(modId));
				EditTabsManager.updateTabs(tabId, menu.options.getKeys());
			}
		});
		outer = MakeElt('div.tabOuter.card-outer.panel-outer.panel-default.panel-border.expand',
			[MakeElt('div.card-header.panel-header.tools-row.align-top',
				[tabLabel = MakeElt('h4.no-margin', tab.label).css('cursor', 'pointer'),
				 (remIcon = MakeElt('div.uii.uii-lg.uii-remove.active-icon'))
				]),
			 menu.render()
			]);
		remIcon.on('click', {tabGroupId: tabId}, function(evt) {
			EditTabsManager.removeGroup(evt.data.tabGroupId);
		});

		tabLabel.on('click', function(evt) {
				var edit = new SUGAR.ui.QuickText({
					size: 36});
				edit.init(tab.label, function() {
					tab.label = this.getValue();
					self.renderTabs();
				});
				edit.showPopup(0, tabLabel);
				if (edit.field)
				    edit.field.onblur = edit.field.onreturn;
				evt.stopPropagation();
		});
		MakeElt('div.column.form-cell').append(MakeElt('div.form-entry.fill', outer)).appendTo(row);
		menu.setup();
	};

	this.updateTabs = function(tabGroupId, tabs) {
		this.tabs.values[tabGroupId].modules = tabs.slice();
	};

	this.addGroup = function() {
		var lbl = 'LBL_GROUPTAB_' + (new Date()).getTime();
		this.tabs.order.push(lbl);
		this.tabs.values[lbl] = {label: 'New Group', modules: []};
		this.renderTabs();
	};

	this.removeGroup = function(tabGroupId) {
		this.tabs.order.remove(tabGroupId);
		delete this.tabs.values[tabGroupId];
		this.renderTabs();
	};

	this.sendGroupForm = function() {
		var i, j, updates = {save_group_tabs: 1, tabs: JSON.stringify(this.tabs)},
			form = SUGAR.ui.getForm('edittabs');
		if(! form) return false;
		return SUGAR.ui.sendForm(form, updates, null, true);
	}

}();

{/literal}

SUGAR.onDomReady(function() {ldelim}
	EditTabsManager.init({$modules_json}, {$tabs_json});
{rdelim});
</script>


{$title}

<div class="detailOuter">
<form name='edittabs' id='edittabs' method='POST' action='index.php' class="card-outer form-outer default-form">
<input type='hidden' name='action' value='index'>
<input type='hidden' name='module' value='NewStudio'>
<input type='hidden' name='wizard' value='GroupTabs'>
{$CSRF_TOKEN}
<div class="card-header form-buttons">
	<div class="row buttons-row">
		<div class="column buttons-inline">
			<div class="inline-elt"><button class="input-button" type="submit" onclick="return EditTabsManager.sendGroupForm();"><div class="input-icon left icon-accept"></div><span class="input-label">{$MOD.LBL_BTN_SAVEPUBLISH}</span></button></div>
			<div class="inline-elt"><button class="input-button" type="button" onclick="{literal}if(confirm(mod_string('NTC_BTN_RESET_GROUPS'))) SUGAR.ui.sendForm(this.form, {reset_group_tabs: 1}, null, true); return false;{/literal}"><div class="input-icon left icon-return"></div><span class="input-label">{$MOD.LBL_BTN_RESET_GROUPS}</span></button></div>
			<div class="inline-elt"><button class="input-button" onclick="SUGAR.util.loadUrl('index.php?module=Administration&action=index');" type="button"><div class="input-icon icon-cancel left"></div><span class="input-label">{$APP.LBL_CANCEL_BUTTON_LABEL}</span></button></div>
		</div>
		<div class="column buttons-inline align-right">
			<div class="inline-elt"><button class="input-button group-right" onclick="EditTabsManager.addGroup();" type="button"><div class="input-icon icon-add left"></div><span class="input-label">{$MOD.LBL_ADD_GROUP}</span></button></div>
		</div>
	</div>
</div>
<div class="card-body form-body">
<div class="form-section">
<div class="row form-row">

<div class="column col-3 form-cell" style="min-width: 16em">
<div class="form-entry">
	<div id="moduleListOuter" class="tabOuter panel-outer panel-dark panel-round panel-border">
	<div class="panel-header tools-row align-top">
		<h4 class="no-margin">Available modules</h4>
	</div>
	<div class="panel-subheader">
		<div id='trashcan' class="input-field input-block">
			<span class="input-label">
				<span class="uii uii-remove"></span> {$MOD.LBL_DELETE_MODULE}
			</span>
		</div>
	</div>
	<div class="panel-body"></div>
	</div>
	
</div>
</div>

<div class="column" id="tabsOuter">

<span class='error'>{$error}</span>
</div>

<div id='logDiv' style='display:none'></div>

</div><!-- /form-row -->
</div>
</div>
</form>
</div>

