<?php

require_once 'modules/NewStudio/wizards/StudioWizardBase.php';
require_once 'include/Sugar_Smarty.php';

class GroupTabsWizard extends StudioWizardBase
{
	public function render()
	{
		global $pageInstance, $mod_strings, $app_strings;
		
		$cpath = AppConfig::local_config_path();
		if(! is_writable($cpath)) {
			echo '<p class="error">' . str_replace('%s', $cpath, translate('MSG_CONFIG_NOT_WRITABLE', 'Configurator')) . '</p>';
		}
		
		$title = get_module_title('Administration', $mod_strings['LBL_CONFIGURE_GROUP_TABS'], true);

		$smarty = new Sugar_Smarty();
		$smarty->assign('tabsPerRow', 3);
		$smarty->assign('MOD', $mod_strings);
		$smarty->assign('APP', $app_strings);
		$smarty->assign('CSRF_TOKEN', create_csrf_input());
		
		global $subModuleList, $tabStructure;
		require_once('include/GroupedTabs/GroupedTabStructure.php');
		$tabManager = new GroupedTabStructure();
		$tabManager->add_home_tabs = false;
		$tabManager->add_other_tab = false;
		$tabManager->add_websites = false;
		$tabs = $tabManager->get_theme_tabs(true, null, 100, 100);
		$smarty->assign('title', $title);
		$smarty->assign('tabs', $tabs);
		$modules = $this->getAvailableModules();
		$smarty->assign('availableModuleList', $modules);
		
		$json = getJSONobj();
		
		$initTabs = ['order' => [], 'values' => []];
		foreach($tabs['tabs'] as $tabLabel => $tabInfo) {
			$tabList = array_keys($tabInfo['tabs']);
			$tabId = $tabInfo['key'];
			$initTabs['order'][] = $tabId;
			$initTabs['values'][$tabId] = [
				'label' => $tabLabel,
				'modules' => $tabList,
			];
		}
		$smarty->assign('tabs_json', $json->encode($initTabs));
		
		require_once('include/layout/forms/EditableForm.php');
		foreach($modules as &$m)
			$m['url'] = '?module='.to_html($m['module']); // enable draggable (quick hack)
		
		$modules_opts = ['maxlen' => 0];
		EditableForm::process_opts($modules, $modules_opts, 'label');
		$smarty->assign('modules_json', $json->encode($modules_opts));
		
		$smarty->display("modules/NewStudio/wizards/EditViewTabs.tpl");
	}

	public function process()
	{
		if (! empty($this->params['reset_group_tabs'])) {
			$tabGroups = AppConfig::setting('module_order.grouped', null, true);
			$hidden = AppConfig::setting('module_order.group_hidden', [], true);
		} else if(! empty($this->params['save_group_tabs'])) {
			$hidden = array_keys(AppConfig::setting('modinfo.by_name'));
			global $app_strings;
			$tabGroups = [];
			$selected_lang = (!empty($this->params['dropdown_lang'])?$this->params['dropdown_lang']:$_SESSION['authenticated_user_language']);
			$update_app_strings = [];
			$tabs = json_decode($this->params['tabs'], true);
			if(is_array($tabs) && isset($tabs['order']) && isset($tabs['values'])) {
				$order = $tabs['order'];
				$values = $tabs['values'];
				foreach($tabs['order'] as $labelID) {
					$tab = $tabs['values'][$labelID];
					$labelValue = $tab['label'];
					if(empty($GLOBALS['app_strings'][$labelID]) || $GLOBALS['app_strings'][$labelID] != $labelValue){
						$update_app_strings[$labelID] = $labelValue;
					}
					$tabGroups[$labelID] = [
						'label' => $labelID,
						'modules' => [],
					];
					if(! empty($tab['modules'])) {
						foreach($tab['modules'] as $mod_name) {
							$tabGroups[$labelID]['modules'][] = $mod_name;
							$idx = array_search($mod_name, $hidden);
							if ($idx !== false)
								unset($hidden[$idx]);
						}
					}
				}
			}
		} else
			return;
		
		if (!empty($update_app_strings)) {
			//$contents = return_custom_app_list_strings_file_contents($selected_lang);
			foreach($update_app_strings as $labelID => $labelValue) {
				AppConfig::set_local("lang.strings.$selected_lang.app.$labelID", $labelValue);
			}
			AppConfig::save_local('lang');
			AppConfig::invalidate_cache('lang');
		}
		
		if($tabGroups) {
			AppConfig::set_local('module_order.grouped', $tabGroups);
			AppConfig::set_local('module_order.group_hidden', array_values($hidden));
			AppConfig::save_local('module_order');
			AppConfig::invalidate_cache('module_order');
		}
		AppConfig::cache_reset();
		
		return array(
			'wizard' => 'GroupTabs',
		);
	}

	
	private function getAvailableModules()
	{
		global $moduleList, $app_list_strings, $modSemiInvisList;
		static $availableModules = array();
		if (!empty($availableModules)) return $availableModules;
		foreach ($moduleList as $value) {
			$availableModules[$value] = array(
				'label' => $app_list_strings['moduleList'][$value],
				'module'=> $value
			);
		}
		foreach ($modSemiInvisList as $value) {
			$availableModules[$value] = array(
				'label'=> $app_list_strings['moduleList'][$value],
				'module' => $value
			);
		}
		array_csort($availableModules, 'label');
		return $availableModules;
    }
}
