<?php

require_once 'modules/NewStudio/wizards/StudioWizardBase.php';
require_once 'include/layout/forms/FormGenerator.php';
require_once 'include/layout/FieldFormatter.php';
require_once 'include/config/format/ConfigParser.php';
require_once 'include/config/format/ConfigWriter.php';

class PersonalDataWizard extends StudioWizardBase
{
	public static $dataTypes = array(
		'varchar', 'text', 'int', 'double', 'bool', 'email',
		'enum', 'multienum', 'date', 'url', 'html', 'calculated',
		'ref', 'related_value', 'currency', 'name',
		'status', 'item_number',
		'module_name',
	);

	public function process()
	{
		$perform = array_get_default($this->params, 'record_perform');
		if (!empty($_POST['save'])) {
			return $this->save();
		}
	}

	protected function save() {
		$module = array_get_default($this->params, 'edit_module');
		$model = AppConfig::module_primary_bean($module);
		$filename = AppConfig::custom_dir()."modules/{$module}/models/bean.{$model}.php";
		if (file_exists($filename))
			$content = ConfigParser::load_file($filename);
		else
			$content = [];
		$content['detail']['personal_data'] = !!array_get_default($_POST, 'personal_data');

		$fields = array_get_default($_POST, 'fields');
		$fields = getJSONobj()->decode($fields);
		$fields = array_filter($fields, function($f) {
			return $f['orig_personal_info'] != $f['personal_info'] || $f['orig_privacy_consent'] != $f['privacy_consent'];
		});
		$defs = AppConfig::setting("model.fields.{$model}");
		foreach ($fields as $f) {
			$name = $f['field'];
			$def = $defs[$name];
			$configName = array_get_default($def, 'from_app_field', $name);
			$personal_info = !!$f['personal_info'];
			$content['fields'][$configName]['personal_info'] = $personal_info;
			$content['fields'][$configName]['privacy_consent'] = !!$f['privacy_consent'];
			if ($personal_info)
				$content['fields'][$configName]['required'] = false;
		}

		$w = new ConfigWriter;
		$w->writeFile($filename, $content);
		AppConfig::invalidate_cache('model');
		AppConfig::cache_reset();
		require_once 'include/database/DBChecker.php';
		$checker = new DBChecker();
		$checker->reloadModels();
		$checker->checkRepairModel($model, DB_CHECK_STANDARD | DB_CHECK_AUDIT, true, false);
		AppConfig::invalidate_cache('model');
		AppConfig::invalidate_cache('module_info');
		return [
			'wizard' => 'PersonalData',
		];
	}

	public function render()
	{
		global $app_list_strings;
		$module = array_get_default($this->params, 'edit_module');
		$label = '';
		if ($module) {
			$label = ' (' . array_get_default($app_list_strings['moduleList'], $module, $module) . ')';
		}
		echo get_module_title('Administration', translate('LBL_CONFIGURE_PERSONAL') . $label, false);
		if (!$module) $this->renderModules();
		else $this->renderForm();
	}

	protected function renderModules()
	{
		global $mod_strings;
		global $app_list_strings;
		$sep = translate('LBL_SEPARATOR', 'app');
		$modules = AppConfig::setting('modinfo.index.normal');
		$mods_trans = [];
		foreach ($modules as $mod) {
			if ($mod == 'DataPrivacy')
				continue;
			$mods_trans[$mod] = array_get_default($app_list_strings['moduleList'], $mod, $mod);
		}
		asort($mods_trans);
		$url_tpl = 'index.php?module=NewStudio&wizard=PersonalData&edit_module={MODULE}';
		echo '<div class="detailOuter">';
		echo parent::renderModuleChooser($mods_trans, $url_tpl, '<h4 class="no-margin">' . $mod_strings['LBL_SELECT_MODULE'] . '</h4>');
		echo '</div>';
	}
	
	protected function renderForm() {
		$layout = $this->getLayout();
		$row = new RowResult();
		$row->fields = ['fields' => ['type' => 'varchar']];
		$module = array_get_default($this->params, 'edit_module');
		$model = AppConfig::module_primary_bean($module);
		$module_is_personal = AppConfig::setting("model.detail.{$model}.personal_data");
		$row->row = [
			'personal_data' => $module_is_personal,
			'fields' => [],
		];
		$fields = $this->prepareFieldsList();
		foreach ($fields as $f) {
			$row->row['fields'][] = [
				'field' => $f['name'],
				'label' => translate(array_get_default($f, 'vname', $f['name']), array_get_default($f, 'vname_module', $module)),
				'personal_info' => array_get_default($f, 'personal_info', false),
				'orig_personal_info' => array_get_default($f, 'personal_info', false),
				'privacy_consent' => array_get_default($f, 'privacy_consent', false),
				'orig_privacy_consent' => array_get_default($f, 'privacy_consent', false),
			];
		}

		$gen = FormGenerator::html_form(null, $layout, 'Personal');
		$gen->layout->addScriptInclude('modules/NewStudio/personal.js', LOAD_PRIORITY_FOOT);
		$gen->layout->addScriptLiteral('personal_info_edit_init("Personal");');
		$copy_req_fields = ['module', 'action', 'wizard', 'edit_module'];
		foreach ($copy_req_fields as $f) {
			$gen->getFormObject()->addHiddenFields([$f => array_get_default($_REQUEST, $f)]);
		}
		$gen->getFormObject()->addHiddenFields(['save' => 1]);
		$gen->renderForm($row);
		echo '<div class="detailOuter">' . $gen->getResult() . '</div>';
		$gen->exportIncludes();
	}

	protected function getLayout() {
		$fields_spec = $this->getFieldsSpec();
		$personal_spec = $this->getPersonalSpec();
		return [
			'type' => 'configform',
			'editable' => true,
			'form_buttons' => [
				'save' => [
					'icon' => 'icon-accept',
					'vname' => 'LBL_SAVE_BUTTON_LABEL',
				],
				'cancel' => [
					'icon' => 'icon-cancel',
					'vname' => 'LBL_CANCEL_BUTTON_LABEL',
					'url' => 'index.php?module=NewStudio&wizard=PersonalData',
				],
			],
			'sections' => [
				[
					'id' => 'main',
					'label_position' => 'top',
					'elements' => [
						$personal_spec,
					],
				],
				[
					'desc' => translate('LBL_CUSTOM_NOTE', 'NewStudio'),
					'id' => 'fields',
					'label_position' => 'top',
					'elements' => [
						$fields_spec,
					],
				],
			],
		];
	}

	protected function getPersonalSpec() {
		return [
			'name' => 'personal_data',
			'vname' => 'LBL_MODULE_IS_PERSONAL',
			'vname_module' => 'NewStudio',
			'type' => 'bool',
		];
	}

	protected function getFieldsSpec() {
		return [
			'name' => 'fields',
			'vname' => 'LBL_SELECT_FIELDS_PERSONAL',
			'vname_module' => 'NewStudio',
			'widget' => 'DynamicListInput',
			'show_handles' => false,
			'show_delete' => false,
			'max_depth' => 0,
			'width' => '100%',
			'colspan' => 2,
			'cols' => [
				[
					'name' => 'label',
					'label' => translate('LBL_PERSONAL_FIELD_NAME', 'NewStudio'),
					'width' => '80%',
				],
				[
					'name' => 'personal_info',
					'label' => translate('LBL_IS_PERSONAL_FIELD', 'NewStudio'),
					'width' => '10%',
					'editable' => 1,
					'format' => 'check',
				],
				[
					'name' => 'privacy_consent',
					'label' => translate('LBL_PRIVACY_CONSENT', 'NewStudio'),
					'width' => '10%',
					'editable' => 1,
					'format' => 'check',
				],
			],
		];
	}

	protected function prepareFieldsList() {
		$module = array_get_default($this->params, 'edit_module');
		$model = AppConfig::module_primary_bean($module);
		$fields = AppConfig::setting("model.fields.$model");

		foreach ($fields as $k => $f) {
			if ($f['source']['type'] == 'custom_field')
				unset($fields[$k]);
			if (!empty($f['protected']))
				unset($fields[$k]);
			if (!empty($f['multi_select_group']) && empty($f['multi_select_count']))
				unset($fields[$k]);
			if (!empty($f['related_value_orig'])) {
				if ($f['related_value_orig']['source']['type'] == 'custom_field') {
					unset($fields[$k]);
				} else {
					$f = array(
						'name' => $f['name'],
						'vname' => $f['vname'],
						'type' => 'related_value',
						'ref' => $f['related_value_orig']['ref'],
						'field' => $f['related_value_orig']['field'],
					);
					$fields[$k] = $f;
				}
			} elseif (!in_array($f['type'], self::$dataTypes)) {
				unset($fields[$k]);
			}
		}
		return $fields;
	}
}

