<?php
require_once 'modules/NewStudio/wizards/StudioWizardBase.php';

class RenameTabsWizard extends StudioWizardBase
{
	public function process()
	{
		$perform = array_get_default($this->params, 'record_perform');
		if($perform == 'reset' || $perform == 'save') {
			$languages = $this->getLanguages();
			$tabs = $this->getAllRows($languages);
			
			require_once 'include/config/format/ConfigWriter.php';
			$json = getJSONObj();
			$cw = new ConfigWriter();

			foreach($languages as $lang => $lang_label) {
				if($perform == 'reset') {
					foreach($tabs[$lang] as $row) {
						$mod = $row['module'];
						$meta = $this->getMetaPath($lang, $mod);
						if(file_exists($meta)) {
							$detail = AppConfig::setting("lang.detail.$lang.$mod");
							$o_detail = AppConfig::setting("lang.detail.$lang.$mod", null, true);
							$detail['label'] = $row['std_label'];
							$cw->writeDiff($meta, array('detail' => $detail), array('detail' => $o_detail));
						}
					}
				}
				else {
					$input = array_get_default($this->params, $lang.'_tabs');
					if(isset($input)) {
						$input_rows = $json->decode($input);
						if($input_rows) {
							$orig = array();
							foreach($tabs[$lang] as $row)
								$orig[$row['module']] = $row;
							foreach($input_rows as $row) {
								$mod = $row['module'];
								if(! isset($orig[$mod])) continue;
								$lbl = $row['label'];
								if($lbl != $orig[$mod]['label']) {
									$detail = AppConfig::setting("lang.detail.$lang.$mod");
									$o_detail = AppConfig::setting("lang.detail.$lang.$mod", null, true);
									if($lbl == $orig[$mod]['std_label'])
										unset($detail['label']);
									else
										$detail['label'] = $lbl;
									$meta = $this->getMetaPath($lang, $mod);
									$cw->writeDiff($meta, array('detail' => $detail), array('detail' => $o_detail));
								}
							}
						}
					}
				}
			}

			AppConfig::invalidate_cache('lang');
		}
	}
	
	function getLanguages()
	{
		return AppConfig::get_languages();
	}
	
	function getMetaPath($lang, $mod)
	{
		return AppConfig::custom_dir() . "modules/$mod/language/lang.$lang.meta.php";
	}
	
	function getModules()
	{
		$all_mods = AppConfig::setting('modinfo.index.all');
		$hidden = AppConfig::setting('modinfo.index.hidden');
		$edit_mods = array();
		foreach($all_mods as $mod) {
			$access = AppConfig::setting("modinfo.by_name.$mod.acl.fixed.access");
			if($access == 'admin' || in_array($mod, $hidden) || ! iml($mod)) continue;
			else $edit_mods[] = $mod;
		}
		return $edit_mods;
	}
	
	function getRows($lang, $modules)
	{
		$ret = array();
		foreach ($modules as $mod) {
			$label = AppConfig::setting("lang.strings.$lang.$mod.LBL_MODULE_TITLE");
			$std_label = AppConfig::setting("lang.detail.$lang.$mod.label", null, true);
			$ret[] = array(
				'module' => $mod,
				'icon' => 'theme-icon module-'.$mod,
				'label' => $label,
				'std_label' => $std_label,
			);
		}
		return $ret;
	}
	
	function getAllRows($languages)
	{
		$edit_mods = $this->getModules();
		$tabs = array();
		foreach($languages as $lang => $lang_label)
			$tabs[$lang] = $this->getRows($lang, $edit_mods);
		return $tabs;
	}

	public function render()
	{
		global $mod_strings;
		$sep = translate('LBL_SEPARATOR', 'app');
		echo get_module_title('Administration', $mod_strings['LBL_MODULE_TITLE'] . $sep . $mod_strings['LBL_RENAME_TABS'], false);

		$languages = $this->getLanguages();
		$tabs = $this->getAllRows($languages);
		require_once('include/layout/forms/FormGenerator.php');
		
		$spec = array(
			'name' => 'tabs',
			'vname' => 'LBL_RENAME_TABS',
			'widget' => 'DynamicListInput',
			'show_handles' => false,
			'show_delete' => false,
			'max_depth' => 0,
			'width' => '70%',
			'colspan' => 2,
			'cols' => array(
				array(
					'name' => 'icon',
					'label' => '',
					'width' => '5%',
					'format' => 'icon',
				),
				array(
					'name' => 'std_label',
					'label' => 'LBL_MODULE',
					'width' => '45%',
				),
				array(
					'name' => 'label',
					'label' => 'LBL_LABEL',
					'width' => '50%',
					'editable' => 1,
				),
			),
		);
		$layout = array(
			'type' => 'configform',
			'editable' => true,
			'form_buttons' => array(
				'save' => array(
					'icon' => 'icon-accept',
					'vname' => 'LBL_SAVE_BUTTON_LABEL',
				),
				'reset' => array(
					'icon' => 'icon-delete',
					'vname' => 'LBL_BTN_RESET_TABS',
					'confirm' => 'NTC_BTN_RESET_TABS',
				),
				'cancel' => array(
					'icon' => 'icon-cancel',
					'vname' => 'LBL_CANCEL_BUTTON_LABEL',
					'url' => 'index.php?module=Administration',
				),
			),
			'sections' => array(
				array(
					'id' => 'main',
					'label_position' => 'top',
					'elements' => array(
						array(
							'name' => 'language',
							'vname' => 'LBL_LANGUAGE',
						),
					),
				),
			),
		);
		$result = new RowResult();
		$result->fields = array(
			'language' => array(
				'name' => 'language',
				'type' => 'enum',
				'options' => $languages,
			),
		);
		$result->row = array(
			'language' => AppConfig::setting('lang.base'),
		);
		foreach($tabs as $lang => $lang_tabs) {
			$result->fields[$lang.'_tabs'] = array('type' => 'varchar');
			$result->row[$lang.'_tabs'] = $lang_tabs;
			$spec['name'] = $lang.'_tabs';
			$layout['sections'][0]['elements'][] = array(
				'id' => $lang.'_tab',
				'type' => 'section',
				'toggle_display' => array(
					'name' => 'language',
					'value' => $lang,
				),
				'elements' => array($spec),
			);
		}

		
		$gen = FormGenerator::html_form(null, $layout, 'TabsForm');
		$gen->getFormObject()->addHiddenFields(
			array('module' => $_REQUEST['module'], 'action' => $_REQUEST['action'], 'wizard' => $_REQUEST['wizard'])
		);
		$gen->renderForm($result);
		echo '<div class="detailOuter">' . $gen->getResult() . '</div>';
		$gen->exportIncludes();
	}

}

