<?php

require_once 'modules/NewStudio/wizards/StudioWizardBase.php';
require_once 'modules/NewStudio/wizards/SubpanelLayoutEditor.php';
require_once 'include/Sugar_Smarty.php';
require_once 'include/ListView/ListViewManager.php';
require_once 'include/config/ModelDef.php';

class SubpanelWizard extends StudioWizardBase
{
	protected $model;

	var $linkModule;
	var $linkModelName;
	var $detail;

	public function __construct($params)
	{
		parent::__construct($params);
		$modelName = AppConfig::module_primary_bean($this->params['edit_module']);
		$this->model = new ModelDef($modelName);

		$link = $this->model->getLinkTargetModel($this->params['subpanel']);
		$this->linkModelName = $link->getModelName();
		$this->linkModule = $link->getModuleDir();
		$joined = $link->getJoinModel();
		if ($joined) {
			$lspec = $joined->getLinkSpec();
		} else {
			$lspec = $link->getLinkSpec();
		}
		$layout = array_get_default($lspec, 'layout', 'Standard');
		$views = array(
			"views.detail.{$this->linkModule}.subpanel.$layout",
			"views.detail.{$this->linkModule}.list.$layout",
		);
		$this->detail = AppConfig::setting($views);
	}

	public function render()
	{
		global $pageInstance;
		$pageInstance->add_js_include('modules/NewStudio/subpanels.js', null, LOAD_PRIORITY_BODY);

		$le = new SubpanelLayoutEditor($this->params['edit_module'], $this->params['subpanel'], $this->params['record'], $this->params['detail_layout']);
		$le->initFromModel($this->linkModelName, $this->params['list_id']);
		$layout = AppConfig::setting("views.layout.{$this->linkModule}.{$this->detail['type']}.{$this->detail['name']}");
		$layout = array(
			'columns_spec' => $layout['columns'],
			'sort_order' => array_get_default($layout, 'sort_order', array()),
			'name' => 'unused',
		);
		$le->loadLayoutDetails($layout, '');
		$le->disableTabs(array('general', 'filters'));
		$le->tab = 'columns';
		$pageInstance->set_partial_page();
		echo $le->render();
	}

	public function process()
	{
		if (!empty($this->params['save'])) {
			$filename = AppConfig::custom_dir() . "modules/{$this->linkModule}/new_views/subpanel.{$this->detail['name']}.php";
			if($this->params['save'] == 'reset') {
				@unlink($filename);
			} else {
				require_once 'include/config/format/ConfigWriter.php';
				$json = getJSONObj();
				$columns = $json->decode($this->params['columns']);
				$sort_order = $json->decode($this->params['sort_order']);
				foreach ($columns as &$column) {
					if($column['label'] && $column['vname']) {
						$label = translate($column['vname'], array_get_default($column, 'vname_module', ''));
						if($label === $column['label']) {
							unset($column['label']);
						}
					}
				}

				ListLayoutEditor::clean_column_input($columns);
				ListLayoutEditor::clean_sort_order_input($sort_order);

				$layout = AppConfig::setting("views.layout.{$this->linkModule}.{$this->detail['type']}.{$this->detail['name']}");
				$layout['columns'] = $columns;
				$layout['sort_order'] = $sort_order;

				$cw = new ConfigWriter;
				$cw->writeFile($filename, array(
						'detail' => array('type' => 'subpanel'),
						'layout' => $layout,
					)
				);
			}
			AppConfig::invalidate_cache('views');
			return array(
				'path' => 'async.php?module=' . $this->params['edit_module']
						. '&action=DetailView'
						. '&record=' . $this->params['record']
						. '&inline=true'
						. '&layout=' . $this->params['detail_layout'],
				'external' => true,
			);
		}
	}


}

