<?php

require_once('include/layout/forms/EditableForm.php');
require_once('include/layout/widgets/DynamicListInput.php');

class NotificationFieldsEditor {


	var $input;
	var $form;

	static $allowedFieldTypes = [
		'varchar', 'name', 'ref', 'enum', 'user_name', 'number', 'date', 'time', 'datetime',
		'bool', 'int', 'float', 'currency', 'double', 'percentage', 'rate_percent', 'text',
		'html', 'phone', 'extension_number', 'email', 'url', 'item_number', 'progress',
		'star_rating', 'skype', 'status', 'sequence_number',
	];

	function __construct($input) {
		$this->input = $input;
		$this->form = new EditableForm('configform', 'NotificationFieldsEditorForm');
		$this->form->setFormAction('index.php');
		$this->form->addHooks([
			'onsubmit' => 'NotificationFieldsEditor.preSubmit(this)',
		]);
	}

	function processRequest() {
		if (!empty($this->input['save'])) {
			$this->save();
			indexRedirect( [ 'path' => 'index.php?module=Notify&action=EditFields' ] );
		}
		$this->render();
	}

	function render() {
		global $pageInstance;
		theme_hide_side_menu(true);

		list($types, $modelFields) = $this->getTypesAndFields();
		$json = getJSONobj();
		$typesJSON = $json->encode($types);
		$fieldsJSON = $json->encode($modelFields);
		echo get_module_title('Administration', translate('LBL_CUSTOMIZE_FIELDS_TITLE', 'Notify'), true);
		echo '<div class="detailOuter">';
		echo $this->form->open();
		$this->renderHidden();
		echo '<div class="card-header form-buttons">';
		echo '<div class="row buttons-row">';
		echo '<div class="column buttons-inline">';
		$this->renderSaveButton();
		$this->renderRevertButtons();
		$this->renderCancelButton();
		echo '</div>';
		echo '<div class="column form-cell">';
		$this->renderTemplateSelect($types);
		echo '</div></div>';
		echo '</div>';
		echo '<div class="card-body form-body">';
		echo '<div class="form-section">';
		echo '<div class="row form-row">';
		echo '<div class="column form-cell col-4">';
		$this->renderFieldsList();
		echo '</div>';
		echo '<div class="column form-cell">';
		$this->renderPreviewArea();
		echo '</div>';
		echo '</div></div>';
		echo '</div>';
		echo $this->form->close();
		echo '</div>';
		$this->form->exportIncludes();
		$pageInstance->add_js_include('modules/Notify/template_fields.js', null, LOAD_PRIORITY_HEAD);
		$pageInstance->add_js_literal("NotificationFieldsEditor.setup(" . $typesJSON . ", " . $fieldsJSON . ", 'notify_action_type', 'notify-preview-iframe');", null, LOAD_PRIORITY_END);
	}

	function renderHidden() {
		echo '<input type="hidden" name="module" value="Notify">';
		echo '<input type="hidden" name="action" value="EditFields">';
		echo '<input type="hidden" name="save" value="1">';
	}

	function renderPreviewArea() {
		echo '<div style="width: 700px; overflow-y: auto; background: #FFF; margin: 0 auto">';
		echo '<iframe id="notify-preview-iframe" style="border: none; width: 700px; height: 600px; overflow-y: auto"></iframe>';
		echo '</div>';
	}

	function renderCancelButton() {
		echo '<div class="inline-elt">';
		echo '<button type="button" onclick="NotificationFieldsEditor.cancel()" class="input-button"><div class="input-icon left icon-cancel"></div><span class="input-label" >';
		echo translate('LBL_CANCEL', 'Notify'); 
		echo '</span></button></div>';
	}

	function renderSaveButton() {
		echo '<div class="inline-elt">';
		echo '<button id="save-all" type="submit" class="input-button"><div class="input-icon left icon-accept"></div><span class="input-label" >';
		echo translate('LBL_SAVE', 'Notify'); 
		echo '</span></button></div>';
	}

	function renderRevertButtons() {
		echo '<div class="inline-elt">';
		echo '<button id="revert-current" onclick="NotificationFieldsEditor.revertCurrent()" type="button" class="input-button"><div class="input-icon left icon-delete"></div><span class="input-label" >';
		echo translate('LBL_RESET_CURRENT', 'Notify'); 
		echo '</span></button></div>';
		echo '<div class="inline-elt">';
		echo '<button id="revert-all" onclick="NotificationFieldsEditor.revertAll()" type="button" class="input-button"><div class="input-icon left icon-delete" "></div><span class="input-label" >';
		echo translate('LBL_RESET_ALL', 'Notify'); 
		echo '</span></button></div>';
	}

	function renderTemplateSelect($types) {
		echo '<div class="form-entry">';
		echo '<label class="form-label" for="notify_action_type">';
		echo translate('LBL_SELECT_TEMPLATE', 'Notify') . '</label>';
		echo '<div class="form-value">';
		echo '<select class="input-field" id="notify_action_type" onchange="NotificationFieldsEditor.templateSelected()">';
		foreach ($types as $i => $type) {
			echo '<option value="' . $i . '">' . to_html($type['label']) . '</option>';
		}
		echo '</select>';
		echo ' <div class="input-field input-check"><input type="checkbox" id="notify_text_format" onchange="NotificationFieldsEditor.templateSelected()"><label class="input-label" for="notify_text_format"> ';
		echo translate('LBL_TEXT_FORMAT', 'Notify') . '</label></div>';
		echo '</div></div>';
	}

	function renderFieldsList() {
		$attrs = array(
			'name' => 'fields',
			'cols' => array(
				array(
					'name' => 'include',
					'label' => translate(''),
					'width' => '2%',
					'editable' => true,
					'format' => 'check',
				),
				array(
					'name' => 'label',
					'label' => translate('LBL_SELECT_FIELDS'),
					'width' => '100%',
					'editable' => false,
				),
			),
			'width' => '100%',
			'depth_attrib' => '',
			'show_delete' => false,
			'show_handles' => true,
			'fixed_attrib' => 'exclude',
			'onupdate' => 'NotificationFieldsEditor.updated',
		);
		$input = new DynamicListInput($attrs);
		$rows = [];
		$body = $input->render($this->form, $rows);
		$input->exportIncludes();
		echo $body;
	}

	function getTypesAndFields() {
		global $app_list_strings;
		$models = AppConfig::setting('model.index.by_type.bean');
		$types = [];
		$modelFields = [];
		foreach ($models as $m) {
			$module = AppConfig::module_for_model($m);
			$messages = AppConfig::setting("display.notify.$m.messages");
			if ($messages) {
				$modelName = $app_list_strings['moduleListSingular'][$module];
				$model = new ModelDef($m);
				$mFields = array_merge($model->getColumnFields(), $model->getCustomFields());
				$allowedFieldTypes = self::$allowedFieldTypes;
				$mFields = array_map(function($fname) use ($model, $allowedFieldTypes, $module) {
					$def = $model->getFieldDefinition($fname);
					if ($def['type'] == 'id' && isset($def['for_ref'])) {
						$fname = $def['for_ref'];
						$def = $model->getFieldDefinition($fname);
					}
					if (empty($def['vname']))
						return;
					if (in_array($def['type'], $allowedFieldTypes)) {
						return ['name' => $fname, 'label' => translate($def['vname'], array_get_default($def, 'vname_module', $module))];
					}
				}, $mFields);
				$modelFields[$m] = array_values(array_filter($mFields));
				foreach ($messages as $action => $msg) {
					$fields = array_get_default($msg, 'override_fields', array_get_default($msg, 'fields', []));
					$default_fields = array_get_default($msg, 'fields', []);
					if (empty($fields)) {
						if (isset($msg['inherit'])) {
							$parent = $messages[$msg['inherit']];
							$fields = array_get_default($parent, 'override_fields', array_get_default($parent, 'fields'));
						}
					}
					if (empty($default_fields)) {
						if (isset($msg['inherit'])) {
							$parent = $messages[$msg['inherit']];
							$default_fields = array_get_default($parent, 'fields');
						}
					}
					$types[] = [
						'label' => $modelName . ' : ' . translate_dd('activity_log_status_dom', $action, $action),
						'model' => $m,
						'action' => $action,
						'fields' => $fields,
						'default_fields' => $default_fields,
					];
				}
			}
		}

		usort($types, function($a, $b) { return strcmp($a['label'], $b['label']);});
		return [$types, $modelFields];
	}

	function save() {
		require_once 'include/config/format/ConfigWriter.php';
		$json = getJSONobj();
		$data = $json->decode($_POST['fields']);
		$GLOBALS['log']->fatal($data);

		if (!empty($data)) {
			foreach ($data as $d) {
				$model = $d['model'];
				$action = $d['action'];
				$fields = $d['fields'];
				$reset = array_get_default($d, 'reset');
				$msg = AppConfig::setting("display.notify.$model.messages.$action");
				if ($msg) {
					$path = AppConfig::custom_dir() . 'notifications/display.' . $model . '.php';
					if ($reset) {
						@unlink($path);
					} else {
						$module = AppConfig::module_for_model($model);
						$displayData = ['notify' => [ 'messages' => [ $action => [ 'override_fields' => $fields ]]]];
						$cw = new ConfigWriter();
						$cw->writeFile($path, $displayData);
					}
				}
			}
			AppConfig::invalidate_cache("display");
		}
	}
}

$editor = new NotificationFieldsEditor($_POST);
$editor->processRequest();

