
var NotificationFieldsEditor = new (function() {

	var self = this;
	var identity = function(x) { return x }

	this.setup = function(types, fields, selectId, previewId) {
		this.types = types;
		this.fields = fields;
		this.selectId = selectId;
		this.previewId = previewId;
		this.updateFieldsInput();
		this.preview();
	}

	this.templateSelected = function() {
		self.updateFieldsInput();
		self.preview();
	}

	this.revertCurrent = function() {
		var idx = parseInt(document.getElementById(this.selectId).value);
		self.types[idx].fields = self.types[idx].default_fields.map(identity);
		self.types[idx].reset = true;
		self.types[idx].edited = false;
		self.updateFieldsInput();
		self.preview();
	}

	this.revertAll = function() {
		self.types = self.types.map(function (el) {
			el.fields = el.default_fields.map(identity);
			el.reset = true;
			el.edited = false;
			return el;
		})
		self.updateFieldsInput();
		self.preview();
	}

	this.preview = function() {
		var self = this;
		var idx = parseInt(document.getElementById(this.selectId).value);
		var model = this.types[idx].model;
		var fields = this.types[idx].fields;
		var action = this.types[idx].action;
		var text = document.getElementById('notify_text_format');
		var req = new SUGAR.conn.JSONRequest('notify_preview', {silent: true}, {module: 'Notify', preview_model: model, preview_action: action, fields: fields, text: text && text.checked});
		req.fetch(function() {
			var r = this.getResult();
			if (r && r.body) {
				var src = 'data:text/html,' + encodeURIComponent(r.body);
				document.getElementById(self.previewId).src = src;
			}
		});
	}

	this.updateFieldsInput = function() {
		var select = document.getElementById(this.selectId);
		var input = SUGAR.ui.getFormInput(select.form, 'fields');
		var idx = parseInt(select.value);
		var model = this.types[idx].model;
		var fields = this.types[idx].fields;
		var allFields = this.fields[model];
		fields = fields.map(function(f) {
			var idx = f.indexOf('!');
			if (~idx) {
				return f.substr(0, idx);
			}
			return f;
		});
		var rows = allFields.map(function (field) {
			var include = !!~fields.indexOf(field.name);
			return {
				'name' : field.name,
				'include' : include,
				'exclude' : !include,
				'label' : field.label
			}
		})
		rows = rows.sort(function(a, b) {
			if (a.include && !b.include) return -1;
			if (!a.include && b.include) return 1;
			if (!a.include && !b.include) return a.label > b.label ? 1 : (a.label < b.label ? -1 : 0);
			return fields.indexOf(a.name) - fields.indexOf(b.name);
		});
		this.setRows(input, rows);
		input.renderSections();
	}

	this.setRows = function(input, _rows) {
		var rows = _rows.map(function(row, index) {
			return {idx: index, row: row}
		})
		rows = rows.sort (function(a, b) {
			if (a.row.include && !b.row.include) return -1;
			if (!a.row.include && b.row.include) return 1;
			if (!a.row.include && !b.row.include) return a.row.label > b.row.label ? 1 : (a.row.label < b.row.label ? -1 : 0);
			return a.idx - b.idx;
		})
		input.sections[0].rows = rows.map(function(el) {var row = el.row; row.exclude = !row.include; return row});
	}

	this.updated = function() {
		self.setRows(this, this.sections[0].rows);
		var select = document.getElementById(self.selectId);
		var input = SUGAR.ui.getFormInput(select.form, 'fields');
		var idx = parseInt(select.value);
		var fields = this.sections[0].rows.filter(function(row) {
			return row.include;
		}).map(function(row) {
			return row.name;
		});
		self.types[idx].reset = false;
		self.types[idx].edited = true;
		self.types[idx].fields = fields;
		self.preview();
	}

	this.preSubmit = function(form) {
		var updated = self.types.filter(function(el) { return el.reset || el.edited; })
		form.fields.value = JSON.stringify(updated);
	}

	this.cancel = function() {
		window.location.href = "index.php?module=Administration";
	}

})();
