<?php

class ForecastReport {

	var $contact;

	private $request = array();
	private $error = null;

	public function __construct($req) {
		$this->request = $req;
	}

	public function processRequest() {
		$this->render();

		echo '<br>';

		if ($this->input('process'))
			$this->process();
	}

	private function displayCountryColumn() {
		$show_country = AppConfig::setting('company.revenue_forecast_table_show_country');

		if ($this->request['report_type'] == 'Revenue Forecast Table' && $show_country) {
			return true;
		} else {
			return false;
		}
	}

	private function getLinks($result) {
		$lnks = array();
		foreach($result as $row) {
			if($this->request['report_type'] == 'Revenue Forecast Table') {
				$opp = ListQuery::quick_fetch_row('Opportunity', $row['opportunity_id']);
				$ym = date("Y-m", strtotime($row['expected_revenue_date']));
				if(array_key_exists($opp['account_id'],$lnks)) {
					if(array_key_exists($ym,$lnks[$opp['account_id']])) {
						if($this->request['weighted'] == 1)
							$lnks[$opp['account_id']][$ym] = (($row['amount_usdollar']+$lnks[$opp['account_id']][$ym])*$opp['probability'])/100;
						else
							$lnks[$opp['account_id']][$ym] = $row['amount_usdollar']+$lnks[$opp['account_id']][$ym];
					} else {
						if($this->request['weighted'] == 1)
							$lnks[$opp['account_id']][$ym] = ($row['amount_usdollar']*$opp['probability'])/100;
						else
							$lnks[$opp['account_id']][$ym] = $row['amount_usdollar'];
					}
				} else {
					if($this->request['weighted'] == 1)
						$lnks[$opp['account_id']][$ym] = ($row['amount_usdollar']*$opp['probability'])/100;
					else
						$lnks[$opp['account_id']][$ym] = $row['amount_usdollar'];
				}
			} else {
				if($this->request['data_type'] == 'Qty') {
					if($row['product_id'] != '') {
						$ym = date("Y-m", strtotime($row['expected_revenue_date']));
						if(array_key_exists($row['product_id'],$lnks)) {
							if(array_key_exists($ym,$lnks[$row['product_id']])) {
								$lnks[$row['product_id']][$ym] = $row['qty']+$lnks[$row['product_id']][$ym];
							} else {
								$lnks[$row['product_id']][$ym] = $row['qty'];
							}
						} else {
							$lnks[$row['product_id']][$ym] = $row['qty'];
						}
					}
				} else {
					if($row['product_id'] != '') {
						$ym = date("Y-m", strtotime($row['expected_revenue_date']));
						if(array_key_exists($row['product_id'],$lnks)) {
							if(array_key_exists($ym,$lnks[$row['product_id']])) {
								$lnks[$row['product_id']][$ym] = $row['amount_usdollar']+$lnks[$row['product_id']][$ym];
							} else {
								$lnks[$row['product_id']][$ym] = $row['amount_usdollar'];
							}
						} else {
							$lnks[$row['product_id']][$ym] = $row['amount_usdollar'];
						}
					}
				}
			}
		}
		return $lnks;
	}

	private function process() {
		$months = $this->request['date_to']+1;
		$date_to = date('Y-m-d', strtotime($this->request['date_from'].' + '.$months.' month')); 

		$lq = new ListQuery("RevenueForecast");
		$lq->addSimpleFilter('expected_revenue_date', $this->request['date_from'], '>=');
		$lq->addSimpleFilter('expected_revenue_date', $date_to, '<=');
		$result = $lq->fetchAll()->rows;

		$lnks = $this->getLinks($result);

		$symbol = AppConfig::setting('locale.base_currency.symbol');

		$html = '<div style="position: relative;" class="listViewOuter listViewPrimary"><div class="card-outer panel-outer listview-panel panel-system panel-border"><div class="card-body panel-body listview-body">'.translate('LBL_CURRENCY_MESSAGE', 'RevenueForecast').' '.$symbol.'<table id="forecastreport_tbl" class="listView" cellpadding="0" cellspacing="0" border="0" width="100%">
					<thead class="listViewColumns">
						<tr class="listHead">
							<th class="listViewThS1"></th>';

		if ($this->displayCountryColumn())
			$html .= '<th class="listViewThS1">'.translate('LBL_OPPORTUNITY_COUNTRY', 'RevenueForecast').'</th>';

		$period = new DatePeriod(
		     new DateTime($this->request['date_from']),
		     new DateInterval('P1M'),
		     new DateTime($date_to)
		);

		$per = array();
		foreach ($period as $key => $value) {
		    $html .= '<th class="listViewThS1">'.$value->format('Y-m').'</th>'; 
		    $per[$value->format('Y-m')] = '-';
		}
		if($this->request['data_type'] == 'Qty')
			$html .= '<th class="listViewThS1"><b>'.translate('LBL_TOTAL').'</b></th>';

		$html .= '</tr>';
		$html .= '</thead>
					<tbody>';

		$total_monthly = array();

		foreach($lnks as $k => $v) {
			$per_values = array_merge($per, $v);

			if($this->request['report_type'] == 'Revenue Forecast Table')  {
				$lnk = ListQuery::quick_fetch_row('Account', $k);

				$html .= '<tr class="listViewRow oddListRowS1">';
				$html .= '<td class="listViewTd"><a href="?module=Accounts&action=DetailView&record='.$lnk['id'].'">'.$lnk['name'].'</a></td>';

				if ($this->displayCountryColumn())
					$html .= '<td class="listViewTd">'.$lnk['billing_address_country'].'</td>';

			} else {
				$lnk = ListQuery::quick_fetch_row('Product', $k);
				$html .= '<tr class="listViewRow oddListRowS1"><td class="listViewTd"><a href="?module=ProductCatalog&action=DetailView&record='.$lnk['id'].'">'.$lnk['name'].'</a></td>';
			}
			
			if($this->request['data_type'] == 'Qty')
				$total_val=0;
			foreach($per_values as $val_k => $val_v) {
				if($val_v!='-' && $this->request['data_type'] != 'Qty')
					$val_v_d = format_number($val_v);
				else 
					$val_v_d = $val_v;
				if($this->request['data_type'] == 'Qty')
					$total_val=$total_val+$val_v_d;
				$html .= '<td class="listViewTd">'.$val_v_d.'</td>';
				if(array_key_exists($val_k,$total_monthly)) {
					$total_monthly[$val_k] = $total_monthly[$val_k]+$val_v;
				} else {
					$total_monthly[$val_k] = $val_v;
				}
			}
			if($this->request['data_type'] == 'Qty')
				$html .= '<td class="listViewTd"><b>'.$total_val.'</b></td>';
			$html .= '</tr>';
		}

		//Total row
		if($this->request['data_type'] != 'Qty') {
			$html .= '<tr class="listViewRow oddListRowS1">';
			$html .= '<td align="center" class="listViewTd"><b>'.translate('LBL_TOTAL_REVENUES').'</b></td>';

			if ($this->displayCountryColumn())
				$html .= '<td class="listViewTd">&nbsp;</td>';

			foreach($total_monthly as $k => $v) {
				if($v==0)
					$v='-';
				else if($this->request['data_type'] != 'Qty')
					$v=format_number($v);
				$html .= '<td class="listViewTd"><b>'.$v.'</b></td>';
			}
			$html .= '</tr>';
		}

		$html .= '</tbody>
				</table><button class="input-button" onclick="downloadCSV()" style="margin:10px"><div class="input-icon left icon-popup"></div><span class="input-label" id="listView-aea2-select-label">'.translate('LBL_EXPORT_CSV').'</span></button></div></div></div>';

		$html .= $this->getJS();

		echo $html;
	}

	private function getJS() {
		$delimited = array(
			'csv' => array('delim' => ',', 'replace' => false, 'ext' => 'csv', 'mime' => 'text/csv'),
			'scsv' => array('delim' => ';', 'replace' => false, 'ext' => 'csv', 'mime' => 'text/csv'),
			'tsv' => array('delim' => "\t", 'replace' => ' ', 'ext' => 'tsv', 'mime' => 'text/tab-separated-values'),
			'xlscsv' => array('delim' => ',', 'replace' => false, 'ext' => 'csv', 'mime' => 'application/vnd.ms-excel', 'no_newlines' => 'xls'),
		);

		global $locale;
		$format = AppConfig::setting('export.list_format');

		if(! isset($delimited[$format]))
			$format = 'csv';

		$format_spec = $delimited[$format];
		$charset = strtoupper($locale->getExportCharset());

		$html = '<script>
					function downloadCSV() {
						var table_id = \'forecastreport_tbl\';
					    var rows = document.querySelectorAll(\'table#\' + table_id + \' tr\');
					    var csv = [];
					    for (var i = 0; i < rows.length; i++) {
					        var row = [], cols = rows[i].querySelectorAll(\'td, th\');
					        for (var j = 0; j < cols.length; j++) {
					            var data = cols[j].innerText.replace(/(\r\n|\n|\r)/gm, \'\').replace(/(\s\s)/gm, \' \')
					            data = data.replace(/"/g, \'""\');
					            row.push(\'"\' + data + \'"\');
					        }
					        csv.push(row.join(\''.$format_spec['delim'].'\'));
					    }
					    var csv_string = csv.join(\'\n\');
					    var filename = \'export_\' + table_id + \'_\' + new Date().toLocaleDateString() + \'.'.$format_spec['ext'].'\';
					    var link = document.createElement(\'a\');
					    link.style.display = \'none\';
					    link.setAttribute(\'target\', \'_blank\');
					    link.setAttribute(\'href\', \'data:'.$format_spec['mime'].';charset='.$charset.',\uFEFF\' + encodeURIComponent(csv_string));
					    link.setAttribute(\'download\', filename);
					    document.body.appendChild(link);
					    link.click();
					    document.body.removeChild(link);
					}
					</script>';

		return $html;
	}

	private function render() {
		if ($this->error) {
			$msgs = [ [to_html($this->error), 'error'] ];
			echo display_flash_messages_default($msgs, false);
		}
		
		list($fields) = $this->getFields();
		$layout = $this->getLayout($fields);
		$result = new RowResult;
		$result->row = array();
		$result->fields = $fields;
		foreach ($fields as $f => $spec) {
			$def = null;
			if (isset($spec['contact_field']))
				$def = $this->contactField($spec['contact_field']);
			$result->row[$f] = $this->input($f, $def);
		}
		
		require_once('include/layout/FieldFormatter.php');
		require_once('include/layout/forms/FormGenerator.php');
		$model = new ModelDef('Account');
		$model->addDisplayFieldDefinitions($result->fields);
		$gen = FormGenerator::html_form($model, $layout, 'ForecastreportForm');
		$gen->getFormObject()->max_ref_width = 50;
		$gen->getFormObject()->addHiddenFields(
			array(
				'module' => 'Opportunities',
				'action' => 'ForecastReport',
				'process' => '1',
			)
		);
		$result->formatted = $result->row;
		$gen->renderForm($result);
		global $pageInstance;
       	$pageInstance->add_js_include('modules/Opportunities/revenue_table.js', null, LOAD_PRIORITY_END);
		echo $gen->getResult();
		$gen->exportIncludes();
	}

	private function input($name, $default = null) {
		return array_get_default($this->request, $name, $default);
	}

	private function getFields() {
		$fields = array(
			array(
				'date_from' => array(
					'vname' => 'LBL_EXP_REVENUE_START_DATE',
					'type' => 'date',
					'required' => true,
					'editable' => true,
				),
				'date_to' => array(
					'vname' => 'LBL_FORECAST_MONTHS',
					'type' => 'enum',
					'options' => 'dom_forecast_months',
					'required' => true,
					'editable' => true,
				),
				'report_type' => array(
					'vname' => 'LBL_FORECAST_REPORT_TYPE',
					'type' => 'enum',
					'options' => 'dom_forecast_report_type',
					'required' => true,
					'editable' => true,
					'onchange' => 'DisplayType(this.form)'
				),
				'weighted' => array(
					'vname' => 'LBL_FORECAST_WEIGHTED',
					'type' => 'bool',
					'editable' => true,
				),
				'data_type' => array(
					'vname' => 'LBL_FORECAST_DATA_TYPE',
					'type' => 'enum',
					'options' => 'dom_forecast_report_datatype',
					'editable' => true,
				),
			),
		);
		return $fields;
	}

	private function getLayout($fields) {
		$layout = array(
			'type' => 'config',
			'editable' => true,
			'buttons_position' => 'bottom',
			'form_buttons' => $this->getFormButtons(),
			'form_hooks' => array('oninit' => 'DisplayType({FORM})'),
			'sections' => array(
				array(
					'vname' => 'LBL_REVENUE_FORECAST_TABLES',
					'label_position' => 'top',
					'columns' => 2,
					'elements' => array_keys($fields),
				),
			),
		);
		return $layout;
	}

	private function getFormButtons() {
		$buttons = array(
			'generate' => array(
				'icon' => 'icon-accept',
				'label' => translate('LBL_GENERATE_REPORT', 'Opportunities'),
				'params' => array(),
				'async' => true,
				'align' => 'left',
			),
		);
		return $buttons;
	}

	private function contactField($name) {
		if ($this->contact)
			return $this->contact->getField($name);
	}
}

$auth = new ForecastReport($_REQUEST);
$auth->processRequest();
