<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

require_once('modules/Workflow/WFProcessorBase.php');

class CreateOpportunityProc extends WFProcessorBase {
	var $related_model = 'Opportunity';

	function initRelatedRecord(RowUpdate $rowUpdate) {
		$opp = parent::initRelatedRecord($rowUpdate);
		$opp->set('date_closed', $this->getRelativeDate('date_closed', false));
		$this->handleRelatedRef($rowUpdate, $opp, 'account');
		$this->handleRelatedRef($rowUpdate, $opp, 'currency');
		$this->handleRelatedValue($rowUpdate, $opp, 'amount');
		return $opp;
	}
	
	function getConfigFields() {
		$fields = parent::getConfigFields();
		
		$model = new ModelDef('Opportunity');
		
		$fields['name'] = $model->getFieldDefinition('name');
		$fields['name']['width'] = 40;
		//$fields['name']['alt_options'] = $this->getRelatedTextFields();
		$fields['name_alt'] = array('type' => 'varchar', 'hidden' => true);
		$fields['account'] = $model->getFieldDefinition('account');
		$fields['account_id'] = array('type' => 'ref_id', 'hidden' => true);
		$fields['account']['alt_options'] = $this->getRelatedRefFields('Account');
		$fields['account_alt'] = array('type' => 'varchar', 'hidden' => true);
		$fields['opportunity_type'] = $model->getFieldDefinition('opportunity_type');
		$fields['currency'] = $model->getFieldDefinition('currency');
		$fields['currency_id'] = array('type' => 'ref_id', 'hidden' => true);
		$fields['currency']['alt_options'] = $this->getRelatedRefFields('Currency');
		$fields['currency_alt'] = array('type' => 'varchar', 'hidden' => true);
		$fields['amount'] = $model->getFieldDefinition('amount');
		$fields['amount']['alt_options'] = $this->getRelatedTextFields('currency');
		$fields['amount_alt'] = array('type' => 'varchar', 'hidden' => true);
		$fields['date_closed'] = $model->getFieldDefinition('date_closed');
		$fields['date_closed']['widget'] = 'RelativeTimeInput';
		$fields['date_closed_type']['hidden'] = true;
		$fields['date_closed_count'] = array(
			'type' => 'int',
			'hidden' => true,
		);
		$fields['lead_source'] = $model->getFieldDefinition('lead_source');
		$fields['sales_stage'] = $model->getFieldDefinition('sales_stage');
		$fields['forecast_category'] = $model->getFieldDefinition('forecast_category');
		$fields['probability'] = $model->getFieldDefinition('probability');
		$fields['description'] = $model->getFieldDefinition('description');
		
		return $fields;
	}

	function beforeRenderConfigForm(HtmlFormGenerator $gen, $context, $entry_id) {
		$gen->getLayout()->addScriptInclude('modules/Opportunities/opportunity.js');
		$gen->getFormObject()->onInitForm("initOppForm({FORM})");
	}
}

