<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/*********************************************************************************
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 ********************************************************************************/
/*********************************************************************************

 * Description:  TODO: To be written.
 * Portions created by SugarCRM are Copyright (C) SugarCRM, Inc.
 * All Rights Reserved.
 * Contributor(s): ______________________________________..
 ********************************************************************************/

require_once 'data/SugarBean.php';
require_once 'modules/PDFTemplates/PDFTemplateProviderBase.php';

class PDFTemplate extends SugarBean 
{
	var $table_name = "pdf_templates";
	var $new_schema = true;
	var $object_name = "PDFTemplate";
	
	static function get_enabled_modules() {
		$ret = array();
		$mod_info = AppConfig::setting("modinfo.by_name");
		foreach ($mod_info as $mod => $info) {
			if (!empty($info['pdf_templates'])) {
				foreach (PDFTemplateProviderBase::get_template_types($mod) as $t => $l) {
					$ret[$mod . '~' . $t] = translate('LBL_MODULE_NAME', $mod) . ' : ' . $l;
				}
			} 
			if (!empty($info['basic_pdf_template'])) {
				$ret[$mod . '~basic'] = translate('LBL_MODULE_NAME', $mod) . ' : ' . translate('LBL_BASIC_TEMPLATE', 'PDFTemplates');
			}
		}
		return $ret;
	}
	
	static function after_save(RowUpdate $upd)
	{
		global $db;
		$id = $upd->getField('id');
		$order = $upd->getField('list_order');
		$module = $upd->getField('template_module');
		$query = sprintf(
			"UPDATE pdf_templates SET list_order = list_order + 1 WHERE list_order >= %d AND id != '%s' AND template_module = '%s' AND deleted = 0",
			$order, $db->quote($id), $db->quote($module)
		);
		$db->query($query);
	}

	static function before_save(RowUpdate $upd)
	{
		require_once 'modules/PDFTemplates/utils.php';
		return PDFTemplate_before_save($upd);
	}

	static function remove_deleted_elements(&$arr)
	{
		foreach ($arr as $k => &$v) {
			if (is_array($v)) {
				if (!empty($v['deleted']))
					unset($arr[$k]);
				else
					self::remove_deleted_elements($v);
				if ($k === 'elements' || $k === 'columns')
					$v = array_values($v);
			}
		}
	}
	
	static function mutateDetailButtons($detail, &$buttons)
	{
		unset($buttons['edit_custom']);
		unset($buttons['edit_layout_view']);
		unset($buttons['edit_layout_edit']);
		$hook = array(
			'file' => 'modules/PDFTemplates/PDFTemplate.php',
			'class' => 'PDFTemplate',
			'class_function' => 'editButtonDisabled',
		);
		if (isset($buttons['edit']))
			$buttons['edit']['hidden'] = array(
				'type' => 'hook',
				'hook' => $hook,
			);
		if (isset($buttons['delete']))
			$buttons['delete']['hidden'] = array(
				'type' => 'hook',
				'hook' => $hook,
			);

		if ($detail->perform == 'view' && $detail->context != 'listview') {
			$targets = self::get_copy_to_targets($_REQUEST['record']);
			if (!empty($targets)) {
				$buttons['copy_to'] = array(
					'vname' => 'LBL_COPY_TO_BUTTON_LABEL',
					'order' => 50,
					'type' => 'group',
					'icon' => 'icon-duplicate',
				);

				foreach ($targets as $i => $t) {	
					$buttons['copy_to_' . $i] = array(
						'vname' => $t['title'],
						'icon' => 'theme-icon module-' . $t['module'],
						'acl' => 'create',
						'params' => array(
							'record_perform' => 'duplicate',
							'copy_to_module' => $t['module'],
							'copy_to_type' => $t['type'],
						),
						'type' => 'button',
						'group' => 'copy_to',
					);
				}
			}
		}
	}
	
	static function init_record(RowUpdate &$upd, $input) {
		if (isset($input['copy_to_module'])) {
			$upd->set('template_module', $input['copy_to_module']);
			$upd->set('template_type', $input['copy_to_type']);
		}
		$vname = $upd->getField('vname');
		if ($vname) {
			$upd->set('vname', '');
			$upd->set('name', translate($vname, 'PDFTemplates'));
		}
	}

	static function get_copy_to_targets($id) {
		$ret = array();
		$row = ListQuery::quick_fetch_row('PDFTemplate', $id);
		if (!$row)
			return $ret;
		$modules = array('Bills', 'CreditNotes', 'Invoice',  'PurchaseOrders', 'Quotes', 'SalesOrders', 'Shipping');
		if (!in_array($row['template_module'], $modules))
			return $ret;

		$enabled = self::get_enabled_modules();
		foreach ($enabled as $key => $title) {
			list($module, $type) = explode('~', $key);
			if ($type == 'basic')
				continue;
			if (!in_array($module, $modules))
				continue;
			$ret[] = array(
				'module' => $module,
				'type' => $type,
				'title' => $title,
			);
		}
		return $ret;
	}
	
	static function editButtonDisabled(&$row, &$flag, &$val)
	{
		if (! ($row instanceof RowResult)) {
			$val = true;
			return;
		}
		$val = !!$row->getField('std');
	}

	static function translate_template(&$tpl, $module)
	{
		foreach ($tpl['sections'] as &$sec) {
			if (isset($sec['vname'])) {
				$sec['title'] = translate($sec['vname'], $module);
				unset($sec['vname']);
			}
			if ($sec['type'] == 'table') {
				$items =& $sec['columns'];
				$key = 'label';
			} else {
				$items =& $sec['elements'];
				$key = 'text';
			}
			if(isset($items))
			foreach ($items as &$item) {
				if (isset($item['vname'])) {
					$item[$key] = translate($item['vname'], $module);
					unset($item['vname']);
				}
			}
		}
	}

	static function update_defaults()
	{
		require_once 'include/database/RowUpdate.php';
		require_once 'include/config/format/ConfigParser.php';

		$lq = new ListQuery('PDFTemplate', ['list_order']);
		$lq->addSimpleFilter('std', '', true);
		$save_order = array_map(function($r) {return $r['list_order'];}, $lq->fetchAllRows());

		$data = ConfigParser::load_file('modules/PDFTemplates/defaults/defaults.php');

		$GLOBALS['db']->query('DELETE FROM pdf_templates WHERE std', true);

		foreach ($data as $row) {
			$id = md5($row['template_module'] . '_' . $row['template_type'] . '_' . $row['name']);
			$order = array_get_default($save_order, $id, array_get_default($row, 'list_order', 1));
			$row['template'] = json_encode($row['template']);
			$row['std'] = 1;
			$row['id'] = $id;
			$row['list_order'] = $order;
			$upd = RowUpdate::blank_for_model('PDFTemplate');
			$upd->set($row);
			$upd->save();
		}

		return count($data);

	}
	
	static function initialize_page_header_footer(&$layout)
	{
		if ($layout['sections'][0]['name'] != 'page_header') {
			$header = array(
				'type' => 'freeform',
				'name' => 'page_header',
				'title' => translate('LBL_PAGE_HEADER', 'PDFTemplates'),
				'height' => 60,
				'elements' => array(
					array(
						'type' => 'text',
						'x' => 0,
						'y' => 0,
						'width' => 800,
						'height' => 16,
						'text' => 'aaaaaaaaaa',
					)
				),
			);
			array_unshift($layout['sections'], $header);
		}
		$idx = count($layout['sections']) - 1;
		if ($layout['sections'][$idx]['name'] != 'page_footer') {
			$footer = array(
				'type' => 'freeform',
				'name' => 'page_footer',
				'title' => translate('LBL_PAGE_FOOTER', 'PDFTemplates'),
				'height' => 60,
				'elements' => array(),
			);
			$layout['sections'][] = $footer;
		}
	}

	public static function get_default($module, $type = null) {
	}
	
	public static function get_display_name($name, $vname) {
		if ($vname)
			return translate($vname, 'PDFTemplates');
		return $name;
	}


	public static function collectInputs(RowResult $row) {
		$inputs = [];
		$layout = json_decode($row->getField('template'), true);
		foreach ($layout['sections'] as $section) {
			if ($section['type'] !== 'freeform') continue;
			foreach ($section['elements'] as $el) {
				if ($el['type'] !== 'text') continue;
				$entries = [];
				if(preg_match_all("~<\{input(!?):([^\\|]+)(\\|(.*))?\}>~U", $el['text'], $entries)) {
					foreach ($entries[2] as $idx => $name) {
						$req = $entries[1][$idx];
						$default_value = $entries[4][$idx];
						if (!isset($inputs[$name])) {
							$inputs[$name] = ['id' => $name];
						}
						if (!isset($inputs[$name]['required']) || $req) {
							$inputs[$name]['required'] = !empty($req);
						}
						if (!isset($inputs[$name]['default_value']) && strlen($default_value)) {
							$inputs[$name]['value'] = $default_value;
						}
					}
				}
			}
		}
		return array_values($inputs);
	}

}

