<?php

/**
 *
 * Each module using the PDFTemplates should provide
 * an implementation of this interface.
 *
 */

interface PDFTemplateProvider
{
	/**
	 * Returns template types supported by this provider
	 */
	public static function template_types();
	
	/**
	 * Returns template skeleton for a template of given type.
	 */
	public static function template_skeleton($type);

	/**
	 * Returns sample data used by template editor.
	 */
	public static function template_sample_data($type);

	/**
	 * Creates an instance of PDFTemplateProvider.
	 * Different implementations may be returned
	 * for different types
	 */
	public static function create($type);

	/**
	 * Initialize the template
	 */
	public function initialize($layout, ListQuery $lq);

	/**
	 * Begins template iterations
	 */
	public function begin();

	/**
	 * Iterates over template sections.
	 * Should call $p->renderSection($section, $data)
	 * for each section to be rendered.
	 */
	public function iterate(PDFTemplatePrinter $p);
	
	/**
	 * Ends template iterations. Should release any resources
	 * allocated in begin().
	 */
	public function end();

	/**
	 * Returns data used to render the template.
	 * Data can be cached. Any cached data must be reset
	 * when initialize() is called.
	 *
	 * If section_id is specified, data for that section only
	 * is returned. Otherwise, data for all sections is returned
	 *
	 */

	//public function data($section_id = null);


	public function getFileName();

	public function getRow();
	public function getData($section = null);

	public function getNoteTitle();
	public function getEmailTitle();
	public function getSignedDocumentTitle();


}

