PDFTemplateEditor_ = function() {

	jscolor.dir = 'include/javascript/jscolor/';
	var editor = this;
	var UI = SUGAR.ui;

	var createLabelCell = function(tr, text, width) {
		var c = tr.insertCell(-1);
		if (width)
			c.width = width;
		c.style.textAlign = "right";
		c.style.verticalAlign = "top";
		c.appendChild(document.createTextNode(mod_string(text)));
	};

	var wrapCheck = function(chk) {
		var outer = createElement2('div', { className: 'input-field input-check' }, chk);
		outer.appendChild(createElement2('label', { className: 'input-label' }));
		return outer;
	};

	this.legend = $('PDFTemplateEditorLegend');

	this.element_colors = {
		text: '#DEF',
		text_border: '#679',
		image: '#DDD',
		field: '#DFD',
		field_border: '#686',
		line: '#000',
		header: '#DDD',
		resize: 'rgba(0, 30, 100, 0.6)',
		page: '#FFF',
		signature: '#DFF',
		signer_note: '#FFD',
		annotation: '#FDD'
	};

	this.legendItems = [
		{ type: 'text', label: 'Text' },
		{ type: 'field', label: 'Field' },
		{ type: 'signature', label: 'Signature Block' },
		{ type: 'signer_note', label: 'Signer\'s Note' }
	];

	this.grid_bg = 'url(modules/PDFTemplates/images/grid.png)';

	this.sectionNumber = 1;
	this.eltNumber = 1;
	this.use_real_colors = false;
	this.render_width = 800;
	this.outer_margin = '10px auto';
	this.outer_border = '20px solid #fff';
	this.orientation = 'portrait';
	this.margin_left = '';
	this.margin_top = '';

	this.setData = function(data) {
		for (var sn in data) {
			if (this.data[sn]) {
				var sec = data[sn];
				if (!Array.isArray(sec)) {
					for (var f in sec) {
						if (this.data[sn][f])
							this.data[sn][f]['value'] = sec[f];
					}
				}
			}
		}
		this.data.header.SIGNATURE = data.header.SIGNATURE;
		this.render();
	};

	this.init = function(params) {
		for (var k in params) {
			this[k] = params[k];
		};
		this.sectionsMap = {};
		this.dataContext = {};
		this.eltToSection = {};
		this.eltBinding = {};
		this.sampleData = deep_clone(this.data);
		this.use_real_colors = !this.editable;
		if (!this.sources || Array.isArray(this.sources))
			this.sources = {};
		var sources = {
			'': mod_string('LBL_PREDEFINED_FIELDS'),
			'_': module_name(this.module)
		};
		for (var i in this.sources) {
			if (this.sources.hasOwnProperty(i))
				sources[i] = this.sources[i];
		}
		this.sources = sources;

		this.renderLegend();
	};

	this.setup = function() {
		var self = this;
		SUGAR.ui.attachFormInputEvent(this.form, 'page_orientation', 'onchange', function(k, v) {
			self.orientation = k;
		});

		SUGAR.ui.attachFormInputEvent(this.form, 'margin_left', 'onchange', function(k, v) {
			self.margin_left = v;
		});

		SUGAR.ui.attachFormInputEvent(this.form, 'margin_top', 'onchange', function(k, v) {
			self.margin_top = v;
		});

		SUGAR.ui.attachFormInputEvent(this.form, 'signers', 'onupdate', function() {
			self.removeOrpanSignerBlocks();
			self.render();
		});
	};

	this.containerId = 'PDFTemplateEditorContainer';
	this.container = null;

	this.toggle_real_colors = function(elt) {
		this.use_real_colors = !this.use_real_colors;
		if (elt) {
			if (!this.use_real_colors)
				SUGAR.ui.addClass(elt, 'checked');
			else
				SUGAR.ui.removeClass(elt, 'checked');
		}
		this.render();
	};

	this.mark_dirty = function(flag) {
		SUGAR.ui.markFormDirty(this.form, flag);
	};


	this.renderLegend = function() {
		return;
		var editor = this;
		this.legendItems.forEach(function(item) {
			var div = createElement2('div', {
				style: {
					width: '50px',
					height: '50px',
					background: editor.element_colors[item.type]
				}
			}, item.label);
			editor.legend.appendChild(div);
		});
	}

	this.renderLater = function() {
		var editor = this;
		setTimeout(function() {
			editor.render();
		}, 50);
	}

	this.render = function() {
		this.sectionsMap = {};
		this.dataContext = {};
		this.eltBinding = {};
		this.container = $(this.containerId);
		this.container.innerHTML = '';
		this.container.style.width = '' + this.render_width + 'px';
		this.container.style.margin = this.outer_margin;
		this.container.style.border = this.outer_border;
		for (var i = 0; i < this.layout.sections.length; i++) {
			var section = this.layout.sections[i];
			var sectionEl = this.render_section(section);
			this.container.appendChild(sectionEl);
		}
	};

	this.render_section = function(section) {
		switch (section.type) {
			case 'table':
				return this.render_table(section);
			default:
				return this.render_free_form(section);
		}
	};

	this.render_table = function(section) {
		var section_offset = section.offset || 0;
		var self = this;
		var header_h = this.fontSize(section.header_font_size || 11, true) + 6;
		var cell_h = this.fontSize(section.cell_font_size || 11, true) + 6;

		var outer = document.createElement('div');
		if (this.editable)
			outer.style.background = this.grid_bg;
		else
			outer.style.background = this.element_colors['page'];
		outer.style.marginLeft = '' + section_offset + 'px';
		outer.style.width = '' + (this.render_width - section_offset) + 'px';

		var div = document.createElement('div');
		div.style.minHeight = '10px';
		div.style.height = '' + header_h + 'px';
		div.style.position = 'relative';
		if (section.title)
			div.title = section.title;

		var dataDiv = document.createElement('div');
		dataDiv.style.minHeight = '10px';
		dataDiv.style.position = 'relative';
		dataDiv.style.clear = 'left';
		if (section.title)
			dataDiv.title = section.title;


		var resizer = document.createElement('div');
		resizer.style.height = '4px';
		resizer.style.backgroundColor = this.element_colors['resize'];
		resizer.style.clear = 'left';

		if (this.editable) {
			var edit_icon = document.createElement('div');
			edit_icon.className = "theme-icon with-background ticon-edit_inline";
			edit_icon.style.position = 'absolute';
			edit_icon.style.right = '2px';
			edit_icon.style.top = '0px';
			edit_icon.style.cursor = 'pointer';
			edit_icon.title = mod_string('LBL_TABLE_PROPERTIES');
			div.appendChild(edit_icon);
			edit_icon.onclick = function() {
				PDFTemplateEditor.TableDialog.show(section);
			};

			var add_icon = document.createElement('div');
			add_icon.className = "theme-icon with-background ticon-plus_inline";
			add_icon.style.position = 'absolute';
			add_icon.style.right = '14px';
			add_icon.style.top = '0px';
			add_icon.style.cursor = 'pointer';
			add_icon.title = mod_string('LBL_ADD_COLUMN');
			div.appendChild(add_icon);
			add_icon.onclick = function(ev) {
				if (ev.target != add_icon)
					return;
				var w = 10;
				for (var i = 0; i < section.columns.length; i++) {
					if (!section.columns[i].deleted)
						w += section.columns[i].width + 2;
				}
				if (w >= self.render_width - section_offset) {
					alert(mod_string('LBL_TOO_MANY_COLUMNS'));
					return;
				}
				section.columns.push({
					width: 10
				});
				PDFTemplateEditor.mark_dirty();
				PDFTemplateEditor.render();
			}
		}


		if (!section.columns)
			section.columns = [];
		for (var i = 0; i < section.columns.length; i++) {
			var col = section.columns[i];
			if (!col) col = { deleted: true };
			if (col.deleted) continue;
			var fn = function(i, col) {
				var colDiv = document.createElement('div');
				var w = get_default(col.width, 15);
				if (this.editable) w -= 2;
				colDiv.setAttribute('style', 'float: left; padding: 3px 5px; box-sizing: border-box');
				colDiv.style.width = '' + w + 'px';
				colDiv.style.minHeight = '12px';
				colDiv.style.fontFamily = this.fontFamily(section.header_font);
				colDiv.style.fontSize = colDiv.style.lineHeight = this.fontSize(section.header_font_size || 11);
				colDiv.style.textAlign = col.header_align || 'left';
				colDiv.style.fontWeight = get_default(section.header_bold, 1) ? 'bold' : 'normal';
				var color = self.use_real_colors ? ('#' + (section.header_background || 'C8C8C8')) : this.element_colors.header;
				colDiv.style.backgroundColor = color;
				var color = self.use_real_colors ? ('#' + (section.header_color || '000')) : '#000';
				colDiv.style.color = color;
				colDiv.style.height = '' + header_h + 'px';
				colDiv.appendChild(document.createTextNode(col.label || ''));
				if (this.editable) {
					colDiv.style.cursor = 'pointer';
					colDiv.onclick = function() {
						self.editColumn(col, self.data[section.name]);
					};
				} else {
					var bw = get_default(section.header_border_width, 1),
						bc = get_default(section.header_border_color, '000');
					if (bw) {
						colDiv.style.border = '' + bw + 'px solid #' + bc;
					}
				}
				if (!i && this.editable) colDiv.style.borderLeft = '2px solid ' + this.element_colors['resize'];

				var dDiv = document.createElement('div');
				dDiv.setAttribute('style', 'float: left; padding: 2px 5px; box-sizing: border-box');
				dDiv.style.width = '' + w + 'px';
				dDiv.style.minHeight = '12px';

				var color = self.use_real_colors ? ('#' + (section.odd_background || 'FFF')) : '#FFF';
				dDiv.style.backgroundColor = color;
				var color = self.use_real_colors ? ('#' + (section.color || '000')) : '#000';
				dDiv.style.color = color;

				dDiv.style.fontFamily = this.fontFamily(section.cell_font);
				dDiv.style.fontSize = dDiv.style.lineHeight = this.fontSize(section.cell_font_size || 11);
				dDiv.style.height = '' + cell_h + 'px';
				dDiv.style.textAlign = col.align || 'left';
				if (col.field) {
					try {
						dDiv.appendChild(document.createTextNode(self.data[section.name][col.field].value));
					} catch (e) {
					}
				}
				if (this.editable) {
					dDiv.style.cursor = 'pointer';
					dDiv.onclick = function() {
						self.editColumn(col, self.data[section.name]);
					};
				}
				if (!i && this.editable) dDiv.style.borderLeft = '2px solid ' + this.element_colors['resize'];

				var resizeDiv = document.createElement('div');
				resizeDiv.setAttribute('style', 'float: left');
				resizeDiv.style.width = '2px';
				resizeDiv.style.height = '' + header_h + 'px';
				resizeDiv.style.minHeight = '10px';
				resizeDiv.style.backgroundColor = this.element_colors['resize'];
				if (this.editable) {
					resizeDiv.style.cursor = 'ew-resize';
					UI.addEventListener(resizeDiv, 'mousedown', function(ev) {
						this.start_column_resize(ev, colDiv, dDiv, section, i);
					}, this);
				}

				var resizeDiv2 = document.createElement('div');
				resizeDiv2.setAttribute('style', 'float: left');
				resizeDiv2.style.width = '2px';
				resizeDiv2.style.minHeight = '10px';
				resizeDiv2.style.height = '' + cell_h + 'px';
				resizeDiv2.style.backgroundColor = this.element_colors['resize'];
				if (this.editable) {
					resizeDiv2.style.cursor = 'ew-resize';
					UI.addEventListener(resizeDiv2, 'mousedown', function(ev) {
						this.start_column_resize(ev, colDiv, dDiv, section, i);
					}, this);
				}

				div.appendChild(colDiv);
				if (this.editable)
					div.appendChild(resizeDiv);
				dataDiv.appendChild(dDiv);
				if (this.editable)
					dataDiv.appendChild(resizeDiv2);

			}
			fn.call(this, i, col);
		}

		outer.appendChild(div);
		outer.appendChild(dataDiv);
		if (this.editable)
			outer.appendChild(resizer);

		return outer;
	};

	this.render_free_form = function(section) {
		var outer = document.createElement('div');
		if (this.editable)
			outer.style.background = this.grid_bg;
		else
			outer.style.background = this.element_colors['page'];
		outer.style.width = '' + this.render_width + 'px';
		var div = document.createElement('div');
		if (section.title)
			div.title = section.title;
		div.id = 'section_' + (this.sectionNumber++);
		this.sectionsMap[div.id] = section;
		div.style.width = '' + this.render_width + 'px';
		div.style.height = section.height + 'px';
		div.style.position = 'relative';

		var self = this;

		var sigWidth, sigHeight;

		if (this.editable) {
			var o = {
				keys: ['text', 'image', 'field', 'signature', 'signer_note', 'annotation', 'line'],
				values: [
					mod_string('LBL_ELEMENT_TYPE_TEXT'),
					mod_string('LBL_ELEMENT_TYPE_IMAGE'),
					mod_string('LBL_ELEMENT_TYPE_DATA_FIELD'),
					mod_string('LBL_ELEMENT_TYPE_SIGNATURE'),
					mod_string('LBL_ELEMENT_TYPE_SIGNER_NOTE'),
					mod_string('LBL_ELEMENT_TYPE_ANNOTATION'),
					mod_string('LBL_ELEMENT_TYPE_LINE')
				]
			};
			var m = new SUGAR.ui.MenuSource(
				div,
				{
					options: o,
					width: 200,
					target_offset: { from_cursor: true, left: -10 },
					only_this_target: true
				}
			);
			m.setup();
			div.onclick = function(ev) {
				if (ev.target != div) {
					ev.stopPropagation();
					return;
				}
				var x = ev.offsetX ? (ev.offsetX) : ev.pageX - div.offsetLeft;
				var y = ev.offsetY ? (ev.offsetY) : ev.pageY - div.offsetTop;
				x -= (x % 5); y -= (y % 5);
				m.onchange = function(k, v) {
					var newEl = {
						x: x, y: y,
						type: k,
						width: 20, height: 20,
					};
					if (k == 'line') {
						newEl.line_width = 2;
						newEl.color = '000000';
					} else if (k == 'signer_note') {
						newEl.label = 'Signer\'s note';
					} else if (k == 'annotation') {
						newEl.text = '';
						newEl.signer_refs = [];
					}

					self.sectionsMap[div.id].elements.push(newEl);
					self.mark_dirty();
					self.render();
				};
			};
		}

		var dataKey = section.data_id || section.name;
		if (!section.elements)
			section.elements = [];
		for (var i = 0; i < section.elements.length; i++) {
			var fn = function(elNum) {
				var elDef = section.elements[elNum];
				if (elDef.deleted)
					return;

				var el = document.createElement('div'),
					x_offs = 0, y_offs = 0;
				el.style.position = 'absolute';
				if (elDef.bold)
					el.style.fontWeight = 'bold';
				if (elDef.italic)
					el.style.fontStyle = 'italic';
				if (elDef.type == 'line') {
					var w = Math.max(1, parseInt(get_default(elDef.line_width, 1), 10));
					if (this.editable && window.devicePixelRatio > 1) w = Math.max(w, 2);
					el.style.left = '' + elDef.x + 'px';
					el.style.top = '' + (elDef.y - 2) + 'px';
					el.style.height = (w + 5) + 'px';
					el.style.backgroundColor = 'transparent';
					el.style.width = '' + elDef.width + 'px';
					y_offs = -2;

					var line = document.createElement('div');
					line.style.position = 'absolute';
					line.style.width = '100%';
					line.style.height = w + 'px';
					line.style.top = '2px';
					line.style.backgroundColor = '#' + get_default(elDef.color, '000');
					el.appendChild(line);
				} else {
					el.style.left = '' + elDef.x + 'px';
					el.style.top = '' + elDef.y + 'px';
					if (elDef.type === 'signature') {
						sigWidth = this.signature_size.width * 1.4;
						sigHeight = this.signature_size.height * 1.4;
						el.style.height = '' + sigHeight + 'px';
						el.style.width = '' + sigWidth + 'px';
					} else {
						el.style.height = '' + elDef.height + 'px';
						el.style.width = '' + elDef.width + 'px';
					}
					el.style.boxSizing = 'border-box';
					el.style.padding = '1px 0 0 1px';
					var borderWidth = get_default(elDef.border_width, 0);
					var borderColor = '#' + get_default(elDef.border_color, '000');
					if (!self.use_real_colors) {
						borderWidth = Math.max(1, borderWidth);
						borderColor = get_default(this.element_colors[elDef.type + '_border'], '#000');
					}
					if (borderWidth > 0) {
						el.style.border = 'solid ' + borderWidth + 'px ' + borderColor;
					}
					var color = self.use_real_colors ? ('#' + (elDef.background || 'FFF')) : this.element_colors[elDef.type];
					el.style.backgroundColor = color;
					el.style.color = self.use_real_colors ? ('#' + (elDef.color || '000')) : '#000';
				}
				div.appendChild(el);

				if (elDef.type == 'image') {
					var img = document.createElement('img');
					var sampleSignature = this.data[dataKey]['SIGNATURE'];
					if (elDef.signature_image) {
						img.src = sampleSignature || PDFTemplateEditor.signature;
					} else {
						img.src = elDef.pdf_logo ? PDFTemplateEditor.pdf_logo : elDef.src;
					}
					img.style.width = "100%";
					img.style.height = "100%";
					el.appendChild(img);
				} else if (elDef.type == 'text') {
					var txt = document.createElement('div');
					txt.style.width = "100%";
					txt.style.height = "100%";
					txt.style.overflow = "hidden";
					txt.style.fontSize = txt.style.lineHeight = this.fontSize(elDef.size || 10);
					if (elDef.line_height)
						txt.style.lineHeight = elDef.line_height;
					var text = this.replaceFields(elDef.text || '', dataKey);
					var lines = text.split(/\r?\n/);
					for (var j = 0; j < lines.length; j++) {
						txt.appendChild(document.createTextNode(lines[j]));
						txt.appendChild(document.createElement('br'));
					}
					el.style.textAlign = elDef.align || 'left';
					el.style.fontFamily = this.fontFamily(elDef.font);
					el.appendChild(txt);
				} else if (elDef.type == 'signature') {
					var txt = createElement2('div', {
						style: {
							width: "100%",
							height: "100%",
							overflow: "hidden",
							fontSize: this.fontSize(12),
							lineHeight: this.fontSize(12),
							textAlign: 'center'
						}
					}, this.signerRole(elDef.signer_ref));
					el.appendChild(txt);
				} else if (elDef.type == 'signer_note') {
					var txt = createElement2('div', {
						style: {
							width: "100%",
							height: "100%",
							overflow: "hidden",
							fontSize: this.fontSize(12),
							lineHeight: this.fontSize(12),
							whiteSpace: 'pre'
						}
					}, (elDef.label || '') + "\n" + this.signerRole(elDef.signer_ref));
					el.appendChild(txt);
				} else if (elDef.type == 'annotation') {
					var txt = createElement2('div', {
						style: {
							width: "100%",
							height: "100%",
							overflow: "hidden",
							fontSize: this.fontSize(12),
							lineHeight: this.fontSize(12)
						}
					}, elDef.text);
					el.appendChild(txt);
				} else if (elDef.type == 'field') {
					var txt = document.createElement('div');
					txt.style.width = "100%";
					txt.style.height = "100%";
					txt.style.overflow = "hidden";
					txt.style.fontSize = txt.style.lineHeight = this.fontSize(elDef.size || 10);
					if (elDef.line_height)
						txt.style.lineHeight = elDef.line_height;
					el.style.fontFamily = this.fontFamily(elDef.font);
					el.style.textAlign = elDef.align || 'left';
					if (elDef.field) {
						try {
							var lines = (this.data[dataKey][elDef.field].value || '').split(/\r?\n/);
							for (var j = 0; j < lines.length; j++) {
								txt.appendChild(document.createTextNode(lines[j]));
								txt.appendChild(document.createElement('br'));
							}
							el.title = this.data[dataKey][elDef.field].label;
						} catch (e) {
						}
					}
					el.appendChild(txt);
				}


				if (this.editable) {
					el.style.cursor = 'pointer';
					UI.addEventListener(el, 'mousedown', function(ev) {
						this.start_el_move(ev, el, div, x_offs, y_offs);
					}, this);
					if (elDef.type !== 'signature') {
						var vg = document.createElement('div');
						vg.style.background = 'url(modules/PDFTemplates/images/grip_vertical.png) bottom right no-repeat';
						vg.style.position = 'absolute';
						vg.style.width = '8px';
						vg.style.height = '8px';
						vg.style.bottom = '0';
						vg.style.right = '0';
						vg.style.cursor = 'nwse-resize';
						UI.addEventListener(vg, 'mousedown', function(ev) {
							this.size_grip_down(ev, el, div);
						}, this);

						el.appendChild(vg);
					}
				}
				el.id = 'elt_' + (this.eltNumber++);
				this.eltBinding[el.id] = elDef;
				this.eltToSection[el.id] = dataKey;
				var contextKey = dataKey;
				if (contextKey == 'page_header')
					contextKey = 'header';
				this.dataContext[el.id] = this.data[contextKey] || {};

			};
			fn.call(this, i);
		}

		var resizer = document.createElement('div');
		resizer.style.height = '4px';
		resizer.style.backgroundColor = this.element_colors['resize'];
		if (this.editable) {
			resizer.style.cursor = 'ns-resize';
			UI.addEventListener(resizer, 'mousedown', function(ev) {
				this.start_section_resize(ev, div);
			}, this);
		}

		outer.appendChild(div);
		if (this.editable)
			outer.appendChild(resizer);
		return outer;
	};

	this.colorFromRGB = function(c) {
		return 'rgb(' + c[0] + ',' + c[1] + ',' + c[2] + ')';
	};

	this.replaceFields = function(text, dataKey) {
		var self = this;
		return text.replaceAll(/<\{field:(.*?)\}>/g, function(match, fieldName) {
			var elt = self.data[dataKey][fieldName] || {};
			var value = elt.value || '';
			return value;
		});
	}

	this.size_grip_down = function(ev, target, container) {
		this.resizingElement = target;
		this.resizingContainer = container;
		this.resizeStart = { x: ev.screenX, y: ev.screenY, zIndex: target.style.zIndex };
		this.resizePos = { x: this.trimPx(target.style.left), y: this.trimPx(target.style.top) };
		this.resizeSize = { x: this.trimPx(target.style.width), y: this.trimPx(target.style.height) };
		this.resizeLimit = { x: this.trimPx(container.style.width), y: this.trimPx(container.style.height) };
		target.style.zIndex = 199;
		UI.addEventListener(document, 'mousemove', this.size_grip_move, this);
		UI.addEventListener(document, 'mouseup', this.size_grip_up, this);
		UI.stopEvent(ev);
		this.add_info_box(target, true);
		return false;
	};

	this.start_column_resize = function(ev, column, column2, section, idx) {
		var columns = section.columns;
		var offset = section.offset || 0;
		this.resizingElement = column;
		this.resizingElement2 = column2;
		this.resizeStart = ev.screenX;
		this.resizeSize = this.trimPx(column.style.width);
		this.resizeColumns = columns;
		this.resizeIdx = idx;

		var total = 0;
		for (var i = 0; i < columns.length; i++) {
			if (i != idx && !columns[i].deleted) total += columns[i].width;
		}
		this.resizeLimit = this.render_width - total - 2 - offset;

		UI.addEventListener(document, 'mousemove', this.column_resize, this);
		UI.addEventListener(document, 'mouseup', this.column_end_resize, this);
		UI.stopEvent(ev);
		return false;
	};

	this.start_section_resize = function(ev, section) {
		this.resizingElement = section;
		this.resizeStart = ev.screenY;
		this.resizePos = this.trimPx(section.style.top);
		this.resizeSize = this.trimPx(section.style.height);
		this.resizeLimit = 10;
		for (var i = 0; i < section.childNodes.length; i++) {
			var child = section.childNodes[i];
			if (child.tagName == 'DIV') {
				var end = this.trimPx(child.style.top) + this.trimPx(child.style.height);
				if (end > this.resizeLimit)
					this.resizeLimit = end;
			}
		}
		UI.addEventListener(document, 'mousemove', this.section_resize, this);
		UI.addEventListener(document, 'mouseup', this.section_end_resize, this);
		UI.stopEvent(ev);
		return false;
	};

	this.add_info_box = function(el, resize, pos) {
		var div = document.createElement('div');
		div.style.boxSizing = 'border-box';
		div.style.width = '80px';
		div.style.height = '14px';
		div.style.backgroundColor = 'rgba(255, 80, 80, 0.7)';
		div.style.color = '#FFF';
		div.style.fontSize = '10px';
		div.style.lineHeight = '10px';
		div.style.position = 'absolute';
		div.style.zIndex = 200;
		div.style.padding = '2px 5px';
		div.style.textAlign = 'right';
		div.style.left = '' + (parseInt(el.style.left, 10) - 80) + 'px';
		div.style.top = '' + (parseInt(el.style.top, 10) - 14) + 'px';
		el.parentElement.appendChild(div);
		this.infoBox = div;
		this.update_info_box(el, resize, pos);
		UI.addEventListener(div, 'mouseup', this.elt_up, this);
	};

	this.remove_info_box = function() {
		if (this.infoBox) {
			this.infoBox.parentElement.removeChild(this.infoBox);
			this.infoBox = null;
		}
	};

	this.update_info_box = function(el, size, pos) {
		var elSize = '' + this.trimPx(el.style.width) + 'x' + this.trimPx(el.style.height);

		var x = this.trimPx(el.style.left);
		var y = this.trimPx(el.style.top);
		if (pos && pos.x_offs) x -= pos.x_offs;
		if (pos && pos.y_offs) y -= pos.y_offs;

		var elPos = '@' + x + ',' + y;

		if (this.infoBox) {
			this.infoBox.innerHTML = '';
			if (size) {
				this.infoBox.appendChild(document.createTextNode(elSize));
			} else {
				this.infoBox.style.left = '' + (parseInt(el.style.left, 10) - 80) + 'px';
				this.infoBox.style.top = '' + (parseInt(el.style.top, 10) - 14) + 'px';
				this.infoBox.appendChild(document.createTextNode(elPos));
			}
		}

	};

	this.start_el_move = function(ev, target, container, x_offs, y_offs) {
		if (!ev || (ev.which || ev.button) != 1 || !target) return;
		this.resizingElement = target;
		this.resizingContainer = container;
		this.resizeStart = { x: ev.screenX, y: ev.screenY, zIndex: target.style.zIndex };
		this.resizePos = { x: this.trimPx(target.style.left), y: this.trimPx(target.style.top), x_offs: get_default(x_offs, 0), y_offs: get_default(y_offs, 0) };
		this.resizeSize = { x: this.trimPx(target.style.width), y: this.trimPx(target.style.height) };
		this.resizeLimit = { x: this.trimPx(container.style.width), y: this.trimPx(container.style.height) };
		this.wasMoved = false;
		this.add_info_box(target, false, this.resizePos);
		target.style.zIndex = 199;
		UI.addEventListener(document, 'mousemove', this.elt_move, this);
		UI.addEventListener(document, 'mouseup', this.elt_end_move, this);
		UI.addEventListener(document, 'keydown', this.elt_keydown, this);
		UI.addEventListener(target, 'mouseup', this.elt_up, this);
		UI.stopEvent(ev);
		return false;
	};

	this.size_grip_up = function(ev) {
		UI.removeEventListener(document, 'mousemove', this.size_grip_move);
		UI.removeEventListener(document, 'mouseup', this.size_grip_up);
		this.resizingElement.style.zIndex = this.resizeStart.zIndex || '';
		this.remove_info_box();
	};

	this.elt_end_move = function(ev) {
		UI.removeEventListener(document, 'mousemove', this.elt_move);
		UI.removeEventListener(document, 'mouseup', this.elt_end_move);
		UI.removeEventListener(document, 'keydown', this.elt_keydown);
		UI.removeEventListener(this.resizingElement, 'mouseup', this.elt_up);
		this.resizingElement.style.zIndex = this.resizeStart.zIndex || '';
		UI.stopEvent(ev);
		this.remove_info_box();
	};

	this.column_end_resize = function(ev) {
		UI.removeEventListener(document, 'mousemove', this.column_resize);
		UI.removeEventListener(document, 'mouseup', this.column_end_resize);
	};

	this.section_end_resize = function(ev) {
		UI.removeEventListener(document, 'mousemove', this.section_resize);
		UI.removeEventListener(document, 'mouseup', this.section_end_resize);
	};

	this.elt_up = function(ev) {
		if (!this.wasMoved) {
			var target = this.resizingElement;
			if (this.eltBinding[target.id]) {
				var type = this.eltBinding[target.id].type;
				if (type == 'text') {
					this.editText(target);
				} else if (type == 'field') {
					this.editField(target);
				} else if (type == 'image') {
					this.editImage(target);
				} else if (type == 'line') {
					this.editLine(target);
				} else if (type == 'signature') {
					this.editSignatureBlock(target);
				} else if (type == 'signer_note') {
					this.editSignerNote(target);
				} else if (type == 'annotation') {
					this.editAnnotation(target);
				}
			}
		}
		this.wasMoved = true;
	};

	this.column_resize = function(ev) {
		var x = ev.screenX - this.resizeStart + this.resizeSize;
		if (x < 10)
			return;
		if (x > this.resizeLimit)
			return;
		this.resizingElement.style.width = '' + x + 'px';
		this.resizingElement2.style.width = '' + x + 'px';
		this.resizeColumns[this.resizeIdx].width = x + 2;
	};

	this.section_resize = function(ev) {
		var y = ev.screenY - this.resizeStart + this.resizeSize;
		if (y >= this.resizeLimit) {
			this.resizingElement.style.height = '' + y + 'px';
			this.sectionsMap[this.resizingElement.id].height = y;
		}
	};

	this.elt_move = function(ev) {
		this.wasMoved = true;
		var x = ev.screenX - this.resizeStart.x + this.resizePos.x;
		var y = ev.screenY - this.resizeStart.y + this.resizePos.y;
		x = this.restrict_offset(ev, this.resizeSize.x, x, this.resizeLimit.x, 5);
		y = this.restrict_offset(ev, this.resizeSize.y, y, this.resizeLimit.y, 5);
		if (isset(x)) {
			this.resizingElement.style.left = '' + (x + this.resizePos.x_offs) + 'px';
			this.eltBinding[this.resizingElement.id].x = x;
		}
		if (isset(y)) {
			this.resizingElement.style.top = '' + (y + this.resizePos.y_offs) + 'px';
			this.eltBinding[this.resizingElement.id].y = y;
		}
		this.update_info_box(this.resizingElement, false, this.resizePos);
	};

	this.elt_keydown = function(ev) {
		if (ev && ev.keyCode == 27) {
			// cancel move/resize here
		}
	};

	this.restrict_offset = function(ev, base, s, limit, step, min) {
		var offs;
		if (s >= 0) {
			if (base + s > limit) s = limit - base;
			if (!ev.shiftKey && step) {
				offs = s % step;
				if (offs) s += Math.round(offs / step) * step - offs;
			}
			if (!min || s >= min) return s;
		}
	};

	this.size_grip_move = function(ev) {
		var w = ev.screenX - this.resizeStart.x + this.resizeSize.x;
		var h = ev.screenY - this.resizeStart.y + this.resizeSize.y;
		w = this.restrict_offset(ev, this.resizePos.x, w, this.resizeLimit.x, 5, 15);
		h = this.restrict_offset(ev, this.resizePos.y, h, this.resizeLimit.y, 5, 15);
		if (w) {
			this.resizingElement.style.width = '' + w + 'px';
			this.eltBinding[this.resizingElement.id].width = w;
		}
		if (this.eltBinding[this.resizingElement.id].type != 'line') {
			if (h) {
				this.resizingElement.style.height = '' + h + 'px';
				this.eltBinding[this.resizingElement.id].height = h;
			}
		}
		this.update_info_box(this.resizingElement, true);
	};

	this.trimPx = function(size) {
		var ret = parseInt(size.replace('px', ''), 10);
		if (!ret)
			ret = 0;
		return ret;
	};

	this.editSignatureBlock = function(elt) {
		this.SignatureBlockDialog.show(this.eltBinding[elt.id]);
	};

	this.editAnnotation = function(elt) {
		this.AnnotationDialog.show(this.eltBinding[elt.id]);
	};

	this.editSignerNote = function(elt) {
		this.SignerNoteDialog.show(this.eltBinding[elt.id]);
	};

	this.editText = function(elt) {
		this.TextDialog.show(this.eltBinding[elt.id], elt.id);
	};

	this.editField = function(elt) {
		this.FieldDialog.show(this.eltBinding[elt.id], this.dataContext[elt.id], this.eltToSection[elt.id]);
	};

	this.editImage = function(elt) {
		this.ImageDialog.show(this.eltBinding[elt.id], this.dataContext[elt.id]);
	};

	this.editLine = function(elt) {
		this.LineDialog.show(this.eltBinding[elt.id], this.dataContext[elt.id]);
	};

	this.displayFieldSelector = function(input, eltId) {
		this.FieldSelectorDialog.show(input, this.dataContext[eltId], this.eltToSection[eltId]);
	};

	this.editColumn = function(col, dataCtx) {
		this.ColumnDialog.show(col, dataCtx);
	};

	this.encode = function() {
		return JSON.stringify(this.layout);
	};

	this.beforeSubmitForm = function(form) {
		var input = form.template;
		if (!input) {
			input = document.createElement('input');
			input.setAttribute('type', 'hidden');
			input.setAttribute('name', 'template');
			form.appendChild(input);
		}
		input.value = JSON.stringify(this.layout);
	}

	this.createFontSelect = function() {
		var fontSelect = createElement2('select', { className: 'input-field' });
		var fonts = {
			almohanad: 'AlMohanad',
			courier: 'Courier',
			freemono: 'FreeMono',
			freesans: 'FreeSans',
			freeserif: 'FreeSerif',
			helvetica: 'Helvetica',
			times: 'Times',
			vera: 'Bitstream Vera Sans',
			verase: 'Bitstream Vera Serif',
			veramo: 'Bitstream Vera Mono',
			sans: 'Sans Serif',
			serif: 'Serif'
		};
		for (var i in fonts) {
			var font = fonts[i];
			var optionEl = document.createElement("option");
			optionEl.value = i;
			optionEl.innerHTML = font;
			fontSelect.appendChild(optionEl);
		}
		return fontSelect;
	};

	this.createAlignSelect = function() {
		var alignSelect = createElement2('select', { className: 'input-field' });
		var opts = { left: 'LBL_ALIGN_LEFT', center: 'LBL_ALIGN_CENTER', right: 'LBL_ALIGN_RIGHT' };
		for (var i in opts) {
			var opt = opts[i];
			var optionEl = document.createElement("option");
			optionEl.value = i;
			optionEl.innerHTML = mod_string(opt);
			alignSelect.appendChild(optionEl);
		}
		return alignSelect;
	};

	this.createColorPicker = function() {
		var input = document.createElement('input');
		input.autocomplete = 'off';
		input.color = new jscolor.color(input);
		input.onblur = function() {
			this.color.hidePicker();
		};
		return input;
	};

	this.createFontSizeSelect = function() {
		sizeSelect = createElement2('select', { className: 'input-field' });
		var sizes = [8, 9, 10, 11, 12, 14, 16, 18, 20, 24, 30, 36, 48, 60, 72, 96];
		for (var i = 0; i < sizes.length; i++) {
			var size = sizes[i];
			var optionEl = document.createElement("option");
			optionEl.value = size;
			optionEl.innerHTML = size;
			sizeSelect.appendChild(optionEl);
		}
		return sizeSelect;
	};

	this.createLineHeightSelect = function() {
		sizeSelect = createElement2('select', { className: 'input-field' });
		var sizes = ['0.9', '0.95', '1.0', '1.1', '1.2', '1.3', '1.4', '1.5', '1.6', '1.7', '1.8', '1.9', '2.0'];
		for (var i = 0; i < sizes.length; i++) {
			var size = sizes[i];
			var optionEl = document.createElement("option");
			optionEl.value = size;
			optionEl.innerHTML = size;
			sizeSelect.appendChild(optionEl);
		}
		return sizeSelect;
	};


	this.createLineWidthSelect = function(include_zero) {
		sizeSelect = createElement2('select', { className: 'input-field' });
		var sizes = [0, 1, 2, 3, 4, 5, 8, 10, 12, 18];
		for (var i = 0; i < sizes.length; i++) {
			var size = sizes[i];
			if (size == 0 && !include_zero)
				continue;
			var optionEl = document.createElement("option");
			optionEl.value = size;
			optionEl.innerHTML = size;
			sizeSelect.appendChild(optionEl);
		}
		return sizeSelect;
	};


	this.ButtonBar = function(no_delete, save_button_label) {
		this.elt = document.createElement('div');
		this.elt.style.textAlign = 'center';
		this.elt.style.paddingTop = '8px';

		this.okButton = createElement2('button', { className: 'input-button', type: 'button' });
		this.okButton.appendChild(createElement2('div', { className: 'input-icon icon-accept left' }));
		this.okButton.appendChild(createElement2('span', { className: 'input-label' }, save_button_label || mod_string('LBL_SAVE')));
		this.elt.appendChild(this.okButton);
		this.elt.appendChild(document.createTextNode('\u00a0'));

		this.cancelButton = createElement2('button', { className: 'input-button', type: 'button' });
		this.cancelButton.appendChild(createElement2('div', { className: 'input-icon icon-cancel left' }));
		this.cancelButton.appendChild(createElement2('span', { className: 'input-label' }, mod_string('LBL_CANCEL')));
		this.cancelButton.onclick = function() {
			SUGAR.ui.PopupManager.close();
		};
		this.elt.appendChild(this.cancelButton);
		this.elt.appendChild(document.createTextNode('\u00a0'));

		if (!no_delete) {
			this.deleteButton = createElement2('button', { className: 'input-button', type: 'button' });
			this.deleteButton.appendChild(createElement2('div', { className: 'input-icon icon-delete left' }));
			this.deleteButton.appendChild(createElement2('span', { className: 'input-label' }, mod_string('LBL_DELETE')));
			this.elt.appendChild(this.deleteButton);
		}
	};

	this.SignatureBlockDialog = new function() {
		var c;
		var outer = document.createElement('div');
		var table = document.createElement('table');
		table.style.width = "100%";
		table.style.borderSpacing = "10px 2px";

		var self = this;

		var tr = table.insertRow(-1);
		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_SIGNER_SELECT');

		c = tr.insertCell(-1);
		this.signerSelect = createElement2('select', { className: 'input-field' });
		c.style.verticalAlign = 'top';
		c.appendChild(this.signerSelect);

		tr = table.insertRow(-1);
		c = tr.insertCell(-1);
		c.align = 'center';
		c.colSpan = 2;
		var buttons = new editor.ButtonBar();
		buttons.okButton.onclick = function() {
			SUGAR.ui.PopupManager.close();
			self.eltCtx.signer_ref = self.signerSelect.value;
			PDFTemplateEditor.mark_dirty();
			PDFTemplateEditor.render();
		};

		buttons.deleteButton.onclick = function() {
			SUGAR.ui.PopupManager.close();
			self.eltCtx.deleted = true;
			PDFTemplateEditor.mark_dirty();
			PDFTemplateEditor.render();
		};

		c.appendChild(buttons.elt);

		outer.appendChild(table);

		this.show = function(elt) {
			this.fillSignerSelect();
			this.popup = new SUGAR.ui.Dialog('text_editor', { content_elt: outer, width: '450px', title: mod_string('LBL_EDIT_SIGNATURE') });
			this.popup.destroy_on_close = true;
			this.eltCtx = elt;
			this.signerSelect.value = elt.signer_ref || '';
			this.popup.render();
			this.popup.show();
		};

		this.fillSignerSelect = function() {
			var signerSelect = this.signerSelect;
			signerSelect.innerHTML = '';
			var signersInput = SUGAR.ui.getFormInput(PDFTemplateEditor.form, 'signers');
			var signers = signersInput.getRows().filter(function(s) {
				return s.type == 'signer';
			});
			signers.forEach(function(s) {
				var option = createElement2('option', {
					value: s.ref
				}, s.role);
				signerSelect.appendChild(option);
			});
		}

	}();

	this.SignerNoteDialog = new function() {
		var c;
		var outer = document.createElement('div');
		var table = document.createElement('table');
		table.style.width = "100%";
		table.style.borderSpacing = "10px 2px";

		var self = this;

		var tr = table.insertRow(-1);
		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_SIGNER_SELECT');

		c = tr.insertCell(-1);
		this.signerSelect = createElement2('select', { className: 'input-field' });
		c.style.verticalAlign = 'top';
		c.appendChild(this.signerSelect);

		tr = table.insertRow(-1);
		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_SIGNER_ZONE_LABEL');

		c = tr.insertCell(-1);
		this.labelInput = createElement2('input', { type: 'text', className: 'input-text' });
		this.labelInput.style.width = '95%';
		c.appendChild(this.labelInput);

		tr = table.insertRow(-1);
		c = tr.insertCell(-1);
		c.align = 'center';
		c.colSpan = 2;
		var buttons = new editor.ButtonBar();
		buttons.okButton.onclick = function() {
			SUGAR.ui.PopupManager.close();
			self.eltCtx.label = self.labelInput.value;
			self.eltCtx.signer_ref = self.signerSelect.value;
			PDFTemplateEditor.mark_dirty();
			PDFTemplateEditor.render();
		};

		buttons.deleteButton.onclick = function() {
			SUGAR.ui.PopupManager.close();
			self.eltCtx.deleted = true;
			PDFTemplateEditor.mark_dirty();
			PDFTemplateEditor.render();
		};

		c.appendChild(buttons.elt);

		outer.appendChild(table);

		this.show = function(elt) {
			this.fillSignerSelect();
			this.popup = new SUGAR.ui.Dialog('text_editor', { content_elt: outer, width: '450px', title: mod_string('LBL_EDIT_SIGNERS_NOTE') });
			this.popup.destroy_on_close = true;
			this.eltCtx = elt;
			this.labelInput.value = elt.label || '';
			this.signerSelect.value = elt.signer_ref || '';
			this.popup.render();
			this.popup.show();
		};

		this.fillSignerSelect = function() {
			var signerSelect = this.signerSelect;
			signerSelect.innerHTML = '';
			var signersInput = SUGAR.ui.getFormInput(PDFTemplateEditor.form, 'signers');
			var signers = signersInput.getRows();
			signers.forEach(function(s) {
				var option = createElement2('option', {
					value: s.ref
				}, s.role);
				signerSelect.appendChild(option);
			});
		}


	}();

	this.AnnotationDialog = new function() {
		var c;
		var outer = document.createElement('div');
		var table = document.createElement('table');
		table.style.width = "100%";
		table.style.borderSpacing = "10px 2px";

		var self = this;

		var tr = table.insertRow(-1);
		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_SIGNERS_SELECT');

		c = tr.insertCell(-1);
		this.signerSelect = createElement2('select', { multiple: true, className: 'input-field' });
		c.style.verticalAlign = 'top';
		c.appendChild(this.signerSelect);

		tr = table.insertRow(-1);
		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_TEXT');

		c = tr.insertCell(-1);
		this.textArea = createElement2('textarea', { className: 'input-text' });
		this.textArea.style.width = '95%';
		c.appendChild(this.textArea);

		tr = table.insertRow(-1);
		c = tr.insertCell(-1);
		c.align = 'center';
		c.colSpan = 2;
		var buttons = new editor.ButtonBar();
		buttons.okButton.onclick = function() {
			SUGAR.ui.PopupManager.close();
			self.eltCtx.text = self.textArea.value;
			self.eltCtx.signer_refs = Array.from(self.signerSelect.selectedOptions).map(function(o) {
				return o.value;
			});
			PDFTemplateEditor.mark_dirty();
			PDFTemplateEditor.render();
		};

		buttons.deleteButton.onclick = function() {
			SUGAR.ui.PopupManager.close();
			self.eltCtx.deleted = true;
			PDFTemplateEditor.mark_dirty();
			PDFTemplateEditor.render();
		};

		c.appendChild(buttons.elt);

		outer.appendChild(table);

		this.show = function(elt) {
			this.fillSignerSelect();
			this.popup = new SUGAR.ui.Dialog('text_editor', { content_elt: outer, width: '450px', title: mod_string('LBL_EDIT_ANNOTATION') });
			this.popup.destroy_on_close = true;
			this.eltCtx = elt;
			this.textArea.value = elt.text || '';
			Array.from(this.signerSelect.options).forEach(function(o) {
				o.selected = !!~elt.signer_refs.indexOf(o.value);
			});
			this.popup.render();
			this.popup.show();
		};

		this.fillSignerSelect = function() {
			var signerSelect = this.signerSelect;
			signerSelect.innerHTML = '';
			var signersInput = SUGAR.ui.getFormInput(PDFTemplateEditor.form, 'signers');
			var signers = signersInput.getRows();
			signers.forEach(function(s) {
				var option = createElement2('option', {
					value: s.ref
				}, s.role);
				signerSelect.appendChild(option);
			});
		}


	}();

	this.TextDialog = new function() {
		var c;
		var outer = document.createElement('div');
		var table = document.createElement('table');
		table.style.width = "100%";
		table.style.borderSpacing = "10px 2px";

		var self = this;

		var tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_TEXT', '30%');

		c = tr.insertCell(-1);
		c.style.width = "50%";
		this.textArea = createElement2('textarea', { className: 'input-text' });
		this.textArea.style.width = '95%';
		UI.addEventListener(this.textArea, 'contextmenu', function(ev) {
			ev.preventDefault();
			ev.stopPropagation();
			PDFTemplateEditor.displayFieldSelector(this, self.eltId);
		});

		c.appendChild(this.textArea);

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_FONT');

		c = tr.insertCell(-1);
		this.fontSelect = editor.createFontSelect();
		c.style.verticalAlign = 'top';
		c.appendChild(this.fontSelect);

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_SIZE');

		c = tr.insertCell(-1);
		this.sizeSelect = editor.createFontSizeSelect();
		c.style.verticalAlign = 'top';
		c.appendChild(this.sizeSelect);


		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_BOLD');

		c = tr.insertCell(-1);
		this.boldCheckbox = createElement2('input', { type: 'checkbox' });
		c.style.verticalAlign = 'top';
		c.appendChild(wrapCheck(this.boldCheckbox));

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_ITALIC');

		c = tr.insertCell(-1);
		this.italicCheckbox = createElement2('input', { type: 'checkbox' });
		c.style.verticalAlign = 'top';
		c.appendChild(wrapCheck(this.italicCheckbox));

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_ALIGN');

		c = tr.insertCell(-1);
		this.alignSelect = editor.createAlignSelect();
		c.style.verticalAlign = 'top';
		c.appendChild(this.alignSelect);

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_LINE_HEIGHT');

		c = tr.insertCell(-1);
		this.lineHeightSelect = editor.createLineHeightSelect();
		c.style.verticalAlign = 'top';
		c.appendChild(this.lineHeightSelect);

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_TEXT_COLOR');

		c = tr.insertCell(-1);
		this.colorSelect = editor.createColorPicker();
		c.style.verticalAlign = 'top';
		c.appendChild(this.colorSelect);

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_BACKGROUND_COLOR');

		c = tr.insertCell(-1);
		this.backgroundSelect = editor.createColorPicker();
		c.style.verticalAlign = 'top';
		c.appendChild(this.backgroundSelect);

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_BORDER_WIDTH');

		c = tr.insertCell(-1);
		this.widthSelect = editor.createLineWidthSelect(true);
		c.style.verticalAlign = 'top';
		c.appendChild(this.widthSelect);

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_BORDER_COLOR');

		c = tr.insertCell(-1);
		this.borderColorSelect = editor.createColorPicker();
		c.style.verticalAlign = 'top';
		c.appendChild(this.borderColorSelect);

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_ADJUST');

		c = tr.insertCell(-1);
		this.adjustCheckbox = createElement2('input', { type: 'checkbox' });
		c.style.verticalAlign = 'top';
		c.appendChild(wrapCheck(this.adjustCheckbox));

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_PAGEBREAK');

		c = tr.insertCell(-1);
		this.breakCheckbox = createElement2('input', { type: 'checkbox' });
		c.style.verticalAlign = 'top';
		c.appendChild(wrapCheck(this.breakCheckbox));

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_URL');

		c = tr.insertCell(-1);
		this.urlInput = createElement2('input', { type: 'text', className: 'input-text' });
		this.urlInput.style.width = '95%';
		c.appendChild(this.urlInput);

		tr = table.insertRow(-1);
		c = tr.insertCell(-1);
		c.align = 'center';
		c.colSpan = 2;
		var buttons = new editor.ButtonBar();
		buttons.okButton.onclick = function() {
			SUGAR.ui.PopupManager.close();
			self.eltCtx.text = self.textArea.value;
			self.eltCtx.url = self.urlInput.value;
			self.eltCtx.align = self.alignSelect.value;
			self.eltCtx.font = self.fontSelect.value;
			self.eltCtx.size = self.sizeSelect.value;
			self.eltCtx.line_height = self.lineHeightSelect.value;
			self.eltCtx.adjust = self.adjustCheckbox.checked;
			self.eltCtx.page_break = self.breakCheckbox.checked;
			self.eltCtx.bold = self.boldCheckbox.checked;
			self.eltCtx.italic = self.italicCheckbox.checked;
			self.eltCtx.background = self.backgroundSelect.color.toString();
			self.eltCtx.color = self.colorSelect.color.toString();
			self.eltCtx.border_color = self.borderColorSelect.color.toString();
			self.eltCtx.border_width = self.widthSelect.value;
			PDFTemplateEditor.mark_dirty();
			PDFTemplateEditor.render();
		};

		buttons.deleteButton.onclick = function() {
			SUGAR.ui.PopupManager.close();
			self.eltCtx.deleted = true;
			PDFTemplateEditor.mark_dirty();
			PDFTemplateEditor.render();
		};

		c.appendChild(buttons.elt);

		outer.appendChild(table);

		this.show = function(elt, eltId) {
			this.popup = new SUGAR.ui.Dialog('text_editor', { content_elt: outer, width: '450px', title: mod_string('LBL_EDIT_TEXT') });
			this.popup.destroy_on_close = true;
			this.popup.onclose = function() {
				self.colorSelect.color.hidePicker();
				self.backgroundSelect.color.hidePicker();
				self.borderColorSelect.color.hidePicker();
			};
			this.eltCtx = elt;
			this.eltId = eltId;
			this.textArea.value = elt.text || '';
			this.urlInput.value = elt.url || '';
			this.fontSelect.value = elt.font || 'sans';
			this.alignSelect.value = elt.align || 'left';
			this.lineHeightSelect.value = elt.line_height || '1.0';
			this.sizeSelect.value = elt.size || 10;
			this.adjustCheckbox.checked = elt.adjust;
			this.breakCheckbox.checked = elt.page_break;
			this.boldCheckbox.checked = elt.bold;
			this.italicCheckbox.checked = elt.italic;
			this.colorSelect.color.fromString(elt.color || '000');
			this.backgroundSelect.color.fromString(elt.background || 'FFF');
			this.borderColorSelect.color.fromString(elt.border_color || '000');
			this.widthSelect.value = elt.border_width || 0;
			this.popup.render();
			this.popup.show();
		};

	}();

	this.LineDialog = new function() {
		var outer = document.createElement('div');
		var table = document.createElement('table');
		table.style.width = "100%";
		table.style.borderSpacing = "10px 2px";

		var self = this;

		tr = table.insertRow(-1);
		c = tr.insertCell(-1);
		c.appendChild(document.createTextNode(mod_string('LBL_LINE_WIDTH')));

		tr = table.insertRow(-1);
		c = tr.insertCell(-1);
		this.widthSelect = editor.createLineWidthSelect();
		c.style.verticalAlign = 'top';
		c.appendChild(this.widthSelect);

		tr = table.insertRow(-1);
		c = tr.insertCell(-1);
		c.appendChild(document.createTextNode(mod_string('LBL_LINE_COLOR')));

		tr = table.insertRow(-1);
		c = tr.insertCell(-1);
		this.colorSelect = editor.createColorPicker();
		c.style.verticalAlign = 'top';
		c.appendChild(this.colorSelect);


		tr = table.insertRow(-1);
		c = tr.insertCell(-1);
		c.align = 'center';
		var buttons = new editor.ButtonBar();
		buttons.okButton.onclick = function() {
			SUGAR.ui.PopupManager.close();
			self.eltCtx.line_width = self.widthSelect.value;
			self.eltCtx.color = self.colorSelect.color.toString();
			PDFTemplateEditor.mark_dirty();
			PDFTemplateEditor.render();
		};

		buttons.deleteButton.onclick = function() {
			SUGAR.ui.PopupManager.close();
			self.eltCtx.deleted = true;
			PDFTemplateEditor.mark_dirty();
			PDFTemplateEditor.render();
		};

		c.appendChild(buttons.elt);

		outer.appendChild(table);

		this.show = function(elt) {
			this.popup = new SUGAR.ui.Dialog('line_editor', { content_elt: outer, width: '350px', title: 'Edit Line' });
			this.popup.destroy_on_close = true;
			var self = this;
			this.popup.onclose = function() {
				self.colorSelect.color.hidePicker();
			};
			this.eltCtx = elt;
			this.widthSelect.value = elt.line_width || 1;
			this.colorSelect.color.fromString(get_default(elt.color, '000'));
			this.popup.render();
			this.popup.show();
		};

	}();

	this.FieldDialog = new function() {
		var c;
		var outer = document.createElement('div');
		var table = document.createElement('table');
		table.style.width = "100%";
		table.style.borderSpacing = "10px 2px";

		var tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_SOURCE', '40%');
		this.sourceSelectRow = tr;

		c = tr.insertCell(-1);
		this.sourceSelect = createElement2('select', { className: 'input-field' });
		c.appendChild(this.sourceSelect);

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_FIELD', '40%');

		c = tr.insertCell(-1);
		this.fieldSelect = createElement2('select', { className: 'input-field' });
		c.appendChild(this.fieldSelect);

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_FONT');

		c = tr.insertCell(-1);
		this.fontSelect = editor.createFontSelect();
		c.style.verticalAlign = 'top';
		c.appendChild(this.fontSelect);

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_SIZE');

		c = tr.insertCell(-1);
		this.sizeSelect = editor.createFontSizeSelect();
		c.style.verticalAlign = 'top';
		c.appendChild(this.sizeSelect);

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_BOLD');

		c = tr.insertCell(-1);
		this.boldCheckbox = document.createElement('input');
		this.boldCheckbox.setAttribute('type', 'checkbox');
		c.style.verticalAlign = 'top';
		c.appendChild(wrapCheck(this.boldCheckbox));

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_ITALIC');

		c = tr.insertCell(-1);
		this.italicCheckbox = document.createElement('input');
		this.italicCheckbox.setAttribute('type', 'checkbox');
		c.style.verticalAlign = 'top';
		c.appendChild(wrapCheck(this.italicCheckbox));

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_ALIGN');

		c = tr.insertCell(-1);
		this.alignSelect = editor.createAlignSelect();
		c.style.verticalAlign = 'top';
		c.appendChild(this.alignSelect);

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_NUMBER_ALIGN');

		c = tr.insertCell(-1);
		this.numberAlignSelect = editor.createAlignSelect();
		c.style.verticalAlign = 'top';
		c.appendChild(this.numberAlignSelect);

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_LINE_HEIGHT');

		c = tr.insertCell(-1);
		this.lineHeightSelect = editor.createLineHeightSelect();
		c.style.verticalAlign = 'top';
		c.appendChild(this.lineHeightSelect);

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_TEXT_COLOR');

		c = tr.insertCell(-1);
		this.colorSelect = editor.createColorPicker();
		c.style.verticalAlign = 'top';
		c.appendChild(this.colorSelect);

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_BACKGROUND_COLOR');

		c = tr.insertCell(-1);
		this.backgroundSelect = editor.createColorPicker();
		c.style.verticalAlign = 'top';
		c.appendChild(this.backgroundSelect);

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_BORDER_WIDTH');

		c = tr.insertCell(-1);
		this.widthSelect = editor.createLineWidthSelect(true);
		c.style.verticalAlign = 'top';
		c.appendChild(this.widthSelect);

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_BORDER_COLOR');

		c = tr.insertCell(-1);
		this.borderColorSelect = editor.createColorPicker();
		c.style.verticalAlign = 'top';
		c.appendChild(this.borderColorSelect);

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_ADJUST');

		c = tr.insertCell(-1);
		this.adjustCheckbox = createElement2('input', { type: 'checkbox' });
		c.style.verticalAlign = 'top';
		c.appendChild(wrapCheck(this.adjustCheckbox));

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_PAGEBREAK');

		c = tr.insertCell(-1);
		this.breakCheckbox = createElement2('input', { type: 'checkbox' });
		c.style.verticalAlign = 'top';
		c.appendChild(wrapCheck(this.breakCheckbox));

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_URL');

		c = tr.insertCell(-1);
		this.urlInput = createElement2('input', { type: 'text', className: 'input-text' });
		this.urlInput.style.width = '98%';
		c.appendChild(this.urlInput);

		tr = table.insertRow(-1);
		c = tr.insertCell(-1);
		c.align = 'center';
		c.colSpan = 2;

		var buttons = new editor.ButtonBar();
		c.appendChild(buttons.elt);
		var self = this;
		buttons.okButton.onclick = function() {
			SUGAR.ui.PopupManager.close();
			self.eltCtx.field = self.fieldSelect.value;
			self.eltCtx.url = self.urlInput.value;
			self.eltCtx.align = self.alignSelect.value;
			self.eltCtx.number_align = self.numberAlignSelect.value;
			self.eltCtx.font = self.fontSelect.value;
			self.eltCtx.size = self.sizeSelect.value;
			self.eltCtx.line_height = self.lineHeightSelect.value;
			self.eltCtx.adjust = self.adjustCheckbox.checked;
			self.eltCtx.page_break = self.breakCheckbox.checked;
			self.eltCtx.bold = self.boldCheckbox.checked;
			self.eltCtx.italic = self.italicCheckbox.checked;
			self.eltCtx.color = self.colorSelect.color.toString();
			self.eltCtx.background = self.backgroundSelect.color.toString();
			self.eltCtx.border_color = self.borderColorSelect.color.toString();
			self.eltCtx.border_width = self.widthSelect.value;
			PDFTemplateEditor.mark_dirty();
			PDFTemplateEditor.render();
		};

		buttons.deleteButton.onclick = function() {
			SUGAR.ui.PopupManager.close();
			self.eltCtx.deleted = true;
			PDFTemplateEditor.mark_dirty();
			PDFTemplateEditor.render();
		};

		outer.appendChild(table);

		this.renderFields = function(dataCtx, prefix) {
			this.fieldSelect.innerHTML = '';
			var prefix2 = '', pos2;
			for (var f in dataCtx) {
				if (f == '_')
					continue;
				pos2 = f.indexOf('.');
				if (f.substr(0, 1) === '_') {
					prefix2 = '_';
				} else if (~pos2) {
					prefix2 = f.substr(0, pos2);
				}
				if (prefix != prefix2)
					continue;
				var def = dataCtx[f];
				var optionEl = document.createElement("option");
				optionEl.value = f;
				optionEl.innerHTML = def.label;
				this.fieldSelect.appendChild(optionEl);
			}
		}

		this.show = function(elt, dataCtx, dataKey) {
			var self = this;
			this.popup = new SUGAR.ui.Dialog('field_editor', { content_elt: outer, width: '450px', title: mod_string('LBL_EDIT_FIELD') });
			this.popup.destroy_on_close = true;

			var prefix = '', pos;

			if ((dataKey == 'header' || dataKey == 'page_header') && elt.field) {
				pos = elt.field.indexOf('.');
				if (elt.field.substr(0, 1) === '_') {
					prefix = '_';
				} else if (~pos) {
					prefix = elt.field.substr(0, pos);
				}
			}


			if (dataKey == 'header' || dataKey == 'page_header') {
				this.sourceSelect.onchange = null;
				this.sourceSelect.innerHTML = '';
				var sources = PDFTemplateEditor.sources;
				for (var f in sources) {
					var optionEl = document.createElement("option");
					optionEl.value = f;
					optionEl.innerHTML = sources[f];
					this.sourceSelect.appendChild(optionEl);
				}
				this.sourceSelect.value = prefix;
				this.sourceSelect.onchange = function() {
					self.renderFields(dataCtx, this.value);
				};
				this.sourceSelectRow.style.display = '';
			} else {
				this.sourceSelectRow.style.display = 'none';
			}
			this.renderFields(dataCtx, prefix);

			this.popup.onclose = function() {
				self.colorSelect.color.hidePicker();
				self.backgroundSelect.color.hidePicker();
				self.borderColorSelect.color.hidePicker();
			};
			this.eltCtx = elt;
			this.fieldSelect.value = elt.field;
			this.urlInput.value = elt.url || '';
			this.fontSelect.value = elt.font || 'sans';
			this.alignSelect.value = elt.align || 'left';
			this.numberAlignSelect.value = elt.number_align || elt.align || 'left';
			this.lineHeightSelect.value = elt.line_height || '1.0';
			this.sizeSelect.value = elt.size || 10;
			this.adjustCheckbox.checked = elt.adjust;
			this.breakCheckbox.checked = elt.page_break;
			this.boldCheckbox.checked = elt.bold;
			this.italicCheckbox.checked = elt.italic;
			this.colorSelect.color.fromString(elt.color || '000');
			this.backgroundSelect.color.fromString(elt.background || 'FFF');
			this.borderColorSelect.color.fromString(elt.border_color || '000');
			this.widthSelect.value = elt.border_width || 0;
			this.popup.render();
			this.popup.show();
		};

	}();

	this.FieldSelectorDialog = new function() {
		var c;
		var outer = document.createElement('div');
		var table = document.createElement('table');
		table.style.width = "100%";
		table.style.borderSpacing = "10px 2px";

		var tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_SOURCE', '40%');
		this.sourceSelectRow = tr;

		c = tr.insertCell(-1);
		this.sourceSelect = createElement2('select', { className: 'input-field' });
		c.appendChild(this.sourceSelect);

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_FIELD', '40%');

		c = tr.insertCell(-1);
		this.fieldSelect = createElement2('select', { className: 'input-field' });
		c.appendChild(this.fieldSelect);

		var buttons = new editor.ButtonBar(true, mod_string('LBL_INSERT_FIELD'));
		c.appendChild(buttons.elt);
		var self = this;
		buttons.okButton.onclick = function() {
			var el = self.input;
			var start = el.selectionStart;
			var end = el.selectionEnd;
			var text = el.value;
			var before = text.substring(0, start)
			var after = text.substring(end, text.length)
			var newText = '<{field:' + self.fieldSelect.value + '}>';
			el.value = (before + newText + after)
			el.selectionStart = el.selectionEnd = start + newText.length
			SUGAR.ui.PopupManager.close();
			el.focus()
		};

		outer.appendChild(table);

		this.renderFields = function(dataCtx, prefix) {
			this.fieldSelect.innerHTML = '';
			var prefix2 = '', pos2;
			for (var f in dataCtx) {
				if (f == '_')
					continue;
				pos2 = f.indexOf('.');
				if (f.substr(0, 1) === '_') {
					prefix2 = '_';
				} else if (~pos2) {
					prefix2 = f.substr(0, pos2);
				}
				if (prefix != prefix2)
					continue;
				var def = dataCtx[f];
				var optionEl = document.createElement("option");
				optionEl.value = f;
				optionEl.innerHTML = def.label;
				this.fieldSelect.appendChild(optionEl);
			}
		}

		this.onClose = function() {
			this.input.focus()
		}

		this.show = function(input, dataCtx, dataKey) {
			var self = this;
			this.popup = new SUGAR.ui.Dialog('field_selector', { content_elt: outer, width: '450px', title: mod_string('LBL_INSERT_FIELD'), onclose: this.onClose.bind(this) });
			this.popup.destroy_on_close = true;

			this.input = input;
			var prefix = '', pos;

			if (dataKey == 'header' || dataKey == 'page_header') {
				this.sourceSelect.onchange = null;
				this.sourceSelect.innerHTML = '';
				var sources = PDFTemplateEditor.sources;
				for (var f in sources) {
					var optionEl = document.createElement("option");
					optionEl.value = f;
					optionEl.innerHTML = sources[f];
					this.sourceSelect.appendChild(optionEl);
				}
				this.sourceSelect.value = prefix;
				this.sourceSelect.onchange = function() {
					self.renderFields(dataCtx, this.value);
				};
				this.sourceSelectRow.style.display = '';
			} else {
				this.sourceSelectRow.style.display = 'none';
			}
			this.renderFields(dataCtx, prefix);

			this.popup.render();
			this.popup.show();
		};

	}();

	this.ColumnDialog = new function() {
		var c;
		var outer = document.createElement('div');
		var table = document.createElement('table');
		table.style.width = "100%";
		table.style.borderSpacing = "10px 2px";

		var tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_FIELD', '30%');

		c = tr.insertCell(-1);
		c.style.width = "80%";
		this.fieldSelect = createElement2('select', { className: 'input-field' });
		c.appendChild(this.fieldSelect);

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_TITLE');

		c = tr.insertCell(-1);
		c.style.width = "80%";
		this.labelInput = createElement2('textarea', { className: 'input-text' });
		this.labelInput.cols = '25';
		this.labelInput.rows = '2';
		c.appendChild(this.labelInput);

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_ALIGN_HEADER');

		c = tr.insertCell(-1);
		this.headerAlignSelect = editor.createAlignSelect();
		c.style.verticalAlign = 'top';
		c.appendChild(this.headerAlignSelect);

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_ALIGN_DATA');

		c = tr.insertCell(-1);
		this.alignSelect = editor.createAlignSelect();
		c.style.verticalAlign = 'top';
		c.appendChild(this.alignSelect);

		var buttons = new editor.ButtonBar();
		c.appendChild(buttons.elt);
		var buttonsDiv = document.createElement('div');
		buttonsDiv.setAttribute('style', 'clear: left; width: 100%');
		buttonsDiv.appendChild(buttons.elt);

		var self = this;
		buttons.okButton.onclick = function() {
			SUGAR.ui.PopupManager.close();
			self.eltCtx.field = self.fieldSelect.value;
			self.eltCtx.label = self.labelInput.value;
			self.eltCtx.align = self.alignSelect.value;
			self.eltCtx.header_align = self.headerAlignSelect.value;
			PDFTemplateEditor.mark_dirty();
			PDFTemplateEditor.render();
		};

		buttons.deleteButton.onclick = function() {
			SUGAR.ui.PopupManager.close();
			self.eltCtx.deleted = true;
			PDFTemplateEditor.mark_dirty();
			PDFTemplateEditor.render();
		};

		outer.appendChild(table);
		outer.appendChild(buttonsDiv);

		this.show = function(elt, dataCtx) {
			this.popup = new SUGAR.ui.Dialog('column_editor', { content_elt: outer, width: '350px', title: mod_string('LBL_EDIT_COLUMN') });
			this.popup.destroy_on_close = true;
			this.fieldSelect.innerHTML = '';
			for (var f in dataCtx) {
				var def = dataCtx[f];
				var optionEl = document.createElement("option");
				optionEl.value = f;
				optionEl.innerHTML = def.label;
				this.fieldSelect.appendChild(optionEl);
			}

			this.eltCtx = elt;
			this.fieldSelect.value = elt.field;
			this.labelInput.value = elt.label || '';
			this.alignSelect.value = elt.align || 'left';
			this.headerAlignSelect.value = elt.header_align || 'left';
			this.popup.render();
			this.popup.show();
		};

	}();

	this.ImageDialog = new function() {
		var self = this;
		var outer = document.createElement('div');
		var currentOuter = document.createElement('div');
		currentOuter.setAttribute('style', 'width:200px; float: left; margin: 5px;');
		outer.appendChild(currentOuter);

		this.image = document.createElement('img');
		currentOuter.appendChild(this.image);
		this.image.style.maxWidth = '200px';
		this.image.style.maxHeight = '200px';

		var editOuter = document.createElement('div');
		editOuter.setAttribute('style', 'width:300px; float: left; margin: 5px;');
		outer.appendChild(editOuter);

		var logoButton = createElement2('button', { className: 'input-button' });
		logoButton.appendChild(document.createTextNode(mod_string('LBL_PDF_LOGO')));
		editOuter.appendChild(logoButton);

		var sigButton = createElement2('button', { className: 'input-button' });
		sigButton.appendChild(document.createTextNode(mod_string('LBL_PDF_SIGNATURE')));
		editOuter.appendChild(sigButton);

		editOuter.appendChild(document.createElement('hr'));

		var frm = document.createElement('form');
		frm.setAttribute('method', 'POST');
		editOuter.appendChild(frm);
		var file = createElement2('input', { className: 'input-field input-file', type: 'file', name: 'file' });
		frm.appendChild(file);
		frm.appendChild(document.createElement('br'));
		var uploadButton = createElement2('button', { className: 'input-button' });
		uploadButton.appendChild(document.createTextNode(mod_string('LBL_UPLOAD')));
		frm.appendChild(uploadButton);

		uploadButton.onclick = function() {
			try {
				var params = { module: 'PDFTemplates', action: 'Upload' };
				SUGAR.ui.sendForm(this.form, params, { resultDiv: self.imagesList, force_revert: true, clear_on_revert: true });
			} catch (e) {
				console.error(e);
			}
			return false;
		};

		logoButton.onclick = function() {
			self.image.src = PDFTemplateEditor.pdf_logo;
			self.image.pdf_logo = true;
			self.image.signature_image = false;
		};

		sigButton.onclick = function() {
			self.image.src = PDFTemplateEditor.signature;
			self.image.signature_image = true;
			self.image.pdf_logo = false;
		};

		this.imagesList = document.createElement('div');
		this.imagesList.setAttribute('style', 'clear: left; width: 100%; height: 100px; overflow-y: scroll');
		outer.appendChild(this.imagesList);

		var buttonsDiv = document.createElement('div');
		buttonsDiv.setAttribute('style', 'clear: left; width: 100%');
		var buttons = new editor.ButtonBar();
		buttonsDiv.appendChild(buttons.elt);
		outer.appendChild(buttonsDiv);

		buttons.okButton.onclick = function() {
			SUGAR.ui.PopupManager.close();
			self.eltCtx.src = self.image.getAttribute('src');
			self.eltCtx.pdf_logo = self.image.pdf_logo;
			self.eltCtx.signature_image = self.image.signature_image;
			PDFTemplateEditor.mark_dirty();
			PDFTemplateEditor.render();
		};

		buttons.deleteButton.onclick = function() {
			SUGAR.ui.PopupManager.close();
			self.eltCtx.deleted = true;
			PDFTemplateEditor.mark_dirty();
			PDFTemplateEditor.render();
		};


		var req = new SUGAR.conn.HTTPRequest('async.php?module=PDFTemplates&action=ListUploads');
		req.fetch(function(req) {
			self.imagesList.innerHTML = req.responseText;
		});


		this.show = function(elt, dataCtx) {
			this.popup = new SUGAR.ui.Dialog('image_editor', { content_elt: outer, width: '600px', height: '450px', title: 'Edit Image' });
			this.popup.destroy_on_close = true;
			this.eltCtx = elt;
			this.image.src = elt.pdf_logo ? PDFTemplateEditor.pdf_logo : elt.src;
			var sampleSignature = PDFTemplateEditor.data['header']['SIGNATURE'];
			if (elt.signature_image) {
				this.image.src = sampleSignature || PDFTemplateEditor.signature;
			} else {
				this.image.src = elt.pdf_logo ? PDFTemplateEditor.pdf_logo : elt.src;
			}
			this.popup.render();
			this.popup.show();
		};

	}

	this.TableDialog = new function() {
		var self = this;
		var outer = document.createElement('div');

		this.cellFont = editor.createFontSelect();
		this.cellFontSize = editor.createFontSizeSelect();
		this.headerFont = editor.createFontSelect();
		this.headerFontSize = editor.createFontSizeSelect();
		this.cellColorSelect = editor.createColorPicker();
		this.oddBackgroundSelect = editor.createColorPicker();
		this.evenBackgroundSelect = editor.createColorPicker();
		this.headerColorSelect = editor.createColorPicker();
		this.headerBackgroundSelect = editor.createColorPicker();

		var table = document.createElement('table');
		table.style.borderSpacing = "10px 2px";

		var tr = table.insertRow(-1);
		var c = tr.insertCell(-1);
		c.appendChild(document.createTextNode(mod_string('LBL_CELL_FONT')));
		c.style.textAlign = 'right';

		c = tr.insertCell(-1);
		c.appendChild(this.cellFont);

		c = tr.insertCell(-1);
		c.appendChild(document.createTextNode(mod_string('LBL_CELL_FONT_SIZE')));
		c.style.textAlign = 'right';
		c = tr.insertCell(-1);
		c.appendChild(this.cellFontSize);

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_CELL_TEXT_COLOR');

		c = tr.insertCell(-1);
		c.style.verticalAlign = 'top';
		c.appendChild(this.cellColorSelect);

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_ODD_BACKGROUND_COLOR');

		c = tr.insertCell(-1);
		c.style.verticalAlign = 'top';
		c.appendChild(this.oddBackgroundSelect);

		createLabelCell(tr, 'LBL_EVEN_BACKGROUND_COLOR');

		c = tr.insertCell(-1);
		c.style.verticalAlign = 'top';
		c.appendChild(this.evenBackgroundSelect);

		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_BORDER_WIDTH');

		c = tr.insertCell(-1);
		this.borderWidthSelect = editor.createLineWidthSelect(true);
		c.style.verticalAlign = 'top';
		c.appendChild(this.borderWidthSelect);

		createLabelCell(tr, 'LBL_BORDER_COLOR');

		c = tr.insertCell(-1);
		this.borderColorSelect = editor.createColorPicker();
		c.style.verticalAlign = 'top';
		c.appendChild(this.borderColorSelect);


		tr = table.insertRow(-1);
		var c = tr.insertCell(-1);
		c.appendChild(document.createTextNode(mod_string('LBL_HEADER_FONT')));
		c.style.textAlign = 'right';
		c = tr.insertCell(-1);
		c.appendChild(this.headerFont);

		c = tr.insertCell(-1);
		c.appendChild(document.createTextNode(mod_string('LBL_HEADER_FONT_SIZE')));
		c.style.textAlign = 'right';
		c = tr.insertCell(-1);
		c.appendChild(this.headerFontSize);


		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_HEADER_TEXT_COLOR');

		c = tr.insertCell(-1);
		c.style.verticalAlign = 'top';
		c.appendChild(this.headerColorSelect);

		createLabelCell(tr, 'LBL_HEADER_BACKGROUND_COLOR');

		c = tr.insertCell(-1);
		c.style.verticalAlign = 'top';
		c.appendChild(this.headerBackgroundSelect);


		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_HEADER_BORDER_WIDTH');

		c = tr.insertCell(-1);
		this.headerBorderWidthSelect = editor.createLineWidthSelect(true);
		c.style.verticalAlign = 'top';
		c.appendChild(this.headerBorderWidthSelect);

		createLabelCell(tr, 'LBL_HEADER_BORDER_COLOR');

		c = tr.insertCell(-1);
		this.headerBorderColorSelect = editor.createColorPicker();
		c.style.verticalAlign = 'top';
		c.appendChild(this.headerBorderColorSelect);


		tr = table.insertRow(-1);
		createLabelCell(tr, 'LBL_HEADER_BOLD');

		c = tr.insertCell(-1);
		this.boldCheckbox = createElement2('input', { type: 'checkbox' });
		c.style.verticalAlign = 'top';
		c.appendChild(wrapCheck(this.boldCheckbox));

		createLabelCell(tr, 'LBL_TABLE_OFFSET');

		c = tr.insertCell(-1);
		this.offsetInput = createElement2('input', { type: 'text', size: 6 });
		c.style.verticalAlign = 'top';
		c.appendChild(this.offsetInput);


		outer.appendChild(table);

		var buttonsDiv = document.createElement('div');
		buttonsDiv.setAttribute('style', 'clear: left; width: 100%');
		var buttons = new editor.ButtonBar(true);
		buttonsDiv.appendChild(buttons.elt);
		outer.appendChild(buttonsDiv);

		buttons.okButton.onclick = function() {
			if (!self.validate())
				return false;
			SUGAR.ui.PopupManager.close();
			self.section.header_font = self.headerFont.value;
			self.section.header_font_size = self.headerFontSize.value;
			self.section.cell_font = self.cellFont.value;
			self.section.cell_font_size = self.cellFontSize.value;
			self.section.cell_border_width = self.borderWidthSelect.value;
			self.section.cell_border_color = self.borderColorSelect.color.toString();
			self.section.color = self.cellColorSelect.color.toString();
			self.section.odd_background = self.oddBackgroundSelect.color.toString();
			self.section.even_background = self.evenBackgroundSelect.color.toString();
			self.section.header_background = self.headerBackgroundSelect.color.toString();
			self.section.header_color = self.headerColorSelect.color.toString();
			self.section.header_border_width = self.headerBorderWidthSelect.value;
			self.section.header_border_color = self.headerBorderColorSelect.color.toString();
			self.section.header_bold = self.boldCheckbox.checked;
			self.section.offset = parseInt(self.offsetInput.value) || 0;
			PDFTemplateEditor.mark_dirty();
			PDFTemplateEditor.render();
		};

		this.validate = function() {
			var offset = parseInt(self.offsetInput.value);
			if (isNaN(offset) || (offset < 0)) {
				alert(mod_string('LBL_INVALID_OFFSET'));
				return false;
			}
			var w = 0;
			for (var i = 0; i < self.section.columns.length; i++) {
				if (!self.section.columns[i].deleted)
					w += self.section.columns[i].width;
			}
			if (w > PDFTemplateEditor.render_width - offset) {
				alert(mod_string('LBL_OFFSET_TOO_BIG'));
				return false;
			}
			return true;
		};

		this.show = function(sec) {
			this.popup = new SUGAR.ui.Dialog('table_editor', { content_elt: outer, width: '600px', height: '450px', title: mod_string('LBL_TABLE_PROPERTIES') });
			this.popup.destroy_on_close = true;
			this.section = sec;
			this.headerFont.value = get_default(this.section.header_font, 'sans');
			this.headerFontSize.value = this.section.header_font_size;
			this.cellFont.value = get_default(this.section.cell_font, 'sans');
			this.cellFontSize.value = this.section.cell_font_size;
			this.oddBackgroundSelect.color.fromString(this.section.odd_background || 'FFF');
			this.evenBackgroundSelect.color.fromString(this.section.even_background || 'ECECEC');
			this.cellColorSelect.color.fromString(this.section.color || '000');
			this.borderWidthSelect.value = this.section.cell_border_width || '0';
			this.borderColorSelect.color.fromString(this.section.cell_border_color || '000');
			this.headerColorSelect.color.fromString(this.section.header_color || '000');
			this.headerBackgroundSelect.color.fromString(this.section.header_background || 'C8C8C8');
			this.headerBorderWidthSelect.value = get_default(this.section.header_border_width, '1');
			this.headerBorderColorSelect.color.fromString(this.section.header_border_color || '000');
			this.boldCheckbox.checked = get_default(this.section.header_bold, 1);
			this.offsetInput.value = get_default(this.section.offset, 0);
			this.popup.onclose = function() {
				self.oddBackgroundSelect.color.hidePicker();
				self.evenBackgroundSelect.color.hidePicker();
				self.cellColorSelect.color.hidePicker();
				self.headerBackgroundSelect.color.hidePicker();
				self.headerColorSelect.color.hidePicker();
				self.headerBorderColorSelect.color.hidePicker();
				self.borderColorSelect.color.hidePicker();
			};
			this.popup.render();
			this.popup.show();
		};

	};

	this.preview = function(form, input_id) {
		var inp = SUGAR.ui.getFormInput(form, input_id);
		var id = inp.getKey();
		if (id) {
			var inputs =
				form.querySelectorAll('input[type=hidden]')
					.values()
					.filter(el => /^_[0-9a-z]{40}$/.test(el.name ?? ""))
					.map(el => [el.name, el.value]);
			var encoded = this.encode();
			var frm = document.createElement('form');
			frm.setAttribute('method', 'POST');
			var conn_params = { url: 'async.php' };
			form.parentElement.appendChild(frm);
			var record = form.record.value;
			if (!record && form.duplicate_of_id)
				record = form.duplicate_of_id.value;
			var fields = {
				module: 'PDFTemplates',
				action: 'Render',
				record: record,
				focus_id: id,
				override_layout: encoded,
				orientation: this.orientation,
				margin_left: this.margin_left,
				margin_top: this.margin_top
			};
			inputs.forEach(nameValue => {
				fields[nameValue[0]] = nameValue[1];
			});
			SUGAR.ui.sendForm(frm, fields, conn_params, '_blank');
		};
	}

	this.request_data = function(id) {
		if (id) {
			var success = function(data) {
				SUGAR.util.evalScript(data.responseText);
				return false;
			}
			var url = "async.php?module=PDFTemplates&action=SampleData&data_module=" + this.module + "&data_id=" + id + "&type=" + this.type;
			SUGAR.conn.asyncRequest(url, {}, success);
			$('DetailForm_preview_label').innerHTML = mod_string('LBL_PREVIEW');
			$('DetailForm_preview').disabled = false;
		} else {
			$('DetailForm_preview_label').innerHTML = mod_string('LBL_PREVIEW_DISABLED');
			$('DetailForm_preview').disabled = true;
			if (this.sampleData) {
				this.data = deep_clone(this.sampleData);
				this.render();
			}
		}
	};

	this.fontFamily = function(f) {
		if (f == 'sans')
			return 'sans-serif';
		return f ? f : 'sans-serif';
	};

	this.fontSize = function(s, num) {
		s = Math.round(s * this.render_width / 720 * 0.9);
		if (!num) s = '' + s + 'px';
		return s;
	};

	this.removeOrpanSignerBlocks = function() {
		var signersInput = SUGAR.ui.getFormInput(PDFTemplateEditor.form, 'signers');
		if (!signersInput) {
			return;
		}
		var signers = signersInput.getRows();
		var updated = false;
		this.layout.sections.forEach(function(section) {
			if (section.type !== 'freeform') return;
			section.elements.forEach(function(elt) {
				var hasSigner = ~(['signature', 'signer_note'].indexOf(elt.type));
				if (hasSigner && elt.signer_ref) {
					if (!signers.find(function(s) {
						return s.ref == elt.signer_ref && s.type !== 'approver';
					})) {
						updated = true;
						elt.deleted = true;
					}
				} else if (elt.type === 'annotation') {
					elt.signer_refs = elt.signer_refs.filter(function(ref) {
						var exists = signers.find(function(s) {
							return s.ref == ref;
						});
						if (!exists) {
							updated = true;
						}
						return exists;
					});
				}
			})
		});
		if (updated) {
			this.mark_dirty();
			this.render();
		}
	};

	this.signerRole = function(ref) {
		var signersInput = SUGAR.ui.getFormInput(PDFTemplateEditor.form, 'signers');
		if (!signersInput) {
			return this.signers[ref];
		}
		var signers = signersInput.getRows();
		var signer = signers.find(function(s) {
			return s.ref == ref;
		});
		return (signer ? signer.role : '') || '';
	};
};

