<?php

require_once 'modules/PDFTemplates/PDFTemplateProviderBase.php';
require_once 'modules/PDFTemplates/PDFTemplate.php';

global $json_supported_actions;
$json_supported_actions['pdftemplates_signing_data'] = array('login_required' => true,);


function json_pdftemplates_signing_data() {

	$result = [
		'signing_enabled' => false,
		'req' => $_REQUEST,
	];

	$list_uids = array_get_default($_REQUEST, 'list_uids', '');
	$record = array_get_default($_REQUEST, 'record', '');
	$template_id = array_get_default($_REQUEST, 'template_id', '');
	if (empty($template_id)) {
		return json_return_value($result);
	}

	if (!empty($list_uids)) {
		$uids = explode(';', $list_uids);
		if ($uids == ['all'] || count($uids) != 1) {
			return json_return_value($result);
		}
		$record = $uids[0];
	}
	if (empty($record) || !is_string($record)) {
		return json_return_value($result);
	}

	$template = ListQuery::quick_fetch('PDFTemplate', $template_id);
	if (!$template || !$template->getField('signing_enabled')) {
		return json_return_value($result);
	}
	$signers = @json_decode($template->getField('signers'), true);
	if (!is_array($signers)) {
		$signers = [];
	}

	$signer_types = PDFTemplateProviderBase::get_predefined_signers_list(
		$template->getField('template_module'),
		$template->getField('template_type'),
		$record
	);

	$fixed = ['Contacts' => [], 'Users' => []];
	$rows = array_reduce(
		$signers,
		function($carry, $row) {
			if ($row['signer']['type'] !== 'Contacts' && $row['signer']['type'] !== 'Users') return $carry;
			$carry[$row['signer']['type']][] = $row['signer']['id'];
			return $carry;
		},
		[
			'Contacts' => [],
			'Users' => [],
		]
	);
	foreach ($rows as $module => $ids) {
		$model = $module === 'Contacts' ? 'Contact' : 'User';
		$lq = new ListQuery($model, ['_display']);
		$lq->addSimpleFilter('id', $ids);
		$result = $lq->fetchAllRows();
		$fixed[$module] = array_map(function($row) {
			return $row['_display'];
		}, $result);
	}

	$inputs = PDFTemplate::collectInputs($template);

	$result['signer_types'] = $signer_types;
	$result['signers'] = $signers;
	$result['fixed_signers'] = $fixed;
	$result['inputs'] = $inputs;
	$result['signing_enabled'] = true;

	json_return_value($result);

}

