AppBase.module('pdf-print-dialog').moduleRequires('ui-controls').init(function(App) {
	var UI = App.ui,
        Dom = App.dom,
		Wrap = Dom.extNode;
	this.initialize = function(form) {
		form = UI.getForm(form);
		var module = form.elements.module.value;
		var record = form.elements.record.value;
		var list_uids = form.elements.list_uids.value;
		var templateInput = UI.getFormInput(form, 'pdf_template');
		var signersInput = UI.getFormInput(form, 'signers');
		var inputsInput = UI.getFormInput(form, 'inputs');
		var tabsInput = UI.getFormInput(form, 'form-tabs');

		var prependInput = UI.getFormInput(form, "prepend_file"); 
		var appendInput = UI.getFormInput(form, "append_file");
		var emailButton = form.querySelector("#" + form.id + "_email");
		var printButton = form.querySelector("#" + form.id + "_print");
		var signButton = form.querySelector("#" + form.id + "_sign");

		var setTabsVisiblity = function(show) {
			var tabs = form.querySelector('.form-tabs');
			if (tabs) {
				tabs.style.display = show ? '' : 'none';
			}
		}

		var dataLoaded = function(data) {
			var templateId = templateInput.getValue();
			var result = data.result;
			setTabsVisiblity(result.signing_enabled);
			if (result.signing_enabled) {
				signersInput.disable_role_editing = true;
				signersInput.clear();
				signersInput.signers = result.signer_types;
				signersInput.fixed_signers = result.fixed_signers;
				result.signers.forEach(function(s) {
					signersInput.insertRow(s);
				});
				inputsInput.clear();
				var inputs = Array.isArray(result.inputs) ? result.inputs : [];
				inputs.forEach(function(input) {
					inputsInput.insertRow(input);
				});
				tabsInput.showHideTab('inputs', inputs.length > 0);
				var t = Wrap(tabsInput.getTab('inputs').outer_elt);
				if (inputs.length > 0) {
					t.show()
				} else {
					t.hide();
				}
			}
			emailButton.style.display = templateId == "_" || result.signing_enabled  ? "none" : "";
			printButton.style.display = result.signing_enabled  ? "none" : "";
			signButton.style.display = !result.signing_enabled  ? "none" : "";

			if (prependInput) prependInput.setDisabled(templateId == "_" || result.signing_enabled);
			if (appendInput) appendInput.setDisabled(templateId == "_" || result.signing_enabled );
		}

		var loadSigningData = function() {
			var templateId = templateInput.getValue();

			emailButton.style.display = 'none';
			printButton.style.display = 'none';
			signButton.style.display = 'none';

			var params = {module: 'PDFTemplates', record: record, list_uids: list_uids, template_id: templateId};
            var req = new App.conn.JSONRequest('pdftemplates_signing_data', {status_msg: app_string('LBL_LOADING')}, params);
            req.fetch(dataLoaded);
            return false;
		}

		UI.attachFormInputEvent(form, 'pdf_template', 'onchange', loadSigningData);
		setTabsVisiblity(false);
		loadSigningData();

	};


	UI.SigningInputsRenderer = function() {
	}

	UI.SigningInputsRenderer.prototype.renderInputId = function(row, field, sect_index, index, editor, cell) {
		var text = row[field];
		return createElement2('span', {}, [
			text,
			row.required ? createElement2('span', {class: "requiredStar"}, '*') : ''
		]);
	}
	UI.SigningInputsRenderer.prototype.renderInputValue = function(row, field, sect_index, index, editor, cell) {
			var value = row[field],
				elt = createElement2('div', {className: 'input-field flat'},
						createElement2('div', {className: 'input-label'}, value));
			elt.onclick = function(evt) {
				var edit = new UI.QuickText({size: 20, show_reset: false});
				edit.init(value, function() {
					editor.updateRow(sect_index, index, {[field]: this.getValue()});
				});
				edit.showPopup(0, this);
				UI.stopEvent(evt);
			}
			return elt;
	}
});
