AppBase.module('signing_projects').moduleRequires('ui-controls').init(function(App) {

	var rowId = 0;

	var UI = App.ui;

	var SignerSelect = UI.SignerSelect = function(id, params) {
		UI.SelectInput.call(this, id, params);
	};

	extendClass(SignerSelect, UI.SelectInput, {
		update: function(key, value, silent) {
			if(key == 'special.select_user')
				this.showSelect('Users');
			else if(key == 'special.select_contact')
				this.showSelect('Contacts');
			else {
				if(isString(value))
					value = {label: value};
				if(! value.icon) value.icon = 'icon-user';
				if(! this.menu.hasKey(key))
					this.menu.addOption(key, value);
				UI.SelectInput.prototype.update.call(this, key, value, silent);
			}
		},
		showSelect: function(module) {
			var request_data = {
					form_name: this.form.getAttribute('id') || this.form.getAttribute('name'),
					call_back_function: 'add_signer_callback',
					passthru_data: {ui_input_name: this.name || this.id, module: module},
					field_to_name_array: this.field_to_name || {id: 'id', label: 'name'}
				},
				params = {module: module, request_data: request_data, inline: true, multiple: false};
			open_popup_window(params);
		},
	});

	var SignersRenderer = UI.SignersRenderer = function() {
	}

	SignersRenderer.prototype = {
		renderRole: function(row, field, sect_index, row_index, editor, cell) {
			if (editor.disable_role_editing) {
				return createElement2('span', {}, row[field]);

			}
			var value = row[field],
				elt = createElement2('div', {className: 'input-field flat'},
						createElement2('div', {className: 'input-label'}, value));
			elt.onclick = function(evt) {
				var edit = new UI.QuickText({size: 20, show_reset: false});
				edit.init(value, function() {
					editor.updateRow(sect_index, row_index, {[field]: this.getValue()});
				});
				edit.showPopup(0, this);
				UI.stopEvent(evt);
			}
			return elt;
		},

		renderRef: function(row, field) {
			return createElement2('span', {}, row[field]);
		},

		renderSigner: function(row, field, sect_index, row_index, editor, cell) {
			var init, d;
			if(! row._init) {
				row._init = { [field]: '' };
			}
			if(! row._init.div) {
				rowId++;
				d = row._init.div = createElement2('div', {id: 'signer-row-' + rowId});
				cell.appendChild(d);

				var keys = editor.signers.map(function(s) {
					return s.key;
				});

				var values = editor.signers.reduce(function(vals, s) {
					vals.push(s.value);
					return vals;
				}, []);

				keys.push('special.select_user');
				keys.push('special.select_contact');
				values.push({
					'label': app_string('LBL_SELECT_SIGNER_USER'),
					'icon': 'theme-icon module-Users'
				});
				values.push({
					'label': app_string('LBL_SELECT_SIGNER_CONTACT'),
					'icon': 'theme-icon module-Contacts'
				});

				var value;
				if (row.signer) {
					if (row.signer.type === 'Contacts' || row.signer.type === 'Users') {
						var signerName = editor.fixed_signers[row.signer.type][row.signer.id];
						if (signerName !== undefined) {
							keys.push('selection');
							values.push({
								label: signerName,
								icon: 'theme-icon module-' + row.signer.type,
								type: row.signer.type,
								id: row.signer.id
							});
							value = 'selection';
						}
					} else {
						value = row.signer.id;
					}
				}

				var sel = new UI.SignerSelect('signer-select-' + rowId,{
						icon_key: 'icon',
						label_key: 'label',
						options: {
							width: '200px',
							keys: keys,
							values: values,
						}
				});
				sel.onchange = function(k, v) {
					var id = k === 'selection' ? v.id : k;
					editor.updateRow(sect_index, row_index, {signer: {id: id, type: v.type}} , true, true);
				}
				d.appendChild(sel.render());
				row._init.ctls = [sel];
				init = function() {
					UI.registerInputs(editor.form, row._init.ctls);
					if (value !== undefined)
						sel.setValue(value);
				}
			} else
				return row._init.div;
			if(init)
				setTimeout(init, 100);
		},

		renderSignerType: function(row, field, sect_index, row_index, editor, cell) {
			var d;
			if(! row._init) {
				row._init = { [field]: 'signer' };
			}
			if(! row._init.type_div) {
				d = row._init.type_div = createElement2('div', {id: 'signer-row-' + rowId});
				cell.appendChild(d);

				var keys = [
					'signer',
					'approver'
				];

				var values = [
					mod_string('LBL_SIGNER_TYPE_SIGNER', 'PDFTemplates'),
					mod_string('LBL_SIGNER_TYPE_APPROVER', 'PDFTemplates')
				];

				var value = row[field] || 'signer';

				var sel = new UI.SelectInput('signer-type-select-' + rowId,{
						options: {
							width: '200px',
							keys: keys,
							values: values,
						}
				});
				sel.onchange = function(k, v) {
					editor.updateRow(sect_index, row_index, {type: k});
				}
				d.appendChild(sel.render());
				init = function() {
					//UI.registerInputs(editor.form, row._init.ctls);
					if (value !== undefined)
						sel.setValue(value);
				}
			} else {
				return row._init.type_div;
			}
			setTimeout(init, 100);
		}
	};

});



function add_signer_callback(result) {
	if(! result) return;
	if (!result.selection_list) return;
	var input = SUGAR.ui.getFormInput(result.form_name, result.passthru_data.ui_input_name);
	
	var id = result.selection_list.ID_0.id;
	var label = result.selection_list.ID_0.label;
	if (!input) return;
	var value = {
		label: label,
		icon: 'theme-icon module-' + result.passthru_data.module,
		type: result.passthru_data.module,
		id: id
	};
	input.menu.addOption('selection', value);
	input.update('selection', value);
}


var addSigner = function(input) {
	var refs = input.getRows().map(function(row) {
		return row.ref;
	});
	var ref = 1;
	for (;;) {
		if (!~refs.indexOf(ref)) break;
		ref++;
	}
	input.insertRow({ ref: ref, role: "Signer " + ref});
}

