<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once 'include/layout/forms/FormTableSection.php';
require_once 'modules/PDFTemplates/utils.php';

class PDFTemplateWidget extends FormTableSection {

	static protected $field_types = array(
		'varchar', 'text',
		'int', 'double', 'bool', 'email', 'enum', 'multienum', 
		'date', 'url', 'ref', 'currency', 'name', 'status',
		'phone', 'fax'
	);

	function init($params=null, $model=null) {
		parent::init($params, $model);
	}
	
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		$json = getJSONObj();
		$form_name = $gen->getFormName();
		$layout = $gen->getLayout();
		$lstyle = $layout->getType();
		$editable = $layout->getDefaultEditable();

		/* There are no means to reach edit view for standard forms via UI, so we do not need to be nice */
		if($row_result->getField('std') && $editable && !$row_result->for_duplicate) {
			pr2('You cannot edit standard PDF forms');
			exit;
		}

		if ($editable) {
			$row_result->assign('std', false);
		}

		$module = $row_result->getField('template_module');
		$type = $row_result->getField('template_type');
		if (isset($_REQUEST['copy_to_module'])) {
			$module = $_REQUEST['copy_to_module'];
			$type = $_REQUEST['copy_to_type'];
		}

		$hidden = '';
		if ($row_result->for_duplicate) {
			$hidden = sprintf(
				'<input type="hidden" name="template_module" value="%s"><input type="hidden" name="template_type" value="%s">',
				$module,
				$type
			);
		}


		$model = AppConfig::module_primary_bean($module);
		$sample = PDFTemplateProviderBase::get_sample_data($module, $type);

		$signers = [];
		if (!$editable) {
			$data = @json_decode($row_result->getField('signers'), true);
			if (!is_array($data)) {
				$data = [];
			}
			foreach ($data as $row) {
				$signers[$row['ref']] = $row['role'];
			}
		}	
		$sample['header']['_'] = array(
			'label' => '',
			'value' => '',
		);

		if (isset($sample['header'])) {
			$modelDef = new ModelDef($model);
			$fields = $modelDef->getFieldDefinitions();
			foreach ($fields as $f) {
				if (isset($f['vname']) && in_array($f['type'], self::$field_types)) {
					$sample['header']['_' . $f['name']] = array(
						'label' => translate($f['vname'], $module),
						'value' => translate($f['vname'], $module),
					);
				}
			}
		}

		// ====================================================
		/*
		$skeleton = PDFTemplateProviderBase::get_template_skeleton($module, $type);
		foreach ($skeleton['sections'] as $k => $s) {
			if ($s['name'] == 'page_footer') {
				unset($skeleton['sections'][$k]);
				break;
			}
		}
		$saved_tpl_layout = $json->decode($row_result->getField('template'));
		if (!is_array($saved_tpl_layout) || !is_array($saved_tpl_layout['sections']))
			$saved_tpl_layout = array('sections' => array());
		$tpl_layout = array();

		foreach ($skeleton['sections'] as $v) {
			$found = false;
			foreach ($saved_tpl_layout['sections'] as $s) {
				if ($s['name'] == $v['name']) {
					$found = $s;
					break;
				}
			}
			if ($found) $tpl_layout['sections'][] = $found;
			else $tpl_layout['sections'][] = $v;
		}
		PDFTemplate::translate_template($tpl_layout, $module);
		*/
		// ====================================================
		$tpl_layout = get_saved_tpl_layout($module, $type, $row_result);
		global $current_language;
		$default_image = "include/language/images/$current_language/default_signature.png";
		if (! file_exists($default_image))
			$default_image = "include/language/images/en_us/default_signature.png";

		$params = [
			'signature_size' => [
				'width' => AppConfig::setting('site.notarius_signature_width', 165),
				'height' => AppConfig::setting('site.notarius_signature_height', 37),
			],
			'signature' => $default_image,
			'pdf_logo' => AppConfig::setting('company.logo_file'),
			'editable' => $editable,
			'signers' => $signers,
			'name' => $this->getName(),
			'layout' => $tpl_layout,
			'data' => $sample,
			'type' => $type,
			'module' => $module,
			'orientation' => $row_result->getField('page_orientation'),
			'margin_left' => $row_result->getField('margin_left'),
			'margin_top' => $row_result->getField('margin_top'),
			'sources' => PDFTemplateProviderBase::get_field_sources($module, $type),
		];
		$layout->addScriptInclude('modules/PDFTemplates/editor.js');
		$layout->addScriptInclude('include/javascript/jscolor/jscolor.js');
		$layout->addScriptLiteral('PDFTemplateEditor = new PDFTemplateEditor_();');
		$layout->addScriptLiteral('PDFTemplateEditor.init(' . $json->encode($params) . ');');
		$layout->addScriptLiteral('PDFTemplateEditor.renderLater();');
		$layout->addScriptLiteral("SUGAR.ui.registerInput('$form_name', PDFTemplateEditor);");


		$frm = $gen->getFormObject();
		$spec = array(
			'name' => '__preview__',
			'type' => 'ref',
			'bean_name' => $model,
			'onchange' => "PDFTemplateEditor.request_data(this.getKey())",
		);
		$ref = $frm->renderRef($row_result, $spec);

		$btn_label = translate('LBL_PREVIEW_DISABLED');
		$btn = '<button type="button" onclick="PDFTemplateEditor.preview(this.form, \'__preview__\');" disabled="disabled" class="input-button disabled" name="DetailForm_preview" id="DetailForm_preview"><div><div class="input-icon left icon-print"></div><span class="input-label" id="DetailForm_preview_label">'.$btn_label.'</span></div></button>';
		if($editable)
			$real_colors = '&nbsp; &nbsp;<div class="input-field input-check" id="DetailForm__real__-input"><input type="checkbox" name="real" id="DetailForm__real" checked value="1" onclick="PDFTemplateEditor.toggle_real_colors(this);"> <label class="input-label" for="DetailForm__real"> ' . translate('LBL_STANDARD_COLORS') . ' </label></div>';
		else
			$real_colors = '';
		$ret =  $hidden . '<div style="text-align: center">' . $ref . '&nbsp;' . $btn . $real_colors . '</div><div style="margin: 10px" id="PDFTemplateEditorLegend"></div><div style="margin: 10px;" id="PDFTemplateEditorContainer"></div>';
		$ret = '<div class="form-section">'
			 . '<div class="row form-row"><div class="column form-cell"><div class="form-entry"><div class="form-value">'
			 . $ret
			 . '</div></div></div></div></div>';
		
		$GLOBALS['pageInstance']->add_js_language($module);

		return $ret;
	}
	
	function getRequiredFields() {
		return array('template_module', 'template_type', 'template');
	}
	
	
	function validateInput(RowUpdate $update) {
		return true;
	}
	
}

?>
