<?php

require_once('include/layout/widgets/DynamicListInput.php');

class SignersWidget extends DynamicListInput {

	var $signers;
	var $fixed_signers;
	var $disable_add_button;

	function init($params=null, $model=null) {
		parent::init($params, $model);
		$this->cols = [
			[
				'name' => 'ref',
				'label' => translate('LBL_SIGNER_REF', 'PDFTemplates'),
				'width' => '5%',
				'render_method' => 'renderRef'
			],
			[
				'name' => 'role',
				'label' => translate('LBL_SIGNER_ROLE', 'PDFTemplates'),
				'width' => '45%',
				'render_method' => 'renderRole'
			],
			[
				'name' => 'type',
				'label' => translate('LBL_SIGNER_TYPE', 'PDFTemplates'),
				'width' => '45%',
				'render_method' => 'renderSignerType'
			],
			[
				'name' => 'signer',
				'label' => translate('LBL_SIGNER_SIGNER', 'PDFTemplates'),
				'width' => '50%',
				'render_method' => 'renderSigner'
			],
		];
		$this->renderer_class = 'SUGAR.ui.SignersRenderer';
		if(! $this->id)
			$this->id = 'signers';
		$this->max_depth = 0;
		$this->disable_add_button = !empty($params['disable_add_button']);
		$this->show_delete = !$this->disable_add_button;
		$this->show_handles = !$this->disable_add_button;
	}
	
	function getRequiredFields() {
		$req = parent::getRequiredFields();
		$req[] = 'signers';
		return $req;
	}
	
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if(!$this->getEditable($context)) {
			return $this->renderHtmlView($row_result);
		}
		$form = $gen->getFormObject();
		
		$module = $row_result->getField('template_module');
		$type = $row_result->getField('template_type');
		$signers = PDFTemplateProviderBase::get_predefined_signers_list($module, $type, $row_result->getField('printing_record'));

		$data = @json_decode($row_result->getField($this->name), true);
		if (!is_array($data)) {
			$data = [];
		}
		$fixed_signers = $this->loadSignersRecords($data);
		$this->signers = $signers;
		$this->fixed_signers = $fixed_signers;
		$row_result->assign($this->name, $data);
		$ret = parent::renderHtml($gen, $row_result, $parents, $context);
		$label = translate('LBL_ADD_SIGNER', 'PDFTemplates');
		$button = <<<HTML
	<div class="buttons-inline">
		<div class="inline-elt" onclick="var input=SUGAR.ui.getFormInput('DetailForm', 'signers'); addSigner(input)" >
			<div id="coupons-coupon-add" class="uii uii-add uii-md active-icon"><label>{$label}</label></div>
		</div>
	</div>
HTML;
		if (!$this->disable_add_button) {
			$ret .= $button;
		}
		$layout = $gen->getLayout();
		$layout->addScriptInclude('modules/PDFTemplates/signers.js');
		$GLOBALS['pageInstance']->add_js_language('PDFTemplates');
		$this->exportIncludes($layout);
		return $ret;
	}

	function loadSignersRecords($data) {
		$ret = ['Contacts' => [], 'Users' => []];
		$rows = array_reduce(
			$data,
			function($carry, $row) {
				if ($row['signer']['type'] !== 'Contacts' && $row['signer']['type'] !== 'Users') return $carry;
				$carry[$row['signer']['type']][] = $row['signer']['id'];
				return $carry;
			},
			[
				'Contacts' => [],
				'Users' => [],
			]
		);
		foreach ($rows as $module => $ids) {
			$model = $module === 'Contacts' ? 'Contact' : 'User';
			$lq = new ListQuery($model, ['_display']);
			$lq->addSimpleFilter('id', $ids);
			$result = $lq->fetchAllRows();
			$ret[$module] = array_map(function($row) {
				return $row['_display'];
			}, $result);
		}
		return $ret;
	}


	function renderHtmlView($row_result) {
		$module = $row_result->getField('template_module');
		$type = $row_result->getField('template_type');
		$signers = PDFTemplateProviderBase::get_predefined_signers_list($module, $type);

		$data = @json_decode($row_result->getField($this->name), true);
		if (!is_array($data)) {
			$data = [];
		}
		$fixed_signers = $this->loadSignersRecords($data);
		$rows = '';
		$odd_even = 'odd';
		foreach ($data as $row) {
			$label = '';
			$icon = '';
			$ref = $row['ref'];
			$role = to_html($row['role']);
			$mode = translate($row['type'] === 'approver' ? 'LBL_SIGNER_TYPE_APPROVER' : 'LBL_SIGNER_TYPE_SIGNER', 'PDFTemplates');
			$signer = $row['signer'];
			$tag = 'i';
			switch ($signer['type']) {
			case 'Contacts':
				$icon = 'module-Contacts';
				$label = to_html(array_get_default($fixed_signers['Contacts'], $signer['id'], ''));
				$tag = 'span';
				break;
			case 'Users':
				$icon = 'module-Users';
				$label = to_html(array_get_default($fixed_signers['Users'], $signer['id'], ''));
				$tag = 'span';
				break;
			default:
				foreach ($signers as $s) {
					if ($s['key'] === $signer['id']) {
						$label = to_html($s['value']['label']);
						$icon = $s['value']['icon'];
						break;
					}
				}
			}
			$rows .= <<<HTML
					<tr class="listViewRow {$odd_even}ListRowS1">
						<td class="depthColumn"><span>$ref</span></td>
						<td><span>$role</span></td>
						<td><span>$mode</span></td>
						<td><div id="signer-select-1-icon" class="input-icon active theme-icon $icon" style="display: inline-block;"></div> <$tag>$label</$tag></td>
					</tr>
HTML;
			$odd_even = $odd_even === 'odd' ? 'even' : 'odd';
		}
		$label_ref = translate('LBL_SIGNER_REF', 'PDFTemplates');
		$label_role = translate('LBL_SIGNER_ROLE', 'PDFTemplates');
		$label_mode = translate('LBL_SIGNER_MODE', 'PDFTemplates');
		$label_signer = translate('LBL_SIGNER_SIGNER', 'PDFTemplates');
		$html = <<<HTML
			<table width="100%" class="listEditor panel-outer panel-default panel-border" cellpadding="0" cellspacing="0">
				<thead>
					<tr class="listEditorHead">
						<th style="width: 5%;"><span>$label_ref</span></th>
						<th style="width: 35%;"><span>$label_role</span></t>
						<th style="width: 30%;"><span>$label_mode</span></t>
						<th style="width: 30%;"><span>$label_signer</span></th>
					</tr>
				</thead>
				<tbody>$rows</tbody>
			</table>
HTML;
		return $html;
	}


	protected function getOptionalAttributes() {
		return [
			'signers' => $this->signers,
			'fixed_signers' => $this->fixed_signers,
		];
	}

}

