<?php

namespace OneCRM\PaymentProcessors;

use OneCRM\API\Errors\BadRequest;

class AddonRecord extends ChargeBeeRecord{

    public $name = 'Addon';                     // name of object at Payment Processor
    public $ppid_field = 'id';                  // PP unique id field name
    public $model = 'Addon';                    // name of matching 1CRM Model
    public $model_ppid_field = 'pp_addon_id';   // 1CRM Model field name holding PP unique id
    public $map = [
        'id' => 'pp_addon_id',
        'name' => 'name',
        'description' => 'description',
        'period_unit' => 'billing_cycle',   // map_types
        'period' => 'billing_cycle_mult',
        'status' => 'active',               // map_types
        'unit' => 'qty_label',
        'updated_at' => 'date_modified',
        'currency_code' => 'currency',      // ISO
        'pricing_model' => 'pricing_model',
        'tiers' => 'pricing_tiers',         // translated by parent
        'price' => 'amount',                // translated by parent
    ];
    public $map_types = [                   // native => onecrm
        'status' => [
            'active' => 'Active',
            'archived' => 'Inactive',
        ],
        'pricing_model' => [
            'flat_fee' => 'Flat Fee',
            'per_unit' => 'Per Unit',
            'volume' => 'Volume',
            'stairstep' => 'Stairstep',
            'tiered' => 'Tiered',
        ],
        'period_unit' => [
            'not_applicable' => 'Single',
            'week' => 'Weekly',
            'month' => 'Monthly',
            'year' => 'Yearly',
        ],
    ];
    public $map_meta_data = [                 // custom records
        'product_id' => 'product_id',
        'qty_label' => 'qty_label',
    ];


    /**
     * normalization of records:
     */

    /**
     * translates $this->native to $this->onecrm
     * @param array|null $data
     * @return BaseRecord $this
     * @throws BadRequest
     */
    public function to_onecrm($data=null)
    {
        $success=parent::to_onecrm($data);
        $this->onecrm['global'] = 0; // Chargebee doesn't support Global Addons
        return $success;
    }

    /**
     * translates $this->onecrm to $this->native
     * @param array|null $data
     * @return BaseRecord $this
     */
    public function to_native($data=null)
    {
        // copy the fields defined by the map:
        parent::to_native($data);

        // accon-specific fields:
        die('TODO: addon to_native()');

        return $this;
    }

    public function onecrm_save($before_update = null)
    {
        $onecrm = &$this->onecrm;
        if (!$onecrm['id'] = \Subscription::find_addon($onecrm)) {
            if (! @$onecrm['product_id']) {
                $res = \Subscription::query_model('Product',['name'=>$onecrm['name']]);
                if ($res) {
                    $onecrm['product_id'] = $res['id'];
                } else {
                    // proceed without a product id. Maybe a warning email?
                }
            }
        }
        return parent::onecrm_save($before_update);
    }


}
