<?php

namespace OneCRM\PaymentProcessors;

use OneCRM\API\Errors\BadRequest;
use OneCRM\API\Errors\InternalError;

class CreditNoteLineRecord extends ChargeBeeRecord
{

    public $name = 'CreditNoteLineItem';               // name of object at Payment Processor
    public $ppid_field = 'id';                      // PP unique id field name
    public $model = 'CreditNoteLine';                  // name of matching 1CRM Model
    public $model_ppid_field = 'pp_lineitem_id';    // 1CRM Model field name holding PP unique id
    public $map = [
        "id" => "pp_lineitem_id",
        'description' => 'name',
        'date_from' => 'date_from',
        'date_to' => 'date_to',
        'quantity' => 'quantity',
        'unit_amount' => 'unit_price',
        'amount' => 'ext_price',
    ];

    /**
     * normalization of records:
     */

    /**
     * CreditNoteLineRecord constructor.
     * @param array $data
     * @param string $format = 'native' by default, this is the only accepted format.
     * @param string|null $processor_id
     * @throws InternalError
     */
    public function __construct(array $data = null, string $format = 'native', string $processor_id = null)
    {
        if (! $format==="native") throw new InternalError('unsuppored format');
        return parent::__construct($data, $format, $processor_id);
    }

    /**
     * translates $this->native to $this->onecrm
     * @param null $data
     * @return BaseRecord $this
     * @throws BadRequest
     */
    public function to_onecrm($data = null)
    {
        // copy the fields defined by the map:
        parent::to_onecrm($data);
        $onecrm =& $this->onecrm;
        $native =& $this->native;

        // sync related plan / addon
        $where=$onecrm;
        if ($native['entity_type']=='plan'){
            $where['pp_plan_id']=$native['entity_id'];
            $row = \Subscription::find_plan($where,['id','product_id']);
            if ($row['id']??null){
                $plan = new PlanRecord($native['entity_id'],'pp_id');
                $plan->onecrm_save();
                $row=$plan->onecrm;
            };
        } else {
            $where['pp_addon_id']=$native['entity_id'];
            $row = \Subscription::find_addon($where,['id','product_id']);
            if ($row['id']??null){
                $addon = new AddonRecord($native['entity_id'],'pp_id');
                $addon->onecrm_save();
                $row=$addon->onecrm;
            };
        }
        $onecrm['related_type'] = 'ProductCatalog';
        $onecrm['related_id'] = $row['product_id']??null;

        return $this;
    }

    /**
     * @param null $before_update
     * @throws InternalError
     */
    public function onecrm_save($before_update = null)
    {
        throw new InternalError('Unsupported method for '.__CLASS__);
    }

}
