<?php

namespace OneCRM\PaymentProcessors;


class CreditNotePayments 
{

	public static function allocate_payments($payments_desc) {
		$cid = $payments_desc['currency_id'];
		$rate = $payments_desc['exchange_rate'];
		$for_credit = $payments_desc['for_credit'] ?? null;
		$for_invoice = $payments_desc['for_invoice'] ?? null;
		$keep = [];
		$cache = [];
		foreach ($payments_desc['allocations'] as $alloc) {
			if (!isset($cache['creditnotes'][$alloc['cn_id']])) {
				$cache['creditnotes'][$alloc['cn_id']] = \Subscription::find_creditnote(['pp_credit_id' => $alloc['cn_id']]);
			}
			$cn_id = $cache['creditnotes'][$alloc['cn_id']] ;
			if (!isset($cache['invoices'][$alloc['inv_id']])) {
				$cache['invoices'][$alloc['inv_id']] = \Subscription::find_invoice(['pp_invoice_id' => $alloc['inv_id']]);
			}
			$inv_id = $cache['invoices'][$alloc['inv_id']] ;
			if (!$inv_id || !$cn_id) {
				continue;
			}

			$lq = new \ListQuery('Payment');
			$lq->addSimpleFilter('pp_credit_credit', $cn_id); 
			$lq->addSimpleFilter('pp_credit_invoice', $inv_id); 
			$lq->addSimpleFilter("payment_processor_id", $payments_desc['processor_id']);
			$results = $lq->fetchAll();
			$existing = null;
			foreach($results->getRowIndexes() as $idx) {
				$existing = $results->getRowUpdate($idx);
			}
			$p = $existing ? $existing : \RowUpdate::blank_for_model('Payment');
			$p->set([
				'assigned_user_id' => \AppConfig::current_user_id(),
				'direction' => 'credit',
				'payment_type' => 'Credit Note',
				'account_id' => $payments_desc['account_id'],
				'payment_date' => gmdate('Y-m-d H:i:s', $alloc['applied_at']),
				'exchange_rate' => $rate,
				'currency_id' => $cid,
				'amount' => $alloc['amount'] / 100,
				'applied_amount' => $alloc['amount'] / 100,
				'pp_credit_credit' => $cn_id,
				'pp_credit_invoice' => $inv_id,
				'payment_processor_id' => $payments_desc['processor_id'],
				'related_invoice_id' => $inv_id,
			]);
			$p->save();
			$keep[$p->getField('id')] = true;
			$p->removeAllLinks('credits');
			$p->removeAllLinks('invoices');
			$p->addUpdateLink('credits', array(
				'id' => $cn_id,
				'amount' => $alloc['amount'] / 100,
				'currency_id' => $cid,
				'exchange_rate' => $rate,
			));
			$p->addUpdateLink('invoices', array(
				'id' => $inv_id,
				'amount' => $alloc['amount'] / 100,
				'currency_id' => $cid,
				'exchange_rate' => $rate,
			));
		}
		if ($for_credit || $for_invoice) {
			$filter_field = $for_credit ? 'pp_credit_credit' : 'pp_credit_invoice';
			$filter_value = $for_credit ? $for_credit : $for_invoice;
			$lq = new \ListQuery('Payment');
			$lq->addSimpleFilter($filter_field, $filter_value);
			$res = $lq->fetchAll();
			foreach ($res->getRowIndexes() as $idx) {
				$upd = $res->getRowUpdate($idx);
				$id = $upd->getField('id');
				if (empty($keep[$id])) {
					$upd->markDeleted();
				}
			}
		}
	}

}
