<?php

namespace OneCRM\PaymentProcessors;

use OneCRM\API\Errors\InternalError;


class InvoiceAdjustmentRecord extends ChargeBeeRecord
{
    public $name = '';                              // name of object at Payment Processor
    public $ppid_field = 'id';                      // PP unique id field name
    public $model = 'InvoiceAdjustment';            // name of matching 1CRM Model
    public $model_ppid_field = 'pp_lineadjust_id';  // 1CRM Model field name holding PP unique id
    public $map = [
        "line_item_id" => "pp_lineadjust_id",
        'coupon_id' => 'pp_coupon_id',
        'description' => 'name',
        'position' => 'position',
        'object'    => 'related_type',
        'discount_type' => 'type',
        'discount_amount' => 'amount',
    ];
    public $map_types = [
        'object'=> [
            'line_item_discount' => 'Discounts',
        ],
        'discount_type' => [
            'item_level_coupon' => 'StdFixedDiscount',
            'document_level_coupon' => 'StdFixedDiscount',
        ],
    ];

    /**
     * InvoiceAdjustRecord constructor.
     * @param array $data
     * @param string $format = 'native' by default, this is the only accepted format.
     * @param string|null $processor_id
     * @throws InternalError
     */
    public function __construct(array $data = null, string $format = 'native', string $processor_id = null)
    {
        if (! $format==="native") throw new InternalError('unsuppored format');
        return parent::__construct($data, $format, $processor_id);
    }

    /**
     * @param null $before_update
     * @throws InternalError
     */
    public function onecrm_save($before_update = null)
    {
        throw new InternalError('Unsupported method for '.__CLASS__);
    }

}
