<?php

require_once 'modules/Quotes/PDFTemplatesProvider.php';
require_once 'modules/Payments/Payment.php';

class PaymentPDFTemplateProvider extends QuotePDFTemplateProvider
{
	
	protected function __construct($type)
	{
		parent::__construct($type);
	}

	public static function template_types()
	{
		return array(
			'incoming' => translate('LBL_PDF_INCOMING', 'Payments'),
			'outgoing' => translate('LBL_PDF_OUTGOING', 'Payments'),
		);
	}

	public static function template_skeleton($type)
	{
		$ret = array('sections' => array(
			array(
				'name' => 'header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_HEADER',
				'height' => 150,
				'elements' => array (),
			),
			array(
				'name' => 'credit_items',
				'type' => 'table',
				'title' => 'LBL_SKEL_CREDIT_ITEMS',
				'columns' => array (),
			),
			array(
				'name' => 'payment_items',
				'type' => 'table',
				'title' => 'LBL_SKEL_PAYMENT_ITEMS',
				'columns' => array (),
			),
			array(
				'name' => 'footer',
				'data_id' => 'header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_FOOTER',
				'height' => 40,
				'elements' => array (),
			),
		));
		foreach ($ret['sections'] as &$sec) $sec['title'] = translate($sec['title'], 'Invoice');
		$ret['sections'] = array_values($ret['sections']);
		return $ret;
	}

	public static function template_sample_data($type)
	{
		$datefmt = AppConfig::setting('locale.defaults.date_format');
		$data = array(
			'header' => array(
				'todays_date' => array(translate('LBL_TODAYS_DATE'), date($datefmt), ),
				'number' => array(translate('LBL_SAMPLE_HEADER_NUMBER_LABEL', 'Payments'), 'P2013-001'),
				'date' => array(translate('LBL_SAMPLE_HEADER_DATE_LABEL', 'Payments'), '2014-11-05'),
				'custref' => array(translate('LBL_SAMPLE_HEADER_CUSTREF_LABEL', 'Payments'), 'CR12345'),
				'currency' => array(translate('LBL_SAMPLE_HEADER_CURRENCY_LABEL', 'Payments'), translate('LBL_SAMPLE_HEADER_CURRENCY_VALUE', 'Payments'), ),
				'amount' => array(translate('LBL_SAMPLE_HEADER_AMOUNT_LABEL', 'Payments'), '1234.56'),
				'payment_type' => array(translate('LBL_SAMPLE_HEADER_PAYMENT_TYPE_LABEL', 'Payments'), translate('LBL_SAMPLE_HEADER_PAYMENT_TYPE_VALUE', 'Payments'), ),
				'billing_address' => array(translate('LBL_SAMPLE_HEADER_BILLING_ADDRESS_LABEL', 'Payments'), translate('LBL_SAMPLE_HEADER_BILLING_ADDRESS_VALUE', 'Payments'), ),
				'shipping_address' => array(translate('LBL_SAMPLE_HEADER_SHIPPING_ADDRESS_LABEL', 'Payments'), translate('LBL_SAMPLE_HEADER_SHIPPING_ADDRESS_VALUE', 'Payments'), ),
				'description' => array(translate('LBL_SAMPLE_HEADER_DESCRIPTION_LABEL', 'Payments'), translate('LBL_SAMPLE_HEADER_DESCRIPTION_VALUE', 'Payments'), ),
			),
			'credit_items' => array(
				'number' => array(translate('LBL_SAMPLE_PAYMENT_ITEMS_NUMBER_LABEL', 'Payments'), '2013-152'),
				'subject' => array(translate('LBL_SAMPLE_PAYMENT_ITEMS_SUBJECT_LABEL', 'Payments'), translate('LBL_SAMPLE_PAYMENT_ITEMS_SUBJECT_VALUE', 'Payments'), ),
				'amount' => array(translate('LBL_SAMPLE_PAYMENT_ITEMS_AMOUNT_LABEL', 'Payments'), '1234.56'),
				'amount_due' => array(translate('LBL_SAMPLE_PAYMENT_ITEMS_AMOUNT_DUE_LABEL', 'Payments'), '12.56'),
				'allocated' => array(translate('LBL_SAMPLE_PAYMENT_ITEMS_ALLOCATED_LABEL', 'Payments'), '1234.56'),
				'date_due' => array(translate('LBL_SAMPLE_PAYMENT_ITEMS_DATE_DUE_LABEL', 'Payments'), '2014-11-05'),
				'invoice_reference' => array(translate('LBL_SAMPLE_PAYMENT_ITEMS_SUPPLIER_INV_LABEL', 'Payments'), '123456'),
				'bill_date' => array(translate('LBL_SAMPLE_PAYMENT_ITEMS_SUPPLIER_INV_DATE_LABEL', 'Payments'), '2016-09-26'),
			),
			'payment_items' => array(
				'number' => array(translate('LBL_SAMPLE_PAYMENT_ITEMS_NUMBER_LABEL', 'Payments'), '2013-152'),
				'subject' => array(translate('LBL_SAMPLE_PAYMENT_ITEMS_SUBJECT_LABEL', 'Payments'), translate('LBL_SAMPLE_PAYMENT_ITEMS_SUBJECT_VALUE', 'Payments'), ),
				'amount' => array(translate('LBL_SAMPLE_PAYMENT_ITEMS_AMOUNT_LABEL', 'Payments'), '1234.56'),
				'amount_due' => array(translate('LBL_SAMPLE_PAYMENT_ITEMS_AMOUNT_DUE_LABEL', 'Payments'), '12.56'),
				'allocated' => array(translate('LBL_SAMPLE_PAYMENT_ITEMS_ALLOCATED_LABEL', 'Payments'), '1234.56'),
				'date_due' => array(translate('LBL_SAMPLE_PAYMENT_ITEMS_DATE_DUE_LABEL', 'Payments'), '2014-11-05'),
				'invoice_reference' => array(translate('LBL_SAMPLE_PAYMENT_ITEMS_SUPPLIER_INV_LABEL', 'Payments'), '123456'),
				'bill_date' => array(translate('LBL_SAMPLE_PAYMENT_ITEMS_SUPPLIER_INV_DATE_LABEL', 'Payments'), '2016-09-26'),
			),
		);
		if ($type == 'outgoing') {
			unset($data['header']['shipping_address']);
		} else {
			unset($data['payment_items']['invoice_reference']);
			unset($data['payment_items']['bill_date']);
		}
		$ret = array();
		foreach ($data as $k => $v) {
			$ret[$k] = array();
			foreach ($v as $kk => $vv) {
				$ret[$k][$kk] = array('label' => $vv[0], 'value' => $vv[1]);
			}
		}
		return $ret;
	}


	public static function create($type)
	{
		return new PaymentPDFTemplateProvider($type);
	}
	
	public function iterate(PDFTemplatePrinter $p)
	{
		$this->iterateHeaderFooter($p, 'header');
		if(! empty($this->data['credit_items']))
			$p->renderSection($this->findSection('credit_items'), $this->data['credit_items']);
		$p->renderSection($this->findSection('payment_items'), $this->data['payment_items']);
		$this->iterateHeaderFooter($p, 'footer');
	}

	protected function _address($pfx, $row=null)
	{
		$focus = $row ? $row : $this->row;

		$acc = ListQuery::quick_fetch('Account', $focus->getField('account_id'));
		if ($acc) {
			$nm = trim($acc->getField('invoice_as'));
			if(! strlen($nm)) $nm = trim($acc->getField('name'));
			if ($pfx == 'billing_') {
				$acc->assign('billing_account_name', $nm);
				return parent::_address('billing_', $acc);
			}
			else {
				$acc->assign('shipping_account_name', $nm);
				return parent::_address('shipping_', $acc);
			}
		}
	}
	
	public function initialize($layout, ListQuery $lq)
	{
		$lq->addField('supplier_contact');
		return parent::initialize($layout, $lq);
	}
	
	public function getFileName() {
		$filename_pfx = translate('LBL_PDF_RECEIPT', 'Payments');
		$filename = $filename_pfx . '_' . $this->row->getField('prefix');
		$filename .= $this->row->getField('payment_id') . '_' . $this->row->getField('account_name');
		$filename = preg_replace('~\.+$~', '', trim($filename));
		$filename = $filename . '.pdf';
		return $filename;
	}

	public function getNoteTitle()
	{
		$mod = ($this->type == 'outgoing') ? 'PaymentsOut' : 'Payments';
		return translate('LBL_PREPARED_RECEIPT', $mod);
	}

	public function getEmailTitle()
	{
		$mod = ($this->type == 'outgoing') ? 'PaymentsOut' : 'Payments';
		return translate('LBL_EMAILED_RECEIPT', $mod);
	}
	
	public function getSignedDocumentTitle() {
		$prefix =
			$this->row
			? '[' . $this->row->getField('prefix') . $this->row->getField('payment_id') . '] '
			: '';

		return $prefix . $this->getNoteTitle();
	}

	protected function createData()
	{
		global $app_list_strings;
		$row =& $this->row;
		$user = new User();
		$user->retrieve($row->getField('assigned_user_id'), false);
		$terms = AppConfig::setting("lang.lists.current.app.terms_dom", array());
		$currency =& $this->get_currency();
		$tax_information = $this->row->getField('tax_information');
		$datefmt = AppConfig::setting('locale.defaults.date_format');
		$data = array(
			'todays_date' => date($datefmt),
			'number' => $row->getField('prefix') . $row->getField('payment_id'),
			'custref' => $row->getField('customer_reference'),
			'currency' => $currency->name . ':' . $currency->symbol,
			'amount' => $this->currency_format($row->getField('amount')),
			'payment_type' => array_get_default($app_list_strings['payment_type_dom'], $row->getField('payment_type')),
			'billing_address' => $this->_address('billing_'),
			'shipping_address' => $this->_address('shipping_'),
			'description' => $row->getField('notes'),
			'date' => $row->getField('payment_date', '', true),
		);
		$data = array_merge($this->header_data, $data);
		if ($this->type == 'outgoing') {
			unset($data['shipping_address']);
		}
		$this->data['header'] = $data;
		$this->createPaymentsData();
	}

	protected function createPaymentsData()
	{
		global $timedate;
		$data = array();
		$currency = $this->get_currency();
		$items = Payment::query_line_items($this->row->getField('id'), true);
		$items_key = ($this->row->getField('direction') == 'incoming') ? 'invoices' : 'bills';
		foreach($items[$items_key] as $key => $value) {
			$data[] = array(
				'number' => $value['invoice_no'],
				'subject' => $value['invoice_name'],
				'amount' => $this->currency_format($value['invoice_amount'], $currency->id),
				'amount_due' => $this->currency_format($value['invoice_amount_due'], $currency->id),
				'allocated' => $this->currency_format($value['amount'], $currency->id),
				'date_due' => $timedate->to_display_date($value['invoice_due_date'], false),
				'invoice_reference' => array_get_default($value, 'invoice_reference', ''),
				'bill_date' => $timedate->to_display_date($value['bill_date'], false),
			);
		}
		$this->data['payment_items'] = $data;

		if(! empty($items['credits'])) {
			$credits = [];
			foreach($items['credits'] as $key => $value) {
				$credits[] = array(
					'number' => $value['invoice_no'],
					'subject' => $value['invoice_name'],
					'amount' => $this->currency_format($value['invoice_amount'], $currency->id),
					'amount_due' => $this->currency_format($value['invoice_amount_due'], $currency->id),
					'allocated' => $this->currency_format($value['amount'], $currency->id),
					'date_due' => '',
					'invoice_reference' => '',
					'bill_date' => $timedate->to_display_date($value['bill_date'], false),
				);
			}
			$this->data['credit_items'] = $credits;
		}
	}

	public static function get_extra_sources() {
		return array(
			array(
				'module' => 'Accounts',
				'model' => 'Account',
				'prefix' => 'account',
				'label' => 'LBL_ACCOUNT_ID',
			),
			array(
				'module' => 'Invoice',
				'model' => 'Invoice',
				'prefix' => 'related_invoice',
				'label' => 'LBL_RELATED_INVOICE',
			),
		);
	}
}
	
