<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


require_once('data/SugarBean.php');

class Payment extends SugarBean 
{
	// Stored fields

    var $id;
    var $deleted;
    var $date_entered;
    var $date_modified;
    var $assigned_user_id;
	var $assigned_user_name;
    var $created_by;
    var $currency_id;
    var $exchange_rate;
	var $currency;
    var $amount;
    var $amount_usdollar;
    var $account_id;
	var $account_name;
    var $payment_date;
    var $payment_id;
    var $customer_reference;
    var $notes;
    var $payment_type;
	var $currency_symbol;
	var $prefix;
	var $direction;
	var $refunded;
	var $refund = 0;

	// used by subpanel
	var $invoice_currency_id;
	var $allocated;

	var $line_items = array();
	var $delete_line_items = array();

	var $table_name = 'payments';
	var $object_name = 'Payment';
	var $module_dir = 'Payments';
	var $new_schema = true;
	
	static $line_links = array(
		'invoices' => array('invoices_payments', 'invoice'),
		'bills' => array('bills_payments', 'bill'),
		'credits' => array('credits_payments', 'credit'),
	);
        
	
	static function getCurrentPrefix() {
		return AppConfig::setting('company.payment_prefix');
	}
	

	static function getCurrentSequenceValue() {
		return AppConfig::current_sequence_value('payment_number_sequence');
	}
	
	static function getNextSequenceValue() {
		return AppConfig::next_sequence_value('payment_number_sequence');
	}
	
	
    function get_allocated_total()
    {
        return ($this->number_formatting_done ? unformat_number($this->amount) : $this->amount);
    }
	

    static function after_delete(RowUpdate $upd) {
    	$lines = self::query_line_items($upd->getPrimaryKeyValue());
    	self::grab_related_ids($lines, $related);
    	self::update_related_amounts($upd, $related);
    }

    static function update_related_amounts(RowUpdate $upd, array $related) {
    	$mod = new ModelDef('Payment');
    	foreach($related as $link => $rows) {
    		$target = $mod->getLinkTargetModel($link);
    		if(! $target || ! $rows) continue;
    		$lq = new ListQuery($target);
    		$result = $lq->queryRecordSet(array_keys($rows));
    		if($result && ! $result->failed) {
    			foreach($result->getRowIndexes() as $idx) {
    				$row = $result->getRowResult($idx);
    				$upd = RowUpdate::for_result($row);
    				$upd->save();
    			}
    		}
    	}
    }
    
    static function grab_related_ids($lines, &$target) {
    	if(! isset($target)) $target = array();
    	foreach(self::$line_links as $k => $info) {
    		if(! isset($lines[$k])) continue;
    		$idf = $info[1].'_id';
			foreach($lines[$k] as $row) {
				$target[$k][$row[$idf]] = 1;
			}
		}
    }
        
    static function query_line_items($payment_id, $get_names=false, $limit_links=null) {
        $ret = array();
        foreach(self::$line_links as $k => $_)
        	$ret[$k] = array();

    	if($payment_id) {
			$lq = new ListQuery();
			$lc = 0;
			foreach(self::$line_links as $k => $info) {
				if($limit_links && ! in_array($k, $limit_links))
					continue;
				$lq1 = new ListQuery($info[0], array(
					'invoice_id' => $info[1].'_id', 'payment_id', 'date_modified',
					'amount_usd' => 'amount_usdollar', 'amount', 'currency_id', 'exchange_rate'),
					array('alias' => $k));
				if($get_names) {
					$lq1->addFields(array(
						'invoice_name' => $info[1] . '.name',
						'invoice_no' => $info[1] . '.full_number',
						'invoice_display' => $info[1],
						'invoice_currency_id' => $info[1] . '.currency_id',
						'invoice_amount' => $info[1] . '.amount',
						'invoice_amount_usd' => $info[1] . '.amount_usdollar',
						'invoice_amount_due' => $info[1] . '.amount_due',
						'invoice_amount_due_usd' => $info[1] . '.amount_due_usdollar',
						'invoice_due_date' => $info[1] . '.due_date',
					));
					if ($info[1] == 'bill') {
						$lq1->addFields(array(
							'invoice_reference' => $info[1] . '.invoice_reference',
							'bill_date' => $info[1] . '.bill_date',
						));
					} else {
						$lq1->addFieldLiteral('invoice_reference', '""');
						$lq1->addFieldLiteral('bill_date', '""');
					}
				}
				$lq1->addSimpleFilter('payment_id', $payment_id);
				$lq1->addSourceNameField('source');
				$lq->addUnionQuery($lq1, $k);
				$lc ++;
			}
			if($lc)
			foreach($lq->fetchAllRows() as $row) {
				$rel_f = self::$line_links[$row['source']][1] . '_id';
				if($rel_f != 'invoice_id')
					$row[$rel_f] = $row['invoice_id'];
				$row['is_credit'] = ($row['source'] == 'credits') ? 1 : 0;
				$ret[$row['source']][$row[$rel_f]] = $row;
			}
		}

        return $ret;
    }

	static function before_subpanel_create($mgr, &$stop)
	{
		$stop = true;
	}
    
	static function after_add_link(RowUpdate $upd, $link_name) {
    	if(isset(self::$line_links[$link_name])) {
			$rel_id = array_get_default($upd->link_update->saved, self::$line_links[$link_name][1] . '_id');
			$tmodel = $upd->getLinkTargetModel($link_name);
			$result = ListQuery::quick_fetch($tmodel, $rel_id);
			if($result && ! $result->failed) {
				$rel = RowUpdate::for_result($result, null, $upd->getUserId());
				if ($tmodel->getModelName() === 'Invoice') {
					$rel->set('locked', true);
				}
				$rel->save();
    		}
		}
		if($link_name == 'invoices') {
			self::direct_update_related($upd->getPrimaryKeyValue());
		}
    }
    
    static function direct_update_related($payment_id) {
    	global $db;
    	$pid = $db->quote($payment_id);
		$q =  "UPDATE payments p LEFT JOIN "
			. "(SELECT COUNT(*) c, payment_id, invoice_id FROM invoices_payments WHERE NOT deleted GROUP BY payment_id) ipj "
			. "ON ipj.payment_id=p.id "
			. "SET p.related_invoice_id=IF(ipj.c=1, ipj.invoice_id, NULL) "
			. "WHERE p.id='$pid'";
    	$db->query($q);
    }
    
    static function update_line_items(RowUpdate $upd, $existing, $lines) {
    	$delete = array();

    	foreach(self::$line_links as $k => $info) {
    		$idf = $info[1].'_id';
    		$delete[$k] = array();
    		
    		if(isset($existing[$k])) {
				foreach($existing[$k] as $row) {
					$delete[$k][$row[$idf]] = 1;
				}
			}
			
			if(isset($lines[$k])) {
				foreach($lines[$k] as $row) {
					$upd->addUpdateLink($k, $row);
					$delete[$k][$row[$idf]] = 0;
				}
			}
			
			foreach($delete[$k] as $id => $v) {
				if($v) $upd->removeLink($k, $id);
			}
    	}
    }

    static function init_record(RowUpdate &$upd, $input) {
        global $timedate, $current_user;
        $update = array();

		if (empty($update['payment_date'])) {
			$update['payment_date'] = date('Y-m-d', $timedate->user_local_time());
		}

        if(! empty ($input['invoice_id'])) {
            $upd->set('direction', 'incoming');
            $source_name = 'Invoice';
            $source_id = $input['invoice_id'];
            $source_field = 'billing_account_id';
        } else if(! empty ($input['creditnote_id'])) {
            $source_name = 'CreditNote';
            $source_id = $input['creditnote_id'];
            $source_field = 'billing_account_id';
            $upd->set('direction', 'credit');
            $upd->set('payment_type', 'Credit Note');
        } else if (! empty($input['bill_id'])) {
            $source_name = 'Bill';
            $source_id = $input['bill_id'];
            $source_field = 'supplier_id';
            $update['direction'] = 'outgoing';
        } else {
			$source_name = null;
			$source_id = null;
			$source_field = '';
		}

        if ($source_name && $source_id) {
            $source = ListQuery::quick_fetch_row($source_name, $source_id, array($source_field, 'amount_due', 'currency_id', 'exchange_rate'));
			if ($source) {
                $update['account_id'] = $source[$source_field];
                if($source_name != 'CreditNote' && (int)$source['amount_due'] >= 0) {
                    $update['amount'] = $source['amount_due'];
                } else {
                    $update['amount'] = 0;
                }
                $update['currency_id'] = $source['currency_id'];
                //$update['exchange_rate'] = $source['exchange_rate'];
                $update['applied_currency_id'] = $update['currency_id'];
                $update['applied_amount'] = $update['amount'];
            }
        }

        $upd->set($update);
    }
    
    static function fill_defaults(RowUpdate $upd) {
    	$cid = $upd->getField('currency_id');
    	$appCid = $upd->getField('applied_currency_id');
    	if(! $appCid) {
    		$upd->set('applied_currency_id', $appCid = $cid);
    		$upd->set('applied_amount', $upd->getField('amount'));
    		$upd->set('effective_rate', 1.0);
    	}
    	$upd->set('currency_converted', $cid == $appCid ? 0 : 1);
		$upd->set('total_amount', $upd->getField('amount'));
    }
    
    static function before_save(RowUpdate $upd) {
    	$cid = $upd->getField('currency_id');
    	$appCid = $upd->getField('applied_currency_id');
    	if(! $appCid) {
    		$upd->set('applied_currency_id', $appCid = $cid);
    	}
    	$upd->set('currency_converted', $cid == $appCid ? 0 : 1);
    	self::load_line_items($upd);
		$applied = $upd->getField('applied_amount');
		$upd->set('total_amount', $applied);
		$amount = $upd->getField('amount');
    	$upd->set('effective_rate', round(
    		($amount == $applied || ! $applied) ? 1 : $amount / $applied,
    		5));
    }
    
	static function get_input_line_items(&$input, $direction) {
        $line_inputs = [
        	'payment_items' => ($direction == 'outgoing' ? 'bills' : 'invoices'),
        	'credit_items' => 'credits',
        ];
        $ret = [];        
        $json = getJSONobj();
        
        foreach($line_inputs as $input_f => $key) {
        	$ret[$key] = [];
			if(isset($input[$input_f])) {
				$items = $json->decode($input[$input_f]);
				if(is_array($items)) {
					foreach($items as $line) {
						$amount = (float)array_get_default($line, 'amount');
						if(! empty($line['invoice_id']) && $amount) {
							$rel_id = self::$line_links[$key][1] . '_id';
							$cid = array_get_default($line, 'invoice_currency_id');
							$exrt = (float)array_get_default($line, 'invoice_exchange_rate');
							if(! $exrt) $exrt = 1;
							$newline = [
								$rel_id => $line['invoice_id'],
								'amount' => $amount,
								'amount_usdollar' => round($amount / $exrt, 5),
								'date_modified' => gmdate('Y-m-d H:i:s'),
								// just for reference
								'currency_id' => $cid,
								'exchange_rate' => $exrt,
							];
							$ret[$key][] = $newline;
						}
					}
				}
			}
		}

		return $ret;
	}
    
    static function load_line_items(RowUpdate $upd) {
		$direction = $upd->getField('direction');
		if(! empty($_REQUEST['update_links'])) {
			$new_lines = self::get_input_line_items($_REQUEST, $direction);
			$upd->setRelatedData('allocations', $new_lines);
		}
    }

    static function save_line_items(RowUpdate $upd) {
		$id = $upd->getPrimaryKeyValue();
		$qid = $upd->new_record ? null : $id;
		$new_lines = $upd->getRelatedData('allocations');
		if(isset($new_lines)) {
			$lines = self::query_line_items($qid);
			self::update_line_items($upd, $lines, $new_lines);
		}
	}
}
?>
