AppBase.module('payments').moduleRequires('ui-controls').init(function(App) {
	var UI = App.ui,
		Wrap = App.dom.elementOrId,
		QueryId = App.dom.queryId,
		MakeElt = App.dom.makeElement;

	var uniqid = 1;
	var mod_string = function(str, mod) { return window.mod_string(str, mod || 'Payments'); }

	UI.PaymentItemsEditor = function(id, params) {
		this.id = 'PaymentItemsEditor';
		this.editable = true;
		this.direction = null;
		this.lines = {};
		this.lines_order = [];
		this.orig_balance = {};
		this.inputs = {};
		this.line_type = 'Invoice';
		this.credit = false;
		this.account = {};
		this.totals = {
			total: 0,
			balance: 0
		};
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		if(params) extendObject(this, params);
		if(! this.account || ! this.account.id)
			this.account = {};
		if(this.init_lines) {
			for(var i = 0; i < this.init_lines.length; i++) {
				var line = this.init_lines[i];
				this.lines_order.push(line.id);
				this.lines[line.id] = line;
				var bal = line.invoice_amount_due,
					amt = line.amount;
				bal = isset(bal) ? parseFloat(bal) : null;
				if(isset(amt)) {
					line.amount = amt = parseFloat(amt);
					bal += amt;
				}
				this.orig_balance[line.invoice_id] = bal;
			}
		}
	};
	
	UI.PaymentItemsEditor.prototype = {

		setup: function() {
			if(this.set_up) return;
			if(! this.elt)
				this.elt = $(this.id + '-main');
			if(! this.elt)
				return;

			var editor = this;
			this.currency_input = UI.getFormInput(this.form, 'currency');
			this.applied_currency_input = UI.getFormInput(this.form, 'applied_currency');
			this.account_input = UI.getFormInput(this.form, 'account');
			this.direction_input = UI.getFormInput(this.form, 'direction');
			this.type_input = UI.getFormInput(this.form, 'payment_type');
			this.amount_input = UI.getFormInput(this.form, 'amount');
			this.applied_amount_input = UI.getFormInput(this.form, 'applied_amount');
			this.effective_rate_input = UI.getFormInput(this.form, 'effective_rate');
			this.add_line_input = new UI.ButtonInput(this.id + '-add-line', {form: this.form});
			this.add_line_input.setup();
			this.total_input = new UI.TextInput(this.id + '-total', {form: this.form, ignore_dirty: true});
			this.total_input.setup();
			this.totals_outer = Wrap(this.id + '-totals');
			this.refunded_input = UI.getFormInput(this.form, 'refunded');
			
			UI.attachInputEvent(this.currency_input, 'onchange', function() { editor.currencyChanged(); });
			UI.attachInputEvent(this.direction_input, 'onchange', function() { editor.directionChanged(this); });
			UI.attachInputEvent(this.amount_input, 'onchange', function() { editor.paymentAmountChanged(); });
			UI.attachInputEvent(this.applied_amount_input, 'onchange', function() { editor.appliedAmountChanged(); });
			UI.attachInputEvent(this.add_line_input, 'onclick', function() { editor.addLine(); return false; });
			
			UI.onInitForm(this.form, function() {
				editor.direction = editor.getDirection();
				editor.fixLineType();
				editor.addAccountExtra();
				editor.initBlank();
				//editor.render();
				editor.currencyChanged(true);
			});
			this.set_up = true;
		},

		initBlank: function() {
			if((! this.credit || this.direction === 'credit' || this.direction === 'refund')
			  && ! this.lines_order.length && this.editable)
			this.addLine(null, true, true);
		},
		
		makeEditable: function() {
			if(! this.editable) {
				this.editable = true;
				this.render();
				Wrap(this.elt).closest('.form-section').findFirst('.payment-edit-btn').hide();
				Wrap(this.elt).closest('.form-section').findFirst('.payment-items-buttons').css('visibility', 'visible');
			}
		},
		
		checkHidden: function() {
			var visib = true;
			if(this.credit) {
				if(this.direction === 'outgoing')
					visib = false;
			}
			else {
				if(this.direction === 'refund')
					visib = false;
			}
			Wrap(this.elt).closest('.form-section').toggleClass('hidden', !visib);
			return !visib;
		},

		render: function() {
			this.disableFields();
			if(! this.checkHidden()) {
				this.renderLines();
				this.calculate();
			}
		},
		
		renderLines: function() {
			var editor = this;
			var tbl = Wrap(this.elt).findFirst('.payment-items-body').ext(0);
			if(! tbl) return;
			tbl.empty();

			var drawLine = function(idx, line) {
				var inputs = this.inputs[line.id];
				if(! inputs) this.inputs[line.id] = inputs = {};
				var row = MakeElt('div.row.col-space-sm.payment-row-data').appendTo(tbl);
				
				var cell = MakeElt('div.column.payment-col-name').appendTo(row);
				if(this.editable) {
					if(! inputs.name) {
						var rel = new UI.RefInput(this.id + '-name-' + line.id, {
							bean_name: this.line_type,
							module: this.getLineModule(),
							ignore_dirty: true, block: true
						});
						rel.addExtraReturnFields({amount: 'amount', amount_due: 'amount_due',
							billing_account: 'billing_account', supplier: 'supplier',
							currency: 'currency', exchange_rate: 'exchange_rate'});
						rel.onchange = function(k, v, passthru) { editor.returnLineItem(k, v, line.id); }
						UI.registerInput(this.form, rel);
						inputs.name = rel;
					}
					cell.append(inputs.name.render());
					this.setLineItemFilter(inputs.name);
					inputs.name.update(line.invoice_id, line.invoice_display, true);
				} else {
					MakeElt('div.tools-row.input-field-pad.align-top',
						MakeElt('div.no-flex',
							MakeElt('div.input-icon.theme-icon.module-' + this.getLineModule() + '.left')),
						UI.makeRecordLink(this.getLineModule(), line.invoice_id, line.invoice_name)
					).appendTo(cell);
				}
		
				cell = MakeElt('div.column.payment-col-prior.text-right').appendTo(row);
				if(this.editable) {
					if(! inputs.prior) {
						var bal = new UI.TextInput(this.id + '-prior-' + line.id, {
							format: 'currency',
							currency_input: this.applied_currency_input,
							size: 12,
							readOnly: true,
							align: 'right',
							ignore_dirty: true
						});
						UI.registerInput(this.form, bal);
						inputs.prior = bal;
					}
					cell.append(inputs.prior.render());
					inputs.prior.setValue(this.getLineBalance(line, true), true);
				} else {
					var span = MakeElt('span.text-number.input-field-pad').appendTo(cell),
						val = this.getLineBalance(line, true);
					if(val) span.append(this.formatCurrency(val));
					else span.html('&mdash;');
				}

				cell = MakeElt('div.column.payment-col-balance.text-right').appendTo(row);
				if(this.editable) {
					if(! inputs.balance) {
						var bal = new UI.TextInput(this.id + '-balance-' + line.id, {
							format: 'currency',
							currency_input: this.applied_currency_input,
							size: 12,
							readOnly: true,
							align: 'right',
							ignore_dirty: true
						});
						UI.registerInput(this.form, bal);
						inputs.balance = bal;
					}
					cell.append(inputs.balance.render());
					inputs.balance.setValue(this.getLineBalance(line), true);
				} else {
					var span = MakeElt('span.text-number.input-field-pad').appendTo(cell),
						val = this.getLineBalance(line);
					if(val) span.append(this.formatCurrency(val));
					else span.html('&mdash;');
				}
		
				var cell = MakeElt('div.column.payment-col-amount.text-right').appendTo(row);
				if(this.editable) {
					if(! inputs.amount) {
						var amt = new UI.TextInput(this.id + '-amount-' + line.id, {
							format: 'currency',
							currency_input: this.applied_currency_input,
							min: 0,
							size: 12,
							align: 'right',
							ignore_dirty: true
						});
						amt.onchange = function() { editor.updateLineAmount(line.id, this.getValue(true)); }
						UI.registerInput(this.form, amt);
						inputs.amount = amt;
					}
					cell.append(inputs.amount.render());
					inputs.amount.max = line.invoice_amount;
					inputs.amount.setValue(isset(line.amount) ? parseFloat(line.amount) : null, true);
					inputs.amount.setDisabled(! line.invoice_id);
				} else {
					var span = MakeElt('span.text-number.input-field-pad').appendTo(cell),
						val = isset(line.amount) ? parseFloat(line.amount) : null;
					if(val) span.append(this.formatCurrency(val));
					else span.html('&mdash;');
				}

				if(this.editable) {
					cell = MakeElt('div.column.payment-col-end.text-right').appendTo(row);
					var link = MakeElt('span.uii.uii-lg.active-icon.uii-remove');
					//lbl = app_string('LBL_REMOVE');
					link.on('click', function() {
						this.removeLine(line.id);
						return false;
					}, {scope: this});
					cell.append(link);
				}
			};
			
			var count = this.iterLines(drawLine);
			UI.setDisplayState(this.id + '-head', !! count);
			if(! count) {
				var row = MakeElt('div.row.payment-row-blank').appendTo(tbl);
				var cell = MakeElt('div.column.payment-col-blank.input-field-pad.text-help').appendTo(row);
				cell.append(mod_string(this.line_type === 'CreditNote' ? 'LBL_NO_CREDITS' : 'LBL_NO_LINES'));
			}
		},
		
		iterLines: function(callback) {
			var ord = this.lines_order;
			for(var idx = 0; idx < ord.length; idx++) {
				callback.call(this, idx, this.lines[ord[idx]]);
			}
			return idx;
		},

		addLine: function(info, nofocus, auto_performed) {
			var editor = this;
			var id = 'newline~' + (uniqid ++);
			this.lines_order.push(id);
			this.lines[id] = {id: id};
			this.renderLines();
			this.calculate();
			if(! nofocus)
				setTimeout(function() { editor.focusLine(id); }, 100);
			if(! auto_performed) UI.markFormDirty(this);
		},
		
		focusLine: function(id) {
			var inputs = this.inputs[id];
			if(inputs && inputs.name) inputs.name.focus();
		},
		
		removeLine: function(id, noupdate) {
			var found;
			for(var idx = 0; idx < this.lines_order.length; idx++) {
				if(this.lines_order[idx] == id) {
					found = idx;
					break;
				}
			}
			if(! isset(found))
				return;
			this.lines_order.splice(found, 1);
			delete this.lines[id];
			if(! noupdate) {
				this.renderLines();
				this.calculate();
			}
			UI.markFormDirty(this);
		},
		
		clearLines: function(norender) {
			this.lines_order = [];
			this.lines = {};
			this.inputs = {};
			if(! norender) {
				this.renderLines();
				this.calculate();
			}
			UI.markFormDirty(this);
		},
		
		clearLinesApplied: function(norender) {
			var direction = this.direction;
			this.iterLines(function(idx, line) {
				line.amount = direction === 'refund' ? line.invoice_balance : 0;
			});
			if(! norender) {
				this.renderLines();
				this.calculate();
			}
			UI.markFormDirty(this);
		},
		
		updateLineAmount: function(id, value, norender) {
			if(id in this.lines) {
				this.lines[id].amount = value;
				if(! norender) {
					this.renderLines();
					this.calculate();
				}
				UI.markFormDirty(this);
			}
		},
		
		getDirection: function() {
			if(this.direction_input) return this.direction_input.getValue();
		},
		
		getLineModule: function() {
			if(this.line_type === 'CreditNote')
				return 'CreditNotes';
			else if(this.line_type === 'Bill')
				return 'Bills';
			return 'Invoice';
		},
		
		getLineBalance: function(line, prior) {
			var ret = line.invoice_amount;
			ret = isset(ret) ? parseFloat(ret) : null;
			var id = line.invoice_id;
			if(! id) return;
			if(isset(this.orig_balance[id]))
				ret = this.orig_balance[id];
			if(! prior) {
				var applied = line.amount;
				applied = isset(applied) ? parseFloat(applied) : 0;
				ret -= applied;
			}
			return ret;
		},
		
		getCurrencyId: function() {
			return this.currency_input.getKey() || '-99';
		},

		getExchangeRate: function(currency_id) {
			return this.currency_input.getRate(true, currency_id);
		},
		
		currencyDecimalPlaces: function(currency_id) {
			return this.currency_input.getDecimals(currency_id);
		},

		formatCurrency: function(val, noRound, symbol) {
			if(! isset(val) || val === '') return '';
			var decimals = this.currencyDecimalPlaces(), ret;
			if(noRound)
				ret = stdFormatNumber(restrictDecimals(parseFloat(val), decimals));
			else
				ret = stdFormatNumber(parseFloat(val * 100).toFixed(decimals) / 100, decimals, decimals);
			if(symbol) {
				var symbol = this.currency_input.getSymbol();
				ret = symbol + ' ' + ret;
			}
			return ret;
		},
		
		setLineItemFilter: function(input) {
			var param = this.credit || (! this.credit && this.direction != 'outgoing')
				? 'billing_account' : 'supplier';
			input.add_filters = [{
				param: param,
				field_name: 'account'
			}];
			if(this.credit) {
				input.add_filters.push({param: 'apply_credit_note', value: 0});
				input.add_filters.push({param: 'cancelled', value: 0});
			}
		},

		returnLineItem: function(key, value, line_id) {
			if(! line_id || ! this.lines[line_id]) return;
			var line = this.lines[line_id],
				update_acct,
				line_upd;
			
			if(value) {
				var found = false, removeMsg, count = 0;
				var acct_id = value.billing_account_id || value.supplier_id;
				this.iterLines(function(idx, line) {
					if(line.invoice_id && line.id !== line_id && line.invoice_id === value.id) {
						found = true;
					}
					if(line.invoice_id || line.id === line_id) {
						count ++;
					}
				});
				if(found) {
					removeMsg = mod_string(this.credit ? 'MSG_CREDIT_ALREADY_REFERENCED' : 'MSG_ALREADY_REFERENCED');
				} else {
					var cid = this.checkAppliedCurrency(false, line_id);
					if(cid && cid !== value.currency_id)
						removeMsg = mod_string('MSG_CURRENCY_MISMATCH');
					else {
						if(! acct_id)
							removeMsg = mod_string('MSG_NO_ACCOUNT');
						else {
							var sel_acct = this.account_input ? this.account_input.getKey() : this.account.id;
							if(! sel_acct)
								update_acct = acct_id;
							else if(sel_acct != acct_id)
								removeMsg = mod_string('MSG_ACCOUNT_MISMATCH');
						}
					}
				}
				if(removeMsg) {
					this.inputs[line_id].name.clear();
					requestAnimationFrame(function() {
						alert(removeMsg);
					});
					return;
				}
				line_upd = {
					invoice_display: value._display,
					invoice_id: value.id,
					invoice_amount: parseFloat(value.amount_due),
					invoice_currency_id: value.currency_id,
					invoice_exchange_rate: value.exchange_rate,
					amount: parseFloat(value.amount_due)
				};
			} else {
				line_upd = {
					invoice_display: '',
					invoice_id: null,
					invoice_amount: null,
					invoice_currency_id: null,
					invoice_exchange_rate: null,
					amount: null
				};
			}
			extendObject(line, line_upd);
			
			this.renderLines();
			this.calculate();
			
			if(update_acct)
				this.account_input.updateWithLookup('Accounts', update_acct, false, true, {updating_line: true});

			UI.markFormDirty(this);
		},
		
		updateEffectiveRate: function() {
			if((this.credit && this.direction !== 'refund') || ! this.effective_rate_input) return;
			var payAmt = this.getPaymentAmount(),
				appAmt = this.getAppliedAmount(),
				rate = 1.0;
			if(appAmt && payAmt != null)
				rate = payAmt / appAmt;
			this.effective_rate_input.ignore_dirty = true;
			this.effective_rate_input.setValue(rate, true);
		},

		updateAmount: function(applied) {
			if(this.effective_rate_input && this.amount_input && isset(applied)) {
				var rate = this.effective_rate_input.getValue(true);
				if(rate) {
					var amount = applied / rate;
					this.amount_input.setValue(amount, true);
				}
			}
		},

        updateFirstLine: function() {
            if (! this.credit || (this.credit && this.direction === 'refund')) {
                var ord = this.lines_order;

                //Update if only one payment line exists
                if (ord.length == 1 && this.editable)
                    this.updateLineAmount(ord[0], this.amount_input.getValue(true));
            }
        },

		calculate: function() {
			var total = 0,
				nonempty = 0,
				count = this.iterLines(function(idx, line) {
					if(isset(line.amount) && line.amount) {
						total += parseFloat(line.amount) || 0;
						nonempty ++;
					}
				});
			this.totals.total = total;
			this.totals_outer.toggle(count > 0);
			this.total_input.setValue(this.applied_currency_input.formatAmount(total));

			if(this.editable) {
				if ((this.direction !== 'refund' && !this.credit) || (this.direction === 'refund' && this.credit)) {
					this.applied_amount_input.ignore_dirty = true;
					this.applied_amount_input.setValue(nonempty ? total : null, true);
					if(this.direction === 'incoming') {
						this.updateEffectiveRate();
					}
					else if(this.direction === 'credit') {
						this.updateAmount(0);
					}
					else {
						this.updateAmount(total);
					}
				}
			}
		},

		setAccountExt: function(currency_id, balance, balance_payable, updating_line) {
			if(! currency_id) return;
			var checkCid = this.checkAppliedCurrency();
			this.account.id = this.account_input.getKey();
			this.account.currency_id = currency_id;
			this.account.balance = balance;
			this.account.balance_payable = balance_payable;
			if(checkCid && updating_line)
				; // don't update currency - leave same as applied currency
			else
				this.currency_input.setValue(currency_id, true);
			this.currencyChanged();
		},

		currencyChanged: function(initial) {
			if(! this.credit) {
				var cid = this.currency_input.getKey();
				var appCid = this.applied_currency_input.getKey();
				var setCid = this.checkAppliedCurrency();
				if(! setCid) setCid = cid;
				if(setCid) {
					if(setCid != appCid) {
						this.applied_currency_input.setValue(setCid, true);
						appCid = setCid;
					}
				}
			}
			this.paymentAmountChanged();
		},
		
		checkAppliedCurrency: function(selfonly, skipline) {
			var cid = null;
			this.iterLines(function(idx, line) {
				if(skipline && line.id == skipline)
					return;
				if(line.invoice_id && line.invoice_currency_id)
					cid = line.invoice_currency_id;
			});
			if(! cid && ! selfonly) {
				var other = UI.getFormInput('DetailForm', this.credit ? 'payment_items' : 'credit_items');
				if(other) cid = other.checkAppliedCurrency(true);
			}
			return cid;
		},
		
		getPaymentAmount: function() {
			return this.amount_input.getValue() === '' ? null : this.amount_input.getValue(true);
		},
		
		getAppliedAmount: function() {
			return this.applied_amount_input.getValue() === '' ? null : this.applied_amount_input.getValue(true);
		},
		
		paymentAmountChanged: function() {
            this.updateFirstLine();
            this.updateEffectiveRate();
			this.render();
		},

		appliedAmountChanged: function() {
			this.updateEffectiveRate();
			this.render();
		},

		accountChanged: function(id, value, passthru) {
			if(isset(value))
				this.setAccountExt(value['currency_id'], value['balance'], value['balance_payable'],
					passthru && passthru.updating_line);
		},
		
		fixLineType: function() {
			if(! this.credit) {
				var add_label;
				if(this.direction === 'outgoing') {
					this.line_type = 'Bill';
					add_label = 'LBL_ADD_BILL_BUTTON_LABEL';
				} else {
					this.line_type = 'Invoice';
					add_label = 'LBL_ADD_LINE_BUTTON_LABEL';
				}
				if(this.add_line_input && add_label) {
					Wrap(this.add_line_input.elt).find('label').text(mod_string(add_label));
				}
			}
		},

		directionChanged: function(input) {
			var olddir = this.direction, newdir = input.getValue();
			this.direction = newdir;
			this.fixLineType();
			if(newdir !== olddir) {
				if((newdir == 'credit' || newdir == 'refund') && (olddir == 'credit' || olddir == 'refund'))
					this.clearLinesApplied(true);
				else
					this.clearLines(true);
				this.initBlank();
				this.render();
				if(! this.credit) {
					if(this.type_input) {
						if(newdir == "credit") {
							this.type_input.setValue("Credit Note", true);
						}
						else if(this.type_input.getValue() == "Credit Note") {
							this.type_input.clear(true, true);
						}
					}
					if((newdir == "outgoing" || olddir == "outgoing") && this.account_input) {
						this.account_input.clear();
					}
				}
			}
			this.addAccountExtra();
		},
		
		getTotalAmount: function() {
			return this.totals.total;
		},
		
		validate: function() {
            var self = this;
			this.invalid = false;
			this.calculate();
			if(! self.invalid)
			var count = this.iterLines(function(idx, line) {
				if(isset(line.amount) && line.amount > line.invoice_amount) {
					self.invalid = true;
					self.invalidMsg = 'invalid';
				}
			});

			if(this.credit && ! count && (this.direction === 'credit' || this.direction === 'refund')) {
				this.invalid = true;
				this.invalidMsg = 'required';
			}
			if(! this.credit && ! count && this.direction !== 'refund') {
				this.invalid = true;
				this.invalidMsg = 'required';
			}
			
			return ! this.invalid;
		},

		checkAllocationsTotal: function() {
			if (!this.credit) return;
			var other = UI.getFormInput('DetailForm', 'payment_items');
			var credits = 0, allocations = 0;
			this.iterLines(function(idx, line) {
				if(isset(line.amount)) {
					credits += line.amount;
				}
			});
			other.iterLines(function(idx, line) {
				if(isset(line.amount)) {
					allocations += line.amount;
				}
			});
			if(this.direction === 'credit') {
				if (allocations != credits) {
					return mod_string('MSG_CANT_PROCESS');
				}
			}
			else if(this.direction === 'incoming') {
				if(allocations < credits) {
					return mod_string('MSG_INVALID_ALLOCATIONS');
				}
			}
			else if(this.direction === 'refund') {

			}
		},

		beforeSubmitForm: function() {
			if(! this.editable) return;
			if(! this.name) console.error("missing input name");
			var allocErr = this.checkAllocationsTotal();
			if(allocErr) {
				alert(allocErr);
				return false;
			}
			var doUpdate = UI.HiddenInput.findOrInsert(this.form, 'update_links');
			doUpdate.setValue('1');
			var saveField = UI.HiddenInput.findOrInsert(this.form, this.name, this.id + '-save');
			var lines = [];
			this.iterLines(function(idx, line) {
				if(line.invoice_id && line.amount)
					lines.push(line);
			});
			saveField.setValue(JSON.stringify(lines));
		},

		disableFields: function() {
			if(this.credit) return;
			this.amount_input.setReadOnly(! this.editable);
			this.currency_input.setReadOnly(! this.editable);
			if(this.account_input)
				this.account_input.setReadOnly(! this.editable);
			this.applied_currency_input.setReadOnly();
			this.effective_rate_input.setReadOnly();
			this.applied_amount_input.setReadOnly();
 			if(this.type_input) {
 				var keys = this.type_input.menu.getKeys();
 				hide_idx = keys.indexOf("Credit Note");
 				if(hide_idx >= 0) {
 					this.type_input.menu.hidden_indexes = [hide_idx];
 				}
 			}
 			var refund_outer = this.refunded_input && Wrap(this.refunded_input.elt).closest('.form-entry');
 			if(refund_outer) {
				refund_outer.toggleClass('cond-hidden', this.direction != 'incoming');
			}
		},

		addAccountExtra: function() {
			if(! this.account_input || this.credit) return;
			if(! this.inited_acct) {
				this.account_input.addExtraReturnFields({'currency_id': 'currency_id', 'balance': 'balance', 'balance_payable': 'balance_payable'});
				this.inited_acct = true;
				var editor = this;
				UI.attachInputEvent(this.account_input, 'onchange', function(key, values, passthru) { 
					editor.accountChanged(key, values, passthru);
				});
			}
			var filter = [];
			if(this.direction == 'outgoing')
				filter[0] = {param: 'is_supplier', value: 1};
			else
				filter[0] = {param: 'is_supplier', value: 0};
			this.account_input.add_filters = filter;
		}
	};
	// end PaymentItemsEditor
	
	
	UI.PaymentFeesEditor = function(id, params) {
		this.id = 'PaymentFeesEditor';
		this.editable = true;
		this.lines = {};
		this.lines_order = [];
		this.inputs = {};
		this.totals = {
			total: 0
		};
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		if(params) extendObject(this, params);
		if(this.init_lines) {
			for(var i = 0; i < this.init_lines.length; i++) {
				var line = this.init_lines[i];
				this.lines_order.push(line.id);
				this.lines[line.id] = line;
			}
		}
	};
	
	UI.PaymentFeesEditor.prototype = {

		setup: function() {
			if(this.set_up) return;
			if(! this.elt)
				this.elt = $(this.id + '-main');
			if(! this.elt)
				return;

			var editor = this;
			this.currency_input = UI.getFormInput(this.form, 'currency');
			this.direction_input = UI.getFormInput(this.form, 'direction');
			this.add_line_input = new UI.ButtonInput(this.id + '-add-line', {form: this.form});
			this.add_line_input.setup();
			
			UI.attachInputEvent(this.add_line_input, 'onclick', function() { editor.addLine(); return false; });
			UI.attachInputEvent(this.direction_input, 'onchange', function() { editor.directionChanged(this); });

			UI.onInitForm(this.form, function() {
				editor.render();
			});
			this.set_up = true;
		},
		
		checkHidden: function() {
			var visib = true;
			var direction = this.direction_input && this.direction_input.getValue();
			if(direction == "credit") {
				visib = false;
			}
			Wrap(this.elt).closest('.form-section').toggleClass('hidden', !visib);
			return !visib;
		},
		
		render: function() {
			if(! this.checkHidden()) {
				this.renderLines();
				this.calculate();
			}
		},
		
		renderLines: function() {
			var editor = this;
			var tbl = Wrap(this.elt).findFirst('.payment-items-body').ext(0);
			if(! tbl) return;
			tbl.empty();

			var drawLine = function(idx, line) {
				var inputs = this.inputs[line.id];
				if(! inputs) this.inputs[line.id] = inputs = {};
				var row = MakeElt('div.row.col-space-sm.payment-row-data').appendTo(tbl);
				
				var cell = MakeElt('div.column.payment-col-fee-type').appendTo(row);
				if(! inputs.name) {
					var typ = new UI.SelectInput(this.id + '-type-' + line.id, {
						options: {dom_name: 'payment_fee_type_dom', add_blank: true},
						ignore_dirty: true,
						block: true
					});
					typ.onchange = function(k, v) {
						line.fee_type = k;
						if(! isset(line.amount)) {
							setTimeout(function() { inputs.amount.focus(); }, 100);
						}
					}
					UI.registerInput(this.form, typ);
					inputs.type = typ;
				}
				cell.append(inputs.type.render());
				inputs.type.setValue(line.fee_type || '', true);

				var cell = MakeElt('div.column.payment-col-fee-amount.text-right').appendTo(row);
				if(! inputs.amount) {
					var amt = new UI.TextInput(this.id + '-amount-' + line.id, {
						format: 'currency',
						currency_input: inputs.currency,
						min: 0,
						size: 12,
						align: 'right',
						ignore_dirty: true
					});
					amt.onchange = function() { editor.updateLineAmount(line.id, this.getValue(true)); }
					UI.registerInput(this.form, amt);
					inputs.amount = amt;
				}
				cell.append(inputs.amount.render());
				inputs.amount.setValue(isset(line.amount) ? parseFloat(line.amount) : null, true);

				cell = MakeElt('div.column.payment-col-fee-currency').appendTo(row);
				if(! inputs.currency) {
					var curr = new UI.CurrencySelect(this.id + '-currency-' + line.id, {
						ignore_dirty: true,
						block: true
					});
					UI.registerInput(this.form, curr);
					inputs.currency = curr;
				}
				cell.append(inputs.currency.render());
				inputs.currency.setValue(line.currency_id, true);
				
				cell = MakeElt('div.column.payment-col-fee-notes').appendTo(row);
				if(! inputs.notes) {
					var nots = new UI.TextInput(this.id + '-notes-' + line.id, {
						size: 30,
						ignore_dirty: true
					});
					nots.onchange = function() { line.notes = this.getValue(); }
					UI.registerInput(this.form, nots);
					inputs.notes = nots;
				}
				cell.append(inputs.notes.render());
				inputs.notes.setValue(line.notes || '', true);

				if(this.editable) {
					cell = MakeElt('div.column.payment-col-end').appendTo(row);
					var link = MakeElt('span.uii.uii-lg.active-icon.uii-remove');
					//lbl = app_string('LBL_REMOVE');
					link.on('click', function() {
						this.removeLine(line.id);
						return false;
					}, {scope: this});
					cell.append(link);
				}
			};
			
			var count = this.iterLines(drawLine);
			UI.setDisplayState(this.id + '-head', !! count);
			if(! count) {
				var row = MakeElt('div.row.payment-row-blank').appendTo(tbl);
				var cell = MakeElt('div.column.payment-col-blank.input-field-pad.text-help').appendTo(row);
				cell.append(mod_string('LBL_NO_FEES'));
			}
		},
		
		iterLines: function(callback) {
			var ord = this.lines_order;
			for(var idx = 0; idx < ord.length; idx++) {
				callback.call(this, idx, this.lines[ord[idx]]);
			}
			return idx;
		},

		addLine: function(info, nofocus) {
			var editor = this;
			var id = 'newline~' + (uniqid ++);
			this.lines_order.push(id);
			this.lines[id] = {id: id, currency_id: this.currency_input.getKey()};
			this.renderLines();
			this.calculate();
			if(! nofocus)
				setTimeout(function() { editor.focusLine(id); }, 100);
			UI.markFormDirty(this);
		},
		
		focusLine: function(id) {
			var inputs = this.inputs[id];
			if(inputs && inputs.type) inputs.type.focus();
		},
		
		removeLine: function(id, noupdate) {
			var found;
			for(var idx = 0; idx < this.lines_order.length; idx++) {
				if(this.lines_order[idx] == id) {
					found = idx;
					break;
				}
			}
			if(! isset(found))
				return;
			this.lines_order.splice(found, 1);
			delete this.lines[id];
			if(! noupdate) {
				this.renderLines();
				this.calculate();
			}
			UI.markFormDirty(this);
		},
		
		clearLines: function(norender) {
			this.lines_order = [];
			this.lines = {};
			this.inputs = {};
			if(! norender) {
				this.renderLines();
				this.calculate();
			}
			UI.markFormDirty(this);
		},
		
		updateLineAmount: function(id, value, norender) {
			if(id in this.lines) {
				this.lines[id].amount = value;
				if(! norender) {
					this.renderLines();
					this.calculate();
				}
				UI.markFormDirty(this);
			}
		},

		directionChanged: function(input) {
			var olddir = this.direction, newdir = input.getValue();
			this.direction = newdir;
			if(newdir !== olddir) {
				this.render();
			}
		},
		
		calculate: function() {
		},

		validate: function() {
			return true;
		},

		beforeSubmitForm: function() {
			if(! this.editable) return;
			if(! this.name) console.error("missing input name");
			var saveField = UI.HiddenInput.findOrInsert(this.form, this.name, this.id + '-save');
			var lines = [];
			this.iterLines(function(idx, line) {
				if(line.fee_type && line.amount)
					lines.push(line);
			});
			saveField.setValue(JSON.stringify(lines));
		}
	};
	// end PaymentFeesEditor

});

