<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormTableSection.php');
require_once('modules/Payments/Payment.php');

class PaymentFeesWidget extends FormTableSection {

	function init($params=null, $model=null) {
		parent::init($params, $model);
		if(! $this->id)
			$this->id = 'payment_fees';
	}
	
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if($gen->getLayout()->getEditingLayout()) {
			$params = array();
			return $this->renderOuterTable($gen, $parents, $context, $params);		
		}

		$lstyle = $gen->getLayout()->getType();
		if($lstyle == 'editview' || ($lstyle == 'record' && $gen->getLayout()->getDefaultEditable())) {
			return $this->renderHtmlEdit($gen, $row_result, $parents, $context);
		}
		return $this->renderHtmlView($gen, $row_result, $parents, $context);
	}
	
	function getRequiredFields() {
		return array('id');
	}
	
	function renderHtmlView(HtmlFormGenerator $gen, RowResult $row_result, $parents, $context) {
		global $mod_strings;

		$lines = $this->fetchLines($row_result->getPrimaryKeyValue(), false, 'view');
		if($lines) {
			$body = '';
			foreach($lines as $line) {
				$body .= '<div class="row form-row">' .
					'<div class="column form-cell">' .
					'<div class="form-entry">' .
					'<div class="form-value">';
				$body .= $line->getFieldValue('fee_type', null, true) . translate('LBL_SEPARATOR', 'app');
				$body .= $line->getFieldValue('amount', null, true);
				$body .= '</div></div></div>';
				$desc = $line->getFieldValue('notes', null, true);
				if($desc) {
					$body .= '<div class="column form-cell">' .
						'<div class="form-entry">' .
						'<div class="form-value">' .
						'<div class="input-icon theme-icon module-Notes"></div> ' . $desc .
						'</div></div></div>';
				}
				$body .= '</div>';
			}
			if($body) {
				$params = compact('body');
				return $this->renderOuterTable($gen, $context, $parents, $params);
			}
		}
	}
	
	function renderHtmlEdit(HtmlFormGenerator $gen, RowResult $row_result, $parents, $context) {
		global $mod_strings;

		$editable = true;

		$body = <<<EOQ
			<div id="{$this->id}-main" class="payment-fees-main">
EOQ;

		$fee_type = translate('LBL_FEE_TYPE', 'Payments');
		$currency = translate('LBL_FEE_CURRENCY', 'Payments');
		$amount = translate('LBL_FEE_AMOUNT', 'Payments');
		$notes = translate('LBL_NOTES', 'Payments');
		$empty_cell = '';
		$panel_edit = '';
		$add_line_label = translate('LBL_ADD_FEE_BUTTON_LABEL', 'Payments');
		$total_label = translate('LBL_TOTAL_ALLOCATED', 'Payments');
		$balance_label = translate('LBL_ACCOUNT_BALANCE', 'Payments');

		$body .= <<<EOQ
			<div class="row form-row">
			<div class="column form-cell expand"><div class="form-entry">
			<section class="card-outer panel-outer panel-default payment-items-outer panel-border panel-round {$panel_edit}">
				<div class="card-header panel-subheader" id="{$this->id}-head" style="display: none">
					<div class="row col-space-sm panel-column-titles">
						<div class="column payment-row-head payment-col-fee-type"><span class="column-title">{$fee_type}</span></div>
						<div class="column payment-row-head payment-col-fee-amount text-right"><span class="column-title">{$amount}</span></div>
						<div class="column payment-row-head payment-col-fee-currency"><span class="column-title">{$currency}</span></div>
						<div class="column payment-row-head payment-col-fee-notes"><span class="column-title">{$notes}</span></div>
						<div class="column payment-row-head payment-col-end">&nbsp;</div>
					</div>
				</div>
				<div class="card-body panel-body panel-pad-body payment-items-body">
					<em class="text-muted">None</em>
				</div>
				<div class="card-footer panel-footer payment-items-bottom">
					<div class="row col-unstack">
						<div class="column col-8 payment-items-buttons" id="{$this->id}-buttons">
							<div class="buttons-inline">
								<div class="inline-elt" id="{$this->id}-add-line-outer">
									<div class="uii uii-add uii-md active-icon payment-button" id="{$this->id}-add-line">
										<label>{$add_line_label}</label>
									</div>
								</div>
							</div>
						</div>
					</div>
				</div>
			</section>
			</div></div>
			</div>
EOQ;

		$layout = $gen->getLayout();

		$layout->addScriptInclude('modules/Payments/payments.js', LOAD_PRIORITY_FOOT);

		$editParams = $this->getEditorParams($row_result);
		$gen->getFormObject()->addControl('PaymentFeesEditor', $this->id, $editParams);

		$params['body'] = $body . '</div>';
		return $this->renderOuterTable($gen, $context, $parents, $params);
	}
	
	function fetchLines($payment_id, $rows=false, $formatted=null) {
		$items = [];
		if($payment_id) {
			$llq = new ListQuery('Payment', true, ['link_name' => 'fees', 'parent_key' => $payment_id]);
			if($formatted) {
				$llq->setFormatted('html', $formatted);
			}
			if($rows) {
				$items = $llq->fetchAllRows();
			} else {
				foreach($llq->fetchAll() as $row) {
					$items[$row->getField('id')] = $row;
				}
			}
		}
		return $items;
	}

	function getEditorParams(RowResult $row_result) {
		$editable = true;
		$payment_id = $row_result->getPrimaryKeyValue();

		$items = $row_result->getRelatedData('payment_fees');
		if(! $items && ! $row_result->new_record) {
			$items = $this->fetchLines($payment_id, true);
			$row_result->getRelatedData('payment_fees', $items);
		}
		
		$params = [
			'editable' => $editable,
			'name' => $this->name,
			'label' => $this->getLabel(),
			'init_lines' => $items ? array_values($items) : null,
		];
		
		return $params;
	}
		
	/*function renderPdf(PdfFormGenerator $gen, $row_result, array $parents, array $context)
	{
		$gen->pdf->set_focus($row_result);
		$gen->pdf->print_main();
	}*/
	
	function loadUpdateRequest(RowUpdate $update, array $input) {
        $related_data = $update->getRelatedData('payment_fees');
        if(! isset($related_data)) {
        	$json = getJSONobj();
        	$related_data = array_get_default($input, $this->id);
        	$related_data = $related_data ? $json->decode($related_data) : [];
        	$update->setRelatedData('payment_fees', $related_data);
        }
	}
	
	function validateInput(RowUpdate $update) {
        return true;
    }
	
	function afterUpdate(RowUpdate $update) {
		global $log;
		$row_updates = $update->getRelatedData('payment_fees');
		if(! is_array($row_updates))
			return;
		
		$payment_id = $update->getPrimaryKeyValue();
		$old_items = $this->fetchLines($payment_id, false);
		$delete = $old_items;

		$pos = 1;
    	foreach($row_updates as $line) {
    		if(! is_array($line)) continue;
    		$id = array_get_default($line, 'id');
    		if($id && isset($old_items[$id])) {
    			$upd = $old_items[$id]->getRowUpdate();
    			unset($delete[$id]);
    		} else {
				$upd = RowUpdate::blank_for_model('payment_fees');
			}
    		unset($line['id']);
    		$upd->set($line);
    		$upd->set('payment_id', $payment_id);
    		$upd->set('position', $pos ++);
    		if(! $upd->save()) {
    			$log->error("Error saving payment fee: ".$upd->getErrors(true));
    		}
		}
			
		foreach($delete as $id => $row) {
			$row->getRowUpdate()->markDeleted();
		}
	}
	
}
?>
