<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormTableSection.php');
require_once('modules/Payments/Payment.php');
require_once('modules/Currencies/CurrencyUtils.php');

class PaymentItemsWidget extends FormTableSection {
	var $credit = false;
	var $line_type = 'Invoice';

	function init($params=null, $model=null) {
		parent::init($params, $model);
		if(! $this->id)
			$this->id = 'payment_items';
		if(is_array($params) && isset($params['credit']))
			$this->credit = $params['credit'];
		if(is_array($params) && isset($params['line_type']))
			$this->line_type = $params['line_type'];
		else if($this->credit)
			$this->line_type = 'CreditNote';
	}
	
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if($gen->getLayout()->getEditingLayout()) {
			$params = array();
			return $this->renderOuterTable($gen, $parents, $context, $params);		
		}

		$lstyle = $gen->getLayout()->getType();
		if($lstyle == 'editview' || ($lstyle == 'record' && $gen->getLayout()->getDefaultEditable())) {
			return $this->renderHtmlEdit($gen, $row_result, $parents, $context);
		}
		return '';
	}
	
	function getRequiredFields() {
		return array('direction', 'refund');
	}
	
	function renderHtmlEdit(HtmlFormGenerator $gen, RowResult $row_result, $parents, $context) {
		global $mod_strings;

		$payment_id = '';
		$prefix = '';
		$editable = true;

		if (! $row_result->new_record) {
			$payment_id = $row_result->getField('payment_id');
			$prefix = $row_result->getField('prefix');
		} else {
			$source = $this->loadSource($_REQUEST);
		}

		$allocation_currency = CurrencyUtils::get_currency_name($row_result->getField('currency_id'), true);
		$total_allocated = $row_result->getField('total_amount');
		$credit_allocated = 0;

		$body = <<<EOQ
			<input name="prefix" value="{$prefix}" type="hidden" />
			<input name="payment_id" value="{$payment_id}" type="hidden" />
EOQ;
		if(! $this->credit && ! $row_result->new_record) {
			$direction = $row_result->getField('direction');
			$dir_q = to_html($direction);
			$body .= <<<EOQ
			<input name="direction" id="direction-input" value="{$dir_q}" type="hidden" />
EOQ;
			$gen->getFormObject()->addControl('HiddenInput', 'direction-input', ['name' => 'direction']);
		}

		$body .= <<<EOQ
			<div id="{$this->id}-main" class="payment-items-main">
EOQ;

		$editParams = $this->getEditorParams($row_result);

		$related = translate('LBL_RELATED_RECORD', 'Payments');
		$prior = translate('LBL_BALANCE_PRIOR', 'Payments');
		$amount = translate($this->credit ? 'LBL_APPLY_CREDIT' : 'LBL_APPLY_PAYMENT', 'Payments');
		$balance = translate('LBL_BALANCE', 'Payments');
		$empty_cell = '';
		$panel_edit = '';
		$add_line_label = translate($this->credit ? 'LBL_ADD_CREDIT_BUTTON_LABEL' : 'LBL_ADD_LINE_BUTTON_LABEL', 'Payments');
		$total_label = translate('LBL_TOTAL_ALLOCATED', 'Payments');
		$balance_label = translate('LBL_ACCOUNT_BALANCE', 'Payments');
		$buttons_style = $editParams['editable'] ? '' : 'visibility: hidden';

		$body .= <<<EOQ
			<div class="row form-row">
			<div class="column form-cell expand"><div class="form-entry">
			<section class="card-outer panel-outer panel-default payment-items-outer panel-border panel-round {$panel_edit}">
				<div class="card-header panel-subheader" id="{$this->id}-head" style="display: none">
					<div class="row col-space-sm panel-column-titles">
						<div class="column payment-row-head payment-col-name"><span class="column-title">{$related}</span></div>
						<div class="column payment-row-head payment-col-prior text-right"><span class="column-title">{$prior}</span></div>
						<div class="column payment-row-head payment-col-balance text-right"><span class="column-title">{$balance}</span></div>
						<div class="column payment-row-head payment-col-amount text-right"><span class="column-title">{$amount}</span></div>
						<div class="column payment-row-head payment-col-end text-right">&nbsp;</div>
					</div>
				</div>
				<div class="card-body panel-body panel-pad-body payment-items-body">
					<em class="text-muted">No credits applied</em>
				</div>
				<div class="card-footer panel-footer payment-items-bottom">
					<div class="row col-unstack">
						<div class="column col-8 payment-items-buttons" id="{$this->id}-buttons" style="{$buttons_style}">
							<div class="buttons-inline">
								<div class="inline-elt" id="{$this->id}-add-line-outer">
									<div class="uii uii-add uii-md active-icon payment-button" id="{$this->id}-add-line">
										<label>{$add_line_label}</label>
									</div>
								</div>
							</div>
						</div>
						<div class="column align-right payment-items-totals" id="{$this->id}-totals" style="display: none">
							<div class="row col-space-sm" id="{$this->id}-total-outer">
								<div class="column">
									<div class="form-label input-field-pad">{$total_label}</div>
								</div>
								<div class="column text-right">
									<div class="form-value"><input type="text" class="input-text text-right" size="14" id="{$this->id}-total" readonly></div>
								</div>
							</div>
							<div class="row col-space-sm" style="display: none" id="{$this->id}-balance-outer">
								<div class="column">
									<div class="form-label input-field-pad">{$balance_label}</div>
								</div>
								<div class="column text-right">
									<div class="form-value"><input type="text" class="input-text text-right" size="14" readonly></div>
								</div>
							</div>
						</div>
					</div>
				</div>
			</section>
			</div></div>
			</div>
EOQ;

		$layout = $gen->getLayout();
		$layout->addScriptInclude('modules/Payments/payments.js', LOAD_PRIORITY_FOOT);

		$gen->getFormObject()->addControl('PaymentItemsEditor', $this->id, $editParams);

		if(! $row_result->new_record && $editable) {
			$btn_label = translate('LBL_EDIT_BUTTON_LABEL');
			$title_extra = ' <button type="button" class="input-button payment-edit-btn" '
				. 'onclick="SUGAR.ui.getFormInput(\'' . $gen->getFormObject()->getFormName() . "', '$this->name').makeEditable();\""
				. 'style="margin-left: 2em">'
				. '<div class="input-icon icon-edit left"></div>'
				. '<span class="input-label" style="text-transform:none; letter-spacing:normal">'
				. to_html($btn_label) . '</span></button>';
		} else {
			$title_extra = '';
		}

		$params = [
			'title_extra' => $title_extra,
			'body' => $body . '</div>',
		];
		return $this->renderOuterTable($gen, $context, $parents, $params);
	}

	/**
	 * Load payment source
	 *
	 * @param array $input - user input ($_REQUEST) 
	 * @return null|RowResult
	 */
	function loadSource($input) {
		$source = null;
		$source_name = null;
		$source_id = null;

		if($this->credit) {
			if (! empty($input['creditnote_id'])) {
				$source_name = 'CreditNote';
				$source_id = $input['creditnote_id'];
			}
		} else {
			if(! empty ($input['invoice_id'])) {
				$source_name = 'Invoice';
				$source_id = $input['invoice_id'];
			} else if (! empty($input['bill_id'])) {
				$source_name = 'Bill';
				$source_id = $input['bill_id'];
			}
		}

		if ($source_name && $source_id) {
			$fields = ['id', '_display', 'currency_id', 'exchange_rate',
				 'account', 'amount_due'];
			if($source_name == 'Bill') {
				$fields['account_id'] = 'supplier_id';
				$fields['account'] = 'supplier';
			} else {
				$fields['account_id'] = 'billing_account_id';
				$fields['account'] = 'billing_account';
			}
			$source = ListQuery::quick_fetch($source_name, $source_id, $fields);
		}

		return $source;
	}

	function getEditorParams(RowResult $row_result) {
		$json = getJSONObj();

		$editable = !! $row_result->new_record;
		$payment_id = $row_result->getPrimaryKeyValue();
		$direction = $row_result->getField('direction');
		$module = 'Payments'; // ($direction != 'outgoing') ? 'Payments' : 'PaymentsOut';
		$account_id = $row_result->getField('account_id');

		$items = $row_result->getRelatedData('lines');
		$lines = [];
		if(! $items) {
			if($row_result->new_record) {
				$src = $this->loadSource($_REQUEST);
				if($src) {
					$line = [
						'invoice_id' => $src->getField('id'),
						'invoice_display' => $src->getField('_display'),
						'invoice_currency_id' => $src->getField('currency_id'),
						'invoice_exchange_rate' => $src->getField('exchange_rate'),
						'invoice_balance' => $src->getField('amount_due'),
						'amount' => $src->getField('amount_due'),
					];
					$lines[] = $line;
					if(! $account_id)
						$account_id = $src->getField('account_id');
				}
			} else {
				$items = Payment::query_line_items($payment_id, true);
				$row_result->setRelatedData('lines', $items);
			}
		}
		if($items) {
			if($this->credit) {
				$key = 'credits';
			} else if($direction == 'outgoing') {
				$key = 'bills';
			} else {
				$key = 'invoices';
			}
			foreach($items[$key] as $line) {
				$line['id'] = $line['invoice_id'];
				$lines[] = $line;
			}
		}
		
		$account = $row_result->getRelatedData('account');
		if(! $account && $account_id) {
			$fields = ['id', 'name', 'is_supplier', 'balance', 'balance_payable'];
			$account = ListQuery::quick_fetch_row('Account', $account_id, $fields);
			$row_result->setRelatedData('account', $account);
		}

		$params = [
			'editable' => $editable,
			'name' => $this->name,
			'label' => $this->getLabel(),
			'payment_module' => $module,
			'account' => $account,
			'init_lines' => $lines,
			'credit' => $this->credit,
			'line_type' => $this->line_type,
		];
		
		return $params;
	}
		
	function renderPdf(PdfFormGenerator $gen, $row_result, array $parents, array $context)
	{
		$gen->pdf->set_focus($row_result);
		$gen->pdf->print_main();
	}
}
?>
