<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

/* Include all other system or application files that you need to reference here.*/
/*Include this file if you want to access sugar specific settings*/

require_once('data/SugarBean.php'); /*Include this file since we are extending SugarBean*/

require_once('modules/Threads/Thread.php'); // for pulling parent forum

class Post extends SugarBean {
	/* Foreach instance of the bean you will need to access the fields in the table.
	 * So define a variable for each one of them, the varaible name should be same as the field name
	 * Use this module's vardef file as a reference to create these variables.
	 */
	var $name;
	var $id;
	var $date_entered;
	var $created_by;
	var $date_modified;
	var $modified_user_id;
	var $deleted;
	var $title;
	var $description_html;
	var $thread_id;

	// non-db
	var $created_by_user;
	var $modified_by_user;
	var $thread_name;

	/* End field definitions*/

	/* variable $table_name is used by SugarBean and methods in this file to constructs queries
	 * set this variables value to the table associated with this bean.
	 */
	var $table_name = 'posts';

	/*This  variable overrides the object_name variable in SugarBean, wher it has a value of null.*/
	var $object_name = 'Post';

	/**/
	var $module_dir = 'Posts';

	/* This is a legacy variable, set its value to true for new modules*/
	var $new_schema = true;

	/* $column_fields holds a list of columns that exist in this bean's table. This list is referenced
	 * when fetching or saving data for the bean. As you modify a table you need to keep this up to date.
	 */
	var $column_fields = Array(
		'id',
		'title',
		'description_html',
		'thread_id',
	);

	/* This is the list of required fields, It is used by some of the utils methods to build the required fields validation JavaScript */
	/* The script is only generated for the Edit View*/
	var $required_fields = array('title' => 1);


	static function notify_reply(RowUpdate $upd) {
		require_once('include/Notify/RecordActionMessage.php');
		$message = RecordActionMessage::for_action('Post', 'thread_reply', $upd->getUserId());
		if($message) {
			$message->addRecordId($upd->getPrimaryKeyValue());
			$mailer = $message->getMailer();
			foreach($message->getNotifiedUsers() as $targ)
				$mailer->addTargetInfo($targ);
			$mailer->sendMessage($message);
		}
	}

	public static function counters_inc_created(RowUpdate $upd) {
		$GLOBALS['db']->query(
				"UPDATE threads
				SET date_modified = NOW()
				WHERE id = '" . $GLOBALS['db']->quote($upd->saved['thread_id']) . "'"
			);
		if ($upd->new_record) {
			$GLOBALS['db']->query(
				"UPDATE threads
				SET postcount = postcount + 1, date_modified = NOW()
				WHERE id = '" . $GLOBALS['db']->quote($upd->saved['thread_id']) . "'"
			);

			$bean_name = AppConfig::module_primary_bean('Threads');
			$row = ListQuery::quick_fetch($bean_name, $upd->saved['thread_id'], array('forum_id'));

			$GLOBALS['db']->query(
				"UPDATE forums
				SET threadandpostcount = threadandpostcount + 1
				WHERE id = '" . $GLOBALS['db']->quote($upd->getField('forum_id')) . "'"
			);
		}
	}

	public static function counters_dec_deleted(RowUpdate $upd) {
		$GLOBALS['db']->query(
			"UPDATE threads
				SET postcount = postcount - 1
				WHERE id = '" . $GLOBALS['db']->quote($upd->orig_row['thread_id']) . "'"
		);

		$bean_name = AppConfig::module_primary_bean('Threads');
		$row = ListQuery::quick_fetch($bean_name, $upd->orig_row['thread_id'], array('forum_id'));

		$GLOBALS['db']->query(
			"UPDATE forums
				SET threadandpostcount = threadandpostcount - 1
				WHERE id = '" . $GLOBALS['db']->quote($row->getField('forum_id')) . "'"
		);
	}

	public static function display_edit(DetailManager $mgr) {
		$hiddens = array(
			'thread_id' => $_REQUEST['thread_id'],
		);
		$mgr->getLayout()->addFormHiddenFields($hiddens);
		if (empty($_REQUEST['record'])) {
			$reply = array_get_default($_REQUEST, 'reply');
			$quote = array_get_default($_REQUEST, 'quote');
			if (!$reply && !$quote) return;
			$original_id = $reply ? $reply : $quote;
			$row = ListQuery::quick_fetch_row('Post', $original_id);
			$description_html = '';
			$title = "Re: ".$row['title'];
			if ($quote) {
				$description_html = "<b><i>" . translate('LBL_TEXT_USER', 'Posts') ." '". get_assigned_user_name($row['created_by'])."' " . translate('LBL_TEXT_SAID', 'Posts') . ":</i></b>
			<br />
			<table width=\"85%\" cellspacing=\"1\" cellpadding=\"1\" border=\"1\" align=\"center\" summary=\"\" style=\"font-style: italic;\">
			  <tbody>
				<tr>
				  <td>".$row['description_html']."</td>
				</tr>
			  </tbody>
			</table>
			<br /><br />
			";
			}
			$mgr->record->assign('title', $title);
			$mgr->record->assign('description_html', $description_html);
		}
	}

	public static function get_post_detail_view($fields) {
		extract($fields);
		global $current_user, $timedate;
		static $photos = array(), $pspec, $fmt;
		require_once 'XTemplate/xtpl.php';
		$xtpl = new XTemplate(dirname(__FILE__) . '/DetailView.html');

		if(! isset($pspec)) {
			$pspec = AppConfig::setting('model.fields.User.photo');
			$pspec['vname_module'] = 'Users';
			require_once('include/layout/FieldFormatter.php');
			$fmt = new FieldFormatter('html', 'view');
		}
		$base = array('id' => $created_by, 'photo' => '', 'photo_filename' => array_get_default($fields, 'created_by_user.photo_filename'),
			'photo_thumb' => array_get_default($fields, 'created_by_user.photo_thumb'));
		$fields['image'] = $fmt->formatRowValue($pspec, $base, '');

		foreach($fields as $k => $v) {
			if($k == 'date_modified' || $k == 'date_entered') {
				$fields[$k] = $timedate->to_display_date_time($v);
			}
		}

		$xtpl->assign('MOD', return_module_language('', 'Posts'));
		$xtpl->assign('APP', return_application_language(''));
		$xtpl->assign('POST', $fields);

		if($current_user->is_admin || $current_user->id == $created_by) {
			$xtpl->parse('main.owner_or_admin');
		}
		if($date_modified != $date_entered || $modified_user_id != $created_by) {
			$xtpl->parse("main.modified");
		}
		$xtpl->parse('main');
		return $xtpl->text('main');
	}
}

?>
