<?php

require_once 'include/layout/forms/FormField.php';
require_once 'modules/PriceBooks/PriceBookFormulaParser.php';

class ProductPriceWidget extends FormField {
	function getReversed() {
		if($this->model && $this->model->getModelName() == 'Product')
			return true;
	}
	
	function getRequiredFields() {
		$f = parent::getRequiredFields();
		$reversed = $this->getReversed();
		$pn = $reversed ? '' : '~parent.';
		$rn = $reversed ? '~parent.' : '';
		$f[] = $rn.'formula';
		$f[] = $rn.'editable';
		$f[] = $rn.'round_nearest';
		$f[] = $rn.'conv_currency';
		$f[] = $pn.'list_price';
		$f[] = $pn.'cost';
		$f[] = $pn.'purchase_price';
		return $f;
	}
	
	function renderListValue(ListFormatter $fmt, ListResult $result, $row_id, $list_params=null) {
		$reversed = $this->getReversed();
		$pn = $reversed ? '' : '~parent.';
		$rn = $reversed ? '~parent.' : '';
		
		$formula = $result->rows[$row_id]['~join.custom_formula'];
		if (!strlen(trim($formula)) || empty($result->rows[$row_id][$rn.'editable'])) {
			$formula = $result->rows[$row_id][$rn.'formula'];
		}
		$round = $result->rows[$row_id][$rn.'round_nearest'];
		$vars = array(
			 'list_price'=> 0,
			 'cost' => 0,
			 'purchase_price' => 0,
		 );
		foreach ($vars as $k => &$v) {
			$v = array_get_default($result->rows[$row_id], $pn . $k, 0);
		}

		$operations = PriceBookFormulaParser::parse($formula);
		$price = PriceBookFormulaParser::execute($operations, $vars, $round);

		$cid = $result->rows[$row_id][$pn.'currency_id'];
		$price = currency_format_number($price, array('currency_id' => $cid));
		return $price ;
	}
}

