<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


require_once('data/SugarBean.php');
require_once('modules/Accounts/Account.php');
require_once('include/utils.php');
require_once('include/utils/misc_functions.php');

class Product extends SugarBean {

	// Stored fields
  	var $id;
	var $name;
	var $purchase_name;
	var $url;
	var $is_available;
	var $date_available;
	var $tax_code_id;
	var $supplier_id;
	var $manufacturer_id;
    var $weight_1;
	var $weight_2;
	var $manufacturers_part_no;
	var $vendor_part_no;
	var $product_category_id;
	var $product_type_id;
	var $model_id;
	var $currency_id;
	var $exchange_rate;
	var $cost;
	var $cost_usdollar;
	var $list_price;
	var $list_usdollar;
	var $purchase_price;
	var $purchase_usdollar;
	var $pricing_formula;
	var $description;
	var $description_long;
	var $ppf_perc;
	var $track_inventory;
	var $date_entered;
	var $created_by;
	var $date_modified;
	var $modified_user_id;

	var $support_cost;
	var $support_cost_usdollar;
	var $support_list_price;
	var $support_list_usdollar;
	var $support_selling_price;
	var $support_selling_usdollar;
	var $support_price_formula;
	var $support_ppf_perc;

	var $supplier_name;
	var $manufacturer_name;
	var $currency_name;
	var $currency_symbol;
	var $currency_iso;
	var $product_category_name;
	var $product_type_name;
	var $model_name;
	var $assembly_id;
	var $created_by_name;
	var $modified_user_name;

	var $eshop;
	var $image_url;
	var $thumbnail_url;

	var $all_stock;

	var $table_name = 'products';
	var $object_name = 'Product';
	var $module_dir = 'ProductCatalog';
	var $new_schema = true;

	var $in_stock; // virtual

	static $stock_prefetch = array();


	// This is used to retrieve related fields from form posts.
	var $additional_column_fields = Array(
		"supplier_name",
		"manufacturer_name",
		"model_name",
		"currency_name",
		"currency_symbol",
		"product_category_name",
		"product_type_name",
		'tax_code_short',
		'tax_code_name',
		'assembly_id',
		'location_id',
		'created_by_name',
		'modified_user_name',
	);

	var $relationship_fields = array(
		'assembly_id' => 'assemblies',
		'supplier_id' => 'suppliers',
	);

	function calc_stock($warehouse_id, $pids=null)
    {
        if(! $warehouse_id)
            return;
        if(! isset(self::$stock_prefetch[$warehouse_id]))
            self::$stock_prefetch[$warehouse_id] = array();
        $cache =& self::$stock_prefetch[$warehouse_id];

        if(isset($pids)) {
            $fetch = array();
            foreach($pids as $id) {
                if(! isset($cache[$id])) {
                    $fetch[] = $id;
                    $cache[$id] = 0;
                }
            }
            if(! count($fetch))
                return;
            $pc = "product_id IN ('".implode("','", $fetch)."')";
        } else {
            if(isset($cache[$this->id])) {
                $this->in_stock = $cache[$this->id];
                return;
            }
            $pc = "product_id = '" . PearDatabase::quote($this->id) . "'";
            $this->in_stock = 0;
        }
        $query = "SELECT product_id,in_stock FROM products_warehouses WHERE warehouse_id='" . PearDatabase::quote($warehouse_id) . "' AND $pc AND deleted = 0 LIMIT 1";
        $res = $this->db->query($query, true);
        while($row = $this->db->fetchByAssoc($res)) {
            $cache[$row['product_id']] = (float)$row['in_stock'];
            if(! isset($pids)) {
                $this->in_stock = (float)$row['in_stock'];
                break;
            }
        }
    }

	function stock_for_warehouse($id = null)
	{
		static $set = null;
		if ($id) {
			$set = $id;
		}
		return $set;
	}


	function fill_in_tax_code_info() {
		require_once('modules/TaxCodes/TaxCode.php');
		$taxcode = new TaxCode();
		$taxcode->retrieve(empty($this->tax_code_id) ? '-99' : $this->tax_code_id);
		$this->tax_code_name = $taxcode->name;
		$this->tax_code_short = $taxcode->code;
		$taxcode->cleanup();
	}

	function get_text_description($description=false) {
		if($description === false)
			$description = $this->description;
		if(strpos($description, '&lt;p&gt;') !== false) {
			require_once('include/utils/html_utils.php');
			$description = trim(html2plaintext(fix_fck_html(from_html($description)), false));
		}
		return $description;
	}


	/// Called when this object is created or modified.
	///
	function save($check_notify = FALSE) {
		// this must be done before interpreting numeric values
		$this->unformat_all_fields();

		// these cannot be null
		if(! isset($this->cost) || number_empty($this->cost))
			$this->cost = 0.0;
		if(! isset($this->list_price) || number_empty($this->list_price))
			$this->list_price = 0.0;
		if(! isset($this->purchase_price) || number_empty($this->purchase_price))
			$this->purchase_price = 0.0;

		$ret = parent::save($check_notify);
		return $ret;
	}

	static function update_assembly_prices(RowUpdate $upd) {
		$lq = $upd->getLinkQuery('assemblies');
		$result = $lq->fetchAll();
		foreach($result->getRowIndexes() as $idx) {
			$subupd = $result->getRowUpdate($idx);
			$subupd->save();
		}
	}

	static function update_attribute_prices(RowUpdate $upd) {
		$lq = $upd->getLinkQuery('productattributes');
		$result = $lq->fetchAll();
		foreach($result->getRowIndexes() as $idx) {
			$subupd = $result->getRowUpdate($idx);
			$subupd->save();
		}
	}

	function get_search_categories($param, &$searchFields)
	{
		require_once 'modules/ProductCategories/ProductCategory.php';
		return array('' => '') + get_product_categories_list(true);
	}

	function get_search_types($param, &$searchFields)
	{
		require_once 'modules/ProductTypes/ProductType.php';
		$pType = new ProductType;
		$category = array_get_default($searchFields['product_category_id'], 'value');
		$arPT = $pType->get_for_category($category, true);
		$pType->cleanup();
		return array('' => '') + $arPT;
	}

	function get_search_categories2(ListFilter &$filter)
	{
		require_once 'modules/ProductCategories/ProductCategory.php';
		return array('' => '') + get_product_categories_list(true);
	}

	function inventory_auto_adjusted($track_inv=-1) {
		if($track_inv == -1)
			$track_inv = $this->track_inventory;
		return (! empty($track_inv) && $track_inv != 'untracked' && $track_inv != 'manual');
	}

	function inventory_tracked($track_inv=-1) {
		if($track_inv == -1)
			$track_inv = $this->track_inventory;
		return (! empty($track_inv) && $track_inv != 'untracked');
	}

	static function get_products_pending_invoice($case_id) {
		global $db;
		$query = sprintf(
			"SELECT products_cases.product_id as id FROM products_cases ".
			"LEFT JOIN cases ON cases.id=products_cases.case_id AND NOT cases.deleted ".
			"WHERE cases.id='%s' AND NOT cases.deleted ".
				"AND NOT products_cases.deleted ", //AND IFNULL(products_cases.invoice_id,'')='' ",
			$db->quote($case_id));
		if(! empty($query)) {
			$seed = new Product();
			// results are not to be HTML encoded
			$result = $seed->build_related_list($query, $seed, 0, -1, false);
			$seed->cleanup();
		}
		else {
			$result = array();
		}
		return $result;
 	}

	function create_default_stock()
	{
		$existing = array();
		$query = sprintf(
			"SELECT warehouse_id FROM products_warehouses WHERE deleted = 0 AND product_id = '%s' ",
			$this->db->quote($this->id)
		);
		$res = $this->db->query($query, true);
		while ($row = $this->db->fetchByAssoc($res)) {
			$existing[$row['warehouse_id']] = true;
		}

		$this->load_relationship('warehouses');

		$query = "SELECT id FROM company_addresses WHERE is_warehouse AND deleted = 0";
		$res = $this->db->query($query, true);
		while ($row = $this->db->fetchByAssoc($res)) {
			if (empty($existing[$row['id']])) {
				$this->warehouses->add($row['id']);
			}
		}

	}

	function popup_form_handler($params, &$form) {
		global $current_user;
		if($current_user->getPreference('product_costs'))
			$form->parse('main.cost_column');
		else
			$form->assign('HIDE_COST', 'style="display: none"');
	}

	static function get_stock_qty_by_id($id) {
		global $db;
		$query = "SELECT SUM(IFNULL(pw.in_stock, 0)) as stock_qty FROM products_warehouses pw LEFT JOIN company_addresses w ON w.id=pw.warehouse_id WHERE pw.product_id = '" . $db->quote($id) . "' AND pw.deleted = 0 AND w.deleted = 0";
		$r = $db->query($query, true);
		$rows = $db->fetchRows($r);
		return 0 + $rows[0]['stock_qty'];
	}

    static function update_stock_qty(RowUpdate &$upd, $link_name) {
        global $db;
        $id = $upd->getPrimaryKeyValue();
        if($upd->getModelName() == 'Product' && $link_name == 'warehouses' && $id) {
			$upd->set('all_stock', self::get_stock_qty_by_id($id));
			$upd->save();
		}
    }


    static function init_record(RowUpdate &$upd, $input) {
        $update = array();

        if (! empty($input['model_id'])) {
            $model = ListQuery::quick_fetch_row('Model', $input['model_id']);

            if ($model != null) {
                $update['model_id'] = $input['model_id'];
                $manuf = ListQuery::quick_fetch_row('Account', $model['manufacturer_id']);
                if ($manuf != null) {
                    $update['manufacturer_id'] = $manuf['id'];
                    $update['manufacturer_name'] = $manuf['name'];
                }
            }
        }

		if (empty($upd->duplicate_of_id))
        	$update['track_inventory'] = 'semiauto';

        $upd->set($update);
    }

	static function update_image_url($sname, $dname, $img_dir, RowUpdate $upd) {
		$fname = $upd->getField($sname);
		if ($fname) {
			require_once('include/upload_file.php');
			$value = AppConfig::site_url() . UploadFile::get_path($fname, '', $img_dir);
			$upd->set($dname, $value);
		}
	}

	static function update_image_urls(RowUpdate &$upd) {
		$spec = $upd->getFieldDefinition('image');
		$img_dir = array_get_default($spec, 'upload_dir', '');
		self::update_image_url('image_filename', 'image_url', $img_dir, $upd);
		self::update_image_url('image_thumb', 'thumbnail_url', $img_dir, $upd);
	}

    static function after_save(RowUpdate &$upd) {
    	$supp = $upd->getField('supplier_id');
    	$o_supp = $upd->getField('supplier_id', null, true);
    	if($supp && $supp != $o_supp) {
    		$upd->addUpdateLink('suppliers', $supp);
    	}
    }

    static function get_request_pricebook_id() {
		if (!empty($_REQUEST['additional_data']['pricebook_id']))
			return $_REQUEST['additional_data']['pricebook_id'];
		else if (!empty($_REQUEST['filters']['pricebook_id']))
			return $_REQUEST['filters']['pricebook_id'];
		else if (!empty($_REQUEST['pricebook_id']))
			return $_REQUEST['pricebook_id'];
    }

    static function list_pricebook_price($spec) {
		$pricebook_id = self::get_request_pricebook_id();
		$vars = array(
			'cost' => $spec['raw_values']['cost'],
			'list_price' => $spec['raw_values']['list_price'],
			'purchase_price' => $spec['raw_values']['purchase_price'],
		);
		if(isset($GLOBALS['pb_discount_vars'])) {
			array_extend($vars, $GLOBALS['pb_discount_vars']);
		}
		$product_id = $spec['raw_values']['id'];
		$price = self::calc_pricebook_price($pricebook_id, $product_id, $vars);
		if ($price) {
			if(substr($spec['name'], 0, 4) == 'raw_')
				return $price[1];
			return $price[0];
		}
    }

    static function product_pricebook_info($pricebook_id, $product_id) {
		static $cache = array();
		static $all_cache = array();

		if (i_e_f() && $pricebook_id && $product_id) {
			$pb = $row = null;

			if (isset($all_cache[$pricebook_id]))
				$pb = $all_cache[$pricebook_id];
			else {
				$pb = AppConfig::db_row('PriceBook', $pricebook_id);
				if(! $pb) $pb = false;
				$all_cache[$pricebook_id] = $pb;
			}

			if($pb) {
				if($pb['apply_to_all'] && ! $pb['editable'])
					return $pb;
				else {
					$key = $product_id . '_' . $pricebook_id;
					if (! isset($cache[$key])) {
						$lq = new ListQuery('Product', true, array('link_name' => 'pricebooks', 'parent_key' => $product_id));
						$lq->addField('~parent.exchange_rate');
						$lq->addField('~join.custom_formula');
						$result = $lq->queryRecord($pricebook_id);
						if(! $result || $result->failed) {
							if($pb['apply_to_all'])
								$cache[$key] = $pb;
							else
								$cache[$key] = false;
						} else
							$cache[$key] = $result->row;
					}
					return $cache[$key];
				}
			}
		}
    }


	static function calc_pricebook_price($pricebook_id, $product_id, array $vars) {
		$row = self::product_pricebook_info($pricebook_id, $product_id);
		if ($row) {
			$formula = array_get_default($row, '~join.custom_formula', '');
			if (!strlen(trim($formula)))
				$formula = $row['formula'];
			require_once 'modules/PriceBooks/PriceBookFormulaParser.php';
			try {
				$price = PriceBook_eval($formula, $vars, $row['round_nearest']);
			} catch (Exception $e) {
				$GLOBALS['log']->error("Error in formula: ".$e->getMessage());
				$price = 0;
			}

			$cRate = array_get_default($row, '~parent.exchange_rate', array_get_default($spec['raw_values'], 'exchange_rate'));
			if (!$cRate)
				$cRate = 1;
			$priceUSD = $price / $cRate;
			return array($priceUSD, $price, $row['conv_currency_id']);
		}
	}

	static function pricebook_part_no($product_id, $part_no) {
		$pricebook_id = self::get_request_pricebook_id();
		$row = self::product_pricebook_info($pricebook_id, $product_id);
		if ($row)
			$part_no = trim($row['part_prefix'] . $part_no . $row['part_suffix']);
		return $part_no;
	}

	static function pricebook_currency_id($product_id) {
		$pricebook_id = self::get_request_pricebook_id();
		$row = self::product_pricebook_info($pricebook_id, $product_id);
		return isset($row['conv_currency_id']) ? $row['conv_currency_id'] : "";
	}

}

