/*****************************************************************************
 * The contents of this file are subject to The Long Reach Corporation
 * Software License Version 1.0 ("License"); You may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * <http://www.thelongreach.com/swlicense.html>
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) The Long Reach Corporation copyright notice,
 * (ii) the "Powered by SugarCRM" logo, and
 * (iii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is:
 *    Info At Hand Add-on Module to SugarCRM Open Source project.
 * The Initial Developer of this Original Code is The Long Reach Corporation
 * and it is Copyright (C) 2004-2007 by The Long Reach Corporation;
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2005 SugarCRM, Inc.;
 * All Rights Reserved.
 *****************************************************************************
 * $Id$
 * File Description:
 * Contributor(s):
*****************************************************************************/


AppBase.module('financials', 'ui-controls').init(function(App) {
	var UI = App.ui,
		Dom = App.dom,
		Wrap = Dom.elementOrId,
		MakeElt = Dom.makeElement;

	UI.FinancialsEditor = function(id, params) {
		this.id = 'FinancialsEditor';
		this.data = {};
		this.periods = [];
		this.updates = {};
		this.json_fields = [
			{name: 'expected_cost', cls: 'expcost', readonly: true},
			{name: 'expected_revenue', cls: 'exprev'},
			{name: 'actual_cost', cls: 'actcost'},
			{name: 'actual_revenue', cls: 'actrev'}
		];
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		if(params) extendObject(this, params);
		if(Array.isArray(this.data)) this.data = {};
		if(typeof(this.start_date) === 'string')
			this.start_date = parseDateString(this.start_date).d;
		if(typeof(this.end_date) === 'string')
			this.end_date = parseDateString(this.end_date).d;
	}
	UI.FinancialsEditor.prototype = {
		setup: function() {
			if(this.set_up) return;
			if(! this.elt)
				this.elt = $(this.id + '-main');
			if(! this.elt)
				return;

			var self = this;
			var cb = function() { self.initDates(true); }
			UI.attachFormInputEvent(this.form, 'date_starting', 'onchange', cb);
			UI.attachFormInputEvent(this.form, 'date_ending', 'onchange', cb);
			this.initDates();
			
			this.updateUseTimesheets();
			UI.attachFormInputEvent(this.form, 'use_timesheets', 'onchange', function() {
				self.updateUseTimesheets(true); });

			UI.attachFormInputEvent(this.form, 'currency', 'onrateupdate', function(oldrate, newrate) {
				self.updateFinancialsRate(oldrate, newrate, true); });
						
			this.set_up = true;
			this.updateDisplay();
		},
		decimalPlaces: function() {
			var currency = UI.getFormInput(this.form, 'currency');
			if(currency) return currency.getDecimals();
			return this.decimal_places || 0;
		},
		initDates: function(display) {
			var inp_start = UI.getFormInput(this.form, 'date_starting'),
				inp_end = UI.getFormInput(this.form, 'date_ending');
			if(inp_start)
				this.start_date = inp_start.getValue(true);
			if(inp_end)
				this.end_date = inp_end.getValue(true);
			this.updatePeriods();
			if(display) this.updateDisplay();
		},
		dateUpdated(name, val) {
			if(name == 'date_starting')
				this.start_date = val;
			else if(name == 'date_ending')
				this.end_date = val;
			this.updateDisplay();
		},
		updatePeriods: function() {
			function mkPeriod(year, month) {
				if(month < 10)
					return year + '-0' + month;
				return year + '-' + month;
			}
			var periodList = [];
			if(this.start_date && this.end_date) {
				var year = this.start_date.getFullYear();
				var month = this.start_date.getMonth() + 1;
				while(year < this.end_date.getFullYear()) {
					while(month <= 12) {
						periodList.push(mkPeriod(year, month));
						month++;
					}
					month = 1;
					year++;
				}
				while(month <= this.end_date.getMonth() + 1) {
					periodList.push(mkPeriod(year, month));
					month++;
				}
			}
			this.periods = periodList;
		},
		getRenderPeriods: function() {
			var plist = this.periods,
				rows = [];
			for(var i = 0; i < plist.length; i++)
				rows.push([plist[i], 'in']);
			for(var period in this.data) {
				if(plist.indexOf(period) < 0)
					rows.push([period, 'out']);
			}
			rows.sort();
			return rows;
		},
		updateDisplay: function() {
			var body = Wrap(this.elt).findFirst('.financials-body').ext(0);
			if(! this.data || ! body)
				return;
			var rowCount = 1,
				periods = this.getRenderPeriods(),
				current_periods = {},
				period, style, row, cell, field, value, dval, textInput,
				decimals = this.decimalPlaces(),
				self = this;
			body.empty();

			for (var i = 0; i < periods.length; i ++) {
				period = periods[i][0];
				current_periods[period] = 1;
				style = periods[i][1];
				row = MakeElt('div.row.col-space-sm.financials-row-data').appendTo(body);
				cell = MakeElt('div.column.financials-col-month.text-number').appendTo(row);
				if(this.editable)
					cell.addClass('input-field-pad');
				if(style === 'out')
					cell.addClass('text-error');
				cell.text(period);
				
				for(var j = 0; j < this.json_fields.length; j++) {
					field = this.json_fields[j];
					cell = MakeElt('div.column.text-right.financials-col-' + field.cls).appendTo(row);
					value = this.data[period] ? this.data[period][field.name] : null;
					if(this.editable) {
						textInput = new UI.TextInput(this.id + '-' + period + '-' + field.cls,
							{format: 'currency', fin_period: period, fin_field: field.name,
							 readOnly: !! field.readonly, init_value: value || 0,
							 decimals: decimals, align: 'right'});
						textInput.onchange = function() {
							self.updateData(this.fin_period, this.fin_field, this.getValue(true));
						}
						cell.append(textInput.render());
						textInput.setup();
					} else {
						if(! value)
							cell.html('&mdash;');
						else {
							dval = stdFormatNumber(parseFloat(value).toFixed(decimals), decimals, decimals);
							cell.text(dval);
						}
					}
				}
				cell = MakeElt('div.column.financials-col-end').appendTo(row);
				if(this.editable) {
					MakeElt('span.uii.uii-lg.uii-remove.active-icon')
						.prop({period: period})
						.on('click', function(evt) { this.removeRow(evt.currentTarget.period, true); evt.stopEvent(); }, {scope: this})
						.appendTo(cell);
				}
			}
			this.clearData(current_periods);
		},
		removeRow: function(period, display) {
			if(! this.data || ! this.data[period])
				return;
			for(var field in this.data[period])
				this.updateData(period, field, null);
			if(this.periods.indexOf(period) < 0)
				delete this.data[period];
			if(display)
				this.updateDisplay();
		},
		updateData: function(period, field, value) {
			if(! this.data[period]) {
				if(! value)
					return;
				this.data[period] = {};
			}
			if(! this.updates[period])
				this.updates[period] = {};
			this.data[period][field] = value;
			this.updates[period][field] = value;
		},
		updateFinancialsRate: function(old_rate, new_rate, display) {
			if(! old_rate) return;
			for(var period in this.data) {
				for(var field in this.data[period]) {
					value = this.data[period][field];
					newval = (value * new_rate / old_rate);
					this.updateData(period, field, newval);
				}
			}
			if(display) this.updateDisplay();
		},
		updateUseTimesheets: function(display) {
			var use_ts = UI.getFormInput(this.form, 'use_timesheets');
			if(use_ts) this.use_timesheets = use_ts.getValue();
			this.json_fields[2].readonly = !! this.use_timesheets;
			if(display) this.updateDisplay();
		},
		clearData: function(except) {
			for(period in this.data) {
				if (! except || ! (period in except)) {
					this.removeRow(period, false, false);
				}
			}
		},
		beforeSubmitForm: function() {
			if(this.editable) {
				if(! this.form.financials_data)
					MakeElt('input', {type: 'hidden', name: 'financials_data'}).appendTo(this.form);
				this.form.financials_data.value = JSON.stringify(this.updates);
			}
		}
	}
});

