<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
class Gantt{
	
	private $xml = null;
	private $min_date = null;
	private $max_date = null;
	private $debug;
	var $app_strings;
	/***
	 * 
	 * @return 
	 * @param $xmldata Object
	 * @param $parameters array(
	 * "DEBUG"
	 * "LABELS"
	 * )
	 */
	function __construct($xmldata,$parameters=array()){
		
		if(!isset($parameters["DEBUG"])){
			$this->debug=false;
		}else{
			$this->debug=$parameters["DEBUG"];
		}
		
		$this->app_strings = $parameters["LABELS"];
		
		$this->xml = new DOMDocument();
		$this->xml->loadXML($xmldata);
		
		$this->min_date = $this->get_min_date();
		
		if($this->debug){
			echo "Data minore = ".date("d/m/Y",$this->min_date)."<br>";
		}

		$this->max_date = $this->get_max_date();

		if($this->debug){
			echo "Data maggiore = ".date("d/m/Y",$this->max_date)."<br>";
		}
	
	}
	
	function data($datadaformattare,$formattazione="d/m/Y"){

		return date($formattazione,$datadaformattare);
	}
	
	function get_array_giorni(){
		$result = array();
		
		for($i=$this->min_date;$i<$this->max_date;$i+=86400){
			$result[$i]=$this->data($i,"d");
		}
		
		return $result;
	}
	
	function get_risorse(){

		$xpath = new DOMXPath($this->xml);
		
		$risorse = $xpath->query("//progetti/progetto/tasks/task/risorsa");
		
		$array_risorse = array();

		foreach($risorse as $risorsa){
			if(!isset($array_risorse[$risorsa->nodeValue])){
				$array_risorse[$risorsa->nodeValue] = $risorsa->nodeValue;
			}
		}
		
		return $array_risorse;
				
	}

	function get_progetti(){

		$xpath = new DOMXPath($this->xml);
		
		$progetti = $xpath->query("//progetti/progetto");
		
		
		$array_progetti = array();

		foreach($progetti as $progetto){
						
			$prog = array();
			$prog["id"]=$progetto->getElementsByTagName("id")->item(0)->nodeValue;
			$prog["nome"]=$progetto->getElementsByTagName("nome")->item(0)->nodeValue;
			$prog["descrizione"]=$progetto->getElementsByTagName("descrizione")->item(0)->nodeValue;
			$prog["inizio"]=strtotime($progetto->getElementsByTagName("inizio")->item(0)->nodeValue);
			$prog["fine"]=strtotime($progetto->getElementsByTagName("fine")->item(0)->nodeValue);
			$prog["stato"]=$progetto->getElementsByTagName("stato")->item(0)->nodeValue;
			array_push($array_progetti,$prog);
		}
		
		return $array_progetti;
				
	}
	
	function get_tasks_risorsa($risorsa){

		$xpath = new DOMXPath($this->xml);
		
		$tasks = $xpath->query("//progetti/progetto/tasks/task[./risorsa=\"$risorsa\"]");
		
		
		$array_task = array();

		foreach($tasks as $task){
						
			$attivita = array();
			$attivita["id"]=$task->getElementsByTagName("id")->item(0)->nodeValue;
			$attivita["nome"]=$task->getElementsByTagName("nome")->item(0)->nodeValue;
			$attivita["progetto_nome"]=$task->parentNode->parentNode->getElementsByTagName("nome")->item(0)->nodeValue;
			$attivita["progetto_id"]=$task->parentNode->parentNode->getElementsByTagName("id")->item(0)->nodeValue;
			$attivita["inizio"]=strtotime($task->getElementsByTagName("inizio")->item(0)->nodeValue);
			$attivita["fine"]=strtotime($task->getElementsByTagName("fine")->item(0)->nodeValue);
			$attivita["stato"]=$task->getElementsByTagName("stato")->item(0)->nodeValue;
			$attivita["percentuale_utilizzo"]=$task->getElementsByTagName("percentuale_utilizzo")->item(0)->nodeValue;
			array_push($array_task,$attivita);
		}
		
		return $array_task;
				
	}

	function get_tasks_progetto($progetto){

		$xpath = new DOMXPath($this->xml);
		
		$tasks = $xpath->query("//progetti/progetto[./id=\"$progetto\"]/tasks/task");
		
		
		$array_task = array();

		foreach($tasks as $task){
						
			$attivita = array();
			$attivita["nome"]=$task->getElementsByTagName("nome")->item(0)->nodeValue;
			$attivita["risorsa"]=$task->getElementsByTagName("risorsa")->item(0)->nodeValue;
			$attivita["progetto_nome"]=$task->parentNode->parentNode->getElementsByTagName("nome")->item(0)->nodeValue;
			$attivita["inizio"]=strtotime($task->getElementsByTagName("inizio")->item(0)->nodeValue);
			$attivita["fine"]=strtotime($task->getElementsByTagName("fine")->item(0)->nodeValue);
			$attivita["stato"]=$task->getElementsByTagName("stato")->item(0)->nodeValue;
			$attivita["percentuale_utilizzo"]=$task->getElementsByTagName("percentuale_utilizzo")->item(0)->nodeValue;
			$attivita["percentuale_completamento"]=$task->getElementsByTagName("percentuale_completamento")->item(0)->nodeValue;
			array_push($array_task,$attivita);
		}
		
		if($this->debug){
			echo "risorse per il progetto $progetto<br>";
			echo "<pre>".print_r($array_task,true)."</pre>";
		}
		
		return $array_task;
				
	}	
	
	/**
	 * quando $perc_completamento è 0 stampo il rosso
	 * quando $perc_completamento è 1 stampo il verde
	 * @return 
	 * @param $perc_completamento Object
	 */
	function get_STYLE_by_percentuale_completamento($perc_completamento){
		if($perc_completamento>100){
			$perc_completamento=100;
		}
		if($perc_completamento<0){
			$perc_completamento=0;
		}
		$perc_completamento = $perc_completamento/100;
				
		$verde = (int)(255*$perc_completamento);
		$rosso = (int)(255*(1-$perc_completamento));
		
		$style = "background-color:rgb($rosso,$verde,0);";
		
		if($this->debug){
			echo $style."<br>";
		}
		return $style;
	}

	function get_STYLE_by_percentuale_utilizzo($perc_utilizzo){
		
		if($perc_utilizzo>100){
			$perc_utilizzo=100;
		}
		if($perc_utilizzo<0){
			$perc_utilizzo=0;
		}
		
		$perc_utilizzo = $perc_utilizzo/100;
		
		$rosso = (int)(255*$perc_utilizzo);
		$verde = (int)(255*(1-$perc_utilizzo));
		
		$style = "background-color:rgb($rosso,$verde,0);";
		
		if($this->debug){
			echo $style."<br>";
		}
		return $style;
	}
	
	function get_CLASS_by_stato($stato){
		$class = "";
		switch($stato){
			case "Not Started":$class="task_status_not_started";break;
			case "In Progress":$class="task_status_in_progress";break;
			case "Completed":$class="task_status_completed";break;
			case "Pending Input":$class="task_status_pending_input";break;
			case "Deferred":$class="task_status_deferred";break;
		}
		return $class;
	}
	
	function get_HTML_legenda(){
		return "<table class=\"table_small\">
		<tr>
		<th>".$this->app_strings['LBL_KUMBEGANTT_Not Started']."</th><td class=\"".$this->get_CLASS_by_stato("Not Started")."\">&nbsp;</td>
		</tr>
		<tr>
		<th>".$this->app_strings['LBL_KUMBEGANTT_In Progress']."</th><td class=\"".$this->get_CLASS_by_stato("In Progress")."\">&nbsp;</td>
		</tr>
		<tr>
		<th>".$this->app_strings['LBL_KUMBEGANTT_Completed']."</th><td class=\"".$this->get_CLASS_by_stato("Completed")."\">&nbsp;</td>
		</tr>
		<tr>
		<th>".$this->app_strings['LBL_KUMBEGANTT_Pending Input']."</th><td class=\"".$this->get_CLASS_by_stato("Pending Input")."\">&nbsp;</td>
		</tr>
		<tr>
		<th>".$this->app_strings['LBL_KUMBEGANTT_Deferred']."</th><td class=\"".$this->get_CLASS_by_stato("Deferred")."\">&nbsp;</td>
		</tr>
		</table>";
		
	}
	

	
	/**
	 * processano l'xml del gantt determina la data minima
	 * @return 
	 */
	function get_min_date(){
		$inizi = $this->xml->getElementsByTagName("inizio");
		
		$tmp = null;
		foreach($inizi as $inizio){

			if(strlen($inizio->nodeValue)==strlen("YYYYMMDD")){

				if($tmp==null){
					$tmp = strtotime($inizio->nodeValue);
				}
				if($tmp>strtotime($inizio->nodeValue)){
					$tmp = strtotime($inizio->nodeValue);
				}				
			}
		}
			return $tmp;		
	}

	/**
	 * processano l'xml del gantt determina la data massima
	 * @return 
	 */
	function get_max_date(){
		$fini = $this->xml->getElementsByTagName("fine");
		
		$tmp = null;
		foreach($fini as $fine){
			
			if(strlen($fine->nodeValue)==strlen("YYYYMMDD")){
			if($tmp==null){
				$tmp = strtotime($fine->nodeValue);
			}
			if($tmp<strtotime($fine->nodeValue)){
				$tmp = strtotime($fine->nodeValue);
			}
			}
		}
			return $tmp;	
	}
	
	function __destruct(){
		
	}
}
?>
