<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

class GanttRender {
	var $per_project = true;
	var $per_resource = true;
	
	function render_unsupported() {
		if(! AppConfig::setting('layout.svg_charts_enabled')) {
			if(! function_exists('gd_info'))
				return 'Please install the GD PHP module (with FreeType support) to enable this functionality';
			if(! function_exists('imageftbbox'))
				return 'PHP must be compiled with FreeType support enabled (--with-freetype-dir=DIR)';
		}
	}
	
	function render($proj_id) {
		global $mod_strings, $app_strings;

		$html = '<div class="main_container_gantt" id="main_container_gantt">';

		require_once ('modules/Project/Project.php');
		require_once("gantt.class.php");
		require_once("gantt_image.php");
		require_once("gantt.xmlgenerator.php");

		$html .= "<form action=\"\" method=\"POST\" name=\"Filter\" target=\"_blank\">
		<input type=\"hidden\" name=\"image\" value=\"1\">
		<input type=\"hidden\" name=\"to_pdf\" value=\"true\">
		<table class=\"table_small\">";
		
		$html .= '</form>';

		$array_conditions = array();

		/**
		 * se l'utente può vedere anche gli altri progetti applico le condizioni di filtro
		 * altrimenti gli consento di vedere solo i suoi progetti / task
		 */
		if(ACLController::checkAccess('ProjectTask', 'list', false)){

			if(isset($_POST["owner"])){
				$array_conditions["task-users-id"]=$_POST["owner"];
			}

			if(!empty($proj_id)){
				$array_conditions["project-id"]=$proj_id;
			}

		}else{
			// forzo il limite ai progetti miei
			$array_conditions["task-users-id"] = AppConfig::current_user_id();

			if(!empty($proj_id)){
				$array_conditions["project-id"]=$proj_id;
			}
		}		

		$project = ListQuery::quick_fetch_row('Project', $array_conditions["project-id"]);
		$lq = new ListQuery('ProjectTask');
		$lq->addSimpleFilter('parent_id', $array_conditions["project-id"], '=');
		if(!empty($array_conditions["task-users-id"])) {
			if(is_array($array_conditions["task-users-id"])) {
				$lq->addSimpleFilter('assigned_user_id', $array_conditions["task-users-id"], 'IN');
			} else {
				$lq->addSimpleFilter('assigned_user_id', $array_conditions["task-users-id"], '=');
			}
		}
		$lq->setOrderBy('order_number, milestone_flag DESC');
		$t_res = $lq->fetchAll()->rows;

		$result = array();
		$result[$project['id']] = $project;
		$project['tasks'] = array();
		

		foreach($t_res as $rs_tasks) {
			$result[$project['id']]['tasks'][$rs_tasks['id']] = $rs_tasks;
		}
		
		$parameters = array("DEBUG"=>false,"LABELS"=>$mod_strings);

		require_once 'XTemplate/xtpl.php';
		$tpl = new XTemplate('modules/Project/gantt.html');
		$tpl->assign('APP', $app_strings);
		$tpl->assign('PROJECT_ID', $proj_id);

		$backend = AppConfig::setting('layout.svg_charts_enabled') ? 'SVG' : 'GD';

		$g = new GanttImage($result, $backend, $parameters);

		$path = 'gantt/' . substr($proj_id, 0,1) . '/' . substr($proj_id, 1, 1) . '/' .  $proj_id . '.' . $g->image->fileExt(); 
		create_cache_directory($path);
		$filename = AppConfig::cache_dir() . $path;
		$g->draw();

		$g->image->render($filename);
		$tpl->assign('IMAGE_HTML', $g->image->renderHTML($filename));
		foreach ($g->imageMap as $area) {
			$tpl->assign('area', $area);
			$tpl->parse('main.GD.' . $area['type']);
		}
		$tpl->parse('main.GD');
		$tpl->parse('main');
		return $tpl->text('main');
	}
}
