<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormTableSection.php');
require_once('modules/Project/Project.php');

class FinancialWidget extends FormTableSection {
	function init($params=null, $model=null) {
		parent::init($params, $model);
		if(! $this->id)
			$this->id = 'project_financial';
	}
	
	function getRequiredFields() {
		return array('total_expected_cost', 'total_expected_revenue', 'total_actual_cost', 'total_actual_revenue');
	}
	
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
        global $current_user, $app_strings, $mod_strings;
        
		if($gen->getLayout()->getEditingLayout()) {
			$params = array();
			return $this->renderOuterTable($gen, $parents, $context, $params);		
		}

		$lstyle = $gen->getLayout()->getType();
		$allow_edit = $gen->getLayout()->getDefaultEditable() ? 1 : 0;
		$form_name = $gen->getFormName();

        $gen->getLayout()->addScriptInclude("modules/Project/financials.js", LOAD_PRIORITY_BODY);

		$use_timesheets = $row_result->getField('use_timesheets');
        $show_financials = (bool)$current_user->getPreference('financial_information');
        if (! $show_financials)
        	return;

		if($row_result->new_record && $_REQUEST['action'] != 'Duplicate') {
			$records = [];
		} else {
			$finance = new ProjectFinancials();
			$upd = RowUpdate::for_result($row_result);
			$finance->load($upd);
			$records = $finance->get_records();
		}
		
		$curr = new Currency();
		$curr->retrieve($row_result->getField('currency_id'));

        $lbl_month = translate('LBL_MONTH', 'Project');
        $lbl_expect_cost = translate('LBL_EXPECTED_COST', 'Project');
        $lbl_expect_revenue = translate('LBL_EXPECTED_REVENUE', 'Project');
        $lbl_actual_cost = translate('LBL_ACTUAL_COST', 'Project');
        $lbl_actual_revenue = translate('LBL_ACTUAL_REVENUE', 'Project');        
        $lbl_totals = translate('LBL_TOTAL', 'Project');        
        $panel_edit = $allow_edit ? '' : 'panel-readonly';

        $total_expected_cost = currency_format_number($row_result->getField('total_expected_cost'));
        $total_expected_revenue = currency_format_number($row_result->getField('total_expected_revenue'));
        $total_actual_cost = currency_format_number($row_result->getField('total_actual_cost'));
        $total_actual_revenue = currency_format_number($row_result->getField('total_actual_revenue'));

		$body = <<<EOQ
			<div id="{$this->id}-main" class="financials-main">
				<div class="row form-row">
					<div class="column form-cell expand"><div class="form-entry">
					<section class="card-outer panel-outer panel-default financials-outer panel-border panel-round $panel_edit">
						<div class="card-header panel-subheader">
							<div class="row col-space-sm panel-column-titles">
								<div class="column financials-row-head financials-col-month"><span class="column-title">{$lbl_month}</span></span></div>
								<div class="column financials-row-head financials-col-expcost"><span class="column-title">{$lbl_expect_cost}</span></div>
								<div class="column financials-row-head financials-col-exprev"><span class="column-title">{$lbl_expect_revenue}</span></div>
								<div class="column financials-row-head financials-col-actcost"><span class="column-title">{$lbl_actual_cost}</span></div>
								<div class="column financials-row-head financials-col-actrev"><span class="column-title">{$lbl_actual_revenue}</span></div>
								<div class="column financials-row-head financials-col-end">&nbsp;</div>
							</div>
						</div>
						<div class="card-body panel-body panel-pad-body financials-body">
						</div>
					</section>
					<section class="card-outer panel-outer panel-default financials-outer panel-border panel-round $panel_edit">
						<div class="card-body panel-body panel-pad-body financials-body">
							<div class="financials-row-data col-space-sm row">
								<div class="text-number financials-col-month column"><b>{$lbl_totals}</b></div>
								<div class="financials-col-expcost text-right column">{$total_expected_cost}</div>
								<div class="financials-col-exprev text-right column">{$total_expected_revenue}</div>
								<div class="financials-col-actcost text-right column">{$total_actual_cost}</div>
								<div class="financials-col-actrev text-right column">{$total_actual_revenue}</div>
								<div class="financials-col-end column"></div>
							</div>
							</div>
						</div>
					</section>
					</div>
				</div>
			</div>
EOQ;

		$editParams = [
			'data' => $records,
			'use_timesheets' => $use_timesheets,
			'editable' => $allow_edit,
			'start_date' => $row_result->getField('date_starting'),
			'end_date' => $row_result->getField('date_ending'),
			'decimal_places' => $curr->decimal_places,
		];
		$gen->getFormObject()->addControl('FinancialsEditor', $this->id, $editParams);

		$params = ['body' => $body];
        return $this->renderOuterTable($gen, $context, $parents, $params);
	}


    function loadUpdateRequest(RowUpdate $update, array $input) {
        $upd_financials = array();
        if(! empty($input['financials_data'])) {
            require_once('include/JSON.php');
            $json = new JSON(JSON_LOOSE_TYPE);
            $upd_financials = $json->decode(from_html($input['financials_data']));
        }
        $update->setRelatedData($this->id.'_rows', $upd_financials);
	}
	
	function validateInput(RowUpdate $update) {
		return true;
	}
	
	function beforeUpdate(RowUpdate $update) {
		$row_updates = $update->getRelatedData($this->id.'_rows');
		$project_id = $update->getField('id');
        if (! $project_id) {
			$project_id = create_guid();
			$update->set('id', $project_id);
		}

        if($project_id != null) {
        	$finance = new ProjectFinancials();
            $finance->load($update);

            if (is_countable($row_updates) && sizeof($row_updates) > 0){
                foreach($row_updates as $period => $data) {
                    $finance->update_for_period($period, $data);
                }
            }

            $finance->calculate_costs($update->getField('use_timesheets'));
            $finance->save();
        }
	}
	
	function renderPdf(PdfFormGenerator $gen, $row_result, array $parents, array $context) {
        global $current_user, $mod_strings;

        $show_financials = (bool)$current_user->getPreference('financial_information');
        
        if ($show_financials) {
			$finance = new ProjectFinancials();
            $finance->load(RowUpdate::for_result($row_result));
            $periods = $finance->get_project_financial_periods();

            $numbers = Array();
            foreach($periods as $period) {
                $numbers[$period] = array();
                foreach(ProjectFinancials::$value_fields as $f)
                    $numbers[$period][$f] = 0.0;
            }

            foreach($finance->get_records() as $period => $values) {
                $numbers[$period] = $values;
            }
            ksort($numbers);

			$title = $this->getLabel();

			$cols = array(
				'month' => array('width' => '12%', 'title' => $mod_strings['LBL_MONTH']),
				'expected_cost' => array('width' => '22%', 'title' => $mod_strings['LBL_EXPECTED_COST']),
				'expected_revenue' => array('width' => '22%', 'title' => $mod_strings['LBL_EXPECTED_REVENUE']),
				'actual_cost' => array('width' => '22%', 'title' => $mod_strings['LBL_ACTUAL_COST']),
				'actual_revenue' => array('width' => '22%', 'title' => $mod_strings['LBL_ACTUAL_REVENUE']),
			);

            $cparams = array('currency_symbol' => false, 'currency_id' => $row_result->getField('currency_id'));
            $body = '';
            
            foreach($numbers as $period => $values) {
                $body .= '<tr><td class="tabDetailViewDL" style="text-align: left" NOWRAP>' .$period. '</td>';

				$row = array('month' => $period);
                foreach(ProjectFinancials::$value_fields as $f) {
                    if($values[$f] == 0)
                        $val = "-";
                    else
                        $val = currency_format_number($values[$f], $cparams);

					$row[$f] = $val;
				}
				$data[] = $row;
            }
			$gen->pdf->DrawTable($data, $cols, $title, true);

        }
	}
}
?>
