<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormTableSection.php');
require_once('modules/ProjectTask/ProjectTask.php');

class CostsWidget extends FormTableSection {

	function init($params=null, $model=null) {
		parent::init($params, $model);
		if(! $this->id)
			$this->id = 'task_costs';
	}
	
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if($gen->getLayout()->getEditingLayout()) {
			$params = array();
			return $this->renderOuterTable($gen, $parents, $context, $params);		
		}
		if($gen->getLayout()->getDefaultEditable())
			return '';

		return $this->renderHtmlView($gen, $row_result, $parents, $context);
	}
	
	function getRequiredFields() {
		return array('estimated_effort', 'estimated_cost', 'actual_effort', 'actual_cost',
			'parent_id', 'currency_id', 'exchange_rate');
	}
	
	function renderHtmlView(HtmlFormGenerator $gen, RowResult $row_result, array $parents, array $context) {
        global $mod_strings, $current_user;

        $id = $row_result->getField('id');
        $project_id = $row_result->getField('parent_id');
        $exchange_rate = $row_result->getField('exchange_rate');
        $currency_id = $row_result->getField('currency_id');

        if (! $currency_id)
            $currency_id = '-99';

        $show_financials = (bool)$current_user->getPreference('financial_information');
        $estimated_effort = $row_result->getField('estimated_effort', null, true);
        $actual_row = '';
        $estimated_html = '';
        $width = "80%";
        

        if ($show_financials) {
            $fparams = array('currency_id' => $currency_id);

            $width = "30%";
            $estimated_cost = $row_result->getField('estimated_cost', null, true);

            $estimated_html = <<<EOQ
				<div class="column form-cell"><div class="form-entry">
					<div class="form-label">{$mod_strings['LBL_ESTIMATED_COST']}</div>
					<div class="form-value">{$estimated_cost}</div>
				</div></div>
EOQ;

            if($this->useTimesheets($project_id)) {
                $actual_effort = $row_result->getField('actual_effort', null, true);                
                $actual_cost = $row_result->getField('actual_cost', null, true);

			$actual_row = <<<EOQ
				<div class="row form-row">
				<div class="column form-cell"><div class="form-entry">
					<div class="form-label">{$mod_strings['LBL_ACTUAL_EFFORT']}</div>
					<div class="form-value">{$actual_effort}</div>
				</div></div>
				<div class="column form-cell"><div class="form-entry">
					<div class="form-label">{$mod_strings['LBL_ACTUAL_COST']}</div>
					<div class="form-value">{$actual_cost}</div>
				</div></div>
				</div>
EOQ;
            }
        }

        $body = <<<EOQ
			<div class="row form-row">
			<div class="column form-cell"><div class="form-entry">
				<div class="form-label">{$mod_strings['LBL_ESTIMATED_EFFORT']}</div>
				<div class="form-value">{$estimated_effort}</div>
			</div></div>
			{$estimated_html}
            </div>
            {$actual_row}
EOQ;

		$params = ['title' => '', 'body' => $body];
        return $this->renderOuterTable($gen, $context, $parents, $params);
	}

    /**
     * Check use parent project timesheets or not
     *
     * @return bool
     */
    function useTimesheets($project_id) {
        return ProjectTask::use_timesheets($project_id);
    }

	function validateInput(RowUpdate $update) {
		return true;
    }
}
?>
