<?php

require_once 'modules/Quotes/PDFTemplatesProvider.php';

class PurchaseOrderPDFTemplateProvider extends QuotePDFTemplateProvider
{

	protected function __construct($type)
	{
		parent::__construct($type);
	}

	public static function template_types()
	{
		return array(
			'purchase_order' => translate('LBL_PDF_PURCHASEORDER_TITLE', 'PurchaseOrders'),
		);
	}

	public static function template_skeleton($type)
	{
		$ret = array('sections' => array(
			array(
				'name' => 'header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_HEADER',
				'height' => 150,
				'elements' => array (),
			),
			array(
				'name' => 'group_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_GROUP_HEADER',
				'height' => 150,
				'elements' => array (),
			),
			array(
				'name' => 'group_items',
				'type' => 'table',
				'title' => 'LBL_SKEL_GROUP_ITEMS',
				'columns' => array (),
			),
			array(
				'name' => 'group_totals_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_GROUP_TOTALS_HEADER',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'group_totals',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_GROUP_TOTALS',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'group_footer',
				'data_id' => 'group_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_GROUP_FOOTER',
				'height' => 150,
				'elements' => array (),
			),
			array(
				'name' => 'quote_totals_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_QUOTE_TOTALS_HEADER',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'quote_totals',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_QUOTE_TOTALS',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'notes',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_NOTES',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'terms',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_TERMS',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'footer',
				'data_id' => 'header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_FOOTER',
				'height' => 40,
				'elements' => array (),
			),
		));
		foreach ($ret['sections'] as &$sec) $sec['title'] = translate($sec['title'], 'PurchaseOrders');
		$ret['sections'] = array_values($ret['sections']);
		return $ret;
	}

	public static function template_sample_data($type)
	{
		$datefmt = AppConfig::setting('locale.defaults.date_format');
		$data = array(
			'header' => array(
				'todays_date' => array(translate('LBL_TODAYS_DATE'), date($datefmt), ),
				'number' => array(translate('LBL_SAMPLE_HEADER_NUMBER_LABEL', 'PurchaseOrders'), 'SO2013-001'),
				'terms' => array(translate('LBL_SAMPLE_HEADER_TERMS_LABEL', 'PurchaseOrders'), translate('LBL_SAMPLE_HEADER_TERMS_VALUE', 'PurchaseOrders'), ),
				'date' => array(translate('LBL_SAMPLE_HEADER_DATE_LABEL', 'PurchaseOrders'), '2014-11-05'),
				'sales_person' => array(translate('LBL_SAMPLE_HEADER_SALES_PERSON_LABEL', 'PurchaseOrders'), translate('LBL_SAMPLE_HEADER_SALES_PERSON_VALUE', 'PurchaseOrders'), ),
				'tax_info' => array(translate('LBL_SAMPLE_HEADER_TAX_INFO_LABEL', 'PurchaseOrders'), ''),
				'billing_address' => array(translate('LBL_SAMPLE_HEADER_BILLING_ADDRESS_LABEL', 'PurchaseOrders'), translate('LBL_SAMPLE_HEADER_BILLING_ADDRESS_VALUE', 'PurchaseOrders'), ),
				'shipping_address' => array(translate('LBL_SAMPLE_HEADER_SHIPPING_ADDRESS_LABEL', 'PurchaseOrders'), translate('LBL_SAMPLE_HEADER_SHIPPING_ADDRESS_VALUE', 'PurchaseOrders'), ),
				'description' => array(translate('LBL_SAMPLE_HEADER_DESCRIPTION_LABEL', 'PurchaseOrders'), translate('LBL_SAMPLE_HEADER_DESCRIPTION_VALUE', 'PurchaseOrders'), ),
				'standard_terms' => array(translate('LBL_SAMPLE_HEADER_STANDARD_TERMS_LABEL', 'PurchaseOrders'), ''),
			),
			'group_header' => array(
				'group_name' => array(translate('LBL_SAMPLE_GROUP_HEADER_GROUP_NAME_LABEL', 'PurchaseOrders'), translate('LBL_SAMPLE_GROUP_HEADER_GROUP_NAME_VALUE', 'PurchaseOrders'), ),
				'status' => array(translate('LBL_SAMPLE_GROUP_HEADER_STATUS_LABEL', 'PurchaseOrders'), translate('LBL_SAMPLE_GROUP_HEADER_STATUS_VALUE', 'PurchaseOrders'), ),
			),
			'group_totals' => array(
				'total_name' => array(translate('LBL_SAMPLE_GROUP_TOTALS_TOTAL_NAME_LABEL', 'PurchaseOrders'), translate('LBL_SAMPLE_GROUP_TOTALS_TOTAL_NAME_VALUE', 'PurchaseOrders'), ),
				'total_value' => array(translate('LBL_SAMPLE_GROUP_TOTALS_TOTAL_VALUE_LABEL', 'PurchaseOrders'), translate('LBL_SAMPLE_GROUP_TOTALS_TOTAL_VALUE_VALUE', 'PurchaseOrders'), ),
			),
			'quote_totals' => array(
				'total_name' => array(translate('LBL_SAMPLE_QUOTE_TOTALS_TOTAL_NAME_LABEL', 'PurchaseOrders'), translate('LBL_SAMPLE_QUOTE_TOTALS_TOTAL_NAME_VALUE', 'PurchaseOrders'), ),
				'total_value' => array(translate('LBL_SAMPLE_QUOTE_TOTALS_TOTAL_VALUE_LABEL', 'PurchaseOrders'), translate('LBL_SAMPLE_QUOTE_TOTALS_TOTAL_VALUE_VALUE', 'PurchaseOrders'), ),
			),
			'group_items' => array(
				'number' => array(translate('LBL_SAMPLE_GROUP_ITEMS_NUMBER_LABEL', 'PurchaseOrders'), '1'),
				'product' => array(translate('LBL_SAMPLE_GROUP_ITEMS_PRODUCT_LABEL', 'PurchaseOrders'), translate('LBL_SAMPLE_GROUP_ITEMS_PRODUCT_VALUE', 'PurchaseOrders'), ),
				'orig_product' => array(translate('LBL_SAMPLE_GROUP_ITEMS_ORIG_PRODUCT_LABEL', 'PurchaseOrders'), translate('LBL_SAMPLE_GROUP_ITEMS_ORIG_PRODUCT_VALUE', 'PurchaseOrders'), ),
				'quantity' => array(translate('LBL_SAMPLE_GROUP_ITEMS_QUANTITY_LABEL', 'PurchaseOrders'), '2'),
				'mfr_part_no' => array(translate('LBL_SAMPLE_GROUP_ITEMS_MFR_PART_NO_LABEL', 'PurchaseOrders'), '12345'),
				'listprice' => array(translate('LBL_SAMPLE_GROUP_ITEMS_LISTPRICE_LABEL', 'PurchaseOrders'), '10.0'),
				'unitprice' => array(translate('LBL_SAMPLE_GROUP_ITEMS_UNITPRICE_LABEL', 'PurchaseOrders'), '20.0'),
				'extprice' => array(translate('LBL_SAMPLE_GROUP_ITEMS_EXTPRICE_LABEL', 'PurchaseOrders'), '40.0'),
			),
		);
		self::injectProductSampleData($data['group_items']);
		$ret = array();
		foreach ($data as $k => $v) {
			$ret[$k] = array();
			foreach ($v as $kk => $vv) {
				$ret[$k][$kk] = array('label' => $vv[0], 'value' => $vv[1]);
			}
		}
		return $ret;
	}


	public static function create($type)
	{
		return new PurchaseOrderPDFTemplateProvider($type);
	}

	protected function _address($pfx, $row=null)
	{
		$focus = $row ? $row : $this->row;

		if ($pfx == 'billing_') {
			$acc = ListQuery::quick_fetch('Account', $focus->getField('supplier_id'));
			if ($acc) {
				$acc->assign('billing_contact_name', $focus->getField('supplier_contact'));
				$acc->assign('billing_account_name', $acc->getField('name'));
				return parent::_address('billing_', $acc);
			}
		} else if ($focus->getField('drop_ship')) {
			return parent::_address('shipping_');
		} else {
			return parent::_address('_shipping_');
		}
	}

	public function initialize($layout, ListQuery $lq)
	{
		$lq->addField('supplier_contact');
		return parent::initialize($layout, $lq);
	}

	public function getFileName() {
		$filename_pfx = translate('LBL_PDF_PURCHASEORDER_TITLE', 'PurchaseOrders');
		$filename = $filename_pfx . '_' . $this->row->getField('prefix');
		$filename .= $this->row->getField('po_number') . '_' . $this->row->getField('supplier_name');
		$filename = preg_replace('~\.+$~', '', trim($filename));
		$filename = $filename . '.pdf';
		return $filename;
	}

	public function getNoteTitle()
	{
		return translate('LBL_PREPARED_PURCHASEORDER', 'PurchaseOrders');
	}

	public function getEmailTitle()
	{
		return translate('LBL_EMAILED_PURCHASEORDER', 'PurchaseOrders');
	}

	public function getSignedDocumentTitle() {
		$prefix =
			$this->row
			? '[' . $this->row->getField('prefix') . $this->row->getField('po_number') . '] '
			: '';

		return $prefix . $this->getNoteTitle();
	}

	protected function createData()
	{
		global $timedate;
		$row =& $this->row;
		$user = new User();
		$user->retrieve($row->getField('assigned_user_id'), false);
		$terms = AppConfig::setting("lang.lists.current.app.terms_dom", array());
		$currency =& $this->get_currency();
		$tax_information = $this->row->getField('tax_information');
		$datefmt = AppConfig::setting('locale.defaults.date_format');
		$data = array(
			'todays_date' => date($datefmt),
			'number' => $row->getField('prefix') . $row->getField('po_number'),
			'terms' => array_get_default($terms, $row->getField('terms'), $row->getField('terms')),
			'date' => $timedate->to_display_date($row->getField('date_entered')),
			'sales_person' => $row->getField('assigned_user_name'),
			'tax_info' => from_html(AppConfig::setting('company.tax_information', '')),
			'billing_address' => $this->_address('billing_') . "\n" . $tax_information,
			'shipping_address' => $this->_address('shipping_'),
			'description' => $row->getField('description'),
			'standard_terms' => trim(AppConfig::setting('company.std_po_terms', '')),
		);
		$data = array_merge($this->header_data, $data);
		$this->data['header'] = $data;
		$this->createGroupData();
	}

	protected function findSection($name) {
		$s = parent::findSection($name);
		if (($name != 'header' && $name != 'footer') || $this->row->getField('drop_ship'))
			return $s;
		foreach ($s['elements'] as $i => $e)  {
			if ($e['type'] == 'field' && $e['field'] == 'shipping_address' && $s['elements'][$i] == '')
				unset($s['elements'][$i]);
			if (!empty($e['is_ship_to_label']) && $s['elements'][$i] == '')
				unset($s['elements'][$i]);
		}
		$s['elements'] = array_values($s['elements']);
		return $s;
	}

	public static function get_extra_sources() {
		return array(
			array(
				'module' => 'Invoice',
				'model' => 'Invoice',
				'prefix' => 'related_invoice',
				'label' => 'LBL_RELATED_INVOICE',
			),
			array(
				'module' => 'SalesOrders',
				'model' => 'SalesOrder',
				'prefix' => 'from_so',
				'label' => 'LBL_RELATED_SO',
			),
			array(
				'module' => 'Accounts',
				'model' => 'Account',
				'prefix' => 'shipping_account',
				'label' => 'LBL_SHIPPING_ACCOUNT',
			),
			array(
				'module' => 'Accounts',
				'model' => 'Account',
				'prefix' => 'supplier',
				'label' => 'LBL_SUPPLIER',
			),
			array(
				'module' => 'Contacts',
				'model' => 'Contact',
				'prefix' => 'supplier_contact',
				'label' => 'LBL_SUPPLIER_CONTACT',
			),
			array(
				'module' => 'Contacts',
				'model' => 'Contact',
				'prefix' => 'shipping_contact',
				'label' => 'LBL_SHIPPING_CONTACT',
			),
			array(
				'module' => 'ShippingProviders',
				'model' => 'ShippingProvider',
				'prefix' => 'shipping_provider',
				'label' => 'LBL_SHIPPING_PROVIDER',
			),
		);
	}

}

