<?php

require_once 'modules/PDFTemplates/PDFTemplateProviderBase.php';
require_once 'modules/Invoice/FinalMilestone.php';

class QuotePDFTemplateProvider extends PDFTemplateProviderBase
{

	public $row;
	public $layout;
	public $type;
	public $use_language;
	public $alt_lang_module;

	protected $taxedShipping = 0;
	protected $shipping = 0;
	public $groups;
	public $gtotal;

	public static function template_types()
	{
		return array(
			'quote' => translate('LBL_PDF_TYPE_QUOTE', 'Quotes'),
			'proforma' => translate('LBL_PDF_TYPE_PROFORMA', 'Quotes'),
		);
	}

	protected function __construct($type)
	{
		$this->type = $type;
	}

	public static function template_skeleton($type)
	{
		$ret = array('sections' => array(
			array(
				'name' => 'header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_HEADER',
				'height' => 150,
				'elements' => array (),
			),
			array(
				'name' => 'group_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_GROUP_HEADER',
				'height' => 150,
				'elements' => array (),
			),
			array(
				'name' => 'group_items',
				'type' => 'table',
				'title' => 'LBL_SKEL_GROUP_ITEMS',
				'columns' => array (),
			),
			array(
				'name' => 'group_totals_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_GROUP_TOTALS_HEADER',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'group_totals',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_GROUP_TOTALS',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'group_footer',
				'data_id' => 'group_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_GROUP_FOOTER',
				'height' => 150,
				'elements' => array (),
			),
			array(
				'name' => 'quote_totals_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_QUOTE_TOTALS_HEADER',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'quote_totals',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_QUOTE_TOTALS',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'notes',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_NOTES',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'terms',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_TERMS',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'quote_accept',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_QUOTE_ACCEPT',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'footer',
				'data_id' => 'header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_FOOTER',
				'height' => 40,
				'elements' => array (),
			),
		));
		foreach ($ret['sections'] as &$sec) $sec['title'] = translate($sec['title'], 'Quotes');
		return $ret;
	}

	public static function template_sample_data($type)
	{
		$datefmt = AppConfig::setting('locale.defaults.date_format');
		$data = array(
			'header' => array(
				'todays_date' => array(translate('LBL_TODAYS_DATE'), date($datefmt), ),
				'number' => array(translate('LBL_SAMPLE_HEADER_NUMBER_LABEL', 'Quotes'), 'Q2013-001'),
				'terms' => array(translate('LBL_SAMPLE_HEADER_TERMS_LABEL', 'Quotes'), translate('LBL_SAMPLE_HEADER_TERMS_VALUE', 'Quotes'), ),
				'date' => array(translate('LBL_SAMPLE_HEADER_DATE_LABEL', 'Quotes'), '2014-11-05'),
				'sales_person' => array(translate('LBL_SAMPLE_HEADER_SALES_PERSON_LABEL', 'Quotes'), translate('LBL_SAMPLE_HEADER_SALES_PERSON_VALUE', 'Quotes'), ),
				'sales_person_email' => array(translate('LBL_SAMPLE_HEADER_SALES_PERSON_EMAIL_LABEL', 'Quotes'), translate('LBL_SAMPLE_HEADER_SALES_PERSON_EMAIL_VALUE', 'Quotes'), ),
				'sales_person_phone' => array(translate('LBL_SAMPLE_HEADER_SALES_PERSON_PHONE_LABEL', 'Quotes'), translate('LBL_SAMPLE_HEADER_SALES_PERSON_PHONE_VALUE', 'Quotes'), ),
				'valid_until' => array(translate('LBL_SAMPLE_HEADER_VALID_UNTIL_LABEL', 'Quotes'), '2014-11-05'),
				'tax_info' => array(translate('LBL_SAMPLE_HEADER_TAX_INFO_LABEL', 'Quotes'), ''),
				'billing_address' => array(translate('LBL_SAMPLE_HEADER_BILLING_ADDRESS_LABEL', 'Quotes'), translate('LBL_SAMPLE_HEADER_BILLING_ADDRESS_VALUE', 'Quotes'), ),
				'billing_address_contact_last' => array(translate('LBL_SAMPLE_HEADER_BILLING_ADDRESS_CONTACT_LAST_LABEL', 'Quotes'), translate('LBL_SAMPLE_HEADER_BILLING_ADDRESS_VALUE', 'Quotes'), ),
				'shipping_address' => array(translate('LBL_SAMPLE_HEADER_SHIPPING_ADDRESS_LABEL', 'Quotes'), translate('LBL_SAMPLE_HEADER_SHIPPING_ADDRESS_VALUE', 'Quotes'), ),
				'shipping_address_contact_last' => array(translate('LBL_SAMPLE_HEADER_SHIPPING_ADDRESS_CONTACT_LAST_LABEL', 'Quotes'), translate('LBL_SAMPLE_HEADER_SHIPPING_ADDRESS_VALUE', 'Quotes'), ),
				'description' => array(translate('LBL_SAMPLE_HEADER_DESCRIPTION_LABEL', 'Quotes'), translate('LBL_SAMPLE_HEADER_DESCRIPTION_VALUE', 'Quotes'), ),
				'standard_terms' => array(translate('LBL_SAMPLE_HEADER_STANDARD_TERMS_LABEL', 'Quotes'), trim(AppConfig::setting('company.std_quote_terms', ''))),
			),
			'group_header' => array(
				'group_name' => array(translate('LBL_SAMPLE_GROUP_HEADER_GROUP_NAME_LABEL', 'Quotes'), translate('LBL_SAMPLE_GROUP_HEADER_GROUP_NAME_VALUE', 'Quotes'), ),
				'status' => array(translate('LBL_SAMPLE_GROUP_HEADER_STATUS_LABEL', 'Quotes'), translate('LBL_SAMPLE_GROUP_HEADER_STATUS_VALUE', 'Quotes'), ),
			),
			'group_totals' => array(
				'total_name' => array(translate('LBL_SAMPLE_GROUP_TOTALS_TOTAL_NAME_LABEL', 'Quotes'), translate('LBL_SAMPLE_GROUP_TOTALS_TOTAL_NAME_VALUE', 'Quotes'), ),
				'total_value' => array(translate('LBL_SAMPLE_GROUP_TOTALS_TOTAL_VALUE_LABEL', 'Quotes'), translate('LBL_SAMPLE_GROUP_TOTALS_TOTAL_VALUE_VALUE', 'Quotes'), ),
			),
			'quote_totals' => array(
				'total_name' => array(translate('LBL_SAMPLE_QUOTE_TOTALS_TOTAL_NAME_LABEL', 'Quotes'), translate('LBL_SAMPLE_QUOTE_TOTALS_TOTAL_NAME_VALUE', 'Quotes'), ),
				'total_value' => array(translate('LBL_SAMPLE_QUOTE_TOTALS_TOTAL_VALUE_LABEL', 'Quotes'), translate('LBL_SAMPLE_QUOTE_TOTALS_TOTAL_VALUE_VALUE', 'Quotes'), ),
			),
			'group_items' => array(
				'number' => array(translate('LBL_SAMPLE_GROUP_ITEMS_NUMBER_LABEL', 'Quotes'), '1'),
				'product' => array(translate('LBL_SAMPLE_GROUP_ITEMS_PRODUCT_LABEL', 'Quotes'), translate('LBL_SAMPLE_GROUP_ITEMS_PRODUCT_VALUE', 'Quotes'), ),
				'quantity' => array(translate('LBL_SAMPLE_GROUP_ITEMS_QUANTITY_LABEL', 'Quotes'), '2'),
				'listprice' => array(translate('LBL_SAMPLE_GROUP_ITEMS_LISTPRICE_LABEL', 'Quotes'), '10.0'),
				'unitprice' => array(translate('LBL_SAMPLE_GROUP_ITEMS_UNITPRICE_LABEL', 'Quotes'), '20.0'),
				'costprice' => array(translate('LBL_SAMPLE_GROUP_ITEMS_COSTPRICE_LABEL', 'Quotes'), '5.0'),
				'extprice' => array(translate('LBL_SAMPLE_GROUP_ITEMS_EXTPRICE_LABEL', 'Quotes'), '40.0'),
				'discount' => array(translate('LBL_SAMPLE_GROUP_ITEMS_DISCOUNT_LABEL', 'Quotes'), '5%'),
				'mfr_part_no' => array(translate('LBL_SAMPLE_GROUP_ITEMS_MFR_PART_NO_LABEL', 'Quotes'), '123456'),
			),
		);
		self::injectProductSampleData($data['group_items']);
		$ret = array();
		foreach ($data as $k => $v) {
			$ret[$k] = array();
			foreach ($v as $kk => $vv) {
				$ret[$k][$kk] = array('label' => $vv[0], 'value' => $vv[1]);
			}
		}
		return $ret;
	}

	protected static function injectProductSampleData(&$data, $row = null)
	{
		global $locale;
		if ($row) {
			require_once 'include/layout/FieldFormatter.php';
			$fmt = new FieldFormatter('pdf');
			$lq = new ListQuery('Product');
			$lq->addColumnFields();
			$lq->addField('weight');
			$fields = $lq->getFields();
		} else {
			$model = new ModelDef('Product');
			$fields = $model->getExtendedFieldDefinitions();
		}

		$fetched_row = null;
		if ($row && $row['related_type'] == 'ProductCatalog' && $row['related_id'])
			$fetched_row = $lq->queryRecord($row['related_id']);
		if (!$fetched_row || $fetched_row->failed)
			$fetched_row = array();
		else
			$fetched_row = $fetched_row->row;
		if ($row) {
			$formatted = $fmt->formatRow($fields, $fetched_row);
			if(empty($data['vendor_part_no']))
				$data['vendor_part_no'] = array_get_default($formatted, 'vendor_part_no', '');
			$data['weight'] = $locale->getUserFormatWeighty(array($formatted['weight_1'], $formatted['weight_2']));
			$data['orig_product'] = array_get_default($formatted, 'name', $data['product']);
		} else {
			if(empty($data['vendor_part_no']))
				$data['vendor_part_no'] = array('Vendor Part No.', 'VP-1234');
			$data['weight'] = array('Weight', '1');
		}

		foreach ($fields as $k => $f) {
			if (isset($f['type']) && in_array($f['type'], self::$field_types)) {
				if (!empty($f['created_by_module_designer'])
					|| array_get_default($f['source'], 'type') == 'custom_field') {
						if ($row) {
							$data[$k] = array_get_default($formatted, $k, '');
						} else {
							$data[$k] = array(
								translate($f['vname'], 'ProductCatalog'),
								translate($f['vname'], 'ProductCatalog'),
							);
						}
				}
			}
		}
	}

	public static function create($type)
	{
		return new QuotePDFTemplateProvider($type);
	}

	public function initialize($layout, ListQuery $lq)
	{
		$lq->addField('assigned_user.name', 'assigned_user_name');
		$lq->addField('assigned_user.phone_work', 'assigned_user_phone');
		$lq->addField('assigned_user.email1', 'assigned_user_email');
		$lq->addField('billing_contact.phone_work', 'billing_contact_phone');
		$lq->addField('billing_contact.email1', 'billing_contact_email');
		$lq->addField('shipping_contact.phone_work', 'shipping_contact_phone');
		$lq->addField('shipping_contact.email1', 'shipping_contact_email');
		$lq->addField('billing_account.invoice_as', 'invoice_as');
		$fs = array_merge(
			$lq->base_model->getColumnFields(),
			$lq->base_model->getCustomFields()
		);


		$fs = $lq->base_model->getFieldDefinitions();
		foreach ($fs as $f) {
			if (isset($f['vname']) && in_array($f['type'], self::$field_types)) {
				$lq->addField($f['name'], '_' . $f['name'], false, false, true);
			}
		}
		parent::initialize($layout, $lq);

	}


	public function begin()
	{
	}

	public function iterate(PDFTemplatePrinter $p)
	{
		$this->iterateHeaderFooter($p, 'header');
		$this->iterateGroups($p);
		$this->iterateTotals($p);
		$this->iterateExtraFooter($p);
		$this->iterateHeaderFooter($p, 'footer');
	}

	protected function iterateGroups(PDFTemplatePrinter $p)
	{
		$n = is_countable($this->data['group_items']) ? count($this->data['group_items']) : 0;
		$header = $this->getData('group_header');
		$items = $this->getData('group_items');
		$totals = $this->getData('group_totals');
		foreach ($items as $i => $item) {
			$p->renderSection($this->findSection('group_header'), $header[$i]);
			$this->iterateGroupItems($p, $item);
			if ($n > 1) {
				$this->iterateGroupTotals($p, $totals[$i]);
			}
			$p->renderSection($this->findSection('group_footer'), $header[$i]);
		}
	}

	protected function iterateTotals(PDFTemplatePrinter $p)
	{
		$totals = $this->getData('quote_totals');
		if(! is_countable($totals) || ! count($totals)) return;
		$h = 0;
		$head = $this->findSection('quote_totals_header');
		if($head) {
			$p->adjustSectionHeight($head, array());
			$h += $head['height'];
		}
		$section = $this->findSection('quote_totals');
		$p->adjustSectionHeight($section, $totals[0]);
		$h += count($totals) * $section['height'];

		$p->testWrapSection($h);
		$p->renderSection($head, array());
		foreach ($totals as $data) {
			$p->renderSection($section, $data);
		}
	}

	protected function iterateGroupItems(PDFTemplatePrinter $p, $data)
	{
		$section = $this->findSection('group_items');
		$add_w = 0;
		if(!$this->row->getField('show_list_prices')) {
			foreach ($section['columns'] as $i => $col) {
				if ($col['field'] == 'listprice') {
					$add_w = array_get_default($section['columns'][$i], 'width');
					unset($section['columns'][$i]);
				}
			}
			if($add_w) {
				foreach ($section['columns'] as $i => $col) {
					if ($col['field'] == 'product' && isset($col['width'])) {
						$section['columns'][$i]['width'] += $add_w;
					}
				}
			}
		}
		$p->renderSection($section, $data);
	}

	protected function iterateGroupTotals(PDFTemplatePrinter $p, $totals)
	{
		if(! is_countable($totals) || ! count($totals)) return;
		$h = 0;
		$head = $this->findSection('group_totals_header');
		if($head) {
			$p->adjustSectionHeight($head, array());
			$h += $head['height'];
		}
		$section = $this->findSection('group_totals');
		$p->adjustSectionHeight($section, $totals[0]);
		$h += count($totals) * $section['height'];

		$p->testWrapSection($h);
		$p->renderSection($head, array());
		foreach ($totals as $data) {
			$p->renderSection($section, $data);
		}
	}

	public function end()
	{
	}

	protected function iterateHeaderFooter(PDFTemplatePrinter $p, $name)
	{
		$p->renderSection($this->findSection($name), $this->getData('header'));
	}

	protected function _address($pfx, $row = null)
	{
		$focus = $row ? $row : $this->row;

		$contact_position = 'first';

		if($pfx == 'billing_c') {
			$contact_position = 'last';
			$pfx = 'billing_';
		}

		if($pfx == 'shipping_c') {
			$contact_position = 'last';
			$pfx = 'shipping_';
		}

		$fid = "{$pfx}contact_id";
		$fnm = "{$pfx}contact_name";
		if(strlen($focus->getField($fid) ?? '') && !strlen($focus->getField($fnm) ?? '')) {
			if( ($rec = ListQuery::quick_fetch('Contact', $focus->getField($fid), array('_display'))) ) {
				$focus->assign($fnm, $rec->getField('_display'));
			}
		}
		$fid = "{$pfx}account_id";
		$fnm = "{$pfx}account_name";
		if(strlen($focus->getField($fid) ?? '') && !strlen($focus->getField($fnm) ?? '')) {
			if( ($rec = ListQuery::quick_fetch('Account', $focus->getField($fid), array('_display', 'invoice_as'))) ) {
				$nm = trim($rec->getField('invoice_as') ?? '');
				$focus->assign($fnm, strlen($nm) ? $nm : $rec->getField('_display'));
			}
		}

		global $locale;
		$ret = $locale->getLocaleBeanFormattedAddress($focus, $pfx, false);
		if($contact_position == 'last') {
			$ret = str_replace($focus->row['_billing_contact'], '', $ret);
			$ret .= "\n".$focus->row['_billing_contact'];
		}
		$phone = $focus->getField("{$pfx}contact_phone");
		$email = $focus->getField("{$pfx}contact_email");
		if($phone && AppConfig::setting('company.quote_pdf_show_contact_phone'))
			$ret .= "\n$phone";
		if($email && AppConfig::setting('company.quote_pdf_show_contact_email'))
			$ret .= "\n$email";
		return trim($ret);
	}

	protected function &get_shipping_provider()
	{
		static $shippers = [];
		$sid = $this->row->getField('shipping_provider_id');
		if(! $sid) $sid = '';
		if(! isset($shippers[$sid])) {
			require_once('modules/ShippingProviders/ShippingProvider.php');
			$shipper = new ShippingProvider();
			if($sid)
				$shipper->retrieve($sid, false);
			$shippers[$sid] = $shipper;
		}
		return $shippers[$sid];
    }

	protected function get_group_totals(&$grp, $final=false) {
		$mod_strings =& $this->mod_strings();
		$currency =& $this->get_currency();
		$shipping_provider =& $this->get_shipping_provider();
		$discounts = array();
		$taxes = array();
		$sep = translate('LBL_SEPARATOR', 'app');
		$shipping_text = $mod_strings['LBL_SHIPPING'];

		$shipping_rows = array();
		$taxed_shipping_value_row = null;
		$shipping_value_row = null;

		if($final) {
			$taxed_shipping_rows = array();
			$taxed_shipping_value_row = array(
				'total_name' => $shipping_text ." (".$mod_strings['LBL_SHIPPING_IS_TAXED'].")" . $sep,
				'total_value' => '',
			);
			$taxed_shipping_rows[] =& $taxed_shipping_value_row;
			if ($this->shipping > 0 || $this->taxedShipping == 0) {
				$shipping_rows[] = array(
					'total_name' => $mod_strings['LBL_SHIPPING_PROVIDER'] . $sep,
					'total_value' => $shipping_provider->name,
				);
			} else {
				$taxed_shipping_rows[] = array(
					'total_name' => $mod_strings['LBL_SHIPPING_PROVIDER'] . $sep,
					'total_value' => $shipping_provider->name,
				);
			}
		}

		$shipping_value_row = array(
			'total_name' => $shipping_text ." (".$mod_strings['LBL_SHIPPING_IS_NOT_TAXED'].")" . $sep,
			'total_value' => '',
		);
		$shipping_rows[] =& $shipping_value_row;

		$pretax = $grp['subtotal'];
		$shipping_taxed = false;
		$combine_tax = AppConfig::setting('company.quote_pdf_combine_taxes') && is_countable($this->groups) && (count($this->groups)  > 1);

		if($grp['id'] == 'GRANDTOTAL') {
			if($grp['discount'] != 0) {
				$discounts[] = array(
					'total_name' => $mod_strings['LBL_DISCOUNT'] . $sep,
					'total_value' => array($this->currency_format(-$grp['discount'], $currency->id), 'currency'),
				);
				$pretax -= $grp['discount'];
			}
			if($combine_tax) {
				if($grp['tax'] != 0)
					$taxes[] = array(
						'total_name' => $mod_strings['LBL_TAX'] . $sep,
						'total_value' => array($this->currency_format($grp['tax'], $currency->id), 'currency'),
					);
			} else if(!empty($grp['taxes'])) {
				foreach ($grp['taxes'] as $t) {
					$taxes[] = array(
						'total_name' => $t['name'] . ' (' . format_number($t['rate'], -1) . '%)' . $sep,
						'total_value' => array($this->currency_format($t['amount'], $currency->id), 'currency'),
					);
				}
			}
			if($grp['shipping'] != 0)
				$shipping_value_row['total_value'] = array($this->currency_format($grp['shipping'], $currency->id), 'currency');
			$shipping_taxed = $grp['shipping_taxed'];
			if($shipping_taxed || $this->taxedShipping != 0) {
				if ($this->taxedShipping <= 0) {
					$pretax += $grp['shipping'];
				} else {
					$pretax += $this->taxedShipping;
				}
			}
		}
		else if(! empty($grp['adjusts'])) {
			foreach($grp['adjusts'] as $idx => $adj) {
				if (!empty($adj['line_id'])) {
					if ($adj['type'] != 'StandardTax' && $adj['type'] != 'CompoundedTax')
						continue;
				}
				$row = array();
				$row['total_name'] = $adj['name'];
				if($adj['type'] != 'StdFixedDiscount')
					$row['total_name'] .= ' (' . format_number($adj['rate'], -1) . '%)';
				if($adj['type'] == 'StandardDiscount' || $adj['type'] == 'StdPercentDiscount' || $adj['type'] == 'StdFixedDiscount') {
					$pretax -= $adj['amount'];
					$adj['amount'] = -$adj['amount'];
				}
				$row['total_name'] .= $sep;
				$val = $this->currency_format($adj['amount'], $currency->id);
				if($adj['type'] == 'StandardDiscount' || $adj['type'] == 'StdPercentDiscount' || $adj['type'] == 'StdFixedDiscount') {
					$row['total_value'] = $val;
					$row['total_raw'] = $adj['amount'];
					$row['discount_id'] = $adj['related_id'];
					$discounts[] = $row;
				}
				else if($adj['type'] == 'StandardTax' || $adj['type'] == 'CompoundedTax') {
					$row['total_value'] = array($val, 'currency');
					$taxes[] = $row;
				}
				else if($adj['type'] == 'TaxedShipping' || $adj['type'] == 'UntaxedShipping') {
					if($adj['amount'] != 0) {
						$shipping_value_row['total_value'] = array($val, 'currency');
					}
					$shipping_taxed = ($adj['type'] == 'TaxedShipping');
					if($shipping_taxed) {
						$shipping_value_row['total_name'] = $shipping_text ." (".$mod_strings['LBL_SHIPPING_IS_TAXED'].")" . $sep;
						$pretax += $adj['amount'];
						$this->taxedShipping += $adj['amount'];
					} else {
						$this->shipping += $adj['amount'];
					}
				}
			}
		}
		$data = array();
		if($final) {
			$data[] = array(
				'total_name' => $mod_strings['LBL_CURRENCY'] . $sep,
				'total_value' => $currency->name,
			);
		}
		$data[] = array(
			'total_name' => $mod_strings['LBL_SUBTOTAL'] . $sep,
			'total_value' => array($this->currency_format($grp['subtotal'], $currency->id), 'currency'),
		);
		if($grp['id'] != 'GRANDTOTAL') {
			$data = array_merge($data, $discounts);
		} else {
			$data[] = 'DISCOUNT_PLACEHOLDER';
		}
		$grp['shipping_taxed'] = $shipping_taxed; // used by caller

		if($grp['id'] == 'GRANDTOTAL') {
			if($shipping_taxed || $this->taxedShipping != 0) {
				if ($this->taxedShipping != 0) {
					if ($taxed_shipping_value_row)
					$taxed_shipping_value_row['total_value'] = array($this->currency_format($this->taxedShipping, $currency->id), 'currency');
					$data = array_merge($data, $taxed_shipping_rows);
				}
			}
		} else {
			if ($shipping_value_row['total_value'] && $shipping_taxed)
				$data = array_merge($data, $shipping_rows);
		}

		if(count($taxes) && $pretax != $grp['subtotal']) {
			$data[] = array(
				'total_name' => $mod_strings['LBL_PRETAX_TOTAL'] . $sep,
				'total_value' => array($this->currency_format($pretax, $currency->id), 'currency'),
			);
		}
		$data = array_merge($data, $taxes);

		if($grp['id'] == 'GRANDTOTAL') {

			$data = array_merge($data, $this->getMilestoneTotals('tax', $grp));

			if(!$shipping_taxed || $this->shipping != 0) {
				if ($this->shipping != 0) {
					if ($shipping_value_row)
					$shipping_value_row['total_value'] = array($this->currency_format($this->shipping, $currency->id), 'currency');
					$data = array_merge($data, $shipping_rows);
				}
			}

			$data = array_merge($data, $this->getMilestoneTotals('shipping', $grp));
			$data = array_merge($data, $this->getMilestoneTotals('total', $grp));

		} else {
			if ($shipping_value_row['total_value'] && !$shipping_taxed)
				$data = array_merge($data, $shipping_rows);
		}

		$data[] = array(
			'total_name' => $mod_strings['LBL_TOTAL'] . $sep,
			'total_value' => array($this->currency_format($grp['total'], $currency->id), 'currency'),
		);

		return array($data, $discounts);
    }

    private function getMilestoneTotals($type, $totals) {
	    $mod_strings = $this->mod_strings();
	    $currency = $this->get_currency();
	    $sep = translate('LBL_SEPARATOR', 'app');
	    $total_params = FinalMilestone::get_grand_totals();
		$result = [];

		if (isset($total_params[$type])) {
			foreach ($total_params[$type] as $param => $label) {
				$val = isset($totals[$param]) ? $totals[$param] : 0;

				if ($val == 0)
					continue;

				$result[] = [
					'total_name' => $mod_strings[$label] . $sep,
					'total_value' => array($this->currency_format($totals[$param], $currency->id), 'currency'),
				];
			}
		}

		return $result;
    }

	protected function &mod_strings() {
		$lang_module = $this->row->getModuleDir();
		if (!empty($this->alt_lang_module))
			$lang_module = $this->alt_lang_module;
		$strings = return_module_language($this->use_language, $lang_module);
		return $strings;
	}

    function &get_currency() {
		static $currencies = [];
		$cid = $this->row->getField('currency_id');
		if(! $cid) $cid = '-99';
		if(! isset($currencies[$cid])) {
			require_once('modules/Currencies/Currency.php');
			$currency = new Currency();
			$currency->retrieve($cid, false);
			$currencies[$cid] = $currency;
		}
		return $currencies[$cid];
    }

	function currency_format($amount, $currency_id = false, $no_round = false)
	{
		$params = array('currency_symbol' => false, 'type' => 'pdf');
		if ($currency_id) {
			$params['currency_symbol'] = true;
			$params['currency_id'] = $currency_id;
		}
		if($no_round)
			$params['round'] = null;
		return currency_format_number($amount, $params);
	}

	protected function get_group_lines(&$grp) {
		$focus =& $this->row;
		$currency =& $this->get_currency();
		$component_num = 1;
		$data = array();
		$num = 1;
		$attribs_by_line = array();
		if(! empty($grp['adjusts'])) {
			foreach ($grp['adjusts'] as $adj) {
				if(! empty($adj['line_id'])) {
					if($adj['type'] == 'ProductAttributes')
						$attribs_by_line[$adj['line_id']][] = $adj;
				}
			}
		}
		if($grp['lines']) foreach($grp['lines'] as $row) {
			$attrib_amount = 0.0;
			if(! empty($attribs_by_line[$row['id']])) {
				foreach($attribs_by_line[$row['id']] as $adj) {
					if($adj['amount']) {
						$attrib_amount += $adj['amount'];
					}
				}
			}
			if(! empty($row['is_comment'])) {
				$data[] = array(
					'product' => $row['body'],
				);
			}
			else {
				$datarow = array(
					'depth' => $row['depth'],
					'quantity' => format_number($row['quantity'], -1),
					'product' => $row['name'],
					'orig_product' => $row['name'],
					'mfr_part_no' => $row['mfr_part_no'],
					'vendor_part_no' => array_get_default($row, 'vendor_part_no', ''),
					'serial_no' => array_get_default($row, 'serial_no', ''),
				);
				$sec = $this->findSection('group_items');
				$serials_present = false;
				foreach ($sec['columns'] as $col) {
					if ($col['field'] == 'serial_no') {
						$serials_present = true;
						break;
					}
				}
				if (!$serials_present) {
					if (strlen($datarow['serial_no'])) {
						$datarow['product'] .= "\n\n" . $datarow['serial_no'];
						$datarow['serial_no'] = '';
					}
				}
				if (!empty($row['description'])) {
					$datarow['product'] .= ' : ' . $row['description'];
				}
				$show_components = $focus->getField('show_components');
				if($row['depth'] && ! $show_components)
					continue;
				if(! $row['depth'] || $show_components != 'names') {
					if (isset($row['list_price'])) {
						$datarow['listprice'] = $this->currency_format($row['list_price'], $currency->id, true);
					}
					$unitprice = $row['unit_price'] - $attrib_amount;
					$datarow['unitprice'] = $this->currency_format($unitprice, $currency->id, true);
					$costprice = $row['cost_price'] ?? 0;
					$datarow['costprice'] = $this->currency_format($costprice, $currency->id, true);
					$ext_price = $row['ext_price'] - $row['ext_quantity'] * $attrib_amount;
					$datarow['extprice'] = $this->currency_format($ext_price, $currency->id);
				}
				if($row['depth'])
					$datarow['number'] = '  ' . $component_num ++;
				else {
					$datarow['number'] = $num ++;
					$component_num = 1;
				}
				$datarow['discount'] = '';
				if(! empty($row['pricing_adjust_id'])) {
					$prc_id = $row['pricing_adjust_id'];
					if(! empty($grp['adjusts'][$prc_id])) {
						$prc_row = $grp['adjusts'][$prc_id];
						if($prc_row['type'] == 'PercentDiscount' || $prc_row['type'] == 'StdPercentDiscount')
							$datarow['discount'] = $prc_row['rate'] . '%';
						else if($prc_row['type'] == 'StdFixedDiscount' && $prc_row['amount'])
							$datarow['discount'] = $this->currency_format($prc_row['amount'], $currency->id, true);
					}
				}

				self::injectProductSampleData($datarow, $row);
				$data[] = $datarow;

				if(! empty($attribs_by_line[$row['id']])) {
					foreach($attribs_by_line[$row['id']] as $adj) {
						if (1 || $adj['amount']) {
							$datarow = array(
								'product' => '    ' . $adj['name'],
								'unitprice' => $show_components == 'names' ? '' : $this->currency_format($adj['amount'], $currency->id, true),
								'extprice' => $show_components == 'names' ? '' : $this->currency_format($adj['amount'] * $row['ext_quantity'], $currency->id),
							);
							$data[] = $datarow;
						}
					}
				}
			}
		}
		return $data;
	}

	protected function iterateExtraFooter(PDFTemplatePrinter $p)
	{
		$header = $this->getData('header');
		$section = $this->findSection('notes');
		if($section && ! empty($header['description']))
			$p->renderSection($section, $header);

		$section = $this->findSection('terms');
		if($section && ! empty($header['standard_terms']))
			$p->renderSection($section, $header);

		$section = $this->findSection('quote_accept');
		if($section && AppConfig::setting('company.quote_pdf_acceptance'))
			$p->renderSection($section, $header, array('no_split' => true));
	}

	public function getFileName()
	{
		$filename_pfx = translate('LBL_PDF_PROPOSAL', 'Quotes');
		$filename = $filename_pfx . '_' . $this->row->getField('prefix');
		$nm = trim($this->row->getField('invoice_as') ?? '');
		$nm = strlen($nm ?? '') ? $nm : ($this->row->getField('billing_account_name') ?? '');
		$filename .= $this->row->getField('quote_number') . '_' . $nm;
		$filename = preg_replace('~\.+$~', '', trim($filename));
		$filename = $filename . '.pdf';
		return $filename;
	}

	public function getRow()
	{
		return $this->row;
	}

	public function getNoteTitle()
	{
		return translate('LBL_PREPARED_QUOTE', 'Quotes');
	}

	public function getEmailTitle()
	{
		return translate('LBL_EMAILED_QUOTE', 'Quotes');
	}

	public function getSignedDocumentTitle() {
		$prefix =
			$this->row
			? '[' . $this->row->getField('prefix') . $this->row->getField('quote_number') . '] '
			: '';

		return $prefix . $this->getNoteTitle();
	}

	public function getData($section = null)
	{
		if (empty($this->data))
			$this->createData();
		if (!$section)
			return $this->data;
		return array_get_default($this->data, $section, array());
	}

	protected function createData()
	{
		global $timedate;
		$this->data = array();

		$row =& $this->row;
		$terms = AppConfig::setting("lang.lists.current.app.terms_dom", array());
		$tax_information = $this->row->getField('tax_information');
		$datefmt = AppConfig::setting('locale.defaults.date_format');
		$data = array(
			'todays_date' => date($datefmt),
			'number' => $row->getField('prefix') . $row->getField('quote_number'),
			'terms' => array_get_default($terms, $row->getField('terms'), $row->getField('terms')),
			'date' => $timedate->to_display_date($row->getField('date_entered')),
			'sales_person' => $row->getField('assigned_user_name'),
			'sales_person_email' => $row->getField('assigned_user_email'),
			'sales_person_phone' => $row->getField('assigned_user_phone'),
			'valid_until' => $row->getField('valid_until', '', true),
			'tax_info' => from_html(AppConfig::setting('company.tax_information', '')),
			'billing_address' => $this->_address('billing_') . "\n" . $tax_information,
			'billing_address_contact_last' => $this->_address('billing_c') . "\n" . $tax_information,
			'shipping_address' => $this->_address('shipping_'),
			'shipping_address_contact_last' => $this->_address('shipping_c'),
			'description' => $row->getField('description'),
			'standard_terms' => trim(AppConfig::setting('company.std_quote_terms', '')),
		);
		$data = array_merge($this->header_data, $data);
		$this->data['header'] = $data;

		$this->createGroupData();

	}

	protected function createGroupData()
	{
		$this->data['group_header'] = array();
		$this->data['group_items'] = array();
		$this->data['group_totals'] = array();
		$discounts = array();

		$milestone_invoices_totals = [
			'total' => 0,
			'tax' => 0,
			'shipping' => 0
		];

		foreach ($this->groups as $grp) {
			$this->data['group_header'][] = array(
				'group_name' => $grp['name'],
				'status' => translate('quote_stage_dom', '', $grp['status'], true),
			);
			$this->data['group_items'][] = $this->get_group_lines($grp);
			list($totals, $group_discounts) = $this->get_group_totals($grp, false);
			$discounts = array_merge($discounts, $group_discounts);
			$this->data['group_totals'][] = $totals;

			if ($this->isMilestoneInvoicesGroup($grp))
				$this->updateMilestoneInvoicesTotals($grp, $milestone_invoices_totals);
		}

		$this->addMilestonesToGrandTotal($milestone_invoices_totals);
		list($totals) = $this->get_group_totals($this->gtotal, true);

		$pre_discount = $post_discount = array();
		$post = false;

		foreach ($totals as $t ) {
			if ($t === 'DISCOUNT_PLACEHOLDER') {
				$post = true;
			} else if ($post) {
				$post_discount[] = $t;
			} else {
				$pre_discount[] = $t;
			}
		}
		$g_discounts = array();
		$combine_disc = AppConfig::setting('company.quote_pdf_combine_discounts') && is_countable($this->groups) && (count($this->groups) > 1);
		$total_disc = 0.0;
		foreach ($discounts as $d) {
			if($combine_disc) {
				$total_disc += $d['total_raw'];
				continue;
			}
			if (!isset($g_discounts[$d['discount_id']])) {
				$g_discounts[$d['discount_id']] = array(
					'total_name' => $d['total_name'],
					'total_raw' => $d['total_raw'],
				);
			} else {
				$g_discounts[$d['discount_id']]['total_raw'] += $d['total_raw'];
			}
		}

		$currency =& $this->get_currency();
		if($combine_disc) {
			if($total_disc != 0)
				$pre_discount[] = array(
					'total_name' => translate('LBL_DISCOUNT', 'Quotes') . translate('LBL_SEPARATOR', 'app'),
					'total_value' => array($this->currency_format($total_disc, $currency->id), 'currency'),
				);
		} else {
			foreach ($g_discounts as $d) {
				$d['total_value'] = array($this->currency_format($d['total_raw'], $currency->id), 'currency');
				$pre_discount[] = $d;
			}
		}
		$this->data['quote_totals'] = array_merge($pre_discount, $post_discount);
	}

	private function isMilestoneInvoicesGroup($group) {
		return FinalMilestone::is_milestone_invoices_group($group);
	}

	private function updateMilestoneInvoicesTotals($group, &$main_totals) {
		$totals = FinalMilestone::calculate_invoices_group_totals($group);

		foreach ($main_totals as $param => $value) {
			if (isset($totals[$param]))
				$main_totals[$param] = $value + $totals[$param];
		}
	}

	private function addMilestonesToGrandTotal($milestones_totals) {

		foreach($milestones_totals as $param => $value) {
			if ($value != 0) {
				$this->gtotal['milestone_' . $param] = $value;

				if ($param == 'tax') {
					$this->gtotal['milestone_tax_remaining'] = $this->gtotal['tax'] - ($milestones_totals['tax'] * -1);
				} elseif ($param == 'shipping') {
					$this->gtotal['milestone_shipping_remaining'] = $this->gtotal['shipping'] - ($milestones_totals['shipping'] * -1);
				}
			}
		}
	}

	public static function get_extra_sources() {
		return array(
			array(
				'module' => 'Opportunities',
				'model' => 'Opportunity',
				'prefix' => 'opportunity',
				'label' => 'LBL_OPPORTUNITY',
			),
			array(
				'module' => 'SalesOrders',
				'model' => 'SalesOrder',
				'prefix' => 'sales_order',
				'label' => 'LBL_SALES_ORDER',
			),
			array(
				'module' => 'Accounts',
				'model' => 'Account',
				'prefix' => 'billing_account',
				'label' => 'LBL_BILLING_ACCOUNT',
			),
			array(
				'module' => 'Accounts',
				'model' => 'Account',
				'prefix' => 'shipping_account',
				'label' => 'LBL_SHIPPING_ACCOUNT',
			),
			array(
				'module' => 'Contacts',
				'model' => 'Contact',
				'prefix' => 'billing_contact',
				'label' => 'LBL_BILLING_CONTACT',
			),
			array(
				'module' => 'Contacts',
				'model' => 'Contact',
				'prefix' => 'shipping_contact',
				'label' => 'LBL_SHIPPING_CONTACT',
			),
			array(
				'module' => 'ShippingProviders',
				'model' => 'ShippingProvider',
				'prefix' => 'shipping_provider',
				'label' => 'LBL_SHIPPING_PROVIDER',
			),
		);
	}

}

