<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


require_once('data/SugarBean.php');
require_once('modules/TaxCodes/TaxCode.php');

class Quote extends SugarBean {
	// Stored fields
	var $id;
	var $quote_number;
	var $prefix;
	var $date_entered;
	var $date_modified;
	var $modified_user_id, $modified_by_name;
	var $created_by, $created_by_name;
	var $assigned_user_id, $assigned_user_name;
	//
	var $name;
	var $quote_stage;
	var $purchase_order_num;
	var $description;
	var $valid_until;
	var $terms;
	var $approval;
	var $approval_problem;
	var $approved_user_id, $approved_user_name;
	//
	var $opportunity_id, $opportunity_name;
	// These are related
	var $billing_account_name, $billing_account_id;
	var $billing_address_street, $billing_address_city, $billing_address_state, $billing_address_postalcode, $billing_address_country;
	// Used by quick-creation form
	var $account_id;
	var $note_id;
	//
	var $shipping_account_name, $shipping_account_id;
	var $shipping_address_street, $shipping_address_city, $shipping_address_state, $shipping_address_postalcode, $shipping_address_country;
	//
	var $billing_contact_id, $billing_contact_name;
	var $shipping_contact_id, $shipping_contact_name;
	var $currency_id, $exchange_rate;
	var $shipping_provider_id;
	var $tax_information;
	var $sales_order_id;
	var $tax_exempt;
	var $discount_before_taxes;

	// line items
	var $line_items = array();
	var $grand_total;

	var $amount, $amount_usdollar;
	var $cost;
	var $markup, $margin;
	var $show_list_prices;
	var $show_components;
	var $budgetary_quote;
	var $gross_profit, $gross_profit_usdollar;
	// ...

    //
	var $table_name = "quotes";
	var $object_name = "Quote";
	var $group_object_name = "QuoteLineGroup";
	var $module_dir = "Quotes";
	var $new_schema = true;
	//
	var $account_table = "accounts";
	var $contact_table = "contacts";
	var $opportunity_table = "opportunities";


	var $additional_column_fields = Array(
		'assigned_user_name',
		'modified_by_name',
		'created_by_name',
		'approved_user_name',
		'account_id',
		'opportunity_name',
		'billing_account_name',
		'shipping_account_name',
		'billing_contact_name',
		'shipping_contact_name',
		'sales_order_name',
	);

	var $relationship_fields = array(
		'note_id' => 'notes',
	);

	static $grp_idx = 0;


	function getCurrentPrefix()
	{
		return AppConfig::get_sequence_prefix('quotes_prefix');
	}

	function getCurrentSequenceValue()
	{
		return AppConfig::current_sequence_value('quotes_number_sequence');
	}

	function getNextSequenceValue()
	{
		return AppConfig::next_sequence_value('quotes_number_sequence');
	}


	function &get_line_groups($set_old_items=false, $reload=false) {
		if($reload || ! isset($this->line_groups) || ! is_array($this->line_groups)) {
			$lgm =& $this->get_line_group_manager();
			$encode = empty($this->pdf_output_mode);
			$this->line_groups =& $lgm->retrieve_all($encode);
			if($set_old_items) {
				$items =& $lgm->lineItemsFromGroups($this->line_groups);
				if(is_array($items))
					$this->line_items = $items;
			}
			$lgm->cleanup();
		}
		return $this->line_groups;
	}

	function cleanup_line_groups(&$groups) {
		$this->cleanup_list($groups);
	}

	function &get_line_group_manager() {
		require_bean('QuoteLineGroup');
		$ret = QuoteLineGroup::newForParent($this);
		return $ret;
	}

	function line_items_editable() {
		return true;
	}

	function save($check_notify = FALSE)
	{
		// need an ID to save line items
		if(empty($this->id)) {
			$this->id = create_guid();
			$this->new_with_id = true;
		}

		// must do this before manipulating numeric fields
		$this->unformat_all_fields();

		$this->save_line_groups();

		return parent::save($check_notify);
	}


	function save_line_groups() {
		if(isset($this->line_groups) && is_array($this->line_groups)) {
			foreach(array_keys($this->line_groups) as $k)
				$this->line_groups[$k]->save();

			$gt =& $this->line_groups['GRANDTOTAL'];
			$this->amount = $gt->total;
			$this->cost = $gt->cost;

			$cost = $gt->cost;
			$std = $gt->subtotal - $gt->discount;
			$this->gross_profit = $gp = $std - $cost;
			if($cost != 0)
				$this->markup = number_format($gp * 100.0 / $cost, 2);
			else
				$this->markup = 0;
			if($std != 0)
				$this->margin = number_format($gp * 100.0 / $std, 2);
			else
				$this->margin = 0;
		}
	}

	function isSubmitted()
	{
		return 'Submitted' == $this->approval;
	}

	static function useApprovals()
	{
		return !! AppConfig::setting('company.approve_quotes');
	}

	static function approvalEditable(&$row, &$flag, &$val) {
		$val = canApproveUser($row->getModuleDir(), AppConfig::current_user_id(), $row->getField('assigned_user_id'), true);
	}

	static function approvalCautionHidden(&$row, &$flag, &$val) {
		$val = !self::meetsApprovalCriteria($row);
		if (!$val) {
			self:: approvalEditable($row, $flag, $val);
			$val = !$val;
		}
	}

	static function approvalHidden(&$row, &$flag, &$val)
	{
		$val = !self::useApprovals();
		if (!$val) {
			$val = !self::meetsApprovalCriteria($row);
		}
	}

	static function needsApproval(&$row, &$flag, &$val)
	{
		self::submitHidden($row, $flag, $val);
		$val = !$val;
	}


	static function printHidden(&$row, &$flag, &$val)
	{
		$val = false;
		if(! self::useApprovals()) {
			return;
		}
		if (canApproveUser($row->getModuleDir(), AppConfig::current_user_id(), $row->getField('assigned_user_id'), true)) {
			return;
		}
		$threshold = (int)AppConfig::setting('company.quote_threshold');
		$margin = (int)AppConfig::setting('company.margin_threshold');
		$approval = $row->getField('approval');
		$val = (
				(($row->getField('amount_usdollar') >= $threshold)
				|| ($row->getField('margin') <= $margin))
				&& 'Approved' != $approval
			);
	}

	static function submitHidden(&$row, &$flag, &$val)
	{
		$val = false;
		// set $val to true to hide Submit button
		if(! self::useApprovals()) {
			$val = true;
			return;
		}
		if (canApproveUser($row->getModuleDir(), AppConfig::current_user_id(), $row->getField('assigned_user_id'), true)) {
			$val = true;
			return;
		}
		$approval = $row->getField('approval');
		$val = !(
				self::meetsApprovalCriteria($row)
				&& 'Approved' != $approval
				&& 'Submitted' != $approval
			);
	}

	static function meetsApprovalCriteria($row) {
		if(! self::useApprovals()) {
			return false;
		}
		$threshold = (int)AppConfig::setting('company.quote_threshold');
		$margin = (int)AppConfig::setting('company.margin_threshold');
		return (
				($row->getField('amount_usdollar') >= $threshold)
				|| ($row->getField('margin') <= $margin)
			);
	}

	function get_view_closed_where_basic($param)
	{
		return $param['value'] ? '1' : "substring(quotes.quote_stage, 1, 7) != 'Closed '";
	}

	function getDefaultListWhereClause()
	{
		return "substring(quotes.quote_stage, 1, 7) != 'Closed '";
	}


	function get_view_closed_where_advanced()
	{
		return '1';
	}

	function get_search_stage_options()
	{
		global $app_list_strings, $mod_strings;
		return array_merge(array(''=>'', "Active" => $mod_strings['LBL_ACTIVE']), $app_list_strings['quote_stage_dom']);
	}


	function get_stage_where_advanced($param)
	{
		return ($param['value'] == 'Active') ? "substring(quotes.quote_stage, 1, 7) != 'Closed '" : ("quotes.quote_stage='" . PearDatabase::quote($param['value']) . "'");
	}

	function get_stage_where_basic($param)
	{
		return '1';
	}

	function search_by_number($param) {
		if (empty($param['value'])) {
			return 1;
		} else {
			return "(quotes.quote_number = '" . PearDatabase::quote($param['value']) . "' OR CONCAT(quotes.prefix, quotes.quote_number) = '" . PearDatabase::quote($param['value']) . "' ) ";
		}
	}

	function cleanup() {
		if(isset($this->line_groups)) {
			$this->cleanup_line_groups($this->line_groups);
			unset($this->line_groups);
		}
		parent::cleanup();
	}

    static function init_from_account($account_id) {

		$lq = new ListQuery('Account');
		$lq->addColumnFields();
		$lq->addField('default_pricebook.editable');
		$result = $lq->queryRecord($account_id);
		if ($result && !$result->failed)
			$account = $result->row;
		else
			$account = null;
        $data = array();

		if ($account != null) {
			init_quote_from_account($data, $account);
        }

        return $data;
    }

	static function show_sales_popup(DetailManager $mgr) {
        require_bean('Account');
        Account::add_account_popup($mgr->getRecord(), 'billing_account_id', 'sales');
	}

    static function init_record(RowUpdate &$upd, $input) {
        $update = array();

        if (!empty($input['opportunity_id'])) {
            $opportunity = ListQuery::quick_fetch_row('Opportunity', $input['opportunity_id']);
            if ($opportunity != null) {
                $update['opportunity_id'] = $opportunity['id'];
                $update['currency_id'] = $opportunity['currency_id'];
                $update['exchange_rate'] = $opportunity['exchange_rate'];
                $update['partner_id'] = $opportunity['partner_id'];
                $account_id = $opportunity['account_id'];
                if (!empty($input['account_id']))
                    $account_id = $input['account_id'];

                $account = self::init_from_account($account_id);
                array_extend($update, $account);
				copy_custom_fields($opportunity['id'], $upd, 'Opportunity');

				self::update_groups($upd, $opportunity['id'], 'Opportunity');
            }
        } elseif(!empty($input['billing_account_id'])) {
            $update = self::init_from_account($input['billing_account_id']);
        }

		if($upd->duplicate_of_id) {
			$upd->set('quote_stage', $upd->getFieldDefaultValue('quote_stage'));
			$upd->set('sales_order_id', null);
			$upd->set('valid_until', null);
			$currency = AppConfig::db_row('Currency', $upd->getField('currency_id'));
			$upd->set('exchange_rate', $currency['conversion_rate']);
		}

		if (empty($update['terms']))
            $update['terms'] = AppConfig::setting('company.invoice_default_terms', 'COD');

        $update['show_list_prices'] = AppConfig::setting('company.quotes_show_list') ? '1' : '0';
        $update['show_components'] = AppConfig::setting('company.quote_default_components', 'all');
        $default_valid_days = '30';
        if(AppConfig::setting('company.quote_valid_days') != '')
        	$default_valid_days = AppConfig::setting('company.quote_valid_days');
        $update['valid_until'] = date('Y-m-d', strtotime('+'.$default_valid_days.' day'));

		foreach ($update as $k => $v)
			if ($upd->isFieldEmpty($k) || $k == 'currency_id')
		        $upd->set($k, $v);
    }

    static function update_groups(TallyUpdate &$quote_upd, $from_id, $from_model) {
        $prodcuts = array();
        if ($from_model == 'Opportunity')
            $prodcuts = self::get_service_parts_group($from_id);

        $groups = $prodcuts;
        if (sizeof($groups) > 0)
            $quote_upd->replaceGroups($groups);
    }

    static function get_service_parts_group($opp_id) {
        $rev = new ListQuery('RevenueForecast', true);
		$rev->addSimpleFilter('opportunity_id', $opp_id);
		$result = $rev->runQuery();
        $new_groups = array();

        if($result->total_count > 0) {
            $grp_id = 'temp~'.(self::$grp_idx ++);
            $new_groups = array(
                $grp_id => array(
                    'id' => $grp_id,
                    'status' => 'Draft',
                    'lines' => array(),
                    'lines_order' => array(),
                    'adjusts' => array(),
                    'adjusts_order' => array(),
                    'name' => translate('LBL_REVENUE_FC_PRODUCTS', 'Quotes'),
                    'group_type' => 'products',
                ),
            );

            $fmap = array(
				'related_id' => 'id',
				'name',
				'purchase_name',
				'tax_class_id' => 'tax_code_id',
				'mfr_part_no' => 'manufacturers_part_no',
				'vendor_part_no',
				'currency_id',
				'exchange_rate',
				'description',
				'raw_cost_price' => 'cost',
				'raw_list_price' => 'list_price',
				'cost_price' => 'cost_usdollar',
				'list_price' => 'list_usdollar',
			);

            foreach($result->rows as $row) {
            	$prod = ListQuery::quick_fetch_row('Product', $row['product_id']);
            	if($prod) {
            		$line = array('related_type' => 'ProductCatalog');
	                $line['id'] = 'temp~' . rand(1000, 2000);
	                foreach($fmap as $k => $f) {
	                    if(is_int($k)) $k = $f;
	                    $line[$k] = $prod[$f];
	                }
	                $line['quantity'] = $row['qty'];
					$line['unit_price'] = $row['unit_price'];
					$line['raw_unit_price'] = $row['unit_price'];
	                $new_groups[$grp_id]['lines'][$line['id']] = $line;
	                $new_groups[$grp_id]['lines_order'][] = $line['id'];
            	}
            }
        }

        return $new_groups;
    }

    static function before_save(RowUpdate $upd) {
		$changes = $upd->getChanges();
		if (isset($changes['approval'])){
			if ($changes['approval'] == 'Approved')
				$upd->set('approved_user_id', $upd->getUserId());
			else
				$upd->set('approved_user_id', null);
		} if (!empty($_POST['__update_from_ui'])) {
			//pr2($upd);
		}
    }

    static function after_save(RowUpdate $upd) {
    	if($upd->new_record) {
			$opp_id = $upd->getField('opportunity_id');
			if($opp_id && ($opp = ListQuery::quick_fetch('Opportunity', $opp_id))) {
				$opp_up = RowUpdate::for_result($opp);
				if(! preg_match('~^Closed ~', $opp_up->getField('sales_stage'))) {
                    global $app_list_strings;
                    $stage = 'Proposal/Price Quote';
                    $forecast_cat = array_get_default($app_list_strings['sales_forecast_dom'], $stage, '');
                    $probability = array_get_default($app_list_strings['sales_probability_dom'], $stage, '');

                    $updated_fields = array('sales_stage' => $stage, 'forecast_category' => $forecast_cat, 'probability' => $probability);
					$opp_up->set($updated_fields);
					$opp_up->save();
				}
			}
    	}
    }

    static function notify_approval(RowUpdate $upd, &$targets) {
		if($upd->getFieldUpdated('approval') && ($approval = $upd->getField('approval'))) {
            if($approval == 'Approved') {
                $action = 'approved';
            } else if($approval == 'Not Approved') {
                $action = 'rejected';
            } else if($approval == 'Submitted') {
				$action = 'submitted';
            } else {
            	$action = null;
			}
            if($action) {
            	$approveTargets = new NotifyTargets();
				if($action == 'submitted') {
					$approver = findApprovalInfo($upd->getModuleDir(), $upd->getUserId());
					if($approver && $approver['id'] != $upd->getUserId()) {
						$approver['bean_name'] = 'User';
						$approveTargets->addTargetInfo($approver);
					}
				} else if( ($aid = $upd->getField('assigned_user_id')) != $upd->getUserId()) {
					$approveTargets->addTargetId('User', $aid);
					// throw IAHActionCompleted: skip update notification?
					// or remove from update targets
				}
				NotifyManager::record_notify_targets(
					$upd->getModelName(), $action, $upd->getPrimaryKeyValue(), $approveTargets, $upd->getUserId());
            }
        }
	}

	static function get_new_activity_status(RowUpdate $upd) {
        $status = 'created';

		if ($upd->getField('opportunity_id')) {
			$status = array(
				'status' => 'created',
				'converted_to_type' => 'Opportunities',
				'converted_to_id' => $upd->getField('opportunity_id'),
			);
		}
        return $status;
    }

	static function mutateDetailButtons($detail, &$buttons)
	{
		if (isset($buttons['edit']))
			$buttons['edit']['hidden'] = array(
				'type' => 'hook',
				'hook' => array(
					'file' => 'modules/Quotes/Quote.php',
					'class' => 'Quote',
					'class_function' => 'editButtonDisabled',
				)
			);
		if (isset($buttons['print']))
			$buttons['print']['hidden'] = array(
				'type' => 'hook',
				'hook' => array(
					'file' => 'modules/Quotes/Quote.php',
					'class' => 'Quote',
					'class_function' => 'printHidden',
				)
			);
		if (isset($buttons['pdf_print']))
			$buttons['pdf_print']['hidden'] = array(
				'type' => 'hook',
				'hook' => array(
					'file' => 'modules/Quotes/Quote.php',
					'class' => 'Quote',
					'class_function' => 'printHidden',
				)
			);
	}

	static function editButtonDisabled(&$row, &$flag, &$val) {
		$val = false;
		if ($row->getField('approval') == 'Approved') {
			if (self::useApprovals() && !canApproveUser($row->getModuleDir(), AppConfig::current_user_id(), $row->getField('assigned_user_id'), true)) {
				$val = true;
			}
		}
	}

	static function edit_redirect(StandardDetailManager $mgr) {
		if($mgr->getPerform() != 'save') return;
		$rp = $mgr->getReturnParams();
		if (array_get_default($rp, 'return_module') == 'Accounts' && array_get_default($rp, 'return_action') == 'DetailView') {
			$redir = array(
				'redirect', array('module' => 'Quotes', 'action' => 'DetailView', 'record' => $mgr->record_id)
			);
			$mgr->setCustomRedirect($redir);
		}

	}

	static function portal_filter($lq, $contact_id, $account_id) {
		if ($account_id) {
			$clause  = ['field' => 'billing_account_id', 'value' => $account_id];
		} else {
			$clause = ['operator' => 'literal', 'value' => '0'];
		}
		$lq->addFilterClause($clause);
	}

}
?>
