<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
require_once 'modules/Quotes/products_dialog/ProductsList.php';

class SupportedProductsDialog {

    /**
     * Dialog initiator model name
     *
     * @var string
     */
    protected $initiator_model;

    /**
     * Dialog initiator ID
     *
     * @var string
     */
    protected $initiator_id;

    /**
     * @var string
     */
    protected $list_id;

    /**
     *
     * @param string $initiator_model
     * @param string $initiator_id
     */
    public function __construct($initiator_model, $initiator_id) {
        $this->initiator_model = $initiator_model;
        $this->initiator_id = $initiator_id;
    }

    /**
     * Render Dialog view
     *
     * @return void
     */
	public function render() {
		echo '<div style="padding: 5px">';
        $this->renderProductsList();
		$this->renderAdditionalOptions();
		echo '</div>';
    }

    /**
     * Render Products list by initiator
     *
     * @return void
     */
    protected function renderProductsList() {
        $products_list = new ProductsList($this->initiator_model, $this->initiator_id);
        $this->list_id = $products_list->getManager()->list_id;
        $products_list->render();
    }

    /**
     * Render additional options like Create Contract and Create Project
     *
     * @return void
     */
    protected function renderAdditionalOptions() {


		$initiator = ListQuery::quick_fetch_row($this->initiator_model, $this->initiator_id, array('billing_account_id'));

        if (! empty($initiator['billing_account_id'])) {
            $account_id = $initiator['billing_account_id'];
			$layout = [];
			require_once('include/layout/forms/FormGenerator.php');
			$layout = [
				'type' => 'config',
				'editable' => true,
				'buttons_position' => 'bottom',
				'form_hidden_fields' => [],
				'form_buttons' => $this->getButtons($account_id),
				'sections' => [
					[
						'label_position' => 'top',
						'columns' => 4,
						'elements' => [
							'__action_subcontract',
							'__subcontract_type',
							'__subcontract',
							'__subcontract_name',
							'__action_project',
							'__project',
							'__project_name',
							''
						]
					],
				],
			];

			$fields = $this->getAdditionalFields($account_id);
			$result = new RowResult;
			$result->row = [
				'__action_subcontract' => 'create_new',
				'__subcontract' => key($fields['__subcontract']['options']),
				'__subcontract_type' => key($fields['__subcontract_type']['options']),
				'__action_project' => 'create_new',
				'__project' => key($fields['__project']['options']),
			];
			$result->fields = $fields;
			$model = new ModelDef('Quote');
			$model->addDisplayFieldDefinitions($result->fields);
			$result->base_model = 'Quote';
			$result->base_model = 'Quote';
			$gen = FormGenerator::html_form($model, $layout, 'CreateProductsForm');
			$gen->setFormClass('card-outer form-outer default-form admin-form');
			$gen->renderForm($result, 'Administration');
			$html = $gen->getResult();
			$gen->exportIncludes();

			echo '<div class="detailOuter">' . $html . '</div>';

			$js = <<<JS
var f = SUGAR.ui.getForm('CreateProductsForm');
if (f) {
	contract_action_change(f, '__action_subcontract');
	contract_action_change(f, '__action_project');
}
JS;
            global $pageInstance;
            $pageInstance->add_js_include('modules/Quotes/products_dialog/products_dialog.js', null, LOAD_PRIORITY_END);
            $pageInstance->add_js_literal($js, null, LOAD_PRIORITY_END);
        }
    }

	protected function getButtons($account_id) {
		$buttons = [
			'proceed' => [
				'vname' => translate('LBL_PROCEED'),
				'perform' => "navigate('back'); return false",
				'id' => 'proceed_button',
				'icon' => 'icon-accept',
				'type' => 'button',
				'perform' => "return create_contracts(this.form, '{$this->list_id}', '{$this->initiator_model}', '{$this->initiator_id}', '{$account_id}');",
			],
			'cancel' => [
				'vname' => translate('LBL_CANCEL_BUTTON_LABEL'),
				'perform' => "navigate('back'); return false",
				'id' => 'cancel_button',
				'icon' => 'icon-cancel',
				'type' => 'button',
				'perform' => 'SUGAR.ui.PopupManager.close()',
			],
		];
		return $buttons;
	}
	protected function getAdditionalFields($account_id) {
		$fields = [
			'__action_subcontract' => [
				'vname' => '&nbsp;',
				'type' => 'enum',
				'editable' => true,
				'options_add_blank' => false,
            	'onchange' => 'contract_action_change(this.form, this.name)',
			],
			'__subcontract' => [
				'vname' => '&nbsp;',
				'type' => 'enum',
				'editable' => true,
				'options_add_blank' => false,
				'options' => $this->buildObjectOptions('SubContract', $account_id),
			],
			'__subcontract_type' => [
				'vname' => 'LBL_CONTRACT_TYPE',
				'vname_module' => 'Quotes',
				'type' => 'enum',
				'editable' => true,
				'options' => $this->buildContractTypeOptions(),
				'options_add_blank' => false,
				'depends_on' => '__action_subcontract',
				'depends_on_value' => 'use_existing',
			],
			'__subcontract_name' => [
				'vname' => 'LBL_CONTRACT_NAME',
				'vname_module' => 'Quotes',
				'type' => 'varchar',
				'editable' => true,
			],
			'__action_project' => [
				'vname' => '&nbsp;',
				'type' => 'enum',
				'editable' => true,
				'options_add_blank' => false,
            	'onchange' => 'contract_action_change(this.form, this.name)',
			],
			'__project' => [
				'vname' => '&nbsp;',
				'type' => 'enum',
				'editable' => true,
				'options' => $this->buildObjectOptions('Project', $account_id),
				'options_add_blank' => false,
				'depends_on' => '__action_project',
				'depends_on_value' => 'use_existing',
			],
			'__project_name' => [
				'vname' => 'LBL_CONTRACT_NAME',
				'vname_module' => 'Quotes',
				'type' => 'varchar',
				'editable' => true,
			],
		];
		$use_existing =  is_countable($fields['__subcontract']['options']) ? count($fields['__subcontract']['options']) : 0;
		$fields['__action_subcontract']['options'] = $this->buildActionOptions('SubContract', $use_existing);
		$use_existing = is_countable($fields['__project']['options']) ? count($fields['__project']['options']) : 0;
		$fields['__action_project']['options'] = $this->buildActionOptions('Project', $use_existing);

		return $fields;
	}

    /**
     * Build Contract Type options
     *
     * @return array:
     * 1. keys
     * 2. values
     * 3. default_id
     * 4. default_name
     */
    protected function buildContractTypeOptions() {
		$result = ListQuery::quick_fetch_all('ContractType');
		$contrTypeOptions = [];

        foreach ($result->getRows() as $row) {
        	$contrTypeOptions[$row['id']] = $row['name'];
        }

        return $contrTypeOptions;
    }

    /**
     * Build options list by object
     *
     * @param string $object - 'SubContract' or 'Project'
     * @param $account_id
     * @return array:
     * 1. keys
     * 2. values
     * 3. default_id
     * 4. default_name
     */
    protected function buildObjectOptions($object, $account_id) {
		$contrOptions = [];

        $lq = new ListQuery($object);
		$lq->addSimpleFilter('account_id', $account_id);
        $result = $lq->runQuery();
        $setDefault = false;

        if ($result->getResultCount()) {
        	foreach ($result->getRows() as $row) {
        		$contrOptions[$row['id']]  = $row['name'];

				/*
        		if (! $setDefault) {
                    $contrOptions['default_id'] = $row['id'];
                    $contrOptions['default_name'] = $row['name'];
                    $setDefault = true;
				}
				 */

        	}
        }

        return $contrOptions;
    }

    /**
     * Build Actions on object options
     *
     * @param string $object: 'SubContract' or 'Project'
     * @param bool $use_existing - add use existing option or not
     * @return array:
     * 1. keys
     * 2. values
     */
    protected function buildActionOptions($object = 'Contract', $use_existing = false) {
        $actOptions = [];
        $new_lbl = $existing_lbl = $not_link_lbl = '';

        if ($object == 'SubContract') {
            $new_lbl = 'LBL_CREATE_CONTRACT';
            $existing_lbl = 'LBL_SELECT_CONTRACT';
            $not_link_lbl = 'LBL_NO_CONTRACT';
        } elseif ($object == 'Project') {
            $new_lbl = 'LBL_CREATE_PROJECT';
            $existing_lbl = 'LBL_SELECT_PROJECT';
            $not_link_lbl = 'LBL_NO_PROJECT';
        }

        if ($object != '' && ($new_lbl != '' && $existing_lbl != '' && $not_link_lbl != '') ) {
            $actOptions['create_new'] = translate($new_lbl);
            if ($use_existing) {
                $actOptions['use_existing'] = translate($existing_lbl);
            }

            $actOptions['not_link'] = translate($not_link_lbl);
        }

        return $actOptions;
    }
}
?>
