// vim: set autoindent smartindent :


AppBase.module('quotes').moduleRequires('ui-controls').init(function(App) {
	var UI = App.ui,
		Wrap = App.dom.elementOrId,
		QueryId = App.dom.queryId,
		MakeElt = App.dom.makeElement;

	var uniqid = 1;

	UI.TallyEditor = function(id, params) {
		this.id = 'TallyEditor';
		this.module = '';
		this.form_name = '';
		this.editable = true;
		this.editable_events = false;
		this.form = null;
		this.totals = {
			subtotal: 0.0,
			shipping: 0.0,
			discounts: 0.0,
			taxes: 0.0,
			total: 0.0
		};
		this.add_product_description = false; // config setting
		this.add_assembly_description = false; // config setting
		this.add_assembly_discounts = false;
		this.tabIndex = 30;
		this.default_tax_code_id = '';
		this.default_discount_id = '';
		this.default_pricing_method = 'Fixed';

		this.catalog_only = true;
		this.fixed_prices = true;
		this.manual_discounts = false;
		this.standard_discounts = false;
		this.product_costs = false;
		this.can_add_related_events = false;
		this.add_booked_hours_comments = false;
		this.show_pretax_totals = false;
		this.show_net_prices = false;
		this.pricebook_editable = '1';
		this.top_labels = SUGAR.themes.version(8);
		this.debug = 0;

		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		if(params) extendObject(this, params);
		this.is_outgoing = (this.module === 'PurchaseOrders' || this.module === 'Bills' || this.module === 'CreditNotes');
		this.loadGroups(this.init_groups);
	}

	UI.TallyEditor.pricing_method_map = {
		'': 'None',
		fixed: 'Fixed',
		inherit: 'Inherit',
		margin: 'Margin',
		markup: 'Markup',
		discount: 'PercentDiscount',
		list: 'SameAsList'
	};

	UI.TallyEditor.prototype = {
		setup: function() {
			if(this.set_up) return;
			if(! this.elt)
				this.elt = $(this.id + '-outer');
			if(! this.elt)
				return;
			this.form_name = this.form.getAttribute('name');
			this.addHooks();
			this.recalculate(true);
			UI.onInitForm(this.form, this.postSetup.bind(this));
			this.checkPageSize();
			Wrap(window).on('page-size', function() {
				this.handleResize();
			}, {scope: this});
			this.set_up = true;
		},

		postSetup: function() {
			if(this.editable && ! this.groups_order.length && this.add_initial_group) {
				this.addGroup(); // causes render
				// this.addLineItem(this.groups_order[0], {'related_type': 'ProductCatalog'});
			} else {
				this.render();
			}

			if(this.init_popup_options) {
				var cid = this.getCurrencyId();
				this.init_popup_options.display_currency_id = cid;
				open_popup_window(this.init_popup_options);
				this.init_popup_options = null;
			}

			var pricebook = UI.getFormInput(this.form, 'pricebook');
			if (pricebook && pricebook.orig_key && this.form.record.value != '') {
				var acct = UI.getFormInput(this.form, 'billing_account');
				if (acct)
					acct.setReadOnly(true);
			}  else if (pricebook && pricebook.orig_key && this.form.record.value == '') {
				pricebook.setReadOnly(this.pricebook_editable != '1');
			}

			UI.attachFormInputEvent(this.form, 'pricebook', 'onchange', function() {
				this.render();
			}.bind(this));

			var edit_btn = $('tally-edit-btn');
			if(edit_btn && ! this.canEdit())
				edit_btn.style.display = 'none';
		},

		addHooks: function() {
			var bill_acct = UI.getFormInput(this.form, 'billing_account'),
				ship_acct = UI.getFormInput(this.form, 'shipping_account'),
				supplier = UI.getFormInput(this.form, 'supplier'),
				bill_ctc = UI.getFormInput(this.form, 'billing_contact'),
				ship_ctc = UI.getFormInput(this.form, 'shipping_contact');
				// This weird field name was chosen to reduce the risk of conflicts with custom fields
				account_balance_input = UI.HiddenInput.findOrInsert(this.form, 'account_credit_hf_warncl', 'account_credit_hf_warncl');
				account_er = UI.HiddenInput.findOrInsert(this.form, 'billing_acct_er_hf_warncl', 'billing_acct_er_hf_warncl');

			var acct_fields = {
					'billing_address_street': 'billing_address_street',
					'billing_address_city': 'billing_address_city',
					'billing_address_state': 'billing_address_state',
					'billing_address_postalcode': 'billing_address_postalcode',
					'billing_address_country': 'billing_address_country',
					'shipping_address_street': 'shipping_address_street',
					'shipping_address_city': 'shipping_address_city',
					'shipping_address_state': 'shipping_address_state',
					'shipping_address_postalcode': 'shipping_address_postalcode',
					'shipping_address_country': 'shipping_address_country',
					'shipping_provider_id' : 'default_shipper_id',
					'default_pricebook' : 'default_pricebook',
					'default_pricebook_id' : 'default_pricebook_id',
					'default_pricebook_editable' : 'default_pricebook.editable',
					'terms': 'default_terms',
					'currency_id': 'currency_id',
					'tax_code_id': 'tax_code_id',
					'raw_balance': 'account_balance',
					'account_credit_limit' :'credit_limit_usd',
					'discount_id' : 'default_discount_id',
					'tax_information': 'tax_information',
					'account_popups': 'account_popups',
					'sales_popup': 'sales_popup'};
			if(bill_acct) {
				bill_acct.addExtraReturnFields(acct_fields);
				UI.attachInputEvent(bill_acct, 'onchange', function(k, v) {
					var pricebook = UI.getFormInput(this.form, 'pricebook');
					if (pricebook) {
						if (bill_acct.getKey() == '')
							pricebook.setReadOnly(false);
					}
					this.setAccount(v, 'billing', true);

					if((parseFloat(v.credit_limit_usd)||0) == 0)
						account_balance_input.setValue('000');
					else
						account_balance_input.setValue(v.credit_limit_usd-v.balance);

					account_er.setValue(v.exchange_rate);
				}.bind(this));
				if(bill_ctc)
					bill_ctc.addFilter({param: 'primary_account_id', field_name: bill_acct.key_name});
			}
			if(ship_acct) {
				delete acct_fields['tax_information'];
				ship_acct.addExtraReturnFields(acct_fields);
				UI.attachInputEvent(ship_acct, 'onchange', function(k, v) {
					this.setAccount(v, 'shipping', true);
				}.bind(this));
				if(ship_ctc)
					ship_ctc.addFilter({param: 'primary_account_id', field_name: ship_acct.key_name});
			}
			if (supplier) {
				supplier.addExtraReturnFields({'terms': 'default_purchase_terms'});
				UI.attachInputEvent(supplier, 'onchange', function(k, v) {
					UI.setFormInputValues(this.form, v);
				});
			}

			this.currency_input = UI.getFormInput(this.form, 'currency');
			if(! this.currency_input) {
				// create a stub currency input
				this.currency_input = new UI.CurrencySelect(this.id + '-currency', {
					elt: createElement2('div'),
					field: this.form.currency_id,
					key_field: this.form.currency_id,
					rate_input: new UI.HiddenInput(this.form.exchange_rate)
				});
				this.currency_input.setup();
			} else {
				UI.attachInputEvent(this.currency_input, 'onrateupdate', function(old_rate, new_rate) {
					this.updateExchangeRate(old_rate, new_rate);
				}.bind(this));
				if(! this.editable)
					this.currency_input.setReadOnly();
			}

			// NOTE: AD: Do not change this if someone complains that they cannot
			// edit billing account!
			// If the items are not editable, make billing account read-only.
			// The reason is that changing account can trigger a change to
			// currency, and that requires recalculation and redraw. While
			// recalculation is perfectly possible when line items are readonly,
			// readraw can be done only to editable line items.
			// Billing account will be made editable in makeEditable()
			if(! this.editable) {
				var acc_input = UI.getFormInput(this.form, 'billing_account');
				if (acc_input) {
					acc_input.setReadOnly();
				}
			}

			var opp = UI.getFormInput(this.form, 'opportunity');
			if(opp && bill_acct) {
				opp.addFilter({param: 'account', field_name: bill_acct.name});
			}
		},

		canEdit: function() {
			return true;
		},

		makeEditable: function(events_only) {
			if(! this.editable) {
				var acc_input = UI.getFormInput(this.form, 'billing_account');
				if (acc_input) {
					acc_input.setReadOnly(false);
				}
				this.currency_input.setReadOnly(false);
				var tax_exempt = UI.getFormInput(this.form, 'tax_exempt');
				if (tax_exempt) tax_exempt.setDisabled(false);
				if (!events_only) {
					this.editable = true;
				} else {
					this.editable_events = true;
				}
				this.recalculate(true);
				this.render();
				UI.setDisplayState('tally-edit-btn', false);
			}
		},


		// -- Group and line management --

		new_group_defaults: {
			status: 'Draft',
			group_type: 'products',
			// set during init
			taxes: [],
			discounts: []
		},

		group_template: {
			name: '',
			status: '',
			pricing_method: 'fixed',
			pricing_percentage: 0.0,
			lines: {},
			lines_order: [],
			adjusts: {},
			adjusts_order: [],
			taxes_order: [],
			discounts_order: [],
			shipping_tax_class_id : '',
			shipping: 0.0,
			shipping_adj_id: '',
			subtotal: 0.0,
			total: 0.0
		},

		line_template: {
			quantity: 1,
			ext_quantity: 1,
			sum_of_components: 0,
			name: '',
			parent_id: '',
			tax_class_id: '',
			mfr_part_no: '',
			vendor_part_no: '',
			cost_price: 0.0,
			list_price: 0.0,
			unit_price: 0.0,
			std_unit_price: 0.0,
			ext_price: 0.0,
			net_price: 0.0,
			discount: 0.0,
			pricing_adjust_id: null,
			adjusts: {}
		},

		price_columns: [
			'cost_price',
			'list_price',
			'unit_price',
			'std_unit_price',
			'ext_price',
			'net_price',
			'discount'
		],

		loadGroups: function(init_groups) {
			this.groups = {};
			this.groups_order = [];
			if(! Array.isArray(init_groups)) return;
			for(var idx = 0; idx < init_groups.length; idx++) {
				var grp = deep_clone(init_groups[idx]),
					grp_lines = grp.lines,
					grp_adjusts = grp.adjusts,
					grp_id = grp.id;
				if(! grp_id) grp_id = grp.id = 'newgroup~' + (uniqid ++);

				grp.lines_order = [];
				grp.lines = {};
				grp.adjusts_order = [];
				grp.adjusts = {};
				grp.taxes_order = [];
				grp.discounts_order = [];
				if(grp.pricing_percentage)
					grp.pricing_percentage = parseFloat(grp.pricing_percentage);
				if(! isset(grp.pricing_method))
					grp.pricing_method = '';
				grp.subtotal = parseFloat(grp.subtotal);
				grp.total = parseFloat(grp.total);
				grp.shipping = 0.0;
				grp.shipping_tax_class_id = this.default_tax_shipping ? (this.default_shipping_tax_code || 'all') : '-99';
				grp.shipping_adj_id = '';

				var has_attrs = [];
				if (Array.isArray(grp_lines)) {
					for(var jdx = 0; jdx < grp_lines.length; jdx++) {
						var line = grp_lines[jdx],
							line_id = line.id,
							line_adjusts = line.adjusts;
						if(! line_id)
							line_id = line.id = 'newline~' + (uniqid ++);
						line.quantity = parseFloat(line.quantity);
						line.ext_quantity = parseFloat(line.ext_quantity);
						line.sum_of_components = parseInt(line.sum_of_components, 10);
						line.depth = parseInt(line.depth, 10);
						line.adjusts_order = [];
						line.adjusts = {};

						if (Array.isArray(line_adjusts)) {
							for(var kdx = 0; kdx < line_adjusts.length; kdx++) {
								var adj = line_adjusts[kdx],
									adj_id = adj.id;
								if(! adj_id)
									adj_id = adj.id = 'newadj~' + (uniqid ++);
								adj.amount = parseFloat(adj.amount);
								adj.rate = parseFloat(adj.rate);
								line.adjusts_order.push(adj_id);
								line.adjusts[adj_id] = adj;
							}
						}

						for(var kdx = 0; kdx < this.price_columns.length; kdx++)
							line[this.price_columns[kdx]] = parseFloat(get_default(line[this.price_columns[kdx]], 0));
						if(this.getLinePricingMethod(idx, line) == '')
							line.custom_unit_price = line.unit_price;
						if(line.attributes_meta)
							has_attrs.push(line_id);

						grp.lines_order.push(line_id);
						grp.lines[line_id] = line;
					}
				}

				if (Array.isArray(grp_adjusts)) {
					for(var jdx = 0; jdx < grp_adjusts.length; jdx++) {
						var adj = grp_adjusts[jdx],
							adj_id = adj.id;
						if(! adj_id)
							adj_id = adj.id = 'newadj~' + (uniqid ++);
						adj.editable = parseInt(adj.editable, 10);
						adj.rate = parseFloat(adj.rate);
						adj.amount = parseFloat(adj.amount);

						if(adj.type == 'TaxedShipping' || adj.type == 'UntaxedShipping') {
							grp.shipping_adj_id = adj_id;
							grp.shipping = adj.amount;
							if (adj.type == 'UntaxedShipping')
								adj.tax_class_id = '-99';
							else {
								if (!adj.tax_class_id)
									adj.tax_class_id = 'all';
							}
							grp.shipping_tax_class_id = adj.tax_class_id;
						}
						else if(adj.related_type == 'TaxRates' && !adj.line_id) {
							grp.taxes_order.push(adj_id);
						}
						else if(adj.related_type == 'Discounts') {
							grp.discounts_order.push(adj_id);
						}

						grp.adjusts_order.push(adj_id);
						grp.adjusts[adj_id] = adj;
					}
					for(var jdx = 0; jdx < has_attrs.length; jdx++) {
						this.checkLineAttributes(grp.lines[has_attrs[jdx]]);
					}
				}

				this.groups_order.push(grp_id);
				this.groups[grp_id] = grp;
			}
		},

		checkLineAttributes: function(line) {
			var missing_attr = {};
			if(! line.attributes_meta || Array.isArray(line.attributes_meta))
				line.attributes_meta = {};
			for(var nm in line.attributes_meta)
				missing_attr[nm] = 1;
			for (var i in line.adjusts) {
				var adj = line.adjusts[i];
				if (adj.type != 'ProductAttributes')
					continue;
				var s = adj.name.split(' : '), preset_value;
				if(s.length > 1) {
					adj.name = s[0];
					adj.value = preset_value = s[1];
				}
				if(isset(missing_attr[adj.name]))
					delete missing_attr[adj.name];
				if(isset(preset_value) && line.attributes_meta[adj.name]) {
					for (var j = 0; j < line.attributes_meta[adj.name].length; j++) {
						var meta = line.attributes_meta[adj.name][j];
						if(isset(preset_value) && preset_value == meta.value) {
							meta.price = adj.amount;
							meta.price_usdollar = adj.amount_usd;
							preset_value = null;
						}
						if(j == line.attributes_meta[adj.name].length - 1 && isset(preset_value)) {
							// add new item
							var new_meta = {
								'type': adj.type,
								'name': adj.name,
								'value': preset_value,
								'price': adj.amount || 0,
								'price_usdollar': adj.amount_usd || 0,
								'related_type': adj.related_type,
								'related_id': adj.related_id
							};
							line.attributes_meta[adj.name].push(new_meta);
							preset_value = null;
						}
					}
				}
			}
			for(var nm in missing_attr)
				this.addLineAdjustment(line, {type: 'ProductAttributes', related_type: 'ProductAttributes', 'name': nm });
		},

		getGroup: function(group) {
			if(typeof(group) === 'string')
				return this.groups[group];
			else if(isPlainObject(group))
				return group;
		},

		iterGroups: function(callback) {
			var ord = this.groups_order;
			for(var idx = 0; idx < ord.length; idx++) {
				if(callback.call(this, ord[idx], this.groups[ord[idx]], idx))
					break;
			}
			return idx;
		},

		addGroup: function(group_info) {
			var group = deep_clone(this.group_template);
			var new_group = false;

			if(! isPlainObject(group_info))
				new_group = true;
			else
				for(var k in group_info) group[k] = group_info[k];
			if(! group.id)
				group.id = 'newgroup~' + (uniqid ++);
			var prevgrp = null;
			if(this.groups_order.length) {
				var previd = this.groups_order[this.groups_order.length - 1];
				prevgrp = this.groups[previd];
			}
			this.groups[group.id] = group;
			this.groups_order.push(group.id);

			if(new_group) {
				function copy_adj(adj) {
					var ret = deep_clone(adj);
					ret.id = 'newadj~' + (uniqid ++);
					ret.amount = 0.0;
					return ret;
				}
				if(prevgrp) {
					// copy values from last group
					group.status = prevgrp.status;
					group.group_type = prevgrp.group_type;
					group.pricing_method = prevgrp.pricing_method;
					group.pricing_percentage = prevgrp.pricing_percentage;
					for(var idx = 0; idx < prevgrp.taxes_order.length; idx++) {
						var adj = copy_adj(prevgrp.adjusts[prevgrp.taxes_order[idx]]);
						group.adjusts[adj.id] = adj;
						group.adjusts_order.push(adj.id);
						group.taxes_order.push(adj.id);
					}
					for(var idx = 0; idx < prevgrp.discounts_order.length; idx++) {
						var adj = copy_adj(prevgrp.adjusts[prevgrp.discounts_order[idx]]);
						group.adjusts[adj.id] = adj;
						group.adjusts_order.push(adj.id);
						group.discounts_order.push(adj.id);
					}
					group.shipping_tax_class_id = prevgrp.shipping_tax_class_id;
				}
				else {
					group.pricing_method = this.default_pricing_method;
					for(var idx in this.new_group_defaults) {
						if(blank(group[idx])) {
							if(isPlainObject(this.new_group_defaults[idx]))
								group[idx] = deep_clone(this.new_group_defaults[idx]);
							else
								group[idx] = this.new_group_defaults[idx];
						}
					}
					group.shipping_tax_class_id = this.default_tax_shipping ? (this.default_shipping_tax_code || 'all') : '-99';
				}
				if(this.default_discount_id && ! group.discounts_order.length) {
					var adj = copy_adj(this.getDiscountAdjustment(this.default_discount_id));
					var disc = SysData.discounts[this.default_discount_id] || {};
					if (!disc.applies_to_selected) {
						group.discounts_order.push(adj.id);
						group.adjusts_order.push(adj.id);
						group.adjusts[adj.id] = adj;
					}
				}
				if(! group.shipping_adj_id) {
					var igd = this.ignore_dirty;
					this.ignore_dirty = true;
					this.updateGroupField(group.id, 'shipping_tax_class_id', group.shipping_tax_class_id);
					this.ignore_dirty = igd;
				}
			}

			var add_lines = group.add_lines;
			delete group.add_lines;
			if(Array.isArray(add_lines)) {
				this.loadLineItems(add_lines, function(lines) {
					for(var idx = 0; idx < lines.length; idx++)
						this.addLineItem(group.id, lines[idx], true, false);
					this.recalcGroup(group.id);
					this.render();
				});
			} else {
				this.recalcGroup(group.id);
				this.render();
			}
		},

		updateGroupField: function(group, field, value, recalc) {
			if(! (group = this.getGroup(group))) return;
			if (typeof(value) == 'number' && isNaN(value)) value = 0;
			group[field] = value;
			if(field === 'group_type') {
				this.clearLineItems(group);
				recalc = false;
			}
			else if(field === 'shipping' || field === 'shipping_tax_class_id') {
				var adj = group.adjusts[group.shipping_adj_id];
				if(field == 'shipping_tax_class_id')
					group.shipping_tax_class_id = value;
				if(adj) {
					if(field === 'shipping')
						adj.amount = value;
					else {
						adj.type = value != '-99' ? 'TaxedShipping' : 'UntaxedShipping';
						adj.tax_class_id = value;
					}
				}
				else {
					var adj_id = this.addAdjustment(group, {
						related_type: 'ShippingProviders',
						type: group.shipping_tax_class_id != '-99' ? 'TaxedShipping' : 'UntaxedShipping',
						amount: field == 'shipping_tax_class_id' ? 0.00 : value,
						tax_class_id: field == 'shipping_tax_class_id' ? value : '-99'
					});
					group.shipping_adj_id = adj_id;
				}
			}
			if(recalc) {
				this.recalcGroup(group);
				this.renderGroup(group);
				this.renderTotals();
			}
			UI.markFormDirty(this);
		},

		moveGroup: function (group_id, offset, check_only) {
			var a = this.groups_order.indexOf(group_id);
			if (!~a)
				return false;
			b = a + offset;
			if (b >= this.groups_order.length || b < 0)
				return false;
			if (check_only)
				return true;
			var tmp = this.groups_order[a];
			this.groups_order[a] = this.groups_order[b];
			this.groups_order[b] = tmp;
			this.render();
		},

		removeGroup: function(group) {
			if(! (group = this.getGroup(group))) return false;
			this.clearGroupInputs(group);
			this.groups_order.remove(group.id);
			delete this.groups[group.id];
			this.render();
			return true;
		},

		getLineItem: function(group, line_id) {
			if(! (group = this.getGroup(group))) return;
			return group.lines[line_id];
		},

		// may modify to iterate all line items when no group is undefined
		iterLineItems: function(callback, group) {
			var idx = 0;
			group = this.getGroup(group);
			if(group && group.lines_order) {
				var ord = group.lines_order;
				for(; idx < ord.length; idx++) {
					if(callback.call(this, ord[idx], group.lines[ord[idx]], idx))
						break;
				}
			}
			return idx;
		},

		clearLineItems: function(group) {
			if( (group = this.getGroup(group)) ) {
				var ord = group.lines_order, lines = group.lines;
				group.lines = {};
				group.lines_order = [];
				this.recalcGroup(group);
				this.renderGroup(group);
				this.renderTotals();
				if(ord) {
					for(var idx = 0; idx < ord.length; idx++)
						this.clearLineInputs(group, lines[idx]);
				}
			}
		},

		addLineItem: function(group, spec, no_redraw, focus) {
			var editor = this,
				inspos = 0,
				found, newline;
			if(! (group = this.getGroup(group))) return;
			if(blank(spec.id))
				spec.id = 'newline~' + (uniqid ++);
			spec.depth = 0;
			if(spec.parent_id) {
				found = 0;
				for(var idx = 0; idx < group.lines_order.length; idx++) {
					var plid = group.lines_order[idx];
					if(group.lines[plid].id == spec.parent_id) {
						found = 1;
						if(group.lines[plid].sum_of_components)
							spec.depth = group.lines[plid].depth + 1;
					}
					else if(group.lines[plid].parent_id == spec.parent_id) {
						found = 1;
					}
					else if(found)
						break;
					inspos++;
				}
			}
			else
				inspos = group.lines_order.length;
			group.lines_order.splice(inspos, 0, spec.id);
			if(spec.related_type == 'Notes')
				newline = { is_comment: 1 };
			else
				newline = deep_clone(this.line_template);
			if(spec.related_type == 'Assemblies' || spec.related_type == 'SupportedAssemblies')
				newline.sum_of_components = 1;
			for(var idx in spec)
				newline[idx] = spec[idx];
			if(! newline.pricing_adjust_id && ! newline.sum_of_components) {
				var parent = newline.parent_id && group.lines[newline.parent_id];
				var prevln, pi;
				if(inspos && (! parent || ! parent.pricing_adjust_id)) {
					for(pi = inspos - 1; pi > 0; pi --) {
						prevln = group.lines[group.lines_order[pi]];
						if(prevln.parent_id || prevln.sum_of_components != newline.sum_of_components)
							prevln = null;
						else
							break;
					}
				}
				if (prevln && !blank(prevln.pricing_adjust_id) && prevln.adjusts) {
					var tpl = deep_clone(prevln.adjusts[prevln.pricing_adjust_id]);
					var adj = this.addLineAdjustment(newline, tpl);
					newline.pricing_adjust_id = adj.id;
				}
			}
			if(this.getLinePricingMethod(group, newline) != '') {
				var fn = this.getPricingMethodFn(group, newline);
				if(fn) fn(newline);
			}
			this.updateExtPrice(newline);
			group.lines[newline.id] = newline;
			if(! no_redraw) {
				this.recalcGroup(group);
				this.renderGroup(group);
				this.renderTotals();
				if(focus)
					this.focusLineItem(group, newline.id, focus === 'open');
			}
			UI.markFormDirty(this);
			return spec.id;
		},

		duplicateLineItem: function(group, line_id, parent_id, noredraw) {
			if(! (group = this.getGroup(group))) return;
			var line = this.getLineItem(group, line_id),
				children = [];
			if(! line) return;
			this.iterLineItems(function(alt_id, alt) {
				if(alt.parent_id === line_id)
					children.push(alt.id);
			}, group);
			var line2 = deep_clone(line);
			delete line2.id;
			if(parent_id)
				line2.parent_id = parent_id;
			var pid = line2.pricing_adjust_id, adj;
			if(pid) {
				adj = line2.adjusts[pid];
				delete line2.adjusts[pid];
				pid = 'newadj~' + (uniqid ++);
				adj.id = line2.pricing_adjust_id = pid;
				line2.adjusts[pid] = adj;
			}
			var new_line_id = this.addLineItem(group, line2, true);
			for(var i = 0; i < children.length; i++) {
				this.duplicateLineItem(group, children[i], new_line_id, true);
			}
			if(! noredraw) {
				this.renderGroup(group);
				this.renderTotals();
				this.focusLineItem(group, new_line_id);
			}
		},

		focusLineItem: function(group_id, line_id, open, field) {
			var doFocus = function() {
				var inputs = this.getLineInputs(group_id, line_id);
				if(! field) field = 'name';
				if(inputs && inputs[field]) {
					if(open && (inputs[field] instanceof UI.RefInput || inputs[field] instanceof PricingInput))
						inputs[field].showSearch();
					else
						inputs[field].focus();
				}
			}
			return requestAnimationFrame(doFocus.bind(this));
		},

		replaceLineItem: function(group, line_id, spec, recalc, no_redraw) {
			if(! (group = this.getGroup(group))) return;
			if(! line_id)
				line_id = this.addLineItem(group, spec, true);
			var line = group.lines[line_id];
			if(! line) return;
			for(var idx in spec) {
				line[idx] = spec[idx];
			}
			if(this.getLinePricingMethod(group, line) != '') {
				var fn = this.getPricingMethodFn(group, line);
				if(fn) fn(line);
			}
			this.updateExtPrice(line);
			delete line.custom_unit_price;
			if(recalc) {
				this.recalcGroup(group);
				if(! no_redraw) {
					this.renderGroup(group);
					this.renderTotals();
				}
			}
			return line_id;
		},

		loadLineItem: function(values, callback) {
			if(! isPlainObject(values)) values = {};
			var tax_exempt = UI.getFormInput(this.form, 'tax_exempt');
			if (tax_exempt && tax_exempt.getValue()) {
				values.tax_class_id = '-99';
			}
			if(! values.tax_class_id && this.default_tax_code_id) {
				values.tax_class_id = this.default_tax_code_id;
			}
			if (this.is_outgoing && !blank(values.purchase_name)) {
				values.name = values.purchase_name;
			}
			var currency_id = this.getCurrencyId(),
				line_currency_id = get_default((values.pricebook_currency_id || values.currency_id), ''),
				conversion_rate = this.getExchangeRate(),
				decimals = this.currencyDecimalPlaces(),
				numval;
			if(! line_currency_id) line_currency_id = '-99';
			for(var idx = 0; idx < this.price_columns.length; idx++) {
				var col = this.price_columns[idx];
				if(line_currency_id == currency_id && isset(values['raw_'+col])) {
					if ((col == 'list_price' || col == 'unit_price') && isset(values.raw_pricebook_price))
						values[col] = trunc(parseFloat(values['raw_pricebook_price']), decimals);
					else
						values[col] = trunc(parseFloat(values['raw_'+col]), decimals);
				} else {
					if ((col == 'list_price' || col == 'unit_price') && isset(values.pricebook_price))
						numval = parseFloat(get_default(values.pricebook_price, 0.0));
					else
						numval = parseFloat(get_default(values[col], 0.0));
					values[col] = trunc(numval * conversion_rate, decimals);
				}
			}
			if( (!isset(values.std_unit_price) || values.std_unit_price == 0) && isset(values.unit_price))
				values.std_unit_price = values.unit_price;
			if(! isset(values.name))
				values.name = '';
			if(! isset(values.related_id))
				values.related_id = '';
			if(! isset(values.mfr_part_no))
				values.mfr_part_no = '';
			delete values.id;
			delete values._display;

			if(values.related_type == 'Assets' || (! isset(values.unit_price) && this.module != 'PurchaseOrders' && this.module != 'Bills' && this.module != 'CreditNotes')) {
				values.unit_price = values.list_price;
			} else if(values.related_type == 'BookingCategories') {
				var extra = [];
				extra.push(SUGAR.language.getListString('booking_location_dom', values.location));
				extra.push(SUGAR.language.getListString('booking_duration_dom', values.duration));
				extra.push(SUGAR.language.getListString('booking_seniority_dom', values.seniority));
				extra = extra.filter(function(v) {
					return typeof v === "string" && v.trim().length > 0;
				});
				if (extra.length > 0) {
					values.name += ' (' + extra.join(' / ') + ')';
				}
				values.unit_price = values.std_unit_price = values.list_price;
				values.cost_price = 0.00;
			}

			if(this.needProductAttrs(values)) {
				this.getProductAttrs(values.related_id, function(attrs) {
					values.attributes_meta = this.getProductAttrs(values.related_id);
					if (callback)
						callback.call(this, values);
				});
				return values;
			} else {
				if(callback)
					callback.call(this, values);
				return values;
			}
		},

		loadLineItems: function(lines, callback) {
			var line, ret = [], fetch_attrs = [];
			for(var i = 0; i < lines.length; i++) {
				line = this.loadLineItem(lines[i]);
				if(this.needProductAttrs(line))
					fetch_attrs.push(line.related_id);
				ret.push(line);
			}
			if(callback) {
				this.getProductAttrs(fetch_attrs, function(attrs) {
					for (var i= 0; i < ret.length; i++) {
						var line = ret[i];
						var l_attrs = attrs[line.related_id] || line.attributes_meta;
						if (l_attrs) {
							for (var aname in l_attrs) {
								this.addLineAdjustment(line, {type: 'ProductAttributes', related_type: 'ProductAttributes', 'name': aname });
							}
							line.attributes_meta = l_attrs;
						}
					}
					callback.call(this, ret);
				});
			}
			return ret;
		},

		returnLineItem: function(group_id, line_id, values, custom_redraw) {
			var group = this.getGroup(group_id);
			if(! group) {
				console.error('Group not found');
				return;
			}

			var new_line_id = this.replaceLineItem(group_id, line_id, values, true, true),
				line = group.lines[new_line_id],
				renderGroup = (new_line_id != line_id && ! custom_redraw),
				editor = this;
			if(line) {
				if(line.adjusts) {
					for (var i in line.adjusts) {
						var adj = line.adjusts[i];
						if (adj.type == 'ProductAttributes') {
							delete line.adjusts[i];
						}
						if (line.related_type === 'ProductCatalog' 
							&&(adj.type == 'StdFixedDiscount' || adj.type == 'StdPercentDiscount') ) {
						
							var disc = SysData.discounts[adj.related_id];
							if (disc && disc.applies_to_selected) {
								if (!~disc.products.indexOf(line.related_id)) {
									setTimeout(function() {
										editor.updateLinePricing(group_id, line_id, {type:null, method: null});
									}, 0);
								}
							}

						}
					}
				}
				if(values.attributes_meta) {
					for (var i in values.attributes_meta) {
						this.addLineAdjustment(line, {type: 'ProductAttributes', related_type: 'ProductAttributes', 'name': i });
					}
				}
				if(! renderGroup && ! custom_redraw) {
					//this.redraw_line_item(group_id, new_line_id);
					//if(line.parent_id)
					//	this.redraw_line_item(group_id, line.parent_id);
					renderGroup = true;
				}

				if(values.related_type == 'Assemblies' || values.related_type == 'SupportedAssemblies') {
					line.tax_class_id = '-99';
					this.clearAssemblyComponents(group_id, new_line_id);
					this.addAssemblyComponents(group_id, new_line_id, values.related_id, values.related_type, this.groups[group_id].group_type == 'support',
						function() {
							if(editor.add_assembly_description && ! blank(values.description_plain))
								editor.addLineItem(group_id, { related_type: 'Notes', parent_id: new_line_id, body: values.description_plain }, true);
							editor.recalcGroup(group_id);
							editor.renderGroup(group_id);
							editor.renderTotals();
						});
				}
				else if(values.related_type == 'ProductCatalog') {
					if(this.add_product_description && ! blank(values.description_plain)) {
						var grp = this.groups[group_id];
						if(grp && grp.lines && ! grp.lines[new_line_id].parent_id) {
							this.addLineItem(group_id, { related_type: 'Notes', parent_id: new_line_id, body: values.description_plain });
						}
					}

				}
				if(custom_redraw !== 1)
					this.recalcGroup(group_id);
				if(renderGroup || custom_redraw === 2) {
					this.renderGroup(group_id);
					this.renderTotals();
					this.focusLineItem(group_id, new_line_id);
				}
				else if(! custom_redraw) {
					this.renderGroupFooter(group_id);
					this.renderTotals();
				}
			}
			UI.markFormDirty(this);
		},

		returnPopupLineItem: function(key, values, parms, custom_redraw) {
			var line_id = get_default(parms.line_id, '');
			if(! values) {
				if(! line_id) return;
				values = {};
			}
			values.related_type = parms.module;
			if(parms.parent_id)
				values.parent_id = parms.parent_id;
			this.loadLineItem(values, function(line) {
				this.returnLineItem(parms.group_id, line_id, line, custom_redraw);
			});
		},

		returnPopupLineItems: function(lines, parms, custom_redraw) {
			if(! lines) return;
			var line_id = get_default(parms.line_id, ''),
				inited = [];
			for(var i = 0; i < lines.length; i++) {
				var values = lines[i];
				if(! values) continue;
				values.related_type = parms.module;
				if(parms.parent_id)
					values.parent_id = parms.parent_id;
				inited.push(values);
			}
			this.loadLineItems(inited, function(lines) {
				for(var i = 0; i < lines.length; i++)
					this.returnLineItem(parms.group_id, line_id, lines[i], i == lines.length-1 ? custom_redraw : 1);
			});
		},

		updateLineItem: function(group, line_id, field, value, recalc) {
			if(! (group = this.getGroup(group))) return;
			var line = group.lines[line_id];
			if(! line) return;
			if(field === 'quantity') {
				if(line.quantity)
					line.ext_quantity = line.ext_quantity / line.quantity * value;
				else
					line.ext_quantity = value; // recalc fixes assembly components
				this.updateExtPrice(line);
			}
			else if(field === 'adjust_price') {
				var old_pmethod = this.getLinePricingMethod(group, line);
				if(old_pmethod === '') {
					field = 'unit_price';
					value = trunc(value + line.unit_price, 5);
				}
				else
					field = null;
			}
			if(field)
				line[field] = value;
			var pmethod = this.getLinePricingMethod(group, line);
			if(field === 'unit_price') {
				if(pmethod === '')
					line.custom_unit_price = line.unit_price;
				this.updateExtPrice(line);
			}
			else if(field === 'tax_class_id' && line.sum_of_components) {
				// update tax class of assembly components
				var parent_id = line.id;
				this.updateGroupRows(group, function(line) {
					if(line.parent_id && line.parent_id === parent_id)
						line.tax_class_id = value;
				});
			}
			if(pmethod !== '') {
				var fn = this.getPricingMethodFn(group, line);
				if(fn) fn(line);
			}
			if(recalc) {
				this.recalcGroup(group);
				this.renderGroup(group);
				this.renderTotals();
			}
			UI.markFormDirty(this);
		},

		removeLineItem: function(group, line_id) {
			if(! (group = this.getGroup(group))) return;
			var dropidx = [];
			for(var idx = 0; idx < group.lines_order.length; idx++) {
				var iid = group.lines_order[idx];
				if(iid === line_id)
					dropidx.push(idx);
				else {
					var line = group.lines[iid];
					if(line && line.parent_id == line_id)
						dropidx.push(idx);
				}
			}
			for(var idx = dropidx.length-1; idx >= 0; idx--) {
				this.clearLineInputs(group, line_id);
				group.lines_order.splice(dropidx[idx], 1);
			}
			this.recalcGroup(group);
			this.renderGroup(group);
			this.renderTotals();
			UI.markFormDirty(this);
			return true;
		},

		needProductAttrs: function(line) {
			return line && line.related_type == 'ProductCatalog'
					&& (this.module == 'Quotes' || this.module == 'SalesOrders' || this.module == 'Invoice' || this.is_outgoing)
					&& line.related_id && ! isset(line.attributes_meta);
		},

		getProductAttrs: function(ids, callback) {
			var fetch = [], req, result = {};
			if(! this.attrs_cache) {
				this.attrs_cache = {};
			}
			if(ids && ids.unshift) {
				for(var i = 0; i < ids.length; i++) {
					if(! ids[i]) continue;
					if(this.attrs_cache[ids[i]])
						result[ids[i]] = this.attrs_cache[ids[i]];
					else
						fetch.push(ids[i]);
				}
			}
			else if(ids) {
				if(this.attrs_cache[ids])
					result = this.attrs_cache[ids];
				else
					fetch.push(ids);
			}

			if(callback && fetch.length) {
				params = {record_ids: fetch.join(','), module: 'ProductAttributes'};
				req = new SUGAR.conn.JSONRequest('get_attributes_for_product', {}, params);
				req.fetch(function(data) {
					if(data && data.result && data.result.length) {
						var rows = data.result,
							decimals = this.currencyDecimalPlaces(),
							currency_id = this.getCurrencyId(),
							conversion_rate = this.getExchangeRate();
						for(var ind = 0; ind < rows.length; ind++) {
							if(! isPlainObject(rows[ind]) || ! rows[ind].parent_id)
								continue;
							var instance = deep_clone(rows[ind]),
								key = instance.name,
								pid = instance.parent_id;
							delete instance.parent_id;
							//if(pid) key += ':'+pid;
							var line_currency_id = get_default(instance.currency_id, '-99');
							instance.price_usdollar = parseFloat(instance.price_usdollar || 0);
							if(line_currency_id == currency_id)
								instance.price = trunc(parseFloat(instance.price || 0), decimals);
							else
								instance.price = trunc(instance.price_usdollar * conversion_rate, decimals);
							if(! this.attrs_cache[pid])
								this.attrs_cache[pid] = {};
							if(! this.attrs_cache[pid][key])
								this.attrs_cache[pid][key] = [];
							this.attrs_cache[pid][key].push(instance);
						}
					}
					for(var i = 0; i < fetch.length; i++) {
						if(this.attrs_cache[fetch[i]])
							result[fetch[i]] = this.attrs_cache[fetch[i]];
					}
					callback.call(this, result);
				}.bind(this));
			} else {
				if(callback) callback.call(this, result);
			}
			return result;
		},

		returnLineEvent: function(group, line_id, values) {
			var line = this.getLineItem(group, line_id);
			if(! line) return;
			if(! values) values = {name: '', id: '', date_start: ''};
			line.event_session_name = values.name;
			line.event_session_id = values.id;
			line.event_session_date = values.date_start;
			if(! line.event_session_id) {
				delete line._show_event;
			}
			this.renderGroup(group);
			this.renderTotals();
		},

		addAssemblyComponents: function(group_id, line_id, related_id, related_type, support, after) {
			if(! related_id) {
				if(after) after();
				return;
			}
			var is_supp_asm = (related_type == 'SupportedAssemblies'),
				req = new SUGAR.conn.JSONRequest('get_product_list', {}, {
					module: related_type,
					record: related_id,
					display_currency_id: this.getCurrencyId(),
					pricebook_id: this.getPricebookId(),
					fields: this.getFieldMap(support ? 'Assets' :  'ProductCatalog', support)}
				),
				editor = this;
			req.fetch(function(data) {
				var result, prod, i, discount, lines = [];
				if(data && (result = data.getResult()) && result.fields) {
					for(i = 0; i < result.fields.length; i++) {
						prod = deep_clone(result.fields[i]);
						prod.parent_id = line_id;
						prod.depth = 1;

						// assembly component discount
						var rawval = parseFloat(get_default(prod.raw_unit_price, '0.00')),
							linecur = prod.currency_id || '-99';
						if(! is_supp_asm && ! support) {
							if (prod.discount_type == 'fixed') {
								discount = parseFloat(prod.discount_value);
								if(linecur != '-99')
									discount /= parseFloat(get_default(prod.exchange_rate, '1.00'));
								prod.raw_unit_price = rawval - discount;
							} else {
								discount = parseFloat(prod.discount_value);
								prod.raw_unit_price = rawval * (100 - discount) / 100;
							}
						}

						lines.push(prod);
					}
				}
				editor.loadLineItems(lines, function(lines) {
					for(var i = 0; i < lines.length; i++)
						this.addLineItem(group_id, lines[i], true);
					if(after)
						after();
				});
			});
		},

		clearAssemblyComponents: function(group_id, line_id) {
			var grp = this.groups[group_id];
			if(! grp) return;
			for(var idx = grp.lines_order.length - 1; idx >= 0; idx --) {
				if(grp.lines[grp.lines_order[idx]].parent_id == line_id) {
					delete grp.lines[grp.lines_order[idx]];
					grp.lines_order.splice(idx, 1);
				}
			}
		},

		moveLineItem: function(group, line_id, offset, check_only) {
			if(! (group = this.getGroup(group))) return;
			var line = group.lines[line_id];
			if(! line) return false;
			var startpos = -1;
			for(var idx = 0; idx < group.lines_order.length; idx++) {
				if(group.lines_order[idx] == line_id) {
					startpos = idx;
					break;
				}
			}
			if(startpos < 0)
				return false;
			var numchild = 0;
			for(var cpos = startpos + 1; cpos < group.lines_order.length; cpos++) {
				if(group.lines[group.lines_order[cpos]].depth <= line.depth)
					break;
				numchild ++;
			}
			function fix_range(x, min, max) {
				return Math.max(0, Math.min(group.lines_order.length - 1, x));
			}
			var nextpos = fix_range(startpos + offset + (offset > 0 ? numchild : 0));
			if(group.lines[group.lines_order[nextpos]].depth < line.depth || startpos == nextpos)
				return false;
			if (check_only)
				return true;
			while(offset < 0 && nextpos > 0
				&& group.lines[group.lines_order[nextpos]].depth > line.depth)
					nextpos --;
			var next = group.lines[group.lines_order[nextpos]];
			var nextchild = 0;
			for(var cpos = nextpos + 1; cpos < group.lines_order.length; cpos++) {
				if(group.lines[group.lines_order[cpos]].depth <= next.depth)
					break;
				nextchild ++;
			}
			var movefrom, moveto, movecount;
			if(offset > 0) {
				movefrom = nextpos; moveto = startpos; movecount = nextchild + 1;
			}
			else {
				movefrom = startpos; moveto = nextpos; movecount = numchild + 1;
			}
			var cut = group.lines_order.splice(movefrom, movecount);
			for(var idx = cut.length - 1; idx >= 0; idx--)
				group.lines_order.splice(moveto, 0, cut[idx]);
			this.renderGroup(group);
			this.renderTotals();
		},


		// -- Calculations --

		recalculate: function(no_redraw) {
			for(var idx = 0; idx < this.groups_order.length; idx++) {
				var gid = this.groups_order[idx];
				this.recalcGroup(gid);
				if(! no_redraw)
					this.renderGroup(gid);
			}
			if(! no_redraw)
				this.renderTotals();
		},

		/*
			Note: when the form is not editable, this method only updates
			group.cost (which is not saved)
		*/
		recalcGroup: function(group) {
			var discount_before_taxes = this.getDiscountBeforeTaxes();
			if(! (group = this.getGroup(group))) return;
			group.cost = 0.0;
			var tally = [];
			var all_taxcodes = {};

			var adj_ord = [];
			var taxes_ord = [];
			var taxes = {};
			var old_taxes = {};
			var rates = [];
			var parents = [];

			var editableRates = {};

			// iterate lines
			var decimals = this.currencyDecimalPlaces();
			for(var idx = 0; idx < group.lines_order.length; idx++) {
				var line = group.lines[group.lines_order[idx]];
				if(! line.is_comment) {
					if(this.editable) {
						if(line.sum_of_components) {
							for(var fdx = 0; fdx < this.price_columns.length; fdx++)
								line[this.price_columns[fdx]] = 0.0;
						}
						if(line.parent_id && group.lines[line.parent_id]) {
							var parent = group.lines[line.parent_id];
							line.ext_quantity = trunc(parent.quantity * line.quantity, 2);
						}
						else
							line.ext_quantity = line.quantity;
						line.unit_price = trunc(line.unit_price, 5);
						line.discount = 0.0;
						this.updateExtPrice(line);
						if(! line.sum_of_components) {
							tally.push({
								unadj_amount: line.ext_price,
								amount: line.ext_price,
								taxcode: line.tax_class_id,
								id: line.id}
							);
							if(line.tax_class_id && line.tax_class_id != '-99')
								all_taxcodes[line.tax_class_id] = 1;
						}


						for(var jdx in line.adjusts) {
							var adj = line.adjusts[jdx];
							//if (adj.line_id) continue;
							if(adj.type == 'StandardTax' || adj.type == 'CompoundedTax') {
								if (SysData.taxrates[adj.related_id]) {
									var r = deep_clone(SysData.taxrates[adj.related_id]);
									var type = r.compounding != 'compounded' ? 'StandardTax' : 'CompoundedTax';
									adj.amount = 0.0;
									r.editable = 1;
									r.line_id = line.id;
									rates.push(r);
								}
							}
						}
					}
					if(! line.sum_of_components)
						group.cost += trunc(line.ext_quantity * line.cost_price, decimals);
				}
				parents.push(line.id);
			}
			for(var idx = 0; idx < group.discounts_order.length; idx++) {
				var adj = group.adjusts[group.discounts_order[idx]];
				if(adj.tax_class_id && adj.tax_class_id != '-99') {
					all_taxcodes[adj.tax_class_id] = 1;
				}
			}

			if(group.shipping_adj_id && isset(group.adjusts[group.shipping_adj_id])) {
				var shipping = group.adjusts[group.shipping_adj_id];
				if (shipping.tax_class_id && shipping.tax_class_id != '-99') {
					all_taxcodes[shipping.tax_class_id] = 1;
				}
			}

			if(! this.editable) {
				// quick calc pretax amount
				var pretax = group.subtotal + (group.shipping_tax_class_id != '-99' ? group.shipping : 0.0);
				for(var idx = 0; idx < group.discounts_order.length; idx++) {
					var adj = group.adjusts[group.discounts_order[idx]];
					pretax -= adj.amount;
				}
				group.pretax = pretax;
			}

			if(this.editable) {
				var discount_taxes = {};
				// calculate group subtotal
				group.subtotal = 0.0;
				for(var idx = 0; idx < tally.length; idx++) {
					group.subtotal += tally[idx].amount;
				}
				// apply discounts to line items and sum total discount
				var disc_total = 0.0;
				for(var idx = 0; idx < group.discounts_order.length; idx++) {
					var adj = group.adjusts[group.discounts_order[idx]];
					if(adj.type != 'StdFixedDiscount') {
						adj.amount = 0.0;
						for(var jdx = 0; jdx < tally.length; jdx++) {
							var disc = trunc(tally[jdx].unadj_amount * adj.rate / 100, decimals);
							tally[jdx].amount -= disc;
							if(tally[jdx].id) {
								group.lines[tally[jdx].id].discount += disc;
								group.lines[tally[jdx].id].net_price = group.lines[tally[jdx].id].ext_price - group.lines[tally[jdx].id].discount;
							}
							adj.amount += disc;
						}
					} else {
						var disc = adj.amount;
						if (tally.length > 0) tally[0].amount -= disc;//because we need subtract fixed discount one time
					}
					if (adj.tax_class_id && adj.tax_class_id != '-99') {
						var r = SysData.get_tax_rates(adj.tax_class_id);
						for(var k = 0; k < r.length; k++) {
							var rate = r[k];
							discount_taxes[rate.id] =
								get_default(discount_taxes[rate.id], 0.0) - adj.amount * rate.rate / 100.0;
						}
					}
					disc_total += adj.amount;
				}

				group.total = group.pretax = group.subtotal - disc_total;

				// update assembly prices
				for(var idx = group.lines_order.length - 1; idx >= 0; idx--) {
					var line_id = group.lines_order[idx];
					var line = group.lines[line_id];
					if(line.parent_id && isset(group.lines[line.parent_id]) && ! line.is_comment) {
						var parent = group.lines[line.parent_id];
						if(parent.sum_of_components) {
							for(var fdx = 0; fdx < this.price_columns.length; fdx++) {
								var col = this.price_columns[fdx];
								if(col == 'discount')
									parent[col] += line[col];
								else if(col != 'ext_price' && col != 'net_price')
									parent[col] += trunc(line[col] * (line.ext_quantity / parent.ext_quantity), decimals);
							}
							parent.ext_price = trunc(parent.ext_quantity * parent.unit_price, decimals);
							parent.net_price = trunc(parent.ext_price - parent.discount, decimals);
						}
					}
				}

				// add shipping
				if(group.shipping_adj_id && isset(group.adjusts[group.shipping_adj_id])) {
					var shipping = group.adjusts[group.shipping_adj_id];
					var shipping_taxed = (shipping.type == 'TaxedShipping');
					group.total += shipping.amount;
					if(group.shipping_tax_class_id !== '-99')
						group.pretax += shipping.amount;
					if(shipping_taxed) {
						tally.push({amount: shipping.amount, unadj_amount: shipping.amount, taxcode: shipping.tax_class_id, is_shipping: true});
					}
				}
				// find applicable taxes
				for(var code_id in all_taxcodes) {
					var r = SysData.get_tax_rates(code_id);
					for(var idx = 0; idx < r.length; idx++) {
						if (!rates.find(function(rate) {
							return rate.id == r[idx].id;
						})) {
							var rr = deep_clone(r[idx]);
							rates.push(rr);
						}
					}
				}
				// update taxes
				for(var idx = 0; idx < group.adjusts_order.length; idx++) {
					var adj = group.adjusts[group.adjusts_order[idx]];
					//if (adj.line_id) continue;
					if(adj.type == 'StandardTax' || adj.type == 'CompoundedTax') {
						var taxuid = adj.type + '~' + adj.related_id + '~' + adj.rate;
						if (!adj.editable) {
							if (!old_taxes[taxuid]) {
								old_taxes[taxuid] = adj;
							}
							delete group.adjusts[adj.id];
						} else {
							if (!editableRates[taxuid]) {
								editableRates[taxuid] = true;
								var existing = rates.findIndex(function(rate) {
									return rate.id == adj.related_id;
								});
								if (SysData.taxrates[adj.related_id]) {
									var r = deep_clone(SysData.taxrates[adj.related_id]);
									var type = r.compounding != 'compounded' ? 'StandardTax' : 'CompoundedTax';
									r.editable = 1;
									r.uid = taxuid;
									rates.push(r);
									if (~existing) {
										rates.splice(existing, 1);
									}
								}
								old_taxes[taxuid] = adj;
							}
							delete group.adjusts[adj.id];
						}
					}
					else
						adj_ord.push(adj.id);
				}
				// calculate totals
				for(var idx = 0; idx < rates.length; idx++) {
					var rate = rates[idx];
					var type = rate.compounding != 'compounded' ? 'StandardTax' : 'CompoundedTax';
					var taxuid = rate.uid || type +'~'+ rate.id +'~'+ rate.rate;
					var adj = null;
					if(! isset(taxes[taxuid])) {
						if(isset(old_taxes[taxuid])) {
							adj = old_taxes[taxuid];
							adj.amount = 0.0;
						}
						else {
							adj = {
								type: type,
								name: rate.name,
								related_type: 'TaxRates',
								related_id: rate.id,
								rate: rate.rate,
								amount: get_default(discount_taxes[rate.id], 0.0),
								editable: 0
							};
							adj.id = 'newadj~' + (uniqid ++);
						}
						taxes[taxuid] = adj.id;
						taxes_ord.push(adj.id);
						adj_ord.push(adj.id);
						group.adjusts[adj.id] = adj;
					}
					else
						adj = group.adjusts[taxes[taxuid]];
					for(var jdx = 0; jdx < tally.length; jdx++) {

						var line = tally[jdx];
						if (!line.taxcode || line.taxcode == "-99") continue;
						if (line.taxcode != 'all') {
							var code = SysData.taxcodes[line.taxcode];
							if (!code) continue;

							var applicable = false;
							switch (code.taxation_scheme) {
								case 'list':
									applicable = !!~code.rates.findIndex(function(r) {
										return r.id == rate.id;
									});
									applicable = applicable || ( !!rate.editable && !line.is_shipping);
									break;
								case 'tax':
									applicable = !!rate.editable;
									break;
								case 'nontax':
									break;
							}
							if (!applicable) continue;
						}


						var base, prevtax = get_default(line.tax, 0.0);
						if (discount_before_taxes) {
							base = line.unadj_amount;
						} else {
							base = line.amount;
						}
						if(adj.type == 'CompoundedTax')
							base += prevtax;

						var amt = base * rate.rate / 100;
						line.tax = prevtax + amt;
						adj.amount += amt;
						/*
						if (code && code.taxation_scheme == 'tax' && rate.editable) {
							if (!rate.line_id || tally[jdx].id == rate.line_id) {
								var base, prevtax = get_default(tally[jdx].tax, 0.0);
								if (discount_before_taxes) {
									base = tally[jdx].unadj_amount;
								} else {
									base = tally[jdx].amount;
								}
								if(adj.type == 'CompoundedTax')
									base += prevtax;
								var amt = base * rate.rate / 100;
								tally[jdx].tax = prevtax + amt;
								adj.amount += amt;
							}
						}
						*/
					}
				}
				for(var taxuid in taxes) {
					var adj = group.adjusts[taxes[taxuid]];
					adj.amount = trunc(adj.amount, decimals);
					group.total += adj.amount;
				}
				group.total = trunc(group.total, decimals);
				group.adjusts_order = adj_ord;
				group.taxes_order = taxes_ord;
			}
			/*if(redraw) {
				this.renderGroupFooter(group);
				this.renderTotals();
			}*/

			if(this.debug) {
				console.log('recalc');
				this.debugShowCalc(group);
			}
		},

		debugShowCalc: function(grp) {
			for(var idx = 0; idx < grp.lines_order.length; idx++) {
				var line_id = grp.lines_order[idx];
				var line = grp.lines[line_id];
				if(line.is_comment) continue;
				var pfx = (line.parent_id) ? ' -> ' : '';
				var pcs = [];
				for(var pdx = 0; pdx < this.price_columns.length; pdx++) {
					var col = this.price_columns[pdx];
					pcs.push(line[col]);
				}
				console.log(pfx + line.name, pcs);
			}
		},

		recalcTotals: function() {
			var cost = 0.0,
				subt = 0.0,
				tax = 0.0,
				disc = 0.0,
				ship = 0.0,
				idx, grp;

			var mil_inv_totals = {
			    total: 0.0,
                tax: 0.0,
                shipping: 0.0
            };

            for(idx = 0; idx < this.groups_order.length; idx++) {
				grp = this.groups[this.groups_order[idx]];
				if(! grp) continue;

				//Cacl Milestone Invoices totals
				if(this.isMilestoneInvoicesGroup(grp)) {
                    this.updateMilestoneInvoicesTotals(grp, mil_inv_totals);
				    continue;
                }

				cost += grp.cost;
                subt += grp.subtotal;

				for(var jdx = 0; jdx < grp.taxes_order.length; jdx++) {
					var tot = grp.adjusts[grp.taxes_order[jdx]];
					tax += tot.amount;
				}
				for(var jdx = 0; jdx < grp.discounts_order.length; jdx++) {
					var tot = grp.adjusts[grp.discounts_order[jdx]];
					disc += tot.amount;
				}
				if(grp.shipping_adj_id && grp.adjusts[grp.shipping_adj_id])
					ship += grp.adjusts[grp.shipping_adj_id].amount;
			}

			this.totals = {
				cost: cost,
				subtotal: subt,
				taxes: tax,
				discounts: disc,
				shipping: ship,
				pretax: subt - disc + ship,
				total: subt - disc + ship + tax + mil_inv_totals.total
			};

            this.addMilestoneInvoicesToTotals(mil_inv_totals);
		},

        addMilestoneInvoicesToTotals: function(mil_inv_totals) {
		    if (mil_inv_totals.total !== 0.0) {
		        this.totals.milestone_invoices_total = mil_inv_totals.total;
            }

            if (mil_inv_totals.tax !== 0.0) {
                this.totals.milestone_invoices_tax = mil_inv_totals.tax;
                this.totals.milestone_invoices_tax_remaining = this.totals.taxes - (mil_inv_totals.tax * -1);
            }

            if (mil_inv_totals.shipping !== 0.0) {
                this.totals.milestone_invoices_shipping = mil_inv_totals.shipping;
                this.totals.milestone_invoices_shipping_remaining = this.totals.shipping - (mil_inv_totals.shipping * -1);
            }
        },

        isMilestoneInvoicesGroup: function(grp) {
		    var result = false;

		    //If at least one line in the group is Invoice so it's Milestone Invoices group
            if(grp.lines_order.length > 0) {
                for(var idx = 0; idx < grp.lines_order.length; idx++) {
                    var line_id = grp.lines_order[idx];
                    var line = grp.lines[line_id];

                    if (line.related_type == 'Invoice') {
                        result = true;
                        break;
                    }
                }
            }

            return result;
        },

        updateMilestoneInvoicesTotals: function(grp, totals) {
            totals.total += grp.total;

            for(var idx = 0; idx < grp.lines_order.length; idx++) {
                var line_id = grp.lines_order[idx];
                var line = grp.lines[line_id];

                if (line.related_type == 'ShippingProviders') {
                    totals.shipping += line.unit_price;
                } else if (line.related_type == 'TaxRates') {
                    totals.tax += line.unit_price;
                }
            }
        },

        updateAllRows: function(func) {
			for(var idx = 0; idx < this.groups_order.length; idx++) {
				this.updateGroupRows(this.groups_order[idx], func);
			}
		},

		updateGroupRows: function(group, func) {
			if(! (group = this.getGroup(group)) ) return;
			// update non-assembly lines
			for(var idx = 0; idx < group.lines_order.length; idx++) {
				var line_id = group.lines_order[idx];
				var line = group.lines[line_id];
				var fnc = func || this.getPricingMethodFn(group, line);
				if(!fnc) continue;
				try {
					fnc.call(this, line, group);
				} catch(exception) {
					if(! this.warned){
						console.error(exception);
						alert('error: ' + exception.trace);
						this.warned = true;
					}
					return false;
				}
			}
			this.recalcGroup(group);
			this.renderGroup(group);
			this.renderTotals();
		},

		updateExtPrice: function(line) {
			var value = line.ext_quantity * line.unit_price;
			var minDec = this.currencyDecimalPlaces();
			line.ext_price = trunc(value, minDec);
			line.net_price = line.ext_price - line.discount;
		},

		getPricingMethodFn: function(group, ln) {
			var calc = this.getPricingFormula(group, ln);
			if(calc) {
				return function(line) {
					console.log(line);
					var attr_adjust = 0.0,
						i, adj, upd, minDec, maxDec;
					if(! calc.no_attrs && line.adjusts) {
						for(i in line.adjusts) {
							adj = line.adjusts[i];
							if(i != line.pricing_adjust_id && adj.type !== 'StdFixedDiscount')
								attr_adjust += parseFloat(adj.amount);
						}
					}
					upd = calc(line);
					if(isset(upd)) {
						upd += attr_adjust;
						minDec = this.currencyDecimalPlaces();
						maxDec = isset(calc.base) ? Math.max(countDecimals(line[calc.base]), minDec) : undefined;
						upd = restrictDecimals(upd, minDec, maxDec);
						line.unit_price = upd;
						this.updateExtPrice(line);
					}
				}.bind(this);
			}
		},

		updateGroupPricingMethod: function(group, newvals) {
			if(newvals) {
				if( (group = this.getGroup(group)) ) {
					group.pricing_method = (newvals.method === null)
						? this.group_template.pricing_method
						: newvals.method;
					group.pricing_percentage = newvals.type === "StdFixedDiscount" ? newvals.amount : newvals.rate;
				}
			}
			this.updateGroupRows(group);
		},

		updateExchangeRate: function(old_rate, new_rate) {
			var price_cols = this.price_columns;
			var upd_pricing = [];
			var decimals = this.currencyDecimalPlaces();
			this.updateAllRows(
				function(line, group_id) {
					for(var idx = 0; idx < price_cols.length; idx++) {
						var col = price_cols[idx];
						if(isset(line[col])) {
							line[col] = trunc(line[col] / old_rate * new_rate, decimals);
						}
					}
					if (line.attributes_meta) for (var i in line.attributes_meta) {
						var attr = line.attributes_meta[i];
						for (var j in attr) {
							line.attributes_meta[i][j].price = trunc(line.attributes_meta[i][j].price / old_rate * new_rate, decimals);
						}
					}
					for (var i in line.adjusts) {
						var adj = line.adjusts[i];
						if (adj.type != 'ProductAttributes' && i != line.pricing_adjust_id) {
							continue;
						}
						var amount = trunc(adj.amount / old_rate * new_rate, decimals);
						this.updateLineAdjustment(group_id, line.id, i, {'amount': amount}, true);
					}
				});
			for(var idx = 0; idx < this.groups_order.length; idx++) {
				var gid = this.groups_order[idx];
				var shipping = this.groups[gid].shipping / old_rate * new_rate;
				this.updateGroupField(gid, 'shipping', shipping, true);
			}
			//for(var idx = 0; idx < upd_pricing.length; idx++)
			//	editor.updateGroupPricingMethod(upd_pricing[idx]);
		},

		getLinePricing: function(group, line, inherit, create) {
			group = this.getGroup(group);
			if(group && typeof(line) === 'string')
				line = group.lines[line];
			if(! group || ! line) return;
			var adj;
			if(line.is_comment)
				return;
			if(line.depth && line.parent_id) {
				var parent = group.lines[line.parent_id];
				if(parent && parent.pricing_adjust_id) {
					return this.getLinePricing(group, parent, inherit);
				}
			}
			if(line.pricing_adjust_id)
				adj = line.adjusts[line.pricing_adjust_id];
			if(! adj && create && (! line.sum_of_components || this.add_assembly_discounts) && ! line.is_comment) {
				adj = this.addLineAdjustment(line, {type: 'Inherit'});
				line.pricing_adjust_id = adj.id;
			}
			if(inherit && (! adj || adj.type === 'Inherit')) {
				adj = {
					type: get_default(UI.TallyEditor.pricing_method_map[group.pricing_method], 'Fixed'),
					rate: group.pricing_percentage,
					inherited: true
				};
			} else if(adj) {
				adj = deep_clone(adj);
			}
			if(adj) {
				var method = '';
				switch (adj.type) {
					case 'PercentDiscount': method = 'discount'; break;
					case 'StdPercentDiscount': method = 'stddiscount'; break;
					case 'StdFixedDiscount': method = 'stdfixeddiscount'; break;
					case 'Fixed': method = 'fixed'; break;
					case 'Markup': method = 'markup'; break;
					case 'Margin': method = 'margin'; break;
					case 'SameAsList': method = 'list'; break;
				}
				adj.method = method;
				// allow custom pricing when no catalog item is selected
				if(adj.method === 'fixed' && ! this.catalog_only && ! line.related_id)
					adj.method = '';
			}
			return adj;
		},

		getLinePricingMethod: function(group, line, noconvert) {
			var pricing = this.getLinePricing(group, line, !noconvert);
			return pricing && (noconvert ? pricing.type : pricing.method);
		},

		getPricingFormula: function(group, ln) {
			group = this.getGroup(group);
			if(! group || ! ln || ln.is_comment) return null;
			var method = this.getLinePricingMethod(group, ln);
			var adj = ln.pricing_adjust_id && ln.adjusts[ln.pricing_adjust_id];
			if(! adj && ln.parent_id && group.lines[ln.parent_id].pricing_adjust_id) {
				adj = group.lines[ln.parent_id].adjusts[group.lines[ln.parent_id].pricing_adjust_id];
			}
			if (method != 'inherit' && adj) {
				if (adj.type == 'StdFixedDiscount') {
					var perc = adj.amount;
				} else {
					var perc = adj.rate;
				}
			} else {
				method = group.pricing_method;
				var perc = group.pricing_percentage;
			}


			if(isNaN(perc))
				perc = 0.0;
			perc = parseFloat(perc);

			var fn = null;
			switch(method) {
				case 'fixed':
					fn = function(line) { return line.std_unit_price; };
					fn.base = 'std_unit_price';
					break;
				case 'margin':
					perc = 100 - perc;
					fn = function(line) { return line.cost_price * 100 / perc; };
					fn.base = 'cost_price';
					break;
				case 'markup':
					perc += 100;
					fn = function(line) { return line.cost_price * perc / 100; };
					fn.base = 'cost_price';
					break;
				case 'discount':
				case 'stddiscount':
					perc = 100 - perc;
					var base_field = 'list_price';
					fn = function(line) {
						if (line.discount_id && line.raw_unit_price) {
							base_field = 'raw_unit_price';
							return line.raw_unit_price * perc / 100;
						} else {
							return line.list_price * perc / 100;
						}
					};
					fn.base = base_field;
					break;
				case 'stdfixeddiscount':
					var base_field = 'list_price';
					fn = function(line) {
						if (line.discount_id && line.raw_unit_price) {
							base_field = 'raw_unit_price';
							return line.raw_unit_price - perc;
						} else {
							return line.list_price - perc;
						}
					};
					fn.base = base_field;
					break;
				case 'list':
					fn = function(line) { return line.list_price; };
					fn.base = 'list_price';
					break;
				case '':
					fn = function(line) { return line.unit_price; };
					fn.base = 'unit_price';
					break;
				default:
					fn = function(line) { return isset(line.custom_unit_price) ? line.custom_unit_price : line.std_unit_price; };
					fn.no_attrs = true;
			}
			if(fn)
				fn.method = method;
			return fn;
		},

		updateLinePricing: function(group, line_id, params) {
			var line = this.getLineItem(group, line_id);
			if(line && params.tax_code_id) {
				line.tax_class_id = params.tax_code_id;
			}
			delete params.tax_code_id;
			this.updatePriceAdjustment(group, line_id, params);
			this.focusLineItem(group, line_id, false, line.pricing_adjust_id ? 'pricing' : true);
		},


		// -- Group/line adjustment management --

		addAdjustment: function(group, spec, no_redraw) {
			var adj_id = 'newadj~' + (uniqid ++),
				order;
			if(! (group = this.getGroup(group)) || ! isset(spec)) return;
			if(spec.related_type == 'Discounts') {
				order = group.discounts_order;
			}
			else if(spec.related_type == 'TaxRates') {
				order = group.taxes_order;
			}
			else if(spec.related_type == 'ShippingProviders') {
				// no order
			}
			else
				return;
			if(! isPlainObject(spec))
				spec = {};
			spec.id = adj_id;
			if(! isset(spec.rate))
				spec.rate = 0.0;
			if(! isset(spec.amount))
				spec.amount = 0.0;

			// TODO check if this required
			/*
			if(spec.related_type == 'Discounts' && this.getDiscountBeforeTaxes()) {
				this.iterate_group_lines(group_id,
					function(group_id, line_id, line) {
						if (line.is_comment) return false;
						spec.tax_class_id = get_default(line.tax_class_id, -99);
						return true;
					}
				);
			}
			*/
			group.adjusts[adj_id] = spec;

			if(order)
				order.push(adj_id);
			group.adjusts_order.push(adj_id);
			if(! no_redraw) {
				this.renderGroupFooter(group);
				this.renderTotals();
			}
			return adj_id;
		},

		iterAdjustments: function(callback, group) {
			var idx = 0;
			group = this.getGroup(group);
			if(group && group.adjusts_order) {
				var ord = group.adjusts_order;
				for(; idx < ord.length; idx++) {
					if(callback.call(this, ord[idx], group.adjusts[ord[idx]], idx))
						break;
				}
			}
			return idx;
		},

		updateAdjustment: function(group, adj_id, spec) {
			if(! (group = this.getGroup(group)) ) return;
			var adj = group.adjusts[adj_id];
			if(! adj) return;
			for(var idx in spec) {
				if(idx == 'id')
					adj.related_id = spec.id;
				else
					adj[idx] = spec[idx];
			}
			if (adj.related_type == 'TaxRates') {
				adj.type = adj.compounding ? 'CompoundedTax' : 'StandardTax';
			}
			adj.rate = parseFloat(adj.rate);
			adj.amount = parseFloat(adj.amount);
			this.recalcGroup(group);
			this.renderGroupFooter(group);
			this.renderTotals();
		},

		removeAdjustment: function(group, adj_id, norecalc) {
			if(! (group = this.getGroup(group)) ) return;
			var adj = group.adjusts[adj_id];
			if(! adj) return;
			if(adj.related_type == 'Discounts')
				group.discounts_order.remove(adj_id);
			else if(adj.related_type == 'TaxRates')
				group.taxes_order.remove(adj_id);
			group.adjusts_order.remove(adj_id);
			delete group.adjusts[adj_id];
			setTimeout(function() {
				this.clearLineInputs(group, adj_id);
				if(! norecalc) {
					this.recalcGroup(group);
					this.renderGroupFooter(group);
					this.renderTotals();
				}
			}.bind(this), 0);
		},

		addLineAdjustment: function(line, template) {
			if (!line.adjusts) line.adjusts = { };
			var adj_id = 'newadj~' + (uniqid ++);
			if(! isPlainObject(template))
				template = {};
			template.id = adj_id;
			if(! isset(template.rate))
				template.rate = 0.0;
			if(! isset(template.amount))
				template.amount = 0.0;
			line.adjusts[adj_id] = template;
			return line.adjusts[adj_id];
		},

		removeLineAdjustment: function(group, line_id, adj_id) {
			if(! (group = this.getGroup(group)) ) return;
			var line = group.lines[line_id];
			if(! line) return;
			var adj = line.adjusts[adj_id];
			if(! adj) return;
			delete line.adjusts[adj_id];
			this.recalcGroup(group);
			this.renderGroup(group);
			this.renderTotals();
		},

		getDiscountAdjustment: function(disc_id) {
			var disc = SysData.discounts[disc_id];
			var adj = {type: 'StdPercentDiscount', related_type: 'Discounts', related_id: disc_id, amount: 0.0};
			if(disc) {
				adj.rate = parseFloat(disc.rate);
				adj.amount = parseFloat(disc.amount);
				adj.type = disc.type;
				adj.name = disc.name;
			}
			return adj;
		},

		updateDiscountTax: function(group, adj_id, value) {
			if(! (group = this.getGroup(group)) ) return;
			var adj = group.adjusts[adj_id];
			if(! adj) return;
			adj.tax_class_id = value;
			this.recalcGroup(group);
			this.renderGroup(group);
			this.renderTotals();
		},

		getDiscountBeforeTaxes: function() {
			var discount_before_taxes = UI.getFormInput(this.form, 'discount_before_taxes');
			if (discount_before_taxes && discount_before_taxes.getValue())
				return true;
			return false;
		},

		updatePriceAdjustment: function(group, line_id, spec, no_group_update) {
			var line = this.getLineItem(group, line_id);
			if (!line) return;
			if(spec && spec.type === null) {
				if(line.pricing_adjust_id) {
					this.removeAdjustment(group, line.pricing_adjust_id, true);
					line.pricing_adjust_id = null;
				}
			} else {
				this.updateLineAdjustment(group, line_id, line.pricing_adjust_id, spec, no_group_update);
			}
			var pmethod = this.getLinePricingMethod(group, line);
			if(pmethod === '' && isset(line.custom_unit_price))
				line.unit_price = line.custom_unit_price;
			if(! no_group_update)
				this.updateGroupPricingMethod(group);
		},

		updateLineAdjustment: function(group, line_id, adj_id, spec, no_group_update) {
			var line = this.getLineItem(group, line_id);
			var adj = line.adjusts[adj_id];
			if(! adj) return;
			var adj_amount = null;
			for(var idx in spec) {
				if(idx === 'id')
					adj.related_id = spec.id;
				else if(idx === 'method' || idx === 'inherited')
					; // skip
				else {
					if(idx === 'amount') {
						spec.amount = parseFloat(spec.amount);
						adj_amount = spec.amount - get_default(adj.amount, 0);
					}
					adj[idx] = spec[idx];
				}
			}
			adj.rate = parseFloat(adj.rate);
			if(isset(adj_amount) && ! this.is_outgoing)
				this.updateLineItem(group, line_id, 'adjust_price', adj_amount);
			else
				this.updateLineItem(group, line_id);
			if(! no_group_update) {
				this.recalcGroup(group);
				this.renderGroup(group);
				this.renderTotals();
			}
		},

		/*addGroupDiscount: function(group_id, parent_id) {
			var grp = this.groups[group_id];
			var line = grp.lines[parent_id];
			if(! line.pricing_adjust_id) {
				var adj = this.addLineAdjustment(line, {type: 'StdPercentDiscount'});
				line.pricing_adjust_id = adj.id;
				for(var i = 0; i < grp.lines_order.length; i++) {
					var sub = grp.lines[grp.lines_order[i]];
					if(sub.parent_id == line.id && sub.pricing_adjust_id) {
						delete sub.adjusts[sub.pricing_adjust_id];
						sub.pricing_adjust_id = null;
					}
				}
				this.recalculate(true);
				this.renderGroup(group_id);
				this.renderTotals();
				return adj;
			}
		},*/

		setDefaultDiscount: function(discount_id) {
			var last_default = this.default_discount_id;
			this.default_discount_id = discount_id;
			var update_groups = true;
			var adjs = {};
			// update existing groups only if they have no discounts, or only the last default discount
			for(var idx = 0; idx < this.groups_order.length; idx++) {
				var grp = this.groups[this.groups_order[idx]];
				for(var jdx = 0; jdx < grp.discounts_order.length; jdx++) {
					var adj = grp.adjusts[grp.discounts_order[jdx]];
					if(adj.related_id && adj.related_id != last_default) {
						update_groups = false;
						break;
					}
					if(adjs[this.groups_order[idx]]) {
						update_groups = false;
					}
					adjs[this.groups_order[idx]] = grp.discounts_order[jdx];
				}
				if(! update_groups)
					break;
			}
			if(! update_groups)
				return;
			var disc = SysData.discounts[discount_id] || {};
			if (!disc.applies_to_selected) {
				for(var idx = 0; idx < this.groups_order.length; idx++) {
					var upd_adj = this.getDiscountAdjustment(discount_id);
					if(adjs[this.groups_order[idx]])
						this.updateAdjustment(this.groups_order[idx], adjs[this.groups_order[idx]], upd_adj);
					else
						this.addAdjustment(this.groups_order[idx], upd_adj, true);
				}
			}
			this.recalculate();
		},


		// Rendering

		render: function() {
			Wrap(this.elt).empty();
			// this.renderHeader();
			this.iterGroups(function(idx, grp) {
				this.renderGroup(idx, true);
			});
			this.renderFooter();
			this.renderTotals();
			this.rendered = true;
		},

		checkPageSize: function() {
			var sz = SUGAR.util.getPageSize();
			this.narrow = (sz === 'xs');
		},

		handleResize: function() {
			var oldNarrow = this.narrow;
			this.checkPageSize();
			if(oldNarrow !== this.narrow && this.rendered)
				this.render();
		},

		destroy: function() {
			// remove event handlers?
			this.rendered = false;
		},

		renderGroup: function(group_id, insert) {
			var grp = typeof(group_id) === 'string' ? this.getGroup(group_id) : group_id,
				inputs = this.getGroupInputs(group_id),
				panel;
			if(! grp) return;
			if(! inputs.outer) {
				inputs.panel = new UI.PanelRender({});
				inputs.outer = MakeElt('div.row.form-row',
					MakeElt('div.column.form-cell',
						MakeElt('div.form-entry',
							MakeElt('div.form-value', inputs.panel.render()))))
					.appendTo(this.elt);
			}
			inputs.panel.elt.toggleClass('panel-readonly', ! this.editable);
			inputs.panel.elt.toggleClass('tally-narrow', this.narrow);
			inputs.panel.elt.toggleClass('tally-normal', ! this.narrow);
			panel = inputs.panel;
			panel.clear();
			var layout = this.getLineLayout(grp);
			inputs.header = panel.addHeader(),
			inputs.columns = panel.addSubheader(),
			inputs.body = panel.addBody({className: 'tally-items-body', pad: true}),
			inputs.footer = panel.addFooter();
			this.renderGroupHeader(grp);
			if(this.renderLines(grp, layout) && ! this.narrow)
				this.renderLineHeader(grp, layout);
			else
				inputs.columns.hide();
			this.renderGroupFooter(grp);
			if(insert)
				inputs.outer.appendTo(this.elt);
		},

		getGroupHeaderLayout: function(grp) {
			var ret = [];
			var toprow = [];
			toprow.push({name: 'name', label: 'LBL_GROUP_NAME'});
			if(this.module === 'Quotes')
				toprow.push({name: 'status', label: 'LBL_GROUP_STAGE'});
			if((!this.is_outgoing || this.module === 'Bills' || this.module === 'CreditNotes') &&
					this.module !== 'Shipping' && this.module !== 'Receiving') {
				toprow.push({name: 'type', label: 'LBL_GROUP_TYPE'});
			}
			if(this.module !== 'Bills') {
				toprow.push({name: 'pricing', label: 'LBL_PRICING'});
			}
			if(this.narrow && toprow.length > 2) {
				ret.push(toprow.slice(0, 2));
				ret[0].push({name: 'move'});
				toprow = toprow.slice(2);
			} else {
				toprow.push({name: 'move'});
			}
			ret.push(toprow);
			return ret;
		},

		renderGroupHeader: function(group) {
			group = this.getGroup(group);
			var elt = this.getGroupInputs(group).header,
				layout = this.getGroupHeaderLayout(group);
			if(! elt) return;
			elt.empty();
			for(var idx = 0; idx < layout.length; idx++) {
				var columns = layout[idx],
					row = MakeElt('div.row.form-row').appendTo(elt);
				for(var jdx = 0; jdx < columns.length; jdx++) {
					var column = columns[jdx],
						label = MakeElt('div.form-label', mod_string(column.label, this.module)),
						val = MakeElt('div.form-value'),
						cell = MakeElt('div.column.form-cell',
							MakeElt('div.form-entry', label, val)).appendTo(row);
					this.renderGroupHeaderColumn(group, column, val, elt);
				}
			}
		},

		getGroupInputs: function(group) {
			if(typeof(group) === 'string')
				group = this.getGroup(group);
			if(! group) return;
			if(! group.inputs) group.inputs = {};
			return group.inputs;
		},

		clearGroupInputs: function(group) {
			if(typeof(group) === 'string')
				group = this.getGroup(group);
			if(! group) return;
			if(group.inputs) {
				for(k in group.inputs) {
					var inp = group.inputs[k];
					if(inp && inp.destroy) inp.destroy();
				}
				delete group.inputs;
			}
			if(group.lines_order) {
				for(var idx = 0; idx < group.lines_order.length; idx++)
					this.clearLineInputs(group, group.lines_order[idx]);
			}
			if(group.adjusts_order) {
				for(var idx = 0; idx < group.adjusts_order.length; idx++)
					this.clearLineInputs(group, group.adjusts_order[idx]);
			}
		},

		renderGroupHeaderColumn: function(grp, column, elt, outer) {
			var inputs = this.getGroupInputs(grp),
				fid = this.id + '-' + column.name + '-' + grp.id,
				group_id = grp.id,
				editor = this;
			if(column.name === 'name') {
				if(this.editable) {
					var txt = inputs.name;
					if(! txt) {
						txt = inputs.name = new UI.TextInput(fid, {
							block: true,
							flat: true,
							ignore_dirty: true,
							placeholder: app_string('LBL_NONE')
						});
						txt.onchange = function(evt) { editor.updateGroupField(group_id, 'name', this.getValue()); }
						txt.render();
						UI.registerInput(this.form, txt);
					}
					elt.append(txt.render());
					txt.setValue(grp.name, true);
				} else {
					var name = trim(grp.name);
					if(name.length)
						MakeElt('span.input-field-pad', name).appendTo(elt);
					else
						MakeElt('span.input-field-pad.text-muted', app_string('LBL_NONE')).appendTo(elt);
				}
			}
			else if(column.name === 'status') {
				if(this.editable) {
					var sel = inputs.status;
					if(! sel) {
						sel = inputs.status = new UI.SelectInput(fid, {
							flat: true, ignore_dirty: true, block: true,
							options: 'quote_stage_dom'
						});
						sel.onchange = function(k, v) { editor.updateGroupField(group_id, 'status', k); }
						sel.render();
						UI.registerInput(this.form, sel);
					}
					elt.append(sel.render());
					sel.setValue(grp.status, true);
				} else {
					var status = html_escape(SUGAR.language.getListString('quote_stage_dom', grp.status));
					if(! status) status = '&mdash;';
					MakeElt('span.input-field-pad').html(status).appendTo(elt);
				}
			}
			else if(column.name === 'type') {
				if(this.editable) {
					var sel = inputs.type;
					if(! sel) {
						sel = inputs.type = new UI.SelectInput(fid, {
							flat: true, ignore_dirty: true, block: true,
							options: 'quote_group_type_dom'
						});
						sel.onchange = function(k, v) {
							if(editor.groups[group_id].lines_order.length > 0) {
								if(! confirm(mod_string('LBL_CONFIRM_GROUP_TYPE'))) {
									this.setValue(editor.groups[group_id].group_type, true);
									return;
								}
							}
							editor.updateGroupField(group_id, 'group_type', k);
						}
						sel.render();
						UI.registerInput(this.form, sel);
					}
					elt.append(sel.render());
					sel.setValue(grp.group_type, true);
				} else {
					var type = html_escape(SUGAR.language.getListString('quote_group_type_dom', grp.group_type));
					MakeElt('span.input-field-pad').html(type || '&mdash;').appendTo(elt);
				}
			}
			else if(column.name === 'pricing') {
				if(this.editable) {
					var sel = inputs.pricing;
					if(! sel) {
						sel = inputs.pricing = new PricingInput(fid, {
							block: true,
							flat: true,
							ignore_dirty: true,
							mark_blank: false,
							standard_discounts: false,
							manual_discounts: this.manual_discounts,
							group_level: true,
						});
						sel.onchange = function() { editor.updateGroupPricingMethod(group_id, this.values); }
						sel.render();
						UI.registerInput(this.form, sel);
					}
					elt.append(sel.render());
					//sel.update({value: {method: grp.pricing_method, rate: grp.pricing_percentage}, silent: true});
				} else {
					var method = html_escape(SUGAR.language.getListString('quote_pricing_method_short_dom', grp.pricing_method));
					MakeElt('span.input-field-pad').html(method || '&mdash;').appendTo(elt);
				}
			}
			else if(column.name === 'move') {
				elt.css({textAlign : 'right', display: 'block'});
				if(this.editable) {
					var isFirst = this.groups_order.indexOf(group_id) === 0;
					var isLast = this.groups_order.lastIndexOf(group_id) === this.groups_order.length - 1;
					var buttons = [];
					if (!isFirst)
						buttons.push(['⬆', -1]);
					if (!isLast)
						buttons.push(['⬇', 1]);

					buttons.forEach(function(b) {
						(function(b) {
							var e = MakeElt('span', b[0]).css({cursor: 'pointer'});
							e.on('click', function() { editor.moveGroup(group_id, b[1]) });
							elt.append(e);
						})(b);
					});
				} else {
				}
			}
		},

		getLineLayout: function(grp, parent_item, child_count) {
			var ret = [], top = {columns: []}, subname, label;

			if(grp.group_type === 'service')
				label = 'LBL_BOOKING_CATEGORY';
			else if(grp.group_type == 'expenses')
				label = 'LBL_EXPENSE_CATEGORY';
			else
				label = parent_item ? 'LBL_COMPONENT_PRODUCT' : 'LBL_PRODUCT_OR_ASSEMBLY';
			var name_rows = {name: 'name-rows', rows: [{columns: [{name: 'name', label: label}]}]};

			if(grp.group_type === 'expenses') {
				name_rows.rows.push({columns: [
					{name: 'description', label: 'LBL_EXPENSE_DESCRIPTION'}
				]});
			} else if(grp.group_type === 'support') {
				name_rows.rows.push({
					columns: [{name: 'serialno', label: 'LBL_SERIAL_NO'}],
					check: function(line) {
						return (line.related_type == 'Assets' || line.related_type == 'SupportedAssemblies');
					}
				});
			} else if(grp.group_type === 'products' && this.add_serials == 1) {
				name_rows.rows.push({
					columns: [{name: 'serialno', label: 'LBL_SERIAL_NO'}],
					check: function(line) {
						return (line.related_type == 'ProductCatalog');
					}
				});
			}
			top.columns.push(name_rows);
			ret.push(top);

			var pricing_row = {
				columns: [{name: 'pricing', label: 'LBL_PRICING'}],
				check: function(line) {
					return !! line.pricing_adjust_id;
				}};

			if(this.narrow) {
				if(grp.group_type !== 'expenses')
					top.columns.push({name: 'qty', label: 'LBL_QUANTITY', align: 'right'});
				if(! this.is_outgoing) {
					if(this.product_costs && grp.group_type !== 'service')
						pricing_row.columns.push({name: 'cost_price', label: 'LBL_COST', align: 'right'});
					pricing_row.columns.push({name: 'list_price', label: 'LBL_LIST_PRICE', align: 'right'});
				}
				ret.push(pricing_row);
				var totals = {columns: []};
				totals.columns.push({name: 'taxcode', label: 'LBL_TAX_CODE'});
				totals.columns.push({name: 'unit_price', label: 'LBL_UNIT_PRICE', align: 'right'});
				totals.columns.push({name: 'ext_price', label: 'LBL_EXT_PRICE', align: 'right'});
				ret.push(totals);
			} else {
				if(this.module === 'Shipping' || this.module === 'Receiving') {
					top.columns.push({name: 'qty', label: 'LBL_QUANTITY', align: 'right'});
					top.columns.push({name: 'mfr_part_no', label: 'LBL_PARTNO'});
					top.columns.push({name: 'vendor_part_no', label: 'LBL_VENDOR_PARTNO'});
				} else {
					var rest = {columns: []}, rest_rows = [rest];
					rest.columns.push({name: 'taxcode', label: 'LBL_TAX_CODE'});
					if(grp.group_type !== 'expenses')
						rest.columns.push({name: 'qty', label: 'LBL_QUANTITY', align: 'right'});

					var prices = {columns: []}, price_rows = [prices];
					if(! this.is_outgoing) {
						if(this.product_costs && grp.group_type !== 'service')
							prices.columns.push({name: 'cost_price', label: 'LBL_COST', align: 'right'});
						prices.columns.push({name: 'list_price', label: 'LBL_LIST_PRICE', align: 'right'});
					}
					prices.columns.push({name: 'unit_price', label: 'LBL_UNIT_PRICE', align: 'right'});
					prices.columns.push({name: 'ext_price', label: 'LBL_EXT_PRICE', align: 'right'});
					if(this.show_net_prices && ! this.is_outgoing) {
						prices.columns.push({name: 'net_price', label: 'LBL_NET_PRICE', align: 'right'});
					}
					rest.columns.push({name: 'prices', rows: price_rows});
					rest_rows.push(pricing_row);
					top.columns.push({name: 'rest', rows: rest_rows});

					rest_rows.push({
						columns: [{name: 'attribute', label: 'LBL_ATTRIBUTE'}],
						check: function(line) {
							return hasProductAttributes(line);
						}
					});

					if(this.module === 'Invoice' && this.can_add_related_events) {
						rest_rows.push({
							columns: [{name: 'event', label: 'LBL_EVENT'}],
							check: function(line) {
								return line.event_session_id || line._show_event;
							}});
					}
				}
			}

			return ret;
		},

		renderLineHeader: function(grp, layout) {
			var elt = this.getGroupInputs(grp).columns;
			if(! elt) return;
			elt.empty();
			var row = MakeElt('div.row.col-space-sm'),
				outer = MakeElt('div.row.tally-row-head.panel-column-titles',
					MakeElt('div.column', row)).appendTo(elt),
				assemble = function(cols) {
					var ret = [], col;
					for(var idx = 0; idx < cols.length; idx++) {
						col = cols[idx];
						if(col.rows) {
							sub = assemble(col.rows[0].columns || []);
							if(sub.length === 1) {
								sub[0] = deep_clone(sub[0]);
								sub[0].name = col.name;
							}
							ret.pushArray(sub);
						} else
							ret.push(col);
					}
					return ret;
				},
				columns = assemble(layout[0].columns);
			for(var idx = 0; idx < columns.length; idx++) {
				var col = columns[idx],
					cell = MakeElt('div.column.tally-col-' + col.name);
				if(col.align) cell.addClass('text-' + col.align);
				if(this.price_columns.indexOf(col.name) >= 0)
					cell.addClass('tally-col-price');
				cell.append(MakeElt('span.column-title', mod_string(col.label, this.module)));
				if(col.name === 'name-rows') {
					var cell_name = MakeElt('div.column.complex.tally-col-name-outer');
					var row_name = MakeElt('div.row.col-space-sm').appendTo(cell_name);
					if(this.editable && ! this.narrow)
						MakeElt('div.column.tally-col-pos').appendTo(row_name);
					row_name.append(cell);
					cell = cell_name;
				}
				cell.appendTo(row);
			}
			if(this.editable) {
				MakeElt('div.column.tally-col-end').appendTo(outer);
			}
		},

		canAdjustPricing: function(grp, line) {
			var adj_pricing = grp.group_type !== 'service'
					&& !this.isPO && this.module !== 'Shipping' && this.module !== 'Receiving';
			if(this.add_assembly_discounts) {
				adj_pricing = adj_pricing && ! line.depth;
			} else {
				adj_pricing = adj_pricing && ! line.sum_of_components;
			}
			return adj_pricing;
		},

		canEditPartNumber: function(grp, line) {
			if(line.is_comment)
				return false;
			if(this.module === 'Shipping' || this.module === 'Receiving')
				// separate field
				return false;
			if(grp.group_type === 'service' || grp.group_type === 'expenses')
				return false;
			if(this.catalog_only)
				return false;
			return true;
		},

		editPartNumber: function(grp, line, evt) {
			var inputs = this.getLineInputs(grp, line),
				editor = this;
			if(inputs && inputs.name) {
				var edit = new UI.QuickText({
					size: 36,
					format: 'number'});
				edit.init(line.mfr_part_no, function() {
					editor.updateLineItem(grp.id, line.id, 'mfr_part_no', this.getValue(), true);
				});
				edit.showPopup(0, inputs.name.elt);
				UI.stopEvent(evt);
			}
		},

		editProductName: function(grp, line, evt) {
			var inputs = this.getLineInputs(grp, line),
				editor = this;
			if(inputs && inputs.name) {
				var edit = new UI.QuickText({
					size: 36,
					format: 'number'});
				edit.init(line.name, function() {
					editor.updateLineItem(grp.id, line.id, 'name', this.getValue(), true);
				});
				edit.showPopup(0, inputs.name.elt);
				UI.stopEvent(evt);
			}
		},

		getLineMenuOptions: function(grp, line) {
			if(line.is_comment)
				return;
			var opts = {keys: [], values: [], width: 200};
			if(this.canAdjustPricing(grp, line)) {
				opts.keys.push('pricing');
				opts.values.push({
					label: mod_string('LBL_PRICING_EDIT', this.module),
					perform: function() {
						this.getLinePricing(grp, line, true, true); // creates adjustment
						this.renderGroup(grp);
						this.focusLineItem(grp, line, true, 'pricing');
					}.bind(this)
				});
			}
			if (line.sum_of_components) {
				var subitem_type = line.related_type == 'Assemblies' ? 'ProductCatalog' : 'Assets';
				opts.keys.push('addproduct');
				opts.values.push({
					label: mod_string('LBL_ADD_COMPONENT', this.module),
					perform: function() {
						this.addLineItem(grp, { related_type: subitem_type, parent_id: line.id }, false, 'open');
					}.bind(this)
				});
				opts.keys.push('addnote');
				opts.values.push({
					label: mod_string('LBL_ADD_COMMENT', this.module),
					perform: function() {
						this.addLineItem(grp, { related_type: 'Notes', parent_id: line.id }, false, true);
					}.bind(this)
				});
			}
			if(this.module === 'Invoice' && this.can_add_related_events) {
				opts.keys.push('addevent');
				opts.values.push({
					label: mod_string('LBL_EVENT_ADJUST', this.module),
					perform: function() {
						line._show_event = true;
						this.renderGroup(grp);
						this.focusLineItem(grp, line, true, 'event');
					}.bind(this)
				});
			}
			if(this.canEditPartNumber(grp, line)) {
				opts.keys.push('partno');
				opts.values.push({
					label: mod_string('LBL_EDIT_PART_NUMBER'),
					perform: function() {
						this.editPartNumber(grp, line);
					}.bind(this)
				});
			}
			opts.keys.push('prodname');
			opts.values.push({
				label: mod_string('LBL_EDIT_PROD_NAME'),
				perform: function() {
					this.editProductName(grp, line);
				}.bind(this)
			});
			opts.keys.push('duplicate');
			opts.values.push({
				label: app_string('LBL_DUPLICATE_BUTTON_LABEL'),
				perform: function() {
					this.duplicateLineItem(grp, line.id);
				}.bind(this)
			});
			if(this.narrow) {
				opts.keys.push('remove');
				opts.values.push({
					label: mod_string('LBL_REMOVE_ROW', this.module),
					perform: function() {
						this.removeLineItem(grp, line.id);
					}.bind(this)
				});
			}
			return opts;
		},

		renderLines: function(grp, layout) {
			var elt = this.getGroupInputs(grp).body;
			if(! elt) return;
			var root = new TreeNode({root: true}), assemblyTree, node;
			grp.dragTree = root;
			elt.empty();
			var count = this.iterLineItems(function(line_id, line, row_idx) {
				if (line.sum_of_components) {
					node = assemblyTree = new TreeNode({line_id: line.id, sum_of_components: true});
					root.appendChild(assemblyTree);
				} else if (line.parent_id && assemblyTree) {
					node = new TreeNode({line_id: line.id});
					assemblyTree.appendChild(node);
				} else {
					node = new TreeNode({line_id: line.id});
					root.appendChild(node);
					assemblyTree = null;
				}
				var outer_row = MakeElt('div.row.tally-row-outer.row-space-sm').appendTo(elt),
					dragger;
				if(line.depth) {
					outer_row.addClass('tally-row-depth-1');
				}
				var pos_col;
				if(this.editable && ! this.narrow) {
					pos_col = MakeElt('div.column.tally-col-pos')
						.append(dragger = MakeElt('div.uii.uii-lg.uii-hamburger.drag-handle.active-icon'));
					var callbacks = {
						beforeStart: function(evt) {
							this.drag_index = row_idx;
							this.drag_row = outer_row;
							this.setDragData(evt);
						}.bind(this),

						afterEnd: function(evt) {
							delete this.drag_index;
							delete this.drag_row;
							UI.removeClass(elt, 'drag-remove');
							UI.clearDragData(evt);
						}.bind(this),

						onState: function(evt, args, extra_args) {
							UI.addRemoveClass(elt, 'drag-remove', (args && args[0] === 'accept'));
						}.bind(this)
					};
					var config = this.dragConfig(dragger);
					config.tallyInfo = {group : grp, line : line, node: node};
					new UI.Draggable(outer_row, config);
					new UI.DragTarget(outer_row, config);
				}
				var outer = MakeElt('div.column').appendTo(outer_row),
					renderRow = function(row_layout, elt) {
						if(row_layout.check && ! row_layout.check(line))
							return;
						var row = MakeElt('div.row.col-space-sm.row-space-sm.tally-row-data'),
							columns = row_layout.columns || [];
						if(line.is_comment) row.addClass('tally-row-comment');
						for(var idx = 0; idx < columns.length; idx++) {
							var col = columns[idx];
							if(line.is_comment && col.name !== 'name' && col.name !== 'name-rows') {
								continue;
							}
							var cell = MakeElt('div.column.tally-col-' + col.name);
							if(this.price_columns.indexOf(col.name) >= 0)
								cell.addClass('tally-col-price');
							if(col.rows) {
								cell.addClass('complex');
								for(var jdx = 0; jdx < col.rows.length; jdx++)
									renderRow.call(this, col.rows[jdx], cell);
							}
							else {
								if(col.align) cell.addClass('text-' + col.align);
								this.renderLineColumn(grp, line, col, cell);
							}
							if(col.name === 'name-rows') {
								var cell_name = MakeElt('div.column.complex.tally-col-name-outer');
								var row_name = MakeElt('div.row.col-space-sm').appendTo(cell_name);
								if(line.depth && ! this.narrow)
									row_name.append(MakeElt('div.column.tally-col-depth-1'));
								if(pos_col)
									row_name.append(pos_col);
								row_name.append(cell);
								cell = cell_name;
							}
							cell.appendTo(row);
						}
						row.appendTo(elt);
					};
				for(var idx = 0; idx < layout.length; idx++)
					renderRow.call(this, layout[idx], outer);
				if(this.editable) {
					var col_end = MakeElt('div.column.tally-col-end'),
						opts = this.getLineMenuOptions(grp, line),
						dd, menu;
					if(opts && opts.keys.length) {
						dd = MakeElt('div.uii.uii-bdown.uii-lg.active-icon', {tabIndex: 0})
							.appendTo(col_end);
						menu = new UI.MenuSource(dd.get(), {
							options: opts
						});
						menu.setup();
					}
					if(! this.narrow) {
						MakeElt('div.uii.uii-remove.uii-lg.active-icon', {tabIndex: 0})
							.on('click, keydown[key=enter|space]', function() {
								//if(confirm(mod_string('NTC_CONFIRM_REMOVE_ROW', this.module)))
								this.removeLineItem(grp.id, line.id);
							}, {scope: this}).appendTo(col_end);
					}
					col_end.appendTo(outer_row);
				}
			}, grp);
			if(! count) {
				var outer = MakeElt('div.row.row-space-sm.tally-row-data').appendTo(elt);
				outer.append(MakeElt('div.column.tally-col-blank',
					MakeElt('span.input-field-pad.text-help', mod_string('LBL_NO_LINE_ITEMS', this.module))));
				var config = this.dragConfig();
				config.tallyInfo = {group : grp};
				new UI.DragTarget(elt, config);
			}
			return count;
		},

		getLineInputs: function(group, line, adj) {
			if(! (group = this.getGroup(group)) ) return;
			if(! group.line_inputs) group.line_inputs = {};
			if(typeof(line) === 'string') {
				line = adj ? group.adjusts[line] : group.lines[line];
			}
			if(! line || ! line.id) return;
			if(! group.line_inputs[line.id])
				group.line_inputs[line.id] = {};
			return group.line_inputs[line.id];
		},

		clearLineInputs: function(group, line_id) {
			if(! (group = this.getGroup(group))) return;
			if(! group.line_inputs || ! group.line_inputs[line_id]) return;
			var inputs = group.line_inputs[line_id], k, inp;
			for(k in inputs) {
				inp = inputs[k];
				if(inp && inp.destroy) inp.destroy();
			}
			delete group.line_inputs[line_id];
		},

		renderLineColumn: function(grp, line, column, cell) {
			var inputs = this.getLineInputs(grp, line),
				group_id = grp.id,
				line_id = line.id,
				fid = this.id + '-' + column.name + '-' + group_id + '-' + line_id,
				editor = this;
			if(column.name === 'name') {
				if(line.is_comment) {
					if(this.editable) {
						var txt = inputs.name;
						if(! txt) {
							txt = inputs.name = new UI.TextAreaInput(fid, {
								block: true,
								ignore_dirty: true,
								rows: 2
							});
							txt.onchange = function(evt) { editor.updateLineItem(group_id, line_id, 'body', this.getValue()); }
							txt.render();
							UI.registerInput(this.form, txt);
						}
						MakeElt('div.input-field.input-field-group.input-block',
							MakeElt('div.input-label.input-field-icon',
								MakeElt('div.input-icon.theme-icon.module-Notes')),
							txt.render()).appendTo(cell);
						txt.setValue(line.body, true);
					} else {
						MakeElt('div.tools-row.input-field-pad.align-top',
							MakeElt('div.no-flex',
								MakeElt('div.input-icon.theme-icon.module-Notes.left')),
							MakeElt('span', line.body).css({"white-space": 'pre-wrap'})).css({"word-break": 'break-word'}).appendTo(cell);
					}
				}
				else {
					if(this.editable) {
						var rel = inputs.name;
						if(! rel) {
							rel = inputs.name = new LineItemInput(fid, {
								allow_custom: ! this.catalog_only,
								allow_rename: ! this.catalog_only,
								block: true,
								ignore_dirty: true,
								module: line.related_type,
								onchange: this.returnPopupLineItem.bind(this),
								onrename: renameLineItem.bind(this)
							});
							if(this.is_outgoing)
								rel.addFilter({param: 'supplier', field_name: 'supplier'});
							if(grp.group_type === 'expenses')
								rel.addFilter({param: 'booking_class', value: 'expenses'});
							rel.addFilter({param: 'display_currency_id', field_name: 'currency_id'});
							rel.addFilter({param: 'pricebook', field_name: 'pricebook'});
							rel.render();
							UI.registerInput(this.form, rel);
						}
						rel.placeholder = '(' + SUGAR.language.getListString('moduleListSingular', line.related_type) + ')';
						rel.extra_fields = this.getFieldMap(line.related_type, grp.group_type === 'support');
						if(this.module != 'Shipping' && this.module != 'Receiving')
							rel.part_number = line.mfr_part_no;
						rel.popup_passthru = {
							group_id: group_id,
							line_id: line_id,
							module: line.related_type,
							pricebook_id: this.getPricebookId()
						};
						cell.append(rel.render());
						rel.update(line.related_id, line.name, true);
					} else {
						MakeElt('div.tools-row.input-field-pad.align-top',
							MakeElt('div.no-flex',
								MakeElt('div.input-icon.theme-icon.module-' + line.related_type + '.left')),
							UI.makeRecordLink(line.related_type, line.related_id, line.name)).appendTo(cell);
					}
				}
			}
			else if(column.name === 'pricing') {
				if(this.narrow) {
					cell.append(MakeElt('p.form-label', mod_string(column.label, this.module)));
				}
				var rel = inputs.pricing;
				if(! rel || rel.narrow != this.narrow) {
					if(rel) rel.destroy();
					rel = inputs.pricing = new PricingInput(fid, {
						block: this.narrow,
						flat: ! this.narrow,
						ignore_dirty: true,
						narrow: this.narrow,
						standard_discounts: this.standard_discounts,
						manual_discounts: this.manual_discounts,
						line: line,
					});
					rel.onchange = function() { editor.updateLinePricing(group_id, line_id, this.values); }
					rel.render();
					UI.registerInput(this.form, rel);
				}
				rel.update({value: this.getLinePricing(group_id, line, true), silent: true});
				if(this.editable) {
					if(this.narrow)
						cell.append(rel.render());
					else
						cell.append(
							MakeElt('div.tools-row.input-field-pad',
								MakeElt('div.form-label.no-flex', mod_string('LBL_PRICING') + app_string('LBL_SEPARATOR'), nbsp()),
								rel.render()
							));
				} else {
					if(this.narrow)
						cell.append(MakeElt('small', rel.getLabel()));
					else
						cell.append(
							MakeElt('div.tools-row.input-field-pad',
								MakeElt('div.form-label.no-flex', mod_string('LBL_PRICING') + app_string('LBL_SEPARATOR'), nbsp()),
								MakeElt('small', rel.getLabel())
							));
				}
			}
			else if(column.name === 'taxcode') {
				if(this.narrow) {
					cell.append(MakeElt('p.form-label', mod_string(column.label, this.module)));
				}
				if(this.editable) {
					var sel = inputs.taxcode;
					if(! sel) {
						sel = inputs.taxcode = new UI.SelectInput(fid, {
							block: true,
							ignore_dirty: true,
							options: this.getTaxCodeOptions(),
							label_key: 'code'
						});
						sel.onchange = function() { editor.updateLineItem(group_id, line_id, 'tax_class_id', this.getValue(), true); }
						sel.render();
						UI.registerInput(this.form, sel);
					}
					cell.append(sel.render());
					sel.setValue(line.tax_class_id || '-99', true);
				} else {
					var code = SysData.taxcodes[line.tax_class_id || '-99'];
					cell.append(MakeElt('span.input-field-pad')
							.html(code && code.id !== '-99' ? code.code : '&mdash;'));
				}
			}
			else if(column.name === 'qty') {
				if(this.editable) {
					var qty = inputs.qty;
					if(! qty) {
						qty = inputs.quantity = new UI.TextInput(fid, {
							block: true,
							format: 'float',
							min_decimals: 0,
							decimals: 2,
							default_num_value: 1,
							size: 5,
							align: 'right',
							ignore_dirty: true
						});
						qty.onchange = function(evt) { editor.updateLineItem(group_id, line_id, 'quantity', this.getValue(true), true); }
						qty.render();
						UI.registerInput(this.form, qty);
					}
					cell.append(qty.render());
					qty.setValue(line.quantity, true);
				} else {
					cell.append(MakeElt('span.text-number.input-field-pad', stdFormatNumber(line.quantity || 0)));
				}
			}
			else if(column.name === 'mfr_part_no' || column.name === 'vendor_part_no') {
				var nm = column.name;
				if(this.editable) {
					var txt = inputs[nm];
					if(! txt) {
						txt = inputs[nm] = new UI.TextInput(fid, {
							block: true,
							format: 'number',
							ignore_dirty: true
						});
						txt.onchange = function(evt) { editor.updateLineItem(group_id, line_id, nm, this.getValue()); }
						txt.render();
						UI.registerInput(this.form, txt);
					}
					cell.append(txt.render());
					txt.setValue(line[nm], true);
				} else if(trim(line[nm]).length) {
					cell.append(MakeElt('span.input-field-pad.text-number', line[nm]));
				}
			}
			else if(column.name === 'description') {
				if(this.editable) {
					var txt = inputs.description;
					if(! txt) {
						txt = inputs.description = new UI.TextInput(fid, {
							block: true,
							ignore_dirty: true,
							placeholder: mod_string('LBL_DESCRIPTION_PLACEHOLDER', this.module)
						});
						txt.onchange = function(evt) { editor.updateLineItem(group_id, line_id, 'description', this.getValue()); }
						txt.render();
						UI.registerInput(this.form, txt);
					}
					cell.append(txt.render());
					txt.setValue(line.description, true);
				} else if(trim(line.description).length) {
					cell.append(MakeElt('span.input-field-pad', line.description));
				}
			}
			else if(column.name === 'serialno') {
				if(this.editable) {
					var txt = inputs.serial_no;
					if(! txt) {
						txt = inputs.serial_no = new UI.TextInput(fid, {
							block: true,
							ignore_dirty: true,
							placeholder: mod_string('LBL_SERIAL_PLACEHOLDER', this.module)
						});
						txt.onchange = function(evt) { editor.updateLineItem(group_id, line_id, 'serial_no', this.getValue()); }
						txt.render();
						UI.registerInput(this.form, txt);
					}
					cell.append(txt.render());
					txt.setValue(line.serial_no, true);
				} else if(trim(line.serial_no).length) {
					cell.append(MakeElt('span.input-field-pad', line.serial_no));
				}
			}
			else if(column.name === 'event') {
				if(this.editable) {
					var rel = inputs.event;
					if(! rel) {
						rel = inputs.event = new UI.RefInput(fid, {
							block: true,
							ignore_dirty: true,
							module: 'EventSessions',
							extra_fields: {
								id: 'id',
								name: 'name',
								date_start: 'date_start'
							},
							onchange: function(k, value) { editor.returnLineEvent(group_id, line_id, value); }
						});
						rel.render();
						UI.registerInput(this.form, rel);
					}
					cell.append(
						MakeElt('div.tools-row.input-field-pad',
							MakeElt('div.form-label.no-flex', mod_string('LBL_EVENT') + app_string('LBL_SEPARATOR'), nbsp()),
							rel.render()
						));
					rel.update(line.event_session_id, line.event_session_name, true);
				} else if(trim(line.event_session_name).length) {
					cell.append(
						MakeElt('div.tools-row.input-field-pad',
							MakeElt('div.form-label.no-flex', mod_string('LBL_EVENT') + app_string('LBL_SEPARATOR'), nbsp()),
							MakeElt('div.input-icon.theme-icon.module-EventSessions.left.no-flex'),
							UI.makeRecordLink('EventSessions', line.event_session_id, line.event_session_name).addClass('small')
						));
				}
			}
			else if(column.name === 'attribute') {
				this.renderProductAttributes(grp, line, cell);
			}
			else if(this.price_columns.indexOf(column.name) >= 0) {
				if(this.narrow) {
					cell.append(MakeElt('p.form-label', mod_string(column.label, this.module)));
				}
				if(this.editable) {
					var prc = inputs[column.name];
					if(! prc) {
						prc = inputs[column.name] = new UI.TextInput(fid, {
							format: 'currency',
							currency_input: this.currency_input,
							size: 12,
							align: 'right',
							ignore_dirty: true
						});
						prc.onchange = function(evt) { editor.updateLineItem(group_id, line_id, column.name, this.getValue(true), true); }
						prc.render();
						UI.registerInput(this.form, prc);
					}
					var readOnly = false,
						method = this.getLinePricingMethod(grp, line);
					if(column.name === 'ext_price' || column.name === 'net_price')
						readOnly = true;
					else if(grp.group_type !== 'expenses') {
						if((line.related_id || this.catalog_only) && this.fixed_prices
								&& (column.name === 'cost_price' || column.name === 'list_price'))
							readOnly = true;
						else if(column.name === 'unit_price') {
							if(line.sum_of_components || this.catalog_only)
								readOnly = true;
							else if(method !== '')
								readOnly = true;
						}
					}
					prc.setReadOnly(readOnly);
					cell.append(prc.render());
					prc.setValue(line[column.name], true);
				} else {
					var span = MakeElt('span.text-number.input-field-pad').appendTo(cell),
						val = line[column.name];
					if(val) span.append(this.formatCurrency(val));
					else span.html('&mdash;');
				}
			}
		},

		renderProductAttributes: function(grp, line, cell) {
			var attrs = [], adj_tpls = {};

			for(var i in line.adjusts) {
				var adj = line.adjusts[i];
				if (adj.type != 'ProductAttributes') {
					continue;
				}

				var s = adj.name.split(' : '), preset_value;
				if(s.length > 1) {
					adj.name = s[0];
					adj.value = preset_value = s[1];
				}

				if(line.attributes_meta && line.attributes_meta[adj.name]) {
					var opts = {keys: [''], values: {'': {label: app_string('LBL_NONE')}}, width: 200}, label;
					adj_tpls[adj.id] = {
						'' : {
							'type': 'ProductAttributes',
							'name': adj.name,
							'value': '',
							'amount': 0.0,
							'amount_usd': 0.0,
							'related_type': 'ProductAttributes',
							'related_id': ''
						}
					};
					for(var j = 0; j < line.attributes_meta[adj.name].length; j++) {
						var meta = line.attributes_meta[adj.name][j];
						if(~opts.keys.indexOf(meta.value)) continue;
						label = meta.value;
						if(meta.price) {
							var price = this.formatCurrency(meta.price, true);
							if(meta.price > 0) price = '+'+price;
								label += ' (' + price + ')';
						}
						opts.keys.push(meta.value);
						opts.values[meta.value] = {
							'label': label
						};
						adj_tpls[adj.id][meta.value] = {
							'type' : 'ProductAttributes',
							'name' : meta.name,
							'value' : meta.value,
							'amount' : meta.price || 0,
							'amount_usd' : meta.price_usdollar || 0,
							'related_type' : 'ProductAttributes',
							'related_id' : meta.id
						};
					}
					attrs.push({
						adj: adj,
						opts: opts,
					});
				}
			}
			attrs.sort(function(a, b) {
				return a.adj.name.localeCompare(b.adj.name);
			});

			for(var idx = 0; idx < attrs.length; idx ++) {
				var attr = attrs[idx],
					fid = this.id + '-attribute-' + grp.id + '-' + line.id + '-' + attr.adj.name,
					editor = this;
				if(this.editable) {
					var sel = new UI.SelectInput(fid, {
						ignore_dirty: true,
						options: attr.opts,
						label_key: 'label',
						adj_id: attr.adj.id
					});
					sel.onchange = function() {
						editor.updateLineAdjustment(grp, line.id, this.adj_id, adj_tpls[this.adj_id][this.getValue()]);
					}
					sel.render();
					var inpgrp = MakeElt('div.input-field.input-field-group.input-block', [
						MakeElt('div.input-label.input-addon.state-label.border-right.tools-row', [
								UI.createIcon('no-flex theme-icon bean-ProductAttribute left'),
								MakeElt('div', attr.adj.name),
							]).cssSize('width', '10em'),
						sel.elt
					]).cssSize('max-width', '30em');
					sel.setValue(attr.adj.value || '', true);
					MakeElt('div.row-space-sm', inpgrp).appendTo(cell);
				} else {
					var label = get_default(attr.adj.value, ''),
						muted = false;
					if(label === '') {
						muted = true;
						label = app_string('LBL_NONE');
					}
					if(attr.adj.amount) {
						var price = this.formatCurrency(attr.adj.amount, true);
						if(attr.adj.amount > 0) price = '+'+price;
							label += ' (' + price + ')';
					}
					MakeElt('div.row-space-sm', [
						MakeElt('span.form-label.inline-label', [
							UI.createIcon('theme-icon bean-ProductAttribute left'),
							(attr.adj.name || '') + app_string('LBL_SEPARATOR')
						]),
						MakeElt('span' + (muted ? '.text-muted' : ''), label)
					]).appendTo(cell);
				}
			}
		},

		getTaxCodeOptions: function() {
			if(! this.tax_code_options) {
				var opts = {keys: [], values: {}, width: 200}, key, code, disp;
				for(var i = 0; i < SysData.taxcodes_order.length; i++) {
					key = SysData.taxcodes_order[i];
					code = SysData.taxcodes[key];
					opts.keys.push(key);
					disp = (key === '-99' ? '' : code.code + ': ') + code.name;
					opts.values[key] = {id: code.id, label: disp, name: code.name, code: code.code};
				}
				this.tax_code_options = new UI.SelectOptions(opts);
			}
			return this.tax_code_options;
		},

		getGroupFooterLinks: function(grp, parent_id) {
			var ret = [],
				editor = this;
			if(! this.editable) return ret;
			if(parent_id) {
				parent = grp.lines[parent_id];
				if(grp.group_type === 'support' && parent.related_type === 'SupportedAssemblies') {
					ret.push({label: 'LBL_ADD_COMPONENT', related_type: 'Assets', type: 'add'});
				} else {
					ret.push({label: 'LBL_ADD_COMPONENT', related_type: 'ProductCatalog', type: 'add'});
				}
				ret.push({label: 'LBL_ADD_COMMENT', related_type: 'Notes', blank: true, type: 'add'});
			}
			else {
				if(grp.group_type === 'support') {
					ret.push({label: 'LBL_ADD_PRODUCT', related_type: 'ProductCatalog', type: 'add'});
					ret.push({label: 'LBL_ADD_ASSEMBLY', related_type: 'Assemblies', type: 'add'});
					ret.push(null);
					ret.push({label: 'LBL_ADD_SUPPORTED_PRODUCT', related_type: 'Assets', type: 'add'});
					ret.push({label: 'LBL_ADD_SUPPORTED_ASSEMBLY', related_type: 'SupportedAssemblies', type: 'add'});
					ret.push(null);
					ret.push({label: 'LBL_ADD_COMMENT', related_type: 'Notes', type: 'add'});
				}
				else if(grp.group_type === 'expenses') {
					ret.push({label: 'LBL_ADD_EXPENSE', related_type: 'BookingCategories', type: 'add'});
					ret.push({label: 'LBL_ADD_COMMENT', related_type: 'Notes', type: 'add'});
				}
				else if (grp.group_type === 'service') {
					ret.push({label: 'LBL_ADD_BOOKING', related_type: 'BookingCategories', type: 'add'});
					ret.push({label: 'LBL_ADD_COMMENT', related_type: 'Notes', type: 'add'});
				}
				else {
					ret.push({label: 'LBL_ADD_ROW', related_type: 'ProductCatalog', type: 'add'});
					ret.push({label: 'LBL_ADD_ASSEMBLY', related_type: 'Assemblies', type: 'add'});
					ret.push({label: 'LBL_ADD_COMMENT', related_type: 'Notes', type: 'add'});
					ret.push({label: 'LBL_ADD_MULT_PRODUCT', related_type: 'ProductCatalog', type: 'add', popup: true});
				}
			}
			ret.push(null);
			ret.push({name: 'remove_group', label: 'LBL_DELETE_GROUP', type: 'remove',
				onclick: function() {
					if(! grp.lines_order.length || confirm(mod_string('NTC_CONFIRM_DELETE_GROUP', editor.module)))
						editor.removeGroup(grp.id);
					return false;
				}});
			return ret;
		},

		getGroupFooterTotals: function(grp) {
			var ret = [];

			if (this.module === 'Shipping' || this.module === 'Receiving')
				return ret;

			ret.push({name: 'subtotal', label: 'LBL_SUBTOTAL', type: 'total'});

			// render discounts
			var discs = grp.discounts_order;
			for(var idx = 0; idx < discs.length; idx++) {
				var adj_id = discs[idx];
				ret.push({id: adj_id, adj: grp.adjusts[adj_id], type: 'discount', editable: this.standard_discounts});
			}
			if(this.editable && this.standard_discounts)
				ret.push({name: 'add_discount', label: 'LBL_ADD_DISCOUNT', type: 'add',
					spec: {related_type: 'Discounts'}});

			var shipping_taxed = get_default(grp.shipping_tax_class_id, '-99') != '-99';
			if(shipping_taxed)
				ret.push({name: 'shipping', label: 'LBL_SHIPPING', type: 'total', editable: true});

			var taxes = grp.taxes_order;
			if(this.show_pretax_totals && taxes.length) {
				ret.push({name: 'pretax', label: 'LBL_PRETAX_TOTAL', type: 'total'});
			}

			for(var idx = 0; idx < taxes.length; idx++) {
				var adj_id = taxes[idx];
				ret.push({id: adj_id, adj: grp.adjusts[adj_id], type: 'tax', editable: grp.adjusts[adj_id].editable});
			}
			if(this.editable)
				ret.push({name: 'add_tax', label: 'LBL_ADD_TAX', type: 'add',
					spec: {related_type: 'TaxRates', editable: 1, type: 'StandardTax'}});

			if(! shipping_taxed)
				ret.push({name: 'shipping', label: 'LBL_SHIPPING', type: 'total', editable: true});

			ret.push({name: 'total', label: 'LBL_TOTAL', type: 'total'});
			return ret;
		},

		renderGroupFooter: function(grp) {
			grp = this.getGroup(grp);
			var elt = this.getGroupInputs(grp).footer,
				group_id = grp.id,
				editor = this;
			if(! elt) return;
			elt.empty();
			var links = this.getGroupFooterLinks(grp),
				totals = grp.lines_order.length ? this.getGroupFooterTotals(grp) : [],
				row = MakeElt('div.row.col-unstack').appendTo(elt),
				link_col = MakeElt('div.column.tally-footer-links').appendTo(row),
				total_col = MakeElt('div.column.tally-footer-totals').appendTo(row),
				total_inner = MakeElt('div.tally-footer-totals-inner').appendTo(total_col),
				link_row, link, total_row;
			for(var idx = 0; idx < links.length; idx++) {
				link = links[idx];
				if(! link) {
					link_row = null;
					continue;
				}
				if(! link_row) link_row = MakeElt('div.buttons-inline').appendTo(link_col);
				var btn = MakeElt('div.uii.uii-md.active-icon', {tabIndex: 0},
						MakeElt('label', mod_string(link.label, this.module)));
				if(link.type === 'remove')
					btn.addClass('uii-remove');
				else
					btn.addClass('uii-add');
				var onclick = link.onclick;
				if(! onclick) {
					if(link.type === 'add') {
						if(link.popup) {
							onclick = function(evt) {
								var link = evt.data,
									support = (grp.group_type == 'support'),
									request_data = {
										call_back_function: 'returnPopupLineItem',
										form_name: editor.form_name,
										passthru_data: {
											group_id: group_id,
											parent_id: link.parent_id,
											module: link.related_type,
											pricebook_id: editor.getPricebookId(),
											ui_input_name: editor.name || editor.id
										},
										field_to_name_array: editor.getFieldMap(link.related_type, support)
									};
								var filter = '';
								if(editor.is_outgoing) {
									var supp = editor.form.supplier_name;
									if(supp && supp.value)
										filter = '&supplier_name='+encodeURIComponent(supp.value);
								}
								var wh = $('warehouse_id');
								if (wh) {
									filter += '&warehouse_id='+encodeURIComponent(wh.value);
								}
								if(editor.module == 'Invoice' && this.reltype == 'BookingCategories') {
									filter = '&booking_class='+encodeURIComponent('billable-work');
								}
								if((editor.module == 'Bills' || grp.group_type === 'expenses') && this.reltype == 'BookingCategories') {
									filter = '&booking_class='+encodeURIComponent('expenses');
								}
								filter += '&display_currency_id=' + editor.getCurrencyId();
								filter += '&pricebook_id=' + editor.getPricebookId();
								var options = {create: ! editor.catalog_only, multiple: true};
								editor.openSelectPopup(link.related_type, request_data, filter, options);
								return false;
							}
						}
						else {
							onclick = function(evt) {
								var link = evt.data;
								editor.addLineItem(group_id, {related_type: link.related_type, parent_id: link.parent_id}, false, 'open');
								return false;
							}
						}
					}
				}
				btn.on('click, keydown[key=enter|space]', onclick, {data: link});
				MakeElt('span.input-field-pad', btn).appendTo(link_row);
			}

			for(var idx = 0; idx < totals.length; idx++) {
				total_row = MakeElt('div.row.row-space-xs.col-space-sm').appendTo(total_inner);
				this.renderGroupFooterTotal(grp, totals[idx], total_row);
			}
		},

		renderGroupFooterTotal: function(grp, total, elt) {
			var inputs = this.getGroupInputs(grp),
				fid = this.id + '-total-' + total.name + '-' + grp.id,
				editor = this, lbl, right, tot, btn, totval;
			if(total.type === 'total') {
				lbl = MakeElt('div.form-label.input-field-pad', mod_string(total.label, this.module));
				if(total.name == 'shipping')
					this.renderShippingTax(grp, total, elt, lbl);
				else
					MakeElt('div.column.tally-col-totalname', lbl).appendTo(elt);
				right = MakeElt('div.column.tally-col-total.text-right').appendTo(elt);
				if(this.editable) {
					tot = inputs[total.name];
					if(! tot) {
						tot = inputs[total.name] = new UI.TextInput(fid, {
							format: 'currency',
							align: 'right',
							block: true,
							currency_input: this.currency_input,
							onchange: function() {
								editor.updateGroupField(grp.id, total.name, this.getValue(true), true);
							}
						});
						tot.render();
						UI.registerInput(this.form, tot);
					}
					tot.setReadOnly(! total.editable);
					tot.setValue(grp[total.name], true);
					right.append(tot.render());
				} else {
					totval = this.formatCurrency(grp[total.name], undefined, true);
					right.append(MakeElt('div.text-number.input-field-pad', totval));
				}
			}
			else if(total.type === 'tax' || total.type === 'discount') {
				this.renderTaxOrDiscount(grp, total, elt);
			}
			else if(total.type === 'add') {
				btn = MakeElt('div.uii.uii-add.uii-sm.uii-link.active-icon', {tabIndex: 0},
						MakeElt('label', mod_string(total.label, this.module)));
				btn.on('click, keydown[key=enter|space]', function() {
					this.addAdjustment(grp.id, total.spec);
				}, {scope: this})
				MakeElt('div.column',
					MakeElt('div.input-field-pad',
						btn)).appendTo(elt);
			}
			else {
				MakeElt('div.column', total.type).appendTo(elt);
			}
		},

		renderShippingTax: function(grp, adjRow, elt, lbl) {
			var editor = this,
				inputs = this.getGroupInputs(grp.id),
				labelCell = MakeElt('div.column.tally-col-totalname').appendTo(elt),
				sel = inputs.ship_tax;
			if(this.editable) {
				if(! sel) {
					var taxOpts = deep_clone(this.getTaxCodeOptions());
					var allLbl = mod_string('LBL_ALL_FROM_GROUP', this.module);
					taxOpts.addOption('all', {id: 'all', label: allLbl, name: allLbl, code: allLbl}, 0);
					taxOpts.values['-99'].code = taxOpts.values['-99'].name;
					sel = inputs.ship_tax = new UI.SelectInput(this.id + '-shiptax', {
						flat: true,
						block: true,
						options: taxOpts,
						label_key: 'code',
						wrap: true,
						ignore_dirty: true,
						icon: 'TaxCodes',
						onchange: function() {
							editor.updateGroupField(grp.id, 'shipping_tax_class_id', this.getValue(), true);
						}
					});
					sel.render()
					UI.registerInput(this.form, sel);
				}
				sel.setValue(grp.shipping_tax_class_id || '-99', true);
				MakeElt('div.tools-row.align-top', lbl, sel.render()).appendTo(labelCell);
			} else {
				var cls = grp.shipping_tax_class_id || '-99',
					opts = this.getTaxCodeOptions(),
					taxicon = UI.createIcon('theme-icon module-TaxCodes'),
					clslbl;
				if(cls === 'all')
					clslbl = [taxicon, ' ', mod_string('LBL_ALL_FROM_GROUP', this.module)];
				else {
					clslbl = opts.values[cls];
					if(clslbl) {
						clslbl = clslbl.code;
						if(cls !== '-99')
							clslbl = [taxicon, ' ', clslbl];
					}
				}
				if(clslbl)
					lbl.append(' (', clslbl, ')');
				labelCell.append(lbl);
			}
		},

		renderTaxOrDiscount: function(grp, adjRow, elt) {
			var editor = this,
				editable = this.editable && adjRow.editable,
				options = {},
				optOrder = [],
				lblPrefix = '',
				inputs = this.getLineInputs(grp, adjRow.id, true),
				labelCell = MakeElt('div.column.tally-col-totalname'),
				valueCell = MakeElt('div.column.tally-col-total.text-right');
			elt.append(labelCell, valueCell);

			if(adjRow.type == 'discount') {
				//lblPrefix = 'LBL_DISCOUNT';
				optOrder = SysData.discounts_order.filter(function(did) {
					return !SysData.discounts[did].applies_to_selected;
				});
				options = optOrder.reduce(function(options, did) {
					options[did] = SysData.discounts[did];
					return options;
				}, {});
			}
			else {
				//lblPrefix = 'LBL_TAX';
				optOrder = SysData.taxrates_order;
				options = SysData.taxrates;
			}

			if(lblPrefix) {
				lblPrefix = mod_string(lblPrefix, this.module) + app_string('LBL_SEPARATOR');
			}

			if(editable) {
				var ddOpts = {keys: [], values: [], width: 250},
					sel = inputs.sel;
				for(var i = 0; i < optOrder.length; i++) {
					var id = optOrder[i];
					var rate, lbl;
					if(options[id].type == 'StdFixedDiscount')
						rate = this.formatCurrency(options[id].amount);
					else {
						rate = options[id].rate;
						if(options[id].type == 'StdPercentDiscount' || options[id].type == 'PercentDiscount') {
							rate = -rate;
							if(rate > 0) rate = '+' + rate;
						}
						rate = stdFormatNumber(rate) + '%';
					}
					lbl = options[id].name;
					if(id && id !== '-99')
						lbl += ' (' + rate + (options[id].compounding ? ' +' : '') + ')';
					ddOpts.keys.push(id);
					ddOpts.values.push(lbl);
				}
				if(! sel) {
					sel = inputs.sel = new UI.SelectInput(this.id + '-' + adjRow.id + '-sel', {
						flat: true,
						block: true,
						wrap: true,
						ignore_dirty: true,
						icon: adjRow.adj.related_type,
						onchange: function() {
							editor.updateAdjustment(grp.id, adjRow.id, options[this.getValue()]);
						}
					});
					sel.render()
					UI.registerInput(this.form, sel);
				}
				sel.setOptions(ddOpts);
				sel.setValue(adjRow.adj.related_id || '-99', true);
				MakeElt('div.tools-row.align-top', lblPrefix, sel.render(),
					MakeElt('span.uii.uii-md.uii-remove.active-icon')
						.on('click, keydown[key=enter|space]', function() { editor.removeAdjustment(grp.id, adjRow.id); })
					).appendTo(labelCell);

				/*if (this.get_discount_before_taxes() && adj.related_type == 'Discounts') {

					left_td.appendChild(nbsp());
					left_td.appendChild(createTextLabel('LBL_TAX_CODE'));
					left_td.appendChild(nbsp());

					var onchange = function() {
						editor.update_discount_tax(group_id, adj_id, this.getValue());
					}

					var codes = {};
					for(var c in SysData.taxcodes)
						codes[c] = SysData.taxcodes[c].code;
					var attrs = {onchange: onchange};
					var taxCls = createSelectInput(SysData.taxcodes_order, codes, adj.tax_class_id||'-99', attrs);
					labelCell.append(taxCls);
				}*/

			}
			else {
				var compounded = (adjRow.adj.type == 'CompoundedTax');
				var rate = '';
				var tax = '';
				if(adjRow.adj.type != 'StdFixedDiscount') {
					rate = ' (' + stdFormatNumber(adjRow.adj.rate) + '%' + (compounded ? ' +' : '') + ')';
					if (adjRow.adj.tax_class_id && adjRow.adj.tax_class_id != '-99') {
						tax = ' (' + SysData.taxcodes[adjRow.adj.tax_class_id].code + ')';
					}
				}
				MakeElt('div.form-label.input-field-pad',
					lblPrefix + adjRow.adj.name + rate + tax).appendTo(labelCell);
			}

			var amtval = adjRow.adj.amount,
				amt = inputs.amount;
			if(adjRow.type == 'discount') amtval = -amtval;
			if(this.editable) {
				if(! amt) {
					amt = inputs.amount = new UI.TextInput(this.id + '-' + adjRow.id + '-amount', {
						block: true,
						readOnly: true,
						format: 'currency',
						currency_input: this.currency_input,
						align: 'right'
					});
					amt.render();
					UI.registerInput(this.form, amt);
				}
				amt.setValue(amtval, true);
				valueCell.content(amt.render());
			} else {
				valueCell.append(MakeElt('div.text-number.input-field-pad',
					this.formatCurrency(amtval, undefined, true)));
			}
		},

		renderFooter: function() {
			var foot = MakeElt('div.tally-footer').appendTo(this.elt);
			if(this.editable) {
				var btn = MakeElt('div.uii.uii-add.uii-md.active-icon',
						MakeElt('label', mod_string('LBL_ADD_GROUP', this.module)));
				btn.on('click, keydown[key=enter|space]', this.addGroup, {scope: this});
				var outer = MakeElt('div.buttons-inline',
					MakeElt('div.inline-elt', btn));
				MakeElt('div.row.form-row',
					MakeElt('div.column.form-cell',
						MakeElt('div.column.form-entry',
							MakeElt('div.column.form-value', outer)))).appendTo(foot);
			}
		},

		renderTotals: function() {
			this.recalcTotals();

			if(this.module === 'Shipping' || this.module === 'Receiving')
				return;

			// update gross profit fields on Invoice
			var totals = this.totals,
				disctotal = totals.subtotal - totals.discounts,
				profit = disctotal - totals.cost,
				pperc = parseFloat((disctotal != 0 ? profit / disctotal * 100 : 0.0).toFixed(1)),
				profit_inp = UI.getFormInput(this.form, 'gross_profit'),
				perc_inp = UI.getFormInput(this.form, 'gross_profit_pc');
			if(profit_inp) {
				if(! this.product_costs) {
					Wrap(profit_inp.elt).closest('.form-entry').hide();
				} else {
					profit_inp.ignore_dirty = true;
					profit_inp.setValue(profit, true);
					SUGAR.ui.addRemoveClass(profit_inp.elt, 'invalid', (profit < 0 ? 1 : 0));
				}
			}
			if(perc_inp) {
				if(! this.product_costs) {
					Wrap(perc_inp.elt).closest('.form-entry').hide();
				} else {
					perc_inp.ignore_dirty = true;
					perc_inp.setValue(pperc, true);
					SUGAR.ui.addRemoveClass(perc_inp.elt, 'invalid', (profit < 0 ? 1 : 0));
				}
			}

			// render grand totals
			var outer = this.totals_outer,
				gtotals = new UI.PanelRender(),
				header = gtotals.addHeader(),
				body = gtotals.addBody();
			if(! outer || ! outer.inDocument()) {
				outer = this.totals_outer = MakeElt('div.tally-grand-totals').appendTo(this.elt);
			}
			if(this.groups_order.length <= 1) {
				outer.hide();
				return;
			}

			outer.empty().show();
			MakeElt('div.row.form-row',
				MakeElt('div.column.form-cell',
					MakeElt('div.form-entry',
						MakeElt('div.form-value', gtotals.render())
					))).appendTo(outer),

			header.append(MakeElt('h4.panel-title', mod_string('LBL_GRAND_TOTALS', this.module)));

			var totals_rows = [
					{name: 'subtotal', label: 'LBL_SUBTOTAL'},
					{name: 'discounts', label: 'LBL_DISCOUNT', autohide: true},
					{name: 'pretax', label: 'LBL_PRETAX'},
					{name: 'taxes', label: 'LBL_TAX', autohide: true},
                    {name: 'milestone_invoices_tax', label: 'LBL_MILESTONE_INVOICES_TAX'},
                    {name: 'milestone_invoices_tax_remaining', label: 'LBL_MILESTONE_INVOICES_TAX_REMAINING'},
					{name: 'shipping', label: 'LBL_SHIPPING', autohide: true},
                    {name: 'milestone_invoices_shipping', label: 'LBL_MILESTONE_INVOICES_SHIPPING'},
                    {name: 'milestone_invoices_shipping_remaining', label: 'LBL_MILESTONE_INVOICES_SHIPPING_REMAINING'},
                    {name: 'milestone_invoices_total', label: 'LBL_MILESTONE_INVOICES'},
					{name: 'total', label: 'LBL_TOTAL'}
				];

            var milestone_invoices_total = 0.0;
            if (this.totals.hasOwnProperty('milestone_invoices_total'))
                milestone_invoices_total = this.totals['milestone_invoices_total'];

            var container_cls = 'div.tally-totals-inner';
            if (milestone_invoices_total !== 0.0)
                container_cls += '.tally-totals-inner-wide';

            var contain = MakeElt(container_cls).appendTo(body);

			for(var idx = 0; idx < totals_rows.length; idx++) {
                var total = totals_rows[idx];

                if ( total.name.includes('milestone_invoices') && ! this.totals.hasOwnProperty(total.name))
                    continue;

                var val = this.totals[total.name],
					cls = '.text-number.input-field-pad';

				if(total.name === 'pretax' && ! this.totals.discounts && ! this.totals.shipping)
					continue;

				if(total.name === 'total') {
					cls += '.strong';
					if(val < 0)
						cls += '.text-error';
				}
				if(val) {
					val = this.formatCurrency(this.totals[total.name], undefined, true);
				} else {
					if(total.autohide) continue;
					val = '&mdash;'
				}
				MakeElt('div.row.col-space-sm',
					MakeElt('div.column.tally-col-totalname',
						MakeElt('div.form-label.input-field-pad',
							mod_string(total.label, this.module))),
						MakeElt('div.column.tally-col-total.text-right',
							MakeElt('div' + cls).html(val)))
					.appendTo(contain);
			}
		},


		// -- Utility methods --

		taxDiscountChanged: function(cb) {
			if (!cb.getValue()) {
				this.iterGroups(function(group_id, grp) {
					for(var jdx = 0; jdx < grp.discounts_order.length; jdx++) {
						var adj = grp.adjusts[grp.discounts_order[jdx]];
						adj.tax_class_id = null;
					}
				});
			}
			this.recalculate();
		},

		resetTaxCodes: function() {
			for(var i = 0; i < this.groups_order.length; i++) {
				var grp = this.groups[this.groups_order[i]];
				for(var idx = 0; idx < grp.lines_order.length; idx++) {
					var line = grp.lines[grp.lines_order[idx]];
					if(! line.is_comment) {
						if(! line.sum_of_components) {
							line.tax_class_id = '-99';
						}
					}
				}
			}
		},

		getCurrencyId: function() {
			return this.currency_input.getKey() || '-99';
		},

		getExchangeRate: function(currency_id) {
			return this.currency_input.getRate(true, currency_id);
		},

		currencyDecimalPlaces: function(currency_id) {
			return this.currency_input.getDecimals(currency_id);
		},

		formatCurrency: function(val, noRound, symbol) {
			if(! isset(val) || val === '') return '';
			var decimals = this.currencyDecimalPlaces(), ret;
			if(noRound)
				ret = stdFormatNumber(restrictDecimals(parseFloat(val), decimals));
			else
				ret = stdFormatNumber(parseFloat(val * 100).toFixed(decimals) / 100, decimals, decimals);
			if(symbol) {
				var symbol = this.currency_input.getSymbol();
				if(this.currency_input.getSymbolAfter() == 1)
					ret = ret + ' ' + symbol;
				else
					ret = symbol + ' ' + ret;
			}
			return ret;
		},

		getPricebookId: function() {
			var pb = UI.getFormInput(this.form, 'pricebook');
			if(pb) return pb.getKey();
		},

		getFieldMap: function(related_type, support) {
			var field_map;
			if(related_type == 'ProductCatalog') {
				field_map = {
					'related_id': 'id',
					'id': 'id',
					'name': 'name',
					'purchase_name': 'purchase_name',
					'tax_class_id': 'tax_code_id',
					'mfr_part_no': 'pricebook_part_no',
					'vendor_part_no': 'vendor_part_no',
					'currency_id': 'currency_id',
					'exchange_rate': 'exchange_rate',
					'description': 'description',
					'description_plain': 'description_plain',
					'pricebook_price': 'pricebook_price',
					'raw_pricebook_price': 'raw_pricebook_price',
					'pricebook_currency_id': 'pricebook_currency_id'
				};
				if (support) {
					field_map['raw_cost_price'] = 'support_cost';
					field_map['raw_list_price'] = 'support_list_price';
					field_map['raw_unit_price'] = 'support_selling_price';
					field_map['cost_price'] = 'support_cost_usdollar';
					field_map['list_price'] = 'support_list_usdollar';
					field_map['unit_price'] = 'support_selling_usdollar';
				} else {
					field_map['raw_cost_price'] = 'cost';
					field_map['raw_list_price'] = 'list_price';
					field_map['raw_unit_price'] = 'purchase_price';
					field_map['cost_price'] = 'cost_usdollar';
					field_map['list_price'] = 'list_usdollar';
					field_map['unit_price'] = 'purchase_usdollar';
				}
				if(this.is_outgoing && this.module != 'CreditNotes') {
					field_map.raw_unit_price = 'cost';
					field_map.unit_price = 'cost_usdollar';
				}
			} else if(related_type == 'Assets') {
				field_map = {
					'related_id': 'id',
					'name': 'name',
					'tax_class_id': 'tax_code_id',
					'mfr_part_no': 'manufacturers_part_no',
					'vendor_part_no': 'vendor_part_no',
					'raw_cost_price': 'support_cost',
					'cost_price': 'support_cost_usdollar',
					'raw_list_price': 'unit_support_price',
					'list_price': 'unit_support_usdollar',
					'currency_id': 'currency_id',
					'exchange_rate': 'exchange_rate',
					'description_plain': 'description',
					'serial_no' : 'serial_no'
				};
			}
			else if(related_type == 'Assemblies' || related_type == 'SupportedAssemblies') {
				field_map = {
					'related_id': 'id',
					'name': 'name',
					'mfr_part_no': 'manufacturers_part_no',
					'description': 'description',
					'description_plain': 'description_plain'
				};
				if(related_type == 'SupportedAssemblies') {
					field_map.serial_no = 'serial_no';
					field_map.description_plain = 'description';
				} else {
					field_map.purchase_name = 'purchase_name';
				}
			} else if(related_type == 'BookingCategories') {
				field_map = {
					'related_id': 'id',
					'name': 'name',
					'location': 'location',
					'duration': 'duration',
					'seniority': 'seniority',
					'currency_id': 'currency_id',
					'raw_list_price': 'billing_rate',
					'list_price': 'billing_rate_usd',
					'exchange_rate': 'exchange_rate',
					'tax_class_id': 'tax_code_id'
				};
			} else if(related_type == 'TaxRates') {
				field_map = {
					'related_id': 'id',
					'name': 'name'
				};
			}
			return field_map;
		},


		// -- Handling saving & validation --

		validate: function() {
			if(this.editable || this.editable_events) {
				if(this.module == 'Invoice') {
					var count = 0;
					for(var idx = 0; idx < this.groups_order.length; idx++) {
						var grp = this.groups[this.groups_order[idx]];
						count += grp.lines_order.length;
					}
					if(! count) {
						this.invalidMsg = mod_string('NTC_ONE_ITEM_NEEDED');
						return false;
					}
				}
			}

			// FIXME - re-enable inventory check - use synchronous JSON call, don't modify actual form attributes
			if (0 && this.module == 'Shipping') {
				this.form.action.value = 'CheckInventory';
				this.form.to_pdf.value = 'true';

				var callback = function(o) {
					if(! o) return;
					try {
						SUGAR.util.evalScript(o.responseText);
						var failure = window.inventoryCheck.failure;
						if (failure) {
							var msg = mod_string('NTC_CONFIRM_UNDER_STOCK1') + "\n";
							for (var name in window.inventoryCheck.outOfStock) {
								msg += "\n" + name + ' : ' + window.inventoryCheck.outOfStock[name];
							}
							msg += "\n\n" +mod_string('NTC_CONFIRM_UNDER_STOCK2') ;
							failure = !confirm(msg);
						}
						if (!failure) {
							TallyEditor.form.submit();
						}
					} catch (e) {
					}
				};
				SUGAR.conn.sendForm(this.form, {status_msg: mod_string('LBL_CHECKING_INVENTORY')}, callback);
				this.form.action.value = 'Save';
				this.form.to_pdf.value = '';
				return false;
			}

			return true;
		},

		beforeSubmitForm: function(form) {
			if((this.module == 'SalesOrders' || this.module == 'Invoice' || this.module == 'Quotes') && (SUGAR.dom.extNode(this.form).find('#tally-edit-btn').length || this.editable)) {
				// These weird hidden field names were chosen to reduce the risk of conflicts with custom fields
				var account_balance_input = parseFloat(UI.HiddenInput.findOrInsert(this.form, 'account_credit_hf_warncl', 'account_credit_hf_warncl').getValue()) || 0,
					soamount_cur = parseFloat(this.totals.total) || 0,
					exchange_rate = parseFloat(this.currency_input.getRate()) || 0,
					soamount = parseFloat(soamount_cur/exchange_rate).toFixed(4),
					sopreamount = parseFloat(UI.HiddenInput.findOrInsert(this.form, 'so_pre_amount_hf_warncl', 'so_pre_amount_hf_warncl').getValue()).toFixed(4) || 0,
					pre_account_id = UI.HiddenInput.findOrInsert(this.form, 'pre_account_id_hf_warncl', 'pre_account_id_hf_warncl').getValue(),
					account_id = UI.HiddenInput.findOrInsert(this.form, 'billing_account_id', 'billing_account_id').getValue();
					if(soamount != sopreamount || pre_account_id != account_id) {
						if(soamount > account_balance_input && account_balance_input != '000') {
							var r = confirm(app_string('LBL_CREDIT_OVERLIMIT'));
							if (r == true) {

							} else {
							    return false;
							}
						}
				}
			}
			if(this.editable || this.editable_events) {
				var ret = this.encodeLineItems(),
					input = UI.HiddenInput.findOrInsert(this.form, this.name, this.id+'-save');

				input.setValue(JSON.stringify(ret));
			}
		},

		default_adj_weight: 4,

		adj_weights: {
			'StandardDiscount': 0,
			'StdPercentDiscount' : 0,
			'StdFixedDiscount': 0,
			'TaxedShipping': 1,
			'CompoundedTax': 2,
			'StandardTax': 2,
			'UntaxedShipping': 3
		},

		adjWeight: function (adj) {
			var w = this.adj_weights[adj.type];
			if (! isset(w))
				w = this.default_adj_weight;
			return w;
		},

		encodeLineItems: function() {
			var ret = [];
			this.iterGroups(function(grp_id, grp) {
				var newGrp = Object.assign({}, grp),
					lineAdjusts = [];
				delete newGrp.dragTree;
				newGrp.lines = [];
				newGrp.adjusts = [];
				delete newGrp.lines_order;
				delete newGrp.adjusts_order;
				delete newGrp.inputs;
				delete newGrp.line_inputs;

				this.iterLineItems(function(line_id, line, idx) {
					var newLine = Object.assign({}, line);
					delete newLine.adjusts;
					newGrp.lines.push(newLine);
					if(line.adjusts) {
						for(var adj_id in line.adjusts) {
							var adj = deep_clone(line.adjusts[adj_id]);
							if(adj.type === 'ProductAttributes') {
								if(! isset(adj.value) || adj.value === '')
									continue;
								adj.name += ' : ' + adj.value;
							}
							adj.line_id = line_id;
							lineAdjusts.push(adj);
						}
					}
				}, grp);

				var counts = [];
				var positions = {};
				for (var k = 0; k <= this.default_adj_weight; k++) {
					counts.push(0);
				}
				this.iterAdjustments(function(adj_id, adj, idx) {
					var w = this.adjWeight(adj);
					counts[w] ++;
				}, grp);
				var pos = 0;
				for (var k = 0; k <= this.default_adj_weight; k++) {
					positions[k] = pos;
					pos += counts[k];
				}
				this.iterAdjustments(function(adj_id, adj, idx) {
					var w = this.adjWeight(adj);
					newGrp.adjusts[positions[w]] = adj;
					positions[w]++;
				}, grp);

				newGrp.adjusts.pushArray(lineAdjusts);

				ret.push(newGrp);
			});
			return ret;
		},

		setAccount: function(result, prefix, force_additional) {
			if(! result || typeof result === 'string') return;
			var upd_currency = false;
			var other_map = {'shipping': 'billing', 'billing': 'shipping'};
			var other_prefix = other_map[prefix];
			var other_input = UI.getFormInput(this.form, other_prefix + '_account');
			var update_opposite = other_input && ! other_input.getKey();
			var update_additional = update_opposite || force_additional;
			this.default_tax_code_id = get_default(result.tax_code_id, '');
			if (this.default_tax_code_id == '-99') {
				var tax_exempt = UI.getFormInput(this.form, 'tax_exempt');
				if (tax_exempt && !tax_exempt.getValue()) {
					tax_exempt.setValue(1);
					this.taxExemptChanged(tax_exempt, true);
				}
			}
			if(isdef(result.currency_id)) {
				if (update_additional && this.editable) {
					var cid = result.currency_id;
					var exchange_rate = result.exchange_rate;
					if(! exchange_rate)  exchange_rate = 1;
					if(! cid)  cid = '-99';
					upd_currency = true;
				}
				delete result.exchange_rate;
				delete result.currency_id;
				delete result.currency;
			}
			if (update_opposite) {
				if(isset(result[prefix + '_phone']))
					result[other_prefix + '_phone'] = result[prefix + '_phone'];
				if(isset(result[prefix + '_email']))
					result[other_prefix + '_email'] = result[prefix + '_email'];
			}

			if(!update_opposite) {
				var re = new RegExp('^' + other_prefix + '_');
				for (var the_key in result) {
					if (the_key.match(re)) delete(result[the_key]);
				}
			}
			if (!update_opposite && prefix != 'shipping' && prefix != 'supplier') {
				delete result.shipping_provider_id;
			}
			if((isdef(result.terms) && ! result.terms) || prefix == 'shipping')
				delete result.terms;

			UI.setFormInputValues(this.form, result);

			if(upd_currency && update_additional) {
				this.currency_input.setValue(cid);
				this.currency_input.setRate(exchange_rate);
			}
			if(!blank(result.account_credit_limit) && (result.account_credit_limit > 0) && !blank(result.account_balance) && update_additional) {
				if(result.account_balance > result.account_credit_limit)
					alert(mod_string('NTC_OVER_CREDIT_LIMIT'));
			}
			if(!blank(result.discount_id) && result.discount_id != '-99' && update_additional && this.editable) {
				this.setDefaultDiscount(result.discount_id);
			}

			if (result.account_popups == 1 && result.sales_popup && prefix == 'billing') {
				SUGAR.popups.showMessage(result.sales_popup, {
					timeout: 15000,
					delay: 250,
					title_html: '<div class="input-icon theme-icon module-Accounts"></div> ' + html_escape(result['_display'])});
			}
			if (update_opposite)
				other_input.update(result.id, result);

			if (this.module == 'Quotes' || this.module == 'Invoice' || this.module == 'SalesOrders') {
				var pricebook = UI.getFormInput(this.form, 'pricebook');
				if (result.default_pricebook_id) {
					if (pricebook) {
						if (!pricebook.getKey() || result.default_pricebook_editable != "1") {
							pricebook.update(result.default_pricebook_id, result.default_pricebook);
						}
					}
				}
				if (pricebook)
					pricebook.setReadOnly(!(result.default_pricebook_editable =="1" || !result.default_pricebook_id));
			}
		},

		taxExemptChanged: function(cb, from_account) {
			if (cb.getValue() && this.editable) {
				var conf = confirm(mod_string(from_account ? 'LBL_TAX_EXEMPT_ACCOUNT' : 'LBL_TAX_EXEMPT_CONFIRMATION', this.module));
				if (conf) {
					this.resetTaxCodes();
				} else {
					cb.setValue(0);
				}
			}
		},

		taxDiscountChanged: function(cb) {
			if (!cb.getValue()) {
				this.iterGroups(
					function(group_id, grp, idx) {
						for(var jdx = 0; jdx < grp.discounts_order.length; jdx++) {
							var adj = grp.adjusts[grp.discounts_order[jdx]];
							adj.tax_class_id = null;
						}
					}
				);
			}
			this.recalculate();
		},

		resetTaxCodes: function() {
			this.iterGroups(function(group_id, grp, gidx) {
				this.iterLineItems(function(line_id, line, idx) {
					if(! line.is_comment && ! line.sum_of_components) {
						line.tax_class_id = '-99';
					}
				}, grp);
			});
			this.recalculate();
		},

		openSelectPopup: function(module, request_data, filter, options) {
			if(! isset(options)) options = {};
			options.inline = true;
			options.module = module;
			options.request_data = request_data;
			if(! options.title)
				options.title = mod_string('LBL_SELECT_PREFIX') + module_name(module, true);
			var width = options.width,
				viewp = UI.getViewport();
			if(! width) {
				var minw = (module == 'BookingCategories') ? 670 : 750;
				width = get_default(options.min_width, minw);
				var scrw = viewp.width;
				if(scrw > 1024)
					width += Math.round(Math.min(150, (scrw - 1024) / 2));
				options.width = width;
			}
			var height = options.height;
			if(! height) {
				height = get_default(options.min_height, 450);
				var scrh = viewp.height;
				if(scrh > 600)
					height += Math.round(Math.min(200, (scrh - 600) / 2));
				options.height = height;
			}
			options.filter = filter;
			if(! isset(options.close_popup))
				options.close_popup = true;
			return open_popup_window(options);
		},

		getDepthInfo: function(group, line) {
		},

		onDragStart: function(evt, data) {
			this.dragState.source = this;
			this.dragState.drag_disabled = false;;
			setTimeout(function() {
				SUGAR.dom.ExtNode(document).find('.tally-row-outer').css('opacity', 0.3);
				SUGAR.dom.ExtNode(evt.target).css('opacity', 1);
			});
		},

		onDragAccept: function(evt, data) {
			return this.dragState.source && !this.dragState.drag_disabled;
		},

		onDragEnter: function(evt, data) {
		},

		onDragLeave: function(evt, data) {
		},

		checkDragSource: function(vt, data) {
			return data.values && data.values.limitedSource === 'tallyEditor';
		},

		onDragIntoEmpty: function(source) {
			var src_node = source.tallyInfo.node;
			src_node.remove();
			this.tallyInfo.group.dragTree.appendChild(src_node);
			this.dragState.target_group = this.tallyInfo.group;
			var childNodes = [];
			if (src_node.isAssembly()) {
				var next = source.elt.get().nextSibling;
				while (next) {
					if (!Wrap(next).find('.tally-col-depth-1').get(0))
						break;
					childNodes.push(next);
					var nextNext = next.nextSibling;
					next = nextNext;
				}
			}
			this.elt.get().appendChild(source.elt.get());
			childNodes.forEach(function(n) {
				this.elt.get().appendChild(n);
			}.bind(this));
		},

		onDragMove: function(evt, data) {
			var source = this.dragState.source;
			var pos = evt.position();
			var region, depth, after;
			var elt = source.elt;
			var p = evt.currentTarget.parentNode;
			var src_node = source.tallyInfo.node;
			var dst_node = this.tallyInfo.node;
			var overSelf = src_node == dst_node;
			if (!overSelf) {
				this.dragState.drag_disabled = false;
				var line = source.tallyInfo.line;
				var group_s = source.tallyInfo.group;
				var group_d = this.tallyInfo.group;
				if (!this.editor.canDragLineIntoGroup(line, group_s, group_d)) {
					this.dragState.drag_disabled = true;
					return;
				}
			}
			if (!dst_node) {
				return this.onDragIntoEmpty(source);
			}
			var target = Wrap(evt.target).closest('.tally-row-outer').get(0);
			if (target) {
				region = SUGAR.ui.getEltRegion(target);
				after = pos.y > region.top + (region.height / 2) || ! this.tallyInfo.line;
			} else {
				target = evt.target.firstChild;
				after = true;
			}

			var spacer = Wrap(target).find('.column').get().find(function(el) { return el.style.display != 'none'});
			if (spacer) {
				region = SUGAR.ui.getEltRegion(spacer);
				depth = pos.x > region.x + region.width ? 1 : 0;
			}

			if (src_node.isAssembly() && dst_node.isAssemblyComponent())
				return;

			if (!overSelf) {
				this.dragState.target_group = this.tallyInfo.group;
				src_node.remove();
				depth = dst_node.moveNode(src_node, after, depth);
			} else {
				if (dst_node.isAssemblyComponent() && dst_node.next) {
					depth = 1;
				} else if (dst_node.isAssemblyComponent() && !depth) {
					var parent = dst_node.parent;
					dst_node.remove();
					parent.append(dst_node);
				} else if (!dst_node.isAssemblyComponent() && dst_node.prev && dst_node.prev.isAssembly() && depth) {
					var prev = dst_node.prev;
					dst_node.remove();
					prev.appendChild(dst_node);
				} else if (dst_node.isAssemblyComponent()) {
					;;
				} else if (!dst_node.prev || !dst_node.prev.isAssembly()) {
					depth = 0;
				}
			}


			if (!target) {
				var target = Wrap(evt.target).closest('.tally-items-body').find('.tally-row-data').get(0);
				if (!target)
					return;
				p = evt.currentTarget;
			}

			if (!src_node.isAssembly()) {
				var depth_col = Wrap(elt.find('.tally-col-depth-1').get(0));
				if (depth) {
					if (!depth_col) {
						depth_col = Wrap(MakeElt('div.tally-col-depth-1.column')).prependTo(elt);
					}
				}
				if (depth_col) {
					depth_col.css('display', depth ? '' : 'none');
				}
			}

			if(after) {
				target = target.nextSibling;
			}
			if (overSelf || target == elt.get())
				return;
			var childNodes = [];
			if (src_node.isAssembly()) {
				var next = elt.get().nextSibling;
				while (next) {
					if (!Wrap(next).find('.tally-col-depth-1').get(0))
						break;
					childNodes.push(next);
					var nextNext = next.nextSibling;
					next.remove();
					next = nextNext;
				}
			}
			if (target) {
				p.insertBefore(elt.get(), target);
				childNodes.forEach(function(n) {
					p.insertBefore(n, target);
				});
			} else {
				p.appendChild(elt.get());
				childNodes.forEach(function(n) {
					p.appendChild(n);
				});
			}
		},

		onDragDrop: function(evt, data) {
			var pos;
			var source = this.dragState.source;
			var src_g = source.tallyInfo.group;
			var src_l = source.tallyInfo.line;
			var dst_g = this.dragState.target_group;

			// if this is inter-group drag, first
			// move dragged line, its children
			// and corresponding adjustments to new group
			if (dst_g && src_g != dst_g) {
				var linesToMove = [src_l];
				var adjToMove = [];
				if (src_l.sum_of_components) {
					src_g.lines_order.forEach(function(line_id) {
						if (src_g.lines[line_id].parent_id == src_l.id)
							linesToMove.push(src_g.lines[line_id]);
					});
				}
				src_g.adjusts_order.forEach(function(adj_id) {
					var adj = src_g.adjusts[adj_id];
					if (linesToMove.find(function(l) {return l.id == adj.line_id})) {
						adjToMove.push(adj);
					}
				});
				// first remove from source group
				linesToMove.forEach(function(line) {
					var line_id = line.id;
					var pos = src_g.lines_order.indexOf(line_id);
					if (~pos) {
						src_g.lines_order.splice(pos, 1);
					}
					delete src_g.lines[line_id];
				});
				adjToMove.forEach(function(adj) {
					var adj_id = adj.id;
					var pos = src_g.adjusts_order.indexOf(adj_id);
					if (~pos) {
						src_g.adjusts_order.splice(pos, 1);
					}
					delete src_g.adjusts[line_id];
				});
				// then insert
				linesToMove.forEach(function(line) {
					var line_id = line.id;
					dst_g.lines_order.push(line_id);
					dst_g.lines[line_id] = line;
				});
				adjToMove.forEach(function(adj) {
					var adj_id = adj.id;
					dst_g.adjusts_order.push(adj_id);
					dst_g.adjusts[adj_id] = adj;
				});
			}
			if (!dst_g)
				dst_g = src_g;
			// now rebuild lines order from the tree maintained by
			// drag operation
			dst_g.lines_order = [];
			var node = dst_g.dragTree.firstChild;
			while (node) {
				dst_g.lines_order.push(node.line_id);
				dst_g.lines[node.line_id].parent_id = null;
				dst_g.lines[node.line_id].depth = 0;
				var child = node.firstChild;
				while (child) {
					dst_g.lines_order.push(child.line_id);
					dst_g.lines[child.line_id].parent_id = node.line_id;
					dst_g.lines[child.line_id].depth = 1;;
					child = child.next;
				}
				node = node.next;
			}


			this.editor.recalcGroup(src_g);
			this.editor.recalcGroup(dst_g);
		},
		onDragEnd: function(evt, data) {
			SUGAR.dom.ExtNode(document).find('.tally-row-outer').css('opacity', 1);
			this.editor.dragState = {};
			this.editor.render();
		},

		dragState: {},
		dragConfig: function(handle) {
			return {
				acceptedSourceTypes: ['tallyEditorLine'],
				sourceType: 'tallyEditorLine',
				editor: this,
				dragState: this.dragState,
				checkAccept: this.onDragAccept,
				onDrop: this.onDragDrop,
				onMove: this.onDragMove,
				onStart: this.onDragStart,
				onEnd: this.onDragEnd,
				onEnter: this.onDragEnter,
				onLeave: this.onDragLeave,
				onDragIntoEmpty: this.onDragIntoEmpty,
				checkDragSource: this.checkDragSource,
				handle: handle,
			}
		},

		canDragLineIntoGroup: function(line, group_s, group_d) {
			if (line.is_comment)
				return true;
			var line_type = line.related_type;
			var group_type = group_d.group_type;
			switch (line_type) {
				case 'Assemblies':
				case 'ProductCatalog':
					return group_type == 'products' || group_type == 'support' || this.module == 'PurchaseOrders';
				case 'Assets':
				case 'SupportedAssemblies':
					return group_type == 'support';
				case 'BookingCategories':
					return (group_type == 'service' || group_type == 'expenses') && (group_type == group_s.group_type);

			}
		}
	};
	// end TallyEditor

	var TreeNode = function(values) {
		Object.assign(this, values);
	};

	TreeNode.prototype = {
		serialize: function() {
			var ret = {sum_of_components: this.sum_of_components, line_id: this.line_id, children: []};
			var c = this.firstChild;
			while (c) {
				ret.children.push(c.serialize());
				c = c.next;
			}
			return ret;
		},
		isAssemblyComponent: function() {
			return this.parent && this.parent.sum_of_components;
		},
		isAssembly: function() {
			return this.sum_of_components;
		},
		appendChild: function(node) {
			node.parent = this;
			if (!this.firstChild) {
				this.firstChild = node;
				node.next = null;
				node.prev = null;
			} else {
				var lastChild = this.firstChild;
				while (lastChild.next)
					lastChild = lastChild.next;
				lastChild.next = node;
				node.prev = lastChild;
			}
		},
		moveNode: function(node, after, depth) {
			node.remove();
			if (after) {
				if (this.isAssembly() && depth) {
					if (this.firstChild) this.firstChild.prepend(node);
					else this.appendChild(node);
					return depth;
				} else if (this.isAssemblyComponent() && (this.next || depth)) {
					this.append(node);
					return 1;
				} else {
					this.parent.appendChild(node);
				}
			} else {
				if (this.isAssemblyComponent()) {
					this.prepend(node);
					return 1;
				} else if (this.prev && this.prev.isAssembly()) {
					if (depth) {
						this.prev.appendChild(node);
						return 1;
					}
					else this.prev.append(node);
				} else {
					this.prepend(node);
				}
			}
		},
		append: function(sibling) {
			if (!this.parent && !this.root)
				return;
			sibling.parent = this.parent;
			sibling.prev = this;
			sibling.next = this.next;
			if (this.next)
				this.next.prev = sibling;
			this.next = sibling;
		},
		prepend: function(sibling) {
			if (!this.parent)
				return;
			sibling.parent = this.parent;
			sibling.next = this;
			sibling.prev = this.prev;
			if (this.prev)
				this.prev.next = sibling;
			this.prev = sibling;
			if (this.parent.firstChild == this)
				this.parent.firstChild = sibling;
		},
		remove: function() {
			if (!this.parent) return;
			if (this.parent.firstChild == this)
				this.parent.firstChild = this.next;
			if (this.prev)
				this.prev.next = this.next;
			if (this.next)
				this.next.prev = this.prev;
			this.parent = this.next = this.prev = null;
		}
	};

	var LineItemInput = function(id, params) {
		this.ext_class = 'tally-line-input';
		UI.RefInput.call(this, id, params);
	};
	extendClass(LineItemInput, UI.RefInput, {
		updateLabel: function(key, value) {
			UI.RefInput.prototype.updateLabel.call(this);
			if(! this.numLabel) {
				this.numLabel = MakeElt('div.text-muted.text-monospace.tally-partno');
			}
			this.numLabel.text(this.part_number);
			this.numLabel.appendTo(this.label_elt);
			this.numLabel.toggle(trim(this.part_number) !== '');
		},
	});


	UI.FieldInput = function(id, params) {
		this.mark_blank = true;
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		this.init(params);
	}
	UI.FieldInput.prototype = {
		init: function(params) {
			if(params) extendObject(this, params);
		},
		setup: function() {
			if(this.set_up) return;

			this.set_up = true;
			this.postSetup();
		},
		postSetup: function() {
			this.updateLabel();
			this.updateDisplay();
			this.addHooks();
		},
		getHooks: function() {
			return {
				keydown: this.handleKeyDown,
				keyup: this.handleKeyUp,
				mouseenter: this.handleMouseEnter,
				mouseleave: this.handleMouseLeave,
				mousedown: this.handleMouseDown,
				click: this.handleClick,
				focus: this.handleFocus,
				blur: this.handleBlur
			};
		},
		addHooks: function() {
			if(this.events || ! this.elt) return;
			this.events = UI.addEventListeners(this.elt, this.getHooks(), this);
		},
		handleKeyDown: function(evt) {
			if(evt.keyCode == 13) {
				if(! this.disabled) this.showPopup();
				UI.stopEvent(evt);
			} else if(evt.keyCode == 8 || evt.keyCode == 46) {
				// backspace/delete
				if(! this.disabled) this.clear();
			} else if(evt.keyCode == 27) {
				// esc
				if(this.opened) {
					this.hidePopup(true);
					this.focus();
					UI.stopEvent(evt);
				} else if(! this.disabled) {
					if(this.clear()) UI.stopEvent(evt);
				}
			} else if(evt.keyCode == 32) {
				// space
				if(this.focused && ! this.disabled) {
					this.showPopup();
					UI.stopEvent(evt);
				}
			}
		},
		handleMouseDown: function(evt) {
			if(! this.disabled && evt.currentTarget === this.elt)
				evt.stopPropagation();
		},
		handleClick: function(evt) {
			if(evt.which == 1) {
				if(this.opened)
					this.hidePopup();
				else if(! this.disabled)
					this.showPopup();
				UI.stopEvent(evt);
			}
		},
		handleMouseEnter: function(evt) {
			this.hovered = true;
			if(! isIPhone) this.updateDisplay(); // screws up input focus event in iOS
		},
		handleMouseLeave: function(evt) {
			this.hovered = false;
			if(! isIPhone) this.updateDisplay();
		},
		handleFocus: function(evt) {
			this.focused = true;
			UI.callEvent(this, this.onfocus, evt);
			this.updateDisplay();
		},
		handleBlur: function(evt) {
			if(this.focused) {
				this.focused = false;
				UI.callEvent(this, this.onblur, evt);
				this.updateDisplay();
			}
		},
		getIcon: function() {
		},
		getLabel: function() {
		},
		updateLabel: function() {
			if(this.icon_elt) {
				var icon = this.getIcon();
				if(icon) {
					if(typeof(icon) === 'string') {
						icon = createElement2('div', {className: icon});
					}
					Wrap(this.icon_elt).content(icon).show();
				} else {
					UI.setDisplayState(this.icon_elt, false);
				}
			}
			if(this.label_elt) {
				UI.setElementText(this.label_elt, this.getLabel());
			}
		},
		updateDisplay: function() {
			UI.addRemoveClass(this.label_elt, 'text-placeholder', this.mark_blank && this.isBlank() && ! this.focused && ! this.opened);
			UI.addRemoveClass(this.elt, 'invalid', !! this.invalid);
			UI.addRemoveClass(this.elt, 'opened', !! this.opened);
			UI.addRemoveClass(this.elt, 'readonly', !! this.disabled);
		},
		focus: function() {
			if(this.elt) this.elt.focus();
		},
		render: function() {
			var ret;
			if(! this.rendered) {
				if(! this.elt) {
					var cls = 'input-field input-field-group rbullet';
					if(this.block) cls += ' input-block';
					if(this.flat) cls += ' flat';
					if(this.ext_class) cls += ' ' + this.ext_class;
					this.elt = createElement2('div', {className: cls, id: this.id, tabIndex: this.tabIndex || 0});
				}
				if(! this.icon_elt) {
					this.icon_elt = createElement2('div', {className: 'input-label input-field-icon'});
				}
				this.elt.appendChild(this.icon_elt);
				if(! this.label_elt) {
					this.label_elt = createElement2('div', {className: 'input-label'});
				}
				this.elt.appendChild(this.label_elt);
				this.outer = MakeElt('div.input-wrapper' + (this.block ? '.input-block' : ''),
					this.elt).get();
				this.rendered = true;
				this.postSetup();
			}
			return this.outer || this.elt;
		},
		isBlank: function() {
			return ! this.getValue();
		},
		getValue: function() {
		},
		clear: function(refocus) {
		},
		isDirty: function() {
		},
		setValue: function(value, silent) {
			this.update({value: value, silent: silent});
		},
		update: function(params) {
			this.postUpdate(params);
		},
		postUpdate: function(params) {
			if(! params) params = {};
			this.updateLabel();
			if(this.isDirty()) UI.markFormDirty(this);
			if(! params.silent)
				UI.callEvent(this, this.onchange, params);
			this.validate();
		},
		userUpdate: function(params) {
			if(! this.focused && ! nofocus)
				this.focus();
			if(! params) params = {};
			params.by_user = true;
			this.update(params);
			var do_submit = (this.auto_submit && (! this.isBlank() || this.submit_empty));
			if(this.track_view)
				this.doTrackView();
			if(do_submit)
				UI.submitForm(this.form);
		},
		validate: function() {
			this.invalid = false;
			if(this.required && this.isBlank()) {
				this.invalid = true;
				this.invalidMsg = 'required';
			}
			else if(this.customValidate)
				this.customValidate();
			this.updateDisplay();
			return ! this.invalid;
		},
		showPopup: function() {
		},
		hidePopup: function() {
		},
		getTargetOffset: function() {
			return {top: -1};
		},
		destroy: function() {
			this.hidePopup();
			UI.removeNode(this.outer, true);
			UI.removeNode(this.elt, true);
			this.elt = this.outer = this.label_elt = null;
			this.rendered = false;
			this.set_up = false;
			this.focused = false;
			UI.callEvent(this, this.ondestroy);
		}
	};


	var PricingInput = function(id, params) {
		UI.FieldInput.call(this, id, params);
		this.ext_class = 'pricing-input';
		this.mark_blank = false;
		if(! this.pricing_options)
			this.pricing_options = UI.getSelectOptions('quote_pricing_method_dom');
		if(! this.pricing_short)
			this.pricing_short = UI.getSelectOptions('quote_pricing_method_short_dom');
	}
	extendClass(PricingInput, UI.FieldInput, {
		isBlank: function() {
			return true;
		},
		clear: function(refocus) {
			if(refocus) this.focus();
			if(! this.isBlank()) {
				//this.userUpdate(null, null);
				return true;
			}
		},
		getMethod: function() {
			return this.values && this.values.method;
		},
		getRate: function() {
			return this.values && this.values.rate;
		},
		getAmount: function() {
			return this.values && this.values.amount;
		},
		getTaxCode: function() {
			var id = this.values && this.values.tax_code_id, obj;
			if(id) {
				obj = SysData.taxcodes[id];
				if(obj) return obj.code;
			}
		},
		getIcon: function() {
			var method = this.getMethod();
			if(method === 'stddiscount' || method === 'stdfixeddiscount')
				return 'ref-icon input-icon theme-icon module-Discounts';
		},
		getLabel: function() {
			var method = this.getMethod(),
				rate = this.getRate(),
				did, disc, lbl;
			if(! method) method = '';
			if(method === 'stddiscount' || method === 'stdfixeddiscount') {
				did = this.values && this.values.related_id;
				lbl = '?';
				if(did) {
					disc = SysData.discounts[did];
					if(disc) lbl = disc.name;
					if(method === 'stdfixeddiscount')
						rate = stdFormatNumber(this.getAmount());
					else
						rate = stdFormatNumber(rate) + '%';
					lbl += ': ' + rate;
				}
			} else {
				lbl = this.pricing_short.getValue(method);
				if((method === 'discount' || method === 'stddiscount') && rate) {
					rate = -rate;
					if(rate > 0) lbl = this.pricing_short.getValue(method = 'premium');
				}
				if(method && method !== 'list' && method !== 'fixed') {
					lbl += ': ' + (method === 'premium' ? '+' : '') + stdFormatNumber(rate) + '%';
				}
			}
			return lbl;
		},
		updateLabel: function() {
			UI.FieldInput.prototype.updateLabel.call(this);
			UI.addClass(this.label_elt, 'text-number');
		},
		update: function(params) {
			this.values = params && params.value || {};
			this.postUpdate(params);
		},
		showSearch: function() {
			this.showPopup();
		},
		showPopup: function() {
			var self = this,
				popup = this.popup = new PricingPopup(this.id + '-popup', {
					target: this.elt,
					target_offset: this.getTargetOffset(),
					values: this.values,
					standard_discounts: get_default(this.standard_discounts, true),
					manual_discounts: get_default(this.manual_discounts, true),
					group_level: this.group_level,
					line: this.line,
				});
			popup.onshow = function() {
				self.opened = true;
				self.updateDisplay();
				popup.method_sel.focus();
			}
			popup.onhide = function() {
				self.opened = false;
				self.updateDisplay();
			}
			popup.onclear = function() {
				popup.close();
				self.update({value: {type: null, method: null}});
			}
			popup.onaccept = function() {
				var vals = popup.values;
				popup.close();
				self.update({value: vals});
			}
			popup.render();
			popup.show();
		},
		hidePopup: function() {
			if(this.popup) this.popup.close();
		}
	});

var PricingPopup = function(id, params) {
		this.standard_discounts = true;
		this.manual_discounts = true;
		UI.PopupSlidePanel.call(this, id, params);
		this.destroy_on_close = true;
		this.width = 250;
		if(! this.values) this.values = {};
	}
	extendClass(PricingPopup, UI.PopupSlidePanel, {
		getPriceOptions: function() {
			var opts = UI.getSelectOptions('quote_pricing_method_dom', undefined, undefined, true),
				key, disc, rate;
			if(! this.manual_discounts)
				opts.removeOption('discount');
			opts.removeOption('stddiscount');
			if(this.standard_discounts || this.group_level) {
				for(var i = 0; i < SysData.discounts_order.length; i++) {
					key = SysData.discounts_order[i];
					disc = SysData.discounts[key];
					if (disc.applies_to_selected) {
						var optKey = 'stddiscount~' + key;
						var product_id = 
							!this.group_level 
							? (this.line.related_type === 'ProductCatalog' 
									? this.line.related_id 
									: undefined
							)
							: undefined;
						if (this.group_level) {
							opts.removeOption(optKey);
							continue;
						} else if (!product_id) {
							opts.removeOption(optKey);
							continue;
						} else {
							if (!~disc.products.indexOf(product_id)) {
								opts.removeOption(optKey);
								continue;
							}
						}
					}
					if(key !== '-99') {
						key = 'stddiscount~' + key;
						if(disc.type === 'StdFixedDiscount')
							rate = stdFormatNumber(disc.amount); // formatCurrency(options[id].amount);
						else
							rate = stdFormatNumber(disc.rate) + '%';
						opts.addOption(key, {
							id: disc.id,
							type: disc.type,
							method: (disc.type === 'StdFixedDiscount') ? 'stdfixeddiscount' : 'stddiscount',
							discount: true,
							rate: disc.rate ? parseFloat(disc.rate) : 0,
							amount: disc.amount ? parseFloat(disc.amount) : 0,
							label: disc.name + ' (' + rate + ')'});
					}
				}
			}
			opts.width = this.width;
			return opts;
		},
		renderContent: function() {
			var header = this.header_elt = createElement2('div', {className: 'card-header panel-header'});
			var hrow = MakeElt('div.tools-row').appendTo(header);
			hrow.append(MakeElt('h5', mod_string('LBL_PRICING_EDIT', 'Quotes')));
			hrow.append(MakeElt('span.uii.uii-remove.uii-lg.active-icon', {tabIndex: 0})
				.on('click, keydown[key=enter|space]', this.handleClear, {scope: this}));
			hrow.append(MakeElt('span.uii.uii-accept.uii-lg.active-icon', {tabIndex: 0})
				.on('click, keydown[key=enter|space]', this.handleAccept, {scope: this}));
			this.elt.appendChild(header);

			this.return_link = MakeElt('a.tools-link', {href: '#', tabIndex: 0},
				MakeElt('span.uii.uii-previous'), nbsp(),
					(this.return_label = MakeElt('span')))
				.on('click', function(evt) {
					this.show_rate = false; this.updateDisplay(); UI.stopEvent(evt); }, {scope: this});
			this.return_row = MakeElt('div.card-header.panel-subheader',
				MakeElt('div.tools-row.popup-title.return-link', this.return_link)).hide().appendTo(this.elt);

			var content = this.content_elt = createElement2('div', {className: 'card-body panel-body'});
			var method = this.values.method;
			if(method === 'stddiscount' || method === 'stdfixeddiscount')
				method = 'stddiscount~' + this.values.related_id;
			this.method_sel = new UI.SelectList(this.id + '-selmethod', {
				flat: true,
				block: true,
				keyboard_select: true,
				mark_blank: false,
				no_scroll: true,
				ext_class: 'card-body panel-body card-outer',
				options: this.getPriceOptions(),
				tabIndex: 0,
				onchange: this.updateMethod.bind(this),
				rows: get_default(this.max_rows, 8)
			});
			this.method_sel.setSelectedKey(method, true);
			this.method_outer = Wrap(this.method_sel.render()).appendTo(this.elt);
			this.method_sel.setup();

			var form = new UI.FormRender({card: true, tagName: 'div'});
			this.rate_inp = new UI.TextInput(this.id + '-rate', {
				block: true,
				label: mod_string('LBL_PRICING_PERCENTAGE', 'Quotes'),
				size: 6,
				format: 'float',
				numFormat: {minimumDecimals: 2, decimalPlaces: 5},
				init_value: this.values.rate || 0,
				onchange: function() { this.values.rate = this.rate_inp.getValue(true); }.bind(this),
				onreturn: function() { this.handleAccept(); }.bind(this)
			});
			form.addSection({fields: [this.rate_inp]});
			form.elt.appendTo(content);
			this.form_outer = Wrap(content).hide().appendTo(this.elt);
			this.updateDisplay();
		},
		updateMethod: function() {
			var k = this.method_sel.getSelectedKey() || '',
				v = this.method_sel.getSelectedValue();
			if(v.discount) {
				this.values.method = v.method;
				this.values.type = v.type;
				this.values.related_type = 'Discounts';
				this.values.related_id = v.id;
				this.values.rate = v.rate;
				this.values.amount = v.amount;
			} else {
				this.values.method = k;
				this.values.type = UI.TallyEditor.pricing_method_map[k];
				this.values.related_type = '';
				this.values.related_id = '';
			}
			var method = this.values.method;
			this.show_rate = method && method !== 'stddiscount' && method !== 'stdfixeddiscount'
					&& method !== 'list' && method !== 'fixed' && method !== '';
			if(this.show_rate) {
				this.updateDisplay();
				requestAnimationFrame(function() {
					this.rate_inp.select();
				}.bind(this));
			} else {
				if(method !== 'stddiscount')
					this.values.rate = 0;
				this.handleAccept();
			}
		},
		updateDisplay: function() {
			this.method_outer.toggle(! this.show_rate);
			this.return_row.toggle(!! this.show_rate);
			this.form_outer.toggle(!! this.show_rate);
			if(this.show_rate)
				this.return_label.text(this.method_sel.getOptionLabel(this.values.method));
		},
		handleAccept: function() {
			UI.callEvent(this, this.onaccept);
		},
		handleClear: function() {
			UI.callEvent(this, this.onclear);
		}
	});


	UI.PanelRender = function(params) {
		this.style = 'default';
		this.border = true;
		this.card = false;
		this.round = true;
		this.readOnly = false;
		if(params) extendObject(this, params);
	}
	UI.PanelRender.prototype = {
		render: function() {
			if(! this.elt) {
				var cls = 'panel-outer' + (this.card ? ' card-outer': '');
				if(this.style) cls += ' panel-' + this.style;
				if(this.border) cls += ' panel-border';
				if(this.round) cls += ' panel-round';
				if(this.readOnly) cls += ' panel-readonly';
				if(this.ext_class) cls += ' ' + this.ext_class;
				this.elt = MakeElt('div', {id: this.id, 'class': cls});
			}
			return this.elt;
		},
		clear: function() {
			this.render().empty();
		},
		addChild: function(child, params) {
			if(! params) params = {};
			if(params.className) child.addClass(params.className);
			this.render().append(child);
			return child;
		},
		addHeader: function(params) {
			return this.addChild(MakeElt('div.panel-header' + (this.card ? '.card-header': '')), params);
		},
		addSubheader: function(params) {
			return this.addChild(MakeElt('div.panel-subheader' + (this.card ? '.card-header': '')), params);
		},
		addBody: function(params) {
			if(! params) params = {};
			return this.addChild(MakeElt('div.panel-body' + (this.card ? '.card-body': '') + (params.pad ? '.panel-pad-body' : '')), params);
		},
		addFooter: function(params) {
			return this.addChild(MakeElt('div.panel-footer' + (this.card ? '.card-footer': '')), params);
		}
	};

});


// compatibility methods
TallyEditor = {
	get_editor: function() {
		return SUGAR.ui.getFormInput('DetailForm', 'line_items');
	},
	tax_exempt_changed: function(cb, from_account) {
		var editor = TallyEditor.get_editor();
		if(editor) {
			return editor.taxExemptChanged(cb, from_account);
		}
	},
	tax_discount_changed: function(cb) {
		var editor = TallyEditor.get_editor();
		if(editor) {
			return editor.taxDiscountChanged(cb);
		}
	}
}


function returnPopupLineItem(data) {
	var parms = data.passthru_data,
		editor = SUGAR.ui.getFormInput(data.form_name, parms.ui_input_name);
	if(data.selection_list) {
		var lines = [];
		var fetch_attrs = [];
		var attrs = {};
		var seen = {};
		for(var idx in data.selection_list) {
			values = data.associated_row_data[idx];
			if(values.related_id) {
				// popup may return duplicates
				if(seen[values.related_id]) continue;
				seen[values.related_id] = 1;
			}
			if(parms.module == 'ProductCatalog' && values.related_id)
				fetch_attrs.push(values.related_id);
			lines.push(deep_clone(values));
		}
		editor.returnPopupLineItems(lines, parms);
	}
}


function renameLineItem(value, parms, redraw) {
	var line_id = get_default(parms.line_id, '');
	this.updateLineItem(parms.group_id, line_id, 'name', value);
}

function trunc(val, places, bankers) {
	if(! isset(val) || val === '') return 0.0;
	var b = Math.pow(10, isset(places) ? places : SUGAR.session.currency.significant_digits);
	var bval = val * b;
	if(b > 1) {
		if(! bankers) bval += (bval < 0 ? -0.1 : 0.1);
		else if(Math.floor(bval) % 2 == 0) bval -= 0.1;
	}
	return Math.round(bval) / b;
}

function hasProductAttributes(line) {
	if (typeof(line.adjusts) == 'object') {
		for (var i in line.adjusts) {
			if(line.adjusts[i].type == 'ProductAttributes')
				return true;
		}
	}
	return false;
}

function setDateFromTerm(form) {
	var getDate = function(days, date_entered) {
		var d;
        if (date_entered) {
            d = new Date(date_entered);
        } else {
            d = new Date();
        }
		while (days > 0) {
			var year = d.getFullYear();
			var mon = d.getMonth();
			var day = d.getDate();
			var max = d.getMonthDays(mon);
			if (day + days <= max) {
				day += days;
				days = 0;
			} else {
				days -= (max - day + 1);
				day = 1;
				mon++;
				if (mon > 12) {
					mon = 1;
					year++;
				}
			}
			d = new Date(year, mon, day, 0, 0, 0);
		}
		return d;
	}

    var UI = SUGAR.ui,
    	date_inp = UI.getFormInput(form, 'due_date'),
    	date_start_inp = UI.getFormInput(form, 'invoice_date') || UI.getFormInput(form, 'bill_date'),
    	terms_inp = UI.getFormInput(form, 'terms'),
    	date_start, term = 0, terms_value, term_matches;

    if (date_inp && date_start_inp && (date_start = date_start_inp.getValue(true)) && terms_inp) {
		terms_value = terms_inp.getValue();
		term_matches = terms_value.match(/(\d+) [dD]ays/);
		if(term_matches)
			term = parseInt(term_matches[1], 10);
		date_inp.setValue(getDate(term, date_start));
	}
}

