<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormTableSection.php');
require_once('include/Tally/TallyUpdate.php');

class TallyWidget extends FormTableSection {
	var $model_name;
	var $show_detail_extended = true;
	var $show_net_prices = false;

	var $pricing_method_map = array(
		'Inherit' => '', // 'inherit',
		'None' => '',
		'Margin' => 'margin',
		'Markup' => 'markup',
		'SameAsList' => 'list',
		'PercentDiscount' => 'discount',
		'StdPercentDiscount' => 'stddiscount',
		'StdFixedDiscount' => 'stddiscount',
	);
    

	function init($params=null, $model=null) {
		parent::init($params, $model);
		if($model)
			$this->model_name = $model->name;
		if(! $this->id)
			$this->id = 'line_items';
		if($this->model_name == 'Shipping' || $this->model_name == 'Receiving')
			$this->show_detail_extended = false;
		if(AppConfig::setting('company.quote_show_net_prices') &&
				in_array($this->model_name, ['Quote', 'Invoice', 'SalesOrder']))
			$this->show_net_prices = true;
	}
	
	function getRequiredFields() {
		return array('currency_id', 'exchange_rate', 'shipping_provider_id', 'date_entered');
	}
	
	function getLabel($context=null) {
		$lbl = parent::getLabel($context);
		if(! $lbl) {
			$mod = $this->model->getModuleDir();
			$lbl = translate('LBL_LINE_ITEMS', $mod);
		}
		return $lbl;
	}
	
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if($gen->getLayout()->getEditingLayout()) {
			$params = array();
			return $this->renderOuterTable($gen, $parents, $context, $params);
		}

		global $app_strings, $app_list_strings, $mod_strings, $pageInstance;

		$editable = $gen->getLayout()->getDefaultEditable();
		$layout = $gen->getLayout();
		$json = getJSONobj();
		
		$layout->addScriptLiteral('
			SysData = {
				taxrates: {},
				taxrates_order: [],
				taxcodes: {},
				taxcodes_order: [],
				discounts: {},
				discounts_order: [],
				get_tax_rates: function(clsid) {
					var ret = [];
					if(isset(this.taxcodes[clsid]))
						ret = this.taxcodes[clsid].rates;
					return ret;
				}
			};', LOAD_PRIORITY_BODY);
		
		require_once('modules/TaxCodes/TaxCode.php');
		$tax = new TaxCode();
		$layout->addScriptLiteral($tax->get_javascript(), LOAD_PRIORITY_FOOT);
        require_once('modules/TaxRates/TaxRate.php');
		$rate = new TaxRate();
		$layout->addScriptLiteral($rate->get_javascript(), LOAD_PRIORITY_FOOT);
		$pageInstance->add_js_model_cache('Currency');

		require_once('modules/Discounts/ListDiscounts.php');
		$disc = new ListDiscounts();
		$disc->lookupDiscounts('Active', '');
		$layout->addScriptLiteral($disc->getJavascript(), LOAD_PRIORITY_FOOT);
		
		$layout->addScriptInclude('modules/Quotes/quotes.js');
		$editParams = $this->getEditorParams($row_result, $editable);
		$gen->getFormObject()->addControl('TallyEditor', $this->id, $editParams);

		if(! $row_result->new_record && $editParams['can_edit']) {
			$btn_label = translate('LBL_EDIT_BUTTON_LABEL');
			$title_extra = ' <button type="button" class="input-button" id="tally-edit-btn" '
				. 'onclick="SUGAR.ui.getFormInput(\'' . $gen->getFormObject()->getFormName() . '\', \'line_items\').makeEditable();" '
				. 'style="margin-left: 2em">'
				. '<div class="input-icon icon-edit left"></div>'
				. '<span class="input-label" style="text-transform:none; letter-spacing:normal">'
				. to_html($btn_label) . '</span></button>';
		} else {
			$title_extra = '';
		}
		$title_extra .= '<span id="tax_exempt_warning" class="required"></span>';
		
		if(! $editable) {
			$gen->getFormObject()->addHiddenFields([
				'currency_id' => $row_result->getField('currency_id'),
				'exchange_rate' => $row_result->getField('exchange_rate')
			]);
		} 

		$params = [
			'title_extra' => $title_extra,
			'body' => '<div id="' . to_html($this->id) . '-outer"></div>',
		];
		return $this->renderOuterTable($gen, $parents, $context, $params);

		/*$body = $this->renderEditorBody(! $row_result->new_record);
		if (!$current_user->getPreference('product_costs')) {
			$body .= '<input type="hidden" name="gross_profit" id="DetailFormgross_profit-input" value="' . $row_result->getField('gross_profit', null, true) . '">';
			$body .= '<input type="hidden" name="gross_profit_pc" id="DetailFormgross_profit_pc-input" value="' . $row_result->getField('gross_profit_pc', null, true) . '">';
		}
		return $body;*/
	}
	
	function getEditorParams(RowResult $row_result, $can_edit=true) {
		global $current_user;
		
		$editable = !! $row_result->new_record;

		$dupe_id = $row_result->new_record ? $row_result->getField('id') : null;
		$targ = RowUpdate::for_result($row_result, $dupe_id);
		if ($row_result->getField('locked'))
			$can_edit = $editable = false;

		$add_comments = AppConfig::setting('company.quote_add_comments');
		$add_booked_hours_comments = AppConfig::setting('company.add_booked_hours', 0);
		$add_product_description = ($add_comments == 'products' || $add_comments == 'all');
		$add_assembly_description = ($add_comments == 'assemblies' || $add_comments == 'all');
		$assembly_discounts = AppConfig::setting('company.quote_assembly_discounts', 0);
		$add_related_events = AppConfig::setting('company.invoice_line_events', 0);
		$add_serials = AppConfig::setting('company.invoice_line_serials', 0);
		$show_pretax = AppConfig::setting('company.show_pretax_totals', 0);
		$show_net_prices = $this->show_net_prices;
		$tax_shipping = AppConfig::setting('company.tax_shipping', 0);
		$shipping_code = AppConfig::setting('company.shipping_tax_code', 'all');
		$pricebook_editable = array_get_default($row_result->row, 'pricebook_editable', 1);
		$date_entered = $targ->getField('date_entered');
		$default_pricing_method = AppConfig::setting('company.default_pricing_method', "");

		$catalog_only = true;
		$fixed_prices = true;
		if($current_user->getPreference('noncatalog_products')) {
			$catalog_only = false;
			if($current_user->getPreference('nonstandard_prices')) {
				$fixed_prices = false;
			}
		}
		$manual_discounts = !! $current_user->getPreference('manual_discounts');
		$standard_discounts = !! $current_user->getPreference('standard_discounts');
		$product_costs = !! $current_user->getPreference('product_costs');
		
		$default_discount = $targ->getField('default_discount_id');
		if($default_discount == '-99')
			$default_discount = null;
		$default_tax_code = $targ->getField('default_tax_code_id');
		if(! $default_tax_code)
			$default_tax_code = null;

		//Get default codes from Billing Account
		if ( (! $default_tax_code || ! $default_discount) && $row_result->getField('billing_account_id') ) {
			$account = $this->loadAccountDefaultCodes($row_result->getField('billing_account_id'));

			if ($account) {
				if (! $default_tax_code && $account->getField('tax_code_id'))
					$default_tax_code = $account->getField('tax_code_id');
				if (! $default_discount && $account->getField('default_discount_id'))
					$default_discount = $account->getField('default_discount_id');
			}

		}

		$add_initial = !! $row_result->new_record;
		$init_popup = null;
		$return_module = array_get_default($_REQUEST, 'return_module');

		if($return_module == 'Project' && ! $can_edit) {
			$init_popup = null;
		} elseif(($return_module == 'Cases' || $return_module == 'Project') && ($case_id = array_get_default($_REQUEST, 'return_record'))) {
			if(! AppConfig::setting('site.features.auto_add_case_line_items')) {
				$add_initial = false;
				$case_id = javascript_escape($case_id);
				$ret_model = AppConfig::module_primary_bean($return_module);
				if($ret_model) {
					$init_popup = [
						'filter' => [
							'parent_model' => $ret_model,
							'record' => $case_id,
						],
						'module' => 'Cases',
						'action' => 'InvoicePopup',
						'inline' => true,
						'width' => 700,
						'height' => 500,
					];
				}
			}
		} 

		$editor_params = array(
			'module' => $targ->getModuleDir(),
			'editable' => $editable,
			'can_edit' => $can_edit,
			'name' => $this->name,
			'label' => $this->getLabel(),
			'init_groups' => $this->getLineGroups($row_result),
			'date_entered' => $date_entered,
			'add_product_description' => $add_product_description,
			'add_assembly_description' => $add_assembly_description,
			'can_add_related_events' => $add_related_events,
			'add_serials' => $add_serials,
			'add_booked_hours_comments' => $add_booked_hours_comments,
			'add_assembly_discounts' => $assembly_discounts,
			'catalog_only' => $catalog_only,
			'fixed_prices' => $fixed_prices,
			'manual_discounts' => $manual_discounts,
			'standard_discounts' => $standard_discounts,
			'product_costs' => $product_costs,
			'default_discount_id' => $default_discount,
			'default_tax_code_id' => $default_tax_code,
			'default_tax_shipping' => $tax_shipping,
			'default_shipping_tax_code' => $shipping_code,
			'show_pretax_totals' => $show_pretax,
			'show_net_prices' => $show_net_prices,
			'add_initial_group' => $add_initial,
			'init_popup_options' => $init_popup,
			'pricebook_editable' => $pricebook_editable,
			'default_pricing_method' => $default_pricing_method,
		);
		
		return $editor_params;
	}

	function loadAccountDefaultCodes($id) {
		return ListQuery::quick_fetch('Account', $id, ['default_discount_id', 'tax_code_id']);
	}
	
	function getLineGroups(RowResult $row_result, $refetch=false) {
		$groups = $row_result->getRelatedData('line_groups');
		if(! $groups || $refetch) {
			$dupe_id = $row_result->new_record ? $row_result->getField('id') : null;
			$line_items = array_get_default($row_result->row, 'line_items', array());
			$targ = RowUpdate::for_result($row_result, $dupe_id);
			if (! empty($line_items))
				$targ->set('line_items', $line_items);
			$groups = $targ->getGroups();
		
			$attr_cache = array();
			foreach ($groups as $gid => &$group) {
				foreach ($group['adjusts'] as $adj_id => $adj) {
					if (! empty($adj['line_id'])) {
						foreach ($group['lines'] as $lid => $adj_line) {
							if ($adj_line['id'] == $adj['line_id']) {
								$groups[$gid]['lines'][$lid]['adjusts'][$adj['id']] = $adj;
								break;
							}
						}
						unset($groups[$gid]['adjusts'][$adj_id]);
					}
				}
				foreach ($group['lines'] as &$line) {
					if (isset($line['related_type']) && $line['related_type'] == 'ProductCatalog' && $line['related_id']) {
						if (!isset($attr_cache[$line['related_id']])) {
							$attrs = array();
							$lq = new ListQuery('Product', true, array('link_name' => 'productattributes'));
							$lq->setParentKey($line['related_id']);
							$res = $lq->runQuery();
							$rows = $res->getRows();
							foreach ($rows as $attr) {
								$attrs[$attr['name']][] = $attr;
							}
							$attr_cache[$line['related_id']] = $attrs;
						}
						$line['attributes_meta'] = $attr_cache[$line['related_id']];
					}
					if(isset($line['adjusts'])) {
						$line['adjusts'] = array_values($line['adjusts']);
					}
				}
				$groups[$gid]['lines'] = array_values($groups[$gid]['lines']);
				$groups[$gid]['adjusts'] = array_values($groups[$gid]['adjusts']);
			}
			$groups = array_values($groups);
		}
		$row_result->setRelatedData('line_groups', $groups);
		return $groups;
	}
	
	function validateInput(RowUpdate $update) {
		// need more flexible line management for this
		return true;
	}
	
	function renderPdf(PdfFormGenerator $gen, $row_result, array $parents, array $context)
	{
		$gen->pdf->set_focus($row_result);
		$gen->pdf->print_main();
	}
}

?>
