<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

$mod = array_get_default($_REQUEST, 'parent_type');
$pid = array_get_default($_REQUEST, 'parent_id');
$remove = array_get_default($_REQUEST, 'remove');

$model = AppConfig::module_primary_bean($mod);
if($model) {
	$record = ListQuery::quick_fetch($model, $pid);
}

if(empty($record))
	die('Record not found');

if($remove) {
	$rem = ListQuery::quick_fetch('RecordAction', $remove, true, array('acl_filters' => 'edit'));
	if($rem && $rem->getField('parent_type') == $mod && $rem->getField('parent_id') == $pid) {
		$rem->getRowUpdate()->markDeleted();
	}
}

$actions = new ListQuery('RecordAction', true);
$actions->setFormatted('html', 'view');
$actions->addSimpleFilter('parent_type', $mod);
$actions->addSimpleFilter('parent_id', $pid);
$actions->addSimpleFilter('hidden', 0);
$actions->addFields(array('assigned_user', 'related'));
$actions->setOrderBy('sequence, date_entered');

$result = $actions->runQuery();

$hier = array();
$seen = array();
$create_label = translate('LBL_CREATE_BUTTON_LABEL', 'RecordActions');
$run_pending_label = translate('LBL_RUN_ALL_PENDING', 'RecordActions');

echo <<<EOF
	<div class="card-outer form-outer default-form expand">
EOF;

if(AppConfig::setting('site.workflow_debug'))
echo <<<EOF
	<div class="card-header form-buttons">
		<div class="row buttons-row">
			<div class="column buttons-inline">
				<div class="inline-elt">
	<button class="input-button" type="button" onclick="SUGAR.popups.close(); SUGAR.util.navigateTo('index.php?module=RecordActions&action=EditView&parent_type={$mod}&parent_id={$pid}'); return false;"><div class="input-icon icon-accept left"></div><span class="input-label">{$create_label}</span></button>
				</div>
			</div>
		</div>
	</div>
EOF;

echo <<<EOF
	<div class="card-body form-body">
		<div class="form-section">
EOF;

foreach($result as $idx => $row) {
	$parent = $row->getField('depends_on_id');
	if($parent) $hier[$parent][] = $idx;
	$seen[$row->getField('id')] = $idx;
}

foreach($result as $idx => $row) {
	$parent = $row->getField('depends_on_id');
	$id = $row->getField('id');
	if($parent && isset($seen[$parent])) continue;
	renderAct($result, $hier, $idx);
}

if(! count($result))
	pr2(translate('LBL_NO_PENDING_ACTIONS', 'RecordActions'));

echo '</div></div>'; // form-section, form-body

if(AppConfig::setting('site.workflow_debug'))
echo <<<EOF
		<div class="card-footer form-buttons">
			<div class="row form-row">
				<div class="column form-cell">
					<div class="form-entry">
						<div class="form-value">
							<a href="?module=Workflow&action=Scheduler">{$run_pending_label}</a>
						</div>
					</div>
				</div>
			</div>
		</div>
EOF;

echo '</div>'; // form-outer

$title = javascript_escape(to_html(translate('LBL_RECORD_ACTIONS', 'RecordActions')));
echo <<<EOF
	<script>
		SUGAR.popups.initContent({title: "<div class=\"input-icon icon-view\"></div> {$title}", width: '500px', fill: true});
		function remRecAction(id) {
			var p = SUGAR.ui.PopupManager.currentModal();
			if(p) {
				var url = "async.php?module=RecordActions&action=PopupActions&parent_type={$mod}&parent_id={$pid}&remove="+id;
				p.fetchContent(url, {}, null, true);
			}
		}
	</script>
EOF;

function renderAct($result, $hier, $idx, $depth=0) {
	$row = $result->getRowResult($idx);
	$id = $row->getField('id');
	$left = $depth ? 'margin-left: '.($depth * 20).'px' : '';
	if($row->getField('status') == 'scheduled' && $row->getField('date_start')) {
		$sched = $row->getField('status', null, true) . translate('LBL_SEPARATOR', 'app') . $row->getField('date_start', null, true);
	} else
		$sched = $row->getField('status', null, true);
	if($row->getField('related')) {
		$relate = '<br>' . $row->getField('related', null, true);
	} else
		$relate = '';
	
	$outer = <<<EOF
		<div class="row form-row">
			<div class="column form-cell">
				<div class="form-entry">
					<div class="form-value">
						<div class="panel-outer panel-default panel-border" style="$left">
							<div class="panel-body panel-pad-body">
								<div class="row">
									<div class="column">
										<div class="input-icon theme-icon module-RecordActions"></div> <a href="index.php?module=RecordActions&action=DetailView&record={$id}" class="tabDetailViewDFLink">{$row->getField('type', null, true)}</a>
										<br>{$sched}
									</div>
									<div class="column text-right">
										<div class="uii uii-remove uii-md active-icon" onclick="remRecAction('{$id}'); return false;"></div><br>
										{$row->getField('assigned_user', null, true)}
										{$relate}
									</div>
								</div>
								<div class="row">
									<div class="column listLongText">
										{$row->getField('description', null, true)}
									</div>
								</div>
							</div>
						</div>
					</div>
				</div>
			</div>
		</div>
EOF;
	echo $outer;

	if(isset($hier[$id]))
		foreach($hier[$id] as $subidx)
			renderAct($result, $hier, $subidx, $depth+1);
}

