<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */



require_once('data/SugarBean.php');
require_once('include/utils.php');

define('DAY_LENGTH', 3600*24);

/*
function pull_pending(&$arr, $member, &$val) {
	if(!array_key_exists($member, $arr))
		return null;
	$succ = false;
	if(count($arr[$member])) {
		$val = array_shift($arr[$member]);
		$succ = true;
	}
	if(!count($arr[$member]))
		unset($arr[$member]);
	return $succ;
}*/

function split_int_list($val) {
	$vals = array();
	foreach(explode(',', $val) as $v)
		if(is_numeric($v))
			$vals[] = $v;
	$vals = array_unique($vals);
	sort($vals);
	return $vals;
}

function iso_year_week_info($wkst, $year) {
	$start = gmmktime(0, 0, 0, 1, 1, $year);
	$wd_start = gmdate('w', $start);
	$first_day_offset = $wkst - $wd_start;
	if($first_day_offset < -3) $first_day_offset += 7;
	else if($first_day_offset > 3) $first_day_offset -= 7;
	
	//if($wkst > $wd_start) $first_day_offset = -$first_day_offset;
	//if($first_day_offset > 3) $first_day_offset -= 7;
	//else if($first_day_offset < -3) $first_day_offset += 7;
	$num_days = ($year % 4 == 0 && ($year % 100 != 0 || $year % 400 == 0)) ? 366 : 365;
	$num_weeks = round(($num_days - $first_day_offset) / 7);
	$iso_year_start = $start + ($first_day_offset * DAY_LENGTH);
	$iso_year_end = $iso_year_start + ($num_weeks * 7) * DAY_LENGTH;
	$info = compact('first_day_offset', 'num_days', 'num_weeks', 'iso_year_start', 'iso_year_end');
	return $info;
}

function iso_week_for_day($wkst, $dt) {
	list($year, $year_day) = explode('|', gmdate('Y|z', $dt));
	$week_info = iso_year_week_info($wkst, $year);
	if($week_info['iso_year_start'] > $dt) {
		$iso_year = $year - 1;
		$week_info = iso_year_week_info($wkst, $iso_year);
		$iso_week = $week_info['num_weeks'];
	}
	else if($week_info['iso_year_end'] <= $dt) {
		$iso_year = $year + 1;
		$iso_week = 1;
	}
	else {
		$iso_year = $year;
		$iso_week = (int)floor(($year_day - $week_info['first_day_offset']) / 7) + 1;
	}
	$info = compact('iso_year', 'iso_week');
	return $info;
}

/**
 *
 */
class RecurrenceRule extends SugarBean {
	// database table columns
	var $id;
	var $date_entered;
	var $date_modified;
	var $deleted;
	
	var $freq;
	var $freq_interval = '1';
	var $until;
	var $limit_count;
	var $rule;
	var $is_restriction;
	var $instance_count;
	var $date_last_instance;
	var $forward_times;

	var $parent_type;
	var $parent_id;
	var $template;
	
	// runtime-only
	var $updated = false;
	var $parent_name;
	var $times;

	var $required_fields = array('freq'=>1, );

	var $object_name = 'RecurrenceRule';
	var $module_dir = 'Recurrence';
	var $new_schema = true;
	var $table_name = 'recurrence_rules';

	var $column_fields = array(
		'id',
		'date_entered',
		'date_modified',
		'parent_type',
		'parent_id',
		'freq',
		'freq_interval',
		'until',
		'limit_count',
		'rule',
		'is_restriction',
		'instance_count',
		'date_last_instance',
		'deleted',
	);

	var $list_fields = array(
		'id',
		'freq',
		'freq_interval',
		'limit_count',
		'until',
		'is_restriction',
		'rule',
	);

	var $additional_column_fields = Array();


	function toString() {
		return "RRULE:FREQ={$this->freq};{$this->rule}";
	}
	
	static function retrieve_rule($id, $filter_expired=false) {
		global $db;
		if(empty($id))
			return null;
		$lq = new ListQuery('RecurrenceRule');
		if ($filter_expired)
			$lq->addSimpleFilter(
				'limit_count',
				'(limit_count IS NULL OR limit_count = 0 OR IFNULL(instance_count,0) < limit_count)',
				'literal'
			);
		$result = $lq->queryRecord($id);
		if (!$result || $result->failed)
			return null;
		if (!preg_match('/^[0\-: ]+$/', $result->getField('until')))
			$result->assign('until', '');
		return $result;
	}
	
	function is_recurring($module, $id) {
		$query = "SELECT count(*) as c FROM `$this->table_name`"
			. " WHERE parent_type='". $module ."' AND parent_id='". $id ."'"
			. " AND NOT deleted";
		$result = $this->db->query($query, true, "Error retrieving recurrence information");

		if(! ($row = $this->db->fetchByAssoc($result)))
			return false;

		return $row['c'] > 0;
	}
	
	function get_recurrence_instance($module, $id) {
		$query = "SELECT MAX(IFNULL(instance_count, 0)) as c FROM `$this->table_name`"
			. " WHERE parent_type='". $module ."' AND parent_id='". $id ."'"
			. " AND NOT deleted";
		$result = $this->db->query($query, true, "Error retrieving recurrence information");
		if(! ($row = $this->db->fetchByAssoc($result)))
			return null;
		return $row['c'];
	}
	
	static function rule_to_JSON($rule) {
		require_once('include/JSON.php');
		$json = new JSON(JSON_LOOSE_TYPE);
		$ruleset = [];
		require_once('include/TimeDate.php');
		$timedate = new TimeDate();
		if ($rule) {
			$jsrule = [
				'id' => $rule->getField('id'),
				'freq' => $rule->getField('freq'),
				'interval' => $rule->getField('freq_interval'),
				'count' => $rule->getField('limit_count'),
				'until' => $rule->getField('until'),
				'is_restriction' => $rule->getField('is_restriction'),
			];
			$ruleset[] = array_merge($jsrule, self::split_rule($rule->getField('rule')));
		}
		$ret = '[]';
		if(count($ruleset))
			$ret = $json->encode($ruleset);
		return $ret;
	}
	
	function update_rules_from_JSON($recur_id, $model, $templateUpdate, $jsobj, $save=false) {
		if(! strlen($jsobj)) return;
		global $timedate;
		require_once('include/JSON.php');
		$json = new JSON(JSON_LOOSE_TYPE);
		$data = $json->decode($jsobj);
		$idx_by_id = array();
		$rule = self::retrieve_rule($recur_id);
		$field_map = array(
			'id' => 'id',
			'freq' => 'freq',
			'interval' => 'freq_interval',
			'until' => 'until',
			'count' => 'limit_count',
			'is_restriction' => 'is_restriction',
		);

		$ruleUpd = $rule ? RowUpdate::for_result($rule) : RowUpdate::blank_for_model('RecurrenceRule');
		if (!$rule) {
			$ruleUpd->set('parent_type', AppConfig::module_for_model($model));
		}
		$update = current($data);
		if (empty($update))
			return;
		$hasUpdated = false;
		if ($update) {
			foreach($field_map as $from => $to) {
				if(!array_key_exists($from, $update))
					continue;
				$val = $rule ? $rule->getField($to) : null;
				if($val !== $update[$from]) {
					$hasUpdated = true;
				}
				$newValue = (string)$update[$from];
				$ruleUpd->set($to, $newValue);
				unset($update[$from]);
				if($from == 'until' && $newValue != '') {
					/*
					if (strpos($newValue, 'T') === 8) {
						$timestamp = strtotime($newValue);
					} else {
						$timestamp = strtotime($newValue . " GMT");
					}
					$ruleUpd->set('until', gmdate("Y-m-d H:i:s", $timestamp));
					 */
					$until = $timedate->to_db($newValue, true, array_get_default($newTemplate, 'assigned_user_id'));
					$ruleUpd->set('until', $until);
				}
			}
		}
		ksort($update);
		$new_rule = self::join_rule($update);
		$oldRule = $rule ? $rule->getField('rule') : null;
		if(!isset($oldRule) || $oldRule != $new_rule) {
			$hasUpdated = true;
		}
		$ruleUpd->set('rule', $new_rule);
		$ruleUpd->set('id', $recur_id);

		if ($templateUpdate) {
			require_bean($model);
			$focus = new $model;
			$template = $rule ? @json_decode($rule->getField('template'), true) : [];
			if (!is_array($template)) $template = [];
			foreach ($focus->recur_chain_fields as $f) {
				if (array_key_exists($f, $templateUpdate)) {
					if (array_get_default($template, $f) !== $templateUpdate[$f]) {
						$template[$f] = $templateUpdate[$f];
						$hasUpdated = true;
					}
				}
			}
			$ruleUpd->set('template', json_encode($template));
		}

		if($save && $hasUpdated)
			$ruleUpd->save();
	}
	
	function save_rules(&$rules) {
		for(reset($rules); ($k = key($rules)) !== null; next($rules)) {
			$rules[$k]->save();
		}
	}
	
	static function split_rule($rule) {
		$parts = array();
		foreach(explode(';', $rule) as $p)
			if(preg_match('/([a-zA-Z]+)=(.*)/', $p, $r))
				$parts[strtolower($r[1])] = $r[2];
		return $parts;
	}
	
	static function join_rule($rule) {
		$parts = array();
		foreach($rule as $k=>$v) {;
			if(isset($v) && $v !== '')
				$parts[] = strtoupper($k)."=$v";
		}
		return implode(';', $parts);
	}
	
	// static method. enum_from inclusive, enum_to exclusive
	// called for daily, monthly, or yearly frequency (when byweekno is not set)
	static function &enum_days_monthly($dtstart, $enum_from, $enum_to, $freq, $interval, $rule, $timezone) {
		global $timedate;
		// need to fix interval
		$days = array();
		if($enum_to < $enum_from || $enum_to <= $dtstart)
			return $days;

		extract($rule);
		if(isset($bymonth)) {
			$bymonths = array();
			foreach(split_int_list($bymonth) as $m)
				if($m >= 1 && $m <= 12)
					$bymonths[] = $m;
		}
		else if($freq == 'YEARLY')
			$bymonths = array(gmdate('n', $dtstart));
		
		if(isset($byyearday))
			$byyeardays = split_int_list($byyearday);

		if(isset($bymonthday))
			$bymonthdays = split_int_list($bymonthday);
		else if($freq != 'DAILY' && !isset($byday))
			$bymonthdays = array(gmdate('j', $dtstart));
		
		$weekday_short = array('SU', 'MO', 'TU', 'WE', 'TH', 'FR', 'SA');
		if(isset($byday)) {
			$bydays = array();
			foreach(explode(',', $byday) as $bd) {
				if(preg_match('/(-?[0-9]+)?(SU|MO|TU|WE|TH|FR|SA)/', $bd, $match))
					$bydays[$match[1]][array_search($match[2], $weekday_short)] = 1;
			}
		}

		list($start_year, $start_month, $start_yearday) = explode('|', gmdate("Y|n|z", $dtstart));
		list($from_year, $from_month, $from_yearday) = explode('|', gmdate('Y|n|z', $enum_from));
		if($freq == 'YEARLY')
			$ival_pos = ($from_year - $start_year) % $interval;
		else if($freq == 'MONTHLY')
			$ival_pos = (($from_year - $start_year)*12 + ($start_month < $from_month ? 1 : -1) * ($from_month - $start_month)) % $interval;
		else { // DAILY
			$from_days = floor($enum_from / DAY_LENGTH / 60);
			$start_days = floor($dtstart / DAY_LENGTH / 60);
			$ival_pos = ($from_days - $start_days) % $interval;
		}

		$to_year = (int)gmdate('Y', $enum_to);
		$offset = $timedate->getTimeZoneOffset($timezone, true);
		$dtstart_time = $dtstart % DAY_LENGTH;
		$from_time = $enum_from % DAY_LENGTH;
		
		for($year = $from_year; $year <= $to_year; $year++) {
			$year_num_days = ($year % 4 == 0 && ($year % 100 != 0 || $year % 400 == 0)) ? 366 : 365;
			if(isset($byyeardays)) {
				$year_days = array();
				foreach($byyeardays as $yd)
					$year_days[] = ($yd < 0 ? $year_num_days + $yd + 1: $yd);
			}
			else	unset($year_days);
			
			if($freq == 'YEARLY' && isset($bydays)) {
				// NOTE: if freq=YEARLY, we could have BYDAY=20MO (twentieth monday in the year) - not handled
			}
			
			for($m = ($year == $from_year ? $from_month : 1); $m <= 12; $m++) {
				
				if(isset($bymonths) && !in_array($m, $bymonths)) {
					if($freq == 'MONTHLY')
						$ival_pos = ($ival_pos + 1) % $interval;
					continue;
				}
				
				$month_start = gmmktime(0, 0, 0, $m, 1, $year);
				list($yearday, $weekday, $num_days) = explode('|', gmdate("z|w|t", $month_start));
				
				if(isset($bymonthdays)) {
					foreach($bymonthdays as $md) {
						$md = $md < 0 ? $num_days + $md + 1 : $md;
						if($md > $num_days || $md < 1)
							continue;
						$month_days[] = $md;
					}
					$month_days = array_unique($month_days);
					sort($month_days);
				}
				else	unset($month_days);
				
				if(($freq == 'DAILY' || $freq == 'MONTHLY' || ($freq == 'YEARLY' && isset($bymonths))) && isset($bydays)) {
					$count_by_wd = array();
					$month_bydays = array();
					for($i = 0; $i < 7; $i++)
						$count_by_wd[($weekday + $i) % 7] = (int)floor(($num_days - $i  - 1) / 7) + 1;
					foreach($bydays as $pos => $wdays) {
						foreach(array_keys($wdays) as $wd) {
							if(is_numeric($pos) && $pos < 0) {
								$pos = $count_by_wd[$wd] + $pos + 1;
								if($pos < 1) continue;
							}
							$daynum = $wd - $weekday + 1;
							if($daynum < 1) $daynum += 7;
							if($pos == '')
								for($i = 0; $i < $count_by_wd[$wd]; $i++) {
									$month_bydays[] = $daynum;
									$daynum += 7;
								}
							else
								$month_bydays[] = $daynum + 7 * ($pos-1);
						}
					}
					$month_bydays = array_unique($month_bydays);
					sort($month_bydays);
				}
				else	unset($month_bydays);

				$day_start = $month_start;
				for($md = 1; $md <= $num_days; $md++) {
					$dt = $day_start + $dtstart_time;

					if ($dtstart_time + $offset >= DAY_LENGTH)
						$dt -= DAY_LENGTH;
					else if ($dtstart_time + $offset < 0)
						$dt += DAY_LENGTH;

					if( $dt >= $enum_from
					    && (!isset($year_days) || in_array($yearday, $year_days))
					    && (!isset($month_days) || in_array($md, $month_days))
					    && (!isset($month_bydays) || in_array($md, $month_bydays))
					) {
						if($dt >= $enum_to)
							return $days;

						if($ival_pos == 0) {
							$days[] = $dt;
						}
						if($freq == 'DAILY')
							$ival_pos = ($ival_pos + 1) % $interval;
					}
					$day_start += DAY_LENGTH;
					$yearday++;
				}
				
				if($freq == 'MONTHLY')
					$ival_pos = ($ival_pos + 1) % $interval;
			}
			
			if($freq == 'YEARLY')
				$ival_pos = ($ival_pos + 1) % $interval;
		}
		return $days;
	}

	// called for weekly frequency, or yearly when byweekno is set
	static function &enum_days_weekly($dtstart, $enum_from, $enum_to, $freq, $interval, $rule, $timezone) {
		global $timedate;
		$days = array();
		if($enum_to < $enum_from || $enum_to <= $dtstart /* || $count < 1*/)
			return $days;

		extract($rule);
		if(isset($bymonth)) {
			$bymonths = array();
			foreach(split_int_list($bymonth) as $m)
				if($m >= 1 && $m <= 12)
					$bymonths[] = $m;
		}
		
		if(isset($byyearday))
			$byyeardays = split_int_list($byyearday);

		if(isset($bymonthday))
			$bymonthdays = split_int_list($bymonthday);
		
		if(isset($byweekno))
			$byweeknos = split_int_list($byweekno);
		
		$weekday_short = array('SU', 'MO', 'TU', 'WE', 'TH', 'FR', 'SA');
		if(isset($byday)) {
			$bydays = array();
			foreach(explode(',', $byday) as $bd) {
				if(preg_match('/(-?[0-9]+)?(SU|MO|TU|WE|TH|FR|SA)/', $bd, $match))
					$bydays[$match[1]][array_search($match[2], $weekday_short)] = 1;
			}
		}
		
		if(!isset($wkst) || !in_array($wkst, $weekday_short))
			$wkst = 'MO';
		$wkst_no = array_search($wkst, $weekday_short);
		
		if($freq == 'YEARLY') {
			list($start_year, $start_month, $start_yearday, $start_weekday) = explode('|', gmdate("Y|n|z|w", $dtstart));
			list($from_year, $from_month, $from_yearday) = explode('|', gmdate('Y|n|z', $enum_from));
			if(!isset($bydays))
				$bydays = array(); // ?
			$ival_pos = ($from_year - $start_year) % $interval;
			$to_year = (int)gmdate('Y', $enum_to);
		}
		else { // WEEKLY
			$start_weekday = gmdate("w", $dtstart);
			if(!isset($bydays))
				$bydays = array('' => array($start_weekday => 1));
			$ival_pos = 0;
			list($start_year, $start_weekno) = array_values(iso_week_for_day($wkst_no, $dtstart));
			list($from_year, $from_weekno) = array_values(iso_week_for_day($wkst_no, $enum_from));
			for($year = $start_year; $year < $from_year; $year++) {
				$year_week_info = iso_year_week_info($wkst_no, $year);
				$ival_pos = ($ival_pos + $year_week_info['num_weeks']) % $interval;
			}
			$ival_pos = ($ival_pos + ($start_weekno < $from_weekno ? 1 : -1) * ($from_weekno - $start_weekno)) % $interval;
			list($to_year,) = array_values(iso_week_for_day($wkst_no, $enum_to));
		}
		
		$offset = $timedate->getTimeZoneOffset($timezone, true);
		$dtstart_time = $dtstart % DAY_LENGTH;
		
		for($year = $from_year; $year <= $to_year; $year++) {
			$year_week_info = iso_year_week_info($wkst_no, $year);
			$num_weeks = $year_week_info['num_weeks'];

			if(isset($byyeardays)) {
				$year_days = array();
				foreach($byyeardays as $yd)
					$year_days[] = ($yd < 0 ? $year_num_days + $yd + 1: $yd);
			}
			else	unset($year_days);
			
			if(isset($byweeknos)) {
				$year_weeknos = array();
				foreach($byweeknos as $wn)
					$year_weeknos[] = ($wn < 0 ? $num_weeks + $wn + 1: $wn);
			}
			else	unset($year_weeknos);
			
			$year_bydays = array();
			foreach($bydays as $pos => $wdays) {
				foreach(array_keys($wdays) as $wd) {
				if(is_numeric($pos) && $pos < 0) {
						$pos = $num_weeks + $pos + 1;
						if($pos < 1) continue;
					}
					$year_bydays[$pos][$wd] = 1;
				}
			}
			
			$day_start = $year_week_info['iso_year_start'];
			$yearday = $year_week_info['first_day_offset'];
			for($weekno = 1; $weekno <= $num_weeks; $weekno++) {
			
				if($year == $from_year && $weekno < $from_weekno) {
					$day_start += 7 * DAY_LENGTH;
					$yearday += 7;
					continue;
				}

				if(isset($year_weeknos) && !in_array($weekno, $year_weeknos)) {
					$day_start += 7 * DAY_LENGTH;
					$yearday += 7;
					if($freq == 'WEEKLY')
						$ival_pos = ($ival_pos + 1) % $interval;
					continue;
				}

				for($wdi = 0; $wdi < 7; $wdi++) {
					$wd = ($wkst_no + $wdi) % 7;
					$dt = $day_start + $dtstart_time;
					if ($dtstart_time + $offset >= DAY_LENGTH)
						$dt -= DAY_LENGTH;
					else if ($dtstart_time + $offset < 0)
						$dt += DAY_LENGTH;

					if($dt >= $enum_to)
						return $days;

					$bymonthcheck = true;
					// FIXME

					if( $dt >= $enum_from
					    && (!isset($year_days) || in_array($yearday, $year_days))
					    && ((!empty($year_bydays['']) && !empty($year_bydays[''][$wd]))
						|| (!empty($year_bydays[$weekno]) && !empty($year_bydays[$weekno][$wd])) )
					    && $bymonthcheck)
					{
						if($ival_pos == 0)
							$days[] = $dt;
					}
					
					$day_start += DAY_LENGTH;
					$yearday ++;
				}
				
				if($freq == 'WEEKLY')
					$ival_pos = ($ival_pos + 1) % $interval;
			}
			if($freq == 'YEARLY')
				$ival_pos = ($ival_pos + 1) % $interval;
		}
		
		return $days;
	}
	
	
	// $dtstart is the beginning of the recurrence
	// $enum_from and $enum_to define the range of time we're interested in
	// must be passed in seconds from unix epoch UTC
	static function get_recurrence_times($rule, $dtstart, $enum_from, $enum_to, $max_count, $timezone) {
		$parts = self::split_rule($rule['rule']);
		$count = array_get_default($rule, 'limit_count', '');
		if($count === '' || $count > $max_count)
			$count = $max_count;
		if(!empty($rule['until'])) {
			$until = strtotime($rule['until'] . ' GMT');

			if($until < $enum_to)
				$enum_to = $until+1;
		}
		// debug code
		/*for($year = 1995; $year < 2000; $year++) {
			print $year.'<br>';
			$info = iso_year_week_info(1, $year);
			print_r(iso_year_week_info(1, $year)); print '<br>';
			print gmdate('Y-m-d H:i:s', $info['iso_year_start']) .'<br>';
			print gmdate('Y-m-d H:i:s', $info['iso_year_end']) .'<br>';
			print '<blockquote>';
			for($d = -9; $d < 10; $d++) {
				$dt = gmmktime(0,0,0, 1, 1, $year);
				print_r(iso_week_for_day(1, $dt + $d * DAY_LENGTH));
				if($d == 0) $GLOBALS['log']->fatal(' &lt;-');
				$GLOBALS['log']->fatal('<br>');
			}
			print '</blockquote>';
			print '<br>';
		}*/
		/*
		for($d = -9; $d < 10; $d++) {
			$dt = gmmktime(0,0,0, 1, 1, 1994);
			print_r(iso_week_for_day(1, $dt + $d * DAY_LENGTH));
			if($d == 0) $GLOBALS['log']->fatal(' &lt;-');
			$GLOBALS['log']->fatal('<br>');
		}*/
		if($rule['freq'] == 'WEEKLY' || ($rule['freq'] == 'YEARLY' && !empty($parts['byweekno'])))
			$fn = 'enum_days_weekly';
		else
			$fn = 'enum_days_monthly';
		$days = self::$fn($dtstart, $enum_from, $enum_to,
			$rule['freq'], $rule['freq_interval'], $parts, $timezone);
		if($max_count)
			$days = array_slice($days, 0, $max_count);
		return $days;
	}
	
	static function date_to_timestamp($date, $time='', $db_f=false) {
		global $timedate;
		if($time != '')
			$date = $timedate->merge_date_time($date, $time);
		$dt = $db_f ? $date : $timedate->to_db($date);
		return strtotime($dt."Z");
	}
	
	static function timestamp_to_display_date($date) {
		global $timedate;
		$dt = gmdate($timedate->get_db_date_time_format(), $date);
		return $timedate->to_display_date_time($dt);
	}

	function _fill_in_additional_parent_fields()
	{
		global $app_strings;

		switch($this->parent_type)
		{
			case 'Meetings':
				require_once("modules/Meetings/Meeting.php");
				$parent = new Meeting();
				break;
		}

		if(!empty($parent))
		{
			$parent->retrieve($this->parent_id);
			$this->parent_name = $parent->name;
			$parent->cleanup();
		}
	}
	
	
	static function getCurrentRecurring($can_die=true) {
		$lq = new ListQuery('RecurrenceRule');
		$lq->addSimpleFilter('limit_count', 'IFNULL(limit_count, 0) = 0 OR IFNULL(instance_count,0) < limit_count', 'literal');
		$result = $lq->runQuery(0, null, false, null, 100);
		return $result->getIterator();
	}

	static function remove_instances(RowUpdate $rule) {
		$bean_name = AppConfig::module_primary_bean($rule->getField('parent_type'));
		if (!$bean_name) return;
		$ruleId = $rule->getField('id');
		$lq = new ListQuery($bean_name);
		$lq->addSimpleFilter('recurrence_rule_id', $ruleId);
		$result = $lq->runQuery(0, null, false, null, 100);
		$it = $result->getIterator();
		foreach ($it as $row) {
			$u = RowUpdate::for_result($row);
			$u->setRelatedData('break_sequence', 'none');
			$u->set('recurrence_exception', 0);
			$u->markDeleted();
		}
	}

	static function before_save(RowUpdate $upd) {
		list($recreate, $doUpdate, $truncate, $updated) = self::get_rule_update_params($upd);
		if ($recreate) {
			$upd->set('instance_count', 0);
			$upd->set('date_last_instance', '');
		}
		if(!version_compare(phpversion(), "5.4", ">="))
			return;
		if (!$upd->getField('id')) {
			$id = create_guid();
			$upd->set('id', $id);
			$upd->set('vcal_uri', $id . '.ics');
		}
		$vcal = vCalUtils::vcal_update($upd, null, true);
		$upd->set('vcal', $vcal);
		$upd->set('vcal_size', strlen($vcal));
	}

	static function update_recurrence(RowUpdate $upd) {
		list($recreate, $doUpdate, $truncate, $updated) = self::get_rule_update_params($upd);
		if ($recreate) 
			self::recreate_instances($upd);
		elseif ($doUpdate)
			self::update_instances($upd, $updated);
		if (!$recreate && $truncate) {
			self::truncate($upd, $truncate);
		}
	}

	static function truncate(RowUpdate $upd, $truncate) {
		global $db;
		$ruleId = $upd->getField('id');
		$template = @json_decode($upd->getField('template'), true);
		$start = array_get_default($template, 'date_start');

		$lq = new ListQuery('Meeting');
		$lq->addSimpleFilter('recurrence_rule_id', $ruleId);
		$lq->addSimpleFilter('recurrence_start_time', $truncate, '>');
		$result = $lq->runQuery(0, null, false, null, 100);
		$it = $result->getIterator();
		foreach ($it as $row) {
			$u = RowUpdate::for_result($row);
			$u->setRelatedData('break_sequence', 'none');
			$u->set('recurrence_rule_id', '');
			$u->markDeleted();
		}

		if ($start > $truncate) {
			$remove = RowUpdate::blank_for_model('RecurrenceRule');
			$remove->new_record = false;
			$remove->setOriginal(['id' => $ruleId, 'deleted' => 0]);
			$remove->markDeleted();
		} else {
			$lq = new ListQuery('Meeting');
			$lq->addSimpleFilter('recurrence_rule_id', $ruleId);
			$lq->addTotal(array('name' => 'recurrence_start_time', 'alias' => 'date_last_instance', 'total' => 'max'));
			$lq->addTotal(array('name' => 'recurrence_index', 'alias' => 'instance_count', 'total' => 'max'));
			$res = $lq->runSummationQuery();
			$date_last_instance = 
				empty($res->rows[0]['date_last_instance']) 
				? 'NULL'
				: "'" . $db->quote($res->rows[0]['date_last_instance']) . "'"
			;
			$query = sprintf(
				"UPDATE recurrence_rules SET date_last_instance = %s, instance_count = %d  WHERE id = '%s'",
				$date_last_instance,
				$db->quote(max(($res->rows[0]['instance_count'] ?? 0) - 1, 0)),
				$ruleId
			);
			$db->query($query);
		}
	}

	static function get_rule_update_params(RowUpdate $upd) {
		global $timedate;

		$compare_recur_fields = ['rule', 'freq', 'freq_interval'];

		$recurChanged = false;
		foreach ($compare_recur_fields as $f) {
			$orig = $upd->getFieldValue($f, '', true);
			$new = $upd->getFieldValue($f, '', false);
			if ($orig != $new) {
				$recurChanged = true;
				break;
			}
		}
		
		$origUntil = $upd->getFieldValue('until', '', true);
		$newUntil = $upd->getFieldValue('until', '', false);
		$origTemplate = @json_decode($upd->getFieldValue('template', '', true), true);
		$newTemplate = @json_decode($upd->getFieldValue('template', '', false), true);
		if (!is_array($origTemplate)) $origTemplate = [];
		if (!is_array($newTemplate)) $newTemplate = [];
		$origDate = array_get_default($origTemplate, 'date_start');
		$newDate = array_get_default($newTemplate, 'date_start');


		$userOrigDate = $timedate->to_display_date($origDate, true, array_get_default($newTemplate, 'assigned_user_id'));
		$userNewDate = $timedate->to_display_date($newDate, true, array_get_default($newTemplate, 'assigned_user_id'));

		$recreate = 
			$userOrigDate != $userNewDate
			|| $recurChanged
		;


		$truncate = false;
		if (!$recreate) {
			if (!$origUntil && $newUntil || $origUntil > $newUntil) {
				$truncate = $newUntil;
			}
		}
		
		$doUpdate = false;
		$updated = [];
		foreach ($newTemplate as $f => $v) {
			if ($v != array_get_default($origTemplate, $f)) {
				$updated[$f] = $v;
				$doUpdate = true;
			}
		}

		return [$recreate, $doUpdate, $truncate, $newTemplate];
	}


	static function update_instances(RowUpdate $rule, array $updates) {
		global $timedate;
		$bean_name = AppConfig::module_primary_bean($rule->getField('parent_type'));
		if ($bean_name !== 'Meeting') return;
		require_bean('Meeting');
		$template = @json_decode($rule->getField('template'), true);
		$user_id = array_get_default($template, 'assigned_user_id');
		$ruleId = $rule->getField('id');
		$lq = new ListQuery($bean_name);
		$lq->addSimpleFilter('recurrence_rule_id', $ruleId);
		$result = $lq->runQuery(0, null, false, 'recurrence_start_time', 100);
		$it = $result->getIterator();
		foreach ($it as $row) {
			$update = $updates;
			if (isset($update['date_start'])) {
				$user_id = $row->getField('assigned_user_id');
				$userOrigDate = $timedate->to_display_date($row->getField('date_start'), true, $user_id);
				$userNewTime = $timedate->to_display_time($update['date_start'], true);
				$update['date_start'] = $timedate->to_db($userOrigDate . ' ' . $userNewTime, true, $user_id);
				$update['recurrence_start_time'] = $update['date_start'];
			}
			$u = RowUpdate::for_result($row);
			$u->setRelatedData('break_sequence', 'none');
			$u->set($update);
			$u->set('recurrence_exception', 0);
			$u->save();
		}
	}

	static function recreate_instances(RowUpdate $rule, $scheduled = false) {
		global $db, $timedate;
		$bean_name = AppConfig::module_primary_bean($rule->getField('parent_type'));
		if($bean_name)
			require_bean($bean_name);
		else
			return;

		$ruleId = $rule->getField('id');


		// if a recurring item was created from the UI, the first instance is already created,
		// and this method is invoked after the recurrence widget created  the recurrence
		// rule. Otherwise, we need to create the first instance
		$skipFirstInstance = $rule->getRelatedData('skip_first_instance', true);
		
		
		if ($ruleId && !$skipFirstInstance) {
			$lq = new ListQuery($bean_name);
			$lq->addSimpleFilter('recurrence_rule_id', $ruleId);
			$result = $lq->runQuery(0, null, false, null, 100);
			$it = $result->getIterator();
			foreach ($it as $row) {
				$u = RowUpdate::for_result($row);
				$u->setRelatedData('break_sequence', 'none');
				$u->set('recurrence_rule_id', '');
				$u->markDeleted();
			}
		}


		$focus = new $bean_name;
		$template = @json_decode($rule->getField('template'), true);
		$relTpl = self::load_relations_template($bean_name, $ruleId);

		if ($scheduled) {
			$max_instance_count = intval($rule->getField('instance_count'));
		} else {
			$max_instance_count = $skipFirstInstance ? 0 : -1;	
		}

		$max_forward = $focus->get_recurrence_forward_instances();
		$max_invocation_create = $max_forward;
		$forward_times = [];
		$date_field = $focus->get_recurrence_date_field();
		$empty_time = "00:00:00";

		$time = time();
		$enum_to = $time + $focus->get_recurrence_scheduled_interval();

		$count = $rule->getField('limit_count');
		if($count)
			$count = min(intval($count) - intval($rule->getField('instance_count')), $max_invocation_create);
		else
			$count = $max_invocation_create;

		if ($bean_name == 'Invoice') {
			$start = self::date_to_timestamp($template[$date_field], $empty_time, true);
		} else {
			$start = self::date_to_timestamp($template[$date_field], '', true);
		}

		if ($scheduled) {
			$date_last_instance = $rule->getField('date_last_instance');
			if(empty($date_last_instance))
				$enum_from = $start;
			else {
				$last_instance = strtotime($date_last_instance . ' GMT');
				$enum_from = $last_instance;
				if ($last_instance < $start)
					$enum_from = $start;
			}
		} else {
			$enum_from = $start;
		}

		$until = $rule->getField('until');
		if ($until) {
			$to = strtotime($until . ' GMT');
		} else {
			$to = $enum_to;
		}
		$timezone = $timedate->getUserTimeZone($template['assigned_user_id']);
		$times = self::get_recurrence_times($rule->getRow(), $start, $enum_from + 60, $to, $count, $timezone);

		foreach ($times as $i => $tm) {
			if ($tm <= $enum_from) unset($times[$i]);
		}
		
		if (!$skipFirstInstance) {
			array_unshift($times, $start);
		}

		$instance_count	= $rule->getField('instance_count');
		$date_last_instance	= $rule->getField('date_last_instance');
		$created = 0;
		foreach($times as $add_time) {
			if (count($forward_times) >= $max_forward) break;
			if ($add_time > $time) {
				$forward_times[] = $add_time;
			}

			$new_upd = RowUpdate::blank_for_model($bean_name);
			$max_instance_count ++;
			$db_dt = gmdate($timedate->get_db_date_time_format(), $add_time);

			$update = $template;
			unset($update['id']);
			unset($update['date_entered']);
			unset($update['date_modified']);
			$update['recurrence_rule_id'] = $rule->getField('id');
			$update['recurrence_index'] = $max_instance_count;
			$update[$date_field] = $db_dt;
			$update['recurrence_start_time'] = $db_dt;

			$new_upd->set($update);
			$new_upd->save();

			if (! empty($focus->recur_copy_relations)) {
				if ($relTpl) {
					$activityInvite = new ActivityInvite($bean_name);
					$invitees = $activityInvite->getList($relTpl);

					if (sizeof($invitees) > 0) {
						for ($i = 0; $i < sizeof($invitees); $i++) {
							$link = $focus->recur_copy_relations[$invitees[$i]['module']];
							if (! empty($link))
								$new_upd->addUpdateLink($link, $invitees[$i]['id']);
						}
					}
				}
			}

			$created ++;
			$instance_count ++;
			$date_last_instance = gmdate("Y-m-d H:i:s", $add_time);
		}


		if ($created) {
			// not using RowUpdate intentionally to avoid triggering the hooks
			$db->query(
				sprintf(
					"UPDATE recurrence_rules SET date_last_instance = '%s', instance_count = %d  WHERE id = '%s'",
					$db->quote($date_last_instance),
					$instance_count,
					$ruleId
				)
			);
		}
		$focus->cleanup();
	}

	static function load_relations_template($model, $recurrenceId) {
		$lq = new ListQuery($model);
		$lq->addSimpleFilter('recurrence_exception', null, 'empty');
		$lq->addSimpleFilter('recurrence_rule_id', $recurrenceId);
		$lq->setOrderBy('recurrence_index');
		return $lq->runQuerySingle();
	}


}

