<?php

class RecurrenceUpdate875 {
	// Upgrade recurring meetings to new data sctructure that exists since 8.7.5.
	// The upgrade is done in 2 steps.
	//
	// First, all recurring meetings that have more than 1 recurrence rule are
	// converted to non-recurring meetings.
	//
	// Second, recurring meetings with 1 rule are modified so that they contain
	// a reference to the recurrence rule. Records in recurrence_rules table
	// are also modified to include the meeting template. 
	static function upgrade_875() {

		global $db;

		// Step 1

		// find multiple recurrence
		$res = self::findMultipleRecurrence();
		while ($row = $db->fetchByAssoc($res)) {
			// clears recurrence_of_id
			self::convertToNonRecurring($row['parent_id']);
			// deletes the recurrence rules
			self::deleteMultiRecurring($row['parent_id']);
		}

		// Step 2

		// find single recurrence
		$res = self::findRecurrence();
		while ($row = $db->fetchByAssoc($res)) {
			// sets recurrence_rule_id
			self::setRecurrenceRuleId($row['id'], $row['parent_id']);
			$mRow = self::getFirstInstance($row['parent_id']);
			if (!$mRow) {
				self::deleteRecurring($row['id']);
				continue;
			}

			$template = self::createTemplate($mRow);
			self::updateSyncRecord($row['parent_id'], $row['id']);
			self::updateTemplate($row['id'], $template);
		}
	}

	static function findMultipleRecurrence() {
		global $db;
		$query = 
			"SELECT parent_id, COUNT(DISTINCT `id`) as `cnt`"
			. " FROM `recurrence_rules` WHERE parent_type = 'Meetings' AND `deleted`=0"
			. " GROUP BY `parent_id` HAVING cnt > 1";

		return $db->query($query, true);
	}

	static function findRecurrence() {
		global $db;
		$query = 
			"SELECT id, parent_id"
				. " FROM `recurrence_rules` WHERE parent_type = 'Meetings' AND `deleted`=0"
			;

		return $db->query($query, true);
	}

	static function setRecurrenceRuleId($rule_id, $recurrence_of_id) {
		global $db;
		$db->query(
			sprintf(
				"UPDATE meetings SET recurrence_rule_id = '%s', recurrence_start_time = date_start"
			   . " WHERE (recurrence_of_id = '%s' OR id = '%s') AND deleted = 0 ",
				$db->quote($rule_id),
				$db->quote($recurrence_of_id),
				$db->quote($recurrence_of_id)
			)
		);
	}

	static function convertToNonRecurring($recurrence_of_id) {
		global $db;
		$db->query(
			sprintf(
				"UPDATE meetings SET recurrence_of_id = NULL, date_modified = '%s'"
					. " WHERE (id = '%s' OR recurrence_of_id = '%s') AND deleted = 0",
				gmdate("Y-m-d H:i:s"),
				$db->quote($recurrence_of_id),
				$db->quote($recurrence_of_id)
			)
		);
	}

	static function deleteMultiRecurring($recurrence_of_id) {
		global $db;
		$db->query(
			sprintf(
				"DELETE FROM recurrence_rules WHERE parent_type = 'Meetings' AND parent_id = '%s'",
				$db->quote($recurrence_of_id)
			)
		);
	}

	static function deleteRecurring($rule_id) {
		global $db;
		$db->query(
			sprintf(
				"DELETE FROM recurrence_rules WHERE id = '%s'",
				$db->quote($rule_id)
			)
		);
	}

	static function getFirstInstance($recurrence_of_id) {
		global $db;
		$res = $db->query(
			sprintf(
				"SELECT * FROM meetings WHERE deleted = 0 AND (id = '%s' OR recurrence_of_id = '%s')"
				. " ORDER BY recurrence_index LIMIT 1",
				$db->quote($recurrence_of_id),
				$db->quote($recurrence_of_id)
			)
		);
		return $db->fetchByAssoc($res);
	}

	static function createTemplate(array $meeting) {
		static $fields = [
			"assigned_user_id",
			"name",
			"duration",
			"reminder_time",
			"is_daylong",
			"date_start",
			"description",
			"location",
		];
		$template = [];
		foreach ($fields as $f) {
			if (isset($meeting[$f])) {
				$template[$f] = $meeting[$f];
			}
		}
		return $template;
	}

	static function updateSyncRecord($meeting_id, $rule_id) {
		global $db;
		$db->query(
			sprintf(
				"UPDATE google_sync SET related_type='Recurrence', related_id='%s'"
				. " WHERE related_id='%s'",
				$db->quote($rule_id),
				$db->quote($meeting_id)
			)
		);
	}

	static function updateTemplate($rule_id, $template) {
		global $db;
		$db->query(
			sprintf(
				"UPDATE recurrence_rules SET parent_type ='Recurrence', template='%s'"
				. " WHERE id='%s'",
				$db->quote(json_encode($template)),
				$db->quote($rule_id)
			)
		);
	}

}
