<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


require_once('data/SugarBean.php');

class RevenueForecast extends SugarBean {
	static function before_save(RowUpdate &$upd) {
		$opportunity = ListQuery::quick_fetch_row('Opportunity', $upd->getField('opportunity_id'));
		$update = array();

		$update['billing_date'] = date('Y-m-d', strtotime($opportunity['date_closed']. ' + '.$upd->getField('lag').' '.$upd->getField('lag_type')));

		if($upd->getField('lag') == '' || $upd->getField('lag_type') == '')
			$update['billing_date'] = $opportunity['date_closed'];

		switch ($upd->getField('terms')) {
            case 'Net 7 Days':
                $days = 7;
                break;
            case 'Net 15 Days':
                $days = 15;
                break;
            case 'Net 30 Days':
                $days = 30;
                break;
            case 'Net 45 Days':
                $days = 45;
                break;
            case 'Net 60 Days':
                $days = 60;
                break;
            default:
                $days = 0;
        }
        $update['expected_revenue_date'] = date('Y-m-d', strtotime($update['billing_date']. ' + '.$days.' days'));

        if(!$upd->new_record) {
			if($upd->getField('unit_price') != $upd->orig_row['unit_price'] || $upd->getField('qty') != $upd->orig_row['qty']) {
				$update['amount'] = $upd->getField('unit_price')*$upd->getField('qty');
 			}
		}

        $upd->set($update);
	}

	static function update_opp_amount(RowUpdate &$upd) {
		if ($upd->getRelatedData('updating_billing_date')) return;
		$lq = new ListQuery('RevenueForecast');
		$lq->addSimpleFilter('opportunity_id', $upd->getField('opportunity_id'), '=');
		$lq->addFieldLiteral('total', 'sum(amount_usdollar)', 'double');
		$lq->addFieldLiteral('total_cur', 'sum(amount)', 'double');
		$lq->addFieldLiteral('currency_id', 'currency_id', 'varchar');
		$lq->setGroupBy('currency_id');
		$result = $lq->fetchAll();

		$total = 0;
		$opportunity = ListQuery::quick_fetch('Opportunity', $upd->getField('opportunity_id'));
		if(!$opportunity)
			return;
		$currency = new Currency();
		$currency->retrieve($opportunity->getField('currency_id'));

		foreach ($result as $row) {
			if($opportunity->getField('currency_id') == $row->getField('currency_id'))
				$amt = $row->getField('total_cur');
			else
				$amt = $currency->convertFromDollar($row->getField('total'), $currency->decimal_places);

			$total = $total+$amt;
		}

		$opportunity_update = RowUpdate::for_result($opportunity);
		$opportunity_update->set(array('amount' => $total));
        $opportunity_update->save();
	}

	static function rem_link(RowUpdate $upd, $link_name = null) {
		$link = $upd->getLinkUpdate();
        if ($link)
            $link->markDeleted();
	}
}
?>
