<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
if(!defined('inScheduler')) die('Unauthorized access');
global $log, $db, $timedate;

require_once('modules/ImportDB/profiles/ImportDBProfileManager.php');

$lq = new ListQuery('SFImport');
$lq->addSimpleFilter('completed', 0);
$result = $lq->fetchAll();

ini_set("max_execution_time", -1);

foreach($result as $res) {
	$line_number = $res->getField('current');

	$upload_dir = AppConfig::upload_dir();

	$array = $fields = array();
	$i = 0;
	$handle = @fopen($upload_dir . $res->getField('filename'), "r");
	if ($handle) {
		while (($row = fgetcsv($handle, 4096, ",")) !== false) {

			if (empty($fields)) {
				$fields = $row;
				continue;
			}
			foreach ($row as $k => $value) {
				$array[$i][$fields[$k]] = $value;
			}
			$i++;
		}
		if (!feof($handle)) {
			echo "Error: unexpected fgets() fail\n";
		}
		fclose($handle);
	}

	$imported = 0;
	$import_limit = 0;

	$import_profile = RowUpdate::blank_for_model("ImportDBProfile");
	$profile = [
		"id" => $res->getPrimaryKeyValue(),
		"name" => 'SF.com CSV Import',
	];
	ImportDBProfileManager::init();
	try {
		$profile_qf = ImportDBProfileManager::getProfile(array_get_default($profile, 'name', ''));
		$profile['id'] = $profile->getId();
	} catch (OutOfBoundsException $e) {
		$profile_qf = false;
	}

	if($profile_qf) {
		try {
			$import_profile->loadInput($profile, true);
			$import_profile->save();
		} catch (Exception $e) {

		}
	}

	$import_mod = RowUpdate::blank_for_model("ImportDBModule");
	$mod = [
		"id" => $res->getPrimaryKeyValue(),
		"profile_id" => $profile['id'],
		"name" => $res->getField('import_module'),
	];
	$module_qf = ListQuery::quick_fetch_row('ImportDBModule', $mod['id']);
	if(!$module_qf) {
		try {
			$import_mod->loadInput($mod, true);
			$import_mod->save();
		} catch (Exception $e) {

		}
	}

	foreach ($array as $data) {
		if($imported < $line_number) {
			$imported++;
			continue;
		}

		if($res->getField('import_module') == 'Accounts') {
			$import_row = RowUpdate::blank_for_model("Account");
			$updates = getAccountStructure($data, $res);
		} else if($res->getField('import_module') == 'Contacts') {
			$import_row = RowUpdate::blank_for_model("Contact");
			$updates = getContactStructure($data, $res);
		} else if($res->getField('import_module') == 'Leads') {
			$import_row = RowUpdate::blank_for_model("Lead");
			$updates = getLeadStructure($data, $res);
		} else if($res->getField('import_module') == 'Opportunities') {
			$import_row = RowUpdate::blank_for_model("Opportunity");
			$updates = getOpportunityStructure($data, $res);
		} else if($res->getField('import_module') == 'Tasks') {
			if($data['Type'] == 'Call')
				continue;
			$import_row = RowUpdate::blank_for_model("Task");
			$updates = getTaskStructure($data, $res);
		}  else if($res->getField('import_module') == 'Notes') {
			$import_row = RowUpdate::blank_for_model("Note");
			$updates = getNoteStructure($data, $res);
		} else if($res->getField('import_module') == 'Users') {
			if($data['UserType'] != 'Standard')
				continue;
			$import_row = RowUpdate::blank_for_model("User");
			$updates = getUserStructure($data, $res);
		} else if($res->getField('import_module') == 'Calls') {
			if($data['Type'] != 'Call')
				continue;
			$import_row = RowUpdate::blank_for_model("Call");
			$updates = getCallStructure($data, $res);
		} else if($res->getField('import_module') == 'Meetings') {
			$import_row = RowUpdate::blank_for_model("Meeting");
			$updates = getMeetingStructure($data, $res);
		} else if($res->getField('import_module') == 'Cases') {
			$import_row = RowUpdate::blank_for_model("aCase");
			$updates = getCaseStructure($data, $res);
		} else {
			continue;
		}


		$import_row->loadInput($updates, false);
		$result = $import_row->save();

		$import_hist = RowUpdate::blank_for_model("ImportDBHistory");
		$hist = [
			"module_id" => $mod['id'],
			"generated_id" => $updates['id'],
		];
		$import_hist->loadInput($hist, true);
		$import_hist->save();

		$line_number++;

		$total_update = RowUpdate::for_result($res);

		if ($line_number == $res->getField('count')) {
			$total_update->set(['completed' => 1, 'current' => $line_number]);
		} else {
			$total_update->set(['current' => $line_number]);
		}

		$total_update->save();
		$imported++;
		$import_limit++;
		if($import_limit == 200)
			break;
	}
}

function getAccountStructure($data, RowResult $res): array
{
	$user = new ListQuery('User');
	$user->addSimpleFilter('sf_id', $data['OwnerId'], '=');
	$user = $user->fetchAllRows();
	if($user) {
		$user_id = array_key_first($user);
	} else {
		$user_id = $res->getField('assigned_user_id');
	}
	return [
		"id" => create_guid(),
		"sf_id" => $data['Id'],
		"name" => $data['Name'],
		"account_type" => $data['Type'],
		"billing_address_street" => $data['BillingStreet'],
		"billing_address_city" => $data['BillingCity'],
		"billing_address_state" => $data['BillingState'],
		"billing_address_postalcode" => $data['BillingPostalCode'],
		"billing_address_country" => $data['BillingCountry'],
		"shipping_address_street" => $data['ShippingStreet'],
		"shipping_address_city" => $data['ShippingCity'],
		"shipping_address_state" => $data['ShippingState'],
		"shipping_address_postalcode" => $data['ShippingPostalCode'],
		"shipping_address_country" => $data['ShippingCountry'],
		"phone_office" => $data['Phone'],
		"phone_fax" => $data['Fax'],
		"website" => $data['Website'],
		"sic_code" => $data['Sic'],
		"industry" => $data['Industry'],
		"annual_revenue" => $data['AnnualRevenue'],
		"employees" => $data['NumberOfEmployees'],
		"ownership" => $data['Ownership'],
		"ticker_symbol" => $data['TickerSymbol'],
		"description" => $data['Description'],
		"rating" => $data['Rating'],
		"date_entered" => $data['CreatedDate'],
		"date_modified" => $data['LastModifiedDate'],
		"last_activity_date" => $data['LastActivityDate'],
		"assigned_user_id" => $user_id
	];
}

function getContactStructure($data, RowResult $res): array
{
	$user = new ListQuery('User');
	$user->addSimpleFilter('sf_id', $data['OwnerId'], '=');
	$user = $user->fetchAllRows();
	if($user) {
		$user_id = array_key_first($user);
	} else {
		$user_id = $res->getField('assigned_user_id');
	}

	$accounts = new ListQuery('Account');
	$accounts->addSimpleFilter('sf_id', $data['AccountId'], '=');
	$acc = $accounts->fetchAllRows();
	if($acc) {
		$acc = array_key_first($acc);
	}
	return [
		"id" => create_guid(),
		"sf_id" => $data['Id'],
		"first_name" => $data['FirstName'].' '.$data['MiddleName'],
		"last_name" => $data['LastName'],
		"primary_account_id" => $acc,
		"salutation" => $data['Salutation'],
		"alt_address_street" => $data['OtherStreet'],
		"alt_address_city" => $data['OtherCity'],
		"alt_address_state" => $data['OtherState'],
		"alt_address_postalcode" => $data['OtherPostalCode'],
		"alt_address_country" => $data['OtherCountry'],
		"primary_address_street" => $data['MailingStreet'],
		"primary_address_city" => $data['MailingCity'],
		"primary_address_state" => $data['MailingState'],
		"primary_address_postalcode" => $data['MailingPostalCode'],
		"primary_address_country" => $data['MailingCountry'],
		"phone_work" => $data['Phone'],
		"phone_mobile" => $data['MobilePhone'],
		"phone_home" => $data['HomePhone'],
		"phone_other" => $data['OtherPhone'],
		"email1" => $data['Email'],
		"title" => $data['Title'],
		"department" => $data['Department'],
		"lead_source" => $data['LeadSource'],
		"birthdate" => $data['Birthdate'],
		"description" => $data['Description'],
		"date_entered" => $data['CreatedDate'],
		"date_modified" => $data['LastModifiedDate'],
		"assigned_user_id" => $user_id
	];
}

function getLeadStructure($data, RowResult $res): array
{
	$user = new ListQuery('User');
	$user->addSimpleFilter('sf_id', $data['OwnerId'], '=');
	$user = $user->fetchAllRows();
	if($user) {
		$user_id = array_key_first($user);
	} else {
		$user_id = $res->getField('assigned_user_id');
	}

	return [
		"id" => create_guid(),
		"sf_id" => $data['Id'],
		"salutation" => $data['Salutation'],
		"first_name" => $data['FirstName'].' '.$data['MiddleName'],
		"last_name" => $data['LastName'],
		"title" => $data['Title'],
		"account_name" => $data['Company'],
		"primary_address_street" => $data['Street'],
		"primary_address_city" => $data['City'],
		"primary_address_state" => $data['State'],
		"primary_address_postalcode" => $data['PostalCode'],
		"primary_address_country" => $data['Country'],
		"phone_work" => $data['Phone'],
		"phone_mobile" => $data['MobilePhone'],
		"email1" => $data['Email'],
		"website" => $data['Website'],
		"description" => $data['Description'],
		"lead_source" => $data['LeadSource'],
		"status" => $data['Status'],
		"temperature" => $data['Rating'],
		"date_entered" => $data['CreatedDate'],
		"date_modified" => $data['LastModifiedDate'],
		"assigned_user_id" => $user_id
	];
}

function getOpportunityStructure($data, RowResult $res): array
{
	$user = new ListQuery('User');
	$user->addSimpleFilter('sf_id', $data['OwnerId'], '=');
	$user = $user->fetchAllRows();
	if($user) {
		$user_id = array_key_first($user);
	} else {
		$user_id = $res->getField('assigned_user_id');
	}

	$accounts = new ListQuery('Account');
	$accounts->addSimpleFilter('sf_id', $data['AccountId'], '=');
	$acc = $accounts->fetchAllRows();
	if($acc) {
		$acc = array_key_first($acc);
	}

	return [
		"id" => create_guid(),
		"sf_id" => $data['Id'],
		"account_id" => $acc,
		"name" => $data['Name'],
		"description" => $data['Description'],
		"sales_stage" => $data['StageName'],
		"amount" => (is_numeric($data['Amount'])) ? $data['Amount'] : 0,
		"probability" => $data['Probability'],
		"date_closed" => $data['CloseDate'],
		"opportunity_type" => $data['Type'],
		"next_step" => $data['NextStep'],
		"lead_source" => $data['LeadSource'],
		"date_entered" => $data['CreatedDate'],
		"date_modified" => $data['LastModifiedDate'],
		"assigned_user_id" => $user_id
	];
}

function getTaskStructure($data, RowResult $res): array
{
	$user = new ListQuery('User');
	$user->addSimpleFilter('sf_id', $data['OwnerId'], '=');
	$user = $user->fetchAllRows();
	if($user) {
		$user_id = array_key_first($user);
	} else {
		$user_id = $res->getField('assigned_user_id');
	}

	$accounts = new ListQuery('Account');
	$accounts->addSimpleFilter('sf_id', $data['AccountId'], '=');
	$acc = $accounts->fetchAllRows();
	if($acc) {
		$acc = array_key_first($acc);
	}
	$contacts = new ListQuery('Contact');
	$contacts->addSimpleFilter('sf_id', $data['WhoId'], '=');
	$contact = $contacts->fetchAllRows();
	if($contact) {
		$contact = array_key_first($contact);
	} else {
		$contacts = new ListQuery('Lead');
		$contacts->addSimpleFilter('sf_id', $data['WhoId'], '=');
		$contact = $contacts->fetchAllRows();
		if($contact) {
			$contact = array_key_first($contact);
		}
	}
	return [
		"id" => create_guid(),
		"sf_id" => $data['Id'],
		"name" => $data['Subject'],
		"account_id" => $acc,
		"contact_id" => $contact,
		"date_start" => $data['ActivityDate'],
		"status" => $data['Status'],
		"priority" => $data['Priority'],
		"description" => $data['Description'],
		"date_entered" => $data['CreatedDate'],
		"date_modified" => $data['LastModifiedDate'],
		"assigned_user_id" => $user_id
	];
}

function getNoteStructure($data, RowResult $res): array
{
	$user = new ListQuery('User');
	$user->addSimpleFilter('sf_id', $data['OwnerId'], '=');
	$user = $user->fetchAllRows();
	if($user) {
		$user_id = array_key_first($user);
	} else {
		$user_id = $res->getField('assigned_user_id');
	}

	$accounts = new ListQuery('Account');
	$accounts->addSimpleFilter('sf_id', $data['AccountId'], '=');
	$acc = $accounts->fetchAllRows();
	if($acc) {
		$acc = array_key_first($acc);
	}
	return [
		"id" => create_guid(),
		"sf_id" => $data['Id'],
		"account_id" => $acc,
		"last_name" => $data['LastName'],
		"name" => $data['Title'],
		"description" => $data['Body'],
		"date_entered" => $data['CreatedDate'],
		"date_modified" => $data['LastModifiedDate'],
		"assigned_user_id" => $user_id
	];
}

function getUserStructure($data, RowResult $res): array
{
	if($data['IsActive'] == 1)
		$active = 'Active';
	else
		$active = 'Inactive';

	if(strlen($data['Username']) > 50)
		$data['Username'] = substr($data['Username'],0,50);
	return [
		"id" => create_guid(),
		"sf_id" => $data['Id'],
		"user_name" => $data['Username'],
		"first_name" => $data['FirstName'].' '.$data['MiddleName'],
		"last_name" => $data['LastName'],
		"title" => $data['Title'],
		"email1" => $data['Email'],
		"phone_work" => $data['Phone'],
		"phone_mobile" => $data['MobilePhone'],
		"status" => $active,
		"date_entered" => $data['CreatedDate'],
		"date_modified" => $data['LastModifiedDate'],
		"assigned_user_id" => $res->getField('assigned_user_id')
	];
}

function getCallStructure($data, RowResult $res): array
{
	$user = new ListQuery('User');
	$user->addSimpleFilter('sf_id', $data['OwnerId'], '=');
	$user = $user->fetchAllRows();
	if($user) {
		$user_id = array_key_first($user);
	} else {
		$user_id = $res->getField('assigned_user_id');
	}

	$accounts = new ListQuery('Account');
	$accounts->addSimpleFilter('sf_id', $data['AccountId'], '=');
	$acc = $accounts->fetchAllRows();
	if($acc) {
		$acc = array_key_first($acc);
	}
	return [
		"id" => create_guid(),
		"sf_id" => $data['Id'],
		"name" => $data['Subject'],
		"account_id" => $acc,
		"date_start" => $data['ActivityDate'],
		"status" => $data['Status'],
		"description" => $data['Description'],
		"date_entered" => $data['CreatedDate'],
		"date_modified" => $data['LastModifiedDate'],
		"assigned_user_id" => $user_id
	];
}

function getMeetingStructure($data, RowResult $res): array
{
	$user = new ListQuery('User');
	$user->addSimpleFilter('sf_id', $data['OwnerId'], '=');
	$user = $user->fetchAllRows();
	if($user) {
		$user_id = array_key_first($user);
	} else {
		$user_id = $res->getField('assigned_user_id');
	}

	$accounts = new ListQuery('Account');
	$accounts->addSimpleFilter('sf_id', $data['AccountId'], '=');
	$acc = $accounts->fetchAllRows();
	if($acc) {
		$acc = array_key_first($acc);
	}
	return [
		"id" => create_guid(),
		"sf_id" => $data['Id'],
		"name" => $data['Subject'],
		"account_id" => $acc,
		"location" => $data['Location'],
		"is_daylong" => $data['IsAllDayEvent'],
		"date_start" => $data['ActivityDateTime'],
		"duration" => $data['DurationInMinutes'],
		"description" => $data['Description'],
		"date_entered" => $data['CreatedDate'],
		"date_modified" => $data['LastModifiedDate'],
		"assigned_user_id" => $user_id
	];
}

function getCaseStructure($data, RowResult $res): array
{
	$user = new ListQuery('User');
	$user->addSimpleFilter('sf_id', $data['OwnerId'], '=');
	$user = $user->fetchAllRows();
	if($user) {
		$user_id = array_key_first($user);
	} else {
		$user_id = $res->getField('assigned_user_id');
	}

	$accounts = new ListQuery('Account');
	$accounts->addSimpleFilter('sf_id', $data['AccountId'], '=');
	$acc = $accounts->fetchAllRows();
	if($acc) {
		$acc = array_key_first($acc);
	}

	$contacts = new ListQuery('Contact');
	$contacts->addSimpleFilter('sf_id', $data['WhoId'], '=');
	$contact = $contacts->fetchAllRows();
	if($contact) {
		$contact = array_key_first($contact);
	}

	return [
		"id" => create_guid(),
		"sf_id" => $data['Id'],
		"name" => $data['Subject'],
		"account_id" => $acc,
		"cust_contact_id" => $contact,
		"type" => $data['Type'],
		"status" => $data['Status'],
		"priority" => $data['Priority'],
		"description" => $data['Description'],
		"date_closed" => $data['ClosedDate'],
		"date_entered" => $data['CreatedDate'],
		"date_modified" => $data['LastModifiedDate'],
		"assigned_user_id" => $user_id
	];
}
