<?php

require_once 'modules/Quotes/PDFTemplatesProvider.php';

class SalesOrderPDFTemplateProvider extends QuotePDFTemplateProvider
{

	protected function __construct($type)
	{
		parent::__construct($type);
	}

	public static function template_types()
	{
		return array(
			'confirmation' => translate('LBL_PDF_CONFIRMATION', 'SalesOrders'),
			'billing_status' => translate('LBL_PDF_BILLING_STATUS_TITLE', 'SalesOrders'),
		);
	}

	public static function template_skeleton($type)
	{
		$ret = array('sections' => array(
			array(
				'name' => 'header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_HEADER',
				'height' => 150,
				'elements' => array (),
			),
			array(
				'name' => 'group_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_GROUP_HEADER',
				'height' => 150,
				'elements' => array (),
			),
			array(
				'name' => 'group_items',
				'type' => 'table',
				'title' => 'LBL_SKEL_GROUP_ITEMS',
				'columns' => array (),
			),
			array(
				'name' => 'group_totals_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_GROUP_TOTALS_HEADER',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'group_totals',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_GROUP_TOTALS',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'group_footer',
				'data_id' => 'group_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_GROUP_FOOTER',
				'height' => 150,
				'elements' => array (),
			),
			array(
				'name' => 'quote_totals_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_QUOTE_TOTALS_HEADER',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'quote_totals',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_QUOTE_TOTALS',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'invoices_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_INVOICES_HEADER',
				'height' => 25,
				'elements' => array (),
			),
			array(
				'name' => 'invoices_table',
				'type' => 'table',
				'title' => 'LBL_SKEL_INVOICES_TABLE',
				'columns' => array (),
			),
			array(
				'name' => 'invoices_footer',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_INVOICES_FOOTER',
				'height' => 30,
				'elements' => array (),
			),
			array(
				'name' => 'invoices_summary',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_INVOICES_SUMMARY',
				'height' => 35,
				'elements' => array (),
			),
			array(
				'name' => 'notes',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_NOTES',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'terms',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_TERMS',
				'height' => 40,
				'elements' => array (),
			),
			array(
				'name' => 'footer',
				'data_id' => 'header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_FOOTER',
				'height' => 40,
				'elements' => array (),
			),
		));

		foreach ($ret['sections'] as &$sec) $sec['title'] = translate($sec['title'], 'SalesOrders');

		if ($type != 'billing_status') {
			unset($ret['sections']['invoices_header']);
			unset($ret['sections']['invoices_table']);
			unset($ret['sections']['invoices_footer']);
			unset($ret['sections']['invoices_summary']);
		}

		$ret['sections'] = array_values($ret['sections']);

		return $ret;
	}

	public static function template_sample_data($type)
	{
		$datefmt = AppConfig::setting('locale.defaults.date_format');
		$data = array(
			'header' => array(
				'todays_date' => array(translate('LBL_TODAYS_DATE'), date($datefmt), ),
				'number' => array(translate('LBL_SAMPLE_HEADER_NUMBER_LABEL', 'SalesOrders'), 'SO2013-001'),
				'terms' => array(translate('LBL_SAMPLE_HEADER_TERMS_LABEL', 'SalesOrders'), translate('LBL_SAMPLE_HEADER_TERMS_VALUE', 'SalesOrders'), ),
				'date' => array(translate('LBL_SAMPLE_HEADER_DATE_LABEL', 'SalesOrders'), '2014-11-05'),
				'date_due' => array(translate('LBL_SAMPLE_HEADER_DATE_DUE_LABEL', 'SalesOrders'), '2014-11-05'),
				'sales_person' => array(translate('LBL_SAMPLE_HEADER_SALES_PERSON_LABEL', 'SalesOrders'), translate('LBL_SAMPLE_HEADER_SALES_PERSON_VALUE', 'SalesOrders'), ),
				'tax_info' => array(translate('LBL_SAMPLE_HEADER_TAX_INFO_LABEL', 'SalesOrders'), ''),
				'billing_address' => array(translate('LBL_SAMPLE_HEADER_BILLING_ADDRESS_LABEL', 'SalesOrders'), translate('LBL_SAMPLE_HEADER_BILLING_ADDRESS_VALUE', 'SalesOrders'), ),
				'billing_address_contact_last' => array(translate('LBL_SAMPLE_HEADER_BILLING_ADDRESS_CONTACT_LAST_LABEL', 'Quotes'), translate('LBL_SAMPLE_HEADER_BILLING_ADDRESS_VALUE', 'Quotes'), ),
				'shipping_address' => array(translate('LBL_SAMPLE_HEADER_SHIPPING_ADDRESS_LABEL', 'SalesOrders'), translate('LBL_SAMPLE_HEADER_SHIPPING_ADDRESS_VALUE', 'SalesOrders'), ),
				'shipping_address_contact_last' => array(translate('LBL_SAMPLE_HEADER_SHIPPING_ADDRESS_CONTACT_LAST_LABEL', 'Quotes'), translate('LBL_SAMPLE_HEADER_SHIPPING_ADDRESS_VALUE', 'Quotes'), ),
				'description' => array(translate('LBL_SAMPLE_HEADER_DESCRIPTION_LABEL', 'SalesOrders'), translate('LBL_SAMPLE_HEADER_DESCRIPTION_VALUE', 'SalesOrders'), ),
				'purchase_order_num' => array(translate('LBL_SAMPLE_HEADER_PO_NUMBER_LABEL', 'SalesOrders'), translate('LBL_SAMPLE_HEADER_PO_NUMBER_VALUE', 'SalesOrders'), ),
				'standard_terms' => array(translate('LBL_SAMPLE_HEADER_STANDARD_TERMS_LABEL', 'SalesOrders'), ''),
			),
			'group_header' => array(
				'group_name' => array(translate('LBL_SAMPLE_GROUP_HEADER_GROUP_NAME_LABEL', 'SalesOrders'), translate('LBL_SAMPLE_GROUP_HEADER_GROUP_NAME_VALUE', 'SalesOrders'), ),
				'status' => array(translate('LBL_SAMPLE_GROUP_HEADER_STATUS_LABEL', 'SalesOrders'), translate('LBL_SAMPLE_GROUP_HEADER_STATUS_VALUE', 'SalesOrders'), ),
			),
			'group_totals' => array(
				'total_name' => array(translate('LBL_SAMPLE_GROUP_TOTALS_TOTAL_NAME_LABEL', 'SalesOrders'), translate('LBL_SAMPLE_GROUP_TOTALS_TOTAL_NAME_VALUE', 'SalesOrders'), ),
				'total_value' => array(translate('LBL_SAMPLE_GROUP_TOTALS_TOTAL_VALUE_LABEL', 'SalesOrders'), translate('LBL_SAMPLE_GROUP_TOTALS_TOTAL_VALUE_VALUE', 'SalesOrders'), ),
			),
			'quote_totals' => array(
				'total_name' => array(translate('LBL_SAMPLE_QUOTE_TOTALS_TOTAL_NAME_LABEL', 'SalesOrders'), translate('LBL_SAMPLE_QUOTE_TOTALS_TOTAL_NAME_VALUE', 'SalesOrders'), ),
				'total_value' => array(translate('LBL_SAMPLE_QUOTE_TOTALS_TOTAL_VALUE_LABEL', 'SalesOrders'), translate('LBL_SAMPLE_QUOTE_TOTALS_TOTAL_VALUE_VALUE', 'SalesOrders'), ),
			),
			'group_items' => array(
				'number' => array(translate('LBL_SAMPLE_GROUP_ITEMS_NUMBER_LABEL', 'SalesOrders'), '1'),
				'product' => array(translate('LBL_SAMPLE_GROUP_ITEMS_PRODUCT_LABEL', 'SalesOrders'), translate('LBL_SAMPLE_GROUP_ITEMS_PRODUCT_VALUE', 'SalesOrders'), ),
				'quantity' => array(translate('LBL_SAMPLE_GROUP_ITEMS_QUANTITY_LABEL', 'SalesOrders'), '2'),
				'listprice' => array(translate('LBL_SAMPLE_GROUP_ITEMS_LISTPRICE_LABEL', 'SalesOrders'), '10.0'),
				'unitprice' => array(translate('LBL_SAMPLE_GROUP_ITEMS_UNITPRICE_LABEL', 'SalesOrders'), '20.0'),
				'costprice' => array(translate('LBL_SAMPLE_GROUP_ITEMS_COSTPRICE_LABEL', 'SalesOrders'), '5.0'),
				'extprice' => array(translate('LBL_SAMPLE_GROUP_ITEMS_EXTPRICE_LABEL', 'SalesOrders'), '40.0'),
				'discount' => array(translate('LBL_SAMPLE_GROUP_ITEMS_DISCOUNT_LABEL', 'SalesOrders'), '5%'),
				'mfr_part_no' => array(translate('LBL_SAMPLE_GROUP_ITEMS_MFR_PART_NO_LABEL', 'SalesOrders'), '123456'),
				'serial_numbers' => array(translate('LBL_SAMPLE_GROUP_ITEMS_SERIAL_NUMBERS_LABEL', 'SalesOrders'), '748905'),
			),
			'invoices_header' => array(),
			'invoices_table' => array(
				'invoice_date' => array(translate('LBL_SAMPLE_INVOICES_TABLE_INVOICE_DATE_LABEL', 'SalesOrders'), '2014-11-05'),
				'invoice_number' => array(translate('LBL_SAMPLE_INVOICES_TABLE_INVOICE_NUMBER_LABEL', 'SalesOrders'), '2014-1235'),
				'pretax_amount' => array(translate('LBL_SAMPLE_INVOICES_TABLE_PRETAX_LABEL', 'SalesOrders'), '38.0'),
				'tax_amount' => array(translate('LBL_SAMPLE_INVOICES_TABLE_TAX_LABEL', 'SalesOrders'), '10.0'),
				'shipping_amount' => array(translate('LBL_SAMPLE_INVOICES_TABLE_SHIPPING_LABEL', 'SalesOrders'), '10.0'),
				'total_amount' => array(translate('LBL_SAMPLE_INVOICES_TABLE_TOTAL_LABEL', 'SalesOrders'), '58.0'),
				'amount_due' => array(translate('LBL_SAMPLE_INVOICES_TABLE_AMOUNT_DUE_LABEL', 'SalesOrders'), '30.0'),
				'paid_amount' => array(translate('LBL_SAMPLE_INVOICES_TABLE_PAID_LABEL', 'SalesOrders'), '28.0'),
			),
			'invoices_footer' => array(
				'total_pretax' => array(translate('LBL_SAMPLE_INVOICES_TABLE_TOTAL_PRETAX_LABEL', 'SalesOrders'), '38.0'),
				'total_tax' => array(translate('LBL_SAMPLE_INVOICES_TABLE_TOTAL_TAX_LABEL', 'SalesOrders'), '10.0'),
				'total_shipping' => array(translate('LBL_SAMPLE_INVOICES_TABLE_TOTAL_SHIPPING_LABEL', 'SalesOrders'), '10.0'),
				'total_amount' => array(translate('LBL_SAMPLE_INVOICES_TABLE_TOTAL_AMOUNT_LABEL', 'SalesOrders'), '58.0'),
				'total_amount_due' => array(translate('LBL_SAMPLE_INVOICES_TABLE_TOTAL_AMOUNT_DUE_LABEL', 'SalesOrders'), '28.0'),
				'total_paid' => array(translate('LBL_SAMPLE_INVOICES_TABLE_TOTAL_PAID_LABEL', 'SalesOrders'), '30.0')
			),
			'invoices_summary' => array(
				'pretax_sum' => array(translate('LBL_SAMPLE_INVOICES_TABLE_PRETAX_SUM_LABEL', 'SalesOrders'), '0.0'),
				'tax_sum' => array(translate('LBL_SAMPLE_INVOICES_TABLE_TAX_SUM_LABEL', 'SalesOrders'), '0.0'),
				'shipping_sum' => array(translate('LBL_SAMPLE_INVOICES_TABLE_SHIPPING_SUM_LABEL', 'SalesOrders'), '0.0'),
				'amount_sum' => array(translate('LBL_SAMPLE_INVOICES_TABLE_AMOUNT_SUM_LABEL', 'SalesOrders'), '0.0'),
				'amount_due_sum' => array(translate('LBL_SAMPLE_INVOICES_TABLE_AMOUNT_DUE_SUM_LABEL', 'SalesOrders'), '28.0')
			),
		);

		self::injectProductSampleData($data['group_items']);
		$ret = array();
		foreach ($data as $k => $v) {
			$ret[$k] = array();
			foreach ($v as $kk => $vv) {
				$ret[$k][$kk] = array('label' => $vv[0], 'value' => $vv[1]);
			}
		}
		return $ret;
	}

	public static function create($type)
	{
		return new SalesOrderPDFTemplateProvider($type);
	}

	protected function iterateExtraFooter(PDFTemplatePrinter $p)
	{
		$section = $this->findSection('invoices_header');
		$p->renderSection($section, $this->data['invoices_header']);

		$section = $this->findSection('invoices_table');
		$p->renderSection($section, $this->data['invoices_table']);

		$section = $this->findSection('invoices_footer');
		$p->renderSection($section, $this->data['invoices_footer']);

		$section = $this->findSection('invoices_summary');
		$p->renderSection($section, $this->data['invoices_summary']);

		parent::iterateExtraFooter($p);
	}

	public function getFileName()
	{
		$filename_pfx = translate($this->type == 'billing_status' ? 'LBL_PDF_BILLING_STATUS_TITLE' : 'LBL_PDF_SALESORDER_TITLE', 'SalesOrders');
		$filename = $filename_pfx . '_' . $this->row->getField('prefix');
		$filename .= $this->row->getField('so_number') . '_' . $this->row->getField('billing_account_name');
		$filename = preg_replace('~\.+$~', '', trim($filename));
		$filename = $filename . '.pdf';
		return $filename;
	}

	public function getNoteTitle()
	{
		return translate('LBL_PREPARED_SALESORDER', 'SalesOrders');
	}

	public function getEmailTitle()
	{
		return translate('LBL_EMAILED_SALESORDER', 'SalesOrders');
	}

	public function getSignedDocumentTitle() {
		$prefix =
			$this->row
			? '[' . $this->row->getField('prefix') . $this->row->getField('so_number') . '] '
			: '';

		return $prefix . $this->getNoteTitle();
	}

	protected function createData()
	{
		global $timedate;
		$row =& $this->row;
		$user = new User();
		$user->retrieve($row->getField('assigned_user_id'), false);
		$terms = AppConfig::setting("lang.lists.current.app.terms_dom", array());
		$tax_information = $this->row->getField('tax_information');
		$datefmt = AppConfig::setting('locale.defaults.date_format');
		$data = array(
			'todays_date' => date($datefmt),
			'number' => $row->getField('prefix') . $row->getField('so_number'),
			'terms' => array_get_default($terms, $row->getField('terms'), $row->getField('terms')),
			'date' => $timedate->to_display_date($row->getField('date_entered')),
			'date_due' => $row->getField('due_date', '', true),
			'sales_person' => $row->getField('assigned_user_name'),
			'tax_info' => from_html(AppConfig::setting('company.tax_information', '')),
			'billing_address' => $this->_address('billing_') . "\n" . $tax_information,
			'billing_address_contact_last' => $this->_address('billing_c') . "\n" . $tax_information,
			'shipping_address' => $this->_address('shipping_'),
			'shipping_address_contact_last' => $this->_address('shipping_c'),
			'description' => $row->getField('description'),
			'purchase_order_num' => $row->getField('purchase_order_num'),
			'standard_terms' => trim(AppConfig::setting('company.std_so_terms', '')),
		);

		$data = array_merge($this->header_data, $data);
		$this->data['header'] = $data;

		$this->createGroupData();
		$this->createInvoicesData();
	}

	protected function createInvoicesData()
	{
		$invoices = $this->row->getLinkResults('invoice');
		$data = [];
		$totals = [
			'total_pretax' => 0,
			'total_tax' => 0,
			'total_shipping' => 0,
			'total_amount' => 0,
			'total_paid' => 0,
			'total_amount_due' => 0
		];

		if ($invoices->getResultCount() > 0) {
			global $timedate;
			$rows = $invoices->getRows();
			$final_invoice = [];

			foreach ($rows as $row) {
				$row_record = ListQuery::quick_fetch('Invoice', $row['id']);
				/** @var TallyUpdate $tally */
				$tally = RowUpdate::for_result($row_record);
				$row_totals = $tally->getTotals();
				$paid = $row['amount'] - $row['amount_due'];

				if ($row['final']) {
					$final_invoice['row'] = $row;
					$final_invoice['totals'] = $row_totals;

					continue;
				}

				$data[] = [
					'invoice_date' => $timedate->to_display_date($row['invoice_date'], false),
					'invoice_number' => $row['prefix'] . $row['invoice_number'],
					'pretax_amount' => $this->currency_format($row['pretax'], $row['currency_id']),
					'tax_amount' => $this->currency_format($row_totals['tax'], $row['currency_id']),
					'shipping_amount' => $this->currency_format($row_totals['shipping'], $row['currency_id']),
					'total_amount' => $this->currency_format($row['amount'], $row['currency_id']),
					'amount_due' => $this->currency_format($row['amount_due'], $row['currency_id']),
					'paid_amount' => $this->currency_format($paid, $row['currency_id']),
				];

				$totals['total_pretax'] += $row['pretax'];
				$totals['total_tax'] += $row_totals['tax'];
				$totals['total_shipping'] += $row_totals['shipping'];
				$totals['total_amount'] += $row['amount'];
				$totals['total_amount_due'] += $row['amount_due'];
				$totals['total_paid'] += $paid;
			}

			if (! empty($final_invoice)) {
				$row = $final_invoice['row'];
				$row_totals = $final_invoice['totals'];
				$pretax = $row['pretax'] - $totals['total_pretax'];
				$tax = $row_totals['tax'] - $totals['total_tax'];
				$shipping = $row_totals['shipping'] - $totals['total_shipping'];
				$paid = $row['amount'] - $row['amount_due'];

				$data[] = [
					'invoice_date' => $timedate->to_display_date($row['invoice_date'], false),
					'invoice_number' => $row['prefix'] . $row['invoice_number'],
					'pretax_amount' => $this->currency_format($pretax, $row['currency_id']),
					'tax_amount' => $this->currency_format($tax, $row['currency_id']),
					'shipping_amount' => $this->currency_format($shipping, $row['currency_id']),
					'total_amount' => $this->currency_format($row['amount'], $row['currency_id']),
					'amount_due' => $this->currency_format($row['amount_due'], $row['currency_id']),
					'paid_amount' => $this->currency_format($paid, $row['currency_id']),
				];

				$totals['total_pretax'] += $pretax;
				$totals['total_tax'] += $tax;
				$totals['total_shipping'] += $shipping;
				$totals['total_amount'] += $row['amount'];
				$totals['total_amount_due'] += $row['amount_due'];
				$totals['total_paid'] += $paid;
			}

		}

		$so_totals = $this->gtotal;
		$currency_id = $this->getRow()->getField('currency_id');

		$summary = [
			'pretax_sum' => $this->currency_format($this->getRow()->getField('pretax') - $totals['total_pretax'], $currency_id),
			'tax_sum' => $this->currency_format($so_totals['tax'] - $totals['total_tax'], $currency_id),
			'shipping_sum' => $this->currency_format($so_totals['shipping'] - $totals['total_shipping'], $currency_id),
			'amount_sum' => $this->currency_format($this->getRow()->getField('amount') - $totals['total_amount'], $currency_id),
			'amount_due_sum' => $this->currency_format($this->getRow()->getField('amount') - $totals['total_paid'], $currency_id)
		];

		array_walk($totals, function (&$item) use($currency_id) {
			$item = $this->currency_format($item, $currency_id);
		});

		$this->data['invoices_table'] = $data;
		$this->data['invoices_footer'] = $totals;
		$this->data['invoices_summary'] = $summary;
	}

	public static function get_extra_sources() {
		return array(
			array(
				'module' => 'Opportunities',
				'model' => 'Opportunity',
				'prefix' => 'opportunity',
				'label' => 'LBL_OPPORTUNITY',
			),
			array(
				'module' => 'Quotes',
				'model' => 'Quote',
				'prefix' => 'related_quote',
				'label' => 'LBL_RELATED_QUOTE',
			),
			array(
				'module' => 'Accounts',
				'model' => 'Account',
				'prefix' => 'billing_account',
				'label' => 'LBL_BILLING_ACCOUNT',
			),
			array(
				'module' => 'Accounts',
				'model' => 'Account',
				'prefix' => 'shipping_account',
				'label' => 'LBL_SHIPPING_ACCOUNT',
			),
			array(
				'module' => 'Contacts',
				'model' => 'Contact',
				'prefix' => 'billing_contact',
				'label' => 'LBL_BILLING_CONTACT',
			),
			array(
				'module' => 'Contacts',
				'model' => 'Contact',
				'prefix' => 'shipping_contact',
				'label' => 'LBL_SHIPPING_CONTACT',
			),
			array(
				'module' => 'ShippingProviders',
				'model' => 'ShippingProvider',
				'prefix' => 'shipping_provider',
				'label' => 'LBL_SHIPPING_PROVIDER',
			),
		);

	}

}
