<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


$json_supported_actions['run_scheduled_task'] = array('admin_only' => true);
function json_run_scheduled_task()
{
	if(is_demo_site())
		return json_bad_request(array('error' => 'demo_site'));
	require_once('modules/Scheduler/Schedule.php');
	require_once('include/ProgressIndicator.php');

	if(! empty($_REQUEST['record'])) {
		$focus = ListQuery::quick_fetch('Schedule', $_REQUEST['record']);
	} else {
		$type = array_get_default($_REQUEST, 'type');
		if(! $type)
			return json_bad_request('Missing scheduled task type');
		$focus = ListQuery::quick_fetch_key('Schedule', 'type', $type);
	}
	if(! $focus)
		return json_bad_request('Error retrieving schedule');

	$pid = scheduler_progress_id();
	$ppath = scheduler_progress_path($pid);
	$progress = $ppath ? new ProgressIndicator($ppath, true) : null;
	
	$ret = ['progress_id' => $pid];
	// return result to user and effectively close connection
	json_return_value($ret, null, true);
	
	if(! defined('inScheduler')) define('inScheduler', 1);
	scheduler_set_time_limit(290);
	scheduler_run_result($focus, true, $progress);
	
	if($progress) $progress->finish('success', translate('LBL_RUN_OVER', 'Scheduler'));
	sugar_cleanup();
}

$json_supported_actions['get_scheduler_progress'] = array('admin_only' => true);
function json_get_scheduler_progress()
{
	$pid = array_get_default($_REQUEST, 'progress_id');
	if(! $pid)
		return json_bad_request('Missing progress ID');
	$timestamp = array_get_default($_REQUEST, 'timestamp');
	require_once('modules/Scheduler/utils.php');
	$ppath = scheduler_progress_path($pid);
	if(! $ppath) $data = [];
	else {
		require_once('include/ProgressIndicator.php');
		$indicator = new ProgressIndicator($ppath);
		if(! empty($timestamp)) {
			$data = $indicator->pollHistory($timestamp, 5);
		} else {
			$data = $indicator->requestHistory();
		}
	}
	json_return_value($data);
}

?>
