<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');

global $scheduler_start_time;

function scheduler_update_config() {
	$last = AppConfig::setting('scheduler.last_cron_run');
	if(strlen($last)) {
		$last = strtotime($last.' GMT');
		AppConfig::set_local('scheduler.last_cron_interval', strtotime('now') - $last);
	}
	AppConfig::set_local('scheduler.last_cron_run', gmdate('Y-m-d H:i:00'));
}


function scheduler_retrieve_pending($type=null, $where='', $when=null) {
	global $scheduler_start_time;
	if(! isset($when)) $when = isset($scheduler_start_time) ? $scheduler_start_time : time();
	$now = gmdate('Y-m-d H:i:00', $when);
	$lq = new ListQuery('Schedule', true);
	$filters = array();
	if($type != null)
		$filters[] = array('field' => 'type', 'value' => $type);
	$filters[] = array('field' => 'enabled', 'operator' => 'true');
	$filters[] = "(next_run IS NULL OR next_run <= '$now')";
	$lq->addFilterClauses($filters);
	return $lq->fetchAll('last_run ASC');
}

function scheduler_run_pending($type=null, $where='', $progress=null) {
	scheduler_set_time_limit(290); // 5 minutes less 10 seconds
	$pending = scheduler_retrieve_pending($type, $where);
	if($pending)
		scheduler_run_list($pending, false, $progress);
}

function scheduler_run_type($type, $force=true, $progress=null) {
	scheduler_set_time_limit(290); // 5 minutes less 10 seconds
	$lq = new ListQuery('Schedule');
	$lq->addSimpleFilter('type', $type);
	$result = $lq->runQuery();
	if($result && ! $result->failed)
		scheduler_run_list($result, $force, $progress);
}

function scheduler_run_list(ListResult $lst, $force=false, $progress=null) {
	global $current_user, $scheduler_start_time;
	if (!defined('inScheduler')) {
		define('inScheduler', 1);
	}
	if(empty($scheduler_start_time))
		$scheduler_start_time = time();
	$last_ignore = ignore_user_abort(true);

	foreach($lst->getRowIndexes() as $idx) {
		$sched = $lst->getRowResult($idx);
		scheduler_run_result($sched, $force, $progress);
	}

	ignore_user_abort($last_ignore);
	unset($scheduler_start_time);
}

function scheduler_run_result(RowResult $sched, $force=false, $progress=null) {
	global $current_user, $db;

	$id = $sched->getField('id');
	$crash_clause = '';
	if (!$force) {
		$crash_time = gmdate('Y-m-d H:i:00', time() - 3600);
		$crash_clause = " AND (IFNULL(status, '') != 'running' OR IFNULL(last_run, '') <= '$crash_time' )";
	}
	$now = gmdate('Y-m-d H:i:s');
	$now_0min = gmdate('Y-m-d H:i:00');
	$next_run = scheduler_get_next_run($sched, $now_0min);
	$max_duration = max(0, strtotime($next_run . ' UTC') - time());

	$where = "AND (next_run <= '$now_0min' OR next_run IS NULL)";
	if ($force)
		$where = '';

	$query = "UPDATE schedules SET 
				last_run = '$now_0min', next_run = '$next_run', status = 'running', date_modified='$now'
				WHERE id='$id' $where $crash_clause";
	$res = $db->query($query);
	if (!$db->getAffectedRowCount())
		return false;

	$orig_user = ! empty($current_user) ? clone($current_user) : null;
	scheduler_perform($sched->getField('type'), $progress, $max_duration);

	$query = "UPDATE schedules SET status = '' WHERE id='$id'";
	$db->query($query);

	$current_user->cleanup();
	$current_user = $orig_user;
}


function scheduler_perform($type, $progress=null, $max_duration=0) {
	static $registered;
	if(empty($registered)) {
		register_shutdown_function('scheduler_cleanup');
		$registered = true;
	}
	$spec = AppConfig::setting("scheduler_tasks.$type");
	if($spec) {
		$restore = array();
		if (!empty($spec['set_globals'])) {
			foreach ($spec['set_globals'] as $name => $value) {
				$restore[$name] = isset($GLOBALS[$name]) ? $GLOBALS[$name] : null;
				$GLOBALS[$name] = $value;
			}
		}
		$task_name = translate_dd('schedule_type_dom', $type, $type, $lang=null, 'Scheduler');
		$GLOBALS['scheduler_progress_obj'] = $progress;
		$GLOBALS['scheduler_task_name'] = $task_name;
		$GLOBALS['scheduler_task_status'] = '';
		scheduler_set_time_limit($max_duration);
		scheduler_task_progress(0);
		if(isset($spec['function'])) {
			require_once($spec['file']);
			call_user_func($spec['function']);
		} else {
			include($spec['file']);
		}
		if($restore)
			array_extend($GLOBALS, $restore);
		scheduler_task_progress(100);
		unset($GLOBALS['scheduler_task_name']);
		unset($GLOBALS['scheduler_start_task']);
		unset($GLOBALS['scheduler_limit_task']);
	}
}


function scheduler_cleanup() {
	global $scheduler_task_name;
	$err = error_get_last();
	$msg = 'Task terminated prematurely';
	if($err && ($err['type'] == E_ERROR || $err['type'] == E_USER_ERROR))
		$msg .= ': '.$err['message'];
	scheduler_task_finish('error', $msg);
}

function scheduler_progress_id() {
	return str_replace('-', '', create_guid());
}

function scheduler_progress_path($pid) {
	if(! $pid) return false;
	$pdir = rtrim(AppConfig::setting('scheduler.progress_log_dir'), '/');
	if($pdir) $pdir .= '/';
	return $pdir . 'scheduler-' . $pid . '.log';
}

function scheduler_set_time_limit($limit) {
	global $scheduler_limit_time, $scheduler_start_time;
	global $scheduler_limit_task, $scheduler_start_task;
	global $scheduler_task_name;
	if($limit > 0) {
		if(! empty($scheduler_task_name)) {
			$remain = scheduler_time_remain(true);
			$scheduler_start_task = time();
			$scheduler_limit_task = $scheduler_start_task + $limit;
			if($remain) {
				$scheduler_limit_task = min(time() + $remain, $scheduler_limit_task);
				$limit = $scheduler_limit_task - $scheduler_start_task;
			}
			//scheduler_task_message("Set task time limit: $limit s");
		} else {
			$scheduler_start_time = time();
			$scheduler_limit_time = $scheduler_start_time + $limit;
			set_time_limit($limit);
			//scheduler_task_message("Set total time limit: $limit s");
		}
	}
}

function scheduler_time_remain($total=false) {
	global $scheduler_limit_time, $scheduler_start_time;
	if(! $total) {
		if(! empty($scheduler_limit_task)) {
			return max(0, $scheduler_limit_task - time());
		}
	}
	if(! empty($scheduler_limit_time)) {
		return max(0, $scheduler_limit_time - time());
	}
}

function scheduler_should_abort($total=false) {
	$remain = scheduler_time_remain($total);
	if(isset($remain)) return $remain < 5; // leave 5 seconds available
}

function scheduler_task_finish($status='success', $message='') {
	global $scheduler_task_name, $scheduler_progress_obj, $log;
	if(isset($scheduler_progress_obj) && ! empty($scheduler_task_name)) {
		$scheduler_progress_obj->finish($status, $message);
	}
	if($status == 'error' && isset($log) && $message && ! empty($scheduler_task_name)) {
		$log->error("Scheduler [$scheduler_task_name]: $message");
	}
}

function scheduler_task_progress($progress) {
	global $scheduler_task_name, $scheduler_progress_obj;
	if(isset($scheduler_progress_obj) && ! empty($scheduler_task_name)) {
		$scheduler_progress_obj->logProgress($scheduler_task_name, round($progress, 1));
	}
}

function scheduler_task_message($message, $severity='info') {
	global $scheduler_progress_obj, $log;
	if(isset($scheduler_progress_obj)) {
		$scheduler_progress_obj->logMessage($message, $severity);
	}
	if($severity == 'error' && isset($log)) {
		$log->error($message);
	}
}


function scheduler_last_run_ts(RowResult &$sched, $time = null) {
	if ($time)
		$last_run = $time;
	else
		$last_run = $sched->getField('last_run');
	if(!empty($last_run)) {
		$t = strtotime($last_run.' GMT');
		if($t == -1 || $t === false)
			return -1;
		return $t;
	}
	return -1;
}


function scheduler_get_next_run(RowResult $sched, $time = null) {
	$last = scheduler_last_run_ts($sched, $time);
	if($last == -1)
		$last = strtotime('now');
	$size = $sched->getField('run_interval');
	$unit = $sched->getField('run_interval_unit');
	if($unit == 'quarters') {
		$unit = 'months';
		$size *= 3;
	}
	if($unit == 'busdays') {
		$unit = 'weekdays';
		$size *= 7;
	}
	$next = strtotime("+$size $unit", $last);
	return gmdate('Y-m-d H:i:00', $next);
}


function scheduler_rebuild_tasks($force=false) {
	$tasks = AppConfig::setting('scheduler_tasks');
	$lq = new ListQuery('Schedule', true, array('filter_deleted' => false));
	$result = $lq->fetchAll();
	$db_tasks = array();
	if($result) {
		foreach($result->getRowIndexes() as $k) {
            $task_row = $result->getRowResult($k);
            $task = RowUpdate::for_result($task_row);
			$db_tasks[$task->getField('type')] = $task;
		}
	}

	foreach ($tasks as $type => $def) {
        if ($def['next_run']??null){
            $def['next_run'] = gmdate('Y-m-d H:i:s', strtotime($def['next_run']));
        }
		if(isset($db_tasks[$type])) {
			if($force) {
				$db_tasks[$type]->set($def);
				$db_tasks[$type]->set('deleted', 0);
				$db_tasks[$type]->save();
			} else {
				if($db_tasks[$type]->getField('deleted'))
					$db_tasks[$type]->markDeleted(false);
			}
		} else {
			$task = RowUpdate::blank_for_model('Schedule');
			$def['type'] = $type;
			$task->set($def);
			$task->save();
			$db_tasks[$type] = $task;
		}
	}

	foreach(array_diff_key($db_tasks, $tasks) as $obsolete => $_) {
		$db_tasks[$obsolete]->markDeleted();
	}
}
