<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

require_once('modules/SecurityGroups/SecurityGroup_sugar.php');
class SecurityGroup extends SecurityGroup_sugar {
	
    
	var $last_run = array('module' => '', 'record' => '', 'action' => '', 'response' => '');

	
	/**
	 * Gets the join statement used for returning all rows in a list view that a user has group rights to.
	 * Make sure any use of this also return records that the user has owner access to. 
	 * (e.g. caller uses getOwnerWhere as well)
	 *
	 * @param GUID $user_id
	 * @return STRING
	 */
	function getGroupWhere($table_name,$module,$user_id)
	{

		//need a different query if doing a securitygroups check
		if($module == "SecurityGroups") {
			return " $table_name.id in (
				select secg.id from securitygroups secg
				inner join securitygroups_users secu on secg.id = secu.securitygroup_id and secu.deleted = 0 
					and secu.user_id = '$user_id'
				where secg.deleted = 0  
			)";
		} else {
			return " $table_name.id in (
				select secr.record_id from securitygroups secg
				inner join securitygroups_users secu on secg.id = secu.securitygroup_id and secu.deleted = 0 
					and secu.user_id = '$user_id'
				inner join securitygroups_records secr on secg.id = secr.securitygroup_id and secr.deleted = 0 
					and secr.module = '$module' 
				where secg.deleted = 0  
			)";

			//and secr.record_id = $table_name.id //not needed as the in clause takes care of this check
		}
	}
	
	/**
	 * @returns true if group is assigned to the record
	 */
	static function groupHasAccess($module,$id)
	{
		if(!isset($id) || $id == '[SELECT_ID_LIST]')
		{
			return true; //means that this is a listview and everybody is an owner of the listview
		}
		$security_modules = SecurityGroup::getSecurityModules();
		if(empty($security_modules[$module])) {
			return false;
		}
		
		global $current_user;
		$cache_key = "SecurityGroups.{$current_user->id}.$module";
		$cache_entry = sugar_cache_get($cache_key, $found);
		if($found && isset($cache_entry[$id]))
			return $cache_entry[$id];
		if(! is_array($cache_entry))
			$cache_entry = array();

		global $db;
		$query = "select count(securitygroups.id) as results from securitygroups "
				."inner join securitygroups_users on securitygroups.id = securitygroups_users.securitygroup_id and securitygroups_users.deleted = 0 "
				."  and securitygroups_users.user_id = '$current_user->id' "
				."inner join securitygroups_records on securitygroups.id = securitygroups_records.securitygroup_id and securitygroups_records.deleted = 0 "
				."	and securitygroups_records.record_id = '$id' "
				."	and securitygroups_records.module = '$module' "
				."where securitygroups.deleted = 0  ";
		$GLOBALS['log']->debug("SecuritySuite: groupHasAccess $query");
		$result = $db->query($query, true);
		$row = $db->fetchByAssoc($result);
		$cache_entry[$id] = (isset($row) && $row['results']>0);
		sugar_cache_put($cache_key, $cache_entry, false, true);
		
		return $cache_entry[$id];
	}
	
	// quickly retrieve access for multiple entries
	static function groupHasAccessList($module, $id_list) {
		if(! is_countable($id_list) || ! count($id_list))
			return true;
		$security_modules = SecurityGroup::getSecurityModules();
		if(empty($security_modules[$module])) {
			return false;
		}
		
		global $current_user;
		$cache_key = "SecurityGroups.{$current_user->id}.$module";
		$cache_entry = sugar_cache_get($cache_key, $found);
		if(! $found || ! is_array($cache_entry))
			$cache_entry = array();
		$missing = array();
		foreach($id_list as $id) {
			if(! isset($cache_entry[$id]))
				$missing[] = $id;
		}
		if(! is_countable($missing) || ! count($missing))
			return;
		
		global $db;
		$query = "SELECT DISTINCT securitygroups_records.record_id FROM securitygroups_records "
				."LEFT JOIN securitygroups ON securitygroups.id = securitygroups_records.securitygroup_id AND securitygroups.deleted = 0 "
				."LEFT JOIN securitygroups_users ON securitygroups.id = securitygroups_users.securitygroup_id AND securitygroups_users.deleted = 0 "
					."  AND securitygroups_users.user_id = '{$current_user->id}' "
				."WHERE securitygroups_records.module = '$module' "
				."AND securitygroups_records.record_id IN ('".implode("','", $missing)."') "
				."AND securitygroups_users.securitygroup_id IS NOT NULL";
		$GLOBALS['log']->debug("SecuritySuite: groupHasAccessList $query");
		$result = $db->query($query, true);
		$ok = array();
		while($row = $db->fetchByAssoc($result, -1, false))
			$ok[$row['record_id']] = 1;
		foreach($missing as $id)
			$cache_entry[$id] = ! empty($ok[$id]);
		sugar_cache_put($cache_key, $cache_entry, false, true);
	}
	
	
	/**
	 * returns # of groups a user is a member of that are inheritable
	 *
	 * TODO: cache this value in the session var
	 */
	function getMembershipCount($user_id) {
		global $db; 
		
		if(!isset($_SESSION['securitygroup_count'])) {
			$query = "select count(securitygroups.id) as results from securitygroups "
					."inner join securitygroups_users on securitygroups.id = securitygroups_users.securitygroup_id "
					." and securitygroups_users.deleted = 0 "
					." where securitygroups.deleted = 0 and securitygroups_users.user_id = '$user_id' "
					."  and (securitygroups.noninheritable is null or securitygroups.noninheritable <> 1) "
					."  and (securitygroups_users.noninheritable is null or securitygroups_users.noninheritable <> 1) ";
			$GLOBALS['log']->debug("SecuritySuite: Inherit One Pre-Check Qualifier: $query");
			$result = $db->query($query);
			$row = $db->fetchByAssoc($result);
			if(isset($row)) {
				$_SESSION['securitygroup_count'] = $row['results'];
			}
		}
		
		return $_SESSION['securitygroup_count'];
	}
	
    
    /**
     * Used to get the modules that are tied to security groups. 
     * There should be a relationship of some sort in order to tie the two together.
     *
     * This will be used for things such as default groups for modules, etc.
     */
    static function getSecurityModules() {
    	global $db;
    	$cache_key = "SecurityGroups.security_modules";
    	$cache_entry = sugar_cache_get($cache_key, $found);
		if($found)
			return $cache_entry;
    	
    	// ajrw - securitygroups still implemented at module level
    	// should probably be moved to bean level
    	$security_modules = array();
    	$beans = AppConfig::setting('model.index.securitygroups', array());
    	foreach($beans as $b) {
    		$module = AppConfig::setting("model.index.bean_module.$b");
    		if($module)
				$security_modules[$module] = $module;
		}
		sugar_cache_put($cache_key, $security_modules);
		
		return $security_modules;
    }

	static function getConfig()
	{
		return AppConfig::setting('aclconfig');
	}

    /** To get the link name used to call load_relationship */
    function getLinkName($this_module, $rel_module) {
    	return 'securitygroups';
    }
    
    /**
     * Add a Security Group to a record
     */
    function addGroupToRecord($module, $record_id, $securitygroup_id) {
    	if(empty($module) || empty($record_id) || empty($securitygroup_id)) {
    		return; //missing data
    	}
		global $db; 
		$query = "insert into securitygroups_records(id,securitygroup_id,record_id,module,date_modified,deleted) "
				."values( '".create_guid()."','".$securitygroup_id."','$record_id','$module',".db_convert('','today').",0) ";
		$GLOBALS['log']->debug("SecuritySuite: addGroupToRecord: $query");
		$db->query($query,true); 
    }
    
    function getUserSecurityGroups($user_id) {
    	return self::get_user_security_groups($user_id);
    }
    
	/**
	 * Return a list of groups that this user belongs to.
	 */
	static function get_user_security_groups($user_id)
	{	
		global $db;
		$query = "select securitygroups.id, securitygroups.name from securitygroups_users "
				."inner join securitygroups on securitygroups_users.securitygroup_id = securitygroups.id "
				."		and securitygroups.deleted = 0 "
				."where securitygroups_users.user_id='$user_id' and securitygroups_users.deleted = 0 ";
		$result = $db->query($query,true,"Error finding the full membership list for a user: ");
		
		$group_array = Array();
		$result = $db->query($query);
		while(($row=$db->fetchByAssoc($result)) != null) {			
			$group_array[$row['id']] = $row;
		}

		return $group_array;
	}

	/**
	 * Return a list of all groups
	 */
	function getAllSecurityGroups()
	{	
		global $db;
		$query = "select id, name from securitygroups "
				."where securitygroups.deleted = 0 "
				."order by name";
		$result = $db->query($query,true,"Error finding the full membership list for a user: ");
		
		$group_array = Array();
		$result = $db->query($query);
		while(($row=$db->fetchByAssoc($result)) != null) {			
			$group_array[$row['id']] = $row;
		}

		return $group_array;
	}
	
	
	static function after_save(RowUpdate $upd) {
		AppConfig::invalidate_cache('acl');
	}
	
	static function update_acl(RowUpdate &$update, $link_name) {
		$model = $update->getModelName();
		if($model == 'SecurityGroup' && ($link_name == 'users' || $link_name == 'aclroles')) {
			AppConfig::invalidate_cache('acl');
		}
	}

}
?>
