/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

AppBase.module('serials').moduleRequires('ui-controls').init(function(App) {
	var UI = App.ui,
		Wrap = App.dom.elementOrId,
		QueryId = App.dom.queryId,
		MakeElt = App.dom.makeElement;

	UI.SerialNumberEditor = function(id, params) {
		this.id = 'SerialNumberEditor';
		this.editable = true;
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		if(params) extendObject(this, params);
		if(! Array.isArray(this.items))
			this.items = [];
	};
	
	UI.SerialNumberEditor.prototype = {
		setup: function() {
			if(this.set_up) return;
			if(! this.elt)
				this.elt = $(this.id + '-main');
			if(! this.elt)
				return;

			this.item_names = App.language.getList('serial_number_item_dom', 'app');
			if (this.editable) {
				//UI.attachFormInputEvent(this.form, 'assigned_user', 'onchange', this.on_user_changed.bind(this));
			}
			this.add_button = Wrap(this.id + '-add-item');
			if(this.add_button) {
				this.add_button.on('click', function() {
					this.addItem();
				}, {scope: this});
			}
			this.updated = false;
			this.renderRows();
			this.set_up = true;			
		},
		
		addItem: function(item) {
			if(! item) item = {};
			this.items.push(item);
			this.updated = true;
			this.renderRows();
		},
		
		removeItem: function(index) {
			this.items.splice(index, 1);
			this.updated = true;
			this.renderRows();
		},

		clearItems: function() {
			this.items = [];
			this.updated = true;
			this.renderRows();
		},

		iterItems: function(callback, extra) {
			for(var idx = 0; idx < this.items.length; idx++) {
				callback.call(this, idx, this.items[idx]);
			}
			return idx;
		},
		
		updateItem: function(idx, field, val) {
			this.items[idx][field] = val;
			this.updated = true;
		},

		renderRows: function() {
			var tbl = Wrap(this.elt).findFirst('.serial-items-body').ext(0);
			if(! tbl) return;
			tbl.empty();
			var rowidx = 0;
			var last_date;
			var drew = 0;
			var self = this;
			var drawLine = function(idx, line) {
				var outer = MakeElt('div.row.serial-row-outer').appendTo(tbl);
				var main = MakeElt('div.column.serial-row-main').appendTo(outer);
				var inner = MakeElt('div.row.col-space-sm.serial-row-inner').appendTo(main);
				
				var cell = MakeElt('div.column.serial-col-name').appendTo(inner);
				if(this.editable) {
					var sel = new UI.SelectInput(this.id + '-type-' + idx, {
						options: 'serial_number_item_dom', options_width: 300,
						ext_class: 'input-block',
						init_value: line.item_name,
						onchange: function() {
							self.updateItem(idx, 'item_name', this.getValue());
						}
					});
					cell.append(sel.render());
					sel.setup();
				} else {
					cell.append(this.item_names.getString(line.item_name));
				}
		
				cell = MakeElt('div.column.serial-col-serial').appendTo(inner);
				if(this.editable) {
					var sel = new UI.TextInput(this.id + '-serial-' + idx, {
						ext_class: 'input-block',
						init_value: line.serial_no,
						onchange: function() {
							self.updateItem(idx, 'serial_no', this.getValue());
						}
					});
					cell.append(sel.render());
					sel.setup();
				} else {
					cell.append(line.serial_no);
				}

				cell = MakeElt('div.column.serial-col-notes').appendTo(inner);
				if(this.editable) {
					var sel = new UI.TextInput(this.id + '-notes-' + idx, {
						ext_class: 'input-block',
						init_value: line.notes,
						onchange: function() {
							self.updateItem(idx, 'notes', this.getValue());
						}
					});
					cell.append(sel.render());
					sel.setup();
				} else {
					cell.append(line.notes);
				}

				cell = MakeElt('div.column.serial-col-end').appendTo(outer);
				if(this.editable) {
					MakeElt('span.uii.uii-lg.active-icon.uii-remove')
						.on('click', function() {
							this.removeItem(idx);
							return false;
						}, {scope: this})
						.appendTo(cell);
				}
			};
			
			if(! this.iterItems(drawLine)) {
				var row = MakeElt('div.row.serial-row-blank').appendTo(tbl);
				var cell = MakeElt('div.column.serial-col-blank.input-field-pad.text-help').appendTo(row);
				cell.append(app_string('LBL_NONE'));
			}
		},

		beforeSubmitForm: function() {
			if(this.updated) {
				var result = this.form.serial_updates;
				if(! result) result = MakeElt('input', {type: 'hidden', name: 'serial_updates'})
					.appendTo(this.form).get();
				result.value = JSON.stringify(this.items);
			}
		}
	}
}); // SerialNumberEditor

/*
var item_name_opts = {};
var image_path;
var default_limit = null;
var request_id = 0; // used by http_fetch_sync

function hasAttribute(element, attr) {
	if (element.hasAttribute) return element.hasAttribute(attr);
	return (typeof(element.getAttribute(attr)) == typeof(''));
}

var serials_editor = new function() {
	var form_id, table_id;
	var assembly_id, asset_id;
	var focus_type, focus_id;
	var current_offset;
	var instances_quantity = 0;
	var current_quantity = 0;
	var rowCount = 1;
	var save_url;
	var serials = [];
	var updated_serials = {};
	var the_editor = this;
	
	this.init = function(_form_id, _table_id, _assembly_id, _asset_id, _focus_type, _focus_id, _save_url) {
		form_id = _form_id;
		table_id = _table_id;
		assembly_id = _assembly_id;
		asset_id = _asset_id;
		focus_type = _focus_type;
		focus_id = _focus_id;
		save_url = _save_url;
	}
	
	this.setSerials = function(_serials, _offset, _totalCount, _for_duplicate) {
		serials = _serials;
		updated_serials = {};
		current_offset = _offset;
		instances_quantity = _totalCount;
		current_quantity = serials.length;
		this.clearTable();
		for(var idx = 0; idx < serials.length; idx++) {
			this.addRow(idx, serials[idx]);
			if (_for_duplicate)
				updated_serials['' + idx] = 1;
		}
	}
	
	this.addSerial = function(newserial) {
		var defaults = {
			assembly_id: assembly_id,
			asset_id: asset_id,
			instance_id: '',
			serial_no: '',
			item_name: '',
			notes: '',
			deleted: 0
		};
		var idx = serials.length;
		for(var f in defaults)
			if(!isset(newserial[f]))
				newserial[f] = defaults[f];
		serials.push(newserial);
		updated_serials[idx] = 1;
		instances_quantity ++;
		current_quantity ++;
		this.addRow(idx, newserial);
	}

	this.addRow = function(serialIndex, serial)
	{
		var form = document.getElementById(form_id);

		/*if (!instance_id && form.quantity.value <= instances_quantity) {
			alert(LANGUAGE.mod_strings.LBL_INSTANCES_ALERT);
			return false;
		}*//*
		
		var table = document.getElementById(table_id);
		var row = table.insertRow(table.rows.length );
		var rowName = 'item_row_' + rowCount;
		row.id = rowName;
		row.serialIndex = serialIndex;
		
		var cell1 = row.insertCell(row.cells.length);
		cell1.appendChild(document.createTextNode('\u00a0'));
		
		var cell3 = row.insertCell(row.cells.length);
		cell3.noWrap = 'nowrap';
		cell3.className = 'dataField';
		var selItemName = document.createElement('select');
		selItemName.serialIndex = serialIndex;
		selItemName.field = 'item_name';
		selItemName.onchange = function() { the_editor.updateField(this); }
        var counter = 0;
		for(var idx in item_name_opts) {
			var opt = document.createElement('option');
			opt.value = idx;
			opt.text = item_name_opts[idx];
			if(idx == serial.item_name) {
				opt.selected = true;
            } else if (! serial.item_name && counter == 0) {
                opt.selected = true;
            }
			try {
				selItemName.add(opt, null);
			}
			catch(e) {
				selItemName.add(opt);
			}
            counter ++;
		}
		cell3.appendChild(selItemName);
		
		var cell4 = row.insertCell(row.cells.length);
		cell4.nowrap = 'nowrap';
		cell4.className = 'dataField';
		var textE2 = document.createElement('input');
		textE2.type = 'text';
		textE2.className = 'input-text';
		textE2.serialIndex = serialIndex;
		textE2.field = 'notes';
		textE2.value = serial.notes;
		textE2.onchange = function() { the_editor.updateField(this); }
		textE2.size = 40;
		cell4.appendChild(textE2);
		
		var cell2 = row.insertCell(row.cells.length);
		cell2.noWrap = 'nowrap';
		cell2.className = 'dataField';
		var textE1 = document.createElement('input');
		textE1.type = 'text';
		textE1.className = 'input-text';
		textE1.serialIndex = serialIndex;
		textE1.field = 'serial_no';
		textE1.value = serial.serial_no;
		textE1.size = 30;
		textE1.onchange = function() { the_editor.updateField(this); }
		cell2.appendChild(textE1);
		
		var cell4 = row.insertCell(row.cells.length);
		cell4.nowrap = 'nowrap';
		var textE3 = document.createElement('a');
		textE3.href = '#';
		textE3.className = 'listViewTdToolsS1';
		textE3.onclick = function() { the_editor.deleteRow(rowName); return false; }
		textE3.innerHTML = '<div class="input-icon active-icon icon-delete left"></div>';
		cell4.appendChild(textE3);
		
		rowCount ++;
		//this.updateRange();
	}
	
	this.updateField = function(field) {
		serials[field.serialIndex][field.field] = field.value;
		updated_serials[field.serialIndex] = 1;
	}
	
	this.deleteRow = function(rowId) {
		var table = document.getElementById(table_id);
		var row = document.getElementById(rowId);
		serials[row.serialIndex].deleted = 1;
		updated_serials[row.serialIndex] = 1;
		table.deleteRow(row.rowIndex);
		current_quantity --;
		instances_quantity --;
		//this.updateRange();
	}

	this.clearTable = function()
	{
		var table = document.getElementById(table_id);
		var form = document.getElementById(form_id);
		var rows = table.rows.length;
		for(i = rows - 1 ; i > 0; i--){
			table.deleteRow(i);
		}
		rowCount = 1;
	}

	this.updateRange = function() {
		var icons = ['start', 'previous', 'next', 'end'];
		for(var i = 0; i < 4; i++) {
			var t = icons[i];
			var enabled;
			if(t == 'start' || t == 'previous')
				enabled = (current_offset > 0);
			else
				enabled = (instances_quantity > current_offset + default_limit);
			document.getElementById('serial_list_nav_'+t+(enabled ? '_on' : '_off')).style.display = 'inline';
			document.getElementById('serial_list_nav_'+t+(enabled ? '_off' : '_on')).style.display = 'none';
		}
		document.getElementById('serial_list_nav_index').innerHTML = '(' + (current_quantity > 0 ? current_offset + 1 : 0) + ' - ' + (current_offset + current_quantity) + ' of ' + instances_quantity + ')';
	}
	
	this.getChanges = function() {
		var changes = [];
		for(var idx in updated_serials) {
			if(updated_serials[idx]) {
				var serial = serials[idx];
				if(serial.instance_id == '' && serial.deleted)
					continue;
				changes.push(serial);
			}
		}
		var json = changes.length ? JSON.stringify(changes) : '';
		return json;
	}

	this.navigate = function(nav_action) {
		var offset;
		switch(nav_action) {
			case 'start':
				offset = 0; break;
			case 'previous':
				offset = Math.max(0, current_offset - default_limit); break;
			case 'next':
				offset = current_offset + current_quantity; break;
			case 'end':
				offset = Math.max(0, instances_quantity - default_limit); break;
		}
		var form = document.getElementById(form_id);
		
		var changed = this.getChanges();
		var post_data = 'serial_updates='+escape(changed);
		post_data += '&serial_offset='+offset;
		post_data += '&object_name='+focus_type;
		post_data += '&record='+focus_id;
		var result = http_fetch_sync(save_url, post_data);
		eval(result.responseText);
	}

};
*/
