<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormTableSection.php');

class SerialNumberWidget extends FormTableSection {

    /**
     * @var ListResult
     */
    var $list;

	var $dup;

	function init($params=null, $model=null) {
		parent::init($params, $model);
		if(! $this->id)
			$this->id = 'serial_numbers';
	}
	
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if($gen->getLayout()->getEditingLayout()) {
			$params = array();
			return $this->renderOuterTable($gen, $parents, $context, $params);		
		}

		$this->list = $this->loadList($row_result);
		$this->dup = !empty($row_result->for_duplicate);

		$allow_edit = $gen->getLayout()->getDefaultEditable() ? 1 : 0;
		
		$body = <<<EOQ
			<div id="{$this->id}-main" class="serial-items-main">
EOQ;

		$form_name = $gen->getFormName();

		$lbl_add = translate('LBL_BUTTON_ADD_LINE', 'SerialNumbers');
		if($allow_edit) {
			$body .= <<<EOQ
				<div class="row form-row col-unstack" id="{$this->id}-nav">
					<div class="column form-cell col-6">
						<div class="form-entry">
						<div class="buttons-inline">
							<div class="inline-elt">
								<button type="button" class="input-button" id="{$this->id}-add-item">
									<span class="uii uii-add"></span> <span class="input-label">$lbl_add</span>
								</button>
							</div>
						</div>
					</div>
				</div>
EOQ;
		}

        $name = translate('LBL_LIST_SERIAL_ITEM_NAME', 'SerialNumbers');
        $serial = translate('LBL_LIST_SERIAL_NO', 'SerialNumbers');
        $notes = translate('LBL_LIST_SERIAL_NOTES', 'SerialNumbers');
        $empty_cell = '';
        $panel_edit = $allow_edit ? '' : 'panel-readonly';

		$body .= <<<EOQ
			<div class="row form-row">
			<div class="column form-cell expand"><div class="form-entry">
			<section class="card-outer panel-outer panel-default serial-items-outer panel-border panel-round {$panel_edit}">
				<div class="card-header panel-subheader">
					<div class="row serial-head-outer panel-column-titles">
						<div class="column serial-head-main">
							<div class="row col-space-sm">
								<div class="column serial-row-head serial-col-name"><span class="column-title">{$name}</span></div>
								<div class="column serial-row-head serial-col-serial"><span class="column-title">{$serial}</span></div>
								<div class="column serial-row-head serial-col-notes"><span class="column-title">{$notes}</span></div>
							</div>
						</div>
						<div class="column serial-row-head serial-col-end">
						</div>
					</div>
				</div>
				<div class="card-body panel-body panel-pad-body serial-items-body">
				</div>
			</section>
			</div></div>
			</div>
EOQ;

		$items = [];
		foreach ($this->list->rows as $id => $row) {
			$items[] = [
				'id' => $id,
				'serial_no' => $row['serial_no'],
				'item_name' => $row['item_name'],
				'notes' => $row['notes']
			];
		}
		$editParams = ['items' => $items, 'editable' => $allow_edit];
            
		$gen->getLayout()->addScriptInclude('modules/SerialNumbers/serials.js', LOAD_PRIORITY_BODY);
		$gen->getFormObject()->addControl('SerialNumberEditor', $this->id, $editParams);

		$params['body'] = $body . '</div>';
        return $this->renderOuterTable($gen, $context, $parents, $params);
	}
	
	function getRequiredFields() {
		return array();
	}
	
    /**
     * Load serial numbers list
     *
     * @param RowResult $row_result
     * @return ListResult|null
     */
    function loadList(RowResult $row_result) {
        $list = null;
        $filter_name = '';
        $row_id = $row_result->getField('id');

        if ($this->model->name == 'Asset') {
            $filter_name = 'asset_id';
        } elseif ($this->model->name == 'SupportedAssembly') {
            $filter_name = 'assembly_id';
        }

        if ($filter_name != '' && $row_id) {
            $fields = array('id', 'serial_no', 'item_name', 'notes', 'asset_id', 'assembly_id', 'deleted');
            $lq = new ListQuery('SerialNumber', $fields, array('acl_filters' => 'view', 'formatted' => 'html'));

            $clauses = array(
                "target" => array(
                    "value" => $row_id,
                    "field" => $filter_name,
                ),
            );

            $lq->addFilterClauses($clauses);
            $list = $lq->runQuery();
        } else {
        	$list = new ListResult();
        	$list->rows = [];
        }

        return $list;
    }

	function loadUpdateRequest(RowUpdate $update, array $input) {
		$upd_numbers = array();
        if (! empty($input['serial_updates']))
            $upd_numbers['serials'] = $input['serial_updates'];

        $update->setRelatedData($this->id.'_rows', $upd_numbers);
	}
	
	function validateInput(RowUpdate $update) {
		return true;
	}
	
	function afterUpdate(RowUpdate $update) {
		$row_updates = $update->getRelatedData($this->id.'_rows');
		if(! $row_updates)
			return;

        if (isset($row_updates['serials'])) {
            $json = getJSONobj();
            $serials = $json->decode($row_updates['serials']);
			$serq = new ListQuery('SerialNumber');
			$upd_type = $update->getModelName();
			$upd_id = $update->getPrimaryKeyValue();
			if($upd_type == 'Asset')
				$serq->addSimpleFilter('asset_id', $upd_id);
			else
				$serq->addSimpleFilter('assembly_id', $upd_id);
			$exist = [];
			$seen = [];
			foreach($serq->fetchAll() as $id => $row) {
				$exist[$id] = $row;
			}

			$elements = ['serial_no', 'item_name', 'notes'];

            if (is_array($serials)) {
				foreach($serials as $row) {
					if(! is_array($row))
						continue;
					$is_empty = true;
					$updates = [];
					foreach ($elements as $e) {
						$value = trim(array_get_default($row, $e));
						if (strlen($value)) {
							$updates[$e] = $value;
							$is_empty = false;
						} else {
							$updates[$e] = '';
						}
					}
					if ($is_empty)
                		continue;
                	if(! empty($row['id']) && isset($exist[$row['id']])) {
                		$upd = $exist[$row['id']]->getRowUpdate();
                		$seen[$row['id']] = 1;
                	} else {
                		$upd = RowUpdate::blank_for_model('SerialNumber');
                	}
                	
                	if($upd_type == 'Asset')
						$upd->set('asset_id', $upd_id);
					else
						$upd->set('assembly_id', $upd_id);
                    $upd->set($updates);
                    $upd->save();
                }
            }
            
            foreach($exist as $id => $row) {
            	if(empty($seen[$id]))
            		$row->getRowUpdate()->markDeleted();
            }
        }
	}
	
}
?>
