<?php

require_once 'modules/Quotes/PDFTemplatesProvider.php';

class ShippingPDFTemplateProvider extends QuotePDFTemplateProvider
{
	
	protected function __construct($type)
	{
		parent::__construct($type);
	}

	public static function template_types()
	{
		return array(
			'shipping' => translate('LBL_PDF_PACKING_SLIP_TITLE', 'Shipping'),
		);
	}

	public static function template_skeleton($type)
	{
		$ret = array('sections' => array(
			array(
				'name' => 'header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_HEADER',
				'height' => 150,
				'elements' => array (),
			),
			array(
				'name' => 'group_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_GROUP_HEADER',
				'height' => 150,
				'elements' => array (),
			),
			array(
				'name' => 'group_items',
				'type' => 'table',
				'title' => 'LBL_SKEL_GROUP_ITEMS',
				'columns' => array (),
			),
			array(
				'name' => 'group_footer',
				'data_id' => 'group_header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_GROUP_FOOTER',
				'height' => 150,
				'elements' => array (),
			),
			array(
				'name' => 'footer',
				'data_id' => 'header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_FOOTER',
				'height' => 40,
				'elements' => array (),
			),
		));
		foreach ($ret['sections'] as &$sec) $sec['title'] = translate($sec['title'], 'Shipping');
		$ret['sections'] = array_values($ret['sections']);
		return $ret;
	}

	public static function template_sample_data($type)
	{
		global $locale;
		$datefmt = AppConfig::setting('locale.defaults.date_format');
		$data = array(
			'header' => array(
				'todays_date' => array(translate('LBL_TODAYS_DATE'), date($datefmt), ),
				'number' => array(translate('LBL_SAMPLE_HEADER_NUMBER_LABEL', 'Shipping'), 'S2013-001'),
				'sales_person' => array(translate('LBL_SAMPLE_HEADER_SALES_PERSON_LABEL', 'Shipping'), translate('LBL_SAMPLE_HEADER_SALES_PERSON_VALUE', 'Shipping'), ),
				'purchase_order' => array(translate('LBL_SAMPLE_HEADER_PURCHASE_ORDER_LABEL', 'Shipping'), translate('LBL_SAMPLE_HEADER_PURCHASE_ORDER_VALUE', 'Shipping'), ),
				'shipping_provider' => array(translate('LBL_SAMPLE_HEADER_SHIPPING_PROVIDER_LABEL', 'Shipping'), translate('LBL_SAMPLE_HEADER_SHIPPING_PROVIDER_VALUE', 'Shipping'), ),
				'shipping_address' => array(translate('LBL_SAMPLE_HEADER_SHIPPING_ADDRESS_LABEL', 'Shipping'), translate('LBL_SAMPLE_HEADER_SHIPPING_ADDRESS_VALUE', 'Shipping'), ),
				'tracking_number' => array(translate('LBL_SAMPLE_HEADER_TRACKING_NUMBER_LABEL', 'Shipping'), '123456789'),
				'description' => array(translate('LBL_SAMPLE_HEADER_DESCRIPTION_LABEL', 'Shipping'), translate('LBL_SAMPLE_HEADER_DESCRIPTION_VALUE', 'Shipping'), ),
				'date_shipped' => array(translate('LBL_SAMPLE_HEADER_DATE_SHIPPED_LABEL', 'Shipping'), '2014-11-05'),
    			'weight' => array(translate('LBL_SAMPLE_HEADER_WEIGHT', 'Shipping'), $locale->getUserFormatWeighty(array(1,1))),
			),
			'group_header' => array(
				'group_name' => array(translate('LBL_SAMPLE_GROUP_HEADER_GROUP_NAME_LABEL', 'Shipping'), translate('LBL_SAMPLE_GROUP_HEADER_GROUP_NAME_VALUE', 'Shipping'), ),
			),
			'group_items' => array(
				'number' => array(translate('LBL_SAMPLE_GROUP_ITEMS_NUMBER_LABEL', 'Shipping'), '1'),
				'product' => array(translate('LBL_SAMPLE_GROUP_ITEMS_PRODUCT_LABEL', 'Shipping'), translate('LBL_SAMPLE_GROUP_ITEMS_PRODUCT_VALUE', 'Shipping'), ),
				'quantity' => array(translate('LBL_SAMPLE_GROUP_ITEMS_QUANTITY_LABEL', 'Shipping'), '2'),
				'mfr_part_no' => array(translate('LBL_SAMPLE_GROUP_ITEMS_MFR_PART_NO_LABEL', 'Shipping'), '123456'),
				'vendor_part_no' => array(translate('LBL_SAMPLE_GROUP_ITEMS_VENDOR_PART_NO_LABEL', 'Shipping'), 'VP-1234'),
				'serial_numbers' => array(translate('LBL_SAMPLE_GROUP_ITEMS_SERIAL_NUMBERS_LABEL', 'Shipping'), '748905'),
			),
		);
		self::injectProductSampleData($data['group_items']);
		$ret = array();
		foreach ($data as $k => $v) {
			$ret[$k] = array();
			foreach ($v as $kk => $vv) {
				$ret[$k][$kk] = array('label' => $vv[0], 'value' => $vv[1]);
			}
		}
		return $ret;
	}


	public static function create($type)
	{
		return new ShippingPDFTemplateProvider($type);
	}

	public function initialize($layout, ListQuery $lq)
	{
		$lq->addField('supplier_contact');
		$lq->addField('supplier.name', 'supplier_name');
		$lq->addField('shipping_provider.name', 'shipping_provider_name');
		$lq->addField('shipping_account.name', 'shipping_account_name');
		return parent::initialize($layout, $lq);
	}
	
	public function getFileName() {
		$filename_pfx = translate('LBL_PDF_PACKING_SLIP_TITLE', 'Shipping');
		$filename = $filename_pfx . '_' . $this->row->getField('prefix');
		$filename .= $this->row->getField('shipping_number') . '_' . $this->row->getField('shipping_account_name');
		$filename = preg_replace('~\.+$~', '', trim($filename));
		$filename = $filename . '.pdf';
		return $filename;
	}

	public function getNoteTitle()
	{
		return translate('LBL_PREPARED_SHIPPING', 'Shipping');
	}

	public function getEmailTitle()
	{
		return translate('LBL_EMAILED_SHIPPING', 'Shipping');
	}

	public function getSignedDocumentTitle() {
		$prefix =
			$this->row
			? '[' . $this->row->getField('prefix') . $this->row->getField('shipping_number') . '] '
			: '';

		return $prefix . $this->getNoteTitle();
	}

	protected function createData()
	{
		global $locale;
		$row =& $this->row;
		$datefmt = AppConfig::setting('locale.defaults.date_format');
		$data = array(
			'todays_date' => date($datefmt),
			'number' => $row->getField('prefix') . $row->getField('shipping_number'),
			'sales_person' => $row->getField('assigned_user_name'),
			'shipping_address' => $this->_address('shipping_'),
			'description' => $row->getField('description'),
			'purchase_order' => $row->getField('purchase_order_num'),
			'shipping_provider' => $row->getField('shipping_provider_name'),
			'tracking_number' => $row->getField('tracking_number'),
			'date_shipped' => $row->getField('date_shipped'),
    		'weight' => $locale->getUserFormatWeighty(array($row->getField('weight_1'), $row->getField('weight_2'))),
		);
		$data = array_merge($this->header_data, $data);
		$this->data['header'] = $data;
		$this->createGroupData();
	}

	public static function get_extra_sources() {
		return array(
			array(
				'module' => 'Invoice',
				'model' => 'Invoice',
				'prefix' => 'invoice',
				'label' => 'LBL_INVOICE',
			),
			array(
				'module' => 'SalesOrders',
				'model' => 'SalesOrder',
				'prefix' => 'so',
				'label' => 'LBL_SALES_ORDER',
			),
			array(
				'module' => 'ShippingProviders',
				'model' => 'ShippingProvider',
				'prefix' => 'shipping_provider',
				'label' => 'LBL_SHIPPING_PROVIDER',
			),
			array(
				'module' => 'Accounts',
				'model' => 'Account',
				'prefix' => 'shipping_account',
				'label' => 'LBL_SHIPPING_ACCOUNT',
			),
			array(
				'module' => 'Contacts',
				'model' => 'Contact',
				'prefix' => 'shipping_contact',
				'label' => 'LBL_SHIPPING_CONTACT',
			),
			array(
				'module' => 'CompanyAddresses',
				'model' => 'CompanyAddress',
				'prefix' => 'warehouse',
				'label' => 'LBL_WAREHOUSE',
			),
		);
	}
}
	
