<?php

require_once 'include/layout/forms/FormTableSection.php';
require_once 'include/ListView/ListFormatter.php';
require_once 'include/layout/FieldFormatter.php';

class SigningActionsWidget extends FormTableSection {
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if($gen->getLayout()->getEditingLayout()) {
			$params = array();
			return $this->renderOuterTable($gen, $parents, $context, $params);		
		}
		return $this->renderHtmlImpl($gen, $row_result, $context, $gen->getLayout()->getDefaultEditable());
    }
	

	function renderHtmlImpl(HtmlFormGenerator $gen, RowResult $row_result, $context, $editable) {
		$fmt = new SigningActionsWidgetFormatter('html', 'view');
		ob_start();
		$lr = new ListResult();
		$rows = $this->getListRows($row_result);
		$lr->rows = $rows;
		$lr->fields = $this->getListFields();

		$fmt->formatListResult($lr);

		$l = new ListFormatter();
		$l->show_mass_update = false;
		$l->show_navigation = false;
		$l->show_checks = false;
		$l->no_sort = true;
		$l->columns = $this->getListColumns();
		$l->outputHtml($lr);
		return ob_get_clean();
	}

	protected function getListColumns() {
		return [
			[
				'field' => 'type',
				'vname' => 'LBL_LIST_ACTION_TYPE',
			],
			[
				'field' => 'name',
				'vname' => 'LBL_LIST_SIGNER_NAME',
			],
			[
				'field' => 'email',
				'vname' => 'LBL_LIST_SIGNER_EMAIL',
			],
			[
				'field' => 'phone',
				'vname' => 'LBL_LIST_SIGNER_PHONE',
			],
			[
				'field' => 'status',
				'vname' => 'LBL_LIST_SIGNER_STATUS',
				'add_fields' => [
					[
						'field' => 'decline_reason',
					]
				],
			],
		];
	}

	protected function getListFields() {
		return [
			'type' => [
				'name' => 'type',
				'type' => 'enum',
				'option' => 'signing_action_type_dom',
			],
			'name' => [
				'name' => 'name',
				'type' => 'varchar',
			],
			'email' => [
				'name' => 'email',
				'type' => 'email',
			],
			'phone' => [
				'name' => 'phone',
				'type' => 'phone',
			],
			'status' => [
				'name' => 'status',
				'type' => 'enum',
				'options' => 'signing_action_status_dom',
			],
			'decline_reason' => [
				'name' => 'decline_reason',
				'type' => 'text',
			],
		];
	}

	protected function getListRows(RowResult $row_result) {
		$w = json_decode($row_result->getField('project_source', ''), true);
		if (!is_array($w))
			$w = [];
		$actions = array_get_default($w, 'actions', []);
		return array_map([$this, 'getSignerRow'], $actions);
	}

	protected function getSignerRow($action) {
		return [
			'type' => 'Signer',
			'name' => $action['signer']['firstName'] . ' ' . $action['signer']['lastName'],
			'email' => $action['signer']['email'],
			'phone' => $action['signer']['phone'],
			'status' => $action['status'],
			'decline_reason' => join('; ', $action['declineReason']),
		];
	}
}

class SigningActionsWidgetFormatter extends FieldFormatter {
	function formatRows(array &$model_fields, array &$rows, $union_key=null) {
		$result = parent::formatRows($model_fields, $rows, $union_key);
		foreach ($result as &$row) {
			foreach ($row as $k => &$v) {
				switch ($k) {
				case 'decline_reason':
					if (strlen($v)) {
						$v = '<i>' . $v . '</i>';
					}
					break;
				}
			}
		}
		return $result;
	}
}
