/*****************************************************************************
 * The contents of this file are subject to The Long Reach Corporation
 * Software License Version 1.0 ("License"); You may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * <http://www.thelongreach.com/swlicense.html>
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) The Long Reach Corporation copyright notice,
 * (ii) the "Powered by SugarCRM" logo, and
 * (iii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is:
 *    Info At Hand Add-on Module to SugarCRM Open Source project.
 * The Initial Developer of this Original Code is The Long Reach Corporation
 * and it is Copyright (C) 2004-2007 by The Long Reach Corporation;
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2005 SugarCRM, Inc.;
 * All Rights Reserved.
 *****************************************************************************
 * $Id: skills.js 7296 2010-06-12 03:09:50Z andrew $
 * File Description:
 * Contributor(s):
*****************************************************************************/

// vim: set ai si :

AppBase.module('skills').moduleRequires('ui-controls').init(function(App) {
	var UI = App.ui,
		Wrap = App.dom.elementOrId,
		MakeElt = App.dom.makeElement;

	UI.SkillsEditor = function(id, params) {
		this.id = 'SkillsEditor';
		this.skills = {};
		this.allSkills = {};
		this.rowCount = 0;
		this.field = null;
		this.name = null;
		this.readonly = false;
		this.lastIndex = 0;
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		if(params) extendObject(this, params);
		if(Array.isArray(this.skills))
			this.skills = {};
		if(Array.isArray(this.allSkills))
			this.allSkills = {};
	}

	UI.SkillsEditor.prototype = {
		setup: function() {
			if(this.set_up) return;
			if(! this.elt)
				this.elt = $(this.id + '-body');
			if(! this.elt)
				return;
			if(! this.field) {
				if(this.name && this.form)
					this.field = this.form.elements[this.name];
				else if(this.field_id)
					this.field = $(this.field_id);
			}
			if(! isPlainObject(this.skills))
				this.skills = {};
			this.body = Wrap(this.elt).findFirst('.form-cell').ext(0);			
			this.renderRows();
			this.set_up = true;
		},
		
		orderedSkills: function() {
			var s = [];
			for(var k in this.skills) {
				s.push(this.skills[k]);
			}
			s.sort(cmpProperty('name', cmpStringFn));
			return s;
		},
		skillOptions: function() {
			var s = {keys: [''], values: [App.language.getString('LBL_OPTION_ADD_SKILL', 'Skills')]};
			for(var k in this.allSkills) {
				if(! (k in this.skills)) {
					s.keys.push(k);
					s.values.push(this.allSkills[k]);
				}
			}
			//s.sort(cmpProperty('name', cmpStringFn));
			return s;
		},
		
		renderRows: function() {
			if(! this.body || ! this.skills) return;
			this.body.empty();
			var vals = this.orderedSkills(),
				extra = this.readonly ? null : this.skillOptions(),
				input = this,
				row, cname, crating, panel, prog, perc, icon,
				outer_cls = 'div.panel-outer.panel-default.panel-round.panel-border';
			if(extra && extra.keys.length > 1)
				vals.push({add_skill: true});
			for(var i = 0; i < vals.length; i++) {
				if(! row || i % 3 == 0) row = MakeElt('div.row.form-row.col-unstack').appendTo(this.body);
				perc = Math.round(Math.max(0, Math.min(100, vals[i].rating)));
				cname = MakeElt('div.column.panel-pad-body.' + (this.readonly ? 'col-8' : 'col-6'));
				crating = MakeElt('div.column.col-4.panel-pad-body');
				if(vals[i].add_skill) {
					var sel = new UI.SelectInput(this.id + '-add',
						{extClassName: 'input-block', options: extra});
					this.select_input = sel;
					cname.append(sel.render());
				} else {
					cname.append(MakeElt('div.input-field-pad').text(vals[i].name));
				}
				if(vals[i].add_skill || ! this.readonly) {
					var rating_id = this.id + (vals[i].add_skill ? '-addrating' : '-rating-' + vals[i].skill_id);
					var rating = new UI.TextInput(rating_id, {
						format: 'int', min: 0, max: 100,
						placeholder: App.language.getString('LBL_PLACEHOLDER_RATING', 'Skills'),
						skill_id: vals[i].skill_id,
						init_value: perc});
					rating.onchange = function() { input.updateRating(this.skill_id, this.getValue()); }
					this.rating_input = rating;
					crating.append(rating.render());
				} else {
					if(perc) {
						prog = MakeElt('div.input-field.input-progress-outer.input-block.input-sm')
							.append(MakeElt('div.input-progress')
								.append(MakeElt('input-progress-bar.state-info').css('width', ''+perc+'%')));
						crating.append(MakeElt('div.input-field-pad').append(prog));
					}
				}
				if(! this.readonly) {
					if(vals[i].add_skill) {
						icon = MakeElt('span.uii.uii-lg.uii-add.active-icon')
							.on('click, keydown[key=enter|space]', function(evt) {
								this.addSkill();
							}, {stop: true, prevent: true, scope: this});
					} else {
						icon = MakeElt('span.uii.uii-lg.uii-remove.active-icon', {'data-skill-id': vals[i].skill_id})
							.on('click, keydown[key=enter|space]', function(evt) {
								this.removeSkill(evt.currentTarget.getAttribute('data-skill-id'));
							}, {stop: true, prevent: true, scope: this});
					}
					icon = MakeElt('div.column.col-2.panel-pad-body.text-right').append(icon);
				} else {
					icon = null;
				}
				panel = MakeElt(outer_cls + (this.readonly ? '.panel-readonly' : ''))
					.append(MakeElt('div.panel-body')
						.append(MakeElt('div.row').content([cname, crating, icon])));
				MakeElt('div.column.col-4.form-cell')
					.append(MakeElt('div.form-entry').append(panel)).appendTo(row);
			}
			if(! i)
				MakeElt('div.text-help', app_string('LBL_NONE')).appendTo(this.body);
		},

		addSkill: function(quiet) {
			if(! this.select_input || ! this.rating_input || this.readonly)
				return;
			var skill_id = this.select_input.getValue(),
				rating = this.rating_input.getValue(),
				src;
			if(! skill_id) {
				if(! quiet)
					alert(App.language.getString('ERR_NO_SKILL_SELECTED', 'Skills'));
			} else {
				src = this.allSkills[skill_id];
				if(src) {
					this.skills[skill_id] = {name: src, skill_id: skill_id, rating: rating};
					this.renderRows();
				}
			}
		},
		
		removeSkill: function(id) {
			if(this.readonly) return;
			delete this.skills[id];
			this.renderRows();
		},
		
		updateRating: function(skill_id, rating) {
			if(skill_id in this.skills)
				this.skills[skill_id].rating = rating;
		},

		beforeSubmitForm: function() {
			if(this.readonly) return;
			if (!this.field) {
				this.field = createElement2('input', {name: this.name, type: 'hidden'});
				this.form.appendChild(this.field);
			}
			this.addSkill(true);
			var values = [], skill;
			for(var k in this.skills) {
				skill = this.skills[k];
				values.push({
					id: skill.skill_id,
					rating: skill.rating
				});
			}
			this.field.value = JSON.stringify(values);
		},

		inheritFromAccount: function(inp) {
			var id = inp.getKey();
			if (!id)
				return;
			var callback = function(a) {
				this.skills = {};
				if(a.result && ! a.result.push)
				for (var i in a.result) {
					if(i in this.allSkills)
						this.skills[i] = {skill_id: i, name: this.allSkills[i], rating: a.result[i]};
				}
				this.renderRows();
			}.bind(this);
			var args = new SUGAR.conn.FormArgs(true);
			args.append('model_name', 'Account');
			args.append('parent_id', id);
			var result = call_json_method(
				'Skills',
				'get_skills',
				args.getData(),
				'skills_list',
				callback
			);
		}
	}
	
	window.SkillsEditor = {
		inherit_from_account: function(account_input) {
			var editor = App.ui.getFormInput(account_input.form, 'required_skills');
			if(editor)
				editor.inheritFromAccount(account_input);
		}
	};
});
