<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


require_once('data/SugarBean.php');
require_once('include/utils.php');


class SubContract extends SugarBean 
{
	// Stored fields
	var $id;
	var $name;
	var $contract_type_id;
	var $description;
	var $date_start;
	var $date_expire;
	var $date_billed;
	var $date_entered;
	var $date_modified;
	
	var $vendor_contract;
	var $assigned_user_id;
	var $customer_contact_id;
	
	
	var $is_active;
	var $status_colour;
	
	var $customer_contact_name;
	var $customer_contact_phone;
	var $contract_type_name;
	var $account_id;
	var $account_name;
	var $created_by_name;
	var $modified_user_name;
	
	
	//var $total_cost;
	//var $total_list;
	var $total_purchase;
	var $total_support_cost;
	//var $total_book;
	var $currency_symbol;
	
	// relationship fields
	var $asset_id;
	var $case_id;

	var $table_name = 'service_subcontracts';
	var $types_table_name = 'service_contracttypes';
	var $object_name = 'SubContract';
	var $module_dir = 'SubContracts';
	var $new_schema = true;
	
	// This is used to retrieve related fields from form posts.
	var $additional_column_fields = Array(
		'created_by_name',
		'modified_user_name',
		'account_name',
		'customer_contact_name',
		'customer_contact_phone',
		'contract_type_name',
	);
	
	var $relationship_fields = Array(
		'asset_id' => 'assets',
		'case_id' => 'cases',
	);

    static function convert_amount($value) {
        global $current_user;

        if( !isset($current_user->currency_for_contract)) {
            require_once('modules/Currencies/Currency.php');
            $currency = new Currency();
            $currency->retrieve($current_user->getPreference('currency'));
            $current_user->currency_for_contract = $currency;
        } else {
            $currency = $current_user->currency_for_contract;
        }

        $total_value = stripslashes( $value );
        $converted = $currency->convertFromDollar($total_value);

        return $converted;
    }

    static function calc_total_purchase($id, $format=false) {
        $cost = 0;

        if ($id) {
            global $db;

            $query = "SELECT SUM(purchase_usdollar * quantity) AS total_purchase
                FROM `assets`
                WHERE service_subcontract_id = '".$id."' AND NOT deleted";

            $result = $db->query($query);
            $data = $db->fetchByAssoc($result);

            if (isset($data['total_purchase']))
                $cost = SubContract::convert_amount($data['total_purchase']);
        }

        if($format)
        	$cost = format_number($cost);

        return $cost;
    }

    static function calc_total_support($id, $format = false) {
        $cost = 0;

        if ($id) {
            global $db;

            $query = "SELECT SUM(unit_support_usdollar * quantity) AS total_support_cost
                FROM `assets`
                WHERE service_subcontract_id = '".$id."' AND NOT deleted";

            $result = $db->query($query);
            $data = $db->fetchByAssoc($result);

            if (isset($data['total_support_cost']))
                $cost = SubContract::convert_amount($data['total_support_cost']);
        }

		if ($format)
	        $cost = format_number($cost);

        return $cost;
    }

    static function convert_balance($id, $format=false) {
        $balance = 0;

        if ($id) {
            $subcontract = ListQuery::quick_fetch_row('SubContract', $id, array('prepaid_balance_usd'));
            if ($subcontract != null)
                $balance = SubContract::convert_amount($subcontract['prepaid_balance_usd']);
        }

		if($format)
			$balance = format_number($balance);

        return $balance;
    }

    static function set_warranty_dates($id) {
        global $db;

        $query = "UPDATE `assets`, `service_subcontracts` SET `assets`.`warranty_start_date` = `service_subcontracts`.`date_start`,
            `assets`.`warranty_expiry_date` = `service_subcontracts`.`date_expire` WHERE
            `service_subcontracts`.`id` = '{$id}' AND `service_subcontracts`.`id` = `assets`.`service_subcontract_id`";

        $db->query($query, true, 'Error setting new warranty dates: ');
    }

    static function update_prepaid_balance($id) {
        global $db;
        if(! $id)
            return;

        $query = "SELECT SUM(CASE transaction_type WHEN 'credit' THEN amount_usdollar ELSE -amount_usdollar END) AS balance_usd
            FROM `prepaid_amounts` ".
            "WHERE subcontract_id = '".$id."' AND NOT deleted";

        $result = $db->query($query);

        if($row = $db->fetchByAssoc($result, -1, false)) {
            $prepaid_balance_usd = $row['balance_usd'] * 1;

            $update_query = "UPDATE `service_subcontracts` SET prepaid_balance_usd = '".$prepaid_balance_usd."'
                WHERE id = '".$id."'";

            $db->query($update_query);
        }
    }

    static function status_subselect($colspec, $table, $for_order=false) {
        return "CASE
            WHEN $table.is_active = 'no' THEN 'grey'
            WHEN $table.date_expire IS NULL OR $table.date_expire > DATE_ADD(CURDATE(),INTERVAL 30 DAY)
            THEN 'green' WHEN $table.date_expire > CURDATE() THEN 'yellow'
            ELSE 'red'
            END ";
    }

    static function show_service_popup(DetailManager $mgr) {
        require_bean('Account');
        Account::add_account_popup($mgr->getRecord(), 'account_id', 'service');
    }

    static function update_balance(RowUpdate $upd) {
        $id = $upd->getPrimaryKeyValue();
        SubContract::update_prepaid_balance($id);
    }
}


?>
