<?php

require_once 'data/SugarBean.php';


/*
 * Addon holds settings for SubscriptionPlan Addons
 * NOTE: several SugarBean functions can corrupt state.  If in doubt avoid parent methods.
 */
class Addon extends SugarBean
{
	// model meta:
	var $table_name = "addons";
	var $object_name = "Addon";
	var $new_schema = true;
	
	// system fields:
	var $id;					// LBL_ADDON_ID
	var $deleted;				
	var $date_entered;			
	var $date_modified;			
	var $name;					// LBL_ADDON_NAME

	// addon state:
	var $external_id;			// LBL_EXTERNAL_ID
	var $amount;				// LBL_AMOUNT
	var $recurring;				// LBL_RECURRING
	var $type;					// LBL_ADDON_TYPE
	var $global;				// LBL_GLOBAL
	var $active;				// LBL_ACTIVE

	// relationships
	var $payment_processor;		// LBL_PAYMENT_PROCESSOR

	
	
	// additional form fields
	var $additional_column_fields = Array(
		'payment_processor_id',
	);
	
	// key => model remationships
	var $relationship_fields = Array(
		'payment_processor_id'=>'payment_processor',
	);
	

	function get_summary_text()
	{
		return $this->name;
	}

    /**
     * @param \RowUpdate $update
     * @param $link_name
     * @throws IAHInternalError
     */
    static function after_add_link(RowUpdate &$update, $link_name)
    {
        $addon_id = ($update->getModelName()=="Addon")?$update->getField("id"):$update->link_update->input_unformat['addon_id'];
        $link_table_name=$update->link_update->model_name;
        switch($link_table_name){
            case 'addons_coupons':
                $lq = new ListQuery('Addon', null, array('link_name' => 'coupons'));
                $lq->addSimpleFilter('~parent.id',$addon_id);
                $coupons = $lq->runQuery()->getRows();
                $coupons = (array)array_column($coupons,'pp_coupon_id');
                $ru = new RowUpdate('Addon');
                $ru->retrieveRecord($addon_id);
                $ru->set('coupons',json_encode($coupons));
                $ru->save();
                break;
            case 'addons_options':
                $lq = new ListQuery('Addon', null, array('link_name' => 'options'));
                $lq->addSimpleFilter('~parent.id', $addon_id);
                $lq->setOrderBy("~join.date_modified ASC");
                $options = $lq->runQuery()->getRows();
                $options = (array)array_column($options, 'option_number');
                $ru = new RowUpdate('Addon');
                $ru->retrieveRecord($addon_id);
                $ru->set('options', json_encode($options));
                $ru->save();
                break;
            DEFAULT:
                $GLOBALS['log']->error(__CLASS__.'->'.__FUNCTION__.' unknown link name ' . $link_name);
        }
    }
    /**
     * @param \RowUpdate $update
     * @param $link_name
     * @throws IAHInternalError
     */
    static function after_remove_link(RowUpdate &$update, $link_name)
    {
        self::after_add_link($update, $link_name);
    }

	static function after_save(RowUpdate &$update) {
		global $db;
		$order = (int) ($update->getField('display_order'));
		$id = $update->getField('id');
		$db->query("SELECT @min_order = MIN(display_order) FROM addons WHERE display_order >= $order AND id != '$id'");
		$db->query("UPDATE addons SET display_order =  display_order - IFNULL(@min_order, 0) + 1 WHERE display_order >= $order AND id != '$id'");
	}

}
