<?php

require_once 'data/SugarBean.php';

/*
 * Subscription holds the state of a subscription and links to customer / account / plan / payment processor
 * NOTE: several SugarBean functions can corrupt state.  If in doubt avoid parent methods.
 */
class Plan extends SugarBean
{
	// model meta:
	var $table_name = "plans";
	var $object_name = "Plan";
	var $new_schema = true;
	
	// system fields:
	var $id;					// LBL_PLAN_ID
	var $deleted;				
	var $date_entered;			
	var $date_modified;			
	var $name;

	// plan state:
	var $interval;				// LBL_INTERVAL
	var $amount;				// LBL_AMOUNT
    var $setup;                 // LBL_SETUP_FEE
    var $pricing_model;         // LBL_PRICING_MODEL
    var $pricing_tiers;         // LBL_PRICING_TIERS
	var $addons;				// LBL_ADDONS
	var $coupons;				// LBL_COUPONS
	var $options;				// LBL_PLAN_OPTIONS

	// relationships
	var $payment_processor;		// LBL_PAYMENT_PROCESSOR
	var $product;				// LBL_PRODUCT

	
	
	// additional form fields  - only assigned user_id should be editable out of these:
	// FIXME: just getting started...
	var $additional_column_fields = Array(
		'payment_processor_id', 'product_id',
	);
	
	// key => model remationships
	// FIXME: need plan_id, option_id, coupon_id, invoice_id, maybe more.
	var $relationship_fields = Array(
		'payment_processor_id'=>'payment_processor','product_id'=>'product',
	);


	function get_summary_text()
	{
		return $this->name;
	}

	function get_options() {
		return $this->get_linked_beans('options','Option');
	}

    /**
     * @param \RowUpdate $update
     * @param $link_name
     * @throws IAHInternalError
     */
    static function after_add_link(RowUpdate &$update, $link_name)
    {
        global $log;
        $plan_id = ($update->getModelName()=="Plan")?$update->getField("id"):$update->link_update->input_unformat['plan_id'];
        $link_table_name=$update->link_update->model_name;
        switch($link_table_name){
            case 'plans_addons':
                $lq = new ListQuery('Plan', null, array('link_name' => 'addons'));
                $lq->addSimpleFilter('~parent.id',$plan_id);
                $addons = $lq->runQuery()->getRows();
                $addons = (array)array_column($addons,'pp_addon_id');
                $ru = new RowUpdate('Plan');
                $ru->retrieveRecord($plan_id);
                $ru->set('addons',json_encode($addons));
                $ru->save();
                break;
            case 'plans_coupons':
                $lq = new ListQuery('Plan', null, array('link_name' => 'coupons'));
                $lq->addSimpleFilter('~parent.id',$plan_id);
                $coupons = $lq->runQuery()->getRows();
                $coupons = (array)array_column($coupons,'pp_coupon_id');
                $ru = new RowUpdate('Plan');
                $ru->retrieveRecord($plan_id);
                $ru->set('coupons',json_encode($coupons));
                $ru->save();
                break;
            case 'plans_options':
                $lq = new ListQuery('Plan', null, array('link_name' => 'options'));
                $lq->addSimpleFilter('~parent.id', $plan_id);
                $lq->setOrderBy("~join.date_modified ASC");
                $options = $lq->runQuery()->getRows();
                $options = (array)array_column($options, 'option_number');
                $ru = new RowUpdate('Plan');
                $ru->retrieveRecord($plan_id);
                $ru->set('options', json_encode($options));
                $ru->save();
                break;
            DEFAULT:
                $GLOBALS['log']->error(__CLASS__.'->'.__FUNCTION__.' unknown link table name ' . $link_name);

        }
    }

    /**
     * @param \RowUpdate $update
     * @param $link_name
     * @throws IAHInternalError
     */
    static function after_remove_link(RowUpdate &$update, $link_name)
    {
        self::after_add_link($update, $link_name);
    }

	
	static function after_save(RowUpdate &$update) {
		global $db;
		$order = (int) ($update->getField('display_order'));
		$id = $update->getField('id');
		$pid = $update->getField('product_id');
		$db->query("SELECT @min_order = MIN(display_order) FROM plans WHERE display_order >= $order AND product_id = '$pid' AND id != '$id'");
		$db->query("UPDATE plans SET display_order =  display_order - IFNULL(@min_order, 0) + 1 WHERE display_order >= $order AND product_id = '$pid' AND id != '$id'");
	}
}
