<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormTableSection.php');

class SubscriptionTiersWidget extends FormTableSection {

	function init($params=null, $model=null) {
		parent::init($params, $model);
	}
	
	function getRequiredFields() {
		$fields = ['pricing_tiers', 'pricing_tiers_desc'];
		$this->required_fields = $fields;
		return $fields;
	}
	
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		return $this->renderHtmlView($gen, $row_result, $parents, $context);
    }
	
	function renderHtmlView(HtmlFormGenerator $gen, RowResult $row_result, array $parents, array $context) {
        global $current_user;
		$json = getJSONobj();
		$tiers = $row_result->getField('pricing_tiers');
		if (!empty($tiers)) {
			$tiers = $json->decode($tiers);
		}
		if (empty($tiers)) return '';
		$tiers_desc = $row_result->getField('pricing_tiers_desc');
		$descriptions = preg_split('~\r?\n~', $tiers_desc);
		if (is_countable($descriptions) && count($descriptions) == 1 && $descriptions[0] === '') {
			$descriptions = [];
		}
		$rows = '';
		$rr = new RowResult('Plan');
		$idx = -1;
		foreach ($tiers as $qty => $price) {
			$idx++;
			$price = currency_format_number($price);
			$description = array_get_default($descriptions, $idx);
			if($gen->getLayout()->getDefaultEditable()) {
				$e = [
					'type' => 'varchar',
					'name' => 'tier_name_' . $idx,
					'width' => 15,
					'required' => false,
				];
				$rr->assign($e['name'], $description);
				$input = $gen->getFormObject()->renderField($rr, $e);
			} else {
				$input = to_html($description);
			}

			$rows .= <<<HTML
				<div class="col-space-sm row">
					<div class="column">$qty+</div>
					<div class="column">$price</div>
					<div class="column">$input</div>
				</div>
HTML;
		}
		$lbl_qty = translate('LBL_TIER_QTY', 'SubscriptionPlans');
		$lbl_price = translate('LBL_TIER_PRICE', 'SubscriptionPlans');
		$lbl_description = translate('LBL_TIER_DESCRIPTION', 'SubscriptionPlans');
		$body = <<<HTML
			<div class="row form-row"><div class="column form-cell expand">
				<div class="form-entry">
					<section class="card-outer panel-outer panel-default timesheet-items-outer panel-border panel-round panel-readonly">
						<div class="card-header panel-subheader">
							<div class="row col-space-sm panel-column-titles">
								<div class="column"><span class="column-title">{$lbl_qty}</span></div>
								<div class="column"><span class="column-title">{$lbl_price}</span></div>
								<div class="column"><span class="column-title">{$lbl_description}</span></div>
							</div>
						</div>
						<div class="card-body panel-body panel-pad-body ">
						{$rows}
						</div>
					</section>
				</div>
			</div></div>
HTML;




		$params = ['title' => translate('LBL_PRICING_TIERS', 'SubscriptionPlans'), 'body' => $body];
		return $this->renderOuterTable($gen, $context, $parents, $params);
	}
	
	function loadUpdateRequest(RowUpdate $update, array $input) {
		if (array_get_default($input, 'record_perform') !== 'save') return;
		$tiers = [];
		foreach ($input as $k => $v) {
			if (strpos($k, 'tier_name_') === 0) {
				$tiers[substr($k, 10)] = $v;
			}
		}
		ksort($tiers);
		$update->set('pricing_tiers_desc', join("\n", $tiers));
	}

}

