<?php

namespace OneCRM\API\Subscription;

use OneCRM\API\Endpoint;
use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use OneCRM\API\Errors;

class AddCard extends Endpoint {

    public function endpoint() {
        return '/subscription/add_card';
    }

	public function allowedMethods() {
        return ['post'];
    }

    public function requiredScopes($method) {
        return ['read', 'write'];
    }

    public function descriptor($method)
    {
        switch ($method) {
            case 'post':
                return $this->descriptorPost();
                break;
        }
        return false;
    }

	public function descriptorPost() {
		static $desc = [
			'hidden' => true,
            'title' => 'Add payment method',
            'docs' => 'Requests an URL that can be used to add a new payment method for a customer',
            'params' => [
                'customer_id' => [
                    'type' => 'String',
                    'location' => 'body',
                    'docs' => 'Customer ID',
                ],
                'return_url' => [
                    'type' => 'String',
                    'location' => 'body',
                    'docs' => 'URL to redirect o after a payment method was added',
                ],
            ],
			'return' => [
				'url' => [
					'type' => 'String',
					'docs' => 'URL to redirect the customer to',
				]
            ],
        ];
        return $desc;
	}

	protected function post($input, Response $res, Request $req) {
		require_once 'modules/PaymentProcessors/PaymentProcessor.php';
		$pp = \PaymentProcessor::getDefault();
		if (!$pp)
           	throw new Errors\InternalError("Payment processor not found");
		try {
			$result = $pp->getPaymentMethodsURL($input['customer_id'], $input['return_url']);
		} catch (\Exception $e) {
			$result = ['error' => $e->getMessage()];
		}

		$res->getBody()->write(json_encode($result));
		return $res->withHeader('Content-Type', 'application/json');
	}

}

