<?php

namespace OneCRM\API\Subscription;

use OneCRM\API\Endpoint;
use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use OneCRM\API\Errors;

class CancelSubscription extends Endpoint {

    public function endpoint() {
        return '/subscription/cancel';
    }

	public function allowedMethods() {
        return ['post'];
    }

    public function requiredScopes($method) {
        return ['read', 'write'];
    }

    public function descriptor($method)
    {
        switch ($method) {
            case 'post':
                return $this->descriptorPost();
                break;
        }
        return false;
    }

	public function descriptorPost() {
        static $desc = [
			'hidden' => true,
            'title' => 'Cancel a subscription',
            'docs' => 'Cancels a subscription',
            'params' => [
				'subscription_id' => [
					'type' => 'String',
					'location' => 'body',
					'required' => true,
				],
				'credits' => [
					'type' => 'Enum',
					'options' => ['none', 'prorate', 'full'],
					'default' => 'prorate',
					'location' => 'body',
				],
				'end_of_term' => [
					'type' => 'Bool',
					'location' => 'body',
				],
            ],
            'return' => [
            ],
        ];
        return $desc;
	}

	protected function post($input, Response $res, Request $req) {
		require_once 'modules/PaymentProcessors/PaymentProcessor.php';
		$pp = \PaymentProcessor::getDefault();
		if (!$pp)
           	throw new Errors\InternalError("Payment processor not found");
		$subscription_id = array_get_default($input, 'subscription_id');
		$params = [
			'credits' => $input['credits'],
			'end_of_term' => !empty($input['end_of_term']),
		];
		$subs = \ListQuery::quick_fetch('Subscription', $subscription_id);
		if (!$subs) throw new Errors\NotFound();
		try {
			$result = $pp->cancelSubscription($subs->getField('pp_subs_id'), $params);
		} catch (\Exception $e) {
			$result = ['error' => $e->getMessage()];
		}

		$res->getBody()->write(json_encode($result));
		return $res->withHeader('Content-Type', 'application/json');
	}

}

