<?php

namespace OneCRM\API\Subscription;

use OneCRM\API\Endpoint;
use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use OneCRM\API\Errors;

class CreateCustomer  extends Endpoint {

    public function endpoint() {
        return '/subscription/create_customer';
    }

	public function allowedMethods() {
        return ['post'];
    }

    public function requiredScopes($method) {
        return ['read', 'write'];
    }

    public function descriptor($method)
    {
        switch ($method) {
            case 'post':
                return $this->descriptorPost();
                break;
        }
        return false;
    }

	public function descriptorPost() {
        static $desc = [
			'hidden' => true,
            'title' => 'Create a customer',
            'docs' => 'Creates a new customer',
            'params' => [
                'contact_id' => [
                    'type' => 'String',
                    'location' => 'body',
                    'docs' => 'Customer\'s contact ID.',
                ],
                'account_id' => [
                    'type' => 'String',
                    'location' => 'body',
                    'docs' => 'Customer\'s account ID.',
                ],
                'customer' => [
					'type' => 'Object',
                    'location' => 'body',
                    'docs' => 'Customer\'s data',
                ],
            ],
            'return' => [
            ],
        ];
        return $desc;
	}

	protected function post($input, Response $res, Request $req) {
		require_once 'modules/PaymentProcessors/PaymentProcessor.php';
		$pp = \PaymentProcessor::getDefault();
		if (!$pp)
			throw new Errors\InternalError("Payment processor not found");
		$customer_id = null;
		try {
			$account_id = array_get_default($input, 'account_id');
			$contact_id = array_get_default($input, 'contact_id');
			$data = [
				'name' => array_get_default($input['customer'], 'account_name'),
				'first_name' => array_get_default($input['customer'], 'first_name'),
				'last_name' => array_get_default($input['customer'], 'last_name'),
				'email1' => array_get_default($input['customer'], 'email'),
				'id' => array_get_default($input, 'account_id'),
				'primary_contact_id' => array_get_default($input, 'contact_id'),
			];
			if ($result = $pp->sync_record('Customer', $data)){
				$customer_id = array_get_default($result, 'id');
			}
			if ($customer_id && $account_id) {
				$account = \ListQuery::quick_fetch('Account', $account_id);
				if ($account) {
					$upd = \RowUpdate::for_result($account);
					$upd->set('pp_customer_id', $customer_id);
					$upd->save();
				}
			}
			$result = ['id' => $customer_id];
		} catch (\Exception $e) {
			$result = ['error' => $e->getMessage()];
		}

		$res->getBody()->write(json_encode($result));
		return $res->withHeader('Content-Type', 'application/json');
	}

}

