<?php

namespace OneCRM\API\Subscription;

use OneCRM\API\Endpoint;
use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;
use OneCRM\API\Errors;

class SubscriptionDetails  extends Endpoint {

    public function endpoint() {
        return '/subscription/details/{id}';
    }

	public function allowedMethods() {
        return ['get'];
    }

    public function requiredScopes($method) {
        return ['read'];
    }

    public function descriptor($method)
    {
        switch ($method) {
            case 'get':
                return $this->descriptorGet();
                break;
        }
        return false;
    }

	public function descriptorGet() {
        static $desc = [
			'hidden' => true,
            'title' => 'Subscription details',
            'docs' => 'Fetches subscription detailed info',
            'params' => [
                'id' => [
                    'type' => 'String',
                    'location' => 'path',
					'docs' => 'Subscription ID.',
					'required' => true,
                ],
            ],
            'return' => [
            ],
        ];
        return $desc;
	}

	protected function get($input, Response $res, Request $req) {
		$json = getJSONobj();
		require_once 'modules/PaymentProcessors/PaymentProcessor.php';
		$pp = \PaymentProcessor::getDefault();
		if (!$pp)
           	throw new Errors\InternalError("Payment processor not found");
		$subsc = \ListQuery::quick_fetch_row('Subscription', $input['id']);
		if (!$subsc) {
			throw new Errors\NotFound;
		}
		$subsc['addons'] = $pp->convertAddons($subsc['addons']);
		$subsc['options'] = $json->decode($subsc['options']) ?? [];
		$native = $json->decode($subsc['native']) ?? [];
		if (isset($native['invoice_notes'])) {
			$subsc['invoice_notes'] = $native['invoice_notes'];
		}

		$res->getBody()->write(json_encode($subsc));
		return $res->withHeader('Content-Type', 'application/json');
	}

}

