<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormTableSection.php');

class SubscriptionOptionsWidget extends FormTableSection {

	function getRequiredFields() {
		return ['options', 'plan_id'];
	}
	
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		return $this->renderHtmlView($gen, $row_result, $parents, $context);
    }
	
	function renderHtmlView(HtmlFormGenerator $gen, RowResult $row_result, array $parents, array $context) {
        global $current_user;
		$json = getJSONobj();
		$options = $row_result->getField('options');
		if (!empty($options)) {
			$options = $json->decode($options);
		}
		if (empty($options)) return '';
		$option_defs = ListQuery::quick_fetch_linked('Plan', $row_result->getField('plan_id'), 'options', ['name', 'input'], null, true);
		$rows = '';
		foreach ($options as $input => $value) {
			$opt_name = $input;
			foreach ($option_defs as $od) {
				if ($od['input'] == $input) {
					$opt_name = $od['name'];
					break;
				}
			}
			$opt_name = to_html($opt_name);
			$value = to_html($value);
    	    $rows .= <<<HTML
				<div class="col-space-sm row">
					<div class="column">$opt_name</div>
					<div class="column">$value</div>
				</div>
HTML;
		}
		$lbl_name = translate('LBL_OPTION_NAME', 'Subscriptions');
		$lbl_qty = translate('LBL_OPTION_VALUE', 'Subscriptions');
		$body = <<<HTML
			<div class="row form-row"><div class="column form-cell expand">
				<div class="form-entry">
					<section class="card-outer panel-outer panel-default timesheet-items-outer panel-border panel-round panel-readonly">
						<div class="card-header panel-subheader">
							<div class="row col-space-sm panel-column-titles">
								<div class="column"><span class="column-title">{$lbl_name}</span></div>
								<div class="column"><span class="column-title">{$lbl_qty}</span></div>
							</div>
						</div>
						<div class="card-body panel-body panel-pad-body ">
						{$rows}
						</div>
					</section>
				</div>
			</div></div>
HTML;

		$params = ['title' => translate('LBL_OTHER_OPTIONS', 'Subscriptions'), 'body' => $body];
        return $this->renderOuterTable($gen, $context, $parents, $params);
	}
}

