<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


require_once('data/SugarBean.php');

class SupportedAssembly extends SugarBean
{
	var $id;
	var $date_entered;
	var $created_by;
	var $date_modified;
	var $modified_user_id;
	var $deleted;
	var $name;
	var $product_category_id;
	var $product_type_id;
	var $product_url;
	var $manufacturers_part_no;
	var $vendor_part_no;
	var $account_id;
	var $account_name;
	var $service_subcontract_id;
	var $quantity = 1;
	var $supplier_id;
	var $manufacturer_id;
	var $model_id;
    
	// not stored
	var $product_category_name;
	var $product_type_name;
	var $subcontract_name;
	var $supplier_name;
	var $manufacturer_name;
	var $model_name;
	var $created_by_name;
	var $modified_user_name;

	
	var $table_name = 'supported_assemblies';
	var $object_name = 'SupportedAssembly';
	var $module_dir = 'SupportedAssemblies';
	
	var $new_schema = true;
	
	
	function set_assets_subcontracts() {
		$query = "UPDATE assets SET service_subcontract_id='$this->service_subcontract_id' WHERE supported_assembly_id='$this->id' AND NOT deleted";
		$this->db->query($query, true, "Error reassigning assets to service subcontract");
	}
	

	function get_search_categories()
	{
		require_once 'modules/ProductCategories/ProductCategory.php';
		return array('' => '') + get_product_categories_list(true);
	}

	function get_search_types($param, $searchFields)
	{
		require_once 'modules/ProductTypes/ProductType.php';
		$pType = new ProductType;
		$arPT = $pType->get_for_category(@$searchFields['product_category_id']['value'], true);
		$pType->cleanup();
		return array('' => '') + $arPT;
	}

	static function get_products_list($id, $fields=null)
	{
		$lq = new ListQuery('SupportedAssembly', null, array('link_name' => 'assets', 'parent_key' => $id, 'use_key_index' => false, 'acl_filters' => 'view'));
		$lq->addPrimaryKey('id');
		if(is_array($fields))
			$lq->addFields($fields);
		if(! $id) $lq->addFilterClause('0'); // require an ID
		$lq->setFormatted('json');
		$result = $lq->runQuery();
		foreach($result->formatted_rows as &$row)
			$row['related_type'] = 'Assets';
		return array(
			'fields' => $result->formatted_rows,
		);
	}

    static function init_record(RowUpdate &$upd, $input) {
        if ( ($subc = $upd->getField('service_subcontract_id')) ) {
            $subcontract = ListQuery::quick_fetch_row('SubContract', $subc, array('account_id'));
            if (is_array($subcontract) && isset($subcontract['account_id']))
                $upd->set('account_id', $subcontract['account_id']);
        }
    }

    static function init_form(DetailManager $mgr) {
        global $pageInstance;
        $pageInstance->add_js_literal("setOnSubmitEvent();", null, LOAD_PRIORITY_FOOT);
        $account_id = $mgr->record->getField('account_id');
        if ($account_id) {
            $pageInstance->add_js_literal("init_form('".$mgr->form_gen->form_obj->form_name."', '".$account_id."');", null, LOAD_PRIORITY_FOOT);
        }
        $pageInstance->add_js_literal("set_asm_name_extra('".$mgr->form_gen->form_obj->form_name."');", null, LOAD_PRIORITY_FOOT);
    }

    static function set_updated_name(RowUpdate &$upd) {
        $upd->set('name', array_get_default($_REQUEST, 'assembly_name', $upd->getField('name')));
    }

    static function after_save(RowUpdate &$upd) {
    	self::set_subcontracts($upd);
    	
    	$supp = $upd->getField('supplier_id');
    	$o_supp = $upd->getField('supplier_id', null, true);
    	if($supp && $supp != $o_supp) {
    		$upd->addUpdateLink('suppliers', $supp);
    	}
    }

    static function set_subcontracts(RowUpdate &$update) {
        if (isset($update->updates['service_subcontract_id'])) {
            global $db;
            $service_contract_id = $update->getField('service_subcontract_id');
            $query = "UPDATE assets SET service_subcontract_id='".$service_contract_id."' WHERE supported_assembly_id='".$update->getField('id')."' AND NOT deleted";
            $db->query($query, true, "Error reassigning assets to service subcontract");
        }
    }

	static function unified_search_hook(&$filter_form, &$lq, &$q, $value) {
		$lq->addLink('serial_numbers');
		$q[] = array(
			'field' => 'serial_numbers.serial_no',
			'value' => $value,
		);
	}

	static function get_all_serials($spec) {
		global $db;
		$serials = array();
		$query = "SELECT * FROM serial_numbers WHERE assembly_id='" . $db->quote($spec['raw_values']['id']) . "' AND deleted = 0";
		$res = $db->query($query, true);
		while ($row = $db->fetchByAssoc($res)) {
			$serials[] = $row['serial_no'];
		}
		return join(", ", $serials);
	}
}

?>
