<?php

require_once('data/SugarBean.php');

class SystemMessage extends SugarBean {
	var $module_dir = 'SystemMessages';
	var $object_name = 'SystemMessage';
	
	// process_list_result hook
	static function load_standard_text(&$result, &$row) {
		if(! empty($row['std_filename']) && (array_key_exists('body', $row) || array_key_exists('name', $row))) {
			$path = $row['std_filename'];
			if(! preg_match('~^modules/.*\.html$~', $row['std_filename']))
				return;
			$lang = AppConfig::language();
			$local = preg_replace('~.html$~', ".$lang.html", $path);
			if(is_readable($local))
				$content = file_get_contents($local);
			else if(is_readable($path))
				$content = file_get_contents($path);
			else
				return;
			if(array_key_exists('name', $row) && preg_match('~<title>(.*?)</title>$~ium', $content, $m))
				$row['name'] = $m[1];
			if(array_key_exists('body', $row)) {
				$content = preg_replace('~</?(body|html|head)\b[^>]*>~ium', '', $content);
				$content = preg_replace('~<title>(.*?)</title>~ium', '', $content);
				$row['body'] = $content;
			}
		}
	}
	
	// after_login hook
	static function check_system_messages($uid, $ltype) {
		if($ltype == 'sugar' && $uid) {
			require_once('modules/SystemMessages/utils.php');
			$result = find_pending_messages($uid, array('id'));
			if($result && ! $result->failed) {
				$_SESSION['pending_system_messages'] = array_keys($result->rows);
			}
		}
	}

	// after_page_render hook
	static function show_system_messages($pageInstance) {
		// this can be called more than once per request, so add a guard
		static $called = false;
		if ($called)
			return;
		if(($pageInstance->module == 'Users' && $pageInstance->action == 'InitUser'))
			return; // defer until user has saved their initial settings
		if($pageInstance->module == 'LicenseInfo' && empty($_REQUEST['updated']))
			return;
		if(AppConfig::current_user_id() && ! empty($_SESSION['pending_system_messages']) && ! $pageInstance->get_async()) {
			$json = getJSONobj();
			$rows = $json->encode($_SESSION['pending_system_messages']);
			$pageInstance->add_js_literal("SUGAR.onDomReady(function() { SUGAR.ui.SystemMessage.showList($rows); });", null, LOAD_PRIORITY_END);
		}
		$called = true;
	}
	
	// view hook
	static function mark_seen_on_view(DetailManager $mgr) {
		$message_id = $mgr->record->getPrimaryKeyValue();
		require_once('modules/SystemMessages/utils.php');
		mark_system_message_seen($message_id);
	}
}

?>
