<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');

function register_standard_messages() {
	require_once('include/config/format/ConfigParser.php');
	$reg = ConfigParser::load_file('modules/SystemMessages/release_notes/standard.php');
	if($reg) {
		$q = new ListQuery('SystemMessage', true, array('filter_deleted' => false));
		$rows = $q->fetchAllRows();
		$messages = array();
		foreach($rows as $row)
			$messages[$row['std_filename']] = $row;

		foreach($reg as $fname => $info) {
			$path = 'modules/SystemMessages/release_notes/' . $fname;
			if(! file_exists($path)) continue;
			$display = array_get_default($info, 'auto_display', true) ? 1 : 0;
			if(isset($messages[$path])) {
				$m = $messages[$path];
				if($m['auto_display'] != $display || $m['name'] != $info['title']) {
					$msg = RowUpdate::for_model('SystemMessage');
					$msg->setOriginal($m);
					$msg->set('name', $info['title']);
					$msg->set('auto_display', $display);
					$body = file_get_contents($path);
					$msg->set('body', $body);
					$msg->validate() && $msg->save();
				}
			}
			else {
				$msg = RowUpdate::blank_for_model('SystemMessage');
				$msg->set('std_filename', $path);
				$msg->set('auto_display', $display);
				$msg->set('date_entered', $info['date']);
				$msg->set('name', $info['title']);
				$msg->set('assigned_user_id', AppConfig::current_user_id());
				$msg->set('shared_with', 'all');
				$body = file_get_contents($path);
				$msg->set('body', $body);
				$msg->validate() && $msg->save();
			}
		}
	}
}

function mark_system_message_seen($message_id, $user_id=null, $seen=1) {
	if(! $user_id) $user_id = AppConfig::current_user_id();
	if(! $user_id || ! $message_id) return false;
	
	$upd = RowUpdate::blank_for_model('messages_seen');
	$upd->set(array(
		'user_id' => $user_id,
		'message_id' => $message_id,
		'seen' => $seen,
	));
	$ret = $upd->putRow();
	
	if($user_id == AppConfig::current_user_id() && ! empty($_SESSION['pending_system_messages'])) {
		$_SESSION['pending_system_messages'] = array_values(array_diff($_SESSION['pending_system_messages'], array($message_id)));
	}
	return $ret;
}

function find_pending_messages($user_id=null, $fields=null, $order_by='date_entered') {
	global $db;
	if(! $fields) $fields = true;
	$lq = new ListQuery('SystemMessage', $fields);
	$tbl = $db->quoteField($lq->getTableAlias(null, true));
	if(! $user_id) $user_id = AppConfig::current_user_id();
	$uid = $db->quote($user_id);
	$lq->addFilterClause("NOT COALESCE((SELECT ms.seen FROM messages_seen ms WHERE ms.message_id=$tbl.id AND ms.user_id='$uid' AND NOT ms.deleted), 0)");
	$lq->addSimpleFilter("auto_display", 1);
	if($order_by) $lq->setOrderBy($order_by);
	return $lq->fetchAll();
}


