<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

require_once('modules/Workflow/WFProcessorBase.php');

class CreateTaskProc extends WFProcessorBase {
	var $related_model = 'Task';

	function initRelatedRecord(RowUpdate $rowUpdate) {
		$task = parent::initRelatedRecord($rowUpdate);
		$module = $rowUpdate->getModuleDir();
		$rowId = $rowUpdate->getPrimaryKeyValue();
		if ($this->getParam('include_contact') && $module == 'Contacts') {
			$task->set('contact_id', $rowId);
		}
		$task->set(array(
			'parent_type' => $module,
			'parent_id' => $rowId,
			'date_start' => $this->getRelativeDate('date_start'),
			'date_due' => $this->getRelativeDate('date_due'),
		));
		$this->setValuesWithAlt($rowUpdate, $task, ['account', 'contact'], $this->getParams());
		return $task;
	}

	function getConfigFields() {
		$fields = parent::getConfigFields();

		$model = new ModelDef('Task');

		$fields['name'] = $model->getFieldDefinition('name');
		$fields['assign_to_parent_user'] = array(
			'type' => 'bool',
			'vname' => 'LBL_ASSIGN_PARENT_USER',
		);
		$fields['name']['width'] = 80;
		$fields['date_start'] = array(
			'vname' => 'LBL_START_DATE',
			'vname_module' => 'Tasks',
			'widget' => 'RelativeTimeInput',
			'check_hide' => 'date_start_type',
		);
		$fields['date_start_type']['hidden'] = true;
		$fields['date_start_count'] = array(
			'type' => 'int',
			'hidden' => true,
		);
		$fields['date_start_time'] = array(
			'type' => 'time',
			'hidden' => true,
		);
		$fields['date_due'] = array(
			'vname' => 'LBL_DUE_DATE',
			'vname_module' => 'Tasks',
			'widget' => 'RelativeTimeInput',
			'check_hide' => 'date_due_type',
		);
		$fields['date_due_type']['hidden'] = true;
		$fields['date_due_count'] = array(
			'type' => 'int',
			'hidden' => true,
		);
		$fields['date_due_time'] = array(
			'type' => 'time',
			'hidden' => true,
		);
		$fields['effort_estim'] = $model->getFieldDefinition('effort_estim');
		$fields['effort_estim']['default'] = 60;
		$fields['status'] = $model->getFieldDefinition('status');
		$fields['priority'] = $model->getFieldDefinition('priority');

		$fields['contact'] = $model->getFieldDefinition('contact');
		$fields['contact_id'] = array('type' => 'ref_id', 'hidden' => true);
		$fields['contact']['alt_options'] = $this->getRelatedRefFields('Contact');
		$fields['contact_alt'] = array('type' => 'varchar', 'hidden' => true);

		$fields['account'] = $model->getFieldDefinition('account');
		$fields['account_id'] = array('type' => 'ref_id', 'hidden' => true);
		$fields['account']['alt_options'] = $this->getRelatedRefFields('Account');
		$fields['account_alt'] = array('type' => 'varchar', 'hidden' => true);


		if($this->module == 'Contacts')
			$fields['include_contact'] = array(
				'vname' => 'LBL_RELATE_TO_RECORD',
				'type' => 'bool',
			);
		$fields['description'] = $model->getFieldDefinition('description');

		return $fields;
	}

}

