<?php

require_once 'modules/Quotes/PDFTemplatesProvider.php';

class TimesheetPDFTemplateProvider extends QuotePDFTemplateProvider
{
	var $cols_data;

	protected function __construct($type)
	{
		parent::__construct($type);
	}

	public static function template_types()
	{
		return array(
			'timesheet' => translate('LBL_PDF_TIMESHEET', 'Timesheets'),
		);
	}

	public static function template_skeleton($type)
	{
		$ret = array('sections' => array(
			array(
				'name' => 'header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_HEADER',
				'height' => 150,
				'elements' => array (),
			),
			array(
				'name' => 'items',
				'type' => 'table',
				'title' => 'LBL_SKEL_ITEMS',
				'columns' => array (),
			),
			array(
				'name' => 'footer',
				'data_id' => 'header',
				'type' => 'freeform',
				'title' => 'LBL_SKEL_FOOTER',
				'height' => 40,
				'elements' => array (),
			),
		));
		foreach ($ret['sections'] as &$sec) $sec['title'] = translate($sec['title'], 'Timesheets');
		$ret['sections'] = array_values($ret['sections']);
		return $ret;
	}

	public static function template_sample_data($type)
	{
		$datefmt = AppConfig::setting('locale.defaults.date_format');
		$data = array(
			'header' => array(
				'todays_date' => array(translate('LBL_TODAYS_DATE'), date($datefmt), ),
				'employee' => array(translate('LBL_SAMPLE_HEADER_EMPLOYEE_LABEL', 'Timesheets'), translate('LBL_SAMPLE_HEADER_EMPLOYEE_VALUE', 'Timesheets'), ),
				'employee_number' => array(translate('LBL_SAMPLE_HEADER_EMPLOYEE_NUMBER_LABEL', 'Timesheets'), '777'),
				'title' => array(translate('LBL_SAMPLE_HEADER_TITLE_LABEL', 'Timesheets'), translate('LBL_SAMPLE_HEADER_TITLE_VALUE', 'Timesheets'), ),
				'department' => array(translate('LBL_SAMPLE_HEADER_DEPARTMENT_LABEL', 'Timesheets'), translate('LBL_SAMPLE_HEADER_DEPARTMENT_VALUE', 'Timesheets'), ),
				'reports_to' => array(translate('LBL_SAMPLE_HEADER_REPORTS_TO_LABEL', 'Timesheets'), translate('LBL_SAMPLE_HEADER_REPORTS_TO_VALUE', 'Timesheets'), ),
				'period_start' => array(translate('LBL_SAMPLE_HEADER_PERIOD_START_LABEL', 'Timesheets'), '2014-11-01'),
				'period_end' => array(translate('LBL_SAMPLE_HEADER_PERIOD_END_LABEL', 'Timesheets'), '2014-11-06'),
				'date_modified' => array(translate('LBL_SAMPLE_HEADER_DATE_MODIFIED_LABEL', 'Timesheets'), '2014-11-05'),
				'total_hours' => array(translate('LBL_SAMPLE_HEADER_TOTAL_HOURS_LABEL', 'Timesheets'), 10),
				'approval_status' => array(translate('LBL_SAMPLE_HEADER_APPROVAL_STATUS_LABEL', 'Timesheets'), translate('LBL_SAMPLE_HEADER_APPROVAL_STATUS_VALUE', 'Timesheets'), ),
				'name' => array(translate('LBL_SAMPLE_HEADER_NAME_LABEL', 'Timesheets'), translate('LBL_SAMPLE_HEADER_NAME_VALUE', 'Timesheets'), ),
				'description' => array(translate('LBL_SAMPLE_HEADER_DESCRIPTION_LABEL', 'Timesheets'), translate('LBL_SAMPLE_HEADER_DESCRIPTION_VALUE', 'Timesheets'), ),
			),
			'items' => array(
				'time' => array(translate('LBL_SAMPLE_ITEMS_TIME_LABEL', 'Timesheets'), translate('LBL_SAMPLE_ITEMS_TIME_VALUE', 'Timesheets'), ),
				'subject' => array(translate('LBL_SAMPLE_ITEMS_SUBJECT_LABEL', 'Timesheets'), translate('LBL_SAMPLE_ITEMS_SUBJECT_VALUE', 'Timesheets'), ),
				'account' => array(translate('LBL_SAMPLE_ITEMS_ACCOUNT_LABEL', 'Timesheets'), translate('LBL_SAMPLE_ITEMS_ACCOUNT_VALUE', 'Timesheets'), ),
				'related_to' => array(translate('LBL_SAMPLE_ITEMS_RELATED_TO_LABEL', 'Timesheets'), translate('LBL_SAMPLE_ITEMS_RELATED_TO_VALUE', 'Timesheets'), ),
			),
		);
		$ret = array();
		foreach ($data as $k => $v) {
			$ret[$k] = array();
			foreach ($v as $kk => $vv) {
				$ret[$k][$kk] = array('label' => $vv[0], 'value' => $vv[1]);
			}
		}
		return $ret;
	}


	public static function create($type)
	{
		return new TimesheetPDFTemplateProvider($type);
	}
	
	public function iterate(PDFTemplatePrinter $p)
	{
		$this->iterateHeaderFooter($p, 'header');
		$this->iterateHours($p);
		$this->iterateHeaderFooter($p, 'footer');
	}

	protected function iterateHours(PDFTemplatePrinter $p)
	{
		$idx = 0;
		foreach ($this->cols_data as $table) {
			$this->renderItemsSection($p, $table['cols'], array_slice($this->data['items'], $idx, $table['count']));
			$idx += $table['count'];
		}
	}
	
	protected function renderItemsSection(PDFTemplatePrinter $p, $cols, $data)
	{
		$section = $this->findSection('items');
		foreach ($section['columns'] as &$c) {
			if (isset($cols[$c['field']])) {
				$c['label'] = $cols[$c['field']];
			}
		}
		$p->renderSection($section, $data);
	}

	public function initialize($layout, ListQuery $lq)
	{
		return parent::initialize($layout, $lq);
	}
	

	public function getFileName() {
		$filename_pfx = translate('LBL_PDF_TIMESHEET', 'Timesheets');
		$filename = $filename_pfx . '_' . $this->row->getField('name');
		$filename = preg_replace('~\.+$~', '', trim($filename));
		$filename = $filename . '.pdf';
		return $filename;
	}

	protected function createData()
	{
		global $app_list_strings;
		require_once 'modules/HR/Employee.php';

		$user = new User();
		$user->retrieve($this->row->getField('assigned_user_id'), false);
		$emp = new Employee;
		$emp->retrieve_for_user_id($user->id);
		$datefmt = AppConfig::setting('locale.defaults.date_format');
		$row =& $this->row;
		$data = array(
			'todays_date' => date($datefmt),
			'employee' => $user->name,
			'employee_number' => $emp->employee_num,
			'title' => $user->title,
			'department' => array_get_default($app_list_strings['departments_dom'], $user->department, $user->department),
			'reports_to' => $user->reports_to_name,
			'period_start' => $this->row->getField('date_starting'),
			'period_end' => $this->row->getField('date_ending'),
			'date_modified' => $this->row->getField('date_modified'),
			'total_hours' => $this->row->getField('total_hours'),
			'approval_status' => array_get_default($app_list_strings['timesheet_status_dom'], $this->row->getField('status'), $this->row->getField('status')),
			'description' => $this->row->getField('description'),
			'name' => $this->row->getField('name'),
		);
		$data = array_merge($this->header_data, $data);
		$this->data['header'] = $data;
		$this->createHoursData();
	}

	protected function createHoursData()
	{
		global $timedate, $app_list_strings;

		$section = $this->findSection('items');
		$cols = array('subject' => '');
		foreach ($section['columns'] as $col) {
			if ($col['field'] == 'subject')
				$cols['subject'] = $col['label'];
		}


		require_once 'modules/Timesheets/Timesheet.php';
		$ts = new Timesheet;
		$ts->retrieve($this->row->getField('id'));

		$grid = $ts->getTotalsByWeek();

		require_once 'modules/Booking/BookedHours.php';
		$hours_seed = new BookedHours;
		$hours = $hours_seed->query_hours(true, $ts->id, '', '', '', '', false);
		$fields = $hours_seed->getFieldDefinitions();
		require_once('include/layout/FieldFormatter.php');
		$fmt = new FieldFormatter('pdf');

		$data = array();
		$cols_data = array();
		$dayTotal = 0;
		
		$date_format = translate('LBL_LINE_DATE_FORMAT', 'Timesheets');
		$total_format = translate('LBL_PDF_DAY_TOTAL', 'Timesheets');

		$count = 0;
		$last_time = null;
		foreach ($hours as $line) {
			$line_time = strtotime($line['date_start'] . ' GMT ');
			$line_time = custom_strftime($date_format, $line_time);

			if ($last_time && $last_time != $line_time) {
				$cols['time'] = $last_time;
				$totalHrs = $fmt->formatRowValue($fields['quantity'], $line, $dayTotal);
				$cols['related_to'] = sprintf($total_format, $totalHrs);
				$cols_data[] = array(
					'count' => $count,
					'cols' => $cols,
				);
				$count = 0;
				$dayTotal = 0;
			}
			$minutes = (int)($line['quantity']);
			$start_time =  $timedate->to_display_time($line['date_start']);
			$end_time = $timedate->to_display_time(date('Y-m-d H:i:s', strtotime($line['date_start'] . " + $minutes minutes")));

			$related_name = '';
			if ($line['related_name'] && $line['related_type']) {
				$related_name = $app_list_strings['moduleListSingular'][$line['related_type']];
				$related_name .= ': ';
				$related_name .= $line['related_name'];
			}

			$cols['time'] = $line_time;
			$quantity = $fmt->formatRowValue($fields['quantity'], $line, $minutes);
			$data[] = array(
				'time' => $start_time . '-' . $end_time . ' (' . $quantity . ')',
				'subject' => $line['name'],
				'account' => $line['account_name'],
				'related_to' => $related_name,
			);
			$count++;
			$dayTotal += $minutes;
			$last_time = $line_time;
		}

		if ($last_time) {
			$cols['time'] = $last_time;
			$line = array();
			$totalHrs = $fmt->formatRowValue($fields['quantity'], $line, $dayTotal);
			$cols['related_to'] = sprintf($total_format, $totalHrs);
			$cols_data[] = array(
				'count' => $count,
				'cols' => $cols,
			);
		}
		$this->data['items'] = $data;
		$this->cols_data = $cols_data;
	}
	
	public static function get_extra_sources() {
		return array();
	}

}
	
