AppBase.module('timesheets').moduleRequires('ui-controls').init(function(App) {
	var UI = App.ui,
		Wrap = App.dom.elementOrId,
		QueryId = App.dom.queryId,
		MakeElt = App.dom.makeElement;

	UI.TimesheetEditor = function(id, params) {
		this.id = 'TimesheetEditor';
		this.editable = true;
		this.hours = {};
		this.hours_order = [];
		this.extra_order = [];
		this.checks = {};
		this.totals = {
			hours: 0,
			approved: 0
		};
		this.period = null;
		this.date_start = null;
		this.date_end = null;
		this.status_strings = {};
		this.timesheet = null;
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		if(params) extendObject(this, params);
		if(typeof(this.date_start) === 'string')
			this.date_start = parseDateString(this.date_start).d;
		if(typeof(this.date_end) === 'string')
			this.date_end = parseDateString(this.date_end).d;
		this.period = parseInt(this.period, 10);
	};
	
	UI.TimesheetEditor.prototype = {
		setup: function() {
			if(this.set_up) return;
			if(! this.elt)
				this.elt = $(this.id + '-main');
			if(! this.elt)
				return;
			
			var hidden = ['line_items', 'hours_data_updated', 'submit_timesheet', 'approve_action'];
			for(var i = 0; i < hidden.length; i++) {
				if(! this.form.elements[hidden[i]])
					this.form.appendChild(createElement2('input', {type: 'hidden', name: hidden[i]}));
			}
			
			var hasSubmit = this.form[this.form.id + '_submit'];
			if (hasSubmit) {
				if (this.timesheet.status == 'approved')
					UI.setDisplayState(hasSubmit, false);
				if (this.timesheet.status == 'submitted')
					hasSubmit.innerHTML = mod_string('LBL_UNSUBMIT_BUTTON_LABEL');
			}
			hasSubmit = this.form[this.form.id + '_submit2'];
			if (hasSubmit) {
				if (this.timesheet.status == 'approved')
					UI.setDisplayState(hasSubmit, false);
				if (this.timesheet.status == 'submitted')
					hasSubmit.innerHTML = mod_string('LBL_UNSUBMIT_BUTTON_LABEL');
			}
			
			if (this.editable) {
				this.form.total_hours.readOnly = true;
				if(this.form.date_starting)
					this.form.date_starting.value = this.date_start.print(SUGAR.session.formats.date);
				this.last_user = this.form.assigned_user_id.value;
			}
			this.lock_elements();
			this.status_strings = SUGAR.language.get('app_list_strings', 'booked_hours_status_dom');
			this.set_hours(this.rows);
			this.record_id = this.form.record.value;
			if (this.editable) {
				if(! this.record_id)
					this.start_date_changed(this.date_start, true);
				UI.attachFormInputEvent(this.form, 'assigned_user', 'onchange', this.on_user_changed.bind(this));
			}
			
			var approveAll = $(this.id + '-approve-all');
			if(approveAll) {
				approveAll.status = 'approveAll';
				approveAll.onclick = this.perform_approval.bind(this);
			}
			var approveSel = $(this.id + '-approve-sel');
			if(approveSel) {
				approveSel.status = 'approveSelected';
				approveSel.onclick = this.perform_approval.bind(this);
			}
			var rejectAll = $(this.id + '-reject-all');
			if(rejectAll) {
				rejectAll.status = 'rejectAll';
				rejectAll.onclick = this.perform_approval.bind(this);
			}
			
			this.set_up = true;			
		},

		lock_elements: function() {
			var elems = ['status', 'date_starting', 'date_ending', 'name'];
			var elem = null;

			for (var i = 0; i < elems.length; i++) {
				elem = SUGAR.ui.getFormInput(this.form, elems[i]);
				if (elem) elem.setReadOnly();
			}
		},

		retrieve_hours: function(date_start, date_end) {
			var user_id = this.form.assigned_user_id.value,
				self = this;
			if(! user_id.length) {
				this.clear_hours();
				return;
			}
			var params = {
				module: 'Booking',
				timesheet_id: this.form.record.value,
				user_id: user_id,
				date_start: date_start,
				date_end: date_end
			};
			var req = new SUGAR.conn.JSONRequest('get_booked_hours', {}, params);
			req.fetch(function(data) {
				if(! data) return;
				var hours = data.getResult();
				self.set_hours(hours);
			});
		},

		on_user_changed: function() {
			/*if (!confirm(mod_string('CONFIRM_CHANGE_USER'))) {
				this.form.assigned_user_id.value = this.last_user;
				return false;
			}
			this.last_user = this.form.assigned_user_id.value;*/
			this.on_range_changed();
			return true;
		},

		on_range_changed: function() {
			var start = this.date_start.print('%Y-%m-%d');
			var end = this.date_end.print('%Y-%m-%d');
			this.retrieve_hours(start, end);
		},

		start_date_disabled: function(date) {
			switch(this.period) {
				case 1:
					return date.getDay() != 0;
				case 2:
					var wn = date.getWeekNumber();
					return date.getDay() != 0 || (wn % 2) != 0;
				case 3:
					return date.getDate() != 1 && date.getDate() != 15;
				case 4:
					return date.getDate() != 1;
			}
		},

		end_date_disabled: function(date) {
			var days = date.getMonthDays(date.getMonth());
			switch(this.period) {
				case 1:
					return date.getDay() != 6;
				case 2:
					var wn = date.getWeekNumber();
					return !(date.getDay() == 6 && (wn % 2==0));
				case 3:
					return date.getDate() != 14 && date.getDate() != days;
				case 4:
					return date.getDate() != days;
			}
		},

		change_period: function(offset) {
			var date = this.date_start;
			var new_date = date;
			switch(this.period) {
				case 1:
					new_date = new Date(date.getFullYear(), date.getMonth(), date.getDate()+7*offset);
					break;
				case 2:
					new_date = new Date(date.getFullYear(), date.getMonth(), date.getDate()+14*offset);
					break;
				case 3:
					var day = date.getDate();
					var month = date.getMonth();
					if(day == 1) {
						day = 16;
						if(offset < 0) month --;
					}
					else {
						day = 1;
						if(offset > 0) month ++;
					}
					new_date = new Date(date.getFullYear(), month, day);
					break;
				case 4:
					var month = date.getMonth();
					new_date = new Date(date.getFullYear(), month+offset, 1);
					break;
			}
			this.start_date_changed(new_date);
		},

		start_date_changed: function(date, no_reload) {
			this.date_start = new Date(date);
			SUGAR.ui.getFormInput(this.form, 'date_starting').setValue(date, true);
			var new_end = this.date_start;
			switch(this.period) {
				case 1:
					new_end = new Date(date.getFullYear(), date.getMonth(), date.getDate()+6);
					break;
				case 2:
					new_end = new Date(date.getFullYear(), date.getMonth(), date.getDate()+13);
					break;
				case 3:
					var day = date.getDate();
					if (day > 15) day = date.getMonthDays(date.getMonth());
					else day = 15;
					new_end = new Date(date.getFullYear(), date.getMonth(), day);
					break;
				case 4:
					var day = date.getMonthDays(date.getMonth());
					new_end = new Date(date.getFullYear(), date.getMonth(), day);
					break;
			}
			this.date_end = new_end;
			SUGAR.ui.getFormInput(this.form, 'date_ending').setValue(new_end, true);
			if (!no_reload)
				this.on_range_changed();
		},

		end_date_changed: function(date) {
			this.date_end = new Date(date);
			this.form.date_ending.value = date.print(SUGAR.session.formats.date);
			var new_start = this.date_end;
			switch(this.period) {
				case 1:
					new_start = new Date(date.getFullYear(), date.getMonth(), date.getDate()-6);
					break;
				case 2:
					new_start = new Date(date.getFullYear(), date.getMonth(), date.getDate()-13);
					break;
				case 3:
					var day = date.getDate();
					if (day < 15) day = 1;
					else day = 15;
					new_start = new Date(date.getFullYear(), date.getMonth(), day);
					break;
				case 4:
					new_start = new Date(date.getFullYear(), date.getMonth(), 1);
					break;
			}
			this.date_start = new_start;
			this.form.date_starting.value = new_start.print(SUGAR.session.formats.date);
			this.on_range_changed();
		},

		/* not used
		this.get_spanned_dates = function() {
			var startDate = new Date(this.date_start);
			var endDate = this.date_end;
			while(startDate <= endDate) {
				dates.push(startDate);
				startDate = new Date(startDate.getTime() + 86400100);
			}
			return dates;
		}*/

		set_hours: function(rows) {
			if(isPlainObject(rows)) {
				this.hours_order = deep_clone(rows.order);
				this.hours = deep_clone(rows.hours);
				if(! this.hours_order.length)
					this.hours_order = [];
				if(rows.extra && rows.extra.length)
					this.extra_order = deep_clone(rows.extra);
				else
					this.extra_order = [];
			}
			this.draw();
		},

		clear_hours: function() {
			this.hours_order = [];
			this.extra_order = [];
			this.hours = {};
			this.draw();
		},

		iter_hours: function(callback, extra) {
			var ord = extra ? this.extra_order : this.hours_order;
			for(var idx = 0; idx < ord.length; idx++) {
				callback.call(this, idx, this.hours[ord[idx]], extra);
			}
			return idx;
		},

		update_totals: function() {
			this.totals = { hours: 0, approved: 0 };
			this.iter_hours(
				function(idx, line) {
					var h = parseFloat(get_default(line.quantity, 0));
					this.totals.hours += h;
					if(line.status == 'approved')
						this.totals.approved += h;
				}
			);
			var total_f = SUGAR.ui.getFormInput(this.form, 'total_hours');
			if(total_f) {
				total_f.ignore_dirty = true;
				total_f.setValue(stdFormatNumber(this.totals.hours / 60));
			}
		},

		parse_date_time: function(db_date, db_time) {
			var day, time;
			if(db_date) {
				day = db_date.split('-');
				if (day.length > 2) {
					var day_time = day[2].split(' ');
					day[2] = day_time[0];
					if (!db_time) {
						db_time = day_time[1];
					}
				}
			}
			else
				day = [0, 1, 0];
			if(db_time)
				time = db_time.split(':');
			else
				time = [0, 0, 0];
			var dt = new Date(day[0], day[1]-1, day[2], time[0], time[1], time[2]);
			return dt;
		},

		format_date: function(db_date) {
			var dt = this.parse_date_time(db_date);
			return dt.print(mod_string('LBL_LINE_DATE_FORMAT'));
		},

		format_time: function(db_date) {
			var dt = this.parse_date_time(db_date);
			return dt.print(SUGAR.session.formats.time);
		},

		draw: function() {
			var buttonsDiv = $(this.id + '-approval');
			if (buttonsDiv && this.timesheet.can_approve) {
				UI.setDisplayState(buttonsDiv, true);
			}
			var tbl = Wrap(this.elt).findFirst('.timesheet-items-body').ext(0);
			if(! tbl) return;
			tbl.empty();
			var rowidx = 0;
			var last_date;
			var drew = 0;
			var drawLine = function(idx, line, extra) {
				var hdr_date = this.format_date(line.date_start);
				if(hdr_date != last_date) {
					var row = MakeElt('div.row.timesheet-row-sep').appendTo(tbl);
					var cell = MakeElt('div.column.expand').appendTo(row);
					var hdr_date = this.format_date(line.date_start);
					var span = MakeElt('span', null, hdr_date).appendTo(cell);
					last_date = hdr_date;
				}
		
				var row = MakeElt('div.row.col-space-sm.timesheet-row-hours').appendTo(tbl);
								
				var cell = MakeElt('div.column.timesheet-col-start').appendTo(row);
				if (this.timesheet.can_approve) {
					var check = this.checks[line.id];
					if(! check) {
						this.checks[line.id] = check = new SUGAR.ui.CheckInput('approve_'+line.id,
							{name: 'approve_hours[]', no_default: true,
							 ext_class: 'large', submit_value: line.id,
							 init_value: line.status == 'approved'});
						check.render();
						SUGAR.ui.registerInput(this.form, check);
					}
					cell.append(check.elt);
					cell.append(nbsp());
					check.setup();
				}
				var time = this.format_time(line.date_start);
				cell.append(time);
		
				var cell = MakeElt('div.column.timesheet-col-qty').appendTo(row);
				var hours = line.quantity_formatted;
				cell.append(hours);
		
				cell = MakeElt('div.column.timesheet-col-name').appendTo(row);
				img = MakeElt('div.input-icon.theme-icon.module-Booking');
				cell.append(img);
				cell.append(nbsp());
				var lbl = line.name;
				if(! this.editable) {
					var href = 'index.php?module=Booking&action=DetailView&record=' + line.id;
					lbl = MakeElt('a', {href: href, 'class': 'tabDetailViewDFLink'}, lbl);
				}
				cell.append(lbl);
		
				cell = MakeElt('div.column.timesheet-col-related').appendTo(row);
				if(line.related_type && line.related_name) {
					img = MakeElt('div.input-icon.theme-icon.module-' + line.related_type);
					cell.append(img);
					cell.append(nbsp());
					lbl = line.related_name;
					if(! this.editable) {
						var href = 'index.php?module=' + line.related_type + '&action=DetailView&record=' + line.related_id;
						lbl = MakeElt('a', {href: href, 'class': 'tabDetailViewDFLink'}, lbl);
					}
					cell.append(lbl);
				} else {
					cell.append(mod_string('LBL_NO_RELATED'));
				}
				if(line.parent_name) {
					cell.append(MakeElt('br'));
					cell.append(nbsp(6));
					var sm = MakeElt('small', null, line.parent_name);
					cell.append(sm);
				}

				cell = MakeElt('div.column.timesheet-col-status').appendTo(row);
				var stat = this.status_strings[line.status];
				if(this.status_dom && line.status in this.status_dom) {
					var state = this.status_dom[line.status];
					stat = MakeElt('div.badge.state-'+state, null, stat);
				}
				if (!stat) stat = '';
				cell.append(stat);

				if(this.editable) {
					cell = MakeElt('div.column.timesheet-col-end').appendTo(row);
					var img, lbl;
					var link = MakeElt('span.uii.uii-lg.active-icon' + (extra ? '.uii-add' : '.uii-remove'));
					if(! extra) {
						//lbl = app_string('LBL_REMOVE');
						link.on('click', function() {
							this.remove_hours(line.id);
							return false;
						}, {scope: this});
					}
					else {
						//lbl = app_string('LBL_INS');
						link.on('click', function() {
							this.add_extra_hours(line.id);
							return false;
						}, {scope: this});
					}
					//link.append(' '+lbl);
					cell.append(link);
				}
			};
			
			if(! this.iter_hours(drawLine)) {
				var row = MakeElt('div.row').appendTo(tbl);
				var cell = MakeElt('div.column.timesheet-col-blank.text-help').appendTo(row);
				cell.append(mod_string('LBL_NO_BOOKED_HOURS'));
			}

			var extra = QueryId(this.id + '-extra');
			tbl = extra.findFirst('.timesheet-items-body').ext(0);
			if(tbl) {
				tbl.empty();
				last_date = null;
				extra.toggle(!! this.iter_hours(drawLine, true));
			}
			this.update_totals();
		},

		add_extra_hours: function(id) {
			var found;
			for(var idx = 0; idx < this.extra_order.length; idx++) {
				if(this.extra_order[idx] == id) {
					found = idx;
					break;
				}
			}
			if(! isset(found))
				return;
			this.extra_order.splice(found, 1);
			this.add_in_order(id);
			this.draw();
		},

		remove_hours: function(id) {
			var found;
			for(var idx = 0; idx < this.hours_order.length; idx++) {
				if(this.hours_order[idx] == id) {
					found = idx;
					break;
				}
			}
			if(! isset(found))
				return;
			this.hours_order.splice(found, 1);
			this.add_in_order(id, true);
			this.draw();
		},

		add_in_order: function(id, extra) {
			var start = this.parse_date_time(this.hours[id].date_start, this.hours[id].time_start);
			var target = extra ? this.extra_order : this.hours_order;
			var idx = 0;
			for(; idx < target.length; idx++) {
				if(! this.hours[target[idx]])
					continue;
				var dt = this.parse_date_time(this.hours[target[idx]].date_start, this.hours[target[idx]].time_start);
				if(dt > start)
					break;
			}
			target.splice(idx, 0, id);
		},

		beforeSubmitForm: function(form) {
			this.setFormValues(form);
		},

		validateForm: function() {
			var form = document.DetailForm;
			if (form.submit_timesheet)
				form.submit_timesheet.value = '';
			return SUGAR.ui.sendForm(form, {'record_perform':'validate'});
		},

		setFormValues: function(form) {
			if (form.line_items) {
				var ret = { order: this.hours_order, hours: this.hours, extra: this.extra_order };
				form.line_items.value = JSON.stringify(ret);
				form.hours_data_updated.value = '1';
			}
		},

		submit_unsubmit: function() {
			if (this.timesheet.status == 'submitted')
				this.form.submit_timesheet.value = '2';
			else
				this.form.submit_timesheet.value = '1';
			return SUGAR.ui.sendForm(this.form, {'record_perform':'save'});
		},
		
		perform_approval: function(evt) {
			var status;
			if(typeof(evt) === 'string')
				status = evt;
			else if(evt && evt.target)
				status = Wrap(evt.target).closest('.input-button').prop('status');
			if(status) {
				this.form.approve_action.value = status;
				SUGAR.ui.sendForm(this.form, {'record_perform':'save'});
			}
		},

		check_save: function() {
			var date1 = this.date_start.print('%Y-%m-%d'),
				date2 = this.date_end.print('%Y-%m-%d'),
				self = this,
				params = {
					record: this.form.record.value,
					date_starting: date1,
					date_ending: date2,
					assigned_user_id: this.form.assigned_user_id.value
				},
				req = new SUGAR.conn.JSONRequest('check_date_range', {}, params);
			req.fetch(function(data) {
				if(! data) return;
				var check = data.getResult();
				if(check) self.final_save();
			});				
			return false;
		},

		final_save: function() {
			if(this.editable) {
				var ret = { order: this.hours_order, hours: this.hours, extra: this.extra_order };
				if(this.form.hours_data) {
					this.form.hours_data.value = JSON.stringify(ret);
					if(this.form.hours_data_updated)
						this.form.hours_data_updated.value = '1';
				}
			}
			this.form.submit();
		}
	}
}); // TimesheetEditor


function perform_submit_unsubmit(form) {
	var editor = SUGAR.ui.getFormInput(form, 'timesheet_items');
	if(editor) editor.submit_unsubmit();
}
