<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormTableSection.php');

class TimesheetItemsWidget extends FormTableSection {
	var $timesheetRows;
	var $hourObjs;

	function init($params=null, $model=null) {
		parent::init($params, $model);
		if(! $this->id)
			$this->id = 'expense_items';
	}

    function getRequiredFields() {
        return array();
    }

	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if($gen->getLayout()->getEditingLayout()) {
			$params = array();
			return $this->renderOuterTable($gen, $parents, $context, $params);		
		}

        global $mod_strings;
		$allow_edit = $gen->getLayout()->getDefaultEditable() ? 1 : 0;
		if (!(($ts_status = $row_result->getField('status')) == 'submitted' || $ts_status == 'approved')) {
			if ($allow_edit) $allow_edit = 2;
		}
		
		$params['description'] = translate('LBL_TIMEZONE_WARNING', 'Timesheets');
		
		$body = <<<EOQ
			<div id="{$this->id}-main" class="timesheet-items-main">
				<div class="row form-row col-unstack" id="{$this->id}-nav">
					<div class="column form-cell col-6">
EOQ;

		$form_name = $gen->getFormName();

		$lbl_prev = $mod_strings['LBL_PREVIOUS_PERIOD'];
		$lbl_next = $mod_strings['LBL_NEXT_PERIOD'];
		if ($allow_edit == 2 && $row_result->new_record)  $body .= <<<EOQ
			<div class="form-entry">
			<div class="buttons-inline">
				<div class="inline-elt">
					<button type="button" class="input-button" onclick="SUGAR.ui.getFormInput('$form_name', '{$this->id}').change_period(-1); return false;">
						<span class="uii uii-previous"></span> <span class="input-label">$lbl_prev</span>
					</button>
				</div>
				<div class="inline-elt">
				<button type="button" class="input-button" onclick="SUGAR.ui.getFormInput('$form_name', '{$this->id}').change_period(1); return false;">
					<span class="input-label">$lbl_next</span> <span class="uii uii-next"></span>
				</button>
				</div>
			</div>
			</div>
EOQ;

		$lbl_app_sel = $mod_strings['LBL_APPROVE_SOME_BUTTON_LABEL'];
		$lbl_app_all = $mod_strings['LBL_APPROVE_ALL_BUTTON_LABEL'];
		$lbl_rej_all = $mod_strings['LBL_REJECT_ALL_BUTTON_LABEL'];

        $body .= <<<EOQ
        </div>
		<div class="column form-cell col-6" id="{$this->id}-approval" style="display: none">
			<div class="form-entry">
			<div class="buttons-inline align-right">
				<div class="inline-elt">
					<button type="button" class="input-button" id="{$this->id}-approve-sel"><span class="uii uii-accept"></span> <span class="input-label">$lbl_app_sel</span></button>
				</div>
				<div class="inline-elt">
					<button type="button" class="input-button" id="{$this->id}-approve-all"><span class="uii uii-accept"></span> <span class="input-label">$lbl_app_all</span></button>
				</div>
				<div class="inline-elt">
					<button type="button" class="input-button" id="{$this->id}-reject-all"><span class="uii uii-cancel"></span> <span class="input-label">$lbl_rej_all</span></button>
				</div>
			</div>
			</div>
		</div>
		</div>
EOQ;

        $name = translate('LBL_SUBJECT', 'Booking');
        $quantity = translate('LBL_QUANTITY', 'Booking');
        $related_to = translate('LBL_RELATED_TO', 'Booking');
        $status = translate('LBL_STATUS', 'Booking');
        $empty_cell = '';
        $panel_edit = $allow_edit ? '' : 'panel-readonly';

		$body .= <<<EOQ
			<div class="row form-row">
			<div class="column form-cell expand"><div class="form-entry">
			<section class="card-outer panel-outer panel-default timesheet-items-outer panel-border panel-round {$panel_edit}">
				<div class="card-header panel-subheader">
					<div class="row col-space-sm panel-column-titles">
						<div class="column timesheet-row-head timesheet-col-start">&nbsp;</div>
						<div class="column timesheet-row-head timesheet-col-qty"><span class="column-title">{$quantity}</span></div>
						<div class="column timesheet-row-head timesheet-col-name"><span class="column-title">{$name}</span></div>
						<div class="column timesheet-row-head timesheet-col-related"><span class="column-title">{$related_to}</span></div>
						<div class="column timesheet-row-head timesheet-col-status"><span class="column-title">{$status}</span></div>
					</div>
				</div>
				<div class="card-body panel-body panel-pad-body timesheet-items-body">
				</div>
			</section>
			</div></div>
			</div>
EOQ;
		
		if ($allow_edit == 2) $body .= <<<EOQ
			<div id="{$this->id}-extra" class="timesheet-items-extra">
				<div class="row form-row">
				<div class="column form-cell expand"><div class="form-entry">
					<section class="card-outer panel-outer panel-default timesheet-items-outer panel-border panel-round">
						<div class="card-header panel-header">
							<h4 class="no-margin">{$mod_strings['LBL_EXTRA_BOOKED_HOURS']}</h4>
						</div>
						<div class="card-body panel-body panel-pad-body timesheet-items-body">
						</div>
					</section>
				</div></div>
				</div>
			</div>
EOQ;

		$editParams = $this->getEditorParams($row_result, $allow_edit);
		$gen->getFormObject()->addControl('TimesheetEditor', $this->id, $editParams);

		$params['body'] = $body . '</div>';
        return $this->renderOuterTable($gen, $context, $parents, $params);
	}
	
	function js_format_date($d) {
		if (empty($d)) $d = date('Y-m-d');
		list($y,$m,$d) = explode('-', $d, 3);
		$d = ltrim($d, '0');
		$m = ltrim($m, '0');
		$y = ltrim($y, '0');
		$m --;
		return "new Date($y, $m, $d)";
	}

    function getEditorParams(RowResult $timesheet, $edit = 0) {
		$json = getJSONObj();

		$dt_start = $timesheet->getField('date_starting');
		$dt_end = $timesheet->getField('date_ending');
		$date_start_js = $this->js_format_date($dt_start);
		$date_end_js = $this->js_format_date($dt_end);

		$ts_status = $timesheet->getField('status');
		$ts_period = $timesheet->getField('timesheet_period');
		$allow_edit = ($edit == 2);

		if (! empty($this->timesheetRows)) {
			$hours_data = $this->timesheetRows;
		} else {
			$hours_data = $this->getHours($timesheet);
		}
		$ts_fields = array(
			'status' => $ts_status,
		);
		$canApprove = false;
		if (!$edit) {
			if($ts_status == 'submitted' || $ts_status == 'rejected')
				$canApprove = canApprove(AppConfig::current_user_id(), $timesheet);
		}
		$ts_fields['can_approve'] = $canApprove;
		
		$params = [
			'editable' => $allow_edit,
			'period' => $ts_period,
			'date_start' => $dt_start,
			'date_end' => $dt_end,
			'rows' => $hours_data,
			'timesheet' => $ts_fields,
			'status_dom' => AppConfig::setting('statuses.booked_hours_status_dom'),
		];
		
        return $params;
    }

    function getHours(RowResult $timesheet) {
        require_bean('BookedHours');
        $hours = new BookedHours();
        $ts_id = $timesheet->getField('id');
        $data = $hours->query_hours(false, $ts_id ? $ts_id : 'none', $timesheet->getField('assigned_user_id'), '', $timesheet->getField('date_starting'), $timesheet->getField('date_ending'), true);
        $order = array_keys($data);

        if($ts_id) {
            $extra = $hours->query_hours(false, 'none', $timesheet->getField('assigned_user_id'), '', $timesheet->getField('date_starting'), $timesheet->getField('date_ending'), true);
            array_extend($data, $extra);
            $extra = array_keys($extra);
        } else {
            $extra = array();
        }

        return  array('order' => $order, 'hours' => $data, 'extra' => $extra);
    }

	function loadUpdateRequest(RowUpdate $update, array $input) {
		if (! empty($input['approve_action'])) {
			$this->approveAction($update, $input);
			return;
		}
        $upd_items = array();
        if (!empty($input['line_items'])) {
            $json = getJSONObj();
            $upd_items = $json->decode(from_html($input['line_items']));
        }
		$update->setRelatedData($this->id.'_rows', $upd_items);
		$update->setRelatedData($this->id.'_hours_updated', array_get_default($input, 'hours_data_updated'));
		$update->setRelatedData($this->id . '_submit_status', array_get_default($input, 'submit_timesheet'));
		$this->timesheetRows = $upd_items;
		$this->updateTotalHours($upd_items, $update, $input);
	}

	function updateTotalHours($hours, RowUpdate $update, array $input) {
		$this->hourObjs = array();
		if (!is_array($hours) || empty($hours)) return;
		if (!array_get_default($input, 'hours_data_updated')) return;
		$total_hours = 0;
		$submit_status = $update->getRelatedData($this->id.'_submit_status');
		require_bean('BookedHours');
		foreach($hours['order'] as $id) {
			$booked = new BookedHours();
			if($booked->retrieve($id)) {
				if($submit_status && $booked->status != 'approved') {
					if($submit_status == 1) $booked->status = 'submitted';
					else if($submit_status == 2) $booked->status = 'pending';
				}
				$this->hourObjs[] = $booked;
				$total_hours += ($booked->quantity / 60);
			}
		}
		$upd = array(
			'total_hours' => $total_hours
		);
		$submit_status = $update->getRelatedData($this->id.'_submit_status');
		if ($submit_status == 1) $upd['status'] = 'submitted';
		if ($submit_status == 2) $upd['status'] = 'draft';
		$update->set($upd);
	}
	
	function validateInput(RowUpdate $update) {
		$id = $update->getField('id');
		$user_id = $update->getField('assigned_user_id');
		$date_starting = $update->getField('date_starting');
		$date_ending = $update->getField('date_ending');
		require_bean('Timesheet');
		$ts = new Timesheet;
		if (!$ts->check_date_range($id, $user_id, $date_starting, $date_ending)) {
            throw new IAHError(translate('MSG_DUPLICATE_PERIOD', 'Timesheets'));
		}
	}
	
	function afterUpdate(RowUpdate $update) {
		$row_updates = $update->getRelatedData($this->id.'_rows');
		$hours_updated = $update->getRelatedData($this->id.'_hours_updated');
		if(! $hours_updated)
			return;
		
        if (!$update->new_record) {
		    $ts_id = $update->getField('id', null, true);
        } else {
            $ts_id = $update->saved['id'];
        }
		if(is_array($row_updates)) {
			if(isset($row_updates['extra']) && !$update->new_record)
			foreach($row_updates['extra'] as $id) {
				$booked = new BookedHours();
				if($booked->retrieve($id) && $booked->timesheet_id == $ts_id) {
					if($booked->status == 'submitted')
						$booked->status = 'pending';
					$booked->timesheet_id = '';
					$booked->save();
				}
				$booked->cleanup();
			}
		}
		
		foreach($this->hourObjs as $booked) {
			$booked->timesheet_id = $ts_id;
			$booked->save();
			$booked->cleanup();
		}

		$submit_status = $update->getRelatedData($this->id.'_submit_status');
		if($ts_id && $submit_status == '2') {
			$lq = new ListQuery('BookedHours');
			$lq->addSimpleFilter('status', 'submitted', '=');
			$lq->addSimpleFilter('timesheet_id', $ts_id, '=');
			$lr = $lq->runQuery();
			foreach ($lr->getRowIndexes() as $idx) {
				$rr = $lr->getRowResult($idx);
				$ru = new RowUpdate($rr);
				$ru->set(array('status' => 'pending'));
				$ru->save();
			}
		}
	}
	
	function approveAction(RowUpdate $update, array $input) {
		$ts_id = $update->getField('id');
		if(! $ts_id) return;
		$ts = ListQuery::quick_fetch('Timesheet', $ts_id, true, array('acl_filters' => 'edit'));
		if(! $ts) return;
		
		$ids = null;
		$status = 'approved';
		switch ($input['approve_action']) {
			case 'approveSelected':
				$ids = array_get_default($input, 'approve_hours', array());
				break;
			case 'rejectAll':
				$status = 'rejected';
				break;
			case 'approveAll':
				break;
			default:
				return;
		}
		
		$counts = [
			'total' => 0,
			'approved' => '0',
		];

		$updatedAny = false;
		$lq = new ListQuery('BookedHours');
		$lq->addSimpleFilter('timesheet_id', $ts_id, '=');
		foreach ($lq->getIterator() as $rr) {
			$counts['total'] ++;
			$hoursStatus = $rr->getField('status');
			$do_update = $hoursStatus != $status;
			if (!is_null($ids)) {
				$do_update = in_array($rr->getField('id'), $ids) && $do_update;
			}
			if ($do_update) {
				$ru = RowUpdate::for_result($rr);
				$ru->set('status', $status);
				if ($ru->validate() && $ru->save()) {
					$updatedAny = true;
					$hoursStatus = $status;
				}
			}
			if (isset($counts[$hoursStatus]))
				$counts[$hoursStatus]++;
		}

		$ts_status = 
			$counts['approved'] == $counts['total'] 
			? 'approved' 
			: ($status == 'rejected' ? 'rejected' : null);
		if ($updatedAny && $ts_status) {
			$tsup = RowUpdate::for_result($ts);
			$tsup->set('status', $ts_status);
			$tsup->validate() && $tsup->save();
		}
	}

    function renderPdf(PdfFormGenerator $gen, $row_result, array $parents, array $context) {
        global $mod_strings, $timedate, $app_list_strings;

        $booked_hours = $this->getHours($row_result);

        if (! empty($booked_hours['hours'])) {
            $data = array();
            $cols = array(
                'date' => array('width' => '20%', 'title' => ''),
                'hours' => array('width' => '10%', 'title' => translate('LBL_QUANTITY', 'Booking')),
                'description' => array('width' => '25%', 'title' => translate('LBL_SUBJECT', 'Booking')),
                'related_name' => array('width' => '30%', 'title' => translate('LBL_RELATED_TO', 'Booking')),
                'status' => array('width' => '15%', 'title' => translate('LBL_STATUS', 'Booking')),
            );

            foreach ($booked_hours['hours'] as $line) {
                $line_time = strtotime($line['date_start']);
                $line_time = custom_strftime($mod_strings['LBL_LINE_DATE_FORMAT'], $line_time);

                $data[] = array(
                    'date' => $line_time,
                    'hours' => '',
                    'description' => '',
                    'related_name' => '',
                    'status' => ''
                );

                $related_name = '';
                if ($line['related_name'] && $line['related_type']) {
                    $related_name = $app_list_strings['moduleListSingular'][$line['related_type']];
                    $related_name .= ': ';
                    $related_name .= $line['related_name'];
                }

                $minutes = (int)($line['quantity']);
                $start_time =  $timedate->to_display_time($line['date_start'], false, false);
                $quantity = ($minutes > 0) ? ($minutes / 60) : 0;

                $data[] = array(
                    'date' => $start_time,
                    'hours' => sprintf($mod_strings['LBL_DETAIL_PDF_NUM_HOURS'], $quantity),
                    'description' => $line['name'],
                    'related_name' => $related_name,
                    'status' => $app_list_strings['booked_hours_status_dom'][$line['status']]

                );

            }

            $opts = array(
                'border' => 0,
                'header' => array(
                    'border' => 0,
                ),
            );

            $gen->pdf->DrawTable($data, $cols, $mod_strings['LBL_TIMESHEET_BOOKED_HOURS'], true, $opts);
        }
    }
}
?>
