<?php

require_once 'include/DetailView/StandardDetailManager.php';


class UserDetailManager extends StandardDetailManager
{
	var $user;
	var $user_locale;

	function loadRecord($id=null) {
		$ret = parent::loadRecord($id);
		if ($this->record instanceof RowResult) {
			$this->loadUser($this->record);
			$this->modifyResultFields($this->record);
		}
		return $ret;
	}

	function loadUser($row_result)
	{
		$user = new User;
		if ($row_result instanceof RowResult)
			$user->id = $row_result->getField('id');
		else
			$user->id = $row_result;
		$this->user_locale = new Localization($user);
		$this->user = $user;
	}

	function modifyResultFields(RowResult &$row_result)
	{
        global $timedate, $mod_strings, $app_list_strings;
		$data = array();
		$mod_strings = is_array($mod_strings) ? $mod_strings : [];
		$row_result->fields['default_export_format']['options'] = $this->user_locale->getExportFormatSelect();
		$data['default_export_format'] = $this->user_locale->getPrecedentPreference('default_export_format');

		$data['default_export_charset'] = $this->user_locale->getExportCharset(false);
		$charsets = $this->user_locale->getCharsetSelect();
		$charsets = array('' => array_get_default($mod_strings, 'LBL_DEFAULT_CHARSET')) + $charsets;
		$row_result->fields['default_export_charset']['options'] = $charsets;

		$user_mode = $this->getUserPreference('file_download_mode');
		$data['file_download_mode'] = ($user_mode != null) ? $user_mode : 'save';
		$row_result->fields['file_download_mode']['options'] = array('save' => array_get_default($mod_strings, 'LBL_DOWNLOAD_SAVE'), 'open' => array_get_default($mod_strings, 'LBL_DOWNLOAD_OPEN'));


        $default_date_format = $this->getUserPreference('default_date_format');
		$default_time_format = $this->getUserPreference('default_time_format');
		$time_formats = AppConfig::setting('locale.time_formats');
        $user_timezone = $this->getUserPreference('timezone');

		$row_result->fields['dateformat']['options'] = AppConfig::setting('locale.date_formats');
		$data['dateformat'] = $default_date_format;
		$row_result->fields['timeformat']['options'] = AppConfig::setting('locale.time_formats');
		$data['timeformat'] = $default_time_format;

		$data['timezone'] = $user_timezone;
		$row_result->fields['timezone']['options'] = $timedate->getTimeZones(true, false);

		require_once('modules/Currencies/ListCurrency.php');
        $list_currency = new ListCurrency();
		$data['currency'] = ($this->getUserPreference('currency')) ? $this->getUserPreference('currency') : '-99';
        $row_result->fields['currency']['options'] = $list_currency->getCurrenciesNames();

		$row_result->fields['default_number_format']['options'] = $this->user_locale->getLocaleNumberFormatOptions();
		$data['default_number_format'] = $this->user_locale->getLocaleNumberFormatName();

		//$data['default_locale_name_format'] = $this->user_locale->getLocaleFormatMacro();

		$row_result->fields['user_holidays']['options'] = $this->getHolidays();
        $hours = $this->getHoursList();
		$row_result->fields['day_begin_hour']['options'] = $hours;
		$row_result->fields['day_end_hour']['options'] = $hours;

		$charset_options = $this->user_locale->getCharsetSelect();
		$charset_options = array('' => array_get_default($mod_strings, 'LBL_DEFAULT_CHARSET')) + $charset_options;
		$row_result->fields['default_email_charset']['options'] = $charset_options;
        $data['default_email_charset'] = $this->user_locale->getOutboundEmailCharset(false);
		
		$row_result->fields['email_display_format']['options'] = array('html' => array_get_default($mod_strings, 'LBL_VIEW_FORMAT_HTML'), 'plain' => array_get_default($mod_strings, 'LBL_VIEW_FORMAT_PLAIN'));
		$row_result->fields['email_compose_format']['options'] = array('html' => array_get_default($mod_strings, 'LBL_VIEW_FORMAT_HTML'), 'plain' => array_get_default($mod_strings, 'LBL_VIEW_FORMAT_PLAIN'));
		$row_result->fields['show_images']['options'] = array('' => array_get_default($mod_strings, 'LBL_HIDE_IMAGES'), 'display' => array_get_default($mod_strings, 'LBL_SHOW_IMAGES'));

		/*
            $week_start_day = $this->renderSelect($gen, 'week_start_day', $app_list_strings['weekdays_long_dom'], $row_result->getField('week_start_day'), null, null, false);
            $day_begin_hour = $this->renderSelect($gen, 'day_begin_hour', $hours, $row_result->getField('day_begin_hour'), null, null, false);
            $day_end_hour = $this->renderSelect($gen, 'day_end_hour', $hours, $row_result->getField('day_end_hour'), null, null, false);
			$user_holidays = $this->renderSelect($gen, 'user_holidays', $holidays_list, $holiday_selected);
		 */

		$data['max_tabs'] = $this->getUserPreference('max_tabs');
		$data['max_subtabs'] = $this->getUserPreference('max_subtabs');
		$data['swap_last_viewed'] = $this->getUserPreference('swap_last_viewed');
		$data['swap_shortcuts'] = $this->getUserPreference('swap_shortcuts');
		$data['navigation_paradigm'] = $this->getUserPreference('navigation_paradigm');
		$data['show_footer_module_links'] = $this->getUserPreference('show_footer_module_links');

        $data['publish_url'] = '';
        $data['search_url'] = '';
        $data['subscribe_url'] = '';
        $data['addressbook_url'] = '';
        $data['contacts_principal_url'] = '';
        $data['calendar_principal_url'] = '';
        $data['calls_calendar_url'] = '';
        $data['meetings_calendar_url'] = '';
        $data['tasks_calendar_url'] = '';
        $main_email = $row_result->getField('email1');
        $publish_key = $this->getUserPreference('calendar_publish_key');

        if ($this->perform != 'view') {
        } else {
            if (! empty($main_email)) {
                $data['publish_url'] = AppConfig::site_url().'vcal_server.php';
                $token = "/";

                //determine if the web server is running IIS
                //if so then change the publish url
                if(isset($_SERVER) && ! empty($_SERVER['SERVER_SOFTWARE'])){
                    $position = strpos(strtolower($_SERVER['SERVER_SOFTWARE']), 'iis');
                    if($position !== false) $token = '?parms=';
                }

                $data['publish_url'] .= $token.'type=vfb&email='.$main_email.'&source=outlook&key=' . $publish_key;
            } else {
                $data['publish_url'] = AppConfig::site_url().'vcal_server.php/type=vfb&user_name=' .$row_result->getField('user_name'). '&source=outlook&key=' . $publish_key;
            }

            $data['search_url'] = AppConfig::site_url().'vcal_server.php/type=vfb&email=%NAME%@%SERVER%';
            $webcal_url = str_replace('http:', 'webcal:', AppConfig::site_url());
            $ical_subscribe = $webcal_url.'vcal_server.php/type=ics&user_name=' .$row_result->getField('name'). '&key=' . $publish_key;
            $data['subscribe_url'] = $ical_subscribe;
            $data['contacts_principal_url'] = AppConfig::site_url() . 'carddav.php/principals/' . urlencode($row_result->getField('user_name'));
            $data['calendar_principal_url'] = AppConfig::site_url() . 'caldav.php/principals/' . urlencode($row_result->getField('user_name'));
			$data['addressbook_url'] = AppConfig::site_url() . 'carddav.php/addressbooks/' . urlencode($row_result->getField('user_name')) . '/contacts';
			$data['calls_calendar_url'] = AppConfig::site_url() . 'caldav.php/calendars/' . urlencode($row_result->getField('user_name')) . '/calls';
			$data['meetings_calendar_url'] = AppConfig::site_url() . 'caldav.php/calendars/' . urlencode($row_result->getField('user_name')) . '/meetings';
			$data['tasks_calendar_url'] = AppConfig::site_url() . 'caldav.php/calendars/' . urlencode($row_result->getField('user_name')) . '/tasks';
		}

		$row_result->fields['google_contacts_which']['options'] = array('' => array_get_default($mod_strings, 'LBL_ASSIGNED_TO_ME'), 'visible' => array_get_default($mod_strings, 'LBL_ALL_VISIBLE'));
		$row_result->fields['google_calendar_meeting']['options'] = array('' => array_get_default($mod_strings, 'LBL_SYNC_NO_SYNC'), '1' => array_get_default($mod_strings, 'LBL_SYNC_ASSIGNED_TO_ME'), '2' => array_get_default($mod_strings, 'LBL_SYNC_INVITED_MEETINGS'));
		$row_result->fields['google_calendar_call']['options'] = array('' => array_get_default($mod_strings, 'LBL_SYNC_NO_SYNC'), '1' => array_get_default($mod_strings, 'LBL_SYNC_ASSIGNED_TO_ME'), '2' => array_get_default($mod_strings, 'LBL_SYNC_INVITED_CALLS'));
		$row_result->fields['google_calendar_task']['options'] = array('' => array_get_default($mod_strings, 'LBL_SYNC_NO_SYNC'), '1' => array_get_default($mod_strings, 'LBL_SYNC_ASSIGNED_TO_ME'));

		$row_result->fields['carddav_contacts_which']['options'] = array('' => array_get_default($mod_strings, 'LBL_ASSIGNED_TO_ME'), 'visible' => array_get_default($mod_strings, 'LBL_ALL_VISIBLE'), 'fav' => array_get_default($mod_strings, 'LBL_SYNC_FAV'));
		$row_result->fields['caldav_meetings']['options'] = array('' => array_get_default($mod_strings, 'LBL_ASSIGNED_TO_ME'), 'visible' => array_get_default($mod_strings, 'LBL_ALL_VISIBLE'), 'invited' => array_get_default($mod_strings, 'LBL_ALL_INVITED'));
		$row_result->fields['caldav_calls']['options'] = array('' => array_get_default($mod_strings, 'LBL_ASSIGNED_TO_ME'), 'visible' => array_get_default($mod_strings, 'LBL_ALL_VISIBLE'), 'invited' => array_get_default($mod_strings, 'LBL_ALL_INVITED'));
		$row_result->fields['caldav_tasks']['options'] = array('' => array_get_default($mod_strings, 'LBL_ASSIGNED_TO_ME'), 'visible' => array_get_default($mod_strings, 'LBL_ALL_VISIBLE'));
		
		$data['mail_sendtype'] = $this->getUserPreference('mail_sendtype');
		if ($data['mail_sendtype'] != 'SMTP')
			$data['mail_sendtype'] = 'sendmail';

		$prefs = $this->getPreferencesList();

		$cleanup_ages = array('trash_cleanup_age', 'inbox_cleanup_age', 'inbox_assoc_cleanup_age','sent_cleanup_age', 'sent_assoc_cleanup_age');
		foreach ($cleanup_ages as $a) {
			$row_result->fields[$a]['options'] = array_get_default($mod_strings, 'email_cleanup_options');
			$data[$a] = $this->getUserPreference($a);
			$data[$a] = empty($data[$a]) ? 0 : $data[$a];
		}

		foreach ($prefs as $field => $pref) {
			if (is_numeric($field))
				$field = $pref;
			if (isset($data[$field]))
				continue;
			$data[$field] = $this->getUserPreference($pref);
			$data[$field] = is_null($data[$field]) ? '' : $data[$field];
			if ($field === 'google_access_token' && is_array($data[$field])) {
				$data[$field] = json_encode($data[$field]);
			}
		}
		$data['email_display_format'] = $data['email_display_format'] == 'html' ? 'html' : 'plain';
		$data['email_compose_format'] = $data['email_compose_format'] == 'html' ? 'html' : 'plain';

		foreach (array('google_calendar_meeting', 'google_calendar_call', 'google_calendar_task',) as $a) {
			if ($data[$a] == 'on') $data[$a] = '1';
			if ($data[$a] == 'off') $data[$a] = '0';
			if ($data[$a] == '0') $data[$a] = '';
		}

		$data['dropbox_auth_status'] =  translate($this->getUserPreference('dropbox_token') ? 'LBL_DROPBOX_AUTH_STATUS_TRUE' : 'LBL_DROPBOX_AUTH_STATUS_FALSE', 'Users');

		array_update($row_result->row, $data);
		$this->form_gen->formatResult($row_result);

		if ($this->perform !== 'view') {
			global $current_language, $mod_strings, $pageInstance;
			$mod_strings = is_array($mod_strings) ? $mod_strings : [];
			$id_explanation = array_get_default($mod_strings, 'LBL_GOOGLE_CALENDAR_ID_EXPLANATION');
            $google_id_image = "include/language/images/$current_language/google-cal-id.gif";
            if (! file_exists($google_id_image))
                $google_id_image = "include/language/images/en_us/google-cal-id.gif";
			
			$id_explanation .= '<br /><a href="' .$google_id_image. '" target="_blank" class="tabDetailViewDFLink">' . array_get_default($mod_strings, 'LBL_GOOGLE_CALENDAR_ID_SHOW') . '</a>';
			$row_result->formatted['google_calendar_id_desc'] = $id_explanation;
		}

		if ($this->getLayout()->getType() == 'record') {
			if ($this->getUserPreference('dav_password'))
				$html = '<span class="text-success">' . array_get_default($mod_strings, 'LBL_DAV_PASSWORD_SET') . '</span>';
			else
				$html = '<span class="text-draft">' . array_get_default($mod_strings, 'LBL_DAV_PASSWORD_NOT_SET') . '</span>';
			$row_result->formatted['dav_password_explanation'] = $html;
		}
	}

	function getUserPreference($name)
	{
        return  UserPreference::getPreference($name, 'global', $this->user);
	}

	function setUserPreference($name, $value)
	{
        UserPreference::setPreference($name, $value, 0, 'global', $this->user);
	}

	function getHolidays() {
        global $mod_strings;
		$mod_strings = is_array($mod_strings) ? $mod_strings : [];

		$holidays = array('' => array_get_default($mod_strings, 'LBL_DEFAULT_HOLIDAYS'));
		$files = glob('modules/Calendar/holidays/*.holidays.php');
		foreach ($files as $file) {
			$HOLIDAYS = ConfigParser::load_file($file);
			$m = array();
			preg_match('~[/\\\\]([^/\\\\]+)\.holidays\.php$~', $file, $m);
			$country = $m[1];
			$holidays[$country] =  $HOLIDAYS['name_default'];

			$variants_ = array_get_default($HOLIDAYS, 'variants', array_get_default($HOLIDAYS, 'variants_default', []));
			$variants = [];

			foreach ($variants_ as $k => $v) {
				$variants['|' . $k] = ' (' . $v . ')';
			}

			if (empty($variants)) {
				$variants = ['' => ''];
				foreach (array_keys($holidays) as $k) {
					if (strpos($k, $country . '|') === 0) {
						$variants = [];
					}
				}
			} else {
				unset($holidays[$country]);
			}

			foreach ($variants as $k => $v) {
				if (isset($HOLIDAYS['lang']['LBL_NAME'])) {
					$holidays[$country . $k] =  $HOLIDAYS['lang']['LBL_NAME'] . $v;
				} else {
					$holidays[$country . $k] =  $HOLIDAYS['name_default'] . $v;
				}
			}
		}
		return $holidays;
	}

	function getHoursList() {
        $hours_list = array();

        for($h = 0; $h < 24; $h += 0.5)
            $hours_list[(string)$h] = format_decimal_time($h);

        return $hours_list;
    }

	function performUpdate() {
		$ret = parent::performUpdate();
		if ($this->perform == 'save') {
			$data =& $_REQUEST;
			$preferences = $this->getPreferencesList();
			$user = $this->loadUser($this->row_update->orig_row['id']);
			foreach($preferences as $key => $pref) {
				if(is_int($key)) $key = $pref;
				if(isset($data[$key])) {
					if ($key == 'carddav_contacts_which') {
						$old = $this->getUserPreference('carddav_contacts_which');
						if ($old != $data[$key])
							$this->setUserPreference('carddav_pref_updated', gmdate('Y-m-d H:i:s'));
					}
					if ($key == 'caldav_meetings' || $key == 'caldav_calls' || $key == 'caldav_tasks') {
						$old = $this->getUserPreference($key);
						if ($old != $data[$key])
							$this->setUserPreference('caldav_pref_updated', gmdate('Y-m-d H:i:s'));
					}

					if($key == 'hide_tabs') {
						$hide_tabs = array_filter(explode('^,^', $data[$key]));
						$data[$key] = $hide_tabs;
						AppConfig::invalidate_cache('modinfo', true);
					}

					if ($key == 'dav_password_edit') {
						$dav_password = $data[$key];
						$data[$key] = '';
						if (strlen($dav_password)) {
							$this->setUserPreference('dav_password', sha1($dav_password));
						}
					}
					if ($key == 'dropbox_auth_token' && ! dropbox_lib_disabled()) {
						$authToken = $data[$key];
						$data[$key] = '';
						if ($authToken) {
							try {
								$provider = new DropboxAuthProvider(false);
								$accessToken = $provider->getAccessToken('authorization_code', ['code' => $authToken]);
								$token = $accessToken->jsonSerialize();
							} catch(Exception $e) {
								$GLOBALS['log']->error('Dropbox init error: '.$e->getMessage());
								$token = null;
							}
							$this->setUserPreference('dropbox_token', $token);
						}
					}

					if ($key == 'google_auth_token') {
						$authToken = $data[$key];
						$data[$key] = '';
						if ($authToken) {
							require_once 'include/GoogleSyncClient.php';

							$cl = GoogleSyncClient::create($this->user);
							try {
								$cl->authenticate($authToken);
								try {
									$service = new Google_Service_Oauth2($cl->getClient());
									$token = $cl->getClient()->getAccessToken();
									$result = $service->tokeninfo(array('access_token' => $token['access_token']));
									$google_sync_user_id = $this->getUserPreference('google_sync_user_id');
									$new_user_id = $result->getUserId();
									if ($google_sync_user_id && ($google_sync_user_id != $new_user_id)) {
										$this->reset_sync_history();
									}
									$this->setUserPreference('google_sync_user_id', $new_user_id);
								} catch (Exception $e) {
									pr2($e->getMessage());
								}
							} catch (Exception $e) {
								$GLOBALS['log']->error('Google auth error: '.$e->getMessage());
							}
						}
					}
					if ($key == 'google_calendar_id') {
						$old_id = $this->getUserPreference($key);
						if ($old_id !== $data[$key]) {
							$this->reset_sync_history('Meetings', 'google_calendar');
							$this->reset_sync_history('Calls');
							$this->reset_sync_history('Tasks');
						}
					}
					$this->setUserPreference($pref, $data[$key]);
				}

			}

			if(!empty($data['gtm_forget_token'])) {
				require_once('include/OnlineMeetings/OnlineMeetingsFactory.php');
				try {
					$onlineMeeting = OnlineMeetingsFactory::getInstance();
					$onlineMeeting->resetAccessToken();
				} catch (OnlineMeetingException $e) {
					$GLOBALS['log']->error("Can't instance Online Meeting object: {$e->getMessage()}");
				}
			}

			$this->setUserPreference('google_calendar_user', '');
			$this->setUserPreference('google_domain', '');
			$this->setUserPreference('google_calendar_pass', '');
			$this->user->savePreferencesToDB();
		}
		return $ret;
	}

	function reset_sync_history($type = null, $config_key = null) {
		global $db;
		$q = "DELETE FROM google_sync WHERE user_id='{$this->user->id}'";
		if ($type) {
			$q .= " AND related_type = '$type'";
		}
		$db->query($q);
		if ($config_key)
			$this->setUserPreference($config_key . '_reset_time', time());
		$this->setUserPreference('google_sync_reset_time', time());
	}


	function loadUpdateRequest(RowUpdate &$update) {
		parent::loadUpdateRequest($update);
		$input =& $_REQUEST;
		if ($update->new_record) {
			$password = '';
			if (isset($input['password'])) {
				$password = $input['password'];
			} elseif (isset($input['user_hash'])) {
				$password = $input['user_hash'];
			}
			if ($password)
				$update->set(array('user_hash' => strtolower(md5($password))));
            if (isset($input['is_admin']) && $input['is_admin'] == 1) {
                $access_upd = array('is_group' => 0, 'portal_only' => 0);
                $update->set($access_upd);
            }
		}
	}

	function getPreferencesList()
	{
		$fields = array(
			'default_address_format',
			'dateformat' => 'default_date_format', 'timeformat' => 'default_time_format',
			'timezone', 'currency', 'default_number_format',
			'default_locale_name_format',
			'default_export_charset', 'default_email_charset',
			'trash_cleanup_age', 'inbox_cleanup_age', 'inbox_assoc_cleanup_age','sent_cleanup_age', 'sent_assoc_cleanup_age',
			'default_export_format',
			'show_reminders', 'reminder_time',
			'file_download_mode',
            'signature_id' => 'signature_default',
			'use_real_names',
			'user_holidays',
			'mail_fromname', 'mail_fromaddress',
			'mail_autobcc_address',
			'signature_prepend',
			'email_link_type',
            'out_of_office',
            'email_display_format', 'email_compose_format',
            'show_images',
			'email_sound_disable',
			'mail_sendtype',
			'mail_smtpserver', 'mail_smtpport', 'mail_smtpauth_req',
			'mail_smtpsecurity', 'mail_smtpuser', 'mail_smtppass', 'mail_smtp_verification',
			'calendar_publish_key',
			'google_calendar_id', 'google_restrict_since', 'google_calendar_direction',
			'google_calendar_meeting', 'google_calendar_call', 'google_calendar_task',
			'google_docs_direction', 'google_docs_spreadsheet',
			'google_docs_doc', 'google_docs_presentation',
			'google_contacts_direction', 'google_contacts_which',
			'dav_password_edit',
			'carddav_contacts_which', 'carddav_create_accounts',
			'caldav_meetings', 'caldav_calls', 'caldav_tasks',
			'financial_information',
			'noncatalog_products', 'nonstandard_prices',
            'manual_discounts', 'standard_discounts',
			'product_costs',
			'gtm_login', 'gtm_password',
			'dropbox_auth_token',
			'google_auth_token',
			'ringcentral_number', 'ringcentral_ext', 'ringcentral_number_direct', 'ringcentral_press_one', 'ringcentral_expand',
			'mobile_calls',
            'hide_tabs',
            'max_tabs',
            'max_subtabs',
            'swap_last_viewed',
            'swap_shortcuts',
            'navigation_paradigm',
			'show_footer_module_links',
			'o365_auth_token' => 'o365_token',
			'google_access_token',
		);
		$extra = AppConfig::setting('display.fields.User');
		foreach ($extra as $f => $d) {
			if (!empty($d['is_user_pref']))
				$fields[] = $f;
		}
		return $fields;
	}

	function showError() {
        sugar_die("Unauthorized access to administration.");
    }

	function isAdmin() {
        return AppConfig::is_admin();
    }

    function isCurrent() {
        return $this->user->id == AppConfig::current_user_id();
    }

	function loadLayoutTemplate() {
		global $mod_strings;
		parent::loadLayoutTemplate();
		if($this->form_def['type'] == 'record' || ($this->form_def['type'] == 'editview' && $this->form_def['name'] == 'Integration')) {
			if ( ($ddmsg = dropbox_lib_disabled()) ) {
				$sec =& $this->findLayoutSection('dropbox');
				$sec['vdesc'] = translate($ddmsg, 'Users');
				$sec['elements'] = array();
			}
		}

		$this->loadUser($this->record_id);
    }

	private function &findLayoutSection($id) {
		$null = null;
		foreach ($this->form_def['sections'] as &$sec) {
			if ($sec['id'] == $id)
				return $sec;
		}
		return $null;
	}

	private function &findLayoutElement($sec_id, $el_name) {
		$null = null;
		$sec =& $this->findLayoutSection($sec_id);
		if (!$sec)
			return $null;
		foreach ($sec['elements'] as &$el) {
			if (is_array($el) && $el['name'] == $el_name)
				return $el;
			else if ($el == $el_name)
				return $el;
		}
		return $null;
	}

}
