<?php

use Psr\Log\LogLevel;

if ( ! defined('sugarEntry') || ! sugarEntry) {
	die('Not A Valid Entry Point');
}

// longest action interval before we stop a session
$timeout_interval = AppConfig::setting('site.user.activity.timeout', 15) * 60;
// assume they're still working for this long:
$grace_interval = AppConfig::setting('site.user.activity.grace', 5) * 60;
$discard_interval = $timeout_interval - $grace_interval;

global $db, $current_user, $log;
/** @var $db DBManager */
/** @var $current_user USER */
/** @var $log \Katzgrau\KLogger\Logger */

$tTracking = $current_user->tracking_table_name;
$tReporting = $current_user->reporting_table_name;
$tDaily = $current_user->reporting_daily_table_name;

$log->setLogLevelThreshold(Loglevel::DEBUG);

$res = $db->query("DELETE FROM $tReporting WHERE date_entered >= DATE(DATE_SUB(NOW(),INTERVAL 1 DAY))");
$res = $db->query("SELECT UNIX_TIMESTAMP(MAX(end_time)) max FROM $tReporting");
if ($res) {
	$res = $db->fetchByAssoc($res);
}
$last_reported = ($res) ? $res['max'] : 0;
$start = 0;

$insert = [];
$work = [];

while (true) {
	$req = "SELECT user_id, UNIX_TIMESTAMP(access_time) `access_time`, `action`, `related_type`, `related_id` " .
		"FROM $tTracking WHERE access_time > FROM_UNIXTIME('$last_reported') ORDER BY access_time LIMIT $start, 1000";
	if ( ! $res = $db->query($req)) {
		return false;
	}
	$count = $db->getAffectedRowCount();
	$start += $count;

	if ( ! $count) {
		break;
	}

	while ($row = $db->fetchByAssoc($res)) {
		$id = $row['user_id'];
		if (isset($work[$id]) && ($work[$id]['end_time'] + $discard_interval) < $row['access_time']) {
			// fixate user work session:
			$insert[] = $work[$id];
			unset($work[$id]);
		}
		if ( ! isset($work[$id])) {
			$work[$id] = $row + ['actions' => 0, 'modules' => 0, 'records_read' => 0, 'records_written' => 0];
		}
		if ($row['action'] == "Logout") {
			$work[$id]['end_time'] = $row['access_time'];
		} else {
			$work[$id]['end_time'] = $row['access_time'] + $grace_interval;
		}

		$work[$id]['actions'] ++;
		if ($row['related_type']) {
			$work[$id]['modules'] ++;
		}
		if ($row['related_id']) {
			$work[$id]['records_read'] ++;
		}
		if ($row['action'] == 'save') {
			$work[$id]['records_written'] ++;
		}

		if ($row['action'] == "Logout") {
			// force fixate user work session:
			$insert[] = $work[$id];
			unset($work[$id]);
		}
	}
};

$insert = array_merge($insert, $work);
unset($work);

if (is_array($insert) && sizeof($insert) > 0) {
	$SQL = "INSERT INTO $tReporting (id, date_entered, user_id, end_time, duration, actions, " .
		"records_read, records_written )\nVALUES\n";

	foreach ($insert as $key => $row) {
		foreach ($row as &$value) {
			$value = $db->quote($value);
		}
		unset($value);
		$guid = create_guid();
		$duration = (int) (($row['end_time'] - $row['access_time'] + 30) / 60);
		if ( ! $duration) {
			$duration = 1;
		}
		$insert[$key]['duration'] = $duration;
		$SQL .= "('$guid',FROM_UNIXTIME(" . $row['access_time'] . "),'" . $row['user_id'] .
			"',FROM_UNIXTIME(" . $row['end_time'] . ")," . $duration . "," .
			$row['actions'] . "," . $row['records_read'] . "," . $row['records_written'] . "),\n";
	}

	$SQL[strlen($SQL) - 2] = ';';
	$res = $db->query($SQL);
	$rows = $db->getAffectedRowCount();
}

$SQL = "";
$daily = [];

foreach ($insert as $row) {
	$unique = $row['user_id'] . date('Y-m-d', $row['access_time']);
	if (isset($daily[$unique])) {
		foreach (['duration', 'actions', 'records_read', 'records_written'] as $key) {
			$daily[$unique][$key] += $row[$key];
			$daily[$unique]['end_time'] = $row['end_time'];
		}
	} else {
		$daily[$unique] = $row;
	}
}
unset($insert);

if (is_array($daily) && sizeof($daily) > 0) {
	$SQL = "INSERT INTO $tDaily (id, date_entered, user_id, end_time, duration, actions, " .
		"records_read, records_written )\nVALUES\n";
	foreach ($daily as $row) {
		foreach ($row as &$value) {
			$value = $db->quote($value);
		}
		unset($value);
		$guid = create_guid();
		$SQL .= "('$guid',FROM_UNIXTIME(" . $row['access_time'] . "),'" . $row['user_id'] .
			"',FROM_UNIXTIME(" . $row['end_time'] . ")," . $row['duration'] . "," .
			$row['actions'] . "," . $row['records_read'] . "," . $row['records_written'] . "),\n";
	}
	$SQL[strlen($SQL) - 2] = ';';
}
$res = $db->query("DELETE FROM $tDaily WHERE date_entered >= DATE(DATE_SUB(NOW(),INTERVAL 1 DAY))");

if ($SQL) {
	$res = $db->query($SQL);
	$rows = $db->getAffectedRowCount();
	$log->info("INSERTED {$rows} row(s) into {$tDaily}");
}

// Keep User tracking 1mo, Session Reporting 3 months, User Daily Reporting 1 year
$db->query("DELETE FROM $tTracking WHERE access_time < DATE(DATE_SUB(NOW(),INTERVAL 1 MONTH))");
$db->query("DELETE FROM $tReporting WHERE date_entered < DATE(DATE_SUB(NOW(),INTERVAL 3 MONTH))");
$db->query("DELETE FROM $tDaily WHERE date_entered < DATE(DATE_SUB(NOW(),INTERVAL 1 YEAR))");
