<?php
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');
/**
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 */



class SugarAuthenticateUser {
	var $invalid_credentials;
	var $auth_error;

	/**
	 * Does the actual authentication of the user and returns an id that will be used 
	 * to load the current user (loadUserOnSession)
	 *
	 * @param STRING $name
	 * @param STRING $password
	 * @return STRING id - used for loading the user
	 */
	function authenticateUser($name, $password, $params=null)
	{
		global $db;
		$name = trim($name);
		$password = trim($password);
		if(! strlen($name) || ! strlen($password)) return false;
		if(! is_array($params)) $params = array();
		$qname = $db->quote($name);
		$query = "SELECT id,user_hash from users where user_name = '$qname' AND IFNULL(portal_only, 0) = 0 AND status = 'Active' AND NOT deleted";
		$result = $db->query($query, false);
		if( ($row = $db->fetchByAssoc($result, -1, false)) ) {
			$check = $this->checkPassword($password, $row['user_hash'], $params);
			if($check) {
				if($check === 'rehash')
					$this->updatePassword($name, $password);
				return $row['id'];
			}
		}
		return '';
	}
	
	
	function checkPassword($password, $hash, $params=null) {
		$native_hash = function_exists('password_hash');
		if(! is_array($params)) $params = array();
		if(AppConfig::setting('site.security.md5_hashed_passwords') || ! $native_hash) {
			if (! empty($params['encoded_password']) && $hash == $password)
				// old style md5-hashed passwords used by the SOAP interface
				return true;
			if ($hash == strtolower(md5($password)))
				return true;
		} else {
			if ($hash == strtolower(md5($password)))
				// support old md5 hashes, but change hashing as soon as one is found
				return 'rehash';
			if (password_verify($password, $hash)) {
				// check for older algorithm / cost
				if (password_needs_rehash($hash, PASSWORD_DEFAULT))
					return 'rehash';
				return true;
			}
		}
		return false;
	}
    
	
	/**
	 * this is called when a user logs in 
	 *
	 * @param STRING $name
	 * @param STRING $password
	 * @return boolean
	 */
	function loadUserOnLogin($name, $password, $params = null) {
		global $log;
		$log->debug("Starting user load for $name");

		$user_id = $this->authenticateUser($name, $password, $params);
		if (empty($user_id))
			return false;

		$ret = $this->loadUserOnSession($user_id);
		if (!$ret) return false;

		return true;
	}


	/**
	 * Update a user's stored password
	 * @param STRING $username
	 * @param STRING $password
	 * @return boolean
	 */
	public function updatePassword($username, $password, $params=null) {
		global $db;
		$username = trim($username);
		$password = trim($password);
		if(! strlen($username) || ! strlen($password)) return false;
		if(! is_array($params)) $params = null;
		$newHash = null;
		if(! AppConfig::setting('site.security.md5_hashed_passwords') && function_exists('password_hash')) {
			$newHash = password_hash($password, PASSWORD_DEFAULT);
			if($newHash === false)
				$newHash = null;
			$newHash = trim($newHash);
		}
		if(! isset($newHash))
			$newHash = strtolower(md5($password));
		$qname = $db->quote($username);
		$qpass = $db->quote($newHash);
		$now = gmdate('Y-m-d H:i:s');
		$uid = $db->quote(AppConfig::current_user_id());
		$pw_change = array_get_default($params, 'pw_change');
		if(isset($pw_change))
			$requpd = ", need_new_password='".($pw_change ? 1 : 0)."'";
		else
			$requpd = '';
		$query = "UPDATE users SET user_hash='$qpass', date_modified='$now', modified_user_id='$uid', login_attempts=0 $requpd WHERE user_name='$qname' AND NOT deleted";
		$res = $db->query($query, false);
		if($db->getAffectedRowCount($res))
			return true;
		return false;
	}
	
	
	/**
	 * Loads the current user bassed on the given user_id 
	 *
	 * @param STRING $user_id
	 * @return boolean
	 */
	function loadUserOnSession($user_id='') {
		global $current_user;
		if(!empty($user_id)) {
			$_SESSION['authenticated_user_id'] = $user_id;
			$_SESSION['Users_user_id'] = "Users:$user_id";
		}
		
		if(!empty($_SESSION['authenticated_user_id']) || !empty($user_id)) {
			$current_user = new User();
			if($current_user->retrieve($_SESSION['authenticated_user_id'])) {
				if($current_user->status != 'Active') {
					$current_user->authenticated = false;
					unset($_SESSION['authenticated_user_id']);
					unset($_SESSION['Users_user_id']);
					$current_user = null;
					return false;
				}
				else {
					//$current_user->update_access_time();
					$GLOBALS['trigger_user_login'] = true;
				}		
				unset($_SESSION['captcha']);
				return true;
			}
		}
		return false;
		
	}

}

?>
