<?php

if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');

$json_supported_actions['guess_timezone'] = array();

function json_guess_timezone()
{
	global $timedate;
	$offset = array_get_default($_REQUEST, 'userOffset');
	if(isset($offset)) $offset *= 60;
	$tz = $timedate->guessTimeZone($offset);
	$ret = array('timezone' => $tz);
	json_return_value($ret);
}

$json_supported_actions['get_mfa_recovery_codes'] = array();

function json_get_mfa_recovery_codes()
{
	action_restricted_for('demo');
	require_once('include/MultiFA/MultiFactorAuthenticator.php');
	$user_id = array_get_default($_REQUEST, 'user_id');

	$user = new User();
	$user->id = $user_id;

	$auth = new \OneCRM\MultiFA\MultiFactorAuthenticator($user, $GLOBALS['log']);
	$codes = $auth->generateRecoveryCodes();

	$success = true;
	$error = '';

	if (empty($codes)) {
		$success = false;
		$error = translate('LBL_MULTI_FA_RECOVERY_CODES_ERROR', 'Users');
	}

	$ret = ['success' => $success, 'error' => $error, 'codes' => $codes];
	json_return_value($ret);
}

$json_supported_actions['get_mfa_qr_code'] = array();

function json_get_mfa_qr_code()
{
	action_restricted_for('demo');
	require_once('include/MultiFA/MultiFactorAuthenticator.php');
	$user_id = array_get_default($_REQUEST, 'user_id');

	$user = new User();
	$user->id = $user_id;

	$auth = new \OneCRM\MultiFA\MultiFactorAuthenticator($user, $GLOBALS['log']);
	$uri = $auth->getQRCodeUri();

	$success = true;
	$error = '';

	if (empty($uri)) {
		$success = false;
		$error = translate('LBL_MULTI_FA_QR_CODE_ERROR', 'Users');
	}

	$ret = ['success' => $success, 'error' => $error, 'qr_code_uri' => $uri];
	json_return_value($ret);
}

$json_supported_actions['get_mfa_secret_code'] = array();

function json_get_mfa_secret_code()
{
	action_restricted_for('demo');
	require_once('include/MultiFA/MultiFactorAuthenticator.php');
	$user_id = array_get_default($_REQUEST, 'user_id');

	$user = new User();
	$user->id = $user_id;

	$auth = new \OneCRM\MultiFA\MultiFactorAuthenticator($user, $GLOBALS['log']);
	$secret = $auth->getSecret();

	$success = true;
	$error = '';

	if (empty($secret)) {
		$success = false;
		$error = translate('LBL_MULTI_FA_TEXT_CODE_ERROR', 'Users');
	}

	$ret = ['success' => $success, 'error' => $error, 'code' => $secret];
	json_return_value($ret);
}

$json_supported_actions['verify_mfa_code'] = array();

function json_verify_mfa_code()
{
	action_restricted_for('demo');
	require_once('include/MultiFA/MultiFactorAuthenticator.php');
	$user_id = array_get_default($_REQUEST, 'user_id');
	$code = array_get_default($_REQUEST, 'code');

	$user = new User();
	$user->id = $user_id;

	$auth = new \OneCRM\MultiFA\MultiFactorAuthenticator($user, $GLOBALS['log']);
	$result = $auth->isCodeValid($code);

	if ($result) {
		$auth->enable();
		setcookie('check_2fa', '0', time() + (86400), "/");
	}

	$ret = ['success' => $result];
	json_return_value($ret);
}

$json_supported_actions['disable_mfa'] = array();

function json_disable_mfa()
{
	action_restricted_for('demo');
	global $authController;
	require_once('include/MultiFA/MultiFactorAuthenticator.php');
	$user_id = array_get_default($_REQUEST, 'user_id');
	$password = array_get_default($_REQUEST, 'password');

	$result = false;

	if($user_id == AppConfig::current_user_id() || AppConfig::is_admin()) {

		$query_result = ListQuery::quick_fetch('User', AppConfig::current_user_id(), ['user_name']);
		$current_username = '';

		if (! $query_result->failed)
			$current_username = $query_result->getField('user_name');

		if( (! empty($current_username) && ! empty($password)) && $authController->testAuthenticate($current_username, $password)) {
			$user = new User();
			$user->id = $user_id;

			$auth = new \OneCRM\MultiFA\MultiFactorAuthenticator($user, $GLOBALS['log']);
			$auth->disable();

			$result = true;
		}
	}

	$ret = ['success' => $result];
	json_return_value($ret);
}

$json_supported_actions['reset_dav_tokens'] = array();

function json_reset_dav_tokens()
{
	require_once 'include/utils/vcard_utils.php';
	require_once 'include/utils/vcal_utils.php';
	$user_id = array_get_default($_REQUEST, 'user_id');

	$user = new User();
	$user->id = $user_id;
	$result = false;
	$message = "LBL_RESET_DAV_TOKENS_ERROR";

	if ($user_id == AppConfig::current_user_id() || AppConfig::is_admin()) {
		vCardUtils::set_resync_value(true, $user);
		vCalUtils::set_resync_value(true, $user);
		$result = true;
		$message = "LBL_RESET_DAV_TOKENS_SUCCESS";
	}

	$ret = ['success' => $result, 'message' => translate($message, "Users")];
	json_return_value($ret);
}
