<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


function user_password_recovery($user_name, $key=null) {
	global $db, $log, $current_user, $authController;
	$u_q = $db->quote($user_name);

	$lq = new ListQuery('User', true);
	$lq->addField('full_name');
	$lq->setFormatted('html', 'email');
	$lq->addFilterClause("user_name='$u_q' OR email1='$u_q'");
	$lq->addSimpleFilter('status', 'Active');
	$lq->addSimpleFilter('portal_only', '0');
	$user = $lq->runQuerySingle();
	
	if(! $user) {
		$log->debug("Password recovery failed: user $user_name not found");
		return false;
	}
	$uid = $user->getField('id');
	$email = $user->getField('email1');
	$timeout = AppConfig::setting('site.security.recovery_timeout_hours');
	if(! $email) {
		$log->error("Password recovery failed: user $user_name has no primary email");
		return false;
	}
	
	$old_user = $current_user;
	$current_user = new User();
	$current_user->retrieve($uid);
	$upd = RowUpdate::for_result($user, $uid);

	$found = false;
	if(! empty($key)) {
		$key = trim($key);
		if($user->getField('recovery_key') === $key) {
			$rtime = $user->getField('recovery_key_time');
			if($rtime && strtotime($rtime) + $timeout*3600 > time()) {
				$upd->set('recovery_key', null);
				$upd->set('need_new_password', 1);
				$found = true;
			}
		}
		if(! $found) {
			$current_user = $old_user;
			return false;
		}
	} else {
		$recovery_key = User::create_recovery_key();
		$recovery_time = gmdate('Y-m-d H:i:s');
		$req_ip = array_get_default($_SERVER, 'REMOTE_ADDR');
		$upd->set('recovery_key', $recovery_key);
		$upd->set('recovery_key_time', $recovery_time);
		$upd->set('recovery_ip_addr', $req_ip);
	}
	
	if(! $upd->validate() || ! $upd->save()) {
		$log->error("Password recovery failed: user $user_name could not be updated");
		$current_user = $old_user;
		return false;
	}
	
	if($found) {
		$_SESSION['authenticated_user_id'] = $user->getPrimaryKeyValue();
		$_SESSION['allow_reset_password'] = 1;
		if($authController->postLogin($user_name) === false) {
			session_destroy();
			$current_user = $old_user;
			return false;
		}
		return true;
	}

	$current_user = $old_user;
	
	require_once('include/Notify/RecordActionMessage.php');
	$msg = RecordActionMessage::for_action('User', 'recover');
	$msg->display_name = 'full_name';
	$msg->title = ['LBL_PASSWORD_RECOVERY', 'Users'];
	$msg->subject = ['LBL_PASSWORD_RECOVERY', 'Users'];
	$msg->fields = ['user_name', 'recovery_key', 'recovery_timeout'];
	$msg->format_hooks[] = ['function' => 'format_recovery_timeout'];
	$msg->buttons['recover'] = [
		'label' => ['LBL_RECOVER_NOW_BUTTON', 'Users'],
		'url' => 'login.php?action=recover&user_name='.$user_name.'&key='.$recovery_key,
		'class' => 'active',
	];
	$msg->addRecordId($uid);

	$mailer = $msg->getMailer();
	$mailer->addTargetInfo(array('name' => $user->getField('full_name'), 'email' => $email));

	$cur_dir = getcwd();
	register_shutdown_function(function() use ($mailer, $msg, $cur_dir, $user_name) {
		global $log;
		chdir($cur_dir);
		$mailer->sendMessage($msg);
		$log->debug("Password recovery message sent to user $user_name");
	});
	
	return true;
}

function format_recovery_timeout($result, $fmt) {
	if(! $result || ! $result->rows) return;
	$timeout = AppConfig::setting('site.security.recovery_timeout_hours');
	$f_timeout = str_replace('{HOURS}', $timeout, $fmt->translate('LBL_RECOVERY_TIMEOUT_TEXT', 'Users'));
	if($fmt->getFormat() == 'html')
		$f_timeout = to_html($f_timeout);
	$result->fields['recovery_timeout'] = ['name' => 'recovery_timeout', 'vname' => 'LBL_RECOVERY_TIMEOUT', 'type' => 'varchar', 'source' => ['type' => 'non-db']];
	foreach($result->rows as $idx => &$r) {
		$r['recovery_timeout'] = $timeout;
		$result->formatted_rows[$idx]['recovery_timeout'] = $f_timeout;
		$result->formatted_rows[$idx]['recovery_key'] = array_get_default($r, 'recovery_key');
	}
}

