<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

use League\OAuth2\Client\Token\AccessToken;

require_once('include/layout/forms/FormField.php');
require_once('include/utils/oauth_utils.php');

class GoogleAuthInput extends FormField {

	var $token_input_id;

	function init($params=null, $model=null) {
		$this->token_input_id = uniqid("google_token_input_");
		return parent::init($params, $model);
	}


	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		$html = "";
		$encoded_token = $row_result->getField($this->name);
		$status = $this->tokenStatus($encoded_token);
		$success_message = to_html($status['success'] ?? "");
		$error_message = to_html($status['error'] ?? "");
		if ($gen->getLayout()->getDefaultEditable()) {
			$authUrl =  buildOAuth2Uri('google_sync', ['input' => $this->token_input_id]);

			$html = '<input type="hidden" id="' . $this->token_input_id . '" name="' . $this->name . '" value="' . to_html($encoded_token)  .  '">';
			$html .= '&nbsp;<button type="button" onclick="open_popup_window({url: \'' . $authUrl . '\'}); return false;" class="input-button google_signin"></button>';
		}
		$html .= '<div style="color: green" id="' . $this->token_input_id . '_success_message">' . $success_message . '</div>';
		$html .= '<div style="color: red" id="' . $this->token_input_id . '_error_message">' . $error_message . '</div>';


		return $html;
	}

	protected function tokenStatus($encoded_token) {
		$token = null;
		$token_array = @json_decode($encoded_token, true);
		if ($token_array) {
			try {
				$t = new AccessToken($token_array);
				if ($t->hasExpired()) {
					$expires = $t->getExpires();
					// Assuming access token expiration is 1 hour, and refresh token expiration is 90 days
					if ($expires + 89 * 24 * 60 * 60 <= time())  {
						return ['error' => translate('LBL_O365_AUTH_EXPIRED_TOKEN', 'Users')];
					}
				}
				$token = $t;
			} catch (Exception $e) {
				pr2($e->getMessage());
			}
		}
		if (!$token) {
			return ['error' => translate('LBL_O365_AUTH_INVALID_TOKEN', 'Users')];
		}
		return ['success' => translate('LBL_O365_AUTH_VALID_TOKEN', 'Users')];
	}

}
?>
