<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2020 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

require_once('include/layout/forms/FormElement.php');
require_once('include/MultiFA/MultiFactorAuthenticator.php');

class MultiFactorAuthWidget extends FormElement {

	/**
	 * @var bool
	 */
	private bool $isPage = false;

	/**
	 * @param $v
	 * @return void
	 */
	public function setIsPage($v): void {
		$this->isPage = $v;
	}

	/**
	 * @param HtmlFormGenerator $gen
	 * @param $row_result
	 * @param array $parents
	 * @param array $context
	 *
	 * @return string
	 */
	public function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if (empty($this->name))
			return '';

		if ($this->name == 'multi_fa_description') {
			$value = "<div class='form-desc' style=''>".translate('LBL_MULTI_FA_DESCRIPTION', 'Users')."</div>";
		} else {
			if($this->getEditable($context))
				$value = $this->renderHtmlEdit($gen, $row_result, $context);
			else
				$value = $this->renderHtmlView($gen, $row_result, $context);
		}

		return $value;
	}

	/**
	 * @return array
	 */
	public function getRequiredFields() {
		return array('id');
	}

	/**
	 * @param HtmlFormGenerator $gen
	 * @param RowResult $row_result
	 * @param array $context
	 *
	 * @return string
	 */
	public function renderHtmlEdit(HtmlFormGenerator $gen, RowResult $row_result, array $context) {
		$user = new User();
		$user->id = $row_result->getField('id');

		$html = $this->getStatus($user);

		$form_layout = $gen->getLayout();

		if ($this->isCurrent($user) || ($this->isEnabled($user) && $this->isAdmin())) {
			/** @var FormLayout $form_layout */
			$form_layout->addScriptInclude('modules/Users/MultiFASetup.js');
			$html .= '&nbsp;&nbsp;' . $this->getButton($user);
		}

		if ($this->isPage) {
			$form_layout->addScriptInclude('modules/Users/MultiFAPageSetup.js');
		}


		return $this->isPage ? '' : $html;
	}

	/**
	 * @param HtmlFormGenerator $gen
	 * @param RowResult $row_result
	 * @param array $context
	 *
	 * @return string
	 */
	public function renderHtmlView(HtmlFormGenerator $gen, RowResult $row_result, array $context) {
		$user = new User();
		$user->id = $row_result->getField('id');

		return $this->getStatus($user);
	}

	/**
	 * Get Multi-Factor Auth status
	 *
	 * @param User $user
	 *
	 * @return string
	 */
	private function getStatus(User $user) {
		if ($this->isEnabled($user)) {
			$status = '<span class="text-success">'.translate('LBL_MULTI_FA_ENABLED', 'Users').'</span>';
		} else {
			$status = '<span class="text-error">'.translate('LBL_MULTI_FA_DISABLED', 'Users').'</span>';
		}

		return $status;
	}

	/**
	 * Get Enable/Reconfigure button
	 *
	 * @param User $user
	 *
	 * @return string
	 */
	private function getButton(User $user) {
		$attributes = [
			'id' => 'MultiFAEnableButton',
			'name' => 'MultiFAEnableButton',
			'type' => 'button'
		];

		if ($this->isEnabled($user)) {
			$attributes['label'] = translate('LBL_MULTI_FA_DISABLE_BUTTON', 'Users');
			$attributes['onclick'] = 'mfa_disable(this.form); return false;';
		} else {
			$attributes['label'] = translate('LBL_MULTI_FA_ENABLE_BUTTON', 'Users');
			$attributes['onclick'] = 'mfa_enable(this.form); return false;';
		}

		return EditableForm::render_button($attributes);
	}

	/**
	 * Is Multi FA enabled or not
	 *
	 * @param User $user
	 *
	 * @return bool
	 */
	private function isEnabled(User $user) {
		$auth = new \OneCRM\MultiFA\MultiFactorAuthenticator($user, $GLOBALS['log']);
		return $auth->isEnabled();
	}

	/**
	 * Checking is showing user current or not
	 *
	 * @param User $user
	 * @return bool
	 */
	private function isCurrent(User $user) {
		if ($user->id != AppConfig::current_user_id()) {
			return false;
		} else {
			return true;
		}
	}

	/**
	 * Checking is showing user current or not
	 *
	 * @return bool
	 */
	private function isAdmin() {
		if (! AppConfig::is_admin()) {
			return false;
		} else {
			return true;
		}
	}
}
