<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormField.php');

class ProjectRateInput extends FormField {

	function init($params=null, $model=null) {
        parent::init($params, $model);
	}

	function getRequiredFields() {
        $fields = array('~join.' . $this->name);

        if ($this->name == 'hourly_cost') {
            $additional = array('~join.hourly_cost_custom', '~join.hourly_cost_usdollar');
            $fields = array_merge($fields, $additional);
        }

		return $fields;
	}

    function renderListValue(ListFormatter $fmt, ListResult $result, $row_id, $list_params=null) {
        $task_id = $result->rows[$row_id]['~join.projecttask_id'];

        if ($this->name == 'hourly_cost') {
            $cost_custom = $result->rows[$row_id]['~join.hourly_cost_custom'];

            if ($cost_custom) {
                $hourly_cost = $result->rows[$row_id]['~join.hourly_cost_usdollar'];
            } else {
                $hourly_cost = User::get_hourly_rate($row_id);
            }
            $val = User::format_cost($task_id, $hourly_cost);
            $fval = User::format_cost($task_id, $hourly_cost, false);
        } else {
            $val = $result->formatted_rows[$row_id]['~join.'.$this->name];
            $fval = $result->rows[$row_id]['~join.'.$this->name];
        }

        $fields = array('estim_hours' => array('type' => 'input', 'format' => 'float'),
            'hourly_cost' => array('type' => 'input', 'format' => 'currency'),
            'booking_status' => array('type' => 'list', 'options' => $result->fields['~join.booking_status']['options'],
                'default' => $result->fields['~join.booking_status']['default']));

        $params = array(
        	'id' => $this->name . '-' . $row_id,
        	'module' => $list_params['module'],
            'record' => array_get_default($list_params, 'parent_key'),
            'link_name' => array_get_default($list_params, 'link_name'),
            'target_id' => $row_id,
            'user_id' => $row_id,
            'task_id' => $task_id,
            'field' => $this->name,
            'list_id' => $this->list_id,
            'type' => $fields[$this->name]['type'],
            'callback' => 1,
            'callback_method' => 'save_assigned_resource_data',
        );

        if ($fields[$this->name]['type'] == 'list') {
            $form = new EditableForm('SubPanel', 'AssignedResources');
            $spec = array('options' => $fields[$this->name]['options'], 'options_add_blank' => false);
            $opts = $form->getSelectOptions($dom, $spec, '', '');
            $params['options'] = $opts;
            $params['default_val'] = $fields[$this->name]['default'];
            $params['selected'] = $result->rows[$row_id]['~join.'.$this->name];
        } else {
            $params['format'] = $fields[$this->name]['format'];
			if ($this->name == 'hourly_cost') {
				$params['show_reset'] = true;
			}
			$params['value'] = $fval;
        }

        $json = getJSONobj();
        $json_params = $json->encode($params);
        //$onclick = 'quickEdit('.$json_params.');'; (custom method in modules/Users/pt_popup.js)
        $attrs = [
        	'class' => 'input-field flat' . (! empty($arrow) ? ' rbullet' : ''),
        	'id' => $params['id'],
        	'onclick' => 'SUGAR.ui.quickEdit('.$json_params.', event);',
        ];
        $html = EditableForm::get_tag('div', $attrs);
        $attrs = ['class' => 'input-label', 'id' => $params['id'].'-label'];
        $html .= EditableForm::get_tag('div', $attrs);
		$html .= $val . '</div></div>';

        return $html;
    }
}
