<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2020 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

class ResetDavTokensWidget extends FormElement {

	/**
	 * @param HtmlFormGenerator $gen
	 * @param $row_result
	 * @param array $parents
	 * @param array $context
	 *
	 * @return string
	 */
	public function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if(empty($this->name))
			return '';

		if($this->getEditable($context))
			$value = $this->renderHtmlEdit($gen, $row_result, $context);
		else
			$value = "";

		return $value;
	}

	/**
	 * @return array
	 */
	public function getRequiredFields() {
		return array('id');
	}

	/**
	 * @param HtmlFormGenerator $gen
	 * @param RowResult $row_result
	 * @param array $context
	 *
	 * @return string
	 */
	public function renderHtmlEdit(HtmlFormGenerator $gen, RowResult $row_result, array $context) {
		$user = new User();
		$user->id = $row_result->getField('id');

		$html = "";

		if ($this->isCurrent($user) || $this->isAdmin()) {
			$html .= $this->getButton($user);
		}

		return $html;
	}


	/**
	 * Get Reset button
	 *
	 * @param User $user
	 *
	 * @return string
	 */
	private function getButton(User $user) {
		$attributes = [
			'id' => 'DavResetTokensButton',
			'name' => 'DavResetTokensButton',
			'type' => 'button'
		];

		$attributes['label'] = translate('LBL_RESET_DAV_TOKENS_BUTTON', 'Users');
		$attributes['onclick'] = 'reset_dav_tokens(this.form); return false;';

		return EditableForm::render_button($attributes);
	}

	/**
	 * Checking is showing user current or not
	 *
	 * @param User $user
	 * @return bool
	 */
	private function isCurrent(User $user) {
		if ($user->id != AppConfig::current_user_id()) {
			return false;
		} else {
			return true;
		}
	}

	/**
	 * Checking is showing user current or not
	 *
	 * @return bool
	 */
	private function isAdmin() {
		if (! AppConfig::is_admin()) {
			return false;
		} else {
			return true;
		}
	}
}
