<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormTableSection.php');
require_once 'include/config/format/ConfigParser.php';

define('HOLYDAYS_SCAN', true);

class UserInfoWidget extends FormTableSection {

    /**
     * @var string - 'view' or 'edit'
     */
    var $context;

    /**
     * @var User
     */
    var $user;

    /**
     * @var FieldFormatter
     */
	var $formatter;

	var $pdfMode = false;

	var $pdf;
	
    var $top_sections = array( 'acl_settings');

    const PREFERENCES_CATEGORY = 'global';

    const DEFAULT_MAIL_SENDTYPE = 'sendmail';

    const DEFAULT_SMTP_PORT = 25;

	function init($params=null, $model=null) {
        $this->user = new User();
        $this->setUserId(array_get_default($_REQUEST, 'record'));
		parent::init($params, $model);
	}
	
    /**
     *
     * @param  string $id
     * @return void
     */
    function setUserId($id) {
        if ($this->user instanceof User)
            $this->user->id = $id;
    }


    /**
     * @param HtmlFormGenerator $gen
     * @return void
     */
    function setFormatter(FormGenerator $gen) {
    	$ltype = $gen->getLayout()->getType();
    	if($ltype == 'record') $ltype = ($gen->getLayout()->getDefaultEditable() ? 'editview' : 'view');
        $this->formatter = new FieldFormatter($gen->format, $ltype);
    }

	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if($gen->getLayout()->getEditingLayout()) {
			$params = array();
			return $this->renderOuterTable($gen, $parents, $context, $params);		
		}

        $this->setUserId($row_result->getField('id'));
        $this->setFormatter($gen);
		//$this->formatter->formatRowResult($row_result);

		if($gen->getLayout()->getDefaultEditable()) {
            $this->context = 'edit';
			if (!$this->isAdmin() && !$this->isCurrent())
				$this->showError();
			return $this->renderHtmlEdit($gen, $row_result);
		} else {
            $this->context = 'view';
            return $this->renderHtmlView($gen, $row_result);
        }
	}
	
	function getRequiredFields() {
        $fields = array('in_directory', 'receive_notifications',
            'week_start_day', 'day_begin_hour', 'day_end_hour', 'email1', 'email2',
            'is_admin', 'portal_only', 'photo_filename', 'user_name');

		return $fields;
	}
	
	function renderHtmlView(HtmlFormGenerator $gen, RowResult $row_result) {
        $style = '';

        if (! in_array($this->id, $this->top_sections))
            $style = 'margin-top: 0.5em;';
        if($this->tab && $gen->getLayout()->getCurrentTab() !== $this->tab)
			$style .= 'display: none;';

		$title = to_html($this->getLabel());
        $body = '<table id="le_section_'.$this->id.'" width="100%" border="0" cellspacing="0" cellpadding="0" class="tabDetailView" style="'.$style.'">';
        $body .= '<tr><th class="tabDetailViewDL" colspan="4" align="left"><h4 class="tabDetailViewDL">'.$title.'</h4></th></tr>';

        $body .= $this->getBody($gen, $row_result);

        $body .= "</table>";

        return $body;
	}

	function renderHtmlEdit(HtmlFormGenerator $gen, RowResult $row_result) {
        $layout = array_get_default($_REQUEST, 'layout', 'Standard');
        if ($row_result->new_record && $layout != 'Standard')
            $this->showError();

        $form_layout = $gen->getLayout();
        $form_layout->addScriptInclude('modules/Users/User.js', LOAD_PRIORITY_BODY);
        if ($this->id == 'user_settings')
            $form_layout->addScriptLiteral("enable_change_password_button();", LOAD_PRIORITY_FOOT);

        $style = '';

        if (! in_array($this->id, $this->top_sections))
            $style = 'margin-top: 0.5em;';
        if($this->tab && $gen->getLayout()->getCurrentTab() !== $this->tab)
        	$style .= 'display: none;';

		$title = to_html($this->getLabel());
        $body = '<table id="le_section_' . $this->id . '" width="100%" border="0" cellspacing="0" cellpadding="0" class="tabForm" style="'.$style.'">';
        $body .= '<tr><th class="dataLabel" colspan="4" width="100%" align="left"><h4 class="dataLabel">'.$title.'</h4></th></tr>';

        $body .= $this->getBody($gen, $row_result);

        $body .= "</table>";

        return $body;
	}

    function getBody(FormGenerator $gen, RowResult $row_result) {
        $body = '';
        
        if($this->id != 'user_settings' && $this->id != 'user_photo' && ! $this->isAdmin() && ! $this->isCurrent()) {
			$this->showError();
		}

        switch ($this->id) {
            case "acl_settings":
                if ($this->isAdmin()) {
                    $body .= $this->renderAcl($gen, $row_result);
                } else {
                    $this->showError();
                }
                break;
            default:
                break;
        }

        return $body;
    }

    /**
     * Get User preference value
     *
     * @param  string $name - preference name
     * @return mixed
     */
    function getPreference($name) {
        return  UserPreference::getPreference($name, UserInfoWidget::PREFERENCES_CATEGORY, $this->user);
    }

    /**
     * Set new value for user preference
     *
     * @param string $name - preference name
     * @param mixed $value
     * @return void
     */
    function setPreference($name, $value) {
        UserPreference::setPreference($name, $value, 0, UserInfoWidget::PREFERENCES_CATEGORY, $this->user);
    }

    /**
     * Checking is admin current user or not
     *
     * @return bool
     */
    function isAdmin() {
        if (!AppConfig::is_admin()) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * Checking is viewing user current or not
     * 
     * @return bool
     */
    function isCurrent() {
        if ($this->user->id != AppConfig::current_user_id()) {
            return false;
        } else {
            return true;
        }
    }

    /**
     * Show unauthorized access error message
     *
     * @return void
     */
    function showError() {
        sugar_die("Unauthorized access to administration.");
    }

	function loadUpdateRequest(RowUpdate $update, array $input) {
        if (!$this->isAdmin() && !$this->isCurrent())
            $this->showError();

        $related_data = $update->getRelatedData('settings_rows');

        if (! $related_data) {
            $upd_user = array('data' => $input);
            $update->setRelatedData('settings_rows', $upd_user);
        }
	}
	
	function validateInput(RowUpdate $update) {
        if ($update->new_record) {
            $password = $_REQUEST['password'];
            $confirm_password = $_REQUEST['confirm_password'];
            $message = '';

            if (empty($password)) {
                $message = 'ERR_ENTER_NEW_PASSWORD';
            } elseif (empty($confirm_password)) {
                $message = 'ERR_ENTER_CONFIRMATION_PASSWORD';
            } elseif ($password != $confirm_password) {
                $message = 'ERR_PASSWORD_MISMATCH';
            }

            if ($message)
                throw new IAHError(translate($message, 'Users'));
        }
        return true;
    }
	
	function afterUpdate(RowUpdate $update) {
        global $app_list_strings;

		$row_updates = $update->getRelatedData('settings_rows');
		if(! $row_updates)
			return;

        $update->setRelatedData('settings_rows', null);

        if (! $update->new_record) {
		    $user_id = $update->getField('id', null, true);
            $user_name = $update->getField('user_name', null, true);
            $name = $update->getField('name', null, true);
        } else {
            $user_id = $update->saved['id'];
            $user_name = $update->saved['user_name'];
            $name = array_get_default($update->saved, 'name');
        }
        $this->setUserId($user_id);
        $this->user->user_name = $user_name;
        $this->user->name = $name;

        $data = $row_updates['data'];

		if (!empty($data['google_contacts_direction']))
			$data['google_contacts_direction'] = 'to_google';

        if(! empty($data['ut'])) {
        	// prompt for timezone
        	$this->setPreference('ut', 0);
		}

        if( isset($data['email_link_type']) && isset($app_list_strings['dom_email_link_type'][$data['email_link_type']]) ) {
            $this->setPreference('email_link_type', $data['email_link_type']);
        }

        $this->user->savePreferencesToDB();
	}


    /**
     * Render and show email signatures popup
     *
     * @static
     * @param DetailManager $mgr
     * @param string $record
     * @param string $user_id
     * @return bool
     */
    static function renderSignaturePopup(DetailManager $mgr, $record = '', $user_id = null) {
        $mgr->module = 'Users';
        $mgr->action = 'SaveSignature';
        $mgr->layout_name = 'Signature';
        $mgr->form_name = 'SignatureForm';
        $mgr->in_popup = true;
        $mgr->perform = 'edit';
        $mgr->model = new ModelDef('UserSignature');
        $mgr->record_id = $record;
        $mgr->editLayout = false;

		$mgr->standardInit();
		$buttons = $mgr->getStandardButtons();
		$buttons['save']['params'] = array('no_redirect' => true, 'close_popup' => true);
		if (!empty($record)) {
	        $buttons['delete'] = array(
		        'vname' => 'LBL_DELETE',
			    'type' => 'button',
                'icon' => 'icon-delete',
				'params' => array('no_redirect' => true, 'close_popup' => true, 'reload_after' => 0, 'record_perform' => 'delete'),
			);
		}
		$mgr->layout->setFormButtons($buttons);

        if ($user_id) {
            $sign_user_id = $user_id;
        } elseif ($mgr->record->getField('user_id')) {
            $sign_user_id = $mgr->record->getField('user_id');
        } else {
            $sign_user_id = AppConfig::current_user_id();
        }

        $hidden = array('return_module' => 'Users', 'return_action' => 'EditView', 'layout'=> 'Email',
            'return_record' => $sign_user_id, 'user_id' => $sign_user_id);
        $mgr->layout->addFormHiddenFields($hidden, false);
        $mgr->form_gen->form_obj->addHiddenFields($hidden);

        $nextAction = $mgr->performUpdate();
        if($nextAction)
            return $nextAction;

        global $pageInstance;
        $t = $mgr->getPageTitle();
        if(strlen($t))
            $pageInstance->set_title($t);

        echo $mgr->renderLayout();
    }
		
	function renderPdf(PdfFormGenerator $gen, $row_result, array $parents, array $context)
	{
        $this->setUserId($row_result->getField('id'));
        $this->setFormatter($gen);
		$lstyle = $gen->getLayout()->getType();

        if (!$this->isAdmin() && !$this->isCurrent())
			$this->showError();

		$this->context = 'view';
		$this->pdfMode = true;
		$this->pdf = $gen->pdf;
		$this->getBody($gen, $row_result);
	}


}
?>
