<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormElement.php');

class UserSignatureInput extends FormElement {	
	var $params;
	
	function init($params=null, $model=null) {
		parent::init($params, $model);
		if(is_array($params))
			$this->params = $params;
	}
	
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if(empty($this->name))
			return '';
		if($this->getEditable($context))
			$value = $this->renderHtmlEdit($gen, $row_result, $context);
		else
			$value = $this->renderHtmlView($gen, $row_result, $context);
		return $value;
	}
	
	function getRequiredFields() {
		return array('id');
	}
	
	function renderHtmlEdit(HtmlFormGenerator $gen, RowResult $row_result, array $context) {
		$spec = array(
			'name' => $this->name,
			'options' => $this->getSignatures($row_result),
			'onchange' =>  'setSigEditButtonVisibility();',
		);
        $user = new User();
        $user->id = $row_result->getField('id');
		$selected = UserPreference::getPreference('signature_default', 'global', $user);
		if (empty($selected))
			$selected = '';
        $select = $gen->form_obj->renderSelect($spec, $selected);
		
		$signature_buttons = $user->getSignatureButtons();

		global $pageInstance;
		$pageInstance->add_js_literal("setSigEditButtonVisibility();", null, LOAD_PRIORITY_FOOT);
		return $select . '&nbsp;' . $signature_buttons;
	}
	
	function renderHtmlView(HtmlFormGenerator $gen, RowResult $row_result, array $context) {
		$options = $this->getSignatures($row_result);
		$selected = $row_result->getField($this->name);
		return array_get_default($options, $selected);
	}
	
	function getSignatures(RowResult $row_result) {
        global $app_strings;
        $fields = array('id', 'name');
        $lq = new ListQuery('UserSignature', $fields);
        $lq->addSimpleFilter('user_id', $row_result->getField('id'));
        $result = $lq->fetchAll('name');
        $signatures = array('' => $app_strings['LBL_NONE']);

        if (! $result->failed) {
            $signatures_list = $result->rows;

            foreach($signatures_list as $id => $data) {
                $signatures[$id] = $data['name'];
            }

        }
        return $signatures;
    }
	
	
}
