function vacationInit(form) {
	checkUserAccess(form);
	SUGAR.ui.attachFormInputEvent(form, 'leave_type', 'onchange', setStatus);
	//SUGAR.ui.attachFormInputEvent(form, 'days', 'onchange', function() { calcEndDate(this.form); });
	SUGAR.ui.attachFormInputEvent(form, 'date_start', 'onchange', afterStartDateUpdate);
	SUGAR.ui.attachFormInputEvent(form, 'date_end', 'onchange', function() { checkEndDate(this.form); });
	SUGAR.ui.attachFormInputEvent(form, 'assigned_user', 'onchange', function() { checkUserAccess(this.form); });
}

function setStatus() {
	var type = this.getValue();
    var status_select = SUGAR.ui.getFormInput(this.form, 'status');

    if (type == 'vacation') {
        status_select.setValue('planned');
    } else if (type == 'sick') {
        status_select.setValue('days_taken');
    }
}

function checkEndDate(form) {
    var start_date = SUGAR.ui.getFormInput(form, 'date_start').getValue(true);
    var end_date = SUGAR.ui.getFormInput(form, 'date_end').getValue(true);

    if (start_date) {
        if (end_date >= start_date) {
            calcDays(start_date, end_date, form);
        } else {
            SUGAR.ui.getFormInput(form, 'date_end').clear();
        }
    }
}

function afterStartDateUpdate() {
    var end_date = SUGAR.ui.getFormInput(this.form, 'date_end').getValue(true);
    var days = SUGAR.ui.getFormInput(this.form, 'days').getValue(true);

    if (end_date) {
        checkEndDate(this.form);
    } else if (days) {
        calcEndDate(this.form);
    }
}

function calcEndDate(form) {
	var days = SUGAR.ui.getFormInput(form, 'days').getValue(true);
	var start_date = SUGAR.ui.getFormInput(form, 'date_start').getValue(true);
	
    if (days > 0 && start_date) {
        var end = new Date(start_date);
        end.setDate(end.getDate() + Math.floor(days - 0.01));

        var end_date_input = SUGAR.ui.getFormInput(form, 'date_end');
        if (end_date_input)
            end_date_input.setValue(end, true);
    }
}

function isWeekend(date1, date2) {
    var d1 = new Date(date1),
        d2 = new Date(date2),
        isWeekend = false,
		weekend_days = 0;

    while (d1 < d2) {
        var day = d1.getDay();
        isWeekend = (day === 6) || (day === 0);
        if (isWeekend) { weekend_days++; } // return immediately if weekend found
        d1.setDate(d1.getDate() + 1);
    }
    return weekend_days;
}

function calcDays(start_date, end_date, form) {
    var result = '';

    var weekend_days = isWeekend(start_date, end_date);

    if (end_date >= start_date)
        result = 1 + Math.round((end_date.getTime() - start_date.getTime()) / 1000 / 60 / 60 / 24);

    result = result-weekend_days;

    var days_input = SUGAR.ui.getFormInput(form, 'days');
    if (days_input)
        days_input.setValue(result, true);
}

function checkUserAccess(form) {
	var assigned_user = SUGAR.ui.getFormInput(form, 'assigned_user'),
		status = SUGAR.ui.getFormInput(form, 'status'),
		record = document.getElementsByName('record')[0].value;
	if(assigned_user) {
		var req = new SUGAR.conn.JSONRequest('get_reporting_users', {}, {
			user_id: assigned_user.getKey(),
			module: 'Vacations',
			status: status.getValue()
		});
	} else {
		var req = new SUGAR.conn.JSONRequest('get_reporting_users', {}, {
			record: record,
			module: 'Vacations',
			status: ''
		});
	}

	if(record == '' && status != undefined) {
		status.menu.removeOption('submitted');
		status.menu.removeOption('cancelled');
	}

	req.fetch(function(data) {
		if(!data.result.can_assign) {
			alert(data.result.message);
			if(assigned_user)
				assigned_user.update(data.result.id, data.result.name);
		}

		if(!data.result.can_approve_self && status != undefined) {
			status.menu.removeOption('approved');
		}

		if(data.result.role == 'current_user' && status != undefined) {
			status.menu.removeOption('approved');
			status.menu.removeOption('not_approved');
			status.menu.removeOption('days_taken');
		}

		if(data.result.role == 'current_user' && data.result.status == 'submitted' && status != undefined) {
			status.menu.removeOption('planned');
			status.menu.removeOption('cancelled');
		}
	});
}
