<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


require_once 'data/SugarBean.php';
require_once 'modules/Webhooks/WebhookFormulaParser.php';
require_once 'modules/Webhooks/WebhookFormulaEvaluator.php';
require_once 'modules/Webhooks/WebhookVarEvaluator.php';


class Webhook extends SugarBean {

	// Stored fields
  	var $id;
	var $model;
	var $url;
	var $type;
	var $deleted;

	var $table_name = 'webhooks';
	var $object_name = 'Webhook';
	var $module_dir = 'Webhooks';
	var $new_schema = true;

	private static $BY_TYPE = [];
	private static $HOOKS = [];
	private static $PARSED = [];
	private static $registered;
	private static $handles = [];

	static function has_pending_calls() {
		return !empty(self::$handles);
	}

	static function load_hooks($model_name) {
		$json = getJSONobj();
		$lq = new ListQuery('Webhook');
		$lq->addSimpleFilter('model', $model_name);
		$hooks = $lq->fetchAllRows();
		self::$BY_TYPE[$model_name] = ['create' => [], 'update' => []];
		foreach ($hooks as $hook) {
			$hook['filters'] = $json->decode($hook['filters']);
			self::$HOOKS[$hook['id']] = $hook;
			if($hook['type'] == 'create_update' || $hook['type'] == 'create')
				self::$BY_TYPE[$model_name]['create'][] = $hook['id'];
			if($hook['type'] == 'create_update' || $hook['type'] == 'update')
				self::$BY_TYPE[$model_name]['update'][] = $hook['id'];
		}
	}

	static function after_save(&$rowUpdate) {
		$model_name = $rowUpdate->model_name;
		if (!isset(self::$BY_TYPE[$model_name][$model_name])) {
			self::load_hooks($model_name);
		}
		$type = $rowUpdate->new_record ? 'create' : 'update';
		foreach (self::$BY_TYPE[$model_name][$type] as $hook_id) {
			self::process_hook($rowUpdate, self::$HOOKS[$hook_id]);
		}
	}

	static function is_applicabe($rowUpdate, $hook) {
		$field = self::get_owner_field($rowUpdate);
		$modelName = $rowUpdate->model_name;
		$module = AppConfig::module_for_model($modelName);
		return ACLAction::userHasAccess($hook['assigned_user_id'], $module, 'view', 'module', $rowUpdate->getField($field), true);
	}

	static function process_formula($rowUpdate, $hook) {
		$filters = $hook['filters'];
		$conditions = $filters['conditions'];
		$formula = array_get_default($filters, 'formula');
		if (!strlen(trim($formula)))
			return false;
		$varEvaluator = new WebhookVarEvaluator($hook['assigned_user_id'], $rowUpdate, $conditions);
		if (!isset(self::$PARSED[$hook['id']])) {
			self::$PARSED[$hook['id']] = WebhookFormulaParser::parse($formula);
		}
		list($success, $parsed) = self::$PARSED[$hook['id']];
		if (!$success) return false;
		$evaluator = new WebhookFormulaEvaluator($parsed, true);
		return $evaluator->evaluate($varEvaluator);
	}

	static function process_simple($rowUpdate, $hook) {
		$filters = $hook['filters'];
		$conditions = $filters['conditions'];
		$evaluator = new WebhookVarEvaluator($hook['assigned_user_id'], $rowUpdate, $conditions);
		$at_least_one = false;

		if(is_array($conditions)) {
			foreach (array_keys($conditions) as $idx) {
				$result = $evaluator($idx);
				if ($result === null) continue;
				$at_least_one = true;
				if ($result && $filters['glue'] != 'and') return true;
				if (!$result && $filters['glue'] == 'and') return false;
			}
		}
		return ($filters['glue'] == 'and') && $at_least_one;
	}

	static function process_hook($rowUpdate, $hook) {
		if (!self::is_applicabe($rowUpdate, $hook))
			return;
		if ($hook['filters']['glue'] == 'formula') {
			$matched = self::process_formula($rowUpdate, $hook);
		} else {
			$matched = self::process_simple($rowUpdate, $hook);
		}
		if ($matched) {
			self::schedule_call($rowUpdate, $hook);
		}
	}

	static function schedule_call($rowUpdate, $hook) {
		$json = getJSONobj();
		if (!self::$registered) {
			self::$registered = true;
			register_shutdown_function([__CLASS__, 'send_calls']);
		}
		$model_name = $rowUpdate->model_name;
		$fields = \AppConfig::setting("model.fields.$model_name");
		$data = ListQuery::quick_fetch($model_name, $rowUpdate->getPrimaryKeyValue(), array_keys($fields), ['process_results' => true, 'zzzz' => true]);
		if (!$data) return;

		$module = \AppConfig::module_for_model($model_name);

		require_once('modules/TaxCodes/TaxCode.php');
		$seedcode = new \TaxCode();

		$invoice = &$data->row;

		$ordering_modules = [
			'Quotes', 'Invoice', 'Bills', 'SalesOrders', 'PurchaseOrders', 'CreditNotes'
		];

		$adj_field = [
			'Quotes' => 'quote_id',
			'Invoice' => 'invoice_id',
			'Bills' => 'bills_id',
			'SalesOrders' => 'sales_orders_id',
			'PurchaseOrders' => 'purchase_orders_id',
			'CreditNotes' => 'credit_id',
		];

		if($module == 'Payments') {
			$relinv_lq = new \ListQuery('invoices_payments');
			$relinv_lq->addSimpleFilter('payment_id', $invoice['id']);
			$invoice['related_invoices'] = $relinv_lq->fetchAllRows();
			foreach($invoice['related_invoices'] as &$pmt_inv) {
            	$invoice_data = \ListQuery::quick_fetch_row('Invoice', $pmt_inv['invoice_id']);
                $pmt_inv['integrated_id'] = $invoice_data['integrated_id'];
			}
		}

		if (in_array($module, $ordering_modules)) {
			$lq_tax = new \ListQuery($model_name.'Adjustment');
			$lq_tax->addSimpleFilter($adj_field[$module], $invoice['id'], '=');
			$lq_tax->addSimpleFilter('related_type', 'TaxRates', '=');
			$tax_result = $lq_tax->fetchAllRows();
			$invoice['total_tax'] = 0;
			foreach($tax_result as $tax_row) {
				$invoice['total_tax'] += $tax_row['amount'];
			}

			$lq_tax = new \ListQuery($model_name.'Adjustment');
			$lq_tax->addSimpleFilter($adj_field[$module], $invoice['id'], '=');
			$lq_tax->addSimpleFilter('related_type', 'Discounts', '=');
			$tax_result = $lq_tax->fetchAllRows();
			$invoice['total_discount'] = 0;
			foreach($tax_result as $tax_row) {
				$invoice['total_discount'] += $tax_row['amount'];
			}

			$lq_tax = new \ListQuery($model_name.'Adjustment');
			$lq_tax->addSimpleFilter($adj_field[$module], $invoice['id'], '=');
			$lq_tax->addSimpleFilter('related_type', 'ShippingProviders', '=');
			$tax_result = $lq_tax->fetchAllRows();
			$invoice['total_shipping'] = 0;
			foreach($tax_result as $tax_row) {
				$invoice['total_shipping'] += $tax_row['amount'];
			}
			require_once('include/ListView/ListFormatter.php');
			require_once('include/api/src/classes/OneCRM/API/Endpoint.php');
			require_once('include/api/src/classes/OneCRM/API/Error.php');
			require_once('include/api/src/classes/OneCRM/API/Errors/InternalError.php');
			require_once('include/api/src/classes/OneCRM/API/Endpoints/Data/ListEndpoint.php');

			$rel_fmt = new \ListFormatter($model_name, 'lines_link', null, 'listview', ['parent_key' => $invoice['id']]);
			$rel_lq =& $rel_fmt->getQuery();
			$rel_lq->addFields(array('id', 'line_group_id', 'name', 'quantity', 'mfr_part_no', 'serial_no', 'cost_price', 'list_price', 'unit_price', 'ext_price', 'tax_class_id'));
			$rel_result = $rel_fmt->getQueryResult(0, 999);
			\OneCRM\API\Endpoints\Data\ListEndpoint::filterExposedFields($rel_result->rows, $rel_result->getModelName());
			$invoice['line_items'] = array_values($rel_result->rows);
			foreach($invoice['line_items'] as &$line) {
				$rates = $seedcode->get_tax_rates($line['tax_class_id'], false);
				$rates_arr = [];
				foreach($rates as $k => $v) {
					$rates_arr[] = $v;
				}
				$lq_tax = new \ListQuery($model_name.'Adjustment');
				$lq_tax->addSimpleFilter('line_id', $line['id'], '=');
				$lq_tax->addSimpleFilter('type', 'PercentDiscount', '=');
				$disc_result = $lq_tax->fetchAllRows();
				$line['line_discount_amount'] = 0;
				$line['line_discount_perc'] = 0;
				foreach($disc_result as $disc_row) {
					$line['line_discount_amount'] += number_format(($line['ext_price']*$disc_row['rate'])/100, 2, '.', '');
					$line['line_discount_perc'] = $disc_row['rate'];
					$invoice['total_discount'] += $line['line_discount_amount'];
				}
				$line['line_tax_perc'] = $rates_arr[0]['rate'];
				$line['line_tax_amount'] = number_format(($line['ext_price']*$rates_arr[0]['rate'])/100, 2, '.', '');
			}
		}

		foreach($data->row as &$row) {
			if($row == '0000-00-00 00:00:00' || $row == '0000-00-00')
				$row = null;
		}

		$ch = curl_init($hook['url']);
		curl_setopt_array($ch, [
			CURLOPT_HTTPHEADER => ['Content-Type: application/json'],
			CURLOPT_POSTFIELDS => $json->encode($data->row),
			CURLOPT_RETURNTRANSFER => true,
		]);
		self::$handles[] = $ch;
	}

	static function send_calls() {
		end_flush_all_buffers();
		flush();
		foreach (self::$handles as $ch) {
			curl_exec($ch);
		}
		self::$handles = [];
	}

	static function before_save(&$rowUpdate) {
		if ($rowUpdate->orig_row) {
			$rowUpdate->setRelatedData('webhookUpdate', array_diff_assoc($rowUpdate->updates, $rowUpdate->orig_row));
			$rowUpdate->setRelatedData('webhookOrig', $rowUpdate->orig_row);
		} else {
			$rowUpdate->setRelatedData('webhookUpdate', $rowUpdate->updates);
			$rowUpdate->setRelatedData('webhookOrig', []);
		}
	}

	static function get_owner_field($rowUpdate) {
		$modelName = $rowUpdate->model_name;
		if($modelName == 'User')
			return 'id';
		else if ( ($nm = AppConfig::setting("model.detail.$modelName.assigned_user_id")) )
			return $nm;
		else if (AppConfig::setting("model.fields.$modelName.assigned_user_id"))
			return 'assigned_user_id';
		elseif (AppConfig::setting("model.fields.$modelName.created_by"))
			return 'created_by';
	}
}

