<?php

require_once __DIR__ . '/WebhookFormulaParser.php';

class WebhookFormulaEvaluator {

	private $valid;
	private $parsed;

	public function __construct($formula, $parsed = false) {
		if ($parsed)
			list($this->valid, $this->parsed) = [true, $formula];
		else
			list($this->valid, $this->parsed) = WebhookFormulaParser::parse($formula);
	}

	private function evaluate_binary($node, $valuesProvider) {
		$value = $this->evaluateNode($node['lhs'], $valuesProvider);
		if ($value && ($node['operator'] == 'or')) return true;
		if (!$value && ($node['operator'] != 'or')) return false;
		return $this->evaluateNode($node['rhs'], $valuesProvider);
	}

	private function evaluate_variable($node, $valuesProvider) {
		return !!$valuesProvider($node['index']);
	}

	private function evaluate_or($node, $valuesProvider) {
		if (!$this->evaluateNode($node['lhs'], $valuesProvider))
			return $this->evaluateNode($node['rhs'], $valuesProvider);
		return true;
	}

	private function evaluate_not($node, $valuesProvider) {
		return !$this->evaluateNode($node['exp'], $valuesProvider);
	}

	private function evaluateNode($node, $valuesProvider) {
		$method = 'evaluate_' . $node['type'];
		if (method_exists($this, $method)) {
			return $this->$method($node, $valuesProvider);
		} else {
			return false;
		}
	}

	public function evaluate($valuesProvider) {
		if (!$this->valid) {
			return false;
		}
		return $this->evaluateNode($this->parsed, $valuesProvider);
	}

}

