<?php

class WebhookFormulaParser {

	private static $__PARSER;

	private $formula;
	private $parser;
	const TOKEN_RE = '~(\$\d+|\band\b|\bor\b|\bnot\b|\(|\))~i';

	private function __construct($formula) {
		$this->formula = strtolower($formula);
		$this->initInstance();
	}


	private static function init() {
		if (self::$__PARSER) return;
		$_ = [];

		$defer = function($name) use (&$_) {
			return function($input) use ($name, &$_) {
				return $_[$name]($input);
			};
		};

		$map = function($parser, $transform) {
			return function($input) use ($parser, $transform) {
				list ($success, $output, $remainder) = $parser($input);
				if ($success)
					return [true, $transform($output), $remainder];
				else
					return [false, $output, $remainder];
			};
		};

		$eof = function($input) {
			if (empty($input))
				return [true, null, []];
			return [false, null, $input];
		};

		$satisfy = function($predicate) {
			return function($input) use ($predicate) {
				if (!empty($input)) {
					if ($predicate($input[0])) {
						return [true, $input[0], array_slice($input, 1)];
					}
				}
				return [false, null, $input];
			};
		};

		$str = function($s) use ($satisfy) {
			return $satisfy(function($token) use ($s) {
				return $token === $s;
			});
		};

		$re = function($e) use ($satisfy) {
			return $satisfy(function($token) use ($e) {
				return preg_match($e, $token);
			});
		};

		$lparen = $str('(');
		$rparen = $str(')');
		$var = $map($re('~^\$\d+$~'), function($token) {
			return ['type' => 'variable', 'index' => (int)substr($token, 1)];
		});

		$last = function($parsers) {
			return function($input) use ($parsers) {
				$output = null;
				foreach ($parsers as $parser) {
					list ($success, $output, $remainder) = $parser($input);
					if (!$success) return [false, $output, $remainder];
					$input = $remainder;
				}
				return [true, $output, $input];
			};
		};

		$seq = function($parsers, $combine) {
			return function($input) use ($parsers, $combine) {
				$parts = [];
				foreach ($parsers as $parser) {
					list ($success, $output, $remainder) = $parser($input);
					if (!$success) return [false, $output, $remainder];
					$parts[] = $output;
					$input = $remainder;
				}
				return [true, $combine($parts), $input];
			};
		};

		$any = function($parsers) {
			return function($input) use ($parsers) {
				foreach ($parsers as $parser) {
					list ($success, $output, $remainder) = $parser($input);
					if ($success)
						return [$success, $output, $remainder];
				}
				return [false, null, $input];
			};
		};

		$peek = function($parser) {
			return function($input) use ($parser) {
				list ($success, $output, $remainder) = $parser($input);
				return [$success, $output, $input];
			};
		};

		$exp_ = $defer('exp_');
		$exp = $defer('exp');

		$binaryOp = $any([ $str('or'), $str('and') ]);
		$notOp = $str('not');

		$parenExp = $seq([$lparen, $exp, $rparen], function($parts) {
			return $parts[1];
		});

		$unaryLogicalExp = $defer('unaryLogicalExp');

		$nonBinaryExp = $any( [$unaryLogicalExp, $var, $parenExp ]);

		$combineBinaryExp = null;
		$combineBinaryExp = function($parts) use (&$combineBinaryExp) {
			if ($parts[1]) {
				$ret = [ 'type' => 'binary', 'operator' => $parts[1][0], 'lhs' => $parts[0], 'rhs' => $parts[1][1] ];
				if ($parts[1][2]) {
					return $combineBinaryExp([$ret, $parts[1][2]]);
				}
				return $ret;
			}
			return $parts[0];
		};

		$peekRParen = $peek($rparen);
		$skipRParen = function($input) use ($peekRParen) {
			list($success, $output, $remainder) = $peekRParen($input);
			return [$success, null, $remainder];
		};

		$_['exp'] = $seq([$nonBinaryExp, $exp_], $combineBinaryExp);
		$_['exp_'] = $any([
			$seq([$binaryOp, $nonBinaryExp, $exp_], function($parts) {
				return $parts;
			}),
			$any([$eof, $skipRParen])
		]);


		$_['unaryLogicalExp'] = $seq([$notOp, $nonBinaryExp ], function($parts) {
			return ['type' => 'not', 'exp' => $parts[1] ];
		});


		self::$__PARSER = $seq([$exp, $eof], function($parts) {
			return $parts[0];
		});

	}

	private function initInstance() {
		self::init();
		$this->parser = self::$__PARSER;

	}

	public static function parse($formula) {
		$parser = new self($formula);
		return $parser->parseFormula();
	}

	private function parseFormula() {
		$tokens = preg_split ( '~(\$\d+|\band\b|\bor\b|\bnot\b|\(|\))~i' , $this->formula, -1, PREG_SPLIT_DELIM_CAPTURE | PREG_SPLIT_NO_EMPTY);
		$tokens = array_filter($tokens, function($s) {return strlen(trim($s));});
		$parser = $this->parser;
		return $parser($tokens);
	}


}
