<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');

include_once 'modules/WooCommerce/automattic_api/vendor/autoload.php';

$json_supported_actions['test_woo_connect'] = array();
function json_test_woo_connect() {
	require_once('modules/WooCommerce/wrappers/WooAuth.php');
	$url = array_get_default($_REQUEST, 'url');
	$key = array_get_default($_REQUEST, 'key');
	$secret = array_get_default($_REQUEST, 'secret');

	$woo_auth = new WooAuth($url, $key, $secret);
	$test = $woo_auth->testConnection();

	if ($test) {
		$ret = ['success' => 1, 'message' => translate('LBL_CONNECT_SUCCESS', 'WooCommerce')];
	} else {
		$ret = ['success' => 0, 'message' => $woo_auth->getErrorMessage()];
	}

	json_return_value($ret);
}

$json_supported_actions['hook_request'] = array('login_required' => false);
function json_hook_request() {
	require_once('modules/WooCommerce/wrappers/WooManager.php');
	global $db;

	$content = file_get_contents('php://input');
	$manager = new WooManager($db);
	$signature = $_SERVER['HTTP_X_WC_WEBHOOK_SIGNATURE'];
	$resource = $_SERVER['HTTP_X_WC_WEBHOOK_RESOURCE'];

	if (! $manager->isSupportedHookResource($resource)) {
		$GLOBALS['log']->fatal("WooCommerce hook handler error: unsupported resource {$resource}");
		$ret = ['message' => 'Data not updated: unsupported Resource'];
		json_return_value($ret);
		exit;
	}

	try {
		$manager->hookHandler($content, $signature, $resource);
		$message = 'Data updated';
	} catch (Exception $e) {
		$GLOBALS['log']->fatal($e->getMessage());
		$message = "Error: {$e->getMessage()}";
	}

	$ret = ['message' => $message];
	json_return_value($ret);
}

$json_supported_actions['woo_resync'] = array();
function json_woo_resync() {
	set_time_limit(0);
	require_once('modules/WooCommerce/wrappers/WooManager.php');
	global $db;
	$manager = new WooManager($db);

	if ($manager->isSyncStarted()) {
		$ret = ['success' => 0, 'message' => translate('LBL_SYNC_ALREADY_STARTED', 'WooCommerce')];
	} else {
		$result = false;

		try {
			$result = $manager->reSync();
			$add_msg = '';
		} catch (Exception $e) {
			$add_msg = $e->getMessage();
		}

		if ($result) {
			$ret = [
				'success' => 1,
				'message' => translate('LBL_SYNC_DONE', 'WooCommerce')
			];
		} else {
			$ret = [
				'success' => 0,
				'message' => translate('LBL_SYNC_ERROR', 'WooCommerce') .' '. $add_msg
			];
		}

		$manager->stopSync();
	}

	json_return_value($ret);
}

$json_supported_actions['woo_reset_status'] = array();
function json_woo_reset_status() {
	require_once('modules/WooCommerce/wrappers/WooManager.php');
	global $db;
	$manager = new WooManager($db);
	$manager->stopSync();

	$ret = [
		'success' => 1,
		'message' => translate('LBL_WOOCOMMERCE_RESET_STATUS_SUCCESS', 'WooCommerce')
	];

	json_return_value($ret);
}
