<?php

require_once('include/layout/forms/FormField.php');
require_once('modules/WooCommerce/wrappers/WooManager.php');

class WooCommerceSyncStatusWidget extends FormField {

	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		global $db;

		$url = AppConfig::site_url() . 'index.php?module=WooCommerce';
		$this->desc = str_replace('%URL%', $url, $this->desc);
		$lstyle = $gen->getLayout()->getType();

		$html = '';
		$manager = new WooManager($db);

		if($lstyle == 'editview' || ($lstyle == 'record' && $gen->getLayout()->getDefaultEditable())) {

			if (! $manager->isInitialSyncCompleted()) {
				$html = 'Not started';
			} else {

				$entries = [
					'ProductCategory' => [
						'push' => 'products_sync',
						'pull' => 'init_woo_products_sync'
					],

					'Product' => [
						'push' => 'products_sync',
						'pull' => 'init_woo_products_sync'
					],

					'Account'  => [
						'pull' => 'init_woo_customers_sync'
					],

					'SalesOrder' => [
						'pull' => '',
					],

					'Invoice' => [
						'pull' => ''
					]
				];

				foreach ($entries as $entry_name => $sync_type) {
					$label = $this->getEntryLabel($entry_name);

					if (empty($label))
						continue;

					$html .= "<div><strong>{$label}:</strong><ul>";

					foreach ($sync_type as $type => $setting) {
						$num = $manager->getSyncedNum($type, $entry_name);

						if (! empty($setting)) {
							if ($manager->getSetting($setting))
								$html .= $this->generateNumLine($num, $type);
						} else {
							$html .= $this->generateNumLine($num, $type);
						}
					}

					$html .= "</ul></div>";
				}
			}
		}

		return $html;
	}

	private function generateNumLine($num, $type) {
		if (empty($num))
			return '';

		if ($type == 'push') {
			$label = 'LBL_TO_WOO';
		} else {
			$label = 'LBL_FROM_WOO';
		}

		$translated = translate($label, 'WooCommerce');

		return  "<li style='display: inline; padding-right: 10px;'>{$translated}: {$num}</li>";
	}

	private function getEntryLabel($entry_name) {
		$labels = [
			'Product' => 'LBL_PRODUCTS',
			'ProductCategory' => 'LBL_PRODUCT_CATEGORIES',
			'Account' => 'LBL_ACCOUNTS',
			'SalesOrder' => 'LBL_SALES_ORDERS',
			'Invoice' => 'LBL_INVOICES'
		];

		$label  = isset($labels[$entry_name]) ? $labels[$entry_name]: '';
		$result = '';

		if ($label)
			$result = translate($label, 'WooCommerce');

		return $result;
	}
}
