<?php
class WooIdLocker {

	/**
	 * @var DBManager
	 */
	private $db;

	/**
	 * WooIdLocker constructor.
	 *
	 * @param DBManager $db
	 */
	public function __construct(DBManager $db) {
		$this->db = $db;
	}

	/**
	 * Lock Entry for updates
	 *
	 * @param string $resource
	 * @param int $id
	 */
	public function lock($resource, $id) {
		$this->runUpdateQuery($resource, $id, true);
	}

	/**
	 * Unlock Entry for updates
	 *
	 * @param string $resource
	 * @param int $id
	 */
	public function unlock($resource, $id) {
		$this->runUpdateQuery($resource, $id, false);
	}

	/**
	 * @param string $resource
	 * @param int $id
	 *
	 * @return bool
	 */
	public function isLocked($resource, $id) {
		$locked = false;

		if (! empty($id)) {
			$query = new ListQuery($resource);
			$query->addSimpleFilter('woo_id', $id);
			$result = $query->runQuerySingle();

			if ( ! $result->failed) {
				$locked = (bool) $result->getField('woo_locked');
			}
		}

		return $locked;
	}

	/**
	 * Uses raw query so as not change date_modified field
	 *
	 * @param string $resource
	 * @param int $id
	 * @param bool $lock
	 */
	private function runUpdateQuery($resource, $id, $lock) {
		if (! empty($id)) {
			$flag = (int)$lock;
			$table = $this->getResourceTable($resource);
			$query = "UPDATE `{$table}` SET `woo_locked` = {$flag} WHERE `woo_id` = '{$id}'";
			$this->db->query($query);
		}
	}

	/**
	 * @param string $resource
	 *
	 * @return string
	 */
	private function getResourceTable($resource) {
		$model =  new ModelDef($resource);

		return $model->getTableName();
	}
}