<?php
require_once('modules/WooCommerce/wrappers/WooAuth.php');

class WooRequest {

	/**
	 * @var \Automattic\WooCommerce\Client
	 */
	protected $client;

	/**
	 * WooEntry constructor.
	 *
	 */
	public function __construct() {
		$this->client = WooAuth::get_client();
	}

	/**
	 * Get result - GET Request
	 *
	 * @param string $endpoint: WooCommerce API endpoint
	 * @param array $params
	 *
	 * @return bool|array|stdClass
	 */
	public function get($endpoint, $params) {
		return $this->run('get', $endpoint, $params);
	}

	/**
	 * Add record - POST Request
	 *
	 * @param string $endpoint: WooCommerce API endpoint
	 * @param array $data
	 *
	 * @return bool|array|stdClass
	 */
	public function add($endpoint, $data) {
		return $this->run('post', $endpoint, $data);
	}

	/**
	 * Update record - PUT Request
	 *
	 * @param string $endpoint: WooCommerce API endpoint
	 * @param array $data
	 *
	 * @return bool|array|stdClass
	 */
	public function update($endpoint, $data) {
		return $this->run('put', $endpoint, $data);
	}

	/**
	 * Delete record - DELETE Request
	 *
	 * @param string $endpoint: WooCommerce API endpoints
	 * @param array $params
	 *
	 * @return bool|array|stdClass
	 */
	public function delete($endpoint, $params) {
		return $this->run('delete', $endpoint, $params);
	}

	/**
	 * Run WooCommerce API method
	 *
	 * @param string $method: get, post, put, delete
	 * @param string $endpoint: WooCommerce API endpoint
	 * @param array $params
	 *
	 * @return bool|array|stdClass: false if error and array or stdClass with data if success
	 */
	private function run($method, $endpoint, $params) {
		if (! $this->client) {
			$GLOBALS['log']->error("WooCommerce API Request Error: Client object wasn't initialized");
			return false;
		}

		if (! method_exists($this->client, $method)) {
			$GLOBALS['log']->error("WooCommerce API Request Error: method {$method} doesn't exists");
			return false;
		}

		try {
			$result = $this->client->$method($endpoint, $params);
		} catch (Exception $e) {
			$GLOBALS['log']->error('WooCommerce API '.strtoupper($method).' Request (' .$endpoint. ') ' . $e->getMessage());
			$result = false;
		}

		return $result;
	}
}