<?php

require_once('WooProduct.php');
require_once('WooTaxRate.php');

class WooLineItem extends WooEntry {

	/**
	 * @var WooTaxRate
	 */
	private $tax_rate;

	/**
	 * @var WooProduct
	 */
	private $product;

	/**
	 * @return WooTaxRate
	 */
	public function getTaxRate() {
		return $this->tax_rate;
	}

	/**
	 * WooLineItem constructor.
	 *
	 * @param WooRequest $request
	 */
	public function __construct(WooRequest $request) {
		parent::__construct($request);

		$this->tax_rate = new WooTaxRate($request);
		$this->product = new WooProduct($request);
	}

	public function store($record) {}

	public function storeInWoo($data) {}

	/**
	 * @param array $data
	 */
	protected function adjustDataFromWoo(&$data) {
		$data['id'] = 'temp~' . rand(1000, 2000);

		$product = $this->getProduct($data['product_id']);

		if (isset($product['id'])) {
			$data['related_id'] = $product['id'];
			$data['related_type'] = 'ProductCatalog';

			$data['cost_price'] = $product['cost'];
			$data['list_price'] = $product['list_price'];
		}

		$data['unit_price'] = $data['total'] / $data['quantity'];

		if (! empty($data['tax_rate_id'])) {
			$tax_details = $this->getTaxDetails($data['tax_rate_id']);

			if (! empty($tax_details['code_id']))
				$data['tax_class_id'] = $tax_details['code_id'];
		}

		$data['name'] = htmlspecialchars_decode($data['name']);

		unset($data['total']);
		unset($data['product_id']);
		unset($data['tax_rate_id']);
	}

	/**
	 * @param int $product_woo_id
	 *
	 * @return null|array
	 */
	private function getProduct($product_woo_id) {
		return $this->product->getByWooId($product_woo_id);
	}

	/**
	 * @param $rate_woo_id
	 *
	 * @return array
	 */
	public function getTaxDetails($rate_woo_id) {
		return $this->tax_rate->getByWooId($rate_woo_id);
	}

	/**
	 * @return array
	 */
	protected function getMap() {
		return [
			'name' => 'name',
			'product_id' => 'product_id',
			'quantity' => 'quantity',
			'total' => 'total',
			'price' => 'list_price',
			'currency_id' => 'currency_id',
			'sku' => 'mfr_part_no',

			'taxes' => [
				'id' => 'tax_rate_id'
			]
		];
	}
}