<?php
require_once('WooEntry.php');

class WooProductAttribute extends WooEntry {

	/**
	 * WooProductCategory constructor.
	 *
	 * @param WooRequest $request
	 */
	public function __construct(WooRequest $request) {
		parent::__construct($request);

		$this->model = 'ProductAttribute';

		$this->init_from_woo = $this->getSetting('init_woo_products_sync');
		$this->init_to_woo = $this->getSetting('products_sync');
		$this->updates_to_woo = $this->getSetting('products_sync');
	}

	/**
	 * @param stdClass $record
	 *
	 * @return bool|string
	 * @throws WooStoreEntryException
	 */
	public function store($record) {
		$id = $this->isExists($this->getIdentifyFilter($record));

		if ($id)
			$record->attr_id = $id;

		$data = $this->map($record);
		$user_id = $this->getAssignedUserId();

		if (isset($data['id'])) {
			$result = ListQuery::quick_fetch($this->model, $data['id']);
			$upd = RowUpdate::for_result($result, null, $user_id);

			unset($data['id']);
		} else {
			$upd = RowUpdate::blank_for_model($this->model, $user_id);
		}

		$upd->set($data);

		try {
			$result = $upd->save();
		} catch (Exception $e) {
			$result = false;
		}

		if ($result) {
			return $upd->getField('id');
		} else {
			return false;
		}
	}

	/**
	 * @param array $record
	 *
	 * @return int|null
	 */
	public function storeInWoo($record) { return null; }

	/**
	 * Adjust data from WooCommerce for compatibility with 1CRM
	 *
	 * @param array $data
	 */
	protected function adjustDataFromWoo(&$data) {
		$data['name'] = htmlspecialchars_decode($data['name']);
	}


	/**
	 * @return array
	 */
	protected function getMap() {
		return [
			'id' => 'woo_id',
			'attr_id' => 'id',
			'product_id' => 'product_id',
			'name' => 'name',
			'value' => 'value',
		];
	}


	/**
	 * @param stdClass $record
	 *
	 * @return array
	 */
	private function getIdentifyFilter($record) {
		return [
			[   'operator' => 'AND',
				'multiple' => [
					['field' => 'product_id', 'value' => $record->product_id],
					['field' => 'woo_id', 'value' => $record->id],
					['field' => 'name', 'value' => $record->name],
					['field' => 'value', 'value' => $record->value],
				]
			]
		];
	}
}
