<?php
require_once('WooOrder.php');

class WooWebHook extends WooEntry {

	/**
	 * WooOrder constructor.
	 *
	 * @param WooRequest $request
	 */
	public function __construct(WooRequest $request) {
		parent::__construct($request);

		$this->endpoint = 'webhooks';
		$this->init_to_woo = true;
	}

	public function store($record) {}

	/**
	 * @param array $data
	 *
	 * @return null
	 */
	public function storeInWoo($data) {
		$url = AppConfig::site_url() . 'json.php?module=WooCommerce&action=hook_request';
		$entries = ['order', 'customer', 'product'];
		$hooks = [];

		foreach ($entries as $entry) {
			$create_name = '1CRM '.translate('LBL_HOOK_'.strtoupper($entry).'_CREATE', 'WooCommerce');

			if (! $this->isExistsInWoo(['search' => $create_name])) {
				$hooks[] = [
					'name' => $create_name,
					'topic' => "{$entry}.created",
					'delivery_url' => $url,
					'secret' => $data['secret']
				];
			}

			$update_name = '1CRM '.translate('LBL_HOOK_'.strtoupper($entry).'_UPDATE', 'WooCommerce');

			if (! $this->isExistsInWoo(['search' => $update_name])) {
				$hooks[] = [
					'name' => $update_name,
					'topic' => "{$entry}.updated",
					'delivery_url' => $url,
					'secret' => $data['secret']
				];
			}
		}

		if (sizeof($hooks) > 0)
			$this->getRequest()->add("{$this->endpoint}/batch", ['create' => $hooks]);

		return null;
	}
}