<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');


require_once('data/SugarBean.php');
require_once('modules/Workflow/WFOperation.php');

class Workflow extends SugarBean {

	// persistent fields
	var $id;
	var $name;
	var $description;
	var $modified_user_id;
	var $assigned_user_id;
	var $created_by;
	var $date_entered;
	var $date_modified;

	var $status = 'inactive';
	var $execute_mode;
	var $occurs_when;
	var $trigger_module;
	var $trigger_action;

	var $operation_mode = 'immediately';
	var $operation_days = '0';
	var $operation_hours = '0';
	var $operation_mins = '0';
	var $operation_owner = 'trigger';

	var $criteria;
	var $criteria_clauses;
	var $criteria_fields;
	var $operations;

	// static fields
	var $table_name = "workflow";
	var $object_name = "Workflow";
	var $module_dir = "Workflow";
	var $new_schema = true;


	function __construct() {
		parent::__construct();
		global $current_language;
		$this->mod_strings = return_module_language($current_language, 'Workflow');
	}

	function lazy_load() {
		$this->get_all_wf_operations();
	}

	function validation_error($msg) {
		$this->validation_error_msg = $msg;
		return false;
	}

	// fetch operations for use by workflow widget or other user-facing interface
	static function get_operations($for_id, $module) {
		require_once('modules/Workflow/WFProcessorBase.php');
		$operations = array();
		if (! empty($for_id)) {
			$lq = new ListQuery('WFOperation');
			$lq->addSimpleFilter('workflow_id', $for_id);
			$lq->setOrderBy('display_order');
			$res = $lq->runQuery();

			foreach ($res->rows as $row) {
				$type = $row['operation_type'];
				$op = WFProcessorBase::create_module_action($module, $type);
				if($op)
					$row['label'] = $op->getLabel();
				$operations[] = $row;
			}
		}
		return $operations;
	}


	function get_all_wf_operations() {
		if ((!isset($this->operations) || empty($this->operations)) && isset($this->id)) {
			$this->operations = array();

			$lq = new ListQuery('WFOperation');
			$lq->addSimpleFilter('workflow_id', $this->id);
			$lq->setOrderBy('display_order');
			$res = $lq->runQuery();

			foreach ($res->rows as $row) {
				$opt = new WFOperation();
				foreach ($row as $k => $v) {
					$opt->$k = $v;
				}
				$this->operations[] = $opt;	
			}
		}
	}

	/*
	 * check if the workflow is applicable for the given bean/timing/action
	 */
	function is_applicable($rowUpdate, $timing, $action, $isUpdate)
	{
		global $timedate;
		$GLOBALS['log']->debug('trigger_action: '.$this->trigger_action);
		$GLOBALS['log']->debug('action: '.$action);
		$GLOBALS['log']->debug('timing: '.$timing);

		// match mode
		if (($this->execute_mode == 'newRecordOnly' && $isUpdate) || ($this->execute_mode == 'existingRecordOnly' && !$isUpdate))
			return false;


		// match action
		if ($action != $this->trigger_action) 
			return false;

		$applicable_operations = array();
		foreach ($this->operations as $idx=>$opt) {
			$perform_when = $opt->perform_when;
			if ($timing == $perform_when || ($timing == 'after' && $perform_when == 'scheduled'))
				$applicable_operations[] = $opt;
		}

		if(! $applicable_operations)
			return false;
		return $applicable_operations;
	}
	
	function validate() {
		if (empty($this->name)) {
			throw new WFException(WFException::ERR_EMPTY_NAME);
		}
		if (empty($this->status)) {
			throw new WFException(WFException::ERR_EMPTY_WF_STATUS);
		}
		if (empty($this->execute_mode)) {
			throw new WFException(WFException::ERR_EMPTY_EXEC_MODE);
		}
		if (empty($this->trigger_module)) {
			throw new WFException(WFException::ERR_EMPTY_EXEC_MODULE);
		}
		if (empty($this->trigger_action)) {
			throw new WFException(WFException::ERR_EMPTY_ACTION);
		}
		/*if (empty($this->occurs_when)) {
			throw new WFException(WFException::ERR_EMPTY_OP_MODE);
		}*/
		if (empty($this->operation_owner)) {
			throw new WFException(WFException::ERR_EMPTY_OWNER);
		}

		foreach ($this->operations as $opt) {
			$opt->validate($this->trigger_action);
		}
		return true;
	}
	
	function checkConditions($rowUpdate) {
		$ret = false;
		if($this->criteria_clauses) {
			$clauses = unserialize(from_html($this->criteria_clauses));
			if(is_array($clauses)) {
				$ret = ListQuery::eval_filter_clauses($clauses, $rowUpdate);
			}
		}
		return $ret;
	}

	static function init_record(RowUpdate &$upd, $input) {
		if (! $upd->getField('trigger_action')) $upd->set('trigger_action', 'saved');
		if (! $upd->getField('occurs_when')) $upd->set('occurs_when', 'saved');
		if (! $upd->getField('execute_mode')) $upd->set('execute_mode', 'newRecordAndExisting');
		if (! $upd->getField('operation_owner')) $upd->set('operation_owner', 'trigger');
	}

	static function get_owner_options($field_name=null, $row=null, $editable=true) {
		global $app_list_strings, $app_strings;
		$value = array_get_default($row, $field_name);
		$ret = array();
		$has_user = true;
		foreach ($app_list_strings['workflow_execute_dom'] as $k => $v) {
			if ($editable)
				$ret[$k] = array(
					'name' => $v,
					'icon' => ($k == 'admin' ? 'icon-adminuser' : 'icon-user'),
				);
			else
				$ret[$k] = $v;
			if ($k == $value)
				$has_user = false;
		}
		
		if ($editable)
			$ret['select']  = array(
				'name' => $app_strings['LBL_SELECT_USER_BUTTON_LABEL'],
				'icon' => 'input-icon icon-popup',
			);
		if ($has_user && $value) {
			if ($editable)
				$ret[$value]  = array(
					'name' => get_assigned_user_name($value),
					'icon' => 'icon-user',
				);
			else
				$ret[$value]  = get_assigned_user_name($value);
		}
		return $ret;
	}

	static function get_target_options() {
        $include_modules = array(
            'Accounts', 'Bugs', 'Bills', 'Booking', 'Calls', 'Campaigns', 'Cases', 'Contacts', 'Documents',
            'Emails', 'Forecasts', 'Leads', 'Meetings', 'Notes', 'Opportunities', 'Quotes',
            'Tasks', 'SubContracts', 'SalesOrders', 'Invoice',
			'PurchaseOrders', 'Payments', 'HR', 'Project', 'ProjectTask', 'Timesheets', 'Vacations',
			'Shipping', 'Receiving', 'EventSessions',
			'ProductCatalog', 'Assemblies', 'Assets', 'SupportedAssemblies',
			'LicenseServer', 'Subscriptions'
        );

		$beans = AppConfig::setting('modinfo.primary_beans');
        $options = array();

		foreach ($beans as $mod => $name) {
            if (in_array($mod, $include_modules)) {
                $options[$mod] = translate_dd('moduleList', $mod, $name);
            }
		}

		asort($options);
        return $options;
	}

}
?>
