<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo,
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry') || !sugarEntry) die('Not A Valid Entry Point');

require_once('modules/Workflow/Workflow.php');
require_once('modules/Workflow/WFProcessorBase.php');

class WorkflowProcessor {

	static function process(&$rowUpdate, $event, $workflow = null, $owner=null) {
		global $db;
		static $skip_mods = [
			'ActivityLog', 'LicenseInfo', 'RecordActions', 'Scheduler', 'Workflow',
		];
		if (defined('IAH_IN_INSTALLER')) return false;
		$module = $rowUpdate->getModuleDir();
		if (AppConfig::module_primary_bean($module) != $rowUpdate->getModelName()) {
			return;
		}
		if (!empty($rowUpdate->prohibit_workflow) || in_array($module, $skip_mods)) {
			return true;
		}
		WFProcessorBase::debug_msg("run workflow: $module/$event");

		$isUpdate = (int)!$rowUpdate->new_record;
		$rowId = $rowUpdate->getPrimaryKeyValue();
		$temp = explode('_', $event, 2);
		$timing = $temp[0];
		$action = $temp[1];
		if ($action == 'save')
			$action = 'saved';
		else
			$action = 'deleted';

		if($workflow) {
			if(! is_array($workflow))
				$workflow = array($workflow);
		}
		else if($timing == 'after') {
			$workflow = $rowUpdate->workflow;
		}

		if (! is_array($workflow)) {
			$sql = "select id,name from workflow where workflow.trigger_module = '"
				.$module."' and workflow.trigger_action='"
				.$action."' and workflow.deleted = 0 and workflow.status = 'active'";
			$res = $db->query($sql);
			// Initialize the workflow flag
			$workflow = array();
			while ($row = $db->fetchByAssoc($res)) {
				$wf = new Workflow();
				$wf->retrieve($row['id']);
				$wf->lazy_load();
				if($wf->occurs_when == 'time' && $rowUpdate->getPrimaryKeyValue())
					self::schedule_workflows($rowUpdate, $wf);
				if($wf->checkConditions($rowUpdate))
					$workflow[] = $wf;
			}
		}
		$rowUpdate->workflow = $workflow;

		$bWorkflowApplied = false;
		if(!$workflow)
			return false;
		foreach($workflow as $wf) {
			if($wf->occurs_when == 'time')
				continue;
			$GLOBALS['log']->debug("workflow found: {$wf->name} ($module/$event)");
			WFProcessorBase::debug_msg($wf->name, "workflow found ($module/$event)");

			$applicable_operations = $wf->is_applicable($rowUpdate, $timing, $wf->trigger_action, $isUpdate);

			if ($applicable_operations) {
				self::execute_operations($wf, $rowUpdate, $event, $applicable_operations);
				// Successfully applied a workflow
				$bWorkflowApplied = true;
			}
			$wf->cleanup();
		}

		// Return the result
		return $bWorkflowApplied;
	}

	static function schedule_workflows($record, $workflow) {
		$lq = new ListQuery('RecordAction', false,['filter_deleted' => false]);
		$lq->addSimpleFilter('parent_id', $record->getPrimaryKeyValue(), '=');
		$lq->addSimpleFilter('workflow_id', $workflow->id, '=');
		$lq->addSimpleFilter('type', 'runWorkflow', '=');
		$res = $lq->fetchAll();

		static $ivals = array('i' => 'minutes', 'h' => 'hours', 'd' => 'days', 'w' => 'weeks', 'm' => 'months', 'y' => 'years');
		$unit = $workflow->occurs_number . $workflow->occurs_type;
		if (preg_match('/^(-?\d+)([a-z])$/', strtolower($unit), $m)) {
			if(isset($ivals[$m[2]])) {
				$offs = $m[1];
				if($offs >= 0) $offs = '+'.$offs;
				$offs .= ' ' . $ivals[$m[2]];
				$date = gmdate('Y-m-d H:i:s', strtotime("NOW {$offs}"));
			}
		}

		$userid = AppConfig::current_user_id();

		$upd = [
			'type' => 'runWorkflow',
			'status' => 'scheduled',
			'date_start' => $date ?? '',
			'source' => 'workflow',
			'action' => 'time',
			'deleted' => '0',
			'parent_type' => $workflow->trigger_module,
			'parent_id' => $record->getPrimaryKeyValue(),
			'workflow_id' => $workflow->id,
			'assigned_user_id' => $userid
		];

		if($res->total_count < 1) {
			$wf_action = RowUpdate::blank_for_model('RecordAction', $userid);
			$wf_action->set($upd);
			$a = $wf_action->save();
		} else {
			foreach($res as $wf) {
				$wf_action = RowUpdate::for_result($wf);
				$wf_action->set($upd);
				$a = $wf_action->save();
			}
		}
	}

	static function process_scheduled() {
		$lq = new ListQuery('RecordAction');
		$lq->addFilterClause(array(
			'operator' => 'OR', 'multiple' => array(
				array('operator' => 'AND', 'multiple' => array(
					array('field' => 'status', 'value' => 'scheduled'),
					array('field' => 'date_start', 'value' => gmdate('Y-m-d H:i:s'), 'operator' => '<='),
				)),
				array('field' => 'status', 'value' => 'pending'),
			)
		));
		$total = 0;
		while(1) {
			$result = $lq->runQuery(0, 20);
			if(! $result->getResultCount()) break;
			foreach($result as $action) {
				if(scheduler_should_abort()) {
					scheduler_task_message('Aborted due to timeout');
					break 2;
				}
				$mod = $action->getField('parent_type');
				$rid = $action->getField('parent_id');
				$type = $action->getField('type');
				WFProcessorBase::debug_msg("$type $mod/$rid (".($total++).')', 'scheduled workflow');
				WFProcessorBase::exec_deferred($action);
			}
		}
		scheduler_task_message("Processed $total pending action(s)");
	}

	static function create_date_time($days, $hours, $minutes, $as_user = null)
	{
		global $timedate;
		$timestamp = time();
		$offset = $timedate->get_seconds_offset($timestamp, true, $as_user);
		$date = gmdate('Y-m-d 00:00:00', $timestamp + $offset);
		$dateStr = $date . ' GMT +' . (int)$days . ' DAYS ' . (int)$hours . ' HOURS +' . (int)$minutes . ' MINUTES';
		$date = gmdate('Y-m-d H:i', strtotime($dateStr) - $offset);
		return $date;
	}

	static function after_delete(&$rowUpdate)
	{
		self::process($rowUpdate, 'after_delete');
	}

	static function before_save(&$rowUpdate)
	{
		self::process($rowUpdate, 'before_save');
	}

	static function after_save(&$rowUpdate)
	{
		self::process($rowUpdate, 'after_save');
		self::audit_data($rowUpdate);
	}

	static function audit_data(&$rowUpdate)
	{
		if (defined('IAH_IN_INSTALLER')) return;
		if (!empty($rowUpdate->prohibit_workflow))
			return;
		global $current_user, $db;
		$module = $rowUpdate->model->module_dir;
		if (AppConfig::module_primary_bean($module) != $rowUpdate->model->name)
			return;
		static $criteria = array();
		if (!isset($criteria[$module])) {
			$criteria[$module] = array();
			$query = "SELECT c.* FROM workflow_criteria c LEFT JOIN workflow w ON w.id=c.workflow_id WHERE w.occurs_when = 'time' AND w.trigger_module = '{$module}' AND w.deleted = 0 AND c.deleted = 0 AND w.status='active'";
			$res = $db->query($query, true);
			while ($row = $db->fetchByAssoc($res)) {
				$criteria[$module][] = $row;
			}
		}
		$isUpdate = (int)!$rowUpdate->new_record;
		$def = new StdClass;
		$fetch_fields = array();
		$saved = null;
		if ($isUpdate) {
			foreach ($criteria[$module] as $row) {
				$fname = $row['field_name'];
				$value = $rowUpdate->getField($fname, $def);
				if ($value instanceof StdClass) {
					$fetch_fields[] = $fname;
				}
			}
			if (!empty($fetch_fields)) {
				$saved = ListQuery::quick_fetch_row($rowUpdate->model->name, $rowUpdate->getField('id'), $fetch_fields);
			}
		}
		foreach ($criteria[$module] as $row) {
			$fname = $row['field_name'];
			if ($saved && isset($saved[$fname])) {
				$value = $db->quote($saved[$fname]);
			} else {
				$value = $db->quote($rowUpdate->getField($fname));
			}
			if (empty($value)) {
				$val = "''";
				$cond = " = '' ";
			} else {
				$val = "'{$value}'";
				$cond = " = '{$value}'";
			}
			$id = $rowUpdate->getPrimaryKeyValue();
			$query = "SELECT related_type FROM workflow_data_audit WHERE related_type = '{$module}' AND related_id = '{$id}' AND field_name = '$fname' AND deleted = 0 AND field_value $cond ";
			$res1 = $db->query($query, true);
			$row1 = $db->fetchByAssoc($res1);
			if (!$row1) {
				$query = "REPLACE INTO workflow_data_audit SET related_type='{$module}', related_id = '{$id}', field_name = '$fname', field_value= $val, date_modified = '" . gmdate('Y-m-d H:i:s') . "', deleted = 0, is_update = $isUpdate, modified_user_id='{$current_user->id}'";
				$db->query($query, true);
				$query = "DELETE FROM workflow_data_audit_link WHERE related_id = '{$id}' AND workflow_id='{$row['workflow_id']}'";
				$db->query($query, true);
			}
		}
	}

	static function execute_operations($workflow, $rowUpdate, $event, $operations, $owner = null)
	{
		foreach ($operations as $idx=>$opt) {
			try {
				WFProcessorBase::process_workflow($rowUpdate, $workflow, $opt, $owner);
			} catch(WFProcessingError $e) {
				$GLOBALS['log']->error("Caught WFProcessingError: ".$e->getMessage());
			}
		}
	}
}


?>
