<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

require_once('modules/Workflow/WFProcessorBase.php');

class ExternalProc extends WFProcessorBase {

	var $always_defer = true;

	function process(RowUpdate $result) {
		$params = $this->getUrlParams($result);
		$url = $this->getTargetUrl($result, $params);
		if(! $url) throw new WFMissingParamError('URL');
		$method = $this->getParam('request_method', 'post');

		$curlInstance = curl_init();
		curl_setopt($curlInstance, CURLOPT_URL, $url);
		if($method == 'post' || $method == 'post_json') {
			curl_setopt($curlInstance, CURLOPT_POST, true);
			if ($method == 'post_json') {
				$params_str = json_encode($params);
				curl_setopt($curlInstance, CURLOPT_HTTPHEADER, array(
				    'Content-Type: application/json',
				    'Content-Length: ' . strlen($params_str))
				);
				curl_setopt($curlInstance, CURLOPT_POSTFIELDS, $params_str);
			} else {
				curl_setopt($curlInstance, CURLOPT_POSTFIELDS, $params);
			}
		}
		curl_setopt($curlInstance, CURLOPT_HEADER, 0);
		curl_setopt($curlInstance, CURLOPT_RETURNTRANSFER, 1);
		//curl_setopt($curlInstance, CURLOPT_CONNECTTIMEOUT, 10);
		curl_setopt($curlInstance, CURLOPT_TIMEOUT, AppConfig::setting('scheduler.external_call_timeout', 30));
		if(! ini_get('open_basedir')) {
			curl_setopt($curlInstance, CURLOPT_FOLLOWLOCATION, true);
			curl_setopt($curlInstance, CURLOPT_MAXREDIRS, 5);
		}
		$proxy = AppConfig::get_proxy_info();
		if($proxy) {
			curl_setopt($curlInstance, CURLOPT_PROXY, $proxy['host']);
			curl_setopt($curlInstance, CURLOPT_PROXYPORT, $proxy['port']);
			if($proxy['auth'])
				curl_setopt($curlInstance, CURLOPT_PROXYUSERPWD, $proxy['username'].":".$proxy['password']);
		}
		$resultString = curl_exec($curlInstance);
		$httpcode = curl_getinfo($curlInstance, CURLINFO_HTTP_CODE);
		$curl_err = curl_errno($curlInstance);
		$curl_str = curl_error($curlInstance);
		curl_close($curlInstance);
		
		$curle_temp = array(
			CURLE_COULDNT_RESOLVE_PROXY,
			CURLE_COULDNT_RESOLVE_HOST,
			CURLE_COULDNT_CONNECT,
			CURLE_OPERATION_TIMEOUTED,
			CURLE_SSL_CONNECT_ERROR,
			CURLE_TOO_MANY_REDIRECTS,
			CURLE_SSL_PEER_CERTIFICATE,
			CURLE_SSL_CERTPROBLEM,
			CURLE_SSL_CACERT,
			CURLE_BAD_PASSWORD_ENTERED,
		);
		if($curl_err) {
			if(in_array($curl_err, $curle_temp))
				throw new WFTemporaryError("cURL Error $curl_err: $curl_str");
			else
				throw new WFProcessingError("cURL Error $curl_err: $curl_str");
		}
		
		$GLOBALS['log']->debug("ExternalProc called $url; response $httpcode");
		if($httpcode >= 200 && $httpcode < 300)
			return true;
		else if($httpcode >= 300 && $httpcode < 400)
			throw new WFTemporaryError("$httpcode redirection");
		else if($httpcode >= 400 && $httpcode < 500)
			throw new WFProcessingError("$httpcode client error");
		else if($httpcode >= 500 && $httpcode < 600)
			throw new WFTemporaryError("$httpcode server error");
		else
			throw new WFProcessingError("Unknown response from $url");
	}
	
	function getDeferParams(RowUpdate $result) {
		$params = parent::getParams();
		$params['url_params'] = $this->getUrlParams($result);
		return $params;
	}
	
	function getUrlParams(RowUpdate $result) {
		$replace = $this->getParam('url_params');
		if(! is_array($replace))
			$replace = array(
				'module' => $result->getModuleDir(),
				'record' => $result->getPrimaryKeyValue(),
				'user' => $this->getUserId(),
				'source' => $this->source,
				'action' => $this->action,
				'when' => $this->perform_when,
				'workflow' => $this->workflow_id,
				'operation' => $this->operation_id,
			);
    	$replace['tracker'] = $this->deferred_id;
		return $replace;
	}
	
	function getTargetUrl(RowUpdate $result, array $params) {
		$url = $this->getParam('url');
		if(! $url) return;
		$replace = array();
		foreach($params as $k => $v)
			$replace[strtoupper('{'.$k.'}')] = $v;
    	return str_replace(array_keys($replace), array_values($replace), $url);
	}
	
	function getConfigFields() {
		$fields = parent::getConfigFields();
		
		$fields['url'] = array(
			'vname' => 'LBL_EXTERNAL_URL',
			'type' => 'url',
			'width' => 60,
			'required' => true,
		);
		$fields['request_method'] = array(
			'vname' => 'LBL_REQUEST_METHOD',
			'type' => 'enum',
			'options' => 'request_method_dom',
			'default' => 'post',
			'required' => true,
		);
		/*$fields['batch_requests'] = array(
			'vname' => 'LBL_BATCH_REQUESTS',
			'type' => 'bool',
		);*/
		$fields['retries'] = array(
			'vname' => 'LBL_RETRIES',
			'type' => 'enum',
			'options' => array(0 => 0, 3 => 3, 5 => 5, 10 => 10, 20 => 20),
			'options_add_blank' => false,
			'default' => 5,
		);
		$fields['description'] = array(
			'vname' => 'LBL_NOTES',
			'type' => 'text',
		);
		
		return $fields;
	}

}
