<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */

require_once('modules/Workflow/WFProcessorBase.php');

class UpdateRelatedProc extends WFProcessorBase {

	function process(RowUpdate $rowUpdate) {
		$link = $this->getParam('link_name');
		if(! $link) return;
		$rel = $rowUpdate->fetchRelated($link);
		if(! $rel) return;
		$relUpd = $rel->getRowUpdate();
		$fields = $this->getParam('fields');
		$vals = $this->getParam('values');
		if(! empty($fields) && ! empty($vals)) {
			$this->setValuesWithAlt($rowUpdate, $relUpd, $fields, $vals);
			$relUpd->prohibit_workflow = true;
			$relUpd->save();
		}
	}
	
	function loadConfigRequest() {
		$ret = parent::loadConfigRequest();
		if($ret && is_array($ret['params'])) {
			if(isset($ret['params']['updates'])) {
				$js = getJSONobj();
				$upd = $js->decode($ret['params']['updates']);
				$fields = $defs = $map = array();
				foreach($upd as $row) {
					$fname = $row['field'];
					$defs[$fname] = $row;
					$fields[$fname] = array('field' => $fname);
					$map[$fname] = 'fvalue_'.$row['_iprefix'];
				}
				$fvals = array();
				foreach($map as $n => $pfx) {
					$fvals[$n] = array_get_default($_REQUEST, $pfx.$n);
					if(! empty($defs[$n]['id_name'])) {
						$idn = $defs[$n]['id_name'];
						$fvals[$idn] = array_get_default($_REQUEST, $pfx.$idn);
					}
					$alt = $n.'_alt';
					$altv = array_get_default($_REQUEST, $pfx.$alt);
					if(isset($altv)) {
						$fvals[$alt] = $altv;
					}
				}
				require_once('include/layout/FieldFormatter.php');
				$fmt = new FieldFormatter('html', 'editview');
				$values = $fmt->unformatRow($defs, $fvals);
				$ret['params']['fields'] = $fields;
				$ret['params']['values'] = $values;
			}
			unset($ret['params']['updates']);
		}
		return $ret;
	}
	
	function getUpdates($edit=true)  {
		$ps = $this->getParams();
		if(! empty($ps['fields'])) {
			$fields = $ps['fields'];
			$values = $ps['values'];
		}
		else if(! empty($ps['field_name'])) {
			$fields = array($ps['field_name']);
			$values = array($ps['field_name'] => $ps['field_value']);
		}
		if(! empty($fields)) {
			require_once('include/ListView/ListLayoutEditor.php');
			$layout = new ListLayoutEditor();
			$link = $this->getParam('link_name');
			if($layout->initFromModel($this->bean_name, null, $this->form_name, 'workflow')) {
				$layout->wf_module = $this->module;
				$render = $layout->formatFieldsForUpdate($fields, $values, $link, $edit);
				return $render;
			}
		}
		return array();
	}
	
	function getConfigParams() {
		$ps = $this->getParams();
		$ps['updates'] = $this->getUpdates();
		return $ps;
	}
	
	function getRenderParams() {
		$ps = $this->getParams();
		$show = '<ul class="noBullet">';
		foreach($this->getUpdates(false) as $upd) {
			$show .= '<li class="noBullet">'.to_html($upd['label']).': '.$upd['_init']['rendered'].'</li>';
		}
		$show .= '</ul>';
		$ps['updates'] = $show;
		return $ps;
	}

	function getSourceOptions() {
		require_once('include/ListView/ListLayoutEditor.php');
		$edit = new ListLayoutEditor();
		$edit->initFromModel($this->bean_name);
		$opts = $edit->getSourcesJson(null, null, true);
		$ret = [];
		foreach($opts as $o) {
        	$ret[$o['field_name']] = [
        		'icon' => $o['icon'],
        		'name' => $o['label'],
        		'field_name' => $o['field_name'],
        		'module' => $o['module'],
        	];
		}
		return $ret;
	}

	function getConfigFields() {
		$fields = parent::getConfigFields();
		
		$link_name = $this->getParam('link_name');
		
		$fields['link_name'] = array(
			'vname' => 'LBL_DM_LINK_NAME',
			'name' => 'link_name',
			'type' => 'enum',
			'options' => $this->getSourceOptions(),
			'options_icon' => 'icon',
			'onchange' => "WorkflowEditor.updateFieldSelect(this.form, this.name);",
			'custom_source' => true,
		);
		$fields['updates'] = array(
			'vname' => 'LBL_DM_FIELD_UPDATES',
			'widget' => 'DynamicListInput',
			'class' => 'listLayoutConfig',
			'renderer_class' => 'SUGAR.ui.FieldListRender',
			'show_action' => 'false',
			'cols' => array(
				array(
					'name' => 'label',
					'label' => translate('LBL_DM_FIELD_NAME', 'Workflow'),
					'width' => '40%',
					'render_method' => 'renderName',
				),
				array(
					'name' => 'value',
					'label' => translate('LBL_DM_FIELD_VALUE', 'Workflow'),
					'width' => '40%',
					'render_method' => 'renderValue',
				),
			)
		);
		$fields['add_field'] = array(
			'name' => 'add_field',
			'vname' => 'LBL_DM_ADD_FIELD',
			'widget' => 'FieldSelectInput',
			'list_name' => 'updates',
			'sel_mode' => 'updates',
			'context' => 'workflow',
			'wf_module' => $this->module,
			'field_model' => $this->bean_name,
			'link_name' => $link_name,
		);

		return $fields;
	}

	function beforeRenderConfigForm(HtmlFormGenerator $gen, $context, $entry_id) {
		$gen->getEditableForm()->onInitForm("WorkflowEditor.initFieldSelect({FORM}, 'link_name')");
	}

	function getRenderFields() {
		$fields = $this->getConfigFields();
		$fields['updates'] = array('type' => 'html', 'vname' => 'LBL_DM_FIELD_UPDATES');
		return $fields;
	}
	
	function getRequiredFields() {
		$req = parent::getRequiredFields();
		$req[] = $this->getParam('link_name');
		return $req;
	}
}
