<?php

function wf_upgraded_criteria($wf_id) {
	global $db;
	$wf_id = $db->quote($wf_id);
	$q = "SELECT * FROM workflow_criteria WHERE workflow_id='$wf_id' AND NOT deleted ORDER BY idx";
	$r = $db->query($q);
	$rows = $db->fetchRows($r, true, false);
	$tree = wf_conditions_tree($rows);
	$tree = wf_convert_tree($tree);
	if($tree) {
		if(isset($tree['multiple']) && $tree['operator'] == 'and') {
			return $tree['multiple'];
		} else {
			return [$tree];
		}
	}
	return [];
}

function wf_convert_tree($crits, $parent=null, $group_op=null) {
	$op = 'and';
	$ops = array();
	$ret = $parent;
	foreach($crits as $crit) {
		if(! empty($crit['condition'])) {
			$cond = wf_convert_criteria($crit['condition']);
			if($cond) {
				if($ret) {
					$glue = strtolower($cond['glue']);
					unset($cond['glue']);
					if($group_op && $group_op == $glue) {
						$ret['multiple'][] = $cond;
					} else {
						$ret = ['multiple' => [$ret, $cond], 'operator' => $glue];
						$group_op = $glue;
					}
				} else {
					$group_op = strtolower($cond['glue']);
					unset($cond['glue']);
					$ret = $cond;
				}
			}
		}
		if(! empty($crit['children'])) {
			$ret = wf_convert_tree($crit['children'], $ret, $group_op);
			if($ret && isset($ret['multiple']))
				$group_op = $ret['operator'];
			else
				$group_op = null;
		}
	}
	return $ret;
}

function wf_split_values($clauses, &$values, $pfx='') {
	if(! is_array($values)) $values = array();
	$ret = array();
	foreach($clauses as $idx => $c) {
		if(isset($c['multiple'])) {
			$cnt = 1;
			$base = 'group~';
			$fn = $base . ($cnt ++);
			while(array_key_exists($fn, $values))
				$fn = $base . ($cnt ++);
			$values[$fn] = null;
			$c['multiple'] = wf_split_values($c['multiple'], $values, $pfx.$fn.'-');
			$c['type'] = 'filtergroup';
			$c['name'] = $fn;
			$ret[] = $c;
		} else {
			if(! empty($c['field'])) {
				$fn = $c['field'];
				$cnt = 1;
				if(preg_match('~^(.+?)\d*$~', $fn, $m))
					$base = $m[1];
				else
					$base = 'filter';
				while(array_key_exists($fn, $values))
					$fn = $base . ($cnt ++);
				$values[$fn] = $values[$pfx.$fn] = array_get_default($c, 'value');
				unset($c['value']);
				$c['name'] = $fn;
				$ret[] = $c;
			}
		}
	}
	return $ret;
}


function wf_conditions_tree($criteria) {
	$groups = array(
		array(
			'level' => 0,
			'conditions' => array(),
		),
	);
	$gsize = 0;
	$level = 0;
	$parents = array();
	foreach ($criteria as $cond) {
		if ($level != $cond['level']) {
			$gsize++;
			$groups[$gsize] = array(
				'level' => $cond['level'],
				'conditions' => array(),
			);
		}
		$groups[$gsize]['conditions'][] = $cond;
		$level = $cond['level'];
	}


	$tree = array(
		array(
			'children' => array(),
		),
	);
	$path = array();
	$path[0] = &$tree[0];
	$level = 0;
	foreach ($groups as $group) {
		$cur =& $path[count($path)-1];
		if ($group['level'] > $level) {
			$countChildren = is_countable($cur['children']) ? count($cur['children']) : 0;
			$cur =& $cur['children'][$countChildren-1];
			$path[$group['level']] =& $cur;
		} elseif ($group['level'] < $level) {
			$keys = array_keys($path);
			while ($keys[count($path)-1] > $group['level']) {
				array_pop($path);
			}
			$cur =& $path[count($path)-1];
		}
		foreach ($group['conditions'] as $cond) {
			$cur['children'][] = array(
				'condition' => $cond,
				'children' => array(),
			);
		}
		$level = $group['level'];
	}
	return $tree;
}


function wf_convert_criteria($cond) {
	$operator = $cond['operator'];
	$cond_value = $cond['field_value'];
	$ret = array('field' => $cond['field_name'], 'value' => $cond_value, 'when' => 'current');
	
	switch ($operator) {
		case "enum_one_of":
			$ret['operator'] = 'any_of';
			$json = getJSONobj();
			$selection = $json->decode($cond_value);
			if (!$selection) {
				$selection = array();
			}
			$ret['value'] = $selection;
			break;
		case "num_is_equal_to":
		case "txt_is_equal_to":
		case "user_is":
		case "ref_is":
			$ret['operator'] = 'eq';
			break;
		case "num_is_not_equal_to":
		case "txt_is_not_equal_to":
		case "user_is_not":
			$ret['operator'] = 'not_eq';
			break;
		case "num_is_less_than":
			$ret['operator'] = 'lt';
			break;
		case "num_is_not_less_than":
			$ret['operator'] = 'gte';
			break;
		case "num_is_greater_than":
			$ret['operator'] = 'gt';
			break;
		case "num_is_not_greater_than":
			$ret['operator'] = 'lte';
			break;
		case "txt_contains":
			$ret['operator'] = 'like';
			break;
		case "txt_not_contain":
			$ret['operator'] = 'not_like';
			break;
		case "txt_begins_with":
			$ret['operator'] = 'like';
			$ret['match'] = 'prefix';
			break;
		case "txt_ends_with":
			$ret['operator'] = 'like';
			$ret['match'] = 'suffix';
			break;
		case "bol_is_false":
			$ret['operator'] = 'false';
			break;
		case "bol_is_true":
			$ret['operator'] = 'true';
			break;
		case "dat_is_equal_to":
			wf_date_range_op($ret, $ret['value']);
			break;
		case "dat_before":
			wf_date_range_op($ret, 'before:'.$ret['value']);
			break;
		case "dat_after":
			wf_date_range_op($ret, 'after:'.$ret['value']);
			break;
		case 'dat_relative_before':
			wf_date_range_op($ret, 'before:'.$ret['value']);
			break;
		case 'dat_relative_after':
			wf_date_range_op($ret, 'after:'.$ret['value']);
			break;
		case 'dat_within':
			wf_date_range_op($ret, 'within:'.$ret['value']);
			break;
		case 'dat_not_within':
			wf_date_range_op($ret, 'not_within:'.$ret['value']);
			break;
		default:
			$GLOBALS['log']->error("Unknown operator: ".$operator);
	}
	
	$ret['glue'] = strtolower($cond['glue']);
	
	return $ret;
}


function wf_date_range_op(&$ret, $val) {
	$ret['operator'] = 'date_range';
	$ret['value'] = $val;
}


function wf_upgrade_operation_params(RowResult $op) {
	$ret = array();
	if($op->getField('performed_before_event'))
		$ret['perform_when'] = 'before';
	else
		$ret['perform_when'] = 'after';
	
	$type = $op->getField('operation_type');
	if($type == 'showAlert') {
		$params = array(
			'body' => $op->getField('notification_content'),
		);
	}
	else if($type == 'sendEmail') {
		$params = array(
			'name' => $op->getField('notification_subject'),
			'description_html' => $op->getField('notification_content'),
			'from_name' => $op->getField('notification_from_name'),
			'from_email' => $op->getField('notification_from_email'),
			'invitee_ids' => $op->getField('notification_invitee_ids'),
			'invitee_types' => $op->getField('notification_invitee_types'),
			'cc_mailbox' => $op->getField('notification_cc_mailbox'),
			'include_initiator' => $op->getField('include_initiator'),
			'include_primary_email' => $op->getField('include_primary_email'),
			'include_secondary_email' => $op->getField('include_secondary_email'),
		);
	}
	else if($type == 'scheduleCall' || $type == 'scheduleMeeting') {
		$params = array(
			'name' => $op->getField('notification_subject'),
			'body' => $op->getField('notification_content'),
			'duration' => $op->getField('notification_duration_hour') * 60 + $op->getField('notification_duration_min'),
			'invitee_ids' => $op->getField('notification_invitee_ids'),
			'invitee_types' => $op->getField('notification_invitee_types'),
			'include_contact' => $op->getField('include_contact'),
		);

		$ch = $op->getField('notification_start_date_choice');
		if($ch == 'C1')
			$params['date_start_type'] = 'same_day';
		else if($ch == 'C2')
			$params['date_start_type'] = 'next_day';
		else if($ch == 'C3') {
			$params['date_start_type'] = 'days_of';
			$params['date_start_count'] = $op->getField('notification_start_date');
		}
		$hr = $op->getField('notification_start_time_hour');
		$mn = $op->getField('notification_start_time_min');
		if(strlen($hr))
			$params['date_start_time'] = sprintf('%02d:%02d:00', $hr, $mn);
	}
	else if($type == 'updateCurrentData') {
		$params = array(
			'field_name' => $op->getField('dm_field_name'),
			'field_value' => $op->getField('dm_field_value'),
		);
	}
	else if($type == 'updateRelatedData') {
		$params = array(
			'link_name' => $op->getField('dm_link_name'),
			'rel_module' => $op->getField('dm_module_name'),
			'field_name' => $op->getField('dm_field_name'),
			'field_value' => $op->getField('dm_field_value'),
		);
	}
	else if($type == 'createTask') {
		$params = array(
			'name' => $op->getField('notification_subject'),
			'description' => $op->getField('notification_content'),
			'status' => $op->getField('task_status'),
			'priority' => $op->getField('task_priority'),
			'contact_id' => $op->getField('contact_id'),
			'include_contact' => $op->getField('relate_to_record'),
		);
		
		$effort = $op->getField('task_est_effort');
		$eunit = $op->getField('task_est_effort_unit');
		if($eunit == 'hours')
			$effort *= 60;
		else if($eunit == 'days')
			$effort *= 60 * 24;
		$params['effort_estim'] = $effort;
		
		if(! $op->getField('task_due_date_flag')) {
			$ch = $op->getField('task_due_date_choice');
			if($ch == 'C1')
				$params['date_due_type'] = 'same_day';
			else if($ch == 'C2')
				$params['date_due_type'] = 'next_day';
			else if($ch == 'C3') {
				$params['date_due_type'] = 'days_of';
				$params['date_due_count'] = $op->getField('task_due_date');
			}
			$hr = $op->getField('task_due_time_hour');
			$mn = $op->getField('task_due_time_min');
			if(strlen($hr))
				$params['date_due_time'] = sprintf('%02d:%02d:00', $hr, $mn);
		} else
			$params['date_due_type'] = '';
					
		if(! $op->getField('task_start_date_flag')) {
			$ch = $op->getField('notification_start_date_choice');
			if($ch == 'C1')
				$params['date_start_type'] = 'same_day';
			else if($ch == 'C2')
				$params['date_start_type'] = 'next_day';
			else if($ch == 'C3') {
				$params['date_start_type'] = 'days_of';
				$params['date_start_count'] = $op->getField('notification_start_date');
			}
			$hr = $op->getField('notification_start_time_hour');
			$mn = $op->getField('notification_start_time_min');
			if(strlen($hr))
				$params['date_start_time'] = sprintf('%02d:%02d:00', $hr, $mn);
		} else
			$params['date_start_type'] = '';
	} else
		return;
	
	$ret['params'] = $params;
	return $ret;
}

function upgradeAllWorkflow() {
	require_bean('Workflow');
	require_bean('WFOperation');
	$wfs = new ListQuery('Workflow');

	foreach($wfs as $wf) {
		$wf_id = $wf->getField('id');
		$criteria = $wf->getField('criteria');
		$module = $wf->getField('trigger_module');
		if(! $criteria) {			
			$ops = $wf->getLinkResults('operations');
			foreach($ops as $o) {
				$params = $o->getField('params');
				if(! is_string($params) || ! strlen($params)) {
					$o_up = $o->getRowUpdate();
					$o_set = wf_upgrade_operation_params($o);
					if($o_set) {
						$o_up->set($o_set);
						$o_up->save();
					}
				}
			}
			
			$wf_up = $wf->getRowUpdate();
			$upg = wf_upgraded_criteria($wf_id);
			$opts = wf_split_values($upg, $values);
			$wf_up->set('criteria', serialize(['options' => $opts, 'values' => $values]));
			$wf_up->save();
		}
	}
}
