<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormElement.php');

class PerformWhenInput extends FormElement {	
	var $params;
	var $opts = array(
		'immed' => 'LBL_OPERATIONS_PERFORM_IMMED',
		'before' => 'LBL_OPERATIONS_PERFORM_BEFORE',
		'after' => 'LBL_OPERATIONS_PERFORM_AFTER',
		'scheduled' => 'LBL_OPERATIONS_PERFORM_SCHEDULED',
	);
	
	function init($params=null, $model=null) {
		parent::init($params, $model);
		if(is_array($params))
			$this->params = $params;
	}
	
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		if(empty($this->name))
			return '';
		if($this->getEditable($context))
			$value = $this->renderHtmlEdit($gen, $row_result, $context);
		else
			$value = $this->renderHtmlView($gen, $row_result, $context);
		return $value;
	}
	
	function getRequiredFields() {
		return array($this->name);
	}

	function renderHtmlView(HtmlFormGenerator $gen, RowResult $row_result, array $context) {
		$when = $row_result->getField('perform_when');
		$lbl = $when ? translate($this->opts[$when], 'Workflow') : '';
		if($when == 'scheduled') {
			$count = $row_result->getField($this->name . '_count');
			$unit = $row_result->getField($this->name . '_unit');
			$unique = $row_result->getField('unique');
			$opts = return_mod_list_strings_language('', 'Workflow');
			$unit = array_get_default($opts['OPTIONS_TIME_CONDITION'], $unit, $unit);
			$lbl .= ': ' . $count . ' ' . $unit;
			if ($unique) {
				$lbl .= ' (' . translate('LBL_UNIQUE_ACTION', 'Workflow') . ')';
			}
		}
		return $lbl;
	}
	
	function renderHtmlEdit(HtmlFormGenerator $gen, RowResult $row_result, array $context) {
		$form =& $gen->getFormObject();
		
		$name = $form->getFieldNamePrefix() . $this->name;
		$sel_id = $form->getFieldIdPrefix() . $this->name;
		$count_name = $this->name.'_count';
		$unit_name = $this->name.'_unit';
		$unique_name = 'unique';
		
		$when_spec = array(
			'name' => $name,
			'type' => 'enum',
			'options' => array(),
			'onchange' => "toggleDisplay('{$sel_id}-unit', (this.getValue() == 'scheduled')); toggleDisplay('{$sel_id}-unique', (this.getValue() == 'scheduled'))",
		);
		if($this->required)
			$when_spec['required'] = true;
		if(isset($this->params['allow_when'])) {
			$allow = $this->params['allow_when'];
			foreach($this->opts as $k => $lbl) {
				if(in_array($k, $allow)) {
					$when_spec['options'][$k] = translate($lbl, 'Workflow');
				}
			}
		}
		$when = $row_result->getField($this->name);
		if(! is_countable($when_spec['options']) || ! count($when_spec['options']))
			return '';
		if(! isset($when_spec['options'][$when]))
			$when = key($when_spec['options']);
		if(is_countable($when_spec['options']) && count($when_spec['options']) == 1) {
			$ret = EditableForm::get_tag('input', array('name' => $name, 'type' => 'hidden', 'value' => $when));
			$ret .= to_html(array_get_default($when_spec['options'], $when));
			return $ret;
		}
		
		$ret = '<div id="'.$sel_id.'">';
		$ret = $form->renderSelect($when_spec, $when);
		/*$count_spec = array(
			'name' => $form->getFieldNamePrefix().$count_name,
			'type' => 'int',
			'placeholder' => '(count)',
			'len' => 5,
		);
		$ret .= ' ';
		$ret .= $form->renderText($count_spec, $row_result->getField($count_name, 1));*/

		$s = ($when == 'scheduled' ? '' : 'display: none');

		$ret .= '<div id="'.$sel_id.'-unit" style="'.$s.'">';
		$unit_spec = array(
			'name' => $form->getFieldNamePrefix().$unit_name,
			'type' => 'enum',
			'options' => 'OPTIONS_TIME_CONDITION',
			'options_add_blank' => false,
		);
		$ret .= ' ';
		$ret .= $form->renderSelect($unit_spec, $row_result->getField($unit_name, '1h'));
		$ret .= '</div>';

		$ret .= '<div id="'.$sel_id.'-unique" style="'.$s.'">';
		$unique_spec = array(
			'name' => $form->getFieldNamePrefix().$unique_name,
			'type' => 'bool',
			'vname' => 'LBL_UNIQUE_ACTION',
			'default' => 0,
			'suffix_label' => true,
			'hide_label' => true,
		);
		$ret .= ' ';
		$ret .= $form->renderCheck($unique_spec, $row_result->getField($unique_name, 0));
		$ret .= '</div>';
		$ret .= '</div>';




        $attrs = array(
        	'name' => $name,
        	'label' => $this->getLabel(),
        	'required' => ! empty($this->required),
        );
        //$form->addControl('DelayInput', $sel_id, $attrs);

		return $ret;
	}
	
}
