<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/widgets/DynamicListInput.php');
require_once('include/layout/FilterEditor.php');

class WFCriteriaInput extends DynamicListInput {
	var $trigger_module;
	var $criteria;
	var $criteria_clauses;
	var $criteria_fields;
	
	function init($params=null, $model=null) {
		$this->max_depth = 3;
		$this->require_group_attrib = true;
		$this->cols = array(
			array(
				'name' => 'label',
				'label' => 'LBL_FIELD',
				'width' => '30%',
				'render_method' => 'renderName',
			),
			array(
				'name' => 'when',
				'label' => translate('LBL_WHEN', 'Workflow'),
				'width' => '25%',
				'render_method' => 'renderWhen',
			),
			array(
				'name' => 'filter',
				'label' => translate('LBL_FILTER', 'Workflow'),
				'width' => '45%',
				'render_method' => 'renderFilter',
			)
		);
		$this->renderer_class = 'SUGAR.ui.WFCriteriaRender';
		parent::init($params, $model);
	}
	
	function getRequiredFields() {
		$req = parent::getRequiredFields();
		$req[] = 'trigger_module';
		$req[] = 'criteria';
		return $req;
	}
	
	function loadResult(RowResult $row_result, array $context) {
		$this->trigger_module = $row_result->getField('trigger_module');
		$this->criteria = $row_result->getField('criteria');
		if(! $this->criteria) {
			// upgrade old criteria
			require_once('modules/Workflow/upgrade_utils.php');
			$upg = wf_upgraded_criteria($row_result->getField('id'));
			$opts = wf_split_values($upg, $values);
			$this->criteria = serialize(['options' => $opts, 'values' => $values]);
		}
	}
	
	function getEditor($form_name, $form_id) {
		if($this->trigger_module) {
			$model = AppConfig::module_primary_bean($this->trigger_module);
			if($model) {
				$lq = new ListQuery($model);
				$editor = new FilterEditor($lq, $form_name, $form_id, 'workflow');
				return $editor;
			}
		}
	}
	
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context) {
		$editable = $gen->getLayout()->getDefaultEditable();		
		$form = $gen->getFormObject();
		$editor = $this->getEditor($form->getFormName(), $form->getFormId());
		$filters = null;

		if($editor && $this->criteria) {
			$crit = unserialize($this->criteria);
			if($crit) {
				$editor->loadFilterOptions($crit['options']);
				$editor->loadFilterValues($crit['values']);
				$filters = $editor->encodeFilters($editable);
			}
		}
		
		if($editable) {
			$row_result->assign($this->name, $filters);
			$ret = parent::renderHtml($gen, $row_result, $parents, $context);
		} else {
			$ret = $editor->formatFilters($filters);
			if(! $ret)
				$ret = translate('LBL_NONE', 'app');
		}
		
		$this->exportIncludes($gen->getLayout());
		return $ret;
	}
	
	function loadUpdateRequest(RowUpdate $update, array $input)
	{
		$this->trigger_module = array_get_default($input, 'trigger_module');
		$edit = $this->getEditor(null, null);
		if($edit && $edit->loadRequestFilter($input, $this->name)) {
			$options = $edit->filter_options;
			$values = $edit->filter_values;
			$crit = compact('options', 'values');
			$this->criteria = serialize($crit);
			$clauses = $edit->getClauses(true);
			$this->criteria_clauses = serialize($clauses);
			$this->criteria_fields = implode(',', $edit->getClauseFields($clauses));
		}
	}
	
	function beforeUpdate(RowUpdate $update) {
		$update->set('criteria', $this->criteria);
		$update->set('criteria_clauses', $this->criteria_clauses);
		$update->set('criteria_fields', $this->criteria_fields);
	}
}
