<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */


require_once('include/layout/forms/FormField.php');

class WFOperationsInput extends FormField {

	var $reload = false;
	var $editable = false;
	var $operations;

	function init($params=null, $model=null) {
		parent::init($params, $model);
		if(! $this->id)
			$this->id = 'workflow_items';
	}
	
	function renderHtml(HtmlFormGenerator $gen, $row_result, array $parents, array $context)
	{
		if($gen->getLayout()->getEditingLayout()) {
			return '';
		}
		$this->editable = $gen->getLayout()->getDefaultEditable();
		global $mod_strings;
		$module = $row_result->getField('trigger_module');
		$js = $this->addJS($row_result, $gen->getFormName(), $module);
		$hidden = $module ? '' : 'display: none';
		$msg = '';
		if(! $this->editable && ! $this->operations)
			$msg = translate('LBL_NONE', 'app');
		$html = <<<EOH
<table style="width:100%" id="workflow_operations"><tbody>{$msg}</tbody></table>
<div id="workflow_operations_footer"></div>
EOH;
		$this->exportIncludes($gen->getLayout());
		return $html;
	}

	function addJS(RowResult $row_result, $form_name, $module)
	{
		global $pageInstance;
		global $app_list_strings;
		$wfId = $row_result->getField('id');
		$action = $row_result->getField('trigger_action');
		$data = array(
			'trigger_module' => $module,
			'trigger_action' => $action,
			'editable' => $this->editable,
			'occurs_when' =>  $row_result->getField('occurs_when'),
		);

		if ($this->reload) {
			$data['operations'] = '';
		} else {
			$data['operations'] = $this->generateOperations($wfId, $module);
		}
		if($row_result->isForDuplicate()) {
			$this->clearRelatedIds($data['operations']);
		}
		$this->operations = $data['operations'];

		$json = getJSONObj();

		$js = "\nSUGAR.ui.registerInput('$form_name', WorkflowEditor);\n";
		$js .= 'WorkflowEditor.init(' . $json->encode($data) . ');';
        $images_meta = $this->getImagesMeta();
		$pageInstance->add_js_literal("ImageMeta = $images_meta;", null, LOAD_PRIORITY_FOOT);
		$pageInstance->add_js_literal($js, null, LOAD_PRIORITY_FOOT);
		
		/*require_once('include/CKeditor_IAH/CKeditor_IAH.php');
		$ed = new CKeditor_IAH();

		// this is ugly - exportIncludes should be fixed (see CKeditor_IAH source)
		$ret = $ed->exportIncludes();*/
		//return $ret;
	}
	
	function clearRelatedIds(&$arr) {
		if(is_array($arr) && count($arr) == 1) {
			reset($arr);
			$idx = key($arr);
			foreach($arr[$idx] as $k => $v) {
				if(is_array($v)) {
					$v['id'] = '';
					$v['workflow_id'] = '';
					$arr[$idx][$k] = $v;
				}
			}
		}
	}

	function generateOperations($wfId, $module)
	{
		require_bean('Workflow');
		require_bean('WFOperation');
		$ops = Workflow::get_operations($wfId, $module);
		$json = getJSONobj();
		foreach($ops as &$o) {
			//if($o['params'] == '{}') $o['params'] = null;// testing
			if(is_string($o['params']) && strlen($o['params']))
				$o['params'] = $json->decode($o['params']);
			else {
				require_once('modules/Workflow/upgrade_utils.php');
				$r = new RowResult;
				$r->row = $o;
				$upd = wf_upgrade_operation_params($r);
				if($upd) array_extend($o, $upd);
			}
		}
		return $ops;
	}

	function getImagesMeta() {
        global $app_strings, $mod_strings;

        $images = array(
            'insert' => array('name' => 'plus_inline', 'title' => $app_strings['LNK_INS']),
            'remove' => array('name' => 'delete_inline', 'title' => $app_strings['LNK_REMOVE']),
            'plus' => array('name' => 'plus_inline'),
            'tree_dots' => array('name' => 'treeDots'),
            'tree_branch' => array('name' => 'treeDotsBranch'),
            'tree_end' => array('name' => 'treeDotsEnd'),
            'blank' => array('name' => 'blank', 'width' => 12, 'height' => 12),
            'Cases' => array(),
            'Project' => array(),
            'pdf' => array('name' => 'pdf_image_inline',),
			'left' => array('name' => 'leftarrow_big', 'title' => $mod_strings['LNK_LEFT']),
			'right' => array('name' => 'rightarrow_big', 'title' => $mod_strings['LNK_RIGHT']),
			'up' => array('name' => 'uparrow_big', 'title' => $mod_strings['LNK_UP']),
			'down' => array('name' => 'downarrow_big', 'title' => $mod_strings['LNK_DOWN']),
        );

        foreach($images as $idx => $img) {
            $src = isset($img['name']) ? $img['name'] : $idx;
            unset($img['name']);
            $images[$idx] = array_merge(get_image_info($src), $img);
        }

        $json = getJSONObj();
        $images_meta = $json->encode($images);

        return $images_meta;
    }
	
	function loadUpdateRequest(RowUpdate $update, array $input)
	{
		$this->reload = array_get_default($input, 'reload');
		$json = getJSONObj();
		$items = $json->decode(array_get_default($input, $this->id));
		$update->setRelatedData($this->id.'_items', $items);
	}
	
	function validateInput(RowUpdate $update)
	{
		return true;
	}
	
	function exportIncludes($layout=null) {
		$script = 'modules/Workflow/workflow.js';
		if($layout)
			$layout->addScriptInclude($script);
		else
			$GLOBALS['pageInstance']->add_js_include($script, null, LOAD_PRIORITY_FORM);
	}
	
	function afterUpdate(RowUpdate $update) {
        $wf_id = $update->getPrimaryKeyValue();
        $old_ops = array();

		if (!$update->new_record) {
			global $db;
			
			$lq = new ListQuery('WFOperation');
			$lq->addSimpleFilter('workflow_id', $wf_id);
			foreach($lq->fetchAll() as $op) {
				$old_ops[$op->getField('id')] = $op;
			}
		}
		
		$items = $update->getRelatedData($this->id.'_items');
		if($items) {
			$operations = array_get_default($items, 'operations', array());
			$order = 0;
			$js = getJSONobj();
			foreach ($operations as $operation) {
				if(! empty($operation['id']) && isset($old_ops[$operation['id']])) {
					$ru = $old_ops[$operation['id']]->getRowUpdate();
					unset($old_ops[$operation['id']]);
				} else
					$ru = RowUpdate::blank_for_model('WFOperation');
				$ru->set('display_order', $order);
				$operation['workflow_id'] = $wf_id;
				$operation['display_order'] = $order;
				if(isset($operation['params']) && is_array($operation['params'])) {
					$operation['unique'] = ($operation['params']['unique'] ?? 0) ? 1 : 0;
					$operation['params'] = $js->encode($operation['params']);
				}
				$ru->request = $operation;
				$ru->loadRequest();
				if($ru->validate() && $ru->save())
					$order++;
				else
					$GLOBALS['log']->error("Error saving workflow operation: ".$ru->getErrors(true));
			}
		}
		
		foreach($old_ops as $id => $op)
			$op->getRowUpdate()->markDeleted();
	}

}
?>
