// vim: set autoindent smartindent :
/*****************************************************************************
 * The contents of this file are subject to The Long Reach Corporation
 * Software License Version 1.0 ("License"); You may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * <http://www.thelongreach.com/swlicense.html>
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations under
 * the License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) The Long Reach Corporation copyright notice,
 * (ii) the "Powered by SugarCRM" logo, and
 * (iii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is:
 *    Info At Hand Add-on Module to SugarCRM Open Source project.
 * The Initial Developer of this Original Code is The Long Reach Corporation
 * and it is Copyright (C) 2004-2007 by The Long Reach Corporation;
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2005 SugarCRM, Inc.;
 * All Rights Reserved.
*****************************************************************************/

SUGAR.module('workflow', 'ui-controls').init(function(App) {
var Dom = App.dom,
	MakeElt = Dom.makeElement;

WorkflowEditor = new function() {
	this.id = 'WorkflowEditor';
	this.uid = 0;
	this.trigger_module = '';
	this.editable = false;
	this.registered_options = {};
	this.operations = [];
	this.criteria = null;
	this.has_related = false;


	this.init = function(params)
	{
		this.editable = params.editable;
		this.operations = params.operations || [];
		this.trigger_module = params.trigger_module;
		this.occurs_when = params.occurs_when;
	};

	this.setup = function()
	{
		if (this.editable) {
			var x = SUGAR.ui.getFormInput(this.form, 'occurs_when');
			if(x) {
				x.onchange = this.occursWhenChanged;
				this.setTriggerState(x.getValue());
			}
			var trigger = SUGAR.ui.attachFormInputEvent(this.form, 'trigger_module', 'onchange',
				function(k, v, ch) {
					WorkflowEditor.trigger_module = k;
					WorkflowEditor.updatedModule();
				});
			var o = SUGAR.ui.getFormInput(this.form, 'operation_owner');
			o.update = function(k, v, s) {
				if (k == 'select') {
					WorkflowEditor.showUserSelect();
				} else {
					SUGAR.ui.SelectInput.prototype.update.call(this, k, v, s);
				}
			}
			this.criteria = SUGAR.ui.getFormInput(this.form, 'workflow_criteria');
		}

		this.renderOperations();
		this.toggleSections();
	};

	this.beforeSubmitForm = function()
	{
		var ret = {
			operations: this.operations
		};
	    this.form.workflow_items.value = JSON.stringify(ret);
	};

	this.clearTable = function(id)
	{
		var table = $(id);
		if (!table) return;
		while(table.rows.length > 0)
			table.deleteRow(table.rows.length-1);
	};


	this.occursWhenChanged = function(k, v, ch)
	{
		if (ch) {
			WorkflowEditor.setTriggerState(k);
		}
	};

	this.setTriggerState = function(k)
	{
		var action = SUGAR.ui.getFormInput(this.form, 'trigger_action');
		this.occurs_when = k;
		if (k == 'time') {
			action.menu.setSelectedKey('saved');

			document.getElementsByClassName('cell-occurs_number')[0].style.display = 'block';
			document.getElementsByClassName('cell-occurs_type')[0].style.display = 'block';
			if(this.form['DetailFormtrigger_action-input'])
				this.form['DetailFormtrigger_action-input'].disabled = true;
		} else {
			document.getElementsByClassName('cell-occurs_number')[0].style.display = 'none';
			document.getElementsByClassName('cell-occurs_type')[0].style.display = 'none';
			if(this.form['DetailFormtrigger_action-input'])
				this.form['DetailFormtrigger_action-input'].disabled = false;
		}
	};

	this.insertOperation = function(type) {
		SUGAR.popups.openUrl('async.php?module=Workflow&action=RenderActionConfig&context=workflow&trigger_module='+this.trigger_module+'&operation_type='+type);
	};

	this.editOperation = function(idx) {
		var op = this.operations[idx], cop;
		if(! op) return;
		cop = deep_clone(op);
		cop.params = JSON.stringify(cop.params);
		var req = new SUGAR.conn.HTTPRequest('async.php?module=Workflow&action=RenderActionConfig&context=workflow&entry_id='+idx+'&trigger_module='+this.trigger_module+'&operation_type='+op.operation_type);
		req.loadPostData(cop);
		SUGAR.popups.openUrl(req, null, {width: '800px'});
	};

	this.removeOperation = function(idx) {
		if(idx >= 0 && idx < this.operations.length) {
			this.operations.splice(idx, 1);
			this.renderOperations();
			this.markUpdated();
		}
	};

	this.renderOperations = function() {
		this.clearTable('workflow_operations');
		this.renderAddOperation();
		for(var i = 0; i < this.operations.length; i++)
			this.renderOperation(i);
	};

	this.renderAddOperation = function() {
		var table = $('workflow_operations_footer'),
			opts = this.operation_opts;
		SUGAR.ui.clearChildNodes(table);
		if(! this.editable || ! this.trigger_module) return;
		if(opts) {
			var link = this.createIconLink('insert', 'LNK_ADD_ACTION');
			$('workflow_operations_footer').appendChild(link);
			var src = SUGAR.ui.registerMenuSource(link, {icon_key: 'icon'}, this.form.id);
			SUGAR.ui.attachInputEvent(src, 'onchange', function(k) {
				WorkflowEditor.insertOperation(k);
			});
			opts.width = '350px';
			src.menu.setOptions(new SUGAR.ui.SelectOptions(opts));
		} else {
			var req = new SUGAR.conn.JSONRequest('get_operation_list', {}, {
				module: 'Workflow',
				trigger_module: this.trigger_module});
			req.fetch(function(data) {
				var result;
				if(data && (result = data.getResult())) {
					WorkflowEditor.operation_opts = result;
					WorkflowEditor.renderAddOperation();
				}
			});
		}
	};

	this.renderOperation = function(idx) {
		var op = this.operations[idx],
			table = $('workflow_operations'),
			row = table.tBodies[0].rows[idx],
			cell, result, text, req, buttons;
		if(! op) return;
		if(! row) row = table.tBodies[0].insertRow();
		cell = row.cells[0];
		if(! cell) cell = row.insertCell();
		SUGAR.ui.clearChildNodes(cell);
		req = new SUGAR.conn.HTTPRequest('async.php', {postData:
				{module: 'Workflow', action: 'RenderActionView', context: 'wf_view', entry_id: idx,
				 trigger_module: this.trigger_module, operation_type: op.operation_type,
				 perform_when: op.perform_when, params: JSON.stringify(op.params)}});
		req.fetch(function(data) {
			if(data && (result = data.responseText)) {
				text = SUGAR.util.disableInlineScripts(result);
				var title = MakeElt('div.panel-header.tools-row'),
					outer_cls = 'div.panel-outer.panel-default.panel-border.panel-round';
				title.append(MakeElt('span', '' + (idx + 1) + '. ' + op.label));
				if(WorkflowEditor.editable) {
					buttons = [
						MakeElt('span.uii.active-icon.uii-lg.uii-dots')
							.on('click, keydown[key=enter|space]', function(evt) {
								WorkflowEditor.editOperation(idx); }),
						MakeElt('span.uii.active-icon.uii-lg.uii-remove')
							.on('click, keydown[key=enter|space]', function(evt) {
								WorkflowEditor.removeOperation(idx); })
					];
					title.append(buttons);
				}
				MakeElt(outer_cls + (WorkflowEditor.editable ? '' : '.panel-readonly'), {style: 'margin: 0.35em 0'})
					.append(title)
					.append(MakeElt('div.panel-body').html(text))
					.appendTo(cell);
				SUGAR.util.evalScript(result);
			}
		});
	};


	this.showUserSelect = function() {
			var request_data = {
					form_name: this.form.getAttribute('id') || this.form.getAttribute('name'),
					call_back_function: 'WorkflowEditor.ui_popup_return_user',
					passthru_data: {ui_input_name: this.name || this.id},
					field_to_name_array: this.field_to_name || {id: 'id', label: 'name'}
				},
				params = {module: 'Users', request_data: request_data, inline: true, multiple: true};
			open_popup_window(params);
	};

	this.updatedModule = function() {
		this.clearCriteria();
		this.clearOperations();
		this.toggleSections();
		if(this.criteria) this.criteria.module = this.trigger_module;
	};

	this.toggleSections = function() {
		var show = this.trigger_module ? true : false;
		toggleDisplay('le_section_workflow_criteria_block', show);
		toggleDisplay('le_section_workflow_operations_block', show);
		var sel = SUGAR.ui.getFormInput(this.form, 'add_criteria');
		if(sel) sel.module = this.trigger_module;
	}

	this.clearCriteria = function() {
		this.criteria && this.criteria.clear();
	};

	this.clearOperations = function() {
		this.operations = [];
		this.renderOperations();
		this.markUpdated();
	};

	this.createIconLink = function(icon, text, onclick, attrs) {
		if((icon == 'up' || icon == 'down' || icon == 'remove' || icon == 'edit') && ! text) {
			if(icon == 'remove') icon = 'delete';
			var elt = createElement2('div',
				{className: 'input-icon icon-'+icon+' active-icon', onclick: onclick});
			setAttrs(elt, attrs);
			return elt;
		}

		if(! isPlainObject(attrs))  attrs = {};
		var link = document.createElement('a');
		link.href = '#';
		link.onclick = onclick;
		var title = get_default(attrs.title, '');
		link.title = title;
		link.className = 'utilsLink';
		if(typeof(icon) == 'string')
			icon = get_icon_image(icon);
		if(! blank(icon)) {
			link.appendChild(icon);
			if(text) link.appendChild(nbsp());
		}
		if(text) {
			var elt = document.createTextNode(mod_string(text));
			var lbl = createElement2('span', {className: 'inlineLabel'}, elt);
			link.appendChild(lbl);
		}
		setAttrs(link, attrs);
		return link;
	}

	this.submitConfigForm = function(form) {
		if(! (vresult = SUGAR.ui.validateForm(form)).status) {
			SUGAR.ui.alertInvalidForm(vresult);
			return false;
		}
		if(! SUGAR.ui.beforeSubmitForm(form))
			return false;
		var req = new SUGAR.conn.JSONRequest('save_operation_params'),
			data = req.loadFormValues(form, new SUGAR.conn.JSONArgs()),
			entry_id = form.entry_id.value,
			context = form.context.value,
			jparams;
		if(context != 'listview') {
			if(entry_id.length)
				entry_id = parseInt(entry_id, 10);
			else
				entry_id = null;
		}
		req.setPostData(data.getData());
		req.setArgs({module: 'Workflow'});
		req.fetch(function(data) {
			var response, idx;
			if(data && (response = data.getResult())) {
				if(response.status == 'ok' && response.result.operation_type) {
					SUGAR.popups.close();
					if(context == 'listview') {
						jparams = JSON.stringify(response.result);
						sListView.sendMassUpdate(entry_id, 'wf_'+response.result.operation_type, null, {'wf_params': jparams});
						return;
					}
					if(isset(entry_id)) {
						idx = entry_id;
						WorkflowEditor.operations[idx] = response.result;
					} else {
						idx = WorkflowEditor.operations.length;
						WorkflowEditor.operations.push(response.result);
					}
					WorkflowEditor.renderOperation(idx);
					WorkflowEditor.markUpdated();
				}
			}
		});
	}

	this.ui_popup_return_user = function(result) {
		if (!result)
			return;
		var input = SUGAR.ui.getFormInput(WorkflowEditor.form, 'operation_owner');
		var id = result.selection_list.ID_0.id;
		var name = result.selection_list.ID_0.label;
		value = {name: name, icon: 'icon-user'};
		if(! input.menu.hasKey(id))
			input.menu.addOption(id, value);
		SUGAR.ui.SelectInput.prototype.update.call(input, id, value);
	}

	this.initFieldSelect = function(form, link_name) {
		var link = SUGAR.ui.getFormInput(form, link_name || 'link_name'),
			sel = SUGAR.ui.getFormInput(form, 'add_field');
		if(link && sel) {
			sel.custom_link_name = link.getValue();
		}
	}

	this.updateFieldSelect = function(form, link_name) {
		var upd = SUGAR.ui.getFormInput(form, 'updates');
		if(upd) upd.clear();
		this.initFieldSelect(form, link_name);
	}

	this.markUpdated = function() {
		SUGAR.ui.markFormDirty(this.form);
	}

}();


SUGAR.ui.WFCriteriaInput = function(id, params) {
	if(! params) params = {};
	SUGAR.ui.FilterListRender.call(this, id, params);
};
if(SUGAR.ui.DynamicList)
extendClass(SUGAR.ui.WFCriteriaInput, SUGAR.ui.DynamicList, {
});

var WFCR = SUGAR.ui.WFCriteriaRender = function() {
	this.no_edit_label = true;
}
WFCR.when_ops = ['changed_to', 'current', 'previous', 'updated', 'not_updated'];
if(SUGAR.ui.FilterListRender)
extendClass(SUGAR.ui.WFCriteriaRender, SUGAR.ui.FilterListRender, {
	getWhenOperator: function(group) {
		var op = group.when, idx = WFCR.when_ops.indexOf(op);
		if(idx < 0) {
			op = group.when = 'changed_to';
			idx = 0;
		}
		return mod_string('LBL_FILTER_WHEN_'+op.toUpperCase());
	},
	getWhenOptions: function() {
		if(! WFCR.sel_when_opts) {
			var opts = {keys: WFCR.when_ops, values: [], width: '200px'};
			for(var i = 0; i < opts.keys.length; i++)
				opts.values[i] = mod_string('LBL_FILTER_WHEN_'+opts.keys[i].toUpperCase());
			WFCR.sel_when_opts = new SUGAR.ui.SelectOptions(opts);
		}
		return WFCR.sel_when_opts;
	},
	renderWhen: function(row, field, sect_index, row_index, editor, cell) {
		if(row.type == 'filtergroup') return;
		var lbl = createElement2('span', {className: 'input-field flat rbullet'},
			createElement2('div', {className: 'input-label single'}, this.getWhenOperator(row)));
		var render = this;
		lbl.onclick = function(evt) {
			var edit = new SUGAR.ui.QuickSelect({options: render.getWhenOptions()});
			edit.init(row.when, function(key, val) {
				editor.updateRow(sect_index, row_index, {when: key});
			});
			edit.showPopup(0, this);
			SUGAR.ui.stopEvent(evt);
		}
		return [lbl];
	},
	renderFilter: function(row, field, sect_index, row_index, editor, cell) {
		if(row.when == 'updated' || row.when == 'not_updated')
			return;
		return SUGAR.ui.FilterListRender.prototype.renderFilter.call(this, row, field, sect_index, row_index, editor, cell);
	}
});

});


