<?php
/**
 * The contents of this file are subject to the SugarCRM Public License Version
 * 1.1.3 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.sugarcrm.com/SPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by SugarCRM" logo and
 *    (ii) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: SugarCRM Open Source
 * The Initial Developer of the Original Code is SugarCRM, Inc.
 * Portions created by SugarCRM are Copyright (C) 2004-2006 SugarCRM, Inc.;
 * All Rights Reserved.
 * Contributor(s): ______________________________________.
 */



define('sugarEntry', true);
define('ONECRM_CSRF_PROTECT', 'none');
require_once 'include/entryPoint.php';
require_once 'include/utils/longreach_utils.php';
require_once 'include/database/RowUpdate.php';
require_once 'include/database/ListQuery.php';
require_once 'modules/Campaigns/utils.php';

AppConfig::restrict_entry_point();

$track = (string)array_get_default($_REQUEST, 'track', '');
$identifier = (string)array_get_default($_REQUEST, 'identifier', '');

$msg = '';

global $current_user;

if (!empty($identifier) && !defined('ONECRM_OPTIN')) {
	$keys = CampaignActivityTracker::log_campaign_activity($identifier, 'removed', true, $track);
    $mod_strings = return_module_language('base', 'Campaigns');
	if (!empty($keys) && $keys['target_type'] == 'Users'){
        //Users cannot opt out of receiving emails, print out warning message.
        $msg = $mod_strings['LBL_USERS_CANNOT_OPTOUT'];
    } elseif(!empty($keys) && isset($keys['campaign_id']) && !empty($keys['campaign_id'])) {
        //we need to unsubscribe the user from this particular campaign
        //require_once('include/modules.php');
		$model = AppConfig::module_primary_bean($keys['target_type']);
		if (require_bean($model)) {
	        $focus = new $model;
	        $focus->retrieve($keys['target_id']);
			CampaignUtils::unsubscribe($keys['campaign_id'], $focus);

			$target = ListQuery::quick_fetch($model, $keys['target_id']);
			if ($target) {
				$upd = RowUpdate::for_result($target);
				$upd->as_user_id = $target->getField('assigned_user_id');
				$upd->set('email_opt_in', 0);
				$upd->set('email_opt_in_date', null);
				$upd->save();
			}
		}
		$msg .= $mod_strings['LBL_ELECTED_TO_OPTOUT'];
    } elseif (!empty($keys)) {
		$id = $keys['target_id'];
		$model = AppConfig::module_primary_bean($keys['target_type']);
		//no opt out for users.
		if ($id && $model != 'User') {
			$row = ListQuery::quick_fetch($model, $id);
			if ($row) {
				$update = RowUpdate::for_result($row);

				$update->set('email_opt_out', 1);
				$update->set('email_opt_in', 0);
				$update->set('email_opt_in_date', null);
				$update->as_user_id = $row->getField('assigned_user_id');
				
				try {
					$status = $update->save();
				} catch (Exception $e) {
					$status = false;
				}
				if(! $status)
					$msg .= "*";
			}
	        //Print Confirmation Message.
		    $msg .= $mod_strings['LBL_ELECTED_TO_OPTOUT'];
		}
	}
} elseif (!empty($identifier)) {
    $mod_strings = return_module_language('base', 'Campaigns');
	CampaignActivityTracker::log_campaign_activity($identifier, 'optin', true, $track);
	$lq = new ListQuery('CampaignLog');
	$lq->addSimpleFilter('target_tracker_key', $identifier);
	$lq->addSimpleFilter('activity_type', 'targeted');
	$row = null;
	$res = $lq->runQuerySingle();
	if ($res) {
		$target_type = $res->getField('target_type');
		$target_id = $res->getField('target_id');
		switch ($target_type) {
			case 'Prospects':
				$model = 'Prospect';
				break;
			case 'Contacts':
				$model = 'Contact';
				break;
			case 'Leads':
				$model = 'Lead';
				break;
			default:
				$model = null;
		}
		if ($model) {
			$target = ListQuery::quick_fetch($model, $target_id);
			if ($target) {
    			$current_user = $current_user->getSystemUser();
				$upd = RowUpdate::for_result($target);
				$upd->set('email_opt_in', 1);
				$upd->set('email_opt_in_date', gmdate('Y-m-d'));
				$upd->save();
			}
		}
	}
	$msg = $mod_strings['LBL_ALREADY_SUBSCRIBED_HEADER'];
}


if (!empty($track)) {
	$tracker = ListQuery::quick_fetch_row('CampaignTracker', $track);
	if ($tracker) {
		$url = $tracker['tracker_url'];
		if ($url && $url != "removeme.php") { // removeme.php is here for BC
			header("Location: $url");
			sugar_cleanup(true);
		}
	}
	if (! empty($tracker['content']))
		$msg = $tracker['content'];
}

sugar_cleanup(true, $msg);
