<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry'))define('sugarEntry', true);

define('ONECRM_CSRF_PROTECT', 'none');
define('SCHEDULER_EXTERNAL_TRIGGER', true);

chdir(dirname(__FILE__));
if(isset($_SERVER['PHP_SELF']))
	$_SERVER['PHP_SELF'] = basename($_SERVER['PHP_SELF']);

if(php_sapi_name() == 'cli') {
	// switch to owner of the current file to preserve file permissions
	if(function_exists('posix_getpwuid')) {
		$puser = posix_getpwuid(posix_geteuid());
		if($puser && $puser['name'] == 'root') {
			$uid = getmyuid();
			$gid = getmygid();
			if($uid !== false && $uid != $puser['uid']) {
				posix_setegid($gid);
				posix_seteuid($uid);
			}
		}
	}
}

require_once('include/entryPoint.php');

if(is_demo_site())
	die('Scheduler disabled for demo site.');
if(AppConfig::setting('site.upgrade_in_progress'))
	die('Scheduler disabled during site upgrade.');
// Prevent access via alternate site urls
if(AppConfig::is_extern_url())
	return;

require_once('modules/Scheduler/Schedule.php');

//error_reporting(E_ALL);
//ini_set('display_errors', '1');

global $current_language, $app_strings, $app_list_strings;
$current_language = AppConfig::setting('locale.defaults.language');
$app_strings = return_application_language($current_language);
$app_list_strings = return_app_list_strings_language($current_language);

if(! AppConfig::session_start()) {
	$GLOBALS['log']->fatal('Error starting session');
	die('Error starting session');
}

// call session_destroy, so that filesystem is not flooded
// with session files. We do not need to keep session data
// between scheduler calls, and CLI cannot resume session
// anyway
session_destroy();

$current_user = new User;
$current_user->retrieveAdminUser();

scheduler_update_config();

$type = array_get_default($_REQUEST, 'schedule_type');
$force = false;

function parseSchedulerOptions() {
	global $argc, $argv;
	$ret = [
		'force' => false,
		'quiet' => false,
		'type' => null,
	];
	for ($i = 1; $i < $argc; $i++) {
		$opt = $argv[$i];
		switch ($opt) {
		case '--force':
			$ret['force'] = true;
			break;
		case '--type':
			if ($argc < $i)
				die('--type option requeres an argument');
			$ret['type'] = $argv[$i+1];
			$i++;
			break;
		case '--quiet':
			$ret['quiet'] = true;
			break;
		default:
			die('invalid option: ' . $opt);
		}
	}
	return $ret;
}

$progressIndicator = null;
if(php_sapi_name() == 'cli') {
	extract(parseSchedulerOptions());
	if (!$quiet) {
		require_once('include/ProgressIndicator.php');
		$pid = scheduler_progress_id();
		$ppath = scheduler_progress_path($pid);
		echo translate('LBL_RUNNING', 'Scheduler');
		$progressIndicator = new ProgressIndicator($ppath, true, true);
	}
}

if($force && $type)
	scheduler_run_type($type, true, $progressIndicator);
else
	scheduler_run_pending($type, '', $progressIndicator);

if(isset($progressIndicator)) {
	$progressIndicator->finish('success', translate('LBL_RUN_OVER', 'Scheduler'));
	$progressIndicator->cleanup();
}

sugar_cleanup();

