<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
define('sugarEntry', 1);
define('ONECRM_CSRF_PROTECT', 'none');

require_once('include/entryPoint.php');

AppConfig::restrict_entry_point();
if(AppConfig::is_extern_url('survey')) {
	return;
}

if(! AppConfig::session_start()) {
	header('HTTP/1.1 503 Service Unavailable');
	die('Error starting session');
}

require_once('include/Notify/RecordActionMessage.php');

$contact = null;
$module = array_get_default($_REQUEST, 'module');
if($module == 'Cases') {
	$record = array_get_default($_REQUEST, 'record');
	$contact_id = array_get_default($_REQUEST, 'contact_id');
	if($record && $contact_id) {
		$case = ListQuery::quick_fetch('aCase', $record);
		$contact_query = $case->getLinkQuery('contacts');
		$contact_query->addFilterPrimaryKey($contact_id);
		$contact = $contact_query->runQuerySingle();
		if($contact && $contact->failed)
			$contact = null;
	}
}

if(! $contact) {
	NotifyErrorMessage::show_error(
		array('LBL_SURVEY_NOT_FOUND', 'Notify'),
		translate('LBL_TITLE_SURVEY_ERROR', 'Notify')
	);
	sugar_cleanup(true);
}

$rating = array_get_default($_REQUEST, 'rating', '');
$rated = strlen($rating) > 0;
$rating = min(5, max(0, (int)$rating));
$commented = ! empty($_REQUEST['commented']);
$accept_comments = AppConfig::setting('company.case_survey_comments', true);
$comments = $accept_comments ? trim(array_get_default($_REQUEST, 'survey_comments')) : null;

if ($rating) {
	global $current_user;
	$current_user = new User();
	$queue_user_id = AppConfig::setting('company.case_queue_user');
	if(! $queue_user_id) {
		$current_user->getSystemUser();
	} else {
		$current_user->retrieve($queue_user_id);
	}

	$upd = RowUpdate::for_result($case);
	$upd->set('rating', $rating);
	$upd->save();
}

if($commented) {
	if(strlen($comments)) {
		$note = RowUpdate::blank_for_model('Note', $case->getField('assigned_user_id'));
		$note->set(array(
			'name' => translate('LBL_SURVEY_COMMENTS', 'Cases'),
			'description' => $comments,
			'parent_type' => 'Cases',
			'parent_id' => $case->getPrimaryKeyValue(),
			'account_id' => $case->getField('account_id'),
			'contact_id' => $contact_id,
			'assigned_user_id' => $case->getField('assigned_user_id'),
		));
		if($note->validate() && $note->save()) {
			// TODO - send notification to user with survey comments
		}
	}
}

$message_action = 'survey';
if($rated) {
	$message_action = 'survey_thanks';
	if($accept_comments && ! $commented)
		$message_action = 'survey_comments';
}
$message = RecordActionMessage::for_action('aCase', $message_action);
if($message) {
	$message->addRecordId($case->getPrimaryKeyValue());
	$message->accept_comments = $accept_comments;
	$message->comments = $comments;
	$notify = NotifyFormatter::create_standalone();
	$notify->setTargetId('Contact', $contact_id);
	echo $notify->renderMessage($message);
} else {
	$GLOBALS['log']->error("Error preparing survey page: no notification found");
}

sugar_cleanup();

?>
