"""Script used to combine theme icons to form icon-strip.png sprite images.
   Meant to be run from inside the themes/ directory of the installation.
   Requires PIL for the Image module.
"""

import glob
from PIL import Image, ImageEnhance
import sys
import re
import math
import os


all = ('Default',) #('Delight', 'Spectrum', 'LongReach', 'ClassicSide', 'Mobile')
if len(sys.argv) > 1:
        selected = sys.argv[1:]
else:
        selected = all

HORIZ = True
SIZE = 16
SPACE = 2
LENGTH = 60

class Theme:
        pass

def load_images(theme):
        path = theme+"/images/";
        ret = Theme()
        ret.name = theme
        ret.path = path
        ret.path32 = theme+"/images32/";
        ret.names = []
        ret.dims = {}
        for fpath in glob.glob(path + "*.gif") + glob.glob(path + "*.gif.png"):
                if fpath[len(fpath)-4:] == ".png":
                        name = fpath[len(path):-8]
                else:
                        name = fpath[len(path):-4]
                if name == 'sqsWait':
                        continue
                #if not re.match("[A-Z][a-z]", f):
                #       continue
                img = Image.open(fpath, "r")
                img.load()
                if img.info.get('loop', 0): # skip animated (sqsWait.gif)
                        del img
                        continue
                if img.size[0] <= 16 and img.size[1] <= 16 and img.size[0] != 1 and img.size[1] != 1:
                        ret.names.append(name)
                        if img.size != (16,16):
                                ret.dims[name] = img.size
                del img
        return ret


#default = load_images('Default')

for theme_name in selected:
        OFFSET = SIZE + SPACE
        theme = load_images(theme_name)
        if theme_name != 'Default':
                names = set(default.names)
                names.update(theme.names)
        else:
                names = set(theme.names)
        names = list(names)
        names.sort()
        names.insert(0, "")
        count = len(names)
        rows = int(math.ceil(float(count) / LENGTH))
        dims = {}
        load32 = {}
        
        if HORIZ:
                sprite = Image.new("RGBA", (LENGTH*OFFSET, rows*OFFSET))
        else:
                sprite = Image.new("RGBA", (rows*OFFSET, LENGTH*OFFSET))
        for i,name in enumerate(names):
                if not name:
                        continue
                x = i % rows
                y = i / rows
                if name in theme.names:
                        fname = theme.path + name + ".gif.png"
                        if not os.path.exists(fname):
                                fname = theme.path + name + ".gif"
                        img = Image.open(fname, "r")
                        if name in theme.dims:
                                dims[name] = theme.dims[name]
                        img32 = theme.path32 + name + ".png"
                        if os.path.exists(img32):
                                load32[i] = img32
                else:
                        fname = default.path + name + ".gif.png"
                        if not os.path.exists(fname):
                                fname = default.path + name + ".gif"
                        img = Image.open(fname, "r")
                        if name in default.dims:
                                dims[name] = default.dims[name]
                        img32 = default.path32 + name + ".png"
                        if os.path.exists(img32):
                                load32[i] = img32
                img.load()
                if HORIZ:
                        sprite.paste(img, (OFFSET*(i%LENGTH), OFFSET*int(i/LENGTH)))
                else:
                        sprite.paste(img, (OFFSET*int(i/LENGTH), OFFSET*(i%LENGTH)))
                del img
        sprite.save(theme.path + "theme-icons.png", optimize=True)
        
        # add greyscale
        grey = ImageEnhance.Brightness(sprite.convert('LA')).enhance(1.1)
        grey.save(theme.path + "theme-icons-grey.png", optimize=True)
        del grey
        
        # add 32x32 spriteset
        size32 = (sprite.size[0] * 2, sprite.size[1] * 2)
        hires = sprite.resize(size32, Image.NEAREST)
        for (i,path) in load32.items():
                img = Image.open(path, "r")
                hires.paste(img, (OFFSET*(i%LENGTH)*2, OFFSET*int(i/LENGTH)*2))
        hires.save(theme.path32 + "theme-icons.png", optimize=True)

        # add greyscale
        grey = ImageEnhance.Brightness(hires.convert('LA')).enhance(1.1)
        grey.save(theme.path32 + "theme-icons-grey.png", optimize=True)
        del hires
        del grey

        del sprite
        
        fphp = open(theme.name + '/theme-icons-base.php', 'w')
        fphp.write("<?php\n$icon_defs = array(")
        fphp.write("\n\"orientation\" => \"" + ("horizontal" if HORIZ else "vertical") + "\",")
        fphp.write("\n\"spacer\" => " + str(SPACE) + ",")
        fphp.write("\n\"length\" => " + str(LENGTH) + ",")
        fphp.write("\n\"dims\" => array(")
        dk = list(dims.keys())
        dk.sort()
        for f in dk:
                fphp.write("\n\""+f+"\" => \""+str(dims[f][0])+","+str(dims[f][1])+"\",")
        fphp.write("\n),\n\"images\" => array(")
        for f in names:
                if f == 'Shippers': f = 'ShippingProviders'
                fphp.write("\n\""+f+"\",")
        fphp.write("\n)\n);\n?>\n")
        fphp.close()
        
        if 0:
                fjs = open(theme.path + 'theme-icons.js', 'w')
                fjs.write("if(! SUGAR.theme_icons) SUGAR.theme_icons={};\nSUGAR.theme_icons[\""+theme+"\"]={")
                fjs.write("\"orientation\": \"" + ("horizontal" if HORIZ else "vertical") + "\","),
                fjs.write("\"dims\": {")
                fjs.write( ",".join( ("\""+f+"\": \""+str(d[0])+","+str(d[1])+"\"") for f,d in dims.items() ) )
                fjs.write("}, \"images\": [")
                fjs.write( ",".join("\""+f+"\"" for f in names) )
                fjs.write("]};\n")
                fjs.close()
