<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
ini_set("memory_limit", "-1");
set_time_limit(600);
if(! defined('sugarEntry')) { define('sugarEntry', true); }
define('IAH_IN_UPGRADE', true);

require_once('include/entryPoint.php');
require_once('modules/UWizard/UWizard.php');
require_once('modules/Administration/upgrade_utils.php');

// Check all necessary entries (installed, update, authenticated, admin)
if(! AppConfig::installed()) { indexRedirect(array('path' => 'install.php')); }
$upgrade_info = UWizard::getUpgradeStatus();
if(! $upgrade_info) { indexRedirect(array('path' => 'index.php')); }

if (! AppConfig::session_start()) {
	header('HTTP/1.1 503 Service Unavailable');
	die('Error starting session, please check your php.ini settings.');
}
if(! handleLogin($upgrade_info)) {
	// form displayed
	return;
}

if(! empty($_POST['cancel_upgrade'])) {
	UWizard::cancelUpgrade();
	header('Location: index.php?module=UWizard&action=index');
}

if(! empty($_POST['confirm_upgrade'])) {
	do {
		try {
			$result = UWizard::runCommitStep();
		} catch(UWizardError $e) {
			$result = array(
				'status' => 'error',
				'label' => 'An unexpected error occurred',
				'message' => $e->getMessage(),
			);
		}
		$content = '';
		$header = array_get_default($result, 'label');

		if(! empty($result['message'])) {
			$content .= '<p class="notice">' . $result['message'] . '</p>';
		}
		if(! empty($result['messages'])) {
			$content .= '<ul>';
			foreach($result['messages'] as $msg) {
				$content .= "<li>$msg</li>";
			}
			$content .= '</ul>';
		}
		if(! empty($result['errors'])) {
			$content .= '<div class="alert alert-danger"><span class="glyphicon glyphicon-exclamation-mark"></span> The following errors were reported:';
			$content .= '<ul>';
			foreach($result['errors'] as $msg) {
				$content .= "<li>$msg</li>";
			}
			$content .= '</ul></div>';
		}
		
		if($content) {
			$content .= '<input name="confirm_upgrade" value="1" type="hidden" />';
			if(UWizard::getUpgradeStatus()) {
				$buttons = array(array(
					'type' => 'submit',
					'name' => 'runUpgrade',
					'onclick' => "changeButtonLabel(this, 'Please wait...'); setTimeout('document.forms.upgradeForm.runUpgrade.disabled=true;', 50);",
					'label' => 'Continue',
					'icon' => 'arrow-right',
					'class' => 'lw-iconbutton lwib-accept',
				));
			} else {
				$buttons = array(array(
					'type' => 'button',
					'name' => 'finishUpgrade',
					'onclick' => "document.location.href='index.php?module=UWizard&action=index';",
					'label' => 'Finish Upgrade',
					'icon' => 'accept',
					'class' => 'lw-iconbutton lwib-accept',
				));
			}
			
			outputForm(compact('content', 'header', 'buttons'), $upgrade_info);
			break;
		}
	} while(1);	
} else {
	showConfirmation($upgrade_info);
}


function showConfirmation($upgrade) {
	global $upgrade_info;
	
	$msg = translate($upgrade['perform'] == 'install' ? 'LBL_PRE_INSTALL' : 'LBL_PRE_UNINSTALL', 'UWizard');
	$content = <<<EOF
<p class="notice">{$msg}</p>
<input name="confirm_upgrade" value="1" type="hidden" />
EOF;
	$warn = $disableCommit = null;
	if(! empty($upgrade['source'])) {
		$uw = UWizard::create($upgrade['source'], $upgrade['perform']);
		$manifest = $uw->getManifest();
		if($manifest)
			$warn = $manifest->path('commit_warnings');
	}
	if($warn) {
		$warn_filt = array();
		foreach($warn as $w) {
			if(is_array($w)) {
				if(isset($w['before_version']) && $w['before_version']) {
					if(isset($upgrade_info['start_version']) && version_compare($upgrade_info['start_version'], $w['before_version'], '>='))
						continue;
				}
				if(isset($w['text']))
					$w = $w['text'];
				else
					continue;
			}
			if(is_string($w))
				$warn_filt[] = $w;
		}
		$warn = $warn_filt;
	}
	if($warn) {
		$disableCommit = 1;
		$scr = <<<EOQ
			function checkUwWarnings() {
				var i, elt, btn = document.getElementById('startUpgrade'), ok = 1;
				for(i = 0; i < 10; i++) {
					elt = document.getElementById('uw_warning' + i);
					if(elt && ! elt.checked)
						ok = 0;
				}
				if(btn) btn.disabled = ok ? '' : 'disabled';
			}
EOQ;
		$content .= '<div class="panel-outer panel-alert panel-border panel-round" style="width: 90%; margin: 1em auto">';
		$content .= '<div class="panel-header">Upgrade Warnings</div>';
		$content .= "<div class=\"panel-body panel-pad-body\">";
		$wi = 0;
		foreach($warn as $w) {
			$in = 'uw_warning' . ($wi++);
			$content .= "<div class=\"row\">";
			$content .= "<div class=\"column sm-1 text-right\">";
			$content .= "<div class=\"input-check\" style=\"margin-right: 0.5rem\"><input id=\"$in\" name=\"$in\" onclick=\"checkUwWarnings()\" type=\"checkbox\" value=\"1\">";
			$content .= "<label></label></div>";
			$content .= '</div><div class="column sm-11">';
			$content .= "<label for=\"$in\">".$w."</label></div></div>";
		}
		$content .= '</div></div>';
		$content .= "<script type=\"text/javascript\">$scr</script>";
	}
	$buttons = array(
		array(
			'type' => 'button',
			'name' => 'cancelUpgrade',
			'id' => 'cancelUpgrade',
			'onclick' => "this.form.confirm_upgrade.name='cancel_upgrade';this.form.submit();",
			'label' => 'Cancel',
			'icon' => 'cancel',
		),	
		array(
			'type' => 'submit',
			'name' => 'startUpgrade',
			'id' => 'startUpgrade',
			'onclick' => "changeButtonLabel(this, 'Please wait...'); setTimeout('document.forms.upgradeForm.startUpgrade.disabled=true;document.forms.upgradeForm.cancelUpgrade.disabled=true', 50);",
			'label' => 'Continue',
			'disabled' => $disableCommit,
			'icon' => 'arrow-right',
			'class' => 'lw-iconbutton lwib-accept',
		),
	);
	$header = translate('LBL_STEP_CONFIRM', 'UWizard');
	outputForm(compact('content', 'header', 'buttons'), $upgrade);
}

function outputForm($params, $upgrade) {
	$def_title = '1CRM ';
	if($upgrade['type'] == 'patch')
		$def_title .= $upgrade['version'] . ' Upgrade';
	else
		$def_title = $upgrade['name'] . ' ' . $upgrade['version'];
	$def_title = translate($upgrade['perform'] == 'install' ? 'LBL_INSTALLING' : 'LBL_UNINSTALLING', 'UWizard') . ': ' . $def_title;
	$title = array_get_default($params, 'title', $def_title);
	$header = array_get_default($params, 'header', '');
	$content = array_get_default($params, 'content', '');
	$buttons = array_get_default($params, 'buttons', array());
	$jsver = js_version_params() . '&upg';
	$csrf_input = create_csrf_input();

echo <<<EOQ
<!DOCTYPE html>
<html>
<head>
<meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1">
<meta charset="utf-8">
<meta name="viewport" content="width=device-width, initial-scale=1">
<link rel="shortcut icon" href="favicon.ico" type="image/vnd.microsoft.icon">
<link rel="icon" href="favicon.ico" type="image/vnd.microsoft.icon">
<title>{$title}</title>
<link type="text/css" rel="stylesheet" href="themes/Default/base.css?{$jsver}">
<link type="text/css" rel="stylesheet" href="themes/Default/upgrade.css?{$jsver}">
<script type="text/javascript" src="include/javascript/base.js?{$jsver}"></script>
<script type="text/javascript" src="include/javascript/lightweight.js?{$jsver}"></script>
<script type="text/javascript" src="include/javascript/conn.js?{$jsver}"></script>
</head>
<body class="upgrade with-form" onload="defaultFocus()">
<div class="container lw-outer">
<div class="row lw-head">
	<a href="http://www.1crm.com/1crm.html" target="_blank"><img src="include/images/iah/1crm-logo.svg?{$jsver}" width="180" height="auto" alt="1CRM"></a>
</div>

<form action="upgrade.php" method="post" name="upgradeForm" class="form-outer lw-form" enctype="multipart/form-data" autocomplete="off">
{$csrf_input}
<div class="row page-row lw-main">
<div class="row">
	<div class="column page-column">
		<h2 class="lw-lead">{$title}</h2>
	</div>
</div>
EOQ;

	//echo '<div style="max-width: 700px; margin: 0 auto">';
	if($header) echo "<div class=\"row\"><div class=\"column page-column\"><h3>$header</h3></div></div>";
	echo '<div class="row page-row">';
	echo '<div class="column page-column">';
    echo $content;
    echo '</div></div>';
	
	if($buttons) {
		echo '<div class="row page-row lw-form-buttons">';
		echo '<div class="column">';
	}
	foreach($buttons as $b) {
		$label = $b['label'];
		unset($b['label']);
		if(! isset($b['type'])) $b['type'] = 'button';
		if(! isset($b['class'])) $b['class'] = ! empty($b['icon']) ? 'lw-iconbutton' : 'btn btn-default';
		if(isset($b['icon'])) {
			$label = '<span class="uii uii-'.$b['icon'].'"></span> <span class="btn-label">' . $label . '</span>';
			unset($b['icon']);
		}
		$attrs = array();
		foreach($b as $attr => $val) {
			if(isset($val))
				$attrs[] = $attr . '="' . to_html($val) . '"';
		}
		echo ' <button ' . implode(' ', $attrs) . '>' . $label . '</button>';
	}
	if($buttons)
		echo '</div></div>';
	
	echo '</div></body></html>';
}

function handleLogin($info) {
	$auth = new AuthenticationController();
	if(! empty($_POST['do_login']) && ! empty($_POST['user_name'])) {
		$auth->login($_POST['user_name'], $_POST['user_password']);
	}
	if(! empty($_SESSION['authenticated_user_id'])) {
		$uid = array_get_default($info, 'user_id');
		if(! $auth->sessionAuthenticate() || ($uid && $_SESSION['authenticated_user_id'] != $uid) || ! AppConfig::is_admin())
			session_destroy();
		else
			return true;
	}
	
	$content = <<<EOF
<h4>Please enter credentials for the 1CRM Administrator who started this upgrade:</h4><br>
<input name="do_login" value="1" type="hidden">
<div class="row form-group">
<div class="column sm-4">User Name</div>
<div class="column sm-8">
	<input name="user_name" type="text" size="25" class="form-control default-focus">
</div></div>
<div class="row form-group">
<div class="column sm-4">Password</div>
<div class="column sm-8">
	<td><input name="user_password" type="password" size="25" class="form-control">
</div></div>
EOF;

	$buttons = array(
		array(
			'type' => 'submit',
			'name' => 'startUpgrade',
			'id' => 'startUpgrade',
			'onclick' => "changeButtonLabel(this, 'Please wait...'); setTimeout('document.forms.upgradeForm.startUpgrade.disabled=true', 50);",
			'label' => 'Login',
			'icon' => 'arrow-right',
			'class' => 'lw-iconbutton lwib-accept',
		),
	);

	outputForm(compact('content', 'buttons'), $info);
}


function indexRedirect($params) {
	if(isset($params['path']))
		$path = $params['path'];
	else {
		$path = array();
		foreach($params as $p => $v)
			$path[] = $p.'='.urlencode($v);
		$path = 'index.php?'.implode('&', $path);
	}
	header('Location: '.$path);
	if(function_exists('sugar_cleanup'))
		sugar_cleanup(true);
	else
		exit;
}


?>
