<?php
/*
 *
 * The contents of this file are subject to the info@hand Software License Agreement Version 1.3
 *
 * ("License"); You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at <http://1crm.com/pdf/swlicense.pdf>.
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for the
 * specific language governing rights and limitations under the License,
 *
 * All copies of the Covered Code must include on each user interface screen:
 * (i) the 1CRM copyright notice,
 * (ii) the "Powered by the 1CRM Engine" logo, 
 *
 * (iii) the "Powered by SugarCRM" logo, and
 * (iv) the SugarCRM copyright notice
 * in the same form as they appear in the distribution.
 * See full license for requirements.
 *
 * The Original Code is : 1CRM Engine proprietary commercial code.
 * The Initial Developer of this Original Code is 1CRM Corp.
 * and it is Copyright (C) 2004-2012 by 1CRM Corp.
 *
 * All Rights Reserved.
 * Portions created by SugarCRM are Copyright (C) 2004-2008 SugarCRM, Inc.;
 * All Rights Reserved.
 *
 */
if(!defined('sugarEntry'))define('sugarEntry', true);

define('ONECRM_CSRF_PROTECT', 'strict');
require_once('include/json/json_utils.php');
json_handle_magic_quotes_gpc();

define('IAH_JSON', true);
define('DISABLE_CLEAN_INCOMING_DATA', 1);
require_once('include/entryPoint.php');

require_once('include/JSON.php');
require_once('include/upload_file.php');

define('UPLOAD_MAX_COUNT', 20);

error_reporting(error_reporting() & ~E_NOTICE);

AppConfig::restrict_entry_point();
// Prevent access via alternate site urls
if(AppConfig::is_extern_url())
	return;

AppConfig::session_start();

$authController = new AuthenticationController();
$not_auth = false;
$current_user = new User();
$authController = new AuthenticationController();
if(empty($_SESSION['authenticated_user_id']))
	$not_auth = true;
else {
	if(! $authController->sessionAuthenticate()) {
		session_destroy();
		$not_auth = true;
	}
}
if ($not_auth)
	die();

//ob_start();
upload_perform();
sugar_cleanup(true);


function upload_perform() {
	$user_id = array_get_default($_REQUEST, 'user_id');
	// user_id is not validated - provide an invalid ID and the file will not be accessible
	if(! $user_id || ! preg_match('~^[0-9a-zA-Z\-_]+$~', $user_id))
		json_bad_request("Missing user_id");
	$setvar = array_get_default($_REQUEST, 'setvar');
	
	$ret = array('status' => 200, 'errors' => array(), 'files' => array());
	$i = null;
	$c = 0;
	while(1) {
		$param = 'file';
		if(isset($i))
			$param .= ($i ++);
		else
			$i = 0;
		if(! isset($_FILES[$param])) {
			if($i) break;
			continue;
		}
		$info = array(
			'id' => '',
			'meta' => array_get_default($_REQUEST, $param . '_meta'),
			'module' => array_get_default($_REQUEST, $param . '_module'),
			'model' => array_get_default($_REQUEST, $param . '_model'),
			'field' => array_get_default($_REQUEST, $param . '_field'),
			'error' => '',
		);
		if($info['module'] && ! $info['model'])
			$info['model'] = AppConfig::module_primary_bean($info['module']);
		if($info['model'] && ! $info['module'])
			$info['module'] = AppConfig::module_for_model($info['model']);
		$upload_params = $info;
		if($info['model'] && $info['field']) {
			$spec = null;
			if(preg_match('~^[a-zA-Z_]\w*$~', $info['model']) && preg_match('~^[a-zA-Z_]\w*$~', $info['field'])) {
				$spec = AppConfig::setting("display.fields.{$info['model']}.{$info['field']}");
				if(! $spec) $spec = AppConfig::setting("model.fields.{$info['model']}.{$info['field']}");
			}
			if($spec) {
				$t = array_get_default($spec, 'type');
				if($t == 'file_ref')
					; // fine
				else if($t == 'image') {
					$upload_params['is_image'] = 1;
					$upload_params['max_size'] = array_get_default($spec, 'max_size');
					$do_thumb = array_get_default($spec, 'thumbnail');
					if($do_thumb === 'auto') {
						$thumb = AppConfig::setting("model.fields.{$info['model']}.{$info['field']}_thumb");
						if($thumb) {
							$upload_params['thumbnail'] = true;
							$upload_params['thumbnail_size'] = array_get_default($spec, 'thumbnail_size');
						}
					}
				}
				else
					$info['error'] = 'Unknown field';
			} else {
				$info['error'] = 'Unknown field';
			}
		}

		if(! $info['error']) {
			$uf = new UploadFile($param);
			try {
				if($uf->confirm_upload()) {
					$id = $uf->save_incoming($user_id, $upload_params);
					$info['id'] = $id;
				}
			} catch(IAHUploadError $e) {
				$GLOBALS['log']->error($e->getMessage());
				$info['error'] = translate('ERR_UPLOAD_FAILED', 'app');
			}
		}
		
		if ($info['error'] && !empty($_REQUEST['fail_on_first_error'])) {
			json_bad_request($info['error']);
		}

		if($info['id'] && $upload_params['is_image']) {
			$inc = ListQuery::quick_fetch_row('incoming_files', $info['id']);
			$info['is_image'] = 1;
			$info['path'] = '';
			$info['width'] = $info['height'] = 0;
			$info['thumbnail'] = '';
			$info['thumbnail_width'] = $info['thumbnail_height'] = 0;
			$inc_path = CacheManager::get_location('incoming/');
			if($inc['filename'] && ($img = resolve_image($inc_path . $inc['filename']))) {
				list($path, $info['width'], $info['height']) = $img;
				$info['path'] = 'download.php?type=incoming&id=' . $info['id'] . '&inline=1';
			}
			if($inc['thumbnail_filename'] && ($thumb = resolve_image($inc_path . $inc['thumbnail_filename']))) {
				list($path, $info['thumbnail_width'], $info['thumbnail_height']) = $thumb;
				$info['thumbnail'] = 'download.php?type=incoming&id=' . $info['id'] . '&thumb=1&inline=1';
			}
		}

		$ret['files'][$param] = $info;
		if($info['error'])
			$ret['errors'][$param] = $info['error'];
				
		if($c ++ > constant('UPLOAD_MAX_COUNT')) break;
	}
	
	register_cleanup_handler(array('UploadFile', 'clean_expired_incoming'));
	
	if(! $c) {
		json_bad_request('No files provided');
	}
	return json_return_value($ret, $setvar);
}

