
AppBase.extendModule('ui', 'ui-controls', function(App) {

	var UI = this,
		Dom = App.dom,
		Query = Dom.query,
		QueryFirst = Dom.queryFirst,
		QueryId = Dom.queryId,
		Wrap = Dom.extNode,
		MakeElt = Dom.makeElement;
	UI.loaded = true;
	UI.conditional_fields = [];

	// -- Classes -- //

	// object to manage all popup windows currently open and direct mouse/keyboard events
	UI.PopupManager = {
		modal_stack: [],
		current_modal: null,
		open_stack: [],
		last_opened: null,
		loading_popup: null,
		uniqid: 0,
		popupBeforeShow: function(popup) {
			this.checkMobile();
			if(! popup || popup === this.last_opened) {
				return;
			}
			if(! popup.stacked) {
				while(this.last_opened && this.last_opened !== this.current_modal) {
					if(this.last_opened === popup)
						return;
					var old = this.last_opened;
					this.last_opened = this.open_stack.pop();
					old.close();
				}
			}
			if(this.last_opened)
				this.open_stack.push(this.last_opened);
			this.last_opened = popup;
			this.attachEvents();
		},
		popupBeforeHide: function(popup) {
			if(popup.modal && ! popup.forms_detached) {
				var forms = Wrap(popup.elt).find('form');
				for(var idx = 0; idx < forms.length; idx++) {
					UI.beforeDetachForm(forms[idx]);
				}
				popup.forms_detached = true;
			}
		},
		canceledShow: function(popup) {
			this.popupHidden(popup);
		},
		popupShown: function(popup) {
			if(! popup || popup !== this.last_opened) {
				return;
			}
			if(popup.modal && popup !== this.current_modal) {
				var old = this.current_modal;
				if(old)
					this.modal_stack.push(old);
				else if(! popup.hide_mask)
					this.showModalMask(popup.invis_mask);
				this.current_modal = popup;
				if(this.last_opened === old)
					this.last_opened = null;
				if(old)
					old.hide();
			}
			if(popup.forms_detached) {
				var forms = Wrap(popup.elt).find('form');
				for(var idx = 0; idx < forms.length; idx++) {
					UI.beforeRestoreForm(forms[idx]);
				}
				popup.forms_detached = false;
			}
		},
		popupHidden: function(popup) {
			if(popup.modal) {
				if(this.last_opened && this.last_opened.target && popup.elt && popup.elt.contains(this.last_opened.target)) {
					this.last_opened.close();
				}
			}
			if(popup && popup === this.last_opened)
				this.last_opened = this.open_stack.pop();
			if(popup && popup === this.current_modal) {
				this.current_modal = this.modal_stack.pop();
				this.last_opened = this.current_modal;
				if(this.current_modal && ! this.closing)
					this.current_modal.show();
			}
			if(! this.current_modal || this.current_modal.hide_mask)
				this.hideModalMask();
			if(! this.last_opened)
				this.removeEvents();
		},
		close: function(popup) {
			var r, done;
			this.closing = true;
			while(this.last_opened && ! done) {
				if(! popup || this.last_opened === popup || this.last_opened.id === popup) {
					done = true;
					this.closing = false;
				}
				if(! (r = this.last_opened.close()) )
					break;
			}
			this.closing = false;
			return r;
		},
		closeAll: function() {
			var r = true, p = this.last_opened;
			this.closing = true;
			while(p) {
				if(! (r = p.close()) || this.last_opened === p)
					break;
				p = this.last_opened;
			}
			this.closing = false;
			return r;
		},
		checkPosition: function(evt) {
			if(this.events) this.reposition();
		},
		doReposition: function() {
			this.checkMobile();
			if(this.current_modal)
				this.positionMask();
			for(var i = 0; i < this.open_stack.length; i++)
				this.open_stack[i].reposition();
			if(this.last_opened)
				this.last_opened.reposition();
		},
		reposition: function() {
			if(this.reposReq)
				cancelAnimationFrame(this.reposReq);
			this.reposReq = requestAnimationFrame(function() {
				UI.PopupManager.doReposition();
			});
		},
		getModalMask: function() {
			if(! this.mask_elt) {
				var mask = this.mask_elt = createElement2('div', {className: 'pageMask'});
				mask.onselectstart = mask.onmousedown = mask.onclick = mask.ontouchmove = mask.ontouchstart = function(e) {
					var tg = e.target,
						curPop = UI.PopupManager.lastOpened();
					if(tg !== mask || ! curPop || ! curPop.modal) return;
					if(e.type == 'touchstart' || e.type == 'touchmove')
						e.stopPropagation();
					else
						UI.stopEvent(e);
				}
				document.body.appendChild(this.mask_elt);
			}
			return this.mask_elt;
		},
		showModalMask: function(invis) {
			if(this.mask_visible)
				return;
			var mask = this.getModalMask();
			if(mask) {
				if(this.mask_timer)
					clearTimeout(this.mask_timer);
				mask.style.display = 'block';
				UI.addRemoveClass(mask, 'invisible', !!invis);
				if(isIPhone) {
					mask.style.webkitTransitionDuration = 0;
				} else {
					mask.style.opacity = 0.0;
					setTimeout(function() { mask.style.opacity = 0.5; }, 0);
				}
				this.mask_visible = true;
				UI.addClass(document.body, 'modal-open');
				this.positionMask();
			}
		},
		hideModalMask: function() {
			if(! this.mask_visible)
				return;
			var mask = this.getModalMask();
			if(! isIPhone) mask.style.opacity = 0.0;
			this.mask_timer = setTimeout(function() { mask.style.display = 'none'; }, isIPhone ? 0 : 500);
			this.mask_visible = false;
			UI.removeClass(document.body, 'modal-open');
		},
		positionMask: function() {
			if(isIPhone) {
				var m = this.mask_elt;
				var viewp = UI.getViewport();
				m.style.position = 'absolute';
				m.style.left = 0;
				m.style.top = 0;
				m.style.width = viewp.doc_width + 'px';
				m.style.height = viewp.doc_height + 'px';
			}
		},
		setFetchingPopup: function(p) {
			this.fetching_popup = p;
			if(p && ! p.visible && p.modal && ! p.hide_mask) {
				this.showModalMask(p.invis_mask);
			}
		},
		setLoadingPopup: function(p) {
			this.fetching_popup = null;
			this.loading_popup = p;
		},
		getLoadingPopup: function() {
			return this.loading_popup;
		},
		initContent: function(p) {
			if(this.loading_popup)
				this.loading_popup.initContent(p);
		},
		attachEvents: function() {
			if(this.events)
				return;
			var evts = {
				document: {
					mousemove: this.handleMouseMoveDoc,
					mousedown: this.handleMouseDownDoc,
					click: this.handleClickDoc,
					keydown: this.handleKeyDownDoc,
					touchstart: this.handleMouseDownDoc,
					touchend: this.handleClickDoc
				},
				window: {
					mouseleave: this.handleMouseLeaveDoc,
					scroll: this.handleScrollDoc
				}
			};
			this.events = {};
			for(var target in evts) {
				this.events[target] = UI.addEventListeners(window[target], evts[target], this);
			}
		},
		removeEvents: function() {
			if(this.events) {
				for(var target in this.events)
					UI.removeEventListeners(window[target], this.events[target]);
				this.events = null;
			}
		},
		uniqueId: function() {
			return 'popup-window-'+(++this.uniqid);
		},
		eventInside: function(evt, popup) {
			var tg = evt.target;
			if(! tg || ! popup || ! popup.elt) return;
			return popup.elt.contains(tg);
		},
		inRenderedPopup: function(elt) {
			if(elt) {
				for(var i = 0; i < this.open_stack.length; i++) {
					if(this.open_stack[i].elt && this.open_stack[i].elt.contains(elt)) {
						return true;
					}
				}
			}
			return false;
		},
		handleMouseMoveDoc: function(evt) {
			if(this.now_dragging) return;
			if(this.last_opened) this.last_opened.handleMouseMoveDoc(evt);
		},
		handleMouseDownDoc: function(evt) {
			var button = evt.which || evt.button, inside;
			if(evt.type != 'touchstart' && button != 1) return;
			if(evt.type == 'touchstart') {
				if(evt.touches.length != 1) return;
				this.touchClose = true;
			}
			if(this.last_opened) {
				if(! this.eventInside(evt, this.last_opened)) {
					if(this.last_opened.modal && evt.type != 'touchstart') {
						evt.preventDefault();
						return this.last_opened.handleClickDoc(evt);
					}
				}
			}
		},
		handleClickDoc: function(evt) {
			var button = evt.which || evt.button;
			if(evt.type == 'touchend') {
				if(! this.touchClose) return;
			} else if(button != 1) return;
			if(this.now_dragging) return;
			if(! this.eventInside(evt, this.last_opened))
				return this.last_opened.handleClickDoc(evt);
		},
		handleMouseLeaveDoc: function(evt) {
			if(evt.toElement || this.now_dragging) return;
			if(this.last_opened) this.last_opened.handleMouseLeave(evt);
		},
		handleKeyDownDoc: function(evt) {
			if(this.now_dragging) return;
			if(evt.keyCode == 27) this.close();
			else this.last_opened.handleKeyDown(evt);
		},
		handleScrollDoc: function(evt) {
			this.touchClose = false;
		},
		nowDragging: function() {
			return this.now_dragging;
		},
		currentModal: function() {
			return this.current_modal;
		},
		lastOpened: function() {
			return this.last_opened;
		},
		checkMobile: function() {
			var m = false;
			if(isIPhone || window.innerWidth < 700 || window.innerHeight < 500)
				m = true;
			this.mobile = m;
		},
		getMobile: function() {
			return this.mobile;
		}
	};
	UI.addScrollHook(UI.PopupManager.checkPosition, UI.PopupManager);
	UI.addResizeHook(UI.PopupManager.checkPosition, UI.PopupManager);


	// mixin class used to manage element dragging behaviour
	UI.DragHandler = function() {}
	UI.DragHandler.prototype = {
		makeDragTarget: function(target, tag, opts) {
			UI.addEventListener(target, 'selectstart', UI.stopEvent);
			UI.addEventListener(target, 'mousedown',
				function(evt) { this.dragStart(evt, target, tag, opts); }, this);
			UI.addEventListener(target, 'touchstart',
				function(evt) { this.dragStart(evt, target, tag, opts); }, this);
		},
		destroyDragTarget: function(target) {
			// FIXME - remove drag events
			// YEvent.purgeElement(target);
		},
		dragStart: function(evt, target, tag, opts) {
			var button = evt.which || evt.button,
				evts = {
					document: {
						mousemove: this.drag,
						touchmove: this.drag,
						mouseup: this.dragEnd,
						touchend: this.dragEnd,
						keydown: this.dragKeyDown,
						mouseexit: this.dragCancel,
						touchcancel: this.dragCancel,
						selectstart: UI.stopEvent
					},
					window: {
						scroll: this.dragScroll
					}
				},
				etarg = evt.target,
				tname, ename;
			if((evt.type == 'mousedown' && button != 1) || (evt.type == 'touchstart' && evt.touches.length != 1))
				return;
			while(etarg) {
				if(etarg === target || etarg === window) break;
				if(etarg.tagName == 'BUTTON' || UI.hasClass(etarg, 'active-icon')) // ignore events originating in child button elements
					return;
				etarg = etarg.parentNode;
			}
			if(evt.type == 'touchstart') {
				if(this.disable_touch_drag) {
					UI.stopEvent(evt);
					return;
				} else {
					UI.preventDefault(evt); // skip mouse events
				}
			}
			this.draginfo = {
					startTime: performance.now(),
					startX: evt.pageX, startY: evt.pageY,
					scrollStartX: window.pageXOffset,
					scrollStartY: window.pageYOffset,
					dragX: 0, dragY: 0,
					lastDragX: 0, lastDragY: 0,
					dragSpeedX: 0, dragSpeedY: 0,
					scrollX: 0, scrollY: 0,
					tag: tag,
					hooks: []};
			for(tname in evts)
				this.draginfo.hooks[tname] = UI.addEventListeners(window[tname], evts[tname], this);
			UI.PopupManager.now_dragging = true;
			var ret = UI.callEvent(this, this.ondragstart, evt, this.draginfo);
			if(evt.type == 'touchstart' && opts && opts.touchstart)
				opts.touchstart(evt);
			else if(opts && opts.mousedown)
				opts.mousedown(evt);
			UI.stopEvent(evt);
			if(ret === false)
				this.dragCancel();
		},
		drag: function(evt) {
			var di = this.draginfo;
			if(! di)
				return;
			di.dragX = evt.pageX - di.startX;
			di.dragY = evt.pageY - di.startY;
			di.scrolled = false;
			if(evt.type == 'touchmove')
				UI.stopEvent(evt);
			if(di.dragX !== di.lastDragX || di.dragY !== di.lastDragY) {
				UI.callEvent(this, this.ondrag, evt, di);
				di.dragSpeedX = di.dragX - di.lastDragX;
				di.dragSpeedY = di.dragY - di.lastDragY;
			}
			di.lastDragX = di.dragX;
			di.lastDragY = di.dragY;
			return false;
		},
		dragCancel: function(evt) {
			var di = this.draginfo;
			if(! di) return;
			di.canceled = true;
			UI.callEvent(this, this.ondragcancel, evt, di);
			this.dragEnd(evt);
		},
		dragKeyDown: function(evt) {
			if(evt.keyCode == 27) {
				UI.stopEvent(evt);
				this.dragCancel(evt);
			}
		},
		dragScroll: function(evt) {
			var di = this.draginfo, x = di.scrollX, y = di.scrollY;
			di.scrollX = window.pageXOffset - di.scrollStartX;
			di.scrollY = window.pageYOffset - di.scrollStartY;
			di.scrolled = {dx: di.scrollX - x, dy: di.scrollY - y};
			UI.callEvent(this, this.ondrag, evt, di);
		},
		dragEnd: function(evt) {
			var button = evt.which || evt.button,
				di = this.draginfo,
				target;
			if(! di || (evt.type == 'mouseup' && button != 1 && button != 3))
				return;
			if(! di.canceled && performance.now() - di.startTime < 250 && Math.abs(di.dragX) < 5 && Math.abs(di.dragY) < 5) {
				// treat as a click, not a drag
				this.dragCancel(evt);
				return;
			}
			var evts = di.hooks;
			if(evts) {
				for(target in evts)
					UI.removeEventListeners(window[target], evts[target]);
				di.hooks = null;
			}
			UI.PopupManager.now_dragging = false;
			UI.callEvent(this, this.ondragend, evt, di);
			this.draginfo = null;
		}
	};


	UI.MultiSlider = function(id, params) {
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		this.min = 0;
		this.max = 1;
		this.values = [];
		this.points = [];
		this.slider_elts = [];
		this.point_elts = [];
		if(params) extendObject(this, params);
	}
	UI.MultiSlider.prototype = {
		setup: function() {
			if(this.set_up)
				return;
			if(! this.elt && this.id)
				this.elt = $(this.id);
			if(! this.elt)
				return;
			this.updateDisplay();
			this.set_up = true;
		},
		setRange: function(min, max) {
			this.min = min;
			this.max = max;
		},
		addPoints: function() {
			for(var i = 0; i < arguments.length; i++) {
				if(Array.isArray(arguments[i]))
					this.points.pushArray(arguments[i]);
				else
					this.points.push(arguments[i]);
			}
		},
		clearPoints: function() {
			this.points.clear();
		},
		getPoints: function() {
			return this.points.slice();
		},
		addValues: function() {
			for(var i = 0; i < arguments.length; i++) {
				if(Array.isArray(arguments[i]))
					this.values.pushArray(arguments[i]);
				else
					this.values.push(arguments[i]);
			}
			this.values.sort(cmpFn);
		},
		clearValues: function() {
			this.values.clear();
		},
		getValues: function() {
			return this.values.slice();
		},
		render: function() {
			if(this.rendered) return;
			this.elt = MakeElt('div', {'class': 'slider-outer ' + (this.className || '')}).get();
			this.rendered = true;
			return this.elt;
		},
		getWidth: function() {
			var bounds = Wrap(this.elt).bounds(), w;
			if(! bounds || ! (w = bounds.width)) return;
			return w - 1;
		},
		renderPoints: function(targets) {
			var rng = this.max - this.min,
				w = this.getWidth(), point;
			Dom.query(this.point_elts).remove();
			this.point_elts = [];
			for(var i = 0; i < this.points.length; i++) {
				point = MakeElt('div.slider-point');
				point.position({left: w * (this.points[i] - this.min) / rng - 0.5, top: 10});
				this.point_elts.push(point);
			}
			Wrap(this.elt).prepend(this.point_elts);
		},
		updateDisplay: function() {
			var rng = this.max - this.min,
				w = this.getWidth(), slider;
			this.renderPoints();
			Dom.query(this.slider_elts).remove();
			this.slider_elts = [];
			for(var i = 0; i < this.values.length; i++) {
				slider = MakeElt('div.slider-indicator.uii.uii-bdown');
				slider.position({left: w * (this.values[i] - this.min) / rng - 9.5, top: -6});
				this.slider_elts.push(slider);
				this.makeDragTarget(slider.get(), i);
			}
			Wrap(this.elt).append(this.slider_elts);
		},
		getTargetPoints: function(value, index) {
			// override to limit range
		},
		ondragstart: function(evt, info) {
			info.targets = this.getTargetPoints(this.values[info.tag], info.tag);
			if(info.targets) {
				info.targets.sort(cmpFn);
				if(! info.targets.length) return false;
			}
			info.bounds = Wrap(this.elt).bounds();
			if(! info.bounds) return false;
			info.width = info.bounds.width - 1;
			info.range = this.max - this.min;
			info.startOffs = info.width * (this.values[info.tag] - this.min) / info.range - 9.5;
			this.slider_elts[info.tag].addClass('active');
		},
		ondrag: function(evt, info) {
			var w = info.bounds.width - 1,
				xpos = info.startOffs + info.dragX + 9.5,
				nval = xpos / w * info.range + this.min,
				npos,
				mdiff, diff = 0, closest, i;
			if(info.targets) {
				for(i = 0; i < info.targets.length; i ++) {
					diff = Math.abs(info.targets[i] - nval);
					if(! isset(mdiff) || mdiff > diff) {
						mdiff = diff;
						closest = info.targets[i];
					}
				}
				nval = closest;
			}
			if(this.rounded) nval = Math.round(nval);
			info.nval = nval;
			npos = info.width * (nval - this.min) / info.range - 9.5;
			this.slider_elts[info.tag].position({left: npos, top: -6});
		},
		ondragend: function(evt, info) {
			this.slider_elts[info.tag].removeClass('active');
			if(isset(info.nval)) {
				if(info.nval != this.values[info.tag]) {
					this.values[info.tag] = info.nval;
					this.values.sort(cmpFn);
					UI.callEvent(this, this.onchange);
				}
				this.updateDisplay();
			}
		}
	};
	extendProto(UI.MultiSlider, UI.DragHandler);


	UI.Popup = function(id, params) {
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		this.target = null;
		this.target_offset = null;
		this.target_active_class = 'active';
		this.desired_position = null;
		this.fixed_position = null;
		this.modal = false;
		this.inline = false;
		this.destroy_on_close = false;
		this.close_on_exit = true;
		this.close_on_ext_click = true;
		this.close_delay = 500;
		this.keep_in_viewport = true;
		this.visible = false;
		this.hovered = false;
		this.focused = false;
		this.change_opacity = ! isIPhone;
		this.prevent_show_timer = null;
		this.width = null;
		if(params) extendObject(this, params);
		if(! this.id)
			this.id = UI.PopupManager.uniqueId();
	}
	UI.Popup.prototype = {
		setup: function() {
			if(this.set_up)
				return;
			if(! this.elt && this.id)
				this.elt = $(this.id);
			if(! this.elt)
				return;
			this.set_up = true;
		},
		getStartViewport: function() {
			// FIXME - clear if screen is rotated
			if(! this.start_viewport)
				this.start_viewport = UI.getViewport(this.getOuterMargin(), this.inline || this.modal);
			return this.start_viewport;
		},
		getSize: function() {
			if(this.elt) {
				return UI.getEltRegion(this.elt);
			}
		},
		setWidth: function(w) {
			this.width = w;
			if(this.elt)
				UI.setWidth(this.elt, w);
			this.reposition();
		},
		setDesiredPosition: function(pos) {
			this.desired_position = pos;
		},
		setFixedPosition: function(pos) {
			this.fixed_position = pos;
		},
		getOuterMargin: function() {
			var t = this.target_offset;
			return get_default(t ? t.outer_margin : null, true);
		},
		getDesiredPosition: function() {
			if(this.desired_position)
				return Object.assign({}, this.desired_position);
			else if(this.target || this.target_offset) {
				var offs = this.target_offset, over = false, above = false, align = null,
					viewp = UI.getViewport(this.getOuterMargin(), this.inline || this.modal),
					tg = UI.getEltRegion(this.target, this.getOffsetContainer()) || viewp,
					size = this.getSize(),
					pos = {x: viewp.x, y: viewp.y};
				if(! tg || ! size)
					return pos;
				var offs = this.target_offset, over = false, above = false, align = null, evt, out;
				if(offs) {
					over = offs.over;
					above = offs.above;
					align = offs.align;
					if(offs.from_cursor && this.mouse_pos) {
						out = UI.getViewport();
						tg = {x: this.mouse_pos.x + out.x, y: this.mouse_pos.y + out.y - (out.offset||0), width: 1, height: 1};
					}
				} else
					offs = {};
				if(above) {
					pos.y = viewp.window_height - (over ? tg.y + tg.height : tg.y);
					pos.above = true;
				} else {
					pos.y = over ? tg.y : tg.y + tg.height;
				}
				if(! align) {
					align = 'auto';
				}
				if(align == 'onleft') {
					pos.x = tg.x - size.width;
				}
				else if(align == 'left') {
					pos.x = tg.x - size.width + Math.min(size.width, tg.width);
				}
				else if(align == 'onright') {
					pos.x = tg.x + tg.width;
				}
				else if(align == 'right') {
					pos.x = tg.x + tg.width - size.width;
				}
				else if(align == 'center') {
					pos.x = tg.x + (tg.width - size.width) / 2;
				}
				else if(align == 'auto') {
					// align between left and right pos depending on viewport
					if(tg.x + size.width > viewp.right && tg.right - size.width > viewp.x)
						pos.x = tg.right - size.width;
					else
						pos.x = tg.x;
				}
				else {
					pos.x = tg.x;
				}
				if(offs.left)
					pos.x += offs.left;
				if(offs.top)
					pos.y += offs.top;
				return pos;
			}
		},
		getFixedSize: function() {
			if(this.fixedSize) return deep_clone(this.fixedSize);
		},
		getMaxAutoSize: function() {
			var sview = this.getStartViewport(),
				viewp = UI.getViewport(this.getOuterMargin(), this.inline || this.modal),
				perc = this.max_percent_size || {},
				max_w = (this.inline || this.modal) ? viewp.view_width : viewp.width,
				max_h = (this.inline || this.modal) ? viewp.view_height : viewp.height,
				max_size = this.max_auto_size,
				size;
			if(this.inline && this.target) {
				var bounds = QueryFirst(this.target).bounds();
				if(bounds) {
					if(this.target_offset && this.target_offset.above) {
						var tg_y = bounds.y + (this.target_offset.over ? bounds.height : 0);
						if(tg_y > viewp.bottom) tg_y = viewp.bottom;
						if(tg_y < viewp.y) tg_y = viewp.y;
						max_h = Math.min(max_h, tg_y - viewp.y);
						if(max_h < 0) max_h = 0;
					}
				}
			}
			if(viewp.width < 1000) perc = {};
			size = {width: (perc.width || 100) * max_w / 100, height: (perc.height || 100) * max_h / 100};
			if(max_size) {
				if(max_size.width < size.width) size.width = max_size.width;
				if(max_size.height < size.height) size.height = max_size.height;
			}
			return size;
		},
		applyDimensions: function() {
			// set maximum size
			var max_size,
				viewp = UI.getViewport(this.getOuterMargin(), this.inline || this.modal),
				fix = this.getFixedSize(),
				mobile = !!(this.modal && UI.PopupManager.getMobile()),
				set_w = null, set_h = null;
			if(fix) {
				set_w = viewp.width;
				set_h = viewp.height;
			}
			else if( (max_size = this.getMaxAutoSize()) ) {
				if(max_size.width)
					set_w = max_size.width;
				if(max_size.height)
					set_h = max_size.height;
			}
			UI.addRemoveClass(this.elt, 'mobile', mobile);
			UI.addRemoveClass(this.elt, 'resizable', !! (this.resizable && ! mobile));
			if(this.max_width) set_w = this.max_width;
			if(this.max_height) set_h = this.max_height;
			this.elt.style.maxWidth = UI.formatSize(set_w);
			if(this.content_elt && isNumeric(set_h)) {
				var outer = Wrap(this.elt).bounds(),
					content = Wrap(this.content_elt).bounds();
				if(outer && content) set_h -= outer.height - content.height;
				this.content_elt.style.maxHeight = UI.formatSize(set_h);
				this.elt.style.maxHeight = null;
			} else {
				this.elt.style.maxHeight = UI.formatSize(set_h);
			}
			if(this.min_width && ! mobile)
				this.elt.style.minWidth = UI.formatSize(this.min_width);
			else this.elt.style.minWidth = null;
			if(this.min_height && ! mobile)
				this.elt.style.minHeight = UI.formatSize(this.min_height);
			else this.elt.style.minHeight = null;
			return viewp;
		},
		restrictViewport: function(pos, size, viewp) {
			var sview = this.getStartViewport(),
				viewx = this.modal ? sview.x : viewp.x,
				viewy = this.modal ? sview.y : viewp.y,
				vieww = this.modal ? sview.view_width : viewp.width,
				viewh = this.modal ? sview.view_height - viewp.offset : viewp.height;
			if(pos.x + size.width > viewx + vieww)
				pos.x = viewx + vieww - size.width;
			if(viewx > pos.x)
				pos.x = viewx;
			if(pos.above)
				pos.y = viewp.window_height - pos.y - size.height;
			if(pos.y + size.height > viewy + viewh)
				pos.y = viewy + viewh - size.height;
			if(viewy > pos.y)
				pos.y = viewy;
			if(pos.above)
				pos.y = viewp.window_height - pos.y - size.height;
		},
		reposition: function(temp_pos) {
			if(! this.elt || ! this.visible)
				return;
			if(this.target && ! UI.inDocument(this.target)) {
				this.close();
				return;
			}
			var viewp = this.applyDimensions(), pos, size, c, tprop, sy, lp,
				above = this.target_offset && this.target_offset.above;

			if(temp_pos)
				pos = temp_pos;
			else if(this.fixed_position)
				pos = this.fixed_position;
			else {
				pos = this.getDesiredPosition();
				size = this.getSize();
				if(pos) {
					var keep_in = this.keep_in_viewport;
					if(keep_in === 'dialog') {
						// only keep in viewport if target is contained in a dialog
						// (standard behaviour for PopupSlidePanel
						var dlg = QueryFirst(this.target).closest('.popup-dialog');
						keep_in = !! dlg.length;
					}
					if(size && keep_in) {
						this.restrictViewport(pos, size, viewp, above);
						// disable extra repositioning events in mobile browsers, allowing user to zoom popups
						// and accounting for random scrolling events when inputs are focused.
						// still redo positioning if the dialog size changes or the window size changes.
						if(! pos.manual) { // manual for when the user has dragged the dialog
							sy = isIPhone ? window.screenY+viewp.margin : viewp.y;
							if( (lp = this.last_pos)
							&& lp.width == size.width && lp.height == size.height
							&& lp.sx == viewp.x && lp.sy == sy
							&& lp.sw == viewp.width && lp.sh == viewp.height) {
								return;
							}
							this.last_pos = lp = Object.assign({}, pos);
							lp.width = size.width; lp.height = size.height;
							lp.sx = viewp.x; lp.sy = sy;
							lp.sw = viewp.width; lp.sh = viewp.height;
						}
					}
				}
			}
			if(! pos)
				pos = {x: 0, y: 0};
			if(pos) {
				c = this.getContainer(true);
				if(c && c.style.position != 'relative' && c.style.position != 'absolute') {
					var cpos = YDom.getXY(c);
					pos.x -= cpos[0];
					pos.y -= cpos[1];
				}
				var pxpos = {
					x: parseFloat(pos.x).toFixed(2) + 'px',
					y: parseFloat(pos.y).toFixed(2) + 'px'};
				if(this.transform_position && (tprop = UI.getTransformProperty()) && ! pos.above) {
					this.elt.style[tprop] = 'translate3d(' + pxpos.x + ', ' + pxpos.y + ', 0)';
				} else {
					this.elt.style.left = pxpos.x;
					if(pos.above) {
						this.elt.style.top = null;
						this.elt.style.bottom = pxpos.y;
					} else {
						this.elt.style.top = pxpos.y;
					}
				}
			}
			if(! temp_pos)
				UI.callEvent(this, this.onresize, pos, viewp);
		},
		cancelShowTimer: function(shown) {
			if(this.showTimer) {
				clearTimeout(this.showTimer);
				this.showTimer = null;
				if(! shown) {
					UI.callEvent(this, this.oncancelshow);
					UI.PopupManager.canceledShow(this);
				}
				return true;
			}
		},
		cancelCloseTimer: function() {
			if(this.closeTimer) {
				clearTimeout(this.closeTimer);
				this.closeTimer = null;
				return true;
			}
		},
		show: function(delay) {
			this.cancelCloseTimer();
			if(this.prevent_show_timer)
				return;
			if(this.visible || (delay && this.showTimer)) {
				this.reposition();
				return;
			}
			if(delay) {
				var inp = this;
				this.showTimer = setTimeout(function() {
					UI.PopupManager.popupBeforeShow(inp);
					inp.performShow(); }, delay);
				return true;
			} else {
				UI.PopupManager.popupBeforeShow(this);
				return this.performShow();
			}
		},
		performShow: function() {
			this.setup();
			this.cancelShowTimer(true);
			var e = this.elt,
				tprop = UI.getTransformProperty(), tform,
				tsprop = UI.getTransitionProperty();
			if(! e || this.visible) return false;
			Wrap(e).swapIn();
			this.last_ext_move = null; // ignore previous mouse events
			this.visible = e.popup_visible = true;
			/* remove transitions for speed
			if(tsprop && isIPhone) {
				e.style[tsprop] = '';
			}*/
			this.reposition({x: -2000, y: -2000});
			if(this.change_opacity)
				e.style.opacity = 0.0;
			e.style.position = 'absolute';
			e.style.visibility = 'hidden';
			UI.setDisplayState(e, true);
			if(this.zIndex)
				e.style.zIndex = this.zIndex;
			if(tprop && ! this.transform_position && (tform = this.getDisplayTransform())) {
				if(tprop == 'webkitTransform') {
					// fix flashing of element in Chrome on Mac
					e.style.webkitBackfaceVisibility = 'hidden';
				}
				e.style[tprop] = tform;
				if(this.target_offset && this.target_offset.over && this.target_offset.top) {
					e.style[tprop + 'Origin'] = 'center ' + (10 - this.target_offset.top) + 'px';
				}
			}
			this.reposition();
			e.style.visibility = 'visible';
			if(this.change_opacity || tform)
				setTimeout(function() { e.style.opacity = 1.0; if(tform) e.style[tprop] = ''; }, 0);
			if(this.target && this.target_active_class)
				UI.addClass(this.target, this.target_active_class);
			UI.PopupManager.popupShown(this);
			this.addHooks();
			if(e.focus && ! this.disable_auto_focus)
				e.focus();
			UI.callEvent(this, this.onshow);
			return true;
		},
		getDisplayTransform: function() {
			if(isset(this.customDisplayTransform))
				return this.customDisplayTransform;
			return ''; //'scale(0.85)';
		},
		hide: function(reason) {
			var e = this.elt, p = this;
			this.cancelShowTimer();
			this.hovered = false;
			this.last_pos = null;
			if(! e || ! this.visible)
				return;
			UI.PopupManager.popupBeforeHide(this);
			this.visible = e.popup_visible = false;
			if(this.keep_in_document) {
				e.style.display = 'none';
			} else if(this.change_opacity) {
				e.style.opacity = 0.0;
				setTimeout(function() {
					if(! p.visible) {
						Wrap(e).swapOut();
					}
				}, 300);
			} else {
				Wrap(e).swapOut();
			}
			if(this.target && this.target_active_class)
				UI.removeClass(this.target, this.target_active_class);
			UI.PopupManager.popupHidden(this);
			UI.callEvent(this, this.onhide, reason);
		},
		handleMouseEnter: function(evt) {
			this.hovered = true;
			if(this.close_on_exit)
				this.cancelCloseTimer();
			UI.callEvent(this, this.onmouseover, evt);
		},
		handleMouseLeave: function(evt) {
			if(UI.PopupManager.nowDragging()) return;
			var current = UI.PopupManager.lastOpened();;
			var isInline = current && current.inline; // AD: check for stacked?
			if (isInline && current !== this) return;
			this.hovered = false;
			if(this.close_on_exit)
				this.close(this.close_delay, 'mouseleave');
			UI.callEvent(this, this.onmouseout, evt);
		},
		handleMouseUp: function(evt) {
			UI.callEvent(this, this.onmouseup, evt);
		},
		handleMouseMove: function(evt) {
			if(! this.draginfo && ! UI.PopupManager.nowDragging())
				UI.stopPropagation(evt);
		},
		handleMouseDown: function(evt) {
			if(this.close_on_ext_click)
				UI.stopPropagation(evt);
		},
		handleClickDoc: function(evt) {
			if(this.close_on_ext_click || this.close_on_exit) {
				this.close(null, 'extclick');
				if(this.modal)
					return false;
			}
		},
		handleMouseMoveDoc: function(evt) {
			if(this.close_on_exit) {
				var keep_open = false;
				if(this.target) {
					var exy = evt.position();
					keep_open = evt.inside(this.target);
					if(! keep_open) {
						if(! this.last_ext_move || (this.last_ext_move.x == exy.x && this.last_ext_move.y == exy.y)) {
							// ignore random mouse events without an actual move
							keep_open = true;
							this.last_ext_move = exy;
						}
					}
				}
				if(keep_open)
					this.cancelCloseTimer();
				else
					this.close(this.close_delay, 'mouseleave');
			}
		},
		handleFocus: function(evt) {
			if(! this.focused) {
				this.focused = true;
				if(this.close_on_exit)
					this.cancelCloseTimer();
				UI.callEvent(this, this.onfocus, evt);
			}
		},
		handleBlur: function(evt) {
			if(this.focused) {
				this.focused = false;
				if(this.close_on_exit)
					this.close(this.close_delay);
				UI.callEvent(this, this.onblur, evt);
			}
		},
		handleKeyDown: function(evt) {
		},
		addHooks: function() {
			var popup = this,
				evts = {
					mouseenter: this.handleMouseEnter,
					mouseleave: this.handleMouseLeave,
					mousedown: this.handleMouseDown,
					mousemove: this.handleMouseMove,
					touchstart: this.handleMouseDown,
					//touchmove: UI.stopPropagation,
					// only possible if element has an assigned tabIndex (SelectInput popup)
					focus: this.handleFocus,
					blur: this.handleBlur
				};
			for(var evt in evts)
				UI.addEventListener(this.elt, evt, evts[evt], this);

			// reposition popup when elements around the target are scrolled
			// FIXME - ignore repeated scroll events (check scrollLeft, scrollTop)
			// and remove events after popup is hidden
			var target = Wrap(this.target), parents, pelt, ov, i, attach;
			if(target) {
				parents = target.parents();
				for(i = 0; i < parents.length; i++) {
					if(parents[i] === document.body) break;
					pelt = parents.ext(i);
					if( (ov = pelt.css('overflow')) === 'auto' || ov === 'scroll'
					||  (ov = pelt.css('overflow-x')) === 'auto' || ov === 'scroll'
					||  (ov = pelt.css('overflow-y')) === 'auto' || ov === 'scroll') {
						pelt.on('scroll', function() { UI.PopupManager.reposition() });
					}
				}
			}
		},
		getContainer: function(if_custom) {
			var c;
			if(this.container) return this.container;
			if(this.container_id && (c = $(this.container_id))) return c;
			return if_custom ? null : document.body;
		},
		getOffsetContainer: function(if_custom) {
			var contain = Wrap(this.getContainer(if_custom));
			if(contain.css('position') === 'static') {
				contain = contain.offsetParent();
			}
			return contain.get();
		},
		getContentElement: function() {
			if(! this.content_elt && this.content_id)
				this.content_elt = $(this.content_id);
			return this.content_elt;
		},
		render: function() {
			this.setup();
			if(this.rendered) return;
			this.rendered = true;
			if(! this.elt) {
				this.elt = createElement2('div', {id: this.id, className: this.className, style: 'display: none'});
				if(this.width)
					UI.setWidth(this.elt, this.width);
				if(isset(this.tabIndex))
					this.elt.tabIndex = this.tabIndex;
				this.renderContent();
				this.getContainer().appendChild(this.elt);
			}
			return this.elt;
		},
		renderContent: function() {
			var content = this.getContentElement();
			if(content) {
				if(this.fill_content) UI.addClass(content, 'fill');
				this.elt.appendChild(content);
				toggleDisplay(content, true);
			}
		},
		canClose: function() { return true; },
		close: function(delay, reason) {
			this.cancelShowTimer();
			if(! this.visible || ! this.canClose() || (delay && this.closeTimer))
				return;
			if(delay) {
				var inp = this;
				this.closeTimer = setTimeout(function() { inp.performClose(null, reason); }, delay);
			} else
				this.performClose(null, reason);
			return true;
		},
		performClose: function(delay, reason) {
			this.cancelCloseTimer();
			var self = this;
			this.hide(reason);
			if(this.destroy_on_close) {
				setTimeout(function() { if(! self.visible) self.destroy(); }, 500);
			} else {
				// used mainly to prevent a re-show when clicking the target to hide the popup
				this.prevent_show_timer = setTimeout(function() { self.prevent_show_timer = null; }, 200);
			}
			UI.callEvent(this, this.onclose, reason);
		},
		destroy: function() {
			UI.removeNode(this.elt, true);
			this.elt = null;
			this.rendered = false;
			delete this.start_viewport;
			UI.callEvent(this, this.ondestroy);
			this.destroyed = true;
		},
		initContent: function(params) {}
	};


	UI.PopupSlidePanel = function(id, params) {
		var defaults = {
			close_on_exit: false,
			className: 'card-outer popup-default panel-outer panel-border panel-default active',
			target_active_class: 'active opened',
			customDisplayTransform: isIPhone ? null : 'scaleY(0.4)',
			disable_auto_focus: true,
			inline: true,
			keep_in_viewport: 'dialog',
			hide_mask: true,
			destroy_on_close: true
		};
		if(params) {
			extendObject(defaults, params);
			if(defaults.ext_class) defaults.className += ' ' + defaults.ext_class;
		}
		UI.Popup.call(this, id, defaults);
		if(this.show_above) {
			this.className += ' panel-round-top';
			if(this.target_active_class)
				this.target_active_class += ' opened-above';
		} else {
			this.className += ' panel-round-bottom';
		}
	}
	extendClass(UI.PopupSlidePanel, UI.Popup);


	UI.Tooltip = function(id, params) {
		this.text = '';
		this.content = null;
		this.show_delay = 500;
		if(! params) params = {};
		if(! params.className) params.className = 'popup-tooltip';
		if(! params.target_offset) params.target_offset = {top: 10};
		UI.Popup.call(this, id, params);
		this.stacked = true;
		if(! this.target)
			this.target = $(this.target_id);
		if(this.target) {
			var popup = this;
			if(! this.pre_triggered)
				this.target.onmouseenter = function() { popup.showTooltip(); }
			this.target.onmouseout = function() { popup.close(); }
		}
		if(this.pre_triggered) {
			this.render();
			this.show(this.show_delay);
		}
	}
	extendClass(UI.Tooltip, UI.Popup, {
		showTooltip: function() {
			this.render();
			this.show(this.show_delay);
		},
		renderContent: function() {
			this.content_elt = createElement2('span', {className: 'tooltip-text'});
			if(this.content)
				this.content_elt.innerHTML = this.content;
			else
				UI.setElementText(this.content_elt, this.text);
			this.elt.appendChild(this.content_elt);
		}
	});


	UI.Dialog = function(id, params) {
		UI.Popup.call(this, id, {className: 'card-outer dialog-outer popup-dialog'});
		this.resizable = true;
		this.draggable = true;
		this.have_title_bar = true;
		this.have_close_button = true;
		this.align_center = true;
		this.transform_position = false;
		this.max_percent_size = {width: 80, height: 90};
		this.max_auto_size = {width: 1000, height: 700};
		//this.change_opacity = false;
		this.destroy_on_close = true;
		this.close_on_exit = false;
		this.close_on_ext_click = false;
		this.disable_touch_drag = true;
		this.modal = true;
		this.title = null;
		if(params) extendObject(this, params);
	}
	extendClass(UI.Dialog, UI.Popup, {
		setTitle: function(title) {
			this.setTitleHtml(title);
		},
		setTitleHtml: function(title) {
			if(isset(title))
				this.title = title;
			this.renderTitle();
		},
		renderTitle: function() {
			if(this.title_text_elt) {
				if(isNode(this.title)) {
					this.title_text_elt.innerHTML = '';
					this.title_text_elt.appendChild(this.title);
				} else if(isset(this.title))
					this.title_text_elt.innerHTML = this.title;
				else
					UI.setElementText(this.title_text_elt, this.title_text || '\u00a0');
			}
		},
		setTitleText: function(title) {
			if(isset(title)) {
				this.title_text = title;
				this.title = null;
			}
			this.renderTitle();
		},
		getDesiredPosition: function() {
			if(this.desired_position)
				return Object.assign({}, this.desired_position);
			if(! this.align_center)
				return UI.Dialog.superclass.getDesiredPosition.call(this);
			var size = this.getSize(),
				view = UI.getViewport(this.getOuterMargin(), this.inline || this.modal),
				sview, ret;
			if(size && view) {
				ret = {
					x: view.x + Math.max(0, (view.body_width - size.width) / 2),
					y: view.y + Math.max(0, (view.height - size.height) / 2) };
				if(this.modal) {
					sview = this.getStartViewport();
					ret = {
						x: sview.margin + (view.body_width - size.width) / 2,
						y: sview.y + (sview.view_height - sview.offset - size.height) / 2 };
				}
				return ret;
			}
		},
		renderContent: function() {
			var title = this.getTitleBar(),
				head = this.renderHeader(),
				foot = this.renderFooter(),
				bottom = this.getBottomBar();
			UI.clearChildNodes(this.elt);
			if(title)
				this.elt.appendChild(title);
			if(head)
				this.elt.appendChild(head);
			UI.Dialog.superclass.renderContent.call(this);
			if(foot)
				this.elt.appendChild(foot);
			if(bottom)
				this.elt.appendChild(bottom);
			this.setTitleHtml();
		},
		renderHeader: function() {
			return this.header_elt;
		},
		renderFooter: function() {
			return this.footer_elt;
		},
		destroy: function() {
			if(this.have_title_bar) {
				var e = this.getTitleBar();
				this.destroyDragTarget(e);
			}
			var e = this.getResizeCorner();
			this.destroyDragTarget(e);
			UI.Dialog.superclass.destroy.call(this);
		},
		fetchContent: function(url, params, callback, autoShow) {
			var conn;
			if(url instanceof App.conn.HTTPRequest)
				conn = url;
			else
				conn = new App.conn.HTTPRequest(url, params);
			UI.PopupManager.setFetchingPopup(this);
			var self = this;
			conn.fetch(function() {
				self.loadContent(this, callback, autoShow);
			}, function() {
				self.loadError(this, callback, null, autoShow);
			});
		},
		fetchFormContent: function(form, form_values, conn_params, callback, autoShow) {
			var req = new App.conn.HTTPRequest(null, conn_params);
			if(req.loadForm(form, form_values))
				this.fetchContent(req, null, callback, autoShow);
		},
		loadContent: function(data, callback, autoShow) {
			var self = this;
			this.render();
			if(data.getResult)
				data = data.getResult();
			if(isset(data.responseText))
				data = data.responseText;
			var text = App.util.disableInlineScripts(data);
			App.session.setLoadingRequest(data);
			UI.PopupManager.setLoadingPopup(this);
			this.setContent(text, true);
			function done() {
				if(callback) callback();
				UI.PopupManager.setLoadingPopup(null);
				App.session.setLoadingRequest(null);
				if(autoShow) self.show();
				else self.reposition();
			}
			if(! this.disable_scripts)
				App.util.evalScript(data, done);
			else
				done();
		},
		initContent: function(params) {
			var content = this.getContentElement();
			if(! params) params = {};
			if(! this.ignore_init_title) {
				if(params.title)
					this.setTitle(params.title);
				else if(params.title_text)
					this.setTitleText(params.title_text);
			}
			if(params.width) this.setWidth(params.width);
			if(params.list_id) this.list_id = params.list_id;
			// register forms here (params.forms?)
			if(params.form_name) {
				var f = document.forms[params.form_name];
				if(f) UI.initForm(f);
			}
			if(params.onshow) {
				UI.attachInputEvent(this, 'onshow', params.onshow);
			}
			if(params.onclose) {
				UI.attachInputEvent(this, 'onclose', params.onclose);
			}
			if(params.content_id)
				content.id = params.content_id;
			UI.addRemoveClass(content, 'fill', !! params.fill);
			UI.callEvent(this, this.oninitcontent);
		},
		setContent: function(content, no_repos) {
			var elt = this.getContentElement();
			if(typeof(content) === 'string') {
				elt.innerHTML = content;
			} else {
				UI.setElementContent(elt, content);
			}
			if(! no_repos)
				this.reposition();
		},
		setFooter: function(text, no_repos) {
			this.getFooterElement().innerHTML = text;
			if(! no_repos)
				this.reposition();
		},
		loadError: function(conn, callback, err_str, autoShow) {
			if(! err_str) err_str = app_string('LBL_ASYNC_JS_ERROR');
			this.loadContent('<p align="center"><small><i>'+err_str+'</i></small></p>', callback, autoShow);
		},
		getTitleBar: function() {
			var t;
			if(this.have_title_bar) {
				if(! this.title_elt) {
					this.title_text_elt = createElement2('h4');
					var t = createElement2('div', {className: 'dialog-title'}, this.title_text_elt);
					this.title_elt = createElement2('div', {className: 'card-header dialog-header tools-row align-top'}, t);
					if(this.have_close_button)
						this.title_elt.appendChild(this.getCloseButton());
					if(this.draggable) {
						this.makeDragTarget(this.title_elt, 'title');
					}
				}
				return this.title_elt;
			}
		},
		getBottomBar: function() {
			if(! this.bottom_elt) {
				this.status_elt = createElement2('div', {className: 'dialog-status text-muted'}, null, this.bottom_elt);
				var e = this.getResizeCorner();
				this.bottom_elt = createElement2('div', {className: 'card-footer dialog-footer tools-row align-bottom'}, [this.status_elt, e]);
				this.makeDragTarget(e, 'resize');
			}
			if(this.resizable || this.have_bottom_bar)
				return this.bottom_elt;
		},
		getContentElement: function() {
			var c = UI.Dialog.superclass.getContentElement.call(this);
			if(! c)
				c = this.content_elt = createElement2('div', {id: this.content_id, className: 'card-body dialog-body'});
			return c;
		},
		getFixedSize: function() {
			if(this.fixedSize) return deep_clone(this.fixedSize);
			//if(UI.PopupManager.getMobile()) return {width: window.innerWidth - 40, height: window.innerHeight - 40};
		},
		getContentSize: function() {
			var sz = this.getFixedSize(), chk;
			if(! sz) {
				chk = UI.getEltRegion(this.elt);
				if(chk)
					sz = {width: chk.width, height: chk.height};
				else
					sz = this.getSize();
			}
			sz.width -= 4;
			sz.height -= 42;
			return sz;
		},
		getFooterElement: function() {
			if(! this.footer_elt) {
				var e;
				if(this.footer_id && (e = $(this.footer_id)))
					this.footer_elt = e;
				else
					this.footer_elt = createElement2('div', {id: this.content_id, className: 'dialog-footer'});
			}
			return this.footer_elt;
		},
		getCloseButton: function() {
			if(! this.close_btn) {
				this.close_btn = createElement2('div', {className: 'uii uii-cancel uii-lg active-icon dialog-close'});
			}
			var self = this;
			//this.close_btn.onmousedown = function(evt) { UI.stopEvent(evt || window.event); }
			this.close_btn.onclick = function() { self.close(); }
			return this.close_btn;
		},
		getResizeCorner: function() {
			if(! this.resize_elt) {
				this.resize_elt = createElement2('div', {className: 'dialog-resizer no-flex'});
			}
			return this.resize_elt;
		},
		getDisplayTransform: function() {
			return ''; //'scale(0.95)';
		},
		setFixedSize: function(size) {
			if(this.elt) {
				if(! this.fixedSize) {
					if(! size)
						size = this.getSize();
					UI.addClass(this.elt, 'fixed-size');
				}
				if(size) {
					if(size.width < this.min_width) size.width = this.min_width;
					if(size.height < this.min_height) size.height = this.min_height;
					// size adjustment is for element border
					this.elt.style.width = '' + (size.width - 4) + 'px';
					this.elt.style.height = '' + (size.height - 4) + 'px';
				}
			}
			this.fixedSize = size;
			UI.callEvent(this, this.onresize);
		},
		ondragstart: function(evt, info) {
			this.close_on_exit = false;
			info.pos = this.getDesiredPosition();
			info.size = this.getSize();
			if(! info.pos)
				info.pos = info.size;
		},
		ondrag: function(evt, info) {
			if(info.tag == 'resize') {
				var viewp = UI.getViewport(this.getOuterMargin(), this.modal),
					w = info.size.width,
					h = info.size.height;
				if(this.align_center && ! this.desired_position)
					factor = 2.0;
				else
					factor = 1.0;
				w += info.dragX * factor;
				h += info.dragY * factor;
				this.setFixedSize({
					width: Math.min(viewp.width, Math.max(250, w)),
					height: Math.min(viewp.height, Math.max(76, h))
				});
			} else {
				this.setDesiredPosition({x: info.pos.x + info.dragX, y: info.pos.y + info.dragY, manual: 1});
			}
			this.reposition();
		},
		ondragend: function(evt, info) {
		}
	});
	extendProto(UI.Dialog, UI.DragHandler);


	UI.MiniDialog = function(id, params) {
		UI.Dialog.call(this, id, {className: 'card-outer dialog-outer popup-dialog compact'});
		this.resizable = false;
		this.align_center = false;
		this.close_on_ext_click = true;
		this.close_on_exit = true;
		this.modal = false;
		if(params && params.modal) {
			this.close_on_exit = false;
			this.invis_mask = true;
		}
		if(params) extendObject(this, params);
	}
	extendClass(UI.MiniDialog, UI.Dialog, {});


	UI.PopupMessage = function(id, params) {
		var defaults = {
			modal: true, align_center: true, close_on_exit: false, close_on_ext_click: false,
			destroy_on_close: true, className: 'card-outer dialog-outer popup-default compact popup-message'
		};
		if(params)
			extendObject(defaults, params);
		UI.MiniDialog.call(this, id, defaults);
	}
	extendClass(UI.PopupMessage, UI.MiniDialog, {
		renderFooter: function() {
			if(this.show_popup_close) {
				var input = this, foot = this.getFooterElement();
				this.popup_close_button = new UI.ButtonInput(this.id + '-clsbtn',
						{icon: 'icon-cancel', label: this.close_button_label || app_string('LBL_ADDITIONAL_DETAILS_CLOSE'),
						 onclick: function(evt) { input.close(); }, autofocus: true});
				foot.appendChild(createElement2('div', {style: 'text-align: right'}, this.popup_close_button.render()));
			}
			return UI.PopupMessage.superclass.renderFooter.call(this);
		},
		performShow: function() {
			var ret = UI.PopupMessage.superclass.performShow.call(this), self = this;
			if(this.timeout)
				setTimeout(function() { self.close(); }, this.timeout);
			var fst = SUGAR.dom.ExtNode(this.elt).findFirst('.autofocus').get(0);
			if(fst && fst.focus) fst.focus();
			return ret;
		}
	});


	UI.Status = {
		uniqid: 0,
		messages: {},
		order: [],
		timeouts: {},
		visible: false,
		adjust_pos: (isIPhone),
		show: function(status, timeout) {
			if(! isset(status))
				return;
			if(! (status instanceof UI.StatusMessage))
				status = new UI.StatusMessage(status);
			if(! status.id || ! this.messages[status.id]) {
				if(! status.id)
					status.id = 'status-msg-'+(++this.uniqid);
				this.messages[status.id] = status;
				this.order.push(status.id);
			}
			if(timeout)
				this.updateTimeout(status, timeout);
			this.updateDisplay();
			return status;
		},
		remove: function(status) {
			if(! status) return;
			if(typeof(status) == 'string' && this.messages[status])
				delete this.messages[status];
			else if(status.id && this.messages[status.id])
				delete this.messages[status.id];
			this.updateDisplay();
		},
		getStatusDiv: function() {
			if(! this.status_div) {
				this.status_div = createElement2('div', {className: 'statusDiv', id: 'ajaxStatusDiv'});
				document.body.appendChild(this.status_div);
			}
			return this.status_div;
		},
		updateTimeout: function(status, timeout) {
			if(! status || ! status.id || ! this.messages[status.id])
				return;
			if(this.timeouts[status.id]) {
				clearTimeout(this.timeouts[status.id]);
				delete this.timeouts[status.id];
			}
			if(timeout)
				this.timeouts[status.id] = setTimeout(function() { UI.Status.remove(status.id); }, timeout);
		},
		cleanup: function() {
			var ord = [];
			for(var i = 0; i < this.order.length; i++)
				if(this.messages[this.order[i]])
					ord.push(this.order[i]);
			this.order = ord;
		},
		updateDisplay: function() {
			var div = this.getStatusDiv();
			if(! div)
				return;
			if(top) div.style.top = '' + top + 'px';
			var content = [], seen = {};
			for(var i = 0; i < this.order.length; i++) {
				var id = this.order[i];
				var status = this.messages[id];
				if(! status)
					continue;
				m = status.getMessage();
				if(seen[m]) continue;
				content.push(m);
				seen[m] = 1;
			}
			if(content.length) {
				div.innerHTML = html_escape(content.join('\n')).replace(/\n/g, '<br>');
				toggleDisplay(div, true);
				this.visible = true;
				this.reposition();
			} else {
				toggleDisplay(div, false);
				UI.clearChildNodes(div);
				this.visible = false;
			}
			this.cleanup();
		},
		reposition: function() {
			var div = this.getStatusDiv();
			if(! div || div.style.display == 'none')
				return;
			var size = UI.getEltRegion(div),
				viewp = UI.getViewport(8);
			if(size && viewp) {
				if(isIPhone) {
					div.style.position = 'absolute';
					div.style.top = viewp.y + 'px';
					div.style.left = ((viewp.width - size.width) / 2) + viewp.x + 'px';
				} else {
					div.style.left = ((viewp.width - size.width) / 2) + 'px';
					div.style.top = '' + (viewp.offset||33) + 'px';
				}
			}
		},
		checkPosition: function() {
			if(this.visible && this.adjust_pos) this.reposition();
		}
	}
	UI.addScrollHook(UI.Status.checkPosition, UI.Status);
	UI.addResizeHook(UI.Status.checkPosition, UI.Status);


	UI.StatusMessage = function(message, params) {
		this.message = message;
		var timeout;
		if(params)
			for(var k in params) {
				if(k == 'timeout') timeout = params[k];
				else this[k] = params[k];
			}
	};
	UI.StatusMessage.prototype = {
		show: function(timeout) {
			UI.Status.show(this, timeout);
		},
		setMessage: function(msg) {
			this.message = msg;
			UI.callEvent(this, this.onchange);
		},
		getMessage: function() {
			return this.message;
		},
		updateTimeout: function(timeout) {
			UI.Status.updateTimeout(this, timeout);
		},
		remove: function() {
			UI.Status.remove(this);
		},
		hide: function() { // alias for remove
			UI.Status.remove(this);
		},
		onchange: function() {
			UI.Status.updateDisplay();
		}
	};


	// -- UI widgets

	UI.SelectOptions = function(params) {
		this.init(params);
	}
	UI.SelectOptions.prototype = {
		clear: function() {
			this.keys = [];
			this.values = [];
		},
		init: function(params) {
			this.display_name = 'label';
			this.add_blank = false;
			this.dom_name = null;
			this.dom_module = null;
			this.language = null;
			this.limit_keys = null;
			this.keys = null;
			this.values = null;
			this.width = null;
			if(params)
				extendObject(this, params);
			var keys = [], vals = [];
			if(this.dom_name) {
				var opts = this.loadOptionsDom(this.dom_name, this.dom_module);
				if(opts) {
					keys = opts.keys;
					vals = opts.values;
				}
				if(! params || ! params.display_name) this.display_name = '_display';
			}
			if(this.limit_keys) {
				if(this.values) vals = this.values;
				var ks = [], i, v, vs = [];
				for(i = 0; i < keys.length; i++) {
					if(this.limit_keys.indexOf(keys[i]) >= 0) {
						ks.push(keys[i]);
						if(Array.isArray(vals))
							vs.push(vals[i]);
						else
							vs.push(vals[keys[i]]);
					}
				}
				keys = ks;
				vals = vs;
			}
			if(! this.keys) this.keys = deep_clone(keys);
			if(! this.values) this.values = deep_clone(vals);
			if(this.add_blank && this.keys.indexOf('') < 0) {
				this.keys.splice(0, 0, '');
				var blank_label = isset(this.blank_label) ? this.blank_label : app_string('LBL_NONE');
				if(Array.isArray(this.values))
					this.values.splice(0, 0, blank_label);
				else
					this.values[''] = blank_label;
			}
		},
		addOption: function(key, value, index) {
			var k = this.getKeyIndex(key);
			if(k < 0) {
				if(isset(index) && index >= 0 && index < this.keys.length) {
					this.keys.splice(index, 0, key);
				} else {
					index = this.keys.length;
					this.keys.push(key);
				}
			} else
				index = k;
			if(Array.isArray(this.values))
				this.values[index] = value;
			else
				this.values[key] = value;
			return true;
		},
		removeOption: function(key) {
			var k = this.getKeyIndex(key);
			if(k >= 0) {
				this.keys.splice(k, 1);
				if(Array.isArray(this.values))
					this.values.splice(k, 1);
				else
					delete this.values[key];
				return true;
			}
		},
		replaceOption: function(key, value) {
			if(this.getKeyIndex(key) >= 0) {
				this.addOption(key, value);
				return true;
			}
		},
		getCount: function() {
			return this.keys && this.keys.length || 0;
		},
		getKeys: function() {
			return this.keys;
		},
		getValues: function(keys) {
			if(Array.isArray(this.values) && (! isset(keys) || keys == this.keys)) {
				return this.values;
			}
			if(! isset(keys))
				keys = this.getKeys();
			var vals = [];
			if(keys) {
				for(var i = 0; i < keys.length; i++)
					vals.push(this.getValue(keys[i]));
			}
			return vals;
		},
		getKeyIndex: function(key) {
			var k = this.getKeys();
			if(k)
				return k.indexOf(key);
			return -1;
		},
		getKey: function(index) {
			var k = this.getKeys();
			if(k) return k[index];
			return null;
		},
		getValue: function(key) {
			if(Array.isArray(this.values)) {
				var kpos = this.getKeyIndex(key);
				if(kpos >= 0)
					return this.values[kpos];
			} else if(this.values)
				return this.values[key];
		},
		getLabel: function(key, display_name) {
			if(! display_name) display_name = this.display_name;
			var value = this.getValue(key);
			if(value !== null && typeof(value) === 'object')
				value = value[display_name];
			return isset(value) ? ''+value : '';
		},
		loadOptionsDom: function(dom, module, lang) {
			return UI.getSelectOptions(dom, module, lang || this.language);
		}
	};

	UI.Draggable = function(elt, params) {
		this.elt = Wrap(elt);
		if(params) extendObject(this, params);
		if(! this.elt) throw new TypeError('missing draggable element');
		this.handle = Wrap(this.handle);
		this.setup();
	}
	UI.Draggable.prototype = {
		setup: function() {
			this.elt.prop('draggable', !this.handle);
			if(this.move_cursor)
				(this.handle || this.elt).css('cursor', 'move');
			this.addHooks();
		},
		addHooks: function() {
			this.evts = this.elt.on({
				dragstart: this.handleDragStart,
				dragend: this.handleDragEnd,
				dragstate: this.handleDragState,
				mouseover: this.handleMouseOver,
				mouseout: this.handleMouseOut
			}, {scope: this});
		},
		getDragTransfer: function() {
			return this.transfer;
		},
		getDragValues: function() {
			return this.values;
		},
		handleMouseOver: function(evt) {
			if (this.handle && document.contains(this.handle.get())) {
				this.elt.prop('draggable', evt.inside(this.handle));
			}
		},
		handleMouseOut: function(evt) {
			if (this.handle)
				this.elt.prop('draggable', false);
		},
		handleDragStart: function(evt) {
			if ((this.handle && document.contains(this.handle.get())) && ! evt.inside(this.handle)) {
				evt.preventDefault();
				return;
			}
			evt.stopPropagation();
			UI.dragSource = this;
			// without this firefox will not start the drag operation
			// we use non-common content type to prevent insertion of unwanted
			// data when dragged onto a text input
			evt.originalEvent.dataTransfer.setData('onecrm/draghack', 'anything');
			(this.source || UI).setDragData(evt, this.getDragTransfer(), this.getDragValues());
			UI.callEvent(this, this.onStart, evt);
		},
		handleDragEnd: function(evt) {
			if (!UI.dragSource || UI.dragSource != this) {
				return;
			}
			UI.dragSource = null;
			var data = UI.getDragData(evt);
			UI.callEvent(this, this.onEnd, evt, data);
			if(this.source) this.source.afterDragContent(evt, this, data);
		},
		handleDragState: function(evt, state) {
			if(this.source) this.source.dragStateChanged(evt, this, state);
			UI.callEvent(this, this.onStateChanged, evt, state);
		},
		destroy: function() {
			if(this.evts) this.elt.off(this.evts);
			this.evts = null;
		}
	};

	UI.DragTarget = function(elt, params) {
		this.elt = Wrap(elt);
		this.target_events = 'dragenter, dragover, dragleave, drop';
		if(params) extendObject(this, params);
		if(! this.elt) throw new TypeError('missing drag target');
		this.setup();
	}
	UI.DragTarget.prototype = {
		setup: function() {
			this.addHooks();
		},
		addHooks: function() {
			this.elt.on(this.target_events, this.handleDragEvent, {scope: this});
		},
		beforeDragContent: function(evt, content) {
			// called when a child node is about to be dragged
		},
		afterDragContent: function(evt, content, data) {
			// called after a child node has been dragged
		},
		dragStateChanged: function(evt, content, state) {
			// called when the drag state on a child changes
		},
		checkAccept: function(evt, data) {
			// check if a dragged element can be dropped here
		},
		getDragPosition: function(evt, data) {
			if(evt) {
				var pos = evt.position();
				if(pos) return pos;
			}
			if(! data) data = UI.getDragData(evt);
			return data.position;
		},
		getDropEffect: function(evt) {
			return this.accept_drop_effect || this.drop_effect || 'copy';
		},
		getDragData: function(evt) {
			return UI.getDragData(evt);
		},
		setDragData: function(evt, transfer, values) {
			if(! values) values = {};
			values.source = this;
			if(this.source_input) values.source_input = this.source_input;
			UI.setDragData(evt, transfer, values, this.elt);
		},
		acceptDragOver: function(evt, data) {
			var accept = this.checkAccept(evt, data) || false;
			if(typeof(accept) === 'string')
				this.accept_drop_effect = accept;
			else
				this.accept_drop_effect = null;
			return !! accept;
		},
		cancelDragOver: function(evt, data) {
			data.over = null;
			data.over_count = 0;
			data.position = this.getDragPosition(evt);
			UI.setDragState(evt, null);
			UI.callEvent(this, this.onLeave, evt, data);
		},
		setOver: function(evt, data) {
			data.over = this;
			data.over_count = 1;
			data.position = this.getDragPosition(evt);
			UI.callEvent(this, this.onEnter, evt, data);
		},
		handleDragEvent: function(evt) {
			var pos, data = this.getDragData(evt);
			var source = UI.dragSource;
			var accepted = null;
			if (this.onAcceptSource) {
				accepted = this.onAcceptSource(evt, data, source);
				if (accepted === 'bubble') {
					return;

				}
			}
			if (accepted === null && this.acceptedSourceTypes) {
				accepted = !!source && ~this.acceptedSourceTypes.indexOf(source.sourceType);
			}
			evt.preventDefault();
			if (accepted !== null && !accepted)
				return;
			evt.stopPropagation();
			if(evt.type === 'dragover') {
				if(! data.over) {
					this.setOver(evt, data);
				}
				if(data.over && (pos = this.getDragPosition(evt, data)) ) {
					var lastPos = data.position,
						lastAccept = data.state === 'accept',
						accept = lastAccept;
					if(! lastPos || lastPos.x != pos.x || lastPos.y != pos.y) {
						data.position = pos;
						accept = this.acceptDragOver(evt, data);
						if(accept != lastAccept)
							UI.setDragState(evt, accept ? (this.drag_ignore ? null : 'accept') : 'reject');
						if(accept && ! lastAccept)
							UI.callEvent(this, this.onAccept, evt, data);
						else if(! accept && lastAccept)
							UI.callEvent(this, this.onReject, evt, data);
						UI.callEvent(this, this.onMove, evt, data);
					}
					if(accept) {
						evt.preventDefault(); // allows drag enter
						evt.dataTransfer.dropEffect = this.getDropEffect(evt);
					}
				}
			}
			else if(evt.type === 'dragenter') {
				if(data.over && ! data.over.elt.equals(this.elt)) {
					data.over.cancelDragOver(evt, data);
				}
				if(! data.over) {
					this.setOver(evt, data);
				} else {
					data.over_count ++;
				}
			}
			else if(evt.type === 'dragleave') {
				if(data.over && data.over.elt.equals(this.elt) && --data.over_count <= 0) {
					this.cancelDragOver(evt, data);
				}
			}
			else if(evt.type === 'drop') {
				evt.preventDefault();
				evt.stopPropagation();
				data.over = null;
				data.position = this.getDragPosition(evt);
				if(data.state === 'accept') {
					UI.setDragState(evt, 'complete'); // freeze state (so .self is accurate)
					UI.callEvent(this, this.onDrop, evt, data);
				}
				UI.callEvent(this, this.onLeave, evt, data);
			}
		},
		destroy: function() {
			this.elt.off(this.target_events);
		}
	};



	UI.SelectList = function(id, params) {
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		this.options = ''; // must be defined in params
		this.last_selected_index = null;
		this.selected_indexes = [];
		this.disabled_indexes = [];
		this.hidden_indexes = [];
		this.current_index = null;
		this.show_checks = false;
		this.icon_key = null;
		this.popup = false;
		this.flat = false;
		this.multi_select = false;
		this.onchange = null;
		this.oncurrent = null;
		this.keystate = {ctrl: 0, shift: 0};
		this.focused = false;
		this.auto_show_selected = true;
		this.require_focus = false;
		this.add_remove_sorted = true;
		this.options_pad = 2;
		this.options_wrap = false;
		this.mark_blank = true;
		this.disable_current = false;
		if(params) extendObject(this, params);
		if(this.columns) this.setColumns(this.columns);
		if(! this.option_height)
			this.option_height = this.popup ? 22 : 20;
		this.initOptions();
	}
	UI.SelectList.option_events = {
		click: function(evt) { evt.data.scroll.optionClick(this, evt.data, evt); },
		mouseenter: function(evt) { evt.data.scroll.optionEvent(this, evt.data, 'over', evt); },
		mouseleave: function(evt) { evt.data.scroll.optionEvent(this, evt.data, 'out', evt); },
		mousedown: function(evt) { evt.data.scroll.optionEvent(this, evt.data, 'down', evt); UI.stopPropagation(evt); },
		mouseup: function(evt) { evt.data.scroll.optionEvent(this, evt.data, 'up', evt); },
		touchstart: function(evt) { evt.data.scroll.optionEvent(this, evt.data, 'over', evt); },
		touchend: function(evt) { evt.data.scroll.optionClick(this, evt.data, evt); }
		//touchmove: function(evt) { evt.data.scroll.optionEvent(this, evt.data, 'move', evt); }
	};
	UI.SelectList.prototype = {
		setup: function() {
			if(this.set_up)
				return;
			var input = this;
			if(! this.popup && ! this.defer_render)
				this.render();
			if(! this.elt)
				return;

			if(this.mode_name && ! this.mode_field) {
				this.mode_field = UI.getFormInput(this.form, this.mode_name);
				var focus = function(evt) { input.handleFocus(evt || window.event); },
					blur = function(evt) { input.handleBlur(evt || window.event); };
				this.mode_field.onfocus = focus;
				this.mode_field.onblur = blur;
				this.mode_field.onchange = function(k, v, upd) { input.modeUpdated(k); }
			}
			if(! this.tabIndex && this.elt.tabIndex > 0)
				this.tabIndex = this.elt.tabIndex;
			if(! this.field) {
				if(this.name && this.form)
					this.field = this.form.elements[this.name];
				else if(this.field_id)
					this.field = $(this.field_id);
			}
			if(this.add_remove && this.ref_name) {
				this.ref_input = UI.getFormInput(this.form, this.ref_name);
				if(this.ref_input) {
					this.ref_input.onchange = function(key, value) {
						this.doTrackView();
						this.update(null, null, true);
						input.optionReturn(key, value);
					}
					this.ref_input.popup_return_method = 'popupReturn';
					this.ref_input.popupReturn = function(val, count, all, silent) {
						if(! count) return;
						for(var k in val)
							input.optionReturn(val[k].id, val[k]);
					}
				}
			}
			if(this.add_remove && this.sel_name) {
				this.sel_input = UI.getFormInput(this.form, this.sel_name);
				if(this.sel_input) {
					this.sel_input.onchange = function(key, value) {
						this.update('', this.placeholder, true);
						input.optionReturn(key, value);
					}
				}
			}
			this.placeholder_elt = Wrap(this.elt).findFirst('.input-label').ext(0);
			if(! this.disable_events) this.addHooks();
			this.initSelectedKeys();
			this.updateConditionals();
			if(! this.popup)
				this.handleShow();
			this.set_up = true;
		},
		addHooks: function() {
			var events = {
				focus: this.handleFocus,
				blur: this.handleBlur,
				keydown: this.handleKeyDown,
				keyup: this.handleKeyUp,
				keypress: this.handleKeyPress,
				mousedown: this.handleMouseDown,
				click: this.handleClick
			};
			this.events = UI.addEventListeners(this.elt, events, this);
			this.initDragTarget();
		},
		updateConditionals: function(hide) {
			var keys = this.getSelectedKeys();
			var callback = function(cf) {
				visible = false;
				for (var j = 0; j < cf.depends_on_value.length; j++) {
					if (~keys.indexOf(cf.depends_on_value[j])) {
						visible = true;
						break;
					}
				}
				return visible;
			}
			UI.updateConditionals(this, callback, hide);
		},
		initOptions: function() {
			if(this.options) {
				var opts = UI.getSelectOptions(this.options, null, null, this.add_remove);
				if(! opts)
					console.error('select options not found:', this.options);
				this.options = opts;
			}
		},

		validate: function() {
			this.invalid = false;
			var value = this.getSelectedKeys();
			if (this.required && !value.length) {
				this.invalid = true;
				this.invalidMsg = 'required';
			}
			this.updateDisplay();
			return ! this.invalid;
		},

		getOptions: function() {
			return this.options;
		},
		setOptions: function(opts) {
			this.options = opts;
			this.initOptions();
			if(this.rendered)
				this.renderOptions();
		},
		addOption: function(key, value, index, norender) {
			if(typeof(key) !== 'string') return;
			if(this.keys) {
				if(this.keys.indexOf(key) < 0)
					this.keys.push(key);
			}
			var o = this.getOptions(), ret, ks;
			if(o) {
				if(this.add_remove && ! this.options_lookup)
					ret = true;
				else
					ret = o.addOption(key, value, index);
				if(this.add_remove) {
					ks = this.getSelectedKeys();
					if(ks.indexOf(key) < 0) {
						if(isset(index) && index >= 0 && index < ks.length)
							ks.splice(index, 0, key);
						else
							ks.push(key);
						this.setSelectedKey(ks, true);
					}
				}
				if(this.add_remove_sorted)
					this.sortOptions();
				if(this.add_remove && (this.focused || this.opened))
					this.setCurrentIndex(this.getKeyIndex(key));
				if(this.rendered && ! norender) this.renderOptions();
			}
			return ret;
		},
		removeOption: function(key, norender) {
			var sel_ks = this.getSelectedKeys(),
				ks = this.getKeys();
			if(this.keys)
				this.keys.remove(key);
			sel_ks.remove(key);
			if(! this.add_remove || this.options_lookup) {
				this.getOptions().removeOption(key);
			}
			this.setSelectedKey(sel_ks, true);
			if(isset(this.current_index) && this.current_index >= ks.length) {
				this.setCurrentIndex(ks.length ? ks.length - 1 : null);
			}
			if(this.rendered && ! norender) {
				this.renderOptions();
				this.updateDisplay();
			}
		},
		optionReturn: function(key, value) {
			if(key && value && this.keys) {
				//var ks = this.getSelectedKeys();
				if(this.keys.indexOf(key) < 0) {
					var dname = this.options.display_name || '_display',
						opt = {id: key};
					opt[dname] = (isObject(value) ? (value[dname] || value['_display']) : value) + '';
					if(this.options_lookup && this.options_lookup.module) {
						opt.icon = this.options_lookup.icon || ('theme-icon module-' + this.options_lookup.module);
						var req = new App.conn.JSONRequest('add_recent', {silent: true, global: true},
							{module: this.options_lookup.module, record: key, source: 'ref'});
						req.fetch();
					}
					this.addOption(key, opt, null, true);
					UI.markFormDirty(this);
				} else {
					this.setCurrentIndex(this.getKeyIndex(key));
				}
				if(this.rendered) {
					this.renderOptions();
					this.focus();
				}
			}
		},
		optionRemove: function(key) {
			if(key && this.keys) {
				if(this.keys.indexOf(key) >= 0) {
					this.removeOption(key, true);
					UI.markFormDirty(this);
				}
				if(this.rendered) {
					this.renderOptions();
					this.focus();
				}
			}
		},
		popupOptionReturn: function(row, count) {
			if(! row) return;
			var result;
			for (var i = 0; i < count; i++) {
				result = row['ID_' + i];
				if(result)
					this.optionReturn(result.id, result);
			}
		},
		sortOptions: function() {
			var self = this, opts = this.options, stdKeys, stdIdx, i;
			if(this.keys) {
				if(this.options_lookup) {
					this.keys.sort(function(a,b) {
						var av = self.getValue(a),
							bv = self.getValue(b),
							dn = self.options.display_name || '_display';
						if(isObject(av))
							av = av[dn];
						if(isObject(bv))
							bv = bv[dn];
						if(! av || ! bv || av === bv) return 0;
						if(av < bv) return -1;
						return 1;
					});
				} else if(opts) {
					stdKeys = opts.getKeys();
					stdIdx = {};
					for(i = 0; i < stdKeys.length; i++) {
						stdIdx[stdKeys[i]] = i;
					}
					this.keys.sort(function(a,b) {
						var ia = stdIdx[a],
							ib = stdIdx[b];
						if(ia !== undefined || ib !== undefined) {
							if(ia === undefined) return 1;
							else if(ib === undefined) return -1;
							if(ia === ib) return 0;
							return (ia < ib ? -1 : 1);
						}
						var av = self.getValue(a),
							bv = self.getValue(b),
							dn = self.options.display_name || '_display';
						if(isObject(av))
							av = av[dn];
						if(isObject(bv))
							bv = bv[dn];
						if(! av || ! bv || av === bv) return 0;
						if(av < bv) return -1;
						return 1;
					});
				}
			}
		},
		initSelectedKeys: function() {
			if(! this.field) return;
			var val = this.field.value, ks = [];
			vals = val.split('^,^');
			if(this.add_remove) {
				this.keys = null;
				for(var i = 0; i < vals.length; i++)
					if(this.hasKey(vals[i])) ks.push(vals[i]);
				this.keys = ks;
				if(this.add_remove_sorted)
					this.sortOptions();
			}
			this.setSelectedKey(vals, true, true);
		},
		getSelectedIndex: function() {
			return this.last_selected_index;
		},
		getSelectedIndexes: function() {
			return this.selected_indexes.slice(0);
		},
		getDisabledIndexes: function() {
			return this.disabled_indexes.slice(0);
		},
		getHiddenIndexes: function() {
			return this.hidden_indexes.slice(0);
		},
		getSelectedKey: function() {
			var k = this.getKeys();
			return k[this.last_selected_index];
		},
		getSelectedValue: function() {
			var o = this.getOptions();
			if(o) return o.getValue(this.getSelectedKey());
		},
		getSelectedKeys: function() {
			var k = this.getKeys();
			var sel = [], idx;
			for(var i = 0; i < this.selected_indexes.length; i++)
				sel.push(k[this.selected_indexes[i]]);
			return sel;
		},
		getDisabledKeys: function() {
			var k = this.getKeys();
			var sel = [];
			for(var i = 0; i < this.disabled_indexes.length; i++)
				sel.push(k[this.disabled_indexes[i]]);
			return sel;
		},
		getKeys: function(options_only) {
			if(this.keys && ! options_only)
				return this.keys;
			var o = this.getOptions();
			if(o) return o.getKeys();
			return [];
		},
		getKeyIndex: function(key) {
			var k = this.getKeys();
			if(k)
				return k.indexOf(key);
			return -1;
		},
		hasKey: function(key) {
			return (this.getKeyIndex(key) >= 0);
		},
		getValues: function(keys) {
			if(! isset(keys))
				keys = this.getKeys();
			var o = this.getOptions();
			if(o) return o.getValues(keys);
			return [];
		},
		getKey: function(index) {
			var o = this.getOptions();
			if(o) return o.getKey(index);
			return null;
		},
		getValue: function(key) {
			var o = this.getOptions();
			if(o) return o.getValue(key);
			return null;
		},
		getOptionLabel: function(key) {
			var o = this.getOptions();
			if(o) return o.getLabel(key);
			return '';
		},
		getBlankKey: function() {
			if(isset(this.blank_key)) return this.blank_key;
			var o = this.getOptions();
			return o && o.blank_key || '';
		},
		getCurrentIndex: function(idx) {
			return this.current_index;
		},
		adjustCurrentIndex: function(adj, nowrap, evt) {
			if(! adj) adj = 0;
			if(! isset(this.current_index)) {
				return this.setCurrentIndex(adj == 1 ? 0 : adj, nowrap);
			} else if(this.current_index == 0 && adj < 0) {
				if(! nowrap) return this.setCurrentIndex(null);
			} else
				return this.setCurrentIndex(this.current_index + adj, nowrap, null, evt);
		},
		setCurrentIndex: function(idx, nowrap, silent, evt) {
			var old = this.current_index;
			if(isset(idx)) {
				var k = this.getKeys();
				if(idx < 0) {
					if(! nowrap)
						idx = k.length + idx;
					if(idx < 0)
						idx = nowrap ? 0 : null;
				}
				if(idx >= k.length) {
					idx = nowrap ? k.length - 1 : null;
				}
			} else
				idx = null;
			if(! this.canPositionOver(idx, evt))
				return false;
			this.current_index = idx;
			this.updateOptionsState([old, idx]);
			if(this.focused || ! evt)
				this.showCurrent(evt);
			if(! silent && old !== idx)
				UI.callEvent(this, this.oncurrent, idx, old);
		},
		getOptionRow: function(index) {
			return this.body ? this.body.childNodes[index] : null;
		},
		getOptionCount: function() {
			return this.getKeys().length;
		},
		getRowCount: function() {
			return this.add_remove ? this.getSelectedIndexes().length : this.getKeys().length;
		},
		getScrollPosition: function() {
			return this.body ? this.body.scrollTop : 0;
		},
		getOptionPosition: function(index) {
			var rgn, rel_top, elt, ergn;
			if(! isset(index) || ! this.body || ! (elt = this.body.childNodes[index])) return;
			rgn = UI.getEltRegion(this.body);
			if(! (ergn = UI.getEltRegion(elt)) || ! ergn.height)
				return;
			ergn.rel_top = ergn.top - rgn.top;
			ergn.body_top = rgn.top;
			ergn.body_height = rgn.height;
			ergn.scroll_top = ergn.rel_top + this.body.scrollTop;
			ergn.scroll_height = this.body.scrollHeight;
			return ergn;
		},
		getRowData: function(indexes) {
			var keys = this.add_remove ? this.getSelectedKeys() : this.getKeys();
			var qkeys;
			if(isset(indexes)) {
				qkeys = [];
				for(var i = 0; i < indexes.length; i++)
					qkeys.push(keys[indexes[i]]);
			} else
				qkeys = keys;
			var values = this.getValues(qkeys);
			var selected = this.getSelectedIndexes();
			var disabled = this.getDisabledIndexes();
			var hidden = this.getHiddenIndexes();
			var sel_map = {}, dis_map = {};
			for(var i = 0; i < selected.length; i++)
				sel_map[selected[i]] = 1;
			for(var i = 0; i < disabled.length; i++)
				dis_map[disabled[i]] = 1;
			var ret = [];
			var sel = 0, dis = 0;
			for(var i = 0; i < qkeys.length; i++) {
				if (hidden.indexOf(i) >= 0)
					continue;
				var key = qkeys[i];
				var idx = isset(indexes) ? indexes[i] : i;
				sel = !! sel_map[idx];
				dis = !! dis_map[idx];
				var status = {selected: sel, disabled: dis, current: idx == this.current_index};
				if(this.mark_blank && key === this.getBlankKey())
					status.blank = true;
				ret.push({key: key, value: values[i], status: status, index: idx});
				sel = dis = 0;
			}
			return ret;
		},
		setColumns: function(cols) {
			if(cols)
				this.columns = cols.slice(0);
		},
		getColumns: function() {
			var src = this.columns;
			var default_column = this.label_key || 'label';
			var o = this.getOptions();
			if(o && o.display_name) default_column = o.display_name;
			if(! src && o) src = o.columns;

			var cols = [], maincol;
			if(this.show_checks)
				cols.push({type: 'check'});
			if(src) {
				for(var i = 0; i < src.length; i++) {
					var c = src[i];
					if(typeof(c) == 'string')
						cols.push({name: c /*, icon: value.icon ? 'icon' : ''*/ });
					else
						cols.push(c);
				}
			} else {
				if(this.icon_key)
					cols.push({name: this.icon_key, type: 'icon'});
				cols.push(maincol = {name: default_column});
				if(this.expand) maincol.expand = this.expand;
				if(this.expand_property) maincol.expand_property = this.expand_property;
			}
			return cols;
		},
		getWidth: function() {
			if(this.width)
				return UI.formatSize(this.width);
			var o = this.getOptions()
			if(o && o.width)
				return UI.formatSize(o.width);
			if(this.popup) {
				if(o && o.maxlen)
					return UI.formatSize('' + (Math.min(Math.max(o.maxlen, 10), 20) * 0.8) + 'em');
				return '150px';
			}
		},
		render: function() {
			if(this.rendered)
				return this.elt;
			if(! this.elt && ! (this.elt = $(this.id)) ) {
				var cls = this.className ? this.className :
					(this.popup ? 'card-outer panel-outer panel-border panel-default panel-round popup-default ' : (this.flat ? '' : 'input-field '));
				if(this.block) cls += ' input-block';
				if(this.ext_class) cls += ' ' + this.ext_class;
				this.elt = createElement2('div', {id: this.id, className: cls, tabIndex: this.tabIndex});
				if(this.popup) toggleDisplay(this.elt, false);
			}
			var w = this.getWidth();
			if(w && ! this.block) this.elt.style.width = w;
			if(! this.body) {
				this.body = Wrap(this.elt).findFirst('.menu-outer').get(0) ||
					createElement2('div', {className: 'card-body menu-outer input-scroll', style:'width: 100%;'});
			}
			this.elt.appendChild(this.body);
			this.rendered = true;
			return this.elt;
		},
		autoShowSelected: function() {
			if(this.rendered) this.showSelected();
		},
		focus: function() {
			if(this.elt)
				this.elt.focus();
		},
		handleFocus: function(evt) {
			if(! this.focused) {
				this.focused = true;
				UI.callEvent(this, this.onfocus, evt);
				this.updateOptionsState();
				this.updateDisplay();
			}
		},
		handleBlur: function(evt) {
			if(this.focused) {
				this.focused = false;
				UI.callEvent(this, this.onblur, evt);
				if(! this.opened) this.setCurrentIndex(null);
				this.updateOptionsState();
				this.updateDisplay();
			}
		},
		setPanelHeight: function(immed) {
			if(! this.body) return;
			if(! immed) setTimeout((function() { this.setPanelHeight(true); }).bind(this), 0);
			var rmax = this.rows,
				rmin = this.min_rows || 0,
				minh = null,
				maxh = null,
				opth = this.getOptionHeight();
			if(this.add_remove) {
				rmin = Math.max(1, rmin);
			}
			if(rmin && opth) {
				minh = rmin * opth + this.options_pad*2;
			}
			if(isset(rmax) && opth) {
				maxh = rmax * opth + this.options_pad*2;
			}
			Wrap(this.body).cssSize('min-height', minh).cssSize('max-height', maxh);
		},
		getOptionHeight: function() {
			if(! this.height_fixed && this.rendered) {
				var cls = this.elt.className;
				if(cls !== this.option_height_cls || ! this.option_height) {
					// height may depend on class (input-sm, input-lg)
					var columns = this.getColumns(),
						row = {key: '_hght_', value: '_', status: {}, index: -1},
						snd = Wrap(this.renderOption(row, columns)).prependTo(this.body),
						fst = Wrap(this.renderOption(row, columns)).prependTo(this.body),
						topOffs = fst.bounds(this.body, true),
						sndOffs = snd.bounds(fst);
					fst.remove();
					snd.remove();
					if(topOffs && topOffs.height) {
						this.options_pad = topOffs ? topOffs.top : 0;
						this.option_height = sndOffs ? sndOffs.top : 0;
						this.option_height_cls = cls;
					}
					if(this.option_height < 14) this.option_height = 20; // try later
				}
			}
			return this.option_height;
		},
		handleResize: function(pos, viewp) {
			this.updateDisplay();
		},
		handleKeyDown: function(evt) {
			if(evt.keyCode == 38) nav = -1;
			else if(evt.keyCode == 40) nav = 1;
			else if(evt.keyCode == 37 || evt.keyCode == 39) {
				UI.callEvent(this, this.onprevnext, evt, evt.keyCode == 39);
				UI.stopEvent(evt);
				return;
			}
			else nav = 0;
			if(nav) {
				this.adjustCurrentIndex(nav, null, evt);
				UI.stopEvent(evt);
			} else if(evt.keyCode == 32 || ((this.popup || this.add_remove || this.keyboard_select) && evt.keyCode == 13)) {
				if(this.add_remove) {
					if(! this.opened && ! this.disabled)
						this.showSearch();
					else
						this.hideSearch();
				}
				else if(isset(this.current_index))
					this.toggleSelected(this.current_index, null, null, null, evt);
				UI.stopEvent(evt);
			} else if(evt.keyCode == 8 || evt.keyCode == 46) { // delete
				if(this.add_remove) {
					var cidx = this.getCurrentIndex(), key;
					if(isset(cidx) && this.keys) {
						this.removeOption(this.keys[cidx]);
						UI.markFormDirty(this);
					}
				}
				UI.stopEvent(evt);
			} else if(! this.popup && evt.keyCode == 13) { // enter
				UI.stopEvent(evt);
				if(this.form) UI.submitForm(this.form);
			} else if(this.popup && evt.keyCode == 192) { // tilde (for debugging)
				var p = App.popups.getCurrent();
				if(p) p.close_on_exit = false;
			} else if(evt.keyCode == 27) { // esc
				this.cancel(true);
				UI.stopEvent(evt);
			}
			UI.callEvent(this, this.onkeydown, evt);
		},
		handleKeyUp: function(evt) {
			UI.callEvent(this, this.onkeyup, evt);
		},
        handleKeyPress: function(evt) {
			if(UI.callEvent(this, this.onkeypress, evt) === false)
				return;
            if(evt.charCode) {
                var last = this.keypress_time,
                    cur = new Date().getTime(),
                    pfx = this.keypress_buffer;
                if(! last || cur - last > 400)
                    pfx = '';
                this.keypress_buffer = pfx + String.fromCharCode(evt.charCode).toLowerCase();
                this.keypress_time = cur;
                var key = this.findKeyPrefix(this.keypress_buffer);
                if (! isset(key)) {
                    this.setCurrentIndex(null, 0, 0, evt);
                    key = this.findKeyPrefix(this.keypress_buffer);
                }
                if(isset(key))
                    this.setCurrentIndex(this.getKeyIndex(key), 0, 0, evt);
            }
        },
        findKeyPrefix: function(pfx) {
            var ks, i, l = pfx.length, sel = this.getSelectedKey(), first;
            if(! l || ! (ks = this.getKeys())) return;
            var curr_idx = this.getCurrentIndex();
            for(i = 0; i < ks.length; i++) {
                if(ks[i] === sel)
                    continue;
                var val = this.getOptionLabel(ks[i]);
                if(val.substring(0, l).toLowerCase() === pfx) {
                    var key_idx = this.getKeyIndex(ks[i]);
                    if (key_idx == curr_idx || key_idx < curr_idx)
                        continue;
                    if(! isset(first)) first = ks[i];
                }
            }
            return first;
        },
		handleMouseDown: function(evt) {
			if(this.add_remove) {
				UI.stopPropagation(evt);
				if(this.opened) return;
			}
			if(! this.focused)
				this.focus();
			UI.callEvent(this, this.onmousedown, evt);
		},
		handleClick: function(evt) {
			if(this.add_remove) {
				if(! this.opened && ! this.disabled)
					this.showSearch();
				else
					this.hideSearch();
				evt.stopEvent();
			}
			else if(! this.focused)
				this.focus();
			UI.callEvent(this, this.onclick, evt);
		},
		canPositionOver: function(idx, evt) {
			var mpos = evt ? {x: evt.clientX || evt.x, y: evt.clientY || evt.y} : null;
			if(! this.scroll_panel || ! isset(idx))
				return true;
			return this.scroll_panel.canShowStep(idx, mpos);
		},
		showIndex: function(idx) {
			var pos = this.getOptionPosition(idx), top, btm;
			if(pos) {
				top = (idx == 0 ? 0 : pos.scroll_top);
				btm = pos.scroll_top + pos.height;
				if(idx == this.body.childNodes.length - 1)
					btm = pos.scroll_height;
				if(top < this.body.scrollTop)
					this.body.scrollTop = top;
				else if(btm > this.body.scrollTop + pos.body_height) {
					this.body.scrollTop = btm - pos.body_height;
				}
			}
		},
		showCurrent: function(evt) {
			var mpos = evt ? {x: evt.clientX || evt.x, y: evt.clientY || evt.y} : null;
			this.showIndex(this.current_index, mpos);
		},
		showSelected: function() {
			var idx = this.getSelectedIndexes();
			if(idx.length) this.showIndex(idx[0]);
		},
		handleShow: function() {
			this.initDragTarget();
			this.renderOptions();
			if(this.auto_show_selected)
				this.autoShowSelected();
		},
		cancel: function(refocus) {
			if(! this.popup)
				this.setCurrentIndex(null);
			UI.callEvent(this, this.oncancel, refocus);
		},
		destroy: function() {
			UI.removeNode(this.elt, true);
			this.elt = this.field = null;
			this.rendered = false;
			this.set_up = false;
			if(this.scroll_panel) this.scroll_panel.destroy();
			this.scroll_panel = null;
			this.opts_table = null;
			this.visible_rows = null;
			this.focused = false;
			UI.callEvent(this, this.ondestroy);
		},
		selectCurrent: function(evt) {
			if(isset(this.current_index)) {
				this.setSelected(this.current_index, null, null, evt);
				return true;
			}
		},
		clearSelected: function() {
			if(this.keys && this.keys.length) {
				for(var i = this.keys.length; i >= 0; i--)
					this.removeOption(this.keys[i]);
				UI.markFormDirty(this);
			}
		},
		setSelected: function(index, silent, nosource, evt) {
			this.selected_indexes = [];
			this.last_selected_index = null;
			this.toggleSelected(index, 1, silent, nosource, evt);
		},
		toggleSelected: function(index, state, silent, nosource, evt) {
			var disabled = this.getDisabledIndexes();
			var dis_map = {};
			for(var i = 0; i < disabled.length; i++)
				dis_map[disabled[i]] = 1;
			if (dis_map[index])
				return;
			var oldSel = this.multi_select ? this.selected_indexes : this.last_selected_index;
			if(! isset(state)) state = -1;
			else if(state !== -1) state = state ? 1 : 0;
			if(isset(index)) {
				if(! Array.isArray(index))
					index = [index];
				var keys = this.getKeys();
				var selPos = {}, sel = this.getSelectedIndexes();
				for(var i = 0; i < sel.length; i++)
					selPos[sel[i]] = i;
				var idx, remove = [];
				for(var i = 0; i < index.length; i++) {
					idx = index[i];
					if(idx < 0 || idx >= keys.length)
						continue;
					if(isset(selPos[idx])) {
						remove.push(selPos[idx]);
						if(state < 1)
							delete selPos[idx];
						else {
							selPos[idx] = sel.length;
							this.last_selected_index = idx;
							sel.push(idx);
						}
					} else if(state) {
						selPos[idx] = sel.length;
						this.last_selected_index = idx;
						sel.push(idx);
					}
				}
				remove.sort(cmpFn);
				for(var i = 0; i < remove.length; i++)
					sel.splice(remove[i], 1);
				if(! this.multi_select && sel.length > 1)
					sel = [sel.pop()];
				this.selected_indexes = sel;
			}
			if(this.add_remove) this.renderOptions();
			else this.updateOptionsState();
			if(! nosource && this.field) {
				var selKeys = [];
				if(isset(index)) {
					for(var k=0; k < sel.length; k++) {
						selKeys.push(keys[sel[k]]);
					}
				}
				this.field.value = selKeys.filter(function(k) {return (''+k).length}).join('^,^');
			}
			if(! silent)
				UI.callEvent(this, this.onchange, oldSel, evt);
			this.updateConditionals();
		},
		setSelectedKey: function(keys, silent, nosource) {
			var idx = [];
			if(! Array.isArray(keys))
				keys = isset(keys) ? [keys] : [];
			for(var i = 0; i < keys.length; i++)
				idx.push(this.getKeyIndex(keys[i]));
			this.setSelected(idx, silent, nosource);
		},
		renderOptions: function() {
			var body = this.body;
			if(! body) return;
			UI.clearChildNodes(body);

			var columns = this.getColumns();
			var rows = this.getRowData();
			for(var i = 0; i < rows.length; i++) {
				if(this.add_remove && this.selected_indexes.indexOf(i) < 0)
					continue;
				var opt = this.renderOption(rows[i], columns);
				if(opt) body.appendChild(opt);
			}
			if(! rows.length && this.placeholder) {
				MakeElt('div.menu-option.static.placeholder', this.placeholder).appendTo(body);
			}

			this.updateDisplay();
		},
		cleanOptionValue: function(row) {
			var value = row.value, opts = this.getOptions(),
				dn = (opts && opts.display_name) || 'label', v;
			if (typeof(value) != 'object') value = '' + value;
			if(typeof(value) == 'string') {
				v = {}; v[dn] = value;
				value = v;
			}
			return value;
		},
		renderOption: function(row, columns) {
			var value = this.cleanOptionValue(row),
				extCls = [],
				dragCfg = this.getDragConfig(),
				elt, ext_url, display, wrapElt, rowCls, staticRow;
			for(var k in row.status) {
				if(row.status[k] && (k !== 'selected' || ! this.add_remove))
					extCls.push(k);
			}
			if(this.disable_current) extCls.push('static');
			if(! this.options_wrap) extCls.push('single');
			if(this.options_active) extCls.push('color');
			extCls = extCls.join(' ');
			rowCls = 'menu-option '+extCls;

			if(value._header) {
				rowCls = 'menu-header';
				staticRow = true;
			} else if(value._separator) {
				rowCls = 'menu-separator';
				staticRow = true;
			}

			elt = createElement2('div', {className: rowCls});
			wrapElt = Wrap(elt).reinit();
			var optinfo = {scroll: this, key: row.key, value: value, index: row.index};
			elt.optinfo = optinfo;

			if(staticRow) {
				if(value._header) {
					UI.setElementContent(elt, value.label);
				}
			}
			else {
				for(var evt in UI.SelectList.option_events) {
					var handler = (this.events && isdef(this.events['on'+evt]))
						? this.events['on'+evt] : UI.SelectList.option_events[evt];
					if(handler)
						wrapElt.on(evt, optinfo, handler);
				}
				for(var ci = 0; ci < columns.length; ci++) {
					var cdef = columns[ci];
					var t = cdef.type;
					var colcls, lbl, icon;
					if(t == 'check')
						colcls = 'option-check';
					else if(t == 'icon')
						colcls = 'option-icon';
					else if(t == 'color')
						colcls = 'option-icon';
					else if(cdef.icon)
						colcls = 'with-icon';
					else
						colcls = '';
					var col = createElement2('div', {className: 'option-cell input-label ' + colcls});
					if(cdef.width) {
						UI.setWidth(col, cdef.width);
						Wrap(col).cssSize('flex-basis', cdef.width);
					}
					if(cdef.align)
						col.style.textAlign = cdef.align;
					if(cdef.expand || (cdef.expand_property && value[cdef.expand_property]))
						UI.addClass(col, 'rbullet rbullet-right');
					if(t == 'check')
						;
					else if(t == 'icon') {
						icon = '';
						if(cdef.name == 'key')
							icon = ''+row.key;
						else if(isset(value) && typeof(value) == 'object')
							icon = ''+value[cdef.name];
						if(this.active_icons) icon += ' active';
						col.appendChild(UI.createIcon(icon));
					}
					else if(t == 'color') {
						icon = ''+value[cdef.name || 'color'];
						icon = MakeElt('div.input-icon.input-color-icon', {style: 'background-color: '+icon});
						icon.appendTo(col);
					}
					else {
						if(cdef.icon) {
							col.appendChild(UI.createIcon(value[cdef.icon]));
						}
						if(value) {
							if(! isset(display)) display = value[cdef.name];
							col.appendChild(document.createTextNode(get_default(value[cdef.name], '')));
						}
					}
					elt.appendChild(col);
				}
			}

			if(value) {
				if(dragCfg && dragCfg.as_source && (get_default(dragCfg.options_default, true)
					|| (dragCfg.options_key && value[dragCfg.options_key]))) {

					new UI.Draggable(elt, {
						sourceType: 'SelectListItem',
						source: this.drag_target,
						onStateChanged: function(evt, state) {
							UI.addRemoveClass(elt, 'drag-remove', state.value === 'accept');
						},
						values: {
							index: row.index,
							key: row.key,
							option: value
						}
					});
				}
				else if(value.url && (! dragCfg || ! dragCfg.options_key)) {
					ext_url = value.url;
					if(ext_url.indexOf('://') < 0)
						ext_url = '' + App.session.site_url + ext_url;
					new UI.Draggable(elt, {
						sourceType: 'SelectListItem',
						source: this.drag_target,
						transfer: {
							'text/plain': display || ext_url,
							'text/uri-list': ext_url,
							'url': ext_url
						},
						values: {
							index: row.index,
							key: row.key,
							option: value,
							url: ext_url
						}
					});
				}
			}

			if(this.add_remove) {
				var col = createElement2('div', {className: 'option-cell input-label option-remove'});
				var rem = createElement2('div', {className: 'uii uii-remove active-icon'});
				var self = this;
				rem.onmousedown = rem.ontouchstart = UI.stopPropagation;
				rem.onmouseup = function(evt) { self.optionEvent(elt, optinfo, 'remove', evt); }
				col.appendChild(rem);
				elt.appendChild(col);
			}

			return elt;
		},
		updateOptionsState: function(indexes) {
			// update non-destructively if possible
			if(! this.body) return;
			var std = {'selected':1, 'current':1, 'disabled':1};
			var done = {};
			var rows = this.getRowData(indexes);
            for(var i = 0; i < rows.length; i++) {
				var row = rows[i];
				if(done[row.index]) continue;
				done[row.index] = 1;
                var tr = this.body.childNodes[row.index];
				if(! tr) continue;
				var cls = [];
				var parts = tr.className.split(/\s+/);
				for(var p = 0; p < parts.length; p++) {
					var c = parts[p];
					if(c && ! std[c]) {
						if(c == 'active' && ! row.status.current)
							continue;
						cls.push(c);
					}
				}
				for(var c in std) {
					if(c == 'current' && ! this.focused && this.require_focus && ! this.add_remove)
						continue;
					if(c == 'current' && this.disable_current)
						continue;
					if(c == 'selected' && this.add_remove)
						continue;
					if(row.status[c])
						cls.push(c);
				}
				tr.className = cls.join(' ');
			}
		},
		optionClick: function(elt, optinfo, evt) {
			if(this.add_remove) {
				return;
			}
			if(evt.type === 'touchend') {
				UI.preventDefault(evt);
				if(this.lastOverPos && Math.abs(this.lastOverPos.x - evt.pageX) + Math.abs(this.lastOverPos.y - evt.pageY) > 5)
					return;
			}
			var undoMeta = null;
			if(isset(this.metaPressed) && this.metaPressed == optinfo.index) {
				undoMeta = window.metaKeyPressed;
				window.metaKeyPressed = true;
			}
			this.toggleSelected(optinfo.index, null, null, null, evt);
			UI.stopPropagation(evt);
			// add short animation (blink)
			//App.popups.hidePopup();
			if(isset(undoMeta))
				window.metaKeyPressed = undoMeta;
		},
		optionEvent: function(elt, info, state, evt) {
			if(this.popup) {
				var p = elt;
				// skip event if in a hidden popup
				while(p && p !== window) {
					if(isset(p.popup_visible) && ! p.popup_visible)
						return;
    				p = p.parentNode;
				}
			}
			if(state == 'over' && ! this.disable_current) {
				//var pos = this.scroll_panel.getPosition();
				//if(pos && evt && evt.pageY <= pos.top + pos.rel_top) return; // ignore top pixel row
				// ignore repeated over event due to scrolling
				if(this.lastOverPos && this.lastOverPos.x == evt.pageX && this.lastOverPos.y == evt.pageY) return;
				this.setCurrentIndex(info.index, 0, 0, evt);
				this.metaPressed = null;
				this.lastOverPos = {x: evt.pageX, y: evt.pageY};
			}
			else if(state == 'move') {
				if(evt && evt.touches.length == 1) {
					// must determine which option is currently hovered (event target is the option from touchstart)
					var c = this.getOptionCount(), i = 0, pos, found;
					for(; i < c; i++) {
						pos = this.getOptionPosition(i);
						if(evt.pageY >= pos.top && evt.pageY <= pos.top + pos.height && evt.pageX >= pos.left && evt.pageX <= pos.left + pos.width) {
							this.setCurrentIndex(i, 0, 0, evt);
							info.index = i;
							found = 1;
							break;
						}
					}
					if(! found) {
						this.setCurrentIndex(null, 0, 0, evt);
						info.index = null;
					}
				}
			}
			else if(this.current_index === info.index) {
				if(state == 'out' || state == 'down' || state == 'up') {
					var targ = evt.relatedTarget;
					while(targ && targ.parentNode) {
						targ = targ.parentNode;
						if(targ === elt)
							return; // ignore event from child node
					}
				}
				if(state == 'out') {
					this.setCurrentIndex(null, 0, 0, evt);
				}
				else if(state == 'down') {
					UI.addClass(elt, 'active');
					if(evt && (evt.which == 2 || (! evt.which && evt.button == 4)))
						this.metaPressed = info.index;
				}
				else if(state == 'up') {
					UI.removeClass(elt, 'active');
				}
				else if(state == 'remove') {
					if(this.keys) {
						this.removeOption(info.key);
						UI.markFormDirty(this);
					}
					if(this.opened)
						this.hideSearch();
					UI.stopEvent(evt);
				}
			}
		},
		modeUpdated: function(mode) {
			this.updateDisplay();
			if(mode) {
				if(this.getRowCount())
					this.focus();
				else
					this.showSearch();
			} else {
				this.clearSelected();
			}
		},
		updateDisplay: function() {
			if(this.mode_field) {
				UI.setDisplayState(this.elt, !! this.mode_field.getValue());
			}
			if(this.header_text) {
				if(! this.header_elt) this.header_elt = MakeElt('div.input-field-header').prependTo(this.elt);
				this.header_elt.text(this.header_text);
			} else {
				UI.setDisplayState(this.header_elt, false);
			}
			UI.addRemoveClass(this.elt, 'active', !! (this.focused || this.opened || this.dragover));
			if(this.add_remove) {
				UI.addRemoveClass(this.elt, 'flat-bottom', !! this.opened);
				if(this.getRowCount()) {
					UI.setDisplayState(this.body, true);
					UI.setDisplayState(this.placeholder_elt, false);
				} else {
					if(! this.placeholder_elt) {
						this.placeholder_elt = MakeElt('div.input-label')
							.appendTo(this.elt);
					}
					this.placeholder_elt.text(this.placeholder || app_string('LBL_NONE'));
					UI.setDisplayState(this.body, !!this.full_height);
					UI.setDisplayState(this.placeholder_elt, !this.full_height);
					this.placeholder_elt.toggleClass('text-placeholder', ! this.focused && ! this.opened);
				}
			}
			this.setPanelHeight();
		},
		initSearcher: function() {
			var self = this,
				lookup = this.options_lookup,
				search_opts = {
					allow_clear: true,
					disable_recent: true,
					target: this.elt,
					target_offset: {top: -1, left: 0},
					form: this.form,
					options: this.options
				};
			if(this.label_key)
				search_opts.display_name = this.label_key;
			if(this.icon_key)
				search_opts.icon_key = this.icon_key;
			if(this.add_remove)
				search_opts.popup_return_multiple = true;
			if(this.add_remove && lookup) {
				delete search_opts.options;
				search_opts.disable_recent = lookup.disable_recent;
				search_opts.module = lookup.module;
				search_opts.extra_fields = lookup.extra_fields;
				search_opts.filters = lookup.filters;
				search_opts.popup_layout = lookup.popup_layout;
				search_opts.quick_create = lookup.quick_create;
				search_opts.allow_popup = get_default(lookup.allow_popup, true);
				search_opts.passthru = {
					ui_input_name: this.name || this.id,
					ui_return_method: 'popupOptionReturn'
				};
			} else {
				search_opts.search_blank = true;
				search_opts.matching_only = true;
			}
			this.searcher = new UI.SearchSelect(this.id + '-search', search_opts);
			this.searcher.onchange = function(k, v, mod, keepOpen) {
				self.optionReturn(k, v);
				if(keepOpen) {
					this.setSourceKeys(self.getKeys());
					this.reposition();
				}
				else
					self.focus();
			}
			this.searcher.onremove = function(k, v, mod, keepOpen) {
				self.optionRemove(k);
				if(keepOpen) {
					this.setSourceKeys(self.getKeys());
					this.reposition();
				}
				else
					self.focus();
			}
			this.searcher.onshow = function() {
				self.opened = true;
				self.updateDisplay();
			}
			this.searcher.onhide = function() {
				self.opened = false;
				self.updateDisplay();
			}
			this.searcher.onclear = function() {
				self.clearSelected();
			}
			this.searcher.oncancel = function() {
				self.focus();
			}
			this.searcher.setup();
		},
		showSearch: function(query) {
			if(! this.searcher) {
				this.initSearcher();
			}
			this.searcher.render();
			var w = UI.getEltRegion(this.elt),
				k = this.getKeys();
			if(w)
				this.searcher.setWidth(Math.max(w.width - 24, 300) + 'px');
			//this.searcher.setKey(k.join('^,^'));
			this.searcher.setSourceKeys(k);
			this.searcher.showDefault(true);
		},
		hideSearch: function() {
			if(this.searcher)
				this.searcher.close();
		},

		getDragConfig: function() {
			return this.drag_config;
		},
		setDragConfig: function(cfg) {
			this.drag_config = cfg;
			this.initDragTarget();
		},
		initDragTarget: function() {
			if(! this.elt) return;
			var cfg = this.getDragConfig();
			if(cfg && cfg.as_source) {
				if(! this.drag_target) {
					this.drag_target = new UI.DragTarget(this.elt, {
						move_cursor: cfg.move_cursor,
						source_input: this,
						beforeDragContent: function(evt, drag) {
							this.drag_index = drag.getDragValues().index;
							this.drag_row = drag.elt;
							cfg.beforeDragContent && cfg.beforeDragContent(evt, drag);
						}.bind(this),
						afterDragContent: function(evt, drag, data) {
							delete this.drag_index;
							delete this.drag_row;
							cfg.afterDragContent && cfg.afterDragContent(evt, drag, data);
						}.bind(this),
						onMove: function(evt, data) {
							this.setDragPosition(evt, data);
							cfg.onMove && cfg.onMove(evt, data);
						}.bind(this),
						checkAccept: cfg.checkAccept || function() {},
						onAccept: cfg.onAccept,
						onLeave: function(evt, data) {
							//if(this.body)
							//	Dom.query(this.body.childNodes).removeClass('current selected');
							this.setDragPosition(evt, data);
							cfg.onLeave && cfg.onLeave(evt, data);
						}.bind(this),
						onDrop: cfg.onDrop
					});
				}
			} else {
				if(this.drag_target) {
					this.drag_target.destroy();
					this.drag_target = null;
				}
			}
		},

		// pos = null to clear current drag operation
		setDragPosition: function(evt, data) {
			var dragCfg = this.getDragConfig(),
				lastIndex, lastPos,
				pos = data.position;
			if(! dragCfg) return;
			lastIndex = data.target_index;
			if(! this.body) return;
			data.target_index = null;
			data.after_elt = null;
			if(pos && data.state === 'accept') {
				var wrapBody = Wrap(this.body),
					bodyPos = wrapBody.bounds(),
					ypos = pos.y - bodyPos.top + wrapBody.scrollTop(),
					idx = 0, prevRow, prevIdx, rowBounds, row, i;
				for(i = 0; i < this.body.childNodes.length; i++) {
					row = this.body.childNodes[i];
					if(UI.hasClass(row, 'drag-indicator') || UI.hasClass(row, 'placeholder'))
						continue;
					if(row.offsetTop >= ypos) {
						// first row below cursor
						data.target_index = idx;
						data.after_elt = row;
						// see if we are above midpoint of previous row
						if(prevRow) {
							rowBounds = Wrap(prevRow).bounds();
							if(rowBounds.top + rowBounds.height / 2 >= pos.y) {
								data.target_index = prevIdx;
								data.after_elt = prevRow;
							}
						}
						break;
					}
					prevRow = row;
					prevIdx = idx;
					idx ++;
				}
				if(data.target_index === null)
					data.target_index = idx;
			}
			if(data.target_index !== lastIndex) {
				var dragIdx = this.drag_index,
					tgtIdx = data.target_index;
				this.drag_ignore = false;
				if(data.self && isset(dragIdx) &&
				(dragIdx === tgtIdx || dragIdx + 1 === tgtIdx)) {
					this.drag_ignore = true;
				}
				if(data.target_index !== null && ! this.drag_ignore) {
					Wrap(this.body).find('.menu-option.placeholder').hide();
					if(! dragCfg.indicator_elt && get_default(dragCfg.position_target, true)) {
						if (dragCfg.create_indicator_elt) {
							dragCfg.indicator_elt = dragCfg.create_indicator_elt(this);
						} else {
							var columns = this.getColumns();
							var row = {key: '_drag_', value: '', status: {}, index: -99};
							var opt = this.renderOption(row, columns);
							if(opt) {
								UI.addClass(opt, 'drag-indicator');
								dragCfg.indicator_elt = Wrap(opt);
							}
						}
					}
					if(dragCfg.indicator_elt) {
						this.body.insertBefore(dragCfg.indicator_elt.get(), data.after_elt || null);
					}
				} else if(dragCfg.indicator_elt) {
					dragCfg.indicator_elt.remove();
					dragCfg.indicator_elt = null;
					Wrap(this.body).find('.menu-option.placeholder').show();
				}
				requestAnimationFrame(this.updateDisplay.bind(this));
			}
		}
	};


	UI.SelectInput = function(id, params) {
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		this.menu_id = this.id + '-popup';
		this.show_checks = true;
		this.close_on_exit = true;
		this.require_existing = false;
		this.mark_blank = true;
		var options;
		if(params && params.options) {
			options = params.options;
			delete params.options;
		} else
			options = '';
		if(params) extendObject(this, params);
		if(! this.menu) {
			if(options instanceof UI.SelectList) {
				this.menu = options;
			}
			else {
				this.menu = new UI.SelectList(this.menu_id, {
					options: options, popup: true, show_checks: this.show_checks,
					width: this.options_width,
					className: this.popup_class_name, icon_key: this.icon_key,
					disabled_indexes: this.disabled_indexes || [],
					mark_blank: this.mark_blank});
			}
		}
	}
	UI.SelectInput.prototype = {
		setup: function() {
			if(this.set_up)
				return;
			if(! this.elt)
				this.elt = $(this.id);
			if(! this.field) {
				if(this.name && this.form)
					this.field = this.form.elements[this.name];
				else if(this.field_id)
					this.field = $(this.field_id);
			}
			if(! this.outer) {
				if(this.outer_id)
					this.outer = $(this.outer_id);
				else
					this.outer = this.elt;
			}
			if(this.menu) {
				if(this.elt && this.elt.tabIndex)
					this.menu.tabIndex = this.elt.tabIndex;
				else if(! this.menu.tabIndex)
					this.menu.tabIndex = 1;
			}
			if(this.dynamic_module && ! this.module_field) {
				this.module_field = this.form.elements[this.dynamic_module];
			}
			if(! this.altInput && this.alt_name) {
				this.altInput = UI.getFormInput(this.form, this.alt_name);
			}
			if(! this.elt)
				return;
			if(! this.tabIndex && this.elt.tabIndex > 0)
				this.tabIndex = this.elt.tabIndex;
			if(this.field) this.orig_value = this.field.value;
			this.addHooks();
			this.updateDisplay();
			this.set_up = true;
			this.updateConditionals();
		},
		addHooks: function() {
			var input = this;
			if(this.elt) {
				UI.addEventListener(this.elt, 'selectstart', UI.stopEvent);
				UI.addEventListener(this.elt, 'mousedown', UI.stopPropagation);
				UI.addEventListener(this.elt, 'click', this.handleClick, this);
				UI.addEventListener(this.elt, 'touchstart', UI.preventDefault);
				UI.addEventListener(this.elt, 'touchend', this.handleClick, this);
				UI.addEventListener(this.elt, 'focus', this.handleFocus, this);
				UI.addEventListener(this.elt, 'blur', this.handleBlur, this);
				UI.addEventListener(this.elt, 'keydown', this.handleKeyDown, this);
				UI.addEventListener(this.elt, 'keypress', this.handleKeyPress, this);
			}
			if(this.altInput) {
				this.altInput.onchange = function() { input.update(); }
			}
			this.menu.onkeypress = function(evt) { input.handleKeyPress(evt); return false; }
		},
		getField: function() {
			return this.field;
		},
		getValue: function() {
			return (this.field ? this.field.value : get_default(this.init_value, null));
		},
		getOptionValue: function(key) {
			if(! isset(key)) key = this.getValue();
			return this.menu ? this.menu.getValue(key) : key;
		},
		checkAltValue: function() {
			var v;
			if(this.altInput && (v = this.altInput.getValue())) {
				return v;
			}
		},
		setValue: function(key, silent) {
			var v = key;
			if(this.menu && ! (v = this.menu.getValue(key))) {
				console.error(this.id + ' key not defined: ' + key);
				return;
			}
			this.update(key, v, silent);
		},
		setOptions: function(opts) {
			if(this.menu) return this.menu.setOptions(opts);
		},
		focus: function() {
			if(this.elt) this.elt.focus();
		},
		handleClick: function(evt) {
			if(this.opened) {
				this.hidePopup();
			}
			else if(! this.disabled) {
				this.showDefault();
			}
			UI.stopEvent(evt);
		},
		handleFocus: function(evt) {
			if(! this.focused) {
				this.focused = true;
				UI.callEvent(this, this.onfocus, evt);
			}
			this.updateDisplay();
		},
		handleBlur: function(evt) {
			if(this.focused) {
				this.focused = false;
				UI.callEvent(this, this.onblur, evt);
				this.updateDisplay();
			}
		},
		handleKeyDown: function(evt) {
			if(evt.keyCode == 27) {
				if(this.clear())
					UI.stopEvent(evt);
			}
			else if(evt.keyCode == 32) {
				UI.stopEvent(evt);
				if(! this.opened && ! this.disabled) this.showDefault(evt);
			}
			else if(evt.keyCode == 13) {
				UI.stopEvent(evt);
				//if(this.form) UI.submitForm(this.form);
				if(! this.opened && ! this.disabled) this.showDefault(evt);
			}
		},
		handleKeyPress: function(evt) {
			if(evt.charCode && evt.charCode != 27) {
				if (!this.disabled && this.opened !== 'search') {
					this.showSearch();
				}
			}
		},
		findKeyPrefix: function(pfx) {
			var ks, i, l = pfx.length, sel = this.getValue(), first, foundsel;
			if(! l || ! this.menu || ! (ks = this.menu.getKeys())) return;
			for(i = 0; i < ks.length; i++) {
				if(ks[i] === sel) {
					foundsel = true;
					continue;
				}
				var val = this.menu.getOptionLabel(ks[i]);
				if(val.substring(0, l).toLowerCase() === pfx) {
					if(foundsel) return ks[i];
					if(! isset(first)) first = ks[i];
				}
			}
			return first;
		},
		getTargetOffset: function(target) {
			var btnpos = UI.getEltRegion(target || this.elt);
			var viewp = UI.getViewport();
			var top = - this.menu.getSelectedIndex() * this.menu.getOptionHeight() - this.menu.options_pad;
			top = Math.max(viewp.y - (btnpos ? btnpos.top : 0) + 10, top);
			return {top:top, left:-19, over: true};
		},
		showDefault: function(evt) {
			if((evt && evt.type == 'touchend' && ! this.show_stacked) || (evt && evt.cmdKey) || this.always_search)
				this.showSearch();
			else
				this.showPopup();
		},
		showPopup: function(timeout, target, offset) {
			this.selected = this.getValue();

			if(! this.menu)
				return;
			this.menu.setSelectedKey(this.selected, true);
			this.menu.setCurrentIndex(this.menu.getSelectedIndex(), null, true);
			var input = this;
			this.menu.onchange = function() {
				if(input.popup)
					input.popup.close();
				input.update(this.getSelectedKey(), this.getSelectedValue());
				input.focus();
			}
			this.menu.oncancel = function(refocus) {
				if(input.popup)
					input.popup.close();
				if(refocus) input.focus();
			}

			var menu_elt = this.menu.render();
			if(! menu_elt)
				return;
			this.menu.setup();
			document.body.appendChild(menu_elt);

			var p = new UI.Popup(this.id + '-popup', {
				elt: menu_elt,
				target: target || this.elt, target_offset: offset || this.getTargetOffset(target),
				destroy_on_close: true, close_on_exit: this.close_on_exit,
				max_width: this.max_width, stacked: this.show_stacked});
			p.onshow = function() {
				input.opened = true;
				input.menu.handleShow();
				this.reposition(); // fix position of tall menu popups
			}
			p.onhide = function(reason) {
				var canc = input.opened;
				input.opened = false;
				input.popup = null;
				if(canc)
					input.menu.cancel(reason === 'mouseleave');
				input.menu.destroy();
			}
			p.onfocus = function() {
				input.menu.handleFocus();
			}
			p.onblur = function() {
				input.menu.handleBlur();
			}
			p.onresize = function(pos, viewp) {
				input.menu.handleResize(pos, viewp);
			}
			this.popup = p;
			p.show();
		},
		hidePopup: function() {
			if(this.popup) this.popup.close();
		},
		update: function(key, value, silent) {
			var changed = true;
			if(this.field) {
				if(this.field.value == key)
					changed = false;
				this.field.value = key;
			}
			if(this.dynamic_module && this.module_field) {
				var dmod = isObject(value) ? value[this.dynamic_module] : null;
				this.module_field.value = dmod || '';
			}

			this.updateLabel(key, value);
			if(changed) UI.markFormDirty(this);
			if(! silent)
				UI.callEvent(this, this.onchange, key, value, changed);

			this.validate();
			if(! silent && this.auto_submit && changed)
				UI.submitForm(this.form);
			this.updateConditionals();
		},
		isBlank: function() {
			var v = this.getValue();
			if(! isset(v) || v === '') return true;
		},
		clear: function(silent, force) {
			var v;
			if(! this.isBlank()) {
				v = this.getOptionValue('');
				if(isset(v) || force) {
					this.update('', v, silent);
					return true;
				}
			}
		},
		updateConditionals : function(hide) {
			var key = this.getValue();

			var callback = function(cf) {
				visible = false;
				for (var j = 0; j < cf.depends_on_value.length; j++) {
					if (key == cf.depends_on_value[j]) {
						visible = true;
						break;
					}
				}
				return visible;
			}
			UI.updateConditionals(this, callback, hide);
			this.updateConditionalOptions();
		},
		updateConditionalOptions : function() {
			var conditions, key;
			for (var i = 0; i < UI.conditional_fields.length; i++) {
				var cf = UI.conditional_fields[i];
				if (cf.name == this.name) {
					if (cf.conditional_options && cf.conditional_options.length) {
						var depends_on = cf.depends_on;
						var input = UI.getFormInput(this.form, depends_on);
						if (input) {
							key = '' + input.getValue();
							conditions = cf.conditional_options;
						}
						break;
					}
				}
			}
			if (!conditions)
				return;
			var options = null;
			for (var i = 0; i < conditions.length; i++) {
				if (conditions[i][0].indexOf(key) >= 0) {
					options = conditions[i][1];
					break;
				}
			}

			this.restrictOptions(options);
		},
		restrictOptions : function (options) {
			if (!this.menu.all_options) {
				this.menu.all_options = {
					'keys': this.menu.options.keys,
					'values': this.menu.options.values
				};
			}
			var all_keys = this.menu.all_options.keys;
			var all_values = this.menu.all_options.values;
			var keys = [], values = [];
			if (options) {
				for (var i = 0; i < all_keys.length; i++) {
					if (options.indexOf(all_keys[i]) >= 0 || all_keys[i] == '') {
						keys.push(all_keys[i]);
						values.push(all_values[i]);
					}
				}
			} else {
				keys = all_keys;
				values = all_values;
			}
			this.menu.options.keys = keys;
			this.menu.options.values = values;
			this.menu.renderOptions();
			var val = this.getValue();
			if (keys.indexOf(val) < 0 && val != '' && val)
				this.setValue('');
		},
		updateLabel: function(key, value) {
			var lbl_key = this.label_key || 'name',
				slbl_key = this.slabel_key || 'symbol',
				icon_key = this.icon_key || 'icon',
				icon = '', slabel = '', label = '';
			if(isset(value) && typeof(value) != 'object') {
				label = '' + value;
			} else if(value) {
				label = value[lbl_key];
				slabel = value[slbl_key];
				icon = value[icon_key];
			}
			if(this.icon) icon = this.icon;
			if(icon_key == 'key') icon = ''+key;
			var slb_tg = $(this.id + '-short');
			if(slb_tg)
				UI.setElementText(slb_tg, slabel);
			var lbl_tg = this.label_elt || $(this.id + '-label');
			if(lbl_tg)
				UI.setElementText(lbl_tg, label);
			var icon_outer = this.icon_outer || QueryFirst(this.elt).find('.input-field-icon').get(0);
			if(this.color_icon) {
				icon = value.color;
				if(icon) {
					UI.clearChildNodes(icon_outer);
					MakeElt('div.input-icon.input-color-icon').css('background-color', icon)
						.appendTo(icon_outer);
				}
				UI.setDisplayState(icon_outer, !! icon);
			}
			else if(! this.static_icon) {
				var ico = this.icon_elt || $(this.id + '-icon');
				if(ico && icon) {
					if(icon.indexOf('-') < 0) icon = 'theme-icon module-' + icon;
					ico.className = 'input-icon active '+icon;
				}
				UI.setDisplayState(icon_outer, !! icon);
			}
			this.updateDisplay();
		},
		validate: function() {
			this.invalid = false;
			var value = this.getValue();
			if(! isset(value)) value = '';
			var have_val = trim(value).length;
			if(this.checkAltValue()) {
				// ok
			} else if(this.required && ! have_val) {
				this.invalid = true;
				this.invalidMsg = 'required';
			} else if(have_val && ! this.update_from_popup && this.require_existing) {
				// check value is in options
				if(this.menu && this.menu.getKeyIndex(value) == -1) {
					this.invalid = true;
					this.invalidMsg = '';
				}
			}
			this.updateDisplay();
			return ! this.invalid;
		},
		updateDisplay: function() {
			UI.setDisplayState(this.elt, ! this.checkAltValue());
			UI.addRemoveClass(this.elt, 'disabled', !! (this.disabled && this.disabled !== 'readonly'));
			UI.addRemoveClass(this.elt, 'readonly', !! (this.disabled === 'readonly'));
			UI.addRemoveClass(this.elt, 'invalid', !! this.invalid);
			UI.addRemoveClass(this.elt, 'active', !! (this.focused && !this.opened));
			UI.addRemoveClass(this.elt, 'opened', !! (this.opened && ! this.popup));
			UI.addRemoveClass(this.elt, 'opened-above', !! (this.opened && ! this.popup && this.show_search_above));
			var ico = this.icon_elt || $(this.id + '-icon'),
				lbl = this.label_elt || $(this.id + '-label');
			if(ico && ! this.no_label) ico.style.display = (trim(ico.className) == 'input-icon active') ? 'none' : 'inline-block';
			this.updateTitleText();
			var blank = this.menu && this.menu.getBlankKey() || '';
			UI.addRemoveClass(lbl, 'text-placeholder', this.mark_blank && this.getValue() === blank && !this.focused);
		},
		updateTitleText: function() {
			var opt = this.getOptionValue(),
				lbl_key = this.label_key || 'name',
				title_key = this.title_key || 'title',
				label;
			if(isset(opt) && typeof(opt) != 'object')
				label = '' + opt;
			else if(opt)
				label = (title_key && opt[title_key]) ? opt[title_key] : opt[lbl_key];
			else
				label = '';
			if(this.elt) this.elt.title = label;
		},
		render: function() {
			if(! this.field) {
				this.field = createElement2('input', {type: 'hidden', name: this.name, value: this.init_value});
			}
			if(! this.elt) {
				var lbl_style = '';
				if(this.width) lbl_style += 'width: '+UI.formatSize(this.width)+';';
				if(this.max_width) lbl_style += 'max_width: '+UI.formatSize(this.max_width)+';';
				var cls = this.className || ('input-field input-field-group rbullet ' + (this.ext_class || ''));
				if(this.flat) cls += ' flat';
				if(this.block) cls += ' input-block';
				var lblCls = 'input-label';
				if(! this.wrap) lblCls += ' single';
				this.icon_outer = createElement2('div', {className: 'input-label input-field-icon'},
					(this.icon_elt = createElement2('div', {id: this.id + '-icon', className: 'input-icon'})));
				this.elt = createElement2('div', {className: cls, id: this.id, tabIndex: this.tabIndex || 0},
					[this.icon_outer,
					this.label_elt = createElement2('div', {className: lblCls, id: this.id + '-label'})]);
				this.setup();
			}
			var key = this.getValue();
			this.updateLabel(key, this.getOptionValue() || '');
			return this.elt;
		},
		destroy: function() {
			if(this.popup) this.popup.destroy();
			this.popup = null;
			if(this.menu) this.menu.destroy();
			this.menu = null;
			UI.removeNode(this.elt, true);
			this.elt = this.outer = null;
			this.field = null;
		},
		setReadOnly: function(flag) {
			this.setDisabled(get_default(flag, true) ? 'readonly' : flag);
		},
		setDisabled: function(disabled) {
			if(! isset(disabled)) disabled = 1;
			this.disabled = disabled;
			if(this.elt) this.elt.disabled = disabled;
			this.updateDisplay();
			if(this.disabled && this.disabled !== 'readonly') {
				if(isset(this.elt.tabIndex)) {
					this.tabIndex = this.elt.tabIndex;
					this.elt.removeAttribute('tabIndex');
				}
			} else if(! isset(this.elt.tabIndex)) {
				this.elt.tabIndex = this.tabIndex || 0;
			}
		},
		initSearcher: function() {
			var self = this;
			this.searcher = new UI.SearchSelect(this.id + '-search',
				{target: this.elt,
				 target_offset: {top: -1, left: 0, above: this.show_search_above},
				 show_above: this.show_search_above,
				 form: this.form,
				 show_checks: true,
				 //allow_clear: true
				 disable_recent: true,
				 search_blank: true,
				 mark_blank: this.mark_blank,
				 matching_only: true});
			this.searcher.onchange = function(k, v, mod, keepOpen) {
				self.update(k, v);
				self.focus();
			}
			this.searcher.onshow = function() {
				self.opened = 'search';
				self.updateDisplay();
			}
			this.searcher.onhide = function() {
				self.opened = false;
				self.updateDisplay();
			}
			this.searcher.oncancel = function() {
				self.focus();
			}
			this.searcher.setup();
		},
		showSearch: function(query) {
			if(! this.searcher) {
				this.initSearcher();
			}
			this.searcher.setOptions(this.menu.options);
			this.searcher.render();
			var w = UI.getEltRegion(this.elt);
			if(w)
				this.searcher.setWidth(Math.max(w.width - 24, this.search_width || 300) + 'px');
			var k = this.getValue();
			this.searcher.setKey(k);
			this.searcher.setQuery(query || '');
			this.searcher.showDefault();
		}
	};


	UI.QuickSelect = function(params) {
		var id = params.id || 'quicksel-'+(++ UI.QuickSelect.uniq_idx);
		UI.SelectInput.call(this, id, params);
	}
	UI.QuickSelect.uniq_idx = 0;
	extendClass(UI.QuickSelect, UI.SelectInput, {
		init: function(key, onchange, oncancel) {
			this.key = key;
			this.onchange = onchange;
			this.oncancel = oncancel;
			this.set_up = false;
			this.setup();
		},
		initSource: function(source, key, onchange, oncancel) {
			this.elt = source;
			this.init(key, onchange, oncancel);
		},
		getValue: function() {
			return this.key;
		},
		update: function(key, value, silent) {
			this.key = key;
			if(isObject(value)) {
				var lkey = this.label_key || this.menu.options.display_name || 'name';
				value = value[lkey];
			}
            if (this.elt)
                UI.setElementText(this.elt, value);
			UI.SelectInput.prototype.update.call(this, key, value, silent);
		}
		// inherits showPopup(timeout, target, offset)
	});


	UI.MenuSource = function(id, params) {
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id || ('menu-source-' + (++ UI.MenuSource.menu_index));
		} else
			this.id = id;
		this.inline = false;
		this.menu_id = this.id + '-popup';
		this.show_checks = false;
		this.hover_show = false;
		this.delay = 0;
		this.icon_key = null;
		this.onchange = null;
		this.now_selecting = false;
		this.className = 'card-outer popup-default panel-outer panel-border panel-default panel-round-bottom active';
		this.target_active_class = 'active opened';
		if(params) extendObject(this, params);
		if(this.hover_show && isIPhone)
			this.hover_show = false;
	}
	UI.MenuSource.menu_index = 0;
	UI.MenuSource.prototype = {
		setup: function() {
			var self = this;
			if(this.set_up)
				return;
			if(! this.elt)
				this.elt = $(this.id);
			if(! this.elt)
				return;
			UI.addEventListener(this.elt, 'selectstart', UI.stopEvent);
			UI.addEventListener(this.elt, 'touchstart', function(evt) { if(this.now_selecting) UI.stopEvent(evt); else UI.preventDefault(evt); }, this);
			UI.addEventListener(this.elt, 'touchend', this.handleClick, this);
			if(this.hover_show) {
				UI.addEventListener(this.elt, 'mouseenter', function(evt) {
						if(this.now_selecting)
							return false;
						var p = UI.PopupManager.last_opened;
						if(p && ! p.modal && ! p.close_on_exit)
							return; // don't trigger popup when a semi-persistent one is open, or it will be dismissed
						this.showPopup(this.delay);
					}, this);
				UI.addEventListener(this.elt, 'mouseleave', function(evt) {
						this.cancelShow();
				}, this);
			} else {
				UI.addEventListener(this.elt, 'mousedown', function(evt) {
						if((evt.which || evt.button) != 1) return;
						UI.stopPropagation(evt);
					}, this);
				UI.addEventListener(this.elt, 'click', this.handleClick, this);
			}
			UI.addEventListener(this.elt, 'keydown', this.handleKeyDown, this);
			if(! this.dynamic_options)
				this.initMenu();
			this.set_up = true;
		},
		initMenu: function() {
			var options = this.options, selected = null, self = this;
			if(! this.menu || this.dynamic_options) {
				if(this.dynamic_options) {
					options = this.dynamic_options(this);
					if('selected' in options)
						this.init_value = options.selected;
				}
				if(options instanceof UI.SelectList)
					this.menu = options;
				else {
					this.menu = new UI.SelectList(this.menu_id,
						{options: options, no_scroll: true, popup: true,
							show_checks: this.show_checks,
							icon_key: this.icon_key, width: this.width,
							className: this.className,
							ext_class: this.ext_class});
				}
			}
			if(this.menu) {
				if(this.elt.tabIndex)
					this.menu.tabIndex = this.elt.tabIndex;
				else if(! this.menu.tabIndex)
					this.menu.tabIndex = 1;
				this.menu.touch_opened = this.touch_opened;
				this.menu.onprevnext = function(evt, dir) {
					self.handlePrevNext(evt, dir);
				}
				this.menu.onchange = function(oldSel, evt) {
					self.handleSelect(this.getSelectedKey(), this.getSelectedValue(), evt);
				}
				this.menu.oncancel = function() {
					self.handleCancel();
				}
			}
		},
		handleKeyDown: function(evt) {
			if(evt.keyCode === 32)
				this.handleClick(evt);
		},
		handleClick: function(evt) {
			if(evt.type == 'click' && (evt.which || evt.button) != 1) { return null; }
			var target = evt.target;
			if(target && target.name == 'check-all') { return null; }
			if(target && target != this.elt && this.only_this_target) { return null; }
			this.touch_opened = (evt.type == 'touchend');
			this.mouse_pos = {x: evt.clientX || evt.x, y: evt.clientY || evt.y};
			if(this.now_selecting)
				this.closePopup();
			else
				this.showPopup();
			UI.stopEvent(evt);
		},
		handlePrevNext: function(evt, dir) {
			var pos = UI.getMenuGroupPos(this.form, this.id), other;
			if(pos) other = dir ? pos.next : pos.prev;
			if(other) other.showPopup();
		},
		focus: function() {
			if(this.elt) this.elt.focus();
		},
		getSelected: function() {
			return this.init_value || null;
		},
		cancelShow: function() {
			if(this.popup && this.now_selecting && this.popup.cancelShowTimer())
				this.now_selecting = false;
		},
		showPopup: function(timeout) {
			this.selected = this.getSelected();
			if(this.popup)
				this.popup.close();

			this.initMenu();
			if(! this.menu)
				return;
			this.menu.setSelectedKey(this.selected, true);
			this.menu.setCurrentIndex(this.menu.getSelectedIndex(), null, true);
			var menu_elt = this.menu.render();
			if(! menu_elt)
				return;
			this.menu.setup();
			document.body.appendChild(menu_elt);

			this.popup = this.createPopup(menu_elt);
			this.now_selecting = true;
			this.popup.show(timeout);
		},
		getPopupOffset: function() {
			var ret = this.target_offset;
			if(! ret && this.elt && UI.hasClass(this.elt, 'menu-source')) {
				ret = {top: -1};
			}
			return ret;
		},
		createPopup: function(menu_elt) {
			// if(this.elt && UI.hasClass(this.elt, 'menu-source')) act_cls += ' flat-bottom';
			var popup = new UI.Popup(menu_elt, {target: this.elt,
					target_offset: this.getPopupOffset(), mouse_pos: this.mouse_pos,
					customDisplayTransform: isIPhone ? null : 'scaleY(0.4)',
					target_active_class: this.target_active_class, destroy_on_close: false,
				stacked: this.stacked, inline: this.inline, zIndex: this.zIndex
			});
			popup.onshow = this.handleShowPopup.bind(this);
			popup.onclose = popup.oncancelshow = this.handleClosePopup.bind(this);
			return popup;
		},
		hasKey: function(key) {
			return this.menu && this.menu.hasKey(key);
		},
		performActionKey: function(key) {
			var val = this.menu ? this.menu.getValue(key) : null;
			if(val) return this.performAction(key, val);
		},
		performAction: function(key, action, evt) {
			if(! action) return;
			if(action.pre_check && UI.callEvent(this, action.pre_check, key, action, evt) === false)
				return false;
			if(action.confirm && ! confirm(action.confirm))
				return false;
			if(action.pre_perform)
				UI.callEvent(this, action.pre_perform, key, action, evt);

			if(action.perform)
				UI.callEvent(this, action.perform, key, action, evt);
			else if(action.url) {
				App.util.navigateTo(action.url, evt, {async: get_default(action.async, true)});
				return false;
			}

			if(action.post_perform)
				UI.callEvent(this, action.post_perform, key, action);
			return true;
		},
		handleSelect: function(key, value, evt) {
			this.last_selected = key;
			this.now_selecting = false;
			this.closePopup();
			if(! value)
				return;
			if(UI.callEvent(this, this.onchange, key, value) === false)
				return;
			this.performAction(key, value, evt);
			this.now_selecting = false;
		},
		handleCancel: function() {
			this.now_selecting = false;
			this.closePopup();
			UI.callEvent(this, this.oncancel);
		},
		closePopup: function() {
			if(this.popup)
				this.popup.close();
		},
		handleShowPopup: function() {
			if(this.menu)
				this.menu.handleShow();
		},
		handleClosePopup: function() {
			if(this.menu) {
				if(this.now_selecting)
					this.menu.cancel();
				this.menu.destroy();
			}
		},
		render: function() {
			// draw select input according to params
		},
		setDisabled: function(disabled) {
			if(! isset(disabled)) disabled = 1;
			this.disabled = !! disabled;
			if(this.elt) this.elt.disabled = disabled;
		},
		destroy: function() {
			if(this.popup) this.popup.destroy();
			this.popup = null;
			if(this.menu) this.menu.destroy();
			this.menu = null;
			UI.removeNode(this.elt, true);
			this.elt = null;
			UI.callEvent(this, this.ondestroy);
		}
	};


	UI.RefInput = function(id, params) {
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		this.allow_custom = false;
		this.allow_rename = false;
		this.allow_clear = true;
		this.track_view = true;
		this.name_column = '_display';
		this.invalid = false;
		this.quick_create = false;
		if(params) extendObject(this, params);
	}
	UI.RefInput.prototype = {
		setup: function() {
			if(this.set_up)
				return;
			if(! this.outer)
				this.outer = $(this.id);
			if(! this.outer) return;
			if(! this.elt) {
				this.elt = (
					Wrap(this.outer).findFirst('.ref-input').get(0) ||
				 	Wrap(this.outer).findFirst('.input-field').get(0)
				 );
			}
			if(! this.field) {
				if(this.name && this.form)
					this.field = this.form.elements[this.name];
				else if(this.field_id)
					this.field = $(this.field_id);
			}
			if(! this.key_field) {
				if(this.key_name && this.form)
					this.key_field = this.form.elements[this.key_name];
				else if(this.key_id)
					this.key_field = $(this.key_id);
			}
			if(! this.module_field) {
				if(this.form && this.module_field_name)
					this.module_field = this.form.elements[this.module_field_name];
				else if(this.module_field_id)
					this.module_field = $(this.module_field_id);
			}
			if(! this.elt)
				return;
			var elts = this.elt.getElementsByClassName('ref-label');
			if(elts && elts.length == 1)
				this.label_elt = elts[0];
			elts = this.elt.getElementsByClassName('ref-icon');
			if(elts && elts.length == 1)
				this.icon_elt = elts[0];
			if(! this.altInput && this.alt_name) {
				this.altInput = UI.getFormInput(this.form, this.alt_name);
			}
			this.elt.draggable = true;
			if(this.elt.parentNode && UI.hasClass(this.elt.parentNode, 'input-wrapper'))
				this.outer = this.elt.parentNode;
			this.set_up = true;
			this.postSetup();
		},
		postSetup: function() {
			if(this.module_field)
				this.module = this.module_field.value;
			if(this.key_field) this.orig_key = this.key_field.value;
			this.updateLabel();
			this.updateDisplay();
			this.addHooks();
		},
		addExtraReturnFields: function(fs) {
			if(! this.extra_fields)
				this.extra_fields = {};
			for(var k in fs) {
				this.extra_fields[k] = fs[k];
			}
		},
		getExtraReturnFields: function() {
			return this.extra_fields;
		},
		getReturnFields: function() {
			var fs = {id: 'id', _display: '_display'}, ext = this.extra_fields;
			if(ext)
				for(var k in ext) fs[k] = ext[k];
			return fs;
		},
		getModule: function() {
			if(this.module_field)
				return this.module_field.value;
			return this.module;
		},
		setModule: function(mod) {
			if(this.module_field)
				this.module_field.value = mod;
			this.moduleUpdated(mod);
		},
		moduleUpdated: function(mod, silent) {
			if(this.module != mod) {
				this.module = mod;
				UI.callEvent(this, this.onmoduleupdate, mod);
				this.update(null, null, silent);
			}
		},
		addHooks: function() {
			if(this.events) return;
			var input = this;
			var source = new UI.Draggable(this.elt, {
				id: this.id,
				sourceType: 'SelectListItem',
				onStart : this.handleDragStart.bind(this)
			});
			var target = new UI.DragTarget(this.elt, {
				acceptedSourceTypes: ['SelectListItem'],
				onMove : this.handleDragOver.bind(this),
				onEnter : this.handleDragEnter.bind(this),
				onLeave : this.handleDragLeave.bind(this),
				onDrop : this.handleDrop.bind(this),
				onAcceptSource : this.onAcceptSource.bind(this),
				checkAccept : this.checkAccept.bind(this)
			});
			this.events = UI.addEventListeners(this.elt, {
				keydown: this.handleKeyDown,
				keyup: this.handleKeyUp,
				keypress: this.handleKeyPress,
				mouseenter: this.handleMouseEnter,
				mouseleave: this.handleMouseLeave,
				mousedown: this.handleMouseDown,
				click: this.handleClick,
				focus: this.handleFocus,
				blur: this.handleBlur/*,
				copy: this.handleCopy,
				paste: this.handlePaste*/}, this);
			if(this.altInput) {
				this.altInput.onchange = function() { input.update(); }
			}
		},
		getDragRef: function() {
			var url = '', mod = this.module, key = this.getKey(), val = this.getValue(), ret = {};
			if(mod && key) url = '' + App.session.site_url + 'index.php?' + encodeQueryString({module: mod, action: 'DetailView', record: key});
			ret = {
				'text/plain': url,
				'text/uri-list': url,
				'text/html': url ? MakeElt('a', {href: url}, val).html() : '',
				'url': url,
				'1crm.ref-module': mod,
				'1crm.ref-value': key,
				'1crm.ref-label': val,
				'1crm.ref-id': this.id
			};
			return ret;
		},
		loadDragRef: function(evt) {
			var url, parts, ret, id, p, src;
			if(! evt) return;
			if(evt.type === 'paste')
				src = evt.clipboardData;
			else
				src = evt.dataTransfer;
			if(! src) return;
			if( (id = src.getData('1crm.ref-id')) ) {
				ret = {
					id: id,
					module: src.getData('1crm.ref-module'),
					record: src.getData('1crm.ref-value'),
					value: src.getData('1crm.ref-label')
				};
			} else {
				url = src.getData('text/uri-list') || src.getData('url');
				if(url && (p = url.indexOf('?')) >= 0) {
					parts = parseQueryString(url.substring(p+1));
					if(parts && parts.action == 'DetailView') {
						ret = {module: parts.module, record: parts.record};
					}
				}
			}
			return ret;
		},
		handleDragStart: function(evt) {
			var info = this.getDragRef();
			for(var k in info)
				evt.dataTransfer.setData(k, info[k]);
			evt.stopPropagation();
		},
		checkAccept: function(evt) {
			return true;
		},
		handleDragOver: function(evt) {
			if(this.disabled) return;
			evt.dataTransfer.dropEffect = 'copy';
		},
		handleDragEnter: function(evt) {
			if(this.disabled || evt.target !== evt.currentTarget) return;
			this.dragover = true;
			this.updateDisplay();
		},
		handleDragLeave: function(evt) {
			if(this.disabled || evt.target !== evt.currentTarget) return;
			this.dragover = false;
			this.updateDisplay();
		},

		onAcceptSource:  function(evt, data, source) {
			if (!source) return true;
			if (source.sourceType == 'SelectListItem')
				return source.id != this.id;
			return null;
		},

		handleDrop: function(evt) {
			if(this.disabled) return;
			evt.stopPropagation();
			evt.preventDefault();
			this.dragover = false;
			var ref = this.loadDragRef(evt);
			if(! ref || (ref.id && ref.id == this.id)) return;
			this.updateWithLookup(ref.module, ref.record);
		},
		handleCopy: function(evt) {
			var info = this.getDragRef();
			for(var k in info)
				evt.clipboardData.setData(k, info[k]);
			UI.preventDefault(evt);
		},
		handlePaste: function(evt) {
			if(this.disabled) return;
			UI.stopEvent(evt);
			var ref = this.loadDragRef(evt);
			if(ref.id && ref.id == this.id) return;
			this.updateWithLookup(ref.module, ref.record);
		},
		handleKeyDown: function(evt) {
			var nav;
			if(evt.keyCode == 38) nav = -1;
			else if(evt.keyCode == 40) nav = 1;
			else nav = 0;
			if(nav) {
				if(this.searcher && this.searcher.visible)
					this.searcher.adjustCurrentIndex(nav, null, evt);
				//else if(! this.disabled)
				//	this.showSearch();
				else
					return;
				UI.stopEvent(evt);
			} else if(evt.keyCode == 13) {
				if(this.searcher && this.searcher.visible) {
					if(! this.searcher.acceptCurrent(evt))
						this.hideSearch();
				} else if(! this.disabled)
					this.showSearch();
				else
					return;
				UI.stopEvent(evt);
			} else if(evt.keyCode == 8 || evt.keyCode == 46) {
				// backspace/delete
				if(! this.disabled) this.clear();
			} else if(evt.keyCode == 27) {
				// esc
				if(this.opened) {
					this.hideSearch(true);
					this.focus();
					UI.stopEvent(evt);
				} else if(! this.disabled) {
					if(this.clear()) UI.stopEvent(evt);
				}
			} else if(evt.keyCode == 32) {
				if(this.focused && ! this.disabled) {
					this.triggerSearch();
					UI.stopEvent(evt);
				}
			}
		},
		handleKeyUp: function(evt) {
			if(evt.keyCode && evt.charCode && evt.keyCode != 27 && evt.keyCode != 13 && evt.keyCode != 9 && evt.keyCode != 16)
				this.updateSearch();
		},
		handleKeyPress: function(evt) {
			if(evt.charCode && evt.charCode != 27) {
				if (!this.disabled && this.opened !== 'search') {
					this.showSearch();
				}
			}
		},
		triggerSearch: function() {
			this.cancelShowSearch();
			this.savePrevValue();
			this.showSearch(); //Delayed();
		},
		handleMouseDown: function(evt) {
			if(! this.disabled && evt.currentTarget === this.elt)
				evt.stopPropagation();
		},
		handleClick: function(evt) {
			if(evt.which == 1) {
				if(this.opened)
					this.hideSearch();
				else if(! this.disabled)
					this.triggerSearch();
				UI.stopEvent(evt);
			}
		},
		handleMouseEnter: function(evt) {
			this.hovered = true;
			if(! isIPhone) this.updateDisplay(); // screws up input focus event in iOS
		},
		handleMouseLeave: function(evt) {
			this.hovered = false;
			if(! isIPhone) this.updateDisplay();
		},
		searchMouseUp: function(evt) {
			//if(this.focused === 'search')
			//	this.field.focus();
		},
		handleFocus: function(evt) {
			this.focused = true;
			UI.callEvent(this, this.onfocus, evt);
			this.updateDisplay();
		},
		handleBlur: function(evt) {
			if(this.focused) {
				this.focused = false;
				//this.hideSearchDelayed(100);
				UI.callEvent(this, this.onblur, evt);
				this.updateDisplay();
			}
		},
		savePrevValue: function() {
			this.prev_key = this.getKey();
			this.prev_value = this.getValue();
		},
		updateLabel: function() {
			var lbl, icon;
			if(this.label_elt) {
				if(! this.isBlank())
					lbl = this.getValue();
				else
					lbl = this.placeholder || app_string('LBL_NONE');
				UI.setElementText(this.label_elt, lbl);
			}
			if(this.icon_elt) {
				icon = this.icon || (this.module == 'Users' ? 'bean-User' : 'module-'+this.module);
				this.icon_elt.className = 'ref-icon input-icon theme-icon '+icon;
				if(this.module_field)
					UI.setDisplayState(this.icon_elt.parentNode, !! this.getKey());
			}
		},
		updateDisplay: function() {
			UI.setDisplayState(this.elt, ! this.checkAltValue());
			if(this.checkAltValue()) {
				this.hideSearch();
				return;
			}
			UI.addRemoveClass(this.label_elt, 'text-placeholder', this.isBlank() && ! this.focused && ! this.opened);
			UI.addRemoveClass(this.elt, 'invalid', this.invalid);
			UI.addRemoveClass(this.elt, 'active', !! this.dragover);
			UI.addRemoveClass(this.elt, 'opened', !! this.opened);
		},
		setDisabled: function(disabled) {
			if(! isset(disabled)) disabled = 1;
			this.disabled = (typeof disabled === 'string' ? disabled : !! disabled);
			UI.addRemoveClass(this.elt, 'disabled', !! (this.disabled && this.disabled !== 'readonly'));
			UI.addRemoveClass(this.elt, 'readonly', !! (this.disabled === 'readonly'));
			if(this.disabled && this.disabled !== 'readonly') {
				if(isset(this.elt.tabIndex)) {
					this.tabIndex = this.elt.tabIndex;
					this.elt.removeAttribute('tabIndex');
				}
			} else if(! isset(this.elt.tabIndex)) {
				this.elt.tabIndex = this.tabIndex || 0;
			}
			if(this.disabled) this.hideSearch();
		},
		setReadOnly: function(flag) {
			this.setDisabled(get_default(flag, true) ? 'readonly' : flag);
		},
		focus: function() {
			if(this.elt)
				this.elt.focus();
		},
		cancelShowSearch: function() {
			if(this.searchTimeout) {
				clearTimeout(this.searchTimeout);
				this.searchTimeout = null;
				return true;
			}
		},
		cancelHideSearch: function() {
			if(this.hideSearchTimeout) {
				clearTimeout(this.hideSearchTimeout);
				this.hideSearchTimeout = null;
				return true;
			}
		},
		render: function() {
			var ret;
			if(! this.rendered) {
				if(! this.key_field) {
					this.key_field = createElement2('input', {type: 'hidden', id: this.key_id || '', name: this.key_name || '', value: this.init_key});
				}
				if(! this.field) {
					this.field = createElement2('input', {type: 'hidden', id: this.field_id || '', name: this.name || '', value: this.init_value});
				}
				if(! this.elt) {
					var cls = 'input-field input-field-group rbullet ref-input';
					var style = {};
					if(this.block) cls += ' input-block';
					if(this.flat) cls += ' flat';
					if(this.ext_class) cls += ' ' + this.ext_class;
					else style.width = this.width || '20em';
					this.elt = createElement2('div', {className: cls, id: this.id, style: style, tabIndex: this.tabIndex || 0});
					if(this.disabled)
						this.elt.className += ' disabled';
				}
				var icon_outer = createElement2('div', {className: 'input-label input-field-icon'});
				if(! this.icon_elt) {
					this.icon_elt = createElement2('div', {className: 'ref-icon'});
				}
				if(! this.label_elt) {
					this.label_elt = createElement2('div', {className: 'input-label ref-label'});
				}
				icon_outer.appendChild(this.icon_elt);
				this.elt.appendChild(icon_outer);
				this.elt.appendChild(this.label_elt);

				this.outer = MakeElt('div.input-wrapper' + (this.block ? '.input-block' : ''),
					this.key_field,
					this.module_field,
					this.field,
					this.elt).get();

				this.rendered = true;
				this.savePrevValue();
				this.postSetup();
			}
			return this.outer || this.elt;
		},
		getKey: function() {
			if(this.key_field)
				return this.key_field.value;
		},
		getValue: function() {
			if(this.field)
				return this.field.value;
		},
		getValueDisplay: function(value) {
			if(isset(value)) {
				if(isString(value)) return value;
				return value[this.name_column];
			}
			return '';
		},
		isBlank: function() {
			if(this.allow_custom) return ! trim(this.getValue()).length;
			return ! trim(this.getKey()).length;
		},
		clear: function(refocus) {
			if(refocus) this.focus();
			if(! this.isBlank()) {
				this.userUpdate(null, null);
				return true;
			}
		},
		update: function(key, value, silent, module, passthru) {
			key = get_default(key, '');
			if(this.union_module && (! isObject(value) || ! value._module))
				key = '';
			if(this.key_field)
				this.key_field.value = key;
			if(this.field && isdef(value)) {
				this.field.value = this.getValueDisplay(value);
			}
			if(this.union_module || module) {
				var mod = (key ? value._module : '') || module;
				if(this.module_field) {
					this.module = mod;
					this.module_field.value = mod;
					UI.callEvent(this, this.onmoduleupdate, mod);
				}
				//if(this.icon_elt) this.icon_elt.className = 'input-icon input-name-icon theme-icon ' + (mod ? 'module-' + mod : '');
			}
			this.savePrevValue();
			this.hideSearch();
			this.updateLabel();
			if(key !== this.orig_key) UI.markFormDirty(this);
			if(! silent)
				UI.callEvent(this, this.onchange, key, value, passthru || this.getPassthru());
			this.validate();
		},
		userUpdate: function(key, value, nofocus, mod, passthru) {
			if(! this.focused && ! nofocus)
				this.focus();
			this.update(key, value, false, mod, passthru);
			var do_submit = (this.auto_submit && (key || this.submit_empty));
		//	if(this.focused && ! do_submit)
		//		this.showSearchDelayed(1500);
			if(this.track_view)
				this.doTrackView();
			if(do_submit)
				UI.submitForm(this.form);
		},
		doTrackView: function() {
			var mod = this.getModule(), rec = this.getKey();
			if(mod && rec) {
				var req = new App.conn.JSONRequest('add_recent', {silent: true, global: true}, {module: mod, record: rec, source: 'ref'});
				req.fetch();
			}
		},
		rename: function(value, silent) {
			if(this.field && isdef(value))
				this.field.value = this.getValueDisplay(value);
			this.savePrevValue();
			this.hideSearch();
			this.updateLabel();
			if(! silent)
				UI.callEvent(this, this.onrename, value, this.getPassthru());
			this.validate();
		},
		userRename: function(value, nofocus) {
			if(! this.focused && ! nofocus)
				this.focus();
			this.rename(value);
			if(this.focused)
				this.showSearchDelayed(1500);
		},
		updateSearch: function() {
			this.showSearch();
		},
		showSearchDelayed: function(delay) {
			this.cancelHideSearch();
			var input = this;
			this.searchTimeout = setTimeout(function() { input.showSearch(); }, delay || 1000);
		},
		initSearcher: function() {
			var mod = this.union_module || this.getModule(), self = this;
			this.searcher = new UI.SearchSelect(this.id + '-search',
				{module: mod,
				 union_module: this.union_module,
				 dynamic_module: !! this.module_field,
				 modules: this.modules,
				 options: this.options,
				 target: this.elt, target_offset: {top: -1, left: 0},
				 extra_fields: this.getExtraReturnFields(),
				 filters: this.getFilters(),
				 passthru: this.getPassthru(),
				 popup_layout: this.popup_layout,
				 popup_callback: this.popup_callback,
				 popup_return_method: this.popup_return_method,
				 popup_return_multiple: this.popup_return_multiple,
				 form: this.form,
				 show_checks: true,
				 allow_popup: true,
				 allow_clear: this.allow_clear,
				 allow_custom: this.allow_custom,
				 allow_rename: this.allow_rename,
				 quick_create: this.quick_create,
				 min_width: this.popup_width});
			if(this.union_module) {
				this.searcher.icon_key = 'icon';
			}
			this.searcher.onchange = function(k, v, mod) {
				self.focus();
				self.userUpdate(k, v, true, mod);
			}
			this.searcher.onrename = function(v) {
				self.userRename(v);
			}
			this.searcher.onmouseup = function(evt) {
				self.searchMouseUp(evt);
			}
			this.searcher.onquickcreate = function(evt, query) {
				self.openQuickCreate(query);
			}
			this.searcher.onshow = function() {
				self.opened = true;
				self.updateDisplay();
			}
			this.searcher.onhide = function() {
				self.opened = false;
				self.updateDisplay();
			}
			this.searcher.oncancel = function() {
				self.focus();
			}
			this.searcher.setup();
		},
		getSearchQuery: function() {
			return this.field ? this.field.value : null;
		},
		showSearch: function() {
			this.cancelShowSearch();
			this.cancelHideSearch();
			var mod = this.union_module || this.getModule(), p = UI.PopupManager.currentModal();
			if(p && ! UI.isAncestor(p.elt, this.elt))
				return;
			//if(blank(mod))
			//	return;
			if(! this.searcher) {
				this.initSearcher();
			} else {
				this.searcher.setModule(mod, true);
				this.searcher.setFilters(this.getFilters());
			}
			this.searcher.render();
			var w = UI.getEltRegion(this.elt);
			if(w)
				this.searcher.setWidth(Math.max(w.width - 24, this.search_width || 300) + 'px');
			var k = this.getKey();
			this.searcher.setKey(k);
			if(this.union_module) this.searcher.setDetailModule(this.getModule());
			this.searcher.showDefault();
		},
		hideSearchDelayed: function(delay) {
			this.cancelShowSearch();
			this.cancelHideSearch();
			var input = this;
			this.hideSearchTimeout = setTimeout(function() { input.hideSearch(); }, delay || 100);
		},
		hideSearch: function(refocus) {
			this.cancelShowSearch();
			this.cancelHideSearch();
			if(this.searcher) {
				var v = this.getValue();
				if(v != this.prev_value) {
					if((v !== '' && ! this.auto_update) || (v === '' && ! this.allow_clear))
						this.update(this.prev_key, this.prev_value, true);
					else
						this.userUpdate(null, v, ! refocus);
				}
				this.searcher.close();
			}
			if(this.focused && refocus)
				this.elt.focus();
		},
		validate: function() {
			this.invalid = false;
			if(this.required && this.isBlank() && ! this.checkAltValue()) {
				this.invalid = true;
				this.invalidMsg = 'required';
			}
			else if(this.customValidate)
				this.customValidate();
			this.updateDisplay();
			return ! this.invalid;
		},
		checkAltValue: function() {
			var v;
			if(this.altInput && (v = this.altInput.getValue())) {
				return v;
			}
		},
		addFilter: function(filter) {
			if(! this.add_filters) this.add_filters = [];
			this.add_filters.push(filter);
		},
		getFilters: function() {
			if(Array.isArray(this.add_filters)) {
				var params = {}, c = 0, i, fp, inp, k, v;
				for(i = 0; i < this.add_filters.length; i++) {
					fp = this.add_filters[i];
					if(! fp.param) continue;
					if(fp.field_name) {
						inp = UI.getFormInput(this.form, fp.field_name);
						if(inp) {
							if(inp instanceof UI.RefInput) {
								k = inp.getKey();
								if(k === '') k = null;
							} else
								k = null;
							v = inp.getValue();
							if(isset(k))
								// FIXME - only key should be necessary, but filter form doesn't currently populate the name
								params[fp.id_param || fp.param+'_id'] = inp.getKey();
							if(isset(v) && v !== '')
								params[fp.param] = v;
							else if(! isset(k))
								continue;
							c ++;
						} else if(this.form.elements[fp.field_name]) {
							v = this.form.elements[fp.field_name].value;
							if(! isset(v) || v === '')
								continue;
							params[fp.param] = v;
							c ++;
						} else
							continue;
					}
					else if(isset(fp.value)) {
						params[fp.param] = fp.value;
						c ++;
					}
				}
				if(c) return params;
			}
		},
		getPassthru: function() {
			var p = this.popup_passthru ? deep_clone(this.popup_passthru) : {};
			p.ui_input_name = this.name || this.id;
			return p;
		},
		showPopup: function() {
			var mod = this.union_module || this.getModule(),
				request_data = {
					form_name: this.form.getAttribute('id') || this.form.getAttribute('name'),
					call_back_function: this.popup_callback || 'ui_popup_return',
					passthru_data: this.getPassthru(),
					field_to_name_array: {id: 'id', _display: '_display'}
				},
				params = {module: mod, layout: this.popup_layout, request_data: request_data, inline: true};
			if(! mod)
				return;
			if(this.union_module)
				request_data.field_to_name_array['_module'] = 'query_module';
			var add_fs = this.getExtraReturnFields();
			if(add_fs) {
				for(i in add_fs) {
					request_data.field_to_name_array[i] = add_fs[i];
				}
			}
			params.filter = this.getFilters();
			if(this.popup_return_method)
				request_data.passthru_data.ui_return_method = this.popup_return_method;
			if(this.popup_return_multiple)
				params.multiple = true;
			open_popup_window(params);
			// FIXME popup does not currently steal focus, and probably should
			this.handleBlur();
		},
		openQuickCreate: function(query) {
			UI.openQuickCreate(this.module, {name: query, for_ref: 1}, this.getCreateCallback());
		},
		getCreateCallback: function() {
			var self = this;
			return function(key, value) {
				self.userUpdate(key, value);
			}
		},
		updateWithLookup: function(module, key, silent, nofocus, passthru) {
			if(! module || module != this.module)
				// fixme - support dynamic module
				return;
			if(! key || key == this.getKey())
				return;
			var self = this,
				callback = function(data) {
					var r, k, v;
					if(! data || ! (r = data.getResult()) ) return;
					if(r.formatted_rows.length) {
						k = r.formatted_rows[0].id;
						v = r.formatted_rows[0];
					}
					if(silent) self.update(k, v, silent, undefined, passthru);
					else self.userUpdate(k, v, nofocus, undefined, passthru);
				},
				data = {
					module: module,
					rows: [key],
					fields: this.getReturnFields(),
					filters: this.getFilters()
				},
				req = new App.conn.JSONRequest('popup_related_data', {silent: true}, data);
			req.fetch(callback);
		}
	};


    UI.EmailRecipsInput = function(id, params) {
        if(isElement(id)) {
            this.elt = id;
            this.id = this.elt.id;
        } else
			this.id = id;
		params.value = params.value || '';
		if(params) extendObject(this, params);
		if (!this.recips_array) {
			this.recips_array = this.value.split(';').map(trim).filter(function(e) {return e.length > 0});
		}
    };

    UI.EmailRecipsInput.prototype = {
        setup: function() {
            if(this.set_up)
                return;
            if(! this.elt)
                this.elt = $(this.id);
            var elts = this.elt.getElementsByClassName('email-recips-input');
            if(elts && elts.length == 1)
                this.input = elts[0];
            elts = this.elt.getElementsByClassName('email-recips-container');
            if(elts && elts.length == 1)
                this.container = elts[0];
			this.render();
			new UI.DragTarget(this.elt, this.getDragConfig());
			this.updateDisplay();
            this.set_up = true;
        },
        render: function() {
            this.buttons = [];
            var self = this;
            this.setupSearcher();
        },

		addRecips: function(rows) {
			var recips = this.getRecipsArray();
			if(rows) {
				for(var i = 0; i < rows.length; i++) {
					var v = rows[i];
					if(! v) continue;
					var r = {
						display: ((v.first_name || "") + ' ' + (v.last_name || "")).trim(),
						email: v.email_matched || ''
						// module: v._module || mod
					};
					recips.push(r);
				}
			}
			this.setRecipsArray(recips);
			this.updateDisplay();
		},

		getRecipsArray: function() {
			if (this.recips_array)
				return this.recips_array;
            if (!this.input)
                return [];
            return this.input.value.split(';').map(trim).filter(function(e) {return e.length > 0});
        },

		setRecipsArray: function(array) {
			this.recips_array = array;
            if (!this.input)
				return;
            this.input.value = array.map(function(e) {return typeof e === 'string' ? 'e' : e.display + ' <' + e.email + '>'}).join('; ');
        },

        updateDisplay: function() {
            if (!this.input || !this.container)
                return;
            UI.clearChildNodes(this.container);
            var self = this,
            	recips_outer = createElement2('div', {className: 'buttons-inline'}, null, this.container);
            var recips = this.getRecipsArray();
            for (var i = 0; i < recips.length; i++) {
                (function(i) {
					var recip = recips[i];
					var r = recip;
					if (typeof r !== 'string')
						r = r.display || r.email;
					var outer = createElement2('div', {className: 'input-field input-field-group', title: recip.email});
					new UI.Draggable(outer, self.getDragConfig(recip, i));
					createElement2('div', {className: 'inline-elt'}, outer, recips_outer);
                    if(recip.module) {
                    	createElement2('div', {className: 'input-label input-field-icon'},
                    		createElement2('div', {className: 'input-icon theme-icon module-' + recip.module}, null, outer),
                    		outer);
                    }
                    createElement2('div', {className: 'input-label'}, r, outer);
                    var removeButton = createElement2('div', {className: 'input-label input-remove', style: 'cursor:pointer'}, '', outer);
                    removeButton.onclick = function(evt) {
                        evt.stopPropagation();
                        var recips = self.getRecipsArray();
                        recips.splice(i, 1);
                        self.setRecipsArray(recips);
                        self.updateDisplay();
                    };
                })(i);
            }
            if(this.input_elt) {
            	createElement2('div', {className: 'inline-elt'}, this.input_elt, recips_outer);
            }
            this.buttons.forEach(function(b) {self.container.appendChild(b); });
        },

		validate: function() {
			this.invalid = false;
			if(this.required && this.isBlank()) {
				this.invalidMsg = 'required';
				this.invalid = true;
			}
			return ! this.invalid;
		},

		isBlank: function() {
			var recips = this.getRecipsArray();
			return ! recips.length;
		},

		focus: function() {
			if(this.input_elt)
				this.input_elt.focus();
		},

        updateMulti: function(rows, count) {
			var add = [];
			for (var i = 0; i < count; i++) {
				add.push(rows['ID_' + i]);
            }
            this.addRecips(add);
        },

        getFilters: UI.RefInput.prototype.getFilters,

        setupSearcher: function() {
            var self = this;
            this.input_elt = createElement2('div', {className: 'input-field input-field-group rbullet', tabIndex: 0},
                [createElement2('div', {className: 'input-label input-field-icon lg'},
                	createElement2('div', {className: 'uii uii-add'})),
                 createElement2('div', {className: 'input-label text-placeholder'},
                	' ' + mod_string('LBL_BUTTON_ADD_RECIPIENTS', 'Emails'))]
            );
            var searcher = new UI.SearchSelect(this.id + '-search-Invitees' ,
				{module: 'EmailRecipients',
                    target: this.elt, target_offset: {top: -1, left: 0},
                    search_blank: true,
                    form: this.form,
					show_checks: true,
                    icon_key: 'icon',
					allow_popup: true,
					dynamic_module: true,
					modules: ['EmailRecipients', 'Contacts', 'Leads', 'Users'],
                    allow_clear: false,
					allow_custom: true,
					display_name: 'name_email',
                    allow_rename: false,
                    popup_return_multiple: true,
                    extra_fields: {'email_matched': 'email_matched', '_display': 'name_email', 'salutation': 'salutation', 'name_email' : 'name_email'},
                    quick_create: false});
			searcher.onchange = function(k, v, mod) {
				if (!k) {
					v = {email_matched: v.name};
				}
				self.addRecips([v]);
            }
            searcher.onshow = function() {
                UI.addRemoveClass(self.input_elt, 'opened', true);
            }
            searcher.onhide = function() {
                UI.addRemoveClass(self.input_elt, 'opened', false);
                self.input_elt.focus();
            }
			//searcher.quicksearch_method = 'email_quicksearch';
            searcher.disable_show_detail = 1;
            searcher.setup();
            searcher.passthru = {ui_input_name: this.name, ui_return_method : 'updateMulti'};
            UI.addEventListener(this.input_elt, 'keydown', function(evt) {
                if(evt.keyCode == 32) {
                    UI.stopEvent(evt);
                    this.showSearch();
                };
            }, this);
            UI.addEventListener(this.input_elt, 'click', function(evt) {
                UI.stopEvent(evt);
                this.showSearch();
            }, this);
            this.searcher = searcher;
        },

        showSearch: function() {
            this.searcher.setFilters(this.getFilters());
            this.searcher.render();
            var w = UI.getEltRegion(this.input_elt);
            if(w)
                this.searcher.setWidth(Math.max(w.width- 200, 325) + 'px');
            this.searcher.showDefault(true);
		},

		onEnter: function(evt, data) {
			if (!data.values || !data.values.source || data.values.source == this) return;
			this.elt.style.backgroundColor = '#eee';
		},

		onLeave: function(evt, data) {
			this.elt.style.backgroundColor = '';
		},

		checkAccept: function(evt, data) {
			return data.values.source != this;
		},

		loadDragRef: function(evt) {
			var url, parts, ret, id, p, src;
			if(! evt) return;
			src = evt.dataTransfer;
			if(! src) return;
			if( (id = src.getData('1crm.ref-id')) ) {
				ret = {
					id: id,
					module: src.getData('1crm.ref-module'),
					record: src.getData('1crm.ref-value'),
					value: src.getData('1crm.ref-label')
				};
			} else {
				url = src.getData('text/uri-list') || src.getData('url');
				if(url && (p = url.indexOf('?')) >= 0) {
					parts = parseQueryString(url.substring(p+1));
					if(parts && parts.action == 'DetailView') {
						ret = {module: parts.module, record: parts.record};
					}
				}
			}
			return ret;
		},

		addFromRecord: function(ref) {
			var module = ref.module, key = ref.record;
			if (!module || !key) return;
			var self = this,
				callback = function(data) {
					var r, k, v, recip;
					if(! data || ! (r = data.getResult()) ) return;
					if(r.rows.length) {
						var row = r.rows[0];
						var email = row['email1'] || row['email2'];
						if (!email) return;
						var name = row['name'];
						if (name) recip = {display: name, email: email};
						else recip = email;
						var recips = self.getRecipsArray();
						recips.push(recip);
						self.setRecipsArray(recips);
						self.updateDisplay();
					}
				},
				data = {
					module: module,
					rows: [key],
					fields: ['_display', 'email1', 'email2'],
				},
				req = new App.conn.JSONRequest('popup_related_data', {silent: true}, data);
			req.fetch(callback);
		},

		onDrop: function(evt, data) {
			var ref = this.loadDragRef(evt);
			if (ref) {
				this.addFromRecord(ref);
				return;
			}
			var idx = data.values.idx;
			var recip = data.values.recip;
			var source = data.values.source;

			var recips = source.getRecipsArray();
			recips.splice(idx, 1);
			source.setRecipsArray(recips);
			source.updateDisplay();

			recips = this.getRecipsArray();
			recips.push(recip);
			this.setRecipsArray(recips);
			this.updateDisplay();
		},

		getDragConfig: function(recip, idx) {
			return {
				acceptedSourceTypes: ['EmailRecip', 'SelectListItem'],
				sourceType: 'EmailRecip',
				checkAccept: this.checkAccept.bind(this),
				onDrop: this.onDrop.bind(this),
				onEnter: this.onEnter.bind(this),
				onLeave: this.onLeave.bind(this),
				values: !recip ? {} : {
					recip: recip,
					idx: idx,
					source: this
				}
			};
		}
    };


	UI.hideConditionals = function (names) {
		for (var i = 0; i < names.length; i++) {
			var name = names[i];
			Wrap(document.body).find('.cell-' + name).forEach(function(elt) {
				Wrap(elt).addClass('cond-hidden');
				UI.removeRowHoles(elt);
			});
		}
	}

	UI.removeRowHoles = function(elt) {
		var row = elt.parentNode;
		var hide = true;
		var re = /\bcell-.*/;
		Wrap(row).find(function(elt) {
			return re.test(elt.className);
		}).forEach(function(el) {
			if (UI.hasClass(el, 'cell-'))
				return;
			if (!UI.hasClass(el, 'cond-hidden')) {
				hide = false;
			}
		});
		row.style.display = hide ? 'none' : '';
	}

	UI.updateConditionals = function(thiz, callback, hide) {
		var form = thiz.form,
			processed = {},
			chk = [],
			cf, cls, visible;
		processed[thiz.name] = true;
		for (var i = 0; i < UI.conditional_fields.length; i++) {
			var cf = UI.conditional_fields[i];
			if (cf.depends_on == thiz.name) {
				chk.push(cf);
			}
		}

		if(chk.length) {
			var $body = Wrap(document.body);
			$body.find('.cell-' + thiz.name).forEach(function(elt) {
				if (elt.style.display == 'none') {
					hide = true;
				}
			});
			for (var i = 0; i < chk.length; i++) {
				cf = chk[i];
				cls = 'cell-' + cf.name;
				visible = true;
				if (cf.depends_on_value && cf.depends_on_value.length) {
					visible = callback(cf);
				}
				visible &= !hide;
				$body.find('.' + cls).forEach(function(elt) {
					if (visible) {
						UI.removeClass(elt, 'cond-hidden');
					} else {
						UI.addClass(elt, 'cond-hidden');
					}
					UI.removeRowHoles(elt);
				});
				if (!processed[cf.name]) {
					processed[cf.name] = true;
					var input = UI.getFormInput(form, cf.name);
					if (input instanceof UI.SelectInput || input instanceof UI.CheckInput)
						input.updateConditionals(!visible);
				}
			}
		}
	};

	UI.SearchSelect = function(id, params) {
		this.id = id || 'searchselect';
		this.module = null;
		this.multiple = false;
		this.searcher = null;
		this.popup = null;
		this.mode = null;
		this.title = '';
		this.key = null;
		this.width = null;
		this.onchange = null;
		this.waiting_result = false; // need to assign ids to specific requests
		this.search_delay = 150;
		this.search_clear = true;
		this.quick_create = false;
		if(params) extendObject(this, params);
		this.initOptions();
	}
	UI.SearchSelect.prototype = {
		initOptions: function() {
			var opts, k, v, kv = [], mod;
			if(this.options) {
				opts = UI.getSelectOptions(this.options);
				if(! opts)
					console.error('select options not found:', this.options);
				this.options = opts;
			}
			if(! this.recent_options)
				this.recent_options = new UI.SelectOptions({display_name: '_display'});
			if(! this.search_options)
				this.search_options = new UI.SelectOptions({display_name: this.display_name || '_display'});
			if(! this.module_options && ! this.options) {
				opts = App.language.getList('moduleList');
				for(var i = 0; i < opts.keys.length; i++) {
					mod = opts.keys[i];
					if(! this.modules || this.modules.indexOf(mod) >= 0) {
						kv.push({
							k: mod,
							v: {_display: opts.values[i], icon: mod}
						});
					}
				}
				if (this.modules) {
					kv.sort(function(a, b) { return this.modules.indexOf(a.k) - this.modules.indexOf(b.k)}.bind(this));
				}
				k = kv.map(function(e) {return e.k});
				v = kv.map(function(e) {return e.v});
				this.module_options = new UI.SelectOptions({keys: k, values: v, display_name: '_display', icon_key: 'icon'});
			}
		},
		setup: function() {
			if(this.set_up) return;
			var self = this;
			this.search_input = new UI.SearchInput(this.id + '-text', {
				searcher: this, width: 200, embedded: true,
				show_clear: this.search_clear,
				onkeydown: function(evt) {
						if(evt && evt.keyCode === 9 && evt.shiftKey) {
							// shift-tab out of search should close dialog
							// (alternative jumps down the page)
							UI.stopEvent(evt);
							self.cancelSearch();
						}
					}
				});
			this.set_up = true;
		},
		render: function() {
			if(this.rendered) return;
			var self = this;
			if(this.flat) {
				this.elt = MakeElt('div', {'class': this.ext_class}).get();
			}
			else if(! this.popup) {
				var params = {
						target: this.target,
						target_offset: this.target_offset,
						destroy_on_close: this.destroy_on_close || false,
						ext_class: this.ext_class,
						show_above: this.show_above,
						min_width: this.min_width || 250,
						onmouseover: this.onmouseover,
						onmouseout: this.onmouseout,
						onmouseup: this.onmouseup,
						target_active_class: this.target_active_class || null};
				if(this.width)
					params.width = this.width;
				this.popup = new UI.PopupSlidePanel(this.id, params);
				this.popup.onshow = function() {
					self.visible = true;
					UI.callEvent(self, self.onshow);
				}
				this.popup.onhide = function() {
					self.visible = false;
					UI.callEvent(self, self.onhide);
				}
				this.popup.render();
				UI.clearChildNodes(this.popup.elt);
				this.elt = this.popup.elt;
			}

			if(this.search_input) {
				this.search_elt = MakeElt('div.card-header.panel-header').appendTo(this.elt).get();
				var search_row = MakeElt('div.tools-row.popup-search').appendTo(this.search_elt);
				search_row.append(this.search_input.render());
				search_row.append(MakeElt('div.spacer'));
				this.search_input.setup();

				this.info_button = MakeElt('div.uii.uii-lg.uii-info.active-icon', {tabindex: 0})
					.appendTo(search_row)
					.on('click, keydown[key=enter|space]', function(evt) {
						self.showDetail();
					}, {stop: true, prevent: true});
				if(this.allow_clear) {
					this.clear_button = MakeElt('div.uii.uii-lg.uii-remove.active-icon', {tabindex: 0})
						.appendTo(search_row)
						.on('click, keydown[key=enter|space]', function(evt) {
							self.setQuery('');
							self.close();
							UI.callEvent(self, self.onclear, evt);
							UI.callEvent(self, self.onchange, '', '', null, null, evt);
						}, {stop: true, prevent: true});
				}
				if(this.allow_popup) {
					this.popup_button = MakeElt('div.uii.uii-lg.uii-extern.active-icon', {tabindex: 0})
						.appendTo(search_row)
						.on('click, keydown[key=enter|space]', function(evt) {
							self.showDialog();
						}, {stop: true, prevent: true});
				}
			}

			this.return_link = MakeElt('a.tools-link', {href: '#'},
				MakeElt('span.uii.uii-previous'), nbsp(), app_string('LBL_REF_SELECT_MODULE'))
				.on('click', function() { self.showModules(); return false; });
			this.return_row = MakeElt('div.card-header.panel-subheader',
				MakeElt('div.tools-row.popup-title.return-link', this.return_link)).appendTo(this.elt);

			this.detail_close = createElement2('div', {className: 'uii uii-cancel active-icon popup-close', style: 'display: none'});
			this.detail_close.onclick = function() {
				if(self.detail_mode == 'search' || self.haveQuery()) self.search();
				else self.showRecent();
			}

			this.filter_elt = createElement2('div', {className: 'popup-filter'});
			this.filter_row = createElement2('div', {className: 'card-header panel-subheader', style: 'display: none'},
				this.filter_elt, this.elt);

			this.title_elt = createElement2('span', null, this.title, this.title_row);
			this.title_row = createElement2('div', {className: 'card-header panel-subheader'},
				createElement2('div', {className: 'tools-row popup-title search-results-title'}, [this.title_elt, this.detail_close]),
				this.elt);

			if(! this.list_elt) {
				this.list_elt = createElement2('div', {id: this.id + '-list', className: 'card-outer card-body panel-body ref-options'});
				this.elt.appendChild(this.list_elt);
			}
			if(! this.detail_elt) {
				this.detail_elt = createElement2('div', {id: this.id + '-detail', className: 'card-body panel-body popup-detail'});
				this.elt.appendChild(this.detail_elt);
			}
			if(! this.list) {
				this.list = new UI.SelectList(this.list_elt, {
					options: this.recent_options, no_scroll: true, popup: true, width: '100%',
					display_name: this.display_name || '_display',
					icon_key: this.icon_key, show_checks: this.show_checks,
					rows: get_default(this.max_rows, 10)});
				this.list.setup();
				this.list.focused = true;
				this.list.render();
				this.list.onchange = function(oldSel, evt) { var k = this.getSelectedKey(); self.rowSelected(k, this.getValue(k), false, evt); };
				this.list.oncurrent = function(idx) { var k; if(isset(idx)) { k = this.getKey(idx); self.rowCurrent(k, this.getValue(k)); } };
			}
			// TODO - combine wait indicator into status elt
			this.status_elt = MakeElt('div.text-help');
			this.status_row = MakeElt('div.card-footer.panel-footer.text-right:hidden', {id: this.id + '-status'},
					this.status_elt).appendTo(this.elt);

			this.wait_elt = MakeElt('div.card-footer.panel-footer.popup-wait:hidden', {id: this.id + '-wait'},
				MakeElt('div', null, App.util.waitIcon({small: true}))).appendTo(this.elt);

			this.rendered = true;
			return this.elt;
		},
		reposition: function() {
			if(this.popup && this.popup.visible) {
				requestAnimationFrame(
					this.popup.reposition.bind(this.popup)
				);
			}
		},
		focus: function() {
			this.focusSearch();
		},
		setWaitingResult: function(val) {
			var self = this;
			if(! val) {
				if(this.waiting_result) {
					clearTimeout(this.waiting_result);
					this.waiting_result = null;
				}
				this.show_waiting = false;
			}
			else {
				this.waiting_result = val;
				if(! this.waiting_timer)
					this.waiting_timer = setTimeout(this.checkWaiting.bind(this), 250);
			}
			this.updateFooter();
		},
		checkWaiting: function() {
			if(this.waiting_result) {
				this.show_waiting = true;
				this.updateFooter();
			}
		},
		setTitle: function(t) {
			this.title = t;
			UI.setElementText(this.title_elt, t);
			toggleDisplay(this.title_row, (isset(t) && t !== '') ? true : false);
		},
		setMode: function(mode) {
			this.mode = mode;
		},
		setModule: function(module, silent) {
			this.module = module;
			if(! silent && this.popup && this.popup.visible) {
				if(! module)
					this.showModules();
				else if(this.mode == 'search')
					this.search(true);
				else if(this.mode == 'recent')
					this.showRecent();
				else if(this.mode == 'detail')
					this.search();
			}
		},
		setFilters: function(fs) {
			this.filters = fs;
		},
		setKey: function(k, force_clear) {
			this.key = k;
			this.force_clear = force_clear;
		},
		setCustomValue: function(v) {
			this.custom_value = v;
		},
		setDetailModule: function(m) {
			this.detail_module = m;
		},
		showDefault: function(reinit) {
			if(reinit) {
				this.setKey('', true);
				this.setQuery('');
				this.perform_remove = false;
				//this.updateDisplay();
			}
			if(this.options) {
				if(this.search_blank || this.haveQuery())
					this.search();
				else
					this.showOptions();
			} else if(! this.module && ! this.global_search) {
				this.showModules();
				return;
			} else if(! this.haveQuery() && ! this.disable_recent && ! this.perform_remove) {
				this.showRecent();
			} else {
				this.search();
			}
			this.focusSearch();
		},
		focusSearch: function() {
			if(this.search_input)
				this.search_input.focus();
		},
		showOptions: function() {
			if(! this.options)
				return;
			var oldmode = this.mode;
			this.mode = 'options';
			this.setTitle(this.options_title || '');
			if(oldmode !== 'options') {
				this.list.setOptions(this.options);
				this.options_inited = true;
			}
			this.list.setSelectedKey(this.key, true);
			this.highlightCurrent();
			UI.setDisplayState(this.detail_elt, false);
			this.setWaitingResult(false);
			if(this.popup) this.popup.show();
			this.updateDisplay();
		},
		getModuleTitle: function() {
			if(this.module && this.module_options) {
				var info = this.module_options.getValue(this.module);
				if(info) return info._display;
			}
			return this.module;
		},
		makeTitle: function(title) {
			var mod_title = this.getModuleTitle();
			if(title && mod_title)
				title += ' (' + mod_title + ')';
			return title;
		},
		showRecent: function() {
			if(this.disable_recent) {
				if(this.popup) this.popup.hide();
				return;
			}
			var oldmode = this.mode;
			this.mode = 'recent';
			this.setTitle(this.makeTitle(this.recent_title || app_string('LBL_REF_RECENT_ENTRIES')));
			if(oldmode !== 'recent') {
				this.recent_options.clear();
				this.list.setOptions(this.recent_options);
			}
			toggleDisplay(this.detail_elt, false);
			var self = this,
				callback = function(data) {
					var rows = self.loadQueryResult(data.getResult());
					self.setSearchResults(rows);
				},
				exclude = this.getExcludeKeys(),
				data = {module: this.module, fields: this.extra_fields, filters: this.filters, exclude_ids: exclude},
				method = this.list_recent_method || 'list_recent',
				req = new App.conn.JSONRequest(method, {silent: true}, data);
			//req.oncomplete = function() { self.searchComplete(); }
			req.fetch(callback);
			this.setWaitingResult(req.request_id);
			this.updateDisplay();
			if(this.popup) this.popup.show();
		},
		/*searchComplete: function() {
			toggleDisplay(this.wait_elt, false);
			this.updateDisplay();
		},*/
		getListRowCount: function() {
			return this.list && this.list.options && this.list.options.keys.length;
		},
		updateFooter: function() {
			if(this.show_waiting) {
				UI.setDisplayState(this.wait_elt, true);
				UI.setDisplayState(this.status_row, false);
			} else {
				UI.setDisplayState(this.wait_elt, false);
				if(this.mode === 'detail' || this.mode === 'module') {
					UI.setDisplayState(this.status_row, false);
				} else {
					if(this.getListRowCount()) {
						if(this.mode === 'recent' || ! this.total_count) {
							UI.setDisplayState(this.status_row, false);
						} else {
							UI.setElementText(this.status_elt, ''+this.total_count+(this.total_count == 1 ? ' result':' results'));
							UI.setDisplayState(this.status_row, true);
						}
					}
					else if(this.global_search && ! this.module) {
						UI.setElementText(this.status_elt, app_string('LBL_REF_PRESS_TO_SEARCH'));
						UI.setDisplayState(this.status_row, true);
					}
					else {
						UI.setElementText(this.status_elt, app_string('LBL_NO_RESULTS'));
						UI.setDisplayState(this.status_row, true);
					}
				}
			}
		},
		setPerformRemove: function(flag, refocus) {
			this.perform_remove = !! get_default(flag, true);
			this.search();
			if(refocus) this.focusSearch();
		},
		updateFilter: function() {
			if(this.source_keys) {
				if(! this.filter_add_rem) {
					var self = this;
					this.filter_add_rem = MakeElt('div.input-button-group');
					this.filter_add = new UI.CheckInput(this.id + '-filtadd', {
						name: this.id + '-addrem',
						input_type: 'radio',
						ext_class: 'input-field',
						label: app_string('LBL_MULTI_ADD_ITEMS'),
						init_value: 1
					});
					this.filter_add.onchange = function() { if(this.getValue()) self.setPerformRemove(false, true); };
					this.filter_add_rem.append(this.filter_add.render());
					this.filter_add.setup();
					this.filter_rem = new UI.CheckInput(this.id + '-filtrem', {
						name: this.id + '-addrem',
						input_type: 'radio',
						ext_class: 'input-field',
						label: app_string('LBL_MULTI_REMOVE_ITEMS')
					});
					this.filter_rem.onchange = function() { if(this.getValue()) self.setPerformRemove(true, true); };
					this.filter_add_rem.append(this.filter_rem.render());
					this.filter_rem.setup();
				}
				if(this.perform_remove) {
					this.filter_add.setValue(false, true);
					this.filter_rem.setValue(true, true);
				}
				else {
					this.filter_rem.setValue(false, true);
					this.filter_add.setValue(true, true);
				}
				UI.setElementContent(this.filter_elt, this.filter_add_rem);
				return true;
			}
		},
		updateDisplay: function() {
			UI.setDisplayState(this.return_row,
				(this.mode !== 'module' && this.dynamic_module && ! this.union_module && ! this.options) ? 1 : 0);
			var show_list = false;
			if(this.mode === 'module') {
				UI.setDisplayState(this.search_elt, false);
				UI.setDisplayState(this.detail_elt, false);
				UI.setDisplayState(this.filter_row, false);
				UI.setDisplayState(this.list_elt, (show_list = true));
			}
			else if(this.mode === 'detail') {
				UI.setDisplayState(this.search_elt, false);
				UI.setDisplayState(this.detail_elt, true);
				UI.setDisplayState(this.detail_close, true);
				UI.setDisplayState(this.filter_row, false);
				UI.setDisplayState(this.list_elt, (show_list = false));
			}
			else {
				UI.setDisplayState(this.search_elt, true);
				UI.setDisplayState(this.detail_elt, false);
				UI.setDisplayState(this.detail_close, false);
				var show_info = !! (this.key &&
					(! this.options || (this.options.dom_name &&
						this.options.dom_name.substring(0, 6) == 'model.')));
				UI.setDisplayState(this.info_button, show_info);
				UI.setDisplayState(this.filter_row, !! this.updateFilter());
				UI.setDisplayState(this.list_elt, (show_list = !! this.getListRowCount()));
			}
			UI.setDisplayState(this.clear_button, !! (this.key || this.force_clear));
			if(show_list) this.list.updateDisplay();
			this.updateFooter();
		},
		showDetail: function() {
			if(this.disable_show_detail)
				return this.cancelSearch();
			this.mode = 'detail';
			if(this.key) {
				var self = this,
					k = this.key,
					callb = function(body, caption, width, buttons) {
						self.setDetails(k, body, caption, width);
					};
				this.setTitle(app_string('LBL_REF_DETAILS'));
				this.setWaitingResult('detail');
				App.util.getAdditionalDetails(this.detail_module || this.module, this.key, callb, undefined, 'refinput');
				this.updateDisplay();
			}
			else if(this.custom_value) {
				this.setTitle(app_string('LBL_REF_CUSTOM_ENTRY'));
				this.detail_elt.innerHTML = html_escape(this.custom_value);
				this.updateDisplay();
				if(this.popup) this.popup.show();
			}
		},
		setDetails: function(key, body, caption, width) {
			if(this.key == key && this.waiting_result === 'detail') {
				this.detail_elt.innerHTML = body;
				this.setWaitingResult(false);
				this.updateDisplay();
				if(this.popup) this.popup.show();
			}
		},
		setSearchResults: function(rows, req_id) {
			if(! this.waiting_result || (isset(req_id) && req_id != this.waiting_result))
				return;
			this.setWaitingResult(false);
			var sel_idx = this.initSearchOptions(true);
			var prefix = '';

			if(rows)
			for(var i = 0; i < rows.length; i++) {
                prefix = '';
				if (! this.global_search && (rows[i][1].hasOwnProperty('is_union') && rows[i][1].hasOwnProperty('_module')) && rows[i][1].is_union) {
                    prefix = rows[i][1]._module + '_';
                }
                this.search_options.addOption('' + prefix + rows[i][0], rows[i][1]);
            }
			this.list.setOptions(this.search_options);
			this.list.setSelectedKey(this.key, true);
			this.highlightCurrent(sel_idx);
			this.updateDisplay();
			this.reposition();
		},
		setOptions: function(opts) {
			this.options = opts;
			this.initOptions();
		},
		renameOptionValue: function(for_result) {
			var lbl = trim(this.query);
			if(this.allow_rename && lbl.length && this.key) {
				if(for_result) return lbl;
				lbl = app_string('LBL_REF_RENAME_ENTRY') + app_string('LBL_SEPARATOR') + lbl;
				return {_display: lbl, name: lbl};
			}
		},
		customOptionValue: function(for_result) {
			var lbl = trim(this.query), opts = this.options, i;
			if(this.allow_custom && lbl.length) {
				if(opts) {
					for(i = 0; i < opts.keys.length; i++) {
						if(opts.values[i] == lbl)
							return;
					}
				}
				if(! for_result) lbl = app_string('LBL_REF_CUSTOM_ENTRY') + app_string('LBL_SEPARATOR') + lbl;
				var ret = { name: lbl};
				ret[this.display_name || '_display'] = lbl;
				return ret;
			}
		},
		quickCreateOptionValue: function(for_result) {
			var lbl = trim(this.query);
			if(this.quick_create && lbl.length) {
				if(! for_result) lbl = app_string('LBL_REF_QUICK_CREATE_ENTRY') + app_string('LBL_SEPARATOR') + lbl;
				return {_display: lbl, name: lbl};
			}
		},
		initSearchOptions: function(clear) {
			var rename = this.renameOptionValue(),
				cust = this.customOptionValue(),
				create = this.quickCreateOptionValue(),
				opts = this.search_options,
				o = this.options;
			if(o) {
				if(o.columns) opts.columns = o.columns;
			}
			if(clear) {
				opts.clear();
				if(rename) opts.addOption('_rename', rename);
				if(cust) opts.addOption('_custom', cust);
				if(create) opts.addOption('_create', create);
			}
			return null;
		},
		haveQuery: function() {
			return isset(this.query) && trim(this.query) !== '';
		},
		search: function(clear_results) {
			this.total_count = null;
			if(! this.module && ! this.options && ! this.global_search) {
				this.showModules();
				return;
			}
			if(! this.haveQuery()) {
				if(this.search_blank || this.perform_remove)
					;
				else if(this.disable_recent)
					return this.cancelSearch();
				else
					return this.showRecent();
			}
			this.mode = 'search';
			this.setTitle(this.makeTitle(
				this.options ? this.options_title
				: (this.search_title || app_string('LBL_REF_SEARCH_RESULTS'))));
			this.cancelRunSearch();

			var sel_idx = this.initSearchOptions(clear_results);
			this.list.setOptions(this.search_options);
			if(clear_results) {
				this.list.setSelectedKey(this.key, true);
				this.highlightCurrent(sel_idx);
			}

			UI.setDisplayState(this.detail_elt, false);
			UI.setDisplayState(this.list_elt, !! this.search_options.keys.length);

			if(! this.module && this.global_search) {
				this.setTitle(null);
				this.updateDisplay();
				if(this.popup) this.popup.show();
				return;
			}

			this.runSearchDelayed(this.search_delay);
			this.updateFooter();
			if(this.popup) this.popup.show();
		},
		showModules: function() {
			this.mode = 'module';
			this.setTitle(this.module_title || app_string('LBL_REF_SELECT_MODULE'));
			this.cancelRunSearch();
			this.list.icon_key = 'icon';
			this.list.setOptions(this.module_options);
			this.list.setSelectedKey(this.module, true);
			this.updateDisplay();
			if(this.popup) this.popup.show();
		},
		highlightCurrent: function(sel_idx) {
			if(! this.list) return;
			var keys = this.list.getKeys(), vals, i, s, q = this.query;
			if(this.mode == 'search') {
				if(! isset(sel_idx)) {
					if(this.search_current_key) {
						for(i = 0; i < keys.length; i++) {
							if(keys[i] == this.search_current_key) {
								sel_idx = i;
								break;
							}
						}
					}
				}
				if(! isset(sel_idx) && ! this.disable_auto_select) {
					if(this.options && isset(q) && q !== '') {
						vals = this.list.getValues();
						for(i = 0; i < keys.length; i++) {
							if(keys[i] == '_create' || keys[i] == '_rename' || keys[i] == '_custom') continue;
							if(isObject(vals[i])) s = vals[i]._display;
							else s = vals[i];
							if(s && s.substring(0, q.length) == q) {
								sel_idx = i;
								break;
							}
						}
					}
					if(! isset(sel_idx)) {
						for(i = 0; i < keys.length; i++) {
							sel_idx = i;
							if(keys[i] != '_create' && keys[i] != '_rename' && keys[i] != '_custom')
								break;
						}
					}
				}
			}
			this.list.setCurrentIndex(sel_idx, null, true);
		},
		cancelRunSearch: function() {
			if(this.searchTimeout) {
				clearTimeout(this.searchTimeout);
				this.searchTimeout = null;
			}
			if(this.waiting_result) {
				App.conn.abortRequest(this.waiting_result);
				this.setWaitingResult(false);
			}
		},
		runSearchDelayed: function(delay) {
			if(delay && ! this.options) {
				var self = this;
				this.searchTimeout = setTimeout(function() { self.runSearch(); }, delay);
			} else
				this.runSearch();
		},
		loadQueryResult: function(result) {
			var rows = [];
			this.total_count = null;
			if(result && ! result.failed) {
				var l = result.formatted_rows,
					pk = result.primary_key,
					mdirs = result.module_dirs;
				this.total_count = result.total_count || result.result_count;
				for(var i = 0; i < l.length; i++) {
					if(mdirs && l[i].query_source) l[i]._module = mdirs[l[i].query_source];
					else if(mdirs) l[i]._module = mdirs[0];
					else l[i]._module = this.module;
					if(l[i]._module) l[i].icon = 'theme-icon module-' + l[i]._module;
					l[i].is_union = result.is_union;
					rows.push([l[i][pk], l[i]]);
				}
				this.is_union = result.is_union;
			}
			return rows;
		},
		setSourceKeys: function(keys) {
			this.source_keys = {};
			if(keys) for(var i = 0; i < keys.length; i++) {
				this.source_keys[keys[i]] = 1;
			}
			if((this.flat || (this.popup && this.popup.visible)) && (this.mode === 'search' || this.mode === 'options'))
				this.search();
		},
		getIncludeKeys: function() {
			if(this.perform_remove && this.source_keys) {
				var ret = [];
				for(var k in this.source_keys)
					ret.push(k);
				if(! ret.length) ret.push('--'); // filter to none
				return ret.join(',');
			}
		},
		getExcludeKeys: function() {
			if(! this.perform_remove && this.source_keys) {
				var ret = [];
				for(var k in this.source_keys)
					ret.push(k);
				return ret.join(',');
			}
		},

		filterKeys: function(idx, key, sel, perform_remove) {
			if(sel && sel[key]) {
				if(! perform_remove) return false;
			} else if(perform_remove) {
				return false;
			}
			return true;
		},

		runSearch: function() {
			this.searchTimeout = null;
			if(this.options) {
				var opts = this.options,
					q = isset(this.query) ? trim(this.query).toLowerCase() : '',
					rows = [], i, k, v, lst = [], tail = [],
					sel = this.source_keys;
				var filter_keys = (this.filter_keys || this.filterKeys).bind(this);
				for(i = 0; i < opts.keys.length; i++) {
					k = opts.keys[i];
					if (!filter_keys(i, k, sel, this.perform_remove))
						continue;
					v = opts.getLabel(k).toLowerCase();
					if(sel && sel[k]) {
						if(! this.perform_remove) continue;
					} else if(this.perform_remove) {
						continue;
					}
					if(q.length) {
						if(v.indexOf(q) < 0) {
							if(this.search_blank && ! this.matching_only)
								tail.push([k, opts.getValue(k)]);
							continue;
						}
						if(v.substring(0, q.length) != q) {
							lst.push([k, opts.getValue(k)]);
							continue;
						}
					}
					rows.push([k, opts.getValue(k)]);
				}
				rows.pushArray(lst);
				rows.pushArray(tail);
				this.total_count = rows.length;
				this.setWaitingResult(true);
				this.setSearchResults(rows);
				return;
			}
			//if(! this.haveQuery()) return;
			var self = this,
				callback = function(data) {
					var rows = self.loadQueryResult(data.getResult());
					self.setSearchResults(rows, data.request_id);
				},
				data = {
					module: this.module,
					bean_name: this.bean_name,
					filter_text: this.query,
					fields: this.extra_fields,
					include_ids: this.getIncludeKeys(),
					exclude_ids: this.getExcludeKeys()
				},
				method = this.quicksearch_method || 'quicksearch',
				req = new App.conn.JSONRequest(method, {silent: true}, data);
			if(this.filters)
				req.setArgs(this.filters);
			//req.oncomplete = function() { self.searchComplete(); }
			req.fetch(callback);
			this.setWaitingResult(req.request_id);
			this.updateDisplay();
		},
		setQuery: function(prefix) {
			var q = this.query;
			if(! isset(q)) q = '';
			if(! isset(prefix)) prefix = '';
			this.query = prefix;
			if(prefix !== q)
				this.search(true);
			if(this.search_input)
				this.search_input.setValue(prefix, true);
		},
		cancelSearch: function() {
			this.close();
			UI.callEvent(this, this.oncancel);
		},
		close: function() {
			this.search_current_key = null;
			this.setWaitingResult(false);
			if(this.popup) {
				this.popup.close();
				if(this.popup.destroy_on_close) {
					this.popup = null;
					this.title_elt = this.list_elt = this.detail_elt = this.list = null;
				}
			}
			if(this.detail_elt)
				this.detail_elt.innerHTML = ''; // get rid of form object
		},
		setWidth: function(w) {
			this.width = w;
			if(this.popup)
				this.popup.setWidth(w);
		},
		adjustCurrentIndex: function(nav, nowrap, evt) {
			if(this.list)
				this.list.adjustCurrentIndex(nav, nowrap, evt);
		},
		acceptCurrent: function(evt) {
			if(this.list)
				return this.list.selectCurrent(evt);
			UI.callEvent(this, this.onaccept, this.search_input.getValue(), evt);
		},
		submitSearch: function(q) {
			return UI.callEvent(this, this.onaccept, q);
		},
		rowSelected: function(k, v, silent, evt) {
			if(this.mode == 'module') {
				this.setModule(k);
				UI.setDisplayState(this.list_elt, false);
				this.list.setOptions(null);
				this.showDefault();
				return;
			}
			if(k === '_custom') {
				k = '';
				v = this.customOptionValue(true);
			}
			else if(k == '_rename') {
				k = this.key;
				if(! silent)
					UI.callEvent(this, this.onrename, this.renameOptionValue(true), evt);
				silent = true;
			}
			else if(k === '_create') {
				this.setKey('');
				var query = this.getQuery();
				this.close();
				UI.callEvent(this, this.onquickcreate, evt, query);
				return;
			} else if (this.is_union) {
				var pos = k.indexOf('_');
				if (~k) {
					k = k.substring(pos + 1);
				}
			}
			this.setKey(k);
			keepOpen = (this.popup_return_multiple && evt && (evt.cmdKey || evt.shiftKey));
			if(! keepOpen)
				this.close();
			if(! silent) {
				if(this.perform_remove)
					UI.callEvent(this, this.onremove, k, v, this.module, keepOpen, evt);
				else
					UI.callEvent(this, this.onchange, k, v, this.module, keepOpen, evt);
			}
			if (keepOpen)
				this.focus();
		},
		rowCurrent: function(k, v, silent) {
			if(this.mode == 'search')
				this.search_current_key = k;
		},
		isHovered: function() {
			return (this.popup && this.popup.hovered);
		},
		getModule: function() {
			return this.module;
		},
		getQuery: function() {
			return this.search_input && this.search_input.getValue();
		},
		showDialog: function() {
			if(UI.callEvent(this, this.onpopup)) return;
			var self = this,
				q = this.getQuery(),
				mod = this.module,
				request_data = {
					form_name: this.form.getAttribute('id') || this.form.getAttribute('name'),
					call_back_function: this.popup_callback || 'ui_popup_return',
					passthru_data: this.passthru,
					field_to_name_array: {id: 'id', _display: '_display'}
				},
				params = {module: mod, layout: this.popup_layout, request_data: request_data, inline: true};
			if(! mod)
				return;
			if(this.union_module)
				request_data.field_to_name_array['_module'] = 'query_module';
			var add_fs = this.extra_fields;
			if(add_fs) {
				for(i in add_fs) {
					request_data.field_to_name_array[i] = add_fs[i];
				}
			}
			params.filter = deep_clone(this.filters || {});
			params.filter.filter_text = trim(q);
			if(this.popup_return_method)
				request_data.passthru_data.ui_return_method = this.popup_return_method;
			if(this.popup_return_multiple)
				params.multiple = true;
			params.onclose = function() {
				self.cancelSearch(); // reset focus to caller
			}
			this.setQuery('');
			open_popup_window(params);
			this.close();
		}
	};


	UI.SearchInput = function(id, params) {
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		this.focused = false;
		this.show_clear = true;
		if(params) extendObject(this, params);
	}
	UI.SearchInput.prototype = {
		setup: function() {
			if(this.set_up)
				return;
			if(! this.elt)
				this.elt = $(this.id);
			if(! this.field) {
				if(this.name && this.form)
					this.field = this.form.elements[this.name];
				else if(this.field_id)
					this.field = $(this.field_id);
			}
			if(! this.elt)
				return;
			if(! this.clear_button) {
				this.clear_button = Wrap(this.elt).findFirst('.search-clear').get(0);
			}
			if(! this.tabIndex && this.field && this.field.tabIndex > 0)
				this.tabIndex = this.field.tabIndex;
			this.addHooks();
			this.updateDisplay();
			this.set_up = true;
		},
		addHooks: function() {
			var events = {
				mousedown: this.handleMouseDown,
				mouseenter: this.handleMouseEnter,
				mouseleave: this.handleMouseLeave,
				click: this.handleMouseClick,
			};
			UI.addEventListeners(this.elt, events, this);
			var field_events = {
				focus: this.handleFocus,
				blur: this.handleBlur,
				mousedown: this.fieldMouseDown,
				keydown: this.fieldKeyDown,
				input: this.fieldInput,
				change: this.handleChanged
			};
			UI.addEventListeners(this.field, field_events, this);
			var clear_events = {
				focus: this.handleFocus,
				blur: this.handleBlur,
				mousedown: this.fieldMouseDown,
				click: this.clearInput
			};
			UI.addEventListeners(this.clear_button, clear_events, this);
			if(this.clear_button) {
				this.clear_button.tabIndex = -1;
			}
		},
		fieldMouseDown: function(evt) {
			UI.stopPropagation(evt);
			UI.callEvent(this, this.onmousedown, evt);
		},
		fieldKeyDown: function(evt) {
			if(evt.keyCode == 38) nav = -1;
			else if(evt.keyCode == 40) nav = 1;
			else nav = 0;
			if(nav) {
				if(this.searcher)
					this.searcher.adjustCurrentIndex(nav, null, evt);
				else
					this.showSearch();
				UI.stopEvent(evt);
			} else if(evt.keyCode == 27) {
				if(this.field.value.length) {
					UI.stopEvent(evt);
					this.clearInput();
				} else if(this.searcher) {
					UI.stopEvent(evt);
					this.hideSearch(true);
				}
			} else if(evt.keyCode == 13) {
				if(this.searcher) {
					var accept = this.searcher.acceptCurrent(evt);
					if(accept) {
						if(! this.embedded) this.hideSearch();
					} else if(accept !== false)
						this.handleManualSubmit();
					UI.stopEvent(evt);
				}
			} else {
				var input = this;
				setTimeout(function() { input.showHideClear(); }, 50);
			}
			UI.callEvent(this, this.onkeydown, evt);
			UI.stopPropagation(evt);
		},
		fieldInput: function(evt) {
			this.updateSearch();
		},
		handleMouseDown: function(evt) {
			this.focus();
			UI.stopEvent(evt);
			UI.callEvent(this, this.onmousedown, evt);
		},
		handleMouseClick: function(evt) {
			UI.callEvent(this, this.onclick, evt);
		},
		handleMouseEnter: function(evt) {
			this.hovered = true;
			this.showHideClear();
		},
		handleMouseLeave: function(evt) {
			this.hovered = false;
			this.showHideClear();
		},
		focus: function() {
			if(this.field)
				this.field.focus();
		},
		select: function() {
			if(this.field) {
				this.field.focus();
				this.field.select();
			}
		},
		handleFocus: function(evt, clear_focused) {
			this.focused = clear_focused ? 'clear' : true;
			if(this.field && ! clear_focused && ! this.did_clear)
				this.focus_value = this.field.value;
			UI.callEvent(this, this.onfocus, evt);
			this.updateDisplay();
		},
		handleBlur: function(evt) {
			var oldfocus = this.focused;
			this.focused = false;
			UI.callEvent(this, this.onblur, evt);
			if(oldfocus != 'clear' && this.did_clear && this.getValue() === '') {
				this.checkSubmit()
				this.did_clear = false;
			}
			this.updateDisplay();
		},
		clear: function() {
			this.field.value = '';
			this.updateDisplay();
		},
		clearInput: function(evt) {
			if(! isset(this.focus_value))
				this.focus_value = this.field.value;
			this.field.value = '';
			if(this.focused) this.did_clear = true;
			this.focus();
			UI.callEvent(this, this.onchange, evt);
			this.updateSearch();
			if(! this.focused)
				this.checkSubmit();
			this.updateDisplay();
		},
		showHideClear: function() {
			if(this.field && this.clear_button) {
				UI.setDisplayState(this.clear_button, (this.show_clear && this.field.value.length && (this.hovered || this.focused)));
				if(this.focused == 'clear')
					this.focus();
			}
		},
		handleManualSubmit: function() {
			this.field.value = trim(this.field.value);
			this.checkSubmit(true);
			this.focus_value = this.field.value;
			this.updateDisplay();
		},
		handleChanged: function(evt) {
			if(this.redirecting) return;
			this.field.value = trim(this.field.value);
			UI.callEvent(this, this.onchange, evt);
			if(this.form) {
				var input = this;
				setTimeout(function() { input.checkSubmit(); }, 300);
			}
			this.updateDisplay();
		},
		checkSubmit: function(force) {
			if(this.redirecting) return;
			if(force || (this.auto_submit && this.field && this.focus_value !== this.field.value)) {
				if(this.searcher) {
					if(this.searcher.submitSearch && this.searcher.submitSearch(this.field.value))
						return true;
					this.hideSearch();
				}
				if(this.form) {
					UI.submitForm(this.form);
					return true;
				}
			}
		},
		getValue: function() {
			return this.field ? this.field.value : null;
		},
		setValue: function(value) {
			this.field.value = value;
			this.updateDisplay();
		},
		getModule: function() {
			if(this.module) return this.module;
			if(this.form && this.form.module) return this.form.module.value;
		},
		getListId: function() {
			if(this.form && this.form.list_id) return this.form.list_id.value;
		},
		updateSearch: function() {
			this.showSearch();
		},
		cancelShowSearch: function() {
			if(this.searchTimeout) {
				clearTimeout(this.searchTimeout);
				this.searchTimeout = null;
				return true;
			}
		},
		cancelHideSearch: function() {
			if(this.hideSearchTimeout) {
				clearTimeout(this.hideSearchTimeout);
				this.hideSearchTimeout = null;
				return true;
			}
		},
		showSearchDelayed: function(delay) {
			this.cancelHideSearch();
			var input = this;
			this.searchTimeout = setTimeout(function() { input.showSearch(); }, delay || 1000);
		},
		initSearcher: function() {
			var mod = this.getModule(), list_id = this.getListId(), self = this;
			if(! this.quick_view || ! mod || ! list_id) return false;
			this.searcher = new UI.SearchSelect(this.id + '-select',
				{module: mod, target: this.elt, target_offset: {top: -1, left: 10},
				 disable_recent: true, search_title: app_string('LBL_SEARCH_QUICK_VIEW'),
				 disable_auto_select: true});
			this.searcher.onchange = function(k, v, mod) {
				if(isset(self.focus_value)) self.setValue(self.focus_value);
				self.redirecting = true;
				if (self.form && self.form.action && self.form.action.value == 'Popup')
					sListView.popupReturn(list_id, mod, k);
				else
					sListView.showDetail(list_id, {module: mod, action: 'DetailView', record: k});
			}
			this.searcher.onmouseup = function(evt) {
				self.searchMouseUp(evt);
			}
			this.own_search = true;
			return true;
		},
		searchMouseUp: function(evt) {
			if(this.focused === 'search')
				this.field.focus();
		},
		showSearch: function() {
			if(! this.searcher && ! this.initSearcher())
				return;
			if(this.own_search) {
				this.cancelShowSearch();
				this.cancelHideSearch();
				this.searcher.render();
				var w = UI.getEltRegion(this.elt);
				if(w)
					this.searcher.setWidth(Math.max(w.width - 14, 325) + 'px');
			}
			if(! this.disabled)
				this.searcher.setQuery(this.getValue());
			this.searcher.showDefault();
		},
		hideSearchDelayed: function(delay) {
			this.cancelShowSearch();
			this.cancelHideSearch();
			var input = this;
			this.hideSearchTimeout = setTimeout(function() { input.hideSearch(); }, delay || 100);
		},
		hideSearch: function(refocus) {
			this.cancelShowSearch();
			this.cancelHideSearch();
			if(this.searcher) {
				this.searcher.cancelSearch();
			}
			if(this.focused && this.field && refocus) {
				this.field.focus();
				this.field.select();
			}
		},
		render: function() {
			if(this.rendered) return;
			if(! this.elt) {
				this.elt = createElement2('div', {className: 'input-field input-field-group input-search', id: this.id});
				if(this.width) UI.setWidth(this.elt, this.width);
			} else
				UI.clearChildNodes(this.elt);
			createElement2('div', {className: 'input-label input-field-icon lg'},
				createElement2('div', {className: 'uii uii-lg uii-search'}), this.elt);
			this.field = createElement2('input', {className: 'input-text', value: this.init_value, placeholder: this.placeholder, autocorrect: 'off', autocomplete: 'off', tabIndex: this.tabIndex}, null, this.elt);
			this.field.spellcheck = false;
			this.clear_button = createElement2('div', {style: {display: 'none'}, className: 'uii uii-clear search-clear'}, null, this.elt);
			this.rendered = true;
			return this.elt;
		},
		updateDisplay: function() {
			this.showHideClear();
			UI.addRemoveClass(this.elt, 'active', !! this.focused);
		}
	};

	UI.TagsInput = function(id, params) {
		var attrs = isObject(params) ? params : {};
		attrs.module = 'KBArticles';
		attrs.quick_view = true;
		attrs.tags = Array.isArray(attrs.tags) ? attrs.tags : [];
		UI.SearchInput.call(this, id, attrs);
		this.animation = new App.animation.Animation()
					.distribute(20, function(percent) { return {opacity: Math.abs((50 - percent) / 80) + 0.375}})
					.duration(500)
				.timing('ease-out')
	}

	extendClass(UI.TagsInput, UI.SearchInput, {
		render: function() {
			if (this.rendered) return;
			var elt = UI.SearchInput.prototype.render.call(this);
			UI.removeClass(this.elt, 'input-search input-field');
			UI.addClass(this.field, 'flat');
			Wrap(elt).find('.uii-search.input-field-icon').remove();
			Wrap(this.clear_button).remove();
			this.tags_container = createElement2('div', {className: 'input-inline input-sm'});
			this.tags_field = createElement2('input', {type: 'hidden', name: this.name, value: this.getJoinedTags()});
			var outer = createElement2('div', {className: 'input-sm '}, [this.tags_field, elt, this.tags_container]);
			this.renderTags();
			return outer;
		},
		getListId: function() {
			return this.id + '-list-id';
		},
		initSearcher: function() {
			var ret = UI.SearchInput.prototype.initSearcher.call(this);
			if (this.searcher) {
				this.searcher.bean_name = 'kb_tags';
				var setSearchResults = this.searcher.setSearchResults;
				var self = this;
				this.searcher.setSearchResults = function(rows, req_id) {
					setSearchResults.call(this, rows, req_id);
					if (!self.removed_text && this.search_options.keys.length) {
						var key = this.search_options.keys[0];
						var best = self.nthMatch(0);
						if (best !== null) {
							var value = self.getValue();
							if (best.indexOf(value) === 0) {
								self.field.value = best;
								self.field.setSelectionRange(value.length, best.length);
							}
						}
					}
					self.removed_text = false;
				}
				this.searcher.onchange = function(k, v, mod) {
					self.addTag(v._display);
					self.clear();
					self.focus();
					self.hideSearch();
				}
				this.searcher.makeTitle = function() {
					return '';
				}
			}
			return ret;
		},
		addTag: function(tag) {
			this.addTags([tag]);
		},
		addTags: function(tags) {
			var idxs = [];
			tags.forEach(function(tag) {
				var idx = this.tags.indexOf(tag);
				if (!~idx) {
					this.tags.push(tag);
					idx = this.tags.length - 1;
				}
				if (!~idxs.indexOf(idx))
					idxs.push(idx);
			}.bind(this));
			this.tags_field.value = this.getJoinedTags();
			this.renderTags(idxs);
		},
		getJoinedTags: function() {
			return this.tags.join(', ');
		},

		handleBlur: function(evt) {
			this.hideSearch();
			UI.SearchInput.prototype.handleBlur.call(this, evt);
		},
		fieldKeyDown: function(evt) {
			if(evt.keyCode == 8 || evt.keyCode == 46) {
				this.removed_text = true;
			}
			if(evt.keyCode == 13) {
				var value;
				if (isset(this.searcher.search_current_key)) {
					value = this.nthMatch(this.searcher.list.current_index);
				}
				if (!isset(value))
					value = this.getValue().trim();
				var add = this.allow_add;
				if (!add) {
					add = this.searcher.search_options.values.reduce(
						function(prev, val) { return (val._display == value ? true : prev); },
						false);
				}
				if (add) {
					var values = value.split(',').map(
						function(x) { return x.trim(); }).filter(
						function(x) { return x.length > 0; });
					this.clear();
					this.addTags(values);
					this.hideSearch();
				}
				return false;
			} else {
				var ret = UI.SearchInput.prototype.fieldKeyDown.call(this, evt);
				return ret;
			}
		},
		nthMatch: function(idx) {
			if (this.searcher && this.searcher.search_options.keys.length > idx) {
				var key = this.searcher.search_options.keys[idx];
				return this.searcher.search_options.values.reduce(
					function(prev, value) { return (value.id == key ? value._display : prev); },
					null);
			}
		},
		renderTags: function(animateIdxs) {
			if (!Array.isArray(animateIdxs))
				animateIdxs = [];
			var self = this;
			var $elt = Wrap(this.tags_container);
			$elt.find('.tag').remove();
			var elements = this.tags.map(function(tag, idx) {
				var label = createElement2('div', {className: 'input-label'}, tag);
				var removeButton = createElement2('div', {style: {cursor: 'pointer'}, className: 'input-label input-remove'});
				var outer = createElement2('div', {className: 'input-field input-field-group state-label tag'}, [label, removeButton]);
				removeButton.onclick = function() {
					self.tags.splice(idx, 1);
					self.renderTags();
					self.tags_field.value = self.getJoinedTags();
					self.focus();
				}
				if (~animateIdxs.indexOf(idx)) {
					setTimeout(function() {	self.animation.run(outer); }, 20);
				}
				return outer;
			});
			$elt.append(elements);
		}
	});



	UI.CheckInput = function(id, params) {
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		this.onchange = null;
		this.focused = false;
		if(params) extendObject(this, params);
	};
	UI.CheckInput.prototype = {
		setup: function() {
			if(this.set_up)
				return;
			if(! this.elt && this.id) {
				this.elt = $(this.id);
			}
			if(! this.field) {
				if(this.field_id)
					this.field = $(this.field_id);
				else if(this.name && this.form) {
					this.field = this.form.elements[this.name];
					if(this.field && this.field.length) {
						if(this.field.length == 2 && this.field[1].type == 'checkbox') {
							this.off_value = this.field[0].value;
							this.field = this.field[1];
						}
						else
							this.field = null;
					}
				}
			}
			if(this.field) {
				this.real_check = (this.field.type == 'checkbox' || this.field.type == 'radio');
				this.orig_value = this.getValue();
				if(! isset(this.off_value)) this.off_value = 0;
			}
			this.addHooks();
			this.updateDisplay();
			this.set_up = true;
			this.updateConditionals();
		},
		addHooks: function() {
			if(this.real_check) {
				UI.addEventListener(this.field, 'change', this.handleChanged, this);
				UI.addEventListener(this.field, 'focus', this.handleFocus, this);
				UI.addEventListener(this.field, 'blur', this.handleBlur, this);
			}
			else if(this.elt) {
				UI.addEventListener(this.elt, 'click', this.handleClick, this);
				UI.addEventListener(this.elt, 'keydown', this.handleKeyDown, this);
			}
		},
		render: function() {
			if(! this.rendered) {
				var hidden = MakeElt('input', {
					type: 'hidden', name: this.name,
					value: 0, id: this.id+'-hidden'});
				var check = MakeElt('input', {
					type: this.input_type || 'checkbox', name: this.name,
					value: this.submit_value || 1, id: this.id+'-checkbox',
					tabindex: this.tabIndex});
				if(this.init_value) check.prop('checked', true);
				if(this.disabled) check.prop('disabled', true);
				this.elt = MakeElt('div', {
					id: this.id,
					'class': (this.className || ('input-check' + (this.ext_class ? ' ' + this.ext_class : '')))});
				if(! this.no_default && this.input_type != 'radio')
					this.elt.append(hidden);
				this.elt.append(this.field = check.get());
				var toggle_switch;
				if (this.add_switch)
					toggle_switch = MakeElt('div', {'class': 'switch'});
				this.elt.append(MakeElt('label', {'for': this.id+'-checkbox', 'class': 'input-label'}, [' ' + (this.label || ''), toggle_switch]));
				this.rendered = true;
			}
			return this.elt;
		},
		focus: function() {
			if(this.real_check && this.field) this.field.focus();
			else if(this.elt) this.elt.focus();
		},
		getValue: function() {
			var val = this.orig_value || 0;
			if(this.field) {
				if(this.real_check)
					val = this.field.checked ? this.field.value : this.off_value;
				else if(isset(this.submit_value) && this.field.value === this.submit_value)
					return 1;
				else
					val = this.field.value;
			}
			if(val === '1' || val === 'true') val = 1;
			if(val === '' || val === '0') val = 0;
			return val;
		},
		setValue: function(v, silent, evt) {
			if(this.real_check && this.field) {
				this.field.checked = !! v;
			} else {
				v = v ? (this.submit_value || 1) : 0;
				if(this.field) this.field.value = v;
			}
			this.afterUpdate(evt, silent);
		},
		toggle: function(silent, evt) {
			this.setValue(! this.getValue(), silent, evt);
		},
		handleClick: function(evt) {
			this.toggle(false, evt);
			this.afterUpdate(evt);
		},
		handleChanged: function(evt) {
			this.afterUpdate(evt);
		},
		afterUpdate: function(evt, silent) {
			var v = this.getValue();
			if(this.orig_value !== v) UI.markFormDirty(this);
			this.updateDisplay();
			if(! silent) UI.callEvent(this, this.onchange, evt, v);
			this.updateConditionals();
			if(this.auto_submit)
				UI.submitForm(this.form);
		},
		handleKeyDown: function(evt) {
			if(evt.keyCode == 13) {
				UI.stopEvent(evt);
				if(this.form) UI.submitForm(this.form);
			}
		},
		handleFocus: function(evt) {
			this.focused = true;
			this.updateDisplay();
			UI.callEvent(this, this.onfocus, evt);
		},
		handleBlur: function(evt) {
			this.focused = false;
			this.updateDisplay();
			UI.callEvent(this, this.onblur, evt);
		},
		updateDisplay: function() {
			if(this.elt) {
				UI.addRemoveClass(this.elt, 'checked', this.getValue());
				UI.addRemoveClass(this.elt, 'active', !! this.focused);
			}
		},
		setDisabled: function(disabled) {
			if(! isset(disabled)) disabled = 1;
			this.disabled = !! disabled;
			if(this.elt) this.elt.disabled = this.disabled;
			if(this.real_check && this.field)
				this.field.disabled = this.disabled;
		},
		updateConditionals : function(hide) {
			var value = this.getValue();

			var callback = function(cf) {
				if (cf.depends_on_value == '0')
					return !value;
				if (cf.depends_on_value == '1')
					return value;
				return true;
			}
			UI.updateConditionals(this, callback, hide);
		}
	};


	UI.QuickCheck = function(params) {
		var id = params.id || 'quickchk-'+(++ UI.QuickCheck.uniq_idx);
		UI.CheckInput.call(this, id, params);
	}
	UI.QuickCheck.uniq_idx = 0;
	extendClass(UI.QuickCheck, UI.CheckInput, {
		init: function(value, onchange) {
			this.orig_value = value;
			this.onchange = onchange;
			this.set_up = false;
			this.setup();
		},
		initSource: function(source, value, onchange, oncancel) {
			this.elt = source;
			this.init(value, onchange, oncancel);
		},
		showPopup: function(timeout, elt) {
			if(elt.firstChild) this.elt = elt.firstChild;
			if(! this.field) this.field = createElement2('input', {type: 'hidden', value: this.getValue()});
			this.toggle();
		}
	});


	UI.TextInput = function(id, params) {
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		this.focused = false;
		this.invalid = false;
		this.allow_blank = true;
		this.show_search = false;
		if(params) extendObject(this, params);
	};
	UI.TextInput.prototype = {
		setup: function() {
			if(this.set_up)
				return;
			if(! this.field) {
				if(this.elt)
					this.field = this.elt;
				else if(this.name && this.form)
					this.field = this.form.elements[this.name];
				else if(this.field_id)
					this.field = $(this.field_id);
				else if(this.id)
					this.field = $(this.id);
			}
			if(! this.elt)
				this.elt = this.field;
			if(this.field) this.orig_value = this.field.value;
			if(! this.altInput && this.alt_name) {
				this.altInput = UI.getFormInput(this.form, this.alt_name);
			}
			this.addHooks();
			this.updateDisplay();
			this.set_up = true;
		},
		addHooks: function() {
			var input = this;
			if(this.field) {
				this.field.onfocus = function(evt) { return input.handleFocus(evt || window.event); }
				this.field.onblur = function(evt) { return input.handleBlur(evt || window.event); }
				this.field.onchange = function(evt) { return input.handleChanged(evt || window.event); }
				if('oninput' in this.field)
					this.field.oninput = function(evt) { return input.handleInput(evt || window.event); }
				else // IE
					this.field.onpropertychange = function() { return input.handleInputCompat(window.event); }
				this.field.onkeydown = function(evt) { return input.handleKeyDown(evt || window.event); }
				this.field.onkeyup = function(evt) { return input.handleKeyUp(evt || window.event); }
				this.field.onmousedown = function(evt) { return input.handleMouseDown(evt || window.event); }
				this.field.onclick = function(evt) { return input.handleClick(evt || window.event); }
			}
			if(this.altInput)
				this.altInput.onchange = function() { input.clear(); }
			if(this.format == 'password' && this.check_strength)
				this.addStrengthCheck();
		},
		addStrengthCheck: function(loaded) {
			var input = this, sz, w, e, ew, next;
			if(! loaded) {
				App.util.addScriptInclude('include/javascript/zxcvbn.js', function() { input.addStrengthCheck(1); });
				return;
			}
			if(! this.str_meter && this.field && this.field.parentNode) {
				sz = UI.getEltRegion(this.field);
				if(sz && (w = sz.width)) {
					this.str_meter = createElement2('div', {style: 'vertical-align: top; width: '+w+'px; clear: left; height: 3px; line-height: 3px; margin-top: 1px'});
					ew = Math.floor((w - 8) / 5);
					for(var i = 0; i < 5; i++) {
						e = createElement2('div', {style: 'display: inline-block; vertical-align: top; height: 3px; line-height: 3px; width: '+ew+'px'}, null, this.str_meter);
						e.style.marginLeft = i ? '2px' : Math.floor((w - ew*5 - 8)/2)+'px';
						e.style.backgroundColor = '#bbb';
					}
					var next = this.field.nextSibling;
					if(next && next.className == 'requiredStar') next = next.nextSibling;
					this.field.parentNode.insertBefore(this.str_meter, next);
				}
			}
			if(this.str_meter) {
				UI.attachInputEvent(this, 'oninput', function() { input.setStrength(zxcvbn(input.getValue()).score); });
			}
		},
		setStrength: function(val) {
			var e, c;
			this.strength = val;
			if(this.str_meter) {
				c = val > 2 ? '#5e6' : (val == 2 ? '#ec6' : '#e66');
				if(this.getValue().length) val ++;
				for(var i = 0; i < 5; i++) {
					e = this.str_meter.childNodes[i];
					e.style.backgroundColor = (i < val) ? c : '#bbb';
				}
			}
		},
		handleFocus: function(evt) {
			this.focused = true;
			this.last_value = this.getValue();
			UI.callEvent(this, this.onfocus, evt);
		},
		handleBlur: function(evt) {
			this.focused = false;
			if(this.opened)
				this.hideSearchDelayed(100);
			UI.callEvent(this, this.onblur, evt);
		},
		getCurrencyInput: function() {
			if(! this.currency_input) {
				if(this.currency_id) {
					var cname = this.currency_id.replace(/_id$/, '');
					this.currency_input = UI.getFormInput(this.form, cname);
					if(! this.currency_input)
						this.currency_input = UI.getFormInput(this.form, this.currency_id);
				}
			}
			return this.currency_input;
		},
		getDecimals: function() {
			var d, cid;
			if(this.format == 'int' || this.format == 'sequence_number') {
				d = 0;
			} else if(this.format == 'currency') {
				d = 2;
				if( (cid = this.getCurrencyInput()) )
					d = cid.getDecimals();
				d = get_default(this.decimals, d);
			} else {
				d = get_default(this.decimals, 0);
			}
			return d;
		},
		getNoFormatNumber: function() {
			return (this.no_format || this.format == 'sequence_number');
		},
		getNumberFormat: function() {
			var ret;
			if(this.numFormat)
				ret = this.numFormat;
			else if(this.format == 'int' || this.format == 'float' || this.format == 'currency' || this.format == 'sequence_number') {
				ret = {
					blankAllowed: this.allow_blank, minimumDecimals: this.min_decimals,
					decimalPlaces: this.getDecimals(), defaultNumValue: this.default_num_value,
					restrictPositive: this.restrict_positive, noFormat: this.getNoFormatNumber()};
				if(this.format == 'currency' && ! isset(ret.minimumDecimals))
					ret.minimumDecimals = ret.decimalPlaces;
			}
			return ret;
		},
		checkNumberInput: function() {
			var numf = this.getNumberFormat(), v, num, upd;
			if(numf) {
				num = validateNumberInput(this.field, numf);
				if(isset(this.min) && ('' + this.min != '') && num < this.min) { num = this.min; upd = 1; }
				else if(isset(this.max) && ('' + this.max != '') && num > this.max) { num = this.max; upd = 1; }
				if(upd)
					this.field.value = this.no_format ? num : stdFormatNumber(num);
				this.numVal = num;
				return true;
			}
		},
		handleChanged: function(evt, silent) {
			if(this.checkNumberInput())
				;
			else if(! this.no_format) {
				if(this.format == 'phone')
					formatPhoneNumber(this.field);
				else if(this.format == 'url')
					formatUrl(this.field);
				else if(this.format == 'email' || this.format == 'hostname')
					this.field.value = trim(this.field.value);
			}
			var v = this.getValue();
			if(this.orig_value !== v) UI.markFormDirty(this);
			if(! silent)
				UI.callEvent(this, this.onchange, evt, v);
			this.validate();
		},
		handleInputCompat: function(evt) {
			if(evt.propertyName == 'value')
				this.handleInput(evt);
		},
		handleInput: function(evt) {
			if(this.show_search) {
				if(! this.opened)
					this.showSearch();
				else
					this.searcher.setQuery(this.getValue());
			}
			UI.callEvent(this, this.oninput, evt);
		},
		handleMouseDown: function(evt) {
			if(this.show_search && ! this.opened)
				UI.stopPropagation(evt);
		},
		handleClick: function(evt) {
			if(this.show_search) {
				if(this.opened)
					this.hideSearch();
				else {
					this.showSearch();
					UI.stopEvent(evt);
				}
			}
		},
		handleKeyDown: function(evt) {
			var nav;
			if(evt.keyCode == 38) nav = -1;
			else if(evt.keyCode == 40) nav = 1;
			else nav = 0;
			if(nav && this.show_search && ! this.disabled) {
				if(this.opened)
					this.searcher.adjustCurrentIndex(nav, null, evt);
				else
					this.showSearch();
				UI.stopEvent(evt);
			} else if(evt.keyCode == 13 && ! this.disabled) {
				if(this.show_search) {
					if(this.opened) {
						if(! this.searcher.acceptCurrent(evt))
							this.hideSearch();
					}  else
						this.showSearch();
					UI.stopEvent(evt);
				} else {
					this.handleChanged(evt);
					if(this.invalid) {
						var summary = UI.validationSummary([{name: this.name, label: this.label, message: this.invalidMsg}]);
						if(summary) alert(summary);
						UI.stopEvent(evt);
					}
					else
						return UI.callEvent(this, this.onreturn, evt);
				}
			} else if(evt.keyCode == 27) {
				if(this.show_search && this.opened) {
					this.hideSearch(true);
					this.setValue(this.search_value);
					this.select();
					UI.stopEvent(evt);
				}
				else if(! this.disabled) {
					if(isset(this.last_value) && this.last_value != this.getValue()) {
						this.setValue(this.last_value);
						this.select();
						UI.stopEvent(evt);
					}
					else if(! this.isBlank()) {
						this.clear();
						UI.stopEvent(evt);
					}
				}
			}
			return UI.callEvent(this, this.onkeydown, evt);
		},
		handleKeyUp: function(evt) {
			return UI.callEvent(this, this.onkeyup, evt);
		},
		checkAltValue: function() {
			var v;
			if(this.altInput && (v = this.altInput.getValue())) {
				return v;
			}
		},
		validate: function() {
			var v = this.getValue();
			this.invalid = false;
			if(this.required && this.isBlank() && ! this.checkAltValue()) {
				this.invalid = true;
				this.invalidMsg = 'required';
			}
			else if(this.format == 'ascii' && ! v.match(/^[\x20-\x7E]*$/)) {
				this.invalid = true;
				this.invalidMsg = 'invalid';
			}
			else if(this.format == 'regexp') {
				var re = new RegExp(this.format_regexp);
				if(!this.getValue().match(re)) {
					this.invalid = true;
					this.invalidMsg = 'invalid';
				}
			}
			else if(this.format == 'email' && v.length && ! v.match(/^\S+@\S+\.[a-zA-Z]+$/)) {
				this.invalid = true;
				this.invalidMsg = 'invalid';
			}
			else if(this.customValidate)
				this.customValidate();
			this.updateDisplay();
			return ! this.invalid;
		},
		formatValue: function(input) {
			var numf = this.getNumberFormat(), num;
			if(numf) {
				num = validateNumber(input, numf);
				input = this.no_format ? num : stdFormatNumber(num);
			}
			return input;
		},
		render: function() {
			if(! this.field) {
				var init_val = this.init_value;
				if(typeof(init_val) == 'number')
					init_val = this.formatValue(init_val);
				var cls = this.className || 'input-text';
				if(this.flat) cls += ' flat';
				if(this.block) cls += ' input-block';
				if(this.ext_class) cls += ' ' + this.ext_class;
				var attrs = {
					type: 'text',
					className: cls,
					id: this.field_id || this.id,
					name: this.name,
					value: init_val};
				var seta = ['size', 'style', 'readOnly', 'disabled', 'placeholder', 'tabIndex'];
				for(var i = 0; i < seta.length; i++)
					if(isset(this[seta[i]])) attrs[seta[i]] = this[seta[i]];
				this.field = createElement2('input', attrs);
				if(this.align) this.field.style.textAlign = this.align;
				this.addHooks();
				this.updateDisplay();
			}
			return this.field;
		},
		focus: function() {
			if(this.field) this.field.focus();
		},
		select: function() {
			if(this.field) {
				this.field.focus();
				this.field.select();
			}
		},
		getValue: function(nativeVal) {
			var ret = this.field ? this.field.value : '';
			if(nativeVal) {
				if(ret === '') return 0;
				if(this.format == 'int')
					ret = parseInt(stdUnformatNumber(ret), 10);
				else if(this.format == 'float' || this.format == 'currency')
					ret = parseFloat(stdUnformatNumber(ret));
			}
			return ret;
		},
		isBlank: function() {
			return ! trim(this.getValue()).length;
		},
		isNumeric: function() {
			return (this.format == 'int' || this.format == 'sequence_number' || this.format == 'float' || this.format == 'currency' || this.format == 'phone');
		},
		setValue: function(val, silent) {
			if(typeof(val) == 'number')
				val = this.formatValue(val);
			if(this.field) {
				if(! isset(val)) val = '';
				this.field.value = val;
				this.checkNumberInput();
			}
			this.validate();
			val = this.getValue();
			if(this.orig_value !== val) UI.markFormDirty(this);
			this.last_value = val;
			if(! silent)
				UI.callEvent(this, this.onchange, null, val);
		},
		clear: function() {
			this.setValue('');
		},
		updateDisplay: function() {
			if(this.altInput)
				UI.setDisplayState(this.field, this.checkAltValue() ? false : true);
			UI.addRemoveClass(this.field, 'invalid', !! this.invalid);
			UI.addRemoveClass(this.field, 'flat-bottom', !! this.opened);
			UI.addRemoveClass(this.field, 'text-number', !! (this.isNumeric() || this.format === 'number'));
		},
		setSize: function(size) {
			this.size = size;
			if(this.field) this.field.size = size;
		},
		setDisabled: function(disabled) {
			if(! isset(disabled)) disabled = 1;
			this.disabled = !! disabled;
			if(this.field) this.field.disabled = this.disabled;
		},
		setReadOnly: function(readonly) {
			if(! isset(readonly)) readonly = 1;
			this.readOnly = !! readonly;
			if(this.field) this.field.readOnly = this.readOnly;
		},
		destroy: function() {
			UI.removeNode(this.elt, true);
			this.elt = null;
		},
		setSearch: function(show, dom_name) {
			this.show_search = show;
			this.search_dom = dom_name;
			if(! show)
				this.hideSearch();
			else if(this.searcher)
				this.searcher.setOptions(dom_name);
		},
		showSearchDelayed: function(delay) {
			this.cancelHideSearch();
			var input = this;
			this.searchTimeout = setTimeout(function() { input.showSearch(); }, delay || this.search_delay || 1000);
		},
		cancelShowSearch: function() {
			if(this.searchTimeout) {
				clearTimeout(this.searchTimeout);
				this.searchTimeout = null;
				return true;
			}
		},
		cancelHideSearch: function() {
			if(this.hideSearchTimeout) {
				clearTimeout(this.hideSearchTimeout);
				this.hideSearchTimeout = null;
				return true;
			}
		},
		searchReturn: function(key, val, mod) {
			if(isObject(val)) val = val._display;
			this.search_key = key;
			if(this.key_name && this.form) {
				var inp = this.form.elements[this.key_name];
				if(inp) inp.value = key;
			}
			this.setValue(val);
			this.last_value = this.search_value;
		},
		initSearcher: function() {
			var self = this;
			this.searcher = new UI.SearchSelect(this.id + '-search',
				{target: this.elt, target_offset: {top: -1, left: 0}, search_blank: this.search_blank,
				 options: this.search_dom, disable_recent: true, allow_custom: true});
			this.searcher.onchange = function(k, v, mod) {
				self.searchReturn(k, v, mod);
				self.select();
			}
			this.searcher.onshow = function() {
				self.opened = true;
				self.updateDisplay();
			}
			this.searcher.onhide = function() {
				self.opened = false;
				self.updateDisplay();
			}
			this.searcher.oncancel = function() {
				self.select();
			}
			this.searcher.onmouseup = function(evt) {
				//self.searchMouseUp(evt);
			}
		},
		showSearch: function() {
			this.cancelShowSearch();
			this.cancelHideSearch();
			var p = UI.PopupManager.currentModal();
			if(p && ! UI.isAncestor(p.elt, this.elt))
				return;
			if(! this.searcher)
				this.initSearcher();
			this.searcher.render();
			var w = UI.getEltRegion(this.elt);
			if(w)
				this.searcher.setWidth(Math.max(w.width - 24, 300) + 'px');
			var v = this.getValue();
			this.search_value = this.last_value;
			this.searcher.setKey(v);
			this.searcher.setQuery(v);
			this.searcher.showDefault();
		},
		hideSearchDelayed: function(delay) {
			this.cancelShowSearch();
			this.cancelHideSearch();
			var input = this;
			this.hideSearchTimeout = setTimeout(function() { input.hideSearch(); }, delay || 100);
		},
		hideSearch: function(refocus) {
			this.cancelShowSearch();
			this.cancelHideSearch();
			if(this.searcher) {
				var v = this.getValue();
				/*if(v != this.prev_value) {
					if((v !== '' && ! this.auto_update) || (v === '' && ! this.allow_clear))
						this.update(this.prev_key, this.prev_value, true);
					else
						this.userUpdate(null, v, ! refocus);
				}*/
				this.searcher.close();
			}
			if(this.focused && refocus) {
				this.focus();
				this.select();
			}
		}
	};


	UI.TextAreaInput = function(id, params) {
		UI.TextInput.call(this, id, params);
	};
	extendClass(UI.TextAreaInput, UI.TextInput, {
		handleKeyDown: function(evt) {
			var ret;
			if(evt.keyCode == 13 && evt.shiftKey) {
				this.handleChanged(evt);
				if(! this.invalid) {
					ret = UI.callEvent(this, this.onreturn, evt);
					if(isset(ret)) return ret;
					UI.stopEvent(evt);
					if(this.form) UI.submitForm(this.form);
				}
			}
			return UI.callEvent(this, this.onkeydown, evt);
		},
		render: function() {
			if(! this.field) {
				var attrs = {className: (this.className || 'input-text'), id: this.field_id || this.id, name: this.name, value: this.init_value, rows: 4, cols: 60};
				var seta = ['rows', 'cols', 'style', 'readOnly', 'disabled', 'placeholder', 'tabIndex'];
				for(var i = 0; i < seta.length; i++)
					if(isset(this[seta[i]])) attrs[seta[i]] = this[seta[i]];
				this.field = createElement2('textarea', attrs);
				this.addHooks();
				this.updateDisplay();
			}
			return this.field;
		}
	});


	UI.QuickText = function(params) {
		this.id = 'quicktxt-'+(++ UI.QuickText.uniq_idx);
		if(params) extendObject(this, params);
	}
	UI.QuickText.uniq_idx = 0;
	UI.QuickText.prototype = {
		init: function(value, onchange, oncancel, validate) {
			this.value = value;
			this.onchange = onchange;
			this.oncancel = oncancel;
			if(validate)
				this.customValidate = validate;
			this.set_up = false;
			this.addHooks();
		},
		initSource: function(source, value, onchange, oncancel, validate) {
			this.elt = source;
			this.init(value, onchange, oncancel, validate);
		},
		setup: function() {
			this.addHooks();
		},
		addHooks: function() {
			if(this.elt) {
				UI.addEventListener(this.elt, 'mousedown', UI.stopPropagation);
				UI.addEventListener(this.elt, 'click', this.handleClick, this);
			}
		},
		handleClick: function(evt) {
			this.showPopup();
			evt.stopEvent();
		},
		getTargetOffset: function() {
			return null;
		},
		getContentElement: function() {
			if(! this.content_elt)
				this.content_elt = createElement2('div', {className: 'quick-input-content'});
			return this.content_elt;
		},
		showPopup: function(timeout, target, offset) {
			var input = this,
				content = this.getContentElement(),
				p = new UI.PopupSlidePanel(this.id,
					{target: target || this.elt, target_offset: offset || this.getTargetOffset(),
					 destroy_on_close: true, content_elt: content});
			p.onclose = function() {
				input.handleClosePopup();
			}
			p.onshow = function() {
				input.field.select();
			}
			p.render();
			if(! this.field) {
				if(this.multiline)
					this.field = new UI.TextAreaInput(this.field_id || this.id+'-input', {label: this.label});
				else
					this.field = new UI.TextInput(this.field_id || this.id+'-input', {format: this.format, decimals: this.decimals, label: this.label});
			}
			if(! this.multiline)
				this.field.setSize(this.size || 16);
			var body = [this.field.render()];
			var buttons = [];
			if(this.multiline || this.show_update) {
				if(! this.update_button)
					this.update_button = new UI.ButtonInput(
						this.update_id || this.id+'-update',
						{icon: 'icon-accept', label: app_string('LBL_UPDATE'),
						 onclick: function(evt) { input.handleReturn(evt); } });
				buttons.push(this.update_button.render());
			}
			if(this.show_reset) {
				if(! this.reset_button)
					this.reset_button = new UI.ButtonInput(
						this.reset_id || this.id+'-reset',
						{icon: 'icon-cancel', label: app_string('LBL_RESET_BUTTON_LABEL'),
						 onclick: function(evt) { input.handleReset(evt); } });
				buttons.push(this.reset_button.render());
			}
			if(buttons.length)
				body.push(createElement2('div', {style: 'margin-top: 0.3em'}, buttons));
			UI.setElementContent(content, body);
			this.field.customValidate = this.customValidate;
			this.field.onreturn = function(evt) { input.handleReturn(evt); return false; }
			this.field.setValue(this.value);
			this.field.validate();

			this.popup = p;
			p.show(timeout);
			this.selecting = true;
		},
		handleReturn: function(evt) {
			if(evt) UI.stopEvent(evt);
			if(this.selecting) {
				this.selecting = false;
				this.value = this.field.getValue();
				this.popup.close();
				UI.callEvent(this, this.onchange);
			}
		},
		handleReset: function(evt) {
			if(this.selecting) {
				this.selecting = false;
				this.value = null;
				this.popup.close();
				if(this.onreset)
					UI.callEvent(this, this.onreset);
				else
					UI.callEvent(this, this.onchange, true);
			}
		},
		handleClosePopup: function() {
			this.popup = null;
			if(this.selecting) {
				this.selecting = false;
				UI.callEvent(this, this.oncancel);
			}
		},
		getValue: function() {
			return this.value;
		},
		validate: function() {
			return true;
		},
		destroy: function() {
			UI.removeNode(this.elt, true);
		}
	};


	UI.HtmlInput = function(id, params) {
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		this.onchange = null;
		this.focused = false;
		if(params) extendObject(this, params);
	}
	UI.HtmlInput.prototype = {
		setup: function() {
			if(this.set_up) return;
			if(! this.elt)
				this.elt = $(this.id);
			if(! this.field && this.form && this.name)
				this.field = this.form.elements[this.name];
			var self = this;
			UI.onInitForm(this.form, function() { self.register(); });
			this.set_up = true;
		},
		handleFocus: function(evt, ed) {
			if(! this.focused) {
				this.focused = true;
				UI.callEvent(this, this.focus, evt, ed);
			}
			this.restorePosition();
		},
		handleBlur: function(evt, ed) {
			if(this.focused) {
				this.focused = false;
				UI.callEvent(this, this.blur, evt, ed);
				if(ed && ed.checkDirty()) UI.markFormDirty(this);
			}
		},
		handleKeyDown: function(evt, ed) {
			UI.markFormDirty(this);
		},
		instanceReady: function(evt, ed) {
			this.ready = true;
			if(this.autofocus) {
				this.focus();
				this.autofocus = false;
			}
			this.restoreContent();
			// reposition containing popup if necessary
			var p = UI.PopupManager.currentModal();
			if(p && p.elt && this.elt) {
				//if editor in popup disable modal mode and set higher z-index to unblock controls
				p.modal = false;
				ed.config.baseFloatZIndex = 100001;

				var chk;
				for(chk = this.elt; chk.parentNode; chk = chk.parentNode) {
					if(chk === p.elt) {
						p.reposition();
						break;
					}
				}
			}
		},
		focus: function() {
			if(! this.ready) this.autofocus = 1;
			else {
				var ed = this.getEditor(); if(ed) ed.focus();
			}
		},
		getEditor: function() {
			if(this.instance) return this.instance;
			if(this.name) return CKEDITOR.instances[this.name];
		},
		insertHtml: function(html) {
			var ed = this.getEditor();
			if(ed) ed.insertHtml(html);
			UI.markFormDirty(this);
		},
		getValue: function() {
			var ed = this.getEditor();
			if(ed) return ed.getData();
		},
		setValue: function(val) {
			var ed = this.getEditor();
			if(ed) ed.setData(val);
			UI.markFormDirty(this);
		},
		updateElement: function() {
			var ed = this.getEditor();
			if(ed) ed.updateElement();
		},
		saveState: function() {
			var ed = this.getEditor();
			ed.focus();
			if(ed) {
				this.bookmark = ed.getSelection().createBookmarks(true);
				this.bookmark_html = ed.document.getBody().getHtml();
			}
		},
		restoreContent: function() {
			var ed = this.getEditor();
			if(ed && isset(this.bookmark_html)) {
				ed.document.getBody().setHtml(this.bookmark_html);
				this.bookmark_html = null;
				//ed.fire('contentDomInvalidated');
			}
		},
		restorePosition: function() {
			var ed = this.getEditor();
			if(ed && this.bookmark) {
				ed.getSelection().selectBookmarks(this.bookmark);
				this.bookmark = null;
			}
		},
		beforeSubmitForm: function() {
			this.updateElement();
		},
		beforeDetachForm: function() {
			this.saveState();
			this.updateElement();
			this.destroy();
		},
		beforeRestoreForm: function() {
			this.register();
		},
		register: function() {
			if(! this.instance && this.field) {
				CKEDITOR.remove(this.field.name);
				var cfg = this.ck_config ? deep_clone(this.ck_config) : {};
				if(window.CKEDITOR_CONFIG)
					cfg.customConfig = window.CKEDITOR_CONFIG;
				this.instance = CKEDITOR.replace(this.field, cfg);
				if(this.instance) {
					var self = this;
					this.instance.on('focus', function(e) { self.handleFocus(e, e.editor); });
					this.instance.on('blur', function(e) { self.handleBlur(e, e.editor); });
					this.instance.on('key', function(e) { self.handleKeyDown(e, e.editor); });
					this.instance.on('instanceReady', function(e) { self.instanceReady(e, e.editor); });
				}
			}
		},
		render: function() {
			if(! this.field && this.name) {
				this.field = createElement2('textarea', {name: this.name, value: this.init_value});
			}
			if(! this.elt) {
				this.elt = createElement2('div', {id: this.id}, this.field);
			}
			return this.elt;
		},
		destroy: function() {
			var inst = this.getEditor();
			if(inst) CKEDITOR.remove(inst);
			this.instance = null;
		}
	}


	UI.DateTimeInput = function(id, params) {
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		this.format = 'datetime';
		this.focused = false;
		this.invalid = false;
		if(params) extendObject(this, params);
	};
	UI.DateTimeInput.prototype = {
		setup: function() {
			if(this.set_up)
				return;
			if(! this.elt)
				this.elt = $(this.id);
			if(! this.elt)
				return;
			if(! this.field) {
				if(this.name && this.form)
					this.field = this.form.elements[this.name];
				else if(this.field_id)
					this.field = $(this.field_id);
			}
			if(! this.input) {
				this.input = Wrap(this.elt).findFirst('.datetime-input').get(0);
			}
			if(! this.altInput && this.alt_name) {
				this.altInput = UI.getFormInput(this.form, this.alt_name);
			}
			this.label_elt = Wrap(this.elt).findFirst('.datetime-label').get(0);
			UI.CalendarInput.initLanguage();
			this.loadValue();
			this.addHooks();
			this.updateDisplay();
			if(this.field) this.orig_value = this.field.value;
			this.set_up = true;
		},
		addHooks: function() {
			if(this.events) return;
			var self = this;
			this.events = UI.addEventListeners(this.input, {
				//dragstart: this.handleDragStart,
				//dragover: this.handleDragOver,
				//drop: this.handleDrop,
				mousedown: this.handleMouseDown,
				click: this.handleClick,
				keydown: this.handleKeyDown,
				focus: this.handleFocus,
				blur: this.handleBlur}, this);
			if(this.altInput) {
				this.altInput.onchange = function() { self.handleChanged(); }
			}
		},
		showCalendar: function() {
			var self = this;
			this.cal_input = new UI.CalendarInput(this.id+'-calendar', {
				format: this.format,
				input_field: this,
				onaccept: function(fdate, date) { self.returnDate(fdate, date); },
				onshow: function() { self.calendarOpened() },
				onhide: function() { self.calendarClosed() },
				oncancel: function() { self.focus(); } });
			this.cal_input.setValue(this.getValue(true));
			this.cal_input.setup();
			this.cal_input.showPopup();
			return false;
		},
		hideCalendar: function() {
			if(this.cal_input) this.cal_input.hidePopup();
		},
		calendarOpened: function() {
			this.opened = true;
			this.updateDisplay();
		},
		calendarClosed: function() {
			this.cal_input = null;
			this.opened = false;
			this.updateDisplay();
		},
		handleKeyDown: function(evt) {
			if(evt.keyCode == 8 || evt.keyCode == 46) { // delete
				if(! this.disabled) this.clear();
				UI.stopEvent(evt);
			}
			if(evt.keyCode == 27) { // esc
				if(this.opened) this.hideCalendar();
				else if(! this.isBlank() && ! this.disabled) this.clear();
				else return;
				UI.stopEvent(evt);
			}
			if(evt.keyCode == 32 || evt.keyCode == 13) { // space/enter
				if(! this.opened && ! this.disabled) this.showCalendar();
				UI.stopEvent(evt);
			}
		},
		handleFocus: function(evt) {
			if(! this.focused) {
				this.focused = true;
				UI.callEvent(this, this.onfocus, evt);
				this.updateDisplay();
			}
		},
		handleBlur: function(evt) {
			if(this.focused) {
				this.focused = false;
				UI.callEvent(this, this.onblur, evt);
				this.updateDisplay();
			}
		},
		handleMouseDown: function(evt) {
			if(! this.disabled)
				evt.stopEvent();
		},
		handleClick: function(evt) {
			if(evt.which == 1) {
				if(this.opened)
					this.hideCalendar();
				else if(! this.disabled)
					this.showCalendar();
				evt.stopEvent();
			}
		},
		handleChanged: function(evt, silent) {
			v = this.getValue();
			if(this.orig_value !== v) UI.markFormDirty(this);
			if(! silent)
				UI.callEvent(this, this.onchange, evt, v);
			this.validate();
			this.updateDisplay();
		},
		returnDate: function(fdate, date) {
			this.setValue(date);
			this.focus();
		},
		clear: function(silent) {
			this.setValue(null, silent);
		},
		isBlank: function() {
			return ! trim(this.getValue()).length;
		},
		validate: function() {
			this.invalid = false;
			if(this.checkAltValue()) {
				// ok
			}
			else if(this.required && this.isBlank()) {
				this.invalid = true;
				this.invalidMsg = 'required';
			}
			else if(this.customValidate)
				this.customValidate();
			this.updateDisplay();
			return ! this.invalid;
		},
		focus: function() {
			if(this.input) this.input.focus();
		},
		loadValue: function() {
			var parsed;
			if(this.field) {
				if(this.format === 'time') {
					parsed = parseTimeString(this.field.value);
				} else {
					parsed = parseDateString(this.field.value);
				}
				this.dateVal = parsed && parsed.d;
			}
		},
		getValue: function(asNative) {
			if(asNative) return this.dateVal;
			return this.field ? this.field.value : '';
		},
		getPrintFormat: function(display) {
			if(this.format === 'date')
				return getDateFormat().print_format;
			else if(this.format === 'time')
				return getTimeFormat().print_format;
			else if(this.format === 'datetime') {
				var fmt = '{DATE} {TIME}';
				if(display) fmt = App.language.getListString('relative_time_dom', 'date_time') || fmt;
				fmt = fmt.replace('{DATE}', getDateFormat().print_format)
					.replace('{TIME}', getTimeFormat().print_format);
				return fmt;
			}
		},
		setValue: function(date, silent) {
			if(date instanceof Date) {
				this.dateVal = date;
				date = date.print(this.getPrintFormat());
			}
			else {
				var n = parseDateString(date);
				if(n) {
					this.dateVal = n.d;
					date = n.display;
				} else {
					this.dateVal = null;
					date = '';
				}
			}
			if(this.field)
				this.field.value = date;
			this.updateDisplay();
			this.handleChanged(null, silent);
		},
		checkAltValue: function() {
			var v;
			if(this.altInput && (v = this.altInput.getValue())) {
				return v;
			}
		},
		updateDisplay: function() {
			UI.addRemoveClass(this.input, 'active', !! (this.focused && !this.opened));
			UI.addRemoveClass(this.input, 'opened', !! this.opened);
			UI.addRemoveClass(this.input, 'invalid', !! this.invalid);
			UI.addRemoveClass(this.input, 'disabled', !! (this.disabled && this.disabled !== 'readonly'));
			UI.addRemoveClass(this.input, 'readonly', !! (this.disabled === 'readonly'));
			if(this.label_elt) {
				var val = this.getValue(true);
				if(val)
					val = val.print(this.getPrintFormat(true));
				else
					val = '';
				UI.setElementText(this.label_elt, val ? val : this.placeholder);
				UI.addRemoveClass(this.label_elt, 'text-placeholder', !val.length && !this.focused && !this.opened);
			}
			if(this.altInput)
				UI.setDisplayState(this.input, ! this.checkAltValue());
		},
		setReadOnly: function(flag) {
			this.setDisabled(get_default(flag, true) ? 'readonly' : flag);
		},
		setDisabled: function(disabled) {
			if(! isset(disabled)) disabled = 1;
			this.disabled = disabled;
			if(disabled) this.hideCalendar();
			this.updateDisplay();
			if(this.disabled && this.disabled !== 'readonly') {
				if(isset(this.input.tabIndex)) {
					this.tabIndex = this.input.tabIndex;
					this.input.removeAttribute('tabIndex');
				}
			} else if(! isset(this.input.tabIndex)) {
				this.input.tabIndex = this.tabIndex || 0;
			}
		},
		render: function() {
			if(! this.rendered) {
				if(! this.field) {
					this.field = MakeElt('input', {
						type: 'hidden', name: this.name,
						id: this.field_id || this.id + '-hidden', value: this.init_value}).get();
				}
				var cls = 'input-field input-field-group rbullet datetime-input';
				if(this.flat) cls += ' flat';
				if(this.block) cls += ' input-block';
				if(this.ext_class) cls += ' ' + this.ext_class;
				var style = (this.no_min_width || this.block) ? '' : 'min-width: 10em';
				var input = MakeElt('div', {
					'class': cls, id: this.id + '-input', style: style, tabindex: this.tabIndex || 0});
				this.input = input.get();
				input.append(MakeElt('div.input-label.input-field-icon.lg')
					.append(MakeElt('div.uii.uii-lg.' + (this.format === 'time' ? 'uii-time' : 'uii-date'))));
				if(! this.label_elt)
					this.label_elt = MakeElt('span.input-label.datetime-label.text-number').get();
				input.append(this.label_elt);
				this.elt = MakeElt('div.input-wrapper' + (this.block ? '.input-block' : ''),
					{id: this.id}).append(this.field).append(input).get();
				if(! this.placeholder)
					this.placeholder = '(' + app_string('NTC_NO_ITEMS_DISPLAY') + ')';
				this.rendered = true;
			}
			return this.elt;
		}
	};


	UI.RelativeTimeInput = function(id, params) {
		this.id = id;
		if(params) extendObject(this, params);
	};
	UI.RelativeTimeInput.prototype = {
		setup: function() {
			if(this.set_up)
				return;
			var self = this;
			this.type_input = UI.getFormInput(this.form, this.name+'_type');
			this.count_input = UI.getFormInput(this.form, this.name+'_count');
			this.time_input = UI.getFormInput(this.form, this.name+'_time');
			if(this.type_input && this.count_input) {
				this.type_input.onchange = function(k, v) {
					self.updateDisplay();
					if(k == 'days_of' && ! self.count_input.getValue())
						self.count_input.focus();
					else if(self.time_input && ! self.time_input.getValue())
						self.time_input.focus();
				}
			}
			this.updateDisplay();
			this.set_up = true;
		},
		updateDisplay: function() {
			var k = this.type_input.getValue();
			if(this.count_input) {
				this.count_input.setDisabled(k == 'days_of' ? false : true);
				UI.setDisplayState(this.count_input.elt, ! this.count_input.disabled);
			}
			if(this.time_input)
				this.time_input.setDisabled(k ? false : true);
		}
	};


	UI.CalendarInput = function(id, params) {
		UI.CalendarInput.initLanguage();
		this.id = id;
		this.auto_close = true;
		this.format = 'date';
		this.mode = 'date';
		if(params) extendObject(this, params);
		if(this.format === 'time')
			this.mode = 'time';
	}
	UI.CalendarInput.initLanguage = function(reload) {
		if(! this.lang_inited || reload) {
			Calendar._DN = app_list_strings('weekdays_long_dom');
			Calendar._SDN = app_list_strings('weekdays_dom');
			var mnsl = app_list_strings('months_long_dom');
			var mns = app_list_strings('months_dom');
			Calendar._MN = [];
			Calendar._SMN = []
			for(var i = 1; i <= 12; i++) {
				Calendar._MN.push(mnsl[i]);
				Calendar._SMN.push(mns[i]);
			}
			Calendar._TT = {
				SEL_DATE: app_string('LNK_SELECT_DATE'),
				PREV_YEAR: app_string('LBL_PREVIOUS_YEAR'),
				PREV_MONTH: app_string('LBL_PREVIOUS_MONTH'),
				TODAY: app_string('LBL_CALENDAR_TODAY'),
				GO_TODAY: app_string('LBL_CALENDAR_TODAY'),
				NEXT_MONTH: app_string('LBL_NEXT_MONTH'),
				NEXT_YEAR: app_string('LBL_NEXT_YEAR'),
				WK: app_string('LBL_WEEK_COL_HEADER'),
				DAY_FIRST: app_string('LBL_CHANGE_FIRST').replace('{NAME}', '%s'),
				PART_TODAY: app_string('LBL_TODAY_SUFFIX')
			}
		}
	}
	UI.CalendarInput.prototype = {
		getCalendarOptions: function() {
			var opts = this.calendar_options ? deep_clone(this.calendar_options) : {};
			if(this.date_format)
				opts.dateFormat = this.date_format;
			return opts;
		},
		setup: function() {
			if(this.set_up) return;
			if(! this.calendar)
				this.calendar = new Calendar(this.getCalendarOptions());
			if(this.attach_target) {
				var attach = $(this.attach_target), self = this;
				if(attach) attach.onclick = function(evt) { self.showPopup(); UI.stopEvent(evt); }
				if(! this.target) this.target = attach;
			}
			if(! this.input_field) {
				if(this.form && this.name) {
					this.input_field = UI.getFormInput(this.form, this.name);
					if(! this.input_field)
						this.input_field = this.form.elements[this.name];
				}
			}
			if(! this.target && this.input_field) {
				if(this.input_field.elt)
					this.target = this.input_field.elt;
				else if(isNode(this.input_field))
					this.target = this.input_field;
			}
			if(! this.search_input) {
				var self = this;
				this.search_input = new UI.SearchInput(this.id + '-text', {
					searcher: this, width: 200,
					show_clear: false,
					onkeydown: function(evt) {
						if(evt && evt.keyCode === 9 && evt.shiftKey) {
							// shift-tab out of search should close dialog
							// (alternative jumps down the page)
							UI.stopEvent(evt);
							self.cancelSearch();
						}
					}});
			}
			this.set_up = true;
		},
		getTargetOffset: function() {
			return this.target_offset || {top: -1};
		},
		getDisplayTransform: function() {
			return this.display_transform || 'scaleY(0.4)';
		},
		getContentElement: function() {
			return this.body_elt && this.body_elt.get();
		},
		focus: function() {
			if(this.search_input) this.search_input.select();
			this.updateDisplay();
		},
		getFormat: function(fmt) {
			if(! fmt) fmt = this.format;
			if(fmt === 'date') {
				return this.date_format || getDateFormat().print_format;
			} else if(fmt === 'time') {
				return this.time_format || getTimeFormat().print_format;
			} else if(fmt === 'datetime') {
				return this.getFormat('date') + ' ' + this.getFormat('time');
			}
		},
		getValue: function(asNative) {
			var d;
			if(this.format === 'time')
				d = this.timeVal;
			else
				d = this.dateVal;
			if(d && this.format === 'datetime') {
				if(this.timeVal)
					d.setHours(this.timeVal.getHours(), this.timeVal.getMinutes(), 0);
				else
					d = null;
			}
			if(asNative) return d;
			if(! d) return '';
			var fmt = this.getFormat();
			return fmt ? d.print(fmt) : '';
		},
		setValue: function(date) {
			if(date instanceof Date) {
				if(this.format === 'datetime' || this.format === 'date')
					this.dateVal = date;
				if(this.format === 'datetime' || this.format === 'time')
					this.timeVal = date;
			}
		},
		returnStatus: function(stat, is_default) {
			UI.callEvent(this, this.onstatus, stat, is_default);
		},
		returnBlank: function() {
			this.dateVal = null;
			this.timeVal = null;
			UI.callEvent(this, this.onaccept, null, null);
			if(this.auto_close && this.popup)
				this.popup.close();
		},
		returnDate: function(fdate, date, accept) {
			this.dateVal = date;
			if(this.format === 'date')
				UI.callEvent(this, this.onchange, fdate, date);
			if(accept)
				this.acceptCurrent();
			else {
				this.initSearchQuery();
				this.focus();
			}
		},
		returnTime: function(ftime, date, accept) {
			this.timeVal = date;
			if(this.format === 'datetime' || this.format === 'time')
				UI.callEvent(this, this.onchange, this.getValue(), this.getValue(true));
			if(accept)
				this.acceptCurrent();
			else {
				this.initSearchQuery();
				this.updateDisplay();
			}
		},
		renderCalendar: function() {
			if(! this.rendered_cal) {
				var self = this;
				this.calendar.hide_status = this.hide_status;
				this.calendar.onstatus = function(cal, stat, def) { self.returnStatus(stat, def); }
				this.calendar.onselect = function(cal, fdate, date) { self.returnDate(fdate, date); }
				this.calendar.onchange = function(cal, fdate, date) { self.returnDate(fdate, date, true); }
				if(this.dateVal)
					this.calendar.setDate(this.dateVal);
				if(! this.calendar.getElement())
					this.calendar.render();
				this.rendered_cal = true;
			}
			return this.calendar.getElement();
		},
		renderTimeEditor: function() {
			if(! this.rendered_time) {
				var editor = new UI.TimeEditor();
				editor.onchange = this.returnTime.bind(this);
				if(this.timeVal)
					editor.setValue(this.timeVal, true);
				editor.render();
				editor.setup();
				this.time_editor = editor;
				this.rendered_time = true;
			}
			return this.time_editor.getElement();
		},
		render: function() {
			if(! this.rendered) {
				var search_row = MakeElt('div.tools-row.popup-search')
						.append(this.search_input.render())
						.append(MakeElt('div.spacer')),
					return_label = MakeElt('span'),
					return_link = MakeElt('a.tools-link', {href: '#'})
						.append(MakeElt('span.uii.uii-previous'))
						.append(nbsp())
						.append(return_label)
						.on('click:stop:prevent', function() { this.setMode('date'); }, {scope: this}),
					return_row = MakeElt('div.card-header.panel-subheader:hidden')
						.append(MakeElt('div.tools-row.popup-title', null, return_link)),
					head = MakeElt('div.card-header.panel-header')
						.append(search_row),
					body = MakeElt('div.card-body.panel-body');
				this.return_label = return_label;
				this.return_row = return_row;
				if(this.mode === 'date')
					body.append(this.renderCalendar());
				else if(this.mode === 'time')
					body.append(this.renderTimeEditor());
				this.body_elt = body;
				this.clear_button = MakeElt('div.uii.uii-lg.uii-remove.active-icon', {tabindex: 0})
					.appendTo(search_row)
					.on('click, keydown[key=enter|space]', function(evt) {
						this.returnBlank();
						this.hidePopup();
					}, {stop: true, prevent: true, scope: this});
				this.time_button = MakeElt('div.uii.uii-lg.uii-time.active-icon:hidden', {tabindex: 0})
					.appendTo(search_row)
					.on('click, keydown[key=enter|space]', function(evt) {
						if(this.dateVal) this.setMode('time');
					}, {stop: true, prevent: true, scope: this});
				this.accept_button = MakeElt('div.uii.uii-lg.uii-accept.active-icon:hidden', {tabindex: 0})
					.appendTo(search_row)
					.on('click, keydown[key=enter|space]', function(evt) {
						this.acceptCurrent(evt);
					}, {stop: true, prevent: true, scope: this});
				var content = MakeElt('div.card-outer.panel-outer.panel-default', null, [head, return_row, body]);
				this.search_input.setup();
				this.elt = content.get();
				this.rendered = true;
				this.updateDisplay();
			}
			return this.elt;
		},
		updateDisplay: function() {
			var show_time = this.format === 'datetime' && this.mode === 'date',
				show_accept = this.format === 'date' || this.format === 'time' || (this.format === 'datetime' && this.mode === 'time');
			this.time_button.toggle(show_time);
			this.time_button.toggleClass('disabled', !this.dateVal);
			this.accept_button.toggle(show_accept);
			this.accept_button.toggleClass('disabled', (this.format !== 'time' && !this.dateVal) || (this.mode === 'time' && !this.timeVal));
			if(this.mode === 'time' && this.format === 'datetime' && this.dateVal) {
				var pfx = App.language.getListString('date_filter_operators_dom', 'on_date') || '';
				this.return_label.text(pfx + ' ' + this.dateVal.print(getDateFormat().print_format));
				this.return_row.show();
			} else {
				this.return_row.hide();
			}
			//this.accept_button.toggle(this.mode === 'date' && this.format === 'datetime');
			if(this.mode === 'time')
				this.time_editor.updateDisplay();
			else
				this.calendar.clearHighlight();
		},
		clearSearch: function() {
			if(this.search_input) this.search_input.clear();
		},
		setQuery: function(q) {
			// called by search input
			if(this.mode === 'date') {
				var parsed = parseDateString(q);
				if(parsed)
					this.dateVal = parsed.d;
				else
					this.dateVal = null;
				this.calendar.setDate(this.dateVal);
				this.updateDisplay();
			}
			else if(this.mode === 'time') {
				var parsed = parseTimeString(q);
				if(parsed)
					this.timeVal = parsed.d;
				else
					this.timeVal = null;
				this.time_editor.setValue(this.timeVal, true);
				this.updateDisplay();
			}
		},
		adjustCurrentIndex: function(offs, nowrap, evt) {
			// called by search input
			if(offs) {
				if(this.mode === 'date') {
					var type = 'day';
					if(evt && evt.shiftKey) type = 'month';
					else if(evt && (evt.ctrlKey || evt.cmdKey)) type = 'week';
					this.calendar.nav(-offs, type);
				}
				else if(this.mode === 'time') {
					var type = 'hour';
					if(evt && evt.shiftKey) type = 'minute';
					else if(evt && (evt.ctrlKey || evt.cmdKey)) type = 'meridiem';
					this.time_editor.adjustPartIndex(type, -offs);
				}
			}
		},
		showDefault: function() {
			// called by search input
		},
		acceptCurrent: function(evt) {
			// called by search input and accept button
			if(this.format === 'datetime') {
				if(this.mode === 'date') {
					this.setMode('time');
					return false;
				}
			}
			var fval = this.getValue(), val;
			if(! fval) return false;
			val = this.getValue(true);
			UI.callEvent(this, this.onaccept, fval, val, evt);
			if(this.auto_close && this.popup)
				this.popup.close();
			return false;
		},
		cancelSearch: function() {
			// called by search input
			this.hidePopup();
			UI.callEvent(this, this.oncancel);
		},
		initSearchQuery: function() {
			var q = '';
			if(this.mode === 'date')
				q = this.dateVal ? this.dateVal.print(getDateFormat().print_format) : '';
			else if(this.mode === 'time')
				q = this.timeVal ? this.timeVal.print(getTimeFormat().print_format) : '';
			if(this.search_input)
				this.search_input.setValue(q, true);
		},
		setMode: function(mode) {
			this.mode = mode;
			if(this.mode === 'time') {
				this.body_elt.content(this.renderTimeEditor());
				if(this.timeVal)
					this.time_editor.setValue(this.timeVal, true);
			} else {
				this.body_elt.content(this.renderCalendar());
			}
			this.initSearchQuery();
			this.updateDisplay();
			this.search_input.select();
		},
		showPopup: function() {
			if(! this.popup) {
				var self = this, cal = this.calendar;
				this.popup = new UI.PopupSlidePanel(this.id + 'popup',
					{content_elt: this.render(), container: this.container,
						target: this.target || this.inputField, target_offset: this.getTargetOffset(),
						tabIndex: 1, destroy_on_close: true,
						customDisplayTransform: this.getDisplayTransform() });
				if(this.attach_target) this.popup.target_active_class = 'active opened';
				this.initSearchQuery();
				this.popup.setWidth(this.width || 300);
				this.popup.keyDown = function(evt) {
					return cal.keyDown(evt);
				}
				this.popup.onshow = function() {
					self.visible = true;
					UI.callEvent(self, self.onshow);
					self.search_input.select();
				}
				this.popup.onhide = function() {
					self.visible = false;
					UI.callEvent(self, self.onhide);
				}
			}
			this.popup.render();
			this.popup.show();
			this.updateDisplay();
		},
		hidePopup: function() {
			if(this.popup) this.popup.close();
		}
	};


	UI.WheelThrottle = function(delay, max_steps, delta_cap) {
		this.delay = delay || 200;
		this.max_steps = max_steps || 3;
		this.delta_cap = delta_cap || 20;
	}
	UI.WheelThrottle.prototype = {
		adjustEvent: function(evt) {
			var now = performance.now();
			evt.deltaX = this.adjustDelta(evt.deltaX);
			evt.deltaY = this.adjustDelta(evt.deltaY);
			evt.deltaZ = this.adjustDelta(evt.deltaZ);
			if(this.lastEvent && this.delay && now - this.lastEvent < this.delay) {
				evt.wheelSkip = true;
			} else
				this.lastEvent = now;
		},
		adjustDelta: function(delta) {
			var ds;
			if(! delta) return 0;
			if(Math.abs(delta) > this.delta_cap)
				delta = this.delta_cap * Math.sign(delta);
			ds = Math.round(this.max_steps * delta / this.delta_cap);
			if(delta && ! ds) ds = Math.sign(delta); // at least 1
			return ds;
		}
	};


	UI.TimeEditor = function() {
		this.part_height = 36;
		this.part_gap = 10;
		this.show_meridiem = getTimeFormat().meridiem;
		this.wheelThrottle = new UI.WheelThrottle();
	}
	UI.TimeEditor.prototype = {
		getElement: function() {
			if(this.elt) return this.elt.get();
		},
		setup: function() {
			var hr_labels, pad = function(val) { val = '' + val; if(val.length == 1) val = '0'+val; return val; };
			this.hour_options = {values: Array.range(24), looped: true};
			this.minute_options = {values: Array.range(60), looped: true};
			this.meridiem_options = {values: ['am', 'pm'], labels: ['AM', 'PM']};
			if(this.show_meridiem) {
				hr_labels = [12];
				hr_labels.pushArray(Array.range(1, 13));
				hr_labels.pushArray(Array.range(1, 12));
			}
			else {
				hr_labels = this.hour_options.values.map(pad);
			}
			this.hour_options.labels = hr_labels;
			this.minute_options.labels = this.minute_options.values.map(pad);
			if(this.init_value)
				this.setIndexes(this.init_value);
		},
		render: function() {
			if(! this.rendered) {
				this.elt = MakeElt('div.row.time-edit-outer');
				this.renderPart('hour');
				this.renderPart('minute');
				if(this.show_meridiem)
					this.renderPart('meridiem');
				this.rendered_time = true;
				this.updateDisplay();
			}
			return this.elt;
		},
		renderPart: function(name) {
			var elt = MakeElt('div.column.time-edit-part.'+name+'-part')
				.appendTo(this.elt)
				.on('wheel', {part_name: name}, this.handleWheel, {scope: this})
				.on('click', {part_name: name}, this.handleClick, {scope: this});
			this[name + '_part'] = elt;
			this[name + '_window'] = MakeElt('div.time-edit-window').appendTo(elt);
			this.makeDragTarget(elt, name);
		},
		handleClick: function(evt) {
			evt.stopEvent();
			var name = evt.data.part_name,
				bounds = this[name + '_part'].bounds(),
				offs;
			if(evt.pageY < bounds.y + bounds.height/3) {
				offs = -1;
			}
			else if(evt.pageY > bounds.y + bounds.height*2/3) {
				offs = 1;
			}
			if(offs)
				this.adjustPartIndex(name, offs);
		},
		updateDisplay: function() {
			this.updateDisplayPart('hour');
			this.updateDisplayPart('minute');
			if(this.show_meridiem)
				this.updateDisplayPart('meridiem');
		},
		getPartOptions: function(name) {
			return this[name + '_options'];
		},
		getPartScroll: function(name) {
			return this[name + '_scroll'] || 0;
		},
		getPartIndex: function(name) {
			return this[name + '_index'] || 0;
		},
		getPartFocus: function(name) {
			var current = this.getPartIndex(name),
				hgt = this.part_height,
				gap = this.part_gap,
				full = hgt + gap,
				voffs = this.getPartScroll(name),
				vidx = voffs / full,
				top = hgt * 0.5 - gap;
			vidx = - (vidx < 0 ? Math.ceil(vidx) : Math.floor(vidx));
			voffs += vidx * full;
			while(voffs > top) {
				voffs -= full;
				vidx --;
			}
			while(voffs + full <= top) {
				voffs += full;
				vidx ++;
			}
			return {index: current + vidx, voffs: voffs};
		},
		getPartScrollIndex: function(name, index) {
			var hgt = this.part_height,
				gap = this.part_gap,
				full = hgt + gap;
			return full * -index;
		},
		fixOptionsOffset: function(opts, offs, inrange) {
			if(! opts || ! opts.values.length || ! isset(offs)) return null;
			if(opts.looped) {
				return modulo(offs, opts.values.length);
			} else {
				if(inrange) {
					if(offs < 0) return 0;
					if(offs >= opts.values.length) return opts.values.length-1;
				} else if(offs < 0 || offs >= opts.values.length) return null;
				return offs || 0;
			}
		},
		updateDisplayPart: function(name) {
			if(! this.label_elts) this.label_elts = {};
			if(! this.label_elts[name]) this.label_elts[name] = [];
			var lbls = this.label_elts[name],
				elt = this[name + '_part'],
				vwindow = elt.findFirst('.time-edit-window'),
				options = this.getPartOptions(name),
				focus = this.getPartFocus(name),
				hgt = this.part_height,
				gap = this.part_gap,
				full = hgt + gap,
				top = focus.voffs + hgt - gap/2,
				vidx = focus.index,
				fidx,
				current = this.fixOptionsOffset(options, vidx),
				lidx = 0;
			elt.cssSize('height', hgt * 3);
			vwindow.cssSize('height', hgt)
				.cssSize('top', hgt);
			while(top + full > 0) {
				vidx --;
				top -= full;
			}
			while(top < hgt * 3) {
				fidx = this.fixOptionsOffset(options, vidx);
				if(fidx !== null) {
					if(lidx === lbls.length) lbls[lidx] = MakeElt('div.time-edit-label').appendTo(elt);
					lbls[lidx].text(options.labels[fidx])
						.cssSize('font-size', hgt * 0.4)
						.cssSize('height', full)
						.cssSize('line-height', full)
						.cssSize('top', top)
						.toggleClass('current', current === fidx)
						.show();
					lidx ++;
				}
				top += full;
				vidx ++;
			}
			for(lidx; lidx < lbls.length; lidx++)
				lbls[lidx].hide();
		},
		setPartIndex: function(name, index, evt, silent) {
			var options = this.getPartOptions(name),
				old_index = this.getPartIndex(name),
				index = this.fixOptionsOffset(options, index, true),
				scroll_pos = this.getPartScroll(name),
				pos1 = this.getPartScrollIndex(name, old_index),
				pos2 = this.getPartScrollIndex(name, index),
				diff = pos1 - pos2,
				total_h;
			if(name === 'meridiem' && diff) {
				var hr = this.hour_index;
				if(index && hr < 12) hr += 12;
				else if(! index && hr >= 12) hr -= 12;
				this.hour_index = hr;
			}
			this[name + '_index'] = index;
			if(name === 'hour' && diff && this.show_meridiem) {
				if(index >= 12) this.setPartIndex('meridiem', 1, evt, true);
				else this.setPartIndex('meridiem', 0, evt, true);
			}
			if(options && options.looped) {
				// take shortest path for scrolling
				total_h = options.values.length * (this.part_height + this.part_gap);
				if(Math.abs(diff) > total_h / 2) {
					if(diff > 0)
						diff -= total_h;
					else
						diff += total_h;
				}
			}
			this[name + '_scroll'] = diff + scroll_pos;
			this.nextStepScroll(name, true);
			if(! silent)
				this.handleChanged(evt);
		},
		adjustPartIndex: function(name, offset, evt, silent) {
			this.setPartIndex(name, this.getPartIndex(name) + offset, evt, silent);
		},
		getValue: function(nativeDate) {
			var hr = this.hour_index,
				min = this.minute_index,
				merid = this.meridiem_index || 0;
			if(! isset(hr) || ! isset(min)) return null;
			var dt = new Date();
			dt.setHours(hr, min);
			if(nativeDate) return dt;
			return dt.print(getTimeFormat().print_format);
		},
		setIndexes: function(val) {
			var hrs = val.getHours(),
				mins = val.getMinutes(),
				merid = 0;
			if(hrs >= 12)
				merid = 1;
			this.setPartIndex('hour', hrs, null, true);
			this.setPartIndex('minute', mins, null, true);
			this.setPartIndex('meridiem', merid, null, true);
		},
		setValue: function(val, quiet) {
			if(typeof(val) === 'string') {
				var parsed = parseTimeString(val);
				if(parsed)
					val = parsed.d;
			}
			if(val instanceof Date) {
				this.setIndexes(val);
				if(! quiet)
					this.handleChanged();
				this.init_value = val;
			}
		},
		handleChanged: function(evt) {
			if(! this.meridiem_index) this.meridiem_index = 0;
			if(! this.hour_index) this.hour_index = 0;
			if(! this.minute_index) this.minute_index = 0;
			UI.callEvent(this, this.onchange, this.getValue(), this.getValue(true));
		},
		handleWheel: function(evt) {
			if(! evt.data) return;
			this.wheelThrottle.adjustEvent(evt);
			var delta = evt.deltaY,
				part_name = evt.data.part_name;
			if(! evt.wheelSkip) {
				while(this.adjustPartIndex(part_name, delta, evt) === false && delta) {
					delta -= Math.sign(delta);
				}
			}
			UI.stopEvent(evt);
		},
		nextStepScroll: function(name, initial) {
			var self = this,
				focus, index, pos1, pos2;
			if(this[name + '_scroll_req']) {
				cancelAnimationFrame(this[name + '_scroll_req']);
				this[name + '_scroll_req'] = null;
			}
			if(name) {
				if(initial) {
					this[name + '_scroll_start'] = this.getPartScroll(name);
					this[name + '_scroll_time'] = performance.now();
				}
				this[name + '_scroll_req'] = requestAnimationFrame(
					function(time) { self.stepScroll(name, time); });
			}
		},
		stepScroll: function(name, time) {
			var pos = this[name + '_scroll_start'],
				start = this[name + '_scroll_time'],
				next;
			if(pos) {
				next = pos * (1 - Math.min(1, (time - start) / 150));
				this[name + '_scroll'] = next;
				this.updateDisplayPart(name);
				if(next) this.nextStepScroll(name);
			}
		},
		ondragstart: function(evt, info) {
			info.scroll_start = this.getPartScroll(info.tag);
			this.nextStepScroll(false);
		},
		ondrag: function(evt, info) {
			var pos = info.scroll_start + info.dragY;
			this[info.tag + '_scroll'] = pos;
			this.updateDisplayPart(info.tag);
		},
		ondragend: function(evt, info) {
			if(info.canceled) {
				this[info.tag + '_scroll'] = info.scroll_start;
				this.updateDisplayPart(info.tag);
				return;
			}
			var focus = this.getPartFocus(info.tag);
			if(focus) {
				this.setPartIndex(info.tag, focus.index, evt);
			}
		}
	};
	extendProto(UI.TimeEditor, UI.DragHandler);


	UI.DurationInput = function(id, params) {
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		this.time_id = this.id + '-time';
		this.onchange = null;
		this.focused = false;
		this.invalid = false;
		if(params) extendObject(this, params);
	};
	UI.DurationInput.prototype = {
		setup: function() {
			if(this.set_up)
				return;
			if(! this.field) {
				if(this.elt)
					this.field = this.elt;
				else if(this.name && this.form)
					this.field = this.form.elements[this.name];
				else if(this.field_id)
					this.field = $(this.field_id);
				else if(this.id)
					this.field = $(this.id);
			}
			if (! this.time_field) {
				this.time_field = $(this.time_id);
			}
			if(! this.all_day_field && this.all_day_name) {
				this.all_day_field = UI.getFormInput(this.form, this.all_day_name);
			}
			if(! this.elt)
				this.elt = this.field;
			if(this.field) this.orig_value = this.field.value;
			this.addHooks();
			this.updateDisplay();
			this.set_up = true;
		},
		addHooks: function() {
			var input = this;
			if(this.time_field) {
				this.time_field.onfocus = function(evt) { return input.handleFocus(evt || window.event); }
				this.time_field.onblur = function(evt) { return input.handleBlur(evt || window.event); }
				this.time_field.onkeydown = function(evt) { input.fieldKeyDown(evt || window.event); }
				this.time_field.onchange = function(evt) { return input.handleChanged(evt || window.event); }
			}
			if(this.all_day_field) {
				this.all_day_field.onchange = function(evt, val) { return input.handleAllDay(evt, val); }
			}
		},
		fieldKeyDown: function(evt) {
			if(evt.keyCode == 27) {
				this.clear();
				UI.stopEvent(evt);
			}
		},
		handleFocus: function(evt) {
			if(! this.focused) {
				this.focused = true;
				UI.callEvent(this, this.onfocus, evt);
			}
		},
		handleBlur: function(evt) {
			if(this.focused) {
				this.focused = false;
				UI.callEvent(this, this.onblur, evt);
			}
		},
		handleAllDay: function(evt, val) {
			this.handleChanged(evt);
			if(! val)
				this.focus();
		},
		handleChanged: function(evt) {
			var v, result;
			if (this.field && this.time_field) {
				var all_day = this.getAllDay();
				if(all_day)
					this.field.value = this.time_field.value = '';
				else {
					result = this.format();
					if (result) {
						this.field.value = result.total_minutes;
						this.time_field.value = result.hours + this.hours_abbrev +' '+ result.minutes + this.mins_abbrev;
					} else {
						this.field.value = this.time_field.value = '';
					}
				}
			}
			v = this.getValue();
			if(this.orig_value !== v) UI.markFormDirty(this);
			UI.callEvent(this, this.onchange, evt, v);
			this.validate();
		},
		setValue: function(value, silent, evt) {
			var hrs, mins;
			if(! isset(value) || value === '') {
				if(this.time_field)
					this.time_field.value = '';
			} else {
				value = Math.round(parseFloat(value));
				if(isNaN(value) || value < 0) value = 0;
				if(this.time_field) {
					hrs = Math.floor(value / 60);
					mins = value - hrs * 60;
					this.time_field.value = hrs + this.hours_abbrev + ' ' + mins + this.mins_abbrev;
				}
			}
			if(this.field) this.field.value = value;
			if(this.orig_value !== ''+value) UI.markFormDirty(this);
			this.validate();
			if(! silent)
				UI.callEvent(this, this.onchange, evt, value);
		},
		clear: function() {
			if (this.field) this.field.value = '';
			if (this.time_field) this.time_field.value = '';
		},
		format: function() {
			var hr, min, temp, val = trim(this.getDisplayValue()),
				pos = -1,
				pat = new RegExp(
					'(.*?)\\s*(:|' + RegExp.escape(this.hours_abbrev) +
					'|' + RegExp.escape(this.mins_abbrev) + ')\\s*(.*)'),
				m = val.match(pat);
			if(! val.length)
				return null;

			if(m) {
				temp = parseFloat(stdUnformatNumber(m[1]));
				if(m[2] == ':' || m[2] == this.hours_abbrev) {
					hr = temp;
					min = parseFloat(stdUnformatNumber(m[3]));
				} else
					min = temp;
			} else {
				temp = parseFloat(stdUnformatNumber(val));
				if(! val.match(/^\d+$/) || (temp > 0 && temp < 10))
					hr = temp;
				else
					min = temp;
			}
			if(isNaN(hr) || hr < 0) hr = 0;
			if(isNaN(min) || min < 0) min = 0;
			else min = Math.round(min);
			var hr_floor = Math.floor(hr);
			min += Math.round((hr - hr_floor) * 60);
			hr = hr_floor;
			if(min >= 60) {
				hr += Math.floor(min / 60);
				min = min % 60;
			}
			var total_min = hr*60+min;
			min = (min < 10) ? '0'+min : min;

			var inp = {hours: hr, minutes: min, total_minutes: total_min};

			return inp;
		},
		isBlank: function() {
			var v = this.getValue(true);
			return ! isset(v) || v == 0;
		},
		validate: function() {
			this.invalid = false;
			var all_day = this.getAllDay();
			if(! all_day && this.required && this.isBlank()) {
				this.invalid = true;
				this.invalidMsg = 'required';
			}
			else if(this.customValidate)
				this.customValidate();
			this.updateDisplay();
			return ! this.invalid;
		},
		focus: function() {
			if(this.getAllDay())
				this.all_day_field.focus();
			else if(this.time_field) {
				this.time_field.focus();
				this.time_field.select();
			}
		},
		getValue: function(nativeVal) {
			var ret = this.field ? this.field.value : '';
			if(nativeVal) {
				ret = (ret === '') ? null : parseFloat(ret || 0);
			}
			return ret;
		},
		getDisplayValue: function() {
			return this.time_field ? this.time_field.value : '';
		},
		getAllDay: function() {
			return this.all_day_field && this.all_day_field.getValue();
		},
		updateDisplay: function() {
			UI.addRemoveClass(this.time_field, 'invalid', this.invalid);
			if(this.time_field)
				this.time_field.disabled = (this.all_day_field && this.all_day_field.getValue());
		}
	};


	UI.DateRangeInput = function(id, params) {
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		this.focused = false;
		if(params) extendObject(this, params);
	}
	UI.DateRangeInput.prototype = {
		setup: function() {
			if(this.set_up)
				return;
			if(! this.elt)
				this.elt = $(this.id);
			var basename = this.name || '';
			if(this.form && basename) {
				if(! this.field)
					this.field = this.form.elements[basename];
			}
			if(! this.elt)
				return;
			UI.CalendarInput.initLanguage();
			this.label_elt = this.elt.firstChild;
			if(! isset(this.elt.tabIndex) || this.elt.tabIndex < 0) this.elt.tabIndex = 0;
			this.addHooks();
			if(this.compat) this.updateLabelCompat();
			this.updateDisplay();
			this.set_up = true;
		},
		reset: function() {
			this.stage = '';
			this.from_mode = this.to_mode = this.result = this.from_op = null;
		},
		showPopup: function() {
			this.popup_count = (this.popup_count || 0) + 1;
			var input = this,
				w = Math.max(275, this.elt.offsetWidth),
				p = new UI.PopupSlidePanel(this.id+'-popup'+this.popup_count, {
					width: w, target: this.elt, target_offset: {top: -1}});
			p.onshow = function() {
				input.opened = true;
				input.updateDisplay();
			}
			p.onhide = function() {
				input.popup = null;
				input.opened = false;
				input.updateDisplay();
			}
			p.render();
			this.popup = p;
			this.renderPopupContent();
			p.show();
		},
		hidePopup: function() {
			if(this.popup) this.popup.hide();
		},
		focus: function() {
			if(this.elt) this.elt.focus();
		},
		addHooks: function() {
			UI.addEventListener(this.elt, 'focus', this.handleFocus, this);
			UI.addEventListener(this.elt, 'blur', this.handleBlur, this);
			UI.addEventListener(this.elt, 'click', this.handleClick, this);
			UI.addEventListener(this.elt, 'keypress', this.handleKeyPress, this);
		},
		handleFocus: function() {
			this.focused = true;
			this.updateDisplay();
		},
		handleBlur: function() {
			this.focused = false;
			this.updateDisplay();
		},
		handleKeyPress: function(evt) {
			if(evt.charCode == 32) {
				UI.preventDefault(evt);
				this.handleClick();
			}
		},
		handleClick: function(evt) {
			if(this.loading || this.disabled) return;
			if(this.opened) this.hidePopup();
			else {
				this.reset();
				this.showPopup();
			}
			evt.stopEvent();
		},
		checkSubmit: function(op_changed) {
			if(this.auto_submit) {
				if(op_changed) {
					var ptype = this.getOperatorParamType();
					if(ptype == 'date' && ! this.date_field.getValue())
						return;
					else if(ptype == 'period' && ! this.period_field.getValue())
						return;
				}
				UI.submitForm(this.form);
			}
		},
		getLabels: function() {
			if(! this.opt_labels) {
				var dom = App.language.getList('date_range_operators_dom'), i, v,
					rdom = App.language.getList('relative_time_dom'),
					copy_rel = ['today', 'tomorrow', 'yesterday', 'now', 'after_today', 'before_today'],
					nodots = ['any', 'empty', 'not_empty', 'plus', 'minus', 'previous', 'next'],
					lbls = {'done': app_string('LBL_DONE_BUTTON_LABEL')};
				if(this.compat) { nodots.remove('previous'); nodots.remove('next'); }
				if(rdom) {
					for(i = 0; i < rdom.keys.length; i++) {
						if(copy_rel.indexOf(rdom.keys[i]) >= 0)
							lbls[rdom.keys[i]] = rdom.values[i];
					}
				}
				if(dom) {
					for(i = 0; i < dom.keys.length; i++) {
						v = dom.values[i];
						if(nodots.indexOf(dom.keys[i]) < 0 && copy_rel.indexOf(dom.keys[i]) < 0) v += '…';
						lbls[dom.keys[i]] = v;
					}
				}
				this.opt_labels = lbls;
			}
			return this.opt_labels;
		},
		getPeriods: function(sing) {
			var opts = {keys: ['day', 'week', 'month', 'quarter', 'year', 'fiscalquarter', 'fiscalyear'],
					values: [], add_blank: false}, i = 0, k, p,
				dom = App.language.getList(sing ? 'interval_unit_dom' : 'interval_units_dom');
			if(this.compat)
				opts.keys.pushArray(['days_7', 'days_14', 'days_30', 'days_60', 'days_90', 'days_180', 'days_365']);
			if(dom) for(; i < opts.keys.length; i++) {
				k = opts.keys[i];
				if(k.substring(0, 5) == 'days_') {
					opts.values[i] = k.substring(5) + ' ' + App.language.getListString('interval_units_dom', 'days');
				} else {
					p = dom.keys.indexOf(k + 's');
					opts.values[i] = (p >= 0) ? dom.values[p] : k;
				}
			}
			return opts;
		},
		getWithinTypes: function() {
			var opts = {values: [], add_blank: false}, i = 0, ls = this.getLabels();
			if(this.stage == 'offset_custom')
				opts.keys = ['plus', 'minus'];
			else
				opts.keys = ['', 'next', 'previous'];
			for(; i < opts.keys.length; i++)
				opts.values[i] = ls[opts.keys[i] || 'range'];
			return opts;
		},
		getPopupTitle: function() {
			var label = null, lbls = this.getLabels();
			if(this.show_field_name)
				label = this.label;
			if(this.stage && lbls[this.stage])
				label = lbls[this.stage];
			if(this.to_mode && lbls[this.to_mode])
				label = lbls[this.to_mode];
			else if(this.from_mode && lbls[this.from_mode])
				label = lbls[this.from_mode];
			return label;
		},
		getBackButton: function() {
			var back, s = this.stage, input = this;
			if(s == 'to_opts' || (this.from_op && s != 'from_opts'))
				back = 'from_opts';
			else if(s)
				back = '';
			if(isset(back)) {
				if(! this.back_button) {
					this.back_button = MakeElt('a.tools-link', {href: '#', style: 'margin-right: 8px'},
						[MakeElt('span.uii.uii-previous'), nbsp(), app_string('LBL_BACK')]);
				}
				this.back_button.on('click', function() { input.goBack(back); return false; });
				return this.back_button;
			}
		},
		renderPopupContent: function() {
			if(! this.popup) return;
			var title_row = MakeElt('div.tools-row.popup-title'),
				title = MakeElt('div.card-header.panel-subheader').append(title_row),
				title_text = this.getPopupTitle(),
				opts = this.getMenuOptions(),
				stat = this.renderStatus(),
				btn = this.getBackButton(),
				content = Wrap(this.popup.elt);
			UI.clearChildNodes(content);
			if(btn) title_row.prepend(btn);
			if(title_text) {
				title_row.append(MakeElt('span.text-right').text(title_text));
				title.appendTo(content);
			}
			if(this.stage == 'from' || this.stage == 'to') {
				content.append(this.renderCalendar());
				this.cal_input.focus();
			}
			if(this.stage == 'from_field' || this.stage == 'to_field') {
				content.append(this.renderFieldList());
			}
			if(stat)
				content.append(stat);
			if(this.stage == 'within' || this.stage == 'offset_custom') {
				content.append(this.renderWithin());
				this.within_inp.focus();
			}
			if(opts)
				content.append(this.renderMenu(opts, this.getMenuSelected()));
			return content.get();
		},
		renderStatus: function() {
			if(! this.result || (this.stage != 'from_opts' && this.stage != 'to_opts' && this.stage != 'within'))
				return;
			var stat = this.getStatusElt().text(this.result.text).show(),
				outer = MakeElt('div.card-header.panel-subheader').append(stat);
			return outer;
		},
		getMenuOptions: function() {
			var keys, lbls = this.getLabels(), dom;
			if(! this.stage) {
				if(this.compat) {
					keys = [
						'any', 'before', 'after', 'before_today', 'after_today', 'date', 'not_date', 'between',
						'empty', 'not_empty',
						'yesterday', 'today', 'tomorrow',
						'previous', 'current', 'next'];
				}
				else {
					keys = [
						'any', 'from', 'from_field', 'before', 'after',
						'empty', 'not_empty', 'offset', 'within', 'current', 'month'
						/*, 'year', 'fiscalyear'*/];
				}
			}
			else if(this.stage == 'before' || this.stage == 'after')
				keys = ['from', 'from_field', 'today', 'start', 'end', 'offset_custom'];
			else if(this.stage == 'offset' || this.stage == 'to_offset')
				keys = ['start', 'end', 'offset_custom'];
			else if(this.stage == 'start' || this.stage == 'end' || this.stage == 'current' || this.stage == 'previous' || this.stage == 'next')
				return this.getPeriods(true);
			else if(this.stage == 'month') {
				keys = []; lbls = [];
				dom = App.language.getList('months_long_dom');
				for(i = 1; i <= 12; i++) {
					keys.push(i);
					lbls.push(dom.values[i]);
				}
			}
			else if(this.stage == 'from_opts') {
				keys = ['done', 'to', 'to_field', 'to_start', 'to_end', 'within', 'to_offset_custom'];
				if(this.from_mode) keys.remove('within');
			}
			else if(this.stage == 'to_opts') {
				keys = ['done'];
			}
			if(keys) return {keys: keys, values: lbls};
		},
		getMenuSelected: function() {
			var filter = this.filter || {operator: 'any'}, sel;
			if(! this.stage) {
				sel = filter.operator;
				if(filter.negate) sel = 'not_' + sel;
			}
			return sel;
		},
		renderMenu: function(options, key) {
			var input = this,
				outer = MakeElt('div.card-outer.card-body.popup-body.filter-options');
			if(! this.menu) {
				this.menu = new UI.SelectList(this.id+'-menu',
					{no_scroll: true, flat: true, blank_key: 'any',
					 require_focus: false});
				this.menu.render();
				this.menu.setup();
				this.menu.onchange = function() {
					input.returnMenu(this.getSelectedKey());
				}
			}
			this.menu.setOptions(options);
			this.menu.setSelectedKey(key, true);
			this.menu.setCurrentIndex(null);
			outer.append(this.menu.elt);
			return outer;
		},
		renderCalendar: function() {
			var input = this;
			if(! this.cal_input) {
				this.cal_input = new UI.CalendarInput(this.id+'-calendar', {format: 'date'});
				//this.cal_input.onstatus = function(stat, def) { input.showCalendarStatus(stat, def); };
				this.cal_input.onaccept = function(fdate, date) { input.returnDate(fdate, date); };
				this.cal_input.setup();
				this.cal_input.render();
			}
			if(this.stage == 'from')
				this.cal_input.setValue(this.from_date);
			else if(this.stage == 'to')
				this.cal_input.setValue(this.to_date ? this.to_date : this.from_date);
			else
				this.cal_input.setValue(null);
			this.cal_input.initSearchQuery();
			this.cal_input.updateDisplay();
			return this.cal_input.render();
		},
		getStatusElt: function() {
			if(! this.status_elt)
				this.status_elt = MakeElt('span.input-label.status-label');
			return this.status_elt;
		},
		renderFieldList: function() {
			var outer = MakeElt('div.card-outer.card-body.popup-body.filter-options'),
				input = this;
			if(! this.field_sel) {
				this.field_sel = new UI.SelectList(this.id+'field', {
					options: this.fields_list, flat: true,
					no_scroll: true, show_checks: true,
					require_focus: false});
				this.field_sel.render();
				this.field_sel.setup();
				this.field_sel.onchange = function() { input.returnField(this.getSelectedKey()); }
			}
			this.field_sel.setSelectedKey(this.stage == 'from_field' ? this.from_field : this.to_field, true);
			this.field_sel.setCurrentIndex(null);
			outer.append(this.field_sel.elt);
			return outer;
		},
		renderWithin: function() {
			var input = this,
				outer = MakeElt('div.card-body.panel-body.panel-pad-body'),
				tout = MakeElt('div', {style: 'margin-bottom: 5px'});
			if(! this.within_inp) {
				this.within_type = new UI.SelectInput(this.id+'wtype', {options: this.getWithinTypes(), show_stacked: true, mark_blank: false});
				this.within_type.setup();
				this.within_type.render();
				this.within_inp = new UI.TextInput(this.id+'within', {format: 'int', restrict_positive: true, size: 5, init_value: 2});
				this.within_inp.onreturn = function() { input.returnWithin(); }
				this.within_inp.setup();
				this.within_inp.render();
				this.within_sel = new UI.SelectInput(this.id+'period', {options: this.getPeriods(), init_value: 'week', show_stacked: true});
				this.within_sel.setup();
				this.within_sel.render();
				this.within_ok = new UI.ButtonInput(this.id+'within-ok', {icon: 'icon-accept', label: app_string('LBL_DONE_BUTTON_LABEL')});
				this.within_ok.onclick = function() { input.returnWithin(); }
				this.within_ok.setup();
				this.within_ok.render();
			}
			this.within_type.setValue(this.stage == 'offset_custom' ? 'plus' : '', true);
			tout.append(this.within_type.elt);
			outer.append([tout, this.within_inp.field, nbsp(), this.within_sel.elt, nbsp(), this.within_ok.elt]);
			return outer;
		},
		showCalendarStatus: function(status, def) {
			if(this.stage != 'from' && this.stage != 'to') return;
			def = !!def;
			toggleDisplay(this.date_input.elt, def);
			toggleDisplay(this.status_elt, ! def);
			UI.setElementText(this.status_elt, status);
		},
		returnMenu: function(key) {
			var op, next;
			if(key == 'not_empty') {
				op = {operator: 'empty', negate: true};
			} else if(key == 'op' || key == 'empty' || key == 'any') {
				op = {operator: key};
			} else if(key == 'date' || key == 'not_date' || key == 'between') {
				this.from_mode = key;
				this.changeStage('from');
			} else if(key == 'previous' || key == 'next') {
				this.changeStage(key);
			} else if(this.stage == 'start' || this.stage == 'end' || this.stage == 'current' || this.stage == 'previous' || this.stage == 'next') {
				if(this.stage == 'current' && key == 'day')
					op = {operator: 'special', value: 'today'};
				else
					op = {operator: this.stage, period: key};
				if(op.period && op.period.substring(0, 5) == 'days_') {
					op.count = parseInt(op.period.substring(5), 10);
					op.period = 'day';
				}
				if(this.from_op)
					op = {operator: 'range', start: this.from_op, end: op};
				else if(this.from_mode)
					op = {operator: this.from_mode, value: op};
				else if((this.stage != 'current' || key == 'day') && ! this.compat) {
					this.from_op = op;
					next = 'from_opts';
				}
			} else if(key == 'from' || key == 'to' || key == 'from_opts' || key == 'to_opts'
			|| key == 'month' || key == 'year' || key == 'offset' || key == 'within'
			|| key == 'start' || key == 'end' || key == 'current' || key == 'offset_custom') {
				this.changeStage(key);
			} else if(key == 'to_start' || key == 'to_end' || key == 'to_offset_custom') {
				this.changeStage(key.substring(3));
				if(key == 'to_offset_custom') this.to_mode = true;
			} else if(key == 'from_field' || key == 'to_field') {
				this.fetchFieldList(key);
			} else if(key == 'before' || key == 'after') {
				if(this.from_op)
					this.to_mode = key;
				else
					this.from_mode = key;
				if(this.compat)
					this.changeStage('from');
				else
					this.changeStage(key);
			} else if(this.stage == 'month' && key) {
				op = {operator: 'month', value: key};
			} else if(key == 'today' || key == 'tomorrow' || key == 'yesterday') {
				op = {operator: 'special', value: key};
				if(this.compat)
					; // ok
				else if(this.from_op)
					op = {operator: 'range', start: this.from_op, end: op};
				else if(this.from_mode)
					op = {operator: this.from_mode, value: op};
				else {
					this.from_op = op;
					next = 'from_opts';
				}
			} else if(key == 'after_today') {
				op = {operator: 'after', value: {operator: 'date', value: key}};
			} else if (key == 'before_today') {
				op = {operator: 'before', value: {operator: 'date', value: key}};
			} else if(key == 'done')
				this.hidePopup();
			if(op)
				this.updateFilter(op, next);
		},
		returnDate: function(fdate, date) {
			var op = {operator: 'date', value: date.print('%Y-%m-%d')}, next;
			if(this.stage == 'from') {
				this.from_date = date;
				next = this.from_mode ? null : 'from_opts';
				if(this.from_mode === 'date' || this.from_mode === 'not_date')
					this.from_op = {operator: this.from_mode, value: op.value};
				else if(this.from_mode === 'between') {
					this.from_op = {operator: 'date', value: op.value};
					next = 'to';
				}
				else
					this.from_op = this.from_mode ? {operator: this.from_mode, value: op} : op;
				if(date) this.updateFilter(this.from_op, next);
			}
			else if(this.stage == 'to') {
				this.to_date = date;
				if(date) this.updateFilter({operator: 'range', 'start': this.from_op, 'end': op});
			}
			if(this.date_input)
				this.date_input.setValue(date, true);
		},
		returnField: function(field) {
			var op = {operator: 'field', value: field}, next;
			if(this.stage == 'from_field') {
				this.from_field = field;
				this.from_op = this.from_mode ? {operator: this.from_mode, value: op} : op;
				next = this.from_mode ? null : 'from_opts';
				this.updateFilter(this.from_op, next);
			}
			else if(this.stage == 'to_field') {
				this.to_field = field;
				this.updateFilter({operator: 'range', 'start': this.from_op, 'end': op});
			}
		},
		returnWithin: function() {
			var count = this.within_inp.getValue(true),
				period = this.within_sel.getValue(),
				type = this.within_type.getValue(),
				offs = (this.stage == 'offset_custom'),
				op = {operator: offs ? 'offset' : (type ? type : 'within'), count: count, period: period, value: this.from_op};
			if(offs && count && period) {
				if(type == 'minus') op.count = - op.count;
				if(this.to_mode) {
					op.value = this.to_op;
					op = {operator: 'range', start: this.from_op, end: op};
					this.updateFilter(op);
				} else {
					if(this.from_mode) op = {operator: this.from_mode, value: op};
					this.from_op = op;
					this.updateFilter(op, this.from_mode ? null : 'from_opts');
				}
			}
			else if(count && period)
				this.updateFilter(op);
		},
		updateFilter: function(filter, stage) {
			var input = this, params, req;
			this.hidePopup();
			if(! stage) this.focus();
			this.setLoading(true);
			this.do_confirm = !! stage;
			this.stage = stage;
			this.filter = filter;
			params = {context: this.context, filter: filter, bean_name: this.bean_name, field_name: this.field_name || this.base_name || this.name};
			req = new App.conn.JSONRequest('compact_date_filter', {status_msg: null}, params);
			req.fetch(function(result) { var data = result.getResult(); input.returnCompact(data); });
		},
		fetchFieldList: function(stage) {
			var input = this, params, req;
			if(this.fields_list)
				return this.changeStage(stage);
			this.stage = stage;
			this.hidePopup();
			this.setLoading(true);
			params = {module: this.module, model: this.bean_name, form_name: this.form_name,
					link_name: '', link_alias: '', context: this.context};
			req = new App.conn.JSONRequest('get_list_fields', {status_msg: null}, params);
			req.fetch(function(result) { var data = result.getResult(); input.returnFieldList(data); });
		},
		setLoading: function(state) {
			state = !!state;
			this.loading = state;
			toggleDisplay(this.label_elt, ! state);
			if(state && ! this.wait_icon) {
				this.wait_icon = App.util.waitIcon();
				this.elt.appendChild(this.wait_icon);
			}
			toggleDisplay(this.wait_icon, state);
		},
		changeStage: function(stage) {
			this.stage = stage;
			if(this.popup) {
				this.renderPopupContent();
				this.popup.reposition();
			}
		},
		goBack: function(to) {
			if(! to) this.reset();
			this.changeStage(to);
		},
		returnCompact: function(data) {
			this.setLoading(false);
			if(! data || ! data.result) {
				console.error('Error parsing date range', data);
				this.invalid = true;
				this.updateDisplay();
				return;
			}
			this.result = data.result;
			this.parsed = this.result.filter;
			if(this.compat) {
				this.setValueCompat(this.parsed);
			} else {
				this.setValue(this.result.encoded);
				UI.setElementText(this.label_elt, this.result.text);
			}
			if(this.do_confirm) {
				this.do_confirm = false;
				this.showPopup();
			}
		},
		returnFieldList: function(data) {
			var fs, i, k, opts = {keys: [], values: []};
			this.setLoading(false);
			if(! data || ! data.result) {
				this.invalid = true;
				this.updateDisplay();
				return;
			}
			fs = this.fields_result = data.result;
			for(i = 0; i < fs.length; i++) {
				if(fs[i].type == 'date' || fs[i].type == 'datetime') {
					k = fs[i].name || fs[i].field_name;
					if(k == this.base_name) continue;
					opts.keys.push(k);
					opts.values.push(fs[i].label);
				}
			}
			this.fields_list = opts;
			this.showPopup();
		},
		getValue: function() {
			if(this.compat) {
				return this.form.elements[this.name + '-operator'].value;
			}
			if(this.field) return this.field.value;
		},
		setValue: function(val) {
			if(this.field) this.field.value = val;
		},
		setValueCompat: function(val) {
			if(! val) val = {};
			var fop = 'any', fval = '', fend = '', fperiod = '',
				inp_op = this.form.elements[this.name + '-operator'],
				inp_val = this.form.elements[this.name],
				inp_end = this.form.elements[this.name + '-end'],
				inp_period = this.form.elements[this.name + '-period'];
			if(! val.operator || val.operator === 'any') {
				fop = '';
			}
			else if(val.operator === 'empty') {
				fop = val.operator;
				if(val.negate) fop = 'not_' + fop;
			}
			else if(val.operator === 'date') {
				fop = 'on_date';
				if(val.negate) fop = 'not_' + fop;
				fval = val.value;
			}
			else if(val.operator === 'special') {
				fop = val.value;
			}
			else if(val.operator === 'before' || val.operator === 'after') {
				fop = val.operator + '_date';
				fval = val.value.value;
			}
			else if(val.operator === 'range') {
				fop = 'between_dates';
				fval = val.start.value;
				fend = val.end.value;
			}
			else if(val.operator === 'current' || val.operator === 'previous' || val.operator === 'next') {
				fop = 'period_' + (val.operator === 'previous' ? 'prev' : val.operator);
				fperiod = ('' + val.period).replace('fiscal', 'fiscal_');
				if(fperiod === 'day' && val.count && val.count != 1)
					fperiod = 'days_' + val.count;
			}
			if(fval && fval != 'tomorrow' && fval != 'yesterday')
				fval = parseDateString(fval).d.print(getDateFormat().print_format);
			if(fend)
				fend = parseDateString(fend).d.print(getDateFormat().print_format);
			inp_op.value = fop;
			inp_val.value = fval;
			if(fval == 'tomorrow' || fval == 'yesterday')
				inp_val.value = 'Today';
			inp_end.value = fend;
			inp_period.value = fperiod;
			this.updateLabelCompat();
		},
		updateLabelCompat: function(val) {
			var inp_op = this.form.elements[this.name + '-operator'],
				inp_val = this.form.elements[this.name],
				inp_end = this.form.elements[this.name + '-end'],
				inp_period = this.form.elements[this.name + '-period'],
				pfx, lbl = '', dom = 'date_range_operators_dom';
			op = inp_op.value;
			if(! op || op === 'any')
				lbl = App.language.getListString(dom, 'any');
			else if(op === 'empty' || op === 'not_empty')
				lbl = App.language.getListString(dom, op);
			else if(op === 'today' || op === 'tomorrow' || op === 'yesterday' || op === 'after_today' || op === 'before_today')
				lbl = App.language.getListString('date_filter_operators_dom', op);
			else if(op === 'on_date' || op === 'not_on_date') {
				lbl = inp_val.value;
				if(op === 'not_on_date') {
					pfx = App.language.getListString(dom, 'not');
					lbl = [MakeElt('span.text-muted.small', pfx + app_string('LBL_SEPARATOR')), lbl];
				}
			}
			else if(op === 'between_dates') {
				pfx = App.language.getListString(dom, 'from_pfx');
				var end = App.language.getListString('date_range_embed_dom', 'to');
				lbl = [MakeElt('span.text-muted.small', pfx + app_string('LBL_SEPARATOR')), inp_val.value,
					MakeElt('span.text-muted.small', ' ' + end + app_string('LBL_SEPARATOR')), inp_end.value];
			}
			else if(op === 'before_date' || op === 'after_date') {
				pfx = App.language.getListString(dom, op === 'before_date' ? 'before_pfx': 'after_pfx');
				lbl = [MakeElt('span.text-muted.small', pfx + app_string('LBL_SEPARATOR')), inp_val.value];
			}
			else if(op === 'period_current' || op === 'period_next' || op === 'period_prev') {
				pfx = op.replace('period_', '');
				if(pfx == 'prev') pfx = 'previous';
				pfx = App.language.getListString(dom, pfx);
				var pd = inp_period.value.replace('fiscal_', 'fiscal');
				if(pd.substring(0, 5) === 'days_') {
					lbl = pd.substring(5) + ' ' + App.language.getListString('interval_units_dom', 'days');
				} else {
					lbl = App.language.getListString('interval_unit_dom', pd + 's');
				}
				lbl = [MakeElt('span.text-muted.small', pfx + app_string('LBL_SEPARATOR')), lbl];
			}
			UI.setElementContent(this.label_elt, lbl);
		},
		updateDisplay: function() {
			var val = this.getValue(),
				ph = !val || val === 'any';
			UI.addRemoveClass(this.elt, 'active', !! (this.focused && ! this.opened));
			UI.addRemoveClass(this.elt, 'opened', !! this.opened);
			UI.addRemoveClass(this.elt, 'invalid', !! this.invalid);
			UI.addRemoveClass(this.label_elt, 'text-placeholder', !this.focused && !this.opened && ph);
		}
	};


	UI.TextFilterInput = function(id, params) {
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		this.invalid = false;
		if(params) extendObject(this, params);
	}
	UI.TextFilterInput.prototype = {
		setup: function() {
			if(this.set_up)
				return;
			if(! this.elt)
				this.elt = $(this.id);
			if(! this.elt)
				return;
			var basename = this.name;
			if(this.form && basename) {
				if(! this.value_field)
					this.value_field = this.form.elements[basename];
				if(! this.operator_field)
					this.operator_field = this.form.elements[basename + '-operator'];
				if(! this.input_elt)
					this.input_elt = Wrap(this.elt).findFirst('.input-field').get(0);
			}
			this.addHooks();
			this.updateDisplay();
			this.set_up = true;
		},
		addHooks: function() {
			if(! this.input_elt) return;
			Wrap(this.input_elt)
				.on('mousedown', '.input-label', function(evt) { evt.preventDefault(); /* prevent click issues */})
				.on('mousedown', this.handleMouseDown, {scope: this})
				.on('click', this.handleClick, {scope: this})
				.on('keydown', this.handleKeyDown, {scope: this})
				.on('focus', this.handleFocus, {scope: this})
				.on('blur', this.handleBlur, {scope: this});
		},
		getOperator: function() {
			return this.operator_field && this.operator_field.value;
		},
		getFilterValue: function() {
			var val = trim(this.value_field && this.value_field.value);
			if(! val.length) val = null;
			return val;
		},
		focus: function() {
			this.input_elt.focus();
		},
		handleKeyDown: function(evt) {
			var nav;
			if(evt.keyCode == 38) nav = -1;
			else if(evt.keyCode == 40) nav = 1;
			else nav = 0;
			if(nav) {
				if(this.opened && this.mode === 'operator')
					this.operator_sel.adjustCurrentIndex(nav, null, evt);
				else if(! this.disabled)
					this.showPopup();
				UI.stopEvent(evt);
			} else if(evt.keyCode == 13) {
				if(this.opened) {
					this.acceptInput();
				} else if(! this.disabled)
					this.showPopup();
				UI.stopEvent(evt);
			} else if(evt.keyCode == 8 || evt.keyCode == 46) {
				// backspace/delete
				if(! this.disabled) this.clear();
			} else if(evt.keyCode == 27) {
				// esc
				if(this.opened) {
					this.hidePopup();
					this.focus();
					UI.stopEvent(evt);
				} else if(! this.disabled) {
					if(this.clear()) UI.stopEvent(evt);
				}
			} else if(evt.keyCode == 32) {
				if(! this.opened && ! this.disabled) {
					this.showPopup();
					UI.stopEvent(evt);
				}
			}
		},
		handleMouseDown: function(evt) {
			if(! this.disabled) {
				if(evt.currentTarget === this.input_elt)
					evt.stopPropagation();
			}
		},
		handleClick: function(evt) {
			if(evt.which == 1) {
				if(this.opened)
					this.hidePopup();
				else if(! this.disabled)
					this.showPopup();
				UI.stopEvent(evt);
			}
		},
		handleFocus: function(evt) {
			this.focused = true;
			this.updateDisplay();
		},
		handleBlur: function(evt) {
			this.focused = false;
			this.updateDisplay();
		},
		switchMode: function(mode, repos) {
			var op = this.operator_next;
			this.mode = mode;
			this.return_row.toggle(mode === 'values');
			this.value_div.toggle(mode === 'values');
			this.list_div.toggle(mode === 'operator');
			this.value_label.text(this.operatorLabel(op));
			if(repos) {
				if(mode === 'values')
					this.value_input.select();
				else
					this.input_elt.focus();
				this.popup.reposition();
			}
		},
		returnValue: function(name, val) {
			if(name === 'operator') {
				if(val && val !== 'empty' && val !== 'not_empty') {
					this.operator_next = val;
					this.switchMode('values', true);
				} else {
					this.operator_field.value = val || '';
					this.value_field.value = '';
					this.hidePopup();
					this.updateDisplay();
					this.focus();
				}
			}
			else if(name === 'value') {
				this.value_next = val;
			}
		},
		acceptInput: function() {
			var op = this.operator_next;
			if(this.opened) {
				if(this.mode === 'operator')
					this.operator_sel.selectCurrent();
				else {
					if(op && op !== 'empty' && op !== 'not_empty') {
						if(! trim(this.value_next).length)
							return this.returnValue('operator', '');
					} else {
						this.value_next = '';
					}
					this.operator_field.value = op;
					this.value_field.value = get_default(this.value_next, '');
					this.hidePopup();
					this.updateDisplay();
					this.focus();
				}
			}
		},
		clear: function() {
			var ret = this.operator_field.value;
			this.operator_field.value = '';
			this.value_field.value = '';
			if(this.opened) {
				this.hidePopup();
				this.focus();
			}
			this.updateDisplay();
			return ret;
		},
		operatorLabel: function(op) {
			if(op === undefined) op = this.getOperator();
			return App.language.getListString('text_filter_operators_dom', op, 'app') || '';
		},
		getPopupContent: function() {
			var self = this;
			if(! this.operator_sel) {
				this.operator_sel = new UI.SelectList(this.id + '-selop',
					{options: 'text_filter_operators_dom', flat: true});
				this.operator_sel.render();
				this.operator_sel.onchange = function() {
					self.returnValue('operator', this.getSelectedKey());
				}
			}
			if(! this.value_input) {
				this.value_input = new UI.TextInput(this.id + '-nval',
					{tabIndex: 0, init_value: this.value_field.value});
				this.value_input.render();
				this.value_input.onchange = function() {
					self.returnValue('value', this.getValue());
				}
				this.value_input.onreturn = function() {
					self.acceptInput();
				}
			}
			if(! this.value_label) {
				this.value_label = MakeElt('h5.form-label.no-margin');
				this.tools_row = MakeElt('div.row.tools-row')
					.append(this.value_label)
					.append(MakeElt('div.uii.uii-lg.active-icon.uii-remove', {tabindex: 0})
						.on('click, keydown[key=enter|space]', function() {
							self.clear();
						}))
					.append(MakeElt('div.uii.uii-lg.active-icon.uii-accept', {tabindex: 0})
						.on('click, keydown[key=enter|space]', function() {
							self.acceptInput();
						}));
			}
			var return_link = MakeElt('a.tools-link', {href: '#'},
				MakeElt('span.uii.uii-previous'), nbsp(), app_string('LBL_FILTER_SELECT_OPERATOR'))
				.on('click', function() { self.switchMode('operator', true); return false; });
			this.return_row = MakeElt('div.card-header.panel-subheader')
				.append(MakeElt('div.tools-row.popup-title', return_link));
			this.list_div = MakeElt('div.card-body.panel-body')
				.append(this.operator_sel.elt);
			this.value_div = MakeElt('div.card-body.panel-body.panel-pad-body')
				.append(this.tools_row)
				.append(MakeElt('div.row.form-row')
					.append(MakeElt('div.column.form-cell').append(
						MakeElt('div.form-entry').append(
							MakeElt('div.form-value').append(this.value_input.field)))));
			return [this.return_row, this.list_div, this.value_div];
		},
		showPopup: function() {
			var self = this,
				content = this.getPopupContent(),
				op = this.getOperator(),
				p = new UI.PopupSlidePanel(this.id+'-popup',
					{target: this.input_elt, target_offset: {top: -1},
					 destroy_on_close: true,
					 min_width: 200});
			p.onshow = function() {
				self.opened = true;
				self.updateDisplay();
				if(self.mode === 'values')
					self.value_input.select();
				else
					self.input_elt.focus();
			}
			p.onclose = function() {
				self.opened = false;
				self.popup = null;
				self.updateDisplay();
			}
			p.render();
			Wrap(p.elt).content(content);
			this.operator_sel.setup();
			this.operator_sel.setSelectedKey(this.operator_next = op, true);
			this.value_input.setup();
			this.value_input.setValue(this.value_next = this.getFilterValue(), true);
			this.popup = p;
			if(op && op !== 'empty' && op !== 'not_empty')
				this.switchMode('values');
			else
				this.switchMode('operator');
			p.show();
		},
		hidePopup: function() {
			if(this.popup) this.popup.close();
		},
		updateLabel: function() {
			var op = this.getOperator(), opers, val, lbl,
				label_elt = Wrap(this.elt).findFirst('.input-label').ext(0);
			if(label_elt) {
				label_elt.empty();
				lbl = op === 'eq' ? '' : this.operatorLabel(op);
				if(op && op !== 'empty' && op !== 'not_empty') {
					val = trim(this.getFilterValue());
					if(! val.length)
						lbl = this.operatorLabel(op = '');
					else if(lbl) {
						label_elt.append(MakeElt('span.text-muted.small', lbl + app_string('LBL_SEPARATOR')));
						lbl = val;
					} else
						lbl = val;
				}
				if(lbl.length)
					label_elt.append(lbl);
				label_elt.toggleClass('text-placeholder', ! op);
			}
		},
		updateDisplay: function() {
			UI.addRemoveClass(this.input_elt, 'active', !! (this.focused || this.opened));
			UI.addRemoveClass(this.input_elt, 'opened', !! this.opened);
			UI.addRemoveClass(this.input_elt, 'invalid', this.invalid);
			this.updateLabel();
		}
	};


    UI.SelectFilterInput = function(id, params) {
        if(isElement(id)) {
            this.elt = id;
            this.id = this.elt.id;
        } else
            this.id = id;
        this.focused = false;
        this.invalid = false;
        this.menu_id = this.id + '-popup';
        this.mark_blank = true;
		var options;
		if(params && params.options) {
			options = params.options;
		} else
			options = '';
		if(params) extendObject(this, params);
		if(! this.menu) {
			this.menu = new UI.SelectList(this.menu_id, {
				options: options, popup: true, show_checks: this.show_checks,
				width: this.options_width,
				className: this.popup_class_name, icon_key: this.icon_key,
				disabled_indexes: this.disabled_indexes || [],
				mark_blank: this.mark_blank});
		}
		if (this.fixed_operator) {
			this.operator = this.fixed_operator;
		}
    };
    UI.SelectFilterInput.prototype = {
        setup: function() {
            if(this.set_up)
                return;
            if(! this.elt)
                this.elt = $(this.id);
            if(! this.elt)
                return;
			if(! this.input_elt)
				this.input_elt = Wrap(this.elt).findFirst('.input-field').get(0);
            var basename = this.name ? this.name.replace(/-filter$/, '') : '';
            var baseid = this.id.replace(/-filter$/, '');
            if(this.form && basename) {
                if(! this.field)
                    this.field = this.form.elements[basename];
                if(! this.operator_field)
                    this.operator_field = this.form.elements[basename + '-operator'];
            }
            this.addHooks();
            this.updateDisplay();
            this.set_up = true;
            if(this.getOperatorParamType() == 'multi_select')
            	setTimeout(this.reRenderField.bind(this), 50);
        },
        addHooks: function() {
			if(this.events) return;
			var input = this;
			this.events = UI.addEventListeners(this.elt, {
				keydown: this.handleKeyDown,
				keyup: this.handleKeyUp,
				mousedown: this.handleMouseDown,
				click: this.handleClick,
				focus: this.handleFocus,
				blur: this.handleBlur}, this);
        },
		handleMouseDown: function(evt) {
			if(! this.disabled && evt.currentTarget === this.elt)
				evt.stopPropagation();
		},
		handleClick: function(evt) {
			if(evt.which == 1) {
				if(this.opened)
					this.hideSearch();
				else if(! this.disabled)
					this.showSearch();
				UI.stopEvent(evt);
			}
		},
        getOperator: function() {
            return this.operator_field && this.operator_field.value || '';
        },
        getOperatorParamType: function() {
            var oper = this.getOperator();
            if(! oper || oper === 'empty' || oper === 'not_empty')
                return '';
            if (oper === 'any_of' || oper === 'not_any_of')
                return 'multi_select';
            return 'select';
        },
        reRenderField: function() {
            var ptype = this.getOperatorParamType(),
            	op = this.getOperator(),
            	render = false,
            	multi = false,
            	wrapper = this.input_elt && Wrap(this.input_elt).closest('.input-wrapper').get(0),
            	value_field;
            if(! wrapper) return;

            if (ptype === 'multi_select' && ! this.multi_sel) {
                render = true;
                multi = true;
            } else if (ptype !== 'multi_select' && this.multi_sel) {
                render = true;
            }

            if (render) {
                var params = this.initRenderParams(multi);

				UI.clearChildNodes(this.input_elt);
                if (multi) {
                    this.multi_sel = new UI.SelectList(this.input_elt, params);
                    UI.addClass(this.input_elt, 'select-input');
                    this.multi_sel.render();
                    this.multi_sel.setup();
                } else {
                	this.multi_sel.destroy();
                	this.multi_sel = null;
                	MakeElt('div.input-label.filter-label').appendTo(this.input_elt);
                	this.input_elt.style.width = null;
                }

				UI.setElementContent(wrapper, [
                	this.operator_field,
                	this.field,
					this.input_elt,
				]);
                this.addHooks(true);
            }

            if(this.multi_sel) {
				this.multi_sel.header_text = this.getOperator() === 'not_any_of' ? this.translateOperator('not_any_of') : '';
				this.multi_sel.updateDisplay();
			}
        },
        initRenderParams: function(multi) {
            var params = {}, i, width, val;
            params.options = this.getBaseOptions(multi);
            val = (params.field ? params.field.value : this.init_value) || '';
			if(! multi && (i = val.indexOf('^,^')) >= 0) {
				val = val.substring(0, i);
			}
			if(params.field) params.field.value = val;
			else params.init_value = val;

            if (multi) {
            	params.add_remove = true;
            	params.disable_current = true;
            	params.disable_events = true;
                params.field = this.field;
                params.multi_select = true;
                params.name = this.name;
                params.rows = this.display_rows || 4;
                //params.className = 'input-field input-field-group';
				if (params.options && params.options.maxlen) {
					width = params.options.maxlen;
				} else {
					width = 28;
				}
				width = (width + 5.5) * 0.5;
				params.width = Math.round(width * 100) / 100 + 'em';
            } else {
                params.show_checks = true;
                //params.className = 'input-select compact';
            }

            return params;
        },
        getBaseOptions: function(for_multi) {
            return this.menu.getOptions();
        },
        handleFocus: function() {
            this.focused = true;
            this.updateDisplay();
        },
        handleBlur: function() {
            this.focused = false;
            this.updateDisplay();
        },
		focus: function() {
			this.input_elt.focus();
		},
		initSearcher: function() {
			var self = this,
				search_opts = {
					allow_clear: true,
					disable_recent: true,
					target: this.elt, target_offset: {top: -1, left: 0},
					form: this.form,
					options: this.menu.getOptions(),
					search_blank: true,
					matching_only: true,
					fixed_operator: this.fixed_operator
				};
			this.searcher = new UI.SearchSelectFilter(this.id + '-search', search_opts);
			this.searcher.onchange = function(k, v, oper, keepOpen) {
				if(self.operator_field !== oper) {
					self.field.value = '';
					self.operator_field.value = oper;
					self.value_updated = true;
				}
				self.reRenderField(keepOpen);
				if(self.multi_sel) {
					self.multi_sel.addOption(k, v);
					self.value_updated = true;
					if(keepOpen) {
						this.setSourceKeys(self.multi_sel.getKeys());
						this.reposition();
					} else if(! self.checkAutoSubmit()) {
						self.focus();
					}
					self.updateDisplay();
				}
				else {
					self.field.value = k;
					self.value_updated = true;
					if(! self.checkAutoSubmit())
						self.focus();
					self.updateDisplay();
				}
			}
			this.searcher.onremove = function(k, v, oper, keepOpen) {
				if(self.operator_field !== oper) {
					self.field.value = '';
					self.operator_field.value = oper;
					self.value_updated = true;
				}
				if(self.multi_sel) {
					self.multi_sel.removeOption(k);
					self.value_updated = true;
					if(keepOpen) {
						this.setSourceKeys(self.multi_sel.getKeys());
						this.reposition();
					} else if(! self.checkAutoSubmit()) {
						self.focus();
					}
				} else {
					self.focus();
				}
				self.updateDisplay();
			}
			this.searcher.onshow = function() {
				self.opened = true;
				self.updateDisplay();
			}
			this.searcher.onhide = function() {
				self.opened = false;
				self.updateDisplay();
				self.checkAutoSubmit();
			}
			this.searcher.onclear = function() {
				self.operator_field.value = '';
				self.field.value = '';
				self.value_updated = true;
				self.reRenderField();
				self.checkAutoSubmit();
			}
			this.searcher.oncancel = function() {
				self.focus();
			}
			this.searcher.setup();
		},
		checkAutoSubmit: function() {
			if(this.auto_submit && this.value_updated) {
				UI.submitForm(this.form);
				return true;
			}
		},
		showSearch: function(query) {
			if(! this.searcher) {
				this.initSearcher();
			}
			this.searcher.render();
			var w = UI.getEltRegion(this.elt);
			if(w)
				this.searcher.setWidth(Math.max(w.width - 24, 300) + 'px');
			this.searcher.setOperator(this.getOperator() || 'eq');
			this.searcher.setSourceKeys(this.multi_sel ? this.multi_sel.getKeys() : []);
			this.searcher.setKey(this.field.value);
			this.searcher.showDefault(true);
		},
		hideSearch: function() {
			if(this.searcher)
				this.searcher.close();
		},

		translateOperator: function(op) {
    		return App.language.getListString('select_filter_operators_dom', op);
    	},
		updateLabel: function() {
			if(this.getOperatorParamType() === 'multi_select') return;
			var label_elt = Wrap(this.input_elt).findFirst('.input-label').ext(0),
				label_key = this.label_key || 'name',
				op = this.getOperator(),
				val, txt, ph = ! op;
			if(! label_elt) return;
			if(this.getOperatorParamType() === '') {
				label_elt.text(this.translateOperator(op));
			} else {
				var lbl_key = this.label_key || 'name';
				val = this.menu.options.getValue(this.field.value);
				if(val && typeof(val) === 'object') val = val[label_key];
				txt = val || '';
				label_elt.empty();
				if(op !== 'eq')
					label_elt.append(MakeElt('span.text-muted.small', this.translateOperator(op) + app_string('LBL_SEPARATOR')));
				label_elt.append(txt);
			}
			label_elt.toggleClass('text-placeholder', ph);
		},
        updateDisplay: function() {
            UI.addRemoveClass(this.input_elt, 'active', !! (this.focused || this.opened));
            UI.addRemoveClass(this.input_elt, 'invalid', this.invalid);
			UI.addRemoveClass(this.input_elt, 'opened', !! (this.opened && ! this.multi_sel));
			UI.addRemoveClass(this.input_elt, 'flat-bottom', !! (this.opened && this.multi_sel));
			this.updateLabel();
        }
    };

    UI.SearchSelectFilter = function(id, params) {
    	UI.SearchSelect.call(this, id, params);
    }
    extendClass(UI.SearchSelectFilter, UI.SearchSelect, {
    	render: function() {
    		UI.SearchSelect.prototype.render.call(this);
    	},
    	showModules: function() {
    		this.showOperator();
    	},
    	showDefault: function(evt) {
    		if(~['any_of', 'not_any_of', 'eq', 'not_eq'].indexOf(this.operator))
    			UI.SearchSelect.prototype.showDefault.call(this, evt);
    		else
    			this.showOperator();
    		this.focusSearch();
    	},
    	setOperator: function(op) {
    		if(op !== this.operator)
				this.perform_remove = false;
    		this.operator = op;
    	},
    	showOperator: function() {
			this.mode = 'operator';
			this.setTitle(this.module_title || app_string('LBL_FILTER_SELECT_OPERATOR'));
			this.list.icon_key = null;
			this.list.setOptions('select_filter_operators_dom');
			this.list.setSelectedKey(this.operator, true);
			this.updateDisplay();
			if(this.popup) this.popup.show();
    	},
    	translateOperator: function() {
    		return App.language.getListString('select_filter_operators_dom', this.operator);
    	},
    	isMultiSelect: function() {
    		return (this.operator === 'any_of' || this.operator === 'not_any_of');
    	},
		rowSelected: function(k, v, silent, evt) {
			if(this.mode == 'operator') {
				this.setOperator(k);
				if(! k || k === 'empty' || k === 'not_empty') {
					UI.callEvent(this, this.onchange, '', '', this.operator, false, evt);
					this.close();
				} else {
					UI.setDisplayState(this.list_elt, false);
					this.showDefault(evt);
				}
				return;
			}
			this.setKey(k);
			keepOpen = (this.isMultiSelect() && evt && (evt.cmdKey || evt.shiftKey));
			if(! keepOpen)
				this.close();
			if(! silent) {
				if(this.perform_remove)
					UI.callEvent(this, this.onremove, k, v, this.operator, keepOpen, evt);
				else
					UI.callEvent(this, this.onchange, k, v, this.operator, keepOpen, evt);
			}
			if (keepOpen)
				this.focus();
		},
		updateFilter: function() {
			if((this.mode === 'options' || this.mode === 'search') && this.isMultiSelect())
				return UI.SearchSelect.prototype.updateFilter.call(this);
		},
		updateDisplay: function() {
			UI.SearchSelect.prototype.updateDisplay.call(this);
			if(this.mode === 'operator' || this.fixed_operator) {
				UI.setDisplayState(this.return_row, false);
				UI.setDisplayState(this.search_elt, false);
			} else {
				this.return_link.content(
					[MakeElt('span.uii.uii-previous'), nbsp(), app_string('LBL_FILTER_SELECT_OPERATOR') +
					' (' + this.translateOperator() + ')']);
				UI.setDisplayState(this.return_row, true);
			}
		}
    });


    UI.NumberFilterInput = function(id, params) {
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		this.format = 'float';
		this.decimals = 2;
		this.invalid = false;
		if(params) extendObject(this, params);
	}
	UI.NumberFilterInput.prototype = {
		setup: function() {
			if(this.set_up)
				return;
			if(! this.elt)
				this.elt = $(this.id);
			if(! this.elt)
				return;
			var basename = this.name;
			if(this.form && basename) {
				if(! this.number_field)
					this.number_field = this.form.elements[basename];
				if(! this.end_field)
					this.end_field = this.form.elements[basename + '-end'];
				if(! this.operator_field)
					this.operator_field = this.form.elements[basename + '-operator'];
				if(! this.input_elt)
					this.input_elt = Wrap(this.elt).findFirst('.input-field').get(0);
			}
			this.addHooks();
			this.updateDisplay();
			this.set_up = true;
		},
		addHooks: function() {
			if(! this.input_elt) return;
			Wrap(this.input_elt)
				.on('mousedown', '.input-label', function(evt) { evt.preventDefault(); /* prevent click issues */})
				.on('mousedown', this.handleMouseDown, {scope: this})
				.on('click', this.handleClick, {scope: this})
				.on('keydown', this.handleKeyDown, {scope: this})
				.on('focus', this.handleFocus, {scope: this})
				.on('blur', this.handleBlur, {scope: this});
		},
		getOperator: function() {
			return this.operator_field && this.operator_field.value;
		},
		getFilterValue: function() {
			var val = trim(this.number_field && this.number_field.value);
			if(! val.length) val = null;
			return val;
		},
		getFilterEnd: function() {
			var val = trim(this.end_field && this.end_field.value);
			if(! val.length) val = null;
			return val;
		},
		focus: function() {
			this.input_elt.focus();
		},
		handleKeyDown: function(evt) {
			var nav;
			if(evt.keyCode == 38) nav = -1;
			else if(evt.keyCode == 40) nav = 1;
			else nav = 0;
			if(nav) {
				if(this.opened && this.mode === 'operator')
					this.operator_sel.adjustCurrentIndex(nav, null, evt);
				else if(! this.disabled)
					this.showPopup();
				UI.stopEvent(evt);
			} else if(evt.keyCode == 13) {
				if(this.opened) {
					this.acceptInput();
				} else if(! this.disabled)
					this.showPopup();
				UI.stopEvent(evt);
			} else if(evt.keyCode == 8 || evt.keyCode == 46) {
				// backspace/delete
				if(! this.disabled) this.clear();
			} else if(evt.keyCode == 27) {
				// esc
				if(this.opened) {
					this.hidePopup();
					this.focus();
					UI.stopEvent(evt);
				} else if(! this.disabled) {
					if(this.clear()) UI.stopEvent(evt);
				}
			} else if(evt.keyCode == 32) {
				if(! this.opened && ! this.disabled) {
					this.showPopup();
					UI.stopEvent(evt);
				}
			}
		},
		handleMouseDown: function(evt) {
			if(! this.disabled) {
				if(evt.currentTarget === this.input_elt)
					evt.stopPropagation();
			}
		},
		handleClick: function(evt) {
			if(evt.which == 1) {
				if(this.opened)
					this.hidePopup();
				else if(! this.disabled)
					this.showPopup();
				UI.stopEvent(evt);
			}
		},
		handleFocus: function(evt) {
			this.focused = true;
			this.updateDisplay();
		},
		handleBlur: function(evt) {
			this.focused = false;
			this.updateDisplay();
		},
		switchMode: function(mode, repos) {
			var op = this.operator_next;
			this.mode = mode;
			this.return_row.toggle(mode === 'values');
			this.value_div.toggle(mode === 'values');
			this.list_div.toggle(mode === 'operator');
			this.value_label.text(this.operatorLabel(op));
			this.end_row.toggle(op === 'between');
			if(repos) {
				if(mode === 'values') {
					this.number_input.field.placeholder =
						'(' + App.language.getListString('filter_properties_dom', (op === 'between' ? 'min' : 'value'), 'app') + ')';
					this.number_input.select();
				} else
					this.input_elt.focus();
				this.popup.reposition();
			}
		},
		returnValue: function(name, val) {
			if(name === 'operator') {
				if(val && val !== 'empty' && val !== 'not_empty') {
					this.operator_next = val;
					this.switchMode('values', true);
				} else {
					this.operator_field.value = val || '';
					this.number_field.value = '';
					this.end_field.value = '';
					this.hidePopup();
					this.updateDisplay();
					this.focus();
				}
			}
			else if(name === 'number') {
				this.number_next = val;
			}
			else if(name === 'end') {
				this.end_next = val;
			}
		},
		acceptInput: function() {
			var op = this.operator_next;
			if(this.opened) {
				if(this.mode === 'operator')
					this.operator_sel.selectCurrent();
				else {
					if(op && op !== 'empty' && op !== 'not_empty') {
						if(! trim(this.number_next).length && (op !== 'between' || ! trim(this.number_end).length))
							return this.returnValue('operator', '');
					} else {
						this.number_next = '';
						this.end_next = '';
					}
					this.operator_field.value = op;
					this.number_field.value = get_default(this.number_next, '');
					this.end_field.value = get_default(this.end_next, '');
					this.hidePopup();
					this.updateDisplay();
					this.focus();
				}
			}
		},
		clear: function() {
			var ret = this.operator_field.value;
			this.operator_field.value = '';
			this.number_field.value = '';
			this.end_field.value = '';
			if(this.opened) {
				this.hidePopup();
				this.focus();
			}
			this.updateDisplay();
			return ret;
		},
		operatorLabel: function(op) {
			if(op === undefined) op = this.getOperator();
			return App.language.getListString('number_filter_operators_dom', op, 'app') || '';
		},
		getPopupContent: function() {
			var self = this;
			if(! this.operator_sel) {
				this.operator_sel = new UI.SelectList(this.id + '-selop',
					{options: 'number_filter_operators_dom', flat: true});
				this.operator_sel.render();
				this.operator_sel.onchange = function() {
					self.returnValue('operator', this.getSelectedKey());
				}
			}
			if(! this.number_input) {
				this.number_input = new UI.TextInput(this.id + '-nval',
					{format: this.format, min_decimals: 0, decimals: this.decimals, tabIndex: 0,
					 init_value: this.number_field.value});
				this.number_input.render();
				this.number_input.onchange = function() {
					self.returnValue('number', this.getValue());
				}
				this.number_input.onreturn = function() {
					self.acceptInput();
				}
			}
			if(! this.end_input) {
				this.end_input = new UI.TextInput(this.id + '-eval',
					{format: this.format, min_decimals: 0, decimals: this.decimals, tabIndex: 0,
					 init_value: this.end_field.value,
					 placeholder: '(' + App.language.getListString('filter_properties_dom', 'max', 'app') + ')'});
				this.end_input.render();
				this.end_input.onchange = function() {
					self.returnValue('end', this.getValue());
				}
				this.end_input.onreturn = function() {
					self.acceptInput();
				}
			}
			if(! this.value_label) {
				this.value_label = MakeElt('h5.form-label.no-margin');
				this.tools_row = MakeElt('div.row.tools-row')
					.append(this.value_label)
					.append(MakeElt('div.uii.uii-lg.active-icon.uii-remove', {tabindex: 0})
						.on('click, keydown[key=enter|space]', function() {
							self.clear();
						}))
					.append(MakeElt('div.uii.uii-lg.active-icon.uii-accept', {tabindex: 0})
						.on('click, keydown[key=enter|space]', function() {
							self.acceptInput();
						}));
			}
			var return_link = MakeElt('a.tools-link', {href: '#'},
				MakeElt('span.uii.uii-previous'), nbsp(), app_string('LBL_FILTER_SELECT_OPERATOR'))
				.on('click', function() { self.switchMode('operator', true); return false; });
			this.return_row = MakeElt('div.card-header.panel-subheader')
				.append(MakeElt('div.tools-row.popup-title', return_link));
			this.list_div = MakeElt('div.card-body.panel-body')
				.append(this.operator_sel.elt);
			this.value_div = MakeElt('div.card-body.panel-body.panel-pad-body')
				.append(this.tools_row)
				.append(MakeElt('div.row.form-row')
					.append(MakeElt('div.column.form-cell').append(
						MakeElt('div.form-entry').append(
							MakeElt('div.form-value').append(this.number_input.field))))
				)
				.append(this.end_row = MakeElt('div.row.form-row')
					.append(MakeElt('div.column.form-cell').append(
						MakeElt('div.form-entry').append(
							MakeElt('div.form-value').append(this.end_input.field)))));
			return [this.return_row, this.list_div, this.value_div];
		},
		showPopup: function() {
			var self = this,
				op = this.getOperator(),
				content = this.getPopupContent(),
				p = new UI.PopupSlidePanel(this.id+'-popup',
					{target: this.input_elt, target_offset: {top: -1},
					 destroy_on_close: true,
					 min_width: 200});
			p.onshow = function() {
				self.opened = true;
				self.updateDisplay();
				if(self.mode === 'values')
					self.number_input.select();
				else
					self.input_elt.focus();
			}
			p.onclose = function() {
				self.opened = false;
				self.popup = null;
				self.updateDisplay();
			}
			p.render();
			Wrap(p.elt).content(content);
			this.operator_sel.setup();
			this.operator_sel.setSelectedKey(this.operator_next = op, true);
			this.number_input.setup();
			this.number_input.setValue(this.number_next = this.getFilterValue(), true);
			this.end_input.setup();
			this.end_input.setValue(this.end_next = this.getFilterEnd(), true);
			this.popup = p;
			if(op && op !== 'empty' && op !== 'not_empty')
				this.switchMode('values');
			else
				this.switchMode('operator');
			p.show();
		},
		hidePopup: function() {
			if(this.popup) this.popup.close();
		},
		updateLabel: function() {
			var op = this.getOperator(), opers, val, lbl,
				label_elt = Wrap(this.elt).findFirst('.input-label').ext(0);
			if(label_elt) {
				label_elt.empty();
				lbl = op === 'eq' ? '' : this.operatorLabel(op);
				if(op && op !== 'empty' && op !== 'not_empty') {
					val = this.getFilterValue();
					if(isset(val))
						val = stdFormatNumber(val);
					else
						val = app_string('LBL_ANY');
					if(lbl)
						label_elt.append(MakeElt('span.text-muted.small', lbl + app_string('LBL_SEPARATOR')));
					lbl = val;
					if(op === 'between') {
						label_elt.append(lbl);
						val = this.getFilterEnd();
						if(isset(val))
							val = stdFormatNumber(val);
						else
							val = app_string('LBL_ANY');
						label_elt.append(MakeElt('span.text-muted.small', app_string('LBL_FILTER_VALUE_AND')));
						lbl = val;
					}
				}
				label_elt.append(lbl);
				label_elt.toggleClass('text-placeholder', ! op);
			}
		},
		updateDisplay: function() {
			UI.addRemoveClass(this.input_elt, 'active', !! (this.focused || this.opened));
			UI.addRemoveClass(this.input_elt, 'opened', !! this.opened);
			UI.addRemoveClass(this.input_elt, 'invalid', this.invalid);
			this.updateLabel();
		}
	};


    UI.AlphabetInput = function(id, params) {
        this.id = id;
        this.onchange = null;
        this.focused = false;
        if(params) extendObject(this, params);
    };
    UI.AlphabetInput.prototype = {
        setup: function() {
            if(this.set_up)
                return;
            if(! this.field) {
                if(this.name && this.form)
                    this.field = this.form.elements[this.name];
                else if(this.field_id)
                    this.field = $(this.field_id);
            }
            this.addHooks();
            this.set_up = true;
        },
        addHooks: function() {
            if(this.id) {
                var ed = this;
                var letter;
                var letter_id = '';
                var letter_elem = null;
                for(optionCounter = 65; optionCounter < 91; optionCounter++) {
                    letter = String.fromCharCode(optionCounter);
                    letter_id = this.id +'-'+ letter;
                    letter_elem = $(letter_id);
                    if (letter_elem)
                        letter_elem.onclick = function(evt) { return ed.handleClick(evt || window.event, this.innerHTML); };
                }
            }
        },
        render: function() {
            // NYI
        },
        getValue: function() {
            return (this.field) ? this.field.value : '';
        },
        handleClick: function(evt, value) {
            var val;
            if(this.field)
                this.field.value = val = value;
            UI.callEvent(this, this.onchange, evt, val);
            if(this.auto_submit)
                UI.submitForm(this.form);
            return false;
        }
    };


	UI.ScrollPanel = function(id, params) {
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		this.pos = 0;
		this.pos_top = 0;
		this.step = 10;
		this.steps = 0;
		if(params) extendObject(this, params);
	}
	UI.ScrollPanel.prototype = {
		setup: function() {
			if(this.set_up)
				return;
			if(! this.elt && this.id)
				this.elt = $(this.id);
			if(this.elt && ! this.content) {
				if(this.content_id)
					this.content = $(this.content_id);
				else if(UI.hasClass(this.elt.firstChild, 'scroll-content'))
					this.content = this.elt.firstChild;
			}
			this.set_up = true;
		},
		render: function() {
			var scroll = this;
			var outer = this.elt;
			if(! outer) {
				var extcls = this.ext_class || '';
				outer = this.elt = createElement2('div', {id: this.id, className: 'input-scroll '+extcls});
			}
			var stop_event = function(evt) { UI.stopEvent(evt); }
			var handle_click = function(evt) { scroll.handleClick(evt); stop_event(evt); }
			UI.addEventListener(outer, 'mousedown',
				function(evt) { handle_click(evt); scroll.scrollContent('bar', evt); }, this);
			UI.addEventListener(outer, 'selectstart', stop_event, this);

			var inner = this.content;
			if(! inner) {
				inner = this.content = createElement2('div', {className: 'scroll-content'});
				outer.appendChild(inner);
			}
			var onclick = function(evt) {
				handle_click(evt);
				scroll.scrollContent(this.act, evt);
			}

			for(var i = 0; i < outer.childNodes.length; i++) {
				var node = outer.childNodes[i];
				if(node !== inner) {
					if(UI.hasClass(node, 'up') && ! this.up_button)
						this.up_button = node;
					else if(UI.hasClass(node, 'down') && ! this.down_button)
						this.down_button = node;
					else if(UI.hasClass(node, 'pos') && ! this.pos_button)
						this.pos_button = node;
					else {
						outer.removeChild(node);
						i --;
					}
				}
			}

			if(! this.up_button) {
				this.up_button = createElement2('div', {className: 'scroll-button up'});
				this.up_button.appendChild(createElement2('div', {className: 'input-icon'}));
				outer.appendChild(this.up_button);
			}
			this.up_button.act = 'up';
			UI.addEventListener(this.up_button, 'click', onclick, this.up_button);
			UI.addEventListener(this.up_button, 'mousedown', handle_click, this.up_button);
			UI.addEventListener(this.up_button, 'selectstart', stop_event, this.up_button);
			if(! this.down_button) {
				this.down_button = createElement2('div', {className: 'scroll-button down'});
				this.down_button.appendChild(createElement2('div', {className: 'input-icon'}));
				outer.appendChild(this.down_button);
			}
			this.down_button.act = 'down';
			UI.addEventListener(this.down_button, 'click', onclick, this.down_button);
			UI.addEventListener(this.down_button, 'mousedown', handle_click, this.down_button);
			UI.addEventListener(this.down_button, 'selectstart', stop_event, this.down_button);
			if(! this.pos_button) {
				this.pos_button = createElement2('div', {className: 'scroll-button pos'});
				this.pos_button.appendChild(createElement2('div', {className: 'input-icon'}));
				outer.appendChild(this.pos_button);
			}
			this.makeDragTarget(this.pos_button, null, {mousedown: handle_click});
			return outer;
		},
		handleClick: function(evt) {
			UI.callEvent(this, this.onclick, evt);
		},
		destroy: function() {
			this.destroyDragTarget(this.pos_button);
			this.elt = null;
			this.inner = null;
			this.up_button = this.down_button = this.pos_button = null;
		},
		getMax: function() {
			return (this.steps - 1) * this.step;
		},
		getDisplayAdjustedStep: function(step, mpos, check) {
			if(! isset(this.pos) || ! mpos || ! isset(mpos.y)) return step;
			var rgn = UI.getEltRegion(this.elt),
				elth = rgn.height,
				yoffs = mpos.y + window.pageYOffset - rgn.y,
				over = Math.floor((yoffs + this.pos) / this.step),
				rstep;
			if(yoffs > 0 && yoffs < elth && step != over) {
				if(step < over) {
					step = Math.round((this.pos - (over - step) * this.step) / this.step);
					rstep = Math.max(0, step);
				}
				else if(step > over) {
					step = Math.round((this.pos + elth - (over - step + 1) * this.step) / this.step);
					rstep = Math.min(this.steps + Math.round(elth / this.step) - 2, step);
				}
			}
			if(isset(rstep)) {
				if(check && rstep != step) return null;
				step = rstep;
			}
			return step;
		},
		canShowStep: function(step, mpos) {
			if(! mpos) return true;
			step = this.getDisplayAdjustedStep(step, mpos, true);
			return isset(step);
		},
		showStep: function(step, mpos) {
			if(! this.elt) return;
			var elth = this.elt.clientHeight || parseFloat(this.elt.style.height);
			if(mpos)
				step = this.getDisplayAdjustedStep(step, mpos);
			var showtop = step * this.step,
				showbottom = showtop + this.step,
				pos_t = this.pos,
				pos_b = pos_t + elth;
			if(! elth) return;
			if(pos_t > showtop) {
				if(pos_b >= showbottom)
					pos_t = showtop;
			} else if(pos_b < showbottom)
				pos_t = showbottom - elth;
			if(pos_t != this.pos)
				this.scrollContent(pos_t);
		},
		setHeight: function(ht) {
			if(this.elt) this.elt.style.height = (ht || 0) + 'px';
		},
		setSteps: function(steps, offset) {
			this.steps = steps;
			if(isset(offset))
				this.showStep(offset);
			this.scrollContent();
		},
		scrollContent: function(dir, evt) {
			if(! this.elt) return;
			var step_mult = 1,
				elth = this.elt.clientHeight || parseFloat(this.elt.style.height),
				pos = this.pos,
				max = this.getMax();
			if(dir == 'bar') {
				if(! this.pos_button) return;
				var btnpos = UI.getEltRegion(this.pos_button),
					xy = YEvent.getXY(evt);
				if(! btnpos) return;
				if(xy[1] < btnpos.top) dir = 'up';
				else dir = 'down';
				step_mult = 3;
			}

			if(dir == 'down') {
				if(pos % this.step > 3)
					pos += this.step * step_mult - (pos % this.step);
				else
					pos += this.step * step_mult + (pos % this.step);
			}
			else if(dir == 'up') {
				if(pos % this.step > 3)
					pos -= (pos % this.step * step_mult);
				else
					pos -= this.step * step_mult + (pos % this.step);
			}
			else if(isset(dir)) {
				pos = dir;
			}

			if(pos < 0) pos = 0;
			else if(pos > max) pos = max;

			if(max <= 0) {
				pos = 0;
				UI.addClass(this.elt, 'disabled');
			} else {
				UI.removeClass(this.elt, 'disabled');
				this.pos_h = Math.round(Math.max(13, elth / (max + elth) * (elth - 32)));
				this.pos_h = Math.min(this.pos_h, 200);
				this.pos_top = Math.round((elth - 34 - this.pos_h) * pos / max);
				if(this.pos_button) {
					this.pos_button.style.height = '' + this.pos_h + 'px';
					this.pos_button.firstChild.style.top = '' + Math.round((this.pos_h - 15) / 2) + 'px';
					this.pos_button.style.top = '' + (this.pos_top + 16) + 'px';
				}
			}
			this.pos = pos;
			var tprop = this.getTransformProperty();
			if(tprop) {
				if(pos || this.content.style[tprop]) {
					if(isIPhone)
						this.content.style[tprop] = 'translate3d(0, ' + (- pos) + 'px, 0)';
					else {
						if(tprop == 'webkitTransform') {
							// fix intermittent screen flash in Chrome 12
							// chromium issue 95026 - remove once fixed in browser
							// note - enforces 3D mode, which currently messes up anti-aliasing a bit
							this.content.style.webkitTransformStyle = 'preserve-3d';
						}
						this.content.style[tprop] = 'translateY(' + (- pos) + 'px)';
					}
				}
			}
			else
				this.content.style.top = '' + (- pos) + 'px';
		},
		getTransformProperty: function() {
			var tprop = UI.getTransformProperty();
			if(! isIPhone) tprop = null; // disabled for now, causes odd position offset in webkit
			return tprop;
		},
		getPosition: function() {
			if(! this.content) return;
			var rgn = UI.getEltRegion(this.content);
			rgn.rel_top = this.pos;
			if(this.getTransformProperty())
				rgn.top -= this.pos;
			return rgn;
		},
		ondragstart: function(evt, info) {
			UI.addClass(this.pos_button, 'active');
			info.pos_top = this.pos_top;
		},
		ondragend: function(evt, info) {
			UI.removeClass(this.pos_button, 'active');
		},
		ondrag: function(evt, info) {
			var max = this.getMax();
			var pos_top = info.pos_top + info.dragY;
			var pos = Math.round(pos_top * max / (this.elt.clientHeight - 32 - this.pos_h));
			this.pos = pos;
			this.scrollContent();
		}
	};
	extendProto(UI.ScrollPanel, UI.DragHandler);


	UI.CurrencySelect = function(id, params) {
		this.bean_name = 'Currency';
		this.module = 'Currencies';
		if(! params) params = {};
		if(! params.options) params.options = {dom_name: 'model.Currency'};
		opts = UI.getSelectOptions(params.options);
		if(! opts)
			console.error('currency options not found:', params.options);
		params.options = opts;
		UI.RefInput.call(this, id, params);
	};
	extendClass(UI.CurrencySelect, UI.RefInput, {
		setup: function() {
			if(this.set_up) return;
			UI.RefInput.prototype.setup.call(this);
			if(! this.rate_input) {
				if(this.form && this.exchange_rate) {
					this.rate_input = UI.getFormInput(this.form, this.exchange_rate);
					if(! this.rate_input) {
						var f = this.form.elements[this.exchange_rate];
						var h = new UI.HiddenInput(f || this.id + '-rate', {name: this.exchange_rate});
						if(this.elt.parentNode)
							this.elt.parentNode.insertBefore(h.render(), this.elt);
						this.rate_input = h;
					}
				}
			}
			var self = this;
			UI.onInitForm(this.form, this.postInit.bind(this));
			UI.attachInputEvent(this.rate_input, 'onchange', function(val) {
				var rate = self.getRate(true);
				UI.callEvent(self, self.onrateupdate, self.old_rate, rate);
				self.old_rate = rate;
			});
		},
		postInit: function() {
			var cid = this.getKey(), rate;
			if(cid && this.rate_input) {
				if(! this.rate_input.getValue()) {
					rate = this.old_rate = this.getStandardRate(cid);
					this.setRate(rate, true);
				} else {
					this.old_rate = this.getRate(true);
				}
			}
		},
		getOptionValue: function(currency_id) {
			if(currency_id === undefined) currency_id = this.getKey();
			return this.options && this.options.getValue(currency_id);
		},
		getDecimals: function(currency_id) {
			var v = this.getOptionValue(currency_id);
			return v ? parseInt(v.decimal_places, 10) : 2;
		},
		getShortName: function(currency_id, add_rate) {
			var v = this.getOptionValue(currency_id),
				ret = v ? v.iso4217 : '';
			if(add_rate && ret) ret += ' : ' + this.getSymbol(currency_id);
			return ret;
		},
		getSymbol: function(currency_id) {
			var v = this.getOptionValue(currency_id);
			return v ? v.symbol : '';
		},
		getSymbolAfter: function(currency_id) {
			var v = this.getOptionValue(currency_id);
			return v ? v.symbol_place_after : '';
		},
		getRate: function(nativeVal, currency_id) {
			var rate = 1.0;
			if(isset(currency_id) && currency_id != this.getKey())
				rate = this.getStandardRate(currency_id);
			else if(this.rate_input)
				rate = stdUnformatNumber(this.rate_input.getValue());
			if(nativeVal) {
				rate = parseFloat(rate);
				if(isNaN(rate)) rate = 1.0;
			}
			return rate;
		},
		getStandardRate: function(currency_id) {
			var rate, v = this.getOptionValue(currency_id);
			if(v) rate = parseFloat(v.conversion_rate);
			return rate || 1.0;
		},
		setRate: function(val, silent) {
			var oldrate = this.old_rate || this.getRate(true);
			var curid = this.getKey();
			if(! curid || curid === '-99' || ! val)
				val = 1.0;
			if(this.rate_input)
				this.rate_input.setValue(stdFormatNumber(val, 5, 5), silent);
			val = this.getRate(true);
			if(! this.rate_input && ! silent)
				UI.callEvent(this, this.onrateupdate, oldrate, val);
			this.old_rate = val;
		},
		haveCustomRate: function(rate) {
			if(! rate) rate = this.getRate(true);
			return (rate != this.getStandardRate());
		},
		update: function(key, value, silent, module, rate) {
			if(! key) {
				key = '-99';
				value = this.getOptionValue(key);
			}
			UI.RefInput.prototype.update.call(this, key, value, silent);
			var rate = get_default(rate, value ? value.conversion_rate : null);
			this.setRate(rate, silent);
		},
		setValueAndRate: function(key, rate, silent) {
			var value = this.getOptionValue(key || '-99');
			this.update(key, value, silent, this.module, rate);
		},
		setValue: function(cid, silent) {
			this.setValueAndRate(cid, undefined, silent);
		},
		formatAmount: function(amt) {
			if(! isset(amt)) return '';
			var decs = this.getDecimals();
			var ret = stdFormatNumber(parseFloat(amt).toFixed(decs));
			return this.getSymbol() + ' ' + ret;
		}
	});


	UI.OwnerSelect = function(id, params) {
		UI.SelectInput.call(this, id, params);
	};
	extendClass(UI.OwnerSelect, UI.SelectInput, {
		userUpdateMulti: function(row, count) {
			if (count == 1) {
				this.update(row.ID_0.id, row.ID_0);
			} else {
				var idx = -1, keys = this.menu.getKeys();
				for (var i = 0; i < keys.length; i++) {
					if (keys[i].match(/#/)) {
						idx = i; break;
					}
				}

				var ids = [], title = [];
				for (var i = 0; i < count; i++) {
					var key = row['ID_' + i].id;
					ids.push(key);
					title.push(row['ID_' + i].label);
				}
				value = {label: app_string('LBL_OWNER_MULTIPLE_USERS') + ' (' + count + ')', icon : 'icon-user', title: title.join(', ')};
				var key = ids.join('#');
				if(idx < 0)
					this.menu.addOption(key, value);
				else {
					var values = this.menu.getValues();
					keys[idx] = key;
					values[idx] = value;
				}
				UI.SelectInput.prototype.update.call(this, key, value);
			}
		},
		userUpdate: function(key, value) {
			this.update(key, value);
		},
		update: function(key, value, silent) {
			if(key == 'select')
				this.showSelectUser();
			else {
				if(isString(value))
					value = {label: value};
				if(! value.icon) value.icon = 'icon-user';
				if(! this.menu.hasKey(key))
					this.menu.addOption(key, value);
				UI.SelectInput.prototype.update.call(this, key, value, silent);
			}
		},
		showSelectUser: function() {
			var request_data = {
					form_name: this.form.getAttribute('id') || this.form.getAttribute('name'),
					call_back_function: this.popup_callback || 'ui_popup_return_owner',
					passthru_data: {ui_input_name: this.name || this.id},
					field_to_name_array: this.field_to_name || {id: 'id', label: 'name'}
				},
				params = {module: 'Users', request_data: request_data, inline: true, multiple: true};
			open_popup_window(params);
		}
	});


	UI.TabGroup = function(id, params) {
		this.id = id;
		this.tabs = [];
		this.current_tab = null;
		this.hover_tab = null;
		this.focus_tab = null;
		this.base_zindex = 1;
		this.manage_sections = {};
		this.wrapped_idx = [];
		if(params)
			extendObject(this, params);
	};
	UI.TabGroup.all_active = [];
	UI.TabGroup.resizeHandler = function() {
		var check = UI.TabGroup.all_active, seen = [];
		for(var i = 0; i < check.length; i++) {
			if(check[i].id && check[i].elt && UI.inDocument(check[i].elt)) {
				seen.push(check[i]);
				check[i].updateDisplay();
			}
		}
		UI.TabGroup.all_active = seen;
	}
	UI.TabGroup.prototype = {
		focus: function() {
			if(! isset(this.focus_tab))
				this.focus_tab = this.active_tab;
			var t = this.getTab(this.focus_tab, true);
			if(t && t.elt)
				t.elt.focus();
		},
		getPosition: function(name) {
			for(var i = 0; i < this.tabs.length; i++)
				if(this.tabs[i].name == name)
					return i;
		},
		loadTabs: function(tabs) {
			if(Array.isArray(tabs))
				this.tabs = tabs;
			var i, t;
			for(i = 0; i < this.tabs.length; i++) {
				// identify tab elements
				t = this.tabs[i];
				if(! isset(t) || typeof(t) != 'object') {
					this.tabs.splice(i, 1);
					i --;
					continue;
				}
				if(t.separator) continue;
				if(! t.elt) {
					if(t.id)
						t.elt = $(t.id);
					else if(t.name)
						t.elt = $(this.id+'-tab-'+t.name);
				}
				if(! t.outer_elt) {
					if(t.outer_id)
						t.outer_elt = $(t.outer_id);
					else if(t.name)
						t.outer_elt = $(this.id+'-tabouter-'+t.name);
				}
			}
		},
		setup: function() {
			if(this.set_up) return;
			if(! this.elt)
				this.elt = $(this.id);
			if(! this.field) {
				if(this.name && this.form)
					this.field = this.form.elements[this.name];
				else if(this.field_id)
					this.field = $(this.field_id);
			}
			this.loadTabs();
			this.addHooks();
			if(this.field && ! isset(this.current_tab))
				this.setValue(this.field.value, true, true);
			this.set_up = true;
			this.showHideSections();
			this.updateDisplay();
			UI.TabGroup.all_active.push(this);
			// TODO!! change to App.moduleLoaded()
			if (App.le)
				App.le.registerTabGroup(this);
		},
		handleClick: function(evt) {
			var tab = evt && evt.data && evt.data.tab;
			this.setValue(tab && tab.name);
			UI.stopEvent(evt);
		},
		handleFocus: function(evt) {
			var tab = evt && evt.data && evt.data.tab;
			this.setFocus(tab && tab.name);
		},
		handleBlur: function(evt) {
			this.setFocus(null);
		},
		handleMouseEnter: function(evt) {
			var tab = evt && evt.data && evt.data.tab;
			this.setHover(tab && tab.name);
		},
		handleMouseLeave: function(evt) {
			this.setHover(null);
		},
		addHooks: function() {
			var i, t;
			for(i = 0; i < this.tabs.length; i++) {
				t = this.tabs[i];
				if(t.hidden || t.separator || t.disabled) continue;
				Wrap(t.elt).on({
					'mouseenter': this.handleMouseEnter,
					'mouseleave': this.handleMouseLeave,
					'click, keydown[key=enter|space]': this.handleClick,
					'focus': this.handleFocus,
					'blur': this.handleBlur},
					{scope: this, data: {tab: t}});
			}
			if(this.elt) this.elt.onmousedown = function() { return false; }
		},
		setHover: function(name) {
			var t = this.getTab(name, true);
			if(t)
				this.hover_tab = name;
			else
				this.hover_tab = this.focus_tab;
			this.updateDisplay();
		},
		setFocus: function(name) {
			var t = this.getTab(name, true);
			if(t)
				this.focus_tab = this.hover_tab = name;
			else
				this.focus_tab = this.hover_tab = null;
			this.updateDisplay();
		},
		getValue: function() {
			if(this.field) return this.field.value;
			return this.current_tab;
		},
		setValue: function(name, silent, no_redraw) {
			var t = this.getTab(name, true);
			if(t) {
				if(! t.disable_select) {
					this.current_tab = name;
					if(this.field)
						this.field.value = this.current_tab;
					if(! no_redraw) {
						this.updateDisplay();
						this.showHideSections();
					}
				}
				if(! silent) {
					if(t.perform)
						UI.callEvent(this, t.perform, t);
					UI.callEvent(this, this.onchange, t);
				}
			}
		},
		showHideTab: function(name, state) {
			var idx = this.getPosition(name);
			if(! isset(state)) state = -1;
			if(isset(idx)) {
				if(state === -1)
					this.tabs[idx].hidden = ! this.tabs[idx].hidden;
				else
					this.tabs[idx].hidden = state ? 0 : 1;
				if(this.tabs[idx].hidden) {
					if(this.focus_tab == name)
						this.focus_tab = null;
					if(this.hover_tab == name)
						this.hover_tab = null;
					if(this.current_tab == name)
						this.setValue(null);
				}
				this.updateDisplay();
			}
		},
		enableDisableTab: function(name, state) {
			var t, idx = this.getPosition(name);
			if(! isset(state)) state = -1;
			if(isset(idx))
				t = this.tabs[idx];
			if(t && ! t.separator) {
				if(state === -1)
					t.disabled = ! t.disabled;
				else
					t.disabled = state ? 1 : 0;
				if(t.disabled) {
					if(this.focus_tab == name)
						this.focus_tab = null;
					if(this.hover_tab == name)
						this.hover_tab = null;
					if(this.current_tab == name)
						this.setValue(null);
				}
				if(! t.hidden)
					this.updateDisplay();
			}
		},
		getTab: function(name, selectable) {
			var t, idx = name ? this.getPosition(name) : null;
			if(isset(idx))
				t = this.tabs[idx];
			if(t && (! selectable || (! t.hidden && ! t.separator && ! t.disabled)))
				return t;
		},
		setLabel: function(name, label) {
			var idx = this.getPosition(name), elt = this.findIdxLabel(idx);
			if(this.tabs[idx]) {
				this.tabs[idx].label = label;
				if(elt) {
					UI.setElementText(elt, label);
				}
			}
		},
		findIdxIcon: function(idx, clsName) {
			var es, i, cls;
			if(this.tabs[idx] && this.tabs[idx].outer_elt) {
				es = this.tabs[idx].outer_elt.getElementsByClassName('input-icon');
				if(es) {
					if(! clsName) return es[0];
					cls = es[0].className;
					cls = cls.replace(/(\b(tab-icon|input-icon|left)\b\s*)*/, '');
					return cls;
				}
			}
		},
		findIdxLabel: function(idx) {
			var es;
			if(this.tabs[idx] && this.tabs[idx].outer_elt) {
				es = this.tabs[idx].outer_elt.getElementsByClassName('input-label');
				if(es) return es[0];
			}
		},
		render: function() {
		},
		renderTab: function(name, label, icon, className) {
			var out = createElement2('div', {id: this.id + '-tabouter-' + name, className: 'tab-outer normal ' + get_default(className, '')}),
				btn = createElement2('div', {id: this.id + '-tab-' + name, className: 'tab-button', type: 'button'}),
				span = createElement2('span', {className: 'input-label tab-label'});
			if(icon) {
				if(icon.indexOf('uii') < 0 && icon.indexOf('input-icon') < 0) icon = 'input-icon ' + icon;
				btn.appendChild(createElement2('div', {className: 'tab-icon ' + icon}));
			}
			if(label) {
				UI.setElementText(span, label);
				btn.appendChild(span);
			}
			out.appendChild(btn);
			return out;
		},
		destroy: function() {
			for(var i = 0; i < this.tabs.length; i++) {
				UI.removeNode(this.tabs[i].elt, true);
				this.tabs[i].elt = this.tabs[i].outer_elt = null;
			}
		},
		getVisibleTabs: function() {
			var ret = [], i, t, rt, sep = false;
			for(i = 0; i < this.tabs.length; i++) {
				t = this.tabs[i];
				if(t.hidden)
					continue;
				if(t.separator) {
					sep = true;
					if(ret.length)
						ret[ret.length-1].pre_sep = true;
					continue;
				}
				rt = {index: i, name: t.name, pre_sep: false, post_sep: sep};
				rt.current = (t.name == this.current_tab);
				rt.hover = (t.name == this.hover_tab);
				rt.focus = (t.name == this.focus_tab);
				rt.disabled = !! t.disabled;
				ret.push(rt);
				sep = false;
			}
			if(ret.length) {
				ret[0].first = true;
				ret[ret.length-1].last = true;
			}
			return ret;
		},
		updateDisplay: function() {
			var i, j, t, hset,
				ts = this.getVisibleTabs(),
				nowrap = UI.hasClass(this.elt, 'nowrap'),
				outer = Wrap(this.elt),
				bounds = outer.bounds(),
				tab, tbound,
				invis,
				current,
				testidx = [],
				extidx = [];
			if(! bounds) return; // not rendered yet
			for(i = 0; i < ts.length; i++) {
				t = this.tabs[ts[i].index];
				tab = Wrap(t.outer_elt);
				if(tab) {
					if(! nowrap && ! hset && t.outer_elt.offsetHeight) {
						this.elt.style.height = '' + t.outer_elt.offsetHeight + 'px';
						hset = true;
					}
					invis = false;
					if(ts[i].current) {
						tab.css('z-index', this.base_zindex + ts.length);
						current = ts[i].index;
					}
					else
						tab.css('z-index', this.base_zindex + ts.length - i - 1);
					tab.show();
					tab.toggleClass('current', ts[i].current);
					tab.toggleClass('active', ts[i].focus);
					tab.toggleClass('hover', ts[i].hover);
					tab.toggleClass('disabled', ts[i].disabled);
					tab.toggleClass('presep', ts[i].pre_sep);
					tab.toggleClass('postsep', ts[i].post_sep);
					if(nowrap && (tbound = tab.bounds(outer)) ) {
						if(tbound.right > bounds.width - 5) {
							if(testidx.length) {
								for(j = 0; j < testidx.length; j++)
									Wrap(this.tabs[testidx[j]].outer_elt).css('visibility', 'hidden');
								extidx.pushArray(testidx);
								testidx.clear();
							}
							invis = true;
						} else if(tbound.right > bounds.width - 50) {
							testidx.push(ts[i].index);
						}
					}
					if(invis) {
						extidx.push(ts[i].index);
						tab.css('visibility', 'hidden');
					} else {
						tab.css('visibility', 'visible');
					}
				}
			}
			if(isset(current) && extidx.indexOf(current) >= 0) {
				// try to force current tab to be visible
				var wrapCur = Wrap(this.tabs[current].outer_elt);
				testidx.clear();
				for(i = 0; this.tabs[ts[i].index].outer_elt && ts[i].index != current; i++) {
					testidx.push(ts[i].index);
				}
				for(i = testidx.length-1; i >= 0; i--) {
					Wrap(this.tabs[testidx[i]].outer_elt).hide();
					extidx.unshift(testidx[i]);
					if(wrapCur.bounds(outer).right <= bounds.width - 50) {
						wrapCur.css('visibility', 'visible');
						extidx.remove(current);
						break;
					}
				}
			}
			if(! extidx.compareTo(this.wrapped_idx)) {
				this.wrapped_idx = extidx;
				this.updateWrapMenu();
			}
		},
		showHideSections: function() {
			if(this.manage_sections) {
				var s, info, hide, i;
				for(s in this.manage_sections) {
					info = this.manage_sections[s];
					hide = true;
					if(Array.isArray(info)) {
						for(i = 0; i < info.length; i++)
							if(info[i] === this.current_tab) {
								hide = false;
								break;
							}
					}
					UI.addRemoveClass(s, 'hidden', hide);
				}
			}
			UI.layoutUpdated();
		},
		getWrapTab: function() {
			if(! this.wrap_tab || ! UI.inDocument(this.wrap_tab)) {
				this.wrap_tab = this.renderTab('wrap-tab', '', 'uii uii-odots', 'tab-wrap');
				this.wrap_tab.style.display = 'none';
				if(this.elt) this.elt.firstChild.appendChild(this.wrap_tab);
			}
			return this.wrap_tab;
		},
		updateWrapMenu: function() {
			var idx = this.wrapped_idx, tab, lbl, text, icon, opts = {keys: [], values: []}, self=this;
			for(var i = 0; i < idx.length; i++) {
				tab = this.tabs[idx[i]];
				if(! tab) continue;
				icon = null;
				if(isset(tab.label)) text = tab.label;
				else {
					lbl = this.findIdxLabel(idx[i]);
					if(lbl)
						text = UI.getElementText(lbl);
					else
						text = null;
				}
				if(isset(tab.icon)) icon = tab.icon;
				else
					icon = this.findIdxIcon(idx[i], true);
				opts.keys.push(idx[i]);
				opts.values.push({icon: icon, label: text, name: tab.name, perform: function(idx, t) { self.setValue(t.name); }});
			}
			if(! opts.keys.length)
				return this.hideWrapMenu();
			tab = this.getWrapTab();
			if(tab) {
				if(! this.wrap_menu) this.wrap_menu = UI.registerMenuSource(tab, {icon_key: 'icon', target_offset: {top: -1}}, this.form);
				this.wrap_menu.menu.setOptions(new UI.SelectOptions(opts));
				UI.setDisplayState(this.wrap_tab, true);
			}
		},
		hideWrapMenu: function() {
			if(this.wrap_tab)
				UI.setDisplayState(this.wrap_tab, false);
		}
	}
	UI.addResizeHook(UI.TabGroup.resizeHandler);


	UI.ButtonInput = function(id, params) {
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		if(params) extendObject(this, params);
	}
	UI.ButtonInput.prototype = {
		setup: function() {
			if(this.set_up)
				return;
			if(! this.elt) {
				if(! (this.elt = $(this.id)) )
					return;
			}
			this.addHooks();
			this.set_up = true;
		},
		addHooks: function() {
			UI.addEventListener(this.elt, 'click', this.handleClick, this);
		},
		handleClick: function(evt) {
			UI.stopPropagation(evt);
			UI.callEvent(this, this.onclick, evt);
		},
		focus: function() {
			if(this.elt) this.elt.focus();
		},
		render: function() {
			if(! this.elt) {
				var body = [];
				if(this.icon)
					body.push(createElement2('div', {className: 'input-icon left '+this.icon}));
				if(this.label)
					body.push(createElement2('span', {className: 'input-label'}, this.label));
				this.elt = createElement2('button', {
					type: this.button_type || 'button',
					className: this.className || 'input-button',
					disabled: this.disabled,
					value: this.value,
					tabIndex: this.tabIndex || 0,
					title: this.title
				}, body);
				if(this.autofocus) UI.addClass(this.elt, 'autofocus');
				if(this.style) setAttr(this.elt, 'style', this.style);
				this.setup();
			}
			return this.elt;
		},
		destroy: function() {
			UI.removeNode(this.elt, true);
			this.elt = null;
		}
	};


	UI.AddressInput = function(id, params) {
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		if(params) extendObject(this, params);
		if(! this.parts)
			this.parts = {};
	}
	UI.AddressInput.prototype = {
		setup: function() {
			if(this.set_up)
				return;
			if(! this.elt) {
				if(! (this.elt = $(this.id)) )
					return;
			}
			if(! this.copy_elt)
				this.copy_elt = $(this.id + '-copy');
			if(this.copy_elt && this.copy_to && ! this.menu) {
				var opts = {keys: [], values: []}, self=this;

                if (this.copy_to_rel_contacts) {
                    opts.keys.push('');
                    opts.values.push({
                        label: app_string('LBL_NONE'),
                        perform: function() { self.updateCopyTo('', ''); }
                    });
                }

				for(var i = 0; i < this.copy_to.length; i++) {
					var inp = UI.getFormInput(this.form, this.copy_to[i]),
                        copy_from = this.name;
					if(inp) {
						opts.keys.push(inp.name);
                        (function(sel_id, input, from, copy_rel) {
                            opts.values.push({
                                label: inp.getLabel(),
                                perform: function() { if (copy_rel) { self.updateCopyTo('', ''); } input.copyFrom(from); }
                            });
                        })(self.id, inp, copy_from, this.copy_to_rel_contacts)
					}
				}

                if (this.copy_to_rel_contacts) {
                    var copy_targets = {primary: 'LBL_ADDRESS_COPY_CONTACTS_PRIMARY',
                        alt: 'LBL_ADDRESS_COPY_CONTACTS_OTHER', both: 'LBL_ADDRESS_COPY_CONTACTS_BOTH'};
                    createElement2('input', {type: 'hidden', name: 'copy_' + copy_from, id: self.id+'-copy_target'}, null, this.form);

                    for (var target in copy_targets) {
                        opts.keys.push(target);
                        var lbl = app_string(copy_targets[target]);
                        (function(sel_id, copy_target, label) {
                            opts.values.push({
                                label: label,
                                perform: function() {
                                    self.updateCopyTo(label, copy_target);
                                    if (this.copy_status_id) UI.hideStatus(this.copy_status_id);
                                    this.copy_status_id = UI.showStatus(app_string('LBL_ADDRESS_COPY_USER_MSG'), 3000);
                                }
                            });
                        })(self.id, target, lbl)
                    }

                    opts.width = '25em';
                }
                this.menu = new UI.MenuSource(this.copy_elt, {options: opts, form: this.form});
				this.menu.setup();
			}
			if(this.parts && this.parts['address_country'] && this.parts['address_state']) {
				var inp_c = UI.getFormInput(this.form, this.parts['address_country']),
					inp_s = UI.getFormInput(this.form, this.parts['address_state']),
					cname, ccode, self=this;
				if(inp_c && inp_s && inp_c.search_dom == 'country_codes') {
					inp_c.onchange = function() {
						var ccode = self.lookupCountryCode(this.getValue()),
							opts = self.popStates(ccode, inp_s), scode;
						if(opts) {
							scode = self.lookupStateCode(ccode, inp_s.getValue());
							if(! scode) inp_s.setValue('');
						}
					}
					inp_c.search_delay = 100;
					inp_c.search_blank = true;
					ccode = this.lookupCountryCode(inp_c.getValue());
					this.loadStates(function() { self.popStates(ccode, inp_s); });
					if(ccode)
						inp_s.setSearch(true, 'states-'+ccode);
				}
			}
			this.set_up = true;
		},
		popStates: function(code, state_input) {
			var lst;
			if(code && UI.country_states && UI.country_states[code]) {
				lst = 'states-'+code;
				App.language.addList(lst, UI.country_states[code]);
				if(state_input) {
					state_input.search_delay = 100;
					state_input.search_blank = true;
					state_input.setSearch(true, lst);
				}
				return lst;
			} else if(state_input) {
				state_input.setSearch(false);
			}
		},
		lookupCountryCode: function(cname) {
			var opts, i;
			cname = (cname || '').toLowerCase().trim();
			if(cname.length) {
				opts = App.language.getList('country_codes', 'app');
				if(opts) {
					for(i = 0; i < opts.values.length; i++) {
						if(opts.values[i].toLowerCase() == cname)
							return opts.keys[i];
					}
				}
			}
		},
		lookupStateCode: function(ccode, sname) {
			var opts, i;
			if(! UI.country_states || ! UI.country_states[ccode])
				return;
			sname = (sname || '').toLowerCase().trim();
			if(sname.length) {
				opts = UI.country_states[ccode];
				for(i = 0; i < opts.values.length; i++) {
					if(opts.values[i].toLowerCase() == sname)
						return opts.keys[i];
				}
			}
		},
        updateCopyTo: function(label, value) {
            var msg = Dom.queryId(this.id+'-message').text(label).toggle(!! label);
            Dom.queryId(this.id+'-copy_target').val(value);
        },
		getLabel: function() {
			return this.label;
		},
		getFields: function() {
			var ret = {};
			for(var k in this.parts)
				ret[k] = UI.getFormInput(this.form, this.parts[k]);
			return ret;
		},
		copyFrom: function(other) {
			var other = UI.getFormInput(this.form, other);
			if(other) {
				var other_fs = other.getFields(),
					my_fs = this.getFields();
				var refs = {}, rest = {};
				for(var k in other_fs) {
					if(my_fs[k]) {
						if(my_fs[k] instanceof UI.RefInput) refs[k] = my_fs[k];
						else rest[k] = my_fs[k]
					}
				}
				for(var k in refs)
					refs[k].update(other_fs[k].getKey(), other_fs[k].getValue());
				for(var k in rest)
					rest[k].setValue(other_fs[k].getValue());
			}
		},
		loadStates: function(callb) {
			if(isset(UI.country_states)) {
				if(callb) callb();
				return;
			}
			if(UI.loading_country_states) {
				if(callb) UI.loading_country_states.push(callb);
				return;
			}
			var req = new App.conn.JSONRequest('get_country_states', {silent: true});
			UI.loading_country_states = [];
			if(callb) UI.loading_country_states.push(callb);
			req.fetch(function(ret) {
				UI.country_states = ret.getResult();
				if(UI.loading_country_states) {
					for(i = 0; i < UI.loading_country_states.length; i++)
						UI.loading_country_states[i]();
				}
				UI.loading_country_states = false;
			});
		}
	};

    UI.QuickElt = function(id, params) {
   		if(isElement(id)) {
   			this.value_elt = id;
   			this.id = this.value_elt.id;
   		} else
   			this.id = id;
        this.elt = null;
   		if(params) extendObject(this, params);
   	}
   	UI.QuickElt.prototype = {
   		setup: function() {
            if(this.set_up)
                return;
            if(! this.value_elt) {
                if(! (this.value_elt = $(this.id)) )
                    return;
            }
            var val = '';
            if (this.type == 'enum' || this.type == 'status') {
               var opts = {'keys': this.options.keys, 'values': this.options.values};
               var select_opts = new UI.SelectOptions(opts);
               this.elt = new UI.QuickSelect({'options': select_opts, 'default': this.default_val, 'elt': this.value_elt});
               val = this.selected;
            } else if(this.type == 'bool') {
            	this.elt = new UI.QuickCheck({});
            	val = this.value;
            } else {
               this.elt = new UI.QuickText({show_reset: this.show_reset});
               val = get_default(this.value, '');
            }
            this.init(this.callback, val);
            this.set_up = true;
   		},
        init: function (need_callback, value) {
            if(this.initialized)
                return;
            var callback = null;

            if (need_callback) {
                function ready() {
                    var row = this.getResult();
                    if(row.result == 'ok') {
                        var frm = row.list_id + '-ListUpdate';
                        var conn_params = {};
                        conn_params.resultDiv = '' + row.list_id + '-outer';
                        return UI.sendForm(frm, {"record_perform": "subpanel_render"}, conn_params);
                    }
                    return false;
                }
                callback = ready;
            }
            var params = this;
            var method = this.callback_method || 'save_quick_edit_result';

            this.elt.init(value, function(reset) {
                var query_params = {
                    module: params.module,
                    record: params.record,
                    link_name: params.link_name,
                    target_id: params.target_id,
                    list_id: params.list_id,
                    field: params.field,
                    field_value: this.getValue(),
                    reset: reset ? true : false
                };

                var req = new App.conn.JSONRequest(method, {status_msg: app_string('LBL_SAVING')}, query_params);
                req.fetch(callback);
            });
			this.initialized = true;
        },
        show: function() {
            this.elt.showPopup(null, this.value_elt);
            if ((this.type != 'enum' || this.type == 'status') && this.type != 'bool') {
                this.elt.field.format = this.type;
                if (this.type == 'float' || this.type == 'double')
                    this.elt.field.decimals = 2;
            }
        }
   	};


	UI.HiddenInput = function(id, params) {
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		if(params) extendObject(this, params);
	}
	UI.HiddenInput.findOrInsert = function(form, name, default_id) {
		if(! form || ! name) return;
		var input = UI.getFormInput(form, default_id || name);
		if(! input) {
			var f = form.elements[name];
			input = new UI.HiddenInput(f || default_id || name + '-input', {form: form, name: name});
			var elt = input.render();
			UI.registerInput(form, input);
			if(! elt.parentNode)
				form.appendChild(elt);
		}
		return input;
	}
	UI.HiddenInput.prototype = {
		setup: function() {
			if(this.set_up)
				return;
			if(! this.elt) {
				if(this.form && this.name && (this.elt = this.form.elements[this.name]))
					;
				else if(! (this.elt = $(this.id)) )
					return;
			}
			this.set_up = true;
		},
		getValue: function() {
			return this.elt ? this.elt.value : '';
		},
		setValue: function(val, silent) {
			if(this.elt) this.elt.value = val;
			if(! silent)
				UI.callEvent(this, this.onchange, val);
		},
		render: function() {
			if(! this.elt) {
				this.elt = createElement2('input', {id: this.id, name: this.name, type: 'hidden', value: this.init_value || ''});
				if(this.form && this.name) this.form.appendChild(this.elt);
				this.setup();
			}
			return this.elt;
		}
	};


	UI.FileInput = function(id, params) {
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		if(params) extendObject(this, params);
	}
	UI.FileInput.prototype = {
		setup: function() {
			if(this.set_up)
				return;
			if(! this.elt) {
				if(this.form && this.name && (this.elt = this.form.elements[this.name]))
					;
				else if(! (this.elt = $(this.id)) )
					return;
			}
			this.clear_btn = UI.getFormInput(this.form, this.name + '_clear');
			this.dupe_btn = UI.getFormInput(this.form, this.name + '_dupe');
			this.addHooks();
			this.set_up = true;
		},
		addHooks: function() {
			var self = this;
			UI.addEventListener(this.elt, 'change', this.handleChanged, this);
			if(this.dupe_btn) {
				UI.attachInputEvent(this.dupe_btn, 'onchange', function() { self.handleDupeChange() });
			}
			if(this.clear_btn) {
				UI.attachInputEvent(this.clear_btn, 'onchange', function() { self.handleClearChange() });
			}
		},
		handleChanged: function(evt) {
			UI.markFormDirty(this);
			UI.callEvent(this, this.onchange, evt);
		},
		handleDupeChange: function(evt) {
			var disable = this.dupe_btn ? this.dupe_btn.getValue() : 0;
			this.elt.disabled = this.disabled || disable;
			UI.callEvent(this, this.ondupechange, evt);
		},
		handleClearChange: function(evt) {
			var disable = this.clear_btn ? this.clear_btn.getValue() : 0;
			this.elt.disabled = this.disabled || disable;
			UI.callEvent(this, this.onclearchange, evt);
		},
		focus: function() {
			if(this.elt) this.elt.focus();
		},
		getValue: function() {
			return this.elt ? this.elt.value : '';
		},
		setValue: function(file, silent) {
			if(this.elt) this.elt.value = file;
			if(! silent) UI.callEvent(this, this.onchange);
		},
		render: function() {
			if(! this.elt) {
				this.elt = createElement2('input', {id: this.id, name: this.name, type: 'file', className: 'input-field input-file'});
				if(this.form && this.name) this.form.appendChild(this.elt);
				this.setup();
			}
			return this.elt;
		},
		validate: function() {
			this.invalid = false;
			var value = this.getValue();
			if(this.required && ! trim(value).length) {
				this.invalid = true;
				this.invalidMsg = 'required';
			}
			this.updateDisplay();
			return ! this.invalid;
		},
		updateDisplay: function() {
			UI.addRemoveClass(this.elt, 'invalid', !! this.invalid);
		}
	};


	UI.RelatedSelectInput = function(id, params) {
		UI.RefInput.call(this, id, params);
	}
	extendClass(UI.RelatedSelectInput, UI.RefInput, {
		update: function(key, val, silent) {
			if(val) {
				if(! isObject(val)) val = {id: key, _display: val};
				val.module = this.getModule();
				if(this.list_name) {
					var input = UI.getFormInput(this.form, this.list_name);
					if(input) input.insertRow(val);
				}
			}
		}
	});


	UI.EmailFromSelect = function(id, params) {
		UI.SelectInput.call(this, id, params);
	}
	extendClass(UI.EmailFromSelect, UI.SelectInput, {
		onchange: function(key, val) {
			var e = $(this.id.replace(/-input$/, '-custom'));
			if(val && val.custom) {
				UI.setDisplayState(e, true);
			} else {
				var from = UI.getFormInput(this.form, 'from_name'),
					addr = UI.getFormInput(this.form, 'from_addr');
				if(from) from.setValue(val ? val.from : '');
				if(addr) addr.setValue(val ? val.email : '');
				UI.setDisplayState(e, false);
			}
		}
	});


	UI.InviteesList = function(id, params) {
		this.id = id;
		if(params) extendObject(this, params);
	}
	UI.InviteesList.prototype = {
		setup: function() {
			if(this.set_up) return;
			this.outer = $(this.form.id + '-' + this.name);
			this.set_up = true;
			this.field = this.form.elements[this.name];
			this.type_field = this.form.elements[this.type_name];
			this.add_input = UI.getFormInput(this.form, 'add_'+this.name);
			this.rems = [];
			this.count = 0;
			if(this.outer && this.field && this.type_field) {
				var vals = this.getValues(), self = this;
				this.count = vals.ids.length;
				this.addButtons();
				if(this.add_input) {
					this.add_input.onchange = function(k, v) {
						if(v) {
							self.addRow(v);
							this.clear();
						}
					}
					//this.add_input.addExtraReturnFields({_module: '_module'});
				}
			}
		},
		getValues: function() {
			return {
				ids: this.field.value.split(';'),
				types: this.type_field.value.split(';')
			};
		},
		setValues: function(v) {
			this.field.value = v.ids.join(';');
			this.type_field.value = v.types.join(';');
			this.count = v.ids.length;
		},
		addButtons: function() {
			var self = this, i, rem;
			for(var i = 0; i < this.rems.length; i++)
				UI.removeNode(this.rems[i]);
			this.rems = [];
			for(var i = 0; i < this.count; i++) {
				if(! this.outer.childNodes[i]) break;
				rem = createElement2('div', {className: 'input-icon icon-delete active-icon',
					style: {float: 'right'},
					onclick: function(evt) { self.removeRow(this.idx); }
				});
				rem.idx = i;
				this.outer.childNodes[i].appendChild(rem);
				this.rems.push(rem);
			}
		},
		addRow: function(v) {
			var div = createElement2('div', null, null, this.outer),
				vals = this.getValues();
			div.appendChild(UI.createIcon(v._module));
			div.appendChild(document.createTextNode(' ' + v._display));
			vals.ids.push(v.id);
			vals.types.push(v._module);
			this.setValues(vals);
			this.addButtons();
		},
		removeRow: function(idx) {
			this.outer.removeChild(this.outer.childNodes[idx]);
			var vals = this.getValues();
			vals.ids.splice(idx, 1);
			vals.types.splice(idx, 1);
			this.setValues(vals);
			this.addButtons();
		}
	};


    UI.SystemMessage = function() {
    	this.num = UI.SystemMessage.num ++;
    }
    UI.SystemMessage.num = 0;
    UI.SystemMessage.show = function(record, mark_seen) {
    	var s = new UI.SystemMessage();
    	s.showMessage(record, mark_seen);
    }
    UI.SystemMessage.showList = function(records, mark_seen) {
    	if(records && records.length) {
    		var record = records.shift();
			var s = new UI.SystemMessage();
			s.showMessage(record, mark_seen, records);
    	}
    }
    UI.SystemMessage.prototype = {
    	showMessage: function(record, mark_seen, remain) {
			var query_params = {module: 'SystemMessages', record: record, mark_seen: get_default(mark_seen,1)};
			var req = new App.conn.JSONRequest('fetch_system_message', {}, query_params);
			var num = this.num;
			req.fetch(function(ret) {
				var json;
				if(ret && (json = ret.getResult()) && json.status == 'ok') {
					var title = '<div class="input-icon theme-icon module-SystemMessages"></div>&nbsp;<span>'
						+ html_escape(json.result.name) + '</span>';
					var dlg = new UI.Dialog('sysmsg-' + num, {
						destroy_on_close: true, min_width: 500, min_height: 400,
						title:  title
						});
					dlg.render();
					var content = dlg.getContentElement();
					UI.addClass(content, 'panel-outer panel-default panel-border');
					var div = createElement2('div', {className: 'panel-body', style: 'padding: 1em 2em'}, null, content);
					div.innerHTML = json.result.body;
					dlg.onclose = function() {
						UI.SystemMessage.showList(remain);
					}
					dlg.show();
				}
			});
    	}
   	}


	UI.Lightbox = function() {
		this.num = UI.Lightbox.num ++;
	}
	UI.Lightbox.num = 0;
	UI.Lightbox.show = function(img, title) {
		var l = new UI.Lightbox();
		l.showImage(img, title);
		return l;
	}
	UI.Lightbox.showGallery = function (images, index, total, parentType, parentId, limit) {
		const galleryLightbox = new GalleryLightbox(images, App.conn, total, parentType, parentId, limit);

		galleryLightbox.showImage(index || 0);
		return galleryLightbox;
	};
	UI.Lightbox.showInput = function(input, file, clear) {
		var l = new UI.Lightbox();
		l.showImageInput(input, file, clear);
		return l;
	}
	UI.Lightbox.prototype = {
		showImageInput: function(input, file, clear) {
			this.input = input;
			this.show_clear = clear;
			if(file)
				this.uploadImage(file, input.label);
			else
				this.showImage(input.path, input.label);
		},
		showImage: function(img, title, clear) {
			var t = this, i = (t.image = new Image());
			t.title = title;
			t.image_w = t.image_h = null;
			t.loaded = t.errored = false;
			i.onload = function() { t.handleLoad(); }
			i.onerror = function() { t.handleError(); }
			t.showPopup();
			if(img)
				setTimeout(function() { i.src = img; }, 0);
			else
				this.handleError();
		},
		initDialog: function() {
			var t = this, content;
			this.dlg = new UI.Dialog('lightbox-' + this.num, {
					destroy_on_close: true, min_width: 500, min_height: 100, tabIndex: 0,
					fill_content: true,
					onshow: function() { t.visible = true; },
					onhide: function() { t.visible = false; t.loaded = false; t.image = null; },
					onclose: function() { t.handleClose(); },
					onresize: function() { t.handleResize(); }
					});
			this.dlg.render();
			content = this.dlg.getContentElement();
			content.style.textAlign = 'center';
			this.outer = MakeElt('div.card-outer.panel-outer.panel-default.expand').appendTo(content);
			if(this.input && this.input.model && this.input.field_name) {
				this.edit = MakeElt('div.card-header.panel-subheader.tools-row').appendTo(this.outer);
				this.renderEdit();
			}
			this.inner = MakeElt('div.card-body.panel-body.panel-pad-body').appendTo(this.outer);
		},
		renderEdit: function() {
			var self = this;
			if(! this.file) {
				this.file = new UI.FileInput(null, {name: 'file'});
				this.file.onchange = function() { self.uploadImage(); }
				this.edit.append(MakeElt('div', this.file.render()));
			}
			if(! this.buttons) {
				this.buttons = MakeElt('div').appendTo(this.edit);
				if(this.show_clear) {
					this.clear_button = new UI.ButtonInput(null, {label: app_string('LBL_IMAGE_CLEAR'), icon: 'icon-delete'});
					this.clear_button.onclick = function() { self.clearImage(); }
					this.buttons.append(this.clear_button.render());
				}
			}
		},
		showPopup: function() {
			if(! this.dlg) this.initDialog();
			this.dlg.setTitleText(this.title || app_string('LBL_IMAGE'));
			this.updateImage();
			if(! this.visible)
				this.dlg.show();
			else
				this.dlg.reposition();
		},
		close: function() {
			if(this.dlg) this.dlg.close();
		},
		uploadImage: function(file, title) {
			if(isset(title)) this.title = title;
			if(! this.visible) this.showPopup();
			var req = new App.conn.HTTPRequest(), self = this,
				attrs = [
					{name: 'model', value: this.input.model},
					{name: 'field', value: this.input.field_name}];
			req.onupload = function(files) {
				var result;
				if(files && files[0] && files[0].result && files[0].result.is_image) {
					result = files[0].result;
					self.upload_result = result;
					if(self.close_on_upload)
						self.dlg.close();
					else
						self.showImage(result.path, self.title);
				}
			}
			req.addFile(this.file.elt, attrs, file ? [file] : null);
			req.force_revert = req.clear_on_revert = true;
			req.uploadFiles();
		},
		acceptUpload: function() {
			var u = this.upload_result;
			if(u && this.input) this.input.setValue('INCOMING~'+u.id, u.path, u.thumbnail);
			this.close();
		},
		clearImage: function() {
			if(this.input) this.input.clear();
			this.close();
		},
		updateImage: function() {
			if(! this.dlg) return;
			var t = this, href, lnk, b1, b2;
			UI.clearChildNodes(this.inner);
			if(this.loaded) {
				href = this.image.src;
				if(href && href.indexOf('download.php') >= 0) href += "&inline=1";
				this.inner.html("<div style=\"height: 22px; line-height: 22px\"><div class='input-icon icon-tlink'></div>&nbsp;<a href='" + href + "' target='_blank' class='tabDetailViewDFLink'>" +app_string('LNK_OPEN_IN_NEW_WINDOW')+ "</a></div>");
				this.inner.append(lnk = MakeElt('a', {href: href, target: '_blank'}, this.image));
				if(this.upload_result && this.upload_result.path) {
					UI.clearChildNodes(this.buttons);
					b1 = new UI.ButtonInput(null, {label: app_string('LBL_ACCEPT_BUTTON_LABEL'), icon: 'icon-accept'}),
					b1.onclick = function() { t.acceptUpload(); }
					b2 = new UI.ButtonInput(null, {label: app_string('LBL_CANCEL_BUTTON_LABEL'), icon: 'icon-cancel'});
					b2.onclick = function() { t.close(); }
					this.buttons.append(b1.render());
					this.buttons.append(nbsp());
					this.buttons.append(b2.render());
				}
			} else if(this.errored) {
				// show error message
			} else {
				this.inner.append(App.util.waitIcon());
			}
			this.dlg.reposition();
		},
		handleLoad: function() {
			this.image_w = this.image.width;
			this.image_h = this.image.height;
			if(! this.image_w || ! this.image_h)
				this.errored = true;
			else
				this.loaded = true;
			this.handleResize();
			this.updateImage();
		},
		handleError: function() {
			this.errored = true;
			this.updateImage();
		},
		handleResize: function() {
			var w = this.image_w, h = this.image_h, outer = UI.getViewport(), s;
			if(this.dlg && this.dlg.getFixedSize()) {
				outer = this.dlg.getContentSize();
				outer.height -= 22;
			} else {
				outer.width -= 100;
				outer.height -= 150;
			}
			if(w && h && (w > outer.width || h > outer.height)) {
				this.image.width = Math.min(outer.width, w);
				this.image.height = h = Math.round(h * this.image.width / w);
				if(h > outer.height) {
					this.image.height = outer.height;
					this.image.width = Math.round(this.image.width * outer.height / h);
				}
			}
		},
		handleClose: function() {
			if(this.input) this.input.focus();
			UI.callEvent(this, this.onclose);
		}
	};


	UI.ImageInput = function(id, params) {
		if(isElement(id)) {
			this.elt = id;
			this.id = this.elt.id;
		} else
			this.id = id;
		this.width = 80;
		this.height = 80;
		this.drag_focus = 0;
		if(params) extendObject(this, params);
	}
	UI.ImageInput.prototype = {
		setup: function() {
			if(this.set_up)
				return;
			if(! this.elt) {
				if(! (this.elt = $(this.id)) )
					return;
			}
			if(this.name && this.form) {
				this.field = this.form.elements[this.name];
				this.clear_field = this.form.elements[this.name+'_clear'];
			}
			else if(this.field_id)
				this.field = $(this.field_id);
			this.inner = this.elt.firstChild;
			this.hover = createElement2('div', {className: 'input-icon icon-popup image-popup-icon'});
			if(this.inner) {
				this.img = this.inner.firstChild;
				this.inner.appendChild(this.hover);
			}
			this.positionImage();
			this.addHooks();
			this.set_up = true;
		},
		positionImage: function() {
			var h1, h2;
			if(this.img && this.inner && this.img.height) {
				h1 = this.img.height;
				h2 = parseInt(this.inner.style.height, 10);
				if(h1 && h2)
					this.img.style.marginTop = '' + Math.max(0, Math.floor((h2 - h1) / 2)) + 'px';
			}
		},
		addHooks: function() {
			UI.addEventListener(this.elt, 'focus', this.handleFocus, this);
			UI.addEventListener(this.elt, 'blur', this.handleBlur, this);
			UI.addEventListener(this.elt, 'keypress', this.keyPress, this);
			UI.addEventListener(this.elt, 'dragenter', this.dragEnter, this);
			UI.addEventListener(this.elt, 'dragleave', this.dragLeave, this);
			UI.addEventListener(this.elt, 'drop', this.dragDrop, this);
			UI.addEventListener(this.inner, 'click', this.handleClick, this);
		},
		focus: function() {
			if(this.elt) this.elt.focus();
		},
		handleFocus: function(evt) {
			this.focused = 1;
			this.updateDisplay();
		},
		handleBlur: function(evt) {
			this.focused = 0;
			this.updateDisplay();
		},
		handleClick: function(evt) {
			if(! this.disabled) this.showPopup();
		},
		keyPress: function(evt) {
			if(evt && evt.charCode == 32) { // space
				if(! this.disabled) this.showPopup();
				UI.stopEvent(evt);
			}
		},
		showPopup: function(params) {
			UI.Lightbox.showInput(this, (params && params.file), (this.clear_field && this.path != this.clear_path));
		},
		setValue: function(value, path, thumb, silent) {
			if(this.field) this.field.value = value;
			if(this.img && this.img.parentNode) UI.removeNode(this.img);
			this.img = new Image();
			this.path = path;
			this.thumb = thumb;
			var self = this, tpath = thumb || path, clearWait = function() {
				if(self.wait && self.wait.parentNode) UI.removeNode(self.wait);
			}, posWait = function() {
				self.wait.style.marginTop = '' + Math.floor((self.height - self.wait.offsetHeight) / 2) + 'px';
			}
			if(! this.wait)
				this.wait = App.util.waitIcon();
			this.img.onload = function() {
				var w = this.width, h = this.height, rx, ry, r;
				if(w > self.width || h > self.height) {
					rx = self.width / w;
					ry = self.height / h;
					r = Math.min(rx, ry);
					this.width = (w = Math.round(w * r));
					this.height = (h = Math.round(h * r));
				}
				clearWait();
				self.positionImage();
				self.inner.appendChild(this);
			}
			this.img.onerror = clearWait;
			if(tpath) {
				this.inner.appendChild(this.wait);
				this.img.src = tpath;
				posWait();
			}
			if(! silent) UI.callEvent(this, this.onchange);
			UI.markFormDirty(this);
		},
		clear: function(silent) {
			if(this.clear_field) this.clear_field.value = '1';
			this.setValue('', this.clear_path, this.clear_thumb, silent);
		},
		render: function() {
			if(! this.elt) {
				this.elt = createElement2('div', {className: 'image-input', id: this.id});
				this.field = createElement2('input', {type: 'hidden', name: this.name}, null, this.elt);
				this.clear_field = createElement2('input', {type: 'hidden', name: this.name+'_clear'}, null, this.elt);
				this.inner = createElement2('div', {className: 'image-inner'}, null, this.elt);
				this.inner.style.width = '' + this.width + 'px';
				this.inner.style.height = '' + this.height + 'px';
				this.setup();
			}
			return this.elt;
		},
		updateDisplay: function() {
			UI.addRemoveClass(this.elt, 'active', (this.focused || this.drag_focus) && ! this.disabled);
		},
		destroy: function() {
			UI.removeNode(this.elt, true);
			this.elt = null;
		},
		dragEnter: function(evt) {
			this.drag_focus ++;
			this.updateDisplay();
		},
		dragLeave: function(evt) {
			this.drag_focus = Math.max(0, this.drag_focus - 1);
			this.updateDisplay();
		},
		dragDrop: function(evt) {
			if(evt && evt.dataTransfer.files) {
				this.showPopup({file: evt.dataTransfer.files[0]});
				UI.stopEvent(evt);
				this.drag_focus = 0;
			}
		}
	};


	UI.attachTeamsUpdate = function(form, fields) {
		var list = UI.getFormInput(form, 'teams');
		if(! list) return;
		if(! isset(fields))
			fields = ['assigned_user'];
		if(fields)
			for(var i = 0; i < fields.length; i++)
				UI.attachFormInputEvent(form, fields[i], 'onchange', function() { UI.updateTeamsList(form); });
	}
	UI.updateTeamsList = function(form, form_values, done) {
		UI.fetchUpdatedTeams(form, form_values, function(teams) {
			var list = UI.getFormInput(form, 'teams');
			UI.addRemoveTeams(list, teams);
			if(done) done();
		});
	}
	UI.addRemoveTeams = function(list, teams, rev) {
		if(! list || ! teams) return;
		var idx, t;
		if(! rev && list.prev_update)
			UI.addRemoveTeams(list, list.prev_update, true);
		if(rev) {
			t = teams.add;
			teams.add = teams.remove;
			teams.remove = t;
		}
		if(teams.add) {
			for(idx=0; idx < teams.add.length; idx++) {
				t = teams.add[idx];
				t.icon = 'theme-icon module-SecurityGroups';
				list.addOption(t.id, t, null, true);
			}
		}
		if(teams.remove) {
			for(idx=0; idx < teams.remove.length; idx++) {
				list.removeOption(teams.remove[idx].id);
			}
		}
		list.prev_update = teams;
	}
	UI.fetchUpdatedTeams = function(form, form_values, callback) {
		var req = new App.conn.JSONRequest('get_updated_teams'), elt, id;
		if(! form_values) form_values = {};
		form = UI.getForm(form);
		if(form) {
			for(id in form.elements) {
				elt = form.elements[id];
				if(elt.name && (elt.name == 'module' || elt.name == 'record' || elt.name.match(/_id$/)) && ! isset(form_values[elt.name]))
					form_values[elt.name] = elt.value;
			}
		}
		req.setPostData(form_values);
		req.fetch(function(ret) {
			var json;
			if(ret && (json = ret.getResult()) && json.teams) {
				callback(json.teams);
			}
		});
	};


	// Various methods for form and input field interaction

	UI.select_options = {};
	UI.all_forms = [];
	UI.form_inits = {};
	UI.getForm = function(form) {
		if(form && form.tagName === 'FORM')
			return form;
		if(typeof(form) == 'string') {
			if(document.forms[form])
				form = document.forms[form];
			else
				form = $(form);
			if(form)
				return form;
		}
		return false;
	}
	UI.getFormId = function(form) {
		form = this.getForm(form);
		if(form && ! isString(form)) form = form.id;
		return form ? form : null;
	}

	UI.registerSelectOptions = function(name, options) {
		if(options instanceof UI.SelectOptions)
			this.select_options[name] = options;
		else
			this.select_options[name] = new UI.SelectOptions(options);
	}
	UI.getSelectOptions = function(dom, module, lang, dupe) {
		var opts;
		if(dom) {
			if(dom instanceof UI.SelectOptions)
				opts = dom;
			else if(isString(dom)) {
				if(! module && UI.select_options[dom])
					opts = UI.select_options[dom];
				else if(dom.substring(0, 6) == 'model.')
					opts = App.modelCache.getModelData(dom.substring(6));
				else {
					opts = App.language.getList(dom, module, lang);
					if(opts) {
						opts = new UI.SelectOptions(opts.getOptions());
						dupe = false;
					}
				}
			}
			else if(isPlainObject(dom)) {
				opts = new UI.SelectOptions(dom);
				dupe = false;
			}
		}
		if(opts) {
			if(dupe) opts = deep_clone(opts);
			if(! opts.keys) opts.keys = [];
			if(! opts.values) opts.values = [];
		}
		return opts;
	}
	UI.global_menus = {};
	UI.menu_groups = {};
	UI.registerMenuSource = function(elt, params, form, group) {
		var m = new UI.MenuSource(elt, params);
		if(m.id) {
			form = this.getForm(form);
			if(form) {
				m.form = form;
				if(! form._regMenus) form._regMenus = {};
				form._regMenus[m.id] = m;
			} else {
				UI.global_menus[m.id] = m;
			}
			if(group) {
				if(! this.menu_groups[group]) this.registerMenuGroup(group, form, [m.id]);
				else this.menu_groups[group].entries.push(m.id);
			}
		}
		m.setup(); // may need to defer this if element doesn't exist yet
		return m;
	}
	UI.getMenuSource = function(form, name) {
		form = this.getForm(form);
		if(form && form._regMenus)
			return form._regMenus[name];
		else
			return UI.global_menus[name];
	}
	UI.registerMenuGroup = function(grpid, form, entries) {
		form = this.getFormId(form);
		this.menu_groups[grpid] = {form: form, entries: entries};
	}
	UI.getMenuGroupPos = function(form, name) {
		var ret, grp, i, j, c;
		form = this.getFormId(form);
		for(i in this.menu_groups) {
			grp = this.menu_groups[i];
			if(grp.form != form) continue;
			for(j = 0; j < grp.entries.length; j++) {
				if(grp.entries[j] === name) {
					ret = {group: i, pos: j};
					c = grp.entries.length;
					if(c > 1) {
						ret.prev = this.getMenuSource(grp.form, grp.entries[j > 0 ? j - 1 : c-1]);
						ret.next = this.getMenuSource(grp.form, grp.entries[j < c-1 ? j + 1 : 0]);
					}
					return ret;
				}
			}
		}
	}

	UI.registerInputs = function(form, inputs) {
		var i, inp, reg, c = 0, setup = [];
		form = this.getForm(form);
		if(form && inputs && inputs.length) {
			if(! form._regInputs) {
				form._regInputs = {};
				form._regNames = [];
			}
			for(i = 0; i < inputs.length; i++) {
				inp = inputs[i];
				reg = inp.name || inp.id;
				if(! reg)
					continue;
				form._regInputs[reg] = inp;
				form._regNames.push(reg);
				inp.form = form;
				c ++;
			}
			if(form._inited || form._initing) {
				for(i = 0; i < inputs.length; i++) {
					inputs[i].setup && inputs[i].setup();
				}
			}
		}
		return c;
	}
	UI.registerInput = function(form, input) {
		var c = UI.registerInputs(form, [input]);
		return c ? input : false;
	}
	UI.unregisterInputs = function(form, inputs) {
		var c = 0;
		if(inputs && inputs.length) {
			for(var i = 0; i < inputs.length; i++) {
				if(UI.unregisterInput(form, inputs[i])) c++;
			}
		}
		return c;
	}
	UI.unregisterInput = function(form, input) {
		form = this.getForm(form);
		if(input && ! isString(input))
			input = input.name || input.id;
		if(form && input && form._regInputs && form._regInputs[input] && form._regNames) {
			delete form._regInputs[input];
			form._regNames.remove(input);
			return true;
		}
		return false;
	}

	UI.getFormInput = function(form, id) {
		form = this.getForm(form);
		if(form && form._regInputs)
			return get_default(form._regInputs[id], null);
	}
	UI.getFormInputs = function(form) {
		form = this.getForm(form);
		return form ? form._regInputs : null;
	}
	UI.getFormInputNames = function(form) {
		form = this.getForm(form);
		return form && form._regNames || [];
	}

	UI.focusInput = function(form, id) {
		var input = this.getFormInput(form, id);
		if(input && input.focus) input.focus();
	}
	UI.getFormInputProperty = function(form, id, name) {
		var elt = this.getFormInput(form, id);
		if(elt) return elt[name];
	}
	UI.setFormInputProperty = function(form, id, name, value) {
		var elt = this.getFormInput(form, id);
		if(elt) elt[name] = value;
	}
	UI.setFormInputValue = function(form, id, value) {
		var form = this.getForm(form),
			elt = this.getFormInput(form, id);
		if(elt && elt.setValue) elt.setValue(value);
		else if(! elt && form && form.elements[id]) form.elements[id].value = value;
	}
	UI.setFormInputValues = function(form, value_map) {
		if(! value_map) return;
		var form = this.getForm(form), elt, key, value, id_name;
		for(var id in value_map) {
			elt = this.getFormInput(form, id);
			value = get_default(value_map[id], '');
			if(elt instanceof UI.RefInput) {
				id_name = elt.key_name;
				if(id_name && id_name in value_map) {
					key = get_default(value_map[id_name], '');
					elt.update(key, value);
				}
			}
			else if(elt && elt.setValue) elt.setValue(value);
			else if(! elt && form && form.elements[id]) form.elements[id].value = value;
		}
	}

	UI.initForm = function(form, inputs, postInit, focus) {
		form = UI.getForm(form);
		if(! form) return;
		form._initing = true;
		if(inputs)
			UI.registerInputs(form, inputs);
		UI.checkFormInits(form);
		var allInputs = UI.getFormInputs(form);
		if(allInputs) {
			for(var k in allInputs) {
				allInputs[k].form = form;
				if(allInputs[k].setup)
					allInputs[k].setup();
			}
		}
		if(postInit)
			UI.onInitForm(form, postInit);
		form._inited = true;
		form._initing = false;
		UI.tidyForms();
		UI.all_forms.push(form);
		if(form._postInit)
			UI.callEvent(form, form._postInit, form);
	}

	UI.focusForm = function(form, controls, handler) {
		var p = UI.PopupManager.getLoadingPopup();
		if(p) {
			if(isIPhone) return; // skip autofocus - can force display of keyboard which is disruptive and buggy in iOS8
			UI.attachInputEvent(p, 'onshow', function() { UI.focusForm(form, controls, handler); });
			return;
		}
        form = UI.getForm(form);
		var doFocus = function(cs, reqd, blank) {
			if(! cs) return;
			var i, c;
			for(i = 0; i < cs.length; i++) {
				c = UI.getFormInput(form, cs[i]);
				if(c && c.focus && (! reqd || c.required)) {
					if(blank) {
						if(c.isBlank) {
							if(! c.isBlank()) continue;
						} else if(c.getValue) {
							if(c.getValue()) continue;
						}
					}
					try { c.focus(); } catch(e) { console.error(e); }
					return true;
				}
			}
		}
        if (handler) {
            p = UI.callEvent(form, handler, form);
            if(p !== false) return true;
        }
		var ns = UI.getFormInputNames(form);
		return doFocus(controls) ||
			doFocus(ns, true, true) ||
			doFocus(ns, false, true) ||
			doFocus(ns) ||
			(form.focus && form.focus());
	}

	UI.tidyForms = function() {
		var remain = [];
		for(var i = 0; i < UI.all_forms.length; i++) {
			if(! UI.inDocument(UI.all_forms[i]) && ! UI.PopupManager.inRenderedPopup(UI.all_forms[i]))
				UI.destroyForm(UI.all_forms[i]);
			else
				remain.push(UI.all_forms[i]);
		}
		UI.all_forms = remain;
	}

	UI.destroyForm = function(form) {
		//console.debug('destroying form: ', form);
		form = UI.getForm(form);
		var inputs = UI.getFormInputs(form),
			inp;
		if(inputs) {
			for(var k in inputs) {
				inp = inputs[k];
				if(inp.form === form) {
					if(inp.destroy) inp.destroy();
					delete inp.elt;
					delete inp.field;
					delete inp.form;
				}
			}
		}
		if(isset(form._regInputs)) delete form._regInputs;
		if(isset(form._regMenus)) delete form._regMenus;
		if(form._destroy)
			UI.callEvent(form, form._destroy, form);
	}

	UI.checkFormInits = function(form) {
		var form_id = form.id || form.name;
		if(form_id && UI.form_inits[form_id]) {
			if(form._postInit)
				form._postInit.pushArray(UI.form_inits[form_id]);
			else
				form._postInit = UI.form_inits[form_id];
			delete UI.form_inits[form_id];
		}
	}

	UI.onInitForm = function(form_name, handler) {
		if(! handler) return false;
		var form = UI.getForm(form_name);
		if(! form) {
			if(typeof(form_name) == 'string') {
				if(! UI.form_inits[form_name])
					UI.form_inits[form_name] = [];
				UI.form_inits[form_name].push(handler);
				return true;
			}
			return false;
		}
		UI.checkFormInits(form);
		if(form._inited)
			UI.callEvent(form, handler, form);
		else if(form._postInit)
			form._postInit.push(handler);
		else
			form._postInit = [handler];
		return true;
	}

	UI.onDestroyForm = function(form_name, handler) {
		var form = UI.getForm(form_name);
		UI.attachInputEvent(form, '_destroy', handler);
	}

	UI.validateForm = function(form) {
		var valid = true, allInputs = UI.getFormInputs(form), lbl, messages = [];
		if(allInputs) {
			for(var k in allInputs) {
				if(allInputs[k].validate) {
					try {
						if(! allInputs[k].validate()) {
							valid = false;
							lbl = allInputs[k].getLabel ? allInputs[k].getLabel() : allInputs[k].label;
							messages.push({name: k, label: lbl, message: allInputs[k].invalidMsg});
						}
					} catch(e) {
						valid = false;
						console.error('error validating '+k, e);
					}
				}
			}
		}
		return {status: valid, messages: messages};
	}

	UI.validationSummary = function(rows) {
		var missing = [], invalid = [], other = [], ret = [], show, i;
		if(rows) {
			for(i = 0; i < rows.length; i++) {
				show = rows[i].label || rows[i].name;
				if(rows[i].message === 'required')
					missing.push(show);
				else if(rows[i].message === 'invalid' || ! rows[i].message)
					invalid.push(show);
				else
					other.push('' + show + ': ' + rows[i].message);
			}
		}
		if(missing.length)
			ret.push(app_string('ERR_MISSING_REQUIRED_FIELDS') + ': ' + missing.join(', '));
		if(invalid.length)
			ret.push(app_string('ERR_INVALID_VALUE') + ': ' + invalid.join(', '));
		ret.pushArray(other);
		return ret.join("\n");
	}

	UI.alertInvalidForm = function(vresult) {
		alert(UI.validationSummary(vresult.messages));
	}

	UI.submitForm = function(form) {
		var elt, i = 0;
		form = UI.getForm(form);
		if(form) {
			for(; i < form.elements.length; i++) {
				elt = form.elements[i];
				if(elt.type === 'submit' && elt.click)
					return elt.click();
			}
			if(form.onsubmit && form.onsubmit() === false)
				return false;
			return form.submit();
		}
	}

	UI.updateForm = function(form, field_params) {
		var restore = {}, v, i, field;
		form = UI.getForm(form);
		if(! form)
			return null;
		if(field_params) {
			for(var k in field_params) {
				v = field_params[k];
				field = form.elements[k];
				if(field && field.length) {
					var arest = [];
					for(i = 0; i < field.length; i++)
						arest.push(field[i].value);
					restore[k] = arest;
					for(i = field.length-1; i > 0; i--)
						UI.removeNode(field[i]);
					field = field[0];
				} else if(field) {
					restore[k] = field.value;
				} else {
					restore[k] = null;
				}
				if(! isset(v)) {
					if(field) UI.removeNode(field);
					delete field;
				}
				else if(Array.isArray(v)) {
					if(field) UI.removeNode(field);
					delete field;
					for(i = 0; i < v.length; i++) {
						field = createElement2('input', {name: k, type: 'hidden', value: v[i]});
						form.appendChild(field);
					}
				}
				else {
					if(! field) {
						field = createElement2('input', {name: k, type: 'hidden', value: v});
						form.appendChild(field);
					}
					field.value = v;
				}
			}
		}
		return restore;
	}

	UI.cancelEdit = function(form, evt) {
		form = UI.getForm(form);
		if(! form)
			return false;
		if(form.in_popup && form.in_popup.value) {
			App.popups.close();
			return false;
		}
		UI.clearCall();
		if(form.return_url && form.return_url.value) {
			App.util.navigateTo(form.return_url.value, evt, {ignoredirty: true});
		}
		else if(form.return_module && form.return_module.value) {
			var ret_mod = form.return_module.value, ret_act, ret_rec, ret_lay;
			if(form.return_action) ret_act = form.return_action.value;
			if(form.return_record) ret_rec = form.return_record.value;
			if(form.return_layout) ret_lay = form.return_layout.value;
            //if(form.layout) ret_lay = form.layout.value;
			if(! ret_act || (ret_act == 'DetailView' && ! ret_rec)) ret_act = 'index';
			App.util.navigateTo(
				{module: ret_mod, action: ret_act, record: ret_rec, layout: ret_lay},
				evt, {ignoredirty: true});
		}
		else if(form.module && form.module.value) {
			UI.returnToList(form);
		}
		return false;
	}

	UI.returnToList = function(form, evt) {
		form = UI.getForm(form);
		if(! form)
			return false;
		 if(form.module && form.module.value) {
			App.util.navigateTo({module: form.module.value, action: 'index'},
				evt, {ignoredirty: true});
		}
	}

	UI.createDuplicateFrom = function(form, target_module, target_id, link_name, param_name) {
		form = UI.getForm(form);
		if(! form)
			return false;
		var mod = form.module.value,
			act = form.action.value,
			layout = form.layout.value,
			rec_id = form.record.value,
			action = target_id ? 'Duplicate' : 'EditView',
			url = 'index.php?module='+target_module+'&action=' + action + '&record='+target_id+
				'&return_module='+mod+'&return_action='+act+'&return_record='+rec_id+'&return_layout='+layout;
		if(link_name)
			url += '&return_panel='+link_name;
		if(param_name)
			url += '&'+param_name+'='+rec_id;
		App.util.navigateTo(url);
	}

	UI.applyFormInputsHook = function(form, method) {
		form = UI.getForm(form);
		var allInputs = UI.getFormInputs(form);
		if(allInputs) {
			for(var k in allInputs) {
				if(allInputs[k][method]) {
					try {
						var result = allInputs[k][method](form);
						if (typeof(result) != 'undefined' && !result)
							return false;
					} catch(e) {
						console.error('error in ' + method + ' ' + k, e);
						return false;
					}
				}
			}
		}
		return true;
	}

	UI.beforeDetachForm = function(form) {
		return UI.applyFormInputsHook(form, 'beforeDetachForm');
	}

	UI.beforeRestoreForm = function(form) {
		return UI.applyFormInputsHook(form, 'beforeRestoreForm');
	}

	UI.beforeSubmitForm = function(form) {
		return UI.applyFormInputsHook(form, 'beforeSubmitForm');
	}

	var preUpload = function(form, complete) {
		var req = new App.conn.HTTPRequest();
		var hasFiles = false;
		req.onupload = function(files) {
			var result;
			complete();
		}

		for(var i = 0; i < form.elements.length; i++) {
			var elt = form.elements[i];
			if (elt.type == 'file' && elt.value) {
				req.addFile(elt, {}, null);
				hasFiles = true;
			}
		}
		if (hasFiles) {
			//req.force_revert = req.clear_on_revert = true;
			req.uploadFiles();
		} else {
			complete();
		}
	};


	UI.sendForm = function(form, field_params, conn_params, target, from_popup) {
		var result, restore, restore_params, vresult, req;

		form = UI.getForm(form);
		if(! form)
			return null;
		if(! conn_params) conn_params = {};
		if (field_params && isObject(field_params)) {
			if (!field_params.format)
				field_params.format = 'html';
		}
		if(SUGAR.session.request_method == 'POST') {
			if(! target && ! conn_params.receiveCallback && get_default(conn_params.resultDiv, 'content-main') == 'content-main') {
				target = true;
			}
		}
		if(target) {
			restore_params = UI.updateForm(form, field_params);
			restore = {action: form.getAttribute('action'), target: form.getAttribute('target')};
			if(conn_params.url)
				form.action = conn_params.url;
			else if(form.getAttribute('method') == 'POST')
				form.action = 'index.php';
			if(target === true) target = '_self';
			form.target = target;
		}
		vresult = UI.checkForm(form, conn_params.no_validate);
		if(isdef(vresult)) return vresult;

		if(target) {
			var complete = function() {
				if(Get_Cookie('ANDROID_APP'))
					form.target = '_self';
				form.submit();
				setTimeout(function() {
					setAttrs(form, restore);
					UI.updateForm(form, restore_params);
				}, 0);
			};
			preUpload(form, complete);
			return false;
		} else {
			App.session.setLoadingState(null, null, true);
		}

		if(! conn_params.status_msg) {
			var perform;
			if(field_params && field_params.record_perform)
				perform = field_params.record_perform;
			else if(form.elements.record_perform)
				perform = form.elements.record_perform.value;
			else if(form.elements.list_layout_perform)
				perform = form.elements.list_layout_perform.value;
			if(perform == 'save') {
				conn_params.status_msg = app_string('LBL_SAVING');
			}
			else if(perform == 'validate') {
				conn_params.status_msg = app_string('LBL_VALIDATING');
			}
		}

        if (from_popup) {
        	if(! conn_params.popupDialog)
				conn_params.popupDialog = window.popup_dialog || App.popups.getCurrent();
			if(field_params && field_params.reload_after)
				conn_params.reloadAfter = 1;
			if(field_params && field_params.close_popup)
				conn_params.closePopup = 1;
        }

		if(! isset(conn_params.resultDiv)) {
			if (conn_params.popupDialog)
				conn_params.resultDiv = conn_params.popupDialog.getContentElement();
			else
				conn_params.resultDiv = 'content-main';
		}
		if(conn_params.resultDiv && ! conn_params.disableMask)
			conn_params.maskId = App.util.maskDiv(conn_params.resultDiv);
		form._sending = true;
		req = App.conn.sendForm(form, conn_params, UI.receiveForm, UI.failForm, field_params);
		App.session.setLoadingRequest(req);
		return false;
    }

	UI.arrangeDetailTabs = function(module, id) {
		var dlg = new SUGAR.ui.Dialog('arrangeDetailTabs');
		var vars = {
			postData: {
				module: 'NewStudio',
				action: 'index',
				wizard: 'ArrangeTabs',
				record: id,
				layout_module: module
			}
		};
		dlg.fetchContent('async.php', vars, null, true);
	};

	UI.checkForm = function(form, no_validate) {
		var result;
		form = UI.getForm(form);
		if(! no_validate && ! (result = UI.validateForm(form)).status) {
			UI.alertInvalidForm(result);
			return false;
		}
		result = UI.beforeSubmitForm(form);
		if(isdef(result) && !result)
			return false;
		UI.clearCall();
		UI.markFormDirty(form, false);
	}

	UI.failForm = function(result, conn) {
		if(this.sentForm) this.sentForm._sending = false;
		App.session.setLoadingRequest(null);
		if(this.maskId) {
			App.util.hideMask(this.maskId);
		}
		if (conn && conn.failure_msg)
			UI.Status.show(conn.failure_msg, 4000);
	}

	UI.receiveForm = function(result) {
		var req = this;
		var done = function() {
			App.session.setLoadingRequest(null);
		}
		if(req.sentForm) req.sentForm._sending = false;
		if(result) {
			var div, body = req.responseText, cb_ret, doScroll,
				noDisplay = body.indexOf('<script nodisplay>') >= 0;
			if(req.resultDiv) div = $(req.resultDiv);
			if(div && ! noDisplay) {
				div.innerHTML = App.util.disableInlineScripts(body);
				doScroll = ! req.noScrollTo
			}
			App.util.evalScript(body, function() {
				if(req.receiveCallback)
					cb_ret = req.receiveCallback(result);
				if (req.popupDialog && (req.closePopup || req.reloadAfter || noDisplay) && ! cb_ret) {
					req.popupDialog.close();
					doScroll = false;
				}
				if (req.reloadAfter && ! cb_ret) {
					setTimeout ("document.location.reload(true)", 200);
					doScroll = false;
				}
				if(doScroll)
					App.util.scrollToElement(div);
				done();
			});
		}
		else {
			// FIXME show connection error
			console.error('retrieve failed');
			if(req.maskId) {
				App.util.hideMask(req.maskId);
			}
			done();
		}
	}

	UI.FormRender = function(params) {
		this.style = ''; //'default-form';
		this.card = false;
		this.tagName = 'form';
		if(params) extendObject(this, params);
		this.render();
	}
	UI.FormRender.prototype = {
		render: function() {
			if(! this.elt) {
				var cls = 'form-outer' + (this.card ? ' card-outer': '');
				if(this.style) cls += ' ' + this.style;
				if(this.ext_class) cls += ' ' + this.ext_class;
				this.elt = MakeElt(this.tagName, {id: this.id, 'class': cls});
			}
			if(! this.body) {
				this.body = MakeElt('div.form-body' + (this.card ? '.card-body' : '')).appendTo(this.elt);
			}
			return this.elt;
		},
		addButtons: function(buttons, params) {
			if(! params) params = {};
			if(! isset(params.card)) params.card = this.card;
			if(! isset(params.form) && this.tagName === 'form') params.form = this.elt.get();
			params.buttons = buttons;
			var sect = new UI.FormButtons(params);
			this.elt.append(sect.elt);
			return sect;
		},
		addSection: function(params) {
			var sect = new UI.FormSection(params);
			this.body.append(sect.elt);
			return sect;
		}
	};

	UI.FormSection = function(params) {
		if(params instanceof UI.FormSection) return params;
		this.columns = 1;
		var fields;
		if(params) {
			fields = params.fields;
			delete params.fields;
			extendObject(this, params);
		}
		this.render();
		if(fields) {
			this.addFields(fields);
		}
	}
	UI.FormSection.prototype = {
		render: function() {
			if(! this.elt) {
				this.elt = MakeElt('div.form-section');
			}
		},
		addRow: function(params) {
			return MakeElt('div.row.form-row').appendTo(this.elt);
		},
		renderEntry: function(params) {
			if(! params) params = {};
			var cell, entry;
			cell = MakeElt('div.column.form-cell');
			entry = MakeElt('div.form-entry').appendTo(cell);
			if(params.label) {
				entry.append(MakeElt('div.form-label').html(params.label));
			}
			if(params.value) {
				entry.append(MakeElt('div.form-value').html(params.value));
			}
			return cell;
		},
		addFields: function(fields) {
			var col = 0;
			var row, field, cell, cpar, entry;
			for(var i = 0; i < fields.length; i++) {
				field = fields[i];
				cpar = {label: field.label, value: field.value};
				if(field.input) {
					if(! ('label' in field))
						cpar.label = field.input.label;
					cpar.value = field.input.render();
					field.input.setup();
				}
				else if(field.render) {
					// probably an input control
					cpar.value = field.render();
					field.setup();
				}
				cell = this.renderEntry(cpar);
				if(! row)
					row = this.addRow();
				row.append(cell);
				if(++ col >= this.columns)
					row = null;
			}
			// add spacer if necessary
		}
	};

	UI.FormButtons = function(params) {
		if(params instanceof UI.FormButtons) return params;
		var buttons, columns;
		if(params) {
			buttons = params.buttons;
			delete params.buttons;
			columns = params.columns;
			delete params.columns;
			extendObject(this, params);
		}
		this.render();
		if(columns) {
			this.addColumns(columns);
		}
		else if(buttons) {
			this.addButtons(buttons);
		}
	}
	UI.FormButtons.prototype = {
		render: function() {
			if(! this.elt) {
				this.elt = MakeElt(
					'div.form-buttons' + (this.card ? (this.footer ? '.card-footer' : '.card-header') : ''));
			}
			this.addRow();
		},
		addRow: function(params) {
			this.row = MakeElt('div.row.buttons-row').appendTo(this.elt);
			return this.row;
		},
		addColumns: function(params) {
			if(Array.isArray(params)) {
				for(var idx = 0; idx < params.length; idx++)
					this.addColumn(params[idx]);
			}
		},
		addColumn: function(params) {
			if(! params) params = {};
			var align = params.align || this.align || 'left',
				cls = params.className || 'column buttons-inline align-' + align;
				elt = MakeElt('div', {'class': cls}).appendTo(this.row);
			if(params.buttons) {
				this.addButtons(params.buttons, {column: elt});
			}
			return elt;
		},
		addButtons: function(buttons, params) {
			if(! params) params = {};
			var column = params.column || this.addColumn({align: params.align});
			if(Array.isArray(buttons)) {
				for(var idx = 0; idx < buttons.length; idx++) {
					var btn = this.renderButton(buttons[idx]);
					MakeElt('div.inline-elt', btn).appendTo(column);
				}
			}
		},
		renderButton: function(params) {
			var btn;
			if(params instanceof UI.ButtonInput)
				btn = params;
			else
				btn = new UI.ButtonInput(null, params);
			if(this.form) UI.registerInput(this.form, btn);
			return btn.render();
		},
		makeIcon: function(name) {
			if(~name.indexOf('uii')) return MakeElt('span', {'class': name});
			else return MakeElt('div', {'class': 'input-icon ' + name});
		}
	};


	UI.callEvent = function(elt, handler) {
		if(arguments.length > 2) {
			var args = Array.prototype.slice.call(arguments, 2);
			return UI.applyEvent(elt, handler, args);
		} else
			return UI.applyEvent(elt, handler);
	}

	UI.applyEvent = function(elt, handler, args) {
		if(! handler) return;
		if(! args) args = []; // IE8 requires this
		if(isString(handler)) {
			var f = resolveVar(handler);
			if(! f)
				f = new Function(handler);
			return f.apply(elt, args);
		} else if(Array.isArray(handler)) {
			var ret;
			for(var i = 0; i < handler.length; i++)
				ret = UI.applyEvent(elt, handler[i], args);
			return ret;
		} else if(handler.apply)
			return handler.apply(elt, args);
	}

	UI.attachFormInputEvent = function(form, id, evt_name, handler) {
		var ctl = UI.getFormInput(form, id);
		if(ctl) return UI.attachInputEvent(ctl, evt_name, handler);
	}

	UI.attachInputEvent = function(input, evt_name, handler) {
		if(typeof(input) != 'object' || ! input || ! handler) return false;
		if(Array.isArray(handler)) {
			var ret = true, i;
			for(i = 0; i < handler.length; i++)
				ret |= UI.attachInputEvent(input, evt_name, handler[i]);
			return ret;
		}
		else if(input[evt_name] && input[evt_name].push)
			input[evt_name].push(handler);
		else if(input[evt_name])
			input[evt_name] = [input[evt_name], handler];
		else
			input[evt_name] = [handler];
		return true;
	}

	UI.changeLayout = function(form, layout) {
		return UI.sendForm(form, {layout: layout, format: 'html'});
	}

	UI.copyAddress = function(form, from_addr, to_addr) {
		var inp = UI.getFormInput(form, to_addr);
		if(inp) inp.copyFrom(from_addr);
	}

    UI.openQuickCreate = function(module, params, callback) {
    	var c = 0;
    	for(; UI.getForm('QuickCreateForm_' + c); c++) {}
        var query = 'mgr_type=quick_create&record_perform=edit&qc_count=' + c + '&';
        query = encodeQueryString(params, query);
        if (module == 'Booking' && isset(params.return_module))
            query += '&related_type=' + params.return_module;
        App.session.quick_create_callback = callback;
        return UI.openEditPopup(module, query, undefined, params && params.dialog_params);
   	}

    UI.openCallLogging = function(params) {
        var query = 'mgr_type=call_logging&record_perform=edit&';
        query = encodeQueryString(params, query);
        return UI.openEditPopup('Calls', query);
   	}

    UI.openEditPopup = function(module, query, record, dialog_params) {
        var options = {module: module, action: 'EditView', inline: true, query_string: query, width: module=='Emails'?850:750, min_height: 100};
        if(record) options.filter = {record: record};
        return open_popup_window(options, dialog_params);
    }

    UI.saveQuickCreate = function(form, view_after, params) {
        function ready(result) {
			var popup_elt = result.popupDialog.elt;
			popup_elt.visibility = 'hidden';
			result.popupDialog.loadContent(result.responseText, function() {
				if(App.session.quick_create_result) {
					App.popups.close();
					if(App.session.quick_create_callback) {
						App.session.quick_create_callback(App.session.quick_create_record, App.session.quick_create_name);
						App.session.quick_create_callback = null;
					} else if (! view_after)
						UI.refreshTarget(params.target_name, params.target_id);
					delete App.session.quick_create_result;
				} else {
					popup_elt.style.visibility = 'normal';
				}
			}, false);
            return true;
        }

        try {
            return UI.sendForm(form, {record_perform: "save", go_to_view: view_after}, {receiveCallback: ready}, false, true);
        } catch(e) {
            console.error(e); return false;
        }
    }

	UI.authorizeCard = function(form) {
		var ready = function(result) {
		}
        try {
            return UI.sendForm(form, {record_perform: "save"}, {receiveCallback: ready}, false, true);
        } catch(e) {
            console.error(e); return false;
        }
	}

    UI.refreshTarget = function(target_name, target_id, callback) {
        if (target_name == 'subpanel') {
            var frm = target_id + '-ListUpdate';
            var conn_params = {resultDiv: '' + target_id + '-outer', receiveCallback: callback};
            return ! UI.sendForm(frm, {"record_perform": "subpanel_render"}, conn_params);
		} else if (target_name == 'subpanel_by_name') {
			var ids;
			if(! Array.isArray(target_id))
				ids = target_id.split(',');
			else
				ids = target_id;
			for(var i = 0; i < ids.length; i++)
				sugarListView.reloadSubpanel(ids[i]);
		} else if (target_name == 'listview') {
			sListView.navigate(target_id);
        } else if(target_name == 'DetailForm') {
        	var frm = UI.getForm('DetailForm');
        	if(frm && frm.action.value == 'DetailView' && (! frm.record_perform || frm.record_perform.value == 'view' || frm.record_perform.value == 'edit')) {
				var conn_params = {resultDiv: 'content-main', receiveCallback: callback};
        		return ! UI.sendForm(frm, {}, conn_params);
        	}
        }
		return false;
    }

	UI.loadFullForm = function(form, params) {
        var field_params = {record_perform: 'edit', 'mgr_type': '', action: 'EditView', in_popup: 0};
        if(! form[params.primary_name] || ! form[params.primary_name].value)
            field_params[params.primary_name] = params.primary_value;

        if (params.hasOwnProperty('module'))
            field_params.module = params.module;

        if (params.target_layout != '')
            field_params.layout = params.target_layout;
		else if (!params.target_layout)
            field_params.layout = '';

		if (params.return_record) {
			field_params.return_record = params.return_record;
			field_params.return_module = params.return_module;
			field_params.return_action = params.return_action || 'DetailView';
		}

        function ready() { App.popups.close(); }

        try {
            return UI.sendForm(form, field_params, {no_validate: true, receiveCallback: ready}, true);
        } catch(e) {
            console.error(e); return false;
        }
    }

    UI.clearCall = function() {
		clearTimeout(UI.call_out_timer);
		UI.call_out_timer = null;
		UI.call_out_timer = null;
		UI.call_button_label = null;
	};

    UI.makeCall = function(form, type, button, edit) {
        var number = '';
        var number_inp = UI.getFormInput(form, 'phone_number');
        if (number_inp) {
            number = number_inp.getValue();
        } else if (document.DetailForm.phone_number) {
            number = document.DetailForm.phone_number.value;
        }

		if (this.call_out_timer) {
			clearTimeout(this.call_out_timer);
			var b = document.getElementById(button.id + '-label');
			if (b && this.call_button_label) {
				b.innerText = this.call_button_label;
			}
			this.call_out_timer = null;
			this.call_button_label = null;
			var delta = (new Date()).getTime() - this.call_start_time.getTime();
			if (!edit) {
				var dt = this.call_start_time;
				var startText = dt.print(getDateFormat().print_format) + ' ' + dt.print(getTimeFormat().print_format);
				if (confirm(app_string('LBL_CALL_OUT_CONFIRM')))
					UI.sendForm(form, {"record_perform":"save", "status": "Held", "date_start" : startText, "duration" : (delta/60000).toFixed(0)}, null);
			} else {
            	var status_sel = UI.getFormInput(form, 'status');
	            if (status_sel)
    	            status_sel.setValue('Held');
				var duration_input = UI.getFormInput(form, 'duration');
				if (duration_input)
					duration_input.setValue((delta/60000).toFixed(0));
				var date_input = UI.getFormInput(form, 'date_start');
				if (date_input)
					date_input.setValue(this.call_start_time);

			}
		} else if (number.match(/^[0-9+().-]/)) {
            var status_sel = UI.getFormInput(form, 'status');
            if (status_sel)
                status_sel.setValue('Held');

			this.call_start_time = new Date();
			clearTimeout(this.call_out_timer);
			var b = document.getElementById(button.id + '-label');
			if (b) {
				this.call_button_label = b.innerText;
			}
			var self = this;
			var callTimer = function() {
				if (button) {
					var b = document.getElementById(button.id + '-label');
					if (b) {
						var delta = ((new Date()).getTime() - self.call_start_time.getTime()) / 1000;
						var hours = '' + parseInt(delta/3600);
						var minutes = '' + parseInt(delta % 3600 / 60);
						var seconds = '' + parseInt(delta % 60);
						if (seconds.length < 2)
							seconds = '0' + seconds;
						var text =  minutes + ':' + seconds;
						if (hours > 0) {
							if (minutes.length < 2)
								text = '0' + text;
							text = hours + ':' + text;
						}
						b.innerText = '[' + text + ']';
					}
				}
				self.call_out_timer = setTimeout(callTimer, 1000);
			};
			this.call_out_timer = setTimeout(callTimer, 300);

            if (type == 'tel') {
				return App.util.loadUrl('tel:' + number.replace(/[ \(\)\-\.]/g, ''), false);
			} else if (type == 'skypeout') {
				return App.util.loadUrl('skype:' + number.replace(/[ \(\)\-\.]/g, ''), false);
			} else if (type == 'ringcentral') {
                return RingCentralOneCRM.initiateCall(number);
			} else if ( type == 'cti'){
				App.conn.asyncRequest('cti/placeCall.php?phoneNr=' + encodeURIComponent(number));
				return true;
			} else {
                return false;
            }
        } else {
            alert(app_string('ERR_WRONG_PHONE_NUMBER'));
        }

    }

    UI.quickEdit = function(params, evt) {
        var edit_elt = new UI.QuickElt(params.id, params);
        edit_elt.setup();
        edit_elt.show();
        if(evt) UI.stopEvent(evt);
    }

    UI.setupEmailParentsMenu = function(form_id, target_id, options, record_id) {
        UI.registerMenuSource(target_id, {icon_key: 'icon', stacked: true, inline: true, zIndex: 100000}, form_id);
		var menu_button = UI.getMenuSource(form_id, target_id);
		if (!menu_button || !menu_button.menu) return;
        UI.attachInputEvent(menu_button, 'onchange',
            function(k) {
                var url = 'index.php?module='+k+'&action=EditView&return_module=Emails&return_action=index&return_record='+record_id+'&return_layout=Standard&return_panel='+k.toLowerCase()+'&email_id='+record_id;
                return App.util.navigateTo(url)
        });
        menu_button.menu.setOptions(new UI.SelectOptions(options));
    }

    UI.isPrimaryForm = function(form) {
    	var parent;
    	form = UI.getForm(form);
    	if(! form) return;
    	if(UI.inDocument(form) && form.getAttribute('name') == 'DetailForm') {
    		parent = form.parentNode;
    		while(parent && parent !== window) {
    			if(parent.id == 'content-main')
    				return true;
    			parent = parent.parentNode;
    		}
    	}
    }

    UI.markFormDirty = function(elt, flag) {
    	if(! elt || elt.ignore_dirty) return;
    	var form = UI.getForm(elt.form || elt), oldf, t, parent, primary;
    	if(! form || form._sending) return;
    	oldf = form._dirty;
    	form._dirty = get_default(flag, true);
    	if(form._dirty != oldf && form._postDirty)
			UI.callEvent(form, form._postDirty, form, form._dirty);
    	if(UI.isPrimaryForm(form)) {
			t = App.util.getPageTitle() || '';
			if(t.match(/^\* /)) {
				if(! form._dirty) App.util.setPageTitle(t.substring(2));
			} else if(form._dirty) {
				App.util.setPageTitle('* ' + t);
			}
    	}
	}

	UI.checkFormDirty = function(form) {
		var form = UI.getForm(form);
		if(form) return form._dirty;
	}

    UI.addToFavorites = function(elt, module, record_id) {
        if (! UI.hasClass(elt, 'checked')) {
            UI.addClass(elt, 'checked');
            elt.title = app_string('LBL_REMOVE_FROM_FAVORITES', 'Favorites');
        } else {
            UI.removeClass(elt, 'checked');
            elt.title = app_string('LBL_ADD_TO_FAVORITES', 'Favorites');
        }

        if (module && record_id) {
            var query_params = {record_module: module, record_id: record_id, module: 'Favorites'};
            var req = new App.conn.JSONRequest('add_to_favorites', {silent: true}, query_params);
            req.fetch(function() {});
            return true;
        } else {
            return false;
        }
    }

	UI.arrangeSubpanels = function(module, id) {
		var tabs = UI.getFormInput('DetailForm', 'layout_tab');
		open_popup_window({
			module: 'NewStudio',
			action: 'index',
			filter: {
				wizard: 'ArrangeSubpanels',
				edit_module: module,
				tab: tabs ? tabs.getValue() : 'Standard',
				record_id: id
			},
			inline: true
		});
	}

    UI.layoutTabUpdated = function(tabs) {
    	if(! tabs) return;
    	var tab = tabs.getValue(),
    		query;
		var theTab = tabs.getTab(tab);
		if (!theTab.editing)
	    	sugarListView.loadSubpanels(tabs.form);
    	var state = App.session.getState();
    	if(state && state.query) {
    		var m = state.query.match(/^(.*?&layout_tab=)([^&]*)(.*)$/);
    		if(m) query = m[1] + encodeURIComponent(tab) + m[3];
    		else query = state.query + '&layout_tab=' + encodeURIComponent(tab);
    		App.session.replaceState(query, state.context, state.vars);
    	}
    }

    UI.toggleExpandList = function(id, link) {
		var elt = $(id), new_state;
		if(elt) {
			new_state = ! UI.hasClass(elt, 'compact');
			UI.addRemoveClass(elt, 'compact', new_state);
			if(link)
				UI.setElementText(link, app_string(new_state ? 'LBL_LIST_SHOW_MORE' : 'LBL_LIST_SHOW_LESS'));
		}
    }

	UI.initConditionalFields = function(conditional) {
		this.conditional_fields = conditional;
	};

	UI.showStatus = function(msg, timeout) {
		return UI.Status.show(msg, timeout);
	};
	UI.hideStatus = function(s) {
		return UI.Status.remove(s);
	};

	UI.record_link_idx = 0;
	UI.makeRecordLink = function(module, id, name, no_popup, context) {
		var params = {module: module, action: 'DetailView', record: id}, ret;
		if(! id) return MakeElt('span', name);
		ret = MakeElt('a.tabDetailViewDFLink', {href: encodeQueryString(params, 'index.php?')}, name)
			.on('click', SUGAR.util.handleLink);
		if(! no_popup) {
			var info_id = 'record-link-' + (UI.record_link_idx ++);
			ret.prop('id', info_id);
			if(! isset(context)) context = 'view';
			ret.on('mouseout, mousedown', function() { SUGAR.util.clearAdditionalDetailsCall(); })
				.on('mouseover', function() {
					return SUGAR.util.getAdditionalDetails(module, id, info_id, null, context, 750);
				});
		}
		return ret;
	};

});


// utility methods
AppBase.module('popups').moduleRequires('ui-controls').init({
	known: {},
	reposition: function() {
		SUGAR.ui.PopupManager.reposition();
	},
	close: function(popup) {
		return SUGAR.ui.PopupManager.close(popup);
	},
    getCurrent: function() {
   		return SUGAR.ui.PopupManager.last_opened;
   	},
   	// note: url may be an HTTPRequest
	openUrl: function(url, conn_params, dlg_params, hooks) {
		var d = new SUGAR.ui.Dialog(null, dlg_params);
		if(hooks && hooks.show)
			d.onshow = hooks.show;
		if(hooks && hooks.close)
			d.onclose = hooks.close;
		d.render();
		d.fetchContent(url, conn_params, null, true);
		popup_dialog = d;
		return d;
	},
	openForm: function(form, form_values, conn_params, dlg_params) {
		var d = new SUGAR.ui.Dialog(null, dlg_params);
		d.render();
		d.fetchFormContent(form, form_values, conn_params, null, true);
		popup_dialog = d;
		return d;
	},
	getKnown: function(popup) {
		var p = this.known[popup];
		if(! p)
			return;
		if(! p.elt || ! SUGAR.ui.inDocument(p.elt)) {
			this.known[popup] = null;
			return;
		}
		return p;
	},
	showPopup: function(tg, popup, offset, delay, hooks, popup_params) {
		if(! (popup instanceof SUGAR.ui.Popup)) {
			popup = $(popup);
			if(! popup) return;
			var p;
			if(popup.id && (p = this.getKnown(popup.id))) {
				popup = p;
				popup.target = tg;
				popup.target_offset = offset;
				if(hooks && hooks.show)
					popup.onshow = hooks.show;
			}
			else {
				var params = {target: tg, target_offset: offset, destroy_on_close: false};
				if(popup && SUGAR.ui.inDocument(popup)) {
					params.keep_in_document = true;
				}
				if(popup_params) extendObject(params, popup_params);
				popup = new SUGAR.ui.Popup(popup, params);
				if(hooks && hooks.show)
					popup.onshow = hooks.show;
				this.known[popup.id] = popup;
				if(! popup.elt.id)
					popup.elt.id = popup.id;
			}
		}
		else {
			if(tg)
				popup.target = tg;
			if(offset)
				popup.target_offset = offset;
			if(hooks && hooks.show)
				popup.onshow = hooks.show;
		}
		popup.show(delay);
	},
	hidePopup: function(popup) {
		SUGAR.popups.close(popup);
	},
	attachPopup: function(tg, popup, params) {
		if(! params) params = {};
		var show = function() {
			if(! params.require_click) {
				var p = SUGAR.ui.PopupManager.last_opened;
				if(p && ! p.modal && ! p.close_on_exit)
					return; // don't trigger popup when a semi-persistent one is open, or it will be dismissed
			}
			SUGAR.popups.showPopup(tg, popup, params.offset, params.delay, params.hooks);
			return false;
		}
		tg = $(tg);
		if(! tg)
			return false;
		if(isIPhone)
			params.require_click = true;
		if(params.require_click) {
			params.delay = 0;
			$(tg).onclick = show;
		} else
			$(tg).onmouseenter = show;
		return true;
	},
	hiliteItem: function(menuItem,changeClass){
		if(changeClass)
			SUGAR.ui.addClass(menuItem, 'hilite');
	},
	unhiliteItem: function(menuItem) {
		SUGAR.ui.removeClass(menuItem, 'hilite');
	},
	quickShow: function(content, title, target, params) {
		if(! params) params = {};
		if(! title) {
			params.have_title_bar = false;
			params.draggable = false;
		} else
			params.title = title;
		if(target)
			params.target = target;
		var d = new SUGAR.ui.MiniDialog(null, params);
		if(isNode(content))
			d.content_elt = content;
		else if(isset(content))
			d.setContent(content);
		d.render() && d.show();
		return d;
	},
	tooltip: function(param, target) {
		var params;
		if(typeof(param) == 'object')
			params = deep_clone(param);
		else
			params = {content: param};
		if(! isset(params.destroy_on_close))
			params.destroy_on_close = true;
		if(target)
			params.target = target;
		params.pre_triggered = true;
		var t = new SUGAR.ui.Tooltip(null, params);
	},
	initContent: function(params) {
		SUGAR.ui.PopupManager.initContent(params);
	},
	setTitle: function(title) {
		var c = SUGAR.popups.getCurrent();
		if(c && c.setTitle) c.setTitle(title);
	},
	setTitleHtml: function(title) {
		var c = SUGAR.popups.getCurrent();
		if(c && c.setTitleHtml) c.setTitleHtml(title);
	},
	showMessage: function(text, title, options) {
		if(title && typeof(title) !== 'string') {
			options = title;
			title = null;
		}
		if(! options) options = {};
		var id = options.id || options.name,
			UI = SUGAR.ui,
			title_text = options.title || title,
			have_title = title_text || options.title_html || options.have_title_bar,
			dlg = new UI.PopupMessage(id, {
				have_title_bar: have_title,
				invis_mask: ! options.show_mask,
				onclose: options.onclose,
				show_popup_close: options.close_button,
				timeout: options.timeout,
				title: options.title_html,
				title_text: title_text,
				width: options.width || 600}),
			content = options.content,
			onaccept = function() {
				dlg.close();
				if(options.onaccept) options.onaccept(this);
				else if(options.url) {
					SUGAR.util.navigateTo(options.url);
				}
			},
			oncancel = function() {
				dlg.close();
				if(options.oncancel) options.oncancel(this);
			},
			form, value, label;
		if(! content) {
			form = new UI.FormRender({card: true, style: 'default-form'});
			content = form.render();
			value = get_default(
				options.html,
				html_escape(trim(text)).replace(/\r?\n/g, '<br>'));
			value = '<div class="message-content">' + value + '</div>';
			if(options.icon)
				value = '<div class="no-flex message-icon">' +
					'<span class="uii uii-lg ' + options.icon + '"></span></div>' +
					value;
			value = '<div class="tools-row align-top message-outer">' + value + '</div>';
			form.addSection({fields: [{value: value}]});
			buttons = options.buttons || [];
			if(options.cancel_button) {
				label = options.cancel_button_label || app_string('LBL_CANCEL_BUTTON_LABEL');
				buttons.push({
					label: label, name: 'cancel',
					icon: 'icon-cancel', onclick: oncancel});
			}
			if(get_default(options.ok_button, ! options.close_button)) {
				label = app_string(options.url ? 'LBL_OPEN_BUTTON_LABEL' : 'LBL_OK_BUTTON_LABEL');
				label = options.ok_button_label || label;
				buttons.push({
					label: label, name: 'accept',
					icon: 'icon-accept', onclick: onaccept, autofocus: true});
			}
			if(buttons)
				form.addButtons(buttons, {footer: true, align: 'right'});
		}
		dlg.render();
		dlg.setContent(content);
		dlg.show(options.delay);
		return dlg;
	}
});

// used by popup_helper.js
get_close_popup = function() { return SUGAR.popups.close; }

ui_popup_return = function(result) {
	if(! result) return;
	var input = SUGAR.ui.getFormInput(result.form_name, result.passthru_data.ui_input_name),
		row, count;
	if(input) {
		if(result.selection_list) {
			row = result.selection_list;
			count = result.selection_count;
		} else if(result.name_to_value_array) {
			row = {ID_0: result.name_to_value_array};
			count = 1;
		}
		else return;
        input.update_from_popup = true;
		if(result.passthru_data.ui_return_method) {
			input[result.passthru_data.ui_return_method](row, count, undefined, result.module);
        } else {
			var pK = result.primary_key || 'id';
			input.userUpdate(row.ID_0[pK], row.ID_0, undefined, result.module);
		}
        input.update_from_popup = false;
	}
}

ui_popup_return_owner = function(result) {
	if(! result) return;
	var input = SUGAR.ui.getFormInput(result.form_name, result.passthru_data.ui_input_name),
		row, count;
	if(input) {
		if(result.selection_list) {
			row = result.selection_list;
			count = result.selection_count;
		}
		else return;
        input.update_from_popup = true;
		if(result.passthru_data.ui_return_method) {
			input[result.passthru_data.ui_return_method](row, count);
        } else {
			input.userUpdateMulti(row, count);
		}
        input.update_from_popup = false;
	}
}


remove_link = function(list_id, link_id) {
    var msg = SUGAR.language.get('app_strings', 'NTC_REMOVE_CONFIRMATION');
    if (confirm(msg)) {
        form_params = {};
        form_params.link_id = link_id;
        form_params.record_perform = 'subpanel_remove';
        var conn_params = {};
        conn_params.resultDiv = '' + list_id + '-outer';
        var frm = list_id + '-ListUpdate';
        SUGAR.ui.sendForm(frm, form_params, conn_params);
    }
}

// for compatibility only
ajaxStatus = {
	shown: [],
	// prefer SUGAR.ui.showStatus
	showStatus: function(text) {
		var last = SUGAR.ui.Status.show(text);
		this.shown.push(last);
		return last;
	},
	// prefer statusMessage.hide();
	hideStatus: function(id) {
		if(id) {
			SUGAR.ui.Status.remove(id);
			while(this.shown.length && this.shown.pop().id != id);
		} else if(this.shown.length)
			SUGAR.ui.Status.remove(this.shown.pop());
	},
	positionStatus: function() {
		SUGAR.ui.Status.reposition();
	}
};

